<?php

return [
    'Names' => [
        'af' => 'Afrikaans',
        'af_NA' => 'Afrikaans (Namibi)',
        'af_ZA' => 'Afrikaans (Afrik di Sid)',
        'am' => 'Amharik',
        'am_ET' => 'Amharik (Ecopi)',
        'ar' => 'Arabic',
        'ar_001' => 'Arabic (àddina)',
        'ar_AE' => 'Arabic (Emira Arab Ini)',
        'ar_BH' => 'Arabic (Bahreyin)',
        'ar_DJ' => 'Arabic (Jibuti)',
        'ar_DZ' => 'Arabic (Alseri)',
        'ar_EG' => 'Arabic (Esipt)',
        'ar_EH' => 'Arabic (Sahara bu sowwu)',
        'ar_ER' => 'Arabic (Eritere)',
        'ar_IL' => 'Arabic (Israyel)',
        'ar_IQ' => 'Arabic (Irag)',
        'ar_JO' => 'Arabic (Sordani)',
        'ar_KM' => 'Arabic (Komoor)',
        'ar_KW' => 'Arabic (Kowet)',
        'ar_LB' => 'Arabic (Libaa)',
        'ar_LY' => 'Arabic (Libi)',
        'ar_MA' => 'Arabic (Marog)',
        'ar_MR' => 'Arabic (Mooritani)',
        'ar_OM' => 'Arabic (Omaan)',
        'ar_PS' => 'Arabic (réew yu Palestine)',
        'ar_QA' => 'Arabic (Kataar)',
        'ar_SA' => 'Arabic (Arabi Sawudi)',
        'ar_SD' => 'Arabic (Sudaŋ)',
        'ar_SO' => 'Arabic (Somali)',
        'ar_SS' => 'Arabic (Sudaŋ di Sid)',
        'ar_SY' => 'Arabic (Siri)',
        'ar_TD' => 'Arabic (Càdd)',
        'ar_TN' => 'Arabic (Tinisi)',
        'ar_YE' => 'Arabic (Yaman)',
        'as' => 'Asame',
        'as_IN' => 'Asame (End)',
        'az' => 'Aserbayjane',
        'az_AZ' => 'Aserbayjane (Aserbayjaŋ)',
        'az_Cyrl' => 'Aserbayjane (Sirilik)',
        'az_Cyrl_AZ' => 'Aserbayjane (Sirilik, Aserbayjaŋ)',
        'az_Latn' => 'Aserbayjane (Latin)',
        'az_Latn_AZ' => 'Aserbayjane (Latin, Aserbayjaŋ)',
        'ba' => 'Baskir',
        'ba_RU' => 'Baskir (Risi)',
        'be' => 'Belaris',
        'be_BY' => 'Belaris (Belaris)',
        'bg' => 'Bilgaar',
        'bg_BG' => 'Bilgaar (Bilgari)',
        'bn' => 'Baŋla',
        'bn_BD' => 'Baŋla (Bengalades)',
        'bn_IN' => 'Baŋla (End)',
        'bo' => 'Tibetan',
        'bo_CN' => 'Tibetan (Siin)',
        'bo_IN' => 'Tibetan (End)',
        'br' => 'Breton',
        'br_FR' => 'Breton (Faraans)',
        'bs' => 'Bosñak',
        'bs_BA' => 'Bosñak (Bosni Ersegowin)',
        'bs_Cyrl' => 'Bosñak (Sirilik)',
        'bs_Cyrl_BA' => 'Bosñak (Sirilik, Bosni Ersegowin)',
        'bs_Latn' => 'Bosñak (Latin)',
        'bs_Latn_BA' => 'Bosñak (Latin, Bosni Ersegowin)',
        'ca' => 'Katalan',
        'ca_AD' => 'Katalan (Andoor)',
        'ca_ES' => 'Katalan (Españ)',
        'ca_FR' => 'Katalan (Faraans)',
        'ca_IT' => 'Katalan (Itali)',
        'cs' => 'Cek',
        'cs_CZ' => 'Cek (Réewum Cek)',
        'cy' => 'Wels',
        'cy_GB' => 'Wels (Ruwaayom Ini)',
        'da' => 'Danuwa',
        'da_DK' => 'Danuwa (Danmàrk)',
        'da_GL' => 'Danuwa (Girinlànd)',
        'de' => 'Almaa',
        'de_AT' => 'Almaa (Ótiriis)',
        'de_BE' => 'Almaa (Belsig)',
        'de_CH' => 'Almaa (Siwis)',
        'de_DE' => 'Almaa (Almaañ)',
        'de_IT' => 'Almaa (Itali)',
        'de_LI' => 'Almaa (Liktensteyin)',
        'de_LU' => 'Almaa (Liksàmbur)',
        'dz' => 'Dsongkaa',
        'dz_BT' => 'Dsongkaa (Butaŋ)',
        'el' => 'Gereg',
        'el_CY' => 'Gereg (Siipar)',
        'el_GR' => 'Gereg (Gerees)',
        'en' => 'Àngale',
        'en_001' => 'Àngale (àddina)',
        'en_150' => 'Àngale (Europe)',
        'en_AE' => 'Àngale (Emira Arab Ini)',
        'en_AG' => 'Àngale (Antiguwa ak Barbuda)',
        'en_AI' => 'Àngale (Angiiy)',
        'en_AS' => 'Àngale (Samowa bu Amerig)',
        'en_AT' => 'Àngale (Ótiriis)',
        'en_AU' => 'Àngale (Ostarali)',
        'en_BB' => 'Àngale (Barbad)',
        'en_BE' => 'Àngale (Belsig)',
        'en_BI' => 'Àngale (Burundi)',
        'en_BM' => 'Àngale (Bermid)',
        'en_BS' => 'Àngale (Bahamas)',
        'en_BW' => 'Àngale (Botswana)',
        'en_BZ' => 'Àngale (Belis)',
        'en_CA' => 'Àngale (Kanadaa)',
        'en_CC' => 'Àngale (Duni Koko [Kilin])',
        'en_CH' => 'Àngale (Siwis)',
        'en_CK' => 'Àngale (Duni Kuuk)',
        'en_CM' => 'Àngale (Kamerun)',
        'en_CX' => 'Àngale (Dunu Kirismas)',
        'en_CY' => 'Àngale (Siipar)',
        'en_CZ' => 'Àngale (Réewum Cek)',
        'en_DE' => 'Àngale (Almaañ)',
        'en_DK' => 'Àngale (Danmàrk)',
        'en_DM' => 'Àngale (Dominik)',
        'en_EE' => 'Àngale (Estoni)',
        'en_ER' => 'Àngale (Eritere)',
        'en_ES' => 'Àngale (Españ)',
        'en_FI' => 'Àngale (Finlànd)',
        'en_FJ' => 'Àngale (Fijji)',
        'en_FK' => 'Àngale (Duni Falkland)',
        'en_FM' => 'Àngale (Mikoronesi)',
        'en_FR' => 'Àngale (Faraans)',
        'en_GB' => 'Àngale (Ruwaayom Ini)',
        'en_GD' => 'Àngale (Garanad)',
        'en_GE' => 'Àngale (Seworsi)',
        'en_GG' => 'Àngale (Gernase)',
        'en_GH' => 'Àngale (Gana)',
        'en_GI' => 'Àngale (Sibraltaar)',
        'en_GM' => 'Àngale (Gàmbi)',
        'en_GS' => 'Àngale (Seworsi di Sid ak Duni Sàndwiis di Sid)',
        'en_GU' => 'Àngale (Guwam)',
        'en_GY' => 'Àngale (Giyaan)',
        'en_HK' => 'Àngale (Ooŋ Koŋ)',
        'en_HU' => 'Àngale (Ongari)',
        'en_ID' => 'Àngale (Indonesi)',
        'en_IE' => 'Àngale (Irlànd)',
        'en_IL' => 'Àngale (Israyel)',
        'en_IM' => 'Àngale (Dunu Maan)',
        'en_IN' => 'Àngale (End)',
        'en_IO' => 'Àngale (Terituwaaru Brëtaañ ci Oseyaa Enjeŋ)',
        'en_IT' => 'Àngale (Itali)',
        'en_JE' => 'Àngale (Serse)',
        'en_JM' => 'Àngale (Samayig)',
        'en_JP' => 'Àngale (Sàppoŋ)',
        'en_KE' => 'Àngale (Keeña)',
        'en_KI' => 'Àngale (Kiribati)',
        'en_KN' => 'Àngale (Saŋ Kits ak Newis)',
        'en_KY' => 'Àngale (Duni Kaymaŋ)',
        'en_LC' => 'Àngale (Saŋ Lusi)',
        'en_LR' => 'Àngale (Liberiya)',
        'en_LS' => 'Àngale (Lesoto)',
        'en_LT' => 'Àngale (Litiyani)',
        'en_LV' => 'Àngale (Letoni)',
        'en_MG' => 'Àngale (Madagaskaar)',
        'en_MH' => 'Àngale (Duni Marsaal)',
        'en_MO' => 'Àngale (Makaawo)',
        'en_MP' => 'Àngale (Duni Mariyaan Noor)',
        'en_MS' => 'Àngale (Mooseraa)',
        'en_MT' => 'Àngale (Malt)',
        'en_MU' => 'Àngale (Moriis)',
        'en_MV' => 'Àngale (Maldiiw)',
        'en_MW' => 'Àngale (Malawi)',
        'en_MY' => 'Àngale (Malesi)',
        'en_NA' => 'Àngale (Namibi)',
        'en_NF' => 'Àngale (Dunu Norfolk)',
        'en_NG' => 'Àngale (Niseriya)',
        'en_NL' => 'Àngale (Peyi Baa)',
        'en_NO' => 'Àngale (Norwees)',
        'en_NR' => 'Àngale (Nawru)',
        'en_NU' => 'Àngale (Niw)',
        'en_NZ' => 'Àngale (Nuwel Selànd)',
        'en_PG' => 'Àngale (Papuwasi Gine Gu Bees)',
        'en_PH' => 'Àngale (Filipin)',
        'en_PK' => 'Àngale (Pakistaŋ)',
        'en_PL' => 'Àngale (Poloñ)',
        'en_PN' => 'Àngale (Duni Pitkayirn)',
        'en_PR' => 'Àngale (Porto Riko)',
        'en_PT' => 'Àngale (Portigaal)',
        'en_PW' => 'Àngale (Palaw)',
        'en_RO' => 'Àngale (Rumani)',
        'en_RW' => 'Àngale (Ruwànda)',
        'en_SB' => 'Àngale (Duni Salmoon)',
        'en_SC' => 'Àngale (Seysel)',
        'en_SD' => 'Àngale (Sudaŋ)',
        'en_SE' => 'Àngale (Suwed)',
        'en_SG' => 'Àngale (Singapuur)',
        'en_SH' => 'Àngale (Saŋ Eleen)',
        'en_SI' => 'Àngale (Esloweni)',
        'en_SK' => 'Àngale (Eslowaki)',
        'en_SL' => 'Àngale (Siyera Lewon)',
        'en_SS' => 'Àngale (Sudaŋ di Sid)',
        'en_SX' => 'Àngale (Sin Marten)',
        'en_SZ' => 'Àngale (Suwasilànd)',
        'en_TC' => 'Àngale (Duni Tirk ak Kaykos)',
        'en_TK' => 'Àngale (Tokoloo)',
        'en_TO' => 'Àngale (Tonga)',
        'en_TT' => 'Àngale (Tirinite ak Tobago)',
        'en_TV' => 'Àngale (Tuwalo)',
        'en_TZ' => 'Àngale (Taŋsani)',
        'en_UA' => 'Àngale (Ikeren)',
        'en_UG' => 'Àngale (Ugànda)',
        'en_UM' => 'Àngale (Duni Amerig Utar meer)',
        'en_US' => 'Àngale (Etaa Sini)',
        'en_VC' => 'Àngale (Saŋ Weesaa ak Garanadin)',
        'en_VG' => 'Àngale (Duni Wirsin yu Brëtaañ)',
        'en_VI' => 'Àngale (Duni Wirsin yu Etaa-sini)',
        'en_VU' => 'Àngale (Wanuatu)',
        'en_WS' => 'Àngale (Samowa)',
        'en_ZA' => 'Àngale (Afrik di Sid)',
        'en_ZM' => 'Àngale (Sàmbi)',
        'en_ZW' => 'Àngale (Simbabwe)',
        'eo' => 'Esperantoo',
        'eo_001' => 'Esperantoo (àddina)',
        'es' => 'Español',
        'es_419' => 'Español (Amerique Latine)',
        'es_AR' => 'Español (Arsàntin)',
        'es_BO' => 'Español (Boliwi)',
        'es_BR' => 'Español (Beresil)',
        'es_BZ' => 'Español (Belis)',
        'es_CL' => 'Español (Sili)',
        'es_CO' => 'Español (Kolombi)',
        'es_CR' => 'Español (Kosta Rika)',
        'es_CU' => 'Español (Kuba)',
        'es_DO' => 'Español (Repiblik Dominiken)',
        'es_EC' => 'Español (Ekwaatër)',
        'es_ES' => 'Español (Españ)',
        'es_GQ' => 'Español (Gine Ekuwatoriyal)',
        'es_GT' => 'Español (Guwatemala)',
        'es_HN' => 'Español (Onduraas)',
        'es_MX' => 'Español (Meksiko)',
        'es_NI' => 'Español (Nikaraguwa)',
        'es_PA' => 'Español (Panama)',
        'es_PE' => 'Español (Peru)',
        'es_PH' => 'Español (Filipin)',
        'es_PR' => 'Español (Porto Riko)',
        'es_PY' => 'Español (Paraguwe)',
        'es_SV' => 'Español (El Salwadoor)',
        'es_US' => 'Español (Etaa Sini)',
        'es_UY' => 'Español (Uruge)',
        'es_VE' => 'Español (Wenesiyela)',
        'et' => 'Estoñiye',
        'et_EE' => 'Estoñiye (Estoni)',
        'eu' => 'Bask',
        'eu_ES' => 'Bask (Españ)',
        'fa' => 'Pers',
        'fa_AF' => 'Pers (Afganistaŋ)',
        'fa_IR' => 'Pers (Iraŋ)',
        'ff' => 'Pël',
        'ff_CM' => 'Pël (Kamerun)',
        'ff_GN' => 'Pël (Gine)',
        'ff_Latn' => 'Pël (Latin)',
        'ff_Latn_BF' => 'Pël (Latin, Burkina Faaso)',
        'ff_Latn_CM' => 'Pël (Latin, Kamerun)',
        'ff_Latn_GH' => 'Pël (Latin, Gana)',
        'ff_Latn_GM' => 'Pël (Latin, Gàmbi)',
        'ff_Latn_GN' => 'Pël (Latin, Gine)',
        'ff_Latn_GW' => 'Pël (Latin, Gine-Bisaawóo)',
        'ff_Latn_LR' => 'Pël (Latin, Liberiya)',
        'ff_Latn_MR' => 'Pël (Latin, Mooritani)',
        'ff_Latn_NE' => 'Pël (Latin, Niiseer)',
        'ff_Latn_NG' => 'Pël (Latin, Niseriya)',
        'ff_Latn_SL' => 'Pël (Latin, Siyera Lewon)',
        'ff_Latn_SN' => 'Pël (Latin, Senegaal)',
        'ff_MR' => 'Pël (Mooritani)',
        'ff_SN' => 'Pël (Senegaal)',
        'fi' => 'Feylànde',
        'fi_FI' => 'Feylànde (Finlànd)',
        'fo' => 'Feroos',
        'fo_DK' => 'Feroos (Danmàrk)',
        'fo_FO' => 'Feroos (Duni Faro)',
        'fr' => 'Farañse',
        'fr_BE' => 'Farañse (Belsig)',
        'fr_BF' => 'Farañse (Burkina Faaso)',
        'fr_BI' => 'Farañse (Burundi)',
        'fr_BJ' => 'Farañse (Benee)',
        'fr_BL' => 'Farañse (Saŋ Bartalemi)',
        'fr_CA' => 'Farañse (Kanadaa)',
        'fr_CD' => 'Farañse (Kongo [R K D])',
        'fr_CF' => 'Farañse (Repiblik Sàntar Afrik)',
        'fr_CG' => 'Farañse (Réewum Kongo)',
        'fr_CH' => 'Farañse (Siwis)',
        'fr_CI' => 'Farañse (Kodiwaar)',
        'fr_CM' => 'Farañse (Kamerun)',
        'fr_DJ' => 'Farañse (Jibuti)',
        'fr_DZ' => 'Farañse (Alseri)',
        'fr_FR' => 'Farañse (Faraans)',
        'fr_GA' => 'Farañse (Gaboŋ)',
        'fr_GF' => 'Farañse (Guyaan Farañse)',
        'fr_GN' => 'Farañse (Gine)',
        'fr_GP' => 'Farañse (Guwaadelup)',
        'fr_GQ' => 'Farañse (Gine Ekuwatoriyal)',
        'fr_HT' => 'Farañse (Ayti)',
        'fr_KM' => 'Farañse (Komoor)',
        'fr_LU' => 'Farañse (Liksàmbur)',
        'fr_MA' => 'Farañse (Marog)',
        'fr_MC' => 'Farañse (Monako)',
        'fr_MF' => 'Farañse (Saŋ Marteŋ)',
        'fr_MG' => 'Farañse (Madagaskaar)',
        'fr_ML' => 'Farañse (Mali)',
        'fr_MQ' => 'Farañse (Martinik)',
        'fr_MR' => 'Farañse (Mooritani)',
        'fr_MU' => 'Farañse (Moriis)',
        'fr_NC' => 'Farañse (Nuwel Kaledoni)',
        'fr_NE' => 'Farañse (Niiseer)',
        'fr_PF' => 'Farañse (Polinesi Farañse)',
        'fr_PM' => 'Farañse (Saŋ Peer ak Mikeloŋ)',
        'fr_RE' => 'Farañse (Reeñoo)',
        'fr_RW' => 'Farañse (Ruwànda)',
        'fr_SC' => 'Farañse (Seysel)',
        'fr_SN' => 'Farañse (Senegaal)',
        'fr_SY' => 'Farañse (Siri)',
        'fr_TD' => 'Farañse (Càdd)',
        'fr_TG' => 'Farañse (Togo)',
        'fr_TN' => 'Farañse (Tinisi)',
        'fr_VU' => 'Farañse (Wanuatu)',
        'fr_WF' => 'Farañse (Walis ak Futuna)',
        'fr_YT' => 'Farañse (Mayot)',
        'ga' => 'Irlànde',
        'ga_GB' => 'Irlànde (Ruwaayom Ini)',
        'ga_IE' => 'Irlànde (Irlànd)',
        'gd' => 'Galuwaa bu Ekos',
        'gd_GB' => 'Galuwaa bu Ekos (Ruwaayom Ini)',
        'gl' => 'Galisiye',
        'gl_ES' => 'Galisiye (Españ)',
        'gu' => 'Gujarati',
        'gu_IN' => 'Gujarati (End)',
        'ha' => 'Hawsa',
        'ha_GH' => 'Hawsa (Gana)',
        'ha_NE' => 'Hawsa (Niiseer)',
        'ha_NG' => 'Hawsa (Niseriya)',
        'he' => 'Ebrë',
        'he_IL' => 'Ebrë (Israyel)',
        'hi' => 'Endo',
        'hi_IN' => 'Endo (End)',
        'hi_Latn' => 'Endo (Latin)',
        'hi_Latn_IN' => 'Endo (Latin, End)',
        'hr' => 'Krowat',
        'hr_BA' => 'Krowat (Bosni Ersegowin)',
        'hr_HR' => 'Krowat (Korowasi)',
        'hu' => 'Ongruwaa',
        'hu_HU' => 'Ongruwaa (Ongari)',
        'hy' => 'Armaniye',
        'hy_AM' => 'Armaniye (Armeni)',
        'id' => 'Endonesiye',
        'id_ID' => 'Endonesiye (Indonesi)',
        'ig' => 'Igbo',
        'ig_NG' => 'Igbo (Niseriya)',
        'is' => 'Islànde',
        'is_IS' => 'Islànde (Islànd)',
        'it' => 'Italiye',
        'it_CH' => 'Italiye (Siwis)',
        'it_IT' => 'Italiye (Itali)',
        'it_SM' => 'Italiye (San Marino)',
        'it_VA' => 'Italiye (Site bu Watikaa)',
        'ja' => 'Sapone',
        'ja_JP' => 'Sapone (Sàppoŋ)',
        'ka' => 'Sorsiye',
        'ka_GE' => 'Sorsiye (Seworsi)',
        'kk' => 'Kasax',
        'kk_Arab' => 'Kasax (Araab)',
        'kk_Arab_CN' => 'Kasax (Araab, Siin)',
        'kk_Cyrl' => 'Kasax (Sirilik)',
        'kk_Cyrl_KZ' => 'Kasax (Sirilik, Kasaxstaŋ)',
        'kk_KZ' => 'Kasax (Kasaxstaŋ)',
        'km' => 'Xmer',
        'km_KH' => 'Xmer (Kàmboj)',
        'kn' => 'Kannadaa',
        'kn_IN' => 'Kannadaa (End)',
        'ko' => 'Koreye',
        'ko_CN' => 'Koreye (Siin)',
        'ko_KP' => 'Koreye (Kore Noor)',
        'ko_KR' => 'Koreye (Corée du Sud)',
        'ks' => 'Kashmiri',
        'ks_Arab' => 'Kashmiri (Araab)',
        'ks_Arab_IN' => 'Kashmiri (Araab, End)',
        'ks_IN' => 'Kashmiri (End)',
        'ku' => 'Kurdi',
        'ku_Latn' => 'Kurdi (Latin)',
        'ku_Latn_IQ' => 'Kurdi (Latin, Irag)',
        'ku_Latn_SY' => 'Kurdi (Latin, Siri)',
        'ku_Latn_TR' => 'Kurdi (Latin, Tirki)',
        'ku_SY' => 'Kurdi (Siri)',
        'ku_TR' => 'Kurdi (Tirki)',
        'ky' => 'Kirgiis',
        'ky_KG' => 'Kirgiis (Kirgistaŋ)',
        'lb' => 'Liksàmbursuwaa',
        'lb_LU' => 'Liksàmbursuwaa (Liksàmbur)',
        'lo' => 'Laaw',
        'lo_LA' => 'Laaw (Lawos)',
        'lt' => 'Lituyaniye',
        'lt_LT' => 'Lituyaniye (Litiyani)',
        'lv' => 'Letoniye',
        'lv_LV' => 'Letoniye (Letoni)',
        'mg' => 'Malagasi',
        'mg_MG' => 'Malagasi (Madagaskaar)',
        'mi' => 'Mawri',
        'mi_NZ' => 'Mawri (Nuwel Selànd)',
        'mk' => 'Maseduwaane',
        'mk_MK' => 'Maseduwaane (Maseduwaan bëj Gànnaar)',
        'ml' => 'Malayalam',
        'ml_IN' => 'Malayalam (End)',
        'mn' => 'Mongoliye',
        'mn_MN' => 'Mongoliye (Mongoli)',
        'mr' => 'Marati',
        'mr_IN' => 'Marati (End)',
        'ms' => 'Malay',
        'ms_BN' => 'Malay (Burney)',
        'ms_ID' => 'Malay (Indonesi)',
        'ms_MY' => 'Malay (Malesi)',
        'ms_SG' => 'Malay (Singapuur)',
        'mt' => 'Malt',
        'mt_MT' => 'Malt (Malt)',
        'my' => 'Birmes',
        'my_MM' => 'Birmes (Miyanmaar)',
        'ne' => 'Nepale',
        'ne_IN' => 'Nepale (End)',
        'ne_NP' => 'Nepale (Nepaal)',
        'nl' => 'Neyerlànde',
        'nl_AW' => 'Neyerlànde (Aruba)',
        'nl_BE' => 'Neyerlànde (Belsig)',
        'nl_BQ' => 'Neyerlànde (Pays-Bas bu Caraïbe)',
        'nl_CW' => 'Neyerlànde (Kursawo)',
        'nl_NL' => 'Neyerlànde (Peyi Baa)',
        'nl_SR' => 'Neyerlànde (Sirinam)',
        'nl_SX' => 'Neyerlànde (Sin Marten)',
        'no' => 'Nerwesiye',
        'no_NO' => 'Nerwesiye (Norwees)',
        'oc' => 'Ositan',
        'oc_ES' => 'Ositan (Españ)',
        'oc_FR' => 'Ositan (Faraans)',
        'om' => 'Oromo',
        'om_ET' => 'Oromo (Ecopi)',
        'om_KE' => 'Oromo (Keeña)',
        'or' => 'Oja',
        'or_IN' => 'Oja (End)',
        'pa' => 'Punjabi',
        'pa_Arab' => 'Punjabi (Araab)',
        'pa_Arab_PK' => 'Punjabi (Araab, Pakistaŋ)',
        'pa_IN' => 'Punjabi (End)',
        'pa_PK' => 'Punjabi (Pakistaŋ)',
        'pl' => 'Polone',
        'pl_PL' => 'Polone (Poloñ)',
        'ps' => 'Pasto',
        'ps_AF' => 'Pasto (Afganistaŋ)',
        'ps_PK' => 'Pasto (Pakistaŋ)',
        'pt' => 'Purtugees',
        'pt_AO' => 'Purtugees (Àngolaa)',
        'pt_BR' => 'Purtugees (Beresil)',
        'pt_CH' => 'Purtugees (Siwis)',
        'pt_CV' => 'Purtugees (Kabo Werde)',
        'pt_GQ' => 'Purtugees (Gine Ekuwatoriyal)',
        'pt_GW' => 'Purtugees (Gine-Bisaawóo)',
        'pt_LU' => 'Purtugees (Liksàmbur)',
        'pt_MO' => 'Purtugees (Makaawo)',
        'pt_MZ' => 'Purtugees (Mosàmbig)',
        'pt_PT' => 'Purtugees (Portigaal)',
        'pt_ST' => 'Purtugees (Sawo Tome ak Pirinsipe)',
        'pt_TL' => 'Purtugees (Timor Leste)',
        'qu' => 'Kesuwa',
        'qu_BO' => 'Kesuwa (Boliwi)',
        'qu_EC' => 'Kesuwa (Ekwaatër)',
        'qu_PE' => 'Kesuwa (Peru)',
        'rm' => 'Romaas',
        'rm_CH' => 'Romaas (Siwis)',
        'ro' => 'Rumaniyee',
        'ro_MD' => 'Rumaniyee (Moldawi)',
        'ro_RO' => 'Rumaniyee (Rumani)',
        'ru' => 'Rus',
        'ru_BY' => 'Rus (Belaris)',
        'ru_KG' => 'Rus (Kirgistaŋ)',
        'ru_KZ' => 'Rus (Kasaxstaŋ)',
        'ru_MD' => 'Rus (Moldawi)',
        'ru_RU' => 'Rus (Risi)',
        'ru_UA' => 'Rus (Ikeren)',
        'rw' => 'Kinyarwànda',
        'rw_RW' => 'Kinyarwànda (Ruwànda)',
        'sa' => 'Sanskrit',
        'sa_IN' => 'Sanskrit (End)',
        'sd' => 'Sindi',
        'sd_Arab' => 'Sindi (Araab)',
        'sd_Arab_PK' => 'Sindi (Araab, Pakistaŋ)',
        'sd_IN' => 'Sindi (End)',
        'sd_PK' => 'Sindi (Pakistaŋ)',
        'se' => 'Penku Sami',
        'se_FI' => 'Penku Sami (Finlànd)',
        'se_NO' => 'Penku Sami (Norwees)',
        'se_SE' => 'Penku Sami (Suwed)',
        'si' => 'Sinala',
        'si_LK' => 'Sinala (Siri Lànka)',
        'sk' => 'Eslowaki [Eslowak]',
        'sk_SK' => 'Eslowaki [Eslowak] (Eslowaki)',
        'sl' => 'Esloweniye',
        'sl_SI' => 'Esloweniye (Esloweni)',
        'so' => 'Somali [làkk]',
        'so_DJ' => 'Somali [làkk] (Jibuti)',
        'so_ET' => 'Somali [làkk] (Ecopi)',
        'so_KE' => 'Somali [làkk] (Keeña)',
        'so_SO' => 'Somali [làkk] (Somali)',
        'sq' => 'Albane',
        'sq_AL' => 'Albane (Albani)',
        'sq_MK' => 'Albane (Maseduwaan bëj Gànnaar)',
        'sr' => 'Serb',
        'sr_BA' => 'Serb (Bosni Ersegowin)',
        'sr_Cyrl' => 'Serb (Sirilik)',
        'sr_Cyrl_BA' => 'Serb (Sirilik, Bosni Ersegowin)',
        'sr_Cyrl_ME' => 'Serb (Sirilik, Montenegoro)',
        'sr_Cyrl_RS' => 'Serb (Sirilik, Serbi)',
        'sr_Latn' => 'Serb (Latin)',
        'sr_Latn_BA' => 'Serb (Latin, Bosni Ersegowin)',
        'sr_Latn_ME' => 'Serb (Latin, Montenegoro)',
        'sr_Latn_RS' => 'Serb (Latin, Serbi)',
        'sr_ME' => 'Serb (Montenegoro)',
        'sr_RS' => 'Serb (Serbi)',
        'sv' => 'Suweduwaa',
        'sv_AX' => 'Suweduwaa (Duni Aalànd)',
        'sv_FI' => 'Suweduwaa (Finlànd)',
        'sv_SE' => 'Suweduwaa (Suwed)',
        'ta' => 'Tamil',
        'ta_IN' => 'Tamil (End)',
        'ta_LK' => 'Tamil (Siri Lànka)',
        'ta_MY' => 'Tamil (Malesi)',
        'ta_SG' => 'Tamil (Singapuur)',
        'te' => 'Telugu',
        'te_IN' => 'Telugu (End)',
        'tg' => 'Tajis',
        'tg_TJ' => 'Tajis (Tajikistaŋ)',
        'th' => 'Tay',
        'th_TH' => 'Tay (Taylànd)',
        'ti' => 'Tigriña',
        'ti_ER' => 'Tigriña (Eritere)',
        'ti_ET' => 'Tigriña (Ecopi)',
        'tk' => 'Tirkmen',
        'tk_TM' => 'Tirkmen (Tirkmenistaŋ)',
        'to' => 'Tongan',
        'to_TO' => 'Tongan (Tonga)',
        'tr' => 'Tirk',
        'tr_CY' => 'Tirk (Siipar)',
        'tr_TR' => 'Tirk (Tirki)',
        'tt' => 'Tatar',
        'tt_RU' => 'Tatar (Risi)',
        'ug' => 'Uygur',
        'ug_CN' => 'Uygur (Siin)',
        'uk' => 'Ikreniye',
        'uk_UA' => 'Ikreniye (Ikeren)',
        'ur' => 'Urdu',
        'ur_IN' => 'Urdu (End)',
        'ur_PK' => 'Urdu (Pakistaŋ)',
        'uz' => 'Usbek',
        'uz_AF' => 'Usbek (Afganistaŋ)',
        'uz_Arab' => 'Usbek (Araab)',
        'uz_Arab_AF' => 'Usbek (Araab, Afganistaŋ)',
        'uz_Cyrl' => 'Usbek (Sirilik)',
        'uz_Cyrl_UZ' => 'Usbek (Sirilik, Usbekistaŋ)',
        'uz_Latn' => 'Usbek (Latin)',
        'uz_Latn_UZ' => 'Usbek (Latin, Usbekistaŋ)',
        'uz_UZ' => 'Usbek (Usbekistaŋ)',
        'vi' => 'Wiyetnaamiye',
        'vi_VN' => 'Wiyetnaamiye (Wiyetnam)',
        'wo' => 'Wolof',
        'wo_SN' => 'Wolof (Senegaal)',
        'yi' => 'Yidis',
        'yi_UA' => 'Yidis (Ikeren)',
        'yo' => 'Yoruba',
        'yo_BJ' => 'Yoruba (Benee)',
        'yo_NG' => 'Yoruba (Niseriya)',
        'zh' => 'Sinuwaa',
        'zh_CN' => 'Sinuwaa (Siin)',
        'zh_HK' => 'Sinuwaa (Ooŋ Koŋ)',
        'zh_Hans' => 'Sinuwaa (Buñ woyofal)',
        'zh_Hans_CN' => 'Sinuwaa (Buñ woyofal, Siin)',
        'zh_Hans_HK' => 'Sinuwaa (Buñ woyofal, Ooŋ Koŋ)',
        'zh_Hans_MO' => 'Sinuwaa (Buñ woyofal, Makaawo)',
        'zh_Hans_MY' => 'Sinuwaa (Buñ woyofal, Malesi)',
        'zh_Hans_SG' => 'Sinuwaa (Buñ woyofal, Singapuur)',
        'zh_Hant' => 'Sinuwaa (Cosaan)',
        'zh_Hant_HK' => 'Sinuwaa (Cosaan, Ooŋ Koŋ)',
        'zh_Hant_MO' => 'Sinuwaa (Cosaan, Makaawo)',
        'zh_Hant_MY' => 'Sinuwaa (Cosaan, Malesi)',
        'zh_Hant_TW' => 'Sinuwaa (Cosaan, Taywan)',
        'zh_MO' => 'Sinuwaa (Makaawo)',
        'zh_SG' => 'Sinuwaa (Singapuur)',
        'zh_TW' => 'Sinuwaa (Taywan)',
    ],
];
