<?php

return [
    'Names' => [
        'af' => 'afrikaans dili',
        'af_NA' => 'afrikaans dili (Namibiýa)',
        'af_ZA' => 'afrikaans dili (Günorta Afrika)',
        'ak' => 'akan dili',
        'ak_GH' => 'akan dili (Gana)',
        'am' => 'amhar dili',
        'am_ET' => 'amhar dili (Efiopiýa)',
        'ar' => 'arap dili',
        'ar_001' => 'arap dili (Dünýä)',
        'ar_AE' => 'arap dili (Birleşen Arap Emirlikleri)',
        'ar_BH' => 'arap dili (Bahreýn)',
        'ar_DJ' => 'arap dili (Jibuti)',
        'ar_DZ' => 'arap dili (Alžir)',
        'ar_EG' => 'arap dili (Müsür)',
        'ar_EH' => 'arap dili (Günbatar Sahara)',
        'ar_ER' => 'arap dili (Eritreýa)',
        'ar_IL' => 'arap dili (Ysraýyl)',
        'ar_IQ' => 'arap dili (Yrak)',
        'ar_JO' => 'arap dili (Iordaniýa)',
        'ar_KM' => 'arap dili (Komor adalary)',
        'ar_KW' => 'arap dili (Kuweýt)',
        'ar_LB' => 'arap dili (Liwan)',
        'ar_LY' => 'arap dili (Liwiýa)',
        'ar_MA' => 'arap dili (Marokko)',
        'ar_MR' => 'arap dili (Mawritaniýa)',
        'ar_OM' => 'arap dili (Oman)',
        'ar_PS' => 'arap dili (Palestina territoriýasy)',
        'ar_QA' => 'arap dili (Katar)',
        'ar_SA' => 'arap dili (Saud Arabystany)',
        'ar_SD' => 'arap dili (Sudan)',
        'ar_SO' => 'arap dili (Somali)',
        'ar_SS' => 'arap dili (Günorta Sudan)',
        'ar_SY' => 'arap dili (Siriýa)',
        'ar_TD' => 'arap dili (Çad)',
        'ar_TN' => 'arap dili (Tunis)',
        'ar_YE' => 'arap dili (Ýemen)',
        'as' => 'assam dili',
        'as_IN' => 'assam dili (Hindistan)',
        'az' => 'azerbaýjan dili',
        'az_AZ' => 'azerbaýjan dili (Azerbaýjan)',
        'az_Cyrl' => 'azerbaýjan dili (Kiril elipbiýi)',
        'az_Cyrl_AZ' => 'azerbaýjan dili (Kiril elipbiýi, Azerbaýjan)',
        'az_Latn' => 'azerbaýjan dili (Latyn elipbiýi)',
        'az_Latn_AZ' => 'azerbaýjan dili (Latyn elipbiýi, Azerbaýjan)',
        'ba' => 'başgyrt dili',
        'ba_RU' => 'başgyrt dili (Russiýa)',
        'be' => 'belarus dili',
        'be_BY' => 'belarus dili (Belarus)',
        'bg' => 'bolgar dili',
        'bg_BG' => 'bolgar dili (Bolgariýa)',
        'bm' => 'bamana',
        'bm_ML' => 'bamana (Mali)',
        'bn' => 'bengal dili',
        'bn_BD' => 'bengal dili (Bangladeş)',
        'bn_IN' => 'bengal dili (Hindistan)',
        'bo' => 'tibet dili',
        'bo_CN' => 'tibet dili (Hytaý)',
        'bo_IN' => 'tibet dili (Hindistan)',
        'br' => 'breton dili',
        'br_FR' => 'breton dili (Fransiýa)',
        'bs' => 'bosniýa dili',
        'bs_BA' => 'bosniýa dili (Bosniýa we Gersegowina)',
        'bs_Cyrl' => 'bosniýa dili (Kiril elipbiýi)',
        'bs_Cyrl_BA' => 'bosniýa dili (Kiril elipbiýi, Bosniýa we Gersegowina)',
        'bs_Latn' => 'bosniýa dili (Latyn elipbiýi)',
        'bs_Latn_BA' => 'bosniýa dili (Latyn elipbiýi, Bosniýa we Gersegowina)',
        'ca' => 'katalan dili',
        'ca_AD' => 'katalan dili (Andorra)',
        'ca_ES' => 'katalan dili (Ispaniýa)',
        'ca_FR' => 'katalan dili (Fransiýa)',
        'ca_IT' => 'katalan dili (Italiýa)',
        'ce' => 'çeçen dili',
        'ce_RU' => 'çeçen dili (Russiýa)',
        'cs' => 'çeh dili',
        'cs_CZ' => 'çeh dili (Çehiýa)',
        'cv' => 'çuwaş dili',
        'cv_RU' => 'çuwaş dili (Russiýa)',
        'cy' => 'walliý dili',
        'cy_GB' => 'walliý dili (Birleşen Patyşalyk)',
        'da' => 'daniýa dili',
        'da_DK' => 'daniýa dili (Daniýa)',
        'da_GL' => 'daniýa dili (Grenlandiýa)',
        'de' => 'nemes dili',
        'de_AT' => 'nemes dili (Awstriýa)',
        'de_BE' => 'nemes dili (Belgiýa)',
        'de_CH' => 'nemes dili (Şweýsariýa)',
        'de_DE' => 'nemes dili (Germaniýa)',
        'de_IT' => 'nemes dili (Italiýa)',
        'de_LI' => 'nemes dili (Lihtenşteýn)',
        'de_LU' => 'nemes dili (Lýuksemburg)',
        'dz' => 'dzong-ke dili',
        'dz_BT' => 'dzong-ke dili (Butan)',
        'ee' => 'ewe dili',
        'ee_GH' => 'ewe dili (Gana)',
        'ee_TG' => 'ewe dili (Togo)',
        'el' => 'grek dili',
        'el_CY' => 'grek dili (Kipr)',
        'el_GR' => 'grek dili (Gresiýa)',
        'en' => 'iňlis dili',
        'en_001' => 'iňlis dili (Dünýä)',
        'en_150' => 'iňlis dili (Ýewropa)',
        'en_AE' => 'iňlis dili (Birleşen Arap Emirlikleri)',
        'en_AG' => 'iňlis dili (Antigua we Barbuda)',
        'en_AI' => 'iňlis dili (Angilýa)',
        'en_AS' => 'iňlis dili (Amerikan Samoasy)',
        'en_AT' => 'iňlis dili (Awstriýa)',
        'en_AU' => 'iňlis dili (Awstraliýa)',
        'en_BB' => 'iňlis dili (Barbados)',
        'en_BE' => 'iňlis dili (Belgiýa)',
        'en_BI' => 'iňlis dili (Burundi)',
        'en_BM' => 'iňlis dili (Bermuda)',
        'en_BS' => 'iňlis dili (Bagama adalary)',
        'en_BW' => 'iňlis dili (Botswana)',
        'en_BZ' => 'iňlis dili (Beliz)',
        'en_CA' => 'iňlis dili (Kanada)',
        'en_CC' => 'iňlis dili (Kokos [Kiling] adalary)',
        'en_CH' => 'iňlis dili (Şweýsariýa)',
        'en_CK' => 'iňlis dili (Kuk adalary)',
        'en_CM' => 'iňlis dili (Kamerun)',
        'en_CX' => 'iňlis dili (Roždestwo adasy)',
        'en_CY' => 'iňlis dili (Kipr)',
        'en_CZ' => 'iňlis dili (Çehiýa)',
        'en_DE' => 'iňlis dili (Germaniýa)',
        'en_DK' => 'iňlis dili (Daniýa)',
        'en_DM' => 'iňlis dili (Dominika)',
        'en_EE' => 'iňlis dili (Estoniýa)',
        'en_ER' => 'iňlis dili (Eritreýa)',
        'en_ES' => 'iňlis dili (Ispaniýa)',
        'en_FI' => 'iňlis dili (Finlýandiýa)',
        'en_FJ' => 'iňlis dili (Fiji)',
        'en_FK' => 'iňlis dili (Folklend adalary)',
        'en_FM' => 'iňlis dili (Mikroneziýa)',
        'en_FR' => 'iňlis dili (Fransiýa)',
        'en_GB' => 'iňlis dili (Birleşen Patyşalyk)',
        'en_GD' => 'iňlis dili (Grenada)',
        'en_GE' => 'iňlis dili (Gruziýa)',
        'en_GG' => 'iňlis dili (Gernsi)',
        'en_GH' => 'iňlis dili (Gana)',
        'en_GI' => 'iňlis dili (Gibraltar)',
        'en_GM' => 'iňlis dili (Gambiýa)',
        'en_GS' => 'iňlis dili (Günorta Georgiýa we Günorta Sendwiç adasy)',
        'en_GU' => 'iňlis dili (Guam)',
        'en_GY' => 'iňlis dili (Gaýana)',
        'en_HK' => 'iňlis dili (Gonkong AAS Hytaý)',
        'en_HU' => 'iňlis dili (Wengriýa)',
        'en_ID' => 'iňlis dili (Indoneziýa)',
        'en_IE' => 'iňlis dili (Irlandiýa)',
        'en_IL' => 'iňlis dili (Ysraýyl)',
        'en_IM' => 'iňlis dili (Men adasy)',
        'en_IN' => 'iňlis dili (Hindistan)',
        'en_IO' => 'iňlis dili (Britaniýanyň Hindi okeanyndaky territoriýalary)',
        'en_IT' => 'iňlis dili (Italiýa)',
        'en_JE' => 'iňlis dili (Jersi)',
        'en_JM' => 'iňlis dili (Ýamaýka)',
        'en_JP' => 'iňlis dili (Ýaponiýa)',
        'en_KE' => 'iňlis dili (Keniýa)',
        'en_KI' => 'iňlis dili (Kiribati)',
        'en_KN' => 'iňlis dili (Sent-Kits we Newis)',
        'en_KY' => 'iňlis dili (Kaýman adalary)',
        'en_LC' => 'iňlis dili (Sent-Lýusiýa)',
        'en_LR' => 'iňlis dili (Liberiýa)',
        'en_LS' => 'iňlis dili (Lesoto)',
        'en_LT' => 'iňlis dili (Litwa)',
        'en_LV' => 'iňlis dili (Latwiýa)',
        'en_MG' => 'iňlis dili (Madagaskar)',
        'en_MH' => 'iňlis dili (Marşall adalary)',
        'en_MO' => 'iňlis dili (Makao AAS Hytaý)',
        'en_MP' => 'iňlis dili (Demirgazyk Mariana adalary)',
        'en_MS' => 'iňlis dili (Monserrat)',
        'en_MT' => 'iňlis dili (Malta)',
        'en_MU' => 'iňlis dili (Mawrikiý)',
        'en_MV' => 'iňlis dili (Maldiwler)',
        'en_MW' => 'iňlis dili (Malawi)',
        'en_MY' => 'iňlis dili (Malaýziýa)',
        'en_NA' => 'iňlis dili (Namibiýa)',
        'en_NF' => 'iňlis dili (Norfolk adasy)',
        'en_NG' => 'iňlis dili (Nigeriýa)',
        'en_NL' => 'iňlis dili (Niderlandlar)',
        'en_NO' => 'iňlis dili (Norwegiýa)',
        'en_NR' => 'iňlis dili (Nauru)',
        'en_NU' => 'iňlis dili (Niue)',
        'en_NZ' => 'iňlis dili (Täze Zelandiýa)',
        'en_PG' => 'iňlis dili (Papua - Täze Gwineýa)',
        'en_PH' => 'iňlis dili (Filippinler)',
        'en_PK' => 'iňlis dili (Pakistan)',
        'en_PL' => 'iňlis dili (Polşa)',
        'en_PN' => 'iňlis dili (Pitkern adalary)',
        'en_PR' => 'iňlis dili (Puerto-Riko)',
        'en_PT' => 'iňlis dili (Portugaliýa)',
        'en_PW' => 'iňlis dili (Palau)',
        'en_RO' => 'iňlis dili (Rumyniýa)',
        'en_RW' => 'iňlis dili (Ruanda)',
        'en_SB' => 'iňlis dili (Solomon adalary)',
        'en_SC' => 'iňlis dili (Seýşel adalary)',
        'en_SD' => 'iňlis dili (Sudan)',
        'en_SE' => 'iňlis dili (Şwesiýa)',
        'en_SG' => 'iňlis dili (Singapur)',
        'en_SH' => 'iňlis dili (Keramatly Ýelena adasy)',
        'en_SI' => 'iňlis dili (Sloweniýa)',
        'en_SK' => 'iňlis dili (Slowakiýa)',
        'en_SL' => 'iňlis dili (Sýerra-Leone)',
        'en_SS' => 'iňlis dili (Günorta Sudan)',
        'en_SX' => 'iňlis dili (Sint-Marten)',
        'en_SZ' => 'iňlis dili (Eswatini)',
        'en_TC' => 'iňlis dili (Terks we Kaýkos adalary)',
        'en_TK' => 'iňlis dili (Tokelau)',
        'en_TO' => 'iňlis dili (Tonga)',
        'en_TT' => 'iňlis dili (Trinidad we Tobago)',
        'en_TV' => 'iňlis dili (Tuwalu)',
        'en_TZ' => 'iňlis dili (Tanzaniýa)',
        'en_UA' => 'iňlis dili (Ukraina)',
        'en_UG' => 'iňlis dili (Uganda)',
        'en_UM' => 'iňlis dili (ABŞ-nyň daşarky adalary)',
        'en_US' => 'iňlis dili (Amerikanyň Birleşen Ştatlary)',
        'en_VC' => 'iňlis dili (Sent-Winsent we Grenadinler)',
        'en_VG' => 'iňlis dili (Britan Wirgin adalary)',
        'en_VI' => 'iňlis dili (ABŞ-nyň Wirgin adalary)',
        'en_VU' => 'iňlis dili (Wanuatu)',
        'en_WS' => 'iňlis dili (Samoa)',
        'en_ZA' => 'iňlis dili (Günorta Afrika)',
        'en_ZM' => 'iňlis dili (Zambiýa)',
        'en_ZW' => 'iňlis dili (Zimbabwe)',
        'eo' => 'esperanto dili',
        'eo_001' => 'esperanto dili (Dünýä)',
        'es' => 'ispan dili',
        'es_419' => 'ispan dili (Latyn Amerikasy)',
        'es_AR' => 'ispan dili (Argentina)',
        'es_BO' => 'ispan dili (Boliwiýa)',
        'es_BR' => 'ispan dili (Braziliýa)',
        'es_BZ' => 'ispan dili (Beliz)',
        'es_CL' => 'ispan dili (Çili)',
        'es_CO' => 'ispan dili (Kolumbiýa)',
        'es_CR' => 'ispan dili (Kosta-Rika)',
        'es_CU' => 'ispan dili (Kuba)',
        'es_DO' => 'ispan dili (Dominikan Respublikasy)',
        'es_EC' => 'ispan dili (Ekwador)',
        'es_ES' => 'ispan dili (Ispaniýa)',
        'es_GQ' => 'ispan dili (Ekwatorial Gwineýa)',
        'es_GT' => 'ispan dili (Gwatemala)',
        'es_HN' => 'ispan dili (Gonduras)',
        'es_MX' => 'ispan dili (Meksika)',
        'es_NI' => 'ispan dili (Nikaragua)',
        'es_PA' => 'ispan dili (Panama)',
        'es_PE' => 'ispan dili (Peru)',
        'es_PH' => 'ispan dili (Filippinler)',
        'es_PR' => 'ispan dili (Puerto-Riko)',
        'es_PY' => 'ispan dili (Paragwaý)',
        'es_SV' => 'ispan dili (Salwador)',
        'es_US' => 'ispan dili (Amerikanyň Birleşen Ştatlary)',
        'es_UY' => 'ispan dili (Urugwaý)',
        'es_VE' => 'ispan dili (Wenesuela)',
        'et' => 'eston dili',
        'et_EE' => 'eston dili (Estoniýa)',
        'eu' => 'bask dili',
        'eu_ES' => 'bask dili (Ispaniýa)',
        'fa' => 'pars dili',
        'fa_AF' => 'pars dili (Owganystan)',
        'fa_IR' => 'pars dili (Eýran)',
        'ff' => 'fula dili',
        'ff_Adlm' => 'fula dili (adlam)',
        'ff_Adlm_BF' => 'fula dili (adlam, Burkina-Faso)',
        'ff_Adlm_CM' => 'fula dili (adlam, Kamerun)',
        'ff_Adlm_GH' => 'fula dili (adlam, Gana)',
        'ff_Adlm_GM' => 'fula dili (adlam, Gambiýa)',
        'ff_Adlm_GN' => 'fula dili (adlam, Gwineýa)',
        'ff_Adlm_GW' => 'fula dili (adlam, Gwineýa-Bisau)',
        'ff_Adlm_LR' => 'fula dili (adlam, Liberiýa)',
        'ff_Adlm_MR' => 'fula dili (adlam, Mawritaniýa)',
        'ff_Adlm_NE' => 'fula dili (adlam, Niger)',
        'ff_Adlm_NG' => 'fula dili (adlam, Nigeriýa)',
        'ff_Adlm_SL' => 'fula dili (adlam, Sýerra-Leone)',
        'ff_Adlm_SN' => 'fula dili (adlam, Senegal)',
        'ff_CM' => 'fula dili (Kamerun)',
        'ff_GN' => 'fula dili (Gwineýa)',
        'ff_Latn' => 'fula dili (Latyn elipbiýi)',
        'ff_Latn_BF' => 'fula dili (Latyn elipbiýi, Burkina-Faso)',
        'ff_Latn_CM' => 'fula dili (Latyn elipbiýi, Kamerun)',
        'ff_Latn_GH' => 'fula dili (Latyn elipbiýi, Gana)',
        'ff_Latn_GM' => 'fula dili (Latyn elipbiýi, Gambiýa)',
        'ff_Latn_GN' => 'fula dili (Latyn elipbiýi, Gwineýa)',
        'ff_Latn_GW' => 'fula dili (Latyn elipbiýi, Gwineýa-Bisau)',
        'ff_Latn_LR' => 'fula dili (Latyn elipbiýi, Liberiýa)',
        'ff_Latn_MR' => 'fula dili (Latyn elipbiýi, Mawritaniýa)',
        'ff_Latn_NE' => 'fula dili (Latyn elipbiýi, Niger)',
        'ff_Latn_NG' => 'fula dili (Latyn elipbiýi, Nigeriýa)',
        'ff_Latn_SL' => 'fula dili (Latyn elipbiýi, Sýerra-Leone)',
        'ff_Latn_SN' => 'fula dili (Latyn elipbiýi, Senegal)',
        'ff_MR' => 'fula dili (Mawritaniýa)',
        'ff_SN' => 'fula dili (Senegal)',
        'fi' => 'fin dili',
        'fi_FI' => 'fin dili (Finlýandiýa)',
        'fo' => 'farer dili',
        'fo_DK' => 'farer dili (Daniýa)',
        'fo_FO' => 'farer dili (Farer adalary)',
        'fr' => 'fransuz dili',
        'fr_BE' => 'fransuz dili (Belgiýa)',
        'fr_BF' => 'fransuz dili (Burkina-Faso)',
        'fr_BI' => 'fransuz dili (Burundi)',
        'fr_BJ' => 'fransuz dili (Benin)',
        'fr_BL' => 'fransuz dili (Sen-Bartelemi)',
        'fr_CA' => 'fransuz dili (Kanada)',
        'fr_CD' => 'fransuz dili (Kongo - Kinşasa)',
        'fr_CF' => 'fransuz dili (Merkezi Afrika Respublikasy)',
        'fr_CG' => 'fransuz dili (Kongo - Brazzawil)',
        'fr_CH' => 'fransuz dili (Şweýsariýa)',
        'fr_CI' => 'fransuz dili (Kot-d’Iwuar)',
        'fr_CM' => 'fransuz dili (Kamerun)',
        'fr_DJ' => 'fransuz dili (Jibuti)',
        'fr_DZ' => 'fransuz dili (Alžir)',
        'fr_FR' => 'fransuz dili (Fransiýa)',
        'fr_GA' => 'fransuz dili (Gabon)',
        'fr_GF' => 'fransuz dili (Fransuz Gwianasy)',
        'fr_GN' => 'fransuz dili (Gwineýa)',
        'fr_GP' => 'fransuz dili (Gwadelupa)',
        'fr_GQ' => 'fransuz dili (Ekwatorial Gwineýa)',
        'fr_HT' => 'fransuz dili (Gaiti)',
        'fr_KM' => 'fransuz dili (Komor adalary)',
        'fr_LU' => 'fransuz dili (Lýuksemburg)',
        'fr_MA' => 'fransuz dili (Marokko)',
        'fr_MC' => 'fransuz dili (Monako)',
        'fr_MF' => 'fransuz dili (Sen-Marten)',
        'fr_MG' => 'fransuz dili (Madagaskar)',
        'fr_ML' => 'fransuz dili (Mali)',
        'fr_MQ' => 'fransuz dili (Martinika)',
        'fr_MR' => 'fransuz dili (Mawritaniýa)',
        'fr_MU' => 'fransuz dili (Mawrikiý)',
        'fr_NC' => 'fransuz dili (Täze Kaledoniýa)',
        'fr_NE' => 'fransuz dili (Niger)',
        'fr_PF' => 'fransuz dili (Fransuz Polineziýasy)',
        'fr_PM' => 'fransuz dili (Sen-Pýer we Mikelon)',
        'fr_RE' => 'fransuz dili (Reýunýon)',
        'fr_RW' => 'fransuz dili (Ruanda)',
        'fr_SC' => 'fransuz dili (Seýşel adalary)',
        'fr_SN' => 'fransuz dili (Senegal)',
        'fr_SY' => 'fransuz dili (Siriýa)',
        'fr_TD' => 'fransuz dili (Çad)',
        'fr_TG' => 'fransuz dili (Togo)',
        'fr_TN' => 'fransuz dili (Tunis)',
        'fr_VU' => 'fransuz dili (Wanuatu)',
        'fr_WF' => 'fransuz dili (Uollis we Futuna)',
        'fr_YT' => 'fransuz dili (Maýotta)',
        'fy' => 'günbatar friz dili',
        'fy_NL' => 'günbatar friz dili (Niderlandlar)',
        'ga' => 'irland dili',
        'ga_GB' => 'irland dili (Birleşen Patyşalyk)',
        'ga_IE' => 'irland dili (Irlandiýa)',
        'gd' => 'şotland kelt dili',
        'gd_GB' => 'şotland kelt dili (Birleşen Patyşalyk)',
        'gl' => 'galisiý dili',
        'gl_ES' => 'galisiý dili (Ispaniýa)',
        'gu' => 'gujarati dili',
        'gu_IN' => 'gujarati dili (Hindistan)',
        'gv' => 'men dili',
        'gv_IM' => 'men dili (Men adasy)',
        'ha' => 'hausa dili',
        'ha_GH' => 'hausa dili (Gana)',
        'ha_NE' => 'hausa dili (Niger)',
        'ha_NG' => 'hausa dili (Nigeriýa)',
        'he' => 'ýewreý dili',
        'he_IL' => 'ýewreý dili (Ysraýyl)',
        'hi' => 'hindi dili',
        'hi_IN' => 'hindi dili (Hindistan)',
        'hi_Latn' => 'hindi dili (Latyn elipbiýi)',
        'hi_Latn_IN' => 'hindi dili (Latyn elipbiýi, Hindistan)',
        'hr' => 'horwat dili',
        'hr_BA' => 'horwat dili (Bosniýa we Gersegowina)',
        'hr_HR' => 'horwat dili (Horwatiýa)',
        'hu' => 'wenger dili',
        'hu_HU' => 'wenger dili (Wengriýa)',
        'hy' => 'ermeni dili',
        'hy_AM' => 'ermeni dili (Ermenistan)',
        'ia' => 'interlingwa dili',
        'ia_001' => 'interlingwa dili (Dünýä)',
        'id' => 'indonez dili',
        'id_ID' => 'indonez dili (Indoneziýa)',
        'ie' => 'interlingwe dili',
        'ie_EE' => 'interlingwe dili (Estoniýa)',
        'ig' => 'igbo dili',
        'ig_NG' => 'igbo dili (Nigeriýa)',
        'ii' => 'syçuan-i dili',
        'ii_CN' => 'syçuan-i dili (Hytaý)',
        'is' => 'island dili',
        'is_IS' => 'island dili (Islandiýa)',
        'it' => 'italýan dili',
        'it_CH' => 'italýan dili (Şweýsariýa)',
        'it_IT' => 'italýan dili (Italiýa)',
        'it_SM' => 'italýan dili (San-Marino)',
        'it_VA' => 'italýan dili (Watikan)',
        'ja' => 'ýapon dili',
        'ja_JP' => 'ýapon dili (Ýaponiýa)',
        'jv' => 'ýawa dili',
        'jv_ID' => 'ýawa dili (Indoneziýa)',
        'ka' => 'gruzin dili',
        'ka_GE' => 'gruzin dili (Gruziýa)',
        'ki' => 'kikuýu dili',
        'ki_KE' => 'kikuýu dili (Keniýa)',
        'kk' => 'gazak dili',
        'kk_Arab' => 'gazak dili (Arap elipbiýi)',
        'kk_Arab_CN' => 'gazak dili (Arap elipbiýi, Hytaý)',
        'kk_Cyrl' => 'gazak dili (Kiril elipbiýi)',
        'kk_Cyrl_KZ' => 'gazak dili (Kiril elipbiýi, Gazagystan)',
        'kk_KZ' => 'gazak dili (Gazagystan)',
        'kl' => 'grenland dili',
        'kl_GL' => 'grenland dili (Grenlandiýa)',
        'km' => 'khmer dili',
        'km_KH' => 'khmer dili (Kamboja)',
        'kn' => 'kannada dili',
        'kn_IN' => 'kannada dili (Hindistan)',
        'ko' => 'koreý dili',
        'ko_CN' => 'koreý dili (Hytaý)',
        'ko_KP' => 'koreý dili (Demirgazyk Koreýa)',
        'ko_KR' => 'koreý dili (Günorta Koreýa)',
        'ks' => 'kaşmiri dili',
        'ks_Arab' => 'kaşmiri dili (Arap elipbiýi)',
        'ks_Arab_IN' => 'kaşmiri dili (Arap elipbiýi, Hindistan)',
        'ks_Deva' => 'kaşmiri dili (Dewanagari elipbiýi)',
        'ks_Deva_IN' => 'kaşmiri dili (Dewanagari elipbiýi, Hindistan)',
        'ks_IN' => 'kaşmiri dili (Hindistan)',
        'ku' => 'kürt dili',
        'ku_Latn' => 'kürt dili (Latyn elipbiýi)',
        'ku_Latn_IQ' => 'kürt dili (Latyn elipbiýi, Yrak)',
        'ku_Latn_SY' => 'kürt dili (Latyn elipbiýi, Siriýa)',
        'ku_Latn_TR' => 'kürt dili (Latyn elipbiýi, Türkiýe)',
        'ku_SY' => 'kürt dili (Siriýa)',
        'ku_TR' => 'kürt dili (Türkiýe)',
        'kw' => 'korn dili',
        'kw_GB' => 'korn dili (Birleşen Patyşalyk)',
        'ky' => 'gyrgyz dili',
        'ky_KG' => 'gyrgyz dili (Gyrgyzystan)',
        'lb' => 'lýuksemburg dili',
        'lb_LU' => 'lýuksemburg dili (Lýuksemburg)',
        'lg' => 'ganda dili',
        'lg_UG' => 'ganda dili (Uganda)',
        'ln' => 'lingala dili',
        'ln_AO' => 'lingala dili (Angola)',
        'ln_CD' => 'lingala dili (Kongo - Kinşasa)',
        'ln_CF' => 'lingala dili (Merkezi Afrika Respublikasy)',
        'ln_CG' => 'lingala dili (Kongo - Brazzawil)',
        'lo' => 'laos dili',
        'lo_LA' => 'laos dili (Laos)',
        'lt' => 'litwa dili',
        'lt_LT' => 'litwa dili (Litwa)',
        'lu' => 'luba-katanga dili',
        'lu_CD' => 'luba-katanga dili (Kongo - Kinşasa)',
        'lv' => 'latyş dili',
        'lv_LV' => 'latyş dili (Latwiýa)',
        'mg' => 'malagasiý dili',
        'mg_MG' => 'malagasiý dili (Madagaskar)',
        'mi' => 'maori dili',
        'mi_NZ' => 'maori dili (Täze Zelandiýa)',
        'mk' => 'makedon dili',
        'mk_MK' => 'makedon dili (Demirgazyk Makedoniýa)',
        'ml' => 'malaýalam dili',
        'ml_IN' => 'malaýalam dili (Hindistan)',
        'mn' => 'mongol dili',
        'mn_MN' => 'mongol dili (Mongoliýa)',
        'mr' => 'marathi dili',
        'mr_IN' => 'marathi dili (Hindistan)',
        'ms' => 'malaý dili',
        'ms_BN' => 'malaý dili (Bruneý)',
        'ms_ID' => 'malaý dili (Indoneziýa)',
        'ms_MY' => 'malaý dili (Malaýziýa)',
        'ms_SG' => 'malaý dili (Singapur)',
        'mt' => 'malta dili',
        'mt_MT' => 'malta dili (Malta)',
        'my' => 'birma dili',
        'my_MM' => 'birma dili (Mýanma [Birma])',
        'nb' => 'norwegiýa bukmol dili',
        'nb_NO' => 'norwegiýa bukmol dili (Norwegiýa)',
        'nb_SJ' => 'norwegiýa bukmol dili (Şpisbergen we Ýan-Maýen)',
        'nd' => 'demirgazyk ndebele dili',
        'nd_ZW' => 'demirgazyk ndebele dili (Zimbabwe)',
        'ne' => 'nepal dili',
        'ne_IN' => 'nepal dili (Hindistan)',
        'ne_NP' => 'nepal dili (Nepal)',
        'nl' => 'niderland dili',
        'nl_AW' => 'niderland dili (Aruba)',
        'nl_BE' => 'niderland dili (Belgiýa)',
        'nl_BQ' => 'niderland dili (Karib Niderlandlary)',
        'nl_CW' => 'niderland dili (Kýurasao)',
        'nl_NL' => 'niderland dili (Niderlandlar)',
        'nl_SR' => 'niderland dili (Surinam)',
        'nl_SX' => 'niderland dili (Sint-Marten)',
        'nn' => 'norwegiýa nýunorsk dili',
        'nn_NO' => 'norwegiýa nýunorsk dili (Norwegiýa)',
        'no' => 'norweg dili',
        'no_NO' => 'norweg dili (Norwegiýa)',
        'oc' => 'oksitan dili',
        'oc_ES' => 'oksitan dili (Ispaniýa)',
        'oc_FR' => 'oksitan dili (Fransiýa)',
        'om' => 'oromo dili',
        'om_ET' => 'oromo dili (Efiopiýa)',
        'om_KE' => 'oromo dili (Keniýa)',
        'or' => 'oriýa dili',
        'or_IN' => 'oriýa dili (Hindistan)',
        'os' => 'osetin dili',
        'os_GE' => 'osetin dili (Gruziýa)',
        'os_RU' => 'osetin dili (Russiýa)',
        'pa' => 'penjab dili',
        'pa_Arab' => 'penjab dili (Arap elipbiýi)',
        'pa_Arab_PK' => 'penjab dili (Arap elipbiýi, Pakistan)',
        'pa_Guru' => 'penjab dili (Gurmuhi elipbiýi)',
        'pa_Guru_IN' => 'penjab dili (Gurmuhi elipbiýi, Hindistan)',
        'pa_IN' => 'penjab dili (Hindistan)',
        'pa_PK' => 'penjab dili (Pakistan)',
        'pl' => 'polýak dili',
        'pl_PL' => 'polýak dili (Polşa)',
        'ps' => 'puştu dili',
        'ps_AF' => 'puştu dili (Owganystan)',
        'ps_PK' => 'puştu dili (Pakistan)',
        'pt' => 'portugal dili',
        'pt_AO' => 'portugal dili (Angola)',
        'pt_BR' => 'portugal dili (Braziliýa)',
        'pt_CH' => 'portugal dili (Şweýsariýa)',
        'pt_CV' => 'portugal dili (Kabo-Werde)',
        'pt_GQ' => 'portugal dili (Ekwatorial Gwineýa)',
        'pt_GW' => 'portugal dili (Gwineýa-Bisau)',
        'pt_LU' => 'portugal dili (Lýuksemburg)',
        'pt_MO' => 'portugal dili (Makao AAS Hytaý)',
        'pt_MZ' => 'portugal dili (Mozambik)',
        'pt_PT' => 'portugal dili (Portugaliýa)',
        'pt_ST' => 'portugal dili (San-Tome we Prinsipi)',
        'pt_TL' => 'portugal dili (Timor-Leste)',
        'qu' => 'keçua dili',
        'qu_BO' => 'keçua dili (Boliwiýa)',
        'qu_EC' => 'keçua dili (Ekwador)',
        'qu_PE' => 'keçua dili (Peru)',
        'rm' => 'retoroman dili',
        'rm_CH' => 'retoroman dili (Şweýsariýa)',
        'rn' => 'rundi dili',
        'rn_BI' => 'rundi dili (Burundi)',
        'ro' => 'rumyn dili',
        'ro_MD' => 'rumyn dili (Moldowa)',
        'ro_RO' => 'rumyn dili (Rumyniýa)',
        'ru' => 'rus dili',
        'ru_BY' => 'rus dili (Belarus)',
        'ru_KG' => 'rus dili (Gyrgyzystan)',
        'ru_KZ' => 'rus dili (Gazagystan)',
        'ru_MD' => 'rus dili (Moldowa)',
        'ru_RU' => 'rus dili (Russiýa)',
        'ru_UA' => 'rus dili (Ukraina)',
        'rw' => 'kinýaruanda dili',
        'rw_RW' => 'kinýaruanda dili (Ruanda)',
        'sa' => 'sanskrit dili',
        'sa_IN' => 'sanskrit dili (Hindistan)',
        'sc' => 'sardin dili',
        'sc_IT' => 'sardin dili (Italiýa)',
        'sd' => 'sindhi dili',
        'sd_Arab' => 'sindhi dili (Arap elipbiýi)',
        'sd_Arab_PK' => 'sindhi dili (Arap elipbiýi, Pakistan)',
        'sd_Deva' => 'sindhi dili (Dewanagari elipbiýi)',
        'sd_Deva_IN' => 'sindhi dili (Dewanagari elipbiýi, Hindistan)',
        'sd_IN' => 'sindhi dili (Hindistan)',
        'sd_PK' => 'sindhi dili (Pakistan)',
        'se' => 'demirgazyk saam dili',
        'se_FI' => 'demirgazyk saam dili (Finlýandiýa)',
        'se_NO' => 'demirgazyk saam dili (Norwegiýa)',
        'se_SE' => 'demirgazyk saam dili (Şwesiýa)',
        'sg' => 'sango dili',
        'sg_CF' => 'sango dili (Merkezi Afrika Respublikasy)',
        'si' => 'singal dili',
        'si_LK' => 'singal dili (Şri-Lanka)',
        'sk' => 'slowak dili',
        'sk_SK' => 'slowak dili (Slowakiýa)',
        'sl' => 'slowen dili',
        'sl_SI' => 'slowen dili (Sloweniýa)',
        'sn' => 'şona dili',
        'sn_ZW' => 'şona dili (Zimbabwe)',
        'so' => 'somali dili',
        'so_DJ' => 'somali dili (Jibuti)',
        'so_ET' => 'somali dili (Efiopiýa)',
        'so_KE' => 'somali dili (Keniýa)',
        'so_SO' => 'somali dili (Somali)',
        'sq' => 'alban dili',
        'sq_AL' => 'alban dili (Albaniýa)',
        'sq_MK' => 'alban dili (Demirgazyk Makedoniýa)',
        'sr' => 'serb dili',
        'sr_BA' => 'serb dili (Bosniýa we Gersegowina)',
        'sr_Cyrl' => 'serb dili (Kiril elipbiýi)',
        'sr_Cyrl_BA' => 'serb dili (Kiril elipbiýi, Bosniýa we Gersegowina)',
        'sr_Cyrl_ME' => 'serb dili (Kiril elipbiýi, Çernogoriýa)',
        'sr_Cyrl_RS' => 'serb dili (Kiril elipbiýi, Serbiýa)',
        'sr_Latn' => 'serb dili (Latyn elipbiýi)',
        'sr_Latn_BA' => 'serb dili (Latyn elipbiýi, Bosniýa we Gersegowina)',
        'sr_Latn_ME' => 'serb dili (Latyn elipbiýi, Çernogoriýa)',
        'sr_Latn_RS' => 'serb dili (Latyn elipbiýi, Serbiýa)',
        'sr_ME' => 'serb dili (Çernogoriýa)',
        'sr_RS' => 'serb dili (Serbiýa)',
        'st' => 'günorta soto dili',
        'st_LS' => 'günorta soto dili (Lesoto)',
        'st_ZA' => 'günorta soto dili (Günorta Afrika)',
        'su' => 'sundan dili',
        'su_ID' => 'sundan dili (Indoneziýa)',
        'su_Latn' => 'sundan dili (Latyn elipbiýi)',
        'su_Latn_ID' => 'sundan dili (Latyn elipbiýi, Indoneziýa)',
        'sv' => 'şwed dili',
        'sv_AX' => 'şwed dili (Aland adalary)',
        'sv_FI' => 'şwed dili (Finlýandiýa)',
        'sv_SE' => 'şwed dili (Şwesiýa)',
        'sw' => 'suahili dili',
        'sw_CD' => 'suahili dili (Kongo - Kinşasa)',
        'sw_KE' => 'suahili dili (Keniýa)',
        'sw_TZ' => 'suahili dili (Tanzaniýa)',
        'sw_UG' => 'suahili dili (Uganda)',
        'ta' => 'tamil dili',
        'ta_IN' => 'tamil dili (Hindistan)',
        'ta_LK' => 'tamil dili (Şri-Lanka)',
        'ta_MY' => 'tamil dili (Malaýziýa)',
        'ta_SG' => 'tamil dili (Singapur)',
        'te' => 'telugu dili',
        'te_IN' => 'telugu dili (Hindistan)',
        'tg' => 'täjik dili',
        'tg_TJ' => 'täjik dili (Täjigistan)',
        'th' => 'taý dili',
        'th_TH' => 'taý dili (Taýland)',
        'ti' => 'tigrinýa dili',
        'ti_ER' => 'tigrinýa dili (Eritreýa)',
        'ti_ET' => 'tigrinýa dili (Efiopiýa)',
        'tk' => 'türkmen dili',
        'tk_TM' => 'türkmen dili (Türkmenistan)',
        'tn' => 'tswana dili',
        'tn_BW' => 'tswana dili (Botswana)',
        'tn_ZA' => 'tswana dili (Günorta Afrika)',
        'to' => 'tongan dili',
        'to_TO' => 'tongan dili (Tonga)',
        'tr' => 'türk dili',
        'tr_CY' => 'türk dili (Kipr)',
        'tr_TR' => 'türk dili (Türkiýe)',
        'tt' => 'tatar dili',
        'tt_RU' => 'tatar dili (Russiýa)',
        'ug' => 'uýgur dili',
        'ug_CN' => 'uýgur dili (Hytaý)',
        'uk' => 'ukrain dili',
        'uk_UA' => 'ukrain dili (Ukraina)',
        'ur' => 'urdu',
        'ur_IN' => 'urdu (Hindistan)',
        'ur_PK' => 'urdu (Pakistan)',
        'uz' => 'özbek dili',
        'uz_AF' => 'özbek dili (Owganystan)',
        'uz_Arab' => 'özbek dili (Arap elipbiýi)',
        'uz_Arab_AF' => 'özbek dili (Arap elipbiýi, Owganystan)',
        'uz_Cyrl' => 'özbek dili (Kiril elipbiýi)',
        'uz_Cyrl_UZ' => 'özbek dili (Kiril elipbiýi, Özbegistan)',
        'uz_Latn' => 'özbek dili (Latyn elipbiýi)',
        'uz_Latn_UZ' => 'özbek dili (Latyn elipbiýi, Özbegistan)',
        'uz_UZ' => 'özbek dili (Özbegistan)',
        'vi' => 'wýetnam dili',
        'vi_VN' => 'wýetnam dili (Wýetnam)',
        'wo' => 'wolof dili',
        'wo_SN' => 'wolof dili (Senegal)',
        'xh' => 'kosa dili',
        'xh_ZA' => 'kosa dili (Günorta Afrika)',
        'yi' => 'idiş dili',
        'yi_UA' => 'idiş dili (Ukraina)',
        'yo' => 'ýoruba dili',
        'yo_BJ' => 'ýoruba dili (Benin)',
        'yo_NG' => 'ýoruba dili (Nigeriýa)',
        'za' => 'çžuan dili',
        'za_CN' => 'çžuan dili (Hytaý)',
        'zh' => 'hytaý dili',
        'zh_CN' => 'hytaý dili (Hytaý)',
        'zh_HK' => 'hytaý dili (Gonkong AAS Hytaý)',
        'zh_Hans' => 'hytaý dili (Ýönekeýleşdirilen)',
        'zh_Hans_CN' => 'hytaý dili (Ýönekeýleşdirilen, Hytaý)',
        'zh_Hans_HK' => 'hytaý dili (Ýönekeýleşdirilen, Gonkong AAS Hytaý)',
        'zh_Hans_MO' => 'hytaý dili (Ýönekeýleşdirilen, Makao AAS Hytaý)',
        'zh_Hans_MY' => 'hytaý dili (Ýönekeýleşdirilen, Malaýziýa)',
        'zh_Hans_SG' => 'hytaý dili (Ýönekeýleşdirilen, Singapur)',
        'zh_Hant' => 'hytaý dili (Adaty)',
        'zh_Hant_HK' => 'hytaý dili (Adaty, Gonkong AAS Hytaý)',
        'zh_Hant_MO' => 'hytaý dili (Adaty, Makao AAS Hytaý)',
        'zh_Hant_MY' => 'hytaý dili (Adaty, Malaýziýa)',
        'zh_Hant_TW' => 'hytaý dili (Adaty, Taýwan)',
        'zh_MO' => 'hytaý dili (Makao AAS Hytaý)',
        'zh_SG' => 'hytaý dili (Singapur)',
        'zh_TW' => 'hytaý dili (Taýwan)',
        'zu' => 'zulu dili',
        'zu_ZA' => 'zulu dili (Günorta Afrika)',
    ],
];
