<?php

return [
    'Names' => [
        'af' => 'африкаанс',
        'af_NA' => 'африкаанс (Намиби)',
        'af_ZA' => 'африкаанс (Өмнөд Африк)',
        'ak' => 'акан',
        'ak_GH' => 'акан (Гана)',
        'am' => 'амхар',
        'am_ET' => 'амхар (Этиоп)',
        'ar' => 'араб',
        'ar_001' => 'араб (Дэлхий)',
        'ar_AE' => 'араб (Арабын Нэгдсэн Эмират Улс)',
        'ar_BH' => 'араб (Бахрейн)',
        'ar_DJ' => 'араб (Джибути)',
        'ar_DZ' => 'араб (Алжир)',
        'ar_EG' => 'араб (Египет)',
        'ar_EH' => 'араб (Баруун Сахар)',
        'ar_ER' => 'араб (Эритрей)',
        'ar_IL' => 'араб (Израил)',
        'ar_IQ' => 'араб (Ирак)',
        'ar_JO' => 'араб (Йордан)',
        'ar_KM' => 'араб (Коморын арлууд)',
        'ar_KW' => 'араб (Кувейт)',
        'ar_LB' => 'араб (Ливан)',
        'ar_LY' => 'араб (Ливи)',
        'ar_MA' => 'араб (Морокко)',
        'ar_MR' => 'араб (Мавритани)',
        'ar_OM' => 'араб (Оман)',
        'ar_PS' => 'араб (Палестины нутаг дэвсгэр)',
        'ar_QA' => 'араб (Катар)',
        'ar_SA' => 'араб (Саудын Араб)',
        'ar_SD' => 'араб (Судан)',
        'ar_SO' => 'араб (Сомали)',
        'ar_SS' => 'араб (Өмнөд Судан)',
        'ar_SY' => 'араб (Сири)',
        'ar_TD' => 'араб (Чад)',
        'ar_TN' => 'араб (Тунис)',
        'ar_YE' => 'араб (Йемен)',
        'as' => 'ассам',
        'as_IN' => 'ассам (Энэтхэг)',
        'az' => 'азербайжан',
        'az_AZ' => 'азербайжан (Азербайжан)',
        'az_Cyrl' => 'азербайжан (кирилл)',
        'az_Cyrl_AZ' => 'азербайжан (кирилл, Азербайжан)',
        'az_Latn' => 'азербайжан (латин)',
        'az_Latn_AZ' => 'азербайжан (латин, Азербайжан)',
        'ba' => 'башкир',
        'ba_RU' => 'башкир (Орос)',
        'be' => 'беларусь',
        'be_BY' => 'беларусь (Беларусь)',
        'bg' => 'болгар',
        'bg_BG' => 'болгар (Болгар)',
        'bm' => 'бамбара',
        'bm_ML' => 'бамбара (Мали)',
        'bn' => 'бенгал',
        'bn_BD' => 'бенгал (Бангладеш)',
        'bn_IN' => 'бенгал (Энэтхэг)',
        'bo' => 'төвд',
        'bo_CN' => 'төвд (Хятад)',
        'bo_IN' => 'төвд (Энэтхэг)',
        'br' => 'бретон',
        'br_FR' => 'бретон (Франц)',
        'bs' => 'босни',
        'bs_BA' => 'босни (Босни-Херцеговин)',
        'bs_Cyrl' => 'босни (кирилл)',
        'bs_Cyrl_BA' => 'босни (кирилл, Босни-Херцеговин)',
        'bs_Latn' => 'босни (латин)',
        'bs_Latn_BA' => 'босни (латин, Босни-Херцеговин)',
        'ca' => 'каталан',
        'ca_AD' => 'каталан (Андорра)',
        'ca_ES' => 'каталан (Испани)',
        'ca_FR' => 'каталан (Франц)',
        'ca_IT' => 'каталан (Итали)',
        'ce' => 'чечень',
        'ce_RU' => 'чечень (Орос)',
        'cs' => 'чех',
        'cs_CZ' => 'чех (Чех)',
        'cv' => 'чуваш',
        'cv_RU' => 'чуваш (Орос)',
        'cy' => 'уэльс',
        'cy_GB' => 'уэльс (Их Британи)',
        'da' => 'дани',
        'da_DK' => 'дани (Дани)',
        'da_GL' => 'дани (Гренланд)',
        'de' => 'герман',
        'de_AT' => 'герман (Австри)',
        'de_BE' => 'герман (Бельги)',
        'de_CH' => 'герман (Швейцар)',
        'de_DE' => 'герман (Герман)',
        'de_IT' => 'герман (Итали)',
        'de_LI' => 'герман (Лихтенштайн)',
        'de_LU' => 'герман (Люксембург)',
        'dz' => 'зонха',
        'dz_BT' => 'зонха (Бутан)',
        'ee' => 'эвэ',
        'ee_GH' => 'эвэ (Гана)',
        'ee_TG' => 'эвэ (Того)',
        'el' => 'грек',
        'el_CY' => 'грек (Кипр)',
        'el_GR' => 'грек (Грек)',
        'en' => 'англи',
        'en_001' => 'англи (Дэлхий)',
        'en_150' => 'англи (Европ)',
        'en_AE' => 'англи (Арабын Нэгдсэн Эмират Улс)',
        'en_AG' => 'англи (Антигуа ба Барбуда)',
        'en_AI' => 'англи (Ангилья)',
        'en_AS' => 'англи (Америкийн Самоа)',
        'en_AT' => 'англи (Австри)',
        'en_AU' => 'англи (Австрали)',
        'en_BB' => 'англи (Барбадос)',
        'en_BE' => 'англи (Бельги)',
        'en_BI' => 'англи (Бурунди)',
        'en_BM' => 'англи (Бермуда)',
        'en_BS' => 'англи (Багамын арлууд)',
        'en_BW' => 'англи (Ботсван)',
        'en_BZ' => 'англи (Белизе)',
        'en_CA' => 'англи (Канад)',
        'en_CC' => 'англи (Кокос [Кийлинг] арлууд)',
        'en_CH' => 'англи (Швейцар)',
        'en_CK' => 'англи (Күүкийн арлууд)',
        'en_CM' => 'англи (Камерун)',
        'en_CX' => 'англи (Зул сарын арал)',
        'en_CY' => 'англи (Кипр)',
        'en_CZ' => 'англи (Чех)',
        'en_DE' => 'англи (Герман)',
        'en_DK' => 'англи (Дани)',
        'en_DM' => 'англи (Доминика)',
        'en_EE' => 'англи (Эстони)',
        'en_ER' => 'англи (Эритрей)',
        'en_ES' => 'англи (Испани)',
        'en_FI' => 'англи (Финланд)',
        'en_FJ' => 'англи (Фижи)',
        'en_FK' => 'англи (Фолклендийн арлууд)',
        'en_FM' => 'англи (Микронези)',
        'en_FR' => 'англи (Франц)',
        'en_GB' => 'англи (Их Британи)',
        'en_GD' => 'англи (Гренада)',
        'en_GE' => 'англи (Гүрж)',
        'en_GG' => 'англи (Гернси)',
        'en_GH' => 'англи (Гана)',
        'en_GI' => 'англи (Гибралтар)',
        'en_GM' => 'англи (Гамби)',
        'en_GS' => 'англи (Өмнөд Жоржиа ба Өмнөд Сэндвичийн арлууд)',
        'en_GU' => 'англи (Гуам)',
        'en_GY' => 'англи (Гайана)',
        'en_HK' => 'англи (БНХАУ-ын Тусгай захиргааны бүс Хонг-Конг)',
        'en_HU' => 'англи (Унгар)',
        'en_ID' => 'англи (Индонез)',
        'en_IE' => 'англи (Ирланд)',
        'en_IL' => 'англи (Израил)',
        'en_IM' => 'англи (Мэн Арал)',
        'en_IN' => 'англи (Энэтхэг)',
        'en_IO' => 'англи (Британийн харьяа Энэтхэгийн далай дахь нутаг дэвсгэр)',
        'en_IT' => 'англи (Итали)',
        'en_JE' => 'англи (Жерси)',
        'en_JM' => 'англи (Ямайка)',
        'en_JP' => 'англи (Япон)',
        'en_KE' => 'англи (Кени)',
        'en_KI' => 'англи (Кирибати)',
        'en_KN' => 'англи (Сент-Киттс ба Невис)',
        'en_KY' => 'англи (Кайманы арлууд)',
        'en_LC' => 'англи (Сент Люсиа)',
        'en_LR' => 'англи (Либери)',
        'en_LS' => 'англи (Лесото)',
        'en_LT' => 'англи (Литва)',
        'en_LV' => 'англи (Латви)',
        'en_MG' => 'англи (Мадагаскар)',
        'en_MH' => 'англи (Маршаллын арлууд)',
        'en_MO' => 'англи (БНХАУ-ын Тусгай захиргааны бүс Макао)',
        'en_MP' => 'англи (Хойд Марианы арлууд)',
        'en_MS' => 'англи (Монтсеррат)',
        'en_MT' => 'англи (Мальта)',
        'en_MU' => 'англи (Маврикий)',
        'en_MV' => 'англи (Мальдив)',
        'en_MW' => 'англи (Малави)',
        'en_MY' => 'англи (Малайз)',
        'en_NA' => 'англи (Намиби)',
        'en_NF' => 'англи (Норфолк арал)',
        'en_NG' => 'англи (Нигери)',
        'en_NL' => 'англи (Нидерланд)',
        'en_NO' => 'англи (Норвег)',
        'en_NR' => 'англи (Науру)',
        'en_NU' => 'англи (Ниуэ)',
        'en_NZ' => 'англи (Шинэ Зеланд)',
        'en_PG' => 'англи (Папуа Шинэ Гвиней)',
        'en_PH' => 'англи (Филиппин)',
        'en_PK' => 'англи (Пакистан)',
        'en_PL' => 'англи (Польш)',
        'en_PN' => 'англи (Питкэрн арлууд)',
        'en_PR' => 'англи (Пуэрто-Рико)',
        'en_PT' => 'англи (Португал)',
        'en_PW' => 'англи (Палау)',
        'en_RO' => 'англи (Румын)',
        'en_RW' => 'англи (Руанда)',
        'en_SB' => 'англи (Соломоны арлууд)',
        'en_SC' => 'англи (Сейшелийн арлууд)',
        'en_SD' => 'англи (Судан)',
        'en_SE' => 'англи (Швед)',
        'en_SG' => 'англи (Сингапур)',
        'en_SH' => 'англи (Сент Хелена)',
        'en_SI' => 'англи (Словени)',
        'en_SK' => 'англи (Словак)',
        'en_SL' => 'англи (Сьерра-Леоне)',
        'en_SS' => 'англи (Өмнөд Судан)',
        'en_SX' => 'англи (Синт Мартен)',
        'en_SZ' => 'англи (Эсватини)',
        'en_TC' => 'англи (Турк ба Кайкосын Арлууд)',
        'en_TK' => 'англи (Токелау)',
        'en_TO' => 'англи (Тонга)',
        'en_TT' => 'англи (Тринидад ба Тобаго)',
        'en_TV' => 'англи (Тувалу)',
        'en_TZ' => 'англи (Танзани)',
        'en_UA' => 'англи (Украин)',
        'en_UG' => 'англи (Уганда)',
        'en_UM' => 'англи (Америкийн Нэгдсэн Улсын бага арлууд)',
        'en_US' => 'англи (Америкийн Нэгдсэн Улс)',
        'en_VC' => 'англи (Сент-Винсент ба Гренадин)',
        'en_VG' => 'англи (Британийн Виржиний арлууд)',
        'en_VI' => 'англи (АНУ-ын Виржиний арлууд)',
        'en_VU' => 'англи (Вануату)',
        'en_WS' => 'англи (Самоа)',
        'en_ZA' => 'англи (Өмнөд Африк)',
        'en_ZM' => 'англи (Замби)',
        'en_ZW' => 'англи (Зимбабве)',
        'eo' => 'эсперанто',
        'eo_001' => 'эсперанто (Дэлхий)',
        'es' => 'испани',
        'es_419' => 'испани (Латин Америк)',
        'es_AR' => 'испани (Аргентин)',
        'es_BO' => 'испани (Боливи)',
        'es_BR' => 'испани (Бразил)',
        'es_BZ' => 'испани (Белизе)',
        'es_CL' => 'испани (Чили)',
        'es_CO' => 'испани (Колумби)',
        'es_CR' => 'испани (Коста-Рика)',
        'es_CU' => 'испани (Куба)',
        'es_DO' => 'испани (Бүгд Найрамдах Доминикан Улс)',
        'es_EC' => 'испани (Эквадор)',
        'es_ES' => 'испани (Испани)',
        'es_GQ' => 'испани (Экваторын Гвиней)',
        'es_GT' => 'испани (Гватемал)',
        'es_HN' => 'испани (Гондурас)',
        'es_MX' => 'испани (Мексик)',
        'es_NI' => 'испани (Никарагуа)',
        'es_PA' => 'испани (Панам)',
        'es_PE' => 'испани (Перу)',
        'es_PH' => 'испани (Филиппин)',
        'es_PR' => 'испани (Пуэрто-Рико)',
        'es_PY' => 'испани (Парагвай)',
        'es_SV' => 'испани (Эль Сальвадор)',
        'es_US' => 'испани (Америкийн Нэгдсэн Улс)',
        'es_UY' => 'испани (Уругвай)',
        'es_VE' => 'испани (Венесуэл)',
        'et' => 'эстони',
        'et_EE' => 'эстони (Эстони)',
        'eu' => 'баск',
        'eu_ES' => 'баск (Испани)',
        'fa' => 'перс',
        'fa_AF' => 'перс (Афганистан)',
        'fa_IR' => 'перс (Иран)',
        'ff' => 'фула',
        'ff_Adlm' => 'фула (Адлам бичиг)',
        'ff_Adlm_BF' => 'фула (Адлам бичиг, Буркина Фасо)',
        'ff_Adlm_CM' => 'фула (Адлам бичиг, Камерун)',
        'ff_Adlm_GH' => 'фула (Адлам бичиг, Гана)',
        'ff_Adlm_GM' => 'фула (Адлам бичиг, Гамби)',
        'ff_Adlm_GN' => 'фула (Адлам бичиг, Гвиней)',
        'ff_Adlm_GW' => 'фула (Адлам бичиг, Гвиней-Бисау)',
        'ff_Adlm_LR' => 'фула (Адлам бичиг, Либери)',
        'ff_Adlm_MR' => 'фула (Адлам бичиг, Мавритани)',
        'ff_Adlm_NE' => 'фула (Адлам бичиг, Нигер)',
        'ff_Adlm_NG' => 'фула (Адлам бичиг, Нигери)',
        'ff_Adlm_SL' => 'фула (Адлам бичиг, Сьерра-Леоне)',
        'ff_Adlm_SN' => 'фула (Адлам бичиг, Сенегал)',
        'ff_CM' => 'фула (Камерун)',
        'ff_GN' => 'фула (Гвиней)',
        'ff_Latn' => 'фула (латин)',
        'ff_Latn_BF' => 'фула (латин, Буркина Фасо)',
        'ff_Latn_CM' => 'фула (латин, Камерун)',
        'ff_Latn_GH' => 'фула (латин, Гана)',
        'ff_Latn_GM' => 'фула (латин, Гамби)',
        'ff_Latn_GN' => 'фула (латин, Гвиней)',
        'ff_Latn_GW' => 'фула (латин, Гвиней-Бисау)',
        'ff_Latn_LR' => 'фула (латин, Либери)',
        'ff_Latn_MR' => 'фула (латин, Мавритани)',
        'ff_Latn_NE' => 'фула (латин, Нигер)',
        'ff_Latn_NG' => 'фула (латин, Нигери)',
        'ff_Latn_SL' => 'фула (латин, Сьерра-Леоне)',
        'ff_Latn_SN' => 'фула (латин, Сенегал)',
        'ff_MR' => 'фула (Мавритани)',
        'ff_SN' => 'фула (Сенегал)',
        'fi' => 'фин',
        'fi_FI' => 'фин (Финланд)',
        'fo' => 'фарер',
        'fo_DK' => 'фарер (Дани)',
        'fo_FO' => 'фарер (Фарерын арлууд)',
        'fr' => 'франц',
        'fr_BE' => 'франц (Бельги)',
        'fr_BF' => 'франц (Буркина Фасо)',
        'fr_BI' => 'франц (Бурунди)',
        'fr_BJ' => 'франц (Бенин)',
        'fr_BL' => 'франц (Сент-Бартельми)',
        'fr_CA' => 'франц (Канад)',
        'fr_CD' => 'франц (Конго-Киншаса)',
        'fr_CF' => 'франц (Төв Африкийн Бүгд Найрамдах Улс)',
        'fr_CG' => 'франц (Конго-Браззавиль)',
        'fr_CH' => 'франц (Швейцар)',
        'fr_CI' => 'франц (Кот-д’Ивуар)',
        'fr_CM' => 'франц (Камерун)',
        'fr_DJ' => 'франц (Джибути)',
        'fr_DZ' => 'франц (Алжир)',
        'fr_FR' => 'франц (Франц)',
        'fr_GA' => 'франц (Габон)',
        'fr_GF' => 'франц (Францын Гвиана)',
        'fr_GN' => 'франц (Гвиней)',
        'fr_GP' => 'франц (Гваделуп)',
        'fr_GQ' => 'франц (Экваторын Гвиней)',
        'fr_HT' => 'франц (Гаити)',
        'fr_KM' => 'франц (Коморын арлууд)',
        'fr_LU' => 'франц (Люксембург)',
        'fr_MA' => 'франц (Морокко)',
        'fr_MC' => 'франц (Монако)',
        'fr_MF' => 'франц (Сент-Мартин)',
        'fr_MG' => 'франц (Мадагаскар)',
        'fr_ML' => 'франц (Мали)',
        'fr_MQ' => 'франц (Мартиник)',
        'fr_MR' => 'франц (Мавритани)',
        'fr_MU' => 'франц (Маврикий)',
        'fr_NC' => 'франц (Шинэ Каледони)',
        'fr_NE' => 'франц (Нигер)',
        'fr_PF' => 'франц (Францын Полинез)',
        'fr_PM' => 'франц (Сент-Пьер ба Микело)',
        'fr_RE' => 'франц (Реюнион)',
        'fr_RW' => 'франц (Руанда)',
        'fr_SC' => 'франц (Сейшелийн арлууд)',
        'fr_SN' => 'франц (Сенегал)',
        'fr_SY' => 'франц (Сири)',
        'fr_TD' => 'франц (Чад)',
        'fr_TG' => 'франц (Того)',
        'fr_TN' => 'франц (Тунис)',
        'fr_VU' => 'франц (Вануату)',
        'fr_WF' => 'франц (Уоллис ба Футуна)',
        'fr_YT' => 'франц (Майотта)',
        'fy' => 'баруун фриз',
        'fy_NL' => 'баруун фриз (Нидерланд)',
        'ga' => 'ирланд',
        'ga_GB' => 'ирланд (Их Британи)',
        'ga_IE' => 'ирланд (Ирланд)',
        'gd' => 'шотландын гел',
        'gd_GB' => 'шотландын гел (Их Британи)',
        'gl' => 'галего',
        'gl_ES' => 'галего (Испани)',
        'gu' => 'гужарати',
        'gu_IN' => 'гужарати (Энэтхэг)',
        'gv' => 'манкс',
        'gv_IM' => 'манкс (Мэн Арал)',
        'ha' => 'хауса',
        'ha_GH' => 'хауса (Гана)',
        'ha_NE' => 'хауса (Нигер)',
        'ha_NG' => 'хауса (Нигери)',
        'he' => 'еврей',
        'he_IL' => 'еврей (Израил)',
        'hi' => 'хинди',
        'hi_IN' => 'хинди (Энэтхэг)',
        'hi_Latn' => 'хинди (латин)',
        'hi_Latn_IN' => 'хинди (латин, Энэтхэг)',
        'hr' => 'хорват',
        'hr_BA' => 'хорват (Босни-Херцеговин)',
        'hr_HR' => 'хорват (Хорват)',
        'hu' => 'мажар',
        'hu_HU' => 'мажар (Унгар)',
        'hy' => 'армен',
        'hy_AM' => 'армен (Армени)',
        'ia' => 'интерлингво',
        'ia_001' => 'интерлингво (Дэлхий)',
        'id' => 'индонези',
        'id_ID' => 'индонези (Индонез)',
        'ie' => 'нэгдмэл хэл',
        'ie_EE' => 'нэгдмэл хэл (Эстони)',
        'ig' => 'игбо',
        'ig_NG' => 'игбо (Нигери)',
        'ii' => 'сычуань и',
        'ii_CN' => 'сычуань и (Хятад)',
        'is' => 'исланд',
        'is_IS' => 'исланд (Исланд)',
        'it' => 'итали',
        'it_CH' => 'итали (Швейцар)',
        'it_IT' => 'итали (Итали)',
        'it_SM' => 'итали (Сан-Марино)',
        'it_VA' => 'итали (Ватикан хот улс)',
        'ja' => 'япон',
        'ja_JP' => 'япон (Япон)',
        'jv' => 'ява',
        'jv_ID' => 'ява (Индонез)',
        'ka' => 'гүрж',
        'ka_GE' => 'гүрж (Гүрж)',
        'ki' => 'кикуюү',
        'ki_KE' => 'кикуюү (Кени)',
        'kk' => 'казах',
        'kk_Arab' => 'казах (араб)',
        'kk_Arab_CN' => 'казах (араб, Хятад)',
        'kk_Cyrl' => 'казах (кирилл)',
        'kk_Cyrl_KZ' => 'казах (кирилл, Казахстан)',
        'kk_KZ' => 'казах (Казахстан)',
        'kl' => 'калалисут',
        'kl_GL' => 'калалисут (Гренланд)',
        'km' => 'кхмер',
        'km_KH' => 'кхмер (Камбож)',
        'kn' => 'каннада',
        'kn_IN' => 'каннада (Энэтхэг)',
        'ko' => 'солонгос',
        'ko_CN' => 'солонгос (Хятад)',
        'ko_KP' => 'солонгос (Хойд Солонгос)',
        'ko_KR' => 'солонгос (Өмнөд Солонгос)',
        'ks' => 'кашмир',
        'ks_Arab' => 'кашмир (араб)',
        'ks_Arab_IN' => 'кашмир (араб, Энэтхэг)',
        'ks_Deva' => 'кашмир (деванагари)',
        'ks_Deva_IN' => 'кашмир (деванагари, Энэтхэг)',
        'ks_IN' => 'кашмир (Энэтхэг)',
        'ku' => 'курд',
        'ku_Latn' => 'курд (латин)',
        'ku_Latn_IQ' => 'курд (латин, Ирак)',
        'ku_Latn_SY' => 'курд (латин, Сири)',
        'ku_Latn_TR' => 'курд (латин, Турк)',
        'ku_SY' => 'курд (Сири)',
        'ku_TR' => 'курд (Турк)',
        'kw' => 'корн',
        'kw_GB' => 'корн (Их Британи)',
        'ky' => 'киргиз',
        'ky_KG' => 'киргиз (Киргиз)',
        'lb' => 'люксембург',
        'lb_LU' => 'люксембург (Люксембург)',
        'lg' => 'ганда',
        'lg_UG' => 'ганда (Уганда)',
        'ln' => 'лингала',
        'ln_AO' => 'лингала (Ангол)',
        'ln_CD' => 'лингала (Конго-Киншаса)',
        'ln_CF' => 'лингала (Төв Африкийн Бүгд Найрамдах Улс)',
        'ln_CG' => 'лингала (Конго-Браззавиль)',
        'lo' => 'лаос',
        'lo_LA' => 'лаос (Лаос)',
        'lt' => 'литва',
        'lt_LT' => 'литва (Литва)',
        'lu' => 'луба-катанга',
        'lu_CD' => 'луба-катанга (Конго-Киншаса)',
        'lv' => 'латви',
        'lv_LV' => 'латви (Латви)',
        'mg' => 'малагаси',
        'mg_MG' => 'малагаси (Мадагаскар)',
        'mi' => 'маори',
        'mi_NZ' => 'маори (Шинэ Зеланд)',
        'mk' => 'македон',
        'mk_MK' => 'македон (Хойд Македон)',
        'ml' => 'малаялам',
        'ml_IN' => 'малаялам (Энэтхэг)',
        'mn' => 'монгол',
        'mn_MN' => 'монгол (Монгол)',
        'mr' => 'марати',
        'mr_IN' => 'марати (Энэтхэг)',
        'ms' => 'малай',
        'ms_BN' => 'малай (Бруней)',
        'ms_ID' => 'малай (Индонез)',
        'ms_MY' => 'малай (Малайз)',
        'ms_SG' => 'малай (Сингапур)',
        'mt' => 'малта',
        'mt_MT' => 'малта (Мальта)',
        'my' => 'бирм',
        'my_MM' => 'бирм (Мьянмар)',
        'nb' => 'норвегийн букмол',
        'nb_NO' => 'норвегийн букмол (Норвег)',
        'nb_SJ' => 'норвегийн букмол (Свалбард ба Ян Майен)',
        'nd' => 'хойд ндебеле',
        'nd_ZW' => 'хойд ндебеле (Зимбабве)',
        'ne' => 'балба',
        'ne_IN' => 'балба (Энэтхэг)',
        'ne_NP' => 'балба (Балба)',
        'nl' => 'нидерланд',
        'nl_AW' => 'нидерланд (Аруба)',
        'nl_BE' => 'нидерланд (Бельги)',
        'nl_BQ' => 'нидерланд (Карибын Нидерланд)',
        'nl_CW' => 'нидерланд (Кюрасао)',
        'nl_NL' => 'нидерланд (Нидерланд)',
        'nl_SR' => 'нидерланд (Суринам)',
        'nl_SX' => 'нидерланд (Синт Мартен)',
        'nn' => 'норвегийн нинорск',
        'nn_NO' => 'норвегийн нинорск (Норвег)',
        'no' => 'норвег',
        'no_NO' => 'норвег (Норвег)',
        'oc' => 'окситан',
        'oc_ES' => 'окситан (Испани)',
        'oc_FR' => 'окситан (Франц)',
        'om' => 'оромо',
        'om_ET' => 'оромо (Этиоп)',
        'om_KE' => 'оромо (Кени)',
        'or' => 'ория',
        'or_IN' => 'ория (Энэтхэг)',
        'os' => 'оссетин',
        'os_GE' => 'оссетин (Гүрж)',
        'os_RU' => 'оссетин (Орос)',
        'pa' => 'панжаби',
        'pa_Arab' => 'панжаби (араб)',
        'pa_Arab_PK' => 'панжаби (араб, Пакистан)',
        'pa_Guru' => 'панжаби (гүрмүх)',
        'pa_Guru_IN' => 'панжаби (гүрмүх, Энэтхэг)',
        'pa_IN' => 'панжаби (Энэтхэг)',
        'pa_PK' => 'панжаби (Пакистан)',
        'pl' => 'польш',
        'pl_PL' => 'польш (Польш)',
        'ps' => 'пушту',
        'ps_AF' => 'пушту (Афганистан)',
        'ps_PK' => 'пушту (Пакистан)',
        'pt' => 'португал',
        'pt_AO' => 'португал (Ангол)',
        'pt_BR' => 'португал (Бразил)',
        'pt_CH' => 'португал (Швейцар)',
        'pt_CV' => 'португал (Кабо-Верде)',
        'pt_GQ' => 'португал (Экваторын Гвиней)',
        'pt_GW' => 'португал (Гвиней-Бисау)',
        'pt_LU' => 'португал (Люксембург)',
        'pt_MO' => 'португал (БНХАУ-ын Тусгай захиргааны бүс Макао)',
        'pt_MZ' => 'португал (Мозамбик)',
        'pt_PT' => 'португал (Португал)',
        'pt_ST' => 'португал (Сан-Томе ба Принсипи)',
        'pt_TL' => 'португал (Тимор-Лесте)',
        'qu' => 'кечуа',
        'qu_BO' => 'кечуа (Боливи)',
        'qu_EC' => 'кечуа (Эквадор)',
        'qu_PE' => 'кечуа (Перу)',
        'rm' => 'романш',
        'rm_CH' => 'романш (Швейцар)',
        'rn' => 'рунди',
        'rn_BI' => 'рунди (Бурунди)',
        'ro' => 'румын',
        'ro_MD' => 'румын (Молдова)',
        'ro_RO' => 'румын (Румын)',
        'ru' => 'орос',
        'ru_BY' => 'орос (Беларусь)',
        'ru_KG' => 'орос (Киргиз)',
        'ru_KZ' => 'орос (Казахстан)',
        'ru_MD' => 'орос (Молдова)',
        'ru_RU' => 'орос (Орос)',
        'ru_UA' => 'орос (Украин)',
        'rw' => 'киньяруанда',
        'rw_RW' => 'киньяруанда (Руанда)',
        'sa' => 'санскрит',
        'sa_IN' => 'санскрит (Энэтхэг)',
        'sc' => 'сардин',
        'sc_IT' => 'сардин (Итали)',
        'sd' => 'синдхи',
        'sd_Arab' => 'синдхи (араб)',
        'sd_Arab_PK' => 'синдхи (араб, Пакистан)',
        'sd_Deva' => 'синдхи (деванагари)',
        'sd_Deva_IN' => 'синдхи (деванагари, Энэтхэг)',
        'sd_IN' => 'синдхи (Энэтхэг)',
        'sd_PK' => 'синдхи (Пакистан)',
        'se' => 'хойд сами',
        'se_FI' => 'хойд сами (Финланд)',
        'se_NO' => 'хойд сами (Норвег)',
        'se_SE' => 'хойд сами (Швед)',
        'sg' => 'санго',
        'sg_CF' => 'санго (Төв Африкийн Бүгд Найрамдах Улс)',
        'sh' => 'хорватын серб',
        'sh_BA' => 'хорватын серб (Босни-Херцеговин)',
        'si' => 'синхала',
        'si_LK' => 'синхала (Шри-Ланка)',
        'sk' => 'словак',
        'sk_SK' => 'словак (Словак)',
        'sl' => 'словени',
        'sl_SI' => 'словени (Словени)',
        'sn' => 'шона',
        'sn_ZW' => 'шона (Зимбабве)',
        'so' => 'сомали',
        'so_DJ' => 'сомали (Джибути)',
        'so_ET' => 'сомали (Этиоп)',
        'so_KE' => 'сомали (Кени)',
        'so_SO' => 'сомали (Сомали)',
        'sq' => 'албани',
        'sq_AL' => 'албани (Албани)',
        'sq_MK' => 'албани (Хойд Македон)',
        'sr' => 'серб',
        'sr_BA' => 'серб (Босни-Херцеговин)',
        'sr_Cyrl' => 'серб (кирилл)',
        'sr_Cyrl_BA' => 'серб (кирилл, Босни-Херцеговин)',
        'sr_Cyrl_ME' => 'серб (кирилл, Монтенегро)',
        'sr_Cyrl_RS' => 'серб (кирилл, Серби)',
        'sr_Latn' => 'серб (латин)',
        'sr_Latn_BA' => 'серб (латин, Босни-Херцеговин)',
        'sr_Latn_ME' => 'серб (латин, Монтенегро)',
        'sr_Latn_RS' => 'серб (латин, Серби)',
        'sr_ME' => 'серб (Монтенегро)',
        'sr_RS' => 'серб (Серби)',
        'st' => 'сесото',
        'st_LS' => 'сесото (Лесото)',
        'st_ZA' => 'сесото (Өмнөд Африк)',
        'su' => 'сундан',
        'su_ID' => 'сундан (Индонез)',
        'su_Latn' => 'сундан (латин)',
        'su_Latn_ID' => 'сундан (латин, Индонез)',
        'sv' => 'швед',
        'sv_AX' => 'швед (Аландын арлууд)',
        'sv_FI' => 'швед (Финланд)',
        'sv_SE' => 'швед (Швед)',
        'sw' => 'свахили',
        'sw_CD' => 'свахили (Конго-Киншаса)',
        'sw_KE' => 'свахили (Кени)',
        'sw_TZ' => 'свахили (Танзани)',
        'sw_UG' => 'свахили (Уганда)',
        'ta' => 'тамил',
        'ta_IN' => 'тамил (Энэтхэг)',
        'ta_LK' => 'тамил (Шри-Ланка)',
        'ta_MY' => 'тамил (Малайз)',
        'ta_SG' => 'тамил (Сингапур)',
        'te' => 'тэлүгү',
        'te_IN' => 'тэлүгү (Энэтхэг)',
        'tg' => 'тажик',
        'tg_TJ' => 'тажик (Тажикистан)',
        'th' => 'тай',
        'th_TH' => 'тай (Тайланд)',
        'ti' => 'тигринья',
        'ti_ER' => 'тигринья (Эритрей)',
        'ti_ET' => 'тигринья (Этиоп)',
        'tk' => 'туркмен',
        'tk_TM' => 'туркмен (Туркменистан)',
        'tn' => 'цвана',
        'tn_BW' => 'цвана (Ботсван)',
        'tn_ZA' => 'цвана (Өмнөд Африк)',
        'to' => 'тонга',
        'to_TO' => 'тонга (Тонга)',
        'tr' => 'турк',
        'tr_CY' => 'турк (Кипр)',
        'tr_TR' => 'турк (Турк)',
        'tt' => 'татар',
        'tt_RU' => 'татар (Орос)',
        'ug' => 'уйгур',
        'ug_CN' => 'уйгур (Хятад)',
        'uk' => 'украин',
        'uk_UA' => 'украин (Украин)',
        'ur' => 'урду',
        'ur_IN' => 'урду (Энэтхэг)',
        'ur_PK' => 'урду (Пакистан)',
        'uz' => 'узбек',
        'uz_AF' => 'узбек (Афганистан)',
        'uz_Arab' => 'узбек (араб)',
        'uz_Arab_AF' => 'узбек (араб, Афганистан)',
        'uz_Cyrl' => 'узбек (кирилл)',
        'uz_Cyrl_UZ' => 'узбек (кирилл, Узбекистан)',
        'uz_Latn' => 'узбек (латин)',
        'uz_Latn_UZ' => 'узбек (латин, Узбекистан)',
        'uz_UZ' => 'узбек (Узбекистан)',
        'vi' => 'вьетнам',
        'vi_VN' => 'вьетнам (Вьетнам)',
        'wo' => 'волоф',
        'wo_SN' => 'волоф (Сенегал)',
        'xh' => 'хоса',
        'xh_ZA' => 'хоса (Өмнөд Африк)',
        'yi' => 'иддиш',
        'yi_UA' => 'иддиш (Украин)',
        'yo' => 'ёруба',
        'yo_BJ' => 'ёруба (Бенин)',
        'yo_NG' => 'ёруба (Нигери)',
        'za' => 'чжуанг',
        'za_CN' => 'чжуанг (Хятад)',
        'zh' => 'хятад',
        'zh_CN' => 'хятад (Хятад)',
        'zh_HK' => 'хятад (БНХАУ-ын Тусгай захиргааны бүс Хонг-Конг)',
        'zh_Hans' => 'хятад (хялбаршуулсан)',
        'zh_Hans_CN' => 'хятад (хялбаршуулсан, Хятад)',
        'zh_Hans_HK' => 'хятад (хялбаршуулсан, БНХАУ-ын Тусгай захиргааны бүс Хонг-Конг)',
        'zh_Hans_MO' => 'хятад (хялбаршуулсан, БНХАУ-ын Тусгай захиргааны бүс Макао)',
        'zh_Hans_MY' => 'хятад (хялбаршуулсан, Малайз)',
        'zh_Hans_SG' => 'хятад (хялбаршуулсан, Сингапур)',
        'zh_Hant' => 'хятад (уламжлалт)',
        'zh_Hant_HK' => 'хятад (уламжлалт, БНХАУ-ын Тусгай захиргааны бүс Хонг-Конг)',
        'zh_Hant_MO' => 'хятад (уламжлалт, БНХАУ-ын Тусгай захиргааны бүс Макао)',
        'zh_Hant_MY' => 'хятад (уламжлалт, Малайз)',
        'zh_Hant_TW' => 'хятад (уламжлалт, Тайвань)',
        'zh_MO' => 'хятад (БНХАУ-ын Тусгай захиргааны бүс Макао)',
        'zh_SG' => 'хятад (Сингапур)',
        'zh_TW' => 'хятад (Тайвань)',
        'zu' => 'зулу',
        'zu_ZA' => 'зулу (Өмнөд Африк)',
    ],
];
