<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\HttpKernel\Tests\EventListener;

use PHPUnit\Framework\Attributes\TestWith;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Console\ConsoleEvents;
use Symfony\Component\Console\Event\ConsoleCommandEvent;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\BufferedOutput;
use Symfony\Component\HttpKernel\EventListener\DumpListener;
use Symfony\Component\VarDumper\Cloner\ClonerInterface;
use Symfony\Component\VarDumper\Cloner\Data;
use Symfony\Component\VarDumper\Dumper\DataDumperInterface;
use Symfony\Component\VarDumper\VarDumper;

/**
 * DumpListenerTest.
 *
 * @author Nicolas Grekas <p@tchwork.com>
 */
class DumpListenerTest extends TestCase
{
    public function testSubscribedEvents()
    {
        $this->assertSame(
            [ConsoleEvents::COMMAND => ['configure', 1024]],
            DumpListener::getSubscribedEvents()
        );
    }

    public function testConfigure()
    {
        $prevDumper = VarDumper::setHandler('var_dump');
        VarDumper::setHandler($prevDumper);

        $cloner = new MockCloner();
        $dumper = new MockDumper();

        ob_start();
        $exception = null;
        $listener = new DumpListener($cloner, $dumper);

        try {
            $listener->configure();

            VarDumper::dump('foo');
            VarDumper::dump('bar');

            $this->assertSame('+foo-+bar-', ob_get_clean());
        } catch (\Exception $exception) {
        }

        VarDumper::setHandler($prevDumper);

        if (null !== $exception) {
            throw $exception;
        }
    }

    #[TestWith([false, false, '+foo-+bar-', []])]
    #[TestWith([true, false, '+foo-+bar-', []])]
    #[TestWith([true, true, '', ['foo-', 'bar-']])]
    public function testConfigureWithProfilerDumper(bool $hasOption, bool $option, string $expectedOutput, array $expectedData)
    {
        $prevDumper = VarDumper::setHandler('var_dump');
        VarDumper::setHandler($prevDumper);

        $cloner = new MockCloner();
        $dumper = new MockDumper();
        $profilerDumper = new MockProfilerDumper();

        $input = $this->createStub(InputInterface::class);
        $input->method('hasOption')->willReturn($hasOption);
        $input->method('getOption')->willReturn($option);

        ob_start();
        $exception = null;
        $listener = new DumpListener($cloner, $dumper, null, $profilerDumper);

        try {
            $listener->configure(new ConsoleCommandEvent(null, $input, new BufferedOutput()));

            VarDumper::dump('foo');
            VarDumper::dump('bar');

            $this->assertSame($expectedOutput, ob_get_clean());
            $this->assertSame($expectedData, $profilerDumper->data);
        } catch (\Exception $exception) {
        }

        VarDumper::setHandler($prevDumper);

        if (null !== $exception) {
            throw $exception;
        }
    }
}

class MockCloner implements ClonerInterface
{
    public function cloneVar($var): Data
    {
        return new Data([[$var.'-']]);
    }
}

class MockDumper implements DataDumperInterface
{
    public function dump(Data $data): ?string
    {
        echo '+'.$data->getValue();

        return null;
    }
}

class MockProfilerDumper implements DataDumperInterface
{
    public array $data = [];

    public function dump(Data $data): ?string
    {
        $this->data[] = $data->getValue();

        return null;
    }
}
