<?php

return [
    'Names' => [
        'Adlm' => 'adlam',
        'Arab' => 'ərəb',
        'Aran' => 'aran',
        'Armi' => 'armi',
        'Armn' => 'erməni',
        'Avst' => 'avestan',
        'Bali' => 'bali',
        'Batk' => 'batak',
        'Beng' => 'benqal',
        'Blis' => 'blissymbols',
        'Bopo' => 'bopomofo',
        'Brah' => 'brahmi',
        'Brai' => 'brayl',
        'Bugi' => 'buqin',
        'Buhd' => 'buhid',
        'Cakm' => 'çakma',
        'Cans' => 'birləşmiş kanada yerli yazısı',
        'Cari' => 'kariyan',
        'Cham' => 'çam',
        'Cher' => 'çiroki',
        'Cirt' => 'sirt',
        'Copt' => 'koptik',
        'Cprt' => 'kipr',
        'Cyrl' => 'kiril',
        'Cyrs' => 'qədimi kilsa kirili',
        'Deva' => 'devanaqari',
        'Dsrt' => 'deseret',
        'Egyd' => 'misir demotik',
        'Egyh' => 'misir hiyeratik',
        'Egyp' => 'misir hiyeroqlif',
        'Ethi' => 'efiop',
        'Geok' => 'gürcü xutsuri',
        'Geor' => 'gürcü',
        'Glag' => 'qlaqolitik',
        'Goth' => 'qotik',
        'Grek' => 'yunan',
        'Gujr' => 'qucarat',
        'Guru' => 'qurmuxi',
        'Hanb' => 'hanb',
        'Hang' => 'hanqıl',
        'Hani' => 'han',
        'Hano' => 'hanunu',
        'Hans' => 'sadələşmiş',
        'Hant' => 'ənənəvi',
        'Hebr' => 'ibrani',
        'Hira' => 'iraqana',
        'Hmng' => 'pahav monq',
        'Hrkt' => 'hecalı yapon əlifbası',
        'Hung' => 'qədimi macar',
        'Inds' => 'hindistan',
        'Ital' => 'qədimi italyalı',
        'Jamo' => 'jamo',
        'Java' => 'cava',
        'Jpan' => 'yapon',
        'Kali' => 'kayax li',
        'Kana' => 'katakana',
        'Khar' => 'xaroşti',
        'Khmr' => 'kxmer',
        'Knda' => 'kannada',
        'Kore' => 'koreya',
        'Kthi' => 'kti',
        'Lana' => 'lanna',
        'Laoo' => 'lao',
        'Latf' => 'fraktur latını',
        'Latg' => 'gael latını',
        'Latn' => 'latın',
        'Lepc' => 'lepçə',
        'Limb' => 'limbu',
        'Lyci' => 'lusian',
        'Lydi' => 'ludian',
        'Mand' => 'mandayen',
        'Mani' => 'maniçayen',
        'Maya' => 'maya hiyeroqlifi',
        'Mero' => 'meroytik',
        'Mlym' => 'malayalam',
        'Mong' => 'monqol',
        'Moon' => 'mun',
        'Mtei' => 'meytey mayek',
        'Mymr' => 'myanmar',
        'Nkoo' => 'nko',
        'Ogam' => 'oğam',
        'Olck' => 'ol çiki',
        'Orkh' => 'orxon',
        'Orya' => 'oriya',
        'Osma' => 'osmanya',
        'Perm' => 'qədimi permik',
        'Phag' => 'faqs-pa',
        'Phli' => 'fli',
        'Phlp' => 'flp',
        'Phlv' => 'kitab paxlavi',
        'Phnx' => 'foenik',
        'Plrd' => 'polard fonetik',
        'Prti' => 'prti',
        'Rjng' => 'recəng',
        'Rohg' => 'hanifi',
        'Roro' => 'ronqoronqo',
        'Runr' => 'runik',
        'Samr' => 'samaritan',
        'Sara' => 'sarati',
        'Saur' => 'saurastra',
        'Sgnw' => 'işarət yazısı',
        'Shaw' => 'şavyan',
        'Sinh' => 'sinhal',
        'Sund' => 'sundan',
        'Sylo' => 'siloti nəqri',
        'Syrc' => 'siryak',
        'Syre' => 'estrangela süryanice',
        'Tagb' => 'taqbanva',
        'Tale' => 'tay le',
        'Talu' => 'təzə tay lu',
        'Taml' => 'tamil',
        'Tavt' => 'tavt',
        'Telu' => 'teluqu',
        'Teng' => 'tengvar',
        'Tfng' => 'tifinaq',
        'Tglg' => 'taqaloq',
        'Thaa' => 'thana',
        'Thai' => 'tay',
        'Tibt' => 'tibet',
        'Ugar' => 'uqarit',
        'Vaii' => 'vay',
        'Visp' => 'danışma səsləri',
        'Xpeo' => 'qədimi fars',
        'Xsux' => 'sumer-akadyan kuneyform',
        'Yiii' => 'yi',
        'Zmth' => 'riyazi notasiya',
        'Zsye' => 'emoji',
        'Zsym' => 'simvollar',
        'Zxxx' => 'yazısız',
        'Zyyy' => 'ümumi yazı',
    ],
];
