<?php

return [
    'Names' => [
        'aa' => 'Afar',
        'ab' => 'Abchasais',
        'ace' => 'Basa Acèh',
        'ach' => 'Acoli',
        'ada' => 'Adangme',
        'ady' => 'Adyghe',
        'ae' => 'Avestanais',
        'aeb' => 'Arabais Thuiniseach',
        'af' => 'Afraganais',
        'afh' => 'Afrihili',
        'agq' => 'Aghem',
        'ain' => 'Ainu',
        'ak' => 'Akan',
        'akk' => 'Acadais',
        'akz' => 'Alabama',
        'ale' => 'Aleutais',
        'aln' => 'Albàinis Ghegeach',
        'alt' => 'Altais Dheasach',
        'am' => 'Amtharais',
        'an' => 'Aragonais',
        'ang' => 'Seann-Bheurla',
        'ann' => 'Obolo',
        'anp' => 'Angika',
        'apc' => 'Arabais Levantach',
        'ar' => 'Arabais',
        'arc' => 'Aramais',
        'arn' => 'Mapudungun',
        'aro' => 'Araona',
        'arp' => 'Arapaho',
        'arq' => 'Arabais Aildireach',
        'ars' => 'Arabais Najdi',
        'arw' => 'Arawak',
        'ary' => 'Arabais Mhorocach',
        'arz' => 'Arabais Èipheiteach',
        'as' => 'Asamais',
        'asa' => 'Asu',
        'ase' => 'Cainnt-shanais na h-Aimeireaga',
        'ast' => 'Astùrais',
        'atj' => 'Atikamekw',
        'av' => 'Avarais',
        'avk' => 'Kotava',
        'awa' => 'Awadhi',
        'ay' => 'Aymara',
        'az' => 'Asarbaideànais',
        'ba' => 'Bashkir',
        'bal' => 'Baluchì',
        'ban' => 'Cànan Bali',
        'bas' => 'Basaa',
        'bax' => 'Bamun',
        'bbc' => 'Batak Toba',
        'bbj' => 'Ghomala',
        'be' => 'Bealaruisis',
        'bej' => 'Beja',
        'bem' => 'Bemba',
        'bew' => 'Betawi',
        'bez' => 'Bena',
        'bfd' => 'Bafut',
        'bfq' => 'Badaga',
        'bg' => 'Bulgarais',
        'bgc' => 'Haryanvi',
        'bgn' => 'Balochi Shiarach',
        'bho' => 'Bhojpuri',
        'bi' => 'Bislama',
        'bik' => 'Bikol',
        'bin' => 'Edo',
        'bjn' => 'Banjar',
        'bkm' => 'Kom',
        'bla' => 'Siksika',
        'blo' => 'Anii',
        'blt' => 'Tai Dam',
        'bm' => 'Bambara',
        'bn' => 'Bangla',
        'bo' => 'Tibeitis',
        'bpy' => 'Bishnupriya',
        'bqi' => 'Bakhtiari',
        'br' => 'Breatnais',
        'bra' => 'Braj',
        'brh' => 'Brahui',
        'brx' => 'Bodo',
        'bs' => 'Bosnais',
        'bss' => 'Akoose',
        'bua' => 'Buriat',
        'bug' => 'Cànan nam Bugis',
        'bum' => 'Bulu',
        'byn' => 'Blin',
        'byv' => 'Medumba',
        'ca' => 'Catalanais',
        'cad' => 'Caddo',
        'car' => 'Carib',
        'cay' => 'Cayuga',
        'cch' => 'Atsam',
        'ccp' => 'Chakma',
        'ce' => 'Deideanais',
        'ceb' => 'Cebuano',
        'cgg' => 'Chiga',
        'ch' => 'Chamorro',
        'chb' => 'Chibcha',
        'chg' => 'Chagatai',
        'chk' => 'Cànan Chuuk',
        'chm' => 'Mari',
        'chn' => 'Chinuk Wawa',
        'cho' => 'Choctaw',
        'chp' => 'Chipewyan',
        'chr' => 'Cherokee',
        'chy' => 'Cheyenne',
        'cic' => 'Chickasaw',
        'ckb' => 'Cùrdais Mheadhanach',
        'clc' => 'Chilcotin',
        'co' => 'Corsais',
        'cop' => 'Coptais',
        'cps' => 'Capiznon',
        'cr' => 'Cree',
        'crg' => 'Michif',
        'crh' => 'Turcais Chriomach',
        'crj' => 'Cree Ear-dheasach',
        'crk' => 'Cree nam Machair',
        'crl' => 'Cree Ear-thuathach',
        'crm' => 'Moose Cree',
        'crr' => 'Algonquianais Charolina',
        'crs' => 'Seiseallais',
        'cs' => 'Seicis',
        'csb' => 'Caisiubais',
        'csw' => 'Omushkego',
        'cu' => 'Slàbhais na h-Eaglaise',
        'cv' => 'Chuvash',
        'cy' => 'Cuimris',
        'da' => 'Danmhairgis',
        'dak' => 'Dakota',
        'dar' => 'Dargwa',
        'dav' => 'Taita',
        'de' => 'Gearmailtis',
        'del' => 'Delaware',
        'den' => 'Slavey',
        'dgr' => 'Dogrib',
        'din' => 'Dinka',
        'dje' => 'Zarma',
        'doi' => 'Dogri',
        'dsb' => 'Sòrbais Ìochdarach',
        'dtp' => 'Dusun Mheadhanach',
        'dua' => 'Duala',
        'dum' => 'Meadhan-Dhuitsis',
        'dv' => 'Divehi',
        'dyo' => 'Jola-Fonyi',
        'dyu' => 'Dyula',
        'dz' => 'Dzongkha',
        'dzg' => 'Dazaga',
        'ebu' => 'Embu',
        'ee' => 'Ewe',
        'efi' => 'Efik',
        'egy' => 'Èipheitis Àrsaidh',
        'eka' => 'Ekajuk',
        'el' => 'Greugais',
        'elx' => 'Elamais',
        'en' => 'Beurla',
        'enm' => 'Meadhan-Bheurla',
        'eo' => 'Esperanto',
        'es' => 'Spàinntis',
        'esu' => 'Yupik Mheadhanach',
        'et' => 'Eastoinis',
        'eu' => 'Basgais',
        'ewo' => 'Ewondo',
        'ext' => 'Cànan na h-Extremadura',
        'fa' => 'Peirsis',
        'fan' => 'Fang',
        'fat' => 'Fanti',
        'ff' => 'Fulah',
        'fi' => 'Fionnlannais',
        'fil' => 'Filipinis',
        'fit' => 'Meänkieli',
        'fj' => 'Fìdis',
        'fo' => 'Fàrothais',
        'fon' => 'Fon',
        'fr' => 'Fraingis',
        'frc' => 'Fraingis nan Cajun',
        'frm' => 'Meadhan-Fhraingis',
        'fro' => 'Seann-Fhraingis',
        'frp' => 'Arpitan',
        'frr' => 'Frìoslannais Thuathach',
        'frs' => 'Frìoslannais Earach',
        'fur' => 'Friùilis',
        'fy' => 'Frìoslannais Shiarach',
        'ga' => 'Gaeilge',
        'gaa' => 'Ga',
        'gag' => 'Gagauz',
        'gan' => 'Gan',
        'gay' => 'Gayo',
        'gba' => 'Gbaya',
        'gbz' => 'Dari Zoroastrach',
        'gd' => 'Gàidhlig',
        'gez' => 'Ge’ez',
        'gil' => 'Ciribeasais',
        'gl' => 'Gailìsis',
        'glk' => 'Gilaki',
        'gmh' => 'Meadhan-Àrd-Gearmailtis',
        'gn' => 'Guaraní',
        'goh' => 'Seann-Àrd-Gearmailtis',
        'gon' => 'Gondi',
        'gor' => 'Gorontalo',
        'got' => 'Gotais',
        'grb' => 'Grebo',
        'grc' => 'Greugais Àrsaidh',
        'gsw' => 'Gearmailtis Eilbheiseach',
        'gu' => 'Gujarati',
        'guc' => 'Wayuu',
        'gur' => 'Frafra',
        'guz' => 'Gusii',
        'gv' => 'Gaelg',
        'gwi' => 'Gwichʼin',
        'ha' => 'Hausa',
        'hai' => 'Haida',
        'hak' => 'Hakka',
        'haw' => 'Cànan Hawai’i',
        'hax' => 'Haida Dheasach',
        'he' => 'Eabhra',
        'hi' => 'Hindis',
        'hif' => 'Hindis Fhìditheach',
        'hil' => 'Hiligaynon',
        'hit' => 'Cànan Het',
        'hmn' => 'Hmong',
        'hnj' => 'Hmong Njua',
        'ho' => 'Hiri Motu',
        'hr' => 'Cròthaisis',
        'hsb' => 'Sòrbais Uachdarach',
        'hsn' => 'Xiang',
        'ht' => 'Crìtheol Haidhti',
        'hu' => 'Ungairis',
        'hup' => 'Hupa',
        'hur' => 'Halkomelem',
        'hy' => 'Airmeinis',
        'hz' => 'Herero',
        'ia' => 'Interlingua',
        'iba' => 'Iban',
        'ibb' => 'Ibibio',
        'id' => 'Innd-Innsis',
        'ie' => 'Interlingue',
        'ig' => 'Igbo',
        'ii' => 'Yi Sichuan',
        'ik' => 'Inupiaq',
        'ikt' => 'Inuktitut Shiarach Chanada',
        'ilo' => 'Iloko',
        'inh' => 'Ingush',
        'io' => 'Ido',
        'is' => 'Innis Tìlis',
        'it' => 'Eadailtis',
        'iu' => 'Inuktitut',
        'ja' => 'Seapanais',
        'jam' => 'Beurla Crìtheolach Diameuga',
        'jbo' => 'Lojban',
        'jgo' => 'Ngomba',
        'jmc' => 'Machame',
        'jpr' => 'Peirsis Iùdhach',
        'jrb' => 'Arabais Iùdhach',
        'jv' => 'Deàbhanais',
        'ka' => 'Cairtbheilis',
        'kaa' => 'Kara-Kalpak',
        'kab' => 'Kabyle',
        'kac' => 'Kachin',
        'kaj' => 'Jju',
        'kam' => 'Kamba',
        'kaw' => 'Kawi',
        'kbd' => 'Cabardais',
        'kbl' => 'Kanembu',
        'kcg' => 'Tyap',
        'kde' => 'Makonde',
        'kea' => 'Kabuverdianu',
        'kek' => 'Qʼeqchiʼ',
        'ken' => 'Kenyang',
        'kfo' => 'Koro',
        'kg' => 'Kongo',
        'kgp' => 'Kaingang',
        'kha' => 'Khasi',
        'kho' => 'Cànan Khotan',
        'khq' => 'Koyra Chiini',
        'khw' => 'Khowar',
        'ki' => 'Kikuyu',
        'kiu' => 'Kirmanjki',
        'kj' => 'Kuanyama',
        'kk' => 'Casachais',
        'kkj' => 'Kako',
        'kl' => 'Kalaallisut',
        'kln' => 'Kalenjin',
        'km' => 'Cmèar',
        'kmb' => 'Kimbundu',
        'kn' => 'Kannada',
        'ko' => 'Coirèanais',
        'koi' => 'Komi-Permyak',
        'kok' => 'Konkani',
        'kpe' => 'Kpelle',
        'kr' => 'Kanuri',
        'krc' => 'Karachay-Balkar',
        'kri' => 'Krio',
        'krj' => 'Kinaray-a',
        'krl' => 'Cairealais',
        'kru' => 'Kurukh',
        'ks' => 'Caismiris',
        'ksb' => 'Shambala',
        'ksf' => 'Bafia',
        'ksh' => 'Gearmailtis Chologne',
        'ku' => 'Cùrdais',
        'kum' => 'Kumyk',
        'kut' => 'Kutenai',
        'kv' => 'Komi',
        'kw' => 'Còrnais',
        'kwk' => 'Kwakʼwala',
        'kxv' => 'Kuvi',
        'ky' => 'Cìorgasais',
        'la' => 'Laideann',
        'lad' => 'Ladino',
        'lag' => 'Langi',
        'lah' => 'Lahnda',
        'lam' => 'Lamba',
        'lb' => 'Lugsamburgais',
        'lez' => 'Leasgais',
        'lfn' => 'Lingua Franca Nova',
        'lg' => 'Ganda',
        'li' => 'Cànan Limburg',
        'lij' => 'Liogùrais',
        'lil' => 'Lillooet',
        'lkt' => 'Lakhóta',
        'lld' => 'Ladainis',
        'lmo' => 'Lombardais',
        'ln' => 'Lingala',
        'lo' => 'Làtho',
        'lol' => 'Mongo',
        'lou' => 'Crìtheol Louisiana',
        'loz' => 'Lozi',
        'lrc' => 'Luri Thuathach',
        'lsm' => 'Saamia',
        'lt' => 'Liotuainis',
        'ltg' => 'Latgailis',
        'lu' => 'Luba-Katanga',
        'lua' => 'Luba-Lulua',
        'lui' => 'Luiseño',
        'lun' => 'Lunda',
        'luo' => 'Luo',
        'lus' => 'Mizo',
        'luy' => 'Luyia',
        'lv' => 'Laitbheis',
        'lzh' => 'Sìnis an Litreachais',
        'lzz' => 'Laz',
        'mad' => 'Cànan Madhura',
        'maf' => 'Mafa',
        'mag' => 'Magahi',
        'mai' => 'Maithili',
        'mak' => 'Makasar',
        'man' => 'Mandingo',
        'mas' => 'Maasai',
        'mde' => 'Maba',
        'mdf' => 'Moksha',
        'mdr' => 'Mandar',
        'men' => 'Mende',
        'mer' => 'Meru',
        'mfe' => 'Morisyen',
        'mg' => 'Malagasais',
        'mga' => 'Meadhan-Ghaeilge',
        'mgh' => 'Makhuwa-Meetto',
        'mgo' => 'Meta’',
        'mh' => 'Marshallais',
        'mhn' => 'Mócheno',
        'mi' => 'Māori',
        'mic' => 'Mi’kmaq',
        'min' => 'Minangkabau',
        'mk' => 'Masadonais',
        'ml' => 'Malayalam',
        'mn' => 'Mongolais',
        'mnc' => 'Manchu',
        'mni' => 'Manipuri',
        'moe' => 'Innu-aimun',
        'moh' => 'Mohawk',
        'mos' => 'Mossi',
        'mr' => 'Marathi',
        'mrj' => 'Mari Shiarach',
        'ms' => 'Malaidhis',
        'mt' => 'Maltais',
        'mua' => 'Mundang',
        'mus' => 'Creek',
        'mwl' => 'Miorandais',
        'mwr' => 'Marwari',
        'mwv' => 'Mentawai',
        'my' => 'Burmais',
        'mye' => 'Myene',
        'myv' => 'Erzya',
        'mzn' => 'Mazanderani',
        'na' => 'Nabhru',
        'nan' => 'Min Nan',
        'nap' => 'Eadailtis Napoli',
        'naq' => 'Nama',
        'nb' => 'Bokmål na Nirribhidh',
        'nd' => 'Ndebele Thuathach',
        'nds' => 'Gearmailtis Ìochdarach',
        'ne' => 'Neapàlais',
        'new' => 'Newari',
        'ng' => 'Ndonga',
        'nia' => 'Nias',
        'niu' => 'Cànan Niue',
        'njo' => 'Ao Naga',
        'nl' => 'Duitsis',
        'nmg' => 'Kwasio',
        'nn' => 'Nynorsk na Nirribhidh',
        'nnh' => 'Ngiemboon',
        'no' => 'Nirribhis',
        'nog' => 'Nogai',
        'non' => 'Seann-Lochlannais',
        'nov' => 'Novial',
        'nqo' => 'N’Ko',
        'nr' => 'Ndebele Dheasach',
        'nso' => 'Sesotho sa Leboa',
        'nus' => 'Nuer',
        'nv' => 'Navajo',
        'nwc' => 'Newari Chlasaigeach',
        'ny' => 'Nyanja',
        'nym' => 'Nyamwezi',
        'nyn' => 'Nyankole',
        'nyo' => 'Nyoro',
        'nzi' => 'Nzima',
        'oc' => 'Ogsatanais',
        'oj' => 'Ojibwa',
        'ojb' => 'Ojibwa Iar-thuathach',
        'ojc' => 'Ojibwa Mheadhanach',
        'ojs' => 'Oji-Cree',
        'ojw' => 'Ojibwa Shiarach',
        'oka' => 'Okanagan',
        'om' => 'Oromo',
        'or' => 'Odia',
        'os' => 'Ossetic',
        'osa' => 'Osage',
        'ota' => 'Turcais Otomanach',
        'pa' => 'Panjabi',
        'pag' => 'Pangasinan',
        'pal' => 'Pahlavi',
        'pam' => 'Pampanga',
        'pap' => 'Papiamentu',
        'pau' => 'Palabhais',
        'pcd' => 'Picard',
        'pcm' => 'Beurla Nigèiriach',
        'pdc' => 'Gearmailtis Phennsylvania',
        'pdt' => 'Plautdietsch',
        'peo' => 'Seann-Pheirsis',
        'phn' => 'Phenicis',
        'pi' => 'Pali',
        'pis' => 'Pijin',
        'pl' => 'Pòlainnis',
        'pms' => 'Piedmontese',
        'pon' => 'Cànan Pohnpei',
        'pqm' => 'Maliseet-Passamaquoddy',
        'prg' => 'Pruisis',
        'pro' => 'Seann-Phrovençal',
        'ps' => 'Pashto',
        'pt' => 'Portagailis',
        'qu' => 'Quechua',
        'quc' => 'K’iche’',
        'qug' => 'Quichua Àrd-tìr Chimborazo',
        'raj' => 'Rajasthani',
        'rap' => 'Rapa Nui',
        'rar' => 'Cànan Rarotonga',
        'rgn' => 'Romagnol',
        'rhg' => 'Rohingya',
        'rif' => 'Tamaisich an Rif',
        'rm' => 'Rumains',
        'rn' => 'Kirundi',
        'ro' => 'Romàinis',
        'rof' => 'Rombo',
        'rom' => 'Romanais',
        'ru' => 'Ruisis',
        'rue' => 'Rusyn',
        'rug' => 'Roviana',
        'rup' => 'Aromanais',
        'rw' => 'Kinyarwanda',
        'rwk' => 'Rwa',
        'sa' => 'Sanskrit',
        'sad' => 'Sandawe',
        'sah' => 'Sakha',
        'sam' => 'Aramais Shamaritanach',
        'saq' => 'Samburu',
        'sas' => 'Sasak',
        'sat' => 'Santali',
        'saz' => 'Saurashtra',
        'sba' => 'Ngambay',
        'sbp' => 'Sangu',
        'sc' => 'Sàrdais',
        'scn' => 'Sisilis',
        'sco' => 'Albais',
        'sd' => 'Sindhi',
        'sdc' => 'Sassarese',
        'sdh' => 'Cùrdais Dheasach',
        'se' => 'Sàmais Thuathach',
        'see' => 'Seneca',
        'seh' => 'Sena',
        'sei' => 'Seri',
        'sel' => 'Selkup',
        'ses' => 'Koyraboro Senni',
        'sg' => 'Sango',
        'sga' => 'Seann-Ghaeilge',
        'sgs' => 'Samogitis',
        'sh' => 'Sèirb-Chròthaisis',
        'shi' => 'Tachelhit',
        'shn' => 'Shan',
        'shu' => 'Arabais Seàdach',
        'si' => 'Sinhala',
        'sid' => 'Sidamo',
        'sk' => 'Slòbhacais',
        'skr' => 'Saraiki',
        'sl' => 'Slòbhainis',
        'slh' => 'Lushootseed Dheasach',
        'sly' => 'Selayar',
        'sm' => 'Samothais',
        'sma' => 'Sàmais Dheasach',
        'smj' => 'Sàmais Lule',
        'smn' => 'Sàmais Inari',
        'sms' => 'Sàmais Skolt',
        'sn' => 'Shona',
        'snk' => 'Soninke',
        'so' => 'Somàilis',
        'sq' => 'Albàinis',
        'sr' => 'Sèirbis',
        'srn' => 'Sranan Tongo',
        'srr' => 'Serer',
        'ss' => 'Swati',
        'ssy' => 'Saho',
        'st' => 'Sesotho',
        'str' => 'Salish a’ Chaolais',
        'su' => 'Cànan Sunda',
        'suk' => 'Sukuma',
        'sus' => 'Susu',
        'sux' => 'Cànan Sumer',
        'suz' => 'Sunwar',
        'sv' => 'Suainis',
        'sw' => 'Kiswahili',
        'swb' => 'Comorais',
        'syc' => 'Suraidheac Chlasaigeach',
        'syr' => 'Suraidheac',
        'szl' => 'Sileisis',
        'ta' => 'Taimilis',
        'tce' => 'Tutchone Dheasach',
        'tcy' => 'Tulu',
        'te' => 'Telugu',
        'tem' => 'Timne',
        'teo' => 'Teso',
        'ter' => 'Terêna',
        'tet' => 'Tetum',
        'tg' => 'Taidigis',
        'tgx' => 'Tagish',
        'th' => 'Cànan nan Tàidh',
        'tht' => 'Tahltan',
        'ti' => 'Tigrinya',
        'tig' => 'Tigre',
        'tiv' => 'Tiv',
        'tk' => 'Turcmanais',
        'tkl' => 'Tokelau',
        'tkr' => 'Tsakhur',
        'tl' => 'Tagalog',
        'tlh' => 'Klingon',
        'tli' => 'Tlingit',
        'tly' => 'Talysh',
        'tmh' => 'Tamashek',
        'tn' => 'Tswana',
        'to' => 'Tonga',
        'tog' => 'Nyasa Tonga',
        'tok' => 'Toki Pona',
        'tpi' => 'Tok Pisin',
        'tr' => 'Turcais',
        'tru' => 'Turoyo',
        'trv' => 'Taroko',
        'trw' => 'Torwali',
        'ts' => 'Tsonga',
        'tsi' => 'Tsimshian',
        'tt' => 'Tatarais',
        'ttm' => 'Tutchone Thuathach',
        'ttt' => 'Tati',
        'tum' => 'Tumbuka',
        'tvl' => 'Tubhalu',
        'tw' => 'Twi',
        'twq' => 'Tasawaq',
        'ty' => 'Cànan Tahiti',
        'tyv' => 'Cànan Tuva',
        'tzm' => 'Tamaisich an Atlais Mheadhanaich',
        'udm' => 'Udmurt',
        'ug' => 'Ùigiurais',
        'uk' => 'Ucràinis',
        'umb' => 'Umbundu',
        'ur' => 'Ùrdu',
        'uz' => 'Usbagais',
        'vai' => 'Vai',
        've' => 'Venda',
        'vec' => 'Bheinisis',
        'vep' => 'Veps',
        'vi' => 'Bhiet-Namais',
        'vls' => 'Flànrais Shiarach',
        'vmw' => 'Makhuwa',
        'vo' => 'Volapük',
        'vro' => 'Võro',
        'vun' => 'Vunjo',
        'wa' => 'Walloon',
        'wae' => 'Gearmailtis Wallis',
        'wal' => 'Wolaytta',
        'war' => 'Waray',
        'was' => 'Washo',
        'wbp' => 'Warlpiri',
        'wo' => 'Wolof',
        'wuu' => 'Wu',
        'xal' => 'Kalmyk',
        'xh' => 'Xhosa',
        'xnr' => 'Kangri',
        'xog' => 'Soga',
        'yao' => 'Yao',
        'yap' => 'Cànan Yap',
        'yav' => 'Yangben',
        'ybb' => 'Yemba',
        'yi' => 'Iùdhais',
        'yo' => 'Yoruba',
        'yrl' => 'Nheengatu',
        'yue' => 'Cantonais',
        'za' => 'Zhuang',
        'zap' => 'Zapotec',
        'zbl' => 'Comharran Bliss',
        'zea' => 'Cànan Zeeland',
        'zen' => 'Zenaga',
        'zgh' => 'Tamaisich Stannardach Moroco',
        'zh' => 'Sìnis',
        'zu' => 'Zulu',
        'zun' => 'Zuñi',
        'zza' => 'Zazaki',
    ],
    'LocalizedNames' => [
        'ar_001' => 'Nuadh-Arabais Stannardach',
        'de_AT' => 'Gearmailtis na h-Ostaire',
        'de_CH' => 'Àrd-Ghearmailtis na h-Eilbheise',
        'en_AU' => 'Beurla Astràilia',
        'en_CA' => 'Beurla Chanada',
        'en_GB' => 'Beurla Bhreatainn',
        'en_US' => 'Beurla na h-Aimeireaga',
        'es_419' => 'Spàinntis na h-Aimeireaga Laidinneach',
        'es_ES' => 'Spàinntis Eòrpach',
        'es_MX' => 'Spàinntis Mheagsagach',
        'fa_AF' => 'Dari',
        'fr_CA' => 'Fraingis Chanada',
        'fr_CH' => 'Fraingis Eilbheiseach',
        'nds_NL' => 'Sagsannais Ìochdarach',
        'nl_BE' => 'Flànrais',
        'pt_BR' => 'Portagailis Bhraisileach',
        'pt_PT' => 'Portagailis Eòrpach',
        'ro_MD' => 'Moldobhais',
        'sw_CD' => 'Kiswahili na Congo',
        'zh_Hans' => 'Sìnis Shimplichte',
        'zh_Hant' => 'Sìnis Thradaiseanta',
    ],
];
