<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\PropertyInfo\Tests\Fixtures;

use Symfony\Component\PropertyInfo\Tests\Fixtures\Extractor\DummyInDifferentNs;

/**
 * @template T of DummyInDifferentNs
 *
 * @extends ParentDummyWithTemplate<T>
 */
class DummyWithTemplateAndParent extends ParentDummyWithTemplate
{
}
