<?php

return [
    'Names' => [
        'ff' => 'Fulah',
        'ff_Adlm' => 'Fulah (Adlam)',
        'ff_Adlm_BF' => 'Fulah (Adlam, Burkina Faso)',
        'ff_Adlm_CM' => 'Fulah (Adlam, Cameroon)',
        'ff_Adlm_GH' => 'Fulah (Adlam, Ghana)',
        'ff_Adlm_GM' => 'Fulah (Adlam, Gambia)',
        'ff_Adlm_GN' => 'Fulah (Adlam, Guinea)',
        'ff_Adlm_GW' => 'Fulah (Adlam, Guinea-Bissau)',
        'ff_Adlm_LR' => 'Fulah (Adlam, Liberia)',
        'ff_Adlm_MR' => 'Fulah (Adlam, Mauritania)',
        'ff_Adlm_NE' => 'Fulah (Adlam, Niger)',
        'ff_Adlm_NG' => 'Fulah (Adlam, Nigeria)',
        'ff_Adlm_SL' => 'Fulah (Adlam, Sierra Leone)',
        'ff_Adlm_SN' => 'Fulah (Adlam, Senegal)',
        'ff_CM' => 'Fulah (Cameroon)',
        'ff_GN' => 'Fulah (Guinea)',
        'ff_Latn' => 'Fulah (Latin)',
        'ff_Latn_BF' => 'Fulah (Latin, Burkina Faso)',
        'ff_Latn_CM' => 'Fulah (Latin, Cameroon)',
        'ff_Latn_GH' => 'Fulah (Latin, Ghana)',
        'ff_Latn_GM' => 'Fulah (Latin, Gambia)',
        'ff_Latn_GN' => 'Fulah (Latin, Guinea)',
        'ff_Latn_GW' => 'Fulah (Latin, Guinea-Bissau)',
        'ff_Latn_LR' => 'Fulah (Latin, Liberia)',
        'ff_Latn_MR' => 'Fulah (Latin, Mauritania)',
        'ff_Latn_NE' => 'Fulah (Latin, Niger)',
        'ff_Latn_NG' => 'Fulah (Latin, Nigeria)',
        'ff_Latn_SL' => 'Fulah (Latin, Sierra Leone)',
        'ff_Latn_SN' => 'Fulah (Latin, Senegal)',
        'ff_MR' => 'Fulah (Mauritania)',
        'ff_SN' => 'Fulah (Senegal)',
    ],
];
