<?php

return [
    'Names' => [
        'aa' => 'afar',
        'ab' => 'abkhaze',
        'ace' => 'aceh',
        'ach' => 'acoli',
        'ada' => 'adangme',
        'ady' => 'adyguéen',
        'ae' => 'avestique',
        'aeb' => 'arabe tunisien',
        'af' => 'afrikaans',
        'afh' => 'afrihili',
        'agq' => 'aghem',
        'ain' => 'aïnou',
        'ak' => 'akan',
        'akk' => 'akkadien',
        'akz' => 'alabama',
        'ale' => 'aléoute',
        'aln' => 'guègue',
        'alt' => 'altaï du Sud',
        'am' => 'amharique',
        'an' => 'aragonais',
        'ang' => 'ancien anglais',
        'ann' => 'obolo',
        'anp' => 'angika',
        'ar' => 'arabe',
        'arc' => 'araméen',
        'arn' => 'mapuche',
        'aro' => 'araona',
        'arp' => 'arapaho',
        'arq' => 'arabe algérien',
        'ars' => 'arabe najdi',
        'arw' => 'arawak',
        'ary' => 'arabe marocain',
        'arz' => 'arabe égyptien',
        'as' => 'assamais',
        'asa' => 'asu',
        'ase' => 'langue des signes américaine',
        'ast' => 'asturien',
        'atj' => 'atikamekw',
        'av' => 'avar',
        'avk' => 'kotava',
        'awa' => 'awadhi',
        'ay' => 'aymara',
        'az' => 'azerbaïdjanais',
        'ba' => 'bachkir',
        'bal' => 'baloutchi',
        'ban' => 'balinais',
        'bar' => 'bavarois',
        'bas' => 'bassa',
        'bax' => 'bamoun',
        'bbc' => 'batak toba',
        'bbj' => 'ghomalaʼ',
        'be' => 'biélorusse',
        'bej' => 'bedja',
        'bem' => 'bemba',
        'bew' => 'betawi',
        'bez' => 'bena',
        'bfd' => 'bafut',
        'bfq' => 'badaga',
        'bg' => 'bulgare',
        'bgc' => 'haryanvi',
        'bgn' => 'baloutchi occidental',
        'bho' => 'bhodjpouri',
        'bi' => 'bichelamar',
        'bik' => 'bikol',
        'bin' => 'bini',
        'bjn' => 'banjar',
        'bkm' => 'kom',
        'bla' => 'siksika',
        'blo' => 'anii',
        'bm' => 'bambara',
        'bn' => 'bengali',
        'bo' => 'tibétain',
        'bpy' => 'bishnupriya',
        'bqi' => 'bakhtiari',
        'br' => 'breton',
        'bra' => 'braj',
        'brh' => 'brahoui',
        'brx' => 'bodo',
        'bs' => 'bosniaque',
        'bss' => 'akoose',
        'bua' => 'bouriate',
        'bug' => 'bugi',
        'bum' => 'boulou',
        'byn' => 'blin',
        'byv' => 'médumba',
        'ca' => 'catalan',
        'cad' => 'caddo',
        'car' => 'caribe',
        'cay' => 'cayuga',
        'cch' => 'atsam',
        'ccp' => 'changma kodha',
        'ce' => 'tchétchène',
        'ceb' => 'cebuano',
        'cgg' => 'kiga',
        'ch' => 'chamorro',
        'chb' => 'chibcha',
        'chg' => 'tchaghataï',
        'chk' => 'chuuk',
        'chm' => 'mari',
        'chn' => 'jargon chinook',
        'cho' => 'choctaw',
        'chp' => 'chipewyan',
        'chr' => 'cherokee',
        'chy' => 'cheyenne',
        'ckb' => 'sorani',
        'clc' => 'chilcotin',
        'co' => 'corse',
        'cop' => 'copte',
        'cps' => 'capiznon',
        'cr' => 'cree',
        'crg' => 'mitchif',
        'crh' => 'tatar de Crimée',
        'crj' => 'cri de l’Est (dialecte du Sud)',
        'crk' => 'cri des plaines',
        'crl' => 'cri de l’Est (dialecte du Nord)',
        'crm' => 'cri de Moose',
        'crr' => 'algonquin de Caroline',
        'crs' => 'créole seychellois',
        'cs' => 'tchèque',
        'csb' => 'kachoube',
        'csw' => 'cri des marais',
        'cu' => 'slavon d’église',
        'cv' => 'tchouvache',
        'cy' => 'gallois',
        'da' => 'danois',
        'dak' => 'dakota',
        'dar' => 'dargwa',
        'dav' => 'taita',
        'de' => 'allemand',
        'del' => 'delaware',
        'den' => 'esclave',
        'dgr' => 'dogrib',
        'din' => 'dinka',
        'dje' => 'zarma',
        'doi' => 'dogri',
        'dsb' => 'bas-sorabe',
        'dtp' => 'dusun central',
        'dua' => 'douala',
        'dum' => 'moyen néerlandais',
        'dv' => 'maldivien',
        'dyo' => 'diola-fogny',
        'dyu' => 'dioula',
        'dz' => 'dzongkha',
        'dzg' => 'dazaga',
        'ebu' => 'embu',
        'ee' => 'éwé',
        'efi' => 'éfik',
        'egl' => 'émilien',
        'egy' => 'égyptien ancien',
        'eka' => 'ékadjouk',
        'el' => 'grec',
        'elx' => 'élamite',
        'en' => 'anglais',
        'enm' => 'moyen anglais',
        'eo' => 'espéranto',
        'es' => 'espagnol',
        'esu' => 'youpik central',
        'et' => 'estonien',
        'eu' => 'basque',
        'ewo' => 'éwondo',
        'ext' => 'estrémègne',
        'fa' => 'persan',
        'fan' => 'fang',
        'fat' => 'fanti',
        'ff' => 'peul',
        'fi' => 'finnois',
        'fil' => 'filipino',
        'fit' => 'finnois tornédalien',
        'fj' => 'fidjien',
        'fo' => 'féroïen',
        'fon' => 'fon',
        'fr' => 'français',
        'frc' => 'français cadien',
        'frm' => 'moyen français',
        'fro' => 'ancien français',
        'frp' => 'francoprovençal',
        'frr' => 'frison septentrional',
        'frs' => 'frison oriental',
        'fur' => 'frioulan',
        'fy' => 'frison occidental',
        'ga' => 'irlandais',
        'gaa' => 'ga',
        'gag' => 'gagaouze',
        'gan' => 'gan',
        'gay' => 'gayo',
        'gba' => 'gbaya',
        'gbz' => 'dari zoroastrien',
        'gd' => 'gaélique écossais',
        'gez' => 'guèze',
        'gil' => 'gilbertin',
        'gl' => 'galicien',
        'glk' => 'gilaki',
        'gmh' => 'moyen haut-allemand',
        'gn' => 'guarani',
        'goh' => 'ancien haut allemand',
        'gon' => 'gondi',
        'gor' => 'gorontalo',
        'got' => 'gotique',
        'grb' => 'grebo',
        'grc' => 'grec ancien',
        'gsw' => 'suisse allemand',
        'gu' => 'goudjarati',
        'guc' => 'wayuu',
        'gur' => 'gurenne',
        'guz' => 'gusii',
        'gv' => 'mannois',
        'gwi' => 'gwichʼin',
        'ha' => 'haoussa',
        'hai' => 'haïda',
        'hak' => 'hakka',
        'haw' => 'hawaïen',
        'hax' => 'haïda du Sud',
        'he' => 'hébreu',
        'hi' => 'hindi',
        'hif' => 'hindi fidjien',
        'hil' => 'hiligaynon',
        'hit' => 'hittite',
        'hmn' => 'hmong',
        'ho' => 'hiri motu',
        'hr' => 'croate',
        'hsb' => 'haut-sorabe',
        'hsn' => 'xiang',
        'ht' => 'créole haïtien',
        'hu' => 'hongrois',
        'hup' => 'hupa',
        'hur' => 'halkomelem',
        'hy' => 'arménien',
        'hz' => 'héréro',
        'ia' => 'interlingua',
        'iba' => 'iban',
        'ibb' => 'ibibio',
        'id' => 'indonésien',
        'ie' => 'interlingue',
        'ig' => 'igbo',
        'ii' => 'yi du Sichuan',
        'ik' => 'inupiaq',
        'ikt' => 'inuktitut de l’Ouest canadien',
        'ilo' => 'ilocano',
        'inh' => 'ingouche',
        'io' => 'ido',
        'is' => 'islandais',
        'it' => 'italien',
        'iu' => 'inuktitut',
        'izh' => 'ingrien',
        'ja' => 'japonais',
        'jam' => 'créole jamaïcain',
        'jbo' => 'lojban',
        'jgo' => 'ngomba',
        'jmc' => 'matchamé',
        'jpr' => 'judéo-persan',
        'jrb' => 'judéo-arabe',
        'jut' => 'jute',
        'jv' => 'javanais',
        'ka' => 'géorgien',
        'kaa' => 'karakalpak',
        'kab' => 'kabyle',
        'kac' => 'kachin',
        'kaj' => 'jju',
        'kam' => 'kamba',
        'kaw' => 'kawi',
        'kbd' => 'kabarde',
        'kbl' => 'kanembou',
        'kcg' => 'tyap',
        'kde' => 'makondé',
        'kea' => 'capverdien',
        'ken' => 'kényang',
        'kfo' => 'koro',
        'kg' => 'kikongo',
        'kgp' => 'caingangue',
        'kha' => 'khasi',
        'kho' => 'khotanais',
        'khq' => 'koyra chiini',
        'khw' => 'khowar',
        'ki' => 'kikuyu',
        'kiu' => 'kirmanjki',
        'kj' => 'kuanyama',
        'kk' => 'kazakh',
        'kkj' => 'kako',
        'kl' => 'groenlandais',
        'kln' => 'kalendjin',
        'km' => 'khmer',
        'kmb' => 'kimboundou',
        'kn' => 'kannada',
        'ko' => 'coréen',
        'koi' => 'komi-permiak',
        'kok' => 'konkani',
        'kos' => 'kosraéen',
        'kpe' => 'kpellé',
        'kr' => 'kanouri',
        'krc' => 'karatchaï balkar',
        'kri' => 'krio',
        'krj' => 'kinaray-a',
        'krl' => 'carélien',
        'kru' => 'kouroukh',
        'ks' => 'cachemiri',
        'ksb' => 'shambala',
        'ksf' => 'bafia',
        'ksh' => 'kölsch',
        'ku' => 'kurde',
        'kum' => 'koumyk',
        'kut' => 'kutenai',
        'kv' => 'komi',
        'kw' => 'cornique',
        'kwk' => 'kwak’wala',
        'kxv' => 'kuvi',
        'ky' => 'kirghize',
        'la' => 'latin',
        'lad' => 'ladino',
        'lag' => 'langi',
        'lah' => 'lahnda',
        'lam' => 'lamba',
        'lb' => 'luxembourgeois',
        'lez' => 'lezghien',
        'lfn' => 'lingua franca nova',
        'lg' => 'ganda',
        'li' => 'limbourgeois',
        'lij' => 'ligure',
        'lil' => 'lillooet',
        'liv' => 'livonien',
        'lkt' => 'lakota',
        'lmo' => 'lombard',
        'ln' => 'lingala',
        'lo' => 'lao',
        'lol' => 'mongo',
        'lou' => 'créole louisianais',
        'loz' => 'lozi',
        'lrc' => 'lori du Nord',
        'lsm' => 'samia',
        'lt' => 'lituanien',
        'ltg' => 'latgalien',
        'lu' => 'luba-katanga (kiluba)',
        'lua' => 'luba-kasaï (ciluba)',
        'lui' => 'luiseño',
        'lun' => 'lunda',
        'lus' => 'lushaï',
        'luy' => 'luyia',
        'lv' => 'letton',
        'lzh' => 'chinois littéraire',
        'lzz' => 'laze',
        'mad' => 'madurais',
        'maf' => 'mafa',
        'mag' => 'magahi',
        'mai' => 'maïthili',
        'mak' => 'makassar',
        'man' => 'mandingue',
        'mas' => 'maasaï',
        'mde' => 'maba',
        'mdf' => 'mokcha',
        'mdr' => 'mandar',
        'men' => 'mendé',
        'mer' => 'meru',
        'mfe' => 'créole mauricien',
        'mg' => 'malgache',
        'mga' => 'moyen irlandais',
        'mgh' => 'makua',
        'mgo' => 'metaʼ',
        'mh' => 'marshallais',
        'mi' => 'maori',
        'mic' => 'micmac',
        'min' => 'minangkabau',
        'mk' => 'macédonien',
        'ml' => 'malayalam',
        'mn' => 'mongol',
        'mnc' => 'mandchou',
        'mni' => 'manipuri',
        'moe' => 'innu-aimun',
        'moh' => 'mohawk',
        'mos' => 'moré',
        'mr' => 'marathi',
        'mrj' => 'mari occidental',
        'ms' => 'malais',
        'mt' => 'maltais',
        'mua' => 'moundang',
        'mus' => 'creek',
        'mwl' => 'mirandais',
        'mwr' => 'marwarî',
        'mwv' => 'mentawaï',
        'my' => 'birman',
        'mye' => 'myènè',
        'myv' => 'erzya',
        'mzn' => 'mazandérani',
        'na' => 'nauruan',
        'nan' => 'minnan',
        'nap' => 'napolitain',
        'naq' => 'nama',
        'nb' => 'norvégien bokmål',
        'nd' => 'ndébélé du Nord',
        'nds' => 'bas-allemand',
        'ne' => 'népalais',
        'new' => 'newari',
        'ng' => 'ndonga',
        'nia' => 'niha',
        'niu' => 'niuéen',
        'njo' => 'ao',
        'nl' => 'néerlandais',
        'nmg' => 'ngoumba',
        'nn' => 'norvégien nynorsk',
        'nnh' => 'ngiemboon',
        'no' => 'norvégien',
        'nog' => 'nogaï',
        'non' => 'vieux norrois',
        'nov' => 'novial',
        'nqo' => 'n’ko',
        'nr' => 'ndébélé du Sud',
        'nso' => 'sotho du Nord',
        'nus' => 'nuer',
        'nv' => 'navajo',
        'nwc' => 'newarî classique',
        'ny' => 'chewa',
        'nym' => 'nyamwezi',
        'nyn' => 'nyankolé',
        'nyo' => 'nyoro',
        'nzi' => 'nzema',
        'oc' => 'occitan',
        'oj' => 'ojibwa',
        'ojb' => 'ojibwé du Nord-Ouest',
        'ojc' => 'ojibwé central',
        'ojs' => 'oji-cri',
        'ojw' => 'ojibwé occidental',
        'oka' => 'colville-okanagan',
        'om' => 'oromo',
        'or' => 'odia',
        'os' => 'ossète',
        'osa' => 'osage',
        'ota' => 'turc ottoman',
        'pa' => 'pendjabi',
        'pag' => 'pangasinan',
        'pal' => 'pahlavi',
        'pam' => 'pampangan',
        'pap' => 'papiamento',
        'pau' => 'palau',
        'pcd' => 'picard',
        'pcm' => 'pidgin nigérian',
        'pdc' => 'pennsilfaanisch',
        'pdt' => 'bas-prussien',
        'peo' => 'persan ancien',
        'pfl' => 'allemand palatin',
        'phn' => 'phénicien',
        'pi' => 'pali',
        'pis' => 'pijin',
        'pl' => 'polonais',
        'pms' => 'piémontais',
        'pnt' => 'pontique',
        'pon' => 'pohnpei',
        'pqm' => 'malécite-passamaquoddy',
        'prg' => 'prussien',
        'pro' => 'provençal ancien',
        'ps' => 'pachto',
        'pt' => 'portugais',
        'qu' => 'quechua',
        'quc' => 'quiché',
        'qug' => 'quichua du Haut-Chimborazo',
        'raj' => 'rajasthani',
        'rap' => 'rapanui',
        'rar' => 'rarotongien',
        'rgn' => 'romagnol',
        'rhg' => 'rohingya',
        'rif' => 'rifain',
        'rm' => 'romanche',
        'rn' => 'roundi',
        'ro' => 'roumain',
        'rof' => 'rombo',
        'rom' => 'romani',
        'rtm' => 'rotuman',
        'ru' => 'russe',
        'rue' => 'ruthène',
        'rug' => 'roviana',
        'rup' => 'aroumain',
        'rw' => 'kinyarwanda',
        'rwk' => 'rwa',
        'sa' => 'sanskrit',
        'sad' => 'sandawe',
        'sah' => 'iakoute',
        'sam' => 'araméen samaritain',
        'saq' => 'samburu',
        'sas' => 'sasak',
        'sat' => 'santali',
        'saz' => 'saurashtra',
        'sba' => 'ngambay',
        'sbp' => 'isangu',
        'sc' => 'sarde',
        'scn' => 'sicilien',
        'sco' => 'écossais',
        'sd' => 'sindhi',
        'sdc' => 'sarde sassarais',
        'sdh' => 'kurde du Sud',
        'se' => 'same du Nord',
        'see' => 'seneca',
        'seh' => 'cisena',
        'sei' => 'séri',
        'sel' => 'selkoupe',
        'ses' => 'koyraboro senni',
        'sg' => 'sango',
        'sga' => 'ancien irlandais',
        'sgs' => 'samogitien',
        'sh' => 'serbo-croate',
        'shi' => 'chleuh',
        'shn' => 'shan',
        'shu' => 'arabe tchadien',
        'si' => 'cingalais',
        'sid' => 'sidamo',
        'sk' => 'slovaque',
        'sl' => 'slovène',
        'slh' => 'lushootseed du Sud',
        'sli' => 'bas-silésien',
        'sly' => 'sélayar',
        'sm' => 'samoan',
        'sma' => 'same du Sud',
        'smj' => 'same de Lule',
        'smn' => 'same d’Inari',
        'sms' => 'same skolt',
        'sn' => 'shona',
        'snk' => 'soninké',
        'so' => 'somali',
        'sog' => 'sogdien',
        'sq' => 'albanais',
        'sr' => 'serbe',
        'srn' => 'sranan tongo',
        'srr' => 'sérère',
        'ss' => 'swati',
        'ssy' => 'saho',
        'st' => 'sotho du Sud',
        'stq' => 'saterlandais',
        'str' => 'salish des détroits',
        'su' => 'soundanais',
        'suk' => 'soukouma',
        'sus' => 'soussou',
        'sux' => 'sumérien',
        'sv' => 'suédois',
        'sw' => 'swahili',
        'swb' => 'comorien',
        'syc' => 'syriaque classique',
        'syr' => 'syriaque',
        'szl' => 'silésien',
        'ta' => 'tamoul',
        'tce' => 'tutchone du Sud',
        'tcy' => 'toulou',
        'te' => 'télougou',
        'tem' => 'timné',
        'teo' => 'teso',
        'ter' => 'tereno',
        'tet' => 'tétoum',
        'tg' => 'tadjik',
        'tgx' => 'tagish',
        'th' => 'thaï',
        'tht' => 'tahltan',
        'ti' => 'tigrigna',
        'tig' => 'tigré',
        'tiv' => 'tiv',
        'tk' => 'turkmène',
        'tkl' => 'tokelau',
        'tkr' => 'tsakhour',
        'tl' => 'tagalog',
        'tlh' => 'klingon',
        'tli' => 'tlingit',
        'tly' => 'talysh',
        'tmh' => 'tamacheq',
        'tn' => 'tswana',
        'to' => 'tongien',
        'tog' => 'tonga nyasa',
        'tok' => 'toki pona',
        'tpi' => 'tok pisin',
        'tr' => 'turc',
        'tru' => 'touroyo',
        'trv' => 'taroko',
        'ts' => 'tsonga',
        'tsd' => 'tsakonien',
        'tsi' => 'tsimshian',
        'tt' => 'tatar',
        'ttm' => 'tutchone du Nord',
        'ttt' => 'tati caucasien',
        'tum' => 'tumbuka',
        'tvl' => 'tuvalu',
        'tw' => 'twi',
        'twq' => 'tasawaq',
        'ty' => 'tahitien',
        'tyv' => 'touvain',
        'tzm' => 'amazighe de l’Atlas central',
        'udm' => 'oudmourte',
        'ug' => 'ouïghour',
        'uga' => 'ougaritique',
        'uk' => 'ukrainien',
        'umb' => 'umbundu',
        'ur' => 'ourdou',
        'uz' => 'ouzbek',
        'vai' => 'vaï',
        've' => 'venda',
        'vec' => 'vénitien',
        'vep' => 'vepse',
        'vi' => 'vietnamien',
        'vls' => 'flamand occidental',
        'vmf' => 'franconien du Main',
        'vmw' => 'macua',
        'vo' => 'volapük',
        'vot' => 'vote',
        'vro' => 'võro',
        'vun' => 'vunjo',
        'wa' => 'wallon',
        'wae' => 'walser',
        'wal' => 'walamo',
        'war' => 'waray',
        'was' => 'washo',
        'wbp' => 'warlpiri',
        'wo' => 'wolof',
        'wuu' => 'wu',
        'xal' => 'kalmouk',
        'xh' => 'xhosa',
        'xmf' => 'mingrélien',
        'xnr' => 'kangri',
        'xog' => 'soga',
        'yao' => 'yao',
        'yap' => 'yapois',
        'yav' => 'yangben',
        'ybb' => 'yemba',
        'yi' => 'yiddish',
        'yo' => 'yoruba',
        'yrl' => 'nheengatou',
        'yue' => 'cantonais',
        'za' => 'zhuang',
        'zap' => 'zapotèque',
        'zbl' => 'symboles Bliss',
        'zea' => 'zélandais',
        'zen' => 'zenaga',
        'zgh' => 'amazighe standard marocain',
        'zh' => 'chinois',
        'zu' => 'zoulou',
        'zun' => 'zuñi',
        'zza' => 'zazaki',
    ],
    'LocalizedNames' => [
        'ar_001' => 'arabe standard moderne',
        'de_AT' => 'allemand autrichien',
        'de_CH' => 'allemand suisse',
        'en_AU' => 'anglais australien',
        'en_CA' => 'anglais canadien',
        'en_GB' => 'anglais britannique',
        'en_US' => 'anglais américain',
        'es_419' => 'espagnol d’Amérique latine',
        'es_ES' => 'espagnol d’Espagne',
        'es_MX' => 'espagnol du Mexique',
        'fa_AF' => 'dari',
        'fr_CA' => 'français canadien',
        'fr_CH' => 'français suisse',
        'nds_NL' => 'bas-saxon néerlandais',
        'nl_BE' => 'flamand',
        'pt_BR' => 'portugais brésilien',
        'pt_PT' => 'portugais européen',
        'ro_MD' => 'moldave',
        'sw_CD' => 'swahili du Congo',
        'zh_Hans' => 'chinois simplifié',
        'zh_Hant' => 'chinois traditionnel',
    ],
];
