<?php

return [
    'Names' => [
        'ADP' => [
            'ADP',
            'peseta andorrana',
        ],
        'AED' => [
            'AED',
            'dírham de los Emiratos Árabes Unidos',
        ],
        'AFA' => [
            'AFA',
            'afgani (1927–2002)',
        ],
        'AFN' => [
            'AFN',
            'afgani afgano',
        ],
        'ALL' => [
            'ALL',
            'lek albanés',
        ],
        'AMD' => [
            'AMD',
            'dram armenio',
        ],
        'ANG' => [
            'ANG',
            'florín antillano',
        ],
        'AOA' => [
            'AOA',
            'kuanza angoleño',
        ],
        'AOK' => [
            'AOK',
            'kwanza angoleño (1977–1990)',
        ],
        'AON' => [
            'AON',
            'nuevo kwanza angoleño (1990–2000)',
        ],
        'AOR' => [
            'AOR',
            'kwanza reajustado angoleño (1995–1999)',
        ],
        'ARA' => [
            'ARA',
            'austral argentino',
        ],
        'ARP' => [
            'ARP',
            'peso argentino (1983–1985)',
        ],
        'ARS' => [
            'ARS',
            'peso argentino',
        ],
        'ATS' => [
            'ATS',
            'chelín austriaco',
        ],
        'AUD' => [
            'AUD',
            'dólar australiano',
        ],
        'AWG' => [
            'AWG',
            'florín arubeño',
        ],
        'AZM' => [
            'AZM',
            'manat azerí (1993–2006)',
        ],
        'AZN' => [
            'AZN',
            'manat azerbaiyano',
        ],
        'BAD' => [
            'BAD',
            'dinar bosnio',
        ],
        'BAM' => [
            'BAM',
            'marco convertible de Bosnia y Herzegovina',
        ],
        'BBD' => [
            'BBD',
            'dólar barbadense',
        ],
        'BDT' => [
            'BDT',
            'taka bangladesí',
        ],
        'BEC' => [
            'BEC',
            'franco belga (convertible)',
        ],
        'BEF' => [
            'BEF',
            'franco belga',
        ],
        'BEL' => [
            'BEL',
            'franco belga (financiero)',
        ],
        'BGL' => [
            'BGL',
            'lev fuerte búlgaro',
        ],
        'BGN' => [
            'BGN',
            'leva búlgara',
        ],
        'BHD' => [
            'BHD',
            'dinar bareiní',
        ],
        'BIF' => [
            'BIF',
            'franco burundés',
        ],
        'BMD' => [
            'BMD',
            'dólar bermudeño',
        ],
        'BND' => [
            'BND',
            'dólar bruneano',
        ],
        'BOB' => [
            'BOB',
            'boliviano',
        ],
        'BOP' => [
            'BOP',
            'peso boliviano',
        ],
        'BOV' => [
            'BOV',
            'MVDOL boliviano',
        ],
        'BRB' => [
            'BRB',
            'nuevo cruceiro brasileño (1967–1986)',
        ],
        'BRC' => [
            'BRC',
            'cruzado brasileño',
        ],
        'BRE' => [
            'BRE',
            'cruceiro brasileño (1990–1993)',
        ],
        'BRL' => [
            'BRL',
            'real brasileño',
        ],
        'BRN' => [
            'BRN',
            'nuevo cruzado brasileño',
        ],
        'BRR' => [
            'BRR',
            'cruceiro brasileño',
        ],
        'BSD' => [
            'BSD',
            'dólar bahameño',
        ],
        'BTN' => [
            'BTN',
            'gultrum butanés',
        ],
        'BUK' => [
            'BUK',
            'kyat birmano',
        ],
        'BWP' => [
            'BWP',
            'pula botsuano',
        ],
        'BYB' => [
            'BYB',
            'nuevo rublo bielorruso (1994–1999)',
        ],
        'BYN' => [
            'BYN',
            'rublo bielorruso',
        ],
        'BYR' => [
            'BYR',
            'rublo bielorruso (2000–2016)',
        ],
        'BZD' => [
            'BZD',
            'dólar beliceño',
        ],
        'CAD' => [
            'CAD',
            'dólar canadiense',
        ],
        'CDF' => [
            'CDF',
            'franco congoleño',
        ],
        'CHE' => [
            'CHE',
            'euro WIR',
        ],
        'CHF' => [
            'CHF',
            'franco suizo',
        ],
        'CHW' => [
            'CHW',
            'franco WIR',
        ],
        'CLF' => [
            'CLF',
            'unidad de fomento chilena',
        ],
        'CLP' => [
            'CLP',
            'peso chileno',
        ],
        'CNH' => [
            'CNH',
            'yuan chino (extracontinental)',
        ],
        'CNY' => [
            'CNY',
            'yuan renminbi',
        ],
        'COP' => [
            'COP',
            'peso colombiano',
        ],
        'COU' => [
            'COU',
            'unidad de valor real colombiana',
        ],
        'CRC' => [
            'CRC',
            'colón costarricense',
        ],
        'CSD' => [
            'CSD',
            'antiguo dinar serbio',
        ],
        'CSK' => [
            'CSK',
            'corona fuerte checoslovaca',
        ],
        'CUC' => [
            'CUC',
            'peso cubano convertible',
        ],
        'CUP' => [
            'CUP',
            'peso cubano',
        ],
        'CVE' => [
            'CVE',
            'escudo de Cabo Verde',
        ],
        'CYP' => [
            'CYP',
            'libra chipriota',
        ],
        'CZK' => [
            'CZK',
            'corona checa',
        ],
        'DDM' => [
            'DDM',
            'ostmark de Alemania del Este',
        ],
        'DEM' => [
            'DEM',
            'marco alemán',
        ],
        'DJF' => [
            'DJF',
            'franco yibutiano',
        ],
        'DKK' => [
            'DKK',
            'corona danesa',
        ],
        'DOP' => [
            'DOP',
            'peso dominicano',
        ],
        'DZD' => [
            'DZD',
            'dinar argelino',
        ],
        'ECS' => [
            'ECS',
            'sucre ecuatoriano',
        ],
        'ECV' => [
            'ECV',
            'unidad de valor constante (UVC) ecuatoriana',
        ],
        'EEK' => [
            'EEK',
            'corona estonia',
        ],
        'EGP' => [
            'EGP',
            'libra egipcia',
        ],
        'ERN' => [
            'ERN',
            'nakfa eritreo',
        ],
        'ESA' => [
            'ESA',
            'peseta española (cuenta A)',
        ],
        'ESB' => [
            'ESB',
            'peseta española (cuenta convertible)',
        ],
        'ESP' => [
            '₧',
            'peseta española',
        ],
        'ETB' => [
            'ETB',
            'bir etíope',
        ],
        'EUR' => [
            '€',
            'euro',
        ],
        'FIM' => [
            'FIM',
            'marco finlandés',
        ],
        'FJD' => [
            'FJD',
            'dólar fiyiano',
        ],
        'FKP' => [
            'FKP',
            'libra malvinense',
        ],
        'FRF' => [
            'FRF',
            'franco francés',
        ],
        'GBP' => [
            'GBP',
            'libra esterlina',
        ],
        'GEK' => [
            'GEK',
            'kupon larit georgiano',
        ],
        'GEL' => [
            'GEL',
            'lari georgiano',
        ],
        'GHC' => [
            'GHC',
            'cedi ghanés (1979–2007)',
        ],
        'GHS' => [
            'GHS',
            'cedi ghanés',
        ],
        'GIP' => [
            'GIP',
            'libra gibraltareña',
        ],
        'GMD' => [
            'GMD',
            'dalasi gambiano',
        ],
        'GNF' => [
            'GNF',
            'franco guineano',
        ],
        'GNS' => [
            'GNS',
            'syli guineano',
        ],
        'GQE' => [
            'GQE',
            'ekuele de Guinea Ecuatorial',
        ],
        'GRD' => [
            'GRD',
            'dracma griego',
        ],
        'GTQ' => [
            'GTQ',
            'quetzal guatemalteco',
        ],
        'GWE' => [
            'GWE',
            'escudo de Guinea Portuguesa',
        ],
        'GWP' => [
            'GWP',
            'peso de Guinea-Bissáu',
        ],
        'GYD' => [
            'GYD',
            'dólar guyanés',
        ],
        'HKD' => [
            'HKD',
            'dólar hongkonés',
        ],
        'HNL' => [
            'HNL',
            'lempira hondureño',
        ],
        'HRD' => [
            'HRD',
            'dinar croata',
        ],
        'HRK' => [
            'HRK',
            'kuna croata',
        ],
        'HTG' => [
            'HTG',
            'gurde haitiano',
        ],
        'HUF' => [
            'HUF',
            'forinto húngaro',
        ],
        'IDR' => [
            'IDR',
            'rupia indonesia',
        ],
        'IEP' => [
            'IEP',
            'libra irlandesa',
        ],
        'ILP' => [
            'ILP',
            'libra israelí',
        ],
        'ILS' => [
            'ILS',
            'nuevo séquel israelí',
        ],
        'INR' => [
            'INR',
            'rupia india',
        ],
        'IQD' => [
            'IQD',
            'dinar iraquí',
        ],
        'IRR' => [
            'IRR',
            'rial iraní',
        ],
        'ISK' => [
            'ISK',
            'corona islandesa',
        ],
        'ITL' => [
            'ITL',
            'lira italiana',
        ],
        'JMD' => [
            'JMD',
            'dólar jamaicano',
        ],
        'JOD' => [
            'JOD',
            'dinar jordano',
        ],
        'JPY' => [
            'JPY',
            'yen japonés',
        ],
        'KES' => [
            'KES',
            'chelín keniano',
        ],
        'KGS' => [
            'KGS',
            'som kirguís',
        ],
        'KHR' => [
            'KHR',
            'riel camboyano',
        ],
        'KMF' => [
            'KMF',
            'franco comorense',
        ],
        'KPW' => [
            'KPW',
            'won norcoreano',
        ],
        'KRW' => [
            'KRW',
            'won surcoreano',
        ],
        'KWD' => [
            'KWD',
            'dinar kuwaití',
        ],
        'KYD' => [
            'KYD',
            'dólar de las Islas Caimán',
        ],
        'KZT' => [
            'KZT',
            'tengue kazajo',
        ],
        'LAK' => [
            'LAK',
            'kip laosiano',
        ],
        'LBP' => [
            'LBP',
            'libra libanesa',
        ],
        'LKR' => [
            'LKR',
            'rupia esrilanquesa',
        ],
        'LRD' => [
            'LRD',
            'dólar liberiano',
        ],
        'LSL' => [
            'LSL',
            'loti lesotense',
        ],
        'LTL' => [
            'LTL',
            'litas lituano',
        ],
        'LTT' => [
            'LTT',
            'talonas lituano',
        ],
        'LUC' => [
            'LUC',
            'franco convertible luxemburgués',
        ],
        'LUF' => [
            'LUF',
            'franco luxemburgués',
        ],
        'LUL' => [
            'LUL',
            'franco financiero luxemburgués',
        ],
        'LVL' => [
            'LVL',
            'lats letón',
        ],
        'LVR' => [
            'LVR',
            'rublo letón',
        ],
        'LYD' => [
            'LYD',
            'dinar libio',
        ],
        'MAD' => [
            'MAD',
            'dírham marroquí',
        ],
        'MAF' => [
            'MAF',
            'franco marroquí',
        ],
        'MDL' => [
            'MDL',
            'leu moldavo',
        ],
        'MGA' => [
            'MGA',
            'ariari malgache',
        ],
        'MGF' => [
            'MGF',
            'franco malgache',
        ],
        'MKD' => [
            'MKD',
            'dinar macedonio',
        ],
        'MLF' => [
            'MLF',
            'franco malí',
        ],
        'MMK' => [
            'MMK',
            'kiat de Myanmar',
        ],
        'MNT' => [
            'MNT',
            'tugrik mongol',
        ],
        'MOP' => [
            'MOP',
            'pataca macaense',
        ],
        'MRO' => [
            'MRO',
            'uguiya (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'uguiya mauritano',
        ],
        'MTL' => [
            'MTL',
            'lira maltesa',
        ],
        'MTP' => [
            'MTP',
            'libra maltesa',
        ],
        'MUR' => [
            'MUR',
            'rupia mauriciana',
        ],
        'MVR' => [
            'MVR',
            'rufiya maldiva',
        ],
        'MWK' => [
            'MWK',
            'kuacha malauí',
        ],
        'MXN' => [
            'MXN',
            'peso mexicano',
        ],
        'MXP' => [
            'MXP',
            'peso de plata mexicano (1861–1992)',
        ],
        'MXV' => [
            'MXV',
            'unidad de inversión (UDI) mexicana',
        ],
        'MYR' => [
            'MYR',
            'ringit malasio',
        ],
        'MZE' => [
            'MZE',
            'escudo mozambiqueño',
        ],
        'MZM' => [
            'MZM',
            'antiguo metical mozambiqueño',
        ],
        'MZN' => [
            'MZN',
            'metical mozambiqueño',
        ],
        'NAD' => [
            'NAD',
            'dólar namibio',
        ],
        'NGN' => [
            'NGN',
            'naira nigeriano',
        ],
        'NIC' => [
            'NIC',
            'córdoba nicaragüense (1988–1991)',
        ],
        'NIO' => [
            'NIO',
            'córdoba oro',
        ],
        'NLG' => [
            'NLG',
            'florín neerlandés',
        ],
        'NOK' => [
            'NOK',
            'corona noruega',
        ],
        'NPR' => [
            'NPR',
            'rupia nepalí',
        ],
        'NZD' => [
            'NZD',
            'dólar neozelandés',
        ],
        'OMR' => [
            'OMR',
            'rial omaní',
        ],
        'PAB' => [
            'PAB',
            'balboa panameño',
        ],
        'PEI' => [
            'PEI',
            'inti peruano',
        ],
        'PEN' => [
            'PEN',
            'sol peruano',
        ],
        'PES' => [
            'PES',
            'sol peruano (1863–1965)',
        ],
        'PGK' => [
            'PGK',
            'kina papú',
        ],
        'PHP' => [
            'PHP',
            'peso filipino',
        ],
        'PKR' => [
            'PKR',
            'rupia pakistaní',
        ],
        'PLN' => [
            'PLN',
            'esloti polaco',
        ],
        'PLZ' => [
            'PLZ',
            'zloty polaco (1950–1995)',
        ],
        'PTE' => [
            'PTE',
            'escudo portugués',
        ],
        'PYG' => [
            'PYG',
            'guaraní paraguayo',
        ],
        'QAR' => [
            'QAR',
            'rial catarí',
        ],
        'RHD' => [
            'RHD',
            'dólar rodesiano',
        ],
        'ROL' => [
            'ROL',
            'antiguo leu rumano',
        ],
        'RON' => [
            'RON',
            'leu rumano',
        ],
        'RSD' => [
            'RSD',
            'dinar serbio',
        ],
        'RUB' => [
            'RUB',
            'rublo ruso',
        ],
        'RUR' => [
            'RUR',
            'rublo ruso (1991–1998)',
        ],
        'RWF' => [
            'RWF',
            'franco ruandés',
        ],
        'SAR' => [
            'SAR',
            'rial saudí',
        ],
        'SBD' => [
            'SBD',
            'dólar salomonense',
        ],
        'SCR' => [
            'SCR',
            'rupia seychellense',
        ],
        'SDD' => [
            'SDD',
            'dinar sudanés',
        ],
        'SDG' => [
            'SDG',
            'libra sudanesa',
        ],
        'SDP' => [
            'SDP',
            'libra sudanesa antigua',
        ],
        'SEK' => [
            'SEK',
            'corona sueca',
        ],
        'SGD' => [
            'SGD',
            'dólar singapurense',
        ],
        'SHP' => [
            'SHP',
            'libra de Santa Elena',
        ],
        'SIT' => [
            'SIT',
            'tólar esloveno',
        ],
        'SKK' => [
            'SKK',
            'corona eslovaca',
        ],
        'SLE' => [
            'SLE',
            'leona sierraleonesa',
        ],
        'SLL' => [
            'SLL',
            'leona sierraleonesa (1964–2022)',
        ],
        'SOS' => [
            'SOS',
            'chelín somalí',
        ],
        'SRD' => [
            'SRD',
            'dólar surinamés',
        ],
        'SRG' => [
            'SRG',
            'florín surinamés',
        ],
        'SSP' => [
            'SSP',
            'libra sursudanesa',
        ],
        'STD' => [
            'STD',
            'dobra (1977–2017)',
        ],
        'STN' => [
            'STN',
            'dobra santotomense',
        ],
        'SUR' => [
            'SUR',
            'rublo soviético',
        ],
        'SVC' => [
            'SVC',
            'colón salvadoreño',
        ],
        'SYP' => [
            'SYP',
            'libra siria',
        ],
        'SZL' => [
            'SZL',
            'lilangeni esuatiní',
        ],
        'THB' => [
            '฿',
            'bat tailandés',
        ],
        'TJR' => [
            'TJR',
            'rublo tayiko',
        ],
        'TJS' => [
            'TJS',
            'somoni tayiko',
        ],
        'TMM' => [
            'TMM',
            'manat turcomano (1993–2009)',
        ],
        'TMT' => [
            'TMT',
            'manat turcomano',
        ],
        'TND' => [
            'TND',
            'dinar tunecino',
        ],
        'TOP' => [
            'TOP',
            'paanga tongano',
        ],
        'TPE' => [
            'TPE',
            'escudo timorense',
        ],
        'TRL' => [
            'TRL',
            'lira turca (1922–2005)',
        ],
        'TRY' => [
            'TRY',
            'lira turca',
        ],
        'TTD' => [
            'TTD',
            'dólar de Trinidad y Tobago',
        ],
        'TWD' => [
            'TWD',
            'nuevo dólar taiwanés',
        ],
        'TZS' => [
            'TZS',
            'chelín tanzano',
        ],
        'UAH' => [
            'UAH',
            'grivna ucraniana',
        ],
        'UAK' => [
            'UAK',
            'karbovanet ucraniano',
        ],
        'UGS' => [
            'UGS',
            'chelín ugandés (1966–1987)',
        ],
        'UGX' => [
            'UGX',
            'chelín ugandés',
        ],
        'USD' => [
            'US$',
            'dólar estadounidense',
        ],
        'USN' => [
            'USN',
            'dólar estadounidense (día siguiente)',
        ],
        'USS' => [
            'USS',
            'dólar estadounidense (mismo día)',
        ],
        'UYI' => [
            'UYI',
            'peso uruguayo en unidades indexadas',
        ],
        'UYP' => [
            'UYP',
            'peso uruguayo (1975–1993)',
        ],
        'UYU' => [
            'UYU',
            'peso uruguayo',
        ],
        'UYW' => [
            'UYW',
            'unidad previsional uruguayo',
        ],
        'UZS' => [
            'UZS',
            'sum uzbeko',
        ],
        'VEB' => [
            'VEB',
            'bolívar venezolano (1871–2008)',
        ],
        'VEF' => [
            'VEF',
            'bolívar venezolano (2008–2018)',
        ],
        'VES' => [
            'VES',
            'bolívar venezolano',
        ],
        'VND' => [
            '₫',
            'dong vietnamita',
        ],
        'VUV' => [
            'VUV',
            'vatu vanuatense',
        ],
        'WST' => [
            'WST',
            'tala samoano',
        ],
        'XAF' => [
            'XAF',
            'franco CFA de África Central',
        ],
        'XCD' => [
            'XCD',
            'dólar del Caribe Oriental',
        ],
        'XCG' => [
            'Cg.',
            'florín caribeño',
        ],
        'XEU' => [
            'XEU',
            'unidad de moneda europea',
        ],
        'XFO' => [
            'XFO',
            'franco oro francés',
        ],
        'XFU' => [
            'XFU',
            'franco UIC francés',
        ],
        'XOF' => [
            'XOF',
            'franco CFA de África Occidental',
        ],
        'XPF' => [
            'CFPF',
            'franco CFP',
        ],
        'XRE' => [
            'XRE',
            'fondos RINET',
        ],
        'YDD' => [
            'YDD',
            'dinar yemení',
        ],
        'YER' => [
            'YER',
            'rial yemení',
        ],
        'YUD' => [
            'YUD',
            'dinar fuerte yugoslavo',
        ],
        'YUM' => [
            'YUM',
            'super dinar yugoslavo',
        ],
        'YUN' => [
            'YUN',
            'dinar convertible yugoslavo',
        ],
        'ZAL' => [
            'ZAL',
            'rand sudafricano (financiero)',
        ],
        'ZAR' => [
            'ZAR',
            'rand sudafricano',
        ],
        'ZMK' => [
            'ZMK',
            'kwacha zambiano (1968–2012)',
        ],
        'ZMW' => [
            'ZMW',
            'kuacha zambiano',
        ],
        'ZRN' => [
            'ZRN',
            'nuevo zaire zaireño',
        ],
        'ZRZ' => [
            'ZRZ',
            'zaire zaireño',
        ],
        'ZWD' => [
            'ZWD',
            'dólar de Zimbabue',
        ],
        'ZWG' => [
            'ZWG',
            'oro zimbabuense',
        ],
        'ZWL' => [
            'ZWL',
            'dólar zimbabuense',
        ],
    ],
];
