<?php

return [
    'Names' => [
        'BMD' => [
            'BMD',
            'Doolaara Beermudaa',
        ],
        'BRL' => [
            'R$',
            'Brazilian Real',
        ],
        'BZD' => [
            'BZD',
            'Doolaara Beliizee',
        ],
        'CAD' => [
            'CA$',
            'Doolaara Kanaadaa',
        ],
        'CNY' => [
            'CN¥',
            'Chinese Yuan Renminbi',
        ],
        'CRC' => [
            'CRC',
            'Koloonii Kostaa Rikaa',
        ],
        'ETB' => [
            'Br',
            'Itoophiyaa Birrii',
        ],
        'EUR' => [
            '€',
            'Euro',
        ],
        'GBP' => [
            '£',
            'British Pound',
        ],
        'INR' => [
            '₹',
            'Indian Rupee',
        ],
        'JPY' => [
            'JP¥',
            'Japanese Yen',
        ],
        'RUB' => [
            'RUB',
            'Russian Ruble',
        ],
        'USD' => [
            'US$',
            'Doolaara Ameerikaa',
        ],
    ],
];
