<?php

return [
    'Names' => [
        'AED' => [
            'AED',
            'Dirham ya Falme za Kiarabu',
        ],
        'AFN' => [
            'AFN',
            'Afghani ya Afghanistan',
        ],
        'ALL' => [
            'ALL',
            'Lek ya Albania',
        ],
        'AMD' => [
            'AMD',
            'Dram ya Armenia',
        ],
        'ANG' => [
            'ANG',
            'Guilder ya Antili za Kiholanzi',
        ],
        'AOA' => [
            'AOA',
            'Kwanza ya Angola',
        ],
        'ARS' => [
            'ARS',
            'Peso ya Ajentina',
        ],
        'AUD' => [
            'A$',
            'Dola ya Australia',
        ],
        'AWG' => [
            'AWG',
            'Florin ya Aruba',
        ],
        'AZN' => [
            'AZN',
            'Manat ya Azerbaijan',
        ],
        'BAM' => [
            'BAM',
            'Convertible Mark ya Bosnia na Hezegovina',
        ],
        'BBD' => [
            'BBD',
            'Dola ya Barbados',
        ],
        'BDT' => [
            'BDT',
            'Taka ya Bangladesh',
        ],
        'BGN' => [
            'BGN',
            'Lev ya Bulgaria',
        ],
        'BHD' => [
            'BHD',
            'Dinari ya Bahareni',
        ],
        'BIF' => [
            'BIF',
            'Faranga ya Burundi',
        ],
        'BMD' => [
            'BMD',
            'Dola ya Bermuda',
        ],
        'BND' => [
            'BND',
            'Dola ya Brunei',
        ],
        'BOB' => [
            'BOB',
            'Boliviano ya Bolivia',
        ],
        'BRL' => [
            'R$',
            'Real ya Brazil',
        ],
        'BSD' => [
            'BSD',
            'Dola ya Bahamas',
        ],
        'BTN' => [
            'BTN',
            'Ngultrum ya Bhutan',
        ],
        'BWP' => [
            'BWP',
            'Pula ya Botswana',
        ],
        'BYN' => [
            'BYN',
            'Ruble ya Belarus',
        ],
        'BYR' => [
            'BYR',
            'Ruble ya Belarusi (2000–2016)',
        ],
        'BZD' => [
            'BZD',
            'Dola ya Belize',
        ],
        'CAD' => [
            'CA$',
            'Dola ya Canada',
        ],
        'CDF' => [
            'CDF',
            'Faranga ya Kongo',
        ],
        'CHF' => [
            'CHF',
            'Faranga ya Uswisi',
        ],
        'CLP' => [
            'CLP',
            'Peso ya Chile',
        ],
        'CNH' => [
            'CNH',
            'Yuan ya Uchina (huru)',
        ],
        'CNY' => [
            'CN¥',
            'Yuan ya Uchina',
        ],
        'COP' => [
            'COP',
            'Peso ya Kolombia',
        ],
        'CRC' => [
            'CRC',
            'Colon ya Kostarika',
        ],
        'CUC' => [
            'CUC',
            'Peso ya Kuba Inayoweza Kubadilishwa',
        ],
        'CUP' => [
            'CUP',
            'Peso ya Kuba',
        ],
        'CVE' => [
            'CVE',
            'Eskudo ya Cape Verde',
        ],
        'CZK' => [
            'CZK',
            'Koruna ya Jamhuri ya Czech',
        ],
        'DJF' => [
            'DJF',
            'Faranga ya Jibuti',
        ],
        'DKK' => [
            'DKK',
            'Krone ya Denmark',
        ],
        'DOP' => [
            'DOP',
            'Peso ya Dominika',
        ],
        'DZD' => [
            'DZD',
            'Dinar ya Aljeria',
        ],
        'EGP' => [
            'EGP',
            'Pauni ya Misri',
        ],
        'ERN' => [
            'ERN',
            'Nakfa ya Eritrea',
        ],
        'ETB' => [
            'ETB',
            'Birr ya Uhabeshi',
        ],
        'EUR' => [
            '€',
            'Yuro',
        ],
        'FJD' => [
            'FJD',
            'Dola ya Fiji',
        ],
        'FKP' => [
            'FKP',
            'Pauni ya Visiwa vya Falkland',
        ],
        'GBP' => [
            '£',
            'Pauni ya Uingereza',
        ],
        'GEL' => [
            'GEL',
            'Lari ya Jojia',
        ],
        'GHC' => [
            'GHC',
            'Sedi ya Ghana',
        ],
        'GHS' => [
            'GHS',
            'Cedi ya Ghana',
        ],
        'GIP' => [
            'GIP',
            'Pauni ya Gibraltar',
        ],
        'GMD' => [
            'GMD',
            'Dalasi ya Gambia',
        ],
        'GNF' => [
            'GNF',
            'Faranga ya Guinea',
        ],
        'GNS' => [
            'GNS',
            'Faranga ya Gine',
        ],
        'GTQ' => [
            'GTQ',
            'Quetzal ya Guatemala',
        ],
        'GYD' => [
            'GYD',
            'Dola ya Guyana',
        ],
        'HKD' => [
            'HK$',
            'Dola ya Hong Kong',
        ],
        'HNL' => [
            'HNL',
            'Lempira ya Hondurasi',
        ],
        'HRK' => [
            'HRK',
            'Kuna ya Korasia',
        ],
        'HTG' => [
            'HTG',
            'Gourde ya Haiti',
        ],
        'HUF' => [
            'HUF',
            'Forint ya Hungaria',
        ],
        'IDR' => [
            'IDR',
            'Rupiah ya Indonesia',
        ],
        'ILS' => [
            '₪',
            'Shekeli Mpya ya Israel',
        ],
        'INR' => [
            '₹',
            'Rupia ya India',
        ],
        'IQD' => [
            'IQD',
            'Dinari ya Iraki',
        ],
        'IRR' => [
            'IRR',
            'Rial ya Iran',
        ],
        'ISK' => [
            'ISK',
            'Krona ya Aisilandi',
        ],
        'JMD' => [
            'JMD',
            'Dola ya Jamaika',
        ],
        'JOD' => [
            'JOD',
            'Dinari ya Jordan',
        ],
        'JPY' => [
            'JP¥',
            'Yen ya Japani',
        ],
        'KES' => [
            'Ksh',
            'Shilingi ya Kenya',
        ],
        'KGS' => [
            'KGS',
            'Som ya Kyrgystan',
        ],
        'KHR' => [
            'KHR',
            'Riel ya Kambodia',
        ],
        'KMF' => [
            'KMF',
            'Faranga ya Komoro',
        ],
        'KPW' => [
            'KPW',
            'Won ya Korea Kaskazini',
        ],
        'KRW' => [
            '₩',
            'Won ya Korea Kusini',
        ],
        'KWD' => [
            'KWD',
            'Dinari ya Kuwait',
        ],
        'KYD' => [
            'KYD',
            'Dola ya Visiwa vya Cayman',
        ],
        'KZT' => [
            'KZT',
            'Tenge ya Kazakhstan',
        ],
        'LAK' => [
            'LAK',
            'Kip ya Laosi',
        ],
        'LBP' => [
            'LBP',
            'Pauni ya Lebanon',
        ],
        'LKR' => [
            'LKR',
            'Rupia ya Sri Lanka',
        ],
        'LRD' => [
            'LRD',
            'Dola ya Liberia',
        ],
        'LSL' => [
            'LSL',
            'Loti ya Lesoto',
        ],
        'LTL' => [
            'LTL',
            'Litas ya Lithuania',
        ],
        'LVL' => [
            'LVL',
            'Lats ya Lativia',
        ],
        'LYD' => [
            'LYD',
            'Dinari ya Libya',
        ],
        'MAD' => [
            'MAD',
            'Dirham ya Moroko',
        ],
        'MDL' => [
            'MDL',
            'Leu ya Moldova',
        ],
        'MGA' => [
            'MGA',
            'Ariari ya Madagaska',
        ],
        'MKD' => [
            'MKD',
            'Denar ya Masedonia',
        ],
        'MMK' => [
            'MMK',
            'Kyat ya Myanmar',
        ],
        'MNT' => [
            'MNT',
            'Tugrik ya Mongolia',
        ],
        'MOP' => [
            'MOP',
            'Pataca ya Macau',
        ],
        'MRO' => [
            'MRO',
            'Ouguiya ya Mauritania (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'Ouguiya ya Moritania',
        ],
        'MUR' => [
            'MUR',
            'Rupia ya Morisi',
        ],
        'MVR' => [
            'MVR',
            'Rufiyaa ya Maldives',
        ],
        'MWK' => [
            'MWK',
            'Kwacha ya Malawi',
        ],
        'MXN' => [
            'MX$',
            'Peso ya Meksiko',
        ],
        'MYR' => [
            'MYR',
            'Ringgit ya Malaysia',
        ],
        'MZM' => [
            'MZM',
            'Metikali ya Msumbiji (1980–2006)',
        ],
        'MZN' => [
            'MZN',
            'Metikali ya Msumbiji',
        ],
        'NAD' => [
            'NAD',
            'Dola ya Namibia',
        ],
        'NGN' => [
            'NGN',
            'Naira ya Nigeria',
        ],
        'NIO' => [
            'NIO',
            'Cordoba ya Nikaragwa',
        ],
        'NOK' => [
            'NOK',
            'Krone ya Norwe',
        ],
        'NPR' => [
            'NPR',
            'Rupia ya Nepal',
        ],
        'NZD' => [
            'NZ$',
            'Dola ya Nyuzilandi',
        ],
        'OMR' => [
            'OMR',
            'Rial ya Omani',
        ],
        'PAB' => [
            'PAB',
            'Balboa ya Panama',
        ],
        'PEN' => [
            'PEN',
            'Sol ya Peru',
        ],
        'PGK' => [
            'PGK',
            'Kina ya Papua New Guinea',
        ],
        'PHP' => [
            'PHP',
            'Peso ya Ufilipino',
        ],
        'PKR' => [
            'PKR',
            'Rupia ya Pakistan',
        ],
        'PLN' => [
            'PLN',
            'Zloty ya Poland',
        ],
        'PYG' => [
            'PYG',
            'Guarani ya Paragwai',
        ],
        'QAR' => [
            'QAR',
            'Rial ya Qatar',
        ],
        'RON' => [
            'RON',
            'Leu ya Romania',
        ],
        'RSD' => [
            'RSD',
            'Dinar ya Serbia',
        ],
        'RUB' => [
            'RUB',
            'Ruble ya Urusi',
        ],
        'RWF' => [
            'RWF',
            'Faranga ya Rwanda',
        ],
        'SAR' => [
            'SAR',
            'Riyal ya Saudia',
        ],
        'SBD' => [
            'SBD',
            'Dola ya Visiwa vya Solomon',
        ],
        'SCR' => [
            'SCR',
            'Rupia ya Ushelisheli',
        ],
        'SDG' => [
            'SDG',
            'Pauni ya Sudan',
        ],
        'SDP' => [
            'SDP',
            'Pauni ya Sudani (1957–1998)',
        ],
        'SEK' => [
            'SEK',
            'Krona ya Uswidi',
        ],
        'SGD' => [
            'SGD',
            'Dola ya Singapore',
        ],
        'SHP' => [
            'SHP',
            'Pauni ya St. Helena',
        ],
        'SLE' => [
            'SLE',
            'Leone ya Siera Leoni',
        ],
        'SLL' => [
            'SLL',
            'Leone ya Siera Leoni (1964—2022)',
        ],
        'SOS' => [
            'SOS',
            'Shilingi ya Somalia',
        ],
        'SRD' => [
            'SRD',
            'Dola ya Suriname',
        ],
        'SSP' => [
            'SSP',
            'Pauni ya Sudan Kusini',
        ],
        'STD' => [
            'STD',
            'Dobra ya Sao Tome na Principe (1977–2017)',
        ],
        'STN' => [
            'STN',
            'Dobra ya Sao Tome na Principe',
        ],
        'SYP' => [
            'SYP',
            'Pauni ya Syria',
        ],
        'SZL' => [
            'SZL',
            'Lilangeni ya Uswazi',
        ],
        'THB' => [
            '฿',
            'Baht ya Tailandi',
        ],
        'TJS' => [
            'TJS',
            'Somoni ya Tajikistan',
        ],
        'TMT' => [
            'TMT',
            'Manat ya Turkmenistan',
        ],
        'TND' => [
            'TND',
            'Dinari ya Tunisia',
        ],
        'TOP' => [
            'TOP',
            'Paʻanga ya Tonga',
        ],
        'TRY' => [
            'TRY',
            'Lira ya Uturuki',
        ],
        'TTD' => [
            'TTD',
            'Dola ya Trinidad na Tobago',
        ],
        'TWD' => [
            'NT$',
            'Dola ya Taiwan',
        ],
        'TZS' => [
            'TSh',
            'Shilingi ya Tanzania',
        ],
        'UAH' => [
            'UAH',
            'Hryvnia ya Ukraine',
        ],
        'UGX' => [
            'UGX',
            'Shilingi ya Uganda',
        ],
        'USD' => [
            'US$',
            'Dola ya Marekani',
        ],
        'UYU' => [
            'UYU',
            'Peso ya Urugwai',
        ],
        'UZS' => [
            'UZS',
            'Som ya Uzbekistan',
        ],
        'VEF' => [
            'VEF',
            'Bolivar ya Venezuela (2008–2018)',
        ],
        'VES' => [
            'VES',
            'Bolivar ya Venezuela',
        ],
        'VND' => [
            '₫',
            'Dong ya Vietnam',
        ],
        'VUV' => [
            'VUV',
            'Vatu ya Vanuatu',
        ],
        'WST' => [
            'WST',
            'Tala ya Samoa',
        ],
        'XAF' => [
            'FCFA',
            'Faranga ya Afrika ya Kati CFA',
        ],
        'XCD' => [
            'EC$',
            'Dola ya Karibi Mashariki',
        ],
        'XCG' => [
            'Cg.',
            'Guilder ya Karibe',
        ],
        'XOF' => [
            'F CFA',
            'Faranga ya Afrika Magharibi CFA',
        ],
        'XPF' => [
            'CFPF',
            'Faranga ya CFP',
        ],
        'YER' => [
            'YER',
            'Rial ya Yemen',
        ],
        'ZAR' => [
            'ZAR',
            'Randi ya Afrika Kusini',
        ],
        'ZMK' => [
            'ZMK',
            'Kwacha ya Zambia (1968–2012)',
        ],
        'ZMW' => [
            'ZMW',
            'Kwacha ya Zambia',
        ],
        'ZWD' => [
            'ZWD',
            'Dola ya Zimbabwe',
        ],
        'ZWG' => [
            'ZWG',
            'Dhahabu ya Zimbabwe',
        ],
    ],
];
