<?php

return [
    'Names' => [
        'af' => 'afrikaans',
        'af_NA' => 'afrikaans (Namibia)',
        'af_ZA' => 'afrikaans (Republika Południowej Afryki)',
        'ak' => 'akan',
        'ak_GH' => 'akan (Ghana)',
        'am' => 'amharski',
        'am_ET' => 'amharski (Etiopia)',
        'ar' => 'arabski',
        'ar_001' => 'arabski (świat)',
        'ar_AE' => 'arabski (Zjednoczone Emiraty Arabskie)',
        'ar_BH' => 'arabski (Bahrajn)',
        'ar_DJ' => 'arabski (Dżibuti)',
        'ar_DZ' => 'arabski (Algieria)',
        'ar_EG' => 'arabski (Egipt)',
        'ar_EH' => 'arabski (Sahara Zachodnia)',
        'ar_ER' => 'arabski (Erytrea)',
        'ar_IL' => 'arabski (Izrael)',
        'ar_IQ' => 'arabski (Irak)',
        'ar_JO' => 'arabski (Jordania)',
        'ar_KM' => 'arabski (Komory)',
        'ar_KW' => 'arabski (Kuwejt)',
        'ar_LB' => 'arabski (Liban)',
        'ar_LY' => 'arabski (Libia)',
        'ar_MA' => 'arabski (Maroko)',
        'ar_MR' => 'arabski (Mauretania)',
        'ar_OM' => 'arabski (Oman)',
        'ar_PS' => 'arabski (Terytoria Palestyńskie)',
        'ar_QA' => 'arabski (Katar)',
        'ar_SA' => 'arabski (Arabia Saudyjska)',
        'ar_SD' => 'arabski (Sudan)',
        'ar_SO' => 'arabski (Somalia)',
        'ar_SS' => 'arabski (Sudan Południowy)',
        'ar_SY' => 'arabski (Syria)',
        'ar_TD' => 'arabski (Czad)',
        'ar_TN' => 'arabski (Tunezja)',
        'ar_YE' => 'arabski (Jemen)',
        'as' => 'asamski',
        'as_IN' => 'asamski (Indie)',
        'az' => 'azerbejdżański',
        'az_AZ' => 'azerbejdżański (Azerbejdżan)',
        'az_Cyrl' => 'azerbejdżański (cyrylica)',
        'az_Cyrl_AZ' => 'azerbejdżański (cyrylica, Azerbejdżan)',
        'az_Latn' => 'azerbejdżański (łacińskie)',
        'az_Latn_AZ' => 'azerbejdżański (łacińskie, Azerbejdżan)',
        'ba' => 'baszkirski',
        'ba_RU' => 'baszkirski (Rosja)',
        'be' => 'białoruski',
        'be_BY' => 'białoruski (Białoruś)',
        'bg' => 'bułgarski',
        'bg_BG' => 'bułgarski (Bułgaria)',
        'bm' => 'bambara',
        'bm_ML' => 'bambara (Mali)',
        'bn' => 'bengalski',
        'bn_BD' => 'bengalski (Bangladesz)',
        'bn_IN' => 'bengalski (Indie)',
        'bo' => 'tybetański',
        'bo_CN' => 'tybetański (Chiny)',
        'bo_IN' => 'tybetański (Indie)',
        'br' => 'bretoński',
        'br_FR' => 'bretoński (Francja)',
        'bs' => 'bośniacki',
        'bs_BA' => 'bośniacki (Bośnia i Hercegowina)',
        'bs_Cyrl' => 'bośniacki (cyrylica)',
        'bs_Cyrl_BA' => 'bośniacki (cyrylica, Bośnia i Hercegowina)',
        'bs_Latn' => 'bośniacki (łacińskie)',
        'bs_Latn_BA' => 'bośniacki (łacińskie, Bośnia i Hercegowina)',
        'ca' => 'kataloński',
        'ca_AD' => 'kataloński (Andora)',
        'ca_ES' => 'kataloński (Hiszpania)',
        'ca_FR' => 'kataloński (Francja)',
        'ca_IT' => 'kataloński (Włochy)',
        'ce' => 'czeczeński',
        'ce_RU' => 'czeczeński (Rosja)',
        'cs' => 'czeski',
        'cs_CZ' => 'czeski (Czechy)',
        'cv' => 'czuwaski',
        'cv_RU' => 'czuwaski (Rosja)',
        'cy' => 'walijski',
        'cy_GB' => 'walijski (Wielka Brytania)',
        'da' => 'duński',
        'da_DK' => 'duński (Dania)',
        'da_GL' => 'duński (Grenlandia)',
        'de' => 'niemiecki',
        'de_AT' => 'niemiecki (Austria)',
        'de_BE' => 'niemiecki (Belgia)',
        'de_CH' => 'niemiecki (Szwajcaria)',
        'de_DE' => 'niemiecki (Niemcy)',
        'de_IT' => 'niemiecki (Włochy)',
        'de_LI' => 'niemiecki (Liechtenstein)',
        'de_LU' => 'niemiecki (Luksemburg)',
        'dz' => 'dzongkha',
        'dz_BT' => 'dzongkha (Bhutan)',
        'ee' => 'ewe',
        'ee_GH' => 'ewe (Ghana)',
        'ee_TG' => 'ewe (Togo)',
        'el' => 'grecki',
        'el_CY' => 'grecki (Cypr)',
        'el_GR' => 'grecki (Grecja)',
        'en' => 'angielski',
        'en_001' => 'angielski (świat)',
        'en_150' => 'angielski (Europa)',
        'en_AE' => 'angielski (Zjednoczone Emiraty Arabskie)',
        'en_AG' => 'angielski (Antigua i Barbuda)',
        'en_AI' => 'angielski (Anguilla)',
        'en_AS' => 'angielski (Samoa Amerykańskie)',
        'en_AT' => 'angielski (Austria)',
        'en_AU' => 'angielski (Australia)',
        'en_BB' => 'angielski (Barbados)',
        'en_BE' => 'angielski (Belgia)',
        'en_BI' => 'angielski (Burundi)',
        'en_BM' => 'angielski (Bermudy)',
        'en_BS' => 'angielski (Bahamy)',
        'en_BW' => 'angielski (Botswana)',
        'en_BZ' => 'angielski (Belize)',
        'en_CA' => 'angielski (Kanada)',
        'en_CC' => 'angielski (Wyspy Kokosowe)',
        'en_CH' => 'angielski (Szwajcaria)',
        'en_CK' => 'angielski (Wyspy Cooka)',
        'en_CM' => 'angielski (Kamerun)',
        'en_CX' => 'angielski (Wyspa Bożego Narodzenia)',
        'en_CY' => 'angielski (Cypr)',
        'en_CZ' => 'angielski (Czechy)',
        'en_DE' => 'angielski (Niemcy)',
        'en_DK' => 'angielski (Dania)',
        'en_DM' => 'angielski (Dominika)',
        'en_EE' => 'angielski (Estonia)',
        'en_ER' => 'angielski (Erytrea)',
        'en_ES' => 'angielski (Hiszpania)',
        'en_FI' => 'angielski (Finlandia)',
        'en_FJ' => 'angielski (Fidżi)',
        'en_FK' => 'angielski (Falklandy)',
        'en_FM' => 'angielski (Mikronezja)',
        'en_FR' => 'angielski (Francja)',
        'en_GB' => 'angielski (Wielka Brytania)',
        'en_GD' => 'angielski (Grenada)',
        'en_GE' => 'angielski (Gruzja)',
        'en_GG' => 'angielski (Guernsey)',
        'en_GH' => 'angielski (Ghana)',
        'en_GI' => 'angielski (Gibraltar)',
        'en_GM' => 'angielski (Gambia)',
        'en_GS' => 'angielski (Georgia Południowa i Sandwich Południowy)',
        'en_GU' => 'angielski (Guam)',
        'en_GY' => 'angielski (Gujana)',
        'en_HK' => 'angielski (SRA Hongkong [Chiny])',
        'en_HU' => 'angielski (Węgry)',
        'en_ID' => 'angielski (Indonezja)',
        'en_IE' => 'angielski (Irlandia)',
        'en_IL' => 'angielski (Izrael)',
        'en_IM' => 'angielski (Wyspa Man)',
        'en_IN' => 'angielski (Indie)',
        'en_IO' => 'angielski (Brytyjskie Terytorium Oceanu Indyjskiego)',
        'en_IT' => 'angielski (Włochy)',
        'en_JE' => 'angielski (Jersey)',
        'en_JM' => 'angielski (Jamajka)',
        'en_JP' => 'angielski (Japonia)',
        'en_KE' => 'angielski (Kenia)',
        'en_KI' => 'angielski (Kiribati)',
        'en_KN' => 'angielski (Saint Kitts i Nevis)',
        'en_KY' => 'angielski (Kajmany)',
        'en_LC' => 'angielski (Saint Lucia)',
        'en_LR' => 'angielski (Liberia)',
        'en_LS' => 'angielski (Lesotho)',
        'en_LT' => 'angielski (Litwa)',
        'en_LV' => 'angielski (Łotwa)',
        'en_MG' => 'angielski (Madagaskar)',
        'en_MH' => 'angielski (Wyspy Marshalla)',
        'en_MO' => 'angielski (SRA Makau [Chiny])',
        'en_MP' => 'angielski (Mariany Północne)',
        'en_MS' => 'angielski (Montserrat)',
        'en_MT' => 'angielski (Malta)',
        'en_MU' => 'angielski (Mauritius)',
        'en_MV' => 'angielski (Malediwy)',
        'en_MW' => 'angielski (Malawi)',
        'en_MY' => 'angielski (Malezja)',
        'en_NA' => 'angielski (Namibia)',
        'en_NF' => 'angielski (Norfolk)',
        'en_NG' => 'angielski (Nigeria)',
        'en_NL' => 'angielski (Holandia)',
        'en_NO' => 'angielski (Norwegia)',
        'en_NR' => 'angielski (Nauru)',
        'en_NU' => 'angielski (Niue)',
        'en_NZ' => 'angielski (Nowa Zelandia)',
        'en_PG' => 'angielski (Papua-Nowa Gwinea)',
        'en_PH' => 'angielski (Filipiny)',
        'en_PK' => 'angielski (Pakistan)',
        'en_PL' => 'angielski (Polska)',
        'en_PN' => 'angielski (Pitcairn)',
        'en_PR' => 'angielski (Portoryko)',
        'en_PT' => 'angielski (Portugalia)',
        'en_PW' => 'angielski (Palau)',
        'en_RO' => 'angielski (Rumunia)',
        'en_RW' => 'angielski (Rwanda)',
        'en_SB' => 'angielski (Wyspy Salomona)',
        'en_SC' => 'angielski (Seszele)',
        'en_SD' => 'angielski (Sudan)',
        'en_SE' => 'angielski (Szwecja)',
        'en_SG' => 'angielski (Singapur)',
        'en_SH' => 'angielski (Wyspa Świętej Heleny)',
        'en_SI' => 'angielski (Słowenia)',
        'en_SK' => 'angielski (Słowacja)',
        'en_SL' => 'angielski (Sierra Leone)',
        'en_SS' => 'angielski (Sudan Południowy)',
        'en_SX' => 'angielski (Sint Maarten)',
        'en_SZ' => 'angielski (Eswatini)',
        'en_TC' => 'angielski (Turks i Caicos)',
        'en_TK' => 'angielski (Tokelau)',
        'en_TO' => 'angielski (Tonga)',
        'en_TT' => 'angielski (Trynidad i Tobago)',
        'en_TV' => 'angielski (Tuvalu)',
        'en_TZ' => 'angielski (Tanzania)',
        'en_UA' => 'angielski (Ukraina)',
        'en_UG' => 'angielski (Uganda)',
        'en_UM' => 'angielski (Dalekie Wyspy Mniejsze Stanów Zjednoczonych)',
        'en_US' => 'angielski (Stany Zjednoczone)',
        'en_VC' => 'angielski (Saint Vincent i Grenadyny)',
        'en_VG' => 'angielski (Brytyjskie Wyspy Dziewicze)',
        'en_VI' => 'angielski (Wyspy Dziewicze Stanów Zjednoczonych)',
        'en_VU' => 'angielski (Vanuatu)',
        'en_WS' => 'angielski (Samoa)',
        'en_ZA' => 'angielski (Republika Południowej Afryki)',
        'en_ZM' => 'angielski (Zambia)',
        'en_ZW' => 'angielski (Zimbabwe)',
        'eo' => 'esperanto',
        'eo_001' => 'esperanto (świat)',
        'es' => 'hiszpański',
        'es_419' => 'hiszpański (Ameryka Łacińska)',
        'es_AR' => 'hiszpański (Argentyna)',
        'es_BO' => 'hiszpański (Boliwia)',
        'es_BR' => 'hiszpański (Brazylia)',
        'es_BZ' => 'hiszpański (Belize)',
        'es_CL' => 'hiszpański (Chile)',
        'es_CO' => 'hiszpański (Kolumbia)',
        'es_CR' => 'hiszpański (Kostaryka)',
        'es_CU' => 'hiszpański (Kuba)',
        'es_DO' => 'hiszpański (Dominikana)',
        'es_EC' => 'hiszpański (Ekwador)',
        'es_ES' => 'hiszpański (Hiszpania)',
        'es_GQ' => 'hiszpański (Gwinea Równikowa)',
        'es_GT' => 'hiszpański (Gwatemala)',
        'es_HN' => 'hiszpański (Honduras)',
        'es_MX' => 'hiszpański (Meksyk)',
        'es_NI' => 'hiszpański (Nikaragua)',
        'es_PA' => 'hiszpański (Panama)',
        'es_PE' => 'hiszpański (Peru)',
        'es_PH' => 'hiszpański (Filipiny)',
        'es_PR' => 'hiszpański (Portoryko)',
        'es_PY' => 'hiszpański (Paragwaj)',
        'es_SV' => 'hiszpański (Salwador)',
        'es_US' => 'hiszpański (Stany Zjednoczone)',
        'es_UY' => 'hiszpański (Urugwaj)',
        'es_VE' => 'hiszpański (Wenezuela)',
        'et' => 'estoński',
        'et_EE' => 'estoński (Estonia)',
        'eu' => 'baskijski',
        'eu_ES' => 'baskijski (Hiszpania)',
        'fa' => 'perski',
        'fa_AF' => 'perski (Afganistan)',
        'fa_IR' => 'perski (Iran)',
        'ff' => 'fulani',
        'ff_Adlm' => 'fulani (adlam)',
        'ff_Adlm_BF' => 'fulani (adlam, Burkina Faso)',
        'ff_Adlm_CM' => 'fulani (adlam, Kamerun)',
        'ff_Adlm_GH' => 'fulani (adlam, Ghana)',
        'ff_Adlm_GM' => 'fulani (adlam, Gambia)',
        'ff_Adlm_GN' => 'fulani (adlam, Gwinea)',
        'ff_Adlm_GW' => 'fulani (adlam, Gwinea Bissau)',
        'ff_Adlm_LR' => 'fulani (adlam, Liberia)',
        'ff_Adlm_MR' => 'fulani (adlam, Mauretania)',
        'ff_Adlm_NE' => 'fulani (adlam, Niger)',
        'ff_Adlm_NG' => 'fulani (adlam, Nigeria)',
        'ff_Adlm_SL' => 'fulani (adlam, Sierra Leone)',
        'ff_Adlm_SN' => 'fulani (adlam, Senegal)',
        'ff_CM' => 'fulani (Kamerun)',
        'ff_GN' => 'fulani (Gwinea)',
        'ff_Latn' => 'fulani (łacińskie)',
        'ff_Latn_BF' => 'fulani (łacińskie, Burkina Faso)',
        'ff_Latn_CM' => 'fulani (łacińskie, Kamerun)',
        'ff_Latn_GH' => 'fulani (łacińskie, Ghana)',
        'ff_Latn_GM' => 'fulani (łacińskie, Gambia)',
        'ff_Latn_GN' => 'fulani (łacińskie, Gwinea)',
        'ff_Latn_GW' => 'fulani (łacińskie, Gwinea Bissau)',
        'ff_Latn_LR' => 'fulani (łacińskie, Liberia)',
        'ff_Latn_MR' => 'fulani (łacińskie, Mauretania)',
        'ff_Latn_NE' => 'fulani (łacińskie, Niger)',
        'ff_Latn_NG' => 'fulani (łacińskie, Nigeria)',
        'ff_Latn_SL' => 'fulani (łacińskie, Sierra Leone)',
        'ff_Latn_SN' => 'fulani (łacińskie, Senegal)',
        'ff_MR' => 'fulani (Mauretania)',
        'ff_SN' => 'fulani (Senegal)',
        'fi' => 'fiński',
        'fi_FI' => 'fiński (Finlandia)',
        'fo' => 'farerski',
        'fo_DK' => 'farerski (Dania)',
        'fo_FO' => 'farerski (Wyspy Owcze)',
        'fr' => 'francuski',
        'fr_BE' => 'francuski (Belgia)',
        'fr_BF' => 'francuski (Burkina Faso)',
        'fr_BI' => 'francuski (Burundi)',
        'fr_BJ' => 'francuski (Benin)',
        'fr_BL' => 'francuski (Saint-Barthélemy)',
        'fr_CA' => 'francuski (Kanada)',
        'fr_CD' => 'francuski (Demokratyczna Republika Konga)',
        'fr_CF' => 'francuski (Republika Środkowoafrykańska)',
        'fr_CG' => 'francuski (Kongo)',
        'fr_CH' => 'francuski (Szwajcaria)',
        'fr_CI' => 'francuski (Côte d’Ivoire)',
        'fr_CM' => 'francuski (Kamerun)',
        'fr_DJ' => 'francuski (Dżibuti)',
        'fr_DZ' => 'francuski (Algieria)',
        'fr_FR' => 'francuski (Francja)',
        'fr_GA' => 'francuski (Gabon)',
        'fr_GF' => 'francuski (Gujana Francuska)',
        'fr_GN' => 'francuski (Gwinea)',
        'fr_GP' => 'francuski (Gwadelupa)',
        'fr_GQ' => 'francuski (Gwinea Równikowa)',
        'fr_HT' => 'francuski (Haiti)',
        'fr_KM' => 'francuski (Komory)',
        'fr_LU' => 'francuski (Luksemburg)',
        'fr_MA' => 'francuski (Maroko)',
        'fr_MC' => 'francuski (Monako)',
        'fr_MF' => 'francuski (Saint-Martin)',
        'fr_MG' => 'francuski (Madagaskar)',
        'fr_ML' => 'francuski (Mali)',
        'fr_MQ' => 'francuski (Martynika)',
        'fr_MR' => 'francuski (Mauretania)',
        'fr_MU' => 'francuski (Mauritius)',
        'fr_NC' => 'francuski (Nowa Kaledonia)',
        'fr_NE' => 'francuski (Niger)',
        'fr_PF' => 'francuski (Polinezja Francuska)',
        'fr_PM' => 'francuski (Saint-Pierre i Miquelon)',
        'fr_RE' => 'francuski (Reunion)',
        'fr_RW' => 'francuski (Rwanda)',
        'fr_SC' => 'francuski (Seszele)',
        'fr_SN' => 'francuski (Senegal)',
        'fr_SY' => 'francuski (Syria)',
        'fr_TD' => 'francuski (Czad)',
        'fr_TG' => 'francuski (Togo)',
        'fr_TN' => 'francuski (Tunezja)',
        'fr_VU' => 'francuski (Vanuatu)',
        'fr_WF' => 'francuski (Wallis i Futuna)',
        'fr_YT' => 'francuski (Majotta)',
        'fy' => 'zachodniofryzyjski',
        'fy_NL' => 'zachodniofryzyjski (Holandia)',
        'ga' => 'irlandzki',
        'ga_GB' => 'irlandzki (Wielka Brytania)',
        'ga_IE' => 'irlandzki (Irlandia)',
        'gd' => 'szkocki gaelicki',
        'gd_GB' => 'szkocki gaelicki (Wielka Brytania)',
        'gl' => 'galicyjski',
        'gl_ES' => 'galicyjski (Hiszpania)',
        'gu' => 'gudżarati',
        'gu_IN' => 'gudżarati (Indie)',
        'gv' => 'manx',
        'gv_IM' => 'manx (Wyspa Man)',
        'ha' => 'hausa',
        'ha_GH' => 'hausa (Ghana)',
        'ha_NE' => 'hausa (Niger)',
        'ha_NG' => 'hausa (Nigeria)',
        'he' => 'hebrajski',
        'he_IL' => 'hebrajski (Izrael)',
        'hi' => 'hindi',
        'hi_IN' => 'hindi (Indie)',
        'hi_Latn' => 'hindi (łacińskie)',
        'hi_Latn_IN' => 'hindi (łacińskie, Indie)',
        'hr' => 'chorwacki',
        'hr_BA' => 'chorwacki (Bośnia i Hercegowina)',
        'hr_HR' => 'chorwacki (Chorwacja)',
        'hu' => 'węgierski',
        'hu_HU' => 'węgierski (Węgry)',
        'hy' => 'ormiański',
        'hy_AM' => 'ormiański (Armenia)',
        'ia' => 'interlingua',
        'ia_001' => 'interlingua (świat)',
        'id' => 'indonezyjski',
        'id_ID' => 'indonezyjski (Indonezja)',
        'ie' => 'interlingue',
        'ie_EE' => 'interlingue (Estonia)',
        'ig' => 'igbo',
        'ig_NG' => 'igbo (Nigeria)',
        'ii' => 'syczuański',
        'ii_CN' => 'syczuański (Chiny)',
        'is' => 'islandzki',
        'is_IS' => 'islandzki (Islandia)',
        'it' => 'włoski',
        'it_CH' => 'włoski (Szwajcaria)',
        'it_IT' => 'włoski (Włochy)',
        'it_SM' => 'włoski (San Marino)',
        'it_VA' => 'włoski (Watykan)',
        'ja' => 'japoński',
        'ja_JP' => 'japoński (Japonia)',
        'jv' => 'jawajski',
        'jv_ID' => 'jawajski (Indonezja)',
        'ka' => 'gruziński',
        'ka_GE' => 'gruziński (Gruzja)',
        'ki' => 'kikuju',
        'ki_KE' => 'kikuju (Kenia)',
        'kk' => 'kazachski',
        'kk_Arab' => 'kazachski (arabskie)',
        'kk_Arab_CN' => 'kazachski (arabskie, Chiny)',
        'kk_Cyrl' => 'kazachski (cyrylica)',
        'kk_Cyrl_KZ' => 'kazachski (cyrylica, Kazachstan)',
        'kk_KZ' => 'kazachski (Kazachstan)',
        'kl' => 'grenlandzki',
        'kl_GL' => 'grenlandzki (Grenlandia)',
        'km' => 'khmerski',
        'km_KH' => 'khmerski (Kambodża)',
        'kn' => 'kannada',
        'kn_IN' => 'kannada (Indie)',
        'ko' => 'koreański',
        'ko_CN' => 'koreański (Chiny)',
        'ko_KP' => 'koreański (Korea Północna)',
        'ko_KR' => 'koreański (Korea Południowa)',
        'ks' => 'kaszmirski',
        'ks_Arab' => 'kaszmirski (arabskie)',
        'ks_Arab_IN' => 'kaszmirski (arabskie, Indie)',
        'ks_Deva' => 'kaszmirski (dewanagari)',
        'ks_Deva_IN' => 'kaszmirski (dewanagari, Indie)',
        'ks_IN' => 'kaszmirski (Indie)',
        'ku' => 'kurdyjski',
        'ku_Latn' => 'kurdyjski (łacińskie)',
        'ku_Latn_IQ' => 'kurdyjski (łacińskie, Irak)',
        'ku_Latn_SY' => 'kurdyjski (łacińskie, Syria)',
        'ku_Latn_TR' => 'kurdyjski (łacińskie, Turcja)',
        'ku_SY' => 'kurdyjski (Syria)',
        'ku_TR' => 'kurdyjski (Turcja)',
        'kw' => 'kornijski',
        'kw_GB' => 'kornijski (Wielka Brytania)',
        'ky' => 'kirgiski',
        'ky_KG' => 'kirgiski (Kirgistan)',
        'lb' => 'luksemburski',
        'lb_LU' => 'luksemburski (Luksemburg)',
        'lg' => 'ganda',
        'lg_UG' => 'ganda (Uganda)',
        'ln' => 'lingala',
        'ln_AO' => 'lingala (Angola)',
        'ln_CD' => 'lingala (Demokratyczna Republika Konga)',
        'ln_CF' => 'lingala (Republika Środkowoafrykańska)',
        'ln_CG' => 'lingala (Kongo)',
        'lo' => 'laotański',
        'lo_LA' => 'laotański (Laos)',
        'lt' => 'litewski',
        'lt_LT' => 'litewski (Litwa)',
        'lu' => 'luba-katanga',
        'lu_CD' => 'luba-katanga (Demokratyczna Republika Konga)',
        'lv' => 'łotewski',
        'lv_LV' => 'łotewski (Łotwa)',
        'mg' => 'malgaski',
        'mg_MG' => 'malgaski (Madagaskar)',
        'mi' => 'maoryjski',
        'mi_NZ' => 'maoryjski (Nowa Zelandia)',
        'mk' => 'macedoński',
        'mk_MK' => 'macedoński (Macedonia Północna)',
        'ml' => 'malajalam',
        'ml_IN' => 'malajalam (Indie)',
        'mn' => 'mongolski',
        'mn_MN' => 'mongolski (Mongolia)',
        'mr' => 'marathi',
        'mr_IN' => 'marathi (Indie)',
        'ms' => 'malajski',
        'ms_BN' => 'malajski (Brunei)',
        'ms_ID' => 'malajski (Indonezja)',
        'ms_MY' => 'malajski (Malezja)',
        'ms_SG' => 'malajski (Singapur)',
        'mt' => 'maltański',
        'mt_MT' => 'maltański (Malta)',
        'my' => 'birmański',
        'my_MM' => 'birmański (Mjanma [Birma])',
        'nb' => 'norweski [bokmål]',
        'nb_NO' => 'norweski [bokmål] (Norwegia)',
        'nb_SJ' => 'norweski [bokmål] (Svalbard i Jan Mayen)',
        'nd' => 'ndebele północny',
        'nd_ZW' => 'ndebele północny (Zimbabwe)',
        'ne' => 'nepalski',
        'ne_IN' => 'nepalski (Indie)',
        'ne_NP' => 'nepalski (Nepal)',
        'nl' => 'niderlandzki',
        'nl_AW' => 'niderlandzki (Aruba)',
        'nl_BE' => 'niderlandzki (Belgia)',
        'nl_BQ' => 'niderlandzki (Niderlandy Karaibskie)',
        'nl_CW' => 'niderlandzki (Curaçao)',
        'nl_NL' => 'niderlandzki (Holandia)',
        'nl_SR' => 'niderlandzki (Surinam)',
        'nl_SX' => 'niderlandzki (Sint Maarten)',
        'nn' => 'norweski [nynorsk]',
        'nn_NO' => 'norweski [nynorsk] (Norwegia)',
        'no' => 'norweski',
        'no_NO' => 'norweski (Norwegia)',
        'oc' => 'oksytański',
        'oc_ES' => 'oksytański (Hiszpania)',
        'oc_FR' => 'oksytański (Francja)',
        'om' => 'oromo',
        'om_ET' => 'oromo (Etiopia)',
        'om_KE' => 'oromo (Kenia)',
        'or' => 'orija',
        'or_IN' => 'orija (Indie)',
        'os' => 'osetyjski',
        'os_GE' => 'osetyjski (Gruzja)',
        'os_RU' => 'osetyjski (Rosja)',
        'pa' => 'pendżabski',
        'pa_Arab' => 'pendżabski (arabskie)',
        'pa_Arab_PK' => 'pendżabski (arabskie, Pakistan)',
        'pa_Guru' => 'pendżabski (gurmukhi)',
        'pa_Guru_IN' => 'pendżabski (gurmukhi, Indie)',
        'pa_IN' => 'pendżabski (Indie)',
        'pa_PK' => 'pendżabski (Pakistan)',
        'pl' => 'polski',
        'pl_PL' => 'polski (Polska)',
        'ps' => 'paszto',
        'ps_AF' => 'paszto (Afganistan)',
        'ps_PK' => 'paszto (Pakistan)',
        'pt' => 'portugalski',
        'pt_AO' => 'portugalski (Angola)',
        'pt_BR' => 'portugalski (Brazylia)',
        'pt_CH' => 'portugalski (Szwajcaria)',
        'pt_CV' => 'portugalski (Republika Zielonego Przylądka)',
        'pt_GQ' => 'portugalski (Gwinea Równikowa)',
        'pt_GW' => 'portugalski (Gwinea Bissau)',
        'pt_LU' => 'portugalski (Luksemburg)',
        'pt_MO' => 'portugalski (SRA Makau [Chiny])',
        'pt_MZ' => 'portugalski (Mozambik)',
        'pt_PT' => 'portugalski (Portugalia)',
        'pt_ST' => 'portugalski (Wyspy Świętego Tomasza i Książęca)',
        'pt_TL' => 'portugalski (Timor Wschodni)',
        'qu' => 'keczua',
        'qu_BO' => 'keczua (Boliwia)',
        'qu_EC' => 'keczua (Ekwador)',
        'qu_PE' => 'keczua (Peru)',
        'rm' => 'retoromański',
        'rm_CH' => 'retoromański (Szwajcaria)',
        'rn' => 'rundi',
        'rn_BI' => 'rundi (Burundi)',
        'ro' => 'rumuński',
        'ro_MD' => 'rumuński (Mołdawia)',
        'ro_RO' => 'rumuński (Rumunia)',
        'ru' => 'rosyjski',
        'ru_BY' => 'rosyjski (Białoruś)',
        'ru_KG' => 'rosyjski (Kirgistan)',
        'ru_KZ' => 'rosyjski (Kazachstan)',
        'ru_MD' => 'rosyjski (Mołdawia)',
        'ru_RU' => 'rosyjski (Rosja)',
        'ru_UA' => 'rosyjski (Ukraina)',
        'rw' => 'kinya-ruanda',
        'rw_RW' => 'kinya-ruanda (Rwanda)',
        'sa' => 'sanskryt',
        'sa_IN' => 'sanskryt (Indie)',
        'sc' => 'sardyński',
        'sc_IT' => 'sardyński (Włochy)',
        'sd' => 'sindhi',
        'sd_Arab' => 'sindhi (arabskie)',
        'sd_Arab_PK' => 'sindhi (arabskie, Pakistan)',
        'sd_Deva' => 'sindhi (dewanagari)',
        'sd_Deva_IN' => 'sindhi (dewanagari, Indie)',
        'sd_IN' => 'sindhi (Indie)',
        'sd_PK' => 'sindhi (Pakistan)',
        'se' => 'północnolapoński',
        'se_FI' => 'północnolapoński (Finlandia)',
        'se_NO' => 'północnolapoński (Norwegia)',
        'se_SE' => 'północnolapoński (Szwecja)',
        'sg' => 'sango',
        'sg_CF' => 'sango (Republika Środkowoafrykańska)',
        'sh' => 'serbsko-chorwacki',
        'sh_BA' => 'serbsko-chorwacki (Bośnia i Hercegowina)',
        'si' => 'syngaleski',
        'si_LK' => 'syngaleski (Sri Lanka)',
        'sk' => 'słowacki',
        'sk_SK' => 'słowacki (Słowacja)',
        'sl' => 'słoweński',
        'sl_SI' => 'słoweński (Słowenia)',
        'sn' => 'shona',
        'sn_ZW' => 'shona (Zimbabwe)',
        'so' => 'somalijski',
        'so_DJ' => 'somalijski (Dżibuti)',
        'so_ET' => 'somalijski (Etiopia)',
        'so_KE' => 'somalijski (Kenia)',
        'so_SO' => 'somalijski (Somalia)',
        'sq' => 'albański',
        'sq_AL' => 'albański (Albania)',
        'sq_MK' => 'albański (Macedonia Północna)',
        'sr' => 'serbski',
        'sr_BA' => 'serbski (Bośnia i Hercegowina)',
        'sr_Cyrl' => 'serbski (cyrylica)',
        'sr_Cyrl_BA' => 'serbski (cyrylica, Bośnia i Hercegowina)',
        'sr_Cyrl_ME' => 'serbski (cyrylica, Czarnogóra)',
        'sr_Cyrl_RS' => 'serbski (cyrylica, Serbia)',
        'sr_Latn' => 'serbski (łacińskie)',
        'sr_Latn_BA' => 'serbski (łacińskie, Bośnia i Hercegowina)',
        'sr_Latn_ME' => 'serbski (łacińskie, Czarnogóra)',
        'sr_Latn_RS' => 'serbski (łacińskie, Serbia)',
        'sr_ME' => 'serbski (Czarnogóra)',
        'sr_RS' => 'serbski (Serbia)',
        'st' => 'sotho południowy',
        'st_LS' => 'sotho południowy (Lesotho)',
        'st_ZA' => 'sotho południowy (Republika Południowej Afryki)',
        'su' => 'sundajski',
        'su_ID' => 'sundajski (Indonezja)',
        'su_Latn' => 'sundajski (łacińskie)',
        'su_Latn_ID' => 'sundajski (łacińskie, Indonezja)',
        'sv' => 'szwedzki',
        'sv_AX' => 'szwedzki (Wyspy Alandzkie)',
        'sv_FI' => 'szwedzki (Finlandia)',
        'sv_SE' => 'szwedzki (Szwecja)',
        'sw' => 'suahili',
        'sw_CD' => 'suahili (Demokratyczna Republika Konga)',
        'sw_KE' => 'suahili (Kenia)',
        'sw_TZ' => 'suahili (Tanzania)',
        'sw_UG' => 'suahili (Uganda)',
        'ta' => 'tamilski',
        'ta_IN' => 'tamilski (Indie)',
        'ta_LK' => 'tamilski (Sri Lanka)',
        'ta_MY' => 'tamilski (Malezja)',
        'ta_SG' => 'tamilski (Singapur)',
        'te' => 'telugu',
        'te_IN' => 'telugu (Indie)',
        'tg' => 'tadżycki',
        'tg_TJ' => 'tadżycki (Tadżykistan)',
        'th' => 'tajski',
        'th_TH' => 'tajski (Tajlandia)',
        'ti' => 'tigrinia',
        'ti_ER' => 'tigrinia (Erytrea)',
        'ti_ET' => 'tigrinia (Etiopia)',
        'tk' => 'turkmeński',
        'tk_TM' => 'turkmeński (Turkmenistan)',
        'tl' => 'tagalski',
        'tl_PH' => 'tagalski (Filipiny)',
        'tn' => 'setswana',
        'tn_BW' => 'setswana (Botswana)',
        'tn_ZA' => 'setswana (Republika Południowej Afryki)',
        'to' => 'tonga',
        'to_TO' => 'tonga (Tonga)',
        'tr' => 'turecki',
        'tr_CY' => 'turecki (Cypr)',
        'tr_TR' => 'turecki (Turcja)',
        'tt' => 'tatarski',
        'tt_RU' => 'tatarski (Rosja)',
        'ug' => 'ujgurski',
        'ug_CN' => 'ujgurski (Chiny)',
        'uk' => 'ukraiński',
        'uk_UA' => 'ukraiński (Ukraina)',
        'ur' => 'urdu',
        'ur_IN' => 'urdu (Indie)',
        'ur_PK' => 'urdu (Pakistan)',
        'uz' => 'uzbecki',
        'uz_AF' => 'uzbecki (Afganistan)',
        'uz_Arab' => 'uzbecki (arabskie)',
        'uz_Arab_AF' => 'uzbecki (arabskie, Afganistan)',
        'uz_Cyrl' => 'uzbecki (cyrylica)',
        'uz_Cyrl_UZ' => 'uzbecki (cyrylica, Uzbekistan)',
        'uz_Latn' => 'uzbecki (łacińskie)',
        'uz_Latn_UZ' => 'uzbecki (łacińskie, Uzbekistan)',
        'uz_UZ' => 'uzbecki (Uzbekistan)',
        'vi' => 'wietnamski',
        'vi_VN' => 'wietnamski (Wietnam)',
        'wo' => 'wolof',
        'wo_SN' => 'wolof (Senegal)',
        'xh' => 'khosa',
        'xh_ZA' => 'khosa (Republika Południowej Afryki)',
        'yi' => 'jidysz',
        'yi_UA' => 'jidysz (Ukraina)',
        'yo' => 'joruba',
        'yo_BJ' => 'joruba (Benin)',
        'yo_NG' => 'joruba (Nigeria)',
        'za' => 'czuang',
        'za_CN' => 'czuang (Chiny)',
        'zh' => 'chiński',
        'zh_CN' => 'chiński (Chiny)',
        'zh_HK' => 'chiński (SRA Hongkong [Chiny])',
        'zh_Hans' => 'chiński (uproszczone)',
        'zh_Hans_CN' => 'chiński (uproszczone, Chiny)',
        'zh_Hans_HK' => 'chiński (uproszczone, SRA Hongkong [Chiny])',
        'zh_Hans_MO' => 'chiński (uproszczone, SRA Makau [Chiny])',
        'zh_Hans_MY' => 'chiński (uproszczone, Malezja)',
        'zh_Hans_SG' => 'chiński (uproszczone, Singapur)',
        'zh_Hant' => 'chiński (tradycyjne)',
        'zh_Hant_HK' => 'chiński (tradycyjne, SRA Hongkong [Chiny])',
        'zh_Hant_MO' => 'chiński (tradycyjne, SRA Makau [Chiny])',
        'zh_Hant_MY' => 'chiński (tradycyjne, Malezja)',
        'zh_Hant_TW' => 'chiński (tradycyjne, Tajwan)',
        'zh_MO' => 'chiński (SRA Makau [Chiny])',
        'zh_SG' => 'chiński (Singapur)',
        'zh_TW' => 'chiński (Tajwan)',
        'zu' => 'zulu',
        'zu_ZA' => 'zulu (Republika Południowej Afryki)',
    ],
];
