<?php

return [
    'Names' => [
        'aa' => 'afar',
        'ab' => 'abchaski',
        'ace' => 'aceh',
        'ach' => 'aczoli',
        'ada' => 'adangme',
        'ady' => 'adygejski',
        'ae' => 'awestyjski',
        'aeb' => 'tunezyjski arabski',
        'af' => 'afrikaans',
        'afh' => 'afrihili',
        'agq' => 'aghem',
        'ain' => 'ajnu',
        'ak' => 'akan',
        'akk' => 'akadyjski',
        'akz' => 'alabama',
        'ale' => 'aleucki',
        'aln' => 'albański gegijski',
        'alt' => 'południowoałtajski',
        'am' => 'amharski',
        'an' => 'aragoński',
        'ang' => 'staroangielski',
        'ann' => 'obolo',
        'anp' => 'angika',
        'ar' => 'arabski',
        'arc' => 'aramejski',
        'arn' => 'mapudungun',
        'aro' => 'araona',
        'arp' => 'arapaho',
        'arq' => 'algierski arabski',
        'ars' => 'arabski nadżdyjski',
        'arw' => 'arawak',
        'ary' => 'marokański arabski',
        'arz' => 'egipski arabski',
        'as' => 'asamski',
        'asa' => 'asu',
        'ase' => 'amerykański język migowy',
        'ast' => 'asturyjski',
        'atj' => 'atikamekw',
        'av' => 'awarski',
        'avk' => 'kotava',
        'awa' => 'awadhi',
        'ay' => 'ajmara',
        'az' => 'azerbejdżański',
        'ba' => 'baszkirski',
        'bal' => 'beludżi',
        'ban' => 'balijski',
        'bar' => 'bawarski',
        'bas' => 'basaa',
        'bax' => 'bamum',
        'bbc' => 'batak toba',
        'bbj' => 'ghomala',
        'be' => 'białoruski',
        'bej' => 'bedża',
        'bem' => 'bemba',
        'bew' => 'betawi',
        'bez' => 'bena',
        'bfd' => 'bafut',
        'bfq' => 'badaga',
        'bg' => 'bułgarski',
        'bgc' => 'haryanvi',
        'bgn' => 'beludżi północny',
        'bho' => 'bhodżpuri',
        'bi' => 'bislama',
        'bik' => 'bikol',
        'bin' => 'bini',
        'bjn' => 'banjar',
        'bkm' => 'kom',
        'bla' => 'siksika',
        'blo' => 'anii',
        'bm' => 'bambara',
        'bn' => 'bengalski',
        'bo' => 'tybetański',
        'bpy' => 'bisznuprija-manipuri',
        'bqi' => 'bachtiarski',
        'br' => 'bretoński',
        'bra' => 'bradź',
        'brh' => 'brahui',
        'brx' => 'bodo',
        'bs' => 'bośniacki',
        'bss' => 'akoose',
        'bua' => 'buriacki',
        'bug' => 'bugijski',
        'bum' => 'bulu',
        'byn' => 'blin',
        'byv' => 'medumba',
        'ca' => 'kataloński',
        'cad' => 'kaddo',
        'car' => 'karaibski',
        'cay' => 'kajuga',
        'cch' => 'atsam',
        'ccp' => 'czakma',
        'ce' => 'czeczeński',
        'ceb' => 'cebuański',
        'cgg' => 'chiga',
        'ch' => 'czamorro',
        'chb' => 'czibcza',
        'chg' => 'czagatajski',
        'chk' => 'chuuk',
        'chm' => 'maryjski',
        'chn' => 'żargon czinucki',
        'cho' => 'czoktawski',
        'chp' => 'czipewiański',
        'chr' => 'czirokeski',
        'chy' => 'czejeński',
        'ckb' => 'sorani',
        'clc' => 'chilcotin',
        'co' => 'korsykański',
        'cop' => 'koptyjski',
        'cps' => 'capiznon',
        'cr' => 'kri',
        'crg' => 'miszif',
        'crh' => 'krymskotatarski',
        'crj' => 'kri południowo-wschodni',
        'crk' => 'kri równinny',
        'crl' => 'kri północno-wschodni',
        'crm' => 'kri Moose',
        'crr' => 'algonkiński (Karolina Północna)',
        'crs' => 'kreolski seszelski',
        'cs' => 'czeski',
        'csb' => 'kaszubski',
        'csw' => 'kri bagienny',
        'cu' => 'cerkiewnosłowiański',
        'cv' => 'czuwaski',
        'cy' => 'walijski',
        'da' => 'duński',
        'dak' => 'dakota',
        'dar' => 'dargwijski',
        'dav' => 'taita',
        'de' => 'niemiecki',
        'del' => 'delaware',
        'den' => 'slave',
        'dgr' => 'dogrib',
        'din' => 'dinka',
        'dje' => 'dżerma',
        'doi' => 'dogri',
        'dsb' => 'dolnołużycki',
        'dtp' => 'dusun centralny',
        'dua' => 'duala',
        'dum' => 'średniowieczny niderlandzki',
        'dv' => 'malediwski',
        'dyo' => 'diola',
        'dyu' => 'diula',
        'dz' => 'dzongkha',
        'dzg' => 'dazaga',
        'ebu' => 'embu',
        'ee' => 'ewe',
        'efi' => 'efik',
        'egl' => 'emilijski',
        'egy' => 'staroegipski',
        'eka' => 'ekajuk',
        'el' => 'grecki',
        'elx' => 'elamicki',
        'en' => 'angielski',
        'enm' => 'średnioangielski',
        'eo' => 'esperanto',
        'es' => 'hiszpański',
        'esu' => 'yupik środkowosyberyjski',
        'et' => 'estoński',
        'eu' => 'baskijski',
        'ewo' => 'ewondo',
        'ext' => 'estremadurski',
        'fa' => 'perski',
        'fan' => 'fang',
        'fat' => 'fanti',
        'ff' => 'fulani',
        'fi' => 'fiński',
        'fil' => 'filipiński',
        'fit' => 'meänkieli',
        'fj' => 'fidżijski',
        'fo' => 'farerski',
        'fon' => 'fon',
        'fr' => 'francuski',
        'frc' => 'cajuński',
        'frm' => 'średniofrancuski',
        'fro' => 'starofrancuski',
        'frp' => 'franko-prowansalski',
        'frr' => 'północnofryzyjski',
        'frs' => 'wschodniofryzyjski',
        'fur' => 'friulski',
        'fy' => 'zachodniofryzyjski',
        'ga' => 'irlandzki',
        'gaa' => 'ga',
        'gag' => 'gagauski',
        'gan' => 'gan',
        'gay' => 'gayo',
        'gba' => 'gbaya',
        'gbz' => 'zaratusztriański dari',
        'gd' => 'szkocki gaelicki',
        'gez' => 'gyyz',
        'gil' => 'gilbertański',
        'gl' => 'galicyjski',
        'glk' => 'giliański',
        'gmh' => 'średnio-wysoko-niemiecki',
        'gn' => 'guarani',
        'goh' => 'staro-wysoko-niemiecki',
        'gon' => 'gondi',
        'gor' => 'gorontalo',
        'got' => 'gocki',
        'grb' => 'grebo',
        'grc' => 'starogrecki',
        'gsw' => 'szwajcarski niemiecki',
        'gu' => 'gudżarati',
        'guc' => 'wayúu',
        'gur' => 'frafra',
        'guz' => 'gusii',
        'gv' => 'manx',
        'gwi' => 'gwichʼin',
        'ha' => 'hausa',
        'hai' => 'haida',
        'hak' => 'hakka',
        'haw' => 'hawajski',
        'hax' => 'haida południowy',
        'he' => 'hebrajski',
        'hi' => 'hindi',
        'hif' => 'hindi fidżyjskie',
        'hil' => 'hiligaynon',
        'hit' => 'hetycki',
        'hmn' => 'hmong',
        'ho' => 'hiri motu',
        'hr' => 'chorwacki',
        'hsb' => 'górnołużycki',
        'hsn' => 'xiang',
        'ht' => 'kreolski haitański',
        'hu' => 'węgierski',
        'hup' => 'hupa',
        'hur' => 'halkomelem',
        'hy' => 'ormiański',
        'hz' => 'herero',
        'ia' => 'interlingua',
        'iba' => 'iban',
        'ibb' => 'ibibio',
        'id' => 'indonezyjski',
        'ie' => 'interlingue',
        'ig' => 'igbo',
        'ii' => 'syczuański',
        'ik' => 'inupiak',
        'ikt' => 'inuktitut zachodniokanadyjski',
        'ilo' => 'ilokano',
        'inh' => 'inguski',
        'io' => 'ido',
        'is' => 'islandzki',
        'it' => 'włoski',
        'iu' => 'inuktitut',
        'izh' => 'ingryjski',
        'ja' => 'japoński',
        'jam' => 'jamajski',
        'jbo' => 'lojban',
        'jgo' => 'ngombe',
        'jmc' => 'machame',
        'jpr' => 'judeo-perski',
        'jrb' => 'judeoarabski',
        'jut' => 'jutlandzki',
        'jv' => 'jawajski',
        'ka' => 'gruziński',
        'kaa' => 'karakałpacki',
        'kab' => 'kabylski',
        'kac' => 'kaczin',
        'kaj' => 'jju',
        'kam' => 'kamba',
        'kaw' => 'kawi',
        'kbd' => 'kabardyjski',
        'kbl' => 'kanembu',
        'kcg' => 'tyap',
        'kde' => 'makonde',
        'kea' => 'kreolski Wysp Zielonego Przylądka',
        'ken' => 'kenyang',
        'kfo' => 'koro',
        'kg' => 'kongo',
        'kgp' => 'kaingang',
        'kha' => 'khasi',
        'kho' => 'chotański',
        'khq' => 'koyra chiini',
        'khw' => 'khowar',
        'ki' => 'kikuju',
        'kiu' => 'kirmandżki',
        'kj' => 'kwanyama',
        'kk' => 'kazachski',
        'kkj' => 'kako',
        'kl' => 'grenlandzki',
        'kln' => 'kalenjin',
        'km' => 'khmerski',
        'kmb' => 'kimbundu',
        'kn' => 'kannada',
        'ko' => 'koreański',
        'koi' => 'komi-permiacki',
        'kok' => 'konkani',
        'kos' => 'kosrae',
        'kpe' => 'kpelle',
        'kr' => 'kanuri',
        'krc' => 'karaczajsko-bałkarski',
        'kri' => 'krio',
        'krj' => 'kinaraya',
        'krl' => 'karelski',
        'kru' => 'kurukh',
        'ks' => 'kaszmirski',
        'ksb' => 'sambala',
        'ksf' => 'bafia',
        'ksh' => 'gwara kolońska',
        'ku' => 'kurdyjski',
        'kum' => 'kumycki',
        'kut' => 'kutenai',
        'kv' => 'komi',
        'kw' => 'kornijski',
        'kwk' => 'kwakiutl',
        'kxv' => 'kuvi',
        'ky' => 'kirgiski',
        'la' => 'łaciński',
        'lad' => 'ladyński',
        'lag' => 'langi',
        'lah' => 'lahnda',
        'lam' => 'lamba',
        'lb' => 'luksemburski',
        'lez' => 'lezgijski',
        'lfn' => 'Lingua Franca Nova',
        'lg' => 'ganda',
        'li' => 'limburski',
        'lij' => 'liguryjski',
        'lil' => 'lillooet',
        'liv' => 'liwski',
        'lkt' => 'lakota',
        'lmo' => 'lombardzki',
        'ln' => 'lingala',
        'lo' => 'laotański',
        'lol' => 'mongo',
        'lou' => 'kreolski luizjański',
        'loz' => 'lozi',
        'lrc' => 'luryjski północny',
        'lsm' => 'saamia',
        'lt' => 'litewski',
        'ltg' => 'łatgalski',
        'lu' => 'luba-katanga',
        'lua' => 'luba-lulua',
        'lui' => 'luiseno',
        'lun' => 'lunda',
        'lus' => 'mizo',
        'luy' => 'luhya',
        'lv' => 'łotewski',
        'lzh' => 'chiński klasyczny',
        'lzz' => 'lazyjski',
        'mad' => 'madurski',
        'maf' => 'mafa',
        'mag' => 'magahi',
        'mai' => 'maithili',
        'mak' => 'makasar',
        'man' => 'mandingo',
        'mas' => 'masajski',
        'mde' => 'maba',
        'mdf' => 'moksza',
        'mdr' => 'mandar',
        'men' => 'mende',
        'mer' => 'meru',
        'mfe' => 'kreolski Mauritiusa',
        'mg' => 'malgaski',
        'mga' => 'średnioirlandzki',
        'mgh' => 'makua-meetto',
        'mgo' => 'meta',
        'mh' => 'marszalski',
        'mi' => 'maoryjski',
        'mic' => 'mikmak',
        'min' => 'minangkabu',
        'mk' => 'macedoński',
        'ml' => 'malajalam',
        'mn' => 'mongolski',
        'mnc' => 'manchu',
        'mni' => 'manipuri',
        'moe' => 'innu-aimun',
        'moh' => 'mohawk',
        'mos' => 'mossi',
        'mr' => 'marathi',
        'mrj' => 'zachodniomaryjski',
        'ms' => 'malajski',
        'mt' => 'maltański',
        'mua' => 'mundang',
        'mus' => 'krik',
        'mwl' => 'mirandyjski',
        'mwr' => 'marwari',
        'mwv' => 'mentawai',
        'my' => 'birmański',
        'mye' => 'myene',
        'myv' => 'erzja',
        'mzn' => 'mazanderański',
        'na' => 'nauruański',
        'nan' => 'minnański',
        'nap' => 'neapolitański',
        'naq' => 'nama',
        'nb' => 'norweski (bokmål)',
        'nd' => 'ndebele północny',
        'nds' => 'dolnoniemiecki',
        'ne' => 'nepalski',
        'new' => 'newarski',
        'ng' => 'ndonga',
        'nia' => 'nias',
        'niu' => 'niue',
        'njo' => 'ao',
        'nl' => 'niderlandzki',
        'nmg' => 'ngumba',
        'nn' => 'norweski (nynorsk)',
        'nnh' => 'ngiemboon',
        'no' => 'norweski',
        'nog' => 'nogajski',
        'non' => 'staronordyjski',
        'nov' => 'novial',
        'nqo' => 'n’ko',
        'nr' => 'ndebele południowy',
        'nso' => 'sotho północny',
        'nus' => 'nuer',
        'nv' => 'nawaho',
        'nwc' => 'newarski klasyczny',
        'ny' => 'njandża',
        'nym' => 'niamwezi',
        'nyn' => 'nyankole',
        'nyo' => 'nyoro',
        'nzi' => 'nzema',
        'oc' => 'oksytański',
        'oj' => 'odżibwa',
        'ojb' => 'odżibwe północno-zachodni',
        'ojc' => 'odżibwe centralny',
        'ojs' => 'odżi-kri',
        'ojw' => 'odżibwe zachodni',
        'oka' => 'okanagan',
        'om' => 'oromo',
        'or' => 'orija',
        'os' => 'osetyjski',
        'osa' => 'osage',
        'ota' => 'osmańsko-turecki',
        'pa' => 'pendżabski',
        'pag' => 'pangasinan',
        'pal' => 'pahlavi',
        'pam' => 'pampango',
        'pap' => 'papiamento',
        'pau' => 'palau',
        'pcd' => 'pikardyjski',
        'pcm' => 'pidżyn nigeryjski',
        'pdc' => 'pensylwański',
        'pdt' => 'plautdietsch',
        'peo' => 'staroperski',
        'pfl' => 'palatynacki',
        'phn' => 'fenicki',
        'pi' => 'palijski',
        'pis' => 'pijin',
        'pl' => 'polski',
        'pms' => 'piemoncki',
        'pnt' => 'pontyjski',
        'pon' => 'ponpejski',
        'pqm' => 'malecite-passamaquoddy',
        'prg' => 'pruski',
        'pro' => 'staroprowansalski',
        'ps' => 'paszto',
        'pt' => 'portugalski',
        'qu' => 'keczua',
        'quc' => 'kicze',
        'qug' => 'keczua górski (Chimborazo)',
        'raj' => 'radźasthani',
        'rap' => 'rapanui',
        'rar' => 'rarotonga',
        'rgn' => 'romagnol',
        'rhg' => 'rohingya',
        'rif' => 'tarifit',
        'rm' => 'retoromański',
        'rn' => 'rundi',
        'ro' => 'rumuński',
        'rof' => 'rombo',
        'rom' => 'cygański',
        'rtm' => 'rotumański',
        'ru' => 'rosyjski',
        'rue' => 'rusiński',
        'rug' => 'roviana',
        'rup' => 'arumuński',
        'rw' => 'kinya-ruanda',
        'rwk' => 'rwa',
        'sa' => 'sanskryt',
        'sad' => 'sandawe',
        'sah' => 'jakucki',
        'sam' => 'samarytański aramejski',
        'saq' => 'samburu',
        'sas' => 'sasak',
        'sat' => 'santali',
        'saz' => 'saurasztryjski',
        'sba' => 'ngambay',
        'sbp' => 'sangu',
        'sc' => 'sardyński',
        'scn' => 'sycylijski',
        'sco' => 'scots',
        'sd' => 'sindhi',
        'sdc' => 'sassarski',
        'sdh' => 'południowokurdyjski',
        'se' => 'północnolapoński',
        'see' => 'seneka',
        'seh' => 'sena',
        'sei' => 'seri',
        'sel' => 'selkupski',
        'ses' => 'koyraboro senni',
        'sg' => 'sango',
        'sga' => 'staroirlandzki',
        'sgs' => 'żmudzki',
        'sh' => 'serbsko-chorwacki',
        'shi' => 'tashelhiyt',
        'shn' => 'szan',
        'shu' => 'arabski (Czad)',
        'si' => 'syngaleski',
        'sid' => 'sidamo',
        'sk' => 'słowacki',
        'sl' => 'słoweński',
        'slh' => 'lushootseed południowy',
        'sli' => 'dolnośląski',
        'sly' => 'selayar',
        'sm' => 'samoański',
        'sma' => 'południowolapoński',
        'smj' => 'lule',
        'smn' => 'inari',
        'sms' => 'skolt',
        'sn' => 'shona',
        'snk' => 'soninke',
        'so' => 'somalijski',
        'sog' => 'sogdyjski',
        'sq' => 'albański',
        'sr' => 'serbski',
        'srn' => 'sranan tongo',
        'srr' => 'serer',
        'ss' => 'suazi',
        'ssy' => 'saho',
        'st' => 'sotho południowy',
        'stq' => 'fryzyjski saterlandzki',
        'str' => 'salisz',
        'su' => 'sundajski',
        'suk' => 'sukuma',
        'sus' => 'susu',
        'sux' => 'sumeryjski',
        'sv' => 'szwedzki',
        'sw' => 'suahili',
        'swb' => 'komoryjski',
        'syc' => 'syriacki',
        'syr' => 'syryjski',
        'szl' => 'śląski',
        'ta' => 'tamilski',
        'tce' => 'tutchone południowy',
        'tcy' => 'tulu',
        'te' => 'telugu',
        'tem' => 'temne',
        'teo' => 'ateso',
        'ter' => 'tereno',
        'tet' => 'tetum',
        'tg' => 'tadżycki',
        'tgx' => 'tagish',
        'th' => 'tajski',
        'tht' => 'tahltan',
        'ti' => 'tigrinia',
        'tig' => 'tigre',
        'tiv' => 'tiw',
        'tk' => 'turkmeński',
        'tkl' => 'tokelau',
        'tkr' => 'cachurski',
        'tl' => 'tagalski',
        'tlh' => 'klingoński',
        'tli' => 'tlingit',
        'tly' => 'tałyski',
        'tmh' => 'tamaszek',
        'tn' => 'setswana',
        'to' => 'tonga',
        'tog' => 'tonga (Niasa)',
        'tok' => 'toki pona',
        'tpi' => 'tok pisin',
        'tr' => 'turecki',
        'tru' => 'turoyo',
        'trv' => 'taroko',
        'ts' => 'tsonga',
        'tsd' => 'cakoński',
        'tsi' => 'tsimshian',
        'tt' => 'tatarski',
        'ttm' => 'tutchone północny',
        'ttt' => 'tacki',
        'tum' => 'tumbuka',
        'tvl' => 'tuvalu',
        'tw' => 'twi',
        'twq' => 'tasawaq',
        'ty' => 'tahitański',
        'tyv' => 'tuwiński',
        'tzm' => 'tamazight (Atlas Środkowy)',
        'udm' => 'udmurcki',
        'ug' => 'ujgurski',
        'uga' => 'ugarycki',
        'uk' => 'ukraiński',
        'umb' => 'umbundu',
        'ur' => 'urdu',
        'uz' => 'uzbecki',
        'vai' => 'wai',
        've' => 'venda',
        'vec' => 'wenecki',
        'vep' => 'wepski',
        'vi' => 'wietnamski',
        'vls' => 'zachodnioflamandzki',
        'vmf' => 'meński frankoński',
        'vmw' => 'makua',
        'vo' => 'wolapik',
        'vot' => 'wotiacki',
        'vro' => 'võro',
        'vun' => 'vunjo',
        'wa' => 'waloński',
        'wae' => 'walser',
        'wal' => 'wolayta',
        'war' => 'waraj',
        'was' => 'washo',
        'wbp' => 'warlpiri',
        'wo' => 'wolof',
        'wuu' => 'wu',
        'xal' => 'kałmucki',
        'xh' => 'khosa',
        'xmf' => 'megrelski',
        'xnr' => 'kangri',
        'xog' => 'soga',
        'yao' => 'yao',
        'yap' => 'japski',
        'yav' => 'yangben',
        'ybb' => 'yemba',
        'yi' => 'jidysz',
        'yo' => 'joruba',
        'yrl' => 'nheengatu',
        'yue' => 'kantoński',
        'za' => 'czuang',
        'zap' => 'zapotecki',
        'zbl' => 'bliss',
        'zea' => 'zelandzki',
        'zen' => 'zenaga',
        'zgh' => 'standardowy marokański tamazight',
        'zh' => 'chiński',
        'zu' => 'zulu',
        'zun' => 'zuni',
        'zza' => 'zazaki',
    ],
    'LocalizedNames' => [
        'ar_001' => 'współczesny arabski',
        'de_AT' => 'niemiecki austriacki',
        'de_CH' => 'wysokoniemiecki szwajcarski',
        'en_AU' => 'angielski australijski',
        'en_CA' => 'angielski kanadyjski',
        'en_GB' => 'angielski brytyjski',
        'en_US' => 'angielski amerykański',
        'es_419' => 'amerykański hiszpański',
        'es_ES' => 'europejski hiszpański',
        'es_MX' => 'meksykański hiszpański',
        'fa_AF' => 'dari',
        'fr_CA' => 'francuski kanadyjski',
        'fr_CH' => 'francuski szwajcarski',
        'hi_Latn' => 'hindi (alfabet łaciński)',
        'nds_NL' => 'dolnosaksoński',
        'nl_BE' => 'flamandzki',
        'pt_BR' => 'brazylijski portugalski',
        'pt_PT' => 'europejski portugalski',
        'ro_MD' => 'mołdawski',
        'sw_CD' => 'kongijski suahili',
        'zh_Hans' => 'chiński uproszczony',
        'zh_Hant' => 'chiński tradycyjny',
    ],
];
