<?php

return [
    'Names' => [
        'ADP' => [
            'ADP',
            'andorranske peseta',
        ],
        'AFA' => [
            'AFA',
            'afghanske afghani (1927–2002)',
        ],
        'AOK' => [
            'AOK',
            'angolske kwanza (1977–1990)',
        ],
        'AON' => [
            'AON',
            'angolske nye kwanza (1990–2000)',
        ],
        'AOR' => [
            'AOR',
            'angolske kwanza reajustado (1995–1999)',
        ],
        'ARA' => [
            'ARA',
            'argentiske austral',
        ],
        'ARP' => [
            'ARP',
            'argentinske peso (1983–1985)',
        ],
        'ATS' => [
            'ATS',
            'austerrikske schilling',
        ],
        'AWG' => [
            'AWG',
            'arubiske florinar',
        ],
        'BAD' => [
            'BAD',
            'bosnisk-hercegovinske dinarar (1992–1994)',
        ],
        'BAN' => [
            'BAN',
            'nye bosnisk-hercegovinske dinarar (1994–1997)',
        ],
        'BEC' => [
            'BEC',
            'belgiske franc (konvertibel)',
        ],
        'BEL' => [
            'BEL',
            'belgiske franc (finansiell)',
        ],
        'BGL' => [
            'BGL',
            'bulgarsk hard lev',
        ],
        'BHD' => [
            'BHD',
            'bahrainske dinarar',
        ],
        'BOP' => [
            'BOP',
            'boliviske peso',
        ],
        'BOV' => [
            'BOV',
            'boliviske mvdol',
        ],
        'BRB' => [
            'BRB',
            'brasiliansk cruzeiro novo (1967–1986)',
        ],
        'BRC' => [
            'BRC',
            'brasilianske cruzado',
        ],
        'BRN' => [
            'BRN',
            'brasilianske cruzado novo',
        ],
        'BRR' => [
            'BRR',
            'brasilianske cruzeiro',
        ],
        'BUK' => [
            'BUK',
            'burmesisk kyat',
        ],
        'BYB' => [
            'BYB',
            'belarusiske nye rublar (1994–1999)',
        ],
        'BYN' => [
            'BYN',
            'nye belarusiske rublar',
        ],
        'BYR' => [
            'BYR',
            'belarusiske rublar (2000–2016)',
        ],
        'CHE' => [
            'CHE',
            'WIR-euro',
        ],
        'CHW' => [
            'CHW',
            'WIR-franc',
        ],
        'COP' => [
            'COP',
            'kolombianske pesos',
        ],
        'COU' => [
            'COU',
            'unidad de valor real',
        ],
        'CRC' => [
            'CRC',
            'kostarikanske colón',
        ],
        'CSD' => [
            'CSD',
            'gamle serbiske dinarer',
        ],
        'CVE' => [
            'CVE',
            'kappverdiske escudo',
        ],
        'DDM' => [
            'DDM',
            'austtyske mark',
        ],
        'DZD' => [
            'DZD',
            'algeriske dinarar',
        ],
        'GBP' => [
            'GBP',
            'britiske pund',
        ],
        'GHC' => [
            'GHC',
            'ghanesiske cedi (1979–2007)',
        ],
        'GRD' => [
            'GRD',
            'greske drakme',
        ],
        'GWP' => [
            'GWP',
            'Guinea-Bissau-peso',
        ],
        'HRD' => [
            'HRD',
            'kroatiske dinar',
        ],
        'HUF' => [
            'HUF',
            'ungarske forintar',
        ],
        'IDR' => [
            'IDR',
            'indonesiske rupiahar',
        ],
        'ILS' => [
            'ILS',
            'nye israelske sheklar',
        ],
        'INR' => [
            'INR',
            'indiske rupiar',
        ],
        'IQD' => [
            'IQD',
            'irakiske dinarar',
        ],
        'IRR' => [
            'IRR',
            'iranske rial',
        ],
        'JOD' => [
            'JOD',
            'jordanske dinarar',
        ],
        'KWD' => [
            'KWD',
            'kuwaitiske dinarar',
        ],
        'LKR' => [
            'LKR',
            'srilankiske rupiar',
        ],
        'LTL' => [
            'LTL',
            'litauiske lita',
        ],
        'LTT' => [
            'LTT',
            'litauiske talona',
        ],
        'LUC' => [
            'LUC',
            'luxemburgske konvertibel franc',
        ],
        'LVL' => [
            'LVL',
            'latviske lat',
        ],
        'LVR' => [
            'LVR',
            'latviske rublar',
        ],
        'LYD' => [
            'LYD',
            'libyske dinarar',
        ],
        'MDL' => [
            'MDL',
            'moldovske leuar',
        ],
        'MKD' => [
            'MKD',
            'makedonske denarar',
        ],
        'MKN' => [
            'MKN',
            'makedonske denarar (1992–1993)',
        ],
        'MUR' => [
            'MUR',
            'mauritiske rupiar',
        ],
        'MVP' => [
            'MVP',
            'maldiviske rupiar',
        ],
        'MXP' => [
            'MXP',
            'meksikanske sølvpeso (1861–1992)',
        ],
        'NIC' => [
            'NIC',
            'nicaraguanske cordoba',
        ],
        'NPR' => [
            'NPR',
            'nepalske rupiar',
        ],
        'NZD' => [
            'NZD',
            'nyzealandske dollar',
        ],
        'OMR' => [
            'OMR',
            'omanske rial',
        ],
        'PKR' => [
            'PKR',
            'pakistanske rupiar',
        ],
        'PYG' => [
            'PYG',
            'paraguayanske guaraní',
        ],
        'QAR' => [
            'QAR',
            'qatarske rial',
        ],
        'ROL' => [
            'ROL',
            'gamle rumenske leu',
        ],
        'RON' => [
            'RON',
            'rumenske leuar',
        ],
        'RSD' => [
            'RSD',
            'serbiske dinarar',
        ],
        'RUB' => [
            'RUB',
            'russiske rublar',
        ],
        'RUR' => [
            'RUR',
            'russiske rublar (1991–1998)',
        ],
        'SAR' => [
            'SAR',
            'saudiarabiske rial',
        ],
        'SCR' => [
            'SCR',
            'seychelliske rupiar',
        ],
        'SDD' => [
            'SDD',
            'gamle sudanske dinarer',
        ],
        'SDP' => [
            'SDP',
            'gamle sudanske pund',
        ],
        'SLL' => [
            'SLL',
            'sierraleonsk leone (1964—2022)',
        ],
        'SUR' => [
            'SUR',
            'sovjetiske rublar',
        ],
        'SZL' => [
            'SZL',
            'eswatinisk lilangeni',
        ],
        'TJR' => [
            'TJR',
            'tadsjikiske rublar',
        ],
        'TND' => [
            'TND',
            'tunisiske dinarar',
        ],
        'TRL' => [
            'TRL',
            'gamle tyrkiske lire',
        ],
        'TRY' => [
            'TRY',
            'tyrkiske lira',
        ],
        'USS' => [
            'USS',
            'amerikanske dollar (same dag)',
        ],
        'UYI' => [
            'UYI',
            'uruguayanske peso en unidades indexadas',
        ],
        'UYP' => [
            'UYP',
            'uruguayanske peso (1975–1993)',
        ],
        'UZS' => [
            'UZS',
            'usbekiske sum',
        ],
        'VUV' => [
            'VUV',
            'vanuatuiske vatu',
        ],
        'XCD' => [
            'XCD',
            'austkaribiske dollar',
        ],
        'XEU' => [
            'XEU',
            'europeiske valutaeiningar',
        ],
        'YDD' => [
            'YDD',
            'jemenittiske dinarar',
        ],
        'YER' => [
            'YER',
            'jemenittiske rial',
        ],
        'YUD' => [
            'YUD',
            'jugoslaviske dinarar (hard)',
        ],
        'YUM' => [
            'YUM',
            'jugoslaviske noviy-dinarar',
        ],
        'YUN' => [
            'YUN',
            'jugoslaviske konvertibel dinarar',
        ],
        'ZAL' => [
            'ZAL',
            'sørafrikanske rand (finansiell)',
        ],
        'ZRN' => [
            'ZRN',
            'zairisk ny zaire',
        ],
        'ZRZ' => [
            'ZRZ',
            'zairisk zaire',
        ],
        'ZWD' => [
            'ZWD',
            'zimbabwisk dollar',
        ],
    ],
];
