<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\TestCase;

/**
 * @covers \DateTime
 */
#[CoversClass(DateTime::class)]
class CoversTest extends TestCase
{
    public function test()
    {
        $this->assertTrue(true);
    }
}
