<?php

namespace Symfony\Component\PropertyInfo\Tests\Fixtures\TraitUsage\AnotherNamespace;

trait DummyTraitInAnotherNamespace
{
    /**
     * @var DummyInAnotherNamespace
     */
    public $dummyInAnotherNamespace;
}
