<?php

$container->setParameter('a_string', 'a string');
$container->setParameter('foo', 'bar');
$container->setParameter('values', []);
$container->setParameter('mixedcase', []);
$container->setParameter('constant', PHP_EOL);
