<?php

return [
    'Names' => [
        'Adlm' => 'Èdè Adam',
        'Arab' => 'èdè Lárúbáwá',
        'Aran' => 'Èdè Aran',
        'Armn' => 'Àmẹ́níà',
        'Bamu' => 'Bamumu',
        'Batk' => 'Bataki',
        'Beng' => 'Báńgílà',
        'Bopo' => 'Bopomófò',
        'Brai' => 'Bíráìlè',
        'Cakm' => 'Kami',
        'Cans' => 'Èdè Apapo Onile Onisilebu ti Kanada',
        'Cher' => 'Èdè Sheroki',
        'Cyrl' => 'èdè ilẹ̀ Rọ́ṣíà',
        'Deva' => 'Dẹfanagárì',
        'Ethi' => 'Ẹtiópíìkì',
        'Geor' => 'Jọ́jíànù',
        'Gong' => 'Gunjala Gondi',
        'Grek' => 'Gíríkì',
        'Gujr' => 'Gujaráti',
        'Guru' => 'Gurumúkhì',
        'Hanb' => 'Han pẹ̀lú Bopomófò',
        'Hang' => 'Háńgùlù',
        'Hani' => 'Háànù',
        'Hans' => 'tí wọ́n mú rọrùn.',
        'Hant' => 'Àbáláyé',
        'Hebr' => 'Hébérù',
        'Hira' => 'Hiragánà',
        'Hmnp' => 'Nyiakengi Puase Himongi',
        'Hrkt' => 'ìlànà àfọwọ́kọ ará Jàpánù',
        'Java' => 'Èdè Jafaniisi',
        'Jpan' => 'èdè jàpáànù',
        'Kali' => 'Èdè Kaya Li',
        'Kana' => 'Katakánà',
        'Khmr' => 'Kẹmẹ̀',
        'Knda' => 'Kanada',
        'Kore' => 'Kóríà',
        'Lana' => 'Èdè Lana',
        'Laoo' => 'Láò',
        'Latn' => 'Èdè Látìn',
        'Lepc' => 'Èdè Lepika',
        'Limb' => 'Èdè Limbu',
        'Lisu' => 'Furasa',
        'Mand' => 'Èdè Mandaiani',
        'Mlym' => 'Málàyálámù',
        'Mong' => 'Èdè Mòngólíà',
        'Mtei' => 'Èdè Meitei Mayeki',
        'Mymr' => 'Myánmarà',
        'Newa' => 'Èdè Newa',
        'Nkoo' => 'Èdè Nkoo',
        'Olck' => 'Èdè Ol Siki',
        'Orya' => 'Òdíà',
        'Osge' => 'Èdè Osage',
        'Plrd' => 'Fonẹtiiki Polaadi',
        'Rohg' => 'Èdè Hanifi',
        'Saur' => 'Èdè Saurasitira',
        'Sinh' => 'Sìnhálà',
        'Sund' => 'Èdè Sundani',
        'Syrc' => 'Èdè Siriaki',
        'Taml' => 'Támílì',
        'Telu' => 'Télúgù',
        'Tfng' => 'Èdè Tifina',
        'Thaa' => 'Taana',
        'Tibt' => 'Tíbétán',
        'Vaii' => 'Èdè Fai',
        'Yiii' => 'Èdè Yi',
        'Zinh' => 'Tí a jogún',
        'Zmth' => 'Àmì Ìṣèsìrò',
        'Zsye' => 'Émójì',
        'Zsym' => 'Àwọn àmì',
        'Zxxx' => 'Aikọsilẹ',
        'Zyyy' => 'Wọ́pọ̀',
    ],
];
