<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\Webhook\Exception;

use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * @author Fabien Potencier <fabien@symfony.com>
 */
class RejectWebhookException extends HttpException
{
    public function __construct(int $statusCode = 406, string $message = '', ?\Throwable $previous = null, array $headers = [], int $code = 0)
    {
        parent::__construct($statusCode, $message, $previous, $headers, $code);
    }
}
