<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\Serializer\Tests\Fixtures\Attributes;

class AccessorishGetters
{
    public function hash(): void
    {
    }

    public function cancel()
    {
    }

    public function getField1()
    {
    }

    public function isField2()
    {
    }

    public function hasField3()
    {
    }

    public function setField4($value)
    {
    }
}
