<?php

return [
    'Names' => [
        'ha' => 'Hausa',
        'ha_GH' => 'Hausa (Ghana)',
        'ha_NE' => 'Hausa (Niger)',
        'ha_NG' => 'Hausa (Nigeria)',
        'nb_SJ' => 'Norwegisch [Bokmål] (Svalbard und Jan Mayen)',
        'sh' => 'Serbokroatisch',
        'sh_BA' => 'Serbokroatisch (Bosnien und Herzegowina)',
    ],
];
