<?php

return [
    '😶‍🌫' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤣𞤫𞤪 𞤣𞤵𞥅𞤤𞤫',
    '😮‍💨' => '𞤴𞤫𞥅𞤧𞤮 𞤣𞤵𞤨𞥆𞤢𞤴𞤲𞤺𞤮',
    '🙂‍↕' => '𞤸𞤢𞤴𞤤𞤢 𞤸𞤮𞤸𞤪𞤫 𞤲𞤮 𞤻𞤭𞤱𞤤𞤭𞤪𞤭',
    '🐕‍🦺' => '𞤪𞤢𞤱𞤢𞥄𞤲𞤣𞤵 𞤸𞤵𞥅𞤱𞤢𞤴𞤲𞤣𞤵',
    '🇦🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤀𞤧𞤢𞤲𞤧𞤮𞥅𞤲',
    '🇦🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤲𞤣𞤮𞤪𞤢𞥄',
    '🇦🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤁𞤫𞤲𞤼𞤢𞤤 𞤋𞤥𞤪𞤢𞥄𞤼𞤭 𞤀𞥄𞤪𞤢𞤦𞤵',
    '🇦🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤬𞤺𞤢𞤲𞤭𞤧𞤼𞤢𞥄𞤲',
    '🇦🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤲𞤼𞤭𞤺𞤵𞤱𞤢 & 𞤄𞤢𞤪𞤦𞤵𞥅𞤣𞤢',
    '🇦🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤲𞤺𞤭𞤤𞤢𞥄',
    '🇦🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤤𞤦𞤢𞤲𞤭𞤴𞤢𞥄',
    '🇦🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤪𞤥𞤫𞤲𞤭𞤴𞤢𞥄',
    '🇦🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤲𞤺𞤮𞤤𞤢𞥄',
    '🇦🇶' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤲𞤼𞤢𞤪𞤼𞤭𞤳𞤢𞥄',
    '🇦🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤪𞤶𞤢𞤲𞤼𞤭𞤲𞤢𞥄',
    '🇦🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞤥𞤵𞤱𞤢 𞤀𞤥𞤫𞤪𞤭𞤳𞤭𞤴𞤢𞤲𞤳𞤮',
    '🇦🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤌𞤼𞤭𞤪𞤧𞤢',
    '🇦🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄',
    '🇦🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤪𞤵𞤦𞤢𞥄',
    '🇦🇽' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤀𞤤𞤢𞤲𞤣',
    '🇦🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞥁𞤫𞤪𞤦𞤢𞤭𞤶𞤢𞥄𞤲',
    '🇧🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤮𞤧𞤲𞤭𞤴𞤢 & 𞤖𞤫𞤪𞤧𞤫𞤳𞤮𞤾𞤭𞤲𞤢𞥄',
    '🇧🇧' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤢𞤪𞤦𞤫𞥅𞤣𞤮𞥅𞤧',
    '🇧🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤢𞤲𞤺𞤭𞤤𞤢𞤣𞤫𞥅𞤧',
    '🇧🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤫𞤤𞤶𞤭𞤳𞤢𞥄',
    '🇧🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤵𞤪𞤳𞤭𞤲𞤢 𞤊𞤢𞤧𞤮𞥅',
    '🇧🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤵𞥅𞤤𞤺𞤢𞤪𞤭𞤴𞤢𞥄',
    '🇧🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤢𞤸𞤢𞤪𞤢𞤴𞤲',
    '🇧🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤵𞤪𞤵𞤲𞤣𞤭',
    '🇧🇯' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤫𞤲𞤫𞤲',
    '🇧🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤼. 𞤄𞤢𞤪𞤼𞤫𞤤𞤭𞤥𞤭',
    '🇧🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤭𞤪𞤥𞤵𞤣𞤢',
    '🇧🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤵𞤪𞤲𞤢𞥄𞤴',
    '🇧🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤮𞤤𞤭𞥅𞤾𞤭𞤴𞤢𞥄',
    '🇧🇶' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞤪𞤦𞤭𞤴𞤢𞥄 𞤖𞤮𞤤𞤢𞤲𞤣𞤭𞤴𞤢𞥄',
    '🇧🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤪𞤢𞤧𞤭𞤤',
    '🇧🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤢𞤸𞤢𞤥𞤢𞥄𞤧',
    '🇧🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤵𞥅𞤼𞤢𞥄𞤲',
    '🇧🇻' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤄𞤵𞥅𞤾𞤫𞥅',
    '🇧🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤮𞤼𞤧𞤵𞤱𞤢𞥄𞤲𞤢',
    '🇧🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤫𞤤𞤢𞤪𞤵𞥅𞤧',
    '🇧🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤄𞤫𞤤𞤭𞥅𞥁',
    '🇨🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞤲𞤢𞤣𞤢𞥄',
    '🇨🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤑𞤮𞤳𞤮𞥅𞤧 (𞤑𞤭𞥅𞤤𞤭𞤲𞤺)',
    '🇨🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞤲𞤺𞤮 - 𞤑𞤭𞤲𞤧𞤢𞤧𞤢',
    '🇨🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤣𞤫𞤲𞤣𞤭 𞤚𞤵𞤥𞤦𞤮𞥅𞤪𞤭 𞤀𞤬𞤪𞤭𞤳𞤢𞥄',
    '🇨🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞤲𞤺𞤮 - 𞤄𞤪𞤢𞥁𞤢𞤾𞤭𞤤',
    '🇨🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤱𞤭𞤪𞤧𞤢𞥄',
    '🇨🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞤼𞤣𞤭𞤾𞤢𞥄𞤪',
    '🇨🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤑𞤵𞥅𞤳',
    '🇨🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤭𞤤𞤫𞥊𞥅',
    '🇨🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞤥𞤢𞤪𞤵𞥅𞤲',
    '🇨🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤢𞤴𞤲𞤢',
    '🇨🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞤤𞤮𞤥𞤦𞤭𞤴𞤢𞥄',
    '🇨🇵' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤑𞤭𞤤𞤭𞤨𞤫𞤪𞤼𞤮𞤲',
    '🇨🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞤧𞤼𞤢 𞤈𞤭𞤳𞤢𞥄',
    '🇨🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤵𞥅𞤦𞤢𞥄',
    '🇨🇻' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞥄𞤦𞤮 𞤜𞤫𞤪𞤣𞤫',
    '🇨🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤵𞥅𞤪𞤢𞤧𞤢𞤱𞤮',
    '🇨🇽' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤑𞤭𞤪𞤧𞤭𞤥𞤢𞥄𞤧',
    '🇨🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤵𞤦𞤪𞤵𞥅𞤧',
    '🇨🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤫𞥅𞤳𞤭𞤴𞤢𞥄',
    '🇩🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤫𞤪𞤥𞤢𞤲𞤭𞥅',
    '🇩🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤮𞤺𞤮 𞤘𞤢𞥄𞤪𞤧𞤭𞤴𞤢',
    '🇩🇯' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤭𞤦𞤵𞥅𞤼𞤭',
    '🇩🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤁𞤢𞤲𞤵𞤥𞤢𞤪𞤳',
    '🇩🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤁𞤮𞤥𞤭𞤲𞤭𞤳𞤢𞥄',
    '🇩🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤣𞤫𞤲𞤣𞤭 𞤁𞤮𞤥𞤭𞤲𞤭𞤳𞤢𞥄',
    '🇩🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤤𞤶𞤢𞤪𞤭𞥅',
    '🇪🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤭𞤼𞥆𞤢 & 𞤃𞤫𞤤𞤭𞤤𞤢',
    '🇪🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤉𞤳𞤵𞤱𞤢𞤣𞤮𞥅𞤪',
    '🇪🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤉𞤧𞤼𞤮𞤲𞤭𞤴𞤢𞥄',
    '🇪🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤭𞤧𞤭𞤪𞤢',
    '🇪🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞥄𞤸𞤢𞤪𞤢 𞤖𞤭𞥅𞤲𞤢𞥄𞤪𞤭',
    '🇪🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤉𞤪𞤭𞥅𞤼𞤫𞤪𞤫',
    '🇪🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤉𞤧𞤨𞤢𞤻𞤢𞥄',
    '🇪🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤦𞤢𞤧𞤭𞤲𞤭𞥅',
    '🇪🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞤱𞤼𞤢𞤤 𞤓𞤪𞤨𞤢',
    '🇫🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤊𞤭𞤲𞤤𞤢𞤲𞤣',
    '🇫🇯' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤊𞤭𞤶𞤭𞥅',
    '🇫🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤊𞤢𞤤𞤳𞤵𞤤𞤢𞤲𞤣',
    '🇫🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤭𞤳𞤪𞤮𞤲𞤫𞥅𞤧𞤭𞤴𞤢',
    '🇫🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤊𞤢𞤪𞤵𞥅𞤧',
    '🇫🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤊𞤢𞤪𞤢𞤲𞤧𞤭',
    '🇬🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤢𞤦𞤮𞤲',
    '🇬🇧' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤁𞤫𞤲𞤼𞤢𞤤 𞤐𞤺𞤫𞤯𞤵𞥅𞤪𞤭',
    '🇬🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤢𞤪𞤲𞤢𞤣𞤢𞥄',
    '🇬🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤮𞤪𞤶𞤭𞤴𞤢𞥄',
    '🇬🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤵𞤴𞤢𞥄𞤲 𞤊𞤪𞤢𞤲𞤧𞤭𞤲𞤳𞤮',
    '🇬🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤢𞤪𞤲𞤫𞤧𞤭𞥅',
    '🇬🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤢𞤲𞤢',
    '🇬🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤭𞤦𞤪𞤢𞤤𞤼𞤢𞥄',
    '🇬🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤭𞤪𞤤𞤢𞤲𞤣𞤭',
    '🇬🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤢𞤥𞤦𞤭𞤴𞤢',
    '🇬🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤭𞤲𞤫',
    '🇬🇵' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤵𞤱𞤢𞤣𞤢𞤤𞤵𞤨𞤫𞥅',
    '🇬🇶' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤭𞤲𞤫 𞤕𞤢𞤳𞤢𞤲𞤼𞤫𞥅𞤪𞤭',
    '🇬🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤤𞤴𞤵𞤲𞤢𞥄𞤲',
    '🇬🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤔𞤮𞤪𞤶𞤭𞤴𞤢 & 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤅𞤢𞤲𞤣𞤵𞤱𞤭𞥅𞤷',
    '🇬🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤵𞤱𞤢𞤼𞤫𞤥𞤢𞤤𞤢𞥄',
    '🇬🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤵𞤱𞤢𞥄𞤥',
    '🇬🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤭𞤲𞤫-𞤄𞤭𞤧𞤢𞤱𞤮𞥅',
    '🇬🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤘𞤢𞤴𞤢𞤲𞤢𞥄',
    '🇭🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤖𞤂𞤀 𞤕𞤢𞤴𞤲𞤢 𞤫 𞤖𞤮𞤲𞤺 𞤑𞤮𞤲𞤺',
    '🇭🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤖𞤭𞤪𞤣𞤭 & 𞤃𞤢𞤳𞤣𞤮𞤲𞤢𞤤',
    '🇭🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤖𞤮𞤲𞤣𞤭𞤪𞤢𞥄𞤧',
    '🇭🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤵𞤪𞤱𞤢𞥄𞤧𞤭𞤴𞤢',
    '🇭🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤖𞤢𞤴𞤼𞤭𞥅',
    '🇭🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤖𞤢𞤲𞤺𞤢𞤪𞤭𞤴𞤢𞥄',
    '🇮🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫-𞤑𞤢𞤲𞤢𞤪𞤭𞥅',
    '🇮🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤋𞤲𞤣𞤮𞤲𞤭𞥅𞤧𞤴𞤢',
    '🇮🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤋𞤪𞤤𞤢𞤲𞤣',
    '🇮🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤋𞤧𞤪𞤢𞥄𞤴𞤭𞥅𞤤',
    '🇮🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤃𞤫𞥅𞤲',
    '🇮🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤋𞤲𞤣𞤭𞤴𞤢',
    '🇮🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤵𞤥𞤦𞤫𞤪𞤫 𞤄𞤪𞤭𞤼𞤢𞤲𞤭𞤲𞤳𞤮𞥅𞤪𞤫 𞤀𞤬𞤪𞤭𞤳𞤭',
    '🇮🇶' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤋𞤪𞤢𞥄𞤳',
    '🇮🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤋𞤪𞤢𞥄𞤲',
    '🇮🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤴𞤧𞤵𞤤𞤢𞤲𞤣',
    '🇮🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤋𞤼𞤢𞤤𞤭𞥅',
    '🇯🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤫𞤪𞤧𞤭𞥅',
    '🇯🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤢𞤥𞤢𞤴𞤳𞤢𞥄',
    '🇯🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤔𞤮𞤪𞤣𞤢𞥄𞤲',
    '🇯🇵' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤭𞤨𞥆𞤮𞤲',
    '🇰🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤫𞤲𞤭𞤴𞤢𞥄',
    '🇰🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤭𞤪𞤶𞤭𞤧𞤼𞤢𞥄𞤲',
    '🇰🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞤥𞤦𞤮𞥅𞤣𞤭𞤴𞤢',
    '🇰🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤭𞤪𞤦𞤢𞤼𞤭𞥅',
    '🇰🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞤥𞤮𞥅𞤪𞤮',
    '🇰🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤼. 𞤑𞤪𞤭𞤧𞤼𞤮𞤦𞤢𞤤 & 𞤐𞤫𞥅𞤾𞤭𞤧',
    '🇰🇵' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞥅𞤪𞤫𞤴𞤢𞥄 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫',
    '🇰🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞥅𞤪𞤫𞤴𞤢𞥄 𞤙𞤢𞤥𞤲𞤢𞥄𞤲𞤺𞤫',
    '🇰🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤵𞤱𞤢𞤴𞤼𞤵',
    '🇰🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤑𞤢𞤴𞤥𞤢𞥄𞤲',
    '🇰🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞥁𞤢𞤧𞤼𞤢𞥄𞤲',
    '🇱🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤢𞤱𞤮𞥅𞤧',
    '🇱🇧' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤭𞤦𞤢𞤲𞤮𞥅𞤲',
    '🇱🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤼. 𞤂𞤵𞥅𞤧𞤭𞤴𞤢',
    '🇱🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤭𞤧𞤼𞤫𞤲𞤧𞤭𞤼𞤫𞥅𞤲',
    '🇱🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤭𞤪 𞤂𞤢𞤲𞤳𞤢𞥄',
    '🇱🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤢𞤦𞤭𞤪𞤭𞤴𞤢𞥄',
    '🇱🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤫𞤧𞤮𞤼𞤮𞥅',
    '🇱🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤭𞤼𞤵𞤾𞤢',
    '🇱🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤵𞤳𞤧𞤢𞤲𞤦𞤵𞥅𞤺',
    '🇱🇻' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤢𞤼𞤾𞤭𞤴𞤢',
    '🇱🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤭𞤦𞤭𞤴𞤢𞥄',
    '🇲🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤪𞤮𞥅𞤳',
    '🇲🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤲𞤢𞤳𞤮𞥅',
    '🇲🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤤𞤣𞤮𞤾𞤢𞥄',
    '🇲🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤲𞤼𞤫𞤲𞤫𞥅𞤺𞤮𞤪𞤮',
    '🇲🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤼. 𞤃𞤢𞤪𞤼𞤫𞤲',
    '🇲🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤣𞤢𞤺𞤢𞤧𞤳𞤢𞥄𞤪',
    '🇲🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤃𞤢𞤪𞥃𞤢𞤤',
    '🇲🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤳𞤫𞤣𞤮𞤲𞤭𞤴𞤢𞥄 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫',
    '🇲🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞥄𞤤𞤭',
    '🇲🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤭𞤴𞤢𞤥𞤢𞥄𞤪 (𞤄𞤵𞥅𞤪𞤥𞤢)',
    '🇲🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤲𞤺𞤮𞤤𞤭𞤴𞤢',
    '🇲🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤖𞤂𞤀 𞤕𞤢𞤴𞤲𞤢 𞤫 𞤃𞤢𞤳𞤢𞤱𞤮𞥅',
    '🇲🇵' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤃𞤢𞤪𞤭𞤴𞤢𞥄𞤲𞤢 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭',
    '🇲🇶' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤪𞤼𞤭𞤲𞤭𞤳𞤢𞥄',
    '🇲🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤪𞤼𞤢𞤲𞤭𞥅',
    '🇲🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤲𞤧𞤭𞤪𞤢𞥄𞤼',
    '🇲🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤤𞤼𞤢',
    '🇲🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤪𞤭𞥅𞤧𞤭',
    '🇲🇻' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤤𞤣𞤭𞥅𞤬',
    '🇲🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤤𞤢𞤱𞤭𞥅',
    '🇲🇽' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤫𞤳𞤧𞤭𞤳𞤮𞥅',
    '🇲🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤤𞤫𞥅𞤧𞤭𞤴𞤢',
    '🇲🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤮𞤧𞤢𞤥𞤦𞤭𞥅𞤳',
    '🇳🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤢𞤥𞤭𞥅𞤦𞤭𞤴𞤢𞥄',
    '🇳🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞤤𞤭𞤣𞤮𞤲𞤭𞤴𞤢𞥄 𞤖𞤫𞤧𞤮',
    '🇳🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤭𞥅𞤶𞤫𞤪',
    '🇳🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤐𞤮𞤪𞤬𞤮𞤤𞤳𞤵',
    '🇳🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤢𞤶𞤫𞤪𞤭𞤴𞤢𞥄',
    '🇳🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤭𞤳𞤢𞤪𞤢𞤺𞤵𞤱𞤢𞥄',
    '🇳🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤖𞤮𞤤𞤢𞤲𞤣𞤭𞤴𞤢𞥄',
    '🇳🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤮𞤪𞤺𞤫𞤴𞤢𞥄',
    '🇳🇵' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤭𞤨𞤢𞥄𞤤',
    '🇳🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤢𞤱𞤪𞤵',
    '🇳🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤵𞥅𞤱𞤭',
    '🇳🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤐𞤫𞤱 𞤟𞤫𞤤𞤢𞤲𞤣',
    '🇴🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤌𞥅𞤥𞤢𞥄𞤲',
    '🇵🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤢𞤲𞤢𞤥𞤢',
    '🇵🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤫𞤪𞤵𞥅',
    '🇵🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤮𞤤𞤭𞤲𞤫𞥅𞤧𞤭𞤴𞤢 𞤊𞤪𞤢𞤲𞤧𞤭𞤲𞤳𞤮',
    '🇵🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤢𞤨𞤵𞤱𞤢 𞤘𞤭𞤲𞤫 𞤖𞤫𞤧𞤮',
    '🇵🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤊𞤭𞤤𞤭𞤨𞤭𞥅𞤲',
    '🇵🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤢𞤳𞤭𞤧𞤼𞤢𞥄𞤲',
    '🇵🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤮𞤤𞤢𞤲𞤣',
    '🇵🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤼. 𞤆𞤭𞤴𞤫𞥅𞤪 & 𞤃𞤭𞤳𞤫𞤤𞤮𞤲',
    '🇵🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤆𞤭𞤼𞤳𞤭𞥅𞤪𞤲𞤵',
    '🇵🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤮𞤪𞤼𞤮 𞤈𞤭𞤳𞤮𞥅',
    '🇵🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤂𞤫𞤧𞤣𞤭𞥅𞤶𞤭 𞤊𞤢𞤤𞤫𞤧𞤼𞤭𞥅𞤲',
    '🇵🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤮𞥅𞤪𞤼𞤵𞤺𞤢𞥄𞤤',
    '🇵🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤢𞤤𞤢𞤱',
    '🇵🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤆𞤢𞥄𞤪𞤢𞤺𞤵𞤱𞤢𞥄𞤴',
    '🇶🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤊𞤢𞤤𞤫𞤧𞤼𞤭𞥅𞤲',
    '🇷🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤈𞤫𞥅𞤲𞤭𞤴𞤮𞤲',
    '🇷🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤈𞤵𞤥𞤢𞥄𞤲𞤭𞤴𞤢',
    '🇷🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤫𞤪𞤦𞤭𞤴𞤢𞥄',
    '🇷🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤈𞤮𞥅𞤧𞤭𞤴𞤢',
    '🇷🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤈𞤵𞤱𞤢𞤲𞤣𞤢𞥄',
    '🇸🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞤵𞥅𞤣 𞤀𞥄𞤪𞤢𞤦𞤭𞤴𞤢𞥄',
    '🇸🇧' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤅𞤵𞤤𞤢𞤴𞤥𞤢𞥄𞤲',
    '🇸🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤫𞤴𞤭𞤧𞤫𞤤',
    '🇸🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤣𞤢𞥄𞤲',
    '🇸🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤱𞤫𞤣𞤭𞤴𞤢𞥄',
    '🇸🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤭𞤲𞤺𞤢𞤨𞤵𞥅𞤪',
    '🇸🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤫𞤲-𞤖𞤫𞤤𞤫𞤲𞤢𞥄',
    '🇸🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤤𞤮𞤾𞤫𞤲𞤭𞤴𞤢𞥄',
    '🇸🇯' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞤾𞤢𞤤𞤦𞤢𞤪𞤣 & 𞤔𞤢𞤲 𞤃𞤢𞤴𞤫𞤲',
    '🇸🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤤𞤮𞤾𞤢𞥄𞤳𞤭𞤴𞤢',
    '🇸🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞤪𞤢𞤤𞤮𞤲',
    '🇸🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞤲 𞤃𞤢𞤪𞤭𞤲𞤮𞥅',
    '🇸🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤫𞤲𞤫𞤺𞤢𞥄𞤤',
    '🇸🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞥅𞤥𞤢𞥄𞤤𞤭',
    '🇸🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤪𞤭𞤲𞤢𞥄𞤥',
    '🇸🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞤣𞤢𞥄𞤲 𞤂𞤫𞤧𞤤𞤫𞤴𞤪𞤭',
    '🇸🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞤱𞤵 𞤚𞤵𞤥𞤫𞥅 & 𞤆𞤫𞤪𞤫𞤲𞤧𞤭𞤨𞤫',
    '🇸🇻' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤉𞤤 𞤅𞤢𞤤𞤾𞤢𞤣𞤮𞥅𞤪',
    '🇸🇽' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤫𞤲𞤼𞤵 𞤃𞤢𞥄𞤪𞤼𞤫𞤲',
    '🇸🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤵𞥅𞤪𞤭𞤴𞤢𞥄',
    '🇸🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤉𞤧𞤱𞤢𞤼𞤭𞤲𞤭',
    '🇹🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤵𞤪𞤧𞤵𞤼𞤢𞤲 𞤁𞤢𞤳𞤵𞤲𞤸𞤢',
    '🇹🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤚𞤵𞤪𞤳𞤵𞤧 & 𞤑𞤢𞤴𞤳𞤮𞥅𞤧',
    '🇹🇩' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤢𞥄𞤣',
    '🇹🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤵𞤥𞤦𞤫 𞤂𞤫𞤧𞤤𞤫𞤴𞤶𞤫 𞤊𞤪𞤢𞤲𞤧𞤭',
    '🇹🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤮𞤺𞤮',
    '🇹🇭' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤢𞥄𞤴𞤤𞤢𞤲𞤣',
    '🇹🇯' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤢𞤶𞤭𞤳𞤭𞤧𞤼𞤢𞥄𞤲',
    '🇹🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤮𞥅𞤳𞤮𞤤𞤢𞥄𞤱𞤵',
    '🇹🇱' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤭𞤥𞤮𞥅𞤪 𞤂𞤫𞤧𞤼𞤫',
    '🇹🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤵𞤪𞤳𞤵𞤥𞤫𞤲𞤭𞤧𞤼𞤢𞥄𞤲',
    '🇹🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤵𞤲𞤭𞥅𞤧𞤢',
    '🇹🇴' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤮𞤲𞤺𞤢',
    '🇹🇷' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤵𞤪𞤳𞤭𞤴𞤢𞥄',
    '🇹🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤭𞤪𞤲𞤭𞤣𞤢𞥄𞤣 & 𞤚𞤮𞤦𞤢𞤺𞤮𞥅',
    '🇹🇻' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤵𞥅𞤾𞤢𞤤𞤵',
    '🇹🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤢𞤴𞤱𞤢𞥄𞤲',
    '🇹🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤚𞤢𞤲𞤧𞤢𞤲𞤭𞥅',
    '🇺🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤓𞤳𞤪𞤫𞥅𞤲𞤭𞤴𞤢',
    '🇺🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤓𞤺𞤢𞤲𞤣𞤢𞥄',
    '🇺🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤁𞤢𞥄𞤴𞤭𞥅𞤯𞤫 𞤁𞤂𞤀',
    '🇺🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤢𞤱𞤼𞤢𞤤 𞤘𞤫𞤲𞤯𞤭',
    '🇺🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤁𞤫𞤲𞤼𞤢𞤤 𞤂𞤢𞤪𞤫',
    '🇺🇾' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤒𞤵𞤪𞤺𞤮𞤴',
    '🇺🇿' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤓𞥁𞤦𞤫𞤳𞤭𞤧𞤼𞤢𞥄𞤲',
    '🇻🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤜𞤢𞤼𞤭𞤳𞤢𞥄𞤲',
    '🇻🇨' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤼. 𞤜𞤭𞤲𞤧𞤢𞤲 & 𞤘𞤭𞤪𞤲𞤢𞤣𞤭𞥅𞤲',
    '🇻🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤜𞤫𞥊𞤲𞤭𞥅𞥁𞤵𞤱𞤫𞤤𞤢𞥄',
    '🇻🇬' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤜𞤭𞤪𞤺𞤭𞥅𞤲 𞤄𞤪𞤭𞤼𞤢𞤲𞤭𞤴𞤢𞤲𞤳𞤮𞥅𞤶𞤫',
    '🇻🇮' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤜𞤭𞤪𞤺𞤭𞥅𞤲 𞤁𞤂𞤀',
    '🇻🇳' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤜𞤭𞤴𞤫𞤼𞤲𞤢𞥄𞤥',
    '🇻🇺' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤜𞤢𞤲𞤵𞤱𞤢𞥄𞤼𞤵',
    '🇼🇫' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤏𞤢𞤤𞥆𞤭𞥅𞤧 & 𞤊𞤵𞤼𞤵𞤲𞤢',
    '🇼🇸' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤅𞤢𞤥𞤵𞤱𞤢',
    '🇽🇰' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤑𞤮𞥅𞤧𞤮𞤾𞤮𞥅',
    '🇾🇪' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤒𞤢𞤥𞤢𞤲',
    '🇾🇹' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤃𞤢𞤴𞤮𞥅𞤼𞤵',
    '🇿🇦' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤀𞤬𞤪𞤭𞤳𞤢 𞤂𞤫𞤧𞤤𞤫𞤴𞤪𞤭',
    '🇿🇲' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤟𞤢𞤥𞤦𞤭𞤴𞤢',
    '🇿🇼' => '𞤼𞤵𞤼𞤢𞤱𞤢𞤤: 𞤟𞤭𞤥𞤦𞤢𞥄𞤥𞤵𞤴𞤢',
    '🔟' => '𞤊𞤫𞤣𞤭𞤲𞤭𞤪𞤣𞤫: 10',
    '😀' => '𞤴𞤫𞥅𞤧𞤮 𞤶𞤢𞤤𞤢𞤴𞤲𞤺𞤮',
    '😃' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤲𞤶𞤢𞤲𞤫',
    '😄' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤫𞤱𞤤𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤥𞤮𞥅𞤧𞤵𞤯𞤫',
    '😁' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤮𞤪𞤯𞤵𞤲𞤺𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤥𞤮𞥅𞤧𞤮',
    '😆' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤫𞤱𞤤𞤵𞤲𞥋𞤺𞤵 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤥𞤵𞤲𞤰𞤭𞥅𞤯𞤫',
    '😅' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤫𞤱𞤤𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤤𞤢𞤳𞥆𞤫𞤪𞤫',
    '🤣' => '𞤶𞤢𞤤𞤢 𞤼𞤢𞤤𞥆𞤮𞥅 𞤫 𞤤𞤫𞤴𞤣𞤭',
    '😂' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤺𞤮𞤲𞤯𞤭 𞤱𞤫𞤤𞤼𞤢𞥄𞤪𞤫',
    '🙂' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤤𞤫𞤧𞤥𞤮𞥅𞤧𞤢𞤲𞤣𞤫',
    '🙃' => '𞤴𞤫𞥅𞤧𞤮 𞤶𞤵𞤲𞥆𞤭𞤼𞤭𞥅𞤲𞤺𞤮',
    '🫠' => '𞤴𞤫𞥅𞤧𞤮 𞤴𞤮𞥅𞤧𞤢𞤴𞤲𞤺𞤮',
    '😉' => '𞤴𞤫𞥅𞤧𞤮 𞤸𞤭𞤲𞤰𞤢𞤴𞤲𞤺𞤮',
    '😊' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞤺𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤥𞤮𞥅𞤧𞤵𞤯𞤫',
    '😇' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤼𞤢𞥄𞤪𞤲𞤣𞤫',
    '🥰' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞤺𞤮 𞤲𞤺𞤮 𞤩𞤫𞤪𞤤𞤫',
    '😍' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤱𞤢 𞤩𞤫𞤪𞤤𞤫',
    '🤩' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤳𞤮𞥅𞤣𞤫',
    '😘' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤮𞤲𞤺𞤮 𞤲𞤫𞤤𞤣𞤵𞤣𞤫 𞤤𞤵𞤲𞥆𞤢𞤲𞤣𞤫',
    '😗' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤮𞤲𞤺𞤮 𞤤𞤵𞤲𞥆𞤢𞥄𞤣𞤫',
    '☺' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞤺𞤮',
    '😚' => '𞤴𞤫𞥅𞤧𞤮 𞤤𞤵𞤲𞥆𞤮𞤼𞤮𞥅𞤲𞤺𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤥𞤵𞤲𞥆𞤭𞥅𞤯𞤫',
    '😙' => '𞤴𞤫𞥅𞤧𞤮 𞤤𞤵𞤲𞥆𞤮𞤼𞤮𞥅𞤲𞤺𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤥𞤮𞥅𞤧𞤵𞤯𞤫',
    '🥲' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤺𞤮𞤥𞤯𞤮𞤤',
    '😋' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤮𞤲𞤺𞤮 𞤣𞤢𞤳𞤥𞤭𞤼𞤢𞥄𞤣𞤫 𞤻𞤢𞥄𞤥𞤣𞤵',
    '😛' => '𞤴𞤫𞥅𞤧𞤮 𞤤𞤢𞥄𞤬𞤵𞤲𞥋𞤺𞤮 𞤯𞤫𞤲𞤺𞤢𞤤',
    '😜' => '𞤴𞤫𞥅𞤧𞤮 𞤸𞤭𞤲𞤰𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤯𞤢𞤲𞤺𞤢𞤤 𞤤𞤢𞥄𞤬𞤢𞥄𞤲𞤺𞤢𞤤',
    '🤪' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤭𞤤𞤭𞥅𞤯𞤮',
    '😝' => '𞤴𞤫𞥅𞤧𞤮 𞤤𞤫𞥅𞤤𞤢𞥄𞤶𞤮 𞤲𞤺𞤮 𞤯𞤫𞤲𞤺𞤢𞤤 𞤤𞤢𞥄𞤬𞤢𞥄𞤲𞤺𞤢𞤤',
    '🤑' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤫 𞤯𞤫𞤲𞤺𞤢𞤤 𞤲𞤦𞤵𞥅𞤯𞤭',
    '🤗' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤵𞤲𞥋𞤺𞤮 𞤲𞤺𞤮 𞤶𞤵𞥅𞤯𞤫 𞤬𞤮𞤲𞤼𞤢𞥄𞤯𞤫',
    '🤭' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤶𞤵𞥅𞤲𞤺𞤮 𞤫 𞤸𞤮𞥅𞤪𞤫 𞤸𞤵𞤲𞤣𞤵𞤳𞤮',
    '🫢' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤮𞤲𞤼𞤭𞥅𞤯𞤫 𞤫 𞤶𞤵𞥅𞤲𞤺𞤮 𞤣𞤮𞤱 𞤸𞤵𞤲𞤣𞤵𞤳𞤮',
    '🫣' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤴𞤭𞤴𞤼𞤫𞤪𞤫 𞤴𞤵𞤪𞤲𞤮𞤼𞤮𞥅𞤲𞤣𞤫',
    '🤫' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤮𞤲𞤺𞤮 𞤫 𞤣𞤫𞤰𞥆𞤭𞤲𞤣𞤫',
    '🤔' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤮𞤲𞤺𞤮 𞤫 𞤥𞤭𞥅𞤶𞤢𞥄𞤣𞤫',
    '🫡' => '𞤴𞤫𞥅𞤧𞤮 𞤱𞤮𞤲𞤺𞤮 𞤫 𞤴𞤫𞤼𞥆𞤵𞤣𞤫',
    '🤐' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤸𞤵𞤲𞤣𞤵𞤳𞤮 𞤸𞤮𞤪𞤧𞤢𞥄𞤳𞤮',
    '🤨' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤼𞤭𞥅𞤥𞤫𞤤𞤭 𞤩𞤢𞤽𞤼𞤭𞤭𞤯𞤭',
    '😐' => '𞤴𞤫𞥅𞤧𞤮 𞤧𞤢𞥄𞤸𞤵𞤲𞥋𞤺𞤮',
    '😑' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤢𞤤𞤢𞥄 𞤸𞤫𞤥𞤧𞤮',
    '😶' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤢𞤤𞤢𞥄 𞤸𞤵𞤲𞤣𞤵𞤳𞤮',
    '🫥' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤷𞤭𞥅𞤬𞤭 𞤥𞤫𞤪𞤻𞤵𞤯𞤭',
    '😏' => '𞤴𞤫𞥅𞤧𞤮 𞤥𞤮𞥅𞤧𞤭𞤲𞤳𞤭𞤲𞤮𞥅𞤲𞤺𞤮',
    '😒' => '𞤴𞤫𞥅𞤧𞤮 𞤢𞤧-𞤧𞤫𞤴𞤲𞤵𞤲𞥋𞤺𞤮',
    '🙄' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤺𞤭𞤴𞤼𞤫 𞤤𞤫𞥅𞤼𞤢𞤴𞤯𞤮',
    '😬' => '𞤴𞤫𞥅𞤧𞤮 𞤩𞤭𞥅𞤻𞤢𞤴𞤲𞤺𞤮',
    '🤥' => '𞤴𞤫𞥅𞤧𞤮 𞤬𞤫𞤲𞤢𞤴𞤯𞤮',
    '🫨' => '𞤴𞤫𞥅𞤧𞤮 𞤣𞤭𞤱𞤲𞤢𞤴𞤲𞤺𞤮',
    '😌' => '𞤴𞤫𞥅𞤧𞤮 𞤸𞤮𞤴𞤬𞤵𞤲𞥋𞤺𞤮',
    '😔' => '𞤴𞤫𞥅𞤧𞤮 𞤶𞤭𞤥𞥆𞤭𞤲𞤭𞥅𞤲𞤺𞤮',
    '😪' => '𞤒𞤫𞥅𞤧𞤮 𞤽𞤮𞤽𞤵𞤲𞥋𞤺𞤮',
    '🤤' => '𞤴𞤫𞥅𞤧𞤮 𞤴𞤮𞥅𞤣𞤮𞤼𞤮𞥅𞤲𞤺𞤮',
    '😴' => '𞤴𞤫𞥅𞤧𞤮 𞤯𞤢𞥄𞤲𞤭𞥅𞤲𞤺𞤮',
    '😷' => '𞤴𞤫𞥅𞤧𞤮 𞤲𞤺𞤮 𞤲𞤢𞥄𞤦𞤮 𞤶𞤢𞤥',
    '🤮' => '𞤴𞤫𞥅𞤧𞤮 𞤼𞤵𞥅𞤼𞤢𞤴𞤲𞤺𞤮',
    '😫' => '𞤴𞤫𞥅𞤧𞤮 𞤪𞤮𞤲𞤳𞤵𞤲𞥋𞤺𞤮',
    '👋' => '𞤸𞤢𞤴𞤤𞤵𞤣𞤫 𞤶𞤵𞥅𞤲𞤺𞤮',
    '🤚' => '𞤩𞤢𞤽𞤼𞤭𞤼𞤢𞤲𞤭𞥅 𞤶𞤵𞥅𞤲𞤺𞤮',
    '✋' => '𞤶𞤵𞥅𞤲𞤺𞤮 𞤼𞤮𞥅𞤱𞤲𞤢𞥄𞤲𞤺𞤮',
    '👂' => '𞤲𞤮𞤬𞤪𞤵',
    '👃' => '𞤳𞤭𞤲𞤢𞤤',
    '🧠' => '𞤲𞤺𞤢𞥄𞤲𞤣𞤭',
    '🐵' => '𞤴𞤫𞥅𞤧𞤮 𞤳𞤵𞤤𞤢𞥄𞤪𞤵',
    '🐒' => '𞤳𞤵𞤤𞤢𞥄𞤪𞤵',
    '🦍' => '𞤣𞤫𞥅𞤥𞤵𞥅𞤪𞤵',
    '🦧' => '𞤮𞤪𞤢𞤲𞤺𞤵𞤼𞤢𞤲𞤪𞤵',
    '🐶' => '𞤴𞤫𞥅𞤧𞤮 𞤪𞤢𞤱𞤢𞥄𞤲𞤣𞤵',
    '🐕' => '𞤪𞤢𞤱𞤢𞥄𞤲𞤣𞤵',
    '🦮' => '𞤪𞤢𞤱𞤢𞥄𞤲𞤣𞤵 𞤯𞤮𞤸𞤢𞤴𞤲𞤣𞤵',
    '🫏' => '𞤲𞤦𞤢𞤦𞥆𞤢',
    '🐐' => '𞤲𞤦𞤫𞥅𞤱𞤢',
    '🐪' => '𞤲𞤺𞤫𞤤𞤮𞥅𞤲𞤦𞤢',
    '🐘' => '𞤥𞤢𞤱𞤦𞤢',
    '🦣' => '𞤻𞤭𞥅𞤱𞤢',
    '🐁' => '𞤲𞤣𞤮𞥅𞤥𞤪𞤵',
    '🐇' => '𞤧𞤢𞤪𞤭𞥅𞤪𞤫',
    '🐦' => '𞤧𞤮𞤤𞤲𞤣𞤵',
    '🐸' => '𞤼𞤮𞤼𞤭𞥅𞤪𞤵',
    '🐊' => '𞤲𞤮𞥅𞤪𞤢',
    '🐢' => '𞤳𞤫𞤳𞤵𞤱𞤢𞤤',
    '🦎' => '𞤨𞤢𞤤𞥆𞤢𞤪𞤣𞤭',
    '🐟' => '𞤤𞤭𞤲𞤺𞤭𞤤',
    '🐝' => '𞤻𞤢𞥄𞤳𞤭𞤤',
    '🪰' => '𞤷𞤮𞥅𞤲𞤺𞤭𞤤',
    '👓' => '𞤲𞤣𞤢𞥄𞤪𞤯𞤵𞤤𞤫',
    '🏧' => '𞤃𞤢𞥄𞤲𞤣𞤫 𞤑𞤑𞤒',
];
