<?php

return [
    'Names' => [
        'Arab' => 'Għarbi',
        'Brai' => 'Braille',
        'Cyrl' => 'Ċirilliku',
        'Grek' => 'Grieg',
        'Hans' => 'Simplifikat',
        'Hant' => 'Tradizzjonali',
        'Jpan' => 'Ġappuniż',
        'Latn' => 'Latin',
        'Xpeo' => 'Persjan Antik',
        'Zxxx' => 'Mhux Miktub',
        'Zyyy' => 'Komuni',
    ],
];
