<?php

return [
    'Names' => [
        'af' => 'Afrikoota',
        'af_NA' => 'Afrikoota (Namiibiyaa)',
        'af_ZA' => 'Afrikoota (Afrikaa Kibbaa)',
        'am' => 'Afaan Amaaraa',
        'am_ET' => 'Afaan Amaaraa (Itoophiyaa)',
        'ar' => 'Arabiffaa',
        'ar_001' => 'Arabiffaa (addunyaa)',
        'ar_AE' => 'Arabiffaa (Yuunaatid Arab Emereet)',
        'ar_BH' => 'Arabiffaa (Baahireen)',
        'ar_DJ' => 'Arabiffaa (Jibuutii)',
        'ar_DZ' => 'Arabiffaa (Aljeeriyaa)',
        'ar_EG' => 'Arabiffaa (Missir)',
        'ar_EH' => 'Arabiffaa (Sahaaraa Dhihaa)',
        'ar_ER' => 'Arabiffaa (Eertiraa)',
        'ar_IL' => 'Arabiffaa (Israa’eel)',
        'ar_IQ' => 'Arabiffaa (Iraaq)',
        'ar_JO' => 'Arabiffaa (Jirdaan)',
        'ar_KM' => 'Arabiffaa (Komoroos)',
        'ar_KW' => 'Arabiffaa (Kuweet)',
        'ar_LB' => 'Arabiffaa (Libaanoon)',
        'ar_LY' => 'Arabiffaa (Liibiyaa)',
        'ar_MA' => 'Arabiffaa (Morookoo)',
        'ar_MR' => 'Arabiffaa (Mawuritaaniyaa)',
        'ar_OM' => 'Arabiffaa (Omaan)',
        'ar_PS' => 'Arabiffaa (Daangaawwan Paalestaayin)',
        'ar_QA' => 'Arabiffaa (Kuwaatar)',
        'ar_SA' => 'Arabiffaa (Saawud Arabiyaa)',
        'ar_SD' => 'Arabiffaa (Sudaan)',
        'ar_SO' => 'Arabiffaa (Somaaliyaa)',
        'ar_SS' => 'Arabiffaa (Sudaan Kibbaa)',
        'ar_SY' => 'Arabiffaa (Sooriyaa)',
        'ar_TD' => 'Arabiffaa (Chaad)',
        'ar_TN' => 'Arabiffaa (Tuniiziyaa)',
        'ar_YE' => 'Arabiffaa (Yemen)',
        'as' => 'Assamese',
        'as_IN' => 'Assamese (Hindii)',
        'az' => 'Afaan Azerbaijani',
        'az_AZ' => 'Afaan Azerbaijani (Azerbaajiyaan)',
        'az_Cyrl' => 'Afaan Azerbaijani (Saayiriilik)',
        'az_Cyrl_AZ' => 'Afaan Azerbaijani (Saayiriilik, Azerbaajiyaan)',
        'az_Latn' => 'Afaan Azerbaijani (Laatinii)',
        'az_Latn_AZ' => 'Afaan Azerbaijani (Laatinii, Azerbaajiyaan)',
        'be' => 'Afaan Belarusia',
        'be_BY' => 'Afaan Belarusia (Beelaarus)',
        'bg' => 'Afaan Bulgariya',
        'bg_BG' => 'Afaan Bulgariya (Bulgaariyaa)',
        'bn' => 'Afaan Baangladeshi',
        'bn_BD' => 'Afaan Baangladeshi (Banglaadish)',
        'bn_IN' => 'Afaan Baangladeshi (Hindii)',
        'br' => 'Bireetoon',
        'br_FR' => 'Bireetoon (Faransaay)',
        'bs' => 'Afaan Bosniyaa',
        'bs_BA' => 'Afaan Bosniyaa (Bosiiniyaa fi Herzoogovinaa)',
        'bs_Cyrl' => 'Afaan Bosniyaa (Saayiriilik)',
        'bs_Cyrl_BA' => 'Afaan Bosniyaa (Saayiriilik, Bosiiniyaa fi Herzoogovinaa)',
        'bs_Latn' => 'Afaan Bosniyaa (Laatinii)',
        'bs_Latn_BA' => 'Afaan Bosniyaa (Laatinii, Bosiiniyaa fi Herzoogovinaa)',
        'ca' => 'Afaan Katalaa',
        'ca_AD' => 'Afaan Katalaa (Andooraa)',
        'ca_ES' => 'Afaan Katalaa (Ispeen)',
        'ca_FR' => 'Afaan Katalaa (Faransaay)',
        'ca_IT' => 'Afaan Katalaa (Xaaliyaan)',
        'cs' => 'Afaan Czech',
        'cs_CZ' => 'Afaan Czech (Cheechiya)',
        'cv' => 'Chuvash',
        'cv_RU' => 'Chuvash (Raashiyaa)',
        'cy' => 'Welishiffaa',
        'cy_GB' => 'Welishiffaa (United Kingdom)',
        'da' => 'Afaan Deenmaark',
        'da_DK' => 'Afaan Deenmaark (Deenmaark)',
        'da_GL' => 'Afaan Deenmaark (Giriinlaand)',
        'de' => 'Afaan Jarmanii',
        'de_AT' => 'Afaan Jarmanii (Awustiriyaa)',
        'de_BE' => 'Afaan Jarmanii (Beeljiyeem)',
        'de_CH' => 'Afaan Jarmanii (Siwizerlaand)',
        'de_DE' => 'Afaan Jarmanii (Jarmanii)',
        'de_IT' => 'Afaan Jarmanii (Xaaliyaan)',
        'de_LI' => 'Afaan Jarmanii (Lichistensteyin)',
        'de_LU' => 'Afaan Jarmanii (Luksembarg)',
        'el' => 'Afaan Giriiki',
        'el_CY' => 'Afaan Giriiki (Qoophiroos)',
        'el_GR' => 'Afaan Giriiki (Giriik)',
        'en' => 'Afaan Ingilizii',
        'en_001' => 'Afaan Ingilizii (addunyaa)',
        'en_150' => 'Afaan Ingilizii (Awurooppaa)',
        'en_AE' => 'Afaan Ingilizii (Yuunaatid Arab Emereet)',
        'en_AG' => 'Afaan Ingilizii (Antiiguyaa fi Barbuudaa)',
        'en_AI' => 'Afaan Ingilizii (Anguyilaa)',
        'en_AS' => 'Afaan Ingilizii (Saamowa Ameerikaa)',
        'en_AT' => 'Afaan Ingilizii (Awustiriyaa)',
        'en_AU' => 'Afaan Ingilizii (Awustiraaliyaa)',
        'en_BB' => 'Afaan Ingilizii (Barbaaros)',
        'en_BE' => 'Afaan Ingilizii (Beeljiyeem)',
        'en_BI' => 'Afaan Ingilizii (Burundii)',
        'en_BM' => 'Afaan Ingilizii (Beermudaa)',
        'en_BS' => 'Afaan Ingilizii (Bahaamas)',
        'en_BW' => 'Afaan Ingilizii (Botosowaanaa)',
        'en_BZ' => 'Afaan Ingilizii (Belize)',
        'en_CA' => 'Afaan Ingilizii (Kanaadaa)',
        'en_CC' => 'Afaan Ingilizii (Odoloota Kokos [Keeliing])',
        'en_CH' => 'Afaan Ingilizii (Siwizerlaand)',
        'en_CK' => 'Afaan Ingilizii (Odoloota Kuuk)',
        'en_CM' => 'Afaan Ingilizii (Kaameruun)',
        'en_CX' => 'Afaan Ingilizii (Odola Kirismaas)',
        'en_CY' => 'Afaan Ingilizii (Qoophiroos)',
        'en_CZ' => 'Afaan Ingilizii (Cheechiya)',
        'en_DE' => 'Afaan Ingilizii (Jarmanii)',
        'en_DK' => 'Afaan Ingilizii (Deenmaark)',
        'en_DM' => 'Afaan Ingilizii (Dominiikaa)',
        'en_EE' => 'Afaan Ingilizii (Istooniyaa)',
        'en_ER' => 'Afaan Ingilizii (Eertiraa)',
        'en_ES' => 'Afaan Ingilizii (Ispeen)',
        'en_FI' => 'Afaan Ingilizii (Fiinlaand)',
        'en_FJ' => 'Afaan Ingilizii (Fiijii)',
        'en_FK' => 'Afaan Ingilizii (Odoloota Faalklaand)',
        'en_FM' => 'Afaan Ingilizii (Maayikirooneeshiyaa)',
        'en_FR' => 'Afaan Ingilizii (Faransaay)',
        'en_GB' => 'Afaan Ingilizii (United Kingdom)',
        'en_GD' => 'Afaan Ingilizii (Girinaada)',
        'en_GE' => 'Afaan Ingilizii (Joorjiyaa)',
        'en_GG' => 'Afaan Ingilizii (Guwernisey)',
        'en_GH' => 'Afaan Ingilizii (Gaanaa)',
        'en_GI' => 'Afaan Ingilizii (Gibraaltar)',
        'en_GM' => 'Afaan Ingilizii (Gaambiyaa)',
        'en_GS' => 'Afaan Ingilizii (Joorjikaa Kibba fi Odoloota Saanduwiich Kibbaa)',
        'en_GU' => 'Afaan Ingilizii (Guwama)',
        'en_GY' => 'Afaan Ingilizii (Guyaanaa)',
        'en_HK' => 'Afaan Ingilizii (Hoong Koong SAR Chaayinaa)',
        'en_HU' => 'Afaan Ingilizii (Hangaarii)',
        'en_ID' => 'Afaan Ingilizii (Indooneeshiyaa)',
        'en_IE' => 'Afaan Ingilizii (Ayeerlaand)',
        'en_IL' => 'Afaan Ingilizii (Israa’eel)',
        'en_IM' => 'Afaan Ingilizii (Islee oof Maan)',
        'en_IN' => 'Afaan Ingilizii (Hindii)',
        'en_IO' => 'Afaan Ingilizii (Daangaa Galaana Hindii Biritish)',
        'en_IT' => 'Afaan Ingilizii (Xaaliyaan)',
        'en_JE' => 'Afaan Ingilizii (Jeersii)',
        'en_JM' => 'Afaan Ingilizii (Jamaayikaa)',
        'en_JP' => 'Afaan Ingilizii (Jaappaan)',
        'en_KE' => 'Afaan Ingilizii (Keeniyaa)',
        'en_KI' => 'Afaan Ingilizii (Kiribaatii)',
        'en_KN' => 'Afaan Ingilizii (St. Kiitis fi Neevis)',
        'en_KY' => 'Afaan Ingilizii (Odoloota Saaymaan)',
        'en_LC' => 'Afaan Ingilizii (St. Suusiyaa)',
        'en_LR' => 'Afaan Ingilizii (Laayibeeriyaa)',
        'en_LS' => 'Afaan Ingilizii (Leseettoo)',
        'en_LT' => 'Afaan Ingilizii (Lutaaniyaa)',
        'en_LV' => 'Afaan Ingilizii (Lativiyaa)',
        'en_MG' => 'Afaan Ingilizii (Madagaaskaar)',
        'en_MH' => 'Afaan Ingilizii (Odoloota Maarshaal)',
        'en_MO' => 'Afaan Ingilizii (Maka’oo SAR Chaayinaa)',
        'en_MP' => 'Afaan Ingilizii (Odola Maariyaanaa Kaabaa)',
        'en_MS' => 'Afaan Ingilizii (Montiseerat)',
        'en_MT' => 'Afaan Ingilizii (Maaltaa)',
        'en_MU' => 'Afaan Ingilizii (Moorishiyees)',
        'en_MV' => 'Afaan Ingilizii (Maaldiivs)',
        'en_MW' => 'Afaan Ingilizii (Maalaawwii)',
        'en_MY' => 'Afaan Ingilizii (Maleeshiyaa)',
        'en_NA' => 'Afaan Ingilizii (Namiibiyaa)',
        'en_NF' => 'Afaan Ingilizii (Odola Noorfoolk)',
        'en_NG' => 'Afaan Ingilizii (Naayijeeriyaa)',
        'en_NL' => 'Afaan Ingilizii (Neezerlaand)',
        'en_NO' => 'Afaan Ingilizii (Noorwey)',
        'en_NR' => 'Afaan Ingilizii (Naawuruu)',
        'en_NU' => 'Afaan Ingilizii (Niwu’e)',
        'en_NZ' => 'Afaan Ingilizii (Neewu Zilaand)',
        'en_PG' => 'Afaan Ingilizii (Papuwa Neawu Giinii)',
        'en_PH' => 'Afaan Ingilizii (Filippiins)',
        'en_PK' => 'Afaan Ingilizii (Paakistaan)',
        'en_PL' => 'Afaan Ingilizii (Poolaand)',
        'en_PN' => 'Afaan Ingilizii (Odoloota Pitikaayirin)',
        'en_PR' => 'Afaan Ingilizii (Poortaar Riikoo)',
        'en_PT' => 'Afaan Ingilizii (Poorchugaal)',
        'en_PW' => 'Afaan Ingilizii (Palaawu)',
        'en_RO' => 'Afaan Ingilizii (Roomaaniyaa)',
        'en_RW' => 'Afaan Ingilizii (Ruwwandaa)',
        'en_SB' => 'Afaan Ingilizii (Odoloota Solomoon)',
        'en_SC' => 'Afaan Ingilizii (Siisheels)',
        'en_SD' => 'Afaan Ingilizii (Sudaan)',
        'en_SE' => 'Afaan Ingilizii (Siwiidin)',
        'en_SG' => 'Afaan Ingilizii (Singaapoor)',
        'en_SH' => 'Afaan Ingilizii (St. Helenaa)',
        'en_SI' => 'Afaan Ingilizii (Islooveeniyaa)',
        'en_SK' => 'Afaan Ingilizii (Isloovaakiyaa)',
        'en_SL' => 'Afaan Ingilizii (Seeraaliyoon)',
        'en_SS' => 'Afaan Ingilizii (Sudaan Kibbaa)',
        'en_SX' => 'Afaan Ingilizii (Siint Maarteen)',
        'en_SZ' => 'Afaan Ingilizii (Iswaatinii)',
        'en_TC' => 'Afaan Ingilizii (Turkis fi Odoloota Kaayikos)',
        'en_TK' => 'Afaan Ingilizii (Tokelau)',
        'en_TO' => 'Afaan Ingilizii (Tonga)',
        'en_TT' => 'Afaan Ingilizii (Tirinidan fi Tobaagoo)',
        'en_TV' => 'Afaan Ingilizii (Tuvalu)',
        'en_TZ' => 'Afaan Ingilizii (Taanzaaniyaa)',
        'en_UA' => 'Afaan Ingilizii (Yuukireen)',
        'en_UG' => 'Afaan Ingilizii (Ugaandaa)',
        'en_UM' => 'Afaan Ingilizii (U.S. Odoloota Alaa)',
        'en_US' => 'Afaan Ingilizii (Yiinaayitid Isteet)',
        'en_VC' => 'Afaan Ingilizii (St. Vinseet fi Gireenadines)',
        'en_VG' => 'Afaan Ingilizii (Odoloota Varjiin Biritish)',
        'en_VI' => 'Afaan Ingilizii (U.S. Odoloota Varjiin)',
        'en_VU' => 'Afaan Ingilizii (Vanuwaatu)',
        'en_WS' => 'Afaan Ingilizii (Saamowa)',
        'en_ZA' => 'Afaan Ingilizii (Afrikaa Kibbaa)',
        'en_ZM' => 'Afaan Ingilizii (Zaambiyaa)',
        'en_ZW' => 'Afaan Ingilizii (Zimbaabuwee)',
        'eo' => 'Afaan Esperantoo',
        'eo_001' => 'Afaan Esperantoo (addunyaa)',
        'es' => 'Afaan Ispeen',
        'es_419' => 'Afaan Ispeen (Laatin Ameerikaa)',
        'es_AR' => 'Afaan Ispeen (Arjentiinaa)',
        'es_BO' => 'Afaan Ispeen (Boliiviyaa)',
        'es_BR' => 'Afaan Ispeen (Biraazil)',
        'es_BZ' => 'Afaan Ispeen (Belize)',
        'es_CL' => 'Afaan Ispeen (Chiilii)',
        'es_CO' => 'Afaan Ispeen (Kolombiyaa)',
        'es_CR' => 'Afaan Ispeen (Kostaa Rikaa)',
        'es_CU' => 'Afaan Ispeen (Kuubaa)',
        'es_DO' => 'Afaan Ispeen (Dominikaa Rippaabilik)',
        'es_EC' => 'Afaan Ispeen (Ekuwaador)',
        'es_ES' => 'Afaan Ispeen (Ispeen)',
        'es_GQ' => 'Afaan Ispeen (Ikkuwaatooriyaal Giinii)',
        'es_GT' => 'Afaan Ispeen (Guwaatimaalaa)',
        'es_HN' => 'Afaan Ispeen (Hondurus)',
        'es_MX' => 'Afaan Ispeen (Meeksiikoo)',
        'es_NI' => 'Afaan Ispeen (Nikaraguwaa)',
        'es_PA' => 'Afaan Ispeen (Paanamaa)',
        'es_PE' => 'Afaan Ispeen (Peeruu)',
        'es_PH' => 'Afaan Ispeen (Filippiins)',
        'es_PR' => 'Afaan Ispeen (Poortaar Riikoo)',
        'es_PY' => 'Afaan Ispeen (Paaraguwaay)',
        'es_SV' => 'Afaan Ispeen (El Salvaadoor)',
        'es_US' => 'Afaan Ispeen (Yiinaayitid Isteet)',
        'es_UY' => 'Afaan Ispeen (Yuraagaay)',
        'es_VE' => 'Afaan Ispeen (Veenzuweelaa)',
        'et' => 'Afaan Istooniya',
        'et_EE' => 'Afaan Istooniya (Istooniyaa)',
        'eu' => 'Afaan Baskuu',
        'eu_ES' => 'Afaan Baskuu (Ispeen)',
        'fa' => 'Afaan Persia',
        'fa_AF' => 'Afaan Persia (Afgaanistaan)',
        'fa_IR' => 'Afaan Persia (Iraan)',
        'ff' => 'Fula',
        'ff_CM' => 'Fula (Kaameruun)',
        'ff_GN' => 'Fula (Giinii)',
        'ff_Latn' => 'Fula (Laatinii)',
        'ff_Latn_BF' => 'Fula (Laatinii, Burkiinaa Faasoo)',
        'ff_Latn_CM' => 'Fula (Laatinii, Kaameruun)',
        'ff_Latn_GH' => 'Fula (Laatinii, Gaanaa)',
        'ff_Latn_GM' => 'Fula (Laatinii, Gaambiyaa)',
        'ff_Latn_GN' => 'Fula (Laatinii, Giinii)',
        'ff_Latn_GW' => 'Fula (Laatinii, Giinii-Bisaawoo)',
        'ff_Latn_LR' => 'Fula (Laatinii, Laayibeeriyaa)',
        'ff_Latn_MR' => 'Fula (Laatinii, Mawuritaaniyaa)',
        'ff_Latn_NE' => 'Fula (Laatinii, Niijer)',
        'ff_Latn_NG' => 'Fula (Laatinii, Naayijeeriyaa)',
        'ff_Latn_SL' => 'Fula (Laatinii, Seeraaliyoon)',
        'ff_Latn_SN' => 'Fula (Laatinii, Senegaal)',
        'ff_MR' => 'Fula (Mawuritaaniyaa)',
        'ff_SN' => 'Fula (Senegaal)',
        'fi' => 'Afaan Fiilaandi',
        'fi_FI' => 'Afaan Fiilaandi (Fiinlaand)',
        'fo' => 'Afaan Faroese',
        'fo_DK' => 'Afaan Faroese (Deenmaark)',
        'fo_FO' => 'Afaan Faroese (Odoloota Fafo’ee)',
        'fr' => 'Afaan Faransaayii',
        'fr_BE' => 'Afaan Faransaayii (Beeljiyeem)',
        'fr_BF' => 'Afaan Faransaayii (Burkiinaa Faasoo)',
        'fr_BI' => 'Afaan Faransaayii (Burundii)',
        'fr_BJ' => 'Afaan Faransaayii (Beenii)',
        'fr_BL' => 'Afaan Faransaayii (St. Barzeleemii)',
        'fr_CA' => 'Afaan Faransaayii (Kanaadaa)',
        'fr_CD' => 'Afaan Faransaayii (Koongoo - Kinshaasaa)',
        'fr_CF' => 'Afaan Faransaayii (Rippaablika Afrikaa Gidduugaleessaa)',
        'fr_CG' => 'Afaan Faransaayii (Koongoo - Biraazaavil)',
        'fr_CH' => 'Afaan Faransaayii (Siwizerlaand)',
        'fr_CI' => 'Afaan Faransaayii (Koti divoor)',
        'fr_CM' => 'Afaan Faransaayii (Kaameruun)',
        'fr_DJ' => 'Afaan Faransaayii (Jibuutii)',
        'fr_DZ' => 'Afaan Faransaayii (Aljeeriyaa)',
        'fr_FR' => 'Afaan Faransaayii (Faransaay)',
        'fr_GA' => 'Afaan Faransaayii (Gaaboon)',
        'fr_GF' => 'Afaan Faransaayii (Faransaay Guyiinaa)',
        'fr_GN' => 'Afaan Faransaayii (Giinii)',
        'fr_GP' => 'Afaan Faransaayii (Gowadelowape)',
        'fr_GQ' => 'Afaan Faransaayii (Ikkuwaatooriyaal Giinii)',
        'fr_HT' => 'Afaan Faransaayii (Haayitii)',
        'fr_KM' => 'Afaan Faransaayii (Komoroos)',
        'fr_LU' => 'Afaan Faransaayii (Luksembarg)',
        'fr_MA' => 'Afaan Faransaayii (Morookoo)',
        'fr_MC' => 'Afaan Faransaayii (Moonaakoo)',
        'fr_MF' => 'Afaan Faransaayii (St. Martiin)',
        'fr_MG' => 'Afaan Faransaayii (Madagaaskaar)',
        'fr_ML' => 'Afaan Faransaayii (Maalii)',
        'fr_MQ' => 'Afaan Faransaayii (Martinikuwee)',
        'fr_MR' => 'Afaan Faransaayii (Mawuritaaniyaa)',
        'fr_MU' => 'Afaan Faransaayii (Moorishiyees)',
        'fr_NC' => 'Afaan Faransaayii (Neewu Kaaleedoniyaa)',
        'fr_NE' => 'Afaan Faransaayii (Niijer)',
        'fr_PF' => 'Afaan Faransaayii (Polineeshiyaa Faransaay)',
        'fr_PM' => 'Afaan Faransaayii (Ql. Piyeeree fi Mikuyelon)',
        'fr_RE' => 'Afaan Faransaayii (Riyuuniyeen)',
        'fr_RW' => 'Afaan Faransaayii (Ruwwandaa)',
        'fr_SC' => 'Afaan Faransaayii (Siisheels)',
        'fr_SN' => 'Afaan Faransaayii (Senegaal)',
        'fr_SY' => 'Afaan Faransaayii (Sooriyaa)',
        'fr_TD' => 'Afaan Faransaayii (Chaad)',
        'fr_TG' => 'Afaan Faransaayii (Toogoo)',
        'fr_TN' => 'Afaan Faransaayii (Tuniiziyaa)',
        'fr_VU' => 'Afaan Faransaayii (Vanuwaatu)',
        'fr_WF' => 'Afaan Faransaayii (Waalis fi Futtuuna)',
        'fr_YT' => 'Afaan Faransaayii (Maayootee)',
        'fy' => 'Afaan Firisiyaani',
        'fy_NL' => 'Afaan Firisiyaani (Neezerlaand)',
        'ga' => 'Afaan Ayirishii',
        'ga_GB' => 'Afaan Ayirishii (United Kingdom)',
        'ga_IE' => 'Afaan Ayirishii (Ayeerlaand)',
        'gd' => 'Scots Gaelic',
        'gd_GB' => 'Scots Gaelic (United Kingdom)',
        'gl' => 'Afaan Galishii',
        'gl_ES' => 'Afaan Galishii (Ispeen)',
        'gu' => 'Afaan Gujarati',
        'gu_IN' => 'Afaan Gujarati (Hindii)',
        'ha' => 'Hawusaa',
        'ha_GH' => 'Hawusaa (Gaanaa)',
        'ha_NE' => 'Hawusaa (Niijer)',
        'ha_NG' => 'Hawusaa (Naayijeeriyaa)',
        'he' => 'Afaan Hebrew',
        'he_IL' => 'Afaan Hebrew (Israa’eel)',
        'hi' => 'Afaan Hindii',
        'hi_IN' => 'Afaan Hindii (Hindii)',
        'hi_Latn' => 'Afaan Hindii (Laatinii)',
        'hi_Latn_IN' => 'Afaan Hindii (Laatinii, Hindii)',
        'hr' => 'Afaan Croatian',
        'hr_BA' => 'Afaan Croatian (Bosiiniyaa fi Herzoogovinaa)',
        'hr_HR' => 'Afaan Croatian (Kirooshiyaa)',
        'hu' => 'Afaan Hangaari',
        'hu_HU' => 'Afaan Hangaari (Hangaarii)',
        'hy' => 'Armeeniyaa',
        'hy_AM' => 'Armeeniyaa (Armeeniyaa)',
        'ia' => 'Interlingua',
        'ia_001' => 'Interlingua (addunyaa)',
        'id' => 'Afaan Indoneziya',
        'id_ID' => 'Afaan Indoneziya (Indooneeshiyaa)',
        'is' => 'Ayiislandiffaa',
        'is_IS' => 'Ayiislandiffaa (Ayeslaand)',
        'it' => 'Afaan Xaaliyaani',
        'it_CH' => 'Afaan Xaaliyaani (Siwizerlaand)',
        'it_IT' => 'Afaan Xaaliyaani (Xaaliyaan)',
        'it_SM' => 'Afaan Xaaliyaani (Saan Mariinoo)',
        'it_VA' => 'Afaan Xaaliyaani (Vaatikaan Siitii)',
        'ja' => 'Afaan Japanii',
        'ja_JP' => 'Afaan Japanii (Jaappaan)',
        'jv' => 'Afaan Java',
        'jv_ID' => 'Afaan Java (Indooneeshiyaa)',
        'ka' => 'Afaan Georgian',
        'ka_GE' => 'Afaan Georgian (Joorjiyaa)',
        'kn' => 'Afaan Kannada',
        'kn_IN' => 'Afaan Kannada (Hindii)',
        'ko' => 'Afaan Korea',
        'ko_CN' => 'Afaan Korea (Chaayinaa)',
        'ko_KP' => 'Afaan Korea (Kooriyaa Kaaba)',
        'ko_KR' => 'Afaan Korea (Kooriyaa Kibbaa)',
        'lt' => 'Afaan Liituniyaa',
        'lt_LT' => 'Afaan Liituniyaa (Lutaaniyaa)',
        'lv' => 'Afaan Lativiyaa',
        'lv_LV' => 'Afaan Lativiyaa (Lativiyaa)',
        'mk' => 'Afaan Macedooniyaa',
        'mk_MK' => 'Afaan Macedooniyaa (Maqdooniyaa Kaabaa)',
        'ml' => 'Malayaalamiffaa',
        'ml_IN' => 'Malayaalamiffaa (Hindii)',
        'mr' => 'Afaan Maratii',
        'mr_IN' => 'Afaan Maratii (Hindii)',
        'ms' => 'Malaayiffaa',
        'ms_BN' => 'Malaayiffaa (Biruniyee)',
        'ms_ID' => 'Malaayiffaa (Indooneeshiyaa)',
        'ms_MY' => 'Malaayiffaa (Maleeshiyaa)',
        'ms_SG' => 'Malaayiffaa (Singaapoor)',
        'mt' => 'Afaan Maltesii',
        'mt_MT' => 'Afaan Maltesii (Maaltaa)',
        'my' => 'Burmeesee',
        'my_MM' => 'Burmeesee (Maayinaamar [Burma])',
        'ne' => 'Afaan Nepalii',
        'ne_IN' => 'Afaan Nepalii (Hindii)',
        'ne_NP' => 'Afaan Nepalii (Neeppal)',
        'nl' => 'Afaan Dachii',
        'nl_AW' => 'Afaan Dachii (Arubaa)',
        'nl_BE' => 'Afaan Dachii (Beeljiyeem)',
        'nl_BQ' => 'Afaan Dachii (Neezerlaandota Kariibaan)',
        'nl_CW' => 'Afaan Dachii (Kurakowaa)',
        'nl_NL' => 'Afaan Dachii (Neezerlaand)',
        'nl_SR' => 'Afaan Dachii (Suriname)',
        'nl_SX' => 'Afaan Dachii (Siint Maarteen)',
        'nn' => 'Afaan Norwegian',
        'nn_NO' => 'Afaan Norwegian (Noorwey)',
        'no' => 'Afaan Norweyii',
        'no_NO' => 'Afaan Norweyii (Noorwey)',
        'oc' => 'Afaan Occit',
        'oc_ES' => 'Afaan Occit (Ispeen)',
        'oc_FR' => 'Afaan Occit (Faransaay)',
        'om' => 'Oromoo',
        'om_ET' => 'Oromoo (Itoophiyaa)',
        'om_KE' => 'Oromoo (Keeniyaa)',
        'pa' => 'Afaan Punjabii',
        'pa_Arab' => 'Afaan Punjabii (Arabiffa)',
        'pa_Arab_PK' => 'Afaan Punjabii (Arabiffa, Paakistaan)',
        'pa_IN' => 'Afaan Punjabii (Hindii)',
        'pa_PK' => 'Afaan Punjabii (Paakistaan)',
        'pl' => 'Afaan Polandii',
        'pl_PL' => 'Afaan Polandii (Poolaand)',
        'pt' => 'Afaan Porchugaal',
        'pt_AO' => 'Afaan Porchugaal (Angoolaa)',
        'pt_BR' => 'Afaan Porchugaal (Biraazil)',
        'pt_CH' => 'Afaan Porchugaal (Siwizerlaand)',
        'pt_CV' => 'Afaan Porchugaal (Keeppi Vaardee)',
        'pt_GQ' => 'Afaan Porchugaal (Ikkuwaatooriyaal Giinii)',
        'pt_GW' => 'Afaan Porchugaal (Giinii-Bisaawoo)',
        'pt_LU' => 'Afaan Porchugaal (Luksembarg)',
        'pt_MO' => 'Afaan Porchugaal (Maka’oo SAR Chaayinaa)',
        'pt_MZ' => 'Afaan Porchugaal (Moozaambik)',
        'pt_PT' => 'Afaan Porchugaal (Poorchugaal)',
        'pt_ST' => 'Afaan Porchugaal (Sa’oo Toomee fi Prinsippee)',
        'pt_TL' => 'Afaan Porchugaal (Tiimoor-Leestee)',
        'ro' => 'Afaan Romaniyaa',
        'ro_MD' => 'Afaan Romaniyaa (Moldoovaa)',
        'ro_RO' => 'Afaan Romaniyaa (Roomaaniyaa)',
        'ru' => 'Afaan Rushiyaa',
        'ru_BY' => 'Afaan Rushiyaa (Beelaarus)',
        'ru_KG' => 'Afaan Rushiyaa (Kiyirigiyizistan)',
        'ru_KZ' => 'Afaan Rushiyaa (Kazakistaan)',
        'ru_MD' => 'Afaan Rushiyaa (Moldoovaa)',
        'ru_RU' => 'Afaan Rushiyaa (Raashiyaa)',
        'ru_UA' => 'Afaan Rushiyaa (Yuukireen)',
        'si' => 'Afaan Sinhalese',
        'si_LK' => 'Afaan Sinhalese (Siri Laankaa)',
        'sk' => 'Afaan Slovak',
        'sk_SK' => 'Afaan Slovak (Isloovaakiyaa)',
        'sl' => 'Afaan Islovaniyaa',
        'sl_SI' => 'Afaan Islovaniyaa (Islooveeniyaa)',
        'sq' => 'Afaan Albaniyaa',
        'sq_AL' => 'Afaan Albaniyaa (Albaaniyaa)',
        'sq_MK' => 'Afaan Albaniyaa (Maqdooniyaa Kaabaa)',
        'sr' => 'Afaan Serbiya',
        'sr_BA' => 'Afaan Serbiya (Bosiiniyaa fi Herzoogovinaa)',
        'sr_Cyrl' => 'Afaan Serbiya (Saayiriilik)',
        'sr_Cyrl_BA' => 'Afaan Serbiya (Saayiriilik, Bosiiniyaa fi Herzoogovinaa)',
        'sr_Cyrl_ME' => 'Afaan Serbiya (Saayiriilik, Montenegiroo)',
        'sr_Cyrl_RS' => 'Afaan Serbiya (Saayiriilik, Serbiyaa)',
        'sr_Latn' => 'Afaan Serbiya (Laatinii)',
        'sr_Latn_BA' => 'Afaan Serbiya (Laatinii, Bosiiniyaa fi Herzoogovinaa)',
        'sr_Latn_ME' => 'Afaan Serbiya (Laatinii, Montenegiroo)',
        'sr_Latn_RS' => 'Afaan Serbiya (Laatinii, Serbiyaa)',
        'sr_ME' => 'Afaan Serbiya (Montenegiroo)',
        'sr_RS' => 'Afaan Serbiya (Serbiyaa)',
        'su' => 'Afaan Sudaanii',
        'su_ID' => 'Afaan Sudaanii (Indooneeshiyaa)',
        'su_Latn' => 'Afaan Sudaanii (Laatinii)',
        'su_Latn_ID' => 'Afaan Sudaanii (Laatinii, Indooneeshiyaa)',
        'sv' => 'Afaan Suwidiin',
        'sv_AX' => 'Afaan Suwidiin (Odoloota Alaand)',
        'sv_FI' => 'Afaan Suwidiin (Fiinlaand)',
        'sv_SE' => 'Afaan Suwidiin (Siwiidin)',
        'sw' => 'Suwahilii',
        'sw_CD' => 'Suwahilii (Koongoo - Kinshaasaa)',
        'sw_KE' => 'Suwahilii (Keeniyaa)',
        'sw_TZ' => 'Suwahilii (Taanzaaniyaa)',
        'sw_UG' => 'Suwahilii (Ugaandaa)',
        'ta' => 'Afaan Tamilii',
        'ta_IN' => 'Afaan Tamilii (Hindii)',
        'ta_LK' => 'Afaan Tamilii (Siri Laankaa)',
        'ta_MY' => 'Afaan Tamilii (Maleeshiyaa)',
        'ta_SG' => 'Afaan Tamilii (Singaapoor)',
        'te' => 'Afaan Telugu',
        'te_IN' => 'Afaan Telugu (Hindii)',
        'th' => 'Afaan Tayii',
        'th_TH' => 'Afaan Tayii (Taayilaand)',
        'ti' => 'Afaan Tigiree',
        'ti_ER' => 'Afaan Tigiree (Eertiraa)',
        'ti_ET' => 'Afaan Tigiree (Itoophiyaa)',
        'tk' => 'Lammii Turkii',
        'tk_TM' => 'Lammii Turkii (Turkimenistaan)',
        'tr' => 'Afaan Turkii',
        'tr_CY' => 'Afaan Turkii (Qoophiroos)',
        'tr_TR' => 'Afaan Turkii (Tarkiye)',
        'uk' => 'Afaan Ukreenii',
        'uk_UA' => 'Afaan Ukreenii (Yuukireen)',
        'ur' => 'Afaan Urdu',
        'ur_IN' => 'Afaan Urdu (Hindii)',
        'ur_PK' => 'Afaan Urdu (Paakistaan)',
        'uz' => 'Afaan Uzbek',
        'uz_AF' => 'Afaan Uzbek (Afgaanistaan)',
        'uz_Arab' => 'Afaan Uzbek (Arabiffa)',
        'uz_Arab_AF' => 'Afaan Uzbek (Arabiffa, Afgaanistaan)',
        'uz_Cyrl' => 'Afaan Uzbek (Saayiriilik)',
        'uz_Cyrl_UZ' => 'Afaan Uzbek (Saayiriilik, Uzbeekistaan)',
        'uz_Latn' => 'Afaan Uzbek (Laatinii)',
        'uz_Latn_UZ' => 'Afaan Uzbek (Laatinii, Uzbeekistaan)',
        'uz_UZ' => 'Afaan Uzbek (Uzbeekistaan)',
        'vi' => 'Afaan Veetinam',
        'vi_VN' => 'Afaan Veetinam (Veetinaam)',
        'xh' => 'Afaan Xhosa',
        'xh_ZA' => 'Afaan Xhosa (Afrikaa Kibbaa)',
        'zh' => 'Chinese',
        'zh_CN' => 'Chinese (Chaayinaa)',
        'zh_HK' => 'Chinese (Hoong Koong SAR Chaayinaa)',
        'zh_Hans' => 'Chinese (Salphifame)',
        'zh_Hans_CN' => 'Chinese (Salphifame, Chaayinaa)',
        'zh_Hans_HK' => 'Chinese (Salphifame, Hoong Koong SAR Chaayinaa)',
        'zh_Hans_MO' => 'Chinese (Salphifame, Maka’oo SAR Chaayinaa)',
        'zh_Hans_MY' => 'Chinese (Salphifame, Maleeshiyaa)',
        'zh_Hans_SG' => 'Chinese (Salphifame, Singaapoor)',
        'zh_Hant' => 'Chinese (Kan Durii)',
        'zh_Hant_HK' => 'Chinese (Kan Durii, Hoong Koong SAR Chaayinaa)',
        'zh_Hant_MO' => 'Chinese (Kan Durii, Maka’oo SAR Chaayinaa)',
        'zh_Hant_MY' => 'Chinese (Kan Durii, Maleeshiyaa)',
        'zh_Hant_TW' => 'Chinese (Kan Durii, Taayiwwan)',
        'zh_MO' => 'Chinese (Maka’oo SAR Chaayinaa)',
        'zh_SG' => 'Chinese (Singaapoor)',
        'zh_TW' => 'Chinese (Taayiwwan)',
        'zu' => 'Afaan Zuulu',
        'zu_ZA' => 'Afaan Zuulu (Afrikaa Kibbaa)',
    ],
];
