<?php

return [
    'Names' => [
        'Adlm' => 'Atarama',
        'Arab' => 'Arapika',
        'Aran' => 'Nātarika',
        'Armn' => 'Āmeniana',
        'Beng' => 'Pāngara',
        'Bopo' => 'Papamawha',
        'Brai' => 'Tuhi Matapō',
        'Cakm' => 'Tiakamā',
        'Cans' => 'Ngā Kūoro o ngā Iwi Taketake o Kānata kua paiheretia',
        'Cher' => 'Terokī',
        'Cyrl' => 'Hīririki',
        'Deva' => 'Tewhangāngari',
        'Ethi' => 'Etiopika',
        'Geor' => 'Hōriana',
        'Grek' => 'Kariki',
        'Gujr' => 'Kutarāti',
        'Guru' => 'Kūmuki',
        'Hanb' => 'Hana me te Papamawha',
        'Hang' => 'Hāngū',
        'Hani' => 'Hana',
        'Hans' => 'Māmā',
        'Hant' => 'Tuku iho',
        'Hebr' => 'Hīperu',
        'Hira' => 'Hirakana',
        'Hrkt' => 'Kūoro Hapanihi',
        'Jamo' => 'Hamo',
        'Jpan' => 'Hapanihi',
        'Kana' => 'Katakana',
        'Khmr' => 'Kimēra',
        'Knda' => 'Kanāra',
        'Kore' => 'Kōreana',
        'Laoo' => 'Rao',
        'Latn' => 'Rātini',
        'Mlym' => 'Maraiārama',
        'Mong' => 'Mongōria',
        'Mtei' => 'Meitei Maeke',
        'Mymr' => 'Mienemā',
        'Nkoo' => 'Unukō',
        'Olck' => 'Ōtiki',
        'Orya' => 'Otia',
        'Rohg' => 'Hāniwhi',
        'Sinh' => 'Hināra',
        'Sund' => 'Hunanihi',
        'Syrc' => 'Hīriaka',
        'Taml' => 'Tamiera',
        'Telu' => 'Teruku',
        'Tfng' => 'Tiwhinā',
        'Thaa' => 'Tāna',
        'Thai' => 'Tai',
        'Tibt' => 'Tipete',
        'Vaii' => 'Wai',
        'Yiii' => 'Eī',
        'Zmth' => 'Reo Tohu Pāngarau',
        'Zsye' => 'Emohi',
        'Zsym' => 'Tohu',
        'Zxxx' => 'Tuhikore',
        'Zyyy' => 'Komona',
    ],
];
