<?php

return [
    'Names' => [
        'ace' => 'acehnés',
        'ady' => 'adigué',
        'arp' => 'arapaho',
        'ars' => 'árabe neyedí',
        'bas' => 'basa',
        'bax' => 'bamun',
        'bho' => 'bhoshpuri',
        'bla' => 'siksika',
        'bua' => 'buriat',
        'crj' => 'cree del sureste',
        'crl' => 'cree del noreste',
        'crr' => 'carolina algonquian',
        'dar' => 'darguin',
        'dum' => 'neerlandés medieval',
        'enm' => 'inglés medieval',
        'eu' => 'euskera',
        'frm' => 'francés medieval',
        'gan' => 'gan (China)',
        'gmh' => 'alemán de la alta edad media',
        'grc' => 'griego antiguo',
        'hak' => 'kejia (China)',
        'hax' => 'haida del sur',
        'hil' => 'hiligainón',
        'hsn' => 'xiang (China)',
        'ikt' => 'inuktitut del oeste de Canadá',
        'inh' => 'ingusetio',
        'kbd' => 'kabardiano',
        'kgp' => 'kaingang',
        'krc' => 'karachái bálkaro',
        'kum' => 'cumuco',
        'mga' => 'irlandés medieval',
        'nan' => 'min nan (Chino)',
        'nr' => 'ndebele meridional',
        'nso' => 'sotho septentrional',
        'ojb' => 'ojibwa del noroeste',
        'ojw' => 'ojibwa del oeste',
        'shu' => 'árabe chadiano',
        'slh' => 'lushootseed del sur',
        'ss' => 'siswati',
        'syr' => 'siriaco',
        'tce' => 'tutchone del sur',
        'wuu' => 'chino wu',
        'zgh' => 'tamazight marroquí estándar',
    ],
    'LocalizedNames' => [
        'sw_CD' => 'suajili del Congo',
    ],
];
