<?php

return [
    'Names' => [
        'af' => 'afrikans',
        'af_NA' => 'afrikans (Namibija)',
        'af_ZA' => 'afrikans (Južnoafrička Republika)',
        'ak' => 'akanski',
        'ak_GH' => 'akanski (Gana)',
        'am' => 'amharski',
        'am_ET' => 'amharski (Etiopija)',
        'ar' => 'arapski',
        'ar_001' => 'arapski (svet)',
        'ar_AE' => 'arapski (Ujedinjeni Arapski Emirati)',
        'ar_BH' => 'arapski (Bahrein)',
        'ar_DJ' => 'arapski (Džibuti)',
        'ar_DZ' => 'arapski (Alžir)',
        'ar_EG' => 'arapski (Egipat)',
        'ar_EH' => 'arapski (Zapadna Sahara)',
        'ar_ER' => 'arapski (Eritreja)',
        'ar_IL' => 'arapski (Izrael)',
        'ar_IQ' => 'arapski (Irak)',
        'ar_JO' => 'arapski (Jordan)',
        'ar_KM' => 'arapski (Komorska Ostrva)',
        'ar_KW' => 'arapski (Kuvajt)',
        'ar_LB' => 'arapski (Liban)',
        'ar_LY' => 'arapski (Libija)',
        'ar_MA' => 'arapski (Maroko)',
        'ar_MR' => 'arapski (Mauritanija)',
        'ar_OM' => 'arapski (Oman)',
        'ar_PS' => 'arapski (Palestinske teritorije)',
        'ar_QA' => 'arapski (Katar)',
        'ar_SA' => 'arapski (Saudijska Arabija)',
        'ar_SD' => 'arapski (Sudan)',
        'ar_SO' => 'arapski (Somalija)',
        'ar_SS' => 'arapski (Južni Sudan)',
        'ar_SY' => 'arapski (Sirija)',
        'ar_TD' => 'arapski (Čad)',
        'ar_TN' => 'arapski (Tunis)',
        'ar_YE' => 'arapski (Jemen)',
        'as' => 'asamski',
        'as_IN' => 'asamski (Indija)',
        'az' => 'azerbejdžanski',
        'az_AZ' => 'azerbejdžanski (Azerbejdžan)',
        'az_Cyrl' => 'azerbejdžanski (ćirilica)',
        'az_Cyrl_AZ' => 'azerbejdžanski (ćirilica, Azerbejdžan)',
        'az_Latn' => 'azerbejdžanski (latinica)',
        'az_Latn_AZ' => 'azerbejdžanski (latinica, Azerbejdžan)',
        'ba' => 'baškirski',
        'ba_RU' => 'baškirski (Rusija)',
        'be' => 'beloruski',
        'be_BY' => 'beloruski (Belorusija)',
        'bg' => 'bugarski',
        'bg_BG' => 'bugarski (Bugarska)',
        'bm' => 'bambara',
        'bm_ML' => 'bambara (Mali)',
        'bn' => 'bengalski',
        'bn_BD' => 'bengalski (Bangladeš)',
        'bn_IN' => 'bengalski (Indija)',
        'bo' => 'tibetanski',
        'bo_CN' => 'tibetanski (Kina)',
        'bo_IN' => 'tibetanski (Indija)',
        'br' => 'bretonski',
        'br_FR' => 'bretonski (Francuska)',
        'bs' => 'bosanski',
        'bs_BA' => 'bosanski (Bosna i Hercegovina)',
        'bs_Cyrl' => 'bosanski (ćirilica)',
        'bs_Cyrl_BA' => 'bosanski (ćirilica, Bosna i Hercegovina)',
        'bs_Latn' => 'bosanski (latinica)',
        'bs_Latn_BA' => 'bosanski (latinica, Bosna i Hercegovina)',
        'ca' => 'katalonski',
        'ca_AD' => 'katalonski (Andora)',
        'ca_ES' => 'katalonski (Španija)',
        'ca_FR' => 'katalonski (Francuska)',
        'ca_IT' => 'katalonski (Italija)',
        'ce' => 'čečenski',
        'ce_RU' => 'čečenski (Rusija)',
        'cs' => 'češki',
        'cs_CZ' => 'češki (Češka)',
        'cv' => 'čuvaški',
        'cv_RU' => 'čuvaški (Rusija)',
        'cy' => 'velški',
        'cy_GB' => 'velški (Ujedinjeno Kraljevstvo)',
        'da' => 'danski',
        'da_DK' => 'danski (Danska)',
        'da_GL' => 'danski (Grenland)',
        'de' => 'nemački',
        'de_AT' => 'nemački (Austrija)',
        'de_BE' => 'nemački (Belgija)',
        'de_CH' => 'nemački (Švajcarska)',
        'de_DE' => 'nemački (Nemačka)',
        'de_IT' => 'nemački (Italija)',
        'de_LI' => 'nemački (Lihtenštajn)',
        'de_LU' => 'nemački (Luksemburg)',
        'dz' => 'džonga',
        'dz_BT' => 'džonga (Butan)',
        'ee' => 'eve',
        'ee_GH' => 'eve (Gana)',
        'ee_TG' => 'eve (Togo)',
        'el' => 'grčki',
        'el_CY' => 'grčki (Kipar)',
        'el_GR' => 'grčki (Grčka)',
        'en' => 'engleski',
        'en_001' => 'engleski (svet)',
        'en_150' => 'engleski (Evropa)',
        'en_AE' => 'engleski (Ujedinjeni Arapski Emirati)',
        'en_AG' => 'engleski (Antigva i Barbuda)',
        'en_AI' => 'engleski (Angvila)',
        'en_AS' => 'engleski (Američka Samoa)',
        'en_AT' => 'engleski (Austrija)',
        'en_AU' => 'engleski (Australija)',
        'en_BB' => 'engleski (Barbados)',
        'en_BE' => 'engleski (Belgija)',
        'en_BI' => 'engleski (Burundi)',
        'en_BM' => 'engleski (Bermudi)',
        'en_BS' => 'engleski (Bahami)',
        'en_BW' => 'engleski (Bocvana)',
        'en_BZ' => 'engleski (Belize)',
        'en_CA' => 'engleski (Kanada)',
        'en_CC' => 'engleski (Kokosova [Kilingova] Ostrva)',
        'en_CH' => 'engleski (Švajcarska)',
        'en_CK' => 'engleski (Kukova Ostrva)',
        'en_CM' => 'engleski (Kamerun)',
        'en_CX' => 'engleski (Božićno Ostrvo)',
        'en_CY' => 'engleski (Kipar)',
        'en_CZ' => 'engleski (Češka)',
        'en_DE' => 'engleski (Nemačka)',
        'en_DK' => 'engleski (Danska)',
        'en_DM' => 'engleski (Dominika)',
        'en_EE' => 'engleski (Estonija)',
        'en_ER' => 'engleski (Eritreja)',
        'en_ES' => 'engleski (Španija)',
        'en_FI' => 'engleski (Finska)',
        'en_FJ' => 'engleski (Fidži)',
        'en_FK' => 'engleski (Foklandska Ostrva)',
        'en_FM' => 'engleski (Mikronezija)',
        'en_FR' => 'engleski (Francuska)',
        'en_GB' => 'engleski (Ujedinjeno Kraljevstvo)',
        'en_GD' => 'engleski (Grenada)',
        'en_GE' => 'engleski (Gruzija)',
        'en_GG' => 'engleski (Gernzi)',
        'en_GH' => 'engleski (Gana)',
        'en_GI' => 'engleski (Gibraltar)',
        'en_GM' => 'engleski (Gambija)',
        'en_GS' => 'engleski (Južna Džordžija i Južna Sendvička Ostrva)',
        'en_GU' => 'engleski (Guam)',
        'en_GY' => 'engleski (Gvajana)',
        'en_HK' => 'engleski (SAR Hongkong [Kina])',
        'en_HU' => 'engleski (Mađarska)',
        'en_ID' => 'engleski (Indonezija)',
        'en_IE' => 'engleski (Irska)',
        'en_IL' => 'engleski (Izrael)',
        'en_IM' => 'engleski (Ostrvo Man)',
        'en_IN' => 'engleski (Indija)',
        'en_IO' => 'engleski (Britanska teritorija Indijskog okeana)',
        'en_IT' => 'engleski (Italija)',
        'en_JE' => 'engleski (Džerzi)',
        'en_JM' => 'engleski (Jamajka)',
        'en_JP' => 'engleski (Japan)',
        'en_KE' => 'engleski (Kenija)',
        'en_KI' => 'engleski (Kiribati)',
        'en_KN' => 'engleski (Sent Kits i Nevis)',
        'en_KY' => 'engleski (Kajmanska Ostrva)',
        'en_LC' => 'engleski (Sveta Lucija)',
        'en_LR' => 'engleski (Liberija)',
        'en_LS' => 'engleski (Lesoto)',
        'en_LT' => 'engleski (Litvanija)',
        'en_LV' => 'engleski (Letonija)',
        'en_MG' => 'engleski (Madagaskar)',
        'en_MH' => 'engleski (Maršalska Ostrva)',
        'en_MO' => 'engleski (SAR Makao [Kina])',
        'en_MP' => 'engleski (Severna Marijanska Ostrva)',
        'en_MS' => 'engleski (Monserat)',
        'en_MT' => 'engleski (Malta)',
        'en_MU' => 'engleski (Mauricijus)',
        'en_MV' => 'engleski (Maldivi)',
        'en_MW' => 'engleski (Malavi)',
        'en_MY' => 'engleski (Malezija)',
        'en_NA' => 'engleski (Namibija)',
        'en_NF' => 'engleski (Ostrvo Norfok)',
        'en_NG' => 'engleski (Nigerija)',
        'en_NL' => 'engleski (Holandija)',
        'en_NO' => 'engleski (Norveška)',
        'en_NR' => 'engleski (Nauru)',
        'en_NU' => 'engleski (Niue)',
        'en_NZ' => 'engleski (Novi Zeland)',
        'en_PG' => 'engleski (Papua Nova Gvineja)',
        'en_PH' => 'engleski (Filipini)',
        'en_PK' => 'engleski (Pakistan)',
        'en_PL' => 'engleski (Poljska)',
        'en_PN' => 'engleski (Pitkern)',
        'en_PR' => 'engleski (Portoriko)',
        'en_PT' => 'engleski (Portugalija)',
        'en_PW' => 'engleski (Palau)',
        'en_RO' => 'engleski (Rumunija)',
        'en_RW' => 'engleski (Ruanda)',
        'en_SB' => 'engleski (Solomonska Ostrva)',
        'en_SC' => 'engleski (Sejšeli)',
        'en_SD' => 'engleski (Sudan)',
        'en_SE' => 'engleski (Švedska)',
        'en_SG' => 'engleski (Singapur)',
        'en_SH' => 'engleski (Sveta Jelena)',
        'en_SI' => 'engleski (Slovenija)',
        'en_SK' => 'engleski (Slovačka)',
        'en_SL' => 'engleski (Sijera Leone)',
        'en_SS' => 'engleski (Južni Sudan)',
        'en_SX' => 'engleski (Sveti Martin [Holandija])',
        'en_SZ' => 'engleski (Svazilend)',
        'en_TC' => 'engleski (Ostrva Turks i Kaikos)',
        'en_TK' => 'engleski (Tokelau)',
        'en_TO' => 'engleski (Tonga)',
        'en_TT' => 'engleski (Trinidad i Tobago)',
        'en_TV' => 'engleski (Tuvalu)',
        'en_TZ' => 'engleski (Tanzanija)',
        'en_UA' => 'engleski (Ukrajina)',
        'en_UG' => 'engleski (Uganda)',
        'en_UM' => 'engleski (Udaljena ostrva SAD)',
        'en_US' => 'engleski (Sjedinjene Države)',
        'en_VC' => 'engleski (Sent Vinsent i Grenadini)',
        'en_VG' => 'engleski (Britanska Devičanska Ostrva)',
        'en_VI' => 'engleski (Američka Devičanska Ostrva)',
        'en_VU' => 'engleski (Vanuatu)',
        'en_WS' => 'engleski (Samoa)',
        'en_ZA' => 'engleski (Južnoafrička Republika)',
        'en_ZM' => 'engleski (Zambija)',
        'en_ZW' => 'engleski (Zimbabve)',
        'eo' => 'esperanto',
        'eo_001' => 'esperanto (svet)',
        'es' => 'španski',
        'es_419' => 'španski (Latinska Amerika)',
        'es_AR' => 'španski (Argentina)',
        'es_BO' => 'španski (Bolivija)',
        'es_BR' => 'španski (Brazil)',
        'es_BZ' => 'španski (Belize)',
        'es_CL' => 'španski (Čile)',
        'es_CO' => 'španski (Kolumbija)',
        'es_CR' => 'španski (Kostarika)',
        'es_CU' => 'španski (Kuba)',
        'es_DO' => 'španski (Dominikanska Republika)',
        'es_EC' => 'španski (Ekvador)',
        'es_ES' => 'španski (Španija)',
        'es_GQ' => 'španski (Ekvatorijalna Gvineja)',
        'es_GT' => 'španski (Gvatemala)',
        'es_HN' => 'španski (Honduras)',
        'es_MX' => 'španski (Meksiko)',
        'es_NI' => 'španski (Nikaragva)',
        'es_PA' => 'španski (Panama)',
        'es_PE' => 'španski (Peru)',
        'es_PH' => 'španski (Filipini)',
        'es_PR' => 'španski (Portoriko)',
        'es_PY' => 'španski (Paragvaj)',
        'es_SV' => 'španski (Salvador)',
        'es_US' => 'španski (Sjedinjene Države)',
        'es_UY' => 'španski (Urugvaj)',
        'es_VE' => 'španski (Venecuela)',
        'et' => 'estonski',
        'et_EE' => 'estonski (Estonija)',
        'eu' => 'baskijski',
        'eu_ES' => 'baskijski (Španija)',
        'fa' => 'persijski',
        'fa_AF' => 'persijski (Avganistan)',
        'fa_IR' => 'persijski (Iran)',
        'ff' => 'fula',
        'ff_Adlm' => 'fula (adlam)',
        'ff_Adlm_BF' => 'fula (adlam, Burkina Faso)',
        'ff_Adlm_CM' => 'fula (adlam, Kamerun)',
        'ff_Adlm_GH' => 'fula (adlam, Gana)',
        'ff_Adlm_GM' => 'fula (adlam, Gambija)',
        'ff_Adlm_GN' => 'fula (adlam, Gvineja)',
        'ff_Adlm_GW' => 'fula (adlam, Gvineja-Bisao)',
        'ff_Adlm_LR' => 'fula (adlam, Liberija)',
        'ff_Adlm_MR' => 'fula (adlam, Mauritanija)',
        'ff_Adlm_NE' => 'fula (adlam, Niger)',
        'ff_Adlm_NG' => 'fula (adlam, Nigerija)',
        'ff_Adlm_SL' => 'fula (adlam, Sijera Leone)',
        'ff_Adlm_SN' => 'fula (adlam, Senegal)',
        'ff_CM' => 'fula (Kamerun)',
        'ff_GN' => 'fula (Gvineja)',
        'ff_Latn' => 'fula (latinica)',
        'ff_Latn_BF' => 'fula (latinica, Burkina Faso)',
        'ff_Latn_CM' => 'fula (latinica, Kamerun)',
        'ff_Latn_GH' => 'fula (latinica, Gana)',
        'ff_Latn_GM' => 'fula (latinica, Gambija)',
        'ff_Latn_GN' => 'fula (latinica, Gvineja)',
        'ff_Latn_GW' => 'fula (latinica, Gvineja-Bisao)',
        'ff_Latn_LR' => 'fula (latinica, Liberija)',
        'ff_Latn_MR' => 'fula (latinica, Mauritanija)',
        'ff_Latn_NE' => 'fula (latinica, Niger)',
        'ff_Latn_NG' => 'fula (latinica, Nigerija)',
        'ff_Latn_SL' => 'fula (latinica, Sijera Leone)',
        'ff_Latn_SN' => 'fula (latinica, Senegal)',
        'ff_MR' => 'fula (Mauritanija)',
        'ff_SN' => 'fula (Senegal)',
        'fi' => 'finski',
        'fi_FI' => 'finski (Finska)',
        'fo' => 'farski',
        'fo_DK' => 'farski (Danska)',
        'fo_FO' => 'farski (Farska Ostrva)',
        'fr' => 'francuski',
        'fr_BE' => 'francuski (Belgija)',
        'fr_BF' => 'francuski (Burkina Faso)',
        'fr_BI' => 'francuski (Burundi)',
        'fr_BJ' => 'francuski (Benin)',
        'fr_BL' => 'francuski (Sveti Bartolomej)',
        'fr_CA' => 'francuski (Kanada)',
        'fr_CD' => 'francuski (Kongo - Kinšasa)',
        'fr_CF' => 'francuski (Centralnoafrička Republika)',
        'fr_CG' => 'francuski (Kongo - Brazavil)',
        'fr_CH' => 'francuski (Švajcarska)',
        'fr_CI' => 'francuski (Obala Slonovače [Kot d’Ivoar])',
        'fr_CM' => 'francuski (Kamerun)',
        'fr_DJ' => 'francuski (Džibuti)',
        'fr_DZ' => 'francuski (Alžir)',
        'fr_FR' => 'francuski (Francuska)',
        'fr_GA' => 'francuski (Gabon)',
        'fr_GF' => 'francuski (Francuska Gvajana)',
        'fr_GN' => 'francuski (Gvineja)',
        'fr_GP' => 'francuski (Gvadelup)',
        'fr_GQ' => 'francuski (Ekvatorijalna Gvineja)',
        'fr_HT' => 'francuski (Haiti)',
        'fr_KM' => 'francuski (Komorska Ostrva)',
        'fr_LU' => 'francuski (Luksemburg)',
        'fr_MA' => 'francuski (Maroko)',
        'fr_MC' => 'francuski (Monako)',
        'fr_MF' => 'francuski (Sveti Martin [Francuska])',
        'fr_MG' => 'francuski (Madagaskar)',
        'fr_ML' => 'francuski (Mali)',
        'fr_MQ' => 'francuski (Martinik)',
        'fr_MR' => 'francuski (Mauritanija)',
        'fr_MU' => 'francuski (Mauricijus)',
        'fr_NC' => 'francuski (Nova Kaledonija)',
        'fr_NE' => 'francuski (Niger)',
        'fr_PF' => 'francuski (Francuska Polinezija)',
        'fr_PM' => 'francuski (Sen Pjer i Mikelon)',
        'fr_RE' => 'francuski (Reinion)',
        'fr_RW' => 'francuski (Ruanda)',
        'fr_SC' => 'francuski (Sejšeli)',
        'fr_SN' => 'francuski (Senegal)',
        'fr_SY' => 'francuski (Sirija)',
        'fr_TD' => 'francuski (Čad)',
        'fr_TG' => 'francuski (Togo)',
        'fr_TN' => 'francuski (Tunis)',
        'fr_VU' => 'francuski (Vanuatu)',
        'fr_WF' => 'francuski (Valis i Futuna)',
        'fr_YT' => 'francuski (Majot)',
        'fy' => 'zapadni frizijski',
        'fy_NL' => 'zapadni frizijski (Holandija)',
        'ga' => 'irski',
        'ga_GB' => 'irski (Ujedinjeno Kraljevstvo)',
        'ga_IE' => 'irski (Irska)',
        'gd' => 'škotski gelski',
        'gd_GB' => 'škotski gelski (Ujedinjeno Kraljevstvo)',
        'gl' => 'galicijski',
        'gl_ES' => 'galicijski (Španija)',
        'gu' => 'gudžarati',
        'gu_IN' => 'gudžarati (Indija)',
        'gv' => 'manks',
        'gv_IM' => 'manks (Ostrvo Man)',
        'ha' => 'hausa',
        'ha_GH' => 'hausa (Gana)',
        'ha_NE' => 'hausa (Niger)',
        'ha_NG' => 'hausa (Nigerija)',
        'he' => 'hebrejski',
        'he_IL' => 'hebrejski (Izrael)',
        'hi' => 'hindi',
        'hi_IN' => 'hindi (Indija)',
        'hi_Latn' => 'hindi (latinica)',
        'hi_Latn_IN' => 'hindi (latinica, Indija)',
        'hr' => 'hrvatski',
        'hr_BA' => 'hrvatski (Bosna i Hercegovina)',
        'hr_HR' => 'hrvatski (Hrvatska)',
        'hu' => 'mađarski',
        'hu_HU' => 'mađarski (Mađarska)',
        'hy' => 'jermenski',
        'hy_AM' => 'jermenski (Jermenija)',
        'ia' => 'interlingva',
        'ia_001' => 'interlingva (svet)',
        'id' => 'indonežanski',
        'id_ID' => 'indonežanski (Indonezija)',
        'ie' => 'interlingve',
        'ie_EE' => 'interlingve (Estonija)',
        'ig' => 'igbo',
        'ig_NG' => 'igbo (Nigerija)',
        'ii' => 'sečuanski ji',
        'ii_CN' => 'sečuanski ji (Kina)',
        'is' => 'islandski',
        'is_IS' => 'islandski (Island)',
        'it' => 'italijanski',
        'it_CH' => 'italijanski (Švajcarska)',
        'it_IT' => 'italijanski (Italija)',
        'it_SM' => 'italijanski (San Marino)',
        'it_VA' => 'italijanski (Vatikan)',
        'ja' => 'japanski',
        'ja_JP' => 'japanski (Japan)',
        'jv' => 'javanski',
        'jv_ID' => 'javanski (Indonezija)',
        'ka' => 'gruzijski',
        'ka_GE' => 'gruzijski (Gruzija)',
        'ki' => 'kikuju',
        'ki_KE' => 'kikuju (Kenija)',
        'kk' => 'kazaški',
        'kk_Arab' => 'kazaški (arapsko pismo)',
        'kk_Arab_CN' => 'kazaški (arapsko pismo, Kina)',
        'kk_Cyrl' => 'kazaški (ćirilica)',
        'kk_Cyrl_KZ' => 'kazaški (ćirilica, Kazahstan)',
        'kk_KZ' => 'kazaški (Kazahstan)',
        'kl' => 'grenlandski',
        'kl_GL' => 'grenlandski (Grenland)',
        'km' => 'kmerski',
        'km_KH' => 'kmerski (Kambodža)',
        'kn' => 'kanada',
        'kn_IN' => 'kanada (Indija)',
        'ko' => 'korejski',
        'ko_CN' => 'korejski (Kina)',
        'ko_KP' => 'korejski (Severna Koreja)',
        'ko_KR' => 'korejski (Južna Koreja)',
        'ks' => 'kašmirski',
        'ks_Arab' => 'kašmirski (arapsko pismo)',
        'ks_Arab_IN' => 'kašmirski (arapsko pismo, Indija)',
        'ks_Deva' => 'kašmirski (devanagari)',
        'ks_Deva_IN' => 'kašmirski (devanagari, Indija)',
        'ks_IN' => 'kašmirski (Indija)',
        'ku' => 'kurdski',
        'ku_Latn' => 'kurdski (latinica)',
        'ku_Latn_IQ' => 'kurdski (latinica, Irak)',
        'ku_Latn_SY' => 'kurdski (latinica, Sirija)',
        'ku_Latn_TR' => 'kurdski (latinica, Turska)',
        'ku_SY' => 'kurdski (Sirija)',
        'ku_TR' => 'kurdski (Turska)',
        'kw' => 'kornvolski',
        'kw_GB' => 'kornvolski (Ujedinjeno Kraljevstvo)',
        'ky' => 'kirgiski',
        'ky_KG' => 'kirgiski (Kirgistan)',
        'lb' => 'luksemburški',
        'lb_LU' => 'luksemburški (Luksemburg)',
        'lg' => 'ganda',
        'lg_UG' => 'ganda (Uganda)',
        'ln' => 'lingala',
        'ln_AO' => 'lingala (Angola)',
        'ln_CD' => 'lingala (Kongo - Kinšasa)',
        'ln_CF' => 'lingala (Centralnoafrička Republika)',
        'ln_CG' => 'lingala (Kongo - Brazavil)',
        'lo' => 'laoski',
        'lo_LA' => 'laoski (Laos)',
        'lt' => 'litvanski',
        'lt_LT' => 'litvanski (Litvanija)',
        'lu' => 'luba-katanga',
        'lu_CD' => 'luba-katanga (Kongo - Kinšasa)',
        'lv' => 'letonski',
        'lv_LV' => 'letonski (Letonija)',
        'mg' => 'malgaški',
        'mg_MG' => 'malgaški (Madagaskar)',
        'mi' => 'maorski',
        'mi_NZ' => 'maorski (Novi Zeland)',
        'mk' => 'makedonski',
        'mk_MK' => 'makedonski (Severna Makedonija)',
        'ml' => 'malajalam',
        'ml_IN' => 'malajalam (Indija)',
        'mn' => 'mongolski',
        'mn_MN' => 'mongolski (Mongolija)',
        'mr' => 'marati',
        'mr_IN' => 'marati (Indija)',
        'ms' => 'malajski',
        'ms_BN' => 'malajski (Brunej)',
        'ms_ID' => 'malajski (Indonezija)',
        'ms_MY' => 'malajski (Malezija)',
        'ms_SG' => 'malajski (Singapur)',
        'mt' => 'malteški',
        'mt_MT' => 'malteški (Malta)',
        'my' => 'burmanski',
        'my_MM' => 'burmanski (Mijanmar [Burma])',
        'nb' => 'norveški bukmol',
        'nb_NO' => 'norveški bukmol (Norveška)',
        'nb_SJ' => 'norveški bukmol (Svalbard i Jan Majen)',
        'nd' => 'severni ndebele',
        'nd_ZW' => 'severni ndebele (Zimbabve)',
        'ne' => 'nepalski',
        'ne_IN' => 'nepalski (Indija)',
        'ne_NP' => 'nepalski (Nepal)',
        'nl' => 'holandski',
        'nl_AW' => 'holandski (Aruba)',
        'nl_BE' => 'holandski (Belgija)',
        'nl_BQ' => 'holandski (Karipska Holandija)',
        'nl_CW' => 'holandski (Kurasao)',
        'nl_NL' => 'holandski (Holandija)',
        'nl_SR' => 'holandski (Surinam)',
        'nl_SX' => 'holandski (Sveti Martin [Holandija])',
        'nn' => 'norveški ninorsk',
        'nn_NO' => 'norveški ninorsk (Norveška)',
        'no' => 'norveški',
        'no_NO' => 'norveški (Norveška)',
        'oc' => 'oksitanski',
        'oc_ES' => 'oksitanski (Španija)',
        'oc_FR' => 'oksitanski (Francuska)',
        'om' => 'oromo',
        'om_ET' => 'oromo (Etiopija)',
        'om_KE' => 'oromo (Kenija)',
        'or' => 'odija',
        'or_IN' => 'odija (Indija)',
        'os' => 'osetinski',
        'os_GE' => 'osetinski (Gruzija)',
        'os_RU' => 'osetinski (Rusija)',
        'pa' => 'pendžapski',
        'pa_Arab' => 'pendžapski (arapsko pismo)',
        'pa_Arab_PK' => 'pendžapski (arapsko pismo, Pakistan)',
        'pa_Guru' => 'pendžapski (gurmuki pismo)',
        'pa_Guru_IN' => 'pendžapski (gurmuki pismo, Indija)',
        'pa_IN' => 'pendžapski (Indija)',
        'pa_PK' => 'pendžapski (Pakistan)',
        'pl' => 'poljski',
        'pl_PL' => 'poljski (Poljska)',
        'ps' => 'paštunski',
        'ps_AF' => 'paštunski (Avganistan)',
        'ps_PK' => 'paštunski (Pakistan)',
        'pt' => 'portugalski',
        'pt_AO' => 'portugalski (Angola)',
        'pt_BR' => 'portugalski (Brazil)',
        'pt_CH' => 'portugalski (Švajcarska)',
        'pt_CV' => 'portugalski (Zelenortska Ostrva)',
        'pt_GQ' => 'portugalski (Ekvatorijalna Gvineja)',
        'pt_GW' => 'portugalski (Gvineja-Bisao)',
        'pt_LU' => 'portugalski (Luksemburg)',
        'pt_MO' => 'portugalski (SAR Makao [Kina])',
        'pt_MZ' => 'portugalski (Mozambik)',
        'pt_PT' => 'portugalski (Portugalija)',
        'pt_ST' => 'portugalski (Sao Tome i Principe)',
        'pt_TL' => 'portugalski (Timor-Leste [Istočni Timor])',
        'qu' => 'kečua',
        'qu_BO' => 'kečua (Bolivija)',
        'qu_EC' => 'kečua (Ekvador)',
        'qu_PE' => 'kečua (Peru)',
        'rm' => 'romanš',
        'rm_CH' => 'romanš (Švajcarska)',
        'rn' => 'kirundi',
        'rn_BI' => 'kirundi (Burundi)',
        'ro' => 'rumunski',
        'ro_MD' => 'rumunski (Moldavija)',
        'ro_RO' => 'rumunski (Rumunija)',
        'ru' => 'ruski',
        'ru_BY' => 'ruski (Belorusija)',
        'ru_KG' => 'ruski (Kirgistan)',
        'ru_KZ' => 'ruski (Kazahstan)',
        'ru_MD' => 'ruski (Moldavija)',
        'ru_RU' => 'ruski (Rusija)',
        'ru_UA' => 'ruski (Ukrajina)',
        'rw' => 'kinjaruanda',
        'rw_RW' => 'kinjaruanda (Ruanda)',
        'sa' => 'sanskrit',
        'sa_IN' => 'sanskrit (Indija)',
        'sc' => 'sardinski',
        'sc_IT' => 'sardinski (Italija)',
        'sd' => 'sindi',
        'sd_Arab' => 'sindi (arapsko pismo)',
        'sd_Arab_PK' => 'sindi (arapsko pismo, Pakistan)',
        'sd_Deva' => 'sindi (devanagari)',
        'sd_Deva_IN' => 'sindi (devanagari, Indija)',
        'sd_IN' => 'sindi (Indija)',
        'sd_PK' => 'sindi (Pakistan)',
        'se' => 'severni sami',
        'se_FI' => 'severni sami (Finska)',
        'se_NO' => 'severni sami (Norveška)',
        'se_SE' => 'severni sami (Švedska)',
        'sg' => 'sango',
        'sg_CF' => 'sango (Centralnoafrička Republika)',
        'sh' => 'srpskohrvatski',
        'sh_BA' => 'srpskohrvatski (Bosna i Hercegovina)',
        'si' => 'sinhaleški',
        'si_LK' => 'sinhaleški (Šri Lanka)',
        'sk' => 'slovački',
        'sk_SK' => 'slovački (Slovačka)',
        'sl' => 'slovenački',
        'sl_SI' => 'slovenački (Slovenija)',
        'sn' => 'šona',
        'sn_ZW' => 'šona (Zimbabve)',
        'so' => 'somalski',
        'so_DJ' => 'somalski (Džibuti)',
        'so_ET' => 'somalski (Etiopija)',
        'so_KE' => 'somalski (Kenija)',
        'so_SO' => 'somalski (Somalija)',
        'sq' => 'albanski',
        'sq_AL' => 'albanski (Albanija)',
        'sq_MK' => 'albanski (Severna Makedonija)',
        'sr' => 'srpski',
        'sr_BA' => 'srpski (Bosna i Hercegovina)',
        'sr_Cyrl' => 'srpski (ćirilica)',
        'sr_Cyrl_BA' => 'srpski (ćirilica, Bosna i Hercegovina)',
        'sr_Cyrl_ME' => 'srpski (ćirilica, Crna Gora)',
        'sr_Cyrl_RS' => 'srpski (ćirilica, Srbija)',
        'sr_Latn' => 'srpski (latinica)',
        'sr_Latn_BA' => 'srpski (latinica, Bosna i Hercegovina)',
        'sr_Latn_ME' => 'srpski (latinica, Crna Gora)',
        'sr_Latn_RS' => 'srpski (latinica, Srbija)',
        'sr_ME' => 'srpski (Crna Gora)',
        'sr_RS' => 'srpski (Srbija)',
        'st' => 'sesoto',
        'st_LS' => 'sesoto (Lesoto)',
        'st_ZA' => 'sesoto (Južnoafrička Republika)',
        'su' => 'sundski',
        'su_ID' => 'sundski (Indonezija)',
        'su_Latn' => 'sundski (latinica)',
        'su_Latn_ID' => 'sundski (latinica, Indonezija)',
        'sv' => 'švedski',
        'sv_AX' => 'švedski (Olandska Ostrva)',
        'sv_FI' => 'švedski (Finska)',
        'sv_SE' => 'švedski (Švedska)',
        'sw' => 'svahili',
        'sw_CD' => 'svahili (Kongo - Kinšasa)',
        'sw_KE' => 'svahili (Kenija)',
        'sw_TZ' => 'svahili (Tanzanija)',
        'sw_UG' => 'svahili (Uganda)',
        'ta' => 'tamilski',
        'ta_IN' => 'tamilski (Indija)',
        'ta_LK' => 'tamilski (Šri Lanka)',
        'ta_MY' => 'tamilski (Malezija)',
        'ta_SG' => 'tamilski (Singapur)',
        'te' => 'telugu',
        'te_IN' => 'telugu (Indija)',
        'tg' => 'tadžički',
        'tg_TJ' => 'tadžički (Tadžikistan)',
        'th' => 'tajski',
        'th_TH' => 'tajski (Tajland)',
        'ti' => 'tigrinja',
        'ti_ER' => 'tigrinja (Eritreja)',
        'ti_ET' => 'tigrinja (Etiopija)',
        'tk' => 'turkmenski',
        'tk_TM' => 'turkmenski (Turkmenistan)',
        'tl' => 'tagalog',
        'tl_PH' => 'tagalog (Filipini)',
        'tn' => 'cvana',
        'tn_BW' => 'cvana (Bocvana)',
        'tn_ZA' => 'cvana (Južnoafrička Republika)',
        'to' => 'tonganski',
        'to_TO' => 'tonganski (Tonga)',
        'tr' => 'turski',
        'tr_CY' => 'turski (Kipar)',
        'tr_TR' => 'turski (Turska)',
        'tt' => 'tatarski',
        'tt_RU' => 'tatarski (Rusija)',
        'ug' => 'ujgurski',
        'ug_CN' => 'ujgurski (Kina)',
        'uk' => 'ukrajinski',
        'uk_UA' => 'ukrajinski (Ukrajina)',
        'ur' => 'urdu',
        'ur_IN' => 'urdu (Indija)',
        'ur_PK' => 'urdu (Pakistan)',
        'uz' => 'uzbečki',
        'uz_AF' => 'uzbečki (Avganistan)',
        'uz_Arab' => 'uzbečki (arapsko pismo)',
        'uz_Arab_AF' => 'uzbečki (arapsko pismo, Avganistan)',
        'uz_Cyrl' => 'uzbečki (ćirilica)',
        'uz_Cyrl_UZ' => 'uzbečki (ćirilica, Uzbekistan)',
        'uz_Latn' => 'uzbečki (latinica)',
        'uz_Latn_UZ' => 'uzbečki (latinica, Uzbekistan)',
        'uz_UZ' => 'uzbečki (Uzbekistan)',
        'vi' => 'vijetnamski',
        'vi_VN' => 'vijetnamski (Vijetnam)',
        'wo' => 'volof',
        'wo_SN' => 'volof (Senegal)',
        'xh' => 'kosa',
        'xh_ZA' => 'kosa (Južnoafrička Republika)',
        'yi' => 'jidiš',
        'yi_UA' => 'jidiš (Ukrajina)',
        'yo' => 'joruba',
        'yo_BJ' => 'joruba (Benin)',
        'yo_NG' => 'joruba (Nigerija)',
        'za' => 'džuanški',
        'za_CN' => 'džuanški (Kina)',
        'zh' => 'kineski',
        'zh_CN' => 'kineski (Kina)',
        'zh_HK' => 'kineski (SAR Hongkong [Kina])',
        'zh_Hans' => 'kineski (pojednostavljeno kinesko pismo)',
        'zh_Hans_CN' => 'kineski (pojednostavljeno kinesko pismo, Kina)',
        'zh_Hans_HK' => 'kineski (pojednostavljeno kinesko pismo, SAR Hongkong [Kina])',
        'zh_Hans_MO' => 'kineski (pojednostavljeno kinesko pismo, SAR Makao [Kina])',
        'zh_Hans_MY' => 'kineski (pojednostavljeno kinesko pismo, Malezija)',
        'zh_Hans_SG' => 'kineski (pojednostavljeno kinesko pismo, Singapur)',
        'zh_Hant' => 'kineski (tradicionalno kinesko pismo)',
        'zh_Hant_HK' => 'kineski (tradicionalno kinesko pismo, SAR Hongkong [Kina])',
        'zh_Hant_MO' => 'kineski (tradicionalno kinesko pismo, SAR Makao [Kina])',
        'zh_Hant_MY' => 'kineski (tradicionalno kinesko pismo, Malezija)',
        'zh_Hant_TW' => 'kineski (tradicionalno kinesko pismo, Tajvan)',
        'zh_MO' => 'kineski (SAR Makao [Kina])',
        'zh_SG' => 'kineski (Singapur)',
        'zh_TW' => 'kineski (Tajvan)',
        'zu' => 'zulu',
        'zu_ZA' => 'zulu (Južnoafrička Republika)',
    ],
];
