<?php

return [
    'Names' => [
        'Africa/Abidjan' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Abidjan་)',
        'Africa/Accra' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Accra་)',
        'Africa/Addis_Ababa' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Addis Ababa་)',
        'Africa/Algiers' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Algiers་)',
        'Africa/Asmera' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Asmara་)',
        'Africa/Asmara' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Asmara་)',
        'Africa/Bamako' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Bamako་)',
        'Africa/Bangui' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Bangui་)',
        'Africa/Banjul' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Banjul་)',
        'Africa/Bissau' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Bissau་)',
        'Africa/Blantyre' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Blantyre་)',
        'Africa/Brazzaville' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Brazzaville་)',
        'Africa/Bujumbura' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Bujumbura་)',
        'Africa/Cairo' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཀཱའི་རོ་)',
        'Africa/Casablanca' => 'ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Casablanca་)',
        'Africa/Ceuta' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Ceuta་)',
        'Africa/Conakry' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Conakry་)',
        'Africa/Dakar' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (ཌཱ་ཀར་)',
        'Africa/Dar_es_Salaam' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Dar es Salaam་)',
        'Africa/Djibouti' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Djibouti་)',
        'Africa/Douala' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Douala་)',
        'Africa/El_Aaiun' => 'ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (El Aaiún་)',
        'Africa/Freetown' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Freetown་)',
        'Africa/Gaborone' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Gaborone་)',
        'Africa/Harare' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Harare་)',
        'Africa/Johannesburg' => 'ལྷོ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Johannesburg་)',
        'Africa/Juba' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Juba་)',
        'Africa/Kampala' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Kampala་)',
        'Africa/Khartoum' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (ཁཱར་ཊུམ་)',
        'Africa/Kigali' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Kigali་)',
        'Africa/Kinshasa' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Kinshasa་)',
        'Africa/Lagos' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Lagos་)',
        'Africa/Libreville' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Libreville་)',
        'Africa/Lome' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (ལོ་མེ་)',
        'Africa/Luanda' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Luanda་)',
        'Africa/Lubumbashi' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Lubumbashi་)',
        'Africa/Lusaka' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Lusaka་)',
        'Africa/Malabo' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Malabo་)',
        'Africa/Maputo' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Maputo་)',
        'Africa/Maseru' => 'ལྷོ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Maseru་)',
        'Africa/Mbabane' => 'ལྷོ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Mbabane་)',
        'Africa/Mogadishu' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Mogadishu་)',
        'Africa/Monrovia' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Monrovia་)',
        'Africa/Nairobi' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Nairobi་)',
        'Africa/Ndjamena' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (N’Djamena་)',
        'Africa/Niamey' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Niamey་)',
        'Africa/Nouakchott' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Nouakchott་)',
        'Africa/Ouagadougou' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Ouagadougou་)',
        'Africa/Porto-Novo' => 'ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Porto-Novo་)',
        'Africa/Sao_Tome' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (São Tomé་)',
        'Africa/Tripoli' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཏྲི་པོ་ལི་)',
        'Africa/Tunis' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཊུ་ནིས྄་)',
        'Africa/Windhoek' => 'དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Windhoek་)',
        'America/Adak' => 'ཧ་ཝའི་-ཨེ་ལིའུ་ཤེན་ཆུ་ཚོད། (Adak་)',
        'America/Anchorage' => 'ཨ་ལསི་ཀ་ཆུ་ཚོད། (Anchorage་)',
        'America/Anguilla' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Anguilla་)',
        'America/Antigua' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (ཨན་ཊི་གུ་ཝ་་)',
        'America/Araguaina' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (Araguaína་)',
        'America/Argentina/La_Rioja' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (La Rioja་)',
        'America/Argentina/Rio_Gallegos' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Río Gallegos་)',
        'America/Argentina/Salta' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Salta་)',
        'America/Argentina/San_Juan' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (San Juan་)',
        'America/Argentina/San_Luis' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (San Luis་)',
        'America/Argentina/Tucuman' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Tucumán་)',
        'America/Argentina/Ushuaia' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Ushuaia་)',
        'America/Aruba' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Aruba་)',
        'America/Asuncion' => 'པ་ར་གུ་ཝའི་ཆུ་ཚོད། (Asunción་)',
        'America/Bahia' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (Bahia་)',
        'America/Bahia_Banderas' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Bahía de Banderas་)',
        'America/Barbados' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (བྷར་བེ་ཌོས་)',
        'America/Belem' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (Belém་)',
        'America/Belize' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (བྷེ་ལིཛ་)',
        'America/Blanc-Sablon' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Blanc-Sablon་)',
        'America/Boa_Vista' => 'ཨེ་མ་ཛཱོན་ཆུ་ཚོད། (Boa Vista་)',
        'America/Bogota' => 'ཀོ་ལོམ་བྷི་ཡ་ཆུ་ཚོད། (བྷོ་ག་ཊ་)',
        'America/Boise' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (Boise་)',
        'America/Buenos_Aires' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Buenos Aires་)',
        'America/Argentina/Buenos_Aires' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Buenos Aires་)',
        'America/Cambridge_Bay' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (ཀེམ་བིརིདཆ་ བའེ་)',
        'America/Campo_Grande' => 'ཨེ་མ་ཛཱོན་ཆུ་ཚོད། (Campo Grande་)',
        'America/Cancun' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Cancún་)',
        'America/Caracas' => 'བེ་ནི་ཛུ་ཝེ་ལ་ཆུ་ཚོད། (Caracas་)',
        'America/Catamarca' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Catamarca་)',
        'America/Argentina/Catamarca' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Catamarca་)',
        'America/Cayenne' => 'ཕིརེནཅ་གི་ཡ་ན་ཆུ་ཚོད། (Cayenne་)',
        'America/Cayman' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Cayman་)',
        'America/Chicago' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (ཅི་ཀཱ་གོ་)',
        'America/Chihuahua' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Chihuahua་)',
        'America/Ciudad_Juarez' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (Ciudad Juárez་)',
        'America/Coral_Harbour' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (ཨ་ཏི་ཀོ་ཀཱན་)',
        'America/Atikokan' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (ཨ་ཏི་ཀོ་ཀཱན་)',
        'America/Cordoba' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Córdoba་)',
        'America/Argentina/Cordoba' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Córdoba་)',
        'America/Costa_Rica' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (ཀོས་ཊ་རི་ཀ་)',
        'America/Coyhaique' => 'ཅི་ལི་ཆུ་ཚོད། (Coyhaique་)',
        'America/Creston' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (Creston་)',
        'America/Cuiaba' => 'ཨེ་མ་ཛཱོན་ཆུ་ཚོད། (Cuiabá་)',
        'America/Curacao' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Curaçao་)',
        'America/Danmarkshavn' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Danmarkshavn་)',
        'America/Dawson' => 'ཀེ་ན་ཌ་ཆུ་ཚོད།། (དའུ་སཱོན་)',
        'America/Dawson_Creek' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (དའུ་སཱོན་ ཀིརིཀ་)',
        'America/Denver' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (Denver་)',
        'America/Detroit' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Detroit་)',
        'America/Dominica' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (ཌོ་མི་ནི་ཀ་)',
        'America/Edmonton' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (ཨེཌ་མཱོན་ཊོན་)',
        'America/Eirunepe' => 'བྲ་ཛིལ་ཆུ་ཚོད།། (Eirunepé་)',
        'America/El_Salvador' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (ཨེལ་ སཱལ་བ་ཌོ་)',
        'America/Fort_Nelson' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (Fort Nelson་)',
        'America/Fortaleza' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (Fortaleza་)',
        'America/Glace_Bay' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Glace Bay་)',
        'America/Godthab' => 'གིརཱིན་ལནཌ྄་ཆུ་ཚོད།། (Nuuk་)',
        'America/Nuuk' => 'གིརཱིན་ལནཌ྄་ཆུ་ཚོད།། (Nuuk་)',
        'America/Goose_Bay' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (གཱུསི་ བའེ་)',
        'America/Grand_Turk' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Grand Turk་)',
        'America/Grenada' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Grenada་)',
        'America/Guadeloupe' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Guadeloupe་)',
        'America/Guatemala' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (གྷོ་ཊ་མ་ལ་)',
        'America/Guayaquil' => 'ཨེ་ཀུ་ཌཽ་ཆུ་ཚོད། (Guayaquil་)',
        'America/Guyana' => 'གུ་ཡ་ན་ཆུ་ཚོད། (Guyana་)',
        'America/Halifax' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (ཧ་ལི་ཕེགསི་)',
        'America/Havana' => 'ཀིའུ་བྷ་ཆུ་ཚོད། (Havana་)',
        'America/Hermosillo' => 'མེཀ་སི་ཀོ་ཆུ་ཚོད།། (Hermosillo་)',
        'America/Indiana/Knox' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Knox, Indiana་)',
        'America/Indiana/Marengo' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Marengo, Indiana་)',
        'America/Indiana/Petersburg' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Petersburg, Indiana་)',
        'America/Indiana/Tell_City' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Tell City, Indiana་)',
        'America/Indiana/Vevay' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Vevay, Indiana་)',
        'America/Indiana/Vincennes' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Vincennes, Indiana་)',
        'America/Indiana/Winamac' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Winamac, Indiana་)',
        'America/Indianapolis' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Indianapolis་)',
        'America/Indiana/Indianapolis' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Indianapolis་)',
        'America/Inuvik' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (ཨི་ནུ་ཝིཀ་)',
        'America/Iqaluit' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Iqaluit་)',
        'America/Jamaica' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Jamaica་)',
        'America/Jujuy' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Jujuy་)',
        'America/Argentina/Jujuy' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Jujuy་)',
        'America/Juneau' => 'ཨ་ལསི་ཀ་ཆུ་ཚོད། (Juneau་)',
        'America/Kentucky/Monticello' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Monticello, Kentucky་)',
        'America/Kralendijk' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Kralendijk་)',
        'America/La_Paz' => 'བྷོ་ལི་བི་ཡ་ཆུ་ཚོད། (ལ་པཱཛ྄་)',
        'America/Lima' => 'པ་རུ་ཆུ་ཚོད། (Lima་)',
        'America/Los_Angeles' => 'བྱང་ཨ་མི་རི་ཀ་པེ་སི་ཕིག་ཆུ་ཚོད། (Los Angeles་)',
        'America/Louisville' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Louisville་)',
        'America/Kentucky/Louisville' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Louisville་)',
        'America/Lower_Princes' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Lower Prince’s Quarter་)',
        'America/Maceio' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (Maceió་)',
        'America/Managua' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Managua་)',
        'America/Manaus' => 'ཨེ་མ་ཛཱོན་ཆུ་ཚོད། (Manaus་)',
        'America/Marigot' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Marigot་)',
        'America/Martinique' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Martinique་)',
        'America/Matamoros' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Matamoros་)',
        'America/Mazatlan' => 'མེཀ་སི་ཀོ་ཆུ་ཚོད།། (Mazatlán་)',
        'America/Mendoza' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Mendoza་)',
        'America/Argentina/Mendoza' => 'ཨར་ཇེན་ཊི་ན་ཆུ་ཚོད། (Mendoza་)',
        'America/Menominee' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Menominee་)',
        'America/Merida' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Mérida་)',
        'America/Metlakatla' => 'ཨ་ལསི་ཀ་ཆུ་ཚོད། (Metlakatla་)',
        'America/Mexico_City' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (མེཀ་སི་ཀོ་ སི་ཊི་)',
        'America/Miquelon' => 'པའི་རི་དང་མི་ཀི་ལཱོན་ཆུ་ཚོད། (Saint-Pierre་)',
        'America/Moncton' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (མཱོངཀ་ཊོན་)',
        'America/Monterrey' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Monterrey་)',
        'America/Montevideo' => 'ཡུ་རུ་གུ་ཝཱའི་ཆུ་ཚོད། (Montevideo་)',
        'America/Montserrat' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Montserrat་)',
        'America/Nassau' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Nassau་)',
        'America/New_York' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (New York་)',
        'America/Nome' => 'ཨ་ལསི་ཀ་ཆུ་ཚོད། (Nome་)',
        'America/Noronha' => 'ཕར་ནེན་ཌོ་ ཌི་ ནོ་རཱོན་ཧ་ཆུ་ཚོད། (Fernando de Noronha་)',
        'America/North_Dakota/Beulah' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Beulah, North Dakota་)',
        'America/North_Dakota/Center' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Center, North Dakota་)',
        'America/North_Dakota/New_Salem' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (New Salem, North Dakota་)',
        'America/Ojinaga' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Ojinaga་)',
        'America/Panama' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (པ་ན་མ་)',
        'America/Paramaribo' => 'སུ་རི་ནཱམ་ཆུ་ཚོད། (Paramaribo་)',
        'America/Phoenix' => 'བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཆུ་ཚོད། (Phoenix་)',
        'America/Port-au-Prince' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (Port-au-Prince་)',
        'America/Port_of_Spain' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Port of Spain་)',
        'America/Porto_Velho' => 'ཨེ་མ་ཛཱོན་ཆུ་ཚོད། (Porto Velho་)',
        'America/Puerto_Rico' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Puerto Rico་)',
        'America/Punta_Arenas' => 'ཅི་ལི་ཆུ་ཚོད། (Punta Arenas་)',
        'America/Rankin_Inlet' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (རེན་ཀིན་ ཨིན་ལེཊ་)',
        'America/Recife' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (Recife་)',
        'America/Regina' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (རི་ཇི་ན་)',
        'America/Resolute' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (རི་སོ་ལིའུཊ་)',
        'America/Rio_Branco' => 'བྲ་ཛིལ་ཆུ་ཚོད།། (Rio Branco་)',
        'America/Santarem' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (Santarém་)',
        'America/Santiago' => 'ཅི་ལི་ཆུ་ཚོད། (སཱན་ཊི་ཡ་གྷོ་)',
        'America/Santo_Domingo' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (སཱན་ཊོ་ ཌོ་མིང་གྷོ་)',
        'America/Sao_Paulo' => 'བྲ་ཛི་ལི་ཡ་ཆུ་ཚོད། (São Paulo་)',
        'America/Scoresbysund' => 'གིརཱིན་ལནཌ྄་ཆུ་ཚོད།། (Ittoqqortoormiit་)',
        'America/Sitka' => 'ཨ་ལསི་ཀ་ཆུ་ཚོད། (Sitka་)',
        'America/St_Barthelemy' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (St. Barthélemy་)',
        'America/St_Johns' => 'ནིའུ་ཕའུནཌ་ལེནཌ་ཆུ་ཚོད། (ཨིསི་ཊེཊ་ ཇཱོནསི་་)',
        'America/St_Kitts' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (St. Kitts་)',
        'America/St_Lucia' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (St. Lucia་)',
        'America/St_Thomas' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (St. Thomas་)',
        'America/St_Vincent' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (St. Vincent་)',
        'America/Swift_Current' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (སུ་ཨིཕཊ་ཀ་རེནཊ་)',
        'America/Tegucigalpa' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (ཊེ་གུ་སི་གཱལ་པ་)',
        'America/Thule' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Thule་)',
        'America/Tijuana' => 'བྱང་ཨ་མི་རི་ཀ་པེ་སི་ཕིག་ཆུ་ཚོད། (ཏིའུ་ཝ་ན་)',
        'America/Toronto' => 'བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཆུ་ཚོད། (ཊོ་རོན་ཊོ་)',
        'America/Tortola' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (Tortola་)',
        'America/Vancouver' => 'བྱང་ཨ་མི་རི་ཀ་པེ་སི་ཕིག་ཆུ་ཚོད། (Vancouver་)',
        'America/Whitehorse' => 'ཀེ་ན་ཌ་ཆུ་ཚོད།། (Whitehorse་)',
        'America/Winnipeg' => 'བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཆུ་ཚོད། (Winnipeg་)',
        'America/Yakutat' => 'ཨ་ལསི་ཀ་ཆུ་ཚོད། (ཡ་ཀུ་ཏཏ་)',
        'Antarctica/Casey' => 'ནུབ་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Casey་)',
        'Antarctica/Davis' => 'འཛམ་གླིང་ལྷོ་མཐའི་ཁྱགས་གླིང་ཆུ་ཚོད།། (ཌེ་ཝིས།་)',
        'Antarctica/DumontDUrville' => 'འཛམ་གླིང་ལྷོ་མཐའི་ཁྱགས་གླིང་ཆུ་ཚོད།། (Dumont-d’Urville་)',
        'Antarctica/Macquarie' => 'ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Macquarie་)',
        'Antarctica/Mawson' => 'འཛམ་གླིང་ལྷོ་མཐའི་ཁྱགས་གླིང་ཆུ་ཚོད།། (མའུ་སཱོན་)',
        'Antarctica/McMurdo' => 'ནིའུ་ཛི་ལེནཌ་ཆུ་ཚོད། (McMurdo་)',
        'Antarctica/Palmer' => 'ཅི་ལི་ཆུ་ཚོད། (Palmer་)',
        'Antarctica/Rothera' => 'འཛམ་གླིང་ལྷོ་མཐའི་ཁྱགས་གླིང་ཆུ་ཚོད།། (Rothera་)',
        'Antarctica/Syowa' => 'འཛམ་གླིང་ལྷོ་མཐའི་ཁྱགས་གླིང་ཆུ་ཚོད།། (Showa་)',
        'Antarctica/Troll' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Troll་)',
        'Antarctica/Vostok' => 'འཛམ་གླིང་ལྷོ་མཐའི་ཁྱགས་གླིང་ཆུ་ཚོད།། (Vostok་)',
        'Arctic/Longyearbyen' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Longyearbyen་)',
        'Asia/Aden' => 'ཨ་རེ་བྷི་ཡན་ཆུ་ཚོད། (Aden་)',
        'Asia/Almaty' => 'ཀ་ཛགས་སཏཱན་ཆུ་ཚོད།། (Almaty་)',
        'Asia/Amman' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Amman་)',
        'Asia/Anadyr' => 'ཨུ་རུ་སུ་ཆུ་ཚོད།། (Anadyr་)',
        'Asia/Aqtau' => 'ཀ་ཛགས་སཏཱན་ཆུ་ཚོད།། (Aqtau་)',
        'Asia/Aqtobe' => 'ཀ་ཛགས་སཏཱན་ཆུ་ཚོད།། (Aqtobe་)',
        'Asia/Ashgabat' => 'ཊཱརཀ་མེནའི་སཏཱན་ཆུ་ཚོད།། (Ashgabat་)',
        'Asia/Atyrau' => 'ཀ་ཛགས་སཏཱན་ཆུ་ཚོད།། (Atyrau་)',
        'Asia/Baghdad' => 'ཨ་རེ་བྷི་ཡན་ཆུ་ཚོད། (Baghdad་)',
        'Asia/Bahrain' => 'ཨ་རེ་བྷི་ཡན་ཆུ་ཚོད། (བྷ་རེན་་)',
        'Asia/Baku' => 'ཨ་ཛར་བྷའི་ཇཱན་ཆུ་ཚོད། (Baku་)',
        'Asia/Bangkok' => 'ཨིན་ཌོ་ཅཱའི་ན་ཆུ་ཚོད། (Bangkok་)',
        'Asia/Barnaul' => 'ཀརསི་ནོ་ཡརསཀི་ཆུ་ཚོད། (Barnaul་)',
        'Asia/Beirut' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Beirut་)',
        'Asia/Bishkek' => 'ཀིར་གིས་སཏཱན་ཆུ་ཚོད།། (Bishkek་)',
        'Asia/Brunei' => 'བྷྲུ་ནའི་ཆུ་ཚོད།། (Brunei་)',
        'Asia/Calcutta' => 'རྒྱ་གར་ཆུ་ཚོད། (Kolkata་)',
        'Asia/Kolkata' => 'རྒྱ་གར་ཆུ་ཚོད། (Kolkata་)',
        'Asia/Chita' => 'ཡ་ཀུཙིཀི་ཆུ་ཚོད། (Chita་)',
        'Asia/Colombo' => 'རྒྱ་གར་ཆུ་ཚོད། (Colombo་)',
        'Asia/Damascus' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Damascus་)',
        'Asia/Dhaka' => 'བངྒ་ལ་དེཤ་ཆུ་ཚོད། (Dhaka་)',
        'Asia/Dili' => 'ཏི་་མོར་ལེ་ཨེསཊ་ཆུ་ཚོད།། (Dili་)',
        'Asia/Dubai' => 'གཱལཕི་ཆུ་ཚོད། (Dubai་)',
        'Asia/Dushanbe' => 'ཏ་ཇིག་གི་སཏཱན་ཆུ་ཚོད།། (Dushanbe་)',
        'Asia/Famagusta' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Famagusta་)',
        'Asia/Gaza' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Gaza་)',
        'Asia/Hebron' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Hebron་)',
        'Asia/Hong_Kong' => 'ཧོང་ཀོང་ཅཱའི་ན་ཆུ་ཚོད།། (Hong Kong་)',
        'Asia/Hovd' => 'སོག་པོ་ཡུལ་ཆུ་ཚོད།། (Khovd་)',
        'Asia/Irkutsk' => 'ཨར་ཀུཙི་ཆུ་ཚོད། (Irkutsk་)',
        'Asia/Jakarta' => 'ནུབ་ཕྱོགས་ཨིན་ཌོ་ནེ་ཤི་ཡ་ཆུ་ཚོད། (Jakarta་)',
        'Asia/Jayapura' => 'ཤར་ཕྱོགས་ཨིན་ཌོ་ནེ་ཤི་ཡ་ཆུ་ཚོད། (Jayapura་)',
        'Asia/Jerusalem' => 'ཨིས་རེལ་ཆུ་ཚོད། (Jerusalem་)',
        'Asia/Kabul' => 'ཨཕ་ག་ནི་ས྄ཏཱནཆུ་ཚོད། (Kabul་)',
        'Asia/Kamchatka' => 'ཨུ་རུ་སུ་ཆུ་ཚོད།། (Kamchatka་)',
        'Asia/Karachi' => 'པ་ཀི་ས྄ཏཱན་ཆུ་ཚོད། (Karachi་)',
        'Asia/Katmandu' => 'ནེ་པཱལ་ཆུ་ཚོད། (Kathmandu་)',
        'Asia/Kathmandu' => 'ནེ་པཱལ་ཆུ་ཚོད། (Kathmandu་)',
        'Asia/Khandyga' => 'ཡ་ཀུཙིཀི་ཆུ་ཚོད། (Khandyga་)',
        'Asia/Krasnoyarsk' => 'ཀརསི་ནོ་ཡརསཀི་ཆུ་ཚོད། (Krasnoyarsk་)',
        'Asia/Kuala_Lumpur' => 'མ་ལེ་ཤི་ཡ་ཆུ་ཚོད།། (Kuala Lumpur་)',
        'Asia/Kuching' => 'མ་ལེ་ཤི་ཡ་ཆུ་ཚོད།། (Kuching་)',
        'Asia/Kuwait' => 'ཨ་རེ་བྷི་ཡན་ཆུ་ཚོད། (Kuwait་)',
        'Asia/Macau' => 'རྒྱ་ནག་ཆུ་ཚོད། (Macao་)',
        'Asia/Magadan' => 'མ་གྷ་དཱན་ཆུ་ཚོད། (Magadan་)',
        'Asia/Makassar' => 'དབུས་ཕྱོགས་ཨིན་ཌོ་ནེ་ཤི་ཡ་ཆུ་ཚོད། (Makassar་)',
        'Asia/Manila' => 'ཕི་ལི་པིནས་ཆུ་ཚོད།། (Manila་)',
        'Asia/Muscat' => 'གཱལཕི་ཆུ་ཚོད། (Muscat་)',
        'Asia/Nicosia' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Nicosia་)',
        'Asia/Novokuznetsk' => 'ཀརསི་ནོ་ཡརསཀི་ཆུ་ཚོད། (Novokuznetsk་)',
        'Asia/Novosibirsk' => 'ཀརསི་ནོ་ཡརསཀི་ཆུ་ཚོད། (Novosibirsk་)',
        'Asia/Omsk' => 'ཨོམསཀི་ཆུ་ཚོད། (Omsk་)',
        'Asia/Oral' => 'ཀ་ཛགས་སཏཱན་ཆུ་ཚོད།། (Oral་)',
        'Asia/Phnom_Penh' => 'ཨིན་ཌོ་ཅཱའི་ན་ཆུ་ཚོད། (Phnom Penh་)',
        'Asia/Pontianak' => 'ནུབ་ཕྱོགས་ཨིན་ཌོ་ནེ་ཤི་ཡ་ཆུ་ཚོད། (Pontianak་)',
        'Asia/Pyongyang' => 'ཀོ་རི་ཡ་ཆུ་ཚོད། (Pyongyang་)',
        'Asia/Qatar' => 'ཨ་རེ་བྷི་ཡན་ཆུ་ཚོད། (Qatar་)',
        'Asia/Qostanay' => 'ཀ་ཛགས་སཏཱན་ཆུ་ཚོད།། (Qostanay་)',
        'Asia/Qyzylorda' => 'ཀ་ཛགས་སཏཱན་ཆུ་ཚོད།། (Qyzylorda་)',
        'Asia/Rangoon' => 'མི་ཡཱན་མར་ (བྷར་མ)་ཆུ་ཚོད།། (Yangon་)',
        'Asia/Yangon' => 'མི་ཡཱན་མར་ (བྷར་མ)་ཆུ་ཚོད།། (Yangon་)',
        'Asia/Riyadh' => 'ཨ་རེ་བྷི་ཡན་ཆུ་ཚོད། (Riyadh་)',
        'Asia/Saigon' => 'ཨིན་ཌོ་ཅཱའི་ན་ཆུ་ཚོད། (Ho Chi Minh་)',
        'Asia/Ho_Chi_Minh' => 'ཨིན་ཌོ་ཅཱའི་ན་ཆུ་ཚོད། (Ho Chi Minh་)',
        'Asia/Sakhalin' => 'མ་གྷ་དཱན་ཆུ་ཚོད། (Sakhalin་)',
        'Asia/Samarkand' => 'ཨུས་བེག་གི་སཏཱན་ཆུ་ཚོད།། (Samarkand་)',
        'Asia/Seoul' => 'ཀོ་རི་ཡ་ཆུ་ཚོད། (Seoul་)',
        'Asia/Shanghai' => 'རྒྱ་ནག་ཆུ་ཚོད། (Shanghai་)',
        'Asia/Singapore' => 'སིང་ག་པོར་ཆུ་ཚོད།། (Singapore་)',
        'Asia/Srednekolymsk' => 'མ་གྷ་དཱན་ཆུ་ཚོད། (Srednekolymsk་)',
        'Asia/Taipei' => 'ཊཱའི་ཝཱན་ཆུ་ཚོད།། (Taipei་)',
        'Asia/Tashkent' => 'ཨུས་བེག་གི་སཏཱན་ཆུ་ཚོད།། (Tashkent་)',
        'Asia/Tbilisi' => 'ཇཽ་ཇཱ་ཆུ་ཚོད། (Tbilisi་)',
        'Asia/Tehran' => 'ཨི་རཱན་ཆུ་ཚོད། (Tehran་)',
        'Asia/Thimphu' => 'འབྲུག་ཡུལ་ཆུ་ཚོད། (ཐིམ་ཕུག་)',
        'Asia/Tokyo' => 'ཇ་པཱན་ཆུ་ཚོད། (Tokyo་)',
        'Asia/Tomsk' => 'ཀརསི་ནོ་ཡརསཀི་ཆུ་ཚོད། (Tomsk་)',
        'Asia/Ulaanbaatar' => 'སོག་པོ་ཡུལ་ཆུ་ཚོད།། (Ulaanbaatar་)',
        'Asia/Urumqi' => 'རྒྱ་ནག་ཆུ་ཚོད།། (Ürümqi་)',
        'Asia/Ust-Nera' => 'བ་ལ་ཌི་བོསི་ཏོཀ་ཆུ་ཚོད། (Ust-Nera་)',
        'Asia/Vientiane' => 'ཨིན་ཌོ་ཅཱའི་ན་ཆུ་ཚོད། (Vientiane་)',
        'Asia/Vladivostok' => 'བ་ལ་ཌི་བོསི་ཏོཀ་ཆུ་ཚོད། (Vladivostok་)',
        'Asia/Yakutsk' => 'ཡ་ཀུཙིཀི་ཆུ་ཚོད། (Yakutsk་)',
        'Asia/Yekaterinburg' => 'ཡེ་ཀ་ཏེ་རིན་བརག་ཆུ་ཚོད། (Yekaterinburg་)',
        'Asia/Yerevan' => 'ཨར་མི་ནི་ཡ་ཆུ་ཚོད། (Yerevan་)',
        'Atlantic/Azores' => 'ཨེ་ཛོརས་ཆུ་ཚོད། (Azores་)',
        'Atlantic/Bermuda' => 'ཨེཊ་ལེན་ཊིཀ་ཆུ་ཚོད། (བར་མུ་ད་)',
        'Atlantic/Canary' => 'ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Canarias་)',
        'Atlantic/Cape_Verde' => 'ཀེཔ་བཱཌ་ཆུ་ཚོད། (Cape Verde་)',
        'Atlantic/Faeroe' => 'ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཕཱའེ་རོ་་)',
        'Atlantic/Faroe' => 'ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཕཱའེ་རོ་་)',
        'Atlantic/Madeira' => 'ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Madeira་)',
        'Atlantic/Reykjavik' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Reykjavik་)',
        'Atlantic/South_Georgia' => 'སཱའུཐ་ཇཽར་ཇཱ་ དང་ སཱའུཐ་སེནཌ྄་ཝིཅ་གླིང་ཚོམ་ཆུ་ཚོད།། (South Georgia་)',
        'Atlantic/St_Helena' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (St. Helena་)',
        'Atlantic/Stanley' => 'ཕལཀ་ལེནཌ་ཨཱའི་ལེནཌས་ཆུ་ཚོད། (Stanley་)',
        'Australia/Adelaide' => 'དབུས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Adelaide་)',
        'Australia/Brisbane' => 'ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Brisbane་)',
        'Australia/Broken_Hill' => 'དབུས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Broken Hill་)',
        'Australia/Darwin' => 'དབུས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Darwin་)',
        'Australia/Eucla' => 'བུས་ནུབ་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Eucla་)',
        'Australia/Hobart' => 'ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Hobart་)',
        'Australia/Lindeman' => 'ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Lindeman་)',
        'Australia/Lord_Howe' => 'ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད།། (Lord Howe་)',
        'Australia/Melbourne' => 'ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Melbourne་)',
        'Australia/Perth' => 'ནུབ་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Perth་)',
        'Australia/Sydney' => 'ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཆུ་ཚོད། (Sydney་)',
        'Etc/GMT' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད',
        'Europe/Amsterdam' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Amsterdam་)',
        'Europe/Andorra' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Andorra་)',
        'Europe/Astrakhan' => 'ཨུ་རུ་སུ་ཆུ་ཚོད།། (Astrakhan་)',
        'Europe/Athens' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཨེ་ཐེནས་་)',
        'Europe/Belgrade' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Belgrade་)',
        'Europe/Berlin' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Berlin་)',
        'Europe/Bratislava' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Bratislava་)',
        'Europe/Brussels' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Brussels་)',
        'Europe/Bucharest' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Bucharest་)',
        'Europe/Budapest' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Budapest་)',
        'Europe/Busingen' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Büsingen་)',
        'Europe/Chisinau' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Chișinău་)',
        'Europe/Copenhagen' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཀོ་པེན་ཧེ་གེན་)',
        'Europe/Dublin' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Dublin་)',
        'Europe/Gibraltar' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Gibraltar་)',
        'Europe/Guernsey' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Guernsey་)',
        'Europe/Helsinki' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཧེལ་སིང་ཀི་)',
        'Europe/Isle_of_Man' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Isle of Man་)',
        'Europe/Istanbul' => 'ཊཱར་ཀི་ཆུ་ཚོད།། (Istanbul་)',
        'Europe/Jersey' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (Jersey་)',
        'Europe/Kaliningrad' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Kaliningrad་)',
        'Europe/Kiev' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Kyiv་)',
        'Europe/Kyiv' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Kyiv་)',
        'Europe/Kirov' => 'མཽས་ཀོ་ཆུ་ཚོད། (Kirov་)',
        'Europe/Lisbon' => 'ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Lisbon་)',
        'Europe/Ljubljana' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Ljubljana་)',
        'Europe/London' => 'གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད། (London་)',
        'Europe/Luxembourg' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Luxembourg་)',
        'Europe/Madrid' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Madrid་)',
        'Europe/Malta' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Malta་)',
        'Europe/Mariehamn' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Mariehamn་)',
        'Europe/Minsk' => 'མཽས་ཀོ་ཆུ་ཚོད། (Minsk་)',
        'Europe/Monaco' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Monaco་)',
        'Europe/Moscow' => 'མཽས་ཀོ་ཆུ་ཚོད། (Moscow་)',
        'Europe/Oslo' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Oslo་)',
        'Europe/Paris' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Paris་)',
        'Europe/Podgorica' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Podgorica་)',
        'Europe/Prague' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Prague་)',
        'Europe/Riga' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Riga་)',
        'Europe/Rome' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Rome་)',
        'Europe/Samara' => 'ཨུ་རུ་སུ་ཆུ་ཚོད།། (Samara་)',
        'Europe/San_Marino' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (San Marino་)',
        'Europe/Sarajevo' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Sarajevo་)',
        'Europe/Saratov' => 'ཨུ་རུ་སུ་ཆུ་ཚོད།། (Saratov་)',
        'Europe/Simferopol' => 'མཽས་ཀོ་ཆུ་ཚོད། (Simferopol་)',
        'Europe/Skopje' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Skopje་)',
        'Europe/Sofia' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Sofia་)',
        'Europe/Stockholm' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Stockholm་)',
        'Europe/Tallinn' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (ཊཱ་ལཱིན་)',
        'Europe/Tirane' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Tirana་)',
        'Europe/Ulyanovsk' => 'ཨུ་རུ་སུ་ཆུ་ཚོད།། (Ulyanovsk་)',
        'Europe/Vaduz' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Vaduz་)',
        'Europe/Vatican' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Vatican་)',
        'Europe/Vienna' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Vienna་)',
        'Europe/Vilnius' => 'ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Vilnius་)',
        'Europe/Volgograd' => 'མཽས་ཀོ་ཆུ་ཚོད། (Volgograd་)',
        'Europe/Warsaw' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Warsaw་)',
        'Europe/Zagreb' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Zagreb་)',
        'Europe/Zurich' => 'དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཆུ་ཚོད། (Zurich་)',
        'Indian/Antananarivo' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Antananarivo་)',
        'Indian/Chagos' => 'རྒྱ་གར་གྱི་རྒྱ་མཚོ་ཆུ་ཚོད། (Chagos་)',
        'Indian/Christmas' => 'ཁི་རིསྟ་མེས་མཚོ་གླིང་ཆུ་ཚོད། (Christmas་)',
        'Indian/Cocos' => 'ཀོ་ཀོས་གླིང་ཚོམ་ཆུ་ཚོད།། (Cocos་)',
        'Indian/Comoro' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Comores་)',
        'Indian/Kerguelen' => 'ཕྲནཅ་གི་ལྷོ་ཕྱོགས་མངའ་ཁོངས་ཆུ་ཚོད།། (Kerguelen་)',
        'Indian/Mahe' => 'སེ་ཤཱལས་ཆུ་ཚོད། (Mahé་)',
        'Indian/Maldives' => 'མཱལ་དིབས་ཆུ་ཚོད། (Maldives་)',
        'Indian/Mauritius' => 'མོ་རི་ཤཱས་ཆུ་ཚོད། (Mauritius་)',
        'Indian/Mayotte' => 'ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད། (Mayotte་)',
        'Indian/Reunion' => 'རི་ཡུ་ནི་ཡཱན་ཆུ་ཚོད། (Réunion་)',
        'Pacific/Apia' => 'ས་མོ་ཨ་ཆུ་ཚོད།། (ཨ་པི་ཡ་)',
        'Pacific/Auckland' => 'ནིའུ་ཛི་ལེནཌ་ཆུ་ཚོད། (Auckland་)',
        'Pacific/Bougainville' => 'པ་པུ་ ནིའུ་གི་ནི་ཆུ་ཚོད།། (Bougainville་)',
        'Pacific/Chatham' => 'ནིའུ་ཛི་ལེནཌ་ཆུ་ཚོད།། (Chatham་)',
        'Pacific/Easter' => 'ཨིསི་ཊར་ཨཱའི་ལེནཌ་ཆུ་ཚོད། (Easter་)',
        'Pacific/Efate' => 'ཝ་ནུ་ཨ་ཏུ་ཆུ་ཚོད།། (Efate་)',
        'Pacific/Enderbury' => 'ཀི་རི་བ་ཏི་མཚོ་གླིང་ཆུ་ཚོད།། (Canton་)',
        'Pacific/Kanton' => 'ཀི་རི་བ་ཏི་མཚོ་གླིང་ཆུ་ཚོད།། (Canton་)',
        'Pacific/Fakaofo' => 'ཏོ་ཀེ་ལའུ་ མཚོ་གླིང་ཆུ་ཚོད།། (Fakaofo་)',
        'Pacific/Fiji' => 'ཕི་ཇི་ཆུ་ཚོད།། (Fiji་)',
        'Pacific/Funafuti' => 'ཏུ་ཝ་ལུ་ཆུ་ཚོད།། (Funafuti་)',
        'Pacific/Galapagos' => 'ག་ལ་པ་གོསི་ཆུ་ཚོད། (Galápagos་)',
        'Pacific/Gambier' => 'ཕྲཱནས྄་ཀྱི་པོ་ལི་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Gambier་)',
        'Pacific/Guadalcanal' => 'སོ་ལོ་མོན་ གླིང་ཚོམ་ཆུ་ཚོད།། (Guadalcanal་)',
        'Pacific/Guam' => 'གུ་འམ་ མཚོ་གླིང་ཆུ་ཚོད།། (Guam་)',
        'Pacific/Honolulu' => 'ཧ་ཝའི་-ཨེ་ལིའུ་ཤེན་ཚད་ལྡན་ཆུ་ཚོད། (Honolulu་)',
        'Pacific/Kiritimati' => 'ཀི་རི་བ་ཏི་མཚོ་གླིང་ཆུ་ཚོད།། (Kiritimati་)',
        'Pacific/Kosrae' => 'མའི་ཀྲོ་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Kosrae་)',
        'Pacific/Kwajalein' => 'མར་ཤེལ་གླིང་ཚོམ་ཆུ་ཚོད།། (Kwajalein་)',
        'Pacific/Majuro' => 'མར་ཤེལ་གླིང་ཚོམ་ཆུ་ཚོད།། (Majuro་)',
        'Pacific/Marquesas' => 'ཕྲཱནས྄་ཀྱི་པོ་ལི་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Marquesas་)',
        'Pacific/Midway' => 'ཡུ་ཨེས་གྱི་མཐའ་མཚམས་མཚོ་གླིང་་ཆུ་ཚོད།། (Midway་)',
        'Pacific/Nauru' => 'ནའུ་རུ་་ཆུ་ཚོད།། (Nauru་)',
        'Pacific/Niue' => 'ནི་ཨུ་ཨཻ་ཆུ་ཚོད།། (Niue་)',
        'Pacific/Norfolk' => 'ནོར་ཕོལཀ་མཚོ་གླིང༌་ཆུ་ཚོད།། (Norfolk་)',
        'Pacific/Noumea' => 'ནིའུ་ཀ་ལི་དོ་ནི་ཡ་ཆུ་ཚོད།། (Nouméa་)',
        'Pacific/Pago_Pago' => 'ས་མོ་ཨ་ཡུ་ཨེས་ཨེ་མངའ་ཁོངས་ཆུ་ཚོད།། (Pago Pago་)',
        'Pacific/Palau' => 'པ་ལའུ་ཆུ་ཚོད།། (Palau་)',
        'Pacific/Pitcairn' => 'པིཊ་ཀེ་ཡེརན་གླིང་ཚོམ་ཆུ་ཚོད།། (Pitcairn་)',
        'Pacific/Ponape' => 'མའི་ཀྲོ་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Pohnpei་)',
        'Pacific/Pohnpei' => 'མའི་ཀྲོ་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Pohnpei་)',
        'Pacific/Port_Moresby' => 'པ་པུ་ ནིའུ་གི་ནི་ཆུ་ཚོད།། (Port Moresby་)',
        'Pacific/Rarotonga' => 'ཀུག་གླིང་ཚོམ་ཆུ་ཚོད།། (Rarotonga་)',
        'Pacific/Saipan' => 'བྱང་ཕྱོགས་ཀྱི་མ་ར་ཡ་ན་གླིང་ཚོམ་ཆུ་ཚོད།། (Saipan་)',
        'Pacific/Tahiti' => 'ཕྲཱནས྄་ཀྱི་པོ་ལི་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Tahiti་)',
        'Pacific/Tarawa' => 'ཀི་རི་བ་ཏི་མཚོ་གླིང་ཆུ་ཚོད།། (Tarawa་)',
        'Pacific/Tongatapu' => 'ཊོང་གྷ་ཆུ་ཚོད།། (Tongatapu་)',
        'Pacific/Truk' => 'མའི་ཀྲོ་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Chuuk་)',
        'Pacific/Chuuk' => 'མའི་ཀྲོ་ནི་ཤི་ཡ་ཆུ་ཚོད།། (Chuuk་)',
        'Pacific/Wake' => 'ཡུ་ཨེས་གྱི་མཐའ་མཚམས་མཚོ་གླིང་་ཆུ་ཚོད།། (Wake་)',
        'Pacific/Wallis' => 'ཝལ་ལིས྄་ ཨེནཌ་ ཕུ་ཏུ་ན་་ཆུ་ཚོད།། (Wallis & Futuna་)',
    ],
    'Meta' => [
        'GmtFormat' => 'ཇི་ཨེམ་ཏི་%s',
    ],
];
