<?php

return [
    'Names' => [
        'af' => 'Awherikāna',
        'af_NA' => 'Awherikāna (Namipia)',
        'af_ZA' => 'Awherikāna (Āwherika ki te Tonga)',
        'ak' => 'Ākana',
        'ak_GH' => 'Ākana (Kāna)',
        'am' => 'Amahereka',
        'am_ET' => 'Amahereka (Etiopia)',
        'ar' => 'Ārapi',
        'ar_001' => 'Ārapi (te ao)',
        'ar_AE' => 'Ārapi (Kotahitanga o ngā Whenua o Ārapi)',
        'ar_BH' => 'Ārapi (Pāreina)',
        'ar_DJ' => 'Ārapi (Tipūti)',
        'ar_DZ' => 'Ārapi (Aratiria)',
        'ar_EG' => 'Ārapi (Īhipa)',
        'ar_EH' => 'Ārapi (Hahāra ki te Tonga)',
        'ar_ER' => 'Ārapi (Eritēria)',
        'ar_IL' => 'Ārapi (Iharaira)',
        'ar_IQ' => 'Ārapi (Irāka)',
        'ar_JO' => 'Ārapi (Hōrano)',
        'ar_KM' => 'Ārapi (Komoro)',
        'ar_KW' => 'Ārapi (Kūweiti)',
        'ar_LB' => 'Ārapi (Repanona)',
        'ar_LY' => 'Ārapi (Ripia)',
        'ar_MA' => 'Ārapi (Moroko)',
        'ar_MR' => 'Ārapi (Mauritānia)',
        'ar_OM' => 'Ārapi (Ōmana)',
        'ar_PS' => 'Ārapi (Ngā Rohe o Parihitini)',
        'ar_QA' => 'Ārapi (Katā)',
        'ar_SA' => 'Ārapi (Hauri Arāpia)',
        'ar_SD' => 'Ārapi (Hūtāne)',
        'ar_SO' => 'Ārapi (Hūmārie)',
        'ar_SS' => 'Ārapi (Hūtāne ki te Tonga)',
        'ar_SY' => 'Ārapi (Hiria)',
        'ar_TD' => 'Ārapi (Kāta)',
        'ar_TN' => 'Ārapi (Tūnihia)',
        'ar_YE' => 'Ārapi (Īmene)',
        'as' => 'Āhamēhi',
        'as_IN' => 'Āhamēhi (Inia)',
        'az' => 'Atepaihānia',
        'az_AZ' => 'Atepaihānia (Atepaihānia)',
        'az_Cyrl' => 'Atepaihānia (Hīririki)',
        'az_Cyrl_AZ' => 'Atepaihānia (Hīririki, Atepaihānia)',
        'az_Latn' => 'Atepaihānia (Rātini)',
        'az_Latn_AZ' => 'Atepaihānia (Rātini, Atepaihānia)',
        'ba' => 'Pākīra',
        'ba_RU' => 'Pākīra (Rūhia)',
        'be' => 'Perarūhiana',
        'be_BY' => 'Perarūhiana (Pērara)',
        'bg' => 'Purukāriana',
        'bg_BG' => 'Purukāriana (Purukāria)',
        'bm' => 'Pāpara',
        'bm_ML' => 'Pāpara (Māri)',
        'bn' => 'Pākara',
        'bn_BD' => 'Pākara (Pākaratēhi)',
        'bn_IN' => 'Pākara (Inia)',
        'bo' => 'Tipete',
        'bo_CN' => 'Tipete (Haina)',
        'bo_IN' => 'Tipete (Inia)',
        'br' => 'Peretana',
        'br_FR' => 'Peretana (Wīwī)',
        'bs' => 'Pōngiana',
        'bs_BA' => 'Pōngiana (Pōngia & Herekōwini)',
        'bs_Cyrl' => 'Pōngiana (Hīririki)',
        'bs_Cyrl_BA' => 'Pōngiana (Hīririki, Pōngia & Herekōwini)',
        'bs_Latn' => 'Pōngiana (Rātini)',
        'bs_Latn_BA' => 'Pōngiana (Rātini, Pōngia & Herekōwini)',
        'ca' => 'Katarana',
        'ca_AD' => 'Katarana (Anatōra)',
        'ca_ES' => 'Katarana (Peina)',
        'ca_FR' => 'Katarana (Wīwī)',
        'ca_IT' => 'Katarana (Itāria)',
        'ce' => 'Tietiene',
        'ce_RU' => 'Tietiene (Rūhia)',
        'cs' => 'Tieke',
        'cs_CZ' => 'Tieke (Tiekia)',
        'cv' => 'Tiuwhāhi',
        'cv_RU' => 'Tiuwhāhi (Rūhia)',
        'cy' => 'Werehi',
        'cy_GB' => 'Werehi (Te Hononga o Piritene)',
        'da' => 'Teina',
        'da_DK' => 'Teina (Tenemāka)',
        'da_GL' => 'Teina (Whenuakāriki)',
        'de' => 'Tiamana',
        'de_AT' => 'Tiamana (Ataria)',
        'de_BE' => 'Tiamana (Peretiama)',
        'de_CH' => 'Tiamana (Huiterangi)',
        'de_DE' => 'Tiamana (Tiamana)',
        'de_IT' => 'Tiamana (Itāria)',
        'de_LI' => 'Tiamana (Rīkenetaina)',
        'de_LU' => 'Tiamana (Rakapuō)',
        'dz' => 'Tonoka',
        'dz_BT' => 'Tonoka (Pūtana)',
        'ee' => 'Ewe',
        'ee_GH' => 'Ewe (Kāna)',
        'ee_TG' => 'Ewe (Toko)',
        'el' => 'Kariki',
        'el_CY' => 'Kariki (Haipara)',
        'el_GR' => 'Kariki (Kirihi)',
        'en' => 'Ingarihi',
        'en_001' => 'Ingarihi (te ao)',
        'en_150' => 'Ingarihi (Ūropi)',
        'en_AE' => 'Ingarihi (Kotahitanga o ngā Whenua o Ārapi)',
        'en_AG' => 'Ingarihi (Motu Nehe me Pāputa)',
        'en_AI' => 'Ingarihi (Anguira)',
        'en_AS' => 'Ingarihi (Hāmoa-Amerika)',
        'en_AT' => 'Ingarihi (Ataria)',
        'en_AU' => 'Ingarihi (Ahitereiria)',
        'en_BB' => 'Ingarihi (Papatohe)',
        'en_BE' => 'Ingarihi (Peretiama)',
        'en_BI' => 'Ingarihi (Puruniti)',
        'en_BM' => 'Ingarihi (Pāmura)',
        'en_BS' => 'Ingarihi (Pahama)',
        'en_BW' => 'Ingarihi (Poriwana)',
        'en_BZ' => 'Ingarihi (Perīhi)',
        'en_CA' => 'Ingarihi (Kānata)',
        'en_CC' => 'Ingarihi (Ngā Moutere Kokoko [Kirini])',
        'en_CH' => 'Ingarihi (Huiterangi)',
        'en_CK' => 'Ingarihi (Kuki Airani)',
        'en_CM' => 'Ingarihi (Kamarūna)',
        'en_CX' => 'Ingarihi (Te Moutere Kirihimete)',
        'en_CY' => 'Ingarihi (Haipara)',
        'en_CZ' => 'Ingarihi (Tiekia)',
        'en_DE' => 'Ingarihi (Tiamana)',
        'en_DK' => 'Ingarihi (Tenemāka)',
        'en_DM' => 'Ingarihi (Tominika)',
        'en_EE' => 'Ingarihi (Etōnia)',
        'en_ER' => 'Ingarihi (Eritēria)',
        'en_ES' => 'Ingarihi (Peina)',
        'en_FI' => 'Ingarihi (Whinarana)',
        'en_FJ' => 'Ingarihi (Whītī)',
        'en_FK' => 'Ingarihi (Motu Whākarangi)',
        'en_FM' => 'Ingarihi (Mekanēhia)',
        'en_FR' => 'Ingarihi (Wīwī)',
        'en_GB' => 'Ingarihi (Te Hononga o Piritene)',
        'en_GD' => 'Ingarihi (Kerenāta)',
        'en_GE' => 'Ingarihi (Hōria)',
        'en_GG' => 'Ingarihi (Kōnihi)',
        'en_GH' => 'Ingarihi (Kāna)',
        'en_GI' => 'Ingarihi (Kāmaka)',
        'en_GM' => 'Ingarihi (Kamopia)',
        'en_GS' => 'Ingarihi (Hōria ki te Tonga me ngā Motu Hanawiti ki te Tonga)',
        'en_GU' => 'Ingarihi (Kuama)',
        'en_GY' => 'Ingarihi (Kaiana)',
        'en_HK' => 'Ingarihi (Hongipua Haina)',
        'en_HU' => 'Ingarihi (Hanekari)',
        'en_ID' => 'Ingarihi (Initonīhia)',
        'en_IE' => 'Ingarihi (Airani)',
        'en_IL' => 'Ingarihi (Iharaira)',
        'en_IM' => 'Ingarihi (Te Moutere Mana)',
        'en_IN' => 'Ingarihi (Inia)',
        'en_IO' => 'Ingarihi (Te Rohe o te Moana Īniana Piritihi)',
        'en_IT' => 'Ingarihi (Itāria)',
        'en_JE' => 'Ingarihi (Tōrehe)',
        'en_JM' => 'Ingarihi (Hemeika)',
        'en_JP' => 'Ingarihi (Hapani)',
        'en_KE' => 'Ingarihi (Kenia)',
        'en_KI' => 'Ingarihi (Kiripati)',
        'en_KN' => 'Ingarihi (Hato Kiti me Newhi)',
        'en_KY' => 'Ingarihi (Ngā Motu Keimana)',
        'en_LC' => 'Ingarihi (Hato Ruhia)',
        'en_LR' => 'Ingarihi (Raipiria)',
        'en_LS' => 'Ingarihi (Teroto)',
        'en_LT' => 'Ingarihi (Rituānia)',
        'en_LV' => 'Ingarihi (Rāwhia)',
        'en_MG' => 'Ingarihi (Matakāhika)',
        'en_MH' => 'Ingarihi (Ngā Motu Māhara)',
        'en_MO' => 'Ingarihi (Makau Haina)',
        'en_MP' => 'Ingarihi (Ngā Motu Mariana ki te Raki)',
        'en_MS' => 'Ingarihi (Monoterā)',
        'en_MT' => 'Ingarihi (Mārata)',
        'en_MU' => 'Ingarihi (Marihi)',
        'en_MV' => 'Ingarihi (Māratiri)',
        'en_MW' => 'Ingarihi (Marāwi)',
        'en_MY' => 'Ingarihi (Mareia)',
        'en_NA' => 'Ingarihi (Namipia)',
        'en_NF' => 'Ingarihi (Te Moutere Nōpoke)',
        'en_NG' => 'Ingarihi (Ngāitiria)',
        'en_NL' => 'Ingarihi (Hōrana)',
        'en_NO' => 'Ingarihi (Nōwei)',
        'en_NR' => 'Ingarihi (Nauru)',
        'en_NU' => 'Ingarihi (Niue)',
        'en_NZ' => 'Ingarihi (Aotearoa)',
        'en_PG' => 'Ingarihi (Papua Nūkini)',
        'en_PH' => 'Ingarihi (Piripīni)',
        'en_PK' => 'Ingarihi (Pakitāne)',
        'en_PL' => 'Ingarihi (Pōrana)',
        'en_PN' => 'Ingarihi (Pitikeina)',
        'en_PR' => 'Ingarihi (Peta Riko)',
        'en_PT' => 'Ingarihi (Potukara)',
        'en_PW' => 'Ingarihi (Pārau)',
        'en_RO' => 'Ingarihi (Romeinia)',
        'en_RW' => 'Ingarihi (Rāwana)',
        'en_SB' => 'Ingarihi (Ngā Motu Horomona)',
        'en_SC' => 'Ingarihi (Heikere)',
        'en_SD' => 'Ingarihi (Hūtāne)',
        'en_SE' => 'Ingarihi (Huitene)',
        'en_SG' => 'Ingarihi (Hingapoa)',
        'en_SH' => 'Ingarihi (Hato Hērena)',
        'en_SI' => 'Ingarihi (Horowinia)',
        'en_SK' => 'Ingarihi (Horowākia)',
        'en_SL' => 'Ingarihi (Te Araone)',
        'en_SS' => 'Ingarihi (Hūtāne ki te Tonga)',
        'en_SX' => 'Ingarihi (Hiti Mātene)',
        'en_SZ' => 'Ingarihi (Ehiwatini)',
        'en_TC' => 'Ingarihi (Koru-Kākoa)',
        'en_TK' => 'Ingarihi (Tokerau)',
        'en_TO' => 'Ingarihi (Tonga)',
        'en_TT' => 'Ingarihi (Tirinaki Tōpako)',
        'en_TV' => 'Ingarihi (Tūwaru)',
        'en_TZ' => 'Ingarihi (Tānahia)',
        'en_UA' => 'Ingarihi (Ukareinga)',
        'en_UG' => 'Ingarihi (Ukānga)',
        'en_UM' => 'Ingarihi (Ngā Moutere Amerika o Waho)',
        'en_US' => 'Ingarihi (Hononga o Amerika)',
        'en_VC' => 'Ingarihi (Hato Wēneti me Keretīni)',
        'en_VG' => 'Ingarihi (Ngā Moutere Puhi Piritene)',
        'en_VI' => 'Ingarihi (Ngā Moutere Puhi Amerika)',
        'en_VU' => 'Ingarihi (Whenuatū)',
        'en_WS' => 'Ingarihi (Hāmoa)',
        'en_ZA' => 'Ingarihi (Āwherika ki te Tonga)',
        'en_ZM' => 'Ingarihi (Tāmipia)',
        'en_ZW' => 'Ingarihi (Timuwawe)',
        'eo' => 'Eheperāto',
        'eo_001' => 'Eheperāto (te ao)',
        'es' => 'Pāniora',
        'es_419' => 'Pāniora (Amerika Rātini)',
        'es_AR' => 'Pāniora (Āketina)',
        'es_BO' => 'Pāniora (Poriwia)',
        'es_BR' => 'Pāniora (Parīhi)',
        'es_BZ' => 'Pāniora (Perīhi)',
        'es_CL' => 'Pāniora (Hiri)',
        'es_CO' => 'Pāniora (Koromōpia)',
        'es_CR' => 'Pāniora (Koto Rīka)',
        'es_CU' => 'Pāniora (Kiupa)',
        'es_DO' => 'Pāniora (Te Whenua Tūhake o Tominika)',
        'es_EC' => 'Pāniora (Ekuatoa)',
        'es_ES' => 'Pāniora (Peina)',
        'es_GQ' => 'Pāniora (Kini Ekuatoria)',
        'es_GT' => 'Pāniora (Kuatamāra)',
        'es_HN' => 'Pāniora (Honotura)',
        'es_MX' => 'Pāniora (Mēhiko)',
        'es_NI' => 'Pāniora (Nikarāhua)',
        'es_PA' => 'Pāniora (Panama)',
        'es_PE' => 'Pāniora (Peru)',
        'es_PH' => 'Pāniora (Piripīni)',
        'es_PR' => 'Pāniora (Peta Riko)',
        'es_PY' => 'Pāniora (Parakai)',
        'es_SV' => 'Pāniora (Whakaora)',
        'es_US' => 'Pāniora (Hononga o Amerika)',
        'es_UY' => 'Pāniora (Urukoi)',
        'es_VE' => 'Pāniora (Penehūera)',
        'et' => 'Etōniana',
        'et_EE' => 'Etōniana (Etōnia)',
        'eu' => 'Pākihi',
        'eu_ES' => 'Pākihi (Peina)',
        'fa' => 'Pāhiana',
        'fa_AF' => 'Pāhiana (Awhekenetāna)',
        'fa_IR' => 'Pāhiana (Irāna)',
        'ff' => 'Whūra',
        'ff_Adlm' => 'Whūra (Atarama)',
        'ff_Adlm_BF' => 'Whūra (Atarama, Pākina Wharo)',
        'ff_Adlm_CM' => 'Whūra (Atarama, Kamarūna)',
        'ff_Adlm_GH' => 'Whūra (Atarama, Kāna)',
        'ff_Adlm_GM' => 'Whūra (Atarama, Kamopia)',
        'ff_Adlm_GN' => 'Whūra (Atarama, Kini)',
        'ff_Adlm_GW' => 'Whūra (Atarama, Kini-Pihao)',
        'ff_Adlm_LR' => 'Whūra (Atarama, Raipiria)',
        'ff_Adlm_MR' => 'Whūra (Atarama, Mauritānia)',
        'ff_Adlm_NE' => 'Whūra (Atarama, Ngāika)',
        'ff_Adlm_NG' => 'Whūra (Atarama, Ngāitiria)',
        'ff_Adlm_SL' => 'Whūra (Atarama, Te Araone)',
        'ff_Adlm_SN' => 'Whūra (Atarama, Henekara)',
        'ff_CM' => 'Whūra (Kamarūna)',
        'ff_GN' => 'Whūra (Kini)',
        'ff_Latn' => 'Whūra (Rātini)',
        'ff_Latn_BF' => 'Whūra (Rātini, Pākina Wharo)',
        'ff_Latn_CM' => 'Whūra (Rātini, Kamarūna)',
        'ff_Latn_GH' => 'Whūra (Rātini, Kāna)',
        'ff_Latn_GM' => 'Whūra (Rātini, Kamopia)',
        'ff_Latn_GN' => 'Whūra (Rātini, Kini)',
        'ff_Latn_GW' => 'Whūra (Rātini, Kini-Pihao)',
        'ff_Latn_LR' => 'Whūra (Rātini, Raipiria)',
        'ff_Latn_MR' => 'Whūra (Rātini, Mauritānia)',
        'ff_Latn_NE' => 'Whūra (Rātini, Ngāika)',
        'ff_Latn_NG' => 'Whūra (Rātini, Ngāitiria)',
        'ff_Latn_SL' => 'Whūra (Rātini, Te Araone)',
        'ff_Latn_SN' => 'Whūra (Rātini, Henekara)',
        'ff_MR' => 'Whūra (Mauritānia)',
        'ff_SN' => 'Whūra (Henekara)',
        'fi' => 'Whinirānia',
        'fi_FI' => 'Whinirānia (Whinarana)',
        'fo' => 'Wharoīhi',
        'fo_DK' => 'Wharoīhi (Tenemāka)',
        'fo_FO' => 'Wharoīhi (Motu Wharau)',
        'fr' => 'Wīwī',
        'fr_BE' => 'Wīwī (Peretiama)',
        'fr_BF' => 'Wīwī (Pākina Wharo)',
        'fr_BI' => 'Wīwī (Puruniti)',
        'fr_BJ' => 'Wīwī (Penīna)',
        'fr_BL' => 'Wīwī (Hato Pāteremi)',
        'fr_CA' => 'Wīwī (Kānata)',
        'fr_CD' => 'Wīwī (Kōngo - Kinihāha)',
        'fr_CF' => 'Wīwī (Te Whenua Tūhake o Āwherika Waenga)',
        'fr_CG' => 'Wīwī (Kōngo - Pārawhe)',
        'fr_CH' => 'Wīwī (Huiterangi)',
        'fr_CI' => 'Wīwī (Te Tai Rei)',
        'fr_CM' => 'Wīwī (Kamarūna)',
        'fr_DJ' => 'Wīwī (Tipūti)',
        'fr_DZ' => 'Wīwī (Aratiria)',
        'fr_FR' => 'Wīwī (Wīwī)',
        'fr_GA' => 'Wīwī (Kāpona)',
        'fr_GF' => 'Wīwī (Kiāna Wīwī)',
        'fr_GN' => 'Wīwī (Kini)',
        'fr_GP' => 'Wīwī (Kuatarupa)',
        'fr_GQ' => 'Wīwī (Kini Ekuatoria)',
        'fr_HT' => 'Wīwī (Haiti)',
        'fr_KM' => 'Wīwī (Komoro)',
        'fr_LU' => 'Wīwī (Rakapuō)',
        'fr_MA' => 'Wīwī (Moroko)',
        'fr_MC' => 'Wīwī (Monāko)',
        'fr_MF' => 'Wīwī (Hato Mātene)',
        'fr_MG' => 'Wīwī (Matakāhika)',
        'fr_ML' => 'Wīwī (Māri)',
        'fr_MQ' => 'Wīwī (Mātiniki)',
        'fr_MR' => 'Wīwī (Mauritānia)',
        'fr_MU' => 'Wīwī (Marihi)',
        'fr_NC' => 'Wīwī (Whenua Kanaki)',
        'fr_NE' => 'Wīwī (Ngāika)',
        'fr_PF' => 'Wīwī (Poronēhia Wīwī)',
        'fr_PM' => 'Wīwī (Hato Piere & Mikerona)',
        'fr_RE' => 'Wīwī (Reūnio)',
        'fr_RW' => 'Wīwī (Rāwana)',
        'fr_SC' => 'Wīwī (Heikere)',
        'fr_SN' => 'Wīwī (Henekara)',
        'fr_SY' => 'Wīwī (Hiria)',
        'fr_TD' => 'Wīwī (Kāta)',
        'fr_TG' => 'Wīwī (Toko)',
        'fr_TN' => 'Wīwī (Tūnihia)',
        'fr_VU' => 'Wīwī (Whenuatū)',
        'fr_WF' => 'Wīwī (Warihi me Whutuna)',
        'fr_YT' => 'Wīwī (Māiota)',
        'fy' => 'Whirīhiana ki te Uru',
        'fy_NL' => 'Whirīhiana ki te Uru (Hōrana)',
        'ga' => 'Airihi',
        'ga_GB' => 'Airihi (Te Hononga o Piritene)',
        'ga_IE' => 'Airihi (Airani)',
        'gd' => 'Keiriki Kotimana',
        'gd_GB' => 'Keiriki Kotimana (Te Hononga o Piritene)',
        'gl' => 'Karīhia',
        'gl_ES' => 'Karīhia (Peina)',
        'gu' => 'Kutarāti',
        'gu_IN' => 'Kutarāti (Inia)',
        'gv' => 'Manaki',
        'gv_IM' => 'Manaki (Te Moutere Mana)',
        'ha' => 'Hauha',
        'ha_GH' => 'Hauha (Kāna)',
        'ha_NE' => 'Hauha (Ngāika)',
        'ha_NG' => 'Hauha (Ngāitiria)',
        'he' => 'Hīperu',
        'he_IL' => 'Hīperu (Iharaira)',
        'hi' => 'Hīni',
        'hi_IN' => 'Hīni (Inia)',
        'hi_Latn' => 'Hīni (Rātini)',
        'hi_Latn_IN' => 'Hīni (Rātini, Inia)',
        'hr' => 'Koroātiana',
        'hr_BA' => 'Koroātiana (Pōngia & Herekōwini)',
        'hr_HR' => 'Koroātiana (Koroātia)',
        'hu' => 'Hanekari',
        'hu_HU' => 'Hanekari (Hanekari)',
        'hy' => 'Āmeniana',
        'hy_AM' => 'Āmeniana (Āmenia)',
        'ia' => 'Inarīngua',
        'ia_001' => 'Inarīngua (te ao)',
        'id' => 'Initonīhiana',
        'id_ID' => 'Initonīhiana (Initonīhia)',
        'ig' => 'Ikapo',
        'ig_NG' => 'Ikapo (Ngāitiria)',
        'ii' => 'Hīhuana Eī',
        'ii_CN' => 'Hīhuana Eī (Haina)',
        'is' => 'Tiorangi',
        'is_IS' => 'Tiorangi (Tiorangi)',
        'it' => 'Itāriana',
        'it_CH' => 'Itāriana (Huiterangi)',
        'it_IT' => 'Itāriana (Itāria)',
        'it_SM' => 'Itāriana (Hana Marino)',
        'it_VA' => 'Itāriana (Te Poho-o-Pita)',
        'ja' => 'Hapanihi',
        'ja_JP' => 'Hapanihi (Hapani)',
        'jv' => 'Hāwhanihi',
        'jv_ID' => 'Hāwhanihi (Initonīhia)',
        'ka' => 'Hōriana',
        'ka_GE' => 'Hōriana (Hōria)',
        'ki' => 'Kikūiu',
        'ki_KE' => 'Kikūiu (Kenia)',
        'kk' => 'Kahāka',
        'kk_Arab' => 'Kahāka (Arapika)',
        'kk_Arab_CN' => 'Kahāka (Arapika, Haina)',
        'kk_Cyrl' => 'Kahāka (Hīririki)',
        'kk_Cyrl_KZ' => 'Kahāka (Hīririki, Katatānga)',
        'kk_KZ' => 'Kahāka (Katatānga)',
        'kl' => 'Kararīhutu',
        'kl_GL' => 'Kararīhutu (Whenuakāriki)',
        'km' => 'Kimēra',
        'km_KH' => 'Kimēra (Kamapōtia)',
        'kn' => 'Kanara',
        'kn_IN' => 'Kanara (Inia)',
        'ko' => 'Kōreana',
        'ko_CN' => 'Kōreana (Haina)',
        'ko_KP' => 'Kōreana (Kōrea ki te Raki)',
        'ko_KR' => 'Kōreana (Kōrea ki te Tonga)',
        'ks' => 'Kahimiri',
        'ks_Arab' => 'Kahimiri (Arapika)',
        'ks_Arab_IN' => 'Kahimiri (Arapika, Inia)',
        'ks_Deva' => 'Kahimiri (Tewhangāngari)',
        'ks_Deva_IN' => 'Kahimiri (Tewhangāngari, Inia)',
        'ks_IN' => 'Kahimiri (Inia)',
        'ku' => 'Kūrihi',
        'ku_Latn' => 'Kūrihi (Rātini)',
        'ku_Latn_IQ' => 'Kūrihi (Rātini, Irāka)',
        'ku_Latn_SY' => 'Kūrihi (Rātini, Hiria)',
        'ku_Latn_TR' => 'Kūrihi (Rātini, Tākei)',
        'ku_SY' => 'Kūrihi (Hiria)',
        'ku_TR' => 'Kūrihi (Tākei)',
        'kw' => 'Kōnihi',
        'kw_GB' => 'Kōnihi (Te Hononga o Piritene)',
        'ky' => 'Kiakihi',
        'ky_KG' => 'Kiakihi (Kikitānga)',
        'lb' => 'Rakapuō',
        'lb_LU' => 'Rakapuō (Rakapuō)',
        'lg' => 'Kānata',
        'lg_UG' => 'Kānata (Ukānga)',
        'ln' => 'Ringāra',
        'ln_AO' => 'Ringāra (Anakora)',
        'ln_CD' => 'Ringāra (Kōngo - Kinihāha)',
        'ln_CF' => 'Ringāra (Te Whenua Tūhake o Āwherika Waenga)',
        'ln_CG' => 'Ringāra (Kōngo - Pārawhe)',
        'lo' => 'Rao',
        'lo_LA' => 'Rao (Rāoho)',
        'lt' => 'Rituānia',
        'lt_LT' => 'Rituānia (Rituānia)',
        'lu' => 'Rupa Katanga',
        'lu_CD' => 'Rupa Katanga (Kōngo - Kinihāha)',
        'lv' => 'Rāwhia',
        'lv_LV' => 'Rāwhia (Rāwhia)',
        'mg' => 'Marakāhi',
        'mg_MG' => 'Marakāhi (Matakāhika)',
        'mi' => 'Māori',
        'mi_NZ' => 'Māori (Aotearoa)',
        'mk' => 'Makerōnia',
        'mk_MK' => 'Makerōnia (Makerōnia ki te Raki)',
        'ml' => 'Mareiārama',
        'ml_IN' => 'Mareiārama (Inia)',
        'mn' => 'Mongōria',
        'mn_MN' => 'Mongōria (Mongōria)',
        'mr' => 'Marati',
        'mr_IN' => 'Marati (Inia)',
        'ms' => 'Marei',
        'ms_BN' => 'Marei (Poronai)',
        'ms_ID' => 'Marei (Initonīhia)',
        'ms_MY' => 'Marei (Mareia)',
        'ms_SG' => 'Marei (Hingapoa)',
        'mt' => 'Mārata',
        'mt_MT' => 'Mārata (Mārata)',
        'my' => 'Pēmīhi',
        'my_MM' => 'Pēmīhi (Pēma)',
        'nb' => 'Pakamō Nōwei',
        'nb_NO' => 'Pakamō Nōwei (Nōwei)',
        'nb_SJ' => 'Pakamō Nōwei (Heopara me Iana Maiana)',
        'nd' => 'Enetepēra ki te Raki',
        'nd_ZW' => 'Enetepēra ki te Raki (Timuwawe)',
        'ne' => 'Nepari',
        'ne_IN' => 'Nepari (Inia)',
        'ne_NP' => 'Nepari (Nepōra)',
        'nl' => 'Tati',
        'nl_AW' => 'Tati (Arūpa)',
        'nl_BE' => 'Tati (Peretiama)',
        'nl_BQ' => 'Tati (Karapīana Hōrana)',
        'nl_CW' => 'Tati (Kurahao)',
        'nl_NL' => 'Tati (Hōrana)',
        'nl_SR' => 'Tati (Huriname)',
        'nl_SX' => 'Tati (Hiti Mātene)',
        'nn' => 'Nīnōka Nōwei',
        'nn_NO' => 'Nīnōka Nōwei (Nōwei)',
        'no' => 'Nōwei',
        'no_NO' => 'Nōwei (Nōwei)',
        'oc' => 'Ōkitana',
        'oc_ES' => 'Ōkitana (Peina)',
        'oc_FR' => 'Ōkitana (Wīwī)',
        'om' => 'Ōromo',
        'om_ET' => 'Ōromo (Etiopia)',
        'om_KE' => 'Ōromo (Kenia)',
        'or' => 'Ōtia',
        'or_IN' => 'Ōtia (Inia)',
        'os' => 'Ōtītiki',
        'os_GE' => 'Ōtītiki (Hōria)',
        'os_RU' => 'Ōtītiki (Rūhia)',
        'pa' => 'Punutapi',
        'pa_Arab' => 'Punutapi (Arapika)',
        'pa_Arab_PK' => 'Punutapi (Arapika, Pakitāne)',
        'pa_Guru' => 'Punutapi (Kūmuki)',
        'pa_Guru_IN' => 'Punutapi (Kūmuki, Inia)',
        'pa_IN' => 'Punutapi (Inia)',
        'pa_PK' => 'Punutapi (Pakitāne)',
        'pl' => 'Pōrihi',
        'pl_PL' => 'Pōrihi (Pōrana)',
        'ps' => 'Pāhitō',
        'ps_AF' => 'Pāhitō (Awhekenetāna)',
        'ps_PK' => 'Pāhitō (Pakitāne)',
        'pt' => 'Pōtukīhi',
        'pt_AO' => 'Pōtukīhi (Anakora)',
        'pt_BR' => 'Pōtukīhi (Parīhi)',
        'pt_CH' => 'Pōtukīhi (Huiterangi)',
        'pt_CV' => 'Pōtukīhi (Te Kūrae Matomato)',
        'pt_GQ' => 'Pōtukīhi (Kini Ekuatoria)',
        'pt_GW' => 'Pōtukīhi (Kini-Pihao)',
        'pt_LU' => 'Pōtukīhi (Rakapuō)',
        'pt_MO' => 'Pōtukīhi (Makau Haina)',
        'pt_MZ' => 'Pōtukīhi (Mohapiki)',
        'pt_PT' => 'Pōtukīhi (Potukara)',
        'pt_ST' => 'Pōtukīhi (Hato Tomei me Pirinipei)',
        'pt_TL' => 'Pōtukīhi (Tīmoa ki te Rāwhiti)',
        'qu' => 'Kētua',
        'qu_BO' => 'Kētua (Poriwia)',
        'qu_EC' => 'Kētua (Ekuatoa)',
        'qu_PE' => 'Kētua (Peru)',
        'rm' => 'Romānihi',
        'rm_CH' => 'Romānihi (Huiterangi)',
        'rn' => 'Rūniti',
        'rn_BI' => 'Rūniti (Puruniti)',
        'ro' => 'Romeinia',
        'ro_MD' => 'Romeinia (Morotawa)',
        'ro_RO' => 'Romeinia (Romeinia)',
        'ru' => 'Ruhiana',
        'ru_BY' => 'Ruhiana (Pērara)',
        'ru_KG' => 'Ruhiana (Kikitānga)',
        'ru_KZ' => 'Ruhiana (Katatānga)',
        'ru_MD' => 'Ruhiana (Morotawa)',
        'ru_RU' => 'Ruhiana (Rūhia)',
        'ru_UA' => 'Ruhiana (Ukareinga)',
        'rw' => 'Kiniawāna',
        'rw_RW' => 'Kiniawāna (Rāwana)',
        'sa' => 'Hanahiti',
        'sa_IN' => 'Hanahiti (Inia)',
        'sc' => 'Hārinia',
        'sc_IT' => 'Hārinia (Itāria)',
        'sd' => 'Hiniti',
        'sd_Arab' => 'Hiniti (Arapika)',
        'sd_Arab_PK' => 'Hiniti (Arapika, Pakitāne)',
        'sd_Deva' => 'Hiniti (Tewhangāngari)',
        'sd_Deva_IN' => 'Hiniti (Tewhangāngari, Inia)',
        'sd_IN' => 'Hiniti (Inia)',
        'sd_PK' => 'Hiniti (Pakitāne)',
        'se' => 'Hami ki te Raki',
        'se_FI' => 'Hami ki te Raki (Whinarana)',
        'se_NO' => 'Hami ki te Raki (Nōwei)',
        'se_SE' => 'Hami ki te Raki (Huitene)',
        'sg' => 'Hāngo',
        'sg_CF' => 'Hāngo (Te Whenua Tūhake o Āwherika Waenga)',
        'si' => 'Hinihāra',
        'si_LK' => 'Hinihāra (Hiri Rānaka)',
        'sk' => 'Horowākia',
        'sk_SK' => 'Horowākia (Horowākia)',
        'sl' => 'Horowinia',
        'sl_SI' => 'Horowinia (Horowinia)',
        'sn' => 'Hōna',
        'sn_ZW' => 'Hōna (Timuwawe)',
        'so' => 'Hamāri',
        'so_DJ' => 'Hamāri (Tipūti)',
        'so_ET' => 'Hamāri (Etiopia)',
        'so_KE' => 'Hamāri (Kenia)',
        'so_SO' => 'Hamāri (Hūmārie)',
        'sq' => 'Arapeiniana',
        'sq_AL' => 'Arapeiniana (Arapeinia)',
        'sq_MK' => 'Arapeiniana (Makerōnia ki te Raki)',
        'sr' => 'Hirupia',
        'sr_BA' => 'Hirupia (Pōngia & Herekōwini)',
        'sr_Cyrl' => 'Hirupia (Hīririki)',
        'sr_Cyrl_BA' => 'Hirupia (Hīririki, Pōngia & Herekōwini)',
        'sr_Cyrl_ME' => 'Hirupia (Hīririki, Maungakororiko)',
        'sr_Cyrl_RS' => 'Hirupia (Hīririki, Hirupia)',
        'sr_Latn' => 'Hirupia (Rātini)',
        'sr_Latn_BA' => 'Hirupia (Rātini, Pōngia & Herekōwini)',
        'sr_Latn_ME' => 'Hirupia (Rātini, Maungakororiko)',
        'sr_Latn_RS' => 'Hirupia (Rātini, Hirupia)',
        'sr_ME' => 'Hirupia (Maungakororiko)',
        'sr_RS' => 'Hirupia (Hirupia)',
        'st' => 'Hōto ki te Tonga',
        'st_LS' => 'Hōto ki te Tonga (Teroto)',
        'st_ZA' => 'Hōto ki te Tonga (Āwherika ki te Tonga)',
        'su' => 'Hunanīhi',
        'su_ID' => 'Hunanīhi (Initonīhia)',
        'su_Latn' => 'Hunanīhi (Rātini)',
        'su_Latn_ID' => 'Hunanīhi (Rātini, Initonīhia)',
        'sv' => 'Huitene',
        'sv_AX' => 'Huitene (Motu Ōrana)',
        'sv_FI' => 'Huitene (Whinarana)',
        'sv_SE' => 'Huitene (Huitene)',
        'sw' => 'Wāhīri',
        'sw_CD' => 'Wāhīri (Kōngo - Kinihāha)',
        'sw_KE' => 'Wāhīri (Kenia)',
        'sw_TZ' => 'Wāhīri (Tānahia)',
        'sw_UG' => 'Wāhīri (Ukānga)',
        'ta' => 'Tamira',
        'ta_IN' => 'Tamira (Inia)',
        'ta_LK' => 'Tamira (Hiri Rānaka)',
        'ta_MY' => 'Tamira (Mareia)',
        'ta_SG' => 'Tamira (Hingapoa)',
        'te' => 'Teruku',
        'te_IN' => 'Teruku (Inia)',
        'tg' => 'Tāhiki',
        'tg_TJ' => 'Tāhiki (Takiritānga)',
        'th' => 'Tai',
        'th_TH' => 'Tai (Tairanga)',
        'ti' => 'Tekirinia',
        'ti_ER' => 'Tekirinia (Eritēria)',
        'ti_ET' => 'Tekirinia (Etiopia)',
        'tk' => 'Tākamana',
        'tk_TM' => 'Tākamana (Tukumanatānga)',
        'tn' => 'Hawāna',
        'tn_BW' => 'Hawāna (Poriwana)',
        'tn_ZA' => 'Hawāna (Āwherika ki te Tonga)',
        'to' => 'Tonga',
        'to_TO' => 'Tonga (Tonga)',
        'tr' => 'Tākei',
        'tr_CY' => 'Tākei (Haipara)',
        'tr_TR' => 'Tākei (Tākei)',
        'tt' => 'Tatā',
        'tt_RU' => 'Tatā (Rūhia)',
        'ug' => 'Wīkura',
        'ug_CN' => 'Wīkura (Haina)',
        'uk' => 'Ukareinga',
        'uk_UA' => 'Ukareinga (Ukareinga)',
        'ur' => 'Ūrutu',
        'ur_IN' => 'Ūrutu (Inia)',
        'ur_PK' => 'Ūrutu (Pakitāne)',
        'uz' => 'Ūpeke',
        'uz_AF' => 'Ūpeke (Awhekenetāna)',
        'uz_Arab' => 'Ūpeke (Arapika)',
        'uz_Arab_AF' => 'Ūpeke (Arapika, Awhekenetāna)',
        'uz_Cyrl' => 'Ūpeke (Hīririki)',
        'uz_Cyrl_UZ' => 'Ūpeke (Hīririki, Uhipeketāne)',
        'uz_Latn' => 'Ūpeke (Rātini)',
        'uz_Latn_UZ' => 'Ūpeke (Rātini, Uhipeketāne)',
        'uz_UZ' => 'Ūpeke (Uhipeketāne)',
        'vi' => 'Whitināmu',
        'vi_VN' => 'Whitināmu (Whitināmu)',
        'wo' => 'Warawhe',
        'wo_SN' => 'Warawhe (Henekara)',
        'xh' => 'Tōha',
        'xh_ZA' => 'Tōha (Āwherika ki te Tonga)',
        'yi' => 'Irihi',
        'yi_UA' => 'Irihi (Ukareinga)',
        'yo' => 'Ōrūpa',
        'yo_BJ' => 'Ōrūpa (Penīna)',
        'yo_NG' => 'Ōrūpa (Ngāitiria)',
        'zh' => 'Hainamana',
        'zh_CN' => 'Hainamana (Haina)',
        'zh_HK' => 'Hainamana (Hongipua Haina)',
        'zh_Hans' => 'Hainamana (Māmā)',
        'zh_Hans_CN' => 'Hainamana (Māmā, Haina)',
        'zh_Hans_HK' => 'Hainamana (Māmā, Hongipua Haina)',
        'zh_Hans_MO' => 'Hainamana (Māmā, Makau Haina)',
        'zh_Hans_MY' => 'Hainamana (Māmā, Mareia)',
        'zh_Hans_SG' => 'Hainamana (Māmā, Hingapoa)',
        'zh_Hant' => 'Hainamana (Tuku iho)',
        'zh_Hant_HK' => 'Hainamana (Tuku iho, Hongipua Haina)',
        'zh_Hant_MO' => 'Hainamana (Tuku iho, Makau Haina)',
        'zh_Hant_MY' => 'Hainamana (Tuku iho, Mareia)',
        'zh_Hant_TW' => 'Hainamana (Tuku iho, Taiwana)',
        'zh_MO' => 'Hainamana (Makau Haina)',
        'zh_SG' => 'Hainamana (Hingapoa)',
        'zh_TW' => 'Hainamana (Taiwana)',
        'zu' => 'Tūru',
        'zu_ZA' => 'Tūru (Āwherika ki te Tonga)',
    ],
];
