<?php

return [
    'Names' => [
        'ADP' => [
            'ADP',
            'Andorranische Pesete',
        ],
        'AED' => [
            'AED',
            'VAE-Dirham',
        ],
        'AFA' => [
            'AFA',
            'Afghanische Afghani (1927–2002)',
        ],
        'AFN' => [
            'AFN',
            'Afghanischer Afghani',
        ],
        'ALK' => [
            'ALK',
            'Albanischer Lek (1946–1965)',
        ],
        'ALL' => [
            'ALL',
            'Albanischer Lek',
        ],
        'AMD' => [
            'AMD',
            'Armenischer Dram',
        ],
        'ANG' => [
            'ANG',
            'Niederländische-Antillen-Gulden',
        ],
        'AOA' => [
            'AOA',
            'Angolanischer Kwanza',
        ],
        'AOK' => [
            'AOK',
            'Angolanischer Kwanza (1977–1990)',
        ],
        'AON' => [
            'AON',
            'Angolanischer Neuer Kwanza (1990–2000)',
        ],
        'AOR' => [
            'AOR',
            'Angolanischer Kwanza Reajustado (1995–1999)',
        ],
        'ARA' => [
            'ARA',
            'Argentinischer Austral',
        ],
        'ARL' => [
            'ARL',
            'Argentinischer Peso Ley (1970–1983)',
        ],
        'ARM' => [
            'ARM',
            'Argentinischer Peso (1881–1970)',
        ],
        'ARP' => [
            'ARP',
            'Argentinischer Peso (1983–1985)',
        ],
        'ARS' => [
            'ARS',
            'Argentinischer Peso',
        ],
        'ATS' => [
            'öS',
            'Österreichischer Schilling',
        ],
        'AUD' => [
            'AU$',
            'Australischer Dollar',
        ],
        'AWG' => [
            'AWG',
            'Aruba-Florin',
        ],
        'AZM' => [
            'AZM',
            'Aserbaidschan-Manat (1993–2006)',
        ],
        'AZN' => [
            'AZN',
            'Aserbaidschan-Manat',
        ],
        'BAD' => [
            'BAD',
            'Bosnien und Herzegowina Dinar (1992–1994)',
        ],
        'BAM' => [
            'BAM',
            'Konvertible Mark Bosnien und Herzegowina',
        ],
        'BAN' => [
            'BAN',
            'Bosnien und Herzegowina Neuer Dinar (1994–1997)',
        ],
        'BBD' => [
            'BBD',
            'Barbados-Dollar',
        ],
        'BDT' => [
            'BDT',
            'Bangladesch-Taka',
        ],
        'BEC' => [
            'BEC',
            'Belgischer Franc (konvertibel)',
        ],
        'BEF' => [
            'BEF',
            'Belgischer Franc',
        ],
        'BEL' => [
            'BEL',
            'Belgischer Finanz-Franc',
        ],
        'BGL' => [
            'BGL',
            'Bulgarische Lew (1962–1999)',
        ],
        'BGM' => [
            'BGK',
            'Bulgarischer Lew (1952–1962)',
        ],
        'BGN' => [
            'BGN',
            'Bulgarischer Lew',
        ],
        'BGO' => [
            'BGJ',
            'Bulgarischer Lew (1879–1952)',
        ],
        'BHD' => [
            'BHD',
            'Bahrain-Dinar',
        ],
        'BIF' => [
            'BIF',
            'Burundi-Franc',
        ],
        'BMD' => [
            'BMD',
            'Bermuda-Dollar',
        ],
        'BND' => [
            'BND',
            'Brunei-Dollar',
        ],
        'BOB' => [
            'BOB',
            'Bolivianischer Boliviano',
        ],
        'BOL' => [
            'BOL',
            'Bolivianischer Boliviano (1863–1963)',
        ],
        'BOP' => [
            'BOP',
            'Bolivianischer Peso',
        ],
        'BOV' => [
            'BOV',
            'Boliviansiche Mvdol',
        ],
        'BRB' => [
            'BRB',
            'Brasilianischer Cruzeiro Novo (1967–1986)',
        ],
        'BRC' => [
            'BRC',
            'Brasilianischer Cruzado (1986–1989)',
        ],
        'BRE' => [
            'BRE',
            'Brasilianischer Cruzeiro (1990–1993)',
        ],
        'BRL' => [
            'R$',
            'Brasilianischer Real',
        ],
        'BRN' => [
            'BRN',
            'Brasilianischer Cruzado Novo (1989–1990)',
        ],
        'BRR' => [
            'BRR',
            'Brasilianischer Cruzeiro (1993–1994)',
        ],
        'BRZ' => [
            'BRZ',
            'Brasilianischer Cruzeiro (1942–1967)',
        ],
        'BSD' => [
            'BSD',
            'Bahamas-Dollar',
        ],
        'BTN' => [
            'BTN',
            'Bhutan-Ngultrum',
        ],
        'BUK' => [
            'BUK',
            'Birmanischer Kyat',
        ],
        'BWP' => [
            'BWP',
            'Botswanischer Pula',
        ],
        'BYB' => [
            'BYB',
            'Belarus-Rubel (1994–1999)',
        ],
        'BYN' => [
            'BYN',
            'Weißrussischer Rubel',
        ],
        'BYR' => [
            'BYR',
            'Weißrussischer Rubel (2000–2016)',
        ],
        'BZD' => [
            'BZD',
            'Belize-Dollar',
        ],
        'CAD' => [
            'CA$',
            'Kanadischer Dollar',
        ],
        'CDF' => [
            'CDF',
            'Kongo-Franc',
        ],
        'CHE' => [
            'CHE',
            'WIR-Euro',
        ],
        'CHF' => [
            'CHF',
            'Schweizer Franken',
        ],
        'CHW' => [
            'CHW',
            'WIR Franken',
        ],
        'CLE' => [
            'CLE',
            'Chilenischer Escudo',
        ],
        'CLF' => [
            'CLF',
            'Chilenische Unidades de Fomento',
        ],
        'CLP' => [
            'CLP',
            'Chilenischer Peso',
        ],
        'CNH' => [
            'CNH',
            'Renminbi-Yuan (Offshore)',
        ],
        'CNX' => [
            'CNX',
            'Dollar der Chinesischen Volksbank',
        ],
        'CNY' => [
            'CN¥',
            'Renminbi Yuan',
        ],
        'COP' => [
            'COP',
            'Kolumbianischer Peso',
        ],
        'COU' => [
            'COU',
            'Kolumbianische Unidades de valor real',
        ],
        'CRC' => [
            'CRC',
            'Costa-Rica-Colón',
        ],
        'CSD' => [
            'CSD',
            'Serbischer Dinar (2002–2006)',
        ],
        'CSK' => [
            'CSK',
            'Tschechoslowakische Krone',
        ],
        'CUC' => [
            'CUC',
            'Kubanischer Peso (konvertibel)',
        ],
        'CUP' => [
            'CUP',
            'Kubanischer Peso',
        ],
        'CVE' => [
            'CVE',
            'Cabo-Verde-Escudo',
        ],
        'CYP' => [
            'CYP',
            'Zypern-Pfund',
        ],
        'CZK' => [
            'CZK',
            'Tschechische Krone',
        ],
        'DDM' => [
            'DDM',
            'Mark der DDR',
        ],
        'DEM' => [
            'DM',
            'Deutsche Mark',
        ],
        'DJF' => [
            'DJF',
            'Dschibuti-Franc',
        ],
        'DKK' => [
            'DKK',
            'Dänische Krone',
        ],
        'DOP' => [
            'DOP',
            'Dominikanischer Peso',
        ],
        'DZD' => [
            'DZD',
            'Algerischer Dinar',
        ],
        'ECS' => [
            'ECS',
            'Ecuadorianischer Sucre',
        ],
        'ECV' => [
            'ECV',
            'Verrechnungseinheit für Ecuador',
        ],
        'EEK' => [
            'EEK',
            'Estnische Krone',
        ],
        'EGP' => [
            'EGP',
            'Ägyptisches Pfund',
        ],
        'ERN' => [
            'ERN',
            'Eritreischer Nakfa',
        ],
        'ESA' => [
            'ESA',
            'Spanische Peseta (A–Konten)',
        ],
        'ESB' => [
            'ESB',
            'Spanische Peseta (konvertibel)',
        ],
        'ESP' => [
            'ESP',
            'Spanische Peseta',
        ],
        'ETB' => [
            'ETB',
            'Äthiopischer Birr',
        ],
        'EUR' => [
            '€',
            'Euro',
        ],
        'FIM' => [
            'FIM',
            'Finnische Mark',
        ],
        'FJD' => [
            'FJD',
            'Fidschi-Dollar',
        ],
        'FKP' => [
            'FKP',
            'Falkland-Pfund',
        ],
        'FRF' => [
            'FRF',
            'Französischer Franc',
        ],
        'GBP' => [
            '£',
            'Britisches Pfund',
        ],
        'GEK' => [
            'GEK',
            'Georgischer Kupon Larit',
        ],
        'GEL' => [
            'GEL',
            'Georgischer Lari',
        ],
        'GHC' => [
            'GHC',
            'Ghanaischer Cedi (1979–2007)',
        ],
        'GHS' => [
            'GHS',
            'Ghanaischer Cedi',
        ],
        'GIP' => [
            'GIP',
            'Gibraltar-Pfund',
        ],
        'GMD' => [
            'GMD',
            'Gambia-Dalasi',
        ],
        'GNF' => [
            'GNF',
            'Guinea-Franc',
        ],
        'GNS' => [
            'GNS',
            'Guineischer Syli',
        ],
        'GQE' => [
            'GQE',
            'Äquatorialguinea-Ekwele',
        ],
        'GRD' => [
            'GRD',
            'Griechische Drachme',
        ],
        'GTQ' => [
            'GTQ',
            'Guatemaltekischer Quetzal',
        ],
        'GWE' => [
            'GWE',
            'Portugiesisch Guinea Escudo',
        ],
        'GWP' => [
            'GWP',
            'Guinea-Bissau Peso',
        ],
        'GYD' => [
            'GYD',
            'Guyana-Dollar',
        ],
        'HKD' => [
            'HK$',
            'Hongkong-Dollar',
        ],
        'HNL' => [
            'HNL',
            'Honduras-Lempira',
        ],
        'HRD' => [
            'HRD',
            'Kroatischer Dinar',
        ],
        'HRK' => [
            'HRK',
            'Kroatischer Kuna',
        ],
        'HTG' => [
            'HTG',
            'Haitianische Gourde',
        ],
        'HUF' => [
            'HUF',
            'Ungarischer Forint',
        ],
        'IDR' => [
            'IDR',
            'Indonesische Rupiah',
        ],
        'IEP' => [
            'IEP',
            'Irisches Pfund',
        ],
        'ILP' => [
            'ILP',
            'Israelisches Pfund',
        ],
        'ILR' => [
            'ILR',
            'Israelischer Schekel (1980–1985)',
        ],
        'ILS' => [
            '₪',
            'Israelischer Neuer Schekel',
        ],
        'INR' => [
            '₹',
            'Indische Rupie',
        ],
        'IQD' => [
            'IQD',
            'Irakischer Dinar',
        ],
        'IRR' => [
            'IRR',
            'Iranischer Rial',
        ],
        'ISJ' => [
            'ISJ',
            'Isländische Krone (1918–1981)',
        ],
        'ISK' => [
            'ISK',
            'Isländische Krone',
        ],
        'ITL' => [
            'ITL',
            'Italienische Lira',
        ],
        'JMD' => [
            'JMD',
            'Jamaika-Dollar',
        ],
        'JOD' => [
            'JOD',
            'Jordanischer Dinar',
        ],
        'JPY' => [
            '¥',
            'Japanischer Yen',
        ],
        'KES' => [
            'KES',
            'Kenia-Schilling',
        ],
        'KGS' => [
            'KGS',
            'Kirgisischer Som',
        ],
        'KHR' => [
            'KHR',
            'Kambodschanischer Riel',
        ],
        'KMF' => [
            'KMF',
            'Komoren-Franc',
        ],
        'KPW' => [
            'KPW',
            'Nordkoreanischer Won',
        ],
        'KRH' => [
            'KRH',
            'Südkoreanischer Hwan (1953–1962)',
        ],
        'KRO' => [
            'KRO',
            'Südkoreanischer Won (1945–1953)',
        ],
        'KRW' => [
            '₩',
            'Südkoreanischer Won',
        ],
        'KWD' => [
            'KWD',
            'Kuwait-Dinar',
        ],
        'KYD' => [
            'KYD',
            'Kaiman-Dollar',
        ],
        'KZT' => [
            'KZT',
            'Kasachischer Tenge',
        ],
        'LAK' => [
            'LAK',
            'Laotischer Kip',
        ],
        'LBP' => [
            'LBP',
            'Libanesisches Pfund',
        ],
        'LKR' => [
            'LKR',
            'Sri-Lanka-Rupie',
        ],
        'LRD' => [
            'LRD',
            'Liberianischer Dollar',
        ],
        'LSL' => [
            'LSL',
            'Loti',
        ],
        'LTL' => [
            'LTL',
            'Litauischer Litas',
        ],
        'LTT' => [
            'LTT',
            'Litauischer Talonas',
        ],
        'LUC' => [
            'LUC',
            'Luxemburgischer Franc (konvertibel)',
        ],
        'LUF' => [
            'LUF',
            'Luxemburgischer Franc',
        ],
        'LUL' => [
            'LUL',
            'Luxemburgischer Finanz-Franc',
        ],
        'LVL' => [
            'LVL',
            'Lettischer Lats',
        ],
        'LVR' => [
            'LVR',
            'Lettischer Rubel',
        ],
        'LYD' => [
            'LYD',
            'Libyscher Dinar',
        ],
        'MAD' => [
            'MAD',
            'Marokkanischer Dirham',
        ],
        'MAF' => [
            'MAF',
            'Marokkanischer Franc',
        ],
        'MCF' => [
            'MCF',
            'Monegassischer Franc',
        ],
        'MDC' => [
            'MDC',
            'Moldau-Cupon',
        ],
        'MDL' => [
            'MDL',
            'Moldau-Leu',
        ],
        'MGA' => [
            'MGA',
            'Madagaskar-Ariary',
        ],
        'MGF' => [
            'MGF',
            'Madagaskar-Franc',
        ],
        'MKD' => [
            'MKD',
            'Mazedonischer Denar',
        ],
        'MKN' => [
            'MKN',
            'Mazedonischer Denar (1992–1993)',
        ],
        'MLF' => [
            'MLF',
            'Malischer Franc',
        ],
        'MMK' => [
            'MMK',
            'Myanmarischer Kyat',
        ],
        'MNT' => [
            'MNT',
            'Mongolischer Tögrög',
        ],
        'MOP' => [
            'MOP',
            'Macao-Pataca',
        ],
        'MRO' => [
            'MRO',
            'Mauretanischer Ouguiya (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'Mauretanischer Ouguiya',
        ],
        'MTL' => [
            'MTL',
            'Maltesische Lira',
        ],
        'MTP' => [
            'MTP',
            'Maltesisches Pfund',
        ],
        'MUR' => [
            'MUR',
            'Mauritius-Rupie',
        ],
        'MVP' => [
            'MVP',
            'Malediven-Rupie (alt)',
        ],
        'MVR' => [
            'MVR',
            'Malediven-Rufiyaa',
        ],
        'MWK' => [
            'MWK',
            'Malawi-Kwacha',
        ],
        'MXN' => [
            'MX$',
            'Mexikanischer Peso',
        ],
        'MXP' => [
            'MXP',
            'Mexikanischer Silber-Peso (1861–1992)',
        ],
        'MXV' => [
            'MXV',
            'Mexicanischer Unidad de Inversion (UDI)',
        ],
        'MYR' => [
            'MYR',
            'Malaysischer Ringgit',
        ],
        'MZE' => [
            'MZE',
            'Mosambikanischer Escudo',
        ],
        'MZM' => [
            'MZM',
            'Mosambikanischer Metical (1980–2006)',
        ],
        'MZN' => [
            'MZN',
            'Mosambikanischer Metical',
        ],
        'NAD' => [
            'NAD',
            'Namibia-Dollar',
        ],
        'NGN' => [
            'NGN',
            'Nigerianischer Naira',
        ],
        'NIC' => [
            'NIC',
            'Nicaraguanischer Córdoba (1988–1991)',
        ],
        'NIO' => [
            'NIO',
            'Nicaragua-Córdoba',
        ],
        'NLG' => [
            'NLG',
            'Niederländischer Gulden',
        ],
        'NOK' => [
            'NOK',
            'Norwegische Krone',
        ],
        'NPR' => [
            'NPR',
            'Nepalesische Rupie',
        ],
        'NZD' => [
            'NZ$',
            'Neuseeland-Dollar',
        ],
        'OMR' => [
            'OMR',
            'Omanischer Rial',
        ],
        'PAB' => [
            'PAB',
            'Panamaischer Balboa',
        ],
        'PEI' => [
            'PEI',
            'Peruanischer Inti',
        ],
        'PEN' => [
            'PEN',
            'Peruanischer Sol',
        ],
        'PES' => [
            'PES',
            'Peruanischer Sol (1863–1965)',
        ],
        'PGK' => [
            'PGK',
            'Papua-neuguineischer Kina',
        ],
        'PHP' => [
            'PHP',
            'Philippinischer Peso',
        ],
        'PKR' => [
            'PKR',
            'Pakistanische Rupie',
        ],
        'PLN' => [
            'PLN',
            'Polnischer Złoty',
        ],
        'PLZ' => [
            'PLZ',
            'Polnischer Zloty (1950–1995)',
        ],
        'PTE' => [
            'PTE',
            'Portugiesischer Escudo',
        ],
        'PYG' => [
            'PYG',
            'Paraguayischer Guaraní',
        ],
        'QAR' => [
            'QAR',
            'Katar-Riyal',
        ],
        'RHD' => [
            'RHD',
            'Rhodesischer Dollar',
        ],
        'ROL' => [
            'ROL',
            'Rumänischer Leu (1952–2006)',
        ],
        'RON' => [
            'RON',
            'Rumänischer Leu',
        ],
        'RSD' => [
            'RSD',
            'Serbischer Dinar',
        ],
        'RUB' => [
            'RUB',
            'Russischer Rubel',
        ],
        'RUR' => [
            'RUR',
            'Russischer Rubel (1991–1998)',
        ],
        'RWF' => [
            'RWF',
            'Ruanda-Franc',
        ],
        'SAR' => [
            'SAR',
            'Saudi-Rial',
        ],
        'SBD' => [
            'SBD',
            'Salomonen-Dollar',
        ],
        'SCR' => [
            'SCR',
            'Seychellen-Rupie',
        ],
        'SDD' => [
            'SDD',
            'Sudanesischer Dinar (1992–2007)',
        ],
        'SDG' => [
            'SDG',
            'Sudanesisches Pfund',
        ],
        'SDP' => [
            'SDP',
            'Sudanesisches Pfund (1957–1998)',
        ],
        'SEK' => [
            'SEK',
            'Schwedische Krone',
        ],
        'SGD' => [
            'SGD',
            'Singapur-Dollar',
        ],
        'SHP' => [
            'SHP',
            'St.-Helena-Pfund',
        ],
        'SIT' => [
            'SIT',
            'Slowenischer Tolar',
        ],
        'SKK' => [
            'SKK',
            'Slowakische Krone',
        ],
        'SLE' => [
            'SLE',
            'Sierra-leonischer Leone',
        ],
        'SLL' => [
            'SLL',
            'Sierra-leonischer Leone (1964–2022)',
        ],
        'SOS' => [
            'SOS',
            'Somalia-Schilling',
        ],
        'SRD' => [
            'SRD',
            'Suriname-Dollar',
        ],
        'SRG' => [
            'SRG',
            'Suriname Gulden',
        ],
        'SSP' => [
            'SSP',
            'Südsudanesisches Pfund',
        ],
        'STD' => [
            'STD',
            'São-toméischer Dobra (1977–2017)',
        ],
        'STN' => [
            'STN',
            'São-toméischer Dobra',
        ],
        'SUR' => [
            'SUR',
            'Sowjetischer Rubel',
        ],
        'SVC' => [
            'SVC',
            'El Salvador Colon',
        ],
        'SYP' => [
            'SYP',
            'Syrisches Pfund',
        ],
        'SZL' => [
            'SZL',
            'Swasiländischer Lilangeni',
        ],
        'THB' => [
            '฿',
            'Thailändischer Baht',
        ],
        'TJR' => [
            'TJR',
            'Tadschikistan Rubel',
        ],
        'TJS' => [
            'TJS',
            'Tadschikistan-Somoni',
        ],
        'TMM' => [
            'TMM',
            'Turkmenistan-Manat (1993–2009)',
        ],
        'TMT' => [
            'TMT',
            'Turkmenistan-Manat',
        ],
        'TND' => [
            'TND',
            'Tunesischer Dinar',
        ],
        'TOP' => [
            'TOP',
            'Tongaischer Paʻanga',
        ],
        'TPE' => [
            'TPE',
            'Timor-Escudo',
        ],
        'TRL' => [
            'TRL',
            'Türkische Lira (1922–2005)',
        ],
        'TRY' => [
            'TRY',
            'Türkische Lira',
        ],
        'TTD' => [
            'TTD',
            'Trinidad-und-Tobago-Dollar',
        ],
        'TWD' => [
            'NT$',
            'Neuer Taiwan-Dollar',
        ],
        'TZS' => [
            'TZS',
            'Tansania-Schilling',
        ],
        'UAH' => [
            'UAH',
            'Ukrainische Hrywnja',
        ],
        'UAK' => [
            'UAK',
            'Ukrainischer Karbovanetz',
        ],
        'UGS' => [
            'UGS',
            'Uganda-Schilling (1966–1987)',
        ],
        'UGX' => [
            'UGX',
            'Uganda-Schilling',
        ],
        'USD' => [
            '$',
            'US-Dollar',
        ],
        'USN' => [
            'USN',
            'US Dollar (Nächster Tag)',
        ],
        'USS' => [
            'USS',
            'US Dollar (Gleicher Tag)',
        ],
        'UYI' => [
            'UYI',
            'Uruguayischer Peso (Indexierte Rechnungseinheiten)',
        ],
        'UYP' => [
            'UYP',
            'Uruguayischer Peso (1975–1993)',
        ],
        'UYU' => [
            'UYU',
            'Uruguayischer Peso',
        ],
        'UZS' => [
            'UZS',
            'Usbekistan-Sum',
        ],
        'VEB' => [
            'VEB',
            'Venezolanischer Bolívar (1871–2008)',
        ],
        'VEF' => [
            'VEF',
            'Venezolanischer Bolívar (2008–2018)',
        ],
        'VES' => [
            'VES',
            'Venezolanischer Bolívar',
        ],
        'VND' => [
            '₫',
            'Vietnamesischer Dong',
        ],
        'VNN' => [
            'VNN',
            'Vietnamesischer Dong(1978–1985)',
        ],
        'VUV' => [
            'VUV',
            'Vanuatu-Vatu',
        ],
        'WST' => [
            'WST',
            'Samoanischer Tala',
        ],
        'XAF' => [
            'FCFA',
            'CFA-Franc (BEAC)',
        ],
        'XCD' => [
            'EC$',
            'Ostkaribischer Dollar',
        ],
        'XCG' => [
            'Cg.',
            'Karibischer Gulden',
        ],
        'XEU' => [
            'XEU',
            'Europäische Währungseinheit (XEU)',
        ],
        'XFO' => [
            'XFO',
            'Französischer Gold-Franc',
        ],
        'XFU' => [
            'XFU',
            'Französischer UIC-Franc',
        ],
        'XOF' => [
            'F CFA',
            'CFA-Franc (BCEAO)',
        ],
        'XPF' => [
            'CFPF',
            'CFP-Franc',
        ],
        'XRE' => [
            'XRE',
            'RINET Funds',
        ],
        'YDD' => [
            'YDD',
            'Jemen-Dinar',
        ],
        'YER' => [
            'YER',
            'Jemen-Rial',
        ],
        'YUD' => [
            'YUD',
            'Jugoslawischer Dinar (1966–1990)',
        ],
        'YUM' => [
            'YUM',
            'Jugoslawischer Neuer Dinar (1994–2002)',
        ],
        'YUN' => [
            'YUN',
            'Jugoslawischer Dinar (konvertibel)',
        ],
        'YUR' => [
            'YUR',
            'Jugoslawischer reformierter Dinar (1992–1993)',
        ],
        'ZAL' => [
            'ZAL',
            'Südafrikanischer Rand (Finanz)',
        ],
        'ZAR' => [
            'ZAR',
            'Südafrikanischer Rand',
        ],
        'ZMK' => [
            'ZMK',
            'Kwacha (1968–2012)',
        ],
        'ZMW' => [
            'ZMW',
            'Kwacha',
        ],
        'ZRN' => [
            'ZRN',
            'Zaire-Neuer Zaïre (1993–1998)',
        ],
        'ZRZ' => [
            'ZRZ',
            'Zaire-Zaïre (1971–1993)',
        ],
        'ZWD' => [
            'ZWD',
            'Simbabwe-Dollar (1980–2008)',
        ],
        'ZWG' => [
            'ZWG',
            'Simbabwe-Gold',
        ],
        'ZWL' => [
            'ZWL',
            'Simbabwe-Dollar (2009)',
        ],
        'ZWR' => [
            'ZWR',
            'Simbabwe-Dollar (2008)',
        ],
    ],
];
