<?php

return [
    'Names' => [
        'AED' => [
            'AED',
            'Արաբական Միացյալ Էմիրությունների դիրհամ',
        ],
        'AFN' => [
            'AFN',
            'աֆղանական աֆղանի',
        ],
        'ALL' => [
            'ALL',
            'ալբանական լեկ',
        ],
        'AMD' => [
            '֏',
            'հայկական դրամ',
        ],
        'ANG' => [
            'ANG',
            'նիդեռլանդական անտիլյան գուլդեն',
        ],
        'AOA' => [
            'AOA',
            'անգոլական կվանզա',
        ],
        'ARS' => [
            'ARS',
            'արգենտինական պեսո',
        ],
        'AUD' => [
            'A$',
            'ավստրալիական դոլար',
        ],
        'AWG' => [
            'AWG',
            'արուբական ֆլորին',
        ],
        'AZN' => [
            'AZN',
            'ադրբեջանական մանաթ',
        ],
        'BAM' => [
            'BAM',
            'Բոսնիա և Հերցեգովինայի փոխարկվող մարկ',
        ],
        'BBD' => [
            'BBD',
            'բարբադոսյան դոլար',
        ],
        'BDT' => [
            'BDT',
            'Բանգլադեշի տակա',
        ],
        'BGN' => [
            'BGN',
            'բուլղարական լև',
        ],
        'BHD' => [
            'BHD',
            'Բահրեյնի դինար',
        ],
        'BIF' => [
            'BIF',
            'բուրունդիական ֆրանկ',
        ],
        'BMD' => [
            'BMD',
            'բերմուդյան դոլար',
        ],
        'BND' => [
            'BND',
            'Բրունեյի դոլար',
        ],
        'BOB' => [
            'BOB',
            'բոլիվիական բոլիվիանո',
        ],
        'BRL' => [
            'R$',
            'բրազիլական ռեալ',
        ],
        'BSD' => [
            'BSD',
            'բահամյան դոլար',
        ],
        'BTN' => [
            'BTN',
            'բութանական նգուլտրում',
        ],
        'BWP' => [
            'BWP',
            'բոտսվանական պուլա',
        ],
        'BYN' => [
            'BYN',
            'բելառուսական ռուբլի',
        ],
        'BYR' => [
            'BYR',
            'Բելառուսական ռուբլի (2000–2016)',
        ],
        'BZD' => [
            'BZD',
            'Բելիզի դոլար',
        ],
        'CAD' => [
            'CA$',
            'կանադական դոլար',
        ],
        'CDF' => [
            'CDF',
            'Կոնգոյի ֆրանկ',
        ],
        'CHF' => [
            'CHF',
            'շվեյցարական ֆրանկ',
        ],
        'CLP' => [
            'CLP',
            'չիլիական պեսո',
        ],
        'CNH' => [
            'CNH',
            'չինական օֆշորային յուան',
        ],
        'CNY' => [
            'CN¥',
            'չինական յուան',
        ],
        'COP' => [
            'COP',
            'կոլումբիական պեսո',
        ],
        'CRC' => [
            'CRC',
            'Կոստա Ռիկայի կոլոն',
        ],
        'CUC' => [
            'CUC',
            'կուբայական փոխարկվող պեսո',
        ],
        'CUP' => [
            'CUP',
            'կուբայական պեսո',
        ],
        'CVE' => [
            'CVE',
            'Կաբո Վերդեի էսկուդո',
        ],
        'CZK' => [
            'CZK',
            'չեխական կրոնա',
        ],
        'DJF' => [
            'DJF',
            'Ջիբութիի ֆրանկ',
        ],
        'DKK' => [
            'DKK',
            'դանիական կրոն',
        ],
        'DOP' => [
            'DOP',
            'դոմինիկյան պեսո',
        ],
        'DZD' => [
            'DZD',
            'ալժիրական դինար',
        ],
        'EGP' => [
            'EGP',
            'եգիպտական ֆունտ',
        ],
        'ERN' => [
            'ERN',
            'էրիթրեական նակվա',
        ],
        'ETB' => [
            'ETB',
            'եթովպիական բիր',
        ],
        'EUR' => [
            '€',
            'եվրո',
        ],
        'FJD' => [
            'FJD',
            'ֆիջիական դոլար',
        ],
        'FKP' => [
            'FKP',
            'Ֆոլքլենդյան կղզիների ֆունտ',
        ],
        'GBP' => [
            '£',
            'բրիտանական ֆունտ ստերլինգ',
        ],
        'GEL' => [
            'GEL',
            'վրացական լարի',
        ],
        'GHS' => [
            'GHS',
            'գայանական սեդի',
        ],
        'GIP' => [
            'GIP',
            'Ջիբրալթարի ֆունտ',
        ],
        'GMD' => [
            'GMD',
            'գամբիական դալասի',
        ],
        'GNF' => [
            'GNF',
            'գվինեական ֆրանկ',
        ],
        'GTQ' => [
            'GTQ',
            'գվատեմալական կետսալ',
        ],
        'GYD' => [
            'GYD',
            'գայանական դոլար',
        ],
        'HKD' => [
            'HK$',
            'Հոնկոնգի դոլար',
        ],
        'HNL' => [
            'HNL',
            'հոնդուրասական լեմպիրա',
        ],
        'HRK' => [
            'HRK',
            'խորվաթական կունա',
        ],
        'HTG' => [
            'HTG',
            'հայիթյան գուրդ',
        ],
        'HUF' => [
            'HUF',
            'հունգարական ֆորինտ',
        ],
        'IDR' => [
            'IDR',
            'ինդոնեզիական ռուփի',
        ],
        'ILS' => [
            '₪',
            'Իսրայելի նոր շեկել',
        ],
        'INR' => [
            '₹',
            'հնդկական ռուփի',
        ],
        'IQD' => [
            'IQD',
            'իրաքյան դինար',
        ],
        'IRR' => [
            'IRR',
            'իրանական ռիալ',
        ],
        'ISK' => [
            'ISK',
            'իսլանդական կրոն',
        ],
        'JMD' => [
            'JMD',
            'Ճամայկայի դոլար',
        ],
        'JOD' => [
            'JOD',
            'հորդանանյան դինար',
        ],
        'JPY' => [
            'JP¥',
            'ճապոնական իեն',
        ],
        'KES' => [
            'KES',
            'քենիական շիլինգ',
        ],
        'KGS' => [
            'KGS',
            'ղրղզական սոմ',
        ],
        'KHR' => [
            'KHR',
            'կամբոջական ռիել',
        ],
        'KMF' => [
            'KMF',
            'կոմորյան ֆրանկ',
        ],
        'KPW' => [
            'KPW',
            'հյուսիսկորեական վոն',
        ],
        'KRW' => [
            '₩',
            'հարավկորեական վոն',
        ],
        'KWD' => [
            'KWD',
            'Քուվեյթի դինար',
        ],
        'KYD' => [
            'KYD',
            'Կայմանյան կղզիների դոլար',
        ],
        'KZT' => [
            'KZT',
            'ղազախական տենգե',
        ],
        'LAK' => [
            'LAK',
            'լաոսական կիպ',
        ],
        'LBP' => [
            'LBP',
            'լիբանանյան ֆունտ',
        ],
        'LKR' => [
            'LKR',
            'Շրի Լանկայի ռուփի',
        ],
        'LRD' => [
            'LRD',
            'լիբերիական դոլար',
        ],
        'LSL' => [
            'LSL',
            'լեսոթոյական լոտի',
        ],
        'LTL' => [
            'LTL',
            'Լիտվական լիտ',
        ],
        'LVL' => [
            'LVL',
            'Լատվիական լատ',
        ],
        'LYD' => [
            'LYD',
            'լիբիական դինար',
        ],
        'MAD' => [
            'MAD',
            'Մարոկկոյի դիրհամ',
        ],
        'MDL' => [
            'MDL',
            'մոլդովական լեյ',
        ],
        'MGA' => [
            'MGA',
            'Մադագասկարի արիարի',
        ],
        'MKD' => [
            'MKD',
            'մակեդոնական դենար',
        ],
        'MMK' => [
            'MMK',
            'Մյանմայի կյատ',
        ],
        'MNT' => [
            'MNT',
            'մոնղոլական տուգրիկ',
        ],
        'MOP' => [
            'MOP',
            'Մակաոյի պատակա',
        ],
        'MRO' => [
            'MRO',
            'մավրիտանական ուգիյա (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'մավրիտանական ուգիյա',
        ],
        'MUR' => [
            'MUR',
            'մավրիկյան ռուփի',
        ],
        'MVR' => [
            'MVR',
            'մալդիվյան ռուֆիյա',
        ],
        'MWK' => [
            'MWK',
            'մալավիական կվաչա',
        ],
        'MXN' => [
            'MX$',
            'մեքսիկական պեսո',
        ],
        'MYR' => [
            'MYR',
            'մալայզիական ռինգիտ',
        ],
        'MZN' => [
            'MZN',
            'մոզամբիկյան մետիկալ',
        ],
        'NAD' => [
            'NAD',
            'նամիբիական դոլար',
        ],
        'NGN' => [
            'NGN',
            'նիգերիական նայրա',
        ],
        'NIO' => [
            'NIO',
            'նիկարագուական կորդոբա',
        ],
        'NOK' => [
            'NOK',
            'նորվեգական կրոն',
        ],
        'NPR' => [
            'NPR',
            'Նեպալի ռուփի',
        ],
        'NZD' => [
            'NZ$',
            'նորզելանդական դոլար',
        ],
        'OMR' => [
            'OMR',
            'Օմանի ռիալ',
        ],
        'PAB' => [
            'PAB',
            'պանամական բալբոա',
        ],
        'PEN' => [
            'PEN',
            'Պերուի սոլ',
        ],
        'PGK' => [
            'PGK',
            'Պապուա Նոր Գվինեայի կինա',
        ],
        'PHP' => [
            'PHP',
            'ֆիլիպինյան պեսո',
        ],
        'PKR' => [
            'PKR',
            'պակիստանյան ռուփի',
        ],
        'PLN' => [
            'PLN',
            'լեհական զլոտի',
        ],
        'PYG' => [
            'PYG',
            'պարագվայական գուարանի',
        ],
        'QAR' => [
            'QAR',
            'Կատարի ռիալ',
        ],
        'RON' => [
            'RON',
            'ռումինական լեյ',
        ],
        'RSD' => [
            'RSD',
            'սերբական դինար',
        ],
        'RUB' => [
            'RUB',
            'ռուսական ռուբլի',
        ],
        'RWF' => [
            'RWF',
            'ռուանդական ֆրանկ',
        ],
        'SAR' => [
            'SAR',
            'Սաուդյան Արաբիայի ռիալ',
        ],
        'SBD' => [
            'SBD',
            'Սողոմոնյան կղզիների դոլար',
        ],
        'SCR' => [
            'SCR',
            'սեյշելյան ռուփի',
        ],
        'SDG' => [
            'SDG',
            'սուդանական ֆունտ',
        ],
        'SEK' => [
            'SEK',
            'շվեդական կրոն',
        ],
        'SGD' => [
            'SGD',
            'Սինգապուրի դոլար',
        ],
        'SHP' => [
            'SHP',
            'Սուրբ Հեղինեի ֆունտ',
        ],
        'SLE' => [
            'SLE',
            'Սիեռա Լեոնեի լեոնե',
        ],
        'SLL' => [
            'SLL',
            'Սիեռա Լեոնեի լեոնե (1964—2022)',
        ],
        'SOS' => [
            'SOS',
            'սոմալիական շիլինգ',
        ],
        'SRD' => [
            'SRD',
            'սուրինամական դոլար',
        ],
        'SSP' => [
            'SSP',
            'հարավսուդանական ֆունտ',
        ],
        'STD' => [
            'STD',
            'Սան Տոմե և Փրինսիպիի դոբրա (1977–2017)',
        ],
        'STN' => [
            'STN',
            'Սան Տոմե և Փրինսիպիի դոբրա',
        ],
        'SYP' => [
            'SYP',
            'սիրիական ֆունտ',
        ],
        'SZL' => [
            'SZL',
            'սվազիլենդական լիլանգենի',
        ],
        'THB' => [
            '฿',
            'թաիլանդական բահտ',
        ],
        'TJS' => [
            'TJS',
            'տաջիկական սոմոնի',
        ],
        'TMT' => [
            'TMT',
            'թուրքմենական մանաթ',
        ],
        'TND' => [
            'TND',
            'թունիսյան դինար',
        ],
        'TOP' => [
            'TOP',
            'Տոնգայի պաանգա',
        ],
        'TRY' => [
            'TRY',
            'թուրքական լիրա',
        ],
        'TTD' => [
            'TTD',
            'Տրինիդադ և Տոբագոյի դոլար',
        ],
        'TWD' => [
            'NT$',
            'թայվանական նոր դոլար',
        ],
        'TZS' => [
            'TZS',
            'տանզանիական շիլինգ',
        ],
        'UAH' => [
            'UAH',
            'ուկրաինական գրիվնա',
        ],
        'UGX' => [
            'UGX',
            'ուգանդական շիլինգ',
        ],
        'USD' => [
            '$',
            'ԱՄՆ դոլար',
        ],
        'UYU' => [
            'UYU',
            'ուրուգվայական պեսո',
        ],
        'UZS' => [
            'UZS',
            'ուզբեկական սոմ',
        ],
        'VEF' => [
            'VEF',
            'վենեսուելական բոլիվար (2008–2018)',
        ],
        'VES' => [
            'VES',
            'վենեսուելական բոլիվար',
        ],
        'VND' => [
            '₫',
            'վիետնամական դոնգ',
        ],
        'VUV' => [
            'VUV',
            'Վանուատուի վատու',
        ],
        'WST' => [
            'WST',
            'սամոական տալա',
        ],
        'XAF' => [
            'FCFA',
            'Կենտրոնական Աֆրիկայի ԿՖԱ ֆրանկ',
        ],
        'XCD' => [
            'EC$',
            'արևելակարիբյան դոլար',
        ],
        'XCG' => [
            'Cg.',
            'կարիբյան գուլդեն',
        ],
        'XOF' => [
            'F CFA',
            'Արևմտյան Աֆրիկայի ԿՖԱ ֆրանկ',
        ],
        'XPF' => [
            'CFPF',
            'ԿՊՖ ֆրանկ',
        ],
        'YER' => [
            'YER',
            'եմենական ռիալ',
        ],
        'ZAR' => [
            'ZAR',
            'հարավաֆրիկյան ռանդ',
        ],
        'ZMK' => [
            'ZMK',
            'Զամբիական կվաչա (1968–2012)',
        ],
        'ZMW' => [
            'ZMW',
            'զամբիական կվաչա',
        ],
        'ZWG' => [
            'ZWG',
            'Զիմբաբվեի ոսկի',
        ],
    ],
];
