<?php

return [
    'Names' => [
        'aa' => 'Αφάρ',
        'ab' => 'Αμπχαζικά',
        'ace' => 'Ατσινιζικά',
        'ach' => 'Ακολί',
        'ada' => 'Αντάνγκμε',
        'ady' => 'Αντιγκέα',
        'ae' => 'Αβεστάν',
        'af' => 'Αφρικάανς',
        'afh' => 'Αφριχίλι',
        'agq' => 'Αγκέμ',
        'ain' => 'Αϊνού',
        'ak' => 'Ακάν',
        'akk' => 'Ακάντιαν',
        'ale' => 'Αλεούτ',
        'alt' => 'Νότια Αλτάι',
        'am' => 'Αμχαρικά',
        'an' => 'Αραγονικά',
        'ang' => 'Παλαιά Αγγλικά',
        'ann' => 'Ομπόλο',
        'anp' => 'Ανγκικά',
        'ar' => 'Αραβικά',
        'arc' => 'Αραμαϊκά',
        'arn' => 'Αραουκανικά',
        'arp' => 'Αραπάχο',
        'ars' => 'Αραβικά Νάτζντι',
        'arw' => 'Αραγουάκ',
        'as' => 'Ασαμικά',
        'asa' => 'Άσου',
        'ast' => 'Αστουριανά',
        'atj' => 'Ατικαμέκ',
        'av' => 'Αβαρικά',
        'awa' => 'Αγουαντί',
        'ay' => 'Αϊμάρα',
        'az' => 'Αζερμπαϊτζανικά',
        'ba' => 'Μπασκίρ',
        'bal' => 'Μπαλούτσι',
        'ban' => 'Μπαλινίζ',
        'bas' => 'Μπάσα',
        'bax' => 'Μπαμούν',
        'bbj' => 'Γκομάλα',
        'be' => 'Λευκορωσικά',
        'bej' => 'Μπέζα',
        'bem' => 'Μπέμπα',
        'bez' => 'Μπένα',
        'bfd' => 'Μπαφούτ',
        'bg' => 'Βουλγαρικά',
        'bgc' => 'Χαργιάνβι',
        'bgn' => 'Δυτικά Μπαλοχικά',
        'bho' => 'Μπότζπουρι',
        'bi' => 'Μπισλάμα',
        'bik' => 'Μπικόλ',
        'bin' => 'Μπίνι',
        'bkm' => 'Κομ',
        'bla' => 'Σικσίκα',
        'blo' => 'Ανίι',
        'bm' => 'Μπαμπάρα',
        'bn' => 'Βεγγαλικά',
        'bo' => 'Θιβετιανά',
        'br' => 'Βρετονικά',
        'bra' => 'Μπρατζ',
        'brx' => 'Μπόντο',
        'bs' => 'Βοσνιακά',
        'bss' => 'Ακόσι',
        'bua' => 'Μπουριάτ',
        'bug' => 'Μπουγκίζ',
        'bum' => 'Μπουλού',
        'byn' => 'Μπλιν',
        'byv' => 'Μεντούμπα',
        'ca' => 'Καταλανικά',
        'cad' => 'Κάντο',
        'car' => 'Καρίμπ',
        'cay' => 'Καγιούγκα',
        'cch' => 'Ατσάμ',
        'ccp' => 'Τσάκμα',
        'ce' => 'Τσετσενικά',
        'ceb' => 'Σεμπουάνο',
        'cgg' => 'Τσίγκα',
        'ch' => 'Τσαμόρο',
        'chb' => 'Τσίμπτσα',
        'chg' => 'Τσαγκατάι',
        'chk' => 'Τσουκίζι',
        'chm' => 'Μάρι',
        'chn' => 'Ιδιωματικά Σινούκ',
        'cho' => 'Τσόκτο',
        'chp' => 'Τσίπιουαν',
        'chr' => 'Τσερόκι',
        'chy' => 'Τσεγιέν',
        'ckb' => 'Κεντρικά Κουρδικά',
        'clc' => 'Τσιλκότιν',
        'co' => 'Κορσικανικά',
        'cop' => 'Κοπτικά',
        'cr' => 'Κρι',
        'crg' => 'Μίτσιφ',
        'crh' => 'Τουρκικά Κριμαίας',
        'crj' => 'Νοτιοανατολικά Κρι',
        'crk' => 'Κρι πεδιάδας',
        'crl' => 'Βορειοανατολικά Κρι',
        'crm' => 'Μους Κρι',
        'crr' => 'Καρολίνα Αλγκονκιάν',
        'crs' => 'Κρεολικά Γαλλικά Σεϋχελλών',
        'cs' => 'Τσεχικά',
        'csb' => 'Κασούμπιαν',
        'csw' => 'Κρι Βάλτου',
        'cu' => 'Εκκλησιαστικά Σλαβικά',
        'cv' => 'Τσουβασικά',
        'cy' => 'Ουαλικά',
        'da' => 'Δανικά',
        'dak' => 'Ντακότα',
        'dar' => 'Ντάργκουα',
        'dav' => 'Τάιτα',
        'de' => 'Γερμανικά',
        'del' => 'Ντέλαγουερ',
        'den' => 'Σλαβικά',
        'dgr' => 'Ντόγκριμπ',
        'din' => 'Ντίνκα',
        'dje' => 'Ζάρμα',
        'doi' => 'Ντόγκρι',
        'dsb' => 'Κάτω Σορβικά',
        'dua' => 'Ντουάλα',
        'dum' => 'Μέσα Ολλανδικά',
        'dv' => 'Ντιβέχι',
        'dyo' => 'Τζόλα-Φόνι',
        'dyu' => 'Ντογιούλα',
        'dz' => 'Ντζόνγκχα',
        'dzg' => 'Νταζάγκα',
        'ebu' => 'Έμπου',
        'ee' => 'Έουε',
        'efi' => 'Εφίκ',
        'egy' => 'Αρχαία Αιγυπτιακά',
        'eka' => 'Εκατζούκ',
        'el' => 'Ελληνικά',
        'elx' => 'Ελαμάιτ',
        'en' => 'Αγγλικά',
        'enm' => 'Μέσα Αγγλικά',
        'eo' => 'Εσπεράντο',
        'es' => 'Ισπανικά',
        'et' => 'Εσθονικά',
        'eu' => 'Βασκικά',
        'ewo' => 'Εγουόντο',
        'fa' => 'Περσικά',
        'fan' => 'Φανγκ',
        'fat' => 'Φάντι',
        'ff' => 'Φουλά',
        'fi' => 'Φινλανδικά',
        'fil' => 'Φιλιππινικά',
        'fj' => 'Φίτζι',
        'fo' => 'Φεροϊκά',
        'fon' => 'Φον',
        'fr' => 'Γαλλικά',
        'frc' => 'Γαλλικά (Λουιζιάνα)',
        'frm' => 'Μέσα Γαλλικά',
        'fro' => 'Παλαιά Γαλλικά',
        'frr' => 'Βόρεια Φριζιανά',
        'frs' => 'Ανατολικά Φριζιανά',
        'fur' => 'Φριουλανικά',
        'fy' => 'Δυτικά Φριζικά',
        'ga' => 'Ιρλανδικά',
        'gaa' => 'Γκα',
        'gag' => 'Γκαγκάουζ',
        'gay' => 'Γκάγιο',
        'gba' => 'Γκμπάγια',
        'gd' => 'Σκωτικά Κελτικά',
        'gez' => 'Γκιζ',
        'gil' => 'Γκιλμπερτίζ',
        'gl' => 'Γαλικιανά',
        'gmh' => 'Μέσα Άνω Γερμανικά',
        'gn' => 'Γκουαρανί',
        'goh' => 'Παλαιά Άνω Γερμανικά',
        'gon' => 'Γκόντι',
        'gor' => 'Γκοροντάλο',
        'got' => 'Γοτθικά',
        'grb' => 'Γκρίμπο',
        'grc' => 'Αρχαία Ελληνικά',
        'gsw' => 'Γερμανικά Ελβετίας',
        'gu' => 'Γκουτζαρατικά',
        'guz' => 'Γκούσι',
        'gv' => 'Μανξ',
        'gwi' => 'Γκουίτσιν',
        'ha' => 'Χάουσα',
        'hai' => 'Χάιντα',
        'haw' => 'Χαβαϊκά',
        'hax' => 'Βόρεια Χάιντα',
        'he' => 'Εβραϊκά',
        'hi' => 'Χίντι',
        'hil' => 'Χιλιγκαϊνόν',
        'hit' => 'Χιτίτε',
        'hmn' => 'Χμονγκ',
        'ho' => 'Χίρι Μότου',
        'hr' => 'Κροατικά',
        'hsb' => 'Άνω Σορβικά',
        'ht' => 'Αϊτιανά',
        'hu' => 'Ουγγρικά',
        'hup' => 'Χούπα',
        'hur' => 'Χαλκομελέμ',
        'hy' => 'Αρμενικά',
        'hz' => 'Χερέρο',
        'ia' => 'Ιντερλίνγκουα',
        'iba' => 'Ιμπάν',
        'ibb' => 'Ιμπίμπιο',
        'id' => 'Ινδονησιακά',
        'ie' => 'Ιντερλίνγκουε',
        'ig' => 'Ίγκμπο',
        'ii' => 'Σίτσουαν Γι',
        'ik' => 'Ινουπιάκ',
        'ikt' => 'Ινουκτιτούτ Δυτικού Καναδά',
        'ilo' => 'Ιλόκο',
        'inh' => 'Ινγκούς',
        'io' => 'Ίντο',
        'is' => 'Ισλανδικά',
        'it' => 'Ιταλικά',
        'iu' => 'Ινούκτιτουτ',
        'ja' => 'Ιαπωνικά',
        'jbo' => 'Λόζμπαν',
        'jgo' => 'Νγκόμπα',
        'jmc' => 'Ματσάμε',
        'jpr' => 'Ιουδαϊκά-Περσικά',
        'jrb' => 'Ιουδαϊκά-Αραβικά',
        'jv' => 'Ιαβανικά',
        'ka' => 'Γεωργιανά',
        'kaa' => 'Κάρα-Καλπάκ',
        'kab' => 'Καμπίλε',
        'kac' => 'Κατσίν',
        'kaj' => 'Τζου',
        'kam' => 'Κάμπα',
        'kaw' => 'Κάουι',
        'kbd' => 'Καμπαρντιανά',
        'kbl' => 'Κανέμπου',
        'kcg' => 'Τιάπ',
        'kde' => 'Μακόντε',
        'kea' => 'Γλώσσα του Πράσινου Ακρωτηρίου',
        'kfo' => 'Κόρο',
        'kg' => 'Κονγκό',
        'kgp' => 'Κάινγκανγκ',
        'kha' => 'Κάσι',
        'kho' => 'Κοτανικά',
        'khq' => 'Κόιρα Τσίνι',
        'ki' => 'Κικούγιου',
        'kj' => 'Κουανιάμα',
        'kk' => 'Καζακικά',
        'kkj' => 'Κάκο',
        'kl' => 'Καλαάλισουτ',
        'kln' => 'Καλεντζίν',
        'km' => 'Χμερ',
        'kmb' => 'Κιμπούντου',
        'kn' => 'Κανάντα',
        'ko' => 'Κορεατικά',
        'koi' => 'Κόμι-Περμιάκ',
        'kok' => 'Κονκανικά',
        'kos' => 'Κοσραενικά',
        'kpe' => 'Κπέλε',
        'kr' => 'Κανούρι',
        'krc' => 'Καρατσάι-Μπαλκάρ',
        'krl' => 'Καρελικά',
        'kru' => 'Κουρούχ',
        'ks' => 'Κασμιρικά',
        'ksb' => 'Σαμπάλα',
        'ksf' => 'Μπάφια',
        'ksh' => 'Κολωνικά',
        'ku' => 'Κουρδικά',
        'kum' => 'Κουμγιούκ',
        'kut' => 'Κουτενάι',
        'kv' => 'Κόμι',
        'kw' => 'Κορνουαλικά',
        'kwk' => 'Κουακουάλα',
        'kxv' => 'Κούβι',
        'ky' => 'Κιργιζικά',
        'la' => 'Λατινικά',
        'lad' => 'Λαδίνο',
        'lag' => 'Λάνγκι',
        'lah' => 'Λάχδα',
        'lam' => 'Λάμπα',
        'lb' => 'Λουξεμβουργιανά',
        'lez' => 'Λεζγκικά',
        'lg' => 'Γκάντα',
        'li' => 'Λιμβουργιανά',
        'lij' => 'Λιγουριανά',
        'lil' => 'Λιλουέτ',
        'lkt' => 'Λακότα',
        'lmo' => 'Λομβαρδικά',
        'ln' => 'Λινγκάλα',
        'lo' => 'Λαοτινά',
        'lol' => 'Μόνγκο',
        'lou' => 'Κρεολικά (Λουιζιάνα)',
        'loz' => 'Λόζι',
        'lrc' => 'Βόρεια Λούρι',
        'lsm' => 'Σαάμια',
        'lt' => 'Λιθουανικά',
        'lu' => 'Λούμπα-Κατάνγκα',
        'lua' => 'Λούμπα-Λουλούα',
        'lui' => 'Λουισένο',
        'lun' => 'Λούντα',
        'luo' => 'Λούο',
        'lus' => 'Μίζο',
        'luy' => 'Λούχια',
        'lv' => 'Λετονικά',
        'mad' => 'Μαντουρίζ',
        'maf' => 'Μάφα',
        'mag' => 'Μαγκάχι',
        'mai' => 'Μαϊτχίλι',
        'mak' => 'Μακασάρ',
        'man' => 'Μαντίνγκο',
        'mas' => 'Μασάι',
        'mde' => 'Μάμπα',
        'mdf' => 'Μόκσα',
        'mdr' => 'Μανδάρ',
        'men' => 'Μέντε',
        'mer' => 'Μέρου',
        'mfe' => 'Μορισιέν',
        'mg' => 'Μαλγασικά',
        'mga' => 'Μέσα Ιρλανδικά',
        'mgh' => 'Μακούβα-Μέτο',
        'mgo' => 'Μέτα',
        'mh' => 'Μαρσαλέζικα',
        'mi' => 'Μαορί',
        'mic' => 'Μικμάκ',
        'min' => 'Μινανγκαμπάου',
        'mk' => 'Σλαβομακεδονικά',
        'ml' => 'Μαλαγιαλαμικά',
        'mn' => 'Μογγολικά',
        'mnc' => 'Μαντσού',
        'mni' => 'Μανιπούρι',
        'moe' => 'Ινου-αϊμούν',
        'moh' => 'Μοχόκ',
        'mos' => 'Μόσι',
        'mr' => 'Μαραθικά',
        'ms' => 'Μαλαισιανά',
        'mt' => 'Μαλτεζικά',
        'mua' => 'Μουντάνγκ',
        'mus' => 'Κρικ',
        'mwl' => 'Μιραντεζικά',
        'mwr' => 'Μαργουάρι',
        'my' => 'Βιρμανικά',
        'mye' => 'Μιένε',
        'myv' => 'Έρζια',
        'mzn' => 'Μαζαντεράνι',
        'na' => 'Ναούρου',
        'nan' => 'Μιν Ναν',
        'nap' => 'Ναπολιτανικά',
        'naq' => 'Νάμα',
        'nb' => 'Νορβηγικά Μποκμάλ',
        'nd' => 'Βόρεια Ντεμπέλε',
        'nds' => 'Κάτω Γερμανικά',
        'ne' => 'Νεπαλικά',
        'new' => 'Νεγουάρι',
        'ng' => 'Ντόνγκα',
        'nia' => 'Νίας',
        'niu' => 'Νιούε',
        'nl' => 'Ολλανδικά',
        'nmg' => 'Κβάσιο',
        'nn' => 'Νορβηγικά Νινόρσκ',
        'nnh' => 'Νγκιεμπούν',
        'no' => 'Νορβηγικά',
        'nog' => 'Νογκάι',
        'non' => 'Παλαιά Νορβηγικά',
        'nqo' => 'Ν’Κο',
        'nr' => 'Νότια Ντεμπέλε',
        'nso' => 'Βόρεια Σόθο',
        'nus' => 'Νούερ',
        'nv' => 'Νάβαχο',
        'nwc' => 'Κλασικά Νεουάρι',
        'ny' => 'Νιάντζα',
        'nym' => 'Νιαμγουέζι',
        'nyn' => 'Νιανκόλε',
        'nyo' => 'Νιόρο',
        'nzi' => 'Νζίμα',
        'oc' => 'Οξιτανικά',
        'oj' => 'Οζιβίγουα',
        'ojb' => 'Βορειοδυτικά Οζιβίγουα',
        'ojc' => 'Κεντρικά Οτζίμπουα',
        'ojs' => 'Ότζι-Κρι',
        'ojw' => 'Δυτικά Οζιβίγουα',
        'oka' => 'Οκανάγκαν',
        'om' => 'Ορόμο',
        'or' => 'Όντια',
        'os' => 'Οσετικά',
        'osa' => 'Οσάζ',
        'ota' => 'Οθωμανικά Τουρκικά',
        'pa' => 'Παντζαπικά',
        'pag' => 'Πανγκασινάν',
        'pal' => 'Παχλάβι',
        'pam' => 'Παμπάνγκα',
        'pap' => 'Παπιαμέντο',
        'pau' => 'Παλάουαν',
        'pcm' => 'Πίτζιν Νιγηρίας',
        'peo' => 'Αρχαία Περσικά',
        'phn' => 'Φοινικικά',
        'pi' => 'Πάλι',
        'pis' => 'Πιτζίν',
        'pl' => 'Πολωνικά',
        'pon' => 'Πομπηικά',
        'pqm' => 'Μαλισιτ-Πασσαμακουόντ',
        'prg' => 'Πρωσικά',
        'pro' => 'Παλαιά Προβανσάλ',
        'ps' => 'Πάστο',
        'pt' => 'Πορτογαλικά',
        'qu' => 'Κέτσουα',
        'quc' => 'Κιτσέ',
        'raj' => 'Ραζασθάνι',
        'rap' => 'Ραπανούι',
        'rar' => 'Ραροτονγκάν',
        'rhg' => 'Ροχίνγκια',
        'rm' => 'Ρομανικά',
        'rn' => 'Ρούντι',
        'ro' => 'Ρουμανικά',
        'rof' => 'Ρόμπο',
        'rom' => 'Ρομανί',
        'ru' => 'Ρωσικά',
        'rup' => 'Αρομανικά',
        'rw' => 'Κινιαρουάντα',
        'rwk' => 'Ρουά',
        'sa' => 'Σανσκριτικά',
        'sad' => 'Σαντάγουε',
        'sah' => 'Σαχά',
        'sam' => 'Σαμαρίτικα Αραμαϊκά',
        'saq' => 'Σαμπούρου',
        'sas' => 'Σασάκ',
        'sat' => 'Σαντάλι',
        'sba' => 'Νγκαμπέι',
        'sbp' => 'Σάνγκου',
        'sc' => 'Σαρδηνιακά',
        'scn' => 'Σικελικά',
        'sco' => 'Σκωτικά',
        'sd' => 'Σίντι',
        'sdh' => 'Νότια Κουρδικά',
        'se' => 'Βόρεια Σάμι',
        'see' => 'Σένεκα',
        'seh' => 'Σένα',
        'sel' => 'Σελκούπ',
        'ses' => 'Κοϊραμπόρο Σένι',
        'sg' => 'Σάνγκο',
        'sga' => 'Παλαιά Ιρλανδικά',
        'sh' => 'Σερβοκροατικά',
        'shi' => 'Τασελχίτ',
        'shn' => 'Σαν',
        'shu' => 'Αραβικά του Τσαντ',
        'si' => 'Σινχαλεζικά',
        'sid' => 'Σιντάμο',
        'sk' => 'Σλοβακικά',
        'sl' => 'Σλοβενικά',
        'slh' => 'Νότια Λάσουτσιντ',
        'sm' => 'Σαμοανά',
        'sma' => 'Νότια Σάμι',
        'smj' => 'Λούλε Σάμι',
        'smn' => 'Ινάρι Σάμι',
        'sms' => 'Σκολτ Σάμι',
        'sn' => 'Σόνα',
        'snk' => 'Σονίνκε',
        'so' => 'Σομαλικά',
        'sog' => 'Σογκντιέν',
        'sq' => 'Αλβανικά',
        'sr' => 'Σερβικά',
        'srn' => 'Σρανάν Τόνγκο',
        'srr' => 'Σερέρ',
        'ss' => 'Σουάτι',
        'ssy' => 'Σάχο',
        'st' => 'Νότια Σόθο',
        'str' => 'Στρέιτς Σαλίς',
        'su' => 'Σουνδανικά',
        'suk' => 'Σουκούμα',
        'sus' => 'Σούσου',
        'sux' => 'Σουμερικά',
        'sv' => 'Σουηδικά',
        'sw' => 'Σουαχίλι',
        'swb' => 'Κομοριανά',
        'syc' => 'Κλασικά Συριακά',
        'syr' => 'Συριακά',
        'szl' => 'Σιλεσικά',
        'ta' => 'Ταμιλικά',
        'tce' => 'Νότια Τουτσόνε',
        'te' => 'Τελούγκου',
        'tem' => 'Τίμνε',
        'teo' => 'Τέσο',
        'ter' => 'Τερένο',
        'tet' => 'Τέτουμ',
        'tg' => 'Τατζικικά',
        'tgx' => 'Τατζίς',
        'th' => 'Ταϊλανδικά',
        'tht' => 'Ταλτάν',
        'ti' => 'Τιγκρινικά',
        'tig' => 'Τίγκρε',
        'tiv' => 'Τιβ',
        'tk' => 'Τουρκμενικά',
        'tkl' => 'Τοκελάου',
        'tl' => 'Τάγκαλογκ',
        'tlh' => 'Κλίνγκον',
        'tli' => 'Τλίνγκιτ',
        'tmh' => 'Ταμασέκ',
        'tn' => 'Τσουάνα',
        'to' => 'Τονγκανικά',
        'tog' => 'Νιάσα Τόνγκα',
        'tok' => 'Τόκι Πόνα',
        'tpi' => 'Τοκ Πισίν',
        'tr' => 'Τουρκικά',
        'trv' => 'Ταρόκο',
        'ts' => 'Τσόνγκα',
        'tsi' => 'Τσίμσιαν',
        'tt' => 'Ταταρικά',
        'ttm' => 'Βόρεια Τουτσόνε',
        'tum' => 'Τουμπούκα',
        'tvl' => 'Τουβαλού',
        'tw' => 'Τούι',
        'twq' => 'Τασαβάκ',
        'ty' => 'Ταϊτιανά',
        'tyv' => 'Τουβινικά',
        'tzm' => 'Ταμαζίτ Κεντρικού Μαρόκο',
        'udm' => 'Ουντμούρτ',
        'ug' => 'Ουιγουρικά',
        'uga' => 'Ουγκαριτικά',
        'uk' => 'Ουκρανικά',
        'umb' => 'Ουμπούντου',
        'ur' => 'Ούρντου',
        'uz' => 'Ουζμπεκικά',
        'vai' => 'Βάι',
        've' => 'Βέντα',
        'vec' => 'Βενετικά',
        'vi' => 'Βιετναμικά',
        'vmw' => 'Μακούα',
        'vo' => 'Βολαπιούκ',
        'vot' => 'Βότικ',
        'vun' => 'Βούντζο',
        'wa' => 'Βαλλωνικά',
        'wae' => 'Βάλσερ',
        'wal' => 'Γουολάιτα',
        'war' => 'Γουάραϊ',
        'was' => 'Γουασό',
        'wbp' => 'Γουαρλπίρι',
        'wo' => 'Γουόλοφ',
        'wuu' => 'Κινεζικά Γου',
        'xal' => 'Καλμίκ',
        'xh' => 'Κόσα',
        'xnr' => 'Κάνγκρι',
        'xog' => 'Σόγκα',
        'yao' => 'Γιάο',
        'yap' => 'Γιαπίζ',
        'yav' => 'Γιανγκμπέν',
        'ybb' => 'Γιέμπα',
        'yi' => 'Γίντις',
        'yo' => 'Γιορούμπα',
        'yrl' => 'Νινγκατού',
        'yue' => 'Καντονέζικα',
        'za' => 'Ζουάνγκ',
        'zap' => 'Ζάποτεκ',
        'zbl' => 'Σύμβολα Bliss',
        'zen' => 'Ζενάγκα',
        'zgh' => 'Τυπικά Ταμαζίτ Μαρόκου',
        'zh' => 'Κινεζικά',
        'zu' => 'Ζουλού',
        'zun' => 'Ζούνι',
        'zza' => 'Ζάζα',
    ],
    'LocalizedNames' => [
        'ar_001' => 'Σύγχρονα Τυπικά Αραβικά',
        'de_AT' => 'Γερμανικά Αυστρίας',
        'de_CH' => 'Υψηλά Γερμανικά Ελβετίας',
        'en_AU' => 'Αγγλικά Αυστραλίας',
        'en_CA' => 'Αγγλικά Καναδά',
        'en_GB' => 'Αγγλικά Βρετανίας',
        'en_US' => 'Αγγλικά Αμερικής',
        'es_419' => 'Ισπανικά Λατινικής Αμερικής',
        'es_ES' => 'Ισπανικά Ευρώπης',
        'es_MX' => 'Ισπανικά Μεξικού',
        'fa_AF' => 'Νταρί',
        'fr_CA' => 'Γαλλικά Καναδά',
        'fr_CH' => 'Γαλλικά Ελβετίας',
        'nds_NL' => 'Κάτω Γερμανικά Ολλανδίας',
        'nl_BE' => 'Φλαμανδικά',
        'pt_BR' => 'Πορτογαλικά Βραζιλίας',
        'pt_PT' => 'Πορτογαλικά Ευρώπης',
        'ro_MD' => 'Μολδαβικά',
        'sw_CD' => 'Κονγκό Σουαχίλι',
        'zh_Hans' => 'Απλοποιημένα Κινεζικά',
        'zh_Hant' => 'Παραδοσιακά Κινεζικά',
    ],
];
