<?php

return [
    'Names' => [
        'af' => 'IsiBhulu',
        'af_NA' => 'IsiBhulu (ENamibia)',
        'af_ZA' => 'IsiBhulu (EMzantsi Afrika)',
        'am' => 'IsiAmharic',
        'am_ET' => 'IsiAmharic (E-Ethiopia)',
        'ar' => 'Isi-Arabhu',
        'ar_001' => 'Isi-Arabhu (ihlabathi)',
        'ar_AE' => 'Isi-Arabhu (E-United Arab Emirates)',
        'ar_BH' => 'Isi-Arabhu (EBahrain)',
        'ar_DJ' => 'Isi-Arabhu (EDjibouti)',
        'ar_DZ' => 'Isi-Arabhu (E-Algeria)',
        'ar_EG' => 'Isi-Arabhu (IYiputa)',
        'ar_EH' => 'Isi-Arabhu (EWestern Sahara)',
        'ar_ER' => 'Isi-Arabhu (E-Eritrea)',
        'ar_IL' => 'Isi-Arabhu (E-Israel)',
        'ar_IQ' => 'Isi-Arabhu (E-Iraq)',
        'ar_JO' => 'Isi-Arabhu (EJordan)',
        'ar_KM' => 'Isi-Arabhu (EComoros)',
        'ar_KW' => 'Isi-Arabhu (EKuwait)',
        'ar_LB' => 'Isi-Arabhu (ELebanon)',
        'ar_LY' => 'Isi-Arabhu (ELibya)',
        'ar_MA' => 'Isi-Arabhu (EMorocco)',
        'ar_MR' => 'Isi-Arabhu (EMauritania)',
        'ar_OM' => 'Isi-Arabhu (E-Oman)',
        'ar_PS' => 'Isi-Arabhu (IPalestinian Territories)',
        'ar_QA' => 'Isi-Arabhu (EQatar)',
        'ar_SA' => 'Isi-Arabhu (ESaudi Arabia)',
        'ar_SD' => 'Isi-Arabhu (ESudan)',
        'ar_SO' => 'Isi-Arabhu (ESomalia)',
        'ar_SS' => 'Isi-Arabhu (ESouth Sudan)',
        'ar_SY' => 'Isi-Arabhu (ESiriya)',
        'ar_TD' => 'Isi-Arabhu (EChad)',
        'ar_TN' => 'Isi-Arabhu (ETunisia)',
        'ar_YE' => 'Isi-Arabhu (EYemen)',
        'as' => 'IsiAssamese',
        'as_IN' => 'IsiAssamese (E-Indiya)',
        'az' => 'Isi-Azerbaijani',
        'az_AZ' => 'Isi-Azerbaijani (E-Azerbaijan)',
        'az_Cyrl' => 'Isi-Azerbaijani (IsiCyrillic)',
        'az_Cyrl_AZ' => 'Isi-Azerbaijani (IsiCyrillic, E-Azerbaijan)',
        'az_Latn' => 'Isi-Azerbaijani (IsiLatin)',
        'az_Latn_AZ' => 'Isi-Azerbaijani (IsiLatin, E-Azerbaijan)',
        'be' => 'Isi-Belarusian',
        'be_BY' => 'Isi-Belarusian (EBelarus)',
        'bg' => 'Isi-Bulgaria',
        'bg_BG' => 'Isi-Bulgaria (EBulgaria)',
        'bn' => 'IsiBangla',
        'bn_BD' => 'IsiBangla (EBangladesh)',
        'bn_IN' => 'IsiBangla (E-Indiya)',
        'br' => 'IsiBreton',
        'br_FR' => 'IsiBreton (EFrance)',
        'bs' => 'Isi-Bosnia',
        'bs_BA' => 'Isi-Bosnia (EBosnia & Herzegovina)',
        'bs_Cyrl' => 'Isi-Bosnia (IsiCyrillic)',
        'bs_Cyrl_BA' => 'Isi-Bosnia (IsiCyrillic, EBosnia & Herzegovina)',
        'bs_Latn' => 'Isi-Bosnia (IsiLatin)',
        'bs_Latn_BA' => 'Isi-Bosnia (IsiLatin, EBosnia & Herzegovina)',
        'ca' => 'Isi-Calatan',
        'ca_AD' => 'Isi-Calatan (E-Andorra)',
        'ca_ES' => 'Isi-Calatan (ESpain)',
        'ca_FR' => 'Isi-Calatan (EFrance)',
        'ca_IT' => 'Isi-Calatan (E-Italy)',
        'cs' => 'Isi-Czech',
        'cs_CZ' => 'Isi-Czech (ECzechia)',
        'cy' => 'Isi-Welsh',
        'cy_GB' => 'Isi-Welsh (E-United Kingdom)',
        'da' => 'Isi-Danish',
        'da_DK' => 'Isi-Danish (EDenmark)',
        'da_GL' => 'Isi-Danish (EGreenland)',
        'de' => 'IsiJamani',
        'de_AT' => 'IsiJamani (E-Austria)',
        'de_BE' => 'IsiJamani (EBelgium)',
        'de_CH' => 'IsiJamani (ESwitzerland)',
        'de_DE' => 'IsiJamani (EJamani)',
        'de_IT' => 'IsiJamani (E-Italy)',
        'de_LI' => 'IsiJamani (ELiechtenstein)',
        'de_LU' => 'IsiJamani (ELuxembourg)',
        'el' => 'Isi-Greek',
        'el_CY' => 'Isi-Greek (ECyprus)',
        'el_GR' => 'Isi-Greek (EGreece)',
        'en' => 'IsiNgesi',
        'en_001' => 'IsiNgesi (ihlabathi)',
        'en_150' => 'IsiNgesi (EYurophu)',
        'en_AE' => 'IsiNgesi (E-United Arab Emirates)',
        'en_AG' => 'IsiNgesi (E-Antigua & Barbuda)',
        'en_AI' => 'IsiNgesi (E-Anguilla)',
        'en_AS' => 'IsiNgesi (E-American Samoa)',
        'en_AT' => 'IsiNgesi (E-Austria)',
        'en_AU' => 'IsiNgesi (E-Australia)',
        'en_BB' => 'IsiNgesi (EBarbados)',
        'en_BE' => 'IsiNgesi (EBelgium)',
        'en_BI' => 'IsiNgesi (EBurundi)',
        'en_BM' => 'IsiNgesi (EBermuda)',
        'en_BS' => 'IsiNgesi (EBahamas)',
        'en_BW' => 'IsiNgesi (EBotswana)',
        'en_BZ' => 'IsiNgesi (EBelize)',
        'en_CA' => 'IsiNgesi (EKhanada)',
        'en_CC' => 'IsiNgesi (ECocos [Keeling] Islands)',
        'en_CH' => 'IsiNgesi (ESwitzerland)',
        'en_CK' => 'IsiNgesi (ECook Islands)',
        'en_CM' => 'IsiNgesi (ECameroon)',
        'en_CX' => 'IsiNgesi (EChristmas Island)',
        'en_CY' => 'IsiNgesi (ECyprus)',
        'en_CZ' => 'IsiNgesi (ECzechia)',
        'en_DE' => 'IsiNgesi (EJamani)',
        'en_DK' => 'IsiNgesi (EDenmark)',
        'en_DM' => 'IsiNgesi (EDominica)',
        'en_EE' => 'IsiNgesi (E-Estonia)',
        'en_ER' => 'IsiNgesi (E-Eritrea)',
        'en_ES' => 'IsiNgesi (ESpain)',
        'en_FI' => 'IsiNgesi (EFinland)',
        'en_FJ' => 'IsiNgesi (EFiji)',
        'en_FK' => 'IsiNgesi (EFalkland Islands)',
        'en_FM' => 'IsiNgesi (EMicronesia)',
        'en_FR' => 'IsiNgesi (EFrance)',
        'en_GB' => 'IsiNgesi (E-United Kingdom)',
        'en_GD' => 'IsiNgesi (EGrenada)',
        'en_GE' => 'IsiNgesi (EGeorgia)',
        'en_GG' => 'IsiNgesi (EGuernsey)',
        'en_GH' => 'IsiNgesi (EGhana)',
        'en_GI' => 'IsiNgesi (EGibraltar)',
        'en_GM' => 'IsiNgesi (EGambia)',
        'en_GS' => 'IsiNgesi (ESouth Georgia & South Sandwich Islands)',
        'en_GU' => 'IsiNgesi (EGuam)',
        'en_GY' => 'IsiNgesi (EGuyana)',
        'en_HK' => 'IsiNgesi (EHong Kong SAR China)',
        'en_HU' => 'IsiNgesi (EHungary)',
        'en_ID' => 'IsiNgesi (E-Indonesia)',
        'en_IE' => 'IsiNgesi (E-Ireland)',
        'en_IL' => 'IsiNgesi (E-Israel)',
        'en_IM' => 'IsiNgesi (E-Isle of Man)',
        'en_IN' => 'IsiNgesi (E-Indiya)',
        'en_IO' => 'IsiNgesi (EBritish Indian Ocean Territory)',
        'en_IT' => 'IsiNgesi (E-Italy)',
        'en_JE' => 'IsiNgesi (EJersey)',
        'en_JM' => 'IsiNgesi (EJamaica)',
        'en_JP' => 'IsiNgesi (EJapan)',
        'en_KE' => 'IsiNgesi (EKenya)',
        'en_KI' => 'IsiNgesi (EKiribati)',
        'en_KN' => 'IsiNgesi (ESt. Kitts & Nevis)',
        'en_KY' => 'IsiNgesi (ECayman Islands)',
        'en_LC' => 'IsiNgesi (ESt. Lucia)',
        'en_LR' => 'IsiNgesi (ELiberia)',
        'en_LS' => 'IsiNgesi (ELesotho)',
        'en_LT' => 'IsiNgesi (ELithuania)',
        'en_LV' => 'IsiNgesi (ELatvia)',
        'en_MG' => 'IsiNgesi (EMadagascar)',
        'en_MH' => 'IsiNgesi (EMarshall Islands)',
        'en_MO' => 'IsiNgesi (EMacao SAR China)',
        'en_MP' => 'IsiNgesi (ENorthern Mariana Islands)',
        'en_MS' => 'IsiNgesi (EMontserrat)',
        'en_MT' => 'IsiNgesi (EMalta)',
        'en_MU' => 'IsiNgesi (EMauritius)',
        'en_MV' => 'IsiNgesi (EMaldives)',
        'en_MW' => 'IsiNgesi (EMalawi)',
        'en_MY' => 'IsiNgesi (EMalaysia)',
        'en_NA' => 'IsiNgesi (ENamibia)',
        'en_NF' => 'IsiNgesi (ENorfolk Island)',
        'en_NG' => 'IsiNgesi (ENigeria)',
        'en_NL' => 'IsiNgesi (ENetherlands)',
        'en_NO' => 'IsiNgesi (ENorway)',
        'en_NR' => 'IsiNgesi (ENauru)',
        'en_NU' => 'IsiNgesi (ENiue)',
        'en_NZ' => 'IsiNgesi (ENew Zealand)',
        'en_PG' => 'IsiNgesi (EPapua New Guinea)',
        'en_PH' => 'IsiNgesi (EPhilippines)',
        'en_PK' => 'IsiNgesi (EPakistan)',
        'en_PL' => 'IsiNgesi (EPoland)',
        'en_PN' => 'IsiNgesi (EPitcairn Islands)',
        'en_PR' => 'IsiNgesi (EPuerto Rico)',
        'en_PT' => 'IsiNgesi (EPortugal)',
        'en_PW' => 'IsiNgesi (EPalau)',
        'en_RO' => 'IsiNgesi (ERomania)',
        'en_RW' => 'IsiNgesi (ERwanda)',
        'en_SB' => 'IsiNgesi (ESolomon Islands)',
        'en_SC' => 'IsiNgesi (ESeychelles)',
        'en_SD' => 'IsiNgesi (ESudan)',
        'en_SE' => 'IsiNgesi (ESweden)',
        'en_SG' => 'IsiNgesi (ESingapore)',
        'en_SH' => 'IsiNgesi (ESt. Helena)',
        'en_SI' => 'IsiNgesi (ESlovenia)',
        'en_SK' => 'IsiNgesi (ESlovakia)',
        'en_SL' => 'IsiNgesi (ESierra Leone)',
        'en_SS' => 'IsiNgesi (ESouth Sudan)',
        'en_SX' => 'IsiNgesi (ESint Maarten)',
        'en_SZ' => 'IsiNgesi (ESwatini)',
        'en_TC' => 'IsiNgesi (ETurks & Caicos Islands)',
        'en_TK' => 'IsiNgesi (ETokelau)',
        'en_TO' => 'IsiNgesi (ETonga)',
        'en_TT' => 'IsiNgesi (ETrinidad & Tobago)',
        'en_TV' => 'IsiNgesi (ETuvalu)',
        'en_TZ' => 'IsiNgesi (ETanzania)',
        'en_UA' => 'IsiNgesi (E-Ukraine)',
        'en_UG' => 'IsiNgesi (E-Uganda)',
        'en_UM' => 'IsiNgesi (I-U.S. Outlying Islands)',
        'en_US' => 'IsiNgesi (EMelika)',
        'en_VC' => 'IsiNgesi (ESt. Vincent & Grenadines)',
        'en_VG' => 'IsiNgesi (EBritish Virgin Islands)',
        'en_VI' => 'IsiNgesi (E-U.S. Virgin Islands)',
        'en_VU' => 'IsiNgesi (EVanuatu)',
        'en_WS' => 'IsiNgesi (ESamoa)',
        'en_ZA' => 'IsiNgesi (EMzantsi Afrika)',
        'en_ZM' => 'IsiNgesi (EZambia)',
        'en_ZW' => 'IsiNgesi (EZimbabwe)',
        'eo' => 'Isi-Esperanto',
        'eo_001' => 'Isi-Esperanto (ihlabathi)',
        'es' => 'Isi-Spanish',
        'es_419' => 'Isi-Spanish (ILatin America)',
        'es_AR' => 'Isi-Spanish (E-Argentina)',
        'es_BO' => 'Isi-Spanish (EBolivia)',
        'es_BR' => 'Isi-Spanish (EBrazil)',
        'es_BZ' => 'Isi-Spanish (EBelize)',
        'es_CL' => 'Isi-Spanish (EChile)',
        'es_CO' => 'Isi-Spanish (EColombia)',
        'es_CR' => 'Isi-Spanish (ECosta Rica)',
        'es_CU' => 'Isi-Spanish (ECuba)',
        'es_DO' => 'Isi-Spanish (EDominican Republic)',
        'es_EC' => 'Isi-Spanish (EEcuador)',
        'es_ES' => 'Isi-Spanish (ESpain)',
        'es_GQ' => 'Isi-Spanish (E-Equatorial Guinea)',
        'es_GT' => 'Isi-Spanish (EGuatemala)',
        'es_HN' => 'Isi-Spanish (EHonduras)',
        'es_MX' => 'Isi-Spanish (EMexico)',
        'es_NI' => 'Isi-Spanish (ENicaragua)',
        'es_PA' => 'Isi-Spanish (EPanama)',
        'es_PE' => 'Isi-Spanish (EPeru)',
        'es_PH' => 'Isi-Spanish (EPhilippines)',
        'es_PR' => 'Isi-Spanish (EPuerto Rico)',
        'es_PY' => 'Isi-Spanish (EParaguay)',
        'es_SV' => 'Isi-Spanish (E-El Salvador)',
        'es_US' => 'Isi-Spanish (EMelika)',
        'es_UY' => 'Isi-Spanish (E-Uruguay)',
        'es_VE' => 'Isi-Spanish (EVenezuela)',
        'et' => 'Isi-Estonian',
        'et_EE' => 'Isi-Estonian (E-Estonia)',
        'eu' => 'Isi-Basque',
        'eu_ES' => 'Isi-Basque (ESpain)',
        'fa' => 'Isi-Persia',
        'fa_AF' => 'Isi-Persia (E-Afghanistan)',
        'fa_IR' => 'Isi-Persia (E-Iran)',
        'fi' => 'Isi-Finnish',
        'fi_FI' => 'Isi-Finnish (EFinland)',
        'fo' => 'Isi-Faroese',
        'fo_DK' => 'Isi-Faroese (EDenmark)',
        'fo_FO' => 'Isi-Faroese (EFaroe Islands)',
        'fr' => 'IsiFrentshi',
        'fr_BE' => 'IsiFrentshi (EBelgium)',
        'fr_BF' => 'IsiFrentshi (EBurkina Faso)',
        'fr_BI' => 'IsiFrentshi (EBurundi)',
        'fr_BJ' => 'IsiFrentshi (EBenin)',
        'fr_BL' => 'IsiFrentshi (ESt. Barthélemy)',
        'fr_CA' => 'IsiFrentshi (EKhanada)',
        'fr_CD' => 'IsiFrentshi (ECongo -Kinshasa)',
        'fr_CF' => 'IsiFrentshi (ECentral African Republic)',
        'fr_CG' => 'IsiFrentshi (ECongo - Brazzaville)',
        'fr_CH' => 'IsiFrentshi (ESwitzerland)',
        'fr_CI' => 'IsiFrentshi (ECôte d’Ivoire)',
        'fr_CM' => 'IsiFrentshi (ECameroon)',
        'fr_DJ' => 'IsiFrentshi (EDjibouti)',
        'fr_DZ' => 'IsiFrentshi (E-Algeria)',
        'fr_FR' => 'IsiFrentshi (EFrance)',
        'fr_GA' => 'IsiFrentshi (EGabon)',
        'fr_GF' => 'IsiFrentshi (EFrench Guiana)',
        'fr_GN' => 'IsiFrentshi (EGuinea)',
        'fr_GP' => 'IsiFrentshi (EGuadeloupe)',
        'fr_GQ' => 'IsiFrentshi (E-Equatorial Guinea)',
        'fr_HT' => 'IsiFrentshi (EHaiti)',
        'fr_KM' => 'IsiFrentshi (EComoros)',
        'fr_LU' => 'IsiFrentshi (ELuxembourg)',
        'fr_MA' => 'IsiFrentshi (EMorocco)',
        'fr_MC' => 'IsiFrentshi (EMonaco)',
        'fr_MF' => 'IsiFrentshi (ESt. Martin)',
        'fr_MG' => 'IsiFrentshi (EMadagascar)',
        'fr_ML' => 'IsiFrentshi (EMali)',
        'fr_MQ' => 'IsiFrentshi (EMartinique)',
        'fr_MR' => 'IsiFrentshi (EMauritania)',
        'fr_MU' => 'IsiFrentshi (EMauritius)',
        'fr_NC' => 'IsiFrentshi (ENew Caledonia)',
        'fr_NE' => 'IsiFrentshi (ENiger)',
        'fr_PF' => 'IsiFrentshi (EFrench Polynesia)',
        'fr_PM' => 'IsiFrentshi (ESt. Pierre & Miquelon)',
        'fr_RE' => 'IsiFrentshi (ERéunion)',
        'fr_RW' => 'IsiFrentshi (ERwanda)',
        'fr_SC' => 'IsiFrentshi (ESeychelles)',
        'fr_SN' => 'IsiFrentshi (ESenegal)',
        'fr_SY' => 'IsiFrentshi (ESiriya)',
        'fr_TD' => 'IsiFrentshi (EChad)',
        'fr_TG' => 'IsiFrentshi (ETogo)',
        'fr_TN' => 'IsiFrentshi (ETunisia)',
        'fr_VU' => 'IsiFrentshi (EVanuatu)',
        'fr_WF' => 'IsiFrentshi (EWallis & Futuna)',
        'fr_YT' => 'IsiFrentshi (EMayotte)',
        'fy' => 'Isi-Frisian',
        'fy_NL' => 'Isi-Frisian (ENetherlands)',
        'ga' => 'Isi-Irish',
        'ga_GB' => 'Isi-Irish (E-United Kingdom)',
        'ga_IE' => 'Isi-Irish (E-Ireland)',
        'gd' => 'Scots Gaelic',
        'gd_GB' => 'Scots Gaelic (E-United Kingdom)',
        'gl' => 'Isi-Galician',
        'gl_ES' => 'Isi-Galician (ESpain)',
        'gu' => 'Isi-Gujarati',
        'gu_IN' => 'Isi-Gujarati (E-Indiya)',
        'he' => 'Isi-Hebrew',
        'he_IL' => 'Isi-Hebrew (E-Israel)',
        'hi' => 'IsiHindi',
        'hi_IN' => 'IsiHindi (E-Indiya)',
        'hi_Latn' => 'IsiHindi (IsiLatin)',
        'hi_Latn_IN' => 'IsiHindi (IsiLatin, E-Indiya)',
        'hr' => 'Isi-Croatia',
        'hr_BA' => 'Isi-Croatia (EBosnia & Herzegovina)',
        'hr_HR' => 'Isi-Croatia (ECroatia)',
        'hu' => 'Isi-Hungarian',
        'hu_HU' => 'Isi-Hungarian (EHungary)',
        'hy' => 'IsiArmenia',
        'hy_AM' => 'IsiArmenia (E-Armenia)',
        'ia' => 'Isi-Interlingua',
        'ia_001' => 'Isi-Interlingua (ihlabathi)',
        'id' => 'Isi-Indonesia',
        'id_ID' => 'Isi-Indonesia (E-Indonesia)',
        'ie' => 'Isi-Interlingue',
        'ie_EE' => 'Isi-Interlingue (E-Estonia)',
        'is' => 'Isi-Icelandic',
        'is_IS' => 'Isi-Icelandic (E-Iceland)',
        'it' => 'IsiTaliyane',
        'it_CH' => 'IsiTaliyane (ESwitzerland)',
        'it_IT' => 'IsiTaliyane (E-Italy)',
        'it_SM' => 'IsiTaliyane (ESan Marino)',
        'it_VA' => 'IsiTaliyane (EVatican City)',
        'ja' => 'IsiJapan',
        'ja_JP' => 'IsiJapan (EJapan)',
        'jv' => 'Isi-Javanese',
        'jv_ID' => 'Isi-Javanese (E-Indonesia)',
        'ka' => 'Isi-Georgia',
        'ka_GE' => 'Isi-Georgia (EGeorgia)',
        'km' => 'IsiCambodia',
        'km_KH' => 'IsiCambodia (ECambodia)',
        'kn' => 'Isi-Kannada',
        'kn_IN' => 'Isi-Kannada (E-Indiya)',
        'ko' => 'Isi-Korean',
        'ko_CN' => 'Isi-Korean (ETshayina)',
        'ko_KP' => 'Isi-Korean (EMntla Korea)',
        'ko_KR' => 'Isi-Korean (EMzantsi Korea)',
        'ku' => 'IsiKurdish',
        'ku_Latn' => 'IsiKurdish (IsiLatin)',
        'ku_Latn_IQ' => 'IsiKurdish (IsiLatin, E-Iraq)',
        'ku_Latn_SY' => 'IsiKurdish (IsiLatin, ESiriya)',
        'ku_Latn_TR' => 'IsiKurdish (IsiLatin, ETurkey)',
        'ku_SY' => 'IsiKurdish (ESiriya)',
        'ku_TR' => 'IsiKurdish (ETurkey)',
        'ky' => 'IsiKyrgz',
        'ky_KG' => 'IsiKyrgz (EKyrgyzstan)',
        'lo' => 'IsiLoathian',
        'lo_LA' => 'IsiLoathian (ELaos)',
        'lt' => 'Isi-Lithuanian',
        'lt_LT' => 'Isi-Lithuanian (ELithuania)',
        'lv' => 'Isi-Latvian',
        'lv_LV' => 'Isi-Latvian (ELatvia)',
        'mk' => 'Isi-Macedonian',
        'mk_MK' => 'Isi-Macedonian (EMntla Macedonia)',
        'ml' => 'Isi-Malayalam',
        'ml_IN' => 'Isi-Malayalam (E-Indiya)',
        'mn' => 'IsiMongolian',
        'mn_MN' => 'IsiMongolian (EMongolia)',
        'mr' => 'Isi-Marathi',
        'mr_IN' => 'Isi-Marathi (E-Indiya)',
        'ms' => 'Isi-Malay',
        'ms_BN' => 'Isi-Malay (eBrunei)',
        'ms_ID' => 'Isi-Malay (E-Indonesia)',
        'ms_MY' => 'Isi-Malay (EMalaysia)',
        'ms_SG' => 'Isi-Malay (ESingapore)',
        'mt' => 'Isi-Maltese',
        'mt_MT' => 'Isi-Maltese (EMalta)',
        'ne' => 'Isi-Nepali',
        'ne_IN' => 'Isi-Nepali (E-Indiya)',
        'ne_NP' => 'Isi-Nepali (ENepal)',
        'nl' => 'IsiDatshi',
        'nl_AW' => 'IsiDatshi (E-Aruba)',
        'nl_BE' => 'IsiDatshi (EBelgium)',
        'nl_BQ' => 'IsiDatshi (ECaribbean Netherlands)',
        'nl_CW' => 'IsiDatshi (ECuraçao)',
        'nl_NL' => 'IsiDatshi (ENetherlands)',
        'nl_SR' => 'IsiDatshi (ESuriname)',
        'nl_SX' => 'IsiDatshi (ESint Maarten)',
        'nn' => 'Isi-Norwegia [Nynorsk]',
        'nn_NO' => 'Isi-Norwegia [Nynorsk] (ENorway)',
        'no' => 'Isi-Norwegian',
        'no_NO' => 'Isi-Norwegian (ENorway)',
        'oc' => 'Iso-Occitan',
        'oc_ES' => 'Iso-Occitan (ESpain)',
        'oc_FR' => 'Iso-Occitan (EFrance)',
        'or' => 'Oriya',
        'or_IN' => 'Oriya (E-Indiya)',
        'pa' => 'Isi-Punjabi',
        'pa_Arab' => 'Isi-Punjabi (Isi-Arabhu)',
        'pa_Arab_PK' => 'Isi-Punjabi (Isi-Arabhu, EPakistan)',
        'pa_IN' => 'Isi-Punjabi (E-Indiya)',
        'pa_PK' => 'Isi-Punjabi (EPakistan)',
        'pl' => 'Isi-Polish',
        'pl_PL' => 'Isi-Polish (EPoland)',
        'ps' => 'IsiPashto',
        'ps_AF' => 'IsiPashto (E-Afghanistan)',
        'ps_PK' => 'IsiPashto (EPakistan)',
        'pt' => 'IsiPhuthukezi',
        'pt_AO' => 'IsiPhuthukezi (E-Angola)',
        'pt_BR' => 'IsiPhuthukezi (EBrazil)',
        'pt_CH' => 'IsiPhuthukezi (ESwitzerland)',
        'pt_CV' => 'IsiPhuthukezi (ECape Verde)',
        'pt_GQ' => 'IsiPhuthukezi (E-Equatorial Guinea)',
        'pt_GW' => 'IsiPhuthukezi (EGuinea-Bissau)',
        'pt_LU' => 'IsiPhuthukezi (ELuxembourg)',
        'pt_MO' => 'IsiPhuthukezi (EMacao SAR China)',
        'pt_MZ' => 'IsiPhuthukezi (EMozambique)',
        'pt_PT' => 'IsiPhuthukezi (EPortugal)',
        'pt_ST' => 'IsiPhuthukezi (ESão Tomé & Príncipe)',
        'pt_TL' => 'IsiPhuthukezi (ETimor-Leste)',
        'ro' => 'Isi-Romanian',
        'ro_MD' => 'Isi-Romanian (EMoldova)',
        'ro_RO' => 'Isi-Romanian (ERomania)',
        'ru' => 'Isi-Russian',
        'ru_BY' => 'Isi-Russian (EBelarus)',
        'ru_KG' => 'Isi-Russian (EKyrgyzstan)',
        'ru_KZ' => 'Isi-Russian (EKazakhstan)',
        'ru_MD' => 'Isi-Russian (EMoldova)',
        'ru_RU' => 'Isi-Russian (ERashiya)',
        'ru_UA' => 'Isi-Russian (E-Ukraine)',
        'sa' => 'IsiSanskrit',
        'sa_IN' => 'IsiSanskrit (E-Indiya)',
        'sd' => 'IsiSindhi',
        'sd_Arab' => 'IsiSindhi (Isi-Arabhu)',
        'sd_Arab_PK' => 'IsiSindhi (Isi-Arabhu, EPakistan)',
        'sd_IN' => 'IsiSindhi (E-Indiya)',
        'sd_PK' => 'IsiSindhi (EPakistan)',
        'si' => 'Isi-Sinhalese',
        'si_LK' => 'Isi-Sinhalese (ESri Lanka)',
        'sk' => 'Isi-Slovak',
        'sk_SK' => 'Isi-Slovak (ESlovakia)',
        'sl' => 'Isi-Slovenian',
        'sl_SI' => 'Isi-Slovenian (ESlovenia)',
        'so' => 'IsiSomaliya',
        'so_DJ' => 'IsiSomaliya (EDjibouti)',
        'so_ET' => 'IsiSomaliya (E-Ethiopia)',
        'so_KE' => 'IsiSomaliya (EKenya)',
        'so_SO' => 'IsiSomaliya (ESomalia)',
        'sq' => 'IsiAlbania',
        'sq_AL' => 'IsiAlbania (E-Albania)',
        'sq_MK' => 'IsiAlbania (EMntla Macedonia)',
        'sr' => 'Isi-Serbia',
        'sr_BA' => 'Isi-Serbia (EBosnia & Herzegovina)',
        'sr_Cyrl' => 'Isi-Serbia (IsiCyrillic)',
        'sr_Cyrl_BA' => 'Isi-Serbia (IsiCyrillic, EBosnia & Herzegovina)',
        'sr_Cyrl_ME' => 'Isi-Serbia (IsiCyrillic, EMontenegro)',
        'sr_Cyrl_RS' => 'Isi-Serbia (IsiCyrillic, ESerbia)',
        'sr_Latn' => 'Isi-Serbia (IsiLatin)',
        'sr_Latn_BA' => 'Isi-Serbia (IsiLatin, EBosnia & Herzegovina)',
        'sr_Latn_ME' => 'Isi-Serbia (IsiLatin, EMontenegro)',
        'sr_Latn_RS' => 'Isi-Serbia (IsiLatin, ESerbia)',
        'sr_ME' => 'Isi-Serbia (EMontenegro)',
        'sr_RS' => 'Isi-Serbia (ESerbia)',
        'st' => 'Sesotho',
        'st_LS' => 'Sesotho (ELesotho)',
        'st_ZA' => 'Sesotho (EMzantsi Afrika)',
        'su' => 'Isi-Sudanese',
        'su_ID' => 'Isi-Sudanese (E-Indonesia)',
        'su_Latn' => 'Isi-Sudanese (IsiLatin)',
        'su_Latn_ID' => 'Isi-Sudanese (IsiLatin, E-Indonesia)',
        'sv' => 'Isi-Swedish',
        'sv_AX' => 'Isi-Swedish (E-Åland Islands)',
        'sv_FI' => 'Isi-Swedish (EFinland)',
        'sv_SE' => 'Isi-Swedish (ESweden)',
        'sw' => 'Isi-Swahili',
        'sw_CD' => 'Isi-Swahili (ECongo -Kinshasa)',
        'sw_KE' => 'Isi-Swahili (EKenya)',
        'sw_TZ' => 'Isi-Swahili (ETanzania)',
        'sw_UG' => 'Isi-Swahili (E-Uganda)',
        'ta' => 'Isi-Tamil',
        'ta_IN' => 'Isi-Tamil (E-Indiya)',
        'ta_LK' => 'Isi-Tamil (ESri Lanka)',
        'ta_MY' => 'Isi-Tamil (EMalaysia)',
        'ta_SG' => 'Isi-Tamil (ESingapore)',
        'te' => 'Isi-Telegu',
        'te_IN' => 'Isi-Telegu (E-Indiya)',
        'th' => 'Isi-Thai',
        'th_TH' => 'Isi-Thai (EThailand)',
        'ti' => 'Isi-Tigrinya',
        'ti_ER' => 'Isi-Tigrinya (E-Eritrea)',
        'ti_ET' => 'Isi-Tigrinya (E-Ethiopia)',
        'tk' => 'IsiTurkmen',
        'tk_TM' => 'IsiTurkmen (ETurkmenistan)',
        'tr' => 'Isi-Turkish',
        'tr_CY' => 'Isi-Turkish (ECyprus)',
        'tr_TR' => 'Isi-Turkish (ETurkey)',
        'ug' => 'Isi Uighur',
        'ug_CN' => 'Isi Uighur (ETshayina)',
        'uk' => 'Isi-Ukranian',
        'uk_UA' => 'Isi-Ukranian (E-Ukraine)',
        'ur' => 'IsiUrdu',
        'ur_IN' => 'IsiUrdu (E-Indiya)',
        'ur_PK' => 'IsiUrdu (EPakistan)',
        'uz' => 'Isi-Uzbek',
        'uz_AF' => 'Isi-Uzbek (E-Afghanistan)',
        'uz_Arab' => 'Isi-Uzbek (Isi-Arabhu)',
        'uz_Arab_AF' => 'Isi-Uzbek (Isi-Arabhu, E-Afghanistan)',
        'uz_Cyrl' => 'Isi-Uzbek (IsiCyrillic)',
        'uz_Cyrl_UZ' => 'Isi-Uzbek (IsiCyrillic, E-Uzbekistan)',
        'uz_Latn' => 'Isi-Uzbek (IsiLatin)',
        'uz_Latn_UZ' => 'Isi-Uzbek (IsiLatin, E-Uzbekistan)',
        'uz_UZ' => 'Isi-Uzbek (E-Uzbekistan)',
        'vi' => 'Isi-Vietnamese',
        'vi_VN' => 'Isi-Vietnamese (EVietnam)',
        'xh' => 'IsiXhosa',
        'xh_ZA' => 'IsiXhosa (EMzantsi Afrika)',
        'zh' => 'IsiMandarin',
        'zh_CN' => 'IsiMandarin (ETshayina)',
        'zh_HK' => 'IsiMandarin (EHong Kong SAR China)',
        'zh_Hans' => 'IsiMandarin (IsiHans Esenziwe Lula)',
        'zh_Hans_CN' => 'IsiMandarin (IsiHans Esenziwe Lula, ETshayina)',
        'zh_Hans_HK' => 'IsiMandarin (IsiHans Esenziwe Lula, EHong Kong SAR China)',
        'zh_Hans_MO' => 'IsiMandarin (IsiHans Esenziwe Lula, EMacao SAR China)',
        'zh_Hans_MY' => 'IsiMandarin (IsiHans Esenziwe Lula, EMalaysia)',
        'zh_Hans_SG' => 'IsiMandarin (IsiHans Esenziwe Lula, ESingapore)',
        'zh_Hant' => 'IsiMandarin (IsiHant Esiqhelekileyo)',
        'zh_Hant_HK' => 'IsiMandarin (IsiHant Esiqhelekileyo, EHong Kong SAR China)',
        'zh_Hant_MO' => 'IsiMandarin (IsiHant Esiqhelekileyo, EMacao SAR China)',
        'zh_Hant_MY' => 'IsiMandarin (IsiHant Esiqhelekileyo, EMalaysia)',
        'zh_Hant_TW' => 'IsiMandarin (IsiHant Esiqhelekileyo, ETaiwan)',
        'zh_MO' => 'IsiMandarin (EMacao SAR China)',
        'zh_SG' => 'IsiMandarin (ESingapore)',
        'zh_TW' => 'IsiMandarin (ETaiwan)',
        'zu' => 'IsiZulu',
        'zu_ZA' => 'IsiZulu (EMzantsi Afrika)',
    ],
];
