<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\Messenger\Stamp;

/**
 * An envelope stamp related to a message.
 *
 * Stamps must be serializable value objects for transport.
 *
 * @author Maxime Steinhausser <maxime.steinhausser@gmail.com>
 */
interface StampInterface
{
}
