<?php

return [
    'Names' => [
        'CDF' => [
            'FC',
            'Faranga ya Kongo',
        ],
        'CNY' => [
            'CN¥',
            'Yuan Renminbi ya China',
        ],
        'ETB' => [
            'ETB',
            'Bir ya Uhabeshi',
        ],
        'JPY' => [
            'JP¥',
            'Sarafu ya Kijapani',
        ],
        'MGA' => [
            'MGA',
            'Ariary ya Bukini',
        ],
        'MRO' => [
            'MRO',
            'Ugwiya ya Moritania (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'Ugwiya ya Moritania',
        ],
        'SCR' => [
            'SCR',
            'Rupia ya Shelisheli',
        ],
        'XAF' => [
            'FCFA',
            'Faranga CFA BEAC',
        ],
        'XOF' => [
            'F CFA',
            'Faranga CFA BCEAO',
        ],
    ],
];
