<?php

return [
    'Names' => [
        'ar' => 'arabic',
        'ar_001' => 'arabic (munde)',
        'ar_ER' => 'arabic (Eritrea)',
        'ar_TD' => 'arabic (Tchad)',
        'cs' => 'tchec',
        'cs_CZ' => 'tchec (Tchekia)',
        'da' => 'danesi',
        'da_DK' => 'danesi (Dania)',
        'de' => 'german',
        'de_AT' => 'german (Austria)',
        'de_BE' => 'german (Belgia)',
        'de_CH' => 'german (Svissia)',
        'de_DE' => 'german (Germania)',
        'de_IT' => 'german (Italia)',
        'de_LI' => 'german (Liechtenstein)',
        'de_LU' => 'german (Luxemburg)',
        'el' => 'grec',
        'el_GR' => 'grec (Grecia)',
        'en' => 'anglesi',
        'en_001' => 'anglesi (munde)',
        'en_150' => 'anglesi (Europa)',
        'en_AT' => 'anglesi (Austria)',
        'en_BE' => 'anglesi (Belgia)',
        'en_CH' => 'anglesi (Svissia)',
        'en_CZ' => 'anglesi (Tchekia)',
        'en_DE' => 'anglesi (Germania)',
        'en_DK' => 'anglesi (Dania)',
        'en_EE' => 'anglesi (Estonia)',
        'en_ER' => 'anglesi (Eritrea)',
        'en_ES' => 'anglesi (Hispania)',
        'en_FI' => 'anglesi (Finland)',
        'en_FJ' => 'anglesi (Fidji)',
        'en_FR' => 'anglesi (Francia)',
        'en_GB' => 'anglesi (Unit Reyia)',
        'en_GG' => 'anglesi (Guernsey)',
        'en_GI' => 'anglesi (Gibraltar)',
        'en_GY' => 'anglesi (Guyana)',
        'en_HU' => 'anglesi (Hungaria)',
        'en_ID' => 'anglesi (Indonesia)',
        'en_IE' => 'anglesi (Irland)',
        'en_IM' => 'anglesi (Insul de Man)',
        'en_IN' => 'anglesi (India)',
        'en_IT' => 'anglesi (Italia)',
        'en_JE' => 'anglesi (Jersey)',
        'en_LT' => 'anglesi (Lituania)',
        'en_LV' => 'anglesi (Lettonia)',
        'en_MT' => 'anglesi (Malta)',
        'en_MU' => 'anglesi (Mauricio)',
        'en_MV' => 'anglesi (Maldivas)',
        'en_NF' => 'anglesi (Insul Norfolk)',
        'en_NL' => 'anglesi (Nederland)',
        'en_NO' => 'anglesi (Norvegia)',
        'en_NR' => 'anglesi (Nauru)',
        'en_NZ' => 'anglesi (Nov-Zeland)',
        'en_PH' => 'anglesi (Filipines)',
        'en_PK' => 'anglesi (Pakistan)',
        'en_PL' => 'anglesi (Polonia)',
        'en_PR' => 'anglesi (Porto-Rico)',
        'en_PT' => 'anglesi (Portugal)',
        'en_PW' => 'anglesi (Palau)',
        'en_RO' => 'anglesi (Rumania)',
        'en_SE' => 'anglesi (Svedia)',
        'en_SI' => 'anglesi (Slovenia)',
        'en_SK' => 'anglesi (Slovakia)',
        'en_SX' => 'anglesi (Sint-Maarten)',
        'en_TC' => 'anglesi (Turks e Caicos)',
        'en_TK' => 'anglesi (Tokelau)',
        'en_TT' => 'anglesi (Trinidad e Tobago)',
        'en_TV' => 'anglesi (Tuvalu)',
        'en_UA' => 'anglesi (Ukraina)',
        'en_VU' => 'anglesi (Vanuatu)',
        'en_WS' => 'anglesi (Samoa)',
        'eo' => 'Esperanto',
        'eo_001' => 'Esperanto (munde)',
        'es' => 'hispan',
        'es_419' => 'hispan (latin America)',
        'es_ES' => 'hispan (Hispania)',
        'es_PE' => 'hispan (Perú)',
        'es_PH' => 'hispan (Filipines)',
        'es_PR' => 'hispan (Porto-Rico)',
        'et' => 'estonian',
        'et_EE' => 'estonian (Estonia)',
        'fa' => 'persian',
        'fa_IR' => 'persian (Iran)',
        'fi' => 'finn',
        'fi_FI' => 'finn (Finland)',
        'fr' => 'francesi',
        'fr_BE' => 'francesi (Belgia)',
        'fr_CH' => 'francesi (Svissia)',
        'fr_FR' => 'francesi (Francia)',
        'fr_LU' => 'francesi (Luxemburg)',
        'fr_MC' => 'francesi (Mónaco)',
        'fr_MQ' => 'francesi (Martinica)',
        'fr_MU' => 'francesi (Mauricio)',
        'fr_TD' => 'francesi (Tchad)',
        'fr_VU' => 'francesi (Vanuatu)',
        'ga' => 'irlandesi',
        'ga_GB' => 'irlandesi (Unit Reyia)',
        'ga_IE' => 'irlandesi (Irland)',
        'gl' => 'galician',
        'gl_ES' => 'galician (Hispania)',
        'ha' => 'hausa',
        'he' => 'hebreic',
        'hi' => 'hindi',
        'hi_IN' => 'hindi (India)',
        'hi_Latn' => 'hindi (latin)',
        'hi_Latn_IN' => 'hindi (latin, India)',
        'hu' => 'hungarian',
        'hu_HU' => 'hungarian (Hungaria)',
        'ia' => 'Interlingua',
        'ia_001' => 'Interlingua (munde)',
        'id' => 'indonesian',
        'id_ID' => 'indonesian (Indonesia)',
        'ie' => 'Interlingue',
        'ie_EE' => 'Interlingue (Estonia)',
        'is' => 'islandesi',
        'is_IS' => 'islandesi (Island)',
        'it' => 'italian',
        'it_CH' => 'italian (Svissia)',
        'it_IT' => 'italian (Italia)',
        'it_SM' => 'italian (San-Marino)',
        'it_VA' => 'italian (Cité de Vatican)',
        'ja' => 'japanesi',
        'jv' => 'javan',
        'jv_ID' => 'javan (Indonesia)',
        'ka' => 'georgian',
        'ko' => 'korean',
        'lv' => 'lettonian',
        'lv_LV' => 'lettonian (Lettonia)',
        'mt' => 'maltesi',
        'mt_MT' => 'maltesi (Malta)',
        'nl' => 'hollandesi',
        'nl_BE' => 'hollandesi (Belgia)',
        'nl_NL' => 'hollandesi (Nederland)',
        'nl_SX' => 'hollandesi (Sint-Maarten)',
        'pl' => 'polonesi',
        'pl_PL' => 'polonesi (Polonia)',
        'pt' => 'portugalesi',
        'pt_CH' => 'portugalesi (Svissia)',
        'pt_LU' => 'portugalesi (Luxemburg)',
        'pt_PT' => 'portugalesi (Portugal)',
        'pt_TL' => 'portugalesi (Ost-Timor)',
        'ru' => 'russ',
        'ru_BY' => 'russ (Bielorussia)',
        'ru_MD' => 'russ (Moldova)',
        'ru_RU' => 'russ (Russia)',
        'ru_UA' => 'russ (Ukraina)',
        'sk' => 'slovac',
        'sk_SK' => 'slovac (Slovakia)',
        'sl' => 'slovenian',
        'sl_SI' => 'slovenian (Slovenia)',
        'sv' => 'sved',
        'sv_AX' => 'sved (Insules Åland)',
        'sv_FI' => 'sved (Finland)',
        'sv_SE' => 'sved (Svedia)',
        'sw' => 'swahili',
        'th' => 'thai',
        'tr' => 'turc',
        'zh' => 'chinesi',
        'zh_Hans' => 'chinesi (simplificat)',
        'zh_Hant' => 'chinesi (traditional)',
    ],
];
