<?php

return [
    'Names' => [
        'af' => 'Afrikaans',
        'ak' => 'Akan',
        'am' => 'Amariki',
        'ar' => 'Arabeke',
        'as' => 'Asamese',
        'ast' => 'Asturiani',
        'az' => 'Asabegyanni',
        'bal' => 'Balukyi',
        'be' => 'Belarus kasa',
        'bg' => 'Bɔlgeria kasa',
        'bgc' => 'Harianvi',
        'bho' => 'Bopuri',
        'blo' => 'Anii',
        'bn' => 'Bengali kasa',
        'br' => 'Britenni',
        'brx' => 'Bodo',
        'bs' => 'Bosniani',
        'ca' => 'Katalan',
        'ceb' => 'Kebuano',
        'chr' => 'Kiroki',
        'cs' => 'Kyɛk kasa',
        'csw' => 'Tadeɛm Kreefoɔ Kasa',
        'cv' => 'Kyuvahyi',
        'cy' => 'Wɛɛhye Kasa',
        'da' => 'Dane kasa',
        'de' => 'Gyaaman',
        'doi' => 'Dɔgri',
        'dsb' => 'Sɔɔbia a ɛwɔ fam',
        'ee' => 'Ayigbe',
        'el' => 'Greek kasa',
        'en' => 'Borɔfo',
        'eo' => 'Esperanto',
        'es' => 'Spain kasa',
        'et' => 'Estonia kasa',
        'eu' => 'Baske',
        'fa' => 'Pɛɛhyia kasa',
        'ff' => 'Fula kasa',
        'fi' => 'Finlande kasa',
        'fil' => 'Filipin kasa',
        'fo' => 'Farosi',
        'fr' => 'Frɛnkye',
        'fy' => 'Atɔeɛ Fam Frihyia Kasa',
        'ga' => 'Aerelande kasa',
        'gd' => 'Skotlandfoɔ Galek Kasa',
        'gl' => 'Galisia kasa',
        'gu' => 'Gugyarata',
        'ha' => 'Hausa',
        'he' => 'Hibri kasa',
        'hi' => 'Hindi',
        'hr' => 'Kurowehyia kasa',
        'hsb' => 'Atifi fam Sɔɔbia Kasa',
        'hu' => 'Hangri kasa',
        'hy' => 'Aameniani',
        'ia' => 'Kasa ntam',
        'id' => 'Indonihyia kasa',
        'ie' => 'Kasa afrafra',
        'ig' => 'Igbo kasa',
        'is' => 'Aeslande kasa',
        'it' => 'Italy kasa',
        'ja' => 'Gyapan kasa',
        'jv' => 'Gyabanis kasa',
        'ka' => 'Gyɔɔgyia kasa',
        'kea' => 'Kabuvadianu',
        'kgp' => 'Kaingang',
        'kk' => 'kasaki kasa',
        'km' => 'Kambodia kasa',
        'kn' => 'Kanada',
        'ko' => 'Korea kasa',
        'kok' => 'Konkani kasa',
        'ks' => 'Kahyimiɛ',
        'ku' => 'Kɛɛde kasa',
        'kxv' => 'Kuvi kasa',
        'ky' => 'Kɛgyese kasa',
        'lb' => 'Lɔsimbɔge kasa',
        'lij' => 'Liguria kasa',
        'lmo' => 'Lombad kasa',
        'lo' => 'Lawo kasa',
        'lt' => 'Lituania kasa',
        'lv' => 'Latvia kasa',
        'mai' => 'Maetili',
        'mi' => 'Mawori',
        'mk' => 'Mɛsidonia kasa',
        'ml' => 'Malayalam kasa',
        'mn' => 'Mongoliafoɔ kasa',
        'mni' => 'Manipuri',
        'mr' => 'Marati',
        'ms' => 'Malay kasa',
        'mt' => 'Malta kasa',
        'my' => 'Bɛɛmis kasa',
        'nds' => 'Gyaaman kasa a ɛwɔ fam',
        'ne' => 'Nɛpal kasa',
        'nl' => 'Dɛɛkye',
        'nn' => 'Nɔwefoɔ Ninɔso',
        'no' => 'Nɔwefoɔ kasa',
        'nqo' => 'Nko',
        'oc' => 'Osita kasa',
        'or' => 'Odia',
        'pa' => 'Pungyabi kasa',
        'pcm' => 'Nigeriafoɔ Pigyin',
        'pl' => 'Pɔland kasa',
        'prg' => 'Prusia kasa',
        'ps' => 'Pahyito',
        'pt' => 'Pɔɔtugal kasa',
        'qu' => 'Kwɛkya',
        'raj' => 'Ragyasitan kasa',
        'rm' => 'Romanhye kasa',
        'ro' => 'Romenia kasa',
        'ru' => 'Rahyia kasa',
        'rw' => 'Rewanda kasa',
        'sa' => 'Sanskrit kasa',
        'sah' => 'Yakut Kasa',
        'sat' => 'Santal kasa',
        'sc' => 'Saadinia kasa',
        'sd' => 'Sindi',
        'si' => 'Sinhala',
        'sk' => 'Slovak Kasa',
        'sl' => 'Slovɛniafoɔ Kasa',
        'so' => 'Somalia kasa',
        'sq' => 'Aabeniani',
        'sr' => 'Sɛbia Kasa',
        'su' => 'Sunda Kasa',
        'sv' => 'Sweden kasa',
        'sw' => 'Swahili',
        'syr' => 'Siiria Kasa',
        'szl' => 'Silesiafoɔ Kasa',
        'ta' => 'Tamil kasa',
        'te' => 'Telugu',
        'tg' => 'Tɛgyeke kasa',
        'th' => 'Taeland kasa',
        'ti' => 'Tigrinya kasa',
        'tk' => 'Tɛkmɛnistan Kasa',
        'to' => 'Tonga kasa',
        'tr' => 'Tɛɛki kasa',
        'tt' => 'Tata kasa',
        'ug' => 'Yugaa Kasa',
        'uk' => 'Ukren kasa',
        'ur' => 'Urdu kasa',
        'uz' => 'Usbɛkistan Kasa',
        'vec' => 'Vɛnihyia Kasa',
        'vi' => 'Viɛtnam kasa',
        'vmw' => 'Makuwa',
        'wo' => 'Wolɔfo Kasa',
        'xh' => 'Hosa Kasa',
        'xnr' => 'Kangri',
        'yo' => 'Yoruba',
        'yrl' => 'Ningatu',
        'yue' => 'Kantonese',
        'za' => 'Zuang',
        'zh' => 'Kyaena kasa',
        'zu' => 'Zulu',
    ],
    'LocalizedNames' => [
        'ar_001' => 'Arabeke Kasa Nhyehyɛeɛ Foforɔ',
        'de_AT' => 'Ɔstria Gyaaman',
        'de_CH' => 'Swisalande Gyaaman',
        'en_GB' => 'Ngresi Borɔfo',
        'en_US' => 'Amɛrika Borɔfo',
        'fr_CA' => 'Kanada Frɛnkye',
        'fr_CH' => 'Swisalande Frɛnkye',
        'hi_Latn' => 'Laatenfoɔ Hindi',
        'nl_BE' => 'Dɛɛkye (Bɛɛgyiɔm',
        'zh_Hans' => 'Kyaena kasa a emu yɛ mmrɛ',
        'zh_Hant' => 'Tete Kyaena kasa',
    ],
];
