<?php

return [
    'Names' => [
        'AED' => [
            'AED',
            'I-Dirham yase-UAE',
        ],
        'AFN' => [
            'AFN',
            'I-Afghani yase-Afghanistan',
        ],
        'ALL' => [
            'ALL',
            'I-Lek yase-Albania',
        ],
        'AMD' => [
            'AMD',
            'I-Dram yase Armenia',
        ],
        'ANG' => [
            'ANG',
            'Netherlands Antillean Guilder',
        ],
        'AOA' => [
            'AOA',
            'IKwanza yaseAngola',
        ],
        'ARS' => [
            'ARS',
            'IPeso yase-Argentina',
        ],
        'AUD' => [
            'A$',
            'IDola yase-Australia',
        ],
        'AWG' => [
            'AWG',
            'Aruban Florin',
        ],
        'AZN' => [
            'AZN',
            'I-Manat yase-Azerbeijan',
        ],
        'BAM' => [
            'BAM',
            'I-Convertible Mark yaseBosnia-Herzegovina',
        ],
        'BBD' => [
            'BBD',
            'Barbadian Dollar',
        ],
        'BDT' => [
            'BDT',
            'I-Taka yaseBangladesh',
        ],
        'BGN' => [
            'BGN',
            'I-Lev yaseBulgaria',
        ],
        'BHD' => [
            'BHD',
            'I-Dinar yaseBahrain',
        ],
        'BIF' => [
            'BIF',
            'I-Franc yaseBurundi',
        ],
        'BMD' => [
            'BMD',
            'Bermudan Dollar',
        ],
        'BND' => [
            'BND',
            'IDola yaseBrunei',
        ],
        'BOB' => [
            'BOB',
            'I-Boliviano yaseBolivia',
        ],
        'BRL' => [
            'R$',
            'I-Real yaseBrazil',
        ],
        'BSD' => [
            'BSD',
            'Bahamian Dollar',
        ],
        'BTN' => [
            'BTN',
            'I-Ngultrum yaseBhutan',
        ],
        'BWP' => [
            'BWP',
            'I-Pula yaseBotswana',
        ],
        'BYN' => [
            'BYN',
            'I-Ruble yaseBelarus',
        ],
        'BZD' => [
            'BZD',
            'Belize Dollar',
        ],
        'CAD' => [
            'CA$',
            'Canadian Dollar',
        ],
        'CDF' => [
            'CDF',
            'IFranc yaseCongo',
        ],
        'CHF' => [
            'CHF',
            'IFranc yaseSwirtzeland',
        ],
        'CLP' => [
            'CLP',
            'I-Peso yaseChile',
        ],
        'CNH' => [
            'I-CNH',
            'I-Chinese Yuan (offshore)',
        ],
        'CNY' => [
            'CN¥',
            'I-Yuan yaseTshayina',
        ],
        'COP' => [
            'COP',
            'I-Peso yaseColombia',
        ],
        'CRC' => [
            'CRC',
            'Costa Rican Colón',
        ],
        'CUC' => [
            'CUC',
            'Cuban Convertible Peso',
        ],
        'CUP' => [
            'CUP',
            'Cuban Peso',
        ],
        'CVE' => [
            'CVE',
            'Cape Verdean Escudo',
        ],
        'CZK' => [
            'CZK',
            'I-Koruna yaseCzech',
        ],
        'DJF' => [
            'DJF',
            'I-Franc yaseDjibouti',
        ],
        'DKK' => [
            'DKK',
            'I-Krone yaseDenmark',
        ],
        'DOP' => [
            'DOP',
            'Dominican Peso',
        ],
        'DZD' => [
            'DZD',
            'I-Dinar yase-Algeria',
        ],
        'EGP' => [
            'EGP',
            'IPonti yase-Egypt',
        ],
        'ERN' => [
            'ERN',
            'I-Nakfa yase-Eritria',
        ],
        'ETB' => [
            'ETB',
            'I-Birr yase-Ethopia',
        ],
        'EUR' => [
            '€',
            'I-Euro',
        ],
        'FJD' => [
            'FJD',
            'IDola yaseFiji',
        ],
        'FKP' => [
            'FKP',
            'IPonti yaseFalkland Islands',
        ],
        'GBP' => [
            '£',
            'IPonti yaseBritane',
        ],
        'GEL' => [
            'GEL',
            'I-Lari yaseGeorgia',
        ],
        'GHS' => [
            'GHS',
            'ICedi yaseGhana',
        ],
        'GIP' => [
            'GIP',
            'IPonti yaseGilbraltar',
        ],
        'GMD' => [
            'GMD',
            'IDalasi yaseGambia',
        ],
        'GNF' => [
            'GNF',
            'IFranc yaseGuinea',
        ],
        'GTQ' => [
            'GTQ',
            'Guatemalan Quetzal',
        ],
        'GYD' => [
            'GYD',
            'IDola yaseGuyana',
        ],
        'HKD' => [
            'HK$',
            'IDola yaseHong Kong',
        ],
        'HNL' => [
            'HNL',
            'Honduran Lempira',
        ],
        'HRK' => [
            'HRK',
            'I-Kuna yaseCrotia',
        ],
        'HTG' => [
            'HTG',
            'Haitian Gourde',
        ],
        'HUF' => [
            'HUF',
            'I-Forint yaseHungay',
        ],
        'IDR' => [
            'IDR',
            'I-Rupiah yase-Indonesia',
        ],
        'ILS' => [
            '₪',
            'I-New Shekel yase-Israel',
        ],
        'INR' => [
            '₹',
            'I-Rupee yase-Indiya',
        ],
        'IQD' => [
            'IQD',
            'I-Dinar yase-Iraq',
        ],
        'IRR' => [
            'IRR',
            'I-Rial yase-Iran',
        ],
        'ISK' => [
            'ISK',
            'IKróna yaseIceland',
        ],
        'JMD' => [
            'JMD',
            'Jamaican Dollar',
        ],
        'JOD' => [
            'JOD',
            'I-Dinar yaseJordan',
        ],
        'JPY' => [
            '¥',
            'I-Yen yaseJapan',
        ],
        'KES' => [
            'KES',
            'I-Shilling yaseKenya',
        ],
        'KGS' => [
            'KGS',
            'I-Som yaseKyrgystan',
        ],
        'KHR' => [
            'KHR',
            'I-Riel yaseCambodia',
        ],
        'KMF' => [
            'KMF',
            'I-Franc yaseComoros',
        ],
        'KPW' => [
            'KPW',
            'I-Won yaseNorth Korea',
        ],
        'KRW' => [
            '₩',
            'I-Won yaseSouth Korea',
        ],
        'KWD' => [
            'KWD',
            'I-Dinar yaseKuwait',
        ],
        'KYD' => [
            'KYD',
            'Cayman Islands Dollar',
        ],
        'KZT' => [
            'KZT',
            'I-Tenge yaseKhazakhstan',
        ],
        'LAK' => [
            'LAK',
            'I-Kip yaseLaos',
        ],
        'LBP' => [
            'LBP',
            'IPonti yaseLebanon',
        ],
        'LKR' => [
            'LKR',
            'I-Rupee yaseSri Lanka',
        ],
        'LRD' => [
            'LRD',
            'IDola yaseLiberia',
        ],
        'LSL' => [
            'LSL',
            'I-Loti yaseLesotho',
        ],
        'LYD' => [
            'LYD',
            'Libyan Dinar',
        ],
        'MAD' => [
            'MAD',
            'Moroccan Dirham',
        ],
        'MDL' => [
            'MDL',
            'Moldovan Leu',
        ],
        'MGA' => [
            'MGA',
            'I-Ariary yaseMadagascar',
        ],
        'MKD' => [
            'MKD',
            'Macedonian Denar',
        ],
        'MMK' => [
            'MMK',
            'I-Kyat yaseMyanmar',
        ],
        'MNT' => [
            'MNT',
            'I-Tugrik yaseMongolia',
        ],
        'MOP' => [
            'MOP',
            'I-Pataca yaseMacao',
        ],
        'MRU' => [
            'MRU',
            'I-Ouguiya yaseMauritania',
        ],
        'MUR' => [
            'MUR',
            'I-Rupee yaseMauritius',
        ],
        'MVR' => [
            'MVR',
            'I-Rufiyaa yaseMaldives',
        ],
        'MWK' => [
            'MWK',
            'I-Kwacha yaseMalawi',
        ],
        'MXN' => [
            'MX$',
            'Mexican Peso',
        ],
        'MYR' => [
            'MYR',
            'I-Ringgit yaseMalysia',
        ],
        'MZN' => [
            'MZN',
            'I-Metical yaseMozambique',
        ],
        'NAD' => [
            'NAD',
            'IDola yaseNamibia',
        ],
        'NGN' => [
            'NGN',
            'INaira yaseNigeria',
        ],
        'NIO' => [
            'NIO',
            'Nicaraguan Córdoba',
        ],
        'NOK' => [
            'NOK',
            'IKrone yaseNorway',
        ],
        'NPR' => [
            'NPR',
            'I-Rupee yaseNepal',
        ],
        'NZD' => [
            'NZ$',
            'IDola yaseNew Zealand',
        ],
        'OMR' => [
            'OMR',
            'I-Rial yase-Oman',
        ],
        'PAB' => [
            'PAB',
            'Panamanian Balboa',
        ],
        'PEN' => [
            'PEN',
            'I-Sol yasePeruvia',
        ],
        'PGK' => [
            'PGK',
            'I-Kina yasePapua New Guinea',
        ],
        'PHP' => [
            '₱',
            'I-Peso yasePhilippines',
        ],
        'PKR' => [
            'PKR',
            'I-Rupee yasePakistan',
        ],
        'PLN' => [
            'PLN',
            'Polish Zloty',
        ],
        'PYG' => [
            'PYG',
            'I-Guarani yaseParaguay',
        ],
        'QAR' => [
            'QAR',
            'I-Riyal yaseQatar',
        ],
        'RON' => [
            'RON',
            'I-Leu yaseRomania',
        ],
        'RSD' => [
            'RSD',
            'I-Dinar yaseSerbia',
        ],
        'RUB' => [
            'RUB',
            'I-Ruble yaseRashiya',
        ],
        'RWF' => [
            'RWF',
            'I-Franc yaseRwanda',
        ],
        'SAR' => [
            'SAR',
            'I-Riyal yaseSaudi',
        ],
        'SBD' => [
            'SBD',
            'IDola yaseSolomon Islands',
        ],
        'SCR' => [
            'SCR',
            'I-Rupee yaseSeychelles',
        ],
        'SDG' => [
            'SDG',
            'Sudanese Pound',
        ],
        'SEK' => [
            'SEK',
            'IKrona yaseSweden',
        ],
        'SGD' => [
            'SGD',
            'IDola yaseSingapore',
        ],
        'SHP' => [
            'SHP',
            'IPonti yaseSt. Helena',
        ],
        'SLE' => [
            'SLE',
            'I-Loeone yaseSierra Leone',
        ],
        'SLL' => [
            'SLL',
            'I-Loeone yaseSierra Leone (1964—2022)',
        ],
        'SOS' => [
            'SOS',
            'I-Shilling yaseSomalia',
        ],
        'SRD' => [
            'SRD',
            'IDola yaseSuriname',
        ],
        'SSP' => [
            'SSP',
            'IPonti yaseSouth Sudan',
        ],
        'STN' => [
            'STN',
            'IDobra yaseSão Tomé & Príncipe',
        ],
        'SYP' => [
            'SYP',
            'IPonti yaseSiriya',
        ],
        'SZL' => [
            'SZL',
            'I-Lilangeni yase-Eswatini',
        ],
        'THB' => [
            'THB',
            'I-Baht yaseThailand',
        ],
        'TJS' => [
            'TJS',
            'I-Somoni yaseTajikistan',
        ],
        'TMT' => [
            'TMT',
            'I-Manat yaseTurkmenistan',
        ],
        'TND' => [
            'TND',
            'Tunisian Dinar',
        ],
        'TOP' => [
            'TOP',
            'I-Paʻanga yaseTonga',
        ],
        'TRY' => [
            'TRY',
            'I-Lira yaseTurkey',
        ],
        'TTD' => [
            'TTD',
            'Trinidad & Tobago Dollar',
        ],
        'TWD' => [
            'NT$',
            'IDola yaseNew Taiwan',
        ],
        'TZS' => [
            'TZS',
            'I-Shilling yaseTanzania',
        ],
        'UAH' => [
            'UAH',
            'I-Hryvnia yase-Ukraine',
        ],
        'UGX' => [
            'UGX',
            'I-Shilling yase-Uganda',
        ],
        'USD' => [
            '$',
            'US Dollar',
        ],
        'UYU' => [
            'UYU',
            'I-Peso yase-Uruguay',
        ],
        'UZS' => [
            'UZS',
            'I-Som yase-Uzbekistan',
        ],
        'VES' => [
            'VES',
            'I-Bolívar yaseVenezuela',
        ],
        'VND' => [
            '₫',
            'I-Dong yaseVietnam',
        ],
        'VUV' => [
            'VUV',
            'I-Vatu yaseVanuatu',
        ],
        'WST' => [
            'WST',
            'I-Tala yaseSamoa',
        ],
        'XAF' => [
            'FCFA',
            'ICFA Franc yaseCentral Africa',
        ],
        'XCD' => [
            'EC$',
            'East Caribbean Dollar',
        ],
        'XCG' => [
            'ICg.',
            'ICaribbean guilder',
        ],
        'XOF' => [
            'F CFA',
            'ICFA Franc yaseWest Africa',
        ],
        'XPF' => [
            'CFPF',
            'I-Franc yaseCFP',
        ],
        'YER' => [
            'YER',
            'I-Rial yaseYemen',
        ],
        'ZAR' => [
            'R',
            'IRandi yaseMzantsi Afrika',
        ],
        'ZMW' => [
            'ZMW',
            'I-Kwacha yaseZambi',
        ],
        'ZWG' => [
            'IZWG',
            'IZimbabwean Gold',
        ],
    ],
];
