<?php

return [
    'Names' => [
        'ADP' => [
            'ADP',
            'Andorrskur peseti',
        ],
        'AED' => [
            'AED',
            'arabískt dírham',
        ],
        'AFN' => [
            'AFN',
            'afgani',
        ],
        'ALL' => [
            'ALL',
            'albanskt lek',
        ],
        'AMD' => [
            'AMD',
            'armenskt dramm',
        ],
        'ANG' => [
            'ANG',
            'hollenskt Antillugyllini',
        ],
        'AOA' => [
            'AOA',
            'angólsk kvansa',
        ],
        'ARA' => [
            'ARA',
            'Argentine Austral',
        ],
        'ARP' => [
            'ARP',
            'Argentískur pesi (1983–1985)',
        ],
        'ARS' => [
            'ARS',
            'argentínskur pesi',
        ],
        'ATS' => [
            'ATS',
            'Austurrískur skildingur',
        ],
        'AUD' => [
            'AUD',
            'ástralskur dalur',
        ],
        'AWG' => [
            'AWG',
            'arúbönsk flórína',
        ],
        'AZN' => [
            'AZN',
            'aserskt manat',
        ],
        'BAM' => [
            'BAM',
            'skiptanlegt Bosníu og Hersegóvínu-mark',
        ],
        'BBD' => [
            'BBD',
            'barbadoskur dalur',
        ],
        'BDT' => [
            'BDT',
            'bangladessk taka',
        ],
        'BEF' => [
            'BEF',
            'Belgískur franki',
        ],
        'BGL' => [
            'BGL',
            'Lef',
        ],
        'BGN' => [
            'BGN',
            'búlgarskt lef',
        ],
        'BHD' => [
            'BHD',
            'bareinskur denari',
        ],
        'BIF' => [
            'BIF',
            'búrúndískur franki',
        ],
        'BMD' => [
            'BMD',
            'Bermúdadalur',
        ],
        'BND' => [
            'BND',
            'brúneiskur dalur',
        ],
        'BOB' => [
            'BOB',
            'bólivíani',
        ],
        'BOP' => [
            'BOP',
            'Bólivískur pesi',
        ],
        'BOV' => [
            'BOV',
            'Bolivian Mvdol',
        ],
        'BRL' => [
            'BRL',
            'brasilískt ríal',
        ],
        'BSD' => [
            'BSD',
            'Bahamadalur',
        ],
        'BTN' => [
            'BTN',
            'bútanskt núltrum',
        ],
        'BUK' => [
            'BUK',
            'Búrmverskt kjat',
        ],
        'BWP' => [
            'BWP',
            'botsvönsk púla',
        ],
        'BYN' => [
            'BYN',
            'hvítrússnesk rúbla',
        ],
        'BYR' => [
            'BYR',
            'hvítrússnesk rúbla (2000–2016)',
        ],
        'BZD' => [
            'BZD',
            'belískur dalur',
        ],
        'CAD' => [
            'CAD',
            'Kanadadalur',
        ],
        'CDF' => [
            'CDF',
            'kongóskur franki',
        ],
        'CHF' => [
            'CHF',
            'svissneskur franki',
        ],
        'CLF' => [
            'CLF',
            'Chilean Unidades de Fomento',
        ],
        'CLP' => [
            'CLP',
            'síleskur pesi',
        ],
        'CNH' => [
            'CNH',
            'kínverskt júan (utan heimalands)',
        ],
        'CNY' => [
            'CN¥',
            'kínverskt júan',
        ],
        'COP' => [
            'COP',
            'kólumbískur pesi',
        ],
        'CRC' => [
            'CRC',
            'kostarískt kólon',
        ],
        'CSK' => [
            'CSK',
            'Tékknesk króna, eldri',
        ],
        'CUC' => [
            'CUC',
            'kúbverskur skiptanlegur pesi',
        ],
        'CUP' => [
            'CUP',
            'kúbverskur pesi',
        ],
        'CVE' => [
            'CVE',
            'grænhöfðeyskur skúti',
        ],
        'CYP' => [
            'CYP',
            'Kýpverskt pund',
        ],
        'CZK' => [
            'CZK',
            'tékknesk króna',
        ],
        'DDM' => [
            'DDM',
            'Austurþýskt mark',
        ],
        'DEM' => [
            'DEM',
            'Þýskt mark',
        ],
        'DJF' => [
            'DJF',
            'djíbútískur franki',
        ],
        'DKK' => [
            'DKK',
            'dönsk króna',
        ],
        'DOP' => [
            'DOP',
            'dóminískur pesi',
        ],
        'DZD' => [
            'DZD',
            'alsírskur denari',
        ],
        'ECS' => [
            'ECS',
            'Ecuador Sucre',
        ],
        'EEK' => [
            'EEK',
            'Eistnesk króna',
        ],
        'EGP' => [
            'EGP',
            'egypskt pund',
        ],
        'ERN' => [
            'ERN',
            'erítresk nakfa',
        ],
        'ESP' => [
            'ESP',
            'Spænskur peseti',
        ],
        'ETB' => [
            'ETB',
            'eþíópískt birr',
        ],
        'EUR' => [
            'EUR',
            'evra',
        ],
        'FIM' => [
            'FIM',
            'Finnskt mark',
        ],
        'FJD' => [
            'FJD',
            'fídjískur dalur',
        ],
        'FKP' => [
            'FKP',
            'falklenskt pund',
        ],
        'FRF' => [
            'FRF',
            'Franskur franki',
        ],
        'GBP' => [
            'GBP',
            'sterlingspund',
        ],
        'GEL' => [
            'GEL',
            'georgískur lari',
        ],
        'GHS' => [
            'GHS',
            'ganverskur sedi',
        ],
        'GIP' => [
            'GIP',
            'Gíbraltarspund',
        ],
        'GMD' => [
            'GMD',
            'gambískur dalasi',
        ],
        'GNF' => [
            'GNF',
            'Gíneufranki',
        ],
        'GRD' => [
            'GRD',
            'Drakma',
        ],
        'GTQ' => [
            'GTQ',
            'gvatemalskt kvesal',
        ],
        'GWE' => [
            'GWE',
            'Portúgalskur, gíneskur skúti',
        ],
        'GYD' => [
            'GYD',
            'gvæjanskur dalur',
        ],
        'HKD' => [
            'HK$',
            'Hong Kong-dalur',
        ],
        'HNL' => [
            'HNL',
            'hondúrsk lempíra',
        ],
        'HRK' => [
            'HRK',
            'króatísk kúna',
        ],
        'HTG' => [
            'HTG',
            'haítískur gúrdi',
        ],
        'HUF' => [
            'HUF',
            'ungversk fórinta',
        ],
        'IDR' => [
            'IDR',
            'indónesísk rúpía',
        ],
        'IEP' => [
            'IEP',
            'Írskt pund',
        ],
        'ILP' => [
            'ILP',
            'Ísraelskt pund',
        ],
        'ILS' => [
            '₪',
            'nýr ísraelskur sikill',
        ],
        'INR' => [
            'INR',
            'indversk rúpía',
        ],
        'IQD' => [
            'IQD',
            'írakskur denari',
        ],
        'IRR' => [
            'IRR',
            'íranskt ríal',
        ],
        'ISK' => [
            'kr.',
            'íslensk króna',
        ],
        'ITL' => [
            'ITL',
            'Ítölsk líra',
        ],
        'JMD' => [
            'JMD',
            'jamaískur dalur',
        ],
        'JOD' => [
            'JOD',
            'jórdanskur denari',
        ],
        'JPY' => [
            'JP¥',
            'japanskt jen',
        ],
        'KES' => [
            'KES',
            'kenískur skildingur',
        ],
        'KGS' => [
            'KGS',
            'kirgiskt som',
        ],
        'KHR' => [
            'KHR',
            'kambódískt ríal',
        ],
        'KMF' => [
            'KMF',
            'kómoreyskur franki',
        ],
        'KPW' => [
            'KPW',
            'norðurkóreskt vonn',
        ],
        'KRW' => [
            'KRW',
            'suðurkóreskt vonn',
        ],
        'KWD' => [
            'KWD',
            'kúveiskur denari',
        ],
        'KYD' => [
            'KYD',
            'caymaneyskur dalur',
        ],
        'KZT' => [
            'KZT',
            'kasakst tengi',
        ],
        'LAK' => [
            'LAK',
            'laoskt kip',
        ],
        'LBP' => [
            'LBP',
            'líbanskt pund',
        ],
        'LKR' => [
            'LKR',
            'srílönsk rúpía',
        ],
        'LRD' => [
            'LRD',
            'líberískur dalur',
        ],
        'LSL' => [
            'LSL',
            'lesótóskur lóti',
        ],
        'LTL' => [
            'LTL',
            'Litháískt lít',
        ],
        'LTT' => [
            'LTT',
            'Lithuanian Talonas',
        ],
        'LUF' => [
            'LUF',
            'Lúxemborgarfranki',
        ],
        'LVL' => [
            'LVL',
            'Lettneskt lat',
        ],
        'LVR' => [
            'LVR',
            'Lettnesk rúbla',
        ],
        'LYD' => [
            'LYD',
            'líbískur denari',
        ],
        'MAD' => [
            'MAD',
            'marokkóskt dírham',
        ],
        'MAF' => [
            'MAF',
            'Marokkóskur franki',
        ],
        'MDL' => [
            'MDL',
            'moldavískt lei',
        ],
        'MGA' => [
            'MGA',
            'Madagaskararjari',
        ],
        'MGF' => [
            'MGF',
            'Madagaskur franki',
        ],
        'MKD' => [
            'MKD',
            'makedónskur denari',
        ],
        'MLF' => [
            'MLF',
            'Malískur franki',
        ],
        'MMK' => [
            'MMK',
            'mjanmarskt kjat',
        ],
        'MNT' => [
            'MNT',
            'mongólskur túríkur',
        ],
        'MOP' => [
            'MOP',
            'makaósk pataka',
        ],
        'MRO' => [
            'MRO',
            'márítönsk úgía (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'márítönsk úgía',
        ],
        'MTL' => [
            'MTL',
            'Meltnesk líra',
        ],
        'MTP' => [
            'MTP',
            'Maltneskt pund',
        ],
        'MUR' => [
            'MUR',
            'máritísk rúpía',
        ],
        'MVR' => [
            'MVR',
            'maldíveysk rúpía',
        ],
        'MWK' => [
            'MWK',
            'malavísk kvaka',
        ],
        'MXN' => [
            'MXN',
            'mexíkóskur pesi',
        ],
        'MXP' => [
            'MXP',
            'Mexíkóskur silfurpesi (1861–1992)',
        ],
        'MXV' => [
            'MXV',
            'Mexíkóskur pesi, UDI',
        ],
        'MYR' => [
            'MYR',
            'malasískt ringit',
        ],
        'MZE' => [
            'MZE',
            'Mósambískur skúti',
        ],
        'MZN' => [
            'MZN',
            'mósambískt metikal',
        ],
        'NAD' => [
            'NAD',
            'namibískur dalur',
        ],
        'NGN' => [
            'NGN',
            'nígerísk næra',
        ],
        'NIC' => [
            'NIC',
            'Níkarögsk kordóva (1988–1991)',
        ],
        'NIO' => [
            'NIO',
            'níkarögsk kordóva',
        ],
        'NLG' => [
            'NLG',
            'Hollenskt gyllini',
        ],
        'NOK' => [
            'NOK',
            'norsk króna',
        ],
        'NPR' => [
            'NPR',
            'nepölsk rúpía',
        ],
        'NZD' => [
            'NZD',
            'nýsjálenskur dalur',
        ],
        'OMR' => [
            'OMR',
            'ómanskt ríal',
        ],
        'PAB' => [
            'PAB',
            'balbói',
        ],
        'PEN' => [
            'PEN',
            'perúskt sól',
        ],
        'PGK' => [
            'PGK',
            'papúsk kína',
        ],
        'PHP' => [
            'PHP',
            'filippseyskur pesi',
        ],
        'PKR' => [
            'PKR',
            'pakistönsk rúpía',
        ],
        'PLN' => [
            'PLN',
            'pólskt slot',
        ],
        'PLZ' => [
            'PLZ',
            'Slot',
        ],
        'PTE' => [
            'PTE',
            'Portúgalskur skúti',
        ],
        'PYG' => [
            'PYG',
            'paragvæskt gvaraní',
        ],
        'QAR' => [
            'QAR',
            'katarskt ríal',
        ],
        'ROL' => [
            'ROL',
            'Rúmenskt lei (1952–2006)',
        ],
        'RON' => [
            'RON',
            'rúmenskt lei',
        ],
        'RSD' => [
            'RSD',
            'serbneskur denari',
        ],
        'RUB' => [
            'RUB',
            'rússnesk rúbla',
        ],
        'RUR' => [
            'RUR',
            'Rússnesk rúbla (1991–1998)',
        ],
        'RWF' => [
            'RWF',
            'rúandskur franki',
        ],
        'SAR' => [
            'SAR',
            'sádíarabískt ríal',
        ],
        'SBD' => [
            'SBD',
            'salómonseyskur dalur',
        ],
        'SCR' => [
            'SCR',
            'Seychellesrúpía',
        ],
        'SDD' => [
            'SDD',
            'Súdanskur denari',
        ],
        'SDG' => [
            'SDG',
            'súdanskt pund',
        ],
        'SDP' => [
            'SDP',
            'Súdanskt pund (1957–1998)',
        ],
        'SEK' => [
            'SEK',
            'sænsk króna',
        ],
        'SGD' => [
            'SGD',
            'singapúrskur dalur',
        ],
        'SHP' => [
            'SHP',
            'helenskt pund',
        ],
        'SIT' => [
            'SIT',
            'Slóvenskur dalur',
        ],
        'SKK' => [
            'SKK',
            'Slóvakísk króna',
        ],
        'SLE' => [
            'SLE',
            'síerraleónsk ljóna',
        ],
        'SLL' => [
            'SLL',
            'síerraleónsk ljóna (1964—2022)',
        ],
        'SOS' => [
            'SOS',
            'sómalískur skildingur',
        ],
        'SRD' => [
            'SRD',
            'Súrínamdalur',
        ],
        'SRG' => [
            'SRG',
            'Suriname Guilder',
        ],
        'SSP' => [
            'SSP',
            'suðursúdanskt pund',
        ],
        'STD' => [
            'STD',
            'Saó Tóme og Prinsípe-dóbra (1977–2017)',
        ],
        'STN' => [
            'STN',
            'Saó Tóme og Prinsípe-dóbra',
        ],
        'SUR' => [
            'SUR',
            'Soviet Rouble',
        ],
        'SVC' => [
            'SVC',
            'El Salvador Colon',
        ],
        'SYP' => [
            'SYP',
            'sýrlenskt pund',
        ],
        'SZL' => [
            'SZL',
            'svasílenskur lílangeni',
        ],
        'THB' => [
            'THB',
            'taílenskt bat',
        ],
        'TJR' => [
            'TJR',
            'Tadsjiksk rúbla',
        ],
        'TJS' => [
            'TJS',
            'tadsjikskur sómóni',
        ],
        'TMM' => [
            'TMM',
            'Túrkmenskt manat (1993–2009)',
        ],
        'TMT' => [
            'TMT',
            'túrkmenskt manat',
        ],
        'TND' => [
            'TND',
            'túniskur denari',
        ],
        'TOP' => [
            'TOP',
            'Tongapanga',
        ],
        'TPE' => [
            'TPE',
            'Tímorskur skúti',
        ],
        'TRL' => [
            'TRL',
            'Tyrknesk líra (1922–2005)',
        ],
        'TRY' => [
            'TRY',
            'tyrknesk líra',
        ],
        'TTD' => [
            'TTD',
            'Trínidad og Tóbagó-dalur',
        ],
        'TWD' => [
            'TWD',
            'taívanskur dalur',
        ],
        'TZS' => [
            'TZS',
            'tansanískur skildingur',
        ],
        'UAH' => [
            'UAH',
            'úkraínsk hrinja',
        ],
        'UAK' => [
            'UAK',
            'Ukrainian Karbovanetz',
        ],
        'UGX' => [
            'UGX',
            'úgandskur skildingur',
        ],
        'USD' => [
            'USD',
            'Bandaríkjadalur',
        ],
        'USN' => [
            'USN',
            'Bandaríkjadalur (næsta dag)',
        ],
        'USS' => [
            'USS',
            'Bandaríkjadalur (sama dag)',
        ],
        'UYU' => [
            'UYU',
            'úrúgvæskur pesi',
        ],
        'UZS' => [
            'UZS',
            'úsbekskt súm',
        ],
        'VEB' => [
            'VEB',
            'Bolívar í Venesúela (1871–2008)',
        ],
        'VEF' => [
            'VEF',
            'venesúelskur bólívari (2008–2018)',
        ],
        'VES' => [
            'VES',
            'venesúelskur bólívari',
        ],
        'VND' => [
            'VND',
            'víetnamskt dong',
        ],
        'VUV' => [
            'VUV',
            'vanúatúskt vatú',
        ],
        'WST' => [
            'WST',
            'Samóatala',
        ],
        'XAF' => [
            'FCFA',
            'miðafrískur franki',
        ],
        'XCD' => [
            'EC$',
            'austurkarabískur dalur',
        ],
        'XCG' => [
            'XCG',
            'Karíbahafsgyllini',
        ],
        'XFO' => [
            'XFO',
            'Franskur gullfranki',
        ],
        'XFU' => [
            'XFU',
            'Franskur franki, UIC',
        ],
        'XOF' => [
            'F CFA',
            'vesturafrískur franki',
        ],
        'XPF' => [
            'CFPF',
            'pólinesískur franki',
        ],
        'YDD' => [
            'YDD',
            'Jemenskur denari',
        ],
        'YER' => [
            'YER',
            'jemenskt ríal',
        ],
        'YUM' => [
            'YUM',
            'Júgóslavneskur denari',
        ],
        'ZAL' => [
            'ZAL',
            'Rand (viðskipta)',
        ],
        'ZAR' => [
            'ZAR',
            'suðurafrískt rand',
        ],
        'ZMK' => [
            'ZMK',
            'Zambian Kwacha (1968–2012)',
        ],
        'ZMW' => [
            'ZMW',
            'sambísk kvaka',
        ],
        'ZWD' => [
            'ZWD',
            'Simbabveskur dalur',
        ],
        'ZWG' => [
            'ZWG',
            'simbabveskt gold',
        ],
    ],
];
