<?php

return [
    'Names' => [
        'Africa/Abidjan' => 'Grinwiç ortaça wagty (Abijan)',
        'Africa/Accra' => 'Grinwiç ortaça wagty (Akkra)',
        'Africa/Addis_Ababa' => 'Gündogar Afrika wagty (Addis-Abeba)',
        'Africa/Algiers' => 'Merkezi Ýewropa wagty (Alžir)',
        'Africa/Asmera' => 'Gündogar Afrika wagty (Asmara)',
        'Africa/Asmara' => 'Gündogar Afrika wagty (Asmara)',
        'Africa/Bamako' => 'Grinwiç ortaça wagty (Bamako)',
        'Africa/Bangui' => 'Günbatar Afrika wagty (Bangi)',
        'Africa/Banjul' => 'Grinwiç ortaça wagty (Banjul)',
        'Africa/Bissau' => 'Grinwiç ortaça wagty (Bisau)',
        'Africa/Blantyre' => 'Merkezi Afrika wagty (Blantaýr)',
        'Africa/Brazzaville' => 'Günbatar Afrika wagty (Brazzawil)',
        'Africa/Bujumbura' => 'Merkezi Afrika wagty (Bujumbura)',
        'Africa/Cairo' => 'Gündogar Ýewropa wagty (Kair)',
        'Africa/Casablanca' => 'Günbatar Ýewropa wagty (Kasablanka)',
        'Africa/Ceuta' => 'Merkezi Ýewropa wagty (Seuta)',
        'Africa/Conakry' => 'Grinwiç ortaça wagty (Konakri)',
        'Africa/Dakar' => 'Grinwiç ortaça wagty (Dakar)',
        'Africa/Dar_es_Salaam' => 'Gündogar Afrika wagty (Dar-es-Salam)',
        'Africa/Djibouti' => 'Gündogar Afrika wagty (Jibuti)',
        'Africa/Douala' => 'Günbatar Afrika wagty (Duala)',
        'Africa/El_Aaiun' => 'Günbatar Ýewropa wagty (El-Aýun)',
        'Africa/Freetown' => 'Grinwiç ortaça wagty (Fritaun)',
        'Africa/Gaborone' => 'Merkezi Afrika wagty (Gaborone)',
        'Africa/Harare' => 'Merkezi Afrika wagty (Harare)',
        'Africa/Johannesburg' => 'Günorta Afrika standart wagty (Ýohannesburg)',
        'Africa/Juba' => 'Merkezi Afrika wagty (Juba)',
        'Africa/Kampala' => 'Gündogar Afrika wagty (Kampala)',
        'Africa/Khartoum' => 'Merkezi Afrika wagty (Hartum)',
        'Africa/Kigali' => 'Merkezi Afrika wagty (Kigali)',
        'Africa/Kinshasa' => 'Günbatar Afrika wagty (Kinşasa)',
        'Africa/Lagos' => 'Günbatar Afrika wagty (Lagos)',
        'Africa/Libreville' => 'Günbatar Afrika wagty (Librewil)',
        'Africa/Lome' => 'Grinwiç ortaça wagty (Lomé)',
        'Africa/Luanda' => 'Günbatar Afrika wagty (Luanda)',
        'Africa/Lubumbashi' => 'Merkezi Afrika wagty (Lubumbaşi)',
        'Africa/Lusaka' => 'Merkezi Afrika wagty (Lusaka)',
        'Africa/Malabo' => 'Günbatar Afrika wagty (Malabo)',
        'Africa/Maputo' => 'Merkezi Afrika wagty (Maputo)',
        'Africa/Maseru' => 'Günorta Afrika standart wagty (Maseru)',
        'Africa/Mbabane' => 'Günorta Afrika standart wagty (Mbabane)',
        'Africa/Mogadishu' => 'Gündogar Afrika wagty (Mogadişo)',
        'Africa/Monrovia' => 'Grinwiç ortaça wagty (Monrowiýa)',
        'Africa/Nairobi' => 'Gündogar Afrika wagty (Naýrobi)',
        'Africa/Ndjamena' => 'Günbatar Afrika wagty (Jamena)',
        'Africa/Niamey' => 'Günbatar Afrika wagty (Niameý)',
        'Africa/Nouakchott' => 'Grinwiç ortaça wagty (Nuakşot)',
        'Africa/Ouagadougou' => 'Grinwiç ortaça wagty (Uagadugu)',
        'Africa/Porto-Novo' => 'Günbatar Afrika wagty (Porto-Nowo)',
        'Africa/Sao_Tome' => 'Grinwiç ortaça wagty (San-Tome)',
        'Africa/Tripoli' => 'Gündogar Ýewropa wagty (Tripoli)',
        'Africa/Tunis' => 'Merkezi Ýewropa wagty (Tunis)',
        'Africa/Windhoek' => 'Merkezi Afrika wagty (Windhuk)',
        'America/Adak' => 'Gawaý-Aleut wagty (Adak adasy)',
        'America/Anchorage' => 'Alýaska wagty (Ankoridž)',
        'America/Anguilla' => 'Atlantik wagty (Angilýa)',
        'America/Antigua' => 'Atlantik wagty (Antigua)',
        'America/Araguaina' => 'Braziliýa wagty (Araguaýna)',
        'America/Argentina/La_Rioja' => 'Argentina wagty (La-Rioha)',
        'America/Argentina/Rio_Gallegos' => 'Argentina wagty (Rio-Galegos)',
        'America/Argentina/Salta' => 'Argentina wagty (Salta)',
        'America/Argentina/San_Juan' => 'Argentina wagty (San-Huan)',
        'America/Argentina/San_Luis' => 'Argentina wagty (San-Luis)',
        'America/Argentina/Tucuman' => 'Argentina wagty (Tukuman)',
        'America/Argentina/Ushuaia' => 'Argentina wagty (Uşuaýa)',
        'America/Aruba' => 'Atlantik wagty (Aruba)',
        'America/Asuncion' => 'Paragwaý wagty (Asunsýon)',
        'America/Bahia' => 'Braziliýa wagty (Baiýa)',
        'America/Bahia_Banderas' => 'Merkezi Amerika (Baiýa-de-Banderas)',
        'America/Barbados' => 'Atlantik wagty (Barbados)',
        'America/Belem' => 'Braziliýa wagty (Belen)',
        'America/Belize' => 'Merkezi Amerika (Beliz)',
        'America/Blanc-Sablon' => 'Atlantik wagty (Blank-Sablon)',
        'America/Boa_Vista' => 'Amazon wagty (Boa-Wista)',
        'America/Bogota' => 'Kolumbiýa wagty (Bogotá)',
        'America/Boise' => 'Demirgazyk Amerika dag wagty (Boýse)',
        'America/Buenos_Aires' => 'Argentina wagty (Buenos-Aýres)',
        'America/Argentina/Buenos_Aires' => 'Argentina wagty (Buenos-Aýres)',
        'America/Cambridge_Bay' => 'Demirgazyk Amerika dag wagty (Kembrij-Beý)',
        'America/Campo_Grande' => 'Amazon wagty (Kampu-Grandi)',
        'America/Cancun' => 'Demirgazyk Amerika gündogar wagty (Kankun)',
        'America/Caracas' => 'Wenesuela wagty (Karakas)',
        'America/Catamarca' => 'Argentina wagty (Katamarka)',
        'America/Argentina/Catamarca' => 'Argentina wagty (Katamarka)',
        'America/Cayenne' => 'Fransuz Gwianasy wagty (Kaýenna)',
        'America/Cayman' => 'Demirgazyk Amerika gündogar wagty (Kaýman adalary)',
        'America/Chicago' => 'Merkezi Amerika (Çikago)',
        'America/Chihuahua' => 'Merkezi Amerika (Çihuahua)',
        'America/Ciudad_Juarez' => 'Demirgazyk Amerika dag wagty (Sýudad-Huares)',
        'America/Coral_Harbour' => 'Demirgazyk Amerika gündogar wagty (Atikokan)',
        'America/Atikokan' => 'Demirgazyk Amerika gündogar wagty (Atikokan)',
        'America/Cordoba' => 'Argentina wagty (Kordowa)',
        'America/Argentina/Cordoba' => 'Argentina wagty (Kordowa)',
        'America/Costa_Rica' => 'Merkezi Amerika (Kosta-Rika)',
        'America/Coyhaique' => 'Çili wagty (Koýýaik)',
        'America/Creston' => 'Demirgazyk Amerika dag wagty (Kreston)',
        'America/Cuiaba' => 'Amazon wagty (Kuýaba)',
        'America/Curacao' => 'Atlantik wagty (Kýurasao)',
        'America/Danmarkshavn' => 'Grinwiç ortaça wagty (Denmarkshawn)',
        'America/Dawson' => 'Ýukon wagty (Douson)',
        'America/Dawson_Creek' => 'Demirgazyk Amerika dag wagty (Douson-Krik)',
        'America/Denver' => 'Demirgazyk Amerika dag wagty (Denwer)',
        'America/Detroit' => 'Demirgazyk Amerika gündogar wagty (Detroýt)',
        'America/Dominica' => 'Atlantik wagty (Dominika)',
        'America/Edmonton' => 'Demirgazyk Amerika dag wagty (Edmonton)',
        'America/Eirunepe' => 'Akri wagty (Eýrunepe)',
        'America/El_Salvador' => 'Merkezi Amerika (Salwador)',
        'America/Fort_Nelson' => 'Demirgazyk Amerika dag wagty (Fort Nelson)',
        'America/Fortaleza' => 'Braziliýa wagty (Fortaleza)',
        'America/Glace_Bay' => 'Atlantik wagty (Gleýs-Beý)',
        'America/Godthab' => 'Grenlandiýa wagty (Nuuk)',
        'America/Nuuk' => 'Grenlandiýa wagty (Nuuk)',
        'America/Goose_Bay' => 'Atlantik wagty (Gus-Beý)',
        'America/Grand_Turk' => 'Demirgazyk Amerika gündogar wagty (Grand-Terk)',
        'America/Grenada' => 'Atlantik wagty (Grenada)',
        'America/Guadeloupe' => 'Atlantik wagty (Gwadelupa)',
        'America/Guatemala' => 'Merkezi Amerika (Gwatemala)',
        'America/Guayaquil' => 'Ekwador wagty (Guýakil)',
        'America/Guyana' => 'Gaýana wagty',
        'America/Halifax' => 'Atlantik wagty (Galifaks)',
        'America/Havana' => 'Kuba wagty (Gawana)',
        'America/Hermosillo' => 'Meksikan Ýuwaş umman wagty (Ermosilo)',
        'America/Indiana/Knox' => 'Merkezi Amerika (Noks, Indiana)',
        'America/Indiana/Marengo' => 'Demirgazyk Amerika gündogar wagty (Marengo, Indiana)',
        'America/Indiana/Petersburg' => 'Demirgazyk Amerika gündogar wagty (Petersburg, Indiana)',
        'America/Indiana/Tell_City' => 'Merkezi Amerika (Tell-Siti, Indiana)',
        'America/Indiana/Vevay' => 'Demirgazyk Amerika gündogar wagty (Wiweý, Indiana)',
        'America/Indiana/Vincennes' => 'Demirgazyk Amerika gündogar wagty (Winsens, Indiana)',
        'America/Indiana/Winamac' => 'Demirgazyk Amerika gündogar wagty (Winamak, Indiana)',
        'America/Indianapolis' => 'Demirgazyk Amerika gündogar wagty (Indianapolis)',
        'America/Indiana/Indianapolis' => 'Demirgazyk Amerika gündogar wagty (Indianapolis)',
        'America/Inuvik' => 'Demirgazyk Amerika dag wagty (Inuwik)',
        'America/Iqaluit' => 'Demirgazyk Amerika gündogar wagty (Ikaluit)',
        'America/Jamaica' => 'Demirgazyk Amerika gündogar wagty (Ýamaýka)',
        'America/Jujuy' => 'Argentina wagty (Žužuý)',
        'America/Argentina/Jujuy' => 'Argentina wagty (Žužuý)',
        'America/Juneau' => 'Alýaska wagty (Džuno)',
        'America/Kentucky/Monticello' => 'Demirgazyk Amerika gündogar wagty (Montisello, Kentuki)',
        'America/Kralendijk' => 'Atlantik wagty (Kralendeýk)',
        'America/La_Paz' => 'Boliwiýa wagty (La-Pas)',
        'America/Lima' => 'Peru wagty (Lima)',
        'America/Los_Angeles' => 'Demirgazyk Amerika Ýuwaş umman wagty (Los-Anjeles)',
        'America/Louisville' => 'Demirgazyk Amerika gündogar wagty (Luiswill)',
        'America/Kentucky/Louisville' => 'Demirgazyk Amerika gündogar wagty (Luiswill)',
        'America/Lower_Princes' => 'Atlantik wagty (Lower-Prinses-Kuorter)',
        'America/Maceio' => 'Braziliýa wagty (Maseýo)',
        'America/Managua' => 'Merkezi Amerika (Managua)',
        'America/Manaus' => 'Amazon wagty (Manaus)',
        'America/Marigot' => 'Atlantik wagty (Marigo)',
        'America/Martinique' => 'Atlantik wagty (Martinika)',
        'America/Matamoros' => 'Merkezi Amerika (Matamoros)',
        'America/Mazatlan' => 'Meksikan Ýuwaş umman wagty (Mazatlán)',
        'America/Mendoza' => 'Argentina wagty (Mendosa)',
        'America/Argentina/Mendoza' => 'Argentina wagty (Mendosa)',
        'America/Menominee' => 'Merkezi Amerika (Menomini)',
        'America/Merida' => 'Merkezi Amerika (Merida)',
        'America/Metlakatla' => 'Alýaska wagty (Metlakatla)',
        'America/Mexico_City' => 'Merkezi Amerika (Mehiko)',
        'America/Miquelon' => 'Sen-Pýer we Mikelon',
        'America/Moncton' => 'Atlantik wagty (Monkton)',
        'America/Monterrey' => 'Merkezi Amerika (Monterreý)',
        'America/Montevideo' => 'Urugwaý wagty (Montewideo)',
        'America/Montserrat' => 'Atlantik wagty (Monserrat)',
        'America/Nassau' => 'Demirgazyk Amerika gündogar wagty (Nassau)',
        'America/New_York' => 'Demirgazyk Amerika gündogar wagty (Nýu-Ýork)',
        'America/Nome' => 'Alýaska wagty (Nom)',
        'America/Noronha' => 'Fernandu-di-Noronýa wagty (Fernando de Noronha)',
        'America/North_Dakota/Beulah' => 'Merkezi Amerika (Boýla, Demirgazyk Dakota)',
        'America/North_Dakota/Center' => 'Merkezi Amerika (Sentr, Demirgazyk Dakota)',
        'America/North_Dakota/New_Salem' => 'Merkezi Amerika (Nýu-Salem, Demirgazyk Dakota)',
        'America/Ojinaga' => 'Merkezi Amerika (Ohinaga)',
        'America/Panama' => 'Demirgazyk Amerika gündogar wagty (Panama)',
        'America/Paramaribo' => 'Surinam wagty (Paramaribo)',
        'America/Phoenix' => 'Demirgazyk Amerika dag wagty (Feniks)',
        'America/Port-au-Prince' => 'Demirgazyk Amerika gündogar wagty (Port-o-Prens)',
        'America/Port_of_Spain' => 'Atlantik wagty (Port-of-Speýn)',
        'America/Porto_Velho' => 'Amazon wagty (Portu-Welýu)',
        'America/Puerto_Rico' => 'Atlantik wagty (Puerto-Riko)',
        'America/Punta_Arenas' => 'Çili wagty (Punta-Arenas)',
        'America/Rankin_Inlet' => 'Merkezi Amerika (Rankin-Inlet)',
        'America/Recife' => 'Braziliýa wagty (Resifi)',
        'America/Regina' => 'Merkezi Amerika (Rejaýna)',
        'America/Resolute' => 'Merkezi Amerika (Rezolýut)',
        'America/Rio_Branco' => 'Akri wagty (Riu-Branku)',
        'America/Santarem' => 'Braziliýa wagty (Santarém)',
        'America/Santiago' => 'Çili wagty (Santýago)',
        'America/Santo_Domingo' => 'Atlantik wagty (Santo-Domingo)',
        'America/Sao_Paulo' => 'Braziliýa wagty (San-Paulu)',
        'America/Scoresbysund' => 'Grenlandiýa wagty (Illokkortoormiut)',
        'America/Sitka' => 'Alýaska wagty (Sitka)',
        'America/St_Barthelemy' => 'Atlantik wagty (Sen-Bartelemi)',
        'America/St_Johns' => 'Nýufaundlend wagty (Sent-Jons)',
        'America/St_Kitts' => 'Atlantik wagty (Sent-Kits)',
        'America/St_Lucia' => 'Atlantik wagty (Sent-Lýusiýa)',
        'America/St_Thomas' => 'Atlantik wagty (Sent-Tomas)',
        'America/St_Vincent' => 'Atlantik wagty (Sent-Winsent)',
        'America/Swift_Current' => 'Merkezi Amerika (Swift-Karent)',
        'America/Tegucigalpa' => 'Merkezi Amerika (Tegusigalpa)',
        'America/Thule' => 'Atlantik wagty (Tule)',
        'America/Tijuana' => 'Demirgazyk Amerika Ýuwaş umman wagty (Tihuana)',
        'America/Toronto' => 'Demirgazyk Amerika gündogar wagty (Toronto)',
        'America/Tortola' => 'Atlantik wagty (Tortola)',
        'America/Vancouver' => 'Demirgazyk Amerika Ýuwaş umman wagty (Wankuwer)',
        'America/Whitehorse' => 'Ýukon wagty (Waýthors)',
        'America/Winnipeg' => 'Merkezi Amerika (Winnipeg)',
        'America/Yakutat' => 'Alýaska wagty (Ýakutat)',
        'Antarctica/Casey' => 'Günbatar Awstraliýa wagty (Keýsi)',
        'Antarctica/Davis' => 'Deýwis wagty',
        'Antarctica/DumontDUrville' => 'Dýumon-d-Ýurwil wagty',
        'Antarctica/Macquarie' => 'Gündogar Awstraliýa wagty (Makkuori)',
        'Antarctica/Mawson' => 'Mouson wagty',
        'Antarctica/McMurdo' => 'Täze Zelandiýa wagty (Mak-Merdo)',
        'Antarctica/Palmer' => 'Çili wagty (Palmer)',
        'Antarctica/Rothera' => 'Rotera wagty',
        'Antarctica/Syowa' => 'Sýowa wagty',
        'Antarctica/Troll' => 'Grinwiç ortaça wagty (Trol)',
        'Antarctica/Vostok' => 'Wostok wagty',
        'Arctic/Longyearbyen' => 'Merkezi Ýewropa wagty (Longir)',
        'Asia/Aden' => 'Arap ýurtlary wagty (Aden)',
        'Asia/Almaty' => 'Gazagystan wagty (Almaty)',
        'Asia/Amman' => 'Gündogar Ýewropa wagty (Amman)',
        'Asia/Anadyr' => 'Petropavlowsk-Kamçatskiý wagty (Anadyr)',
        'Asia/Aqtau' => 'Gazagystan wagty (Aktau)',
        'Asia/Aqtobe' => 'Gazagystan wagty (Aktobe)',
        'Asia/Ashgabat' => 'Türkmenistan wagty (Aşgabat)',
        'Asia/Atyrau' => 'Gazagystan wagty (Atyrau)',
        'Asia/Baghdad' => 'Arap ýurtlary wagty (Bagdat)',
        'Asia/Bahrain' => 'Arap ýurtlary wagty (Bahreýn)',
        'Asia/Baku' => 'Azerbaýjan wagty (Baku)',
        'Asia/Bangkok' => 'Hindihytaý wagty (Bangkok)',
        'Asia/Barnaul' => 'Krasnoýarsk wagty (Barnaul)',
        'Asia/Beirut' => 'Gündogar Ýewropa wagty (Beýrut)',
        'Asia/Bishkek' => 'Gyrgyzystan wagty (Bişkek)',
        'Asia/Brunei' => 'Bruneý-Darussalam wagty',
        'Asia/Calcutta' => 'Hindistan standart wagty (Kalkutta)',
        'Asia/Kolkata' => 'Hindistan standart wagty (Kalkutta)',
        'Asia/Chita' => 'Ýakutsk wagty (Çita)',
        'Asia/Colombo' => 'Hindistan standart wagty (Kolombo)',
        'Asia/Damascus' => 'Gündogar Ýewropa wagty (Damask)',
        'Asia/Dhaka' => 'Bangladeş wagty (Dakka)',
        'Asia/Dili' => 'Gündogar Timor wagty (Dili)',
        'Asia/Dubai' => 'Pars aýlagy standart wagty (Dubaý)',
        'Asia/Dushanbe' => 'Täjigistan wagty (Duşanbe)',
        'Asia/Famagusta' => 'Gündogar Ýewropa wagty (Famagusta)',
        'Asia/Gaza' => 'Gündogar Ýewropa wagty (Gaza)',
        'Asia/Hebron' => 'Gündogar Ýewropa wagty (Hewron)',
        'Asia/Hong_Kong' => 'Gonkong wagty',
        'Asia/Hovd' => 'Howd wagty',
        'Asia/Irkutsk' => 'Irkutsk wagty',
        'Asia/Jakarta' => 'Günbatar Indoneziýa wagty (Jakarta)',
        'Asia/Jayapura' => 'Gündogar Indoneziýa wagty (Jaýapura)',
        'Asia/Jerusalem' => 'Ysraýyl wagty (Iýerusalim)',
        'Asia/Kabul' => 'Owganystan wagty (Kabul)',
        'Asia/Kamchatka' => 'Petropavlowsk-Kamçatskiý wagty (Kamçatka)',
        'Asia/Karachi' => 'Pakistan wagty (Karaçi)',
        'Asia/Katmandu' => 'Nepal wagty (Katmandu)',
        'Asia/Kathmandu' => 'Nepal wagty (Katmandu)',
        'Asia/Khandyga' => 'Ýakutsk wagty (Handyga)',
        'Asia/Krasnoyarsk' => 'Krasnoýarsk wagty',
        'Asia/Kuala_Lumpur' => 'Malaýziýa wagty (Kuala-Lumpur)',
        'Asia/Kuching' => 'Malaýziýa wagty (Kuçing)',
        'Asia/Kuwait' => 'Arap ýurtlary wagty (Kuweýt)',
        'Asia/Macau' => 'Hytaý wagty (Makao)',
        'Asia/Magadan' => 'Magadan wagty',
        'Asia/Makassar' => 'Merkezi Indoneziýa wagty (Makasar)',
        'Asia/Manila' => 'Filippinler wagty (Manila)',
        'Asia/Muscat' => 'Pars aýlagy standart wagty (Maskat)',
        'Asia/Nicosia' => 'Gündogar Ýewropa wagty (Nikosiýa)',
        'Asia/Novokuznetsk' => 'Krasnoýarsk wagty (Nowokuznesk)',
        'Asia/Novosibirsk' => 'Krasnoýarsk wagty (Nowosibirsk)',
        'Asia/Omsk' => 'Omsk wagty',
        'Asia/Oral' => 'Gazagystan wagty (Oral)',
        'Asia/Phnom_Penh' => 'Hindihytaý wagty (Pnompen)',
        'Asia/Pontianak' => 'Günbatar Indoneziýa wagty (Pontianak)',
        'Asia/Pyongyang' => 'Koreýa wagty (Phenýan)',
        'Asia/Qatar' => 'Arap ýurtlary wagty (Katar)',
        'Asia/Qostanay' => 'Gazagystan wagty (Kostanaý)',
        'Asia/Qyzylorda' => 'Gazagystan wagty (Gyzylorda)',
        'Asia/Rangoon' => 'Mýanma wagty (Ýangon)',
        'Asia/Yangon' => 'Mýanma wagty (Ýangon)',
        'Asia/Riyadh' => 'Arap ýurtlary wagty (Er-Riýad)',
        'Asia/Saigon' => 'Hindihytaý wagty (Hoşimin)',
        'Asia/Ho_Chi_Minh' => 'Hindihytaý wagty (Hoşimin)',
        'Asia/Sakhalin' => 'Magadan wagty (Sahalin)',
        'Asia/Samarkand' => 'Özbegistan wagty (Samarkand)',
        'Asia/Seoul' => 'Koreýa wagty (Seul)',
        'Asia/Shanghai' => 'Hytaý wagty (Şanhaý)',
        'Asia/Singapore' => 'Singapur wagty',
        'Asia/Srednekolymsk' => 'Magadan wagty (Srednekolymsk)',
        'Asia/Taipei' => 'Taýbeý wagty',
        'Asia/Tashkent' => 'Özbegistan wagty (Taşkent)',
        'Asia/Tbilisi' => 'Gruziýa wagty (Tbilisi)',
        'Asia/Tehran' => 'Eýran wagty (Tähran)',
        'Asia/Thimphu' => 'Butan wagty (Timpu)',
        'Asia/Tokyo' => 'Ýaponiýa wagty (Tokio)',
        'Asia/Tomsk' => 'Krasnoýarsk wagty (Tomsk)',
        'Asia/Ulaanbaatar' => 'Ulan-Bator wagty',
        'Asia/Urumqi' => 'Hytaý wagty (Urumçi)',
        'Asia/Ust-Nera' => 'Wladiwostok wagty (Ust-Nera)',
        'Asia/Vientiane' => 'Hindihytaý wagty (Wýentýan)',
        'Asia/Vladivostok' => 'Wladiwostok wagty',
        'Asia/Yakutsk' => 'Ýakutsk wagty',
        'Asia/Yekaterinburg' => 'Ýekaterinburg wagty',
        'Asia/Yerevan' => 'Ermenistan wagty (Ýerewan)',
        'Atlantic/Azores' => 'Azor adalary wagty',
        'Atlantic/Bermuda' => 'Atlantik wagty (Bermuda)',
        'Atlantic/Canary' => 'Günbatar Ýewropa wagty (Kanar adalary)',
        'Atlantic/Cape_Verde' => 'Kabo-Werde wagty',
        'Atlantic/Faeroe' => 'Günbatar Ýewropa wagty (Farer adalary)',
        'Atlantic/Faroe' => 'Günbatar Ýewropa wagty (Farer adalary)',
        'Atlantic/Madeira' => 'Günbatar Ýewropa wagty (Madeýra adalary)',
        'Atlantic/Reykjavik' => 'Grinwiç ortaça wagty (Reýkýawik)',
        'Atlantic/South_Georgia' => 'Günorta Georgiýa wagty',
        'Atlantic/St_Helena' => 'Grinwiç ortaça wagty (Keramatly Ýelena adasy)',
        'Atlantic/Stanley' => 'Folklend adalary wagty (Stenli)',
        'Australia/Adelaide' => 'Merkezi Awstraliýa wagty (Adelaida)',
        'Australia/Brisbane' => 'Gündogar Awstraliýa wagty (Brisben)',
        'Australia/Broken_Hill' => 'Merkezi Awstraliýa wagty (Broken-Hil)',
        'Australia/Darwin' => 'Merkezi Awstraliýa wagty (Darwin)',
        'Australia/Eucla' => 'Merkezi Awstraliýa günbatar wagty (Ýukla)',
        'Australia/Hobart' => 'Gündogar Awstraliýa wagty (Hobart)',
        'Australia/Lindeman' => 'Gündogar Awstraliýa wagty (Lindeman)',
        'Australia/Lord_Howe' => 'Lord-Hau wagty',
        'Australia/Melbourne' => 'Gündogar Awstraliýa wagty (Melburn)',
        'Australia/Perth' => 'Günbatar Awstraliýa wagty (Pert)',
        'Australia/Sydney' => 'Gündogar Awstraliýa wagty (Sidneý)',
        'Etc/GMT' => 'Grinwiç ortaça wagty',
        'Etc/UTC' => 'Utgaşdyrylýan ähliumumy wagt',
        'Europe/Amsterdam' => 'Merkezi Ýewropa wagty (Amsterdam)',
        'Europe/Andorra' => 'Merkezi Ýewropa wagty (Andorra)',
        'Europe/Astrakhan' => 'Samara wagty (Astrahan)',
        'Europe/Athens' => 'Gündogar Ýewropa wagty (Afiny)',
        'Europe/Belgrade' => 'Merkezi Ýewropa wagty (Belgrad)',
        'Europe/Berlin' => 'Merkezi Ýewropa wagty (Berlin)',
        'Europe/Bratislava' => 'Merkezi Ýewropa wagty (Bratislawa)',
        'Europe/Brussels' => 'Merkezi Ýewropa wagty (Brýussel)',
        'Europe/Bucharest' => 'Gündogar Ýewropa wagty (Buharest)',
        'Europe/Budapest' => 'Merkezi Ýewropa wagty (Budapeşt)',
        'Europe/Busingen' => 'Merkezi Ýewropa wagty (Býuzingen)',
        'Europe/Chisinau' => 'Gündogar Ýewropa wagty (Kişinýow)',
        'Europe/Copenhagen' => 'Merkezi Ýewropa wagty (Kopengagen)',
        'Europe/Dublin' => 'Grinwiç ortaça wagty (Dublin)',
        'Europe/Gibraltar' => 'Merkezi Ýewropa wagty (Gibraltar)',
        'Europe/Guernsey' => 'Grinwiç ortaça wagty (Gernsi)',
        'Europe/Helsinki' => 'Gündogar Ýewropa wagty (Helsinki)',
        'Europe/Isle_of_Man' => 'Grinwiç ortaça wagty (Men adasy)',
        'Europe/Istanbul' => 'Türkiýe wagty (Stambul)',
        'Europe/Jersey' => 'Grinwiç ortaça wagty (Jersi)',
        'Europe/Kaliningrad' => 'Gündogar Ýewropa wagty (Kaliningrad)',
        'Europe/Kiev' => 'Gündogar Ýewropa wagty (Kiýew)',
        'Europe/Kyiv' => 'Gündogar Ýewropa wagty (Kiýew)',
        'Europe/Kirov' => 'Moskwa wagty (Kirow)',
        'Europe/Lisbon' => 'Günbatar Ýewropa wagty (Lissabon)',
        'Europe/Ljubljana' => 'Merkezi Ýewropa wagty (Lýublýana)',
        'Europe/London' => 'Grinwiç ortaça wagty (London)',
        'Europe/Luxembourg' => 'Merkezi Ýewropa wagty (Lýuksemburg)',
        'Europe/Madrid' => 'Merkezi Ýewropa wagty (Madrid)',
        'Europe/Malta' => 'Merkezi Ýewropa wagty (Malta)',
        'Europe/Mariehamn' => 'Gündogar Ýewropa wagty (Mariýehamn)',
        'Europe/Minsk' => 'Moskwa wagty (Minsk)',
        'Europe/Monaco' => 'Merkezi Ýewropa wagty (Monako)',
        'Europe/Moscow' => 'Moskwa wagty',
        'Europe/Oslo' => 'Merkezi Ýewropa wagty (Oslo)',
        'Europe/Paris' => 'Merkezi Ýewropa wagty (Pariž)',
        'Europe/Podgorica' => 'Merkezi Ýewropa wagty (Podgorisa)',
        'Europe/Prague' => 'Merkezi Ýewropa wagty (Praga)',
        'Europe/Riga' => 'Gündogar Ýewropa wagty (Riga)',
        'Europe/Rome' => 'Merkezi Ýewropa wagty (Rim)',
        'Europe/Samara' => 'Samara wagty',
        'Europe/San_Marino' => 'Merkezi Ýewropa wagty (San-Marino)',
        'Europe/Sarajevo' => 'Merkezi Ýewropa wagty (Saraýewo)',
        'Europe/Saratov' => 'Samara wagty (Saratow)',
        'Europe/Simferopol' => 'Moskwa wagty (Simferopol)',
        'Europe/Skopje' => 'Merkezi Ýewropa wagty (Skopýe)',
        'Europe/Sofia' => 'Gündogar Ýewropa wagty (Sofiýa)',
        'Europe/Stockholm' => 'Merkezi Ýewropa wagty (Stokgolm)',
        'Europe/Tallinn' => 'Gündogar Ýewropa wagty (Tallin)',
        'Europe/Tirane' => 'Merkezi Ýewropa wagty (Tirana)',
        'Europe/Ulyanovsk' => 'Samara wagty (Ulýanowsk)',
        'Europe/Vaduz' => 'Merkezi Ýewropa wagty (Waduz)',
        'Europe/Vatican' => 'Merkezi Ýewropa wagty (Watikan)',
        'Europe/Vienna' => 'Merkezi Ýewropa wagty (Wena)',
        'Europe/Vilnius' => 'Gündogar Ýewropa wagty (Wilnýus)',
        'Europe/Volgograd' => 'Moskwa wagty (Wolgograd)',
        'Europe/Warsaw' => 'Merkezi Ýewropa wagty (Warşawa)',
        'Europe/Zagreb' => 'Merkezi Ýewropa wagty (Zagreb)',
        'Europe/Zurich' => 'Merkezi Ýewropa wagty (Sýurih)',
        'Indian/Antananarivo' => 'Gündogar Afrika wagty (Antananariwu)',
        'Indian/Chagos' => 'Hindi ummany wagty (Çagos)',
        'Indian/Christmas' => 'Roždestwo adasy wagty',
        'Indian/Cocos' => 'Kokos adalary wagty',
        'Indian/Comoro' => 'Gündogar Afrika wagty (Komor adalary)',
        'Indian/Kerguelen' => 'Fransuz Günorta we Antarktika ýerleri wagty (Kergelen)',
        'Indian/Mahe' => 'Seýşel adalary wagty (Maýe)',
        'Indian/Maldives' => 'Maldiwler wagty',
        'Indian/Mauritius' => 'Mawrikiý wagty',
        'Indian/Mayotte' => 'Gündogar Afrika wagty (Maýotta)',
        'Indian/Reunion' => 'Reýunýon wagty',
        'Pacific/Apia' => 'Apia wagty',
        'Pacific/Auckland' => 'Täze Zelandiýa wagty (Oklend)',
        'Pacific/Bougainville' => 'Papua - Täze Gwineýa wagty (Bugenwil)',
        'Pacific/Chatham' => 'Çatem wagty',
        'Pacific/Easter' => 'Pasha adasy wagty',
        'Pacific/Efate' => 'Wanuatu wagty (Efate)',
        'Pacific/Enderbury' => 'Feniks adalary wagty (Kanton adasy)',
        'Pacific/Kanton' => 'Feniks adalary wagty (Kanton adasy)',
        'Pacific/Fakaofo' => 'Tokelau wagty (Fakaofo)',
        'Pacific/Fiji' => 'Fiji wagty',
        'Pacific/Funafuti' => 'Tuwalu wagty (Funafuti)',
        'Pacific/Galapagos' => 'Galapagos adalary wagty',
        'Pacific/Gambier' => 'Gambýe wagty',
        'Pacific/Guadalcanal' => 'Solomon adalary wagty (Gwadalkanal)',
        'Pacific/Guam' => 'Çamorro wagty (Guam)',
        'Pacific/Honolulu' => 'Gawaý-Aleut standart wagty (Gonolulu)',
        'Pacific/Kiritimati' => 'Laýn adalary wagty (Kiritimati)',
        'Pacific/Kosrae' => 'Kosraýe wagty',
        'Pacific/Kwajalein' => 'Marşall adalary wagty (Kwajaleýn)',
        'Pacific/Majuro' => 'Marşall adalary wagty (Majuro)',
        'Pacific/Marquesas' => 'Markiz adalary wagty',
        'Pacific/Midway' => 'Samoa wagty (Midueý)',
        'Pacific/Nauru' => 'Nauru wagty',
        'Pacific/Niue' => 'Niue wagty',
        'Pacific/Norfolk' => 'Norfolk adasy wagty',
        'Pacific/Noumea' => 'Täze Kaledoniýa wagty (Numea)',
        'Pacific/Pago_Pago' => 'Samoa wagty (Pago-Pago)',
        'Pacific/Palau' => 'Palau wagty',
        'Pacific/Pitcairn' => 'Pitkern wagty',
        'Pacific/Ponape' => 'Ponape wagty',
        'Pacific/Pohnpei' => 'Ponape wagty',
        'Pacific/Port_Moresby' => 'Papua - Täze Gwineýa wagty (Port-Morsbi)',
        'Pacific/Rarotonga' => 'Kuk adalary wagty (Rarotonga)',
        'Pacific/Saipan' => 'Çamorro wagty (Saýpan)',
        'Pacific/Tahiti' => 'Taiti wagty',
        'Pacific/Tarawa' => 'Gilbert adalary wagty (Tarawa)',
        'Pacific/Tongatapu' => 'Tonga wagty (Tongatapu)',
        'Pacific/Truk' => 'Çuuk wagty',
        'Pacific/Chuuk' => 'Çuuk wagty',
        'Pacific/Wake' => 'Weýk adasy wagty',
        'Pacific/Wallis' => 'Uollis we Futuna wagty',
    ],
    'Meta' => [],
];
