<?php

return [
    'Names' => [
        'Adlm' => '𞤀𞤁𞤂𞤢𞤃',
        'Aghb' => '𞤀𞤹𞤦𞤢𞤲𞤭𞤴𞤢',
        'Ahom' => '𞤀𞤸𞤮𞤥',
        'Arab' => '𞤀𞥄𞤪𞤢𞤦𞤵',
        'Aran' => '𞤐𞤢𞤧𞤼𞤢𞤤𞤭𞤹',
        'Armi' => '𞤀𞤪𞤢𞤥𞤭𞤴𞤢 𞤉𞤥𞤨𞤫𞤪𞤭𞤴𞤢𞤤',
        'Armn' => '𞤀𞤪𞤥𞤫𞤲𞤭𞤴𞤢𞤲',
        'Avst' => '𞤀𞤾𞤫𞤧𞤼𞤢𞤲',
        'Bali' => '𞤄𞤢𞤤𞤭𞤲𞤭𞥅𞤧',
        'Bamu' => '𞤄𞤢𞤥𞤵',
        'Bass' => '𞤄𞤢𞤧𞤢𞥄 𞤜𞤢𞥄',
        'Batk' => '𞤄𞤢𞤼𞤢𞤳',
        'Beng' => '𞤄𞤫𞤲𞤺𞤢𞤤𞤭',
        'Bhks' => '𞤄𞤢𞤴𞤳𞤵𞤧𞤵𞤳𞤭',
        'Bopo' => '𞤄𞤮𞤨𞤮𞤥𞤮𞤬𞤮',
        'Brah' => '𞤄𞤪𞤢𞤸𞤢𞤥𞤭',
        'Brai' => '𞤄𞤢𞤪𞤢𞤭𞥅𞤤𞤵',
        'Bugi' => '𞤄𞤵𞤺𞤭𞤲𞤭𞤴𞤢',
        'Buhd' => '𞤄𞤵𞤸𞤭𞤣',
        'Cakm' => '𞤕𞤢𞤳𞤥𞤢',
        'Cans' => '𞤑𞤢𞤱𞤪𞤢𞤤 𞤅𞤭𞤺𞤢𞤲𞤯𞤫 𞤚𞤢𞥄𞤳𞤢𞤲𞤶𞤫 𞤑𞤢𞤲𞤢𞤣𞤢𞥄',
        'Cari' => '𞤑𞤢𞤪𞤭𞤴𞤢𞤲',
        'Cham' => '𞤕𞤢𞥄𞤥',
        'Cher' => '𞤕𞤫𞤪𞤮𞤳𞤭𞥅',
        'Chrs' => '𞤑𞤮𞤪𞤢𞥄𞤧𞤥𞤭𞤴𞤢',
        'Copt' => '𞤑𞤭𞤦𞤯𞤭𞤲𞤳𞤮',
        'Cpmn' => '𞤅𞤭𞥅𞤨𞤪𞤮 𞤃𞤭𞤲𞤮𞤴𞤢',
        'Cprt' => '𞤑𞤵𞤦𞤭𞤪𞤧𞤵',
        'Cyrl' => '𞤅𞤭𞤪𞤤𞤭𞤳',
        'Deva' => '𞤁𞤫𞤾𞤢𞤲𞤢𞤺𞤢𞤪𞤭',
        'Diak' => '𞤁𞤭𞤾𞤫𞥅𞤧 𞤀𞤳𞤵𞤪𞤵',
        'Dogr' => '𞤁𞤮𞤺𞤪𞤢',
        'Dsrt' => '𞤁𞤫𞤧𞤫𞤪𞤫𞤼',
        'Dupl' => '𞤁𞤵𞤨𞤤𞤮𞤴𞤢𞤲 𞤅𞤮𞥅𞤪𞤼𞤤𞤢𞤲𞤣',
        'Egyp' => '𞤖𞤭𞤪𞤮𞤺𞤭𞤪𞤬𞤵 𞤃𞤭𞤧𞤭𞤪𞤢',
        'Elba' => '𞤉𞤤𞤦𞤢𞤧𞤢𞤲',
        'Elym' => '𞤉𞤤𞤴𞤥𞤢𞤴𞤳',
        'Ethi' => '𞤖𞤢𞤦𞤢𞤧𞤭𞤲𞤳𞤮',
        'Geor' => '𞤔𞤮𞤪𞤶𞤭𞤴𞤢𞤲',
        'Glag' => '𞤘𞤭𞤤𞤢𞤺𞤮𞤤𞤭𞤼𞤭𞤳',
        'Gong' => '𞤘𞤵𞤲𞤶𞤢𞤤𞤢 𞤘𞤮𞤲𞤣𞤭',
        'Gonm' => '𞤃𞤢𞤧𞤢𞤪𞤢𞤲 𞤘𞤮𞤲𞤣𞤭',
        'Goth' => '𞤘𞤵𞥅𞤼𞤭𞤲𞤳𞤮',
        'Gran' => '𞤘𞤢𞤪𞤢𞤲𞤼𞤢',
        'Grek' => '𞤘𞤭𞤪𞤧𞤢',
        'Gujr' => '𞤘𞤵𞤶𞤢𞤪𞤢𞤼𞤭𞥅',
        'Guru' => '𞤘𞤵𞤪𞤥𞤵𞤿𞤭',
        'Hanb' => '𞤖𞤢𞥄𞤲 𞤫 𞤄𞤮𞤨𞤮𞤥𞤮𞤬𞤮',
        'Hang' => '𞤖𞤢𞤲𞤺𞤵𞥅𞤤',
        'Hani' => '𞤖𞤢𞥄𞤲',
        'Hano' => '𞤖𞤢𞤲𞤵𞥅𞤲𞤮',
        'Hans' => '𞤖𞤮𞤴𞤲𞤢𞥄𞤲𞤣𞤫',
        'Hant' => '𞤚𞤢𞤱𞤢𞥄𞤲𞤣𞤫',
        'Hatr' => '𞤖𞤢𞤼𞤢𞤪𞤢𞤲',
        'Hebr' => '𞤖𞤢𞤦𞤵𞤪𞤢',
        'Hira' => '𞤖𞤭𞤪𞤢𞤺𞤢𞤲𞤢',
        'Hluw' => '𞤖𞤭𞤪𞤮𞤺𞤭𞤪𞤬𞤵 𞤀𞤲𞤢𞤼𞤮𞤤𞤭𞤴𞤢',
        'Hmng' => '𞤆𞤢𞤸𞤢𞤱 𞤖𞤢𞤥𞤮𞤲𞤺',
        'Hmnp' => '𞤙𞤭𞤢𞤳𞤫𞤲𞤺 𞤆𞤵𞤢𞤧𞤵𞥅 𞤖𞤥𞤮𞤲𞤺',
        'Hrkt' => '𞤅𞤭𞤺𞤢𞤲𞤯𞤫 𞤐𞤭𞤨𞤮𞤲𞤶𞤫',
        'Hung' => '𞤑𞤭𞤯𞥆𞤭 𞤖𞤢𞤲𞤺𞤢𞤪𞤭𞤴𞤢𞥄',
        'Ital' => '𞤑𞤭𞤯𞤭 𞤋𞤼𞤢𞤤𞤭𞤳',
        'Jamo' => '𞤔𞤢𞤥𞤮',
        'Java' => '𞤟𞤢𞤾𞤢𞥄',
        'Jpan' => '𞤐𞤭𞤨𞤮𞤲',
        'Kali' => '𞤑𞤢𞤴𞤢 𞤂𞤭',
        'Kana' => '𞤑𞤢𞤼𞤢𞤳𞤢𞤲𞤢',
        'Kawi' => '𞤑𞤢𞤱𞤭',
        'Khar' => '𞤑𞤢𞤪𞤮𞥃𞤢𞤼𞤭',
        'Khmr' => '𞤑𞤵𞤥𞤫𞥅𞤪',
        'Khoj' => '𞤑𞤮𞤶𞤳𞤭',
        'Kits' => '𞤄𞤭𞤲𞤳𞤮𞤴 𞤑𞤭𞤼𞤢𞤲',
        'Knda' => '𞤑𞤢𞤲𞥆𞤢𞤣𞤢',
        'Kore' => '𞤑𞤮𞥅𞤪𞤫𞤴𞤢𞤲',
        'Kthi' => '𞤑𞤢𞤭𞤼𞤭',
        'Lana' => '𞤂𞤢𞤲𞥆𞤢',
        'Laoo' => '𞤂𞤢𞤱𞤮𞥅',
        'Latn' => '𞤂𞤢𞤼𞤫𞤲',
        'Lepc' => '𞤂𞤫𞤨𞤷𞤢',
        'Limb' => '𞤂𞤭𞤥𞤦𞤵',
        'Lina' => '𞤊𞤮𞤷𞥆𞤭𞥅𞤲𞤺𞤮𞤤 𞤀',
        'Linb' => '𞤊𞤮𞤷𞥆𞤭𞥅𞤲𞤺𞤮𞤤 𞤄',
        'Lisu' => '𞤂𞤭𞤧𞤵',
        'Lyci' => '𞤂𞤭𞥅𞤧𞤭𞤴𞤢𞤲',
        'Lydi' => '𞤂𞤭𞤣𞤭𞤴𞤢𞤲',
        'Mahj' => '𞤃𞤢𞤸𞤢𞤶𞤢𞤲𞤭𞥅',
        'Maka' => '𞤃𞤢𞤳𞤢𞤧𞤢𞤪',
        'Mand' => '𞤃𞤢𞤲𞤣𞤫𞥅𞤲',
        'Mani' => '𞤃𞤢𞤲𞤭𞤳𞤭𞤴𞤫𞤲',
        'Marc' => '𞤃𞤢𞤪𞤷𞤫𞤲',
        'Medf' => '𞤃𞤢𞤣𞤬𞤫𞤣𞤭𞤪𞤭𞥅𞤲',
        'Mend' => '𞤃𞤫𞤲𞤣𞤫',
        'Merc' => '𞤃𞤫𞤪𞤱𞤫𞤼𞤭𞤳 𞤅𞤢𞤪𞤰𞤵𞤯𞤭',
        'Mero' => '𞤃𞤫𞤪𞤱𞤫𞤼𞤭𞤳',
        'Mlym' => '𞤃𞤢𞤤𞤢𞤴𞤢𞤤𞤢𞤥',
        'Modi' => '𞤃𞤮𞤣𞤭',
        'Mong' => '𞤃𞤮𞤲𞤺𞤮𞤤𞤭𞤴𞤢𞤲',
        'Mroo' => '𞤃𞤮𞤪𞤮𞥅',
        'Mtei' => '𞤃𞤫𞤼𞤭 𞤃𞤢𞤴𞤫𞤳',
        'Mult' => '𞤃𞤵𞤤𞤼𞤢𞤲𞤭',
        'Mymr' => '𞤃𞤭𞤴𞤢𞤥𞤢𞥄𞤪',
        'Nagm' => '𞤐𞤢𞤺 𞤃𞤵𞤲𞤣𞤢𞤪𞤭',
        'Nand' => '𞤐𞤢𞤲𞤣𞤭𞤲𞤢𞤺𞤢𞤪𞤭',
        'Narb' => '𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫𞤲𞤳𞤮 𞤀𞥄𞤪𞤢𞤦𞤵 𞤑𞤭𞤯𞤭',
        'Nbat' => '𞤐𞤢𞤦𞤢𞤼𞤭𞤴𞤢𞤲',
        'Newa' => '𞤐𞤭𞤱𞤢',
        'Nkoo' => '𞤐𞤳𞤮𞥅',
        'Nshu' => '𞤐𞤵𞥅𞤧𞤵',
        'Ogam' => '𞤌𞥅𞤺𞤢𞤥𞤵',
        'Olck' => '𞤌𞤭-𞤕𞤭𞤳𞤭',
        'Orkh' => '𞤌𞤪𞤳𞤮𞥅𞤲',
        'Orya' => '𞤌𞤪𞤭𞤴𞤢',
        'Osge' => '𞤌𞤧𞤢𞥄𞤶',
        'Osma' => '𞤌𞤧𞤥𞤢𞤲𞤭𞤴𞤢',
        'Ougr' => '𞤏𞤭𞤺𞤵𞥅𞤪 𞤑𞤭𞤯𞥆𞤵𞤲',
        'Palm' => '𞤆𞤢𞤤𞤥𞤫𞤪𞤫𞥅𞤲',
        'Pauc' => '𞤆𞤢𞤱 𞤅𞤭𞥅𞤲 𞤖𞤢𞥄𞤱',
        'Perm' => '𞤆𞤫𞤪𞤥𞤭𞤳 𞤑𞤭𞤯𞥆𞤵𞤲',
        'Phag' => '𞤊𞤢𞤺𞤧 𞤆𞤢',
        'Phli' => '𞤄𞤭𞤲𞤣𞤭 𞤆𞤢𞤤𞤢𞤾𞤭',
        'Phlp' => '𞤅𞤮𞤤𞤼𞤮𞥅 𞤆𞤢𞤤𞤢𞤾𞤭',
        'Phnx' => '𞤊𞤭𞤲𞤭𞤳𞤭𞤴𞤢𞤲𞤳𞤮',
        'Plrd' => '𞤖𞤭𞤼𞤮𞤲𞤳𞤮 𞤆𞤮𞤤𞥆𞤢𞤪𞤣',
        'Prti' => '𞤄𞤭𞤲𞤣𞤭 𞤆𞤢𞤪𞤧𞤭𞤴𞤢𞤲',
        'Qaag' => '𞤟𞤢𞤱𞤺𞤭𞥅𞤴𞤵',
        'Rjng' => '𞤈𞤭𞤶𞤢𞤲𞤺',
        'Rohg' => '𞤖𞤢𞤲𞤭𞤬𞤭',
        'Runr' => '𞤈𞤵𞤲𞤭𞥅𞤳',
        'Samr' => '𞤅𞤢𞤥𞤢𞤪𞤭𞤼𞤢𞤲',
        'Sarb' => '𞤙𞤢𞥄𞤥𞤲𞤢𞥄𞤲𞤺𞤫𞤲𞤳𞤮 𞤀𞥄𞤪𞤢𞤦𞤵 𞤑𞤭𞤯𞤭',
        'Saur' => '𞤅𞤢𞤵𞤪𞤢𞥃𞤼𞤪𞤢',
        'Sgnw' => '𞤄𞤭𞤲𞤣𞤭 𞤊𞤭𞤲𞤣𞤫',
        'Shaw' => '𞤅𞤢𞤬𞤭𞤴𞤢𞥄𞤲',
        'Shrd' => '𞤡𞤢𞤪𞤢𞤣𞤢',
        'Sidd' => '𞤅𞤭𞤣𞥆𞤢𞥄𞤥',
        'Sind' => '𞤑𞤵𞤣𞤢𞤱𞤢𞤣𞤭',
        'Sinh' => '𞤅𞤭𞤲𞤸𞤢𞤤𞤢',
        'Sogd' => '𞤅𞤮𞤺𞤮𞤣𞤭𞤴𞤢𞤲',
        'Sogo' => '𞤅𞤮𞤺𞤮𞤣𞤭𞤴𞤢𞤲 𞤑𞤭𞤯𞥆𞤵𞤲',
        'Sora' => '𞤅𞤢𞤪𞤢 𞤅𞤮𞤥𞤨𞤢𞤲𞤺',
        'Soyo' => '𞤅𞤮𞤴𞤮𞤥𞤦𞤮',
        'Sund' => '𞤅𞤵𞤲𞤣𞤢𞤲𞤭',
        'Sylo' => '𞤅𞤴𞤤𞤮𞤼𞤭𞥅 𞤐𞤢𞤺𞤪𞤭',
        'Syrc' => '𞤅𞤭𞥅𞤪𞤴𞤢𞤳',
        'Tagb' => '𞤚𞤢𞤺𞤦𞤢𞤲𞤱𞤢',
        'Takr' => '𞤚𞤢𞤳𞤪𞤭',
        'Tale' => '𞤚𞤢𞥄𞤴 𞤂𞤭𞥅',
        'Talu' => '𞤚𞤢𞥄𞤴 𞤂𞤵𞤫 𞤑𞤫𞤧𞤮',
        'Taml' => '𞤚𞤢𞤥𞤭𞤤',
        'Tang' => '𞤚𞤢𞤲𞤺𞤵𞤼',
        'Tavt' => '𞤚𞤢𞥄𞤴 𞤜𞤭𞤫𞥅𞤼',
        'Telu' => '𞤚𞤫𞤤𞤵𞤺𞤵',
        'Tfng' => '𞤚𞤭𞤬𞤭𞤲𞤢𞥄𞤺',
        'Tglg' => '𞤚𞤢𞤺𞤢𞤤𞤮𞤺',
        'Thaa' => '𞤡𞤢𞥄𞤲𞤢',
        'Thai' => '𞤚𞤢𞤱𞤤𞤢𞤲𞤣',
        'Tibt' => '𞤚𞤭𞤦𞤫𞤼𞤢𞤲',
        'Tirh' => '𞤚𞤭𞤪𞤸𞤵𞤼𞤢',
        'Tnsa' => '𞤚𞤢𞤲𞤺𞤧𞤢',
        'Toto' => '𞤚𞤮𞤼𞤮',
        'Ugar' => '𞤓𞤺𞤢𞤪𞤭𞤼𞤭𞤳',
        'Vaii' => '𞤜𞤢𞥄𞤴',
        'Vith' => '𞤜𞤭𞤼𞤳𞤵𞤹𞤭',
        'Wara' => '𞤜𞤢𞤪𞤢𞤲𞤺 𞤑𞤭𞥃𞤼𞤭',
        'Wcho' => '𞤏𞤢𞤲𞤷𞤮𞥅',
        'Xpeo' => '𞤊𞤢𞥄𞤪𞤭𞤧𞤭𞤴𞤢𞤲𞤳𞤮 𞤑𞤭𞤯𞥆𞤵𞤲',
        'Xsux' => '𞤅𞤵𞤥𞤫𞤪𞤮 𞤀𞤳𞥆𞤢𞤣𞤭𞤴𞤢𞤲 𞤑𞤵𞤲𞤫𞤬𞤮𞤪𞤥',
        'Yezi' => '𞤒𞤢𞤶𞤭𞤣𞤭𞥅𞤴𞤵',
        'Yiii' => '𞤒𞤭𞥅',
        'Zanb' => '𞤟𞤢𞤲𞤢𞤦𞤢𞥁𞤢𞥄𞤪 𞤁𞤭𞤲𞤺𞤫𞤪𞤫',
        'Zinh' => '𞤁𞤮𞤲𞤣𞤭',
        'Zmth' => '𞤍𞤵𞤪𞥆𞤢𞥄𞤲𞤺𞤮 𞤂𞤭𞤥𞤭𞤲𞤳𞤮',
        'Zsye' => '𞤐𞤺𞤮𞤼𞥆𞤭',
        'Zsym' => '𞤋𞤥𞥆𞤮𞤪𞤫',
        'Zxxx' => '𞤀𞤧𞤱𞤭𞤲𞤣𞤢𞥄𞤯𞤵𞤲',
        'Zyyy' => '𞤑𞤢𞤬𞤵',
    ],
];
