<?php

return [
    'Names' => [
        'America/Miquelon' => 'St Pierre & Miquelon Time (Saint-Pierre)',
        'America/St_Barthelemy' => 'Atlantic Time (St Barthélemy)',
        'America/St_Johns' => 'Newfoundland Time (St John’s)',
        'America/St_Kitts' => 'Atlantic Time (St Kitts)',
        'America/St_Lucia' => 'Atlantic Time (St Lucia)',
        'America/St_Thomas' => 'Atlantic Time (St Thomas)',
        'America/St_Vincent' => 'Atlantic Time (St Vincent)',
        'Atlantic/St_Helena' => 'Greenwich Mean Time (St Helena)',
    ],
    'Meta' => [],
];
