<?php

return [
    'Names' => [
        'ADP' => [
            'ADP',
            'پزتای آندورا',
        ],
        'AED' => [
            'AED',
            'درهم امارات متحدهٔ عربی',
        ],
        'AFA' => [
            'AFA',
            'افغانی افغانستان (۱۹۲۷ تا ۲۰۰۲)',
        ],
        'AFN' => [
            '؋',
            'افغانی افغانستان',
        ],
        'ALK' => [
            'ALK',
            'لک آلبانی (۱۹۴۶ تا ۱۹۶۵)',
        ],
        'ALL' => [
            'ALL',
            'لک آلبانی',
        ],
        'AMD' => [
            'AMD',
            'درام ارمنستان',
        ],
        'ANG' => [
            'ANG',
            'گیلدر آنتیل هلند',
        ],
        'AOA' => [
            'AOA',
            'کوانزای آنگولا',
        ],
        'ARM' => [
            'ARM',
            'پزوی آرژانتین (۱۸۸۱ تا ۱۹۷۰)',
        ],
        'ARP' => [
            'ARP',
            'پزوی آرژانتین (۱۹۸۳ تا ۱۹۸۵)‏',
        ],
        'ARS' => [
            'ARS',
            'پزوی آرژانتین',
        ],
        'ATS' => [
            'ATS',
            'شیلینگ اتریش',
        ],
        'AUD' => [
            'A$',
            'دلار استرالیا',
        ],
        'AWG' => [
            'AWG',
            'فلورین آروبا',
        ],
        'AZM' => [
            'AZM',
            'منات جمهوری آذربایجان (۱۹۹۳ تا ۲۰۰۶)',
        ],
        'AZN' => [
            'AZN',
            'منات جمهوری آذربایجان',
        ],
        'BAD' => [
            'BAD',
            'دینار بوسنی و هرزگوین (۱۹۹۲ تا ۱۹۹۴)',
        ],
        'BAM' => [
            'BAM',
            'مارک تبدیل‌پذیر بوسنی و هرزگوین',
        ],
        'BBD' => [
            'BBD',
            'دلار باربادوس',
        ],
        'BDT' => [
            'BDT',
            'تاکای بنگلادش',
        ],
        'BEF' => [
            'BEF',
            'فرانک بلژیک',
        ],
        'BEL' => [
            'BEL',
            'فرانک بلژیک (مالی)',
        ],
        'BGN' => [
            'BGN',
            'لو بلغارستان',
        ],
        'BHD' => [
            'BHD',
            'دینار بحرین',
        ],
        'BIF' => [
            'BIF',
            'فرانک بوروندی',
        ],
        'BMD' => [
            'BMD',
            'دلار برمودا',
        ],
        'BND' => [
            'BND',
            'دلار برونئی',
        ],
        'BOB' => [
            'BOB',
            'بولیویانوی بولیوی',
        ],
        'BOP' => [
            'BOP',
            'پزوی بولیوی',
        ],
        'BRL' => [
            'R$',
            'رئال برزیل',
        ],
        'BSD' => [
            'BSD',
            'دلار باهاما',
        ],
        'BTN' => [
            'BTN',
            'انگولتروم بوتان',
        ],
        'BWP' => [
            'BWP',
            'پولای بوتسوانا',
        ],
        'BYB' => [
            'BYB',
            'روبل جدید بلاروس (۱۹۹۴ تا ۱۹۹۹)',
        ],
        'BYN' => [
            'BYN',
            'روبل بلاروس',
        ],
        'BYR' => [
            'BYR',
            'روبل بلاروسی (۲۰۱۶–۲۰۰۰)',
        ],
        'BZD' => [
            'BZD',
            'دلار بلیز',
        ],
        'CAD' => [
            '$CA',
            'دلار کانادا',
        ],
        'CDF' => [
            'CDF',
            'فرانک کنگو',
        ],
        'CHF' => [
            'CHF',
            'فرانک سوئیس',
        ],
        'CLP' => [
            'CLP',
            'پزوی شیلی',
        ],
        'CNH' => [
            'CNH',
            'یوآن چین (برون‌مرزی)',
        ],
        'CNY' => [
            '¥CN',
            'یوآن چین',
        ],
        'COP' => [
            'COP',
            'پزوی کلمبیا',
        ],
        'CRC' => [
            'CRC',
            'کولون کاستاریکا',
        ],
        'CSD' => [
            'CSD',
            'دینار صربستان (۲۰۰۲ تا ۲۰۰۶)',
        ],
        'CUC' => [
            'CUC',
            'پزوی تبدیل‌پذیر کوبا',
        ],
        'CUP' => [
            'CUP',
            'پزوی کوبا',
        ],
        'CVE' => [
            'CVE',
            'اسکودوی کیپ‌ورد',
        ],
        'CYP' => [
            'CYP',
            'پوند قبرس',
        ],
        'CZK' => [
            'CZK',
            'کورونای جمهوری چک',
        ],
        'DDM' => [
            'DDM',
            'مارک آلمان شرقی',
        ],
        'DEM' => [
            'DEM',
            'مارک آلمان',
        ],
        'DJF' => [
            'DJF',
            'فرانک جیبوتی',
        ],
        'DKK' => [
            'DKK',
            'کرون دانمارک',
        ],
        'DOP' => [
            'DOP',
            'پزوی جمهوری دومینیکن',
        ],
        'DZD' => [
            'DZD',
            'دینار الجزایر',
        ],
        'EEK' => [
            'EEK',
            'کرون استونی',
        ],
        'EGP' => [
            'EGP',
            'پوند مصر',
        ],
        'ERN' => [
            'ERN',
            'ناکفای اریتره',
        ],
        'ESP' => [
            'ESP',
            'پزتای اسپانیا',
        ],
        'ETB' => [
            'ETB',
            'بیر اتیوپی',
        ],
        'EUR' => [
            '€',
            'یورو',
        ],
        'FJD' => [
            'FJD',
            'دلار فیجی',
        ],
        'FKP' => [
            'FKP',
            'پوند جزایر فالکلند',
        ],
        'FRF' => [
            'FRF',
            'فرانک فرانسه',
        ],
        'GBP' => [
            '£',
            'پوند بریتانیا',
        ],
        'GEL' => [
            'GEL',
            'لاری گرجستان',
        ],
        'GHS' => [
            'GHS',
            'سدی غنا',
        ],
        'GIP' => [
            'GIP',
            'پوند جبل‌الطارق',
        ],
        'GMD' => [
            'GMD',
            'دالاسی گامبیا',
        ],
        'GNF' => [
            'GNF',
            'فرانک گینه',
        ],
        'GRD' => [
            'GRD',
            'دراخمای یونان',
        ],
        'GTQ' => [
            'GTQ',
            'کتزال گواتمالا',
        ],
        'GWP' => [
            'GWP',
            'پزوی گینهٔ بیسائو',
        ],
        'GYD' => [
            'GYD',
            'دلار گویانا',
        ],
        'HKD' => [
            '$HK',
            'دلار هنگ‌کنگ',
        ],
        'HNL' => [
            'HNL',
            'لمپیرای هندوراس',
        ],
        'HRD' => [
            'HRD',
            'دینار کرواسی',
        ],
        'HRK' => [
            'HRK',
            'کونای کرواسی',
        ],
        'HTG' => [
            'HTG',
            'گورد هائیتی',
        ],
        'HUF' => [
            'HUF',
            'فورینت مجارستان',
        ],
        'IDR' => [
            'IDR',
            'روپیهٔ اندونزی',
        ],
        'IEP' => [
            'IEP',
            'پوند ایرلند',
        ],
        'ILP' => [
            'ILP',
            'پوند اسرائیل',
        ],
        'ILR' => [
            'ILR',
            'شقل اسرائیل (۱۹۸۰ تا ۱۹۸۵)',
        ],
        'ILS' => [
            '₪',
            'شِکِل جدید اسرائیل',
        ],
        'INR' => [
            '₹',
            'روپیهٔ هند',
        ],
        'IQD' => [
            'IQD',
            'دینار عراق',
        ],
        'IRR' => [
            'ریال',
            'ریال ایران',
        ],
        'ISJ' => [
            'ISJ',
            'کرونای ایسلند (۱۹۱۸ تا ۱۹۸۱)',
        ],
        'ISK' => [
            'ISK',
            'کرونای ایسلند',
        ],
        'ITL' => [
            'ITL',
            'لیرهٔ ایتالیا',
        ],
        'JMD' => [
            'JMD',
            'دلار جامائیکا',
        ],
        'JOD' => [
            'JOD',
            'دینار اردن',
        ],
        'JPY' => [
            '¥',
            'ین ژاپن',
        ],
        'KES' => [
            'KES',
            'شیلینگ کنیا',
        ],
        'KGS' => [
            'KGS',
            'سوم قرقیزستان',
        ],
        'KHR' => [
            'KHR',
            'ری‌یل کامبوج',
        ],
        'KMF' => [
            'KMF',
            'فرانک کومورو',
        ],
        'KPW' => [
            'KPW',
            'وون کرهٔ شمالی',
        ],
        'KRO' => [
            'KRO',
            'وون کرهٔ جنوبی (۱۹۴۵ تا ۱۹۵۳)',
        ],
        'KRW' => [
            '₩',
            'وون کرهٔ جنوبی',
        ],
        'KWD' => [
            'KWD',
            'دینار کویت',
        ],
        'KYD' => [
            'KYD',
            'دلار جزایر کِیمن',
        ],
        'KZT' => [
            'KZT',
            'تنگهٔ قزاقستان',
        ],
        'LAK' => [
            'LAK',
            'کیپ لائوس',
        ],
        'LBP' => [
            'LBP',
            'لیرهٔ لبنان',
        ],
        'LKR' => [
            'LKR',
            'روپیهٔ سری‌لانکا',
        ],
        'LRD' => [
            'LRD',
            'دلار لیبریا',
        ],
        'LSL' => [
            'LSL',
            'لوتی لسوتو',
        ],
        'LTL' => [
            'LTL',
            'لیتاس لیتوانی',
        ],
        'LTT' => [
            'LTT',
            'تالوناس لیتوانی',
        ],
        'LUF' => [
            'LUF',
            'فرانک لوکزامبورگ',
        ],
        'LUL' => [
            'LUL',
            'فرانک مالی لوگزامبورگ',
        ],
        'LVL' => [
            'LVL',
            'لاتس لتونی',
        ],
        'LVR' => [
            'LVR',
            'روبل لتونی',
        ],
        'LYD' => [
            'LYD',
            'دینار لیبی',
        ],
        'MAD' => [
            'MAD',
            'درهم مراکش',
        ],
        'MAF' => [
            'MAF',
            'فرانک مراکش',
        ],
        'MCF' => [
            'MCF',
            'فرانک موناکو',
        ],
        'MDL' => [
            'MDL',
            'لئوی مولداوی',
        ],
        'MGA' => [
            'MGA',
            'آریاری مالاگاسی',
        ],
        'MGF' => [
            'MGF',
            'فرانک ماداگاسکار',
        ],
        'MKD' => [
            'MKD',
            'دینار مقدونیه',
        ],
        'MKN' => [
            'MKN',
            'دینار مقدونیه (۱۹۹۲ تا ۱۹۹۳)',
        ],
        'MLF' => [
            'MLF',
            'فرانک مالی',
        ],
        'MMK' => [
            'MMK',
            'کیات میانمار',
        ],
        'MNT' => [
            'MNT',
            'توگریک مغولستان',
        ],
        'MOP' => [
            'MOP',
            'پاتاکای ماکائو',
        ],
        'MRO' => [
            'MRO',
            'اوگوئیای موریتانی (۱۹۷۳ تا ۲۰۱۷)',
        ],
        'MRU' => [
            'MRU',
            'اوگوئیای موریتانی',
        ],
        'MTL' => [
            'MTL',
            'لیرهٔ مالت',
        ],
        'MTP' => [
            'MTP',
            'پوند مالت',
        ],
        'MUR' => [
            'MUR',
            'روپیهٔ موریس',
        ],
        'MVP' => [
            'MVP',
            'روپیهٔ مالدیو (۱۹۴۷ تا ۱۹۸۱)',
        ],
        'MVR' => [
            'MVR',
            'روپیهٔ مالدیو',
        ],
        'MWK' => [
            'MWK',
            'کواچای مالاوی',
        ],
        'MXN' => [
            '$MX',
            'پزوی مکزیک',
        ],
        'MXP' => [
            'MXP',
            'پزوی نقرهٔ مکزیک (۱۸۶۱ تا ۱۹۹۲)',
        ],
        'MYR' => [
            'MYR',
            'رینگیت مالزی',
        ],
        'MZE' => [
            'MZE',
            'اسکودوی موزامبیک',
        ],
        'MZN' => [
            'MZN',
            'متیکال موزامبیک',
        ],
        'NAD' => [
            'NAD',
            'دلار نامیبیا',
        ],
        'NGN' => [
            'NGN',
            'نایرای نیجریه',
        ],
        'NIO' => [
            'NIO',
            'کوردوبای نیکاراگوئه',
        ],
        'NLG' => [
            'NLG',
            'گیلدر هلند',
        ],
        'NOK' => [
            'NOK',
            'کرون نروژ',
        ],
        'NPR' => [
            'NPR',
            'روپیهٔ نپال',
        ],
        'NZD' => [
            '$NZ',
            'دلار زلاند نو',
        ],
        'OMR' => [
            'OMR',
            'ریال عمان',
        ],
        'PAB' => [
            'PAB',
            'بالبوای پاناما',
        ],
        'PEI' => [
            'PEI',
            'اینتی پرو',
        ],
        'PEN' => [
            'PEN',
            'سول پرو',
        ],
        'PES' => [
            'PES',
            'سول پرو (۱۸۶۳ تا ۱۹۶۵)',
        ],
        'PGK' => [
            'PGK',
            'کینای پاپوا گینهٔ نو',
        ],
        'PHP' => [
            'PHP',
            'پزوی فیلیپین',
        ],
        'PKR' => [
            'PKR',
            'روپیهٔ پاکستان',
        ],
        'PLN' => [
            'PLN',
            'زلوتی لهستان',
        ],
        'PTE' => [
            'PTE',
            'اسکودوی پرتغال',
        ],
        'PYG' => [
            'PYG',
            'گوارانی پاراگوئه',
        ],
        'QAR' => [
            'QAR',
            'ریال قطر',
        ],
        'RHD' => [
            'RHD',
            'دلار رودزیا',
        ],
        'RON' => [
            'RON',
            'لئوی رومانی',
        ],
        'RSD' => [
            'RSD',
            'دینار صربستان',
        ],
        'RUB' => [
            'RUB',
            'روبل روسیه',
        ],
        'RUR' => [
            'RUR',
            'روبل روسیه (۱۹۹۱ تا ۱۹۹۸)',
        ],
        'RWF' => [
            'RWF',
            'فرانک رواندا',
        ],
        'SAR' => [
            'SAR',
            'ریال سعودی',
        ],
        'SBD' => [
            'SBD',
            'دلار جزایر سلیمان',
        ],
        'SCR' => [
            'SCR',
            'روپیهٔ سیشل',
        ],
        'SDD' => [
            'SDD',
            'دینار سودان (۱۹۹۲ تا ۲۰۰۷)',
        ],
        'SDG' => [
            'SDG',
            'پوند سودان',
        ],
        'SEK' => [
            'SEK',
            'کرون سوئد',
        ],
        'SGD' => [
            'SGD',
            'دلار سنگاپور',
        ],
        'SHP' => [
            'SHP',
            'پوند سنت هلن',
        ],
        'SLE' => [
            'SLE',
            'لئون سیرالئون',
        ],
        'SLL' => [
            'SLL',
            'لئون سیرالئون - 1964-2022',
        ],
        'SOS' => [
            'SOS',
            'شیلینگ سومالی',
        ],
        'SRD' => [
            'SRD',
            'دلار سورینام',
        ],
        'SRG' => [
            'SRG',
            'گیلدر سورینام',
        ],
        'SSP' => [
            'SSP',
            'پوند سودان جنوبی',
        ],
        'STD' => [
            'STD',
            'دوبرای سائوتومه و پرنسیپ (۱۹۷۷ تا ۲۰۱۷)',
        ],
        'STN' => [
            'STN',
            'دوبرای سائوتومه و پرنسیپ',
        ],
        'SUR' => [
            'SUR',
            'روبل شوروی',
        ],
        'SYP' => [
            'SYP',
            'لیرهٔ سوریه',
        ],
        'SZL' => [
            'SZL',
            'لیلانگنی سوازیلند',
        ],
        'THB' => [
            '฿',
            'بات تایلند',
        ],
        'TJR' => [
            'TJR',
            'روبل تاجیکستان',
        ],
        'TJS' => [
            'TJS',
            'سامانی تاجیکستان',
        ],
        'TMM' => [
            'TMM',
            'منات ترکمنستان (۱۹۹۳ تا ۲۰۰۹)',
        ],
        'TMT' => [
            'TMT',
            'منات ترکمنستان',
        ],
        'TND' => [
            'TND',
            'دینار تونس',
        ],
        'TOP' => [
            'TOP',
            'پاآنگای تونگا',
        ],
        'TPE' => [
            'TPE',
            'اسکودوی تیمور',
        ],
        'TRL' => [
            'TRL',
            'لیرهٔ ترکیه (۱۹۲۲ تا ۲۰۰۵)',
        ],
        'TRY' => [
            'TRY',
            'لیرهٔ ترکیه',
        ],
        'TTD' => [
            'TTD',
            'دلار ترینیداد و توباگو',
        ],
        'TWD' => [
            'NT$',
            'دلار جدید تایوان',
        ],
        'TZS' => [
            'TZS',
            'شیلینگ تانزانیا',
        ],
        'UAH' => [
            'UAH',
            'هریونیای اوکراین',
        ],
        'UGS' => [
            'UGS',
            'شیلینگ اوگاندا (۱۹۶۶ تا ۱۹۸۷)',
        ],
        'UGX' => [
            'UGX',
            'شیلینگ اوگاندا',
        ],
        'USD' => [
            '$',
            'دلار آمریکا',
        ],
        'USN' => [
            'USN',
            'دلار امریکا (روز بعد)',
        ],
        'USS' => [
            'USS',
            'دلار امریکا (همان روز)',
        ],
        'UYP' => [
            'UYP',
            'پزوی اوروگوئه (۱۹۷۵ تا ۱۹۹۳)',
        ],
        'UYU' => [
            'UYU',
            'پزوی اوروگوئه',
        ],
        'UZS' => [
            'UZS',
            'سوم ازبکستان',
        ],
        'VEB' => [
            'VEB',
            'بولیوار ونزوئلا (۱۸۷۱ تا ۲۰۰۸)',
        ],
        'VEF' => [
            'VEF',
            'بولیوار ونزوئلا (۲۰۰۸ تا ۲۰۱۸)',
        ],
        'VES' => [
            'VES',
            'بولیوار ونزوئلا',
        ],
        'VND' => [
            '₫',
            'دانگ ویتنام',
        ],
        'VUV' => [
            'VUV',
            'واتوی وانوواتو',
        ],
        'WST' => [
            'WST',
            'تالای ساموا',
        ],
        'XAF' => [
            'FCFA',
            'فرانک CFA مرکز افریقا',
        ],
        'XCD' => [
            '$EC',
            'دلار شرق کارائیب',
        ],
        'XCG' => [
            'Cg.',
            'گیلدر کارائیب',
        ],
        'XFO' => [
            'XFO',
            'فرانک طلای فرانسه',
        ],
        'XOF' => [
            'فرانک CFA',
            'فرانک CFA غرب افریقا',
        ],
        'XPF' => [
            'CFPF',
            'فرانک اقیانوسیه',
        ],
        'YDD' => [
            'YDD',
            'دینار یمن',
        ],
        'YER' => [
            'YER',
            'ریال یمن',
        ],
        'ZAR' => [
            'ZAR',
            'راند افریقای جنوبی',
        ],
        'ZMK' => [
            'ZMK',
            'کواچای زامبیا (۱۹۶۸ تا ۲۰۱۲)',
        ],
        'ZMW' => [
            'ZMW',
            'کواچای زامبیا',
        ],
        'ZWD' => [
            'ZWD',
            'دلار زیمبابوه (۱۹۸۰ تا ۲۰۰۸)',
        ],
        'ZWG' => [
            'ZWG',
            'طلای زیمبابوه',
        ],
        'ZWL' => [
            'ZWL',
            'دلار زیمبابوه (۲۰۰۹)',
        ],
        'ZWR' => [
            'ZWR',
            'دلار زیمبابوه (۲۰۰۸)',
        ],
    ],
];
