<?php

return [
    'Names' => [
        'Adlm' => 'Adlam Simi',
        'Arab' => 'Arabe Simi',
        'Aran' => 'Nastaliq qillqa',
        'Armn' => 'Armenio Simi',
        'Beng' => 'Bangla Simi',
        'Bopo' => 'Bopomofo Simi',
        'Brai' => 'Braile',
        'Cakm' => 'Chakma Simi',
        'Cans' => 'Silabeo aborigen Simi (Canadiense unificado)',
        'Cher' => 'Cherokee Simi',
        'Cyrl' => 'Cirilico',
        'Deva' => 'Devanagari',
        'Ethi' => 'Etiope',
        'Geor' => 'Georgiano',
        'Grek' => 'Griego Simi',
        'Gujr' => 'Gujarati Simi',
        'Guru' => 'Gurmukhi Simi',
        'Hanb' => 'Han with Bopomofo',
        'Hang' => 'Hangul',
        'Hani' => 'Han',
        'Hans' => 'Simplificado',
        'Hant' => 'Tradicional',
        'Hebr' => 'Hebreo Simi',
        'Hira' => 'Hiragana',
        'Hrkt' => 'Japones silabico sananpakuna',
        'Jpan' => 'Japones Simi',
        'Kana' => 'Katakana',
        'Khmr' => 'Khmer',
        'Knda' => 'Kannada Simi',
        'Kore' => 'Coreano Simi',
        'Laoo' => 'Lao Simi',
        'Latn' => 'Latin Simi',
        'Mlym' => 'Malayalam Simi',
        'Mong' => 'Mongol Simi',
        'Mtei' => 'Meitei Mayek Simi',
        'Mymr' => 'Myanmar',
        'Nkoo' => 'N’Ko Simi',
        'Olck' => 'Ol Chiki Simi',
        'Orya' => 'Odia Simi',
        'Rohg' => 'Hanifi Simi',
        'Sinh' => 'Cingales Simi',
        'Sund' => 'Sundanese Simi',
        'Syrc' => 'Sirio Simi',
        'Taml' => 'Tamil Simi',
        'Telu' => 'Tegulu Simi',
        'Tfng' => 'Tifinagh Simi',
        'Thaa' => 'Thaana Simi',
        'Thai' => 'Tailandes Simi',
        'Tibt' => 'Tibetano Simi',
        'Vaii' => 'Vai Simi',
        'Yiii' => 'Yi Simi',
        'Zmth' => 'Matimatica Willay',
        'Zsye' => 'Emoji',
        'Zsym' => 'Unanchakuna',
        'Zxxx' => 'Mana qillqasqa',
        'Zyyy' => 'Common Simi',
    ],
];
