<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\Serializer\Tests\Fixtures;

final class AsymmetricVisibilityDummy
{
    private(set) string $type;

    public function __construct(
        public readonly string $item,
    ) {
        $this->type = 'final';
    }
}
