<?php

namespace Symfony\Component\DependencyInjection\Tests\Fixtures\PrototypeAsAlias;

use Symfony\Component\DependencyInjection\Attribute\AsAlias;

#[AsAlias(id: AliasBarInterface::class, when: ['dev', 'prod'])]
class WithAsAliasBothEnv
{
}
