<?php

return [
    'Names' => [
        'Africa/Abidjan' => 'ग्रीनविच मीन समयः (Abidjan)',
        'Africa/Accra' => 'ग्रीनविच मीन समयः (Accra)',
        'Africa/Algiers' => 'मध्य यूरोपीय समयः (Algiers)',
        'Africa/Bamako' => 'ग्रीनविच मीन समयः (Bamako)',
        'Africa/Banjul' => 'ग्रीनविच मीन समयः (Banjul)',
        'Africa/Bissau' => 'ग्रीनविच मीन समयः (Bissau)',
        'Africa/Cairo' => 'पौर्व यूरोपीय समयः (Cairo)',
        'Africa/Casablanca' => 'पाश्चात्य यूरोपीय समयः (Casablanca)',
        'Africa/Ceuta' => 'मध्य यूरोपीय समयः (Ceuta)',
        'Africa/Conakry' => 'ग्रीनविच मीन समयः (Conakry)',
        'Africa/Dakar' => 'ग्रीनविच मीन समयः (Dakar)',
        'Africa/El_Aaiun' => 'पाश्चात्य यूरोपीय समयः (El Aaiún)',
        'Africa/Freetown' => 'ग्रीनविच मीन समयः (Freetown)',
        'Africa/Lome' => 'ग्रीनविच मीन समयः (Lomé)',
        'Africa/Monrovia' => 'ग्रीनविच मीन समयः (Monrovia)',
        'Africa/Nouakchott' => 'ग्रीनविच मीन समयः (Nouakchott)',
        'Africa/Ouagadougou' => 'ग्रीनविच मीन समयः (Ouagadougou)',
        'Africa/Sao_Tome' => 'ग्रीनविच मीन समयः (São Tomé)',
        'Africa/Tripoli' => 'पौर्व यूरोपीय समयः (Tripoli)',
        'Africa/Tunis' => 'मध्य यूरोपीय समयः (Tunis)',
        'America/Adak' => 'संयुक्त राज्य: समय: (Adak)',
        'America/Anchorage' => 'संयुक्त राज्य: समय: (Anchorage)',
        'America/Anguilla' => 'अटलाण्टिक समयः (Anguilla)',
        'America/Antigua' => 'अटलाण्टिक समयः (Antigua)',
        'America/Araguaina' => 'ब्राजील समय: (Araguaína)',
        'America/Aruba' => 'अटलाण्टिक समयः (Aruba)',
        'America/Bahia' => 'ब्राजील समय: (Bahia)',
        'America/Bahia_Banderas' => 'उत्तर अमेरिका: मध्य समयः (Bahía de Banderas)',
        'America/Barbados' => 'अटलाण्टिक समयः (Barbados)',
        'America/Belem' => 'ब्राजील समय: (Belém)',
        'America/Belize' => 'उत्तर अमेरिका: मध्य समयः (Belize)',
        'America/Blanc-Sablon' => 'अटलाण्टिक समयः (Blanc-Sablon)',
        'America/Boa_Vista' => 'ब्राजील समय: (Boa Vista)',
        'America/Boise' => 'उत्तर अमेरिका: शैल समयः (Boise)',
        'America/Cambridge_Bay' => 'उत्तर अमेरिका: शैल समयः (Cambridge Bay)',
        'America/Campo_Grande' => 'ब्राजील समय: (Campo Grande)',
        'America/Cancun' => 'उत्तर अमेरिका: पौर्व समयः (Cancún)',
        'America/Cayman' => 'उत्तर अमेरिका: पौर्व समयः (Cayman)',
        'America/Chicago' => 'उत्तर अमेरिका: मध्य समयः (Chicago)',
        'America/Chihuahua' => 'उत्तर अमेरिका: मध्य समयः (Chihuahua)',
        'America/Ciudad_Juarez' => 'उत्तर अमेरिका: शैल समयः (Ciudad Juárez)',
        'America/Coral_Harbour' => 'उत्तर अमेरिका: पौर्व समयः (Atikokan)',
        'America/Atikokan' => 'उत्तर अमेरिका: पौर्व समयः (Atikokan)',
        'America/Costa_Rica' => 'उत्तर अमेरिका: मध्य समयः (Costa Rica)',
        'America/Creston' => 'उत्तर अमेरिका: शैल समयः (Creston)',
        'America/Cuiaba' => 'ब्राजील समय: (Cuiabá)',
        'America/Curacao' => 'अटलाण्टिक समयः (Curaçao)',
        'America/Danmarkshavn' => 'ग्रीनविच मीन समयः (Danmarkshavn)',
        'America/Dawson_Creek' => 'उत्तर अमेरिका: शैल समयः (Dawson Creek)',
        'America/Denver' => 'उत्तर अमेरिका: शैल समयः (Denver)',
        'America/Detroit' => 'उत्तर अमेरिका: पौर्व समयः (Detroit)',
        'America/Dominica' => 'अटलाण्टिक समयः (Dominica)',
        'America/Edmonton' => 'उत्तर अमेरिका: शैल समयः (Edmonton)',
        'America/Eirunepe' => 'ब्राजील समय: (Eirunepé)',
        'America/El_Salvador' => 'उत्तर अमेरिका: मध्य समयः (El Salvador)',
        'America/Fort_Nelson' => 'उत्तर अमेरिका: शैल समयः (Fort Nelson)',
        'America/Fortaleza' => 'ब्राजील समय: (Fortaleza)',
        'America/Glace_Bay' => 'अटलाण्टिक समयः (Glace Bay)',
        'America/Goose_Bay' => 'अटलाण्टिक समयः (Goose Bay)',
        'America/Grand_Turk' => 'उत्तर अमेरिका: पौर्व समयः (Grand Turk)',
        'America/Grenada' => 'अटलाण्टिक समयः (Grenada)',
        'America/Guadeloupe' => 'अटलाण्टिक समयः (Guadeloupe)',
        'America/Guatemala' => 'उत्तर अमेरिका: मध्य समयः (Guatemala)',
        'America/Halifax' => 'अटलाण्टिक समयः (Halifax)',
        'America/Indiana/Knox' => 'उत्तर अमेरिका: मध्य समयः (Knox, Indiana)',
        'America/Indiana/Marengo' => 'उत्तर अमेरिका: पौर्व समयः (Marengo, Indiana)',
        'America/Indiana/Petersburg' => 'उत्तर अमेरिका: पौर्व समयः (Petersburg, Indiana)',
        'America/Indiana/Tell_City' => 'उत्तर अमेरिका: मध्य समयः (Tell City, Indiana)',
        'America/Indiana/Vevay' => 'उत्तर अमेरिका: पौर्व समयः (Vevay, Indiana)',
        'America/Indiana/Vincennes' => 'उत्तर अमेरिका: पौर्व समयः (Vincennes, Indiana)',
        'America/Indiana/Winamac' => 'उत्तर अमेरिका: पौर्व समयः (Winamac, Indiana)',
        'America/Indianapolis' => 'उत्तर अमेरिका: पौर्व समयः (Indianapolis)',
        'America/Indiana/Indianapolis' => 'उत्तर अमेरिका: पौर्व समयः (Indianapolis)',
        'America/Inuvik' => 'उत्तर अमेरिका: शैल समयः (Inuvik)',
        'America/Iqaluit' => 'उत्तर अमेरिका: पौर्व समयः (Iqaluit)',
        'America/Jamaica' => 'उत्तर अमेरिका: पौर्व समयः (Jamaica)',
        'America/Juneau' => 'संयुक्त राज्य: समय: (Juneau)',
        'America/Kentucky/Monticello' => 'उत्तर अमेरिका: पौर्व समयः (Monticello, Kentucky)',
        'America/Kralendijk' => 'अटलाण्टिक समयः (Kralendijk)',
        'America/Los_Angeles' => 'उत्तर अमेरिका: सन्धिप्रिय समयः (Los Angeles)',
        'America/Louisville' => 'उत्तर अमेरिका: पौर्व समयः (Louisville)',
        'America/Kentucky/Louisville' => 'उत्तर अमेरिका: पौर्व समयः (Louisville)',
        'America/Lower_Princes' => 'अटलाण्टिक समयः (Lower Prince’s Quarter)',
        'America/Maceio' => 'ब्राजील समय: (Maceió)',
        'America/Managua' => 'उत्तर अमेरिका: मध्य समयः (Managua)',
        'America/Manaus' => 'ब्राजील समय: (Manaus)',
        'America/Marigot' => 'अटलाण्टिक समयः (Marigot)',
        'America/Martinique' => 'अटलाण्टिक समयः (Martinique)',
        'America/Matamoros' => 'उत्तर अमेरिका: मध्य समयः (Matamoros)',
        'America/Menominee' => 'उत्तर अमेरिका: मध्य समयः (Menominee)',
        'America/Merida' => 'उत्तर अमेरिका: मध्य समयः (Mérida)',
        'America/Metlakatla' => 'संयुक्त राज्य: समय: (Metlakatla)',
        'America/Mexico_City' => 'उत्तर अमेरिका: मध्य समयः (Ciudad de México)',
        'America/Moncton' => 'अटलाण्टिक समयः (Moncton)',
        'America/Monterrey' => 'उत्तर अमेरिका: मध्य समयः (Monterrey)',
        'America/Montserrat' => 'अटलाण्टिक समयः (Montserrat)',
        'America/Nassau' => 'उत्तर अमेरिका: पौर्व समयः (Nassau)',
        'America/New_York' => 'उत्तर अमेरिका: पौर्व समयः (New York)',
        'America/Nome' => 'संयुक्त राज्य: समय: (Nome)',
        'America/Noronha' => 'ब्राजील समय: (Fernando de Noronha)',
        'America/North_Dakota/Beulah' => 'उत्तर अमेरिका: मध्य समयः (Beulah, North Dakota)',
        'America/North_Dakota/Center' => 'उत्तर अमेरिका: मध्य समयः (Center, North Dakota)',
        'America/North_Dakota/New_Salem' => 'उत्तर अमेरिका: मध्य समयः (New Salem, North Dakota)',
        'America/Ojinaga' => 'उत्तर अमेरिका: मध्य समयः (Ojinaga)',
        'America/Panama' => 'उत्तर अमेरिका: पौर्व समयः (Panama)',
        'America/Phoenix' => 'उत्तर अमेरिका: शैल समयः (Phoenix)',
        'America/Port-au-Prince' => 'उत्तर अमेरिका: पौर्व समयः (Port-au-Prince)',
        'America/Port_of_Spain' => 'अटलाण्टिक समयः (Port of Spain)',
        'America/Porto_Velho' => 'ब्राजील समय: (Porto Velho)',
        'America/Puerto_Rico' => 'अटलाण्टिक समयः (Puerto Rico)',
        'America/Rankin_Inlet' => 'उत्तर अमेरिका: मध्य समयः (Rankin Inlet)',
        'America/Recife' => 'ब्राजील समय: (Recife)',
        'America/Regina' => 'उत्तर अमेरिका: मध्य समयः (Regina)',
        'America/Resolute' => 'उत्तर अमेरिका: मध्य समयः (Resolute)',
        'America/Rio_Branco' => 'ब्राजील समय: (Rio Branco)',
        'America/Santarem' => 'ब्राजील समय: (Santarém)',
        'America/Santo_Domingo' => 'अटलाण्टिक समयः (Santo Domingo)',
        'America/Sao_Paulo' => 'ब्राजील समय: (São Paulo)',
        'America/Sitka' => 'संयुक्त राज्य: समय: (Sitka)',
        'America/St_Barthelemy' => 'अटलाण्टिक समयः (St. Barthélemy)',
        'America/St_Kitts' => 'अटलाण्टिक समयः (St. Kitts)',
        'America/St_Lucia' => 'अटलाण्टिक समयः (St. Lucia)',
        'America/St_Thomas' => 'अटलाण्टिक समयः (St. Thomas)',
        'America/St_Vincent' => 'अटलाण्टिक समयः (St. Vincent)',
        'America/Swift_Current' => 'उत्तर अमेरिका: मध्य समयः (Swift Current)',
        'America/Tegucigalpa' => 'उत्तर अमेरिका: मध्य समयः (Tegucigalpa)',
        'America/Thule' => 'अटलाण्टिक समयः (Thule)',
        'America/Tijuana' => 'उत्तर अमेरिका: सन्धिप्रिय समयः (Tijuana)',
        'America/Toronto' => 'उत्तर अमेरिका: पौर्व समयः (Toronto)',
        'America/Tortola' => 'अटलाण्टिक समयः (Tortola)',
        'America/Vancouver' => 'उत्तर अमेरिका: सन्धिप्रिय समयः (Vancouver)',
        'America/Winnipeg' => 'उत्तर अमेरिका: मध्य समयः (Winnipeg)',
        'America/Yakutat' => 'संयुक्त राज्य: समय: (Yakutat)',
        'Antarctica/Troll' => 'ग्रीनविच मीन समयः (Troll)',
        'Arctic/Longyearbyen' => 'मध्य यूरोपीय समयः (Longyearbyen)',
        'Asia/Amman' => 'पौर्व यूरोपीय समयः (Amman)',
        'Asia/Anadyr' => 'रष्यदेश: समय: (Anadyr)',
        'Asia/Barnaul' => 'रष्यदेश: समय: (Barnaul)',
        'Asia/Beirut' => 'पौर्व यूरोपीय समयः (Beirut)',
        'Asia/Calcutta' => 'भारतः समय: (Kolkata)',
        'Asia/Kolkata' => 'भारतः समय: (Kolkata)',
        'Asia/Chita' => 'रष्यदेश: समय: (Chita)',
        'Asia/Damascus' => 'पौर्व यूरोपीय समयः (Damascus)',
        'Asia/Famagusta' => 'पौर्व यूरोपीय समयः (Famagusta)',
        'Asia/Gaza' => 'पौर्व यूरोपीय समयः (Gaza)',
        'Asia/Hebron' => 'पौर्व यूरोपीय समयः (Hebron)',
        'Asia/Irkutsk' => 'रष्यदेश: समय: (Irkutsk)',
        'Asia/Kamchatka' => 'रष्यदेश: समय: (Kamchatka)',
        'Asia/Khandyga' => 'रष्यदेश: समय: (Khandyga)',
        'Asia/Krasnoyarsk' => 'रष्यदेश: समय: (Krasnoyarsk)',
        'Asia/Magadan' => 'रष्यदेश: समय: (Magadan)',
        'Asia/Nicosia' => 'पौर्व यूरोपीय समयः (Nicosia)',
        'Asia/Novokuznetsk' => 'रष्यदेश: समय: (Novokuznetsk)',
        'Asia/Novosibirsk' => 'रष्यदेश: समय: (Novosibirsk)',
        'Asia/Omsk' => 'रष्यदेश: समय: (Omsk)',
        'Asia/Sakhalin' => 'रष्यदेश: समय: (Sakhalin)',
        'Asia/Shanghai' => 'चीन: समय: (Shanghai)',
        'Asia/Srednekolymsk' => 'रष्यदेश: समय: (Srednekolymsk)',
        'Asia/Tokyo' => 'जापन: समय: (Tokyo)',
        'Asia/Tomsk' => 'रष्यदेश: समय: (Tomsk)',
        'Asia/Urumqi' => 'चीन: समय: (Ürümqi)',
        'Asia/Ust-Nera' => 'रष्यदेश: समय: (Ust-Nera)',
        'Asia/Vladivostok' => 'रष्यदेश: समय: (Vladivostok)',
        'Asia/Yakutsk' => 'रष्यदेश: समय: (Yakutsk)',
        'Asia/Yekaterinburg' => 'रष्यदेश: समय: (Yekaterinburg)',
        'Atlantic/Bermuda' => 'अटलाण्टिक समयः (Bermuda)',
        'Atlantic/Canary' => 'पाश्चात्य यूरोपीय समयः (Canarias)',
        'Atlantic/Faeroe' => 'पाश्चात्य यूरोपीय समयः (Faroe)',
        'Atlantic/Faroe' => 'पाश्चात्य यूरोपीय समयः (Faroe)',
        'Atlantic/Madeira' => 'पाश्चात्य यूरोपीय समयः (Madeira)',
        'Atlantic/Reykjavik' => 'ग्रीनविच मीन समयः (Reykjavik)',
        'Atlantic/St_Helena' => 'ग्रीनविच मीन समयः (St. Helena)',
        'Etc/GMT' => 'ग्रीनविच मीन समयः',
        'Etc/UTC' => 'समन्वितः वैश्विक समय:',
        'Europe/Amsterdam' => 'मध्य यूरोपीय समयः (Amsterdam)',
        'Europe/Andorra' => 'मध्य यूरोपीय समयः (Andorra)',
        'Europe/Astrakhan' => 'रष्यदेश: समय: (Astrakhan)',
        'Europe/Athens' => 'पौर्व यूरोपीय समयः (Athens)',
        'Europe/Belgrade' => 'मध्य यूरोपीय समयः (Belgrade)',
        'Europe/Berlin' => 'मध्य यूरोपीय समयः (Berlin)',
        'Europe/Bratislava' => 'मध्य यूरोपीय समयः (Bratislava)',
        'Europe/Brussels' => 'मध्य यूरोपीय समयः (Brussels)',
        'Europe/Bucharest' => 'पौर्व यूरोपीय समयः (Bucharest)',
        'Europe/Budapest' => 'मध्य यूरोपीय समयः (Budapest)',
        'Europe/Busingen' => 'मध्य यूरोपीय समयः (Büsingen)',
        'Europe/Chisinau' => 'पौर्व यूरोपीय समयः (Chișinău)',
        'Europe/Copenhagen' => 'मध्य यूरोपीय समयः (Copenhagen)',
        'Europe/Dublin' => 'ग्रीनविच मीन समयः (Dublin)',
        'Europe/Gibraltar' => 'मध्य यूरोपीय समयः (Gibraltar)',
        'Europe/Guernsey' => 'ग्रीनविच मीन समयः (Guernsey)',
        'Europe/Helsinki' => 'पौर्व यूरोपीय समयः (Helsinki)',
        'Europe/Isle_of_Man' => 'ग्रीनविच मीन समयः (Isle of Man)',
        'Europe/Jersey' => 'ग्रीनविच मीन समयः (Jersey)',
        'Europe/Kaliningrad' => 'पौर्व यूरोपीय समयः (Kaliningrad)',
        'Europe/Kiev' => 'पौर्व यूरोपीय समयः (Kyiv)',
        'Europe/Kyiv' => 'पौर्व यूरोपीय समयः (Kyiv)',
        'Europe/Kirov' => 'रष्यदेश: समय: (Kirov)',
        'Europe/Lisbon' => 'पाश्चात्य यूरोपीय समयः (Lisbon)',
        'Europe/Ljubljana' => 'मध्य यूरोपीय समयः (Ljubljana)',
        'Europe/London' => 'ग्रीनविच मीन समयः (London)',
        'Europe/Luxembourg' => 'मध्य यूरोपीय समयः (Luxembourg)',
        'Europe/Madrid' => 'मध्य यूरोपीय समयः (Madrid)',
        'Europe/Malta' => 'मध्य यूरोपीय समयः (Malta)',
        'Europe/Mariehamn' => 'पौर्व यूरोपीय समयः (Mariehamn)',
        'Europe/Monaco' => 'मध्य यूरोपीय समयः (Monaco)',
        'Europe/Moscow' => 'रष्यदेश: समय: (Moscow)',
        'Europe/Oslo' => 'मध्य यूरोपीय समयः (Oslo)',
        'Europe/Paris' => 'मध्य यूरोपीय समयः (Paris)',
        'Europe/Podgorica' => 'मध्य यूरोपीय समयः (Podgorica)',
        'Europe/Prague' => 'मध्य यूरोपीय समयः (Prague)',
        'Europe/Riga' => 'पौर्व यूरोपीय समयः (Riga)',
        'Europe/Rome' => 'मध्य यूरोपीय समयः (Rome)',
        'Europe/Samara' => 'रष्यदेश: समय: (Samara)',
        'Europe/San_Marino' => 'मध्य यूरोपीय समयः (San Marino)',
        'Europe/Sarajevo' => 'मध्य यूरोपीय समयः (Sarajevo)',
        'Europe/Saratov' => 'रष्यदेश: समय: (Saratov)',
        'Europe/Skopje' => 'मध्य यूरोपीय समयः (Skopje)',
        'Europe/Sofia' => 'पौर्व यूरोपीय समयः (Sofia)',
        'Europe/Stockholm' => 'मध्य यूरोपीय समयः (Stockholm)',
        'Europe/Tallinn' => 'पौर्व यूरोपीय समयः (Tallinn)',
        'Europe/Tirane' => 'मध्य यूरोपीय समयः (Tirana)',
        'Europe/Ulyanovsk' => 'रष्यदेश: समय: (Ulyanovsk)',
        'Europe/Vaduz' => 'मध्य यूरोपीय समयः (Vaduz)',
        'Europe/Vatican' => 'मध्य यूरोपीय समयः (Vatican)',
        'Europe/Vienna' => 'मध्य यूरोपीय समयः (Vienna)',
        'Europe/Vilnius' => 'पौर्व यूरोपीय समयः (Vilnius)',
        'Europe/Volgograd' => 'रष्यदेश: समय: (Volgograd)',
        'Europe/Warsaw' => 'मध्य यूरोपीय समयः (Warsaw)',
        'Europe/Zagreb' => 'मध्य यूरोपीय समयः (Zagreb)',
        'Europe/Zurich' => 'मध्य यूरोपीय समयः (Zurich)',
        'Pacific/Honolulu' => 'संयुक्त राज्य: समय: (Honolulu)',
    ],
    'Meta' => [
        'GmtFormat' => 'जी.एम.टी. %s',
    ],
];
