<?php

namespace Symfony\Component\ErrorHandler\Tests\Fixtures;

abstract class ReturnTypeGrandParent
{
    /**
     * @return string
     */
    public function returnTypeGrandParent()
    {
    }
}
