<?php

return [
    'Names' => [
        'GNF' => [
            'GNF',
            '𞤊𞤢𞤪𞤢𞤲 𞤘𞤭𞤲𞤫𞤲𞤳𞤮',
        ],
        'MRU' => [
            'UM',
            '𞤓𞤺𞤭𞤴𞤢 𞤃𞤮𞤪𞤭𞤼𞤢𞤲𞤭𞤴𞤢𞤲𞤳𞤮',
        ],
    ],
];
