<?php

return [
    'Names' => [
        'Adlm' => 'adlamî',
        'Aghb' => 'albanîya qefqasî',
        'Ahom' => 'ahomî',
        'Arab' => 'erebî',
        'Aran' => 'nestalîq',
        'Armi' => 'aramîya împeratorî',
        'Armn' => 'ermenî',
        'Avst' => 'avestayî',
        'Bali' => 'balînî',
        'Bamu' => 'bamumî',
        'Bass' => 'bassa vah',
        'Batk' => 'batakî',
        'Beng' => 'bengalî',
        'Berf' => 'beria erfe',
        'Bhks' => 'bhaîksukî',
        'Bopo' => 'bopomofo',
        'Brah' => 'brahmî',
        'Brai' => 'braille',
        'Bugi' => 'bugî',
        'Buhd' => 'buhid',
        'Cakm' => 'çakmayî',
        'Cans' => 'heceyên aborjînî yên kanadî yên yekgirtî',
        'Cari' => 'carian',
        'Cham' => 'çamî',
        'Cher' => 'çerokî',
        'Chrs' => 'xwarezmî',
        'Copt' => 'qiptî',
        'Cpmn' => 'qibris-mînoanî',
        'Cprt' => 'qibrisî',
        'Cyrl' => 'kirîlî',
        'Cyrs' => 'slavonîka krîlî ya dêrê ya kevn',
        'Deva' => 'devanagarî',
        'Diak' => 'dives akuruyî',
        'Dogr' => 'dograyî',
        'Dsrt' => 'deseret',
        'Dupl' => 'duployé stenografî',
        'Egyp' => 'hîyeroglîfên misirî',
        'Elba' => 'elbasan',
        'Elym' => 'elymaic',
        'Ethi' => 'etîyopîk',
        'Gara' => 'garayî',
        'Geor' => 'gurcî',
        'Glag' => 'glagolîtîk',
        'Gong' => 'gunjala gondî',
        'Gonm' => 'masaram gondî',
        'Goth' => 'gotîk',
        'Gran' => 'grantha',
        'Grek' => 'yûnanî',
        'Gujr' => 'gujeratî',
        'Gukh' => 'gurung khema',
        'Guru' => 'gurmuxî',
        'Hanb' => 'hanîya bi bopomofoyê',
        'Hang' => 'hangulî',
        'Hani' => 'hanî',
        'Hano' => 'hanunoo',
        'Hans' => 'sadekirî',
        'Hant' => 'edetî',
        'Hatr' => 'hatranî',
        'Hebr' => 'îbranî',
        'Hira' => 'hîraganayî',
        'Hluw' => 'hîyeroglîfên anadolî',
        'Hmng' => 'pahawh hmongî',
        'Hmnp' => 'nyiakeng puachue hmongî',
        'Hrkt' => 'nivîsên heceyî yên japonî',
        'Hung' => 'macarîya kevn',
        'Ital' => 'îtalîya kevn',
        'Jamo' => 'jamoyî',
        'Java' => 'javayî',
        'Jpan' => 'japonî',
        'Kali' => 'kayah liyî',
        'Kana' => 'katakanayî',
        'Kawi' => 'kawî',
        'Khar' => 'kharoshthi',
        'Khmr' => 'khmerî',
        'Khoj' => 'khojkî',
        'Kits' => 'nivîsa biçûk ya xîtanî',
        'Knda' => 'kannadayî',
        'Kore' => 'koreyî',
        'Krai' => 'kirat rai',
        'Kthi' => 'haithi',
        'Lana' => 'lannayî',
        'Laoo' => 'laoyî',
        'Latf' => 'latînîya frakturî',
        'Latg' => 'latînîya gaelîk',
        'Latn' => 'latînî',
        'Lepc' => 'lepçayî',
        'Limb' => 'lîmbuyî',
        'Lina' => 'lînear a',
        'Linb' => 'lînear b',
        'Lisu' => 'fraser',
        'Lyci' => 'lîkyayî',
        'Lydi' => 'lîdyayî',
        'Mahj' => 'mahajanî',
        'Maka' => 'makasarî',
        'Mand' => 'mandaean',
        'Mani' => 'manîheyî',
        'Marc' => 'marchen',
        'Medf' => 'medefaidrin',
        'Mend' => 'mende',
        'Merc' => 'destnivîsa meroytîk',
        'Mero' => 'meroytîk',
        'Mlym' => 'malayamî',
        'Modi' => 'modî',
        'Mong' => 'moxolî',
        'Mroo' => 'mro',
        'Mtei' => 'meitei mayekî',
        'Mult' => 'multanî',
        'Mymr' => 'myanmarî',
        'Nagm' => 'nag mundarî',
        'Nand' => 'nandînagarî',
        'Narb' => 'erebîya bakurî ya kevn',
        'Nbat' => 'nebatî',
        'Newa' => 'newayî',
        'Nkoo' => 'n’koyî',
        'Nshu' => 'nüshu',
        'Ogam' => 'ogham',
        'Olck' => 'ol chiki',
        'Onao' => 'ol onal',
        'Orkh' => 'orxun',
        'Orya' => 'odiayî',
        'Osge' => 'osageyî',
        'Osma' => 'osmanya',
        'Ougr' => 'oygurîya kevn',
        'Palm' => 'palmîrayî',
        'Pauc' => 'pau cin hau',
        'Perm' => 'permîka kevn',
        'Phag' => 'phags-pa',
        'Phli' => 'pehlewîya nivîskî',
        'Phlp' => 'pehlewîya psalter',
        'Phnx' => 'fenîkeyî',
        'Plrd' => 'pollard fonetîk',
        'Prti' => 'partîya nivîskî',
        'Qaag' => 'zawgyi',
        'Rjng' => 'rejang',
        'Rohg' => 'hanîfî',
        'Runr' => 'runîk',
        'Samr' => 'samarîtî',
        'Sarb' => 'erebîya başûrî ya kevn',
        'Saur' => 'sauraştrayî',
        'Sgnw' => 'zimanê îşaretê',
        'Shaw' => 'shavian',
        'Shrd' => 'sharada',
        'Sidd' => 'siddham',
        'Sidt' => 'sidetic',
        'Sind' => 'khudawadi',
        'Sinh' => 'sînhalayî',
        'Sogd' => 'sogdî',
        'Sogo' => 'sogdîya kevn',
        'Sora' => 'sora sompengî',
        'Soyo' => 'soyomboyî',
        'Sund' => 'sundanî',
        'Sunu' => 'sunuwarî',
        'Sylo' => 'syloti nagri',
        'Syrc' => 'siryanî',
        'Syre' => 'siryanîya estrangelo',
        'Syrj' => 'siryanîya rojavayî',
        'Syrn' => 'siryanîya rojhilatî',
        'Tagb' => 'tagbanwa',
        'Takr' => 'takrî',
        'Tale' => 'taî leyî',
        'Talu' => 'new tai lue',
        'Taml' => 'tamîlî',
        'Tang' => 'tangut',
        'Tavt' => 'tai viet',
        'Tayo' => 'tai yo',
        'Telu' => 'teluguyî',
        'Tfng' => 'tifinagh',
        'Tglg' => 'tagalogî',
        'Thaa' => 'thaanayî',
        'Thai' => 'tayî',
        'Tibt' => 'tîbetî',
        'Tirh' => 'tirhuta',
        'Tnsa' => 'tangsayî',
        'Todr' => 'todhrî',
        'Tols' => 'tolong sikî',
        'Toto' => 'totoyî',
        'Tutg' => 'tulu-tîgalarî',
        'Ugar' => 'ugarîtîk',
        'Vaii' => 'vaî',
        'Vith' => 'vithkuqi',
        'Wara' => 'varang kshiti',
        'Wcho' => 'wançoyî',
        'Xpeo' => 'farisîya kevn',
        'Xsux' => 'nivîsên bizmarî yên sumero-akadî',
        'Yezi' => 'êzidî',
        'Yiii' => 'yî',
        'Zanb' => 'meydana zanabazar',
        'Zinh' => 'irsî',
        'Zmth' => 'nîşandana matematîkî',
        'Zsye' => 'emojî',
        'Zsym' => 'sembol',
        'Zxxx' => 'nenivîskî',
        'Zyyy' => 'hevpar',
    ],
];
