<?php

return [
    'Names' => [
        'arn' => 'мапудунгун',
        'be' => 'бјелоруски',
        'bm' => 'бамананкан',
        'bn' => 'бангла',
        'ff' => 'фулах',
        'ht' => 'хаићански креолски',
        'lo' => 'лаошки',
        'moh' => 'мохок',
        'nqo' => 'н’ко',
        'shi' => 'јужни шилха',
        'xh' => 'исикоса',
        'zgh' => 'стандардни марокански тамашек',
        'zu' => 'исизулу',
    ],
    'LocalizedNames' => [],
];
