<?php

return [
    'Names' => [
        'aa' => 'Afar',
        'ab' => 'Abkasies',
        'ace' => 'Atsjenees',
        'ach' => 'Akoli',
        'ada' => 'Adangme',
        'ady' => 'Adyghe',
        'af' => 'Afrikaans',
        'agq' => 'Aghem',
        'ain' => 'Ainu',
        'ak' => 'Akan',
        'ale' => 'Aleut',
        'alt' => 'Suid-Altai',
        'am' => 'Amharies',
        'an' => 'Aragonees',
        'ann' => 'Obolo',
        'anp' => 'Angika',
        'ar' => 'Arabies',
        'arc' => 'Aramees',
        'arn' => 'Mapuche',
        'arp' => 'Arapaho',
        'ars' => 'Najdi-Arabies',
        'as' => 'Assamees',
        'asa' => 'Asu',
        'ast' => 'Asturies',
        'atj' => 'Atikamekw',
        'av' => 'Avaries',
        'awa' => 'Awadhi',
        'ay' => 'Aymara',
        'az' => 'Azerbeidjans',
        'ba' => 'Baskir',
        'bal' => 'Baloetsji',
        'ban' => 'Balinees',
        'bas' => 'Basaa',
        'be' => 'Belarussies',
        'bem' => 'Bemba',
        'bez' => 'Bena',
        'bg' => 'Bulgaars',
        'bgc' => 'Haryanvi',
        'bgn' => 'Wes-Balochi',
        'bho' => 'Bhojpuri',
        'bi' => 'Bislama',
        'bin' => 'Bini',
        'bla' => 'Siksika',
        'blo' => 'Anii',
        'bm' => 'Bambara',
        'bn' => 'Bengaals',
        'bo' => 'Tibettaans',
        'br' => 'Bretons',
        'brx' => 'Bodo',
        'bs' => 'Bosnies',
        'bug' => 'Buginees',
        'byn' => 'Blin',
        'ca' => 'Katalaans',
        'cay' => 'Cayuga',
        'ccp' => 'Tsjaakma',
        'ce' => 'Tsjetsjeens',
        'ceb' => 'Cebuano',
        'cgg' => 'Kiga',
        'ch' => 'Chamorro',
        'chk' => 'Chuukees',
        'chm' => 'Mari',
        'cho' => 'Choctaw',
        'chp' => 'Chipewyan',
        'chr' => 'Cherokees',
        'chy' => 'Cheyennees',
        'ckb' => 'Sorani',
        'clc' => 'Tzilkotin',
        'co' => 'Korsikaans',
        'cop' => 'Kopties',
        'crg' => 'Michif',
        'crj' => 'Suidoos-Cree',
        'crk' => 'Laagvlakte-Cree',
        'crl' => 'Noordoos-Cree',
        'crm' => 'Moose Cree',
        'crr' => 'Carolina-Algonkin',
        'crs' => 'Seselwa Franskreools',
        'cs' => 'Tsjeggies',
        'csw' => 'Swampy Cree',
        'cu' => 'Kerkslawies',
        'cv' => 'Chuvash',
        'cy' => 'Wallies',
        'da' => 'Deens',
        'dak' => 'Dakotaans',
        'dar' => 'Dakota',
        'dav' => 'Taita',
        'de' => 'Duits',
        'dgr' => 'Dogrib',
        'dje' => 'Zarma',
        'doi' => 'Dogri',
        'dsb' => 'Nedersorbies',
        'dua' => 'Duala',
        'dv' => 'Divehi',
        'dyo' => 'Jola-Fonyi',
        'dz' => 'Dzongkha',
        'dzg' => 'Dazaga',
        'ebu' => 'Embu',
        'ee' => 'Ewe',
        'efi' => 'Efik',
        'egy' => 'Antieke Egipties',
        'eka' => 'Ekajuk',
        'el' => 'Grieks',
        'en' => 'Engels',
        'eo' => 'Esperanto',
        'es' => 'Spaans',
        'et' => 'Estnies',
        'eu' => 'Baskies',
        'ewo' => 'Ewondo',
        'fa' => 'Persies',
        'ff' => 'Fulah',
        'fi' => 'Fins',
        'fil' => 'Filippyns',
        'fj' => 'Fidjiaans',
        'fo' => 'Faroëes',
        'fon' => 'Fon',
        'fr' => 'Frans',
        'frc' => 'Cajun',
        'frr' => 'Noord-Fries',
        'fur' => 'Friuliaans',
        'fy' => 'Fries',
        'ga' => 'Iers',
        'gaa' => 'Gaa',
        'gag' => 'Gagauz',
        'gan' => 'Gan-Sjinees',
        'gd' => 'Skotse Gallies',
        'gez' => 'Geez',
        'gil' => 'Gilbertees',
        'gl' => 'Galisies',
        'gn' => 'Guarani',
        'gor' => 'Gorontalo',
        'got' => 'Goties',
        'grc' => 'Antieke Grieks',
        'gsw' => 'Switserse Duits',
        'gu' => 'Goedjarati',
        'guz' => 'Gusii',
        'gv' => 'Manx',
        'gwi' => 'Gwichʼin',
        'ha' => 'Hausa',
        'hai' => 'Haida',
        'hak' => 'Hakka-Sjinees',
        'haw' => 'Hawais',
        'hax' => 'Suid-Haida',
        'he' => 'Hebreeus',
        'hi' => 'Hindi',
        'hil' => 'Hiligaynon',
        'hit' => 'Hetities',
        'hmn' => 'Hmong',
        'hr' => 'Kroaties',
        'hsb' => 'Oppersorbies',
        'hsn' => 'Xiang-Sjinees',
        'ht' => 'Haïtiaans',
        'hu' => 'Hongaars',
        'hup' => 'Hupa',
        'hur' => 'Halkomelem',
        'hy' => 'Armeens',
        'hz' => 'Herero',
        'ia' => 'Interlingua',
        'iba' => 'Ibanees',
        'ibb' => 'Ibibio',
        'id' => 'Indonesies',
        'ie' => 'Interlingue',
        'ig' => 'Igbo',
        'ii' => 'Sichuan Yi',
        'ikt' => 'Wes-Kanadese Inoektitoet',
        'ilo' => 'Iloko',
        'inh' => 'Ingush',
        'io' => 'Ido',
        'is' => 'Yslands',
        'it' => 'Italiaans',
        'iu' => 'Inoektitoet',
        'ja' => 'Japannees',
        'jbo' => 'Lojban',
        'jgo' => 'Ngomba',
        'jmc' => 'Machame',
        'jv' => 'Javaans',
        'ka' => 'Georgies',
        'kab' => 'Kabyle',
        'kac' => 'Kachin',
        'kaj' => 'Jju',
        'kam' => 'Kamba',
        'kbd' => 'Kabardiaans',
        'kcg' => 'Tyap',
        'kde' => 'Makonde',
        'kea' => 'Kabuverdianu',
        'kfo' => 'Koro',
        'kg' => 'Kongolees',
        'kgp' => 'Kaingang',
        'kha' => 'Khasi',
        'khq' => 'Koyra Chiini',
        'ki' => 'Kikuyu',
        'kj' => 'Kuanyama',
        'kk' => 'Kazaks',
        'kkj' => 'Kako',
        'kl' => 'Kalaallisut',
        'kln' => 'Kalenjin',
        'km' => 'Khmer',
        'kmb' => 'Kimbundu',
        'kn' => 'Kannada',
        'ko' => 'Koreaans',
        'koi' => 'Komi-Permyaks',
        'kok' => 'Konkani',
        'kpe' => 'Kpellees',
        'kr' => 'Kanuri',
        'krc' => 'Karachay-Balkar',
        'krl' => 'Karelies',
        'kru' => 'Kurukh',
        'ks' => 'Kasjmiri',
        'ksb' => 'Shambala',
        'ksf' => 'Bafia',
        'ksh' => 'Keuls',
        'ku' => 'Koerdies',
        'kum' => 'Kumyk',
        'kv' => 'Komi',
        'kw' => 'Kornies',
        'kwk' => 'Kwak’wala',
        'kxv' => 'Kuvi',
        'ky' => 'Kirgisies',
        'la' => 'Latyn',
        'lad' => 'Ladino',
        'lag' => 'Langi',
        'lb' => 'Luxemburgs',
        'lez' => 'Lezghies',
        'lg' => 'Ganda',
        'li' => 'Limburgs',
        'lij' => 'Liguries',
        'lil' => 'Lillooet',
        'lkt' => 'Lakota',
        'lmo' => 'Lombardies',
        'ln' => 'Lingaals',
        'lo' => 'Lao',
        'lou' => 'Louisiana Kreool',
        'loz' => 'Lozi',
        'lrc' => 'Noord-Luri',
        'lsm' => 'Saamia',
        'lt' => 'Litaus',
        'lu' => 'Luba-Katanga',
        'lua' => 'Luba-Lulua',
        'lun' => 'Lunda',
        'luo' => 'Luo',
        'lus' => 'Mizo',
        'luy' => 'Luyia',
        'lv' => 'Letties',
        'mad' => 'Madurees',
        'mag' => 'Magahi',
        'mai' => 'Maithili',
        'mak' => 'Makasar',
        'mas' => 'Masai',
        'mdf' => 'Moksha',
        'men' => 'Mende',
        'mer' => 'Meru',
        'mfe' => 'Morisjen',
        'mg' => 'Malgassies',
        'mgh' => 'Makhuwa-Meetto',
        'mgo' => 'Meta’',
        'mh' => 'Marshallees',
        'mi' => 'Maori',
        'mic' => 'Micmac',
        'min' => 'Minangkabaus',
        'mk' => 'Masedonies',
        'ml' => 'Malabaars',
        'mn' => 'Mongools',
        'mni' => 'Manipuri',
        'moe' => 'Innu-aimun',
        'moh' => 'Mohawk',
        'mos' => 'Mossi',
        'mr' => 'Marathi',
        'ms' => 'Maleis',
        'mt' => 'Maltees',
        'mua' => 'Mundang',
        'mus' => 'Kreek',
        'mwl' => 'Mirandees',
        'my' => 'Birmaans',
        'myv' => 'Erzya',
        'mzn' => 'Masanderani',
        'na' => 'Nauru',
        'nan' => 'Min Nan-Sjinees',
        'nap' => 'Neapolitaans',
        'naq' => 'Nama',
        'nb' => 'Boeknoors',
        'nd' => 'Noord-Ndebele',
        'nds' => 'Nederduits',
        'ne' => 'Nepalees',
        'new' => 'Newari',
        'ng' => 'Ndonga',
        'nia' => 'Nias',
        'niu' => 'Niueaans',
        'nl' => 'Nederlands',
        'nmg' => 'Kwasio',
        'nn' => 'Nuwe Noors',
        'nnh' => 'Ngiemboon',
        'no' => 'Noors',
        'nog' => 'Nogai',
        'nqo' => 'N’Ko',
        'nr' => 'Suid-Ndebele',
        'nso' => 'Noord-Sotho',
        'nus' => 'Nuer',
        'nv' => 'Navajo',
        'ny' => 'Nyanja',
        'nyn' => 'Nyankole',
        'oc' => 'Oksitaans',
        'ojb' => 'Noordwes-Ojibwa',
        'ojc' => 'Sentraal-Ojibwa',
        'ojs' => 'Oji-Cree',
        'ojw' => 'Wes-Ojibwa',
        'oka' => 'Okanagan',
        'om' => 'Oromo',
        'or' => 'Odia',
        'os' => 'Osseties',
        'pa' => 'Pandjabi',
        'pag' => 'Pangasinan',
        'pam' => 'Pampanga',
        'pap' => 'Papiamento',
        'pau' => 'Palauaans',
        'pcm' => 'Nigeriese Pidgin',
        'phn' => 'Fenisies',
        'pis' => 'Pijin',
        'pl' => 'Pools',
        'pqm' => 'Maliseet-Passamaquoddy',
        'prg' => 'Pruisies',
        'ps' => 'Pasjtoe',
        'pt' => 'Portugees',
        'qu' => 'Quechua',
        'quc' => 'K’iche’',
        'raj' => 'Rajasthani',
        'rap' => 'Rapanui',
        'rar' => 'Rarotongaans',
        'rhg' => 'Rohingya',
        'rm' => 'Reto-Romaans',
        'rn' => 'Rundi',
        'ro' => 'Roemeens',
        'rof' => 'Rombo',
        'ru' => 'Russies',
        'rup' => 'Aromanies',
        'rw' => 'Rwandees',
        'rwk' => 'Rwa',
        'sa' => 'Sanskrit',
        'sad' => 'Sandawees',
        'sah' => 'Jakoeties',
        'saq' => 'Samburu',
        'sat' => 'Santalies',
        'sba' => 'Ngambay',
        'sbp' => 'Sangu',
        'sc' => 'Sardinies',
        'scn' => 'Sisiliaans',
        'sco' => 'Skots',
        'sd' => 'Sindhi',
        'sdh' => 'Suid-Koerdies',
        'se' => 'Noord-Sami',
        'seh' => 'Sena',
        'ses' => 'Koyraboro Senni',
        'sg' => 'Sango',
        'sh' => 'Serwo-Kroaties',
        'shi' => 'Tachelhit',
        'shn' => 'Shan',
        'si' => 'Sinhala',
        'sk' => 'Slowaaks',
        'sl' => 'Sloweens',
        'slh' => 'Suid-Lushootseed',
        'sm' => 'Samoaans',
        'sma' => 'Suid-Sami',
        'smj' => 'Lule Sami',
        'smn' => 'Inari Sami',
        'sms' => 'Skolt Sami',
        'sn' => 'Shona',
        'snk' => 'Soninke',
        'so' => 'Somalies',
        'sq' => 'Albanees',
        'sr' => 'Serwies',
        'srn' => 'Sranan Tongo',
        'ss' => 'Swazi',
        'ssy' => 'Saho',
        'st' => 'Suid-Sotho',
        'str' => 'Straits Salish',
        'su' => 'Sundanees',
        'suk' => 'Sukuma',
        'sv' => 'Sweeds',
        'sw' => 'Swahili',
        'swb' => 'Comoraans',
        'syr' => 'Siries',
        'szl' => 'Silesies',
        'ta' => 'Tamil',
        'tce' => 'Suid-Tutchone',
        'te' => 'Teloegoe',
        'tem' => 'Timne',
        'teo' => 'Teso',
        'tet' => 'Tetoem',
        'tg' => 'Tadjiks',
        'tgx' => 'Tagish',
        'th' => 'Thai',
        'tht' => 'Tahltan',
        'ti' => 'Tigrinja',
        'tig' => 'Tigre',
        'tk' => 'Turkmeens',
        'tlh' => 'Klingon',
        'tli' => 'Tlingit',
        'tn' => 'Tswana',
        'to' => 'Tongaans',
        'tok' => 'Toki Pona',
        'tpi' => 'Tok Pisin',
        'tr' => 'Turks',
        'trv' => 'Taroko',
        'ts' => 'Tsonga',
        'tt' => 'Tataars',
        'ttm' => 'Noord-Tutchone',
        'tum' => 'Toemboeka',
        'tvl' => 'Tuvalu',
        'tw' => 'Twi',
        'twq' => 'Tasawaq',
        'ty' => 'Tahities',
        'tyv' => 'Tuvinees',
        'tzm' => 'Sentraal-Atlas-Tamazight',
        'udm' => 'Udmurt',
        'ug' => 'Uighur',
        'uk' => 'Oekraïens',
        'umb' => 'Umbundu',
        'ur' => 'Oerdoe',
        'uz' => 'Oesbekies',
        'vai' => 'Vai',
        've' => 'Venda',
        'vec' => 'Venesiaans',
        'vi' => 'Viëtnamees',
        'vmw' => 'Makhuwa',
        'vo' => 'Volapük',
        'vun' => 'Vunjo',
        'wa' => 'Walloon',
        'wae' => 'Walser',
        'wal' => 'Wolaytta',
        'war' => 'Waray',
        'wbp' => 'Warlpiri',
        'wo' => 'Wolof',
        'wuu' => 'Wu-Sjinees',
        'xal' => 'Kalmyk',
        'xh' => 'Xhosa',
        'xnr' => 'Kangri',
        'xog' => 'Soga',
        'yav' => 'Yangben',
        'ybb' => 'Yemba',
        'yi' => 'Jiddisj',
        'yo' => 'Joroeba',
        'yrl' => 'Nheengatu',
        'yue' => 'Kantonees',
        'za' => 'Zhuang',
        'zgh' => 'Standaard Marokkaanse Tamazight',
        'zh' => 'Chinees',
        'zu' => 'Zoeloe',
        'zun' => 'Zuni',
        'zza' => 'Zaza',
    ],
    'LocalizedNames' => [
        'ar_001' => 'Moderne Standaardarabies',
        'fa_AF' => 'Dari',
        'nds_NL' => 'Nedersaksies',
        'nl_BE' => 'Vlaams',
    ],
];
