<?php

return [
    'Names' => [
        'Adlm' => 'adlam pismo',
        'Afak' => 'afaka pismo',
        'Arab' => 'arapsko pismo',
        'Aran' => 'nastaliq pismo',
        'Armi' => 'aramejsko pismo',
        'Armn' => 'armensko pismo',
        'Avst' => 'avestansko pismo',
        'Bali' => 'balijsko pismo',
        'Bamu' => 'bamum pismo',
        'Bass' => 'bassa vah pismo',
        'Batk' => 'batak pismo',
        'Beng' => 'bengalsko pismo',
        'Blis' => 'blissymbols',
        'Bopo' => 'bopomofo pismo',
        'Brah' => 'brahmi pismo',
        'Brai' => 'brajica',
        'Bugi' => 'buginsko pismo',
        'Buhd' => 'buhid pismo',
        'Cakm' => 'čakmansko pismo',
        'Cans' => 'unificirani kanadski aboriđinski slogovi',
        'Cari' => 'karijsko pismo',
        'Cham' => 'čamsko pismo',
        'Cher' => 'čeroki pismo',
        'Cirt' => 'cirth pismo',
        'Copt' => 'koptsko pismo',
        'Cprt' => 'cypriot pismo',
        'Cyrl' => 'ćirilica',
        'Cyrs' => 'staroslavenska crkvena čirilica',
        'Deva' => 'devangari pismo',
        'Dsrt' => 'deseret pismo',
        'Egyd' => 'egipatsko narodno pismo',
        'Egyh' => 'egipatsko hijeratsko pismo',
        'Egyp' => 'egipatski hijeroglifi',
        'Ethi' => 'etiopsko pismo',
        'Geok' => 'gruzijsko khutsuri pismo',
        'Geor' => 'gruzijsko pismo',
        'Glag' => 'glagoljica',
        'Goth' => 'gotičko pismo',
        'Gran' => 'grantha pismo',
        'Grek' => 'grčko pismo',
        'Gujr' => 'gudžaratsko pismo',
        'Guru' => 'gurmukhi pismo',
        'Hanb' => 'hanb pismo',
        'Hang' => 'hangul pismo',
        'Hani' => 'hansko pismo',
        'Hano' => 'hanunoo pismo',
        'Hans' => 'pojednostavljeno pismo',
        'Hant' => 'tradicionalno pismo',
        'Hebr' => 'hebrejsko pismo',
        'Hira' => 'hiragana pismo',
        'Hluw' => 'anatolijski hijeroglifi',
        'Hmng' => 'pahawh hmong pismo',
        'Hrkt' => 'japansko slogovno pismo',
        'Hung' => 'staro mađarsko pismo',
        'Inds' => 'indijsko pismo',
        'Ital' => 'staro talijansko pismo',
        'Jamo' => 'jamo pismo',
        'Java' => 'javansko pismo',
        'Jpan' => 'japansko pismo',
        'Jurc' => 'jurchen pismo',
        'Kali' => 'kayah li pismo',
        'Kana' => 'katakana pismo',
        'Khar' => 'kharoshthi pismo',
        'Khmr' => 'kmersko pismo',
        'Khoj' => 'khojki pismo',
        'Knda' => 'kannada pismo',
        'Kore' => 'korejsko pismo',
        'Kpel' => 'kpelle pismo',
        'Kthi' => 'kaithi pismo',
        'Lana' => 'lanna pismo',
        'Laoo' => 'laosko pismo',
        'Latf' => 'fraktur latinica',
        'Latg' => 'keltska latinica',
        'Latn' => 'latinica',
        'Lepc' => 'lepcha pismo',
        'Limb' => 'limbu pismo',
        'Lina' => 'linear A pismo',
        'Linb' => 'linear B pismo',
        'Lisu' => 'fraser pismo',
        'Loma' => 'loma pismo',
        'Lyci' => 'likijsko pismo',
        'Lydi' => 'lidijsko pismo',
        'Mand' => 'mandai pismo',
        'Mani' => 'manihejsko pismo',
        'Maya' => 'majanski hijeroglifi',
        'Mend' => 'mende pismo',
        'Merc' => 'meroitski kurziv',
        'Mero' => 'meroitic pismo',
        'Mlym' => 'malajalamsko pismo',
        'Mong' => 'mongolsko pismo',
        'Moon' => 'moon pismo',
        'Mroo' => 'mro pismo',
        'Mtei' => 'meitei mayek pismo',
        'Mymr' => 'mjanmarsko pismo',
        'Narb' => 'staro sjevernoarapsko pismo',
        'Nbat' => 'nabatejsko pismo',
        'Nkgb' => 'naxi geba pismo',
        'Nkoo' => 'n’ko pismo',
        'Nshu' => 'nushu pismo',
        'Ogam' => 'ogham pismo',
        'Olck' => 'ol chiki pismo',
        'Orkh' => 'orkhon pismo',
        'Orya' => 'orijsko pismo',
        'Osma' => 'osmanya pismo',
        'Palm' => 'palmyrene pismo',
        'Perm' => 'staro permic pismo',
        'Phag' => 'phags-pa pismo',
        'Phli' => 'pisani pahlavi',
        'Phlp' => 'psalter pahlavi',
        'Phlv' => 'pahlavi pismo',
        'Phnx' => 'feničko pismo',
        'Plrd' => 'pollard fonetsko pismo',
        'Prti' => 'pisani parthian',
        'Qaag' => 'zawgyi',
        'Rjng' => 'rejang pismo',
        'Rohg' => 'hanifi pismo',
        'Roro' => 'rongorongo pismo',
        'Runr' => 'runsko pismo',
        'Samr' => 'samaritansko pismo',
        'Sara' => 'sarati pismo',
        'Sarb' => 'staro južnoarapsko pismo',
        'Saur' => 'saurashtra pismo',
        'Sgnw' => 'znakovno pismo',
        'Shaw' => 'shavian pismo',
        'Shrd' => 'sharada pismo',
        'Sind' => 'khudawadi pismo',
        'Sinh' => 'sinhaleško pismo',
        'Sora' => 'sora sompeng pismo',
        'Sund' => 'sundansko pismo',
        'Sylo' => 'syloti nagri pismo',
        'Syrc' => 'sirijsko pismo',
        'Syre' => 'sirijsko estrangelo pismo',
        'Syrj' => 'pismo zapadne Sirije',
        'Syrn' => 'pismo istočne Sirije',
        'Tagb' => 'tagbanwa pismo',
        'Takr' => 'takri pismo',
        'Tale' => 'tai le pismo',
        'Talu' => 'novo tai lue pismo',
        'Taml' => 'tamilsko pismo',
        'Tang' => 'tangut pismo',
        'Tavt' => 'tai viet pismo',
        'Telu' => 'teluško pismo',
        'Teng' => 'tengwar pismo',
        'Tfng' => 'tifinagh pismo',
        'Tglg' => 'tagalog pismo',
        'Thaa' => 'thaana pismo',
        'Thai' => 'tajsko pismo',
        'Tibt' => 'tibetansko pismo',
        'Tirh' => 'tirhuta pismo',
        'Ugar' => 'ugaritsko pismo',
        'Vaii' => 'vai pismo',
        'Visp' => 'Visible Speech',
        'Wara' => 'varang kshiti pismo',
        'Wole' => 'woleai pismo',
        'Xpeo' => 'staro perzijsko pismo',
        'Xsux' => 'sumersko-akadsko cuneiform pismo',
        'Yiii' => 'yi pismo',
        'Zinh' => 'nasljedno pismo',
        'Zmth' => 'matematičko znakovlje',
        'Zsye' => 'emotikoni',
        'Zsym' => 'simboli',
        'Zxxx' => 'jezik bez pismenosti',
        'Zyyy' => 'zajedničko pismo',
    ],
];
