<?php

return [
    'Names' => [
        'ADP' => [
            'ADP',
            'Πεσέτα Ανδόρας',
        ],
        'AED' => [
            'AED',
            'Ντιράμ Ηνωμένων Αραβικών Εμιράτων',
        ],
        'AFA' => [
            'AFA',
            'Αφγανί Αφγανιστάν (1927–2002)',
        ],
        'AFN' => [
            'AFN',
            'Αφγάνι Αφγανιστάν',
        ],
        'ALL' => [
            'ALL',
            'Λεκ Αλβανίας',
        ],
        'AMD' => [
            'AMD',
            'Ντραμ Αρμενίας',
        ],
        'ANG' => [
            'ANG',
            'Γκίλντα Ολλανδικών Αντιλλών',
        ],
        'AOA' => [
            'AOA',
            'Κουάνζα Ανγκόλας',
        ],
        'AOK' => [
            'AOK',
            'Κουάνζα Ανγκόλας (1977–1990)',
        ],
        'AON' => [
            'AON',
            'Νέα Κουάνζα Ανγκόλας (1990–2000)',
        ],
        'ARA' => [
            'ARA',
            'Ωστράλ Αργετινής',
        ],
        'ARP' => [
            'ARP',
            'Πέσο Αργεντινής (1983–1985)',
        ],
        'ARS' => [
            'ARS',
            'Πέσο Αργεντινής',
        ],
        'ATS' => [
            'ATS',
            'Σελίνι Αυστρίας',
        ],
        'AUD' => [
            'A$',
            'Δολάριο Αυστραλίας',
        ],
        'AWG' => [
            'AWG',
            'Φλορίνι Αρούμπας',
        ],
        'AZM' => [
            'AZM',
            'Μανάτ Αζερμπαϊτζάν (1993–2006)',
        ],
        'AZN' => [
            'AZN',
            'Μανάτ Αζερμπαϊτζάν',
        ],
        'BAD' => [
            'BAD',
            'Δηνάριο Βοσνίας-Ερζεγοβίνης',
        ],
        'BAM' => [
            'BAM',
            'Μετατρέψιμο Μάρκο Βοσνίας-Ερζεγοβίνης',
        ],
        'BBD' => [
            'BBD',
            'Δολάριο Μπαρμπέιντος',
        ],
        'BDT' => [
            'BDT',
            'Τάκα Μπαγκλαντές',
        ],
        'BEC' => [
            'BEC',
            'Φράγκο Βελγίου (μετατρέψιμο)',
        ],
        'BEF' => [
            'BEF',
            'Φράγκο Βελγίου',
        ],
        'BEL' => [
            'BEL',
            'Φράγκο Βελγίου (οικονομικό)',
        ],
        'BGL' => [
            'BGL',
            'Μεταλλικό Λεβ Βουλγαρίας',
        ],
        'BGN' => [
            'BGN',
            'Λεβ Βουλγαρίας',
        ],
        'BHD' => [
            'BHD',
            'Δηνάριο Μπαχρέιν',
        ],
        'BIF' => [
            'BIF',
            'Φράγκο Μπουρούντι',
        ],
        'BMD' => [
            'BMD',
            'Δολάριο Βερμούδων',
        ],
        'BND' => [
            'BND',
            'Δολάριο Μπρουνέι',
        ],
        'BOB' => [
            'BOB',
            'Μπολιβιάνο Βολιβίας',
        ],
        'BOP' => [
            'BOP',
            'Πέσο Βολιβίας',
        ],
        'BOV' => [
            'BOV',
            'Μβδολ Βολιβίας',
        ],
        'BRB' => [
            'BRB',
            'Νέο Κρουζιέρο Βραζιλίας (1967–1986)',
        ],
        'BRC' => [
            'BRC',
            'Κρουζάντο Βραζιλίας',
        ],
        'BRE' => [
            'BRE',
            'Κρουζιέρο Βραζιλίας (1990–1993)',
        ],
        'BRL' => [
            'R$',
            'Ρεάλ Βραζιλίας',
        ],
        'BRN' => [
            'BRN',
            'Νέο Κρουζάντο Βραζιλίας',
        ],
        'BRR' => [
            'BRR',
            'Κρουζιέρο Βραζιλίας',
        ],
        'BSD' => [
            'BSD',
            'Δολάριο Μπαχαμών',
        ],
        'BTN' => [
            'BTN',
            'Νγκούλτρουμ Μπουτάν',
        ],
        'BUK' => [
            'BUK',
            'Κιατ Βιρμανίας',
        ],
        'BWP' => [
            'BWP',
            'Πούλα Μποτσουάνας',
        ],
        'BYB' => [
            'BYB',
            'Νέο Ρούβλι Λευκορωσίας (1994–1999)',
        ],
        'BYN' => [
            'BYN',
            'Ρούβλι Λευκορωσίας',
        ],
        'BYR' => [
            'BYR',
            'Ρούβλι Λευκορωσίας (2000–2016)',
        ],
        'BZD' => [
            'BZD',
            'Δολάριο Μπελίζ',
        ],
        'CAD' => [
            'CA$',
            'Δολάριο Καναδά',
        ],
        'CDF' => [
            'CDF',
            'Φράγκο Κονγκό',
        ],
        'CHE' => [
            'CHE',
            'Ευρώ WIR',
        ],
        'CHF' => [
            'CHF',
            'Φράγκο Ελβετίας',
        ],
        'CHW' => [
            'CHW',
            'Φράγκο WIR',
        ],
        'CLF' => [
            'CLF',
            'Ουνιδάδες ντε φομέντο Χιλής',
        ],
        'CLP' => [
            'CLP',
            'Πέσο Χιλής',
        ],
        'CNH' => [
            'CNH',
            'Γουάν Κίνας (υπεράκτιο)',
        ],
        'CNY' => [
            'CN¥',
            'Γουάν Κίνας',
        ],
        'COP' => [
            'COP',
            'Πέσο Κολομβίας',
        ],
        'CRC' => [
            'CRC',
            'Κολόν Κόστα Ρίκα',
        ],
        'CSD' => [
            'CSD',
            'Παλαιό Δηνάριο Σερβίας',
        ],
        'CSK' => [
            'CSK',
            'Σκληρή Κορόνα Τσεχοσλοβακίας',
        ],
        'CUC' => [
            'CUC',
            'Μετατρέψιμο πέσο Κούβας',
        ],
        'CUP' => [
            'CUP',
            'Πέσο Κούβας',
        ],
        'CVE' => [
            'CVE',
            'Εσκούδο Πράσινου Ακρωτηρίου',
        ],
        'CYP' => [
            'CYP',
            'Λίρα Κύπρου',
        ],
        'CZK' => [
            'CZK',
            'Κορόνα Τσεχίας',
        ],
        'DDM' => [
            'DDM',
            'Οστμάρκ Ανατολικής Γερμανίας',
        ],
        'DEM' => [
            'DEM',
            'Μάρκο Γερμανίας',
        ],
        'DJF' => [
            'DJF',
            'Φράγκο Τζιμπουτί',
        ],
        'DKK' => [
            'DKK',
            'Κορόνα Δανίας',
        ],
        'DOP' => [
            'DOP',
            'Πέσο Δομινικανής Δημοκρατίας',
        ],
        'DZD' => [
            'DZD',
            'Δηνάριο Αλγερίας',
        ],
        'ECS' => [
            'ECS',
            'Σούκρε Εκουαδόρ',
        ],
        'EEK' => [
            'EEK',
            'Κορόνα Εσθονίας',
        ],
        'EGP' => [
            'EGP',
            'Λίρα Αιγύπτου',
        ],
        'ERN' => [
            'ERN',
            'Νάκφα Ερυθραίας',
        ],
        'ESA' => [
            'ESA',
            'πεσέτα Ισπανίας (λογαριασμός Α)',
        ],
        'ESB' => [
            'ESB',
            'πεσέτα Ισπανίας (μετατρέψιμος λογαριασμός)',
        ],
        'ESP' => [
            'ESP',
            'Πεσέτα Ισπανίας',
        ],
        'ETB' => [
            'ETB',
            'Μπιρ Αιθιοπίας',
        ],
        'EUR' => [
            '€',
            'Ευρώ',
        ],
        'FIM' => [
            'FIM',
            'Μάρκο Φινλανδίας',
        ],
        'FJD' => [
            'FJD',
            'Δολάριο Φίτζι',
        ],
        'FKP' => [
            'FKP',
            'Λίρα Νήσων Φόκλαντ',
        ],
        'FRF' => [
            'FRF',
            'Φράγκο Γαλλίας',
        ],
        'GBP' => [
            '£',
            'Λίρα Στερλίνα Βρετανίας',
        ],
        'GEK' => [
            'GEK',
            'Κούπον Λάρι Γεωργίας',
        ],
        'GEL' => [
            'GEL',
            'Λάρι Γεωργίας',
        ],
        'GHC' => [
            'GHC',
            'Σέντι Γκάνας (1979–2007)',
        ],
        'GHS' => [
            'GHS',
            'Σέντι Γκάνας',
        ],
        'GIP' => [
            'GIP',
            'Λίρα Γιβραλτάρ',
        ],
        'GMD' => [
            'GMD',
            'Νταλάσι Γκάμπιας',
        ],
        'GNF' => [
            'GNF',
            'Φράγκο Γουινέας',
        ],
        'GNS' => [
            'GNS',
            'Συλί Γουινέας',
        ],
        'GQE' => [
            'GQE',
            'Εκγουέλε Ισημερινής Γουινέας',
        ],
        'GRD' => [
            'Δρχ',
            'Δραχμή Ελλάδας',
        ],
        'GTQ' => [
            'GTQ',
            'Κουετσάλ Γουατεμάλας',
        ],
        'GWE' => [
            'GWE',
            'Γκινέα Εσκούδο Πορτογαλίας',
        ],
        'GWP' => [
            'GWP',
            'Πέσο Γουινέας-Μπισάου',
        ],
        'GYD' => [
            'GYD',
            'Δολάριο Γουιάνας',
        ],
        'HKD' => [
            'HK$',
            'Δολάριο Χονγκ Κονγκ',
        ],
        'HNL' => [
            'HNL',
            'Λεμπίρα Ονδούρας',
        ],
        'HRD' => [
            'HRD',
            'Δηνάριο Κροατίας',
        ],
        'HRK' => [
            'HRK',
            'Κούνα Κροατίας',
        ],
        'HTG' => [
            'HTG',
            'Γκουρντ Αϊτής',
        ],
        'HUF' => [
            'HUF',
            'Φιορίνι Ουγγαρίας',
        ],
        'IDR' => [
            'IDR',
            'Ρουπία Ινδονησίας',
        ],
        'IEP' => [
            'IEP',
            'Λίρα Ιρλανδίας',
        ],
        'ILP' => [
            'ILP',
            'Λίρα Ισραήλ',
        ],
        'ILR' => [
            'ILR',
            'παλιό σεκέλ Ισραήλ',
        ],
        'ILS' => [
            '₪',
            'Νέο Σέκελ Ισραήλ',
        ],
        'INR' => [
            '₹',
            'Ρουπία Ινδίας',
        ],
        'IQD' => [
            'IQD',
            'Δηνάριο Ιράκ',
        ],
        'IRR' => [
            'IRR',
            'Ριάλ Ιράν',
        ],
        'ISJ' => [
            'ISJ',
            'Παλιά κορόνα Ισλανδίας',
        ],
        'ISK' => [
            'ISK',
            'Κορόνα Ισλανδίας',
        ],
        'ITL' => [
            'ITL',
            'Λιρέτα Ιταλίας',
        ],
        'JMD' => [
            'JMD',
            'Δολάριο Τζαμάικας',
        ],
        'JOD' => [
            'JOD',
            'Δηνάριο Ιορδανίας',
        ],
        'JPY' => [
            'JP¥',
            'Γιεν Ιαπωνίας',
        ],
        'KES' => [
            'KES',
            'Σελίνι Κένυας',
        ],
        'KGS' => [
            'KGS',
            'Σομ Κιργιζίας',
        ],
        'KHR' => [
            'KHR',
            'Ρίελ Καμπότζης',
        ],
        'KMF' => [
            'KMF',
            'Φράγκο Κομορών',
        ],
        'KPW' => [
            'KPW',
            'Γουόν Βόρειας Κορέας',
        ],
        'KRW' => [
            '₩',
            'Γουόν Νότιας Κορέας',
        ],
        'KWD' => [
            'KWD',
            'Δηνάριο Κουβέιτ',
        ],
        'KYD' => [
            'KYD',
            'Δολάριο Νήσων Κέιμαν',
        ],
        'KZT' => [
            'KZT',
            'Τένγκε Καζακστάν',
        ],
        'LAK' => [
            'LAK',
            'Κιπ Λάος',
        ],
        'LBP' => [
            'LBP',
            'Λίρα Λιβάνου',
        ],
        'LKR' => [
            'LKR',
            'Ρουπία Σρι Λάνκα',
        ],
        'LRD' => [
            'LRD',
            'Δολάριο Λιβερίας',
        ],
        'LSL' => [
            'LSL',
            'Λότι Λεσότο',
        ],
        'LTL' => [
            'LTL',
            'Λίτα Λιθουανίας',
        ],
        'LTT' => [
            'LTT',
            'Ταλόνας Λιθουανίας',
        ],
        'LUC' => [
            'LUC',
            'Μετατρέψιμο Φράγκο Λουξεμβούργου',
        ],
        'LUF' => [
            'LUF',
            'Φράγκο Λουξεμβούργου',
        ],
        'LUL' => [
            'LUL',
            'Οικονομικό Φράγκο Λουξεμβούργου',
        ],
        'LVL' => [
            'LVL',
            'Λατς Λετονίας',
        ],
        'LVR' => [
            'LVR',
            'Ρούβλι Λετονίας',
        ],
        'LYD' => [
            'LYD',
            'Δηνάριο Λιβύης',
        ],
        'MAD' => [
            'MAD',
            'Ντιράμ Μαρόκου',
        ],
        'MAF' => [
            'MAF',
            'Φράγκο Μαρόκου',
        ],
        'MDL' => [
            'MDL',
            'Λέου Μολδαβίας',
        ],
        'MGA' => [
            'MGA',
            'Αριάρι Μαδαγασκάρης',
        ],
        'MGF' => [
            'MGF',
            'Φράγκο Μαδαγασκάρης',
        ],
        'MKD' => [
            'MKD',
            'Δηνάριο ΠΓΔΜ',
        ],
        'MLF' => [
            'MLF',
            'Φράγκο Μαλί',
        ],
        'MMK' => [
            'MMK',
            'Κιάτ Μιανμάρ',
        ],
        'MNT' => [
            'MNT',
            'Τουγκρίκ Μογγολίας',
        ],
        'MOP' => [
            'MOP',
            'Πατάκα Μακάο',
        ],
        'MRO' => [
            'MRO',
            'Ουγκίγια Μαυριτανίας (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'Ουγκίγια Μαυριτανίας',
        ],
        'MTL' => [
            'MTL',
            'Λιρέτα Μάλτας',
        ],
        'MTP' => [
            'MTP',
            'Λίρα Μάλτας',
        ],
        'MUR' => [
            'MUR',
            'Ρουπία Μαυρικίου',
        ],
        'MVR' => [
            'MVR',
            'Ρουφίγια Μαλδίβων',
        ],
        'MWK' => [
            'MWK',
            'Κουάτσα Μαλάουι',
        ],
        'MXN' => [
            'MX$',
            'Πέσο Μεξικού',
        ],
        'MXP' => [
            'MXP',
            'Ασημένιο Πέσο Μεξικού (1861–1992)',
        ],
        'MYR' => [
            'MYR',
            'Ρινγκίτ Μαλαισίας',
        ],
        'MZE' => [
            'MZE',
            'Εσκούδο Μοζαμβίκης',
        ],
        'MZM' => [
            'MZM',
            'Παλαιό Μετικάλ Μοζαμβίκης',
        ],
        'MZN' => [
            'MZN',
            'Μετικάλ Μοζαμβίκης',
        ],
        'NAD' => [
            'NAD',
            'Δολάριο Ναμίμπιας',
        ],
        'NGN' => [
            'NGN',
            'Νάιρα Νιγηρίας',
        ],
        'NIC' => [
            'NIC',
            'Κόρδοβα Νικαράγουας',
        ],
        'NIO' => [
            'NIO',
            'Χρυσή Κόρδοβα Νικαράγουας',
        ],
        'NLG' => [
            'NLG',
            'Γκίλντα Ολλανδίας',
        ],
        'NOK' => [
            'NOK',
            'Κορόνα Νορβηγίας',
        ],
        'NPR' => [
            'NPR',
            'Ρουπία Νεπάλ',
        ],
        'NZD' => [
            'NZ$',
            'Δολάριο Νέας Ζηλανδίας',
        ],
        'OMR' => [
            'OMR',
            'Ριάλ Ομάν',
        ],
        'PAB' => [
            'PAB',
            'Μπαλμπόα Παναμά',
        ],
        'PEI' => [
            'PEI',
            'Ίντι Περού',
        ],
        'PEN' => [
            'PEN',
            'Σολ Περού',
        ],
        'PES' => [
            'PES',
            'Σολ Περού (1863–1965)',
        ],
        'PGK' => [
            'PGK',
            'Κίνα Παπούας Νέας Γουινέας',
        ],
        'PHP' => [
            'PHP',
            'Πέσο Φιλιππίνων',
        ],
        'PKR' => [
            'PKR',
            'Ρουπία Πακιστάν',
        ],
        'PLN' => [
            'PLN',
            'Ζλότι Πολωνίας',
        ],
        'PLZ' => [
            'PLZ',
            'Ζλότυ Πολωνίας (1950–1995)',
        ],
        'PTE' => [
            'PTE',
            'Εσκούδο Πορτογαλίας',
        ],
        'PYG' => [
            'PYG',
            'Γκουαρανί Παραγουάης',
        ],
        'QAR' => [
            'QAR',
            'Ριάλ Κατάρ',
        ],
        'RHD' => [
            'RHD',
            'Δολάριο Ροδεσίας',
        ],
        'ROL' => [
            'ROL',
            'Λέι Ρουμανίας',
        ],
        'RON' => [
            'RON',
            'Λέου Ρουμανίας',
        ],
        'RSD' => [
            'RSD',
            'Δηνάριο Σερβίας',
        ],
        'RUB' => [
            'RUB',
            'Ρούβλι Ρωσίας',
        ],
        'RUR' => [
            'RUR',
            'Ρούβλι Ρωσίας (1991–1998)',
        ],
        'RWF' => [
            'RWF',
            'Φράγκο Ρουάντας',
        ],
        'SAR' => [
            'SAR',
            'Ριάλ Σαουδικής Αραβίας',
        ],
        'SBD' => [
            'SBD',
            'Δολάριο Νήσων Σολομώντος',
        ],
        'SCR' => [
            'SCR',
            'Ρουπία Σεϋχελλών',
        ],
        'SDD' => [
            'SDD',
            'Δηνάριο Σουδάν',
        ],
        'SDG' => [
            'SDG',
            'Λίρα Σουδάν',
        ],
        'SDP' => [
            'SDP',
            'Παλαιά Λίρα Σουδάν',
        ],
        'SEK' => [
            'SEK',
            'Κορόνα Σουηδίας',
        ],
        'SGD' => [
            'SGD',
            'Δολάριο Σιγκαπούρης',
        ],
        'SHP' => [
            'SHP',
            'Λίρα Αγίας Ελένης',
        ],
        'SIT' => [
            'SIT',
            'Τόλαρ Σλοβενίας',
        ],
        'SKK' => [
            'SKK',
            'Κορόνα Σλοβενίας',
        ],
        'SLE' => [
            'SLE',
            'Λεόνε Σιέρα Λεόνε',
        ],
        'SLL' => [
            'SLL',
            'Λεόνε Σιέρα Λεόνε (1964—2022)',
        ],
        'SOS' => [
            'SOS',
            'Σελίνι Σομαλίας',
        ],
        'SRD' => [
            'SRD',
            'Δολάριο Σουρινάμ',
        ],
        'SRG' => [
            'SRG',
            'Γκίλντα Σουρινάμ',
        ],
        'SSP' => [
            'SSP',
            'Λίρα Νότιου Σουδάν',
        ],
        'STD' => [
            'STD',
            'Ντόμπρα Σάο Τομέ και Πρίνσιπε (1977–2017)',
        ],
        'STN' => [
            'STN',
            'Ντόμπρα Σάο Τομέ και Πρίνσιπε',
        ],
        'SUR' => [
            'SUR',
            'Σοβιετικό Ρούβλι',
        ],
        'SVC' => [
            'SVC',
            'Κολόν Ελ Σαλβαδόρ',
        ],
        'SYP' => [
            'SYP',
            'Λίρα Συρίας',
        ],
        'SZL' => [
            'SZL',
            'Λιλανγκένι Σουαζιλάνδης',
        ],
        'THB' => [
            '฿',
            'Μπατ Ταϊλάνδης',
        ],
        'TJR' => [
            'TJR',
            'Ρούβλι Τατζικιστάν',
        ],
        'TJS' => [
            'TJS',
            'Σομόνι Τατζικιστάν',
        ],
        'TMM' => [
            'TMM',
            'Μανάτ Τουρκμενιστάν',
        ],
        'TMT' => [
            'TMT',
            'Μάνατ Τουρκμενιστάν',
        ],
        'TND' => [
            'TND',
            'Δηνάριο Τυνησίας',
        ],
        'TOP' => [
            'TOP',
            'Παάγκα Τόνγκα',
        ],
        'TPE' => [
            'TPE',
            'Εσκούδο Τιμόρ',
        ],
        'TRL' => [
            'TRL',
            'Παλιά Λίρα Τουρκίας',
        ],
        'TRY' => [
            'TRY',
            'Λίρα Τουρκίας',
        ],
        'TTD' => [
            'TTD',
            'Δολάριο Τρινιντάντ και Τομπάγκο',
        ],
        'TWD' => [
            'NT$',
            'Νέο δολάριο Ταϊβάν',
        ],
        'TZS' => [
            'TZS',
            'Σελίνι Τανζανίας',
        ],
        'UAH' => [
            'UAH',
            'Γρίβνα Ουκρανίας',
        ],
        'UAK' => [
            'UAK',
            'Καρμποβανέτς Ουκρανίας',
        ],
        'UGS' => [
            'UGS',
            'Σελίνι Ουγκάντας (1966–1987)',
        ],
        'UGX' => [
            'UGX',
            'Σελίνι Ουγκάντας',
        ],
        'USD' => [
            '$',
            'Δολάριο ΗΠΑ',
        ],
        'USN' => [
            'USN',
            'Δολάριο ΗΠΑ (επόμενη ημέρα)',
        ],
        'USS' => [
            'USS',
            'Δολάριο ΗΠΑ (ίδια ημέρα)',
        ],
        'UYP' => [
            'UYP',
            'Πέσο Ουρουγουάης (1975–1993)',
        ],
        'UYU' => [
            'UYU',
            'Πέσο Ουρουγουάης',
        ],
        'UZS' => [
            'UZS',
            'Σομ Ουζμπεκιστάν',
        ],
        'VEB' => [
            'VEB',
            'Μπολιβάρ Βενεζουέλας (1871–2008)',
        ],
        'VEF' => [
            'VEF',
            'Μπολιβάρ Βενεζουέλας (2008–2018)',
        ],
        'VES' => [
            'VES',
            'Μπολιβάρ Βενεζουέλας',
        ],
        'VND' => [
            '₫',
            'Ντονγκ Βιετνάμ',
        ],
        'VUV' => [
            'VUV',
            'Βατού Βανουάτου',
        ],
        'WST' => [
            'WST',
            'Τάλα Σαμόα',
        ],
        'XAF' => [
            'FCFA',
            'Φράγκο CFA Κεντρικής Αφρικής',
        ],
        'XCD' => [
            'EC$',
            'Δολάριο Ανατολικής Καραϊβικής',
        ],
        'XCG' => [
            'Cg.',
            'γκίλντα Καραϊβικής',
        ],
        'XEU' => [
            'XEU',
            'Ευρωπαϊκή Συναλλαγματική Μονάδα',
        ],
        'XFO' => [
            'XFO',
            'Χρυσό Φράγκο Γαλλίας',
        ],
        'XFU' => [
            'XFU',
            'UIC-Φράγκο Γαλλίας',
        ],
        'XOF' => [
            'F CFA',
            'Φράγκο CFA Δυτικής Αφρικής',
        ],
        'XPF' => [
            'CFPF',
            'Φράγκο CFP',
        ],
        'YDD' => [
            'YDD',
            'Δηνάριο Υεμένης',
        ],
        'YER' => [
            'YER',
            'Ριάλ Υεμένης',
        ],
        'YUD' => [
            'YUD',
            'Μεταλλικό Δηνάριο Γιουγκοσλαβίας',
        ],
        'YUM' => [
            'YUM',
            'Νέο Δηνάριο Γιουγκοσλαβίας',
        ],
        'YUN' => [
            'YUN',
            'Μετατρέψιμο Δηνάριο Γιουγκοσλαβίας',
        ],
        'ZAL' => [
            'ZAL',
            'Ραντ Νότιας Αφρικής (οικονομικό)',
        ],
        'ZAR' => [
            'ZAR',
            'Ραντ Νότιας Αφρικής',
        ],
        'ZMK' => [
            'ZMK',
            'Κουάνζα Ζαΐρ (1968–2012)',
        ],
        'ZMW' => [
            'ZMW',
            'Κουάτσα Ζάμπιας',
        ],
        'ZRN' => [
            'ZRN',
            'Νέο Ζαΐρ Ζαΐρ',
        ],
        'ZRZ' => [
            'ZRZ',
            'Ζαΐρ Ζαΐρ',
        ],
        'ZWD' => [
            'ZWD',
            'Δολάριο Ζιμπάμπουε',
        ],
        'ZWG' => [
            'ZWG',
            'Χρυσός της Ζιμπάμπουε',
        ],
        'ZWL' => [
            'ZWL',
            'Δολάριο Ζιμπάμπουε (2009)',
        ],
    ],
];
