<?php

return [
    'Names' => [
        'ak' => 'Liakan',
        'am' => 'Liamhariki',
        'ar' => 'Arabi',
        'be' => 'Belarusi',
        'bg' => 'Bulegari',
        'bn' => 'Bengali',
        'cs' => 'Tsheki',
        'de' => 'Lizelumani',
        'el' => 'Giliki',
        'en' => 'Lingelesa',
        'es' => 'Lihispania',
        'fa' => 'Mpepajemi',
        'fr' => 'Mfwàlànsa',
        'ha' => 'Hausa',
        'hi' => 'Hindi',
        'hu' => 'Hongili',
        'id' => 'Lindonezia',
        'ig' => 'Igbo',
        'it' => 'Litali',
        'ja' => 'Liyapani',
        'jv' => 'Java',
        'ko' => 'Likoreya',
        'lu' => 'Tshiluba',
        'ms' => 'Limalezia',
        'ne' => 'nepali',
        'nl' => 'olandi',
        'pa' => 'Lipunjabi',
        'pl' => 'Mpoloni',
        'pt' => 'Mputulugɛsi',
        'ro' => 'Liromani',
        'ru' => 'Lirisi',
        'rw' => 'kinyarwanda',
        'so' => 'Lisomali',
        'sv' => 'Lisuwidi',
        'ta' => 'Mtamuili',
        'th' => 'Ntailandi',
        'tr' => 'Ntuluki',
        'uk' => 'Nkrani',
        'ur' => 'Urdu',
        'vi' => 'Liviyetinamu',
        'yo' => 'Nyoruba',
        'zh' => 'shinɛ',
        'zu' => 'Nzulu',
    ],
    'LocalizedNames' => [],
];
