<?php

return [
    'UserAssignedNames' => [
        'XK' => 'Kosovo',
    ],
    'Names' => [
        'AD' => 'Andora',
        'AE' => 'United Arab Emirates',
        'AF' => 'Afganistan',
        'AG' => 'Antigua ne Baabuda',
        'AI' => 'Anguila',
        'AL' => 'Albenia',
        'AM' => 'Aamenia',
        'AO' => 'Angola',
        'AQ' => 'Antaatika',
        'AR' => 'Agyɛntina',
        'AS' => 'Amɛrika Samoa',
        'AT' => 'Ɔstria',
        'AU' => 'Ɔstrelia',
        'AW' => 'Aruba',
        'AX' => 'Aland Aeland',
        'AZ' => 'Asabegyan',
        'BA' => 'Bosnia ne Hɛzegovina',
        'BB' => 'Baabados',
        'BD' => 'Bangladɛhye',
        'BE' => 'Bɛlgyium',
        'BF' => 'Bɔkina Faso',
        'BG' => 'Bɔlgeria',
        'BH' => 'Baren',
        'BI' => 'Burundi',
        'BJ' => 'Bɛnin',
        'BL' => 'St. Baatilemi',
        'BM' => 'Bɛmuda',
        'BN' => 'Brunae',
        'BO' => 'Bolivia',
        'BQ' => 'Caribbean Netherlands',
        'BR' => 'Brazil',
        'BS' => 'Bahamase',
        'BT' => 'Butan',
        'BV' => 'Bouvet Island',
        'BW' => 'Bɔtswana',
        'BY' => 'Bɛlarus',
        'BZ' => 'Beliz',
        'CA' => 'Kanada',
        'CC' => 'Kokoso Supɔ',
        'CD' => 'Kongo Kinhyaahya',
        'CF' => 'Afrika Finimfin Man',
        'CG' => 'Kongo',
        'CH' => 'Swetzaland',
        'CI' => 'Kodivuwa',
        'CK' => 'Kuk Nsupɔ',
        'CL' => 'Kyili',
        'CM' => 'Kamɛrun',
        'CN' => 'Kyaena',
        'CO' => 'Kolombia',
        'CR' => 'Kɔsta Rika',
        'CU' => 'Kiuba',
        'CV' => 'Kepvɛdfo Islands',
        'CW' => 'Kurakaw',
        'CX' => 'Buronya Supɔ',
        'CY' => 'Saeprɔso',
        'CZ' => 'Kyɛk',
        'DE' => 'Gyaaman',
        'DJ' => 'Gyibuti',
        'DK' => 'Dɛnmak',
        'DM' => 'Dɔmeneka',
        'DO' => 'Dɔmeneka Man',
        'DZ' => 'Ɔlgyeria',
        'EC' => 'Yikuwedɔ',
        'EE' => 'Ɛstonia',
        'EG' => 'Misrim',
        'EH' => 'Sahara Atɔeɛ',
        'ER' => 'Ɛritrea',
        'ES' => 'Spain',
        'ET' => 'Yitiopia',
        'FI' => 'Finland',
        'FJ' => 'Figyi',
        'FK' => 'Fɔkman Aeland',
        'FM' => 'Maekronehyia',
        'FO' => 'Faro Aeland',
        'FR' => 'Franse',
        'GA' => 'Gabɔn',
        'GB' => 'UK',
        'GD' => 'Grenada',
        'GE' => 'Gyɔgyea',
        'GF' => 'Frɛnkye Gayana',
        'GG' => 'Guɛnse',
        'GH' => 'Gaana',
        'GI' => 'Gyebralta',
        'GL' => 'Greenman',
        'GM' => 'Gambia',
        'GN' => 'Gini',
        'GP' => 'Guwadelup',
        'GQ' => 'Gini Ikuweta',
        'GR' => 'Greekman',
        'GS' => 'Gyɔɔgyia Anaafoɔ ne Sandwich Aeland Anaafoɔ',
        'GT' => 'Guwatemala',
        'GU' => 'Guam',
        'GW' => 'Gini Bisaw',
        'GY' => 'Gayana',
        'HK' => 'Hɔnkɔn Kyaena',
        'HM' => 'Heard ne McDonald Supɔ',
        'HN' => 'Hɔnduras',
        'HR' => 'Krowehyia',
        'HT' => 'Heiti',
        'HU' => 'Hangari',
        'ID' => 'Ndoniihyia',
        'IE' => 'Aereland',
        'IL' => 'Israe',
        'IM' => 'Isle of Man',
        'IN' => 'India',
        'IO' => 'Britenfo Man Wɔ India Po No Mu',
        'IQ' => 'Irak',
        'IR' => 'Yiran',
        'IS' => 'Aesland',
        'IT' => 'Itali',
        'JE' => 'Gyɛsi',
        'JM' => 'Gyameka',
        'JO' => 'Gyɔdan',
        'JP' => 'Gyapan',
        'KE' => 'Kenya',
        'KG' => 'Kɛɛgestan',
        'KH' => 'Kambodia',
        'KI' => 'Kiribati',
        'KM' => 'Kɔmɔrɔs',
        'KN' => 'Saint Kitts ne Nevis',
        'KP' => 'Korea Atifi',
        'KR' => 'Korea Anaafoɔ',
        'KW' => 'Kuweti',
        'KY' => 'Kemanfo Islands',
        'KZ' => 'Kazakstan',
        'LA' => 'Laos',
        'LB' => 'Lɛbanɔn',
        'LC' => 'Saint Lucia',
        'LI' => 'Lektenstaen',
        'LK' => 'Sri Lanka',
        'LR' => 'Laeberia',
        'LS' => 'Lesoto',
        'LT' => 'Lituwenia',
        'LU' => 'Lusimbɛg',
        'LV' => 'Latvia',
        'LY' => 'Libya',
        'MA' => 'Moroko',
        'MC' => 'Monako',
        'MD' => 'Mɔldova',
        'ME' => 'Mɔntenegro',
        'MF' => 'St. Maatin',
        'MG' => 'Madagaska',
        'MH' => 'Mahyaa Aeland',
        'MK' => 'Mesidonia Atifi',
        'ML' => 'Mali',
        'MM' => 'Mayaama (Bɛɛma)',
        'MN' => 'Mɔngolia',
        'MO' => 'Makaw Kyaena',
        'MP' => 'Mariana Atifi Fam Aeland',
        'MQ' => 'Matinik',
        'MR' => 'Mɔretenia',
        'MS' => 'Mantserat',
        'MT' => 'Mɔlta',
        'MU' => 'Mɔrehyeɔs',
        'MV' => 'Maldives',
        'MW' => 'Malawi',
        'MX' => 'Mɛksiko',
        'MY' => 'Malehyia',
        'MZ' => 'Mozambik',
        'NA' => 'Namibia',
        'NC' => 'Kaledonia Foforo',
        'NE' => 'Nigyɛɛ',
        'NF' => 'Norfold Supɔ',
        'NG' => 'Naegyeria',
        'NI' => 'Nekaraguwa',
        'NL' => 'Nɛdɛland',
        'NO' => 'Nɔɔwe',
        'NP' => 'Nɛpal',
        'NR' => 'Naworu',
        'NU' => 'Niyu',
        'NZ' => 'Ziland Foforo',
        'OM' => 'Oman',
        'PA' => 'Panama',
        'PE' => 'Peru',
        'PF' => 'Frɛnkye Pɔlenehyia',
        'PG' => 'Papua Gini Foforɔ',
        'PH' => 'Filipin',
        'PK' => 'Pakistan',
        'PL' => 'Pɔland',
        'PM' => 'Saint Pierre ne Miquelon',
        'PN' => 'Pitkaan Nsupɔ',
        'PR' => 'Puɛto Riko',
        'PS' => 'Palestaen West Bank ne Gaza',
        'PT' => 'Pɔtugal',
        'PW' => 'Palau',
        'PY' => 'Paraguae',
        'QA' => 'Kata',
        'RE' => 'Reyuniɔn',
        'RO' => 'Romenia',
        'RS' => 'Sɛbia',
        'RU' => 'Rɔhyea',
        'RW' => 'Rewanda',
        'SA' => 'Saudi Arabia',
        'SB' => 'Solomɔn Aeland',
        'SC' => 'Seyhyɛl',
        'SD' => 'Sudan',
        'SE' => 'Sweden',
        'SG' => 'Singapɔ',
        'SH' => 'Saint Helena',
        'SI' => 'Slovenia',
        'SJ' => 'Svalbard ne Jan Mayen',
        'SK' => 'Slovakia',
        'SL' => 'Sɛra Liɔn',
        'SM' => 'San Marino',
        'SN' => 'Senegal',
        'SO' => 'Somalia',
        'SR' => 'Suriname',
        'SS' => 'Sudan Anaafoɔ',
        'ST' => 'São Tomé ne Príncipe',
        'SV' => 'Ɛl Salvadɔ',
        'SX' => 'Sint Maaten',
        'SY' => 'Siria',
        'SZ' => 'Swaziland',
        'TC' => 'Turks ne Caicos Islands',
        'TD' => 'Kyad',
        'TF' => 'Franse Anaafoɔ Nsaase',
        'TG' => 'Togo',
        'TH' => 'Taeland',
        'TJ' => 'Tagyikistan',
        'TK' => 'Tokelau',
        'TL' => 'Timɔ Boka',
        'TM' => 'Tɛkmɛnistan',
        'TN' => 'Tunihyia',
        'TO' => 'Tonga',
        'TR' => 'Tɛɛki',
        'TT' => 'Trinidad ne Tobago',
        'TV' => 'Tuvalu',
        'TW' => 'Taiwan',
        'TZ' => 'Tansania',
        'UA' => 'Ukren',
        'UG' => 'Yuganda',
        'UM' => 'U.S. Nkyɛnnkyɛn Supɔ Ahodoɔ',
        'US' => 'Amɛrika',
        'UY' => 'Yurugwae',
        'UZ' => 'Usbɛkistan',
        'VA' => 'Vatican Man',
        'VC' => 'Saint Vincent ne Grenadines',
        'VE' => 'Venezuela',
        'VG' => 'Ngresifoɔ Virgin Island',
        'VI' => 'Amɛrika Virgin Islands',
        'VN' => 'Viɛtnam',
        'VU' => 'Vanuatu',
        'WF' => 'Wallis ne Futuna',
        'WS' => 'Samoa',
        'YE' => 'Yɛmɛn',
        'YT' => 'Mayɔte',
        'ZA' => 'Abibirem Anaafoɔ',
        'ZM' => 'Zambia',
        'ZW' => 'Zimbabue',
    ],
];
