<?php

return [
    '👩‍❤‍💋‍👨' => 'kys: kvinde, mand',
    '👨‍❤‍💋‍👨' => 'kys: mand, mand',
    '👩‍❤‍💋‍👩' => 'kys: kvinde, kvinde',
    '👨‍👩‍👧‍👦' => 'familie: mand, kvinde, pige, dreng',
    '👨‍👩‍👦‍👦' => 'familie: mand, kvinde, dreng, dreng',
    '👨‍👩‍👧‍👧' => 'familie: mand, kvinde, pige, pige',
    '👨‍👨‍👧‍👦' => 'familie: mand, mand, pige, dreng',
    '👨‍👨‍👦‍👦' => 'familie: mand, mand, dreng, dreng',
    '👨‍👨‍👧‍👧' => 'familie: mand, mand, pige, pige',
    '👩‍👩‍👧‍👦' => 'familie: kvinde, kvinde, pige, dreng',
    '👩‍👩‍👦‍👦' => 'familie: kvinde, kvinde, dreng, dreng',
    '👩‍👩‍👧‍👧' => 'familie: kvinde, kvinde, pige, pige',
    '👩‍❤‍👨' => 'par med hjerte: kvinde, mand',
    '👨‍❤‍👨' => 'par med hjerte: mand, mand',
    '👩‍❤‍👩' => 'par med hjerte: kvinde, kvinde',
    '👨‍👩‍👦' => 'familie: mand, kvinde, dreng',
    '👨‍👩‍👧' => 'familie: mand, kvinde, pige',
    '👨‍👨‍👦' => 'familie: mand, mand, dreng',
    '👨‍👨‍👧' => 'familie: mand, mand, pige',
    '👩‍👩‍👦' => 'familie: kvinde, kvinde, dreng',
    '👩‍👩‍👧' => 'familie: kvinde, kvinde, pige',
    '👨‍👦‍👦' => 'familie: mand, dreng, dreng',
    '👨‍👧‍👦' => 'familie: mand, pige, dreng',
    '👨‍👧‍👧' => 'familie: mand, pige, pige',
    '👩‍👦‍👦' => 'familie: kvinde, dreng, dreng',
    '👩‍👧‍👦' => 'familie: kvinde, pige, dreng',
    '👩‍👧‍👧' => 'familie: kvinde, pige, pige',
    '👨‍👦' => 'familie: mand, dreng',
    '👨‍👧' => 'familie: mand, pige',
    '👩‍👦' => 'familie: kvinde, dreng',
    '👩‍👧' => 'familie: kvinde, pige',
    '👁‍🗨' => 'øje i taleboble',
    '👱‍♂' => 'blond mand',
    '👱‍♀' => 'blond kvinde',
    '🙍‍♂' => 'mand rynker panden',
    '🙍‍♀' => 'kvinde rynker panden',
    '🙎‍♂' => 'mand surmuler',
    '🙎‍♀' => 'kvinde surmuler',
    '🙅‍♂' => 'mand signalerer nej',
    '🙅‍♀' => 'kvinde signalerer nej',
    '🙆‍♂' => 'mand signalerer ok',
    '🙆‍♀' => 'kvinde signalerer ok',
    '💁‍♂' => 'mand vender håndfladen op',
    '💁‍♀' => 'kvinde vender håndfladen op',
    '🙋‍♂' => 'mand med løftet hånd',
    '🙋‍♀' => 'kvinde med løftet hånd',
    '🙇‍♂' => 'mand bukker dybt',
    '🙇‍♀' => 'kvinde bukker dybt',
    '🤦‍♂' => 'mand slår sig for panden',
    '🤦‍♀' => 'kvinde slår sig for panden',
    '🤷‍♂' => 'mand trækker på skuldrene',
    '🤷‍♀' => 'kvinde trækker på skuldrene',
    '👨‍⚕' => 'mandlig sundhedsperson',
    '👩‍⚕' => 'kvindelig sundhedsperson',
    '👨‍🎓' => 'studerende',
    '👩‍🎓' => 'kvindelig studerende',
    '👨‍🏫' => 'lærer',
    '👩‍🏫' => 'kvindelig lærer',
    '👨‍⚖' => 'dommer',
    '👩‍⚖' => 'kvindelig dommer',
    '👨‍🌾' => 'landmand',
    '👩‍🌾' => 'kvindelig landmand',
    '👨‍🍳' => 'kok',
    '👩‍🍳' => 'kvindelig kok',
    '👨‍🔧' => 'mekaniker',
    '👩‍🔧' => 'kvindelig mekaniker',
    '👨‍🏭' => 'fabriksarbejder',
    '👩‍🏭' => 'kvindelig fabriksarbejder',
    '👨‍💼' => 'kontoransat',
    '👩‍💼' => 'kvindelig kontoransat',
    '👨‍🔬' => 'forsker',
    '👩‍🔬' => 'kvindelig forsker',
    '👨‍💻' => 'IT-medarbejder',
    '👩‍💻' => 'kvindelig IT-medarbejder',
    '👨‍🎤' => 'sanger',
    '👩‍🎤' => 'sangerinde',
    '👨‍🎨' => 'kunstner',
    '👩‍🎨' => 'kvindelig kunstner',
    '👨‍✈' => 'pilot',
    '👩‍✈' => 'kvindelig pilot',
    '👨‍🚀' => 'astronaut',
    '👩‍🚀' => 'kvindelig astronaut',
    '👨‍🚒' => 'brandmand',
    '👩‍🚒' => 'kvindelig brandmand',
    '👮‍♂' => 'mandlig politibetjent',
    '👮‍♀' => 'kvindelig politibetjent',
    '💂‍♂' => 'mandlig garder',
    '💂‍♀' => 'kvindelig garder',
    '👷‍♂' => 'bygningsarbejder',
    '👷‍♀' => 'kvindelig bygningsarbejder',
    '👳‍♂' => 'mand med turban',
    '👳‍♀' => 'kvinde med turban',
    '💆‍♂' => 'mand får ansigtsmassage',
    '💆‍♀' => 'kvinde får ansigtsmassage',
    '💇‍♂' => 'mand bliver klippet',
    '💇‍♀' => 'kvinde bliver klippet',
    '🚶‍♂' => 'mand går',
    '🚶‍♀' => 'kvinde går',
    '🏃‍♂' => 'mand løber',
    '🏃‍♀' => 'kvinde løber',
    '👯‍♂' => 'mænd fester',
    '👯‍♀' => 'kvinder fester',
    '🏄‍♂' => 'surfer',
    '🏄‍♀' => 'kvindelig surfer',
    '🚣‍♂' => 'mand i robåd',
    '🚣‍♀' => 'kvinde i robåd',
    '🏊‍♂' => 'mandlig svømmer',
    '🏊‍♀' => 'kvindelig svømmer',
    '🚴‍♂' => 'mandlig cyklist',
    '🚴‍♀' => 'kvindelig cyklist',
    '🚵‍♂' => 'mand på mountainbike',
    '🚵‍♀' => 'kvinde på mountainbike',
    '🤸‍♂' => 'mand slår vejrmøller',
    '🤸‍♀' => 'kvinde slår vejrmøller',
    '🤼‍♂' => 'mandlige brydere',
    '🤼‍♀' => 'kvindelige brydere',
    '🤽‍♂' => 'vandpolospiller',
    '🤽‍♀' => 'kvindelig vandpolospiller',
    '🤾‍♂' => 'håndboldspiller',
    '🤾‍♀' => 'kvindelig håndboldspiller',
    '🤹‍♂' => 'jonglør',
    '🤹‍♀' => 'kvindelig jonglør',
    '🏳‍🌈' => 'regnbueflag',
    '😀' => 'grinende ansigt',
    '😃' => 'smilende ansigt med åben mund',
    '😄' => 'smilende ansigt med åben mund og smilende øjne',
    '😁' => 'grinende ansigt med smilende øjne',
    '😆' => 'smilende ansigt med åben mund og lukkede øjne',
    '😅' => 'smilende ansigt med åben mund og koldsved',
    '🤣' => 'ruller rundt på gulvet af grin',
    '😂' => 'ansigt med glædestårer',
    '🙂' => 'let smilende ansigt',
    '🙃' => 'omvendt ansigt',
    '😉' => 'blinkende ansigt',
    '😊' => 'smilende ansigt og øjne',
    '😇' => 'smilende ansigt med glorie',
    '😍' => 'smilende ansigt med hjertøjne',
    '😘' => 'kyssende ansigt',
    '😗' => 'kysseansigt',
    '☺' => 'smilende ansigt',
    '😚' => 'kysseansigt med lukkede øjne',
    '😙' => 'kysseansigt med smilende øjne',
    '😋' => 'nydende ansigt',
    '😛' => 'ansigt med tungen ude',
    '😜' => 'ansigt med tungen ude og blinkende øje',
    '😝' => 'ansigt med tungen ude og sammenknebne øjne',
    '🤑' => 'ansigt med dollartegn',
    '🤗' => 'krammende ansigt',
    '🤔' => 'tænkende ansigt',
    '🤐' => 'ansigt med lynlåsmund',
    '😐' => 'neutralt ansigt',
    '😑' => 'udtryksløst ansigt',
    '😶' => 'ansigt uden mund',
    '😏' => 'ansigt med smørret grin',
    '😒' => 'uimponeret ansigt',
    '🙄' => 'ansigt med rullende øjne',
    '😬' => 'sammenbidt ansigt',
    '🤥' => 'løgneransigt',
    '😌' => 'lettet ansigt',
    '😔' => 'eftertænksomt ansigt',
    '😪' => 'dyb søvn',
    '🤤' => 'savlende ansigt',
    '😴' => 'sovende ansigt',
    '😷' => 'ansigt med maske',
    '🤒' => 'ansigt med termometer',
    '🤕' => 'ansigt med bandage',
    '🤢' => 'ansigt med kvalme',
    '🤧' => 'nysende ansigt',
    '😵' => 'svimmelt ansigt',
    '🤠' => 'cowboyhat ansigt',
    '😎' => 'smilende ansigt med solbriller',
    '🤓' => 'nørdansigt',
    '😕' => 'forvirret ansigt',
    '😟' => 'bekymret ansigt',
    '🙁' => 'lidt trist ansigt',
    '☹' => 'trist ansigt',
    '😮' => 'ansigt med åben mund',
    '😯' => 'tavst ansigt',
    '😲' => 'overrasket ansigt',
    '😳' => 'rødmende ansigt',
    '😦' => 'måbende ansigt med åben mund',
    '😧' => 'forpint ansigt',
    '😨' => 'bange ansigt',
    '😰' => 'ansigt med åben mund og koldsved',
    '😥' => 'skuffet men lettet ansigt',
    '😢' => 'grædende ansigt',
    '😭' => 'grådkvalt ansigt',
    '😱' => 'skrigende ansigt',
    '😖' => 'forbistret ansigt',
    '😣' => 'stædigt ansigt',
    '😞' => 'skuffet ansigt',
    '😓' => 'ansigt med koldsved',
    '😩' => 'udmattet ansigt',
    '😫' => 'træt ansigt',
    '😤' => 'fnysende ansigt',
    '😡' => 'surt ansigt',
    '😠' => 'vredt ansigt',
    '😈' => 'smilende ansigt med horn',
    '👿' => 'lilla djævel',
    '💀' => 'kranium',
    '☠' => 'kranie og korslagte knogler',
    '💩' => 'bæ',
    '🤡' => 'klovneansigt',
    '👹' => 'trold',
    '👺' => 'ånd',
    '👻' => 'spøgelse',
    '👽' => 'rumvæsen',
    '👾' => 'alien-monster',
    '🤖' => 'robotansigt',
    '😺' => 'kat smiler',
    '😸' => 'kat griner',
    '😹' => 'kat med glædestårer',
    '😻' => 'glad kat med hjerter',
    '😼' => 'kat med ironisk smil',
    '😽' => 'kyssende kat med lukkede øjne',
    '🙀' => 'træt kat',
    '😿' => 'grædende kat',
    '😾' => 'sur kat',
    '🙈' => 'ikke se',
    '🙉' => 'ikke høre',
    '🙊' => 'ikke tale',
    '💌' => 'kærlighedsbrev',
    '💘' => 'hjerte med pil',
    '💝' => 'hjerte med bånd',
    '💖' => 'hjerte med stjerner',
    '💗' => 'voksende hjerte',
    '💓' => 'bankende hjerte',
    '💞' => 'roterende hjerter',
    '💕' => 'to hjerter',
    '💟' => 'hjertedekoration',
    '❣' => 'hjerte som udråbstegn',
    '💔' => 'knust hjerte',
    '❤' => 'rødt hjerte',
    '💛' => 'gult hjerte',
    '💚' => 'grønt hjerte',
    '💙' => 'blåt hjerte',
    '💜' => 'violet hjerte',
    '🖤' => 'sort hjerte',
    '💋' => 'læbeaftryk',
    '💯' => '100 point',
    '💢' => 'vrede',
    '💥' => 'kollision',
    '💫' => 'svimmel',
    '💦' => 'sveddråber',
    '💨' => 'smutter',
    '🕳' => 'hul',
    '💬' => 'taleboble',
    '🗨' => 'taleboble til venstre',
    '🗯' => 'vred taleboble til højre',
    '💭' => 'tankeboble',
    '💤' => 'søvnig',
    '👋' => 'vinker',
    '🤚' => 'løftet baghånd',
    '🖐' => 'hånd med spredte fingre',
    '✋' => 'løftet hånd',
    '🖖' => 'vulcan-hilsen',
    '👌' => 'ok-tegn',
    '✌' => 'v-tegn',
    '🤞' => 'krydsede fingre',
    '🤘' => 'djævletegn',
    '🤙' => 'ring til mig-hånd',
    '👈' => 'venstre',
    '👉' => 'højre',
    '👆' => 'finger peger op',
    '🖕' => 'langefinger',
    '👇' => 'ned',
    '☝' => 'opadvendt pegefinger',
    '👍' => 'tommelfinger op',
    '👎' => 'tommelfinger ned',
    '✊' => 'løftet knytnæve',
    '👊' => 'knytnæve',
    '🤛' => 'venstrevendt knytnæve',
    '🤜' => 'højrevendt knytnæve',
    '👏' => 'hænder klapper',
    '🙌' => 'hurra',
    '👐' => 'åbne hænder',
    '🤝' => 'håndtryk',
    '🙏' => 'foldede hænder',
    '✍' => 'skrivende hånd',
    '💅' => 'neglelak',
    '🤳' => 'selfie',
    '💪' => 'armmuskler',
    '👂' => 'øre',
    '👃' => 'næse',
    '👀' => 'øjne',
    '👁' => 'øje',
    '👅' => 'tunge',
    '👄' => 'mund',
    '👶' => 'baby',
    '👦' => 'dreng',
    '👧' => 'pige',
    '👱' => 'person',
    '👨' => 'mand',
    '👩' => 'kvinde',
    '👴' => 'gammel mand',
    '👵' => 'gammel kvinde',
    '🙍' => 'rynkede bryn',
    '🙎' => 'surmulende person',
    '🙅' => 'signalerer nej',
    '🙆' => 'signalerer ok',
    '💁' => 'information',
    '🙋' => 'glad person',
    '🤦' => 'facepalm',
    '🤷' => 'trækker på skuldrene',
    '👮' => 'politibetjent',
    '🕵' => 'detektiv',
    '💂' => 'garder',
    '👷' => 'byggearbejder',
    '🤴' => 'prins',
    '👸' => 'prinsesse',
    '👲' => 'mand med kinesisk hat',
    '🤵' => 'mand i smoking',
    '👰' => 'brud',
    '🤰' => 'gravid kvinde',
    '👼' => 'babyengel',
    '🎅' => 'julemand',
    '🤶' => 'juledame',
    '💆' => 'ansigtsmassage',
    '💇' => 'frisør',
    '🚶' => 'fodgænger',
    '🏃' => 'løber',
    '💃' => 'danser',
    '🕺' => 'dansende mand',
    '🕴' => 'svævende forretningsmand',
    '👯' => 'kvinder med kaninører',
    '🤺' => 'fægter',
    '🏇' => 'hestevæddeløb',
    '⛷' => 'skiløber',
    '🏂' => 'snowboarder',
    '🏌' => 'golfspiller',
    '🏄' => 'surfing',
    '🚣' => 'robåd',
    '🏊' => 'svømmer',
    '⛹' => 'person med bold',
    '🏋' => 'vægtløfter',
    '🚴' => 'cyklist',
    '🚵' => 'mountainbikerytter',
    '🤸' => 'vejrmølle',
    '🤼' => 'brydere',
    '🤽' => 'vandpolo',
    '🤾' => 'håndbold',
    '🤹' => 'jonglerer',
    '🛀' => 'bad',
    '🛌' => 'person i seng',
    '👭' => 'to kvinder',
    '👫' => 'man og kvinde',
    '👬' => 'to mænd',
    '💏' => 'kys',
    '💑' => 'par med hjerte',
    '🗣' => 'talende hoved',
    '👤' => 'buste i silhuet',
    '👥' => 'buster i silhuet',
    '👪' => 'familie',
    '👣' => 'fodspor',
    '🐵' => 'abeansigt',
    '🐒' => 'abe',
    '🦍' => 'gorilla',
    '🐶' => 'hundeansigt',
    '🐕' => 'hund',
    '🐩' => 'puddel',
    '🐺' => 'ulvehoved',
    '🦊' => 'rævehoved',
    '🐱' => 'katteansigt',
    '🐈' => 'kat',
    '🦁' => 'løvehoved',
    '🐯' => 'tigeransigt',
    '🐅' => 'tiger',
    '🐆' => 'leopard',
    '🐴' => 'hestehoved',
    '🐎' => 'hest',
    '🦄' => 'enhjørning',
    '🦌' => 'hjort',
    '🐮' => 'koansigt',
    '🐂' => 'okse',
    '🐃' => 'vandbøffel',
    '🐄' => 'ko',
    '🐷' => 'griseansigt',
    '🐖' => 'gris',
    '🐗' => 'vildsvin',
    '🐽' => 'grisetryne',
    '🐏' => 'vædder',
    '🐑' => 'får',
    '🐐' => 'ged',
    '🐪' => 'dromedar',
    '🐫' => 'kamel',
    '🐘' => 'elefant',
    '🦏' => 'næsehorn',
    '🐭' => 'museansigt',
    '🐁' => 'mus',
    '🐀' => 'rotte',
    '🐹' => 'hamsteransigt',
    '🐰' => 'kaninansigt',
    '🐇' => 'kanin',
    '🐿' => 'jordegern',
    '🦇' => 'flagermus',
    '🐻' => 'bjørneansigt',
    '🐨' => 'koala',
    '🐼' => 'pandaansigt',
    '🐾' => 'poteaftryk',
    '🦃' => 'kalkun',
    '🐔' => 'høne',
    '🐓' => 'hane',
    '🐣' => 'ny udklækket kylling',
    '🐤' => 'kylling',
    '🐥' => 'kylling forfra',
    '🐦' => 'fugl',
    '🐧' => 'pingvin',
    '🕊' => 'due',
    '🦅' => 'ørn',
    '🦆' => 'and',
    '🦉' => 'ugle',
    '🐸' => 'frøansigt',
    '🐊' => 'krokodille',
    '🐢' => 'skildpadde',
    '🦎' => 'firben',
    '🐍' => 'slange',
    '🐲' => 'drageansigt',
    '🐉' => 'drage',
    '🐳' => 'hval med vandsprøjt',
    '🐋' => 'hval',
    '🐬' => 'delfin',
    '🐟' => 'fisk',
    '🐠' => 'tropisk fisk',
    '🐡' => 'kuglefisk',
    '🦈' => 'haj',
    '🐙' => 'blæksprutte',
    '🐚' => 'konkylie',
    '🦀' => 'krabbe',
    '🦐' => 'reje',
    '🦑' => 'sprutte',
    '🐌' => 'snegl',
    '🦋' => 'sommerfugl',
    '🐛' => 'larve',
    '🐜' => 'myre',
    '🐝' => 'honningbi',
    '🐞' => 'mariehøne',
    '🕷' => 'edderkop',
    '🕸' => 'edderkoppespind',
    '🦂' => 'skorpion',
    '💐' => 'blomsterbuket',
    '🌸' => 'kirsebærblomst',
    '💮' => 'blomsterstempel',
    '🏵' => 'roset',
    '🌹' => 'rose',
    '🥀' => 'vissen blomst',
    '🌺' => 'hibiscus',
    '🌻' => 'solsikke',
    '🌼' => 'blomst',
    '🌷' => 'tulipan',
    '🌱' => 'spire',
    '🌲' => 'grantræ',
    '🌳' => 'træ',
    '🌴' => 'palme',
    '🌵' => 'kaktus',
    '🌾' => 'risplante',
    '🌿' => 'urt',
    '☘' => 'kløver',
    '🍀' => 'firkløver',
    '🍁' => 'blad',
    '🍂' => 'faldende blade',
    '🍃' => 'blad i vinden',
    '🍄' => 'svamp',
    '🍇' => 'vindruer',
    '🍈' => 'melon',
    '🍉' => 'vandmelon',
    '🍊' => 'appelsin',
    '🍋' => 'citron',
    '🍌' => 'banan',
    '🍍' => 'ananas',
    '🍎' => 'rødt æble',
    '🍏' => 'grønt æble',
    '🍐' => 'pære',
    '🍑' => 'fersken',
    '🍒' => 'kirsebær',
    '🍓' => 'jordbær',
    '🥝' => 'kiwifrugt',
    '🍅' => 'tomat',
    '🥑' => 'avokado',
    '🍆' => 'aubergine',
    '🥔' => 'kartoffel',
    '🥕' => 'gulerod',
    '🌽' => 'majs',
    '🌶' => 'chili',
    '🥒' => 'agurk',
    '🥜' => 'peanuts',
    '🌰' => 'kastanje',
    '🍞' => 'brød',
    '🥐' => 'croissant',
    '🥖' => 'flute',
    '🥞' => 'pandekager',
    '🧀' => 'et stykke ost',
    '🍖' => 'kød',
    '🍗' => 'kyllingelår',
    '🥓' => 'bacon',
    '🍔' => 'hamburger',
    '🍟' => 'pomfritter',
    '🍕' => 'pizza',
    '🌭' => 'hotdog',
    '🌮' => 'taco',
    '🌯' => 'burrito',
    '🥙' => 'fladbrød med fyld',
    '🥚' => 'æg',
    '🍳' => 'stegepande',
    '🥘' => 'pande med mad',
    '🍲' => 'gryde med mad',
    '🥗' => 'grøn salat',
    '🍿' => 'popcorn',
    '🍱' => 'bento',
    '🍘' => 'riskiks',
    '🍙' => 'riskugle',
    '🍚' => 'ris',
    '🍛' => 'ris og karry',
    '🍜' => 'nudler',
    '🍝' => 'spaghetti',
    '🍠' => 'sød kartoffel',
    '🍢' => 'mad på spyd',
    '🍣' => 'sushi',
    '🍤' => 'dybstegt reje',
    '🍥' => 'fiskebolle',
    '🍡' => 'iskugler på pind',
    '🍦' => 'softice',
    '🍧' => 'snedessert',
    '🍨' => 'is',
    '🍩' => 'donut',
    '🍪' => 'cookie',
    '🎂' => 'fødselsdagskage',
    '🍰' => 'lagkage',
    '🍫' => 'chokolade',
    '🍬' => 'bolsje',
    '🍭' => 'slikpind',
    '🍮' => 'dessert',
    '🍯' => 'honning',
    '🍼' => 'sutteflaske',
    '🥛' => 'glas mælk',
    '☕' => 'varm drik',
    '🍵' => 'tekop',
    '🍶' => 'sake',
    '🍾' => 'flaske med flyvende prop',
    '🍷' => 'vin',
    '🍸' => 'cocktailglas',
    '🍹' => 'tropisk drink',
    '🍺' => 'ølkrus',
    '🍻' => 'skålende ølkrus',
    '🥂' => 'skål',
    '🥃' => 'fladbundet glas',
    '🍽' => 'kniv og gaffel med tallerken',
    '🍴' => 'kniv og gaffel',
    '🥄' => 'ske',
    '🔪' => 'køkkenkniv',
    '🏺' => 'amphora',
    '🌍' => 'globus med europa og afrika',
    '🌎' => 'globus med nord- og sydamerika',
    '🌏' => 'globus med asien og australien',
    '🌐' => 'globus med længdegrader',
    '🗺' => 'verdenskort',
    '🗾' => 'Japan',
    '🏔' => 'sneklædt bjerg',
    '⛰' => 'bjerg',
    '🌋' => 'vulkan',
    '🗻' => 'Mount Fuji',
    '🏕' => 'camping',
    '🏖' => 'strand med parasol',
    '🏜' => 'ørken',
    '🏝' => 'øde ø',
    '🏞' => 'nationalpark',
    '🏟' => 'stadion',
    '🏛' => 'klassisk bygning',
    '🏗' => 'bygningskonstruktion',
    '🏘' => 'huse',
    '🏚' => 'faldefærdigt hus',
    '🏠' => 'hus',
    '🏡' => 'hus med have',
    '🏢' => 'kontorbygning',
    '🏣' => 'japansk posthus',
    '🏤' => 'posthus',
    '🏥' => 'hospital',
    '🏦' => 'bank',
    '🏨' => 'hotel',
    '🏩' => 'kærlighedshotel',
    '🏪' => 'døgnkiosk',
    '🏫' => 'skole',
    '🏬' => 'stormagasin',
    '🏭' => 'fabrik',
    '🏯' => 'japansk slot',
    '🏰' => 'slot',
    '💒' => 'bryllup',
    '🗼' => 'Tokyo Tower',
    '🗽' => 'Frihedsgudinden',
    '⛪' => 'kirke',
    '🕌' => 'moské',
    '🕍' => 'synagoge',
    '⛩' => 'shinto-alter',
    '🕋' => 'kaaba',
    '⛲' => 'springvand',
    '⛺' => 'telt',
    '🌁' => 'tåget',
    '🌃' => 'stjernehimmel',
    '🏙' => 'bylandskab',
    '🌄' => 'solopgang over bjerge',
    '🌅' => 'solopgang',
    '🌆' => 'by i solnedgang',
    '🌇' => 'solnedgang',
    '🌉' => 'bro i mørke',
    '♨' => 'varme kilder',
    '🎠' => 'karrusel',
    '🎡' => 'pariserhjul',
    '🎢' => 'rutsjebane',
    '💈' => 'barber',
    '🎪' => 'cirkus',
    '🚂' => 'damplokomotiv',
    '🚃' => 'togvogn',
    '🚄' => 'lyntog',
    '🚅' => 'lyntog med projektilnæse',
    '🚆' => 'tog',
    '🚇' => 'metro',
    '🚈' => 'letbane',
    '🚉' => 'togstation',
    '🚊' => 'sporvogn',
    '🚝' => 'monorail',
    '🚞' => 'bjergbane',
    '🚋' => 'vogn',
    '🚌' => 'bus',
    '🚍' => 'bus forfra',
    '🚎' => 'trolleybus',
    '🚐' => 'minibus',
    '🚑' => 'ambulance',
    '🚒' => 'brandbil',
    '🚓' => 'politibil',
    '🚔' => 'politibil forfra',
    '🚕' => 'taxa',
    '🚖' => 'taxa forfra',
    '🚗' => 'bil',
    '🚘' => 'bil forfra',
    '🚙' => 'firehjulstræk',
    '🚚' => 'lastbil',
    '🚛' => 'lastvogn',
    '🚜' => 'traktor',
    '🏎' => 'racerbil',
    '🏍' => 'motorcykel',
    '🛵' => 'scooter',
    '🚲' => 'cykel',
    '🛴' => 'løbehjul',
    '🚏' => 'busstoppested',
    '🛣' => 'motorvej',
    '🛤' => 'togspor',
    '🛢' => 'olietønde',
    '⛽' => 'benzin',
    '🚨' => 'blinklys',
    '🚥' => 'liggende trafiklys',
    '🚦' => 'trafiklys',
    '🛑' => 'stopskilt',
    '🚧' => 'byggeri',
    '⚓' => 'anker',
    '⛵' => 'sejlbåd',
    '🛶' => 'kano',
    '🚤' => 'speedbåd',
    '🛳' => 'passagerskib',
    '⛴' => 'færge',
    '🛥' => 'motorbåd',
    '🚢' => 'skib',
    '✈' => 'fly',
    '🛩' => 'lille fly',
    '🛫' => 'fly afgang',
    '🛬' => 'fly ankomst',
    '💺' => 'sæde',
    '🚁' => 'helikopter',
    '🚟' => 'luftbane',
    '🚠' => 'kabelbane',
    '🚡' => 'svævebane',
    '🛰' => 'satellit',
    '🚀' => 'raket',
    '🛎' => 'hotelklokke',
    '⌛' => 'timeglas',
    '⏳' => 'timeglas med løbende sand',
    '⌚' => 'ur',
    '⏰' => 'vækkeur',
    '⏱' => 'stopur',
    '⏲' => 'æggeur',
    '🕰' => 'pynteur',
    '🕛' => 'kl.12',
    '🕐' => 'kl. 1',
    '🕜' => 'halv to',
    '🕑' => 'kl. 2',
    '🕝' => 'halv tre',
    '🕒' => 'kl. 3',
    '🕞' => 'halv fire',
    '🕓' => 'kl. 4',
    '🕟' => 'halv fem',
    '🕔' => 'kl. 5',
    '🕠' => 'halv seks',
    '🕕' => 'kl. 6',
    '🕡' => 'halv syv',
    '🕖' => 'kl. 7',
    '🕢' => 'halv otte',
    '🕗' => 'kl. 8',
    '🕣' => 'halv ni',
    '🕘' => 'kl. 9',
    '🕤' => 'halv ti',
    '🕙' => 'kl. 10',
    '🕥' => 'halv elleve',
    '🕚' => 'kl.11',
    '🕦' => 'halv tolv',
    '🌑' => 'nymåne',
    '🌒' => 'tiltagende måne',
    '🌓' => 'halvmåne første kvarter',
    '🌔' => 'tiltagende måne næsten fuld',
    '🌕' => 'fuldmåne',
    '🌖' => 'aftagende måne næsten fuld',
    '🌗' => 'halvmåne sidste kvarter',
    '🌘' => 'aftagende måne',
    '🌙' => 'halvmåne',
    '🌚' => 'nymåne med ansigt',
    '🌛' => 'måne med ansigt første kvarter',
    '🌜' => 'måne med ansigt andet kvarter',
    '🌡' => 'termometer',
    '☀' => 'sol',
    '🌝' => 'fuldmåneansigt',
    '🌞' => 'sol med ansigt',
    '⭐' => 'hvid mellemstor stjerne',
    '🌟' => 'blinkende stjerne',
    '🌠' => 'stjerneskud',
    '🌌' => 'mælkevejen',
    '☁' => 'sky',
    '⛅' => 'sol bag sky',
    '⛈' => 'sky med lyn og regn',
    '🌤' => 'sol bag lille sky',
    '🌥' => 'sol bag stor sky',
    '🌦' => 'sol bag sky med regn',
    '🌧' => 'sky med regn',
    '🌨' => 'sky med sne',
    '🌩' => 'tordensky',
    '🌪' => 'tornado',
    '🌫' => 'tåge',
    '🌬' => 'vindansigt',
    '🌀' => 'cyklon',
    '🌈' => 'regnbue',
    '🌂' => 'lukket paraply',
    '☂' => 'paraply',
    '☔' => 'paraply med regndråber',
    '⛱' => 'parasol',
    '⚡' => 'højspænding',
    '❄' => 'snefnug',
    '☃' => 'snemand',
    '⛄' => 'snemand uden snevejr',
    '☄' => 'komet',
    '🔥' => 'ild',
    '💧' => 'dråbe',
    '🌊' => 'bølge',
    '🎃' => 'græskarlygte',
    '🎄' => 'juletræ',
    '🎆' => 'fyrværkeri',
    '🎇' => 'stjernekaster',
    '✨' => 'stjerner',
    '🎈' => 'ballon',
    '🎉' => 'bordbombe',
    '🎊' => 'konfetti',
    '🎋' => 'træ med papirpynt',
    '🎍' => 'fyrtræsdekoration',
    '🎎' => 'dukker',
    '🎏' => 'banner',
    '🎐' => 'vindspil',
    '🎑' => 'måneceremoni',
    '🎀' => 'sløjfe',
    '🎁' => 'gave',
    '🎗' => 'mindebånd',
    '🎟' => 'adgangsbillet',
    '🎫' => 'billet',
    '🎖' => 'militærmedalje',
    '🏆' => 'pokal',
    '🏅' => 'sportsmedalje',
    '🥇' => 'guldmedalje',
    '🥈' => 'sølvmedalje',
    '🥉' => 'bronzemedalje',
    '⚽' => 'fodbold',
    '⚾' => 'baseball',
    '🏀' => 'basketball',
    '🏐' => 'volleyball',
    '🏈' => 'amerikansk fodbold',
    '🏉' => 'rugbybold',
    '🎾' => 'tennis',
    '🎳' => 'bowling',
    '🏏' => 'cricket',
    '🏑' => 'hockey',
    '🏒' => 'ishockeystav og puck',
    '🏓' => 'bordtennis',
    '🏸' => 'badminton',
    '🥊' => 'boksehandske',
    '🥋' => 'kampsportsuniform',
    '🥅' => 'målnet',
    '⛳' => 'golf flag',
    '⛸' => 'skøjte',
    '🎣' => 'fiske',
    '🎽' => 'løbetrøje',
    '🎿' => 'ski',
    '🎯' => 'pletskud',
    '🔫' => 'pistol',
    '🎱' => 'billard',
    '🔮' => 'krystalkugle',
    '🎮' => 'videospil',
    '🕹' => 'joystick',
    '🎰' => 'spillemaskine',
    '🎲' => 'terning',
    '♠' => 'spar',
    '♥' => 'hjerter',
    '♦' => 'ruder',
    '♣' => 'klør',
    '🃏' => 'joker',
    '🀄' => 'mahjong-brik',
    '🎴' => 'spillekort',
    '🎭' => 'masker',
    '🖼' => 'ramme med billede',
    '🎨' => 'palet',
    '👓' => 'briller',
    '🕶' => 'solbriller',
    '👔' => 'slips',
    '👕' => 'T-shirt',
    '👖' => 'bukser',
    '👗' => 'kjole',
    '👘' => 'kimono',
    '👙' => 'bikini',
    '👚' => 'bluse',
    '👛' => 'pung',
    '👜' => 'håndtaske',
    '👝' => 'taske',
    '🛍' => 'indkøbsposer',
    '🎒' => 'rygsæk',
    '👞' => 'herresko',
    '👟' => 'gummisko',
    '👠' => 'høje hæle',
    '👡' => 'damesandaler',
    '👢' => 'damestøvler',
    '👑' => 'krone',
    '👒' => 'hat',
    '🎩' => 'høj hat',
    '🎓' => 'eksamenshat',
    '⛑' => 'hjelm med hvidt kors',
    '📿' => 'bedekrans',
    '💄' => 'læbestift',
    '💍' => 'ring',
    '💎' => 'diamant',
    '🔇' => 'højttaler fra',
    '🔈' => 'højttaler',
    '🔉' => 'højttaler til',
    '🔊' => 'højttaler høj',
    '📢' => 'højttaler med håndtag',
    '📣' => 'megafon',
    '📯' => 'posthorn',
    '🔔' => 'klokke',
    '🔕' => 'ingen klokke',
    '🎼' => 'partitur',
    '🎵' => 'node',
    '🎶' => 'noder',
    '🎙' => 'studiemikrofon',
    '🎚' => 'volumenkontrol',
    '🎛' => 'knapper',
    '🎤' => 'mikrofon',
    '🎧' => 'hovedtelefon',
    '📻' => 'radio',
    '🎷' => 'saxofon',
    '🎺' => 'trompet',
    '🎸' => 'guitar',
    '🎹' => 'klaver',
    '🎻' => 'violin',
    '🥁' => 'tromme',
    '📱' => 'mobil',
    '📲' => 'mobil med pil',
    '☎' => 'telefon',
    '📞' => 'telefonrør',
    '📟' => 'personsøger',
    '📠' => 'fax',
    '🔋' => 'batteri',
    '🔌' => 'stik',
    '💻' => 'computer',
    '🖥' => 'stationær computer',
    '🖨' => 'printer',
    '⌨' => 'tastatur',
    '🖱' => 'computermus',
    '🖲' => 'trackball',
    '💽' => 'mini disk',
    '💾' => 'diskette',
    '💿' => 'cd',
    '📀' => 'DVD',
    '🎥' => 'filmkamera',
    '🎞' => 'filmstrimmel',
    '📽' => 'filmprojektor',
    '🎬' => 'klaptræ',
    '📺' => 'tv',
    '📷' => 'kamera',
    '📸' => 'kamera med blitz',
    '📹' => 'videokamera',
    '📼' => 'videobånd',
    '🔍' => 'forstørrelsesglas til venstre',
    '🔎' => 'forstørrelsesglas til højre',
    '🕯' => 'stearinlys',
    '💡' => 'elpære',
    '🔦' => 'lommelygte',
    '🏮' => 'lygte',
    '📔' => 'notesbog med udsmykning',
    '📕' => 'lukket bog',
    '📖' => 'åben bog',
    '📗' => 'grøn bog',
    '📘' => 'blå bog',
    '📙' => 'orange bog',
    '📚' => 'bøger',
    '📓' => 'notesbog',
    '📒' => 'regnskabsbog',
    '📃' => 'side med fold',
    '📜' => 'skriftrulle',
    '📄' => 'dokument',
    '📰' => 'avis',
    '🗞' => 'sammenrullet avis',
    '📑' => 'faner',
    '🔖' => 'bogmærke',
    '🏷' => 'label',
    '💰' => 'pengepose',
    '💴' => 'yenseddel',
    '💵' => 'dollarseddel',
    '💶' => 'euroseddel',
    '💷' => 'pundseddel',
    '💸' => 'flyvende penge',
    '💳' => 'kreditkort',
    '💹' => 'stigende marked',
    '✉' => 'konvolut',
    '📧' => 'e-mail',
    '📨' => 'indgående post',
    '📩' => 'udgående post',
    '📤' => 'udbakke',
    '📥' => 'indbakke',
    '📦' => 'pakke',
    '📫' => 'lukket postkasse med hævet flag',
    '📪' => 'lukket postkasse med sænket flag',
    '📬' => 'åben postkasse med hævet flag',
    '📭' => 'åben postkasse med sænket flag',
    '📮' => 'postkasse',
    '🗳' => 'valgurne med stemmeseddel',
    '✏' => 'blyant',
    '✒' => 'sort fyldepen',
    '🖋' => 'fyldepen',
    '🖊' => 'kuglepen',
    '🖌' => 'malerpensel',
    '🖍' => 'farvekridt',
    '📝' => 'notat',
    '💼' => 'attachémappe',
    '📁' => 'mappe',
    '📂' => 'åben mappe',
    '🗂' => 'kartotekskort',
    '📅' => 'kalender',
    '📆' => 'kalenderblok',
    '🗒' => 'notesblok',
    '🗓' => 'spiralkalender',
    '📇' => 'kartotek',
    '📈' => 'opadgående graf',
    '📉' => 'nedadgående graf',
    '📊' => 'søjlediagram',
    '📋' => 'clipboard',
    '📌' => 'tegnestift',
    '📍' => 'rund tegnestift',
    '📎' => 'papirclips',
    '🖇' => 'sammensatte papirclips',
    '📏' => 'lineal',
    '📐' => 'trekant',
    '✂' => 'saks',
    '🗃' => 'arkivkasse',
    '🗄' => 'arkivskab',
    '🗑' => 'papirkurv',
    '🔒' => 'hængelås',
    '🔓' => 'åben hængelås',
    '🔏' => 'hængelås med pen',
    '🔐' => 'lukket hængelås med nøgle',
    '🔑' => 'nøgle',
    '🗝' => 'gammel nøgle',
    '🔨' => 'hammer',
    '⛏' => 'hakke',
    '⚒' => 'hammer og hakke',
    '🛠' => 'hammer og skruenøgle',
    '🗡' => 'daggert',
    '⚔' => 'korslagte sværd',
    '💣' => 'bombe',
    '🏹' => 'bue og pil',
    '🛡' => 'skjold',
    '🔧' => 'skruenøgle',
    '🔩' => 'møtrik og bolt',
    '⚙' => 'tandhjul',
    '🗜' => 'skruetvinge',
    '⚖' => 'vægt',
    '🔗' => 'link',
    '⛓' => 'kæder',
    '⚗' => 'kolbe',
    '🔬' => 'mikroskop',
    '🔭' => 'teleskop',
    '📡' => 'parabolantenne',
    '💉' => 'sprøjte',
    '💊' => 'pille',
    '🚪' => 'dør',
    '🛏' => 'seng',
    '🛋' => 'sofa og lampe',
    '🚽' => 'toilet',
    '🚿' => 'brusebad',
    '🛁' => 'badekar',
    '🛒' => 'indkøbsvogn',
    '🚬' => 'rygeskilt',
    '⚰' => 'kiste',
    '⚱' => 'urne',
    '🗿' => 'statue',
    '🏧' => 'pengeautomat',
    '🚮' => 'smid i skraldespanden',
    '🚰' => 'drikkevand',
    '♿' => 'rullestolsadgang',
    '🚹' => 'mænd',
    '🚺' => 'kvinder',
    '🚻' => 'toiletskilt',
    '🚼' => 'babysymbol',
    '🚾' => 'wc',
    '🛂' => 'paskontrol',
    '🛃' => 'told',
    '🛄' => 'baggageudlevering',
    '🛅' => 'hittegods',
    '⚠' => 'advarselsskilt',
    '🚸' => 'børn krydser vej',
    '⛔' => 'adgang forbudt',
    '🚫' => 'ingen adgang',
    '🚳' => 'cykler forbudt',
    '🚭' => 'rygning forbudt',
    '🚯' => 'smid ikke affald',
    '🚱' => 'ikke drikkevand',
    '🚷' => 'ingen fodgængere',
    '📵' => 'mobiltelefoner forbudt',
    '🔞' => 'forbudt for under 18',
    '☢' => 'radioaktiv',
    '☣' => 'biologisk fare',
    '⬆' => 'opadvendt pil',
    '↗' => 'opadvendt pil mod højre',
    '➡' => 'pil mod højre',
    '↘' => 'nedadvendt pil mod højre',
    '⬇' => 'nedadvendt pil',
    '↙' => 'nedadvendt pil mod venstre',
    '⬅' => 'pil mod venstre',
    '↖' => 'opadvendt pil mod venstre',
    '↕' => 'op- og nedadvendt pil',
    '↔' => 'pil mod venstre og højre',
    '↩' => 'pil mod højre med sving til venstre',
    '↪' => 'pil mod venstre med sving til højre',
    '⤴' => 'pil mod højre med sving opad',
    '⤵' => 'pil mod højre med sving nedad',
    '🔃' => 'genindlæse',
    '🔄' => 'mod uret',
    '🔙' => 'tilbage',
    '🔚' => 'end',
    '🔛' => 'ON-pil',
    '🔜' => 'snart',
    '🔝' => 'top',
    '🛐' => 'bedested',
    '⚛' => 'atomsymbol',
    '🕉' => 'om',
    '✡' => 'davidsstjerne',
    '☸' => 'dharma-hjul',
    '☯' => 'yin og yang',
    '✝' => 'latinsk kors',
    '☦' => 'ortodokst kors',
    '☪' => 'stjerne og halvmåne',
    '☮' => 'fredssymbol',
    '🕎' => 'syvarmet lysestage',
    '🔯' => 'sekstakket stjerne med prik',
    '♈' => 'vædderen',
    '♉' => 'tyren',
    '♊' => 'tvillingen',
    '♋' => 'krebsen',
    '♌' => 'løven',
    '♍' => 'jomfruen',
    '♎' => 'vægten',
    '♏' => 'skorpionen',
    '♐' => 'skytten',
    '♑' => 'stenbukken',
    '♒' => 'vandmanden',
    '♓' => 'fisken',
    '⛎' => 'slangebæreren',
    '🔀' => 'krydsede pile',
    '🔁' => 'gentageknap',
    '🔂' => 'knap til én gentagelse',
    '▶' => 'afspilningsknap',
    '⏩' => 'spole frem',
    '⏭' => 'knap til næste nummer',
    '⏯' => 'knap til afspilning og pause',
    '◀' => 'knap til baglæns afspilning',
    '⏪' => 'spole tilbage',
    '⏮' => 'knap til forrige nummer',
    '🔼' => 'op-knap',
    '⏫' => 'pil op',
    '🔽' => 'ned-knap',
    '⏬' => 'pil ned',
    '⏸' => 'pauseknap',
    '⏹' => 'stopknap',
    '⏺' => 'optageknap',
    '⏏' => 'udskubningsknap',
    '🎦' => 'film',
    '🔅' => 'lav lysstyrke',
    '🔆' => 'høj lysstyrke',
    '📶' => 'mobilsignal',
    '📳' => 'vibration',
    '📴' => 'mobil slået fra',
    '✖' => 'stort kryds',
    '➕' => 'plus',
    '➖' => 'minus',
    '➗' => 'division',
    '‼' => 'dobbelt udråbstegn',
    '⁉' => 'udråbstegn og spørgsmålstegn',
    '❓' => 'spørgsmålstegn',
    '❔' => 'hvidt spørgsmålstegn',
    '❕' => 'hvidt udråbstegn',
    '❗' => 'udråbstegn',
    '〰' => 'bølgestreg',
    '💱' => 'valutaveksling',
    '💲' => 'dollartegn',
    '⚕' => 'medicinsymbol',
    '♻' => 'genbrug',
    '⚜' => 'fransk lilje',
    '🔱' => 'trefork',
    '📛' => 'navneskilt',
    '🔰' => 'grønt og gult blad',
    '⭕' => 'stor kraftig cirkel',
    '✅' => 'flueben',
    '☑' => 'afkrydsningsfelt med flueben',
    '✔' => 'stort flueben',
    '❌' => 'kryds',
    '❎' => 'krydsknap',
    '➰' => 'krølle',
    '➿' => 'dobbelt krølle',
    '〽' => 'japansk kommatering',
    '✳' => 'ottetakket spids stjerne',
    '✴' => 'ottetakket stjerne',
    '❇' => 'funkle',
    '™' => 'varemærke',
    '🔠' => 'store bogstaver',
    '🔡' => 'små bogstaver',
    '🔢' => 'tal',
    '🔣' => 'symboler',
    '🔤' => 'latinske alfabet',
    '🅰' => 'blodtype A',
    '🆎' => 'blodtype AB',
    '🅱' => 'blodtype B',
    '🆑' => 'ryd',
    '🆒' => 'cool',
    '🆓' => 'gratis',
    'ℹ' => 'informationskilde',
    '🆔' => 'id',
    'Ⓜ' => 'm i cirkel',
    '🆕' => 'nyt',
    '🆖' => 'ikke godt',
    '🅾' => 'blodtype O',
    '🆗' => 'OK',
    '🅿' => 'parkering',
    '🆘' => 'SOS',
    '🆙' => 'OP!',
    '🆚' => 'versus',
    '🈁' => 'kvadratisk katakana koko',
    '🈂' => 'kvadratisk katakana sa',
    '🈷' => 'kvadratisk ideogram for måne',
    '🈶' => 'kvadratisk ideogram for eksistens',
    '🈯' => 'kvadratisk ideogram for finger',
    '🉐' => 'cirkulært ideogram for fordel',
    '🈹' => 'kvadratisk ideogram for opdeling',
    '🈚' => 'kvadratisk ideogram for forhandling',
    '🈲' => 'kvadratisk ideogram for forbud',
    '🉑' => 'cirkulært ideogram for accept',
    '🈸' => 'kvadratisk ideogram for anvendelse',
    '🈴' => 'kvadratisk ideogram for sammen',
    '🈳' => 'kvadratisk ideogram for tom',
    '㊗' => 'cirkulært ideogram for lykønskning',
    '㊙' => 'cirkulært ideogram for hemmelighed',
    '🈺' => 'kvadratisk ideogram for drift',
    '🈵' => 'kvadratisk ideogram for fuld',
    '🔴' => 'rød cirkel',
    '🔵' => 'blå cirkel',
    '⚫' => 'sort cirkel',
    '⚪' => 'hvid cirkel',
    '⬛' => 'stort sort kvadrat',
    '⬜' => 'stort hvidt kvadrat',
    '◼' => 'mellemstort sort kvadrat',
    '◻' => 'mellemstort hvidt kvadrat',
    '◾' => 'halvlille sort kvadrat',
    '◽' => 'halvlille hvidt kvadrat',
    '▪' => 'lille sort kvadrat',
    '▫' => 'lille hvidt kvadrat',
    '🔶' => 'stor orange diamant',
    '🔷' => 'stor blå diamant',
    '🔸' => 'lille orange diamant',
    '🔹' => 'lille blå diamant',
    '🔺' => 'op-trekant',
    '🔻' => 'ned-trekant',
    '💠' => 'diamant med prik',
    '🔘' => 'knap',
    '🔳' => 'hvid kvadratisk knap',
    '🔲' => 'sort kvadratisk knap',
    '🏁' => 'ternet flag',
    '🚩' => 'flag',
    '🎌' => 'to flag',
    '🏴' => 'sort flag',
    '🏳' => 'hvidt flag',
];
