<?php return array (
  '@hotwired/stimulus' =>
  array (
    'version' => '3.2.1',
    'dependencies' => array(),
  ),
  'lodash' =>
  array (
    'version' => '4.17.21',
    'dependencies' => array(),
  ),
);
