<?php

return [
    'Names' => [
        'AED' => [
            'AED',
            'dirham del Emiratos Arabe Unite',
        ],
        'AFN' => [
            'AFN',
            'afghani',
        ],
        'ALL' => [
            'ALL',
            'lek albanese',
        ],
        'AMD' => [
            'AMD',
            'dram armenie',
        ],
        'ANG' => [
            'ANG',
            'florino antillan',
        ],
        'AOA' => [
            'AOA',
            'kwanza angolan',
        ],
        'ARS' => [
            'ARS',
            'peso argentin',
        ],
        'AUD' => [
            'A$',
            'dollar australian',
        ],
        'AWG' => [
            'AWG',
            'florino aruban',
        ],
        'AZN' => [
            'AZN',
            'manat azeri',
        ],
        'BAM' => [
            'BAM',
            'marco convertibile de Bosnia-Herzegovina',
        ],
        'BBD' => [
            'BBD',
            'dollar barbadian',
        ],
        'BDT' => [
            'BDT',
            'taka bengalese',
        ],
        'BGN' => [
            'BGN',
            'lev bulgare',
        ],
        'BHD' => [
            'BHD',
            'dinar bahreini',
        ],
        'BIF' => [
            'BIF',
            'franco burundese',
        ],
        'BMD' => [
            'BMD',
            'dollar bermudan',
        ],
        'BND' => [
            'BND',
            'dollar de Brunei',
        ],
        'BOB' => [
            'BOB',
            'boliviano bolivian',
        ],
        'BRL' => [
            'R$',
            'real brasilian',
        ],
        'BSD' => [
            'BSD',
            'dollar bahamian',
        ],
        'BTN' => [
            'BTN',
            'ngultrum bhutanese',
        ],
        'BWP' => [
            'BWP',
            'pula botswanese',
        ],
        'BYN' => [
            'BYN',
            'rublo bielorusse',
        ],
        'BZD' => [
            'BZD',
            'dollar belizan',
        ],
        'CAD' => [
            'CA$',
            'dollar canadian',
        ],
        'CDF' => [
            'CDF',
            'franco congolese',
        ],
        'CHF' => [
            'CHF',
            'franco suisse',
        ],
        'CLP' => [
            'CLP',
            'peso chilen',
        ],
        'CNH' => [
            'CNH',
            'yuan chinese (extracontinental)',
        ],
        'CNY' => [
            'CN¥',
            'yuan chinese',
        ],
        'COP' => [
            'COP',
            'peso colombian',
        ],
        'CRC' => [
            'CRC',
            'colon costarican',
        ],
        'CUC' => [
            'CUC',
            'peso cuban convertibile',
        ],
        'CUP' => [
            'CUP',
            'peso cuban',
        ],
        'CVE' => [
            'CVE',
            'escudo capoverdian',
        ],
        'CZK' => [
            'CZK',
            'corona chec',
        ],
        'DEM' => [
            'DEM',
            'Marco geman',
        ],
        'DJF' => [
            'DJF',
            'franco djibutian',
        ],
        'DKK' => [
            'DKK',
            'corona danese',
        ],
        'DOP' => [
            'DOP',
            'peso dominican',
        ],
        'DZD' => [
            'DZD',
            'dinar algerin',
        ],
        'EEK' => [
            'EEK',
            'Corona estonian',
        ],
        'EGP' => [
            'EGP',
            'libra egyptie',
        ],
        'ERN' => [
            'ERN',
            'nakfa eritree',
        ],
        'ETB' => [
            'ETB',
            'birr ethiope',
        ],
        'EUR' => [
            '€',
            'euro',
        ],
        'FIM' => [
            'FIM',
            'Marco finnese',
        ],
        'FJD' => [
            'FJD',
            'dollar fijian',
        ],
        'FKP' => [
            'FKP',
            'libra falklandese',
        ],
        'FRF' => [
            'FRF',
            'Franco francese',
        ],
        'GBP' => [
            '£',
            'libra sterling',
        ],
        'GEL' => [
            'GEL',
            'lari georgian',
        ],
        'GHS' => [
            'GHS',
            'cedi ghanese',
        ],
        'GIP' => [
            'GIP',
            'libra de Gibraltar',
        ],
        'GMD' => [
            'GMD',
            'dalasi gambian',
        ],
        'GNF' => [
            'GNF',
            'franco guinean',
        ],
        'GTQ' => [
            'GTQ',
            'quetzal guatemaltec',
        ],
        'GYD' => [
            'GYD',
            'dollar guyanese',
        ],
        'HKD' => [
            'HK$',
            'dollar hongkongese',
        ],
        'HNL' => [
            'HNL',
            'lempira hondurese',
        ],
        'HRK' => [
            'HRK',
            'kuna croate',
        ],
        'HTG' => [
            'HTG',
            'gourde haitian',
        ],
        'HUF' => [
            'HUF',
            'forint hungare',
        ],
        'IDR' => [
            'IDR',
            'rupia indonesian',
        ],
        'IEP' => [
            'IEP',
            'Libra irlandese',
        ],
        'ILS' => [
            '₪',
            'nove shekel israeli',
        ],
        'INR' => [
            '₹',
            'rupia indian',
        ],
        'IQD' => [
            'IQD',
            'dinar iraqi',
        ],
        'IRR' => [
            'IRR',
            'rial iranian',
        ],
        'ISK' => [
            'ISK',
            'corona islandese',
        ],
        'JMD' => [
            'JMD',
            'dollar jamaican',
        ],
        'JOD' => [
            'JOD',
            'dinar jordan',
        ],
        'JPY' => [
            'JP¥',
            'yen japonese',
        ],
        'KES' => [
            'KES',
            'shilling kenyan',
        ],
        'KGS' => [
            'KGS',
            'som kirghiz',
        ],
        'KHR' => [
            'KHR',
            'riel cambodgian',
        ],
        'KMF' => [
            'KMF',
            'franco comorian',
        ],
        'KPW' => [
            'KPW',
            'won nordkorean',
        ],
        'KRW' => [
            '₩',
            'won sudkorean',
        ],
        'KWD' => [
            'KWD',
            'dinar kuwaiti',
        ],
        'KYD' => [
            'KYD',
            'dollar del Insulas Caiman',
        ],
        'KZT' => [
            'KZT',
            'tenge kazakh',
        ],
        'LAK' => [
            'LAK',
            'kip laotian',
        ],
        'LBP' => [
            'LBP',
            'libra libanese',
        ],
        'LKR' => [
            'LKR',
            'rupia de Sri Lanka',
        ],
        'LRD' => [
            'LRD',
            'dollar liberian',
        ],
        'LSL' => [
            'LSL',
            'loti de Lesotho',
        ],
        'LYD' => [
            'LYD',
            'dinar libyc',
        ],
        'MAD' => [
            'MAD',
            'dirham marocchin',
        ],
        'MDL' => [
            'MDL',
            'leu moldave',
        ],
        'MGA' => [
            'MGA',
            'ariary malgache',
        ],
        'MKD' => [
            'MKD',
            'denar macedonie',
        ],
        'MMK' => [
            'MMK',
            'kyat de Myanmar',
        ],
        'MNT' => [
            'MNT',
            'tugrik mongol',
        ],
        'MOP' => [
            'MOP',
            'pataca de Macao',
        ],
        'MRO' => [
            'MRO',
            'ouguiya mauritan (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'ouguiya mauritan',
        ],
        'MUR' => [
            'MUR',
            'rupia mauritian',
        ],
        'MVR' => [
            'MVR',
            'rufiyaa del Maldivas',
        ],
        'MWK' => [
            'MWK',
            'kwacha malawian',
        ],
        'MXN' => [
            'MX$',
            'peso mexican',
        ],
        'MYR' => [
            'MYR',
            'ringgit malay',
        ],
        'MZN' => [
            'MZN',
            'metical mozambican',
        ],
        'NAD' => [
            'NAD',
            'dollar namibian',
        ],
        'NGN' => [
            'NGN',
            'naira nigerian',
        ],
        'NIO' => [
            'NIO',
            'cordoba nicaraguan',
        ],
        'NLG' => [
            'ƒ',
            'Florino nederlandese',
        ],
        'NOK' => [
            'NOK',
            'corona norvegian',
        ],
        'NPR' => [
            'NPR',
            'rupia nepalese',
        ],
        'NZD' => [
            'NZ$',
            'dollar neozelandese',
        ],
        'OMR' => [
            'OMR',
            'rial omani',
        ],
        'PAB' => [
            'PAB',
            'balboa panamen',
        ],
        'PEN' => [
            'PEN',
            'sol peruvian',
        ],
        'PGK' => [
            'PGK',
            'kina papuan',
        ],
        'PHP' => [
            '₱',
            'peso philippin',
        ],
        'PKR' => [
            'PKR',
            'rupia pakistani',
        ],
        'PLN' => [
            'PLN',
            'zloty polonese',
        ],
        'PYG' => [
            'PYG',
            'guarani paraguayan',
        ],
        'QAR' => [
            'QAR',
            'rial qatari',
        ],
        'RON' => [
            'RON',
            'leu romanian',
        ],
        'RSD' => [
            'RSD',
            'dinar serbe',
        ],
        'RUB' => [
            '₽',
            'rublo russe',
        ],
        'RWF' => [
            'RWF',
            'franco ruandese',
        ],
        'SAR' => [
            'SAR',
            'rial saudi',
        ],
        'SBD' => [
            'SBD',
            'dollar del insulas Salomon',
        ],
        'SCR' => [
            'SCR',
            'rupia seychellese',
        ],
        'SDG' => [
            'SDG',
            'libra sudanese',
        ],
        'SEK' => [
            'SEK',
            'corona svedese',
        ],
        'SGD' => [
            'SGD',
            'dollar singaporese',
        ],
        'SHP' => [
            'SHP',
            'libra de St. Helena',
        ],
        'SLE' => [
            'SLE',
            'leone sierraleonese',
        ],
        'SLL' => [
            'SLL',
            'leone sierraleonese (1964—2022)',
        ],
        'SOS' => [
            'SOS',
            'shilling somali',
        ],
        'SRD' => [
            'SRD',
            'dollar surinamese',
        ],
        'SSP' => [
            'SSP',
            'libra sud-sudanese',
        ],
        'STN' => [
            'STN',
            'dobra de São Tomé e Príncipe',
        ],
        'SYP' => [
            'SYP',
            'libra syriac',
        ],
        'SZL' => [
            'SZL',
            'lilangeni swazilandese',
        ],
        'THB' => [
            'THB',
            'baht thailandese',
        ],
        'TJS' => [
            'TJS',
            'somoni tajik',
        ],
        'TMT' => [
            'TMT',
            'manat turkmen',
        ],
        'TND' => [
            'TND',
            'dinar tunisian',
        ],
        'TOP' => [
            'TOP',
            'paʻanga tongan',
        ],
        'TRY' => [
            'TRY',
            'lira turc',
        ],
        'TTD' => [
            'TTD',
            'dollar de Trinidad e Tobago',
        ],
        'TWD' => [
            'NT$',
            'nove dollar taiwanese',
        ],
        'TZS' => [
            'TZS',
            'shilling tanzanian',
        ],
        'UAH' => [
            'UAH',
            'hryvnia ukrainian',
        ],
        'UGX' => [
            'UGX',
            'shilling ugandese',
        ],
        'USD' => [
            'US$',
            'dollar statounitese',
        ],
        'UYU' => [
            'UYU',
            'peso uruguayan',
        ],
        'UZS' => [
            'UZS',
            'som uzbek',
        ],
        'VEF' => [
            'VEF',
            'bolivar venezuelan (2008–2018)',
        ],
        'VES' => [
            'VES',
            'bolivar venezuelan',
        ],
        'VND' => [
            '₫',
            'dong vietnamese',
        ],
        'VUV' => [
            'VUV',
            'vatu vanuatuan',
        ],
        'WST' => [
            'WST',
            'tala samoan',
        ],
        'XAF' => [
            'FCFA',
            'franco CFA de Africa Central',
        ],
        'XCD' => [
            'EC$',
            'dollar del Caribes Oriental',
        ],
        'XCG' => [
            'Cg.',
            'florino caribe',
        ],
        'XOF' => [
            'F CFA',
            'franco CFA de Africa Occidental',
        ],
        'XPF' => [
            'CFPF',
            'franco CFP',
        ],
        'YER' => [
            'YER',
            'rial yemeni',
        ],
        'ZAR' => [
            'ZAR',
            'rand sudafrican',
        ],
        'ZMW' => [
            'ZMW',
            'kwacha zambian',
        ],
        'ZWG' => [
            'ZWG',
            'auro de Zimbwabwe',
        ],
    ],
];
