<?php

return [
    'Names' => [
        'af' => 'Αφρικάανς',
        'af_NA' => 'Αφρικάανς (Ναμίμπια)',
        'af_ZA' => 'Αφρικάανς (Νότια Αφρική)',
        'ak' => 'Ακάν',
        'ak_GH' => 'Ακάν (Γκάνα)',
        'am' => 'Αμχαρικά',
        'am_ET' => 'Αμχαρικά (Αιθιοπία)',
        'ar' => 'Αραβικά',
        'ar_001' => 'Αραβικά (Κόσμος)',
        'ar_AE' => 'Αραβικά (Ηνωμένα Αραβικά Εμιράτα)',
        'ar_BH' => 'Αραβικά (Μπαχρέιν)',
        'ar_DJ' => 'Αραβικά (Τζιμπουτί)',
        'ar_DZ' => 'Αραβικά (Αλγερία)',
        'ar_EG' => 'Αραβικά (Αίγυπτος)',
        'ar_EH' => 'Αραβικά (Δυτική Σαχάρα)',
        'ar_ER' => 'Αραβικά (Ερυθραία)',
        'ar_IL' => 'Αραβικά (Ισραήλ)',
        'ar_IQ' => 'Αραβικά (Ιράκ)',
        'ar_JO' => 'Αραβικά (Ιορδανία)',
        'ar_KM' => 'Αραβικά (Κομόρες)',
        'ar_KW' => 'Αραβικά (Κουβέιτ)',
        'ar_LB' => 'Αραβικά (Λίβανος)',
        'ar_LY' => 'Αραβικά (Λιβύη)',
        'ar_MA' => 'Αραβικά (Μαρόκο)',
        'ar_MR' => 'Αραβικά (Μαυριτανία)',
        'ar_OM' => 'Αραβικά (Ομάν)',
        'ar_PS' => 'Αραβικά (Παλαιστινιακά Εδάφη)',
        'ar_QA' => 'Αραβικά (Κατάρ)',
        'ar_SA' => 'Αραβικά (Σαουδική Αραβία)',
        'ar_SD' => 'Αραβικά (Σουδάν)',
        'ar_SO' => 'Αραβικά (Σομαλία)',
        'ar_SS' => 'Αραβικά (Νότιο Σουδάν)',
        'ar_SY' => 'Αραβικά (Συρία)',
        'ar_TD' => 'Αραβικά (Τσαντ)',
        'ar_TN' => 'Αραβικά (Τυνησία)',
        'ar_YE' => 'Αραβικά (Υεμένη)',
        'as' => 'Ασαμικά',
        'as_IN' => 'Ασαμικά (Ινδία)',
        'az' => 'Αζερμπαϊτζανικά',
        'az_AZ' => 'Αζερμπαϊτζανικά (Αζερμπαϊτζάν)',
        'az_Cyrl' => 'Αζερμπαϊτζανικά (Κυριλλικό)',
        'az_Cyrl_AZ' => 'Αζερμπαϊτζανικά (Κυριλλικό, Αζερμπαϊτζάν)',
        'az_Latn' => 'Αζερμπαϊτζανικά (Λατινικό)',
        'az_Latn_AZ' => 'Αζερμπαϊτζανικά (Λατινικό, Αζερμπαϊτζάν)',
        'ba' => 'Μπασκίρ',
        'ba_RU' => 'Μπασκίρ (Ρωσία)',
        'be' => 'Λευκορωσικά',
        'be_BY' => 'Λευκορωσικά (Λευκορωσία)',
        'bg' => 'Βουλγαρικά',
        'bg_BG' => 'Βουλγαρικά (Βουλγαρία)',
        'bm' => 'Μπαμπάρα',
        'bm_ML' => 'Μπαμπάρα (Μάλι)',
        'bn' => 'Βεγγαλικά',
        'bn_BD' => 'Βεγγαλικά (Μπανγκλαντές)',
        'bn_IN' => 'Βεγγαλικά (Ινδία)',
        'bo' => 'Θιβετιανά',
        'bo_CN' => 'Θιβετιανά (Κίνα)',
        'bo_IN' => 'Θιβετιανά (Ινδία)',
        'br' => 'Βρετονικά',
        'br_FR' => 'Βρετονικά (Γαλλία)',
        'bs' => 'Βοσνιακά',
        'bs_BA' => 'Βοσνιακά (Βοσνία - Ερζεγοβίνη)',
        'bs_Cyrl' => 'Βοσνιακά (Κυριλλικό)',
        'bs_Cyrl_BA' => 'Βοσνιακά (Κυριλλικό, Βοσνία - Ερζεγοβίνη)',
        'bs_Latn' => 'Βοσνιακά (Λατινικό)',
        'bs_Latn_BA' => 'Βοσνιακά (Λατινικό, Βοσνία - Ερζεγοβίνη)',
        'ca' => 'Καταλανικά',
        'ca_AD' => 'Καταλανικά (Ανδόρα)',
        'ca_ES' => 'Καταλανικά (Ισπανία)',
        'ca_FR' => 'Καταλανικά (Γαλλία)',
        'ca_IT' => 'Καταλανικά (Ιταλία)',
        'ce' => 'Τσετσενικά',
        'ce_RU' => 'Τσετσενικά (Ρωσία)',
        'cs' => 'Τσεχικά',
        'cs_CZ' => 'Τσεχικά (Τσεχία)',
        'cv' => 'Τσουβασικά',
        'cv_RU' => 'Τσουβασικά (Ρωσία)',
        'cy' => 'Ουαλικά',
        'cy_GB' => 'Ουαλικά (Ηνωμένο Βασίλειο)',
        'da' => 'Δανικά',
        'da_DK' => 'Δανικά (Δανία)',
        'da_GL' => 'Δανικά (Γροιλανδία)',
        'de' => 'Γερμανικά',
        'de_AT' => 'Γερμανικά (Αυστρία)',
        'de_BE' => 'Γερμανικά (Βέλγιο)',
        'de_CH' => 'Γερμανικά (Ελβετία)',
        'de_DE' => 'Γερμανικά (Γερμανία)',
        'de_IT' => 'Γερμανικά (Ιταλία)',
        'de_LI' => 'Γερμανικά (Λιχτενστάιν)',
        'de_LU' => 'Γερμανικά (Λουξεμβούργο)',
        'dz' => 'Ντζόνγκχα',
        'dz_BT' => 'Ντζόνγκχα (Μπουτάν)',
        'ee' => 'Έουε',
        'ee_GH' => 'Έουε (Γκάνα)',
        'ee_TG' => 'Έουε (Τόγκο)',
        'el' => 'Ελληνικά',
        'el_CY' => 'Ελληνικά (Κύπρος)',
        'el_GR' => 'Ελληνικά (Ελλάδα)',
        'en' => 'Αγγλικά',
        'en_001' => 'Αγγλικά (Κόσμος)',
        'en_150' => 'Αγγλικά (Ευρώπη)',
        'en_AE' => 'Αγγλικά (Ηνωμένα Αραβικά Εμιράτα)',
        'en_AG' => 'Αγγλικά (Αντίγκουα και Μπαρμπούντα)',
        'en_AI' => 'Αγγλικά (Ανγκουίλα)',
        'en_AS' => 'Αγγλικά (Αμερικανική Σαμόα)',
        'en_AT' => 'Αγγλικά (Αυστρία)',
        'en_AU' => 'Αγγλικά (Αυστραλία)',
        'en_BB' => 'Αγγλικά (Μπαρμπέιντος)',
        'en_BE' => 'Αγγλικά (Βέλγιο)',
        'en_BI' => 'Αγγλικά (Μπουρούντι)',
        'en_BM' => 'Αγγλικά (Βερμούδες)',
        'en_BS' => 'Αγγλικά (Μπαχάμες)',
        'en_BW' => 'Αγγλικά (Μποτσουάνα)',
        'en_BZ' => 'Αγγλικά (Μπελίζ)',
        'en_CA' => 'Αγγλικά (Καναδάς)',
        'en_CC' => 'Αγγλικά (Νήσοι Κόκος [Κίλινγκ])',
        'en_CH' => 'Αγγλικά (Ελβετία)',
        'en_CK' => 'Αγγλικά (Νήσοι Κουκ)',
        'en_CM' => 'Αγγλικά (Καμερούν)',
        'en_CX' => 'Αγγλικά (Νήσος των Χριστουγέννων)',
        'en_CY' => 'Αγγλικά (Κύπρος)',
        'en_CZ' => 'Αγγλικά (Τσεχία)',
        'en_DE' => 'Αγγλικά (Γερμανία)',
        'en_DK' => 'Αγγλικά (Δανία)',
        'en_DM' => 'Αγγλικά (Ντομίνικα)',
        'en_EE' => 'Αγγλικά (Εσθονία)',
        'en_ER' => 'Αγγλικά (Ερυθραία)',
        'en_ES' => 'Αγγλικά (Ισπανία)',
        'en_FI' => 'Αγγλικά (Φινλανδία)',
        'en_FJ' => 'Αγγλικά (Φίτζι)',
        'en_FK' => 'Αγγλικά (Νήσοι Φόκλαντ)',
        'en_FM' => 'Αγγλικά (Μικρονησία)',
        'en_FR' => 'Αγγλικά (Γαλλία)',
        'en_GB' => 'Αγγλικά (Ηνωμένο Βασίλειο)',
        'en_GD' => 'Αγγλικά (Γρενάδα)',
        'en_GE' => 'Αγγλικά (Γεωργία)',
        'en_GG' => 'Αγγλικά (Γκέρνζι)',
        'en_GH' => 'Αγγλικά (Γκάνα)',
        'en_GI' => 'Αγγλικά (Γιβραλτάρ)',
        'en_GM' => 'Αγγλικά (Γκάμπια)',
        'en_GS' => 'Αγγλικά (Νήσοι Νότια Γεωργία και Νότιες Σάντουιτς)',
        'en_GU' => 'Αγγλικά (Γκουάμ)',
        'en_GY' => 'Αγγλικά (Γουιάνα)',
        'en_HK' => 'Αγγλικά (Χονγκ Κονγκ ΕΔΠ Κίνας)',
        'en_HU' => 'Αγγλικά (Ουγγαρία)',
        'en_ID' => 'Αγγλικά (Ινδονησία)',
        'en_IE' => 'Αγγλικά (Ιρλανδία)',
        'en_IL' => 'Αγγλικά (Ισραήλ)',
        'en_IM' => 'Αγγλικά (Νήσος του Μαν)',
        'en_IN' => 'Αγγλικά (Ινδία)',
        'en_IO' => 'Αγγλικά (Βρετανικά Εδάφη Ινδικού Ωκεανού)',
        'en_IT' => 'Αγγλικά (Ιταλία)',
        'en_JE' => 'Αγγλικά (Τζέρζι)',
        'en_JM' => 'Αγγλικά (Τζαμάικα)',
        'en_JP' => 'Αγγλικά (Ιαπωνία)',
        'en_KE' => 'Αγγλικά (Κένυα)',
        'en_KI' => 'Αγγλικά (Κιριμπάτι)',
        'en_KN' => 'Αγγλικά (Σεν Κιτς και Νέβις)',
        'en_KY' => 'Αγγλικά (Νήσοι Κέιμαν)',
        'en_LC' => 'Αγγλικά (Αγία Λουκία)',
        'en_LR' => 'Αγγλικά (Λιβερία)',
        'en_LS' => 'Αγγλικά (Λεσότο)',
        'en_LT' => 'Αγγλικά (Λιθουανία)',
        'en_LV' => 'Αγγλικά (Λετονία)',
        'en_MG' => 'Αγγλικά (Μαδαγασκάρη)',
        'en_MH' => 'Αγγλικά (Νήσοι Μάρσαλ)',
        'en_MO' => 'Αγγλικά (Μακάο ΕΔΠ Κίνας)',
        'en_MP' => 'Αγγλικά (Νήσοι Βόρειες Μαριάνες)',
        'en_MS' => 'Αγγλικά (Μονσεράτ)',
        'en_MT' => 'Αγγλικά (Μάλτα)',
        'en_MU' => 'Αγγλικά (Μαυρίκιος)',
        'en_MV' => 'Αγγλικά (Μαλδίβες)',
        'en_MW' => 'Αγγλικά (Μαλάουι)',
        'en_MY' => 'Αγγλικά (Μαλαισία)',
        'en_NA' => 'Αγγλικά (Ναμίμπια)',
        'en_NF' => 'Αγγλικά (Νήσος Νόρφολκ)',
        'en_NG' => 'Αγγλικά (Νιγηρία)',
        'en_NL' => 'Αγγλικά (Κάτω Χώρες)',
        'en_NO' => 'Αγγλικά (Νορβηγία)',
        'en_NR' => 'Αγγλικά (Ναουρού)',
        'en_NU' => 'Αγγλικά (Νιούε)',
        'en_NZ' => 'Αγγλικά (Νέα Ζηλανδία)',
        'en_PG' => 'Αγγλικά (Παπούα Νέα Γουινέα)',
        'en_PH' => 'Αγγλικά (Φιλιππίνες)',
        'en_PK' => 'Αγγλικά (Πακιστάν)',
        'en_PL' => 'Αγγλικά (Πολωνία)',
        'en_PN' => 'Αγγλικά (Νήσοι Πίτκερν)',
        'en_PR' => 'Αγγλικά (Πουέρτο Ρίκο)',
        'en_PT' => 'Αγγλικά (Πορτογαλία)',
        'en_PW' => 'Αγγλικά (Παλάου)',
        'en_RO' => 'Αγγλικά (Ρουμανία)',
        'en_RW' => 'Αγγλικά (Ρουάντα)',
        'en_SB' => 'Αγγλικά (Νήσοι Σολομώντος)',
        'en_SC' => 'Αγγλικά (Σεϋχέλλες)',
        'en_SD' => 'Αγγλικά (Σουδάν)',
        'en_SE' => 'Αγγλικά (Σουηδία)',
        'en_SG' => 'Αγγλικά (Σιγκαπούρη)',
        'en_SH' => 'Αγγλικά (Αγία Ελένη)',
        'en_SI' => 'Αγγλικά (Σλοβενία)',
        'en_SK' => 'Αγγλικά (Σλοβακία)',
        'en_SL' => 'Αγγλικά (Σιέρα Λεόνε)',
        'en_SS' => 'Αγγλικά (Νότιο Σουδάν)',
        'en_SX' => 'Αγγλικά (Άγιος Μαρτίνος [Ολλανδικό τμήμα])',
        'en_SZ' => 'Αγγλικά (Εσουατίνι)',
        'en_TC' => 'Αγγλικά (Νήσοι Τερκς και Κάικος)',
        'en_TK' => 'Αγγλικά (Τοκελάου)',
        'en_TO' => 'Αγγλικά (Τόνγκα)',
        'en_TT' => 'Αγγλικά (Τρινιντάντ και Τομπάγκο)',
        'en_TV' => 'Αγγλικά (Τουβαλού)',
        'en_TZ' => 'Αγγλικά (Τανζανία)',
        'en_UA' => 'Αγγλικά (Ουκρανία)',
        'en_UG' => 'Αγγλικά (Ουγκάντα)',
        'en_UM' => 'Αγγλικά (Απομακρυσμένες Νησίδες ΗΠΑ)',
        'en_US' => 'Αγγλικά (Ηνωμένες Πολιτείες)',
        'en_VC' => 'Αγγλικά (Άγιος Βικέντιος και Γρεναδίνες)',
        'en_VG' => 'Αγγλικά (Βρετανικές Παρθένες Νήσοι)',
        'en_VI' => 'Αγγλικά (Αμερικανικές Παρθένες Νήσοι)',
        'en_VU' => 'Αγγλικά (Βανουάτου)',
        'en_WS' => 'Αγγλικά (Σαμόα)',
        'en_ZA' => 'Αγγλικά (Νότια Αφρική)',
        'en_ZM' => 'Αγγλικά (Ζάμπια)',
        'en_ZW' => 'Αγγλικά (Ζιμπάμπουε)',
        'eo' => 'Εσπεράντο',
        'eo_001' => 'Εσπεράντο (Κόσμος)',
        'es' => 'Ισπανικά',
        'es_419' => 'Ισπανικά (Λατινική Αμερική)',
        'es_AR' => 'Ισπανικά (Αργεντινή)',
        'es_BO' => 'Ισπανικά (Βολιβία)',
        'es_BR' => 'Ισπανικά (Βραζιλία)',
        'es_BZ' => 'Ισπανικά (Μπελίζ)',
        'es_CL' => 'Ισπανικά (Χιλή)',
        'es_CO' => 'Ισπανικά (Κολομβία)',
        'es_CR' => 'Ισπανικά (Κόστα Ρίκα)',
        'es_CU' => 'Ισπανικά (Κούβα)',
        'es_DO' => 'Ισπανικά (Δομινικανή Δημοκρατία)',
        'es_EC' => 'Ισπανικά (Ισημερινός)',
        'es_ES' => 'Ισπανικά (Ισπανία)',
        'es_GQ' => 'Ισπανικά (Ισημερινή Γουινέα)',
        'es_GT' => 'Ισπανικά (Γουατεμάλα)',
        'es_HN' => 'Ισπανικά (Ονδούρα)',
        'es_MX' => 'Ισπανικά (Μεξικό)',
        'es_NI' => 'Ισπανικά (Νικαράγουα)',
        'es_PA' => 'Ισπανικά (Παναμάς)',
        'es_PE' => 'Ισπανικά (Περού)',
        'es_PH' => 'Ισπανικά (Φιλιππίνες)',
        'es_PR' => 'Ισπανικά (Πουέρτο Ρίκο)',
        'es_PY' => 'Ισπανικά (Παραγουάη)',
        'es_SV' => 'Ισπανικά (Ελ Σαλβαδόρ)',
        'es_US' => 'Ισπανικά (Ηνωμένες Πολιτείες)',
        'es_UY' => 'Ισπανικά (Ουρουγουάη)',
        'es_VE' => 'Ισπανικά (Βενεζουέλα)',
        'et' => 'Εσθονικά',
        'et_EE' => 'Εσθονικά (Εσθονία)',
        'eu' => 'Βασκικά',
        'eu_ES' => 'Βασκικά (Ισπανία)',
        'fa' => 'Περσικά',
        'fa_AF' => 'Περσικά (Αφγανιστάν)',
        'fa_IR' => 'Περσικά (Ιράν)',
        'ff' => 'Φουλά',
        'ff_Adlm' => 'Φουλά (Άντλαμ)',
        'ff_Adlm_BF' => 'Φουλά (Άντλαμ, Μπουρκίνα Φάσο)',
        'ff_Adlm_CM' => 'Φουλά (Άντλαμ, Καμερούν)',
        'ff_Adlm_GH' => 'Φουλά (Άντλαμ, Γκάνα)',
        'ff_Adlm_GM' => 'Φουλά (Άντλαμ, Γκάμπια)',
        'ff_Adlm_GN' => 'Φουλά (Άντλαμ, Γουινέα)',
        'ff_Adlm_GW' => 'Φουλά (Άντλαμ, Γουινέα Μπισάου)',
        'ff_Adlm_LR' => 'Φουλά (Άντλαμ, Λιβερία)',
        'ff_Adlm_MR' => 'Φουλά (Άντλαμ, Μαυριτανία)',
        'ff_Adlm_NE' => 'Φουλά (Άντλαμ, Νίγηρας)',
        'ff_Adlm_NG' => 'Φουλά (Άντλαμ, Νιγηρία)',
        'ff_Adlm_SL' => 'Φουλά (Άντλαμ, Σιέρα Λεόνε)',
        'ff_Adlm_SN' => 'Φουλά (Άντλαμ, Σενεγάλη)',
        'ff_CM' => 'Φουλά (Καμερούν)',
        'ff_GN' => 'Φουλά (Γουινέα)',
        'ff_Latn' => 'Φουλά (Λατινικό)',
        'ff_Latn_BF' => 'Φουλά (Λατινικό, Μπουρκίνα Φάσο)',
        'ff_Latn_CM' => 'Φουλά (Λατινικό, Καμερούν)',
        'ff_Latn_GH' => 'Φουλά (Λατινικό, Γκάνα)',
        'ff_Latn_GM' => 'Φουλά (Λατινικό, Γκάμπια)',
        'ff_Latn_GN' => 'Φουλά (Λατινικό, Γουινέα)',
        'ff_Latn_GW' => 'Φουλά (Λατινικό, Γουινέα Μπισάου)',
        'ff_Latn_LR' => 'Φουλά (Λατινικό, Λιβερία)',
        'ff_Latn_MR' => 'Φουλά (Λατινικό, Μαυριτανία)',
        'ff_Latn_NE' => 'Φουλά (Λατινικό, Νίγηρας)',
        'ff_Latn_NG' => 'Φουλά (Λατινικό, Νιγηρία)',
        'ff_Latn_SL' => 'Φουλά (Λατινικό, Σιέρα Λεόνε)',
        'ff_Latn_SN' => 'Φουλά (Λατινικό, Σενεγάλη)',
        'ff_MR' => 'Φουλά (Μαυριτανία)',
        'ff_SN' => 'Φουλά (Σενεγάλη)',
        'fi' => 'Φινλανδικά',
        'fi_FI' => 'Φινλανδικά (Φινλανδία)',
        'fo' => 'Φεροϊκά',
        'fo_DK' => 'Φεροϊκά (Δανία)',
        'fo_FO' => 'Φεροϊκά (Νήσοι Φερόες)',
        'fr' => 'Γαλλικά',
        'fr_BE' => 'Γαλλικά (Βέλγιο)',
        'fr_BF' => 'Γαλλικά (Μπουρκίνα Φάσο)',
        'fr_BI' => 'Γαλλικά (Μπουρούντι)',
        'fr_BJ' => 'Γαλλικά (Μπενίν)',
        'fr_BL' => 'Γαλλικά (Άγιος Βαρθολομαίος)',
        'fr_CA' => 'Γαλλικά (Καναδάς)',
        'fr_CD' => 'Γαλλικά (Κονγκό - Κινσάσα)',
        'fr_CF' => 'Γαλλικά (Κεντροαφρικανική Δημοκρατία)',
        'fr_CG' => 'Γαλλικά (Κονγκό - Μπραζαβίλ)',
        'fr_CH' => 'Γαλλικά (Ελβετία)',
        'fr_CI' => 'Γαλλικά (Ακτή Ελεφαντοστού)',
        'fr_CM' => 'Γαλλικά (Καμερούν)',
        'fr_DJ' => 'Γαλλικά (Τζιμπουτί)',
        'fr_DZ' => 'Γαλλικά (Αλγερία)',
        'fr_FR' => 'Γαλλικά (Γαλλία)',
        'fr_GA' => 'Γαλλικά (Γκαμπόν)',
        'fr_GF' => 'Γαλλικά (Γαλλική Γουιάνα)',
        'fr_GN' => 'Γαλλικά (Γουινέα)',
        'fr_GP' => 'Γαλλικά (Γουαδελούπη)',
        'fr_GQ' => 'Γαλλικά (Ισημερινή Γουινέα)',
        'fr_HT' => 'Γαλλικά (Αϊτή)',
        'fr_KM' => 'Γαλλικά (Κομόρες)',
        'fr_LU' => 'Γαλλικά (Λουξεμβούργο)',
        'fr_MA' => 'Γαλλικά (Μαρόκο)',
        'fr_MC' => 'Γαλλικά (Μονακό)',
        'fr_MF' => 'Γαλλικά (Άγιος Μαρτίνος [Γαλλικό τμήμα])',
        'fr_MG' => 'Γαλλικά (Μαδαγασκάρη)',
        'fr_ML' => 'Γαλλικά (Μάλι)',
        'fr_MQ' => 'Γαλλικά (Μαρτινίκα)',
        'fr_MR' => 'Γαλλικά (Μαυριτανία)',
        'fr_MU' => 'Γαλλικά (Μαυρίκιος)',
        'fr_NC' => 'Γαλλικά (Νέα Καληδονία)',
        'fr_NE' => 'Γαλλικά (Νίγηρας)',
        'fr_PF' => 'Γαλλικά (Γαλλική Πολυνησία)',
        'fr_PM' => 'Γαλλικά (Σεν Πιερ και Μικελόν)',
        'fr_RE' => 'Γαλλικά (Ρεϊνιόν)',
        'fr_RW' => 'Γαλλικά (Ρουάντα)',
        'fr_SC' => 'Γαλλικά (Σεϋχέλλες)',
        'fr_SN' => 'Γαλλικά (Σενεγάλη)',
        'fr_SY' => 'Γαλλικά (Συρία)',
        'fr_TD' => 'Γαλλικά (Τσαντ)',
        'fr_TG' => 'Γαλλικά (Τόγκο)',
        'fr_TN' => 'Γαλλικά (Τυνησία)',
        'fr_VU' => 'Γαλλικά (Βανουάτου)',
        'fr_WF' => 'Γαλλικά (Γουάλις και Φουτούνα)',
        'fr_YT' => 'Γαλλικά (Μαγιότ)',
        'fy' => 'Δυτικά Φριζικά',
        'fy_NL' => 'Δυτικά Φριζικά (Κάτω Χώρες)',
        'ga' => 'Ιρλανδικά',
        'ga_GB' => 'Ιρλανδικά (Ηνωμένο Βασίλειο)',
        'ga_IE' => 'Ιρλανδικά (Ιρλανδία)',
        'gd' => 'Σκωτικά Κελτικά',
        'gd_GB' => 'Σκωτικά Κελτικά (Ηνωμένο Βασίλειο)',
        'gl' => 'Γαλικιανά',
        'gl_ES' => 'Γαλικιανά (Ισπανία)',
        'gu' => 'Γκουτζαρατικά',
        'gu_IN' => 'Γκουτζαρατικά (Ινδία)',
        'gv' => 'Μανξ',
        'gv_IM' => 'Μανξ (Νήσος του Μαν)',
        'ha' => 'Χάουσα',
        'ha_GH' => 'Χάουσα (Γκάνα)',
        'ha_NE' => 'Χάουσα (Νίγηρας)',
        'ha_NG' => 'Χάουσα (Νιγηρία)',
        'he' => 'Εβραϊκά',
        'he_IL' => 'Εβραϊκά (Ισραήλ)',
        'hi' => 'Χίντι',
        'hi_IN' => 'Χίντι (Ινδία)',
        'hi_Latn' => 'Χίντι (Λατινικό)',
        'hi_Latn_IN' => 'Χίντι (Λατινικό, Ινδία)',
        'hr' => 'Κροατικά',
        'hr_BA' => 'Κροατικά (Βοσνία - Ερζεγοβίνη)',
        'hr_HR' => 'Κροατικά (Κροατία)',
        'hu' => 'Ουγγρικά',
        'hu_HU' => 'Ουγγρικά (Ουγγαρία)',
        'hy' => 'Αρμενικά',
        'hy_AM' => 'Αρμενικά (Αρμενία)',
        'ia' => 'Ιντερλίνγκουα',
        'ia_001' => 'Ιντερλίνγκουα (Κόσμος)',
        'id' => 'Ινδονησιακά',
        'id_ID' => 'Ινδονησιακά (Ινδονησία)',
        'ie' => 'Ιντερλίνγκουε',
        'ie_EE' => 'Ιντερλίνγκουε (Εσθονία)',
        'ig' => 'Ίγκμπο',
        'ig_NG' => 'Ίγκμπο (Νιγηρία)',
        'ii' => 'Σίτσουαν Γι',
        'ii_CN' => 'Σίτσουαν Γι (Κίνα)',
        'is' => 'Ισλανδικά',
        'is_IS' => 'Ισλανδικά (Ισλανδία)',
        'it' => 'Ιταλικά',
        'it_CH' => 'Ιταλικά (Ελβετία)',
        'it_IT' => 'Ιταλικά (Ιταλία)',
        'it_SM' => 'Ιταλικά (Άγιος Μαρίνος)',
        'it_VA' => 'Ιταλικά (Βατικανό)',
        'ja' => 'Ιαπωνικά',
        'ja_JP' => 'Ιαπωνικά (Ιαπωνία)',
        'jv' => 'Ιαβανικά',
        'jv_ID' => 'Ιαβανικά (Ινδονησία)',
        'ka' => 'Γεωργιανά',
        'ka_GE' => 'Γεωργιανά (Γεωργία)',
        'ki' => 'Κικούγιου',
        'ki_KE' => 'Κικούγιου (Κένυα)',
        'kk' => 'Καζακικά',
        'kk_Arab' => 'Καζακικά (Αραβικό)',
        'kk_Arab_CN' => 'Καζακικά (Αραβικό, Κίνα)',
        'kk_Cyrl' => 'Καζακικά (Κυριλλικό)',
        'kk_Cyrl_KZ' => 'Καζακικά (Κυριλλικό, Καζακστάν)',
        'kk_KZ' => 'Καζακικά (Καζακστάν)',
        'kl' => 'Καλαάλισουτ',
        'kl_GL' => 'Καλαάλισουτ (Γροιλανδία)',
        'km' => 'Χμερ',
        'km_KH' => 'Χμερ (Καμπότζη)',
        'kn' => 'Κανάντα',
        'kn_IN' => 'Κανάντα (Ινδία)',
        'ko' => 'Κορεατικά',
        'ko_CN' => 'Κορεατικά (Κίνα)',
        'ko_KP' => 'Κορεατικά (Βόρεια Κορέα)',
        'ko_KR' => 'Κορεατικά (Νότια Κορέα)',
        'ks' => 'Κασμιρικά',
        'ks_Arab' => 'Κασμιρικά (Αραβικό)',
        'ks_Arab_IN' => 'Κασμιρικά (Αραβικό, Ινδία)',
        'ks_Deva' => 'Κασμιρικά (Ντεβαναγκάρι)',
        'ks_Deva_IN' => 'Κασμιρικά (Ντεβαναγκάρι, Ινδία)',
        'ks_IN' => 'Κασμιρικά (Ινδία)',
        'ku' => 'Κουρδικά',
        'ku_Latn' => 'Κουρδικά (Λατινικό)',
        'ku_Latn_IQ' => 'Κουρδικά (Λατινικό, Ιράκ)',
        'ku_Latn_SY' => 'Κουρδικά (Λατινικό, Συρία)',
        'ku_Latn_TR' => 'Κουρδικά (Λατινικό, Τουρκία)',
        'ku_SY' => 'Κουρδικά (Συρία)',
        'ku_TR' => 'Κουρδικά (Τουρκία)',
        'kw' => 'Κορνουαλικά',
        'kw_GB' => 'Κορνουαλικά (Ηνωμένο Βασίλειο)',
        'ky' => 'Κιργιζικά',
        'ky_KG' => 'Κιργιζικά (Κιργιστάν)',
        'lb' => 'Λουξεμβουργιανά',
        'lb_LU' => 'Λουξεμβουργιανά (Λουξεμβούργο)',
        'lg' => 'Γκάντα',
        'lg_UG' => 'Γκάντα (Ουγκάντα)',
        'ln' => 'Λινγκάλα',
        'ln_AO' => 'Λινγκάλα (Αγκόλα)',
        'ln_CD' => 'Λινγκάλα (Κονγκό - Κινσάσα)',
        'ln_CF' => 'Λινγκάλα (Κεντροαφρικανική Δημοκρατία)',
        'ln_CG' => 'Λινγκάλα (Κονγκό - Μπραζαβίλ)',
        'lo' => 'Λαοτινά',
        'lo_LA' => 'Λαοτινά (Λάος)',
        'lt' => 'Λιθουανικά',
        'lt_LT' => 'Λιθουανικά (Λιθουανία)',
        'lu' => 'Λούμπα-Κατάνγκα',
        'lu_CD' => 'Λούμπα-Κατάνγκα (Κονγκό - Κινσάσα)',
        'lv' => 'Λετονικά',
        'lv_LV' => 'Λετονικά (Λετονία)',
        'mg' => 'Μαλγασικά',
        'mg_MG' => 'Μαλγασικά (Μαδαγασκάρη)',
        'mi' => 'Μαορί',
        'mi_NZ' => 'Μαορί (Νέα Ζηλανδία)',
        'mk' => 'Σλαβομακεδονικά',
        'mk_MK' => 'Σλαβομακεδονικά (Βόρεια Μακεδονία)',
        'ml' => 'Μαλαγιαλαμικά',
        'ml_IN' => 'Μαλαγιαλαμικά (Ινδία)',
        'mn' => 'Μογγολικά',
        'mn_MN' => 'Μογγολικά (Μογγολία)',
        'mr' => 'Μαραθικά',
        'mr_IN' => 'Μαραθικά (Ινδία)',
        'ms' => 'Μαλαισιανά',
        'ms_BN' => 'Μαλαισιανά (Μπρουνέι)',
        'ms_ID' => 'Μαλαισιανά (Ινδονησία)',
        'ms_MY' => 'Μαλαισιανά (Μαλαισία)',
        'ms_SG' => 'Μαλαισιανά (Σιγκαπούρη)',
        'mt' => 'Μαλτεζικά',
        'mt_MT' => 'Μαλτεζικά (Μάλτα)',
        'my' => 'Βιρμανικά',
        'my_MM' => 'Βιρμανικά (Μιανμάρ [Βιρμανία])',
        'nb' => 'Νορβηγικά Μποκμάλ',
        'nb_NO' => 'Νορβηγικά Μποκμάλ (Νορβηγία)',
        'nb_SJ' => 'Νορβηγικά Μποκμάλ (Σβάλμπαρντ και Γιαν Μαγιέν)',
        'nd' => 'Βόρεια Ντεμπέλε',
        'nd_ZW' => 'Βόρεια Ντεμπέλε (Ζιμπάμπουε)',
        'ne' => 'Νεπαλικά',
        'ne_IN' => 'Νεπαλικά (Ινδία)',
        'ne_NP' => 'Νεπαλικά (Νεπάλ)',
        'nl' => 'Ολλανδικά',
        'nl_AW' => 'Ολλανδικά (Αρούμπα)',
        'nl_BE' => 'Ολλανδικά (Βέλγιο)',
        'nl_BQ' => 'Ολλανδικά (Ολλανδία Καραϊβικής)',
        'nl_CW' => 'Ολλανδικά (Κουρασάο)',
        'nl_NL' => 'Ολλανδικά (Κάτω Χώρες)',
        'nl_SR' => 'Ολλανδικά (Σουρινάμ)',
        'nl_SX' => 'Ολλανδικά (Άγιος Μαρτίνος [Ολλανδικό τμήμα])',
        'nn' => 'Νορβηγικά Νινόρσκ',
        'nn_NO' => 'Νορβηγικά Νινόρσκ (Νορβηγία)',
        'no' => 'Νορβηγικά',
        'no_NO' => 'Νορβηγικά (Νορβηγία)',
        'oc' => 'Οξιτανικά',
        'oc_ES' => 'Οξιτανικά (Ισπανία)',
        'oc_FR' => 'Οξιτανικά (Γαλλία)',
        'om' => 'Ορόμο',
        'om_ET' => 'Ορόμο (Αιθιοπία)',
        'om_KE' => 'Ορόμο (Κένυα)',
        'or' => 'Όντια',
        'or_IN' => 'Όντια (Ινδία)',
        'os' => 'Οσετικά',
        'os_GE' => 'Οσετικά (Γεωργία)',
        'os_RU' => 'Οσετικά (Ρωσία)',
        'pa' => 'Παντζαπικά',
        'pa_Arab' => 'Παντζαπικά (Αραβικό)',
        'pa_Arab_PK' => 'Παντζαπικά (Αραβικό, Πακιστάν)',
        'pa_Guru' => 'Παντζαπικά (Γκουρμουκχί)',
        'pa_Guru_IN' => 'Παντζαπικά (Γκουρμουκχί, Ινδία)',
        'pa_IN' => 'Παντζαπικά (Ινδία)',
        'pa_PK' => 'Παντζαπικά (Πακιστάν)',
        'pl' => 'Πολωνικά',
        'pl_PL' => 'Πολωνικά (Πολωνία)',
        'ps' => 'Πάστο',
        'ps_AF' => 'Πάστο (Αφγανιστάν)',
        'ps_PK' => 'Πάστο (Πακιστάν)',
        'pt' => 'Πορτογαλικά',
        'pt_AO' => 'Πορτογαλικά (Αγκόλα)',
        'pt_BR' => 'Πορτογαλικά (Βραζιλία)',
        'pt_CH' => 'Πορτογαλικά (Ελβετία)',
        'pt_CV' => 'Πορτογαλικά (Πράσινο Ακρωτήριο)',
        'pt_GQ' => 'Πορτογαλικά (Ισημερινή Γουινέα)',
        'pt_GW' => 'Πορτογαλικά (Γουινέα Μπισάου)',
        'pt_LU' => 'Πορτογαλικά (Λουξεμβούργο)',
        'pt_MO' => 'Πορτογαλικά (Μακάο ΕΔΠ Κίνας)',
        'pt_MZ' => 'Πορτογαλικά (Μοζαμβίκη)',
        'pt_PT' => 'Πορτογαλικά (Πορτογαλία)',
        'pt_ST' => 'Πορτογαλικά (Σάο Τομέ και Πρίνσιπε)',
        'pt_TL' => 'Πορτογαλικά (Τιμόρ-Λέστε)',
        'qu' => 'Κέτσουα',
        'qu_BO' => 'Κέτσουα (Βολιβία)',
        'qu_EC' => 'Κέτσουα (Ισημερινός)',
        'qu_PE' => 'Κέτσουα (Περού)',
        'rm' => 'Ρομανικά',
        'rm_CH' => 'Ρομανικά (Ελβετία)',
        'rn' => 'Ρούντι',
        'rn_BI' => 'Ρούντι (Μπουρούντι)',
        'ro' => 'Ρουμανικά',
        'ro_MD' => 'Ρουμανικά (Μολδαβία)',
        'ro_RO' => 'Ρουμανικά (Ρουμανία)',
        'ru' => 'Ρωσικά',
        'ru_BY' => 'Ρωσικά (Λευκορωσία)',
        'ru_KG' => 'Ρωσικά (Κιργιστάν)',
        'ru_KZ' => 'Ρωσικά (Καζακστάν)',
        'ru_MD' => 'Ρωσικά (Μολδαβία)',
        'ru_RU' => 'Ρωσικά (Ρωσία)',
        'ru_UA' => 'Ρωσικά (Ουκρανία)',
        'rw' => 'Κινιαρουάντα',
        'rw_RW' => 'Κινιαρουάντα (Ρουάντα)',
        'sa' => 'Σανσκριτικά',
        'sa_IN' => 'Σανσκριτικά (Ινδία)',
        'sc' => 'Σαρδηνιακά',
        'sc_IT' => 'Σαρδηνιακά (Ιταλία)',
        'sd' => 'Σίντι',
        'sd_Arab' => 'Σίντι (Αραβικό)',
        'sd_Arab_PK' => 'Σίντι (Αραβικό, Πακιστάν)',
        'sd_Deva' => 'Σίντι (Ντεβαναγκάρι)',
        'sd_Deva_IN' => 'Σίντι (Ντεβαναγκάρι, Ινδία)',
        'sd_IN' => 'Σίντι (Ινδία)',
        'sd_PK' => 'Σίντι (Πακιστάν)',
        'se' => 'Βόρεια Σάμι',
        'se_FI' => 'Βόρεια Σάμι (Φινλανδία)',
        'se_NO' => 'Βόρεια Σάμι (Νορβηγία)',
        'se_SE' => 'Βόρεια Σάμι (Σουηδία)',
        'sg' => 'Σάνγκο',
        'sg_CF' => 'Σάνγκο (Κεντροαφρικανική Δημοκρατία)',
        'sh' => 'Σερβοκροατικά',
        'sh_BA' => 'Σερβοκροατικά (Βοσνία - Ερζεγοβίνη)',
        'si' => 'Σινχαλεζικά',
        'si_LK' => 'Σινχαλεζικά (Σρι Λάνκα)',
        'sk' => 'Σλοβακικά',
        'sk_SK' => 'Σλοβακικά (Σλοβακία)',
        'sl' => 'Σλοβενικά',
        'sl_SI' => 'Σλοβενικά (Σλοβενία)',
        'sn' => 'Σόνα',
        'sn_ZW' => 'Σόνα (Ζιμπάμπουε)',
        'so' => 'Σομαλικά',
        'so_DJ' => 'Σομαλικά (Τζιμπουτί)',
        'so_ET' => 'Σομαλικά (Αιθιοπία)',
        'so_KE' => 'Σομαλικά (Κένυα)',
        'so_SO' => 'Σομαλικά (Σομαλία)',
        'sq' => 'Αλβανικά',
        'sq_AL' => 'Αλβανικά (Αλβανία)',
        'sq_MK' => 'Αλβανικά (Βόρεια Μακεδονία)',
        'sr' => 'Σερβικά',
        'sr_BA' => 'Σερβικά (Βοσνία - Ερζεγοβίνη)',
        'sr_Cyrl' => 'Σερβικά (Κυριλλικό)',
        'sr_Cyrl_BA' => 'Σερβικά (Κυριλλικό, Βοσνία - Ερζεγοβίνη)',
        'sr_Cyrl_ME' => 'Σερβικά (Κυριλλικό, Μαυροβούνιο)',
        'sr_Cyrl_RS' => 'Σερβικά (Κυριλλικό, Σερβία)',
        'sr_Latn' => 'Σερβικά (Λατινικό)',
        'sr_Latn_BA' => 'Σερβικά (Λατινικό, Βοσνία - Ερζεγοβίνη)',
        'sr_Latn_ME' => 'Σερβικά (Λατινικό, Μαυροβούνιο)',
        'sr_Latn_RS' => 'Σερβικά (Λατινικό, Σερβία)',
        'sr_ME' => 'Σερβικά (Μαυροβούνιο)',
        'sr_RS' => 'Σερβικά (Σερβία)',
        'st' => 'Νότια Σόθο',
        'st_LS' => 'Νότια Σόθο (Λεσότο)',
        'st_ZA' => 'Νότια Σόθο (Νότια Αφρική)',
        'su' => 'Σουνδανικά',
        'su_ID' => 'Σουνδανικά (Ινδονησία)',
        'su_Latn' => 'Σουνδανικά (Λατινικό)',
        'su_Latn_ID' => 'Σουνδανικά (Λατινικό, Ινδονησία)',
        'sv' => 'Σουηδικά',
        'sv_AX' => 'Σουηδικά (Νήσοι Όλαντ)',
        'sv_FI' => 'Σουηδικά (Φινλανδία)',
        'sv_SE' => 'Σουηδικά (Σουηδία)',
        'sw' => 'Σουαχίλι',
        'sw_CD' => 'Σουαχίλι (Κονγκό - Κινσάσα)',
        'sw_KE' => 'Σουαχίλι (Κένυα)',
        'sw_TZ' => 'Σουαχίλι (Τανζανία)',
        'sw_UG' => 'Σουαχίλι (Ουγκάντα)',
        'ta' => 'Ταμιλικά',
        'ta_IN' => 'Ταμιλικά (Ινδία)',
        'ta_LK' => 'Ταμιλικά (Σρι Λάνκα)',
        'ta_MY' => 'Ταμιλικά (Μαλαισία)',
        'ta_SG' => 'Ταμιλικά (Σιγκαπούρη)',
        'te' => 'Τελούγκου',
        'te_IN' => 'Τελούγκου (Ινδία)',
        'tg' => 'Τατζικικά',
        'tg_TJ' => 'Τατζικικά (Τατζικιστάν)',
        'th' => 'Ταϊλανδικά',
        'th_TH' => 'Ταϊλανδικά (Ταϊλάνδη)',
        'ti' => 'Τιγκρινικά',
        'ti_ER' => 'Τιγκρινικά (Ερυθραία)',
        'ti_ET' => 'Τιγκρινικά (Αιθιοπία)',
        'tk' => 'Τουρκμενικά',
        'tk_TM' => 'Τουρκμενικά (Τουρκμενιστάν)',
        'tl' => 'Τάγκαλογκ',
        'tl_PH' => 'Τάγκαλογκ (Φιλιππίνες)',
        'tn' => 'Τσουάνα',
        'tn_BW' => 'Τσουάνα (Μποτσουάνα)',
        'tn_ZA' => 'Τσουάνα (Νότια Αφρική)',
        'to' => 'Τονγκανικά',
        'to_TO' => 'Τονγκανικά (Τόνγκα)',
        'tr' => 'Τουρκικά',
        'tr_CY' => 'Τουρκικά (Κύπρος)',
        'tr_TR' => 'Τουρκικά (Τουρκία)',
        'tt' => 'Ταταρικά',
        'tt_RU' => 'Ταταρικά (Ρωσία)',
        'ug' => 'Ουιγουρικά',
        'ug_CN' => 'Ουιγουρικά (Κίνα)',
        'uk' => 'Ουκρανικά',
        'uk_UA' => 'Ουκρανικά (Ουκρανία)',
        'ur' => 'Ούρντου',
        'ur_IN' => 'Ούρντου (Ινδία)',
        'ur_PK' => 'Ούρντου (Πακιστάν)',
        'uz' => 'Ουζμπεκικά',
        'uz_AF' => 'Ουζμπεκικά (Αφγανιστάν)',
        'uz_Arab' => 'Ουζμπεκικά (Αραβικό)',
        'uz_Arab_AF' => 'Ουζμπεκικά (Αραβικό, Αφγανιστάν)',
        'uz_Cyrl' => 'Ουζμπεκικά (Κυριλλικό)',
        'uz_Cyrl_UZ' => 'Ουζμπεκικά (Κυριλλικό, Ουζμπεκιστάν)',
        'uz_Latn' => 'Ουζμπεκικά (Λατινικό)',
        'uz_Latn_UZ' => 'Ουζμπεκικά (Λατινικό, Ουζμπεκιστάν)',
        'uz_UZ' => 'Ουζμπεκικά (Ουζμπεκιστάν)',
        'vi' => 'Βιετναμικά',
        'vi_VN' => 'Βιετναμικά (Βιετνάμ)',
        'wo' => 'Γουόλοφ',
        'wo_SN' => 'Γουόλοφ (Σενεγάλη)',
        'xh' => 'Κόσα',
        'xh_ZA' => 'Κόσα (Νότια Αφρική)',
        'yi' => 'Γίντις',
        'yi_UA' => 'Γίντις (Ουκρανία)',
        'yo' => 'Γιορούμπα',
        'yo_BJ' => 'Γιορούμπα (Μπενίν)',
        'yo_NG' => 'Γιορούμπα (Νιγηρία)',
        'za' => 'Ζουάνγκ',
        'za_CN' => 'Ζουάνγκ (Κίνα)',
        'zh' => 'Κινεζικά',
        'zh_CN' => 'Κινεζικά (Κίνα)',
        'zh_HK' => 'Κινεζικά (Χονγκ Κονγκ ΕΔΠ Κίνας)',
        'zh_Hans' => 'Κινεζικά (Απλοποιημένο)',
        'zh_Hans_CN' => 'Κινεζικά (Απλοποιημένο, Κίνα)',
        'zh_Hans_HK' => 'Κινεζικά (Απλοποιημένο, Χονγκ Κονγκ ΕΔΠ Κίνας)',
        'zh_Hans_MO' => 'Κινεζικά (Απλοποιημένο, Μακάο ΕΔΠ Κίνας)',
        'zh_Hans_MY' => 'Κινεζικά (Απλοποιημένο, Μαλαισία)',
        'zh_Hans_SG' => 'Κινεζικά (Απλοποιημένο, Σιγκαπούρη)',
        'zh_Hant' => 'Κινεζικά (Παραδοσιακό)',
        'zh_Hant_HK' => 'Κινεζικά (Παραδοσιακό, Χονγκ Κονγκ ΕΔΠ Κίνας)',
        'zh_Hant_MO' => 'Κινεζικά (Παραδοσιακό, Μακάο ΕΔΠ Κίνας)',
        'zh_Hant_MY' => 'Κινεζικά (Παραδοσιακό, Μαλαισία)',
        'zh_Hant_TW' => 'Κινεζικά (Παραδοσιακό, Ταϊβάν)',
        'zh_MO' => 'Κινεζικά (Μακάο ΕΔΠ Κίνας)',
        'zh_SG' => 'Κινεζικά (Σιγκαπούρη)',
        'zh_TW' => 'Κινεζικά (Ταϊβάν)',
        'zu' => 'Ζουλού',
        'zu_ZA' => 'Ζουλού (Νότια Αφρική)',
    ],
];
