<?php

return [
    'Names' => [
        'aa' => 'afarî',
        'ab' => 'abxazî',
        'ace' => 'açehî',
        'ada' => 'adangmeyî',
        'ady' => 'adîgeyî',
        'af' => 'afrîkansî',
        'agq' => 'aghemî',
        'ain' => 'aynuyî',
        'ak' => 'akanî',
        'ale' => 'alêwîtî',
        'alt' => 'altayîya başûrî',
        'am' => 'amharî',
        'an' => 'aragonî',
        'ann' => 'obolo',
        'anp' => 'angîkayî',
        'apc' => 'erebîya bakurê Şamê',
        'ar' => 'erebî',
        'arn' => 'mapuçî',
        'arp' => 'arapahoyî',
        'ars' => 'erebîya necdî',
        'as' => 'asamî',
        'asa' => 'asûyî',
        'ast' => 'astûrî',
        'atj' => 'atîkamekî',
        'av' => 'avarî',
        'awa' => 'awadhî',
        'ay' => 'aymarayî',
        'az' => 'azerbaycanî',
        'ba' => 'başkîrî',
        'bal' => 'belûcî',
        'ban' => 'balînî',
        'bas' => 'basayî',
        'be' => 'belarûsî',
        'bem' => 'bembayî',
        'bew' => 'betawî',
        'bez' => 'benayî',
        'bg' => 'bulgarî',
        'bgc' => 'haryanvîyî',
        'bgn' => 'belûcîya rojavayî',
        'bho' => 'bojpûrî',
        'bi' => 'bîslamayî',
        'bin' => 'bînîyî',
        'bla' => 'blakfotî',
        'blo' => 'bloyî',
        'blt' => 'tay dam',
        'bm' => 'bambarayî',
        'bn' => 'bengalî',
        'bo' => 'tîbetî',
        'bqi' => 'bextiyarî',
        'br' => 'bretonî',
        'brx' => 'bodoyî',
        'bs' => 'bosnî',
        'bss' => 'akooseyî',
        'bua' => 'buriat',
        'bug' => 'bugî',
        'byn' => 'blînî',
        'ca' => 'katalanî',
        'cad' => 'kadoyî',
        'cay' => 'kayugayî',
        'cch' => 'atsamî',
        'ccp' => 'çakmayî',
        'ce' => 'çeçenî',
        'ceb' => 'sebwanoyî',
        'cgg' => 'kîgayî',
        'ch' => 'çamoroyî',
        'chk' => 'çûkî',
        'chm' => 'marî',
        'cho' => 'çoktavî',
        'chp' => 'çîpevyayî',
        'chr' => 'çerokî',
        'chy' => 'çeyenî',
        'cic' => 'çîkasawî',
        'ckb' => 'kurdî (soranî)',
        'clc' => 'çilkotînî',
        'co' => 'korsîkayî',
        'cop' => 'qiptî',
        'crg' => 'mîçîfî',
        'crj' => 'krîya rojhilat ya başûrî',
        'crk' => 'krîya bejayî',
        'crl' => 'krîya rojhilat ya bakurî',
        'crm' => 'krîya mûsî',
        'crr' => 'zimanê karolina algonquianî',
        'cs' => 'çekî',
        'csw' => 'krîya swampî',
        'cu' => 'slavîya kenîseyî',
        'cv' => 'çuvaşî',
        'cy' => 'weylsî',
        'da' => 'danîmarkî',
        'dak' => 'dakotayî',
        'dar' => 'dargînî',
        'dav' => 'tayîtayî',
        'de' => 'almanî',
        'dgr' => 'dogrîbî',
        'dje' => 'zarma',
        'doi' => 'dogrîyî',
        'dsb' => 'sorbîya jêrîn',
        'dua' => 'diwalayî',
        'dv' => 'divehî',
        'dyo' => 'jola-fonyi',
        'dz' => 'conxayî',
        'dzg' => 'dazagayî',
        'ebu' => 'embuyî',
        'ee' => 'eweyî',
        'efi' => 'efîkî',
        'eka' => 'ekajukî',
        'el' => 'yûnanî',
        'en' => 'îngilîzî',
        'eo' => 'esperantoyî',
        'es' => 'spanî',
        'et' => 'estonî',
        'eu' => 'baskî',
        'ewo' => 'ewondoyî',
        'fa' => 'farisî',
        'ff' => 'fulahî',
        'fi' => 'fînî',
        'fil' => 'fîlîpînî',
        'fj' => 'fîjî',
        'fo' => 'ferî',
        'fon' => 'fonî',
        'fr' => 'fransizî',
        'frc' => 'fransizîya kajûnê',
        'frr' => 'frîsîya bakur',
        'fur' => 'frîyolî',
        'fy' => 'frîsîya rojavayî',
        'ga' => 'îrlendî',
        'gaa' => 'gayî',
        'gd' => 'gaelîka skotî',
        'gez' => 'geez',
        'gil' => 'kîrîbatî',
        'gl' => 'galîsî',
        'gn' => 'guwaranî',
        'gor' => 'gorontaloyî',
        'gsw' => 'elmanîşî',
        'gu' => 'gujaratî',
        'guz' => 'gusîî',
        'gv' => 'manksî',
        'gwi' => 'gwichʼin',
        'ha' => 'hawsayî',
        'hai' => 'haydayî',
        'haw' => 'hawayî',
        'hax' => 'haîdaya başûrî',
        'he' => 'îbranî',
        'hi' => 'hindî',
        'hil' => 'hîlîgaynonî',
        'hmn' => 'hmongî',
        'hnj' => 'hmongîya njuayî',
        'hr' => 'xirwatî',
        'hsb' => 'sorbîya jorîn',
        'ht' => 'haîtî',
        'hu' => 'macarî',
        'hup' => 'hupayî',
        'hur' => 'halkomelemî',
        'hy' => 'ermenî',
        'hz' => 'hereroyî',
        'ia' => 'înterlîngua',
        'iba' => 'iban',
        'ibb' => 'îbîbîoyî',
        'id' => 'endonezyayî',
        'ie' => 'înterlîngue',
        'ig' => 'îgboyî',
        'ii' => 'yîyîya siçuwayî',
        'ikt' => 'inuvialuktun',
        'ilo' => 'îlokanoyî',
        'inh' => 'îngûşî',
        'io' => 'îdoyî',
        'is' => 'îzlendî',
        'it' => 'îtalî',
        'iu' => 'înuîtî',
        'ja' => 'japonî',
        'jbo' => 'lojbanî',
        'jgo' => 'ngomba',
        'jmc' => 'machame',
        'jv' => 'javayî',
        'ka' => 'gurcî',
        'kaa' => 'kara-kalpakî',
        'kab' => 'kabîlî',
        'kac' => 'kaçînî',
        'kaj' => 'jju',
        'kam' => 'kambayî',
        'kbd' => 'kabardî',
        'kcg' => 'tyap',
        'kde' => 'makondeyî',
        'kea' => 'kapverdî',
        'kek' => 'qeqçîyî',
        'ken' => 'kenyangî',
        'kfo' => 'koro',
        'kgp' => 'kayingangî',
        'kha' => 'khasi',
        'khq' => 'koyra chiini',
        'ki' => 'kîkûyûyî',
        'kj' => 'kwanyamayî',
        'kk' => 'qazaxî',
        'kkj' => 'kako',
        'kl' => 'kalalîsûtî',
        'kln' => 'kalencînî',
        'km' => 'khmerî',
        'kmb' => 'kîmbunduyî',
        'kn' => 'kannadayî',
        'ko' => 'koreyî',
        'kok' => 'konkanî',
        'kpe' => 'kpelleyî',
        'kr' => 'kanurîyî',
        'krc' => 'karaçay-balkarî',
        'krl' => 'karelî',
        'kru' => 'kurukh',
        'ks' => 'keşmîrî',
        'ksb' => 'shambala',
        'ksf' => 'bafyayî',
        'ksh' => 'rîpwarî',
        'ku' => 'kurdî (kurmancî)',
        'kum' => 'kumikî',
        'kv' => 'komî',
        'kw' => 'kornî',
        'kwk' => 'kwak’walayî',
        'kxv' => 'kuvî',
        'ky' => 'qirgizî',
        'la' => 'latînî',
        'lad' => 'ladînoyî',
        'lag' => 'langî',
        'lb' => 'luksembûrgî',
        'lez' => 'lezgînî',
        'lg' => 'lugandayî',
        'li' => 'lîmbûrgî',
        'lij' => 'lîgûrî',
        'lil' => 'lillooet',
        'lkt' => 'lakotayî',
        'lld' => 'ladînî',
        'lmo' => 'lombardî',
        'ln' => 'lingalayî',
        'lo' => 'lao',
        'lou' => 'kreyolîya louisianayê',
        'loz' => 'lozî',
        'lrc' => 'lurîya bakur',
        'lsm' => 'saamia',
        'lt' => 'lîtwanî',
        'ltg' => 'latgalî',
        'lu' => 'luba-katangayî',
        'lua' => 'luba-kasayî',
        'lun' => 'lunda',
        'luo' => 'luoyî',
        'lus' => 'mizoyî',
        'luy' => 'luhyayî',
        'lv' => 'latvîyayî',
        'lzz' => 'lazî',
        'mad' => 'madurayî',
        'mag' => 'magahî',
        'mai' => 'maithili',
        'mak' => 'makasarî',
        'mas' => 'masayî',
        'mdf' => 'mokşayî',
        'men' => 'mende',
        'mer' => 'meruyî',
        'mfe' => 'morisyenî',
        'mg' => 'malagasî',
        'mgh' => 'makhuwa-meetto',
        'mgo' => 'meta’',
        'mh' => 'marşalî',
        'mhn' => 'moçenoyî',
        'mi' => 'maorî',
        'mic' => 'mîkmakî',
        'min' => 'mînangkabawî',
        'mk' => 'makedonî',
        'ml' => 'malayalamî',
        'mn' => 'moxolî',
        'mni' => 'manipuri',
        'moe' => 'înûyîya rojhilatî',
        'moh' => 'mohawkî',
        'mos' => 'moreyî',
        'mr' => 'maratî',
        'ms' => 'malezî',
        'mt' => 'maltayî',
        'mua' => 'mundangî',
        'mus' => 'krîkî',
        'mwl' => 'mîrandî',
        'mww' => 'hmongîya dawî',
        'my' => 'burmayî',
        'myv' => 'erzayî',
        'mzn' => 'mazenderanî',
        'na' => 'nawrûyî',
        'nan' => 'çînîya mîn nanî',
        'nap' => 'napolîtanî',
        'naq' => 'namayî',
        'nb' => 'norwecî (bokmål)',
        'nd' => 'ndebelîya bakurî',
        'nds' => 'nedersaksî',
        'ne' => 'nepalî',
        'new' => 'newarî',
        'ng' => 'ndongayî',
        'nia' => 'nîasî',
        'niu' => 'nîwî',
        'nl' => 'holendî',
        'nmg' => 'kwasio',
        'nn' => 'norwecî (nynorsk)',
        'nnh' => 'ngiemboon',
        'no' => 'norwecî',
        'nog' => 'nogayî',
        'nqo' => 'n’Ko',
        'nr' => 'ndebelîya başûrî',
        'nso' => 'sotoyîya bakur',
        'nus' => 'nuer',
        'nv' => 'navajoyî',
        'ny' => 'çîçewayî',
        'nyn' => 'nyankole',
        'oc' => 'oksîtanî',
        'ojb' => 'ojibweyîya bakurî',
        'ojc' => 'ojibwayîya navîn',
        'ojs' => 'oji-cree',
        'ojw' => 'ojîbweyîya rojavayî',
        'oka' => 'okanagan',
        'om' => 'oromoyî',
        'or' => 'oriyayî',
        'os' => 'osetî',
        'osa' => 'osageyî',
        'pa' => 'puncabî',
        'pag' => 'pangasînanî',
        'pam' => 'kapampanganî',
        'pap' => 'papyamentoyî',
        'pau' => 'palawî',
        'pcm' => 'pîdgînîya nîjeryayî',
        'pi' => 'palî',
        'pis' => 'pijînî',
        'pl' => 'polonî',
        'pms' => 'piedmontîsî',
        'pqm' => 'malecite-passamaquoddy',
        'prg' => 'prûsyayî',
        'ps' => 'peştûyî',
        'pt' => 'portugalî',
        'qu' => 'keçwayî',
        'quc' => 'k’iche’',
        'raj' => 'rajasthanî',
        'rap' => 'rapanuyî',
        'rar' => 'rarotongî',
        'rhg' => 'rohingyayî',
        'rif' => 'tarifit',
        'rm' => 'romancî',
        'rn' => 'rundî',
        'ro' => 'romanî',
        'rof' => 'rombo',
        'ru' => 'rûsî',
        'rup' => 'aromanî',
        'rw' => 'kînyariwandayî',
        'rwk' => 'rwa',
        'sa' => 'sanskrîtî',
        'sad' => 'sandawe',
        'sah' => 'yakutî',
        'saq' => 'samburuyî',
        'sat' => 'santalî',
        'sba' => 'ngambay',
        'sbp' => 'sanguyî',
        'sc' => 'sardînî',
        'scn' => 'sicîlî',
        'sco' => 'skotî',
        'sd' => 'sindhî',
        'sdh' => 'kurdî (xwarîn)',
        'se' => 'samîya bakur',
        'seh' => 'sena',
        'ses' => 'sennîya koyraboro',
        'sg' => 'sangoyî',
        'sgs' => 'samogîtî',
        'shi' => 'taşelhîtî',
        'shn' => 'şanî',
        'si' => 'kîngalî',
        'sid' => 'sîdamo',
        'sk' => 'slovakî',
        'skr' => 'seraîkî',
        'sl' => 'slovenî',
        'slh' => 'lushootseeda başûrî',
        'sm' => 'samoayî',
        'sma' => 'samîya başûr',
        'smj' => 'samîya lule',
        'smn' => 'samîya înarî',
        'sms' => 'samîya skoltî',
        'sn' => 'şonayî',
        'snk' => 'soninke',
        'so' => 'somalî',
        'sq' => 'arnawidî',
        'sr' => 'sirbî',
        'srn' => 'sirananî',
        'ss' => 'swazî',
        'ssy' => 'sahoyî',
        'st' => 'sotoyîya başûr',
        'str' => 'saanîçî',
        'su' => 'sundanî',
        'suk' => 'sukuma',
        'suz' => 'sunwarî',
        'sv' => 'swêdî',
        'sw' => 'swahîlî',
        'swb' => 'komorî',
        'syr' => 'siryanî',
        'szl' => 'silesî',
        'ta' => 'tamîlî',
        'tce' => 'totuçena başûrî',
        'te' => 'telûgûyî',
        'tem' => 'temne',
        'teo' => 'teso',
        'tet' => 'tetûmî',
        'tg' => 'tacikî',
        'tgx' => 'tagîşî',
        'th' => 'tayî',
        'tht' => 'tahltan',
        'ti' => 'tigrînî',
        'tig' => 'tigre',
        'tk' => 'tirkmenî',
        'tlh' => 'klîngonî',
        'tli' => 'tlingit',
        'tn' => 'tswanayî',
        'to' => 'tongî',
        'tok' => 'toki pona',
        'tpi' => 'tokpisinî',
        'tr' => 'tirkî',
        'trv' => 'tarokoyî',
        'trw' => 'torwalî',
        'ts' => 'tsongayî',
        'tt' => 'teterî',
        'ttm' => 'tutoçenîya bakur',
        'tum' => 'tumbukayî',
        'tvl' => 'tuvalûyî',
        'twq' => 'tasawaq',
        'ty' => 'tahîtî',
        'tyv' => 'tuvanî',
        'tzm' => 'temazîxtî',
        'udm' => 'udmurtî',
        'ug' => 'uyxurî',
        'uk' => 'ukraynî',
        'umb' => 'umbunduyî',
        'ur' => 'urdûyî',
        'uz' => 'ozbekî',
        'vai' => 'vaî',
        've' => 'venda',
        'vec' => 'venîsî',
        'vi' => 'vîetnamî',
        'vmw' => 'makhuwayî',
        'vo' => 'volapûkî',
        'vun' => 'vunjo',
        'wa' => 'walonî',
        'wae' => 'walserî',
        'wal' => 'wolaytta',
        'war' => 'warayî',
        'wbp' => 'warlpiri',
        'wo' => 'wolofî',
        'wuu' => 'çînîya wuyî',
        'xal' => 'kalmîkî',
        'xh' => 'xosayî',
        'xnr' => 'kangrî',
        'xog' => 'sogayî',
        'yav' => 'yangben',
        'ybb' => 'yemba',
        'yi' => 'yidîşî',
        'yo' => 'yorubayî',
        'yrl' => 'nhêngatûyî',
        'yue' => 'kantonî',
        'za' => 'zhuangî',
        'zgh' => 'amazîxîya fasî',
        'zh' => 'çînî',
        'zu' => 'zuluyî',
        'zun' => 'zunîyî',
        'zza' => 'zazakî',
    ],
    'LocalizedNames' => [
        'ar_001' => 'erebîya modern a standard',
        'de_CH' => 'almanîya bilind a swîsreyî',
        'en_GB' => 'îngilîzî (brîtanî)',
        'en_US' => 'îngilîzî (amerîkî)',
        'es_ES' => 'spanî (Ewropa)',
        'fa_AF' => 'darî',
        'nl_BE' => 'flamî',
        'pt_PT' => 'portugalî (Ewropa)',
        'sw_CD' => 'swahîlîya kongoyî',
        'zh_Hans' => 'çînîya sadekirî',
        'zh_Hant' => 'çînîya edetî',
    ],
];
