<?php

return [
    'Names' => [
        'Africa/Abidjan' => '格林威治標準時間（阿比讓）',
        'Africa/Accra' => '格林威治標準時間（阿克拉）',
        'Africa/Addis_Ababa' => '東非時間（阿迪斯阿貝巴）',
        'Africa/Algiers' => '中歐時間（阿爾及爾）',
        'Africa/Asmera' => '東非時間（阿斯瑪拉）',
        'Africa/Asmara' => '東非時間（阿斯瑪拉）',
        'Africa/Bamako' => '格林威治標準時間（巴馬科）',
        'Africa/Bangui' => '西非時間（班吉）',
        'Africa/Banjul' => '格林威治標準時間（班竹）',
        'Africa/Bissau' => '格林威治標準時間（比紹）',
        'Africa/Blantyre' => '中非時間（布蘭太爾）',
        'Africa/Brazzaville' => '西非時間（布拉柴維爾）',
        'Africa/Bujumbura' => '中非時間（布松布拉）',
        'Africa/Cairo' => '東歐時間（開羅）',
        'Africa/Casablanca' => '西歐時間（卡薩布蘭卡）',
        'Africa/Ceuta' => '中歐時間（休達）',
        'Africa/Conakry' => '格林威治標準時間（柯那克里）',
        'Africa/Dakar' => '格林威治標準時間（達喀爾）',
        'Africa/Dar_es_Salaam' => '東非時間（沙蘭港）',
        'Africa/Djibouti' => '東非時間（吉布地）',
        'Africa/Douala' => '西非時間（杜阿拉）',
        'Africa/El_Aaiun' => '西歐時間（阿尤恩）',
        'Africa/Freetown' => '格林威治標準時間（自由城）',
        'Africa/Gaborone' => '中非時間（嘉柏隆里）',
        'Africa/Harare' => '中非時間（哈拉雷）',
        'Africa/Johannesburg' => '南非標準時間（約翰尼斯堡）',
        'Africa/Juba' => '中非時間（朱巴）',
        'Africa/Kampala' => '東非時間（坎帕拉）',
        'Africa/Khartoum' => '中非時間（喀土穆）',
        'Africa/Kigali' => '中非時間（基加利）',
        'Africa/Kinshasa' => '西非時間（金夏沙）',
        'Africa/Lagos' => '西非時間（拉哥斯）',
        'Africa/Libreville' => '西非時間（自由市）',
        'Africa/Lome' => '格林威治標準時間（洛美）',
        'Africa/Luanda' => '西非時間（羅安達）',
        'Africa/Lubumbashi' => '中非時間（盧本巴希）',
        'Africa/Lusaka' => '中非時間（路沙卡）',
        'Africa/Malabo' => '西非時間（馬拉博）',
        'Africa/Maputo' => '中非時間（馬普托）',
        'Africa/Maseru' => '南非標準時間（馬賽魯）',
        'Africa/Mbabane' => '南非標準時間（墨巴本）',
        'Africa/Mogadishu' => '東非時間（摩加迪休）',
        'Africa/Monrovia' => '格林威治標準時間（蒙羅維亞）',
        'Africa/Nairobi' => '東非時間（奈洛比）',
        'Africa/Ndjamena' => '西非時間（恩賈梅納）',
        'Africa/Niamey' => '西非時間（尼亞美）',
        'Africa/Nouakchott' => '格林威治標準時間（諾克少）',
        'Africa/Ouagadougou' => '格林威治標準時間（瓦加杜古）',
        'Africa/Porto-Novo' => '西非時間（波多諾佛）',
        'Africa/Sao_Tome' => '格林威治標準時間（聖多美）',
        'Africa/Tripoli' => '東歐時間（的黎波里）',
        'Africa/Tunis' => '中歐時間（突尼斯）',
        'Africa/Windhoek' => '中非時間（溫得和克）',
        'America/Adak' => '夏威夷-阿留申時間（艾達克）',
        'America/Anchorage' => '阿拉斯加時間（安克拉治）',
        'America/Anguilla' => '大西洋時間（安奎拉）',
        'America/Antigua' => '大西洋時間（安地卡）',
        'America/Araguaina' => '巴西利亞時間（阿拉圭那）',
        'America/Argentina/La_Rioja' => '阿根廷時間（拉略哈）',
        'America/Argentina/Rio_Gallegos' => '阿根廷時間（里奧加耶戈斯）',
        'America/Argentina/Salta' => '阿根廷時間（薩爾塔）',
        'America/Argentina/San_Juan' => '阿根廷時間（聖胡安）',
        'America/Argentina/San_Luis' => '阿根廷時間（聖路易）',
        'America/Argentina/Tucuman' => '阿根廷時間（吐庫曼）',
        'America/Argentina/Ushuaia' => '阿根廷時間（烏斯懷亞）',
        'America/Aruba' => '大西洋時間（荷屬阿魯巴）',
        'America/Asuncion' => '巴拉圭時間（亞松森）',
        'America/Bahia' => '巴西利亞時間（巴伊阿）',
        'America/Bahia_Banderas' => '中部時間（巴伊亞班德拉斯）',
        'America/Barbados' => '大西洋時間（巴貝多）',
        'America/Belem' => '巴西利亞時間（貝倫）',
        'America/Belize' => '中部時間（貝里斯）',
        'America/Blanc-Sablon' => '大西洋時間（白朗薩布隆）',
        'America/Boa_Vista' => '亞馬遜時間（保維斯塔）',
        'America/Bogota' => '哥倫比亞時間（波哥大）',
        'America/Boise' => '山區時間（波夕）',
        'America/Buenos_Aires' => '阿根廷時間（布宜諾斯艾利斯）',
        'America/Argentina/Buenos_Aires' => '阿根廷時間（布宜諾斯艾利斯）',
        'America/Cambridge_Bay' => '山區時間（劍橋灣）',
        'America/Campo_Grande' => '亞馬遜時間（格蘭場）',
        'America/Cancun' => '東部時間（坎昆）',
        'America/Caracas' => '委內瑞拉時間（卡拉卡斯）',
        'America/Catamarca' => '阿根廷時間（卡塔馬卡）',
        'America/Argentina/Catamarca' => '阿根廷時間（卡塔馬卡）',
        'America/Cayenne' => '法屬圭亞那時間（開雲）',
        'America/Cayman' => '東部時間（開曼群島）',
        'America/Chicago' => '中部時間（芝加哥）',
        'America/Chihuahua' => '中部時間（奇華華）',
        'America/Ciudad_Juarez' => '山區時間（華雷斯）',
        'America/Coral_Harbour' => '東部時間（阿蒂科肯）',
        'America/Atikokan' => '東部時間（阿蒂科肯）',
        'America/Cordoba' => '阿根廷時間（哥多華）',
        'America/Argentina/Cordoba' => '阿根廷時間（哥多華）',
        'America/Costa_Rica' => '中部時間（哥斯大黎加）',
        'America/Coyhaique' => '智利時間（科伊艾克）',
        'America/Creston' => '山區時間（克雷斯頓）',
        'America/Cuiaba' => '亞馬遜時間（古雅巴）',
        'America/Curacao' => '大西洋時間（庫拉索）',
        'America/Danmarkshavn' => '格林威治標準時間（丹馬沙文）',
        'America/Dawson' => '育空地區時間（道森）',
        'America/Dawson_Creek' => '山區時間（道森克里克）',
        'America/Denver' => '山區時間（丹佛）',
        'America/Detroit' => '東部時間（底特律）',
        'America/Dominica' => '大西洋時間（多米尼克）',
        'America/Edmonton' => '山區時間（艾德蒙吞）',
        'America/Eirunepe' => '艾克時間（艾魯內佩）',
        'America/El_Salvador' => '中部時間（薩爾瓦多）',
        'America/Fort_Nelson' => '山區時間（納爾遜堡）',
        'America/Fortaleza' => '巴西利亞時間（福塔力莎）',
        'America/Glace_Bay' => '大西洋時間（格雷斯貝）',
        'America/Godthab' => '格陵蘭時間（努克）',
        'America/Nuuk' => '格陵蘭時間（努克）',
        'America/Goose_Bay' => '大西洋時間（鵝灣）',
        'America/Grand_Turk' => '東部時間（大特克島）',
        'America/Grenada' => '大西洋時間（格瑞納達）',
        'America/Guadeloupe' => '大西洋時間（瓜地洛普）',
        'America/Guatemala' => '中部時間（瓜地馬拉）',
        'America/Guayaquil' => '厄瓜多時間（瓜亞基爾）',
        'America/Guyana' => '蓋亞那時間',
        'America/Halifax' => '大西洋時間（哈里法克斯）',
        'America/Havana' => '古巴時間（哈瓦那）',
        'America/Hermosillo' => '墨西哥太平洋時間（埃莫西約）',
        'America/Indiana/Knox' => '中部時間（印第安那州諾克斯）',
        'America/Indiana/Marengo' => '東部時間（印第安那州馬倫哥）',
        'America/Indiana/Petersburg' => '東部時間（印第安那州彼得堡）',
        'America/Indiana/Tell_City' => '中部時間（印第安那州泰爾城）',
        'America/Indiana/Vevay' => '東部時間（印第安那州維威）',
        'America/Indiana/Vincennes' => '東部時間（印第安那州溫森斯）',
        'America/Indiana/Winamac' => '東部時間（印第安那州威納馬克）',
        'America/Indianapolis' => '東部時間（印第安那波里斯）',
        'America/Indiana/Indianapolis' => '東部時間（印第安那波里斯）',
        'America/Inuvik' => '山區時間（伊奴維克）',
        'America/Iqaluit' => '東部時間（伊魁特）',
        'America/Jamaica' => '東部時間（牙買加）',
        'America/Jujuy' => '阿根廷時間（胡胡伊）',
        'America/Argentina/Jujuy' => '阿根廷時間（胡胡伊）',
        'America/Juneau' => '阿拉斯加時間（朱諾）',
        'America/Kentucky/Monticello' => '東部時間（肯塔基州蒙地卻羅）',
        'America/Kralendijk' => '大西洋時間（克拉倫代克）',
        'America/La_Paz' => '玻利維亞時間（拉巴斯）',
        'America/Lima' => '秘魯時間（利馬）',
        'America/Los_Angeles' => '太平洋時間（洛杉磯）',
        'America/Louisville' => '東部時間（路易斯維爾）',
        'America/Kentucky/Louisville' => '東部時間（路易斯維爾）',
        'America/Lower_Princes' => '大西洋時間（下太子區）',
        'America/Maceio' => '巴西利亞時間（馬瑟歐）',
        'America/Managua' => '中部時間（馬拿瓜）',
        'America/Manaus' => '亞馬遜時間（瑪瑙斯）',
        'America/Marigot' => '大西洋時間（馬里戈特）',
        'America/Martinique' => '大西洋時間（馬丁尼克）',
        'America/Matamoros' => '中部時間（馬塔莫羅斯）',
        'America/Mazatlan' => '墨西哥太平洋時間（馬薩特蘭）',
        'America/Mendoza' => '阿根廷時間（門多薩）',
        'America/Argentina/Mendoza' => '阿根廷時間（門多薩）',
        'America/Menominee' => '中部時間（美諾米尼）',
        'America/Merida' => '中部時間（梅里達）',
        'America/Metlakatla' => '阿拉斯加時間（梅特拉卡特拉）',
        'America/Mexico_City' => '中部時間（墨西哥市）',
        'America/Miquelon' => '聖皮埃與密克隆群島時間（密啟崙）',
        'America/Moncton' => '大西洋時間（蒙克頓）',
        'America/Monterrey' => '中部時間（蒙特瑞）',
        'America/Montevideo' => '烏拉圭時間（蒙特維多）',
        'America/Montserrat' => '大西洋時間（蒙哲臘）',
        'America/Nassau' => '東部時間（拿索）',
        'America/New_York' => '東部時間（紐約）',
        'America/Nome' => '阿拉斯加時間（諾姆）',
        'America/Noronha' => '費爾南多 - 迪諾羅尼亞時間（諾倫哈）',
        'America/North_Dakota/Beulah' => '中部時間（北達科他州布由拉）',
        'America/North_Dakota/Center' => '中部時間（北達科他州中心）',
        'America/North_Dakota/New_Salem' => '中部時間（北達科他州紐沙倫）',
        'America/Ojinaga' => '中部時間（奧希納加）',
        'America/Panama' => '東部時間（巴拿馬）',
        'America/Paramaribo' => '蘇利南時間（巴拉馬利波）',
        'America/Phoenix' => '山區時間（鳳凰城）',
        'America/Port-au-Prince' => '東部時間（太子港）',
        'America/Port_of_Spain' => '大西洋時間（西班牙港）',
        'America/Porto_Velho' => '亞馬遜時間（維留港）',
        'America/Puerto_Rico' => '大西洋時間（波多黎各）',
        'America/Punta_Arenas' => '智利時間（蓬塔阿雷納斯）',
        'America/Rankin_Inlet' => '中部時間（蘭今灣）',
        'America/Recife' => '巴西利亞時間（雷西非）',
        'America/Regina' => '中部時間（里賈納）',
        'America/Resolute' => '中部時間（羅斯魯特）',
        'America/Rio_Branco' => '艾克時間（里約布蘭）',
        'America/Santarem' => '巴西利亞時間（聖塔倫）',
        'America/Santiago' => '智利時間（聖地牙哥）',
        'America/Santo_Domingo' => '大西洋時間（聖多明哥）',
        'America/Sao_Paulo' => '巴西利亞時間（聖保羅）',
        'America/Scoresbysund' => '格陵蘭時間（伊托科爾托米特）',
        'America/Sitka' => '阿拉斯加時間（錫特卡）',
        'America/St_Barthelemy' => '大西洋時間（聖巴托洛繆島）',
        'America/St_Johns' => '紐芬蘭時間（聖約翰）',
        'America/St_Kitts' => '大西洋時間（聖基茨）',
        'America/St_Lucia' => '大西洋時間（聖露西亞）',
        'America/St_Thomas' => '大西洋時間（聖托馬斯）',
        'America/St_Vincent' => '大西洋時間（聖文森）',
        'America/Swift_Current' => '中部時間（斯威夫特卡倫特）',
        'America/Tegucigalpa' => '中部時間（德古斯加巴）',
        'America/Thule' => '大西洋時間（杜里）',
        'America/Tijuana' => '太平洋時間（提華納）',
        'America/Toronto' => '東部時間（多倫多）',
        'America/Tortola' => '大西洋時間（托爾托拉）',
        'America/Vancouver' => '太平洋時間（溫哥華）',
        'America/Whitehorse' => '育空地區時間（懷特霍斯）',
        'America/Winnipeg' => '中部時間（溫尼伯）',
        'America/Yakutat' => '阿拉斯加時間（雅庫塔）',
        'Antarctica/Casey' => '澳洲西部時間（凱西）',
        'Antarctica/Davis' => '戴維斯時間',
        'Antarctica/DumontDUrville' => '杜蒙杜比爾時間',
        'Antarctica/Macquarie' => '澳洲東部時間（麥覺理）',
        'Antarctica/Mawson' => '莫森時間',
        'Antarctica/McMurdo' => '紐西蘭時間（麥克默多）',
        'Antarctica/Palmer' => '智利時間（帕麥）',
        'Antarctica/Rothera' => '羅瑟拉時間',
        'Antarctica/Syowa' => '昭和基地時間',
        'Antarctica/Troll' => '格林威治標準時間（綽爾）',
        'Antarctica/Vostok' => '沃斯托克時間',
        'Arctic/Longyearbyen' => '中歐時間（隆意耳拜恩）',
        'Asia/Aden' => '阿拉伯時間（亞丁）',
        'Asia/Almaty' => '哈薩克時間（阿拉木圖）',
        'Asia/Amman' => '東歐時間（安曼）',
        'Asia/Anadyr' => '彼得羅巴甫洛夫斯克時間（阿那底）',
        'Asia/Aqtau' => '哈薩克時間（阿克套）',
        'Asia/Aqtobe' => '哈薩克時間（阿克托比）',
        'Asia/Ashgabat' => '土庫曼時間（阿什哈巴特）',
        'Asia/Atyrau' => '哈薩克時間（阿特勞）',
        'Asia/Baghdad' => '阿拉伯時間（巴格達）',
        'Asia/Bahrain' => '阿拉伯時間（巴林）',
        'Asia/Baku' => '亞塞拜然時間（巴庫）',
        'Asia/Bangkok' => '中南半島時間（曼谷）',
        'Asia/Barnaul' => '克拉斯諾亞爾斯克時間（巴爾瑙爾）',
        'Asia/Beirut' => '東歐時間（貝魯特）',
        'Asia/Bishkek' => '吉爾吉斯時間（比什凱克）',
        'Asia/Brunei' => '汶萊時間',
        'Asia/Calcutta' => '印度標準時間（加爾各答）',
        'Asia/Kolkata' => '印度標準時間（加爾各答）',
        'Asia/Chita' => '雅庫次克時間（赤塔）',
        'Asia/Colombo' => '印度標準時間（可倫坡）',
        'Asia/Damascus' => '東歐時間（大馬士革）',
        'Asia/Dhaka' => '孟加拉時間（達卡）',
        'Asia/Dili' => '東帝汶時間（帝力）',
        'Asia/Dubai' => '波斯灣海域標準時間（杜拜）',
        'Asia/Dushanbe' => '塔吉克時間（杜桑貝）',
        'Asia/Famagusta' => '東歐時間（法馬古斯塔）',
        'Asia/Gaza' => '東歐時間（加薩）',
        'Asia/Hebron' => '東歐時間（赫布隆）',
        'Asia/Hong_Kong' => '香港時間',
        'Asia/Hovd' => '科布多時間',
        'Asia/Irkutsk' => '伊爾庫次克時間',
        'Asia/Jakarta' => '印尼西部時間（雅加達）',
        'Asia/Jayapura' => '印尼東部時間（加亞布拉）',
        'Asia/Jerusalem' => '以色列時間（耶路撒冷）',
        'Asia/Kabul' => '阿富汗時間（喀布爾）',
        'Asia/Kamchatka' => '彼得羅巴甫洛夫斯克時間（堪察加）',
        'Asia/Karachi' => '巴基斯坦時間（喀拉蚩）',
        'Asia/Katmandu' => '尼泊爾時間（加德滿都）',
        'Asia/Kathmandu' => '尼泊爾時間（加德滿都）',
        'Asia/Khandyga' => '雅庫次克時間（堪地加）',
        'Asia/Krasnoyarsk' => '克拉斯諾亞爾斯克時間',
        'Asia/Kuala_Lumpur' => '馬來西亞時間（吉隆坡）',
        'Asia/Kuching' => '馬來西亞時間（古晉）',
        'Asia/Kuwait' => '阿拉伯時間（科威特）',
        'Asia/Macau' => '中國時間（澳門）',
        'Asia/Magadan' => '馬加丹時間',
        'Asia/Makassar' => '印尼中部時間（馬卡沙爾）',
        'Asia/Manila' => '菲律賓時間（馬尼拉）',
        'Asia/Muscat' => '波斯灣海域標準時間（馬斯開特）',
        'Asia/Nicosia' => '東歐時間（尼古西亞）',
        'Asia/Novokuznetsk' => '克拉斯諾亞爾斯克時間（新庫茲涅茨克）',
        'Asia/Novosibirsk' => '克拉斯諾亞爾斯克時間（新西伯利亞）',
        'Asia/Omsk' => '鄂木斯克時間',
        'Asia/Oral' => '哈薩克時間（烏拉爾）',
        'Asia/Phnom_Penh' => '中南半島時間（金邊）',
        'Asia/Pontianak' => '印尼西部時間（坤甸）',
        'Asia/Pyongyang' => '韓國時間（平壤）',
        'Asia/Qatar' => '阿拉伯時間（卡達）',
        'Asia/Qostanay' => '哈薩克時間（庫斯塔奈）',
        'Asia/Qyzylorda' => '哈薩克時間（克孜勒奧爾達）',
        'Asia/Rangoon' => '緬甸時間（仰光）',
        'Asia/Yangon' => '緬甸時間（仰光）',
        'Asia/Riyadh' => '阿拉伯時間（利雅德）',
        'Asia/Saigon' => '中南半島時間（胡志明市）',
        'Asia/Ho_Chi_Minh' => '中南半島時間（胡志明市）',
        'Asia/Sakhalin' => '馬加丹時間（庫頁島）',
        'Asia/Samarkand' => '烏茲別克時間（撒馬爾罕）',
        'Asia/Seoul' => '韓國時間（首爾）',
        'Asia/Shanghai' => '中國時間（上海）',
        'Asia/Singapore' => '新加坡標準時間',
        'Asia/Srednekolymsk' => '馬加丹時間（中科雷姆斯克）',
        'Asia/Taipei' => '台北時間',
        'Asia/Tashkent' => '烏茲別克時間（塔什干）',
        'Asia/Tbilisi' => '喬治亞時間（第比利斯）',
        'Asia/Tehran' => '伊朗時間（德黑蘭）',
        'Asia/Thimphu' => '不丹時間（廷布）',
        'Asia/Tokyo' => '日本時間（東京）',
        'Asia/Tomsk' => '克拉斯諾亞爾斯克時間（托木斯克）',
        'Asia/Ulaanbaatar' => '烏蘭巴托時間',
        'Asia/Urumqi' => '中國時間（烏魯木齊）',
        'Asia/Ust-Nera' => '海參崴時間（烏斯內拉）',
        'Asia/Vientiane' => '中南半島時間（永珍）',
        'Asia/Vladivostok' => '海參崴時間',
        'Asia/Yakutsk' => '雅庫次克時間',
        'Asia/Yekaterinburg' => '葉卡捷琳堡時間（葉卡捷林堡）',
        'Asia/Yerevan' => '亞美尼亞時間（葉里溫）',
        'Atlantic/Azores' => '亞速爾群島時間',
        'Atlantic/Bermuda' => '大西洋時間（百慕達）',
        'Atlantic/Canary' => '西歐時間（加納利）',
        'Atlantic/Cape_Verde' => '維德角時間',
        'Atlantic/Faeroe' => '西歐時間（法羅群島）',
        'Atlantic/Faroe' => '西歐時間（法羅群島）',
        'Atlantic/Madeira' => '西歐時間（馬得拉群島）',
        'Atlantic/Reykjavik' => '格林威治標準時間（雷克雅維克）',
        'Atlantic/South_Georgia' => '南喬治亞時間',
        'Atlantic/St_Helena' => '格林威治標準時間（聖赫勒拿島）',
        'Atlantic/Stanley' => '福克蘭群島時間（史坦利）',
        'Australia/Adelaide' => '澳洲中部時間（阿得雷德）',
        'Australia/Brisbane' => '澳洲東部時間（布利斯班）',
        'Australia/Broken_Hill' => '澳洲中部時間（布羅肯希爾）',
        'Australia/Darwin' => '澳洲中部時間（達爾文）',
        'Australia/Eucla' => '澳洲中西部時間（尤克拉）',
        'Australia/Hobart' => '澳洲東部時間（荷巴特）',
        'Australia/Lindeman' => '澳洲東部時間（林德曼）',
        'Australia/Lord_Howe' => '豪勳爵島時間',
        'Australia/Melbourne' => '澳洲東部時間（墨爾本）',
        'Australia/Perth' => '澳洲西部時間（伯斯）',
        'Australia/Sydney' => '澳洲東部時間（雪梨）',
        'Etc/GMT' => '格林威治標準時間',
        'Etc/UTC' => '世界標準時間',
        'Europe/Amsterdam' => '中歐時間（阿姆斯特丹）',
        'Europe/Andorra' => '中歐時間（安道爾）',
        'Europe/Astrakhan' => '薩馬拉時間（阿斯特拉罕）',
        'Europe/Athens' => '東歐時間（雅典）',
        'Europe/Belgrade' => '中歐時間（貝爾格勒）',
        'Europe/Berlin' => '中歐時間（柏林）',
        'Europe/Bratislava' => '中歐時間（布拉提斯拉瓦）',
        'Europe/Brussels' => '中歐時間（布魯塞爾）',
        'Europe/Bucharest' => '東歐時間（布加勒斯特）',
        'Europe/Budapest' => '中歐時間（布達佩斯）',
        'Europe/Busingen' => '中歐時間（布辛根）',
        'Europe/Chisinau' => '東歐時間（基西紐）',
        'Europe/Copenhagen' => '中歐時間（哥本哈根）',
        'Europe/Dublin' => '格林威治標準時間（都柏林）',
        'Europe/Gibraltar' => '中歐時間（直布羅陀）',
        'Europe/Guernsey' => '格林威治標準時間（根息島）',
        'Europe/Helsinki' => '東歐時間（赫爾辛基）',
        'Europe/Isle_of_Man' => '格林威治標準時間（曼島）',
        'Europe/Istanbul' => '土耳其時間（伊斯坦堡）',
        'Europe/Jersey' => '格林威治標準時間（澤西島）',
        'Europe/Kaliningrad' => '東歐時間（加里寧格勒）',
        'Europe/Kiev' => '東歐時間（基輔）',
        'Europe/Kyiv' => '東歐時間（基輔）',
        'Europe/Kirov' => '莫斯科時間（基洛夫）',
        'Europe/Lisbon' => '西歐時間（里斯本）',
        'Europe/Ljubljana' => '中歐時間（盧比安納）',
        'Europe/London' => '格林威治標準時間（倫敦）',
        'Europe/Luxembourg' => '中歐時間（盧森堡）',
        'Europe/Madrid' => '中歐時間（馬德里）',
        'Europe/Malta' => '中歐時間（馬爾他）',
        'Europe/Mariehamn' => '東歐時間（瑪麗港）',
        'Europe/Minsk' => '莫斯科時間（明斯克）',
        'Europe/Monaco' => '中歐時間（摩納哥）',
        'Europe/Moscow' => '莫斯科時間',
        'Europe/Oslo' => '中歐時間（奧斯陸）',
        'Europe/Paris' => '中歐時間（巴黎）',
        'Europe/Podgorica' => '中歐時間（波多里察）',
        'Europe/Prague' => '中歐時間（布拉格）',
        'Europe/Riga' => '東歐時間（里加）',
        'Europe/Rome' => '中歐時間（羅馬）',
        'Europe/Samara' => '薩馬拉時間（沙馬拉）',
        'Europe/San_Marino' => '中歐時間（聖馬利諾）',
        'Europe/Sarajevo' => '中歐時間（塞拉耶佛）',
        'Europe/Saratov' => '薩馬拉時間（薩拉托夫）',
        'Europe/Simferopol' => '莫斯科時間（辛非洛浦）',
        'Europe/Skopje' => '中歐時間（史高比耶）',
        'Europe/Sofia' => '東歐時間（索菲亞）',
        'Europe/Stockholm' => '中歐時間（斯德哥爾摩）',
        'Europe/Tallinn' => '東歐時間（塔林）',
        'Europe/Tirane' => '中歐時間（地拉那）',
        'Europe/Ulyanovsk' => '薩馬拉時間（烏里揚諾夫斯克）',
        'Europe/Vaduz' => '中歐時間（瓦都茲）',
        'Europe/Vatican' => '中歐時間（梵蒂岡）',
        'Europe/Vienna' => '中歐時間（維也納）',
        'Europe/Vilnius' => '東歐時間（維爾紐斯）',
        'Europe/Volgograd' => '莫斯科時間（伏爾加格勒）',
        'Europe/Warsaw' => '中歐時間（華沙）',
        'Europe/Zagreb' => '中歐時間（札格瑞布）',
        'Europe/Zurich' => '中歐時間（蘇黎世）',
        'Indian/Antananarivo' => '東非時間（安塔那那利弗）',
        'Indian/Chagos' => '印度洋時間（查戈斯）',
        'Indian/Christmas' => '聖誕島時間',
        'Indian/Cocos' => '科科斯群島時間',
        'Indian/Comoro' => '東非時間（科摩羅群島）',
        'Indian/Kerguelen' => '法國南方及南極時間（凱爾蓋朗島）',
        'Indian/Mahe' => '塞席爾時間（馬埃島）',
        'Indian/Maldives' => '馬爾地夫時間',
        'Indian/Mauritius' => '模里西斯時間',
        'Indian/Mayotte' => '東非時間（馬約特島）',
        'Indian/Reunion' => '留尼旺時間（留尼旺島）',
        'Pacific/Apia' => '阿皮亞時間',
        'Pacific/Auckland' => '紐西蘭時間（奧克蘭）',
        'Pacific/Bougainville' => '巴布亞紐幾內亞時間（布干維爾）',
        'Pacific/Chatham' => '查坦群島時間',
        'Pacific/Easter' => '復活節島時間（復活島）',
        'Pacific/Efate' => '萬那杜時間（埃法特）',
        'Pacific/Enderbury' => '鳳凰群島時間（坎頓島）',
        'Pacific/Kanton' => '鳳凰群島時間（坎頓島）',
        'Pacific/Fakaofo' => '托克勞群島時間（法考福）',
        'Pacific/Fiji' => '斐濟時間',
        'Pacific/Funafuti' => '吐瓦魯時間（富那富提）',
        'Pacific/Galapagos' => '加拉巴哥群島時間',
        'Pacific/Gambier' => '甘比爾群島時間',
        'Pacific/Guadalcanal' => '索羅門群島時間（瓜達康納爾島）',
        'Pacific/Guam' => '查莫洛時間（關島）',
        'Pacific/Honolulu' => '夏威夷-阿留申標準時間（檀香山）',
        'Pacific/Kiritimati' => '萊恩群島時間（基里地馬地島）',
        'Pacific/Kosrae' => '科斯瑞時間',
        'Pacific/Kwajalein' => '馬紹爾群島時間（瓜加林島）',
        'Pacific/Majuro' => '馬紹爾群島時間（馬朱諾）',
        'Pacific/Marquesas' => '馬可薩斯時間（馬可薩斯島）',
        'Pacific/Midway' => '薩摩亞時間（中途島）',
        'Pacific/Nauru' => '諾魯時間',
        'Pacific/Niue' => '紐埃島時間',
        'Pacific/Norfolk' => '諾福克島時間',
        'Pacific/Noumea' => '新喀里多尼亞時間（諾美亞）',
        'Pacific/Pago_Pago' => '薩摩亞時間（巴哥巴哥）',
        'Pacific/Palau' => '帛琉時間',
        'Pacific/Pitcairn' => '皮特肯時間（皮特肯群島）',
        'Pacific/Ponape' => '波納佩時間',
        'Pacific/Pohnpei' => '波納佩時間',
        'Pacific/Port_Moresby' => '巴布亞紐幾內亞時間（莫士比港）',
        'Pacific/Rarotonga' => '庫克群島時間（拉羅湯加）',
        'Pacific/Saipan' => '查莫洛時間（塞班）',
        'Pacific/Tahiti' => '大溪地時間',
        'Pacific/Tarawa' => '吉爾伯特群島時間（塔拉瓦）',
        'Pacific/Tongatapu' => '東加時間（東加塔布島）',
        'Pacific/Truk' => '楚克島時間',
        'Pacific/Chuuk' => '楚克島時間',
        'Pacific/Wake' => '威克島時間',
        'Pacific/Wallis' => '瓦利斯和富圖納群島時間',
    ],
    'Meta' => [],
];
