<?php

return [
    'Names' => [
        'ace' => 'ačehgiella',
        'be' => 'vilgesruoššagiella',
        'bn' => 'bengalagiella',
        'fj' => 'fižigiella',
        'hy' => 'armenagiella',
        'kk' => 'kazakhgiella',
        'km' => 'kambožagiella',
        'ne' => 'nepalagiella',
        'pa' => 'panjabagiella',
        'swb' => 'komoragiella',
        'th' => 'thaigiella',
        'vi' => 'vietnamagiella',
    ],
    'LocalizedNames' => [
        'ar_001' => 'standárda arábagiella',
        'de_AT' => 'nuortariikkalaš duiskkagiella',
        'de_CH' => 'šveicalaš duiskkagiella',
        'en_AU' => 'austrálialaš eaŋgalsgiella',
        'en_CA' => 'kanádalaš eaŋgalsgiella',
        'en_GB' => 'brihttalaš eaŋgalsgiella',
        'en_US' => 'amerihkálaš eaŋgalsgiella',
        'es_419' => 'latiinna-amerihkalaš spánskkagiella',
        'es_ES' => 'espánjalaš spánskkagiella',
        'es_MX' => 'meksikolaš spánskkagiella',
        'fr_CA' => 'kanádalaš fránskkagiella',
        'fr_CH' => 'šveicalaš fránskkagiella',
        'nl_BE' => 'belgialaš hollánddagiella',
        'pt_BR' => 'brasilialaš portugálagiella',
        'pt_PT' => 'portugálalaš portugálagiella',
        'ro_MD' => 'moldávialaš romániagiella',
        'zh_Hans' => 'álkes kiinnágiella',
    ],
];
