<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\RemoteEvent;

/**
 * @author Fabien Potencier <fabien@symfony.com>
 */
class RemoteEvent
{
    public function __construct(
        private readonly string $name,
        private readonly string $id,
        private readonly array $payload,
    ) {
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function getId(): string
    {
        return $this->id;
    }

    public function getPayload(): array
    {
        return $this->payload;
    }
}
