<?php

return [
    'Names' => [
        'GBP' => [
            'GB£',
            'جنيه إسترليني',
        ],
        'SSP' => [
            '£',
            'جنيه جنوب السودان',
        ],
    ],
];
