<?php

return [
    'Names' => [
        'Adlm' => 'Adlam',
        'Arab' => 'hija’iyah',
        'Aran' => 'Nastalik',
        'Armn' => 'Armenia',
        'Beng' => 'Bangla',
        'Bopo' => 'Bopomofo',
        'Brai' => 'Braille',
        'Cakm' => 'Chakma',
        'Cans' => 'Wanda Manunggal Aborigin Kanada',
        'Cher' => 'Sherokee',
        'Cyrl' => 'Sirilik',
        'Deva' => 'Devanagari',
        'Ethi' => 'Ethiopik',
        'Geor' => 'Georgia',
        'Grek' => 'Yunani',
        'Gujr' => 'Gujarati',
        'Guru' => 'Gurmukhi',
        'Hanb' => 'Han nganggo Bopomofo',
        'Hang' => 'Hangul',
        'Hani' => 'Han',
        'Hans' => 'Prasaja',
        'Hant' => 'Tradhisional',
        'Hebr' => 'Ibrani',
        'Hira' => 'Hiragana',
        'Hrkt' => 'Silabaris Jepang',
        'Jpan' => 'Jepang',
        'Kana' => 'Katakana',
        'Khmr' => 'Khmer',
        'Knda' => 'Kannada',
        'Kore' => 'Korea',
        'Laoo' => 'Lao',
        'Latn' => 'Latin',
        'Mlym' => 'Malayalam',
        'Mong' => 'Mongolia',
        'Mtei' => 'Meitei Mayek',
        'Mymr' => 'Myanmar',
        'Nkoo' => 'N’Ko',
        'Olck' => 'Ol Chiki',
        'Orya' => 'Odia',
        'Rohg' => 'Hanifi',
        'Sinh' => 'Sinhala',
        'Sund' => 'Sunda',
        'Syrc' => 'Siriak',
        'Taml' => 'Tamil',
        'Telu' => 'Telugu',
        'Tfng' => 'Tifinak',
        'Thaa' => 'Thaana',
        'Thai' => 'Thailand',
        'Tibt' => 'Tibet',
        'Vaii' => 'Vai',
        'Yiii' => 'Yi',
        'Zmth' => 'Notasi Matematika',
        'Zsye' => 'Emoji',
        'Zsym' => 'Simbol',
        'Zxxx' => 'Ora Ketulis',
        'Zyyy' => 'Umum',
    ],
];
