<?php

return [
    'Names' => [
        'AED' => [
            'AED',
            'Diramu ya Falme za Kiarabu',
        ],
        'AFN' => [
            'AFN',
            'Afghani ya Afghanistani',
        ],
        'ANG' => [
            'ANG',
            'Gilda ya Antili ya Uholanzi',
        ],
        'AZN' => [
            'AZN',
            'Manati ya Azabajani',
        ],
        'BAM' => [
            'BAM',
            'Maki ya Bosnia na Hezegovina Inayoweza Kubadilishwa',
        ],
        'BBD' => [
            'BBD',
            'Dola ya Babadosi',
        ],
        'BDT' => [
            'BDT',
            'Taka ya Bangladeshi',
        ],
        'BMD' => [
            'BMD',
            'Dola ya Bamuda',
        ],
        'BRL' => [
            'R$',
            'Reale ya Brazili',
        ],
        'BSD' => [
            'BSD',
            'Dola ya Bahama',
        ],
        'BTN' => [
            'BTN',
            'Ngultrumi ya Bhutani',
        ],
        'BYN' => [
            'BYN',
            'Ruble ya Belarusi',
        ],
        'CAD' => [
            'CA$',
            'Dola ya Kanada',
        ],
        'CNH' => [
            'CNH',
            'Yuan ya China (huru)',
        ],
        'CNY' => [
            'CN¥',
            'Yuan ya China',
        ],
        'CVE' => [
            'CVE',
            'Eskudo ya Kepuvede',
        ],
        'CZK' => [
            'CZK',
            'Koruna ya Cheki',
        ],
        'DKK' => [
            'DKK',
            'Kroni ya Denmaki',
        ],
        'DZD' => [
            'DZD',
            'Dinari ya Aljeria',
        ],
        'GEL' => [
            'GEL',
            'Lari ya Jiojia',
        ],
        'GHS' => [
            'GHS',
            'Sidi ya Ghana',
        ],
        'GIP' => [
            'GIP',
            'Pauni ya Jibrata',
        ],
        'HRK' => [
            'HRK',
            'Kuna ya Kroeshia',
        ],
        'HTG' => [
            'HTG',
            'Godi ya Haiti',
        ],
        'HUF' => [
            'HUF',
            'Forinti ya Hungaria',
        ],
        'IDR' => [
            'IDR',
            'Rupia ya Indonesia',
        ],
        'IRR' => [
            'IRR',
            'Riali ya Irani',
        ],
        'JOD' => [
            'JOD',
            'Dinari ya Yordani',
        ],
        'JPY' => [
            'JP¥',
            'Yeni ya Japani',
        ],
        'KHR' => [
            'KHR',
            'Rieli ya Kambodia',
        ],
        'KZT' => [
            'KZT',
            'Tenge ya Kazakistani',
        ],
        'LBP' => [
            'LBP',
            'Pauni ya Lebanoni',
        ],
        'MAD' => [
            'MAD',
            'Diramu ya Moroko',
        ],
        'MKD' => [
            'MKD',
            'Dinari ya Masedonia',
        ],
        'MMK' => [
            'MMK',
            'Kiati ya Myama',
        ],
        'MOP' => [
            'MOP',
            'Pataka ya Macau',
        ],
        'MVR' => [
            'MVR',
            'Rufiyaa ya Maldivi',
        ],
        'MYR' => [
            'MYR',
            'Ringgiti ya Malesia',
        ],
        'NGN' => [
            'NGN',
            'Naira ya Naijeria',
        ],
        'NOK' => [
            'NOK',
            'Kroni ya Norwe',
        ],
        'NPR' => [
            'NPR',
            'Rupia ya Nepali',
        ],
        'OMR' => [
            'OMR',
            'Riali ya Omani',
        ],
        'PKR' => [
            'PKR',
            'Rupia ya Pakistani',
        ],
        'PLN' => [
            'PLN',
            'Zloti ya Polandi',
        ],
        'QAR' => [
            'QAR',
            'Riali ya Katari',
        ],
        'RSD' => [
            'RSD',
            'Dinari ya Serbia',
        ],
        'SAR' => [
            'SAR',
            'Riyali ya Saudia',
        ],
        'SBD' => [
            'SBD',
            'Dola ya Visiwa vya Solomoni',
        ],
        'SDG' => [
            'SDG',
            'Pauni ya Sudani',
        ],
        'SGD' => [
            'SGD',
            'Dola ya Singapoo',
        ],
        'SLE' => [
            'SLE',
            'Leoni ya Siera Leoni',
        ],
        'SLL' => [
            'SLL',
            'Leoni ya Siera Leoni (1964—2022)',
        ],
        'SSP' => [
            'SSP',
            'Pauni ya Sudani Kusini',
        ],
        'THB' => [
            '฿',
            'Bahti ya Tailandi',
        ],
        'TJS' => [
            'TJS',
            'Somoni ya Tajikistani',
        ],
        'TMT' => [
            'TMT',
            'Manati ya Turkmenistani',
        ],
        'TTD' => [
            'TTD',
            'Dola ya Trinidadi na Tobago',
        ],
        'TWD' => [
            'NT$',
            'Dola ya Taiwani',
        ],
        'UAH' => [
            'UAH',
            'Hryvnia ya Ukraini',
        ],
        'USD' => [
            '$',
            'Dola ya Marekani',
        ],
        'UZS' => [
            'UZS',
            'Som ya Uzbekistani',
        ],
        'VES' => [
            'VES',
            'Boliva ya Venezuela',
        ],
        'VND' => [
            '₫',
            'Dong ya Vietnamu',
        ],
        'XAF' => [
            'FCFA',
            'Faranga ya CFA ya Afrika ya Kati',
        ],
        'XOF' => [
            'F CFA',
            'Faranga ya CFA ya Afrika Magharibi',
        ],
        'YER' => [
            'YER',
            'Riali ya Yemeni',
        ],
    ],
];
