{% extends '@WebProfiler/Profiler/layout.html.twig' %}

{% block toolbar %}
    {% set icon %}
        {{ source('@WebProfiler/Icon/ajax.svg') }}
        <span class="sf-toolbar-value sf-toolbar-ajax-request-counter">0</span>
    {% endset %}

    {% set text %}
        <div class="sf-toolbar-info-piece">
            <span class="sf-toolbar-header">
                <b class="sf-toolbar-ajax-info"></b>
                <b class="sf-toolbar-action">(<a class="sf-toolbar-ajax-clear" href="javascript:void(0);">Clear</a>)</b>
            </span>
        </div>
        <div class="sf-toolbar-info-piece">
            <table class="sf-toolbar-ajax-requests">
                <thead>
                    <tr>
                        <th>#</th>
                        <th>Profile</th>
                        <th>Method</th>
                        <th>Type</th>
                        <th>Status</th>
                        <th>URL</th>
                        <th>Time</th>
                    </tr>
                </thead>
                <tbody class="sf-toolbar-ajax-request-list"></tbody>
            </table>
        </div>
    {% endset %}

    {{ include('@WebProfiler/Profiler/toolbar_item.html.twig', {link: false}) }}
{% endblock %}
