<?php

use Symfony\Component\DependencyInjection\Argument\RewindableGenerator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\Container;
use Symfony\Component\DependencyInjection\Exception\LogicException;
use Symfony\Component\DependencyInjection\Exception\ParameterNotFoundException;
use Symfony\Component\DependencyInjection\Exception\RuntimeException;
use Symfony\Component\DependencyInjection\ParameterBag\FrozenParameterBag;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;

/**
 * @internal This class has been auto-generated by the Symfony Dependency Injection Component.
 */
class Symfony_DI_PhpDumper_Test_AutowireCallable_With_ServiceLocator extends Container
{
    protected $parameters = [];
    protected \Closure $getService;

    public function __construct()
    {
        $this->services = $this->privates = [];
        $this->methodMap = [
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\ListenerResolver' => 'getListenerResolverService',
        ];

        $this->aliases = [];
    }

    public function compile(): void
    {
        throw new LogicException('You cannot compile a dumped container that was already compiled.');
    }

    public function isCompiled(): bool
    {
        return true;
    }

    public function getRemovedIds(): array
    {
        return [
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener1' => true,
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener2' => true,
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\MyInlineService' => true,
            'stdClass' => true,
        ];
    }

    /**
     * Gets the public 'Symfony\Component\DependencyInjection\Tests\Compiler\ListenerResolver' shared service.
     *
     * @return \Symfony\Component\DependencyInjection\Tests\Compiler\ListenerResolver
     */
    protected static function getListenerResolverService($container)
    {
        return $container->services['Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\ListenerResolver'] = new \Symfony\Component\DependencyInjection\Tests\Compiler\ListenerResolver(new \Symfony\Component\DependencyInjection\Argument\ServiceLocator($container->getService ??= $container->getService(...), [
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener1' => ['privates', 'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener1', 'getListener1Service', false],
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener2' => ['privates', 'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener2', 'getListener2Service', false],
        ], [
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener1' => 'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener1',
            'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener2' => 'Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener2',
        ]));
    }

    /**
     * Gets the private 'Symfony\Component\DependencyInjection\Tests\Compiler\Listener1' shared autowired service.
     *
     * @return \Symfony\Component\DependencyInjection\Tests\Compiler\Listener1
     */
    protected static function getListener1Service($container)
    {
        return $container->privates['Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener1'] = new \Symfony\Component\DependencyInjection\Tests\Compiler\Listener1(($container->privates['Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\MyInlineService'] ??= new \Symfony\Component\DependencyInjection\Tests\Compiler\MyInlineService())->someMethod1(...));
    }

    /**
     * Gets the private 'Symfony\Component\DependencyInjection\Tests\Compiler\Listener2' shared autowired service.
     *
     * @return \Symfony\Component\DependencyInjection\Tests\Compiler\Listener2
     */
    protected static function getListener2Service($container)
    {
        return $container->privates['Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\Listener2'] = new \Symfony\Component\DependencyInjection\Tests\Compiler\Listener2(($container->privates['Symfony\\Component\\DependencyInjection\\Tests\\Compiler\\MyInlineService'] ??= new \Symfony\Component\DependencyInjection\Tests\Compiler\MyInlineService())->someMethod2(...), new \stdClass());
    }
}
