<?php

// This file has been auto-generated by the Symfony Serializer Component.

return [
    'Symfony\Component\Serializer\Tests\Fixtures\Dummy' => [
        [
            'foo' => [[], null, null],
            'bar' => [[], null, null],
            'baz' => [[], null, null],
            'qux' => [[], null, null],
        ],
        null,
    ],
];
