<?php

return [
    'UserAssignedNames' => [
        'XK' => 'Kosova',
    ],
    'Names' => [
        'AD' => 'Andorra',
        'AE' => 'Mîrgehên Erebî yên Yekbûyî',
        'AF' => 'Efxanistan',
        'AG' => 'Antîgua û Berbûda',
        'AI' => 'Anguîla',
        'AL' => 'Albanya',
        'AM' => 'Ermenistan',
        'AO' => 'Angola',
        'AQ' => 'Antarktîka',
        'AR' => 'Arjantîn',
        'AS' => 'Samoaya Amerîkanî',
        'AT' => 'Awistirya',
        'AU' => 'Awistralya',
        'AW' => 'Arûba',
        'AX' => 'Giravên Alandê',
        'AZ' => 'Azerbeycan',
        'BA' => 'Bosna û Hersek',
        'BB' => 'Barbados',
        'BD' => 'Bengladeş',
        'BE' => 'Belçîka',
        'BF' => 'Burkîna Faso',
        'BG' => 'Bulgaristan',
        'BH' => 'Behreyn',
        'BI' => 'Bûrûndî',
        'BJ' => 'Bênîn',
        'BL' => 'Saint Barthelemy',
        'BM' => 'Bermûda',
        'BN' => 'Brûney',
        'BO' => 'Bolîvya',
        'BQ' => 'Holendaya Karayîbê',
        'BR' => 'Brezîlya',
        'BS' => 'Bahama',
        'BT' => 'Bûtan',
        'BV' => 'Girava Bouvetê',
        'BW' => 'Botswana',
        'BY' => 'Belarûs',
        'BZ' => 'Belîze',
        'CA' => 'Kanada',
        'CC' => 'Giravên Kokosê (Keeling)',
        'CD' => 'Kongo - Kînşasa',
        'CF' => 'Cimhûrîyeta Afrîkaya Navîn',
        'CG' => 'Kongo - Brazzaville',
        'CH' => 'Swîsre',
        'CI' => 'Côte d’Ivoire',
        'CK' => 'Giravên Cookê',
        'CL' => 'Şîlî',
        'CM' => 'Kamerûn',
        'CN' => 'Çîn',
        'CO' => 'Kolombîya',
        'CR' => 'Kosta Rîka',
        'CU' => 'Kuba',
        'CV' => 'Kap Verde',
        'CW' => 'Curaçao',
        'CX' => 'Girava Christmasê',
        'CY' => 'Qibris',
        'CZ' => 'Çekya',
        'DE' => 'Almanya',
        'DJ' => 'Cîbûtî',
        'DK' => 'Danîmarka',
        'DM' => 'Domînîka',
        'DO' => 'Cimhûrîyeta Domînîkê',
        'DZ' => 'Cezayîr',
        'EC' => 'Ekwador',
        'EE' => 'Estonya',
        'EG' => 'Misir',
        'EH' => 'Sahraya Rojava',
        'ER' => 'Erître',
        'ES' => 'Spanya',
        'ET' => 'Etîyopya',
        'FI' => 'Fînlenda',
        'FJ' => 'Fîjî',
        'FK' => 'Giravên Falklandê',
        'FM' => 'Mîkronezya',
        'FO' => 'Giravên Faroeyê',
        'FR' => 'Fransa',
        'GA' => 'Gabon',
        'GB' => 'Qiralîyeta Yekbûyî',
        'GD' => 'Grenada',
        'GE' => 'Gurcistan',
        'GF' => 'Guyanaya Fransî',
        'GG' => 'Guernsey',
        'GH' => 'Gana',
        'GI' => 'Cebelîtariq',
        'GL' => 'Grînlanda',
        'GM' => 'Gambîya',
        'GN' => 'Gîne',
        'GP' => 'Guadeloupe',
        'GQ' => 'Gîneya Ekwadorê',
        'GR' => 'Yûnanistan',
        'GS' => 'Giravên Georgîyaya Başûr û Sandwicha Başûr',
        'GT' => 'Guatemala',
        'GU' => 'Guam',
        'GW' => 'Gîne-Bissau',
        'GY' => 'Guyana',
        'HK' => 'Hong Konga HîT ya Çînê',
        'HM' => 'Giravên Heard û MacDonaldê',
        'HN' => 'Hondûras',
        'HR' => 'Xirwatistan',
        'HT' => 'Haîtî',
        'HU' => 'Macaristan',
        'ID' => 'Endonezya',
        'IE' => 'Îrlanda',
        'IL' => 'Îsraîl',
        'IM' => 'Girava Manê',
        'IN' => 'Hindistan',
        'IO' => 'Herêma Okyanûsa Hindî ya Brîtanyayê',
        'IQ' => 'Îraq',
        'IR' => 'Îran',
        'IS' => 'Îslanda',
        'IT' => 'Îtalya',
        'JE' => 'Jersey',
        'JM' => 'Jamaîka',
        'JO' => 'Urdun',
        'JP' => 'Japonya',
        'KE' => 'Kenya',
        'KG' => 'Qirgizistan',
        'KH' => 'Kamboçya',
        'KI' => 'Kirîbatî',
        'KM' => 'Komor',
        'KN' => 'Saint Kitts û Nevîs',
        'KP' => 'Koreya Bakur',
        'KR' => 'Koreya Başûr',
        'KW' => 'Kuweyt',
        'KY' => 'Giravên Kaymanê',
        'KZ' => 'Qazaxistan',
        'LA' => 'Laos',
        'LB' => 'Libnan',
        'LC' => 'Saint Lucia',
        'LI' => 'Liechtenstein',
        'LK' => 'Srî Lanka',
        'LR' => 'Lîberya',
        'LS' => 'Lesoto',
        'LT' => 'Lîtvanya',
        'LU' => 'Luksembûrg',
        'LV' => 'Letonya',
        'LY' => 'Lîbya',
        'MA' => 'Fas',
        'MC' => 'Monako',
        'MD' => 'Moldova',
        'ME' => 'Qeredax',
        'MF' => 'Saint Martin',
        'MG' => 'Madagaskar',
        'MH' => 'Giravên Marşalê',
        'MK' => 'Makedonyaya Bakur',
        'ML' => 'Malî',
        'MM' => 'Myanmar (Bûrma)',
        'MN' => 'Moxolistan',
        'MO' => 'Makaoya Hît ya Çînê',
        'MP' => 'Giravên Bakurê Marianan',
        'MQ' => 'Martînîk',
        'MR' => 'Morîtanya',
        'MS' => 'Montserat',
        'MT' => 'Malta',
        'MU' => 'Mauritius',
        'MV' => 'Maldîv',
        'MW' => 'Malawî',
        'MX' => 'Meksîka',
        'MY' => 'Malezya',
        'MZ' => 'Mozambîk',
        'NA' => 'Namîbya',
        'NC' => 'Kaledonyaya Nû',
        'NE' => 'Nîjer',
        'NF' => 'Girava Norfolkê',
        'NG' => 'Nîjerya',
        'NI' => 'Nîkaragua',
        'NL' => 'Holanda',
        'NO' => 'Norwec',
        'NP' => 'Nepal',
        'NR' => 'Naûrû',
        'NU' => 'Niûe',
        'NZ' => 'Zelandaya Nû',
        'OM' => 'Uman',
        'PA' => 'Panama',
        'PE' => 'Perû',
        'PF' => 'Polînezyaya Fransizî',
        'PG' => 'Papua Gîneya Nû',
        'PH' => 'Fîlîpîn',
        'PK' => 'Pakistan',
        'PL' => 'Polonya',
        'PM' => 'Saint-Pierre û Miquelon',
        'PN' => 'Giravên Pitcairnê',
        'PR' => 'Porto Rîko',
        'PS' => 'Herêmên Filistînî',
        'PT' => 'Portugal',
        'PW' => 'Palau',
        'PY' => 'Paragûay',
        'QA' => 'Qeter',
        'RE' => 'Réunion',
        'RO' => 'Romanya',
        'RS' => 'Sirbistan',
        'RU' => 'Rûsya',
        'RW' => 'Rwanda',
        'SA' => 'Erebistana Siûdî',
        'SB' => 'Giravên Solomonê',
        'SC' => 'Seyşel',
        'SD' => 'Sûdan',
        'SE' => 'Swêd',
        'SG' => 'Sîngapûr',
        'SH' => 'Saint Helena',
        'SI' => 'Slovenya',
        'SJ' => 'Svalbard û Jan Mayen',
        'SK' => 'Slovakya',
        'SL' => 'Sierra Leone',
        'SM' => 'San Marîno',
        'SN' => 'Senegal',
        'SO' => 'Somalya',
        'SR' => 'Surînam',
        'SS' => 'Sûdana Başûr',
        'ST' => 'Sao Tome û Prînsîpe',
        'SV' => 'El Salvador',
        'SX' => 'Sint Marteen',
        'SY' => 'Sûrîye',
        'SZ' => 'Eswatînî',
        'TC' => 'Giravên Turks û Kaîkosê',
        'TD' => 'Çad',
        'TF' => 'Herêmên Başûr ên Fransayê',
        'TG' => 'Togo',
        'TH' => 'Tayland',
        'TJ' => 'Tacikistan',
        'TK' => 'Tokelau',
        'TL' => 'Tîmor-Leste',
        'TM' => 'Tirkmenistan',
        'TN' => 'Tûnis',
        'TO' => 'Tonga',
        'TR' => 'Türkiye',
        'TT' => 'Trînîdad û Tobago',
        'TV' => 'Tûvalû',
        'TW' => 'Taywan',
        'TZ' => 'Tanzanya',
        'UA' => 'Ûkrayna',
        'UG' => 'Ûganda',
        'UM' => 'Giravên Biçûk ên Derveyî DAYê',
        'US' => 'Dewletên Amerîkayê yên Yekbûyî',
        'UY' => 'Ûrûguay',
        'UZ' => 'Ozbekistan',
        'VA' => 'Vatîkan',
        'VC' => 'Saint Vincent û Giravên Grenadînê',
        'VE' => 'Venezuela',
        'VG' => 'Giravên Vîrjînê yên Brîtanyayê',
        'VI' => 'Giravên Vîrjînê yên Amerîkayê',
        'VN' => 'Vîetnam',
        'VU' => 'Vanûatû',
        'WF' => 'Wallis û Futuna',
        'WS' => 'Samoa',
        'YE' => 'Yemen',
        'YT' => 'Mayotte',
        'ZA' => 'Afrîkaya Başûr',
        'ZM' => 'Zambîya',
        'ZW' => 'Zîmbabwe',
    ],
];
