<?php

return [
    'Names' => [
        'Adlm' => 'Adlam kasa',
        'Arab' => 'Arabeke',
        'Aran' => 'Nastaliki kasa',
        'Armn' => 'Amenia kasa',
        'Beng' => 'Bangala kasa',
        'Bopo' => 'Bopomofo kasa',
        'Brai' => 'Anifrafoɔ kasa',
        'Cakm' => 'Kyakma kasa',
        'Cans' => 'Kanadafoɔ Kann Kasa a Wɔakeka Abom',
        'Cher' => 'Kɛroki',
        'Cyrl' => 'Kreleke',
        'Deva' => 'Dɛvanagari kasa',
        'Ethi' => 'Yitiopia kasa',
        'Geor' => 'Dwɔɔgyia kasa',
        'Grek' => 'Griiki kasa',
        'Gujr' => 'Gudwurati kasa',
        'Guru' => 'Gurumuki kasa',
        'Hanb' => 'Hanse a Bopomofo kasa ka ho',
        'Hang' => 'Hangul kasa',
        'Hani' => 'Han',
        'Hans' => 'Kyaena Kasa Hanse',
        'Hant' => 'Tete',
        'Hebr' => 'Hibri kasa',
        'Hira' => 'Hiragana kasa',
        'Hrkt' => 'Gyapanfoɔ selabolo kasa',
        'Jamo' => 'Gyamo kasa',
        'Jpan' => 'Gyapanfoɔ kasa',
        'Kana' => 'Katakana kasa',
        'Khmr' => 'Kɛma kasa',
        'Knda' => 'Kanada kasa',
        'Kore' => 'Korea kasa',
        'Laoo' => 'Lawo kasa',
        'Latn' => 'Laatin',
        'Mlym' => 'Malayalam kasa',
        'Mong' => 'Mongoliafoɔ kasa',
        'Mtei' => 'Meeti Mayɛke kasa',
        'Mymr' => 'Mayama kasa',
        'Nkoo' => 'Nko kasa',
        'Olck' => 'Ol Kyiki kasa',
        'Orya' => 'Odia kasa',
        'Rohg' => 'Hanifi kasa',
        'Sinh' => 'Sinhala kasa',
        'Sund' => 'Sudanni kasa',
        'Syrc' => 'Siiria Tete kasa',
        'Taml' => 'Tamil kasa',
        'Telu' => 'Telugu kasa',
        'Tfng' => 'Tifinafo kasa',
        'Thaa' => 'Taana kasa',
        'Thai' => 'Taelanfoɔ kasa',
        'Tibt' => 'Tibɛtanfoɔ kasa',
        'Vaii' => 'Vai kasa',
        'Yiii' => 'Yifoɔ kasa',
        'Zmth' => 'Nkontabudeɛ',
        'Zsye' => 'Yimogyi',
        'Zsym' => 'Ahyɛnsodeɛ',
        'Zxxx' => 'Deɛ wɔntwerɛeɛ',
        'Zyyy' => 'obiara nim',
    ],
];
