<?php

return [
    'Names' => [
        'Arab' => 'arabe',
        'Armn' => 'armenie',
        'Beng' => 'bengalese',
        'Bopo' => 'bopomofo',
        'Brai' => 'braille',
        'Cyrl' => 'cyrillic',
        'Deva' => 'devanagari',
        'Ethi' => 'ethiope',
        'Geor' => 'georgian',
        'Grek' => 'grec',
        'Gujr' => 'gujarati',
        'Guru' => 'gurmukhi',
        'Hanb' => 'han con bopomofo',
        'Hang' => 'hangul',
        'Hani' => 'han',
        'Hans' => 'simplificate',
        'Hant' => 'traditional',
        'Hebr' => 'hebraic',
        'Hira' => 'hiragana',
        'Hrkt' => 'syllabarios japonese',
        'Jamo' => 'jamo',
        'Jpan' => 'japonese',
        'Kana' => 'katakana',
        'Khmr' => 'khmer',
        'Knda' => 'kannada',
        'Kore' => 'corean',
        'Laoo' => 'lao',
        'Latn' => 'latin',
        'Mlym' => 'malayalam',
        'Mong' => 'mongol',
        'Mymr' => 'birman',
        'Orya' => 'orya',
        'Sinh' => 'cingalese',
        'Taml' => 'tamil',
        'Telu' => 'telugu',
        'Thaa' => 'thaana',
        'Thai' => 'thailandese',
        'Tibt' => 'tibetan',
        'Zinh' => 'hereditate',
        'Zmth' => 'notation mathematic',
        'Zsye' => 'emoji',
        'Zsym' => 'symbolos',
        'Zxxx' => 'non scripte',
        'Zyyy' => 'commun',
    ],
];
