<?php

return [
    'Names' => [
        'Adlm' => 'اډلام',
        'Arab' => 'عربي',
        'Aran' => 'نستعلیق',
        'Armn' => 'ارمانیایي',
        'Beng' => 'بنګله',
        'Bopo' => 'بوپوموفو',
        'Brai' => 'بریلي',
        'Cakm' => 'چکما',
        'Cans' => 'متحد کاناډایی ابوریجینل سلیبکس',
        'Cher' => 'چیروکي',
        'Cyrl' => 'سیریلیک',
        'Deva' => 'دیواناګري',
        'Ethi' => 'ایتوپي',
        'Geor' => 'ګرجستاني',
        'Grek' => 'یوناني',
        'Gujr' => 'ګجراتي',
        'Guru' => 'ګرومي',
        'Hanb' => 'هن او بوپوفومو',
        'Hang' => 'هنګولي',
        'Hani' => 'هن',
        'Hans' => 'ساده شوی',
        'Hant' => 'دودیزه',
        'Hebr' => 'عبراني',
        'Hira' => 'هیراګانا',
        'Hrkt' => 'د جاپاني سیلابري',
        'Jamo' => 'جامو',
        'Jpan' => 'جاپاني',
        'Kana' => 'کاتاکانا',
        'Khmr' => 'خمر',
        'Knda' => 'کناډا',
        'Kore' => 'کوریایی',
        'Laoo' => 'لاوو',
        'Latn' => 'لاتين/لاتيني',
        'Mlym' => 'مالایالم',
        'Mong' => 'منګولیایي',
        'Mtei' => 'میټي مایک',
        'Mymr' => 'میانمار',
        'Nkoo' => 'ناکو',
        'Olck' => 'اول چکی۔',
        'Orya' => 'اوديا',
        'Rohg' => 'حنفی',
        'Sinh' => 'سنهالا',
        'Sund' => 'سنڈانی',
        'Syrc' => 'سریانی',
        'Taml' => 'تامل',
        'Telu' => 'تیلیګو',
        'Tfng' => 'ٹیفناګ',
        'Thaa' => 'تهانا',
        'Thai' => 'تایلنډي',
        'Tibt' => 'تبتي',
        'Vaii' => 'وای',
        'Yiii' => 'یی',
        'Zmth' => 'د ریاضیاتو نوټیشن',
        'Zsye' => 'ایموجي',
        'Zsym' => 'سمبولونه',
        'Zxxx' => 'ناليکلی',
        'Zyyy' => 'عام',
    ],
];
