<?php

return [
    'Names' => [
        'Africa/Abidjan' => 'Greenwiški srednji čas (Abidžan)',
        'Africa/Accra' => 'Greenwiški srednji čas (Akra)',
        'Africa/Addis_Ababa' => 'Vzhodnoafriški čas (Adis Abeba)',
        'Africa/Algiers' => 'Srednjeevropski čas (Alžir)',
        'Africa/Asmera' => 'Vzhodnoafriški čas (Asmara)',
        'Africa/Asmara' => 'Vzhodnoafriški čas (Asmara)',
        'Africa/Bamako' => 'Greenwiški srednji čas (Bamako)',
        'Africa/Bangui' => 'Zahodnoafriški čas (Bangui)',
        'Africa/Banjul' => 'Greenwiški srednji čas (Banjul)',
        'Africa/Bissau' => 'Greenwiški srednji čas (Bissau)',
        'Africa/Blantyre' => 'Centralnoafriški čas (Blantyre)',
        'Africa/Brazzaville' => 'Zahodnoafriški čas (Brazzaville)',
        'Africa/Bujumbura' => 'Centralnoafriški čas (Bujumbura)',
        'Africa/Cairo' => 'Vzhodnoevropski čas (Kairo)',
        'Africa/Casablanca' => 'Zahodnoevropski čas (Casablanca)',
        'Africa/Ceuta' => 'Srednjeevropski čas (Ceuta)',
        'Africa/Conakry' => 'Greenwiški srednji čas (Conakry)',
        'Africa/Dakar' => 'Greenwiški srednji čas (Dakar)',
        'Africa/Dar_es_Salaam' => 'Vzhodnoafriški čas (Dar es Salaam)',
        'Africa/Djibouti' => 'Vzhodnoafriški čas (Džibuti)',
        'Africa/Douala' => 'Zahodnoafriški čas (Douala)',
        'Africa/El_Aaiun' => 'Zahodnoevropski čas (El Aaiún)',
        'Africa/Freetown' => 'Greenwiški srednji čas (Freetown)',
        'Africa/Gaborone' => 'Centralnoafriški čas (Gaborone)',
        'Africa/Harare' => 'Centralnoafriški čas (Harare)',
        'Africa/Johannesburg' => 'Južnoafriški čas (Johannesburg)',
        'Africa/Juba' => 'Centralnoafriški čas (Juba)',
        'Africa/Kampala' => 'Vzhodnoafriški čas (Kampala)',
        'Africa/Khartoum' => 'Centralnoafriški čas (Kartum)',
        'Africa/Kigali' => 'Centralnoafriški čas (Kigali)',
        'Africa/Kinshasa' => 'Zahodnoafriški čas (Kinšasa)',
        'Africa/Lagos' => 'Zahodnoafriški čas (Lagos)',
        'Africa/Libreville' => 'Zahodnoafriški čas (Libreville)',
        'Africa/Lome' => 'Greenwiški srednji čas (Lomé)',
        'Africa/Luanda' => 'Zahodnoafriški čas (Luanda)',
        'Africa/Lubumbashi' => 'Centralnoafriški čas (Lubumbaši)',
        'Africa/Lusaka' => 'Centralnoafriški čas (Lusaka)',
        'Africa/Malabo' => 'Zahodnoafriški čas (Malabo)',
        'Africa/Maputo' => 'Centralnoafriški čas (Maputo)',
        'Africa/Maseru' => 'Južnoafriški čas (Maseru)',
        'Africa/Mbabane' => 'Južnoafriški čas (Mbabane)',
        'Africa/Mogadishu' => 'Vzhodnoafriški čas (Mogadišu)',
        'Africa/Monrovia' => 'Greenwiški srednji čas (Monrovia)',
        'Africa/Nairobi' => 'Vzhodnoafriški čas (Nairobi)',
        'Africa/Ndjamena' => 'Zahodnoafriški čas (N’Djamena)',
        'Africa/Niamey' => 'Zahodnoafriški čas (Niamey)',
        'Africa/Nouakchott' => 'Greenwiški srednji čas (Nouakchott)',
        'Africa/Ouagadougou' => 'Greenwiški srednji čas (Ouagadougou)',
        'Africa/Porto-Novo' => 'Zahodnoafriški čas (Porto-Novo)',
        'Africa/Sao_Tome' => 'Greenwiški srednji čas (São Tomé)',
        'Africa/Tripoli' => 'Vzhodnoevropski čas (Tripoli)',
        'Africa/Tunis' => 'Srednjeevropski čas (Tunis)',
        'Africa/Windhoek' => 'Centralnoafriški čas (Windhoek)',
        'America/Adak' => 'Havajski aleutski čas (Adak)',
        'America/Anchorage' => 'Aljaški čas (Anchorage)',
        'America/Anguilla' => 'Atlantski čas (Angvila)',
        'America/Antigua' => 'Atlantski čas (Antigua)',
        'America/Araguaina' => 'Brasilski čas (Araguaína)',
        'America/Argentina/La_Rioja' => 'Argentinski čas (La Rioja)',
        'America/Argentina/Rio_Gallegos' => 'Argentinski čas (Río Gallegos)',
        'America/Argentina/Salta' => 'Argentinski čas (Salta)',
        'America/Argentina/San_Juan' => 'Argentinski čas (San Juan)',
        'America/Argentina/San_Luis' => 'Argentinski čas (San Luis)',
        'America/Argentina/Tucuman' => 'Argentinski čas (Tucumán)',
        'America/Argentina/Ushuaia' => 'Argentinski čas (Ushuaia)',
        'America/Aruba' => 'Atlantski čas (Aruba)',
        'America/Asuncion' => 'Paragvajski čas (Asunción)',
        'America/Bahia' => 'Brasilski čas (Bahia)',
        'America/Bahia_Banderas' => 'Centralni čas (Bahia de Banderas)',
        'America/Barbados' => 'Atlantski čas (Barbados)',
        'America/Belem' => 'Brasilski čas (Belém)',
        'America/Belize' => 'Centralni čas (Belize)',
        'America/Blanc-Sablon' => 'Atlantski čas (Blanc-Sablon)',
        'America/Boa_Vista' => 'Amazonski čas (Boa Vista)',
        'America/Bogota' => 'Kolumbijski čas (Bogotá)',
        'America/Boise' => 'Gorski čas (Boise)',
        'America/Buenos_Aires' => 'Argentinski čas (Buenos Aires)',
        'America/Argentina/Buenos_Aires' => 'Argentinski čas (Buenos Aires)',
        'America/Cambridge_Bay' => 'Gorski čas (Cambridge Bay)',
        'America/Campo_Grande' => 'Amazonski čas (Campo Grande)',
        'America/Cancun' => 'Vzhodni čas (Cancun)',
        'America/Caracas' => 'Venezuelski čas (Caracas)',
        'America/Catamarca' => 'Argentinski čas (Catamarca)',
        'America/Argentina/Catamarca' => 'Argentinski čas (Catamarca)',
        'America/Cayenne' => 'Čas: Francoska Gvajana (Cayenne)',
        'America/Cayman' => 'Vzhodni čas (Kajman)',
        'America/Chicago' => 'Centralni čas (Chicago)',
        'America/Chihuahua' => 'Centralni čas (Chihuahua)',
        'America/Ciudad_Juarez' => 'Gorski čas (Ciudad Juárez)',
        'America/Coral_Harbour' => 'Vzhodni čas (Atikokan)',
        'America/Atikokan' => 'Vzhodni čas (Atikokan)',
        'America/Cordoba' => 'Argentinski čas (Córdoba)',
        'America/Argentina/Cordoba' => 'Argentinski čas (Córdoba)',
        'America/Costa_Rica' => 'Centralni čas (Kostarika)',
        'America/Coyhaique' => 'Čilski čas (Coyhaique)',
        'America/Creston' => 'Gorski čas (Creston)',
        'America/Cuiaba' => 'Amazonski čas (Cuiabá)',
        'America/Curacao' => 'Atlantski čas (Curaçao)',
        'America/Danmarkshavn' => 'Greenwiški srednji čas (Danmarkshavn)',
        'America/Dawson' => 'Jukonski čas (Dawson)',
        'America/Dawson_Creek' => 'Gorski čas (Dawson Creek)',
        'America/Denver' => 'Gorski čas (Denver)',
        'America/Detroit' => 'Vzhodni čas (Detroit)',
        'America/Dominica' => 'Atlantski čas (Dominika)',
        'America/Edmonton' => 'Gorski čas (Edmonton)',
        'America/Eirunepe' => 'Brazilija (lokalni čas) (Eirunepé)',
        'America/El_Salvador' => 'Centralni čas (Salvador)',
        'America/Fort_Nelson' => 'Gorski čas (Fort Nelson)',
        'America/Fortaleza' => 'Brasilski čas (Fortaleza)',
        'America/Glace_Bay' => 'Atlantski čas (Glace Bay)',
        'America/Godthab' => 'Grenlandija (lokalni čas) (Nuuk)',
        'America/Nuuk' => 'Grenlandija (lokalni čas) (Nuuk)',
        'America/Goose_Bay' => 'Atlantski čas (Goose Bay)',
        'America/Grand_Turk' => 'Vzhodni čas (Grand Turk)',
        'America/Grenada' => 'Atlantski čas (Grenada)',
        'America/Guadeloupe' => 'Atlantski čas (Guadeloupe)',
        'America/Guatemala' => 'Centralni čas (Gvatemala)',
        'America/Guayaquil' => 'Ekvadorski čas (Guayaquil)',
        'America/Guyana' => 'Gvajanski čas (Gvajana)',
        'America/Halifax' => 'Atlantski čas (Halifax)',
        'America/Havana' => 'Kubanski čas (Havana)',
        'America/Hermosillo' => 'Mehiški pacifiški čas (Hermosillo)',
        'America/Indiana/Knox' => 'Centralni čas (Knox, Indiana)',
        'America/Indiana/Marengo' => 'Vzhodni čas (Marengo, Indiana)',
        'America/Indiana/Petersburg' => 'Vzhodni čas (Petersburg, Indiana)',
        'America/Indiana/Tell_City' => 'Centralni čas (Tell City, Indiana)',
        'America/Indiana/Vevay' => 'Vzhodni čas (Vevay, Indiana)',
        'America/Indiana/Vincennes' => 'Vzhodni čas (Vincennes, Indiana)',
        'America/Indiana/Winamac' => 'Vzhodni čas (Winamac, Indiana)',
        'America/Indianapolis' => 'Vzhodni čas (Indianapolis)',
        'America/Indiana/Indianapolis' => 'Vzhodni čas (Indianapolis)',
        'America/Inuvik' => 'Gorski čas (Inuvik)',
        'America/Iqaluit' => 'Vzhodni čas (Iqaluit)',
        'America/Jamaica' => 'Vzhodni čas (Jamajka)',
        'America/Jujuy' => 'Argentinski čas (Jujuy)',
        'America/Argentina/Jujuy' => 'Argentinski čas (Jujuy)',
        'America/Juneau' => 'Aljaški čas (Juneau)',
        'America/Kentucky/Monticello' => 'Vzhodni čas (Monticello, Kentucky)',
        'America/Kralendijk' => 'Atlantski čas (Kralendijk)',
        'America/La_Paz' => 'Bolivijski čas (La Paz)',
        'America/Lima' => 'Perujski čas (Lima)',
        'America/Los_Angeles' => 'Pacifiški čas (Los Angeles)',
        'America/Louisville' => 'Vzhodni čas (Louisville)',
        'America/Kentucky/Louisville' => 'Vzhodni čas (Louisville)',
        'America/Lower_Princes' => 'Atlantski čas (Lower Prince’s Quarter)',
        'America/Maceio' => 'Brasilski čas (Maceió)',
        'America/Managua' => 'Centralni čas (Managua)',
        'America/Manaus' => 'Amazonski čas (Manaus)',
        'America/Marigot' => 'Atlantski čas (Marigot)',
        'America/Martinique' => 'Atlantski čas (Martinik)',
        'America/Matamoros' => 'Centralni čas (Matamoros)',
        'America/Mazatlan' => 'Mehiški pacifiški čas (Mazatlán)',
        'America/Mendoza' => 'Argentinski čas (Mendoza)',
        'America/Argentina/Mendoza' => 'Argentinski čas (Mendoza)',
        'America/Menominee' => 'Centralni čas (Menominee)',
        'America/Merida' => 'Centralni čas (Merida)',
        'America/Metlakatla' => 'Aljaški čas (Metlakatla)',
        'America/Mexico_City' => 'Centralni čas (Ciudad de Mexico)',
        'America/Miquelon' => 'Čas: Saint Pierre in Miquelon',
        'America/Moncton' => 'Atlantski čas (Moncton)',
        'America/Monterrey' => 'Centralni čas (Monterrey)',
        'America/Montevideo' => 'Urugvajski čas (Montevideo)',
        'America/Montserrat' => 'Atlantski čas (Montserrat)',
        'America/Nassau' => 'Vzhodni čas (Nassau)',
        'America/New_York' => 'Vzhodni čas (New York)',
        'America/Nome' => 'Aljaški čas (Nome)',
        'America/Noronha' => 'Fernando de Noronški čas (Fernando de Noronha)',
        'America/North_Dakota/Beulah' => 'Centralni čas (Beulah, Severna Dakota)',
        'America/North_Dakota/Center' => 'Centralni čas (Center, Severna Dakota)',
        'America/North_Dakota/New_Salem' => 'Centralni čas (New Salem, Severna Dakota)',
        'America/Ojinaga' => 'Centralni čas (Ojinaga)',
        'America/Panama' => 'Vzhodni čas (Panama)',
        'America/Paramaribo' => 'Surinamski čas (Paramaribo)',
        'America/Phoenix' => 'Gorski čas (Phoenix)',
        'America/Port-au-Prince' => 'Vzhodni čas (Port-au-Prince)',
        'America/Port_of_Spain' => 'Atlantski čas (Port of Spain)',
        'America/Porto_Velho' => 'Amazonski čas (Porto Velho)',
        'America/Puerto_Rico' => 'Atlantski čas (Portoriko)',
        'America/Punta_Arenas' => 'Čilski čas (Punta Arenas)',
        'America/Rankin_Inlet' => 'Centralni čas (Rankin Inlet)',
        'America/Recife' => 'Brasilski čas (Recife)',
        'America/Regina' => 'Centralni čas (Regina)',
        'America/Resolute' => 'Centralni čas (Resolute)',
        'America/Rio_Branco' => 'Brazilija (lokalni čas) (Rio Branco)',
        'America/Santarem' => 'Brasilski čas (Santarém)',
        'America/Santiago' => 'Čilski čas (Santiago)',
        'America/Santo_Domingo' => 'Atlantski čas (Santo Domingo)',
        'America/Sao_Paulo' => 'Brasilski čas (São Paulo)',
        'America/Scoresbysund' => 'Grenlandija (lokalni čas) (Ittoqqortoormiit)',
        'America/Sitka' => 'Aljaški čas (Sitka)',
        'America/St_Barthelemy' => 'Atlantski čas (Sv. Bartolomej)',
        'America/St_Johns' => 'Novofundlandski čas (St. John’s)',
        'America/St_Kitts' => 'Atlantski čas (Sv. Krištof)',
        'America/St_Lucia' => 'Atlantski čas (Sv. Lucija)',
        'America/St_Thomas' => 'Atlantski čas (Sv. Tomaž)',
        'America/St_Vincent' => 'Atlantski čas (Sv. Vincencij)',
        'America/Swift_Current' => 'Centralni čas (Swift Current)',
        'America/Tegucigalpa' => 'Centralni čas (Tegucigalpa)',
        'America/Thule' => 'Atlantski čas (Thule)',
        'America/Tijuana' => 'Pacifiški čas (Tijuana)',
        'America/Toronto' => 'Vzhodni čas (Toronto)',
        'America/Tortola' => 'Atlantski čas (Tortola)',
        'America/Vancouver' => 'Pacifiški čas (Vancouver)',
        'America/Whitehorse' => 'Jukonski čas (Whitehorse)',
        'America/Winnipeg' => 'Centralni čas (Winnipeg)',
        'America/Yakutat' => 'Aljaški čas (Yakutat)',
        'Antarctica/Casey' => 'Avstralski zahodni čas (Casey)',
        'Antarctica/Davis' => 'Čas: Davis',
        'Antarctica/DumontDUrville' => 'Čas: Dumont-d’Urville',
        'Antarctica/Macquarie' => 'Avstralski vzhodni čas (Macquarie)',
        'Antarctica/Mawson' => 'Mawsonski čas',
        'Antarctica/McMurdo' => 'Novozelandski čas (McMurdo)',
        'Antarctica/Palmer' => 'Čilski čas (Palmer)',
        'Antarctica/Rothera' => 'Rotherski čas (Rothera)',
        'Antarctica/Syowa' => 'Čas: Syowa (Showa)',
        'Antarctica/Troll' => 'Greenwiški srednji čas (Troll)',
        'Antarctica/Vostok' => 'Vostoški čas (Vostok)',
        'Arctic/Longyearbyen' => 'Srednjeevropski čas (Longyearbyen)',
        'Asia/Aden' => 'Arabski čas (Aden)',
        'Asia/Almaty' => 'Kazahstanski čas (Almati)',
        'Asia/Amman' => 'Vzhodnoevropski čas (Aman)',
        'Asia/Anadyr' => 'Petropavlovsk-Kamčatski čas (Anadir)',
        'Asia/Aqtau' => 'Kazahstanski čas (Aktau)',
        'Asia/Aqtobe' => 'Kazahstanski čas (Aktobe)',
        'Asia/Ashgabat' => 'Turkmenistanski čas (Ašhabad)',
        'Asia/Atyrau' => 'Kazahstanski čas (Atyrau)',
        'Asia/Baghdad' => 'Arabski čas (Bagdad)',
        'Asia/Bahrain' => 'Arabski čas (Bahrajn)',
        'Asia/Baku' => 'Azerbajdžanski čas (Baku)',
        'Asia/Bangkok' => 'Indokitajski čas (Bangkok)',
        'Asia/Barnaul' => 'Krasnojarski čas (Barnaul)',
        'Asia/Beirut' => 'Vzhodnoevropski čas (Bejrut)',
        'Asia/Bishkek' => 'Kirgizistanski čas (Biškek)',
        'Asia/Brunei' => 'Brunejski čas',
        'Asia/Calcutta' => 'Indijski standardni čas (Kalkuta)',
        'Asia/Kolkata' => 'Indijski standardni čas (Kalkuta)',
        'Asia/Chita' => 'Jakutski čas (Čita)',
        'Asia/Colombo' => 'Indijski standardni čas (Kolombo)',
        'Asia/Damascus' => 'Vzhodnoevropski čas (Damask)',
        'Asia/Dhaka' => 'Bangladeški čas (Daka)',
        'Asia/Dili' => 'Vzhodnotimorski čas (Dili)',
        'Asia/Dubai' => 'Zalivski standardni čas (Dubaj)',
        'Asia/Dushanbe' => 'Tadžikistanski čas (Dušanbe)',
        'Asia/Famagusta' => 'Vzhodnoevropski čas (Famagusta)',
        'Asia/Gaza' => 'Vzhodnoevropski čas (Gaza)',
        'Asia/Hebron' => 'Vzhodnoevropski čas (Hebron)',
        'Asia/Hong_Kong' => 'Hongkonški čas (Hongkong)',
        'Asia/Hovd' => 'Hovdski čas (Khovd)',
        'Asia/Irkutsk' => 'Irkutski čas',
        'Asia/Jakarta' => 'Indonezijski zahodni čas (Džakarta)',
        'Asia/Jayapura' => 'Indonezijski vzhodni čas (Jayapura)',
        'Asia/Jerusalem' => 'Izraelski čas (Jeruzalem)',
        'Asia/Kabul' => 'Afganistanski čas (Kabul)',
        'Asia/Kamchatka' => 'Petropavlovsk-Kamčatski čas (Kamčatka)',
        'Asia/Karachi' => 'Pakistanski čas (Karači)',
        'Asia/Katmandu' => 'Nepalski čas (Katmandu)',
        'Asia/Kathmandu' => 'Nepalski čas (Katmandu)',
        'Asia/Khandyga' => 'Jakutski čas (Handiga)',
        'Asia/Krasnoyarsk' => 'Krasnojarski čas',
        'Asia/Kuala_Lumpur' => 'Malezijski čas (Kuala Lumpur)',
        'Asia/Kuching' => 'Malezijski čas (Kuching)',
        'Asia/Kuwait' => 'Arabski čas (Kuvajt)',
        'Asia/Macau' => 'Kitajski čas (Macao)',
        'Asia/Magadan' => 'Magadanski čas',
        'Asia/Makassar' => 'Indonezijski osrednji čas (Makasar)',
        'Asia/Manila' => 'Filipinski čas (Manila)',
        'Asia/Muscat' => 'Zalivski standardni čas (Muškat)',
        'Asia/Nicosia' => 'Vzhodnoevropski čas (Nikozija)',
        'Asia/Novokuznetsk' => 'Krasnojarski čas (Novokuznetsk)',
        'Asia/Novosibirsk' => 'Krasnojarski čas (Novosibirsk)',
        'Asia/Omsk' => 'Omski čas',
        'Asia/Oral' => 'Kazahstanski čas (Uralsk)',
        'Asia/Phnom_Penh' => 'Indokitajski čas (Phnom Penh)',
        'Asia/Pontianak' => 'Indonezijski zahodni čas (Pontianak)',
        'Asia/Pyongyang' => 'Korejski čas (Pjongjang)',
        'Asia/Qatar' => 'Arabski čas (Katar)',
        'Asia/Qostanay' => 'Kazahstanski čas (Kostanaj)',
        'Asia/Qyzylorda' => 'Kazahstanski čas (Kizlorda)',
        'Asia/Rangoon' => 'Mjanmarski čas (Rangun)',
        'Asia/Yangon' => 'Mjanmarski čas (Rangun)',
        'Asia/Riyadh' => 'Arabski čas (Rijad)',
        'Asia/Saigon' => 'Indokitajski čas (Hošiminh)',
        'Asia/Ho_Chi_Minh' => 'Indokitajski čas (Hošiminh)',
        'Asia/Sakhalin' => 'Magadanski čas (Sahalin)',
        'Asia/Samarkand' => 'Uzbekistanski čas (Samarkand)',
        'Asia/Seoul' => 'Korejski čas (Seul)',
        'Asia/Shanghai' => 'Kitajski čas (Šanghaj)',
        'Asia/Singapore' => 'Singapurski standardni čas',
        'Asia/Srednekolymsk' => 'Magadanski čas (Srednekolimsk)',
        'Asia/Taipei' => 'Tajpejski čas',
        'Asia/Tashkent' => 'Uzbekistanski čas (Taškent)',
        'Asia/Tbilisi' => 'Gruzijski čas (Tbilisi)',
        'Asia/Tehran' => 'Iranski čas (Teheran)',
        'Asia/Thimphu' => 'Butanski čas (Timpu)',
        'Asia/Tokyo' => 'Japonski čas (Tokio)',
        'Asia/Tomsk' => 'Krasnojarski čas (Tomsk)',
        'Asia/Ulaanbaatar' => 'Ulanbatorski čas (Ulan Bator)',
        'Asia/Urumqi' => 'Kitajska (lokalni čas) (Urumči)',
        'Asia/Ust-Nera' => 'Vladivostoški čas (Ust-Nera)',
        'Asia/Vientiane' => 'Indokitajski čas (Vientiane)',
        'Asia/Vladivostok' => 'Vladivostoški čas (Vladivostok)',
        'Asia/Yakutsk' => 'Jakutski čas',
        'Asia/Yekaterinburg' => 'Jekaterinburški čas (Jekaterinburg)',
        'Asia/Yerevan' => 'Armenski čas (Erevan)',
        'Atlantic/Azores' => 'Azorski čas (Azori)',
        'Atlantic/Bermuda' => 'Atlantski čas (Bermudi)',
        'Atlantic/Canary' => 'Zahodnoevropski čas (Kanarski otoki)',
        'Atlantic/Cape_Verde' => 'Kapverdski čas (Zelenortski otoki)',
        'Atlantic/Faeroe' => 'Zahodnoevropski čas (Ferski otoki)',
        'Atlantic/Faroe' => 'Zahodnoevropski čas (Ferski otoki)',
        'Atlantic/Madeira' => 'Zahodnoevropski čas (Madeira)',
        'Atlantic/Reykjavik' => 'Greenwiški srednji čas (Reykjavik)',
        'Atlantic/South_Georgia' => 'Južnogeorgijski čas (Južna Georgia)',
        'Atlantic/St_Helena' => 'Greenwiški srednji čas (Sv. Helena)',
        'Atlantic/Stanley' => 'Čas: Falklandsko otočje (Stanley)',
        'Australia/Adelaide' => 'Avstralski centralni čas (Adelaide)',
        'Australia/Brisbane' => 'Avstralski vzhodni čas (Brisbane)',
        'Australia/Broken_Hill' => 'Avstralski centralni čas (Broken Hill)',
        'Australia/Darwin' => 'Avstralski centralni čas (Darwin)',
        'Australia/Eucla' => 'Avstralski centralni zahodni čas (Eucla)',
        'Australia/Hobart' => 'Avstralski vzhodni čas (Hobart)',
        'Australia/Lindeman' => 'Avstralski vzhodni čas (Lindeman)',
        'Australia/Lord_Howe' => 'Čas otoka Lord Howe',
        'Australia/Melbourne' => 'Avstralski vzhodni čas (Melbourne)',
        'Australia/Perth' => 'Avstralski zahodni čas (Perth)',
        'Australia/Sydney' => 'Avstralski vzhodni čas (Sydney)',
        'Etc/GMT' => 'Greenwiški srednji čas',
        'Etc/UTC' => 'univerzalni koordinirani čas',
        'Europe/Amsterdam' => 'Srednjeevropski čas (Amsterdam)',
        'Europe/Andorra' => 'Srednjeevropski čas (Andora)',
        'Europe/Astrakhan' => 'Samarski čas (Astrahan)',
        'Europe/Athens' => 'Vzhodnoevropski čas (Atene)',
        'Europe/Belgrade' => 'Srednjeevropski čas (Beograd)',
        'Europe/Berlin' => 'Srednjeevropski čas (Berlin)',
        'Europe/Bratislava' => 'Srednjeevropski čas (Bratislava)',
        'Europe/Brussels' => 'Srednjeevropski čas (Bruselj)',
        'Europe/Bucharest' => 'Vzhodnoevropski čas (Bukarešta)',
        'Europe/Budapest' => 'Srednjeevropski čas (Budimpešta)',
        'Europe/Busingen' => 'Srednjeevropski čas (Büsingen)',
        'Europe/Chisinau' => 'Vzhodnoevropski čas (Kišinjev)',
        'Europe/Copenhagen' => 'Srednjeevropski čas (Köbenhavn)',
        'Europe/Dublin' => 'Greenwiški srednji čas (Dublin)',
        'Europe/Gibraltar' => 'Srednjeevropski čas (Gibraltar)',
        'Europe/Guernsey' => 'Greenwiški srednji čas (Guernsey)',
        'Europe/Helsinki' => 'Vzhodnoevropski čas (Helsinki)',
        'Europe/Isle_of_Man' => 'Greenwiški srednji čas (Otok Man)',
        'Europe/Istanbul' => 'Turški čas (Istanbul)',
        'Europe/Jersey' => 'Greenwiški srednji čas (Jersey)',
        'Europe/Kaliningrad' => 'Vzhodnoevropski čas (Kaliningrad)',
        'Europe/Kiev' => 'Vzhodnoevropski čas (Kijev)',
        'Europe/Kyiv' => 'Vzhodnoevropski čas (Kijev)',
        'Europe/Kirov' => 'Moskovski čas (Kirov)',
        'Europe/Lisbon' => 'Zahodnoevropski čas (Lizbona)',
        'Europe/Ljubljana' => 'Srednjeevropski čas (Ljubljana)',
        'Europe/London' => 'Greenwiški srednji čas (London)',
        'Europe/Luxembourg' => 'Srednjeevropski čas (Luksemburg)',
        'Europe/Madrid' => 'Srednjeevropski čas (Madrid)',
        'Europe/Malta' => 'Srednjeevropski čas (Malta)',
        'Europe/Mariehamn' => 'Vzhodnoevropski čas (Mariehamn)',
        'Europe/Minsk' => 'Moskovski čas (Minsk)',
        'Europe/Monaco' => 'Srednjeevropski čas (Monako)',
        'Europe/Moscow' => 'Moskovski čas (Moskva)',
        'Europe/Oslo' => 'Srednjeevropski čas (Oslo)',
        'Europe/Paris' => 'Srednjeevropski čas (Pariz)',
        'Europe/Podgorica' => 'Srednjeevropski čas (Podgorica)',
        'Europe/Prague' => 'Srednjeevropski čas (Praga)',
        'Europe/Riga' => 'Vzhodnoevropski čas (Riga)',
        'Europe/Rome' => 'Srednjeevropski čas (Rim)',
        'Europe/Samara' => 'Samarski čas (Samara)',
        'Europe/San_Marino' => 'Srednjeevropski čas (San Marino)',
        'Europe/Sarajevo' => 'Srednjeevropski čas (Sarajevo)',
        'Europe/Saratov' => 'Samarski čas (Saratov)',
        'Europe/Simferopol' => 'Moskovski čas (Simferopol)',
        'Europe/Skopje' => 'Srednjeevropski čas (Skopje)',
        'Europe/Sofia' => 'Vzhodnoevropski čas (Sofija)',
        'Europe/Stockholm' => 'Srednjeevropski čas (Stockholm)',
        'Europe/Tallinn' => 'Vzhodnoevropski čas (Talin)',
        'Europe/Tirane' => 'Srednjeevropski čas (Tirana)',
        'Europe/Ulyanovsk' => 'Samarski čas (Uljanovsk)',
        'Europe/Vaduz' => 'Srednjeevropski čas (Vaduz)',
        'Europe/Vatican' => 'Srednjeevropski čas (Vatikan)',
        'Europe/Vienna' => 'Srednjeevropski čas (Dunaj)',
        'Europe/Vilnius' => 'Vzhodnoevropski čas (Vilna)',
        'Europe/Volgograd' => 'Moskovski čas (Volgograd)',
        'Europe/Warsaw' => 'Srednjeevropski čas (Varšava)',
        'Europe/Zagreb' => 'Srednjeevropski čas (Zagreb)',
        'Europe/Zurich' => 'Srednjeevropski čas (Zürich)',
        'Indian/Antananarivo' => 'Vzhodnoafriški čas (Antananarivo)',
        'Indian/Chagos' => 'Indijskooceanski čas (Chagos)',
        'Indian/Christmas' => 'Božičnootoški čas (Božični otok)',
        'Indian/Cocos' => 'Čas: Kokosovi otoki',
        'Indian/Comoro' => 'Vzhodnoafriški čas (Komori)',
        'Indian/Kerguelen' => 'Francoski južni in antarktični čas (Kerguelen)',
        'Indian/Mahe' => 'Sejšelski čas (Mahé)',
        'Indian/Maldives' => 'Maldivski čas (Maldivi)',
        'Indian/Mauritius' => 'Mauricijski čas (Mauritius)',
        'Indian/Mayotte' => 'Vzhodnoafriški čas (Mayotte)',
        'Indian/Reunion' => 'Reunionski čas (Réunion)',
        'Pacific/Apia' => 'Čas: Apia',
        'Pacific/Auckland' => 'Novozelandski čas (Auckland)',
        'Pacific/Bougainville' => 'Papuanski čas (Bougainville)',
        'Pacific/Chatham' => 'Čatamski čas (Chatham)',
        'Pacific/Easter' => 'Čas: Velikonočni otok',
        'Pacific/Efate' => 'Vanuatujski čas (Efate)',
        'Pacific/Enderbury' => 'Čas: Otočje Feniks (Kanton)',
        'Pacific/Kanton' => 'Čas: Otočje Feniks (Kanton)',
        'Pacific/Fakaofo' => 'Tokelavski čas (Fakaofo)',
        'Pacific/Fiji' => 'Fidžijski čas',
        'Pacific/Funafuti' => 'Tuvalujski čas (Funafuti)',
        'Pacific/Galapagos' => 'Galapaški čas (Galápagos)',
        'Pacific/Gambier' => 'Gambierski čas',
        'Pacific/Guadalcanal' => 'Salomonovootoški čas (Guadalcanal)',
        'Pacific/Guam' => 'Čamorski standardni čas (Guam)',
        'Pacific/Honolulu' => 'Havajski aleutski standardni čas (Honolulu)',
        'Pacific/Kiritimati' => 'Čas: Ekvatorski otoki (Kiritimati)',
        'Pacific/Kosrae' => 'Kosrajški čas (Kosrae)',
        'Pacific/Kwajalein' => 'Čas: Marshallovi otoki (Kwajalein)',
        'Pacific/Majuro' => 'Čas: Marshallovi otoki (Majuro)',
        'Pacific/Marquesas' => 'Čas: Markizni otoki (Marquesas)',
        'Pacific/Midway' => 'Samoanski čas (Midway)',
        'Pacific/Nauru' => 'Naurujski čas',
        'Pacific/Niue' => 'Niuejski čas',
        'Pacific/Norfolk' => 'Čas: Norfolški otoki (Norfolk)',
        'Pacific/Noumea' => 'Novokaledonijski čas (Nouméa)',
        'Pacific/Pago_Pago' => 'Samoanski čas (Pago Pago)',
        'Pacific/Palau' => 'Palavski čas (Palau)',
        'Pacific/Pitcairn' => 'Pitcairnski čas',
        'Pacific/Ponape' => 'Ponapski čas (Pohnpei)',
        'Pacific/Pohnpei' => 'Ponapski čas (Pohnpei)',
        'Pacific/Port_Moresby' => 'Papuanski čas (Port Moresby)',
        'Pacific/Rarotonga' => 'Cookovootoški čas (Rarotonga)',
        'Pacific/Saipan' => 'Čamorski standardni čas (Saipan)',
        'Pacific/Tahiti' => 'Tahitijski čas',
        'Pacific/Tarawa' => 'Čas: Gilbertovi otoki (Tarawa)',
        'Pacific/Tongatapu' => 'Tongovski čas (Tongatapu)',
        'Pacific/Truk' => 'Čas: Otok Chuuk',
        'Pacific/Chuuk' => 'Čas: Otok Chuuk',
        'Pacific/Wake' => 'Čas: Otok Wake',
        'Pacific/Wallis' => 'Čas: Wallis in Futuna (Wallis & Futuna)',
    ],
    'Meta' => [
        'GmtFormat' => 'GMT %s',
        'HourFormatPos' => '+%02d.%02d',
        'HourFormatNeg' => '-%02d.%02d',
    ],
];
