<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\Validator\Tests\Fixtures;

class CascadingEntity
{
    public string $scalar;

    public CascadedChild $requiredChild;

    public ?CascadedChild $optionalChild;

    public static ?CascadedChild $staticChild;

    /**
     * @var CascadedChild[]
     */
    public array $children;
}
