<?php

return [
    'Names' => [
        'Africa/Abidjan' => 'Гринвич уртача вакыты (Идиҗан)',
        'Africa/Accra' => 'Гринвич уртача вакыты (Аккра)',
        'Africa/Addis_Ababa' => 'Көнчыгыш Африка вакыты (Аддис-Абеба)',
        'Africa/Algiers' => 'Үзәк Европа вакыты (Алжир)',
        'Africa/Asmera' => 'Көнчыгыш Африка вакыты (Асмэра)',
        'Africa/Asmara' => 'Көнчыгыш Африка вакыты (Асмэра)',
        'Africa/Bamako' => 'Гринвич уртача вакыты (Бамако)',
        'Africa/Bangui' => 'Көнбатыш Африка вакыты (Банги)',
        'Africa/Banjul' => 'Гринвич уртача вакыты (Банҗул)',
        'Africa/Bissau' => 'Гринвич уртача вакыты (Биссау)',
        'Africa/Blantyre' => 'Үзәк Африка вакыты (Блантайр)',
        'Africa/Brazzaville' => 'Көнбатыш Африка вакыты (Браззавиль)',
        'Africa/Bujumbura' => 'Үзәк Африка вакыты (Бөҗүмбура)',
        'Africa/Cairo' => 'Көнчыгыш Европа вакыты (Каһирә)',
        'Africa/Casablanca' => 'Көнбатыш Европа вакыты (Касабланка)',
        'Africa/Ceuta' => 'Үзәк Европа вакыты (Сеута)',
        'Africa/Conakry' => 'Гринвич уртача вакыты (Конакри)',
        'Africa/Dakar' => 'Гринвич уртача вакыты (Дакар)',
        'Africa/Dar_es_Salaam' => 'Көнчыгыш Африка вакыты (Дар-эс-Салам)',
        'Africa/Djibouti' => 'Көнчыгыш Африка вакыты (Джибути)',
        'Africa/Douala' => 'Көнбатыш Африка вакыты (Дуала)',
        'Africa/El_Aaiun' => 'Көнбатыш Европа вакыты (Эль-Аюн)',
        'Africa/Freetown' => 'Гринвич уртача вакыты (Фритаун)',
        'Africa/Gaborone' => 'Үзәк Африка вакыты (Габороне)',
        'Africa/Harare' => 'Үзәк Африка вакыты (Хараре)',
        'Africa/Johannesburg' => 'Көньяк Африка вакыты (Йоханнесбург)',
        'Africa/Juba' => 'Үзәк Африка вакыты (Җуба)',
        'Africa/Kampala' => 'Көнчыгыш Африка вакыты (Кампала)',
        'Africa/Khartoum' => 'Үзәк Африка вакыты (Һартум)',
        'Africa/Kigali' => 'Үзәк Африка вакыты (Кигали)',
        'Africa/Kinshasa' => 'Көнбатыш Африка вакыты (Киншаса)',
        'Africa/Lagos' => 'Көнбатыш Африка вакыты (Лагос)',
        'Africa/Libreville' => 'Көнбатыш Африка вакыты (Либревиль)',
        'Africa/Lome' => 'Гринвич уртача вакыты (Ломе)',
        'Africa/Luanda' => 'Көнбатыш Африка вакыты (Луанда)',
        'Africa/Lubumbashi' => 'Үзәк Африка вакыты (Лубумбаши)',
        'Africa/Lusaka' => 'Үзәк Африка вакыты (Лусака)',
        'Africa/Malabo' => 'Көнбатыш Африка вакыты (Малабо)',
        'Africa/Maputo' => 'Үзәк Африка вакыты (Мапуто)',
        'Africa/Maseru' => 'Көньяк Африка вакыты (Масеру)',
        'Africa/Mbabane' => 'Көньяк Африка вакыты (Мбабане)',
        'Africa/Mogadishu' => 'Көнчыгыш Африка вакыты (Могадишо)',
        'Africa/Monrovia' => 'Гринвич уртача вакыты (Монровия)',
        'Africa/Nairobi' => 'Көнчыгыш Африка вакыты (Найроби)',
        'Africa/Ndjamena' => 'Көнбатыш Африка вакыты (Нҗамен)',
        'Africa/Niamey' => 'Көнбатыш Африка вакыты (Ниамей)',
        'Africa/Nouakchott' => 'Гринвич уртача вакыты (Нуакшот)',
        'Africa/Ouagadougou' => 'Гринвич уртача вакыты (Уагадугу)',
        'Africa/Porto-Novo' => 'Көнбатыш Африка вакыты (Порто-Ново)',
        'Africa/Sao_Tome' => 'Гринвич уртача вакыты (Сан-Томе)',
        'Africa/Tripoli' => 'Көнчыгыш Европа вакыты (Триполи)',
        'Africa/Tunis' => 'Үзәк Европа вакыты (Тунис)',
        'Africa/Windhoek' => 'Үзәк Африка вакыты (Виндхук)',
        'America/Adak' => 'Гавай-Алеут вакыты (Адак)',
        'America/Anchorage' => 'Аляска вакыты (Анкоридж)',
        'America/Anguilla' => 'Төньяк Америка атлантик вакыты (Ангилья)',
        'America/Antigua' => 'Төньяк Америка атлантик вакыты (Антигуа)',
        'America/Araguaina' => 'Бразилиа вакыты (Арагуайна)',
        'America/Argentina/La_Rioja' => 'Аргентина вакыты (Ла Риоха)',
        'America/Argentina/Rio_Gallegos' => 'Аргентина вакыты (Рио-Гальегос)',
        'America/Argentina/Salta' => 'Аргентина вакыты (Сальта)',
        'America/Argentina/San_Juan' => 'Аргентина вакыты (Сан-Хуан)',
        'America/Argentina/San_Luis' => 'Аргентина вакыты (Сан-Луис)',
        'America/Argentina/Tucuman' => 'Аргентина вакыты (Тукуман)',
        'America/Argentina/Ushuaia' => 'Аргентина вакыты (Ушуайя)',
        'America/Aruba' => 'Төньяк Америка атлантик вакыты (Аруба)',
        'America/Asuncion' => 'Парагвай вакыты (Асунсьон)',
        'America/Bahia' => 'Бразилиа вакыты (Баия)',
        'America/Bahia_Banderas' => 'Төньяк Америка үзәк вакыты (Баия-де-Бандерас)',
        'America/Barbados' => 'Төньяк Америка атлантик вакыты (Барбадос)',
        'America/Belem' => 'Бразилиа вакыты (Белем)',
        'America/Belize' => 'Төньяк Америка үзәк вакыты (Белиз)',
        'America/Blanc-Sablon' => 'Төньяк Америка атлантик вакыты (Блан-Саблон)',
        'America/Boa_Vista' => 'Амазонка вакыты (Боа-Виста)',
        'America/Bogota' => 'Колумбия вакыты (Богота)',
        'America/Boise' => 'Төньяк Америка тау вакыты (Бойсе)',
        'America/Buenos_Aires' => 'Аргентина вакыты (Буэнос-Айрес)',
        'America/Argentina/Buenos_Aires' => 'Аргентина вакыты (Буэнос-Айрес)',
        'America/Cambridge_Bay' => 'Төньяк Америка тау вакыты (Кембридж Бэй)',
        'America/Campo_Grande' => 'Амазонка вакыты (Кампо-Гранде)',
        'America/Cancun' => 'Төньяк Америка көнчыгыш вакыты (Канкун)',
        'America/Caracas' => 'Венесуэла вакыты (Каракас)',
        'America/Catamarca' => 'Аргентина вакыты (Катамарка)',
        'America/Argentina/Catamarca' => 'Аргентина вакыты (Катамарка)',
        'America/Cayenne' => 'Француз Гвиана вакыты (Кайенна)',
        'America/Cayman' => 'Төньяк Америка көнчыгыш вакыты (Кайман утраулары)',
        'America/Chicago' => 'Төньяк Америка үзәк вакыты (Чикаго)',
        'America/Chihuahua' => 'Төньяк Америка үзәк вакыты (Чихуахуа)',
        'America/Ciudad_Juarez' => 'Төньяк Америка тау вакыты (Сьюдад-Хуарес)',
        'America/Coral_Harbour' => 'Төньяк Америка көнчыгыш вакыты (Атикокан)',
        'America/Atikokan' => 'Төньяк Америка көнчыгыш вакыты (Атикокан)',
        'America/Cordoba' => 'Аргентина вакыты (Кордоба)',
        'America/Argentina/Cordoba' => 'Аргентина вакыты (Кордоба)',
        'America/Costa_Rica' => 'Төньяк Америка үзәк вакыты (Коста-Рика)',
        'America/Coyhaique' => 'Чили вакыты (Куяйкә)',
        'America/Creston' => 'Төньяк Америка тау вакыты (Крестон)',
        'America/Cuiaba' => 'Амазонка вакыты (Куяба)',
        'America/Curacao' => 'Төньяк Америка атлантик вакыты (Кюрасао)',
        'America/Danmarkshavn' => 'Гринвич уртача вакыты (Данмарксхавн)',
        'America/Dawson' => 'Юкон вакыты (Доусон)',
        'America/Dawson_Creek' => 'Төньяк Америка тау вакыты (Доусон-Крик)',
        'America/Denver' => 'Төньяк Америка тау вакыты (Денвер)',
        'America/Detroit' => 'Төньяк Америка көнчыгыш вакыты (Детройт)',
        'America/Dominica' => 'Төньяк Америка атлантик вакыты (Доминика)',
        'America/Edmonton' => 'Төньяк Америка тау вакыты (Эдмонтон)',
        'America/Eirunepe' => 'Акр вакыты (Эйрунепе)',
        'America/El_Salvador' => 'Төньяк Америка үзәк вакыты (Сальвадор)',
        'America/Fort_Nelson' => 'Төньяк Америка тау вакыты (Форт Нельсон)',
        'America/Fortaleza' => 'Бразилиа вакыты (Форталеза)',
        'America/Glace_Bay' => 'Төньяк Америка атлантик вакыты (Глейс Бэй)',
        'America/Godthab' => 'Гренландия вакыты (Нуук)',
        'America/Nuuk' => 'Гренландия вакыты (Нуук)',
        'America/Goose_Bay' => 'Төньяк Америка атлантик вакыты (Каз бухтасы)',
        'America/Grand_Turk' => 'Төньяк Америка көнчыгыш вакыты (Гранд Терк)',
        'America/Grenada' => 'Төньяк Америка атлантик вакыты (Гренада)',
        'America/Guadeloupe' => 'Төньяк Америка атлантик вакыты (Гваделупа)',
        'America/Guatemala' => 'Төньяк Америка үзәк вакыты (Гватемала)',
        'America/Guayaquil' => 'Эквадор вакыты (Гуаякиль)',
        'America/Guyana' => 'Гайана вакыты',
        'America/Halifax' => 'Төньяк Америка атлантик вакыты (Галифакс)',
        'America/Havana' => 'Куба вакыты (Гавана)',
        'America/Hermosillo' => 'Мексика Тыныч океан вакыты (Эрмосильо)',
        'America/Indiana/Knox' => 'Төньяк Америка үзәк вакыты (Нокс, Индиана)',
        'America/Indiana/Marengo' => 'Төньяк Америка көнчыгыш вакыты (Маренго, Индиана)',
        'America/Indiana/Petersburg' => 'Төньяк Америка көнчыгыш вакыты (Петербург, Индиана)',
        'America/Indiana/Tell_City' => 'Төньяк Америка үзәк вакыты (Телль-Сити, Индиана)',
        'America/Indiana/Vevay' => 'Төньяк Америка көнчыгыш вакыты (Вевей, Индиана)',
        'America/Indiana/Vincennes' => 'Төньяк Америка көнчыгыш вакыты (Винсеннес, Индиана)',
        'America/Indiana/Winamac' => 'Төньяк Америка көнчыгыш вакыты (Уинамак, Индиана)',
        'America/Indianapolis' => 'Төньяк Америка көнчыгыш вакыты (Индианаполис)',
        'America/Indiana/Indianapolis' => 'Төньяк Америка көнчыгыш вакыты (Индианаполис)',
        'America/Inuvik' => 'Төньяк Америка тау вакыты (Инувик)',
        'America/Iqaluit' => 'Төньяк Америка көнчыгыш вакыты (Икалуит)',
        'America/Jamaica' => 'Төньяк Америка көнчыгыш вакыты (Ямайка)',
        'America/Jujuy' => 'Аргентина вакыты (Жужуй)',
        'America/Argentina/Jujuy' => 'Аргентина вакыты (Жужуй)',
        'America/Juneau' => 'Аляска вакыты (Джуно)',
        'America/Kentucky/Monticello' => 'Төньяк Америка көнчыгыш вакыты (Монтичелло, Кентукки)',
        'America/Kralendijk' => 'Төньяк Америка атлантик вакыты (Кралендейк)',
        'America/La_Paz' => 'Боливия вакыты (Ла-Пас)',
        'America/Lima' => 'Перу вакыты (Лима)',
        'America/Los_Angeles' => 'Төньяк Америка Тын океан вакыты (Лос-Анджелес)',
        'America/Louisville' => 'Төньяк Америка көнчыгыш вакыты (Луисвиль)',
        'America/Kentucky/Louisville' => 'Төньяк Америка көнчыгыш вакыты (Луисвиль)',
        'America/Lower_Princes' => 'Төньяк Америка атлантик вакыты (Түбәнге Кенәз кварталы)',
        'America/Maceio' => 'Бразилиа вакыты (Масейо)',
        'America/Managua' => 'Төньяк Америка үзәк вакыты (Манагуа)',
        'America/Manaus' => 'Амазонка вакыты (Манаус)',
        'America/Marigot' => 'Төньяк Америка атлантик вакыты (Мариго)',
        'America/Martinique' => 'Төньяк Америка атлантик вакыты (Мартиника)',
        'America/Matamoros' => 'Төньяк Америка үзәк вакыты (Матаморос)',
        'America/Mazatlan' => 'Мексика Тыныч океан вакыты (Масатлан)',
        'America/Mendoza' => 'Аргентина вакыты (Мендоса)',
        'America/Argentina/Mendoza' => 'Аргентина вакыты (Мендоса)',
        'America/Menominee' => 'Төньяк Америка үзәк вакыты (Меномини)',
        'America/Merida' => 'Төньяк Америка үзәк вакыты (Мерида)',
        'America/Metlakatla' => 'Аляска вакыты (Метлакатла)',
        'America/Mexico_City' => 'Төньяк Америка үзәк вакыты (Мехико)',
        'America/Miquelon' => 'Сен-Пьер һәм Микелон вакыты',
        'America/Moncton' => 'Төньяк Америка атлантик вакыты (Монктон)',
        'America/Monterrey' => 'Төньяк Америка үзәк вакыты (Монтеррей)',
        'America/Montevideo' => 'Уругвай вакыты (Монтевидео)',
        'America/Montserrat' => 'Төньяк Америка атлантик вакыты (Монсеррат)',
        'America/Nassau' => 'Төньяк Америка көнчыгыш вакыты (Нассау)',
        'America/New_York' => 'Төньяк Америка көнчыгыш вакыты (Нью-Йорк)',
        'America/Nome' => 'Аляска вакыты (Номе)',
        'America/Noronha' => 'Фернанду-ди-Норонья вакыты',
        'America/North_Dakota/Beulah' => 'Төньяк Америка үзәк вакыты (Бьюла, Төньяк Дакота)',
        'America/North_Dakota/Center' => 'Төньяк Америка үзәк вакыты (Үзәк, Төньяк Дакота)',
        'America/North_Dakota/New_Salem' => 'Төньяк Америка үзәк вакыты (Нью-Салем, Төньяк Дакота)',
        'America/Ojinaga' => 'Төньяк Америка үзәк вакыты (Охинага)',
        'America/Panama' => 'Төньяк Америка көнчыгыш вакыты (Панама)',
        'America/Paramaribo' => 'Суринам вакыты (Парамарибо)',
        'America/Phoenix' => 'Төньяк Америка тау вакыты (Феникс)',
        'America/Port-au-Prince' => 'Төньяк Америка көнчыгыш вакыты (Порт-о-Пренс)',
        'America/Port_of_Spain' => 'Төньяк Америка атлантик вакыты (Порт-оф-Спейн)',
        'America/Porto_Velho' => 'Амазонка вакыты (Порту-Велью)',
        'America/Puerto_Rico' => 'Төньяк Америка атлантик вакыты (Пуэрто-Рико)',
        'America/Punta_Arenas' => 'Чили вакыты (Пунта-Аренас)',
        'America/Rankin_Inlet' => 'Төньяк Америка үзәк вакыты (Ранкин-Инлет)',
        'America/Recife' => 'Бразилиа вакыты (Ресифи)',
        'America/Regina' => 'Төньяк Америка үзәк вакыты (Регина)',
        'America/Resolute' => 'Төньяк Америка үзәк вакыты (Резолют)',
        'America/Rio_Branco' => 'Акр вакыты (Рио-Бранко)',
        'America/Santarem' => 'Бразилиа вакыты (Сантарем)',
        'America/Santiago' => 'Чили вакыты (Сантьяго)',
        'America/Santo_Domingo' => 'Төньяк Америка атлантик вакыты (Санто-Доминго)',
        'America/Sao_Paulo' => 'Бразилиа вакыты (Сан-Паулу)',
        'America/Scoresbysund' => 'Гренландия вакыты (Иттоккортоормиит)',
        'America/Sitka' => 'Аляска вакыты (Ситка)',
        'America/St_Barthelemy' => 'Төньяк Америка атлантик вакыты (Изге Варфоломей)',
        'America/St_Johns' => 'Ньюфаундленд вакыты (Сент-Джонс)',
        'America/St_Kitts' => 'Төньяк Америка атлантик вакыты (Сент-Китс)',
        'America/St_Lucia' => 'Төньяк Америка атлантик вакыты (Изге Люсия)',
        'America/St_Thomas' => 'Төньяк Америка атлантик вакыты (Сент-Томас)',
        'America/St_Vincent' => 'Төньяк Америка атлантик вакыты (Сент-Винсент)',
        'America/Swift_Current' => 'Төньяк Америка үзәк вакыты (Swift Current)',
        'America/Tegucigalpa' => 'Төньяк Америка үзәк вакыты (Тегусигальпа)',
        'America/Thule' => 'Төньяк Америка атлантик вакыты (Туле)',
        'America/Tijuana' => 'Төньяк Америка Тын океан вакыты (Тихуана)',
        'America/Toronto' => 'Төньяк Америка көнчыгыш вакыты (Торонто)',
        'America/Tortola' => 'Төньяк Америка атлантик вакыты (Тортола)',
        'America/Vancouver' => 'Төньяк Америка Тын океан вакыты (Ванкувер)',
        'America/Whitehorse' => 'Юкон вакыты (Уайтхорс)',
        'America/Winnipeg' => 'Төньяк Америка үзәк вакыты (Виннипег)',
        'America/Yakutat' => 'Аляска вакыты (Якутат)',
        'Antarctica/Casey' => 'Көнбатыш Австралия вакыты (Кейси)',
        'Antarctica/Davis' => 'Дэвис вакыты',
        'Antarctica/DumontDUrville' => 'Дюмон д’Юрвиль вакыты',
        'Antarctica/Macquarie' => 'Көнчыгыш Австралия вакыты (Маккуори)',
        'Antarctica/Mawson' => 'Моусон вакыты',
        'Antarctica/McMurdo' => 'Яңа Зеландия вакыты (МакМёрдо)',
        'Antarctica/Palmer' => 'Чили вакыты (Палмер)',
        'Antarctica/Rothera' => 'Ротера вакыты',
        'Antarctica/Syowa' => 'Сёва вакыты',
        'Antarctica/Troll' => 'Гринвич уртача вакыты (Тролль)',
        'Antarctica/Vostok' => 'Восток вакыты',
        'Arctic/Longyearbyen' => 'Үзәк Европа вакыты (Лонгиербиен)',
        'Asia/Aden' => 'Гарәп вакыты (Аден)',
        'Asia/Almaty' => 'Казахстан вакыты (Алма-Ата)',
        'Asia/Amman' => 'Көнчыгыш Европа вакыты (Әмман)',
        'Asia/Anadyr' => 'Петропавловск-Камчатский вакыты (Анадырь)',
        'Asia/Aqtau' => 'Казахстан вакыты (Актау)',
        'Asia/Aqtobe' => 'Казахстан вакыты (Актобе)',
        'Asia/Ashgabat' => 'Төркмәнстан вакыты (Ашхабад)',
        'Asia/Atyrau' => 'Казахстан вакыты (Атырау)',
        'Asia/Baghdad' => 'Гарәп вакыты (Багдад)',
        'Asia/Bahrain' => 'Гарәп вакыты (Бахрейн)',
        'Asia/Baku' => 'Әзербайҗан вакыты (Баку)',
        'Asia/Bangkok' => 'Һинд-кытай вакыты (Бангкок)',
        'Asia/Barnaul' => 'Красноярск вакыты (Барнаул)',
        'Asia/Beirut' => 'Көнчыгыш Европа вакыты (Бәйрүт)',
        'Asia/Bishkek' => 'Кыргызстан вакыты (Бишкек)',
        'Asia/Brunei' => 'Бруней-Даруссалам вакыты',
        'Asia/Calcutta' => 'Һинд стандарт вакыты (Калькутта)',
        'Asia/Kolkata' => 'Һинд стандарт вакыты (Калькутта)',
        'Asia/Chita' => 'Якутск вакыты (Чита)',
        'Asia/Colombo' => 'Һинд стандарт вакыты (Коломбо)',
        'Asia/Damascus' => 'Көнчыгыш Европа вакыты (Дәмәшкъ)',
        'Asia/Dhaka' => 'Бангладеш вакыты (Дакка)',
        'Asia/Dili' => 'Көнчыгыш Тимор вакыты (Дили)',
        'Asia/Dubai' => 'Фарсы култыгының стандарт вакыты (Дубай)',
        'Asia/Dushanbe' => 'Таҗикстан вакыты (Душанбе)',
        'Asia/Famagusta' => 'Көнчыгыш Европа вакыты (Фамагуста)',
        'Asia/Gaza' => 'Көнчыгыш Европа вакыты (Газа)',
        'Asia/Hebron' => 'Көнчыгыш Европа вакыты (Һеврон)',
        'Asia/Hong_Kong' => 'Гонконг вакыты',
        'Asia/Hovd' => 'Ховд вакыты',
        'Asia/Irkutsk' => 'Иркутск вакыты',
        'Asia/Jakarta' => 'Көнбатыш Индонезия вакыты (Җакарта)',
        'Asia/Jayapura' => 'Көнчыгыш Индонезия вакыты (Җәяпура)',
        'Asia/Jerusalem' => 'Исраил вакыты (Иерусалим)',
        'Asia/Kabul' => 'Әфганстан вакыты (Кабул)',
        'Asia/Kamchatka' => 'Петропавловск-Камчатский вакыты (Камчатка)',
        'Asia/Karachi' => 'Пакистан вакыты (Карачи)',
        'Asia/Katmandu' => 'Непал вакыты (Катманду)',
        'Asia/Kathmandu' => 'Непал вакыты (Катманду)',
        'Asia/Khandyga' => 'Якутск вакыты (Хандыга)',
        'Asia/Krasnoyarsk' => 'Красноярск вакыты',
        'Asia/Kuala_Lumpur' => 'Малайзия вакыты (Куала-Лумпур)',
        'Asia/Kuching' => 'Малайзия вакыты (Кучинг)',
        'Asia/Kuwait' => 'Гарәп вакыты (Кувейт)',
        'Asia/Macau' => 'Кытай вакыты (Макао)',
        'Asia/Magadan' => 'Магадан вакыты',
        'Asia/Makassar' => 'Үзәк Индонезия вакыты (Макассар)',
        'Asia/Manila' => 'Филиппин вакыты (Манила)',
        'Asia/Muscat' => 'Фарсы култыгының стандарт вакыты (Маскат)',
        'Asia/Nicosia' => 'Көнчыгыш Европа вакыты (Никосия)',
        'Asia/Novokuznetsk' => 'Красноярск вакыты (Новокузнецк)',
        'Asia/Novosibirsk' => 'Красноярск вакыты (Новосибирск)',
        'Asia/Omsk' => 'Омск вакыты',
        'Asia/Oral' => 'Казахстан вакыты (Орал)',
        'Asia/Phnom_Penh' => 'Һинд-кытай вакыты (Пномпень)',
        'Asia/Pontianak' => 'Көнбатыш Индонезия вакыты (Понтианак)',
        'Asia/Pyongyang' => 'Корея вакыты (Пхеньян)',
        'Asia/Qatar' => 'Гарәп вакыты (Катар)',
        'Asia/Qostanay' => 'Казахстан вакыты (Костанай)',
        'Asia/Qyzylorda' => 'Казахстан вакыты (Кызылорда)',
        'Asia/Rangoon' => 'Мьянма вакыты (Янгон)',
        'Asia/Yangon' => 'Мьянма вакыты (Янгон)',
        'Asia/Riyadh' => 'Гарәп вакыты (Эр-Рияд)',
        'Asia/Saigon' => 'Һинд-кытай вакыты (Хо Ши Мин)',
        'Asia/Ho_Chi_Minh' => 'Һинд-кытай вакыты (Хо Ши Мин)',
        'Asia/Sakhalin' => 'Магадан вакыты (Сахалин)',
        'Asia/Samarkand' => 'Үзбәкстан вакыты (Сәмәрканд)',
        'Asia/Seoul' => 'Корея вакыты (Сеул)',
        'Asia/Shanghai' => 'Кытай вакыты (Шанхай)',
        'Asia/Singapore' => 'Сингапур стандарт вакыты',
        'Asia/Srednekolymsk' => 'Магадан вакыты (Среднеколымск)',
        'Asia/Taipei' => 'Тайпей вакыты',
        'Asia/Tashkent' => 'Үзбәкстан вакыты (Ташкент)',
        'Asia/Tbilisi' => 'Грузия вакыты (Тбилиси)',
        'Asia/Tehran' => 'Иран вакыты (Тәһран)',
        'Asia/Thimphu' => 'Бутан вакыты (Тхимпху)',
        'Asia/Tokyo' => 'Япон вакыты (Токио)',
        'Asia/Tomsk' => 'Красноярск вакыты (Томск)',
        'Asia/Ulaanbaatar' => 'Улан-Батор вакыты',
        'Asia/Urumqi' => 'Кытай вакыты (Урумчи)',
        'Asia/Ust-Nera' => 'Владивосток вакыты (Усть-Нера)',
        'Asia/Vientiane' => 'Һинд-кытай вакыты (Вьентьян)',
        'Asia/Vladivostok' => 'Владивосток вакыты',
        'Asia/Yakutsk' => 'Якутск вакыты',
        'Asia/Yekaterinburg' => 'Екатеринбург вакыты',
        'Asia/Yerevan' => 'Армения вакыты (Ереван)',
        'Atlantic/Azores' => 'Азор утраулары вакыты',
        'Atlantic/Bermuda' => 'Төньяк Америка атлантик вакыты (Бермуд утраулары)',
        'Atlantic/Canary' => 'Көнбатыш Европа вакыты (Канар утраулары)',
        'Atlantic/Cape_Verde' => 'Кабо-Верде вакыты',
        'Atlantic/Faeroe' => 'Көнбатыш Европа вакыты (Фарер утраулары)',
        'Atlantic/Faroe' => 'Көнбатыш Европа вакыты (Фарер утраулары)',
        'Atlantic/Madeira' => 'Көнбатыш Европа вакыты (Мадейра)',
        'Atlantic/Reykjavik' => 'Гринвич уртача вакыты (Рейкьявик)',
        'Atlantic/South_Georgia' => 'Көньяк Джорджия вакыты',
        'Atlantic/St_Helena' => 'Гринвич уртача вакыты (Изге Елена)',
        'Atlantic/Stanley' => 'Фолкленд утраулары вакыты (Стэнли)',
        'Australia/Adelaide' => 'Үзәк Австралия вакыты (Аделаида)',
        'Australia/Brisbane' => 'Көнчыгыш Австралия вакыты (Брисбен)',
        'Australia/Broken_Hill' => 'Үзәк Австралия вакыты (Брокен-Хилл)',
        'Australia/Darwin' => 'Үзәк Австралия вакыты (Дарвин)',
        'Australia/Eucla' => 'Австралия үзәк көнбатыш вакыты (Юкла)',
        'Australia/Hobart' => 'Көнчыгыш Австралия вакыты (Хобарт)',
        'Australia/Lindeman' => 'Көнчыгыш Австралия вакыты (Линдеман)',
        'Australia/Lord_Howe' => 'Лорд Хау вакыты',
        'Australia/Melbourne' => 'Көнчыгыш Австралия вакыты (Мельбурн)',
        'Australia/Perth' => 'Көнбатыш Австралия вакыты (Перт)',
        'Australia/Sydney' => 'Көнчыгыш Австралия вакыты (Сидней)',
        'Etc/GMT' => 'Гринвич уртача вакыты',
        'Etc/UTC' => 'Бөтендөнья килештерелгән вакыты',
        'Europe/Amsterdam' => 'Үзәк Европа вакыты (Амстердам)',
        'Europe/Andorra' => 'Үзәк Европа вакыты (Андорра)',
        'Europe/Astrakhan' => 'Самара вакыты (Әстерхан)',
        'Europe/Athens' => 'Көнчыгыш Европа вакыты (Афин)',
        'Europe/Belgrade' => 'Үзәк Европа вакыты (Белград)',
        'Europe/Berlin' => 'Үзәк Европа вакыты (Берлин)',
        'Europe/Bratislava' => 'Үзәк Европа вакыты (Братислава)',
        'Europe/Brussels' => 'Үзәк Европа вакыты (Брюссель)',
        'Europe/Bucharest' => 'Көнчыгыш Европа вакыты (Бухарест)',
        'Europe/Budapest' => 'Үзәк Европа вакыты (Будапешт)',
        'Europe/Busingen' => 'Үзәк Европа вакыты (Бюзинген)',
        'Europe/Chisinau' => 'Көнчыгыш Европа вакыты (Кишинев)',
        'Europe/Copenhagen' => 'Үзәк Европа вакыты (Копенгаген)',
        'Europe/Dublin' => 'Гринвич уртача вакыты (Дублин)',
        'Europe/Gibraltar' => 'Үзәк Европа вакыты (Гибралтар)',
        'Europe/Guernsey' => 'Гринвич уртача вакыты (Гернси)',
        'Europe/Helsinki' => 'Көнчыгыш Европа вакыты (Хельсинки)',
        'Europe/Isle_of_Man' => 'Гринвич уртача вакыты (Мэн утравы)',
        'Europe/Istanbul' => 'Төркия вакыты (Истанбул)',
        'Europe/Jersey' => 'Гринвич уртача вакыты (Джерси)',
        'Europe/Kaliningrad' => 'Көнчыгыш Европа вакыты (Калининград)',
        'Europe/Kiev' => 'Көнчыгыш Европа вакыты (Киев)',
        'Europe/Kyiv' => 'Көнчыгыш Европа вакыты (Киев)',
        'Europe/Kirov' => 'Мәскәү вакыты (Киров)',
        'Europe/Lisbon' => 'Көнбатыш Европа вакыты (Лиссабон)',
        'Europe/Ljubljana' => 'Үзәк Европа вакыты (Любляна)',
        'Europe/London' => 'Гринвич уртача вакыты (Лондон)',
        'Europe/Luxembourg' => 'Үзәк Европа вакыты (Люксембург)',
        'Europe/Madrid' => 'Үзәк Европа вакыты (Мадрид)',
        'Europe/Malta' => 'Үзәк Европа вакыты (Мальта)',
        'Europe/Mariehamn' => 'Көнчыгыш Европа вакыты (Мариехамн)',
        'Europe/Minsk' => 'Мәскәү вакыты (Минск)',
        'Europe/Monaco' => 'Үзәк Европа вакыты (Монако)',
        'Europe/Moscow' => 'Мәскәү вакыты',
        'Europe/Oslo' => 'Үзәк Европа вакыты (Осло)',
        'Europe/Paris' => 'Үзәк Европа вакыты (Париж)',
        'Europe/Podgorica' => 'Үзәк Европа вакыты (Подгорица)',
        'Europe/Prague' => 'Үзәк Европа вакыты (Прага)',
        'Europe/Riga' => 'Көнчыгыш Европа вакыты (Рига)',
        'Europe/Rome' => 'Үзәк Европа вакыты (Рим)',
        'Europe/Samara' => 'Самара вакыты',
        'Europe/San_Marino' => 'Үзәк Европа вакыты (Сан-Марино)',
        'Europe/Sarajevo' => 'Үзәк Европа вакыты (Сараево)',
        'Europe/Saratov' => 'Самара вакыты (Саратов)',
        'Europe/Simferopol' => 'Мәскәү вакыты (Симферополь)',
        'Europe/Skopje' => 'Үзәк Европа вакыты (Скопье)',
        'Europe/Sofia' => 'Көнчыгыш Европа вакыты (София)',
        'Europe/Stockholm' => 'Үзәк Европа вакыты (Стокгольм)',
        'Europe/Tallinn' => 'Көнчыгыш Европа вакыты (Таллин)',
        'Europe/Tirane' => 'Үзәк Европа вакыты (Тиран)',
        'Europe/Ulyanovsk' => 'Самара вакыты (Ульяновск)',
        'Europe/Vaduz' => 'Үзәк Европа вакыты (Вадуц)',
        'Europe/Vatican' => 'Үзәк Европа вакыты (Ватикан)',
        'Europe/Vienna' => 'Үзәк Европа вакыты (Вена)',
        'Europe/Vilnius' => 'Көнчыгыш Европа вакыты (Вильнюс)',
        'Europe/Volgograd' => 'Мәскәү вакыты (Волгоград)',
        'Europe/Warsaw' => 'Үзәк Европа вакыты (Варшава)',
        'Europe/Zagreb' => 'Үзәк Европа вакыты (Загреб)',
        'Europe/Zurich' => 'Үзәк Европа вакыты (Цюрих)',
        'Indian/Antananarivo' => 'Көнчыгыш Африка вакыты (Антананариву)',
        'Indian/Chagos' => 'Һинд океаны вакыты (Чагос)',
        'Indian/Christmas' => 'Раштуа утравы вакыты',
        'Indian/Cocos' => 'Кокос утраулары вакыты',
        'Indian/Comoro' => 'Көнчыгыш Африка вакыты (Коморо)',
        'Indian/Kerguelen' => 'Француз көньяк һәм Антарктика вакыты (Кергелен)',
        'Indian/Mahe' => 'Сейшел утраулары вакыты (Маэ)',
        'Indian/Maldives' => 'Мальдив утраулары вакыты',
        'Indian/Mauritius' => 'Маврикий вакыты',
        'Indian/Mayotte' => 'Көнчыгыш Африка вакыты (Майотта)',
        'Indian/Reunion' => 'Реюньон вакыты',
        'Pacific/Apia' => 'Апиа вакыты',
        'Pacific/Auckland' => 'Яңа Зеландия вакыты (Окленд)',
        'Pacific/Bougainville' => 'Папуа Яңа Гвинея вакыты (Бугенвиль)',
        'Pacific/Chatham' => 'Чатем вакыты',
        'Pacific/Easter' => 'Пасха утравы вакыты',
        'Pacific/Efate' => 'Вануату вакыты (Эфате)',
        'Pacific/Enderbury' => 'Феникс утраулары вакыты (Кантон)',
        'Pacific/Kanton' => 'Феникс утраулары вакыты (Кантон)',
        'Pacific/Fakaofo' => 'Токелау вакыты (Факаофо)',
        'Pacific/Fiji' => 'Фиджи вакыты',
        'Pacific/Funafuti' => 'Тувалу вакыты (Фунафути)',
        'Pacific/Galapagos' => 'Галапагос утраулары вакыты',
        'Pacific/Gambier' => 'Гамбье вакыты',
        'Pacific/Guadalcanal' => 'Соломон утраулары вакыты (Гуадалканал)',
        'Pacific/Guam' => 'Чаморро стандарт вакыты (Гуам)',
        'Pacific/Honolulu' => 'Гавай-Алеут стандарт вакыты (Honolulu)',
        'Pacific/Kiritimati' => 'Лайн утраулары вакыты (Киритимати)',
        'Pacific/Kosrae' => 'Косраэ вакыты',
        'Pacific/Kwajalein' => 'Маршалл утраулары вакыты (Кваджалейн)',
        'Pacific/Majuro' => 'Маршалл утраулары вакыты (Маджуро)',
        'Pacific/Marquesas' => 'Маркиз утраулары вакыты',
        'Pacific/Midway' => 'Самоа вакыты (Мидуэй)',
        'Pacific/Nauru' => 'Науру вакыты',
        'Pacific/Niue' => 'Ниуэ вакыты',
        'Pacific/Norfolk' => 'Норфолк утравы вакыты',
        'Pacific/Noumea' => 'Яңа Каледония вакыты (Нумеа)',
        'Pacific/Pago_Pago' => 'Самоа вакыты (Паго Паго)',
        'Pacific/Palau' => 'Палау вакыты',
        'Pacific/Pitcairn' => 'Питкэрн вакыты',
        'Pacific/Ponape' => 'Понапе вакыты (Понпеи)',
        'Pacific/Pohnpei' => 'Понапе вакыты (Понпеи)',
        'Pacific/Port_Moresby' => 'Папуа Яңа Гвинея вакыты (Порт-Морсби)',
        'Pacific/Rarotonga' => 'Кук утраулары вакыты (Раротонга)',
        'Pacific/Saipan' => 'Чаморро стандарт вакыты (Сайпан)',
        'Pacific/Tahiti' => 'Таити вакыты',
        'Pacific/Tarawa' => 'Гилберт утраулары вакыты (Тарава)',
        'Pacific/Tongatapu' => 'Тонга вакыты (Тонгатапу)',
        'Pacific/Truk' => 'Чуук вакыты',
        'Pacific/Chuuk' => 'Чуук вакыты',
        'Pacific/Wake' => 'Уэйк утравы вакыты (Вейк)',
        'Pacific/Wallis' => 'Уоллис һәм Футуна вакыты',
    ],
    'Meta' => [],
];
