<?php

return [
    'Names' => [
        'Adlm' => 'адлам',
        'Afak' => 'афака',
        'Arab' => 'арабица',
        'Aran' => 'насталик',
        'Armi' => 'арамейская',
        'Armn' => 'армянская',
        'Avst' => 'авестийская',
        'Bali' => 'балийская',
        'Bamu' => 'бамум',
        'Bass' => 'басса (вах)',
        'Batk' => 'батакская',
        'Beng' => 'бенгальская',
        'Blis' => 'блиссимволика',
        'Bopo' => 'бопомофо',
        'Brah' => 'брахми',
        'Brai' => 'шрифт Брайля',
        'Bugi' => 'бугинизийская',
        'Buhd' => 'бухид',
        'Cakm' => 'чакмийская',
        'Cans' => 'канадское слоговое письмо',
        'Cari' => 'карийская',
        'Cham' => 'чамская',
        'Cher' => 'чероки',
        'Cirt' => 'кирт',
        'Copt' => 'коптская',
        'Cprt' => 'кипрская',
        'Cyrl' => 'кириллица',
        'Cyrs' => 'старославянская',
        'Deva' => 'деванагари',
        'Dsrt' => 'дезерет',
        'Dupl' => 'дуплоянская скоропись',
        'Egyd' => 'египетская демотическая',
        'Egyh' => 'египетская иератическая',
        'Egyp' => 'египетская иероглифическая',
        'Ethi' => 'эфиопская',
        'Geok' => 'грузинская хуцури',
        'Geor' => 'грузинская',
        'Glag' => 'глаголица',
        'Goth' => 'готская',
        'Gran' => 'грантха',
        'Grek' => 'греческая',
        'Gujr' => 'гуджарати',
        'Guru' => 'гурмукхи',
        'Hanb' => 'ханьб',
        'Hang' => 'хангыль',
        'Hani' => 'китайская',
        'Hano' => 'хануну',
        'Hans' => 'упрощенная',
        'Hant' => 'традиционная',
        'Hebr' => 'еврейская',
        'Hira' => 'хирагана',
        'Hluw' => 'лувийские иероглифы',
        'Hmng' => 'пахау хмонг',
        'Hrkt' => 'катакана и хирагана',
        'Hung' => 'старовенгерская',
        'Inds' => 'хараппская (письменность долины Инда)',
        'Ital' => 'староитальянская',
        'Jamo' => 'джамо',
        'Java' => 'яванская',
        'Jpan' => 'японская',
        'Jurc' => 'чжурчжэньская',
        'Kali' => 'кайа',
        'Kana' => 'катакана',
        'Khar' => 'кхароштхи',
        'Khmr' => 'кхмерская',
        'Khoj' => 'ходжики',
        'Knda' => 'каннада',
        'Kore' => 'корейская',
        'Kpel' => 'кпелле',
        'Kthi' => 'кайтхи',
        'Lana' => 'ланна',
        'Laoo' => 'лаосская',
        'Latf' => 'латинская фрактура',
        'Latg' => 'гэльская латинская',
        'Latn' => 'латиница',
        'Lepc' => 'лепха',
        'Limb' => 'лимбу',
        'Lina' => 'линейное письмо А',
        'Linb' => 'линейное письмо Б',
        'Lisu' => 'лису',
        'Loma' => 'лома',
        'Lyci' => 'лициан',
        'Lydi' => 'лидийская',
        'Mand' => 'мандейская',
        'Mani' => 'манихейская',
        'Maya' => 'майя',
        'Mend' => 'менде',
        'Merc' => 'мероитская курсивная',
        'Mero' => 'мероитская',
        'Mlym' => 'малаялам',
        'Mong' => 'монгольская',
        'Moon' => 'азбука муна',
        'Mroo' => 'мро',
        'Mtei' => 'манипури',
        'Mymr' => 'мьянманская',
        'Narb' => 'северноаравийское',
        'Nbat' => 'набатейская',
        'Nkgb' => 'наси геба',
        'Nkoo' => 'нко',
        'Nshu' => 'нюй-шу',
        'Ogam' => 'огамическая',
        'Olck' => 'ол чики',
        'Orkh' => 'орхоно-енисейская',
        'Orya' => 'ория',
        'Osma' => 'османская',
        'Palm' => 'пальмиры',
        'Perm' => 'древнепермская',
        'Phag' => 'пагспа',
        'Phli' => 'пехлевийская',
        'Phlp' => 'пахлави псалтирная',
        'Phlv' => 'пахлави книжная',
        'Phnx' => 'финикийская',
        'Plrd' => 'поллардовская фонетика',
        'Prti' => 'парфянская',
        'Qaag' => 'зоджи',
        'Rjng' => 'реджангская',
        'Rohg' => 'ханифи',
        'Roro' => 'ронго-ронго',
        'Runr' => 'руническая',
        'Samr' => 'самаритянская',
        'Sara' => 'сарати',
        'Sarb' => 'староюжноарабская',
        'Saur' => 'саураштра',
        'Sgnw' => 'язык знаков',
        'Shaw' => 'алфавит Шоу',
        'Shrd' => 'шарада',
        'Sind' => 'кхудавади',
        'Sinh' => 'сингальская',
        'Sora' => 'сора-сонпенг',
        'Sund' => 'сунданская',
        'Sylo' => 'силоти нагри',
        'Syrc' => 'сирийская',
        'Syre' => 'сирийская эстрангело',
        'Syrj' => 'западносирийская',
        'Syrn' => 'восточно-сирийская',
        'Tagb' => 'тагбанва',
        'Takr' => 'такри',
        'Tale' => 'тайский ле',
        'Talu' => 'новый тайский ле',
        'Taml' => 'тамильская',
        'Tang' => 'тангутское меня',
        'Tavt' => 'тай-вьет',
        'Telu' => 'телугу',
        'Teng' => 'тенгварская',
        'Tfng' => 'древнеливийская',
        'Tglg' => 'тагалог',
        'Thaa' => 'тана',
        'Thai' => 'тайская',
        'Tibt' => 'тибетская',
        'Tirh' => 'тирхута',
        'Ugar' => 'угаритская',
        'Vaii' => 'вайская',
        'Visp' => 'видимая речь',
        'Wara' => 'варанг-кшити',
        'Wole' => 'волеаи',
        'Xpeo' => 'староперсидская',
        'Xsux' => 'шумеро-аккадская клинопись',
        'Yiii' => 'и',
        'Zinh' => 'унаследованная',
        'Zmth' => 'математические обозначения',
        'Zsye' => 'эмодзи',
        'Zsym' => 'символы',
        'Zxxx' => 'нет письменности',
        'Zyyy' => 'общепринятая',
    ],
];
