{% extends 'base.html.twig' %}

{% block body %}
    Hello {{ app.user.userIdentifier }}!<br /><br />
    You're browsing to path "{{ app.request.pathInfo }}".<br /><br />
    <a href="{{ logout_path('default') }}">Log out</a>.
    <a href="{{ logout_url('default') }}">Log out</a>.
{% endblock %}
