<?php

return [
    'Names' => [
        'Africa/Abidjan' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Abidjan)',
        'Africa/Accra' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Accra)',
        'Africa/Addis_Ababa' => 'Sa’aatii Baha Afrikaa (Addis Ababa)',
        'Africa/Algiers' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Algiers)',
        'Africa/Asmera' => 'Sa’aatii Baha Afrikaa (Asmara)',
        'Africa/Asmara' => 'Sa’aatii Baha Afrikaa (Asmara)',
        'Africa/Bamako' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Bamako)',
        'Africa/Bangui' => 'Sa’aatii Afrikaa Dhihaa (Bangui)',
        'Africa/Banjul' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Banjul)',
        'Africa/Bissau' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Bissau)',
        'Africa/Blantyre' => 'Sa’aatii Afrikaa Gidduugaleessaa (Blantyre)',
        'Africa/Brazzaville' => 'Sa’aatii Afrikaa Dhihaa (Brazzaville)',
        'Africa/Bujumbura' => 'Sa’aatii Afrikaa Gidduugaleessaa (Bujumbura)',
        'Africa/Cairo' => 'Saaatii Awurooppaa Bahaa (Cairo)',
        'Africa/Casablanca' => 'Sa’aatii Awurooppaa Dhihaa (Casablanca)',
        'Africa/Ceuta' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Ceuta)',
        'Africa/Conakry' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Conakry)',
        'Africa/Dakar' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Dakar)',
        'Africa/Dar_es_Salaam' => 'Sa’aatii Baha Afrikaa (Dar es Salaam)',
        'Africa/Djibouti' => 'Sa’aatii Baha Afrikaa (Djibouti)',
        'Africa/Douala' => 'Sa’aatii Afrikaa Dhihaa (Douala)',
        'Africa/El_Aaiun' => 'Sa’aatii Awurooppaa Dhihaa (El Aaiún)',
        'Africa/Freetown' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Freetown)',
        'Africa/Gaborone' => 'Sa’aatii Afrikaa Gidduugaleessaa (Gaborone)',
        'Africa/Harare' => 'Sa’aatii Afrikaa Gidduugaleessaa (Harare)',
        'Africa/Johannesburg' => 'Sa’aatii Istaandaardii Afrikaa Kibbaa (Johannesburg)',
        'Africa/Juba' => 'Sa’aatii Afrikaa Gidduugaleessaa (Juba)',
        'Africa/Kampala' => 'Sa’aatii Baha Afrikaa (Kampala)',
        'Africa/Khartoum' => 'Sa’aatii Afrikaa Gidduugaleessaa (Khartoum)',
        'Africa/Kigali' => 'Sa’aatii Afrikaa Gidduugaleessaa (Kigali)',
        'Africa/Kinshasa' => 'Sa’aatii Afrikaa Dhihaa (Kinshasa)',
        'Africa/Lagos' => 'Sa’aatii Afrikaa Dhihaa (Lagos)',
        'Africa/Libreville' => 'Sa’aatii Afrikaa Dhihaa (Libreville)',
        'Africa/Lome' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Lomé)',
        'Africa/Luanda' => 'Sa’aatii Afrikaa Dhihaa (Luanda)',
        'Africa/Lubumbashi' => 'Sa’aatii Afrikaa Gidduugaleessaa (Lubumbashi)',
        'Africa/Lusaka' => 'Sa’aatii Afrikaa Gidduugaleessaa (Lusaka)',
        'Africa/Malabo' => 'Sa’aatii Afrikaa Dhihaa (Malabo)',
        'Africa/Maputo' => 'Sa’aatii Afrikaa Gidduugaleessaa (Maputo)',
        'Africa/Maseru' => 'Sa’aatii Istaandaardii Afrikaa Kibbaa (Maseru)',
        'Africa/Mbabane' => 'Sa’aatii Istaandaardii Afrikaa Kibbaa (Mbabane)',
        'Africa/Mogadishu' => 'Sa’aatii Baha Afrikaa (Mogadishu)',
        'Africa/Monrovia' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Monrovia)',
        'Africa/Nairobi' => 'Sa’aatii Baha Afrikaa (Nairobi)',
        'Africa/Ndjamena' => 'Sa’aatii Afrikaa Dhihaa (N’Djamena)',
        'Africa/Niamey' => 'Sa’aatii Afrikaa Dhihaa (Niamey)',
        'Africa/Nouakchott' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Nouakchott)',
        'Africa/Ouagadougou' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Ouagadougou)',
        'Africa/Porto-Novo' => 'Sa’aatii Afrikaa Dhihaa (Porto-Novo)',
        'Africa/Sao_Tome' => 'Sa’aatii Giriinwiich Gidduugaleessaa (São Tomé)',
        'Africa/Tripoli' => 'Saaatii Awurooppaa Bahaa (Tripoli)',
        'Africa/Tunis' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Tunis)',
        'Africa/Windhoek' => 'Sa’aatii Afrikaa Gidduugaleessaa (Windhoek)',
        'America/Adak' => 'Sa’aatii Haawayi-Alewutiyan (Adak)',
        'America/Anchorage' => 'Sa’aatii Alaaskaa (Anchorage)',
        'America/Anguilla' => 'Sa’aatii Atilaantiik (Anguilla)',
        'America/Antigua' => 'Sa’aatii Atilaantiik (Antigua)',
        'America/Araguaina' => 'Sa’aatii Biraaziliyaa (Araguaína)',
        'America/Argentina/La_Rioja' => 'Sa’aatii Arjentiinaa (La Rioja)',
        'America/Argentina/Rio_Gallegos' => 'Sa’aatii Arjentiinaa (Río Gallegos)',
        'America/Argentina/Salta' => 'Sa’aatii Arjentiinaa (Salta)',
        'America/Argentina/San_Juan' => 'Sa’aatii Arjentiinaa (San Juan)',
        'America/Argentina/San_Luis' => 'Sa’aatii Arjentiinaa (San Luis)',
        'America/Argentina/Tucuman' => 'Sa’aatii Arjentiinaa (Tucumán)',
        'America/Argentina/Ushuaia' => 'Sa’aatii Arjentiinaa (Ushuaia)',
        'America/Aruba' => 'Sa’aatii Atilaantiik (Aruba)',
        'America/Asuncion' => 'Sa’aatii Paaraaguwaayi (Asunción)',
        'America/Bahia' => 'Sa’aatii Biraaziliyaa (Bahia)',
        'America/Bahia_Banderas' => 'Sa’aatii Gidduugaleessaa (Bahía de Banderas)',
        'America/Barbados' => 'Sa’aatii Atilaantiik (Barbados)',
        'America/Belem' => 'Sa’aatii Biraaziliyaa (Belém)',
        'America/Belize' => 'Sa’aatii Gidduugaleessaa (Belize)',
        'America/Blanc-Sablon' => 'Sa’aatii Atilaantiik (Blanc-Sablon)',
        'America/Boa_Vista' => 'Sa’aatii Amazoon (Boa Vista)',
        'America/Bogota' => 'Sa’aatii Kolombiyaa (Bogotá)',
        'America/Boise' => 'Sa’aatii Maawonteen (Boise)',
        'America/Buenos_Aires' => 'Sa’aatii Arjentiinaa (Buenos Aires)',
        'America/Argentina/Buenos_Aires' => 'Sa’aatii Arjentiinaa (Buenos Aires)',
        'America/Cambridge_Bay' => 'Sa’aatii Maawonteen (Cambridge Bay)',
        'America/Campo_Grande' => 'Sa’aatii Amazoon (Campo Grande)',
        'America/Cancun' => 'Sa’aatii Bahaa (Cancún)',
        'America/Caracas' => 'Sa’aatii Veenzuweelaa (Caracas)',
        'America/Catamarca' => 'Sa’aatii Arjentiinaa (Catamarca)',
        'America/Argentina/Catamarca' => 'Sa’aatii Arjentiinaa (Catamarca)',
        'America/Cayenne' => 'Sa’aatii Fireench Guyinaa (Cayenne)',
        'America/Cayman' => 'Sa’aatii Bahaa (Cayman)',
        'America/Chicago' => 'Sa’aatii Gidduugaleessaa (Chicago)',
        'America/Chihuahua' => 'Sa’aatii Gidduugaleessaa (Chihuahua)',
        'America/Ciudad_Juarez' => 'Sa’aatii Maawonteen (Ciudad Juárez)',
        'America/Coral_Harbour' => 'Sa’aatii Bahaa (Atikokan)',
        'America/Atikokan' => 'Sa’aatii Bahaa (Atikokan)',
        'America/Cordoba' => 'Sa’aatii Arjentiinaa (Córdoba)',
        'America/Argentina/Cordoba' => 'Sa’aatii Arjentiinaa (Córdoba)',
        'America/Costa_Rica' => 'Sa’aatii Gidduugaleessaa (Costa Rica)',
        'America/Coyhaique' => 'Sa’aatii Chiilii (Coyhaique)',
        'America/Creston' => 'Sa’aatii Maawonteen (Creston)',
        'America/Cuiaba' => 'Sa’aatii Amazoon (Cuiabá)',
        'America/Curacao' => 'Sa’aatii Atilaantiik (Curaçao)',
        'America/Danmarkshavn' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Danmarkshavn)',
        'America/Dawson' => 'Sa’aatii Yuukoon (Dawson)',
        'America/Dawson_Creek' => 'Sa’aatii Maawonteen (Dawson Creek)',
        'America/Denver' => 'Sa’aatii Maawonteen (Denver)',
        'America/Detroit' => 'Sa’aatii Bahaa (Detroit)',
        'America/Dominica' => 'Sa’aatii Atilaantiik (Dominica)',
        'America/Edmonton' => 'Sa’aatii Maawonteen (Edmonton)',
        'America/Eirunepe' => 'Sa’aatii Biraazil (Eirunepé)',
        'America/El_Salvador' => 'Sa’aatii Gidduugaleessaa (El Salvador)',
        'America/Fort_Nelson' => 'Sa’aatii Maawonteen (Fort Nelson)',
        'America/Fortaleza' => 'Sa’aatii Biraaziliyaa (Fortaleza)',
        'America/Glace_Bay' => 'Sa’aatii Atilaantiik (Glace Bay)',
        'America/Godthab' => 'Sa’aatii Giriinlaand (Nuuk)',
        'America/Nuuk' => 'Sa’aatii Giriinlaand (Nuuk)',
        'America/Goose_Bay' => 'Sa’aatii Atilaantiik (Goose Bay)',
        'America/Grand_Turk' => 'Sa’aatii Bahaa (Grand Turk)',
        'America/Grenada' => 'Sa’aatii Atilaantiik (Grenada)',
        'America/Guadeloupe' => 'Sa’aatii Atilaantiik (Guadeloupe)',
        'America/Guatemala' => 'Sa’aatii Gidduugaleessaa (Guatemala)',
        'America/Guayaquil' => 'Sa’aatii Ikkuwaadoor (Guayaquil)',
        'America/Guyana' => 'Sa’aatii Guyaanaa (Guyana)',
        'America/Halifax' => 'Sa’aatii Atilaantiik (Halifax)',
        'America/Havana' => 'Sa’aatii Kuubaa (Havana)',
        'America/Hermosillo' => 'Sa’aatii Paasfiik Meksiikaan (Hermosillo)',
        'America/Indiana/Knox' => 'Sa’aatii Gidduugaleessaa (Knox, Indiana)',
        'America/Indiana/Marengo' => 'Sa’aatii Bahaa (Marengo, Indiana)',
        'America/Indiana/Petersburg' => 'Sa’aatii Bahaa (Petersburg, Indiana)',
        'America/Indiana/Tell_City' => 'Sa’aatii Gidduugaleessaa (Tell City, Indiana)',
        'America/Indiana/Vevay' => 'Sa’aatii Bahaa (Vevay, Indiana)',
        'America/Indiana/Vincennes' => 'Sa’aatii Bahaa (Vincennes, Indiana)',
        'America/Indiana/Winamac' => 'Sa’aatii Bahaa (Winamac, Indiana)',
        'America/Indianapolis' => 'Sa’aatii Bahaa (Indianapolis)',
        'America/Indiana/Indianapolis' => 'Sa’aatii Bahaa (Indianapolis)',
        'America/Inuvik' => 'Sa’aatii Maawonteen (Inuvik)',
        'America/Iqaluit' => 'Sa’aatii Bahaa (Iqaluit)',
        'America/Jamaica' => 'Sa’aatii Bahaa (Jamaica)',
        'America/Jujuy' => 'Sa’aatii Arjentiinaa (Jujuy)',
        'America/Argentina/Jujuy' => 'Sa’aatii Arjentiinaa (Jujuy)',
        'America/Juneau' => 'Sa’aatii Alaaskaa (Juneau)',
        'America/Kentucky/Monticello' => 'Sa’aatii Bahaa (Monticello, Kentucky)',
        'America/Kralendijk' => 'Sa’aatii Atilaantiik (Kralendijk)',
        'America/La_Paz' => 'Sa’aatii Boliiviyaa (La Paz)',
        'America/Lima' => 'Sa’aatii Peeruu (Lima)',
        'America/Los_Angeles' => 'Sa’aatii Paasfiik (Los Angeles)',
        'America/Louisville' => 'Sa’aatii Bahaa (Louisville)',
        'America/Kentucky/Louisville' => 'Sa’aatii Bahaa (Louisville)',
        'America/Lower_Princes' => 'Sa’aatii Atilaantiik (Lower Prince’s Quarter)',
        'America/Maceio' => 'Sa’aatii Biraaziliyaa (Maceió)',
        'America/Managua' => 'Sa’aatii Gidduugaleessaa (Managua)',
        'America/Manaus' => 'Sa’aatii Amazoon (Manaus)',
        'America/Marigot' => 'Sa’aatii Atilaantiik (Marigot)',
        'America/Martinique' => 'Sa’aatii Atilaantiik (Martinique)',
        'America/Matamoros' => 'Sa’aatii Gidduugaleessaa (Matamoros)',
        'America/Mazatlan' => 'Sa’aatii Paasfiik Meksiikaan (Mazatlán)',
        'America/Mendoza' => 'Sa’aatii Arjentiinaa (Mendoza)',
        'America/Argentina/Mendoza' => 'Sa’aatii Arjentiinaa (Mendoza)',
        'America/Menominee' => 'Sa’aatii Gidduugaleessaa (Menominee)',
        'America/Merida' => 'Sa’aatii Gidduugaleessaa (Mérida)',
        'America/Metlakatla' => 'Sa’aatii Alaaskaa (Metlakatla)',
        'America/Mexico_City' => 'Sa’aatii Gidduugaleessaa (Ciudad de México)',
        'America/Miquelon' => 'Sa’aatii Ql. Piyeeree fi Mikuyelo (Saint-Pierre)',
        'America/Moncton' => 'Sa’aatii Atilaantiik (Moncton)',
        'America/Monterrey' => 'Sa’aatii Gidduugaleessaa (Monterrey)',
        'America/Montevideo' => 'Sa’aatii Yuraagaayi (Montevideo)',
        'America/Montserrat' => 'Sa’aatii Atilaantiik (Montserrat)',
        'America/Nassau' => 'Sa’aatii Bahaa (Nassau)',
        'America/New_York' => 'Sa’aatii Bahaa (New York)',
        'America/Nome' => 'Sa’aatii Alaaskaa (Nome)',
        'America/Noronha' => 'Sa’aatii Fernando de Noronha',
        'America/North_Dakota/Beulah' => 'Sa’aatii Gidduugaleessaa (Beulah, North Dakota)',
        'America/North_Dakota/Center' => 'Sa’aatii Gidduugaleessaa (Center, North Dakota)',
        'America/North_Dakota/New_Salem' => 'Sa’aatii Gidduugaleessaa (New Salem, North Dakota)',
        'America/Ojinaga' => 'Sa’aatii Gidduugaleessaa (Ojinaga)',
        'America/Panama' => 'Sa’aatii Bahaa (Panama)',
        'America/Paramaribo' => 'Sa’aatii Surinaame (Paramaribo)',
        'America/Phoenix' => 'Sa’aatii Maawonteen (Phoenix)',
        'America/Port-au-Prince' => 'Sa’aatii Bahaa (Port-au-Prince)',
        'America/Port_of_Spain' => 'Sa’aatii Atilaantiik (Port of Spain)',
        'America/Porto_Velho' => 'Sa’aatii Amazoon (Porto Velho)',
        'America/Puerto_Rico' => 'Sa’aatii Atilaantiik (Puerto Rico)',
        'America/Punta_Arenas' => 'Sa’aatii Chiilii (Punta Arenas)',
        'America/Rankin_Inlet' => 'Sa’aatii Gidduugaleessaa (Rankin Inlet)',
        'America/Recife' => 'Sa’aatii Biraaziliyaa (Recife)',
        'America/Regina' => 'Sa’aatii Gidduugaleessaa (Regina)',
        'America/Resolute' => 'Sa’aatii Gidduugaleessaa (Resolute)',
        'America/Rio_Branco' => 'Sa’aatii Biraazil (Rio Branco)',
        'America/Santarem' => 'Sa’aatii Biraaziliyaa (Santarém)',
        'America/Santiago' => 'Sa’aatii Chiilii (Santiago)',
        'America/Santo_Domingo' => 'Sa’aatii Atilaantiik (Santo Domingo)',
        'America/Sao_Paulo' => 'Sa’aatii Biraaziliyaa (São Paulo)',
        'America/Scoresbysund' => 'Sa’aatii Giriinlaand (Ittoqqortoormiit)',
        'America/Sitka' => 'Sa’aatii Alaaskaa (Sitka)',
        'America/St_Barthelemy' => 'Sa’aatii Atilaantiik (St. Barthélemy)',
        'America/St_Johns' => 'Sa’aatii Newufaawondlaand (St. John’s)',
        'America/St_Kitts' => 'Sa’aatii Atilaantiik (St. Kitts)',
        'America/St_Lucia' => 'Sa’aatii Atilaantiik (St. Lucia)',
        'America/St_Thomas' => 'Sa’aatii Atilaantiik (St. Thomas)',
        'America/St_Vincent' => 'Sa’aatii Atilaantiik (St. Vincent)',
        'America/Swift_Current' => 'Sa’aatii Gidduugaleessaa (Swift Current)',
        'America/Tegucigalpa' => 'Sa’aatii Gidduugaleessaa (Tegucigalpa)',
        'America/Thule' => 'Sa’aatii Atilaantiik (Thule)',
        'America/Tijuana' => 'Sa’aatii Paasfiik (Tijuana)',
        'America/Toronto' => 'Sa’aatii Bahaa (Toronto)',
        'America/Tortola' => 'Sa’aatii Atilaantiik (Tortola)',
        'America/Vancouver' => 'Sa’aatii Paasfiik (Vancouver)',
        'America/Whitehorse' => 'Sa’aatii Yuukoon (Whitehorse)',
        'America/Winnipeg' => 'Sa’aatii Gidduugaleessaa (Winnipeg)',
        'America/Yakutat' => 'Sa’aatii Alaaskaa (Yakutat)',
        'Antarctica/Casey' => 'Sa’aatii Awustiraaliyaa Dhihaa (Casey)',
        'Antarctica/Davis' => 'Sa’aatii Daaviis (Davis)',
        'Antarctica/DumontDUrville' => 'Sa’aatii Dumont-d’Urville',
        'Antarctica/Macquarie' => 'Sa’aatii Awustiraaliyaa Bahaa (Macquarie)',
        'Antarctica/Mawson' => 'Sa’aatii Mawson',
        'Antarctica/McMurdo' => 'Sa’aatii New Zealand (McMurdo)',
        'Antarctica/Palmer' => 'Sa’aatii Chiilii (Palmer)',
        'Antarctica/Rothera' => 'Sa’aatii Rothera',
        'Antarctica/Syowa' => 'Sa’aatii Syowa (Showa)',
        'Antarctica/Troll' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Troll)',
        'Antarctica/Vostok' => 'Sa’aatii Vostok',
        'Arctic/Longyearbyen' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Longyearbyen)',
        'Asia/Aden' => 'Sa’aatii Arabaa (Aden)',
        'Asia/Almaty' => 'Sa’aatii Kaazaakistaan (Almaty)',
        'Asia/Amman' => 'Saaatii Awurooppaa Bahaa (Amman)',
        'Asia/Anadyr' => 'Sa’aatii Raashiyaa (Anadyr)',
        'Asia/Aqtau' => 'Sa’aatii Kaazaakistaan (Aqtau)',
        'Asia/Aqtobe' => 'Sa’aatii Kaazaakistaan (Aqtobe)',
        'Asia/Ashgabat' => 'Sa’aatii Turkemenistaan (Ashgabat)',
        'Asia/Atyrau' => 'Sa’aatii Kaazaakistaan (Atyrau)',
        'Asia/Baghdad' => 'Sa’aatii Arabaa (Baghdad)',
        'Asia/Bahrain' => 'Sa’aatii Arabaa (Bahrain)',
        'Asia/Baku' => 'Sa’aatii Azerbaajiyaan (Baku)',
        'Asia/Bangkok' => 'Sa’aatii IndooChaayinaa (Bangkok)',
        'Asia/Barnaul' => 'Sa’aatii Krasnoyarsk (Barnaul)',
        'Asia/Beirut' => 'Saaatii Awurooppaa Bahaa (Beirut)',
        'Asia/Bishkek' => 'Sa’aatii Kiyirigiyistan (Bishkek)',
        'Asia/Brunei' => 'Sa’aatii Bruunee Darusalaam (Brunei)',
        'Asia/Calcutta' => 'Sa’aatii Istaandaardii Hindii (Kolkata)',
        'Asia/Kolkata' => 'Sa’aatii Istaandaardii Hindii (Kolkata)',
        'Asia/Chita' => 'Sa’aatii Yakutsk (Chita)',
        'Asia/Colombo' => 'Sa’aatii Istaandaardii Hindii (Colombo)',
        'Asia/Damascus' => 'Saaatii Awurooppaa Bahaa (Damascus)',
        'Asia/Dhaka' => 'Sa’aatii Baangilaadish (Dhaka)',
        'Asia/Dili' => 'Sa’aatii Tiimoor Bahaa (Dili)',
        'Asia/Dubai' => 'Sa’aatii Istaandaardii Gaalfii (Dubai)',
        'Asia/Dushanbe' => 'Sa’aatii Tajikistaan (Dushanbe)',
        'Asia/Famagusta' => 'Saaatii Awurooppaa Bahaa (Famagusta)',
        'Asia/Gaza' => 'Saaatii Awurooppaa Bahaa (Gaza)',
        'Asia/Hebron' => 'Saaatii Awurooppaa Bahaa (Hebron)',
        'Asia/Hong_Kong' => 'Sa’aatii Hoong Koong (Hong Kong)',
        'Asia/Hovd' => 'Sa’aatii Hoovd (Khovd)',
        'Asia/Irkutsk' => 'Sa’aatii Irkutsk',
        'Asia/Jakarta' => 'Sa’aatii Indooneeshiyaa Dhihaa (Jakarta)',
        'Asia/Jayapura' => 'Sa’aatii Indooneshiyaa Bahaa (Jayapura)',
        'Asia/Jerusalem' => 'Sa’aatii Israa’eel (Jerusalem)',
        'Asia/Kabul' => 'Sa’aatii Afgaanistaan (Kabul)',
        'Asia/Kamchatka' => 'Sa’aatii Raashiyaa (Kamchatka)',
        'Asia/Karachi' => 'Sa’aatii Paakistaan (Karachi)',
        'Asia/Katmandu' => 'Sa’aatii Neeppaal (Kathmandu)',
        'Asia/Kathmandu' => 'Sa’aatii Neeppaal (Kathmandu)',
        'Asia/Khandyga' => 'Sa’aatii Yakutsk (Khandyga)',
        'Asia/Krasnoyarsk' => 'Sa’aatii Krasnoyarsk',
        'Asia/Kuala_Lumpur' => 'Sa’aatii Maaleeshiyaa (Kuala Lumpur)',
        'Asia/Kuching' => 'Sa’aatii Maaleeshiyaa (Kuching)',
        'Asia/Kuwait' => 'Sa’aatii Arabaa (Kuwait)',
        'Asia/Macau' => 'Sa’aatii Chaayinaa (Macao)',
        'Asia/Magadan' => 'Sa’aatii Magadan',
        'Asia/Makassar' => 'Sa’aatii Indooneeshiyaa Gidduugaleessaa (Makassar)',
        'Asia/Manila' => 'Sa’aatii Filippiins (Manila)',
        'Asia/Muscat' => 'Sa’aatii Istaandaardii Gaalfii (Muscat)',
        'Asia/Nicosia' => 'Saaatii Awurooppaa Bahaa (Nicosia)',
        'Asia/Novokuznetsk' => 'Sa’aatii Krasnoyarsk (Novokuznetsk)',
        'Asia/Novosibirsk' => 'Sa’aatii Krasnoyarsk (Novosibirsk)',
        'Asia/Omsk' => 'Sa’aatii Omsk',
        'Asia/Oral' => 'Sa’aatii Kaazaakistaan (Oral)',
        'Asia/Phnom_Penh' => 'Sa’aatii IndooChaayinaa (Phnom Penh)',
        'Asia/Pontianak' => 'Sa’aatii Indooneeshiyaa Dhihaa (Pontianak)',
        'Asia/Pyongyang' => 'Sa’aatii Kooriyaa (Pyongyang)',
        'Asia/Qatar' => 'Sa’aatii Arabaa (Qatar)',
        'Asia/Qostanay' => 'Sa’aatii Kaazaakistaan (Qostanay)',
        'Asia/Qyzylorda' => 'Sa’aatii Kaazaakistaan (Qyzylorda)',
        'Asia/Rangoon' => 'Sa’aatii Maayinaamaar (Yangon)',
        'Asia/Yangon' => 'Sa’aatii Maayinaamaar (Yangon)',
        'Asia/Riyadh' => 'Sa’aatii Arabaa (Riyadh)',
        'Asia/Saigon' => 'Sa’aatii IndooChaayinaa (Ho Chi Minh)',
        'Asia/Ho_Chi_Minh' => 'Sa’aatii IndooChaayinaa (Ho Chi Minh)',
        'Asia/Sakhalin' => 'Sa’aatii Magadan (Sakhalin)',
        'Asia/Samarkand' => 'Sa’aatii Uzbeekistaan (Samarkand)',
        'Asia/Seoul' => 'Sa’aatii Kooriyaa (Seoul)',
        'Asia/Shanghai' => 'Sa’aatii Chaayinaa (Shanghai)',
        'Asia/Singapore' => 'Sa’aatii Istaandaardii Singaapoor (Singapore)',
        'Asia/Srednekolymsk' => 'Sa’aatii Magadan (Srednekolymsk)',
        'Asia/Taipei' => 'Sa’aatii Tayipeyi (Taipei)',
        'Asia/Tashkent' => 'Sa’aatii Uzbeekistaan (Tashkent)',
        'Asia/Tbilisi' => 'Sa’aatii Joorjiyaa (Tbilisi)',
        'Asia/Tehran' => 'Sa’aatii Iraan (Tehran)',
        'Asia/Thimphu' => 'Sa’aatii Bihutaan (Thimphu)',
        'Asia/Tokyo' => 'Sa’aatii Jaappaan (Tokyo)',
        'Asia/Tomsk' => 'Sa’aatii Krasnoyarsk (Tomsk)',
        'Asia/Ulaanbaatar' => 'Sa’aatii Ulaanbaatar',
        'Asia/Urumqi' => 'Sa’aatii Chaayinaa (Ürümqi)',
        'Asia/Ust-Nera' => 'Sa’aatii Vladivostok (Ust-Nera)',
        'Asia/Vientiane' => 'Sa’aatii IndooChaayinaa (Vientiane)',
        'Asia/Vladivostok' => 'Sa’aatii Vladivostok',
        'Asia/Yakutsk' => 'Sa’aatii Yakutsk',
        'Asia/Yekaterinburg' => 'Sa’aatii Yekaterinburg',
        'Asia/Yerevan' => 'Sa’aatii Armaaniyaa (Yerevan)',
        'Atlantic/Azores' => 'Sa’aatii Azeeroos (Azores)',
        'Atlantic/Bermuda' => 'Sa’aatii Atilaantiik (Bermuda)',
        'Atlantic/Canary' => 'Sa’aatii Awurooppaa Dhihaa (Canarias)',
        'Atlantic/Cape_Verde' => 'Sa’aatii Keep Veerdee (Cape Verde)',
        'Atlantic/Faeroe' => 'Sa’aatii Awurooppaa Dhihaa (Faroe)',
        'Atlantic/Faroe' => 'Sa’aatii Awurooppaa Dhihaa (Faroe)',
        'Atlantic/Madeira' => 'Sa’aatii Awurooppaa Dhihaa (Madeira)',
        'Atlantic/Reykjavik' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Reykjavik)',
        'Atlantic/South_Georgia' => 'Sa’aatii Joorjiyaa Kibbaa (South Georgia)',
        'Atlantic/St_Helena' => 'Sa’aatii Giriinwiich Gidduugaleessaa (St. Helena)',
        'Atlantic/Stanley' => 'Sa’aatii Odoloota Faalklaand (Stanley)',
        'Australia/Adelaide' => 'Sa’aatii Awustiraaliyaa Gidduugaleessaa (Adelaide)',
        'Australia/Brisbane' => 'Sa’aatii Awustiraaliyaa Bahaa (Brisbane)',
        'Australia/Broken_Hill' => 'Sa’aatii Awustiraaliyaa Gidduugaleessaa (Broken Hill)',
        'Australia/Darwin' => 'Sa’aatii Awustiraaliyaa Gidduugaleessaa (Darwin)',
        'Australia/Eucla' => 'Sa’aatii Dhiha Awustiraaliyaa Gidduugaleessaa (Eucla)',
        'Australia/Hobart' => 'Sa’aatii Awustiraaliyaa Bahaa (Hobart)',
        'Australia/Lindeman' => 'Sa’aatii Awustiraaliyaa Bahaa (Lindeman)',
        'Australia/Lord_Howe' => 'Sa’aatii Lord Howe',
        'Australia/Melbourne' => 'Sa’aatii Awustiraaliyaa Bahaa (Melbourne)',
        'Australia/Perth' => 'Sa’aatii Awustiraaliyaa Dhihaa (Perth)',
        'Australia/Sydney' => 'Sa’aatii Awustiraaliyaa Bahaa (Sydney)',
        'Etc/GMT' => 'Sa’aatii Giriinwiich Gidduugaleessaa',
        'Etc/UTC' => 'Sa’aatii Idil-Addunyaa Qindaa’e',
        'Europe/Amsterdam' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Amsterdam)',
        'Europe/Andorra' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Andorra)',
        'Europe/Astrakhan' => 'Sa’aatii Raashiyaa (Astrakhan)',
        'Europe/Athens' => 'Saaatii Awurooppaa Bahaa (Athens)',
        'Europe/Belgrade' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Belgrade)',
        'Europe/Berlin' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Berlin)',
        'Europe/Bratislava' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Bratislava)',
        'Europe/Brussels' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Brussels)',
        'Europe/Bucharest' => 'Saaatii Awurooppaa Bahaa (Bucharest)',
        'Europe/Budapest' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Budapest)',
        'Europe/Busingen' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Büsingen)',
        'Europe/Chisinau' => 'Saaatii Awurooppaa Bahaa (Chișinău)',
        'Europe/Copenhagen' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Copenhagen)',
        'Europe/Dublin' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Dublin)',
        'Europe/Gibraltar' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Gibraltar)',
        'Europe/Guernsey' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Guernsey)',
        'Europe/Helsinki' => 'Saaatii Awurooppaa Bahaa (Helsinki)',
        'Europe/Isle_of_Man' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Isle of Man)',
        'Europe/Istanbul' => 'Sa’aatii Tarkiye (Istanbul)',
        'Europe/Jersey' => 'Sa’aatii Giriinwiich Gidduugaleessaa (Jersey)',
        'Europe/Kaliningrad' => 'Saaatii Awurooppaa Bahaa (Kaliningrad)',
        'Europe/Kiev' => 'Saaatii Awurooppaa Bahaa (Kyiv)',
        'Europe/Kyiv' => 'Saaatii Awurooppaa Bahaa (Kyiv)',
        'Europe/Kirov' => 'Sa’aatii Mooskoo (Kirov)',
        'Europe/Lisbon' => 'Sa’aatii Awurooppaa Dhihaa (Lisbon)',
        'Europe/Ljubljana' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Ljubljana)',
        'Europe/London' => 'Sa’aatii Giriinwiich Gidduugaleessaa (London)',
        'Europe/Luxembourg' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Luxembourg)',
        'Europe/Madrid' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Madrid)',
        'Europe/Malta' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Malta)',
        'Europe/Mariehamn' => 'Saaatii Awurooppaa Bahaa (Mariehamn)',
        'Europe/Minsk' => 'Sa’aatii Mooskoo (Minsk)',
        'Europe/Monaco' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Monaco)',
        'Europe/Moscow' => 'Sa’aatii Mooskoo (Moscow)',
        'Europe/Oslo' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Oslo)',
        'Europe/Paris' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Paris)',
        'Europe/Podgorica' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Podgorica)',
        'Europe/Prague' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Prague)',
        'Europe/Riga' => 'Saaatii Awurooppaa Bahaa (Riga)',
        'Europe/Rome' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Rome)',
        'Europe/Samara' => 'Sa’aatii Raashiyaa (Samara)',
        'Europe/San_Marino' => 'Sa’aatii Awurooppaa Gidduugaleessaa (San Marino)',
        'Europe/Sarajevo' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Sarajevo)',
        'Europe/Saratov' => 'Sa’aatii Raashiyaa (Saratov)',
        'Europe/Simferopol' => 'Sa’aatii Mooskoo (Simferopol)',
        'Europe/Skopje' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Skopje)',
        'Europe/Sofia' => 'Saaatii Awurooppaa Bahaa (Sofia)',
        'Europe/Stockholm' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Stockholm)',
        'Europe/Tallinn' => 'Saaatii Awurooppaa Bahaa (Tallinn)',
        'Europe/Tirane' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Tirana)',
        'Europe/Ulyanovsk' => 'Sa’aatii Raashiyaa (Ulyanovsk)',
        'Europe/Vaduz' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Vaduz)',
        'Europe/Vatican' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Vatican)',
        'Europe/Vienna' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Vienna)',
        'Europe/Vilnius' => 'Saaatii Awurooppaa Bahaa (Vilnius)',
        'Europe/Volgograd' => 'Sa’aatii Mooskoo (Volgograd)',
        'Europe/Warsaw' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Warsaw)',
        'Europe/Zagreb' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Zagreb)',
        'Europe/Zurich' => 'Sa’aatii Awurooppaa Gidduugaleessaa (Zurich)',
        'Indian/Antananarivo' => 'Sa’aatii Baha Afrikaa (Antananarivo)',
        'Indian/Chagos' => 'Sa’aatii Galaana Hindii (Chagos)',
        'Indian/Christmas' => 'Sa’aatii Odola Kirismaas (Christmas)',
        'Indian/Cocos' => 'Sa’aatii Odoloota Kokos (Cocos)',
        'Indian/Comoro' => 'Sa’aatii Baha Afrikaa (Comores)',
        'Indian/Kerguelen' => 'Sa’aatii Firaans Kibbaa fi Antaarktikaa (Kerguelen)',
        'Indian/Mahe' => 'Sa’aatii Siisheels (Mahé)',
        'Indian/Maldives' => 'Sa’aatii Maaldiivs (Maldives)',
        'Indian/Mauritius' => 'Sa’aatii Mooriishiyees (Mauritius)',
        'Indian/Mayotte' => 'Sa’aatii Baha Afrikaa (Mayotte)',
        'Indian/Reunion' => 'Sa’aatii Riiyuuniyeen (Réunion)',
        'Pacific/Apia' => 'Sa’aatii Apia',
        'Pacific/Auckland' => 'Sa’aatii New Zealand (Auckland)',
        'Pacific/Bougainville' => 'Sa’aatii Paapuwaa Giinii Haaraa (Bougainville)',
        'Pacific/Chatham' => 'Sa’aatii Chatham',
        'Pacific/Easter' => 'Sa’aatii Odola Bahaa (Easter)',
        'Pacific/Efate' => 'Sa’aatii Vanuwatu (Efate)',
        'Pacific/Enderbury' => 'Sa’aatii Odoloota Fooneeks (Canton)',
        'Pacific/Kanton' => 'Sa’aatii Odoloota Fooneeks (Canton)',
        'Pacific/Fakaofo' => 'Sa’aatii Takelawu (Fakaofo)',
        'Pacific/Fiji' => 'Sa’aatii Fiijii (Fiji)',
        'Pacific/Funafuti' => 'Sa’aatii Tuvalu (Funafuti)',
        'Pacific/Galapagos' => 'Sa’aatii Galaapagoos (Galápagos)',
        'Pacific/Gambier' => 'Sa’aatii Gaambiyeer (Gambier)',
        'Pacific/Guadalcanal' => 'Sa’aatii Odoloota Solomoon (Guadalcanal)',
        'Pacific/Guam' => 'Sa’aatii Istaandaardii Kamoroo (Guam)',
        'Pacific/Honolulu' => 'Sa’aatii Istaandaardii Haawayi-Alewutiyan (Honolulu)',
        'Pacific/Kiritimati' => 'Sa’aatii Odoloota Line (Kiritimati)',
        'Pacific/Kosrae' => 'Sa’aatii Koosreyaa (Kosrae)',
        'Pacific/Kwajalein' => 'Sa’aatii Odoloota Maarshaal (Kwajalein)',
        'Pacific/Majuro' => 'Sa’aatii Odoloota Maarshaal (Majuro)',
        'Pacific/Marquesas' => 'Sa’aatii Marquesas',
        'Pacific/Midway' => 'Sa’aatii Saamowaa (Midway)',
        'Pacific/Nauru' => 'Sa’aatii Naawuruu (Nauru)',
        'Pacific/Niue' => 'Sa’aatii Niue',
        'Pacific/Norfolk' => 'Sa’aatii Norfolk Island',
        'Pacific/Noumea' => 'Sa’aatii Kaaledooniyaa Haaraa (Nouméa)',
        'Pacific/Pago_Pago' => 'Sa’aatii Saamowaa (Pago Pago)',
        'Pacific/Palau' => 'Sa’aatii Palawu (Palau)',
        'Pacific/Pitcairn' => 'Sa’aatii Pitcairn',
        'Pacific/Ponape' => 'Sa’aatii Ponape (Pohnpei)',
        'Pacific/Pohnpei' => 'Sa’aatii Ponape (Pohnpei)',
        'Pacific/Port_Moresby' => 'Sa’aatii Paapuwaa Giinii Haaraa (Port Moresby)',
        'Pacific/Rarotonga' => 'Sa’aatii Odoloota Kuuk (Rarotonga)',
        'Pacific/Saipan' => 'Sa’aatii Istaandaardii Kamoroo (Saipan)',
        'Pacific/Tahiti' => 'Sa’aatii Tahiti',
        'Pacific/Tarawa' => 'Sa’aatii Odoloota Giilbert (Tarawa)',
        'Pacific/Tongatapu' => 'Sa’aatii Tonga (Tongatapu)',
        'Pacific/Truk' => 'Sa’aatii Chuuk',
        'Pacific/Chuuk' => 'Sa’aatii Chuuk',
        'Pacific/Wake' => 'Sa’aatii Odola Wake',
        'Pacific/Wallis' => 'Sa’aatii Wallis fi Futuna (Wallis & Futuna)',
    ],
    'Meta' => [],
];
