<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'nụ hôn: người, người, màu da sáng, màu da sáng trung bình',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'nụ hôn: người, người, màu da sáng, màu da trung bình',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'nụ hôn: người, người, màu da sáng, màu da tối trung bình',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'nụ hôn: người, người, màu da sáng, màu da tối',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'nụ hôn: người, người, màu da sáng trung bình, màu da sáng',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'nụ hôn: người, người, màu da sáng trung bình, màu da trung bình',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'nụ hôn: người, người, màu da sáng trung bình, màu da tối trung bình',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'nụ hôn: người, người, màu da sáng trung bình, màu da tối',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'nụ hôn: người, người, màu da trung bình, màu da sáng',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'nụ hôn: người, người, màu da trung bình, màu da sáng trung bình',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'nụ hôn: người, người, màu da trung bình, màu da tối trung bình',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'nụ hôn: người, người, màu da trung bình, màu da tối',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'nụ hôn: người, người, màu da tối trung bình, màu da sáng',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'nụ hôn: người, người, màu da tối trung bình, màu da sáng trung bình',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'nụ hôn: người, người, màu da tối trung bình, màu da trung bình',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'nụ hôn: người, người, màu da tối trung bình, màu da tối',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'nụ hôn: người, người, màu da tối, màu da sáng',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'nụ hôn: người, người, màu da tối, màu da sáng trung bình',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'nụ hôn: người, người, màu da tối, màu da trung bình',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'nụ hôn: người, người, màu da tối, màu da tối trung bình',
    '👨🏻‍❤‍💋‍👨🏻' => 'nụ hôn: đàn ông, đàn ông, màu da sáng',
    '👨🏻‍❤‍💋‍👨🏼' => 'nụ hôn: đàn ông, đàn ông, màu da sáng, màu da sáng trung bình',
    '👨🏻‍❤‍💋‍👨🏽' => 'nụ hôn: đàn ông, đàn ông, màu da sáng, màu da trung bình',
    '👨🏻‍❤‍💋‍👨🏾' => 'nụ hôn: đàn ông, đàn ông, màu da sáng, màu da tối trung bình',
    '👨🏻‍❤‍💋‍👨🏿' => 'nụ hôn: đàn ông, đàn ông, màu da sáng, màu da tối',
    '👨🏼‍❤‍💋‍👨🏻' => 'nụ hôn: đàn ông, đàn ông, màu da sáng trung bình, màu da sáng',
    '👨🏼‍❤‍💋‍👨🏼' => 'nụ hôn: đàn ông, đàn ông, màu da sáng trung bình',
    '👨🏼‍❤‍💋‍👨🏽' => 'nụ hôn: đàn ông, đàn ông, màu da sáng trung bình, màu da trung bình',
    '👨🏼‍❤‍💋‍👨🏾' => 'nụ hôn: đàn ông, đàn ông, màu da sáng trung bình, màu da tối trung bình',
    '👨🏼‍❤‍💋‍👨🏿' => 'nụ hôn: đàn ông, đàn ông, màu da sáng trung bình, màu da tối',
    '👨🏽‍❤‍💋‍👨🏻' => 'nụ hôn: đàn ông, đàn ông, màu da trung bình, màu da sáng',
    '👨🏽‍❤‍💋‍👨🏼' => 'nụ hôn: đàn ông, đàn ông, màu da trung bình, màu da sáng trung bình',
    '👨🏽‍❤‍💋‍👨🏽' => 'nụ hôn: đàn ông, đàn ông, màu da trung bình',
    '👨🏽‍❤‍💋‍👨🏾' => 'nụ hôn: đàn ông, đàn ông, màu da trung bình, màu da tối trung bình',
    '👨🏽‍❤‍💋‍👨🏿' => 'nụ hôn: đàn ông, đàn ông, màu da trung bình, màu da tối',
    '👨🏾‍❤‍💋‍👨🏻' => 'nụ hôn: đàn ông, đàn ông, màu da tối trung bình, màu da sáng',
    '👨🏾‍❤‍💋‍👨🏼' => 'nụ hôn: đàn ông, đàn ông, màu da tối trung bình, màu da sáng trung bình',
    '👨🏾‍❤‍💋‍👨🏽' => 'nụ hôn: đàn ông, đàn ông, màu da tối trung bình, màu da trung bình',
    '👨🏾‍❤‍💋‍👨🏾' => 'nụ hôn: đàn ông, đàn ông, màu da tối trung bình',
    '👨🏾‍❤‍💋‍👨🏿' => 'nụ hôn: đàn ông, đàn ông, màu da tối trung bình, màu da tối',
    '👨🏿‍❤‍💋‍👨🏻' => 'nụ hôn: đàn ông, đàn ông, màu da tối, màu da sáng',
    '👨🏿‍❤‍💋‍👨🏼' => 'nụ hôn: đàn ông, đàn ông, màu da tối, màu da sáng trung bình',
    '👨🏿‍❤‍💋‍👨🏽' => 'nụ hôn: đàn ông, đàn ông, màu da tối, màu da trung bình',
    '👨🏿‍❤‍💋‍👨🏾' => 'nụ hôn: đàn ông, đàn ông, màu da tối, màu da tối trung bình',
    '👨🏿‍❤‍💋‍👨🏿' => 'nụ hôn: đàn ông, đàn ông, màu da tối',
    '👩🏻‍❤‍💋‍👨🏻' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng',
    '👩🏻‍❤‍💋‍👨🏼' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng, màu da sáng trung bình',
    '👩🏻‍❤‍💋‍👨🏽' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng, màu da trung bình',
    '👩🏻‍❤‍💋‍👨🏾' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng, màu da tối trung bình',
    '👩🏻‍❤‍💋‍👨🏿' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng, màu da tối',
    '👩🏼‍❤‍💋‍👨🏻' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng trung bình, màu da sáng',
    '👩🏼‍❤‍💋‍👨🏼' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng trung bình',
    '👩🏼‍❤‍💋‍👨🏽' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng trung bình, màu da trung bình',
    '👩🏼‍❤‍💋‍👨🏾' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍❤‍💋‍👨🏿' => 'nụ hôn: phụ nữ, đàn ông, màu da sáng trung bình, màu da tối',
    '👩🏽‍❤‍💋‍👨🏻' => 'nụ hôn: phụ nữ, đàn ông, màu da trung bình, màu da sáng',
    '👩🏽‍❤‍💋‍👨🏼' => 'nụ hôn: phụ nữ, đàn ông, màu da trung bình, màu da sáng trung bình',
    '👩🏽‍❤‍💋‍👨🏽' => 'nụ hôn: phụ nữ, đàn ông, màu da trung bình',
    '👩🏽‍❤‍💋‍👨🏾' => 'nụ hôn: phụ nữ, đàn ông, màu da trung bình, màu da tối trung bình',
    '👩🏽‍❤‍💋‍👨🏿' => 'nụ hôn: phụ nữ, đàn ông, màu da trung bình, màu da tối',
    '👩🏾‍❤‍💋‍👨🏻' => 'nụ hôn: phụ nữ, đàn ông, màu da tối trung bình, màu da sáng',
    '👩🏾‍❤‍💋‍👨🏼' => 'nụ hôn: phụ nữ, đàn ông, màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍❤‍💋‍👨🏽' => 'nụ hôn: phụ nữ, đàn ông, màu da tối trung bình, màu da trung bình',
    '👩🏾‍❤‍💋‍👨🏾' => 'nụ hôn: phụ nữ, đàn ông, màu da tối trung bình',
    '👩🏾‍❤‍💋‍👨🏿' => 'nụ hôn: phụ nữ, đàn ông, màu da tối trung bình, màu da tối',
    '👩🏿‍❤‍💋‍👨🏻' => 'nụ hôn: phụ nữ, đàn ông, màu da tối, màu da sáng',
    '👩🏿‍❤‍💋‍👨🏼' => 'nụ hôn: phụ nữ, đàn ông, màu da tối, màu da sáng trung bình',
    '👩🏿‍❤‍💋‍👨🏽' => 'nụ hôn: phụ nữ, đàn ông, màu da tối, màu da trung bình',
    '👩🏿‍❤‍💋‍👨🏾' => 'nụ hôn: phụ nữ, đàn ông, màu da tối, màu da tối trung bình',
    '👩🏿‍❤‍💋‍👨🏿' => 'nụ hôn: phụ nữ, đàn ông, màu da tối',
    '👩🏻‍❤‍💋‍👩🏻' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng',
    '👩🏻‍❤‍💋‍👩🏼' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng, màu da sáng trung bình',
    '👩🏻‍❤‍💋‍👩🏽' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng, màu da trung bình',
    '👩🏻‍❤‍💋‍👩🏾' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng, màu da tối trung bình',
    '👩🏻‍❤‍💋‍👩🏿' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng, màu da tối',
    '👩🏼‍❤‍💋‍👩🏻' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng trung bình, màu da sáng',
    '👩🏼‍❤‍💋‍👩🏼' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng trung bình',
    '👩🏼‍❤‍💋‍👩🏽' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng trung bình, màu da trung bình',
    '👩🏼‍❤‍💋‍👩🏾' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍❤‍💋‍👩🏿' => 'nụ hôn: phụ nữ, phụ nữ, màu da sáng trung bình, màu da tối',
    '👩🏽‍❤‍💋‍👩🏻' => 'nụ hôn: phụ nữ, phụ nữ, màu da trung bình, màu da sáng',
    '👩🏽‍❤‍💋‍👩🏼' => 'nụ hôn: phụ nữ, phụ nữ, màu da trung bình, màu da sáng trung bình',
    '👩🏽‍❤‍💋‍👩🏽' => 'nụ hôn: phụ nữ, phụ nữ, màu da trung bình',
    '👩🏽‍❤‍💋‍👩🏾' => 'nụ hôn: phụ nữ, phụ nữ, màu da trung bình, màu da tối trung bình',
    '👩🏽‍❤‍💋‍👩🏿' => 'nụ hôn: phụ nữ, phụ nữ, màu da trung bình, màu da tối',
    '👩🏾‍❤‍💋‍👩🏻' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối trung bình, màu da sáng',
    '👩🏾‍❤‍💋‍👩🏼' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍❤‍💋‍👩🏽' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối trung bình, màu da trung bình',
    '👩🏾‍❤‍💋‍👩🏾' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối trung bình',
    '👩🏾‍❤‍💋‍👩🏿' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối trung bình, màu da tối',
    '👩🏿‍❤‍💋‍👩🏻' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối, màu da sáng',
    '👩🏿‍❤‍💋‍👩🏼' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối, màu da sáng trung bình',
    '👩🏿‍❤‍💋‍👩🏽' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối, màu da trung bình',
    '👩🏿‍❤‍💋‍👩🏾' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối, màu da tối trung bình',
    '👩🏿‍❤‍💋‍👩🏿' => 'nụ hôn: phụ nữ, phụ nữ, màu da tối',
    '🧑🏻‍🫯‍🧑🏼' => 'người chơi vật: màu da sáng, màu da sáng trung bình',
    '🧑🏻‍🫯‍🧑🏽' => 'người chơi vật: màu da sáng, màu da trung bình',
    '🧑🏻‍🫯‍🧑🏾' => 'người chơi vật: màu da sáng, màu da tối trung bình',
    '🧑🏻‍🫯‍🧑🏿' => 'người chơi vật: màu da sáng, màu da tối',
    '🧑🏼‍🫯‍🧑🏻' => 'người chơi vật: màu da sáng trung bình, màu da sáng',
    '🧑🏼‍🫯‍🧑🏽' => 'người chơi vật: màu da sáng trung bình, màu da trung bình',
    '🧑🏼‍🫯‍🧑🏾' => 'người chơi vật: màu da sáng trung bình, màu da tối trung bình',
    '🧑🏼‍🫯‍🧑🏿' => 'người chơi vật: màu da sáng trung bình, màu da tối',
    '🧑🏽‍🫯‍🧑🏻' => 'người chơi vật: màu da trung bình, màu da sáng',
    '🧑🏽‍🫯‍🧑🏼' => 'người chơi vật: màu da trung bình, màu da sáng trung bình',
    '🧑🏽‍🫯‍🧑🏾' => 'người chơi vật: màu da trung bình, màu da tối trung bình',
    '🧑🏽‍🫯‍🧑🏿' => 'người chơi vật: màu da trung bình, màu da tối',
    '🧑🏾‍🫯‍🧑🏻' => 'người chơi vật: màu da tối trung bình, màu da sáng',
    '🧑🏾‍🫯‍🧑🏼' => 'người chơi vật: màu da tối trung bình, màu da sáng trung bình',
    '🧑🏾‍🫯‍🧑🏽' => 'người chơi vật: màu da tối trung bình, màu da trung bình',
    '🧑🏾‍🫯‍🧑🏿' => 'người chơi vật: màu da tối trung bình, màu da tối',
    '🧑🏿‍🫯‍🧑🏻' => 'người chơi vật: màu da tối, màu da sáng',
    '🧑🏿‍🫯‍🧑🏼' => 'người chơi vật: màu da tối, màu da sáng trung bình',
    '🧑🏿‍🫯‍🧑🏽' => 'người chơi vật: màu da tối, màu da trung bình',
    '🧑🏿‍🫯‍🧑🏾' => 'người chơi vật: màu da tối, màu da tối trung bình',
    '🧑🏻‍❤‍🧑🏼' => 'cặp đôi với trái tim: người, người, màu da sáng, màu da sáng trung bình',
    '🧑🏻‍❤‍🧑🏽' => 'cặp đôi với trái tim: người, người, màu da sáng, màu da trung bình',
    '🧑🏻‍❤‍🧑🏾' => 'cặp đôi với trái tim: người, người, màu da sáng, màu da tối trung bình',
    '🧑🏻‍❤‍🧑🏿' => 'cặp đôi với trái tim: người, người, màu da sáng, màu da tối',
    '🧑🏼‍❤‍🧑🏻' => 'cặp đôi với trái tim: người, người, màu da sáng trung bình, màu da sáng',
    '🧑🏼‍❤‍🧑🏽' => 'cặp đôi với trái tim: người, người, màu da sáng trung bình, màu da trung bình',
    '🧑🏼‍❤‍🧑🏾' => 'cặp đôi với trái tim: người, người, màu da sáng trung bình, màu da tối trung bình',
    '🧑🏼‍❤‍🧑🏿' => 'cặp đôi với trái tim: người, người, màu da sáng trung bình, màu da tối',
    '🧑🏽‍❤‍🧑🏻' => 'cặp đôi với trái tim: người, người, màu da trung bình, màu da sáng',
    '🧑🏽‍❤‍🧑🏼' => 'cặp đôi với trái tim: người, người, màu da trung bình, màu da sáng trung bình',
    '🧑🏽‍❤‍🧑🏾' => 'cặp đôi với trái tim: người, người, màu da trung bình, màu da tối trung bình',
    '🧑🏽‍❤‍🧑🏿' => 'cặp đôi với trái tim: người, người, màu da trung bình, màu da tối',
    '🧑🏾‍❤‍🧑🏻' => 'cặp đôi với trái tim: người, người, màu da tối trung bình, màu da sáng',
    '🧑🏾‍❤‍🧑🏼' => 'cặp đôi với trái tim: người, người, màu da tối trung bình, màu da sáng trung bình',
    '🧑🏾‍❤‍🧑🏽' => 'cặp đôi với trái tim: người, người, màu da tối trung bình, màu da trung bình',
    '🧑🏾‍❤‍🧑🏿' => 'cặp đôi với trái tim: người, người, màu da tối trung bình, màu da tối',
    '🧑🏿‍❤‍🧑🏻' => 'cặp đôi với trái tim: người, người, màu da tối, màu da sáng',
    '🧑🏿‍❤‍🧑🏼' => 'cặp đôi với trái tim: người, người, màu da tối, màu da sáng trung bình',
    '🧑🏿‍❤‍🧑🏽' => 'cặp đôi với trái tim: người, người, màu da tối, màu da trung bình',
    '🧑🏿‍❤‍🧑🏾' => 'cặp đôi với trái tim: người, người, màu da tối, màu da tối trung bình',
    '🧑🏻‍🐰‍🧑🏼' => 'những người đeo tai thỏ: màu da sáng, màu da sáng trung bình',
    '🧑🏻‍🐰‍🧑🏽' => 'những người đeo tai thỏ: màu da sáng, màu da trung bình',
    '🧑🏻‍🐰‍🧑🏾' => 'những người đeo tai thỏ: màu da sáng, màu da tối trung bình',
    '🧑🏻‍🐰‍🧑🏿' => 'những người đeo tai thỏ: màu da sáng, màu da tối',
    '🧑🏼‍🐰‍🧑🏻' => 'những người đeo tai thỏ: màu da sáng trung bình, màu da sáng',
    '🧑🏼‍🐰‍🧑🏽' => 'những người đeo tai thỏ: màu da sáng trung bình, màu da trung bình',
    '🧑🏼‍🐰‍🧑🏾' => 'những người đeo tai thỏ: màu da sáng trung bình, màu da tối trung bình',
    '🧑🏼‍🐰‍🧑🏿' => 'những người đeo tai thỏ: màu da sáng trung bình, màu da tối',
    '🧑🏽‍🐰‍🧑🏻' => 'những người đeo tai thỏ: màu da trung bình, màu da sáng',
    '🧑🏽‍🐰‍🧑🏼' => 'những người đeo tai thỏ: màu da trung bình, màu da sáng trung bình',
    '🧑🏽‍🐰‍🧑🏾' => 'những người đeo tai thỏ: màu da trung bình, màu da tối trung bình',
    '🧑🏽‍🐰‍🧑🏿' => 'những người đeo tai thỏ: màu da trung bình, màu da tối',
    '🧑🏾‍🐰‍🧑🏻' => 'những người đeo tai thỏ: màu da tối trung bình, màu da sáng',
    '🧑🏾‍🐰‍🧑🏼' => 'những người đeo tai thỏ: màu da tối trung bình, màu da sáng trung bình',
    '🧑🏾‍🐰‍🧑🏽' => 'những người đeo tai thỏ: màu da tối trung bình, màu da trung bình',
    '🧑🏾‍🐰‍🧑🏿' => 'những người đeo tai thỏ: màu da tối trung bình, màu da tối',
    '🧑🏿‍🐰‍🧑🏻' => 'những người đeo tai thỏ: màu da tối, màu da sáng',
    '🧑🏿‍🐰‍🧑🏼' => 'những người đeo tai thỏ: màu da tối, màu da sáng trung bình',
    '🧑🏿‍🐰‍🧑🏽' => 'những người đeo tai thỏ: màu da tối, màu da trung bình',
    '🧑🏿‍🐰‍🧑🏾' => 'những người đeo tai thỏ: màu da tối, màu da tối trung bình',
    '👨🏻‍🫯‍👨🏼' => 'những người đàn ông chơi vật: màu da sáng, màu da sáng trung bình',
    '👨🏻‍🫯‍👨🏽' => 'những người đàn ông chơi vật: màu da sáng, màu da trung bình',
    '👨🏻‍🫯‍👨🏾' => 'những người đàn ông chơi vật: màu da sáng, màu da tối trung bình',
    '👨🏻‍🫯‍👨🏿' => 'những người đàn ông chơi vật: màu da sáng, màu da tối',
    '👨🏼‍🫯‍👨🏻' => 'những người đàn ông chơi vật: màu da sáng trung bình, màu da sáng',
    '👨🏼‍🫯‍👨🏽' => 'những người đàn ông chơi vật: màu da sáng trung bình, màu da trung bình',
    '👨🏼‍🫯‍👨🏾' => 'những người đàn ông chơi vật: màu da sáng trung bình, màu da tối trung bình',
    '👨🏼‍🫯‍👨🏿' => 'những người đàn ông chơi vật: màu da sáng trung bình, màu da tối',
    '👨🏽‍🫯‍👨🏻' => 'những người đàn ông chơi vật: màu da trung bình, màu da sáng',
    '👨🏽‍🫯‍👨🏼' => 'những người đàn ông chơi vật: màu da trung bình, màu da sáng trung bình',
    '👨🏽‍🫯‍👨🏾' => 'những người đàn ông chơi vật: màu da trung bình, màu da tối trung bình',
    '👨🏽‍🫯‍👨🏿' => 'những người đàn ông chơi vật: màu da trung bình, màu da tối',
    '👨🏾‍🫯‍👨🏻' => 'những người đàn ông chơi vật: màu da tối trung bình, màu da sáng',
    '👨🏾‍🫯‍👨🏼' => 'những người đàn ông chơi vật: màu da tối trung bình, màu da sáng trung bình',
    '👨🏾‍🫯‍👨🏽' => 'những người đàn ông chơi vật: màu da tối trung bình, màu da trung bình',
    '👨🏾‍🫯‍👨🏿' => 'những người đàn ông chơi vật: màu da tối trung bình, màu da tối',
    '👨🏿‍🫯‍👨🏻' => 'những người đàn ông chơi vật: màu da tối, màu da sáng',
    '👨🏿‍🫯‍👨🏼' => 'những người đàn ông chơi vật: màu da tối, màu da sáng trung bình',
    '👨🏿‍🫯‍👨🏽' => 'những người đàn ông chơi vật: màu da tối, màu da trung bình',
    '👨🏿‍🫯‍👨🏾' => 'những người đàn ông chơi vật: màu da tối, màu da tối trung bình',
    '👨🏻‍❤‍👨🏻' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng',
    '👨🏻‍❤‍👨🏼' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng, màu da sáng trung bình',
    '👨🏻‍❤‍👨🏽' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng, màu da trung bình',
    '👨🏻‍❤‍👨🏾' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng, màu da tối trung bình',
    '👨🏻‍❤‍👨🏿' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng, màu da tối',
    '👨🏼‍❤‍👨🏻' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng trung bình, màu da sáng',
    '👨🏼‍❤‍👨🏼' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng trung bình',
    '👨🏼‍❤‍👨🏽' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng trung bình, màu da trung bình',
    '👨🏼‍❤‍👨🏾' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng trung bình, màu da tối trung bình',
    '👨🏼‍❤‍👨🏿' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da sáng trung bình, màu da tối',
    '👨🏽‍❤‍👨🏻' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da trung bình, màu da sáng',
    '👨🏽‍❤‍👨🏼' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da trung bình, màu da sáng trung bình',
    '👨🏽‍❤‍👨🏽' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da trung bình',
    '👨🏽‍❤‍👨🏾' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da trung bình, màu da tối trung bình',
    '👨🏽‍❤‍👨🏿' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da trung bình, màu da tối',
    '👨🏾‍❤‍👨🏻' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối trung bình, màu da sáng',
    '👨🏾‍❤‍👨🏼' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối trung bình, màu da sáng trung bình',
    '👨🏾‍❤‍👨🏽' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối trung bình, màu da trung bình',
    '👨🏾‍❤‍👨🏾' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối trung bình',
    '👨🏾‍❤‍👨🏿' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối trung bình, màu da tối',
    '👨🏿‍❤‍👨🏻' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối, màu da sáng',
    '👨🏿‍❤‍👨🏼' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối, màu da sáng trung bình',
    '👨🏿‍❤‍👨🏽' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối, màu da trung bình',
    '👨🏿‍❤‍👨🏾' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối, màu da tối trung bình',
    '👨🏿‍❤‍👨🏿' => 'cặp đôi với trái tim: đàn ông, đàn ông, màu da tối',
    '👨🏻‍🐰‍👨🏼' => 'những người đàn ông đeo tai thỏ: màu da sáng, màu da sáng trung bình',
    '👨🏻‍🐰‍👨🏽' => 'những người đàn ông đeo tai thỏ: màu da sáng, màu da trung bình',
    '👨🏻‍🐰‍👨🏾' => 'những người đàn ông đeo tai thỏ: màu da sáng, màu da tối trung bình',
    '👨🏻‍🐰‍👨🏿' => 'những người đàn ông đeo tai thỏ: màu da sáng, màu da tối',
    '👨🏼‍🐰‍👨🏻' => 'những người đàn ông đeo tai thỏ: màu da sáng trung bình, màu da sáng',
    '👨🏼‍🐰‍👨🏽' => 'những người đàn ông đeo tai thỏ: màu da sáng trung bình, màu da trung bình',
    '👨🏼‍🐰‍👨🏾' => 'những người đàn ông đeo tai thỏ: màu da sáng trung bình, màu da tối trung bình',
    '👨🏼‍🐰‍👨🏿' => 'những người đàn ông đeo tai thỏ: màu da sáng trung bình, màu da tối',
    '👨🏽‍🐰‍👨🏻' => 'những người đàn ông đeo tai thỏ: màu da trung bình, màu da sáng',
    '👨🏽‍🐰‍👨🏼' => 'những người đàn ông đeo tai thỏ: màu da trung bình, màu da sáng trung bình',
    '👨🏽‍🐰‍👨🏾' => 'những người đàn ông đeo tai thỏ: màu da trung bình, màu da tối trung bình',
    '👨🏽‍🐰‍👨🏿' => 'những người đàn ông đeo tai thỏ: màu da trung bình, màu da tối',
    '👨🏾‍🐰‍👨🏻' => 'những người đàn ông đeo tai thỏ: màu da tối trung bình, màu da sáng',
    '👨🏾‍🐰‍👨🏼' => 'những người đàn ông đeo tai thỏ: màu da tối trung bình, màu da sáng trung bình',
    '👨🏾‍🐰‍👨🏽' => 'những người đàn ông đeo tai thỏ: màu da tối trung bình, màu da trung bình',
    '👨🏾‍🐰‍👨🏿' => 'những người đàn ông đeo tai thỏ: màu da tối trung bình, màu da tối',
    '👨🏿‍🐰‍👨🏻' => 'những người đàn ông đeo tai thỏ: màu da tối, màu da sáng',
    '👨🏿‍🐰‍👨🏼' => 'những người đàn ông đeo tai thỏ: màu da tối, màu da sáng trung bình',
    '👨🏿‍🐰‍👨🏽' => 'những người đàn ông đeo tai thỏ: màu da tối, màu da trung bình',
    '👨🏿‍🐰‍👨🏾' => 'những người đàn ông đeo tai thỏ: màu da tối, màu da tối trung bình',
    '👩🏻‍🫯‍👩🏼' => 'những người phụ nữ chơi vật: màu da sáng, màu da sáng trung bình',
    '👩🏻‍🫯‍👩🏽' => 'những người phụ nữ chơi vật: màu da sáng, màu da trung bình',
    '👩🏻‍🫯‍👩🏾' => 'những người phụ nữ chơi vật: màu da sáng, màu da tối trung bình',
    '👩🏻‍🫯‍👩🏿' => 'những người phụ nữ chơi vật: màu da sáng, màu da tối',
    '👩🏼‍🫯‍👩🏻' => 'những người phụ nữ chơi vật: màu da sáng trung bình, màu da sáng',
    '👩🏼‍🫯‍👩🏽' => 'những người phụ nữ chơi vật: màu da sáng trung bình, màu da trung bình',
    '👩🏼‍🫯‍👩🏾' => 'những người phụ nữ chơi vật: màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍🫯‍👩🏿' => 'những người phụ nữ chơi vật: màu da sáng trung bình, màu da tối',
    '👩🏽‍🫯‍👩🏻' => 'những người phụ nữ chơi vật: màu da trung bình, màu da sáng',
    '👩🏽‍🫯‍👩🏼' => 'những người phụ nữ chơi vật: màu da trung bình, màu da sáng trung bình',
    '👩🏽‍🫯‍👩🏾' => 'những người phụ nữ chơi vật: màu da trung bình, màu da tối trung bình',
    '👩🏽‍🫯‍👩🏿' => 'những người phụ nữ chơi vật: màu da trung bình, màu da tối',
    '👩🏾‍🫯‍👩🏻' => 'những người phụ nữ chơi vật: màu da tối trung bình, màu da sáng',
    '👩🏾‍🫯‍👩🏼' => 'những người phụ nữ chơi vật: màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍🫯‍👩🏽' => 'những người phụ nữ chơi vật: màu da tối trung bình, màu da trung bình',
    '👩🏾‍🫯‍👩🏿' => 'những người phụ nữ chơi vật: màu da tối trung bình, màu da tối',
    '👩🏿‍🫯‍👩🏻' => 'những người phụ nữ chơi vật: màu da tối, màu da sáng',
    '👩🏿‍🫯‍👩🏼' => 'những người phụ nữ chơi vật: màu da tối, màu da sáng trung bình',
    '👩🏿‍🫯‍👩🏽' => 'những người phụ nữ chơi vật: màu da tối, màu da trung bình',
    '👩🏿‍🫯‍👩🏾' => 'những người phụ nữ chơi vật: màu da tối, màu da tối trung bình',
    '👩🏻‍❤‍👨🏻' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng',
    '👩🏻‍❤‍👨🏼' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng, màu da sáng trung bình',
    '👩🏻‍❤‍👨🏽' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng, màu da trung bình',
    '👩🏻‍❤‍👨🏾' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng, màu da tối trung bình',
    '👩🏻‍❤‍👨🏿' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng, màu da tối',
    '👩🏼‍❤‍👨🏻' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng trung bình, màu da sáng',
    '👩🏼‍❤‍👨🏼' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng trung bình',
    '👩🏼‍❤‍👨🏽' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng trung bình, màu da trung bình',
    '👩🏼‍❤‍👨🏾' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍❤‍👨🏿' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da sáng trung bình, màu da tối',
    '👩🏽‍❤‍👨🏻' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da trung bình, màu da sáng',
    '👩🏽‍❤‍👨🏼' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da trung bình, màu da sáng trung bình',
    '👩🏽‍❤‍👨🏽' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da trung bình',
    '👩🏽‍❤‍👨🏾' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da trung bình, màu da tối trung bình',
    '👩🏽‍❤‍👨🏿' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da trung bình, màu da tối',
    '👩🏾‍❤‍👨🏻' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối trung bình, màu da sáng',
    '👩🏾‍❤‍👨🏼' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍❤‍👨🏽' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối trung bình, màu da trung bình',
    '👩🏾‍❤‍👨🏾' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối trung bình',
    '👩🏾‍❤‍👨🏿' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối trung bình, màu da tối',
    '👩🏿‍❤‍👨🏻' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối, màu da sáng',
    '👩🏿‍❤‍👨🏼' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối, màu da sáng trung bình',
    '👩🏿‍❤‍👨🏽' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối, màu da trung bình',
    '👩🏿‍❤‍👨🏾' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối, màu da tối trung bình',
    '👩🏿‍❤‍👨🏿' => 'cặp đôi với trái tim: phụ nữ, đàn ông, màu da tối',
    '👩🏻‍❤‍👩🏻' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng',
    '👩🏻‍❤‍👩🏼' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng, màu da sáng trung bình',
    '👩🏻‍❤‍👩🏽' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng, màu da trung bình',
    '👩🏻‍❤‍👩🏾' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng, màu da tối trung bình',
    '👩🏻‍❤‍👩🏿' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng, màu da tối',
    '👩🏼‍❤‍👩🏻' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng trung bình, màu da sáng',
    '👩🏼‍❤‍👩🏼' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng trung bình',
    '👩🏼‍❤‍👩🏽' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng trung bình, màu da trung bình',
    '👩🏼‍❤‍👩🏾' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍❤‍👩🏿' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da sáng trung bình, màu da tối',
    '👩🏽‍❤‍👩🏻' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da trung bình, màu da sáng',
    '👩🏽‍❤‍👩🏼' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da trung bình, màu da sáng trung bình',
    '👩🏽‍❤‍👩🏽' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da trung bình',
    '👩🏽‍❤‍👩🏾' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da trung bình, màu da tối trung bình',
    '👩🏽‍❤‍👩🏿' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da trung bình, màu da tối',
    '👩🏾‍❤‍👩🏻' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối trung bình, màu da sáng',
    '👩🏾‍❤‍👩🏼' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍❤‍👩🏽' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối trung bình, màu da trung bình',
    '👩🏾‍❤‍👩🏾' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối trung bình',
    '👩🏾‍❤‍👩🏿' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối trung bình, màu da tối',
    '👩🏿‍❤‍👩🏻' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối, màu da sáng',
    '👩🏿‍❤‍👩🏼' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối, màu da sáng trung bình',
    '👩🏿‍❤‍👩🏽' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối, màu da trung bình',
    '👩🏿‍❤‍👩🏾' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối, màu da tối trung bình',
    '👩🏿‍❤‍👩🏿' => 'cặp đôi với trái tim: phụ nữ, phụ nữ, màu da tối',
    '👩🏻‍🐰‍👩🏼' => 'những người phụ nữ đeo tai thỏ: màu da sáng, màu da sáng trung bình',
    '👩🏻‍🐰‍👩🏽' => 'những người phụ nữ đeo tai thỏ: màu da sáng, màu da trung bình',
    '👩🏻‍🐰‍👩🏾' => 'những người phụ nữ đeo tai thỏ: màu da sáng, màu da tối trung bình',
    '👩🏻‍🐰‍👩🏿' => 'những người phụ nữ đeo tai thỏ: màu da sáng, màu da tối',
    '👩🏼‍🐰‍👩🏻' => 'những người phụ nữ đeo tai thỏ: màu da sáng trung bình, màu da sáng',
    '👩🏼‍🐰‍👩🏽' => 'những người phụ nữ đeo tai thỏ: màu da sáng trung bình, màu da trung bình',
    '👩🏼‍🐰‍👩🏾' => 'những người phụ nữ đeo tai thỏ: màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍🐰‍👩🏿' => 'những người phụ nữ đeo tai thỏ: màu da sáng trung bình, màu da tối',
    '👩🏽‍🐰‍👩🏻' => 'những người phụ nữ đeo tai thỏ: màu da trung bình, màu da sáng',
    '👩🏽‍🐰‍👩🏼' => 'những người phụ nữ đeo tai thỏ: màu da trung bình, màu da sáng trung bình',
    '👩🏽‍🐰‍👩🏾' => 'những người phụ nữ đeo tai thỏ: màu da trung bình, màu da tối trung bình',
    '👩🏽‍🐰‍👩🏿' => 'những người phụ nữ đeo tai thỏ: màu da trung bình, màu da tối',
    '👩🏾‍🐰‍👩🏻' => 'những người phụ nữ đeo tai thỏ: màu da tối trung bình, màu da sáng',
    '👩🏾‍🐰‍👩🏼' => 'những người phụ nữ đeo tai thỏ: màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍🐰‍👩🏽' => 'những người phụ nữ đeo tai thỏ: màu da tối trung bình, màu da trung bình',
    '👩🏾‍🐰‍👩🏿' => 'những người phụ nữ đeo tai thỏ: màu da tối trung bình, màu da tối',
    '👩🏿‍🐰‍👩🏻' => 'những người phụ nữ đeo tai thỏ: màu da tối, màu da sáng',
    '👩🏿‍🐰‍👩🏼' => 'những người phụ nữ đeo tai thỏ: màu da tối, màu da sáng trung bình',
    '👩🏿‍🐰‍👩🏽' => 'những người phụ nữ đeo tai thỏ: màu da tối, màu da trung bình',
    '👩🏿‍🐰‍👩🏾' => 'những người phụ nữ đeo tai thỏ: màu da tối, màu da tối trung bình',
    '🧑🏻‍🤝‍🧑🏻' => 'hai người nắm tay: màu da sáng',
    '🧑🏻‍🤝‍🧑🏼' => 'hai người nắm tay: màu da sáng, màu da sáng trung bình',
    '🧑🏻‍🤝‍🧑🏽' => 'hai người nắm tay: màu da sáng, màu da trung bình',
    '🧑🏻‍🤝‍🧑🏾' => 'hai người nắm tay: màu da sáng, màu da tối trung bình',
    '🧑🏻‍🤝‍🧑🏿' => 'hai người nắm tay: màu da sáng, màu da tối',
    '🧑🏼‍🤝‍🧑🏻' => 'hai người nắm tay: màu da sáng trung bình, màu da sáng',
    '🧑🏼‍🤝‍🧑🏼' => 'hai người nắm tay: màu da sáng trung bình',
    '🧑🏼‍🤝‍🧑🏽' => 'hai người nắm tay: màu da sáng trung bình, màu da trung bình',
    '🧑🏼‍🤝‍🧑🏾' => 'hai người nắm tay: màu da sáng trung bình, màu da tối trung bình',
    '🧑🏼‍🤝‍🧑🏿' => 'hai người nắm tay: màu da sáng trung bình, màu da tối',
    '🧑🏽‍🤝‍🧑🏻' => 'hai người nắm tay: màu da trung bình, màu da sáng',
    '🧑🏽‍🤝‍🧑🏼' => 'hai người nắm tay: màu da trung bình, màu da sáng trung bình',
    '🧑🏽‍🤝‍🧑🏽' => 'hai người nắm tay: màu da trung bình',
    '🧑🏽‍🤝‍🧑🏾' => 'hai người nắm tay: màu da trung bình, màu da tối trung bình',
    '🧑🏽‍🤝‍🧑🏿' => 'hai người nắm tay: màu da trung bình, màu da tối',
    '🧑🏾‍🤝‍🧑🏻' => 'hai người nắm tay: màu da tối trung bình, màu da sáng',
    '🧑🏾‍🤝‍🧑🏼' => 'hai người nắm tay: màu da tối trung bình, màu da sáng trung bình',
    '🧑🏾‍🤝‍🧑🏽' => 'hai người nắm tay: màu da tối trung bình, màu da trung bình',
    '🧑🏾‍🤝‍🧑🏾' => 'hai người nắm tay: màu da tối trung bình',
    '🧑🏾‍🤝‍🧑🏿' => 'hai người nắm tay: màu da tối trung bình, màu da tối',
    '🧑🏿‍🤝‍🧑🏻' => 'hai người nắm tay: màu da tối, màu da sáng',
    '🧑🏿‍🤝‍🧑🏼' => 'hai người nắm tay: màu da tối, màu da sáng trung bình',
    '🧑🏿‍🤝‍🧑🏽' => 'hai người nắm tay: màu da tối, màu da trung bình',
    '🧑🏿‍🤝‍🧑🏾' => 'hai người nắm tay: màu da tối, màu da tối trung bình',
    '🧑🏿‍🤝‍🧑🏿' => 'hai người nắm tay: màu da tối',
    '👩🏻‍🤝‍👩🏼' => 'hai người phụ nữ nắm tay: màu da sáng, màu da sáng trung bình',
    '👩🏻‍🤝‍👩🏽' => 'hai người phụ nữ nắm tay: màu da sáng, màu da trung bình',
    '👩🏻‍🤝‍👩🏾' => 'hai người phụ nữ nắm tay: màu da sáng, màu da tối trung bình',
    '👩🏻‍🤝‍👩🏿' => 'hai người phụ nữ nắm tay: màu da sáng, màu da tối',
    '👩🏼‍🤝‍👩🏻' => 'hai người phụ nữ nắm tay: màu da sáng trung bình, màu da sáng',
    '👩🏼‍🤝‍👩🏽' => 'hai người phụ nữ nắm tay: màu da sáng trung bình, màu da trung bình',
    '👩🏼‍🤝‍👩🏾' => 'hai người phụ nữ nắm tay: màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍🤝‍👩🏿' => 'hai người phụ nữ nắm tay: màu da sáng trung bình, màu da tối',
    '👩🏽‍🤝‍👩🏻' => 'hai người phụ nữ nắm tay: màu da trung bình, màu da sáng',
    '👩🏽‍🤝‍👩🏼' => 'hai người phụ nữ nắm tay: màu da trung bình, màu da sáng trung bình',
    '👩🏽‍🤝‍👩🏾' => 'hai người phụ nữ nắm tay: màu da trung bình, màu da tối trung bình',
    '👩🏽‍🤝‍👩🏿' => 'hai người phụ nữ nắm tay: màu da trung bình, màu da tối',
    '👩🏾‍🤝‍👩🏻' => 'hai người phụ nữ nắm tay: màu da tối trung bình, màu da sáng',
    '👩🏾‍🤝‍👩🏼' => 'hai người phụ nữ nắm tay: màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍🤝‍👩🏽' => 'hai người phụ nữ nắm tay: màu da tối trung bình, màu da trung bình',
    '👩🏾‍🤝‍👩🏿' => 'hai người phụ nữ nắm tay: màu da tối trung bình, màu da tối',
    '👩🏿‍🤝‍👩🏻' => 'hai người phụ nữ nắm tay: màu da tối, màu da sáng',
    '👩🏿‍🤝‍👩🏼' => 'hai người phụ nữ nắm tay: màu da tối, màu da sáng trung bình',
    '👩🏿‍🤝‍👩🏽' => 'hai người phụ nữ nắm tay: màu da tối, màu da trung bình',
    '👩🏿‍🤝‍👩🏾' => 'hai người phụ nữ nắm tay: màu da tối, màu da tối trung bình',
    '👩🏻‍🤝‍👨🏼' => 'người đàn ông và phụ nữ nắm tay: màu da sáng, màu da sáng trung bình',
    '👩🏻‍🤝‍👨🏽' => 'người đàn ông và phụ nữ nắm tay: màu da sáng, màu da trung bình',
    '👩🏻‍🤝‍👨🏾' => 'người đàn ông và phụ nữ nắm tay: màu da sáng, màu da tối trung bình',
    '👩🏻‍🤝‍👨🏿' => 'người đàn ông và phụ nữ nắm tay: màu da sáng, màu da tối',
    '👩🏼‍🤝‍👨🏻' => 'người đàn ông và phụ nữ nắm tay: màu da sáng trung bình, màu da sáng',
    '👩🏼‍🤝‍👨🏽' => 'người đàn ông và phụ nữ nắm tay: màu da sáng trung bình, màu da trung bình',
    '👩🏼‍🤝‍👨🏾' => 'người đàn ông và phụ nữ nắm tay: màu da sáng trung bình, màu da tối trung bình',
    '👩🏼‍🤝‍👨🏿' => 'người đàn ông và phụ nữ nắm tay: màu da sáng trung bình, màu da tối',
    '👩🏽‍🤝‍👨🏻' => 'người đàn ông và phụ nữ nắm tay: màu da trung bình, màu da sáng',
    '👩🏽‍🤝‍👨🏼' => 'người đàn ông và phụ nữ nắm tay: màu da trung bình, màu da sáng trung bình',
    '👩🏽‍🤝‍👨🏾' => 'người đàn ông và phụ nữ nắm tay: màu da trung bình, màu da tối trung bình',
    '👩🏽‍🤝‍👨🏿' => 'người đàn ông và phụ nữ nắm tay: màu da trung bình, màu da tối',
    '👩🏾‍🤝‍👨🏻' => 'người đàn ông và phụ nữ nắm tay: màu da tối trung bình, màu da sáng',
    '👩🏾‍🤝‍👨🏼' => 'người đàn ông và phụ nữ nắm tay: màu da tối trung bình, màu da sáng trung bình',
    '👩🏾‍🤝‍👨🏽' => 'người đàn ông và phụ nữ nắm tay: màu da tối trung bình, màu da trung bình',
    '👩🏾‍🤝‍👨🏿' => 'người đàn ông và phụ nữ nắm tay: màu da tối trung bình, màu da tối',
    '👩🏿‍🤝‍👨🏻' => 'người đàn ông và phụ nữ nắm tay: màu da tối, màu da sáng',
    '👩🏿‍🤝‍👨🏼' => 'người đàn ông và phụ nữ nắm tay: màu da tối, màu da sáng trung bình',
    '👩🏿‍🤝‍👨🏽' => 'người đàn ông và phụ nữ nắm tay: màu da tối, màu da trung bình',
    '👩🏿‍🤝‍👨🏾' => 'người đàn ông và phụ nữ nắm tay: màu da tối, màu da tối trung bình',
    '👨🏻‍🤝‍👨🏼' => 'hai người đàn ông nắm tay: màu da sáng, màu da sáng trung bình',
    '👨🏻‍🤝‍👨🏽' => 'hai người đàn ông nắm tay: màu da sáng, màu da trung bình',
    '👨🏻‍🤝‍👨🏾' => 'hai người đàn ông nắm tay: màu da sáng, màu da tối trung bình',
    '👨🏻‍🤝‍👨🏿' => 'hai người đàn ông nắm tay: màu da sáng, màu da tối',
    '👨🏼‍🤝‍👨🏻' => 'hai người đàn ông nắm tay: màu da sáng trung bình, màu da sáng',
    '👨🏼‍🤝‍👨🏽' => 'hai người đàn ông nắm tay: màu da sáng trung bình, màu da trung bình',
    '👨🏼‍🤝‍👨🏾' => 'hai người đàn ông nắm tay: màu da sáng trung bình, màu da tối trung bình',
    '👨🏼‍🤝‍👨🏿' => 'hai người đàn ông nắm tay: màu da sáng trung bình, màu da tối',
    '👨🏽‍🤝‍👨🏻' => 'hai người đàn ông nắm tay: màu da trung bình, màu da sáng',
    '👨🏽‍🤝‍👨🏼' => 'hai người đàn ông nắm tay: màu da trung bình, màu da sáng trung bình',
    '👨🏽‍🤝‍👨🏾' => 'hai người đàn ông nắm tay: màu da trung bình, màu da tối trung bình',
    '👨🏽‍🤝‍👨🏿' => 'hai người đàn ông nắm tay: màu da trung bình, màu da tối',
    '👨🏾‍🤝‍👨🏻' => 'hai người đàn ông nắm tay: màu da tối trung bình, màu da sáng',
    '👨🏾‍🤝‍👨🏼' => 'hai người đàn ông nắm tay: màu da tối trung bình, màu da sáng trung bình',
    '👨🏾‍🤝‍👨🏽' => 'hai người đàn ông nắm tay: màu da tối trung bình, màu da trung bình',
    '👨🏾‍🤝‍👨🏿' => 'hai người đàn ông nắm tay: màu da tối trung bình, màu da tối',
    '👨🏿‍🤝‍👨🏻' => 'hai người đàn ông nắm tay: màu da tối, màu da sáng',
    '👨🏿‍🤝‍👨🏼' => 'hai người đàn ông nắm tay: màu da tối, màu da sáng trung bình',
    '👨🏿‍🤝‍👨🏽' => 'hai người đàn ông nắm tay: màu da tối, màu da trung bình',
    '👨🏿‍🤝‍👨🏾' => 'hai người đàn ông nắm tay: màu da tối, màu da tối trung bình',
    '👩‍❤‍💋‍👨' => 'nụ hôn: phụ nữ, đàn ông',
    '👨‍❤‍💋‍👨' => 'nụ hôn: đàn ông, đàn ông',
    '👩‍❤‍💋‍👩' => 'nụ hôn: phụ nữ, phụ nữ',
    '👨‍👩‍👧‍👦' => 'gia đình: đàn ông, phụ nữ, con gái, con trai',
    '👨‍👩‍👦‍👦' => 'gia đình: đàn ông, phụ nữ, con trai, con trai',
    '👨‍👩‍👧‍👧' => 'gia đình: đàn ông, phụ nữ, con gái, con gái',
    '👨‍👨‍👧‍👦' => 'gia đình: đàn ông, đàn ông, con gái, con trai',
    '👨‍👨‍👦‍👦' => 'gia đình: đàn ông, đàn ông, con trai, con trai',
    '👨‍👨‍👧‍👧' => 'gia đình: đàn ông, đàn ông, con gái, con gái',
    '👩‍👩‍👧‍👦' => 'gia đình: phụ nữ, phụ nữ, con gái, con trai',
    '👩‍👩‍👦‍👦' => 'gia đình: phụ nữ, phụ nữ, con trai, con trai',
    '👩‍👩‍👧‍👧' => 'gia đình: phụ nữ, phụ nữ, con gái, con gái',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'cờ: Anh Quốc',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'cờ: Scotland',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'cờ: Xứ Wales',
    '🧑‍🧑‍🧒‍🧒' => 'gia đình: người lớn, người lớn, trẻ em, trẻ em',
    '🚶🏻‍♀‍➡' => 'người phụ nữ đi bộ: màu da sáng, hướng sang phải',
    '🚶🏼‍♀‍➡' => 'người phụ nữ đi bộ: màu da sáng trung bình, hướng sang phải',
    '🚶🏽‍♀‍➡' => 'người phụ nữ đi bộ: màu da trung bình, hướng sang phải',
    '🚶🏾‍♀‍➡' => 'người phụ nữ đi bộ: màu da tối trung bình, hướng sang phải',
    '🚶🏿‍♀‍➡' => 'người phụ nữ đi bộ: màu da tối, hướng sang phải',
    '🚶🏻‍♂‍➡' => 'người đàn ông đi bộ: màu da sáng, hướng sang phải',
    '🚶🏼‍♂‍➡' => 'người đàn ông đi bộ: màu da sáng trung bình, hướng sang phải',
    '🚶🏽‍♂‍➡' => 'người đàn ông đi bộ: màu da trung bình, hướng sang phải',
    '🚶🏾‍♂‍➡' => 'người đàn ông đi bộ: màu da tối trung bình, hướng sang phải',
    '🚶🏿‍♂‍➡' => 'người đàn ông đi bộ: màu da tối, hướng sang phải',
    '🧎🏻‍♀‍➡' => 'người phụ nữ quỳ: màu da sáng, hướng sang phải',
    '🧎🏼‍♀‍➡' => 'người phụ nữ quỳ: màu da sáng trung bình, hướng sang phải',
    '🧎🏽‍♀‍➡' => 'người phụ nữ quỳ: màu da trung bình, hướng sang phải',
    '🧎🏾‍♀‍➡' => 'người phụ nữ quỳ: màu da tối trung bình, hướng sang phải',
    '🧎🏿‍♀‍➡' => 'người phụ nữ quỳ: màu da tối, hướng sang phải',
    '🧎🏻‍♂‍➡' => 'người đàn ông quỳ: màu da sáng, hướng sang phải',
    '🧎🏼‍♂‍➡' => 'người đàn ông quỳ: màu da sáng trung bình, hướng sang phải',
    '🧎🏽‍♂‍➡' => 'người đàn ông quỳ: màu da trung bình, hướng sang phải',
    '🧎🏾‍♂‍➡' => 'người đàn ông quỳ: màu da tối trung bình, hướng sang phải',
    '🧎🏿‍♂‍➡' => 'người đàn ông quỳ: màu da tối, hướng sang phải',
    '🧑🏻‍🦯‍➡' => 'người với gậy dò đường: màu da sáng, hướng sang phải',
    '🧑🏼‍🦯‍➡' => 'người với gậy dò đường: màu da sáng trung bình, hướng sang phải',
    '🧑🏽‍🦯‍➡' => 'người với gậy dò đường: màu da trung bình, hướng sang phải',
    '🧑🏾‍🦯‍➡' => 'người với gậy dò đường: màu da tối trung bình, hướng sang phải',
    '🧑🏿‍🦯‍➡' => 'người với gậy dò đường: màu da tối, hướng sang phải',
    '👨🏻‍🦯‍➡' => 'người đàn ông với gậy dò đường: màu da sáng, hướng sang phải',
    '👨🏼‍🦯‍➡' => 'người đàn ông với gậy dò đường: màu da sáng trung bình, hướng sang phải',
    '👨🏽‍🦯‍➡' => 'người đàn ông với gậy dò đường: màu da trung bình, hướng sang phải',
    '👨🏾‍🦯‍➡' => 'người đàn ông với gậy dò đường: màu da tối trung bình, hướng sang phải',
    '👨🏿‍🦯‍➡' => 'người đàn ông với gậy dò đường: màu da tối, hướng sang phải',
    '👩🏻‍🦯‍➡' => 'người phụ nữ với gậy dò đường: màu da sáng, hướng sang phải',
    '👩🏼‍🦯‍➡' => 'người phụ nữ với gậy dò đường: màu da sáng trung bình, hướng sang phải',
    '👩🏽‍🦯‍➡' => 'người phụ nữ với gậy dò đường: màu da trung bình, hướng sang phải',
    '👩🏾‍🦯‍➡' => 'người phụ nữ với gậy dò đường: màu da tối trung bình, hướng sang phải',
    '👩🏿‍🦯‍➡' => 'người phụ nữ với gậy dò đường: màu da tối, hướng sang phải',
    '🧑🏻‍🦼‍➡' => 'người trên xe lăn điện: màu da sáng, hướng sang phải',
    '🧑🏼‍🦼‍➡' => 'người trên xe lăn điện: màu da sáng trung bình, hướng sang phải',
    '🧑🏽‍🦼‍➡' => 'người trên xe lăn điện: màu da trung bình, hướng sang phải',
    '🧑🏾‍🦼‍➡' => 'người trên xe lăn điện: màu da tối trung bình, hướng sang phải',
    '🧑🏿‍🦼‍➡' => 'người trên xe lăn điện: màu da tối, hướng sang phải',
    '👨🏻‍🦼‍➡' => 'người đàn ông trên xe lăn điện: màu da sáng, hướng sang phải',
    '👨🏼‍🦼‍➡' => 'người đàn ông trên xe lăn điện: màu da sáng trung bình, hướng sang phải',
    '👨🏽‍🦼‍➡' => 'người đàn ông trên xe lăn điện: màu da trung bình, hướng sang phải',
    '👨🏾‍🦼‍➡' => 'người đàn ông trên xe lăn điện: màu da tối trung bình, hướng sang phải',
    '👨🏿‍🦼‍➡' => 'người đàn ông trên xe lăn điện: màu da tối, hướng sang phải',
    '👩🏻‍🦼‍➡' => 'người phụ nữ trên xe lăn điện: màu da sáng, hướng sang phải',
    '👩🏼‍🦼‍➡' => 'người phụ nữ trên xe lăn điện: màu da sáng trung bình, hướng sang phải',
    '👩🏽‍🦼‍➡' => 'người phụ nữ trên xe lăn điện: màu da trung bình, hướng sang phải',
    '👩🏾‍🦼‍➡' => 'người phụ nữ trên xe lăn điện: màu da tối trung bình, hướng sang phải',
    '👩🏿‍🦼‍➡' => 'người phụ nữ trên xe lăn điện: màu da tối, hướng sang phải',
    '🧑🏻‍🦽‍➡' => 'người trên xe lăn tay: màu da sáng, hướng sang phải',
    '🧑🏼‍🦽‍➡' => 'người trên xe lăn tay: màu da sáng trung bình, hướng sang phải',
    '🧑🏽‍🦽‍➡' => 'người trên xe lăn tay: màu da trung bình, hướng sang phải',
    '🧑🏾‍🦽‍➡' => 'người trên xe lăn tay: màu da tối trung bình, hướng sang phải',
    '🧑🏿‍🦽‍➡' => 'người trên xe lăn tay: màu da tối, hướng sang phải',
    '👨🏻‍🦽‍➡' => 'người đàn ông trên xe lăn tay: màu da sáng, hướng sang phải',
    '👨🏼‍🦽‍➡' => 'người đàn ông trên xe lăn tay: màu da sáng trung bình, hướng sang phải',
    '👨🏽‍🦽‍➡' => 'người đàn ông trên xe lăn tay: màu da trung bình, hướng sang phải',
    '👨🏾‍🦽‍➡' => 'người đàn ông trên xe lăn tay: màu da tối trung bình, hướng sang phải',
    '👨🏿‍🦽‍➡' => 'người đàn ông trên xe lăn tay: màu da tối, hướng sang phải',
    '👩🏻‍🦽‍➡' => 'người phụ nữ trên xe lăn tay: màu da sáng, hướng sang phải',
    '👩🏼‍🦽‍➡' => 'người phụ nữ trên xe lăn tay: màu da sáng trung bình, hướng sang phải',
    '👩🏽‍🦽‍➡' => 'người phụ nữ trên xe lăn tay: màu da trung bình, hướng sang phải',
    '👩🏾‍🦽‍➡' => 'người phụ nữ trên xe lăn tay: màu da tối trung bình, hướng sang phải',
    '👩🏿‍🦽‍➡' => 'người phụ nữ trên xe lăn tay: màu da tối, hướng sang phải',
    '🏃🏻‍♀‍➡' => 'người phụ nữ đang chạy: màu da sáng, hướng sang phải',
    '🏃🏼‍♀‍➡' => 'người phụ nữ đang chạy: màu da sáng trung bình, hướng sang phải',
    '🏃🏽‍♀‍➡' => 'người phụ nữ đang chạy: màu da trung bình, hướng sang phải',
    '🏃🏾‍♀‍➡' => 'người phụ nữ đang chạy: màu da tối trung bình, hướng sang phải',
    '🏃🏿‍♀‍➡' => 'người phụ nữ đang chạy: màu da tối, hướng sang phải',
    '🏃🏻‍♂‍➡' => 'người đàn ông đang chạy: màu da sáng, hướng sang phải',
    '🏃🏼‍♂‍➡' => 'người đàn ông đang chạy: màu da sáng trung bình, hướng sang phải',
    '🏃🏽‍♂‍➡' => 'người đàn ông đang chạy: màu da trung bình, hướng sang phải',
    '🏃🏾‍♂‍➡' => 'người đàn ông đang chạy: màu da tối trung bình, hướng sang phải',
    '🏃🏿‍♂‍➡' => 'người đàn ông đang chạy: màu da tối, hướng sang phải',
    '🫱🏻‍🫲🏼' => 'bắt tay: màu da sáng, màu da sáng trung bình',
    '🫱🏻‍🫲🏽' => 'bắt tay: màu da sáng, màu da trung bình',
    '🫱🏻‍🫲🏾' => 'bắt tay: màu da sáng, màu da tối trung bình',
    '🫱🏻‍🫲🏿' => 'bắt tay: màu da sáng, màu da tối',
    '🫱🏼‍🫲🏻' => 'bắt tay: màu da sáng trung bình, màu da sáng',
    '🫱🏼‍🫲🏽' => 'bắt tay: màu da sáng trung bình, màu da trung bình',
    '🫱🏼‍🫲🏾' => 'bắt tay: màu da sáng trung bình, màu da tối trung bình',
    '🫱🏼‍🫲🏿' => 'bắt tay: màu da sáng trung bình, màu da tối',
    '🫱🏽‍🫲🏻' => 'bắt tay: màu da trung bình, màu da sáng',
    '🫱🏽‍🫲🏼' => 'bắt tay: màu da trung bình, màu da sáng trung bình',
    '🫱🏽‍🫲🏾' => 'bắt tay: màu da trung bình, màu da tối trung bình',
    '🫱🏽‍🫲🏿' => 'bắt tay: màu da trung bình, màu da tối',
    '🫱🏾‍🫲🏻' => 'bắt tay: màu da tối trung bình, màu da sáng',
    '🫱🏾‍🫲🏼' => 'bắt tay: màu da tối trung bình, màu da sáng trung bình',
    '🫱🏾‍🫲🏽' => 'bắt tay: màu da tối trung bình, màu da trung bình',
    '🫱🏾‍🫲🏿' => 'bắt tay: màu da tối trung bình, màu da tối',
    '🫱🏿‍🫲🏻' => 'bắt tay: màu da tối, màu da sáng',
    '🫱🏿‍🫲🏼' => 'bắt tay: màu da tối, màu da sáng trung bình',
    '🫱🏿‍🫲🏽' => 'bắt tay: màu da tối, màu da trung bình',
    '🫱🏿‍🫲🏾' => 'bắt tay: màu da tối, màu da tối trung bình',
    '🚶‍♀‍➡' => 'người phụ nữ đi bộ: hướng sang phải',
    '🚶‍♂‍➡' => 'người đàn ông đi bộ: hướng sang phải',
    '🧎‍♀‍➡' => 'người phụ nữ quỳ: hướng sang phải',
    '🧎‍♂‍➡' => 'người đàn ông quỳ: hướng sang phải',
    '🧑‍🦯‍➡' => 'người với gậy dò đường: hướng sang phải',
    '👨‍🦯‍➡' => 'người đàn ông với gậy dò đường: hướng sang phải',
    '👩‍🦯‍➡' => 'người phụ nữ với gậy dò đường: hướng sang phải',
    '🧑‍🦼‍➡' => 'người trên xe lăn điện: hướng sang phải',
    '👨‍🦼‍➡' => 'người đàn ông trên xe lăn điện: hướng sang phải',
    '👩‍🦼‍➡' => 'người phụ nữ trên xe lăn điện: hướng sang phải',
    '🧑‍🦽‍➡' => 'người trên xe lăn tay: hướng sang phải',
    '👨‍🦽‍➡' => 'người đàn ông trên xe lăn tay: hướng sang phải',
    '👩‍🦽‍➡' => 'người phụ nữ trên xe lăn tay: hướng sang phải',
    '🏃‍♀‍➡' => 'người phụ nữ đang chạy: hướng sang phải',
    '🏃‍♂‍➡' => 'người đàn ông đang chạy: hướng sang phải',
    '👩‍❤‍👨' => 'cặp đôi với trái tim: phụ nữ, đàn ông',
    '👨‍❤‍👨' => 'cặp đôi với trái tim: đàn ông, đàn ông',
    '👩‍❤‍👩' => 'cặp đôi với trái tim: phụ nữ, phụ nữ',
    '👨‍👩‍👦' => 'gia đình: đàn ông, phụ nữ, con trai',
    '👨‍👩‍👧' => 'gia đình: đàn ông, phụ nữ, con gái',
    '👨‍👨‍👦' => 'gia đình: đàn ông, đàn ông, con trai',
    '👨‍👨‍👧' => 'gia đình: đàn ông, đàn ông, con gái',
    '👩‍👩‍👦' => 'gia đình: phụ nữ, phụ nữ, con trai',
    '👩‍👩‍👧' => 'gia đình: phụ nữ, phụ nữ, con gái',
    '👨‍👦‍👦' => 'gia đình: đàn ông, con trai, con trai',
    '👨‍👧‍👦' => 'gia đình: đàn ông, con gái, con trai',
    '👨‍👧‍👧' => 'gia đình: đàn ông, con gái, con gái',
    '👩‍👦‍👦' => 'gia đình: phụ nữ, con trai, con trai',
    '👩‍👧‍👦' => 'gia đình: phụ nữ, con gái, con trai',
    '👩‍👧‍👧' => 'gia đình: phụ nữ, con gái, con gái',
    '🧑‍🤝‍🧑' => 'hai người nắm tay',
    '🧑‍🧑‍🧒' => 'gia đình: người lớn, người lớn, trẻ em',
    '🧑‍🧒‍🧒' => 'gia đình: người lớn, trẻ em, trẻ em',
    '🧑🏻‍🦰' => 'người: màu da sáng, tóc đỏ',
    '🧑🏼‍🦰' => 'người: màu da sáng trung bình, tóc đỏ',
    '🧑🏽‍🦰' => 'người: màu da trung bình, tóc đỏ',
    '🧑🏾‍🦰' => 'người: màu da tối trung bình, tóc đỏ',
    '🧑🏿‍🦰' => 'người: màu da tối, tóc đỏ',
    '🧑🏻‍🦱' => 'người: màu da sáng, tóc xoăn',
    '🧑🏼‍🦱' => 'người: màu da sáng trung bình, tóc xoăn',
    '🧑🏽‍🦱' => 'người: màu da trung bình, tóc xoăn',
    '🧑🏾‍🦱' => 'người: màu da tối trung bình, tóc xoăn',
    '🧑🏿‍🦱' => 'người: màu da tối, tóc xoăn',
    '🧑🏻‍🦳' => 'người: màu da sáng, tóc trắng',
    '🧑🏼‍🦳' => 'người: màu da sáng trung bình, tóc trắng',
    '🧑🏽‍🦳' => 'người: màu da trung bình, tóc trắng',
    '🧑🏾‍🦳' => 'người: màu da tối trung bình, tóc trắng',
    '🧑🏿‍🦳' => 'người: màu da tối, tóc trắng',
    '🧑🏻‍🦲' => 'người: màu da sáng, hói',
    '🧑🏼‍🦲' => 'người: màu da sáng trung bình, hói',
    '🧑🏽‍🦲' => 'người: màu da trung bình, hói',
    '🧑🏾‍🦲' => 'người: màu da tối trung bình, hói',
    '🧑🏿‍🦲' => 'người: màu da tối, hói',
    '🧑🏻‍🩰' => 'vũ công ba lê: màu da sáng',
    '🧑🏼‍🩰' => 'vũ công ba lê: màu da sáng trung bình',
    '🧑🏽‍🩰' => 'vũ công ba lê: màu da trung bình',
    '🧑🏾‍🩰' => 'vũ công ba lê: màu da tối trung bình',
    '🧑🏿‍🩰' => 'vũ công ba lê: màu da tối',
    '🧔🏻‍♂' => 'người đàn ông có râu: màu da sáng',
    '🧔🏼‍♂' => 'người đàn ông có râu: màu da sáng trung bình',
    '🧔🏽‍♂' => 'người đàn ông có râu: màu da trung bình',
    '🧔🏾‍♂' => 'người đàn ông có râu: màu da tối trung bình',
    '🧔🏿‍♂' => 'người đàn ông có râu: màu da tối',
    '👱🏻‍♂' => 'người đàn ông tóc vàng hoe: màu da sáng',
    '👱🏼‍♂' => 'người đàn ông tóc vàng hoe: màu da sáng trung bình',
    '👱🏽‍♂' => 'người đàn ông tóc vàng hoe: màu da trung bình',
    '👱🏾‍♂' => 'người đàn ông tóc vàng hoe: màu da tối trung bình',
    '👱🏿‍♂' => 'người đàn ông tóc vàng hoe: màu da tối',
    '👨🏻‍🦰' => 'đàn ông: màu da sáng, tóc đỏ',
    '👨🏼‍🦰' => 'đàn ông: màu da sáng trung bình, tóc đỏ',
    '👨🏽‍🦰' => 'đàn ông: màu da trung bình, tóc đỏ',
    '👨🏾‍🦰' => 'đàn ông: màu da tối trung bình, tóc đỏ',
    '👨🏿‍🦰' => 'đàn ông: màu da tối, tóc đỏ',
    '👨🏻‍🦱' => 'đàn ông: màu da sáng, tóc xoăn',
    '👨🏼‍🦱' => 'đàn ông: màu da sáng trung bình, tóc xoăn',
    '👨🏽‍🦱' => 'đàn ông: màu da trung bình, tóc xoăn',
    '👨🏾‍🦱' => 'đàn ông: màu da tối trung bình, tóc xoăn',
    '👨🏿‍🦱' => 'đàn ông: màu da tối, tóc xoăn',
    '👨🏻‍🦳' => 'đàn ông: màu da sáng, tóc trắng',
    '👨🏼‍🦳' => 'đàn ông: màu da sáng trung bình, tóc trắng',
    '👨🏽‍🦳' => 'đàn ông: màu da trung bình, tóc trắng',
    '👨🏾‍🦳' => 'đàn ông: màu da tối trung bình, tóc trắng',
    '👨🏿‍🦳' => 'đàn ông: màu da tối, tóc trắng',
    '👨🏻‍🦲' => 'đàn ông: màu da sáng, hói',
    '👨🏼‍🦲' => 'đàn ông: màu da sáng trung bình, hói',
    '👨🏽‍🦲' => 'đàn ông: màu da trung bình, hói',
    '👨🏾‍🦲' => 'đàn ông: màu da tối trung bình, hói',
    '👨🏿‍🦲' => 'đàn ông: màu da tối, hói',
    '🧔🏻‍♀' => 'người phụ nữ có râu: màu da sáng',
    '🧔🏼‍♀' => 'người phụ nữ có râu: màu da sáng trung bình',
    '🧔🏽‍♀' => 'người phụ nữ có râu: màu da trung bình',
    '🧔🏾‍♀' => 'người phụ nữ có râu: màu da tối trung bình',
    '🧔🏿‍♀' => 'người phụ nữ có râu: màu da tối',
    '👱🏻‍♀' => 'người phụ nữ tóc vàng hoe: màu da sáng',
    '👱🏼‍♀' => 'người phụ nữ tóc vàng hoe: màu da sáng trung bình',
    '👱🏽‍♀' => 'người phụ nữ tóc vàng hoe: màu da trung bình',
    '👱🏾‍♀' => 'người phụ nữ tóc vàng hoe: màu da tối trung bình',
    '👱🏿‍♀' => 'người phụ nữ tóc vàng hoe: màu da tối',
    '👩🏻‍🦰' => 'phụ nữ: màu da sáng, tóc đỏ',
    '👩🏼‍🦰' => 'phụ nữ: màu da sáng trung bình, tóc đỏ',
    '👩🏽‍🦰' => 'phụ nữ: màu da trung bình, tóc đỏ',
    '👩🏾‍🦰' => 'phụ nữ: màu da tối trung bình, tóc đỏ',
    '👩🏿‍🦰' => 'phụ nữ: màu da tối, tóc đỏ',
    '👩🏻‍🦱' => 'phụ nữ: màu da sáng, tóc xoăn',
    '👩🏼‍🦱' => 'phụ nữ: màu da sáng trung bình, tóc xoăn',
    '👩🏽‍🦱' => 'phụ nữ: màu da trung bình, tóc xoăn',
    '👩🏾‍🦱' => 'phụ nữ: màu da tối trung bình, tóc xoăn',
    '👩🏿‍🦱' => 'phụ nữ: màu da tối, tóc xoăn',
    '👩🏻‍🦳' => 'phụ nữ: màu da sáng, tóc trắng',
    '👩🏼‍🦳' => 'phụ nữ: màu da sáng trung bình, tóc trắng',
    '👩🏽‍🦳' => 'phụ nữ: màu da trung bình, tóc trắng',
    '👩🏾‍🦳' => 'phụ nữ: màu da tối trung bình, tóc trắng',
    '👩🏿‍🦳' => 'phụ nữ: màu da tối, tóc trắng',
    '👩🏻‍🦲' => 'phụ nữ: màu da sáng, hói',
    '👩🏼‍🦲' => 'phụ nữ: màu da sáng trung bình, hói',
    '👩🏽‍🦲' => 'phụ nữ: màu da trung bình, hói',
    '👩🏾‍🦲' => 'phụ nữ: màu da tối trung bình, hói',
    '👩🏿‍🦲' => 'phụ nữ: màu da tối, hói',
    '🙍🏻‍♂' => 'người đàn ông cau mày: màu da sáng',
    '🙍🏼‍♂' => 'người đàn ông cau mày: màu da sáng trung bình',
    '🙍🏽‍♂' => 'người đàn ông cau mày: màu da trung bình',
    '🙍🏾‍♂' => 'người đàn ông cau mày: màu da tối trung bình',
    '🙍🏿‍♂' => 'người đàn ông cau mày: màu da tối',
    '🙍🏻‍♀' => 'người phụ nữ cau mày: màu da sáng',
    '🙍🏼‍♀' => 'người phụ nữ cau mày: màu da sáng trung bình',
    '🙍🏽‍♀' => 'người phụ nữ cau mày: màu da trung bình',
    '🙍🏾‍♀' => 'người phụ nữ cau mày: màu da tối trung bình',
    '🙍🏿‍♀' => 'người phụ nữ cau mày: màu da tối',
    '🙎🏻‍♂' => 'người đàn ông bĩu môi: màu da sáng',
    '🙎🏼‍♂' => 'người đàn ông bĩu môi: màu da sáng trung bình',
    '🙎🏽‍♂' => 'người đàn ông bĩu môi: màu da trung bình',
    '🙎🏾‍♂' => 'người đàn ông bĩu môi: màu da tối trung bình',
    '🙎🏿‍♂' => 'người đàn ông bĩu môi: màu da tối',
    '🙎🏻‍♀' => 'người phụ nữ bĩu môi: màu da sáng',
    '🙎🏼‍♀' => 'người phụ nữ bĩu môi: màu da sáng trung bình',
    '🙎🏽‍♀' => 'người phụ nữ bĩu môi: màu da trung bình',
    '🙎🏾‍♀' => 'người phụ nữ bĩu môi: màu da tối trung bình',
    '🙎🏿‍♀' => 'người phụ nữ bĩu môi: màu da tối',
    '🙅🏻‍♂' => 'người đàn ông ra hiệu phản đối: màu da sáng',
    '🙅🏼‍♂' => 'người đàn ông ra hiệu phản đối: màu da sáng trung bình',
    '🙅🏽‍♂' => 'người đàn ông ra hiệu phản đối: màu da trung bình',
    '🙅🏾‍♂' => 'người đàn ông ra hiệu phản đối: màu da tối trung bình',
    '🙅🏿‍♂' => 'người đàn ông ra hiệu phản đối: màu da tối',
    '🙅🏻‍♀' => 'người phụ nữ ra hiệu phản đối: màu da sáng',
    '🙅🏼‍♀' => 'người phụ nữ ra hiệu phản đối: màu da sáng trung bình',
    '🙅🏽‍♀' => 'người phụ nữ ra hiệu phản đối: màu da trung bình',
    '🙅🏾‍♀' => 'người phụ nữ ra hiệu phản đối: màu da tối trung bình',
    '🙅🏿‍♀' => 'người phụ nữ ra hiệu phản đối: màu da tối',
    '🙆🏻‍♂' => 'người đàn ông ra hiệu đồng ý: màu da sáng',
    '🙆🏼‍♂' => 'người đàn ông ra hiệu đồng ý: màu da sáng trung bình',
    '🙆🏽‍♂' => 'người đàn ông ra hiệu đồng ý: màu da trung bình',
    '🙆🏾‍♂' => 'người đàn ông ra hiệu đồng ý: màu da tối trung bình',
    '🙆🏿‍♂' => 'người đàn ông ra hiệu đồng ý: màu da tối',
    '🙆🏻‍♀' => 'người phụ nữ ra hiệu đồng ý: màu da sáng',
    '🙆🏼‍♀' => 'người phụ nữ ra hiệu đồng ý: màu da sáng trung bình',
    '🙆🏽‍♀' => 'người phụ nữ ra hiệu đồng ý: màu da trung bình',
    '🙆🏾‍♀' => 'người phụ nữ ra hiệu đồng ý: màu da tối trung bình',
    '🙆🏿‍♀' => 'người phụ nữ ra hiệu đồng ý: màu da tối',
    '💁🏻‍♂' => 'người đàn ông ra hiệu trợ giúp: màu da sáng',
    '💁🏼‍♂' => 'người đàn ông ra hiệu trợ giúp: màu da sáng trung bình',
    '💁🏽‍♂' => 'người đàn ông ra hiệu trợ giúp: màu da trung bình',
    '💁🏾‍♂' => 'người đàn ông ra hiệu trợ giúp: màu da tối trung bình',
    '💁🏿‍♂' => 'người đàn ông ra hiệu trợ giúp: màu da tối',
    '💁🏻‍♀' => 'người phụ nữ ra hiệu trợ giúp: màu da sáng',
    '💁🏼‍♀' => 'người phụ nữ ra hiệu trợ giúp: màu da sáng trung bình',
    '💁🏽‍♀' => 'người phụ nữ ra hiệu trợ giúp: màu da trung bình',
    '💁🏾‍♀' => 'người phụ nữ ra hiệu trợ giúp: màu da tối trung bình',
    '💁🏿‍♀' => 'người phụ nữ ra hiệu trợ giúp: màu da tối',
    '🙋🏻‍♂' => 'người đàn ông giơ cao tay: màu da sáng',
    '🙋🏼‍♂' => 'người đàn ông giơ cao tay: màu da sáng trung bình',
    '🙋🏽‍♂' => 'người đàn ông giơ cao tay: màu da trung bình',
    '🙋🏾‍♂' => 'người đàn ông giơ cao tay: màu da tối trung bình',
    '🙋🏿‍♂' => 'người đàn ông giơ cao tay: màu da tối',
    '🙋🏻‍♀' => 'người phụ nữ giơ cao tay: màu da sáng',
    '🙋🏼‍♀' => 'người phụ nữ giơ cao tay: màu da sáng trung bình',
    '🙋🏽‍♀' => 'người phụ nữ giơ cao tay: màu da trung bình',
    '🙋🏾‍♀' => 'người phụ nữ giơ cao tay: màu da tối trung bình',
    '🙋🏿‍♀' => 'người phụ nữ giơ cao tay: màu da tối',
    '🧏🏻‍♂' => 'người đàn ông khiếm thính: màu da sáng',
    '🧏🏼‍♂' => 'người đàn ông khiếm thính: màu da sáng trung bình',
    '🧏🏽‍♂' => 'người đàn ông khiếm thính: màu da trung bình',
    '🧏🏾‍♂' => 'người đàn ông khiếm thính: màu da tối trung bình',
    '🧏🏿‍♂' => 'người đàn ông khiếm thính: màu da tối',
    '🧏🏻‍♀' => 'người phụ nữ khiếm thính: màu da sáng',
    '🧏🏼‍♀' => 'người phụ nữ khiếm thính: màu da sáng trung bình',
    '🧏🏽‍♀' => 'người phụ nữ khiếm thính: màu da trung bình',
    '🧏🏾‍♀' => 'người phụ nữ khiếm thính: màu da tối trung bình',
    '🧏🏿‍♀' => 'người phụ nữ khiếm thính: màu da tối',
    '🙇🏻‍♂' => 'người đàn ông cúi đầu: màu da sáng',
    '🙇🏼‍♂' => 'người đàn ông cúi đầu: màu da sáng trung bình',
    '🙇🏽‍♂' => 'người đàn ông cúi đầu: màu da trung bình',
    '🙇🏾‍♂' => 'người đàn ông cúi đầu: màu da tối trung bình',
    '🙇🏿‍♂' => 'người đàn ông cúi đầu: màu da tối',
    '🙇🏻‍♀' => 'người phụ nữ cúi đầu: màu da sáng',
    '🙇🏼‍♀' => 'người phụ nữ cúi đầu: màu da sáng trung bình',
    '🙇🏽‍♀' => 'người phụ nữ cúi đầu: màu da trung bình',
    '🙇🏾‍♀' => 'người phụ nữ cúi đầu: màu da tối trung bình',
    '🙇🏿‍♀' => 'người phụ nữ cúi đầu: màu da tối',
    '🤦🏻‍♂' => 'người đàn ông lấy tay che mặt: màu da sáng',
    '🤦🏼‍♂' => 'người đàn ông lấy tay che mặt: màu da sáng trung bình',
    '🤦🏽‍♂' => 'người đàn ông lấy tay che mặt: màu da trung bình',
    '🤦🏾‍♂' => 'người đàn ông lấy tay che mặt: màu da tối trung bình',
    '🤦🏿‍♂' => 'người đàn ông lấy tay che mặt: màu da tối',
    '🤦🏻‍♀' => 'người phụ nữ lấy tay che mặt: màu da sáng',
    '🤦🏼‍♀' => 'người phụ nữ lấy tay che mặt: màu da sáng trung bình',
    '🤦🏽‍♀' => 'người phụ nữ lấy tay che mặt: màu da trung bình',
    '🤦🏾‍♀' => 'người phụ nữ lấy tay che mặt: màu da tối trung bình',
    '🤦🏿‍♀' => 'người phụ nữ lấy tay che mặt: màu da tối',
    '🤷🏻‍♂' => 'người đàn ông nhún vai: màu da sáng',
    '🤷🏼‍♂' => 'người đàn ông nhún vai: màu da sáng trung bình',
    '🤷🏽‍♂' => 'người đàn ông nhún vai: màu da trung bình',
    '🤷🏾‍♂' => 'người đàn ông nhún vai: màu da tối trung bình',
    '🤷🏿‍♂' => 'người đàn ông nhún vai: màu da tối',
    '🤷🏻‍♀' => 'người phụ nữ nhún vai: màu da sáng',
    '🤷🏼‍♀' => 'người phụ nữ nhún vai: màu da sáng trung bình',
    '🤷🏽‍♀' => 'người phụ nữ nhún vai: màu da trung bình',
    '🤷🏾‍♀' => 'người phụ nữ nhún vai: màu da tối trung bình',
    '🤷🏿‍♀' => 'người phụ nữ nhún vai: màu da tối',
    '🧑🏻‍⚕' => 'nhân viên y tế: màu da sáng',
    '🧑🏼‍⚕' => 'nhân viên y tế: màu da sáng trung bình',
    '🧑🏽‍⚕' => 'nhân viên y tế: màu da trung bình',
    '🧑🏾‍⚕' => 'nhân viên y tế: màu da tối trung bình',
    '🧑🏿‍⚕' => 'nhân viên y tế: màu da tối',
    '👨🏻‍⚕' => 'nhân viên y tế nam: màu da sáng',
    '👨🏼‍⚕' => 'nhân viên y tế nam: màu da sáng trung bình',
    '👨🏽‍⚕' => 'nhân viên y tế nam: màu da trung bình',
    '👨🏾‍⚕' => 'nhân viên y tế nam: màu da tối trung bình',
    '👨🏿‍⚕' => 'nhân viên y tế nam: màu da tối',
    '👩🏻‍⚕' => 'nhân viên y tế nữ: màu da sáng',
    '👩🏼‍⚕' => 'nhân viên y tế nữ: màu da sáng trung bình',
    '👩🏽‍⚕' => 'nhân viên y tế nữ: màu da trung bình',
    '👩🏾‍⚕' => 'nhân viên y tế nữ: màu da tối trung bình',
    '👩🏿‍⚕' => 'nhân viên y tế nữ: màu da tối',
    '🧑🏻‍🎓' => 'sinh viên: màu da sáng',
    '🧑🏼‍🎓' => 'sinh viên: màu da sáng trung bình',
    '🧑🏽‍🎓' => 'sinh viên: màu da trung bình',
    '🧑🏾‍🎓' => 'sinh viên: màu da tối trung bình',
    '🧑🏿‍🎓' => 'sinh viên: màu da tối',
    '👨🏻‍🎓' => 'sinh viên nam: màu da sáng',
    '👨🏼‍🎓' => 'sinh viên nam: màu da sáng trung bình',
    '👨🏽‍🎓' => 'sinh viên nam: màu da trung bình',
    '👨🏾‍🎓' => 'sinh viên nam: màu da tối trung bình',
    '👨🏿‍🎓' => 'sinh viên nam: màu da tối',
    '👩🏻‍🎓' => 'sinh viên nữ: màu da sáng',
    '👩🏼‍🎓' => 'sinh viên nữ: màu da sáng trung bình',
    '👩🏽‍🎓' => 'sinh viên nữ: màu da trung bình',
    '👩🏾‍🎓' => 'sinh viên nữ: màu da tối trung bình',
    '👩🏿‍🎓' => 'sinh viên nữ: màu da tối',
    '🧑🏻‍🏫' => 'giáo viên: màu da sáng',
    '🧑🏼‍🏫' => 'giáo viên: màu da sáng trung bình',
    '🧑🏽‍🏫' => 'giáo viên: màu da trung bình',
    '🧑🏾‍🏫' => 'giáo viên: màu da tối trung bình',
    '🧑🏿‍🏫' => 'giáo viên: màu da tối',
    '👨🏻‍🏫' => 'giáo viên nam: màu da sáng',
    '👨🏼‍🏫' => 'giáo viên nam: màu da sáng trung bình',
    '👨🏽‍🏫' => 'giáo viên nam: màu da trung bình',
    '👨🏾‍🏫' => 'giáo viên nam: màu da tối trung bình',
    '👨🏿‍🏫' => 'giáo viên nam: màu da tối',
    '👩🏻‍🏫' => 'giáo viên nữ: màu da sáng',
    '👩🏼‍🏫' => 'giáo viên nữ: màu da sáng trung bình',
    '👩🏽‍🏫' => 'giáo viên nữ: màu da trung bình',
    '👩🏾‍🏫' => 'giáo viên nữ: màu da tối trung bình',
    '👩🏿‍🏫' => 'giáo viên nữ: màu da tối',
    '🧑🏻‍⚖' => 'thẩm phán: màu da sáng',
    '🧑🏼‍⚖' => 'thẩm phán: màu da sáng trung bình',
    '🧑🏽‍⚖' => 'thẩm phán: màu da trung bình',
    '🧑🏾‍⚖' => 'thẩm phán: màu da tối trung bình',
    '🧑🏿‍⚖' => 'thẩm phán: màu da tối',
    '👨🏻‍⚖' => 'thẩm phán nam: màu da sáng',
    '👨🏼‍⚖' => 'thẩm phán nam: màu da sáng trung bình',
    '👨🏽‍⚖' => 'thẩm phán nam: màu da trung bình',
    '👨🏾‍⚖' => 'thẩm phán nam: màu da tối trung bình',
    '👨🏿‍⚖' => 'thẩm phán nam: màu da tối',
    '👩🏻‍⚖' => 'thẩm phán nữ: màu da sáng',
    '👩🏼‍⚖' => 'thẩm phán nữ: màu da sáng trung bình',
    '👩🏽‍⚖' => 'thẩm phán nữ: màu da trung bình',
    '👩🏾‍⚖' => 'thẩm phán nữ: màu da tối trung bình',
    '👩🏿‍⚖' => 'thẩm phán nữ: màu da tối',
    '🧑🏻‍🌾' => 'nông dân: màu da sáng',
    '🧑🏼‍🌾' => 'nông dân: màu da sáng trung bình',
    '🧑🏽‍🌾' => 'nông dân: màu da trung bình',
    '🧑🏾‍🌾' => 'nông dân: màu da tối trung bình',
    '🧑🏿‍🌾' => 'nông dân: màu da tối',
    '👨🏻‍🌾' => 'nông dân nam: màu da sáng',
    '👨🏼‍🌾' => 'nông dân nam: màu da sáng trung bình',
    '👨🏽‍🌾' => 'nông dân nam: màu da trung bình',
    '👨🏾‍🌾' => 'nông dân nam: màu da tối trung bình',
    '👨🏿‍🌾' => 'nông dân nam: màu da tối',
    '👩🏻‍🌾' => 'nông dân nữ: màu da sáng',
    '👩🏼‍🌾' => 'nông dân nữ: màu da sáng trung bình',
    '👩🏽‍🌾' => 'nông dân nữ: màu da trung bình',
    '👩🏾‍🌾' => 'nông dân nữ: màu da tối trung bình',
    '👩🏿‍🌾' => 'nông dân nữ: màu da tối',
    '🧑🏻‍🍳' => 'đầu bếp: màu da sáng',
    '🧑🏼‍🍳' => 'đầu bếp: màu da sáng trung bình',
    '🧑🏽‍🍳' => 'đầu bếp: màu da trung bình',
    '🧑🏾‍🍳' => 'đầu bếp: màu da tối trung bình',
    '🧑🏿‍🍳' => 'đầu bếp: màu da tối',
    '👨🏻‍🍳' => 'đầu bếp nam: màu da sáng',
    '👨🏼‍🍳' => 'đầu bếp nam: màu da sáng trung bình',
    '👨🏽‍🍳' => 'đầu bếp nam: màu da trung bình',
    '👨🏾‍🍳' => 'đầu bếp nam: màu da tối trung bình',
    '👨🏿‍🍳' => 'đầu bếp nam: màu da tối',
    '👩🏻‍🍳' => 'đầu bếp nữ: màu da sáng',
    '👩🏼‍🍳' => 'đầu bếp nữ: màu da sáng trung bình',
    '👩🏽‍🍳' => 'đầu bếp nữ: màu da trung bình',
    '👩🏾‍🍳' => 'đầu bếp nữ: màu da tối trung bình',
    '👩🏿‍🍳' => 'đầu bếp nữ: màu da tối',
    '🧑🏻‍🔧' => 'thợ máy: màu da sáng',
    '🧑🏼‍🔧' => 'thợ máy: màu da sáng trung bình',
    '🧑🏽‍🔧' => 'thợ máy: màu da trung bình',
    '🧑🏾‍🔧' => 'thợ máy: màu da tối trung bình',
    '🧑🏿‍🔧' => 'thợ máy: màu da tối',
    '👨🏻‍🔧' => 'thợ máy nam: màu da sáng',
    '👨🏼‍🔧' => 'thợ máy nam: màu da sáng trung bình',
    '👨🏽‍🔧' => 'thợ máy nam: màu da trung bình',
    '👨🏾‍🔧' => 'thợ máy nam: màu da tối trung bình',
    '👨🏿‍🔧' => 'thợ máy nam: màu da tối',
    '👩🏻‍🔧' => 'thợ máy nữ: màu da sáng',
    '👩🏼‍🔧' => 'thợ máy nữ: màu da sáng trung bình',
    '👩🏽‍🔧' => 'thợ máy nữ: màu da trung bình',
    '👩🏾‍🔧' => 'thợ máy nữ: màu da tối trung bình',
    '👩🏿‍🔧' => 'thợ máy nữ: màu da tối',
    '🧑🏻‍🏭' => 'công nhân nhà máy: màu da sáng',
    '🧑🏼‍🏭' => 'công nhân nhà máy: màu da sáng trung bình',
    '🧑🏽‍🏭' => 'công nhân nhà máy: màu da trung bình',
    '🧑🏾‍🏭' => 'công nhân nhà máy: màu da tối trung bình',
    '🧑🏿‍🏭' => 'công nhân nhà máy: màu da tối',
    '👨🏻‍🏭' => 'công nhân nhà máy nam: màu da sáng',
    '👨🏼‍🏭' => 'công nhân nhà máy nam: màu da sáng trung bình',
    '👨🏽‍🏭' => 'công nhân nhà máy nam: màu da trung bình',
    '👨🏾‍🏭' => 'công nhân nhà máy nam: màu da tối trung bình',
    '👨🏿‍🏭' => 'công nhân nhà máy nam: màu da tối',
    '👩🏻‍🏭' => 'công nhân nhà máy nữ: màu da sáng',
    '👩🏼‍🏭' => 'công nhân nhà máy nữ: màu da sáng trung bình',
    '👩🏽‍🏭' => 'công nhân nhà máy nữ: màu da trung bình',
    '👩🏾‍🏭' => 'công nhân nhà máy nữ: màu da tối trung bình',
    '👩🏿‍🏭' => 'công nhân nhà máy nữ: màu da tối',
    '🧑🏻‍💼' => 'nhân viên văn phòng: màu da sáng',
    '🧑🏼‍💼' => 'nhân viên văn phòng: màu da sáng trung bình',
    '🧑🏽‍💼' => 'nhân viên văn phòng: màu da trung bình',
    '🧑🏾‍💼' => 'nhân viên văn phòng: màu da tối trung bình',
    '🧑🏿‍💼' => 'nhân viên văn phòng: màu da tối',
    '👨🏻‍💼' => 'nhân viên văn phòng nam: màu da sáng',
    '👨🏼‍💼' => 'nhân viên văn phòng nam: màu da sáng trung bình',
    '👨🏽‍💼' => 'nhân viên văn phòng nam: màu da trung bình',
    '👨🏾‍💼' => 'nhân viên văn phòng nam: màu da tối trung bình',
    '👨🏿‍💼' => 'nhân viên văn phòng nam: màu da tối',
    '👩🏻‍💼' => 'nhân viên văn phòng nữ: màu da sáng',
    '👩🏼‍💼' => 'nhân viên văn phòng nữ: màu da sáng trung bình',
    '👩🏽‍💼' => 'nhân viên văn phòng nữ: màu da trung bình',
    '👩🏾‍💼' => 'nhân viên văn phòng nữ: màu da tối trung bình',
    '👩🏿‍💼' => 'nhân viên văn phòng nữ: màu da tối',
    '🧑🏻‍🔬' => 'nhà khoa học: màu da sáng',
    '🧑🏼‍🔬' => 'nhà khoa học: màu da sáng trung bình',
    '🧑🏽‍🔬' => 'nhà khoa học: màu da trung bình',
    '🧑🏾‍🔬' => 'nhà khoa học: màu da tối trung bình',
    '🧑🏿‍🔬' => 'nhà khoa học: màu da tối',
    '👨🏻‍🔬' => 'nhà khoa học nam: màu da sáng',
    '👨🏼‍🔬' => 'nhà khoa học nam: màu da sáng trung bình',
    '👨🏽‍🔬' => 'nhà khoa học nam: màu da trung bình',
    '👨🏾‍🔬' => 'nhà khoa học nam: màu da tối trung bình',
    '👨🏿‍🔬' => 'nhà khoa học nam: màu da tối',
    '👩🏻‍🔬' => 'nhà khoa học nữ: màu da sáng',
    '👩🏼‍🔬' => 'nhà khoa học nữ: màu da sáng trung bình',
    '👩🏽‍🔬' => 'nhà khoa học nữ: màu da trung bình',
    '👩🏾‍🔬' => 'nhà khoa học nữ: màu da tối trung bình',
    '👩🏿‍🔬' => 'nhà khoa học nữ: màu da tối',
    '🧑🏻‍💻' => 'kỹ sư công nghệ: màu da sáng',
    '🧑🏼‍💻' => 'kỹ sư công nghệ: màu da sáng trung bình',
    '🧑🏽‍💻' => 'kỹ sư công nghệ: màu da trung bình',
    '🧑🏾‍💻' => 'kỹ sư công nghệ: màu da tối trung bình',
    '🧑🏿‍💻' => 'kỹ sư công nghệ: màu da tối',
    '👨🏻‍💻' => 'kỹ sư công nghệ nam: màu da sáng',
    '👨🏼‍💻' => 'kỹ sư công nghệ nam: màu da sáng trung bình',
    '👨🏽‍💻' => 'kỹ sư công nghệ nam: màu da trung bình',
    '👨🏾‍💻' => 'kỹ sư công nghệ nam: màu da tối trung bình',
    '👨🏿‍💻' => 'kỹ sư công nghệ nam: màu da tối',
    '👩🏻‍💻' => 'kỹ sư công nghệ nữ: màu da sáng',
    '👩🏼‍💻' => 'kỹ sư công nghệ nữ: màu da sáng trung bình',
    '👩🏽‍💻' => 'kỹ sư công nghệ nữ: màu da trung bình',
    '👩🏾‍💻' => 'kỹ sư công nghệ nữ: màu da tối trung bình',
    '👩🏿‍💻' => 'kỹ sư công nghệ nữ: màu da tối',
    '🧑🏻‍🎤' => 'ca sĩ: màu da sáng',
    '🧑🏼‍🎤' => 'ca sĩ: màu da sáng trung bình',
    '🧑🏽‍🎤' => 'ca sĩ: màu da trung bình',
    '🧑🏾‍🎤' => 'ca sĩ: màu da tối trung bình',
    '🧑🏿‍🎤' => 'ca sĩ: màu da tối',
    '👨🏻‍🎤' => 'nam ca sĩ: màu da sáng',
    '👨🏼‍🎤' => 'nam ca sĩ: màu da sáng trung bình',
    '👨🏽‍🎤' => 'nam ca sĩ: màu da trung bình',
    '👨🏾‍🎤' => 'nam ca sĩ: màu da tối trung bình',
    '👨🏿‍🎤' => 'nam ca sĩ: màu da tối',
    '👩🏻‍🎤' => 'nữ ca sĩ: màu da sáng',
    '👩🏼‍🎤' => 'nữ ca sĩ: màu da sáng trung bình',
    '👩🏽‍🎤' => 'nữ ca sĩ: màu da trung bình',
    '👩🏾‍🎤' => 'nữ ca sĩ: màu da tối trung bình',
    '👩🏿‍🎤' => 'nữ ca sĩ: màu da tối',
    '🧑🏻‍🎨' => 'họa sĩ: màu da sáng',
    '🧑🏼‍🎨' => 'họa sĩ: màu da sáng trung bình',
    '🧑🏽‍🎨' => 'họa sĩ: màu da trung bình',
    '🧑🏾‍🎨' => 'họa sĩ: màu da tối trung bình',
    '🧑🏿‍🎨' => 'họa sĩ: màu da tối',
    '👨🏻‍🎨' => 'họa sĩ nam: màu da sáng',
    '👨🏼‍🎨' => 'họa sĩ nam: màu da sáng trung bình',
    '👨🏽‍🎨' => 'họa sĩ nam: màu da trung bình',
    '👨🏾‍🎨' => 'họa sĩ nam: màu da tối trung bình',
    '👨🏿‍🎨' => 'họa sĩ nam: màu da tối',
    '👩🏻‍🎨' => 'họa sĩ nữ: màu da sáng',
    '👩🏼‍🎨' => 'họa sĩ nữ: màu da sáng trung bình',
    '👩🏽‍🎨' => 'họa sĩ nữ: màu da trung bình',
    '👩🏾‍🎨' => 'họa sĩ nữ: màu da tối trung bình',
    '👩🏿‍🎨' => 'họa sĩ nữ: màu da tối',
    '🧑🏻‍✈' => 'phi công: màu da sáng',
    '🧑🏼‍✈' => 'phi công: màu da sáng trung bình',
    '🧑🏽‍✈' => 'phi công: màu da trung bình',
    '🧑🏾‍✈' => 'phi công: màu da tối trung bình',
    '🧑🏿‍✈' => 'phi công: màu da tối',
    '👨🏻‍✈' => 'phi công nam: màu da sáng',
    '👨🏼‍✈' => 'phi công nam: màu da sáng trung bình',
    '👨🏽‍✈' => 'phi công nam: màu da trung bình',
    '👨🏾‍✈' => 'phi công nam: màu da tối trung bình',
    '👨🏿‍✈' => 'phi công nam: màu da tối',
    '👩🏻‍✈' => 'phi công nữ: màu da sáng',
    '👩🏼‍✈' => 'phi công nữ: màu da sáng trung bình',
    '👩🏽‍✈' => 'phi công nữ: màu da trung bình',
    '👩🏾‍✈' => 'phi công nữ: màu da tối trung bình',
    '👩🏿‍✈' => 'phi công nữ: màu da tối',
    '🧑🏻‍🚀' => 'phi hành gia: màu da sáng',
    '🧑🏼‍🚀' => 'phi hành gia: màu da sáng trung bình',
    '🧑🏽‍🚀' => 'phi hành gia: màu da trung bình',
    '🧑🏾‍🚀' => 'phi hành gia: màu da tối trung bình',
    '🧑🏿‍🚀' => 'phi hành gia: màu da tối',
    '👨🏻‍🚀' => 'phi hành gia nam: màu da sáng',
    '👨🏼‍🚀' => 'phi hành gia nam: màu da sáng trung bình',
    '👨🏽‍🚀' => 'phi hành gia nam: màu da trung bình',
    '👨🏾‍🚀' => 'phi hành gia nam: màu da tối trung bình',
    '👨🏿‍🚀' => 'phi hành gia nam: màu da tối',
    '👩🏻‍🚀' => 'phi hành gia nữ: màu da sáng',
    '👩🏼‍🚀' => 'phi hành gia nữ: màu da sáng trung bình',
    '👩🏽‍🚀' => 'phi hành gia nữ: màu da trung bình',
    '👩🏾‍🚀' => 'phi hành gia nữ: màu da tối trung bình',
    '👩🏿‍🚀' => 'phi hành gia nữ: màu da tối',
    '🧑🏻‍🚒' => 'lính cứu hỏa: màu da sáng',
    '🧑🏼‍🚒' => 'lính cứu hỏa: màu da sáng trung bình',
    '🧑🏽‍🚒' => 'lính cứu hỏa: màu da trung bình',
    '🧑🏾‍🚒' => 'lính cứu hỏa: màu da tối trung bình',
    '🧑🏿‍🚒' => 'lính cứu hỏa: màu da tối',
    '👨🏻‍🚒' => 'lính cứu hỏa nam: màu da sáng',
    '👨🏼‍🚒' => 'lính cứu hỏa nam: màu da sáng trung bình',
    '👨🏽‍🚒' => 'lính cứu hỏa nam: màu da trung bình',
    '👨🏾‍🚒' => 'lính cứu hỏa nam: màu da tối trung bình',
    '👨🏿‍🚒' => 'lính cứu hỏa nam: màu da tối',
    '👩🏻‍🚒' => 'lính cứu hỏa nữ: màu da sáng',
    '👩🏼‍🚒' => 'lính cứu hỏa nữ: màu da sáng trung bình',
    '👩🏽‍🚒' => 'lính cứu hỏa nữ: màu da trung bình',
    '👩🏾‍🚒' => 'lính cứu hỏa nữ: màu da tối trung bình',
    '👩🏿‍🚒' => 'lính cứu hỏa nữ: màu da tối',
    '👮🏻‍♂' => 'cảnh sát nam: màu da sáng',
    '👮🏼‍♂' => 'cảnh sát nam: màu da sáng trung bình',
    '👮🏽‍♂' => 'cảnh sát nam: màu da trung bình',
    '👮🏾‍♂' => 'cảnh sát nam: màu da tối trung bình',
    '👮🏿‍♂' => 'cảnh sát nam: màu da tối',
    '👮🏻‍♀' => 'cảnh sát nữ: màu da sáng',
    '👮🏼‍♀' => 'cảnh sát nữ: màu da sáng trung bình',
    '👮🏽‍♀' => 'cảnh sát nữ: màu da trung bình',
    '👮🏾‍♀' => 'cảnh sát nữ: màu da tối trung bình',
    '👮🏿‍♀' => 'cảnh sát nữ: màu da tối',
    '🕵🏻‍♂' => 'thám tử nam: màu da sáng',
    '🕵🏼‍♂' => 'thám tử nam: màu da sáng trung bình',
    '🕵🏽‍♂' => 'thám tử nam: màu da trung bình',
    '🕵🏾‍♂' => 'thám tử nam: màu da tối trung bình',
    '🕵🏿‍♂' => 'thám tử nam: màu da tối',
    '🕵🏻‍♀' => 'thám tử nữ: màu da sáng',
    '🕵🏼‍♀' => 'thám tử nữ: màu da sáng trung bình',
    '🕵🏽‍♀' => 'thám tử nữ: màu da trung bình',
    '🕵🏾‍♀' => 'thám tử nữ: màu da tối trung bình',
    '🕵🏿‍♀' => 'thám tử nữ: màu da tối',
    '💂🏻‍♂' => 'bảo vệ nam: màu da sáng',
    '💂🏼‍♂' => 'bảo vệ nam: màu da sáng trung bình',
    '💂🏽‍♂' => 'bảo vệ nam: màu da trung bình',
    '💂🏾‍♂' => 'bảo vệ nam: màu da tối trung bình',
    '💂🏿‍♂' => 'bảo vệ nam: màu da tối',
    '💂🏻‍♀' => 'bảo vệ nữ: màu da sáng',
    '💂🏼‍♀' => 'bảo vệ nữ: màu da sáng trung bình',
    '💂🏽‍♀' => 'bảo vệ nữ: màu da trung bình',
    '💂🏾‍♀' => 'bảo vệ nữ: màu da tối trung bình',
    '💂🏿‍♀' => 'bảo vệ nữ: màu da tối',
    '👷🏻‍♂' => 'công nhân xây dựng nam: màu da sáng',
    '👷🏼‍♂' => 'công nhân xây dựng nam: màu da sáng trung bình',
    '👷🏽‍♂' => 'công nhân xây dựng nam: màu da trung bình',
    '👷🏾‍♂' => 'công nhân xây dựng nam: màu da tối trung bình',
    '👷🏿‍♂' => 'công nhân xây dựng nam: màu da tối',
    '👷🏻‍♀' => 'công nhân xây dựng nữ: màu da sáng',
    '👷🏼‍♀' => 'công nhân xây dựng nữ: màu da sáng trung bình',
    '👷🏽‍♀' => 'công nhân xây dựng nữ: màu da trung bình',
    '👷🏾‍♀' => 'công nhân xây dựng nữ: màu da tối trung bình',
    '👷🏿‍♀' => 'công nhân xây dựng nữ: màu da tối',
    '👳🏻‍♂' => 'người đàn ông đội khăn xếp: màu da sáng',
    '👳🏼‍♂' => 'người đàn ông đội khăn xếp: màu da sáng trung bình',
    '👳🏽‍♂' => 'người đàn ông đội khăn xếp: màu da trung bình',
    '👳🏾‍♂' => 'người đàn ông đội khăn xếp: màu da tối trung bình',
    '👳🏿‍♂' => 'người đàn ông đội khăn xếp: màu da tối',
    '👳🏻‍♀' => 'người phụ nữ đội khăn xếp: màu da sáng',
    '👳🏼‍♀' => 'người phụ nữ đội khăn xếp: màu da sáng trung bình',
    '👳🏽‍♀' => 'người phụ nữ đội khăn xếp: màu da trung bình',
    '👳🏾‍♀' => 'người phụ nữ đội khăn xếp: màu da tối trung bình',
    '👳🏿‍♀' => 'người phụ nữ đội khăn xếp: màu da tối',
    '🤵🏻‍♂' => 'người đàn ông mặc bộ vest: màu da sáng',
    '🤵🏼‍♂' => 'người đàn ông mặc bộ vest: màu da sáng trung bình',
    '🤵🏽‍♂' => 'người đàn ông mặc bộ vest: màu da trung bình',
    '🤵🏾‍♂' => 'người đàn ông mặc bộ vest: màu da tối trung bình',
    '🤵🏿‍♂' => 'người đàn ông mặc bộ vest: màu da tối',
    '🤵🏻‍♀' => 'người phụ nữ mặc bộ vest: màu da sáng',
    '🤵🏼‍♀' => 'người phụ nữ mặc bộ vest: màu da sáng trung bình',
    '🤵🏽‍♀' => 'người phụ nữ mặc bộ vest: màu da trung bình',
    '🤵🏾‍♀' => 'người phụ nữ mặc bộ vest: màu da tối trung bình',
    '🤵🏿‍♀' => 'người phụ nữ mặc bộ vest: màu da tối',
    '👰🏻‍♂' => 'người đàn ông đội khăn voan: màu da sáng',
    '👰🏼‍♂' => 'người đàn ông đội khăn voan: màu da sáng trung bình',
    '👰🏽‍♂' => 'người đàn ông đội khăn voan: màu da trung bình',
    '👰🏾‍♂' => 'người đàn ông đội khăn voan: màu da tối trung bình',
    '👰🏿‍♂' => 'người đàn ông đội khăn voan: màu da tối',
    '👰🏻‍♀' => 'người phụ nữ đội khăn voan: màu da sáng',
    '👰🏼‍♀' => 'người phụ nữ đội khăn voan: màu da sáng trung bình',
    '👰🏽‍♀' => 'người phụ nữ đội khăn voan: màu da trung bình',
    '👰🏾‍♀' => 'người phụ nữ đội khăn voan: màu da tối trung bình',
    '👰🏿‍♀' => 'người phụ nữ đội khăn voan: màu da tối',
    '👩🏻‍🍼' => 'người phụ nữ cho con bú: màu da sáng',
    '👩🏼‍🍼' => 'người phụ nữ cho con bú: màu da sáng trung bình',
    '👩🏽‍🍼' => 'người phụ nữ cho con bú: màu da trung bình',
    '👩🏾‍🍼' => 'người phụ nữ cho con bú: màu da tối trung bình',
    '👩🏿‍🍼' => 'người phụ nữ cho con bú: màu da tối',
    '👨🏻‍🍼' => 'người cho con bú: màu da sáng',
    '👨🏼‍🍼' => 'người cho con bú: màu da sáng trung bình',
    '👨🏽‍🍼' => 'người cho con bú: màu da trung bình',
    '👨🏾‍🍼' => 'người cho con bú: màu da tối trung bình',
    '👨🏿‍🍼' => 'người cho con bú: màu da tối',
    '🧑🏻‍🍼' => 'người cho em bé bú: màu da sáng',
    '🧑🏼‍🍼' => 'người cho em bé bú: màu da sáng trung bình',
    '🧑🏽‍🍼' => 'người cho em bé bú: màu da trung bình',
    '🧑🏾‍🍼' => 'người cho em bé bú: màu da tối trung bình',
    '🧑🏿‍🍼' => 'người cho em bé bú: màu da tối',
    '🧑🏻‍🎄' => 'bà già Noel: màu da sáng',
    '🧑🏼‍🎄' => 'bà già Noel: màu da sáng trung bình',
    '🧑🏽‍🎄' => 'bà già Noel: màu da trung bình',
    '🧑🏾‍🎄' => 'bà già Noel: màu da tối trung bình',
    '🧑🏿‍🎄' => 'bà già Noel: màu da tối',
    '🦸🏻‍♂' => 'nam siêu anh hùng: màu da sáng',
    '🦸🏼‍♂' => 'nam siêu anh hùng: màu da sáng trung bình',
    '🦸🏽‍♂' => 'nam siêu anh hùng: màu da trung bình',
    '🦸🏾‍♂' => 'nam siêu anh hùng: màu da tối trung bình',
    '🦸🏿‍♂' => 'nam siêu anh hùng: màu da tối',
    '🦸🏻‍♀' => 'nữ siêu anh hùng: màu da sáng',
    '🦸🏼‍♀' => 'nữ siêu anh hùng: màu da sáng trung bình',
    '🦸🏽‍♀' => 'nữ siêu anh hùng: màu da trung bình',
    '🦸🏾‍♀' => 'nữ siêu anh hùng: màu da tối trung bình',
    '🦸🏿‍♀' => 'nữ siêu anh hùng: màu da tối',
    '🦹🏻‍♂' => 'nam siêu phản diện: màu da sáng',
    '🦹🏼‍♂' => 'nam siêu phản diện: màu da sáng trung bình',
    '🦹🏽‍♂' => 'nam siêu phản diện: màu da trung bình',
    '🦹🏾‍♂' => 'nam siêu phản diện: màu da tối trung bình',
    '🦹🏿‍♂' => 'nam siêu phản diện: màu da tối',
    '🦹🏻‍♀' => 'nữ siêu phản diện: màu da sáng',
    '🦹🏼‍♀' => 'nữ siêu phản diện: màu da sáng trung bình',
    '🦹🏽‍♀' => 'nữ siêu phản diện: màu da trung bình',
    '🦹🏾‍♀' => 'nữ siêu phản diện: màu da tối trung bình',
    '🦹🏿‍♀' => 'nữ siêu phản diện: màu da tối',
    '🧙🏻‍♂' => 'pháp sư nam: màu da sáng',
    '🧙🏼‍♂' => 'pháp sư nam: màu da sáng trung bình',
    '🧙🏽‍♂' => 'pháp sư nam: màu da trung bình',
    '🧙🏾‍♂' => 'pháp sư nam: màu da tối trung bình',
    '🧙🏿‍♂' => 'pháp sư nam: màu da tối',
    '🧙🏻‍♀' => 'pháp sư nữ: màu da sáng',
    '🧙🏼‍♀' => 'pháp sư nữ: màu da sáng trung bình',
    '🧙🏽‍♀' => 'pháp sư nữ: màu da trung bình',
    '🧙🏾‍♀' => 'pháp sư nữ: màu da tối trung bình',
    '🧙🏿‍♀' => 'pháp sư nữ: màu da tối',
    '🧚🏻‍♂' => 'ông tiên: màu da sáng',
    '🧚🏼‍♂' => 'ông tiên: màu da sáng trung bình',
    '🧚🏽‍♂' => 'ông tiên: màu da trung bình',
    '🧚🏾‍♂' => 'ông tiên: màu da tối trung bình',
    '🧚🏿‍♂' => 'ông tiên: màu da tối',
    '🧚🏻‍♀' => 'bà tiên: màu da sáng',
    '🧚🏼‍♀' => 'bà tiên: màu da sáng trung bình',
    '🧚🏽‍♀' => 'bà tiên: màu da trung bình',
    '🧚🏾‍♀' => 'bà tiên: màu da tối trung bình',
    '🧚🏿‍♀' => 'bà tiên: màu da tối',
    '🧛🏻‍♂' => 'ma cà rồng nam: màu da sáng',
    '🧛🏼‍♂' => 'ma cà rồng nam: màu da sáng trung bình',
    '🧛🏽‍♂' => 'ma cà rồng nam: màu da trung bình',
    '🧛🏾‍♂' => 'ma cà rồng nam: màu da tối trung bình',
    '🧛🏿‍♂' => 'ma cà rồng nam: màu da tối',
    '🧛🏻‍♀' => 'ma cà rồng nữ: màu da sáng',
    '🧛🏼‍♀' => 'ma cà rồng nữ: màu da sáng trung bình',
    '🧛🏽‍♀' => 'ma cà rồng nữ: màu da trung bình',
    '🧛🏾‍♀' => 'ma cà rồng nữ: màu da tối trung bình',
    '🧛🏿‍♀' => 'ma cà rồng nữ: màu da tối',
    '🧜🏻‍♂' => 'chàng tiên cá: màu da sáng',
    '🧜🏼‍♂' => 'chàng tiên cá: màu da sáng trung bình',
    '🧜🏽‍♂' => 'chàng tiên cá: màu da trung bình',
    '🧜🏾‍♂' => 'chàng tiên cá: màu da tối trung bình',
    '🧜🏿‍♂' => 'chàng tiên cá: màu da tối',
    '🧜🏻‍♀' => 'nàng tiên cá: màu da sáng',
    '🧜🏼‍♀' => 'nàng tiên cá: màu da sáng trung bình',
    '🧜🏽‍♀' => 'nàng tiên cá: màu da trung bình',
    '🧜🏾‍♀' => 'nàng tiên cá: màu da tối trung bình',
    '🧜🏿‍♀' => 'nàng tiên cá: màu da tối',
    '🧝🏻‍♂' => 'nam quỷ: màu da sáng',
    '🧝🏼‍♂' => 'nam quỷ: màu da sáng trung bình',
    '🧝🏽‍♂' => 'nam quỷ: màu da trung bình',
    '🧝🏾‍♂' => 'nam quỷ: màu da tối trung bình',
    '🧝🏿‍♂' => 'nam quỷ: màu da tối',
    '🧝🏻‍♀' => 'nữ quỷ: màu da sáng',
    '🧝🏼‍♀' => 'nữ quỷ: màu da sáng trung bình',
    '🧝🏽‍♀' => 'nữ quỷ: màu da trung bình',
    '🧝🏾‍♀' => 'nữ quỷ: màu da tối trung bình',
    '🧝🏿‍♀' => 'nữ quỷ: màu da tối',
    '💆🏻‍♂' => 'người đàn ông được xoa bóp mặt: màu da sáng',
    '💆🏼‍♂' => 'người đàn ông được xoa bóp mặt: màu da sáng trung bình',
    '💆🏽‍♂' => 'người đàn ông được xoa bóp mặt: màu da trung bình',
    '💆🏾‍♂' => 'người đàn ông được xoa bóp mặt: màu da tối trung bình',
    '💆🏿‍♂' => 'người đàn ông được xoa bóp mặt: màu da tối',
    '💆🏻‍♀' => 'người phụ nữ được xoa bóp mặt: màu da sáng',
    '💆🏼‍♀' => 'người phụ nữ được xoa bóp mặt: màu da sáng trung bình',
    '💆🏽‍♀' => 'người phụ nữ được xoa bóp mặt: màu da trung bình',
    '💆🏾‍♀' => 'người phụ nữ được xoa bóp mặt: màu da tối trung bình',
    '💆🏿‍♀' => 'người phụ nữ được xoa bóp mặt: màu da tối',
    '💇🏻‍♂' => 'người đàn ông được cắt tóc: màu da sáng',
    '💇🏼‍♂' => 'người đàn ông được cắt tóc: màu da sáng trung bình',
    '💇🏽‍♂' => 'người đàn ông được cắt tóc: màu da trung bình',
    '💇🏾‍♂' => 'người đàn ông được cắt tóc: màu da tối trung bình',
    '💇🏿‍♂' => 'người đàn ông được cắt tóc: màu da tối',
    '💇🏻‍♀' => 'người phụ nữ được cắt tóc: màu da sáng',
    '💇🏼‍♀' => 'người phụ nữ được cắt tóc: màu da sáng trung bình',
    '💇🏽‍♀' => 'người phụ nữ được cắt tóc: màu da trung bình',
    '💇🏾‍♀' => 'người phụ nữ được cắt tóc: màu da tối trung bình',
    '💇🏿‍♀' => 'người phụ nữ được cắt tóc: màu da tối',
    '🚶🏻‍♂' => 'người đàn ông đi bộ: màu da sáng',
    '🚶🏼‍♂' => 'người đàn ông đi bộ: màu da sáng trung bình',
    '🚶🏽‍♂' => 'người đàn ông đi bộ: màu da trung bình',
    '🚶🏾‍♂' => 'người đàn ông đi bộ: màu da tối trung bình',
    '🚶🏿‍♂' => 'người đàn ông đi bộ: màu da tối',
    '🚶🏻‍♀' => 'người phụ nữ đi bộ: màu da sáng',
    '🚶🏼‍♀' => 'người phụ nữ đi bộ: màu da sáng trung bình',
    '🚶🏽‍♀' => 'người phụ nữ đi bộ: màu da trung bình',
    '🚶🏾‍♀' => 'người phụ nữ đi bộ: màu da tối trung bình',
    '🚶🏿‍♀' => 'người phụ nữ đi bộ: màu da tối',
    '🚶🏻‍➡' => 'người đi bộ: màu da sáng, hướng sang phải',
    '🚶🏼‍➡' => 'người đi bộ: màu da sáng trung bình, hướng sang phải',
    '🚶🏽‍➡' => 'người đi bộ: màu da trung bình, hướng sang phải',
    '🚶🏾‍➡' => 'người đi bộ: màu da tối trung bình, hướng sang phải',
    '🚶🏿‍➡' => 'người đi bộ: màu da tối, hướng sang phải',
    '🧍🏻‍♂' => 'người đàn ông đứng: màu da sáng',
    '🧍🏼‍♂' => 'người đàn ông đứng: màu da sáng trung bình',
    '🧍🏽‍♂' => 'người đàn ông đứng: màu da trung bình',
    '🧍🏾‍♂' => 'người đàn ông đứng: màu da tối trung bình',
    '🧍🏿‍♂' => 'người đàn ông đứng: màu da tối',
    '🧍🏻‍♀' => 'người phụ nữ đứng: màu da sáng',
    '🧍🏼‍♀' => 'người phụ nữ đứng: màu da sáng trung bình',
    '🧍🏽‍♀' => 'người phụ nữ đứng: màu da trung bình',
    '🧍🏾‍♀' => 'người phụ nữ đứng: màu da tối trung bình',
    '🧍🏿‍♀' => 'người phụ nữ đứng: màu da tối',
    '🧎🏻‍♂' => 'người đàn ông quỳ: màu da sáng',
    '🧎🏼‍♂' => 'người đàn ông quỳ: màu da sáng trung bình',
    '🧎🏽‍♂' => 'người đàn ông quỳ: màu da trung bình',
    '🧎🏾‍♂' => 'người đàn ông quỳ: màu da tối trung bình',
    '🧎🏿‍♂' => 'người đàn ông quỳ: màu da tối',
    '🧎🏻‍♀' => 'người phụ nữ quỳ: màu da sáng',
    '🧎🏼‍♀' => 'người phụ nữ quỳ: màu da sáng trung bình',
    '🧎🏽‍♀' => 'người phụ nữ quỳ: màu da trung bình',
    '🧎🏾‍♀' => 'người phụ nữ quỳ: màu da tối trung bình',
    '🧎🏿‍♀' => 'người phụ nữ quỳ: màu da tối',
    '🧎🏻‍➡' => 'người quỳ: màu da sáng, hướng sang phải',
    '🧎🏼‍➡' => 'người quỳ: màu da sáng trung bình, hướng sang phải',
    '🧎🏽‍➡' => 'người quỳ: màu da trung bình, hướng sang phải',
    '🧎🏾‍➡' => 'người quỳ: màu da tối trung bình, hướng sang phải',
    '🧎🏿‍➡' => 'người quỳ: màu da tối, hướng sang phải',
    '🧑🏻‍🦯' => 'người với gậy dò đường: màu da sáng',
    '🧑🏼‍🦯' => 'người với gậy dò đường: màu da sáng trung bình',
    '🧑🏽‍🦯' => 'người với gậy dò đường: màu da trung bình',
    '🧑🏾‍🦯' => 'người với gậy dò đường: màu da tối trung bình',
    '🧑🏿‍🦯' => 'người với gậy dò đường: màu da tối',
    '👨🏻‍🦯' => 'người đàn ông với gậy dò đường: màu da sáng',
    '👨🏼‍🦯' => 'người đàn ông với gậy dò đường: màu da sáng trung bình',
    '👨🏽‍🦯' => 'người đàn ông với gậy dò đường: màu da trung bình',
    '👨🏾‍🦯' => 'người đàn ông với gậy dò đường: màu da tối trung bình',
    '👨🏿‍🦯' => 'người đàn ông với gậy dò đường: màu da tối',
    '👩🏻‍🦯' => 'người phụ nữ với gậy dò đường: màu da sáng',
    '👩🏼‍🦯' => 'người phụ nữ với gậy dò đường: màu da sáng trung bình',
    '👩🏽‍🦯' => 'người phụ nữ với gậy dò đường: màu da trung bình',
    '👩🏾‍🦯' => 'người phụ nữ với gậy dò đường: màu da tối trung bình',
    '👩🏿‍🦯' => 'người phụ nữ với gậy dò đường: màu da tối',
    '🧑🏻‍🦼' => 'người trên xe lăn điện: màu da sáng',
    '🧑🏼‍🦼' => 'người trên xe lăn điện: màu da sáng trung bình',
    '🧑🏽‍🦼' => 'người trên xe lăn điện: màu da trung bình',
    '🧑🏾‍🦼' => 'người trên xe lăn điện: màu da tối trung bình',
    '🧑🏿‍🦼' => 'người trên xe lăn điện: màu da tối',
    '👨🏻‍🦼' => 'người đàn ông trên xe lăn điện: màu da sáng',
    '👨🏼‍🦼' => 'người đàn ông trên xe lăn điện: màu da sáng trung bình',
    '👨🏽‍🦼' => 'người đàn ông trên xe lăn điện: màu da trung bình',
    '👨🏾‍🦼' => 'người đàn ông trên xe lăn điện: màu da tối trung bình',
    '👨🏿‍🦼' => 'người đàn ông trên xe lăn điện: màu da tối',
    '👩🏻‍🦼' => 'người phụ nữ trên xe lăn điện: màu da sáng',
    '👩🏼‍🦼' => 'người phụ nữ trên xe lăn điện: màu da sáng trung bình',
    '👩🏽‍🦼' => 'người phụ nữ trên xe lăn điện: màu da trung bình',
    '👩🏾‍🦼' => 'người phụ nữ trên xe lăn điện: màu da tối trung bình',
    '👩🏿‍🦼' => 'người phụ nữ trên xe lăn điện: màu da tối',
    '🧑🏻‍🦽' => 'người trên xe lăn tay: màu da sáng',
    '🧑🏼‍🦽' => 'người trên xe lăn tay: màu da sáng trung bình',
    '🧑🏽‍🦽' => 'người trên xe lăn tay: màu da trung bình',
    '🧑🏾‍🦽' => 'người trên xe lăn tay: màu da tối trung bình',
    '🧑🏿‍🦽' => 'người trên xe lăn tay: màu da tối',
    '👨🏻‍🦽' => 'người đàn ông trên xe lăn tay: màu da sáng',
    '👨🏼‍🦽' => 'người đàn ông trên xe lăn tay: màu da sáng trung bình',
    '👨🏽‍🦽' => 'người đàn ông trên xe lăn tay: màu da trung bình',
    '👨🏾‍🦽' => 'người đàn ông trên xe lăn tay: màu da tối trung bình',
    '👨🏿‍🦽' => 'người đàn ông trên xe lăn tay: màu da tối',
    '👩🏻‍🦽' => 'người phụ nữ trên xe lăn tay: màu da sáng',
    '👩🏼‍🦽' => 'người phụ nữ trên xe lăn tay: màu da sáng trung bình',
    '👩🏽‍🦽' => 'người phụ nữ trên xe lăn tay: màu da trung bình',
    '👩🏾‍🦽' => 'người phụ nữ trên xe lăn tay: màu da tối trung bình',
    '👩🏿‍🦽' => 'người phụ nữ trên xe lăn tay: màu da tối',
    '🏃🏻‍♂' => 'người đàn ông đang chạy: màu da sáng',
    '🏃🏼‍♂' => 'người đàn ông đang chạy: màu da sáng trung bình',
    '🏃🏽‍♂' => 'người đàn ông đang chạy: màu da trung bình',
    '🏃🏾‍♂' => 'người đàn ông đang chạy: màu da tối trung bình',
    '🏃🏿‍♂' => 'người đàn ông đang chạy: màu da tối',
    '🏃🏻‍♀' => 'người phụ nữ đang chạy: màu da sáng',
    '🏃🏼‍♀' => 'người phụ nữ đang chạy: màu da sáng trung bình',
    '🏃🏽‍♀' => 'người phụ nữ đang chạy: màu da trung bình',
    '🏃🏾‍♀' => 'người phụ nữ đang chạy: màu da tối trung bình',
    '🏃🏿‍♀' => 'người phụ nữ đang chạy: màu da tối',
    '🏃🏻‍➡' => 'người chạy: màu da sáng, hướng sang phải',
    '🏃🏼‍➡' => 'người chạy: màu da sáng trung bình, hướng sang phải',
    '🏃🏽‍➡' => 'người chạy: màu da trung bình, hướng sang phải',
    '🏃🏾‍➡' => 'người chạy: màu da tối trung bình, hướng sang phải',
    '🏃🏿‍➡' => 'người chạy: màu da tối, hướng sang phải',
    '👯🏻‍♀' => 'những người phụ nữ đeo tai thỏ: màu da sáng',
    '👯🏼‍♀' => 'những người phụ nữ đeo tai thỏ: màu da sáng trung bình',
    '👯🏽‍♀' => 'những người phụ nữ đeo tai thỏ: màu da trung bình',
    '👯🏾‍♀' => 'những người phụ nữ đeo tai thỏ: màu da tối trung bình',
    '👯🏿‍♀' => 'những người phụ nữ đeo tai thỏ: màu da tối',
    '👯🏻‍♂' => 'những người đàn ông đeo tai thỏ: màu da sáng',
    '👯🏼‍♂' => 'những người đàn ông đeo tai thỏ: màu da sáng trung bình',
    '👯🏽‍♂' => 'những người đàn ông đeo tai thỏ: màu da trung bình',
    '👯🏾‍♂' => 'những người đàn ông đeo tai thỏ: màu da tối trung bình',
    '👯🏿‍♂' => 'những người đàn ông đeo tai thỏ: màu da tối',
    '🧖🏻‍♂' => 'người đàn ông ở trong phòng xông hơi: màu da sáng',
    '🧖🏼‍♂' => 'người đàn ông ở trong phòng xông hơi: màu da sáng trung bình',
    '🧖🏽‍♂' => 'người đàn ông ở trong phòng xông hơi: màu da trung bình',
    '🧖🏾‍♂' => 'người đàn ông ở trong phòng xông hơi: màu da tối trung bình',
    '🧖🏿‍♂' => 'người đàn ông ở trong phòng xông hơi: màu da tối',
    '🧖🏻‍♀' => 'người phụ nữ ở trong phòng xông hơi: màu da sáng',
    '🧖🏼‍♀' => 'người phụ nữ ở trong phòng xông hơi: màu da sáng trung bình',
    '🧖🏽‍♀' => 'người phụ nữ ở trong phòng xông hơi: màu da trung bình',
    '🧖🏾‍♀' => 'người phụ nữ ở trong phòng xông hơi: màu da tối trung bình',
    '🧖🏿‍♀' => 'người phụ nữ ở trong phòng xông hơi: màu da tối',
    '🧗🏻‍♂' => 'người đàn ông đang leo núi: màu da sáng',
    '🧗🏼‍♂' => 'người đàn ông đang leo núi: màu da sáng trung bình',
    '🧗🏽‍♂' => 'người đàn ông đang leo núi: màu da trung bình',
    '🧗🏾‍♂' => 'người đàn ông đang leo núi: màu da tối trung bình',
    '🧗🏿‍♂' => 'người đàn ông đang leo núi: màu da tối',
    '🧗🏻‍♀' => 'người phụ nữ đang leo núi: màu da sáng',
    '🧗🏼‍♀' => 'người phụ nữ đang leo núi: màu da sáng trung bình',
    '🧗🏽‍♀' => 'người phụ nữ đang leo núi: màu da trung bình',
    '🧗🏾‍♀' => 'người phụ nữ đang leo núi: màu da tối trung bình',
    '🧗🏿‍♀' => 'người phụ nữ đang leo núi: màu da tối',
    '🏌🏻‍♂' => 'người đàn ông chơi gôn: màu da sáng',
    '🏌🏼‍♂' => 'người đàn ông chơi gôn: màu da sáng trung bình',
    '🏌🏽‍♂' => 'người đàn ông chơi gôn: màu da trung bình',
    '🏌🏾‍♂' => 'người đàn ông chơi gôn: màu da tối trung bình',
    '🏌🏿‍♂' => 'người đàn ông chơi gôn: màu da tối',
    '🏌🏻‍♀' => 'người phụ nữ chơi gôn: màu da sáng',
    '🏌🏼‍♀' => 'người phụ nữ chơi gôn: màu da sáng trung bình',
    '🏌🏽‍♀' => 'người phụ nữ chơi gôn: màu da trung bình',
    '🏌🏾‍♀' => 'người phụ nữ chơi gôn: màu da tối trung bình',
    '🏌🏿‍♀' => 'người phụ nữ chơi gôn: màu da tối',
    '🏄🏻‍♂' => 'người đàn ông lướt sóng: màu da sáng',
    '🏄🏼‍♂' => 'người đàn ông lướt sóng: màu da sáng trung bình',
    '🏄🏽‍♂' => 'người đàn ông lướt sóng: màu da trung bình',
    '🏄🏾‍♂' => 'người đàn ông lướt sóng: màu da tối trung bình',
    '🏄🏿‍♂' => 'người đàn ông lướt sóng: màu da tối',
    '🏄🏻‍♀' => 'người phụ nữ lướt sóng: màu da sáng',
    '🏄🏼‍♀' => 'người phụ nữ lướt sóng: màu da sáng trung bình',
    '🏄🏽‍♀' => 'người phụ nữ lướt sóng: màu da trung bình',
    '🏄🏾‍♀' => 'người phụ nữ lướt sóng: màu da tối trung bình',
    '🏄🏿‍♀' => 'người phụ nữ lướt sóng: màu da tối',
    '🚣🏻‍♂' => 'người đàn ông chèo thuyền: màu da sáng',
    '🚣🏼‍♂' => 'người đàn ông chèo thuyền: màu da sáng trung bình',
    '🚣🏽‍♂' => 'người đàn ông chèo thuyền: màu da trung bình',
    '🚣🏾‍♂' => 'người đàn ông chèo thuyền: màu da tối trung bình',
    '🚣🏿‍♂' => 'người đàn ông chèo thuyền: màu da tối',
    '🚣🏻‍♀' => 'người phụ nữ chèo thuyền: màu da sáng',
    '🚣🏼‍♀' => 'người phụ nữ chèo thuyền: màu da sáng trung bình',
    '🚣🏽‍♀' => 'người phụ nữ chèo thuyền: màu da trung bình',
    '🚣🏾‍♀' => 'người phụ nữ chèo thuyền: màu da tối trung bình',
    '🚣🏿‍♀' => 'người phụ nữ chèo thuyền: màu da tối',
    '🏊🏻‍♂' => 'người đàn ông đang bơi: màu da sáng',
    '🏊🏼‍♂' => 'người đàn ông đang bơi: màu da sáng trung bình',
    '🏊🏽‍♂' => 'người đàn ông đang bơi: màu da trung bình',
    '🏊🏾‍♂' => 'người đàn ông đang bơi: màu da tối trung bình',
    '🏊🏿‍♂' => 'người đàn ông đang bơi: màu da tối',
    '🏊🏻‍♀' => 'người phụ nữ đang bơi: màu da sáng',
    '🏊🏼‍♀' => 'người phụ nữ đang bơi: màu da sáng trung bình',
    '🏊🏽‍♀' => 'người phụ nữ đang bơi: màu da trung bình',
    '🏊🏾‍♀' => 'người phụ nữ đang bơi: màu da tối trung bình',
    '🏊🏿‍♀' => 'người phụ nữ đang bơi: màu da tối',
    '⛹🏻‍♂' => 'người đàn ông chơi bóng: màu da sáng',
    '⛹🏼‍♂' => 'người đàn ông chơi bóng: màu da sáng trung bình',
    '⛹🏽‍♂' => 'người đàn ông chơi bóng: màu da trung bình',
    '⛹🏾‍♂' => 'người đàn ông chơi bóng: màu da tối trung bình',
    '⛹🏿‍♂' => 'người đàn ông chơi bóng: màu da tối',
    '⛹🏻‍♀' => 'người phụ nữ chơi bóng: màu da sáng',
    '⛹🏼‍♀' => 'người phụ nữ chơi bóng: màu da sáng trung bình',
    '⛹🏽‍♀' => 'người phụ nữ chơi bóng: màu da trung bình',
    '⛹🏾‍♀' => 'người phụ nữ chơi bóng: màu da tối trung bình',
    '⛹🏿‍♀' => 'người phụ nữ chơi bóng: màu da tối',
    '🏋🏻‍♂' => 'người đàn ông cử tạ: màu da sáng',
    '🏋🏼‍♂' => 'người đàn ông cử tạ: màu da sáng trung bình',
    '🏋🏽‍♂' => 'người đàn ông cử tạ: màu da trung bình',
    '🏋🏾‍♂' => 'người đàn ông cử tạ: màu da tối trung bình',
    '🏋🏿‍♂' => 'người đàn ông cử tạ: màu da tối',
    '🏋🏻‍♀' => 'người phụ nữ cử tạ: màu da sáng',
    '🏋🏼‍♀' => 'người phụ nữ cử tạ: màu da sáng trung bình',
    '🏋🏽‍♀' => 'người phụ nữ cử tạ: màu da trung bình',
    '🏋🏾‍♀' => 'người phụ nữ cử tạ: màu da tối trung bình',
    '🏋🏿‍♀' => 'người phụ nữ cử tạ: màu da tối',
    '🚴🏻‍♂' => 'người đàn ông đi xe đạp: màu da sáng',
    '🚴🏼‍♂' => 'người đàn ông đi xe đạp: màu da sáng trung bình',
    '🚴🏽‍♂' => 'người đàn ông đi xe đạp: màu da trung bình',
    '🚴🏾‍♂' => 'người đàn ông đi xe đạp: màu da tối trung bình',
    '🚴🏿‍♂' => 'người đàn ông đi xe đạp: màu da tối',
    '🚴🏻‍♀' => 'người phụ nữ đi xe đạp: màu da sáng',
    '🚴🏼‍♀' => 'người phụ nữ đi xe đạp: màu da sáng trung bình',
    '🚴🏽‍♀' => 'người phụ nữ đi xe đạp: màu da trung bình',
    '🚴🏾‍♀' => 'người phụ nữ đi xe đạp: màu da tối trung bình',
    '🚴🏿‍♀' => 'người phụ nữ đi xe đạp: màu da tối',
    '🚵🏻‍♂' => 'người đàn ông đi xe đạp leo núi: màu da sáng',
    '🚵🏼‍♂' => 'người đàn ông đi xe đạp leo núi: màu da sáng trung bình',
    '🚵🏽‍♂' => 'người đàn ông đi xe đạp leo núi: màu da trung bình',
    '🚵🏾‍♂' => 'người đàn ông đi xe đạp leo núi: màu da tối trung bình',
    '🚵🏿‍♂' => 'người đàn ông đi xe đạp leo núi: màu da tối',
    '🚵🏻‍♀' => 'người phụ nữ đi xe đạp leo núi: màu da sáng',
    '🚵🏼‍♀' => 'người phụ nữ đi xe đạp leo núi: màu da sáng trung bình',
    '🚵🏽‍♀' => 'người phụ nữ đi xe đạp leo núi: màu da trung bình',
    '🚵🏾‍♀' => 'người phụ nữ đi xe đạp leo núi: màu da tối trung bình',
    '🚵🏿‍♀' => 'người phụ nữ đi xe đạp leo núi: màu da tối',
    '🤸🏻‍♂' => 'người đàn ông nhào lộn: màu da sáng',
    '🤸🏼‍♂' => 'người đàn ông nhào lộn: màu da sáng trung bình',
    '🤸🏽‍♂' => 'người đàn ông nhào lộn: màu da trung bình',
    '🤸🏾‍♂' => 'người đàn ông nhào lộn: màu da tối trung bình',
    '🤸🏿‍♂' => 'người đàn ông nhào lộn: màu da tối',
    '🤸🏻‍♀' => 'người phụ nữ nhào lộn: màu da sáng',
    '🤸🏼‍♀' => 'người phụ nữ nhào lộn: màu da sáng trung bình',
    '🤸🏽‍♀' => 'người phụ nữ nhào lộn: màu da trung bình',
    '🤸🏾‍♀' => 'người phụ nữ nhào lộn: màu da tối trung bình',
    '🤸🏿‍♀' => 'người phụ nữ nhào lộn: màu da tối',
    '🤼🏻‍♀' => 'những người phụ nữ chơi vật: màu da sáng',
    '🤼🏼‍♀' => 'những người phụ nữ chơi vật: màu da sáng trung bình',
    '🤼🏽‍♀' => 'những người phụ nữ chơi vật: màu da trung bình',
    '🤼🏾‍♀' => 'những người phụ nữ chơi vật: màu da tối trung bình',
    '🤼🏿‍♀' => 'những người phụ nữ chơi vật: màu da tối',
    '🤼🏻‍♂' => 'những người đàn ông chơi vật: màu da sáng',
    '🤼🏼‍♂' => 'những người đàn ông chơi vật: màu da sáng trung bình',
    '🤼🏽‍♂' => 'những người đàn ông chơi vật: màu da trung bình',
    '🤼🏾‍♂' => 'những người đàn ông chơi vật: màu da tối trung bình',
    '🤼🏿‍♂' => 'những người đàn ông chơi vật: màu da tối',
    '🤽🏻‍♂' => 'người đàn ông chơi bóng nước: màu da sáng',
    '🤽🏼‍♂' => 'người đàn ông chơi bóng nước: màu da sáng trung bình',
    '🤽🏽‍♂' => 'người đàn ông chơi bóng nước: màu da trung bình',
    '🤽🏾‍♂' => 'người đàn ông chơi bóng nước: màu da tối trung bình',
    '🤽🏿‍♂' => 'người đàn ông chơi bóng nước: màu da tối',
    '🤽🏻‍♀' => 'người phụ nữ chơi bóng nước: màu da sáng',
    '🤽🏼‍♀' => 'người phụ nữ chơi bóng nước: màu da sáng trung bình',
    '🤽🏽‍♀' => 'người phụ nữ chơi bóng nước: màu da trung bình',
    '🤽🏾‍♀' => 'người phụ nữ chơi bóng nước: màu da tối trung bình',
    '🤽🏿‍♀' => 'người phụ nữ chơi bóng nước: màu da tối',
    '🤾🏻‍♂' => 'người đàn ông chơi bóng ném: màu da sáng',
    '🤾🏼‍♂' => 'người đàn ông chơi bóng ném: màu da sáng trung bình',
    '🤾🏽‍♂' => 'người đàn ông chơi bóng ném: màu da trung bình',
    '🤾🏾‍♂' => 'người đàn ông chơi bóng ném: màu da tối trung bình',
    '🤾🏿‍♂' => 'người đàn ông chơi bóng ném: màu da tối',
    '🤾🏻‍♀' => 'người phụ nữ chơi bóng ném: màu da sáng',
    '🤾🏼‍♀' => 'người phụ nữ chơi bóng ném: màu da sáng trung bình',
    '🤾🏽‍♀' => 'người phụ nữ chơi bóng ném: màu da trung bình',
    '🤾🏾‍♀' => 'người phụ nữ chơi bóng ném: màu da tối trung bình',
    '🤾🏿‍♀' => 'người phụ nữ chơi bóng ném: màu da tối',
    '🤹🏻‍♂' => 'người đàn ông tung hứng: màu da sáng',
    '🤹🏼‍♂' => 'người đàn ông tung hứng: màu da sáng trung bình',
    '🤹🏽‍♂' => 'người đàn ông tung hứng: màu da trung bình',
    '🤹🏾‍♂' => 'người đàn ông tung hứng: màu da tối trung bình',
    '🤹🏿‍♂' => 'người đàn ông tung hứng: màu da tối',
    '🤹🏻‍♀' => 'người phụ nữ tung hứng: màu da sáng',
    '🤹🏼‍♀' => 'người phụ nữ tung hứng: màu da sáng trung bình',
    '🤹🏽‍♀' => 'người phụ nữ tung hứng: màu da trung bình',
    '🤹🏾‍♀' => 'người phụ nữ tung hứng: màu da tối trung bình',
    '🤹🏿‍♀' => 'người phụ nữ tung hứng: màu da tối',
    '🧘🏻‍♂' => 'người đàn ông ngồi kiết già: màu da sáng',
    '🧘🏼‍♂' => 'người đàn ông ngồi kiết già: màu da sáng trung bình',
    '🧘🏽‍♂' => 'người đàn ông ngồi kiết già: màu da trung bình',
    '🧘🏾‍♂' => 'người đàn ông ngồi kiết già: màu da tối trung bình',
    '🧘🏿‍♂' => 'người đàn ông ngồi kiết già: màu da tối',
    '🧘🏻‍♀' => 'người phụ nữ ngồi kiết già: màu da sáng',
    '🧘🏼‍♀' => 'người phụ nữ ngồi kiết già: màu da sáng trung bình',
    '🧘🏽‍♀' => 'người phụ nữ ngồi kiết già: màu da trung bình',
    '🧘🏾‍♀' => 'người phụ nữ ngồi kiết già: màu da tối trung bình',
    '🧘🏿‍♀' => 'người phụ nữ ngồi kiết già: màu da tối',
    '🧑‍🦰' => 'người: tóc đỏ',
    '🧑‍🦱' => 'người: tóc xoăn',
    '🧑‍🦳' => 'người: tóc trắng',
    '🧑‍🦲' => 'người: hói',
    '👨‍🦰' => 'đàn ông: tóc đỏ',
    '👨‍🦱' => 'đàn ông: tóc xoăn',
    '👨‍🦳' => 'đàn ông: tóc trắng',
    '👨‍🦲' => 'đàn ông: hói',
    '👩‍🦰' => 'phụ nữ: tóc đỏ',
    '👩‍🦱' => 'phụ nữ: tóc xoăn',
    '👩‍🦳' => 'phụ nữ: tóc trắng',
    '👩‍🦲' => 'phụ nữ: hói',
    '🚶‍➡' => 'người đi bộ: hướng sang phải',
    '🧎‍➡' => 'người quỳ: hướng sang phải',
    '🏃‍➡' => 'người chạy: hướng sang phải',
    '👨‍👦' => 'gia đình: đàn ông, con trai',
    '👨‍👧' => 'gia đình: đàn ông, con gái',
    '👩‍👦' => 'gia đình: phụ nữ, con trai',
    '👩‍👧' => 'gia đình: phụ nữ, con gái',
    '😶‍🌫' => 'mặt trong đám mây',
    '😮‍💨' => 'mặt thở ra',
    '🙂‍↕' => 'gật đầu',
    '😵‍💫' => 'mặt có đôi mắt xoắn ốc',
    '❤‍🔥' => 'trái tim bốc lửa',
    '❤‍🩹' => 'trái tim đang lành',
    '👁‍🗨' => 'mắt trong bong bóng lời nói',
    '🧔‍♂' => 'người đàn ông có râu',
    '👱‍♂' => 'người đàn ông tóc vàng hoe',
    '🧔‍♀' => 'người phụ nữ có râu',
    '👱‍♀' => 'người phụ nữ tóc vàng hoe',
    '🙍‍♂' => 'người đàn ông cau mày',
    '🙍‍♀' => 'người phụ nữ cau mày',
    '🙎‍♂' => 'người đàn ông bĩu môi',
    '🙎‍♀' => 'người phụ nữ bĩu môi',
    '🙅‍♂' => 'người đàn ông ra hiệu phản đối',
    '🙅‍♀' => 'người phụ nữ ra hiệu phản đối',
    '🙆‍♂' => 'người đàn ông ra hiệu đồng ý',
    '🙆‍♀' => 'người phụ nữ ra hiệu đồng ý',
    '💁‍♂' => 'người đàn ông ra hiệu trợ giúp',
    '💁‍♀' => 'người phụ nữ ra hiệu trợ giúp',
    '🙋‍♂' => 'người đàn ông giơ cao tay',
    '🙋‍♀' => 'người phụ nữ giơ cao tay',
    '🧏‍♂' => 'người đàn ông khiếm thính',
    '🧏‍♀' => 'người phụ nữ khiếm thính',
    '🙇‍♂' => 'người đàn ông cúi đầu',
    '🙇‍♀' => 'người phụ nữ cúi đầu',
    '🤦‍♂' => 'người đàn ông lấy tay che mặt',
    '🤦‍♀' => 'người phụ nữ lấy tay che mặt',
    '🤷‍♂' => 'người đàn ông nhún vai',
    '🤷‍♀' => 'người phụ nữ nhún vai',
    '🧑‍⚕' => 'nhân viên y tế',
    '👨‍⚕' => 'nhân viên y tế nam',
    '👩‍⚕' => 'nhân viên y tế nữ',
    '🧑‍🎓' => 'sinh viên',
    '👨‍🎓' => 'sinh viên nam',
    '👩‍🎓' => 'sinh viên nữ',
    '🧑‍🏫' => 'giáo viên',
    '👨‍🏫' => 'giáo viên nam',
    '👩‍🏫' => 'giáo viên nữ',
    '🧑‍⚖' => 'thẩm phán',
    '👨‍⚖' => 'thẩm phán nam',
    '👩‍⚖' => 'thẩm phán nữ',
    '🧑‍🌾' => 'nông dân',
    '👨‍🌾' => 'nông dân nam',
    '👩‍🌾' => 'nông dân nữ',
    '🧑‍🍳' => 'đầu bếp',
    '👨‍🍳' => 'đầu bếp nam',
    '👩‍🍳' => 'đầu bếp nữ',
    '🧑‍🔧' => 'thợ máy',
    '👨‍🔧' => 'thợ máy nam',
    '👩‍🔧' => 'thợ máy nữ',
    '🧑‍🏭' => 'công nhân nhà máy',
    '👨‍🏭' => 'công nhân nhà máy nam',
    '👩‍🏭' => 'công nhân nhà máy nữ',
    '🧑‍💼' => 'nhân viên văn phòng',
    '👨‍💼' => 'nhân viên văn phòng nam',
    '👩‍💼' => 'nhân viên văn phòng nữ',
    '🧑‍🔬' => 'nhà khoa học',
    '👨‍🔬' => 'nhà khoa học nam',
    '👩‍🔬' => 'nhà khoa học nữ',
    '🧑‍💻' => 'kỹ sư công nghệ',
    '👨‍💻' => 'kỹ sư công nghệ nam',
    '👩‍💻' => 'kỹ sư công nghệ nữ',
    '🧑‍🎤' => 'ca sĩ',
    '👨‍🎤' => 'nam ca sĩ',
    '👩‍🎤' => 'nữ ca sĩ',
    '🧑‍🎨' => 'họa sĩ',
    '👨‍🎨' => 'họa sĩ nam',
    '👩‍🎨' => 'họa sĩ nữ',
    '🧑‍✈' => 'phi công',
    '👨‍✈' => 'phi công nam',
    '👩‍✈' => 'phi công nữ',
    '🧑‍🚀' => 'phi hành gia',
    '👨‍🚀' => 'phi hành gia nam',
    '👩‍🚀' => 'phi hành gia nữ',
    '🧑‍🚒' => 'lính cứu hỏa',
    '👨‍🚒' => 'lính cứu hỏa nam',
    '👩‍🚒' => 'lính cứu hỏa nữ',
    '👮‍♂' => 'cảnh sát nam',
    '👮‍♀' => 'cảnh sát nữ',
    '🕵‍♂' => 'thám tử nam',
    '🕵‍♀' => 'thám tử nữ',
    '💂‍♂' => 'bảo vệ nam',
    '💂‍♀' => 'bảo vệ nữ',
    '👷‍♂' => 'công nhân xây dựng nam',
    '👷‍♀' => 'công nhân xây dựng nữ',
    '👳‍♂' => 'người đàn ông đội khăn xếp',
    '👳‍♀' => 'người phụ nữ đội khăn xếp',
    '🤵‍♂' => 'người đàn ông mặc bộ vest',
    '🤵‍♀' => 'người phụ nữ mặc bộ vest',
    '👰‍♂' => 'người đàn ông đội khăn voan',
    '👰‍♀' => 'người phụ nữ đội khăn voan',
    '👩‍🍼' => 'người phụ nữ cho con bú',
    '👨‍🍼' => 'người cho con bú',
    '🧑‍🍼' => 'người cho em bé bú',
    '🧑‍🎄' => 'bà già Noel',
    '🦸‍♂' => 'nam siêu anh hùng',
    '🦸‍♀' => 'nữ siêu anh hùng',
    '🦹‍♂' => 'nam siêu phản diện',
    '🦹‍♀' => 'nữ siêu phản diện',
    '🧙‍♂' => 'pháp sư nam',
    '🧙‍♀' => 'pháp sư nữ',
    '🧚‍♂' => 'ông tiên',
    '🧚‍♀' => 'bà tiên',
    '🧛‍♂' => 'ma cà rồng nam',
    '🧛‍♀' => 'ma cà rồng nữ',
    '🧜‍♂' => 'chàng tiên cá',
    '🧜‍♀' => 'nàng tiên cá',
    '🧝‍♂' => 'nam quỷ',
    '🧝‍♀' => 'nữ quỷ',
    '🧞‍♂' => 'nam thần',
    '🧞‍♀' => 'nữ thần',
    '🧟‍♂' => 'xác chết nam sống lại',
    '🧟‍♀' => 'xác chết nữ sống lại',
    '💆‍♂' => 'người đàn ông được xoa bóp mặt',
    '💆‍♀' => 'người phụ nữ được xoa bóp mặt',
    '💇‍♂' => 'người đàn ông được cắt tóc',
    '💇‍♀' => 'người phụ nữ được cắt tóc',
    '🚶‍♂' => 'người đàn ông đi bộ',
    '🚶‍♀' => 'người phụ nữ đi bộ',
    '🧍‍♂' => 'người đàn ông đứng',
    '🧍‍♀' => 'người phụ nữ đứng',
    '🧎‍♂' => 'người đàn ông quỳ',
    '🧎‍♀' => 'người phụ nữ quỳ',
    '🧑‍🦯' => 'người với gậy dò đường',
    '👨‍🦯' => 'người đàn ông với gậy dò đường',
    '👩‍🦯' => 'người phụ nữ với gậy dò đường',
    '🧑‍🦼' => 'người trên xe lăn điện',
    '👨‍🦼' => 'người đàn ông trên xe lăn điện',
    '👩‍🦼' => 'người phụ nữ trên xe lăn điện',
    '🧑‍🦽' => 'người trên xe lăn tay',
    '👨‍🦽' => 'người đàn ông trên xe lăn tay',
    '👩‍🦽' => 'người phụ nữ trên xe lăn tay',
    '🏃‍♂' => 'người đàn ông đang chạy',
    '🏃‍♀' => 'người phụ nữ đang chạy',
    '🧑‍🩰' => 'vũ công ba lê',
    '👯‍♂' => 'những người đàn ông đeo tai thỏ',
    '👯‍♀' => 'những người phụ nữ đeo tai thỏ',
    '🧖‍♂' => 'người đàn ông ở trong phòng xông hơi',
    '🧖‍♀' => 'người phụ nữ ở trong phòng xông hơi',
    '🧗‍♂' => 'người đàn ông đang leo núi',
    '🧗‍♀' => 'người phụ nữ đang leo núi',
    '🏌‍♂' => 'người đàn ông chơi gôn',
    '🏌‍♀' => 'người phụ nữ chơi gôn',
    '🏄‍♂' => 'người đàn ông lướt sóng',
    '🏄‍♀' => 'người phụ nữ lướt sóng',
    '🚣‍♂' => 'người đàn ông chèo thuyền',
    '🚣‍♀' => 'người phụ nữ chèo thuyền',
    '🏊‍♂' => 'người đàn ông đang bơi',
    '🏊‍♀' => 'người phụ nữ đang bơi',
    '⛹‍♂' => 'người đàn ông chơi bóng',
    '⛹‍♀' => 'người phụ nữ chơi bóng',
    '🏋‍♂' => 'người đàn ông cử tạ',
    '🏋‍♀' => 'người phụ nữ cử tạ',
    '🚴‍♂' => 'người đàn ông đi xe đạp',
    '🚴‍♀' => 'người phụ nữ đi xe đạp',
    '🚵‍♂' => 'người đàn ông đi xe đạp leo núi',
    '🚵‍♀' => 'người phụ nữ đi xe đạp leo núi',
    '🤸‍♂' => 'người đàn ông nhào lộn',
    '🤸‍♀' => 'người phụ nữ nhào lộn',
    '🤼‍♂' => 'những người đàn ông chơi vật',
    '🤼‍♀' => 'những người phụ nữ chơi vật',
    '🤽‍♂' => 'người đàn ông chơi bóng nước',
    '🤽‍♀' => 'người phụ nữ chơi bóng nước',
    '🤾‍♂' => 'người đàn ông chơi bóng ném',
    '🤾‍♀' => 'người phụ nữ chơi bóng ném',
    '🤹‍♂' => 'người đàn ông tung hứng',
    '🤹‍♀' => 'người phụ nữ tung hứng',
    '🧘‍♂' => 'người đàn ông ngồi kiết già',
    '🧘‍♀' => 'người phụ nữ ngồi kiết già',
    '🧑‍🧒' => 'gia đình: người lớn, trẻ em',
    '🐕‍🦺' => 'chó hỗ trợ',
    '🐈‍⬛' => 'mèo đen',
    '🐻‍❄' => 'Gấu trắng Bắc Cực',
    '🐦‍⬛' => 'chim hoét đen',
    '🐦‍🔥' => 'phượng hoàng',
    '🍋‍🟩' => 'chanh ta',
    '🍄‍🟫' => 'nấm màu nâu',
    '⛓‍💥' => 'xích bị đứt',
    '🏳‍🌈' => 'cờ cầu vồng',
    '🏳‍⚧' => 'cờ màu xanh, hồng và trắng',
    '🏴‍☠' => 'cờ cướp biển',
    '👋🏻' => 'vẫy tay: màu da sáng',
    '👋🏼' => 'vẫy tay: màu da sáng trung bình',
    '👋🏽' => 'vẫy tay: màu da trung bình',
    '👋🏾' => 'vẫy tay: màu da tối trung bình',
    '👋🏿' => 'vẫy tay: màu da tối',
    '🤚🏻' => 'mu bàn tay giơ lên: màu da sáng',
    '🤚🏼' => 'mu bàn tay giơ lên: màu da sáng trung bình',
    '🤚🏽' => 'mu bàn tay giơ lên: màu da trung bình',
    '🤚🏾' => 'mu bàn tay giơ lên: màu da tối trung bình',
    '🤚🏿' => 'mu bàn tay giơ lên: màu da tối',
    '🖐🏻' => 'tay xòe ngón giơ lên: màu da sáng',
    '🖐🏼' => 'tay xòe ngón giơ lên: màu da sáng trung bình',
    '🖐🏽' => 'tay xòe ngón giơ lên: màu da trung bình',
    '🖐🏾' => 'tay xòe ngón giơ lên: màu da tối trung bình',
    '🖐🏿' => 'tay xòe ngón giơ lên: màu da tối',
    '✋🏻' => 'bàn tay giơ lên: màu da sáng',
    '✋🏼' => 'bàn tay giơ lên: màu da sáng trung bình',
    '✋🏽' => 'bàn tay giơ lên: màu da trung bình',
    '✋🏾' => 'bàn tay giơ lên: màu da tối trung bình',
    '✋🏿' => 'bàn tay giơ lên: màu da tối',
    '🖖🏻' => 'tay thần lửa: màu da sáng',
    '🖖🏼' => 'tay thần lửa: màu da sáng trung bình',
    '🖖🏽' => 'tay thần lửa: màu da trung bình',
    '🖖🏾' => 'tay thần lửa: màu da tối trung bình',
    '🖖🏿' => 'tay thần lửa: màu da tối',
    '🫱🏻' => 'bàn tay hướng sang bên phải: màu da sáng',
    '🫱🏼' => 'bàn tay hướng sang bên phải: màu da sáng trung bình',
    '🫱🏽' => 'bàn tay hướng sang bên phải: màu da trung bình',
    '🫱🏾' => 'bàn tay hướng sang bên phải: màu da tối trung bình',
    '🫱🏿' => 'bàn tay hướng sang bên phải: màu da tối',
    '🫲🏻' => 'bàn tay hướng sang bên trái: màu da sáng',
    '🫲🏼' => 'bàn tay hướng sang bên trái: màu da sáng trung bình',
    '🫲🏽' => 'bàn tay hướng sang bên trái: màu da trung bình',
    '🫲🏾' => 'bàn tay hướng sang bên trái: màu da tối trung bình',
    '🫲🏿' => 'bàn tay hướng sang bên trái: màu da tối',
    '🫳🏻' => 'bàn tay có lòng bàn tay hướng xuống: màu da sáng',
    '🫳🏼' => 'bàn tay có lòng bàn tay hướng xuống: màu da sáng trung bình',
    '🫳🏽' => 'bàn tay có lòng bàn tay hướng xuống: màu da trung bình',
    '🫳🏾' => 'bàn tay có lòng bàn tay hướng xuống: màu da tối trung bình',
    '🫳🏿' => 'bàn tay có lòng bàn tay hướng xuống: màu da tối',
    '🫴🏻' => 'bàn tay có lòng bàn tay hướng lên: màu da sáng',
    '🫴🏼' => 'bàn tay có lòng bàn tay hướng lên: màu da sáng trung bình',
    '🫴🏽' => 'bàn tay có lòng bàn tay hướng lên: màu da trung bình',
    '🫴🏾' => 'bàn tay có lòng bàn tay hướng lên: màu da tối trung bình',
    '🫴🏿' => 'bàn tay có lòng bàn tay hướng lên: màu da tối',
    '🫷🏻' => 'bàn tay đẩy sang trái: màu da sáng',
    '🫷🏼' => 'bàn tay đẩy sang trái: màu da sáng trung bình',
    '🫷🏽' => 'bàn tay đẩy sang trái: màu da trung bình',
    '🫷🏾' => 'bàn tay đẩy sang trái: màu da tối trung bình',
    '🫷🏿' => 'bàn tay đẩy sang trái: màu da tối',
    '🫸🏻' => 'bàn tay đẩy sang phải: màu da sáng',
    '🫸🏼' => 'bàn tay đẩy sang phải: màu da sáng trung bình',
    '🫸🏽' => 'bàn tay đẩy sang phải: màu da trung bình',
    '🫸🏾' => 'bàn tay đẩy sang phải: màu da tối trung bình',
    '🫸🏿' => 'bàn tay đẩy sang phải: màu da tối',
    '👌🏻' => 'bàn tay làm dấu OK: màu da sáng',
    '👌🏼' => 'bàn tay làm dấu OK: màu da sáng trung bình',
    '👌🏽' => 'bàn tay làm dấu OK: màu da trung bình',
    '👌🏾' => 'bàn tay làm dấu OK: màu da tối trung bình',
    '👌🏿' => 'bàn tay làm dấu OK: màu da tối',
    '🤌🏻' => 'ngón tay khum lại: màu da sáng',
    '🤌🏼' => 'ngón tay khum lại: màu da sáng trung bình',
    '🤌🏽' => 'ngón tay khum lại: màu da trung bình',
    '🤌🏾' => 'ngón tay khum lại: màu da tối trung bình',
    '🤌🏿' => 'ngón tay khum lại: màu da tối',
    '🤏🏻' => 'tay chụm: màu da sáng',
    '🤏🏼' => 'tay chụm: màu da sáng trung bình',
    '🤏🏽' => 'tay chụm: màu da trung bình',
    '🤏🏾' => 'tay chụm: màu da tối trung bình',
    '🤏🏿' => 'tay chụm: màu da tối',
    '✌🏻' => 'tay chiến thắng: màu da sáng',
    '✌🏼' => 'tay chiến thắng: màu da sáng trung bình',
    '✌🏽' => 'tay chiến thắng: màu da trung bình',
    '✌🏾' => 'tay chiến thắng: màu da tối trung bình',
    '✌🏿' => 'tay chiến thắng: màu da tối',
    '🤞🏻' => 'hai ngón tay bắt chéo: màu da sáng',
    '🤞🏼' => 'hai ngón tay bắt chéo: màu da sáng trung bình',
    '🤞🏽' => 'hai ngón tay bắt chéo: màu da trung bình',
    '🤞🏾' => 'hai ngón tay bắt chéo: màu da tối trung bình',
    '🤞🏿' => 'hai ngón tay bắt chéo: màu da tối',
    '🫰🏻' => 'bàn tay với ngón trỏ và ngón cái bắt chéo: màu da sáng',
    '🫰🏼' => 'bàn tay với ngón trỏ và ngón cái bắt chéo: màu da sáng trung bình',
    '🫰🏽' => 'bàn tay với ngón trỏ và ngón cái bắt chéo: màu da trung bình',
    '🫰🏾' => 'bàn tay với ngón trỏ và ngón cái bắt chéo: màu da tối trung bình',
    '🫰🏿' => 'bàn tay với ngón trỏ và ngón cái bắt chéo: màu da tối',
    '🤟🏻' => 'cử chỉ yêu nhau: màu da sáng',
    '🤟🏼' => 'cử chỉ yêu nhau: màu da sáng trung bình',
    '🤟🏽' => 'cử chỉ yêu nhau: màu da trung bình',
    '🤟🏾' => 'cử chỉ yêu nhau: màu da tối trung bình',
    '🤟🏿' => 'cử chỉ yêu nhau: màu da tối',
    '🤘🏻' => 'ký hiệu cặp sừng: màu da sáng',
    '🤘🏼' => 'ký hiệu cặp sừng: màu da sáng trung bình',
    '🤘🏽' => 'ký hiệu cặp sừng: màu da trung bình',
    '🤘🏾' => 'ký hiệu cặp sừng: màu da tối trung bình',
    '🤘🏿' => 'ký hiệu cặp sừng: màu da tối',
    '🤙🏻' => 'bàn tay gọi cho tôi: màu da sáng',
    '🤙🏼' => 'bàn tay gọi cho tôi: màu da sáng trung bình',
    '🤙🏽' => 'bàn tay gọi cho tôi: màu da trung bình',
    '🤙🏾' => 'bàn tay gọi cho tôi: màu da tối trung bình',
    '🤙🏿' => 'bàn tay gọi cho tôi: màu da tối',
    '👈🏻' => 'ngón trỏ trái chỉ sang trái: màu da sáng',
    '👈🏼' => 'ngón trỏ trái chỉ sang trái: màu da sáng trung bình',
    '👈🏽' => 'ngón trỏ trái chỉ sang trái: màu da trung bình',
    '👈🏾' => 'ngón trỏ trái chỉ sang trái: màu da tối trung bình',
    '👈🏿' => 'ngón trỏ trái chỉ sang trái: màu da tối',
    '👉🏻' => 'ngón trỏ trái chỉ sang phải: màu da sáng',
    '👉🏼' => 'ngón trỏ trái chỉ sang phải: màu da sáng trung bình',
    '👉🏽' => 'ngón trỏ trái chỉ sang phải: màu da trung bình',
    '👉🏾' => 'ngón trỏ trái chỉ sang phải: màu da tối trung bình',
    '👉🏿' => 'ngón trỏ trái chỉ sang phải: màu da tối',
    '👆🏻' => 'ngón trỏ trái chỉ lên: màu da sáng',
    '👆🏼' => 'ngón trỏ trái chỉ lên: màu da sáng trung bình',
    '👆🏽' => 'ngón trỏ trái chỉ lên: màu da trung bình',
    '👆🏾' => 'ngón trỏ trái chỉ lên: màu da tối trung bình',
    '👆🏿' => 'ngón trỏ trái chỉ lên: màu da tối',
    '🖕🏻' => 'ngón giữa: màu da sáng',
    '🖕🏼' => 'ngón giữa: màu da sáng trung bình',
    '🖕🏽' => 'ngón giữa: màu da trung bình',
    '🖕🏾' => 'ngón giữa: màu da tối trung bình',
    '🖕🏿' => 'ngón giữa: màu da tối',
    '👇🏻' => 'ngón trỏ trái chỉ xuống: màu da sáng',
    '👇🏼' => 'ngón trỏ trái chỉ xuống: màu da sáng trung bình',
    '👇🏽' => 'ngón trỏ trái chỉ xuống: màu da trung bình',
    '👇🏾' => 'ngón trỏ trái chỉ xuống: màu da tối trung bình',
    '👇🏿' => 'ngón trỏ trái chỉ xuống: màu da tối',
    '☝🏻' => 'ngón trỏ chỉ lên trên: màu da sáng',
    '☝🏼' => 'ngón trỏ chỉ lên trên: màu da sáng trung bình',
    '☝🏽' => 'ngón trỏ chỉ lên trên: màu da trung bình',
    '☝🏾' => 'ngón trỏ chỉ lên trên: màu da tối trung bình',
    '☝🏿' => 'ngón trỏ chỉ lên trên: màu da tối',
    '🫵🏻' => 'ngón trỏ chỉ vào người xem: màu da sáng',
    '🫵🏼' => 'ngón trỏ chỉ vào người xem: màu da sáng trung bình',
    '🫵🏽' => 'ngón trỏ chỉ vào người xem: màu da trung bình',
    '🫵🏾' => 'ngón trỏ chỉ vào người xem: màu da tối trung bình',
    '🫵🏿' => 'ngón trỏ chỉ vào người xem: màu da tối',
    '👍🏻' => 'dấu ra hiệu đồng ý: màu da sáng',
    '👍🏼' => 'dấu ra hiệu đồng ý: màu da sáng trung bình',
    '👍🏽' => 'dấu ra hiệu đồng ý: màu da trung bình',
    '👍🏾' => 'dấu ra hiệu đồng ý: màu da tối trung bình',
    '👍🏿' => 'dấu ra hiệu đồng ý: màu da tối',
    '👎🏻' => 'dấu ra hiệu từ chối: màu da sáng',
    '👎🏼' => 'dấu ra hiệu từ chối: màu da sáng trung bình',
    '👎🏽' => 'dấu ra hiệu từ chối: màu da trung bình',
    '👎🏾' => 'dấu ra hiệu từ chối: màu da tối trung bình',
    '👎🏿' => 'dấu ra hiệu từ chối: màu da tối',
    '✊🏻' => 'nắm đấm giơ lên: màu da sáng',
    '✊🏼' => 'nắm đấm giơ lên: màu da sáng trung bình',
    '✊🏽' => 'nắm đấm giơ lên: màu da trung bình',
    '✊🏾' => 'nắm đấm giơ lên: màu da tối trung bình',
    '✊🏿' => 'nắm đấm giơ lên: màu da tối',
    '👊🏻' => 'nắm đấm: màu da sáng',
    '👊🏼' => 'nắm đấm: màu da sáng trung bình',
    '👊🏽' => 'nắm đấm: màu da trung bình',
    '👊🏾' => 'nắm đấm: màu da tối trung bình',
    '👊🏿' => 'nắm đấm: màu da tối',
    '🤛🏻' => 'nắm đấm hướng phía trái: màu da sáng',
    '🤛🏼' => 'nắm đấm hướng phía trái: màu da sáng trung bình',
    '🤛🏽' => 'nắm đấm hướng phía trái: màu da trung bình',
    '🤛🏾' => 'nắm đấm hướng phía trái: màu da tối trung bình',
    '🤛🏿' => 'nắm đấm hướng phía trái: màu da tối',
    '🤜🏻' => 'nắm đấm hướng sang phải: màu da sáng',
    '🤜🏼' => 'nắm đấm hướng sang phải: màu da sáng trung bình',
    '🤜🏽' => 'nắm đấm hướng sang phải: màu da trung bình',
    '🤜🏾' => 'nắm đấm hướng sang phải: màu da tối trung bình',
    '🤜🏿' => 'nắm đấm hướng sang phải: màu da tối',
    '👏🏻' => 'vỗ tay: màu da sáng',
    '👏🏼' => 'vỗ tay: màu da sáng trung bình',
    '👏🏽' => 'vỗ tay: màu da trung bình',
    '👏🏾' => 'vỗ tay: màu da tối trung bình',
    '👏🏿' => 'vỗ tay: màu da tối',
    '🙌🏻' => 'hai bàn tay giơ lên: màu da sáng',
    '🙌🏼' => 'hai bàn tay giơ lên: màu da sáng trung bình',
    '🙌🏽' => 'hai bàn tay giơ lên: màu da trung bình',
    '🙌🏾' => 'hai bàn tay giơ lên: màu da tối trung bình',
    '🙌🏿' => 'hai bàn tay giơ lên: màu da tối',
    '🫶🏻' => 'hai bàn tay tạo hình trái tim: màu da sáng',
    '🫶🏼' => 'hai bàn tay tạo hình trái tim: màu da sáng trung bình',
    '🫶🏽' => 'hai bàn tay tạo hình trái tim: màu da trung bình',
    '🫶🏾' => 'hai bàn tay tạo hình trái tim: màu da tối trung bình',
    '🫶🏿' => 'hai bàn tay tạo hình trái tim: màu da tối',
    '👐🏻' => 'hai bàn tay đang xòe: màu da sáng',
    '👐🏼' => 'hai bàn tay đang xòe: màu da sáng trung bình',
    '👐🏽' => 'hai bàn tay đang xòe: màu da trung bình',
    '👐🏾' => 'hai bàn tay đang xòe: màu da tối trung bình',
    '👐🏿' => 'hai bàn tay đang xòe: màu da tối',
    '🤲🏻' => 'hai bàn tay chạm vào nhau: màu da sáng',
    '🤲🏼' => 'hai bàn tay chạm vào nhau: màu da sáng trung bình',
    '🤲🏽' => 'hai bàn tay chạm vào nhau: màu da trung bình',
    '🤲🏾' => 'hai bàn tay chạm vào nhau: màu da tối trung bình',
    '🤲🏿' => 'hai bàn tay chạm vào nhau: màu da tối',
    '🤝🏻' => 'bắt tay: màu da sáng',
    '🤝🏼' => 'bắt tay: màu da sáng trung bình',
    '🤝🏽' => 'bắt tay: màu da trung bình',
    '🤝🏾' => 'bắt tay: màu da tối trung bình',
    '🤝🏿' => 'bắt tay: màu da tối',
    '🙏🏻' => 'chắp tay: màu da sáng',
    '🙏🏼' => 'chắp tay: màu da sáng trung bình',
    '🙏🏽' => 'chắp tay: màu da trung bình',
    '🙏🏾' => 'chắp tay: màu da tối trung bình',
    '🙏🏿' => 'chắp tay: màu da tối',
    '✍🏻' => 'bàn tay đang viết: màu da sáng',
    '✍🏼' => 'bàn tay đang viết: màu da sáng trung bình',
    '✍🏽' => 'bàn tay đang viết: màu da trung bình',
    '✍🏾' => 'bàn tay đang viết: màu da tối trung bình',
    '✍🏿' => 'bàn tay đang viết: màu da tối',
    '💅🏻' => 'sơn móng tay: màu da sáng',
    '💅🏼' => 'sơn móng tay: màu da sáng trung bình',
    '💅🏽' => 'sơn móng tay: màu da trung bình',
    '💅🏾' => 'sơn móng tay: màu da tối trung bình',
    '💅🏿' => 'sơn móng tay: màu da tối',
    '🤳🏻' => 'tự sướng: màu da sáng',
    '🤳🏼' => 'tự sướng: màu da sáng trung bình',
    '🤳🏽' => 'tự sướng: màu da trung bình',
    '🤳🏾' => 'tự sướng: màu da tối trung bình',
    '🤳🏿' => 'tự sướng: màu da tối',
    '💪🏻' => 'bắp tay gập lại: màu da sáng',
    '💪🏼' => 'bắp tay gập lại: màu da sáng trung bình',
    '💪🏽' => 'bắp tay gập lại: màu da trung bình',
    '💪🏾' => 'bắp tay gập lại: màu da tối trung bình',
    '💪🏿' => 'bắp tay gập lại: màu da tối',
    '🦵🏻' => 'Chân: màu da sáng',
    '🦵🏼' => 'Chân: màu da sáng trung bình',
    '🦵🏽' => 'Chân: màu da trung bình',
    '🦵🏾' => 'Chân: màu da tối trung bình',
    '🦵🏿' => 'Chân: màu da tối',
    '🦶🏻' => 'bàn chân: màu da sáng',
    '🦶🏼' => 'bàn chân: màu da sáng trung bình',
    '🦶🏽' => 'bàn chân: màu da trung bình',
    '🦶🏾' => 'bàn chân: màu da tối trung bình',
    '🦶🏿' => 'bàn chân: màu da tối',
    '👂🏻' => 'tai: màu da sáng',
    '👂🏼' => 'tai: màu da sáng trung bình',
    '👂🏽' => 'tai: màu da trung bình',
    '👂🏾' => 'tai: màu da tối trung bình',
    '👂🏿' => 'tai: màu da tối',
    '🦻🏻' => 'tai đeo thiết bị trợ thính: màu da sáng',
    '🦻🏼' => 'tai đeo thiết bị trợ thính: màu da sáng trung bình',
    '🦻🏽' => 'tai đeo thiết bị trợ thính: màu da trung bình',
    '🦻🏾' => 'tai đeo thiết bị trợ thính: màu da tối trung bình',
    '🦻🏿' => 'tai đeo thiết bị trợ thính: màu da tối',
    '👃🏻' => 'mũi: màu da sáng',
    '👃🏼' => 'mũi: màu da sáng trung bình',
    '👃🏽' => 'mũi: màu da trung bình',
    '👃🏾' => 'mũi: màu da tối trung bình',
    '👃🏿' => 'mũi: màu da tối',
    '👶🏻' => 'trẻ con: màu da sáng',
    '👶🏼' => 'trẻ con: màu da sáng trung bình',
    '👶🏽' => 'trẻ con: màu da trung bình',
    '👶🏾' => 'trẻ con: màu da tối trung bình',
    '👶🏿' => 'trẻ con: màu da tối',
    '🧒🏻' => 'trẻ em: màu da sáng',
    '🧒🏼' => 'trẻ em: màu da sáng trung bình',
    '🧒🏽' => 'trẻ em: màu da trung bình',
    '🧒🏾' => 'trẻ em: màu da tối trung bình',
    '🧒🏿' => 'trẻ em: màu da tối',
    '👦🏻' => 'con trai: màu da sáng',
    '👦🏼' => 'con trai: màu da sáng trung bình',
    '👦🏽' => 'con trai: màu da trung bình',
    '👦🏾' => 'con trai: màu da tối trung bình',
    '👦🏿' => 'con trai: màu da tối',
    '👧🏻' => 'con gái: màu da sáng',
    '👧🏼' => 'con gái: màu da sáng trung bình',
    '👧🏽' => 'con gái: màu da trung bình',
    '👧🏾' => 'con gái: màu da tối trung bình',
    '👧🏿' => 'con gái: màu da tối',
    '🧑🏻' => 'người: màu da sáng',
    '🧑🏼' => 'người: màu da sáng trung bình',
    '🧑🏽' => 'người: màu da trung bình',
    '🧑🏾' => 'người: màu da tối trung bình',
    '🧑🏿' => 'người: màu da tối',
    '👱🏻' => 'người tóc vàng hoe: màu da sáng',
    '👱🏼' => 'người tóc vàng hoe: màu da sáng trung bình',
    '👱🏽' => 'người tóc vàng hoe: màu da trung bình',
    '👱🏾' => 'người tóc vàng hoe: màu da tối trung bình',
    '👱🏿' => 'người tóc vàng hoe: màu da tối',
    '👨🏻' => 'đàn ông: màu da sáng',
    '👨🏼' => 'đàn ông: màu da sáng trung bình',
    '👨🏽' => 'đàn ông: màu da trung bình',
    '👨🏾' => 'đàn ông: màu da tối trung bình',
    '👨🏿' => 'đàn ông: màu da tối',
    '🧔🏻' => 'người có râu: màu da sáng',
    '🧔🏼' => 'người có râu: màu da sáng trung bình',
    '🧔🏽' => 'người có râu: màu da trung bình',
    '🧔🏾' => 'người có râu: màu da tối trung bình',
    '🧔🏿' => 'người có râu: màu da tối',
    '👩🏻' => 'phụ nữ: màu da sáng',
    '👩🏼' => 'phụ nữ: màu da sáng trung bình',
    '👩🏽' => 'phụ nữ: màu da trung bình',
    '👩🏾' => 'phụ nữ: màu da tối trung bình',
    '👩🏿' => 'phụ nữ: màu da tối',
    '🧓🏻' => 'người lớn tuổi: màu da sáng',
    '🧓🏼' => 'người lớn tuổi: màu da sáng trung bình',
    '🧓🏽' => 'người lớn tuổi: màu da trung bình',
    '🧓🏾' => 'người lớn tuổi: màu da tối trung bình',
    '🧓🏿' => 'người lớn tuổi: màu da tối',
    '👴🏻' => 'cụ ông: màu da sáng',
    '👴🏼' => 'cụ ông: màu da sáng trung bình',
    '👴🏽' => 'cụ ông: màu da trung bình',
    '👴🏾' => 'cụ ông: màu da tối trung bình',
    '👴🏿' => 'cụ ông: màu da tối',
    '👵🏻' => 'cụ bà: màu da sáng',
    '👵🏼' => 'cụ bà: màu da sáng trung bình',
    '👵🏽' => 'cụ bà: màu da trung bình',
    '👵🏾' => 'cụ bà: màu da tối trung bình',
    '👵🏿' => 'cụ bà: màu da tối',
    '🙍🏻' => 'người đang cau mày: màu da sáng',
    '🙍🏼' => 'người đang cau mày: màu da sáng trung bình',
    '🙍🏽' => 'người đang cau mày: màu da trung bình',
    '🙍🏾' => 'người đang cau mày: màu da tối trung bình',
    '🙍🏿' => 'người đang cau mày: màu da tối',
    '🙎🏻' => 'người đang bĩu môi: màu da sáng',
    '🙎🏼' => 'người đang bĩu môi: màu da sáng trung bình',
    '🙎🏽' => 'người đang bĩu môi: màu da trung bình',
    '🙎🏾' => 'người đang bĩu môi: màu da tối trung bình',
    '🙎🏿' => 'người đang bĩu môi: màu da tối',
    '🙅🏻' => 'người ra hiệu phản đối: màu da sáng',
    '🙅🏼' => 'người ra hiệu phản đối: màu da sáng trung bình',
    '🙅🏽' => 'người ra hiệu phản đối: màu da trung bình',
    '🙅🏾' => 'người ra hiệu phản đối: màu da tối trung bình',
    '🙅🏿' => 'người ra hiệu phản đối: màu da tối',
    '🙆🏻' => 'người ra hiệu đồng ý: màu da sáng',
    '🙆🏼' => 'người ra hiệu đồng ý: màu da sáng trung bình',
    '🙆🏽' => 'người ra hiệu đồng ý: màu da trung bình',
    '🙆🏾' => 'người ra hiệu đồng ý: màu da tối trung bình',
    '🙆🏿' => 'người ra hiệu đồng ý: màu da tối',
    '💁🏻' => 'người ra hiệu trợ giúp: màu da sáng',
    '💁🏼' => 'người ra hiệu trợ giúp: màu da sáng trung bình',
    '💁🏽' => 'người ra hiệu trợ giúp: màu da trung bình',
    '💁🏾' => 'người ra hiệu trợ giúp: màu da tối trung bình',
    '💁🏿' => 'người ra hiệu trợ giúp: màu da tối',
    '🙋🏻' => 'người giơ cao tay: màu da sáng',
    '🙋🏼' => 'người giơ cao tay: màu da sáng trung bình',
    '🙋🏽' => 'người giơ cao tay: màu da trung bình',
    '🙋🏾' => 'người giơ cao tay: màu da tối trung bình',
    '🙋🏿' => 'người giơ cao tay: màu da tối',
    '🧏🏻' => 'người khiếm thính: màu da sáng',
    '🧏🏼' => 'người khiếm thính: màu da sáng trung bình',
    '🧏🏽' => 'người khiếm thính: màu da trung bình',
    '🧏🏾' => 'người khiếm thính: màu da tối trung bình',
    '🧏🏿' => 'người khiếm thính: màu da tối',
    '🙇🏻' => 'người đang cúi đầu: màu da sáng',
    '🙇🏼' => 'người đang cúi đầu: màu da sáng trung bình',
    '🙇🏽' => 'người đang cúi đầu: màu da trung bình',
    '🙇🏾' => 'người đang cúi đầu: màu da tối trung bình',
    '🙇🏿' => 'người đang cúi đầu: màu da tối',
    '🤦🏻' => 'người lấy tay che mặt: màu da sáng',
    '🤦🏼' => 'người lấy tay che mặt: màu da sáng trung bình',
    '🤦🏽' => 'người lấy tay che mặt: màu da trung bình',
    '🤦🏾' => 'người lấy tay che mặt: màu da tối trung bình',
    '🤦🏿' => 'người lấy tay che mặt: màu da tối',
    '🤷🏻' => 'người nhún vai: màu da sáng',
    '🤷🏼' => 'người nhún vai: màu da sáng trung bình',
    '🤷🏽' => 'người nhún vai: màu da trung bình',
    '🤷🏾' => 'người nhún vai: màu da tối trung bình',
    '🤷🏿' => 'người nhún vai: màu da tối',
    '👮🏻' => 'cảnh sát: màu da sáng',
    '👮🏼' => 'cảnh sát: màu da sáng trung bình',
    '👮🏽' => 'cảnh sát: màu da trung bình',
    '👮🏾' => 'cảnh sát: màu da tối trung bình',
    '👮🏿' => 'cảnh sát: màu da tối',
    '🕵🏻' => 'thám tử: màu da sáng',
    '🕵🏼' => 'thám tử: màu da sáng trung bình',
    '🕵🏽' => 'thám tử: màu da trung bình',
    '🕵🏾' => 'thám tử: màu da tối trung bình',
    '🕵🏿' => 'thám tử: màu da tối',
    '💂🏻' => 'bảo vệ: màu da sáng',
    '💂🏼' => 'bảo vệ: màu da sáng trung bình',
    '💂🏽' => 'bảo vệ: màu da trung bình',
    '💂🏾' => 'bảo vệ: màu da tối trung bình',
    '💂🏿' => 'bảo vệ: màu da tối',
    '🥷🏻' => 'ninja: màu da sáng',
    '🥷🏼' => 'ninja: màu da sáng trung bình',
    '🥷🏽' => 'ninja: màu da trung bình',
    '🥷🏾' => 'ninja: màu da tối trung bình',
    '🥷🏿' => 'ninja: màu da tối',
    '👷🏻' => 'công nhân xây dựng: màu da sáng',
    '👷🏼' => 'công nhân xây dựng: màu da sáng trung bình',
    '👷🏽' => 'công nhân xây dựng: màu da trung bình',
    '👷🏾' => 'công nhân xây dựng: màu da tối trung bình',
    '👷🏿' => 'công nhân xây dựng: màu da tối',
    '🫅🏻' => 'người đội vương miện: màu da sáng',
    '🫅🏼' => 'người đội vương miện: màu da sáng trung bình',
    '🫅🏽' => 'người đội vương miện: màu da trung bình',
    '🫅🏾' => 'người đội vương miện: màu da tối trung bình',
    '🫅🏿' => 'người đội vương miện: màu da tối',
    '🤴🏻' => 'hoàng tử: màu da sáng',
    '🤴🏼' => 'hoàng tử: màu da sáng trung bình',
    '🤴🏽' => 'hoàng tử: màu da trung bình',
    '🤴🏾' => 'hoàng tử: màu da tối trung bình',
    '🤴🏿' => 'hoàng tử: màu da tối',
    '👸🏻' => 'công chúa: màu da sáng',
    '👸🏼' => 'công chúa: màu da sáng trung bình',
    '👸🏽' => 'công chúa: màu da trung bình',
    '👸🏾' => 'công chúa: màu da tối trung bình',
    '👸🏿' => 'công chúa: màu da tối',
    '👳🏻' => 'người đội khăn xếp: màu da sáng',
    '👳🏼' => 'người đội khăn xếp: màu da sáng trung bình',
    '👳🏽' => 'người đội khăn xếp: màu da trung bình',
    '👳🏾' => 'người đội khăn xếp: màu da tối trung bình',
    '👳🏿' => 'người đội khăn xếp: màu da tối',
    '👲🏻' => 'người đàn ông đội mũ Trung Quốc: màu da sáng',
    '👲🏼' => 'người đàn ông đội mũ Trung Quốc: màu da sáng trung bình',
    '👲🏽' => 'người đàn ông đội mũ Trung Quốc: màu da trung bình',
    '👲🏾' => 'người đàn ông đội mũ Trung Quốc: màu da tối trung bình',
    '👲🏿' => 'người đàn ông đội mũ Trung Quốc: màu da tối',
    '🧕🏻' => 'người phụ nữ quàng khăn trùm đầu: màu da sáng',
    '🧕🏼' => 'người phụ nữ quàng khăn trùm đầu: màu da sáng trung bình',
    '🧕🏽' => 'người phụ nữ quàng khăn trùm đầu: màu da trung bình',
    '🧕🏾' => 'người phụ nữ quàng khăn trùm đầu: màu da tối trung bình',
    '🧕🏿' => 'người phụ nữ quàng khăn trùm đầu: màu da tối',
    '🤵🏻' => 'người mặc bộ vest: màu da sáng',
    '🤵🏼' => 'người mặc bộ vest: màu da sáng trung bình',
    '🤵🏽' => 'người mặc bộ vest: màu da trung bình',
    '🤵🏾' => 'người mặc bộ vest: màu da tối trung bình',
    '🤵🏿' => 'người mặc bộ vest: màu da tối',
    '👰🏻' => 'người đội khăn voan: màu da sáng',
    '👰🏼' => 'người đội khăn voan: màu da sáng trung bình',
    '👰🏽' => 'người đội khăn voan: màu da trung bình',
    '👰🏾' => 'người đội khăn voan: màu da tối trung bình',
    '👰🏿' => 'người đội khăn voan: màu da tối',
    '🤰🏻' => 'người phụ nữ có thai: màu da sáng',
    '🤰🏼' => 'người phụ nữ có thai: màu da sáng trung bình',
    '🤰🏽' => 'người phụ nữ có thai: màu da trung bình',
    '🤰🏾' => 'người phụ nữ có thai: màu da tối trung bình',
    '🤰🏿' => 'người phụ nữ có thai: màu da tối',
    '🫃🏻' => 'người đàn ông mang bầu: màu da sáng',
    '🫃🏼' => 'người đàn ông mang bầu: màu da sáng trung bình',
    '🫃🏽' => 'người đàn ông mang bầu: màu da trung bình',
    '🫃🏾' => 'người đàn ông mang bầu: màu da tối trung bình',
    '🫃🏿' => 'người đàn ông mang bầu: màu da tối',
    '🫄🏻' => 'người mang bầu: màu da sáng',
    '🫄🏼' => 'người mang bầu: màu da sáng trung bình',
    '🫄🏽' => 'người mang bầu: màu da trung bình',
    '🫄🏾' => 'người mang bầu: màu da tối trung bình',
    '🫄🏿' => 'người mang bầu: màu da tối',
    '🤱🏻' => 'cho con bú: màu da sáng',
    '🤱🏼' => 'cho con bú: màu da sáng trung bình',
    '🤱🏽' => 'cho con bú: màu da trung bình',
    '🤱🏾' => 'cho con bú: màu da tối trung bình',
    '🤱🏿' => 'cho con bú: màu da tối',
    '👼🏻' => 'thiên thần nhỏ: màu da sáng',
    '👼🏼' => 'thiên thần nhỏ: màu da sáng trung bình',
    '👼🏽' => 'thiên thần nhỏ: màu da trung bình',
    '👼🏾' => 'thiên thần nhỏ: màu da tối trung bình',
    '👼🏿' => 'thiên thần nhỏ: màu da tối',
    '🎅🏻' => 'Ông già Noel: màu da sáng',
    '🎅🏼' => 'Ông già Noel: màu da sáng trung bình',
    '🎅🏽' => 'Ông già Noel: màu da trung bình',
    '🎅🏾' => 'Ông già Noel: màu da tối trung bình',
    '🎅🏿' => 'Ông già Noel: màu da tối',
    '🤶🏻' => 'mẹ giáng sinh: màu da sáng',
    '🤶🏼' => 'mẹ giáng sinh: màu da sáng trung bình',
    '🤶🏽' => 'mẹ giáng sinh: màu da trung bình',
    '🤶🏾' => 'mẹ giáng sinh: màu da tối trung bình',
    '🤶🏿' => 'mẹ giáng sinh: màu da tối',
    '🦸🏻' => 'siêu anh hùng: màu da sáng',
    '🦸🏼' => 'siêu anh hùng: màu da sáng trung bình',
    '🦸🏽' => 'siêu anh hùng: màu da trung bình',
    '🦸🏾' => 'siêu anh hùng: màu da tối trung bình',
    '🦸🏿' => 'siêu anh hùng: màu da tối',
    '🦹🏻' => 'nhân vật siêu phản diện: màu da sáng',
    '🦹🏼' => 'nhân vật siêu phản diện: màu da sáng trung bình',
    '🦹🏽' => 'nhân vật siêu phản diện: màu da trung bình',
    '🦹🏾' => 'nhân vật siêu phản diện: màu da tối trung bình',
    '🦹🏿' => 'nhân vật siêu phản diện: màu da tối',
    '🧙🏻' => 'pháp sư: màu da sáng',
    '🧙🏼' => 'pháp sư: màu da sáng trung bình',
    '🧙🏽' => 'pháp sư: màu da trung bình',
    '🧙🏾' => 'pháp sư: màu da tối trung bình',
    '🧙🏿' => 'pháp sư: màu da tối',
    '🧚🏻' => 'tiên: màu da sáng',
    '🧚🏼' => 'tiên: màu da sáng trung bình',
    '🧚🏽' => 'tiên: màu da trung bình',
    '🧚🏾' => 'tiên: màu da tối trung bình',
    '🧚🏿' => 'tiên: màu da tối',
    '🧛🏻' => 'ma cà rồng: màu da sáng',
    '🧛🏼' => 'ma cà rồng: màu da sáng trung bình',
    '🧛🏽' => 'ma cà rồng: màu da trung bình',
    '🧛🏾' => 'ma cà rồng: màu da tối trung bình',
    '🧛🏿' => 'ma cà rồng: màu da tối',
    '🧜🏻' => 'người cá: màu da sáng',
    '🧜🏼' => 'người cá: màu da sáng trung bình',
    '🧜🏽' => 'người cá: màu da trung bình',
    '🧜🏾' => 'người cá: màu da tối trung bình',
    '🧜🏿' => 'người cá: màu da tối',
    '🧝🏻' => 'yêu ma: màu da sáng',
    '🧝🏼' => 'yêu ma: màu da sáng trung bình',
    '🧝🏽' => 'yêu ma: màu da trung bình',
    '🧝🏾' => 'yêu ma: màu da tối trung bình',
    '🧝🏿' => 'yêu ma: màu da tối',
    '💆🏻' => 'người được xoa bóp mặt: màu da sáng',
    '💆🏼' => 'người được xoa bóp mặt: màu da sáng trung bình',
    '💆🏽' => 'người được xoa bóp mặt: màu da trung bình',
    '💆🏾' => 'người được xoa bóp mặt: màu da tối trung bình',
    '💆🏿' => 'người được xoa bóp mặt: màu da tối',
    '💇🏻' => 'người được cắt tóc: màu da sáng',
    '💇🏼' => 'người được cắt tóc: màu da sáng trung bình',
    '💇🏽' => 'người được cắt tóc: màu da trung bình',
    '💇🏾' => 'người được cắt tóc: màu da tối trung bình',
    '💇🏿' => 'người được cắt tóc: màu da tối',
    '🚶🏻' => 'người đi bộ: màu da sáng',
    '🚶🏼' => 'người đi bộ: màu da sáng trung bình',
    '🚶🏽' => 'người đi bộ: màu da trung bình',
    '🚶🏾' => 'người đi bộ: màu da tối trung bình',
    '🚶🏿' => 'người đi bộ: màu da tối',
    '🧍🏻' => 'người đứng: màu da sáng',
    '🧍🏼' => 'người đứng: màu da sáng trung bình',
    '🧍🏽' => 'người đứng: màu da trung bình',
    '🧍🏾' => 'người đứng: màu da tối trung bình',
    '🧍🏿' => 'người đứng: màu da tối',
    '🧎🏻' => 'người quỳ: màu da sáng',
    '🧎🏼' => 'người quỳ: màu da sáng trung bình',
    '🧎🏽' => 'người quỳ: màu da trung bình',
    '🧎🏾' => 'người quỳ: màu da tối trung bình',
    '🧎🏿' => 'người quỳ: màu da tối',
    '🏃🏻' => 'người chạy: màu da sáng',
    '🏃🏼' => 'người chạy: màu da sáng trung bình',
    '🏃🏽' => 'người chạy: màu da trung bình',
    '🏃🏾' => 'người chạy: màu da tối trung bình',
    '🏃🏿' => 'người chạy: màu da tối',
    '💃🏻' => 'người phụ nữ đang khiêu vũ: màu da sáng',
    '💃🏼' => 'người phụ nữ đang khiêu vũ: màu da sáng trung bình',
    '💃🏽' => 'người phụ nữ đang khiêu vũ: màu da trung bình',
    '💃🏾' => 'người phụ nữ đang khiêu vũ: màu da tối trung bình',
    '💃🏿' => 'người phụ nữ đang khiêu vũ: màu da tối',
    '🕺🏻' => 'người đàn ông đang khiêu vũ: màu da sáng',
    '🕺🏼' => 'người đàn ông đang khiêu vũ: màu da sáng trung bình',
    '🕺🏽' => 'người đàn ông đang khiêu vũ: màu da trung bình',
    '🕺🏾' => 'người đàn ông đang khiêu vũ: màu da tối trung bình',
    '🕺🏿' => 'người đàn ông đang khiêu vũ: màu da tối',
    '🕴🏻' => 'người đàn ông mặc vest đang bay lên: màu da sáng',
    '🕴🏼' => 'người đàn ông mặc vest đang bay lên: màu da sáng trung bình',
    '🕴🏽' => 'người đàn ông mặc vest đang bay lên: màu da trung bình',
    '🕴🏾' => 'người đàn ông mặc vest đang bay lên: màu da tối trung bình',
    '🕴🏿' => 'người đàn ông mặc vest đang bay lên: màu da tối',
    '👯🏻' => 'những người đeo tai thỏ: màu da sáng',
    '👯🏼' => 'những người đeo tai thỏ: màu da sáng trung bình',
    '👯🏽' => 'những người đeo tai thỏ: màu da trung bình',
    '👯🏾' => 'những người đeo tai thỏ: màu da tối trung bình',
    '👯🏿' => 'những người đeo tai thỏ: màu da tối',
    '🧖🏻' => 'người ở trong phòng xông hơi: màu da sáng',
    '🧖🏼' => 'người ở trong phòng xông hơi: màu da sáng trung bình',
    '🧖🏽' => 'người ở trong phòng xông hơi: màu da trung bình',
    '🧖🏾' => 'người ở trong phòng xông hơi: màu da tối trung bình',
    '🧖🏿' => 'người ở trong phòng xông hơi: màu da tối',
    '🧗🏻' => 'người đang leo núi: màu da sáng',
    '🧗🏼' => 'người đang leo núi: màu da sáng trung bình',
    '🧗🏽' => 'người đang leo núi: màu da trung bình',
    '🧗🏾' => 'người đang leo núi: màu da tối trung bình',
    '🧗🏿' => 'người đang leo núi: màu da tối',
    '🏇🏻' => 'đua ngựa: màu da sáng',
    '🏇🏼' => 'đua ngựa: màu da sáng trung bình',
    '🏇🏽' => 'đua ngựa: màu da trung bình',
    '🏇🏾' => 'đua ngựa: màu da tối trung bình',
    '🏇🏿' => 'đua ngựa: màu da tối',
    '🏂🏻' => 'người trượt ván tuyết: màu da sáng',
    '🏂🏼' => 'người trượt ván tuyết: màu da sáng trung bình',
    '🏂🏽' => 'người trượt ván tuyết: màu da trung bình',
    '🏂🏾' => 'người trượt ván tuyết: màu da tối trung bình',
    '🏂🏿' => 'người trượt ván tuyết: màu da tối',
    '🏌🏻' => 'người chơi gôn: màu da sáng',
    '🏌🏼' => 'người chơi gôn: màu da sáng trung bình',
    '🏌🏽' => 'người chơi gôn: màu da trung bình',
    '🏌🏾' => 'người chơi gôn: màu da tối trung bình',
    '🏌🏿' => 'người chơi gôn: màu da tối',
    '🏄🏻' => 'người lướt sóng: màu da sáng',
    '🏄🏼' => 'người lướt sóng: màu da sáng trung bình',
    '🏄🏽' => 'người lướt sóng: màu da trung bình',
    '🏄🏾' => 'người lướt sóng: màu da tối trung bình',
    '🏄🏿' => 'người lướt sóng: màu da tối',
    '🚣🏻' => 'người chèo thuyền: màu da sáng',
    '🚣🏼' => 'người chèo thuyền: màu da sáng trung bình',
    '🚣🏽' => 'người chèo thuyền: màu da trung bình',
    '🚣🏾' => 'người chèo thuyền: màu da tối trung bình',
    '🚣🏿' => 'người chèo thuyền: màu da tối',
    '🏊🏻' => 'người bơi: màu da sáng',
    '🏊🏼' => 'người bơi: màu da sáng trung bình',
    '🏊🏽' => 'người bơi: màu da trung bình',
    '🏊🏾' => 'người bơi: màu da tối trung bình',
    '🏊🏿' => 'người bơi: màu da tối',
    '⛹🏻' => 'người chơi bóng: màu da sáng',
    '⛹🏼' => 'người chơi bóng: màu da sáng trung bình',
    '⛹🏽' => 'người chơi bóng: màu da trung bình',
    '⛹🏾' => 'người chơi bóng: màu da tối trung bình',
    '⛹🏿' => 'người chơi bóng: màu da tối',
    '🏋🏻' => 'người cử tạ: màu da sáng',
    '🏋🏼' => 'người cử tạ: màu da sáng trung bình',
    '🏋🏽' => 'người cử tạ: màu da trung bình',
    '🏋🏾' => 'người cử tạ: màu da tối trung bình',
    '🏋🏿' => 'người cử tạ: màu da tối',
    '🚴🏻' => 'người đi xe đạp: màu da sáng',
    '🚴🏼' => 'người đi xe đạp: màu da sáng trung bình',
    '🚴🏽' => 'người đi xe đạp: màu da trung bình',
    '🚴🏾' => 'người đi xe đạp: màu da tối trung bình',
    '🚴🏿' => 'người đi xe đạp: màu da tối',
    '🚵🏻' => 'người đi xe đạp leo núi: màu da sáng',
    '🚵🏼' => 'người đi xe đạp leo núi: màu da sáng trung bình',
    '🚵🏽' => 'người đi xe đạp leo núi: màu da trung bình',
    '🚵🏾' => 'người đi xe đạp leo núi: màu da tối trung bình',
    '🚵🏿' => 'người đi xe đạp leo núi: màu da tối',
    '🤸🏻' => 'người nhào lộn: màu da sáng',
    '🤸🏼' => 'người nhào lộn: màu da sáng trung bình',
    '🤸🏽' => 'người nhào lộn: màu da trung bình',
    '🤸🏾' => 'người nhào lộn: màu da tối trung bình',
    '🤸🏿' => 'người nhào lộn: màu da tối',
    '🤼🏻' => 'người chơi vật: màu da sáng',
    '🤼🏼' => 'người chơi vật: màu da sáng trung bình',
    '🤼🏽' => 'người chơi vật: màu da trung bình',
    '🤼🏾' => 'người chơi vật: màu da tối trung bình',
    '🤼🏿' => 'người chơi vật: màu da tối',
    '🤽🏻' => 'người chơi bóng nước: màu da sáng',
    '🤽🏼' => 'người chơi bóng nước: màu da sáng trung bình',
    '🤽🏽' => 'người chơi bóng nước: màu da trung bình',
    '🤽🏾' => 'người chơi bóng nước: màu da tối trung bình',
    '🤽🏿' => 'người chơi bóng nước: màu da tối',
    '🤾🏻' => 'người chơi bóng ném: màu da sáng',
    '🤾🏼' => 'người chơi bóng ném: màu da sáng trung bình',
    '🤾🏽' => 'người chơi bóng ném: màu da trung bình',
    '🤾🏾' => 'người chơi bóng ném: màu da tối trung bình',
    '🤾🏿' => 'người chơi bóng ném: màu da tối',
    '🤹🏻' => 'người tung hứng: màu da sáng',
    '🤹🏼' => 'người tung hứng: màu da sáng trung bình',
    '🤹🏽' => 'người tung hứng: màu da trung bình',
    '🤹🏾' => 'người tung hứng: màu da tối trung bình',
    '🤹🏿' => 'người tung hứng: màu da tối',
    '🧘🏻' => 'người ngồi kiết già: màu da sáng',
    '🧘🏼' => 'người ngồi kiết già: màu da sáng trung bình',
    '🧘🏽' => 'người ngồi kiết già: màu da trung bình',
    '🧘🏾' => 'người ngồi kiết già: màu da tối trung bình',
    '🧘🏿' => 'người ngồi kiết già: màu da tối',
    '🛀🏻' => 'người đang tắm bồn: màu da sáng',
    '🛀🏼' => 'người đang tắm bồn: màu da sáng trung bình',
    '🛀🏽' => 'người đang tắm bồn: màu da trung bình',
    '🛀🏾' => 'người đang tắm bồn: màu da tối trung bình',
    '🛀🏿' => 'người đang tắm bồn: màu da tối',
    '🛌🏻' => 'người trên giường: màu da sáng',
    '🛌🏼' => 'người trên giường: màu da sáng trung bình',
    '🛌🏽' => 'người trên giường: màu da trung bình',
    '🛌🏾' => 'người trên giường: màu da tối trung bình',
    '🛌🏿' => 'người trên giường: màu da tối',
    '👭🏻' => 'hai người phụ nữ nắm tay: màu da sáng',
    '👭🏼' => 'hai người phụ nữ nắm tay: màu da sáng trung bình',
    '👭🏽' => 'hai người phụ nữ nắm tay: màu da trung bình',
    '👭🏾' => 'hai người phụ nữ nắm tay: màu da tối trung bình',
    '👭🏿' => 'hai người phụ nữ nắm tay: màu da tối',
    '👫🏻' => 'người đàn ông và phụ nữ nắm tay: màu da sáng',
    '👫🏼' => 'người đàn ông và phụ nữ nắm tay: màu da sáng trung bình',
    '👫🏽' => 'người đàn ông và phụ nữ nắm tay: màu da trung bình',
    '👫🏾' => 'người đàn ông và phụ nữ nắm tay: màu da tối trung bình',
    '👫🏿' => 'người đàn ông và phụ nữ nắm tay: màu da tối',
    '👬🏻' => 'hai người đàn ông nắm tay: màu da sáng',
    '👬🏼' => 'hai người đàn ông nắm tay: màu da sáng trung bình',
    '👬🏽' => 'hai người đàn ông nắm tay: màu da trung bình',
    '👬🏾' => 'hai người đàn ông nắm tay: màu da tối trung bình',
    '👬🏿' => 'hai người đàn ông nắm tay: màu da tối',
    '💏🏻' => 'nụ hôn: màu da sáng',
    '💏🏼' => 'nụ hôn: màu da sáng trung bình',
    '💏🏽' => 'nụ hôn: màu da trung bình',
    '💏🏾' => 'nụ hôn: màu da tối trung bình',
    '💏🏿' => 'nụ hôn: màu da tối',
    '💑🏻' => 'cặp đôi với trái tim: màu da sáng',
    '💑🏼' => 'cặp đôi với trái tim: màu da sáng trung bình',
    '💑🏽' => 'cặp đôi với trái tim: màu da trung bình',
    '💑🏾' => 'cặp đôi với trái tim: màu da tối trung bình',
    '💑🏿' => 'cặp đôi với trái tim: màu da tối',
    '🇦🇨' => 'cờ: Đảo Ascension',
    '🇦🇩' => 'cờ: Andorra',
    '🇦🇪' => 'cờ: Các Tiểu Vương quốc Ả Rập Thống nhất',
    '🇦🇫' => 'cờ: Afghanistan',
    '🇦🇬' => 'cờ: Antigua và Barbuda',
    '🇦🇮' => 'cờ: Anguilla',
    '🇦🇱' => 'cờ: Albania',
    '🇦🇲' => 'cờ: Armenia',
    '🇦🇴' => 'cờ: Angola',
    '🇦🇶' => 'cờ: Nam Cực',
    '🇦🇷' => 'cờ: Argentina',
    '🇦🇸' => 'cờ: Samoa thuộc Mỹ',
    '🇦🇹' => 'cờ: Áo',
    '🇦🇺' => 'cờ: Úc',
    '🇦🇼' => 'cờ: Aruba',
    '🇦🇽' => 'cờ: Quần đảo Åland',
    '🇦🇿' => 'cờ: Azerbaijan',
    '🇧🇦' => 'cờ: Bosnia và Herzegovina',
    '🇧🇧' => 'cờ: Barbados',
    '🇧🇩' => 'cờ: Bangladesh',
    '🇧🇪' => 'cờ: Bỉ',
    '🇧🇫' => 'cờ: Burkina Faso',
    '🇧🇬' => 'cờ: Bulgaria',
    '🇧🇭' => 'cờ: Bahrain',
    '🇧🇮' => 'cờ: Burundi',
    '🇧🇯' => 'cờ: Benin',
    '🇧🇱' => 'cờ: St. Barthélemy',
    '🇧🇲' => 'cờ: Bermuda',
    '🇧🇳' => 'cờ: Brunei',
    '🇧🇴' => 'cờ: Bolivia',
    '🇧🇶' => 'cờ: Ca-ri-bê Hà Lan',
    '🇧🇷' => 'cờ: Brazil',
    '🇧🇸' => 'cờ: Bahamas',
    '🇧🇹' => 'cờ: Bhutan',
    '🇧🇻' => 'cờ: Đảo Bouvet',
    '🇧🇼' => 'cờ: Botswana',
    '🇧🇾' => 'cờ: Belarus',
    '🇧🇿' => 'cờ: Belize',
    '🇨🇦' => 'cờ: Canada',
    '🇨🇨' => 'cờ: Quần đảo Cocos (Keeling)',
    '🇨🇩' => 'cờ: Congo - Kinshasa',
    '🇨🇫' => 'cờ: Cộng hòa Trung Phi',
    '🇨🇬' => 'cờ: Congo - Brazzaville',
    '🇨🇭' => 'cờ: Thụy Sĩ',
    '🇨🇮' => 'cờ: Côte d’Ivoire',
    '🇨🇰' => 'cờ: Quần đảo Cook',
    '🇨🇱' => 'cờ: Chile',
    '🇨🇲' => 'cờ: Cameroon',
    '🇨🇳' => 'cờ: Trung Quốc',
    '🇨🇴' => 'cờ: Colombia',
    '🇨🇵' => 'cờ: Đảo Clipperton',
    '🇨🇶' => 'cờ: Sark',
    '🇨🇷' => 'cờ: Costa Rica',
    '🇨🇺' => 'cờ: Cuba',
    '🇨🇻' => 'cờ: Cape Verde',
    '🇨🇼' => 'cờ: Curaçao',
    '🇨🇽' => 'cờ: Đảo Giáng Sinh',
    '🇨🇾' => 'cờ: Síp',
    '🇨🇿' => 'cờ: Séc',
    '🇩🇪' => 'cờ: Đức',
    '🇩🇬' => 'cờ: Diego Garcia',
    '🇩🇯' => 'cờ: Djibouti',
    '🇩🇰' => 'cờ: Đan Mạch',
    '🇩🇲' => 'cờ: Dominica',
    '🇩🇴' => 'cờ: Cộng hòa Dominica',
    '🇩🇿' => 'cờ: Algeria',
    '🇪🇦' => 'cờ: Ceuta và Melilla',
    '🇪🇨' => 'cờ: Ecuador',
    '🇪🇪' => 'cờ: Estonia',
    '🇪🇬' => 'cờ: Ai Cập',
    '🇪🇭' => 'cờ: Tây Sahara',
    '🇪🇷' => 'cờ: Eritrea',
    '🇪🇸' => 'cờ: Tây Ban Nha',
    '🇪🇹' => 'cờ: Ethiopia',
    '🇪🇺' => 'cờ: Liên Minh Châu Âu',
    '🇫🇮' => 'cờ: Phần Lan',
    '🇫🇯' => 'cờ: Fiji',
    '🇫🇰' => 'cờ: Quần đảo Falkland',
    '🇫🇲' => 'cờ: Micronesia',
    '🇫🇴' => 'cờ: Quần đảo Faroe',
    '🇫🇷' => 'cờ: Pháp',
    '🇬🇦' => 'cờ: Gabon',
    '🇬🇧' => 'cờ: Vương quốc Anh',
    '🇬🇩' => 'cờ: Grenada',
    '🇬🇪' => 'cờ: Georgia',
    '🇬🇫' => 'cờ: Guiana thuộc Pháp',
    '🇬🇬' => 'cờ: Guernsey',
    '🇬🇭' => 'cờ: Ghana',
    '🇬🇮' => 'cờ: Gibraltar',
    '🇬🇱' => 'cờ: Greenland',
    '🇬🇲' => 'cờ: Gambia',
    '🇬🇳' => 'cờ: Guinea',
    '🇬🇵' => 'cờ: Guadeloupe',
    '🇬🇶' => 'cờ: Guinea Xích Đạo',
    '🇬🇷' => 'cờ: Hy Lạp',
    '🇬🇸' => 'cờ: Nam Georgia & Quần đảo Nam Sandwich',
    '🇬🇹' => 'cờ: Guatemala',
    '🇬🇺' => 'cờ: Guam',
    '🇬🇼' => 'cờ: Guinea-Bissau',
    '🇬🇾' => 'cờ: Guyana',
    '🇭🇰' => 'cờ: Đặc khu Hành chính Hồng Kông, Trung Quốc',
    '🇭🇲' => 'cờ: Quần đảo Heard và McDonald',
    '🇭🇳' => 'cờ: Honduras',
    '🇭🇷' => 'cờ: Croatia',
    '🇭🇹' => 'cờ: Haiti',
    '🇭🇺' => 'cờ: Hungary',
    '🇮🇨' => 'cờ: Quần đảo Canary',
    '🇮🇩' => 'cờ: Indonesia',
    '🇮🇪' => 'cờ: Ireland',
    '🇮🇱' => 'cờ: Israel',
    '🇮🇲' => 'cờ: Đảo Man',
    '🇮🇳' => 'cờ: Ấn Độ',
    '🇮🇴' => 'cờ: Lãnh thổ Ấn Độ Dương thuộc Anh',
    '🇮🇶' => 'cờ: Iraq',
    '🇮🇷' => 'cờ: Iran',
    '🇮🇸' => 'cờ: Iceland',
    '🇮🇹' => 'cờ: Italy',
    '🇯🇪' => 'cờ: Jersey',
    '🇯🇲' => 'cờ: Jamaica',
    '🇯🇴' => 'cờ: Jordan',
    '🇯🇵' => 'cờ: Nhật Bản',
    '🇰🇪' => 'cờ: Kenya',
    '🇰🇬' => 'cờ: Kyrgyzstan',
    '🇰🇭' => 'cờ: Campuchia',
    '🇰🇮' => 'cờ: Kiribati',
    '🇰🇲' => 'cờ: Comoros',
    '🇰🇳' => 'cờ: St. Kitts và Nevis',
    '🇰🇵' => 'cờ: Triều Tiên',
    '🇰🇷' => 'cờ: Hàn Quốc',
    '🇰🇼' => 'cờ: Kuwait',
    '🇰🇾' => 'cờ: Quần đảo Cayman',
    '🇰🇿' => 'cờ: Kazakhstan',
    '🇱🇦' => 'cờ: Lào',
    '🇱🇧' => 'cờ: Li-băng',
    '🇱🇨' => 'cờ: St. Lucia',
    '🇱🇮' => 'cờ: Liechtenstein',
    '🇱🇰' => 'cờ: Sri Lanka',
    '🇱🇷' => 'cờ: Liberia',
    '🇱🇸' => 'cờ: Lesotho',
    '🇱🇹' => 'cờ: Litva',
    '🇱🇺' => 'cờ: Luxembourg',
    '🇱🇻' => 'cờ: Latvia',
    '🇱🇾' => 'cờ: Libya',
    '🇲🇦' => 'cờ: Ma-rốc',
    '🇲🇨' => 'cờ: Monaco',
    '🇲🇩' => 'cờ: Moldova',
    '🇲🇪' => 'cờ: Montenegro',
    '🇲🇫' => 'cờ: St. Martin',
    '🇲🇬' => 'cờ: Madagascar',
    '🇲🇭' => 'cờ: Quần đảo Marshall',
    '🇲🇰' => 'cờ: Bắc Macedonia',
    '🇲🇱' => 'cờ: Mali',
    '🇲🇲' => 'cờ: Myanmar (Miến Điện)',
    '🇲🇳' => 'cờ: Mông Cổ',
    '🇲🇴' => 'cờ: Đặc khu Hành chính Macao, Trung Quốc',
    '🇲🇵' => 'cờ: Quần đảo Bắc Mariana',
    '🇲🇶' => 'cờ: Martinique',
    '🇲🇷' => 'cờ: Mauritania',
    '🇲🇸' => 'cờ: Montserrat',
    '🇲🇹' => 'cờ: Malta',
    '🇲🇺' => 'cờ: Mauritius',
    '🇲🇻' => 'cờ: Maldives',
    '🇲🇼' => 'cờ: Malawi',
    '🇲🇽' => 'cờ: Mexico',
    '🇲🇾' => 'cờ: Malaysia',
    '🇲🇿' => 'cờ: Mozambique',
    '🇳🇦' => 'cờ: Namibia',
    '🇳🇨' => 'cờ: New Caledonia',
    '🇳🇪' => 'cờ: Niger',
    '🇳🇫' => 'cờ: Đảo Norfolk',
    '🇳🇬' => 'cờ: Nigeria',
    '🇳🇮' => 'cờ: Nicaragua',
    '🇳🇱' => 'cờ: Hà Lan',
    '🇳🇴' => 'cờ: Na Uy',
    '🇳🇵' => 'cờ: Nepal',
    '🇳🇷' => 'cờ: Nauru',
    '🇳🇺' => 'cờ: Niue',
    '🇳🇿' => 'cờ: New Zealand',
    '🇴🇲' => 'cờ: Oman',
    '🇵🇦' => 'cờ: Panama',
    '🇵🇪' => 'cờ: Peru',
    '🇵🇫' => 'cờ: Polynesia thuộc Pháp',
    '🇵🇬' => 'cờ: Papua New Guinea',
    '🇵🇭' => 'cờ: Philippines',
    '🇵🇰' => 'cờ: Pakistan',
    '🇵🇱' => 'cờ: Ba Lan',
    '🇵🇲' => 'cờ: Saint Pierre và Miquelon',
    '🇵🇳' => 'cờ: Quần đảo Pitcairn',
    '🇵🇷' => 'cờ: Puerto Rico',
    '🇵🇸' => 'cờ: Lãnh thổ Palestine',
    '🇵🇹' => 'cờ: Bồ Đào Nha',
    '🇵🇼' => 'cờ: Palau',
    '🇵🇾' => 'cờ: Paraguay',
    '🇶🇦' => 'cờ: Qatar',
    '🇷🇪' => 'cờ: Réunion',
    '🇷🇴' => 'cờ: Romania',
    '🇷🇸' => 'cờ: Serbia',
    '🇷🇺' => 'cờ: Nga',
    '🇷🇼' => 'cờ: Rwanda',
    '🇸🇦' => 'cờ: Ả Rập Xê-út',
    '🇸🇧' => 'cờ: Quần đảo Solomon',
    '🇸🇨' => 'cờ: Seychelles',
    '🇸🇩' => 'cờ: Sudan',
    '🇸🇪' => 'cờ: Thụy Điển',
    '🇸🇬' => 'cờ: Singapore',
    '🇸🇭' => 'cờ: St. Helena',
    '🇸🇮' => 'cờ: Slovenia',
    '🇸🇯' => 'cờ: Svalbard và Jan Mayen',
    '🇸🇰' => 'cờ: Slovakia',
    '🇸🇱' => 'cờ: Sierra Leone',
    '🇸🇲' => 'cờ: San Marino',
    '🇸🇳' => 'cờ: Senegal',
    '🇸🇴' => 'cờ: Somalia',
    '🇸🇷' => 'cờ: Suriname',
    '🇸🇸' => 'cờ: Nam Sudan',
    '🇸🇹' => 'cờ: São Tomé và Príncipe',
    '🇸🇻' => 'cờ: El Salvador',
    '🇸🇽' => 'cờ: Sint Maarten',
    '🇸🇾' => 'cờ: Syria',
    '🇸🇿' => 'cờ: Eswatini',
    '🇹🇦' => 'cờ: Tristan da Cunha',
    '🇹🇨' => 'cờ: Quần đảo Turks và Caicos',
    '🇹🇩' => 'cờ: Chad',
    '🇹🇫' => 'cờ: Lãnh thổ phía Nam Thuộc Pháp',
    '🇹🇬' => 'cờ: Togo',
    '🇹🇭' => 'cờ: Thái Lan',
    '🇹🇯' => 'cờ: Tajikistan',
    '🇹🇰' => 'cờ: Tokelau',
    '🇹🇱' => 'cờ: Timor-Leste',
    '🇹🇲' => 'cờ: Turkmenistan',
    '🇹🇳' => 'cờ: Tunisia',
    '🇹🇴' => 'cờ: Tonga',
    '🇹🇷' => 'cờ: Thổ Nhĩ Kỳ',
    '🇹🇹' => 'cờ: Trinidad và Tobago',
    '🇹🇻' => 'cờ: Tuvalu',
    '🇹🇼' => 'cờ: Đài Loan',
    '🇹🇿' => 'cờ: Tanzania',
    '🇺🇦' => 'cờ: Ukraina',
    '🇺🇬' => 'cờ: Uganda',
    '🇺🇲' => 'cờ: Các tiểu đảo xa của Hoa Kỳ',
    '🇺🇳' => 'cờ: Liên hợp quốc',
    '🇺🇸' => 'cờ: Hoa Kỳ',
    '🇺🇾' => 'cờ: Uruguay',
    '🇺🇿' => 'cờ: Uzbekistan',
    '🇻🇦' => 'cờ: Thành Vatican',
    '🇻🇨' => 'cờ: St. Vincent và Grenadines',
    '🇻🇪' => 'cờ: Venezuela',
    '🇻🇬' => 'cờ: Quần đảo Virgin thuộc Anh',
    '🇻🇮' => 'cờ: Quần đảo Virgin thuộc Hoa Kỳ',
    '🇻🇳' => 'cờ: Việt Nam',
    '🇻🇺' => 'cờ: Vanuatu',
    '🇼🇫' => 'cờ: Wallis và Futuna',
    '🇼🇸' => 'cờ: Samoa',
    '🇽🇰' => 'cờ: Kosovo',
    '🇾🇪' => 'cờ: Yemen',
    '🇾🇹' => 'cờ: Mayotte',
    '🇿🇦' => 'cờ: Nam Phi',
    '🇿🇲' => 'cờ: Zambia',
    '🇿🇼' => 'cờ: Zimbabwe',
    '🔟' => 'mũ phím: 10',
    '🏻' => 'màu da sáng',
    '🏼' => 'màu da sáng trung bình',
    '🏽' => 'màu da trung bình',
    '🏾' => 'màu da tối trung bình',
    '🏿' => 'màu da tối',
    '😀' => 'mặt cười toe toét',
    '😃' => 'mặt cười miệng há với mắt to',
    '😄' => 'mặt cười miệng há mắt cười',
    '😁' => 'mặt cười nhe răng và mắt cười',
    '😆' => 'mặt cười miệng há mắt nhắm chặt',
    '😅' => 'mặt cười toát mồ hôi',
    '🤣' => 'mặt cười lăn cười bò',
    '😂' => 'mặt cười với nước mắt vui sướng',
    '🙂' => 'mặt cười mỉm',
    '🙃' => 'mặt lộn ngược',
    '🫠' => 'mặt tan chảy',
    '😉' => 'mặt nháy mắt',
    '😊' => 'mặt cười híp mắt',
    '😇' => 'mặt cười có hào quang',
    '🥰' => 'mặt cười với 3 trái tim',
    '😍' => 'mặt cười mắt hình trái tim',
    '🤩' => 'ngưỡng mộ',
    '😘' => 'mặt đang hôn gió',
    '😗' => 'mặt hôn',
    '☺' => 'mặt cười',
    '😚' => 'mặt hôn mắt nhắm',
    '😙' => 'mặt hôn mắt cười',
    '🥲' => 'mặt cười với nước mắt',
    '😋' => 'mặt thưởng thức món ngon',
    '😛' => 'mặt lè lưỡi',
    '😜' => 'mặt lè lưỡi nháy mắt',
    '🤪' => 'mặt làm trò hề',
    '😝' => 'mặt thè lưỡi mắt nhắm chặt',
    '🤑' => 'mặt có tiền ở miệng',
    '🤗' => 'mặt ôm',
    '🤭' => 'mặt với tay che miệng',
    '🫢' => 'mặt với cặp mắt mở to và tay che miệng',
    '🫣' => 'mặt hé mắt nhìn trộm',
    '🤫' => 'mặt ra dấu suỵt',
    '🤔' => 'mặt suy nghĩ',
    '🫡' => 'mặt chào',
    '🤐' => 'mặt có miệng bị kéo khóa',
    '🤨' => 'mặt với lông mày rướn lên',
    '😐' => 'mặt trung lập',
    '😑' => 'mặt vô cảm',
    '😶' => 'mặt không có miệng',
    '🫥' => 'mặt có đường viền chấm chấm',
    '😏' => 'mặt cười khẩy',
    '😒' => 'mặt buồn',
    '🙄' => 'mặt có mắt đu đưa',
    '😬' => 'mặt nhăn nhó',
    '🤥' => 'mặt nói dối',
    '🫨' => 'mặt lắc',
    '😌' => 'mặt nhẹ nhõm',
    '😔' => 'mặt trầm ngâm',
    '😪' => 'mặt buồn ngủ',
    '🤤' => 'mặt chảy dãi',
    '😴' => 'mặt đang ngủ',
    '🫩' => 'mặt có túi mắt',
    '😷' => 'mặt đeo khẩu trang y tế',
    '🤒' => 'mặt có miệng ngậm nhiệt kế',
    '🤕' => 'mặt đeo băng đầu',
    '🤢' => 'mặt buồn nôn',
    '🤮' => 'mặt nôn mửa',
    '🤧' => 'mặt hắt hơi',
    '🥵' => 'mặt nóng',
    '🥶' => 'mặt lạnh',
    '🥴' => 'mặt choáng váng',
    '😵' => 'mặt chóng mặt',
    '🤯' => 'đầu nổ tung',
    '🤠' => 'mặt đội mũ cao bồi',
    '🥳' => 'mặt tiệc tùng',
    '🥸' => 'mặt trá hình',
    '😎' => 'mặt cười đeo kính',
    '🤓' => 'mặt mọt sách',
    '🧐' => 'mặt với kính một mắt',
    '😕' => 'mặt bối rối',
    '🫤' => 'mặt có miệng xiên',
    '😟' => 'mặt lo lắng',
    '🙁' => 'mặt hơi cau mày',
    '☹' => 'mặt cau mày',
    '😮' => 'mặt có miệng há',
    '😯' => 'mặt làm thinh',
    '😲' => 'mặt kinh ngạc',
    '😳' => 'mặt đỏ ửng',
    '🫪' => 'mặt méo',
    '🥺' => 'mặt cầu xin',
    '🥹' => 'mặt kìm nén nước mắt',
    '😦' => 'mặt cau miệng há',
    '😧' => 'mặt đau khổ',
    '😨' => 'mặt sợ hãi',
    '😰' => 'mặt lo lắng và toát mồ hôi',
    '😥' => 'mặt thất vọng nhưng nhẹ nhõm',
    '😢' => 'mặt khóc',
    '😭' => 'mặt khóc to',
    '😱' => 'mặt la hét kinh hãi',
    '😖' => 'mặt xấu hổ',
    '😣' => 'mặt kiên nhẫn',
    '😞' => 'mặt thất vọng',
    '😓' => 'mặt chán nản với mồ hôi',
    '😩' => 'mặt kiệt sức',
    '😫' => 'mặt mệt mỏi',
    '🥱' => 'mặt đang ngáp',
    '😤' => 'mặt có mũi đang phì hơi',
    '😡' => 'mặt hờn dỗi',
    '😠' => 'mặt giận giữ',
    '🤬' => 'mặt có các ký hiệu trên miệng',
    '😈' => 'mặt cười có sừng',
    '👿' => 'mặt giận giữ có sừng',
    '💀' => 'đầu lâu',
    '☠' => 'đầu lâu xương chéo',
    '💩' => 'đống phân',
    '🤡' => 'mặt hề',
    '👹' => 'mặt quỷ',
    '👺' => 'yêu tinh',
    '👻' => 'ma',
    '👽' => 'người ngoài hành tinh',
    '👾' => 'quái vật ngoài hành tinh',
    '🤖' => 'mặt rô-bốt',
    '😺' => 'mặt mèo cười tươi',
    '😸' => 'mặt mèo cười tươi với hai mắt híp lại',
    '😹' => 'mặt mèo có nước mắt vui sướng',
    '😻' => 'mặt mèo cười mắt hình trái tim',
    '😼' => 'mặt mèo cười gượng',
    '😽' => 'mặt mèo hôn mắt nhắm',
    '🙀' => 'mặt mèo mệt lử',
    '😿' => 'mặt mèo đang khóc',
    '😾' => 'mặt mèo hờn dỗi',
    '🙈' => 'khỉ không nhìn điều xấu',
    '🙉' => 'khỉ không nghe điều xấu',
    '🙊' => 'khỉ không nói điều xấu',
    '💌' => 'thư tình',
    '💘' => 'trái tim với mũi tên',
    '💝' => 'trái tim với ruy băng',
    '💖' => 'trái tim lấp lánh',
    '💗' => 'trái tim lớn dần',
    '💓' => 'trái tim đang đập',
    '💞' => 'trái tim xoay vòng',
    '💕' => 'hai trái tim',
    '💟' => 'hình trang trí trái tim',
    '❣' => 'dấu chấm than hình trái tim đậm',
    '💔' => 'trái tim tan vỡ',
    '❤' => 'trái tim màu đỏ',
    '🩷' => 'trái tim màu hồng',
    '🧡' => 'trái tim màu cam',
    '💛' => 'trái tim màu vàng',
    '💚' => 'trái tim màu lục',
    '💙' => 'trái tim màu lam',
    '🩵' => 'trái tim màu xanh dương nhạt',
    '💜' => 'trái tim màu tím',
    '🤎' => 'trái tim màu nâu',
    '🖤' => 'trái tim màu đen',
    '🩶' => 'trái tim màu xám',
    '🤍' => 'trái tim màu trắng',
    '💋' => 'dấu nụ hôn',
    '💯' => '100 điểm',
    '💢' => 'biểu tượng giận giữ',
    '🫯' => 'đám mây chiến đấu',
    '💥' => 'va chạm',
    '💫' => 'choáng váng',
    '💦' => 'giọt mồ hôi',
    '💨' => 'lao đi',
    '🕳' => 'lỗ',
    '💬' => 'bong bóng lời nói',
    '🗨' => 'bong bóng lời nói trái',
    '🗯' => 'bong bóng góc phải',
    '💭' => 'bong bóng suy nghĩ',
    '💤' => 'buồn ngủ',
    '👋' => 'vẫy tay',
    '🤚' => 'mu bàn tay giơ lên',
    '🖐' => 'tay xòe ngón giơ lên',
    '✋' => 'bàn tay giơ lên',
    '🖖' => 'tay thần lửa',
    '🫱' => 'bàn tay hướng sang bên phải',
    '🫲' => 'bàn tay hướng sang bên trái',
    '🫳' => 'bàn tay có lòng bàn tay hướng xuống',
    '🫴' => 'bàn tay có lòng bàn tay hướng lên',
    '🫷' => 'bàn tay đẩy sang trái',
    '🫸' => 'bàn tay đẩy sang phải',
    '👌' => 'bàn tay làm dấu OK',
    '🤌' => 'ngón tay khum lại',
    '🤏' => 'tay chụm',
    '✌' => 'tay chiến thắng',
    '🤞' => 'hai ngón tay bắt chéo',
    '🫰' => 'bàn tay với ngón trỏ và ngón cái bắt chéo',
    '🤟' => 'cử chỉ yêu nhau',
    '🤘' => 'ký hiệu cặp sừng',
    '🤙' => 'bàn tay gọi cho tôi',
    '👈' => 'ngón trỏ trái chỉ sang trái',
    '👉' => 'ngón trỏ trái chỉ sang phải',
    '👆' => 'ngón trỏ trái chỉ lên',
    '🖕' => 'ngón giữa',
    '👇' => 'ngón trỏ trái chỉ xuống',
    '☝' => 'ngón trỏ chỉ lên trên',
    '🫵' => 'ngón trỏ chỉ vào người xem',
    '👍' => 'dấu ra hiệu đồng ý',
    '👎' => 'dấu ra hiệu từ chối',
    '✊' => 'nắm đấm giơ lên',
    '👊' => 'nắm đấm',
    '🤛' => 'nắm đấm hướng phía trái',
    '🤜' => 'nắm đấm hướng sang phải',
    '👏' => 'vỗ tay',
    '🙌' => 'hai bàn tay giơ lên',
    '🫶' => 'hai bàn tay tạo hình trái tim',
    '👐' => 'hai bàn tay đang xòe',
    '🤲' => 'hai bàn tay chạm vào nhau',
    '🤝' => 'bắt tay',
    '🙏' => 'chắp tay',
    '✍' => 'bàn tay đang viết',
    '💅' => 'sơn móng tay',
    '🤳' => 'tự sướng',
    '💪' => 'bắp tay gập lại',
    '🦾' => 'cánh tay giả',
    '🦿' => 'chân giả',
    '🦵' => 'Chân',
    '🦶' => 'bàn chân',
    '👂' => 'tai',
    '🦻' => 'tai đeo thiết bị trợ thính',
    '👃' => 'mũi',
    '🧠' => 'não',
    '🫀' => 'cơ quan tim',
    '🫁' => 'phổi',
    '🦷' => 'răng',
    '🦴' => 'xương',
    '👀' => 'đôi mắt',
    '👁' => 'mắt',
    '👅' => 'lưỡi',
    '👄' => 'miệng',
    '🫦' => 'cắn môi',
    '👶' => 'trẻ con',
    '🧒' => 'trẻ em',
    '👦' => 'con trai',
    '👧' => 'con gái',
    '🧑' => 'người',
    '👱' => 'người tóc vàng hoe',
    '👨' => 'đàn ông',
    '🧔' => 'người có râu',
    '👩' => 'phụ nữ',
    '🧓' => 'người lớn tuổi',
    '👴' => 'cụ ông',
    '👵' => 'cụ bà',
    '🙍' => 'người đang cau mày',
    '🙎' => 'người đang bĩu môi',
    '🙅' => 'người ra hiệu phản đối',
    '🙆' => 'người ra hiệu đồng ý',
    '💁' => 'người ra hiệu trợ giúp',
    '🙋' => 'người giơ cao tay',
    '🧏' => 'người khiếm thính',
    '🙇' => 'người đang cúi đầu',
    '🤦' => 'người lấy tay che mặt',
    '🤷' => 'người nhún vai',
    '👮' => 'cảnh sát',
    '🕵' => 'thám tử',
    '💂' => 'bảo vệ',
    '🥷' => 'ninja',
    '👷' => 'công nhân xây dựng',
    '🫅' => 'người đội vương miện',
    '🤴' => 'hoàng tử',
    '👸' => 'công chúa',
    '👳' => 'người đội khăn xếp',
    '👲' => 'người đàn ông đội mũ Trung Quốc',
    '🧕' => 'người phụ nữ quàng khăn trùm đầu',
    '🤵' => 'người mặc bộ vest',
    '👰' => 'người đội khăn voan',
    '🤰' => 'người phụ nữ có thai',
    '🫃' => 'người đàn ông mang bầu',
    '🫄' => 'người mang bầu',
    '🤱' => 'cho con bú',
    '👼' => 'thiên thần nhỏ',
    '🎅' => 'Ông già Noel',
    '🤶' => 'mẹ giáng sinh',
    '🦸' => 'siêu anh hùng',
    '🦹' => 'nhân vật siêu phản diện',
    '🧙' => 'pháp sư',
    '🧚' => 'tiên',
    '🧛' => 'ma cà rồng',
    '🧜' => 'người cá',
    '🧝' => 'yêu ma',
    '🧞' => 'thần',
    '🧟' => 'xác chết sống lại',
    '🧌' => 'người khổng lồ độc ác',
    '🫈' => 'sinh vật nhiều lông',
    '💆' => 'người được xoa bóp mặt',
    '💇' => 'người được cắt tóc',
    '🚶' => 'người đi bộ',
    '🧍' => 'người đứng',
    '🧎' => 'người quỳ',
    '🏃' => 'người chạy',
    '💃' => 'người phụ nữ đang khiêu vũ',
    '🕺' => 'người đàn ông đang khiêu vũ',
    '🕴' => 'người đàn ông mặc vest đang bay lên',
    '👯' => 'những người đeo tai thỏ',
    '🧖' => 'người ở trong phòng xông hơi',
    '🧗' => 'người đang leo núi',
    '🤺' => 'người đấu kiếm',
    '🏇' => 'đua ngựa',
    '⛷' => 'người trượt tuyết',
    '🏂' => 'người trượt ván tuyết',
    '🏌' => 'người chơi gôn',
    '🏄' => 'người lướt sóng',
    '🚣' => 'người chèo thuyền',
    '🏊' => 'người bơi',
    '⛹' => 'người chơi bóng',
    '🏋' => 'người cử tạ',
    '🚴' => 'người đi xe đạp',
    '🚵' => 'người đi xe đạp leo núi',
    '🤸' => 'người nhào lộn',
    '🤼' => 'người chơi vật',
    '🤽' => 'người chơi bóng nước',
    '🤾' => 'người chơi bóng ném',
    '🤹' => 'người tung hứng',
    '🧘' => 'người ngồi kiết già',
    '🛀' => 'người đang tắm bồn',
    '🛌' => 'người trên giường',
    '👭' => 'hai người phụ nữ nắm tay',
    '👫' => 'người đàn ông và phụ nữ nắm tay',
    '👬' => 'hai người đàn ông nắm tay',
    '💏' => 'nụ hôn',
    '💑' => 'cặp đôi với trái tim',
    '🗣' => 'đầu người đang nói',
    '👤' => 'bóng tượng bán thân',
    '👥' => 'các bóng tượng bán thân',
    '🫂' => 'hai người ôm nhau',
    '👪' => 'gia đình',
    '👣' => 'dấu chân',
    '🫆' => 'dấu vân tay',
    '🦰' => 'tóc đỏ',
    '🦱' => 'tóc xoăn',
    '🦳' => 'tóc trắng',
    '🦲' => 'hói',
    '🐵' => 'mặt khỉ',
    '🐒' => 'khỉ',
    '🦍' => 'khỉ đột',
    '🦧' => 'đười ươi',
    '🐶' => 'mặt cún',
    '🐕' => 'chó',
    '🦮' => 'chó dẫn đường',
    '🐩' => 'chó xù',
    '🐺' => 'chó sói',
    '🦊' => 'cáo',
    '🦝' => 'gấu trúc',
    '🐱' => 'mặt mèo',
    '🐈' => 'mèo',
    '🦁' => 'sư tử',
    '🐯' => 'mặt hổ',
    '🐅' => 'hổ',
    '🐆' => 'báo hoa mai',
    '🐴' => 'mặt ngựa',
    '🫎' => 'nai sừng tấm Bắc Mỹ',
    '🫏' => 'con lừa',
    '🐎' => 'ngựa',
    '🦄' => 'kỳ lân',
    '🦓' => 'ngựa vằn',
    '🦌' => 'hươu',
    '🦬' => 'bò rừng',
    '🐮' => 'mặt bò',
    '🐂' => 'bò đực',
    '🐃' => 'trâu nước',
    '🐄' => 'bò cái',
    '🐷' => 'mặt lợn',
    '🐖' => 'lợn',
    '🐗' => 'lợn rừng',
    '🐽' => 'mũi lợn',
    '🐏' => 'cừu đực',
    '🐑' => 'cừu cái',
    '🐐' => 'dê',
    '🐪' => 'lạc đà',
    '🐫' => 'lạc đà hai bướu',
    '🦙' => 'lạc đà không bướu',
    '🦒' => 'hươu cao cổ',
    '🐘' => 'voi',
    '🦣' => 'voi ma mút',
    '🦏' => 'tê giác',
    '🦛' => 'hà mã',
    '🐭' => 'mặt chuột',
    '🐁' => 'chuột',
    '🐀' => 'chuột cống',
    '🐹' => 'mặt hamster',
    '🐰' => 'mặt thỏ',
    '🐇' => 'thỏ',
    '🐿' => 'sóc chuột',
    '🦫' => 'hải ly',
    '🦔' => 'nhím',
    '🦇' => 'dơi',
    '🐻' => 'gấu',
    '🐨' => 'gấu túi',
    '🐼' => 'mặt gấu trúc',
    '🦥' => 'con lười',
    '🦦' => 'rái cá',
    '🦨' => 'chồn hôi Mỹ',
    '🦘' => 'chuột túi',
    '🦡' => 'con lửng',
    '🐾' => 'dấu chân móng vuốt',
    '🦃' => 'gà tây',
    '🐔' => 'gà',
    '🐓' => 'gà trống',
    '🐣' => 'gà mới nở',
    '🐤' => 'gà con',
    '🐥' => 'mặt trước gà con',
    '🐦' => 'chim',
    '🐧' => 'chim cánh cụt',
    '🕊' => 'bồ câu',
    '🦅' => 'đại bàng',
    '🦆' => 'vịt',
    '🦢' => 'thiên nga',
    '🦉' => 'cú',
    '🦤' => 'chim cưu',
    '🪶' => 'lông chim',
    '🦩' => 'hồng hạc',
    '🦚' => 'con công',
    '🦜' => 'con vẹt',
    '🪽' => 'cánh',
    '🪿' => 'con ngỗng',
    '🐸' => 'ếch',
    '🐊' => 'cá sấu',
    '🐢' => 'rùa',
    '🦎' => 'thằn lằn',
    '🐍' => 'rắn',
    '🐲' => 'mặt rồng',
    '🐉' => 'rồng',
    '🦕' => 'khủng long chân thằn lằn',
    '🦖' => 'khủng long bạo chúa',
    '🐳' => 'cá voi đang phun nước',
    '🐋' => 'cá voi',
    '🐬' => 'cá heo',
    '🫍' => 'cá voi sát thủ',
    '🦭' => 'chó biển',
    '🐟' => 'cá',
    '🐠' => 'cá nhiệt đới',
    '🐡' => 'cá nóc',
    '🦈' => 'cá mập',
    '🐙' => 'bạch tuộc',
    '🐚' => 'vỏ xoắn ốc',
    '🪸' => 'san hô',
    '🪼' => 'sứa',
    '🦀' => 'cua',
    '🦞' => 'tôm hùm',
    '🦐' => 'tôm',
    '🦑' => 'mực ống',
    '🦪' => 'hàu',
    '🐌' => 'ốc sên',
    '🦋' => 'bướm',
    '🐛' => 'con bọ',
    '🐜' => 'kiến',
    '🐝' => 'ong mật',
    '🪲' => 'bọ cánh cứng',
    '🐞' => 'bọ rùa',
    '🦗' => 'dế',
    '🪳' => 'con gián',
    '🕷' => 'nhện',
    '🕸' => 'mạng nhện',
    '🦂' => 'bọ cạp',
    '🦟' => 'con muỗi',
    '🪰' => 'con ruồi',
    '🪱' => 'giun',
    '🦠' => 'vi trùng',
    '💐' => 'bó hoa',
    '🌸' => 'hoa anh đào',
    '💮' => 'hoa trắng',
    '🪷' => 'hoa sen',
    '🏵' => 'nơ hoa hồng',
    '🌹' => 'hoa hồng',
    '🥀' => 'hoa héo',
    '🌺' => 'hoa dâm bụt',
    '🌻' => 'hoa hướng dương',
    '🌼' => 'hoa',
    '🌷' => 'hoa tulip',
    '🪻' => 'lục bình',
    '🌱' => 'cây non',
    '🪴' => 'cây trồng trong chậu',
    '🌲' => 'cây thường xanh',
    '🌳' => 'cây rụng lá',
    '🌴' => 'cây cọ',
    '🌵' => 'cây xương rồng',
    '🌾' => 'bó lúa',
    '🌿' => 'thảo mộc',
    '☘' => 'cỏ ba lá',
    '🍀' => 'cỏ bốn lá',
    '🍁' => 'lá phong',
    '🍂' => 'lá rụng',
    '🍃' => 'lá rung rinh trong gió',
    '🪹' => 'tổ trống rỗng',
    '🪺' => 'tổ có trứng',
    '🍄' => 'nấm',
    '🪾' => 'cây trụi lá',
    '🍇' => 'chùm nho',
    '🍈' => 'dưa',
    '🍉' => 'dưa hấu',
    '🍊' => 'quýt',
    '🍋' => 'chanh tây',
    '🍌' => 'chuối',
    '🍍' => 'dứa',
    '🥭' => 'xoài',
    '🍎' => 'táo đỏ',
    '🍏' => 'táo xanh',
    '🍐' => 'lê',
    '🍑' => 'đào',
    '🍒' => 'anh đào',
    '🍓' => 'dâu tây',
    '🫐' => 'quả việt quất',
    '🥝' => 'quả kiwi',
    '🍅' => 'cà chua',
    '🫒' => 'ôliu',
    '🥥' => 'dừa',
    '🥑' => 'quả bơ',
    '🍆' => 'cà tím',
    '🥔' => 'khoai tây',
    '🥕' => 'cà rốt',
    '🌽' => 'bắp ngô',
    '🌶' => 'quả ớt',
    '🫑' => 'ớt chuông',
    '🥒' => 'dưa chuột',
    '🥬' => 'xanh lá',
    '🥦' => 'xúp lơ xanh',
    '🧄' => 'tỏi',
    '🧅' => 'hành',
    '🥜' => 'đậu phộng',
    '🫘' => 'hạt đậu',
    '🌰' => 'hạt dẻ',
    '🫚' => 'củ gừng',
    '🫛' => 'đậu Hà Lan',
    '🫜' => 'rau củ',
    '🍞' => 'bánh mì',
    '🥐' => 'bánh sừng bò',
    '🥖' => 'bánh mì que',
    '🫓' => 'bánh mỳ cắt lát',
    '🥨' => 'bánh quy xoắn',
    '🥯' => 'bánh mỳ vòng',
    '🥞' => 'bánh kếp',
    '🧇' => 'bánh quế',
    '🧀' => 'miếng pho mát',
    '🍖' => 'thịt bám xương',
    '🍗' => 'đùi gia cầm',
    '🥩' => 'tảng thịt',
    '🥓' => 'thịt xông khói',
    '🍔' => 'bánh hamburger',
    '🍟' => 'khoai tây chiên',
    '🍕' => 'bánh pizza',
    '🌭' => 'bánh mì xúc xích',
    '🥪' => 'bánh mỳ kẹp',
    '🌮' => 'bánh taco',
    '🌯' => 'bánh burrito',
    '🫔' => 'bánh tamale',
    '🥙' => 'bánh mì kẹp thịt',
    '🧆' => 'bánh rán nhân gà',
    '🥚' => 'trứng',
    '🍳' => 'nấu ăn',
    '🥘' => 'chảo thức ăn nông',
    '🍲' => 'nồi thức ăn',
    '🫕' => 'lẩu pho mai',
    '🥣' => 'bát và thìa',
    '🥗' => 'salad rau xanh',
    '🍿' => 'bỏng ngô',
    '🧈' => 'bơ',
    '🧂' => 'muối',
    '🥫' => 'thực phẩm đóng hộp',
    '🍱' => 'hộp cơm bento',
    '🍘' => 'bánh gạo',
    '🍙' => 'cơm nắm',
    '🍚' => 'cơm',
    '🍛' => 'cơm cà ri',
    '🍜' => 'bát mì',
    '🍝' => 'spaghetti',
    '🍠' => 'khoai lang nướng',
    '🍢' => 'món oden',
    '🍣' => 'sushi',
    '🍤' => 'tôm chiên',
    '🍥' => 'bánh cá có hình xoắn',
    '🥮' => 'bánh trung thu',
    '🍡' => 'bánh trôi Nhật Bản',
    '🥟' => 'bánh bao',
    '🥠' => 'bánh quy may mắn',
    '🥡' => 'hộp đựng đồ ăn mang đi',
    '🍦' => 'kem mềm',
    '🍧' => 'đá bào',
    '🍨' => 'kem',
    '🍩' => 'bánh rán vòng',
    '🍪' => 'bánh quy',
    '🎂' => 'bánh sinh nhật',
    '🍰' => 'bánh ngọt',
    '🧁' => 'bánh nướng nhỏ',
    '🥧' => 'bánh nướng',
    '🍫' => 'thanh sô cô la',
    '🍬' => 'kẹo',
    '🍭' => 'kẹo mút',
    '🍮' => 'bánh trứng',
    '🍯' => 'hũ mật ong',
    '🍼' => 'bình sữa trẻ em',
    '🥛' => 'cốc sữa',
    '☕' => 'đồ uống nóng',
    '🫖' => 'ấm trà',
    '🍵' => 'tách trà không có quai',
    '🍶' => 'rượu sake',
    '🍾' => 'chai rượu với nút gỗ bật ra',
    '🍷' => 'ly rượu vang',
    '🍸' => 'ly cocktail',
    '🍹' => 'đồ uống nhiệt đới',
    '🍺' => 'cốc bia',
    '🍻' => 'chạm cốc bia',
    '🥂' => 'chạm cốc',
    '🥃' => 'ly đáy phẳng',
    '🫗' => 'đổ chất lỏng',
    '🥤' => 'cốc và ống hút',
    '🧋' => 'trà sữa trân châu',
    '🧃' => 'hộp đồ uống',
    '🧉' => 'trà nhựa ruồi',
    '🧊' => 'viên đá',
    '🥢' => 'đũa',
    '🍽' => 'dĩa và dao với đĩa',
    '🍴' => 'dĩa và dao',
    '🥄' => 'thìa',
    '🔪' => 'dao làm bếp',
    '🫙' => 'bình đựng',
    '🏺' => 'vò hai quai',
    '🌍' => 'địa cầu thể hiện Châu Âu-Châu Phi',
    '🌎' => 'địa cầu thể hiện Châu Mỹ',
    '🌏' => 'địa cầu thể hiện Châu Á-Châu Úc',
    '🌐' => 'địa cầu có đường kinh tuyến',
    '🗺' => 'bản đồ thế giới',
    '🗾' => 'bản đồ Nhật Bản',
    '🧭' => 'la bàn',
    '🏔' => 'đỉnh núi phủ tuyết',
    '⛰' => 'núi',
    '🛘' => 'lở đất',
    '🌋' => 'núi lửa',
    '🗻' => 'núi Phú Sĩ',
    '🏕' => 'cắm trại',
    '🏖' => 'bãi biển với chiếc ô',
    '🏜' => 'sa mạc',
    '🏝' => 'đảo hoang',
    '🏞' => 'công viên quốc gia',
    '🏟' => 'sân vận động',
    '🏛' => 'tòa nhà cổ điển',
    '🏗' => 'công trường xây dựng',
    '🧱' => 'gạch',
    '🪨' => 'đá',
    '🪵' => 'gỗ',
    '🛖' => 'túp lều',
    '🏘' => 'những ngôi nhà',
    '🏚' => 'ngôi nhà bỏ hoang',
    '🏠' => 'ngôi nhà',
    '🏡' => 'nhà có vườn',
    '🏢' => 'tòa nhà văn phòng',
    '🏣' => 'bưu điện Nhật Bản',
    '🏤' => 'bưu điện',
    '🏥' => 'bệnh viện',
    '🏦' => 'ngân hàng',
    '🏨' => 'khách sạn',
    '🏩' => 'khách sạn tình yêu',
    '🏪' => 'cửa hàng tiện lợi',
    '🏫' => 'trường học',
    '🏬' => 'cửa hàng bách hóa',
    '🏭' => 'nhà máy',
    '🏯' => 'lâu đài Nhật Bản',
    '🏰' => 'lâu đài',
    '💒' => 'đám cưới',
    '🗼' => 'tháp Tokyo',
    '🗽' => 'Tượng Nữ thần Tự do',
    '⛪' => 'nhà thờ',
    '🕌' => 'nhà thờ hồi giáo',
    '🛕' => 'đền hindu',
    '🕍' => 'giáo đường Do thái',
    '⛩' => 'đền thờ Nhật Bản',
    '🕋' => 'kaaba',
    '⛲' => 'đài phun nước',
    '⛺' => 'lều',
    '🌁' => 'có sương',
    '🌃' => 'đêm có sao',
    '🏙' => 'khung cảnh thành phố',
    '🌄' => 'bình minh trên núi',
    '🌅' => 'bình minh',
    '🌆' => 'khung cảnh thành phố lúc chạng vạng',
    '🌇' => 'hoàng hôn',
    '🌉' => 'cây cầu về đêm',
    '♨' => 'suối nước nóng',
    '🎠' => 'ngựa đu quay',
    '🛝' => 'cầu trượt',
    '🎡' => 'vòng đu quay',
    '🎢' => 'tàu lượn siêu tốc',
    '💈' => 'biển hiệu của thợ cắt tóc',
    '🎪' => 'lều rạp xiếc',
    '🚂' => 'đầu máy xe lửa',
    '🚃' => 'toa tàu',
    '🚄' => 'tàu cao tốc',
    '🚅' => 'tàu viên đạn',
    '🚆' => 'tàu hỏa',
    '🚇' => 'tàu điện ngầm',
    '🚈' => 'tàu điện nhẹ',
    '🚉' => 'ga tàu',
    '🚊' => 'xe điện',
    '🚝' => 'đường một ray',
    '🚞' => 'đường ray leo núi',
    '🚋' => 'tàu điện',
    '🚌' => 'xe buýt',
    '🚍' => 'xe buýt đang tới',
    '🚎' => 'ô tô điện',
    '🚐' => 'xe buýt nhỏ',
    '🚑' => 'xe cứu thương',
    '🚒' => 'xe cứu hỏa',
    '🚓' => 'xe cảnh sát',
    '🚔' => 'xe cảnh sát đang tới',
    '🚕' => 'taxi',
    '🚖' => 'taxi đang tới',
    '🚗' => 'ô tô',
    '🚘' => 'ô tô đang tới',
    '🚙' => 'xe thể thao đa dụng',
    '🛻' => 'xe bán tải',
    '🚚' => 'xe tải giao hàng',
    '🚛' => 'đầu kéo rơ-moóc',
    '🚜' => 'máy kéo',
    '🏎' => 'xe đua',
    '🏍' => 'xe máy',
    '🛵' => 'xe tay ga',
    '🦽' => 'xe lăn tay',
    '🦼' => 'xe lăn tự động',
    '🛺' => 'xe lam',
    '🚲' => 'xe đạp',
    '🛴' => 'xe hẩy',
    '🛹' => 'ván trượt',
    '🛼' => 'giày trượt patin',
    '🚏' => 'điểm dừng xe buýt',
    '🛣' => 'đường cao tốc',
    '🛤' => 'đường ray',
    '🛢' => 'thùng dầu',
    '⛽' => 'bơm nhiên liệu',
    '🛞' => 'bánh xe',
    '🚨' => 'đèn xe cảnh sát',
    '🚥' => 'đèn giao thông ngang',
    '🚦' => 'đèn giao thông dọc',
    '🛑' => 'ký hiệu dừng',
    '🚧' => 'công trường',
    '⚓' => 'mỏ neo',
    '🛟' => 'phao cứu sinh',
    '⛵' => 'thuyền buồm',
    '🛶' => 'xuồng',
    '🚤' => 'xuồng cao tốc',
    '🛳' => 'tàu chở khách',
    '⛴' => 'phà',
    '🛥' => 'thuyền máy',
    '🚢' => 'tàu thủy',
    '✈' => 'máy bay',
    '🛩' => 'máy bay nhỏ',
    '🛫' => 'máy bay khởi hành',
    '🛬' => 'máy bay đến nơi',
    '🪂' => 'dù',
    '💺' => 'cái ghế',
    '🚁' => 'máy bay trực thăng',
    '🚟' => 'đường sắt trên cao',
    '🚠' => 'cáp treo trên núi',
    '🚡' => 'cáp treo',
    '🛰' => 'vệ tinh',
    '🚀' => 'tên lửa',
    '🛸' => 'đĩa bay',
    '🛎' => 'chuông gọi phục vụ',
    '🧳' => 'hành lý',
    '⌛' => 'đồng hồ cát đã chảy hết',
    '⏳' => 'đồng hồ cát đang chảy',
    '⌚' => 'đồng hồ đeo tay',
    '⏰' => 'đồng hồ báo thức',
    '⏱' => 'đồng hồ bấm giờ',
    '⏲' => 'đồng hồ hẹn giờ',
    '🕰' => 'đồng hồ để bàn',
    '🕛' => 'mười hai giờ',
    '🕧' => 'mười hai giờ ba mươi phút',
    '🕐' => 'một giờ',
    '🕜' => 'một giờ ba mươi phút',
    '🕑' => 'hai giờ',
    '🕝' => 'hai giờ ba mươi phút',
    '🕒' => 'ba giờ',
    '🕞' => 'ba giờ ba mươi phút',
    '🕓' => 'bốn giờ',
    '🕟' => 'bốn giờ ba mươi phút',
    '🕔' => 'năm giờ',
    '🕠' => 'năm giờ ba mươi phút',
    '🕕' => 'sáu giờ',
    '🕡' => 'sáu giờ ba mươi phút',
    '🕖' => 'bảy giờ',
    '🕢' => 'bảy giờ ba mươi phút',
    '🕗' => 'tám giờ',
    '🕣' => 'tám giờ ba mươi phút',
    '🕘' => 'chín giờ',
    '🕤' => 'chín giờ ba mươi phút',
    '🕙' => 'mười giờ',
    '🕥' => 'mười giờ ba mươi phút',
    '🕚' => 'mười một giờ',
    '🕦' => 'mười một giờ ba mươi phút',
    '🌑' => 'trăng non',
    '🌒' => 'trăng lưỡi liềm đầu tháng',
    '🌓' => 'trăng thượng huyền',
    '🌔' => 'trăng khuyết',
    '🌕' => 'trăng tròn',
    '🌖' => 'trăng khuyết cuối tháng',
    '🌗' => 'trăng hạ huyền',
    '🌘' => 'trăng lưỡi liềm cuối tháng',
    '🌙' => 'trăng lưỡi liềm',
    '🌚' => 'mặt trăng non',
    '🌛' => 'trăng thượng huyền hình mặt người',
    '🌜' => 'trăng hạ huyền hình mặt người',
    '🌡' => 'nhiệt kế',
    '☀' => 'mặt trời',
    '🌝' => 'khuôn mặt trăng tròn',
    '🌞' => 'mặt trời có hình mặt người',
    '🪐' => 'hành tinh có vành đai bao quanh',
    '⭐' => 'ngôi sao',
    '🌟' => 'ngôi sao lấp lánh',
    '🌠' => 'sao băng',
    '🌌' => 'dải ngân hà',
    '☁' => 'đám mây',
    '⛅' => 'mặt trời sau đám mây',
    '⛈' => 'đám mây kèm chớp và mưa',
    '🌤' => 'mặt trời sau đám mây nhỏ',
    '🌥' => 'mặt trời sau đám mây lớn',
    '🌦' => 'mặt trời sau đám mây mưa',
    '🌧' => 'mây và mưa',
    '🌨' => 'đám mây với tuyết',
    '🌩' => 'đám mây với tia chớp',
    '🌪' => 'lốc xoáy',
    '🌫' => 'sương mù',
    '🌬' => 'khuôn mặt gió',
    '🌀' => 'hình lốc xoáy',
    '🌈' => 'cầu vồng',
    '🌂' => 'ô đã đóng',
    '☂' => 'cái ô',
    '☔' => 'cái ô với những giọt nước mưa',
    '⛱' => 'cái ô trên mặt đất',
    '⚡' => 'điện cao thế',
    '❄' => 'bông tuyết',
    '☃' => 'người tuyết',
    '⛄' => 'người tuyết không có tuyết',
    '☄' => 'sao chổi',
    '🔥' => 'lửa',
    '💧' => 'giọt nước',
    '🌊' => 'sóng nước',
    '🎃' => 'đèn lồng bí ngô',
    '🎄' => 'cây thông noel',
    '🎆' => 'pháo hoa',
    '🎇' => 'pháo bông',
    '🧨' => 'pháo',
    '✨' => 'ánh lấp lánh',
    '🎈' => 'bóng bay',
    '🎉' => 'pháo giấy buổi tiệc',
    '🎊' => 'bóng hoa giấy',
    '🎋' => 'cây tanabata',
    '🎍' => 'trang trí cây thông',
    '🎎' => 'búp bê Nhật Bản',
    '🎏' => 'cờ cá chép',
    '🎐' => 'chuông gió',
    '🎑' => 'lễ ngắm trăng',
    '🧧' => 'bao lì xì',
    '🎀' => 'ruy băng',
    '🎁' => 'gói quà',
    '🎗' => 'ruy băng nhắc nhở',
    '🎟' => 'vé vào cửa',
    '🎫' => 'vé',
    '🎖' => 'huân chương quân đội',
    '🏆' => 'cúp',
    '🏅' => 'huy chương thể thao',
    '🥇' => 'huy chương vàng',
    '🥈' => 'huy chương bạc',
    '🥉' => 'huy chương đồng',
    '⚽' => 'bóng đá',
    '⚾' => 'bóng chày',
    '🥎' => 'bóng mềm',
    '🏀' => 'bóng rổ',
    '🏐' => 'bóng chuyền',
    '🏈' => 'bóng bầu dục Mỹ',
    '🏉' => 'bóng bầu dục',
    '🎾' => 'quần vợt',
    '🥏' => 'đĩa bay trò chơi',
    '🎳' => 'bowling',
    '🏏' => 'trò chơi cricket',
    '🏑' => 'khúc côn cầu trên cỏ',
    '🏒' => 'khúc côn cầu trên băng',
    '🥍' => 'bóng vợt',
    '🏓' => 'bóng bàn',
    '🏸' => 'cầu lông',
    '🥊' => 'găng tay quyền anh',
    '🥋' => 'đồng phục võ thuật',
    '🥅' => 'khung thành',
    '⛳' => 'cờ trong lỗ',
    '⛸' => 'giày trượt băng',
    '🎣' => 'cần câu cá',
    '🤿' => 'mặt nạ lặn',
    '🎽' => 'áo chạy bộ',
    '🎿' => 'ván trượt tuyết',
    '🛷' => 'xe trượt tuyết',
    '🥌' => 'bi đá trên băng',
    '🎯' => 'trúng đích',
    '🪀' => 'yo-yo',
    '🪁' => 'diều',
    '🔫' => 'súng nước',
    '🎱' => 'bi-a 8 bóng',
    '🔮' => 'quả cầu pha lê',
    '🪄' => 'gậy thần',
    '🎮' => 'trò chơi điện tử',
    '🕹' => 'cần điều khiển',
    '🎰' => 'máy giật xèng',
    '🎲' => 'trò xúc xắc',
    '🧩' => 'ghép hình',
    '🧸' => 'gấu bông',
    '🪅' => 'piñata',
    '🪩' => 'quả cầu gương',
    '🪆' => 'búp bê matryoshka',
    '♠' => 'bộ bích',
    '♥' => 'bộ cơ',
    '♦' => 'bộ rô',
    '♣' => 'bộ nhép',
    '♟' => 'quân tốt',
    '🃏' => 'quân phăng teo',
    '🀄' => 'mạt chược rồng đỏ',
    '🎴' => 'quân bài hình hoa',
    '🎭' => 'nghệ thuật biểu diễn',
    '🖼' => 'tranh trong khung',
    '🎨' => 'bảng màu',
    '🧵' => 'sợi chỉ',
    '🪡' => 'kim may',
    '🧶' => 'sợi',
    '🪢' => 'nút thắt',
    '👓' => 'kính mắt',
    '🕶' => 'kính râm',
    '🥽' => 'kính bảo hộ',
    '🥼' => 'áo phòng thí nghiệm',
    '🦺' => 'áo bảo hộ',
    '👔' => 'cà vạt',
    '👕' => 'áo phông',
    '👖' => 'quần bò',
    '🧣' => 'khăn quàng',
    '🧤' => 'găng tay',
    '🧥' => 'áo khoác',
    '🧦' => 'tất',
    '👗' => 'váy',
    '👘' => 'kimono',
    '🥻' => 'xari',
    '🩱' => 'áo tắm một mảnh',
    '🩲' => 'quần lót',
    '🩳' => 'quần soóc',
    '👙' => 'áo tắm hai mảnh',
    '👚' => 'quần áo nữ',
    '🪭' => 'quạt giấy',
    '👛' => 'ví',
    '👜' => 'túi xách',
    '👝' => 'túi cắp nách',
    '🛍' => 'túi mua sắm',
    '🎒' => 'ba lô đi học',
    '🩴' => 'dép xỏ ngón',
    '👞' => 'giày nam',
    '👟' => 'giày chạy',
    '🥾' => 'giày leo núi',
    '🥿' => 'giày bệt',
    '👠' => 'giày cao gót',
    '👡' => 'xăng đan nữ',
    '🩰' => 'giày múa balê',
    '👢' => 'giày bốt nữ',
    '🪮' => 'lược Afro Pick',
    '👑' => 'vương miện',
    '👒' => 'mũ phụ nữ',
    '🎩' => 'mũ chóp cao',
    '🎓' => 'mũ tốt nghiệp',
    '🧢' => 'mũ lưỡi trai',
    '🪖' => 'mũ sắt quân đội',
    '⛑' => 'mũ bảo hiểm của lính cứu hộ',
    '📿' => 'tràng hạt',
    '💄' => 'son môi',
    '💍' => 'nhẫn',
    '💎' => 'đá quý',
    '🔇' => 'tắt loa',
    '🔈' => 'âm lượng loa thấp',
    '🔉' => 'âm lượng loa trung bình',
    '🔊' => 'âm lượng loa cao',
    '📢' => 'loa phát thanh',
    '📣' => 'loa phóng thanh',
    '📯' => 'kèn bưu chính',
    '🔔' => 'cái chuông',
    '🔕' => 'chuông với dấu gạch chéo',
    '🎼' => 'khuông nhạc',
    '🎵' => 'nốt nhạc',
    '🎶' => 'các nốt nhạc',
    '🎙' => 'micrô phòng thu âm',
    '🎚' => 'thanh trượt mức',
    '🎛' => 'núm điều khiển',
    '🎤' => 'micrô',
    '🎧' => 'tai nghe',
    '📻' => 'radio',
    '🎷' => 'kèn saxophone',
    '🎺' => 'kèn trumpet',
    '🪊' => 'kèn trombone',
    '🪗' => 'phong cầm',
    '🎸' => 'đàn ghi-ta',
    '🎹' => 'phím đàn',
    '🎻' => 'đàn violin',
    '🪕' => 'đàn banjo',
    '🥁' => 'trống',
    '🪘' => 'trống dài',
    '🪇' => 'maraca',
    '🪈' => 'sáo',
    '🪉' => 'đàn hạc',
    '📱' => 'điện thoại di động',
    '📲' => 'điện thoại di động có mũi tên',
    '☎' => 'điện thoại bàn',
    '📞' => 'ống nghe điện thoại bàn',
    '📟' => 'máy nhắn tin',
    '📠' => 'máy fax',
    '🔋' => 'pin',
    '🪫' => 'pin yếu',
    '🔌' => 'phích cắm điện',
    '💻' => 'máy tính xách tay',
    '🖥' => 'máy tính để bàn',
    '🖨' => 'máy in',
    '⌨' => 'bàn phím',
    '🖱' => 'chuột máy tính',
    '🖲' => 'bi xoay',
    '💽' => 'đĩa máy tính',
    '💾' => 'đĩa mềm',
    '💿' => 'đĩa quang',
    '📀' => 'dvd',
    '🧮' => 'bàn tính',
    '🎥' => 'máy quay phim',
    '🎞' => 'khung hình phim',
    '📽' => 'máy chiếu phim',
    '🎬' => 'bảng clapper',
    '📺' => 'tivi',
    '📷' => 'máy ảnh',
    '📸' => 'máy ảnh có đèn flash',
    '📹' => 'máy quay video',
    '📼' => 'băng video',
    '🔍' => 'kính phóng đại nghiêng sang trái',
    '🔎' => 'kính lúp nghiêng sang phải',
    '🕯' => 'nến',
    '💡' => 'bóng đèn',
    '🔦' => 'đèn pin',
    '🏮' => 'đèn lồng giấy màu đỏ',
    '🪔' => 'đèn diya',
    '📔' => 'sổ ghi chép có bìa trang trí',
    '📕' => 'sách đóng',
    '📖' => 'sách đang mở',
    '📗' => 'sách màu lục',
    '📘' => 'sách màu lam',
    '📙' => 'sách màu cam',
    '📚' => 'sách',
    '📓' => 'sổ ghi chép',
    '📒' => 'sổ cái',
    '📃' => 'trang bị quăn',
    '📜' => 'cuộn sách',
    '📄' => 'trang hướng lên trên',
    '📰' => 'báo',
    '🗞' => 'tờ báo được cuộn tròn',
    '📑' => 'tab dấu trang',
    '🔖' => 'đánh dấu trang',
    '🏷' => 'nhãn',
    '🪙' => 'đồng tiền',
    '💰' => 'túi tiền',
    '🪎' => 'rương báu',
    '💴' => 'tiền giấy yên',
    '💵' => 'tiền giấy đô la',
    '💶' => 'tiền giấy euro',
    '💷' => 'tiền giấy bảng',
    '💸' => 'tiền có cánh',
    '💳' => 'thẻ tín dụng',
    '🧾' => 'biên nhận',
    '💹' => 'biểu đồ đồng yên tăng',
    '✉' => 'phong bì',
    '📧' => 'email',
    '📨' => 'phong bì thư đến',
    '📩' => 'phong bì với mũi tên',
    '📤' => 'khay thư đi',
    '📥' => 'khay thư đến',
    '📦' => 'gói',
    '📫' => 'hộp thư được đóng với lá cờ đứng',
    '📪' => 'hộp thư được đóng với lá cờ nằm ngang',
    '📬' => 'hộp thư được mở với lá cờ đứng',
    '📭' => 'hộp thư được mở với lá cờ nằm ngang',
    '📮' => 'hòm thư',
    '🗳' => 'hòm phiếu với lá phiếu',
    '✏' => 'bút chì',
    '✒' => 'ngòi bút đen',
    '🖋' => 'bút máy',
    '🖊' => 'bút',
    '🖌' => 'cọ vẽ tranh',
    '🖍' => 'bút sáp màu',
    '📝' => 'sổ ghi nhớ',
    '💼' => 'cặp tài liệu',
    '📁' => 'thư mục tệp',
    '📂' => 'thư mục tệp mở',
    '🗂' => 'dụng cụ chia chỉ mục thẻ',
    '📅' => 'lịch',
    '📆' => 'quyển lịch bloc',
    '🗒' => 'sổ ghi chú gáy xoắn',
    '🗓' => 'lịch gáy xoắn',
    '📇' => 'chỉ mục thẻ',
    '📈' => 'biểu đồ tăng',
    '📉' => 'biểu đồ giảm',
    '📊' => 'biểu đồ thanh',
    '📋' => 'bảng ghi nhớ',
    '📌' => 'đinh ghim',
    '📍' => 'đinh ghim hình tròn',
    '📎' => 'kẹp giấy',
    '🖇' => 'kẹp giấy được nối',
    '📏' => 'thước thẳng',
    '📐' => 'thước tam giác',
    '✂' => 'kéo',
    '🗃' => 'hộp đựng hồ sơ',
    '🗄' => 'tủ hồ sơ',
    '🗑' => 'sọt rác',
    '🔒' => 'khóa',
    '🔓' => 'mở khóa',
    '🔏' => 'khóa với bút',
    '🔐' => 'khóa với chìa',
    '🔑' => 'chìa khóa',
    '🗝' => 'chìa khóa cũ',
    '🔨' => 'búa',
    '🪓' => 'rìu',
    '⛏' => 'búa chim',
    '⚒' => 'búa và búa chim',
    '🛠' => 'búa và cờ lê',
    '🗡' => 'dao găm',
    '⚔' => 'kiếm bắt chéo',
    '💣' => 'bom',
    '🪃' => 'bumơrang',
    '🏹' => 'cung tên',
    '🛡' => 'cái khiên',
    '🪚' => 'cưa mộc',
    '🔧' => 'cờ lê',
    '🪛' => 'tua vít',
    '🔩' => 'đai ốc và bu lông',
    '⚙' => 'bánh răng',
    '🗜' => 'cái kẹp',
    '⚖' => 'cân thăng bằng',
    '🦯' => 'gậy dò đường',
    '🔗' => 'mắt xích',
    '⛓' => 'dây xích',
    '🪝' => 'móc',
    '🧰' => 'hộp dụng cụ',
    '🧲' => 'nam châm',
    '🪜' => 'thang',
    '🪏' => 'xẻng',
    '⚗' => 'bình chưng cất',
    '🧪' => 'ống nghiệm',
    '🧫' => 'đĩa petri',
    '🧬' => 'adn',
    '🔬' => 'kính hiển vi',
    '🔭' => 'kính viễn vọng',
    '📡' => 'ăng-ten vệ tinh',
    '💉' => 'ống tiêm',
    '🩸' => 'giọt máu',
    '💊' => 'viên thuốc',
    '🩹' => 'băng dính',
    '🩼' => 'nạng',
    '🩺' => 'ống nghe',
    '🩻' => 'x-quang',
    '🚪' => 'cửa',
    '🛗' => 'thang máy',
    '🪞' => 'gương',
    '🪟' => 'cửa sổ',
    '🛏' => 'giường',
    '🛋' => 'ghế dài và đèn',
    '🪑' => 'ghế',
    '🚽' => 'bồn cầu',
    '🪠' => 'dụng cụ thông tắc',
    '🚿' => 'vòi hoa sen',
    '🛁' => 'bồn tắm',
    '🪤' => 'bẫy chuột',
    '🪒' => 'dao cạo',
    '🧴' => 'chai sữa dưỡng da',
    '🧷' => 'ghim băng',
    '🧹' => 'cây chổi',
    '🧺' => 'cái giỏ',
    '🧻' => 'cuộn giấy',
    '🪣' => 'xô',
    '🧼' => 'xà phòng',
    '🫧' => 'bong bóng',
    '🪥' => 'bàn chải răng',
    '🧽' => 'bọt biển',
    '🧯' => 'bình chữa cháy',
    '🛒' => 'xe mua hàng',
    '🚬' => 'thuốc lá',
    '⚰' => 'quan tài',
    '🪦' => 'bia mộ',
    '⚱' => 'bình đựng tro cốt',
    '🧿' => 'bùa may mắn nazar',
    '🪬' => 'hamsa',
    '🗿' => 'tượng moai',
    '🪧' => 'bảng hiệu',
    '🪪' => 'thẻ nhận dạng',
    '🏧' => 'ký hiệu ATM',
    '🚮' => 'ký hiệu bỏ rác vào thùng',
    '🚰' => 'nước uống',
    '♿' => 'biểu tượng xe lăn',
    '🚹' => 'nhà vệ sinh nam',
    '🚺' => 'nhà vệ sinh nữ',
    '🚻' => 'nhà vệ sinh',
    '🚼' => 'biểu tượng trẻ em',
    '🚾' => 'WC',
    '🛂' => 'kiểm soát hộ chiếu',
    '🛃' => 'hải quan',
    '🛄' => 'nơi nhận hành lý',
    '🛅' => 'hành lý bỏ quên',
    '⚠' => 'cảnh báo',
    '🚸' => 'qua đường cho trẻ em',
    '⛔' => 'cấm vào',
    '🚫' => 'bị cấm',
    '🚳' => 'cấm xe đạp',
    '🚭' => 'cấm hút thuốc',
    '🚯' => 'cấm xả rác',
    '🚱' => 'nước không uống được',
    '🚷' => 'cấm người đi bộ',
    '📵' => 'cấm điện thoại di động',
    '🔞' => 'cấm người dưới 18 tuổi',
    '☢' => 'phóng xạ',
    '☣' => 'nguy hiểm sinh học',
    '⬆' => 'mũi tên lên',
    '↗' => 'mũi tên lên bên phải',
    '➡' => 'mũi tên phải',
    '↘' => 'mũi tên xuống bên phải',
    '⬇' => 'mũi tên xuống',
    '↙' => 'mũi tên xuống bên trái',
    '⬅' => 'mũi tên trái',
    '↖' => 'mũi tên lên bên trái',
    '↕' => 'mũi tên lên xuống',
    '↔' => 'mũi tên trái phải',
    '↩' => 'mũi tên phải cong sang trái',
    '↪' => 'mũi tên trái cong sang phải',
    '⤴' => 'mũi tên phải cong lên',
    '⤵' => 'mũi tên phải cong xuống',
    '🔃' => 'mũi tên thẳng đứng theo chiều kim đồng hồ',
    '🔄' => 'nút mũi tên ngược chiều kim đồng hồ',
    '🔙' => 'mũi tên BACK',
    '🔚' => 'mũi tên END',
    '🔛' => 'mũi tên ON!',
    '🔜' => 'mũi tên SOON',
    '🔝' => 'mũi tên TOP',
    '🛐' => 'nơi thờ phụng',
    '⚛' => 'biểu tượng nguyên tử',
    '🕉' => 'om',
    '✡' => 'ngôi sao sáu cánh',
    '☸' => 'pháp luân',
    '☯' => 'âm dương',
    '✝' => 'thánh giá la-tinh',
    '☦' => 'thập giá chính thống giáo',
    '☪' => 'sao và trăng lưỡi liềm',
    '☮' => 'biểu tượng hòa bình',
    '🕎' => 'cây đèn menorah',
    '🔯' => 'ngôi sao sáu cánh có dấu chấm',
    '🪯' => 'biểu tượng khanda',
    '♈' => 'Bạch Dương',
    '♉' => 'Kim Ngưu',
    '♊' => 'Song Tử',
    '♋' => 'Cự Giải',
    '♌' => 'Sư Tử',
    '♍' => 'Xử Nữ',
    '♎' => 'Thiên Bình',
    '♏' => 'Bọ Cạp',
    '♐' => 'Nhân Mã',
    '♑' => 'Ma Kết',
    '♒' => 'Bảo Bình',
    '♓' => 'Song Ngư',
    '⛎' => 'Xà Phu',
    '🔀' => 'nút xáo trộn bài',
    '🔁' => 'nút lặp lại',
    '🔂' => 'nút lặp lại một lần',
    '▶' => 'nút phát',
    '⏩' => 'nút tua đi nhanh',
    '⏭' => 'nút bài tiếp theo',
    '⏯' => 'nút phát hoặc tạm dừng',
    '◀' => 'nút tua lại',
    '⏪' => 'nút tua lại nhanh',
    '⏮' => 'nút bài trước',
    '🔼' => 'nút đi lên',
    '⏫' => 'nút lên nhanh',
    '🔽' => 'nút đi xuống',
    '⏬' => 'nút xuống nhanh',
    '⏸' => 'nút tạm dừng',
    '⏹' => 'nút dừng',
    '⏺' => 'nút ghi',
    '⏏' => 'nút tháo',
    '🎦' => 'rạp chiếu phim',
    '🔅' => 'nút mờ',
    '🔆' => 'nút sáng',
    '📶' => 'vạch ăng-ten',
    '🛜' => 'không dây',
    '📳' => 'chế độ rung',
    '📴' => 'tắt điện thoại di động',
    '♀' => 'ký hiệu nữ',
    '♂' => 'ký hiệu nam',
    '⚧' => 'biểu tượng chuyển giới',
    '✖' => 'dấu nhân',
    '➕' => 'cộng',
    '➖' => 'trừ',
    '➗' => 'dấu chia',
    '🟰' => 'dấu bằng đậm',
    '♾' => 'vô cực',
    '‼' => 'hai dấu chấm than',
    '⁉' => 'dấu chấm than và dấu chấm hỏi',
    '❓' => 'dấu chấm hỏi',
    '❔' => 'dấu chấm hỏi màu trắng',
    '❕' => 'dấu chấm than màu trắng',
    '❗' => 'dấu chấm than đỏ',
    '〰' => 'dấu gạch ngang lượn sóng',
    '💱' => 'đổi tiền',
    '💲' => 'ký hiệu đô la đậm',
    '⚕' => 'biểu tượng y tế',
    '♻' => 'biểu tượng tái chế',
    '⚜' => 'hoa diên vĩ',
    '🔱' => 'biểu tượng đinh ba',
    '📛' => 'thẻ tên',
    '🔰' => 'ký hiệu của Nhật cho người mới bắt đầu',
    '⭕' => 'vòng tròn lớn đậm',
    '✅' => 'dấu chọn đậm màu trắng',
    '☑' => 'hộp kiểm có dấu chọn',
    '✔' => 'dấu chọn',
    '❌' => 'dấu gạch chéo',
    '❎' => 'nút dấu gạch chéo',
    '➰' => 'vòng lặp',
    '➿' => 'vòng lặp đôi',
    '〽' => 'ký hiệu thay đổi luân phiên',
    '✳' => 'dấu hoa thị tám cánh',
    '✴' => 'ngôi sao tám cánh',
    '❇' => 'lấp lánh',
    '™' => 'nhãn hiệu',
    '🫟' => 'bắn tung tóe',
    '🔠' => 'nhập chữ hoa la tinh',
    '🔡' => 'nhập chữ thường la tinh',
    '🔢' => 'nhập số',
    '🔣' => 'nhập biểu tượng',
    '🔤' => 'nhập chữ cái la tinh',
    '🅰' => 'nút A (nhóm máu)',
    '🆎' => 'nút AB (nhóm máu)',
    '🅱' => 'nút B (nhóm máu)',
    '🆑' => 'nút CL',
    '🆒' => 'nút COOL',
    '🆓' => 'nút FREE',
    'ℹ' => 'thông tin',
    '🆔' => 'nút ID',
    'Ⓜ' => 'chữ M trong vòng tròn',
    '🆕' => 'nút NEW',
    '🆖' => 'nút NG',
    '🅾' => 'nút O (nhóm máu)',
    '🆗' => 'nút OK',
    '🅿' => 'nút P',
    '🆘' => 'nút SOS',
    '🆙' => 'nút UP!',
    '🆚' => 'nút VS',
    '🈁' => 'nút Tiếng Nhật “tại đây”',
    '🈂' => 'nút Tiếng Nhật “phí dịch vụ”',
    '🈷' => 'nút Tiếng Nhật “số tiền hàng tháng”',
    '🈶' => 'nút Tiếng Nhật “không miễn phí”',
    '🈯' => 'nút Tiếng Nhật “đặt trước”',
    '🉐' => 'nút Tiếng Nhật “mặc cả”',
    '🈹' => 'nút Tiếng Nhật “giảm giá”',
    '🈚' => 'nút Tiếng Nhật “miễn phí”',
    '🈲' => 'nút Tiếng Nhật “bị cấm”',
    '🉑' => 'nút Tiếng Nhật “có thể chấp nhận”',
    '🈸' => 'nút Tiếng Nhật “đơn xin”',
    '🈴' => 'nút Tiếng Nhật “vượt cấp”',
    '🈳' => 'nút Tiếng Nhật “còn trống”',
    '㊗' => 'nút Tiếng Nhật “chúc mừng”',
    '㊙' => 'nút Tiếng Nhật “bí mật”',
    '🈺' => 'nút Tiếng Nhật “mở cửa kinh doanh”',
    '🈵' => 'nút "hết phòng" bằng tiếng Nhật',
    '🔴' => 'hình tròn màu đỏ',
    '🟠' => 'hình tròn màu cam',
    '🟡' => 'hình tròn màu vàng',
    '🟢' => 'hình tròn màu xanh lá cây',
    '🔵' => 'hình tròn màu lam',
    '🟣' => 'hình tròn màu tím',
    '🟤' => 'hình tròn màu nâu',
    '⚫' => 'hình tròn màu đen',
    '⚪' => 'hình tròn màu trắng',
    '🟥' => 'hình vuông màu đỏ',
    '🟧' => 'hình vuông màu cam',
    '🟨' => 'hình vuông màu vàng',
    '🟩' => 'hình vuông màu xanh lá cây',
    '🟦' => 'hình vuông màu xanh da trời',
    '🟪' => 'hình vuông màu tím',
    '🟫' => 'hình vuông màu nâu',
    '⬛' => 'hình vuông lớn màu đen',
    '⬜' => 'hình vuông lớn màu trắng',
    '◼' => 'hình vuông trung bình màu đen',
    '◻' => 'hình vuông trung bình màu trắng',
    '◾' => 'hình vuông nhỏ vừa màu đen',
    '◽' => 'hình vuông nhỏ vừa màu trắng',
    '▪' => 'hình vuông nhỏ màu đen',
    '▫' => 'hình vuông nhỏ màu trắng',
    '🔶' => 'hình thoi lớn màu cam',
    '🔷' => 'hình thoi lớn màu lam',
    '🔸' => 'hình thoi nhỏ màu cam',
    '🔹' => 'hình thoi nhỏ màu lam',
    '🔺' => 'tam giác màu đỏ trỏ lên trên',
    '🔻' => 'tam giác màu đỏ trỏ xuống dưới',
    '💠' => 'hình thoi có dấu chấm',
    '🔘' => 'nút radio',
    '🔳' => 'nút hình vuông màu trắng',
    '🔲' => 'nút hình vuông màu đen',
    '🏁' => 'cờ ô vuông',
    '🚩' => 'cờ tam giác',
    '🎌' => 'cờ bắt chéo',
    '🏴' => 'cờ đen',
    '🏳' => 'cờ trắng',
];
