<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'үбеү: өлкән, өлкән, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'үбеү: өлкән, өлкән, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'үбеү: өлкән, өлкән, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'үбеү: өлкән, өлкән, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'үбеү: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'үбеү: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'үбеү: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'үбеү: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'үбеү: өлкән, өлкән, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'үбеү: өлкән, өлкән, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'үбеү: өлкән, өлкән, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'үбеү: өлкән, өлкән, уртаса тән төҫө һәм ҡара тән төҫө',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'үбеү: өлкән, өлкән, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'үбеү: өлкән, өлкән, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'үбеү: өлкән, өлкән, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'үбеү: өлкән, өлкән, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'үбеү: өлкән, өлкән, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'үбеү: өлкән, өлкән, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'үбеү: өлкән, өлкән, ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'үбеү: өлкән, өлкән, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍❤‍💋‍👨🏻' => 'үбеү: ир, ир һәм аҡһыл тән төҫө',
    '👨🏻‍❤‍💋‍👨🏼' => 'үбеү: ир, ир, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏻‍❤‍💋‍👨🏽' => 'үбеү: ир, ир, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏻‍❤‍💋‍👨🏾' => 'үбеү: ир, ир, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍❤‍💋‍👨🏿' => 'үбеү: ир, ир, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏼‍❤‍💋‍👨🏻' => 'үбеү: ир, ир, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👨🏼‍❤‍💋‍👨🏼' => 'үбеү: ир, ир һәм уртаса аҡһыл тән төҫө',
    '👨🏼‍❤‍💋‍👨🏽' => 'үбеү: ир, ир, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏼‍❤‍💋‍👨🏾' => 'үбеү: ир, ир, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏼‍❤‍💋‍👨🏿' => 'үбеү: ир, ир, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏽‍❤‍💋‍👨🏻' => 'үбеү: ир, ир, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👨🏽‍❤‍💋‍👨🏼' => 'үбеү: ир, ир, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏽‍❤‍💋‍👨🏽' => 'үбеү: ир, ир һәм уртаса тән төҫө',
    '👨🏽‍❤‍💋‍👨🏾' => 'үбеү: ир, ир, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏽‍❤‍💋‍👨🏿' => 'үбеү: ир, ир, уртаса тән төҫө һәм ҡара тән төҫө',
    '👨🏾‍❤‍💋‍👨🏻' => 'үбеү: ир, ир, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏾‍❤‍💋‍👨🏼' => 'үбеү: ир, ир, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏾‍❤‍💋‍👨🏽' => 'үбеү: ир, ир, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏾‍❤‍💋‍👨🏾' => 'үбеү: ир, ир һәм уртаса ҡара тән төҫө',
    '👨🏾‍❤‍💋‍👨🏿' => 'үбеү: ир, ир, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👨🏿‍❤‍💋‍👨🏻' => 'үбеү: ир, ир, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏿‍❤‍💋‍👨🏼' => 'үбеү: ир, ир, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏿‍❤‍💋‍👨🏽' => 'үбеү: ир, ир, ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏿‍❤‍💋‍👨🏾' => 'үбеү: ир, ир, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏿‍❤‍💋‍👨🏿' => 'үбеү: ир, ир һәм ҡара тән төҫө',
    '👩🏻‍❤‍💋‍👨🏻' => 'үбеү: ҡатын, ир һәм аҡһыл тән төҫө',
    '👩🏻‍❤‍💋‍👨🏼' => 'үбеү: ҡатын, ир, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍❤‍💋‍👨🏽' => 'үбеү: ҡатын, ир, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍❤‍💋‍👨🏾' => 'үбеү: ҡатын, ир, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍❤‍💋‍👨🏿' => 'үбеү: ҡатын, ир, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍❤‍💋‍👨🏻' => 'үбеү: ҡатын, ир, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍❤‍💋‍👨🏼' => 'үбеү: ҡатын, ир һәм уртаса аҡһыл тән төҫө',
    '👩🏼‍❤‍💋‍👨🏽' => 'үбеү: ҡатын, ир, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍❤‍💋‍👨🏾' => 'үбеү: ҡатын, ир, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍❤‍💋‍👨🏿' => 'үбеү: ҡатын, ир, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍❤‍💋‍👨🏻' => 'үбеү: ҡатын, ир, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍❤‍💋‍👨🏼' => 'үбеү: ҡатын, ир, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍❤‍💋‍👨🏽' => 'үбеү: ҡатын, ир һәм уртаса тән төҫө',
    '👩🏽‍❤‍💋‍👨🏾' => 'үбеү: ҡатын, ир, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍❤‍💋‍👨🏿' => 'үбеү: ҡатын, ир, уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍❤‍💋‍👨🏻' => 'үбеү: ҡатын, ир, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍❤‍💋‍👨🏼' => 'үбеү: ҡатын, ир, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍❤‍💋‍👨🏽' => 'үбеү: ҡатын, ир, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍❤‍💋‍👨🏾' => 'үбеү: ҡатын, ир һәм уртаса ҡара тән төҫө',
    '👩🏾‍❤‍💋‍👨🏿' => 'үбеү: ҡатын, ир, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍❤‍💋‍👨🏻' => 'үбеү: ҡатын, ир, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍❤‍💋‍👨🏼' => 'үбеү: ҡатын, ир, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍❤‍💋‍👨🏽' => 'үбеү: ҡатын, ир, ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍❤‍💋‍👨🏾' => 'үбеү: ҡатын, ир, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏿‍❤‍💋‍👨🏿' => 'үбеү: ҡатын, ир һәм ҡара тән төҫө',
    '👩🏻‍❤‍💋‍👩🏻' => 'үбеү: ҡатын, ҡатын һәм аҡһыл тән төҫө',
    '👩🏻‍❤‍💋‍👩🏼' => 'үбеү: ҡатын, ҡатын, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍❤‍💋‍👩🏽' => 'үбеү: ҡатын, ҡатын, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍❤‍💋‍👩🏾' => 'үбеү: ҡатын, ҡатын, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍❤‍💋‍👩🏿' => 'үбеү: ҡатын, ҡатын, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍❤‍💋‍👩🏻' => 'үбеү: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍❤‍💋‍👩🏼' => 'үбеү: ҡатын, ҡатын һәм уртаса аҡһыл тән төҫө',
    '👩🏼‍❤‍💋‍👩🏽' => 'үбеү: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍❤‍💋‍👩🏾' => 'үбеү: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍❤‍💋‍👩🏿' => 'үбеү: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍❤‍💋‍👩🏻' => 'үбеү: ҡатын, ҡатын, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍❤‍💋‍👩🏼' => 'үбеү: ҡатын, ҡатын, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍❤‍💋‍👩🏽' => 'үбеү: ҡатын, ҡатын һәм уртаса тән төҫө',
    '👩🏽‍❤‍💋‍👩🏾' => 'үбеү: ҡатын, ҡатын, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍❤‍💋‍👩🏿' => 'үбеү: ҡатын, ҡатын, уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍❤‍💋‍👩🏻' => 'үбеү: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍❤‍💋‍👩🏼' => 'үбеү: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍❤‍💋‍👩🏽' => 'үбеү: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍❤‍💋‍👩🏾' => 'үбеү: ҡатын, ҡатын һәм уртаса ҡара тән төҫө',
    '👩🏾‍❤‍💋‍👩🏿' => 'үбеү: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍❤‍💋‍👩🏻' => 'үбеү: ҡатын, ҡатын, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍❤‍💋‍👩🏼' => 'үбеү: ҡатын, ҡатын, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍❤‍💋‍👩🏽' => 'үбеү: ҡатын, ҡатын, ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍❤‍💋‍👩🏾' => 'үбеү: ҡатын, ҡатын, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏿‍❤‍💋‍👩🏿' => 'үбеү: ҡатын, ҡатын һәм ҡара тән төҫө',
    '🧑🏻‍🫯‍🧑🏼' => 'көрәшселәр: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏻‍🫯‍🧑🏽' => 'көрәшселәр: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏻‍🫯‍🧑🏾' => 'көрәшселәр: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍🫯‍🧑🏿' => 'көрәшселәр: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏼‍🫯‍🧑🏻' => 'көрәшселәр: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏼‍🫯‍🧑🏽' => 'көрәшселәр: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏼‍🫯‍🧑🏾' => 'көрәшселәр: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏼‍🫯‍🧑🏿' => 'көрәшселәр: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏽‍🫯‍🧑🏻' => 'көрәшселәр: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏽‍🫯‍🧑🏼' => 'көрәшселәр: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏽‍🫯‍🧑🏾' => 'көрәшселәр: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏽‍🫯‍🧑🏿' => 'көрәшселәр: уртаса тән төҫө һәм ҡара тән төҫө',
    '🧑🏾‍🫯‍🧑🏻' => 'көрәшселәр: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏾‍🫯‍🧑🏼' => 'көрәшселәр: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏾‍🫯‍🧑🏽' => 'көрәшселәр: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏾‍🫯‍🧑🏿' => 'көрәшселәр: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '🧑🏿‍🫯‍🧑🏻' => 'көрәшселәр: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏿‍🫯‍🧑🏼' => 'көрәшселәр: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏿‍🫯‍🧑🏽' => 'көрәшселәр: ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏿‍🫯‍🧑🏾' => 'көрәшселәр: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍❤‍🧑🏼' => 'йөрәкле пар: өлкән, өлкән, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏻‍❤‍🧑🏽' => 'йөрәкле пар: өлкән, өлкән, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏻‍❤‍🧑🏾' => 'йөрәкле пар: өлкән, өлкән, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍❤‍🧑🏿' => 'йөрәкле пар: өлкән, өлкән, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏼‍❤‍🧑🏻' => 'йөрәкле пар: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏼‍❤‍🧑🏽' => 'йөрәкле пар: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏼‍❤‍🧑🏾' => 'йөрәкле пар: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏼‍❤‍🧑🏿' => 'йөрәкле пар: өлкән, өлкән, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏽‍❤‍🧑🏻' => 'йөрәкле пар: өлкән, өлкән, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏽‍❤‍🧑🏼' => 'йөрәкле пар: өлкән, өлкән, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏽‍❤‍🧑🏾' => 'йөрәкле пар: өлкән, өлкән, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏽‍❤‍🧑🏿' => 'йөрәкле пар: өлкән, өлкән, уртаса тән төҫө һәм ҡара тән төҫө',
    '🧑🏾‍❤‍🧑🏻' => 'йөрәкле пар: өлкән, өлкән, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏾‍❤‍🧑🏼' => 'йөрәкле пар: өлкән, өлкән, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏾‍❤‍🧑🏽' => 'йөрәкле пар: өлкән, өлкән, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏾‍❤‍🧑🏿' => 'йөрәкле пар: өлкән, өлкән, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '🧑🏿‍❤‍🧑🏻' => 'йөрәкле пар: өлкән, өлкән, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏿‍❤‍🧑🏼' => 'йөрәкле пар: өлкән, өлкән, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏿‍❤‍🧑🏽' => 'йөрәкле пар: өлкән, өлкән, ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏿‍❤‍🧑🏾' => 'йөрәкле пар: өлкән, өлкән, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍🐰‍🧑🏼' => 'ҡуян ҡолаҡлы кешеләр: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏻‍🐰‍🧑🏽' => 'ҡуян ҡолаҡлы кешеләр: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏻‍🐰‍🧑🏾' => 'ҡуян ҡолаҡлы кешеләр: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍🐰‍🧑🏿' => 'ҡуян ҡолаҡлы кешеләр: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏼‍🐰‍🧑🏻' => 'ҡуян ҡолаҡлы кешеләр: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏼‍🐰‍🧑🏽' => 'ҡуян ҡолаҡлы кешеләр: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏼‍🐰‍🧑🏾' => 'ҡуян ҡолаҡлы кешеләр: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏼‍🐰‍🧑🏿' => 'ҡуян ҡолаҡлы кешеләр: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏽‍🐰‍🧑🏻' => 'ҡуян ҡолаҡлы кешеләр: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏽‍🐰‍🧑🏼' => 'ҡуян ҡолаҡлы кешеләр: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏽‍🐰‍🧑🏾' => 'ҡуян ҡолаҡлы кешеләр: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏽‍🐰‍🧑🏿' => 'ҡуян ҡолаҡлы кешеләр: уртаса тән төҫө һәм ҡара тән төҫө',
    '🧑🏾‍🐰‍🧑🏻' => 'ҡуян ҡолаҡлы кешеләр: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏾‍🐰‍🧑🏼' => 'ҡуян ҡолаҡлы кешеләр: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏾‍🐰‍🧑🏽' => 'ҡуян ҡолаҡлы кешеләр: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏾‍🐰‍🧑🏿' => 'ҡуян ҡолаҡлы кешеләр: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '🧑🏿‍🐰‍🧑🏻' => 'ҡуян ҡолаҡлы кешеләр: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏿‍🐰‍🧑🏼' => 'ҡуян ҡолаҡлы кешеләр: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏿‍🐰‍🧑🏽' => 'ҡуян ҡолаҡлы кешеләр: ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏿‍🐰‍🧑🏾' => 'ҡуян ҡолаҡлы кешеләр: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍🫯‍👨🏼' => 'ир-ат көрәшселәр: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏻‍🫯‍👨🏽' => 'ир-ат көрәшселәр: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏻‍🫯‍👨🏾' => 'ир-ат көрәшселәр: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍🫯‍👨🏿' => 'ир-ат көрәшселәр: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏼‍🫯‍👨🏻' => 'ир-ат көрәшселәр: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👨🏼‍🫯‍👨🏽' => 'ир-ат көрәшселәр: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏼‍🫯‍👨🏾' => 'ир-ат көрәшселәр: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏼‍🫯‍👨🏿' => 'ир-ат көрәшселәр: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏽‍🫯‍👨🏻' => 'ир-ат көрәшселәр: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👨🏽‍🫯‍👨🏼' => 'ир-ат көрәшселәр: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏽‍🫯‍👨🏾' => 'ир-ат көрәшселәр: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏽‍🫯‍👨🏿' => 'ир-ат көрәшселәр: уртаса тән төҫө һәм ҡара тән төҫө',
    '👨🏾‍🫯‍👨🏻' => 'ир-ат көрәшселәр: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏾‍🫯‍👨🏼' => 'ир-ат көрәшселәр: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏾‍🫯‍👨🏽' => 'ир-ат көрәшселәр: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏾‍🫯‍👨🏿' => 'ир-ат көрәшселәр: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👨🏿‍🫯‍👨🏻' => 'ир-ат көрәшселәр: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏿‍🫯‍👨🏼' => 'ир-ат көрәшселәр: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏿‍🫯‍👨🏽' => 'ир-ат көрәшселәр: ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏿‍🫯‍👨🏾' => 'ир-ат көрәшселәр: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍❤‍👨🏻' => 'йөрәкле пар: ир, ир һәм аҡһыл тән төҫө',
    '👨🏻‍❤‍👨🏼' => 'йөрәкле пар: ир, ир, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏻‍❤‍👨🏽' => 'йөрәкле пар: ир, ир, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏻‍❤‍👨🏾' => 'йөрәкле пар: ир, ир, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍❤‍👨🏿' => 'йөрәкле пар: ир, ир, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏼‍❤‍👨🏻' => 'йөрәкле пар: ир, ир, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👨🏼‍❤‍👨🏼' => 'йөрәкле пар: ир, ир һәм уртаса аҡһыл тән төҫө',
    '👨🏼‍❤‍👨🏽' => 'йөрәкле пар: ир, ир, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏼‍❤‍👨🏾' => 'йөрәкле пар: ир, ир, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏼‍❤‍👨🏿' => 'йөрәкле пар: ир, ир, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏽‍❤‍👨🏻' => 'йөрәкле пар: ир, ир, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👨🏽‍❤‍👨🏼' => 'йөрәкле пар: ир, ир, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏽‍❤‍👨🏽' => 'йөрәкле пар: ир, ир һәм уртаса тән төҫө',
    '👨🏽‍❤‍👨🏾' => 'йөрәкле пар: ир, ир, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏽‍❤‍👨🏿' => 'йөрәкле пар: ир, ир, уртаса тән төҫө һәм ҡара тән төҫө',
    '👨🏾‍❤‍👨🏻' => 'йөрәкле пар: ир, ир, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏾‍❤‍👨🏼' => 'йөрәкле пар: ир, ир, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏾‍❤‍👨🏽' => 'йөрәкле пар: ир, ир, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏾‍❤‍👨🏾' => 'йөрәкле пар: ир, ир һәм уртаса ҡара тән төҫө',
    '👨🏾‍❤‍👨🏿' => 'йөрәкле пар: ир, ир, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👨🏿‍❤‍👨🏻' => 'йөрәкле пар: ир, ир, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏿‍❤‍👨🏼' => 'йөрәкле пар: ир, ир, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏿‍❤‍👨🏽' => 'йөрәкле пар: ир, ир, ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏿‍❤‍👨🏾' => 'йөрәкле пар: ир, ир, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏿‍❤‍👨🏿' => 'йөрәкле пар: ир, ир һәм ҡара тән төҫө',
    '👨🏻‍🐰‍👨🏼' => 'ҡуян ҡолаҡлы ирҙәр: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏻‍🐰‍👨🏽' => 'ҡуян ҡолаҡлы ирҙәр: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏻‍🐰‍👨🏾' => 'ҡуян ҡолаҡлы ирҙәр: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍🐰‍👨🏿' => 'ҡуян ҡолаҡлы ирҙәр: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏼‍🐰‍👨🏻' => 'ҡуян ҡолаҡлы ирҙәр: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👨🏼‍🐰‍👨🏽' => 'ҡуян ҡолаҡлы ирҙәр: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏼‍🐰‍👨🏾' => 'ҡуян ҡолаҡлы ирҙәр: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏼‍🐰‍👨🏿' => 'ҡуян ҡолаҡлы ирҙәр: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏽‍🐰‍👨🏻' => 'ҡуян ҡолаҡлы ирҙәр: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👨🏽‍🐰‍👨🏼' => 'ҡуян ҡолаҡлы ирҙәр: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏽‍🐰‍👨🏾' => 'ҡуян ҡолаҡлы ирҙәр: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏽‍🐰‍👨🏿' => 'ҡуян ҡолаҡлы ирҙәр: уртаса тән төҫө һәм ҡара тән төҫө',
    '👨🏾‍🐰‍👨🏻' => 'ҡуян ҡолаҡлы ирҙәр: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏾‍🐰‍👨🏼' => 'ҡуян ҡолаҡлы ирҙәр: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏾‍🐰‍👨🏽' => 'ҡуян ҡолаҡлы ирҙәр: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏾‍🐰‍👨🏿' => 'ҡуян ҡолаҡлы ирҙәр: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👨🏿‍🐰‍👨🏻' => 'ҡуян ҡолаҡлы ирҙәр: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏿‍🐰‍👨🏼' => 'ҡуян ҡолаҡлы ирҙәр: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏿‍🐰‍👨🏽' => 'ҡуян ҡолаҡлы ирҙәр: ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏿‍🐰‍👨🏾' => 'ҡуян ҡолаҡлы ирҙәр: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍🫯‍👩🏼' => 'ҡатын-ҡыҙ көрәшселәр: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍🫯‍👩🏽' => 'ҡатын-ҡыҙ көрәшселәр: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍🫯‍👩🏾' => 'ҡатын-ҡыҙ көрәшселәр: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍🫯‍👩🏿' => 'ҡатын-ҡыҙ көрәшселәр: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍🫯‍👩🏻' => 'ҡатын-ҡыҙ көрәшселәр: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍🫯‍👩🏽' => 'ҡатын-ҡыҙ көрәшселәр: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍🫯‍👩🏾' => 'ҡатын-ҡыҙ көрәшселәр: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍🫯‍👩🏿' => 'ҡатын-ҡыҙ көрәшселәр: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍🫯‍👩🏻' => 'ҡатын-ҡыҙ көрәшселәр: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍🫯‍👩🏼' => 'ҡатын-ҡыҙ көрәшселәр: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍🫯‍👩🏾' => 'ҡатын-ҡыҙ көрәшселәр: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍🫯‍👩🏿' => 'ҡатын-ҡыҙ көрәшселәр: уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍🫯‍👩🏻' => 'ҡатын-ҡыҙ көрәшселәр: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍🫯‍👩🏼' => 'ҡатын-ҡыҙ көрәшселәр: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍🫯‍👩🏽' => 'ҡатын-ҡыҙ көрәшселәр: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍🫯‍👩🏿' => 'ҡатын-ҡыҙ көрәшселәр: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍🫯‍👩🏻' => 'ҡатын-ҡыҙ көрәшселәр: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍🫯‍👩🏼' => 'ҡатын-ҡыҙ көрәшселәр: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍🫯‍👩🏽' => 'ҡатын-ҡыҙ көрәшселәр: ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍🫯‍👩🏾' => 'ҡатын-ҡыҙ көрәшселәр: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍❤‍👨🏻' => 'йөрәкле пар: ҡатын, ир һәм аҡһыл тән төҫө',
    '👩🏻‍❤‍👨🏼' => 'йөрәкле пар: ҡатын, ир, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍❤‍👨🏽' => 'йөрәкле пар: ҡатын, ир, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍❤‍👨🏾' => 'йөрәкле пар: ҡатын, ир, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍❤‍👨🏿' => 'йөрәкле пар: ҡатын, ир, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍❤‍👨🏻' => 'йөрәкле пар: ҡатын, ир, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍❤‍👨🏼' => 'йөрәкле пар: ҡатын, ир һәм уртаса аҡһыл тән төҫө',
    '👩🏼‍❤‍👨🏽' => 'йөрәкле пар: ҡатын, ир, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍❤‍👨🏾' => 'йөрәкле пар: ҡатын, ир, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍❤‍👨🏿' => 'йөрәкле пар: ҡатын, ир, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍❤‍👨🏻' => 'йөрәкле пар: ҡатын, ир, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍❤‍👨🏼' => 'йөрәкле пар: ҡатын, ир, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍❤‍👨🏽' => 'йөрәкле пар: ҡатын, ир һәм уртаса тән төҫө',
    '👩🏽‍❤‍👨🏾' => 'йөрәкле пар: ҡатын, ир, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍❤‍👨🏿' => 'йөрәкле пар: ҡатын, ир, уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍❤‍👨🏻' => 'йөрәкле пар: ҡатын, ир, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍❤‍👨🏼' => 'йөрәкле пар: ҡатын, ир, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍❤‍👨🏽' => 'йөрәкле пар: ҡатын, ир, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍❤‍👨🏾' => 'йөрәкле пар: ҡатын, ир һәм уртаса ҡара тән төҫө',
    '👩🏾‍❤‍👨🏿' => 'йөрәкле пар: ҡатын, ир, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍❤‍👨🏻' => 'йөрәкле пар: ҡатын, ир, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍❤‍👨🏼' => 'йөрәкле пар: ҡатын, ир, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍❤‍👨🏽' => 'йөрәкле пар: ҡатын, ир, ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍❤‍👨🏾' => 'йөрәкле пар: ҡатын, ир, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏿‍❤‍👨🏿' => 'йөрәкле пар: ҡатын, ир һәм ҡара тән төҫө',
    '👩🏻‍❤‍👩🏻' => 'йөрәкле пар: ҡатын, ҡатын һәм аҡһыл тән төҫө',
    '👩🏻‍❤‍👩🏼' => 'йөрәкле пар: ҡатын, ҡатын, аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍❤‍👩🏽' => 'йөрәкле пар: ҡатын, ҡатын, аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍❤‍👩🏾' => 'йөрәкле пар: ҡатын, ҡатын, аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍❤‍👩🏿' => 'йөрәкле пар: ҡатын, ҡатын, аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍❤‍👩🏻' => 'йөрәкле пар: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍❤‍👩🏼' => 'йөрәкле пар: ҡатын, ҡатын һәм уртаса аҡһыл тән төҫө',
    '👩🏼‍❤‍👩🏽' => 'йөрәкле пар: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍❤‍👩🏾' => 'йөрәкле пар: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍❤‍👩🏿' => 'йөрәкле пар: ҡатын, ҡатын, уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍❤‍👩🏻' => 'йөрәкле пар: ҡатын, ҡатын, уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍❤‍👩🏼' => 'йөрәкле пар: ҡатын, ҡатын, уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍❤‍👩🏽' => 'йөрәкле пар: ҡатын, ҡатын һәм уртаса тән төҫө',
    '👩🏽‍❤‍👩🏾' => 'йөрәкле пар: ҡатын, ҡатын, уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍❤‍👩🏿' => 'йөрәкле пар: ҡатын, ҡатын, уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍❤‍👩🏻' => 'йөрәкле пар: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍❤‍👩🏼' => 'йөрәкле пар: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍❤‍👩🏽' => 'йөрәкле пар: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍❤‍👩🏾' => 'йөрәкле пар: ҡатын, ҡатын һәм уртаса ҡара тән төҫө',
    '👩🏾‍❤‍👩🏿' => 'йөрәкле пар: ҡатын, ҡатын, уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍❤‍👩🏻' => 'йөрәкле пар: ҡатын, ҡатын, ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍❤‍👩🏼' => 'йөрәкле пар: ҡатын, ҡатын, ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍❤‍👩🏽' => 'йөрәкле пар: ҡатын, ҡатын, ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍❤‍👩🏾' => 'йөрәкле пар: ҡатын, ҡатын, ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏿‍❤‍👩🏿' => 'йөрәкле пар: ҡатын, ҡатын һәм ҡара тән төҫө',
    '👩🏻‍🐰‍👩🏼' => 'ҡуян ҡолаҡлы ҡатындар: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍🐰‍👩🏽' => 'ҡуян ҡолаҡлы ҡатындар: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍🐰‍👩🏾' => 'ҡуян ҡолаҡлы ҡатындар: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍🐰‍👩🏿' => 'ҡуян ҡолаҡлы ҡатындар: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍🐰‍👩🏻' => 'ҡуян ҡолаҡлы ҡатындар: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍🐰‍👩🏽' => 'ҡуян ҡолаҡлы ҡатындар: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍🐰‍👩🏾' => 'ҡуян ҡолаҡлы ҡатындар: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍🐰‍👩🏿' => 'ҡуян ҡолаҡлы ҡатындар: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍🐰‍👩🏻' => 'ҡуян ҡолаҡлы ҡатындар: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍🐰‍👩🏼' => 'ҡуян ҡолаҡлы ҡатындар: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍🐰‍👩🏾' => 'ҡуян ҡолаҡлы ҡатындар: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍🐰‍👩🏿' => 'ҡуян ҡолаҡлы ҡатындар: уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍🐰‍👩🏻' => 'ҡуян ҡолаҡлы ҡатындар: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍🐰‍👩🏼' => 'ҡуян ҡолаҡлы ҡатындар: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍🐰‍👩🏽' => 'ҡуян ҡолаҡлы ҡатындар: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍🐰‍👩🏿' => 'ҡуян ҡолаҡлы ҡатындар: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍🐰‍👩🏻' => 'ҡуян ҡолаҡлы ҡатындар: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍🐰‍👩🏼' => 'ҡуян ҡолаҡлы ҡатындар: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍🐰‍👩🏽' => 'ҡуян ҡолаҡлы ҡатындар: ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍🐰‍👩🏾' => 'ҡуян ҡолаҡлы ҡатындар: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍🤝‍🧑🏻' => 'ҡул тотошҡан кешеләр: аҡһыл тән төҫө',
    '🧑🏻‍🤝‍🧑🏼' => 'ҡул тотошҡан кешеләр: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏻‍🤝‍🧑🏽' => 'ҡул тотошҡан кешеләр: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏻‍🤝‍🧑🏾' => 'ҡул тотошҡан кешеләр: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏻‍🤝‍🧑🏿' => 'ҡул тотошҡан кешеләр: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏼‍🤝‍🧑🏻' => 'ҡул тотошҡан кешеләр: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏼‍🤝‍🧑🏼' => 'ҡул тотошҡан кешеләр: уртаса аҡһыл тән төҫө',
    '🧑🏼‍🤝‍🧑🏽' => 'ҡул тотошҡан кешеләр: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🧑🏼‍🤝‍🧑🏾' => 'ҡул тотошҡан кешеләр: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏼‍🤝‍🧑🏿' => 'ҡул тотошҡан кешеләр: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🧑🏽‍🤝‍🧑🏻' => 'ҡул тотошҡан кешеләр: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏽‍🤝‍🧑🏼' => 'ҡул тотошҡан кешеләр: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏽‍🤝‍🧑🏽' => 'ҡул тотошҡан кешеләр: уртаса тән төҫө',
    '🧑🏽‍🤝‍🧑🏾' => 'ҡул тотошҡан кешеләр: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏽‍🤝‍🧑🏿' => 'ҡул тотошҡан кешеләр: уртаса тән төҫө һәм ҡара тән төҫө',
    '🧑🏾‍🤝‍🧑🏻' => 'ҡул тотошҡан кешеләр: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏾‍🤝‍🧑🏼' => 'ҡул тотошҡан кешеләр: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏾‍🤝‍🧑🏽' => 'ҡул тотошҡан кешеләр: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏾‍🤝‍🧑🏾' => 'ҡул тотошҡан кешеләр: уртаса ҡара тән төҫө',
    '🧑🏾‍🤝‍🧑🏿' => 'ҡул тотошҡан кешеләр: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '🧑🏿‍🤝‍🧑🏻' => 'ҡул тотошҡан кешеләр: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🧑🏿‍🤝‍🧑🏼' => 'ҡул тотошҡан кешеләр: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🧑🏿‍🤝‍🧑🏽' => 'ҡул тотошҡан кешеләр: ҡара тән төҫө һәм уртаса тән төҫө',
    '🧑🏿‍🤝‍🧑🏾' => 'ҡул тотошҡан кешеләр: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '🧑🏿‍🤝‍🧑🏿' => 'ҡул тотошҡан кешеләр: ҡара тән төҫө',
    '👩🏻‍🤝‍👩🏼' => 'ҡул тотошҡан ҡатындар: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍🤝‍👩🏽' => 'ҡул тотошҡан ҡатындар: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍🤝‍👩🏾' => 'ҡул тотошҡан ҡатындар: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍🤝‍👩🏿' => 'ҡул тотошҡан ҡатындар: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍🤝‍👩🏻' => 'ҡул тотошҡан ҡатындар: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍🤝‍👩🏽' => 'ҡул тотошҡан ҡатындар: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍🤝‍👩🏾' => 'ҡул тотошҡан ҡатындар: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍🤝‍👩🏿' => 'ҡул тотошҡан ҡатындар: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍🤝‍👩🏻' => 'ҡул тотошҡан ҡатындар: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍🤝‍👩🏼' => 'ҡул тотошҡан ҡатындар: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍🤝‍👩🏾' => 'ҡул тотошҡан ҡатындар: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍🤝‍👩🏿' => 'ҡул тотошҡан ҡатындар: уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍🤝‍👩🏻' => 'ҡул тотошҡан ҡатындар: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍🤝‍👩🏼' => 'ҡул тотошҡан ҡатындар: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍🤝‍👩🏽' => 'ҡул тотошҡан ҡатындар: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍🤝‍👩🏿' => 'ҡул тотошҡан ҡатындар: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍🤝‍👩🏻' => 'ҡул тотошҡан ҡатындар: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍🤝‍👩🏼' => 'ҡул тотошҡан ҡатындар: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍🤝‍👩🏽' => 'ҡул тотошҡан ҡатындар: ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍🤝‍👩🏾' => 'ҡул тотошҡан ҡатындар: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍🤝‍👨🏼' => 'ҡул тотошҡан ҡатын һәм ир: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏻‍🤝‍👨🏽' => 'ҡул тотошҡан ҡатын һәм ир: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏻‍🤝‍👨🏾' => 'ҡул тотошҡан ҡатын һәм ир: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏻‍🤝‍👨🏿' => 'ҡул тотошҡан ҡатын һәм ир: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏼‍🤝‍👨🏻' => 'ҡул тотошҡан ҡатын һәм ир: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👩🏼‍🤝‍👨🏽' => 'ҡул тотошҡан ҡатын һәм ир: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👩🏼‍🤝‍👨🏾' => 'ҡул тотошҡан ҡатын һәм ир: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏼‍🤝‍👨🏿' => 'ҡул тотошҡан ҡатын һәм ир: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👩🏽‍🤝‍👨🏻' => 'ҡул тотошҡан ҡатын һәм ир: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👩🏽‍🤝‍👨🏼' => 'ҡул тотошҡан ҡатын һәм ир: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏽‍🤝‍👨🏾' => 'ҡул тотошҡан ҡатын һәм ир: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👩🏽‍🤝‍👨🏿' => 'ҡул тотошҡан ҡатын һәм ир: уртаса тән төҫө һәм ҡара тән төҫө',
    '👩🏾‍🤝‍👨🏻' => 'ҡул тотошҡан ҡатын һәм ир: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏾‍🤝‍👨🏼' => 'ҡул тотошҡан ҡатын һәм ир: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏾‍🤝‍👨🏽' => 'ҡул тотошҡан ҡатын һәм ир: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏾‍🤝‍👨🏿' => 'ҡул тотошҡан ҡатын һәм ир: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👩🏿‍🤝‍👨🏻' => 'ҡул тотошҡан ҡатын һәм ир: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👩🏿‍🤝‍👨🏼' => 'ҡул тотошҡан ҡатын һәм ир: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👩🏿‍🤝‍👨🏽' => 'ҡул тотошҡан ҡатын һәм ир: ҡара тән төҫө һәм уртаса тән төҫө',
    '👩🏿‍🤝‍👨🏾' => 'ҡул тотошҡан ҡатын һәм ир: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍🤝‍👨🏼' => 'ҡул тотошҡан ирҙәр: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏻‍🤝‍👨🏽' => 'ҡул тотошҡан ирҙәр: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏻‍🤝‍👨🏾' => 'ҡул тотошҡан ирҙәр: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏻‍🤝‍👨🏿' => 'ҡул тотошҡан ирҙәр: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏼‍🤝‍👨🏻' => 'ҡул тотошҡан ирҙәр: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '👨🏼‍🤝‍👨🏽' => 'ҡул тотошҡан ирҙәр: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '👨🏼‍🤝‍👨🏾' => 'ҡул тотошҡан ирҙәр: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏼‍🤝‍👨🏿' => 'ҡул тотошҡан ирҙәр: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '👨🏽‍🤝‍👨🏻' => 'ҡул тотошҡан ирҙәр: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '👨🏽‍🤝‍👨🏼' => 'ҡул тотошҡан ирҙәр: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏽‍🤝‍👨🏾' => 'ҡул тотошҡан ирҙәр: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '👨🏽‍🤝‍👨🏿' => 'ҡул тотошҡан ирҙәр: уртаса тән төҫө һәм ҡара тән төҫө',
    '👨🏾‍🤝‍👨🏻' => 'ҡул тотошҡан ирҙәр: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏾‍🤝‍👨🏼' => 'ҡул тотошҡан ирҙәр: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏾‍🤝‍👨🏽' => 'ҡул тотошҡан ирҙәр: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏾‍🤝‍👨🏿' => 'ҡул тотошҡан ирҙәр: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '👨🏿‍🤝‍👨🏻' => 'ҡул тотошҡан ирҙәр: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '👨🏿‍🤝‍👨🏼' => 'ҡул тотошҡан ирҙәр: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '👨🏿‍🤝‍👨🏽' => 'ҡул тотошҡан ирҙәр: ҡара тән төҫө һәм уртаса тән төҫө',
    '👨🏿‍🤝‍👨🏾' => 'ҡул тотошҡан ирҙәр: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '👩‍❤‍💋‍👨' => 'үбеү: ҡатын һәм ир',
    '👨‍❤‍💋‍👨' => 'үбеү: ир һәм ир',
    '👩‍❤‍💋‍👩' => 'үбеү: ҡатын һәм ҡатын',
    '👨‍👩‍👧‍👦' => 'ғаилә: ир, ҡатын, ҡыҙ һәм малай',
    '👨‍👩‍👦‍👦' => 'ғаилә: ир, ҡатын, малай һәм малай',
    '👨‍👩‍👧‍👧' => 'ғаилә: ир, ҡатын, ҡыҙ һәм ҡыҙ',
    '👨‍👨‍👧‍👦' => 'ғаилә: ир, ир, ҡыҙ һәм малай',
    '👨‍👨‍👦‍👦' => 'ғаилә: ир, ир, малай һәм малай',
    '👨‍👨‍👧‍👧' => 'ғаилә: ир, ир, ҡыҙ һәм ҡыҙ',
    '👩‍👩‍👧‍👦' => 'ғаилә: ҡатын, ҡатын, ҡыҙ һәм малай',
    '👩‍👩‍👦‍👦' => 'ғаилә: ҡатын, ҡатын, малай һәм малай',
    '👩‍👩‍👧‍👧' => 'ғаилә: ҡатын, ҡатын, ҡыҙ һәм ҡыҙ',
    '🧑‍🧑‍🧒‍🧒' => 'ғаилә: ике өлкән һәм ике бала',
    '🚶🏻‍♀‍➡' => 'йөрөүсе ҡатын-ҡыҙ: аҡһыл тән төҫө һәм уңға ҡарай',
    '🚶🏼‍♀‍➡' => 'йөрөүсе ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🚶🏽‍♀‍➡' => 'йөрөүсе ҡатын-ҡыҙ: уртаса тән төҫө һәм уңға ҡарай',
    '🚶🏾‍♀‍➡' => 'йөрөүсе ҡатын-ҡыҙ: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🚶🏿‍♀‍➡' => 'йөрөүсе ҡатын-ҡыҙ: ҡара тән төҫө һәм уңға ҡарай',
    '🚶🏻‍♂‍➡' => 'йөрөүсе ир-ат: аҡһыл тән төҫө һәм уңға ҡарай',
    '🚶🏼‍♂‍➡' => 'йөрөүсе ир-ат: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🚶🏽‍♂‍➡' => 'йөрөүсе ир-ат: уртаса тән төҫө һәм уңға ҡарай',
    '🚶🏾‍♂‍➡' => 'йөрөүсе ир-ат: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🚶🏿‍♂‍➡' => 'йөрөүсе ир-ат: ҡара тән төҫө һәм уңға ҡарай',
    '🧎🏻‍♀‍➡' => 'тубыҡланған ҡатын: аҡһыл тән төҫө һәм уңға ҡарай',
    '🧎🏼‍♀‍➡' => 'тубыҡланған ҡатын: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🧎🏽‍♀‍➡' => 'тубыҡланған ҡатын: уртаса тән төҫө һәм уңға ҡарай',
    '🧎🏾‍♀‍➡' => 'тубыҡланған ҡатын: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🧎🏿‍♀‍➡' => 'тубыҡланған ҡатын: ҡара тән төҫө һәм уңға ҡарай',
    '🧎🏻‍♂‍➡' => 'тубыҡланған ир: аҡһыл тән төҫө һәм уңға ҡарай',
    '🧎🏼‍♂‍➡' => 'тубыҡланған ир: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🧎🏽‍♂‍➡' => 'тубыҡланған ир: уртаса тән төҫө һәм уңға ҡарай',
    '🧎🏾‍♂‍➡' => 'тубыҡланған ир: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🧎🏿‍♂‍➡' => 'тубыҡланған ир: ҡара тән төҫө һәм уңға ҡарай',
    '🧑🏻‍🦯‍➡' => 'аҡ таяҡлы кеше: аҡһыл тән төҫө һәм уңға ҡарай',
    '🧑🏼‍🦯‍➡' => 'аҡ таяҡлы кеше: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🧑🏽‍🦯‍➡' => 'аҡ таяҡлы кеше: уртаса тән төҫө һәм уңға ҡарай',
    '🧑🏾‍🦯‍➡' => 'аҡ таяҡлы кеше: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🧑🏿‍🦯‍➡' => 'аҡ таяҡлы кеше: ҡара тән төҫө һәм уңға ҡарай',
    '👨🏻‍🦯‍➡' => 'аҡ таяҡлы ир: аҡһыл тән төҫө һәм уңға ҡарай',
    '👨🏼‍🦯‍➡' => 'аҡ таяҡлы ир: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '👨🏽‍🦯‍➡' => 'аҡ таяҡлы ир: уртаса тән төҫө һәм уңға ҡарай',
    '👨🏾‍🦯‍➡' => 'аҡ таяҡлы ир: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '👨🏿‍🦯‍➡' => 'аҡ таяҡлы ир: ҡара тән төҫө һәм уңға ҡарай',
    '👩🏻‍🦯‍➡' => 'аҡ таяҡлы ҡатын: аҡһыл тән төҫө һәм уңға ҡарай',
    '👩🏼‍🦯‍➡' => 'аҡ таяҡлы ҡатын: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '👩🏽‍🦯‍➡' => 'аҡ таяҡлы ҡатын: уртаса тән төҫө һәм уңға ҡарай',
    '👩🏾‍🦯‍➡' => 'аҡ таяҡлы ҡатын: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '👩🏿‍🦯‍➡' => 'аҡ таяҡлы ҡатын: ҡара тән төҫө һәм уңға ҡарай',
    '🧑🏻‍🦼‍➡' => 'моторлы инвалид коляскаһындағы кеше: аҡһыл тән төҫө һәм уңға ҡарай',
    '🧑🏼‍🦼‍➡' => 'моторлы инвалид коляскаһындағы кеше: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🧑🏽‍🦼‍➡' => 'моторлы инвалид коляскаһындағы кеше: уртаса тән төҫө һәм уңға ҡарай',
    '🧑🏾‍🦼‍➡' => 'моторлы инвалид коляскаһындағы кеше: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🧑🏿‍🦼‍➡' => 'моторлы инвалид коляскаһындағы кеше: ҡара тән төҫө һәм уңға ҡарай',
    '👨🏻‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ир: аҡһыл тән төҫө һәм уңға ҡарай',
    '👨🏼‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ир: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '👨🏽‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ир: уртаса тән төҫө һәм уңға ҡарай',
    '👨🏾‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ир: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '👨🏿‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ир: ҡара тән төҫө һәм уңға ҡарай',
    '👩🏻‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ҡатын: аҡһыл тән төҫө һәм уңға ҡарай',
    '👩🏼‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ҡатын: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '👩🏽‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ҡатын: уртаса тән төҫө һәм уңға ҡарай',
    '👩🏾‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ҡатын: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '👩🏿‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ҡатын: ҡара тән төҫө һәм уңға ҡарай',
    '🧑🏻‍🦽‍➡' => 'инвалид коляскаһындағы кеше: аҡһыл тән төҫө һәм уңға ҡарай',
    '🧑🏼‍🦽‍➡' => 'инвалид коляскаһындағы кеше: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🧑🏽‍🦽‍➡' => 'инвалид коляскаһындағы кеше: уртаса тән төҫө һәм уңға ҡарай',
    '🧑🏾‍🦽‍➡' => 'инвалид коляскаһындағы кеше: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🧑🏿‍🦽‍➡' => 'инвалид коляскаһындағы кеше: ҡара тән төҫө һәм уңға ҡарай',
    '👨🏻‍🦽‍➡' => 'инвалид коляскаһындағы ир-ат: аҡһыл тән төҫө һәм уңға ҡарай',
    '👨🏼‍🦽‍➡' => 'инвалид коляскаһындағы ир-ат: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '👨🏽‍🦽‍➡' => 'инвалид коляскаһындағы ир-ат: уртаса тән төҫө һәм уңға ҡарай',
    '👨🏾‍🦽‍➡' => 'инвалид коляскаһындағы ир-ат: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '👨🏿‍🦽‍➡' => 'инвалид коляскаһындағы ир-ат: ҡара тән төҫө һәм уңға ҡарай',
    '👩🏻‍🦽‍➡' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: аҡһыл тән төҫө һәм уңға ҡарай',
    '👩🏼‍🦽‍➡' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '👩🏽‍🦽‍➡' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: уртаса тән төҫө һәм уңға ҡарай',
    '👩🏾‍🦽‍➡' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '👩🏿‍🦽‍➡' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: ҡара тән төҫө һәм уңға ҡарай',
    '🏃🏻‍♀‍➡' => 'йүгереүсе ҡатын: аҡһыл тән төҫө һәм уңға ҡарай',
    '🏃🏼‍♀‍➡' => 'йүгереүсе ҡатын: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🏃🏽‍♀‍➡' => 'йүгереүсе ҡатын: уртаса тән төҫө һәм уңға ҡарай',
    '🏃🏾‍♀‍➡' => 'йүгереүсе ҡатын: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🏃🏿‍♀‍➡' => 'йүгереүсе ҡатын: ҡара тән төҫө һәм уңға ҡарай',
    '🏃🏻‍♂‍➡' => 'йүгереүсе ир: аҡһыл тән төҫө һәм уңға ҡарай',
    '🏃🏼‍♂‍➡' => 'йүгереүсе ир: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🏃🏽‍♂‍➡' => 'йүгереүсе ир: уртаса тән төҫө һәм уңға ҡарай',
    '🏃🏾‍♂‍➡' => 'йүгереүсе ир: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🏃🏿‍♂‍➡' => 'йүгереүсе ир: ҡара тән төҫө һәм уңға ҡарай',
    '🫱🏻‍🫲🏼' => 'ҡул бирешеү: аҡһыл тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🫱🏻‍🫲🏽' => 'ҡул бирешеү: аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🫱🏻‍🫲🏾' => 'ҡул бирешеү: аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🫱🏻‍🫲🏿' => 'ҡул бирешеү: аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🫱🏼‍🫲🏻' => 'ҡул бирешеү: уртаса аҡһыл тән төҫө һәм аҡһыл тән төҫө',
    '🫱🏼‍🫲🏽' => 'ҡул бирешеү: уртаса аҡһыл тән төҫө һәм уртаса тән төҫө',
    '🫱🏼‍🫲🏾' => 'ҡул бирешеү: уртаса аҡһыл тән төҫө һәм уртаса ҡара тән төҫө',
    '🫱🏼‍🫲🏿' => 'ҡул бирешеү: уртаса аҡһыл тән төҫө һәм ҡара тән төҫө',
    '🫱🏽‍🫲🏻' => 'ҡул бирешеү: уртаса тән төҫө һәм аҡһыл тән төҫө',
    '🫱🏽‍🫲🏼' => 'ҡул бирешеү: уртаса тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🫱🏽‍🫲🏾' => 'ҡул бирешеү: уртаса тән төҫө һәм уртаса ҡара тән төҫө',
    '🫱🏽‍🫲🏿' => 'ҡул бирешеү: уртаса тән төҫө һәм ҡара тән төҫө',
    '🫱🏾‍🫲🏻' => 'ҡул бирешеү: уртаса ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🫱🏾‍🫲🏼' => 'ҡул бирешеү: уртаса ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🫱🏾‍🫲🏽' => 'ҡул бирешеү: уртаса ҡара тән төҫө һәм уртаса тән төҫө',
    '🫱🏾‍🫲🏿' => 'ҡул бирешеү: уртаса ҡара тән төҫө һәм ҡара тән төҫө',
    '🫱🏿‍🫲🏻' => 'ҡул бирешеү: ҡара тән төҫө һәм аҡһыл тән төҫө',
    '🫱🏿‍🫲🏼' => 'ҡул бирешеү: ҡара тән төҫө һәм уртаса аҡһыл тән төҫө',
    '🫱🏿‍🫲🏽' => 'ҡул бирешеү: ҡара тән төҫө һәм уртаса тән төҫө',
    '🫱🏿‍🫲🏾' => 'ҡул бирешеү: ҡара тән төҫө һәм уртаса ҡара тән төҫө',
    '🚶‍♀‍➡' => 'йөрөүсе ҡатын-ҡыҙ: уңға ҡарай',
    '🚶‍♂‍➡' => 'йөрөүсе ир-ат: уңға ҡарай',
    '🧎‍♀‍➡' => 'тубыҡланған ҡатын: уңға ҡарай',
    '🧎‍♂‍➡' => 'тубыҡланған ир: уңға ҡарай',
    '🧑‍🦯‍➡' => 'аҡ таяҡлы кеше: уңға ҡарай',
    '👨‍🦯‍➡' => 'аҡ таяҡлы ир: уңға ҡарай',
    '👩‍🦯‍➡' => 'аҡ таяҡлы ҡатын: уңға ҡарай',
    '🧑‍🦼‍➡' => 'моторлы инвалид коляскаһындағы кеше: уңға ҡарай',
    '👨‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ир: уңға ҡарай',
    '👩‍🦼‍➡' => 'моторлы инвалид коляскаһындағы ҡатын: уңға ҡарай',
    '🧑‍🦽‍➡' => 'инвалид коляскаһындағы кеше: уңға ҡарай',
    '👨‍🦽‍➡' => 'инвалид коляскаһындағы ир-ат: уңға ҡарай',
    '👩‍🦽‍➡' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: уңға ҡарай',
    '🏃‍♀‍➡' => 'йүгереүсе ҡатын: уңға ҡарай',
    '🏃‍♂‍➡' => 'йүгереүсе ир: уңға ҡарай',
    '👩‍❤‍👨' => 'йөрәкле пар: ҡатын һәм ир',
    '👨‍❤‍👨' => 'йөрәкле пар: ир һәм ир',
    '👩‍❤‍👩' => 'йөрәкле пар: ҡатын һәм ҡатын',
    '👨‍👩‍👦' => 'ғаилә: ир, ҡатын һәм малай',
    '👨‍👩‍👧' => 'ғаилә: ир, ҡатын һәм ҡыҙ',
    '👨‍👨‍👦' => 'ғаилә: ир, ир һәм малай',
    '👨‍👨‍👧' => 'ғаилә: ир, ир һәм ҡыҙ',
    '👩‍👩‍👦' => 'ғаилә: ҡатын, ҡатын һәм малай',
    '👩‍👩‍👧' => 'ғаилә: ҡатын, ҡатын һәм ҡыҙ',
    '👨‍👦‍👦' => 'ғаилә: ир, малай һәм малай',
    '👨‍👧‍👦' => 'ғаилә: ир, ҡыҙ һәм малай',
    '👨‍👧‍👧' => 'ғаилә: ир, ҡыҙ һәм ҡыҙ',
    '👩‍👦‍👦' => 'ғаилә: ҡатын, малай һәм малай',
    '👩‍👧‍👦' => 'ғаилә: ҡатын, ҡыҙ һәм малай',
    '👩‍👧‍👧' => 'ғаилә: ҡатын, ҡыҙ һәм ҡыҙ',
    '🧑‍🤝‍🧑' => 'ҡул тотошҡан кешеләр',
    '🧑‍🧑‍🧒' => 'ғаилә: ике өлкән һәм бала',
    '🧑‍🧒‍🧒' => 'ғаилә: өлкән һәм ике бала',
    '🧑🏻‍🦰' => 'өлкән: аҡһыл тән төҫө һәм ерән сәс',
    '🧑🏼‍🦰' => 'өлкән: уртаса аҡһыл тән төҫө һәм ерән сәс',
    '🧑🏽‍🦰' => 'өлкән: уртаса тән төҫө һәм ерән сәс',
    '🧑🏾‍🦰' => 'өлкән: уртаса ҡара тән төҫө һәм ерән сәс',
    '🧑🏿‍🦰' => 'өлкән: ҡара тән төҫө һәм ерән сәс',
    '🧑🏻‍🦱' => 'өлкән: аҡһыл тән төҫө һәм бөҙрә сәс',
    '🧑🏼‍🦱' => 'өлкән: уртаса аҡһыл тән төҫө һәм бөҙрә сәс',
    '🧑🏽‍🦱' => 'өлкән: уртаса тән төҫө һәм бөҙрә сәс',
    '🧑🏾‍🦱' => 'өлкән: уртаса ҡара тән төҫө һәм бөҙрә сәс',
    '🧑🏿‍🦱' => 'өлкән: ҡара тән төҫө һәм бөҙрә сәс',
    '🧑🏻‍🦳' => 'өлкән: аҡһыл тән төҫө һәм аҡ сәс',
    '🧑🏼‍🦳' => 'өлкән: уртаса аҡһыл тән төҫө һәм аҡ сәс',
    '🧑🏽‍🦳' => 'өлкән: уртаса тән төҫө һәм аҡ сәс',
    '🧑🏾‍🦳' => 'өлкән: уртаса ҡара тән төҫө һәм аҡ сәс',
    '🧑🏿‍🦳' => 'өлкән: ҡара тән төҫө һәм аҡ сәс',
    '🧑🏻‍🦲' => 'өлкән: аҡһыл тән төҫө һәм таҙ',
    '🧑🏼‍🦲' => 'өлкән: уртаса аҡһыл тән төҫө һәм таҙ',
    '🧑🏽‍🦲' => 'өлкән: уртаса тән төҫө һәм таҙ',
    '🧑🏾‍🦲' => 'өлкән: уртаса ҡара тән төҫө һәм таҙ',
    '🧑🏿‍🦲' => 'өлкән: ҡара тән төҫө һәм таҙ',
    '🧑🏻‍🩰' => 'балет бейеүсеһе: аҡһыл тән төҫө',
    '🧑🏼‍🩰' => 'балет бейеүсеһе: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🩰' => 'балет бейеүсеһе: уртаса тән төҫө',
    '🧑🏾‍🩰' => 'балет бейеүсеһе: уртаса ҡара тән төҫө',
    '🧑🏿‍🩰' => 'балет бейеүсеһе: ҡара тән төҫө',
    '🧔🏻‍♂' => 'һаҡаллы ир: аҡһыл тән төҫө',
    '🧔🏼‍♂' => 'һаҡаллы ир: уртаса аҡһыл тән төҫө',
    '🧔🏽‍♂' => 'һаҡаллы ир: уртаса тән төҫө',
    '🧔🏾‍♂' => 'һаҡаллы ир: уртаса ҡара тән төҫө',
    '🧔🏿‍♂' => 'һаҡаллы ир: ҡара тән төҫө',
    '👱🏻‍♂' => 'ир: аҡһыл тән төҫө һәм аҡһыл сәсле',
    '👱🏼‍♂' => 'ир: уртаса аҡһыл тән төҫө һәм аҡһыл сәсле',
    '👱🏽‍♂' => 'ир: уртаса тән төҫө һәм аҡһыл сәсле',
    '👱🏾‍♂' => 'ир: уртаса ҡара тән төҫө һәм аҡһыл сәсле',
    '👱🏿‍♂' => 'ир: ҡара тән төҫө һәм аҡһыл сәсле',
    '👨🏻‍🦰' => 'ир: аҡһыл тән төҫө һәм ерән сәс',
    '👨🏼‍🦰' => 'ир: уртаса аҡһыл тән төҫө һәм ерән сәс',
    '👨🏽‍🦰' => 'ир: уртаса тән төҫө һәм ерән сәс',
    '👨🏾‍🦰' => 'ир: уртаса ҡара тән төҫө һәм ерән сәс',
    '👨🏿‍🦰' => 'ир: ҡара тән төҫө һәм ерән сәс',
    '👨🏻‍🦱' => 'ир: аҡһыл тән төҫө һәм бөҙрә сәс',
    '👨🏼‍🦱' => 'ир: уртаса аҡһыл тән төҫө һәм бөҙрә сәс',
    '👨🏽‍🦱' => 'ир: уртаса тән төҫө һәм бөҙрә сәс',
    '👨🏾‍🦱' => 'ир: уртаса ҡара тән төҫө һәм бөҙрә сәс',
    '👨🏿‍🦱' => 'ир: ҡара тән төҫө һәм бөҙрә сәс',
    '👨🏻‍🦳' => 'ир: аҡһыл тән төҫө һәм аҡ сәс',
    '👨🏼‍🦳' => 'ир: уртаса аҡһыл тән төҫө һәм аҡ сәс',
    '👨🏽‍🦳' => 'ир: уртаса тән төҫө һәм аҡ сәс',
    '👨🏾‍🦳' => 'ир: уртаса ҡара тән төҫө һәм аҡ сәс',
    '👨🏿‍🦳' => 'ир: ҡара тән төҫө һәм аҡ сәс',
    '👨🏻‍🦲' => 'ир: аҡһыл тән төҫө һәм таҙ',
    '👨🏼‍🦲' => 'ир: уртаса аҡһыл тән төҫө һәм таҙ',
    '👨🏽‍🦲' => 'ир: уртаса тән төҫө һәм таҙ',
    '👨🏾‍🦲' => 'ир: уртаса ҡара тән төҫө һәм таҙ',
    '👨🏿‍🦲' => 'ир: ҡара тән төҫө һәм таҙ',
    '🧔🏻‍♀' => 'һаҡаллы ҡатын: аҡһыл тән төҫө',
    '🧔🏼‍♀' => 'һаҡаллы ҡатын: уртаса аҡһыл тән төҫө',
    '🧔🏽‍♀' => 'һаҡаллы ҡатын: уртаса тән төҫө',
    '🧔🏾‍♀' => 'һаҡаллы ҡатын: уртаса ҡара тән төҫө',
    '🧔🏿‍♀' => 'һаҡаллы ҡатын: ҡара тән төҫө',
    '👱🏻‍♀' => 'ҡатын: аҡһыл тән төҫө һәм аҡһыл сәсле',
    '👱🏼‍♀' => 'ҡатын: уртаса аҡһыл тән төҫө һәм аҡһыл сәсле',
    '👱🏽‍♀' => 'ҡатын: уртаса тән төҫө һәм аҡһыл сәсле',
    '👱🏾‍♀' => 'ҡатын: уртаса ҡара тән төҫө һәм аҡһыл сәсле',
    '👱🏿‍♀' => 'ҡатын: ҡара тән төҫө һәм аҡһыл сәсле',
    '👩🏻‍🦰' => 'ҡатын: аҡһыл тән төҫө һәм ерән сәс',
    '👩🏼‍🦰' => 'ҡатын: уртаса аҡһыл тән төҫө һәм ерән сәс',
    '👩🏽‍🦰' => 'ҡатын: уртаса тән төҫө һәм ерән сәс',
    '👩🏾‍🦰' => 'ҡатын: уртаса ҡара тән төҫө һәм ерән сәс',
    '👩🏿‍🦰' => 'ҡатын: ҡара тән төҫө һәм ерән сәс',
    '👩🏻‍🦱' => 'ҡатын: аҡһыл тән төҫө һәм бөҙрә сәс',
    '👩🏼‍🦱' => 'ҡатын: уртаса аҡһыл тән төҫө һәм бөҙрә сәс',
    '👩🏽‍🦱' => 'ҡатын: уртаса тән төҫө һәм бөҙрә сәс',
    '👩🏾‍🦱' => 'ҡатын: уртаса ҡара тән төҫө һәм бөҙрә сәс',
    '👩🏿‍🦱' => 'ҡатын: ҡара тән төҫө һәм бөҙрә сәс',
    '👩🏻‍🦳' => 'ҡатын: аҡһыл тән төҫө һәм аҡ сәс',
    '👩🏼‍🦳' => 'ҡатын: уртаса аҡһыл тән төҫө һәм аҡ сәс',
    '👩🏽‍🦳' => 'ҡатын: уртаса тән төҫө һәм аҡ сәс',
    '👩🏾‍🦳' => 'ҡатын: уртаса ҡара тән төҫө һәм аҡ сәс',
    '👩🏿‍🦳' => 'ҡатын: ҡара тән төҫө һәм аҡ сәс',
    '👩🏻‍🦲' => 'ҡатын: аҡһыл тән төҫө һәм таҙ',
    '👩🏼‍🦲' => 'ҡатын: уртаса аҡһыл тән төҫө һәм таҙ',
    '👩🏽‍🦲' => 'ҡатын: уртаса тән төҫө һәм таҙ',
    '👩🏾‍🦲' => 'ҡатын: уртаса ҡара тән төҫө һәм таҙ',
    '👩🏿‍🦲' => 'ҡатын: ҡара тән төҫө һәм таҙ',
    '🙍🏻‍♂' => 'йөҙ сирыусы ир-ат: аҡһыл тән төҫө',
    '🙍🏼‍♂' => 'йөҙ сирыусы ир-ат: уртаса аҡһыл тән төҫө',
    '🙍🏽‍♂' => 'йөҙ сирыусы ир-ат: уртаса тән төҫө',
    '🙍🏾‍♂' => 'йөҙ сирыусы ир-ат: уртаса ҡара тән төҫө',
    '🙍🏿‍♂' => 'йөҙ сирыусы ир-ат: ҡара тән төҫө',
    '🙍🏻‍♀' => 'йөҙ сирыусы ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🙍🏼‍♀' => 'йөҙ сирыусы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🙍🏽‍♀' => 'йөҙ сирыусы ҡатын-ҡыҙ: уртаса тән төҫө',
    '🙍🏾‍♀' => 'йөҙ сирыусы ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🙍🏿‍♀' => 'йөҙ сирыусы ҡатын-ҡыҙ: ҡара тән төҫө',
    '🙎🏻‍♂' => 'ирен сирыусы ир-ат: аҡһыл тән төҫө',
    '🙎🏼‍♂' => 'ирен сирыусы ир-ат: уртаса аҡһыл тән төҫө',
    '🙎🏽‍♂' => 'ирен сирыусы ир-ат: уртаса тән төҫө',
    '🙎🏾‍♂' => 'ирен сирыусы ир-ат: уртаса ҡара тән төҫө',
    '🙎🏿‍♂' => 'ирен сирыусы ир-ат: ҡара тән төҫө',
    '🙎🏻‍♀' => 'ирен сирыусы ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🙎🏼‍♀' => 'ирен сирыусы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🙎🏽‍♀' => 'ирен сирыусы ҡатын-ҡыҙ: уртаса тән төҫө',
    '🙎🏾‍♀' => 'ирен сирыусы ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🙎🏿‍♀' => 'ирен сирыусы ҡатын-ҡыҙ: ҡара тән төҫө',
    '🙅🏻‍♂' => 'ЮҠ тип ымлаусы ир-ат: аҡһыл тән төҫө',
    '🙅🏼‍♂' => 'ЮҠ тип ымлаусы ир-ат: уртаса аҡһыл тән төҫө',
    '🙅🏽‍♂' => 'ЮҠ тип ымлаусы ир-ат: уртаса тән төҫө',
    '🙅🏾‍♂' => 'ЮҠ тип ымлаусы ир-ат: уртаса ҡара тән төҫө',
    '🙅🏿‍♂' => 'ЮҠ тип ымлаусы ир-ат: ҡара тән төҫө',
    '🙅🏻‍♀' => 'ЮҠ тип ымлаусы ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🙅🏼‍♀' => 'ЮҠ тип ымлаусы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🙅🏽‍♀' => 'ЮҠ тип ымлаусы ҡатын-ҡыҙ: уртаса тән төҫө',
    '🙅🏾‍♀' => 'ЮҠ тип ымлаусы ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🙅🏿‍♀' => 'ЮҠ тип ымлаусы ҡатын-ҡыҙ: ҡара тән төҫө',
    '🙆🏻‍♂' => 'ЯҠШЫ тип ымлаусы ир-ат: аҡһыл тән төҫө',
    '🙆🏼‍♂' => 'ЯҠШЫ тип ымлаусы ир-ат: уртаса аҡһыл тән төҫө',
    '🙆🏽‍♂' => 'ЯҠШЫ тип ымлаусы ир-ат: уртаса тән төҫө',
    '🙆🏾‍♂' => 'ЯҠШЫ тип ымлаусы ир-ат: уртаса ҡара тән төҫө',
    '🙆🏿‍♂' => 'ЯҠШЫ тип ымлаусы ир-ат: ҡара тән төҫө',
    '🙆🏻‍♀' => 'ЯҠШЫ тип ымлаусы ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🙆🏼‍♀' => 'ЯҠШЫ тип ымлаусы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🙆🏽‍♀' => 'ЯҠШЫ тип ымлаусы ҡатын-ҡыҙ: уртаса тән төҫө',
    '🙆🏾‍♀' => 'ЯҠШЫ тип ымлаусы ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🙆🏿‍♀' => 'ЯҠШЫ тип ымлаусы ҡатын-ҡыҙ: ҡара тән төҫө',
    '💁🏻‍♂' => 'ҡул һуҙған ир: аҡһыл тән төҫө',
    '💁🏼‍♂' => 'ҡул һуҙған ир: уртаса аҡһыл тән төҫө',
    '💁🏽‍♂' => 'ҡул һуҙған ир: уртаса тән төҫө',
    '💁🏾‍♂' => 'ҡул һуҙған ир: уртаса ҡара тән төҫө',
    '💁🏿‍♂' => 'ҡул һуҙған ир: ҡара тән төҫө',
    '💁🏻‍♀' => 'ҡул һуҙған ҡатын: аҡһыл тән төҫө',
    '💁🏼‍♀' => 'ҡул һуҙған ҡатын: уртаса аҡһыл тән төҫө',
    '💁🏽‍♀' => 'ҡул һуҙған ҡатын: уртаса тән төҫө',
    '💁🏾‍♀' => 'ҡул һуҙған ҡатын: уртаса ҡара тән төҫө',
    '💁🏿‍♀' => 'ҡул һуҙған ҡатын: ҡара тән төҫө',
    '🙋🏻‍♂' => 'ҡул күтәреүсе ир-ат: аҡһыл тән төҫө',
    '🙋🏼‍♂' => 'ҡул күтәреүсе ир-ат: уртаса аҡһыл тән төҫө',
    '🙋🏽‍♂' => 'ҡул күтәреүсе ир-ат: уртаса тән төҫө',
    '🙋🏾‍♂' => 'ҡул күтәреүсе ир-ат: уртаса ҡара тән төҫө',
    '🙋🏿‍♂' => 'ҡул күтәреүсе ир-ат: ҡара тән төҫө',
    '🙋🏻‍♀' => 'ҡул күтәреүсе ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🙋🏼‍♀' => 'ҡул күтәреүсе ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🙋🏽‍♀' => 'ҡул күтәреүсе ҡатын-ҡыҙ: уртаса тән төҫө',
    '🙋🏾‍♀' => 'ҡул күтәреүсе ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🙋🏿‍♀' => 'ҡул күтәреүсе ҡатын-ҡыҙ: ҡара тән төҫө',
    '🧏🏻‍♂' => 'һаңғырау ир: аҡһыл тән төҫө',
    '🧏🏼‍♂' => 'һаңғырау ир: уртаса аҡһыл тән төҫө',
    '🧏🏽‍♂' => 'һаңғырау ир: уртаса тән төҫө',
    '🧏🏾‍♂' => 'һаңғырау ир: уртаса ҡара тән төҫө',
    '🧏🏿‍♂' => 'һаңғырау ир: ҡара тән төҫө',
    '🧏🏻‍♀' => 'һаңғырау ҡатын: аҡһыл тән төҫө',
    '🧏🏼‍♀' => 'һаңғырау ҡатын: уртаса аҡһыл тән төҫө',
    '🧏🏽‍♀' => 'һаңғырау ҡатын: уртаса тән төҫө',
    '🧏🏾‍♀' => 'һаңғырау ҡатын: уртаса ҡара тән төҫө',
    '🧏🏿‍♀' => 'һаңғырау ҡатын: ҡара тән төҫө',
    '🙇🏻‍♂' => 'баш эйеүсе ир-ат: аҡһыл тән төҫө',
    '🙇🏼‍♂' => 'баш эйеүсе ир-ат: уртаса аҡһыл тән төҫө',
    '🙇🏽‍♂' => 'баш эйеүсе ир-ат: уртаса тән төҫө',
    '🙇🏾‍♂' => 'баш эйеүсе ир-ат: уртаса ҡара тән төҫө',
    '🙇🏿‍♂' => 'баш эйеүсе ир-ат: ҡара тән төҫө',
    '🙇🏻‍♀' => 'баш эйеүсе ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🙇🏼‍♀' => 'баш эйеүсе ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🙇🏽‍♀' => 'баш эйеүсе ҡатын-ҡыҙ: уртаса тән төҫө',
    '🙇🏾‍♀' => 'баш эйеүсе ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🙇🏿‍♀' => 'баш эйеүсе ҡатын-ҡыҙ: ҡара тән төҫө',
    '🤦🏻‍♂' => 'йөҙөн ҡул менән ҡаплаусы ир-ат: аҡһыл тән төҫө',
    '🤦🏼‍♂' => 'йөҙөн ҡул менән ҡаплаусы ир-ат: уртаса аҡһыл тән төҫө',
    '🤦🏽‍♂' => 'йөҙөн ҡул менән ҡаплаусы ир-ат: уртаса тән төҫө',
    '🤦🏾‍♂' => 'йөҙөн ҡул менән ҡаплаусы ир-ат: уртаса ҡара тән төҫө',
    '🤦🏿‍♂' => 'йөҙөн ҡул менән ҡаплаусы ир-ат: ҡара тән төҫө',
    '🤦🏻‍♀' => 'йөҙөн ҡул менән ҡаплаусы ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🤦🏼‍♀' => 'йөҙөн ҡул менән ҡаплаусы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🤦🏽‍♀' => 'йөҙөн ҡул менән ҡаплаусы ҡатын-ҡыҙ: уртаса тән төҫө',
    '🤦🏾‍♀' => 'йөҙөн ҡул менән ҡаплаусы ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🤦🏿‍♀' => 'йөҙөн ҡул менән ҡаплаусы ҡатын-ҡыҙ: ҡара тән төҫө',
    '🤷🏻‍♂' => 'ир яурынын һикертә: аҡһыл тән төҫө',
    '🤷🏼‍♂' => 'ир яурынын һикертә: уртаса аҡһыл тән төҫө',
    '🤷🏽‍♂' => 'ир яурынын һикертә: уртаса тән төҫө',
    '🤷🏾‍♂' => 'ир яурынын һикертә: уртаса ҡара тән төҫө',
    '🤷🏿‍♂' => 'ир яурынын һикертә: ҡара тән төҫө',
    '🤷🏻‍♀' => 'ҡатын яурынын һикертә: аҡһыл тән төҫө',
    '🤷🏼‍♀' => 'ҡатын яурынын һикертә: уртаса аҡһыл тән төҫө',
    '🤷🏽‍♀' => 'ҡатын яурынын һикертә: уртаса тән төҫө',
    '🤷🏾‍♀' => 'ҡатын яурынын һикертә: уртаса ҡара тән төҫө',
    '🤷🏿‍♀' => 'ҡатын яурынын һикертә: ҡара тән төҫө',
    '🧑🏻‍⚕' => 'һаулыҡ һаҡлау хеҙмәткәре: аҡһыл тән төҫө',
    '🧑🏼‍⚕' => 'һаулыҡ һаҡлау хеҙмәткәре: уртаса аҡһыл тән төҫө',
    '🧑🏽‍⚕' => 'һаулыҡ һаҡлау хеҙмәткәре: уртаса тән төҫө',
    '🧑🏾‍⚕' => 'һаулыҡ һаҡлау хеҙмәткәре: уртаса ҡара тән төҫө',
    '🧑🏿‍⚕' => 'һаулыҡ һаҡлау хеҙмәткәре: ҡара тән төҫө',
    '👨🏻‍⚕' => 'ир-ат медицина хеҙмәткәре: аҡһыл тән төҫө',
    '👨🏼‍⚕' => 'ир-ат медицина хеҙмәткәре: уртаса аҡһыл тән төҫө',
    '👨🏽‍⚕' => 'ир-ат медицина хеҙмәткәре: уртаса тән төҫө',
    '👨🏾‍⚕' => 'ир-ат медицина хеҙмәткәре: уртаса ҡара тән төҫө',
    '👨🏿‍⚕' => 'ир-ат медицина хеҙмәткәре: ҡара тән төҫө',
    '👩🏻‍⚕' => 'ҡатын-ҡыҙ медицина хеҙмәткәре: аҡһыл тән төҫө',
    '👩🏼‍⚕' => 'ҡатын-ҡыҙ медицина хеҙмәткәре: уртаса аҡһыл тән төҫө',
    '👩🏽‍⚕' => 'ҡатын-ҡыҙ медицина хеҙмәткәре: уртаса тән төҫө',
    '👩🏾‍⚕' => 'ҡатын-ҡыҙ медицина хеҙмәткәре: уртаса ҡара тән төҫө',
    '👩🏿‍⚕' => 'ҡатын-ҡыҙ медицина хеҙмәткәре: ҡара тән төҫө',
    '🧑🏻‍🎓' => 'студент: аҡһыл тән төҫө',
    '🧑🏼‍🎓' => 'студент: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🎓' => 'студент: уртаса тән төҫө',
    '🧑🏾‍🎓' => 'студент: уртаса ҡара тән төҫө',
    '🧑🏿‍🎓' => 'студент: ҡара тән төҫө',
    '👨🏻‍🎓' => 'ир-ат студент: аҡһыл тән төҫө',
    '👨🏼‍🎓' => 'ир-ат студент: уртаса аҡһыл тән төҫө',
    '👨🏽‍🎓' => 'ир-ат студент: уртаса тән төҫө',
    '👨🏾‍🎓' => 'ир-ат студент: уртаса ҡара тән төҫө',
    '👨🏿‍🎓' => 'ир-ат студент: ҡара тән төҫө',
    '👩🏻‍🎓' => 'ҡатын-ҡыҙ студент: аҡһыл тән төҫө',
    '👩🏼‍🎓' => 'ҡатын-ҡыҙ студент: уртаса аҡһыл тән төҫө',
    '👩🏽‍🎓' => 'ҡатын-ҡыҙ студент: уртаса тән төҫө',
    '👩🏾‍🎓' => 'ҡатын-ҡыҙ студент: уртаса ҡара тән төҫө',
    '👩🏿‍🎓' => 'ҡатын-ҡыҙ студент: ҡара тән төҫө',
    '🧑🏻‍🏫' => 'уҡытыусы: аҡһыл тән төҫө',
    '🧑🏼‍🏫' => 'уҡытыусы: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🏫' => 'уҡытыусы: уртаса тән төҫө',
    '🧑🏾‍🏫' => 'уҡытыусы: уртаса ҡара тән төҫө',
    '🧑🏿‍🏫' => 'уҡытыусы: ҡара тән төҫө',
    '👨🏻‍🏫' => 'ир-ат уҡытыусы: аҡһыл тән төҫө',
    '👨🏼‍🏫' => 'ир-ат уҡытыусы: уртаса аҡһыл тән төҫө',
    '👨🏽‍🏫' => 'ир-ат уҡытыусы: уртаса тән төҫө',
    '👨🏾‍🏫' => 'ир-ат уҡытыусы: уртаса ҡара тән төҫө',
    '👨🏿‍🏫' => 'ир-ат уҡытыусы: ҡара тән төҫө',
    '👩🏻‍🏫' => 'ҡатын-ҡыҙ уҡытыусы: аҡһыл тән төҫө',
    '👩🏼‍🏫' => 'ҡатын-ҡыҙ уҡытыусы: уртаса аҡһыл тән төҫө',
    '👩🏽‍🏫' => 'ҡатын-ҡыҙ уҡытыусы: уртаса тән төҫө',
    '👩🏾‍🏫' => 'ҡатын-ҡыҙ уҡытыусы: уртаса ҡара тән төҫө',
    '👩🏿‍🏫' => 'ҡатын-ҡыҙ уҡытыусы: ҡара тән төҫө',
    '🧑🏻‍⚖' => 'хөкөмсө: аҡһыл тән төҫө',
    '🧑🏼‍⚖' => 'хөкөмсө: уртаса аҡһыл тән төҫө',
    '🧑🏽‍⚖' => 'хөкөмсө: уртаса тән төҫө',
    '🧑🏾‍⚖' => 'хөкөмсө: уртаса ҡара тән төҫө',
    '🧑🏿‍⚖' => 'хөкөмсө: ҡара тән төҫө',
    '👨🏻‍⚖' => 'ир-ат хөкөмсө: аҡһыл тән төҫө',
    '👨🏼‍⚖' => 'ир-ат хөкөмсө: уртаса аҡһыл тән төҫө',
    '👨🏽‍⚖' => 'ир-ат хөкөмсө: уртаса тән төҫө',
    '👨🏾‍⚖' => 'ир-ат хөкөмсө: уртаса ҡара тән төҫө',
    '👨🏿‍⚖' => 'ир-ат хөкөмсө: ҡара тән төҫө',
    '👩🏻‍⚖' => 'ҡатын-ҡыҙ хөкөмсө: аҡһыл тән төҫө',
    '👩🏼‍⚖' => 'ҡатын-ҡыҙ хөкөмсө: уртаса аҡһыл тән төҫө',
    '👩🏽‍⚖' => 'ҡатын-ҡыҙ хөкөмсө: уртаса тән төҫө',
    '👩🏾‍⚖' => 'ҡатын-ҡыҙ хөкөмсө: уртаса ҡара тән төҫө',
    '👩🏿‍⚖' => 'ҡатын-ҡыҙ хөкөмсө: ҡара тән төҫө',
    '🧑🏻‍🌾' => 'фермер: аҡһыл тән төҫө',
    '🧑🏼‍🌾' => 'фермер: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🌾' => 'фермер: уртаса тән төҫө',
    '🧑🏾‍🌾' => 'фермер: уртаса ҡара тән төҫө',
    '🧑🏿‍🌾' => 'фермер: ҡара тән төҫө',
    '👨🏻‍🌾' => 'ир-ат фермер: аҡһыл тән төҫө',
    '👨🏼‍🌾' => 'ир-ат фермер: уртаса аҡһыл тән төҫө',
    '👨🏽‍🌾' => 'ир-ат фермер: уртаса тән төҫө',
    '👨🏾‍🌾' => 'ир-ат фермер: уртаса ҡара тән төҫө',
    '👨🏿‍🌾' => 'ир-ат фермер: ҡара тән төҫө',
    '👩🏻‍🌾' => 'ҡатын-ҡыҙ фермер: аҡһыл тән төҫө',
    '👩🏼‍🌾' => 'ҡатын-ҡыҙ фермер: уртаса аҡһыл тән төҫө',
    '👩🏽‍🌾' => 'ҡатын-ҡыҙ фермер: уртаса тән төҫө',
    '👩🏾‍🌾' => 'ҡатын-ҡыҙ фермер: уртаса ҡара тән төҫө',
    '👩🏿‍🌾' => 'ҡатын-ҡыҙ фермер: ҡара тән төҫө',
    '🧑🏻‍🍳' => 'ашнаҡсы: аҡһыл тән төҫө',
    '🧑🏼‍🍳' => 'ашнаҡсы: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🍳' => 'ашнаҡсы: уртаса тән төҫө',
    '🧑🏾‍🍳' => 'ашнаҡсы: уртаса ҡара тән төҫө',
    '🧑🏿‍🍳' => 'ашнаҡсы: ҡара тән төҫө',
    '👨🏻‍🍳' => 'ир-ат ашнаҡсы: аҡһыл тән төҫө',
    '👨🏼‍🍳' => 'ир-ат ашнаҡсы: уртаса аҡһыл тән төҫө',
    '👨🏽‍🍳' => 'ир-ат ашнаҡсы: уртаса тән төҫө',
    '👨🏾‍🍳' => 'ир-ат ашнаҡсы: уртаса ҡара тән төҫө',
    '👨🏿‍🍳' => 'ир-ат ашнаҡсы: ҡара тән төҫө',
    '👩🏻‍🍳' => 'ҡатын-ҡыҙ ашнаҡсы: аҡһыл тән төҫө',
    '👩🏼‍🍳' => 'ҡатын-ҡыҙ ашнаҡсы: уртаса аҡһыл тән төҫө',
    '👩🏽‍🍳' => 'ҡатын-ҡыҙ ашнаҡсы: уртаса тән төҫө',
    '👩🏾‍🍳' => 'ҡатын-ҡыҙ ашнаҡсы: уртаса ҡара тән төҫө',
    '👩🏿‍🍳' => 'ҡатын-ҡыҙ ашнаҡсы: ҡара тән төҫө',
    '🧑🏻‍🔧' => 'механик: аҡһыл тән төҫө',
    '🧑🏼‍🔧' => 'механик: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🔧' => 'механик: уртаса тән төҫө',
    '🧑🏾‍🔧' => 'механик: уртаса ҡара тән төҫө',
    '🧑🏿‍🔧' => 'механик: ҡара тән төҫө',
    '👨🏻‍🔧' => 'ир-ат механик: аҡһыл тән төҫө',
    '👨🏼‍🔧' => 'ир-ат механик: уртаса аҡһыл тән төҫө',
    '👨🏽‍🔧' => 'ир-ат механик: уртаса тән төҫө',
    '👨🏾‍🔧' => 'ир-ат механик: уртаса ҡара тән төҫө',
    '👨🏿‍🔧' => 'ир-ат механик: ҡара тән төҫө',
    '👩🏻‍🔧' => 'ҡатын-ҡыҙ механик: аҡһыл тән төҫө',
    '👩🏼‍🔧' => 'ҡатын-ҡыҙ механик: уртаса аҡһыл тән төҫө',
    '👩🏽‍🔧' => 'ҡатын-ҡыҙ механик: уртаса тән төҫө',
    '👩🏾‍🔧' => 'ҡатын-ҡыҙ механик: уртаса ҡара тән төҫө',
    '👩🏿‍🔧' => 'ҡатын-ҡыҙ механик: ҡара тән төҫө',
    '🧑🏻‍🏭' => 'завод эшсеһе: аҡһыл тән төҫө',
    '🧑🏼‍🏭' => 'завод эшсеһе: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🏭' => 'завод эшсеһе: уртаса тән төҫө',
    '🧑🏾‍🏭' => 'завод эшсеһе: уртаса ҡара тән төҫө',
    '🧑🏿‍🏭' => 'завод эшсеһе: ҡара тән төҫө',
    '👨🏻‍🏭' => 'ир-ат завод эшсеһе: аҡһыл тән төҫө',
    '👨🏼‍🏭' => 'ир-ат завод эшсеһе: уртаса аҡһыл тән төҫө',
    '👨🏽‍🏭' => 'ир-ат завод эшсеһе: уртаса тән төҫө',
    '👨🏾‍🏭' => 'ир-ат завод эшсеһе: уртаса ҡара тән төҫө',
    '👨🏿‍🏭' => 'ир-ат завод эшсеһе: ҡара тән төҫө',
    '👩🏻‍🏭' => 'ҡатын-ҡыҙ завод эшсеһе: аҡһыл тән төҫө',
    '👩🏼‍🏭' => 'ҡатын-ҡыҙ завод эшсеһе: уртаса аҡһыл тән төҫө',
    '👩🏽‍🏭' => 'ҡатын-ҡыҙ завод эшсеһе: уртаса тән төҫө',
    '👩🏾‍🏭' => 'ҡатын-ҡыҙ завод эшсеһе: уртаса ҡара тән төҫө',
    '👩🏿‍🏭' => 'ҡатын-ҡыҙ завод эшсеһе: ҡара тән төҫө',
    '🧑🏻‍💼' => 'офис хеҙмәткәре: аҡһыл тән төҫө',
    '🧑🏼‍💼' => 'офис хеҙмәткәре: уртаса аҡһыл тән төҫө',
    '🧑🏽‍💼' => 'офис хеҙмәткәре: уртаса тән төҫө',
    '🧑🏾‍💼' => 'офис хеҙмәткәре: уртаса ҡара тән төҫө',
    '🧑🏿‍💼' => 'офис хеҙмәткәре: ҡара тән төҫө',
    '👨🏻‍💼' => 'ир-ат офис эшсеһе: аҡһыл тән төҫө',
    '👨🏼‍💼' => 'ир-ат офис эшсеһе: уртаса аҡһыл тән төҫө',
    '👨🏽‍💼' => 'ир-ат офис эшсеһе: уртаса тән төҫө',
    '👨🏾‍💼' => 'ир-ат офис эшсеһе: уртаса ҡара тән төҫө',
    '👨🏿‍💼' => 'ир-ат офис эшсеһе: ҡара тән төҫө',
    '👩🏻‍💼' => 'ҡатын-ҡыҙ офис эшсеһе: аҡһыл тән төҫө',
    '👩🏼‍💼' => 'ҡатын-ҡыҙ офис эшсеһе: уртаса аҡһыл тән төҫө',
    '👩🏽‍💼' => 'ҡатын-ҡыҙ офис эшсеһе: уртаса тән төҫө',
    '👩🏾‍💼' => 'ҡатын-ҡыҙ офис эшсеһе: уртаса ҡара тән төҫө',
    '👩🏿‍💼' => 'ҡатын-ҡыҙ офис эшсеһе: ҡара тән төҫө',
    '🧑🏻‍🔬' => 'ғалим: аҡһыл тән төҫө',
    '🧑🏼‍🔬' => 'ғалим: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🔬' => 'ғалим: уртаса тән төҫө',
    '🧑🏾‍🔬' => 'ғалим: уртаса ҡара тән төҫө',
    '🧑🏿‍🔬' => 'ғалим: ҡара тән төҫө',
    '👨🏻‍🔬' => 'ир-ат ғалим: аҡһыл тән төҫө',
    '👨🏼‍🔬' => 'ир-ат ғалим: уртаса аҡһыл тән төҫө',
    '👨🏽‍🔬' => 'ир-ат ғалим: уртаса тән төҫө',
    '👨🏾‍🔬' => 'ир-ат ғалим: уртаса ҡара тән төҫө',
    '👨🏿‍🔬' => 'ир-ат ғалим: ҡара тән төҫө',
    '👩🏻‍🔬' => 'ҡатын-ҡыҙ ғалим: аҡһыл тән төҫө',
    '👩🏼‍🔬' => 'ҡатын-ҡыҙ ғалим: уртаса аҡһыл тән төҫө',
    '👩🏽‍🔬' => 'ҡатын-ҡыҙ ғалим: уртаса тән төҫө',
    '👩🏾‍🔬' => 'ҡатын-ҡыҙ ғалим: уртаса ҡара тән төҫө',
    '👩🏿‍🔬' => 'ҡатын-ҡыҙ ғалим: ҡара тән төҫө',
    '🧑🏻‍💻' => 'технолог: аҡһыл тән төҫө',
    '🧑🏼‍💻' => 'технолог: уртаса аҡһыл тән төҫө',
    '🧑🏽‍💻' => 'технолог: уртаса тән төҫө',
    '🧑🏾‍💻' => 'технолог: уртаса ҡара тән төҫө',
    '🧑🏿‍💻' => 'технолог: ҡара тән төҫө',
    '👨🏻‍💻' => 'ир-ат технолог: аҡһыл тән төҫө',
    '👨🏼‍💻' => 'ир-ат технолог: уртаса аҡһыл тән төҫө',
    '👨🏽‍💻' => 'ир-ат технолог: уртаса тән төҫө',
    '👨🏾‍💻' => 'ир-ат технолог: уртаса ҡара тән төҫө',
    '👨🏿‍💻' => 'ир-ат технолог: ҡара тән төҫө',
    '👩🏻‍💻' => 'ҡатын-ҡыҙ технолог: аҡһыл тән төҫө',
    '👩🏼‍💻' => 'ҡатын-ҡыҙ технолог: уртаса аҡһыл тән төҫө',
    '👩🏽‍💻' => 'ҡатын-ҡыҙ технолог: уртаса тән төҫө',
    '👩🏾‍💻' => 'ҡатын-ҡыҙ технолог: уртаса ҡара тән төҫө',
    '👩🏿‍💻' => 'ҡатын-ҡыҙ технолог: ҡара тән төҫө',
    '🧑🏻‍🎤' => 'йырсы: аҡһыл тән төҫө',
    '🧑🏼‍🎤' => 'йырсы: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🎤' => 'йырсы: уртаса тән төҫө',
    '🧑🏾‍🎤' => 'йырсы: уртаса ҡара тән төҫө',
    '🧑🏿‍🎤' => 'йырсы: ҡара тән төҫө',
    '👨🏻‍🎤' => 'ир-ат йырсы: аҡһыл тән төҫө',
    '👨🏼‍🎤' => 'ир-ат йырсы: уртаса аҡһыл тән төҫө',
    '👨🏽‍🎤' => 'ир-ат йырсы: уртаса тән төҫө',
    '👨🏾‍🎤' => 'ир-ат йырсы: уртаса ҡара тән төҫө',
    '👨🏿‍🎤' => 'ир-ат йырсы: ҡара тән төҫө',
    '👩🏻‍🎤' => 'ҡатын-ҡыҙ йырсы: аҡһыл тән төҫө',
    '👩🏼‍🎤' => 'ҡатын-ҡыҙ йырсы: уртаса аҡһыл тән төҫө',
    '👩🏽‍🎤' => 'ҡатын-ҡыҙ йырсы: уртаса тән төҫө',
    '👩🏾‍🎤' => 'ҡатын-ҡыҙ йырсы: уртаса ҡара тән төҫө',
    '👩🏿‍🎤' => 'ҡатын-ҡыҙ йырсы: ҡара тән төҫө',
    '🧑🏻‍🎨' => 'рәссам: аҡһыл тән төҫө',
    '🧑🏼‍🎨' => 'рәссам: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🎨' => 'рәссам: уртаса тән төҫө',
    '🧑🏾‍🎨' => 'рәссам: уртаса ҡара тән төҫө',
    '🧑🏿‍🎨' => 'рәссам: ҡара тән төҫө',
    '👨🏻‍🎨' => 'ир-ат рәссам: аҡһыл тән төҫө',
    '👨🏼‍🎨' => 'ир-ат рәссам: уртаса аҡһыл тән төҫө',
    '👨🏽‍🎨' => 'ир-ат рәссам: уртаса тән төҫө',
    '👨🏾‍🎨' => 'ир-ат рәссам: уртаса ҡара тән төҫө',
    '👨🏿‍🎨' => 'ир-ат рәссам: ҡара тән төҫө',
    '👩🏻‍🎨' => 'ҡатын-ҡыҙ рәссам: аҡһыл тән төҫө',
    '👩🏼‍🎨' => 'ҡатын-ҡыҙ рәссам: уртаса аҡһыл тән төҫө',
    '👩🏽‍🎨' => 'ҡатын-ҡыҙ рәссам: уртаса тән төҫө',
    '👩🏾‍🎨' => 'ҡатын-ҡыҙ рәссам: уртаса ҡара тән төҫө',
    '👩🏿‍🎨' => 'ҡатын-ҡыҙ рәссам: ҡара тән төҫө',
    '🧑🏻‍✈' => 'пилот: аҡһыл тән төҫө',
    '🧑🏼‍✈' => 'пилот: уртаса аҡһыл тән төҫө',
    '🧑🏽‍✈' => 'пилот: уртаса тән төҫө',
    '🧑🏾‍✈' => 'пилот: уртаса ҡара тән төҫө',
    '🧑🏿‍✈' => 'пилот: ҡара тән төҫө',
    '👨🏻‍✈' => 'ир-ат пилот: аҡһыл тән төҫө',
    '👨🏼‍✈' => 'ир-ат пилот: уртаса аҡһыл тән төҫө',
    '👨🏽‍✈' => 'ир-ат пилот: уртаса тән төҫө',
    '👨🏾‍✈' => 'ир-ат пилот: уртаса ҡара тән төҫө',
    '👨🏿‍✈' => 'ир-ат пилот: ҡара тән төҫө',
    '👩🏻‍✈' => 'ҡатын-ҡыҙ пилот: аҡһыл тән төҫө',
    '👩🏼‍✈' => 'ҡатын-ҡыҙ пилот: уртаса аҡһыл тән төҫө',
    '👩🏽‍✈' => 'ҡатын-ҡыҙ пилот: уртаса тән төҫө',
    '👩🏾‍✈' => 'ҡатын-ҡыҙ пилот: уртаса ҡара тән төҫө',
    '👩🏿‍✈' => 'ҡатын-ҡыҙ пилот: ҡара тән төҫө',
    '🧑🏻‍🚀' => 'космонавт: аҡһыл тән төҫө',
    '🧑🏼‍🚀' => 'космонавт: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🚀' => 'космонавт: уртаса тән төҫө',
    '🧑🏾‍🚀' => 'космонавт: уртаса ҡара тән төҫө',
    '🧑🏿‍🚀' => 'космонавт: ҡара тән төҫө',
    '👨🏻‍🚀' => 'ир-ат космонавт: аҡһыл тән төҫө',
    '👨🏼‍🚀' => 'ир-ат космонавт: уртаса аҡһыл тән төҫө',
    '👨🏽‍🚀' => 'ир-ат космонавт: уртаса тән төҫө',
    '👨🏾‍🚀' => 'ир-ат космонавт: уртаса ҡара тән төҫө',
    '👨🏿‍🚀' => 'ир-ат космонавт: ҡара тән төҫө',
    '👩🏻‍🚀' => 'ҡатын-ҡыҙ космонавт: аҡһыл тән төҫө',
    '👩🏼‍🚀' => 'ҡатын-ҡыҙ космонавт: уртаса аҡһыл тән төҫө',
    '👩🏽‍🚀' => 'ҡатын-ҡыҙ космонавт: уртаса тән төҫө',
    '👩🏾‍🚀' => 'ҡатын-ҡыҙ космонавт: уртаса ҡара тән төҫө',
    '👩🏿‍🚀' => 'ҡатын-ҡыҙ космонавт: ҡара тән төҫө',
    '🧑🏻‍🚒' => 'янғын һүндереүсе: аҡһыл тән төҫө',
    '🧑🏼‍🚒' => 'янғын һүндереүсе: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🚒' => 'янғын һүндереүсе: уртаса тән төҫө',
    '🧑🏾‍🚒' => 'янғын һүндереүсе: уртаса ҡара тән төҫө',
    '🧑🏿‍🚒' => 'янғын һүндереүсе: ҡара тән төҫө',
    '👨🏻‍🚒' => 'ир-ат янғын һүндереүсе: аҡһыл тән төҫө',
    '👨🏼‍🚒' => 'ир-ат янғын һүндереүсе: уртаса аҡһыл тән төҫө',
    '👨🏽‍🚒' => 'ир-ат янғын һүндереүсе: уртаса тән төҫө',
    '👨🏾‍🚒' => 'ир-ат янғын һүндереүсе: уртаса ҡара тән төҫө',
    '👨🏿‍🚒' => 'ир-ат янғын һүндереүсе: ҡара тән төҫө',
    '👩🏻‍🚒' => 'ҡатын-ҡыҙ янғын һүндереүсе: аҡһыл тән төҫө',
    '👩🏼‍🚒' => 'ҡатын-ҡыҙ янғын һүндереүсе: уртаса аҡһыл тән төҫө',
    '👩🏽‍🚒' => 'ҡатын-ҡыҙ янғын һүндереүсе: уртаса тән төҫө',
    '👩🏾‍🚒' => 'ҡатын-ҡыҙ янғын һүндереүсе: уртаса ҡара тән төҫө',
    '👩🏿‍🚒' => 'ҡатын-ҡыҙ янғын һүндереүсе: ҡара тән төҫө',
    '👮🏻‍♂' => 'ир-ат полиция хеҙмәткәре: аҡһыл тән төҫө',
    '👮🏼‍♂' => 'ир-ат полиция хеҙмәткәре: уртаса аҡһыл тән төҫө',
    '👮🏽‍♂' => 'ир-ат полиция хеҙмәткәре: уртаса тән төҫө',
    '👮🏾‍♂' => 'ир-ат полиция хеҙмәткәре: уртаса ҡара тән төҫө',
    '👮🏿‍♂' => 'ир-ат полиция хеҙмәткәре: ҡара тән төҫө',
    '👮🏻‍♀' => 'ҡатын-ҡыҙ полиция хеҙмәткәре: аҡһыл тән төҫө',
    '👮🏼‍♀' => 'ҡатын-ҡыҙ полиция хеҙмәткәре: уртаса аҡһыл тән төҫө',
    '👮🏽‍♀' => 'ҡатын-ҡыҙ полиция хеҙмәткәре: уртаса тән төҫө',
    '👮🏾‍♀' => 'ҡатын-ҡыҙ полиция хеҙмәткәре: уртаса ҡара тән төҫө',
    '👮🏿‍♀' => 'ҡатын-ҡыҙ полиция хеҙмәткәре: ҡара тән төҫө',
    '🕵🏻‍♂' => 'ир-ат детектив: аҡһыл тән төҫө',
    '🕵🏼‍♂' => 'ир-ат детектив: уртаса аҡһыл тән төҫө',
    '🕵🏽‍♂' => 'ир-ат детектив: уртаса тән төҫө',
    '🕵🏾‍♂' => 'ир-ат детектив: уртаса ҡара тән төҫө',
    '🕵🏿‍♂' => 'ир-ат детектив: ҡара тән төҫө',
    '🕵🏻‍♀' => 'ҡатын-ҡыҙ детектив: аҡһыл тән төҫө',
    '🕵🏼‍♀' => 'ҡатын-ҡыҙ детектив: уртаса аҡһыл тән төҫө',
    '🕵🏽‍♀' => 'ҡатын-ҡыҙ детектив: уртаса тән төҫө',
    '🕵🏾‍♀' => 'ҡатын-ҡыҙ детектив: уртаса ҡара тән төҫө',
    '🕵🏿‍♀' => 'ҡатын-ҡыҙ детектив: ҡара тән төҫө',
    '💂🏻‍♂' => 'ир-ат һаҡсы: аҡһыл тән төҫө',
    '💂🏼‍♂' => 'ир-ат һаҡсы: уртаса аҡһыл тән төҫө',
    '💂🏽‍♂' => 'ир-ат һаҡсы: уртаса тән төҫө',
    '💂🏾‍♂' => 'ир-ат һаҡсы: уртаса ҡара тән төҫө',
    '💂🏿‍♂' => 'ир-ат һаҡсы: ҡара тән төҫө',
    '💂🏻‍♀' => 'ҡатын-ҡыҙ һаҡсы: аҡһыл тән төҫө',
    '💂🏼‍♀' => 'ҡатын-ҡыҙ һаҡсы: уртаса аҡһыл тән төҫө',
    '💂🏽‍♀' => 'ҡатын-ҡыҙ һаҡсы: уртаса тән төҫө',
    '💂🏾‍♀' => 'ҡатын-ҡыҙ һаҡсы: уртаса ҡара тән төҫө',
    '💂🏿‍♀' => 'ҡатын-ҡыҙ һаҡсы: ҡара тән төҫө',
    '👷🏻‍♂' => 'ир-ат төҙөүсе: аҡһыл тән төҫө',
    '👷🏼‍♂' => 'ир-ат төҙөүсе: уртаса аҡһыл тән төҫө',
    '👷🏽‍♂' => 'ир-ат төҙөүсе: уртаса тән төҫө',
    '👷🏾‍♂' => 'ир-ат төҙөүсе: уртаса ҡара тән төҫө',
    '👷🏿‍♂' => 'ир-ат төҙөүсе: ҡара тән төҫө',
    '👷🏻‍♀' => 'ҡатын-ҡыҙ төҙөүсе: аҡһыл тән төҫө',
    '👷🏼‍♀' => 'ҡатын-ҡыҙ төҙөүсе: уртаса аҡһыл тән төҫө',
    '👷🏽‍♀' => 'ҡатын-ҡыҙ төҙөүсе: уртаса тән төҫө',
    '👷🏾‍♀' => 'ҡатын-ҡыҙ төҙөүсе: уртаса ҡара тән төҫө',
    '👷🏿‍♀' => 'ҡатын-ҡыҙ төҙөүсе: ҡара тән төҫө',
    '👳🏻‍♂' => 'салма кейгән ир: аҡһыл тән төҫө',
    '👳🏼‍♂' => 'салма кейгән ир: уртаса аҡһыл тән төҫө',
    '👳🏽‍♂' => 'салма кейгән ир: уртаса тән төҫө',
    '👳🏾‍♂' => 'салма кейгән ир: уртаса ҡара тән төҫө',
    '👳🏿‍♂' => 'салма кейгән ир: ҡара тән төҫө',
    '👳🏻‍♀' => 'салма кейгән ҡатын: аҡһыл тән төҫө',
    '👳🏼‍♀' => 'салма кейгән ҡатын: уртаса аҡһыл тән төҫө',
    '👳🏽‍♀' => 'салма кейгән ҡатын: уртаса тән төҫө',
    '👳🏾‍♀' => 'салма кейгән ҡатын: уртаса ҡара тән төҫө',
    '👳🏿‍♀' => 'салма кейгән ҡатын: ҡара тән төҫө',
    '🤵🏻‍♂' => 'смокингтағы ир: аҡһыл тән төҫө',
    '🤵🏼‍♂' => 'смокингтағы ир: уртаса аҡһыл тән төҫө',
    '🤵🏽‍♂' => 'смокингтағы ир: уртаса тән төҫө',
    '🤵🏾‍♂' => 'смокингтағы ир: уртаса ҡара тән төҫө',
    '🤵🏿‍♂' => 'смокингтағы ир: ҡара тән төҫө',
    '🤵🏻‍♀' => 'смокингтағы ҡатын: аҡһыл тән төҫө',
    '🤵🏼‍♀' => 'смокингтағы ҡатын: уртаса аҡһыл тән төҫө',
    '🤵🏽‍♀' => 'смокингтағы ҡатын: уртаса тән төҫө',
    '🤵🏾‍♀' => 'смокингтағы ҡатын: уртаса ҡара тән төҫө',
    '🤵🏿‍♀' => 'смокингтағы ҡатын: ҡара тән төҫө',
    '👰🏻‍♂' => 'фаталы ир: аҡһыл тән төҫө',
    '👰🏼‍♂' => 'фаталы ир: уртаса аҡһыл тән төҫө',
    '👰🏽‍♂' => 'фаталы ир: уртаса тән төҫө',
    '👰🏾‍♂' => 'фаталы ир: уртаса ҡара тән төҫө',
    '👰🏿‍♂' => 'фаталы ир: ҡара тән төҫө',
    '👰🏻‍♀' => 'фаталы ҡатын: аҡһыл тән төҫө',
    '👰🏼‍♀' => 'фаталы ҡатын: уртаса аҡһыл тән төҫө',
    '👰🏽‍♀' => 'фаталы ҡатын: уртаса тән төҫө',
    '👰🏾‍♀' => 'фаталы ҡатын: уртаса ҡара тән төҫө',
    '👰🏿‍♀' => 'фаталы ҡатын: ҡара тән төҫө',
    '👩🏻‍🍼' => 'балаһын ашатҡан ҡатын: аҡһыл тән төҫө',
    '👩🏼‍🍼' => 'балаһын ашатҡан ҡатын: уртаса аҡһыл тән төҫө',
    '👩🏽‍🍼' => 'балаһын ашатҡан ҡатын: уртаса тән төҫө',
    '👩🏾‍🍼' => 'балаһын ашатҡан ҡатын: уртаса ҡара тән төҫө',
    '👩🏿‍🍼' => 'балаһын ашатҡан ҡатын: ҡара тән төҫө',
    '👨🏻‍🍼' => 'балаһын ашатҡан ир: аҡһыл тән төҫө',
    '👨🏼‍🍼' => 'балаһын ашатҡан ир: уртаса аҡһыл тән төҫө',
    '👨🏽‍🍼' => 'балаһын ашатҡан ир: уртаса тән төҫө',
    '👨🏾‍🍼' => 'балаһын ашатҡан ир: уртаса ҡара тән төҫө',
    '👨🏿‍🍼' => 'балаһын ашатҡан ир: ҡара тән төҫө',
    '🧑🏻‍🍼' => 'балаһын ашатҡан кеше: аҡһыл тән төҫө',
    '🧑🏼‍🍼' => 'балаһын ашатҡан кеше: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🍼' => 'балаһын ашатҡан кеше: уртаса тән төҫө',
    '🧑🏾‍🍼' => 'балаһын ашатҡан кеше: уртаса ҡара тән төҫө',
    '🧑🏿‍🍼' => 'балаһын ашатҡан кеше: ҡара тән төҫө',
    '🧑🏻‍🎄' => 'Mx Клаус: аҡһыл тән төҫө',
    '🧑🏼‍🎄' => 'Mx Клаус: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🎄' => 'Mx Клаус: уртаса тән төҫө',
    '🧑🏾‍🎄' => 'Mx Клаус: уртаса ҡара тән төҫө',
    '🧑🏿‍🎄' => 'Mx Клаус: ҡара тән төҫө',
    '🦸🏻‍♂' => 'ир-ат супергерой: аҡһыл тән төҫө',
    '🦸🏼‍♂' => 'ир-ат супергерой: уртаса аҡһыл тән төҫө',
    '🦸🏽‍♂' => 'ир-ат супергерой: уртаса тән төҫө',
    '🦸🏾‍♂' => 'ир-ат супергерой: уртаса ҡара тән төҫө',
    '🦸🏿‍♂' => 'ир-ат супергерой: ҡара тән төҫө',
    '🦸🏻‍♀' => 'ҡатын-ҡыҙ супергерой: аҡһыл тән төҫө',
    '🦸🏼‍♀' => 'ҡатын-ҡыҙ супергерой: уртаса аҡһыл тән төҫө',
    '🦸🏽‍♀' => 'ҡатын-ҡыҙ супергерой: уртаса тән төҫө',
    '🦸🏾‍♀' => 'ҡатын-ҡыҙ супергерой: уртаса ҡара тән төҫө',
    '🦸🏿‍♀' => 'ҡатын-ҡыҙ супергерой: ҡара тән төҫө',
    '🦹🏻‍♂' => 'ир-ат супер яуыз: аҡһыл тән төҫө',
    '🦹🏼‍♂' => 'ир-ат супер яуыз: уртаса аҡһыл тән төҫө',
    '🦹🏽‍♂' => 'ир-ат супер яуыз: уртаса тән төҫө',
    '🦹🏾‍♂' => 'ир-ат супер яуыз: уртаса ҡара тән төҫө',
    '🦹🏿‍♂' => 'ир-ат супер яуыз: ҡара тән төҫө',
    '🦹🏻‍♀' => 'ҡатын-ҡыҙ супер яуыз: аҡһыл тән төҫө',
    '🦹🏼‍♀' => 'ҡатын-ҡыҙ супер яуыз: уртаса аҡһыл тән төҫө',
    '🦹🏽‍♀' => 'ҡатын-ҡыҙ супер яуыз: уртаса тән төҫө',
    '🦹🏾‍♀' => 'ҡатын-ҡыҙ супер яуыз: уртаса ҡара тән төҫө',
    '🦹🏿‍♀' => 'ҡатын-ҡыҙ супер яуыз: ҡара тән төҫө',
    '🧙🏻‍♂' => 'тылсымсы: аҡһыл тән төҫө',
    '🧙🏼‍♂' => 'тылсымсы: уртаса аҡһыл тән төҫө',
    '🧙🏽‍♂' => 'тылсымсы: уртаса тән төҫө',
    '🧙🏾‍♂' => 'тылсымсы: уртаса ҡара тән төҫө',
    '🧙🏿‍♂' => 'тылсымсы: ҡара тән төҫө',
    '🧙🏻‍♀' => 'ҡатын-ҡыҙ тылсымсы: аҡһыл тән төҫө',
    '🧙🏼‍♀' => 'ҡатын-ҡыҙ тылсымсы: уртаса аҡһыл тән төҫө',
    '🧙🏽‍♀' => 'ҡатын-ҡыҙ тылсымсы: уртаса тән төҫө',
    '🧙🏾‍♀' => 'ҡатын-ҡыҙ тылсымсы: уртаса ҡара тән төҫө',
    '🧙🏿‍♀' => 'ҡатын-ҡыҙ тылсымсы: ҡара тән төҫө',
    '🧚🏻‍♂' => 'ир-ат фея: аҡһыл тән төҫө',
    '🧚🏼‍♂' => 'ир-ат фея: уртаса аҡһыл тән төҫө',
    '🧚🏽‍♂' => 'ир-ат фея: уртаса тән төҫө',
    '🧚🏾‍♂' => 'ир-ат фея: уртаса ҡара тән төҫө',
    '🧚🏿‍♂' => 'ир-ат фея: ҡара тән төҫө',
    '🧚🏻‍♀' => 'ҡатын-ҡыҙ фея: аҡһыл тән төҫө',
    '🧚🏼‍♀' => 'ҡатын-ҡыҙ фея: уртаса аҡһыл тән төҫө',
    '🧚🏽‍♀' => 'ҡатын-ҡыҙ фея: уртаса тән төҫө',
    '🧚🏾‍♀' => 'ҡатын-ҡыҙ фея: уртаса ҡара тән төҫө',
    '🧚🏿‍♀' => 'ҡатын-ҡыҙ фея: ҡара тән төҫө',
    '🧛🏻‍♂' => 'ир-ат вампир: аҡһыл тән төҫө',
    '🧛🏼‍♂' => 'ир-ат вампир: уртаса аҡһыл тән төҫө',
    '🧛🏽‍♂' => 'ир-ат вампир: уртаса тән төҫө',
    '🧛🏾‍♂' => 'ир-ат вампир: уртаса ҡара тән төҫө',
    '🧛🏿‍♂' => 'ир-ат вампир: ҡара тән төҫө',
    '🧛🏻‍♀' => 'ҡатын-ҡыҙ вампир: аҡһыл тән төҫө',
    '🧛🏼‍♀' => 'ҡатын-ҡыҙ вампир: уртаса аҡһыл тән төҫө',
    '🧛🏽‍♀' => 'ҡатын-ҡыҙ вампир: уртаса тән төҫө',
    '🧛🏾‍♀' => 'ҡатын-ҡыҙ вампир: уртаса ҡара тән төҫө',
    '🧛🏿‍♀' => 'ҡатын-ҡыҙ вампир: ҡара тән төҫө',
    '🧜🏻‍♂' => 'һыу ире: аҡһыл тән төҫө',
    '🧜🏼‍♂' => 'һыу ире: уртаса аҡһыл тән төҫө',
    '🧜🏽‍♂' => 'һыу ире: уртаса тән төҫө',
    '🧜🏾‍♂' => 'һыу ире: уртаса ҡара тән төҫө',
    '🧜🏿‍♂' => 'һыу ире: ҡара тән төҫө',
    '🧜🏻‍♀' => 'һыу ҡыҙы: аҡһыл тән төҫө',
    '🧜🏼‍♀' => 'һыу ҡыҙы: уртаса аҡһыл тән төҫө',
    '🧜🏽‍♀' => 'һыу ҡыҙы: уртаса тән төҫө',
    '🧜🏾‍♀' => 'һыу ҡыҙы: уртаса ҡара тән төҫө',
    '🧜🏿‍♀' => 'һыу ҡыҙы: ҡара тән төҫө',
    '🧝🏻‍♂' => 'ир-ат эльф: аҡһыл тән төҫө',
    '🧝🏼‍♂' => 'ир-ат эльф: уртаса аҡһыл тән төҫө',
    '🧝🏽‍♂' => 'ир-ат эльф: уртаса тән төҫө',
    '🧝🏾‍♂' => 'ир-ат эльф: уртаса ҡара тән төҫө',
    '🧝🏿‍♂' => 'ир-ат эльф: ҡара тән төҫө',
    '🧝🏻‍♀' => 'ҡатын-ҡыҙ эльф: аҡһыл тән төҫө',
    '🧝🏼‍♀' => 'ҡатын-ҡыҙ эльф: уртаса аҡһыл тән төҫө',
    '🧝🏽‍♀' => 'ҡатын-ҡыҙ эльф: уртаса тән төҫө',
    '🧝🏾‍♀' => 'ҡатын-ҡыҙ эльф: уртаса ҡара тән төҫө',
    '🧝🏿‍♀' => 'ҡатын-ҡыҙ эльф: ҡара тән төҫө',
    '💆🏻‍♂' => 'массаж яһатыусы ир: аҡһыл тән төҫө',
    '💆🏼‍♂' => 'массаж яһатыусы ир: уртаса аҡһыл тән төҫө',
    '💆🏽‍♂' => 'массаж яһатыусы ир: уртаса тән төҫө',
    '💆🏾‍♂' => 'массаж яһатыусы ир: уртаса ҡара тән төҫө',
    '💆🏿‍♂' => 'массаж яһатыусы ир: ҡара тән төҫө',
    '💆🏻‍♀' => 'массаж яһатыусы ҡатын: аҡһыл тән төҫө',
    '💆🏼‍♀' => 'массаж яһатыусы ҡатын: уртаса аҡһыл тән төҫө',
    '💆🏽‍♀' => 'массаж яһатыусы ҡатын: уртаса тән төҫө',
    '💆🏾‍♀' => 'массаж яһатыусы ҡатын: уртаса ҡара тән төҫө',
    '💆🏿‍♀' => 'массаж яһатыусы ҡатын: ҡара тән төҫө',
    '💇🏻‍♂' => 'сәс киҫтереүсе ир: аҡһыл тән төҫө',
    '💇🏼‍♂' => 'сәс киҫтереүсе ир: уртаса аҡһыл тән төҫө',
    '💇🏽‍♂' => 'сәс киҫтереүсе ир: уртаса тән төҫө',
    '💇🏾‍♂' => 'сәс киҫтереүсе ир: уртаса ҡара тән төҫө',
    '💇🏿‍♂' => 'сәс киҫтереүсе ир: ҡара тән төҫө',
    '💇🏻‍♀' => 'сәс киҫтереүсе ҡатын: аҡһыл тән төҫө',
    '💇🏼‍♀' => 'сәс киҫтереүсе ҡатын: уртаса аҡһыл тән төҫө',
    '💇🏽‍♀' => 'сәс киҫтереүсе ҡатын: уртаса тән төҫө',
    '💇🏾‍♀' => 'сәс киҫтереүсе ҡатын: уртаса ҡара тән төҫө',
    '💇🏿‍♀' => 'сәс киҫтереүсе ҡатын: ҡара тән төҫө',
    '🚶🏻‍♂' => 'йөрөүсе ир-ат: аҡһыл тән төҫө',
    '🚶🏼‍♂' => 'йөрөүсе ир-ат: уртаса аҡһыл тән төҫө',
    '🚶🏽‍♂' => 'йөрөүсе ир-ат: уртаса тән төҫө',
    '🚶🏾‍♂' => 'йөрөүсе ир-ат: уртаса ҡара тән төҫө',
    '🚶🏿‍♂' => 'йөрөүсе ир-ат: ҡара тән төҫө',
    '🚶🏻‍♀' => 'йөрөүсе ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🚶🏼‍♀' => 'йөрөүсе ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🚶🏽‍♀' => 'йөрөүсе ҡатын-ҡыҙ: уртаса тән төҫө',
    '🚶🏾‍♀' => 'йөрөүсе ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🚶🏿‍♀' => 'йөрөүсе ҡатын-ҡыҙ: ҡара тән төҫө',
    '🚶🏻‍➡' => 'йөрөүсе кеше: аҡһыл тән төҫө һәм уңға ҡарай',
    '🚶🏼‍➡' => 'йөрөүсе кеше: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🚶🏽‍➡' => 'йөрөүсе кеше: уртаса тән төҫө һәм уңға ҡарай',
    '🚶🏾‍➡' => 'йөрөүсе кеше: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🚶🏿‍➡' => 'йөрөүсе кеше: ҡара тән төҫө һәм уңға ҡарай',
    '🧍🏻‍♂' => 'баҫып торған ир: аҡһыл тән төҫө',
    '🧍🏼‍♂' => 'баҫып торған ир: уртаса аҡһыл тән төҫө',
    '🧍🏽‍♂' => 'баҫып торған ир: уртаса тән төҫө',
    '🧍🏾‍♂' => 'баҫып торған ир: уртаса ҡара тән төҫө',
    '🧍🏿‍♂' => 'баҫып торған ир: ҡара тән төҫө',
    '🧍🏻‍♀' => 'баҫып торған ҡатын: аҡһыл тән төҫө',
    '🧍🏼‍♀' => 'баҫып торған ҡатын: уртаса аҡһыл тән төҫө',
    '🧍🏽‍♀' => 'баҫып торған ҡатын: уртаса тән төҫө',
    '🧍🏾‍♀' => 'баҫып торған ҡатын: уртаса ҡара тән төҫө',
    '🧍🏿‍♀' => 'баҫып торған ҡатын: ҡара тән төҫө',
    '🧎🏻‍♂' => 'тубыҡланған ир: аҡһыл тән төҫө',
    '🧎🏼‍♂' => 'тубыҡланған ир: уртаса аҡһыл тән төҫө',
    '🧎🏽‍♂' => 'тубыҡланған ир: уртаса тән төҫө',
    '🧎🏾‍♂' => 'тубыҡланған ир: уртаса ҡара тән төҫө',
    '🧎🏿‍♂' => 'тубыҡланған ир: ҡара тән төҫө',
    '🧎🏻‍♀' => 'тубыҡланған ҡатын: аҡһыл тән төҫө',
    '🧎🏼‍♀' => 'тубыҡланған ҡатын: уртаса аҡһыл тән төҫө',
    '🧎🏽‍♀' => 'тубыҡланған ҡатын: уртаса тән төҫө',
    '🧎🏾‍♀' => 'тубыҡланған ҡатын: уртаса ҡара тән төҫө',
    '🧎🏿‍♀' => 'тубыҡланған ҡатын: ҡара тән төҫө',
    '🧎🏻‍➡' => 'тубыҡланған кеше: аҡһыл тән төҫө һәм уңға ҡарай',
    '🧎🏼‍➡' => 'тубыҡланған кеше: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🧎🏽‍➡' => 'тубыҡланған кеше: уртаса тән төҫө һәм уңға ҡарай',
    '🧎🏾‍➡' => 'тубыҡланған кеше: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🧎🏿‍➡' => 'тубыҡланған кеше: ҡара тән төҫө һәм уңға ҡарай',
    '🧑🏻‍🦯' => 'аҡ таяҡлы кеше: аҡһыл тән төҫө',
    '🧑🏼‍🦯' => 'аҡ таяҡлы кеше: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🦯' => 'аҡ таяҡлы кеше: уртаса тән төҫө',
    '🧑🏾‍🦯' => 'аҡ таяҡлы кеше: уртаса ҡара тән төҫө',
    '🧑🏿‍🦯' => 'аҡ таяҡлы кеше: ҡара тән төҫө',
    '👨🏻‍🦯' => 'аҡ таяҡлы ир: аҡһыл тән төҫө',
    '👨🏼‍🦯' => 'аҡ таяҡлы ир: уртаса аҡһыл тән төҫө',
    '👨🏽‍🦯' => 'аҡ таяҡлы ир: уртаса тән төҫө',
    '👨🏾‍🦯' => 'аҡ таяҡлы ир: уртаса ҡара тән төҫө',
    '👨🏿‍🦯' => 'аҡ таяҡлы ир: ҡара тән төҫө',
    '👩🏻‍🦯' => 'аҡ таяҡлы ҡатын: аҡһыл тән төҫө',
    '👩🏼‍🦯' => 'аҡ таяҡлы ҡатын: уртаса аҡһыл тән төҫө',
    '👩🏽‍🦯' => 'аҡ таяҡлы ҡатын: уртаса тән төҫө',
    '👩🏾‍🦯' => 'аҡ таяҡлы ҡатын: уртаса ҡара тән төҫө',
    '👩🏿‍🦯' => 'аҡ таяҡлы ҡатын: ҡара тән төҫө',
    '🧑🏻‍🦼' => 'моторлы инвалид коляскаһындағы кеше: аҡһыл тән төҫө',
    '🧑🏼‍🦼' => 'моторлы инвалид коляскаһындағы кеше: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🦼' => 'моторлы инвалид коляскаһындағы кеше: уртаса тән төҫө',
    '🧑🏾‍🦼' => 'моторлы инвалид коляскаһындағы кеше: уртаса ҡара тән төҫө',
    '🧑🏿‍🦼' => 'моторлы инвалид коляскаһындағы кеше: ҡара тән төҫө',
    '👨🏻‍🦼' => 'моторлы инвалид коляскаһындағы ир: аҡһыл тән төҫө',
    '👨🏼‍🦼' => 'моторлы инвалид коляскаһындағы ир: уртаса аҡһыл тән төҫө',
    '👨🏽‍🦼' => 'моторлы инвалид коляскаһындағы ир: уртаса тән төҫө',
    '👨🏾‍🦼' => 'моторлы инвалид коляскаһындағы ир: уртаса ҡара тән төҫө',
    '👨🏿‍🦼' => 'моторлы инвалид коляскаһындағы ир: ҡара тән төҫө',
    '👩🏻‍🦼' => 'моторлы инвалид коляскаһындағы ҡатын: аҡһыл тән төҫө',
    '👩🏼‍🦼' => 'моторлы инвалид коляскаһындағы ҡатын: уртаса аҡһыл тән төҫө',
    '👩🏽‍🦼' => 'моторлы инвалид коляскаһындағы ҡатын: уртаса тән төҫө',
    '👩🏾‍🦼' => 'моторлы инвалид коляскаһындағы ҡатын: уртаса ҡара тән төҫө',
    '👩🏿‍🦼' => 'моторлы инвалид коляскаһындағы ҡатын: ҡара тән төҫө',
    '🧑🏻‍🦽' => 'инвалид коляскаһындағы кеше: аҡһыл тән төҫө',
    '🧑🏼‍🦽' => 'инвалид коляскаһындағы кеше: уртаса аҡһыл тән төҫө',
    '🧑🏽‍🦽' => 'инвалид коляскаһындағы кеше: уртаса тән төҫө',
    '🧑🏾‍🦽' => 'инвалид коляскаһындағы кеше: уртаса ҡара тән төҫө',
    '🧑🏿‍🦽' => 'инвалид коляскаһындағы кеше: ҡара тән төҫө',
    '👨🏻‍🦽' => 'инвалид коляскаһындағы ир-ат: аҡһыл тән төҫө',
    '👨🏼‍🦽' => 'инвалид коляскаһындағы ир-ат: уртаса аҡһыл тән төҫө',
    '👨🏽‍🦽' => 'инвалид коляскаһындағы ир-ат: уртаса тән төҫө',
    '👨🏾‍🦽' => 'инвалид коляскаһындағы ир-ат: уртаса ҡара тән төҫө',
    '👨🏿‍🦽' => 'инвалид коляскаһындағы ир-ат: ҡара тән төҫө',
    '👩🏻‍🦽' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '👩🏼‍🦽' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '👩🏽‍🦽' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: уртаса тән төҫө',
    '👩🏾‍🦽' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '👩🏿‍🦽' => 'инвалид коляскаһындағы ҡатын-ҡыҙ: ҡара тән төҫө',
    '🏃🏻‍♂' => 'йүгереүсе ир: аҡһыл тән төҫө',
    '🏃🏼‍♂' => 'йүгереүсе ир: уртаса аҡһыл тән төҫө',
    '🏃🏽‍♂' => 'йүгереүсе ир: уртаса тән төҫө',
    '🏃🏾‍♂' => 'йүгереүсе ир: уртаса ҡара тән төҫө',
    '🏃🏿‍♂' => 'йүгереүсе ир: ҡара тән төҫө',
    '🏃🏻‍♀' => 'йүгереүсе ҡатын: аҡһыл тән төҫө',
    '🏃🏼‍♀' => 'йүгереүсе ҡатын: уртаса аҡһыл тән төҫө',
    '🏃🏽‍♀' => 'йүгереүсе ҡатын: уртаса тән төҫө',
    '🏃🏾‍♀' => 'йүгереүсе ҡатын: уртаса ҡара тән төҫө',
    '🏃🏿‍♀' => 'йүгереүсе ҡатын: ҡара тән төҫө',
    '🏃🏻‍➡' => 'йүгереүсе кеше: аҡһыл тән төҫө һәм уңға ҡарай',
    '🏃🏼‍➡' => 'йүгереүсе кеше: уртаса аҡһыл тән төҫө һәм уңға ҡарай',
    '🏃🏽‍➡' => 'йүгереүсе кеше: уртаса тән төҫө һәм уңға ҡарай',
    '🏃🏾‍➡' => 'йүгереүсе кеше: уртаса ҡара тән төҫө һәм уңға ҡарай',
    '🏃🏿‍➡' => 'йүгереүсе кеше: ҡара тән төҫө һәм уңға ҡарай',
    '👯🏻‍♀' => 'ҡуян ҡолаҡлы ҡатындар: аҡһыл тән төҫө',
    '👯🏼‍♀' => 'ҡуян ҡолаҡлы ҡатындар: уртаса аҡһыл тән төҫө',
    '👯🏽‍♀' => 'ҡуян ҡолаҡлы ҡатындар: уртаса тән төҫө',
    '👯🏾‍♀' => 'ҡуян ҡолаҡлы ҡатындар: уртаса ҡара тән төҫө',
    '👯🏿‍♀' => 'ҡуян ҡолаҡлы ҡатындар: ҡара тән төҫө',
    '👯🏻‍♂' => 'ҡуян ҡолаҡлы ирҙәр: аҡһыл тән төҫө',
    '👯🏼‍♂' => 'ҡуян ҡолаҡлы ирҙәр: уртаса аҡһыл тән төҫө',
    '👯🏽‍♂' => 'ҡуян ҡолаҡлы ирҙәр: уртаса тән төҫө',
    '👯🏾‍♂' => 'ҡуян ҡолаҡлы ирҙәр: уртаса ҡара тән төҫө',
    '👯🏿‍♂' => 'ҡуян ҡолаҡлы ирҙәр: ҡара тән төҫө',
    '🧖🏻‍♂' => 'мунсалағы ир: аҡһыл тән төҫө',
    '🧖🏼‍♂' => 'мунсалағы ир: уртаса аҡһыл тән төҫө',
    '🧖🏽‍♂' => 'мунсалағы ир: уртаса тән төҫө',
    '🧖🏾‍♂' => 'мунсалағы ир: уртаса ҡара тән төҫө',
    '🧖🏿‍♂' => 'мунсалағы ир: ҡара тән төҫө',
    '🧖🏻‍♀' => 'мунсалағы ҡатын: аҡһыл тән төҫө',
    '🧖🏼‍♀' => 'мунсалағы ҡатын: уртаса аҡһыл тән төҫө',
    '🧖🏽‍♀' => 'мунсалағы ҡатын: уртаса тән төҫө',
    '🧖🏾‍♀' => 'мунсалағы ҡатын: уртаса ҡара тән төҫө',
    '🧖🏿‍♀' => 'мунсалағы ҡатын: ҡара тән төҫө',
    '🧗🏻‍♂' => 'ҡаяға үрмәләүсе ир: аҡһыл тән төҫө',
    '🧗🏼‍♂' => 'ҡаяға үрмәләүсе ир: уртаса аҡһыл тән төҫө',
    '🧗🏽‍♂' => 'ҡаяға үрмәләүсе ир: уртаса тән төҫө',
    '🧗🏾‍♂' => 'ҡаяға үрмәләүсе ир: уртаса ҡара тән төҫө',
    '🧗🏿‍♂' => 'ҡаяға үрмәләүсе ир: ҡара тән төҫө',
    '🧗🏻‍♀' => 'ҡаяға үрмәләүсе ҡатын: аҡһыл тән төҫө',
    '🧗🏼‍♀' => 'ҡаяға үрмәләүсе ҡатын: уртаса аҡһыл тән төҫө',
    '🧗🏽‍♀' => 'ҡаяға үрмәләүсе ҡатын: уртаса тән төҫө',
    '🧗🏾‍♀' => 'ҡаяға үрмәләүсе ҡатын: уртаса ҡара тән төҫө',
    '🧗🏿‍♀' => 'ҡаяға үрмәләүсе ҡатын: ҡара тән төҫө',
    '🏌🏻‍♂' => 'гольф уйнаусы ир: аҡһыл тән төҫө',
    '🏌🏼‍♂' => 'гольф уйнаусы ир: уртаса аҡһыл тән төҫө',
    '🏌🏽‍♂' => 'гольф уйнаусы ир: уртаса тән төҫө',
    '🏌🏾‍♂' => 'гольф уйнаусы ир: уртаса ҡара тән төҫө',
    '🏌🏿‍♂' => 'гольф уйнаусы ир: ҡара тән төҫө',
    '🏌🏻‍♀' => 'гольф уйнаусы ҡатын: аҡһыл тән төҫө',
    '🏌🏼‍♀' => 'гольф уйнаусы ҡатын: уртаса аҡһыл тән төҫө',
    '🏌🏽‍♀' => 'гольф уйнаусы ҡатын: уртаса тән төҫө',
    '🏌🏾‍♀' => 'гольф уйнаусы ҡатын: уртаса ҡара тән төҫө',
    '🏌🏿‍♀' => 'гольф уйнаусы ҡатын: ҡара тән төҫө',
    '🏄🏻‍♂' => 'серфингсы ир-ат: аҡһыл тән төҫө',
    '🏄🏼‍♂' => 'серфингсы ир-ат: уртаса аҡһыл тән төҫө',
    '🏄🏽‍♂' => 'серфингсы ир-ат: уртаса тән төҫө',
    '🏄🏾‍♂' => 'серфингсы ир-ат: уртаса ҡара тән төҫө',
    '🏄🏿‍♂' => 'серфингсы ир-ат: ҡара тән төҫө',
    '🏄🏻‍♀' => 'серфингсы ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🏄🏼‍♀' => 'серфингсы ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🏄🏽‍♀' => 'серфингсы ҡатын-ҡыҙ: уртаса тән төҫө',
    '🏄🏾‍♀' => 'серфингсы ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🏄🏿‍♀' => 'серфингсы ҡатын-ҡыҙ: ҡара тән төҫө',
    '🚣🏻‍♂' => 'кәмә ишеүсе ир: аҡһыл тән төҫө',
    '🚣🏼‍♂' => 'кәмә ишеүсе ир: уртаса аҡһыл тән төҫө',
    '🚣🏽‍♂' => 'кәмә ишеүсе ир: уртаса тән төҫө',
    '🚣🏾‍♂' => 'кәмә ишеүсе ир: уртаса ҡара тән төҫө',
    '🚣🏿‍♂' => 'кәмә ишеүсе ир: ҡара тән төҫө',
    '🚣🏻‍♀' => 'кәмә ишеүсе ҡатын: аҡһыл тән төҫө',
    '🚣🏼‍♀' => 'кәмә ишеүсе ҡатын: уртаса аҡһыл тән төҫө',
    '🚣🏽‍♀' => 'кәмә ишеүсе ҡатын: уртаса тән төҫө',
    '🚣🏾‍♀' => 'кәмә ишеүсе ҡатын: уртаса ҡара тән төҫө',
    '🚣🏿‍♀' => 'кәмә ишеүсе ҡатын: ҡара тән төҫө',
    '🏊🏻‍♂' => 'йөҙөүсе ир: аҡһыл тән төҫө',
    '🏊🏼‍♂' => 'йөҙөүсе ир: уртаса аҡһыл тән төҫө',
    '🏊🏽‍♂' => 'йөҙөүсе ир: уртаса тән төҫө',
    '🏊🏾‍♂' => 'йөҙөүсе ир: уртаса ҡара тән төҫө',
    '🏊🏿‍♂' => 'йөҙөүсе ир: ҡара тән төҫө',
    '🏊🏻‍♀' => 'йөҙөүсе ҡатын: аҡһыл тән төҫө',
    '🏊🏼‍♀' => 'йөҙөүсе ҡатын: уртаса аҡһыл тән төҫө',
    '🏊🏽‍♀' => 'йөҙөүсе ҡатын: уртаса тән төҫө',
    '🏊🏾‍♀' => 'йөҙөүсе ҡатын: уртаса ҡара тән төҫө',
    '🏊🏿‍♀' => 'йөҙөүсе ҡатын: ҡара тән төҫө',
    '⛹🏻‍♂' => 'туп һуғыусы ир: аҡһыл тән төҫө',
    '⛹🏼‍♂' => 'туп һуғыусы ир: уртаса аҡһыл тән төҫө',
    '⛹🏽‍♂' => 'туп һуғыусы ир: уртаса тән төҫө',
    '⛹🏾‍♂' => 'туп һуғыусы ир: уртаса ҡара тән төҫө',
    '⛹🏿‍♂' => 'туп һуғыусы ир: ҡара тән төҫө',
    '⛹🏻‍♀' => 'туп һуғыусы ҡатын: аҡһыл тән төҫө',
    '⛹🏼‍♀' => 'туп һуғыусы ҡатын: уртаса аҡһыл тән төҫө',
    '⛹🏽‍♀' => 'туп һуғыусы ҡатын: уртаса тән төҫө',
    '⛹🏾‍♀' => 'туп һуғыусы ҡатын: уртаса ҡара тән төҫө',
    '⛹🏿‍♀' => 'туп һуғыусы ҡатын: ҡара тән төҫө',
    '🏋🏻‍♂' => 'ауыр атлетика менән шөғөлләнеүсе ир: аҡһыл тән төҫө',
    '🏋🏼‍♂' => 'ауыр атлетика менән шөғөлләнеүсе ир: уртаса аҡһыл тән төҫө',
    '🏋🏽‍♂' => 'ауыр атлетика менән шөғөлләнеүсе ир: уртаса тән төҫө',
    '🏋🏾‍♂' => 'ауыр атлетика менән шөғөлләнеүсе ир: уртаса ҡара тән төҫө',
    '🏋🏿‍♂' => 'ауыр атлетика менән шөғөлләнеүсе ир: ҡара тән төҫө',
    '🏋🏻‍♀' => 'ауыр атлетика менән шөғөлләнеүсе ҡатын: аҡһыл тән төҫө',
    '🏋🏼‍♀' => 'ауыр атлетика менән шөғөлләнеүсе ҡатын: уртаса аҡһыл тән төҫө',
    '🏋🏽‍♀' => 'ауыр атлетика менән шөғөлләнеүсе ҡатын: уртаса тән төҫө',
    '🏋🏾‍♀' => 'ауыр атлетика менән шөғөлләнеүсе ҡатын: уртаса ҡара тән төҫө',
    '🏋🏿‍♀' => 'ауыр атлетика менән шөғөлләнеүсе ҡатын: ҡара тән төҫө',
    '🚴🏻‍♂' => 'велосипедта йөрөүсе ир-ат: аҡһыл тән төҫө',
    '🚴🏼‍♂' => 'велосипедта йөрөүсе ир-ат: уртаса аҡһыл тән төҫө',
    '🚴🏽‍♂' => 'велосипедта йөрөүсе ир-ат: уртаса тән төҫө',
    '🚴🏾‍♂' => 'велосипедта йөрөүсе ир-ат: уртаса ҡара тән төҫө',
    '🚴🏿‍♂' => 'велосипедта йөрөүсе ир-ат: ҡара тән төҫө',
    '🚴🏻‍♀' => 'велосипедта йөрөүсе ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🚴🏼‍♀' => 'велосипедта йөрөүсе ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🚴🏽‍♀' => 'велосипедта йөрөүсе ҡатын-ҡыҙ: уртаса тән төҫө',
    '🚴🏾‍♀' => 'велосипедта йөрөүсе ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🚴🏿‍♀' => 'велосипедта йөрөүсе ҡатын-ҡыҙ: ҡара тән төҫө',
    '🚵🏻‍♂' => 'тау велосипедында йөрөүсе ир-ат: аҡһыл тән төҫө',
    '🚵🏼‍♂' => 'тау велосипедында йөрөүсе ир-ат: уртаса аҡһыл тән төҫө',
    '🚵🏽‍♂' => 'тау велосипедында йөрөүсе ир-ат: уртаса тән төҫө',
    '🚵🏾‍♂' => 'тау велосипедында йөрөүсе ир-ат: уртаса ҡара тән төҫө',
    '🚵🏿‍♂' => 'тау велосипедында йөрөүсе ир-ат: ҡара тән төҫө',
    '🚵🏻‍♀' => 'тау велосипедында йөрөүсе ҡатын-ҡыҙ: аҡһыл тән төҫө',
    '🚵🏼‍♀' => 'тау велосипедында йөрөүсе ҡатын-ҡыҙ: уртаса аҡһыл тән төҫө',
    '🚵🏽‍♀' => 'тау велосипедында йөрөүсе ҡатын-ҡыҙ: уртаса тән төҫө',
    '🚵🏾‍♀' => 'тау велосипедында йөрөүсе ҡатын-ҡыҙ: уртаса ҡара тән төҫө',
    '🚵🏿‍♀' => 'тау велосипедында йөрөүсе ҡатын-ҡыҙ: ҡара тән төҫө',
    '🤸🏻‍♂' => 'ир ҡулдан ҡулға әйләнә: аҡһыл тән төҫө',
    '🤸🏼‍♂' => 'ир ҡулдан ҡулға әйләнә: уртаса аҡһыл тән төҫө',
    '🤸🏽‍♂' => 'ир ҡулдан ҡулға әйләнә: уртаса тән төҫө',
    '🤸🏾‍♂' => 'ир ҡулдан ҡулға әйләнә: уртаса ҡара тән төҫө',
    '🤸🏿‍♂' => 'ир ҡулдан ҡулға әйләнә: ҡара тән төҫө',
    '🤸🏻‍♀' => 'ҡатын ҡулдан ҡулға әйләнә: аҡһыл тән төҫө',
    '🤸🏼‍♀' => 'ҡатын ҡулдан ҡулға әйләнә: уртаса аҡһыл тән төҫө',
    '🤸🏽‍♀' => 'ҡатын ҡулдан ҡулға әйләнә: уртаса тән төҫө',
    '🤸🏾‍♀' => 'ҡатын ҡулдан ҡулға әйләнә: уртаса ҡара тән төҫө',
    '🤸🏿‍♀' => 'ҡатын ҡулдан ҡулға әйләнә: ҡара тән төҫө',
    '🤼🏻‍♀' => 'ҡатын-ҡыҙ көрәшселәр: аҡһыл тән төҫө',
    '🤼🏼‍♀' => 'ҡатын-ҡыҙ көрәшселәр: уртаса аҡһыл тән төҫө',
    '🤼🏽‍♀' => 'ҡатын-ҡыҙ көрәшселәр: уртаса тән төҫө',
    '🤼🏾‍♀' => 'ҡатын-ҡыҙ көрәшселәр: уртаса ҡара тән төҫө',
    '🤼🏿‍♀' => 'ҡатын-ҡыҙ көрәшселәр: ҡара тән төҫө',
    '🤼🏻‍♂' => 'ир-ат көрәшселәр: аҡһыл тән төҫө',
    '🤼🏼‍♂' => 'ир-ат көрәшселәр: уртаса аҡһыл тән төҫө',
    '🤼🏽‍♂' => 'ир-ат көрәшселәр: уртаса тән төҫө',
    '🤼🏾‍♂' => 'ир-ат көрәшселәр: уртаса ҡара тән төҫө',
    '🤼🏿‍♂' => 'ир-ат көрәшселәр: ҡара тән төҫө',
    '🤽🏻‍♂' => 'ир һыу полоһы уйнай: аҡһыл тән төҫө',
    '🤽🏼‍♂' => 'ир һыу полоһы уйнай: уртаса аҡһыл тән төҫө',
    '🤽🏽‍♂' => 'ир һыу полоһы уйнай: уртаса тән төҫө',
    '🤽🏾‍♂' => 'ир һыу полоһы уйнай: уртаса ҡара тән төҫө',
    '🤽🏿‍♂' => 'ир һыу полоһы уйнай: ҡара тән төҫө',
    '🤽🏻‍♀' => 'ҡатын һыу полоһы уйнай: аҡһыл тән төҫө',
    '🤽🏼‍♀' => 'ҡатын һыу полоһы уйнай: уртаса аҡһыл тән төҫө',
    '🤽🏽‍♀' => 'ҡатын һыу полоһы уйнай: уртаса тән төҫө',
    '🤽🏾‍♀' => 'ҡатын һыу полоһы уйнай: уртаса ҡара тән төҫө',
    '🤽🏿‍♀' => 'ҡатын һыу полоһы уйнай: ҡара тән төҫө',
    '🤾🏻‍♂' => 'ир гандбол уйнай: аҡһыл тән төҫө',
    '🤾🏼‍♂' => 'ир гандбол уйнай: уртаса аҡһыл тән төҫө',
    '🤾🏽‍♂' => 'ир гандбол уйнай: уртаса тән төҫө',
    '🤾🏾‍♂' => 'ир гандбол уйнай: уртаса ҡара тән төҫө',
    '🤾🏿‍♂' => 'ир гандбол уйнай: ҡара тән төҫө',
    '🤾🏻‍♀' => 'ҡатын гандбол уйнай: аҡһыл тән төҫө',
    '🤾🏼‍♀' => 'ҡатын гандбол уйнай: уртаса аҡһыл тән төҫө',
    '🤾🏽‍♀' => 'ҡатын гандбол уйнай: уртаса тән төҫө',
    '🤾🏾‍♀' => 'ҡатын гандбол уйнай: уртаса ҡара тән төҫө',
    '🤾🏿‍♀' => 'ҡатын гандбол уйнай: ҡара тән төҫө',
    '🤹🏻‍♂' => 'ир жонглерлыҡ итә: аҡһыл тән төҫө',
    '🤹🏼‍♂' => 'ир жонглерлыҡ итә: уртаса аҡһыл тән төҫө',
    '🤹🏽‍♂' => 'ир жонглерлыҡ итә: уртаса тән төҫө',
    '🤹🏾‍♂' => 'ир жонглерлыҡ итә: уртаса ҡара тән төҫө',
    '🤹🏿‍♂' => 'ир жонглерлыҡ итә: ҡара тән төҫө',
    '🤹🏻‍♀' => 'ҡатын жонглерлыҡ итә: аҡһыл тән төҫө',
    '🤹🏼‍♀' => 'ҡатын жонглерлыҡ итә: уртаса аҡһыл тән төҫө',
    '🤹🏽‍♀' => 'ҡатын жонглерлыҡ итә: уртаса тән төҫө',
    '🤹🏾‍♀' => 'ҡатын жонглерлыҡ итә: уртаса ҡара тән төҫө',
    '🤹🏿‍♀' => 'ҡатын жонглерлыҡ итә: ҡара тән төҫө',
    '🧘🏻‍♂' => 'лотос позаһындағы ир: аҡһыл тән төҫө',
    '🧘🏼‍♂' => 'лотос позаһындағы ир: уртаса аҡһыл тән төҫө',
    '🧘🏽‍♂' => 'лотос позаһындағы ир: уртаса тән төҫө',
    '🧘🏾‍♂' => 'лотос позаһындағы ир: уртаса ҡара тән төҫө',
    '🧘🏿‍♂' => 'лотос позаһындағы ир: ҡара тән төҫө',
    '🧘🏻‍♀' => 'лотос позаһындағы ҡатын: аҡһыл тән төҫө',
    '🧘🏼‍♀' => 'лотос позаһындағы ҡатын: уртаса аҡһыл тән төҫө',
    '🧘🏽‍♀' => 'лотос позаһындағы ҡатын: уртаса тән төҫө',
    '🧘🏾‍♀' => 'лотос позаһындағы ҡатын: уртаса ҡара тән төҫө',
    '🧘🏿‍♀' => 'лотос позаһындағы ҡатын: ҡара тән төҫө',
    '🧑‍🦰' => 'өлкән: ерән сәс',
    '🧑‍🦱' => 'өлкән: бөҙрә сәс',
    '🧑‍🦳' => 'өлкән: аҡ сәс',
    '🧑‍🦲' => 'өлкән: таҙ',
    '👨‍🦰' => 'ир: ерән сәс',
    '👨‍🦱' => 'ир: бөҙрә сәс',
    '👨‍🦳' => 'ир: аҡ сәс',
    '👨‍🦲' => 'ир: таҙ',
    '👩‍🦰' => 'ҡатын: ерән сәс',
    '👩‍🦱' => 'ҡатын: бөҙрә сәс',
    '👩‍🦳' => 'ҡатын: аҡ сәс',
    '👩‍🦲' => 'ҡатын: таҙ',
    '🚶‍➡' => 'йөрөүсе кеше: уңға ҡарай',
    '🧎‍➡' => 'тубыҡланған кеше: уңға ҡарай',
    '🏃‍➡' => 'йүгереүсе кеше: уңға ҡарай',
    '👨‍👦' => 'ғаилә: ир һәм малай',
    '👨‍👧' => 'ғаилә: ир һәм ҡыҙ',
    '👩‍👦' => 'ғаилә: ҡатын һәм малай',
    '👩‍👧' => 'ғаилә: ҡатын һәм ҡыҙ',
    '😮‍💨' => 'һулыш сығарыусы йөҙ',
    '🙂‍↕' => 'вертикаль баш сайҡау',
    '😵‍💫' => 'спираль күҙҙәр менән йөҙ',
    '❤‍🔥' => 'янған йөрәк',
    '❤‍🩹' => 'төҙәлеүсе йөрәк',
    '👁‍🗨' => 'һөйләшеү ҡыуығында күҙ',
    '🧔‍♂' => 'һаҡаллы ир',
    '👱‍♂' => 'ир: аҡһыл сәсле',
    '🧔‍♀' => 'һаҡаллы ҡатын',
    '👱‍♀' => 'ҡатын: аҡһыл сәсле',
    '🙍‍♂' => 'йөҙ сирыусы ир-ат',
    '🙍‍♀' => 'йөҙ сирыусы ҡатын-ҡыҙ',
    '🙎‍♂' => 'ирен сирыусы ир-ат',
    '🙎‍♀' => 'ирен сирыусы ҡатын-ҡыҙ',
    '🙅‍♂' => 'ЮҠ тип ымлаусы ир-ат',
    '🙅‍♀' => 'ЮҠ тип ымлаусы ҡатын-ҡыҙ',
    '🙆‍♂' => 'ЯҠШЫ тип ымлаусы ир-ат',
    '🙆‍♀' => 'ЯҠШЫ тип ымлаусы ҡатын-ҡыҙ',
    '💁‍♂' => 'ҡул һуҙған ир',
    '💁‍♀' => 'ҡул һуҙған ҡатын',
    '🙋‍♂' => 'ҡул күтәреүсе ир-ат',
    '🙋‍♀' => 'ҡул күтәреүсе ҡатын-ҡыҙ',
    '🧏‍♂' => 'һаңғырау ир',
    '🧏‍♀' => 'һаңғырау ҡатын',
    '🙇‍♂' => 'баш эйеүсе ир-ат',
    '🙇‍♀' => 'баш эйеүсе ҡатын-ҡыҙ',
    '🤦‍♂' => 'йөҙөн ҡул менән ҡаплаусы ир-ат',
    '🤦‍♀' => 'йөҙөн ҡул менән ҡаплаусы ҡатын-ҡыҙ',
    '🤷‍♂' => 'ир яурынын һикертә',
    '🤷‍♀' => 'ҡатын яурынын һикертә',
    '🧑‍⚕' => 'һаулыҡ һаҡлау хеҙмәткәре',
    '👨‍⚕' => 'ир-ат медицина хеҙмәткәре',
    '👩‍⚕' => 'ҡатын-ҡыҙ медицина хеҙмәткәре',
    '🧑‍🎓' => 'студент',
    '👨‍🎓' => 'ир-ат студент',
    '👩‍🎓' => 'ҡатын-ҡыҙ студент',
    '🧑‍🏫' => 'уҡытыусы',
    '👨‍🏫' => 'ир-ат уҡытыусы',
    '👩‍🏫' => 'ҡатын-ҡыҙ уҡытыусы',
    '🧑‍⚖' => 'хөкөмсө',
    '👨‍⚖' => 'ир-ат хөкөмсө',
    '👩‍⚖' => 'ҡатын-ҡыҙ хөкөмсө',
    '🧑‍🌾' => 'фермер',
    '👨‍🌾' => 'ир-ат фермер',
    '👩‍🌾' => 'ҡатын-ҡыҙ фермер',
    '🧑‍🍳' => 'ашнаҡсы',
    '👨‍🍳' => 'ир-ат ашнаҡсы',
    '👩‍🍳' => 'ҡатын-ҡыҙ ашнаҡсы',
    '🧑‍🔧' => 'механик',
    '👨‍🔧' => 'ир-ат механик',
    '👩‍🔧' => 'ҡатын-ҡыҙ механик',
    '🧑‍🏭' => 'завод эшсеһе',
    '👨‍🏭' => 'ир-ат завод эшсеһе',
    '👩‍🏭' => 'ҡатын-ҡыҙ завод эшсеһе',
    '🧑‍💼' => 'офис хеҙмәткәре',
    '👨‍💼' => 'ир-ат офис эшсеһе',
    '👩‍💼' => 'ҡатын-ҡыҙ офис эшсеһе',
    '🧑‍🔬' => 'ғалим',
    '👨‍🔬' => 'ир-ат ғалим',
    '👩‍🔬' => 'ҡатын-ҡыҙ ғалим',
    '🧑‍💻' => 'технолог',
    '👨‍💻' => 'ир-ат технолог',
    '👩‍💻' => 'ҡатын-ҡыҙ технолог',
    '🧑‍🎤' => 'йырсы',
    '👨‍🎤' => 'ир-ат йырсы',
    '👩‍🎤' => 'ҡатын-ҡыҙ йырсы',
    '🧑‍🎨' => 'рәссам',
    '👨‍🎨' => 'ир-ат рәссам',
    '👩‍🎨' => 'ҡатын-ҡыҙ рәссам',
    '🧑‍✈' => 'пилот',
    '👨‍✈' => 'ир-ат пилот',
    '👩‍✈' => 'ҡатын-ҡыҙ пилот',
    '🧑‍🚀' => 'космонавт',
    '👨‍🚀' => 'ир-ат космонавт',
    '👩‍🚀' => 'ҡатын-ҡыҙ космонавт',
    '🧑‍🚒' => 'янғын һүндереүсе',
    '👨‍🚒' => 'ир-ат янғын һүндереүсе',
    '👩‍🚒' => 'ҡатын-ҡыҙ янғын һүндереүсе',
    '👮‍♂' => 'ир-ат полиция хеҙмәткәре',
    '👮‍♀' => 'ҡатын-ҡыҙ полиция хеҙмәткәре',
    '🕵‍♂' => 'ир-ат детектив',
    '🕵‍♀' => 'ҡатын-ҡыҙ детектив',
    '💂‍♂' => 'ир-ат һаҡсы',
    '💂‍♀' => 'ҡатын-ҡыҙ һаҡсы',
    '👷‍♂' => 'ир-ат төҙөүсе',
    '👷‍♀' => 'ҡатын-ҡыҙ төҙөүсе',
    '👳‍♂' => 'салма кейгән ир',
    '👳‍♀' => 'салма кейгән ҡатын',
    '🤵‍♂' => 'смокингтағы ир',
    '🤵‍♀' => 'смокингтағы ҡатын',
    '👰‍♂' => 'фаталы ир',
    '👰‍♀' => 'фаталы ҡатын',
    '👩‍🍼' => 'балаһын ашатҡан ҡатын',
    '👨‍🍼' => 'балаһын ашатҡан ир',
    '🧑‍🍼' => 'балаһын ашатҡан кеше',
    '🧑‍🎄' => 'Mx Клаус',
    '🦸‍♂' => 'ир-ат супергерой',
    '🦸‍♀' => 'ҡатын-ҡыҙ супергерой',
    '🦹‍♂' => 'ир-ат супер яуыз',
    '🦹‍♀' => 'ҡатын-ҡыҙ супер яуыз',
    '🧙‍♂' => 'тылсымсы',
    '🧙‍♀' => 'ҡатын-ҡыҙ тылсымсы',
    '🧚‍♂' => 'ир-ат фея',
    '🧚‍♀' => 'ҡатын-ҡыҙ фея',
    '🧛‍♂' => 'ир-ат вампир',
    '🧛‍♀' => 'ҡатын-ҡыҙ вампир',
    '🧜‍♂' => 'һыу ире',
    '🧜‍♀' => 'һыу ҡыҙы',
    '🧝‍♂' => 'ир-ат эльф',
    '🧝‍♀' => 'ҡатын-ҡыҙ эльф',
    '🧞‍♂' => 'ир-ат джин',
    '🧞‍♀' => 'ҡатын-ҡыҙ джин',
    '🧟‍♂' => 'ир-ат зомби',
    '🧟‍♀' => 'ҡатын-ҡыҙ зомби',
    '💆‍♂' => 'массаж яһатыусы ир',
    '💆‍♀' => 'массаж яһатыусы ҡатын',
    '💇‍♂' => 'сәс киҫтереүсе ир',
    '💇‍♀' => 'сәс киҫтереүсе ҡатын',
    '🚶‍♂' => 'йөрөүсе ир-ат',
    '🚶‍♀' => 'йөрөүсе ҡатын-ҡыҙ',
    '🧍‍♂' => 'баҫып торған ир',
    '🧍‍♀' => 'баҫып торған ҡатын',
    '🧎‍♂' => 'тубыҡланған ир',
    '🧎‍♀' => 'тубыҡланған ҡатын',
    '🧑‍🦯' => 'аҡ таяҡлы кеше',
    '👨‍🦯' => 'аҡ таяҡлы ир',
    '👩‍🦯' => 'аҡ таяҡлы ҡатын',
    '🧑‍🦼' => 'моторлы инвалид коляскаһындағы кеше',
    '👨‍🦼' => 'моторлы инвалид коляскаһындағы ир',
    '👩‍🦼' => 'моторлы инвалид коляскаһындағы ҡатын',
    '🧑‍🦽' => 'инвалид коляскаһындағы кеше',
    '👨‍🦽' => 'инвалид коляскаһындағы ир-ат',
    '👩‍🦽' => 'инвалид коляскаһындағы ҡатын-ҡыҙ',
    '🏃‍♂' => 'йүгереүсе ир',
    '🏃‍♀' => 'йүгереүсе ҡатын',
    '🧑‍🩰' => 'балет бейеүсеһе',
    '👯‍♂' => 'ҡуян ҡолаҡлы ирҙәр',
    '👯‍♀' => 'ҡуян ҡолаҡлы ҡатындар',
    '🧖‍♂' => 'мунсалағы ир',
    '🧖‍♀' => 'мунсалағы ҡатын',
    '🧗‍♂' => 'ҡаяға үрмәләүсе ир',
    '🧗‍♀' => 'ҡаяға үрмәләүсе ҡатын',
    '🏌‍♂' => 'гольф уйнаусы ир',
    '🏌‍♀' => 'гольф уйнаусы ҡатын',
    '🏄‍♂' => 'серфингсы ир-ат',
    '🏄‍♀' => 'серфингсы ҡатын-ҡыҙ',
    '🚣‍♂' => 'кәмә ишеүсе ир',
    '🚣‍♀' => 'кәмә ишеүсе ҡатын',
    '🏊‍♂' => 'йөҙөүсе ир',
    '🏊‍♀' => 'йөҙөүсе ҡатын',
    '⛹‍♂' => 'туп һуғыусы ир',
    '⛹‍♀' => 'туп һуғыусы ҡатын',
    '🏋‍♂' => 'ауыр атлетика менән шөғөлләнеүсе ир',
    '🏋‍♀' => 'ауыр атлетика менән шөғөлләнеүсе ҡатын',
    '🚴‍♂' => 'велосипедта йөрөүсе ир-ат',
    '🚴‍♀' => 'велосипедта йөрөүсе ҡатын-ҡыҙ',
    '🚵‍♂' => 'тау велосипедында йөрөүсе ир-ат',
    '🚵‍♀' => 'тау велосипедында йөрөүсе ҡатын-ҡыҙ',
    '🤸‍♂' => 'ир ҡулдан ҡулға әйләнә',
    '🤸‍♀' => 'ҡатын ҡулдан ҡулға әйләнә',
    '🤼‍♂' => 'ир-ат көрәшселәр',
    '🤼‍♀' => 'ҡатын-ҡыҙ көрәшселәр',
    '🤽‍♂' => 'ир һыу полоһы уйнай',
    '🤽‍♀' => 'ҡатын һыу полоһы уйнай',
    '🤾‍♂' => 'ир гандбол уйнай',
    '🤾‍♀' => 'ҡатын гандбол уйнай',
    '🤹‍♂' => 'ир жонглерлыҡ итә',
    '🤹‍♀' => 'ҡатын жонглерлыҡ итә',
    '🧘‍♂' => 'лотос позаһындағы ир',
    '🧘‍♀' => 'лотос позаһындағы ҡатын',
    '🧑‍🧒' => 'ғаилә: өлкән һәм бала',
    '🐕‍🦺' => 'хеҙмәт эте',
    '🐈‍⬛' => 'ҡара бесәй',
    '🐻‍❄' => 'аҡ айыу башы',
    '🐦‍⬛' => 'ҡара ҡош',
    '🐦‍🔥' => 'феникс',
    '🍋‍🟩' => 'лайм',
    '🍄‍🟫' => 'көрән бәшмәк',
    '⛓‍💥' => 'эшләмәгән һылтанма',
    '🏳‍🌈' => 'йәйғор флагы',
    '🏳‍⚧' => 'трансгендер флагы',
    '🏴‍☠' => 'пират флагы',
    '👋🏻' => 'ҡул болғау: аҡһыл тән төҫө',
    '👋🏼' => 'ҡул болғау: уртаса аҡһыл тән төҫө',
    '👋🏽' => 'ҡул болғау: уртаса тән төҫө',
    '👋🏾' => 'ҡул болғау: уртаса ҡара тән төҫө',
    '👋🏿' => 'ҡул болғау: ҡара тән төҫө',
    '🤚🏻' => 'күтәрелгән ҡул арҡаһы: аҡһыл тән төҫө',
    '🤚🏼' => 'күтәрелгән ҡул арҡаһы: уртаса аҡһыл тән төҫө',
    '🤚🏽' => 'күтәрелгән ҡул арҡаһы: уртаса тән төҫө',
    '🤚🏾' => 'күтәрелгән ҡул арҡаһы: уртаса ҡара тән төҫө',
    '🤚🏿' => 'күтәрелгән ҡул арҡаһы: ҡара тән төҫө',
    '🖐🏻' => 'бармаҡтары йәйелгән ҡул: аҡһыл тән төҫө',
    '🖐🏼' => 'бармаҡтары йәйелгән ҡул: уртаса аҡһыл тән төҫө',
    '🖐🏽' => 'бармаҡтары йәйелгән ҡул: уртаса тән төҫө',
    '🖐🏾' => 'бармаҡтары йәйелгән ҡул: уртаса ҡара тән төҫө',
    '🖐🏿' => 'бармаҡтары йәйелгән ҡул: ҡара тән төҫө',
    '✋🏻' => 'күтәрелгән ҡул: аҡһыл тән төҫө',
    '✋🏼' => 'күтәрелгән ҡул: уртаса аҡһыл тән төҫө',
    '✋🏽' => 'күтәрелгән ҡул: уртаса тән төҫө',
    '✋🏾' => 'күтәрелгән ҡул: уртаса ҡара тән төҫө',
    '✋🏿' => 'күтәрелгән ҡул: ҡара тән төҫө',
    '🖖🏻' => 'вулкан сәләме: аҡһыл тән төҫө',
    '🖖🏼' => 'вулкан сәләме: уртаса аҡһыл тән төҫө',
    '🖖🏽' => 'вулкан сәләме: уртаса тән төҫө',
    '🖖🏾' => 'вулкан сәләме: уртаса ҡара тән төҫө',
    '🖖🏿' => 'вулкан сәләме: ҡара тән төҫө',
    '🫱🏻' => 'уңға ҡул: аҡһыл тән төҫө',
    '🫱🏼' => 'уңға ҡул: уртаса аҡһыл тән төҫө',
    '🫱🏽' => 'уңға ҡул: уртаса тән төҫө',
    '🫱🏾' => 'уңға ҡул: уртаса ҡара тән төҫө',
    '🫱🏿' => 'уңға ҡул: ҡара тән төҫө',
    '🫲🏻' => 'һулға ҡул: аҡһыл тән төҫө',
    '🫲🏼' => 'һулға ҡул: уртаса аҡһыл тән төҫө',
    '🫲🏽' => 'һулға ҡул: уртаса тән төҫө',
    '🫲🏾' => 'һулға ҡул: уртаса ҡара тән төҫө',
    '🫲🏿' => 'һулға ҡул: ҡара тән төҫө',
    '🫳🏻' => 'ус төбө аҫҡа ҡараған ҡул: аҡһыл тән төҫө',
    '🫳🏼' => 'ус төбө аҫҡа ҡараған ҡул: уртаса аҡһыл тән төҫө',
    '🫳🏽' => 'ус төбө аҫҡа ҡараған ҡул: уртаса тән төҫө',
    '🫳🏾' => 'ус төбө аҫҡа ҡараған ҡул: уртаса ҡара тән төҫө',
    '🫳🏿' => 'ус төбө аҫҡа ҡараған ҡул: ҡара тән төҫө',
    '🫴🏻' => 'ус төбө өҫкә ҡараған ҡул: аҡһыл тән төҫө',
    '🫴🏼' => 'ус төбө өҫкә ҡараған ҡул: уртаса аҡһыл тән төҫө',
    '🫴🏽' => 'ус төбө өҫкә ҡараған ҡул: уртаса тән төҫө',
    '🫴🏾' => 'ус төбө өҫкә ҡараған ҡул: уртаса ҡара тән төҫө',
    '🫴🏿' => 'ус төбө өҫкә ҡараған ҡул: ҡара тән төҫө',
    '🫷🏻' => 'һулға этәреүсе ҡул: аҡһыл тән төҫө',
    '🫷🏼' => 'һулға этәреүсе ҡул: уртаса аҡһыл тән төҫө',
    '🫷🏽' => 'һулға этәреүсе ҡул: уртаса тән төҫө',
    '🫷🏾' => 'һулға этәреүсе ҡул: уртаса ҡара тән төҫө',
    '🫷🏿' => 'һулға этәреүсе ҡул: ҡара тән төҫө',
    '🫸🏻' => 'уңға этәреүсе ҡул: аҡһыл тән төҫө',
    '🫸🏼' => 'уңға этәреүсе ҡул: уртаса аҡһыл тән төҫө',
    '🫸🏽' => 'уңға этәреүсе ҡул: уртаса тән төҫө',
    '🫸🏾' => 'уңға этәреүсе ҡул: уртаса ҡара тән төҫө',
    '🫸🏿' => 'уңға этәреүсе ҡул: ҡара тән төҫө',
    '👌🏻' => 'OK ҡул: аҡһыл тән төҫө',
    '👌🏼' => 'OK ҡул: уртаса аҡһыл тән төҫө',
    '👌🏽' => 'OK ҡул: уртаса тән төҫө',
    '👌🏾' => 'OK ҡул: уртаса ҡара тән төҫө',
    '👌🏿' => 'OK ҡул: ҡара тән төҫө',
    '🤌🏻' => 'ҡыҫылған бармаҡтар: аҡһыл тән төҫө',
    '🤌🏼' => 'ҡыҫылған бармаҡтар: уртаса аҡһыл тән төҫө',
    '🤌🏽' => 'ҡыҫылған бармаҡтар: уртаса тән төҫө',
    '🤌🏾' => 'ҡыҫылған бармаҡтар: уртаса ҡара тән төҫө',
    '🤌🏿' => 'ҡыҫылған бармаҡтар: ҡара тән төҫө',
    '🤏🏻' => 'ҡыҫыусы ҡул: аҡһыл тән төҫө',
    '🤏🏼' => 'ҡыҫыусы ҡул: уртаса аҡһыл тән төҫө',
    '🤏🏽' => 'ҡыҫыусы ҡул: уртаса тән төҫө',
    '🤏🏾' => 'ҡыҫыусы ҡул: уртаса ҡара тән төҫө',
    '🤏🏿' => 'ҡыҫыусы ҡул: ҡара тән төҫө',
    '✌🏻' => 'еңеү ҡулы: аҡһыл тән төҫө',
    '✌🏼' => 'еңеү ҡулы: уртаса аҡһыл тән төҫө',
    '✌🏽' => 'еңеү ҡулы: уртаса тән төҫө',
    '✌🏾' => 'еңеү ҡулы: уртаса ҡара тән төҫө',
    '✌🏿' => 'еңеү ҡулы: ҡара тән төҫө',
    '🤞🏻' => 'киҫешкән бармаҡтар: аҡһыл тән төҫө',
    '🤞🏼' => 'киҫешкән бармаҡтар: уртаса аҡһыл тән төҫө',
    '🤞🏽' => 'киҫешкән бармаҡтар: уртаса тән төҫө',
    '🤞🏾' => 'киҫешкән бармаҡтар: уртаса ҡара тән төҫө',
    '🤞🏿' => 'киҫешкән бармаҡтар: ҡара тән төҫө',
    '🫰🏻' => 'аҡса ишаралаусы ҡул: аҡһыл тән төҫө',
    '🫰🏼' => 'аҡса ишаралаусы ҡул: уртаса аҡһыл тән төҫө',
    '🫰🏽' => 'аҡса ишаралаусы ҡул: уртаса тән төҫө',
    '🫰🏾' => 'аҡса ишаралаусы ҡул: уртаса ҡара тән төҫө',
    '🫰🏿' => 'аҡса ишаралаусы ҡул: ҡара тән төҫө',
    '🤟🏻' => 'мин һине яратам ымлауы: аҡһыл тән төҫө',
    '🤟🏼' => 'мин һине яратам ымлауы: уртаса аҡһыл тән төҫө',
    '🤟🏽' => 'мин һине яратам ымлауы: уртаса тән төҫө',
    '🤟🏾' => 'мин һине яратам ымлауы: уртаса ҡара тән төҫө',
    '🤟🏿' => 'мин һине яратам ымлауы: ҡара тән төҫө',
    '🤘🏻' => 'мөгөҙ билдәһе: аҡһыл тән төҫө',
    '🤘🏼' => 'мөгөҙ билдәһе: уртаса аҡһыл тән төҫө',
    '🤘🏽' => 'мөгөҙ билдәһе: уртаса тән төҫө',
    '🤘🏾' => 'мөгөҙ билдәһе: уртаса ҡара тән төҫө',
    '🤘🏿' => 'мөгөҙ билдәһе: ҡара тән төҫө',
    '🤙🏻' => 'миңә шылтырат ҡулы: аҡһыл тән төҫө',
    '🤙🏼' => 'миңә шылтырат ҡулы: уртаса аҡһыл тән төҫө',
    '🤙🏽' => 'миңә шылтырат ҡулы: уртаса тән төҫө',
    '🤙🏾' => 'миңә шылтырат ҡулы: уртаса ҡара тән төҫө',
    '🤙🏿' => 'миңә шылтырат ҡулы: ҡара тән төҫө',
    '👈🏻' => 'һулға күрһәтеүсе ҡул арты: аҡһыл тән төҫө',
    '👈🏼' => 'һулға күрһәтеүсе ҡул арты: уртаса аҡһыл тән төҫө',
    '👈🏽' => 'һулға күрһәтеүсе ҡул арты: уртаса тән төҫө',
    '👈🏾' => 'һулға күрһәтеүсе ҡул арты: уртаса ҡара тән төҫө',
    '👈🏿' => 'һулға күрһәтеүсе ҡул арты: ҡара тән төҫө',
    '👉🏻' => 'уңға күрһәтеүсе ҡул арты: аҡһыл тән төҫө',
    '👉🏼' => 'уңға күрһәтеүсе ҡул арты: уртаса аҡһыл тән төҫө',
    '👉🏽' => 'уңға күрһәтеүсе ҡул арты: уртаса тән төҫө',
    '👉🏾' => 'уңға күрһәтеүсе ҡул арты: уртаса ҡара тән төҫө',
    '👉🏿' => 'уңға күрһәтеүсе ҡул арты: ҡара тән төҫө',
    '👆🏻' => 'өҫкә күрһәтеүсе ҡул арты: аҡһыл тән төҫө',
    '👆🏼' => 'өҫкә күрһәтеүсе ҡул арты: уртаса аҡһыл тән төҫө',
    '👆🏽' => 'өҫкә күрһәтеүсе ҡул арты: уртаса тән төҫө',
    '👆🏾' => 'өҫкә күрһәтеүсе ҡул арты: уртаса ҡара тән төҫө',
    '👆🏿' => 'өҫкә күрһәтеүсе ҡул арты: ҡара тән төҫө',
    '🖕🏻' => 'урта бармаҡ: аҡһыл тән төҫө',
    '🖕🏼' => 'урта бармаҡ: уртаса аҡһыл тән төҫө',
    '🖕🏽' => 'урта бармаҡ: уртаса тән төҫө',
    '🖕🏾' => 'урта бармаҡ: уртаса ҡара тән төҫө',
    '🖕🏿' => 'урта бармаҡ: ҡара тән төҫө',
    '👇🏻' => 'аҫҡа күрһәтеүсе ҡул арты: аҡһыл тән төҫө',
    '👇🏼' => 'аҫҡа күрһәтеүсе ҡул арты: уртаса аҡһыл тән төҫө',
    '👇🏽' => 'аҫҡа күрһәтеүсе ҡул арты: уртаса тән төҫө',
    '👇🏾' => 'аҫҡа күрһәтеүсе ҡул арты: уртаса ҡара тән төҫө',
    '👇🏿' => 'аҫҡа күрһәтеүсе ҡул арты: ҡара тән төҫө',
    '☝🏻' => 'өҫкә күрһәтеүсе һуҡ бармаҡ: аҡһыл тән төҫө',
    '☝🏼' => 'өҫкә күрһәтеүсе һуҡ бармаҡ: уртаса аҡһыл тән төҫө',
    '☝🏽' => 'өҫкә күрһәтеүсе һуҡ бармаҡ: уртаса тән төҫө',
    '☝🏾' => 'өҫкә күрһәтеүсе һуҡ бармаҡ: уртаса ҡара тән төҫө',
    '☝🏿' => 'өҫкә күрһәтеүсе һуҡ бармаҡ: ҡара тән төҫө',
    '🫵🏻' => 'ҡараусыға күрһәтеүсе бармаҡ: аҡһыл тән төҫө',
    '🫵🏼' => 'ҡараусыға күрһәтеүсе бармаҡ: уртаса аҡһыл тән төҫө',
    '🫵🏽' => 'ҡараусыға күрһәтеүсе бармаҡ: уртаса тән төҫө',
    '🫵🏾' => 'ҡараусыға күрһәтеүсе бармаҡ: уртаса ҡара тән төҫө',
    '🫵🏿' => 'ҡараусыға күрһәтеүсе бармаҡ: ҡара тән төҫө',
    '👍🏻' => 'баш бармаҡ өҫкә: аҡһыл тән төҫө',
    '👍🏼' => 'баш бармаҡ өҫкә: уртаса аҡһыл тән төҫө',
    '👍🏽' => 'баш бармаҡ өҫкә: уртаса тән төҫө',
    '👍🏾' => 'баш бармаҡ өҫкә: уртаса ҡара тән төҫө',
    '👍🏿' => 'баш бармаҡ өҫкә: ҡара тән төҫө',
    '👎🏻' => 'баш бармаҡ аҫҡа: аҡһыл тән төҫө',
    '👎🏼' => 'баш бармаҡ аҫҡа: уртаса аҡһыл тән төҫө',
    '👎🏽' => 'баш бармаҡ аҫҡа: уртаса тән төҫө',
    '👎🏾' => 'баш бармаҡ аҫҡа: уртаса ҡара тән төҫө',
    '👎🏿' => 'баш бармаҡ аҫҡа: ҡара тән төҫө',
    '✊🏻' => 'күтәрелгән йоҙроҡ: аҡһыл тән төҫө',
    '✊🏼' => 'күтәрелгән йоҙроҡ: уртаса аҡһыл тән төҫө',
    '✊🏽' => 'күтәрелгән йоҙроҡ: уртаса тән төҫө',
    '✊🏾' => 'күтәрелгән йоҙроҡ: уртаса ҡара тән төҫө',
    '✊🏿' => 'күтәрелгән йоҙроҡ: ҡара тән төҫө',
    '👊🏻' => 'ҡаршы килеүсе йоҙроҡ: аҡһыл тән төҫө',
    '👊🏼' => 'ҡаршы килеүсе йоҙроҡ: уртаса аҡһыл тән төҫө',
    '👊🏽' => 'ҡаршы килеүсе йоҙроҡ: уртаса тән төҫө',
    '👊🏾' => 'ҡаршы килеүсе йоҙроҡ: уртаса ҡара тән төҫө',
    '👊🏿' => 'ҡаршы килеүсе йоҙроҡ: ҡара тән төҫө',
    '🤛🏻' => 'һулға ҡараған йоҙроҡ: аҡһыл тән төҫө',
    '🤛🏼' => 'һулға ҡараған йоҙроҡ: уртаса аҡһыл тән төҫө',
    '🤛🏽' => 'һулға ҡараған йоҙроҡ: уртаса тән төҫө',
    '🤛🏾' => 'һулға ҡараған йоҙроҡ: уртаса ҡара тән төҫө',
    '🤛🏿' => 'һулға ҡараған йоҙроҡ: ҡара тән төҫө',
    '🤜🏻' => 'уңға ҡараған йоҙроҡ: аҡһыл тән төҫө',
    '🤜🏼' => 'уңға ҡараған йоҙроҡ: уртаса аҡһыл тән төҫө',
    '🤜🏽' => 'уңға ҡараған йоҙроҡ: уртаса тән төҫө',
    '🤜🏾' => 'уңға ҡараған йоҙроҡ: уртаса ҡара тән төҫө',
    '🤜🏿' => 'уңға ҡараған йоҙроҡ: ҡара тән төҫө',
    '👏🏻' => 'ҡул сабыу: аҡһыл тән төҫө',
    '👏🏼' => 'ҡул сабыу: уртаса аҡһыл тән төҫө',
    '👏🏽' => 'ҡул сабыу: уртаса тән төҫө',
    '👏🏾' => 'ҡул сабыу: уртаса ҡара тән төҫө',
    '👏🏿' => 'ҡул сабыу: ҡара тән төҫө',
    '🙌🏻' => 'ҡулдарҙы күтәреү: аҡһыл тән төҫө',
    '🙌🏼' => 'ҡулдарҙы күтәреү: уртаса аҡһыл тән төҫө',
    '🙌🏽' => 'ҡулдарҙы күтәреү: уртаса тән төҫө',
    '🙌🏾' => 'ҡулдарҙы күтәреү: уртаса ҡара тән төҫө',
    '🙌🏿' => 'ҡулдарҙы күтәреү: ҡара тән төҫө',
    '🫶🏻' => 'йөрәк ҡулдар: аҡһыл тән төҫө',
    '🫶🏼' => 'йөрәк ҡулдар: уртаса аҡһыл тән төҫө',
    '🫶🏽' => 'йөрәк ҡулдар: уртаса тән төҫө',
    '🫶🏾' => 'йөрәк ҡулдар: уртаса ҡара тән төҫө',
    '🫶🏿' => 'йөрәк ҡулдар: ҡара тән төҫө',
    '👐🏻' => 'асыҡ ҡулдар: аҡһыл тән төҫө',
    '👐🏼' => 'асыҡ ҡулдар: уртаса аҡһыл тән төҫө',
    '👐🏽' => 'асыҡ ҡулдар: уртаса тән төҫө',
    '👐🏾' => 'асыҡ ҡулдар: уртаса ҡара тән төҫө',
    '👐🏿' => 'асыҡ ҡулдар: ҡара тән төҫө',
    '🤲🏻' => 'ус төптәре бергә: аҡһыл тән төҫө',
    '🤲🏼' => 'ус төптәре бергә: уртаса аҡһыл тән төҫө',
    '🤲🏽' => 'ус төптәре бергә: уртаса тән төҫө',
    '🤲🏾' => 'ус төптәре бергә: уртаса ҡара тән төҫө',
    '🤲🏿' => 'ус төптәре бергә: ҡара тән төҫө',
    '🤝🏻' => 'ҡул бирешеү: аҡһыл тән төҫө',
    '🤝🏼' => 'ҡул бирешеү: уртаса аҡһыл тән төҫө',
    '🤝🏽' => 'ҡул бирешеү: уртаса тән төҫө',
    '🤝🏾' => 'ҡул бирешеү: уртаса ҡара тән төҫө',
    '🤝🏿' => 'ҡул бирешеү: ҡара тән төҫө',
    '🙏🏻' => 'ҡулдарҙы бергә тотоу: аҡһыл тән төҫө',
    '🙏🏼' => 'ҡулдарҙы бергә тотоу: уртаса аҡһыл тән төҫө',
    '🙏🏽' => 'ҡулдарҙы бергә тотоу: уртаса тән төҫө',
    '🙏🏾' => 'ҡулдарҙы бергә тотоу: уртаса ҡара тән төҫө',
    '🙏🏿' => 'ҡулдарҙы бергә тотоу: ҡара тән төҫө',
    '✍🏻' => 'яҙыусы ҡул: аҡһыл тән төҫө',
    '✍🏼' => 'яҙыусы ҡул: уртаса аҡһыл тән төҫө',
    '✍🏽' => 'яҙыусы ҡул: уртаса тән төҫө',
    '✍🏾' => 'яҙыусы ҡул: уртаса ҡара тән төҫө',
    '✍🏿' => 'яҙыусы ҡул: ҡара тән төҫө',
    '💅🏻' => 'тырнаҡтарҙы буяу: аҡһыл тән төҫө',
    '💅🏼' => 'тырнаҡтарҙы буяу: уртаса аҡһыл тән төҫө',
    '💅🏽' => 'тырнаҡтарҙы буяу: уртаса тән төҫө',
    '💅🏾' => 'тырнаҡтарҙы буяу: уртаса ҡара тән төҫө',
    '💅🏿' => 'тырнаҡтарҙы буяу: ҡара тән төҫө',
    '🤳🏻' => 'селфи: аҡһыл тән төҫө',
    '🤳🏼' => 'селфи: уртаса аҡһыл тән төҫө',
    '🤳🏽' => 'селфи: уртаса тән төҫө',
    '🤳🏾' => 'селфи: уртаса ҡара тән төҫө',
    '🤳🏿' => 'селфи: ҡара тән төҫө',
    '💪🏻' => 'бицепс күрһәтеүсе ҡул: аҡһыл тән төҫө',
    '💪🏼' => 'бицепс күрһәтеүсе ҡул: уртаса аҡһыл тән төҫө',
    '💪🏽' => 'бицепс күрһәтеүсе ҡул: уртаса тән төҫө',
    '💪🏾' => 'бицепс күрһәтеүсе ҡул: уртаса ҡара тән төҫө',
    '💪🏿' => 'бицепс күрһәтеүсе ҡул: ҡара тән төҫө',
    '🦵🏻' => 'аяҡ: аҡһыл тән төҫө',
    '🦵🏼' => 'аяҡ: уртаса аҡһыл тән төҫө',
    '🦵🏽' => 'аяҡ: уртаса тән төҫө',
    '🦵🏾' => 'аяҡ: уртаса ҡара тән төҫө',
    '🦵🏿' => 'аяҡ: ҡара тән төҫө',
    '👂🏻' => 'ҡолаҡ: аҡһыл тән төҫө',
    '👂🏼' => 'ҡолаҡ: уртаса аҡһыл тән төҫө',
    '👂🏽' => 'ҡолаҡ: уртаса тән төҫө',
    '👂🏾' => 'ҡолаҡ: уртаса ҡара тән төҫө',
    '👂🏿' => 'ҡолаҡ: ҡара тән төҫө',
    '🦻🏻' => 'ишетеү аппараты менән ҡолаҡ: аҡһыл тән төҫө',
    '🦻🏼' => 'ишетеү аппараты менән ҡолаҡ: уртаса аҡһыл тән төҫө',
    '🦻🏽' => 'ишетеү аппараты менән ҡолаҡ: уртаса тән төҫө',
    '🦻🏾' => 'ишетеү аппараты менән ҡолаҡ: уртаса ҡара тән төҫө',
    '🦻🏿' => 'ишетеү аппараты менән ҡолаҡ: ҡара тән төҫө',
    '👃🏻' => 'танау: аҡһыл тән төҫө',
    '👃🏼' => 'танау: уртаса аҡһыл тән төҫө',
    '👃🏽' => 'танау: уртаса тән төҫө',
    '👃🏾' => 'танау: уртаса ҡара тән төҫө',
    '👃🏿' => 'танау: ҡара тән төҫө',
    '👶🏻' => 'сабый: аҡһыл тән төҫө',
    '👶🏼' => 'сабый: уртаса аҡһыл тән төҫө',
    '👶🏽' => 'сабый: уртаса тән төҫө',
    '👶🏾' => 'сабый: уртаса ҡара тән төҫө',
    '👶🏿' => 'сабый: ҡара тән төҫө',
    '🧒🏻' => 'бала: аҡһыл тән төҫө',
    '🧒🏼' => 'бала: уртаса аҡһыл тән төҫө',
    '🧒🏽' => 'бала: уртаса тән төҫө',
    '🧒🏾' => 'бала: уртаса ҡара тән төҫө',
    '🧒🏿' => 'бала: ҡара тән төҫө',
    '👦🏻' => 'малай: аҡһыл тән төҫө',
    '👦🏼' => 'малай: уртаса аҡһыл тән төҫө',
    '👦🏽' => 'малай: уртаса тән төҫө',
    '👦🏾' => 'малай: уртаса ҡара тән төҫө',
    '👦🏿' => 'малай: ҡара тән төҫө',
    '👧🏻' => 'ҡыҙ: аҡһыл тән төҫө',
    '👧🏼' => 'ҡыҙ: уртаса аҡһыл тән төҫө',
    '👧🏽' => 'ҡыҙ: уртаса тән төҫө',
    '👧🏾' => 'ҡыҙ: уртаса ҡара тән төҫө',
    '👧🏿' => 'ҡыҙ: ҡара тән төҫө',
    '🧑🏻' => 'өлкән: аҡһыл тән төҫө',
    '🧑🏼' => 'өлкән: уртаса аҡһыл тән төҫө',
    '🧑🏽' => 'өлкән: уртаса тән төҫө',
    '🧑🏾' => 'өлкән: уртаса ҡара тән төҫө',
    '🧑🏿' => 'өлкән: ҡара тән төҫө',
    '👱🏻' => 'кеше: аҡһыл тән төҫө һәм аҡһыл сәсле',
    '👱🏼' => 'кеше: уртаса аҡһыл тән төҫө һәм аҡһыл сәсле',
    '👱🏽' => 'кеше: уртаса тән төҫө һәм аҡһыл сәсле',
    '👱🏾' => 'кеше: уртаса ҡара тән төҫө һәм аҡһыл сәсле',
    '👱🏿' => 'кеше: ҡара тән төҫө һәм аҡһыл сәсле',
    '👨🏻' => 'ир: аҡһыл тән төҫө',
    '👨🏼' => 'ир: уртаса аҡһыл тән төҫө',
    '👨🏽' => 'ир: уртаса тән төҫө',
    '👨🏾' => 'ир: уртаса ҡара тән төҫө',
    '👨🏿' => 'ир: ҡара тән төҫө',
    '🧔🏻' => 'һаҡаллы кеше: аҡһыл тән төҫө',
    '🧔🏼' => 'һаҡаллы кеше: уртаса аҡһыл тән төҫө',
    '🧔🏽' => 'һаҡаллы кеше: уртаса тән төҫө',
    '🧔🏾' => 'һаҡаллы кеше: уртаса ҡара тән төҫө',
    '🧔🏿' => 'һаҡаллы кеше: ҡара тән төҫө',
    '👩🏻' => 'ҡатын: аҡһыл тән төҫө',
    '👩🏼' => 'ҡатын: уртаса аҡһыл тән төҫө',
    '👩🏽' => 'ҡатын: уртаса тән төҫө',
    '👩🏾' => 'ҡатын: уртаса ҡара тән төҫө',
    '👩🏿' => 'ҡатын: ҡара тән төҫө',
    '🧓🏻' => 'оло кеше: аҡһыл тән төҫө',
    '🧓🏼' => 'оло кеше: уртаса аҡһыл тән төҫө',
    '🧓🏽' => 'оло кеше: уртаса тән төҫө',
    '🧓🏾' => 'оло кеше: уртаса ҡара тән төҫө',
    '🧓🏿' => 'оло кеше: ҡара тән төҫө',
    '👴🏻' => 'ҡарт ир: аҡһыл тән төҫө',
    '👴🏼' => 'ҡарт ир: уртаса аҡһыл тән төҫө',
    '👴🏽' => 'ҡарт ир: уртаса тән төҫө',
    '👴🏾' => 'ҡарт ир: уртаса ҡара тән төҫө',
    '👴🏿' => 'ҡарт ир: ҡара тән төҫө',
    '👵🏻' => 'ҡарт ҡатын: аҡһыл тән төҫө',
    '👵🏼' => 'ҡарт ҡатын: уртаса аҡһыл тән төҫө',
    '👵🏽' => 'ҡарт ҡатын: уртаса тән төҫө',
    '👵🏾' => 'ҡарт ҡатын: уртаса ҡара тән төҫө',
    '👵🏿' => 'ҡарт ҡатын: ҡара тән төҫө',
    '🙍🏻' => 'йөҙ сирыусы кеше: аҡһыл тән төҫө',
    '🙍🏼' => 'йөҙ сирыусы кеше: уртаса аҡһыл тән төҫө',
    '🙍🏽' => 'йөҙ сирыусы кеше: уртаса тән төҫө',
    '🙍🏾' => 'йөҙ сирыусы кеше: уртаса ҡара тән төҫө',
    '🙍🏿' => 'йөҙ сирыусы кеше: ҡара тән төҫө',
    '🙎🏻' => 'ирен сирыусы кеше: аҡһыл тән төҫө',
    '🙎🏼' => 'ирен сирыусы кеше: уртаса аҡһыл тән төҫө',
    '🙎🏽' => 'ирен сирыусы кеше: уртаса тән төҫө',
    '🙎🏾' => 'ирен сирыусы кеше: уртаса ҡара тән төҫө',
    '🙎🏿' => 'ирен сирыусы кеше: ҡара тән төҫө',
    '🙅🏻' => 'ЮҠ тип ымлаусы кеше: аҡһыл тән төҫө',
    '🙅🏼' => 'ЮҠ тип ымлаусы кеше: уртаса аҡһыл тән төҫө',
    '🙅🏽' => 'ЮҠ тип ымлаусы кеше: уртаса тән төҫө',
    '🙅🏾' => 'ЮҠ тип ымлаусы кеше: уртаса ҡара тән төҫө',
    '🙅🏿' => 'ЮҠ тип ымлаусы кеше: ҡара тән төҫө',
    '🙆🏻' => 'ЯҠШЫ тип ымлаусы кеше: аҡһыл тән төҫө',
    '🙆🏼' => 'ЯҠШЫ тип ымлаусы кеше: уртаса аҡһыл тән төҫө',
    '🙆🏽' => 'ЯҠШЫ тип ымлаусы кеше: уртаса тән төҫө',
    '🙆🏾' => 'ЯҠШЫ тип ымлаусы кеше: уртаса ҡара тән төҫө',
    '🙆🏿' => 'ЯҠШЫ тип ымлаусы кеше: ҡара тән төҫө',
    '💁🏻' => 'ҡул һуҙған кеше: аҡһыл тән төҫө',
    '💁🏼' => 'ҡул һуҙған кеше: уртаса аҡһыл тән төҫө',
    '💁🏽' => 'ҡул һуҙған кеше: уртаса тән төҫө',
    '💁🏾' => 'ҡул һуҙған кеше: уртаса ҡара тән төҫө',
    '💁🏿' => 'ҡул һуҙған кеше: ҡара тән төҫө',
    '🙋🏻' => 'ҡул күтәреүсе кеше: аҡһыл тән төҫө',
    '🙋🏼' => 'ҡул күтәреүсе кеше: уртаса аҡһыл тән төҫө',
    '🙋🏽' => 'ҡул күтәреүсе кеше: уртаса тән төҫө',
    '🙋🏾' => 'ҡул күтәреүсе кеше: уртаса ҡара тән төҫө',
    '🙋🏿' => 'ҡул күтәреүсе кеше: ҡара тән төҫө',
    '🧏🏻' => 'һаңғырау кеше: аҡһыл тән төҫө',
    '🧏🏼' => 'һаңғырау кеше: уртаса аҡһыл тән төҫө',
    '🧏🏽' => 'һаңғырау кеше: уртаса тән төҫө',
    '🧏🏾' => 'һаңғырау кеше: уртаса ҡара тән төҫө',
    '🧏🏿' => 'һаңғырау кеше: ҡара тән төҫө',
    '🙇🏻' => 'баш эйеүсе кеше: аҡһыл тән төҫө',
    '🙇🏼' => 'баш эйеүсе кеше: уртаса аҡһыл тән төҫө',
    '🙇🏽' => 'баш эйеүсе кеше: уртаса тән төҫө',
    '🙇🏾' => 'баш эйеүсе кеше: уртаса ҡара тән төҫө',
    '🙇🏿' => 'баш эйеүсе кеше: ҡара тән төҫө',
    '🤦🏻' => 'йөҙөн ҡул менән ҡаплаусы кеше: аҡһыл тән төҫө',
    '🤦🏼' => 'йөҙөн ҡул менән ҡаплаусы кеше: уртаса аҡһыл тән төҫө',
    '🤦🏽' => 'йөҙөн ҡул менән ҡаплаусы кеше: уртаса тән төҫө',
    '🤦🏾' => 'йөҙөн ҡул менән ҡаплаусы кеше: уртаса ҡара тән төҫө',
    '🤦🏿' => 'йөҙөн ҡул менән ҡаплаусы кеше: ҡара тән төҫө',
    '🤷🏻' => 'кеше яурынын һикертә: аҡһыл тән төҫө',
    '🤷🏼' => 'кеше яурынын һикертә: уртаса аҡһыл тән төҫө',
    '🤷🏽' => 'кеше яурынын һикертә: уртаса тән төҫө',
    '🤷🏾' => 'кеше яурынын һикертә: уртаса ҡара тән төҫө',
    '🤷🏿' => 'кеше яурынын һикертә: ҡара тән төҫө',
    '👮🏻' => 'полиция хеҙмәткәре: аҡһыл тән төҫө',
    '👮🏼' => 'полиция хеҙмәткәре: уртаса аҡһыл тән төҫө',
    '👮🏽' => 'полиция хеҙмәткәре: уртаса тән төҫө',
    '👮🏾' => 'полиция хеҙмәткәре: уртаса ҡара тән төҫө',
    '👮🏿' => 'полиция хеҙмәткәре: ҡара тән төҫө',
    '🕵🏻' => 'детектив: аҡһыл тән төҫө',
    '🕵🏼' => 'детектив: уртаса аҡһыл тән төҫө',
    '🕵🏽' => 'детектив: уртаса тән төҫө',
    '🕵🏾' => 'детектив: уртаса ҡара тән төҫө',
    '🕵🏿' => 'детектив: ҡара тән төҫө',
    '💂🏻' => 'һаҡсы: аҡһыл тән төҫө',
    '💂🏼' => 'һаҡсы: уртаса аҡһыл тән төҫө',
    '💂🏽' => 'һаҡсы: уртаса тән төҫө',
    '💂🏾' => 'һаҡсы: уртаса ҡара тән төҫө',
    '💂🏿' => 'һаҡсы: ҡара тән төҫө',
    '🥷🏻' => 'ниндзя: аҡһыл тән төҫө',
    '🥷🏼' => 'ниндзя: уртаса аҡһыл тән төҫө',
    '🥷🏽' => 'ниндзя: уртаса тән төҫө',
    '🥷🏾' => 'ниндзя: уртаса ҡара тән төҫө',
    '🥷🏿' => 'ниндзя: ҡара тән төҫө',
    '👷🏻' => 'төҙөүсе: аҡһыл тән төҫө',
    '👷🏼' => 'төҙөүсе: уртаса аҡһыл тән төҫө',
    '👷🏽' => 'төҙөүсе: уртаса тән төҫө',
    '👷🏾' => 'төҙөүсе: уртаса ҡара тән төҫө',
    '👷🏿' => 'төҙөүсе: ҡара тән төҫө',
    '🫅🏻' => 'тажлы кеше: аҡһыл тән төҫө',
    '🫅🏼' => 'тажлы кеше: уртаса аҡһыл тән төҫө',
    '🫅🏽' => 'тажлы кеше: уртаса тән төҫө',
    '🫅🏾' => 'тажлы кеше: уртаса ҡара тән төҫө',
    '🫅🏿' => 'тажлы кеше: ҡара тән төҫө',
    '🤴🏻' => 'принц: аҡһыл тән төҫө',
    '🤴🏼' => 'принц: уртаса аҡһыл тән төҫө',
    '🤴🏽' => 'принц: уртаса тән төҫө',
    '🤴🏾' => 'принц: уртаса ҡара тән төҫө',
    '🤴🏿' => 'принц: ҡара тән төҫө',
    '👸🏻' => 'принцесса: аҡһыл тән төҫө',
    '👸🏼' => 'принцесса: уртаса аҡһыл тән төҫө',
    '👸🏽' => 'принцесса: уртаса тән төҫө',
    '👸🏾' => 'принцесса: уртаса ҡара тән төҫө',
    '👸🏿' => 'принцесса: ҡара тән төҫө',
    '👳🏻' => 'салма кейгән кеше: аҡһыл тән төҫө',
    '👳🏼' => 'салма кейгән кеше: уртаса аҡһыл тән төҫө',
    '👳🏽' => 'салма кейгән кеше: уртаса тән төҫө',
    '👳🏾' => 'салма кейгән кеше: уртаса ҡара тән төҫө',
    '👳🏿' => 'салма кейгән кеше: ҡара тән төҫө',
    '👲🏻' => 'түбәтәй кейгән кеше: аҡһыл тән төҫө',
    '👲🏼' => 'түбәтәй кейгән кеше: уртаса аҡһыл тән төҫө',
    '👲🏽' => 'түбәтәй кейгән кеше: уртаса тән төҫө',
    '👲🏾' => 'түбәтәй кейгән кеше: уртаса ҡара тән төҫө',
    '👲🏿' => 'түбәтәй кейгән кеше: ҡара тән төҫө',
    '🧕🏻' => 'яулыҡлы ҡатын: аҡһыл тән төҫө',
    '🧕🏼' => 'яулыҡлы ҡатын: уртаса аҡһыл тән төҫө',
    '🧕🏽' => 'яулыҡлы ҡатын: уртаса тән төҫө',
    '🧕🏾' => 'яулыҡлы ҡатын: уртаса ҡара тән төҫө',
    '🧕🏿' => 'яулыҡлы ҡатын: ҡара тән төҫө',
    '🤵🏻' => 'смокингтағы кеше: аҡһыл тән төҫө',
    '🤵🏼' => 'смокингтағы кеше: уртаса аҡһыл тән төҫө',
    '🤵🏽' => 'смокингтағы кеше: уртаса тән төҫө',
    '🤵🏾' => 'смокингтағы кеше: уртаса ҡара тән төҫө',
    '🤵🏿' => 'смокингтағы кеше: ҡара тән төҫө',
    '👰🏻' => 'фаталы кеше: аҡһыл тән төҫө',
    '👰🏼' => 'фаталы кеше: уртаса аҡһыл тән төҫө',
    '👰🏽' => 'фаталы кеше: уртаса тән төҫө',
    '👰🏾' => 'фаталы кеше: уртаса ҡара тән төҫө',
    '👰🏿' => 'фаталы кеше: ҡара тән төҫө',
    '🤰🏻' => 'йөклө ҡатын: аҡһыл тән төҫө',
    '🤰🏼' => 'йөклө ҡатын: уртаса аҡһыл тән төҫө',
    '🤰🏽' => 'йөклө ҡатын: уртаса тән төҫө',
    '🤰🏾' => 'йөклө ҡатын: уртаса ҡара тән төҫө',
    '🤰🏿' => 'йөклө ҡатын: ҡара тән төҫө',
    '🫃🏻' => 'йөклө ир: аҡһыл тән төҫө',
    '🫃🏼' => 'йөклө ир: уртаса аҡһыл тән төҫө',
    '🫃🏽' => 'йөклө ир: уртаса тән төҫө',
    '🫃🏾' => 'йөклө ир: уртаса ҡара тән төҫө',
    '🫃🏿' => 'йөклө ир: ҡара тән төҫө',
    '🫄🏻' => 'йөклө кеше: аҡһыл тән төҫө',
    '🫄🏼' => 'йөклө кеше: уртаса аҡһыл тән төҫө',
    '🫄🏽' => 'йөклө кеше: уртаса тән төҫө',
    '🫄🏾' => 'йөклө кеше: уртаса ҡара тән төҫө',
    '🫄🏿' => 'йөклө кеше: ҡара тән төҫө',
    '🤱🏻' => 'имеҙеү: аҡһыл тән төҫө',
    '🤱🏼' => 'имеҙеү: уртаса аҡһыл тән төҫө',
    '🤱🏽' => 'имеҙеү: уртаса тән төҫө',
    '🤱🏾' => 'имеҙеү: уртаса ҡара тән төҫө',
    '🤱🏿' => 'имеҙеү: ҡара тән төҫө',
    '👼🏻' => 'сабый фәрештә: аҡһыл тән төҫө',
    '👼🏼' => 'сабый фәрештә: уртаса аҡһыл тән төҫө',
    '👼🏽' => 'сабый фәрештә: уртаса тән төҫө',
    '👼🏾' => 'сабый фәрештә: уртаса ҡара тән төҫө',
    '👼🏿' => 'сабый фәрештә: ҡара тән төҫө',
    '🎅🏻' => 'Ҡыш бабай: аҡһыл тән төҫө',
    '🎅🏼' => 'Ҡыш бабай: уртаса аҡһыл тән төҫө',
    '🎅🏽' => 'Ҡыш бабай: уртаса тән төҫө',
    '🎅🏾' => 'Ҡыш бабай: уртаса ҡара тән төҫө',
    '🎅🏿' => 'Ҡыш бабай: ҡара тән төҫө',
    '🤶🏻' => 'Клаус ханым: аҡһыл тән төҫө',
    '🤶🏼' => 'Клаус ханым: уртаса аҡһыл тән төҫө',
    '🤶🏽' => 'Клаус ханым: уртаса тән төҫө',
    '🤶🏾' => 'Клаус ханым: уртаса ҡара тән төҫө',
    '🤶🏿' => 'Клаус ханым: ҡара тән төҫө',
    '🦸🏻' => 'супергерой: аҡһыл тән төҫө',
    '🦸🏼' => 'супергерой: уртаса аҡһыл тән төҫө',
    '🦸🏽' => 'супергерой: уртаса тән төҫө',
    '🦸🏾' => 'супергерой: уртаса ҡара тән төҫө',
    '🦸🏿' => 'супергерой: ҡара тән төҫө',
    '🦹🏻' => 'супер яуыз: аҡһыл тән төҫө',
    '🦹🏼' => 'супер яуыз: уртаса аҡһыл тән төҫө',
    '🦹🏽' => 'супер яуыз: уртаса тән төҫө',
    '🦹🏾' => 'супер яуыз: уртаса ҡара тән төҫө',
    '🦹🏿' => 'супер яуыз: ҡара тән төҫө',
    '🧚🏻' => 'фея: аҡһыл тән төҫө',
    '🧚🏼' => 'фея: уртаса аҡһыл тән төҫө',
    '🧚🏽' => 'фея: уртаса тән төҫө',
    '🧚🏾' => 'фея: уртаса ҡара тән төҫө',
    '🧚🏿' => 'фея: ҡара тән төҫө',
    '🧛🏻' => 'вампир: аҡһыл тән төҫө',
    '🧛🏼' => 'вампир: уртаса аҡһыл тән төҫө',
    '🧛🏽' => 'вампир: уртаса тән төҫө',
    '🧛🏾' => 'вампир: уртаса ҡара тән төҫө',
    '🧛🏿' => 'вампир: ҡара тән төҫө',
    '🧜🏻' => 'һыу кешеһе: аҡһыл тән төҫө',
    '🧜🏼' => 'һыу кешеһе: уртаса аҡһыл тән төҫө',
    '🧜🏽' => 'һыу кешеһе: уртаса тән төҫө',
    '🧜🏾' => 'һыу кешеһе: уртаса ҡара тән төҫө',
    '🧜🏿' => 'һыу кешеһе: ҡара тән төҫө',
    '🧝🏻' => 'эльф: аҡһыл тән төҫө',
    '🧝🏼' => 'эльф: уртаса аҡһыл тән төҫө',
    '🧝🏽' => 'эльф: уртаса тән төҫө',
    '🧝🏾' => 'эльф: уртаса ҡара тән төҫө',
    '🧝🏿' => 'эльф: ҡара тән төҫө',
    '💆🏻' => 'массаж алыусы кеше: аҡһыл тән төҫө',
    '💆🏼' => 'массаж алыусы кеше: уртаса аҡһыл тән төҫө',
    '💆🏽' => 'массаж алыусы кеше: уртаса тән төҫө',
    '💆🏾' => 'массаж алыусы кеше: уртаса ҡара тән төҫө',
    '💆🏿' => 'массаж алыусы кеше: ҡара тән төҫө',
    '💇🏻' => 'сәс киҫтереүсе кеше: аҡһыл тән төҫө',
    '💇🏼' => 'сәс киҫтереүсе кеше: уртаса аҡһыл тән төҫө',
    '💇🏽' => 'сәс киҫтереүсе кеше: уртаса тән төҫө',
    '💇🏾' => 'сәс киҫтереүсе кеше: уртаса ҡара тән төҫө',
    '💇🏿' => 'сәс киҫтереүсе кеше: ҡара тән төҫө',
    '🚶🏻' => 'йөрөүсе кеше: аҡһыл тән төҫө',
    '🚶🏼' => 'йөрөүсе кеше: уртаса аҡһыл тән төҫө',
    '🚶🏽' => 'йөрөүсе кеше: уртаса тән төҫө',
    '🚶🏾' => 'йөрөүсе кеше: уртаса ҡара тән төҫө',
    '🚶🏿' => 'йөрөүсе кеше: ҡара тән төҫө',
    '🧍🏻' => 'баҫып торған кеше: аҡһыл тән төҫө',
    '🧍🏼' => 'баҫып торған кеше: уртаса аҡһыл тән төҫө',
    '🧍🏽' => 'баҫып торған кеше: уртаса тән төҫө',
    '🧍🏾' => 'баҫып торған кеше: уртаса ҡара тән төҫө',
    '🧍🏿' => 'баҫып торған кеше: ҡара тән төҫө',
    '🧎🏻' => 'тубыҡланған кеше: аҡһыл тән төҫө',
    '🧎🏼' => 'тубыҡланған кеше: уртаса аҡһыл тән төҫө',
    '🧎🏽' => 'тубыҡланған кеше: уртаса тән төҫө',
    '🧎🏾' => 'тубыҡланған кеше: уртаса ҡара тән төҫө',
    '🧎🏿' => 'тубыҡланған кеше: ҡара тән төҫө',
    '🏃🏻' => 'йүгереүсе кеше: аҡһыл тән төҫө',
    '🏃🏼' => 'йүгереүсе кеше: уртаса аҡһыл тән төҫө',
    '🏃🏽' => 'йүгереүсе кеше: уртаса тән төҫө',
    '🏃🏾' => 'йүгереүсе кеше: уртаса ҡара тән төҫө',
    '🏃🏿' => 'йүгереүсе кеше: ҡара тән төҫө',
    '💃🏻' => 'бейеүсе ҡатын: аҡһыл тән төҫө',
    '💃🏼' => 'бейеүсе ҡатын: уртаса аҡһыл тән төҫө',
    '💃🏽' => 'бейеүсе ҡатын: уртаса тән төҫө',
    '💃🏾' => 'бейеүсе ҡатын: уртаса ҡара тән төҫө',
    '💃🏿' => 'бейеүсе ҡатын: ҡара тән төҫө',
    '🕺🏻' => 'бейеүсе ир-ат: аҡһыл тән төҫө',
    '🕺🏼' => 'бейеүсе ир-ат: уртаса аҡһыл тән төҫө',
    '🕺🏽' => 'бейеүсе ир-ат: уртаса тән төҫө',
    '🕺🏾' => 'бейеүсе ир-ат: уртаса ҡара тән төҫө',
    '🕺🏿' => 'бейеүсе ир-ат: ҡара тән төҫө',
    '🕴🏻' => 'һауала осоусы костюмлы кеше: аҡһыл тән төҫө',
    '🕴🏼' => 'һауала осоусы костюмлы кеше: уртаса аҡһыл тән төҫө',
    '🕴🏽' => 'һауала осоусы костюмлы кеше: уртаса тән төҫө',
    '🕴🏾' => 'һауала осоусы костюмлы кеше: уртаса ҡара тән төҫө',
    '🕴🏿' => 'һауала осоусы костюмлы кеше: ҡара тән төҫө',
    '👯🏻' => 'ҡуян ҡолаҡлы кешеләр: аҡһыл тән төҫө',
    '👯🏼' => 'ҡуян ҡолаҡлы кешеләр: уртаса аҡһыл тән төҫө',
    '👯🏽' => 'ҡуян ҡолаҡлы кешеләр: уртаса тән төҫө',
    '👯🏾' => 'ҡуян ҡолаҡлы кешеләр: уртаса ҡара тән төҫө',
    '👯🏿' => 'ҡуян ҡолаҡлы кешеләр: ҡара тән төҫө',
    '🧖🏻' => 'мунсалағы кеше: аҡһыл тән төҫө',
    '🧖🏼' => 'мунсалағы кеше: уртаса аҡһыл тән төҫө',
    '🧖🏽' => 'мунсалағы кеше: уртаса тән төҫө',
    '🧖🏾' => 'мунсалағы кеше: уртаса ҡара тән төҫө',
    '🧖🏿' => 'мунсалағы кеше: ҡара тән төҫө',
    '🧗🏻' => 'ҡаяға үрмәләүсе кеше: аҡһыл тән төҫө',
    '🧗🏼' => 'ҡаяға үрмәләүсе кеше: уртаса аҡһыл тән төҫө',
    '🧗🏽' => 'ҡаяға үрмәләүсе кеше: уртаса тән төҫө',
    '🧗🏾' => 'ҡаяға үрмәләүсе кеше: уртаса ҡара тән төҫө',
    '🧗🏿' => 'ҡаяға үрмәләүсе кеше: ҡара тән төҫө',
    '🏇🏻' => 'ат сабышы: аҡһыл тән төҫө',
    '🏇🏼' => 'ат сабышы: уртаса аҡһыл тән төҫө',
    '🏇🏽' => 'ат сабышы: уртаса тән төҫө',
    '🏇🏾' => 'ат сабышы: уртаса ҡара тән төҫө',
    '🏇🏿' => 'ат сабышы: ҡара тән төҫө',
    '🏂🏻' => 'сноубордсы: аҡһыл тән төҫө',
    '🏂🏼' => 'сноубордсы: уртаса аҡһыл тән төҫө',
    '🏂🏽' => 'сноубордсы: уртаса тән төҫө',
    '🏂🏾' => 'сноубордсы: уртаса ҡара тән төҫө',
    '🏂🏿' => 'сноубордсы: ҡара тән төҫө',
    '🏌🏻' => 'гольф уйнаусы кеше: аҡһыл тән төҫө',
    '🏌🏼' => 'гольф уйнаусы кеше: уртаса аҡһыл тән төҫө',
    '🏌🏽' => 'гольф уйнаусы кеше: уртаса тән төҫө',
    '🏌🏾' => 'гольф уйнаусы кеше: уртаса ҡара тән төҫө',
    '🏌🏿' => 'гольф уйнаусы кеше: ҡара тән төҫө',
    '🏄🏻' => 'серфингсы кеше: аҡһыл тән төҫө',
    '🏄🏼' => 'серфингсы кеше: уртаса аҡһыл тән төҫө',
    '🏄🏽' => 'серфингсы кеше: уртаса тән төҫө',
    '🏄🏾' => 'серфингсы кеше: уртаса ҡара тән төҫө',
    '🏄🏿' => 'серфингсы кеше: ҡара тән төҫө',
    '🚣🏻' => 'кәмә ишеүсе кеше: аҡһыл тән төҫө',
    '🚣🏼' => 'кәмә ишеүсе кеше: уртаса аҡһыл тән төҫө',
    '🚣🏽' => 'кәмә ишеүсе кеше: уртаса тән төҫө',
    '🚣🏾' => 'кәмә ишеүсе кеше: уртаса ҡара тән төҫө',
    '🚣🏿' => 'кәмә ишеүсе кеше: ҡара тән төҫө',
    '🏊🏻' => 'йөҙөүсе кеше: аҡһыл тән төҫө',
    '🏊🏼' => 'йөҙөүсе кеше: уртаса аҡһыл тән төҫө',
    '🏊🏽' => 'йөҙөүсе кеше: уртаса тән төҫө',
    '🏊🏾' => 'йөҙөүсе кеше: уртаса ҡара тән төҫө',
    '🏊🏿' => 'йөҙөүсе кеше: ҡара тән төҫө',
    '⛹🏻' => 'туп һуғыусы кеше: аҡһыл тән төҫө',
    '⛹🏼' => 'туп һуғыусы кеше: уртаса аҡһыл тән төҫө',
    '⛹🏽' => 'туп һуғыусы кеше: уртаса тән төҫө',
    '⛹🏾' => 'туп һуғыусы кеше: уртаса ҡара тән төҫө',
    '⛹🏿' => 'туп һуғыусы кеше: ҡара тән төҫө',
    '🏋🏻' => 'ауыр атлетика менән шөғөлләнеүсе кеше: аҡһыл тән төҫө',
    '🏋🏼' => 'ауыр атлетика менән шөғөлләнеүсе кеше: уртаса аҡһыл тән төҫө',
    '🏋🏽' => 'ауыр атлетика менән шөғөлләнеүсе кеше: уртаса тән төҫө',
    '🏋🏾' => 'ауыр атлетика менән шөғөлләнеүсе кеше: уртаса ҡара тән төҫө',
    '🏋🏿' => 'ауыр атлетика менән шөғөлләнеүсе кеше: ҡара тән төҫө',
    '🚴🏻' => 'велосипедта йөрөүсе кеше: аҡһыл тән төҫө',
    '🚴🏼' => 'велосипедта йөрөүсе кеше: уртаса аҡһыл тән төҫө',
    '🚴🏽' => 'велосипедта йөрөүсе кеше: уртаса тән төҫө',
    '🚴🏾' => 'велосипедта йөрөүсе кеше: уртаса ҡара тән төҫө',
    '🚴🏿' => 'велосипедта йөрөүсе кеше: ҡара тән төҫө',
    '🚵🏻' => 'тау велосипедында йөрөүсе кеше: аҡһыл тән төҫө',
    '🚵🏼' => 'тау велосипедында йөрөүсе кеше: уртаса аҡһыл тән төҫө',
    '🚵🏽' => 'тау велосипедында йөрөүсе кеше: уртаса тән төҫө',
    '🚵🏾' => 'тау велосипедында йөрөүсе кеше: уртаса ҡара тән төҫө',
    '🚵🏿' => 'тау велосипедында йөрөүсе кеше: ҡара тән төҫө',
    '🤸🏻' => 'кеше ҡулдан ҡулға әйләнә: аҡһыл тән төҫө',
    '🤸🏼' => 'кеше ҡулдан ҡулға әйләнә: уртаса аҡһыл тән төҫө',
    '🤸🏽' => 'кеше ҡулдан ҡулға әйләнә: уртаса тән төҫө',
    '🤸🏾' => 'кеше ҡулдан ҡулға әйләнә: уртаса ҡара тән төҫө',
    '🤸🏿' => 'кеше ҡулдан ҡулға әйләнә: ҡара тән төҫө',
    '🤼🏻' => 'көрәшселәр: аҡһыл тән төҫө',
    '🤼🏼' => 'көрәшселәр: уртаса аҡһыл тән төҫө',
    '🤼🏽' => 'көрәшселәр: уртаса тән төҫө',
    '🤼🏾' => 'көрәшселәр: уртаса ҡара тән төҫө',
    '🤼🏿' => 'көрәшселәр: ҡара тән төҫө',
    '🤽🏻' => 'кеше һыу полоһы уйнай: аҡһыл тән төҫө',
    '🤽🏼' => 'кеше һыу полоһы уйнай: уртаса аҡһыл тән төҫө',
    '🤽🏽' => 'кеше һыу полоһы уйнай: уртаса тән төҫө',
    '🤽🏾' => 'кеше һыу полоһы уйнай: уртаса ҡара тән төҫө',
    '🤽🏿' => 'кеше һыу полоһы уйнай: ҡара тән төҫө',
    '🤾🏻' => 'кеше гандбол уйнай: аҡһыл тән төҫө',
    '🤾🏼' => 'кеше гандбол уйнай: уртаса аҡһыл тән төҫө',
    '🤾🏽' => 'кеше гандбол уйнай: уртаса тән төҫө',
    '🤾🏾' => 'кеше гандбол уйнай: уртаса ҡара тән төҫө',
    '🤾🏿' => 'кеше гандбол уйнай: ҡара тән төҫө',
    '🤹🏻' => 'кеше жонглерлыҡ итә: аҡһыл тән төҫө',
    '🤹🏼' => 'кеше жонглерлыҡ итә: уртаса аҡһыл тән төҫө',
    '🤹🏽' => 'кеше жонглерлыҡ итә: уртаса тән төҫө',
    '🤹🏾' => 'кеше жонглерлыҡ итә: уртаса ҡара тән төҫө',
    '🤹🏿' => 'кеше жонглерлыҡ итә: ҡара тән төҫө',
    '🧘🏻' => 'лотос позаһындағы кеше: аҡһыл тән төҫө',
    '🧘🏼' => 'лотос позаһындағы кеше: уртаса аҡһыл тән төҫө',
    '🧘🏽' => 'лотос позаһындағы кеше: уртаса тән төҫө',
    '🧘🏾' => 'лотос позаһындағы кеше: уртаса ҡара тән төҫө',
    '🧘🏿' => 'лотос позаһындағы кеше: ҡара тән төҫө',
    '🛀🏻' => 'ванна инеүсе кеше: аҡһыл тән төҫө',
    '🛀🏼' => 'ванна инеүсе кеше: уртаса аҡһыл тән төҫө',
    '🛀🏽' => 'ванна инеүсе кеше: уртаса тән төҫө',
    '🛀🏾' => 'ванна инеүсе кеше: уртаса ҡара тән төҫө',
    '🛀🏿' => 'ванна инеүсе кеше: ҡара тән төҫө',
    '🛌🏻' => 'карауаттағы кеше: аҡһыл тән төҫө',
    '🛌🏼' => 'карауаттағы кеше: уртаса аҡһыл тән төҫө',
    '🛌🏽' => 'карауаттағы кеше: уртаса тән төҫө',
    '🛌🏾' => 'карауаттағы кеше: уртаса ҡара тән төҫө',
    '🛌🏿' => 'карауаттағы кеше: ҡара тән төҫө',
    '👭🏻' => 'ҡул тотошҡан ҡатындар: аҡһыл тән төҫө',
    '👭🏼' => 'ҡул тотошҡан ҡатындар: уртаса аҡһыл тән төҫө',
    '👭🏽' => 'ҡул тотошҡан ҡатындар: уртаса тән төҫө',
    '👭🏾' => 'ҡул тотошҡан ҡатындар: уртаса ҡара тән төҫө',
    '👭🏿' => 'ҡул тотошҡан ҡатындар: ҡара тән төҫө',
    '👫🏻' => 'ҡул тотошҡан ҡатын һәм ир: аҡһыл тән төҫө',
    '👫🏼' => 'ҡул тотошҡан ҡатын һәм ир: уртаса аҡһыл тән төҫө',
    '👫🏽' => 'ҡул тотошҡан ҡатын һәм ир: уртаса тән төҫө',
    '👫🏾' => 'ҡул тотошҡан ҡатын һәм ир: уртаса ҡара тән төҫө',
    '👫🏿' => 'ҡул тотошҡан ҡатын һәм ир: ҡара тән төҫө',
    '👬🏻' => 'ҡул тотошҡан ирҙәр: аҡһыл тән төҫө',
    '👬🏼' => 'ҡул тотошҡан ирҙәр: уртаса аҡһыл тән төҫө',
    '👬🏽' => 'ҡул тотошҡан ирҙәр: уртаса тән төҫө',
    '👬🏾' => 'ҡул тотошҡан ирҙәр: уртаса ҡара тән төҫө',
    '👬🏿' => 'ҡул тотошҡан ирҙәр: ҡара тән төҫө',
    '💏🏻' => 'үбеү: аҡһыл тән төҫө',
    '💏🏼' => 'үбеү: уртаса аҡһыл тән төҫө',
    '💏🏽' => 'үбеү: уртаса тән төҫө',
    '💏🏾' => 'үбеү: уртаса ҡара тән төҫө',
    '💏🏿' => 'үбеү: ҡара тән төҫө',
    '💑🏻' => 'йөрәкле пар: аҡһыл тән төҫө',
    '💑🏼' => 'йөрәкле пар: уртаса аҡһыл тән төҫө',
    '💑🏽' => 'йөрәкле пар: уртаса тән төҫө',
    '💑🏾' => 'йөрәкле пар: уртаса ҡара тән төҫө',
    '💑🏿' => 'йөрәкле пар: ҡара тән төҫө',
    '🇦🇨' => 'флаг: Асеншен утрауы',
    '🇦🇩' => 'флаг: Андорра',
    '🇦🇪' => 'флаг: Берләшкән Ғәрәп Әмирлектәре',
    '🇦🇫' => 'флаг: Афғанстан',
    '🇦🇬' => 'флаг: Антигуа һәм Барбуда',
    '🇦🇮' => 'флаг: Ангилья',
    '🇦🇱' => 'флаг: Албания',
    '🇦🇲' => 'флаг: Әрмәнстан',
    '🇦🇴' => 'флаг: Ангола',
    '🇦🇶' => 'флаг: Антарктида',
    '🇦🇷' => 'флаг: Аргентина',
    '🇦🇸' => 'флаг: Америка Самоаһы',
    '🇦🇹' => 'флаг: Австрия',
    '🇦🇺' => 'флаг: Австралия',
    '🇦🇼' => 'флаг: Аруба',
    '🇦🇽' => 'флаг: Аланд утрауҙары',
    '🇦🇿' => 'флаг: Әзербайжан',
    '🇧🇦' => 'флаг: Босния һәм Герцеговина',
    '🇧🇧' => 'флаг: Барбадос',
    '🇧🇩' => 'флаг: Бангладеш',
    '🇧🇪' => 'флаг: Бельгия',
    '🇧🇫' => 'флаг: Буркина-Фасо',
    '🇧🇬' => 'флаг: Болгария',
    '🇧🇭' => 'флаг: Бәхрәйн',
    '🇧🇮' => 'флаг: Бурунди',
    '🇧🇯' => 'флаг: Бенин',
    '🇧🇱' => 'флаг: Сен-Бартелеми',
    '🇧🇲' => 'флаг: Бермуд утрауҙары',
    '🇧🇳' => 'флаг: Бруней',
    '🇧🇴' => 'флаг: Боливия',
    '🇧🇶' => 'флаг: Карибтағы Нидерланд',
    '🇧🇷' => 'флаг: Бразилия',
    '🇧🇸' => 'флаг: Багам утрауҙары',
    '🇧🇹' => 'флаг: Бутан',
    '🇧🇻' => 'флаг: Буве утрауы',
    '🇧🇼' => 'флаг: Ботсвана',
    '🇧🇾' => 'флаг: Беларусь',
    '🇧🇿' => 'флаг: Белиз',
    '🇨🇦' => 'флаг: Канада',
    '🇨🇨' => 'флаг: Кокос утрауҙары',
    '🇨🇩' => 'флаг: Конго - Киншаса',
    '🇨🇫' => 'флаг: Үҙәк Африка Республикаһы',
    '🇨🇬' => 'флаг: Конго - Браззавиль',
    '🇨🇭' => 'флаг: Швейцария',
    '🇨🇮' => 'флаг: Кот-д’Ивуар',
    '🇨🇰' => 'флаг: Кук утрауҙары',
    '🇨🇱' => 'флаг: Чили',
    '🇨🇲' => 'флаг: Камерун',
    '🇨🇳' => 'флаг: Ҡытай',
    '🇨🇴' => 'флаг: Колумбия',
    '🇨🇵' => 'флаг: Клиппертон утрауы',
    '🇨🇶' => 'флаг: Сарк',
    '🇨🇷' => 'флаг: Коста-Рика',
    '🇨🇺' => 'флаг: Куба',
    '🇨🇻' => 'флаг: Кабо-Верде',
    '🇨🇼' => 'флаг: Кюрасао',
    '🇨🇽' => 'флаг: Раштыуа Утрауы',
    '🇨🇾' => 'флаг: Кипр',
    '🇨🇿' => 'флаг: Чехия',
    '🇩🇪' => 'флаг: Германия',
    '🇩🇬' => 'флаг: Диего-Гарсия',
    '🇩🇯' => 'флаг: Джибути',
    '🇩🇰' => 'флаг: Дания',
    '🇩🇲' => 'флаг: Доминика',
    '🇩🇴' => 'флаг: Доминикан Республикаһы',
    '🇩🇿' => 'флаг: Алжир',
    '🇪🇦' => 'флаг: Сеута һәм Мелилья',
    '🇪🇨' => 'флаг: Эквадор',
    '🇪🇪' => 'флаг: Эстония',
    '🇪🇬' => 'флаг: Мысыр',
    '🇪🇭' => 'флаг: Көнбайыш Сахара',
    '🇪🇷' => 'флаг: Эритрея',
    '🇪🇸' => 'флаг: Испания',
    '🇪🇹' => 'флаг: Эфиопия',
    '🇪🇺' => 'флаг: Европа Берлеге',
    '🇫🇮' => 'флаг: Финляндия',
    '🇫🇯' => 'флаг: Фиджи',
    '🇫🇰' => 'флаг: Фолкленд утрауҙары',
    '🇫🇲' => 'флаг: Микронезия',
    '🇫🇴' => 'флаг: Фарер утрауҙары',
    '🇫🇷' => 'флаг: Франция',
    '🇬🇦' => 'флаг: Габон',
    '🇬🇧' => 'флаг: Бөйөк Британия',
    '🇬🇩' => 'флаг: Гренада',
    '🇬🇪' => 'флаг: Грузия',
    '🇬🇫' => 'флаг: Француз Гвианаһы',
    '🇬🇬' => 'флаг: Гернси',
    '🇬🇭' => 'флаг: Гана',
    '🇬🇮' => 'флаг: Гибралтар',
    '🇬🇱' => 'флаг: Гренландия',
    '🇬🇲' => 'флаг: Гамбия',
    '🇬🇳' => 'флаг: Гвинея',
    '🇬🇵' => 'флаг: Гваделупа',
    '🇬🇶' => 'флаг: Экваториаль Гвинея',
    '🇬🇷' => 'флаг: Греция',
    '🇬🇸' => 'флаг: Көньяҡ Георгия һәм Көньяҡ Сандвич утрауҙары',
    '🇬🇹' => 'флаг: Гватемала',
    '🇬🇺' => 'флаг: Гуам',
    '🇬🇼' => 'флаг: Гвинея-Бисау',
    '🇬🇾' => 'флаг: Гайана',
    '🇭🇰' => 'флаг: Гонконг, Ҡытай МАР-ы',
    '🇭🇲' => 'флаг: Херд һәм Макдональд утрауҙары',
    '🇭🇳' => 'флаг: Гондурас',
    '🇭🇷' => 'флаг: Хорватия',
    '🇭🇹' => 'флаг: Гаити',
    '🇭🇺' => 'флаг: Венгрия',
    '🇮🇨' => 'флаг: Канар утрауҙары',
    '🇮🇩' => 'флаг: Индонезия',
    '🇮🇪' => 'флаг: Ирландия',
    '🇮🇱' => 'флаг: Израиль',
    '🇮🇲' => 'флаг: Мэн утрауы',
    '🇮🇳' => 'флаг: Һиндостан',
    '🇮🇴' => 'флаг: Британияның Һинд океанындағы биләмәһе',
    '🇮🇶' => 'флаг: Ираҡ',
    '🇮🇷' => 'флаг: Иран',
    '🇮🇸' => 'флаг: Исландия',
    '🇮🇹' => 'флаг: Италия',
    '🇯🇪' => 'флаг: Джерси',
    '🇯🇲' => 'флаг: Ямайка',
    '🇯🇴' => 'флаг: Иордания',
    '🇯🇵' => 'флаг: Япония',
    '🇰🇪' => 'флаг: Кения',
    '🇰🇬' => 'флаг: Ҡырғыҙстан',
    '🇰🇭' => 'флаг: Камбоджа',
    '🇰🇮' => 'флаг: Кирибати',
    '🇰🇲' => 'флаг: Комор утрауҙары',
    '🇰🇳' => 'флаг: Сент-Китс һәм Невис',
    '🇰🇵' => 'флаг: Төньяҡ Корея',
    '🇰🇷' => 'флаг: Көньяҡ Корея',
    '🇰🇼' => 'флаг: Кувейт',
    '🇰🇾' => 'флаг: Кайман утрауҙары',
    '🇰🇿' => 'флаг: Ҡаҙағстан',
    '🇱🇦' => 'флаг: Лаос',
    '🇱🇧' => 'флаг: Ливан',
    '🇱🇨' => 'флаг: Сент-Люсия',
    '🇱🇮' => 'флаг: Лихтенштейн',
    '🇱🇰' => 'флаг: Шри-Ланка',
    '🇱🇷' => 'флаг: Либерия',
    '🇱🇸' => 'флаг: Лесото',
    '🇱🇹' => 'флаг: Литва',
    '🇱🇺' => 'флаг: Люксембург',
    '🇱🇻' => 'флаг: Латвия',
    '🇱🇾' => 'флаг: Ливия',
    '🇲🇦' => 'флаг: Марокко',
    '🇲🇨' => 'флаг: Монако',
    '🇲🇩' => 'флаг: Молдова',
    '🇲🇪' => 'флаг: Черногория',
    '🇲🇫' => 'флаг: Сен-Мартен',
    '🇲🇬' => 'флаг: Мадагаскар',
    '🇲🇭' => 'флаг: Маршалл утрауҙары',
    '🇲🇰' => 'флаг: Төньяҡ Македония',
    '🇲🇱' => 'флаг: Мали',
    '🇲🇲' => 'флаг: Мьянма (Бирма)',
    '🇲🇳' => 'флаг: Монголия',
    '🇲🇴' => 'флаг: Макао, Ҡытай МАР-ы',
    '🇲🇵' => 'флаг: Төньяҡ Мариан утрауҙары',
    '🇲🇶' => 'флаг: Мартиника',
    '🇲🇷' => 'флаг: Мавритания',
    '🇲🇸' => 'флаг: Монтсеррат',
    '🇲🇹' => 'флаг: Мальта',
    '🇲🇺' => 'флаг: Маврикий',
    '🇲🇻' => 'флаг: Мальдив утрауҙары',
    '🇲🇼' => 'флаг: Малави',
    '🇲🇽' => 'флаг: Мексика',
    '🇲🇾' => 'флаг: Малайзия',
    '🇲🇿' => 'флаг: Мозамбик',
    '🇳🇦' => 'флаг: Намибия',
    '🇳🇨' => 'флаг: Яңы Каледония',
    '🇳🇪' => 'флаг: Нигер',
    '🇳🇫' => 'флаг: Норфолк утрауы',
    '🇳🇬' => 'флаг: Нигерия',
    '🇳🇮' => 'флаг: Никарагуа',
    '🇳🇱' => 'флаг: Нидерланд',
    '🇳🇴' => 'флаг: Норвегия',
    '🇳🇵' => 'флаг: Непал',
    '🇳🇷' => 'флаг: Науру',
    '🇳🇺' => 'флаг: Ниуэ',
    '🇳🇿' => 'флаг: Яңы Зеландия',
    '🇴🇲' => 'флаг: Оман',
    '🇵🇦' => 'флаг: Панама',
    '🇵🇪' => 'флаг: Перу',
    '🇵🇫' => 'флаг: Француз Полинезияһы',
    '🇵🇬' => 'флаг: Папуа – Яңы Гвинея',
    '🇵🇭' => 'флаг: Филиппин',
    '🇵🇰' => 'флаг: Пакистан',
    '🇵🇱' => 'флаг: Польша',
    '🇵🇲' => 'флаг: Сен-Пьер һәм Микелон',
    '🇵🇳' => 'флаг: Питкэрн утрауҙары',
    '🇵🇷' => 'флаг: Пуэрто-Рико',
    '🇵🇸' => 'флаг: Фәләстин территориялары',
    '🇵🇹' => 'флаг: Португалия',
    '🇵🇼' => 'флаг: Палау',
    '🇵🇾' => 'флаг: Парагвай',
    '🇶🇦' => 'флаг: Катар',
    '🇷🇪' => 'флаг: Реюньон',
    '🇷🇴' => 'флаг: Румыния',
    '🇷🇸' => 'флаг: Сербия',
    '🇷🇺' => 'флаг: Рәсәй',
    '🇷🇼' => 'флаг: Руанда',
    '🇸🇦' => 'флаг: Сәғүд Ғәрәбстаны',
    '🇸🇧' => 'флаг: Соломон утрауҙары',
    '🇸🇨' => 'флаг: Сейшель утрауҙары',
    '🇸🇩' => 'флаг: Судан',
    '🇸🇪' => 'флаг: Швеция',
    '🇸🇬' => 'флаг: Сингапур',
    '🇸🇭' => 'флаг: Изге Елена утрауы',
    '🇸🇮' => 'флаг: Словения',
    '🇸🇯' => 'флаг: Шпицберген һәм Ян-Майен',
    '🇸🇰' => 'флаг: Словакия',
    '🇸🇱' => 'флаг: Сьерра-Леоне',
    '🇸🇲' => 'флаг: Сан-Марино',
    '🇸🇳' => 'флаг: Сенегал',
    '🇸🇴' => 'флаг: Сомали',
    '🇸🇷' => 'флаг: Суринам',
    '🇸🇸' => 'флаг: Көньяҡ Судан',
    '🇸🇹' => 'флаг: Сан-Томе һәм Принсипи',
    '🇸🇻' => 'флаг: Сальвадор',
    '🇸🇽' => 'флаг: Синт-Мартен',
    '🇸🇾' => 'флаг: Сүриә',
    '🇸🇿' => 'флаг: Эсватини',
    '🇹🇦' => 'флаг: Тристан-да-Кунья',
    '🇹🇨' => 'флаг: Төркс һәм Кайкос утрауҙары',
    '🇹🇩' => 'флаг: Чад',
    '🇹🇫' => 'флаг: Францияның Көньяҡ Биләмәләре',
    '🇹🇬' => 'флаг: Того',
    '🇹🇭' => 'флаг: Тайланд',
    '🇹🇯' => 'флаг: Тажикстан',
    '🇹🇰' => 'флаг: Токелау',
    '🇹🇱' => 'флаг: Тимор-Лесте',
    '🇹🇲' => 'флаг: Төркмәнстан',
    '🇹🇳' => 'флаг: Тунис',
    '🇹🇴' => 'флаг: Тонга',
    '🇹🇷' => 'флаг: Төркиә',
    '🇹🇹' => 'флаг: Тринидад һәм Тобаго',
    '🇹🇻' => 'флаг: Тувалу',
    '🇹🇼' => 'флаг: Тайвань',
    '🇹🇿' => 'флаг: Танзания',
    '🇺🇦' => 'флаг: Украина',
    '🇺🇬' => 'флаг: Уганда',
    '🇺🇲' => 'флаг: АҠШ-ның ситтәге утрауҙары',
    '🇺🇳' => 'флаг: Берләшкән Милләттәр Ойошмаһы',
    '🇺🇸' => 'флаг: Америка Ҡушма Штаттары',
    '🇺🇾' => 'флаг: Уругвай',
    '🇺🇿' => 'флаг: Үзбәкстан',
    '🇻🇦' => 'флаг: Ватикан',
    '🇻🇨' => 'флаг: Сент-Винсент һәм Гренадиндар',
    '🇻🇪' => 'флаг: Венесуэла',
    '🇻🇬' => 'флаг: Британия Виргин утрауҙары',
    '🇻🇮' => 'флаг: АҠШ Виргин утрауҙары',
    '🇻🇳' => 'флаг: Вьетнам',
    '🇻🇺' => 'флаг: Вануату',
    '🇼🇫' => 'флаг: Уоллис һәм Футуна',
    '🇼🇸' => 'флаг: Самоа',
    '🇽🇰' => 'флаг: Косово',
    '🇾🇪' => 'флаг: Йәмән',
    '🇾🇹' => 'флаг: Майотта',
    '🇿🇦' => 'флаг: Көньяҡ Африка Республикаһы',
    '🇿🇲' => 'флаг: Замбия',
    '🇿🇼' => 'флаг: Зимбабве',
    '🔟' => 'клавиша: 10',
    '🏻' => 'аҡһыл тән төҫө',
    '🏼' => 'уртаса аҡһыл тән төҫө',
    '🏽' => 'уртаса тән төҫө',
    '🏾' => 'уртаса ҡара тән төҫө',
    '🏿' => 'ҡара тән төҫө',
    '😃' => 'ҙур күҙле йылмайыусы йөҙ',
    '😄' => 'ҡыҫыҡ күҙле йылмайыусы йөҙ',
    '😁' => 'тештәре менән йылмайыусы йөҙ',
    '😆' => 'ҡыҫылған күҙҙәр менән йылмайыусы йөҙ',
    '😅' => 'тирләгән йылмайыусы йөҙ',
    '🤣' => 'иҙәндә көлөү',
    '😂' => 'йәш түгеп шатланған йөҙ',
    '🙂' => 'аҙ ғына йылмайыусы йөҙ',
    '🙃' => 'баштүбән йөҙ',
    '🫠' => 'иреүсе йөҙ',
    '😉' => 'күҙ ҡыҫыусы йөҙ',
    '😊' => 'йылмайыусы күҙҙәр менән йылмайыусы йөҙ',
    '😇' => 'нимблы йылмайыусы йөҙ',
    '🥰' => 'йөрәктәр менән йылмайыусы йөҙ',
    '😍' => 'йөрәк күҙҙәре менән йылмайыусы йөҙ',
    '🤩' => 'йондоҙҙар менән һуғылған',
    '😘' => 'һауа үбеүе ебәреүсе йөҙ',
    '😗' => 'үбеүсе йөҙ',
    '☺' => 'йылмайыусы йөҙ',
    '😚' => 'йомоҡ күҙҙәр менән үбеүсе йөҙ',
    '😙' => 'йылмайыусы күҙҙәр менән үбеүсе йөҙ',
    '🥲' => 'күҙ йәше менән йылмайыусы йөҙ',
    '😋' => 'ашамлыҡтан ләззәтләнеүсе йөҙ',
    '😛' => 'тел менән йөҙ',
    '😜' => 'тел менән күҙ ҡыҫыусы йөҙ',
    '🤪' => 'аҡылһыҙ йөҙ',
    '😝' => 'тел менән ҡыҫылған йөҙ',
    '🤑' => 'аҡса ауыҙлы йөҙ',
    '🤗' => 'асыҡ ҡулдар менән йылмайыусы йөҙ',
    '🤭' => 'ауыҙын ҡулы менән ҡаплаған йөҙ',
    '🫢' => 'асыҡ күҙле һәм ауыҙын ҡулы менән ҡаплаған йөҙ',
    '🫣' => 'күҙ һалып торған йөҙ',
    '🤫' => 'шым бул йөҙө',
    '🤔' => 'уйланыусы йөҙ',
    '🫡' => 'сәләмләүсе йөҙ',
    '🤐' => 'эләктергес ауыҙлы йөҙ',
    '🤨' => 'күтәрелгән ҡашлы йөҙ',
    '😐' => 'нейтраль йөҙ',
    '😑' => 'хистәрһеҙ йөҙ',
    '😶' => 'ауыҙһыҙ йөҙ',
    '🫥' => 'пунктир һыҙыҡлы йөҙ',
    '😏' => 'мыҫҡыллы йөҙ',
    '😒' => 'ҡәнәғәтһеҙ йөҙ',
    '🙄' => 'күҙ әйләнеүсе йөҙ',
    '🤥' => 'алдаусы йөҙ',
    '🫨' => 'ҡалтыраусы йөҙ',
    '😌' => 'еңеллек тойған йөҙ',
    '😔' => 'уйсан йөҙ',
    '😪' => 'йоҡоһо килгән йөҙ',
    '🤤' => 'шайыҡ ағыусы ауыҙ',
    '😴' => 'йоҡлаусы йөҙ',
    '🫩' => 'күҙ аҫтында тоҡсайҙар булған йөҙ',
    '😷' => 'медицина битлеге менән йөҙ',
    '🤒' => 'термометрлы йөҙ',
    '🤕' => 'баш бәйләгес менән йөҙ',
    '🤢' => 'ҡоҫҡоһо килгән йөҙ',
    '🤮' => 'ҡоҫҡан йөҙ',
    '🤧' => 'сөскөрөүсе йөҙ',
    '🥵' => 'ҡайнар йөҙ',
    '🥶' => 'һыуыҡ йөҙ',
    '🥴' => 'баш әйләнеүсе йөҙ',
    '😵' => 'күҙҙәре крест менән һыҙылған йөҙ',
    '🤯' => 'баш шартлауы',
    '🤠' => 'ковбой шляпалы йөҙ',
    '🥳' => 'кисәләге йөҙ',
    '🥸' => 'йәшерелгән йөҙ',
    '😎' => 'ҡояш күҙлеге менән йылмайыусы йөҙ',
    '🤓' => 'күҙлекле йөҙ',
    '🧐' => 'монокль менән йөҙ',
    '😕' => 'буталған йөҙ',
    '🫤' => 'диагональ ауыҙлы йөҙ',
    '😟' => 'борсоулы йөҙ',
    '🙁' => 'әҙ генә ҡашын йыйырған йөҙ',
    '☹' => 'йөҙөн йыйырыу',
    '😮' => 'асыҡ ауыҙлы йөҙ',
    '😯' => 'тауышһыҙ йөҙ',
    '😲' => 'шаҡ ҡатҡан йөҙ',
    '😳' => 'ҡыҙарған йөҙ',
    '🫪' => 'бәлшәйгән йөҙ',
    '🥺' => 'ялбарыусы йөҙ',
    '🥹' => 'күҙ йәштәрен тыйыусы йөҙ',
    '😦' => 'асыҡ ауыҙлы ҡаш йыйған йөҙ',
    '😧' => 'ғазаплы йөҙ',
    '😨' => 'ҡурҡҡан йөҙ',
    '😰' => 'тирләгән борсоулы йөҙ',
    '😥' => 'моңһоу, әммә еңеллек тойған йөҙ',
    '😢' => 'илаусы йөҙ',
    '😭' => 'ҡысҡырып илаусы йөҙ',
    '😱' => 'ҡурҡыуҙан ҡысҡырыусы йөҙ',
    '😖' => 'аптыраған йөҙ',
    '😣' => 'ныҡышмалы йөҙ',
    '😞' => 'күңеле ҡайтҡан йөҙ',
    '😓' => 'тирләгән төшөнкө йөҙ',
    '😩' => 'арыған йөҙ',
    '🥱' => 'иҫнәүсе йөҙ',
    '😤' => 'танауҙан пар сыҡҡан йөҙ',
    '😡' => 'ярһыған йөҙ',
    '😠' => 'асыулы йөҙ',
    '🤬' => 'ауыҙында символдар булған йөҙ',
    '😈' => 'мөгөҙлө йылмайыусы йөҙ',
    '👿' => 'мөгөҙлө асыулы йөҙ',
    '💀' => 'баш һөйәге',
    '☠' => 'баш һөйәге һәм һөйәктәр',
    '💩' => 'тиҙәк өйөмө',
    '🤡' => 'клоун йөҙө',
    '👹' => 'огр',
    '👺' => 'гоблин',
    '👻' => 'өрәк',
    '👽' => 'сит планета кешеһе',
    '👾' => 'сит планеталы монстр',
    '🤖' => 'робот',
    '😺' => 'йылмайыусы бесәй',
    '😸' => 'көлөүсе бесәй',
    '😹' => 'шатлыҡ йәштәре менән бесәй',
    '😻' => 'йөрәк күҙҙәре менән йылмайыусы бесәй',
    '😼' => 'мыҫҡыллы йылмайыусы бесәй',
    '😽' => 'үбеүсе бесәй',
    '🙀' => 'арыған бесәй',
    '😿' => 'илаусы бесәй',
    '😾' => 'үпкәләгән бесәй',
    '🙈' => 'күрмәҫкә тырышҡан маймыл',
    '🙉' => 'ишетмәҫкә тырышҡан маймыл',
    '🙊' => 'ауыҙын япҡан маймыл',
    '💌' => 'мөхәббәт хаты',
    '💘' => 'уҡлы йөрәк',
    '💝' => 'таҫмалы йөрәк',
    '💖' => 'ялтырауыҡлы йөрәк',
    '💗' => 'үҫеүсе йөрәк',
    '💓' => 'тибеүсе йөрәк',
    '💞' => 'әйләнеүсе йөрәктәр',
    '💕' => 'ике йөрәк',
    '💟' => 'йөрәк биҙәге',
    '❣' => 'йөрәк өндәүе',
    '💔' => 'ватыҡ йөрәк',
    '❤' => 'ҡыҙыл йөрәк',
    '🩷' => 'алһыу йөрәк',
    '🧡' => 'ҡыҙғылт һары йөрәк',
    '💛' => 'һары йөрәк',
    '💚' => 'йәшел йөрәк',
    '💙' => 'күк йөрәк',
    '🩵' => 'зәңгәр йөрәк',
    '💜' => 'шәмәхә йөрәк',
    '🤎' => 'көрән йөрәк',
    '🖤' => 'ҡара йөрәк',
    '🩶' => 'һоро йөрәк',
    '🤍' => 'аҡ йөрәк',
    '💋' => 'үбешеү эҙе',
    '💯' => 'йөҙ балл',
    '💢' => 'асыуланыу символы',
    '🫯' => 'һуғыш болото',
    '💥' => 'шартлау',
    '💫' => 'өйөрөлөүсе йондоҙ',
    '💦' => 'тир тамсылары',
    '💨' => 'йүгереп китеү',
    '🕳' => 'тишек',
    '💬' => 'һөйләшеү ҡыуығы',
    '🗨' => 'һул һөйләм ҡыуығы',
    '🗯' => 'уң яҡ асыулы ҡыуыҡ',
    '💭' => 'уй ҡыуығы',
    '💤' => 'ZZZ',
    '👋' => 'ҡул болғау',
    '🤚' => 'күтәрелгән ҡул арҡаһы',
    '🖐' => 'бармаҡтары йәйелгән ҡул',
    '✋' => 'күтәрелгән ҡул',
    '🖖' => 'вулкан сәләме',
    '🫱' => 'уңға ҡул',
    '🫲' => 'һулға ҡул',
    '🫳' => 'ус төбө аҫҡа ҡараған ҡул',
    '🫴' => 'ус төбө өҫкә ҡараған ҡул',
    '🫷' => 'һулға этәреүсе ҡул',
    '🫸' => 'уңға этәреүсе ҡул',
    '👌' => 'OK ҡул',
    '🤌' => 'ҡыҫылған бармаҡтар',
    '🤏' => 'ҡыҫыусы ҡул',
    '✌' => 'еңеү ҡулы',
    '🤞' => 'киҫешкән бармаҡтар',
    '🫰' => 'аҡса ишаралаусы ҡул',
    '🤟' => 'мин һине яратам ымлауы',
    '🤘' => 'мөгөҙ билдәһе',
    '🤙' => 'миңә шылтырат ҡулы',
    '👈' => 'һулға күрһәтеүсе ҡул арты',
    '👉' => 'уңға күрһәтеүсе ҡул арты',
    '👆' => 'өҫкә күрһәтеүсе ҡул арты',
    '🖕' => 'урта бармаҡ',
    '👇' => 'аҫҡа күрһәтеүсе ҡул арты',
    '☝' => 'өҫкә күрһәтеүсе һуҡ бармаҡ',
    '🫵' => 'ҡараусыға күрһәтеүсе бармаҡ',
    '👍' => 'баш бармаҡ өҫкә',
    '👎' => 'баш бармаҡ аҫҡа',
    '✊' => 'күтәрелгән йоҙроҡ',
    '👊' => 'ҡаршы килеүсе йоҙроҡ',
    '🤛' => 'һулға ҡараған йоҙроҡ',
    '🤜' => 'уңға ҡараған йоҙроҡ',
    '👏' => 'ҡул сабыу',
    '🙌' => 'ҡулдарҙы күтәреү',
    '🫶' => 'йөрәк ҡулдар',
    '👐' => 'асыҡ ҡулдар',
    '🤲' => 'ус төптәре бергә',
    '🤝' => 'ҡул бирешеү',
    '🙏' => 'ҡулдарҙы бергә тотоу',
    '✍' => 'яҙыусы ҡул',
    '💅' => 'тырнаҡтарҙы буяу',
    '🤳' => 'селфи',
    '💪' => 'бицепс күрһәтеүсе ҡул',
    '🦾' => 'механик ҡул',
    '🦿' => 'механик аяҡ',
    '🦵' => 'аяҡ',
    '👂' => 'ҡолаҡ',
    '🦻' => 'ишетеү аппараты менән ҡолаҡ',
    '👃' => 'танау',
    '🧠' => 'мейе',
    '🫀' => 'кеше йөрәге',
    '🫁' => 'үпкә',
    '🦷' => 'теш',
    '🦴' => 'һөйәк',
    '👀' => 'күҙҙәр',
    '👁' => 'күҙ',
    '👅' => 'тел',
    '👄' => 'ауыҙ',
    '🫦' => 'ирен тешләү',
    '👶' => 'сабый',
    '🧒' => 'бала',
    '👦' => 'малай',
    '👧' => 'ҡыҙ',
    '🧑' => 'өлкән',
    '👱' => 'кеше: аҡһыл сәсле',
    '👨' => 'ир',
    '🧔' => 'һаҡаллы кеше',
    '👩' => 'ҡатын',
    '🧓' => 'оло кеше',
    '👴' => 'ҡарт ир',
    '👵' => 'ҡарт ҡатын',
    '🙍' => 'йөҙ сирыусы кеше',
    '🙎' => 'ирен сирыусы кеше',
    '🙅' => 'ЮҠ тип ымлаусы кеше',
    '🙆' => 'ЯҠШЫ тип ымлаусы кеше',
    '💁' => 'ҡул һуҙған кеше',
    '🙋' => 'ҡул күтәреүсе кеше',
    '🧏' => 'һаңғырау кеше',
    '🙇' => 'баш эйеүсе кеше',
    '🤦' => 'йөҙөн ҡул менән ҡаплаусы кеше',
    '🤷' => 'кеше яурынын һикертә',
    '👮' => 'полиция хеҙмәткәре',
    '🕵' => 'детектив',
    '💂' => 'һаҡсы',
    '🥷' => 'ниндзя',
    '👷' => 'төҙөүсе',
    '🫅' => 'тажлы кеше',
    '🤴' => 'принц',
    '👸' => 'принцесса',
    '👳' => 'салма кейгән кеше',
    '👲' => 'түбәтәй кейгән кеше',
    '🧕' => 'яулыҡлы ҡатын',
    '🤵' => 'смокингтағы кеше',
    '👰' => 'фаталы кеше',
    '🤰' => 'йөклө ҡатын',
    '🫃' => 'йөклө ир',
    '🫄' => 'йөклө кеше',
    '🤱' => 'имеҙеү',
    '👼' => 'сабый фәрештә',
    '🎅' => 'Ҡыш бабай',
    '🤶' => 'Клаус ханым',
    '🦸' => 'супергерой',
    '🦹' => 'супер яуыз',
    '🧚' => 'фея',
    '🧛' => 'вампир',
    '🧜' => 'һыу кешеһе',
    '🧝' => 'эльф',
    '🧞' => 'джин',
    '🧟' => 'зомби',
    '🧌' => 'тролль',
    '🫈' => 'йөнлө йән эйәһе',
    '💆' => 'массаж алыусы кеше',
    '💇' => 'сәс киҫтереүсе кеше',
    '🚶' => 'йөрөүсе кеше',
    '🧍' => 'баҫып торған кеше',
    '🧎' => 'тубыҡланған кеше',
    '🏃' => 'йүгереүсе кеше',
    '💃' => 'бейеүсе ҡатын',
    '🕺' => 'бейеүсе ир-ат',
    '🕴' => 'һауала осоусы костюмлы кеше',
    '👯' => 'ҡуян ҡолаҡлы кешеләр',
    '🧖' => 'мунсалағы кеше',
    '🧗' => 'ҡаяға үрмәләүсе кеше',
    '🤺' => 'кеше фехтование менән шөғөлләнә',
    '🏇' => 'ат сабышы',
    '⛷' => 'саңғысы',
    '🏂' => 'сноубордсы',
    '🏌' => 'гольф уйнаусы кеше',
    '🏄' => 'серфингсы кеше',
    '🚣' => 'кәмә ишеүсе кеше',
    '🏊' => 'йөҙөүсе кеше',
    '⛹' => 'туп һуғыусы кеше',
    '🏋' => 'ауыр атлетика менән шөғөлләнеүсе кеше',
    '🚴' => 'велосипедта йөрөүсе кеше',
    '🚵' => 'тау велосипедында йөрөүсе кеше',
    '🤸' => 'кеше ҡулдан ҡулға әйләнә',
    '🤼' => 'көрәшселәр',
    '🤽' => 'кеше һыу полоһы уйнай',
    '🤾' => 'кеше гандбол уйнай',
    '🤹' => 'кеше жонглерлыҡ итә',
    '🧘' => 'лотос позаһындағы кеше',
    '🛀' => 'ванна инеүсе кеше',
    '🛌' => 'карауаттағы кеше',
    '👭' => 'ҡул тотошҡан ҡатындар',
    '👫' => 'ҡул тотошҡан ҡатын һәм ир',
    '👬' => 'ҡул тотошҡан ирҙәр',
    '💏' => 'үбеү',
    '💑' => 'йөрәкле пар',
    '🗣' => 'һөйләшеүсе баш',
    '👤' => 'кеше силуэты',
    '👥' => 'ике силуэт',
    '🫂' => 'ҡосаҡлашҡан кешеләр',
    '👪' => 'ғаилә',
    '👣' => 'аяҡ эҙҙәре',
    '🫆' => 'бармаҡ эҙе',
    '🦰' => 'ерән сәс',
    '🦱' => 'бөҙрә сәс',
    '🦳' => 'аҡ сәс',
    '🦲' => 'таҙ',
    '🐵' => 'маймыл башы',
    '🐒' => 'маймыл',
    '🦍' => 'горилла',
    '🦧' => 'орангутан',
    '🐶' => 'эт башы',
    '🐕' => 'эт',
    '🦮' => 'юл күрһәтеүсе эт',
    '🐩' => 'пудель',
    '🐺' => 'бүре башы',
    '🦊' => 'төлкө башы',
    '🦝' => 'янут башы',
    '🐱' => 'бесәй башы',
    '🐈' => 'бесәй',
    '🦁' => 'арыҫлан башы',
    '🐯' => 'юлбарыҫ башы',
    '🐅' => 'юлбарыҫ',
    '🐆' => 'леопард',
    '🐴' => 'ат башы',
    '🫎' => 'мышы',
    '🫏' => 'ишәк',
    '🐎' => 'ат',
    '🦄' => 'һыңармөгөҙ башы',
    '🦓' => 'зебра',
    '🦌' => 'болан',
    '🦬' => 'бизон',
    '🐮' => 'һыйыр башы',
    '🐂' => 'үгеҙ',
    '🐃' => 'азия буйволы',
    '🐄' => 'һыйыр',
    '🐷' => 'сусҡа башы',
    '🐖' => 'сусҡа',
    '🐗' => 'ҡабан башы',
    '🐽' => 'сусҡа танауы',
    '🐏' => 'һарыҡ тәкәһе',
    '🐑' => 'һарыҡ',
    '🐐' => 'кәзә',
    '🐪' => 'дөйә',
    '🐫' => 'ике үркәсле дөйә',
    '🦙' => 'лама',
    '🦒' => 'жираф',
    '🐘' => 'фил',
    '🦣' => 'мамонт',
    '🦏' => 'мөгөҙморон',
    '🦛' => 'гиппопотам',
    '🐭' => 'сысҡан башы',
    '🐀' => 'сысҡан',
    '🐹' => 'алйырҙан',
    '🐰' => 'ҡуян башы',
    '🐇' => 'ҡуян',
    '🐿' => 'әлгәнйәк',
    '🦫' => 'ҡондоҙ',
    '🦔' => 'терпе',
    '🦇' => 'ярғанат',
    '🐻' => 'айыу башы',
    '🐨' => 'коала башы',
    '🐼' => 'панда башы',
    '🦥' => 'ялҡау маймыл',
    '🦦' => 'ҡама',
    '🦨' => 'скунс',
    '🦘' => 'кенгуру',
    '🦡' => 'бурһыҡ',
    '🐾' => 'тәпәй эҙҙәре',
    '🦃' => 'күркә',
    '🐔' => 'тауыҡ',
    '🐓' => 'әтәс',
    '🐣' => 'йомортҡанан сыҡҡан себеш',
    '🐤' => 'себеш',
    '🐥' => 'алға ҡараған себеш',
    '🐦' => 'ҡош',
    '🐧' => 'пингвин',
    '🕊' => 'күгәрсен',
    '🦅' => 'бөркөт',
    '🦆' => 'өйрәк',
    '🦢' => 'аҡҡош',
    '🦉' => 'өкө',
    '🦤' => 'додо',
    '🪶' => 'ҡауырһын',
    '🦩' => 'фламинго',
    '🦚' => 'тауис',
    '🦜' => 'тутыйғош',
    '🪽' => 'ҡанат',
    '🪿' => 'ҡаҙ',
    '🐸' => 'тәлмәрйен башы',
    '🐊' => 'крокодил',
    '🐢' => 'ташбаҡа',
    '🦎' => 'кеҫәртке',
    '🐍' => 'йылан',
    '🐲' => 'аждаһа башы',
    '🐉' => 'аждаһа',
    '🦕' => 'зауропод',
    '🦖' => 'Ти-Рекс',
    '🐳' => 'фонтанлы кит',
    '🐋' => 'кит',
    '🐬' => 'дельфин',
    '🫍' => 'косатка',
    '🦭' => 'тюлень',
    '🐟' => 'балыҡ',
    '🐠' => 'тропик балыҡ',
    '🐡' => 'энәбалыҡ',
    '🦈' => 'акула',
    '🐙' => 'һигеҙаяҡ',
    '🐚' => 'спираль ҡабырсаҡ',
    '🪸' => 'коралл',
    '🪼' => 'медуза',
    '🦀' => 'краб',
    '🦞' => 'омар',
    '🦐' => 'креветка',
    '🦑' => 'кальмар',
    '🦪' => 'устрица',
    '🐌' => 'ҡусҡар',
    '🦋' => 'күбәләк',
    '🐛' => 'ҡарышлауыҡ',
    '🐜' => 'ҡырмыҫҡа',
    '🐝' => 'бал ҡорто',
    '🐞' => 'ҡуңыҙ',
    '🦗' => 'сиңерткә',
    '🪳' => 'тараҡан',
    '🕷' => 'үрмәксе',
    '🕸' => 'үрмәксе ауы',
    '🦂' => 'саян',
    '🦟' => 'серәкәй',
    '🪰' => 'себен',
    '🪱' => 'селәүсен',
    '🦠' => 'микроб',
    '💐' => 'букет',
    '🌸' => 'сейә сәскәһе',
    '💮' => 'аҡ сәскә',
    '🪷' => 'лотос',
    '🏵' => 'розетка',
    '🌹' => 'рауза',
    '🥀' => 'һулыған сәскә',
    '🌺' => 'гибискус',
    '🌻' => 'көнбағыш',
    '🌼' => 'сәскә',
    '🌷' => 'ләлә',
    '🪻' => 'гиацинт',
    '🌱' => 'үҫенте',
    '🪴' => 'һауыттағы үҫемлек',
    '🌲' => 'мәңге йәшел ағас',
    '🌳' => 'япраҡлы ағас',
    '🌴' => 'пальма',
    '🌵' => 'кактус',
    '🌾' => 'дөгө көлтәһе',
    '🌿' => 'үлән',
    '☘' => 'өсъяпраҡ',
    '🍀' => 'дүрт япраҡлы клевер',
    '🍁' => 'саған япрағы',
    '🍂' => 'ҡойолған япраҡ',
    '🍃' => 'елдә еблерләгән япраҡ',
    '🪹' => 'буш оя',
    '🪺' => 'йомортҡалы оя',
    '🍄' => 'бәшмәк',
    '🪾' => 'япраҡһыҙ ағас',
    '🍇' => 'йөҙөм',
    '🍈' => 'ҡауын',
    '🍉' => 'ҡарбуз',
    '🍊' => 'мандарин',
    '🍋' => 'лимон',
    '🍌' => 'банан',
    '🍍' => 'ананас',
    '🥭' => 'манго',
    '🍎' => 'ҡыҙыл алма',
    '🍏' => 'йәшел алма',
    '🍐' => 'армыт',
    '🍑' => 'шафталы',
    '🍒' => 'сейә',
    '🍓' => 'еләк',
    '🫐' => 'күк көртмәле',
    '🥝' => 'киви емеше',
    '🍅' => 'помидор',
    '🫒' => 'зәйтүн',
    '🥥' => 'кокос',
    '🥑' => 'авокадо',
    '🍆' => 'баклажан',
    '🥔' => 'картуф',
    '🥕' => 'кишер',
    '🌽' => 'кукуруз башағы',
    '🌶' => 'әсе борос',
    '🫑' => 'болгар боросо',
    '🥒' => 'ҡыяр',
    '🥬' => 'япраҡлы йәшелсә',
    '🥦' => 'брокколи',
    '🧄' => 'һарымһаҡ',
    '🧅' => 'һуған',
    '🥜' => 'ҡытай сәтләүеге',
    '🫘' => 'фасоль',
    '🌰' => 'каштан',
    '🫚' => 'имбир тамыры',
    '🫛' => 'борсаҡ ҡуҙағы',
    '🫜' => 'сөгөлдөр',
    '🍞' => 'икмәк',
    '🥐' => 'круассан',
    '🥖' => 'багет',
    '🫓' => 'йәйпәк икмәк',
    '🥨' => 'крендель',
    '🥯' => 'бублик',
    '🥞' => 'ҡоймаҡ',
    '🧇' => 'вафли',
    '🧀' => 'сыр киҫәге',
    '🍖' => 'һөйәкле ит',
    '🍗' => 'ҡош ите аяғы',
    '🥩' => 'ит киҫәге',
    '🥓' => 'бекон',
    '🍔' => 'гамбургер',
    '🍟' => 'фри картуфы',
    '🍕' => 'пицца',
    '🌭' => 'хот-дог',
    '🥪' => 'сэндвич',
    '🌮' => 'тако',
    '🌯' => 'буррито',
    '🫔' => 'тамале',
    '🥙' => 'көбәп',
    '🧆' => 'фалафель',
    '🥚' => 'йомортҡа',
    '🍳' => 'ашарға әҙерләү',
    '🥘' => 'ризыҡ һалынған таба',
    '🍲' => 'аш һауыты',
    '🫕' => 'фондю',
    '🥣' => 'ҡалаҡлы һауыт',
    '🥗' => 'йәшел салат',
    '🍿' => 'попкорн',
    '🧈' => 'май',
    '🧂' => 'тоҙ һауыты',
    '🥫' => 'консерва',
    '🍱' => 'бенто йәшниге',
    '🍘' => 'дөгө крекеры',
    '🍙' => 'дөгө тубы',
    '🍚' => 'бешерелгән дөгө',
    '🍛' => 'карри дөгөһө',
    '🍜' => 'парлы һауыт',
    '🍝' => 'спагетти',
    '🍠' => 'ҡыҙҙырылған татлы картуф',
    '🍢' => 'одэн',
    '🍣' => 'суши',
    '🍤' => 'ҡурылған креветка',
    '🍥' => 'балыҡ торты',
    '🥮' => 'ай бәлеше',
    '🍡' => 'данго',
    '🥟' => 'билмән',
    '🥠' => 'күрәҙәлек печеньеһы',
    '🥡' => 'алып китеү өсөн ҡумта',
    '🍦' => 'йомшаҡ туңдырма',
    '🍧' => 'ҡырылған туңдырма',
    '🍨' => 'туңдырма',
    '🍩' => 'пончик',
    '🍪' => 'печенье',
    '🎂' => 'тыуған көн торты',
    '🍰' => 'торт киҫәге',
    '🧁' => 'капкейк',
    '🥧' => 'бәлеш',
    '🍫' => 'шоколад',
    '🍬' => 'кәнфит',
    '🍭' => 'таяҡта кәнфит',
    '🍮' => 'пудинг',
    '🍯' => 'бал һауыты',
    '🍼' => 'бала шешәһе',
    '🥛' => 'стәкән һөт',
    '☕' => 'эҫе эсемлек',
    '🫖' => 'сәйгүн',
    '🍵' => 'тотҡаһыҙ сәй сынаяғы',
    '🍶' => 'саке',
    '🍾' => 'шампан шешәһе',
    '🍷' => 'шарап бокалы',
    '🍸' => 'коктейль бокалы',
    '🍹' => 'тропик эсемлек',
    '🍺' => 'һыра көрөшкәһе',
    '🍻' => 'һыра көрөшкәләрен сәкешеү',
    '🥂' => 'сәкешеүсе бокалдар',
    '🥃' => 'виски стаканы',
    '🫗' => 'шыйыҡса ҡойоу',
    '🥤' => 'һаламлы сынаяҡ',
    '🧋' => 'ҡыуыҡлы сәй',
    '🧃' => 'эсемлектәр йәшниге',
    '🧉' => 'мате',
    '🧊' => 'боҙ киҫәге',
    '🥢' => 'таяҡсалар',
    '🍽' => 'тәрилкәле сәнске һәм бысаҡ',
    '🍴' => 'сәнске һәм бысаҡ',
    '🥄' => 'ҡалаҡ',
    '🔪' => 'бысаҡ',
    '🫙' => 'банка',
    '🏺' => 'амфора',
    '🌍' => 'Европа-Африканы күрһәтеүсе глобус',
    '🌎' => 'Американы күрһәтеүсе глобус',
    '🌏' => 'Азия-Австралияны күрһәтеүсе глобус',
    '🌐' => 'меридиандар менән глобус',
    '🗺' => 'донъя картаһы',
    '🗾' => 'Япония картаһы',
    '🧭' => 'компас',
    '🏔' => 'ҡарлы тау',
    '⛰' => 'тау',
    '🛘' => 'ер ишелеүе',
    '🌋' => 'вулкан',
    '🗻' => 'Фудзияма тауы',
    '🏕' => 'кемпинг',
    '🏖' => 'зонт менән пляж',
    '🏜' => 'сүл',
    '🏝' => 'сүл утрауы',
    '🏞' => 'милли парк',
    '🏟' => 'стадион',
    '🏛' => 'классик бина',
    '🏗' => 'бина төҙөлөшө',
    '🧱' => 'кирбестәр',
    '🪨' => 'таш',
    '🪵' => 'ағас',
    '🛖' => 'аласыҡ',
    '🏘' => 'йорттар',
    '🏚' => 'ташландыҡ йорт',
    '🏠' => 'йорт',
    '🏡' => 'баҡсалы йорт',
    '🏢' => 'офис бинаһы',
    '🏣' => 'япон почтаһы',
    '🏤' => 'почта',
    '🏥' => 'дауахана',
    '🏦' => 'банк',
    '🏨' => 'ҡунаҡхана',
    '🏩' => 'мөхәббәт ҡунаҡханаһы',
    '🏪' => 'уңайлыҡ магазины',
    '🏫' => 'мәктәп',
    '🏬' => 'универмаг',
    '🏭' => 'завод',
    '🏯' => 'япон һарайы',
    '🏰' => 'һарай',
    '💒' => 'туй',
    '🗼' => 'Токио башняһы',
    '🗽' => 'Азатлыҡ статуяһы',
    '⛪' => 'сиркәү',
    '🕌' => 'мәсет',
    '🛕' => 'индус ғибәҙәтханаһы',
    '🕍' => 'синагога',
    '⛩' => 'синтоист ғибәҙәтханаһы',
    '🕋' => 'Ҡәғбә',
    '⛲' => 'фонтан',
    '⛺' => 'сатыр',
    '🌁' => 'томанлы',
    '🌃' => 'йондоҙло төн',
    '🏙' => 'ҡала күренеше',
    '🌄' => 'тауҙар өҫтөндә ҡояш сығыуы',
    '🌅' => 'ҡояш сығыуы',
    '🌆' => 'ҡала пейзажы эңерҙә',
    '🌇' => 'ҡояш байыуы',
    '🌉' => 'төнгө күпер',
    '♨' => 'эҫе сығанаҡтар',
    '🎠' => 'карусель аты',
    '🛝' => 'балалар майҙансығы шыуғысы',
    '🎡' => 'күҙәтеү тәгәрмәсе',
    '🎢' => 'шыуыу тауы',
    '💈' => 'парикмахер бағанаһы',
    '🎪' => 'цирк сатыры',
    '🚂' => 'локомотив',
    '🚃' => 'тимер юл вагоны',
    '🚄' => 'тиҙ йөрөшлө поезд',
    '🚅' => 'пуля поезд',
    '🚆' => 'поезд',
    '🚇' => 'метро',
    '🚈' => 'еңел тимер юл',
    '🚉' => 'тимер юл станцияһы',
    '🚊' => 'трамвай',
    '🚝' => 'монорельс',
    '🚞' => 'тау тимер юлы',
    '🚋' => 'трамвай вагоны',
    '🚌' => 'автобус',
    '🚍' => 'ҡаршы килеүсе автобус',
    '🚎' => 'троллейбус',
    '🚐' => 'микроавтобус',
    '🚑' => 'ашығыс ярҙам машинаһы',
    '🚒' => 'янғын һүндереү машинаһы',
    '🚓' => 'полиция машинаһы',
    '🚔' => 'ҡаршы килеүсе полиция машинаһы',
    '🚕' => 'такси',
    '🚖' => 'ҡаршы килеүсе такси',
    '🚗' => 'автомобиль',
    '🚘' => 'ҡаршы килеүсе автомобиль',
    '🚙' => 'джип',
    '🛻' => 'пикап',
    '🚚' => 'йөк машинаһы',
    '🚛' => 'фура',
    '🚜' => 'трактор',
    '🏎' => 'ярыш автомобиле',
    '🏍' => 'мотоцикл',
    '🦽' => 'инвалид коляскаһы',
    '🦼' => 'моторлы инвалид коляскаһы',
    '🛺' => 'авторикша',
    '🚲' => 'велосипед',
    '🛴' => 'скутер',
    '🛹' => 'скейтборд',
    '🛼' => 'роликлы коньки',
    '🚏' => 'автобус туҡталышы',
    '🛣' => 'автомагистраль',
    '🛤' => 'тимер юл',
    '🛢' => 'нефть мискәһе',
    '⛽' => 'яғыулыҡ насосы',
    '🛞' => 'тәгәрмәс',
    '🚨' => 'полиция яҡтырғысы',
    '🚥' => 'горизонталь светофор',
    '🚦' => 'вертикаль светофор',
    '🛑' => 'туҡтау билдәһе',
    '🚧' => 'төҙөлөш',
    '⚓' => 'якорь',
    '🛟' => 'ҡотҡарыу түңәрәге',
    '⛵' => 'елкәнле кәмә',
    '🛶' => 'каноэ',
    '🚤' => 'тиҙ йөрөшлө кәмә',
    '🛳' => 'пассажир карабы',
    '⛴' => 'паром',
    '🛥' => 'моторлы кәмә',
    '🚢' => 'карап',
    '✈' => 'самолет',
    '🛩' => 'бәләкәй самолет',
    '🛫' => 'самолет осоуы',
    '🛬' => 'самолет килеүе',
    '🪂' => 'парашют',
    '💺' => 'урындыҡ',
    '🚁' => 'вертолет',
    '🚟' => 'аҫылмалы тимер юлы',
    '🚠' => 'тау кабелле юлы',
    '🚡' => 'һауа трамвай юлы',
    '🛰' => 'юлдаш',
    '🚀' => 'ракета',
    '🛸' => 'осоусы тәрилкә',
    '🛎' => 'ҡунаҡхана ҡыңғырауы',
    '🧳' => 'багаж',
    '⌛' => 'ҡом сәғәте тамам',
    '⏳' => 'ҡом сәғәте тамамланмаған',
    '⌚' => 'ҡул сәғәте',
    '⏰' => 'уятҡыс',
    '⏱' => 'секундомер',
    '⏲' => 'таймер сәғәте',
    '🕰' => 'камин сәғәте',
    '🕛' => 'сәғәт ун ике',
    '🕧' => 'беренсе ярты',
    '🕐' => 'сәғәт бер',
    '🕜' => 'икенсе ярты',
    '🕑' => 'сәғәт ике',
    '🕝' => 'өсөнсө ярты',
    '🕒' => 'сәғәт өс',
    '🕞' => 'дүртенсе ярты',
    '🕓' => 'сәғәт дүрт',
    '🕟' => 'бишенсе ярты',
    '🕔' => 'сәғәт биш',
    '🕠' => 'алтынсы ярты',
    '🕕' => 'сәғәт алты',
    '🕡' => 'етенсе ярты',
    '🕖' => 'сәғәт ете',
    '🕢' => 'һигеҙенсе ярты',
    '🕗' => 'сәғәт һигеҙ',
    '🕣' => 'туғыҙынсы ярты',
    '🕘' => 'сәғәт туғыҙ',
    '🕤' => 'унынсы ярты',
    '🕙' => 'сәғәт ун',
    '🕥' => 'ун беренсе ярты',
    '🕚' => 'сәғәт ун бер',
    '🕦' => 'ун икенсе ярты',
    '🌑' => 'яңы ай',
    '🌒' => 'үҫеүсе ярым ай',
    '🌓' => 'беренсе сирек ай',
    '🌔' => 'үҫеүсе тулы ай',
    '🌕' => 'тулы ай',
    '🌖' => 'кәмеүсе тулы ай',
    '🌗' => 'һуңғы сирек ай',
    '🌘' => 'кәмеүсе ярым ай',
    '🌙' => 'ярым ай',
    '🌚' => 'йөҙлө яңы ай',
    '🌛' => 'йөҙлө беренсе сирек ай',
    '🌜' => 'йөҙлө һуңғы сирек ай',
    '🌡' => 'термометр',
    '☀' => 'ҡояш',
    '🌝' => 'йөҙлө тулы ай',
    '🌞' => 'йөҙлө ҡояш',
    '🪐' => 'ҡулсалы планета',
    '⭐' => 'йондоҙ',
    '🌟' => 'яҡты йондоҙ',
    '🌠' => 'атыусы йондоҙ',
    '🌌' => 'Ҡош юлы',
    '☁' => 'болот',
    '⛅' => 'болот артындағы ҡояш',
    '⛈' => 'йәшенле һәм ямғырлы болот',
    '🌤' => 'бәләкәй болот артында ҡояш',
    '🌥' => 'ҙур болот артында ҡояш',
    '🌦' => 'ямғырлы болот артында ҡояш',
    '🌧' => 'ямғырлы болот',
    '🌨' => 'ҡарлы болот',
    '🌩' => 'йәшенле болот',
    '🌪' => 'торнадо',
    '🌬' => 'ел йөҙө',
    '🌀' => 'циклон',
    '🌈' => 'йәйғор',
    '🌂' => 'ябыҡ зонт',
    '☂' => 'зонт',
    '☔' => 'ямғыр тамсылары менән зонт',
    '⛱' => 'ерҙәге зонт',
    '⚡' => 'юғары көсөргәнеш',
    '❄' => 'ҡар бөртөгө',
    '☃' => 'ҡар бабай',
    '⛄' => 'ҡарһыҙ ҡар бабай',
    '☄' => 'комета',
    '🔥' => 'ут',
    '💧' => 'тамсы',
    '🌊' => 'һыу тулҡыны',
    '🎃' => 'фонарь-ҡабаҡ',
    '🎄' => 'раштыуа ағасы',
    '🎆' => 'фейерверк',
    '🎇' => 'бенгаль уты',
    '🧨' => 'динамит',
    '✨' => 'осҡондар',
    '🎈' => 'һауа шары',
    '🎊' => 'конфетти тубы',
    '🎋' => 'танабата ағасы',
    '🎍' => 'ҡарағай биҙәүесе',
    '🎎' => 'япон ҡурсаҡтары',
    '🎏' => 'карп флагы',
    '🎐' => 'ел ҡыңғырауы',
    '🎑' => 'ай ҡарау тантанаһы',
    '🧧' => 'ҡыҙыл конверт',
    '🎀' => 'таҫма',
    '🎁' => 'төрөлгән бүләк',
    '🎗' => 'иҫкә төшөрөү таҫмаһы',
    '🎟' => 'инеү билеттары',
    '🎫' => 'билет',
    '🎖' => 'хәрби миҙал',
    '🏆' => 'кубок',
    '🏅' => 'спорт миҙалы',
    '🥇' => '1-се урын миҙалы',
    '🥈' => '2-се урын миҙалы',
    '🥉' => '3-сө урын миҙалы',
    '⚽' => 'футбол тубы',
    '⚾' => 'бейсбол',
    '🥎' => 'софтбол',
    '🏀' => 'баскетбол',
    '🏐' => 'волейбол',
    '🏈' => 'америка футболы',
    '🏉' => 'регби тубы',
    '🎾' => 'теннис',
    '🥏' => 'осоусы диск',
    '🎳' => 'боулинг',
    '🏏' => 'крикет уйыны',
    '🏑' => 'үлән хоккейы',
    '🏒' => 'хоккей',
    '🥍' => 'лакросс',
    '🏓' => 'пинг-понг',
    '🏸' => 'бадминтон',
    '🥊' => 'бокс бирсәткәһе',
    '🥋' => 'хәрби сәнғәт кейеме',
    '🥅' => 'ҡапҡа селтәре',
    '⛳' => 'соҡорҙағы флаг',
    '⛸' => 'коньки',
    '🎣' => 'балыҡ менән ҡармаҡ',
    '🤿' => 'һыу аҫтында йөҙөү өсөн битлек',
    '🎽' => 'йүгереү футболкаһы',
    '🎿' => 'саңғы',
    '🛷' => 'сана',
    '🥌' => 'керлинг ташы',
    '🎯' => 'дартс',
    '🪀' => 'йо-йо',
    '🪁' => 'ҡағыҙ осорғос',
    '🔫' => 'һыу пистолеты',
    '🎱' => 'бильярд шары',
    '🔮' => 'гәлсәр шар',
    '🪄' => 'тылсымлы таяҡ',
    '🎮' => 'видеоуйын',
    '🕹' => 'джойстик',
    '🎰' => 'уйын автоматы',
    '🎲' => 'уйын һөйәге',
    '🧩' => 'башватҡыс киҫәге',
    '🧸' => 'йомшаҡ айыу',
    '🪅' => 'пиньята',
    '🪩' => 'көҙгө шар',
    '🪆' => 'матрешка',
    '♠' => 'ҡарға тамғаһы',
    '♥' => 'йөрәк тамғаһы',
    '♦' => 'бубий тамғаһы',
    '♣' => 'тәре тамғаһы',
    '♟' => 'шахмат пешкаһы',
    '🃏' => 'джокер',
    '🀄' => 'маджонг ҡыҙыл аждаһа',
    '🎴' => 'сәскәле уйын кәрттәре',
    '🎭' => 'театр маскалары',
    '🖼' => 'рамлы һүрәт',
    '🎨' => 'рәссам палитраһы',
    '🧵' => 'еп',
    '🪡' => 'тегеү энәһе',
    '🧶' => 'йөн йомғағы',
    '🪢' => 'төйөн',
    '👓' => 'күҙлек',
    '🕶' => 'ҡояш күҙлеге',
    '🥽' => 'һаҡлағыс күҙлек',
    '🥼' => 'лаборатория халаты',
    '🦺' => 'ҡотҡарыу жилеты',
    '👔' => 'галстук',
    '👕' => 'футболка',
    '👖' => 'джинсы',
    '🧣' => 'шарф',
    '🧤' => 'бирсәткә',
    '🧥' => 'пальто',
    '🧦' => 'ойоҡ',
    '👗' => 'күлдәк',
    '👘' => 'кимоно',
    '🥻' => 'сари',
    '🩱' => 'бер өлөшлө һыу кейеме',
    '🩲' => 'трусик',
    '🩳' => 'шорты',
    '👙' => 'бикини',
    '👚' => 'ҡатын-ҡыҙ кейеме',
    '🪭' => 'ҡул елпеүесе',
    '👛' => 'янсыҡ',
    '👜' => 'сумка',
    '👝' => 'клатч',
    '🛍' => 'һатып алыу сумкалары',
    '🎒' => 'рюкзак',
    '🩴' => 'вьетнамка',
    '👞' => 'ир-ат аяҡ кейеме',
    '👟' => 'кроссовка',
    '🥾' => 'поход ботинкаһы',
    '🥿' => 'яҫы аяҡ кейеме',
    '👠' => 'бейек үксәле аяҡ кейеме',
    '👡' => 'ҡатын-ҡыҙ сандалы',
    '🩰' => 'балет аяҡ кейеме',
    '👢' => 'ҡатын-ҡыҙ итеге',
    '🪮' => 'афро-тараҡ',
    '👑' => 'таж',
    '👒' => 'ҡатын-ҡыҙ эшләпәһе',
    '🎩' => 'цилиндр',
    '🎓' => 'сығарылыш башлығы',
    '🧢' => 'бейсболка',
    '🪖' => 'хәрби шлем',
    '⛑' => 'ҡотҡарыусының шлемы',
    '📿' => 'доға йөҙөгө',
    '💄' => 'ирен буяуы',
    '💍' => 'балдаҡ',
    '💎' => 'аҫыл таш',
    '🔇' => 'тауышһыҙ динамик',
    '🔈' => 'динамиктың тауышы түбән',
    '🔉' => 'динамиктың уртаса тауышы',
    '🔊' => 'динамиктың юғары тауышы',
    '📢' => 'тауыш көсәйткес',
    '📣' => 'мегафон',
    '📯' => 'почта мөгөҙө',
    '🔔' => 'ҡыңғырау',
    '🔕' => 'һыҙыҡлы ҡыңғырау',
    '🎼' => 'музыкаль партитура',
    '🎵' => 'музыкаль нота',
    '🎶' => 'музыкаль ноталар',
    '🎙' => 'студия микрофоны',
    '🎚' => 'кимәл слайдеры',
    '🎛' => 'идара итеү тотҡалары',
    '🎤' => 'микрофон',
    '🎧' => 'ҡолаҡса',
    '📻' => 'радио',
    '🎷' => 'саксофон',
    '🎺' => 'торба',
    '🪊' => 'тромбон',
    '🪗' => 'аккордеон',
    '🎸' => 'гитара',
    '🎹' => 'музыкаль баҫмаҡтар',
    '🎻' => 'скрипка',
    '🪕' => 'банджо',
    '🥁' => 'барабан',
    '🪘' => 'оҙон барабан',
    '🪇' => 'маракас',
    '🪈' => 'флейта',
    '🪉' => 'арфа',
    '📱' => 'кәрәҙле телефон',
    '📲' => 'уҡ менән кәрәҙле телефон',
    '☎' => 'телефон',
    '📞' => 'телефон трубкаһы',
    '📟' => 'пейджер',
    '📠' => 'факс аппараты',
    '🔋' => 'батарейка',
    '🪫' => 'түбән заряд',
    '🔌' => 'электр вилкаһы',
    '💻' => 'ноутбук',
    '🖥' => 'өҫтәл компьютеры',
    '🖨' => 'принтер',
    '⌨' => 'баҫмаҡтар',
    '🖱' => 'компьютер сысҡаны',
    '🖲' => 'трекбол',
    '💽' => 'компьютер дискы',
    '💾' => 'дискета',
    '💿' => 'оптик диск',
    '📀' => 'DVD',
    '🧮' => 'иҫәпләү таҡтаһы',
    '🎥' => 'кинокамера',
    '🎞' => 'кино кадрҙары',
    '📽' => 'кинопроектор',
    '🎬' => 'кино таҡтаһы',
    '📺' => 'телевизор',
    '📷' => 'фотоаппарат',
    '📸' => 'балҡышлы фотоаппарат',
    '📹' => 'видеокамера',
    '📼' => 'видеокассета',
    '🔍' => 'һулға эйелгән лупа',
    '🔎' => 'уңға эйелгән лупа',
    '🕯' => 'шәм',
    '💡' => 'лампочка',
    '🔦' => 'фонарик',
    '🏮' => 'ҡыҙыл ҡағыҙ фонарь',
    '🪔' => 'дия лампаһы',
    '📔' => 'биҙәкле тышлы дәфтәр',
    '📕' => 'ябыҡ китап',
    '📖' => 'асыҡ китап',
    '📗' => 'йәшел китап',
    '📘' => 'зәңгәр китап',
    '📙' => 'ҡыҙғылт һары китап',
    '📚' => 'китаптар',
    '📓' => 'дәфтәр',
    '📒' => 'иҫәп китабы',
    '📃' => 'бөгөлгән бит',
    '📜' => 'төргәк',
    '📄' => 'өҫкә ҡараған бит',
    '📰' => 'гәзит',
    '🗞' => 'төрөлгән гәзит',
    '📑' => 'билдәләү ҡулланмалары',
    '🔖' => 'билдәләү',
    '🏷' => 'бирка',
    '🪙' => 'тәңкә',
    '💰' => 'аҡса тоҡсаһы',
    '🪎' => 'хазина һандығы',
    '💴' => 'иена банкнотаһы',
    '💵' => 'доллар банкнотаһы',
    '💶' => 'евро банкнотаһы',
    '💷' => 'фунт банкнотаһы',
    '💸' => 'ҡанатлы аҡса',
    '💳' => 'кредит картаһы',
    '🧾' => 'чек',
    '💹' => 'иена менән үҫеүсе график',
    '✉' => 'конверт',
    '📧' => 'электрон почта',
    '📨' => 'килгән конверт',
    '📩' => 'уҡлы конверт',
    '📤' => 'китеүсе хат тартмаһы',
    '📥' => 'килеүсе хат тартмаһы',
    '📦' => 'посылка',
    '📫' => 'флагы күтәрелгән ябыҡ почта йәшниге',
    '📪' => 'флагы төшөрөлгән ябыҡ почта йәшниге',
    '📬' => 'флагы күтәрелгән асыҡ почта йәшниге',
    '📭' => 'флагы төшөрөлгән асыҡ почта йәшниге',
    '📮' => 'почта йәшниге',
    '🗳' => 'һайлау йәшниге',
    '✏' => 'ҡәләм',
    '✒' => 'перо',
    '🖋' => 'перолы ручка',
    '🖊' => 'ручка',
    '🖌' => 'бумала',
    '📝' => 'яҙма',
    '💼' => 'портфель',
    '📁' => 'папка',
    '📂' => 'асыҡ папка',
    '🗂' => 'картотека бүлгестәре',
    '📅' => 'календарь',
    '📆' => 'йыртыу календары',
    '🗒' => 'спираль блокнот',
    '🗓' => 'спираль календарь',
    '📇' => 'картотека',
    '📈' => 'үҫеүсе график',
    '📉' => 'кәмеүсе график',
    '📊' => 'бағаналы диаграмма',
    '📋' => 'ҡыҫҡыслы планшет',
    '📌' => 'төймә',
    '📍' => 'түңәрәк төймә',
    '📎' => 'ҡыҫтырғыс',
    '🖇' => 'тоташтырылған ҡағыҙ ҡыҫтырғыстар',
    '📏' => 'тура линейка',
    '📐' => 'өсмөйөш һыҙғыс',
    '✂' => 'ҡайсы',
    '🗃' => 'картотека тартмаһы',
    '🗄' => 'файл шкафы',
    '🗑' => 'сүп-сар кәрзине',
    '🔒' => 'бикле йоҙаҡ',
    '🔓' => 'асыҡ йоҙаҡ',
    '🔏' => 'ручка менән бикле йоҙаҡ',
    '🔐' => 'асҡыс менән бикле йоҙаҡ',
    '🔑' => 'асҡыс',
    '🗝' => 'иҫке асҡыс',
    '🔨' => 'сүкеш',
    '🪓' => 'балта',
    '⛏' => 'кирка',
    '⚒' => 'сүкеш һәм кирка',
    '🛠' => 'сүкеш һәм гайка асҡысы',
    '🗡' => 'хәнйәр',
    '⚔' => 'киҫешкән ҡылыстар',
    '💣' => 'бомба',
    '🪃' => 'бумеранг',
    '🏹' => 'йәйә һәм уҡ',
    '🛡' => 'ҡалҡан',
    '🪚' => 'ағас бысҡыһы',
    '🔧' => 'гайка асҡысы',
    '🪛' => 'отвертка',
    '🔩' => 'гайка һәм болт',
    '⚙' => 'шестерня',
    '🗜' => 'ҡыҫҡыс',
    '⚖' => 'үлсәү',
    '🦯' => 'аҡ таяҡ',
    '🔗' => 'һылтанма',
    '⛓' => 'сылбырҙар',
    '🪝' => 'ҡармаҡ',
    '🧰' => 'ҡоралдар йәшниге',
    '🧲' => 'магнит',
    '🪜' => 'баҫҡыс',
    '🪏' => 'көрәк',
    '⚗' => 'алембик',
    '🧪' => 'пробирка',
    '🧫' => 'Петри сынаяғы',
    '🧬' => 'ДНК',
    '🔬' => 'микроскоп',
    '🔭' => 'телескоп',
    '📡' => 'спутник антеннаһы',
    '💉' => 'шприц',
    '🩸' => 'ҡан тамсыһы',
    '💊' => 'дарыу',
    '🩹' => 'йәбештереү бинты',
    '🩼' => 'ҡултыҡ таяғы',
    '🩺' => 'стетоскоп',
    '🩻' => 'рентген',
    '🚪' => 'ишек',
    '🛗' => 'лифт',
    '🪞' => 'көҙгө',
    '🪟' => 'тәҙрә',
    '🛏' => 'карауат',
    '🛋' => 'диван һәм лампа',
    '🪑' => 'ултырғыс',
    '🚽' => 'унитаз',
    '🪠' => 'вантуз',
    '🚿' => 'душ',
    '🛁' => 'ванна',
    '🪤' => 'сысҡан ҡапҡаны',
    '🪒' => 'ҡырынғыс',
    '🧴' => 'лосьон шешәһе',
    '🧷' => 'инглиз төйрәүесе',
    '🧹' => 'һепертке',
    '🧺' => 'кәрзин',
    '🧻' => 'ҡағыҙ рулоны',
    '🪣' => 'биҙрә',
    '🧼' => 'һабын',
    '🫧' => 'ҡыуыҡтар',
    '🪥' => 'теш щеткаһы',
    '🧽' => 'губка',
    '🧯' => 'янғын һүндергес',
    '🛒' => 'һатып алыу арбаһы',
    '🚬' => 'сигәрит',
    '⚰' => 'табут',
    '🪦' => 'ҡәбер ташы',
    '⚱' => 'ерләү урнаһы',
    '🧿' => 'назар амулеты',
    '🪬' => 'хамса',
    '🗿' => 'моаи',
    '🪧' => 'плакат',
    '🪪' => 'шәхес таныҡлығы',
    '🏧' => 'банкомат билдәһе',
    '🚮' => 'сүп-сар һауыты билдәһе',
    '🚰' => 'эсәр һыу',
    '♿' => 'инвалид коляскаһы символы',
    '🚹' => 'ирҙәр бүлмәһе',
    '🚺' => 'ҡатын-ҡыҙҙар бүлмәһе',
    '🚻' => 'бәҙрәф',
    '🚼' => 'бала билдәһе',
    '🛂' => 'паспорт контроле',
    '🛃' => 'таможня',
    '🛄' => 'багаж алыу',
    '🛅' => 'ҡалдырылған багаж',
    '⚠' => 'иҫкәртеү',
    '🚸' => 'балалар үткәүеле',
    '⛔' => 'инеү тыйылған',
    '🚫' => 'тыйылған',
    '🚳' => 'велосипедтар тыйылған',
    '🚭' => 'тәмәке тартыу тыйылған',
    '🚯' => 'сүпләмәгеҙ',
    '🚱' => 'эсәр булмаған һыу',
    '🚷' => 'йәйәүлеләр тыйылған',
    '📵' => 'мобиль телефондар тыйылған',
    '🔞' => 'ун һигеҙ йәшкә тиклемгеләргә ярамай',
    '☢' => 'радиоактив',
    '☣' => 'биохәүеф',
    '⬆' => 'өҫкә уҡ',
    '↗' => 'өҫкә-уңға уҡ',
    '➡' => 'уңға уҡ',
    '↘' => 'аҫҡа-уң уҡ',
    '⬇' => 'аҫҡа уҡ',
    '↙' => 'аҫҡа-һул уҡ',
    '⬅' => 'һулға уҡ',
    '↖' => 'өҫкә-һул уҡ',
    '↕' => 'өҫкә-аҫҡа уҡ',
    '↔' => 'һул-уң уҡ',
    '↩' => 'уң уҡ һулға бөгөлгән',
    '↪' => 'һул уҡ уңға бөгөлгән',
    '⤴' => 'өҫкә бөгөлгән уңға уҡ',
    '⤵' => 'аҫҡа бөгөлгән уңға уҡ',
    '🔃' => 'сәғәт йүнәлешендә вертикаль уҡтар',
    '🔄' => 'сәғәт йүнәлешенә ҡаршы уҡтар төймәһе',
    '🔙' => 'АРТҠА уғы',
    '🔚' => 'АХЫР уғы',
    '🔛' => 'ҠАБЫНҒАН! уғы',
    '🔜' => 'ОҘАҠЛАМАЙ уғы',
    '🔝' => 'ӨҪ уғы',
    '🛐' => 'ғибәҙәт урыны',
    '⚛' => 'атом символы',
    '🕉' => 'ом',
    '✡' => 'Дауыт йондоҙо',
    '☸' => 'дхарма тәгәрмәсе',
    '☯' => 'инь-ян',
    '✝' => 'латин тәреһе',
    '☦' => 'православие тәреһе',
    '☪' => 'йондоҙ һәм ярым ай',
    '☮' => 'тыныслыҡ символы',
    '🕎' => 'менора',
    '🔯' => 'нөктәле алты мөйөшлө йондоҙ',
    '🪯' => 'кханда',
    '♈' => 'Ҡуҙы',
    '♉' => 'Буға',
    '♊' => 'Игеҙәктәр',
    '♋' => 'Ҡыҫала',
    '♌' => 'Арыҫлан',
    '♍' => 'Ҡыҙ',
    '♎' => 'Үлсәү',
    '♏' => 'Саян',
    '♐' => 'Уҡсы',
    '♑' => 'Ылаҡ',
    '♒' => 'Һыуғояр',
    '♓' => 'Балыҡ',
    '⛎' => 'Йылан тотоусы',
    '🔀' => 'тректарҙы бутау төймәһе',
    '🔁' => 'ҡабатлау төймәһе',
    '🔂' => 'бер тапҡыр ҡабатлау төймәһе',
    '▶' => 'уйнау төймәһе',
    '⏩' => 'алға тиҙләтеү төймәһе',
    '⏭' => 'киләһе трек төймәһе',
    '⏯' => 'уйнау йәки пауза төймәһе',
    '◀' => 'кирегә төймәһе',
    '⏪' => 'кире тиҙләтеү төймәһе',
    '⏮' => 'үткән трек төймәһе',
    '🔼' => 'өҫкә төймәһе',
    '⏫' => 'өҫкә тиҙләтеү төймәһе',
    '🔽' => 'аҫҡа төймә',
    '⏬' => 'аҫҡа тиҙләтеү төймәһе',
    '⏸' => 'пауза төймәһе',
    '⏹' => 'туҡтатыу төймәһе',
    '⏺' => 'яҙыу төймәһе',
    '⏏' => 'сығарыу төймәһе',
    '🎦' => 'кино',
    '🔅' => 'тоноҡ төймәһе',
    '🔆' => 'яҡты төймәһе',
    '📶' => 'антенна һыҙаттары',
    '🛜' => 'сымһыҙ',
    '📳' => 'вибрация режимы',
    '📴' => 'мобиль телефон һүндерелгән',
    '♀' => 'ҡатын-ҡыҙ билдәһе',
    '♂' => 'ир-ат билдәһе',
    '⚧' => 'трансгендер символы',
    '✖' => 'ҡабатлау',
    '➕' => 'плюс',
    '➖' => 'минус',
    '➗' => 'бүлеү',
    '🟰' => 'ҡуйы тигеҙлек билдәһе',
    '♾' => 'сикһеҙлек',
    '‼' => 'ике өндәү билдәһе',
    '⁉' => 'өндәү һорау билдәһе',
    '❓' => 'ҡыҙыл һорау билдәһе',
    '❔' => 'аҡ һорау билдәһе',
    '❕' => 'аҡ өндәү билдәһе',
    '❗' => 'ҡыҙыл өндәү билдәһе',
    '〰' => 'тулҡынлы һыҙыҡ',
    '💱' => 'валюта алмашыу',
    '💲' => 'ҡалын доллар билдәһе',
    '⚕' => 'медицина символы',
    '♻' => 'эшкәртеү символы',
    '⚜' => 'геральдик ләлә',
    '🔱' => 'өс тешле герб',
    '📛' => 'бейдж',
    '🔰' => 'япон яңы башлаусы символы',
    '⭕' => 'эсе буш ҡыҙыл түңәрәк',
    '✅' => 'билдә ҡуйыу төймәһе',
    '☑' => 'квадратҡа ҡуйылған билдә',
    '✔' => 'билдә',
    '❌' => 'тәре билдәһе',
    '❎' => 'тәре билдәһе төймәһе',
    '➰' => 'элмәк',
    '➿' => 'икеләтә элмәк',
    '〽' => 'өлөш алмашыу билдәһе',
    '✳' => 'һигеҙ нурлы йондоҙ',
    '✴' => 'һигеҙ осло йондоҙ',
    '❇' => 'осҡон',
    '™' => 'тауар билдәһе',
    '🫟' => 'тап',
    '🔠' => 'латин баш хәрефтәрен индереү',
    '🔡' => 'латин бәләкәй хәрефтәрен индереү',
    '🔢' => 'һандарҙы индереү',
    '🔣' => 'символдарҙы индереү',
    '🔤' => 'латин хәрефтәрен индереү',
    '🅰' => 'A ҡан төркөмө',
    '🆎' => 'AB ҡан төркөмө',
    '🅱' => 'B ҡан төркөмө',
    '🆑' => '"таҙартыу" төймәһе',
    '🆒' => '"шәп" төймәһе',
    '🆓' => '"бушлай" төймәһе',
    'ℹ' => 'мәғлүмәт билдәһе',
    '🆔' => '"асыҡлау" төймәһе',
    'Ⓜ' => 'түңәрәкләнгән M',
    '🆕' => '"яңы" төймәһе',
    '🆖' => '"ярамай" төймәһе',
    '🅾' => '0 ҡаң төркөмө',
    '🆗' => 'OK төймәһе',
    '🅿' => '"парковка" төймәһе',
    '🆘' => 'SOS төймәһе',
    '🆙' => 'UP! төймәһе',
    '🆚' => '"ҡаршы" төймәһе',
    '🈁' => 'японса «бында» төймәһе',
    '🈂' => 'японса «хеҙмәт хаҡы» төймәһе',
    '🈷' => 'японса «айлыҡ сумма» төймәһе',
    '🈶' => 'японса «бушлай түгел» төймәһе',
    '🈯' => 'японса «броньланған» төймәһе',
    '🉐' => 'японса «килешеү» төймәһе',
    '🈹' => 'японса «ташлама» төймәһе',
    '🈚' => 'японса «бушлай» төймәһе',
    '🈲' => 'японса «тыйылған» төймәһе',
    '🉑' => 'японса «ҡабул ителә» төймәһе',
    '🈸' => 'японса «ғариза» төймәһе',
    '🈴' => 'японса «үтеү баллы» төймәһе',
    '🈳' => 'японса «буш урын» төймәһе',
    '㊗' => 'японса «ҡотлауҙар» төймәһе',
    '㊙' => 'японса «сер» төймәһе',
    '🈺' => 'японса «эш өсөн асыҡ» төймәһе',
    '🈵' => 'японса «буш урын юҡ» төймәһе',
    '🔴' => 'ҡыҙыл түңәрәк',
    '🟠' => 'ҡыҙғылт һары түңәрәк',
    '🟡' => 'һары түңәрәк',
    '🟢' => 'йәшел түңәрәк',
    '🔵' => 'күк түңәрәк',
    '🟣' => 'шәмәхә түңәрәк',
    '🟤' => 'көрән түңәрәк',
    '⚫' => 'ҡара түңәрәк',
    '⚪' => 'аҡ түңәрәк',
    '🟥' => 'ҡыҙыл квадрат',
    '🟧' => 'ҡыҙғылт һары квадрат',
    '🟨' => 'һары квадрат',
    '🟩' => 'йәшел квадрат',
    '🟦' => 'күк квадрат',
    '🟪' => 'шәмәхә квадрат',
    '🟫' => 'көрән квадрат',
    '⬛' => 'ҙур ҡара квадрат',
    '⬜' => 'ҙур аҡ квадрат',
    '◼' => 'ҡара уртаса квадрат',
    '◻' => 'аҡ уртаса квадрат',
    '◾' => 'ҡара уртаса-бәләкәй квадрат',
    '◽' => 'аҡ уртаса-бәләкәй квадрат',
    '▪' => 'ҡара бәләкәй квадрат',
    '▫' => 'аҡ бәләкәй квадрат',
    '🔶' => 'ҙур ҡыҙғылт һары ромб',
    '🔷' => 'ҙур күк ромб',
    '🔸' => 'бәләкәй ҡыҙғылт һары ромб',
    '🔹' => 'бәләкәй күк ромб',
    '🔺' => 'өҫкә йүнәлтелгән ҡыҙыл өсмөйөш',
    '🔻' => 'аҫҡа йүнәлтелгән ҡыҙыл өсмөйөш',
    '💠' => 'нөктәле ромб',
    '🔳' => 'аҡ квадрат төймә',
    '🔲' => 'ҡара квадрат төймә',
    '🏁' => 'шашкалы флаг',
    '🚩' => 'өсмөйөшлө флаг',
    '🎌' => 'киҫешеүсе флагтар',
    '🏴' => 'ҡара флаг',
    '🏳' => 'аҡ флаг',
];
