<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'pòg: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'pòg: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'pòg: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'pòg: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'pòg: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'pòg: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'pòg: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'pòg: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'pòg: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'pòg: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'pòg: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'pòg: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'pòg: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'pòg: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'pòg: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'pòg: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'pòg: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'pòg: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'pòg: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'pòg: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍❤‍💋‍👨🏻' => 'pòg: duine, duine ’s tòna soilleir a’ chraicinn',
    '👨🏻‍❤‍💋‍👨🏼' => 'pòg: duine, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏻‍❤‍💋‍👨🏽' => 'pòg: duine, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏻‍❤‍💋‍👨🏾' => 'pòg: duine, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍❤‍💋‍👨🏿' => 'pòg: duine, duine, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏼‍❤‍💋‍👨🏻' => 'pòg: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏼‍❤‍💋‍👨🏼' => 'pòg: duine, duine ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏼‍❤‍💋‍👨🏽' => 'pòg: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏼‍❤‍💋‍👨🏾' => 'pòg: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏼‍❤‍💋‍👨🏿' => 'pòg: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏽‍❤‍💋‍👨🏻' => 'pòg: duine, duine, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏽‍❤‍💋‍👨🏼' => 'pòg: duine, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍❤‍💋‍👨🏽' => 'pòg: duine, duine ’s tòna meadhanach a’ chraicinn',
    '👨🏽‍❤‍💋‍👨🏾' => 'pòg: duine, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏽‍❤‍💋‍👨🏿' => 'pòg: duine, duine, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏾‍❤‍💋‍👨🏻' => 'pòg: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏾‍❤‍💋‍👨🏼' => 'pòg: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏾‍❤‍💋‍👨🏽' => 'pòg: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏾‍❤‍💋‍👨🏾' => 'pòg: duine, duine ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏾‍❤‍💋‍👨🏿' => 'pòg: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏿‍❤‍💋‍👨🏻' => 'pòg: duine, duine, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏿‍❤‍💋‍👨🏼' => 'pòg: duine, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏿‍❤‍💋‍👨🏽' => 'pòg: duine, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏿‍❤‍💋‍👨🏾' => 'pòg: duine, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍❤‍💋‍👨🏿' => 'pòg: duine, duine ’s tòna doilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👨🏻' => 'pòg: boireannach, duine ’s tòna soilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👨🏼' => 'pòg: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👨🏽' => 'pòg: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍❤‍💋‍👨🏾' => 'pòg: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👨🏿' => 'pòg: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👨🏻' => 'pòg: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👨🏼' => 'pòg: boireannach, duine ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👨🏽' => 'pòg: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍❤‍💋‍👨🏾' => 'pòg: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👨🏿' => 'pòg: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👨🏻' => 'pòg: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👨🏼' => 'pòg: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👨🏽' => 'pòg: boireannach, duine ’s tòna meadhanach a’ chraicinn',
    '👩🏽‍❤‍💋‍👨🏾' => 'pòg: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👨🏿' => 'pòg: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👨🏻' => 'pòg: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👨🏼' => 'pòg: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👨🏽' => 'pòg: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍❤‍💋‍👨🏾' => 'pòg: boireannach, duine ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👨🏿' => 'pòg: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👨🏻' => 'pòg: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👨🏼' => 'pòg: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👨🏽' => 'pòg: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍❤‍💋‍👨🏾' => 'pòg: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👨🏿' => 'pòg: boireannach, duine ’s tòna doilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👩🏻' => 'pòg: boireannach, boireannach ’s tòna soilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👩🏼' => 'pòg: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👩🏽' => 'pòg: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍❤‍💋‍👩🏾' => 'pòg: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍❤‍💋‍👩🏿' => 'pòg: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👩🏻' => 'pòg: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👩🏼' => 'pòg: boireannach, boireannach ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👩🏽' => 'pòg: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍❤‍💋‍👩🏾' => 'pòg: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍❤‍💋‍👩🏿' => 'pòg: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👩🏻' => 'pòg: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👩🏼' => 'pòg: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👩🏽' => 'pòg: boireannach, boireannach ’s tòna meadhanach a’ chraicinn',
    '👩🏽‍❤‍💋‍👩🏾' => 'pòg: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍❤‍💋‍👩🏿' => 'pòg: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👩🏻' => 'pòg: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👩🏼' => 'pòg: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👩🏽' => 'pòg: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍❤‍💋‍👩🏾' => 'pòg: boireannach, boireannach ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏾‍❤‍💋‍👩🏿' => 'pòg: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👩🏻' => 'pòg: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👩🏼' => 'pòg: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👩🏽' => 'pòg: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍❤‍💋‍👩🏾' => 'pòg: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍❤‍💋‍👩🏿' => 'pòg: boireannach, boireannach ’s tòna doilleir a’ chraicinn',
    '🧑🏻‍🫯‍🧑🏼' => 'daoine ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏻‍🫯‍🧑🏽' => 'daoine ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏻‍🫯‍🧑🏾' => 'daoine ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍🫯‍🧑🏿' => 'daoine ri gleacadh: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏼‍🫯‍🧑🏻' => 'daoine ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏼‍🫯‍🧑🏽' => 'daoine ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏼‍🫯‍🧑🏾' => 'daoine ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏼‍🫯‍🧑🏿' => 'daoine ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏽‍🫯‍🧑🏻' => 'daoine ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏽‍🫯‍🧑🏼' => 'daoine ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🫯‍🧑🏾' => 'daoine ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏽‍🫯‍🧑🏿' => 'daoine ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏾‍🫯‍🧑🏻' => 'daoine ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏾‍🫯‍🧑🏼' => 'daoine ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏾‍🫯‍🧑🏽' => 'daoine ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏾‍🫯‍🧑🏿' => 'daoine ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏿‍🫯‍🧑🏻' => 'daoine ri gleacadh: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏿‍🫯‍🧑🏼' => 'daoine ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏿‍🫯‍🧑🏽' => 'daoine ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏿‍🫯‍🧑🏾' => 'daoine ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍❤‍🧑🏼' => 'càraid le cridhe: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏻‍❤‍🧑🏽' => 'càraid le cridhe: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏻‍❤‍🧑🏾' => 'càraid le cridhe: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍❤‍🧑🏿' => 'càraid le cridhe: inbheach, inbheach, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏼‍❤‍🧑🏻' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏼‍❤‍🧑🏽' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏼‍❤‍🧑🏾' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏼‍❤‍🧑🏿' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏽‍❤‍🧑🏻' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏽‍❤‍🧑🏼' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍❤‍🧑🏾' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏽‍❤‍🧑🏿' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏾‍❤‍🧑🏻' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏾‍❤‍🧑🏼' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏾‍❤‍🧑🏽' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏾‍❤‍🧑🏿' => 'càraid le cridhe: inbheach, inbheach, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏿‍❤‍🧑🏻' => 'càraid le cridhe: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏿‍❤‍🧑🏼' => 'càraid le cridhe: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏿‍❤‍🧑🏽' => 'càraid le cridhe: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏿‍❤‍🧑🏾' => 'càraid le cridhe: inbheach, inbheach, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍🐰‍🧑🏼' => 'daoine le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏻‍🐰‍🧑🏽' => 'daoine le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏻‍🐰‍🧑🏾' => 'daoine le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍🐰‍🧑🏿' => 'daoine le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏼‍🐰‍🧑🏻' => 'daoine le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏼‍🐰‍🧑🏽' => 'daoine le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏼‍🐰‍🧑🏾' => 'daoine le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏼‍🐰‍🧑🏿' => 'daoine le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏽‍🐰‍🧑🏻' => 'daoine le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏽‍🐰‍🧑🏼' => 'daoine le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🐰‍🧑🏾' => 'daoine le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏽‍🐰‍🧑🏿' => 'daoine le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏾‍🐰‍🧑🏻' => 'daoine le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏾‍🐰‍🧑🏼' => 'daoine le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏾‍🐰‍🧑🏽' => 'daoine le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏾‍🐰‍🧑🏿' => 'daoine le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏿‍🐰‍🧑🏻' => 'daoine le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏿‍🐰‍🧑🏼' => 'daoine le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏿‍🐰‍🧑🏽' => 'daoine le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏿‍🐰‍🧑🏾' => 'daoine le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍🫯‍👨🏼' => 'fireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏻‍🫯‍👨🏽' => 'fireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏻‍🫯‍👨🏾' => 'fireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍🫯‍👨🏿' => 'fireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏼‍🫯‍👨🏻' => 'fireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏼‍🫯‍👨🏽' => 'fireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏼‍🫯‍👨🏾' => 'fireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏼‍🫯‍👨🏿' => 'fireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏽‍🫯‍👨🏻' => 'fireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏽‍🫯‍👨🏼' => 'fireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🫯‍👨🏾' => 'fireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏽‍🫯‍👨🏿' => 'fireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏾‍🫯‍👨🏻' => 'fireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏾‍🫯‍👨🏼' => 'fireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏾‍🫯‍👨🏽' => 'fireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏾‍🫯‍👨🏿' => 'fireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏿‍🫯‍👨🏻' => 'fireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏿‍🫯‍👨🏼' => 'fireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏿‍🫯‍👨🏽' => 'fireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏿‍🫯‍👨🏾' => 'fireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍❤‍👨🏻' => 'càraid le cridhe: duine, duine ’s tòna soilleir a’ chraicinn',
    '👨🏻‍❤‍👨🏼' => 'càraid le cridhe: duine, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏻‍❤‍👨🏽' => 'càraid le cridhe: duine, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏻‍❤‍👨🏾' => 'càraid le cridhe: duine, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍❤‍👨🏿' => 'càraid le cridhe: duine, duine, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏼‍❤‍👨🏻' => 'càraid le cridhe: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏼‍❤‍👨🏼' => 'càraid le cridhe: duine, duine ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏼‍❤‍👨🏽' => 'càraid le cridhe: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏼‍❤‍👨🏾' => 'càraid le cridhe: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏼‍❤‍👨🏿' => 'càraid le cridhe: duine, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏽‍❤‍👨🏻' => 'càraid le cridhe: duine, duine, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏽‍❤‍👨🏼' => 'càraid le cridhe: duine, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍❤‍👨🏽' => 'càraid le cridhe: duine, duine ’s tòna meadhanach a’ chraicinn',
    '👨🏽‍❤‍👨🏾' => 'càraid le cridhe: duine, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏽‍❤‍👨🏿' => 'càraid le cridhe: duine, duine, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏾‍❤‍👨🏻' => 'càraid le cridhe: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏾‍❤‍👨🏼' => 'càraid le cridhe: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏾‍❤‍👨🏽' => 'càraid le cridhe: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏾‍❤‍👨🏾' => 'càraid le cridhe: duine, duine ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏾‍❤‍👨🏿' => 'càraid le cridhe: duine, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏿‍❤‍👨🏻' => 'càraid le cridhe: duine, duine, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏿‍❤‍👨🏼' => 'càraid le cridhe: duine, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏿‍❤‍👨🏽' => 'càraid le cridhe: duine, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏿‍❤‍👨🏾' => 'càraid le cridhe: duine, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍❤‍👨🏿' => 'càraid le cridhe: duine, duine ’s tòna doilleir a’ chraicinn',
    '👨🏻‍🐰‍👨🏼' => 'fireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏻‍🐰‍👨🏽' => 'fireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏻‍🐰‍👨🏾' => 'fireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍🐰‍👨🏿' => 'fireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏼‍🐰‍👨🏻' => 'fireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏼‍🐰‍👨🏽' => 'fireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏼‍🐰‍👨🏾' => 'fireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏼‍🐰‍👨🏿' => 'fireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏽‍🐰‍👨🏻' => 'fireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏽‍🐰‍👨🏼' => 'fireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🐰‍👨🏾' => 'fireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏽‍🐰‍👨🏿' => 'fireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏾‍🐰‍👨🏻' => 'fireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏾‍🐰‍👨🏼' => 'fireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏾‍🐰‍👨🏽' => 'fireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏾‍🐰‍👨🏿' => 'fireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏿‍🐰‍👨🏻' => 'fireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏿‍🐰‍👨🏼' => 'fireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏿‍🐰‍👨🏽' => 'fireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏿‍🐰‍👨🏾' => 'fireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍🫯‍👩🏼' => 'boireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍🫯‍👩🏽' => 'boireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍🫯‍👩🏾' => 'boireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍🫯‍👩🏿' => 'boireannaich ri gleacadh: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍🫯‍👩🏻' => 'boireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍🫯‍👩🏽' => 'boireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍🫯‍👩🏾' => 'boireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍🫯‍👩🏿' => 'boireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍🫯‍👩🏻' => 'boireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍🫯‍👩🏼' => 'boireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🫯‍👩🏾' => 'boireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍🫯‍👩🏿' => 'boireannaich ri gleacadh: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍🫯‍👩🏻' => 'boireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍🫯‍👩🏼' => 'boireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍🫯‍👩🏽' => 'boireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍🫯‍👩🏿' => 'boireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍🫯‍👩🏻' => 'boireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍🫯‍👩🏼' => 'boireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍🫯‍👩🏽' => 'boireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍🫯‍👩🏾' => 'boireannaich ri gleacadh: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍❤‍👨🏻' => 'càraid le cridhe: boireannach, duine ’s tòna soilleir a’ chraicinn',
    '👩🏻‍❤‍👨🏼' => 'càraid le cridhe: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍❤‍👨🏽' => 'càraid le cridhe: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍❤‍👨🏾' => 'càraid le cridhe: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍❤‍👨🏿' => 'càraid le cridhe: boireannach, duine, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍❤‍👨🏻' => 'càraid le cridhe: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍❤‍👨🏼' => 'càraid le cridhe: boireannach, duine ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏼‍❤‍👨🏽' => 'càraid le cridhe: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍❤‍👨🏾' => 'càraid le cridhe: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍❤‍👨🏿' => 'càraid le cridhe: boireannach, duine, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍❤‍👨🏻' => 'càraid le cridhe: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍❤‍👨🏼' => 'càraid le cridhe: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍❤‍👨🏽' => 'càraid le cridhe: boireannach, duine ’s tòna meadhanach a’ chraicinn',
    '👩🏽‍❤‍👨🏾' => 'càraid le cridhe: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍❤‍👨🏿' => 'càraid le cridhe: boireannach, duine, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍❤‍👨🏻' => 'càraid le cridhe: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍❤‍👨🏼' => 'càraid le cridhe: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍❤‍👨🏽' => 'càraid le cridhe: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍❤‍👨🏾' => 'càraid le cridhe: boireannach, duine ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏾‍❤‍👨🏿' => 'càraid le cridhe: boireannach, duine, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍❤‍👨🏻' => 'càraid le cridhe: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍❤‍👨🏼' => 'càraid le cridhe: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍❤‍👨🏽' => 'càraid le cridhe: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍❤‍👨🏾' => 'càraid le cridhe: boireannach, duine, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍❤‍👨🏿' => 'càraid le cridhe: boireannach, duine ’s tòna doilleir a’ chraicinn',
    '👩🏻‍❤‍👩🏻' => 'càraid le cridhe: boireannach, boireannach ’s tòna soilleir a’ chraicinn',
    '👩🏻‍❤‍👩🏼' => 'càraid le cridhe: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍❤‍👩🏽' => 'càraid le cridhe: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍❤‍👩🏾' => 'càraid le cridhe: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍❤‍👩🏿' => 'càraid le cridhe: boireannach, boireannach, tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍❤‍👩🏻' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍❤‍👩🏼' => 'càraid le cridhe: boireannach, boireannach ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏼‍❤‍👩🏽' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍❤‍👩🏾' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍❤‍👩🏿' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍❤‍👩🏻' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍❤‍👩🏼' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍❤‍👩🏽' => 'càraid le cridhe: boireannach, boireannach ’s tòna meadhanach a’ chraicinn',
    '👩🏽‍❤‍👩🏾' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍❤‍👩🏿' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍❤‍👩🏻' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍❤‍👩🏼' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍❤‍👩🏽' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍❤‍👩🏾' => 'càraid le cridhe: boireannach, boireannach ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏾‍❤‍👩🏿' => 'càraid le cridhe: boireannach, boireannach, tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍❤‍👩🏻' => 'càraid le cridhe: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍❤‍👩🏼' => 'càraid le cridhe: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍❤‍👩🏽' => 'càraid le cridhe: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍❤‍👩🏾' => 'càraid le cridhe: boireannach, boireannach, tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍❤‍👩🏿' => 'càraid le cridhe: boireannach, boireannach ’s tòna doilleir a’ chraicinn',
    '👩🏻‍🐰‍👩🏼' => 'boireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍🐰‍👩🏽' => 'boireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍🐰‍👩🏾' => 'boireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍🐰‍👩🏿' => 'boireannaich le cluasan rabaid: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍🐰‍👩🏻' => 'boireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍🐰‍👩🏽' => 'boireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍🐰‍👩🏾' => 'boireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍🐰‍👩🏿' => 'boireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍🐰‍👩🏻' => 'boireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍🐰‍👩🏼' => 'boireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🐰‍👩🏾' => 'boireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍🐰‍👩🏿' => 'boireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍🐰‍👩🏻' => 'boireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍🐰‍👩🏼' => 'boireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍🐰‍👩🏽' => 'boireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍🐰‍👩🏿' => 'boireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍🐰‍👩🏻' => 'boireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍🐰‍👩🏼' => 'boireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍🐰‍👩🏽' => 'boireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍🐰‍👩🏾' => 'boireannaich le cluasan rabaid: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍🤝‍🧑🏻' => 'daoine a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn',
    '🧑🏻‍🤝‍🧑🏼' => 'daoine a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏻‍🤝‍🧑🏽' => 'daoine a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏻‍🤝‍🧑🏾' => 'daoine a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏻‍🤝‍🧑🏿' => 'daoine a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏼‍🤝‍🧑🏻' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏼‍🤝‍🧑🏼' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏼‍🤝‍🧑🏽' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏼‍🤝‍🧑🏾' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏼‍🤝‍🧑🏿' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏽‍🤝‍🧑🏻' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏽‍🤝‍🧑🏼' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🤝‍🧑🏽' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn',
    '🧑🏽‍🤝‍🧑🏾' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏽‍🤝‍🧑🏿' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏾‍🤝‍🧑🏻' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏾‍🤝‍🧑🏼' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏾‍🤝‍🧑🏽' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏾‍🤝‍🧑🏾' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏾‍🤝‍🧑🏿' => 'daoine a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🧑🏿‍🤝‍🧑🏻' => 'daoine a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🧑🏿‍🤝‍🧑🏼' => 'daoine a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🧑🏿‍🤝‍🧑🏽' => 'daoine a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🧑🏿‍🤝‍🧑🏾' => 'daoine a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🤝‍🧑🏿' => 'daoine a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn',
    '👩🏻‍🤝‍👩🏼' => 'boireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍🤝‍👩🏽' => 'boireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍🤝‍👩🏾' => 'boireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍🤝‍👩🏿' => 'boireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍🤝‍👩🏻' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍🤝‍👩🏽' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍🤝‍👩🏾' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍🤝‍👩🏿' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍🤝‍👩🏻' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍🤝‍👩🏼' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🤝‍👩🏾' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍🤝‍👩🏿' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍🤝‍👩🏻' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍🤝‍👩🏼' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍🤝‍👩🏽' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍🤝‍👩🏿' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍🤝‍👩🏻' => 'boireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍🤝‍👩🏼' => 'boireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍🤝‍👩🏽' => 'boireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍🤝‍👩🏾' => 'boireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍🤝‍👨🏼' => 'tè is fear a’ gabhail làmhan a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏻‍🤝‍👨🏽' => 'tè is fear a’ gabhail làmhan a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏻‍🤝‍👨🏾' => 'tè is fear a’ gabhail làmhan a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏻‍🤝‍👨🏿' => 'tè is fear a’ gabhail làmhan a chèile: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏼‍🤝‍👨🏻' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏼‍🤝‍👨🏽' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏼‍🤝‍👨🏾' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏼‍🤝‍👨🏿' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏽‍🤝‍👨🏻' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏽‍🤝‍👨🏼' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🤝‍👨🏾' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩🏽‍🤝‍👨🏿' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏾‍🤝‍👨🏻' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏾‍🤝‍👨🏼' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏾‍🤝‍👨🏽' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏾‍🤝‍👨🏿' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👩🏿‍🤝‍👨🏻' => 'tè is fear a’ gabhail làmhan a chèile: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👩🏿‍🤝‍👨🏼' => 'tè is fear a’ gabhail làmhan a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👩🏿‍🤝‍👨🏽' => 'tè is fear a’ gabhail làmhan a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👩🏿‍🤝‍👨🏾' => 'tè is fear a’ gabhail làmhan a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍🤝‍👨🏼' => 'fireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏻‍🤝‍👨🏽' => 'fireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏻‍🤝‍👨🏾' => 'fireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏻‍🤝‍👨🏿' => 'fireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏼‍🤝‍👨🏻' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏼‍🤝‍👨🏽' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏼‍🤝‍👨🏾' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏼‍🤝‍👨🏿' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏽‍🤝‍👨🏻' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏽‍🤝‍👨🏼' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🤝‍👨🏾' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👨🏽‍🤝‍👨🏿' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏾‍🤝‍👨🏻' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏾‍🤝‍👨🏼' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏾‍🤝‍👨🏽' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏾‍🤝‍👨🏿' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '👨🏿‍🤝‍👨🏻' => 'fireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '👨🏿‍🤝‍👨🏼' => 'fireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '👨🏿‍🤝‍👨🏽' => 'fireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '👨🏿‍🤝‍👨🏾' => 'fireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '👩‍❤‍💋‍👨' => 'pòg: boireannach ’s duine',
    '👨‍❤‍💋‍👨' => 'pòg: duine ’s duine',
    '👩‍❤‍💋‍👩' => 'pòg: boireannach ’s boireannach',
    '👨‍👩‍👧‍👦' => 'teaghlach: duine, boireannach, caileag ’s gille',
    '👨‍👩‍👦‍👦' => 'teaghlach: duine, boireannach, gille ’s gille',
    '👨‍👩‍👧‍👧' => 'teaghlach: duine, boireannach, caileag ’s caileag',
    '👨‍👨‍👧‍👦' => 'teaghlach: duine, duine, caileag ’s gille',
    '👨‍👨‍👦‍👦' => 'teaghlach: duine, duine, gille ’s gille',
    '👨‍👨‍👧‍👧' => 'teaghlach: duine, duine, caileag ’s caileag',
    '👩‍👩‍👧‍👦' => 'teaghlach: boireannach, boireannach, caileag ’s gille',
    '👩‍👩‍👦‍👦' => 'teaghlach: boireannach, boireannach, gille ’s gille',
    '👩‍👩‍👧‍👧' => 'teaghlach: boireannach, boireannach, caileag ’s caileag',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'bratach: Sasainn',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'bratach: Alba',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'bratach: A’ Chuimrigh',
    '🧑‍🧑‍🧒‍🧒' => 'teaghlach: dithis inbheach, dithis cloinne',
    '🚶🏻‍♀‍➡' => 'boireannach a’ coiseachd: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏼‍♀‍➡' => 'boireannach a’ coiseachd: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏽‍♀‍➡' => 'boireannach a’ coiseachd: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🚶🏾‍♀‍➡' => 'boireannach a’ coiseachd: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏿‍♀‍➡' => 'boireannach a’ coiseachd: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏻‍♂‍➡' => 'duine a’ coiseachd: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏼‍♂‍➡' => 'duine a’ coiseachd: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏽‍♂‍➡' => 'duine a’ coiseachd: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🚶🏾‍♂‍➡' => 'duine a’ coiseachd: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏿‍♂‍➡' => 'duine a’ coiseachd: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏻‍♀‍➡' => 'boireanach air a ghlùinean: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏼‍♀‍➡' => 'boireanach air a ghlùinean: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏽‍♀‍➡' => 'boireanach air a ghlùinean: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🧎🏾‍♀‍➡' => 'boireanach air a ghlùinean: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏿‍♀‍➡' => 'boireanach air a ghlùinean: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏻‍♂‍➡' => 'duine air a ghlùinean: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏼‍♂‍➡' => 'duine air a ghlùinean: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏽‍♂‍➡' => 'duine air a ghlùinean: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🧎🏾‍♂‍➡' => 'duine air a ghlùinean: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏿‍♂‍➡' => 'duine air a ghlùinean: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏻‍🦯‍➡' => 'neach dall le batachan: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏼‍🦯‍➡' => 'neach dall le batachan: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏽‍🦯‍➡' => 'neach dall le batachan: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🧑🏾‍🦯‍➡' => 'neach dall le batachan: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏿‍🦯‍➡' => 'neach dall le batachan: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏻‍🦯‍➡' => 'duine dall le batachan: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏼‍🦯‍➡' => 'duine dall le batachan: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏽‍🦯‍➡' => 'duine dall le batachan: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '👨🏾‍🦯‍➡' => 'duine dall le batachan: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏿‍🦯‍➡' => 'duine dall le batachan: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏻‍🦯‍➡' => 'boireannach dall le batachan: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏼‍🦯‍➡' => 'boireannach dall le batachan: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏽‍🦯‍➡' => 'boireannach dall le batachan: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '👩🏾‍🦯‍➡' => 'boireannach dall le batachan: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏿‍🦯‍➡' => 'boireannach dall le batachan: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏻‍🦼‍➡' => 'neach ann an cathair-chuibhle le einnsein: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏼‍🦼‍➡' => 'neach ann an cathair-chuibhle le einnsein: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏽‍🦼‍➡' => 'neach ann an cathair-chuibhle le einnsein: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🧑🏾‍🦼‍➡' => 'neach ann an cathair-chuibhle le einnsein: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏿‍🦼‍➡' => 'neach ann an cathair-chuibhle le einnsein: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏻‍🦼‍➡' => 'duine ann an cathair-chuibhle le einnsein: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏼‍🦼‍➡' => 'duine ann an cathair-chuibhle le einnsein: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏽‍🦼‍➡' => 'duine ann an cathair-chuibhle le einnsein: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '👨🏾‍🦼‍➡' => 'duine ann an cathair-chuibhle le einnsein: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏿‍🦼‍➡' => 'duine ann an cathair-chuibhle le einnsein: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏻‍🦼‍➡' => 'tè ann an cathair-chuibhle le einnsein: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏼‍🦼‍➡' => 'tè ann an cathair-chuibhle le einnsein: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏽‍🦼‍➡' => 'tè ann an cathair-chuibhle le einnsein: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '👩🏾‍🦼‍➡' => 'tè ann an cathair-chuibhle le einnsein: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏿‍🦼‍➡' => 'tè ann an cathair-chuibhle le einnsein: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏻‍🦽‍➡' => 'neach ann an cathair-chuibhle a làimh: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏼‍🦽‍➡' => 'neach ann an cathair-chuibhle a làimh: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏽‍🦽‍➡' => 'neach ann an cathair-chuibhle a làimh: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🧑🏾‍🦽‍➡' => 'neach ann an cathair-chuibhle a làimh: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏿‍🦽‍➡' => 'neach ann an cathair-chuibhle a làimh: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏻‍🦽‍➡' => 'duine ann an cathair-chuibhle a làimh: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏼‍🦽‍➡' => 'duine ann an cathair-chuibhle a làimh: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏽‍🦽‍➡' => 'duine ann an cathair-chuibhle a làimh: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '👨🏾‍🦽‍➡' => 'duine ann an cathair-chuibhle a làimh: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '👨🏿‍🦽‍➡' => 'duine ann an cathair-chuibhle a làimh: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏻‍🦽‍➡' => 'tè ann an cathair-chuibhle a làimh: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏼‍🦽‍➡' => 'tè ann an cathair-chuibhle a làimh: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏽‍🦽‍➡' => 'tè ann an cathair-chuibhle a làimh: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '👩🏾‍🦽‍➡' => 'tè ann an cathair-chuibhle a làimh: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '👩🏿‍🦽‍➡' => 'tè ann an cathair-chuibhle a làimh: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏻‍♀‍➡' => 'boireannach a’ ruith: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏼‍♀‍➡' => 'boireannach a’ ruith: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏽‍♀‍➡' => 'boireannach a’ ruith: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🏃🏾‍♀‍➡' => 'boireannach a’ ruith: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏿‍♀‍➡' => 'boireannach a’ ruith: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏻‍♂‍➡' => 'duine a’ ruith: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏼‍♂‍➡' => 'duine a’ ruith: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏽‍♂‍➡' => 'duine a’ ruith: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🏃🏾‍♂‍➡' => 'duine a’ ruith: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏿‍♂‍➡' => 'duine a’ ruith: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🫱🏻‍🫲🏼' => 'crathadh-làimhe: tòna soilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🫱🏻‍🫲🏽' => 'crathadh-làimhe: tòna soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🫱🏻‍🫲🏾' => 'crathadh-làimhe: tòna soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🫱🏻‍🫲🏿' => 'crathadh-làimhe: tòna soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🫱🏼‍🫲🏻' => 'crathadh-làimhe: tòna meadhanach soilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🫱🏼‍🫲🏽' => 'crathadh-làimhe: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🫱🏼‍🫲🏾' => 'crathadh-làimhe: tòna meadhanach soilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🫱🏼‍🫲🏿' => 'crathadh-làimhe: tòna meadhanach soilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🫱🏽‍🫲🏻' => 'crathadh-làimhe: tòna meadhanach a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🫱🏽‍🫲🏼' => 'crathadh-làimhe: tòna meadhanach a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🫱🏽‍🫲🏾' => 'crathadh-làimhe: tòna meadhanach a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🫱🏽‍🫲🏿' => 'crathadh-làimhe: tòna meadhanach a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🫱🏾‍🫲🏻' => 'crathadh-làimhe: tòna meadhanach doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🫱🏾‍🫲🏼' => 'crathadh-làimhe: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🫱🏾‍🫲🏽' => 'crathadh-làimhe: tòna meadhanach doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🫱🏾‍🫲🏿' => 'crathadh-làimhe: tòna meadhanach doilleir a’ chraicinn ’s tòna doilleir a’ chraicinn',
    '🫱🏿‍🫲🏻' => 'crathadh-làimhe: tòna doilleir a’ chraicinn ’s tòna soilleir a’ chraicinn',
    '🫱🏿‍🫲🏼' => 'crathadh-làimhe: tòna doilleir a’ chraicinn ’s tòna meadhanach soilleir a’ chraicinn',
    '🫱🏿‍🫲🏽' => 'crathadh-làimhe: tòna doilleir a’ chraicinn ’s tòna meadhanach a’ chraicinn',
    '🫱🏿‍🫲🏾' => 'crathadh-làimhe: tòna doilleir a’ chraicinn ’s tòna meadhanach doilleir a’ chraicinn',
    '🚶‍♀‍➡' => 'boireannach a’ coiseachd: ris an taobh deas',
    '🚶‍♂‍➡' => 'duine a’ coiseachd: ris an taobh deas',
    '🧎‍♀‍➡' => 'boireanach air a ghlùinean: ris an taobh deas',
    '🧎‍♂‍➡' => 'duine air a ghlùinean: ris an taobh deas',
    '🧑‍🦯‍➡' => 'neach dall le batachan: ris an taobh deas',
    '👨‍🦯‍➡' => 'duine dall le batachan: ris an taobh deas',
    '👩‍🦯‍➡' => 'boireannach dall le batachan: ris an taobh deas',
    '🧑‍🦼‍➡' => 'neach ann an cathair-chuibhle le einnsein: ris an taobh deas',
    '👨‍🦼‍➡' => 'duine ann an cathair-chuibhle le einnsein: ris an taobh deas',
    '👩‍🦼‍➡' => 'tè ann an cathair-chuibhle le einnsein: ris an taobh deas',
    '🧑‍🦽‍➡' => 'neach ann an cathair-chuibhle a làimh: ris an taobh deas',
    '👨‍🦽‍➡' => 'duine ann an cathair-chuibhle a làimh: ris an taobh deas',
    '👩‍🦽‍➡' => 'tè ann an cathair-chuibhle a làimh: ris an taobh deas',
    '🏃‍♀‍➡' => 'boireannach a’ ruith: ris an taobh deas',
    '🏃‍♂‍➡' => 'duine a’ ruith: ris an taobh deas',
    '👩‍❤‍👨' => 'càraid le cridhe: boireannach ’s duine',
    '👨‍❤‍👨' => 'càraid le cridhe: duine ’s duine',
    '👩‍❤‍👩' => 'càraid le cridhe: boireannach ’s boireannach',
    '👨‍👩‍👦' => 'teaghlach: duine, boireannach ’s gille',
    '👨‍👩‍👧' => 'teaghlach: duine, boireannach ’s caileag',
    '👨‍👨‍👦' => 'teaghlach: duine, duine ’s gille',
    '👨‍👨‍👧' => 'teaghlach: duine, duine ’s caileag',
    '👩‍👩‍👦' => 'teaghlach: boireannach, boireannach ’s gille',
    '👩‍👩‍👧' => 'teaghlach: boireannach, boireannach ’s caileag',
    '👨‍👦‍👦' => 'teaghlach: duine, gille ’s gille',
    '👨‍👧‍👦' => 'teaghlach: duine, caileag ’s gille',
    '👨‍👧‍👧' => 'teaghlach: duine, caileag ’s caileag',
    '👩‍👦‍👦' => 'teaghlach: boireannach, gille ’s gille',
    '👩‍👧‍👦' => 'teaghlach: boireannach, caileag ’s gille',
    '👩‍👧‍👧' => 'teaghlach: boireannach, caileag ’s caileag',
    '🧑‍🤝‍🧑' => 'daoine a’ glacadh air làmh a chèile',
    '🧑‍🧑‍🧒' => 'teaghlach: dithis inbheach, duine-cloinne',
    '🧑‍🧒‍🧒' => 'teaghlach: inbheach, dithis cloinne',
    '🧑🏻‍🦰' => 'inbheach: tòna soilleir a’ chraicinn ’s falt ruadh',
    '🧑🏼‍🦰' => 'inbheach: tòna meadhanach soilleir a’ chraicinn ’s falt ruadh',
    '🧑🏽‍🦰' => 'inbheach: tòna meadhanach a’ chraicinn ’s falt ruadh',
    '🧑🏾‍🦰' => 'inbheach: tòna meadhanach doilleir a’ chraicinn ’s falt ruadh',
    '🧑🏿‍🦰' => 'inbheach: tòna doilleir a’ chraicinn ’s falt ruadh',
    '🧑🏻‍🦱' => 'inbheach: tòna soilleir a’ chraicinn ’s falt dualach',
    '🧑🏼‍🦱' => 'inbheach: tòna meadhanach soilleir a’ chraicinn ’s falt dualach',
    '🧑🏽‍🦱' => 'inbheach: tòna meadhanach a’ chraicinn ’s falt dualach',
    '🧑🏾‍🦱' => 'inbheach: tòna meadhanach doilleir a’ chraicinn ’s falt dualach',
    '🧑🏿‍🦱' => 'inbheach: tòna doilleir a’ chraicinn ’s falt dualach',
    '🧑🏻‍🦳' => 'inbheach: tòna soilleir a’ chraicinn ’s falt bàn',
    '🧑🏼‍🦳' => 'inbheach: tòna meadhanach soilleir a’ chraicinn ’s falt bàn',
    '🧑🏽‍🦳' => 'inbheach: tòna meadhanach a’ chraicinn ’s falt bàn',
    '🧑🏾‍🦳' => 'inbheach: tòna meadhanach doilleir a’ chraicinn ’s falt bàn',
    '🧑🏿‍🦳' => 'inbheach: tòna doilleir a’ chraicinn ’s falt bàn',
    '🧑🏻‍🦲' => 'inbheach: tòna soilleir a’ chraicinn ’s maoilean',
    '🧑🏼‍🦲' => 'inbheach: tòna meadhanach soilleir a’ chraicinn ’s maoilean',
    '🧑🏽‍🦲' => 'inbheach: tòna meadhanach a’ chraicinn ’s maoilean',
    '🧑🏾‍🦲' => 'inbheach: tòna meadhanach doilleir a’ chraicinn ’s maoilean',
    '🧑🏿‍🦲' => 'inbheach: tòna doilleir a’ chraicinn ’s maoilean',
    '🧑🏻‍🩰' => 'dannsair baileit: tòna soilleir a’ chraicinn',
    '🧑🏼‍🩰' => 'dannsair baileit: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🩰' => 'dannsair baileit: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🩰' => 'dannsair baileit: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🩰' => 'dannsair baileit: tòna doilleir a’ chraicinn',
    '🧔🏻‍♂' => 'fireannach: tòna soilleir a’ chraicinn ’s feusag',
    '🧔🏼‍♂' => 'fireannach: tòna meadhanach soilleir a’ chraicinn ’s feusag',
    '🧔🏽‍♂' => 'fireannach: tòna meadhanach a’ chraicinn ’s feusag',
    '🧔🏾‍♂' => 'fireannach: tòna meadhanach doilleir a’ chraicinn ’s feusag',
    '🧔🏿‍♂' => 'fireannach: tòna doilleir a’ chraicinn ’s feusag',
    '👱🏻‍♂' => 'duine le falt bàn: tòna soilleir a’ chraicinn',
    '👱🏼‍♂' => 'duine le falt bàn: tòna meadhanach soilleir a’ chraicinn',
    '👱🏽‍♂' => 'duine le falt bàn: tòna meadhanach a’ chraicinn',
    '👱🏾‍♂' => 'duine le falt bàn: tòna meadhanach doilleir a’ chraicinn',
    '👱🏿‍♂' => 'duine le falt bàn: tòna doilleir a’ chraicinn',
    '👨🏻‍🦰' => 'duine: tòna soilleir a’ chraicinn ’s falt ruadh',
    '👨🏼‍🦰' => 'duine: tòna meadhanach soilleir a’ chraicinn ’s falt ruadh',
    '👨🏽‍🦰' => 'duine: tòna meadhanach a’ chraicinn ’s falt ruadh',
    '👨🏾‍🦰' => 'duine: tòna meadhanach doilleir a’ chraicinn ’s falt ruadh',
    '👨🏿‍🦰' => 'duine: tòna doilleir a’ chraicinn ’s falt ruadh',
    '👨🏻‍🦱' => 'duine: tòna soilleir a’ chraicinn ’s falt dualach',
    '👨🏼‍🦱' => 'duine: tòna meadhanach soilleir a’ chraicinn ’s falt dualach',
    '👨🏽‍🦱' => 'duine: tòna meadhanach a’ chraicinn ’s falt dualach',
    '👨🏾‍🦱' => 'duine: tòna meadhanach doilleir a’ chraicinn ’s falt dualach',
    '👨🏿‍🦱' => 'duine: tòna doilleir a’ chraicinn ’s falt dualach',
    '👨🏻‍🦳' => 'duine: tòna soilleir a’ chraicinn ’s falt bàn',
    '👨🏼‍🦳' => 'duine: tòna meadhanach soilleir a’ chraicinn ’s falt bàn',
    '👨🏽‍🦳' => 'duine: tòna meadhanach a’ chraicinn ’s falt bàn',
    '👨🏾‍🦳' => 'duine: tòna meadhanach doilleir a’ chraicinn ’s falt bàn',
    '👨🏿‍🦳' => 'duine: tòna doilleir a’ chraicinn ’s falt bàn',
    '👨🏻‍🦲' => 'duine: tòna soilleir a’ chraicinn ’s maoilean',
    '👨🏼‍🦲' => 'duine: tòna meadhanach soilleir a’ chraicinn ’s maoilean',
    '👨🏽‍🦲' => 'duine: tòna meadhanach a’ chraicinn ’s maoilean',
    '👨🏾‍🦲' => 'duine: tòna meadhanach doilleir a’ chraicinn ’s maoilean',
    '👨🏿‍🦲' => 'duine: tòna doilleir a’ chraicinn ’s maoilean',
    '🧔🏻‍♀' => 'boireannach: tòna soilleir a’ chraicinn ’s feusag',
    '🧔🏼‍♀' => 'boireannach: tòna meadhanach soilleir a’ chraicinn ’s feusag',
    '🧔🏽‍♀' => 'boireannach: tòna meadhanach a’ chraicinn ’s feusag',
    '🧔🏾‍♀' => 'boireannach: tòna meadhanach doilleir a’ chraicinn ’s feusag',
    '🧔🏿‍♀' => 'boireannach: tòna doilleir a’ chraicinn ’s feusag',
    '👱🏻‍♀' => 'boireannach le falt bàn: tòna soilleir a’ chraicinn',
    '👱🏼‍♀' => 'boireannach le falt bàn: tòna meadhanach soilleir a’ chraicinn',
    '👱🏽‍♀' => 'boireannach le falt bàn: tòna meadhanach a’ chraicinn',
    '👱🏾‍♀' => 'boireannach le falt bàn: tòna meadhanach doilleir a’ chraicinn',
    '👱🏿‍♀' => 'boireannach le falt bàn: tòna doilleir a’ chraicinn',
    '👩🏻‍🦰' => 'boireannach: tòna soilleir a’ chraicinn ’s falt ruadh',
    '👩🏼‍🦰' => 'boireannach: tòna meadhanach soilleir a’ chraicinn ’s falt ruadh',
    '👩🏽‍🦰' => 'boireannach: tòna meadhanach a’ chraicinn ’s falt ruadh',
    '👩🏾‍🦰' => 'boireannach: tòna meadhanach doilleir a’ chraicinn ’s falt ruadh',
    '👩🏿‍🦰' => 'boireannach: tòna doilleir a’ chraicinn ’s falt ruadh',
    '👩🏻‍🦱' => 'boireannach: tòna soilleir a’ chraicinn ’s falt dualach',
    '👩🏼‍🦱' => 'boireannach: tòna meadhanach soilleir a’ chraicinn ’s falt dualach',
    '👩🏽‍🦱' => 'boireannach: tòna meadhanach a’ chraicinn ’s falt dualach',
    '👩🏾‍🦱' => 'boireannach: tòna meadhanach doilleir a’ chraicinn ’s falt dualach',
    '👩🏿‍🦱' => 'boireannach: tòna doilleir a’ chraicinn ’s falt dualach',
    '👩🏻‍🦳' => 'boireannach: tòna soilleir a’ chraicinn ’s falt bàn',
    '👩🏼‍🦳' => 'boireannach: tòna meadhanach soilleir a’ chraicinn ’s falt bàn',
    '👩🏽‍🦳' => 'boireannach: tòna meadhanach a’ chraicinn ’s falt bàn',
    '👩🏾‍🦳' => 'boireannach: tòna meadhanach doilleir a’ chraicinn ’s falt bàn',
    '👩🏿‍🦳' => 'boireannach: tòna doilleir a’ chraicinn ’s falt bàn',
    '👩🏻‍🦲' => 'boireannach: tòna soilleir a’ chraicinn ’s maoilean',
    '👩🏼‍🦲' => 'boireannach: tòna meadhanach soilleir a’ chraicinn ’s maoilean',
    '👩🏽‍🦲' => 'boireannach: tòna meadhanach a’ chraicinn ’s maoilean',
    '👩🏾‍🦲' => 'boireannach: tòna meadhanach doilleir a’ chraicinn ’s maoilean',
    '👩🏿‍🦲' => 'boireannach: tòna doilleir a’ chraicinn ’s maoilean',
    '🙍🏻‍♂' => 'duine le drèin: tòna soilleir a’ chraicinn',
    '🙍🏼‍♂' => 'duine le drèin: tòna meadhanach soilleir a’ chraicinn',
    '🙍🏽‍♂' => 'duine le drèin: tòna meadhanach a’ chraicinn',
    '🙍🏾‍♂' => 'duine le drèin: tòna meadhanach doilleir a’ chraicinn',
    '🙍🏿‍♂' => 'duine le drèin: tòna doilleir a’ chraicinn',
    '🙍🏻‍♀' => 'boireannach le drèin: tòna soilleir a’ chraicinn',
    '🙍🏼‍♀' => 'boireannach le drèin: tòna meadhanach soilleir a’ chraicinn',
    '🙍🏽‍♀' => 'boireannach le drèin: tòna meadhanach a’ chraicinn',
    '🙍🏾‍♀' => 'boireannach le drèin: tòna meadhanach doilleir a’ chraicinn',
    '🙍🏿‍♀' => 'boireannach le drèin: tòna doilleir a’ chraicinn',
    '🙎🏻‍♂' => 'duine le bus air: tòna soilleir a’ chraicinn',
    '🙎🏼‍♂' => 'duine le bus air: tòna meadhanach soilleir a’ chraicinn',
    '🙎🏽‍♂' => 'duine le bus air: tòna meadhanach a’ chraicinn',
    '🙎🏾‍♂' => 'duine le bus air: tòna meadhanach doilleir a’ chraicinn',
    '🙎🏿‍♂' => 'duine le bus air: tòna doilleir a’ chraicinn',
    '🙎🏻‍♀' => 'boireannach le bus oirre: tòna soilleir a’ chraicinn',
    '🙎🏼‍♀' => 'boireannach le bus oirre: tòna meadhanach soilleir a’ chraicinn',
    '🙎🏽‍♀' => 'boireannach le bus oirre: tòna meadhanach a’ chraicinn',
    '🙎🏾‍♀' => 'boireannach le bus oirre: tòna meadhanach doilleir a’ chraicinn',
    '🙎🏿‍♀' => 'boireannach le bus oirre: tòna doilleir a’ chraicinn',
    '🙅🏻‍♂' => 'duine a’ diùltadh rud: tòna soilleir a’ chraicinn',
    '🙅🏼‍♂' => 'duine a’ diùltadh rud: tòna meadhanach soilleir a’ chraicinn',
    '🙅🏽‍♂' => 'duine a’ diùltadh rud: tòna meadhanach a’ chraicinn',
    '🙅🏾‍♂' => 'duine a’ diùltadh rud: tòna meadhanach doilleir a’ chraicinn',
    '🙅🏿‍♂' => 'duine a’ diùltadh rud: tòna doilleir a’ chraicinn',
    '🙅🏻‍♀' => 'boireannach a’ diùltadh rud: tòna soilleir a’ chraicinn',
    '🙅🏼‍♀' => 'boireannach a’ diùltadh rud: tòna meadhanach soilleir a’ chraicinn',
    '🙅🏽‍♀' => 'boireannach a’ diùltadh rud: tòna meadhanach a’ chraicinn',
    '🙅🏾‍♀' => 'boireannach a’ diùltadh rud: tòna meadhanach doilleir a’ chraicinn',
    '🙅🏿‍♀' => 'boireannach a’ diùltadh rud: tòna doilleir a’ chraicinn',
    '🙆🏻‍♂' => 'duine ag aontachadh ri rud: tòna soilleir a’ chraicinn',
    '🙆🏼‍♂' => 'duine ag aontachadh ri rud: tòna meadhanach soilleir a’ chraicinn',
    '🙆🏽‍♂' => 'duine ag aontachadh ri rud: tòna meadhanach a’ chraicinn',
    '🙆🏾‍♂' => 'duine ag aontachadh ri rud: tòna meadhanach doilleir a’ chraicinn',
    '🙆🏿‍♂' => 'duine ag aontachadh ri rud: tòna doilleir a’ chraicinn',
    '🙆🏻‍♀' => 'boireannach ag aontachadh ri rud: tòna soilleir a’ chraicinn',
    '🙆🏼‍♀' => 'boireannach ag aontachadh ri rud: tòna meadhanach soilleir a’ chraicinn',
    '🙆🏽‍♀' => 'boireannach ag aontachadh ri rud: tòna meadhanach a’ chraicinn',
    '🙆🏾‍♀' => 'boireannach ag aontachadh ri rud: tòna meadhanach doilleir a’ chraicinn',
    '🙆🏿‍♀' => 'boireannach ag aontachadh ri rud: tòna doilleir a’ chraicinn',
    '💁🏻‍♂' => 'duine le làmh sìnte, bas air uachdar: tòna soilleir a’ chraicinn',
    '💁🏼‍♂' => 'duine le làmh sìnte, bas air uachdar: tòna meadhanach soilleir a’ chraicinn',
    '💁🏽‍♂' => 'duine le làmh sìnte, bas air uachdar: tòna meadhanach a’ chraicinn',
    '💁🏾‍♂' => 'duine le làmh sìnte, bas air uachdar: tòna meadhanach doilleir a’ chraicinn',
    '💁🏿‍♂' => 'duine le làmh sìnte, bas air uachdar: tòna doilleir a’ chraicinn',
    '💁🏻‍♀' => 'tè le làmh sìnte a-mach, bas air uachdar: tòna soilleir a’ chraicinn',
    '💁🏼‍♀' => 'tè le làmh sìnte a-mach, bas air uachdar: tòna meadhanach soilleir a’ chraicinn',
    '💁🏽‍♀' => 'tè le làmh sìnte a-mach, bas air uachdar: tòna meadhanach a’ chraicinn',
    '💁🏾‍♀' => 'tè le làmh sìnte a-mach, bas air uachdar: tòna meadhanach doilleir a’ chraicinn',
    '💁🏿‍♀' => 'tè le làmh sìnte a-mach, bas air uachdar: tòna doilleir a’ chraicinn',
    '🙋🏻‍♂' => 'duine a’ togail a làmh: tòna soilleir a’ chraicinn',
    '🙋🏼‍♂' => 'duine a’ togail a làmh: tòna meadhanach soilleir a’ chraicinn',
    '🙋🏽‍♂' => 'duine a’ togail a làmh: tòna meadhanach a’ chraicinn',
    '🙋🏾‍♂' => 'duine a’ togail a làmh: tòna meadhanach doilleir a’ chraicinn',
    '🙋🏿‍♂' => 'duine a’ togail a làmh: tòna doilleir a’ chraicinn',
    '🙋🏻‍♀' => 'boireannach a’ togail a làmh: tòna soilleir a’ chraicinn',
    '🙋🏼‍♀' => 'boireannach a’ togail a làmh: tòna meadhanach soilleir a’ chraicinn',
    '🙋🏽‍♀' => 'boireannach a’ togail a làmh: tòna meadhanach a’ chraicinn',
    '🙋🏾‍♀' => 'boireannach a’ togail a làmh: tòna meadhanach doilleir a’ chraicinn',
    '🙋🏿‍♀' => 'boireannach a’ togail a làmh: tòna doilleir a’ chraicinn',
    '🧏🏻‍♂' => 'duine bodhar: tòna soilleir a’ chraicinn',
    '🧏🏼‍♂' => 'duine bodhar: tòna meadhanach soilleir a’ chraicinn',
    '🧏🏽‍♂' => 'duine bodhar: tòna meadhanach a’ chraicinn',
    '🧏🏾‍♂' => 'duine bodhar: tòna meadhanach doilleir a’ chraicinn',
    '🧏🏿‍♂' => 'duine bodhar: tòna doilleir a’ chraicinn',
    '🧏🏻‍♀' => 'boireannach bodhar: tòna soilleir a’ chraicinn',
    '🧏🏼‍♀' => 'boireannach bodhar: tòna meadhanach soilleir a’ chraicinn',
    '🧏🏽‍♀' => 'boireannach bodhar: tòna meadhanach a’ chraicinn',
    '🧏🏾‍♀' => 'boireannach bodhar: tòna meadhanach doilleir a’ chraicinn',
    '🧏🏿‍♀' => 'boireannach bodhar: tòna doilleir a’ chraicinn',
    '🙇🏻‍♂' => 'duine a’ cromadh: tòna soilleir a’ chraicinn',
    '🙇🏼‍♂' => 'duine a’ cromadh: tòna meadhanach soilleir a’ chraicinn',
    '🙇🏽‍♂' => 'duine a’ cromadh: tòna meadhanach a’ chraicinn',
    '🙇🏾‍♂' => 'duine a’ cromadh: tòna meadhanach doilleir a’ chraicinn',
    '🙇🏿‍♂' => 'duine a’ cromadh: tòna doilleir a’ chraicinn',
    '🙇🏻‍♀' => 'boireannach a’ cromadh: tòna soilleir a’ chraicinn',
    '🙇🏼‍♀' => 'boireannach a’ cromadh: tòna meadhanach soilleir a’ chraicinn',
    '🙇🏽‍♀' => 'boireannach a’ cromadh: tòna meadhanach a’ chraicinn',
    '🙇🏾‍♀' => 'boireannach a’ cromadh: tòna meadhanach doilleir a’ chraicinn',
    '🙇🏿‍♀' => 'boireannach a’ cromadh: tòna doilleir a’ chraicinn',
    '🤦🏻‍♂' => 'duine le aodann na làmhan: tòna soilleir a’ chraicinn',
    '🤦🏼‍♂' => 'duine le aodann na làmhan: tòna meadhanach soilleir a’ chraicinn',
    '🤦🏽‍♂' => 'duine le aodann na làmhan: tòna meadhanach a’ chraicinn',
    '🤦🏾‍♂' => 'duine le aodann na làmhan: tòna meadhanach doilleir a’ chraicinn',
    '🤦🏿‍♂' => 'duine le aodann na làmhan: tòna doilleir a’ chraicinn',
    '🤦🏻‍♀' => 'boireannach le aodann na làmhan: tòna soilleir a’ chraicinn',
    '🤦🏼‍♀' => 'boireannach le aodann na làmhan: tòna meadhanach soilleir a’ chraicinn',
    '🤦🏽‍♀' => 'boireannach le aodann na làmhan: tòna meadhanach a’ chraicinn',
    '🤦🏾‍♀' => 'boireannach le aodann na làmhan: tòna meadhanach doilleir a’ chraicinn',
    '🤦🏿‍♀' => 'boireannach le aodann na làmhan: tòna doilleir a’ chraicinn',
    '🤷🏻‍♂' => 'duine a’ druiteadh: tòna soilleir a’ chraicinn',
    '🤷🏼‍♂' => 'duine a’ druiteadh: tòna meadhanach soilleir a’ chraicinn',
    '🤷🏽‍♂' => 'duine a’ druiteadh: tòna meadhanach a’ chraicinn',
    '🤷🏾‍♂' => 'duine a’ druiteadh: tòna meadhanach doilleir a’ chraicinn',
    '🤷🏿‍♂' => 'duine a’ druiteadh: tòna doilleir a’ chraicinn',
    '🤷🏻‍♀' => 'boireannach a’ druiteadh: tòna soilleir a’ chraicinn',
    '🤷🏼‍♀' => 'boireannach a’ druiteadh: tòna meadhanach soilleir a’ chraicinn',
    '🤷🏽‍♀' => 'boireannach a’ druiteadh: tòna meadhanach a’ chraicinn',
    '🤷🏾‍♀' => 'boireannach a’ druiteadh: tòna meadhanach doilleir a’ chraicinn',
    '🤷🏿‍♀' => 'boireannach a’ druiteadh: tòna doilleir a’ chraicinn',
    '🧑🏻‍⚕' => 'obraiche-slàinte: tòna soilleir a’ chraicinn',
    '🧑🏼‍⚕' => 'obraiche-slàinte: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍⚕' => 'obraiche-slàinte: tòna meadhanach a’ chraicinn',
    '🧑🏾‍⚕' => 'obraiche-slàinte: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍⚕' => 'obraiche-slàinte: tòna doilleir a’ chraicinn',
    '👨🏻‍⚕' => 'obraiche-slàinte fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍⚕' => 'obraiche-slàinte fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍⚕' => 'obraiche-slàinte fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍⚕' => 'obraiche-slàinte fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍⚕' => 'obraiche-slàinte fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍⚕' => 'obraiche-slàinte boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍⚕' => 'obraiche-slàinte boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍⚕' => 'obraiche-slàinte boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍⚕' => 'obraiche-slàinte boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍⚕' => 'obraiche-slàinte boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍🎓' => 'oileanach: tòna soilleir a’ chraicinn',
    '🧑🏼‍🎓' => 'oileanach: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🎓' => 'oileanach: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🎓' => 'oileanach: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🎓' => 'oileanach: tòna doilleir a’ chraicinn',
    '👨🏻‍🎓' => 'oileanach fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🎓' => 'oileanach fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🎓' => 'oileanach fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🎓' => 'oileanach fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🎓' => 'oileanach fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🎓' => 'ban-oileanach: tòna soilleir a’ chraicinn',
    '👩🏼‍🎓' => 'ban-oileanach: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🎓' => 'ban-oileanach: tòna meadhanach a’ chraicinn',
    '👩🏾‍🎓' => 'ban-oileanach: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🎓' => 'ban-oileanach: tòna doilleir a’ chraicinn',
    '🧑🏻‍🏫' => 'tidsear: tòna soilleir a’ chraicinn',
    '🧑🏼‍🏫' => 'tidsear: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🏫' => 'tidsear: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🏫' => 'tidsear: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🏫' => 'tidsear: tòna doilleir a’ chraicinn',
    '👨🏻‍🏫' => 'oide fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🏫' => 'oide fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🏫' => 'oide fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🏫' => 'oide fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🏫' => 'oide fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🏫' => 'ban-oide: tòna soilleir a’ chraicinn',
    '👩🏼‍🏫' => 'ban-oide: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🏫' => 'ban-oide: tòna meadhanach a’ chraicinn',
    '👩🏾‍🏫' => 'ban-oide: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🏫' => 'ban-oide: tòna doilleir a’ chraicinn',
    '🧑🏻‍⚖' => 'britheamh: tòna soilleir a’ chraicinn',
    '🧑🏼‍⚖' => 'britheamh: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍⚖' => 'britheamh: tòna meadhanach a’ chraicinn',
    '🧑🏾‍⚖' => 'britheamh: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍⚖' => 'britheamh: tòna doilleir a’ chraicinn',
    '👨🏻‍⚖' => 'britheamh fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍⚖' => 'britheamh fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍⚖' => 'britheamh fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍⚖' => 'britheamh fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍⚖' => 'britheamh fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍⚖' => 'britheamh boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍⚖' => 'britheamh boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍⚖' => 'britheamh boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍⚖' => 'britheamh boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍⚖' => 'britheamh boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍🌾' => 'tuathanach: tòna soilleir a’ chraicinn',
    '🧑🏼‍🌾' => 'tuathanach: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🌾' => 'tuathanach: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🌾' => 'tuathanach: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🌾' => 'tuathanach: tòna doilleir a’ chraicinn',
    '👨🏻‍🌾' => 'tuathanaiche fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🌾' => 'tuathanaiche fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🌾' => 'tuathanaiche fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🌾' => 'tuathanaiche fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🌾' => 'tuathanaiche fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🌾' => 'tuathanaiche boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍🌾' => 'tuathanaiche boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🌾' => 'tuathanaiche boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍🌾' => 'tuathanaiche boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🌾' => 'tuathanaiche boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍🍳' => 'còcaire: tòna soilleir a’ chraicinn',
    '🧑🏼‍🍳' => 'còcaire: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🍳' => 'còcaire: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🍳' => 'còcaire: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🍳' => 'còcaire: tòna doilleir a’ chraicinn',
    '👨🏻‍🍳' => 'còcaire fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🍳' => 'còcaire fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🍳' => 'còcaire fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🍳' => 'còcaire fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🍳' => 'còcaire fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🍳' => 'ban-chòcaire: tòna soilleir a’ chraicinn',
    '👩🏼‍🍳' => 'ban-chòcaire: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🍳' => 'ban-chòcaire: tòna meadhanach a’ chraicinn',
    '👩🏾‍🍳' => 'ban-chòcaire: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🍳' => 'ban-chòcaire: tòna doilleir a’ chraicinn',
    '🧑🏻‍🔧' => 'meacanaig: tòna soilleir a’ chraicinn',
    '🧑🏼‍🔧' => 'meacanaig: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🔧' => 'meacanaig: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🔧' => 'meacanaig: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🔧' => 'meacanaig: tòna doilleir a’ chraicinn',
    '👨🏻‍🔧' => 'meacanaig fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🔧' => 'meacanaig fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🔧' => 'meacanaig fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🔧' => 'meacanaig fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🔧' => 'meacanaig fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🔧' => 'meacanaig boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍🔧' => 'meacanaig boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🔧' => 'meacanaig boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍🔧' => 'meacanaig boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🔧' => 'meacanaig boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍🏭' => 'neach-obrach factaraidh: tòna soilleir a’ chraicinn',
    '🧑🏼‍🏭' => 'neach-obrach factaraidh: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🏭' => 'neach-obrach factaraidh: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🏭' => 'neach-obrach factaraidh: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🏭' => 'neach-obrach factaraidh: tòna doilleir a’ chraicinn',
    '👨🏻‍🏭' => 'obraiche-factaraidh fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🏭' => 'obraiche-factaraidh fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🏭' => 'obraiche-factaraidh fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🏭' => 'obraiche-factaraidh fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🏭' => 'obraiche-factaraidh fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🏭' => 'obraiche-factaraidh boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍🏭' => 'obraiche-factaraidh boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🏭' => 'obraiche-factaraidh boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍🏭' => 'obraiche-factaraidh boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🏭' => 'obraiche-factaraidh boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍💼' => 'neach-obrach oifise: tòna soilleir a’ chraicinn',
    '🧑🏼‍💼' => 'neach-obrach oifise: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍💼' => 'neach-obrach oifise: tòna meadhanach a’ chraicinn',
    '🧑🏾‍💼' => 'neach-obrach oifise: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍💼' => 'neach-obrach oifise: tòna doilleir a’ chraicinn',
    '👨🏻‍💼' => 'obraiche-oifis fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍💼' => 'obraiche-oifis fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍💼' => 'obraiche-oifis fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍💼' => 'obraiche-oifis fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍💼' => 'obraiche-oifis fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍💼' => 'obraiche-oifis boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍💼' => 'obraiche-oifis boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍💼' => 'obraiche-oifis boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍💼' => 'obraiche-oifis boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍💼' => 'obraiche-oifis boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍🔬' => 'neach-saidheans: tòna soilleir a’ chraicinn',
    '🧑🏼‍🔬' => 'neach-saidheans: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🔬' => 'neach-saidheans: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🔬' => 'neach-saidheans: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🔬' => 'neach-saidheans: tòna doilleir a’ chraicinn',
    '👨🏻‍🔬' => 'neach-saidheans fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🔬' => 'neach-saidheans fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🔬' => 'neach-saidheans fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🔬' => 'neach-saidheans fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🔬' => 'neach-saidheans fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🔬' => 'neach-saidheans boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍🔬' => 'neach-saidheans boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🔬' => 'neach-saidheans boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍🔬' => 'neach-saidheans boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🔬' => 'neach-saidheans boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍💻' => 'teicneolaiche: tòna soilleir a’ chraicinn',
    '🧑🏼‍💻' => 'teicneolaiche: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍💻' => 'teicneolaiche: tòna meadhanach a’ chraicinn',
    '🧑🏾‍💻' => 'teicneolaiche: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍💻' => 'teicneolaiche: tòna doilleir a’ chraicinn',
    '👨🏻‍💻' => 'teicneolaiche fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍💻' => 'teicneolaiche fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍💻' => 'teicneolaiche fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍💻' => 'teicneolaiche fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍💻' => 'teicneolaiche fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍💻' => 'teicneolaiche boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍💻' => 'teicneolaiche boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍💻' => 'teicneolaiche boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍💻' => 'teicneolaiche boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍💻' => 'teicneolaiche boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍🎤' => 'seinneadair: tòna soilleir a’ chraicinn',
    '🧑🏼‍🎤' => 'seinneadair: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🎤' => 'seinneadair: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🎤' => 'seinneadair: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🎤' => 'seinneadair: tòna doilleir a’ chraicinn',
    '👨🏻‍🎤' => 'seinneadair fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🎤' => 'seinneadair fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🎤' => 'seinneadair fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🎤' => 'seinneadair fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🎤' => 'seinneadair fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🎤' => 'ban-seinneadair: tòna soilleir a’ chraicinn',
    '👩🏼‍🎤' => 'ban-seinneadair: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🎤' => 'ban-seinneadair: tòna meadhanach a’ chraicinn',
    '👩🏾‍🎤' => 'ban-seinneadair: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🎤' => 'ban-seinneadair: tòna doilleir a’ chraicinn',
    '🧑🏻‍🎨' => 'neach-ealain: tòna soilleir a’ chraicinn',
    '🧑🏼‍🎨' => 'neach-ealain: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🎨' => 'neach-ealain: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🎨' => 'neach-ealain: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🎨' => 'neach-ealain: tòna doilleir a’ chraicinn',
    '👨🏻‍🎨' => 'fear-ealain: tòna soilleir a’ chraicinn',
    '👨🏼‍🎨' => 'fear-ealain: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🎨' => 'fear-ealain: tòna meadhanach a’ chraicinn',
    '👨🏾‍🎨' => 'fear-ealain: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🎨' => 'fear-ealain: tòna doilleir a’ chraicinn',
    '👩🏻‍🎨' => 'neach-ealain boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍🎨' => 'neach-ealain boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🎨' => 'neach-ealain boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍🎨' => 'neach-ealain boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🎨' => 'neach-ealain boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍✈' => 'paidhleat: tòna soilleir a’ chraicinn',
    '🧑🏼‍✈' => 'paidhleat: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍✈' => 'paidhleat: tòna meadhanach a’ chraicinn',
    '🧑🏾‍✈' => 'paidhleat: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍✈' => 'paidhleat: tòna doilleir a’ chraicinn',
    '👨🏻‍✈' => 'paidhleat fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍✈' => 'paidhleat fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍✈' => 'paidhleat fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍✈' => 'paidhleat fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍✈' => 'paidhleat fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍✈' => 'paidhleat boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍✈' => 'paidhleat boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍✈' => 'paidhleat boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍✈' => 'paidhleat boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍✈' => 'paidhleat boireann: tòna doilleir a’ chraicinn',
    '🧑🏻‍🚀' => 'speuradair: tòna soilleir a’ chraicinn',
    '🧑🏼‍🚀' => 'speuradair: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🚀' => 'speuradair: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🚀' => 'speuradair: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🚀' => 'speuradair: tòna doilleir a’ chraicinn',
    '👨🏻‍🚀' => 'speuradair fireann: tòna soilleir a’ chraicinn',
    '👨🏼‍🚀' => 'speuradair fireann: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🚀' => 'speuradair fireann: tòna meadhanach a’ chraicinn',
    '👨🏾‍🚀' => 'speuradair fireann: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🚀' => 'speuradair fireann: tòna doilleir a’ chraicinn',
    '👩🏻‍🚀' => 'ban-speuradair: tòna soilleir a’ chraicinn',
    '👩🏼‍🚀' => 'ban-speuradair: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🚀' => 'ban-speuradair: tòna meadhanach a’ chraicinn',
    '👩🏾‍🚀' => 'ban-speuradair: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🚀' => 'ban-speuradair: tòna doilleir a’ chraicinn',
    '🧑🏻‍🚒' => 'smàladair: tòna soilleir a’ chraicinn',
    '🧑🏼‍🚒' => 'smàladair: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🚒' => 'smàladair: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🚒' => 'smàladair: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🚒' => 'smàladair: tòna doilleir a’ chraicinn',
    '👨🏻‍🚒' => 'fear-smàlaidh: tòna soilleir a’ chraicinn',
    '👨🏼‍🚒' => 'fear-smàlaidh: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🚒' => 'fear-smàlaidh: tòna meadhanach a’ chraicinn',
    '👨🏾‍🚒' => 'fear-smàlaidh: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🚒' => 'fear-smàlaidh: tòna doilleir a’ chraicinn',
    '👩🏻‍🚒' => 'neach-smàlaidh boireann: tòna soilleir a’ chraicinn',
    '👩🏼‍🚒' => 'neach-smàlaidh boireann: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🚒' => 'neach-smàlaidh boireann: tòna meadhanach a’ chraicinn',
    '👩🏾‍🚒' => 'neach-smàlaidh boireann: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🚒' => 'neach-smàlaidh boireann: tòna doilleir a’ chraicinn',
    '👮🏻‍♂' => 'oifigear poileis fireann: tòna soilleir a’ chraicinn',
    '👮🏼‍♂' => 'oifigear poileis fireann: tòna meadhanach soilleir a’ chraicinn',
    '👮🏽‍♂' => 'oifigear poileis fireann: tòna meadhanach a’ chraicinn',
    '👮🏾‍♂' => 'oifigear poileis fireann: tòna meadhanach doilleir a’ chraicinn',
    '👮🏿‍♂' => 'oifigear poileis fireann: tòna doilleir a’ chraicinn',
    '👮🏻‍♀' => 'oifigear poileis boireann: tòna soilleir a’ chraicinn',
    '👮🏼‍♀' => 'oifigear poileis boireann: tòna meadhanach soilleir a’ chraicinn',
    '👮🏽‍♀' => 'oifigear poileis boireann: tòna meadhanach a’ chraicinn',
    '👮🏾‍♀' => 'oifigear poileis boireann: tòna meadhanach doilleir a’ chraicinn',
    '👮🏿‍♀' => 'oifigear poileis boireann: tòna doilleir a’ chraicinn',
    '🕵🏻‍♂' => 'lorgaire fireann: tòna soilleir a’ chraicinn',
    '🕵🏼‍♂' => 'lorgaire fireann: tòna meadhanach soilleir a’ chraicinn',
    '🕵🏽‍♂' => 'lorgaire fireann: tòna meadhanach a’ chraicinn',
    '🕵🏾‍♂' => 'lorgaire fireann: tòna meadhanach doilleir a’ chraicinn',
    '🕵🏿‍♂' => 'lorgaire fireann: tòna doilleir a’ chraicinn',
    '🕵🏻‍♀' => 'ban-lorgaire: tòna soilleir a’ chraicinn',
    '🕵🏼‍♀' => 'ban-lorgaire: tòna meadhanach soilleir a’ chraicinn',
    '🕵🏽‍♀' => 'ban-lorgaire: tòna meadhanach a’ chraicinn',
    '🕵🏾‍♀' => 'ban-lorgaire: tòna meadhanach doilleir a’ chraicinn',
    '🕵🏿‍♀' => 'ban-lorgaire: tòna doilleir a’ chraicinn',
    '💂🏻‍♂' => 'geàrd fireann: tòna soilleir a’ chraicinn',
    '💂🏼‍♂' => 'geàrd fireann: tòna meadhanach soilleir a’ chraicinn',
    '💂🏽‍♂' => 'geàrd fireann: tòna meadhanach a’ chraicinn',
    '💂🏾‍♂' => 'geàrd fireann: tòna meadhanach doilleir a’ chraicinn',
    '💂🏿‍♂' => 'geàrd fireann: tòna doilleir a’ chraicinn',
    '💂🏻‍♀' => 'bana-gheàrd: tòna soilleir a’ chraicinn',
    '💂🏼‍♀' => 'bana-gheàrd: tòna meadhanach soilleir a’ chraicinn',
    '💂🏽‍♀' => 'bana-gheàrd: tòna meadhanach a’ chraicinn',
    '💂🏾‍♀' => 'bana-gheàrd: tòna meadhanach doilleir a’ chraicinn',
    '💂🏿‍♀' => 'bana-gheàrd: tòna doilleir a’ chraicinn',
    '👷🏻‍♂' => 'fear-togail: tòna soilleir a’ chraicinn',
    '👷🏼‍♂' => 'fear-togail: tòna meadhanach soilleir a’ chraicinn',
    '👷🏽‍♂' => 'fear-togail: tòna meadhanach a’ chraicinn',
    '👷🏾‍♂' => 'fear-togail: tòna meadhanach doilleir a’ chraicinn',
    '👷🏿‍♂' => 'fear-togail: tòna doilleir a’ chraicinn',
    '👷🏻‍♀' => 'neach-togail boireann: tòna soilleir a’ chraicinn',
    '👷🏼‍♀' => 'neach-togail boireann: tòna meadhanach soilleir a’ chraicinn',
    '👷🏽‍♀' => 'neach-togail boireann: tòna meadhanach a’ chraicinn',
    '👷🏾‍♀' => 'neach-togail boireann: tòna meadhanach doilleir a’ chraicinn',
    '👷🏿‍♀' => 'neach-togail boireann: tòna doilleir a’ chraicinn',
    '👳🏻‍♂' => 'duine le turban air: tòna soilleir a’ chraicinn',
    '👳🏼‍♂' => 'duine le turban air: tòna meadhanach soilleir a’ chraicinn',
    '👳🏽‍♂' => 'duine le turban air: tòna meadhanach a’ chraicinn',
    '👳🏾‍♂' => 'duine le turban air: tòna meadhanach doilleir a’ chraicinn',
    '👳🏿‍♂' => 'duine le turban air: tòna doilleir a’ chraicinn',
    '👳🏻‍♀' => 'boireannach le turban oirre: tòna soilleir a’ chraicinn',
    '👳🏼‍♀' => 'boireannach le turban oirre: tòna meadhanach soilleir a’ chraicinn',
    '👳🏽‍♀' => 'boireannach le turban oirre: tòna meadhanach a’ chraicinn',
    '👳🏾‍♀' => 'boireannach le turban oirre: tòna meadhanach doilleir a’ chraicinn',
    '👳🏿‍♀' => 'boireannach le turban oirre: tòna doilleir a’ chraicinn',
    '🤵🏻‍♂' => 'duine ann an tuxedo: tòna soilleir a’ chraicinn',
    '🤵🏼‍♂' => 'duine ann an tuxedo: tòna meadhanach soilleir a’ chraicinn',
    '🤵🏽‍♂' => 'duine ann an tuxedo: tòna meadhanach a’ chraicinn',
    '🤵🏾‍♂' => 'duine ann an tuxedo: tòna meadhanach doilleir a’ chraicinn',
    '🤵🏿‍♂' => 'duine ann an tuxedo: tòna doilleir a’ chraicinn',
    '🤵🏻‍♀' => 'boireannach ann an tuxedo: tòna soilleir a’ chraicinn',
    '🤵🏼‍♀' => 'boireannach ann an tuxedo: tòna meadhanach soilleir a’ chraicinn',
    '🤵🏽‍♀' => 'boireannach ann an tuxedo: tòna meadhanach a’ chraicinn',
    '🤵🏾‍♀' => 'boireannach ann an tuxedo: tòna meadhanach doilleir a’ chraicinn',
    '🤵🏿‍♀' => 'boireannach ann an tuxedo: tòna doilleir a’ chraicinn',
    '👰🏻‍♂' => 'duine le brat-gnùise: tòna soilleir a’ chraicinn',
    '👰🏼‍♂' => 'duine le brat-gnùise: tòna meadhanach soilleir a’ chraicinn',
    '👰🏽‍♂' => 'duine le brat-gnùise: tòna meadhanach a’ chraicinn',
    '👰🏾‍♂' => 'duine le brat-gnùise: tòna meadhanach doilleir a’ chraicinn',
    '👰🏿‍♂' => 'duine le brat-gnùise: tòna doilleir a’ chraicinn',
    '👰🏻‍♀' => 'boireannach le brat-gnùise: tòna soilleir a’ chraicinn',
    '👰🏼‍♀' => 'boireannach le brat-gnùise: tòna meadhanach soilleir a’ chraicinn',
    '👰🏽‍♀' => 'boireannach le brat-gnùise: tòna meadhanach a’ chraicinn',
    '👰🏾‍♀' => 'boireannach le brat-gnùise: tòna meadhanach doilleir a’ chraicinn',
    '👰🏿‍♀' => 'boireannach le brat-gnùise: tòna doilleir a’ chraicinn',
    '👩🏻‍🍼' => 'boireannach ag altram naoidhein: tòna soilleir a’ chraicinn',
    '👩🏼‍🍼' => 'boireannach ag altram naoidhein: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🍼' => 'boireannach ag altram naoidhein: tòna meadhanach a’ chraicinn',
    '👩🏾‍🍼' => 'boireannach ag altram naoidhein: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🍼' => 'boireannach ag altram naoidhein: tòna doilleir a’ chraicinn',
    '👨🏻‍🍼' => 'duine ag altram naoidhein: tòna soilleir a’ chraicinn',
    '👨🏼‍🍼' => 'duine ag altram naoidhein: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🍼' => 'duine ag altram naoidhein: tòna meadhanach a’ chraicinn',
    '👨🏾‍🍼' => 'duine ag altram naoidhein: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🍼' => 'duine ag altram naoidhein: tòna doilleir a’ chraicinn',
    '🧑🏻‍🍼' => 'neach ag altram naoidhein: tòna soilleir a’ chraicinn',
    '🧑🏼‍🍼' => 'neach ag altram naoidhein: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🍼' => 'neach ag altram naoidhein: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🍼' => 'neach ag altram naoidhein: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🍼' => 'neach ag altram naoidhein: tòna doilleir a’ chraicinn',
    '🧑🏻‍🎄' => 'Neach na Nollaig: tòna soilleir a’ chraicinn',
    '🧑🏼‍🎄' => 'Neach na Nollaig: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🎄' => 'Neach na Nollaig: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🎄' => 'Neach na Nollaig: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🎄' => 'Neach na Nollaig: tòna doilleir a’ chraicinn',
    '🦸🏻‍♂' => 'sàr-ghaisgeach fireann: tòna soilleir a’ chraicinn',
    '🦸🏼‍♂' => 'sàr-ghaisgeach fireann: tòna meadhanach soilleir a’ chraicinn',
    '🦸🏽‍♂' => 'sàr-ghaisgeach fireann: tòna meadhanach a’ chraicinn',
    '🦸🏾‍♂' => 'sàr-ghaisgeach fireann: tòna meadhanach doilleir a’ chraicinn',
    '🦸🏿‍♂' => 'sàr-ghaisgeach fireann: tòna doilleir a’ chraicinn',
    '🦸🏻‍♀' => 'sàr-ghaisgeach boireann: tòna soilleir a’ chraicinn',
    '🦸🏼‍♀' => 'sàr-ghaisgeach boireann: tòna meadhanach soilleir a’ chraicinn',
    '🦸🏽‍♀' => 'sàr-ghaisgeach boireann: tòna meadhanach a’ chraicinn',
    '🦸🏾‍♀' => 'sàr-ghaisgeach boireann: tòna meadhanach doilleir a’ chraicinn',
    '🦸🏿‍♀' => 'sàr-ghaisgeach boireann: tòna doilleir a’ chraicinn',
    '🦹🏻‍♂' => 'sàr-shlaightear fireann: tòna soilleir a’ chraicinn',
    '🦹🏼‍♂' => 'sàr-shlaightear fireann: tòna meadhanach soilleir a’ chraicinn',
    '🦹🏽‍♂' => 'sàr-shlaightear fireann: tòna meadhanach a’ chraicinn',
    '🦹🏾‍♂' => 'sàr-shlaightear fireann: tòna meadhanach doilleir a’ chraicinn',
    '🦹🏿‍♂' => 'sàr-shlaightear fireann: tòna doilleir a’ chraicinn',
    '🦹🏻‍♀' => 'sàr-shlaightear boireann: tòna soilleir a’ chraicinn',
    '🦹🏼‍♀' => 'sàr-shlaightear boireann: tòna meadhanach soilleir a’ chraicinn',
    '🦹🏽‍♀' => 'sàr-shlaightear boireann: tòna meadhanach a’ chraicinn',
    '🦹🏾‍♀' => 'sàr-shlaightear boireann: tòna meadhanach doilleir a’ chraicinn',
    '🦹🏿‍♀' => 'sàr-shlaightear boireann: tòna doilleir a’ chraicinn',
    '🧙🏻‍♂' => 'draoidh fireann: tòna soilleir a’ chraicinn',
    '🧙🏼‍♂' => 'draoidh fireann: tòna meadhanach soilleir a’ chraicinn',
    '🧙🏽‍♂' => 'draoidh fireann: tòna meadhanach a’ chraicinn',
    '🧙🏾‍♂' => 'draoidh fireann: tòna meadhanach doilleir a’ chraicinn',
    '🧙🏿‍♂' => 'draoidh fireann: tòna doilleir a’ chraicinn',
    '🧙🏻‍♀' => 'ban-draoidh: tòna soilleir a’ chraicinn',
    '🧙🏼‍♀' => 'ban-draoidh: tòna meadhanach soilleir a’ chraicinn',
    '🧙🏽‍♀' => 'ban-draoidh: tòna meadhanach a’ chraicinn',
    '🧙🏾‍♀' => 'ban-draoidh: tòna meadhanach doilleir a’ chraicinn',
    '🧙🏿‍♀' => 'ban-draoidh: tòna doilleir a’ chraicinn',
    '🧚🏻‍♂' => 'sìtheach: tòna soilleir a’ chraicinn',
    '🧚🏼‍♂' => 'sìtheach: tòna meadhanach soilleir a’ chraicinn',
    '🧚🏽‍♂' => 'sìtheach: tòna meadhanach a’ chraicinn',
    '🧚🏾‍♂' => 'sìtheach: tòna meadhanach doilleir a’ chraicinn',
    '🧚🏿‍♂' => 'sìtheach: tòna doilleir a’ chraicinn',
    '🧚🏻‍♀' => 'bean-shìthe: tòna soilleir a’ chraicinn',
    '🧚🏼‍♀' => 'bean-shìthe: tòna meadhanach soilleir a’ chraicinn',
    '🧚🏽‍♀' => 'bean-shìthe: tòna meadhanach a’ chraicinn',
    '🧚🏾‍♀' => 'bean-shìthe: tòna meadhanach doilleir a’ chraicinn',
    '🧚🏿‍♀' => 'bean-shìthe: tòna doilleir a’ chraicinn',
    '🧛🏻‍♂' => 'bhampair fireann: tòna soilleir a’ chraicinn',
    '🧛🏼‍♂' => 'bhampair fireann: tòna meadhanach soilleir a’ chraicinn',
    '🧛🏽‍♂' => 'bhampair fireann: tòna meadhanach a’ chraicinn',
    '🧛🏾‍♂' => 'bhampair fireann: tòna meadhanach doilleir a’ chraicinn',
    '🧛🏿‍♂' => 'bhampair fireann: tòna doilleir a’ chraicinn',
    '🧛🏻‍♀' => 'bhampair boireann: tòna soilleir a’ chraicinn',
    '🧛🏼‍♀' => 'bhampair boireann: tòna meadhanach soilleir a’ chraicinn',
    '🧛🏽‍♀' => 'bhampair boireann: tòna meadhanach a’ chraicinn',
    '🧛🏾‍♀' => 'bhampair boireann: tòna meadhanach doilleir a’ chraicinn',
    '🧛🏿‍♀' => 'bhampair boireann: tòna doilleir a’ chraicinn',
    '🧜🏻‍♂' => 'duine-mara: tòna soilleir a’ chraicinn',
    '🧜🏼‍♂' => 'duine-mara: tòna meadhanach soilleir a’ chraicinn',
    '🧜🏽‍♂' => 'duine-mara: tòna meadhanach a’ chraicinn',
    '🧜🏾‍♂' => 'duine-mara: tòna meadhanach doilleir a’ chraicinn',
    '🧜🏿‍♂' => 'duine-mara: tòna doilleir a’ chraicinn',
    '🧜🏻‍♀' => 'maighdeann-mhara: tòna soilleir a’ chraicinn',
    '🧜🏼‍♀' => 'maighdeann-mhara: tòna meadhanach soilleir a’ chraicinn',
    '🧜🏽‍♀' => 'maighdeann-mhara: tòna meadhanach a’ chraicinn',
    '🧜🏾‍♀' => 'maighdeann-mhara: tòna meadhanach doilleir a’ chraicinn',
    '🧜🏿‍♀' => 'maighdeann-mhara: tòna doilleir a’ chraicinn',
    '🧝🏻‍♂' => 'ailbhear fireann: tòna soilleir a’ chraicinn',
    '🧝🏼‍♂' => 'ailbhear fireann: tòna meadhanach soilleir a’ chraicinn',
    '🧝🏽‍♂' => 'ailbhear fireann: tòna meadhanach a’ chraicinn',
    '🧝🏾‍♂' => 'ailbhear fireann: tòna meadhanach doilleir a’ chraicinn',
    '🧝🏿‍♂' => 'ailbhear fireann: tòna doilleir a’ chraicinn',
    '🧝🏻‍♀' => 'bean-ailbhear: tòna soilleir a’ chraicinn',
    '🧝🏼‍♀' => 'bean-ailbhear: tòna meadhanach soilleir a’ chraicinn',
    '🧝🏽‍♀' => 'bean-ailbhear: tòna meadhanach a’ chraicinn',
    '🧝🏾‍♀' => 'bean-ailbhear: tòna meadhanach doilleir a’ chraicinn',
    '🧝🏿‍♀' => 'bean-ailbhear: tòna doilleir a’ chraicinn',
    '💆🏻‍♂' => 'duine a’ faighinn massage: tòna soilleir a’ chraicinn',
    '💆🏼‍♂' => 'duine a’ faighinn massage: tòna meadhanach soilleir a’ chraicinn',
    '💆🏽‍♂' => 'duine a’ faighinn massage: tòna meadhanach a’ chraicinn',
    '💆🏾‍♂' => 'duine a’ faighinn massage: tòna meadhanach doilleir a’ chraicinn',
    '💆🏿‍♂' => 'duine a’ faighinn massage: tòna doilleir a’ chraicinn',
    '💆🏻‍♀' => 'boireannach a’ faighinn massage: tòna soilleir a’ chraicinn',
    '💆🏼‍♀' => 'boireannach a’ faighinn massage: tòna meadhanach soilleir a’ chraicinn',
    '💆🏽‍♀' => 'boireannach a’ faighinn massage: tòna meadhanach a’ chraicinn',
    '💆🏾‍♀' => 'boireannach a’ faighinn massage: tòna meadhanach doilleir a’ chraicinn',
    '💆🏿‍♀' => 'boireannach a’ faighinn massage: tòna doilleir a’ chraicinn',
    '💇🏻‍♂' => 'duine a’ faighinn cliop: tòna soilleir a’ chraicinn',
    '💇🏼‍♂' => 'duine a’ faighinn cliop: tòna meadhanach soilleir a’ chraicinn',
    '💇🏽‍♂' => 'duine a’ faighinn cliop: tòna meadhanach a’ chraicinn',
    '💇🏾‍♂' => 'duine a’ faighinn cliop: tòna meadhanach doilleir a’ chraicinn',
    '💇🏿‍♂' => 'duine a’ faighinn cliop: tòna doilleir a’ chraicinn',
    '💇🏻‍♀' => 'boireannach a’ faighinn cliop: tòna soilleir a’ chraicinn',
    '💇🏼‍♀' => 'boireannach a’ faighinn cliop: tòna meadhanach soilleir a’ chraicinn',
    '💇🏽‍♀' => 'boireannach a’ faighinn cliop: tòna meadhanach a’ chraicinn',
    '💇🏾‍♀' => 'boireannach a’ faighinn cliop: tòna meadhanach doilleir a’ chraicinn',
    '💇🏿‍♀' => 'boireannach a’ faighinn cliop: tòna doilleir a’ chraicinn',
    '🚶🏻‍♂' => 'duine a’ coiseachd: tòna soilleir a’ chraicinn',
    '🚶🏼‍♂' => 'duine a’ coiseachd: tòna meadhanach soilleir a’ chraicinn',
    '🚶🏽‍♂' => 'duine a’ coiseachd: tòna meadhanach a’ chraicinn',
    '🚶🏾‍♂' => 'duine a’ coiseachd: tòna meadhanach doilleir a’ chraicinn',
    '🚶🏿‍♂' => 'duine a’ coiseachd: tòna doilleir a’ chraicinn',
    '🚶🏻‍♀' => 'boireannach a’ coiseachd: tòna soilleir a’ chraicinn',
    '🚶🏼‍♀' => 'boireannach a’ coiseachd: tòna meadhanach soilleir a’ chraicinn',
    '🚶🏽‍♀' => 'boireannach a’ coiseachd: tòna meadhanach a’ chraicinn',
    '🚶🏾‍♀' => 'boireannach a’ coiseachd: tòna meadhanach doilleir a’ chraicinn',
    '🚶🏿‍♀' => 'boireannach a’ coiseachd: tòna doilleir a’ chraicinn',
    '🚶🏻‍➡' => 'neach a’ coiseachd: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏼‍➡' => 'neach a’ coiseachd: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏽‍➡' => 'neach a’ coiseachd: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🚶🏾‍➡' => 'neach a’ coiseachd: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🚶🏿‍➡' => 'neach a’ coiseachd: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🧍🏻‍♂' => 'duine ’na sheasamh: tòna soilleir a’ chraicinn',
    '🧍🏼‍♂' => 'duine ’na sheasamh: tòna meadhanach soilleir a’ chraicinn',
    '🧍🏽‍♂' => 'duine ’na sheasamh: tòna meadhanach a’ chraicinn',
    '🧍🏾‍♂' => 'duine ’na sheasamh: tòna meadhanach doilleir a’ chraicinn',
    '🧍🏿‍♂' => 'duine ’na sheasamh: tòna doilleir a’ chraicinn',
    '🧍🏻‍♀' => 'boireannach ’na sheasamh: tòna soilleir a’ chraicinn',
    '🧍🏼‍♀' => 'boireannach ’na sheasamh: tòna meadhanach soilleir a’ chraicinn',
    '🧍🏽‍♀' => 'boireannach ’na sheasamh: tòna meadhanach a’ chraicinn',
    '🧍🏾‍♀' => 'boireannach ’na sheasamh: tòna meadhanach doilleir a’ chraicinn',
    '🧍🏿‍♀' => 'boireannach ’na sheasamh: tòna doilleir a’ chraicinn',
    '🧎🏻‍♂' => 'duine air a ghlùinean: tòna soilleir a’ chraicinn',
    '🧎🏼‍♂' => 'duine air a ghlùinean: tòna meadhanach soilleir a’ chraicinn',
    '🧎🏽‍♂' => 'duine air a ghlùinean: tòna meadhanach a’ chraicinn',
    '🧎🏾‍♂' => 'duine air a ghlùinean: tòna meadhanach doilleir a’ chraicinn',
    '🧎🏿‍♂' => 'duine air a ghlùinean: tòna doilleir a’ chraicinn',
    '🧎🏻‍♀' => 'boireanach air a ghlùinean: tòna soilleir a’ chraicinn',
    '🧎🏼‍♀' => 'boireanach air a ghlùinean: tòna meadhanach soilleir a’ chraicinn',
    '🧎🏽‍♀' => 'boireanach air a ghlùinean: tòna meadhanach a’ chraicinn',
    '🧎🏾‍♀' => 'boireanach air a ghlùinean: tòna meadhanach doilleir a’ chraicinn',
    '🧎🏿‍♀' => 'boireanach air a ghlùinean: tòna doilleir a’ chraicinn',
    '🧎🏻‍➡' => 'neach air a ghlùinean: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏼‍➡' => 'neach air a ghlùinean: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏽‍➡' => 'neach air a ghlùinean: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🧎🏾‍➡' => 'neach air a ghlùinean: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🧎🏿‍➡' => 'neach air a ghlùinean: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '🧑🏻‍🦯' => 'neach dall le batachan: tòna soilleir a’ chraicinn',
    '🧑🏼‍🦯' => 'neach dall le batachan: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🦯' => 'neach dall le batachan: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🦯' => 'neach dall le batachan: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🦯' => 'neach dall le batachan: tòna doilleir a’ chraicinn',
    '👨🏻‍🦯' => 'duine dall le batachan: tòna soilleir a’ chraicinn',
    '👨🏼‍🦯' => 'duine dall le batachan: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🦯' => 'duine dall le batachan: tòna meadhanach a’ chraicinn',
    '👨🏾‍🦯' => 'duine dall le batachan: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🦯' => 'duine dall le batachan: tòna doilleir a’ chraicinn',
    '👩🏻‍🦯' => 'boireannach dall le batachan: tòna soilleir a’ chraicinn',
    '👩🏼‍🦯' => 'boireannach dall le batachan: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🦯' => 'boireannach dall le batachan: tòna meadhanach a’ chraicinn',
    '👩🏾‍🦯' => 'boireannach dall le batachan: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🦯' => 'boireannach dall le batachan: tòna doilleir a’ chraicinn',
    '🧑🏻‍🦼' => 'neach ann an cathair-chuibhle le einnsein: tòna soilleir a’ chraicinn',
    '🧑🏼‍🦼' => 'neach ann an cathair-chuibhle le einnsein: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🦼' => 'neach ann an cathair-chuibhle le einnsein: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🦼' => 'neach ann an cathair-chuibhle le einnsein: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🦼' => 'neach ann an cathair-chuibhle le einnsein: tòna doilleir a’ chraicinn',
    '👨🏻‍🦼' => 'duine ann an cathair-chuibhle le einnsein: tòna soilleir a’ chraicinn',
    '👨🏼‍🦼' => 'duine ann an cathair-chuibhle le einnsein: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🦼' => 'duine ann an cathair-chuibhle le einnsein: tòna meadhanach a’ chraicinn',
    '👨🏾‍🦼' => 'duine ann an cathair-chuibhle le einnsein: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🦼' => 'duine ann an cathair-chuibhle le einnsein: tòna doilleir a’ chraicinn',
    '👩🏻‍🦼' => 'tè ann an cathair-chuibhle le einnsein: tòna soilleir a’ chraicinn',
    '👩🏼‍🦼' => 'tè ann an cathair-chuibhle le einnsein: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🦼' => 'tè ann an cathair-chuibhle le einnsein: tòna meadhanach a’ chraicinn',
    '👩🏾‍🦼' => 'tè ann an cathair-chuibhle le einnsein: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🦼' => 'tè ann an cathair-chuibhle le einnsein: tòna doilleir a’ chraicinn',
    '🧑🏻‍🦽' => 'neach ann an cathair-chuibhle a làimh: tòna soilleir a’ chraicinn',
    '🧑🏼‍🦽' => 'neach ann an cathair-chuibhle a làimh: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽‍🦽' => 'neach ann an cathair-chuibhle a làimh: tòna meadhanach a’ chraicinn',
    '🧑🏾‍🦽' => 'neach ann an cathair-chuibhle a làimh: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿‍🦽' => 'neach ann an cathair-chuibhle a làimh: tòna doilleir a’ chraicinn',
    '👨🏻‍🦽' => 'duine ann an cathair-chuibhle a làimh: tòna soilleir a’ chraicinn',
    '👨🏼‍🦽' => 'duine ann an cathair-chuibhle a làimh: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽‍🦽' => 'duine ann an cathair-chuibhle a làimh: tòna meadhanach a’ chraicinn',
    '👨🏾‍🦽' => 'duine ann an cathair-chuibhle a làimh: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿‍🦽' => 'duine ann an cathair-chuibhle a làimh: tòna doilleir a’ chraicinn',
    '👩🏻‍🦽' => 'tè ann an cathair-chuibhle a làimh: tòna soilleir a’ chraicinn',
    '👩🏼‍🦽' => 'tè ann an cathair-chuibhle a làimh: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽‍🦽' => 'tè ann an cathair-chuibhle a làimh: tòna meadhanach a’ chraicinn',
    '👩🏾‍🦽' => 'tè ann an cathair-chuibhle a làimh: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿‍🦽' => 'tè ann an cathair-chuibhle a làimh: tòna doilleir a’ chraicinn',
    '🏃🏻‍♂' => 'duine a’ ruith: tòna soilleir a’ chraicinn',
    '🏃🏼‍♂' => 'duine a’ ruith: tòna meadhanach soilleir a’ chraicinn',
    '🏃🏽‍♂' => 'duine a’ ruith: tòna meadhanach a’ chraicinn',
    '🏃🏾‍♂' => 'duine a’ ruith: tòna meadhanach doilleir a’ chraicinn',
    '🏃🏿‍♂' => 'duine a’ ruith: tòna doilleir a’ chraicinn',
    '🏃🏻‍♀' => 'boireannach a’ ruith: tòna soilleir a’ chraicinn',
    '🏃🏼‍♀' => 'boireannach a’ ruith: tòna meadhanach soilleir a’ chraicinn',
    '🏃🏽‍♀' => 'boireannach a’ ruith: tòna meadhanach a’ chraicinn',
    '🏃🏾‍♀' => 'boireannach a’ ruith: tòna meadhanach doilleir a’ chraicinn',
    '🏃🏿‍♀' => 'boireannach a’ ruith: tòna doilleir a’ chraicinn',
    '🏃🏻‍➡' => 'neach a’ ruith: tòna soilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏼‍➡' => 'neach a’ ruith: tòna meadhanach soilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏽‍➡' => 'neach a’ ruith: tòna meadhanach a’ chraicinn ’s ris an taobh deas',
    '🏃🏾‍➡' => 'neach a’ ruith: tòna meadhanach doilleir a’ chraicinn ’s ris an taobh deas',
    '🏃🏿‍➡' => 'neach a’ ruith: tòna doilleir a’ chraicinn ’s ris an taobh deas',
    '👯🏻‍♀' => 'boireannaich le cluasan rabaid: tòna soilleir a’ chraicinn',
    '👯🏼‍♀' => 'boireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn',
    '👯🏽‍♀' => 'boireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn',
    '👯🏾‍♀' => 'boireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn',
    '👯🏿‍♀' => 'boireannaich le cluasan rabaid: tòna doilleir a’ chraicinn',
    '👯🏻‍♂' => 'fireannaich le cluasan rabaid: tòna soilleir a’ chraicinn',
    '👯🏼‍♂' => 'fireannaich le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn',
    '👯🏽‍♂' => 'fireannaich le cluasan rabaid: tòna meadhanach a’ chraicinn',
    '👯🏾‍♂' => 'fireannaich le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn',
    '👯🏿‍♂' => 'fireannaich le cluasan rabaid: tòna doilleir a’ chraicinn',
    '🧖🏻‍♂' => 'duine ann an sauna: tòna soilleir a’ chraicinn',
    '🧖🏼‍♂' => 'duine ann an sauna: tòna meadhanach soilleir a’ chraicinn',
    '🧖🏽‍♂' => 'duine ann an sauna: tòna meadhanach a’ chraicinn',
    '🧖🏾‍♂' => 'duine ann an sauna: tòna meadhanach doilleir a’ chraicinn',
    '🧖🏿‍♂' => 'duine ann an sauna: tòna doilleir a’ chraicinn',
    '🧖🏻‍♀' => 'boireannach ann an sauna: tòna soilleir a’ chraicinn',
    '🧖🏼‍♀' => 'boireannach ann an sauna: tòna meadhanach soilleir a’ chraicinn',
    '🧖🏽‍♀' => 'boireannach ann an sauna: tòna meadhanach a’ chraicinn',
    '🧖🏾‍♀' => 'boireannach ann an sauna: tòna meadhanach doilleir a’ chraicinn',
    '🧖🏿‍♀' => 'boireannach ann an sauna: tòna doilleir a’ chraicinn',
    '🧗🏻‍♂' => 'duine a’ sreap: tòna soilleir a’ chraicinn',
    '🧗🏼‍♂' => 'duine a’ sreap: tòna meadhanach soilleir a’ chraicinn',
    '🧗🏽‍♂' => 'duine a’ sreap: tòna meadhanach a’ chraicinn',
    '🧗🏾‍♂' => 'duine a’ sreap: tòna meadhanach doilleir a’ chraicinn',
    '🧗🏿‍♂' => 'duine a’ sreap: tòna doilleir a’ chraicinn',
    '🧗🏻‍♀' => 'boireannach a’ sreap: tòna soilleir a’ chraicinn',
    '🧗🏼‍♀' => 'boireannach a’ sreap: tòna meadhanach soilleir a’ chraicinn',
    '🧗🏽‍♀' => 'boireannach a’ sreap: tòna meadhanach a’ chraicinn',
    '🧗🏾‍♀' => 'boireannach a’ sreap: tòna meadhanach doilleir a’ chraicinn',
    '🧗🏿‍♀' => 'boireannach a’ sreap: tòna doilleir a’ chraicinn',
    '🏌🏻‍♂' => 'duine ri goilf: tòna soilleir a’ chraicinn',
    '🏌🏼‍♂' => 'duine ri goilf: tòna meadhanach soilleir a’ chraicinn',
    '🏌🏽‍♂' => 'duine ri goilf: tòna meadhanach a’ chraicinn',
    '🏌🏾‍♂' => 'duine ri goilf: tòna meadhanach doilleir a’ chraicinn',
    '🏌🏿‍♂' => 'duine ri goilf: tòna doilleir a’ chraicinn',
    '🏌🏻‍♀' => 'boireannach ri goilf: tòna soilleir a’ chraicinn',
    '🏌🏼‍♀' => 'boireannach ri goilf: tòna meadhanach soilleir a’ chraicinn',
    '🏌🏽‍♀' => 'boireannach ri goilf: tòna meadhanach a’ chraicinn',
    '🏌🏾‍♀' => 'boireannach ri goilf: tòna meadhanach doilleir a’ chraicinn',
    '🏌🏿‍♀' => 'boireannach ri goilf: tòna doilleir a’ chraicinn',
    '🏄🏻‍♂' => 'duine a’ surfadh: tòna soilleir a’ chraicinn',
    '🏄🏼‍♂' => 'duine a’ surfadh: tòna meadhanach soilleir a’ chraicinn',
    '🏄🏽‍♂' => 'duine a’ surfadh: tòna meadhanach a’ chraicinn',
    '🏄🏾‍♂' => 'duine a’ surfadh: tòna meadhanach doilleir a’ chraicinn',
    '🏄🏿‍♂' => 'duine a’ surfadh: tòna doilleir a’ chraicinn',
    '🏄🏻‍♀' => 'boireannach a’ surfadh: tòna soilleir a’ chraicinn',
    '🏄🏼‍♀' => 'boireannach a’ surfadh: tòna meadhanach soilleir a’ chraicinn',
    '🏄🏽‍♀' => 'boireannach a’ surfadh: tòna meadhanach a’ chraicinn',
    '🏄🏾‍♀' => 'boireannach a’ surfadh: tòna meadhanach doilleir a’ chraicinn',
    '🏄🏿‍♀' => 'boireannach a’ surfadh: tòna doilleir a’ chraicinn',
    '🚣🏻‍♂' => 'duine ag iomradh bàta: tòna soilleir a’ chraicinn',
    '🚣🏼‍♂' => 'duine ag iomradh bàta: tòna meadhanach soilleir a’ chraicinn',
    '🚣🏽‍♂' => 'duine ag iomradh bàta: tòna meadhanach a’ chraicinn',
    '🚣🏾‍♂' => 'duine ag iomradh bàta: tòna meadhanach doilleir a’ chraicinn',
    '🚣🏿‍♂' => 'duine ag iomradh bàta: tòna doilleir a’ chraicinn',
    '🚣🏻‍♀' => 'boireannach ag iomradh bàta: tòna soilleir a’ chraicinn',
    '🚣🏼‍♀' => 'boireannach ag iomradh bàta: tòna meadhanach soilleir a’ chraicinn',
    '🚣🏽‍♀' => 'boireannach ag iomradh bàta: tòna meadhanach a’ chraicinn',
    '🚣🏾‍♀' => 'boireannach ag iomradh bàta: tòna meadhanach doilleir a’ chraicinn',
    '🚣🏿‍♀' => 'boireannach ag iomradh bàta: tòna doilleir a’ chraicinn',
    '🏊🏻‍♂' => 'duine a’ snàmh: tòna soilleir a’ chraicinn',
    '🏊🏼‍♂' => 'duine a’ snàmh: tòna meadhanach soilleir a’ chraicinn',
    '🏊🏽‍♂' => 'duine a’ snàmh: tòna meadhanach a’ chraicinn',
    '🏊🏾‍♂' => 'duine a’ snàmh: tòna meadhanach doilleir a’ chraicinn',
    '🏊🏿‍♂' => 'duine a’ snàmh: tòna doilleir a’ chraicinn',
    '🏊🏻‍♀' => 'boireannach a’ snàmh: tòna soilleir a’ chraicinn',
    '🏊🏼‍♀' => 'boireannach a’ snàmh: tòna meadhanach soilleir a’ chraicinn',
    '🏊🏽‍♀' => 'boireannach a’ snàmh: tòna meadhanach a’ chraicinn',
    '🏊🏾‍♀' => 'boireannach a’ snàmh: tòna meadhanach doilleir a’ chraicinn',
    '🏊🏿‍♀' => 'boireannach a’ snàmh: tòna doilleir a’ chraicinn',
    '⛹🏻‍♂' => 'duine a’ bocadh bàla: tòna soilleir a’ chraicinn',
    '⛹🏼‍♂' => 'duine a’ bocadh bàla: tòna meadhanach soilleir a’ chraicinn',
    '⛹🏽‍♂' => 'duine a’ bocadh bàla: tòna meadhanach a’ chraicinn',
    '⛹🏾‍♂' => 'duine a’ bocadh bàla: tòna meadhanach doilleir a’ chraicinn',
    '⛹🏿‍♂' => 'duine a’ bocadh bàla: tòna doilleir a’ chraicinn',
    '⛹🏻‍♀' => 'boireannach a’ bocadh bàla: tòna soilleir a’ chraicinn',
    '⛹🏼‍♀' => 'boireannach a’ bocadh bàla: tòna meadhanach soilleir a’ chraicinn',
    '⛹🏽‍♀' => 'boireannach a’ bocadh bàla: tòna meadhanach a’ chraicinn',
    '⛹🏾‍♀' => 'boireannach a’ bocadh bàla: tòna meadhanach doilleir a’ chraicinn',
    '⛹🏿‍♀' => 'boireannach a’ bocadh bàla: tòna doilleir a’ chraicinn',
    '🏋🏻‍♂' => 'duine a’ togail chuideaman: tòna soilleir a’ chraicinn',
    '🏋🏼‍♂' => 'duine a’ togail chuideaman: tòna meadhanach soilleir a’ chraicinn',
    '🏋🏽‍♂' => 'duine a’ togail chuideaman: tòna meadhanach a’ chraicinn',
    '🏋🏾‍♂' => 'duine a’ togail chuideaman: tòna meadhanach doilleir a’ chraicinn',
    '🏋🏿‍♂' => 'duine a’ togail chuideaman: tòna doilleir a’ chraicinn',
    '🏋🏻‍♀' => 'boireannach a’ togail chuideaman: tòna soilleir a’ chraicinn',
    '🏋🏼‍♀' => 'boireannach a’ togail chuideaman: tòna meadhanach soilleir a’ chraicinn',
    '🏋🏽‍♀' => 'boireannach a’ togail chuideaman: tòna meadhanach a’ chraicinn',
    '🏋🏾‍♀' => 'boireannach a’ togail chuideaman: tòna meadhanach doilleir a’ chraicinn',
    '🏋🏿‍♀' => 'boireannach a’ togail chuideaman: tòna doilleir a’ chraicinn',
    '🚴🏻‍♂' => 'duine ri rothaireachd: tòna soilleir a’ chraicinn',
    '🚴🏼‍♂' => 'duine ri rothaireachd: tòna meadhanach soilleir a’ chraicinn',
    '🚴🏽‍♂' => 'duine ri rothaireachd: tòna meadhanach a’ chraicinn',
    '🚴🏾‍♂' => 'duine ri rothaireachd: tòna meadhanach doilleir a’ chraicinn',
    '🚴🏿‍♂' => 'duine ri rothaireachd: tòna doilleir a’ chraicinn',
    '🚴🏻‍♀' => 'boireannach ri rothaireachd: tòna soilleir a’ chraicinn',
    '🚴🏼‍♀' => 'boireannach ri rothaireachd: tòna meadhanach soilleir a’ chraicinn',
    '🚴🏽‍♀' => 'boireannach ri rothaireachd: tòna meadhanach a’ chraicinn',
    '🚴🏾‍♀' => 'boireannach ri rothaireachd: tòna meadhanach doilleir a’ chraicinn',
    '🚴🏿‍♀' => 'boireannach ri rothaireachd: tòna doilleir a’ chraicinn',
    '🚵🏻‍♂' => 'duine ri rothaireachd bheanntan: tòna soilleir a’ chraicinn',
    '🚵🏼‍♂' => 'duine ri rothaireachd bheanntan: tòna meadhanach soilleir a’ chraicinn',
    '🚵🏽‍♂' => 'duine ri rothaireachd bheanntan: tòna meadhanach a’ chraicinn',
    '🚵🏾‍♂' => 'duine ri rothaireachd bheanntan: tòna meadhanach doilleir a’ chraicinn',
    '🚵🏿‍♂' => 'duine ri rothaireachd bheanntan: tòna doilleir a’ chraicinn',
    '🚵🏻‍♀' => 'boireannach ri rothaireachd bheanntan: tòna soilleir a’ chraicinn',
    '🚵🏼‍♀' => 'boireannach ri rothaireachd bheanntan: tòna meadhanach soilleir a’ chraicinn',
    '🚵🏽‍♀' => 'boireannach ri rothaireachd bheanntan: tòna meadhanach a’ chraicinn',
    '🚵🏾‍♀' => 'boireannach ri rothaireachd bheanntan: tòna meadhanach doilleir a’ chraicinn',
    '🚵🏿‍♀' => 'boireannach ri rothaireachd bheanntan: tòna doilleir a’ chraicinn',
    '🤸🏻‍♂' => 'duine a’ cur nan rothan: tòna soilleir a’ chraicinn',
    '🤸🏼‍♂' => 'duine a’ cur nan rothan: tòna meadhanach soilleir a’ chraicinn',
    '🤸🏽‍♂' => 'duine a’ cur nan rothan: tòna meadhanach a’ chraicinn',
    '🤸🏾‍♂' => 'duine a’ cur nan rothan: tòna meadhanach doilleir a’ chraicinn',
    '🤸🏿‍♂' => 'duine a’ cur nan rothan: tòna doilleir a’ chraicinn',
    '🤸🏻‍♀' => 'boireannach a’ cur nan rothan: tòna soilleir a’ chraicinn',
    '🤸🏼‍♀' => 'boireannach a’ cur nan rothan: tòna meadhanach soilleir a’ chraicinn',
    '🤸🏽‍♀' => 'boireannach a’ cur nan rothan: tòna meadhanach a’ chraicinn',
    '🤸🏾‍♀' => 'boireannach a’ cur nan rothan: tòna meadhanach doilleir a’ chraicinn',
    '🤸🏿‍♀' => 'boireannach a’ cur nan rothan: tòna doilleir a’ chraicinn',
    '🤼🏻‍♀' => 'boireannaich ri gleacadh: tòna soilleir a’ chraicinn',
    '🤼🏼‍♀' => 'boireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn',
    '🤼🏽‍♀' => 'boireannaich ri gleacadh: tòna meadhanach a’ chraicinn',
    '🤼🏾‍♀' => 'boireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn',
    '🤼🏿‍♀' => 'boireannaich ri gleacadh: tòna doilleir a’ chraicinn',
    '🤼🏻‍♂' => 'fireannaich ri gleacadh: tòna soilleir a’ chraicinn',
    '🤼🏼‍♂' => 'fireannaich ri gleacadh: tòna meadhanach soilleir a’ chraicinn',
    '🤼🏽‍♂' => 'fireannaich ri gleacadh: tòna meadhanach a’ chraicinn',
    '🤼🏾‍♂' => 'fireannaich ri gleacadh: tòna meadhanach doilleir a’ chraicinn',
    '🤼🏿‍♂' => 'fireannaich ri gleacadh: tòna doilleir a’ chraicinn',
    '🤽🏻‍♂' => 'duine a’ cluich polo-uisge: tòna soilleir a’ chraicinn',
    '🤽🏼‍♂' => 'duine a’ cluich polo-uisge: tòna meadhanach soilleir a’ chraicinn',
    '🤽🏽‍♂' => 'duine a’ cluich polo-uisge: tòna meadhanach a’ chraicinn',
    '🤽🏾‍♂' => 'duine a’ cluich polo-uisge: tòna meadhanach doilleir a’ chraicinn',
    '🤽🏿‍♂' => 'duine a’ cluich polo-uisge: tòna doilleir a’ chraicinn',
    '🤽🏻‍♀' => 'boireannach a’ cluich polo-uisge: tòna soilleir a’ chraicinn',
    '🤽🏼‍♀' => 'boireannach a’ cluich polo-uisge: tòna meadhanach soilleir a’ chraicinn',
    '🤽🏽‍♀' => 'boireannach a’ cluich polo-uisge: tòna meadhanach a’ chraicinn',
    '🤽🏾‍♀' => 'boireannach a’ cluich polo-uisge: tòna meadhanach doilleir a’ chraicinn',
    '🤽🏿‍♀' => 'boireannach a’ cluich polo-uisge: tòna doilleir a’ chraicinn',
    '🤾🏻‍♂' => 'duine a’ cluich ball-làimhe: tòna soilleir a’ chraicinn',
    '🤾🏼‍♂' => 'duine a’ cluich ball-làimhe: tòna meadhanach soilleir a’ chraicinn',
    '🤾🏽‍♂' => 'duine a’ cluich ball-làimhe: tòna meadhanach a’ chraicinn',
    '🤾🏾‍♂' => 'duine a’ cluich ball-làimhe: tòna meadhanach doilleir a’ chraicinn',
    '🤾🏿‍♂' => 'duine a’ cluich ball-làimhe: tòna doilleir a’ chraicinn',
    '🤾🏻‍♀' => 'boireannach a’ cluich ball-làimhe: tòna soilleir a’ chraicinn',
    '🤾🏼‍♀' => 'boireannach a’ cluich ball-làimhe: tòna meadhanach soilleir a’ chraicinn',
    '🤾🏽‍♀' => 'boireannach a’ cluich ball-làimhe: tòna meadhanach a’ chraicinn',
    '🤾🏾‍♀' => 'boireannach a’ cluich ball-làimhe: tòna meadhanach doilleir a’ chraicinn',
    '🤾🏿‍♀' => 'boireannach a’ cluich ball-làimhe: tòna doilleir a’ chraicinn',
    '🤹🏻‍♂' => 'duine ri caisreabhachd: tòna soilleir a’ chraicinn',
    '🤹🏼‍♂' => 'duine ri caisreabhachd: tòna meadhanach soilleir a’ chraicinn',
    '🤹🏽‍♂' => 'duine ri caisreabhachd: tòna meadhanach a’ chraicinn',
    '🤹🏾‍♂' => 'duine ri caisreabhachd: tòna meadhanach doilleir a’ chraicinn',
    '🤹🏿‍♂' => 'duine ri caisreabhachd: tòna doilleir a’ chraicinn',
    '🤹🏻‍♀' => 'boireannach ri caisreabhachd: tòna soilleir a’ chraicinn',
    '🤹🏼‍♀' => 'boireannach ri caisreabhachd: tòna meadhanach soilleir a’ chraicinn',
    '🤹🏽‍♀' => 'boireannach ri caisreabhachd: tòna meadhanach a’ chraicinn',
    '🤹🏾‍♀' => 'boireannach ri caisreabhachd: tòna meadhanach doilleir a’ chraicinn',
    '🤹🏿‍♀' => 'boireannach ri caisreabhachd: tòna doilleir a’ chraicinn',
    '🧘🏻‍♂' => 'duine na shuidhe lòtais: tòna soilleir a’ chraicinn',
    '🧘🏼‍♂' => 'duine na shuidhe lòtais: tòna meadhanach soilleir a’ chraicinn',
    '🧘🏽‍♂' => 'duine na shuidhe lòtais: tòna meadhanach a’ chraicinn',
    '🧘🏾‍♂' => 'duine na shuidhe lòtais: tòna meadhanach doilleir a’ chraicinn',
    '🧘🏿‍♂' => 'duine na shuidhe lòtais: tòna doilleir a’ chraicinn',
    '🧘🏻‍♀' => 'boireannach na shuidhe lòtais: tòna soilleir a’ chraicinn',
    '🧘🏼‍♀' => 'boireannach na shuidhe lòtais: tòna meadhanach soilleir a’ chraicinn',
    '🧘🏽‍♀' => 'boireannach na shuidhe lòtais: tòna meadhanach a’ chraicinn',
    '🧘🏾‍♀' => 'boireannach na shuidhe lòtais: tòna meadhanach doilleir a’ chraicinn',
    '🧘🏿‍♀' => 'boireannach na shuidhe lòtais: tòna doilleir a’ chraicinn',
    '🧑‍🦰' => 'inbheach: falt ruadh',
    '🧑‍🦱' => 'inbheach: falt dualach',
    '🧑‍🦳' => 'inbheach: falt bàn',
    '🧑‍🦲' => 'inbheach: maoilean',
    '👨‍🦰' => 'duine: falt ruadh',
    '👨‍🦱' => 'duine: falt dualach',
    '👨‍🦳' => 'duine: falt bàn',
    '👨‍🦲' => 'duine: maoilean',
    '👩‍🦰' => 'boireannach: falt ruadh',
    '👩‍🦱' => 'boireannach: falt dualach',
    '👩‍🦳' => 'boireannach: falt bàn',
    '👩‍🦲' => 'boireannach: maoilean',
    '🚶‍➡' => 'neach a’ coiseachd: ris an taobh deas',
    '🧎‍➡' => 'neach air a ghlùinean: ris an taobh deas',
    '🏃‍➡' => 'neach a’ ruith: ris an taobh deas',
    '👨‍👦' => 'teaghlach: duine ’s gille',
    '👨‍👧' => 'teaghlach: duine ’s caileag',
    '👩‍👦' => 'teaghlach: boireannach ’s gille',
    '👩‍👧' => 'teaghlach: boireannach ’s caileag',
    '😶‍🌫' => 'aodan sna neòil',
    '😮‍💨' => 'aodan a’ leigeil anail',
    '🙂‍↕' => 'cromadh-cinn',
    '😵‍💫' => 'aodann le sùilean snìomhanach',
    '❤‍🔥' => 'cridhe air theine',
    '❤‍🩹' => 'slànachadh cridhe',
    '👁‍🗨' => 'sùil ann am builgean còmhraidh',
    '🧔‍♂' => 'fireannach: feusag',
    '👱‍♂' => 'duine le falt bàn',
    '🧔‍♀' => 'boireannach: feusag',
    '👱‍♀' => 'boireannach le falt bàn',
    '🙍‍♂' => 'duine le drèin',
    '🙍‍♀' => 'boireannach le drèin',
    '🙎‍♂' => 'duine le bus air',
    '🙎‍♀' => 'boireannach le bus oirre',
    '🙅‍♂' => 'duine a’ diùltadh rud',
    '🙅‍♀' => 'boireannach a’ diùltadh rud',
    '🙆‍♂' => 'duine ag aontachadh ri rud',
    '🙆‍♀' => 'boireannach ag aontachadh ri rud',
    '💁‍♂' => 'duine le làmh sìnte, bas air uachdar',
    '💁‍♀' => 'tè le làmh sìnte a-mach, bas air uachdar',
    '🙋‍♂' => 'duine a’ togail a làmh',
    '🙋‍♀' => 'boireannach a’ togail a làmh',
    '🧏‍♂' => 'duine bodhar',
    '🧏‍♀' => 'boireannach bodhar',
    '🙇‍♂' => 'duine a’ cromadh',
    '🙇‍♀' => 'boireannach a’ cromadh',
    '🤦‍♂' => 'duine le aodann na làmhan',
    '🤦‍♀' => 'boireannach le aodann na làmhan',
    '🤷‍♂' => 'duine a’ druiteadh',
    '🤷‍♀' => 'boireannach a’ druiteadh',
    '🧑‍⚕' => 'obraiche-slàinte',
    '👨‍⚕' => 'obraiche-slàinte fireann',
    '👩‍⚕' => 'obraiche-slàinte boireann',
    '🧑‍🎓' => 'oileanach',
    '👨‍🎓' => 'oileanach fireann',
    '👩‍🎓' => 'ban-oileanach',
    '🧑‍🏫' => 'tidsear',
    '👨‍🏫' => 'oide fireann',
    '👩‍🏫' => 'ban-oide',
    '🧑‍⚖' => 'britheamh',
    '👨‍⚖' => 'britheamh fireann',
    '👩‍⚖' => 'britheamh boireann',
    '🧑‍🌾' => 'tuathanach',
    '👨‍🌾' => 'tuathanaiche fireann',
    '👩‍🌾' => 'tuathanaiche boireann',
    '🧑‍🍳' => 'còcaire',
    '👨‍🍳' => 'còcaire fireann',
    '👩‍🍳' => 'ban-chòcaire',
    '🧑‍🔧' => 'meacanaig',
    '👨‍🔧' => 'meacanaig fireann',
    '👩‍🔧' => 'meacanaig boireann',
    '🧑‍🏭' => 'neach-obrach factaraidh',
    '👨‍🏭' => 'obraiche-factaraidh fireann',
    '👩‍🏭' => 'obraiche-factaraidh boireann',
    '🧑‍💼' => 'neach-obrach oifise',
    '👨‍💼' => 'obraiche-oifis fireann',
    '👩‍💼' => 'obraiche-oifis boireann',
    '🧑‍🔬' => 'neach-saidheans',
    '👨‍🔬' => 'neach-saidheans fireann',
    '👩‍🔬' => 'neach-saidheans boireann',
    '🧑‍💻' => 'teicneolaiche',
    '👨‍💻' => 'teicneolaiche fireann',
    '👩‍💻' => 'teicneolaiche boireann',
    '🧑‍🎤' => 'seinneadair',
    '👨‍🎤' => 'seinneadair fireann',
    '👩‍🎤' => 'ban-seinneadair',
    '🧑‍🎨' => 'neach-ealain',
    '👨‍🎨' => 'fear-ealain',
    '👩‍🎨' => 'neach-ealain boireann',
    '🧑‍✈' => 'paidhleat',
    '👨‍✈' => 'paidhleat fireann',
    '👩‍✈' => 'paidhleat boireann',
    '🧑‍🚀' => 'speuradair',
    '👨‍🚀' => 'speuradair fireann',
    '👩‍🚀' => 'ban-speuradair',
    '🧑‍🚒' => 'smàladair',
    '👨‍🚒' => 'fear-smàlaidh',
    '👩‍🚒' => 'neach-smàlaidh boireann',
    '👮‍♂' => 'oifigear poileis fireann',
    '👮‍♀' => 'oifigear poileis boireann',
    '🕵‍♂' => 'lorgaire fireann',
    '🕵‍♀' => 'ban-lorgaire',
    '💂‍♂' => 'geàrd fireann',
    '💂‍♀' => 'bana-gheàrd',
    '👷‍♂' => 'fear-togail',
    '👷‍♀' => 'neach-togail boireann',
    '👳‍♂' => 'duine le turban air',
    '👳‍♀' => 'boireannach le turban oirre',
    '🤵‍♂' => 'duine ann an tuxedo',
    '🤵‍♀' => 'boireannach ann an tuxedo',
    '👰‍♂' => 'duine le brat-gnùise',
    '👰‍♀' => 'boireannach le brat-gnùise',
    '👩‍🍼' => 'boireannach ag altram naoidhein',
    '👨‍🍼' => 'duine ag altram naoidhein',
    '🧑‍🍼' => 'neach ag altram naoidhein',
    '🧑‍🎄' => 'Neach na Nollaig',
    '🦸‍♂' => 'sàr-ghaisgeach fireann',
    '🦸‍♀' => 'sàr-ghaisgeach boireann',
    '🦹‍♂' => 'sàr-shlaightear fireann',
    '🦹‍♀' => 'sàr-shlaightear boireann',
    '🧙‍♂' => 'draoidh fireann',
    '🧙‍♀' => 'ban-draoidh',
    '🧚‍♂' => 'sìtheach',
    '🧚‍♀' => 'bean-shìthe',
    '🧛‍♂' => 'bhampair fireann',
    '🧛‍♀' => 'bhampair boireann',
    '🧜‍♂' => 'duine-mara',
    '🧜‍♀' => 'maighdeann-mhara',
    '🧝‍♂' => 'ailbhear fireann',
    '🧝‍♀' => 'bean-ailbhear',
    '🧞‍♂' => 'taibhse fhireann',
    '🧞‍♀' => 'taibhse bhoireann',
    '🧟‍♂' => 'zombie fireann',
    '🧟‍♀' => 'zombie boireann',
    '💆‍♂' => 'duine a’ faighinn massage',
    '💆‍♀' => 'boireannach a’ faighinn massage',
    '💇‍♂' => 'duine a’ faighinn cliop',
    '💇‍♀' => 'boireannach a’ faighinn cliop',
    '🚶‍♂' => 'duine a’ coiseachd',
    '🚶‍♀' => 'boireannach a’ coiseachd',
    '🧍‍♂' => 'duine ’na sheasamh',
    '🧍‍♀' => 'boireannach ’na sheasamh',
    '🧎‍♂' => 'duine air a ghlùinean',
    '🧎‍♀' => 'boireanach air a ghlùinean',
    '🧑‍🦯' => 'neach dall le batachan',
    '👨‍🦯' => 'duine dall le batachan',
    '👩‍🦯' => 'boireannach dall le batachan',
    '🧑‍🦼' => 'neach ann an cathair-chuibhle le einnsein',
    '👨‍🦼' => 'duine ann an cathair-chuibhle le einnsein',
    '👩‍🦼' => 'tè ann an cathair-chuibhle le einnsein',
    '🧑‍🦽' => 'neach ann an cathair-chuibhle a làimh',
    '👨‍🦽' => 'duine ann an cathair-chuibhle a làimh',
    '👩‍🦽' => 'tè ann an cathair-chuibhle a làimh',
    '🏃‍♂' => 'duine a’ ruith',
    '🏃‍♀' => 'boireannach a’ ruith',
    '🧑‍🩰' => 'dannsair baileit',
    '👯‍♂' => 'fireannaich le cluasan rabaid',
    '👯‍♀' => 'boireannaich le cluasan rabaid',
    '🧖‍♂' => 'duine ann an sauna',
    '🧖‍♀' => 'boireannach ann an sauna',
    '🧗‍♂' => 'duine a’ sreap',
    '🧗‍♀' => 'boireannach a’ sreap',
    '🏌‍♂' => 'duine ri goilf',
    '🏌‍♀' => 'boireannach ri goilf',
    '🏄‍♂' => 'duine a’ surfadh',
    '🏄‍♀' => 'boireannach a’ surfadh',
    '🚣‍♂' => 'duine ag iomradh bàta',
    '🚣‍♀' => 'boireannach ag iomradh bàta',
    '🏊‍♂' => 'duine a’ snàmh',
    '🏊‍♀' => 'boireannach a’ snàmh',
    '⛹‍♂' => 'duine a’ bocadh bàla',
    '⛹‍♀' => 'boireannach a’ bocadh bàla',
    '🏋‍♂' => 'duine a’ togail chuideaman',
    '🏋‍♀' => 'boireannach a’ togail chuideaman',
    '🚴‍♂' => 'duine ri rothaireachd',
    '🚴‍♀' => 'boireannach ri rothaireachd',
    '🚵‍♂' => 'duine ri rothaireachd bheanntan',
    '🚵‍♀' => 'boireannach ri rothaireachd bheanntan',
    '🤸‍♂' => 'duine a’ cur nan rothan',
    '🤸‍♀' => 'boireannach a’ cur nan rothan',
    '🤼‍♂' => 'fireannaich ri gleacadh',
    '🤼‍♀' => 'boireannaich ri gleacadh',
    '🤽‍♂' => 'duine a’ cluich polo-uisge',
    '🤽‍♀' => 'boireannach a’ cluich polo-uisge',
    '🤾‍♂' => 'duine a’ cluich ball-làimhe',
    '🤾‍♀' => 'boireannach a’ cluich ball-làimhe',
    '🤹‍♂' => 'duine ri caisreabhachd',
    '🤹‍♀' => 'boireannach ri caisreabhachd',
    '🧘‍♂' => 'duine na shuidhe lòtais',
    '🧘‍♀' => 'boireannach na shuidhe lòtais',
    '🧑‍🧒' => 'teaghlach: inbheach, duine-cloinne',
    '🐕‍🦺' => 'cù-taice',
    '🐈‍⬛' => 'cat dubh',
    '🐻‍❄' => 'mathan-bàn',
    '🐦‍⬛' => 'eun-dubh',
    '🐦‍🔥' => 'ainneamhag',
    '🍋‍🟩' => 'liomaideag',
    '🍄‍🟫' => 'balg-buachair donn',
    '⛓‍💥' => 'slabhraidh bhriste',
    '🏳‍🌈' => 'bratach bogha-froise',
    '🏳‍⚧' => 'bratach ghorm, phinc is gheal',
    '🏴‍☠' => 'bratach spùinneadair',
    '👋🏻' => 'làmh a’ smèideadh: tòna soilleir a’ chraicinn',
    '👋🏼' => 'làmh a’ smèideadh: tòna meadhanach soilleir a’ chraicinn',
    '👋🏽' => 'làmh a’ smèideadh: tòna meadhanach a’ chraicinn',
    '👋🏾' => 'làmh a’ smèideadh: tòna meadhanach doilleir a’ chraicinn',
    '👋🏿' => 'làmh a’ smèideadh: tòna doilleir a’ chraicinn',
    '🤚🏻' => 'cùl làimhe air a thogail: tòna soilleir a’ chraicinn',
    '🤚🏼' => 'cùl làimhe air a thogail: tòna meadhanach soilleir a’ chraicinn',
    '🤚🏽' => 'cùl làimhe air a thogail: tòna meadhanach a’ chraicinn',
    '🤚🏾' => 'cùl làimhe air a thogail: tòna meadhanach doilleir a’ chraicinn',
    '🤚🏿' => 'cùl làimhe air a thogail: tòna doilleir a’ chraicinn',
    '🖐🏻' => 'làmh air a togail le corragan sìnte: tòna soilleir a’ chraicinn',
    '🖐🏼' => 'làmh air a togail le corragan sìnte: tòna meadhanach soilleir a’ chraicinn',
    '🖐🏽' => 'làmh air a togail le corragan sìnte: tòna meadhanach a’ chraicinn',
    '🖐🏾' => 'làmh air a togail le corragan sìnte: tòna meadhanach doilleir a’ chraicinn',
    '🖐🏿' => 'làmh air a togail le corragan sìnte: tòna doilleir a’ chraicinn',
    '✋🏻' => 'làmh air a togail: tòna soilleir a’ chraicinn',
    '✋🏼' => 'làmh air a togail: tòna meadhanach soilleir a’ chraicinn',
    '✋🏽' => 'làmh air a togail: tòna meadhanach a’ chraicinn',
    '✋🏾' => 'làmh air a togail: tòna meadhanach doilleir a’ chraicinn',
    '✋🏿' => 'làmh air a togail: tòna doilleir a’ chraicinn',
    '🖖🏻' => 'beannachd muinntir Vulcan: tòna soilleir a’ chraicinn',
    '🖖🏼' => 'beannachd muinntir Vulcan: tòna meadhanach soilleir a’ chraicinn',
    '🖖🏽' => 'beannachd muinntir Vulcan: tòna meadhanach a’ chraicinn',
    '🖖🏾' => 'beannachd muinntir Vulcan: tòna meadhanach doilleir a’ chraicinn',
    '🖖🏿' => 'beannachd muinntir Vulcan: tòna doilleir a’ chraicinn',
    '🫱🏻' => 'làmh ris an taobh deas: tòna soilleir a’ chraicinn',
    '🫱🏼' => 'làmh ris an taobh deas: tòna meadhanach soilleir a’ chraicinn',
    '🫱🏽' => 'làmh ris an taobh deas: tòna meadhanach a’ chraicinn',
    '🫱🏾' => 'làmh ris an taobh deas: tòna meadhanach doilleir a’ chraicinn',
    '🫱🏿' => 'làmh ris an taobh deas: tòna doilleir a’ chraicinn',
    '🫲🏻' => 'làmh ris an taobh chlì: tòna soilleir a’ chraicinn',
    '🫲🏼' => 'làmh ris an taobh chlì: tòna meadhanach soilleir a’ chraicinn',
    '🫲🏽' => 'làmh ris an taobh chlì: tòna meadhanach a’ chraicinn',
    '🫲🏾' => 'làmh ris an taobh chlì: tòna meadhanach doilleir a’ chraicinn',
    '🫲🏿' => 'làmh ris an taobh chlì: tòna doilleir a’ chraicinn',
    '🫳🏻' => 'làmh le a bas sìos: tòna soilleir a’ chraicinn',
    '🫳🏼' => 'làmh le a bas sìos: tòna meadhanach soilleir a’ chraicinn',
    '🫳🏽' => 'làmh le a bas sìos: tòna meadhanach a’ chraicinn',
    '🫳🏾' => 'làmh le a bas sìos: tòna meadhanach doilleir a’ chraicinn',
    '🫳🏿' => 'làmh le a bas sìos: tòna doilleir a’ chraicinn',
    '🫴🏻' => 'làmh le a bas suas: tòna soilleir a’ chraicinn',
    '🫴🏼' => 'làmh le a bas suas: tòna meadhanach soilleir a’ chraicinn',
    '🫴🏽' => 'làmh le a bas suas: tòna meadhanach a’ chraicinn',
    '🫴🏾' => 'làmh le a bas suas: tòna meadhanach doilleir a’ chraicinn',
    '🫴🏿' => 'làmh le a bas suas: tòna doilleir a’ chraicinn',
    '🫷🏻' => 'làmh a’ putadh dhan taobh chlì: tòna soilleir a’ chraicinn',
    '🫷🏼' => 'làmh a’ putadh dhan taobh chlì: tòna meadhanach soilleir a’ chraicinn',
    '🫷🏽' => 'làmh a’ putadh dhan taobh chlì: tòna meadhanach a’ chraicinn',
    '🫷🏾' => 'làmh a’ putadh dhan taobh chlì: tòna meadhanach doilleir a’ chraicinn',
    '🫷🏿' => 'làmh a’ putadh dhan taobh chlì: tòna doilleir a’ chraicinn',
    '🫸🏻' => 'làmh a’ putadh dhan taobh dheas: tòna soilleir a’ chraicinn',
    '🫸🏼' => 'làmh a’ putadh dhan taobh dheas: tòna meadhanach soilleir a’ chraicinn',
    '🫸🏽' => 'làmh a’ putadh dhan taobh dheas: tòna meadhanach a’ chraicinn',
    '🫸🏾' => 'làmh a’ putadh dhan taobh dheas: tòna meadhanach doilleir a’ chraicinn',
    '🫸🏿' => 'làmh a’ putadh dhan taobh dheas: tòna doilleir a’ chraicinn',
    '👌🏻' => 'samhla làimhe OK: tòna soilleir a’ chraicinn',
    '👌🏼' => 'samhla làimhe OK: tòna meadhanach soilleir a’ chraicinn',
    '👌🏽' => 'samhla làimhe OK: tòna meadhanach a’ chraicinn',
    '👌🏾' => 'samhla làimhe OK: tòna meadhanach doilleir a’ chraicinn',
    '👌🏿' => 'samhla làimhe OK: tòna doilleir a’ chraicinn',
    '🤌🏻' => 'crudhagan: tòna soilleir a’ chraicinn',
    '🤌🏼' => 'crudhagan: tòna meadhanach soilleir a’ chraicinn',
    '🤌🏽' => 'crudhagan: tòna meadhanach a’ chraicinn',
    '🤌🏾' => 'crudhagan: tòna meadhanach doilleir a’ chraicinn',
    '🤌🏿' => 'crudhagan: tòna doilleir a’ chraicinn',
    '🤏🏻' => 'làmh le sradag: tòna soilleir a’ chraicinn',
    '🤏🏼' => 'làmh le sradag: tòna meadhanach soilleir a’ chraicinn',
    '🤏🏽' => 'làmh le sradag: tòna meadhanach a’ chraicinn',
    '🤏🏾' => 'làmh le sradag: tòna meadhanach doilleir a’ chraicinn',
    '🤏🏿' => 'làmh le sradag: tòna doilleir a’ chraicinn',
    '✌🏻' => 'làmh a thug buaidh: tòna soilleir a’ chraicinn',
    '✌🏼' => 'làmh a thug buaidh: tòna meadhanach soilleir a’ chraicinn',
    '✌🏽' => 'làmh a thug buaidh: tòna meadhanach a’ chraicinn',
    '✌🏾' => 'làmh a thug buaidh: tòna meadhanach doilleir a’ chraicinn',
    '✌🏿' => 'làmh a thug buaidh: tòna doilleir a’ chraicinn',
    '🤞🏻' => 'beò an dòchas: tòna soilleir a’ chraicinn',
    '🤞🏼' => 'beò an dòchas: tòna meadhanach soilleir a’ chraicinn',
    '🤞🏽' => 'beò an dòchas: tòna meadhanach a’ chraicinn',
    '🤞🏾' => 'beò an dòchas: tòna meadhanach doilleir a’ chraicinn',
    '🤞🏿' => 'beò an dòchas: tòna doilleir a’ chraicinn',
    '🫰🏻' => 'làmh le a colgag is òrdag ri chèile: tòna soilleir a’ chraicinn',
    '🫰🏼' => 'làmh le a colgag is òrdag ri chèile: tòna meadhanach soilleir a’ chraicinn',
    '🫰🏽' => 'làmh le a colgag is òrdag ri chèile: tòna meadhanach a’ chraicinn',
    '🫰🏾' => 'làmh le a colgag is òrdag ri chèile: tòna meadhanach doilleir a’ chraicinn',
    '🫰🏿' => 'làmh le a colgag is òrdag ri chèile: tòna doilleir a’ chraicinn',
    '🤟🏻' => 'gluasad “Tha gaol agam ort”: tòna soilleir a’ chraicinn',
    '🤟🏼' => 'gluasad “Tha gaol agam ort”: tòna meadhanach soilleir a’ chraicinn',
    '🤟🏽' => 'gluasad “Tha gaol agam ort”: tòna meadhanach a’ chraicinn',
    '🤟🏾' => 'gluasad “Tha gaol agam ort”: tòna meadhanach doilleir a’ chraicinn',
    '🤟🏿' => 'gluasad “Tha gaol agam ort”: tòna doilleir a’ chraicinn',
    '🤘🏻' => 'samhla nan adharc: tòna soilleir a’ chraicinn',
    '🤘🏼' => 'samhla nan adharc: tòna meadhanach soilleir a’ chraicinn',
    '🤘🏽' => 'samhla nan adharc: tòna meadhanach a’ chraicinn',
    '🤘🏾' => 'samhla nan adharc: tòna meadhanach doilleir a’ chraicinn',
    '🤘🏿' => 'samhla nan adharc: tòna doilleir a’ chraicinn',
    '🤙🏻' => 'làmh a’ comharradh “Cuir fòn thugam”: tòna soilleir a’ chraicinn',
    '🤙🏼' => 'làmh a’ comharradh “Cuir fòn thugam”: tòna meadhanach soilleir a’ chraicinn',
    '🤙🏽' => 'làmh a’ comharradh “Cuir fòn thugam”: tòna meadhanach a’ chraicinn',
    '🤙🏾' => 'làmh a’ comharradh “Cuir fòn thugam”: tòna meadhanach doilleir a’ chraicinn',
    '🤙🏿' => 'làmh a’ comharradh “Cuir fòn thugam”: tòna doilleir a’ chraicinn',
    '👈🏻' => 'druim na làimhe, colgag a’ tomhadh clì: tòna soilleir a’ chraicinn',
    '👈🏼' => 'druim na làimhe, colgag a’ tomhadh clì: tòna meadhanach soilleir a’ chraicinn',
    '👈🏽' => 'druim na làimhe, colgag a’ tomhadh clì: tòna meadhanach a’ chraicinn',
    '👈🏾' => 'druim na làimhe, colgag a’ tomhadh clì: tòna meadhanach doilleir a’ chraicinn',
    '👈🏿' => 'druim na làimhe, colgag a’ tomhadh clì: tòna doilleir a’ chraicinn',
    '👉🏻' => 'druim na làimhe, colgag a’ tomhadh deas: tòna soilleir a’ chraicinn',
    '👉🏼' => 'druim na làimhe, colgag a’ tomhadh deas: tòna meadhanach soilleir a’ chraicinn',
    '👉🏽' => 'druim na làimhe, colgag a’ tomhadh deas: tòna meadhanach a’ chraicinn',
    '👉🏾' => 'druim na làimhe, colgag a’ tomhadh deas: tòna meadhanach doilleir a’ chraicinn',
    '👉🏿' => 'druim na làimhe, colgag a’ tomhadh deas: tòna doilleir a’ chraicinn',
    '👆🏻' => 'druim na làimhe, colgag a’ tomhadh suas: tòna soilleir a’ chraicinn',
    '👆🏼' => 'druim na làimhe, colgag a’ tomhadh suas: tòna meadhanach soilleir a’ chraicinn',
    '👆🏽' => 'druim na làimhe, colgag a’ tomhadh suas: tòna meadhanach a’ chraicinn',
    '👆🏾' => 'druim na làimhe, colgag a’ tomhadh suas: tòna meadhanach doilleir a’ chraicinn',
    '👆🏿' => 'druim na làimhe, colgag a’ tomhadh suas: tòna doilleir a’ chraicinn',
    '🖕🏻' => 'Fionnlagh Fada / Màiri Fhada: tòna soilleir a’ chraicinn',
    '🖕🏼' => 'Fionnlagh Fada / Màiri Fhada: tòna meadhanach soilleir a’ chraicinn',
    '🖕🏽' => 'Fionnlagh Fada / Màiri Fhada: tòna meadhanach a’ chraicinn',
    '🖕🏾' => 'Fionnlagh Fada / Màiri Fhada: tòna meadhanach doilleir a’ chraicinn',
    '🖕🏿' => 'Fionnlagh Fada / Màiri Fhada: tòna doilleir a’ chraicinn',
    '👇🏻' => 'druim na làimhe, colgag a’ tomhadh sìos: tòna soilleir a’ chraicinn',
    '👇🏼' => 'druim na làimhe, colgag a’ tomhadh sìos: tòna meadhanach soilleir a’ chraicinn',
    '👇🏽' => 'druim na làimhe, colgag a’ tomhadh sìos: tòna meadhanach a’ chraicinn',
    '👇🏾' => 'druim na làimhe, colgag a’ tomhadh sìos: tòna meadhanach doilleir a’ chraicinn',
    '👇🏿' => 'druim na làimhe, colgag a’ tomhadh sìos: tòna doilleir a’ chraicinn',
    '☝🏻' => 'colgag a’ tomhadh suas: tòna soilleir a’ chraicinn',
    '☝🏼' => 'colgag a’ tomhadh suas: tòna meadhanach soilleir a’ chraicinn',
    '☝🏽' => 'colgag a’ tomhadh suas: tòna meadhanach a’ chraicinn',
    '☝🏾' => 'colgag a’ tomhadh suas: tòna meadhanach doilleir a’ chraicinn',
    '☝🏿' => 'colgag a’ tomhadh suas: tòna doilleir a’ chraicinn',
    '🫵🏻' => 'colgag a’ tomhadh ort: tòna soilleir a’ chraicinn',
    '🫵🏼' => 'colgag a’ tomhadh ort: tòna meadhanach soilleir a’ chraicinn',
    '🫵🏽' => 'colgag a’ tomhadh ort: tòna meadhanach a’ chraicinn',
    '🫵🏾' => 'colgag a’ tomhadh ort: tòna meadhanach doilleir a’ chraicinn',
    '🫵🏿' => 'colgag a’ tomhadh ort: tòna doilleir a’ chraicinn',
    '👍🏻' => 'math fhèin: tòna soilleir a’ chraicinn',
    '👍🏼' => 'math fhèin: tòna meadhanach soilleir a’ chraicinn',
    '👍🏽' => 'math fhèin: tòna meadhanach a’ chraicinn',
    '👍🏾' => 'math fhèin: tòna meadhanach doilleir a’ chraicinn',
    '👍🏿' => 'math fhèin: tòna doilleir a’ chraicinn',
    '👎🏻' => 'mo chreach: tòna soilleir a’ chraicinn',
    '👎🏼' => 'mo chreach: tòna meadhanach soilleir a’ chraicinn',
    '👎🏽' => 'mo chreach: tòna meadhanach a’ chraicinn',
    '👎🏾' => 'mo chreach: tòna meadhanach doilleir a’ chraicinn',
    '👎🏿' => 'mo chreach: tòna doilleir a’ chraicinn',
    '✊🏻' => 'dòrn air a thogail: tòna soilleir a’ chraicinn',
    '✊🏼' => 'dòrn air a thogail: tòna meadhanach soilleir a’ chraicinn',
    '✊🏽' => 'dòrn air a thogail: tòna meadhanach a’ chraicinn',
    '✊🏾' => 'dòrn air a thogail: tòna meadhanach doilleir a’ chraicinn',
    '✊🏿' => 'dòrn air a thogail: tòna doilleir a’ chraicinn',
    '👊🏻' => 'dòrn a’ tighinn thugad: tòna soilleir a’ chraicinn',
    '👊🏼' => 'dòrn a’ tighinn thugad: tòna meadhanach soilleir a’ chraicinn',
    '👊🏽' => 'dòrn a’ tighinn thugad: tòna meadhanach a’ chraicinn',
    '👊🏾' => 'dòrn a’ tighinn thugad: tòna meadhanach doilleir a’ chraicinn',
    '👊🏿' => 'dòrn a’ tighinn thugad: tòna doilleir a’ chraicinn',
    '🤛🏻' => 'dòrn ris an taobh chlì: tòna soilleir a’ chraicinn',
    '🤛🏼' => 'dòrn ris an taobh chlì: tòna meadhanach soilleir a’ chraicinn',
    '🤛🏽' => 'dòrn ris an taobh chlì: tòna meadhanach a’ chraicinn',
    '🤛🏾' => 'dòrn ris an taobh chlì: tòna meadhanach doilleir a’ chraicinn',
    '🤛🏿' => 'dòrn ris an taobh chlì: tòna doilleir a’ chraicinn',
    '🤜🏻' => 'dòrn ris an taobh deas: tòna soilleir a’ chraicinn',
    '🤜🏼' => 'dòrn ris an taobh deas: tòna meadhanach soilleir a’ chraicinn',
    '🤜🏽' => 'dòrn ris an taobh deas: tòna meadhanach a’ chraicinn',
    '🤜🏾' => 'dòrn ris an taobh deas: tòna meadhanach doilleir a’ chraicinn',
    '🤜🏿' => 'dòrn ris an taobh deas: tòna doilleir a’ chraicinn',
    '👏🏻' => 'bualadh bhasan: tòna soilleir a’ chraicinn',
    '👏🏼' => 'bualadh bhasan: tòna meadhanach soilleir a’ chraicinn',
    '👏🏽' => 'bualadh bhasan: tòna meadhanach a’ chraicinn',
    '👏🏾' => 'bualadh bhasan: tòna meadhanach doilleir a’ chraicinn',
    '👏🏿' => 'bualadh bhasan: tòna doilleir a’ chraicinn',
    '🙌🏻' => 'a’ togail làmhan: tòna soilleir a’ chraicinn',
    '🙌🏼' => 'a’ togail làmhan: tòna meadhanach soilleir a’ chraicinn',
    '🙌🏽' => 'a’ togail làmhan: tòna meadhanach a’ chraicinn',
    '🙌🏾' => 'a’ togail làmhan: tòna meadhanach doilleir a’ chraicinn',
    '🙌🏿' => 'a’ togail làmhan: tòna doilleir a’ chraicinn',
    '🫶🏻' => 'dà làimh ann an cumadh cridhe: tòna soilleir a’ chraicinn',
    '🫶🏼' => 'dà làimh ann an cumadh cridhe: tòna meadhanach soilleir a’ chraicinn',
    '🫶🏽' => 'dà làimh ann an cumadh cridhe: tòna meadhanach a’ chraicinn',
    '🫶🏾' => 'dà làimh ann an cumadh cridhe: tòna meadhanach doilleir a’ chraicinn',
    '🫶🏿' => 'dà làimh ann an cumadh cridhe: tòna doilleir a’ chraicinn',
    '👐🏻' => 'làmhan fosgailte: tòna soilleir a’ chraicinn',
    '👐🏼' => 'làmhan fosgailte: tòna meadhanach soilleir a’ chraicinn',
    '👐🏽' => 'làmhan fosgailte: tòna meadhanach a’ chraicinn',
    '👐🏾' => 'làmhan fosgailte: tòna meadhanach doilleir a’ chraicinn',
    '👐🏿' => 'làmhan fosgailte: tòna doilleir a’ chraicinn',
    '🤲🏻' => 'an dà bhois suas còmhla: tòna soilleir a’ chraicinn',
    '🤲🏼' => 'an dà bhois suas còmhla: tòna meadhanach soilleir a’ chraicinn',
    '🤲🏽' => 'an dà bhois suas còmhla: tòna meadhanach a’ chraicinn',
    '🤲🏾' => 'an dà bhois suas còmhla: tòna meadhanach doilleir a’ chraicinn',
    '🤲🏿' => 'an dà bhois suas còmhla: tòna doilleir a’ chraicinn',
    '🤝🏻' => 'crathadh-làimhe: tòna soilleir a’ chraicinn',
    '🤝🏼' => 'crathadh-làimhe: tòna meadhanach soilleir a’ chraicinn',
    '🤝🏽' => 'crathadh-làimhe: tòna meadhanach a’ chraicinn',
    '🤝🏾' => 'crathadh-làimhe: tòna meadhanach doilleir a’ chraicinn',
    '🤝🏿' => 'crathadh-làimhe: tòna doilleir a’ chraicinn',
    '🙏🏻' => 'làmhan paisgte: tòna soilleir a’ chraicinn',
    '🙏🏼' => 'làmhan paisgte: tòna meadhanach soilleir a’ chraicinn',
    '🙏🏽' => 'làmhan paisgte: tòna meadhanach a’ chraicinn',
    '🙏🏾' => 'làmhan paisgte: tòna meadhanach doilleir a’ chraicinn',
    '🙏🏿' => 'làmhan paisgte: tòna doilleir a’ chraicinn',
    '✍🏻' => 'làmh a’ sgrìobhadh: tòna soilleir a’ chraicinn',
    '✍🏼' => 'làmh a’ sgrìobhadh: tòna meadhanach soilleir a’ chraicinn',
    '✍🏽' => 'làmh a’ sgrìobhadh: tòna meadhanach a’ chraicinn',
    '✍🏾' => 'làmh a’ sgrìobhadh: tòna meadhanach doilleir a’ chraicinn',
    '✍🏿' => 'làmh a’ sgrìobhadh: tòna doilleir a’ chraicinn',
    '💅🏻' => 'lìomh ìnean: tòna soilleir a’ chraicinn',
    '💅🏼' => 'lìomh ìnean: tòna meadhanach soilleir a’ chraicinn',
    '💅🏽' => 'lìomh ìnean: tòna meadhanach a’ chraicinn',
    '💅🏾' => 'lìomh ìnean: tòna meadhanach doilleir a’ chraicinn',
    '💅🏿' => 'lìomh ìnean: tòna doilleir a’ chraicinn',
    '🤳🏻' => 'fèineag: tòna soilleir a’ chraicinn',
    '🤳🏼' => 'fèineag: tòna meadhanach soilleir a’ chraicinn',
    '🤳🏽' => 'fèineag: tòna meadhanach a’ chraicinn',
    '🤳🏾' => 'fèineag: tòna meadhanach doilleir a’ chraicinn',
    '🤳🏿' => 'fèineag: tòna doilleir a’ chraicinn',
    '💪🏻' => 'giorrachadh fèithean dà-cheannach: tòna soilleir a’ chraicinn',
    '💪🏼' => 'giorrachadh fèithean dà-cheannach: tòna meadhanach soilleir a’ chraicinn',
    '💪🏽' => 'giorrachadh fèithean dà-cheannach: tòna meadhanach a’ chraicinn',
    '💪🏾' => 'giorrachadh fèithean dà-cheannach: tòna meadhanach doilleir a’ chraicinn',
    '💪🏿' => 'giorrachadh fèithean dà-cheannach: tòna doilleir a’ chraicinn',
    '🦵🏻' => 'sliasaid: tòna soilleir a’ chraicinn',
    '🦵🏼' => 'sliasaid: tòna meadhanach soilleir a’ chraicinn',
    '🦵🏽' => 'sliasaid: tòna meadhanach a’ chraicinn',
    '🦵🏾' => 'sliasaid: tòna meadhanach doilleir a’ chraicinn',
    '🦵🏿' => 'sliasaid: tòna doilleir a’ chraicinn',
    '🦶🏻' => 'cas: tòna soilleir a’ chraicinn',
    '🦶🏼' => 'cas: tòna meadhanach soilleir a’ chraicinn',
    '🦶🏽' => 'cas: tòna meadhanach a’ chraicinn',
    '🦶🏾' => 'cas: tòna meadhanach doilleir a’ chraicinn',
    '🦶🏿' => 'cas: tòna doilleir a’ chraicinn',
    '👂🏻' => 'cluas: tòna soilleir a’ chraicinn',
    '👂🏼' => 'cluas: tòna meadhanach soilleir a’ chraicinn',
    '👂🏽' => 'cluas: tòna meadhanach a’ chraicinn',
    '👂🏾' => 'cluas: tòna meadhanach doilleir a’ chraicinn',
    '👂🏿' => 'cluas: tòna doilleir a’ chraicinn',
    '🦻🏻' => 'cluas le claisteachan: tòna soilleir a’ chraicinn',
    '🦻🏼' => 'cluas le claisteachan: tòna meadhanach soilleir a’ chraicinn',
    '🦻🏽' => 'cluas le claisteachan: tòna meadhanach a’ chraicinn',
    '🦻🏾' => 'cluas le claisteachan: tòna meadhanach doilleir a’ chraicinn',
    '🦻🏿' => 'cluas le claisteachan: tòna doilleir a’ chraicinn',
    '👃🏻' => 'sròn: tòna soilleir a’ chraicinn',
    '👃🏼' => 'sròn: tòna meadhanach soilleir a’ chraicinn',
    '👃🏽' => 'sròn: tòna meadhanach a’ chraicinn',
    '👃🏾' => 'sròn: tòna meadhanach doilleir a’ chraicinn',
    '👃🏿' => 'sròn: tòna doilleir a’ chraicinn',
    '👶🏻' => 'naoidhean: tòna soilleir a’ chraicinn',
    '👶🏼' => 'naoidhean: tòna meadhanach soilleir a’ chraicinn',
    '👶🏽' => 'naoidhean: tòna meadhanach a’ chraicinn',
    '👶🏾' => 'naoidhean: tòna meadhanach doilleir a’ chraicinn',
    '👶🏿' => 'naoidhean: tòna doilleir a’ chraicinn',
    '🧒🏻' => 'duine-cloinne: tòna soilleir a’ chraicinn',
    '🧒🏼' => 'duine-cloinne: tòna meadhanach soilleir a’ chraicinn',
    '🧒🏽' => 'duine-cloinne: tòna meadhanach a’ chraicinn',
    '🧒🏾' => 'duine-cloinne: tòna meadhanach doilleir a’ chraicinn',
    '🧒🏿' => 'duine-cloinne: tòna doilleir a’ chraicinn',
    '👦🏻' => 'gille: tòna soilleir a’ chraicinn',
    '👦🏼' => 'gille: tòna meadhanach soilleir a’ chraicinn',
    '👦🏽' => 'gille: tòna meadhanach a’ chraicinn',
    '👦🏾' => 'gille: tòna meadhanach doilleir a’ chraicinn',
    '👦🏿' => 'gille: tòna doilleir a’ chraicinn',
    '👧🏻' => 'caileag: tòna soilleir a’ chraicinn',
    '👧🏼' => 'caileag: tòna meadhanach soilleir a’ chraicinn',
    '👧🏽' => 'caileag: tòna meadhanach a’ chraicinn',
    '👧🏾' => 'caileag: tòna meadhanach doilleir a’ chraicinn',
    '👧🏿' => 'caileag: tòna doilleir a’ chraicinn',
    '🧑🏻' => 'inbheach: tòna soilleir a’ chraicinn',
    '🧑🏼' => 'inbheach: tòna meadhanach soilleir a’ chraicinn',
    '🧑🏽' => 'inbheach: tòna meadhanach a’ chraicinn',
    '🧑🏾' => 'inbheach: tòna meadhanach doilleir a’ chraicinn',
    '🧑🏿' => 'inbheach: tòna doilleir a’ chraicinn',
    '👱🏻' => 'neach le falt bàn: tòna soilleir a’ chraicinn',
    '👱🏼' => 'neach le falt bàn: tòna meadhanach soilleir a’ chraicinn',
    '👱🏽' => 'neach le falt bàn: tòna meadhanach a’ chraicinn',
    '👱🏾' => 'neach le falt bàn: tòna meadhanach doilleir a’ chraicinn',
    '👱🏿' => 'neach le falt bàn: tòna doilleir a’ chraicinn',
    '👨🏻' => 'duine: tòna soilleir a’ chraicinn',
    '👨🏼' => 'duine: tòna meadhanach soilleir a’ chraicinn',
    '👨🏽' => 'duine: tòna meadhanach a’ chraicinn',
    '👨🏾' => 'duine: tòna meadhanach doilleir a’ chraicinn',
    '👨🏿' => 'duine: tòna doilleir a’ chraicinn',
    '🧔🏻' => 'duine le feusag: tòna soilleir a’ chraicinn',
    '🧔🏼' => 'duine le feusag: tòna meadhanach soilleir a’ chraicinn',
    '🧔🏽' => 'duine le feusag: tòna meadhanach a’ chraicinn',
    '🧔🏾' => 'duine le feusag: tòna meadhanach doilleir a’ chraicinn',
    '🧔🏿' => 'duine le feusag: tòna doilleir a’ chraicinn',
    '👩🏻' => 'boireannach: tòna soilleir a’ chraicinn',
    '👩🏼' => 'boireannach: tòna meadhanach soilleir a’ chraicinn',
    '👩🏽' => 'boireannach: tòna meadhanach a’ chraicinn',
    '👩🏾' => 'boireannach: tòna meadhanach doilleir a’ chraicinn',
    '👩🏿' => 'boireannach: tòna doilleir a’ chraicinn',
    '🧓🏻' => 'seann-inbheach: tòna soilleir a’ chraicinn',
    '🧓🏼' => 'seann-inbheach: tòna meadhanach soilleir a’ chraicinn',
    '🧓🏽' => 'seann-inbheach: tòna meadhanach a’ chraicinn',
    '🧓🏾' => 'seann-inbheach: tòna meadhanach doilleir a’ chraicinn',
    '🧓🏿' => 'seann-inbheach: tòna doilleir a’ chraicinn',
    '👴🏻' => 'bodach: tòna soilleir a’ chraicinn',
    '👴🏼' => 'bodach: tòna meadhanach soilleir a’ chraicinn',
    '👴🏽' => 'bodach: tòna meadhanach a’ chraicinn',
    '👴🏾' => 'bodach: tòna meadhanach doilleir a’ chraicinn',
    '👴🏿' => 'bodach: tòna doilleir a’ chraicinn',
    '👵🏻' => 'cailleach: tòna soilleir a’ chraicinn',
    '👵🏼' => 'cailleach: tòna meadhanach soilleir a’ chraicinn',
    '👵🏽' => 'cailleach: tòna meadhanach a’ chraicinn',
    '👵🏾' => 'cailleach: tòna meadhanach doilleir a’ chraicinn',
    '👵🏿' => 'cailleach: tòna doilleir a’ chraicinn',
    '🙍🏻' => 'neach le drèin: tòna soilleir a’ chraicinn',
    '🙍🏼' => 'neach le drèin: tòna meadhanach soilleir a’ chraicinn',
    '🙍🏽' => 'neach le drèin: tòna meadhanach a’ chraicinn',
    '🙍🏾' => 'neach le drèin: tòna meadhanach doilleir a’ chraicinn',
    '🙍🏿' => 'neach le drèin: tòna doilleir a’ chraicinn',
    '🙎🏻' => 'neach le bus air: tòna soilleir a’ chraicinn',
    '🙎🏼' => 'neach le bus air: tòna meadhanach soilleir a’ chraicinn',
    '🙎🏽' => 'neach le bus air: tòna meadhanach a’ chraicinn',
    '🙎🏾' => 'neach le bus air: tòna meadhanach doilleir a’ chraicinn',
    '🙎🏿' => 'neach le bus air: tòna doilleir a’ chraicinn',
    '🙅🏻' => 'neach a’ diùltadh rud: tòna soilleir a’ chraicinn',
    '🙅🏼' => 'neach a’ diùltadh rud: tòna meadhanach soilleir a’ chraicinn',
    '🙅🏽' => 'neach a’ diùltadh rud: tòna meadhanach a’ chraicinn',
    '🙅🏾' => 'neach a’ diùltadh rud: tòna meadhanach doilleir a’ chraicinn',
    '🙅🏿' => 'neach a’ diùltadh rud: tòna doilleir a’ chraicinn',
    '🙆🏻' => 'neach ag aontachadh ri rud: tòna soilleir a’ chraicinn',
    '🙆🏼' => 'neach ag aontachadh ri rud: tòna meadhanach soilleir a’ chraicinn',
    '🙆🏽' => 'neach ag aontachadh ri rud: tòna meadhanach a’ chraicinn',
    '🙆🏾' => 'neach ag aontachadh ri rud: tòna meadhanach doilleir a’ chraicinn',
    '🙆🏿' => 'neach ag aontachadh ri rud: tòna doilleir a’ chraicinn',
    '💁🏻' => 'neach le làmh sìnte, bas air uachdar: tòna soilleir a’ chraicinn',
    '💁🏼' => 'neach le làmh sìnte, bas air uachdar: tòna meadhanach soilleir a’ chraicinn',
    '💁🏽' => 'neach le làmh sìnte, bas air uachdar: tòna meadhanach a’ chraicinn',
    '💁🏾' => 'neach le làmh sìnte, bas air uachdar: tòna meadhanach doilleir a’ chraicinn',
    '💁🏿' => 'neach le làmh sìnte, bas air uachdar: tòna doilleir a’ chraicinn',
    '🙋🏻' => 'neach a’ togail a làmh: tòna soilleir a’ chraicinn',
    '🙋🏼' => 'neach a’ togail a làmh: tòna meadhanach soilleir a’ chraicinn',
    '🙋🏽' => 'neach a’ togail a làmh: tòna meadhanach a’ chraicinn',
    '🙋🏾' => 'neach a’ togail a làmh: tòna meadhanach doilleir a’ chraicinn',
    '🙋🏿' => 'neach a’ togail a làmh: tòna doilleir a’ chraicinn',
    '🧏🏻' => 'neach bodhar: tòna soilleir a’ chraicinn',
    '🧏🏼' => 'neach bodhar: tòna meadhanach soilleir a’ chraicinn',
    '🧏🏽' => 'neach bodhar: tòna meadhanach a’ chraicinn',
    '🧏🏾' => 'neach bodhar: tòna meadhanach doilleir a’ chraicinn',
    '🧏🏿' => 'neach bodhar: tòna doilleir a’ chraicinn',
    '🙇🏻' => 'neach a’ cromadh: tòna soilleir a’ chraicinn',
    '🙇🏼' => 'neach a’ cromadh: tòna meadhanach soilleir a’ chraicinn',
    '🙇🏽' => 'neach a’ cromadh: tòna meadhanach a’ chraicinn',
    '🙇🏾' => 'neach a’ cromadh: tòna meadhanach doilleir a’ chraicinn',
    '🙇🏿' => 'neach a’ cromadh: tòna doilleir a’ chraicinn',
    '🤦🏻' => 'neach le aodann na làmhan: tòna soilleir a’ chraicinn',
    '🤦🏼' => 'neach le aodann na làmhan: tòna meadhanach soilleir a’ chraicinn',
    '🤦🏽' => 'neach le aodann na làmhan: tòna meadhanach a’ chraicinn',
    '🤦🏾' => 'neach le aodann na làmhan: tòna meadhanach doilleir a’ chraicinn',
    '🤦🏿' => 'neach le aodann na làmhan: tòna doilleir a’ chraicinn',
    '🤷🏻' => 'neach a’ druiteadh: tòna soilleir a’ chraicinn',
    '🤷🏼' => 'neach a’ druiteadh: tòna meadhanach soilleir a’ chraicinn',
    '🤷🏽' => 'neach a’ druiteadh: tòna meadhanach a’ chraicinn',
    '🤷🏾' => 'neach a’ druiteadh: tòna meadhanach doilleir a’ chraicinn',
    '🤷🏿' => 'neach a’ druiteadh: tòna doilleir a’ chraicinn',
    '👮🏻' => 'oifigear poileis: tòna soilleir a’ chraicinn',
    '👮🏼' => 'oifigear poileis: tòna meadhanach soilleir a’ chraicinn',
    '👮🏽' => 'oifigear poileis: tòna meadhanach a’ chraicinn',
    '👮🏾' => 'oifigear poileis: tòna meadhanach doilleir a’ chraicinn',
    '👮🏿' => 'oifigear poileis: tòna doilleir a’ chraicinn',
    '🕵🏻' => 'lorgaire: tòna soilleir a’ chraicinn',
    '🕵🏼' => 'lorgaire: tòna meadhanach soilleir a’ chraicinn',
    '🕵🏽' => 'lorgaire: tòna meadhanach a’ chraicinn',
    '🕵🏾' => 'lorgaire: tòna meadhanach doilleir a’ chraicinn',
    '🕵🏿' => 'lorgaire: tòna doilleir a’ chraicinn',
    '💂🏻' => 'geàrd: tòna soilleir a’ chraicinn',
    '💂🏼' => 'geàrd: tòna meadhanach soilleir a’ chraicinn',
    '💂🏽' => 'geàrd: tòna meadhanach a’ chraicinn',
    '💂🏾' => 'geàrd: tòna meadhanach doilleir a’ chraicinn',
    '💂🏿' => 'geàrd: tòna doilleir a’ chraicinn',
    '🥷🏻' => 'ninja: tòna soilleir a’ chraicinn',
    '🥷🏼' => 'ninja: tòna meadhanach soilleir a’ chraicinn',
    '🥷🏽' => 'ninja: tòna meadhanach a’ chraicinn',
    '🥷🏾' => 'ninja: tòna meadhanach doilleir a’ chraicinn',
    '🥷🏿' => 'ninja: tòna doilleir a’ chraicinn',
    '👷🏻' => 'neach-togail: tòna soilleir a’ chraicinn',
    '👷🏼' => 'neach-togail: tòna meadhanach soilleir a’ chraicinn',
    '👷🏽' => 'neach-togail: tòna meadhanach a’ chraicinn',
    '👷🏾' => 'neach-togail: tòna meadhanach doilleir a’ chraicinn',
    '👷🏿' => 'neach-togail: tòna doilleir a’ chraicinn',
    '🫅🏻' => 'neach le crùn air: tòna soilleir a’ chraicinn',
    '🫅🏼' => 'neach le crùn air: tòna meadhanach soilleir a’ chraicinn',
    '🫅🏽' => 'neach le crùn air: tòna meadhanach a’ chraicinn',
    '🫅🏾' => 'neach le crùn air: tòna meadhanach doilleir a’ chraicinn',
    '🫅🏿' => 'neach le crùn air: tòna doilleir a’ chraicinn',
    '🤴🏻' => 'prionnsa: tòna soilleir a’ chraicinn',
    '🤴🏼' => 'prionnsa: tòna meadhanach soilleir a’ chraicinn',
    '🤴🏽' => 'prionnsa: tòna meadhanach a’ chraicinn',
    '🤴🏾' => 'prionnsa: tòna meadhanach doilleir a’ chraicinn',
    '🤴🏿' => 'prionnsa: tòna doilleir a’ chraicinn',
    '👸🏻' => 'bana-phrionnsa: tòna soilleir a’ chraicinn',
    '👸🏼' => 'bana-phrionnsa: tòna meadhanach soilleir a’ chraicinn',
    '👸🏽' => 'bana-phrionnsa: tòna meadhanach a’ chraicinn',
    '👸🏾' => 'bana-phrionnsa: tòna meadhanach doilleir a’ chraicinn',
    '👸🏿' => 'bana-phrionnsa: tòna doilleir a’ chraicinn',
    '👳🏻' => 'neach le turban air: tòna soilleir a’ chraicinn',
    '👳🏼' => 'neach le turban air: tòna meadhanach soilleir a’ chraicinn',
    '👳🏽' => 'neach le turban air: tòna meadhanach a’ chraicinn',
    '👳🏾' => 'neach le turban air: tòna meadhanach doilleir a’ chraicinn',
    '👳🏿' => 'neach le turban air: tòna doilleir a’ chraicinn',
    '👲🏻' => 'duine le currac Sìneach: tòna soilleir a’ chraicinn',
    '👲🏼' => 'duine le currac Sìneach: tòna meadhanach soilleir a’ chraicinn',
    '👲🏽' => 'duine le currac Sìneach: tòna meadhanach a’ chraicinn',
    '👲🏾' => 'duine le currac Sìneach: tòna meadhanach doilleir a’ chraicinn',
    '👲🏿' => 'duine le currac Sìneach: tòna doilleir a’ chraicinn',
    '🧕🏻' => 'boireannach le brèid oirre: tòna soilleir a’ chraicinn',
    '🧕🏼' => 'boireannach le brèid oirre: tòna meadhanach soilleir a’ chraicinn',
    '🧕🏽' => 'boireannach le brèid oirre: tòna meadhanach a’ chraicinn',
    '🧕🏾' => 'boireannach le brèid oirre: tòna meadhanach doilleir a’ chraicinn',
    '🧕🏿' => 'boireannach le brèid oirre: tòna doilleir a’ chraicinn',
    '🤵🏻' => 'neach ann an tuxedo: tòna soilleir a’ chraicinn',
    '🤵🏼' => 'neach ann an tuxedo: tòna meadhanach soilleir a’ chraicinn',
    '🤵🏽' => 'neach ann an tuxedo: tòna meadhanach a’ chraicinn',
    '🤵🏾' => 'neach ann an tuxedo: tòna meadhanach doilleir a’ chraicinn',
    '🤵🏿' => 'neach ann an tuxedo: tòna doilleir a’ chraicinn',
    '👰🏻' => 'neach le brat-gnùise: tòna soilleir a’ chraicinn',
    '👰🏼' => 'neach le brat-gnùise: tòna meadhanach soilleir a’ chraicinn',
    '👰🏽' => 'neach le brat-gnùise: tòna meadhanach a’ chraicinn',
    '👰🏾' => 'neach le brat-gnùise: tòna meadhanach doilleir a’ chraicinn',
    '👰🏿' => 'neach le brat-gnùise: tòna doilleir a’ chraicinn',
    '🤰🏻' => 'boireannach leatromach: tòna soilleir a’ chraicinn',
    '🤰🏼' => 'boireannach leatromach: tòna meadhanach soilleir a’ chraicinn',
    '🤰🏽' => 'boireannach leatromach: tòna meadhanach a’ chraicinn',
    '🤰🏾' => 'boireannach leatromach: tòna meadhanach doilleir a’ chraicinn',
    '🤰🏿' => 'boireannach leatromach: tòna doilleir a’ chraicinn',
    '🫃🏻' => 'fireannach trom: tòna soilleir a’ chraicinn',
    '🫃🏼' => 'fireannach trom: tòna meadhanach soilleir a’ chraicinn',
    '🫃🏽' => 'fireannach trom: tòna meadhanach a’ chraicinn',
    '🫃🏾' => 'fireannach trom: tòna meadhanach doilleir a’ chraicinn',
    '🫃🏿' => 'fireannach trom: tòna doilleir a’ chraicinn',
    '🫄🏻' => 'neach trom: tòna soilleir a’ chraicinn',
    '🫄🏼' => 'neach trom: tòna meadhanach soilleir a’ chraicinn',
    '🫄🏽' => 'neach trom: tòna meadhanach a’ chraicinn',
    '🫄🏾' => 'neach trom: tòna meadhanach doilleir a’ chraicinn',
    '🫄🏿' => 'neach trom: tòna doilleir a’ chraicinn',
    '🤱🏻' => 'toirt cìoch: tòna soilleir a’ chraicinn',
    '🤱🏼' => 'toirt cìoch: tòna meadhanach soilleir a’ chraicinn',
    '🤱🏽' => 'toirt cìoch: tòna meadhanach a’ chraicinn',
    '🤱🏾' => 'toirt cìoch: tòna meadhanach doilleir a’ chraicinn',
    '🤱🏿' => 'toirt cìoch: tòna doilleir a’ chraicinn',
    '👼🏻' => 'aingeal òg: tòna soilleir a’ chraicinn',
    '👼🏼' => 'aingeal òg: tòna meadhanach soilleir a’ chraicinn',
    '👼🏽' => 'aingeal òg: tòna meadhanach a’ chraicinn',
    '👼🏾' => 'aingeal òg: tòna meadhanach doilleir a’ chraicinn',
    '👼🏿' => 'aingeal òg: tòna doilleir a’ chraicinn',
    '🎅🏻' => 'bodach na Nollaig: tòna soilleir a’ chraicinn',
    '🎅🏼' => 'bodach na Nollaig: tòna meadhanach soilleir a’ chraicinn',
    '🎅🏽' => 'bodach na Nollaig: tòna meadhanach a’ chraicinn',
    '🎅🏾' => 'bodach na Nollaig: tòna meadhanach doilleir a’ chraicinn',
    '🎅🏿' => 'bodach na Nollaig: tòna doilleir a’ chraicinn',
    '🤶🏻' => 'bean Bodach na Nollaig: tòna soilleir a’ chraicinn',
    '🤶🏼' => 'bean Bodach na Nollaig: tòna meadhanach soilleir a’ chraicinn',
    '🤶🏽' => 'bean Bodach na Nollaig: tòna meadhanach a’ chraicinn',
    '🤶🏾' => 'bean Bodach na Nollaig: tòna meadhanach doilleir a’ chraicinn',
    '🤶🏿' => 'bean Bodach na Nollaig: tòna doilleir a’ chraicinn',
    '🦸🏻' => 'sàr-ghaisgeach: tòna soilleir a’ chraicinn',
    '🦸🏼' => 'sàr-ghaisgeach: tòna meadhanach soilleir a’ chraicinn',
    '🦸🏽' => 'sàr-ghaisgeach: tòna meadhanach a’ chraicinn',
    '🦸🏾' => 'sàr-ghaisgeach: tòna meadhanach doilleir a’ chraicinn',
    '🦸🏿' => 'sàr-ghaisgeach: tòna doilleir a’ chraicinn',
    '🦹🏻' => 'sàr-shlaightear: tòna soilleir a’ chraicinn',
    '🦹🏼' => 'sàr-shlaightear: tòna meadhanach soilleir a’ chraicinn',
    '🦹🏽' => 'sàr-shlaightear: tòna meadhanach a’ chraicinn',
    '🦹🏾' => 'sàr-shlaightear: tòna meadhanach doilleir a’ chraicinn',
    '🦹🏿' => 'sàr-shlaightear: tòna doilleir a’ chraicinn',
    '🧙🏻' => 'draoidh: tòna soilleir a’ chraicinn',
    '🧙🏼' => 'draoidh: tòna meadhanach soilleir a’ chraicinn',
    '🧙🏽' => 'draoidh: tòna meadhanach a’ chraicinn',
    '🧙🏾' => 'draoidh: tòna meadhanach doilleir a’ chraicinn',
    '🧙🏿' => 'draoidh: tòna doilleir a’ chraicinn',
    '🧚🏻' => 'sìthiche: tòna soilleir a’ chraicinn',
    '🧚🏼' => 'sìthiche: tòna meadhanach soilleir a’ chraicinn',
    '🧚🏽' => 'sìthiche: tòna meadhanach a’ chraicinn',
    '🧚🏾' => 'sìthiche: tòna meadhanach doilleir a’ chraicinn',
    '🧚🏿' => 'sìthiche: tòna doilleir a’ chraicinn',
    '🧛🏻' => 'bhampair: tòna soilleir a’ chraicinn',
    '🧛🏼' => 'bhampair: tòna meadhanach soilleir a’ chraicinn',
    '🧛🏽' => 'bhampair: tòna meadhanach a’ chraicinn',
    '🧛🏾' => 'bhampair: tòna meadhanach doilleir a’ chraicinn',
    '🧛🏿' => 'bhampair: tòna doilleir a’ chraicinn',
    '🧜🏻' => 'neach-mara: tòna soilleir a’ chraicinn',
    '🧜🏼' => 'neach-mara: tòna meadhanach soilleir a’ chraicinn',
    '🧜🏽' => 'neach-mara: tòna meadhanach a’ chraicinn',
    '🧜🏾' => 'neach-mara: tòna meadhanach doilleir a’ chraicinn',
    '🧜🏿' => 'neach-mara: tòna doilleir a’ chraicinn',
    '🧝🏻' => 'ailbhear: tòna soilleir a’ chraicinn',
    '🧝🏼' => 'ailbhear: tòna meadhanach soilleir a’ chraicinn',
    '🧝🏽' => 'ailbhear: tòna meadhanach a’ chraicinn',
    '🧝🏾' => 'ailbhear: tòna meadhanach doilleir a’ chraicinn',
    '🧝🏿' => 'ailbhear: tòna doilleir a’ chraicinn',
    '💆🏻' => 'neach a’ faighinn massage: tòna soilleir a’ chraicinn',
    '💆🏼' => 'neach a’ faighinn massage: tòna meadhanach soilleir a’ chraicinn',
    '💆🏽' => 'neach a’ faighinn massage: tòna meadhanach a’ chraicinn',
    '💆🏾' => 'neach a’ faighinn massage: tòna meadhanach doilleir a’ chraicinn',
    '💆🏿' => 'neach a’ faighinn massage: tòna doilleir a’ chraicinn',
    '💇🏻' => 'neach a’ faighinn cliop: tòna soilleir a’ chraicinn',
    '💇🏼' => 'neach a’ faighinn cliop: tòna meadhanach soilleir a’ chraicinn',
    '💇🏽' => 'neach a’ faighinn cliop: tòna meadhanach a’ chraicinn',
    '💇🏾' => 'neach a’ faighinn cliop: tòna meadhanach doilleir a’ chraicinn',
    '💇🏿' => 'neach a’ faighinn cliop: tòna doilleir a’ chraicinn',
    '🚶🏻' => 'neach a’ coiseachd: tòna soilleir a’ chraicinn',
    '🚶🏼' => 'neach a’ coiseachd: tòna meadhanach soilleir a’ chraicinn',
    '🚶🏽' => 'neach a’ coiseachd: tòna meadhanach a’ chraicinn',
    '🚶🏾' => 'neach a’ coiseachd: tòna meadhanach doilleir a’ chraicinn',
    '🚶🏿' => 'neach a’ coiseachd: tòna doilleir a’ chraicinn',
    '🧍🏻' => 'neach ’na sheasamh: tòna soilleir a’ chraicinn',
    '🧍🏼' => 'neach ’na sheasamh: tòna meadhanach soilleir a’ chraicinn',
    '🧍🏽' => 'neach ’na sheasamh: tòna meadhanach a’ chraicinn',
    '🧍🏾' => 'neach ’na sheasamh: tòna meadhanach doilleir a’ chraicinn',
    '🧍🏿' => 'neach ’na sheasamh: tòna doilleir a’ chraicinn',
    '🧎🏻' => 'neach air a ghlùinean: tòna soilleir a’ chraicinn',
    '🧎🏼' => 'neach air a ghlùinean: tòna meadhanach soilleir a’ chraicinn',
    '🧎🏽' => 'neach air a ghlùinean: tòna meadhanach a’ chraicinn',
    '🧎🏾' => 'neach air a ghlùinean: tòna meadhanach doilleir a’ chraicinn',
    '🧎🏿' => 'neach air a ghlùinean: tòna doilleir a’ chraicinn',
    '🏃🏻' => 'neach a’ ruith: tòna soilleir a’ chraicinn',
    '🏃🏼' => 'neach a’ ruith: tòna meadhanach soilleir a’ chraicinn',
    '🏃🏽' => 'neach a’ ruith: tòna meadhanach a’ chraicinn',
    '🏃🏾' => 'neach a’ ruith: tòna meadhanach doilleir a’ chraicinn',
    '🏃🏿' => 'neach a’ ruith: tòna doilleir a’ chraicinn',
    '💃🏻' => 'boireannach a’ dannsadh: tòna soilleir a’ chraicinn',
    '💃🏼' => 'boireannach a’ dannsadh: tòna meadhanach soilleir a’ chraicinn',
    '💃🏽' => 'boireannach a’ dannsadh: tòna meadhanach a’ chraicinn',
    '💃🏾' => 'boireannach a’ dannsadh: tòna meadhanach doilleir a’ chraicinn',
    '💃🏿' => 'boireannach a’ dannsadh: tòna doilleir a’ chraicinn',
    '🕺🏻' => 'duine a’ dannsadh: tòna soilleir a’ chraicinn',
    '🕺🏼' => 'duine a’ dannsadh: tòna meadhanach soilleir a’ chraicinn',
    '🕺🏽' => 'duine a’ dannsadh: tòna meadhanach a’ chraicinn',
    '🕺🏾' => 'duine a’ dannsadh: tòna meadhanach doilleir a’ chraicinn',
    '🕺🏿' => 'duine a’ dannsadh: tòna doilleir a’ chraicinn',
    '🕴🏻' => 'duine air flod ’s deise gnìomhachais air: tòna soilleir a’ chraicinn',
    '🕴🏼' => 'duine air flod ’s deise gnìomhachais air: tòna meadhanach soilleir a’ chraicinn',
    '🕴🏽' => 'duine air flod ’s deise gnìomhachais air: tòna meadhanach a’ chraicinn',
    '🕴🏾' => 'duine air flod ’s deise gnìomhachais air: tòna meadhanach doilleir a’ chraicinn',
    '🕴🏿' => 'duine air flod ’s deise gnìomhachais air: tòna doilleir a’ chraicinn',
    '👯🏻' => 'daoine le cluasan rabaid: tòna soilleir a’ chraicinn',
    '👯🏼' => 'daoine le cluasan rabaid: tòna meadhanach soilleir a’ chraicinn',
    '👯🏽' => 'daoine le cluasan rabaid: tòna meadhanach a’ chraicinn',
    '👯🏾' => 'daoine le cluasan rabaid: tòna meadhanach doilleir a’ chraicinn',
    '👯🏿' => 'daoine le cluasan rabaid: tòna doilleir a’ chraicinn',
    '🧖🏻' => 'neach ann an sauna: tòna soilleir a’ chraicinn',
    '🧖🏼' => 'neach ann an sauna: tòna meadhanach soilleir a’ chraicinn',
    '🧖🏽' => 'neach ann an sauna: tòna meadhanach a’ chraicinn',
    '🧖🏾' => 'neach ann an sauna: tòna meadhanach doilleir a’ chraicinn',
    '🧖🏿' => 'neach ann an sauna: tòna doilleir a’ chraicinn',
    '🧗🏻' => 'neach a’ sreap: tòna soilleir a’ chraicinn',
    '🧗🏼' => 'neach a’ sreap: tòna meadhanach soilleir a’ chraicinn',
    '🧗🏽' => 'neach a’ sreap: tòna meadhanach a’ chraicinn',
    '🧗🏾' => 'neach a’ sreap: tòna meadhanach doilleir a’ chraicinn',
    '🧗🏿' => 'neach a’ sreap: tòna doilleir a’ chraicinn',
    '🏇🏻' => 'rèiseadh each: tòna soilleir a’ chraicinn',
    '🏇🏼' => 'rèiseadh each: tòna meadhanach soilleir a’ chraicinn',
    '🏇🏽' => 'rèiseadh each: tòna meadhanach a’ chraicinn',
    '🏇🏾' => 'rèiseadh each: tòna meadhanach doilleir a’ chraicinn',
    '🏇🏿' => 'rèiseadh each: tòna doilleir a’ chraicinn',
    '🏂🏻' => 'neach sneachda-bhòrd: tòna soilleir a’ chraicinn',
    '🏂🏼' => 'neach sneachda-bhòrd: tòna meadhanach soilleir a’ chraicinn',
    '🏂🏽' => 'neach sneachda-bhòrd: tòna meadhanach a’ chraicinn',
    '🏂🏾' => 'neach sneachda-bhòrd: tòna meadhanach doilleir a’ chraicinn',
    '🏂🏿' => 'neach sneachda-bhòrd: tòna doilleir a’ chraicinn',
    '🏌🏻' => 'neach ri goilf: tòna soilleir a’ chraicinn',
    '🏌🏼' => 'neach ri goilf: tòna meadhanach soilleir a’ chraicinn',
    '🏌🏽' => 'neach ri goilf: tòna meadhanach a’ chraicinn',
    '🏌🏾' => 'neach ri goilf: tòna meadhanach doilleir a’ chraicinn',
    '🏌🏿' => 'neach ri goilf: tòna doilleir a’ chraicinn',
    '🏄🏻' => 'neach a’ surfadh: tòna soilleir a’ chraicinn',
    '🏄🏼' => 'neach a’ surfadh: tòna meadhanach soilleir a’ chraicinn',
    '🏄🏽' => 'neach a’ surfadh: tòna meadhanach a’ chraicinn',
    '🏄🏾' => 'neach a’ surfadh: tòna meadhanach doilleir a’ chraicinn',
    '🏄🏿' => 'neach a’ surfadh: tòna doilleir a’ chraicinn',
    '🚣🏻' => 'neach ag iomradh bàta: tòna soilleir a’ chraicinn',
    '🚣🏼' => 'neach ag iomradh bàta: tòna meadhanach soilleir a’ chraicinn',
    '🚣🏽' => 'neach ag iomradh bàta: tòna meadhanach a’ chraicinn',
    '🚣🏾' => 'neach ag iomradh bàta: tòna meadhanach doilleir a’ chraicinn',
    '🚣🏿' => 'neach ag iomradh bàta: tòna doilleir a’ chraicinn',
    '🏊🏻' => 'neach a’ snàmh: tòna soilleir a’ chraicinn',
    '🏊🏼' => 'neach a’ snàmh: tòna meadhanach soilleir a’ chraicinn',
    '🏊🏽' => 'neach a’ snàmh: tòna meadhanach a’ chraicinn',
    '🏊🏾' => 'neach a’ snàmh: tòna meadhanach doilleir a’ chraicinn',
    '🏊🏿' => 'neach a’ snàmh: tòna doilleir a’ chraicinn',
    '⛹🏻' => 'neach a’ bocadh bàla: tòna soilleir a’ chraicinn',
    '⛹🏼' => 'neach a’ bocadh bàla: tòna meadhanach soilleir a’ chraicinn',
    '⛹🏽' => 'neach a’ bocadh bàla: tòna meadhanach a’ chraicinn',
    '⛹🏾' => 'neach a’ bocadh bàla: tòna meadhanach doilleir a’ chraicinn',
    '⛹🏿' => 'neach a’ bocadh bàla: tòna doilleir a’ chraicinn',
    '🏋🏻' => 'neach a’ togail chuideaman: tòna soilleir a’ chraicinn',
    '🏋🏼' => 'neach a’ togail chuideaman: tòna meadhanach soilleir a’ chraicinn',
    '🏋🏽' => 'neach a’ togail chuideaman: tòna meadhanach a’ chraicinn',
    '🏋🏾' => 'neach a’ togail chuideaman: tòna meadhanach doilleir a’ chraicinn',
    '🏋🏿' => 'neach a’ togail chuideaman: tòna doilleir a’ chraicinn',
    '🚴🏻' => 'neach ri rothaireachd: tòna soilleir a’ chraicinn',
    '🚴🏼' => 'neach ri rothaireachd: tòna meadhanach soilleir a’ chraicinn',
    '🚴🏽' => 'neach ri rothaireachd: tòna meadhanach a’ chraicinn',
    '🚴🏾' => 'neach ri rothaireachd: tòna meadhanach doilleir a’ chraicinn',
    '🚴🏿' => 'neach ri rothaireachd: tòna doilleir a’ chraicinn',
    '🚵🏻' => 'neach ri rothaireachd bheanntan: tòna soilleir a’ chraicinn',
    '🚵🏼' => 'neach ri rothaireachd bheanntan: tòna meadhanach soilleir a’ chraicinn',
    '🚵🏽' => 'neach ri rothaireachd bheanntan: tòna meadhanach a’ chraicinn',
    '🚵🏾' => 'neach ri rothaireachd bheanntan: tòna meadhanach doilleir a’ chraicinn',
    '🚵🏿' => 'neach ri rothaireachd bheanntan: tòna doilleir a’ chraicinn',
    '🤸🏻' => 'neach a’ cur nan rothan: tòna soilleir a’ chraicinn',
    '🤸🏼' => 'neach a’ cur nan rothan: tòna meadhanach soilleir a’ chraicinn',
    '🤸🏽' => 'neach a’ cur nan rothan: tòna meadhanach a’ chraicinn',
    '🤸🏾' => 'neach a’ cur nan rothan: tòna meadhanach doilleir a’ chraicinn',
    '🤸🏿' => 'neach a’ cur nan rothan: tòna doilleir a’ chraicinn',
    '🤼🏻' => 'daoine ri gleacadh: tòna soilleir a’ chraicinn',
    '🤼🏼' => 'daoine ri gleacadh: tòna meadhanach soilleir a’ chraicinn',
    '🤼🏽' => 'daoine ri gleacadh: tòna meadhanach a’ chraicinn',
    '🤼🏾' => 'daoine ri gleacadh: tòna meadhanach doilleir a’ chraicinn',
    '🤼🏿' => 'daoine ri gleacadh: tòna doilleir a’ chraicinn',
    '🤽🏻' => 'neach a’ cluich polo-uisge: tòna soilleir a’ chraicinn',
    '🤽🏼' => 'neach a’ cluich polo-uisge: tòna meadhanach soilleir a’ chraicinn',
    '🤽🏽' => 'neach a’ cluich polo-uisge: tòna meadhanach a’ chraicinn',
    '🤽🏾' => 'neach a’ cluich polo-uisge: tòna meadhanach doilleir a’ chraicinn',
    '🤽🏿' => 'neach a’ cluich polo-uisge: tòna doilleir a’ chraicinn',
    '🤾🏻' => 'neach a’ cluich ball-làimhe: tòna soilleir a’ chraicinn',
    '🤾🏼' => 'neach a’ cluich ball-làimhe: tòna meadhanach soilleir a’ chraicinn',
    '🤾🏽' => 'neach a’ cluich ball-làimhe: tòna meadhanach a’ chraicinn',
    '🤾🏾' => 'neach a’ cluich ball-làimhe: tòna meadhanach doilleir a’ chraicinn',
    '🤾🏿' => 'neach a’ cluich ball-làimhe: tòna doilleir a’ chraicinn',
    '🤹🏻' => 'neach ri caisreabhachd: tòna soilleir a’ chraicinn',
    '🤹🏼' => 'neach ri caisreabhachd: tòna meadhanach soilleir a’ chraicinn',
    '🤹🏽' => 'neach ri caisreabhachd: tòna meadhanach a’ chraicinn',
    '🤹🏾' => 'neach ri caisreabhachd: tòna meadhanach doilleir a’ chraicinn',
    '🤹🏿' => 'neach ri caisreabhachd: tòna doilleir a’ chraicinn',
    '🧘🏻' => 'neach na shuidhe lòtais: tòna soilleir a’ chraicinn',
    '🧘🏼' => 'neach na shuidhe lòtais: tòna meadhanach soilleir a’ chraicinn',
    '🧘🏽' => 'neach na shuidhe lòtais: tòna meadhanach a’ chraicinn',
    '🧘🏾' => 'neach na shuidhe lòtais: tòna meadhanach doilleir a’ chraicinn',
    '🧘🏿' => 'neach na shuidhe lòtais: tòna doilleir a’ chraicinn',
    '🛀🏻' => 'neach san amar: tòna soilleir a’ chraicinn',
    '🛀🏼' => 'neach san amar: tòna meadhanach soilleir a’ chraicinn',
    '🛀🏽' => 'neach san amar: tòna meadhanach a’ chraicinn',
    '🛀🏾' => 'neach san amar: tòna meadhanach doilleir a’ chraicinn',
    '🛀🏿' => 'neach san amar: tòna doilleir a’ chraicinn',
    '🛌🏻' => 'neach san leabaidh: tòna soilleir a’ chraicinn',
    '🛌🏼' => 'neach san leabaidh: tòna meadhanach soilleir a’ chraicinn',
    '🛌🏽' => 'neach san leabaidh: tòna meadhanach a’ chraicinn',
    '🛌🏾' => 'neach san leabaidh: tòna meadhanach doilleir a’ chraicinn',
    '🛌🏿' => 'neach san leabaidh: tòna doilleir a’ chraicinn',
    '👭🏻' => 'boireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn',
    '👭🏼' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn',
    '👭🏽' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn',
    '👭🏾' => 'boireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn',
    '👭🏿' => 'boireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn',
    '👫🏻' => 'tè is fear a’ gabhail làmhan a chèile: tòna soilleir a’ chraicinn',
    '👫🏼' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach soilleir a’ chraicinn',
    '👫🏽' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach a’ chraicinn',
    '👫🏾' => 'tè is fear a’ gabhail làmhan a chèile: tòna meadhanach doilleir a’ chraicinn',
    '👫🏿' => 'tè is fear a’ gabhail làmhan a chèile: tòna doilleir a’ chraicinn',
    '👬🏻' => 'fireannaich a’ glacadh air làmh a chèile: tòna soilleir a’ chraicinn',
    '👬🏼' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach soilleir a’ chraicinn',
    '👬🏽' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach a’ chraicinn',
    '👬🏾' => 'fireannaich a’ glacadh air làmh a chèile: tòna meadhanach doilleir a’ chraicinn',
    '👬🏿' => 'fireannaich a’ glacadh air làmh a chèile: tòna doilleir a’ chraicinn',
    '💏🏻' => 'pòg: tòna soilleir a’ chraicinn',
    '💏🏼' => 'pòg: tòna meadhanach soilleir a’ chraicinn',
    '💏🏽' => 'pòg: tòna meadhanach a’ chraicinn',
    '💏🏾' => 'pòg: tòna meadhanach doilleir a’ chraicinn',
    '💏🏿' => 'pòg: tòna doilleir a’ chraicinn',
    '💑🏻' => 'càraid le cridhe: tòna soilleir a’ chraicinn',
    '💑🏼' => 'càraid le cridhe: tòna meadhanach soilleir a’ chraicinn',
    '💑🏽' => 'càraid le cridhe: tòna meadhanach a’ chraicinn',
    '💑🏾' => 'càraid le cridhe: tòna meadhanach doilleir a’ chraicinn',
    '💑🏿' => 'càraid le cridhe: tòna doilleir a’ chraicinn',
    '🇦🇨' => 'bratach: Eilean na Deasgabhalach',
    '🇦🇩' => 'bratach: Andorra',
    '🇦🇪' => 'bratach: Na h-Iomaratan Arabach Aonaichte',
    '🇦🇫' => 'bratach: Afghanastàn',
    '🇦🇬' => 'bratach: Aintìoga is Barbuda',
    '🇦🇮' => 'bratach: Anguillia',
    '🇦🇱' => 'bratach: Albàinia',
    '🇦🇲' => 'bratach: Airmeinea',
    '🇦🇴' => 'bratach: Angòla',
    '🇦🇶' => 'bratach: An Antartaig',
    '🇦🇷' => 'bratach: An Argantain',
    '🇦🇸' => 'bratach: Samotha na h-Aimeireaga',
    '🇦🇹' => 'bratach: An Ostair',
    '🇦🇺' => 'bratach: Astràilia',
    '🇦🇼' => 'bratach: Arùba',
    '🇦🇽' => 'bratach: Na h-Eileanan Åland',
    '🇦🇿' => 'bratach: Asarbaideàn',
    '🇧🇦' => 'bratach: Bosna is Hearsagobhana',
    '🇧🇧' => 'bratach: Barbados',
    '🇧🇩' => 'bratach: Bangladais',
    '🇧🇪' => 'bratach: A’ Bheilg',
    '🇧🇫' => 'bratach: Buirciona Faso',
    '🇧🇬' => 'bratach: A’ Bhulgair',
    '🇧🇭' => 'bratach: Bachrain',
    '🇧🇮' => 'bratach: Burundaidh',
    '🇧🇯' => 'bratach: Beinin',
    '🇧🇱' => 'bratach: Saint Barthélemy',
    '🇧🇲' => 'bratach: Bearmùda',
    '🇧🇳' => 'bratach: Brùnaigh',
    '🇧🇴' => 'bratach: Boilibhia',
    '🇧🇶' => 'bratach: Na Tìrean Ìsle Caraibeach',
    '🇧🇷' => 'bratach: Braisil',
    '🇧🇸' => 'bratach: Na h-Eileanan Bhathama',
    '🇧🇹' => 'bratach: Butàn',
    '🇧🇻' => 'bratach: Eilean Bouvet',
    '🇧🇼' => 'bratach: Botsuana',
    '🇧🇾' => 'bratach: A’ Bhealaruis',
    '🇧🇿' => 'bratach: A’ Bheilìs',
    '🇨🇦' => 'bratach: Canada',
    '🇨🇨' => 'bratach: Na h-Eileanan Chocos (Keeling)',
    '🇨🇩' => 'bratach: Congo - Kinshasa',
    '🇨🇫' => 'bratach: Poblachd Meadhan Afraga',
    '🇨🇬' => 'bratach: A’ Chongo - Brazzaville',
    '🇨🇭' => 'bratach: An Eilbheis',
    '🇨🇮' => 'bratach: Côte d’Ivoire',
    '🇨🇰' => 'bratach: Eileanan Cook',
    '🇨🇱' => 'bratach: An t-Sile',
    '🇨🇲' => 'bratach: Camarun',
    '🇨🇳' => 'bratach: An t-Sìn',
    '🇨🇴' => 'bratach: Coloimbia',
    '🇨🇵' => 'bratach: Eilean Clipperton',
    '🇨🇶' => 'bratach: Sarc',
    '🇨🇷' => 'bratach: Costa Rìcea',
    '🇨🇺' => 'bratach: Cùba',
    '🇨🇻' => 'bratach: An Ceap Uaine',
    '🇨🇼' => 'bratach: Curaçao',
    '🇨🇽' => 'bratach: Eilean na Nollaig',
    '🇨🇾' => 'bratach: Cìopras',
    '🇨🇿' => 'bratach: An t-Seic',
    '🇩🇪' => 'bratach: A’ Ghearmailt',
    '🇩🇬' => 'bratach: Diego Garcia',
    '🇩🇯' => 'bratach: Diobùtaidh',
    '🇩🇰' => 'bratach: An Danmhairg',
    '🇩🇲' => 'bratach: Doiminicea',
    '🇩🇴' => 'bratach: A’ Phoblachd Dhoiminiceach',
    '🇩🇿' => 'bratach: Aildiria',
    '🇪🇦' => 'bratach: Ceuta agus Melilla',
    '🇪🇨' => 'bratach: Eacuador',
    '🇪🇪' => 'bratach: An Eastoin',
    '🇪🇬' => 'bratach: An Èipheit',
    '🇪🇭' => 'bratach: Sathara an Iar',
    '🇪🇷' => 'bratach: Eartra',
    '🇪🇸' => 'bratach: An Spàinnt',
    '🇪🇹' => 'bratach: An Itiop',
    '🇪🇺' => 'bratach: An t-Aonadh Eòrpach',
    '🇫🇮' => 'bratach: An Fhionnlann',
    '🇫🇯' => 'bratach: Fìdi',
    '🇫🇰' => 'bratach: Na h-Eileanan Fàclannach',
    '🇫🇲' => 'bratach: Na Meanbh-eileanan',
    '🇫🇴' => 'bratach: Na h-Eileanan Fàro',
    '🇫🇷' => 'bratach: An Fhraing',
    '🇬🇦' => 'bratach: Gabon',
    '🇬🇧' => 'bratach: An Rìoghachd Aonaichte',
    '🇬🇩' => 'bratach: Greanàda',
    '🇬🇪' => 'bratach: A’ Chairtbheil',
    '🇬🇫' => 'bratach: Guidheàna na Frainge',
    '🇬🇬' => 'bratach: Geàrnsaidh',
    '🇬🇭' => 'bratach: Gàna',
    '🇬🇮' => 'bratach: Diobraltar',
    '🇬🇱' => 'bratach: A’ Ghraonlann',
    '🇬🇲' => 'bratach: A’ Ghaimbia',
    '🇬🇳' => 'bratach: Gini',
    '🇬🇵' => 'bratach: Guadalup',
    '🇬🇶' => 'bratach: Gini Mheadhan-Chriosach',
    '🇬🇷' => 'bratach: A’ Ghreug',
    '🇬🇸' => 'bratach: Seòirsea a Deas is na h-Eileanan Sandwich a Deas',
    '🇬🇹' => 'bratach: Guatamala',
    '🇬🇺' => 'bratach: Guam',
    '🇬🇼' => 'bratach: Gini-Bioso',
    '🇬🇾' => 'bratach: Guidheàna',
    '🇭🇰' => 'bratach: Hong Kong SAR na Sìne',
    '🇭🇲' => 'bratach: Eilean Heard is Eileanan MhicDhòmhnaill',
    '🇭🇳' => 'bratach: Hondùras',
    '🇭🇷' => 'bratach: A’ Chròthais',
    '🇭🇹' => 'bratach: Haidhti',
    '🇭🇺' => 'bratach: An Ungair',
    '🇮🇨' => 'bratach: Na h-Eileanan Canàrach',
    '🇮🇩' => 'bratach: Na h-Innd-innse',
    '🇮🇪' => 'bratach: Èirinn',
    '🇮🇱' => 'bratach: Iosrael',
    '🇮🇲' => 'bratach: Eilean Mhanainn',
    '🇮🇳' => 'bratach: Na h-Innseachan',
    '🇮🇴' => 'bratach: Ranntair Breatannach Cuan nan Innseachan',
    '🇮🇶' => 'bratach: Ioràc',
    '🇮🇷' => 'bratach: Ioràn',
    '🇮🇸' => 'bratach: Innis Tìle',
    '🇮🇹' => 'bratach: An Eadailt',
    '🇯🇪' => 'bratach: Deàrsaidh',
    '🇯🇲' => 'bratach: Diameuga',
    '🇯🇴' => 'bratach: Iòrdan',
    '🇯🇵' => 'bratach: An t-Seapan',
    '🇰🇪' => 'bratach: Ceinia',
    '🇰🇬' => 'bratach: Cìorgastan',
    '🇰🇭' => 'bratach: Cambuidea',
    '🇰🇮' => 'bratach: Ciribeas',
    '🇰🇲' => 'bratach: Comoros',
    '🇰🇳' => 'bratach: Naomh Crìstean is Nibheis',
    '🇰🇵' => 'bratach: Coirèa a Tuath',
    '🇰🇷' => 'bratach: Coirèa',
    '🇰🇼' => 'bratach: Cuibhèit',
    '🇰🇾' => 'bratach: Na h-Eileanan Caimean',
    '🇰🇿' => 'bratach: Casachstàn',
    '🇱🇦' => 'bratach: Làthos',
    '🇱🇧' => 'bratach: Leabanon',
    '🇱🇨' => 'bratach: Naomh Lùisea',
    '🇱🇮' => 'bratach: Lichtenstein',
    '🇱🇰' => 'bratach: Sri Lanca',
    '🇱🇷' => 'bratach: Libèir',
    '🇱🇸' => 'bratach: Leasoto',
    '🇱🇹' => 'bratach: An Liotuain',
    '🇱🇺' => 'bratach: Lugsamburg',
    '🇱🇻' => 'bratach: An Laitbhe',
    '🇱🇾' => 'bratach: Libia',
    '🇲🇦' => 'bratach: Moroco',
    '🇲🇨' => 'bratach: Monaco',
    '🇲🇩' => 'bratach: A’ Mholdobha',
    '🇲🇪' => 'bratach: Am Monadh Neagrach',
    '🇲🇫' => 'bratach: Naomh Màrtainn',
    '🇲🇬' => 'bratach: Madagasgar',
    '🇲🇭' => 'bratach: Eileanan Mharshall',
    '🇲🇰' => 'bratach: A’ Mhasadon a Tuath',
    '🇲🇱' => 'bratach: Màili',
    '🇲🇲' => 'bratach: Miànmar',
    '🇲🇳' => 'bratach: Dùthaich nam Mongol',
    '🇲🇴' => 'bratach: Macàthu SAR na Sìne',
    '🇲🇵' => 'bratach: Na h-Eileanan Mairianach a Tuath',
    '🇲🇶' => 'bratach: Mairtinic',
    '🇲🇷' => 'bratach: Moratàinea',
    '🇲🇸' => 'bratach: Montsarat',
    '🇲🇹' => 'bratach: Malta',
    '🇲🇺' => 'bratach: Na h-Eileanan Mhoiriseas',
    '🇲🇻' => 'bratach: Na h-Eileanan Mhaladaibh',
    '🇲🇼' => 'bratach: Malabhaidh',
    '🇲🇽' => 'bratach: Meagsago',
    '🇲🇾' => 'bratach: Malaidhsea',
    '🇲🇿' => 'bratach: Mòsaimbic',
    '🇳🇦' => 'bratach: An Namaib',
    '🇳🇨' => 'bratach: Cailleann Nuadh',
    '🇳🇪' => 'bratach: Nìgeir',
    '🇳🇫' => 'bratach: Eilean Norfolk',
    '🇳🇬' => 'bratach: Nigèiria',
    '🇳🇮' => 'bratach: Niocaragua',
    '🇳🇱' => 'bratach: Na Tìrean Ìsle',
    '🇳🇴' => 'bratach: Nirribhidh',
    '🇳🇵' => 'bratach: Neapàl',
    '🇳🇷' => 'bratach: Nabhru',
    '🇳🇺' => 'bratach: Niue',
    '🇳🇿' => 'bratach: Sealainn Nuadh',
    '🇴🇲' => 'bratach: Omàn',
    '🇵🇦' => 'bratach: Panama',
    '🇵🇪' => 'bratach: Pearù',
    '🇵🇫' => 'bratach: Poilinèis na Frainge',
    '🇵🇬' => 'bratach: Gini Nuadh Phaputhach',
    '🇵🇭' => 'bratach: Na h-Eileanan Filipineach',
    '🇵🇰' => 'bratach: Pagastàn',
    '🇵🇱' => 'bratach: A’ Phòlainn',
    '🇵🇲' => 'bratach: Saint Pierre agus Miquelon',
    '🇵🇳' => 'bratach: Eileanan Pheit a’ Chàirn',
    '🇵🇷' => 'bratach: Porto Rìceo',
    '🇵🇸' => 'bratach: Ùghdarras nam Palastaineach',
    '🇵🇹' => 'bratach: A’ Phortagail',
    '🇵🇼' => 'bratach: Palabh',
    '🇵🇾' => 'bratach: Paraguaidh',
    '🇶🇦' => 'bratach: Catar',
    '🇷🇪' => 'bratach: Réunion',
    '🇷🇴' => 'bratach: Romàinia',
    '🇷🇸' => 'bratach: An t-Sèirb',
    '🇷🇺' => 'bratach: An Ruis',
    '🇷🇼' => 'bratach: Rubhanda',
    '🇸🇦' => 'bratach: Aràibia nan Sabhd',
    '🇸🇧' => 'bratach: Eileanan Sholaimh',
    '🇸🇨' => 'bratach: Na h-Eileanan Sheiseall',
    '🇸🇩' => 'bratach: Sudàn',
    '🇸🇪' => 'bratach: An t-Suain',
    '🇸🇬' => 'bratach: Singeapòr',
    '🇸🇭' => 'bratach: Eilean Naomh Eilidh',
    '🇸🇮' => 'bratach: An t-Slòbhain',
    '🇸🇯' => 'bratach: Svalbard is Jan Mayen',
    '🇸🇰' => 'bratach: An t-Slòbhac',
    '🇸🇱' => 'bratach: Siarra Leòmhann',
    '🇸🇲' => 'bratach: San Marino',
    '🇸🇳' => 'bratach: Seanagal',
    '🇸🇴' => 'bratach: Somàilia',
    '🇸🇷' => 'bratach: Suranam',
    '🇸🇸' => 'bratach: Sudàn a Deas',
    '🇸🇹' => 'bratach: São Tomé agus Príncipe',
    '🇸🇻' => 'bratach: An Salbhador',
    '🇸🇽' => 'bratach: Sint Maarten',
    '🇸🇾' => 'bratach: Siridhea',
    '🇸🇿' => 'bratach: eSwatini',
    '🇹🇦' => 'bratach: Tristan da Cunha',
    '🇹🇨' => 'bratach: Na h-Eileanan Turcach is Caiceo',
    '🇹🇩' => 'bratach: An t-Seàd',
    '🇹🇫' => 'bratach: Ranntairean a Deas na Frainge',
    '🇹🇬' => 'bratach: Togo',
    '🇹🇭' => 'bratach: Dùthaich nan Tàidh',
    '🇹🇯' => 'bratach: Taidigeastàn',
    '🇹🇰' => 'bratach: Tokelau',
    '🇹🇱' => 'bratach: Timor-Leste',
    '🇹🇲' => 'bratach: Turcmanastàn',
    '🇹🇳' => 'bratach: Tuinisea',
    '🇹🇴' => 'bratach: Tonga',
    '🇹🇷' => 'bratach: An Tuirc',
    '🇹🇹' => 'bratach: Trianaid agus Tobago',
    '🇹🇻' => 'bratach: Tubhalu',
    '🇹🇼' => 'bratach: Taidh-Bhàn',
    '🇹🇿' => 'bratach: An Tansan',
    '🇺🇦' => 'bratach: An Ucràin',
    '🇺🇬' => 'bratach: Uganda',
    '🇺🇲' => 'bratach: Meanbh-Eileanan Iomallach nan SA',
    '🇺🇳' => 'bratach: Na Dùthchannan Aonaichte',
    '🇺🇸' => 'bratach: Na Stàitean Aonaichte',
    '🇺🇾' => 'bratach: Uruguaidh',
    '🇺🇿' => 'bratach: Usbagastàn',
    '🇻🇦' => 'bratach: Cathair na Bhatacain',
    '🇻🇨' => 'bratach: Naomh Bhionsant agus Eileanan Greanadach',
    '🇻🇪' => 'bratach: A’ Bheiniseala',
    '🇻🇬' => 'bratach: Eileanan Breatannach na Maighdinn',
    '🇻🇮' => 'bratach: Eileanan na Maighdinn aig na SA',
    '🇻🇳' => 'bratach: Bhiet-Nam',
    '🇻🇺' => 'bratach: Vanuatu',
    '🇼🇫' => 'bratach: Uallas agus Futuna',
    '🇼🇸' => 'bratach: Samotha',
    '🇽🇰' => 'bratach: A’ Chosobho',
    '🇾🇪' => 'bratach: An Eaman',
    '🇾🇹' => 'bratach: Mayotte',
    '🇿🇦' => 'bratach: Afraga a Deas',
    '🇿🇲' => 'bratach: Sàimbia',
    '🇿🇼' => 'bratach: An t-Sìombab',
    '🔟' => 'còmhdachadh iuchrach: 10',
    '🏻' => 'tòna soilleir a’ chraicinn',
    '🏼' => 'tòna meadhanach soilleir a’ chraicinn',
    '🏽' => 'tòna meadhanach a’ chraicinn',
    '🏾' => 'tòna meadhanach doilleir a’ chraicinn',
    '🏿' => 'tòna doilleir a’ chraicinn',
    '😀' => 'aodann le gàire',
    '😃' => 'aodann le gàire is sùilean fosgailte',
    '😄' => 'aodann le gàire na shùilean',
    '😁' => 'aodann le gàire mhòr',
    '😆' => 'aodann caog-shùileach le gàire',
    '😅' => 'aodann le gàire a’ cur fallas',
    '🤣' => 'a’ ruidhleadh air an làr a’ gàireachdainn',
    '😂' => 'aodann le deòir shona',
    '🙂' => 'aodann le fiamh gàire beag',
    '🙃' => 'aodann bun os cionn',
    '🫠' => 'aodann a’ leaghadh',
    '😉' => 'aodann le sùil bheag',
    '😊' => 'aodann le gàire agus sùilean toilichte',
    '😇' => 'aodann le gàire agus riomball',
    '🥰' => 'aodann le fiamh-ghàire is cridheachan',
    '😍' => 'aodann le gàire ⁊ sùilean nan cridheachan',
    '🤩' => 'aodann toilichte le rionnagan na shùilean',
    '😘' => 'aodann a’ caitheamh pòg',
    '😗' => 'aodann pòige',
    '☺' => 'aodann le fiamh-ghàire',
    '😚' => 'aodann a’ pògadh le sùilean dùinte',
    '😙' => 'aodann a’ pògadh le sùilean toilichte',
    '🥲' => 'aodann le fiamh-ghàire is deur',
    '😋' => 'aodann ag ithe biadh blasta',
    '😛' => 'aodann a’ cur a-mach na teanga',
    '😜' => 'aodann le theanga a-mach ⁊ sùil bheag',
    '🤪' => 'aodann às a rian',
    '😝' => 'aodann caog-shùileach le theanga a-mach',
    '🤑' => 'aodann le sùilean is teanga airgid',
    '🤗' => 'aodann a’ pasgadh gàirdeanan timcheall',
    '🤭' => 'aodann le làmh air a beul',
    '🫢' => 'aodann le sùilean fosgailte ⁊ làmh air beul',
    '🫣' => 'aodann le sùil a’ dìdeadh',
    '🤫' => 'aodann “Ist!”',
    '🤔' => 'aodann ri smaoineachadh',
    '🫡' => 'aodann ag altachadh',
    '🤐' => 'aodann le bilean seulaichte',
    '🤨' => 'aodann le mala togta',
    '😐' => 'aodann gun fhiamh',
    '😑' => 'aodann gun fhaireachdainn',
    '😶' => 'aodann gun bheul',
    '🫥' => 'aodann le loidhne dhotagach',
    '😏' => 'aodann le plìonas',
    '😒' => 'aodann nuair nach eil e èibhinn',
    '🙄' => 'aodann a’ casadh nan sùilean',
    '😬' => 'aodann le drèin air',
    '🤥' => 'aodann nam breug',
    '🫨' => 'aodann le crathadh air',
    '😌' => 'aodann a fhuair faochadh',
    '😔' => 'aodann fo throm-smuain',
    '😪' => 'aodann cadalach',
    '🤤' => 'aodann a’ sileadh sglongaid',
    '😴' => 'aodann na chadal',
    '🫩' => 'aodann le buaile mu shùilean',
    '😷' => 'aodann le masg dotair air',
    '🤒' => 'aodann le teas-mheidh',
    '🤕' => 'aodann le clobhd air a’ cheann',
    '🤢' => 'aodann sgreamha',
    '🤮' => 'aodann a’ cur a-mach',
    '🤧' => 'aodann ri sreothairt',
    '🥵' => 'aodann teth',
    '🥶' => 'aodann fuar',
    '🥴' => 'aodann misgeach',
    '😵' => 'aodann na thuaineal',
    '🤯' => 'ceann a’ spreadhadh',
    '🤠' => 'aodann le ad cowboy',
    '🥳' => 'aodann aig pàrtaidh',
    '🥸' => 'aodann le riochd air',
    '😎' => 'aodann toilichte le speuclairean-grèine',
    '🤓' => 'aodann gìog',
    '🧐' => 'aodann le leth-speuclair',
    '😕' => 'aodann tro chèile',
    '🫤' => 'aodann le beul trastanach',
    '😟' => 'aodann fo dhragh',
    '🙁' => 'aodann le fiamh drèin beag',
    '☹' => 'aodann le fiamh drèin',
    '😮' => 'aodann le beul fosgailte',
    '😯' => 'aodann iongnaidh le sùilean fosgailte',
    '😲' => 'aodann a ghabh mòr-iongnadh',
    '😳' => 'aodann air ruadhadh',
    '🫪' => 'aodann mì-dhealbhaichte',
    '🥺' => 'aodann tagrach',
    '🥹' => 'aodann le deòir na sùil',
    '😦' => 'aodann le drèin is beul fosgailte',
    '😧' => 'aodann àmhghair',
    '😨' => 'aodann fo eagal',
    '😰' => 'aodann le beul fosgailte a’ cur fallas',
    '😥' => 'aodann a fhuair briseadh-dùil ⁊ faochadh',
    '😢' => 'aodann a’ caoineadh',
    '😭' => 'aodann a’ rànaich',
    '😱' => 'aodann a’ leigeil sgreuch eagail',
    '😖' => 'aodann am breislich',
    '😣' => 'aodann rag',
    '😞' => 'aodann a fhuair briseadh-dùil',
    '😓' => 'aodann a’ cur fallas fuar',
    '😩' => 'aodann air chlaoidh',
    '😫' => 'aodann sgìth',
    '🥱' => 'aodann a’ mèananaich',
    '😤' => 'aodann le toit às a chuinneanan',
    '😡' => 'aodann busach',
    '😠' => 'aodann feargach',
    '🤬' => 'aodann le samhlaidhean air a beul',
    '😈' => 'aodann le gàire agus adharcan',
    '👿' => 'aodann feargach le adharcan',
    '💀' => 'claigeann',
    '☠' => 'claigeann le cnàmhan',
    '💩' => 'càrn cac',
    '🤡' => 'aodann tuaisteir',
    '👹' => 'diabhal Seapanach',
    '👺' => 'bòcan Seapanach',
    '👻' => 'taibhse',
    '👽' => 'creutair neo-thalmhaidh',
    '👾' => 'uilebheist neo-thalmhaidh',
    '🤖' => 'robotair',
    '😺' => 'cat le gàire',
    '😸' => 'cat le gàire agus sùilean toilichte',
    '😹' => 'cat le deòir shona',
    '😻' => 'cat le gàire agus sùilean an cumadh cridhe',
    '😼' => 'cat le gàire cham',
    '😽' => 'cat a’ pògadh',
    '🙀' => 'cat air chlaoidh',
    '😿' => 'cat a’ caoineadh',
    '😾' => 'cat busach',
    '🙈' => 'muncaidh nach fhaic olc',
    '🙉' => 'muncaidh nach cluinn olc',
    '🙊' => 'muncaidh nach can olc',
    '💌' => 'litir gaoil',
    '💘' => 'cridhe le saighead',
    '💝' => 'cridhe le ribean',
    '💖' => 'cridhe lainnireach',
    '💗' => 'cridhe a’ fàs',
    '💓' => 'cridhe a’ bualadh',
    '💞' => 'cridheachan a’ dol mun cuairt',
    '💕' => 'dà chridhe',
    '💟' => 'sgeadas cridhe',
    '❣' => 'clisg-phuing cridhe',
    '💔' => 'cridhe briste',
    '❤' => 'cridhe dearg',
    '🩷' => 'crithe pinc',
    '🧡' => 'cridhe orains',
    '💛' => 'cridhe buidhe',
    '💚' => 'cridhe uaine',
    '💙' => 'cridhe gorm',
    '🩵' => 'cridhe solleir-ghorm',
    '💜' => 'cridhe purpaidh',
    '🤎' => 'cridhe donn',
    '🖤' => 'cridhe dubh',
    '🩶' => 'cridhe glas',
    '🤍' => 'cridhe geal',
    '💋' => 'làrach pòige',
    '💯' => 'ceud puing',
    '💢' => 'samhla feirge',
    '🫯' => 'neul sabaide',
    '💥' => 'bualadh ri chèile',
    '💫' => 'tuaineal',
    '💦' => 'boinneagan fallais',
    '💨' => 'a’ falbh nad dheann',
    '🕳' => 'toll',
    '💬' => 'builgean còmhraidh',
    '🗨' => 'builgean còmhraidh dhan taobh chlì',
    '🗯' => 'builgean feirg dhan taobh deas',
    '💭' => 'builgean smaoineachaidh',
    '💤' => 'suain',
    '👋' => 'làmh a’ smèideadh',
    '🤚' => 'cùl làimhe air a thogail',
    '🖐' => 'làmh air a togail le corragan sìnte',
    '✋' => 'làmh air a togail',
    '🖖' => 'beannachd muinntir Vulcan',
    '🫱' => 'làmh ris an taobh deas',
    '🫲' => 'làmh ris an taobh chlì',
    '🫳' => 'làmh le a bas sìos',
    '🫴' => 'làmh le a bas suas',
    '🫷' => 'làmh a’ putadh dhan taobh chlì',
    '🫸' => 'làmh a’ putadh dhan taobh dheas',
    '👌' => 'samhla làimhe OK',
    '🤌' => 'crudhagan',
    '🤏' => 'làmh le sradag',
    '✌' => 'làmh a thug buaidh',
    '🤞' => 'beò an dòchas',
    '🫰' => 'làmh le a colgag is òrdag ri chèile',
    '🤟' => 'gluasad “Tha gaol agam ort”',
    '🤘' => 'samhla nan adharc',
    '🤙' => 'làmh a’ comharradh “Cuir fòn thugam”',
    '👈' => 'druim na làimhe, colgag a’ tomhadh clì',
    '👉' => 'druim na làimhe, colgag a’ tomhadh deas',
    '👆' => 'druim na làimhe, colgag a’ tomhadh suas',
    '🖕' => 'Fionnlagh Fada / Màiri Fhada',
    '👇' => 'druim na làimhe, colgag a’ tomhadh sìos',
    '☝' => 'colgag a’ tomhadh suas',
    '🫵' => 'colgag a’ tomhadh ort',
    '👍' => 'math fhèin',
    '👎' => 'mo chreach',
    '✊' => 'dòrn air a thogail',
    '👊' => 'dòrn a’ tighinn thugad',
    '🤛' => 'dòrn ris an taobh chlì',
    '🤜' => 'dòrn ris an taobh deas',
    '👏' => 'bualadh bhasan',
    '🙌' => 'a’ togail làmhan',
    '🫶' => 'dà làimh ann an cumadh cridhe',
    '👐' => 'làmhan fosgailte',
    '🤲' => 'an dà bhois suas còmhla',
    '🤝' => 'crathadh-làimhe',
    '🙏' => 'làmhan paisgte',
    '✍' => 'làmh a’ sgrìobhadh',
    '💅' => 'lìomh ìnean',
    '🤳' => 'fèineag',
    '💪' => 'giorrachadh fèithean dà-cheannach',
    '🦾' => 'gàirdean meacanaigeach',
    '🦿' => 'cas mheacanaigeach',
    '🦵' => 'sliasaid',
    '🦶' => 'cas',
    '👂' => 'cluas',
    '🦻' => 'cluas le claisteachan',
    '👃' => 'sròn',
    '🧠' => 'eanchainn',
    '🫀' => 'cridhe',
    '🫁' => 'sgamhan',
    '🦷' => 'fiacaill',
    '🦴' => 'cnàimh',
    '👀' => 'sùilean',
    '👁' => 'sùil',
    '👅' => 'teanga',
    '👄' => 'beul',
    '🫦' => 'bile ga bìdeach',
    '👶' => 'naoidhean',
    '🧒' => 'duine-cloinne',
    '👦' => 'gille',
    '👧' => 'caileag',
    '🧑' => 'inbheach',
    '👱' => 'neach le falt bàn',
    '👨' => 'duine',
    '🧔' => 'duine le feusag',
    '👩' => 'boireannach',
    '🧓' => 'seann-inbheach',
    '👴' => 'bodach',
    '👵' => 'cailleach',
    '🙍' => 'neach le drèin',
    '🙎' => 'neach le bus air',
    '🙅' => 'neach a’ diùltadh rud',
    '🙆' => 'neach ag aontachadh ri rud',
    '💁' => 'neach le làmh sìnte, bas air uachdar',
    '🙋' => 'neach a’ togail a làmh',
    '🧏' => 'neach bodhar',
    '🙇' => 'neach a’ cromadh',
    '🤦' => 'neach le aodann na làmhan',
    '🤷' => 'neach a’ druiteadh',
    '👮' => 'oifigear poileis',
    '🕵' => 'lorgaire',
    '💂' => 'geàrd',
    '🥷' => 'ninja',
    '👷' => 'neach-togail',
    '🫅' => 'neach le crùn air',
    '🤴' => 'prionnsa',
    '👸' => 'bana-phrionnsa',
    '👳' => 'neach le turban air',
    '👲' => 'duine le currac Sìneach',
    '🧕' => 'boireannach le brèid oirre',
    '🤵' => 'neach ann an tuxedo',
    '👰' => 'neach le brat-gnùise',
    '🤰' => 'boireannach leatromach',
    '🫃' => 'fireannach trom',
    '🫄' => 'neach trom',
    '🤱' => 'toirt cìoch',
    '👼' => 'aingeal òg',
    '🎅' => 'bodach na Nollaig',
    '🤶' => 'bean Bodach na Nollaig',
    '🦸' => 'sàr-ghaisgeach',
    '🦹' => 'sàr-shlaightear',
    '🧙' => 'draoidh',
    '🧚' => 'sìthiche',
    '🧛' => 'bhampair',
    '🧜' => 'neach-mara',
    '🧝' => 'ailbhear',
    '🧞' => 'taibhse Arabach',
    '🧟' => 'zombie',
    '🧌' => 'trobha',
    '🫈' => 'creutair molach',
    '💆' => 'neach a’ faighinn massage',
    '💇' => 'neach a’ faighinn cliop',
    '🚶' => 'neach a’ coiseachd',
    '🧍' => 'neach ’na sheasamh',
    '🧎' => 'neach air a ghlùinean',
    '🏃' => 'neach a’ ruith',
    '💃' => 'boireannach a’ dannsadh',
    '🕺' => 'duine a’ dannsadh',
    '🕴' => 'duine air flod ’s deise gnìomhachais air',
    '👯' => 'daoine le cluasan rabaid',
    '🧖' => 'neach ann an sauna',
    '🧗' => 'neach a’ sreap',
    '🤺' => 'neach ri feannsaireachd',
    '🏇' => 'rèiseadh each',
    '⛷' => 'sgìtheadair',
    '🏂' => 'neach sneachda-bhòrd',
    '🏌' => 'neach ri goilf',
    '🏄' => 'neach a’ surfadh',
    '🚣' => 'neach ag iomradh bàta',
    '🏊' => 'neach a’ snàmh',
    '⛹' => 'neach a’ bocadh bàla',
    '🏋' => 'neach a’ togail chuideaman',
    '🚴' => 'neach ri rothaireachd',
    '🚵' => 'neach ri rothaireachd bheanntan',
    '🤸' => 'neach a’ cur nan rothan',
    '🤼' => 'daoine ri gleacadh',
    '🤽' => 'neach a’ cluich polo-uisge',
    '🤾' => 'neach a’ cluich ball-làimhe',
    '🤹' => 'neach ri caisreabhachd',
    '🧘' => 'neach na shuidhe lòtais',
    '🛀' => 'neach san amar',
    '🛌' => 'neach san leabaidh',
    '👭' => 'boireannaich a’ glacadh air làmh a chèile',
    '👫' => 'tè is fear a’ gabhail làmhan a chèile',
    '👬' => 'fireannaich a’ glacadh air làmh a chèile',
    '💏' => 'pòg',
    '💑' => 'càraid le cridhe',
    '🗣' => 'ceann a’ bruidhinn',
    '👤' => 'sgàil-riochd de chom',
    '👥' => 'sgàil-riochd de chuim',
    '🫂' => 'caidreabh',
    '👪' => 'teaghlach',
    '👣' => 'lorgan coise',
    '🫆' => 'lorg-mheòir',
    '🦰' => 'falt ruadh',
    '🦱' => 'falt dualach',
    '🦳' => 'falt bàn',
    '🦲' => 'maoilean',
    '🐵' => 'aodann muncaidh',
    '🐒' => 'muncaidh',
    '🦍' => 'goiriola',
    '🦧' => 'orangutan',
    '🐶' => 'aodainn coin',
    '🐕' => 'cù',
    '🦮' => 'cù-iùil',
    '🐩' => 'pùdal',
    '🐺' => 'madadh-allaidh',
    '🦊' => 'madadh-ruadh',
    '🦝' => 'mathan-nighe',
    '🐱' => 'aodann cait',
    '🐈' => 'cat',
    '🦁' => 'leòmhann',
    '🐯' => 'aodann tìgeir',
    '🐅' => 'tìgear',
    '🐆' => 'liopard',
    '🐴' => 'aodann eich',
    '🫎' => 'lon',
    '🫏' => 'asal',
    '🐎' => 'each',
    '🦄' => 'aon-adharcach',
    '🦓' => 'sìobra',
    '🦌' => 'fiadh',
    '🦬' => 'buabhall',
    '🐮' => 'aodann bà',
    '🐂' => 'damh',
    '🐃' => 'buabhall-uisge',
    '🐄' => 'bò',
    '🐷' => 'aodann muice',
    '🐖' => 'muc',
    '🐗' => 'torc',
    '🐽' => 'gnos muice',
    '🐏' => 'reithe',
    '🐑' => 'caora',
    '🐐' => 'gobhar',
    '🐪' => 'dromadair',
    '🐫' => 'càmhal',
    '🦙' => 'lama',
    '🦒' => 'sioraf',
    '🐘' => 'ailbhean',
    '🦣' => 'mamot',
    '🦏' => 'sròn-adharcach',
    '🦛' => 'each-aibhne',
    '🐭' => 'aodann lucha',
    '🐁' => 'luch',
    '🐀' => 'radan',
    '🐹' => 'hamstair',
    '🐰' => 'aodann rabaid',
    '🐇' => 'rabaid',
    '🐿' => 'feòrag-stiallach',
    '🦫' => 'bìobhair',
    '🦔' => 'gràineag',
    '🦇' => 'ialtag',
    '🐻' => 'mathan',
    '🐨' => 'koala',
    '🐼' => 'panda',
    '🦥' => 'lunndaire-craoibhe',
    '🦦' => 'dòbhran',
    '🦨' => 'tùdair-stiallach',
    '🦘' => 'cangaru',
    '🦡' => 'broc',
    '🐾' => 'lorgan spòige',
    '🦃' => 'eun-Frangach',
    '🐔' => 'cearc',
    '🐓' => 'coileach',
    '🐣' => 'isean ga ghur',
    '🐤' => 'isean beag',
    '🐥' => 'isean le aghaidh riut',
    '🐦' => 'eun',
    '🐧' => 'ceann-fionn',
    '🕊' => 'calman',
    '🦅' => 'iolaire',
    '🦆' => 'tunnag',
    '🦢' => 'eala',
    '🦉' => 'cailleach-oidhche',
    '🦤' => 'dòdo',
    '🪶' => 'ite',
    '🦩' => 'flamingo',
    '🦚' => 'peucag',
    '🦜' => 'pearraid',
    '🪽' => 'sgiath',
    '🪿' => 'gèadh',
    '🐸' => 'losgann',
    '🐊' => 'crogall',
    '🐢' => 'turtar',
    '🦎' => 'laghairt',
    '🐍' => 'nathair',
    '🐲' => 'aodann dràgoin',
    '🐉' => 'dràgon',
    '🦕' => 'dìneasar',
    '🦖' => 't-rex',
    '🐳' => 'muc-mhara a’ sèideadh',
    '🐋' => 'muc-mhara',
    '🐬' => 'leumadair-mara',
    '🦭' => 'ròn',
    '🐟' => 'iasg',
    '🐠' => 'iasg tropaigeach',
    '🐡' => 'iasg-sèididh',
    '🦈' => 'cearban',
    '🐙' => 'ochd-chasach',
    '🐚' => 'slige shnìomhanach',
    '🪸' => 'coireal',
    '🪼' => 'muir-tèachd',
    '🦀' => 'crùbag',
    '🦞' => 'giomach',
    '🦐' => 'carrain',
    '🦑' => 'gibearnach',
    '🦪' => 'eisir',
    '🐌' => 'seilcheag',
    '🦋' => 'dealan-dè',
    '🐛' => 'biastag',
    '🐜' => 'seangan',
    '🐝' => 'seillean',
    '🪲' => 'daolag',
    '🐞' => 'daolag-bhreac',
    '🦗' => 'greollan',
    '🪳' => 'càrnan-gallda',
    '🕷' => 'damhan-allaidh',
    '🕸' => 'lìon damhain-allaidh',
    '🦂' => 'sgairp',
    '🦟' => 'mosgaid',
    '🪰' => 'cuileag',
    '🪱' => 'cnuimh',
    '🦠' => 'bitheag',
    '💐' => 'bad fhlùraichean',
    '🌸' => 'blàth siris',
    '💮' => 'flùr geal',
    '🪷' => 'lòtas',
    '🏵' => 'ròsag',
    '🌹' => 'ròs',
    '🥀' => 'flùr seacte',
    '🌺' => 'ìbisg',
    '🌻' => 'neòinean-grèine',
    '🌼' => 'blàth',
    '🌷' => 'tuiliop',
    '🪻' => 'camais',
    '🌱' => 'sìolag',
    '🪴' => 'lus ann am poit',
    '🌲' => 'craobh sìor-uaine',
    '🌳' => 'craobh sheargach',
    '🌴' => 'craobh-phailm',
    '🌵' => 'cactas',
    '🌾' => 'dias rìs',
    '🌿' => 'luibh',
    '☘' => 'seamrag',
    '🍀' => 'seamrag cheithir-dhuilleagach',
    '🍁' => 'duilleag o chraobh-mhalpais',
    '🍂' => 'duilleagan an fhoghair',
    '🍃' => 'duilleag a’ falbh leis a’ ghaoith',
    '🪹' => 'nead falamh',
    '🪺' => 'nead le uighean',
    '🍄' => 'balgan-buachrach',
    '🪾' => 'craobh lom',
    '🍇' => 'fìon-dhearcan',
    '🍈' => 'meal-bhuc',
    '🍉' => 'meal-bhuc uisge',
    '🍊' => 'tangerine',
    '🍋' => 'liomaid',
    '🍌' => 'banana',
    '🍍' => 'anann',
    '🥭' => 'mango',
    '🍎' => 'ubhal dearg',
    '🍏' => 'ubhal uaine',
    '🍐' => 'peur',
    '🍑' => 'peitseag',
    '🍒' => 'sirisean',
    '🍓' => 'sùbh-làir',
    '🫐' => 'dearcan-gorma',
    '🥝' => 'meas kiwi',
    '🍅' => 'tomàto',
    '🫒' => 'dearc-ola',
    '🥥' => 'cnò-chòco',
    '🥑' => 'abhocado',
    '🍆' => 'aubergine',
    '🥔' => 'buntàta',
    '🥕' => 'curran',
    '🌽' => 'dias Innseanach',
    '🌶' => 'piobar-teth',
    '🫑' => 'piobar-milis',
    '🥒' => 'cularan',
    '🥬' => 'càl',
    '🥦' => 'brocail',
    '🧄' => 'creamh',
    '🧅' => 'uinnean',
    '🥜' => 'cnòthan-talmhainn',
    '🫘' => 'pònairean',
    '🌰' => 'geanm-chnò',
    '🫚' => 'dinnsear',
    '🫛' => 'builgean-peasrach',
    '🫜' => 'bun-lus',
    '🍞' => 'aran',
    '🥐' => 'croissant',
    '🥖' => 'baguette',
    '🫓' => 'aran-rèidh',
    '🥨' => 'pretzel',
    '🥯' => 'bagel',
    '🥞' => 'breacagan',
    '🧇' => 'waffle',
    '🧀' => 'staoig càise',
    '🍖' => 'feòil air a’ chnàimh',
    '🍗' => 'cas circe',
    '🥩' => 'feòil',
    '🥓' => 'beucon',
    '🍔' => 'hamburgair',
    '🍟' => 'sliseagan',
    '🍕' => 'pizza',
    '🌭' => 'hot dog',
    '🥪' => 'ceapaire',
    '🌮' => 'taco',
    '🌯' => 'burrito',
    '🫔' => 'tamale',
    '🥙' => 'aran-rèidh lìonta',
    '🧆' => 'falafel',
    '🥚' => 'ugh',
    '🍳' => 'còcaireachd',
    '🥘' => 'adhann le biadh',
    '🍲' => 'poit le biadh',
    '🫕' => 'fondue',
    '🥣' => 'bobhla le spàin',
    '🥗' => 'sailead uaine',
    '🍿' => 'popcorn',
    '🧈' => 'ìm',
    '🧂' => 'salainn',
    '🥫' => 'biadh ann an cana',
    '🍱' => 'bogsa bento',
    '🍘' => 'cnacair rìs',
    '🍙' => 'ball rìs',
    '🍚' => 'rìs bruich',
    '🍛' => 'coiridh is rìs',
    '🍜' => 'bobhla le ceò às',
    '🍝' => 'spaghetti',
    '🍠' => 'buntàta milis ròsta',
    '🍢' => 'oden',
    '🍣' => 'sushi',
    '🍤' => 'muasgain-chaola air am fraidhigeadh',
    '🍥' => 'cèic-èisg le cuairteag sa mheadhain',
    '🥮' => 'cèic-ghealaich',
    '🍡' => 'dango',
    '🥟' => 'dumplag',
    '🥠' => 'briosgaid fortain',
    '🥡' => 'bogsa takeout',
    '🍦' => 'reòiteag bhog',
    '🍧' => 'deigh bheàrrte',
    '🍨' => 'reòiteag',
    '🍩' => 'doughnut',
    '🍪' => 'briosgaid',
    '🎂' => 'cèic co-là-breith',
    '🍰' => 'shortcake',
    '🧁' => 'cèic-chuachaige',
    '🥧' => 'pàidh',
    '🍫' => 'bàr teòclaid',
    '🍬' => 'suiteas',
    '🍭' => 'lollipop',
    '🍮' => 'ughagan',
    '🍯' => 'poit meala',
    '🍼' => 'botal leinibh',
    '🥛' => 'glainne de bhainne',
    '☕' => 'deoch theth',
    '🫖' => 'poit-tì',
    '🍵' => 'cupa-tì gun chluas',
    '🍶' => 'sake',
    '🍾' => 'botal le àirc a’ leumadh às',
    '🍷' => 'glainne-fhìona',
    '🍸' => 'glainne cocktail',
    '🍹' => 'deoch thropaigeach',
    '🍺' => 'muga beòir',
    '🍻' => 'mugaichean beòir a’ gliongadaich',
    '🥂' => 'glaineachan a’ gliongadaich',
    '🥃' => 'tumblair',
    '🫗' => 'a’ dòrtadh deoch',
    '🥤' => 'cupa le stràbh',
    '🧋' => 'tì builgein',
    '🧃' => 'carton dighe',
    '🧉' => 'tì mate',
    '🧊' => 'ciùb deighe',
    '🥢' => 'bioran-ithe',
    '🍽' => 'forc is sgian le truinnsear',
    '🍴' => 'forc is sgian',
    '🥄' => 'spàin',
    '🔪' => 'sgian cidsin',
    '🫙' => 'crogan',
    '🏺' => 'amfora',
    '🌍' => 'cruinne leis an Roinn-Eòrpa ’s Afraga',
    '🌎' => 'cruinne le Aimeireaga',
    '🌏' => 'cruinne le Àisia ’s Astràilia',
    '🌐' => 'cruinne le meadhan-loidhnichean',
    '🗺' => 'mapa an t-saoghail',
    '🗾' => 'mapa dhen t-Seapan',
    '🧭' => 'combaist',
    '🏔' => 'beinn le sneachd air a mullach',
    '⛰' => 'beinn',
    '🛘' => 'maoim-talmhainn',
    '🌋' => 'beinn-theine',
    '🗻' => 'Beinn Fuji',
    '🏕' => 'campachadh',
    '🏖' => 'tràigh le sgàilean',
    '🏜' => 'fàsach',
    '🏝' => 'eilean fàs',
    '🏞' => 'pàirc nàiseanta',
    '🏟' => 'stèideam',
    '🏛' => 'togalach clasaigeach',
    '🏗' => 'togail taighe',
    '🧱' => 'breige',
    '🪨' => 'creag',
    '🪵' => 'fiodh',
    '🛖' => 'bothan',
    '🏘' => 'taighean',
    '🏚' => 'tobhta',
    '🏠' => 'taigh',
    '🏡' => 'taigh le gàrradh',
    '🏢' => 'togalach oifisean',
    '🏣' => 'oifis-puist Sheapanach',
    '🏤' => 'oifis a’ phuist',
    '🏥' => 'ospadal',
    '🏦' => 'banca',
    '🏨' => 'taigh-òsta',
    '🏩' => 'taigh-òsta gaoil',
    '🏪' => 'bùth ghoireasach',
    '🏫' => 'sgoil',
    '🏬' => 'mòr-bhùth',
    '🏭' => 'factaraidh',
    '🏯' => 'caisteal Seapanach',
    '🏰' => 'caisteal',
    '💒' => 'banais',
    '🗼' => 'tùr Tokyo',
    '🗽' => 'Ìomhaigh na Saorsa',
    '⛪' => 'eaglais',
    '🕌' => 'mosg',
    '🛕' => 'teampall Hionduthach',
    '🕍' => 'sionagog',
    '⛩' => 'naomh-chiste shinto',
    '🕋' => 'ka’aba',
    '⛲' => 'fuaran',
    '⛺' => 'teanta',
    '🌁' => 'ceòthach',
    '🌃' => 'oidhche rionnagach',
    '🏙' => 'mòr-bhaile',
    '🌄' => 'èirigh na grèine thar bheanntan',
    '🌅' => 'èirigh na grèine',
    '🌆' => 'ciaradh sa bhaile mhòr',
    '🌇' => 'laighe na grèine',
    '🌉' => 'drochaid air an oidhche',
    '♨' => 'fuaran teth',
    '🎠' => 'each-timcheallain',
    '🛝' => 'sleamhnag',
    '🎡' => 'roth-mòr',
    '🎢' => 'rolair-còrsair',
    '💈' => 'pòla barbair',
    '🎪' => 'pàillean siorcais',
    '🚂' => 'carbad-smùide',
    '🚃' => 'còidse rèile',
    '🚄' => 'trèana fhìor luath',
    '🚅' => 'trèana fhìor luath le sròn chruinn',
    '🚆' => 'trèana',
    '🚇' => 'meatro',
    '🚈' => 'rèile aotrom',
    '🚉' => 'stèisean',
    '🚊' => 'trama',
    '🚝' => 'aona-rèile',
    '🚞' => 'rèile-bheinne',
    '🚋' => 'còidse trama',
    '🚌' => 'bus',
    '🚍' => 'bus a’ tighinn thugad',
    '🚎' => 'bus-troilidh',
    '🚐' => 'meanbh-bhus',
    '🚑' => 'carbad-eiridinn',
    '🚒' => 'einnsean-smàlaidh',
    '🚓' => 'càr poileis',
    '🚔' => 'càr poileis a’ tighinn thugad',
    '🚕' => 'tacsaidh',
    '🚖' => 'tagsaidh a’ tighinn thugad',
    '🚗' => 'càr',
    '🚘' => 'càr a’ tighinn thugad',
    '🚙' => 'sport utility vehicle',
    '🛻' => 'truca pickup',
    '🚚' => 'làraidh-lìbhrigidh',
    '🚛' => 'làraidh altanach',
    '🚜' => 'tractar',
    '🏎' => 'càr rèisidh',
    '🏍' => 'motar-baidhg',
    '🛵' => 'sgùtair-motair',
    '🦽' => 'cathair-chuibhle a làimh',
    '🦼' => 'cathair-chuibhle le einnsein',
    '🛺' => 'càr-rickshaw',
    '🚲' => 'baidhseagal',
    '🛴' => 'sgùtair-breabaidh',
    '🛹' => 'spèileabord',
    '🛼' => 'bròg ròla-spèilidh',
    '🚏' => 'stad-bhusaichean',
    '🛣' => 'mòr-rathad',
    '🛤' => 'slighe-iarainn',
    '🛢' => 'druma ola',
    '⛽' => 'pumpa-connaidh',
    '🛞' => 'cuibhle',
    '🚨' => 'solas càr poileis',
    '🚥' => 'solais-thrafaig chòmhnard',
    '🚦' => 'solais-thrafaig inghearach',
    '🛑' => 'sanas “Stad”',
    '🚧' => 'togail',
    '⚓' => 'acair',
    '🛟' => 'fleodrainn-teasairginn',
    '⛵' => 'bàta-seòlaidh',
    '🛶' => 'canù',
    '🚤' => 'luath-bhàta',
    '🛳' => 'long thaistealach',
    '⛴' => 'bàta-aiseig',
    '🛥' => 'bàta-motair',
    '🚢' => 'long',
    '✈' => 'itealan',
    '🛩' => 'itealan beag',
    '🛫' => 'fàgail itealain',
    '🛬' => 'ruigsinn itealain',
    '🪂' => 'paraisiut',
    '💺' => 'suidheachan',
    '🚁' => 'heileacoptar',
    '🚟' => 'rèile-chrochaidh',
    '🚠' => 'carbad-càbaill beinne',
    '🚡' => 'trama-adhair',
    '🛰' => 'saideal',
    '🚀' => 'rocaid',
    '🛸' => 'soitheach-fànais',
    '🛎' => 'clag taigh-òsta',
    '🧳' => 'bagaist',
    '⌛' => 'glainne-uarach dheiseil',
    '⏳' => 'glainne-uarach nach eil deiseil',
    '⌚' => 'uaireadair',
    '⏰' => 'cloc caismeachd',
    '⏱' => 'stad-uaireadair',
    '⏲' => 'cleoc tìmeir',
    '🕰' => 'cleoc sgeilpe',
    '🕛' => 'dà uair dheug',
    '🕧' => 'leth-uair an dèidh dà dheug',
    '🕐' => 'uair',
    '🕜' => 'leth-uair an dèidh uair',
    '🕑' => 'dà uair',
    '🕝' => 'leth-uair an dèidh dà',
    '🕒' => 'trì uairean',
    '🕞' => 'leth-uair an dèidh trì',
    '🕓' => 'ceithir uairean',
    '🕟' => 'leth-uair an dèidh ceithir',
    '🕔' => 'còig uairean',
    '🕠' => 'leth-uair an dèidh còig',
    '🕕' => 'sia uairean',
    '🕡' => 'leth-uair an dèidh sia',
    '🕖' => 'seachd uairean',
    '🕢' => 'leth-uair an dèidh seachd',
    '🕗' => 'ochd uairean',
    '🕣' => 'leth-uair an dèidh ochd',
    '🕘' => 'naoidh uairean',
    '🕤' => 'leth-uair an dèidh naoidh',
    '🕙' => 'deich uairean',
    '🕥' => 'leth-uair an dèidh deich',
    '🕚' => 'aon uair deug',
    '🕦' => 'leth-uair an dèidh aon uair deug',
    '🌑' => 'gealach ùr',
    '🌒' => 'gealach chorranach a’ cinntinn',
    '🌓' => 'solas ùr na gealaich',
    '🌔' => 'gealach chrotach a’ fàs',
    '🌕' => 'làn-ghealach',
    '🌖' => 'gealach chrotach a’ traoghadh',
    '🌗' => 'deireadh na gealaich',
    '🌘' => 'gealach chorranach a’ traoghadh',
    '🌙' => 'gealach chorranach',
    '🌚' => 'gealach ùr le aodann',
    '🌛' => 'solas ùr na gealaich le aodann',
    '🌜' => 'deireadh na gealaich le aodann',
    '🌡' => 'teas-mheidh',
    '☀' => 'a’ ghrian',
    '🌝' => 'gealach làn le aodann',
    '🌞' => 'a’ ghrian le aodann oirre',
    '🪐' => 'planaid fhàinneach',
    '⭐' => 'rionnag mheadhanach gheal',
    '🌟' => 'rionnag dhealrach',
    '🌠' => 'rionnag earballach',
    '🌌' => 'Slighe Chlann Uisnich',
    '☁' => 'neul',
    '⛅' => 'a’ ghrian air cùl sgòtha',
    '⛈' => 'neul le dealanaich ’s uisge',
    '🌤' => 'a’ ghrian air cùl neòil bhig',
    '🌥' => 'a’ ghrian air cùl sgòtha mhòir',
    '🌦' => 'a’ ghrian air cùl sgòtha uisge',
    '🌧' => 'sgòth le uisge',
    '🌨' => 'sgòth le sneachd',
    '🌩' => 'neul le dealanaich',
    '🌪' => 'tornàdo',
    '🌫' => 'ceò',
    '🌬' => 'aodann na gaoithe',
    '🌀' => 'cuairt-gaoithe',
    '🌈' => 'bogha-froise',
    '🌂' => 'sgàilean-uisge dùinte',
    '☂' => 'sgàilean-uisge',
    '☔' => 'sgàilean-uisge le boinnean uisge',
    '⛱' => 'sgàilean air an làr',
    '⚡' => 'voltachd àrd',
    '❄' => 'bleideag sneachda',
    '☃' => 'bodach-sneachda',
    '⛄' => 'bodach-sneachda gun sneachd',
    '☄' => 'comaid',
    '🔥' => 'teine',
    '💧' => 'boinneag',
    '🌊' => 'stuagh',
    '🎃' => 'lanntair peapaig',
    '🎄' => 'craobh Nollaig',
    '🎆' => 'cleasan-teine',
    '🎇' => 'lainnireag',
    '🧨' => 'dìorrasan-fùdair',
    '✨' => 'lainnirean',
    '🎈' => 'bailiùn',
    '🎉' => 'bragadair pàrtaidh',
    '🎊' => 'ball confetti',
    '🎋' => 'craobh tanabata',
    '🎍' => 'sgeadas giuthais',
    '🎎' => 'doileagan Seapanach',
    '🎏' => 'bratach carbhain',
    '🎐' => 'clagan-gaoithe',
    '🎑' => 'latha coimhead na gealaich',
    '🧧' => 'cèiseag dhearg',
    '🎀' => 'ribean',
    '🎁' => 'prèasant paisgte',
    '🎗' => 'ribean cuimhneachaidh',
    '🎟' => 'ticeadan inntrigidh',
    '🎫' => 'ticead',
    '🎖' => 'bonn an airm',
    '🏆' => 'cuach-bhuaidhe',
    '🏅' => 'bonn spòrs',
    '🥇' => 'bonn a’ chiad àite',
    '🥈' => 'bonn an dàrna àite',
    '🥉' => 'bonn an treas àite',
    '⚽' => 'ball-coise',
    '⚾' => 'ball-beusa',
    '🥎' => 'ball-beusa bog',
    '🏀' => 'ball-basgaid',
    '🏐' => 'ball-bholaidh',
    '🏈' => 'ball-coise Aimeireaganach',
    '🏉' => 'ball-rugbaidh',
    '🎾' => 'teanas',
    '🥏' => 'frisbee',
    '🎳' => 'bobhladh',
    '🏏' => 'criogaid',
    '🏑' => 'hocaidh raoin',
    '🏒' => 'hocaidh-deigh',
    '🥍' => 'lacrosse',
    '🏓' => 'ping-pong',
    '🏸' => 'badmantan',
    '🥊' => 'miotag dòrnaireachd',
    '🥋' => 'èideadh nan ealainean còmhraige',
    '🥅' => 'lìon tadhail',
    '⛳' => 'bratach ann an toll',
    '⛸' => 'bròg-spèilidh',
    '🎣' => 'slat-iasgaich',
    '🤿' => 'masg-daoibhigidh',
    '🎽' => 'lèine-ruith',
    '🎿' => 'sgìthean',
    '🛷' => 'slaod',
    '🥌' => 'clach-chrolaidh',
    '🎯' => 'san teis-mheadhan',
    '🪀' => 'yo-yo',
    '🪁' => 'itealag',
    '🔫' => 'daga',
    '🎱' => 'bàla a h-ochd',
    '🔮' => 'ball criostail',
    '🪄' => 'slatan-draoidheachd',
    '🎮' => 'geama video',
    '🕹' => 'bioran-smachd',
    '🎰' => 'inneal shlotaichean',
    '🎲' => 'dìsinn',
    '🧩' => 'mìrean-measgaichte',
    '🧸' => 'teadaidh',
    '🪅' => 'piñata',
    '🪩' => 'ball-sgàthain',
    '🪆' => 'doileagan matryoshka',
    '♠' => 'dath speura',
    '♥' => 'dath cridhe',
    '♦' => 'dath daoimein',
    '♣' => 'dath club',
    '♟' => 'pàn tàileisg',
    '🃏' => 'amadan',
    '🀄' => 'leacag Mahjong le dràgon dearg',
    '🎴' => 'cairtean nam flùraichean',
    '🎭' => 'ealain-chluiche',
    '🖼' => 'dealbh le frèam',
    '🎨' => 'paileat neach-ealain',
    '🧵' => 'snàithlean',
    '🪡' => 'snàthad',
    '🧶' => 'snàth',
    '🪢' => 'snaidhm',
    '👓' => 'speuclairean',
    '🕶' => 'speuclairean-grèine',
    '🥽' => 'glainne-dhìona',
    '🥼' => 'còta deuchainn-lainn',
    '🦺' => 'èideadh sàbhailteachd',
    '👔' => 'tàidh',
    '👕' => 'lèine-t',
    '👖' => 'dìnichean',
    '🧣' => 'stoc-amhaich',
    '🧤' => 'miotagan',
    '🧥' => 'còta',
    '🧦' => 'stocainnean',
    '👗' => 'dreasa',
    '👘' => 'kimono',
    '🥻' => 'sari',
    '🩱' => 'deise-shnàimh',
    '🩲' => 'briogais-shnàimh',
    '🩳' => 'briogais-ghlùine',
    '👙' => 'bikini',
    '👚' => 'aodach boireannaich',
    '🪭' => 'gaotharan-làimhe',
    '👛' => 'sporan',
    '👜' => 'baga-làimhe',
    '👝' => 'clutch bag',
    '🛍' => 'bagaichean seopadaireachd',
    '🎒' => 'baga-droma sgoile',
    '🩴' => 'cuaran-èill',
    '👞' => 'bròg fir',
    '👟' => 'bròg-ruith',
    '🥾' => 'botan coiseachd',
    '🥿' => 'bròg',
    '👠' => 'bròg le sàil àrd',
    '👡' => 'cuaran boireannaich',
    '🩰' => 'brògan ballet',
    '👢' => 'bòtann boireannaich',
    '🪮' => 'cìr bheag',
    '👑' => 'crùn',
    '👒' => 'ad boireannaich',
    '🎩' => 'ad àrd',
    '🎓' => 'currac ceumnachaidh',
    '🧢' => 'bonaid bhileach',
    '🪖' => 'clogaid airm',
    '⛑' => 'clogaid neach-teasairginn',
    '📿' => 'grìogagan ùrnaigh',
    '💄' => 'beul-dath',
    '💍' => 'fàinne',
    '💎' => 'leug',
    '🔇' => 'glaodhaire mùchte',
    '🔈' => 'glaodhaire ’s fhuaim ìseal',
    '🔉' => 'glaodhaire ’s fhuaim mheadhanach',
    '🔊' => 'glaodhaire ’s fhuaim àrd',
    '📢' => 'glaodhaire',
    '📣' => 'meaga-fòn',
    '📯' => 'còrn a’ phuist',
    '🔔' => 'clag',
    '🔕' => 'clag le loidhne troimhe',
    '🎼' => 'sgòr-ciùil',
    '🎵' => 'pong',
    '🎶' => 'pongan ciùil',
    '🎙' => 'micreofon stiùideo',
    '🎚' => 'sleamhnachan àirde',
    '🎛' => 'putanan-smachd',
    '🎤' => 'micreofon',
    '🎧' => 'headphone',
    '📻' => 'rèidio',
    '🎷' => 'sacsafon',
    '🎺' => 'trombaid',
    '🪊' => 'trompan',
    '🪗' => 'bogsa-ciùil',
    '🎸' => 'giotàr',
    '🎹' => 'meur-chlàr ciùil',
    '🎻' => 'fìdheall',
    '🪕' => 'bainsio',
    '🥁' => 'druma',
    '🪘' => 'druma fada',
    '🪇' => 'maracas',
    '🪈' => 'duiseal',
    '🪉' => 'clàrsach',
    '📱' => 'fòn-làimhe',
    '📲' => 'fòn-làimhe le saighead',
    '☎' => 'fòn',
    '📞' => 'glacadair-fòn',
    '📟' => 'pèidsear',
    '📠' => 'inneal facs',
    '🔋' => 'bataraidh',
    '🪫' => 'bataraidh fann',
    '🔌' => 'pluga dealain',
    '💻' => 'laptop',
    '🖥' => 'coimpiutair deasg',
    '🖨' => 'clò-bhualadair',
    '⌨' => 'meur-chlàr',
    '🖱' => 'luchag',
    '🖲' => 'trackball',
    '💽' => 'diosga coimpiutair',
    '💾' => 'diosg bog',
    '💿' => 'diosg optaigeach',
    '📀' => 'DVD',
    '🧮' => 'abacas',
    '🎥' => 'camara-film',
    '🎞' => 'frèamaichean film',
    '📽' => 'proiseactar',
    '🎬' => 'bòrd-braig',
    '📺' => 'telebhisean',
    '📷' => 'camara',
    '📸' => 'camara le solas-boillsgidh',
    '📹' => 'camara-video',
    '📼' => 'cèiseag-video',
    '🔍' => 'glainne-mheudachaidh dhan taobh chlì',
    '🔎' => 'glainne-mheudachaidh dhan taobh dheas',
    '🕯' => 'coinneal',
    '💡' => 'bolgan solais',
    '🔦' => 'toirds dealain',
    '🏮' => 'lanntair-pàipeir dearg',
    '🪔' => 'lampa-eòlain',
    '📔' => 'leabhar-nòtaichean sgeadaichte',
    '📕' => 'leabhar dùinte',
    '📖' => 'leabhar fosgailte',
    '📗' => 'leabhar uaine',
    '📘' => 'leabhar gorm',
    '📙' => 'leabhar orains',
    '📚' => 'leabhraichean',
    '📓' => 'leabhar-nòtaichean',
    '📒' => 'leabhar-cunntais',
    '📃' => 'duilleag le lùbadh',
    '📜' => 'sgrola',
    '📄' => 'duilleag le h-aghaidh os a cionn',
    '📰' => 'pàipear-naidheachd',
    '🗞' => 'pàipear-naidheachd truiste',
    '📑' => 'tabaichean-leabhair',
    '🔖' => 'comharra-leabhair',
    '🏷' => 'bileag',
    '🪙' => 'bonn',
    '💰' => 'mogan airgid',
    '🪎' => 'ciste-ulaidh',
    '💴' => 'not Yen',
    '💵' => 'not dolair',
    '💶' => 'not Eòro',
    '💷' => 'not Punnd',
    '💸' => 'airgead le sgiathan',
    '💳' => 'cairt-chreideis',
    '🧾' => 'cuidhteas',
    '💹' => 'cairt a’ dol suas le Yen',
    '✉' => 'cèis-litreach',
    '📧' => 'post-d',
    '📨' => 'cèis a-steach',
    '📩' => 'cèis le saighead',
    '📤' => 'treidhe a-mach',
    '📥' => 'treidhe a-steach',
    '📦' => 'pacaid',
    '📫' => 'bogsa-puist dùinte le bratach gu h-àrd',
    '📪' => 'bogsa-puist dùinte le bratach gu h-ìosal',
    '📬' => 'bogsa-puist fosgailte le bratach gu h-àrd',
    '📭' => 'bogsa-puist fosgailte le bratach gu h-ìosal',
    '📮' => 'bogsa-puist',
    '🗳' => 'bogsa taghaidh le baileat',
    '✏' => 'peansail',
    '✒' => 'gob dubh',
    '🖋' => 'peann',
    '🖊' => 'peann-bàla',
    '🖌' => 'bruis-pheant',
    '🖍' => 'crèidhean',
    '📝' => 'cuimhneachan',
    '💼' => 'màileid-oifise',
    '📁' => 'pasgan faidhle',
    '📂' => 'pasgan fhaidhlichean fosgailte',
    '🗂' => 'roinneadairean clàr-amais chairtean',
    '📅' => 'mìosachan',
    '📆' => 'mìosachan le duilleagan a thig dheth',
    '🗒' => 'pada-nòtaichean le cuachag',
    '🗓' => 'mìosachan le cuachag',
    '📇' => 'clàr-amais chairtean',
    '📈' => 'cairt a’ dol suas',
    '📉' => 'cairt a’ dol sìos',
    '📊' => 'cairt-bhàraichean',
    '📋' => 'stòr-bhòrd',
    '📌' => 'tacaid-bhrùthaidh',
    '📍' => 'tacaid-bhrùthaidh chruinn',
    '📎' => 'clip-phàipeir',
    '🖇' => 'cliopaichean-pàipeir ceangailte',
    '📏' => 'rùilear dìreach',
    '📐' => 'rùilear triantanach',
    '✂' => 'siosar',
    '🗃' => 'bogsa fhaidhlichean',
    '🗄' => 'caibineat fhaidhlichean',
    '🗑' => 'bogsa-sgudail',
    '🔒' => 'glaiste',
    '🔓' => 'neo-ghlaiste',
    '🔏' => 'glaiste le peann',
    '🔐' => 'glaiste le iuchair',
    '🔑' => 'iuchair',
    '🗝' => 'iuchair seann-nòsach',
    '🔨' => 'òrd',
    '🪓' => 'tuagh',
    '⛏' => 'piocaid',
    '⚒' => 'òrd is piocaid',
    '🛠' => 'òrd is toinnear',
    '🗡' => 'biodag',
    '⚔' => 'claidheamhan tarsainn a chèile',
    '💣' => 'boma',
    '🪃' => 'bùmarang',
    '🏹' => 'bogha agus saighead',
    '🛡' => 'sgiath-dìon',
    '🪚' => 'sàbh saorsainneachd',
    '🔧' => 'toinnear',
    '🪛' => 'sgriubhaire',
    '🔩' => 'cnò ’s bolta',
    '⚙' => 'gèar',
    '🗜' => 'clabhd',
    '⚖' => 'meidh co-chothromach',
    '🦯' => 'batachan',
    '🔗' => 'ceangal',
    '⛓' => 'sèineachan',
    '🪝' => 'dubhan',
    '🧰' => 'bogsa-innealan',
    '🧲' => 'clach-iùil',
    '🪜' => 'fàradh',
    '🪏' => 'sluasaid',
    '⚗' => 'ceimigeachd',
    '🧪' => 'tiùb-deuchainne',
    '🧫' => 'soitheach Petri',
    '🧬' => 'dna',
    '🔬' => 'micreosgop',
    '🔭' => 'glainne-amhairc',
    '📡' => 'iadhaire-saideil',
    '💉' => 'steallair',
    '🩸' => 'boinne fala',
    '💊' => 'pile',
    '🩹' => 'bann-araid',
    '🩼' => 'cruidse',
    '🩺' => 'steatasgop',
    '🩻' => 'gath-òmair',
    '🚪' => 'doras',
    '🛗' => 'àrdaichear',
    '🪞' => 'sgàthan',
    '🪟' => 'uinneag',
    '🛏' => 'leabaidh',
    '🛋' => 'langasaid ’s lampa',
    '🪑' => 'sèithear',
    '🚽' => 'taigh-beag',
    '🪠' => 'loinid',
    '🚿' => 'frasair',
    '🛁' => 'ballan-ionnlaid',
    '🪤' => 'trap lucha',
    '🪒' => 'ealtainn',
    '🧴' => 'botal lòisein',
    '🧷' => 'prìne-dùbailte',
    '🧹' => 'sguab',
    '🧺' => 'basgaid',
    '🧻' => 'rola pàipeir',
    '🪣' => 'bucaid',
    '🧼' => 'siabann',
    '🫧' => 'builgean',
    '🪥' => 'bruis-fhiaclan',
    '🧽' => 'spong',
    '🧯' => 'inneal-smàlaidh',
    '🛒' => 'cairt seopadaireachd',
    '🚬' => 'toitean',
    '⚰' => 'ciste-laighe',
    '🪦' => 'leac-lighe',
    '⚱' => 'poit-tasgaidh',
    '🧿' => 'ortha-sùla',
    '🪬' => 'hamsa',
    '🗿' => 'mo‘ai',
    '🪧' => 'sanas',
    '🪪' => 'cairt-aithneachaidh',
    '🏧' => 'sanas ATM',
    '🚮' => 'samhla “Sgudal dhan bhiona”',
    '🚰' => 'uisge so-òlta',
    '♿' => 'samhla sèitheir-chuibhle',
    '🚹' => 'taigh-beag nam fireannach',
    '🚺' => 'taigh-beag nam boireannach',
    '🚻' => 'sanas taighe-bhig',
    '🚼' => 'samhla leanabhain',
    '🚾' => 'clòsaid-uisge',
    '🛂' => 'sgrùdadh nam pas',
    '🛃' => 'cusbainn',
    '🛄' => 'ionad-tagraidh bagaist',
    '🛅' => 'bagaist air fhàgail',
    '⚠' => 'rabhadh',
    '🚸' => 'clann a’ dol thairis',
    '⛔' => 'gun inntrigeadh',
    '🚫' => 'toirmisgte',
    '🚳' => 'rothaireachd toirmisgte',
    '🚭' => 'smocadh toirmisgte',
    '🚯' => 'na tilg sgudal',
    '🚱' => 'uisge do-òlta',
    '🚷' => 'coiseachd toirmisgte',
    '📵' => 'fònaichean-làimhe toirmisgte',
    '🔞' => 'daoine os cionn 18 a-mhàin',
    '☢' => 'beò-rèididheachd',
    '☣' => 'cunnart bith-eòlach',
    '⬆' => 'saighead suas',
    '↗' => 'saighead gun ear-thuath',
    '➡' => 'saighead dhan taobh deas',
    '↘' => 'saighead gun ear-dheas',
    '⬇' => 'saighead sìos',
    '↙' => 'saighead gun iar-dheas',
    '⬅' => 'saighead dhan taobh chlì',
    '↖' => 'saighead gun iar-thuath',
    '↕' => 'saighead suas is sìos',
    '↔' => 'saighead chlì is dheas',
    '↩' => 'saighead dheas a’ lùbadh dhan taobh chlì',
    '↪' => 'saighead chlì a’ lùbadh dhan taobh deas',
    '⤴' => 'saighead dheas a’ lùbadh suas',
    '⤵' => 'saighead dheas a’ lùbadh sìos',
    '🔃' => 'saighdean inghearach a’ dol deiseil',
    '🔄' => 'putan saighdean a’ dol tuathail',
    '🔙' => 'saighead “BACK”',
    '🔚' => 'saighead “END”',
    '🔛' => 'saighead “ON”',
    '🔜' => 'saighead “SOON”',
    '🔝' => 'saighead “TOP”',
    '🛐' => 'àite-adhraidh',
    '⚛' => 'samhla ataim',
    '🕉' => 'om',
    '✡' => 'reul Dhaibhidh',
    '☸' => 'cuibhle dharma',
    '☯' => 'Ying Yang',
    '✝' => 'crois Laideann',
    '☦' => 'crois nan Ortodogsach',
    '☪' => 'corran is rionnag',
    '☮' => 'samhla sìthe',
    '🕎' => 'menorah',
    '🔯' => 'rionnag shia-rinneach dhotagach',
    '🪯' => 'khanda',
    '♈' => 'An Reithe',
    '♉' => 'An Tarbh',
    '♊' => 'Na Leth-aonan',
    '♋' => 'Am Partan',
    '♌' => 'An Leòmhann',
    '♍' => 'A’ Mhaighdeann',
    '♎' => 'A’ Mheidh',
    '♏' => 'An Sgairp',
    '♐' => 'An Saighdear',
    '♑' => 'An Gobhar',
    '♒' => 'Am Fear-giùlain Uisge',
    '♓' => 'Na h-Èisg',
    '⛎' => 'An Giùlanair-nathrach',
    '🔀' => 'putan a chuireas tracaichean air thuaiream',
    '🔁' => 'putan na h-ath-chluich',
    '🔂' => 'putan na h-ath-chluich shingilte',
    '▶' => 'putan “Cluich”',
    '⏩' => 'putan “Toinn air adhart”',
    '⏭' => 'putan “An ath-thraca”',
    '⏯' => 'putan “Cluich no cuir na stad”',
    '◀' => 'putan “Deireadh air thoiseach”',
    '⏪' => 'putan “Air ais gu luath”',
    '⏮' => 'putan “An traca roimhe”',
    '🔼' => 'putan suas',
    '⏫' => 'putan “Suas gu luath”',
    '🔽' => 'putan sìos',
    '⏬' => 'putan “Sìos gu luath”',
    '⏸' => 'putan “Cuir na stad”',
    '⏹' => 'putan “Cuir stad air”',
    '⏺' => 'putan “Clàraich”',
    '⏏' => 'putan “Gluais a-mach”',
    '🎦' => 'taigh-dhealbh',
    '🔅' => 'putan doilleireachaidh',
    '🔆' => 'putan soilleireachaidh',
    '📶' => 'iadhaire le bàraichean',
    '🛜' => 'uèirleas',
    '📳' => 'caismeachd crithe',
    '📴' => 'fòn-làimhe dheth',
    '♀' => 'samhla boireann',
    '♂' => 'samhla fireann',
    '⚧' => 'samhla tar-ghnèitheach',
    '✖' => 'iomadaich',
    '➕' => 'cuir ris',
    '➖' => 'thoir air falbh',
    '➗' => 'roinn',
    '🟰' => 'samhla co-ionnannachd trom',
    '♾' => 'neo-chrìochnachd',
    '‼' => 'clisg-phuing dhùbailte',
    '⁉' => 'clisg-phuing is comharra-ceiste',
    '❓' => 'comharra-ceiste dearg',
    '❔' => 'comharra-ceiste geal',
    '❕' => 'clisg-phuing gheal',
    '❗' => 'clisg-phuing dhearg',
    '〰' => 'loidhne lùbach',
    '💱' => 'iomlaid airgid',
    '💲' => 'samhla dolair trom',
    '⚕' => 'samhla meadaigeach',
    '♻' => 'samhla ath-chuartachaidh',
    '⚜' => 'fleur-de-lis',
    '🔱' => 'samhla muirgheidh',
    '📛' => 'baidse ainm',
    '🔰' => 'samhla Seapanach air neach-tòiseachaidh',
    '⭕' => 'cearcall dearg falamh',
    '✅' => 'putan cromaige',
    '☑' => 'bogsa-cromaige le cromag',
    '✔' => 'cromag dearbhaidh',
    '❌' => 'samhla croise',
    '❎' => 'samhla croise ann am putan',
    '➰' => 'lùb dhualach',
    '➿' => 'lùb dhualach dhùbailte',
    '〽' => 'comharra airson atharrachadh phàirtean',
    '✳' => 'reultag le ochd rinnean',
    '✴' => 'rionnag le ochd rinnean',
    '❇' => 'lainnir',
    '™' => 'comharra-malairt',
    '🫟' => 'steallag',
    '🔠' => 'ion-chur litrichean mòra Laideann',
    '🔡' => 'ion-chur litrichean beaga Laideann',
    '🔢' => 'ion-chur àireamhan',
    '🔣' => 'ion-chur shamhlaichean',
    '🔤' => 'ion-chur litrichean Laideann',
    '🅰' => 'putan A (seòrsa fala)',
    '🆎' => 'putan AB (seòrsa fala)',
    '🅱' => 'Putan B (seòrsa fala)',
    '🆑' => 'putan CL',
    '🆒' => 'putan COOL',
    '🆓' => 'putan FREE',
    'ℹ' => 'fiosrachadh',
    '🆔' => 'putan ID',
    'Ⓜ' => 'M le cearcall timcheall air',
    '🆕' => 'putan NEW',
    '🆖' => 'putan NG',
    '🅾' => 'Putan O (seòrsa fala)',
    '🆗' => 'putan OK',
    '🅿' => 'putan P',
    '🆘' => 'putan “SOS”',
    '🆙' => 'putan “UP!”',
    '🆚' => 'putan “VS”',
    '🈁' => 'putan Seapanais “An-seo”',
    '🈂' => 'putan Seapanais “tàille seirbheise”',
    '🈷' => 'putan Seapanais “tàille mhìosail”',
    '🈶' => 'putan Seapanais “Chan ann an-asgaidh”',
    '🈯' => 'putan Seapanais “Glèidhte”',
    '🉐' => 'putan Seapanais “Bargan”',
    '🈹' => 'putan Seapanais “Lasachadh”',
    '🈚' => 'putan Seapanais “Saor ’s an-asgaidh”',
    '🈲' => 'putan Seapanais “Toirmisgte”',
    '🉑' => 'putan Seapanais “Ceart gu leòr”',
    '🈸' => 'putan Seapanais “Tagradh”',
    '🈴' => 'putan Seapanais “Ìre pas”',
    '🈳' => 'putan Seapanais “Seòmar saor”',
    '㊗' => 'putan Seapanais “Meal do naidheachd”',
    '㊙' => 'putan Seapanais “Dìomhair”',
    '🈺' => 'putan Seapanais “Fosgailte”',
    '🈵' => 'putan Seapanais “Gun seòmar saor”',
    '🔴' => 'cearcall dearg',
    '🟠' => 'cearcall orainds',
    '🟡' => 'cearcall buidhe',
    '🟢' => 'cearcall uaine',
    '🔵' => 'cearcall gorm',
    '🟣' => 'cearcall purpaidh',
    '🟤' => 'cearcall donn',
    '⚫' => 'cearcall dubh',
    '⚪' => 'cearcall geal',
    '🟥' => 'ceàrnach dearg',
    '🟧' => 'ceàrnach orainds',
    '🟨' => 'ceàrnach buidhe',
    '🟩' => 'ceàrnach uaine',
    '🟦' => 'ceàrnach gorm',
    '🟪' => 'ceàrnach purpaidh',
    '🟫' => 'ceàrnach donn',
    '⬛' => 'ceàrnag mhòr dhubh',
    '⬜' => 'ceàrnag mhòr gheal',
    '◼' => 'ceàrnag mheadhanach dhubh',
    '◻' => 'ceàrnag mheadhanach gheal',
    '◾' => 'ceàrnag mheadhanach bheag dhubh',
    '◽' => 'ceàrnag mheadhanach bheag gheal',
    '▪' => 'ceàrnag bheag dhubh',
    '▫' => 'ceàrnag bheag gheal',
    '🔶' => 'daoimean mòr orains',
    '🔷' => 'daoimean mòr gorm',
    '🔸' => 'daoimean beag orains',
    '🔹' => 'daoimean beag gorm',
    '🔺' => 'triantan dearg a’ tomhadh suas',
    '🔻' => 'triantan dearg a’ tomhadh sìos',
    '💠' => 'daoimean le dotag na bhroinn',
    '🔘' => 'putan-rèidio',
    '🔳' => 'putan geal ceàrnagach',
    '🔲' => 'putan ceàrnagach dubh',
    '🏁' => 'bratach bhreacte',
    '🚩' => 'bratach thriantanach',
    '🎌' => 'brataichean tarsainn a chèile',
    '🏴' => 'bratach dhubh',
    '🏳' => 'bratach gheal',
];
