<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'póg: aosach, aosach, ton cnis geal, ton cnis mheánach-gheal',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'póg: aosach, aosach, ton cnis geal, ton cnis meánach',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'póg: aosach, aosach, ton cnis geal, ton cnis meándorcha',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'póg: aosach, aosach, ton cnis geal, ton cnis dorcha',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'póg: aosach, aosach, ton cnis mheánach-gheal, ton cnis geal',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'póg: aosach, aosach, ton cnis mheánach-gheal, ton cnis meánach',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'póg: aosach, aosach, ton cnis mheánach-gheal, ton cnis meándorcha',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'póg: aosach, aosach, ton cnis mheánach-gheal, ton cnis dorcha',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'póg: aosach, aosach, ton cnis meánach, ton cnis geal',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'póg: aosach, aosach, ton cnis meánach, ton cnis mheánach-gheal',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'póg: aosach, aosach, ton cnis meánach, ton cnis meándorcha',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'póg: aosach, aosach, ton cnis meánach, ton cnis dorcha',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'póg: aosach, aosach, ton cnis meándorcha, ton cnis geal',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'póg: aosach, aosach, ton cnis meándorcha, ton cnis mheánach-gheal',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'póg: aosach, aosach, ton cnis meándorcha, ton cnis meánach',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'póg: aosach, aosach, ton cnis meándorcha, ton cnis dorcha',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'póg: aosach, aosach, ton cnis dorcha, ton cnis geal',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'póg: aosach, aosach, ton cnis dorcha, ton cnis mheánach-gheal',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'póg: aosach, aosach, ton cnis dorcha, ton cnis meánach',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'póg: aosach, aosach, ton cnis dorcha, ton cnis meándorcha',
    '👨🏻‍❤‍💋‍👨🏻' => 'póg: fear, fear, ton cnis geal',
    '👨🏻‍❤‍💋‍👨🏼' => 'póg: fear, fear, ton cnis geal, ton cnis mheánach-gheal',
    '👨🏻‍❤‍💋‍👨🏽' => 'póg: fear, fear, ton cnis geal, ton cnis meánach',
    '👨🏻‍❤‍💋‍👨🏾' => 'póg: fear, fear, ton cnis geal, ton cnis meándorcha',
    '👨🏻‍❤‍💋‍👨🏿' => 'póg: fear, fear, ton cnis geal, ton cnis dorcha',
    '👨🏼‍❤‍💋‍👨🏻' => 'póg: fear, fear, ton cnis mheánach-gheal, ton cnis geal',
    '👨🏼‍❤‍💋‍👨🏼' => 'póg: fear, fear, ton cnis mheánach-gheal',
    '👨🏼‍❤‍💋‍👨🏽' => 'póg: fear, fear, ton cnis mheánach-gheal, ton cnis meánach',
    '👨🏼‍❤‍💋‍👨🏾' => 'póg: fear, fear, ton cnis mheánach-gheal, ton cnis meándorcha',
    '👨🏼‍❤‍💋‍👨🏿' => 'póg: fear, fear, ton cnis mheánach-gheal, ton cnis dorcha',
    '👨🏽‍❤‍💋‍👨🏻' => 'póg: fear, fear, ton cnis meánach, ton cnis geal',
    '👨🏽‍❤‍💋‍👨🏼' => 'póg: fear, fear, ton cnis meánach, ton cnis mheánach-gheal',
    '👨🏽‍❤‍💋‍👨🏽' => 'póg: fear, fear, ton cnis meánach',
    '👨🏽‍❤‍💋‍👨🏾' => 'póg: fear, fear, ton cnis meánach, ton cnis meándorcha',
    '👨🏽‍❤‍💋‍👨🏿' => 'póg: fear, fear, ton cnis meánach, ton cnis dorcha',
    '👨🏾‍❤‍💋‍👨🏻' => 'póg: fear, fear, ton cnis meándorcha, ton cnis geal',
    '👨🏾‍❤‍💋‍👨🏼' => 'póg: fear, fear, ton cnis meándorcha, ton cnis mheánach-gheal',
    '👨🏾‍❤‍💋‍👨🏽' => 'póg: fear, fear, ton cnis meándorcha, ton cnis meánach',
    '👨🏾‍❤‍💋‍👨🏾' => 'póg: fear, fear, ton cnis meándorcha',
    '👨🏾‍❤‍💋‍👨🏿' => 'póg: fear, fear, ton cnis meándorcha, ton cnis dorcha',
    '👨🏿‍❤‍💋‍👨🏻' => 'póg: fear, fear, ton cnis dorcha, ton cnis geal',
    '👨🏿‍❤‍💋‍👨🏼' => 'póg: fear, fear, ton cnis dorcha, ton cnis mheánach-gheal',
    '👨🏿‍❤‍💋‍👨🏽' => 'póg: fear, fear, ton cnis dorcha, ton cnis meánach',
    '👨🏿‍❤‍💋‍👨🏾' => 'póg: fear, fear, ton cnis dorcha, ton cnis meándorcha',
    '👨🏿‍❤‍💋‍👨🏿' => 'póg: fear, fear, ton cnis dorcha',
    '👩🏻‍❤‍💋‍👨🏻' => 'póg: bean, fear, ton cnis geal',
    '👩🏻‍❤‍💋‍👨🏼' => 'póg: bean, fear, ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍❤‍💋‍👨🏽' => 'póg: bean, fear, ton cnis geal, ton cnis meánach',
    '👩🏻‍❤‍💋‍👨🏾' => 'póg: bean, fear, ton cnis geal, ton cnis meándorcha',
    '👩🏻‍❤‍💋‍👨🏿' => 'póg: bean, fear, ton cnis geal, ton cnis dorcha',
    '👩🏼‍❤‍💋‍👨🏻' => 'póg: bean, fear, ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍❤‍💋‍👨🏼' => 'póg: bean, fear, ton cnis mheánach-gheal',
    '👩🏼‍❤‍💋‍👨🏽' => 'póg: bean, fear, ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍❤‍💋‍👨🏾' => 'póg: bean, fear, ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍❤‍💋‍👨🏿' => 'póg: bean, fear, ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍❤‍💋‍👨🏻' => 'póg: bean, fear, ton cnis meánach, ton cnis geal',
    '👩🏽‍❤‍💋‍👨🏼' => 'póg: bean, fear, ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍❤‍💋‍👨🏽' => 'póg: bean, fear, ton cnis meánach',
    '👩🏽‍❤‍💋‍👨🏾' => 'póg: bean, fear, ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍❤‍💋‍👨🏿' => 'póg: bean, fear, ton cnis meánach, ton cnis dorcha',
    '👩🏾‍❤‍💋‍👨🏻' => 'póg: bean, fear, ton cnis meándorcha, ton cnis geal',
    '👩🏾‍❤‍💋‍👨🏼' => 'póg: bean, fear, ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍❤‍💋‍👨🏽' => 'póg: bean, fear, ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍❤‍💋‍👨🏾' => 'póg: bean, fear, ton cnis meándorcha',
    '👩🏾‍❤‍💋‍👨🏿' => 'póg: bean, fear, ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍❤‍💋‍👨🏻' => 'póg: bean, fear, ton cnis dorcha, ton cnis geal',
    '👩🏿‍❤‍💋‍👨🏼' => 'póg: bean, fear, ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍❤‍💋‍👨🏽' => 'póg: bean, fear, ton cnis dorcha, ton cnis meánach',
    '👩🏿‍❤‍💋‍👨🏾' => 'póg: bean, fear, ton cnis dorcha, ton cnis meándorcha',
    '👩🏿‍❤‍💋‍👨🏿' => 'póg: bean, fear, ton cnis dorcha',
    '👩🏻‍❤‍💋‍👩🏻' => 'póg: bean, bean, ton cnis geal',
    '👩🏻‍❤‍💋‍👩🏼' => 'póg: bean, bean, ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍❤‍💋‍👩🏽' => 'póg: bean, bean, ton cnis geal, ton cnis meánach',
    '👩🏻‍❤‍💋‍👩🏾' => 'póg: bean, bean, ton cnis geal, ton cnis meándorcha',
    '👩🏻‍❤‍💋‍👩🏿' => 'póg: bean, bean, ton cnis geal, ton cnis dorcha',
    '👩🏼‍❤‍💋‍👩🏻' => 'póg: bean, bean, ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍❤‍💋‍👩🏼' => 'póg: bean, bean, ton cnis mheánach-gheal',
    '👩🏼‍❤‍💋‍👩🏽' => 'póg: bean, bean, ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍❤‍💋‍👩🏾' => 'póg: bean, bean, ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍❤‍💋‍👩🏿' => 'póg: bean, bean, ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍❤‍💋‍👩🏻' => 'póg: bean, bean, ton cnis meánach, ton cnis geal',
    '👩🏽‍❤‍💋‍👩🏼' => 'póg: bean, bean, ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍❤‍💋‍👩🏽' => 'póg: bean, bean, ton cnis meánach',
    '👩🏽‍❤‍💋‍👩🏾' => 'póg: bean, bean, ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍❤‍💋‍👩🏿' => 'póg: bean, bean, ton cnis meánach, ton cnis dorcha',
    '👩🏾‍❤‍💋‍👩🏻' => 'póg: bean, bean, ton cnis meándorcha, ton cnis geal',
    '👩🏾‍❤‍💋‍👩🏼' => 'póg: bean, bean, ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍❤‍💋‍👩🏽' => 'póg: bean, bean, ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍❤‍💋‍👩🏾' => 'póg: bean, bean, ton cnis meándorcha',
    '👩🏾‍❤‍💋‍👩🏿' => 'póg: bean, bean, ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍❤‍💋‍👩🏻' => 'póg: bean, bean, ton cnis dorcha, ton cnis geal',
    '👩🏿‍❤‍💋‍👩🏼' => 'póg: bean, bean, ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍❤‍💋‍👩🏽' => 'póg: bean, bean, ton cnis dorcha, ton cnis meánach',
    '👩🏿‍❤‍💋‍👩🏾' => 'póg: bean, bean, ton cnis dorcha, ton cnis meándorcha',
    '👩🏿‍❤‍💋‍👩🏿' => 'póg: bean, bean, ton cnis dorcha',
    '🧑🏻‍🫯‍🧑🏼' => 'coraíocht: ton cnis geal, ton cnis mheánach-gheal',
    '🧑🏻‍🫯‍🧑🏽' => 'coraíocht: ton cnis geal, ton cnis meánach',
    '🧑🏻‍🫯‍🧑🏾' => 'coraíocht: ton cnis geal, ton cnis meándorcha',
    '🧑🏻‍🫯‍🧑🏿' => 'coraíocht: ton cnis geal, ton cnis dorcha',
    '🧑🏼‍🫯‍🧑🏻' => 'coraíocht: ton cnis mheánach-gheal, ton cnis geal',
    '🧑🏼‍🫯‍🧑🏽' => 'coraíocht: ton cnis mheánach-gheal, ton cnis meánach',
    '🧑🏼‍🫯‍🧑🏾' => 'coraíocht: ton cnis mheánach-gheal, ton cnis meándorcha',
    '🧑🏼‍🫯‍🧑🏿' => 'coraíocht: ton cnis mheánach-gheal, ton cnis dorcha',
    '🧑🏽‍🫯‍🧑🏻' => 'coraíocht: ton cnis meánach, ton cnis geal',
    '🧑🏽‍🫯‍🧑🏼' => 'coraíocht: ton cnis meánach, ton cnis mheánach-gheal',
    '🧑🏽‍🫯‍🧑🏾' => 'coraíocht: ton cnis meánach, ton cnis meándorcha',
    '🧑🏽‍🫯‍🧑🏿' => 'coraíocht: ton cnis meánach, ton cnis dorcha',
    '🧑🏾‍🫯‍🧑🏻' => 'coraíocht: ton cnis meándorcha, ton cnis geal',
    '🧑🏾‍🫯‍🧑🏼' => 'coraíocht: ton cnis meándorcha, ton cnis mheánach-gheal',
    '🧑🏾‍🫯‍🧑🏽' => 'coraíocht: ton cnis meándorcha, ton cnis meánach',
    '🧑🏾‍🫯‍🧑🏿' => 'coraíocht: ton cnis meándorcha, ton cnis dorcha',
    '🧑🏿‍🫯‍🧑🏻' => 'coraíocht: ton cnis dorcha, ton cnis geal',
    '🧑🏿‍🫯‍🧑🏼' => 'coraíocht: ton cnis dorcha, ton cnis mheánach-gheal',
    '🧑🏿‍🫯‍🧑🏽' => 'coraíocht: ton cnis dorcha, ton cnis meánach',
    '🧑🏿‍🫯‍🧑🏾' => 'coraíocht: ton cnis dorcha, ton cnis meándorcha',
    '🧑🏻‍❤‍🧑🏼' => 'lanúin le croí: aosach, aosach, ton cnis geal, ton cnis mheánach-gheal',
    '🧑🏻‍❤‍🧑🏽' => 'lanúin le croí: aosach, aosach, ton cnis geal, ton cnis meánach',
    '🧑🏻‍❤‍🧑🏾' => 'lanúin le croí: aosach, aosach, ton cnis geal, ton cnis meándorcha',
    '🧑🏻‍❤‍🧑🏿' => 'lanúin le croí: aosach, aosach, ton cnis geal, ton cnis dorcha',
    '🧑🏼‍❤‍🧑🏻' => 'lanúin le croí: aosach, aosach, ton cnis mheánach-gheal, ton cnis geal',
    '🧑🏼‍❤‍🧑🏽' => 'lanúin le croí: aosach, aosach, ton cnis mheánach-gheal, ton cnis meánach',
    '🧑🏼‍❤‍🧑🏾' => 'lanúin le croí: aosach, aosach, ton cnis mheánach-gheal, ton cnis meándorcha',
    '🧑🏼‍❤‍🧑🏿' => 'lanúin le croí: aosach, aosach, ton cnis mheánach-gheal, ton cnis dorcha',
    '🧑🏽‍❤‍🧑🏻' => 'lanúin le croí: aosach, aosach, ton cnis meánach, ton cnis geal',
    '🧑🏽‍❤‍🧑🏼' => 'lanúin le croí: aosach, aosach, ton cnis meánach, ton cnis mheánach-gheal',
    '🧑🏽‍❤‍🧑🏾' => 'lanúin le croí: aosach, aosach, ton cnis meánach, ton cnis meándorcha',
    '🧑🏽‍❤‍🧑🏿' => 'lanúin le croí: aosach, aosach, ton cnis meánach, ton cnis dorcha',
    '🧑🏾‍❤‍🧑🏻' => 'lanúin le croí: aosach, aosach, ton cnis meándorcha, ton cnis geal',
    '🧑🏾‍❤‍🧑🏼' => 'lanúin le croí: aosach, aosach, ton cnis meándorcha, ton cnis mheánach-gheal',
    '🧑🏾‍❤‍🧑🏽' => 'lanúin le croí: aosach, aosach, ton cnis meándorcha, ton cnis meánach',
    '🧑🏾‍❤‍🧑🏿' => 'lanúin le croí: aosach, aosach, ton cnis meándorcha, ton cnis dorcha',
    '🧑🏿‍❤‍🧑🏻' => 'lanúin le croí: aosach, aosach, ton cnis dorcha, ton cnis geal',
    '🧑🏿‍❤‍🧑🏼' => 'lanúin le croí: aosach, aosach, ton cnis dorcha, ton cnis mheánach-gheal',
    '🧑🏿‍❤‍🧑🏽' => 'lanúin le croí: aosach, aosach, ton cnis dorcha, ton cnis meánach',
    '🧑🏿‍❤‍🧑🏾' => 'lanúin le croí: aosach, aosach, ton cnis dorcha, ton cnis meándorcha',
    '🧑🏻‍🐰‍🧑🏼' => 'daoine le cluasa coiníní orthu: ton cnis geal, ton cnis mheánach-gheal',
    '🧑🏻‍🐰‍🧑🏽' => 'daoine le cluasa coiníní orthu: ton cnis geal, ton cnis meánach',
    '🧑🏻‍🐰‍🧑🏾' => 'daoine le cluasa coiníní orthu: ton cnis geal, ton cnis meándorcha',
    '🧑🏻‍🐰‍🧑🏿' => 'daoine le cluasa coiníní orthu: ton cnis geal, ton cnis dorcha',
    '🧑🏼‍🐰‍🧑🏻' => 'daoine le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis geal',
    '🧑🏼‍🐰‍🧑🏽' => 'daoine le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis meánach',
    '🧑🏼‍🐰‍🧑🏾' => 'daoine le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis meándorcha',
    '🧑🏼‍🐰‍🧑🏿' => 'daoine le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis dorcha',
    '🧑🏽‍🐰‍🧑🏻' => 'daoine le cluasa coiníní orthu: ton cnis meánach, ton cnis geal',
    '🧑🏽‍🐰‍🧑🏼' => 'daoine le cluasa coiníní orthu: ton cnis meánach, ton cnis mheánach-gheal',
    '🧑🏽‍🐰‍🧑🏾' => 'daoine le cluasa coiníní orthu: ton cnis meánach, ton cnis meándorcha',
    '🧑🏽‍🐰‍🧑🏿' => 'daoine le cluasa coiníní orthu: ton cnis meánach, ton cnis dorcha',
    '🧑🏾‍🐰‍🧑🏻' => 'daoine le cluasa coiníní orthu: ton cnis meándorcha, ton cnis geal',
    '🧑🏾‍🐰‍🧑🏼' => 'daoine le cluasa coiníní orthu: ton cnis meándorcha, ton cnis mheánach-gheal',
    '🧑🏾‍🐰‍🧑🏽' => 'daoine le cluasa coiníní orthu: ton cnis meándorcha, ton cnis meánach',
    '🧑🏾‍🐰‍🧑🏿' => 'daoine le cluasa coiníní orthu: ton cnis meándorcha, ton cnis dorcha',
    '🧑🏿‍🐰‍🧑🏻' => 'daoine le cluasa coiníní orthu: ton cnis dorcha, ton cnis geal',
    '🧑🏿‍🐰‍🧑🏼' => 'daoine le cluasa coiníní orthu: ton cnis dorcha, ton cnis mheánach-gheal',
    '🧑🏿‍🐰‍🧑🏽' => 'daoine le cluasa coiníní orthu: ton cnis dorcha, ton cnis meánach',
    '🧑🏿‍🐰‍🧑🏾' => 'daoine le cluasa coiníní orthu: ton cnis dorcha, ton cnis meándorcha',
    '👨🏻‍🫯‍👨🏼' => 'fir ag coraíocht: ton cnis geal, ton cnis mheánach-gheal',
    '👨🏻‍🫯‍👨🏽' => 'fir ag coraíocht: ton cnis geal, ton cnis meánach',
    '👨🏻‍🫯‍👨🏾' => 'fir ag coraíocht: ton cnis geal, ton cnis meándorcha',
    '👨🏻‍🫯‍👨🏿' => 'fir ag coraíocht: ton cnis geal, ton cnis dorcha',
    '👨🏼‍🫯‍👨🏻' => 'fir ag coraíocht: ton cnis mheánach-gheal, ton cnis geal',
    '👨🏼‍🫯‍👨🏽' => 'fir ag coraíocht: ton cnis mheánach-gheal, ton cnis meánach',
    '👨🏼‍🫯‍👨🏾' => 'fir ag coraíocht: ton cnis mheánach-gheal, ton cnis meándorcha',
    '👨🏼‍🫯‍👨🏿' => 'fir ag coraíocht: ton cnis mheánach-gheal, ton cnis dorcha',
    '👨🏽‍🫯‍👨🏻' => 'fir ag coraíocht: ton cnis meánach, ton cnis geal',
    '👨🏽‍🫯‍👨🏼' => 'fir ag coraíocht: ton cnis meánach, ton cnis mheánach-gheal',
    '👨🏽‍🫯‍👨🏾' => 'fir ag coraíocht: ton cnis meánach, ton cnis meándorcha',
    '👨🏽‍🫯‍👨🏿' => 'fir ag coraíocht: ton cnis meánach, ton cnis dorcha',
    '👨🏾‍🫯‍👨🏻' => 'fir ag coraíocht: ton cnis meándorcha, ton cnis geal',
    '👨🏾‍🫯‍👨🏼' => 'fir ag coraíocht: ton cnis meándorcha, ton cnis mheánach-gheal',
    '👨🏾‍🫯‍👨🏽' => 'fir ag coraíocht: ton cnis meándorcha, ton cnis meánach',
    '👨🏾‍🫯‍👨🏿' => 'fir ag coraíocht: ton cnis meándorcha, ton cnis dorcha',
    '👨🏿‍🫯‍👨🏻' => 'fir ag coraíocht: ton cnis dorcha, ton cnis geal',
    '👨🏿‍🫯‍👨🏼' => 'fir ag coraíocht: ton cnis dorcha, ton cnis mheánach-gheal',
    '👨🏿‍🫯‍👨🏽' => 'fir ag coraíocht: ton cnis dorcha, ton cnis meánach',
    '👨🏿‍🫯‍👨🏾' => 'fir ag coraíocht: ton cnis dorcha, ton cnis meándorcha',
    '👨🏻‍❤‍👨🏻' => 'lanúin le croí: fear, fear, ton cnis geal',
    '👨🏻‍❤‍👨🏼' => 'lanúin le croí: fear, fear, ton cnis geal, ton cnis mheánach-gheal',
    '👨🏻‍❤‍👨🏽' => 'lanúin le croí: fear, fear, ton cnis geal, ton cnis meánach',
    '👨🏻‍❤‍👨🏾' => 'lanúin le croí: fear, fear, ton cnis geal, ton cnis meándorcha',
    '👨🏻‍❤‍👨🏿' => 'lanúin le croí: fear, fear, ton cnis geal, ton cnis dorcha',
    '👨🏼‍❤‍👨🏻' => 'lanúin le croí: fear, fear, ton cnis mheánach-gheal, ton cnis geal',
    '👨🏼‍❤‍👨🏼' => 'lanúin le croí: fear, fear, ton cnis mheánach-gheal',
    '👨🏼‍❤‍👨🏽' => 'lanúin le croí: fear, fear, ton cnis mheánach-gheal, ton cnis meánach',
    '👨🏼‍❤‍👨🏾' => 'lanúin le croí: fear, fear, ton cnis mheánach-gheal, ton cnis meándorcha',
    '👨🏼‍❤‍👨🏿' => 'lanúin le croí: fear, fear, ton cnis mheánach-gheal, ton cnis dorcha',
    '👨🏽‍❤‍👨🏻' => 'lanúin le croí: fear, fear, ton cnis meánach, ton cnis geal',
    '👨🏽‍❤‍👨🏼' => 'lanúin le croí: fear, fear, ton cnis meánach, ton cnis mheánach-gheal',
    '👨🏽‍❤‍👨🏽' => 'lanúin le croí: fear, fear, ton cnis meánach',
    '👨🏽‍❤‍👨🏾' => 'lanúin le croí: fear, fear, ton cnis meánach, ton cnis meándorcha',
    '👨🏽‍❤‍👨🏿' => 'lanúin le croí: fear, fear, ton cnis meánach, ton cnis dorcha',
    '👨🏾‍❤‍👨🏻' => 'lanúin le croí: fear, fear, ton cnis meándorcha, ton cnis geal',
    '👨🏾‍❤‍👨🏼' => 'lanúin le croí: fear, fear, ton cnis meándorcha, ton cnis mheánach-gheal',
    '👨🏾‍❤‍👨🏽' => 'lanúin le croí: fear, fear, ton cnis meándorcha, ton cnis meánach',
    '👨🏾‍❤‍👨🏾' => 'lanúin le croí: fear, fear, ton cnis meándorcha',
    '👨🏾‍❤‍👨🏿' => 'lanúin le croí: fear, fear, ton cnis meándorcha, ton cnis dorcha',
    '👨🏿‍❤‍👨🏻' => 'lanúin le croí: fear, fear, ton cnis dorcha, ton cnis geal',
    '👨🏿‍❤‍👨🏼' => 'lanúin le croí: fear, fear, ton cnis dorcha, ton cnis mheánach-gheal',
    '👨🏿‍❤‍👨🏽' => 'lanúin le croí: fear, fear, ton cnis dorcha, ton cnis meánach',
    '👨🏿‍❤‍👨🏾' => 'lanúin le croí: fear, fear, ton cnis dorcha, ton cnis meándorcha',
    '👨🏿‍❤‍👨🏿' => 'lanúin le croí: fear, fear, ton cnis dorcha',
    '👨🏻‍🐰‍👨🏼' => 'fir le cluasa coiníní orthu: ton cnis geal, ton cnis mheánach-gheal',
    '👨🏻‍🐰‍👨🏽' => 'fir le cluasa coiníní orthu: ton cnis geal, ton cnis meánach',
    '👨🏻‍🐰‍👨🏾' => 'fir le cluasa coiníní orthu: ton cnis geal, ton cnis meándorcha',
    '👨🏻‍🐰‍👨🏿' => 'fir le cluasa coiníní orthu: ton cnis geal, ton cnis dorcha',
    '👨🏼‍🐰‍👨🏻' => 'fir le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis geal',
    '👨🏼‍🐰‍👨🏽' => 'fir le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis meánach',
    '👨🏼‍🐰‍👨🏾' => 'fir le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis meándorcha',
    '👨🏼‍🐰‍👨🏿' => 'fir le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis dorcha',
    '👨🏽‍🐰‍👨🏻' => 'fir le cluasa coiníní orthu: ton cnis meánach, ton cnis geal',
    '👨🏽‍🐰‍👨🏼' => 'fir le cluasa coiníní orthu: ton cnis meánach, ton cnis mheánach-gheal',
    '👨🏽‍🐰‍👨🏾' => 'fir le cluasa coiníní orthu: ton cnis meánach, ton cnis meándorcha',
    '👨🏽‍🐰‍👨🏿' => 'fir le cluasa coiníní orthu: ton cnis meánach, ton cnis dorcha',
    '👨🏾‍🐰‍👨🏻' => 'fir le cluasa coiníní orthu: ton cnis meándorcha, ton cnis geal',
    '👨🏾‍🐰‍👨🏼' => 'fir le cluasa coiníní orthu: ton cnis meándorcha, ton cnis mheánach-gheal',
    '👨🏾‍🐰‍👨🏽' => 'fir le cluasa coiníní orthu: ton cnis meándorcha, ton cnis meánach',
    '👨🏾‍🐰‍👨🏿' => 'fir le cluasa coiníní orthu: ton cnis meándorcha, ton cnis dorcha',
    '👨🏿‍🐰‍👨🏻' => 'fir le cluasa coiníní orthu: ton cnis dorcha, ton cnis geal',
    '👨🏿‍🐰‍👨🏼' => 'fir le cluasa coiníní orthu: ton cnis dorcha, ton cnis mheánach-gheal',
    '👨🏿‍🐰‍👨🏽' => 'fir le cluasa coiníní orthu: ton cnis dorcha, ton cnis meánach',
    '👨🏿‍🐰‍👨🏾' => 'fir le cluasa coiníní orthu: ton cnis dorcha, ton cnis meándorcha',
    '👩🏻‍🫯‍👩🏼' => 'mná ag coraíocht: ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍🫯‍👩🏽' => 'mná ag coraíocht: ton cnis geal, ton cnis meánach',
    '👩🏻‍🫯‍👩🏾' => 'mná ag coraíocht: ton cnis geal, ton cnis meándorcha',
    '👩🏻‍🫯‍👩🏿' => 'mná ag coraíocht: ton cnis geal, ton cnis dorcha',
    '👩🏼‍🫯‍👩🏻' => 'mná ag coraíocht: ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍🫯‍👩🏽' => 'mná ag coraíocht: ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍🫯‍👩🏾' => 'mná ag coraíocht: ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍🫯‍👩🏿' => 'mná ag coraíocht: ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍🫯‍👩🏻' => 'mná ag coraíocht: ton cnis meánach, ton cnis geal',
    '👩🏽‍🫯‍👩🏼' => 'mná ag coraíocht: ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍🫯‍👩🏾' => 'mná ag coraíocht: ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍🫯‍👩🏿' => 'mná ag coraíocht: ton cnis meánach, ton cnis dorcha',
    '👩🏾‍🫯‍👩🏻' => 'mná ag coraíocht: ton cnis meándorcha, ton cnis geal',
    '👩🏾‍🫯‍👩🏼' => 'mná ag coraíocht: ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍🫯‍👩🏽' => 'mná ag coraíocht: ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍🫯‍👩🏿' => 'mná ag coraíocht: ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍🫯‍👩🏻' => 'mná ag coraíocht: ton cnis dorcha, ton cnis geal',
    '👩🏿‍🫯‍👩🏼' => 'mná ag coraíocht: ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍🫯‍👩🏽' => 'mná ag coraíocht: ton cnis dorcha, ton cnis meánach',
    '👩🏿‍🫯‍👩🏾' => 'mná ag coraíocht: ton cnis dorcha, ton cnis meándorcha',
    '👩🏻‍❤‍👨🏻' => 'lanúin le croí: bean, fear, ton cnis geal',
    '👩🏻‍❤‍👨🏼' => 'lanúin le croí: bean, fear, ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍❤‍👨🏽' => 'lanúin le croí: bean, fear, ton cnis geal, ton cnis meánach',
    '👩🏻‍❤‍👨🏾' => 'lanúin le croí: bean, fear, ton cnis geal, ton cnis meándorcha',
    '👩🏻‍❤‍👨🏿' => 'lanúin le croí: bean, fear, ton cnis geal, ton cnis dorcha',
    '👩🏼‍❤‍👨🏻' => 'lanúin le croí: bean, fear, ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍❤‍👨🏼' => 'lanúin le croí: bean, fear, ton cnis mheánach-gheal',
    '👩🏼‍❤‍👨🏽' => 'lanúin le croí: bean, fear, ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍❤‍👨🏾' => 'lanúin le croí: bean, fear, ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍❤‍👨🏿' => 'lanúin le croí: bean, fear, ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍❤‍👨🏻' => 'lanúin le croí: bean, fear, ton cnis meánach, ton cnis geal',
    '👩🏽‍❤‍👨🏼' => 'lanúin le croí: bean, fear, ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍❤‍👨🏽' => 'lanúin le croí: bean, fear, ton cnis meánach',
    '👩🏽‍❤‍👨🏾' => 'lanúin le croí: bean, fear, ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍❤‍👨🏿' => 'lanúin le croí: bean, fear, ton cnis meánach, ton cnis dorcha',
    '👩🏾‍❤‍👨🏻' => 'lanúin le croí: bean, fear, ton cnis meándorcha, ton cnis geal',
    '👩🏾‍❤‍👨🏼' => 'lanúin le croí: bean, fear, ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍❤‍👨🏽' => 'lanúin le croí: bean, fear, ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍❤‍👨🏾' => 'lanúin le croí: bean, fear, ton cnis meándorcha',
    '👩🏾‍❤‍👨🏿' => 'lanúin le croí: bean, fear, ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍❤‍👨🏻' => 'lanúin le croí: bean, fear, ton cnis dorcha, ton cnis geal',
    '👩🏿‍❤‍👨🏼' => 'lanúin le croí: bean, fear, ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍❤‍👨🏽' => 'lanúin le croí: bean, fear, ton cnis dorcha, ton cnis meánach',
    '👩🏿‍❤‍👨🏾' => 'lanúin le croí: bean, fear, ton cnis dorcha, ton cnis meándorcha',
    '👩🏿‍❤‍👨🏿' => 'lanúin le croí: bean, fear, ton cnis dorcha',
    '👩🏻‍❤‍👩🏻' => 'lanúin le croí: bean, bean, ton cnis geal',
    '👩🏻‍❤‍👩🏼' => 'lanúin le croí: bean, bean, ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍❤‍👩🏽' => 'lanúin le croí: bean, bean, ton cnis geal, ton cnis meánach',
    '👩🏻‍❤‍👩🏾' => 'lanúin le croí: bean, bean, ton cnis geal, ton cnis meándorcha',
    '👩🏻‍❤‍👩🏿' => 'lanúin le croí: bean, bean, ton cnis geal, ton cnis dorcha',
    '👩🏼‍❤‍👩🏻' => 'lanúin le croí: bean, bean, ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍❤‍👩🏼' => 'lanúin le croí: bean, bean, ton cnis mheánach-gheal',
    '👩🏼‍❤‍👩🏽' => 'lanúin le croí: bean, bean, ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍❤‍👩🏾' => 'lanúin le croí: bean, bean, ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍❤‍👩🏿' => 'lanúin le croí: bean, bean, ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍❤‍👩🏻' => 'lanúin le croí: bean, bean, ton cnis meánach, ton cnis geal',
    '👩🏽‍❤‍👩🏼' => 'lanúin le croí: bean, bean, ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍❤‍👩🏽' => 'lanúin le croí: bean, bean, ton cnis meánach',
    '👩🏽‍❤‍👩🏾' => 'lanúin le croí: bean, bean, ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍❤‍👩🏿' => 'lanúin le croí: bean, bean, ton cnis meánach, ton cnis dorcha',
    '👩🏾‍❤‍👩🏻' => 'lanúin le croí: bean, bean, ton cnis meándorcha, ton cnis geal',
    '👩🏾‍❤‍👩🏼' => 'lanúin le croí: bean, bean, ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍❤‍👩🏽' => 'lanúin le croí: bean, bean, ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍❤‍👩🏾' => 'lanúin le croí: bean, bean, ton cnis meándorcha',
    '👩🏾‍❤‍👩🏿' => 'lanúin le croí: bean, bean, ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍❤‍👩🏻' => 'lanúin le croí: bean, bean, ton cnis dorcha, ton cnis geal',
    '👩🏿‍❤‍👩🏼' => 'lanúin le croí: bean, bean, ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍❤‍👩🏽' => 'lanúin le croí: bean, bean, ton cnis dorcha, ton cnis meánach',
    '👩🏿‍❤‍👩🏾' => 'lanúin le croí: bean, bean, ton cnis dorcha, ton cnis meándorcha',
    '👩🏿‍❤‍👩🏿' => 'lanúin le croí: bean, bean, ton cnis dorcha',
    '👩🏻‍🐰‍👩🏼' => 'mná le cluasa coiníní orthu: ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍🐰‍👩🏽' => 'mná le cluasa coiníní orthu: ton cnis geal, ton cnis meánach',
    '👩🏻‍🐰‍👩🏾' => 'mná le cluasa coiníní orthu: ton cnis geal, ton cnis meándorcha',
    '👩🏻‍🐰‍👩🏿' => 'mná le cluasa coiníní orthu: ton cnis geal, ton cnis dorcha',
    '👩🏼‍🐰‍👩🏻' => 'mná le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍🐰‍👩🏽' => 'mná le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍🐰‍👩🏾' => 'mná le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍🐰‍👩🏿' => 'mná le cluasa coiníní orthu: ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍🐰‍👩🏻' => 'mná le cluasa coiníní orthu: ton cnis meánach, ton cnis geal',
    '👩🏽‍🐰‍👩🏼' => 'mná le cluasa coiníní orthu: ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍🐰‍👩🏾' => 'mná le cluasa coiníní orthu: ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍🐰‍👩🏿' => 'mná le cluasa coiníní orthu: ton cnis meánach, ton cnis dorcha',
    '👩🏾‍🐰‍👩🏻' => 'mná le cluasa coiníní orthu: ton cnis meándorcha, ton cnis geal',
    '👩🏾‍🐰‍👩🏼' => 'mná le cluasa coiníní orthu: ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍🐰‍👩🏽' => 'mná le cluasa coiníní orthu: ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍🐰‍👩🏿' => 'mná le cluasa coiníní orthu: ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍🐰‍👩🏻' => 'mná le cluasa coiníní orthu: ton cnis dorcha, ton cnis geal',
    '👩🏿‍🐰‍👩🏼' => 'mná le cluasa coiníní orthu: ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍🐰‍👩🏽' => 'mná le cluasa coiníní orthu: ton cnis dorcha, ton cnis meánach',
    '👩🏿‍🐰‍👩🏾' => 'mná le cluasa coiníní orthu: ton cnis dorcha, ton cnis meándorcha',
    '🧑🏻‍🤝‍🧑🏻' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis geal',
    '🧑🏻‍🤝‍🧑🏼' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis geal, ton cnis mheánach-gheal',
    '🧑🏻‍🤝‍🧑🏽' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis geal, ton cnis meánach',
    '🧑🏻‍🤝‍🧑🏾' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis geal, ton cnis meándorcha',
    '🧑🏻‍🤝‍🧑🏿' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis geal, ton cnis dorcha',
    '🧑🏼‍🤝‍🧑🏻' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis mheánach-gheal, ton cnis geal',
    '🧑🏼‍🤝‍🧑🏼' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis mheánach-gheal',
    '🧑🏼‍🤝‍🧑🏽' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis mheánach-gheal, ton cnis meánach',
    '🧑🏼‍🤝‍🧑🏾' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis mheánach-gheal, ton cnis meándorcha',
    '🧑🏼‍🤝‍🧑🏿' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis mheánach-gheal, ton cnis dorcha',
    '🧑🏽‍🤝‍🧑🏻' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meánach, ton cnis geal',
    '🧑🏽‍🤝‍🧑🏼' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meánach, ton cnis mheánach-gheal',
    '🧑🏽‍🤝‍🧑🏽' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meánach',
    '🧑🏽‍🤝‍🧑🏾' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meánach, ton cnis meándorcha',
    '🧑🏽‍🤝‍🧑🏿' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meánach, ton cnis dorcha',
    '🧑🏾‍🤝‍🧑🏻' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meándorcha, ton cnis geal',
    '🧑🏾‍🤝‍🧑🏼' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meándorcha, ton cnis mheánach-gheal',
    '🧑🏾‍🤝‍🧑🏽' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meándorcha, ton cnis meánach',
    '🧑🏾‍🤝‍🧑🏾' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meándorcha',
    '🧑🏾‍🤝‍🧑🏿' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis meándorcha, ton cnis dorcha',
    '🧑🏿‍🤝‍🧑🏻' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis dorcha, ton cnis geal',
    '🧑🏿‍🤝‍🧑🏼' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis dorcha, ton cnis mheánach-gheal',
    '🧑🏿‍🤝‍🧑🏽' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis dorcha, ton cnis meánach',
    '🧑🏿‍🤝‍🧑🏾' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis dorcha, ton cnis meándorcha',
    '🧑🏿‍🤝‍🧑🏿' => 'daone a bhfuil greim láimhe acu ar a chéile: ton cnis dorcha',
    '👩🏻‍🤝‍👩🏼' => 'beirt bhan le chéile: ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍🤝‍👩🏽' => 'beirt bhan le chéile: ton cnis geal, ton cnis meánach',
    '👩🏻‍🤝‍👩🏾' => 'beirt bhan le chéile: ton cnis geal, ton cnis meándorcha',
    '👩🏻‍🤝‍👩🏿' => 'beirt bhan le chéile: ton cnis geal, ton cnis dorcha',
    '👩🏼‍🤝‍👩🏻' => 'beirt bhan le chéile: ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍🤝‍👩🏽' => 'beirt bhan le chéile: ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍🤝‍👩🏾' => 'beirt bhan le chéile: ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍🤝‍👩🏿' => 'beirt bhan le chéile: ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍🤝‍👩🏻' => 'beirt bhan le chéile: ton cnis meánach, ton cnis geal',
    '👩🏽‍🤝‍👩🏼' => 'beirt bhan le chéile: ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍🤝‍👩🏾' => 'beirt bhan le chéile: ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍🤝‍👩🏿' => 'beirt bhan le chéile: ton cnis meánach, ton cnis dorcha',
    '👩🏾‍🤝‍👩🏻' => 'beirt bhan le chéile: ton cnis meándorcha, ton cnis geal',
    '👩🏾‍🤝‍👩🏼' => 'beirt bhan le chéile: ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍🤝‍👩🏽' => 'beirt bhan le chéile: ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍🤝‍👩🏿' => 'beirt bhan le chéile: ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍🤝‍👩🏻' => 'beirt bhan le chéile: ton cnis dorcha, ton cnis geal',
    '👩🏿‍🤝‍👩🏼' => 'beirt bhan le chéile: ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍🤝‍👩🏽' => 'beirt bhan le chéile: ton cnis dorcha, ton cnis meánach',
    '👩🏿‍🤝‍👩🏾' => 'beirt bhan le chéile: ton cnis dorcha, ton cnis meándorcha',
    '👩🏻‍🤝‍👨🏼' => 'fear agus bean le chéile: ton cnis geal, ton cnis mheánach-gheal',
    '👩🏻‍🤝‍👨🏽' => 'fear agus bean le chéile: ton cnis geal, ton cnis meánach',
    '👩🏻‍🤝‍👨🏾' => 'fear agus bean le chéile: ton cnis geal, ton cnis meándorcha',
    '👩🏻‍🤝‍👨🏿' => 'fear agus bean le chéile: ton cnis geal, ton cnis dorcha',
    '👩🏼‍🤝‍👨🏻' => 'fear agus bean le chéile: ton cnis mheánach-gheal, ton cnis geal',
    '👩🏼‍🤝‍👨🏽' => 'fear agus bean le chéile: ton cnis mheánach-gheal, ton cnis meánach',
    '👩🏼‍🤝‍👨🏾' => 'fear agus bean le chéile: ton cnis mheánach-gheal, ton cnis meándorcha',
    '👩🏼‍🤝‍👨🏿' => 'fear agus bean le chéile: ton cnis mheánach-gheal, ton cnis dorcha',
    '👩🏽‍🤝‍👨🏻' => 'fear agus bean le chéile: ton cnis meánach, ton cnis geal',
    '👩🏽‍🤝‍👨🏼' => 'fear agus bean le chéile: ton cnis meánach, ton cnis mheánach-gheal',
    '👩🏽‍🤝‍👨🏾' => 'fear agus bean le chéile: ton cnis meánach, ton cnis meándorcha',
    '👩🏽‍🤝‍👨🏿' => 'fear agus bean le chéile: ton cnis meánach, ton cnis dorcha',
    '👩🏾‍🤝‍👨🏻' => 'fear agus bean le chéile: ton cnis meándorcha, ton cnis geal',
    '👩🏾‍🤝‍👨🏼' => 'fear agus bean le chéile: ton cnis meándorcha, ton cnis mheánach-gheal',
    '👩🏾‍🤝‍👨🏽' => 'fear agus bean le chéile: ton cnis meándorcha, ton cnis meánach',
    '👩🏾‍🤝‍👨🏿' => 'fear agus bean le chéile: ton cnis meándorcha, ton cnis dorcha',
    '👩🏿‍🤝‍👨🏻' => 'fear agus bean le chéile: ton cnis dorcha, ton cnis geal',
    '👩🏿‍🤝‍👨🏼' => 'fear agus bean le chéile: ton cnis dorcha, ton cnis mheánach-gheal',
    '👩🏿‍🤝‍👨🏽' => 'fear agus bean le chéile: ton cnis dorcha, ton cnis meánach',
    '👩🏿‍🤝‍👨🏾' => 'fear agus bean le chéile: ton cnis dorcha, ton cnis meándorcha',
    '👨🏻‍🤝‍👨🏼' => 'beirt fhear le chéile: ton cnis geal, ton cnis mheánach-gheal',
    '👨🏻‍🤝‍👨🏽' => 'beirt fhear le chéile: ton cnis geal, ton cnis meánach',
    '👨🏻‍🤝‍👨🏾' => 'beirt fhear le chéile: ton cnis geal, ton cnis meándorcha',
    '👨🏻‍🤝‍👨🏿' => 'beirt fhear le chéile: ton cnis geal, ton cnis dorcha',
    '👨🏼‍🤝‍👨🏻' => 'beirt fhear le chéile: ton cnis mheánach-gheal, ton cnis geal',
    '👨🏼‍🤝‍👨🏽' => 'beirt fhear le chéile: ton cnis mheánach-gheal, ton cnis meánach',
    '👨🏼‍🤝‍👨🏾' => 'beirt fhear le chéile: ton cnis mheánach-gheal, ton cnis meándorcha',
    '👨🏼‍🤝‍👨🏿' => 'beirt fhear le chéile: ton cnis mheánach-gheal, ton cnis dorcha',
    '👨🏽‍🤝‍👨🏻' => 'beirt fhear le chéile: ton cnis meánach, ton cnis geal',
    '👨🏽‍🤝‍👨🏼' => 'beirt fhear le chéile: ton cnis meánach, ton cnis mheánach-gheal',
    '👨🏽‍🤝‍👨🏾' => 'beirt fhear le chéile: ton cnis meánach, ton cnis meándorcha',
    '👨🏽‍🤝‍👨🏿' => 'beirt fhear le chéile: ton cnis meánach, ton cnis dorcha',
    '👨🏾‍🤝‍👨🏻' => 'beirt fhear le chéile: ton cnis meándorcha, ton cnis geal',
    '👨🏾‍🤝‍👨🏼' => 'beirt fhear le chéile: ton cnis meándorcha, ton cnis mheánach-gheal',
    '👨🏾‍🤝‍👨🏽' => 'beirt fhear le chéile: ton cnis meándorcha, ton cnis meánach',
    '👨🏾‍🤝‍👨🏿' => 'beirt fhear le chéile: ton cnis meándorcha, ton cnis dorcha',
    '👨🏿‍🤝‍👨🏻' => 'beirt fhear le chéile: ton cnis dorcha, ton cnis geal',
    '👨🏿‍🤝‍👨🏼' => 'beirt fhear le chéile: ton cnis dorcha, ton cnis mheánach-gheal',
    '👨🏿‍🤝‍👨🏽' => 'beirt fhear le chéile: ton cnis dorcha, ton cnis meánach',
    '👨🏿‍🤝‍👨🏾' => 'beirt fhear le chéile: ton cnis dorcha, ton cnis meándorcha',
    '👩‍❤‍💋‍👨' => 'póg: bean, fear',
    '👨‍❤‍💋‍👨' => 'póg: fear, fear',
    '👩‍❤‍💋‍👩' => 'póg: bean, bean',
    '👨‍👩‍👧‍👦' => 'teaghlach: fear, bean, cailín, buachaill',
    '👨‍👩‍👦‍👦' => 'teaghlach: fear, bean, buachaill, buachaill',
    '👨‍👩‍👧‍👧' => 'teaghlach: fear, bean, cailín, cailín',
    '👨‍👨‍👧‍👦' => 'teaghlach: fear, fear, cailín, buachaill',
    '👨‍👨‍👦‍👦' => 'teaghlach: fear, fear, buachaill, buachaill',
    '👨‍👨‍👧‍👧' => 'teaghlach: fear, fear, cailín, cailín',
    '👩‍👩‍👧‍👦' => 'teaghlach: bean, bean, cailín, buachaill',
    '👩‍👩‍👦‍👦' => 'teaghlach: bean, bean, buachaill, buachaill',
    '👩‍👩‍👧‍👧' => 'teaghlach: bean, bean, cailín, cailín',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'bratach: Sasana',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'bratach: Albain',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'bratach: an Bhreatain Bheag',
    '🧑‍🧑‍🧒‍🧒' => 'teaghlach: duine fásta, duine fásta, leanbh, leanbh',
    '🚶🏻‍♀‍➡' => 'bean ag siúl: ton cnis geal, aghaidh ar dheis',
    '🚶🏼‍♀‍➡' => 'bean ag siúl: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🚶🏽‍♀‍➡' => 'bean ag siúl: ton cnis meánach, aghaidh ar dheis',
    '🚶🏾‍♀‍➡' => 'bean ag siúl: ton cnis meándorcha, aghaidh ar dheis',
    '🚶🏿‍♀‍➡' => 'bean ag siúl: ton cnis dorcha, aghaidh ar dheis',
    '🚶🏻‍♂‍➡' => 'fear ag siúl: ton cnis geal, aghaidh ar dheis',
    '🚶🏼‍♂‍➡' => 'fear ag siúl: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🚶🏽‍♂‍➡' => 'fear ag siúl: ton cnis meánach, aghaidh ar dheis',
    '🚶🏾‍♂‍➡' => 'fear ag siúl: ton cnis meándorcha, aghaidh ar dheis',
    '🚶🏿‍♂‍➡' => 'fear ag siúl: ton cnis dorcha, aghaidh ar dheis',
    '🧎🏻‍♀‍➡' => 'bean ar a glúine: ton cnis geal, aghaidh ar dheis',
    '🧎🏼‍♀‍➡' => 'bean ar a glúine: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🧎🏽‍♀‍➡' => 'bean ar a glúine: ton cnis meánach, aghaidh ar dheis',
    '🧎🏾‍♀‍➡' => 'bean ar a glúine: ton cnis meándorcha, aghaidh ar dheis',
    '🧎🏿‍♀‍➡' => 'bean ar a glúine: ton cnis dorcha, aghaidh ar dheis',
    '🧎🏻‍♂‍➡' => 'fear ar a ghlúine: ton cnis geal, aghaidh ar dheis',
    '🧎🏼‍♂‍➡' => 'fear ar a ghlúine: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🧎🏽‍♂‍➡' => 'fear ar a ghlúine: ton cnis meánach, aghaidh ar dheis',
    '🧎🏾‍♂‍➡' => 'fear ar a ghlúine: ton cnis meándorcha, aghaidh ar dheis',
    '🧎🏿‍♂‍➡' => 'fear ar a ghlúine: ton cnis dorcha, aghaidh ar dheis',
    '🧑🏻‍🦯‍➡' => 'duine le cána bán: ton cnis geal, aghaidh ar dheis',
    '🧑🏼‍🦯‍➡' => 'duine le cána bán: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🧑🏽‍🦯‍➡' => 'duine le cána bán: ton cnis meánach, aghaidh ar dheis',
    '🧑🏾‍🦯‍➡' => 'duine le cána bán: ton cnis meándorcha, aghaidh ar dheis',
    '🧑🏿‍🦯‍➡' => 'duine le cána bán: ton cnis dorcha, aghaidh ar dheis',
    '👨🏻‍🦯‍➡' => 'fear le cána bán: ton cnis geal, aghaidh ar dheis',
    '👨🏼‍🦯‍➡' => 'fear le cána bán: ton cnis mheánach-gheal, aghaidh ar dheis',
    '👨🏽‍🦯‍➡' => 'fear le cána bán: ton cnis meánach, aghaidh ar dheis',
    '👨🏾‍🦯‍➡' => 'fear le cána bán: ton cnis meándorcha, aghaidh ar dheis',
    '👨🏿‍🦯‍➡' => 'fear le cána bán: ton cnis dorcha, aghaidh ar dheis',
    '👩🏻‍🦯‍➡' => 'bean le cána bán: ton cnis geal, aghaidh ar dheis',
    '👩🏼‍🦯‍➡' => 'bean le cána bán: ton cnis mheánach-gheal, aghaidh ar dheis',
    '👩🏽‍🦯‍➡' => 'bean le cána bán: ton cnis meánach, aghaidh ar dheis',
    '👩🏾‍🦯‍➡' => 'bean le cána bán: ton cnis meándorcha, aghaidh ar dheis',
    '👩🏿‍🦯‍➡' => 'bean le cána bán: ton cnis dorcha, aghaidh ar dheis',
    '🧑🏻‍🦼‍➡' => 'duine i gcathaoir rothaí innealta: ton cnis geal, aghaidh ar dheis',
    '🧑🏼‍🦼‍➡' => 'duine i gcathaoir rothaí innealta: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🧑🏽‍🦼‍➡' => 'duine i gcathaoir rothaí innealta: ton cnis meánach, aghaidh ar dheis',
    '🧑🏾‍🦼‍➡' => 'duine i gcathaoir rothaí innealta: ton cnis meándorcha, aghaidh ar dheis',
    '🧑🏿‍🦼‍➡' => 'duine i gcathaoir rothaí innealta: ton cnis dorcha, aghaidh ar dheis',
    '👨🏻‍🦼‍➡' => 'fear i cathaoir rothaí innealta: ton cnis geal, aghaidh ar dheis',
    '👨🏼‍🦼‍➡' => 'fear i cathaoir rothaí innealta: ton cnis mheánach-gheal, aghaidh ar dheis',
    '👨🏽‍🦼‍➡' => 'fear i cathaoir rothaí innealta: ton cnis meánach, aghaidh ar dheis',
    '👨🏾‍🦼‍➡' => 'fear i cathaoir rothaí innealta: ton cnis meándorcha, aghaidh ar dheis',
    '👨🏿‍🦼‍➡' => 'fear i cathaoir rothaí innealta: ton cnis dorcha, aghaidh ar dheis',
    '👩🏻‍🦼‍➡' => 'bean i gcathaoir rothaí innealta: ton cnis geal, aghaidh ar dheis',
    '👩🏼‍🦼‍➡' => 'bean i gcathaoir rothaí innealta: ton cnis mheánach-gheal, aghaidh ar dheis',
    '👩🏽‍🦼‍➡' => 'bean i gcathaoir rothaí innealta: ton cnis meánach, aghaidh ar dheis',
    '👩🏾‍🦼‍➡' => 'bean i gcathaoir rothaí innealta: ton cnis meándorcha, aghaidh ar dheis',
    '👩🏿‍🦼‍➡' => 'bean i gcathaoir rothaí innealta: ton cnis dorcha, aghaidh ar dheis',
    '🧑🏻‍🦽‍➡' => 'duine i gcathaoir rothaí láimhe: ton cnis geal, aghaidh ar dheis',
    '🧑🏼‍🦽‍➡' => 'duine i gcathaoir rothaí láimhe: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🧑🏽‍🦽‍➡' => 'duine i gcathaoir rothaí láimhe: ton cnis meánach, aghaidh ar dheis',
    '🧑🏾‍🦽‍➡' => 'duine i gcathaoir rothaí láimhe: ton cnis meándorcha, aghaidh ar dheis',
    '🧑🏿‍🦽‍➡' => 'duine i gcathaoir rothaí láimhe: ton cnis dorcha, aghaidh ar dheis',
    '👨🏻‍🦽‍➡' => 'fear i gcathaoir rothaí láimhe: ton cnis geal, aghaidh ar dheis',
    '👨🏼‍🦽‍➡' => 'fear i gcathaoir rothaí láimhe: ton cnis mheánach-gheal, aghaidh ar dheis',
    '👨🏽‍🦽‍➡' => 'fear i gcathaoir rothaí láimhe: ton cnis meánach, aghaidh ar dheis',
    '👨🏾‍🦽‍➡' => 'fear i gcathaoir rothaí láimhe: ton cnis meándorcha, aghaidh ar dheis',
    '👨🏿‍🦽‍➡' => 'fear i gcathaoir rothaí láimhe: ton cnis dorcha, aghaidh ar dheis',
    '👩🏻‍🦽‍➡' => 'bean i gcathaoir rothaí láimhe: ton cnis geal, aghaidh ar dheis',
    '👩🏼‍🦽‍➡' => 'bean i gcathaoir rothaí láimhe: ton cnis mheánach-gheal, aghaidh ar dheis',
    '👩🏽‍🦽‍➡' => 'bean i gcathaoir rothaí láimhe: ton cnis meánach, aghaidh ar dheis',
    '👩🏾‍🦽‍➡' => 'bean i gcathaoir rothaí láimhe: ton cnis meándorcha, aghaidh ar dheis',
    '👩🏿‍🦽‍➡' => 'bean i gcathaoir rothaí láimhe: ton cnis dorcha, aghaidh ar dheis',
    '🏃🏻‍♀‍➡' => 'bean ag rith: ton cnis geal, aghaidh ar dheis',
    '🏃🏼‍♀‍➡' => 'bean ag rith: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🏃🏽‍♀‍➡' => 'bean ag rith: ton cnis meánach, aghaidh ar dheis',
    '🏃🏾‍♀‍➡' => 'bean ag rith: ton cnis meándorcha, aghaidh ar dheis',
    '🏃🏿‍♀‍➡' => 'bean ag rith: ton cnis dorcha, aghaidh ar dheis',
    '🏃🏻‍♂‍➡' => 'fear ag rith: ton cnis geal, aghaidh ar dheis',
    '🏃🏼‍♂‍➡' => 'fear ag rith: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🏃🏽‍♂‍➡' => 'fear ag rith: ton cnis meánach, aghaidh ar dheis',
    '🏃🏾‍♂‍➡' => 'fear ag rith: ton cnis meándorcha, aghaidh ar dheis',
    '🏃🏿‍♂‍➡' => 'fear ag rith: ton cnis dorcha, aghaidh ar dheis',
    '🫱🏻‍🫲🏼' => 'croitheadh lámh: ton cnis geal, ton cnis mheánach-gheal',
    '🫱🏻‍🫲🏽' => 'croitheadh lámh: ton cnis geal, ton cnis meánach',
    '🫱🏻‍🫲🏾' => 'croitheadh lámh: ton cnis geal, ton cnis meándorcha',
    '🫱🏻‍🫲🏿' => 'croitheadh lámh: ton cnis geal, ton cnis dorcha',
    '🫱🏼‍🫲🏻' => 'croitheadh lámh: ton cnis mheánach-gheal, ton cnis geal',
    '🫱🏼‍🫲🏽' => 'croitheadh lámh: ton cnis mheánach-gheal, ton cnis meánach',
    '🫱🏼‍🫲🏾' => 'croitheadh lámh: ton cnis mheánach-gheal, ton cnis meándorcha',
    '🫱🏼‍🫲🏿' => 'croitheadh lámh: ton cnis mheánach-gheal, ton cnis dorcha',
    '🫱🏽‍🫲🏻' => 'croitheadh lámh: ton cnis meánach, ton cnis geal',
    '🫱🏽‍🫲🏼' => 'croitheadh lámh: ton cnis meánach, ton cnis mheánach-gheal',
    '🫱🏽‍🫲🏾' => 'croitheadh lámh: ton cnis meánach, ton cnis meándorcha',
    '🫱🏽‍🫲🏿' => 'croitheadh lámh: ton cnis meánach, ton cnis dorcha',
    '🫱🏾‍🫲🏻' => 'croitheadh lámh: ton cnis meándorcha, ton cnis geal',
    '🫱🏾‍🫲🏼' => 'croitheadh lámh: ton cnis meándorcha, ton cnis mheánach-gheal',
    '🫱🏾‍🫲🏽' => 'croitheadh lámh: ton cnis meándorcha, ton cnis meánach',
    '🫱🏾‍🫲🏿' => 'croitheadh lámh: ton cnis meándorcha, ton cnis dorcha',
    '🫱🏿‍🫲🏻' => 'croitheadh lámh: ton cnis dorcha, ton cnis geal',
    '🫱🏿‍🫲🏼' => 'croitheadh lámh: ton cnis dorcha, ton cnis mheánach-gheal',
    '🫱🏿‍🫲🏽' => 'croitheadh lámh: ton cnis dorcha, ton cnis meánach',
    '🫱🏿‍🫲🏾' => 'croitheadh lámh: ton cnis dorcha, ton cnis meándorcha',
    '🚶‍♀‍➡' => 'bean ag siúl: aghaidh ar dheis',
    '🚶‍♂‍➡' => 'fear ag siúl: aghaidh ar dheis',
    '🧎‍♀‍➡' => 'bean ar a glúine: aghaidh ar dheis',
    '🧎‍♂‍➡' => 'fear ar a ghlúine: aghaidh ar dheis',
    '🧑‍🦯‍➡' => 'duine le cána bán: aghaidh ar dheis',
    '👨‍🦯‍➡' => 'fear le cána bán: aghaidh ar dheis',
    '👩‍🦯‍➡' => 'bean le cána bán: aghaidh ar dheis',
    '🧑‍🦼‍➡' => 'duine i gcathaoir rothaí innealta: aghaidh ar dheis',
    '👨‍🦼‍➡' => 'fear i cathaoir rothaí innealta: aghaidh ar dheis',
    '👩‍🦼‍➡' => 'bean i gcathaoir rothaí innealta: aghaidh ar dheis',
    '🧑‍🦽‍➡' => 'duine i gcathaoir rothaí láimhe: aghaidh ar dheis',
    '👨‍🦽‍➡' => 'fear i gcathaoir rothaí láimhe: aghaidh ar dheis',
    '👩‍🦽‍➡' => 'bean i gcathaoir rothaí láimhe: aghaidh ar dheis',
    '🏃‍♀‍➡' => 'bean ag rith: aghaidh ar dheis',
    '🏃‍♂‍➡' => 'fear ag rith: aghaidh ar dheis',
    '👩‍❤‍👨' => 'lanúin le croí: bean, fear',
    '👨‍❤‍👨' => 'lanúin le croí: fear, fear',
    '👩‍❤‍👩' => 'lanúin le croí: bean, bean',
    '👨‍👩‍👦' => 'teaghlach: fear, bean, buachaill',
    '👨‍👩‍👧' => 'teaghlach: fear, bean, cailín',
    '👨‍👨‍👦' => 'teaghlach: fear, fear, buachaill',
    '👨‍👨‍👧' => 'teaghlach: fear, fear, cailín',
    '👩‍👩‍👦' => 'teaghlach: bean, bean, buachaill',
    '👩‍👩‍👧' => 'teaghlach: bean, bean, cailín',
    '👨‍👦‍👦' => 'teaghlach: fear, buachaill, buachaill',
    '👨‍👧‍👦' => 'teaghlach: fear, cailín, buachaill',
    '👨‍👧‍👧' => 'teaghlach: fear, cailín, cailín',
    '👩‍👦‍👦' => 'teaghlach: bean, buachaill, buachaill',
    '👩‍👧‍👦' => 'teaghlach: bean, cailín, buachaill',
    '👩‍👧‍👧' => 'teaghlach: bean, cailín, cailín',
    '🧑‍🤝‍🧑' => 'daone a bhfuil greim láimhe acu ar a chéile',
    '🧑‍🧑‍🧒' => 'teaghlach: duine fásta, duine fásta, leanbh',
    '🧑‍🧒‍🧒' => 'teaghlach: duine fásta, leanbh, leanbh',
    '🧑🏻‍🦰' => 'aosach: ton cnis geal, gruaig rua',
    '🧑🏼‍🦰' => 'aosach: ton cnis mheánach-gheal, gruaig rua',
    '🧑🏽‍🦰' => 'aosach: ton cnis meánach, gruaig rua',
    '🧑🏾‍🦰' => 'aosach: ton cnis meándorcha, gruaig rua',
    '🧑🏿‍🦰' => 'aosach: ton cnis dorcha, gruaig rua',
    '🧑🏻‍🦱' => 'aosach: ton cnis geal, gruaig chatach',
    '🧑🏼‍🦱' => 'aosach: ton cnis mheánach-gheal, gruaig chatach',
    '🧑🏽‍🦱' => 'aosach: ton cnis meánach, gruaig chatach',
    '🧑🏾‍🦱' => 'aosach: ton cnis meándorcha, gruaig chatach',
    '🧑🏿‍🦱' => 'aosach: ton cnis dorcha, gruaig chatach',
    '🧑🏻‍🦳' => 'aosach: ton cnis geal, gruaig bhán',
    '🧑🏼‍🦳' => 'aosach: ton cnis mheánach-gheal, gruaig bhán',
    '🧑🏽‍🦳' => 'aosach: ton cnis meánach, gruaig bhán',
    '🧑🏾‍🦳' => 'aosach: ton cnis meándorcha, gruaig bhán',
    '🧑🏿‍🦳' => 'aosach: ton cnis dorcha, gruaig bhán',
    '🧑🏻‍🦲' => 'aosach: ton cnis geal, maol',
    '🧑🏼‍🦲' => 'aosach: ton cnis mheánach-gheal, maol',
    '🧑🏽‍🦲' => 'aosach: ton cnis meánach, maol',
    '🧑🏾‍🦲' => 'aosach: ton cnis meándorcha, maol',
    '🧑🏿‍🦲' => 'aosach: ton cnis dorcha, maol',
    '🧑🏻‍🩰' => 'damhsóir bailé: ton cnis geal',
    '🧑🏼‍🩰' => 'damhsóir bailé: ton cnis mheánach-gheal',
    '🧑🏽‍🩰' => 'damhsóir bailé: ton cnis meánach',
    '🧑🏾‍🩰' => 'damhsóir bailé: ton cnis meándorcha',
    '🧑🏿‍🩰' => 'damhsóir bailé: ton cnis dorcha',
    '🧔🏻‍♂' => 'fear: ton cnis geal, feasóg',
    '🧔🏼‍♂' => 'fear: ton cnis mheánach-gheal, feasóg',
    '🧔🏽‍♂' => 'fear: ton cnis meánach, feasóg',
    '🧔🏾‍♂' => 'fear: ton cnis meándorcha, feasóg',
    '🧔🏿‍♂' => 'fear: ton cnis dorcha, feasóg',
    '👱🏻‍♂' => 'fear: ton cnis geal, gruaig fhionn',
    '👱🏼‍♂' => 'fear: ton cnis mheánach-gheal, gruaig fhionn',
    '👱🏽‍♂' => 'fear: ton cnis meánach, gruaig fhionn',
    '👱🏾‍♂' => 'fear: ton cnis meándorcha, gruaig fhionn',
    '👱🏿‍♂' => 'fear: ton cnis dorcha, gruaig fhionn',
    '👨🏻‍🦰' => 'fear: ton cnis geal, gruaig rua',
    '👨🏼‍🦰' => 'fear: ton cnis mheánach-gheal, gruaig rua',
    '👨🏽‍🦰' => 'fear: ton cnis meánach, gruaig rua',
    '👨🏾‍🦰' => 'fear: ton cnis meándorcha, gruaig rua',
    '👨🏿‍🦰' => 'fear: ton cnis dorcha, gruaig rua',
    '👨🏻‍🦱' => 'fear: ton cnis geal, gruaig chatach',
    '👨🏼‍🦱' => 'fear: ton cnis mheánach-gheal, gruaig chatach',
    '👨🏽‍🦱' => 'fear: ton cnis meánach, gruaig chatach',
    '👨🏾‍🦱' => 'fear: ton cnis meándorcha, gruaig chatach',
    '👨🏿‍🦱' => 'fear: ton cnis dorcha, gruaig chatach',
    '👨🏻‍🦳' => 'fear: ton cnis geal, gruaig bhán',
    '👨🏼‍🦳' => 'fear: ton cnis mheánach-gheal, gruaig bhán',
    '👨🏽‍🦳' => 'fear: ton cnis meánach, gruaig bhán',
    '👨🏾‍🦳' => 'fear: ton cnis meándorcha, gruaig bhán',
    '👨🏿‍🦳' => 'fear: ton cnis dorcha, gruaig bhán',
    '👨🏻‍🦲' => 'fear: ton cnis geal, maol',
    '👨🏼‍🦲' => 'fear: ton cnis mheánach-gheal, maol',
    '👨🏽‍🦲' => 'fear: ton cnis meánach, maol',
    '👨🏾‍🦲' => 'fear: ton cnis meándorcha, maol',
    '👨🏿‍🦲' => 'fear: ton cnis dorcha, maol',
    '🧔🏻‍♀' => 'bean: ton cnis geal, feasóg',
    '🧔🏼‍♀' => 'bean: ton cnis mheánach-gheal, feasóg',
    '🧔🏽‍♀' => 'bean: ton cnis meánach, feasóg',
    '🧔🏾‍♀' => 'bean: ton cnis meándorcha, feasóg',
    '🧔🏿‍♀' => 'bean: ton cnis dorcha, feasóg',
    '👱🏻‍♀' => 'bean: ton cnis geal, gruaig fhionn',
    '👱🏼‍♀' => 'bean: ton cnis mheánach-gheal, gruaig fhionn',
    '👱🏽‍♀' => 'bean: ton cnis meánach, gruaig fhionn',
    '👱🏾‍♀' => 'bean: ton cnis meándorcha, gruaig fhionn',
    '👱🏿‍♀' => 'bean: ton cnis dorcha, gruaig fhionn',
    '👩🏻‍🦰' => 'bean: ton cnis geal, gruaig rua',
    '👩🏼‍🦰' => 'bean: ton cnis mheánach-gheal, gruaig rua',
    '👩🏽‍🦰' => 'bean: ton cnis meánach, gruaig rua',
    '👩🏾‍🦰' => 'bean: ton cnis meándorcha, gruaig rua',
    '👩🏿‍🦰' => 'bean: ton cnis dorcha, gruaig rua',
    '👩🏻‍🦱' => 'bean: ton cnis geal, gruaig chatach',
    '👩🏼‍🦱' => 'bean: ton cnis mheánach-gheal, gruaig chatach',
    '👩🏽‍🦱' => 'bean: ton cnis meánach, gruaig chatach',
    '👩🏾‍🦱' => 'bean: ton cnis meándorcha, gruaig chatach',
    '👩🏿‍🦱' => 'bean: ton cnis dorcha, gruaig chatach',
    '👩🏻‍🦳' => 'bean: ton cnis geal, gruaig bhán',
    '👩🏼‍🦳' => 'bean: ton cnis mheánach-gheal, gruaig bhán',
    '👩🏽‍🦳' => 'bean: ton cnis meánach, gruaig bhán',
    '👩🏾‍🦳' => 'bean: ton cnis meándorcha, gruaig bhán',
    '👩🏿‍🦳' => 'bean: ton cnis dorcha, gruaig bhán',
    '👩🏻‍🦲' => 'bean: ton cnis geal, maol',
    '👩🏼‍🦲' => 'bean: ton cnis mheánach-gheal, maol',
    '👩🏽‍🦲' => 'bean: ton cnis meánach, maol',
    '👩🏾‍🦲' => 'bean: ton cnis meándorcha, maol',
    '👩🏿‍🦲' => 'bean: ton cnis dorcha, maol',
    '🙍🏻‍♂' => 'fear le púic air: ton cnis geal',
    '🙍🏼‍♂' => 'fear le púic air: ton cnis mheánach-gheal',
    '🙍🏽‍♂' => 'fear le púic air: ton cnis meánach',
    '🙍🏾‍♂' => 'fear le púic air: ton cnis meándorcha',
    '🙍🏿‍♂' => 'fear le púic air: ton cnis dorcha',
    '🙍🏻‍♀' => 'bean le púic uirthi: ton cnis geal',
    '🙍🏼‍♀' => 'bean le púic uirthi: ton cnis mheánach-gheal',
    '🙍🏽‍♀' => 'bean le púic uirthi: ton cnis meánach',
    '🙍🏾‍♀' => 'bean le púic uirthi: ton cnis meándorcha',
    '🙍🏿‍♀' => 'bean le púic uirthi: ton cnis dorcha',
    '🙎🏻‍♂' => 'fear le puisín air: ton cnis geal',
    '🙎🏼‍♂' => 'fear le puisín air: ton cnis mheánach-gheal',
    '🙎🏽‍♂' => 'fear le puisín air: ton cnis meánach',
    '🙎🏾‍♂' => 'fear le puisín air: ton cnis meándorcha',
    '🙎🏿‍♂' => 'fear le puisín air: ton cnis dorcha',
    '🙎🏻‍♀' => 'bean le puisín uirthi: ton cnis geal',
    '🙎🏼‍♀' => 'bean le puisín uirthi: ton cnis mheánach-gheal',
    '🙎🏽‍♀' => 'bean le puisín uirthi: ton cnis meánach',
    '🙎🏾‍♀' => 'bean le puisín uirthi: ton cnis meándorcha',
    '🙎🏿‍♀' => 'bean le puisín uirthi: ton cnis dorcha',
    '🙅🏻‍♂' => 'fear ag comharthú NÍL: ton cnis geal',
    '🙅🏼‍♂' => 'fear ag comharthú NÍL: ton cnis mheánach-gheal',
    '🙅🏽‍♂' => 'fear ag comharthú NÍL: ton cnis meánach',
    '🙅🏾‍♂' => 'fear ag comharthú NÍL: ton cnis meándorcha',
    '🙅🏿‍♂' => 'fear ag comharthú NÍL: ton cnis dorcha',
    '🙅🏻‍♀' => 'bean ag comharthú NÍL: ton cnis geal',
    '🙅🏼‍♀' => 'bean ag comharthú NÍL: ton cnis mheánach-gheal',
    '🙅🏽‍♀' => 'bean ag comharthú NÍL: ton cnis meánach',
    '🙅🏾‍♀' => 'bean ag comharthú NÍL: ton cnis meándorcha',
    '🙅🏿‍♀' => 'bean ag comharthú NÍL: ton cnis dorcha',
    '🙆🏻‍♂' => 'fear ag comharthú OK: ton cnis geal',
    '🙆🏼‍♂' => 'fear ag comharthú OK: ton cnis mheánach-gheal',
    '🙆🏽‍♂' => 'fear ag comharthú OK: ton cnis meánach',
    '🙆🏾‍♂' => 'fear ag comharthú OK: ton cnis meándorcha',
    '🙆🏿‍♂' => 'fear ag comharthú OK: ton cnis dorcha',
    '🙆🏻‍♀' => 'bean ag comharthú OK: ton cnis geal',
    '🙆🏼‍♀' => 'bean ag comharthú OK: ton cnis mheánach-gheal',
    '🙆🏽‍♀' => 'bean ag comharthú OK: ton cnis meánach',
    '🙆🏾‍♀' => 'bean ag comharthú OK: ton cnis meándorcha',
    '🙆🏿‍♀' => 'bean ag comharthú OK: ton cnis dorcha',
    '💁🏻‍♂' => 'fear lena lámh amach: ton cnis geal',
    '💁🏼‍♂' => 'fear lena lámh amach: ton cnis mheánach-gheal',
    '💁🏽‍♂' => 'fear lena lámh amach: ton cnis meánach',
    '💁🏾‍♂' => 'fear lena lámh amach: ton cnis meándorcha',
    '💁🏿‍♂' => 'fear lena lámh amach: ton cnis dorcha',
    '💁🏻‍♀' => 'bean lena lámh amach: ton cnis geal',
    '💁🏼‍♀' => 'bean lena lámh amach: ton cnis mheánach-gheal',
    '💁🏽‍♀' => 'bean lena lámh amach: ton cnis meánach',
    '💁🏾‍♀' => 'bean lena lámh amach: ton cnis meándorcha',
    '💁🏿‍♀' => 'bean lena lámh amach: ton cnis dorcha',
    '🙋🏻‍♂' => 'fear lena lámh suas: ton cnis geal',
    '🙋🏼‍♂' => 'fear lena lámh suas: ton cnis mheánach-gheal',
    '🙋🏽‍♂' => 'fear lena lámh suas: ton cnis meánach',
    '🙋🏾‍♂' => 'fear lena lámh suas: ton cnis meándorcha',
    '🙋🏿‍♂' => 'fear lena lámh suas: ton cnis dorcha',
    '🙋🏻‍♀' => 'bean lena lámh suas: ton cnis geal',
    '🙋🏼‍♀' => 'bean lena lámh suas: ton cnis mheánach-gheal',
    '🙋🏽‍♀' => 'bean lena lámh suas: ton cnis meánach',
    '🙋🏾‍♀' => 'bean lena lámh suas: ton cnis meándorcha',
    '🙋🏿‍♀' => 'bean lena lámh suas: ton cnis dorcha',
    '🧏🏻‍♂' => 'fear bodhar: ton cnis geal',
    '🧏🏼‍♂' => 'fear bodhar: ton cnis mheánach-gheal',
    '🧏🏽‍♂' => 'fear bodhar: ton cnis meánach',
    '🧏🏾‍♂' => 'fear bodhar: ton cnis meándorcha',
    '🧏🏿‍♂' => 'fear bodhar: ton cnis dorcha',
    '🧏🏻‍♀' => 'bean bhodhar: ton cnis geal',
    '🧏🏼‍♀' => 'bean bhodhar: ton cnis mheánach-gheal',
    '🧏🏽‍♀' => 'bean bhodhar: ton cnis meánach',
    '🧏🏾‍♀' => 'bean bhodhar: ton cnis meándorcha',
    '🧏🏿‍♀' => 'bean bhodhar: ton cnis dorcha',
    '🙇🏻‍♂' => 'fear ag umhlú: ton cnis geal',
    '🙇🏼‍♂' => 'fear ag umhlú: ton cnis mheánach-gheal',
    '🙇🏽‍♂' => 'fear ag umhlú: ton cnis meánach',
    '🙇🏾‍♂' => 'fear ag umhlú: ton cnis meándorcha',
    '🙇🏿‍♂' => 'fear ag umhlú: ton cnis dorcha',
    '🙇🏻‍♀' => 'bean ag umhlú: ton cnis geal',
    '🙇🏼‍♀' => 'bean ag umhlú: ton cnis mheánach-gheal',
    '🙇🏽‍♀' => 'bean ag umhlú: ton cnis meánach',
    '🙇🏾‍♀' => 'bean ag umhlú: ton cnis meándorcha',
    '🙇🏿‍♀' => 'bean ag umhlú: ton cnis dorcha',
    '🤦🏻‍♂' => 'fear lena aghaidh ina bhos: ton cnis geal',
    '🤦🏼‍♂' => 'fear lena aghaidh ina bhos: ton cnis mheánach-gheal',
    '🤦🏽‍♂' => 'fear lena aghaidh ina bhos: ton cnis meánach',
    '🤦🏾‍♂' => 'fear lena aghaidh ina bhos: ton cnis meándorcha',
    '🤦🏿‍♂' => 'fear lena aghaidh ina bhos: ton cnis dorcha',
    '🤦🏻‍♀' => 'bean lena haghaidh ina bos: ton cnis geal',
    '🤦🏼‍♀' => 'bean lena haghaidh ina bos: ton cnis mheánach-gheal',
    '🤦🏽‍♀' => 'bean lena haghaidh ina bos: ton cnis meánach',
    '🤦🏾‍♀' => 'bean lena haghaidh ina bos: ton cnis meándorcha',
    '🤦🏿‍♀' => 'bean lena haghaidh ina bos: ton cnis dorcha',
    '🤷🏻‍♂' => 'fear ag croitheadh: ton cnis geal',
    '🤷🏼‍♂' => 'fear ag croitheadh: ton cnis mheánach-gheal',
    '🤷🏽‍♂' => 'fear ag croitheadh: ton cnis meánach',
    '🤷🏾‍♂' => 'fear ag croitheadh: ton cnis meándorcha',
    '🤷🏿‍♂' => 'fear ag croitheadh: ton cnis dorcha',
    '🤷🏻‍♀' => 'bean ag croitheadh: ton cnis geal',
    '🤷🏼‍♀' => 'bean ag croitheadh: ton cnis mheánach-gheal',
    '🤷🏽‍♀' => 'bean ag croitheadh: ton cnis meánach',
    '🤷🏾‍♀' => 'bean ag croitheadh: ton cnis meándorcha',
    '🤷🏿‍♀' => 'bean ag croitheadh: ton cnis dorcha',
    '🧑🏻‍⚕' => 'oibrí sláinte: ton cnis geal',
    '🧑🏼‍⚕' => 'oibrí sláinte: ton cnis mheánach-gheal',
    '🧑🏽‍⚕' => 'oibrí sláinte: ton cnis meánach',
    '🧑🏾‍⚕' => 'oibrí sláinte: ton cnis meándorcha',
    '🧑🏿‍⚕' => 'oibrí sláinte: ton cnis dorcha',
    '👨🏻‍⚕' => 'oibrí sláinte fireann: ton cnis geal',
    '👨🏼‍⚕' => 'oibrí sláinte fireann: ton cnis mheánach-gheal',
    '👨🏽‍⚕' => 'oibrí sláinte fireann: ton cnis meánach',
    '👨🏾‍⚕' => 'oibrí sláinte fireann: ton cnis meándorcha',
    '👨🏿‍⚕' => 'oibrí sláinte fireann: ton cnis dorcha',
    '👩🏻‍⚕' => 'oibrí sláinte baineann: ton cnis geal',
    '👩🏼‍⚕' => 'oibrí sláinte baineann: ton cnis mheánach-gheal',
    '👩🏽‍⚕' => 'oibrí sláinte baineann: ton cnis meánach',
    '👩🏾‍⚕' => 'oibrí sláinte baineann: ton cnis meándorcha',
    '👩🏿‍⚕' => 'oibrí sláinte baineann: ton cnis dorcha',
    '🧑🏻‍🎓' => 'mac léinn: ton cnis geal',
    '🧑🏼‍🎓' => 'mac léinn: ton cnis mheánach-gheal',
    '🧑🏽‍🎓' => 'mac léinn: ton cnis meánach',
    '🧑🏾‍🎓' => 'mac léinn: ton cnis meándorcha',
    '🧑🏿‍🎓' => 'mac léinn: ton cnis dorcha',
    '👨🏻‍🎓' => 'scoláire fir: ton cnis geal',
    '👨🏼‍🎓' => 'scoláire fir: ton cnis mheánach-gheal',
    '👨🏽‍🎓' => 'scoláire fir: ton cnis meánach',
    '👨🏾‍🎓' => 'scoláire fir: ton cnis meándorcha',
    '👨🏿‍🎓' => 'scoláire fir: ton cnis dorcha',
    '👩🏻‍🎓' => 'scoláire baineann: ton cnis geal',
    '👩🏼‍🎓' => 'scoláire baineann: ton cnis mheánach-gheal',
    '👩🏽‍🎓' => 'scoláire baineann: ton cnis meánach',
    '👩🏾‍🎓' => 'scoláire baineann: ton cnis meándorcha',
    '👩🏿‍🎓' => 'scoláire baineann: ton cnis dorcha',
    '🧑🏻‍🏫' => 'múinteoir: ton cnis geal',
    '🧑🏼‍🏫' => 'múinteoir: ton cnis mheánach-gheal',
    '🧑🏽‍🏫' => 'múinteoir: ton cnis meánach',
    '🧑🏾‍🏫' => 'múinteoir: ton cnis meándorcha',
    '🧑🏿‍🏫' => 'múinteoir: ton cnis dorcha',
    '👨🏻‍🏫' => 'múinteoir fireann: ton cnis geal',
    '👨🏼‍🏫' => 'múinteoir fireann: ton cnis mheánach-gheal',
    '👨🏽‍🏫' => 'múinteoir fireann: ton cnis meánach',
    '👨🏾‍🏫' => 'múinteoir fireann: ton cnis meándorcha',
    '👨🏿‍🏫' => 'múinteoir fireann: ton cnis dorcha',
    '👩🏻‍🏫' => 'múinteoir baineann: ton cnis geal',
    '👩🏼‍🏫' => 'múinteoir baineann: ton cnis mheánach-gheal',
    '👩🏽‍🏫' => 'múinteoir baineann: ton cnis meánach',
    '👩🏾‍🏫' => 'múinteoir baineann: ton cnis meándorcha',
    '👩🏿‍🏫' => 'múinteoir baineann: ton cnis dorcha',
    '🧑🏻‍⚖' => 'breitheamh: ton cnis geal',
    '🧑🏼‍⚖' => 'breitheamh: ton cnis mheánach-gheal',
    '🧑🏽‍⚖' => 'breitheamh: ton cnis meánach',
    '🧑🏾‍⚖' => 'breitheamh: ton cnis meándorcha',
    '🧑🏿‍⚖' => 'breitheamh: ton cnis dorcha',
    '👨🏻‍⚖' => 'breitheamh fireann: ton cnis geal',
    '👨🏼‍⚖' => 'breitheamh fireann: ton cnis mheánach-gheal',
    '👨🏽‍⚖' => 'breitheamh fireann: ton cnis meánach',
    '👨🏾‍⚖' => 'breitheamh fireann: ton cnis meándorcha',
    '👨🏿‍⚖' => 'breitheamh fireann: ton cnis dorcha',
    '👩🏻‍⚖' => 'breitheamh baineann: ton cnis geal',
    '👩🏼‍⚖' => 'breitheamh baineann: ton cnis mheánach-gheal',
    '👩🏽‍⚖' => 'breitheamh baineann: ton cnis meánach',
    '👩🏾‍⚖' => 'breitheamh baineann: ton cnis meándorcha',
    '👩🏿‍⚖' => 'breitheamh baineann: ton cnis dorcha',
    '🧑🏻‍🌾' => 'feirmeoir: ton cnis geal',
    '🧑🏼‍🌾' => 'feirmeoir: ton cnis mheánach-gheal',
    '🧑🏽‍🌾' => 'feirmeoir: ton cnis meánach',
    '🧑🏾‍🌾' => 'feirmeoir: ton cnis meándorcha',
    '🧑🏿‍🌾' => 'feirmeoir: ton cnis dorcha',
    '👨🏻‍🌾' => 'feirmeoir fireann: ton cnis geal',
    '👨🏼‍🌾' => 'feirmeoir fireann: ton cnis mheánach-gheal',
    '👨🏽‍🌾' => 'feirmeoir fireann: ton cnis meánach',
    '👨🏾‍🌾' => 'feirmeoir fireann: ton cnis meándorcha',
    '👨🏿‍🌾' => 'feirmeoir fireann: ton cnis dorcha',
    '👩🏻‍🌾' => 'feirmeoir baineann: ton cnis geal',
    '👩🏼‍🌾' => 'feirmeoir baineann: ton cnis mheánach-gheal',
    '👩🏽‍🌾' => 'feirmeoir baineann: ton cnis meánach',
    '👩🏾‍🌾' => 'feirmeoir baineann: ton cnis meándorcha',
    '👩🏿‍🌾' => 'feirmeoir baineann: ton cnis dorcha',
    '🧑🏻‍🍳' => 'cócaire: ton cnis geal',
    '🧑🏼‍🍳' => 'cócaire: ton cnis mheánach-gheal',
    '🧑🏽‍🍳' => 'cócaire: ton cnis meánach',
    '🧑🏾‍🍳' => 'cócaire: ton cnis meándorcha',
    '🧑🏿‍🍳' => 'cócaire: ton cnis dorcha',
    '👨🏻‍🍳' => 'cócaire fireann: ton cnis geal',
    '👨🏼‍🍳' => 'cócaire fireann: ton cnis mheánach-gheal',
    '👨🏽‍🍳' => 'cócaire fireann: ton cnis meánach',
    '👨🏾‍🍳' => 'cócaire fireann: ton cnis meándorcha',
    '👨🏿‍🍳' => 'cócaire fireann: ton cnis dorcha',
    '👩🏻‍🍳' => 'cócaire baineann: ton cnis geal',
    '👩🏼‍🍳' => 'cócaire baineann: ton cnis mheánach-gheal',
    '👩🏽‍🍳' => 'cócaire baineann: ton cnis meánach',
    '👩🏾‍🍳' => 'cócaire baineann: ton cnis meándorcha',
    '👩🏿‍🍳' => 'cócaire baineann: ton cnis dorcha',
    '🧑🏻‍🔧' => 'meicneoir: ton cnis geal',
    '🧑🏼‍🔧' => 'meicneoir: ton cnis mheánach-gheal',
    '🧑🏽‍🔧' => 'meicneoir: ton cnis meánach',
    '🧑🏾‍🔧' => 'meicneoir: ton cnis meándorcha',
    '🧑🏿‍🔧' => 'meicneoir: ton cnis dorcha',
    '👨🏻‍🔧' => 'meicneoir fireann: ton cnis geal',
    '👨🏼‍🔧' => 'meicneoir fireann: ton cnis mheánach-gheal',
    '👨🏽‍🔧' => 'meicneoir fireann: ton cnis meánach',
    '👨🏾‍🔧' => 'meicneoir fireann: ton cnis meándorcha',
    '👨🏿‍🔧' => 'meicneoir fireann: ton cnis dorcha',
    '👩🏻‍🔧' => 'meicneoir baineann: ton cnis geal',
    '👩🏼‍🔧' => 'meicneoir baineann: ton cnis mheánach-gheal',
    '👩🏽‍🔧' => 'meicneoir baineann: ton cnis meánach',
    '👩🏾‍🔧' => 'meicneoir baineann: ton cnis meándorcha',
    '👩🏿‍🔧' => 'meicneoir baineann: ton cnis dorcha',
    '🧑🏻‍🏭' => 'oibrí monarchan: ton cnis geal',
    '🧑🏼‍🏭' => 'oibrí monarchan: ton cnis mheánach-gheal',
    '🧑🏽‍🏭' => 'oibrí monarchan: ton cnis meánach',
    '🧑🏾‍🏭' => 'oibrí monarchan: ton cnis meándorcha',
    '🧑🏿‍🏭' => 'oibrí monarchan: ton cnis dorcha',
    '👨🏻‍🏭' => 'oibrí monarchan fireann: ton cnis geal',
    '👨🏼‍🏭' => 'oibrí monarchan fireann: ton cnis mheánach-gheal',
    '👨🏽‍🏭' => 'oibrí monarchan fireann: ton cnis meánach',
    '👨🏾‍🏭' => 'oibrí monarchan fireann: ton cnis meándorcha',
    '👨🏿‍🏭' => 'oibrí monarchan fireann: ton cnis dorcha',
    '👩🏻‍🏭' => 'oibrí monarchan baineann: ton cnis geal',
    '👩🏼‍🏭' => 'oibrí monarchan baineann: ton cnis mheánach-gheal',
    '👩🏽‍🏭' => 'oibrí monarchan baineann: ton cnis meánach',
    '👩🏾‍🏭' => 'oibrí monarchan baineann: ton cnis meándorcha',
    '👩🏿‍🏭' => 'oibrí monarchan baineann: ton cnis dorcha',
    '🧑🏻‍💼' => 'oibrí oifige: ton cnis geal',
    '🧑🏼‍💼' => 'oibrí oifige: ton cnis mheánach-gheal',
    '🧑🏽‍💼' => 'oibrí oifige: ton cnis meánach',
    '🧑🏾‍💼' => 'oibrí oifige: ton cnis meándorcha',
    '🧑🏿‍💼' => 'oibrí oifige: ton cnis dorcha',
    '👨🏻‍💼' => 'oibrí oifige fireann: ton cnis geal',
    '👨🏼‍💼' => 'oibrí oifige fireann: ton cnis mheánach-gheal',
    '👨🏽‍💼' => 'oibrí oifige fireann: ton cnis meánach',
    '👨🏾‍💼' => 'oibrí oifige fireann: ton cnis meándorcha',
    '👨🏿‍💼' => 'oibrí oifige fireann: ton cnis dorcha',
    '👩🏻‍💼' => 'oibrí oifige baineann: ton cnis geal',
    '👩🏼‍💼' => 'oibrí oifige baineann: ton cnis mheánach-gheal',
    '👩🏽‍💼' => 'oibrí oifige baineann: ton cnis meánach',
    '👩🏾‍💼' => 'oibrí oifige baineann: ton cnis meándorcha',
    '👩🏿‍💼' => 'oibrí oifige baineann: ton cnis dorcha',
    '🧑🏻‍🔬' => 'eolaí: ton cnis geal',
    '🧑🏼‍🔬' => 'eolaí: ton cnis mheánach-gheal',
    '🧑🏽‍🔬' => 'eolaí: ton cnis meánach',
    '🧑🏾‍🔬' => 'eolaí: ton cnis meándorcha',
    '🧑🏿‍🔬' => 'eolaí: ton cnis dorcha',
    '👨🏻‍🔬' => 'eolaí fireann: ton cnis geal',
    '👨🏼‍🔬' => 'eolaí fireann: ton cnis mheánach-gheal',
    '👨🏽‍🔬' => 'eolaí fireann: ton cnis meánach',
    '👨🏾‍🔬' => 'eolaí fireann: ton cnis meándorcha',
    '👨🏿‍🔬' => 'eolaí fireann: ton cnis dorcha',
    '👩🏻‍🔬' => 'eolaí baineann: ton cnis geal',
    '👩🏼‍🔬' => 'eolaí baineann: ton cnis mheánach-gheal',
    '👩🏽‍🔬' => 'eolaí baineann: ton cnis meánach',
    '👩🏾‍🔬' => 'eolaí baineann: ton cnis meándorcha',
    '👩🏿‍🔬' => 'eolaí baineann: ton cnis dorcha',
    '🧑🏻‍💻' => 'teicneolaí: ton cnis geal',
    '🧑🏼‍💻' => 'teicneolaí: ton cnis mheánach-gheal',
    '🧑🏽‍💻' => 'teicneolaí: ton cnis meánach',
    '🧑🏾‍💻' => 'teicneolaí: ton cnis meándorcha',
    '🧑🏿‍💻' => 'teicneolaí: ton cnis dorcha',
    '👨🏻‍💻' => 'teicneolaí fireann: ton cnis geal',
    '👨🏼‍💻' => 'teicneolaí fireann: ton cnis mheánach-gheal',
    '👨🏽‍💻' => 'teicneolaí fireann: ton cnis meánach',
    '👨🏾‍💻' => 'teicneolaí fireann: ton cnis meándorcha',
    '👨🏿‍💻' => 'teicneolaí fireann: ton cnis dorcha',
    '👩🏻‍💻' => 'teicneolaí baineann: ton cnis geal',
    '👩🏼‍💻' => 'teicneolaí baineann: ton cnis mheánach-gheal',
    '👩🏽‍💻' => 'teicneolaí baineann: ton cnis meánach',
    '👩🏾‍💻' => 'teicneolaí baineann: ton cnis meándorcha',
    '👩🏿‍💻' => 'teicneolaí baineann: ton cnis dorcha',
    '🧑🏻‍🎤' => 'amhránaí: ton cnis geal',
    '🧑🏼‍🎤' => 'amhránaí: ton cnis mheánach-gheal',
    '🧑🏽‍🎤' => 'amhránaí: ton cnis meánach',
    '🧑🏾‍🎤' => 'amhránaí: ton cnis meándorcha',
    '🧑🏿‍🎤' => 'amhránaí: ton cnis dorcha',
    '👨🏻‍🎤' => 'amhránaí fireann: ton cnis geal',
    '👨🏼‍🎤' => 'amhránaí fireann: ton cnis mheánach-gheal',
    '👨🏽‍🎤' => 'amhránaí fireann: ton cnis meánach',
    '👨🏾‍🎤' => 'amhránaí fireann: ton cnis meándorcha',
    '👨🏿‍🎤' => 'amhránaí fireann: ton cnis dorcha',
    '👩🏻‍🎤' => 'amhránaí baineann: ton cnis geal',
    '👩🏼‍🎤' => 'amhránaí baineann: ton cnis mheánach-gheal',
    '👩🏽‍🎤' => 'amhránaí baineann: ton cnis meánach',
    '👩🏾‍🎤' => 'amhránaí baineann: ton cnis meándorcha',
    '👩🏿‍🎤' => 'amhránaí baineann: ton cnis dorcha',
    '🧑🏻‍🎨' => 'ealaíontóir: ton cnis geal',
    '🧑🏼‍🎨' => 'ealaíontóir: ton cnis mheánach-gheal',
    '🧑🏽‍🎨' => 'ealaíontóir: ton cnis meánach',
    '🧑🏾‍🎨' => 'ealaíontóir: ton cnis meándorcha',
    '🧑🏿‍🎨' => 'ealaíontóir: ton cnis dorcha',
    '👨🏻‍🎨' => 'ealaíontóir fearann: ton cnis geal',
    '👨🏼‍🎨' => 'ealaíontóir fearann: ton cnis mheánach-gheal',
    '👨🏽‍🎨' => 'ealaíontóir fearann: ton cnis meánach',
    '👨🏾‍🎨' => 'ealaíontóir fearann: ton cnis meándorcha',
    '👨🏿‍🎨' => 'ealaíontóir fearann: ton cnis dorcha',
    '👩🏻‍🎨' => 'ealaíontóir baineann: ton cnis geal',
    '👩🏼‍🎨' => 'ealaíontóir baineann: ton cnis mheánach-gheal',
    '👩🏽‍🎨' => 'ealaíontóir baineann: ton cnis meánach',
    '👩🏾‍🎨' => 'ealaíontóir baineann: ton cnis meándorcha',
    '👩🏿‍🎨' => 'ealaíontóir baineann: ton cnis dorcha',
    '🧑🏻‍✈' => 'píolóta: ton cnis geal',
    '🧑🏼‍✈' => 'píolóta: ton cnis mheánach-gheal',
    '🧑🏽‍✈' => 'píolóta: ton cnis meánach',
    '🧑🏾‍✈' => 'píolóta: ton cnis meándorcha',
    '🧑🏿‍✈' => 'píolóta: ton cnis dorcha',
    '👨🏻‍✈' => 'píolóta fireann: ton cnis geal',
    '👨🏼‍✈' => 'píolóta fireann: ton cnis mheánach-gheal',
    '👨🏽‍✈' => 'píolóta fireann: ton cnis meánach',
    '👨🏾‍✈' => 'píolóta fireann: ton cnis meándorcha',
    '👨🏿‍✈' => 'píolóta fireann: ton cnis dorcha',
    '👩🏻‍✈' => 'píolóta baineann: ton cnis geal',
    '👩🏼‍✈' => 'píolóta baineann: ton cnis mheánach-gheal',
    '👩🏽‍✈' => 'píolóta baineann: ton cnis meánach',
    '👩🏾‍✈' => 'píolóta baineann: ton cnis meándorcha',
    '👩🏿‍✈' => 'píolóta baineann: ton cnis dorcha',
    '🧑🏻‍🚀' => 'spásaire: ton cnis geal',
    '🧑🏼‍🚀' => 'spásaire: ton cnis mheánach-gheal',
    '🧑🏽‍🚀' => 'spásaire: ton cnis meánach',
    '🧑🏾‍🚀' => 'spásaire: ton cnis meándorcha',
    '🧑🏿‍🚀' => 'spásaire: ton cnis dorcha',
    '👨🏻‍🚀' => 'spásaire fireann: ton cnis geal',
    '👨🏼‍🚀' => 'spásaire fireann: ton cnis mheánach-gheal',
    '👨🏽‍🚀' => 'spásaire fireann: ton cnis meánach',
    '👨🏾‍🚀' => 'spásaire fireann: ton cnis meándorcha',
    '👨🏿‍🚀' => 'spásaire fireann: ton cnis dorcha',
    '👩🏻‍🚀' => 'spásaire baineann: ton cnis geal',
    '👩🏼‍🚀' => 'spásaire baineann: ton cnis mheánach-gheal',
    '👩🏽‍🚀' => 'spásaire baineann: ton cnis meánach',
    '👩🏾‍🚀' => 'spásaire baineann: ton cnis meándorcha',
    '👩🏿‍🚀' => 'spásaire baineann: ton cnis dorcha',
    '🧑🏻‍🚒' => 'comhraiceoir tine: ton cnis geal',
    '🧑🏼‍🚒' => 'comhraiceoir tine: ton cnis mheánach-gheal',
    '🧑🏽‍🚒' => 'comhraiceoir tine: ton cnis meánach',
    '🧑🏾‍🚒' => 'comhraiceoir tine: ton cnis meándorcha',
    '🧑🏿‍🚒' => 'comhraiceoir tine: ton cnis dorcha',
    '👨🏻‍🚒' => 'fear múchta dóiteán: ton cnis geal',
    '👨🏼‍🚒' => 'fear múchta dóiteán: ton cnis mheánach-gheal',
    '👨🏽‍🚒' => 'fear múchta dóiteán: ton cnis meánach',
    '👨🏾‍🚒' => 'fear múchta dóiteán: ton cnis meándorcha',
    '👨🏿‍🚒' => 'fear múchta dóiteán: ton cnis dorcha',
    '👩🏻‍🚒' => 'bean mhúchta dóiteán: ton cnis geal',
    '👩🏼‍🚒' => 'bean mhúchta dóiteán: ton cnis mheánach-gheal',
    '👩🏽‍🚒' => 'bean mhúchta dóiteán: ton cnis meánach',
    '👩🏾‍🚒' => 'bean mhúchta dóiteán: ton cnis meándorcha',
    '👩🏿‍🚒' => 'bean mhúchta dóiteán: ton cnis dorcha',
    '👮🏻‍♂' => 'garda: ton cnis geal',
    '👮🏼‍♂' => 'garda: ton cnis mheánach-gheal',
    '👮🏽‍♂' => 'garda: ton cnis meánach',
    '👮🏾‍♂' => 'garda: ton cnis meándorcha',
    '👮🏿‍♂' => 'garda: ton cnis dorcha',
    '👮🏻‍♀' => 'bangharda: ton cnis geal',
    '👮🏼‍♀' => 'bangharda: ton cnis mheánach-gheal',
    '👮🏽‍♀' => 'bangharda: ton cnis meánach',
    '👮🏾‍♀' => 'bangharda: ton cnis meándorcha',
    '👮🏿‍♀' => 'bangharda: ton cnis dorcha',
    '🕵🏻‍♂' => 'bleachtaire fireann: ton cnis geal',
    '🕵🏼‍♂' => 'bleachtaire fireann: ton cnis mheánach-gheal',
    '🕵🏽‍♂' => 'bleachtaire fireann: ton cnis meánach',
    '🕵🏾‍♂' => 'bleachtaire fireann: ton cnis meándorcha',
    '🕵🏿‍♂' => 'bleachtaire fireann: ton cnis dorcha',
    '🕵🏻‍♀' => 'bleachtaire baineann: ton cnis geal',
    '🕵🏼‍♀' => 'bleachtaire baineann: ton cnis mheánach-gheal',
    '🕵🏽‍♀' => 'bleachtaire baineann: ton cnis meánach',
    '🕵🏾‍♀' => 'bleachtaire baineann: ton cnis meándorcha',
    '🕵🏿‍♀' => 'bleachtaire baineann: ton cnis dorcha',
    '💂🏻‍♂' => 'garda oifigeach: ton cnis geal',
    '💂🏼‍♂' => 'garda oifigeach: ton cnis mheánach-gheal',
    '💂🏽‍♂' => 'garda oifigeach: ton cnis meánach',
    '💂🏾‍♂' => 'garda oifigeach: ton cnis meándorcha',
    '💂🏿‍♂' => 'garda oifigeach: ton cnis dorcha',
    '💂🏻‍♀' => 'bangharda oifigeach: ton cnis geal',
    '💂🏼‍♀' => 'bangharda oifigeach: ton cnis mheánach-gheal',
    '💂🏽‍♀' => 'bangharda oifigeach: ton cnis meánach',
    '💂🏾‍♀' => 'bangharda oifigeach: ton cnis meándorcha',
    '💂🏿‍♀' => 'bangharda oifigeach: ton cnis dorcha',
    '👷🏻‍♂' => 'tógálaí fireann: ton cnis geal',
    '👷🏼‍♂' => 'tógálaí fireann: ton cnis mheánach-gheal',
    '👷🏽‍♂' => 'tógálaí fireann: ton cnis meánach',
    '👷🏾‍♂' => 'tógálaí fireann: ton cnis meándorcha',
    '👷🏿‍♂' => 'tógálaí fireann: ton cnis dorcha',
    '👷🏻‍♀' => 'tógálaí baineann: ton cnis geal',
    '👷🏼‍♀' => 'tógálaí baineann: ton cnis mheánach-gheal',
    '👷🏽‍♀' => 'tógálaí baineann: ton cnis meánach',
    '👷🏾‍♀' => 'tógálaí baineann: ton cnis meándorcha',
    '👷🏿‍♀' => 'tógálaí baineann: ton cnis dorcha',
    '👳🏻‍♂' => 'fear le turban air: ton cnis geal',
    '👳🏼‍♂' => 'fear le turban air: ton cnis mheánach-gheal',
    '👳🏽‍♂' => 'fear le turban air: ton cnis meánach',
    '👳🏾‍♂' => 'fear le turban air: ton cnis meándorcha',
    '👳🏿‍♂' => 'fear le turban air: ton cnis dorcha',
    '👳🏻‍♀' => 'bean le turban uirthi: ton cnis geal',
    '👳🏼‍♀' => 'bean le turban uirthi: ton cnis mheánach-gheal',
    '👳🏽‍♀' => 'bean le turban uirthi: ton cnis meánach',
    '👳🏾‍♀' => 'bean le turban uirthi: ton cnis meándorcha',
    '👳🏿‍♀' => 'bean le turban uirthi: ton cnis dorcha',
    '🤵🏻‍♂' => 'fear a bhfuil culaith dhinnéir air: ton cnis geal',
    '🤵🏼‍♂' => 'fear a bhfuil culaith dhinnéir air: ton cnis mheánach-gheal',
    '🤵🏽‍♂' => 'fear a bhfuil culaith dhinnéir air: ton cnis meánach',
    '🤵🏾‍♂' => 'fear a bhfuil culaith dhinnéir air: ton cnis meándorcha',
    '🤵🏿‍♂' => 'fear a bhfuil culaith dhinnéir air: ton cnis dorcha',
    '🤵🏻‍♀' => 'bean a bhfuil culaith dhinnéir uirthi: ton cnis geal',
    '🤵🏼‍♀' => 'bean a bhfuil culaith dhinnéir uirthi: ton cnis mheánach-gheal',
    '🤵🏽‍♀' => 'bean a bhfuil culaith dhinnéir uirthi: ton cnis meánach',
    '🤵🏾‍♀' => 'bean a bhfuil culaith dhinnéir uirthi: ton cnis meándorcha',
    '🤵🏿‍♀' => 'bean a bhfuil culaith dhinnéir uirthi: ton cnis dorcha',
    '👰🏻‍♂' => 'fear a bhfuil caille air: ton cnis geal',
    '👰🏼‍♂' => 'fear a bhfuil caille air: ton cnis mheánach-gheal',
    '👰🏽‍♂' => 'fear a bhfuil caille air: ton cnis meánach',
    '👰🏾‍♂' => 'fear a bhfuil caille air: ton cnis meándorcha',
    '👰🏿‍♂' => 'fear a bhfuil caille air: ton cnis dorcha',
    '👰🏻‍♀' => 'bean a bhfuil caille uirthi: ton cnis geal',
    '👰🏼‍♀' => 'bean a bhfuil caille uirthi: ton cnis mheánach-gheal',
    '👰🏽‍♀' => 'bean a bhfuil caille uirthi: ton cnis meánach',
    '👰🏾‍♀' => 'bean a bhfuil caille uirthi: ton cnis meándorcha',
    '👰🏿‍♀' => 'bean a bhfuil caille uirthi: ton cnis dorcha',
    '👩🏻‍🍼' => 'bean ag beathú linbh: ton cnis geal',
    '👩🏼‍🍼' => 'bean ag beathú linbh: ton cnis mheánach-gheal',
    '👩🏽‍🍼' => 'bean ag beathú linbh: ton cnis meánach',
    '👩🏾‍🍼' => 'bean ag beathú linbh: ton cnis meándorcha',
    '👩🏿‍🍼' => 'bean ag beathú linbh: ton cnis dorcha',
    '👨🏻‍🍼' => 'fear ag beathú linbh: ton cnis geal',
    '👨🏼‍🍼' => 'fear ag beathú linbh: ton cnis mheánach-gheal',
    '👨🏽‍🍼' => 'fear ag beathú linbh: ton cnis meánach',
    '👨🏾‍🍼' => 'fear ag beathú linbh: ton cnis meándorcha',
    '👨🏿‍🍼' => 'fear ag beathú linbh: ton cnis dorcha',
    '🧑🏻‍🍼' => 'duine ag beathú linbh: ton cnis geal',
    '🧑🏼‍🍼' => 'duine ag beathú linbh: ton cnis mheánach-gheal',
    '🧑🏽‍🍼' => 'duine ag beathú linbh: ton cnis meánach',
    '🧑🏾‍🍼' => 'duine ag beathú linbh: ton cnis meándorcha',
    '🧑🏿‍🍼' => 'duine ag beathú linbh: ton cnis dorcha',
    '🧑🏻‍🎄' => 'Bean Shaintí: ton cnis geal',
    '🧑🏼‍🎄' => 'Bean Shaintí: ton cnis mheánach-gheal',
    '🧑🏽‍🎄' => 'Bean Shaintí: ton cnis meánach',
    '🧑🏾‍🎄' => 'Bean Shaintí: ton cnis meándorcha',
    '🧑🏿‍🎄' => 'Bean Shaintí: ton cnis dorcha',
    '🦸🏻‍♂' => 'fear-sárlaoch: ton cnis geal',
    '🦸🏼‍♂' => 'fear-sárlaoch: ton cnis mheánach-gheal',
    '🦸🏽‍♂' => 'fear-sárlaoch: ton cnis meánach',
    '🦸🏾‍♂' => 'fear-sárlaoch: ton cnis meándorcha',
    '🦸🏿‍♂' => 'fear-sárlaoch: ton cnis dorcha',
    '🦸🏻‍♀' => 'ban-sárlaoch: ton cnis geal',
    '🦸🏼‍♀' => 'ban-sárlaoch: ton cnis mheánach-gheal',
    '🦸🏽‍♀' => 'ban-sárlaoch: ton cnis meánach',
    '🦸🏾‍♀' => 'ban-sárlaoch: ton cnis meándorcha',
    '🦸🏿‍♀' => 'ban-sárlaoch: ton cnis dorcha',
    '🦹🏻‍♂' => 'fear-dhrochdhuine: ton cnis geal',
    '🦹🏼‍♂' => 'fear-dhrochdhuine: ton cnis mheánach-gheal',
    '🦹🏽‍♂' => 'fear-dhrochdhuine: ton cnis meánach',
    '🦹🏾‍♂' => 'fear-dhrochdhuine: ton cnis meándorcha',
    '🦹🏿‍♂' => 'fear-dhrochdhuine: ton cnis dorcha',
    '🦹🏻‍♀' => 'ban-drochdhuine: ton cnis geal',
    '🦹🏼‍♀' => 'ban-drochdhuine: ton cnis mheánach-gheal',
    '🦹🏽‍♀' => 'ban-drochdhuine: ton cnis meánach',
    '🦹🏾‍♀' => 'ban-drochdhuine: ton cnis meándorcha',
    '🦹🏿‍♀' => 'ban-drochdhuine: ton cnis dorcha',
    '🧙🏻‍♂' => 'fear draíochta: ton cnis geal',
    '🧙🏼‍♂' => 'fear draíochta: ton cnis mheánach-gheal',
    '🧙🏽‍♂' => 'fear draíochta: ton cnis meánach',
    '🧙🏾‍♂' => 'fear draíochta: ton cnis meándorcha',
    '🧙🏿‍♂' => 'fear draíochta: ton cnis dorcha',
    '🧙🏻‍♀' => 'bean dhraíochta: ton cnis geal',
    '🧙🏼‍♀' => 'bean dhraíochta: ton cnis mheánach-gheal',
    '🧙🏽‍♀' => 'bean dhraíochta: ton cnis meánach',
    '🧙🏾‍♀' => 'bean dhraíochta: ton cnis meándorcha',
    '🧙🏿‍♀' => 'bean dhraíochta: ton cnis dorcha',
    '🧚🏻‍♂' => 'sióg fhireann: ton cnis geal',
    '🧚🏼‍♂' => 'sióg fhireann: ton cnis mheánach-gheal',
    '🧚🏽‍♂' => 'sióg fhireann: ton cnis meánach',
    '🧚🏾‍♂' => 'sióg fhireann: ton cnis meándorcha',
    '🧚🏿‍♂' => 'sióg fhireann: ton cnis dorcha',
    '🧚🏻‍♀' => 'sióg bhaineann: ton cnis geal',
    '🧚🏼‍♀' => 'sióg bhaineann: ton cnis mheánach-gheal',
    '🧚🏽‍♀' => 'sióg bhaineann: ton cnis meánach',
    '🧚🏾‍♀' => 'sióg bhaineann: ton cnis meándorcha',
    '🧚🏿‍♀' => 'sióg bhaineann: ton cnis dorcha',
    '🧛🏻‍♂' => 'vaimpír fhireann: ton cnis geal',
    '🧛🏼‍♂' => 'vaimpír fhireann: ton cnis mheánach-gheal',
    '🧛🏽‍♂' => 'vaimpír fhireann: ton cnis meánach',
    '🧛🏾‍♂' => 'vaimpír fhireann: ton cnis meándorcha',
    '🧛🏿‍♂' => 'vaimpír fhireann: ton cnis dorcha',
    '🧛🏻‍♀' => 'vaimpír bhaineann: ton cnis geal',
    '🧛🏼‍♀' => 'vaimpír bhaineann: ton cnis mheánach-gheal',
    '🧛🏽‍♀' => 'vaimpír bhaineann: ton cnis meánach',
    '🧛🏾‍♀' => 'vaimpír bhaineann: ton cnis meándorcha',
    '🧛🏿‍♀' => 'vaimpír bhaineann: ton cnis dorcha',
    '🧜🏻‍♂' => 'murúch fir: ton cnis geal',
    '🧜🏼‍♂' => 'murúch fir: ton cnis mheánach-gheal',
    '🧜🏽‍♂' => 'murúch fir: ton cnis meánach',
    '🧜🏾‍♂' => 'murúch fir: ton cnis meándorcha',
    '🧜🏿‍♂' => 'murúch fir: ton cnis dorcha',
    '🧜🏻‍♀' => 'murúch: ton cnis geal',
    '🧜🏼‍♀' => 'murúch: ton cnis mheánach-gheal',
    '🧜🏽‍♀' => 'murúch: ton cnis meánach',
    '🧜🏾‍♀' => 'murúch: ton cnis meándorcha',
    '🧜🏿‍♀' => 'murúch: ton cnis dorcha',
    '🧝🏻‍♂' => 'síofra fireann: ton cnis geal',
    '🧝🏼‍♂' => 'síofra fireann: ton cnis mheánach-gheal',
    '🧝🏽‍♂' => 'síofra fireann: ton cnis meánach',
    '🧝🏾‍♂' => 'síofra fireann: ton cnis meándorcha',
    '🧝🏿‍♂' => 'síofra fireann: ton cnis dorcha',
    '🧝🏻‍♀' => 'síofra baineann: ton cnis geal',
    '🧝🏼‍♀' => 'síofra baineann: ton cnis mheánach-gheal',
    '🧝🏽‍♀' => 'síofra baineann: ton cnis meánach',
    '🧝🏾‍♀' => 'síofra baineann: ton cnis meándorcha',
    '🧝🏿‍♀' => 'síofra baineann: ton cnis dorcha',
    '💆🏻‍♂' => 'fear ag fáil suathaireachta: ton cnis geal',
    '💆🏼‍♂' => 'fear ag fáil suathaireachta: ton cnis mheánach-gheal',
    '💆🏽‍♂' => 'fear ag fáil suathaireachta: ton cnis meánach',
    '💆🏾‍♂' => 'fear ag fáil suathaireachta: ton cnis meándorcha',
    '💆🏿‍♂' => 'fear ag fáil suathaireachta: ton cnis dorcha',
    '💆🏻‍♀' => 'bean ag fáil suathaireachta: ton cnis geal',
    '💆🏼‍♀' => 'bean ag fáil suathaireachta: ton cnis mheánach-gheal',
    '💆🏽‍♀' => 'bean ag fáil suathaireachta: ton cnis meánach',
    '💆🏾‍♀' => 'bean ag fáil suathaireachta: ton cnis meándorcha',
    '💆🏿‍♀' => 'bean ag fáil suathaireachta: ton cnis dorcha',
    '💇🏻‍♂' => 'fear ag fáil bearradh gruaige: ton cnis geal',
    '💇🏼‍♂' => 'fear ag fáil bearradh gruaige: ton cnis mheánach-gheal',
    '💇🏽‍♂' => 'fear ag fáil bearradh gruaige: ton cnis meánach',
    '💇🏾‍♂' => 'fear ag fáil bearradh gruaige: ton cnis meándorcha',
    '💇🏿‍♂' => 'fear ag fáil bearradh gruaige: ton cnis dorcha',
    '💇🏻‍♀' => 'bean ag fáil bearradh gruaige: ton cnis geal',
    '💇🏼‍♀' => 'bean ag fáil bearradh gruaige: ton cnis mheánach-gheal',
    '💇🏽‍♀' => 'bean ag fáil bearradh gruaige: ton cnis meánach',
    '💇🏾‍♀' => 'bean ag fáil bearradh gruaige: ton cnis meándorcha',
    '💇🏿‍♀' => 'bean ag fáil bearradh gruaige: ton cnis dorcha',
    '🚶🏻‍♂' => 'fear ag siúl: ton cnis geal',
    '🚶🏼‍♂' => 'fear ag siúl: ton cnis mheánach-gheal',
    '🚶🏽‍♂' => 'fear ag siúl: ton cnis meánach',
    '🚶🏾‍♂' => 'fear ag siúl: ton cnis meándorcha',
    '🚶🏿‍♂' => 'fear ag siúl: ton cnis dorcha',
    '🚶🏻‍♀' => 'bean ag siúl: ton cnis geal',
    '🚶🏼‍♀' => 'bean ag siúl: ton cnis mheánach-gheal',
    '🚶🏽‍♀' => 'bean ag siúl: ton cnis meánach',
    '🚶🏾‍♀' => 'bean ag siúl: ton cnis meándorcha',
    '🚶🏿‍♀' => 'bean ag siúl: ton cnis dorcha',
    '🚶🏻‍➡' => 'coisí: ton cnis geal, aghaidh ar dheis',
    '🚶🏼‍➡' => 'coisí: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🚶🏽‍➡' => 'coisí: ton cnis meánach, aghaidh ar dheis',
    '🚶🏾‍➡' => 'coisí: ton cnis meándorcha, aghaidh ar dheis',
    '🚶🏿‍➡' => 'coisí: ton cnis dorcha, aghaidh ar dheis',
    '🧍🏻‍♂' => 'fear ina sheasamh: ton cnis geal',
    '🧍🏼‍♂' => 'fear ina sheasamh: ton cnis mheánach-gheal',
    '🧍🏽‍♂' => 'fear ina sheasamh: ton cnis meánach',
    '🧍🏾‍♂' => 'fear ina sheasamh: ton cnis meándorcha',
    '🧍🏿‍♂' => 'fear ina sheasamh: ton cnis dorcha',
    '🧍🏻‍♀' => 'bean ina seasamh: ton cnis geal',
    '🧍🏼‍♀' => 'bean ina seasamh: ton cnis mheánach-gheal',
    '🧍🏽‍♀' => 'bean ina seasamh: ton cnis meánach',
    '🧍🏾‍♀' => 'bean ina seasamh: ton cnis meándorcha',
    '🧍🏿‍♀' => 'bean ina seasamh: ton cnis dorcha',
    '🧎🏻‍♂' => 'fear ar a ghlúine: ton cnis geal',
    '🧎🏼‍♂' => 'fear ar a ghlúine: ton cnis mheánach-gheal',
    '🧎🏽‍♂' => 'fear ar a ghlúine: ton cnis meánach',
    '🧎🏾‍♂' => 'fear ar a ghlúine: ton cnis meándorcha',
    '🧎🏿‍♂' => 'fear ar a ghlúine: ton cnis dorcha',
    '🧎🏻‍♀' => 'bean ar a glúine: ton cnis geal',
    '🧎🏼‍♀' => 'bean ar a glúine: ton cnis mheánach-gheal',
    '🧎🏽‍♀' => 'bean ar a glúine: ton cnis meánach',
    '🧎🏾‍♀' => 'bean ar a glúine: ton cnis meándorcha',
    '🧎🏿‍♀' => 'bean ar a glúine: ton cnis dorcha',
    '🧎🏻‍➡' => 'duine ar a ghlúine: ton cnis geal, aghaidh ar dheis',
    '🧎🏼‍➡' => 'duine ar a ghlúine: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🧎🏽‍➡' => 'duine ar a ghlúine: ton cnis meánach, aghaidh ar dheis',
    '🧎🏾‍➡' => 'duine ar a ghlúine: ton cnis meándorcha, aghaidh ar dheis',
    '🧎🏿‍➡' => 'duine ar a ghlúine: ton cnis dorcha, aghaidh ar dheis',
    '🧑🏻‍🦯' => 'duine le cána bán: ton cnis geal',
    '🧑🏼‍🦯' => 'duine le cána bán: ton cnis mheánach-gheal',
    '🧑🏽‍🦯' => 'duine le cána bán: ton cnis meánach',
    '🧑🏾‍🦯' => 'duine le cána bán: ton cnis meándorcha',
    '🧑🏿‍🦯' => 'duine le cána bán: ton cnis dorcha',
    '👨🏻‍🦯' => 'fear le cána bán: ton cnis geal',
    '👨🏼‍🦯' => 'fear le cána bán: ton cnis mheánach-gheal',
    '👨🏽‍🦯' => 'fear le cána bán: ton cnis meánach',
    '👨🏾‍🦯' => 'fear le cána bán: ton cnis meándorcha',
    '👨🏿‍🦯' => 'fear le cána bán: ton cnis dorcha',
    '👩🏻‍🦯' => 'bean le cána bán: ton cnis geal',
    '👩🏼‍🦯' => 'bean le cána bán: ton cnis mheánach-gheal',
    '👩🏽‍🦯' => 'bean le cána bán: ton cnis meánach',
    '👩🏾‍🦯' => 'bean le cána bán: ton cnis meándorcha',
    '👩🏿‍🦯' => 'bean le cána bán: ton cnis dorcha',
    '🧑🏻‍🦼' => 'duine i gcathaoir rothaí innealta: ton cnis geal',
    '🧑🏼‍🦼' => 'duine i gcathaoir rothaí innealta: ton cnis mheánach-gheal',
    '🧑🏽‍🦼' => 'duine i gcathaoir rothaí innealta: ton cnis meánach',
    '🧑🏾‍🦼' => 'duine i gcathaoir rothaí innealta: ton cnis meándorcha',
    '🧑🏿‍🦼' => 'duine i gcathaoir rothaí innealta: ton cnis dorcha',
    '👨🏻‍🦼' => 'fear i cathaoir rothaí innealta: ton cnis geal',
    '👨🏼‍🦼' => 'fear i cathaoir rothaí innealta: ton cnis mheánach-gheal',
    '👨🏽‍🦼' => 'fear i cathaoir rothaí innealta: ton cnis meánach',
    '👨🏾‍🦼' => 'fear i cathaoir rothaí innealta: ton cnis meándorcha',
    '👨🏿‍🦼' => 'fear i cathaoir rothaí innealta: ton cnis dorcha',
    '👩🏻‍🦼' => 'bean i gcathaoir rothaí innealta: ton cnis geal',
    '👩🏼‍🦼' => 'bean i gcathaoir rothaí innealta: ton cnis mheánach-gheal',
    '👩🏽‍🦼' => 'bean i gcathaoir rothaí innealta: ton cnis meánach',
    '👩🏾‍🦼' => 'bean i gcathaoir rothaí innealta: ton cnis meándorcha',
    '👩🏿‍🦼' => 'bean i gcathaoir rothaí innealta: ton cnis dorcha',
    '🧑🏻‍🦽' => 'duine i gcathaoir rothaí láimhe: ton cnis geal',
    '🧑🏼‍🦽' => 'duine i gcathaoir rothaí láimhe: ton cnis mheánach-gheal',
    '🧑🏽‍🦽' => 'duine i gcathaoir rothaí láimhe: ton cnis meánach',
    '🧑🏾‍🦽' => 'duine i gcathaoir rothaí láimhe: ton cnis meándorcha',
    '🧑🏿‍🦽' => 'duine i gcathaoir rothaí láimhe: ton cnis dorcha',
    '👨🏻‍🦽' => 'fear i gcathaoir rothaí láimhe: ton cnis geal',
    '👨🏼‍🦽' => 'fear i gcathaoir rothaí láimhe: ton cnis mheánach-gheal',
    '👨🏽‍🦽' => 'fear i gcathaoir rothaí láimhe: ton cnis meánach',
    '👨🏾‍🦽' => 'fear i gcathaoir rothaí láimhe: ton cnis meándorcha',
    '👨🏿‍🦽' => 'fear i gcathaoir rothaí láimhe: ton cnis dorcha',
    '👩🏻‍🦽' => 'bean i gcathaoir rothaí láimhe: ton cnis geal',
    '👩🏼‍🦽' => 'bean i gcathaoir rothaí láimhe: ton cnis mheánach-gheal',
    '👩🏽‍🦽' => 'bean i gcathaoir rothaí láimhe: ton cnis meánach',
    '👩🏾‍🦽' => 'bean i gcathaoir rothaí láimhe: ton cnis meándorcha',
    '👩🏿‍🦽' => 'bean i gcathaoir rothaí láimhe: ton cnis dorcha',
    '🏃🏻‍♂' => 'fear ag rith: ton cnis geal',
    '🏃🏼‍♂' => 'fear ag rith: ton cnis mheánach-gheal',
    '🏃🏽‍♂' => 'fear ag rith: ton cnis meánach',
    '🏃🏾‍♂' => 'fear ag rith: ton cnis meándorcha',
    '🏃🏿‍♂' => 'fear ag rith: ton cnis dorcha',
    '🏃🏻‍♀' => 'bean ag rith: ton cnis geal',
    '🏃🏼‍♀' => 'bean ag rith: ton cnis mheánach-gheal',
    '🏃🏽‍♀' => 'bean ag rith: ton cnis meánach',
    '🏃🏾‍♀' => 'bean ag rith: ton cnis meándorcha',
    '🏃🏿‍♀' => 'bean ag rith: ton cnis dorcha',
    '🏃🏻‍➡' => 'reathaí: ton cnis geal, aghaidh ar dheis',
    '🏃🏼‍➡' => 'reathaí: ton cnis mheánach-gheal, aghaidh ar dheis',
    '🏃🏽‍➡' => 'reathaí: ton cnis meánach, aghaidh ar dheis',
    '🏃🏾‍➡' => 'reathaí: ton cnis meándorcha, aghaidh ar dheis',
    '🏃🏿‍➡' => 'reathaí: ton cnis dorcha, aghaidh ar dheis',
    '👯🏻‍♀' => 'mná le cluasa coiníní orthu: ton cnis geal',
    '👯🏼‍♀' => 'mná le cluasa coiníní orthu: ton cnis mheánach-gheal',
    '👯🏽‍♀' => 'mná le cluasa coiníní orthu: ton cnis meánach',
    '👯🏾‍♀' => 'mná le cluasa coiníní orthu: ton cnis meándorcha',
    '👯🏿‍♀' => 'mná le cluasa coiníní orthu: ton cnis dorcha',
    '👯🏻‍♂' => 'fir le cluasa coiníní orthu: ton cnis geal',
    '👯🏼‍♂' => 'fir le cluasa coiníní orthu: ton cnis mheánach-gheal',
    '👯🏽‍♂' => 'fir le cluasa coiníní orthu: ton cnis meánach',
    '👯🏾‍♂' => 'fir le cluasa coiníní orthu: ton cnis meándorcha',
    '👯🏿‍♂' => 'fir le cluasa coiníní orthu: ton cnis dorcha',
    '🧖🏻‍♂' => 'fear i seomra gaile: ton cnis geal',
    '🧖🏼‍♂' => 'fear i seomra gaile: ton cnis mheánach-gheal',
    '🧖🏽‍♂' => 'fear i seomra gaile: ton cnis meánach',
    '🧖🏾‍♂' => 'fear i seomra gaile: ton cnis meándorcha',
    '🧖🏿‍♂' => 'fear i seomra gaile: ton cnis dorcha',
    '🧖🏻‍♀' => 'bean i seomra gaile: ton cnis geal',
    '🧖🏼‍♀' => 'bean i seomra gaile: ton cnis mheánach-gheal',
    '🧖🏽‍♀' => 'bean i seomra gaile: ton cnis meánach',
    '🧖🏾‍♀' => 'bean i seomra gaile: ton cnis meándorcha',
    '🧖🏿‍♀' => 'bean i seomra gaile: ton cnis dorcha',
    '🧗🏻‍♂' => 'fear ag dreapadóireacht: ton cnis geal',
    '🧗🏼‍♂' => 'fear ag dreapadóireacht: ton cnis mheánach-gheal',
    '🧗🏽‍♂' => 'fear ag dreapadóireacht: ton cnis meánach',
    '🧗🏾‍♂' => 'fear ag dreapadóireacht: ton cnis meándorcha',
    '🧗🏿‍♂' => 'fear ag dreapadóireacht: ton cnis dorcha',
    '🧗🏻‍♀' => 'bean ag dreapadóireacht: ton cnis geal',
    '🧗🏼‍♀' => 'bean ag dreapadóireacht: ton cnis mheánach-gheal',
    '🧗🏽‍♀' => 'bean ag dreapadóireacht: ton cnis meánach',
    '🧗🏾‍♀' => 'bean ag dreapadóireacht: ton cnis meándorcha',
    '🧗🏿‍♀' => 'bean ag dreapadóireacht: ton cnis dorcha',
    '🏌🏻‍♂' => 'fear ag imirt gailf: ton cnis geal',
    '🏌🏼‍♂' => 'fear ag imirt gailf: ton cnis mheánach-gheal',
    '🏌🏽‍♂' => 'fear ag imirt gailf: ton cnis meánach',
    '🏌🏾‍♂' => 'fear ag imirt gailf: ton cnis meándorcha',
    '🏌🏿‍♂' => 'fear ag imirt gailf: ton cnis dorcha',
    '🏌🏻‍♀' => 'bean ag imirt gailf: ton cnis geal',
    '🏌🏼‍♀' => 'bean ag imirt gailf: ton cnis mheánach-gheal',
    '🏌🏽‍♀' => 'bean ag imirt gailf: ton cnis meánach',
    '🏌🏾‍♀' => 'bean ag imirt gailf: ton cnis meándorcha',
    '🏌🏿‍♀' => 'bean ag imirt gailf: ton cnis dorcha',
    '🏄🏻‍♂' => 'fear ag surfáil: ton cnis geal',
    '🏄🏼‍♂' => 'fear ag surfáil: ton cnis mheánach-gheal',
    '🏄🏽‍♂' => 'fear ag surfáil: ton cnis meánach',
    '🏄🏾‍♂' => 'fear ag surfáil: ton cnis meándorcha',
    '🏄🏿‍♂' => 'fear ag surfáil: ton cnis dorcha',
    '🏄🏻‍♀' => 'bean ag surfáil: ton cnis geal',
    '🏄🏼‍♀' => 'bean ag surfáil: ton cnis mheánach-gheal',
    '🏄🏽‍♀' => 'bean ag surfáil: ton cnis meánach',
    '🏄🏾‍♀' => 'bean ag surfáil: ton cnis meándorcha',
    '🏄🏿‍♀' => 'bean ag surfáil: ton cnis dorcha',
    '🚣🏻‍♂' => 'fear ag iomair báid: ton cnis geal',
    '🚣🏼‍♂' => 'fear ag iomair báid: ton cnis mheánach-gheal',
    '🚣🏽‍♂' => 'fear ag iomair báid: ton cnis meánach',
    '🚣🏾‍♂' => 'fear ag iomair báid: ton cnis meándorcha',
    '🚣🏿‍♂' => 'fear ag iomair báid: ton cnis dorcha',
    '🚣🏻‍♀' => 'bean ag iomair báid: ton cnis geal',
    '🚣🏼‍♀' => 'bean ag iomair báid: ton cnis mheánach-gheal',
    '🚣🏽‍♀' => 'bean ag iomair báid: ton cnis meánach',
    '🚣🏾‍♀' => 'bean ag iomair báid: ton cnis meándorcha',
    '🚣🏿‍♀' => 'bean ag iomair báid: ton cnis dorcha',
    '🏊🏻‍♂' => 'fear ag snámh: ton cnis geal',
    '🏊🏼‍♂' => 'fear ag snámh: ton cnis mheánach-gheal',
    '🏊🏽‍♂' => 'fear ag snámh: ton cnis meánach',
    '🏊🏾‍♂' => 'fear ag snámh: ton cnis meándorcha',
    '🏊🏿‍♂' => 'fear ag snámh: ton cnis dorcha',
    '🏊🏻‍♀' => 'bean ag snámh: ton cnis geal',
    '🏊🏼‍♀' => 'bean ag snámh: ton cnis mheánach-gheal',
    '🏊🏽‍♀' => 'bean ag snámh: ton cnis meánach',
    '🏊🏾‍♀' => 'bean ag snámh: ton cnis meándorcha',
    '🏊🏿‍♀' => 'bean ag snámh: ton cnis dorcha',
    '⛹🏻‍♂' => 'fear le liathróid: ton cnis geal',
    '⛹🏼‍♂' => 'fear le liathróid: ton cnis mheánach-gheal',
    '⛹🏽‍♂' => 'fear le liathróid: ton cnis meánach',
    '⛹🏾‍♂' => 'fear le liathróid: ton cnis meándorcha',
    '⛹🏿‍♂' => 'fear le liathróid: ton cnis dorcha',
    '⛹🏻‍♀' => 'bean le liathróid: ton cnis geal',
    '⛹🏼‍♀' => 'bean le liathróid: ton cnis mheánach-gheal',
    '⛹🏽‍♀' => 'bean le liathróid: ton cnis meánach',
    '⛹🏾‍♀' => 'bean le liathróid: ton cnis meándorcha',
    '⛹🏿‍♀' => 'bean le liathróid: ton cnis dorcha',
    '🏋🏻‍♂' => 'fear ag tógáil mheáchan: ton cnis geal',
    '🏋🏼‍♂' => 'fear ag tógáil mheáchan: ton cnis mheánach-gheal',
    '🏋🏽‍♂' => 'fear ag tógáil mheáchan: ton cnis meánach',
    '🏋🏾‍♂' => 'fear ag tógáil mheáchan: ton cnis meándorcha',
    '🏋🏿‍♂' => 'fear ag tógáil mheáchan: ton cnis dorcha',
    '🏋🏻‍♀' => 'bean ag tógáil mheáchan: ton cnis geal',
    '🏋🏼‍♀' => 'bean ag tógáil mheáchan: ton cnis mheánach-gheal',
    '🏋🏽‍♀' => 'bean ag tógáil mheáchan: ton cnis meánach',
    '🏋🏾‍♀' => 'bean ag tógáil mheáchan: ton cnis meándorcha',
    '🏋🏿‍♀' => 'bean ag tógáil mheáchan: ton cnis dorcha',
    '🚴🏻‍♂' => 'fear ag rothaíocht: ton cnis geal',
    '🚴🏼‍♂' => 'fear ag rothaíocht: ton cnis mheánach-gheal',
    '🚴🏽‍♂' => 'fear ag rothaíocht: ton cnis meánach',
    '🚴🏾‍♂' => 'fear ag rothaíocht: ton cnis meándorcha',
    '🚴🏿‍♂' => 'fear ag rothaíocht: ton cnis dorcha',
    '🚴🏻‍♀' => 'bean ag rothaíocht: ton cnis geal',
    '🚴🏼‍♀' => 'bean ag rothaíocht: ton cnis mheánach-gheal',
    '🚴🏽‍♀' => 'bean ag rothaíocht: ton cnis meánach',
    '🚴🏾‍♀' => 'bean ag rothaíocht: ton cnis meándorcha',
    '🚴🏿‍♀' => 'bean ag rothaíocht: ton cnis dorcha',
    '🚵🏻‍♂' => 'fear ag rothaíocht sléibhe: ton cnis geal',
    '🚵🏼‍♂' => 'fear ag rothaíocht sléibhe: ton cnis mheánach-gheal',
    '🚵🏽‍♂' => 'fear ag rothaíocht sléibhe: ton cnis meánach',
    '🚵🏾‍♂' => 'fear ag rothaíocht sléibhe: ton cnis meándorcha',
    '🚵🏿‍♂' => 'fear ag rothaíocht sléibhe: ton cnis dorcha',
    '🚵🏻‍♀' => 'bean ag rothaíocht sléibhe: ton cnis geal',
    '🚵🏼‍♀' => 'bean ag rothaíocht sléibhe: ton cnis mheánach-gheal',
    '🚵🏽‍♀' => 'bean ag rothaíocht sléibhe: ton cnis meánach',
    '🚵🏾‍♀' => 'bean ag rothaíocht sléibhe: ton cnis meándorcha',
    '🚵🏿‍♀' => 'bean ag rothaíocht sléibhe: ton cnis dorcha',
    '🤸🏻‍♂' => 'fear ag déanamh rothchasaidh: ton cnis geal',
    '🤸🏼‍♂' => 'fear ag déanamh rothchasaidh: ton cnis mheánach-gheal',
    '🤸🏽‍♂' => 'fear ag déanamh rothchasaidh: ton cnis meánach',
    '🤸🏾‍♂' => 'fear ag déanamh rothchasaidh: ton cnis meándorcha',
    '🤸🏿‍♂' => 'fear ag déanamh rothchasaidh: ton cnis dorcha',
    '🤸🏻‍♀' => 'bean ag rothchasadh: ton cnis geal',
    '🤸🏼‍♀' => 'bean ag rothchasadh: ton cnis mheánach-gheal',
    '🤸🏽‍♀' => 'bean ag rothchasadh: ton cnis meánach',
    '🤸🏾‍♀' => 'bean ag rothchasadh: ton cnis meándorcha',
    '🤸🏿‍♀' => 'bean ag rothchasadh: ton cnis dorcha',
    '🤼🏻‍♀' => 'mná ag coraíocht: ton cnis geal',
    '🤼🏼‍♀' => 'mná ag coraíocht: ton cnis mheánach-gheal',
    '🤼🏽‍♀' => 'mná ag coraíocht: ton cnis meánach',
    '🤼🏾‍♀' => 'mná ag coraíocht: ton cnis meándorcha',
    '🤼🏿‍♀' => 'mná ag coraíocht: ton cnis dorcha',
    '🤼🏻‍♂' => 'fir ag coraíocht: ton cnis geal',
    '🤼🏼‍♂' => 'fir ag coraíocht: ton cnis mheánach-gheal',
    '🤼🏽‍♂' => 'fir ag coraíocht: ton cnis meánach',
    '🤼🏾‍♂' => 'fir ag coraíocht: ton cnis meándorcha',
    '🤼🏿‍♂' => 'fir ag coraíocht: ton cnis dorcha',
    '🤽🏻‍♂' => 'fear ag imirt póló uisce: ton cnis geal',
    '🤽🏼‍♂' => 'fear ag imirt póló uisce: ton cnis mheánach-gheal',
    '🤽🏽‍♂' => 'fear ag imirt póló uisce: ton cnis meánach',
    '🤽🏾‍♂' => 'fear ag imirt póló uisce: ton cnis meándorcha',
    '🤽🏿‍♂' => 'fear ag imirt póló uisce: ton cnis dorcha',
    '🤽🏻‍♀' => 'bean ag imirt póló uisce: ton cnis geal',
    '🤽🏼‍♀' => 'bean ag imirt póló uisce: ton cnis mheánach-gheal',
    '🤽🏽‍♀' => 'bean ag imirt póló uisce: ton cnis meánach',
    '🤽🏾‍♀' => 'bean ag imirt póló uisce: ton cnis meándorcha',
    '🤽🏿‍♀' => 'bean ag imirt póló uisce: ton cnis dorcha',
    '🤾🏻‍♂' => 'fear ag imirt liathróid láimhe: ton cnis geal',
    '🤾🏼‍♂' => 'fear ag imirt liathróid láimhe: ton cnis mheánach-gheal',
    '🤾🏽‍♂' => 'fear ag imirt liathróid láimhe: ton cnis meánach',
    '🤾🏾‍♂' => 'fear ag imirt liathróid láimhe: ton cnis meándorcha',
    '🤾🏿‍♂' => 'fear ag imirt liathróid láimhe: ton cnis dorcha',
    '🤾🏻‍♀' => 'bean ag imirt liathróid láimhe: ton cnis geal',
    '🤾🏼‍♀' => 'bean ag imirt liathróid láimhe: ton cnis mheánach-gheal',
    '🤾🏽‍♀' => 'bean ag imirt liathróid láimhe: ton cnis meánach',
    '🤾🏾‍♀' => 'bean ag imirt liathróid láimhe: ton cnis meándorcha',
    '🤾🏿‍♀' => 'bean ag imirt liathróid láimhe: ton cnis dorcha',
    '🤹🏻‍♂' => 'fear ag déanamh lámhchleasaíochta: ton cnis geal',
    '🤹🏼‍♂' => 'fear ag déanamh lámhchleasaíochta: ton cnis mheánach-gheal',
    '🤹🏽‍♂' => 'fear ag déanamh lámhchleasaíochta: ton cnis meánach',
    '🤹🏾‍♂' => 'fear ag déanamh lámhchleasaíochta: ton cnis meándorcha',
    '🤹🏿‍♂' => 'fear ag déanamh lámhchleasaíochta: ton cnis dorcha',
    '🤹🏻‍♀' => 'bean ag déanamh lámhchleasaíochta: ton cnis geal',
    '🤹🏼‍♀' => 'bean ag déanamh lámhchleasaíochta: ton cnis mheánach-gheal',
    '🤹🏽‍♀' => 'bean ag déanamh lámhchleasaíochta: ton cnis meánach',
    '🤹🏾‍♀' => 'bean ag déanamh lámhchleasaíochta: ton cnis meándorcha',
    '🤹🏿‍♀' => 'bean ag déanamh lámhchleasaíochta: ton cnis dorcha',
    '🧘🏻‍♂' => 'fear i suíomh na loiteoige: ton cnis geal',
    '🧘🏼‍♂' => 'fear i suíomh na loiteoige: ton cnis mheánach-gheal',
    '🧘🏽‍♂' => 'fear i suíomh na loiteoige: ton cnis meánach',
    '🧘🏾‍♂' => 'fear i suíomh na loiteoige: ton cnis meándorcha',
    '🧘🏿‍♂' => 'fear i suíomh na loiteoige: ton cnis dorcha',
    '🧘🏻‍♀' => 'bean i suíomh na loiteoige: ton cnis geal',
    '🧘🏼‍♀' => 'bean i suíomh na loiteoige: ton cnis mheánach-gheal',
    '🧘🏽‍♀' => 'bean i suíomh na loiteoige: ton cnis meánach',
    '🧘🏾‍♀' => 'bean i suíomh na loiteoige: ton cnis meándorcha',
    '🧘🏿‍♀' => 'bean i suíomh na loiteoige: ton cnis dorcha',
    '🧑‍🦰' => 'aosach: gruaig rua',
    '🧑‍🦱' => 'aosach: gruaig chatach',
    '🧑‍🦳' => 'aosach: gruaig bhán',
    '🧑‍🦲' => 'aosach: maol',
    '👨‍🦰' => 'fear: gruaig rua',
    '👨‍🦱' => 'fear: gruaig chatach',
    '👨‍🦳' => 'fear: gruaig bhán',
    '👨‍🦲' => 'fear: maol',
    '👩‍🦰' => 'bean: gruaig rua',
    '👩‍🦱' => 'bean: gruaig chatach',
    '👩‍🦳' => 'bean: gruaig bhán',
    '👩‍🦲' => 'bean: maol',
    '🚶‍➡' => 'coisí: aghaidh ar dheis',
    '🧎‍➡' => 'duine ar a ghlúine: aghaidh ar dheis',
    '🏃‍➡' => 'reathaí: aghaidh ar dheis',
    '👨‍👦' => 'teaghlach: fear, buachaill',
    '👨‍👧' => 'teaghlach: fear, cailín',
    '👩‍👦' => 'teaghlach: bean, buachaill',
    '👩‍👧' => 'teaghlach: bean, cailín',
    '😶‍🌫' => 'aghaidh sna scamaill',
    '😮‍💨' => 'aghaidh ag análú',
    '🙂‍↕' => 'croitheadh ceann go hingearach',
    '😵‍💫' => 'aghaidh le súile bíse',
    '❤‍🔥' => 'croí trí thine',
    '❤‍🩹' => 'croí atá ag dul i bhfeabhas',
    '👁‍🗨' => 'súil i mbolgán cainte',
    '🧔‍♂' => 'fear: feasóg',
    '👱‍♂' => 'fear: gruaig fhionn',
    '🧔‍♀' => 'bean: feasóg',
    '👱‍♀' => 'bean: gruaig fhionn',
    '🙍‍♂' => 'fear le púic air',
    '🙍‍♀' => 'bean le púic uirthi',
    '🙎‍♂' => 'fear le puisín air',
    '🙎‍♀' => 'bean le puisín uirthi',
    '🙅‍♂' => 'fear ag comharthú NÍL',
    '🙅‍♀' => 'bean ag comharthú NÍL',
    '🙆‍♂' => 'fear ag comharthú OK',
    '🙆‍♀' => 'bean ag comharthú OK',
    '💁‍♂' => 'fear lena lámh amach',
    '💁‍♀' => 'bean lena lámh amach',
    '🙋‍♂' => 'fear lena lámh suas',
    '🙋‍♀' => 'bean lena lámh suas',
    '🧏‍♂' => 'fear bodhar',
    '🧏‍♀' => 'bean bhodhar',
    '🙇‍♂' => 'fear ag umhlú',
    '🙇‍♀' => 'bean ag umhlú',
    '🤦‍♂' => 'fear lena aghaidh ina bhos',
    '🤦‍♀' => 'bean lena haghaidh ina bos',
    '🤷‍♂' => 'fear ag croitheadh',
    '🤷‍♀' => 'bean ag croitheadh',
    '🧑‍⚕' => 'oibrí sláinte',
    '👨‍⚕' => 'oibrí sláinte fireann',
    '👩‍⚕' => 'oibrí sláinte baineann',
    '🧑‍🎓' => 'mac léinn',
    '👨‍🎓' => 'scoláire fir',
    '👩‍🎓' => 'scoláire baineann',
    '🧑‍🏫' => 'múinteoir',
    '👨‍🏫' => 'múinteoir fireann',
    '👩‍🏫' => 'múinteoir baineann',
    '🧑‍⚖' => 'breitheamh',
    '👨‍⚖' => 'breitheamh fireann',
    '👩‍⚖' => 'breitheamh baineann',
    '🧑‍🌾' => 'feirmeoir',
    '👨‍🌾' => 'feirmeoir fireann',
    '👩‍🌾' => 'feirmeoir baineann',
    '🧑‍🍳' => 'cócaire',
    '👨‍🍳' => 'cócaire fireann',
    '👩‍🍳' => 'cócaire baineann',
    '🧑‍🔧' => 'meicneoir',
    '👨‍🔧' => 'meicneoir fireann',
    '👩‍🔧' => 'meicneoir baineann',
    '🧑‍🏭' => 'oibrí monarchan',
    '👨‍🏭' => 'oibrí monarchan fireann',
    '👩‍🏭' => 'oibrí monarchan baineann',
    '🧑‍💼' => 'oibrí oifige',
    '👨‍💼' => 'oibrí oifige fireann',
    '👩‍💼' => 'oibrí oifige baineann',
    '🧑‍🔬' => 'eolaí',
    '👨‍🔬' => 'eolaí fireann',
    '👩‍🔬' => 'eolaí baineann',
    '🧑‍💻' => 'teicneolaí',
    '👨‍💻' => 'teicneolaí fireann',
    '👩‍💻' => 'teicneolaí baineann',
    '🧑‍🎤' => 'amhránaí',
    '👨‍🎤' => 'amhránaí fireann',
    '👩‍🎤' => 'amhránaí baineann',
    '🧑‍🎨' => 'ealaíontóir',
    '👨‍🎨' => 'ealaíontóir fearann',
    '👩‍🎨' => 'ealaíontóir baineann',
    '🧑‍✈' => 'píolóta',
    '👨‍✈' => 'píolóta fireann',
    '👩‍✈' => 'píolóta baineann',
    '🧑‍🚀' => 'spásaire',
    '👨‍🚀' => 'spásaire fireann',
    '👩‍🚀' => 'spásaire baineann',
    '🧑‍🚒' => 'comhraiceoir tine',
    '👨‍🚒' => 'fear múchta dóiteán',
    '👩‍🚒' => 'bean mhúchta dóiteán',
    '👮‍♂' => 'garda',
    '👮‍♀' => 'bangharda',
    '🕵‍♂' => 'bleachtaire fireann',
    '🕵‍♀' => 'bleachtaire baineann',
    '💂‍♂' => 'garda oifigeach',
    '💂‍♀' => 'bangharda oifigeach',
    '👷‍♂' => 'tógálaí fireann',
    '👷‍♀' => 'tógálaí baineann',
    '👳‍♂' => 'fear le turban air',
    '👳‍♀' => 'bean le turban uirthi',
    '🤵‍♂' => 'fear a bhfuil culaith dhinnéir air',
    '🤵‍♀' => 'bean a bhfuil culaith dhinnéir uirthi',
    '👰‍♂' => 'fear a bhfuil caille air',
    '👰‍♀' => 'bean a bhfuil caille uirthi',
    '👩‍🍼' => 'bean ag beathú linbh',
    '👨‍🍼' => 'fear ag beathú linbh',
    '🧑‍🍼' => 'duine ag beathú linbh',
    '🧑‍🎄' => 'Bean Shaintí',
    '🦸‍♂' => 'fear-sárlaoch',
    '🦸‍♀' => 'ban-sárlaoch',
    '🦹‍♂' => 'fear-dhrochdhuine',
    '🦹‍♀' => 'ban-drochdhuine',
    '🧙‍♂' => 'fear draíochta',
    '🧙‍♀' => 'bean dhraíochta',
    '🧚‍♂' => 'sióg fhireann',
    '🧚‍♀' => 'sióg bhaineann',
    '🧛‍♂' => 'vaimpír fhireann',
    '🧛‍♀' => 'vaimpír bhaineann',
    '🧜‍♂' => 'murúch fir',
    '🧜‍♀' => 'murúch',
    '🧝‍♂' => 'síofra fireann',
    '🧝‍♀' => 'síofra baineann',
    '🧞‍♂' => 'ginid fhireann',
    '🧞‍♀' => 'ginid bhaineann',
    '🧟‍♂' => 'zombaí fireann',
    '🧟‍♀' => 'zombaí baineann',
    '💆‍♂' => 'fear ag fáil suathaireachta',
    '💆‍♀' => 'bean ag fáil suathaireachta',
    '💇‍♂' => 'fear ag fáil bearradh gruaige',
    '💇‍♀' => 'bean ag fáil bearradh gruaige',
    '🚶‍♂' => 'fear ag siúl',
    '🚶‍♀' => 'bean ag siúl',
    '🧍‍♂' => 'fear ina sheasamh',
    '🧍‍♀' => 'bean ina seasamh',
    '🧎‍♂' => 'fear ar a ghlúine',
    '🧎‍♀' => 'bean ar a glúine',
    '🧑‍🦯' => 'duine le cána bán',
    '👨‍🦯' => 'fear le cána bán',
    '👩‍🦯' => 'bean le cána bán',
    '🧑‍🦼' => 'duine i gcathaoir rothaí innealta',
    '👨‍🦼' => 'fear i cathaoir rothaí innealta',
    '👩‍🦼' => 'bean i gcathaoir rothaí innealta',
    '🧑‍🦽' => 'duine i gcathaoir rothaí láimhe',
    '👨‍🦽' => 'fear i gcathaoir rothaí láimhe',
    '👩‍🦽' => 'bean i gcathaoir rothaí láimhe',
    '🏃‍♂' => 'fear ag rith',
    '🏃‍♀' => 'bean ag rith',
    '🧑‍🩰' => 'damhsóir bailé',
    '👯‍♂' => 'fir le cluasa coiníní orthu',
    '👯‍♀' => 'mná le cluasa coiníní orthu',
    '🧖‍♂' => 'fear i seomra gaile',
    '🧖‍♀' => 'bean i seomra gaile',
    '🧗‍♂' => 'fear ag dreapadóireacht',
    '🧗‍♀' => 'bean ag dreapadóireacht',
    '🏌‍♂' => 'fear ag imirt gailf',
    '🏌‍♀' => 'bean ag imirt gailf',
    '🏄‍♂' => 'fear ag surfáil',
    '🏄‍♀' => 'bean ag surfáil',
    '🚣‍♂' => 'fear ag iomair báid',
    '🚣‍♀' => 'bean ag iomair báid',
    '🏊‍♂' => 'fear ag snámh',
    '🏊‍♀' => 'bean ag snámh',
    '⛹‍♂' => 'fear le liathróid',
    '⛹‍♀' => 'bean le liathróid',
    '🏋‍♂' => 'fear ag tógáil mheáchan',
    '🏋‍♀' => 'bean ag tógáil mheáchan',
    '🚴‍♂' => 'fear ag rothaíocht',
    '🚴‍♀' => 'bean ag rothaíocht',
    '🚵‍♂' => 'fear ag rothaíocht sléibhe',
    '🚵‍♀' => 'bean ag rothaíocht sléibhe',
    '🤸‍♂' => 'fear ag déanamh rothchasaidh',
    '🤸‍♀' => 'bean ag rothchasadh',
    '🤼‍♂' => 'fir ag coraíocht',
    '🤼‍♀' => 'mná ag coraíocht',
    '🤽‍♂' => 'fear ag imirt póló uisce',
    '🤽‍♀' => 'bean ag imirt póló uisce',
    '🤾‍♂' => 'fear ag imirt liathróid láimhe',
    '🤾‍♀' => 'bean ag imirt liathróid láimhe',
    '🤹‍♂' => 'fear ag déanamh lámhchleasaíochta',
    '🤹‍♀' => 'bean ag déanamh lámhchleasaíochta',
    '🧘‍♂' => 'fear i suíomh na loiteoige',
    '🧘‍♀' => 'bean i suíomh na loiteoige',
    '🧑‍🧒' => 'teaghlach: duine fásta, leanbh',
    '🐕‍🦺' => 'mardra seirbhíse',
    '🐈‍⬛' => 'cat dubh',
    '🐻‍❄' => 'béar bán',
    '🐦‍⬛' => 'lon dubh',
    '🐦‍🔥' => 'feinics',
    '🍋‍🟩' => 'líoma',
    '🍄‍🟫' => 'beacán donn',
    '⛓‍💥' => 'slabhra briste',
    '🏳‍🌈' => 'bratach ildaite',
    '🏳‍⚧' => 'bratach ghorm, bhándearg agus bhán',
    '🏴‍☠' => 'bratach foghlaithe',
    '👋🏻' => 'lámh ag croitheadh: ton cnis geal',
    '👋🏼' => 'lámh ag croitheadh: ton cnis mheánach-gheal',
    '👋🏽' => 'lámh ag croitheadh: ton cnis meánach',
    '👋🏾' => 'lámh ag croitheadh: ton cnis meándorcha',
    '👋🏿' => 'lámh ag croitheadh: ton cnis dorcha',
    '🤚🏻' => 'cúl láimhe in airde: ton cnis geal',
    '🤚🏼' => 'cúl láimhe in airde: ton cnis mheánach-gheal',
    '🤚🏽' => 'cúl láimhe in airde: ton cnis meánach',
    '🤚🏾' => 'cúl láimhe in airde: ton cnis meándorcha',
    '🤚🏿' => 'cúl láimhe in airde: ton cnis dorcha',
    '🖐🏻' => 'lámh le méara spréite: ton cnis geal',
    '🖐🏼' => 'lámh le méara spréite: ton cnis mheánach-gheal',
    '🖐🏽' => 'lámh le méara spréite: ton cnis meánach',
    '🖐🏾' => 'lámh le méara spréite: ton cnis meándorcha',
    '🖐🏿' => 'lámh le méara spréite: ton cnis dorcha',
    '✋🏻' => 'lámh ardaithe: ton cnis geal',
    '✋🏼' => 'lámh ardaithe: ton cnis mheánach-gheal',
    '✋🏽' => 'lámh ardaithe: ton cnis meánach',
    '✋🏾' => 'lámh ardaithe: ton cnis meándorcha',
    '✋🏿' => 'lámh ardaithe: ton cnis dorcha',
    '🖖🏻' => 'cuirtéis Bholcáin: ton cnis geal',
    '🖖🏼' => 'cuirtéis Bholcáin: ton cnis mheánach-gheal',
    '🖖🏽' => 'cuirtéis Bholcáin: ton cnis meánach',
    '🖖🏾' => 'cuirtéis Bholcáin: ton cnis meándorcha',
    '🖖🏿' => 'cuirtéis Bholcáin: ton cnis dorcha',
    '🫱🏻' => 'lámh ar dheis: ton cnis geal',
    '🫱🏼' => 'lámh ar dheis: ton cnis mheánach-gheal',
    '🫱🏽' => 'lámh ar dheis: ton cnis meánach',
    '🫱🏾' => 'lámh ar dheis: ton cnis meándorcha',
    '🫱🏿' => 'lámh ar dheis: ton cnis dorcha',
    '🫲🏻' => 'lámh ar chlé: ton cnis geal',
    '🫲🏼' => 'lámh ar chlé: ton cnis mheánach-gheal',
    '🫲🏽' => 'lámh ar chlé: ton cnis meánach',
    '🫲🏾' => 'lámh ar chlé: ton cnis meándorcha',
    '🫲🏿' => 'lámh ar chlé: ton cnis dorcha',
    '🫳🏻' => 'lámh le bos faoi: ton cnis geal',
    '🫳🏼' => 'lámh le bos faoi: ton cnis mheánach-gheal',
    '🫳🏽' => 'lámh le bos faoi: ton cnis meánach',
    '🫳🏾' => 'lámh le bos faoi: ton cnis meándorcha',
    '🫳🏿' => 'lámh le bos faoi: ton cnis dorcha',
    '🫴🏻' => 'lámh le bos suas: ton cnis geal',
    '🫴🏼' => 'lámh le bos suas: ton cnis mheánach-gheal',
    '🫴🏽' => 'lámh le bos suas: ton cnis meánach',
    '🫴🏾' => 'lámh le bos suas: ton cnis meándorcha',
    '🫴🏿' => 'lámh le bos suas: ton cnis dorcha',
    '🫷🏻' => 'brú láimhe ar chlé: ton cnis geal',
    '🫷🏼' => 'brú láimhe ar chlé: ton cnis mheánach-gheal',
    '🫷🏽' => 'brú láimhe ar chlé: ton cnis meánach',
    '🫷🏾' => 'brú láimhe ar chlé: ton cnis meándorcha',
    '🫷🏿' => 'brú láimhe ar chlé: ton cnis dorcha',
    '🫸🏻' => 'brú láimhe ar dheis: ton cnis geal',
    '🫸🏼' => 'brú láimhe ar dheis: ton cnis mheánach-gheal',
    '🫸🏽' => 'brú láimhe ar dheis: ton cnis meánach',
    '🫸🏾' => 'brú láimhe ar dheis: ton cnis meándorcha',
    '🫸🏿' => 'brú láimhe ar dheis: ton cnis dorcha',
    '👌🏻' => 'lámh OK: ton cnis geal',
    '👌🏼' => 'lámh OK: ton cnis mheánach-gheal',
    '👌🏽' => 'lámh OK: ton cnis meánach',
    '👌🏾' => 'lámh OK: ton cnis meándorcha',
    '👌🏿' => 'lámh OK: ton cnis dorcha',
    '🤌🏻' => 'méara fáiscthe le chéile: ton cnis geal',
    '🤌🏼' => 'méara fáiscthe le chéile: ton cnis mheánach-gheal',
    '🤌🏽' => 'méara fáiscthe le chéile: ton cnis meánach',
    '🤌🏾' => 'méara fáiscthe le chéile: ton cnis meándorcha',
    '🤌🏿' => 'méara fáiscthe le chéile: ton cnis dorcha',
    '🤏🏻' => 'lámh ag pinseáil: ton cnis geal',
    '🤏🏼' => 'lámh ag pinseáil: ton cnis mheánach-gheal',
    '🤏🏽' => 'lámh ag pinseáil: ton cnis meánach',
    '🤏🏾' => 'lámh ag pinseáil: ton cnis meándorcha',
    '🤏🏿' => 'lámh ag pinseáil: ton cnis dorcha',
    '✌🏻' => 'comhartha an bhua: ton cnis geal',
    '✌🏼' => 'comhartha an bhua: ton cnis mheánach-gheal',
    '✌🏽' => 'comhartha an bhua: ton cnis meánach',
    '✌🏾' => 'comhartha an bhua: ton cnis meándorcha',
    '✌🏿' => 'comhartha an bhua: ton cnis dorcha',
    '🤞🏻' => 'méarsa trasna: ton cnis geal',
    '🤞🏼' => 'méarsa trasna: ton cnis mheánach-gheal',
    '🤞🏽' => 'méarsa trasna: ton cnis meánach',
    '🤞🏾' => 'méarsa trasna: ton cnis meándorcha',
    '🤞🏿' => 'méarsa trasna: ton cnis dorcha',
    '🫰🏻' => 'lámh a bhfuil a corrmhéar agus a ordóg trasna ar a chéile: ton cnis geal',
    '🫰🏼' => 'lámh a bhfuil a corrmhéar agus a ordóg trasna ar a chéile: ton cnis mheánach-gheal',
    '🫰🏽' => 'lámh a bhfuil a corrmhéar agus a ordóg trasna ar a chéile: ton cnis meánach',
    '🫰🏾' => 'lámh a bhfuil a corrmhéar agus a ordóg trasna ar a chéile: ton cnis meándorcha',
    '🫰🏿' => 'lámh a bhfuil a corrmhéar agus a ordóg trasna ar a chéile: ton cnis dorcha',
    '🤟🏻' => 'comhartha grá: ton cnis geal',
    '🤟🏼' => 'comhartha grá: ton cnis mheánach-gheal',
    '🤟🏽' => 'comhartha grá: ton cnis meánach',
    '🤟🏾' => 'comhartha grá: ton cnis meándorcha',
    '🤟🏿' => 'comhartha grá: ton cnis dorcha',
    '🤘🏻' => 'siombail na n-adharc: ton cnis geal',
    '🤘🏼' => 'siombail na n-adharc: ton cnis mheánach-gheal',
    '🤘🏽' => 'siombail na n-adharc: ton cnis meánach',
    '🤘🏾' => 'siombail na n-adharc: ton cnis meándorcha',
    '🤘🏿' => 'siombail na n-adharc: ton cnis dorcha',
    '🤙🏻' => 'an lámh cur glao orm: ton cnis geal',
    '🤙🏼' => 'an lámh cur glao orm: ton cnis mheánach-gheal',
    '🤙🏽' => 'an lámh cur glao orm: ton cnis meánach',
    '🤙🏾' => 'an lámh cur glao orm: ton cnis meándorcha',
    '🤙🏿' => 'an lámh cur glao orm: ton cnis dorcha',
    '👈🏻' => 'cúl corrmhéire ag díriú ar chlé: ton cnis geal',
    '👈🏼' => 'cúl corrmhéire ag díriú ar chlé: ton cnis mheánach-gheal',
    '👈🏽' => 'cúl corrmhéire ag díriú ar chlé: ton cnis meánach',
    '👈🏾' => 'cúl corrmhéire ag díriú ar chlé: ton cnis meándorcha',
    '👈🏿' => 'cúl corrmhéire ag díriú ar chlé: ton cnis dorcha',
    '👉🏻' => 'cúl corrmhéire ag díriú ar dheis: ton cnis geal',
    '👉🏼' => 'cúl corrmhéire ag díriú ar dheis: ton cnis mheánach-gheal',
    '👉🏽' => 'cúl corrmhéire ag díriú ar dheis: ton cnis meánach',
    '👉🏾' => 'cúl corrmhéire ag díriú ar dheis: ton cnis meándorcha',
    '👉🏿' => 'cúl corrmhéire ag díriú ar dheis: ton cnis dorcha',
    '👆🏻' => 'cúl corrmhéire ag díriú suas: ton cnis geal',
    '👆🏼' => 'cúl corrmhéire ag díriú suas: ton cnis mheánach-gheal',
    '👆🏽' => 'cúl corrmhéire ag díriú suas: ton cnis meánach',
    '👆🏾' => 'cúl corrmhéire ag díriú suas: ton cnis meándorcha',
    '👆🏿' => 'cúl corrmhéire ag díriú suas: ton cnis dorcha',
    '🖕🏻' => 'méar láir: ton cnis geal',
    '🖕🏼' => 'méar láir: ton cnis mheánach-gheal',
    '🖕🏽' => 'méar láir: ton cnis meánach',
    '🖕🏾' => 'méar láir: ton cnis meándorcha',
    '🖕🏿' => 'méar láir: ton cnis dorcha',
    '👇🏻' => 'cúl corrmhéire ag díriú síos: ton cnis geal',
    '👇🏼' => 'cúl corrmhéire ag díriú síos: ton cnis mheánach-gheal',
    '👇🏽' => 'cúl corrmhéire ag díriú síos: ton cnis meánach',
    '👇🏾' => 'cúl corrmhéire ag díriú síos: ton cnis meándorcha',
    '👇🏿' => 'cúl corrmhéire ag díriú síos: ton cnis dorcha',
    '☝🏻' => 'corrmhéar ag díriú suas: ton cnis geal',
    '☝🏼' => 'corrmhéar ag díriú suas: ton cnis mheánach-gheal',
    '☝🏽' => 'corrmhéar ag díriú suas: ton cnis meánach',
    '☝🏾' => 'corrmhéar ag díriú suas: ton cnis meándorcha',
    '☝🏿' => 'corrmhéar ag díriú suas: ton cnis dorcha',
    '🫵🏻' => 'corrmhéar ag díriú ar an mbreathnóir: ton cnis geal',
    '🫵🏼' => 'corrmhéar ag díriú ar an mbreathnóir: ton cnis mheánach-gheal',
    '🫵🏽' => 'corrmhéar ag díriú ar an mbreathnóir: ton cnis meánach',
    '🫵🏾' => 'corrmhéar ag díriú ar an mbreathnóir: ton cnis meándorcha',
    '🫵🏿' => 'corrmhéar ag díriú ar an mbreathnóir: ton cnis dorcha',
    '👍🏻' => 'ordóg suas: ton cnis geal',
    '👍🏼' => 'ordóg suas: ton cnis mheánach-gheal',
    '👍🏽' => 'ordóg suas: ton cnis meánach',
    '👍🏾' => 'ordóg suas: ton cnis meándorcha',
    '👍🏿' => 'ordóg suas: ton cnis dorcha',
    '👎🏻' => 'ordóg síos: ton cnis geal',
    '👎🏼' => 'ordóg síos: ton cnis mheánach-gheal',
    '👎🏽' => 'ordóg síos: ton cnis meánach',
    '👎🏾' => 'ordóg síos: ton cnis meándorcha',
    '👎🏿' => 'ordóg síos: ton cnis dorcha',
    '✊🏻' => 'dorn ardaithe: ton cnis geal',
    '✊🏼' => 'dorn ardaithe: ton cnis mheánach-gheal',
    '✊🏽' => 'dorn ardaithe: ton cnis meánach',
    '✊🏾' => 'dorn ardaithe: ton cnis meándorcha',
    '✊🏿' => 'dorn ardaithe: ton cnis dorcha',
    '👊🏻' => 'dorn atá chugat: ton cnis geal',
    '👊🏼' => 'dorn atá chugat: ton cnis mheánach-gheal',
    '👊🏽' => 'dorn atá chugat: ton cnis meánach',
    '👊🏾' => 'dorn atá chugat: ton cnis meándorcha',
    '👊🏿' => 'dorn atá chugat: ton cnis dorcha',
    '🤛🏻' => 'dorn ar chlé: ton cnis geal',
    '🤛🏼' => 'dorn ar chlé: ton cnis mheánach-gheal',
    '🤛🏽' => 'dorn ar chlé: ton cnis meánach',
    '🤛🏾' => 'dorn ar chlé: ton cnis meándorcha',
    '🤛🏿' => 'dorn ar chlé: ton cnis dorcha',
    '🤜🏻' => 'dorn ar dheis: ton cnis geal',
    '🤜🏼' => 'dorn ar dheis: ton cnis mheánach-gheal',
    '🤜🏽' => 'dorn ar dheis: ton cnis meánach',
    '🤜🏾' => 'dorn ar dheis: ton cnis meándorcha',
    '🤜🏿' => 'dorn ar dheis: ton cnis dorcha',
    '👏🏻' => 'bualadh bos: ton cnis geal',
    '👏🏼' => 'bualadh bos: ton cnis mheánach-gheal',
    '👏🏽' => 'bualadh bos: ton cnis meánach',
    '👏🏾' => 'bualadh bos: ton cnis meándorcha',
    '👏🏿' => 'bualadh bos: ton cnis dorcha',
    '🙌🏻' => 'lámha ardaithe: ton cnis geal',
    '🙌🏼' => 'lámha ardaithe: ton cnis mheánach-gheal',
    '🙌🏽' => 'lámha ardaithe: ton cnis meánach',
    '🙌🏾' => 'lámha ardaithe: ton cnis meándorcha',
    '🙌🏿' => 'lámha ardaithe: ton cnis dorcha',
    '🫶🏻' => 'lámha croí: ton cnis geal',
    '🫶🏼' => 'lámha croí: ton cnis mheánach-gheal',
    '🫶🏽' => 'lámha croí: ton cnis meánach',
    '🫶🏾' => 'lámha croí: ton cnis meándorcha',
    '🫶🏿' => 'lámha croí: ton cnis dorcha',
    '👐🏻' => 'lámha ar oscailt: ton cnis geal',
    '👐🏼' => 'lámha ar oscailt: ton cnis mheánach-gheal',
    '👐🏽' => 'lámha ar oscailt: ton cnis meánach',
    '👐🏾' => 'lámha ar oscailt: ton cnis meándorcha',
    '👐🏿' => 'lámha ar oscailt: ton cnis dorcha',
    '🤲🏻' => 'bosa le chéile: ton cnis geal',
    '🤲🏼' => 'bosa le chéile: ton cnis mheánach-gheal',
    '🤲🏽' => 'bosa le chéile: ton cnis meánach',
    '🤲🏾' => 'bosa le chéile: ton cnis meándorcha',
    '🤲🏿' => 'bosa le chéile: ton cnis dorcha',
    '🤝🏻' => 'croitheadh lámh: ton cnis geal',
    '🤝🏼' => 'croitheadh lámh: ton cnis mheánach-gheal',
    '🤝🏽' => 'croitheadh lámh: ton cnis meánach',
    '🤝🏾' => 'croitheadh lámh: ton cnis meándorcha',
    '🤝🏿' => 'croitheadh lámh: ton cnis dorcha',
    '🙏🏻' => 'lámha le chéile: ton cnis geal',
    '🙏🏼' => 'lámha le chéile: ton cnis mheánach-gheal',
    '🙏🏽' => 'lámha le chéile: ton cnis meánach',
    '🙏🏾' => 'lámha le chéile: ton cnis meándorcha',
    '🙏🏿' => 'lámha le chéile: ton cnis dorcha',
    '✍🏻' => 'lámh scríbhneoireachta: ton cnis geal',
    '✍🏼' => 'lámh scríbhneoireachta: ton cnis mheánach-gheal',
    '✍🏽' => 'lámh scríbhneoireachta: ton cnis meánach',
    '✍🏾' => 'lámh scríbhneoireachta: ton cnis meándorcha',
    '✍🏿' => 'lámh scríbhneoireachta: ton cnis dorcha',
    '💅🏻' => 'vearnais ingne: ton cnis geal',
    '💅🏼' => 'vearnais ingne: ton cnis mheánach-gheal',
    '💅🏽' => 'vearnais ingne: ton cnis meánach',
    '💅🏾' => 'vearnais ingne: ton cnis meándorcha',
    '💅🏿' => 'vearnais ingne: ton cnis dorcha',
    '🤳🏻' => 'féinín: ton cnis geal',
    '🤳🏼' => 'féinín: ton cnis mheánach-gheal',
    '🤳🏽' => 'féinín: ton cnis meánach',
    '🤳🏾' => 'féinín: ton cnis meándorcha',
    '🤳🏿' => 'féinín: ton cnis dorcha',
    '💪🏻' => 'bícéips lúbtha: ton cnis geal',
    '💪🏼' => 'bícéips lúbtha: ton cnis mheánach-gheal',
    '💪🏽' => 'bícéips lúbtha: ton cnis meánach',
    '💪🏾' => 'bícéips lúbtha: ton cnis meándorcha',
    '💪🏿' => 'bícéips lúbtha: ton cnis dorcha',
    '🦵🏻' => 'cos: ton cnis geal',
    '🦵🏼' => 'cos: ton cnis mheánach-gheal',
    '🦵🏽' => 'cos: ton cnis meánach',
    '🦵🏾' => 'cos: ton cnis meándorcha',
    '🦵🏿' => 'cos: ton cnis dorcha',
    '🦶🏻' => 'troigh: ton cnis geal',
    '🦶🏼' => 'troigh: ton cnis mheánach-gheal',
    '🦶🏽' => 'troigh: ton cnis meánach',
    '🦶🏾' => 'troigh: ton cnis meándorcha',
    '🦶🏿' => 'troigh: ton cnis dorcha',
    '👂🏻' => 'cluas: ton cnis geal',
    '👂🏼' => 'cluas: ton cnis mheánach-gheal',
    '👂🏽' => 'cluas: ton cnis meánach',
    '👂🏾' => 'cluas: ton cnis meándorcha',
    '👂🏿' => 'cluas: ton cnis dorcha',
    '🦻🏻' => 'cluas le háis éisteachta: ton cnis geal',
    '🦻🏼' => 'cluas le háis éisteachta: ton cnis mheánach-gheal',
    '🦻🏽' => 'cluas le háis éisteachta: ton cnis meánach',
    '🦻🏾' => 'cluas le háis éisteachta: ton cnis meándorcha',
    '🦻🏿' => 'cluas le háis éisteachta: ton cnis dorcha',
    '👃🏻' => 'srón: ton cnis geal',
    '👃🏼' => 'srón: ton cnis mheánach-gheal',
    '👃🏽' => 'srón: ton cnis meánach',
    '👃🏾' => 'srón: ton cnis meándorcha',
    '👃🏿' => 'srón: ton cnis dorcha',
    '👶🏻' => 'babaí: ton cnis geal',
    '👶🏼' => 'babaí: ton cnis mheánach-gheal',
    '👶🏽' => 'babaí: ton cnis meánach',
    '👶🏾' => 'babaí: ton cnis meándorcha',
    '👶🏿' => 'babaí: ton cnis dorcha',
    '🧒🏻' => 'leanbh: ton cnis geal',
    '🧒🏼' => 'leanbh: ton cnis mheánach-gheal',
    '🧒🏽' => 'leanbh: ton cnis meánach',
    '🧒🏾' => 'leanbh: ton cnis meándorcha',
    '🧒🏿' => 'leanbh: ton cnis dorcha',
    '👦🏻' => 'buachaill: ton cnis geal',
    '👦🏼' => 'buachaill: ton cnis mheánach-gheal',
    '👦🏽' => 'buachaill: ton cnis meánach',
    '👦🏾' => 'buachaill: ton cnis meándorcha',
    '👦🏿' => 'buachaill: ton cnis dorcha',
    '👧🏻' => 'cailín: ton cnis geal',
    '👧🏼' => 'cailín: ton cnis mheánach-gheal',
    '👧🏽' => 'cailín: ton cnis meánach',
    '👧🏾' => 'cailín: ton cnis meándorcha',
    '👧🏿' => 'cailín: ton cnis dorcha',
    '🧑🏻' => 'aosach: ton cnis geal',
    '🧑🏼' => 'aosach: ton cnis mheánach-gheal',
    '🧑🏽' => 'aosach: ton cnis meánach',
    '🧑🏾' => 'aosach: ton cnis meándorcha',
    '🧑🏿' => 'aosach: ton cnis dorcha',
    '👱🏻' => 'duine: ton cnis geal, gruaig fhionn',
    '👱🏼' => 'duine: ton cnis mheánach-gheal, gruaig fhionn',
    '👱🏽' => 'duine: ton cnis meánach, gruaig fhionn',
    '👱🏾' => 'duine: ton cnis meándorcha, gruaig fhionn',
    '👱🏿' => 'duine: ton cnis dorcha, gruaig fhionn',
    '👨🏻' => 'fear: ton cnis geal',
    '👨🏼' => 'fear: ton cnis mheánach-gheal',
    '👨🏽' => 'fear: ton cnis meánach',
    '👨🏾' => 'fear: ton cnis meándorcha',
    '👨🏿' => 'fear: ton cnis dorcha',
    '🧔🏻' => 'duine: ton cnis geal, feasóg',
    '🧔🏼' => 'duine: ton cnis mheánach-gheal, feasóg',
    '🧔🏽' => 'duine: ton cnis meánach, feasóg',
    '🧔🏾' => 'duine: ton cnis meándorcha, feasóg',
    '🧔🏿' => 'duine: ton cnis dorcha, feasóg',
    '👩🏻' => 'bean: ton cnis geal',
    '👩🏼' => 'bean: ton cnis mheánach-gheal',
    '👩🏽' => 'bean: ton cnis meánach',
    '👩🏾' => 'bean: ton cnis meándorcha',
    '👩🏿' => 'bean: ton cnis dorcha',
    '🧓🏻' => 'seanduine: ton cnis geal',
    '🧓🏼' => 'seanduine: ton cnis mheánach-gheal',
    '🧓🏽' => 'seanduine: ton cnis meánach',
    '🧓🏾' => 'seanduine: ton cnis meándorcha',
    '🧓🏿' => 'seanduine: ton cnis dorcha',
    '👴🏻' => 'seanfhear: ton cnis geal',
    '👴🏼' => 'seanfhear: ton cnis mheánach-gheal',
    '👴🏽' => 'seanfhear: ton cnis meánach',
    '👴🏾' => 'seanfhear: ton cnis meándorcha',
    '👴🏿' => 'seanfhear: ton cnis dorcha',
    '👵🏻' => 'seanbhean: ton cnis geal',
    '👵🏼' => 'seanbhean: ton cnis mheánach-gheal',
    '👵🏽' => 'seanbhean: ton cnis meánach',
    '👵🏾' => 'seanbhean: ton cnis meándorcha',
    '👵🏿' => 'seanbhean: ton cnis dorcha',
    '🙍🏻' => 'duine le púic air: ton cnis geal',
    '🙍🏼' => 'duine le púic air: ton cnis mheánach-gheal',
    '🙍🏽' => 'duine le púic air: ton cnis meánach',
    '🙍🏾' => 'duine le púic air: ton cnis meándorcha',
    '🙍🏿' => 'duine le púic air: ton cnis dorcha',
    '🙎🏻' => 'duine le puisín air: ton cnis geal',
    '🙎🏼' => 'duine le puisín air: ton cnis mheánach-gheal',
    '🙎🏽' => 'duine le puisín air: ton cnis meánach',
    '🙎🏾' => 'duine le puisín air: ton cnis meándorcha',
    '🙎🏿' => 'duine le puisín air: ton cnis dorcha',
    '🙅🏻' => 'duine ag comharthú NÍL: ton cnis geal',
    '🙅🏼' => 'duine ag comharthú NÍL: ton cnis mheánach-gheal',
    '🙅🏽' => 'duine ag comharthú NÍL: ton cnis meánach',
    '🙅🏾' => 'duine ag comharthú NÍL: ton cnis meándorcha',
    '🙅🏿' => 'duine ag comharthú NÍL: ton cnis dorcha',
    '🙆🏻' => 'duine ag comharthú OK: ton cnis geal',
    '🙆🏼' => 'duine ag comharthú OK: ton cnis mheánach-gheal',
    '🙆🏽' => 'duine ag comharthú OK: ton cnis meánach',
    '🙆🏾' => 'duine ag comharthú OK: ton cnis meándorcha',
    '🙆🏿' => 'duine ag comharthú OK: ton cnis dorcha',
    '💁🏻' => 'duine lena lámh amach: ton cnis geal',
    '💁🏼' => 'duine lena lámh amach: ton cnis mheánach-gheal',
    '💁🏽' => 'duine lena lámh amach: ton cnis meánach',
    '💁🏾' => 'duine lena lámh amach: ton cnis meándorcha',
    '💁🏿' => 'duine lena lámh amach: ton cnis dorcha',
    '🙋🏻' => 'duine lena lámh suas: ton cnis geal',
    '🙋🏼' => 'duine lena lámh suas: ton cnis mheánach-gheal',
    '🙋🏽' => 'duine lena lámh suas: ton cnis meánach',
    '🙋🏾' => 'duine lena lámh suas: ton cnis meándorcha',
    '🙋🏿' => 'duine lena lámh suas: ton cnis dorcha',
    '🧏🏻' => 'duine bodhar: ton cnis geal',
    '🧏🏼' => 'duine bodhar: ton cnis mheánach-gheal',
    '🧏🏽' => 'duine bodhar: ton cnis meánach',
    '🧏🏾' => 'duine bodhar: ton cnis meándorcha',
    '🧏🏿' => 'duine bodhar: ton cnis dorcha',
    '🙇🏻' => 'duine ag umhlú: ton cnis geal',
    '🙇🏼' => 'duine ag umhlú: ton cnis mheánach-gheal',
    '🙇🏽' => 'duine ag umhlú: ton cnis meánach',
    '🙇🏾' => 'duine ag umhlú: ton cnis meándorcha',
    '🙇🏿' => 'duine ag umhlú: ton cnis dorcha',
    '🤦🏻' => 'duine lena aghaidh ina bhos: ton cnis geal',
    '🤦🏼' => 'duine lena aghaidh ina bhos: ton cnis mheánach-gheal',
    '🤦🏽' => 'duine lena aghaidh ina bhos: ton cnis meánach',
    '🤦🏾' => 'duine lena aghaidh ina bhos: ton cnis meándorcha',
    '🤦🏿' => 'duine lena aghaidh ina bhos: ton cnis dorcha',
    '🤷🏻' => 'duine ag croitheadh a ghuaillí: ton cnis geal',
    '🤷🏼' => 'duine ag croitheadh a ghuaillí: ton cnis mheánach-gheal',
    '🤷🏽' => 'duine ag croitheadh a ghuaillí: ton cnis meánach',
    '🤷🏾' => 'duine ag croitheadh a ghuaillí: ton cnis meándorcha',
    '🤷🏿' => 'duine ag croitheadh a ghuaillí: ton cnis dorcha',
    '👮🏻' => 'póilín: ton cnis geal',
    '👮🏼' => 'póilín: ton cnis mheánach-gheal',
    '👮🏽' => 'póilín: ton cnis meánach',
    '👮🏾' => 'póilín: ton cnis meándorcha',
    '👮🏿' => 'póilín: ton cnis dorcha',
    '🕵🏻' => 'bleachtaire: ton cnis geal',
    '🕵🏼' => 'bleachtaire: ton cnis mheánach-gheal',
    '🕵🏽' => 'bleachtaire: ton cnis meánach',
    '🕵🏾' => 'bleachtaire: ton cnis meándorcha',
    '🕵🏿' => 'bleachtaire: ton cnis dorcha',
    '💂🏻' => 'garda an rí: ton cnis geal',
    '💂🏼' => 'garda an rí: ton cnis mheánach-gheal',
    '💂🏽' => 'garda an rí: ton cnis meánach',
    '💂🏾' => 'garda an rí: ton cnis meándorcha',
    '💂🏿' => 'garda an rí: ton cnis dorcha',
    '🥷🏻' => 'ninja: ton cnis geal',
    '🥷🏼' => 'ninja: ton cnis mheánach-gheal',
    '🥷🏽' => 'ninja: ton cnis meánach',
    '🥷🏾' => 'ninja: ton cnis meándorcha',
    '🥷🏿' => 'ninja: ton cnis dorcha',
    '👷🏻' => 'tógálaí: ton cnis geal',
    '👷🏼' => 'tógálaí: ton cnis mheánach-gheal',
    '👷🏽' => 'tógálaí: ton cnis meánach',
    '👷🏾' => 'tógálaí: ton cnis meándorcha',
    '👷🏿' => 'tógálaí: ton cnis dorcha',
    '🫅🏻' => 'duine le coróin air: ton cnis geal',
    '🫅🏼' => 'duine le coróin air: ton cnis mheánach-gheal',
    '🫅🏽' => 'duine le coróin air: ton cnis meánach',
    '🫅🏾' => 'duine le coróin air: ton cnis meándorcha',
    '🫅🏿' => 'duine le coróin air: ton cnis dorcha',
    '🤴🏻' => 'prionsa: ton cnis geal',
    '🤴🏼' => 'prionsa: ton cnis mheánach-gheal',
    '🤴🏽' => 'prionsa: ton cnis meánach',
    '🤴🏾' => 'prionsa: ton cnis meándorcha',
    '🤴🏿' => 'prionsa: ton cnis dorcha',
    '👸🏻' => 'banphrionsa: ton cnis geal',
    '👸🏼' => 'banphrionsa: ton cnis mheánach-gheal',
    '👸🏽' => 'banphrionsa: ton cnis meánach',
    '👸🏾' => 'banphrionsa: ton cnis meándorcha',
    '👸🏿' => 'banphrionsa: ton cnis dorcha',
    '👳🏻' => 'duine le turban air: ton cnis geal',
    '👳🏼' => 'duine le turban air: ton cnis mheánach-gheal',
    '👳🏽' => 'duine le turban air: ton cnis meánach',
    '👳🏾' => 'duine le turban air: ton cnis meándorcha',
    '👳🏿' => 'duine le turban air: ton cnis dorcha',
    '👲🏻' => 'fear le gua pi mao air: ton cnis geal',
    '👲🏼' => 'fear le gua pi mao air: ton cnis mheánach-gheal',
    '👲🏽' => 'fear le gua pi mao air: ton cnis meánach',
    '👲🏾' => 'fear le gua pi mao air: ton cnis meándorcha',
    '👲🏿' => 'fear le gua pi mao air: ton cnis dorcha',
    '🧕🏻' => 'bean le caifirín: ton cnis geal',
    '🧕🏼' => 'bean le caifirín: ton cnis mheánach-gheal',
    '🧕🏽' => 'bean le caifirín: ton cnis meánach',
    '🧕🏾' => 'bean le caifirín: ton cnis meándorcha',
    '🧕🏿' => 'bean le caifirín: ton cnis dorcha',
    '🤵🏻' => 'duine a bhfuil culaith dhinnéir air: ton cnis geal',
    '🤵🏼' => 'duine a bhfuil culaith dhinnéir air: ton cnis mheánach-gheal',
    '🤵🏽' => 'duine a bhfuil culaith dhinnéir air: ton cnis meánach',
    '🤵🏾' => 'duine a bhfuil culaith dhinnéir air: ton cnis meándorcha',
    '🤵🏿' => 'duine a bhfuil culaith dhinnéir air: ton cnis dorcha',
    '👰🏻' => 'duine le caille air: ton cnis geal',
    '👰🏼' => 'duine le caille air: ton cnis mheánach-gheal',
    '👰🏽' => 'duine le caille air: ton cnis meánach',
    '👰🏾' => 'duine le caille air: ton cnis meándorcha',
    '👰🏿' => 'duine le caille air: ton cnis dorcha',
    '🤰🏻' => 'bean ag iompar clainne: ton cnis geal',
    '🤰🏼' => 'bean ag iompar clainne: ton cnis mheánach-gheal',
    '🤰🏽' => 'bean ag iompar clainne: ton cnis meánach',
    '🤰🏾' => 'bean ag iompar clainne: ton cnis meándorcha',
    '🤰🏿' => 'bean ag iompar clainne: ton cnis dorcha',
    '🫃🏻' => 'fear torrach: ton cnis geal',
    '🫃🏼' => 'fear torrach: ton cnis mheánach-gheal',
    '🫃🏽' => 'fear torrach: ton cnis meánach',
    '🫃🏾' => 'fear torrach: ton cnis meándorcha',
    '🫃🏿' => 'fear torrach: ton cnis dorcha',
    '🫄🏻' => 'duine torrach: ton cnis geal',
    '🫄🏼' => 'duine torrach: ton cnis mheánach-gheal',
    '🫄🏽' => 'duine torrach: ton cnis meánach',
    '🫄🏾' => 'duine torrach: ton cnis meándorcha',
    '🫄🏿' => 'duine torrach: ton cnis dorcha',
    '🤱🏻' => 'cothú cíche: ton cnis geal',
    '🤱🏼' => 'cothú cíche: ton cnis mheánach-gheal',
    '🤱🏽' => 'cothú cíche: ton cnis meánach',
    '🤱🏾' => 'cothú cíche: ton cnis meándorcha',
    '🤱🏿' => 'cothú cíche: ton cnis dorcha',
    '👼🏻' => 'aingeal óg: ton cnis geal',
    '👼🏼' => 'aingeal óg: ton cnis mheánach-gheal',
    '👼🏽' => 'aingeal óg: ton cnis meánach',
    '👼🏾' => 'aingeal óg: ton cnis meándorcha',
    '👼🏿' => 'aingeal óg: ton cnis dorcha',
    '🎅🏻' => 'Daidí na Nollag: ton cnis geal',
    '🎅🏼' => 'Daidí na Nollag: ton cnis mheánach-gheal',
    '🎅🏽' => 'Daidí na Nollag: ton cnis meánach',
    '🎅🏾' => 'Daidí na Nollag: ton cnis meándorcha',
    '🎅🏿' => 'Daidí na Nollag: ton cnis dorcha',
    '🤶🏻' => 'Bean Uí Claus: ton cnis geal',
    '🤶🏼' => 'Bean Uí Claus: ton cnis mheánach-gheal',
    '🤶🏽' => 'Bean Uí Claus: ton cnis meánach',
    '🤶🏾' => 'Bean Uí Claus: ton cnis meándorcha',
    '🤶🏿' => 'Bean Uí Claus: ton cnis dorcha',
    '🦸🏻' => 'sárlaoch: ton cnis geal',
    '🦸🏼' => 'sárlaoch: ton cnis mheánach-gheal',
    '🦸🏽' => 'sárlaoch: ton cnis meánach',
    '🦸🏾' => 'sárlaoch: ton cnis meándorcha',
    '🦸🏿' => 'sárlaoch: ton cnis dorcha',
    '🦹🏻' => 'sárbhithiúnach: ton cnis geal',
    '🦹🏼' => 'sárbhithiúnach: ton cnis mheánach-gheal',
    '🦹🏽' => 'sárbhithiúnach: ton cnis meánach',
    '🦹🏾' => 'sárbhithiúnach: ton cnis meándorcha',
    '🦹🏿' => 'sárbhithiúnach: ton cnis dorcha',
    '🧙🏻' => 'duine draíochta: ton cnis geal',
    '🧙🏼' => 'duine draíochta: ton cnis mheánach-gheal',
    '🧙🏽' => 'duine draíochta: ton cnis meánach',
    '🧙🏾' => 'duine draíochta: ton cnis meándorcha',
    '🧙🏿' => 'duine draíochta: ton cnis dorcha',
    '🧚🏻' => 'síog: ton cnis geal',
    '🧚🏼' => 'síog: ton cnis mheánach-gheal',
    '🧚🏽' => 'síog: ton cnis meánach',
    '🧚🏾' => 'síog: ton cnis meándorcha',
    '🧚🏿' => 'síog: ton cnis dorcha',
    '🧛🏻' => 'vaimpír: ton cnis geal',
    '🧛🏼' => 'vaimpír: ton cnis mheánach-gheal',
    '🧛🏽' => 'vaimpír: ton cnis meánach',
    '🧛🏾' => 'vaimpír: ton cnis meándorcha',
    '🧛🏿' => 'vaimpír: ton cnis dorcha',
    '🧜🏻' => 'duine murúch: ton cnis geal',
    '🧜🏼' => 'duine murúch: ton cnis mheánach-gheal',
    '🧜🏽' => 'duine murúch: ton cnis meánach',
    '🧜🏾' => 'duine murúch: ton cnis meándorcha',
    '🧜🏿' => 'duine murúch: ton cnis dorcha',
    '🧝🏻' => 'lucharachán: ton cnis geal',
    '🧝🏼' => 'lucharachán: ton cnis mheánach-gheal',
    '🧝🏽' => 'lucharachán: ton cnis meánach',
    '🧝🏾' => 'lucharachán: ton cnis meándorcha',
    '🧝🏿' => 'lucharachán: ton cnis dorcha',
    '💆🏻' => 'duine ag fáil suathaireachta: ton cnis geal',
    '💆🏼' => 'duine ag fáil suathaireachta: ton cnis mheánach-gheal',
    '💆🏽' => 'duine ag fáil suathaireachta: ton cnis meánach',
    '💆🏾' => 'duine ag fáil suathaireachta: ton cnis meándorcha',
    '💆🏿' => 'duine ag fáil suathaireachta: ton cnis dorcha',
    '💇🏻' => 'duine ag fáil bearradh gruaige: ton cnis geal',
    '💇🏼' => 'duine ag fáil bearradh gruaige: ton cnis mheánach-gheal',
    '💇🏽' => 'duine ag fáil bearradh gruaige: ton cnis meánach',
    '💇🏾' => 'duine ag fáil bearradh gruaige: ton cnis meándorcha',
    '💇🏿' => 'duine ag fáil bearradh gruaige: ton cnis dorcha',
    '🚶🏻' => 'coisí: ton cnis geal',
    '🚶🏼' => 'coisí: ton cnis mheánach-gheal',
    '🚶🏽' => 'coisí: ton cnis meánach',
    '🚶🏾' => 'coisí: ton cnis meándorcha',
    '🚶🏿' => 'coisí: ton cnis dorcha',
    '🧍🏻' => 'duine ina sheasamh: ton cnis geal',
    '🧍🏼' => 'duine ina sheasamh: ton cnis mheánach-gheal',
    '🧍🏽' => 'duine ina sheasamh: ton cnis meánach',
    '🧍🏾' => 'duine ina sheasamh: ton cnis meándorcha',
    '🧍🏿' => 'duine ina sheasamh: ton cnis dorcha',
    '🧎🏻' => 'duine ar a ghlúine: ton cnis geal',
    '🧎🏼' => 'duine ar a ghlúine: ton cnis mheánach-gheal',
    '🧎🏽' => 'duine ar a ghlúine: ton cnis meánach',
    '🧎🏾' => 'duine ar a ghlúine: ton cnis meándorcha',
    '🧎🏿' => 'duine ar a ghlúine: ton cnis dorcha',
    '🏃🏻' => 'reathaí: ton cnis geal',
    '🏃🏼' => 'reathaí: ton cnis mheánach-gheal',
    '🏃🏽' => 'reathaí: ton cnis meánach',
    '🏃🏾' => 'reathaí: ton cnis meándorcha',
    '🏃🏿' => 'reathaí: ton cnis dorcha',
    '💃🏻' => 'bean ag damhsa: ton cnis geal',
    '💃🏼' => 'bean ag damhsa: ton cnis mheánach-gheal',
    '💃🏽' => 'bean ag damhsa: ton cnis meánach',
    '💃🏾' => 'bean ag damhsa: ton cnis meándorcha',
    '💃🏿' => 'bean ag damhsa: ton cnis dorcha',
    '🕺🏻' => 'fear ag damhsa: ton cnis geal',
    '🕺🏼' => 'fear ag damhsa: ton cnis mheánach-gheal',
    '🕺🏽' => 'fear ag damhsa: ton cnis meánach',
    '🕺🏾' => 'fear ag damhsa: ton cnis meándorcha',
    '🕺🏿' => 'fear ag damhsa: ton cnis dorcha',
    '🕴🏻' => 'fear i gculaith gnó ar eadarbhuas: ton cnis geal',
    '🕴🏼' => 'fear i gculaith gnó ar eadarbhuas: ton cnis mheánach-gheal',
    '🕴🏽' => 'fear i gculaith gnó ar eadarbhuas: ton cnis meánach',
    '🕴🏾' => 'fear i gculaith gnó ar eadarbhuas: ton cnis meándorcha',
    '🕴🏿' => 'fear i gculaith gnó ar eadarbhuas: ton cnis dorcha',
    '👯🏻' => 'daoine le cluasa coiníní orthu: ton cnis geal',
    '👯🏼' => 'daoine le cluasa coiníní orthu: ton cnis mheánach-gheal',
    '👯🏽' => 'daoine le cluasa coiníní orthu: ton cnis meánach',
    '👯🏾' => 'daoine le cluasa coiníní orthu: ton cnis meándorcha',
    '👯🏿' => 'daoine le cluasa coiníní orthu: ton cnis dorcha',
    '🧖🏻' => 'duine i seomra gaile: ton cnis geal',
    '🧖🏼' => 'duine i seomra gaile: ton cnis mheánach-gheal',
    '🧖🏽' => 'duine i seomra gaile: ton cnis meánach',
    '🧖🏾' => 'duine i seomra gaile: ton cnis meándorcha',
    '🧖🏿' => 'duine i seomra gaile: ton cnis dorcha',
    '🧗🏻' => 'duine ag dreapadóireacht: ton cnis geal',
    '🧗🏼' => 'duine ag dreapadóireacht: ton cnis mheánach-gheal',
    '🧗🏽' => 'duine ag dreapadóireacht: ton cnis meánach',
    '🧗🏾' => 'duine ag dreapadóireacht: ton cnis meándorcha',
    '🧗🏿' => 'duine ag dreapadóireacht: ton cnis dorcha',
    '🏇🏻' => 'rásaíocht capall: ton cnis geal',
    '🏇🏼' => 'rásaíocht capall: ton cnis mheánach-gheal',
    '🏇🏽' => 'rásaíocht capall: ton cnis meánach',
    '🏇🏾' => 'rásaíocht capall: ton cnis meándorcha',
    '🏇🏿' => 'rásaíocht capall: ton cnis dorcha',
    '🏂🏻' => 'clárscimeálaí sneachta: ton cnis geal',
    '🏂🏼' => 'clárscimeálaí sneachta: ton cnis mheánach-gheal',
    '🏂🏽' => 'clárscimeálaí sneachta: ton cnis meánach',
    '🏂🏾' => 'clárscimeálaí sneachta: ton cnis meándorcha',
    '🏂🏿' => 'clárscimeálaí sneachta: ton cnis dorcha',
    '🏌🏻' => 'duine ag imirt gailf: ton cnis geal',
    '🏌🏼' => 'duine ag imirt gailf: ton cnis mheánach-gheal',
    '🏌🏽' => 'duine ag imirt gailf: ton cnis meánach',
    '🏌🏾' => 'duine ag imirt gailf: ton cnis meándorcha',
    '🏌🏿' => 'duine ag imirt gailf: ton cnis dorcha',
    '🏄🏻' => 'duine ag surfáil: ton cnis geal',
    '🏄🏼' => 'duine ag surfáil: ton cnis mheánach-gheal',
    '🏄🏽' => 'duine ag surfáil: ton cnis meánach',
    '🏄🏾' => 'duine ag surfáil: ton cnis meándorcha',
    '🏄🏿' => 'duine ag surfáil: ton cnis dorcha',
    '🚣🏻' => 'bád rámhaíochta: ton cnis geal',
    '🚣🏼' => 'bád rámhaíochta: ton cnis mheánach-gheal',
    '🚣🏽' => 'bád rámhaíochta: ton cnis meánach',
    '🚣🏾' => 'bád rámhaíochta: ton cnis meándorcha',
    '🚣🏿' => 'bád rámhaíochta: ton cnis dorcha',
    '🏊🏻' => 'snámhaí: ton cnis geal',
    '🏊🏼' => 'snámhaí: ton cnis mheánach-gheal',
    '🏊🏽' => 'snámhaí: ton cnis meánach',
    '🏊🏾' => 'snámhaí: ton cnis meándorcha',
    '🏊🏿' => 'snámhaí: ton cnis dorcha',
    '⛹🏻' => 'duine le liathróid: ton cnis geal',
    '⛹🏼' => 'duine le liathróid: ton cnis mheánach-gheal',
    '⛹🏽' => 'duine le liathróid: ton cnis meánach',
    '⛹🏾' => 'duine le liathróid: ton cnis meándorcha',
    '⛹🏿' => 'duine le liathróid: ton cnis dorcha',
    '🏋🏻' => 'duine ag tógáil meachán: ton cnis geal',
    '🏋🏼' => 'duine ag tógáil meachán: ton cnis mheánach-gheal',
    '🏋🏽' => 'duine ag tógáil meachán: ton cnis meánach',
    '🏋🏾' => 'duine ag tógáil meachán: ton cnis meándorcha',
    '🏋🏿' => 'duine ag tógáil meachán: ton cnis dorcha',
    '🚴🏻' => 'rothaí: ton cnis geal',
    '🚴🏼' => 'rothaí: ton cnis mheánach-gheal',
    '🚴🏽' => 'rothaí: ton cnis meánach',
    '🚴🏾' => 'rothaí: ton cnis meándorcha',
    '🚴🏿' => 'rothaí: ton cnis dorcha',
    '🚵🏻' => 'rothaí sléibhe: ton cnis geal',
    '🚵🏼' => 'rothaí sléibhe: ton cnis mheánach-gheal',
    '🚵🏽' => 'rothaí sléibhe: ton cnis meánach',
    '🚵🏾' => 'rothaí sléibhe: ton cnis meándorcha',
    '🚵🏿' => 'rothaí sléibhe: ton cnis dorcha',
    '🤸🏻' => 'rothchasadh: ton cnis geal',
    '🤸🏼' => 'rothchasadh: ton cnis mheánach-gheal',
    '🤸🏽' => 'rothchasadh: ton cnis meánach',
    '🤸🏾' => 'rothchasadh: ton cnis meándorcha',
    '🤸🏿' => 'rothchasadh: ton cnis dorcha',
    '🤼🏻' => 'coraíocht: ton cnis geal',
    '🤼🏼' => 'coraíocht: ton cnis mheánach-gheal',
    '🤼🏽' => 'coraíocht: ton cnis meánach',
    '🤼🏾' => 'coraíocht: ton cnis meándorcha',
    '🤼🏿' => 'coraíocht: ton cnis dorcha',
    '🤽🏻' => 'póló uisce: ton cnis geal',
    '🤽🏼' => 'póló uisce: ton cnis mheánach-gheal',
    '🤽🏽' => 'póló uisce: ton cnis meánach',
    '🤽🏾' => 'póló uisce: ton cnis meándorcha',
    '🤽🏿' => 'póló uisce: ton cnis dorcha',
    '🤾🏻' => 'liathróid láimhe: ton cnis geal',
    '🤾🏼' => 'liathróid láimhe: ton cnis mheánach-gheal',
    '🤾🏽' => 'liathróid láimhe: ton cnis meánach',
    '🤾🏾' => 'liathróid láimhe: ton cnis meándorcha',
    '🤾🏿' => 'liathróid láimhe: ton cnis dorcha',
    '🤹🏻' => 'lámhchleasaíocht: ton cnis geal',
    '🤹🏼' => 'lámhchleasaíocht: ton cnis mheánach-gheal',
    '🤹🏽' => 'lámhchleasaíocht: ton cnis meánach',
    '🤹🏾' => 'lámhchleasaíocht: ton cnis meándorcha',
    '🤹🏿' => 'lámhchleasaíocht: ton cnis dorcha',
    '🧘🏻' => 'duine i suíomh na loiteoige: ton cnis geal',
    '🧘🏼' => 'duine i suíomh na loiteoige: ton cnis mheánach-gheal',
    '🧘🏽' => 'duine i suíomh na loiteoige: ton cnis meánach',
    '🧘🏾' => 'duine i suíomh na loiteoige: ton cnis meándorcha',
    '🧘🏿' => 'duine i suíomh na loiteoige: ton cnis dorcha',
    '🛀🏻' => 'duine ag folcadh: ton cnis geal',
    '🛀🏼' => 'duine ag folcadh: ton cnis mheánach-gheal',
    '🛀🏽' => 'duine ag folcadh: ton cnis meánach',
    '🛀🏾' => 'duine ag folcadh: ton cnis meándorcha',
    '🛀🏿' => 'duine ag folcadh: ton cnis dorcha',
    '🛌🏻' => 'duine sa leaba: ton cnis geal',
    '🛌🏼' => 'duine sa leaba: ton cnis mheánach-gheal',
    '🛌🏽' => 'duine sa leaba: ton cnis meánach',
    '🛌🏾' => 'duine sa leaba: ton cnis meándorcha',
    '🛌🏿' => 'duine sa leaba: ton cnis dorcha',
    '👭🏻' => 'beirt bhan le chéile: ton cnis geal',
    '👭🏼' => 'beirt bhan le chéile: ton cnis mheánach-gheal',
    '👭🏽' => 'beirt bhan le chéile: ton cnis meánach',
    '👭🏾' => 'beirt bhan le chéile: ton cnis meándorcha',
    '👭🏿' => 'beirt bhan le chéile: ton cnis dorcha',
    '👫🏻' => 'fear agus bean le chéile: ton cnis geal',
    '👫🏼' => 'fear agus bean le chéile: ton cnis mheánach-gheal',
    '👫🏽' => 'fear agus bean le chéile: ton cnis meánach',
    '👫🏾' => 'fear agus bean le chéile: ton cnis meándorcha',
    '👫🏿' => 'fear agus bean le chéile: ton cnis dorcha',
    '👬🏻' => 'beirt fhear le chéile: ton cnis geal',
    '👬🏼' => 'beirt fhear le chéile: ton cnis mheánach-gheal',
    '👬🏽' => 'beirt fhear le chéile: ton cnis meánach',
    '👬🏾' => 'beirt fhear le chéile: ton cnis meándorcha',
    '👬🏿' => 'beirt fhear le chéile: ton cnis dorcha',
    '💏🏻' => 'póg: ton cnis geal',
    '💏🏼' => 'póg: ton cnis mheánach-gheal',
    '💏🏽' => 'póg: ton cnis meánach',
    '💏🏾' => 'póg: ton cnis meándorcha',
    '💏🏿' => 'póg: ton cnis dorcha',
    '💑🏻' => 'lanúin le croí: ton cnis geal',
    '💑🏼' => 'lanúin le croí: ton cnis mheánach-gheal',
    '💑🏽' => 'lanúin le croí: ton cnis meánach',
    '💑🏾' => 'lanúin le croí: ton cnis meándorcha',
    '💑🏿' => 'lanúin le croí: ton cnis dorcha',
    '🇦🇨' => 'bratach: Oileán na Deascabhála',
    '🇦🇩' => 'bratach: Andóra',
    '🇦🇪' => 'bratach: Aontas na nÉimíríochtaí Arabacha',
    '🇦🇫' => 'bratach: an Afganastáin',
    '🇦🇬' => 'bratach: Antigua agus Barbúda',
    '🇦🇮' => 'bratach: Angaíle',
    '🇦🇱' => 'bratach: an Albáin',
    '🇦🇲' => 'bratach: an Airméin',
    '🇦🇴' => 'bratach: Angóla',
    '🇦🇶' => 'bratach: Antartaice',
    '🇦🇷' => 'bratach: an Airgintín',
    '🇦🇸' => 'bratach: Samó Mheiriceá',
    '🇦🇹' => 'bratach: an Ostair',
    '🇦🇺' => 'bratach: an Astráil',
    '🇦🇼' => 'bratach: Arúba',
    '🇦🇽' => 'bratach: Oileáin Åland',
    '🇦🇿' => 'bratach: an Asarbaiseáin',
    '🇧🇦' => 'bratach: an Bhoisnia agus an Heirseagaivéin',
    '🇧🇧' => 'bratach: Barbadós',
    '🇧🇩' => 'bratach: an Bhanglaidéis',
    '🇧🇪' => 'bratach: an Bheilg',
    '🇧🇫' => 'bratach: Buircíne Fasó',
    '🇧🇬' => 'bratach: an Bhulgáir',
    '🇧🇭' => 'bratach: Bairéin',
    '🇧🇮' => 'bratach: an Bhurúin',
    '🇧🇯' => 'bratach: Beinin',
    '🇧🇱' => 'bratach: Saint Barthélemy',
    '🇧🇲' => 'bratach: Beirmiúda',
    '🇧🇳' => 'bratach: Brúiné',
    '🇧🇴' => 'bratach: an Bholaiv',
    '🇧🇶' => 'bratach: an Ísiltír Chairibeach',
    '🇧🇷' => 'bratach: an Bhrasaíl',
    '🇧🇸' => 'bratach: na Bahámaí',
    '🇧🇹' => 'bratach: an Bhútáin',
    '🇧🇻' => 'bratach: Oileán Bouvet',
    '🇧🇼' => 'bratach: an Bhotsuáin',
    '🇧🇾' => 'bratach: an Bhealarúis',
    '🇧🇿' => 'bratach: an Bheilís',
    '🇨🇦' => 'bratach: Ceanada',
    '🇨🇨' => 'bratach: Oileáin Cocos (Keeling)',
    '🇨🇩' => 'bratach: Poblacht Dhaonlathach an Chongó',
    '🇨🇫' => 'bratach: Poblacht na hAfraice Láir',
    '🇨🇬' => 'bratach: Congó-Brazzaville',
    '🇨🇭' => 'bratach: an Eilvéis',
    '🇨🇮' => 'bratach: Côte d’Ivoire',
    '🇨🇰' => 'bratach: Oileáin Cook',
    '🇨🇱' => 'bratach: an tSile',
    '🇨🇲' => 'bratach: Camarún',
    '🇨🇳' => 'bratach: an tSín',
    '🇨🇴' => 'bratach: an Cholóim',
    '🇨🇵' => 'bratach: Oileán Clipperton',
    '🇨🇶' => 'bratach: an tSairc',
    '🇨🇷' => 'bratach: Cósta Ríce',
    '🇨🇺' => 'bratach: Cúba',
    '🇨🇻' => 'bratach: Poblacht Cabo Verde',
    '🇨🇼' => 'bratach: Cúrasó',
    '🇨🇽' => 'bratach: Oileán na Nollag',
    '🇨🇾' => 'bratach: an Chipir',
    '🇨🇿' => 'bratach: an tSeicia',
    '🇩🇪' => 'bratach: an Ghearmáin',
    '🇩🇬' => 'bratach: Diego Garcia',
    '🇩🇯' => 'bratach: Djibouti',
    '🇩🇰' => 'bratach: an Danmhairg',
    '🇩🇲' => 'bratach: Doiminice',
    '🇩🇴' => 'bratach: an Phoblacht Dhoiminiceach',
    '🇩🇿' => 'bratach: an Ailgéir',
    '🇪🇦' => 'bratach: Ceuta agus Melilla',
    '🇪🇨' => 'bratach: Eacuadór',
    '🇪🇪' => 'bratach: an Eastóin',
    '🇪🇬' => 'bratach: an Éigipt',
    '🇪🇭' => 'bratach: an Sahára Thiar',
    '🇪🇷' => 'bratach: an Eiritré',
    '🇪🇸' => 'bratach: an Spáinn',
    '🇪🇹' => 'bratach: an Aetóip',
    '🇪🇺' => 'bratach: an tAontas Eorpach',
    '🇫🇮' => 'bratach: an Fhionlainn',
    '🇫🇯' => 'bratach: Fidsí',
    '🇫🇰' => 'bratach: Oileáin Fháclainne',
    '🇫🇲' => 'bratach: an Mhicrinéis',
    '🇫🇴' => 'bratach: Oileáin Fharó',
    '🇫🇷' => 'bratach: an Fhrainc',
    '🇬🇦' => 'bratach: an Ghabúin',
    '🇬🇧' => 'bratach: an Ríocht Aontaithe',
    '🇬🇩' => 'bratach: Greanáda',
    '🇬🇪' => 'bratach: an tSeoirsia',
    '🇬🇫' => 'bratach: Guáin na Fraince',
    '🇬🇬' => 'bratach: Geansaí',
    '🇬🇭' => 'bratach: Gána',
    '🇬🇮' => 'bratach: Giobráltar',
    '🇬🇱' => 'bratach: an Ghraonlainn',
    '🇬🇲' => 'bratach: an Ghaimbia',
    '🇬🇳' => 'bratach: an Ghuine',
    '🇬🇵' => 'bratach: Guadalúip',
    '🇬🇶' => 'bratach: an Ghuine Mheánchiorclach',
    '🇬🇷' => 'bratach: an Ghréig',
    '🇬🇸' => 'bratach: An tSeoirsia Theas agus Oileáin Sandwich Theas',
    '🇬🇹' => 'bratach: Guatamala',
    '🇬🇺' => 'bratach: Guam',
    '🇬🇼' => 'bratach: Guine Bissau',
    '🇬🇾' => 'bratach: An Ghuáin',
    '🇭🇰' => 'bratach: Sainréigiún Riaracháin Hong Cong, Daonphoblacht na Síne',
    '🇭🇲' => 'bratach: Oileán Heard agus Oileáin McDonald',
    '🇭🇳' => 'bratach: Hondúras',
    '🇭🇷' => 'bratach: an Chróit',
    '🇭🇹' => 'bratach: Háítí',
    '🇭🇺' => 'bratach: an Ungáir',
    '🇮🇨' => 'bratach: na hOileáin Chanáracha',
    '🇮🇩' => 'bratach: an Indinéis',
    '🇮🇪' => 'bratach: Éire',
    '🇮🇱' => 'bratach: Iosrael',
    '🇮🇲' => 'bratach: Oileán Mhanann',
    '🇮🇳' => 'bratach: an India',
    '🇮🇴' => 'bratach: Críoch Aigéan Indiach na Breataine',
    '🇮🇶' => 'bratach: an Iaráic',
    '🇮🇷' => 'bratach: an Iaráin',
    '🇮🇸' => 'bratach: an Íoslainn',
    '🇮🇹' => 'bratach: an Iodáil',
    '🇯🇪' => 'bratach: Geirsí',
    '🇯🇲' => 'bratach: Iamáice',
    '🇯🇴' => 'bratach: an Iordáin',
    '🇯🇵' => 'bratach: an tSeapáin',
    '🇰🇪' => 'bratach: an Chéinia',
    '🇰🇬' => 'bratach: an Chirgeastáin',
    '🇰🇭' => 'bratach: an Chambóid',
    '🇰🇮' => 'bratach: Ciribeas',
    '🇰🇲' => 'bratach: Oileáin Chomóra',
    '🇰🇳' => 'bratach: San Críostóir-Nimheas',
    '🇰🇵' => 'bratach: an Chóiré Thuaidh',
    '🇰🇷' => 'bratach: an Chóiré Theas',
    '🇰🇼' => 'bratach: Cuáit',
    '🇰🇾' => 'bratach: Oileáin Cayman',
    '🇰🇿' => 'bratach: an Chasacstáin',
    '🇱🇦' => 'bratach: Laos',
    '🇱🇧' => 'bratach: an Liobáin',
    '🇱🇨' => 'bratach: Saint Lucia',
    '🇱🇮' => 'bratach: Lichtinstéin',
    '🇱🇰' => 'bratach: Srí Lanca',
    '🇱🇷' => 'bratach: an Libéir',
    '🇱🇸' => 'bratach: Leosóta',
    '🇱🇹' => 'bratach: an Liotuáin',
    '🇱🇺' => 'bratach: Lucsamburg',
    '🇱🇻' => 'bratach: an Laitvia',
    '🇱🇾' => 'bratach: an Libia',
    '🇲🇦' => 'bratach: Maracó',
    '🇲🇨' => 'bratach: Monacó',
    '🇲🇩' => 'bratach: an Mholdóiv',
    '🇲🇪' => 'bratach: Montainéagró',
    '🇲🇫' => 'bratach: Saint-Martin',
    '🇲🇬' => 'bratach: Madagascar',
    '🇲🇭' => 'bratach: Oileáin Marshall',
    '🇲🇰' => 'bratach: an Mhacadóin Thuaidh',
    '🇲🇱' => 'bratach: Mailí',
    '🇲🇲' => 'bratach: Maenmar (Burma)',
    '🇲🇳' => 'bratach: an Mhongóil',
    '🇲🇴' => 'bratach: Sainréigiún Riaracháin Macao, Daonphoblacht na Síne',
    '🇲🇵' => 'bratach: Na hOileáin Mháirianacha Thuaidh',
    '🇲🇶' => 'bratach: Martinique',
    '🇲🇷' => 'bratach: an Mháratáin',
    '🇲🇸' => 'bratach: Montsarat',
    '🇲🇹' => 'bratach: Málta',
    '🇲🇺' => 'bratach: Oileán Mhuirís',
    '🇲🇻' => 'bratach: Oileáin Mhaildíve',
    '🇲🇼' => 'bratach: an Mhaláiv',
    '🇲🇽' => 'bratach: Meicsiceo',
    '🇲🇾' => 'bratach: an Mhalaeisia',
    '🇲🇿' => 'bratach: Mósaimbíc',
    '🇳🇦' => 'bratach: an Namaib',
    '🇳🇨' => 'bratach: an Nua-Chaladóin',
    '🇳🇪' => 'bratach: an Nígir',
    '🇳🇫' => 'bratach: Oileán Norfolk',
    '🇳🇬' => 'bratach: an Nigéir',
    '🇳🇮' => 'bratach: Nicearagua',
    '🇳🇱' => 'bratach: an Ísiltír',
    '🇳🇴' => 'bratach: an Iorua',
    '🇳🇵' => 'bratach: Neipeal',
    '🇳🇷' => 'bratach: Nárú',
    '🇳🇺' => 'bratach: Niue',
    '🇳🇿' => 'bratach: an Nua-Shéalainn',
    '🇴🇲' => 'bratach: Óman',
    '🇵🇦' => 'bratach: Panama',
    '🇵🇪' => 'bratach: Peiriú',
    '🇵🇫' => 'bratach: Polainéis na Fraince',
    '🇵🇬' => 'bratach: Nua-Ghuine Phapua',
    '🇵🇭' => 'bratach: Na hOileáin Fhilipíneacha',
    '🇵🇰' => 'bratach: an Phacastáin',
    '🇵🇱' => 'bratach: an Pholainn',
    '🇵🇲' => 'bratach: San Pierre agus Miquelon',
    '🇵🇳' => 'bratach: Oileáin Pitcairn',
    '🇵🇷' => 'bratach: Pórtó Ríce',
    '🇵🇸' => 'bratach: na Críocha Palaistíneacha',
    '🇵🇹' => 'bratach: an Phortaingéil',
    '🇵🇼' => 'bratach: Oileáin Palau',
    '🇵🇾' => 'bratach: Paragua',
    '🇶🇦' => 'bratach: Catar',
    '🇷🇪' => 'bratach: La Réunion',
    '🇷🇴' => 'bratach: an Rómáin',
    '🇷🇸' => 'bratach: an tSeirbia',
    '🇷🇺' => 'bratach: an Rúis',
    '🇷🇼' => 'bratach: Ruanda',
    '🇸🇦' => 'bratach: an Araib Shádach',
    '🇸🇧' => 'bratach: Oileáin Sholaimh',
    '🇸🇨' => 'bratach: na Séiséil',
    '🇸🇩' => 'bratach: an tSúdáin',
    '🇸🇪' => 'bratach: an tSualainn',
    '🇸🇬' => 'bratach: Singeapór',
    '🇸🇭' => 'bratach: San Héilin',
    '🇸🇮' => 'bratach: an tSlóivéin',
    '🇸🇯' => 'bratach: Svalbard agus Jan Mayen',
    '🇸🇰' => 'bratach: an tSlóvaic',
    '🇸🇱' => 'bratach: Siarra Leon',
    '🇸🇲' => 'bratach: San Mairíne',
    '🇸🇳' => 'bratach: an tSeineagáil',
    '🇸🇴' => 'bratach: an tSomáil',
    '🇸🇷' => 'bratach: Suranam',
    '🇸🇸' => 'bratach: an tSúdáin Theas',
    '🇸🇹' => 'bratach: São Tomé agus Príncipe',
    '🇸🇻' => 'bratach: An tSalvadóir',
    '🇸🇽' => 'bratach: Sint Maarten',
    '🇸🇾' => 'bratach: an tSiria',
    '🇸🇿' => 'bratach: Esuaitíní',
    '🇹🇦' => 'bratach: Tristan da Cunha',
    '🇹🇨' => 'bratach: Oileáin na dTurcach agus Caicos',
    '🇹🇩' => 'bratach: Sead',
    '🇹🇫' => 'bratach: Críocha Francacha Dheisceart an Domhain',
    '🇹🇬' => 'bratach: Tóga',
    '🇹🇭' => 'bratach: an Téalainn',
    '🇹🇯' => 'bratach: an Táidsíceastáin',
    '🇹🇰' => 'bratach: Tócalá',
    '🇹🇱' => 'bratach: Tíomór Thoir',
    '🇹🇲' => 'bratach: an Tuircméanastáin',
    '🇹🇳' => 'bratach: an Tuinéis',
    '🇹🇴' => 'bratach: Tonga',
    '🇹🇷' => 'bratach: an Tuirc',
    '🇹🇹' => 'bratach: Oileán na Tríonóide agus Tobága',
    '🇹🇻' => 'bratach: Túvalú',
    '🇹🇼' => 'bratach: an Téaváin',
    '🇹🇿' => 'bratach: an Tansáin',
    '🇺🇦' => 'bratach: an Úcráin',
    '🇺🇬' => 'bratach: Uganda',
    '🇺🇲' => 'bratach: Oileáin Imeallacha S.A.M.',
    '🇺🇳' => 'bratach: na Náisiúin Aontaithe',
    '🇺🇸' => 'bratach: Stáit Aontaithe Mheiriceá',
    '🇺🇾' => 'bratach: Uragua',
    '🇺🇿' => 'bratach: an Úisbéiceastáin',
    '🇻🇦' => 'bratach: Cathair na Vatacáine',
    '🇻🇨' => 'bratach: San Uinseann agus na Greanáidíní',
    '🇻🇪' => 'bratach: Veiniséala',
    '🇻🇬' => 'bratach: Oileáin Bhriotanacha na Maighdean',
    '🇻🇮' => 'bratach: Oileáin Mheiriceánacha na Maighdean',
    '🇻🇳' => 'bratach: Vítneam',
    '🇻🇺' => 'bratach: Vanuatú',
    '🇼🇫' => 'bratach: Vailís agus Futúna',
    '🇼🇸' => 'bratach: Samó',
    '🇽🇰' => 'bratach: an Chosaiv',
    '🇾🇪' => 'bratach: Éimin',
    '🇾🇹' => 'bratach: Mayotte',
    '🇿🇦' => 'bratach: an Afraic Theas',
    '🇿🇲' => 'bratach: an tSaimbia',
    '🇿🇼' => 'bratach: an tSiombáib',
    '🔟' => 'caipín eochrach: 10',
    '🏻' => 'ton cnis geal',
    '🏼' => 'ton cnis mheánach-gheal',
    '🏽' => 'ton cnis meánach',
    '🏾' => 'ton cnis meándorcha',
    '🏿' => 'ton cnis dorcha',
    '😀' => 'straoiseog ghealgháireach',
    '😃' => 'straoiseog ag gáire le súile móra',
    '😄' => 'straoiseog ag gáire le súile gealgháireacha uirthi',
    '😁' => 'straoiseog gháire le súile & miongháire',
    '😆' => 'straoiseog ag gáire le súile dúnta',
    '😅' => 'straoiseog ag gáire le fuarallas',
    '🤣' => 'sna trithí gáire',
    '😂' => 'straoiseog le deora áthais',
    '🙂' => 'straoiseog ag miongháire de bheagán',
    '🙃' => 'straoiseog bun os cionn',
    '🫠' => 'straoiseog atá ag leá',
    '😉' => 'straoiseog ag caochadh',
    '😊' => 'straoiseog aoibhiúil le súile le miongháire',
    '😇' => 'straoiseog ag gáire le luan',
    '🥰' => 'straoiseog ag miongháire le croíthe',
    '😍' => 'staoiseog ag gáire le croíthe ina súile',
    '🤩' => 'straoiseog faoi gheasa',
    '😘' => 'straoiseog ag séideadh póige',
    '😗' => 'straoiseog ag pógadh',
    '☺' => 'straoiseog ag miongháire',
    '😚' => 'straoiseog ag pógadh le súile dúnta',
    '😙' => 'straoiseog ag pógadh le súile le miongháire',
    '🥲' => 'straoiseog shásta le deora',
    '😋' => 'straoiseog ag baint taitneamh as bia blasta',
    '😛' => 'straoiseog le teanga amach',
    '😜' => 'straoiseog ag caochadh le teanga amach',
    '🤪' => 'straoiseog chraiceáilte',
    '😝' => 'straoiseog le teanga amach agus súile dúnta',
    '🤑' => 'straoiseog le hairgead ina béal',
    '🤗' => 'aghaidh ghealgháireach agus na lámha oscailte',
    '🤭' => 'straoiseog lena lámh ar a béal',
    '🫢' => 'straoiseog le súile oscailte agus lámh ar an mbéal',
    '🫣' => 'straoiseog le súil ag píceáil',
    '🤫' => 'straoiseog do do chur i do thost',
    '🤔' => 'straoiseog ag smaoineamh',
    '🫡' => 'straoiseog ag déanamh cúirtéise',
    '🤐' => 'straoiseog le béal zipeáilte',
    '🤨' => 'straoiseog le mala ardaithe',
    '😐' => 'straoiseog neodrach',
    '😑' => 'straoiseog le haghaidh fholamh',
    '😶' => 'straoiseog gan bhéal',
    '🫥' => 'straoiseog le líne phoncaithe',
    '😏' => 'straoiseog ag déanamh seitgháire',
    '😒' => 'straoiseog mhíshásta',
    '🙄' => 'straoiseog le súile ag casadh',
    '😬' => 'straoiseog le cár uirthi',
    '🤥' => 'straoiseog ag insint bréag',
    '🫨' => 'aghaidh croitheadh',
    '😌' => 'straoiseog a bhfuil faoiseamh uirthi',
    '😔' => 'straoiseog mhachnamhach',
    '😪' => 'straoiseog chodlatach',
    '🤤' => 'straoiseog ag prislíneacht',
    '😴' => 'straoiseog ina codladh',
    '🫩' => 'aghaidh le sprochaillí faoi na súile',
    '😷' => 'straoiseog le masc leighis',
    '🤒' => 'straoiseog le teirmiméadar',
    '🤕' => 'straoiseog le bindealán ar a ceann',
    '🤢' => 'straoiseog bhreoite',
    '🤮' => 'straoiseog ag cur amach',
    '🤧' => 'straoiseog ag sraotharthach',
    '🥵' => 'straoiseog the',
    '🥶' => 'straoiseog fhuar',
    '🥴' => 'straoiseog trína chéile',
    '😵' => 'aghaidh a bhfuil crosa curtha trína súile',
    '🤯' => 'straoiseog lena ceann ag pléasadh',
    '🤠' => 'straoiseog le hata buachaill bó',
    '🥳' => 'straoiseog chóisireachta',
    '🥸' => 'straiseog faoi bhréagriocht',
    '😎' => 'straoiseog ag gáire le spéaclaí gréine',
    '🤓' => 'straoiseog shaoithíneach',
    '🧐' => 'straoiseog le leathspéacla',
    '😕' => 'straoiseog mhearaithe',
    '🫤' => 'straoiseog le béal fiarthrasna',
    '😟' => 'straoiseog imníoch',
    '🙁' => 'straoiseog le mionghrainc uirthi',
    '☹' => 'straoiseog le grainc uirthi',
    '😮' => 'straoiseog le béal oscailte',
    '😯' => 'straoiseog ina tost',
    '😲' => 'straoiseog iontais',
    '😳' => 'straoiseog dhearg',
    '🫪' => 'aghaidh anchumtha',
    '🥺' => 'straoiseog impíoch',
    '🥹' => 'straoiseog ag coinneáil siar na deora',
    '😦' => 'straoiseag ghruama le béal oscailte',
    '😧' => 'straoiseog chráite',
    '😨' => 'straoiseog eaglach',
    '😰' => 'straoiseog fhaiteach le hallas',
    '😥' => 'straoiseog a bhfuil díoma ach le faoiseamh',
    '😢' => 'straoiseog chaointeach',
    '😭' => 'straoiseog ag caoineadh os ard',
    '😱' => 'straoiseog ag béiceach amach le heagla',
    '😖' => 'straoiseog le mearbhall uirthi',
    '😣' => 'straoiseog bhuanseasmhachta',
    '😞' => 'straoiseog dhíomách',
    '😓' => 'straoiseog in ísle brí le fuarallas',
    '😩' => 'straoiseog thraochta',
    '😫' => 'straoiseog thuirseach',
    '🥱' => 'aghaidh ag méanfach',
    '😤' => 'straoiseog le gal ón tsrón',
    '😡' => 'aghaidh fheargach',
    '😠' => 'straoiseog fheargach',
    '🤬' => 'straoiseog le siombailí ar an mbéal',
    '😈' => 'straoiseog ag gáire le hadharca',
    '👿' => 'straoiseog fheargach le hadharca',
    '💀' => 'blaosc',
    '☠' => 'blaosc agus croschnámah',
    '💩' => 'carn caca',
    '🤡' => 'fearr grinn',
    '👹' => 'fathach',
    '👺' => 'gruagach',
    '👻' => 'taibhse',
    '👽' => 'eachtrán',
    '👾' => 'arracht eachtráin',
    '🤖' => 'róbat',
    '😺' => 'cat ag miongháire',
    '😸' => 'straoiseog cait le súile le miongháire',
    '😹' => 'straoiseog cait le deora áthais',
    '😻' => 'straoiseog cait le croíthe ina súile',
    '😼' => 'straoiseog cait le miongháire dóite',
    '😽' => 'cat ag pógadh',
    '🙀' => 'cat tuirseach',
    '😿' => 'cat ag gol',
    '😾' => 'cat le puisín air',
    '🙈' => 'moncaí nach bhfeiceann aon olc',
    '🙉' => 'moncaí nach gcloiseann aon olc',
    '🙊' => 'moncaí nach ndeir aon olc',
    '💌' => 'litir ghrá',
    '💘' => 'croí agus saighead',
    '💝' => 'croí le ribín',
    '💖' => 'croí ag lonrú',
    '💗' => 'croí ag fás',
    '💓' => 'croí ag preabadh',
    '💞' => 'croíthe ag rothlú',
    '💕' => 'dhá chroí',
    '💟' => 'maisiúchán croí',
    '❣' => 'comhartha uaillbhreasa croí',
    '💔' => 'croíbhriste',
    '❤' => 'croí dearg',
    '🩷' => 'croí bándearg',
    '🧡' => 'croí flannbhuí',
    '💛' => 'croí buí',
    '💚' => 'croí glas',
    '💙' => 'croí gorm',
    '🩵' => 'croí bánghorm',
    '💜' => 'croí corcra',
    '🤎' => 'croí donn',
    '🖤' => 'croí dubh',
    '🩶' => 'croí liath',
    '🤍' => 'croí bán',
    '💋' => 'marc póg',
    '💯' => 'scór iomlán',
    '💢' => 'siombail feirge',
    '🫯' => 'scamall troda',
    '💥' => 'pléasc',
    '💫' => 'meadhrán',
    '💦' => 'broainíní allais',
    '💨' => 'rith',
    '🕳' => 'poll',
    '💬' => 'bolgán cainte',
    '🗨' => 'bolgán cainte ar chlé',
    '🗯' => 'bolgán feirge ar dheis',
    '💭' => 'bolgán smaointe',
    '💤' => 'ZZZ',
    '👋' => 'lámh ag croitheadh',
    '🤚' => 'cúl láimhe in airde',
    '🖐' => 'lámh le méara spréite',
    '✋' => 'lámh ardaithe',
    '🖖' => 'cuirtéis Bholcáin',
    '🫱' => 'lámh ar dheis',
    '🫲' => 'lámh ar chlé',
    '🫳' => 'lámh le bos faoi',
    '🫴' => 'lámh le bos suas',
    '🫷' => 'brú láimhe ar chlé',
    '🫸' => 'brú láimhe ar dheis',
    '👌' => 'lámh OK',
    '🤌' => 'méara fáiscthe le chéile',
    '🤏' => 'lámh ag pinseáil',
    '✌' => 'comhartha an bhua',
    '🤞' => 'méarsa trasna',
    '🫰' => 'lámh a bhfuil a corrmhéar agus a ordóg trasna ar a chéile',
    '🤟' => 'comhartha grá',
    '🤘' => 'siombail na n-adharc',
    '🤙' => 'an lámh cur glao orm',
    '👈' => 'cúl corrmhéire ag díriú ar chlé',
    '👉' => 'cúl corrmhéire ag díriú ar dheis',
    '👆' => 'cúl corrmhéire ag díriú suas',
    '🖕' => 'méar láir',
    '👇' => 'cúl corrmhéire ag díriú síos',
    '☝' => 'corrmhéar ag díriú suas',
    '🫵' => 'corrmhéar ag díriú ar an mbreathnóir',
    '👍' => 'ordóg suas',
    '👎' => 'ordóg síos',
    '✊' => 'dorn ardaithe',
    '👊' => 'dorn atá chugat',
    '🤛' => 'dorn ar chlé',
    '🤜' => 'dorn ar dheis',
    '👏' => 'bualadh bos',
    '🙌' => 'lámha ardaithe',
    '🫶' => 'lámha croí',
    '👐' => 'lámha ar oscailt',
    '🤲' => 'bosa le chéile',
    '🤝' => 'croitheadh lámh',
    '🙏' => 'lámha le chéile',
    '✍' => 'lámh scríbhneoireachta',
    '💅' => 'vearnais ingne',
    '🤳' => 'féinín',
    '💪' => 'bícéips lúbtha',
    '🦾' => 'lámh mheicniúil',
    '🦿' => 'cos mheicniúil',
    '🦵' => 'cos',
    '🦶' => 'troigh',
    '👂' => 'cluas',
    '🦻' => 'cluas le háis éisteachta',
    '👃' => 'srón',
    '🧠' => 'inchinn',
    '🫀' => 'croí anatamaíoch',
    '🫁' => 'scámhóga',
    '🦷' => 'fiacal',
    '🦴' => 'cnámh',
    '👀' => 'súile',
    '👁' => 'súil',
    '👅' => 'teanga amach',
    '👄' => 'béal',
    '🫦' => 'ag baint greim as an liopa',
    '👶' => 'babaí',
    '🧒' => 'leanbh',
    '👦' => 'buachaill',
    '👧' => 'cailín',
    '🧑' => 'aosach',
    '👱' => 'duine: gruaig fhionn',
    '👨' => 'fear',
    '🧔' => 'duine: feasóg',
    '👩' => 'bean',
    '🧓' => 'seanduine',
    '👴' => 'seanfhear',
    '👵' => 'seanbhean',
    '🙍' => 'duine le púic air',
    '🙎' => 'duine le puisín air',
    '🙅' => 'duine ag comharthú NÍL',
    '🙆' => 'duine ag comharthú OK',
    '💁' => 'duine lena lámh amach',
    '🙋' => 'duine lena lámh suas',
    '🧏' => 'duine bodhar',
    '🙇' => 'duine ag umhlú',
    '🤦' => 'duine lena aghaidh ina bhos',
    '🤷' => 'duine ag croitheadh a ghuaillí',
    '👮' => 'póilín',
    '🕵' => 'bleachtaire',
    '💂' => 'garda an rí',
    '🥷' => 'ninja',
    '👷' => 'tógálaí',
    '🫅' => 'duine le coróin air',
    '🤴' => 'prionsa',
    '👸' => 'banphrionsa',
    '👳' => 'duine le turban air',
    '👲' => 'fear le gua pi mao air',
    '🧕' => 'bean le caifirín',
    '🤵' => 'duine a bhfuil culaith dhinnéir air',
    '👰' => 'duine le caille air',
    '🤰' => 'bean ag iompar clainne',
    '🫃' => 'fear torrach',
    '🫄' => 'duine torrach',
    '🤱' => 'cothú cíche',
    '👼' => 'aingeal óg',
    '🎅' => 'Daidí na Nollag',
    '🤶' => 'Bean Uí Claus',
    '🦸' => 'sárlaoch',
    '🦹' => 'sárbhithiúnach',
    '🧙' => 'duine draíochta',
    '🧚' => 'síog',
    '🧛' => 'vaimpír',
    '🧜' => 'duine murúch',
    '🧝' => 'lucharachán',
    '🧞' => 'ginid',
    '🧟' => 'zombaí',
    '🧌' => 'troll',
    '🫈' => 'neach gruagach',
    '💆' => 'duine ag fáil suathaireachta',
    '💇' => 'duine ag fáil bearradh gruaige',
    '🚶' => 'coisí',
    '🧍' => 'duine ina sheasamh',
    '🧎' => 'duine ar a ghlúine',
    '🏃' => 'reathaí',
    '💃' => 'bean ag damhsa',
    '🕺' => 'fear ag damhsa',
    '🕴' => 'fear i gculaith gnó ar eadarbhuas',
    '👯' => 'daoine le cluasa coiníní orthu',
    '🧖' => 'duine i seomra gaile',
    '🧗' => 'duine ag dreapadóireacht',
    '🤺' => 'duine ag pionsóireacht',
    '🏇' => 'rásaíocht capall',
    '⛷' => 'sciálaí',
    '🏂' => 'clárscimeálaí sneachta',
    '🏌' => 'duine ag imirt gailf',
    '🏄' => 'duine ag surfáil',
    '🚣' => 'bád rámhaíochta',
    '🏊' => 'snámhaí',
    '⛹' => 'duine le liathróid',
    '🏋' => 'duine ag tógáil meachán',
    '🚴' => 'rothaí',
    '🚵' => 'rothaí sléibhe',
    '🤸' => 'rothchasadh',
    '🤼' => 'coraíocht',
    '🤽' => 'póló uisce',
    '🤾' => 'liathróid láimhe',
    '🤹' => 'lámhchleasaíocht',
    '🧘' => 'duine i suíomh na loiteoige',
    '🛀' => 'duine ag folcadh',
    '🛌' => 'duine sa leaba',
    '👭' => 'beirt bhan le chéile',
    '👫' => 'fear agus bean le chéile',
    '👬' => 'beirt fhear le chéile',
    '💏' => 'póg',
    '💑' => 'lanúin le croí',
    '🗣' => 'ceann ag caint',
    '👤' => 'scáthchruth cinn',
    '👥' => 'scáthchruth ceann',
    '🫂' => 'daoine ag tabhairt barróige',
    '👪' => 'teaghlach',
    '👣' => 'loirg choise',
    '🫆' => 'méarlorg',
    '🦰' => 'gruaig rua',
    '🦱' => 'gruaig chatach',
    '🦳' => 'gruaig bhán',
    '🦲' => 'maol',
    '🐵' => 'aghaidh moncaí',
    '🐒' => 'moncaí',
    '🦍' => 'goraille',
    '🦧' => 'órang-útan',
    '🐶' => 'aghaidh madra',
    '🐕' => 'madra',
    '🦮' => 'madra treorach',
    '🐩' => 'púdal',
    '🐺' => 'aghaidh mic tíre',
    '🦊' => 'aghaidh sionnaigh',
    '🦝' => 'racún',
    '🐱' => 'aghaidh cait',
    '🐈' => 'cat',
    '🦁' => 'aghaidh leoin',
    '🐯' => 'aghaidh tíogair',
    '🐅' => 'tíogar',
    '🐆' => 'liopard',
    '🐴' => 'aghaidh capaill',
    '🫎' => 'mús',
    '🫏' => 'asal',
    '🐎' => 'capall',
    '🦄' => 'aghaidh aonbheannaigh',
    '🦓' => 'séabra',
    '🦌' => 'fia',
    '🦬' => 'bíosún',
    '🐮' => 'aghaidh bó',
    '🐂' => 'damh',
    '🐃' => 'buabhall uisce',
    '🐄' => 'bó',
    '🐷' => 'aghaidh muice',
    '🐖' => 'muc',
    '🐗' => 'torc',
    '🐽' => 'srón muice',
    '🐏' => 'reithe',
    '🐑' => 'caora',
    '🐐' => 'gabhar',
    '🐪' => 'camall',
    '🐫' => 'camall le dhá dhronn',
    '🦙' => 'láma',
    '🦒' => 'sioráf',
    '🐘' => 'eilifint',
    '🦣' => 'mamat',
    '🦏' => 'srónbheannach',
    '🦛' => 'dobhareach',
    '🐭' => 'aghaidh luiche',
    '🐁' => 'luch',
    '🐀' => 'francach',
    '🐹' => 'aghaidh hamstair',
    '🐰' => 'aghaidh coinín',
    '🐇' => 'coinín',
    '🐿' => 'iora talún',
    '🦫' => 'béabhar',
    '🦔' => 'gráinneog',
    '🦇' => 'ialtóg',
    '🐻' => 'aghaidh béair',
    '🐨' => 'cóála',
    '🐼' => 'aghaidh panda',
    '🦥' => 'spadán',
    '🦦' => 'madra uisce',
    '🦨' => 'scúnc',
    '🦘' => 'cangarú',
    '🦡' => 'broc',
    '🐾' => 'lorg lapaí',
    '🦃' => 'turcaí',
    '🐔' => 'sicín',
    '🐓' => 'coileach',
    '🐣' => 'éan ag teacht as an ubh',
    '🐤' => 'gearrshicín',
    '🐥' => 'gearrshicín éadain',
    '🐦' => 'éan',
    '🐧' => 'piongain',
    '🕊' => 'colm',
    '🦅' => 'iolar',
    '🦆' => 'lacha',
    '🦢' => 'eala',
    '🦉' => 'ulchabhán',
    '🦤' => 'dódó',
    '🪶' => 'cleite',
    '🦩' => 'lasairéan',
    '🦚' => 'péacóg',
    '🦜' => 'pearóid',
    '🪽' => 'sciathán',
    '🪿' => 'gé',
    '🐸' => 'aghaidh froig',
    '🐊' => 'crogall',
    '🐢' => 'turtar',
    '🦎' => 'earc luachra',
    '🐍' => 'nathair',
    '🐲' => 'aghaidh dragain',
    '🐉' => 'dragan',
    '🦕' => 'sárapód',
    '🦖' => 'T-Rex',
    '🐳' => 'míol mór ag séideadh',
    '🐋' => 'míol mór',
    '🐬' => 'deilf',
    '🫍' => 'cráin dhubh',
    '🦭' => 'rón',
    '🐟' => 'iasc',
    '🐠' => 'iasc teochreasa',
    '🐡' => 'iasc torcánach',
    '🦈' => 'siorc',
    '🐙' => 'ochtapas',
    '🐚' => 'sliogán',
    '🪸' => 'coiréal',
    '🪼' => 'smugairle róin',
    '🦀' => 'portán',
    '🦞' => 'gliomach',
    '🦐' => 'ribe róibéis',
    '🦑' => 'scuid',
    '🦪' => 'oisre',
    '🐌' => 'seilide',
    '🦋' => 'féileacán',
    '🐛' => 'míol',
    '🐜' => 'seangán',
    '🐝' => 'beach mheala',
    '🪲' => 'ciaróg',
    '🐞' => 'bóín',
    '🦗' => 'criogar',
    '🪳' => 'blatóg choiteann',
    '🕷' => 'damhán alla',
    '🕸' => 'líon damháin alla',
    '🦂' => 'scairp',
    '🦟' => 'muiscít',
    '🪰' => 'cuileog',
    '🪱' => 'péist',
    '🦠' => 'miocrób',
    '💐' => 'crobhaing bláthanna',
    '🌸' => 'bláth silíní',
    '💮' => 'bláth bán',
    '🪷' => 'loiteog',
    '🏵' => 'róiséad',
    '🌹' => 'rós',
    '🥀' => 'bláth sleabhctha',
    '🌺' => 'roiseog',
    '🌻' => 'lus na gréine',
    '🌼' => 'bláthú',
    '🌷' => 'tiúilip',
    '🪻' => 'hiasaint',
    '🌱' => 'síolphlanda',
    '🪴' => 'planda pota',
    '🌲' => 'crann síorghlas',
    '🌳' => 'crann duillsilteach',
    '🌴' => 'pailm',
    '🌵' => 'cachtas',
    '🌾' => 'punann ríse',
    '🌿' => 'luibh',
    '☘' => 'seamróg',
    '🍀' => 'seamair Mhuire',
    '🍁' => 'duilleog mhailpe',
    '🍂' => 'duilleog thite',
    '🍃' => 'duilleog ag séideadh sa ghaoth',
    '🪹' => 'nead fholamh',
    '🪺' => 'nead le huibheacha',
    '🍄' => 'beacán',
    '🪾' => 'crann lom',
    '🍇' => 'caora fíniúna',
    '🍈' => 'mealbhacán',
    '🍉' => 'mealbhacán uisce',
    '🍊' => 'táinséirín',
    '🍋' => 'líomóid',
    '🍌' => 'banana',
    '🍍' => 'anann',
    '🥭' => 'mangó',
    '🍎' => 'úll dearg',
    '🍏' => 'úll glas',
    '🍐' => 'piorra',
    '🍑' => 'péitseog',
    '🍒' => 'silíní',
    '🍓' => 'sú talún',
    '🫐' => 'fraochán',
    '🥝' => 'cíobhaí',
    '🍅' => 'tráta',
    '🫒' => 'ológ',
    '🥥' => 'cnó cócó',
    '🥑' => 'abhacád',
    '🍆' => 'ubhthoradh',
    '🥔' => 'práta',
    '🥕' => 'meacan dearg',
    '🌽' => 'dias arbhair',
    '🌶' => 'píobar te',
    '🫑' => 'piobar',
    '🥒' => 'cúcamar',
    '🥬' => 'glasra duilleach',
    '🥦' => 'brocailí',
    '🧄' => 'gairleog',
    '🧅' => 'oiniún',
    '🥜' => 'pis talún',
    '🫘' => 'pónairí',
    '🌰' => 'cnó capaill',
    '🫚' => 'sinséar fréimhe',
    '🫛' => 'faighneog pise',
    '🫜' => 'glasra fréimhe',
    '🍞' => 'arán',
    '🥐' => 'croissant',
    '🥖' => 'baguette',
    '🫓' => 'arán leathan',
    '🥨' => 'preatsal',
    '🥯' => 'béigeal',
    '🥞' => 'pancóga',
    '🧇' => 'vaifeal',
    '🧀' => 'ding cháise',
    '🍖' => 'feoil le cnámh',
    '🍗' => 'cos sicín',
    '🥩' => 'gearradh feola',
    '🥓' => 'bagún',
    '🍔' => 'burgar',
    '🍟' => 'sceallóga',
    '🍕' => 'píotsa',
    '🌭' => 'brocaire te',
    '🥪' => 'ceapaire',
    '🌮' => 'taco',
    '🌯' => 'burrito',
    '🫔' => 'tamale',
    '🥙' => 'arán leathan stuáilte',
    '🧆' => 'falafel',
    '🥚' => 'ubh',
    '🍳' => 'cócaireacht',
    '🥘' => 'panna tanaí le bia',
    '🍲' => 'pota le bia',
    '🫕' => 'fondú',
    '🥣' => 'babhla le spúnóg',
    '🥗' => 'sailéad',
    '🍿' => 'grán rósta',
    '🧈' => 'im',
    '🧂' => 'salann',
    '🥫' => 'bia stánaithe',
    '🍱' => 'bosca bento',
    '🍘' => 'craicear ríse',
    '🍙' => 'liathróid ríse',
    '🍚' => 'rís chócaráilte',
    '🍛' => 'curaí agus rís',
    '🍜' => 'ramen',
    '🍝' => 'spaigití',
    '🍠' => 'práta milis',
    '🍢' => 'bia mara',
    '🍣' => 'sushi',
    '🍤' => 'séacla friochta',
    '🍥' => 'cáca éisc le guairneán',
    '🥮' => 'cáca gealaí',
    '🍡' => 'dango',
    '🥟' => 'domplagán',
    '🥠' => 'brioscán feasa',
    '🥡' => 'bosca beir leat',
    '🍦' => 'uachtar reoite coipthe',
    '🍧' => 'oighear bearrtha',
    '🍨' => 'uachtar reoite',
    '🍩' => 'taoschnó',
    '🍪' => 'briosca',
    '🎂' => 'cáca breithe',
    '🍰' => 'brioscarán',
    '🧁' => 'cístín cupa',
    '🥧' => 'pióg',
    '🍫' => 'seacláid',
    '🍬' => 'milseán',
    '🍭' => 'líreacán',
    '🍮' => 'custard',
    '🍯' => 'pota meala',
    '🍼' => 'buidéal babaí',
    '🥛' => 'gloine bainne',
    '☕' => 'deoch the',
    '🫖' => 'taephota',
    '🍵' => 'cupán gan chluas',
    '🍶' => 'sáicí',
    '🍾' => 'buidéal le corc ag popáil',
    '🍷' => 'gloine fíona',
    '🍸' => 'gloine mhanglaim',
    '🍹' => 'deoch thrópaiceach',
    '🍺' => 'pionta beorach',
    '🍻' => 'gloiní beorach',
    '🥂' => 'gloiní a ghligleáil',
    '🥃' => 'gloine',
    '🫗' => 'leacht a dhoirteadh',
    '🥤' => 'cupán le sop',
    '🧋' => 'tae bolgán',
    '🧃' => 'bosca dí',
    '🧉' => 'maité',
    '🧊' => 'ciúb oighir',
    '🥢' => 'cipíní itheacháin',
    '🍽' => 'forc agus scian le pláta',
    '🍴' => 'scian agus forc',
    '🥄' => 'spúnóg',
    '🔪' => 'scian',
    '🫙' => 'crúsca',
    '🏺' => 'amfara',
    '🌍' => 'cruinneog ag taispeáint na hEorpa agus na hAfraice',
    '🌎' => 'cruinneog ag taispeáint Chríocha Mheiriceá',
    '🌏' => 'cruinneog ag taispeáint na hÁsie agus na hAstráile',
    '🌐' => 'cruinneog le fadlínte',
    '🗺' => 'léarscáil an domhain',
    '🗾' => 'léarscáil na Seapáine',
    '🧭' => 'compás',
    '🏔' => 'sliabh le sneachta',
    '⛰' => 'sliabh',
    '🛘' => 'sciorradh talún',
    '🌋' => 'bolcán',
    '🗻' => 'Sliabh Fuji',
    '🏕' => 'ag campáil',
    '🏖' => 'trá le scáth gréine',
    '🏜' => 'fásach',
    '🏝' => 'oileán díthreibhe',
    '🏞' => 'páirc náisiúnta',
    '🏟' => 'staidiam',
    '🏛' => 'foirgneamh clasaiceach',
    '🏗' => 'foirgníocht',
    '🧱' => 'bríce',
    '🪨' => 'carraig',
    '🪵' => 'adhmad',
    '🛖' => 'bothán',
    '🏘' => 'tithe',
    '🏚' => 'teach tréigthe',
    '🏠' => 'teach',
    '🏡' => 'teach le gairdín',
    '🏢' => 'foirgneamh oifigí',
    '🏣' => 'oifig an phoist de chuid na Seapáine',
    '🏤' => 'oifig an phoist',
    '🏥' => 'ospidéal',
    '🏦' => 'banc',
    '🏨' => 'óstán',
    '🏩' => 'óstán grá',
    '🏪' => 'siopa áise',
    '🏫' => 'scoil',
    '🏬' => 'siopa ilrannach',
    '🏭' => 'monarcha',
    '🏯' => 'caisleán Seápánach',
    '🏰' => 'caisleán Eorpach',
    '💒' => 'bainis',
    '🗼' => 'Túr Thóiceo',
    '🗽' => 'Dealbh na Saoirse',
    '⛪' => 'séipéal',
    '🕌' => 'mosc',
    '🛕' => 'teampall hiondúch',
    '🕍' => 'sionagóg',
    '⛩' => 'scrín Sinteochais',
    '🕋' => 'cába',
    '⛲' => 'scairdeán',
    '⛺' => 'puball',
    '🌁' => 'ceomhar',
    '🌃' => 'oíche le réaltaí',
    '🏙' => 'dreach cathrach',
    '🌄' => 'éirí na gréine thar na sléibhte',
    '🌅' => 'éirí na gréine',
    '🌆' => 'dreach cathrach i dtitim na hoíche',
    '🌇' => 'dul faoi na gréine',
    '🌉' => 'droichead istoíche',
    '♨' => 'foinse the',
    '🎠' => 'capall an roithleagáin ró',
    '🛝' => 'sleamhnán súgartha',
    '🎡' => 'roth Ferris',
    '🎢' => 'rollchóstóir',
    '💈' => 'cuaille bearbóra',
    '🎪' => 'sorcas',
    '🚂' => 'inneall traenach',
    '🚃' => 'carráiste iarnróid',
    '🚄' => 'traein ardluais',
    '🚅' => 'traein philéir',
    '🚆' => 'traein',
    '🚇' => 'traein faoi thalamh',
    '🚈' => 'iarnród éadrom',
    '🚉' => 'stáisiún traenach',
    '🚊' => 'tram',
    '🚝' => 'aonráille',
    '🚞' => 'iarnród sléibhe',
    '🚋' => 'carráiste tram',
    '🚌' => 'bus',
    '🚍' => 'bus atá ag teacht',
    '🚎' => 'bus tralaí',
    '🚐' => 'mionbhus',
    '🚑' => 'otharcharr',
    '🚒' => 'inneall dóiteáin',
    '🚓' => 'carr gardaí',
    '🚔' => 'carr gardaí ag teacht',
    '🚕' => 'tacsaí',
    '🚖' => 'tacsaí ag teacht',
    '🚗' => 'gluaisteán',
    '🚘' => 'gluaisteán atá ag teacht',
    '🚙' => 'SUV',
    '🛻' => 'trucail oscailte',
    '🚚' => 'trucail seachadta',
    '🚛' => 'leoraí',
    '🚜' => 'tarracóir',
    '🏎' => 'carr rása',
    '🏍' => 'gluaisrothar',
    '🛵' => 'scútar',
    '🦽' => 'cathaoir rothaí láimhe',
    '🦼' => 'cathaoir rothaí mheicniúil',
    '🛺' => 'ricseá uathoibríoch',
    '🚲' => 'rothar',
    '🛴' => 'scútar gan inneall',
    '🛹' => 'clár scátála',
    '🛼' => 'scáta rothach',
    '🚏' => 'stad bus',
    '🛣' => 'mótarbhealach',
    '🛤' => 'iarnród',
    '🛢' => 'druma ola',
    '⛽' => 'pumpa breosla',
    '🛞' => 'roth',
    '🚨' => 'solas rothlach',
    '🚥' => 'soilse tráchta cothrománacha',
    '🚦' => 'soilse tráchta ceartingearacha',
    '🛑' => 'comhartha stop',
    '🚧' => 'suíomh tógála',
    '⚓' => 'ancaire',
    '🛟' => 'baoi fáinneach',
    '⛵' => 'bád seoil',
    '🛶' => 'canú',
    '🚤' => 'luasbhád',
    '🛳' => 'long paisinéirí',
    '⛴' => 'bád farantóireachta',
    '🛥' => 'mótarbhád',
    '🚢' => 'long',
    '✈' => 'eitleán',
    '🛩' => 'eitleán beag',
    '🛫' => 'imeacht eitleáin',
    '🛬' => 'teacht eitleáin',
    '🪂' => 'paraisiút',
    '💺' => 'suíochán',
    '🚁' => 'héileacaptar',
    '🚟' => 'iarnród crochta',
    '🚠' => 'cáblabhealach sléibhe',
    '🚡' => 'trambhealach aerga',
    '🛰' => 'satailít',
    '🚀' => 'roicéad',
    '🛸' => 'sásar eitilte',
    '🛎' => 'clog óstáin',
    '🧳' => 'málaí',
    '⌛' => 'orláiste críochnaithe',
    '⏳' => 'orláiste lán',
    '⌚' => 'uaireadóir',
    '⏰' => 'clog aláraim',
    '⏱' => 'stopuaireadóir',
    '⏲' => 'clog ama',
    '🕰' => 'clog matail',
    '🕛' => '12 an chloig',
    '🕧' => 'leathuair tar éis a dó dhéag',
    '🕐' => '1 an chloig',
    '🕜' => 'leathuair tar éis a haon',
    '🕑' => '2 an chloig',
    '🕝' => 'leathuair tar éis a dó',
    '🕒' => '3 an chloig',
    '🕞' => 'leathuair tar éis a trí',
    '🕓' => '4 an chloig',
    '🕟' => 'leathuair tar éis a ceathair',
    '🕔' => '5 an chloig',
    '🕠' => 'leathuair tar éis a cúig',
    '🕕' => '6 an chloig',
    '🕡' => 'leathuair tar éis a sé',
    '🕖' => '7 an chloig',
    '🕢' => 'leathuair tar éis a seacht',
    '🕗' => '8 an chloig',
    '🕣' => 'leathuair tar éis a hocht',
    '🕘' => '9 an chloig',
    '🕤' => 'leathuair tar éis a naoi',
    '🕙' => '10 an chloig',
    '🕥' => 'leathuair tar éis a deich',
    '🕚' => '11 an chloig',
    '🕦' => 'leathuair tar éis a haon déag',
    '🌑' => 'gealach úr',
    '🌒' => 'corrán gealaí atá ag líonadh',
    '🌓' => 'céad cheathrú na gealí',
    '🌔' => 'gealach scothlán atá ag líonadh',
    '🌕' => 'gealach lán',
    '🌖' => 'gealach scothlán atá ag caitheamh',
    '🌗' => 'ceathrú deireanach na gealaí',
    '🌘' => 'corrán gealaí atá ag caitheamh',
    '🌙' => 'corrán gealaí',
    '🌚' => 'gealach úr le haghaidh',
    '🌛' => 'céad cheathrú na gealaí le haghaidh',
    '🌜' => 'ceathrú deireanach na gealaí le haghaidh',
    '🌡' => 'teirmiméadar',
    '☀' => 'grian',
    '🌝' => 'aghaidh na gealaí láine',
    '🌞' => 'an ghrian le haghaidh',
    '🪐' => 'pláinéad le fáinne',
    '⭐' => 'réalta',
    '🌟' => 'réalta ag lonrú',
    '🌠' => 'réalta reatha',
    '🌌' => 'Bealach na Bó Finne',
    '☁' => 'scamall',
    '⛅' => 'grian taobh thiar de na scamaill',
    '⛈' => 'scamall le tintreach agus fearthainn',
    '🌤' => 'grian taobh thiar de scamall beag',
    '🌥' => 'grian taobh thiar de scamall mór',
    '🌦' => 'grian taobh thiar de scamall le fearthainn',
    '🌧' => 'scamall le fearthainn',
    '🌨' => 'scamall le sneachta',
    '🌩' => 'scamall le tintreach',
    '🌪' => 'tornádó',
    '🌫' => 'ceo',
    '🌬' => 'aghaidh na gaoithe',
    '🌀' => 'cioclón',
    '🌈' => 'bogha báistí',
    '🌂' => 'scáth báistí dúnta',
    '☂' => 'scáth báistí',
    '☔' => 'scáth báistí le braonta báistí',
    '⛱' => 'scáth báistí ar an talamh',
    '⚡' => 'ardvoltas',
    '❄' => 'calóg shneachta',
    '☃' => 'fear sneachta',
    '⛄' => 'fear sneachta gan sneachta',
    '☄' => 'cóiméad',
    '🔥' => 'tine',
    '💧' => 'braoinín',
    '🌊' => 'tonn san uisce',
    '🎃' => 'Seán na gealaí',
    '🎄' => 'crann Nollag',
    '🎆' => 'tine ealaíne',
    '🎇' => 'spréachaire',
    '🧨' => 'pléascóg thine',
    '✨' => 'drithlí',
    '🎈' => 'balún',
    '🎉' => 'pléiscín cóisire',
    '🎊' => 'meall coinfití',
    '🎋' => 'crann tanabata',
    '🎍' => 'maisiúchán péine',
    '🎎' => 'babóga Seapánacha',
    '🎏' => 'sraoilleán éisc',
    '🎐' => 'clingíní gaoithe',
    '🎑' => 'deasghnáth amhairc ar an ngealach',
    '🧧' => 'clúdach litreach dearg',
    '🎀' => 'ribín',
    '🎁' => 'bronntanas',
    '🎗' => 'ribín cuimhneacháin',
    '🎟' => 'ticéid cead isteach',
    '🎫' => 'ticéad',
    '🎖' => 'bonn míleata',
    '🏆' => 'corn',
    '🏅' => 'bonn spóirt',
    '🥇' => 'bonn óir',
    '🥈' => 'bonn airgid',
    '🥉' => 'bonn cré-umha',
    '⚽' => 'liathróid sacair',
    '⚾' => 'daorchluiche',
    '🥎' => 'bogliathróid',
    '🏀' => 'cispheil',
    '🏐' => 'eitpheil',
    '🏈' => 'peil Mheiriceánach',
    '🏉' => 'rugbaí',
    '🎾' => 'leadóg',
    '🥏' => 'diosca eitilte',
    '🎳' => 'babhláil',
    '🏏' => 'cruicéad',
    '🏑' => 'haca',
    '🏒' => 'haca oighir',
    '🥍' => 'crosógaíocht',
    '🏓' => 'ping pang',
    '🏸' => 'badmantan',
    '🥊' => 'lámhainn dornálaíochta',
    '🥋' => 'culaith ealaíne troda',
    '🥅' => 'cúl',
    '⛳' => 'brat i bpoll',
    '⛸' => 'scátáil oighir',
    '🎣' => 'slat iascaireachta',
    '🤿' => 'masc tumadóireachta',
    '🎽' => 'léine reatha',
    '🎿' => 'scíonna',
    '🛷' => 'sleamhnán',
    '🥌' => 'cloch churlála',
    '🎯' => 'buille díreach',
    '🪀' => 'yó-yó',
    '🪁' => 'eitleog',
    '🔫' => 'piostal',
    '🎱' => 'liathróid 8 púil',
    '🔮' => 'cruinneog chriostail',
    '🪄' => 'slat draíochta',
    '🎮' => 'cluiche ríomhaire',
    '🕹' => 'luamhán stiúrtha',
    '🎰' => 'meaisín sliotáin',
    '🎲' => 'dísle',
    '🧩' => 'mír mearaí',
    '🧸' => 'béirín',
    '🪅' => 'piñata',
    '🪩' => 'liathróid scátháin',
    '🪆' => 'bábóga Rúiseacha',
    '♠' => 'dath spéireata',
    '♥' => 'dath hairt',
    '♦' => 'dath muileata',
    '♣' => 'dath truif',
    '♟' => 'fichillín',
    '🃏' => 'fear na gcrúb',
    '🀄' => 'Dragan rua mah-jongg',
    '🎴' => 'cártaí imeartha bláthanna',
    '🎭' => 'na taibhealaíona',
    '🖼' => 'pictiúr i bhfráma',
    '🎨' => 'pailéad dathanna',
    '🧵' => 'snáth',
    '🪡' => 'snáthaid fuála',
    '🧶' => 'abhras',
    '🪢' => 'snaidhm',
    '👓' => 'spéaclaí',
    '🕶' => 'spéaclaí gréine',
    '🥽' => 'spéaclaí cosanta',
    '🥼' => 'cóta saotharlainne',
    '🦺' => 'veist sábháilteachta',
    '👔' => 'carbhat',
    '👕' => 't-léine',
    '👖' => 'brístí géine',
    '🧣' => 'scairf',
    '🧤' => 'lámhainní',
    '🧥' => 'cóta',
    '🧦' => 'stocaí',
    '👗' => 'gúna',
    '👘' => 'cimeonó',
    '🥻' => 'sáirí',
    '🩱' => 'culaith shnámha aonphíosa',
    '🩲' => 'culaith shnámha fir',
    '🩳' => 'brístí gearra',
    '👙' => 'bicíní',
    '👚' => 'éadaí mná',
    '🪭' => 'fean láimhe infhillte',
    '👛' => 'sparán',
    '👜' => 'mála láimhe',
    '👝' => 'mála',
    '🛍' => 'málaí siopadóireachta',
    '🎒' => 'mála scoile',
    '🩴' => 'cuarán',
    '👞' => 'bróg fir',
    '👟' => 'bróg reatha',
    '🥾' => 'buatais siúlóireachta',
    '🥿' => 'bróg chothrom',
    '👠' => 'sáil ard',
    '👡' => 'cuarán mná',
    '🩰' => 'bróga bailé',
    '👢' => 'buataisí mná',
    '🪮' => 'biorán gruaige',
    '👑' => 'coróin',
    '👒' => 'hata mná',
    '🎩' => 'hata ard',
    '🎓' => 'caipín bronnta céime',
    '🧢' => 'caipín speiceach',
    '🪖' => 'clogad míleata',
    '⛑' => 'clogad tarrthálaí',
    '📿' => 'paidrín urnaí',
    '💄' => 'béaldath',
    '💍' => 'fáinne',
    '💎' => 'geamchloch',
    '🔇' => 'callaire maothaithe',
    '🔈' => 'callaire íslithe',
    '🔉' => 'callaire fuaime ísle',
    '🔊' => 'callaire fuaime airde',
    '📢' => 'callaire',
    '📣' => 'meigeafón',
    '📯' => 'corn poist',
    '🔔' => 'cloigín',
    '🔕' => 'cloigín ar ceal',
    '🎼' => 'scór ceoil',
    '🎵' => 'nóta ceoil',
    '🎶' => 'nótaí ceoil',
    '🎙' => 'micreafón stiúideo',
    '🎚' => 'sleamhnán leibhéil',
    '🎛' => 'cnaipí rialaithe',
    '🎤' => 'micreafón',
    '🎧' => 'cluasáin',
    '📻' => 'raidió',
    '🎷' => 'sacsafón',
    '🎺' => 'trumpa',
    '🪊' => 'trombón',
    '🪗' => 'bosca ceoil',
    '🎸' => 'giotár',
    '🎹' => 'pianó',
    '🎻' => 'veidhlín',
    '🪕' => 'bainseó',
    '🥁' => 'druma',
    '🪘' => 'druma fada',
    '🪇' => 'maracas',
    '🪈' => 'fliúit',
    '🪉' => 'cláirseach',
    '📱' => 'guthán',
    '📲' => 'guthán le saighead',
    '☎' => 'teileafón',
    '📞' => 'glacadán fóin',
    '📟' => 'glaoire',
    '📠' => 'meaisín facsála',
    '🔋' => 'ceallra',
    '🪫' => 'cadhnra íseal',
    '🔌' => 'pluga leictreach',
    '💻' => 'ríomhaire glúine',
    '🖥' => 'ríomhaire deisce',
    '🖨' => 'printéir',
    '⌨' => 'méarchlár',
    '🖱' => 'luchóg ríomhaire',
    '🖲' => 'rianliathróid',
    '💽' => 'diosca ríomhaire',
    '💾' => 'diosca bog',
    '💿' => 'diosca optúil',
    '📀' => 'DVD',
    '🧮' => 'abacas',
    '🎥' => 'ceamara scannáin',
    '🎞' => 'frámaí scánnáin',
    '📽' => 'teilgeoir scannáin',
    '🎬' => 'clabaire',
    '📺' => 'teilifís',
    '📷' => 'ceamara',
    '📸' => 'ceamara le splanc',
    '📹' => 'físcheamara',
    '📼' => 'fístéip',
    '🔍' => 'gloine formhéadúcháin ar chlé',
    '🔎' => 'gloine formhéadúcháin ar dheis',
    '🕯' => 'coinneal',
    '💡' => 'bolgán solais',
    '🔦' => 'tóirse',
    '🏮' => 'laindéar dearg',
    '🪔' => 'lampa diya',
    '📔' => 'leabhar nótaí maisithe',
    '📕' => 'leabhar dúnta',
    '📖' => 'leabhar oscailte',
    '📗' => 'leabhar glas',
    '📘' => 'leabhar gorm',
    '📙' => 'leabhar flannbhuí',
    '📚' => 'leabhair',
    '📓' => 'leabhar nótaí',
    '📒' => 'mórleabhar',
    '📃' => 'leathanach le coirnín',
    '📜' => 'scrolla',
    '📄' => 'leathanach iompaithe suas',
    '📰' => 'nuachtán',
    '🗞' => 'nuachtán rollaithe suas',
    '📑' => 'clibeanna leabharmhairc',
    '🔖' => 'leabharmharc',
    '🏷' => 'lipéad',
    '🪙' => 'bonn',
    '💰' => 'mála airgead',
    '🪎' => 'cófra taisce',
    '💴' => 'nóta bainc yen',
    '💵' => 'nóta bainc dollair',
    '💶' => 'nóta bainc euro',
    '💷' => 'nóta bainc puint',
    '💸' => 'airgead ag eitilt',
    '💳' => 'cárta creidmheasa',
    '🧾' => 'admháil',
    '💹' => 'margadh ag méadú le yen',
    '✉' => 'clúdach litreach',
    '📧' => 'ríomhphost',
    '📨' => 'post isteach',
    '📩' => 'clúdach litreach le saighead',
    '📤' => 'trae an bhosca amach',
    '📥' => 'trae an bhosca isteach',
    '📦' => 'pacáiste',
    '📫' => 'bosca poist dúnta le bratach ardaithe',
    '📪' => 'bosca poist dúnta le bratach íslithe',
    '📬' => 'bosca poist oscailte le bratach ardaithe',
    '📭' => 'bosca poist oscailte le bratach íslithe',
    '📮' => 'bosca poist',
    '🗳' => 'bosca ballóide le ballóid',
    '✏' => 'peann luaidhe',
    '✒' => 'gob dubh',
    '🖋' => 'peann tobair',
    '🖊' => 'peann',
    '🖌' => 'scuab phéinteála',
    '🖍' => 'crián',
    '📝' => 'meabhrán',
    '💼' => 'mála cáipéisí',
    '📁' => 'fillteán comhad',
    '📂' => 'fillteán oscailte',
    '🗂' => 'roinnteoirí innéacs cárta',
    '📅' => 'féilire',
    '📆' => 'féilire instróicthe',
    '🗒' => 'ceap nótaí bíseach',
    '🗓' => 'féilire bíseach',
    '📇' => 'innéacs cártaí',
    '📈' => 'cairt mhéadaitheach',
    '📉' => 'cairt íslitheach',
    '📊' => 'barrachairt',
    '📋' => 'gearrthaisce',
    '📌' => 'tacóid ordóige',
    '📍' => 'tacóid chruinn',
    '📎' => 'fáiscín páipéir',
    '🖇' => 'fáiscíní páipéir nasctha',
    '📏' => 'rialóir díreach',
    '📐' => 'rialóir triantánach',
    '✂' => 'siosúr',
    '🗃' => 'bosca comhad cárta',
    '🗄' => 'caibinéad comhad',
    '🗑' => 'bosca bruscair',
    '🔒' => 'glasáilte',
    '🔓' => 'neamhghlasáilte',
    '🔏' => 'glasáilte le peann',
    '🔐' => 'glasáilte le heochair',
    '🔑' => 'eochair',
    '🗝' => 'sean-eochair',
    '🔨' => 'casúr',
    '🪓' => 'tua',
    '⛏' => 'piocóid',
    '⚒' => 'casúr agus piocóid',
    '🛠' => 'casúr agus rinse',
    '🗡' => 'miodóg',
    '⚔' => 'claimhte crosáilte',
    '💣' => 'buama',
    '🪃' => 'búmarang',
    '🏹' => 'saighead is bogha',
    '🛡' => 'sciath',
    '🪚' => 'sábh cearpantóireachta',
    '🔧' => 'rinse',
    '🪛' => 'scriúire',
    '🔩' => 'cnó agus bolta',
    '⚙' => 'giar',
    '🗜' => 'clampa',
    '⚖' => 'scála',
    '🦯' => 'cána bán',
    '🔗' => 'nasc',
    '⛓' => 'slabhraí',
    '🪝' => 'crúca',
    '🧰' => 'bosca uirlisí',
    '🧲' => 'maighnéad',
    '🪜' => 'dréimire',
    '🪏' => 'sluasaid',
    '⚗' => 'leamóg',
    '🧪' => 'triaileadán',
    '🧫' => 'mias Petri',
    '🧬' => 'ADN',
    '🔬' => 'micreascóp',
    '🔭' => 'teileascóp',
    '📡' => 'aeróg satailíte',
    '💉' => 'instealladh',
    '🩸' => 'braoinín fola',
    '💊' => 'piolla',
    '🩹' => 'bindealán greamaitheach',
    '🩼' => 'maide croise',
    '🩺' => 'steiteascóp',
    '🩻' => 'x-gha',
    '🚪' => 'doras',
    '🛗' => 'ardaitheoir',
    '🪞' => 'scáthán',
    '🪟' => 'fuinneog',
    '🛏' => 'leaba',
    '🛋' => 'tolg agus lampa',
    '🪑' => 'cathaoir',
    '🚽' => 'leithreas',
    '🪠' => 'suncaire',
    '🚿' => 'cithfholcadán',
    '🛁' => 'folcadán',
    '🪤' => 'gaiste luchóg',
    '🪒' => 'rásúr',
    '🧴' => 'buidéal lóise',
    '🧷' => 'biorán dúnta',
    '🧹' => 'scruab',
    '🧺' => 'ciseán',
    '🧻' => 'rolla páipéir',
    '🪣' => 'buicéad',
    '🧼' => 'sópa',
    '🫧' => 'boilgeoga',
    '🪥' => 'scuab fiacla',
    '🧽' => 'spúinse',
    '🧯' => 'múchtóir tine',
    '🛒' => 'tralaí',
    '🚬' => 'toitín',
    '⚰' => 'cónra',
    '🪦' => 'leac uaighe',
    '⚱' => 'próca',
    '🧿' => 'briocht nazar',
    '🪬' => 'hamsa',
    '🗿' => 'dealbh Oileán na Cásca',
    '🪧' => 'plaiceard',
    '🪪' => 'cárta aitheantais',
    '🏧' => 'comhartha ATM',
    '🚮' => 'comhartha bosca bruscair',
    '🚰' => 'uisce inólta',
    '♿' => 'bealach isteach do chathaoireacha rothaí',
    '🚹' => 'Fir',
    '🚺' => 'Mná',
    '🚻' => 'comhartha na leithreas',
    '🚼' => 'comhartha linbh',
    '🚾' => 'na leithris',
    '🛂' => 'rialú pasanna',
    '🛃' => 'custaim',
    '🛄' => 'limistéar bagáiste',
    '🛅' => 'bagáiste',
    '⚠' => 'comhartha rabhaidh',
    '🚸' => 'páistí ag trasnú',
    '⛔' => 'níl cead dul isteach',
    '🚫' => 'cosc ar dhul isteach',
    '🚳' => 'cosc ar rothair',
    '🚭' => 'cosc ar thobac',
    '🚯' => 'ná caitear bruscar',
    '🚱' => 'uisce nach bhfuil le hól',
    '🚷' => 'cosc ar choisithe',
    '📵' => 'cosc ar fhóin phóca',
    '🔞' => 'os cionn 18',
    '☢' => 'radaighníomhach',
    '☣' => 'bithghuais',
    '⬆' => 'saighead suas',
    '↗' => 'saighead suas ar dheis',
    '➡' => 'saighead ar dheis',
    '↘' => 'saighead síos ar dheis',
    '⬇' => 'saighead síos',
    '↙' => 'saighead síos ar chlé',
    '⬅' => 'saighead ar chlé',
    '↖' => 'saighead suas ar chlé',
    '↕' => 'saighead suas-síos',
    '↔' => 'saighead chlé-dheas',
    '↩' => 'saighead dheas ag dul ar chlé',
    '↪' => 'saighead chlé ag dul ar dheis',
    '⤴' => 'saighead dheas ag dul suas',
    '⤵' => 'saighead dheas ag dul síos',
    '🔃' => 'saigheada ingéaracha ar deiseal',
    '🔄' => 'cnaipe saighead tuathail',
    '🔙' => 'saighead SIAR',
    '🔚' => 'saighead DEIRIDH',
    '🔛' => 'saighead AR SIÚL!',
    '🔜' => 'saighead GO LUATH',
    '🔝' => 'saighead BHAIRR',
    '🛐' => 'teach adhartha',
    '⚛' => 'siombail adaimh',
    '🕉' => 'om',
    '✡' => 'Réalta Dháiví',
    '☸' => 'roth dharma',
    '☯' => 'yin yang',
    '✝' => 'cros laidineach',
    '☦' => 'cros ortadocsach',
    '☪' => 'réalta agus corrán',
    '☮' => 'siombail síochána',
    '🕎' => 'meanóra',
    '🔯' => 'réalta shé-rinneach le ponc sa lár',
    '🪯' => 'khanda',
    '♈' => 'an Reithe',
    '♉' => 'an Tarbh',
    '♊' => 'an Cúpla',
    '♋' => 'an Portán',
    '♌' => 'an Leon',
    '♍' => 'an Mhaighdean',
    '♎' => 'an Mheá',
    '♏' => 'an Scairp',
    '♐' => 'an Saighdeoir',
    '♑' => 'an Gabhar',
    '♒' => 'Iompróir an Uisce',
    '♓' => 'na hÉisc',
    '⛎' => 'Ophiuchus',
    '🔀' => 'saighead traiceanna ar suaitheadh',
    '🔁' => 'cnaipe athsheanma',
    '🔂' => 'cnaipe athsheanma uair amháin',
    '▶' => 'cnaipe seinnte',
    '⏩' => 'cnaipe mearchasaidh',
    '⏭' => 'cnaipe don chéad amhrán eile',
    '⏯' => 'cnaipe seinnte nó sosa',
    '◀' => 'cnaipe siar',
    '⏪' => 'cnaipe siar ar luas',
    '⏮' => 'cnaipe don amhrán deireanach',
    '🔼' => 'cnaipe suas',
    '⏫' => 'cnaipe suas ar luas',
    '🔽' => 'cnaipe síos',
    '⏬' => 'cnaipe síos ar luas',
    '⏸' => 'cnaipe sosa',
    '⏹' => 'cnaipe stop',
    '⏺' => 'cnaipe taifeadta',
    '⏏' => 'cnaipe díchuir',
    '🎦' => 'pictiúrlann',
    '🔅' => 'cnaipe íslithe',
    '🔆' => 'cnaipe gile',
    '📶' => 'comharthaí fóin',
    '🛜' => 'gan sreang',
    '📳' => 'mód creatha',
    '📴' => 'fón casta as',
    '♀' => 'comhartha baineann',
    '♂' => 'comhartha fireann',
    '⚧' => 'siombail thrasinsnce',
    '✖' => 'comhartha iolrúcháin',
    '➕' => 'móide',
    '➖' => 'lúide',
    '➗' => 'roinn',
    '🟰' => 'comhartha cothrom le',
    '♾' => 'éigríoch',
    '‼' => 'comhartha uaillbhreasa dúbailte',
    '⁉' => 'comhartha ceiste agus uaillbhreasa',
    '❓' => 'comhartha ceiste dearg',
    '❔' => 'comhartha ceiste bán',
    '❕' => 'comhartha uailbhresa bán',
    '❗' => 'comhartha uaillbhreasa dearg',
    '〰' => 'dais chorrach',
    '💱' => 'malartán airgeadra',
    '💲' => 'comhartha dollair trom',
    '⚕' => 'siombail leighis',
    '♻' => 'siombail athchúrsála',
    '⚜' => 'fleur-de-lis',
    '🔱' => 'comhartha trírinne',
    '📛' => 'suaitheantas ainm',
    '🔰' => 'siombal Sheapánach le haghaidh tosaitheoirí',
    '⭕' => 'ciorcal dearg toll',
    '✅' => 'cnaipe ticmhairc',
    '☑' => 'bosca ballóide le tic',
    '✔' => 'ticmharc',
    '❌' => 'crosmharc',
    '❎' => 'cnaipe crosmhairc',
    '➰' => 'lúb chatach',
    '➿' => 'lúb dhúbailte chatach',
    '〽' => 'cuid de mharc ailtéarnaithe',
    '✳' => 'réiltín le hocht spóca',
    '✴' => 'réalta le hocht bpointe',
    '❇' => 'gealán',
    '™' => 'comhartha trádmhairc',
    '🫟' => 'stealladh',
    '🔠' => 'ceannlitreacha Laidineacha ionchuir',
    '🔡' => 'mionlitreacha Laidineacha ionchuir',
    '🔢' => 'uimhreacha ionchuir',
    '🔣' => 'siombailí ionchuir',
    '🔤' => 'litreacha Laidineacha ionchuir',
    '🅰' => 'fuilghrúpa A',
    '🆎' => 'fuilghrúpa AB',
    '🅱' => 'fuilghrúpa B',
    '🆑' => 'CL',
    '🆒' => 'cnaipe CÚLÁILTE',
    '🆓' => 'cnaipe SAOR IN AISCE',
    'ℹ' => 'foinse eolais',
    '🆔' => 'cnaipe aitheantais',
    'Ⓜ' => 'litir M le ciorcal timpeall air',
    '🆕' => 'cnaipe NUA',
    '🆖' => 'NG',
    '🅾' => 'fuilghrúpa O',
    '🆗' => 'cnaipe OK',
    '🅿' => 'cnaipe P',
    '🆘' => 'SOS',
    '🆙' => 'cnaipe THUAS!',
    '🆚' => 'cnaipe VS',
    '🈁' => 'cnaipe Seapáinise "anseo"',
    '🈂' => 'cnaipe Seapáinise "táille seirbhíse"',
    '🈷' => 'cnaipe Seapáinise "méid míosúil"',
    '🈶' => 'cnaipe Seapáinise "nach bhfuil saor in aisce"',
    '🈯' => 'cnaipe Seapáinise "coinnithe"',
    '🉐' => 'cnaipe Seapáinise "sladmhargadh"',
    '🈹' => 'cnaipe Seapáinise "lascaine"',
    '🈚' => 'cnáipe Seapáinise "saor in aisce"',
    '🈲' => 'cnaipe Seapáinise "cosc"',
    '🉑' => 'cnaipe Seapáinise "inghlactha"',
    '🈸' => 'cnaipe Seapáinise "iarratas"',
    '🈴' => 'cnaipe Seapáinise "pasmharc"',
    '🈳' => 'cnaipe Seapáinise "folúntas"',
    '㊗' => 'cnaipe Seapáinise "comhghairdeas"',
    '㊙' => 'cnaipe Seapáinise "rún"',
    '🈺' => 'cnaipe Seapáinise "oscailte le haghaidh gnó"',
    '🈵' => 'cnaipe Seapáinise "níl aon fholúntas"',
    '🔴' => 'ciorcal dearg',
    '🟠' => 'ciorcal flannbhuí',
    '🟡' => 'ciorcal buí',
    '🟢' => 'ciorcal glas',
    '🔵' => 'ciorcal gorm',
    '🟣' => 'ciorcal corcra',
    '🟤' => 'ciorcal donn',
    '⚫' => 'ciorcal dubh',
    '⚪' => 'ciorcal bán',
    '🟥' => 'cearnóg dhearg',
    '🟧' => 'cearnóg fhlannbhuí',
    '🟨' => 'cearnóg bhuí',
    '🟩' => 'cearnóg ghlas',
    '🟦' => 'cearnóg ghorm',
    '🟪' => 'cearnóg chorcra',
    '🟫' => 'cearnóg dhonn',
    '⬛' => 'cearnóg mhór dhubh',
    '⬜' => 'cearnóg mhór bhán',
    '◼' => 'meánchearnóg dhubh',
    '◻' => 'meánchearnóg bhán',
    '◾' => 'meánchearnóg bheag dhubh',
    '◽' => 'meánchearnóg bheag bhán',
    '▪' => 'cearnóg dhubh bheag',
    '▫' => 'cearnóg bhán bheag',
    '🔶' => 'muileata mór flannbhuí',
    '🔷' => 'muileata mór gorm',
    '🔸' => 'muileata beag flannbhuí',
    '🔹' => 'muileata beag gorm',
    '🔺' => 'triantán dearg suas',
    '🔻' => 'triantán dearg síos',
    '💠' => 'muileata le ponc',
    '🔘' => 'cnaipe raidió',
    '🔳' => 'cearnóg bhán',
    '🔲' => 'cearnóg dhubh',
    '🏁' => 'bratach chríche',
    '🚩' => 'bratach thriantánach',
    '🎌' => 'bratacha trasnaithe',
    '🏴' => 'bratach dhubh',
    '🏳' => 'bratach bhán',
];
