<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'poljub: odrasla oseba, odrasla oseba, svetel ten kože in srednje svetel ten kože',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'poljub: odrasla oseba, odrasla oseba, svetel ten kože in srednji ten kože',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'poljub: odrasla oseba, odrasla oseba, svetel ten kože in srednje temen ten kože',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'poljub: odrasla oseba, odrasla oseba, svetel ten kože in temen ten kože',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'poljub: odrasla oseba, odrasla oseba, srednje svetel ten kože in svetel ten kože',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'poljub: odrasla oseba, odrasla oseba, srednje svetel ten kože in srednji ten kože',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'poljub: odrasla oseba, odrasla oseba, srednje svetel ten kože in srednje temen ten kože',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'poljub: odrasla oseba, odrasla oseba, srednje svetel ten kože in temen ten kože',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'poljub: odrasla oseba, odrasla oseba, srednji ten kože in svetel ten kože',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'poljub: odrasla oseba, odrasla oseba, srednji ten kože in srednje svetel ten kože',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'poljub: odrasla oseba, odrasla oseba, srednji ten kože in srednje temen ten kože',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'poljub: odrasla oseba, odrasla oseba, srednji ten kože in temen ten kože',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'poljub: odrasla oseba, odrasla oseba, srednje temen ten kože in svetel ten kože',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'poljub: odrasla oseba, odrasla oseba, srednje temen ten kože in srednje svetel ten kože',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'poljub: odrasla oseba, odrasla oseba, srednje temen ten kože in srednji ten kože',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'poljub: odrasla oseba, odrasla oseba, srednje temen ten kože in temen ten kože',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'poljub: odrasla oseba, odrasla oseba, temen ten kože in svetel ten kože',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'poljub: odrasla oseba, odrasla oseba, temen ten kože in srednje svetel ten kože',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'poljub: odrasla oseba, odrasla oseba, temen ten kože in srednji ten kože',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'poljub: odrasla oseba, odrasla oseba, temen ten kože in srednje temen ten kože',
    '👨🏻‍❤‍💋‍👨🏻' => 'poljub: moški, moški in svetel ten kože',
    '👨🏻‍❤‍💋‍👨🏼' => 'poljub: moški, moški, svetel ten kože in srednje svetel ten kože',
    '👨🏻‍❤‍💋‍👨🏽' => 'poljub: moški, moški, svetel ten kože in srednji ten kože',
    '👨🏻‍❤‍💋‍👨🏾' => 'poljub: moški, moški, svetel ten kože in srednje temen ten kože',
    '👨🏻‍❤‍💋‍👨🏿' => 'poljub: moški, moški, svetel ten kože in temen ten kože',
    '👨🏼‍❤‍💋‍👨🏻' => 'poljub: moški, moški, srednje svetel ten kože in svetel ten kože',
    '👨🏼‍❤‍💋‍👨🏼' => 'poljub: moški, moški in srednje svetel ten kože',
    '👨🏼‍❤‍💋‍👨🏽' => 'poljub: moški, moški, srednje svetel ten kože in srednji ten kože',
    '👨🏼‍❤‍💋‍👨🏾' => 'poljub: moški, moški, srednje svetel ten kože in srednje temen ten kože',
    '👨🏼‍❤‍💋‍👨🏿' => 'poljub: moški, moški, srednje svetel ten kože in temen ten kože',
    '👨🏽‍❤‍💋‍👨🏻' => 'poljub: moški, moški, srednji ten kože in svetel ten kože',
    '👨🏽‍❤‍💋‍👨🏼' => 'poljub: moški, moški, srednji ten kože in srednje svetel ten kože',
    '👨🏽‍❤‍💋‍👨🏽' => 'poljub: moški, moški in srednji ten kože',
    '👨🏽‍❤‍💋‍👨🏾' => 'poljub: moški, moški, srednji ten kože in srednje temen ten kože',
    '👨🏽‍❤‍💋‍👨🏿' => 'poljub: moški, moški, srednji ten kože in temen ten kože',
    '👨🏾‍❤‍💋‍👨🏻' => 'poljub: moški, moški, srednje temen ten kože in svetel ten kože',
    '👨🏾‍❤‍💋‍👨🏼' => 'poljub: moški, moški, srednje temen ten kože in srednje svetel ten kože',
    '👨🏾‍❤‍💋‍👨🏽' => 'poljub: moški, moški, srednje temen ten kože in srednji ten kože',
    '👨🏾‍❤‍💋‍👨🏾' => 'poljub: moški, moški in srednje temen ten kože',
    '👨🏾‍❤‍💋‍👨🏿' => 'poljub: moški, moški, srednje temen ten kože in temen ten kože',
    '👨🏿‍❤‍💋‍👨🏻' => 'poljub: moški, moški, temen ten kože in svetel ten kože',
    '👨🏿‍❤‍💋‍👨🏼' => 'poljub: moški, moški, temen ten kože in srednje svetel ten kože',
    '👨🏿‍❤‍💋‍👨🏽' => 'poljub: moški, moški, temen ten kože in srednji ten kože',
    '👨🏿‍❤‍💋‍👨🏾' => 'poljub: moški, moški, temen ten kože in srednje temen ten kože',
    '👨🏿‍❤‍💋‍👨🏿' => 'poljub: moški, moški in temen ten kože',
    '👩🏻‍❤‍💋‍👨🏻' => 'poljub: ženska, moški in svetel ten kože',
    '👩🏻‍❤‍💋‍👨🏼' => 'poljub: ženska, moški, svetel ten kože in srednje svetel ten kože',
    '👩🏻‍❤‍💋‍👨🏽' => 'poljub: ženska, moški, svetel ten kože in srednji ten kože',
    '👩🏻‍❤‍💋‍👨🏾' => 'poljub: ženska, moški, svetel ten kože in srednje temen ten kože',
    '👩🏻‍❤‍💋‍👨🏿' => 'poljub: ženska, moški, svetel ten kože in temen ten kože',
    '👩🏼‍❤‍💋‍👨🏻' => 'poljub: ženska, moški, srednje svetel ten kože in svetel ten kože',
    '👩🏼‍❤‍💋‍👨🏼' => 'poljub: ženska, moški in srednje svetel ten kože',
    '👩🏼‍❤‍💋‍👨🏽' => 'poljub: ženska, moški, srednje svetel ten kože in srednji ten kože',
    '👩🏼‍❤‍💋‍👨🏾' => 'poljub: ženska, moški, srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍❤‍💋‍👨🏿' => 'poljub: ženska, moški, srednje svetel ten kože in temen ten kože',
    '👩🏽‍❤‍💋‍👨🏻' => 'poljub: ženska, moški, srednji ten kože in svetel ten kože',
    '👩🏽‍❤‍💋‍👨🏼' => 'poljub: ženska, moški, srednji ten kože in srednje svetel ten kože',
    '👩🏽‍❤‍💋‍👨🏽' => 'poljub: ženska, moški in srednji ten kože',
    '👩🏽‍❤‍💋‍👨🏾' => 'poljub: ženska, moški, srednji ten kože in srednje temen ten kože',
    '👩🏽‍❤‍💋‍👨🏿' => 'poljub: ženska, moški, srednji ten kože in temen ten kože',
    '👩🏾‍❤‍💋‍👨🏻' => 'poljub: ženska, moški, srednje temen ten kože in svetel ten kože',
    '👩🏾‍❤‍💋‍👨🏼' => 'poljub: ženska, moški, srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍❤‍💋‍👨🏽' => 'poljub: ženska, moški, srednje temen ten kože in srednji ten kože',
    '👩🏾‍❤‍💋‍👨🏾' => 'poljub: ženska, moški in srednje temen ten kože',
    '👩🏾‍❤‍💋‍👨🏿' => 'poljub: ženska, moški, srednje temen ten kože in temen ten kože',
    '👩🏿‍❤‍💋‍👨🏻' => 'poljub: ženska, moški, temen ten kože in svetel ten kože',
    '👩🏿‍❤‍💋‍👨🏼' => 'poljub: ženska, moški, temen ten kože in srednje svetel ten kože',
    '👩🏿‍❤‍💋‍👨🏽' => 'poljub: ženska, moški, temen ten kože in srednji ten kože',
    '👩🏿‍❤‍💋‍👨🏾' => 'poljub: ženska, moški, temen ten kože in srednje temen ten kože',
    '👩🏿‍❤‍💋‍👨🏿' => 'poljub: ženska, moški in temen ten kože',
    '👩🏻‍❤‍💋‍👩🏻' => 'poljub: ženska, ženska in svetel ten kože',
    '👩🏻‍❤‍💋‍👩🏼' => 'poljub: ženska, ženska, svetel ten kože in srednje svetel ten kože',
    '👩🏻‍❤‍💋‍👩🏽' => 'poljub: ženska, ženska, svetel ten kože in srednji ten kože',
    '👩🏻‍❤‍💋‍👩🏾' => 'poljub: ženska, ženska, svetel ten kože in srednje temen ten kože',
    '👩🏻‍❤‍💋‍👩🏿' => 'poljub: ženska, ženska, svetel ten kože in temen ten kože',
    '👩🏼‍❤‍💋‍👩🏻' => 'poljub: ženska, ženska, srednje svetel ten kože in svetel ten kože',
    '👩🏼‍❤‍💋‍👩🏼' => 'poljub: ženska, ženska in srednje svetel ten kože',
    '👩🏼‍❤‍💋‍👩🏽' => 'poljub: ženska, ženska, srednje svetel ten kože in srednji ten kože',
    '👩🏼‍❤‍💋‍👩🏾' => 'poljub: ženska, ženska, srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍❤‍💋‍👩🏿' => 'poljub: ženska, ženska, srednje svetel ten kože in temen ten kože',
    '👩🏽‍❤‍💋‍👩🏻' => 'poljub: ženska, ženska, srednji ten kože in svetel ten kože',
    '👩🏽‍❤‍💋‍👩🏼' => 'poljub: ženska, ženska, srednji ten kože in srednje svetel ten kože',
    '👩🏽‍❤‍💋‍👩🏽' => 'poljub: ženska, ženska in srednji ten kože',
    '👩🏽‍❤‍💋‍👩🏾' => 'poljub: ženska, ženska, srednji ten kože in srednje temen ten kože',
    '👩🏽‍❤‍💋‍👩🏿' => 'poljub: ženska, ženska, srednji ten kože in temen ten kože',
    '👩🏾‍❤‍💋‍👩🏻' => 'poljub: ženska, ženska, srednje temen ten kože in svetel ten kože',
    '👩🏾‍❤‍💋‍👩🏼' => 'poljub: ženska, ženska, srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍❤‍💋‍👩🏽' => 'poljub: ženska, ženska, srednje temen ten kože in srednji ten kože',
    '👩🏾‍❤‍💋‍👩🏾' => 'poljub: ženska, ženska in srednje temen ten kože',
    '👩🏾‍❤‍💋‍👩🏿' => 'poljub: ženska, ženska, srednje temen ten kože in temen ten kože',
    '👩🏿‍❤‍💋‍👩🏻' => 'poljub: ženska, ženska, temen ten kože in svetel ten kože',
    '👩🏿‍❤‍💋‍👩🏼' => 'poljub: ženska, ženska, temen ten kože in srednje svetel ten kože',
    '👩🏿‍❤‍💋‍👩🏽' => 'poljub: ženska, ženska, temen ten kože in srednji ten kože',
    '👩🏿‍❤‍💋‍👩🏾' => 'poljub: ženska, ženska, temen ten kože in srednje temen ten kože',
    '👩🏿‍❤‍💋‍👩🏿' => 'poljub: ženska, ženska in temen ten kože',
    '🧑🏻‍🫯‍🧑🏼' => 'rokoborba: svetel ten kože in srednje svetel ten kože',
    '🧑🏻‍🫯‍🧑🏽' => 'rokoborba: svetel ten kože in srednji ten kože',
    '🧑🏻‍🫯‍🧑🏾' => 'rokoborba: svetel ten kože in srednje temen ten kože',
    '🧑🏻‍🫯‍🧑🏿' => 'rokoborba: svetel ten kože in temen ten kože',
    '🧑🏼‍🫯‍🧑🏻' => 'rokoborba: srednje svetel ten kože in svetel ten kože',
    '🧑🏼‍🫯‍🧑🏽' => 'rokoborba: srednje svetel ten kože in srednji ten kože',
    '🧑🏼‍🫯‍🧑🏾' => 'rokoborba: srednje svetel ten kože in srednje temen ten kože',
    '🧑🏼‍🫯‍🧑🏿' => 'rokoborba: srednje svetel ten kože in temen ten kože',
    '🧑🏽‍🫯‍🧑🏻' => 'rokoborba: srednji ten kože in svetel ten kože',
    '🧑🏽‍🫯‍🧑🏼' => 'rokoborba: srednji ten kože in srednje svetel ten kože',
    '🧑🏽‍🫯‍🧑🏾' => 'rokoborba: srednji ten kože in srednje temen ten kože',
    '🧑🏽‍🫯‍🧑🏿' => 'rokoborba: srednji ten kože in temen ten kože',
    '🧑🏾‍🫯‍🧑🏻' => 'rokoborba: srednje temen ten kože in svetel ten kože',
    '🧑🏾‍🫯‍🧑🏼' => 'rokoborba: srednje temen ten kože in srednje svetel ten kože',
    '🧑🏾‍🫯‍🧑🏽' => 'rokoborba: srednje temen ten kože in srednji ten kože',
    '🧑🏾‍🫯‍🧑🏿' => 'rokoborba: srednje temen ten kože in temen ten kože',
    '🧑🏿‍🫯‍🧑🏻' => 'rokoborba: temen ten kože in svetel ten kože',
    '🧑🏿‍🫯‍🧑🏼' => 'rokoborba: temen ten kože in srednje svetel ten kože',
    '🧑🏿‍🫯‍🧑🏽' => 'rokoborba: temen ten kože in srednji ten kože',
    '🧑🏿‍🫯‍🧑🏾' => 'rokoborba: temen ten kože in srednje temen ten kože',
    '🧑🏻‍❤‍🧑🏼' => 'par s srčkom: odrasla oseba, odrasla oseba, svetel ten kože in srednje svetel ten kože',
    '🧑🏻‍❤‍🧑🏽' => 'par s srčkom: odrasla oseba, odrasla oseba, svetel ten kože in srednji ten kože',
    '🧑🏻‍❤‍🧑🏾' => 'par s srčkom: odrasla oseba, odrasla oseba, svetel ten kože in srednje temen ten kože',
    '🧑🏻‍❤‍🧑🏿' => 'par s srčkom: odrasla oseba, odrasla oseba, svetel ten kože in temen ten kože',
    '🧑🏼‍❤‍🧑🏻' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje svetel ten kože in svetel ten kože',
    '🧑🏼‍❤‍🧑🏽' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje svetel ten kože in srednji ten kože',
    '🧑🏼‍❤‍🧑🏾' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje svetel ten kože in srednje temen ten kože',
    '🧑🏼‍❤‍🧑🏿' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje svetel ten kože in temen ten kože',
    '🧑🏽‍❤‍🧑🏻' => 'par s srčkom: odrasla oseba, odrasla oseba, srednji ten kože in svetel ten kože',
    '🧑🏽‍❤‍🧑🏼' => 'par s srčkom: odrasla oseba, odrasla oseba, srednji ten kože in srednje svetel ten kože',
    '🧑🏽‍❤‍🧑🏾' => 'par s srčkom: odrasla oseba, odrasla oseba, srednji ten kože in srednje temen ten kože',
    '🧑🏽‍❤‍🧑🏿' => 'par s srčkom: odrasla oseba, odrasla oseba, srednji ten kože in temen ten kože',
    '🧑🏾‍❤‍🧑🏻' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje temen ten kože in svetel ten kože',
    '🧑🏾‍❤‍🧑🏼' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje temen ten kože in srednje svetel ten kože',
    '🧑🏾‍❤‍🧑🏽' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje temen ten kože in srednji ten kože',
    '🧑🏾‍❤‍🧑🏿' => 'par s srčkom: odrasla oseba, odrasla oseba, srednje temen ten kože in temen ten kože',
    '🧑🏿‍❤‍🧑🏻' => 'par s srčkom: odrasla oseba, odrasla oseba, temen ten kože in svetel ten kože',
    '🧑🏿‍❤‍🧑🏼' => 'par s srčkom: odrasla oseba, odrasla oseba, temen ten kože in srednje svetel ten kože',
    '🧑🏿‍❤‍🧑🏽' => 'par s srčkom: odrasla oseba, odrasla oseba, temen ten kože in srednji ten kože',
    '🧑🏿‍❤‍🧑🏾' => 'par s srčkom: odrasla oseba, odrasla oseba, temen ten kože in srednje temen ten kože',
    '🧑🏻‍🐰‍🧑🏼' => 'žurerski ženski: svetel ten kože in srednje svetel ten kože',
    '🧑🏻‍🐰‍🧑🏽' => 'žurerski ženski: svetel ten kože in srednji ten kože',
    '🧑🏻‍🐰‍🧑🏾' => 'žurerski ženski: svetel ten kože in srednje temen ten kože',
    '🧑🏻‍🐰‍🧑🏿' => 'žurerski ženski: svetel ten kože in temen ten kože',
    '🧑🏼‍🐰‍🧑🏻' => 'žurerski ženski: srednje svetel ten kože in svetel ten kože',
    '🧑🏼‍🐰‍🧑🏽' => 'žurerski ženski: srednje svetel ten kože in srednji ten kože',
    '🧑🏼‍🐰‍🧑🏾' => 'žurerski ženski: srednje svetel ten kože in srednje temen ten kože',
    '🧑🏼‍🐰‍🧑🏿' => 'žurerski ženski: srednje svetel ten kože in temen ten kože',
    '🧑🏽‍🐰‍🧑🏻' => 'žurerski ženski: srednji ten kože in svetel ten kože',
    '🧑🏽‍🐰‍🧑🏼' => 'žurerski ženski: srednji ten kože in srednje svetel ten kože',
    '🧑🏽‍🐰‍🧑🏾' => 'žurerski ženski: srednji ten kože in srednje temen ten kože',
    '🧑🏽‍🐰‍🧑🏿' => 'žurerski ženski: srednji ten kože in temen ten kože',
    '🧑🏾‍🐰‍🧑🏻' => 'žurerski ženski: srednje temen ten kože in svetel ten kože',
    '🧑🏾‍🐰‍🧑🏼' => 'žurerski ženski: srednje temen ten kože in srednje svetel ten kože',
    '🧑🏾‍🐰‍🧑🏽' => 'žurerski ženski: srednje temen ten kože in srednji ten kože',
    '🧑🏾‍🐰‍🧑🏿' => 'žurerski ženski: srednje temen ten kože in temen ten kože',
    '🧑🏿‍🐰‍🧑🏻' => 'žurerski ženski: temen ten kože in svetel ten kože',
    '🧑🏿‍🐰‍🧑🏼' => 'žurerski ženski: temen ten kože in srednje svetel ten kože',
    '🧑🏿‍🐰‍🧑🏽' => 'žurerski ženski: temen ten kože in srednji ten kože',
    '🧑🏿‍🐰‍🧑🏾' => 'žurerski ženski: temen ten kože in srednje temen ten kože',
    '👨🏻‍🫯‍👨🏼' => 'moška rokoborba: svetel ten kože in srednje svetel ten kože',
    '👨🏻‍🫯‍👨🏽' => 'moška rokoborba: svetel ten kože in srednji ten kože',
    '👨🏻‍🫯‍👨🏾' => 'moška rokoborba: svetel ten kože in srednje temen ten kože',
    '👨🏻‍🫯‍👨🏿' => 'moška rokoborba: svetel ten kože in temen ten kože',
    '👨🏼‍🫯‍👨🏻' => 'moška rokoborba: srednje svetel ten kože in svetel ten kože',
    '👨🏼‍🫯‍👨🏽' => 'moška rokoborba: srednje svetel ten kože in srednji ten kože',
    '👨🏼‍🫯‍👨🏾' => 'moška rokoborba: srednje svetel ten kože in srednje temen ten kože',
    '👨🏼‍🫯‍👨🏿' => 'moška rokoborba: srednje svetel ten kože in temen ten kože',
    '👨🏽‍🫯‍👨🏻' => 'moška rokoborba: srednji ten kože in svetel ten kože',
    '👨🏽‍🫯‍👨🏼' => 'moška rokoborba: srednji ten kože in srednje svetel ten kože',
    '👨🏽‍🫯‍👨🏾' => 'moška rokoborba: srednji ten kože in srednje temen ten kože',
    '👨🏽‍🫯‍👨🏿' => 'moška rokoborba: srednji ten kože in temen ten kože',
    '👨🏾‍🫯‍👨🏻' => 'moška rokoborba: srednje temen ten kože in svetel ten kože',
    '👨🏾‍🫯‍👨🏼' => 'moška rokoborba: srednje temen ten kože in srednje svetel ten kože',
    '👨🏾‍🫯‍👨🏽' => 'moška rokoborba: srednje temen ten kože in srednji ten kože',
    '👨🏾‍🫯‍👨🏿' => 'moška rokoborba: srednje temen ten kože in temen ten kože',
    '👨🏿‍🫯‍👨🏻' => 'moška rokoborba: temen ten kože in svetel ten kože',
    '👨🏿‍🫯‍👨🏼' => 'moška rokoborba: temen ten kože in srednje svetel ten kože',
    '👨🏿‍🫯‍👨🏽' => 'moška rokoborba: temen ten kože in srednji ten kože',
    '👨🏿‍🫯‍👨🏾' => 'moška rokoborba: temen ten kože in srednje temen ten kože',
    '👨🏻‍❤‍👨🏻' => 'par s srčkom: moški, moški in svetel ten kože',
    '👨🏻‍❤‍👨🏼' => 'par s srčkom: moški, moški, svetel ten kože in srednje svetel ten kože',
    '👨🏻‍❤‍👨🏽' => 'par s srčkom: moški, moški, svetel ten kože in srednji ten kože',
    '👨🏻‍❤‍👨🏾' => 'par s srčkom: moški, moški, svetel ten kože in srednje temen ten kože',
    '👨🏻‍❤‍👨🏿' => 'par s srčkom: moški, moški, svetel ten kože in temen ten kože',
    '👨🏼‍❤‍👨🏻' => 'par s srčkom: moški, moški, srednje svetel ten kože in svetel ten kože',
    '👨🏼‍❤‍👨🏼' => 'par s srčkom: moški, moški in srednje svetel ten kože',
    '👨🏼‍❤‍👨🏽' => 'par s srčkom: moški, moški, srednje svetel ten kože in srednji ten kože',
    '👨🏼‍❤‍👨🏾' => 'par s srčkom: moški, moški, srednje svetel ten kože in srednje temen ten kože',
    '👨🏼‍❤‍👨🏿' => 'par s srčkom: moški, moški, srednje svetel ten kože in temen ten kože',
    '👨🏽‍❤‍👨🏻' => 'par s srčkom: moški, moški, srednji ten kože in svetel ten kože',
    '👨🏽‍❤‍👨🏼' => 'par s srčkom: moški, moški, srednji ten kože in srednje svetel ten kože',
    '👨🏽‍❤‍👨🏽' => 'par s srčkom: moški, moški in srednji ten kože',
    '👨🏽‍❤‍👨🏾' => 'par s srčkom: moški, moški, srednji ten kože in srednje temen ten kože',
    '👨🏽‍❤‍👨🏿' => 'par s srčkom: moški, moški, srednji ten kože in temen ten kože',
    '👨🏾‍❤‍👨🏻' => 'par s srčkom: moški, moški, srednje temen ten kože in svetel ten kože',
    '👨🏾‍❤‍👨🏼' => 'par s srčkom: moški, moški, srednje temen ten kože in srednje svetel ten kože',
    '👨🏾‍❤‍👨🏽' => 'par s srčkom: moški, moški, srednje temen ten kože in srednji ten kože',
    '👨🏾‍❤‍👨🏾' => 'par s srčkom: moški, moški in srednje temen ten kože',
    '👨🏾‍❤‍👨🏿' => 'par s srčkom: moški, moški, srednje temen ten kože in temen ten kože',
    '👨🏿‍❤‍👨🏻' => 'par s srčkom: moški, moški, temen ten kože in svetel ten kože',
    '👨🏿‍❤‍👨🏼' => 'par s srčkom: moški, moški, temen ten kože in srednje svetel ten kože',
    '👨🏿‍❤‍👨🏽' => 'par s srčkom: moški, moški, temen ten kože in srednji ten kože',
    '👨🏿‍❤‍👨🏾' => 'par s srčkom: moški, moški, temen ten kože in srednje temen ten kože',
    '👨🏿‍❤‍👨🏿' => 'par s srčkom: moški, moški in temen ten kože',
    '👨🏻‍🐰‍👨🏼' => 'moška zabava: svetel ten kože in srednje svetel ten kože',
    '👨🏻‍🐰‍👨🏽' => 'moška zabava: svetel ten kože in srednji ten kože',
    '👨🏻‍🐰‍👨🏾' => 'moška zabava: svetel ten kože in srednje temen ten kože',
    '👨🏻‍🐰‍👨🏿' => 'moška zabava: svetel ten kože in temen ten kože',
    '👨🏼‍🐰‍👨🏻' => 'moška zabava: srednje svetel ten kože in svetel ten kože',
    '👨🏼‍🐰‍👨🏽' => 'moška zabava: srednje svetel ten kože in srednji ten kože',
    '👨🏼‍🐰‍👨🏾' => 'moška zabava: srednje svetel ten kože in srednje temen ten kože',
    '👨🏼‍🐰‍👨🏿' => 'moška zabava: srednje svetel ten kože in temen ten kože',
    '👨🏽‍🐰‍👨🏻' => 'moška zabava: srednji ten kože in svetel ten kože',
    '👨🏽‍🐰‍👨🏼' => 'moška zabava: srednji ten kože in srednje svetel ten kože',
    '👨🏽‍🐰‍👨🏾' => 'moška zabava: srednji ten kože in srednje temen ten kože',
    '👨🏽‍🐰‍👨🏿' => 'moška zabava: srednji ten kože in temen ten kože',
    '👨🏾‍🐰‍👨🏻' => 'moška zabava: srednje temen ten kože in svetel ten kože',
    '👨🏾‍🐰‍👨🏼' => 'moška zabava: srednje temen ten kože in srednje svetel ten kože',
    '👨🏾‍🐰‍👨🏽' => 'moška zabava: srednje temen ten kože in srednji ten kože',
    '👨🏾‍🐰‍👨🏿' => 'moška zabava: srednje temen ten kože in temen ten kože',
    '👨🏿‍🐰‍👨🏻' => 'moška zabava: temen ten kože in svetel ten kože',
    '👨🏿‍🐰‍👨🏼' => 'moška zabava: temen ten kože in srednje svetel ten kože',
    '👨🏿‍🐰‍👨🏽' => 'moška zabava: temen ten kože in srednji ten kože',
    '👨🏿‍🐰‍👨🏾' => 'moška zabava: temen ten kože in srednje temen ten kože',
    '👩🏻‍🫯‍👩🏼' => 'ženska rokoborba: svetel ten kože in srednje svetel ten kože',
    '👩🏻‍🫯‍👩🏽' => 'ženska rokoborba: svetel ten kože in srednji ten kože',
    '👩🏻‍🫯‍👩🏾' => 'ženska rokoborba: svetel ten kože in srednje temen ten kože',
    '👩🏻‍🫯‍👩🏿' => 'ženska rokoborba: svetel ten kože in temen ten kože',
    '👩🏼‍🫯‍👩🏻' => 'ženska rokoborba: srednje svetel ten kože in svetel ten kože',
    '👩🏼‍🫯‍👩🏽' => 'ženska rokoborba: srednje svetel ten kože in srednji ten kože',
    '👩🏼‍🫯‍👩🏾' => 'ženska rokoborba: srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍🫯‍👩🏿' => 'ženska rokoborba: srednje svetel ten kože in temen ten kože',
    '👩🏽‍🫯‍👩🏻' => 'ženska rokoborba: srednji ten kože in svetel ten kože',
    '👩🏽‍🫯‍👩🏼' => 'ženska rokoborba: srednji ten kože in srednje svetel ten kože',
    '👩🏽‍🫯‍👩🏾' => 'ženska rokoborba: srednji ten kože in srednje temen ten kože',
    '👩🏽‍🫯‍👩🏿' => 'ženska rokoborba: srednji ten kože in temen ten kože',
    '👩🏾‍🫯‍👩🏻' => 'ženska rokoborba: srednje temen ten kože in svetel ten kože',
    '👩🏾‍🫯‍👩🏼' => 'ženska rokoborba: srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍🫯‍👩🏽' => 'ženska rokoborba: srednje temen ten kože in srednji ten kože',
    '👩🏾‍🫯‍👩🏿' => 'ženska rokoborba: srednje temen ten kože in temen ten kože',
    '👩🏿‍🫯‍👩🏻' => 'ženska rokoborba: temen ten kože in svetel ten kože',
    '👩🏿‍🫯‍👩🏼' => 'ženska rokoborba: temen ten kože in srednje svetel ten kože',
    '👩🏿‍🫯‍👩🏽' => 'ženska rokoborba: temen ten kože in srednji ten kože',
    '👩🏿‍🫯‍👩🏾' => 'ženska rokoborba: temen ten kože in srednje temen ten kože',
    '👩🏻‍❤‍👨🏻' => 'par s srčkom: ženska, moški in svetel ten kože',
    '👩🏻‍❤‍👨🏼' => 'par s srčkom: ženska, moški, svetel ten kože in srednje svetel ten kože',
    '👩🏻‍❤‍👨🏽' => 'par s srčkom: ženska, moški, svetel ten kože in srednji ten kože',
    '👩🏻‍❤‍👨🏾' => 'par s srčkom: ženska, moški, svetel ten kože in srednje temen ten kože',
    '👩🏻‍❤‍👨🏿' => 'par s srčkom: ženska, moški, svetel ten kože in temen ten kože',
    '👩🏼‍❤‍👨🏻' => 'par s srčkom: ženska, moški, srednje svetel ten kože in svetel ten kože',
    '👩🏼‍❤‍👨🏼' => 'par s srčkom: ženska, moški in srednje svetel ten kože',
    '👩🏼‍❤‍👨🏽' => 'par s srčkom: ženska, moški, srednje svetel ten kože in srednji ten kože',
    '👩🏼‍❤‍👨🏾' => 'par s srčkom: ženska, moški, srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍❤‍👨🏿' => 'par s srčkom: ženska, moški, srednje svetel ten kože in temen ten kože',
    '👩🏽‍❤‍👨🏻' => 'par s srčkom: ženska, moški, srednji ten kože in svetel ten kože',
    '👩🏽‍❤‍👨🏼' => 'par s srčkom: ženska, moški, srednji ten kože in srednje svetel ten kože',
    '👩🏽‍❤‍👨🏽' => 'par s srčkom: ženska, moški in srednji ten kože',
    '👩🏽‍❤‍👨🏾' => 'par s srčkom: ženska, moški, srednji ten kože in srednje temen ten kože',
    '👩🏽‍❤‍👨🏿' => 'par s srčkom: ženska, moški, srednji ten kože in temen ten kože',
    '👩🏾‍❤‍👨🏻' => 'par s srčkom: ženska, moški, srednje temen ten kože in svetel ten kože',
    '👩🏾‍❤‍👨🏼' => 'par s srčkom: ženska, moški, srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍❤‍👨🏽' => 'par s srčkom: ženska, moški, srednje temen ten kože in srednji ten kože',
    '👩🏾‍❤‍👨🏾' => 'par s srčkom: ženska, moški in srednje temen ten kože',
    '👩🏾‍❤‍👨🏿' => 'par s srčkom: ženska, moški, srednje temen ten kože in temen ten kože',
    '👩🏿‍❤‍👨🏻' => 'par s srčkom: ženska, moški, temen ten kože in svetel ten kože',
    '👩🏿‍❤‍👨🏼' => 'par s srčkom: ženska, moški, temen ten kože in srednje svetel ten kože',
    '👩🏿‍❤‍👨🏽' => 'par s srčkom: ženska, moški, temen ten kože in srednji ten kože',
    '👩🏿‍❤‍👨🏾' => 'par s srčkom: ženska, moški, temen ten kože in srednje temen ten kože',
    '👩🏿‍❤‍👨🏿' => 'par s srčkom: ženska, moški in temen ten kože',
    '👩🏻‍❤‍👩🏻' => 'par s srčkom: ženska, ženska in svetel ten kože',
    '👩🏻‍❤‍👩🏼' => 'par s srčkom: ženska, ženska, svetel ten kože in srednje svetel ten kože',
    '👩🏻‍❤‍👩🏽' => 'par s srčkom: ženska, ženska, svetel ten kože in srednji ten kože',
    '👩🏻‍❤‍👩🏾' => 'par s srčkom: ženska, ženska, svetel ten kože in srednje temen ten kože',
    '👩🏻‍❤‍👩🏿' => 'par s srčkom: ženska, ženska, svetel ten kože in temen ten kože',
    '👩🏼‍❤‍👩🏻' => 'par s srčkom: ženska, ženska, srednje svetel ten kože in svetel ten kože',
    '👩🏼‍❤‍👩🏼' => 'par s srčkom: ženska, ženska in srednje svetel ten kože',
    '👩🏼‍❤‍👩🏽' => 'par s srčkom: ženska, ženska, srednje svetel ten kože in srednji ten kože',
    '👩🏼‍❤‍👩🏾' => 'par s srčkom: ženska, ženska, srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍❤‍👩🏿' => 'par s srčkom: ženska, ženska, srednje svetel ten kože in temen ten kože',
    '👩🏽‍❤‍👩🏻' => 'par s srčkom: ženska, ženska, srednji ten kože in svetel ten kože',
    '👩🏽‍❤‍👩🏼' => 'par s srčkom: ženska, ženska, srednji ten kože in srednje svetel ten kože',
    '👩🏽‍❤‍👩🏽' => 'par s srčkom: ženska, ženska in srednji ten kože',
    '👩🏽‍❤‍👩🏾' => 'par s srčkom: ženska, ženska, srednji ten kože in srednje temen ten kože',
    '👩🏽‍❤‍👩🏿' => 'par s srčkom: ženska, ženska, srednji ten kože in temen ten kože',
    '👩🏾‍❤‍👩🏻' => 'par s srčkom: ženska, ženska, srednje temen ten kože in svetel ten kože',
    '👩🏾‍❤‍👩🏼' => 'par s srčkom: ženska, ženska, srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍❤‍👩🏽' => 'par s srčkom: ženska, ženska, srednje temen ten kože in srednji ten kože',
    '👩🏾‍❤‍👩🏾' => 'par s srčkom: ženska, ženska in srednje temen ten kože',
    '👩🏾‍❤‍👩🏿' => 'par s srčkom: ženska, ženska, srednje temen ten kože in temen ten kože',
    '👩🏿‍❤‍👩🏻' => 'par s srčkom: ženska, ženska, temen ten kože in svetel ten kože',
    '👩🏿‍❤‍👩🏼' => 'par s srčkom: ženska, ženska, temen ten kože in srednje svetel ten kože',
    '👩🏿‍❤‍👩🏽' => 'par s srčkom: ženska, ženska, temen ten kože in srednji ten kože',
    '👩🏿‍❤‍👩🏾' => 'par s srčkom: ženska, ženska, temen ten kože in srednje temen ten kože',
    '👩🏿‍❤‍👩🏿' => 'par s srčkom: ženska, ženska in temen ten kože',
    '👩🏻‍🐰‍👩🏼' => 'ženska zabava: svetel ten kože in srednje svetel ten kože',
    '👩🏻‍🐰‍👩🏽' => 'ženska zabava: svetel ten kože in srednji ten kože',
    '👩🏻‍🐰‍👩🏾' => 'ženska zabava: svetel ten kože in srednje temen ten kože',
    '👩🏻‍🐰‍👩🏿' => 'ženska zabava: svetel ten kože in temen ten kože',
    '👩🏼‍🐰‍👩🏻' => 'ženska zabava: srednje svetel ten kože in svetel ten kože',
    '👩🏼‍🐰‍👩🏽' => 'ženska zabava: srednje svetel ten kože in srednji ten kože',
    '👩🏼‍🐰‍👩🏾' => 'ženska zabava: srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍🐰‍👩🏿' => 'ženska zabava: srednje svetel ten kože in temen ten kože',
    '👩🏽‍🐰‍👩🏻' => 'ženska zabava: srednji ten kože in svetel ten kože',
    '👩🏽‍🐰‍👩🏼' => 'ženska zabava: srednji ten kože in srednje svetel ten kože',
    '👩🏽‍🐰‍👩🏾' => 'ženska zabava: srednji ten kože in srednje temen ten kože',
    '👩🏽‍🐰‍👩🏿' => 'ženska zabava: srednji ten kože in temen ten kože',
    '👩🏾‍🐰‍👩🏻' => 'ženska zabava: srednje temen ten kože in svetel ten kože',
    '👩🏾‍🐰‍👩🏼' => 'ženska zabava: srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍🐰‍👩🏽' => 'ženska zabava: srednje temen ten kože in srednji ten kože',
    '👩🏾‍🐰‍👩🏿' => 'ženska zabava: srednje temen ten kože in temen ten kože',
    '👩🏿‍🐰‍👩🏻' => 'ženska zabava: temen ten kože in svetel ten kože',
    '👩🏿‍🐰‍👩🏼' => 'ženska zabava: temen ten kože in srednje svetel ten kože',
    '👩🏿‍🐰‍👩🏽' => 'ženska zabava: temen ten kože in srednji ten kože',
    '👩🏿‍🐰‍👩🏾' => 'ženska zabava: temen ten kože in srednje temen ten kože',
    '🧑🏻‍🤝‍🧑🏻' => 'osebi se držita za roke: svetel ten kože',
    '🧑🏻‍🤝‍🧑🏼' => 'osebi se držita za roke: svetel ten kože in srednje svetel ten kože',
    '🧑🏻‍🤝‍🧑🏽' => 'osebi se držita za roke: svetel ten kože in srednji ten kože',
    '🧑🏻‍🤝‍🧑🏾' => 'osebi se držita za roke: svetel ten kože in srednje temen ten kože',
    '🧑🏻‍🤝‍🧑🏿' => 'osebi se držita za roke: svetel ten kože in temen ten kože',
    '🧑🏼‍🤝‍🧑🏻' => 'osebi se držita za roke: srednje svetel ten kože in svetel ten kože',
    '🧑🏼‍🤝‍🧑🏼' => 'osebi se držita za roke: srednje svetel ten kože',
    '🧑🏼‍🤝‍🧑🏽' => 'osebi se držita za roke: srednje svetel ten kože in srednji ten kože',
    '🧑🏼‍🤝‍🧑🏾' => 'osebi se držita za roke: srednje svetel ten kože in srednje temen ten kože',
    '🧑🏼‍🤝‍🧑🏿' => 'osebi se držita za roke: srednje svetel ten kože in temen ten kože',
    '🧑🏽‍🤝‍🧑🏻' => 'osebi se držita za roke: srednji ten kože in svetel ten kože',
    '🧑🏽‍🤝‍🧑🏼' => 'osebi se držita za roke: srednji ten kože in srednje svetel ten kože',
    '🧑🏽‍🤝‍🧑🏽' => 'osebi se držita za roke: srednji ten kože',
    '🧑🏽‍🤝‍🧑🏾' => 'osebi se držita za roke: srednji ten kože in srednje temen ten kože',
    '🧑🏽‍🤝‍🧑🏿' => 'osebi se držita za roke: srednji ten kože in temen ten kože',
    '🧑🏾‍🤝‍🧑🏻' => 'osebi se držita za roke: srednje temen ten kože in svetel ten kože',
    '🧑🏾‍🤝‍🧑🏼' => 'osebi se držita za roke: srednje temen ten kože in srednje svetel ten kože',
    '🧑🏾‍🤝‍🧑🏽' => 'osebi se držita za roke: srednje temen ten kože in srednji ten kože',
    '🧑🏾‍🤝‍🧑🏾' => 'osebi se držita za roke: srednje temen ten kože',
    '🧑🏾‍🤝‍🧑🏿' => 'osebi se držita za roke: srednje temen ten kože in temen ten kože',
    '🧑🏿‍🤝‍🧑🏻' => 'osebi se držita za roke: temen ten kože in svetel ten kože',
    '🧑🏿‍🤝‍🧑🏼' => 'osebi se držita za roke: temen ten kože in srednje svetel ten kože',
    '🧑🏿‍🤝‍🧑🏽' => 'osebi se držita za roke: temen ten kože in srednji ten kože',
    '🧑🏿‍🤝‍🧑🏾' => 'osebi se držita za roke: temen ten kože in srednje temen ten kože',
    '🧑🏿‍🤝‍🧑🏿' => 'osebi se držita za roke: temen ten kože',
    '👩🏻‍🤝‍👩🏼' => 'ženski se držita za roke: svetel ten kože in srednje svetel ten kože',
    '👩🏻‍🤝‍👩🏽' => 'ženski se držita za roke: svetel ten kože in srednji ten kože',
    '👩🏻‍🤝‍👩🏾' => 'ženski se držita za roke: svetel ten kože in srednje temen ten kože',
    '👩🏻‍🤝‍👩🏿' => 'ženski se držita za roke: svetel ten kože in temen ten kože',
    '👩🏼‍🤝‍👩🏻' => 'ženski se držita za roke: srednje svetel ten kože in svetel ten kože',
    '👩🏼‍🤝‍👩🏽' => 'ženski se držita za roke: srednje svetel ten kože in srednji ten kože',
    '👩🏼‍🤝‍👩🏾' => 'ženski se držita za roke: srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍🤝‍👩🏿' => 'ženski se držita za roke: srednje svetel ten kože in temen ten kože',
    '👩🏽‍🤝‍👩🏻' => 'ženski se držita za roke: srednji ten kože in svetel ten kože',
    '👩🏽‍🤝‍👩🏼' => 'ženski se držita za roke: srednji ten kože in srednje svetel ten kože',
    '👩🏽‍🤝‍👩🏾' => 'ženski se držita za roke: srednji ten kože in srednje temen ten kože',
    '👩🏽‍🤝‍👩🏿' => 'ženski se držita za roke: srednji ten kože in temen ten kože',
    '👩🏾‍🤝‍👩🏻' => 'ženski se držita za roke: srednje temen ten kože in svetel ten kože',
    '👩🏾‍🤝‍👩🏼' => 'ženski se držita za roke: srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍🤝‍👩🏽' => 'ženski se držita za roke: srednje temen ten kože in srednji ten kože',
    '👩🏾‍🤝‍👩🏿' => 'ženski se držita za roke: srednje temen ten kože in temen ten kože',
    '👩🏿‍🤝‍👩🏻' => 'ženski se držita za roke: temen ten kože in svetel ten kože',
    '👩🏿‍🤝‍👩🏼' => 'ženski se držita za roke: temen ten kože in srednje svetel ten kože',
    '👩🏿‍🤝‍👩🏽' => 'ženski se držita za roke: temen ten kože in srednji ten kože',
    '👩🏿‍🤝‍👩🏾' => 'ženski se držita za roke: temen ten kože in srednje temen ten kože',
    '👩🏻‍🤝‍👨🏼' => 'moški in ženska se držita za roke: svetel ten kože in srednje svetel ten kože',
    '👩🏻‍🤝‍👨🏽' => 'moški in ženska se držita za roke: svetel ten kože in srednji ten kože',
    '👩🏻‍🤝‍👨🏾' => 'moški in ženska se držita za roke: svetel ten kože in srednje temen ten kože',
    '👩🏻‍🤝‍👨🏿' => 'moški in ženska se držita za roke: svetel ten kože in temen ten kože',
    '👩🏼‍🤝‍👨🏻' => 'moški in ženska se držita za roke: srednje svetel ten kože in svetel ten kože',
    '👩🏼‍🤝‍👨🏽' => 'moški in ženska se držita za roke: srednje svetel ten kože in srednji ten kože',
    '👩🏼‍🤝‍👨🏾' => 'moški in ženska se držita za roke: srednje svetel ten kože in srednje temen ten kože',
    '👩🏼‍🤝‍👨🏿' => 'moški in ženska se držita za roke: srednje svetel ten kože in temen ten kože',
    '👩🏽‍🤝‍👨🏻' => 'moški in ženska se držita za roke: srednji ten kože in svetel ten kože',
    '👩🏽‍🤝‍👨🏼' => 'moški in ženska se držita za roke: srednji ten kože in srednje svetel ten kože',
    '👩🏽‍🤝‍👨🏾' => 'moški in ženska se držita za roke: srednji ten kože in srednje temen ten kože',
    '👩🏽‍🤝‍👨🏿' => 'moški in ženska se držita za roke: srednji ten kože in temen ten kože',
    '👩🏾‍🤝‍👨🏻' => 'moški in ženska se držita za roke: srednje temen ten kože in svetel ten kože',
    '👩🏾‍🤝‍👨🏼' => 'moški in ženska se držita za roke: srednje temen ten kože in srednje svetel ten kože',
    '👩🏾‍🤝‍👨🏽' => 'moški in ženska se držita za roke: srednje temen ten kože in srednji ten kože',
    '👩🏾‍🤝‍👨🏿' => 'moški in ženska se držita za roke: srednje temen ten kože in temen ten kože',
    '👩🏿‍🤝‍👨🏻' => 'moški in ženska se držita za roke: temen ten kože in svetel ten kože',
    '👩🏿‍🤝‍👨🏼' => 'moški in ženska se držita za roke: temen ten kože in srednje svetel ten kože',
    '👩🏿‍🤝‍👨🏽' => 'moški in ženska se držita za roke: temen ten kože in srednji ten kože',
    '👩🏿‍🤝‍👨🏾' => 'moški in ženska se držita za roke: temen ten kože in srednje temen ten kože',
    '👨🏻‍🤝‍👨🏼' => 'moška se držita za roke: svetel ten kože in srednje svetel ten kože',
    '👨🏻‍🤝‍👨🏽' => 'moška se držita za roke: svetel ten kože in srednji ten kože',
    '👨🏻‍🤝‍👨🏾' => 'moška se držita za roke: svetel ten kože in srednje temen ten kože',
    '👨🏻‍🤝‍👨🏿' => 'moška se držita za roke: svetel ten kože in temen ten kože',
    '👨🏼‍🤝‍👨🏻' => 'moška se držita za roke: srednje svetel ten kože in svetel ten kože',
    '👨🏼‍🤝‍👨🏽' => 'moška se držita za roke: srednje svetel ten kože in srednji ten kože',
    '👨🏼‍🤝‍👨🏾' => 'moška se držita za roke: srednje svetel ten kože in srednje temen ten kože',
    '👨🏼‍🤝‍👨🏿' => 'moška se držita za roke: srednje svetel ten kože in temen ten kože',
    '👨🏽‍🤝‍👨🏻' => 'moška se držita za roke: srednji ten kože in svetel ten kože',
    '👨🏽‍🤝‍👨🏼' => 'moška se držita za roke: srednji ten kože in srednje svetel ten kože',
    '👨🏽‍🤝‍👨🏾' => 'moška se držita za roke: srednji ten kože in srednje temen ten kože',
    '👨🏽‍🤝‍👨🏿' => 'moška se držita za roke: srednji ten kože in temen ten kože',
    '👨🏾‍🤝‍👨🏻' => 'moška se držita za roke: srednje temen ten kože in svetel ten kože',
    '👨🏾‍🤝‍👨🏼' => 'moška se držita za roke: srednje temen ten kože in srednje svetel ten kože',
    '👨🏾‍🤝‍👨🏽' => 'moška se držita za roke: srednje temen ten kože in srednji ten kože',
    '👨🏾‍🤝‍👨🏿' => 'moška se držita za roke: srednje temen ten kože in temen ten kože',
    '👨🏿‍🤝‍👨🏻' => 'moška se držita za roke: temen ten kože in svetel ten kože',
    '👨🏿‍🤝‍👨🏼' => 'moška se držita za roke: temen ten kože in srednje svetel ten kože',
    '👨🏿‍🤝‍👨🏽' => 'moška se držita za roke: temen ten kože in srednji ten kože',
    '👨🏿‍🤝‍👨🏾' => 'moška se držita za roke: temen ten kože in srednje temen ten kože',
    '👩‍❤‍💋‍👨' => 'poljub: ženska in moški',
    '👨‍❤‍💋‍👨' => 'poljub: moški in moški',
    '👩‍❤‍💋‍👩' => 'poljub: ženska in ženska',
    '👨‍👩‍👧‍👦' => 'družina: moški, ženska, dekle in fant',
    '👨‍👩‍👦‍👦' => 'družina: moški, ženska, fant in fant',
    '👨‍👩‍👧‍👧' => 'družina: moški, ženska, dekle in dekle',
    '👨‍👨‍👧‍👦' => 'družina: moški, moški, dekle in fant',
    '👨‍👨‍👦‍👦' => 'družina: moški, moški, fant in fant',
    '👨‍👨‍👧‍👧' => 'družina: moški, moški, dekle in dekle',
    '👩‍👩‍👧‍👦' => 'družina: ženska, ženska, dekle in fant',
    '👩‍👩‍👦‍👦' => 'družina: ženska, ženska, fant in fant',
    '👩‍👩‍👧‍👧' => 'družina: ženska, ženska, dekle in dekle',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'zastava: Anglija',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'zastava: Škotska',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'zastava: Wales',
    '🧑‍🧑‍🧒‍🧒' => 'družina: odrasel, odrasel, otrok, otrok',
    '🚶🏻‍♀‍➡' => 'ženska, ki hodi: svetel ten kože in , obrnjen v desno',
    '🚶🏼‍♀‍➡' => 'ženska, ki hodi: srednje svetel ten kože in , obrnjen v desno',
    '🚶🏽‍♀‍➡' => 'ženska, ki hodi: srednji ten kože in , obrnjen v desno',
    '🚶🏾‍♀‍➡' => 'ženska, ki hodi: srednje temen ten kože in , obrnjen v desno',
    '🚶🏿‍♀‍➡' => 'ženska, ki hodi: temen ten kože in , obrnjen v desno',
    '🚶🏻‍♂‍➡' => 'moški, ki hodi: svetel ten kože in , obrnjen v desno',
    '🚶🏼‍♂‍➡' => 'moški, ki hodi: srednje svetel ten kože in , obrnjen v desno',
    '🚶🏽‍♂‍➡' => 'moški, ki hodi: srednji ten kože in , obrnjen v desno',
    '🚶🏾‍♂‍➡' => 'moški, ki hodi: srednje temen ten kože in , obrnjen v desno',
    '🚶🏿‍♂‍➡' => 'moški, ki hodi: temen ten kože in , obrnjen v desno',
    '🧎🏻‍♀‍➡' => 'klečeča ženska: svetel ten kože in , obrnjen v desno',
    '🧎🏼‍♀‍➡' => 'klečeča ženska: srednje svetel ten kože in , obrnjen v desno',
    '🧎🏽‍♀‍➡' => 'klečeča ženska: srednji ten kože in , obrnjen v desno',
    '🧎🏾‍♀‍➡' => 'klečeča ženska: srednje temen ten kože in , obrnjen v desno',
    '🧎🏿‍♀‍➡' => 'klečeča ženska: temen ten kože in , obrnjen v desno',
    '🧎🏻‍♂‍➡' => 'klečeč moški: svetel ten kože in , obrnjen v desno',
    '🧎🏼‍♂‍➡' => 'klečeč moški: srednje svetel ten kože in , obrnjen v desno',
    '🧎🏽‍♂‍➡' => 'klečeč moški: srednji ten kože in , obrnjen v desno',
    '🧎🏾‍♂‍➡' => 'klečeč moški: srednje temen ten kože in , obrnjen v desno',
    '🧎🏿‍♂‍➡' => 'klečeč moški: temen ten kože in , obrnjen v desno',
    '🧑🏻‍🦯‍➡' => 'oseba s palico za slepe: svetel ten kože in , obrnjen v desno',
    '🧑🏼‍🦯‍➡' => 'oseba s palico za slepe: srednje svetel ten kože in , obrnjen v desno',
    '🧑🏽‍🦯‍➡' => 'oseba s palico za slepe: srednji ten kože in , obrnjen v desno',
    '🧑🏾‍🦯‍➡' => 'oseba s palico za slepe: srednje temen ten kože in , obrnjen v desno',
    '🧑🏿‍🦯‍➡' => 'oseba s palico za slepe: temen ten kože in , obrnjen v desno',
    '👨🏻‍🦯‍➡' => 'moški s palico za slepe: svetel ten kože in , obrnjen v desno',
    '👨🏼‍🦯‍➡' => 'moški s palico za slepe: srednje svetel ten kože in , obrnjen v desno',
    '👨🏽‍🦯‍➡' => 'moški s palico za slepe: srednji ten kože in , obrnjen v desno',
    '👨🏾‍🦯‍➡' => 'moški s palico za slepe: srednje temen ten kože in , obrnjen v desno',
    '👨🏿‍🦯‍➡' => 'moški s palico za slepe: temen ten kože in , obrnjen v desno',
    '👩🏻‍🦯‍➡' => 'ženska s palico za slepe: svetel ten kože in , obrnjen v desno',
    '👩🏼‍🦯‍➡' => 'ženska s palico za slepe: srednje svetel ten kože in , obrnjen v desno',
    '👩🏽‍🦯‍➡' => 'ženska s palico za slepe: srednji ten kože in , obrnjen v desno',
    '👩🏾‍🦯‍➡' => 'ženska s palico za slepe: srednje temen ten kože in , obrnjen v desno',
    '👩🏿‍🦯‍➡' => 'ženska s palico za slepe: temen ten kože in , obrnjen v desno',
    '🧑🏻‍🦼‍➡' => 'oseba na motoriziranem invalidskem vozičku: svetel ten kože in , obrnjen v desno',
    '🧑🏼‍🦼‍➡' => 'oseba na motoriziranem invalidskem vozičku: srednje svetel ten kože in , obrnjen v desno',
    '🧑🏽‍🦼‍➡' => 'oseba na motoriziranem invalidskem vozičku: srednji ten kože in , obrnjen v desno',
    '🧑🏾‍🦼‍➡' => 'oseba na motoriziranem invalidskem vozičku: srednje temen ten kože in , obrnjen v desno',
    '🧑🏿‍🦼‍➡' => 'oseba na motoriziranem invalidskem vozičku: temen ten kože in , obrnjen v desno',
    '👨🏻‍🦼‍➡' => 'moški na motoriziranem invalidskem vozičku: svetel ten kože in , obrnjen v desno',
    '👨🏼‍🦼‍➡' => 'moški na motoriziranem invalidskem vozičku: srednje svetel ten kože in , obrnjen v desno',
    '👨🏽‍🦼‍➡' => 'moški na motoriziranem invalidskem vozičku: srednji ten kože in , obrnjen v desno',
    '👨🏾‍🦼‍➡' => 'moški na motoriziranem invalidskem vozičku: srednje temen ten kože in , obrnjen v desno',
    '👨🏿‍🦼‍➡' => 'moški na motoriziranem invalidskem vozičku: temen ten kože in , obrnjen v desno',
    '👩🏻‍🦼‍➡' => 'ženska na motoriziranem invalidskem vozičku: svetel ten kože in , obrnjen v desno',
    '👩🏼‍🦼‍➡' => 'ženska na motoriziranem invalidskem vozičku: srednje svetel ten kože in , obrnjen v desno',
    '👩🏽‍🦼‍➡' => 'ženska na motoriziranem invalidskem vozičku: srednji ten kože in , obrnjen v desno',
    '👩🏾‍🦼‍➡' => 'ženska na motoriziranem invalidskem vozičku: srednje temen ten kože in , obrnjen v desno',
    '👩🏿‍🦼‍➡' => 'ženska na motoriziranem invalidskem vozičku: temen ten kože in , obrnjen v desno',
    '🧑🏻‍🦽‍➡' => 'oseba na ročnem invalidskem vozičku: svetel ten kože in , obrnjen v desno',
    '🧑🏼‍🦽‍➡' => 'oseba na ročnem invalidskem vozičku: srednje svetel ten kože in , obrnjen v desno',
    '🧑🏽‍🦽‍➡' => 'oseba na ročnem invalidskem vozičku: srednji ten kože in , obrnjen v desno',
    '🧑🏾‍🦽‍➡' => 'oseba na ročnem invalidskem vozičku: srednje temen ten kože in , obrnjen v desno',
    '🧑🏿‍🦽‍➡' => 'oseba na ročnem invalidskem vozičku: temen ten kože in , obrnjen v desno',
    '👨🏻‍🦽‍➡' => 'moški na ročnem invalidskem vozičku: svetel ten kože in , obrnjen v desno',
    '👨🏼‍🦽‍➡' => 'moški na ročnem invalidskem vozičku: srednje svetel ten kože in , obrnjen v desno',
    '👨🏽‍🦽‍➡' => 'moški na ročnem invalidskem vozičku: srednji ten kože in , obrnjen v desno',
    '👨🏾‍🦽‍➡' => 'moški na ročnem invalidskem vozičku: srednje temen ten kože in , obrnjen v desno',
    '👨🏿‍🦽‍➡' => 'moški na ročnem invalidskem vozičku: temen ten kože in , obrnjen v desno',
    '👩🏻‍🦽‍➡' => 'ženska na ročnem invalidskem vozičku: svetel ten kože in , obrnjen v desno',
    '👩🏼‍🦽‍➡' => 'ženska na ročnem invalidskem vozičku: srednje svetel ten kože in , obrnjen v desno',
    '👩🏽‍🦽‍➡' => 'ženska na ročnem invalidskem vozičku: srednji ten kože in , obrnjen v desno',
    '👩🏾‍🦽‍➡' => 'ženska na ročnem invalidskem vozičku: srednje temen ten kože in , obrnjen v desno',
    '👩🏿‍🦽‍➡' => 'ženska na ročnem invalidskem vozičku: temen ten kože in , obrnjen v desno',
    '🏃🏻‍♀‍➡' => 'ženska, ki teče: svetel ten kože in , obrnjen v desno',
    '🏃🏼‍♀‍➡' => 'ženska, ki teče: srednje svetel ten kože in , obrnjen v desno',
    '🏃🏽‍♀‍➡' => 'ženska, ki teče: srednji ten kože in , obrnjen v desno',
    '🏃🏾‍♀‍➡' => 'ženska, ki teče: srednje temen ten kože in , obrnjen v desno',
    '🏃🏿‍♀‍➡' => 'ženska, ki teče: temen ten kože in , obrnjen v desno',
    '🏃🏻‍♂‍➡' => 'moški, ki teče: svetel ten kože in , obrnjen v desno',
    '🏃🏼‍♂‍➡' => 'moški, ki teče: srednje svetel ten kože in , obrnjen v desno',
    '🏃🏽‍♂‍➡' => 'moški, ki teče: srednji ten kože in , obrnjen v desno',
    '🏃🏾‍♂‍➡' => 'moški, ki teče: srednje temen ten kože in , obrnjen v desno',
    '🏃🏿‍♂‍➡' => 'moški, ki teče: temen ten kože in , obrnjen v desno',
    '🫱🏻‍🫲🏼' => 'rokovanje: svetel ten kože in srednje svetel ten kože',
    '🫱🏻‍🫲🏽' => 'rokovanje: svetel ten kože in srednji ten kože',
    '🫱🏻‍🫲🏾' => 'rokovanje: svetel ten kože in srednje temen ten kože',
    '🫱🏻‍🫲🏿' => 'rokovanje: svetel ten kože in temen ten kože',
    '🫱🏼‍🫲🏻' => 'rokovanje: srednje svetel ten kože in svetel ten kože',
    '🫱🏼‍🫲🏽' => 'rokovanje: srednje svetel ten kože in srednji ten kože',
    '🫱🏼‍🫲🏾' => 'rokovanje: srednje svetel ten kože in srednje temen ten kože',
    '🫱🏼‍🫲🏿' => 'rokovanje: srednje svetel ten kože in temen ten kože',
    '🫱🏽‍🫲🏻' => 'rokovanje: srednji ten kože in svetel ten kože',
    '🫱🏽‍🫲🏼' => 'rokovanje: srednji ten kože in srednje svetel ten kože',
    '🫱🏽‍🫲🏾' => 'rokovanje: srednji ten kože in srednje temen ten kože',
    '🫱🏽‍🫲🏿' => 'rokovanje: srednji ten kože in temen ten kože',
    '🫱🏾‍🫲🏻' => 'rokovanje: srednje temen ten kože in svetel ten kože',
    '🫱🏾‍🫲🏼' => 'rokovanje: srednje temen ten kože in srednje svetel ten kože',
    '🫱🏾‍🫲🏽' => 'rokovanje: srednje temen ten kože in srednji ten kože',
    '🫱🏾‍🫲🏿' => 'rokovanje: srednje temen ten kože in temen ten kože',
    '🫱🏿‍🫲🏻' => 'rokovanje: temen ten kože in svetel ten kože',
    '🫱🏿‍🫲🏼' => 'rokovanje: temen ten kože in srednje svetel ten kože',
    '🫱🏿‍🫲🏽' => 'rokovanje: temen ten kože in srednji ten kože',
    '🫱🏿‍🫲🏾' => 'rokovanje: temen ten kože in srednje temen ten kože',
    '🚶‍♀‍➡' => 'ženska, ki hodi: , obrnjen v desno',
    '🚶‍♂‍➡' => 'moški, ki hodi: , obrnjen v desno',
    '🧎‍♀‍➡' => 'klečeča ženska: , obrnjen v desno',
    '🧎‍♂‍➡' => 'klečeč moški: , obrnjen v desno',
    '🧑‍🦯‍➡' => 'oseba s palico za slepe: , obrnjen v desno',
    '👨‍🦯‍➡' => 'moški s palico za slepe: , obrnjen v desno',
    '👩‍🦯‍➡' => 'ženska s palico za slepe: , obrnjen v desno',
    '🧑‍🦼‍➡' => 'oseba na motoriziranem invalidskem vozičku: , obrnjen v desno',
    '👨‍🦼‍➡' => 'moški na motoriziranem invalidskem vozičku: , obrnjen v desno',
    '👩‍🦼‍➡' => 'ženska na motoriziranem invalidskem vozičku: , obrnjen v desno',
    '🧑‍🦽‍➡' => 'oseba na ročnem invalidskem vozičku: , obrnjen v desno',
    '👨‍🦽‍➡' => 'moški na ročnem invalidskem vozičku: , obrnjen v desno',
    '👩‍🦽‍➡' => 'ženska na ročnem invalidskem vozičku: , obrnjen v desno',
    '🏃‍♀‍➡' => 'ženska, ki teče: , obrnjen v desno',
    '🏃‍♂‍➡' => 'moški, ki teče: , obrnjen v desno',
    '👩‍❤‍👨' => 'par s srčkom: ženska in moški',
    '👨‍❤‍👨' => 'par s srčkom: moški in moški',
    '👩‍❤‍👩' => 'par s srčkom: ženska in ženska',
    '👨‍👩‍👦' => 'družina: moški, ženska in fant',
    '👨‍👩‍👧' => 'družina: moški, ženska in dekle',
    '👨‍👨‍👦' => 'družina: moški, moški in fant',
    '👨‍👨‍👧' => 'družina: moški, moški in dekle',
    '👩‍👩‍👦' => 'družina: ženska, ženska in fant',
    '👩‍👩‍👧' => 'družina: ženska, ženska in dekle',
    '👨‍👦‍👦' => 'družina: moški, fant in fant',
    '👨‍👧‍👦' => 'družina: moški, dekle in fant',
    '👨‍👧‍👧' => 'družina: moški, dekle in dekle',
    '👩‍👦‍👦' => 'družina: ženska, fant in fant',
    '👩‍👧‍👦' => 'družina: ženska, dekle in fant',
    '👩‍👧‍👧' => 'družina: ženska, dekle in dekle',
    '🧑‍🤝‍🧑' => 'osebi se držita za roke',
    '🧑‍🧑‍🧒' => 'družina: odrasel, odrasel, otrok',
    '🧑‍🧒‍🧒' => 'družina: odrasla oseba, otrok, otrok',
    '🧑🏻‍🦰' => 'odrasla oseba: svetel ten kože in rdeči lasje',
    '🧑🏼‍🦰' => 'odrasla oseba: srednje svetel ten kože in rdeči lasje',
    '🧑🏽‍🦰' => 'odrasla oseba: srednji ten kože in rdeči lasje',
    '🧑🏾‍🦰' => 'odrasla oseba: srednje temen ten kože in rdeči lasje',
    '🧑🏿‍🦰' => 'odrasla oseba: temen ten kože in rdeči lasje',
    '🧑🏻‍🦱' => 'odrasla oseba: svetel ten kože in kodrasti lasje',
    '🧑🏼‍🦱' => 'odrasla oseba: srednje svetel ten kože in kodrasti lasje',
    '🧑🏽‍🦱' => 'odrasla oseba: srednji ten kože in kodrasti lasje',
    '🧑🏾‍🦱' => 'odrasla oseba: srednje temen ten kože in kodrasti lasje',
    '🧑🏿‍🦱' => 'odrasla oseba: temen ten kože in kodrasti lasje',
    '🧑🏻‍🦳' => 'odrasla oseba: svetel ten kože in sivi lasje',
    '🧑🏼‍🦳' => 'odrasla oseba: srednje svetel ten kože in sivi lasje',
    '🧑🏽‍🦳' => 'odrasla oseba: srednji ten kože in sivi lasje',
    '🧑🏾‍🦳' => 'odrasla oseba: srednje temen ten kože in sivi lasje',
    '🧑🏿‍🦳' => 'odrasla oseba: temen ten kože in sivi lasje',
    '🧑🏻‍🦲' => 'odrasla oseba: svetel ten kože in plešast',
    '🧑🏼‍🦲' => 'odrasla oseba: srednje svetel ten kože in plešast',
    '🧑🏽‍🦲' => 'odrasla oseba: srednji ten kože in plešast',
    '🧑🏾‍🦲' => 'odrasla oseba: srednje temen ten kože in plešast',
    '🧑🏿‍🦲' => 'odrasla oseba: temen ten kože in plešast',
    '🧑🏻‍🩰' => 'baletka: svetel ten kože',
    '🧑🏼‍🩰' => 'baletka: srednje svetel ten kože',
    '🧑🏽‍🩰' => 'baletka: srednji ten kože',
    '🧑🏾‍🩰' => 'baletka: srednje temen ten kože',
    '🧑🏿‍🩰' => 'baletka: temen ten kože',
    '🧔🏻‍♂' => 'moški: svetel ten kože in brada',
    '🧔🏼‍♂' => 'moški: srednje svetel ten kože in brada',
    '🧔🏽‍♂' => 'moški: srednji ten kože in brada',
    '🧔🏾‍♂' => 'moški: srednje temen ten kože in brada',
    '🧔🏿‍♂' => 'moški: temen ten kože in brada',
    '👱🏻‍♂' => 'moški: svetel ten kože in blondinec',
    '👱🏼‍♂' => 'moški: srednje svetel ten kože in blondinec',
    '👱🏽‍♂' => 'moški: srednji ten kože in blondinec',
    '👱🏾‍♂' => 'moški: srednje temen ten kože in blondinec',
    '👱🏿‍♂' => 'moški: temen ten kože in blondinec',
    '👨🏻‍🦰' => 'moški: svetel ten kože in rdeči lasje',
    '👨🏼‍🦰' => 'moški: srednje svetel ten kože in rdeči lasje',
    '👨🏽‍🦰' => 'moški: srednji ten kože in rdeči lasje',
    '👨🏾‍🦰' => 'moški: srednje temen ten kože in rdeči lasje',
    '👨🏿‍🦰' => 'moški: temen ten kože in rdeči lasje',
    '👨🏻‍🦱' => 'moški: svetel ten kože in kodrasti lasje',
    '👨🏼‍🦱' => 'moški: srednje svetel ten kože in kodrasti lasje',
    '👨🏽‍🦱' => 'moški: srednji ten kože in kodrasti lasje',
    '👨🏾‍🦱' => 'moški: srednje temen ten kože in kodrasti lasje',
    '👨🏿‍🦱' => 'moški: temen ten kože in kodrasti lasje',
    '👨🏻‍🦳' => 'moški: svetel ten kože in sivi lasje',
    '👨🏼‍🦳' => 'moški: srednje svetel ten kože in sivi lasje',
    '👨🏽‍🦳' => 'moški: srednji ten kože in sivi lasje',
    '👨🏾‍🦳' => 'moški: srednje temen ten kože in sivi lasje',
    '👨🏿‍🦳' => 'moški: temen ten kože in sivi lasje',
    '👨🏻‍🦲' => 'moški: svetel ten kože in plešast',
    '👨🏼‍🦲' => 'moški: srednje svetel ten kože in plešast',
    '👨🏽‍🦲' => 'moški: srednji ten kože in plešast',
    '👨🏾‍🦲' => 'moški: srednje temen ten kože in plešast',
    '👨🏿‍🦲' => 'moški: temen ten kože in plešast',
    '🧔🏻‍♀' => 'ženska: svetel ten kože in brada',
    '🧔🏼‍♀' => 'ženska: srednje svetel ten kože in brada',
    '🧔🏽‍♀' => 'ženska: srednji ten kože in brada',
    '🧔🏾‍♀' => 'ženska: srednje temen ten kože in brada',
    '🧔🏿‍♀' => 'ženska: temen ten kože in brada',
    '👱🏻‍♀' => 'ženska: svetel ten kože in blondinka',
    '👱🏼‍♀' => 'ženska: srednje svetel ten kože in blondinka',
    '👱🏽‍♀' => 'ženska: srednji ten kože in blondinka',
    '👱🏾‍♀' => 'ženska: srednje temen ten kože in blondinka',
    '👱🏿‍♀' => 'ženska: temen ten kože in blondinka',
    '👩🏻‍🦰' => 'ženska: svetel ten kože in rdeči lasje',
    '👩🏼‍🦰' => 'ženska: srednje svetel ten kože in rdeči lasje',
    '👩🏽‍🦰' => 'ženska: srednji ten kože in rdeči lasje',
    '👩🏾‍🦰' => 'ženska: srednje temen ten kože in rdeči lasje',
    '👩🏿‍🦰' => 'ženska: temen ten kože in rdeči lasje',
    '👩🏻‍🦱' => 'ženska: svetel ten kože in kodrasti lasje',
    '👩🏼‍🦱' => 'ženska: srednje svetel ten kože in kodrasti lasje',
    '👩🏽‍🦱' => 'ženska: srednji ten kože in kodrasti lasje',
    '👩🏾‍🦱' => 'ženska: srednje temen ten kože in kodrasti lasje',
    '👩🏿‍🦱' => 'ženska: temen ten kože in kodrasti lasje',
    '👩🏻‍🦳' => 'ženska: svetel ten kože in sivi lasje',
    '👩🏼‍🦳' => 'ženska: srednje svetel ten kože in sivi lasje',
    '👩🏽‍🦳' => 'ženska: srednji ten kože in sivi lasje',
    '👩🏾‍🦳' => 'ženska: srednje temen ten kože in sivi lasje',
    '👩🏿‍🦳' => 'ženska: temen ten kože in sivi lasje',
    '👩🏻‍🦲' => 'ženska: svetel ten kože in plešast',
    '👩🏼‍🦲' => 'ženska: srednje svetel ten kože in plešast',
    '👩🏽‍🦲' => 'ženska: srednji ten kože in plešast',
    '👩🏾‍🦲' => 'ženska: srednje temen ten kože in plešast',
    '👩🏿‍🦲' => 'ženska: temen ten kože in plešast',
    '🙍🏻‍♂' => 'moški z namrščenim obrazom: svetel ten kože',
    '🙍🏼‍♂' => 'moški z namrščenim obrazom: srednje svetel ten kože',
    '🙍🏽‍♂' => 'moški z namrščenim obrazom: srednji ten kože',
    '🙍🏾‍♂' => 'moški z namrščenim obrazom: srednje temen ten kože',
    '🙍🏿‍♂' => 'moški z namrščenim obrazom: temen ten kože',
    '🙍🏻‍♀' => 'ženska z namrščenim obrazom: svetel ten kože',
    '🙍🏼‍♀' => 'ženska z namrščenim obrazom: srednje svetel ten kože',
    '🙍🏽‍♀' => 'ženska z namrščenim obrazom: srednji ten kože',
    '🙍🏾‍♀' => 'ženska z namrščenim obrazom: srednje temen ten kože',
    '🙍🏿‍♀' => 'ženska z namrščenim obrazom: temen ten kože',
    '🙎🏻‍♂' => 'moški, ki kuha mulo: svetel ten kože',
    '🙎🏼‍♂' => 'moški, ki kuha mulo: srednje svetel ten kože',
    '🙎🏽‍♂' => 'moški, ki kuha mulo: srednji ten kože',
    '🙎🏾‍♂' => 'moški, ki kuha mulo: srednje temen ten kože',
    '🙎🏿‍♂' => 'moški, ki kuha mulo: temen ten kože',
    '🙎🏻‍♀' => 'ženska, ki kuha mulo: svetel ten kože',
    '🙎🏼‍♀' => 'ženska, ki kuha mulo: srednje svetel ten kože',
    '🙎🏽‍♀' => 'ženska, ki kuha mulo: srednji ten kože',
    '🙎🏾‍♀' => 'ženska, ki kuha mulo: srednje temen ten kože',
    '🙎🏿‍♀' => 'ženska, ki kuha mulo: temen ten kože',
    '🙅🏻‍♂' => 'moški, ki s kretnjo izraža prepoved: svetel ten kože',
    '🙅🏼‍♂' => 'moški, ki s kretnjo izraža prepoved: srednje svetel ten kože',
    '🙅🏽‍♂' => 'moški, ki s kretnjo izraža prepoved: srednji ten kože',
    '🙅🏾‍♂' => 'moški, ki s kretnjo izraža prepoved: srednje temen ten kože',
    '🙅🏿‍♂' => 'moški, ki s kretnjo izraža prepoved: temen ten kože',
    '🙅🏻‍♀' => 'ženska, ki s kretnjo izraža prepoved: svetel ten kože',
    '🙅🏼‍♀' => 'ženska, ki s kretnjo izraža prepoved: srednje svetel ten kože',
    '🙅🏽‍♀' => 'ženska, ki s kretnjo izraža prepoved: srednji ten kože',
    '🙅🏾‍♀' => 'ženska, ki s kretnjo izraža prepoved: srednje temen ten kože',
    '🙅🏿‍♀' => 'ženska, ki s kretnjo izraža prepoved: temen ten kože',
    '🙆🏻‍♂' => 'moški, ki s kretnjo izraža odobravanje: svetel ten kože',
    '🙆🏼‍♂' => 'moški, ki s kretnjo izraža odobravanje: srednje svetel ten kože',
    '🙆🏽‍♂' => 'moški, ki s kretnjo izraža odobravanje: srednji ten kože',
    '🙆🏾‍♂' => 'moški, ki s kretnjo izraža odobravanje: srednje temen ten kože',
    '🙆🏿‍♂' => 'moški, ki s kretnjo izraža odobravanje: temen ten kože',
    '🙆🏻‍♀' => 'ženska, ki s kretnjo izraža odobravanje: svetel ten kože',
    '🙆🏼‍♀' => 'ženska, ki s kretnjo izraža odobravanje: srednje svetel ten kože',
    '🙆🏽‍♀' => 'ženska, ki s kretnjo izraža odobravanje: srednji ten kože',
    '🙆🏾‍♀' => 'ženska, ki s kretnjo izraža odobravanje: srednje temen ten kože',
    '🙆🏿‍♀' => 'ženska, ki s kretnjo izraža odobravanje: temen ten kože',
    '💁🏻‍♂' => 'uslužbenec na informacijah: svetel ten kože',
    '💁🏼‍♂' => 'uslužbenec na informacijah: srednje svetel ten kože',
    '💁🏽‍♂' => 'uslužbenec na informacijah: srednji ten kože',
    '💁🏾‍♂' => 'uslužbenec na informacijah: srednje temen ten kože',
    '💁🏿‍♂' => 'uslužbenec na informacijah: temen ten kože',
    '💁🏻‍♀' => 'uslužbenka na informacijah: svetel ten kože',
    '💁🏼‍♀' => 'uslužbenka na informacijah: srednje svetel ten kože',
    '💁🏽‍♀' => 'uslužbenka na informacijah: srednji ten kože',
    '💁🏾‍♀' => 'uslužbenka na informacijah: srednje temen ten kože',
    '💁🏿‍♀' => 'uslužbenka na informacijah: temen ten kože',
    '🙋🏻‍♂' => 'moški, ki dviguje roko: svetel ten kože',
    '🙋🏼‍♂' => 'moški, ki dviguje roko: srednje svetel ten kože',
    '🙋🏽‍♂' => 'moški, ki dviguje roko: srednji ten kože',
    '🙋🏾‍♂' => 'moški, ki dviguje roko: srednje temen ten kože',
    '🙋🏿‍♂' => 'moški, ki dviguje roko: temen ten kože',
    '🙋🏻‍♀' => 'ženska, ki dviguje roko: svetel ten kože',
    '🙋🏼‍♀' => 'ženska, ki dviguje roko: srednje svetel ten kože',
    '🙋🏽‍♀' => 'ženska, ki dviguje roko: srednji ten kože',
    '🙋🏾‍♀' => 'ženska, ki dviguje roko: srednje temen ten kože',
    '🙋🏿‍♀' => 'ženska, ki dviguje roko: temen ten kože',
    '🧏🏻‍♂' => 'gluh moški: svetel ten kože',
    '🧏🏼‍♂' => 'gluh moški: srednje svetel ten kože',
    '🧏🏽‍♂' => 'gluh moški: srednji ten kože',
    '🧏🏾‍♂' => 'gluh moški: srednje temen ten kože',
    '🧏🏿‍♂' => 'gluh moški: temen ten kože',
    '🧏🏻‍♀' => 'gluha ženska: svetel ten kože',
    '🧏🏼‍♀' => 'gluha ženska: srednje svetel ten kože',
    '🧏🏽‍♀' => 'gluha ženska: srednji ten kože',
    '🧏🏾‍♀' => 'gluha ženska: srednje temen ten kože',
    '🧏🏿‍♀' => 'gluha ženska: temen ten kože',
    '🙇🏻‍♂' => 'moški, ki se globoko priklanja: svetel ten kože',
    '🙇🏼‍♂' => 'moški, ki se globoko priklanja: srednje svetel ten kože',
    '🙇🏽‍♂' => 'moški, ki se globoko priklanja: srednji ten kože',
    '🙇🏾‍♂' => 'moški, ki se globoko priklanja: srednje temen ten kože',
    '🙇🏿‍♂' => 'moški, ki se globoko priklanja: temen ten kože',
    '🙇🏻‍♀' => 'ženska, ki se globoko priklanja: svetel ten kože',
    '🙇🏼‍♀' => 'ženska, ki se globoko priklanja: srednje svetel ten kože',
    '🙇🏽‍♀' => 'ženska, ki se globoko priklanja: srednji ten kože',
    '🙇🏾‍♀' => 'ženska, ki se globoko priklanja: srednje temen ten kože',
    '🙇🏿‍♀' => 'ženska, ki se globoko priklanja: temen ten kože',
    '🤦🏻‍♂' => 'moški, ki obraz prekriva z dlanjo: svetel ten kože',
    '🤦🏼‍♂' => 'moški, ki obraz prekriva z dlanjo: srednje svetel ten kože',
    '🤦🏽‍♂' => 'moški, ki obraz prekriva z dlanjo: srednji ten kože',
    '🤦🏾‍♂' => 'moški, ki obraz prekriva z dlanjo: srednje temen ten kože',
    '🤦🏿‍♂' => 'moški, ki obraz prekriva z dlanjo: temen ten kože',
    '🤦🏻‍♀' => 'ženska, ki obraz prekriva z dlanjo: svetel ten kože',
    '🤦🏼‍♀' => 'ženska, ki obraz prekriva z dlanjo: srednje svetel ten kože',
    '🤦🏽‍♀' => 'ženska, ki obraz prekriva z dlanjo: srednji ten kože',
    '🤦🏾‍♀' => 'ženska, ki obraz prekriva z dlanjo: srednje temen ten kože',
    '🤦🏿‍♀' => 'ženska, ki obraz prekriva z dlanjo: temen ten kože',
    '🤷🏻‍♂' => 'moški, ki skomiga z rameni: svetel ten kože',
    '🤷🏼‍♂' => 'moški, ki skomiga z rameni: srednje svetel ten kože',
    '🤷🏽‍♂' => 'moški, ki skomiga z rameni: srednji ten kože',
    '🤷🏾‍♂' => 'moški, ki skomiga z rameni: srednje temen ten kože',
    '🤷🏿‍♂' => 'moški, ki skomiga z rameni: temen ten kože',
    '🤷🏻‍♀' => 'ženska, ki skomiga z rameni: svetel ten kože',
    '🤷🏼‍♀' => 'ženska, ki skomiga z rameni: srednje svetel ten kože',
    '🤷🏽‍♀' => 'ženska, ki skomiga z rameni: srednji ten kože',
    '🤷🏾‍♀' => 'ženska, ki skomiga z rameni: srednje temen ten kože',
    '🤷🏿‍♀' => 'ženska, ki skomiga z rameni: temen ten kože',
    '🧑🏻‍⚕' => 'delavec v zdravstvu: svetel ten kože',
    '🧑🏼‍⚕' => 'delavec v zdravstvu: srednje svetel ten kože',
    '🧑🏽‍⚕' => 'delavec v zdravstvu: srednji ten kože',
    '🧑🏾‍⚕' => 'delavec v zdravstvu: srednje temen ten kože',
    '🧑🏿‍⚕' => 'delavec v zdravstvu: temen ten kože',
    '👨🏻‍⚕' => 'zdravstveni delavec: svetel ten kože',
    '👨🏼‍⚕' => 'zdravstveni delavec: srednje svetel ten kože',
    '👨🏽‍⚕' => 'zdravstveni delavec: srednji ten kože',
    '👨🏾‍⚕' => 'zdravstveni delavec: srednje temen ten kože',
    '👨🏿‍⚕' => 'zdravstveni delavec: temen ten kože',
    '👩🏻‍⚕' => 'zdravstvena delavka: svetel ten kože',
    '👩🏼‍⚕' => 'zdravstvena delavka: srednje svetel ten kože',
    '👩🏽‍⚕' => 'zdravstvena delavka: srednji ten kože',
    '👩🏾‍⚕' => 'zdravstvena delavka: srednje temen ten kože',
    '👩🏿‍⚕' => 'zdravstvena delavka: temen ten kože',
    '🧑🏻‍🎓' => 'diplomant: svetel ten kože',
    '🧑🏼‍🎓' => 'diplomant: srednje svetel ten kože',
    '🧑🏽‍🎓' => 'diplomant: srednji ten kože',
    '🧑🏾‍🎓' => 'diplomant: srednje temen ten kože',
    '🧑🏿‍🎓' => 'diplomant: temen ten kože',
    '👨🏻‍🎓' => 'študent: svetel ten kože',
    '👨🏼‍🎓' => 'študent: srednje svetel ten kože',
    '👨🏽‍🎓' => 'študent: srednji ten kože',
    '👨🏾‍🎓' => 'študent: srednje temen ten kože',
    '👨🏿‍🎓' => 'študent: temen ten kože',
    '👩🏻‍🎓' => 'študentka: svetel ten kože',
    '👩🏼‍🎓' => 'študentka: srednje svetel ten kože',
    '👩🏽‍🎓' => 'študentka: srednji ten kože',
    '👩🏾‍🎓' => 'študentka: srednje temen ten kože',
    '👩🏿‍🎓' => 'študentka: temen ten kože',
    '🧑🏻‍🏫' => 'profesor: svetel ten kože',
    '🧑🏼‍🏫' => 'profesor: srednje svetel ten kože',
    '🧑🏽‍🏫' => 'profesor: srednji ten kože',
    '🧑🏾‍🏫' => 'profesor: srednje temen ten kože',
    '🧑🏿‍🏫' => 'profesor: temen ten kože',
    '👨🏻‍🏫' => 'učitelj: svetel ten kože',
    '👨🏼‍🏫' => 'učitelj: srednje svetel ten kože',
    '👨🏽‍🏫' => 'učitelj: srednji ten kože',
    '👨🏾‍🏫' => 'učitelj: srednje temen ten kože',
    '👨🏿‍🏫' => 'učitelj: temen ten kože',
    '👩🏻‍🏫' => 'učiteljica: svetel ten kože',
    '👩🏼‍🏫' => 'učiteljica: srednje svetel ten kože',
    '👩🏽‍🏫' => 'učiteljica: srednji ten kože',
    '👩🏾‍🏫' => 'učiteljica: srednje temen ten kože',
    '👩🏿‍🏫' => 'učiteljica: temen ten kože',
    '🧑🏻‍⚖' => 'varuh zakona: svetel ten kože',
    '🧑🏼‍⚖' => 'varuh zakona: srednje svetel ten kože',
    '🧑🏽‍⚖' => 'varuh zakona: srednji ten kože',
    '🧑🏾‍⚖' => 'varuh zakona: srednje temen ten kože',
    '🧑🏿‍⚖' => 'varuh zakona: temen ten kože',
    '👨🏻‍⚖' => 'sodnik: svetel ten kože',
    '👨🏼‍⚖' => 'sodnik: srednje svetel ten kože',
    '👨🏽‍⚖' => 'sodnik: srednji ten kože',
    '👨🏾‍⚖' => 'sodnik: srednje temen ten kože',
    '👨🏿‍⚖' => 'sodnik: temen ten kože',
    '👩🏻‍⚖' => 'sodnica: svetel ten kože',
    '👩🏼‍⚖' => 'sodnica: srednje svetel ten kože',
    '👩🏽‍⚖' => 'sodnica: srednji ten kože',
    '👩🏾‍⚖' => 'sodnica: srednje temen ten kože',
    '👩🏿‍⚖' => 'sodnica: temen ten kože',
    '🧑🏻‍🌾' => 'kmet: svetel ten kože',
    '🧑🏼‍🌾' => 'kmet: srednje svetel ten kože',
    '🧑🏽‍🌾' => 'kmet: srednji ten kože',
    '🧑🏾‍🌾' => 'kmet: srednje temen ten kože',
    '🧑🏿‍🌾' => 'kmet: temen ten kože',
    '👨🏻‍🌾' => 'kmetovalec: svetel ten kože',
    '👨🏼‍🌾' => 'kmetovalec: srednje svetel ten kože',
    '👨🏽‍🌾' => 'kmetovalec: srednji ten kože',
    '👨🏾‍🌾' => 'kmetovalec: srednje temen ten kože',
    '👨🏿‍🌾' => 'kmetovalec: temen ten kože',
    '👩🏻‍🌾' => 'kmetovalka: svetel ten kože',
    '👩🏼‍🌾' => 'kmetovalka: srednje svetel ten kože',
    '👩🏽‍🌾' => 'kmetovalka: srednji ten kože',
    '👩🏾‍🌾' => 'kmetovalka: srednje temen ten kože',
    '👩🏿‍🌾' => 'kmetovalka: temen ten kože',
    '🧑🏻‍🍳' => 'kuharski mojster: svetel ten kože',
    '🧑🏼‍🍳' => 'kuharski mojster: srednje svetel ten kože',
    '🧑🏽‍🍳' => 'kuharski mojster: srednji ten kože',
    '🧑🏾‍🍳' => 'kuharski mojster: srednje temen ten kože',
    '🧑🏿‍🍳' => 'kuharski mojster: temen ten kože',
    '👨🏻‍🍳' => 'kuhar: svetel ten kože',
    '👨🏼‍🍳' => 'kuhar: srednje svetel ten kože',
    '👨🏽‍🍳' => 'kuhar: srednji ten kože',
    '👨🏾‍🍳' => 'kuhar: srednje temen ten kože',
    '👨🏿‍🍳' => 'kuhar: temen ten kože',
    '👩🏻‍🍳' => 'kuharica: svetel ten kože',
    '👩🏼‍🍳' => 'kuharica: srednje svetel ten kože',
    '👩🏽‍🍳' => 'kuharica: srednji ten kože',
    '👩🏾‍🍳' => 'kuharica: srednje temen ten kože',
    '👩🏿‍🍳' => 'kuharica: temen ten kože',
    '🧑🏻‍🔧' => 'serviser: svetel ten kože',
    '🧑🏼‍🔧' => 'serviser: srednje svetel ten kože',
    '🧑🏽‍🔧' => 'serviser: srednji ten kože',
    '🧑🏾‍🔧' => 'serviser: srednje temen ten kože',
    '🧑🏿‍🔧' => 'serviser: temen ten kože',
    '👨🏻‍🔧' => 'mehanik: svetel ten kože',
    '👨🏼‍🔧' => 'mehanik: srednje svetel ten kože',
    '👨🏽‍🔧' => 'mehanik: srednji ten kože',
    '👨🏾‍🔧' => 'mehanik: srednje temen ten kože',
    '👨🏿‍🔧' => 'mehanik: temen ten kože',
    '👩🏻‍🔧' => 'mehaničarka: svetel ten kože',
    '👩🏼‍🔧' => 'mehaničarka: srednje svetel ten kože',
    '👩🏽‍🔧' => 'mehaničarka: srednji ten kože',
    '👩🏾‍🔧' => 'mehaničarka: srednje temen ten kože',
    '👩🏿‍🔧' => 'mehaničarka: temen ten kože',
    '🧑🏻‍🏭' => 'tovarniški delavec: svetel ten kože',
    '🧑🏼‍🏭' => 'tovarniški delavec: srednje svetel ten kože',
    '🧑🏽‍🏭' => 'tovarniški delavec: srednji ten kože',
    '🧑🏾‍🏭' => 'tovarniški delavec: srednje temen ten kože',
    '🧑🏿‍🏭' => 'tovarniški delavec: temen ten kože',
    '👨🏻‍🏭' => 'delavec v tovarni: svetel ten kože',
    '👨🏼‍🏭' => 'delavec v tovarni: srednje svetel ten kože',
    '👨🏽‍🏭' => 'delavec v tovarni: srednji ten kože',
    '👨🏾‍🏭' => 'delavec v tovarni: srednje temen ten kože',
    '👨🏿‍🏭' => 'delavec v tovarni: temen ten kože',
    '👩🏻‍🏭' => 'delavka v tovarni: svetel ten kože',
    '👩🏼‍🏭' => 'delavka v tovarni: srednje svetel ten kože',
    '👩🏽‍🏭' => 'delavka v tovarni: srednji ten kože',
    '👩🏾‍🏭' => 'delavka v tovarni: srednje temen ten kože',
    '👩🏿‍🏭' => 'delavka v tovarni: temen ten kože',
    '🧑🏻‍💼' => 'delavec v pisarni: svetel ten kože',
    '🧑🏼‍💼' => 'delavec v pisarni: srednje svetel ten kože',
    '🧑🏽‍💼' => 'delavec v pisarni: srednji ten kože',
    '🧑🏾‍💼' => 'delavec v pisarni: srednje temen ten kože',
    '🧑🏿‍💼' => 'delavec v pisarni: temen ten kože',
    '👨🏻‍💼' => 'pisarniški delavec: svetel ten kože',
    '👨🏼‍💼' => 'pisarniški delavec: srednje svetel ten kože',
    '👨🏽‍💼' => 'pisarniški delavec: srednji ten kože',
    '👨🏾‍💼' => 'pisarniški delavec: srednje temen ten kože',
    '👨🏿‍💼' => 'pisarniški delavec: temen ten kože',
    '👩🏻‍💼' => 'pisarniška delavka: svetel ten kože',
    '👩🏼‍💼' => 'pisarniška delavka: srednje svetel ten kože',
    '👩🏽‍💼' => 'pisarniška delavka: srednji ten kože',
    '👩🏾‍💼' => 'pisarniška delavka: srednje temen ten kože',
    '👩🏿‍💼' => 'pisarniška delavka: temen ten kože',
    '🧑🏻‍🔬' => 'raziskovalec: svetel ten kože',
    '🧑🏼‍🔬' => 'raziskovalec: srednje svetel ten kože',
    '🧑🏽‍🔬' => 'raziskovalec: srednji ten kože',
    '🧑🏾‍🔬' => 'raziskovalec: srednje temen ten kože',
    '🧑🏿‍🔬' => 'raziskovalec: temen ten kože',
    '👨🏻‍🔬' => 'znanstvenik: svetel ten kože',
    '👨🏼‍🔬' => 'znanstvenik: srednje svetel ten kože',
    '👨🏽‍🔬' => 'znanstvenik: srednji ten kože',
    '👨🏾‍🔬' => 'znanstvenik: srednje temen ten kože',
    '👨🏿‍🔬' => 'znanstvenik: temen ten kože',
    '👩🏻‍🔬' => 'znanstvenica: svetel ten kože',
    '👩🏼‍🔬' => 'znanstvenica: srednje svetel ten kože',
    '👩🏽‍🔬' => 'znanstvenica: srednji ten kože',
    '👩🏾‍🔬' => 'znanstvenica: srednje temen ten kože',
    '👩🏿‍🔬' => 'znanstvenica: temen ten kože',
    '🧑🏻‍💻' => 'strokovnjak za tehnologijo: svetel ten kože',
    '🧑🏼‍💻' => 'strokovnjak za tehnologijo: srednje svetel ten kože',
    '🧑🏽‍💻' => 'strokovnjak za tehnologijo: srednji ten kože',
    '🧑🏾‍💻' => 'strokovnjak za tehnologijo: srednje temen ten kože',
    '🧑🏿‍💻' => 'strokovnjak za tehnologijo: temen ten kože',
    '👨🏻‍💻' => 'tehnolog: svetel ten kože',
    '👨🏼‍💻' => 'tehnolog: srednje svetel ten kože',
    '👨🏽‍💻' => 'tehnolog: srednji ten kože',
    '👨🏾‍💻' => 'tehnolog: srednje temen ten kože',
    '👨🏿‍💻' => 'tehnolog: temen ten kože',
    '👩🏻‍💻' => 'tehnologinja: svetel ten kože',
    '👩🏼‍💻' => 'tehnologinja: srednje svetel ten kože',
    '👩🏽‍💻' => 'tehnologinja: srednji ten kože',
    '👩🏾‍💻' => 'tehnologinja: srednje temen ten kože',
    '👩🏿‍💻' => 'tehnologinja: temen ten kože',
    '🧑🏻‍🎤' => 'izvajalec: svetel ten kože',
    '🧑🏼‍🎤' => 'izvajalec: srednje svetel ten kože',
    '🧑🏽‍🎤' => 'izvajalec: srednji ten kože',
    '🧑🏾‍🎤' => 'izvajalec: srednje temen ten kože',
    '🧑🏿‍🎤' => 'izvajalec: temen ten kože',
    '👨🏻‍🎤' => 'pevec: svetel ten kože',
    '👨🏼‍🎤' => 'pevec: srednje svetel ten kože',
    '👨🏽‍🎤' => 'pevec: srednji ten kože',
    '👨🏾‍🎤' => 'pevec: srednje temen ten kože',
    '👨🏿‍🎤' => 'pevec: temen ten kože',
    '👩🏻‍🎤' => 'pevka: svetel ten kože',
    '👩🏼‍🎤' => 'pevka: srednje svetel ten kože',
    '👩🏽‍🎤' => 'pevka: srednji ten kože',
    '👩🏾‍🎤' => 'pevka: srednje temen ten kože',
    '👩🏿‍🎤' => 'pevka: temen ten kože',
    '🧑🏻‍🎨' => 'slikar: svetel ten kože',
    '🧑🏼‍🎨' => 'slikar: srednje svetel ten kože',
    '🧑🏽‍🎨' => 'slikar: srednji ten kože',
    '🧑🏾‍🎨' => 'slikar: srednje temen ten kože',
    '🧑🏿‍🎨' => 'slikar: temen ten kože',
    '👨🏻‍🎨' => 'umetnik: svetel ten kože',
    '👨🏼‍🎨' => 'umetnik: srednje svetel ten kože',
    '👨🏽‍🎨' => 'umetnik: srednji ten kože',
    '👨🏾‍🎨' => 'umetnik: srednje temen ten kože',
    '👨🏿‍🎨' => 'umetnik: temen ten kože',
    '👩🏻‍🎨' => 'umetnica: svetel ten kože',
    '👩🏼‍🎨' => 'umetnica: srednje svetel ten kože',
    '👩🏽‍🎨' => 'umetnica: srednji ten kože',
    '👩🏾‍🎨' => 'umetnica: srednje temen ten kože',
    '👩🏿‍🎨' => 'umetnica: temen ten kože',
    '🧑🏻‍✈' => 'pilot letala: svetel ten kože',
    '🧑🏼‍✈' => 'pilot letala: srednje svetel ten kože',
    '🧑🏽‍✈' => 'pilot letala: srednji ten kože',
    '🧑🏾‍✈' => 'pilot letala: srednje temen ten kože',
    '🧑🏿‍✈' => 'pilot letala: temen ten kože',
    '👨🏻‍✈' => 'pilot: svetel ten kože',
    '👨🏼‍✈' => 'pilot: srednje svetel ten kože',
    '👨🏽‍✈' => 'pilot: srednji ten kože',
    '👨🏾‍✈' => 'pilot: srednje temen ten kože',
    '👨🏿‍✈' => 'pilot: temen ten kože',
    '👩🏻‍✈' => 'pilotka: svetel ten kože',
    '👩🏼‍✈' => 'pilotka: srednje svetel ten kože',
    '👩🏽‍✈' => 'pilotka: srednji ten kože',
    '👩🏾‍✈' => 'pilotka: srednje temen ten kože',
    '👩🏿‍✈' => 'pilotka: temen ten kože',
    '🧑🏻‍🚀' => 'kozmonavt: svetel ten kože',
    '🧑🏼‍🚀' => 'kozmonavt: srednje svetel ten kože',
    '🧑🏽‍🚀' => 'kozmonavt: srednji ten kože',
    '🧑🏾‍🚀' => 'kozmonavt: srednje temen ten kože',
    '🧑🏿‍🚀' => 'kozmonavt: temen ten kože',
    '👨🏻‍🚀' => 'astronavt: svetel ten kože',
    '👨🏼‍🚀' => 'astronavt: srednje svetel ten kože',
    '👨🏽‍🚀' => 'astronavt: srednji ten kože',
    '👨🏾‍🚀' => 'astronavt: srednje temen ten kože',
    '👨🏿‍🚀' => 'astronavt: temen ten kože',
    '👩🏻‍🚀' => 'astronavtka: svetel ten kože',
    '👩🏼‍🚀' => 'astronavtka: srednje svetel ten kože',
    '👩🏽‍🚀' => 'astronavtka: srednji ten kože',
    '👩🏾‍🚀' => 'astronavtka: srednje temen ten kože',
    '👩🏿‍🚀' => 'astronavtka: temen ten kože',
    '🧑🏻‍🚒' => 'poklicni gasilec: svetel ten kože',
    '🧑🏼‍🚒' => 'poklicni gasilec: srednje svetel ten kože',
    '🧑🏽‍🚒' => 'poklicni gasilec: srednji ten kože',
    '🧑🏾‍🚒' => 'poklicni gasilec: srednje temen ten kože',
    '🧑🏿‍🚒' => 'poklicni gasilec: temen ten kože',
    '👨🏻‍🚒' => 'gasilec: svetel ten kože',
    '👨🏼‍🚒' => 'gasilec: srednje svetel ten kože',
    '👨🏽‍🚒' => 'gasilec: srednji ten kože',
    '👨🏾‍🚒' => 'gasilec: srednje temen ten kože',
    '👨🏿‍🚒' => 'gasilec: temen ten kože',
    '👩🏻‍🚒' => 'gasilka: svetel ten kože',
    '👩🏼‍🚒' => 'gasilka: srednje svetel ten kože',
    '👩🏽‍🚒' => 'gasilka: srednji ten kože',
    '👩🏾‍🚒' => 'gasilka: srednje temen ten kože',
    '👩🏿‍🚒' => 'gasilka: temen ten kože',
    '👮🏻‍♂' => 'policist: svetel ten kože',
    '👮🏼‍♂' => 'policist: srednje svetel ten kože',
    '👮🏽‍♂' => 'policist: srednji ten kože',
    '👮🏾‍♂' => 'policist: srednje temen ten kože',
    '👮🏿‍♂' => 'policist: temen ten kože',
    '👮🏻‍♀' => 'policistka: svetel ten kože',
    '👮🏼‍♀' => 'policistka: srednje svetel ten kože',
    '👮🏽‍♀' => 'policistka: srednji ten kože',
    '👮🏾‍♀' => 'policistka: srednje temen ten kože',
    '👮🏿‍♀' => 'policistka: temen ten kože',
    '🕵🏻‍♂' => 'detektiv: svetel ten kože',
    '🕵🏼‍♂' => 'detektiv: srednje svetel ten kože',
    '🕵🏽‍♂' => 'detektiv: srednji ten kože',
    '🕵🏾‍♂' => 'detektiv: srednje temen ten kože',
    '🕵🏿‍♂' => 'detektiv: temen ten kože',
    '🕵🏻‍♀' => 'detektivka: svetel ten kože',
    '🕵🏼‍♀' => 'detektivka: srednje svetel ten kože',
    '🕵🏽‍♀' => 'detektivka: srednji ten kože',
    '🕵🏾‍♀' => 'detektivka: srednje temen ten kože',
    '🕵🏿‍♀' => 'detektivka: temen ten kože',
    '💂🏻‍♂' => 'varnostnik: svetel ten kože',
    '💂🏼‍♂' => 'varnostnik: srednje svetel ten kože',
    '💂🏽‍♂' => 'varnostnik: srednji ten kože',
    '💂🏾‍♂' => 'varnostnik: srednje temen ten kože',
    '💂🏿‍♂' => 'varnostnik: temen ten kože',
    '💂🏻‍♀' => 'varnostnica: svetel ten kože',
    '💂🏼‍♀' => 'varnostnica: srednje svetel ten kože',
    '💂🏽‍♀' => 'varnostnica: srednji ten kože',
    '💂🏾‍♀' => 'varnostnica: srednje temen ten kože',
    '💂🏿‍♀' => 'varnostnica: temen ten kože',
    '👷🏻‍♂' => 'gradbeni delavec: svetel ten kože',
    '👷🏼‍♂' => 'gradbeni delavec: srednje svetel ten kože',
    '👷🏽‍♂' => 'gradbeni delavec: srednji ten kože',
    '👷🏾‍♂' => 'gradbeni delavec: srednje temen ten kože',
    '👷🏿‍♂' => 'gradbeni delavec: temen ten kože',
    '👷🏻‍♀' => 'gradbena delavka: svetel ten kože',
    '👷🏼‍♀' => 'gradbena delavka: srednje svetel ten kože',
    '👷🏽‍♀' => 'gradbena delavka: srednji ten kože',
    '👷🏾‍♀' => 'gradbena delavka: srednje temen ten kože',
    '👷🏿‍♀' => 'gradbena delavka: temen ten kože',
    '👳🏻‍♂' => 'moški s turbanom: svetel ten kože',
    '👳🏼‍♂' => 'moški s turbanom: srednje svetel ten kože',
    '👳🏽‍♂' => 'moški s turbanom: srednji ten kože',
    '👳🏾‍♂' => 'moški s turbanom: srednje temen ten kože',
    '👳🏿‍♂' => 'moški s turbanom: temen ten kože',
    '👳🏻‍♀' => 'ženska s turbanom: svetel ten kože',
    '👳🏼‍♀' => 'ženska s turbanom: srednje svetel ten kože',
    '👳🏽‍♀' => 'ženska s turbanom: srednji ten kože',
    '👳🏾‍♀' => 'ženska s turbanom: srednje temen ten kože',
    '👳🏿‍♀' => 'ženska s turbanom: temen ten kože',
    '🤵🏻‍♂' => 'moški v smokingu: svetel ten kože',
    '🤵🏼‍♂' => 'moški v smokingu: srednje svetel ten kože',
    '🤵🏽‍♂' => 'moški v smokingu: srednji ten kože',
    '🤵🏾‍♂' => 'moški v smokingu: srednje temen ten kože',
    '🤵🏿‍♂' => 'moški v smokingu: temen ten kože',
    '🤵🏻‍♀' => 'ženska v smokingu: svetel ten kože',
    '🤵🏼‍♀' => 'ženska v smokingu: srednje svetel ten kože',
    '🤵🏽‍♀' => 'ženska v smokingu: srednji ten kože',
    '🤵🏾‍♀' => 'ženska v smokingu: srednje temen ten kože',
    '🤵🏿‍♀' => 'ženska v smokingu: temen ten kože',
    '👰🏻‍♂' => 'moški s pajčolanom: svetel ten kože',
    '👰🏼‍♂' => 'moški s pajčolanom: srednje svetel ten kože',
    '👰🏽‍♂' => 'moški s pajčolanom: srednji ten kože',
    '👰🏾‍♂' => 'moški s pajčolanom: srednje temen ten kože',
    '👰🏿‍♂' => 'moški s pajčolanom: temen ten kože',
    '👰🏻‍♀' => 'ženska s pajčolanom: svetel ten kože',
    '👰🏼‍♀' => 'ženska s pajčolanom: srednje svetel ten kože',
    '👰🏽‍♀' => 'ženska s pajčolanom: srednji ten kože',
    '👰🏾‍♀' => 'ženska s pajčolanom: srednje temen ten kože',
    '👰🏿‍♀' => 'ženska s pajčolanom: temen ten kože',
    '👩🏻‍🍼' => 'ženska, ki doji dojenčka: svetel ten kože',
    '👩🏼‍🍼' => 'ženska, ki doji dojenčka: srednje svetel ten kože',
    '👩🏽‍🍼' => 'ženska, ki doji dojenčka: srednji ten kože',
    '👩🏾‍🍼' => 'ženska, ki doji dojenčka: srednje temen ten kože',
    '👩🏿‍🍼' => 'ženska, ki doji dojenčka: temen ten kože',
    '👨🏻‍🍼' => 'moški, ki hrani dojenčka: svetel ten kože',
    '👨🏼‍🍼' => 'moški, ki hrani dojenčka: srednje svetel ten kože',
    '👨🏽‍🍼' => 'moški, ki hrani dojenčka: srednji ten kože',
    '👨🏾‍🍼' => 'moški, ki hrani dojenčka: srednje temen ten kože',
    '👨🏿‍🍼' => 'moški, ki hrani dojenčka: temen ten kože',
    '🧑🏻‍🍼' => 'oseba, ki hrani dojenčka: svetel ten kože',
    '🧑🏼‍🍼' => 'oseba, ki hrani dojenčka: srednje svetel ten kože',
    '🧑🏽‍🍼' => 'oseba, ki hrani dojenčka: srednji ten kože',
    '🧑🏾‍🍼' => 'oseba, ki hrani dojenčka: srednje temen ten kože',
    '🧑🏿‍🍼' => 'oseba, ki hrani dojenčka: temen ten kože',
    '🧑🏻‍🎄' => 'mx božiček: svetel ten kože',
    '🧑🏼‍🎄' => 'mx božiček: srednje svetel ten kože',
    '🧑🏽‍🎄' => 'mx božiček: srednji ten kože',
    '🧑🏾‍🎄' => 'mx božiček: srednje temen ten kože',
    '🧑🏿‍🎄' => 'mx božiček: temen ten kože',
    '🦸🏻‍♂' => 'superjunak: svetel ten kože',
    '🦸🏼‍♂' => 'superjunak: srednje svetel ten kože',
    '🦸🏽‍♂' => 'superjunak: srednji ten kože',
    '🦸🏾‍♂' => 'superjunak: srednje temen ten kože',
    '🦸🏿‍♂' => 'superjunak: temen ten kože',
    '🦸🏻‍♀' => 'superjunakinja: svetel ten kože',
    '🦸🏼‍♀' => 'superjunakinja: srednje svetel ten kože',
    '🦸🏽‍♀' => 'superjunakinja: srednji ten kože',
    '🦸🏾‍♀' => 'superjunakinja: srednje temen ten kože',
    '🦸🏿‍♀' => 'superjunakinja: temen ten kože',
    '🦹🏻‍♂' => 'moški super negativec: svetel ten kože',
    '🦹🏼‍♂' => 'moški super negativec: srednje svetel ten kože',
    '🦹🏽‍♂' => 'moški super negativec: srednji ten kože',
    '🦹🏾‍♂' => 'moški super negativec: srednje temen ten kože',
    '🦹🏿‍♂' => 'moški super negativec: temen ten kože',
    '🦹🏻‍♀' => 'super negativka: svetel ten kože',
    '🦹🏼‍♀' => 'super negativka: srednje svetel ten kože',
    '🦹🏽‍♀' => 'super negativka: srednji ten kože',
    '🦹🏾‍♀' => 'super negativka: srednje temen ten kože',
    '🦹🏿‍♀' => 'super negativka: temen ten kože',
    '🧙🏻‍♂' => 'čarovnik: svetel ten kože',
    '🧙🏼‍♂' => 'čarovnik: srednje svetel ten kože',
    '🧙🏽‍♂' => 'čarovnik: srednji ten kože',
    '🧙🏾‍♂' => 'čarovnik: srednje temen ten kože',
    '🧙🏿‍♂' => 'čarovnik: temen ten kože',
    '🧙🏻‍♀' => 'čarovnica: svetel ten kože',
    '🧙🏼‍♀' => 'čarovnica: srednje svetel ten kože',
    '🧙🏽‍♀' => 'čarovnica: srednji ten kože',
    '🧙🏾‍♀' => 'čarovnica: srednje temen ten kože',
    '🧙🏿‍♀' => 'čarovnica: temen ten kože',
    '🧚🏻‍♂' => 'vilinec: svetel ten kože',
    '🧚🏼‍♂' => 'vilinec: srednje svetel ten kože',
    '🧚🏽‍♂' => 'vilinec: srednji ten kože',
    '🧚🏾‍♂' => 'vilinec: srednje temen ten kože',
    '🧚🏿‍♂' => 'vilinec: temen ten kože',
    '🧚🏻‍♀' => 'vila: svetel ten kože',
    '🧚🏼‍♀' => 'vila: srednje svetel ten kože',
    '🧚🏽‍♀' => 'vila: srednji ten kože',
    '🧚🏾‍♀' => 'vila: srednje temen ten kože',
    '🧚🏿‍♀' => 'vila: temen ten kože',
    '🧛🏻‍♂' => 'vampir: svetel ten kože',
    '🧛🏼‍♂' => 'vampir: srednje svetel ten kože',
    '🧛🏽‍♂' => 'vampir: srednji ten kože',
    '🧛🏾‍♂' => 'vampir: srednje temen ten kože',
    '🧛🏿‍♂' => 'vampir: temen ten kože',
    '🧛🏻‍♀' => 'vampirka: svetel ten kože',
    '🧛🏼‍♀' => 'vampirka: srednje svetel ten kože',
    '🧛🏽‍♀' => 'vampirka: srednji ten kože',
    '🧛🏾‍♀' => 'vampirka: srednje temen ten kože',
    '🧛🏿‍♀' => 'vampirka: temen ten kože',
    '🧜🏻‍♂' => 'morski mož: svetel ten kože',
    '🧜🏼‍♂' => 'morski mož: srednje svetel ten kože',
    '🧜🏽‍♂' => 'morski mož: srednji ten kože',
    '🧜🏾‍♂' => 'morski mož: srednje temen ten kože',
    '🧜🏿‍♂' => 'morski mož: temen ten kože',
    '🧜🏻‍♀' => 'morska deklica: svetel ten kože',
    '🧜🏼‍♀' => 'morska deklica: srednje svetel ten kože',
    '🧜🏽‍♀' => 'morska deklica: srednji ten kože',
    '🧜🏾‍♀' => 'morska deklica: srednje temen ten kože',
    '🧜🏿‍♀' => 'morska deklica: temen ten kože',
    '🧝🏻‍♂' => 'škrat: svetel ten kože',
    '🧝🏼‍♂' => 'škrat: srednje svetel ten kože',
    '🧝🏽‍♂' => 'škrat: srednji ten kože',
    '🧝🏾‍♂' => 'škrat: srednje temen ten kože',
    '🧝🏿‍♂' => 'škrat: temen ten kože',
    '🧝🏻‍♀' => 'škratovka: svetel ten kože',
    '🧝🏼‍♀' => 'škratovka: srednje svetel ten kože',
    '🧝🏽‍♀' => 'škratovka: srednji ten kože',
    '🧝🏾‍♀' => 'škratovka: srednje temen ten kože',
    '🧝🏿‍♀' => 'škratovka: temen ten kože',
    '💆🏻‍♂' => 'moški na masaži obraza: svetel ten kože',
    '💆🏼‍♂' => 'moški na masaži obraza: srednje svetel ten kože',
    '💆🏽‍♂' => 'moški na masaži obraza: srednji ten kože',
    '💆🏾‍♂' => 'moški na masaži obraza: srednje temen ten kože',
    '💆🏿‍♂' => 'moški na masaži obraza: temen ten kože',
    '💆🏻‍♀' => 'ženska na masaži obraza: svetel ten kože',
    '💆🏼‍♀' => 'ženska na masaži obraza: srednje svetel ten kože',
    '💆🏽‍♀' => 'ženska na masaži obraza: srednji ten kože',
    '💆🏾‍♀' => 'ženska na masaži obraza: srednje temen ten kože',
    '💆🏿‍♀' => 'ženska na masaži obraza: temen ten kože',
    '💇🏻‍♂' => 'moški pri frizerju: svetel ten kože',
    '💇🏼‍♂' => 'moški pri frizerju: srednje svetel ten kože',
    '💇🏽‍♂' => 'moški pri frizerju: srednji ten kože',
    '💇🏾‍♂' => 'moški pri frizerju: srednje temen ten kože',
    '💇🏿‍♂' => 'moški pri frizerju: temen ten kože',
    '💇🏻‍♀' => 'ženska pri frizerju: svetel ten kože',
    '💇🏼‍♀' => 'ženska pri frizerju: srednje svetel ten kože',
    '💇🏽‍♀' => 'ženska pri frizerju: srednji ten kože',
    '💇🏾‍♀' => 'ženska pri frizerju: srednje temen ten kože',
    '💇🏿‍♀' => 'ženska pri frizerju: temen ten kože',
    '🚶🏻‍♂' => 'moški, ki hodi: svetel ten kože',
    '🚶🏼‍♂' => 'moški, ki hodi: srednje svetel ten kože',
    '🚶🏽‍♂' => 'moški, ki hodi: srednji ten kože',
    '🚶🏾‍♂' => 'moški, ki hodi: srednje temen ten kože',
    '🚶🏿‍♂' => 'moški, ki hodi: temen ten kože',
    '🚶🏻‍♀' => 'ženska, ki hodi: svetel ten kože',
    '🚶🏼‍♀' => 'ženska, ki hodi: srednje svetel ten kože',
    '🚶🏽‍♀' => 'ženska, ki hodi: srednji ten kože',
    '🚶🏾‍♀' => 'ženska, ki hodi: srednje temen ten kože',
    '🚶🏿‍♀' => 'ženska, ki hodi: temen ten kože',
    '🚶🏻‍➡' => 'pešec: svetel ten kože in , obrnjen v desno',
    '🚶🏼‍➡' => 'pešec: srednje svetel ten kože in , obrnjen v desno',
    '🚶🏽‍➡' => 'pešec: srednji ten kože in , obrnjen v desno',
    '🚶🏾‍➡' => 'pešec: srednje temen ten kože in , obrnjen v desno',
    '🚶🏿‍➡' => 'pešec: temen ten kože in , obrnjen v desno',
    '🧍🏻‍♂' => 'stoječ moški: svetel ten kože',
    '🧍🏼‍♂' => 'stoječ moški: srednje svetel ten kože',
    '🧍🏽‍♂' => 'stoječ moški: srednji ten kože',
    '🧍🏾‍♂' => 'stoječ moški: srednje temen ten kože',
    '🧍🏿‍♂' => 'stoječ moški: temen ten kože',
    '🧍🏻‍♀' => 'stoječa ženska: svetel ten kože',
    '🧍🏼‍♀' => 'stoječa ženska: srednje svetel ten kože',
    '🧍🏽‍♀' => 'stoječa ženska: srednji ten kože',
    '🧍🏾‍♀' => 'stoječa ženska: srednje temen ten kože',
    '🧍🏿‍♀' => 'stoječa ženska: temen ten kože',
    '🧎🏻‍♂' => 'klečeč moški: svetel ten kože',
    '🧎🏼‍♂' => 'klečeč moški: srednje svetel ten kože',
    '🧎🏽‍♂' => 'klečeč moški: srednji ten kože',
    '🧎🏾‍♂' => 'klečeč moški: srednje temen ten kože',
    '🧎🏿‍♂' => 'klečeč moški: temen ten kože',
    '🧎🏻‍♀' => 'klečeča ženska: svetel ten kože',
    '🧎🏼‍♀' => 'klečeča ženska: srednje svetel ten kože',
    '🧎🏽‍♀' => 'klečeča ženska: srednji ten kože',
    '🧎🏾‍♀' => 'klečeča ženska: srednje temen ten kože',
    '🧎🏿‍♀' => 'klečeča ženska: temen ten kože',
    '🧎🏻‍➡' => 'klečeča oseba: svetel ten kože in , obrnjen v desno',
    '🧎🏼‍➡' => 'klečeča oseba: srednje svetel ten kože in , obrnjen v desno',
    '🧎🏽‍➡' => 'klečeča oseba: srednji ten kože in , obrnjen v desno',
    '🧎🏾‍➡' => 'klečeča oseba: srednje temen ten kože in , obrnjen v desno',
    '🧎🏿‍➡' => 'klečeča oseba: temen ten kože in , obrnjen v desno',
    '🧑🏻‍🦯' => 'oseba s palico za slepe: svetel ten kože',
    '🧑🏼‍🦯' => 'oseba s palico za slepe: srednje svetel ten kože',
    '🧑🏽‍🦯' => 'oseba s palico za slepe: srednji ten kože',
    '🧑🏾‍🦯' => 'oseba s palico za slepe: srednje temen ten kože',
    '🧑🏿‍🦯' => 'oseba s palico za slepe: temen ten kože',
    '👨🏻‍🦯' => 'moški s palico za slepe: svetel ten kože',
    '👨🏼‍🦯' => 'moški s palico za slepe: srednje svetel ten kože',
    '👨🏽‍🦯' => 'moški s palico za slepe: srednji ten kože',
    '👨🏾‍🦯' => 'moški s palico za slepe: srednje temen ten kože',
    '👨🏿‍🦯' => 'moški s palico za slepe: temen ten kože',
    '👩🏻‍🦯' => 'ženska s palico za slepe: svetel ten kože',
    '👩🏼‍🦯' => 'ženska s palico za slepe: srednje svetel ten kože',
    '👩🏽‍🦯' => 'ženska s palico za slepe: srednji ten kože',
    '👩🏾‍🦯' => 'ženska s palico za slepe: srednje temen ten kože',
    '👩🏿‍🦯' => 'ženska s palico za slepe: temen ten kože',
    '🧑🏻‍🦼' => 'oseba na motoriziranem invalidskem vozičku: svetel ten kože',
    '🧑🏼‍🦼' => 'oseba na motoriziranem invalidskem vozičku: srednje svetel ten kože',
    '🧑🏽‍🦼' => 'oseba na motoriziranem invalidskem vozičku: srednji ten kože',
    '🧑🏾‍🦼' => 'oseba na motoriziranem invalidskem vozičku: srednje temen ten kože',
    '🧑🏿‍🦼' => 'oseba na motoriziranem invalidskem vozičku: temen ten kože',
    '👨🏻‍🦼' => 'moški na motoriziranem invalidskem vozičku: svetel ten kože',
    '👨🏼‍🦼' => 'moški na motoriziranem invalidskem vozičku: srednje svetel ten kože',
    '👨🏽‍🦼' => 'moški na motoriziranem invalidskem vozičku: srednji ten kože',
    '👨🏾‍🦼' => 'moški na motoriziranem invalidskem vozičku: srednje temen ten kože',
    '👨🏿‍🦼' => 'moški na motoriziranem invalidskem vozičku: temen ten kože',
    '👩🏻‍🦼' => 'ženska na motoriziranem invalidskem vozičku: svetel ten kože',
    '👩🏼‍🦼' => 'ženska na motoriziranem invalidskem vozičku: srednje svetel ten kože',
    '👩🏽‍🦼' => 'ženska na motoriziranem invalidskem vozičku: srednji ten kože',
    '👩🏾‍🦼' => 'ženska na motoriziranem invalidskem vozičku: srednje temen ten kože',
    '👩🏿‍🦼' => 'ženska na motoriziranem invalidskem vozičku: temen ten kože',
    '🧑🏻‍🦽' => 'oseba na ročnem invalidskem vozičku: svetel ten kože',
    '🧑🏼‍🦽' => 'oseba na ročnem invalidskem vozičku: srednje svetel ten kože',
    '🧑🏽‍🦽' => 'oseba na ročnem invalidskem vozičku: srednji ten kože',
    '🧑🏾‍🦽' => 'oseba na ročnem invalidskem vozičku: srednje temen ten kože',
    '🧑🏿‍🦽' => 'oseba na ročnem invalidskem vozičku: temen ten kože',
    '👨🏻‍🦽' => 'moški na ročnem invalidskem vozičku: svetel ten kože',
    '👨🏼‍🦽' => 'moški na ročnem invalidskem vozičku: srednje svetel ten kože',
    '👨🏽‍🦽' => 'moški na ročnem invalidskem vozičku: srednji ten kože',
    '👨🏾‍🦽' => 'moški na ročnem invalidskem vozičku: srednje temen ten kože',
    '👨🏿‍🦽' => 'moški na ročnem invalidskem vozičku: temen ten kože',
    '👩🏻‍🦽' => 'ženska na ročnem invalidskem vozičku: svetel ten kože',
    '👩🏼‍🦽' => 'ženska na ročnem invalidskem vozičku: srednje svetel ten kože',
    '👩🏽‍🦽' => 'ženska na ročnem invalidskem vozičku: srednji ten kože',
    '👩🏾‍🦽' => 'ženska na ročnem invalidskem vozičku: srednje temen ten kože',
    '👩🏿‍🦽' => 'ženska na ročnem invalidskem vozičku: temen ten kože',
    '🏃🏻‍♂' => 'moški, ki teče: svetel ten kože',
    '🏃🏼‍♂' => 'moški, ki teče: srednje svetel ten kože',
    '🏃🏽‍♂' => 'moški, ki teče: srednji ten kože',
    '🏃🏾‍♂' => 'moški, ki teče: srednje temen ten kože',
    '🏃🏿‍♂' => 'moški, ki teče: temen ten kože',
    '🏃🏻‍♀' => 'ženska, ki teče: svetel ten kože',
    '🏃🏼‍♀' => 'ženska, ki teče: srednje svetel ten kože',
    '🏃🏽‍♀' => 'ženska, ki teče: srednji ten kože',
    '🏃🏾‍♀' => 'ženska, ki teče: srednje temen ten kože',
    '🏃🏿‍♀' => 'ženska, ki teče: temen ten kože',
    '🏃🏻‍➡' => 'tekač: svetel ten kože in , obrnjen v desno',
    '🏃🏼‍➡' => 'tekač: srednje svetel ten kože in , obrnjen v desno',
    '🏃🏽‍➡' => 'tekač: srednji ten kože in , obrnjen v desno',
    '🏃🏾‍➡' => 'tekač: srednje temen ten kože in , obrnjen v desno',
    '🏃🏿‍➡' => 'tekač: temen ten kože in , obrnjen v desno',
    '👯🏻‍♀' => 'ženska zabava: svetel ten kože',
    '👯🏼‍♀' => 'ženska zabava: srednje svetel ten kože',
    '👯🏽‍♀' => 'ženska zabava: srednji ten kože',
    '👯🏾‍♀' => 'ženska zabava: srednje temen ten kože',
    '👯🏿‍♀' => 'ženska zabava: temen ten kože',
    '👯🏻‍♂' => 'moška zabava: svetel ten kože',
    '👯🏼‍♂' => 'moška zabava: srednje svetel ten kože',
    '👯🏽‍♂' => 'moška zabava: srednji ten kože',
    '👯🏾‍♂' => 'moška zabava: srednje temen ten kože',
    '👯🏿‍♂' => 'moška zabava: temen ten kože',
    '🧖🏻‍♂' => 'moški v parni savni: svetel ten kože',
    '🧖🏼‍♂' => 'moški v parni savni: srednje svetel ten kože',
    '🧖🏽‍♂' => 'moški v parni savni: srednji ten kože',
    '🧖🏾‍♂' => 'moški v parni savni: srednje temen ten kože',
    '🧖🏿‍♂' => 'moški v parni savni: temen ten kože',
    '🧖🏻‍♀' => 'ženska v parni savni: svetel ten kože',
    '🧖🏼‍♀' => 'ženska v parni savni: srednje svetel ten kože',
    '🧖🏽‍♀' => 'ženska v parni savni: srednji ten kože',
    '🧖🏾‍♀' => 'ženska v parni savni: srednje temen ten kože',
    '🧖🏿‍♀' => 'ženska v parni savni: temen ten kože',
    '🧗🏻‍♂' => 'plezalec: svetel ten kože',
    '🧗🏼‍♂' => 'plezalec: srednje svetel ten kože',
    '🧗🏽‍♂' => 'plezalec: srednji ten kože',
    '🧗🏾‍♂' => 'plezalec: srednje temen ten kože',
    '🧗🏿‍♂' => 'plezalec: temen ten kože',
    '🧗🏻‍♀' => 'plezalka: svetel ten kože',
    '🧗🏼‍♀' => 'plezalka: srednje svetel ten kože',
    '🧗🏽‍♀' => 'plezalka: srednji ten kože',
    '🧗🏾‍♀' => 'plezalka: srednje temen ten kože',
    '🧗🏿‍♀' => 'plezalka: temen ten kože',
    '🏌🏻‍♂' => 'golfist: svetel ten kože',
    '🏌🏼‍♂' => 'golfist: srednje svetel ten kože',
    '🏌🏽‍♂' => 'golfist: srednji ten kože',
    '🏌🏾‍♂' => 'golfist: srednje temen ten kože',
    '🏌🏿‍♂' => 'golfist: temen ten kože',
    '🏌🏻‍♀' => 'golfistka: svetel ten kože',
    '🏌🏼‍♀' => 'golfistka: srednje svetel ten kože',
    '🏌🏽‍♀' => 'golfistka: srednji ten kože',
    '🏌🏾‍♀' => 'golfistka: srednje temen ten kože',
    '🏌🏿‍♀' => 'golfistka: temen ten kože',
    '🏄🏻‍♂' => 'deskar: svetel ten kože',
    '🏄🏼‍♂' => 'deskar: srednje svetel ten kože',
    '🏄🏽‍♂' => 'deskar: srednji ten kože',
    '🏄🏾‍♂' => 'deskar: srednje temen ten kože',
    '🏄🏿‍♂' => 'deskar: temen ten kože',
    '🏄🏻‍♀' => 'deskarka: svetel ten kože',
    '🏄🏼‍♀' => 'deskarka: srednje svetel ten kože',
    '🏄🏽‍♀' => 'deskarka: srednji ten kože',
    '🏄🏾‍♀' => 'deskarka: srednje temen ten kože',
    '🏄🏿‍♀' => 'deskarka: temen ten kože',
    '🚣🏻‍♂' => 'veslač: svetel ten kože',
    '🚣🏼‍♂' => 'veslač: srednje svetel ten kože',
    '🚣🏽‍♂' => 'veslač: srednji ten kože',
    '🚣🏾‍♂' => 'veslač: srednje temen ten kože',
    '🚣🏿‍♂' => 'veslač: temen ten kože',
    '🚣🏻‍♀' => 'veslačica: svetel ten kože',
    '🚣🏼‍♀' => 'veslačica: srednje svetel ten kože',
    '🚣🏽‍♀' => 'veslačica: srednji ten kože',
    '🚣🏾‍♀' => 'veslačica: srednje temen ten kože',
    '🚣🏿‍♀' => 'veslačica: temen ten kože',
    '🏊🏻‍♂' => 'plavalec: svetel ten kože',
    '🏊🏼‍♂' => 'plavalec: srednje svetel ten kože',
    '🏊🏽‍♂' => 'plavalec: srednji ten kože',
    '🏊🏾‍♂' => 'plavalec: srednje temen ten kože',
    '🏊🏿‍♂' => 'plavalec: temen ten kože',
    '🏊🏻‍♀' => 'plavalka: svetel ten kože',
    '🏊🏼‍♀' => 'plavalka: srednje svetel ten kože',
    '🏊🏽‍♀' => 'plavalka: srednji ten kože',
    '🏊🏾‍♀' => 'plavalka: srednje temen ten kože',
    '🏊🏿‍♀' => 'plavalka: temen ten kože',
    '⛹🏻‍♂' => 'moški z žogo: svetel ten kože',
    '⛹🏼‍♂' => 'moški z žogo: srednje svetel ten kože',
    '⛹🏽‍♂' => 'moški z žogo: srednji ten kože',
    '⛹🏾‍♂' => 'moški z žogo: srednje temen ten kože',
    '⛹🏿‍♂' => 'moški z žogo: temen ten kože',
    '⛹🏻‍♀' => 'ženska z žogo: svetel ten kože',
    '⛹🏼‍♀' => 'ženska z žogo: srednje svetel ten kože',
    '⛹🏽‍♀' => 'ženska z žogo: srednji ten kože',
    '⛹🏾‍♀' => 'ženska z žogo: srednje temen ten kože',
    '⛹🏿‍♀' => 'ženska z žogo: temen ten kože',
    '🏋🏻‍♂' => 'dvigovalec uteži: svetel ten kože',
    '🏋🏼‍♂' => 'dvigovalec uteži: srednje svetel ten kože',
    '🏋🏽‍♂' => 'dvigovalec uteži: srednji ten kože',
    '🏋🏾‍♂' => 'dvigovalec uteži: srednje temen ten kože',
    '🏋🏿‍♂' => 'dvigovalec uteži: temen ten kože',
    '🏋🏻‍♀' => 'dvigovalka uteži: svetel ten kože',
    '🏋🏼‍♀' => 'dvigovalka uteži: srednje svetel ten kože',
    '🏋🏽‍♀' => 'dvigovalka uteži: srednji ten kože',
    '🏋🏾‍♀' => 'dvigovalka uteži: srednje temen ten kože',
    '🏋🏿‍♀' => 'dvigovalka uteži: temen ten kože',
    '🚴🏻‍♂' => 'kolesar: svetel ten kože',
    '🚴🏼‍♂' => 'kolesar: srednje svetel ten kože',
    '🚴🏽‍♂' => 'kolesar: srednji ten kože',
    '🚴🏾‍♂' => 'kolesar: srednje temen ten kože',
    '🚴🏿‍♂' => 'kolesar: temen ten kože',
    '🚴🏻‍♀' => 'kolesarka: svetel ten kože',
    '🚴🏼‍♀' => 'kolesarka: srednje svetel ten kože',
    '🚴🏽‍♀' => 'kolesarka: srednji ten kože',
    '🚴🏾‍♀' => 'kolesarka: srednje temen ten kože',
    '🚴🏿‍♀' => 'kolesarka: temen ten kože',
    '🚵🏻‍♂' => 'moško gorsko kolesarjenje: svetel ten kože',
    '🚵🏼‍♂' => 'moško gorsko kolesarjenje: srednje svetel ten kože',
    '🚵🏽‍♂' => 'moško gorsko kolesarjenje: srednji ten kože',
    '🚵🏾‍♂' => 'moško gorsko kolesarjenje: srednje temen ten kože',
    '🚵🏿‍♂' => 'moško gorsko kolesarjenje: temen ten kože',
    '🚵🏻‍♀' => 'žensko gorsko kolesarjenje: svetel ten kože',
    '🚵🏼‍♀' => 'žensko gorsko kolesarjenje: srednje svetel ten kože',
    '🚵🏽‍♀' => 'žensko gorsko kolesarjenje: srednji ten kože',
    '🚵🏾‍♀' => 'žensko gorsko kolesarjenje: srednje temen ten kože',
    '🚵🏿‍♀' => 'žensko gorsko kolesarjenje: temen ten kože',
    '🤸🏻‍♂' => 'moški, ki dela premet vstran: svetel ten kože',
    '🤸🏼‍♂' => 'moški, ki dela premet vstran: srednje svetel ten kože',
    '🤸🏽‍♂' => 'moški, ki dela premet vstran: srednji ten kože',
    '🤸🏾‍♂' => 'moški, ki dela premet vstran: srednje temen ten kože',
    '🤸🏿‍♂' => 'moški, ki dela premet vstran: temen ten kože',
    '🤸🏻‍♀' => 'ženska, ki dela premet vstran: svetel ten kože',
    '🤸🏼‍♀' => 'ženska, ki dela premet vstran: srednje svetel ten kože',
    '🤸🏽‍♀' => 'ženska, ki dela premet vstran: srednji ten kože',
    '🤸🏾‍♀' => 'ženska, ki dela premet vstran: srednje temen ten kože',
    '🤸🏿‍♀' => 'ženska, ki dela premet vstran: temen ten kože',
    '🤼🏻‍♀' => 'ženska rokoborba: svetel ten kože',
    '🤼🏼‍♀' => 'ženska rokoborba: srednje svetel ten kože',
    '🤼🏽‍♀' => 'ženska rokoborba: srednji ten kože',
    '🤼🏾‍♀' => 'ženska rokoborba: srednje temen ten kože',
    '🤼🏿‍♀' => 'ženska rokoborba: temen ten kože',
    '🤼🏻‍♂' => 'moška rokoborba: svetel ten kože',
    '🤼🏼‍♂' => 'moška rokoborba: srednje svetel ten kože',
    '🤼🏽‍♂' => 'moška rokoborba: srednji ten kože',
    '🤼🏾‍♂' => 'moška rokoborba: srednje temen ten kože',
    '🤼🏿‍♂' => 'moška rokoborba: temen ten kože',
    '🤽🏻‍♂' => 'moški vaterpolo: svetel ten kože',
    '🤽🏼‍♂' => 'moški vaterpolo: srednje svetel ten kože',
    '🤽🏽‍♂' => 'moški vaterpolo: srednji ten kože',
    '🤽🏾‍♂' => 'moški vaterpolo: srednje temen ten kože',
    '🤽🏿‍♂' => 'moški vaterpolo: temen ten kože',
    '🤽🏻‍♀' => 'ženski vaterpolo: svetel ten kože',
    '🤽🏼‍♀' => 'ženski vaterpolo: srednje svetel ten kože',
    '🤽🏽‍♀' => 'ženski vaterpolo: srednji ten kože',
    '🤽🏾‍♀' => 'ženski vaterpolo: srednje temen ten kože',
    '🤽🏿‍♀' => 'ženski vaterpolo: temen ten kože',
    '🤾🏻‍♂' => 'rokometaš: svetel ten kože',
    '🤾🏼‍♂' => 'rokometaš: srednje svetel ten kože',
    '🤾🏽‍♂' => 'rokometaš: srednji ten kože',
    '🤾🏾‍♂' => 'rokometaš: srednje temen ten kože',
    '🤾🏿‍♂' => 'rokometaš: temen ten kože',
    '🤾🏻‍♀' => 'rokometašica: svetel ten kože',
    '🤾🏼‍♀' => 'rokometašica: srednje svetel ten kože',
    '🤾🏽‍♀' => 'rokometašica: srednji ten kože',
    '🤾🏾‍♀' => 'rokometašica: srednje temen ten kože',
    '🤾🏿‍♀' => 'rokometašica: temen ten kože',
    '🤹🏻‍♂' => 'žongler: svetel ten kože',
    '🤹🏼‍♂' => 'žongler: srednje svetel ten kože',
    '🤹🏽‍♂' => 'žongler: srednji ten kože',
    '🤹🏾‍♂' => 'žongler: srednje temen ten kože',
    '🤹🏿‍♂' => 'žongler: temen ten kože',
    '🤹🏻‍♀' => 'žonglerka: svetel ten kože',
    '🤹🏼‍♀' => 'žonglerka: srednje svetel ten kože',
    '🤹🏽‍♀' => 'žonglerka: srednji ten kože',
    '🤹🏾‍♀' => 'žonglerka: srednje temen ten kože',
    '🤹🏿‍♀' => 'žonglerka: temen ten kože',
    '🧘🏻‍♂' => 'moški v lotusovem položaju: svetel ten kože',
    '🧘🏼‍♂' => 'moški v lotusovem položaju: srednje svetel ten kože',
    '🧘🏽‍♂' => 'moški v lotusovem položaju: srednji ten kože',
    '🧘🏾‍♂' => 'moški v lotusovem položaju: srednje temen ten kože',
    '🧘🏿‍♂' => 'moški v lotusovem položaju: temen ten kože',
    '🧘🏻‍♀' => 'ženska v lotusovem položaju: svetel ten kože',
    '🧘🏼‍♀' => 'ženska v lotusovem položaju: srednje svetel ten kože',
    '🧘🏽‍♀' => 'ženska v lotusovem položaju: srednji ten kože',
    '🧘🏾‍♀' => 'ženska v lotusovem položaju: srednje temen ten kože',
    '🧘🏿‍♀' => 'ženska v lotusovem položaju: temen ten kože',
    '🧑‍🦰' => 'odrasla oseba: rdeči lasje',
    '🧑‍🦱' => 'odrasla oseba: kodrasti lasje',
    '🧑‍🦳' => 'odrasla oseba: sivi lasje',
    '🧑‍🦲' => 'odrasla oseba: plešast',
    '👨‍🦰' => 'moški: rdeči lasje',
    '👨‍🦱' => 'moški: kodrasti lasje',
    '👨‍🦳' => 'moški: sivi lasje',
    '👨‍🦲' => 'moški: plešast',
    '👩‍🦰' => 'ženska: rdeči lasje',
    '👩‍🦱' => 'ženska: kodrasti lasje',
    '👩‍🦳' => 'ženska: sivi lasje',
    '👩‍🦲' => 'ženska: plešast',
    '🚶‍➡' => 'pešec: , obrnjen v desno',
    '🧎‍➡' => 'klečeča oseba: , obrnjen v desno',
    '🏃‍➡' => 'tekač: , obrnjen v desno',
    '👨‍👦' => 'družina: moški in fant',
    '👨‍👧' => 'družina: moški in dekle',
    '👩‍👦' => 'družina: ženska in fant',
    '👩‍👧' => 'družina: ženska in dekle',
    '😶‍🌫' => 'obraz v oblakih',
    '😮‍💨' => 'obraz med izdihovanjem',
    '🙂‍↕' => 'navpično premikanje glave',
    '😵‍💫' => 'obraz s spiralnimi očmi',
    '❤‍🔥' => 'goreče srce',
    '❤‍🩹' => 'zakrpano srce',
    '👁‍🗨' => 'oko v oblačku z govorom',
    '🧔‍♂' => 'moški: brada',
    '👱‍♂' => 'moški: blondinec',
    '🧔‍♀' => 'ženska: brada',
    '👱‍♀' => 'ženska: blondinka',
    '🙍‍♂' => 'moški z namrščenim obrazom',
    '🙍‍♀' => 'ženska z namrščenim obrazom',
    '🙎‍♂' => 'moški, ki kuha mulo',
    '🙎‍♀' => 'ženska, ki kuha mulo',
    '🙅‍♂' => 'moški, ki s kretnjo izraža prepoved',
    '🙅‍♀' => 'ženska, ki s kretnjo izraža prepoved',
    '🙆‍♂' => 'moški, ki s kretnjo izraža odobravanje',
    '🙆‍♀' => 'ženska, ki s kretnjo izraža odobravanje',
    '💁‍♂' => 'uslužbenec na informacijah',
    '💁‍♀' => 'uslužbenka na informacijah',
    '🙋‍♂' => 'moški, ki dviguje roko',
    '🙋‍♀' => 'ženska, ki dviguje roko',
    '🧏‍♂' => 'gluh moški',
    '🧏‍♀' => 'gluha ženska',
    '🙇‍♂' => 'moški, ki se globoko priklanja',
    '🙇‍♀' => 'ženska, ki se globoko priklanja',
    '🤦‍♂' => 'moški, ki obraz prekriva z dlanjo',
    '🤦‍♀' => 'ženska, ki obraz prekriva z dlanjo',
    '🤷‍♂' => 'moški, ki skomiga z rameni',
    '🤷‍♀' => 'ženska, ki skomiga z rameni',
    '🧑‍⚕' => 'delavec v zdravstvu',
    '👨‍⚕' => 'zdravstveni delavec',
    '👩‍⚕' => 'zdravstvena delavka',
    '🧑‍🎓' => 'diplomant',
    '👨‍🎓' => 'študent',
    '👩‍🎓' => 'študentka',
    '🧑‍🏫' => 'profesor',
    '👨‍🏫' => 'učitelj',
    '👩‍🏫' => 'učiteljica',
    '🧑‍⚖' => 'varuh zakona',
    '👨‍⚖' => 'sodnik',
    '👩‍⚖' => 'sodnica',
    '🧑‍🌾' => 'kmet',
    '👨‍🌾' => 'kmetovalec',
    '👩‍🌾' => 'kmetovalka',
    '🧑‍🍳' => 'kuharski mojster',
    '👨‍🍳' => 'kuhar',
    '👩‍🍳' => 'kuharica',
    '🧑‍🔧' => 'serviser',
    '👨‍🔧' => 'mehanik',
    '👩‍🔧' => 'mehaničarka',
    '🧑‍🏭' => 'tovarniški delavec',
    '👨‍🏭' => 'delavec v tovarni',
    '👩‍🏭' => 'delavka v tovarni',
    '🧑‍💼' => 'delavec v pisarni',
    '👨‍💼' => 'pisarniški delavec',
    '👩‍💼' => 'pisarniška delavka',
    '🧑‍🔬' => 'raziskovalec',
    '👨‍🔬' => 'znanstvenik',
    '👩‍🔬' => 'znanstvenica',
    '🧑‍💻' => 'strokovnjak za tehnologijo',
    '👨‍💻' => 'tehnolog',
    '👩‍💻' => 'tehnologinja',
    '🧑‍🎤' => 'izvajalec',
    '👨‍🎤' => 'pevec',
    '👩‍🎤' => 'pevka',
    '🧑‍🎨' => 'slikar',
    '👨‍🎨' => 'umetnik',
    '👩‍🎨' => 'umetnica',
    '🧑‍✈' => 'pilot letala',
    '👨‍✈' => 'pilot',
    '👩‍✈' => 'pilotka',
    '🧑‍🚀' => 'kozmonavt',
    '👨‍🚀' => 'astronavt',
    '👩‍🚀' => 'astronavtka',
    '🧑‍🚒' => 'poklicni gasilec',
    '👨‍🚒' => 'gasilec',
    '👩‍🚒' => 'gasilka',
    '👮‍♂' => 'policist',
    '👮‍♀' => 'policistka',
    '🕵‍♂' => 'detektiv',
    '🕵‍♀' => 'detektivka',
    '💂‍♂' => 'varnostnik',
    '💂‍♀' => 'varnostnica',
    '👷‍♂' => 'gradbeni delavec',
    '👷‍♀' => 'gradbena delavka',
    '👳‍♂' => 'moški s turbanom',
    '👳‍♀' => 'ženska s turbanom',
    '🤵‍♂' => 'moški v smokingu',
    '🤵‍♀' => 'ženska v smokingu',
    '👰‍♂' => 'moški s pajčolanom',
    '👰‍♀' => 'ženska s pajčolanom',
    '👩‍🍼' => 'ženska, ki doji dojenčka',
    '👨‍🍼' => 'moški, ki hrani dojenčka',
    '🧑‍🍼' => 'oseba, ki hrani dojenčka',
    '🧑‍🎄' => 'mx božiček',
    '🦸‍♂' => 'superjunak',
    '🦸‍♀' => 'superjunakinja',
    '🦹‍♂' => 'moški super negativec',
    '🦹‍♀' => 'super negativka',
    '🧙‍♂' => 'čarovnik',
    '🧙‍♀' => 'čarovnica',
    '🧚‍♂' => 'vilinec',
    '🧚‍♀' => 'vila',
    '🧛‍♂' => 'vampir',
    '🧛‍♀' => 'vampirka',
    '🧜‍♂' => 'morski mož',
    '🧜‍♀' => 'morska deklica',
    '🧝‍♂' => 'škrat',
    '🧝‍♀' => 'škratovka',
    '🧞‍♂' => 'moški duh iz steklenice',
    '🧞‍♀' => 'ženski duh iz steklenice',
    '🧟‍♂' => 'zombi moški',
    '🧟‍♀' => 'zombi ženska',
    '💆‍♂' => 'moški na masaži obraza',
    '💆‍♀' => 'ženska na masaži obraza',
    '💇‍♂' => 'moški pri frizerju',
    '💇‍♀' => 'ženska pri frizerju',
    '🚶‍♂' => 'moški, ki hodi',
    '🚶‍♀' => 'ženska, ki hodi',
    '🧍‍♂' => 'stoječ moški',
    '🧍‍♀' => 'stoječa ženska',
    '🧎‍♂' => 'klečeč moški',
    '🧎‍♀' => 'klečeča ženska',
    '🧑‍🦯' => 'oseba s palico za slepe',
    '👨‍🦯' => 'moški s palico za slepe',
    '👩‍🦯' => 'ženska s palico za slepe',
    '🧑‍🦼' => 'oseba na motoriziranem invalidskem vozičku',
    '👨‍🦼' => 'moški na motoriziranem invalidskem vozičku',
    '👩‍🦼' => 'ženska na motoriziranem invalidskem vozičku',
    '🧑‍🦽' => 'oseba na ročnem invalidskem vozičku',
    '👨‍🦽' => 'moški na ročnem invalidskem vozičku',
    '👩‍🦽' => 'ženska na ročnem invalidskem vozičku',
    '🏃‍♂' => 'moški, ki teče',
    '🏃‍♀' => 'ženska, ki teče',
    '🧑‍🩰' => 'baletka',
    '👯‍♂' => 'moška zabava',
    '👯‍♀' => 'ženska zabava',
    '🧖‍♂' => 'moški v parni savni',
    '🧖‍♀' => 'ženska v parni savni',
    '🧗‍♂' => 'plezalec',
    '🧗‍♀' => 'plezalka',
    '🏌‍♂' => 'golfist',
    '🏌‍♀' => 'golfistka',
    '🏄‍♂' => 'deskar',
    '🏄‍♀' => 'deskarka',
    '🚣‍♂' => 'veslač',
    '🚣‍♀' => 'veslačica',
    '🏊‍♂' => 'plavalec',
    '🏊‍♀' => 'plavalka',
    '⛹‍♂' => 'moški z žogo',
    '⛹‍♀' => 'ženska z žogo',
    '🏋‍♂' => 'dvigovalec uteži',
    '🏋‍♀' => 'dvigovalka uteži',
    '🚴‍♂' => 'kolesar',
    '🚴‍♀' => 'kolesarka',
    '🚵‍♂' => 'moško gorsko kolesarjenje',
    '🚵‍♀' => 'žensko gorsko kolesarjenje',
    '🤸‍♂' => 'moški, ki dela premet vstran',
    '🤸‍♀' => 'ženska, ki dela premet vstran',
    '🤼‍♂' => 'moška rokoborba',
    '🤼‍♀' => 'ženska rokoborba',
    '🤽‍♂' => 'moški vaterpolo',
    '🤽‍♀' => 'ženski vaterpolo',
    '🤾‍♂' => 'rokometaš',
    '🤾‍♀' => 'rokometašica',
    '🤹‍♂' => 'žongler',
    '🤹‍♀' => 'žonglerka',
    '🧘‍♂' => 'moški v lotusovem položaju',
    '🧘‍♀' => 'ženska v lotusovem položaju',
    '🧑‍🧒' => 'družina: odrasla oseba, otrok',
    '🐕‍🦺' => 'službeni pes',
    '🐈‍⬛' => 'črna mačka',
    '🐻‍❄' => 'severni medved',
    '🐦‍⬛' => 'kos',
    '🐦‍🔥' => 'feniks',
    '🍋‍🟩' => 'limeta',
    '🍄‍🟫' => 'šampinjon',
    '⛓‍💥' => 'strgana veriga',
    '🏳‍🌈' => 'mavrična zastava',
    '🏳‍⚧' => 'transspolna zastava',
    '🏴‍☠' => 'piratska zastava',
    '👋🏻' => 'mahajoča dlan: svetel ten kože',
    '👋🏼' => 'mahajoča dlan: srednje svetel ten kože',
    '👋🏽' => 'mahajoča dlan: srednji ten kože',
    '👋🏾' => 'mahajoča dlan: srednje temen ten kože',
    '👋🏿' => 'mahajoča dlan: temen ten kože',
    '🤚🏻' => 'dvignjena hrbtna stran dlani: svetel ten kože',
    '🤚🏼' => 'dvignjena hrbtna stran dlani: srednje svetel ten kože',
    '🤚🏽' => 'dvignjena hrbtna stran dlani: srednji ten kože',
    '🤚🏾' => 'dvignjena hrbtna stran dlani: srednje temen ten kože',
    '🤚🏿' => 'dvignjena hrbtna stran dlani: temen ten kože',
    '🖐🏻' => 'dvignjena dlan z razprtimi prsti: svetel ten kože',
    '🖐🏼' => 'dvignjena dlan z razprtimi prsti: srednje svetel ten kože',
    '🖐🏽' => 'dvignjena dlan z razprtimi prsti: srednji ten kože',
    '🖐🏾' => 'dvignjena dlan z razprtimi prsti: srednje temen ten kože',
    '🖐🏿' => 'dvignjena dlan z razprtimi prsti: temen ten kože',
    '✋🏻' => 'dvignjena dlan: svetel ten kože',
    '✋🏼' => 'dvignjena dlan: srednje svetel ten kože',
    '✋🏽' => 'dvignjena dlan: srednji ten kože',
    '✋🏾' => 'dvignjena dlan: srednje temen ten kože',
    '✋🏿' => 'dvignjena dlan: temen ten kože',
    '🖖🏻' => 'vulkanski pozdrav: svetel ten kože',
    '🖖🏼' => 'vulkanski pozdrav: srednje svetel ten kože',
    '🖖🏽' => 'vulkanski pozdrav: srednji ten kože',
    '🖖🏾' => 'vulkanski pozdrav: srednje temen ten kože',
    '🖖🏿' => 'vulkanski pozdrav: temen ten kože',
    '🫱🏻' => 'roka, obrnjena v desno: svetel ten kože',
    '🫱🏼' => 'roka, obrnjena v desno: srednje svetel ten kože',
    '🫱🏽' => 'roka, obrnjena v desno: srednji ten kože',
    '🫱🏾' => 'roka, obrnjena v desno: srednje temen ten kože',
    '🫱🏿' => 'roka, obrnjena v desno: temen ten kože',
    '🫲🏻' => 'roka, obrnjena v levo: svetel ten kože',
    '🫲🏼' => 'roka, obrnjena v levo: srednje svetel ten kože',
    '🫲🏽' => 'roka, obrnjena v levo: srednji ten kože',
    '🫲🏾' => 'roka, obrnjena v levo: srednje temen ten kože',
    '🫲🏿' => 'roka, obrnjena v levo: temen ten kože',
    '🫳🏻' => 'roka z dlanjo navzdol: svetel ten kože',
    '🫳🏼' => 'roka z dlanjo navzdol: srednje svetel ten kože',
    '🫳🏽' => 'roka z dlanjo navzdol: srednji ten kože',
    '🫳🏾' => 'roka z dlanjo navzdol: srednje temen ten kože',
    '🫳🏿' => 'roka z dlanjo navzdol: temen ten kože',
    '🫴🏻' => 'roka z dlanjo navzgor: svetel ten kože',
    '🫴🏼' => 'roka z dlanjo navzgor: srednje svetel ten kože',
    '🫴🏽' => 'roka z dlanjo navzgor: srednji ten kože',
    '🫴🏾' => 'roka z dlanjo navzgor: srednje temen ten kože',
    '🫴🏿' => 'roka z dlanjo navzgor: temen ten kože',
    '🫷🏻' => 'dlan, ki potiska v levo: svetel ten kože',
    '🫷🏼' => 'dlan, ki potiska v levo: srednje svetel ten kože',
    '🫷🏽' => 'dlan, ki potiska v levo: srednji ten kože',
    '🫷🏾' => 'dlan, ki potiska v levo: srednje temen ten kože',
    '🫷🏿' => 'dlan, ki potiska v levo: temen ten kože',
    '🫸🏻' => 'dlan, ki potiska v desno: svetel ten kože',
    '🫸🏼' => 'dlan, ki potiska v desno: srednje svetel ten kože',
    '🫸🏽' => 'dlan, ki potiska v desno: srednji ten kože',
    '🫸🏾' => 'dlan, ki potiska v desno: srednje temen ten kože',
    '🫸🏿' => 'dlan, ki potiska v desno: temen ten kože',
    '👌🏻' => 'roka, ki kaže V REDU: svetel ten kože',
    '👌🏼' => 'roka, ki kaže V REDU: srednje svetel ten kože',
    '👌🏽' => 'roka, ki kaže V REDU: srednji ten kože',
    '👌🏾' => 'roka, ki kaže V REDU: srednje temen ten kože',
    '👌🏿' => 'roka, ki kaže V REDU: temen ten kože',
    '🤌🏻' => 'prsti, stisnjeni skupaj: svetel ten kože',
    '🤌🏼' => 'prsti, stisnjeni skupaj: srednje svetel ten kože',
    '🤌🏽' => 'prsti, stisnjeni skupaj: srednji ten kože',
    '🤌🏾' => 'prsti, stisnjeni skupaj: srednje temen ten kože',
    '🤌🏿' => 'prsti, stisnjeni skupaj: temen ten kože',
    '🤏🏻' => 'roka s ščepcem: svetel ten kože',
    '🤏🏼' => 'roka s ščepcem: srednje svetel ten kože',
    '🤏🏽' => 'roka s ščepcem: srednji ten kože',
    '🤏🏾' => 'roka s ščepcem: srednje temen ten kože',
    '🤏🏿' => 'roka s ščepcem: temen ten kože',
    '✌🏻' => 'dlan s prstoma, ki kažeta znak za zmago: svetel ten kože',
    '✌🏼' => 'dlan s prstoma, ki kažeta znak za zmago: srednje svetel ten kože',
    '✌🏽' => 'dlan s prstoma, ki kažeta znak za zmago: srednji ten kože',
    '✌🏾' => 'dlan s prstoma, ki kažeta znak za zmago: srednje temen ten kože',
    '✌🏿' => 'dlan s prstoma, ki kažeta znak za zmago: temen ten kože',
    '🤞🏻' => 'prekrižani prsti: svetel ten kože',
    '🤞🏼' => 'prekrižani prsti: srednje svetel ten kože',
    '🤞🏽' => 'prekrižani prsti: srednji ten kože',
    '🤞🏾' => 'prekrižani prsti: srednje temen ten kože',
    '🤞🏿' => 'prekrižani prsti: temen ten kože',
    '🫰🏻' => 'roka s prekrižanima kazalcem in palcem: svetel ten kože',
    '🫰🏼' => 'roka s prekrižanima kazalcem in palcem: srednje svetel ten kože',
    '🫰🏽' => 'roka s prekrižanima kazalcem in palcem: srednji ten kože',
    '🫰🏾' => 'roka s prekrižanima kazalcem in palcem: srednje temen ten kože',
    '🫰🏿' => 'roka s prekrižanima kazalcem in palcem: temen ten kože',
    '🤟🏻' => 'znak za ljubim te: svetel ten kože',
    '🤟🏼' => 'znak za ljubim te: srednje svetel ten kože',
    '🤟🏽' => 'znak za ljubim te: srednji ten kože',
    '🤟🏾' => 'znak za ljubim te: srednje temen ten kože',
    '🤟🏿' => 'znak za ljubim te: temen ten kože',
    '🤘🏻' => 'znak za rogove: svetel ten kože',
    '🤘🏼' => 'znak za rogove: srednje svetel ten kože',
    '🤘🏽' => 'znak za rogove: srednji ten kože',
    '🤘🏾' => 'znak za rogove: srednje temen ten kože',
    '🤘🏿' => 'znak za rogove: temen ten kože',
    '🤙🏻' => 'pokliči me: svetel ten kože',
    '🤙🏼' => 'pokliči me: srednje svetel ten kože',
    '🤙🏽' => 'pokliči me: srednji ten kože',
    '🤙🏾' => 'pokliči me: srednje temen ten kože',
    '🤙🏿' => 'pokliči me: temen ten kože',
    '👈🏻' => 'hrbtna stran dlani s kazalcem, ki kaže levo: svetel ten kože',
    '👈🏼' => 'hrbtna stran dlani s kazalcem, ki kaže levo: srednje svetel ten kože',
    '👈🏽' => 'hrbtna stran dlani s kazalcem, ki kaže levo: srednji ten kože',
    '👈🏾' => 'hrbtna stran dlani s kazalcem, ki kaže levo: srednje temen ten kože',
    '👈🏿' => 'hrbtna stran dlani s kazalcem, ki kaže levo: temen ten kože',
    '👉🏻' => 'hrbtna stran dlani s kazalcem, ki kaže desno: svetel ten kože',
    '👉🏼' => 'hrbtna stran dlani s kazalcem, ki kaže desno: srednje svetel ten kože',
    '👉🏽' => 'hrbtna stran dlani s kazalcem, ki kaže desno: srednji ten kože',
    '👉🏾' => 'hrbtna stran dlani s kazalcem, ki kaže desno: srednje temen ten kože',
    '👉🏿' => 'hrbtna stran dlani s kazalcem, ki kaže desno: temen ten kože',
    '👆🏻' => 'hrbtna stran dlani s kazalcem, ki kaže gor: svetel ten kože',
    '👆🏼' => 'hrbtna stran dlani s kazalcem, ki kaže gor: srednje svetel ten kože',
    '👆🏽' => 'hrbtna stran dlani s kazalcem, ki kaže gor: srednji ten kože',
    '👆🏾' => 'hrbtna stran dlani s kazalcem, ki kaže gor: srednje temen ten kože',
    '👆🏿' => 'hrbtna stran dlani s kazalcem, ki kaže gor: temen ten kože',
    '🖕🏻' => 'sredinec: svetel ten kože',
    '🖕🏼' => 'sredinec: srednje svetel ten kože',
    '🖕🏽' => 'sredinec: srednji ten kože',
    '🖕🏾' => 'sredinec: srednje temen ten kože',
    '🖕🏿' => 'sredinec: temen ten kože',
    '👇🏻' => 'hrbtna stran dlani s kazalcem, ki kaže dol: svetel ten kože',
    '👇🏼' => 'hrbtna stran dlani s kazalcem, ki kaže dol: srednje svetel ten kože',
    '👇🏽' => 'hrbtna stran dlani s kazalcem, ki kaže dol: srednji ten kože',
    '👇🏾' => 'hrbtna stran dlani s kazalcem, ki kaže dol: srednje temen ten kože',
    '👇🏿' => 'hrbtna stran dlani s kazalcem, ki kaže dol: temen ten kože',
    '☝🏻' => 'kazalec, ki kaže gor: svetel ten kože',
    '☝🏼' => 'kazalec, ki kaže gor: srednje svetel ten kože',
    '☝🏽' => 'kazalec, ki kaže gor: srednji ten kože',
    '☝🏾' => 'kazalec, ki kaže gor: srednje temen ten kože',
    '☝🏿' => 'kazalec, ki kaže gor: temen ten kože',
    '🫵🏻' => 'kazalec, usmerjen v opazovalca: svetel ten kože',
    '🫵🏼' => 'kazalec, usmerjen v opazovalca: srednje svetel ten kože',
    '🫵🏽' => 'kazalec, usmerjen v opazovalca: srednji ten kože',
    '🫵🏾' => 'kazalec, usmerjen v opazovalca: srednje temen ten kože',
    '🫵🏿' => 'kazalec, usmerjen v opazovalca: temen ten kože',
    '👍🏻' => 'palec gor: svetel ten kože',
    '👍🏼' => 'palec gor: srednje svetel ten kože',
    '👍🏽' => 'palec gor: srednji ten kože',
    '👍🏾' => 'palec gor: srednje temen ten kože',
    '👍🏿' => 'palec gor: temen ten kože',
    '👎🏻' => 'palec dol: svetel ten kože',
    '👎🏼' => 'palec dol: srednje svetel ten kože',
    '👎🏽' => 'palec dol: srednji ten kože',
    '👎🏾' => 'palec dol: srednje temen ten kože',
    '👎🏿' => 'palec dol: temen ten kože',
    '✊🏻' => 'dvignjena pest: svetel ten kože',
    '✊🏼' => 'dvignjena pest: srednje svetel ten kože',
    '✊🏽' => 'dvignjena pest: srednji ten kože',
    '✊🏾' => 'dvignjena pest: srednje temen ten kože',
    '✊🏿' => 'dvignjena pest: temen ten kože',
    '👊🏻' => 'bližajoča se pest: svetel ten kože',
    '👊🏼' => 'bližajoča se pest: srednje svetel ten kože',
    '👊🏽' => 'bližajoča se pest: srednji ten kože',
    '👊🏾' => 'bližajoča se pest: srednje temen ten kože',
    '👊🏿' => 'bližajoča se pest: temen ten kože',
    '🤛🏻' => 'pest, obrnjena levo: svetel ten kože',
    '🤛🏼' => 'pest, obrnjena levo: srednje svetel ten kože',
    '🤛🏽' => 'pest, obrnjena levo: srednji ten kože',
    '🤛🏾' => 'pest, obrnjena levo: srednje temen ten kože',
    '🤛🏿' => 'pest, obrnjena levo: temen ten kože',
    '🤜🏻' => 'pest, obrnjena desno: svetel ten kože',
    '🤜🏼' => 'pest, obrnjena desno: srednje svetel ten kože',
    '🤜🏽' => 'pest, obrnjena desno: srednji ten kože',
    '🤜🏾' => 'pest, obrnjena desno: srednje temen ten kože',
    '🤜🏿' => 'pest, obrnjena desno: temen ten kože',
    '👏🏻' => 'ploskajoči dlani: svetel ten kože',
    '👏🏼' => 'ploskajoči dlani: srednje svetel ten kože',
    '👏🏽' => 'ploskajoči dlani: srednji ten kože',
    '👏🏾' => 'ploskajoči dlani: srednje temen ten kože',
    '👏🏿' => 'ploskajoči dlani: temen ten kože',
    '🙌🏻' => 'oseba, ki dvigne roke: svetel ten kože',
    '🙌🏼' => 'oseba, ki dvigne roke: srednje svetel ten kože',
    '🙌🏽' => 'oseba, ki dvigne roke: srednji ten kože',
    '🙌🏾' => 'oseba, ki dvigne roke: srednje temen ten kože',
    '🙌🏿' => 'oseba, ki dvigne roke: temen ten kože',
    '🫶🏻' => 'roki, sklenjeni v obliki srca: svetel ten kože',
    '🫶🏼' => 'roki, sklenjeni v obliki srca: srednje svetel ten kože',
    '🫶🏽' => 'roki, sklenjeni v obliki srca: srednji ten kože',
    '🫶🏾' => 'roki, sklenjeni v obliki srca: srednje temen ten kože',
    '🫶🏿' => 'roki, sklenjeni v obliki srca: temen ten kože',
    '👐🏻' => 'odprti dlani: svetel ten kože',
    '👐🏼' => 'odprti dlani: srednje svetel ten kože',
    '👐🏽' => 'odprti dlani: srednji ten kože',
    '👐🏾' => 'odprti dlani: srednje temen ten kože',
    '👐🏿' => 'odprti dlani: temen ten kože',
    '🤲🏻' => 'dvignjeni odprti dlani: svetel ten kože',
    '🤲🏼' => 'dvignjeni odprti dlani: srednje svetel ten kože',
    '🤲🏽' => 'dvignjeni odprti dlani: srednji ten kože',
    '🤲🏾' => 'dvignjeni odprti dlani: srednje temen ten kože',
    '🤲🏿' => 'dvignjeni odprti dlani: temen ten kože',
    '🤝🏻' => 'rokovanje: svetel ten kože',
    '🤝🏼' => 'rokovanje: srednje svetel ten kože',
    '🤝🏽' => 'rokovanje: srednji ten kože',
    '🤝🏾' => 'rokovanje: srednje temen ten kože',
    '🤝🏿' => 'rokovanje: temen ten kože',
    '🙏🏻' => 'staknjeni dlani: svetel ten kože',
    '🙏🏼' => 'staknjeni dlani: srednje svetel ten kože',
    '🙏🏽' => 'staknjeni dlani: srednji ten kože',
    '🙏🏾' => 'staknjeni dlani: srednje temen ten kože',
    '🙏🏿' => 'staknjeni dlani: temen ten kože',
    '✍🏻' => 'pišoča dlan: svetel ten kože',
    '✍🏼' => 'pišoča dlan: srednje svetel ten kože',
    '✍🏽' => 'pišoča dlan: srednji ten kože',
    '✍🏾' => 'pišoča dlan: srednje temen ten kože',
    '✍🏿' => 'pišoča dlan: temen ten kože',
    '💅🏻' => 'lak za nohte: svetel ten kože',
    '💅🏼' => 'lak za nohte: srednje svetel ten kože',
    '💅🏽' => 'lak za nohte: srednji ten kože',
    '💅🏾' => 'lak za nohte: srednje temen ten kože',
    '💅🏿' => 'lak za nohte: temen ten kože',
    '🤳🏻' => 'selfi: svetel ten kože',
    '🤳🏼' => 'selfi: srednje svetel ten kože',
    '🤳🏽' => 'selfi: srednji ten kože',
    '🤳🏾' => 'selfi: srednje temen ten kože',
    '🤳🏿' => 'selfi: temen ten kože',
    '💪🏻' => 'napeti bicepsi: svetel ten kože',
    '💪🏼' => 'napeti bicepsi: srednje svetel ten kože',
    '💪🏽' => 'napeti bicepsi: srednji ten kože',
    '💪🏾' => 'napeti bicepsi: srednje temen ten kože',
    '💪🏿' => 'napeti bicepsi: temen ten kože',
    '🦵🏻' => 'noga: svetel ten kože',
    '🦵🏼' => 'noga: srednje svetel ten kože',
    '🦵🏽' => 'noga: srednji ten kože',
    '🦵🏾' => 'noga: srednje temen ten kože',
    '🦵🏿' => 'noga: temen ten kože',
    '🦶🏻' => 'stopalo: svetel ten kože',
    '🦶🏼' => 'stopalo: srednje svetel ten kože',
    '🦶🏽' => 'stopalo: srednji ten kože',
    '🦶🏾' => 'stopalo: srednje temen ten kože',
    '🦶🏿' => 'stopalo: temen ten kože',
    '👂🏻' => 'uho: svetel ten kože',
    '👂🏼' => 'uho: srednje svetel ten kože',
    '👂🏽' => 'uho: srednji ten kože',
    '👂🏾' => 'uho: srednje temen ten kože',
    '👂🏿' => 'uho: temen ten kože',
    '🦻🏻' => 'uho s slušnim aparatom: svetel ten kože',
    '🦻🏼' => 'uho s slušnim aparatom: srednje svetel ten kože',
    '🦻🏽' => 'uho s slušnim aparatom: srednji ten kože',
    '🦻🏾' => 'uho s slušnim aparatom: srednje temen ten kože',
    '🦻🏿' => 'uho s slušnim aparatom: temen ten kože',
    '👃🏻' => 'nos: svetel ten kože',
    '👃🏼' => 'nos: srednje svetel ten kože',
    '👃🏽' => 'nos: srednji ten kože',
    '👃🏾' => 'nos: srednje temen ten kože',
    '👃🏿' => 'nos: temen ten kože',
    '👶🏻' => 'dojenček: svetel ten kože',
    '👶🏼' => 'dojenček: srednje svetel ten kože',
    '👶🏽' => 'dojenček: srednji ten kože',
    '👶🏾' => 'dojenček: srednje temen ten kože',
    '👶🏿' => 'dojenček: temen ten kože',
    '🧒🏻' => 'otrok: svetel ten kože',
    '🧒🏼' => 'otrok: srednje svetel ten kože',
    '🧒🏽' => 'otrok: srednji ten kože',
    '🧒🏾' => 'otrok: srednje temen ten kože',
    '🧒🏿' => 'otrok: temen ten kože',
    '👦🏻' => 'fant: svetel ten kože',
    '👦🏼' => 'fant: srednje svetel ten kože',
    '👦🏽' => 'fant: srednji ten kože',
    '👦🏾' => 'fant: srednje temen ten kože',
    '👦🏿' => 'fant: temen ten kože',
    '👧🏻' => 'dekle: svetel ten kože',
    '👧🏼' => 'dekle: srednje svetel ten kože',
    '👧🏽' => 'dekle: srednji ten kože',
    '👧🏾' => 'dekle: srednje temen ten kože',
    '👧🏿' => 'dekle: temen ten kože',
    '🧑🏻' => 'odrasla oseba: svetel ten kože',
    '🧑🏼' => 'odrasla oseba: srednje svetel ten kože',
    '🧑🏽' => 'odrasla oseba: srednji ten kože',
    '🧑🏾' => 'odrasla oseba: srednje temen ten kože',
    '🧑🏿' => 'odrasla oseba: temen ten kože',
    '👱🏻' => 'oseba: svetel ten kože in svetli lasje',
    '👱🏼' => 'oseba: srednje svetel ten kože in svetli lasje',
    '👱🏽' => 'oseba: srednji ten kože in svetli lasje',
    '👱🏾' => 'oseba: srednje temen ten kože in svetli lasje',
    '👱🏿' => 'oseba: temen ten kože in svetli lasje',
    '👨🏻' => 'moški: svetel ten kože',
    '👨🏼' => 'moški: srednje svetel ten kože',
    '👨🏽' => 'moški: srednji ten kože',
    '👨🏾' => 'moški: srednje temen ten kože',
    '👨🏿' => 'moški: temen ten kože',
    '🧔🏻' => 'oseba: svetel ten kože in brada',
    '🧔🏼' => 'oseba: srednje svetel ten kože in brada',
    '🧔🏽' => 'oseba: srednji ten kože in brada',
    '🧔🏾' => 'oseba: srednje temen ten kože in brada',
    '🧔🏿' => 'oseba: temen ten kože in brada',
    '👩🏻' => 'ženska: svetel ten kože',
    '👩🏼' => 'ženska: srednje svetel ten kože',
    '👩🏽' => 'ženska: srednji ten kože',
    '👩🏾' => 'ženska: srednje temen ten kože',
    '👩🏿' => 'ženska: temen ten kože',
    '🧓🏻' => 'starejši odrasli: svetel ten kože',
    '🧓🏼' => 'starejši odrasli: srednje svetel ten kože',
    '🧓🏽' => 'starejši odrasli: srednji ten kože',
    '🧓🏾' => 'starejši odrasli: srednje temen ten kože',
    '🧓🏿' => 'starejši odrasli: temen ten kože',
    '👴🏻' => 'starec: svetel ten kože',
    '👴🏼' => 'starec: srednje svetel ten kože',
    '👴🏽' => 'starec: srednji ten kože',
    '👴🏾' => 'starec: srednje temen ten kože',
    '👴🏿' => 'starec: temen ten kože',
    '👵🏻' => 'starka: svetel ten kože',
    '👵🏼' => 'starka: srednje svetel ten kože',
    '👵🏽' => 'starka: srednji ten kože',
    '👵🏾' => 'starka: srednje temen ten kože',
    '👵🏿' => 'starka: temen ten kože',
    '🙍🏻' => 'oseba, ki se mršči: svetel ten kože',
    '🙍🏼' => 'oseba, ki se mršči: srednje svetel ten kože',
    '🙍🏽' => 'oseba, ki se mršči: srednji ten kože',
    '🙍🏾' => 'oseba, ki se mršči: srednje temen ten kože',
    '🙍🏿' => 'oseba, ki se mršči: temen ten kože',
    '🙎🏻' => 'oseba, ki kuha mulo: svetel ten kože',
    '🙎🏼' => 'oseba, ki kuha mulo: srednje svetel ten kože',
    '🙎🏽' => 'oseba, ki kuha mulo: srednji ten kože',
    '🙎🏾' => 'oseba, ki kuha mulo: srednje temen ten kože',
    '🙎🏿' => 'oseba, ki kuha mulo: temen ten kože',
    '🙅🏻' => 'oseba, ki kaže NE: svetel ten kože',
    '🙅🏼' => 'oseba, ki kaže NE: srednje svetel ten kože',
    '🙅🏽' => 'oseba, ki kaže NE: srednji ten kože',
    '🙅🏾' => 'oseba, ki kaže NE: srednje temen ten kože',
    '🙅🏿' => 'oseba, ki kaže NE: temen ten kože',
    '🙆🏻' => 'oseba, ki kaže V REDU: svetel ten kože',
    '🙆🏼' => 'oseba, ki kaže V REDU: srednje svetel ten kože',
    '🙆🏽' => 'oseba, ki kaže V REDU: srednji ten kože',
    '🙆🏾' => 'oseba, ki kaže V REDU: srednje temen ten kože',
    '🙆🏿' => 'oseba, ki kaže V REDU: temen ten kože',
    '💁🏻' => 'oseba, ki dela na informacijah: svetel ten kože',
    '💁🏼' => 'oseba, ki dela na informacijah: srednje svetel ten kože',
    '💁🏽' => 'oseba, ki dela na informacijah: srednji ten kože',
    '💁🏾' => 'oseba, ki dela na informacijah: srednje temen ten kože',
    '💁🏿' => 'oseba, ki dela na informacijah: temen ten kože',
    '🙋🏻' => 'vesela oseba, ki dvigne roko: svetel ten kože',
    '🙋🏼' => 'vesela oseba, ki dvigne roko: srednje svetel ten kože',
    '🙋🏽' => 'vesela oseba, ki dvigne roko: srednji ten kože',
    '🙋🏾' => 'vesela oseba, ki dvigne roko: srednje temen ten kože',
    '🙋🏿' => 'vesela oseba, ki dvigne roko: temen ten kože',
    '🧏🏻' => 'gluha oseba: svetel ten kože',
    '🧏🏼' => 'gluha oseba: srednje svetel ten kože',
    '🧏🏽' => 'gluha oseba: srednji ten kože',
    '🧏🏾' => 'gluha oseba: srednje temen ten kože',
    '🧏🏿' => 'gluha oseba: temen ten kože',
    '🙇🏻' => 'oseba, ki se prikloni: svetel ten kože',
    '🙇🏼' => 'oseba, ki se prikloni: srednje svetel ten kože',
    '🙇🏽' => 'oseba, ki se prikloni: srednji ten kože',
    '🙇🏾' => 'oseba, ki se prikloni: srednje temen ten kože',
    '🙇🏿' => 'oseba, ki se prikloni: temen ten kože',
    '🤦🏻' => 'oseba z dlanjo na obrazu: svetel ten kože',
    '🤦🏼' => 'oseba z dlanjo na obrazu: srednje svetel ten kože',
    '🤦🏽' => 'oseba z dlanjo na obrazu: srednji ten kože',
    '🤦🏾' => 'oseba z dlanjo na obrazu: srednje temen ten kože',
    '🤦🏿' => 'oseba z dlanjo na obrazu: temen ten kože',
    '🤷🏻' => 'oseba, ki skomiga: svetel ten kože',
    '🤷🏼' => 'oseba, ki skomiga: srednje svetel ten kože',
    '🤷🏽' => 'oseba, ki skomiga: srednji ten kože',
    '🤷🏾' => 'oseba, ki skomiga: srednje temen ten kože',
    '🤷🏿' => 'oseba, ki skomiga: temen ten kože',
    '👮🏻' => 'policaj: svetel ten kože',
    '👮🏼' => 'policaj: srednje svetel ten kože',
    '👮🏽' => 'policaj: srednji ten kože',
    '👮🏾' => 'policaj: srednje temen ten kože',
    '👮🏿' => 'policaj: temen ten kože',
    '🕵🏻' => 'preiskovalec: svetel ten kože',
    '🕵🏼' => 'preiskovalec: srednje svetel ten kože',
    '🕵🏽' => 'preiskovalec: srednji ten kože',
    '🕵🏾' => 'preiskovalec: srednje temen ten kože',
    '🕵🏿' => 'preiskovalec: temen ten kože',
    '💂🏻' => 'stražar: svetel ten kože',
    '💂🏼' => 'stražar: srednje svetel ten kože',
    '💂🏽' => 'stražar: srednji ten kože',
    '💂🏾' => 'stražar: srednje temen ten kože',
    '💂🏿' => 'stražar: temen ten kože',
    '🥷🏻' => 'nindža: svetel ten kože',
    '🥷🏼' => 'nindža: srednje svetel ten kože',
    '🥷🏽' => 'nindža: srednji ten kože',
    '🥷🏾' => 'nindža: srednje temen ten kože',
    '🥷🏿' => 'nindža: temen ten kože',
    '👷🏻' => 'gradbenik: svetel ten kože',
    '👷🏼' => 'gradbenik: srednje svetel ten kože',
    '👷🏽' => 'gradbenik: srednji ten kože',
    '👷🏾' => 'gradbenik: srednje temen ten kože',
    '👷🏿' => 'gradbenik: temen ten kože',
    '🫅🏻' => 'oseba s krono: svetel ten kože',
    '🫅🏼' => 'oseba s krono: srednje svetel ten kože',
    '🫅🏽' => 'oseba s krono: srednji ten kože',
    '🫅🏾' => 'oseba s krono: srednje temen ten kože',
    '🫅🏿' => 'oseba s krono: temen ten kože',
    '🤴🏻' => 'princ: svetel ten kože',
    '🤴🏼' => 'princ: srednje svetel ten kože',
    '🤴🏽' => 'princ: srednji ten kože',
    '🤴🏾' => 'princ: srednje temen ten kože',
    '🤴🏿' => 'princ: temen ten kože',
    '👸🏻' => 'princesa: svetel ten kože',
    '👸🏼' => 'princesa: srednje svetel ten kože',
    '👸🏽' => 'princesa: srednji ten kože',
    '👸🏾' => 'princesa: srednje temen ten kože',
    '👸🏿' => 'princesa: temen ten kože',
    '👳🏻' => 'oseba s turbanom: svetel ten kože',
    '👳🏼' => 'oseba s turbanom: srednje svetel ten kože',
    '👳🏽' => 'oseba s turbanom: srednji ten kože',
    '👳🏾' => 'oseba s turbanom: srednje temen ten kože',
    '👳🏿' => 'oseba s turbanom: temen ten kože',
    '👲🏻' => 'moški s kitajsko čepico: svetel ten kože',
    '👲🏼' => 'moški s kitajsko čepico: srednje svetel ten kože',
    '👲🏽' => 'moški s kitajsko čepico: srednji ten kože',
    '👲🏾' => 'moški s kitajsko čepico: srednje temen ten kože',
    '👲🏿' => 'moški s kitajsko čepico: temen ten kože',
    '🧕🏻' => 'ženska z naglavno ruto: svetel ten kože',
    '🧕🏼' => 'ženska z naglavno ruto: srednje svetel ten kože',
    '🧕🏽' => 'ženska z naglavno ruto: srednji ten kože',
    '🧕🏾' => 'ženska z naglavno ruto: srednje temen ten kože',
    '🧕🏿' => 'ženska z naglavno ruto: temen ten kože',
    '🤵🏻' => 'oseba v smokingu: svetel ten kože',
    '🤵🏼' => 'oseba v smokingu: srednje svetel ten kože',
    '🤵🏽' => 'oseba v smokingu: srednji ten kože',
    '🤵🏾' => 'oseba v smokingu: srednje temen ten kože',
    '🤵🏿' => 'oseba v smokingu: temen ten kože',
    '👰🏻' => 'oseba s pajčolanom: svetel ten kože',
    '👰🏼' => 'oseba s pajčolanom: srednje svetel ten kože',
    '👰🏽' => 'oseba s pajčolanom: srednji ten kože',
    '👰🏾' => 'oseba s pajčolanom: srednje temen ten kože',
    '👰🏿' => 'oseba s pajčolanom: temen ten kože',
    '🤰🏻' => 'nosečnica: svetel ten kože',
    '🤰🏼' => 'nosečnica: srednje svetel ten kože',
    '🤰🏽' => 'nosečnica: srednji ten kože',
    '🤰🏾' => 'nosečnica: srednje temen ten kože',
    '🤰🏿' => 'nosečnica: temen ten kože',
    '🫃🏻' => 'noseč moški: svetel ten kože',
    '🫃🏼' => 'noseč moški: srednje svetel ten kože',
    '🫃🏽' => 'noseč moški: srednji ten kože',
    '🫃🏾' => 'noseč moški: srednje temen ten kože',
    '🫃🏿' => 'noseč moški: temen ten kože',
    '🫄🏻' => 'noseča oseba: svetel ten kože',
    '🫄🏼' => 'noseča oseba: srednje svetel ten kože',
    '🫄🏽' => 'noseča oseba: srednji ten kože',
    '🫄🏾' => 'noseča oseba: srednje temen ten kože',
    '🫄🏿' => 'noseča oseba: temen ten kože',
    '🤱🏻' => 'dojenje: svetel ten kože',
    '🤱🏼' => 'dojenje: srednje svetel ten kože',
    '🤱🏽' => 'dojenje: srednji ten kože',
    '🤱🏾' => 'dojenje: srednje temen ten kože',
    '🤱🏿' => 'dojenje: temen ten kože',
    '👼🏻' => 'angelček: svetel ten kože',
    '👼🏼' => 'angelček: srednje svetel ten kože',
    '👼🏽' => 'angelček: srednji ten kože',
    '👼🏾' => 'angelček: srednje temen ten kože',
    '👼🏿' => 'angelček: temen ten kože',
    '🎅🏻' => 'Božiček: svetel ten kože',
    '🎅🏼' => 'Božiček: srednje svetel ten kože',
    '🎅🏽' => 'Božiček: srednji ten kože',
    '🎅🏾' => 'Božiček: srednje temen ten kože',
    '🎅🏿' => 'Božiček: temen ten kože',
    '🤶🏻' => 'gospa Božiček: svetel ten kože',
    '🤶🏼' => 'gospa Božiček: srednje svetel ten kože',
    '🤶🏽' => 'gospa Božiček: srednji ten kože',
    '🤶🏾' => 'gospa Božiček: srednje temen ten kože',
    '🤶🏿' => 'gospa Božiček: temen ten kože',
    '🦸🏻' => 'superjunak/superjunakinja: svetel ten kože',
    '🦸🏼' => 'superjunak/superjunakinja: srednje svetel ten kože',
    '🦸🏽' => 'superjunak/superjunakinja: srednji ten kože',
    '🦸🏾' => 'superjunak/superjunakinja: srednje temen ten kože',
    '🦸🏿' => 'superjunak/superjunakinja: temen ten kože',
    '🦹🏻' => 'super negativec: svetel ten kože',
    '🦹🏼' => 'super negativec: srednje svetel ten kože',
    '🦹🏽' => 'super negativec: srednji ten kože',
    '🦹🏾' => 'super negativec: srednje temen ten kože',
    '🦹🏿' => 'super negativec: temen ten kože',
    '🧙🏻' => 'oseba, ki čara: svetel ten kože',
    '🧙🏼' => 'oseba, ki čara: srednje svetel ten kože',
    '🧙🏽' => 'oseba, ki čara: srednji ten kože',
    '🧙🏾' => 'oseba, ki čara: srednje temen ten kože',
    '🧙🏿' => 'oseba, ki čara: temen ten kože',
    '🧚🏻' => 'vilinsko bitje: svetel ten kože',
    '🧚🏼' => 'vilinsko bitje: srednje svetel ten kože',
    '🧚🏽' => 'vilinsko bitje: srednji ten kože',
    '🧚🏾' => 'vilinsko bitje: srednje temen ten kože',
    '🧚🏿' => 'vilinsko bitje: temen ten kože',
    '🧛🏻' => 'vampirji: svetel ten kože',
    '🧛🏼' => 'vampirji: srednje svetel ten kože',
    '🧛🏽' => 'vampirji: srednji ten kože',
    '🧛🏾' => 'vampirji: srednje temen ten kože',
    '🧛🏿' => 'vampirji: temen ten kože',
    '🧜🏻' => 'morska oseba: svetel ten kože',
    '🧜🏼' => 'morska oseba: srednje svetel ten kože',
    '🧜🏽' => 'morska oseba: srednji ten kože',
    '🧜🏾' => 'morska oseba: srednje temen ten kože',
    '🧜🏿' => 'morska oseba: temen ten kože',
    '🧝🏻' => 'škrati: svetel ten kože',
    '🧝🏼' => 'škrati: srednje svetel ten kože',
    '🧝🏽' => 'škrati: srednji ten kože',
    '🧝🏾' => 'škrati: srednje temen ten kože',
    '🧝🏿' => 'škrati: temen ten kože',
    '💆🏻' => 'masaža obraza: svetel ten kože',
    '💆🏼' => 'masaža obraza: srednje svetel ten kože',
    '💆🏽' => 'masaža obraza: srednji ten kože',
    '💆🏾' => 'masaža obraza: srednje temen ten kože',
    '💆🏿' => 'masaža obraza: temen ten kože',
    '💇🏻' => 'striženje: svetel ten kože',
    '💇🏼' => 'striženje: srednje svetel ten kože',
    '💇🏽' => 'striženje: srednji ten kože',
    '💇🏾' => 'striženje: srednje temen ten kože',
    '💇🏿' => 'striženje: temen ten kože',
    '🚶🏻' => 'pešec: svetel ten kože',
    '🚶🏼' => 'pešec: srednje svetel ten kože',
    '🚶🏽' => 'pešec: srednji ten kože',
    '🚶🏾' => 'pešec: srednje temen ten kože',
    '🚶🏿' => 'pešec: temen ten kože',
    '🧍🏻' => 'stoječa oseba: svetel ten kože',
    '🧍🏼' => 'stoječa oseba: srednje svetel ten kože',
    '🧍🏽' => 'stoječa oseba: srednji ten kože',
    '🧍🏾' => 'stoječa oseba: srednje temen ten kože',
    '🧍🏿' => 'stoječa oseba: temen ten kože',
    '🧎🏻' => 'klečeča oseba: svetel ten kože',
    '🧎🏼' => 'klečeča oseba: srednje svetel ten kože',
    '🧎🏽' => 'klečeča oseba: srednji ten kože',
    '🧎🏾' => 'klečeča oseba: srednje temen ten kože',
    '🧎🏿' => 'klečeča oseba: temen ten kože',
    '🏃🏻' => 'tekač: svetel ten kože',
    '🏃🏼' => 'tekač: srednje svetel ten kože',
    '🏃🏽' => 'tekač: srednji ten kože',
    '🏃🏾' => 'tekač: srednje temen ten kože',
    '🏃🏿' => 'tekač: temen ten kože',
    '💃🏻' => 'ženska, ki pleše: svetel ten kože',
    '💃🏼' => 'ženska, ki pleše: srednje svetel ten kože',
    '💃🏽' => 'ženska, ki pleše: srednji ten kože',
    '💃🏾' => 'ženska, ki pleše: srednje temen ten kože',
    '💃🏿' => 'ženska, ki pleše: temen ten kože',
    '🕺🏻' => 'plesalec: svetel ten kože',
    '🕺🏼' => 'plesalec: srednje svetel ten kože',
    '🕺🏽' => 'plesalec: srednji ten kože',
    '🕺🏾' => 'plesalec: srednje temen ten kože',
    '🕺🏿' => 'plesalec: temen ten kože',
    '🕴🏻' => 'moški v poslovni obleki, ki lebdi: svetel ten kože',
    '🕴🏼' => 'moški v poslovni obleki, ki lebdi: srednje svetel ten kože',
    '🕴🏽' => 'moški v poslovni obleki, ki lebdi: srednji ten kože',
    '🕴🏾' => 'moški v poslovni obleki, ki lebdi: srednje temen ten kože',
    '🕴🏿' => 'moški v poslovni obleki, ki lebdi: temen ten kože',
    '👯🏻' => 'žurerski ženski: svetel ten kože',
    '👯🏼' => 'žurerski ženski: srednje svetel ten kože',
    '👯🏽' => 'žurerski ženski: srednji ten kože',
    '👯🏾' => 'žurerski ženski: srednje temen ten kože',
    '👯🏿' => 'žurerski ženski: temen ten kože',
    '🧖🏻' => 'oseba v parni savni: svetel ten kože',
    '🧖🏼' => 'oseba v parni savni: srednje svetel ten kože',
    '🧖🏽' => 'oseba v parni savni: srednji ten kože',
    '🧖🏾' => 'oseba v parni savni: srednje temen ten kože',
    '🧖🏿' => 'oseba v parni savni: temen ten kože',
    '🧗🏻' => 'oseba, ki pleza: svetel ten kože',
    '🧗🏼' => 'oseba, ki pleza: srednje svetel ten kože',
    '🧗🏽' => 'oseba, ki pleza: srednji ten kože',
    '🧗🏾' => 'oseba, ki pleza: srednje temen ten kože',
    '🧗🏿' => 'oseba, ki pleza: temen ten kože',
    '🏇🏻' => 'konjske dirke: svetel ten kože',
    '🏇🏼' => 'konjske dirke: srednje svetel ten kože',
    '🏇🏽' => 'konjske dirke: srednji ten kože',
    '🏇🏾' => 'konjske dirke: srednje temen ten kože',
    '🏇🏿' => 'konjske dirke: temen ten kože',
    '🏂🏻' => 'deskar na snegu: svetel ten kože',
    '🏂🏼' => 'deskar na snegu: srednje svetel ten kože',
    '🏂🏽' => 'deskar na snegu: srednji ten kože',
    '🏂🏾' => 'deskar na snegu: srednje temen ten kože',
    '🏂🏿' => 'deskar na snegu: temen ten kože',
    '🏌🏻' => 'igralec golfa: svetel ten kože',
    '🏌🏼' => 'igralec golfa: srednje svetel ten kože',
    '🏌🏽' => 'igralec golfa: srednji ten kože',
    '🏌🏾' => 'igralec golfa: srednje temen ten kože',
    '🏌🏿' => 'igralec golfa: temen ten kože',
    '🏄🏻' => 'oseba deska: svetel ten kože',
    '🏄🏼' => 'oseba deska: srednje svetel ten kože',
    '🏄🏽' => 'oseba deska: srednji ten kože',
    '🏄🏾' => 'oseba deska: srednje temen ten kože',
    '🏄🏿' => 'oseba deska: temen ten kože',
    '🚣🏻' => 'oseba v čolnu na vesla: svetel ten kože',
    '🚣🏼' => 'oseba v čolnu na vesla: srednje svetel ten kože',
    '🚣🏽' => 'oseba v čolnu na vesla: srednji ten kože',
    '🚣🏾' => 'oseba v čolnu na vesla: srednje temen ten kože',
    '🚣🏿' => 'oseba v čolnu na vesla: temen ten kože',
    '🏊🏻' => 'oseba plava: svetel ten kože',
    '🏊🏼' => 'oseba plava: srednje svetel ten kože',
    '🏊🏽' => 'oseba plava: srednji ten kože',
    '🏊🏾' => 'oseba plava: srednje temen ten kože',
    '🏊🏿' => 'oseba plava: temen ten kože',
    '⛹🏻' => 'oseba z žogo: svetel ten kože',
    '⛹🏼' => 'oseba z žogo: srednje svetel ten kože',
    '⛹🏽' => 'oseba z žogo: srednji ten kože',
    '⛹🏾' => 'oseba z žogo: srednje temen ten kože',
    '⛹🏿' => 'oseba z žogo: temen ten kože',
    '🏋🏻' => 'oseba dviga uteži: svetel ten kože',
    '🏋🏼' => 'oseba dviga uteži: srednje svetel ten kože',
    '🏋🏽' => 'oseba dviga uteži: srednji ten kože',
    '🏋🏾' => 'oseba dviga uteži: srednje temen ten kože',
    '🏋🏿' => 'oseba dviga uteži: temen ten kože',
    '🚴🏻' => 'oseba na kolesu: svetel ten kože',
    '🚴🏼' => 'oseba na kolesu: srednje svetel ten kože',
    '🚴🏽' => 'oseba na kolesu: srednji ten kože',
    '🚴🏾' => 'oseba na kolesu: srednje temen ten kože',
    '🚴🏿' => 'oseba na kolesu: temen ten kože',
    '🚵🏻' => 'gorski kolesar: svetel ten kože',
    '🚵🏼' => 'gorski kolesar: srednje svetel ten kože',
    '🚵🏽' => 'gorski kolesar: srednji ten kože',
    '🚵🏾' => 'gorski kolesar: srednje temen ten kože',
    '🚵🏿' => 'gorski kolesar: temen ten kože',
    '🤸🏻' => 'oseba izvaja kolo: svetel ten kože',
    '🤸🏼' => 'oseba izvaja kolo: srednje svetel ten kože',
    '🤸🏽' => 'oseba izvaja kolo: srednji ten kože',
    '🤸🏾' => 'oseba izvaja kolo: srednje temen ten kože',
    '🤸🏿' => 'oseba izvaja kolo: temen ten kože',
    '🤼🏻' => 'rokoborba: svetel ten kože',
    '🤼🏼' => 'rokoborba: srednje svetel ten kože',
    '🤼🏽' => 'rokoborba: srednji ten kože',
    '🤼🏾' => 'rokoborba: srednje temen ten kože',
    '🤼🏿' => 'rokoborba: temen ten kože',
    '🤽🏻' => 'vaterpolo: svetel ten kože',
    '🤽🏼' => 'vaterpolo: srednje svetel ten kože',
    '🤽🏽' => 'vaterpolo: srednji ten kože',
    '🤽🏾' => 'vaterpolo: srednje temen ten kože',
    '🤽🏿' => 'vaterpolo: temen ten kože',
    '🤾🏻' => 'rokomet: svetel ten kože',
    '🤾🏼' => 'rokomet: srednje svetel ten kože',
    '🤾🏽' => 'rokomet: srednji ten kože',
    '🤾🏾' => 'rokomet: srednje temen ten kože',
    '🤾🏿' => 'rokomet: temen ten kože',
    '🤹🏻' => 'žongliranje: svetel ten kože',
    '🤹🏼' => 'žongliranje: srednje svetel ten kože',
    '🤹🏽' => 'žongliranje: srednji ten kože',
    '🤹🏾' => 'žongliranje: srednje temen ten kože',
    '🤹🏿' => 'žongliranje: temen ten kože',
    '🧘🏻' => 'oseba v lotusovem položaju: svetel ten kože',
    '🧘🏼' => 'oseba v lotusovem položaju: srednje svetel ten kože',
    '🧘🏽' => 'oseba v lotusovem položaju: srednji ten kože',
    '🧘🏾' => 'oseba v lotusovem položaju: srednje temen ten kože',
    '🧘🏿' => 'oseba v lotusovem položaju: temen ten kože',
    '🛀🏻' => 'oseba, ki se kopa: svetel ten kože',
    '🛀🏼' => 'oseba, ki se kopa: srednje svetel ten kože',
    '🛀🏽' => 'oseba, ki se kopa: srednji ten kože',
    '🛀🏾' => 'oseba, ki se kopa: srednje temen ten kože',
    '🛀🏿' => 'oseba, ki se kopa: temen ten kože',
    '🛌🏻' => 'oseba v postelji: svetel ten kože',
    '🛌🏼' => 'oseba v postelji: srednje svetel ten kože',
    '🛌🏽' => 'oseba v postelji: srednji ten kože',
    '🛌🏾' => 'oseba v postelji: srednje temen ten kože',
    '🛌🏿' => 'oseba v postelji: temen ten kože',
    '👭🏻' => 'ženski se držita za roke: svetel ten kože',
    '👭🏼' => 'ženski se držita za roke: srednje svetel ten kože',
    '👭🏽' => 'ženski se držita za roke: srednji ten kože',
    '👭🏾' => 'ženski se držita za roke: srednje temen ten kože',
    '👭🏿' => 'ženski se držita za roke: temen ten kože',
    '👫🏻' => 'moški in ženska se držita za roke: svetel ten kože',
    '👫🏼' => 'moški in ženska se držita za roke: srednje svetel ten kože',
    '👫🏽' => 'moški in ženska se držita za roke: srednji ten kože',
    '👫🏾' => 'moški in ženska se držita za roke: srednje temen ten kože',
    '👫🏿' => 'moški in ženska se držita za roke: temen ten kože',
    '👬🏻' => 'moška se držita za roke: svetel ten kože',
    '👬🏼' => 'moška se držita za roke: srednje svetel ten kože',
    '👬🏽' => 'moška se držita za roke: srednji ten kože',
    '👬🏾' => 'moška se držita za roke: srednje temen ten kože',
    '👬🏿' => 'moška se držita za roke: temen ten kože',
    '💏🏻' => 'poljub: svetel ten kože',
    '💏🏼' => 'poljub: srednje svetel ten kože',
    '💏🏽' => 'poljub: srednji ten kože',
    '💏🏾' => 'poljub: srednje temen ten kože',
    '💏🏿' => 'poljub: temen ten kože',
    '💑🏻' => 'par s srčkom: svetel ten kože',
    '💑🏼' => 'par s srčkom: srednje svetel ten kože',
    '💑🏽' => 'par s srčkom: srednji ten kože',
    '💑🏾' => 'par s srčkom: srednje temen ten kože',
    '💑🏿' => 'par s srčkom: temen ten kože',
    '🇦🇨' => 'zastava: Otok Ascension',
    '🇦🇩' => 'zastava: Andora',
    '🇦🇪' => 'zastava: Združeni arabski emirati',
    '🇦🇫' => 'zastava: Afganistan',
    '🇦🇬' => 'zastava: Antigva in Barbuda',
    '🇦🇮' => 'zastava: Angvila',
    '🇦🇱' => 'zastava: Albanija',
    '🇦🇲' => 'zastava: Armenija',
    '🇦🇴' => 'zastava: Angola',
    '🇦🇶' => 'zastava: Antarktika',
    '🇦🇷' => 'zastava: Argentina',
    '🇦🇸' => 'zastava: Ameriška Samoa',
    '🇦🇹' => 'zastava: Avstrija',
    '🇦🇺' => 'zastava: Avstralija',
    '🇦🇼' => 'zastava: Aruba',
    '🇦🇽' => 'zastava: Alandski otoki',
    '🇦🇿' => 'zastava: Azerbajdžan',
    '🇧🇦' => 'zastava: Bosna in Hercegovina',
    '🇧🇧' => 'zastava: Barbados',
    '🇧🇩' => 'zastava: Bangladeš',
    '🇧🇪' => 'zastava: Belgija',
    '🇧🇫' => 'zastava: Burkina Faso',
    '🇧🇬' => 'zastava: Bolgarija',
    '🇧🇭' => 'zastava: Bahrajn',
    '🇧🇮' => 'zastava: Burundi',
    '🇧🇯' => 'zastava: Benin',
    '🇧🇱' => 'zastava: Sveti Bartolomej',
    '🇧🇲' => 'zastava: Bermudi',
    '🇧🇳' => 'zastava: Brunej',
    '🇧🇴' => 'zastava: Bolivija',
    '🇧🇶' => 'zastava: Nizozemski Karibi',
    '🇧🇷' => 'zastava: Brazilija',
    '🇧🇸' => 'zastava: Bahami',
    '🇧🇹' => 'zastava: Butan',
    '🇧🇻' => 'zastava: Bouvetov otok',
    '🇧🇼' => 'zastava: Bocvana',
    '🇧🇾' => 'zastava: Belorusija',
    '🇧🇿' => 'zastava: Belize',
    '🇨🇦' => 'zastava: Kanada',
    '🇨🇨' => 'zastava: Kokosovi otoki',
    '🇨🇩' => 'zastava: Demokratična republika Kongo',
    '🇨🇫' => 'zastava: Srednjeafriška republika',
    '🇨🇬' => 'zastava: Kongo - Brazzaville',
    '🇨🇭' => 'zastava: Švica',
    '🇨🇮' => 'zastava: Slonokoščena obala',
    '🇨🇰' => 'zastava: Cookovi otoki',
    '🇨🇱' => 'zastava: Čile',
    '🇨🇲' => 'zastava: Kamerun',
    '🇨🇳' => 'zastava: Kitajska',
    '🇨🇴' => 'zastava: Kolumbija',
    '🇨🇵' => 'zastava: Otok Clipperton',
    '🇨🇶' => 'zastava: Sark',
    '🇨🇷' => 'zastava: Kostarika',
    '🇨🇺' => 'zastava: Kuba',
    '🇨🇻' => 'zastava: Zelenortski otoki',
    '🇨🇼' => 'zastava: Curaçao',
    '🇨🇽' => 'zastava: Božični otok',
    '🇨🇾' => 'zastava: Ciper',
    '🇨🇿' => 'zastava: Češka',
    '🇩🇪' => 'zastava: Nemčija',
    '🇩🇬' => 'zastava: Diego Garcia',
    '🇩🇯' => 'zastava: Džibuti',
    '🇩🇰' => 'zastava: Danska',
    '🇩🇲' => 'zastava: Dominika',
    '🇩🇴' => 'zastava: Dominikanska republika',
    '🇩🇿' => 'zastava: Alžirija',
    '🇪🇦' => 'zastava: Ceuta in Melilla',
    '🇪🇨' => 'zastava: Ekvador',
    '🇪🇪' => 'zastava: Estonija',
    '🇪🇬' => 'zastava: Egipt',
    '🇪🇭' => 'zastava: Zahodna Sahara',
    '🇪🇷' => 'zastava: Eritreja',
    '🇪🇸' => 'zastava: Španija',
    '🇪🇹' => 'zastava: Etiopija',
    '🇪🇺' => 'zastava: Evropska unija',
    '🇫🇮' => 'zastava: Finska',
    '🇫🇯' => 'zastava: Fidži',
    '🇫🇰' => 'zastava: Falklandski otoki',
    '🇫🇲' => 'zastava: Mikronezija',
    '🇫🇴' => 'zastava: Ferski otoki',
    '🇫🇷' => 'zastava: Francija',
    '🇬🇦' => 'zastava: Gabon',
    '🇬🇧' => 'zastava: Združeno kraljestvo',
    '🇬🇩' => 'zastava: Grenada',
    '🇬🇪' => 'zastava: Gruzija',
    '🇬🇫' => 'zastava: Francoska Gvajana',
    '🇬🇬' => 'zastava: Guernsey',
    '🇬🇭' => 'zastava: Gana',
    '🇬🇮' => 'zastava: Gibraltar',
    '🇬🇱' => 'zastava: Grenlandija',
    '🇬🇲' => 'zastava: Gambija',
    '🇬🇳' => 'zastava: Gvineja',
    '🇬🇵' => 'zastava: Gvadelup',
    '🇬🇶' => 'zastava: Ekvatorialna Gvineja',
    '🇬🇷' => 'zastava: Grčija',
    '🇬🇸' => 'zastava: Južna Georgia in Južni Sandwichevi otoki',
    '🇬🇹' => 'zastava: Gvatemala',
    '🇬🇺' => 'zastava: Gvam',
    '🇬🇼' => 'zastava: Gvineja Bissau',
    '🇬🇾' => 'zastava: Gvajana',
    '🇭🇰' => 'zastava: Posebno upravno območje Ljudske republike Kitajske Hongkong',
    '🇭🇲' => 'zastava: Heardov otok in McDonaldovi otoki',
    '🇭🇳' => 'zastava: Honduras',
    '🇭🇷' => 'zastava: Hrvaška',
    '🇭🇹' => 'zastava: Haiti',
    '🇭🇺' => 'zastava: Madžarska',
    '🇮🇨' => 'zastava: Kanarski otoki',
    '🇮🇩' => 'zastava: Indonezija',
    '🇮🇪' => 'zastava: Irska',
    '🇮🇱' => 'zastava: Izrael',
    '🇮🇲' => 'zastava: Otok Man',
    '🇮🇳' => 'zastava: Indija',
    '🇮🇴' => 'zastava: Britansko ozemlje v Indijskem oceanu',
    '🇮🇶' => 'zastava: Irak',
    '🇮🇷' => 'zastava: Iran',
    '🇮🇸' => 'zastava: Islandija',
    '🇮🇹' => 'zastava: Italija',
    '🇯🇪' => 'zastava: Jersey',
    '🇯🇲' => 'zastava: Jamajka',
    '🇯🇴' => 'zastava: Jordanija',
    '🇯🇵' => 'zastava: Japonska',
    '🇰🇪' => 'zastava: Kenija',
    '🇰🇬' => 'zastava: Kirgizistan',
    '🇰🇭' => 'zastava: Kambodža',
    '🇰🇮' => 'zastava: Kiribati',
    '🇰🇲' => 'zastava: Komori',
    '🇰🇳' => 'zastava: Sveti Krištof in Nevis',
    '🇰🇵' => 'zastava: Severna Koreja',
    '🇰🇷' => 'zastava: Južna Koreja',
    '🇰🇼' => 'zastava: Kuvajt',
    '🇰🇾' => 'zastava: Kajmanji otoki',
    '🇰🇿' => 'zastava: Kazahstan',
    '🇱🇦' => 'zastava: Laos',
    '🇱🇧' => 'zastava: Libanon',
    '🇱🇨' => 'zastava: Sveta Lucija',
    '🇱🇮' => 'zastava: Lihtenštajn',
    '🇱🇰' => 'zastava: Šrilanka',
    '🇱🇷' => 'zastava: Liberija',
    '🇱🇸' => 'zastava: Lesoto',
    '🇱🇹' => 'zastava: Litva',
    '🇱🇺' => 'zastava: Luksemburg',
    '🇱🇻' => 'zastava: Latvija',
    '🇱🇾' => 'zastava: Libija',
    '🇲🇦' => 'zastava: Maroko',
    '🇲🇨' => 'zastava: Monako',
    '🇲🇩' => 'zastava: Moldavija',
    '🇲🇪' => 'zastava: Črna gora',
    '🇲🇫' => 'zastava: Francoski Sveti Martin',
    '🇲🇬' => 'zastava: Madagaskar',
    '🇲🇭' => 'zastava: Marshallovi otoki',
    '🇲🇰' => 'zastava: Severna Makedonija',
    '🇲🇱' => 'zastava: Mali',
    '🇲🇲' => 'zastava: Mjanmar (Burma)',
    '🇲🇳' => 'zastava: Mongolija',
    '🇲🇴' => 'zastava: Posebno upravno območje Kitajske Makav',
    '🇲🇵' => 'zastava: Severni Marianski otoki',
    '🇲🇶' => 'zastava: Martinik',
    '🇲🇷' => 'zastava: Mavretanija',
    '🇲🇸' => 'zastava: Montserrat',
    '🇲🇹' => 'zastava: Malta',
    '🇲🇺' => 'zastava: Mavricij',
    '🇲🇻' => 'zastava: Maldivi',
    '🇲🇼' => 'zastava: Malavi',
    '🇲🇽' => 'zastava: Mehika',
    '🇲🇾' => 'zastava: Malezija',
    '🇲🇿' => 'zastava: Mozambik',
    '🇳🇦' => 'zastava: Namibija',
    '🇳🇨' => 'zastava: Nova Kaledonija',
    '🇳🇪' => 'zastava: Niger',
    '🇳🇫' => 'zastava: Norfolški otok',
    '🇳🇬' => 'zastava: Nigerija',
    '🇳🇮' => 'zastava: Nikaragva',
    '🇳🇱' => 'zastava: Nizozemska',
    '🇳🇴' => 'zastava: Norveška',
    '🇳🇵' => 'zastava: Nepal',
    '🇳🇷' => 'zastava: Nauru',
    '🇳🇺' => 'zastava: Niue',
    '🇳🇿' => 'zastava: Nova Zelandija',
    '🇴🇲' => 'zastava: Oman',
    '🇵🇦' => 'zastava: Panama',
    '🇵🇪' => 'zastava: Peru',
    '🇵🇫' => 'zastava: Francoska Polinezija',
    '🇵🇬' => 'zastava: Papua Nova Gvineja',
    '🇵🇭' => 'zastava: Filipini',
    '🇵🇰' => 'zastava: Pakistan',
    '🇵🇱' => 'zastava: Poljska',
    '🇵🇲' => 'zastava: Sveta Peter in Mihael',
    '🇵🇳' => 'zastava: Pitcairn',
    '🇵🇷' => 'zastava: Portoriko',
    '🇵🇸' => 'zastava: Palestinsko ozemlje',
    '🇵🇹' => 'zastava: Portugalska',
    '🇵🇼' => 'zastava: Palav',
    '🇵🇾' => 'zastava: Paragvaj',
    '🇶🇦' => 'zastava: Katar',
    '🇷🇪' => 'zastava: Reunion',
    '🇷🇴' => 'zastava: Romunija',
    '🇷🇸' => 'zastava: Srbija',
    '🇷🇺' => 'zastava: Rusija',
    '🇷🇼' => 'zastava: Ruanda',
    '🇸🇦' => 'zastava: Saudova Arabija',
    '🇸🇧' => 'zastava: Salomonovi otoki',
    '🇸🇨' => 'zastava: Sejšeli',
    '🇸🇩' => 'zastava: Sudan',
    '🇸🇪' => 'zastava: Švedska',
    '🇸🇬' => 'zastava: Singapur',
    '🇸🇭' => 'zastava: Sveta Helena',
    '🇸🇮' => 'zastava: Slovenija',
    '🇸🇯' => 'zastava: Svalbard in Jan Mayen',
    '🇸🇰' => 'zastava: Slovaška',
    '🇸🇱' => 'zastava: Sierra Leone',
    '🇸🇲' => 'zastava: San Marino',
    '🇸🇳' => 'zastava: Senegal',
    '🇸🇴' => 'zastava: Somalija',
    '🇸🇷' => 'zastava: Surinam',
    '🇸🇸' => 'zastava: Južni Sudan',
    '🇸🇹' => 'zastava: Sveti Tomaž in Princ',
    '🇸🇻' => 'zastava: Salvador',
    '🇸🇽' => 'zastava: Nizozemski Sveti Martin',
    '🇸🇾' => 'zastava: Sirija',
    '🇸🇿' => 'zastava: Esvatini',
    '🇹🇦' => 'zastava: Tristan da Cunha',
    '🇹🇨' => 'zastava: Otoki Turks in Caicos',
    '🇹🇩' => 'zastava: Čad',
    '🇹🇫' => 'zastava: Francosko južno ozemlje',
    '🇹🇬' => 'zastava: Togo',
    '🇹🇭' => 'zastava: Tajska',
    '🇹🇯' => 'zastava: Tadžikistan',
    '🇹🇰' => 'zastava: Tokelav',
    '🇹🇱' => 'zastava: Timor-Leste',
    '🇹🇲' => 'zastava: Turkmenistan',
    '🇹🇳' => 'zastava: Tunizija',
    '🇹🇴' => 'zastava: Tonga',
    '🇹🇷' => 'zastava: Turčija',
    '🇹🇹' => 'zastava: Trinidad in Tobago',
    '🇹🇻' => 'zastava: Tuvalu',
    '🇹🇼' => 'zastava: Tajvan',
    '🇹🇿' => 'zastava: Tanzanija',
    '🇺🇦' => 'zastava: Ukrajina',
    '🇺🇬' => 'zastava: Uganda',
    '🇺🇲' => 'zastava: Stranski zunanji otoki Združenih držav',
    '🇺🇳' => 'zastava: Združeni narodi',
    '🇺🇸' => 'zastava: Združene države Amerike',
    '🇺🇾' => 'zastava: Urugvaj',
    '🇺🇿' => 'zastava: Uzbekistan',
    '🇻🇦' => 'zastava: Vatikan',
    '🇻🇨' => 'zastava: Sveti Vincencij in Grenadine',
    '🇻🇪' => 'zastava: Venezuela',
    '🇻🇬' => 'zastava: Britanski Deviški otoki',
    '🇻🇮' => 'zastava: Ameriški Deviški otoki',
    '🇻🇳' => 'zastava: Vietnam',
    '🇻🇺' => 'zastava: Vanuatu',
    '🇼🇫' => 'zastava: Wallis in Futuna',
    '🇼🇸' => 'zastava: Samoa',
    '🇽🇰' => 'zastava: Kosovo',
    '🇾🇪' => 'zastava: Jemen',
    '🇾🇹' => 'zastava: Mayotte',
    '🇿🇦' => 'zastava: Južnoafriška republika',
    '🇿🇲' => 'zastava: Zambija',
    '🇿🇼' => 'zastava: Zimbabve',
    '🏻' => 'svetel ten kože',
    '🏼' => 'srednje svetel ten kože',
    '🏽' => 'srednji ten kože',
    '🏾' => 'srednje temen ten kože',
    '🏿' => 'temen ten kože',
    '😀' => 'režeči obraz',
    '😃' => 'smejoči obraz z odprtimi usti',
    '😄' => 'smejoči obraz z odprtimi usti in smejočimi očmi',
    '😁' => 'režeči obraz s smejočimi očmi',
    '😆' => 'smejoči obraz z odprtimi usti in trdno zaprtimi očmi',
    '😅' => 'smejoči obraz z odprtimi usti in mrzlim potom',
    '🤣' => 'krohotanje',
    '😂' => 'obraz s solzami veselja',
    '🙂' => 'rahlo smejoči obraz',
    '🙃' => 'na glavo obrnjen obraz',
    '🫠' => 'taleči se obraz',
    '😉' => 'obraz s pomežikom',
    '😊' => 'smejoči obraz s smejočimi očmi',
    '😇' => 'smejoči obraz s svetniškim sijem',
    '🥰' => 'smejoči obraz s srčki',
    '😍' => 'smejoči obraz z očmi v obliki srčka',
    '🤩' => 'obraz z očmi v obliki zvezdic',
    '😘' => 'obraz, ki pošlje ljubčka',
    '😗' => 'obraz, ki poljublja',
    '☺' => 'smejoči obraz',
    '😚' => 'obraz z zaprtimi očmi, ki poljublja',
    '😙' => 'obraz s smejočimi očmi, ki poljublja',
    '🥲' => 'smejoči obraz s solzo',
    '😋' => 'obraz, ki z užitkom pomalica okusno hrano',
    '😛' => 'obraz z molečim jezikom',
    '😜' => 'obraz z molečim jezikom in pomežikom',
    '🤪' => 'nor obraz',
    '😝' => 'obraz z molečim jezikom in zaprtimi očmi',
    '🤑' => 'obraz z denarjem namesto ust',
    '🤗' => 'obraz, ki objema',
    '🤭' => 'obraz z roko čez usta',
    '🫢' => 'obraz z razprtimi očmi in roko pred usti',
    '🫣' => 'obraz s pogledujočim očesom',
    '🤫' => 'obraz s prstom čez usta',
    '🤔' => 'razmišljujoči obraz',
    '🫡' => 'salutirajoč obraz',
    '🤐' => 'obraz z usti v obliki zadrge',
    '🤨' => 'obraz z dvignjenimi obrvmi',
    '😐' => 'nevtralni obraz',
    '😑' => 'brezizrazni obraz',
    '😶' => 'obraz brez ust',
    '🫥' => 'obraz s črtkasto črto',
    '😏' => 'obraz s privoščljivim nasmeškom',
    '😒' => 'obraz s hladnim nasmeškom',
    '🙄' => 'obraz z zavijajočimi očmi',
    '😬' => 'spačen obraz',
    '🤥' => 'lažnivi obraz',
    '🫨' => 'tresoči se obraz',
    '😌' => 'olajšan obraz',
    '😔' => 'brezvoljen obraz',
    '😪' => 'zaspan obraz',
    '🤤' => 'obraz, ki se slini',
    '😴' => 'speči obraz',
    '🫩' => 'obraz s podočnjaki',
    '😷' => 'obraz z medicinsko masko',
    '🤒' => 'obraz s termometrom',
    '🤕' => 'obraz, obvezan s povojem',
    '🤢' => 'obraz, ki se počuti slabo',
    '🤮' => 'obraz, ki bruha',
    '🤧' => 'obraz, ki kiha',
    '🥵' => 'vroč obraz',
    '🥶' => 'mrzel obraz',
    '🥴' => 'okajen obraz',
    '😵' => 'omotičen obraz',
    '🤯' => 'eksplozija v glavi',
    '🤠' => 'obraz s kavbojskim klobukom',
    '🥳' => 'praznujoči obraz',
    '🥸' => 'maska',
    '😎' => 'smejoči obraz s sončnimi očali',
    '🤓' => 'piflarski obraz',
    '🧐' => 'obraz z monoklom',
    '😕' => 'zmeden obraz',
    '🫤' => 'obraz s poševnimi usti',
    '😟' => 'zaskrbljen obraz',
    '🙁' => 'nekoliko namrščen obraz',
    '☹' => 'namrščen obraz',
    '😮' => 'obraz z odprtimi usti',
    '😯' => 'osupljen obraz',
    '😲' => 'začuden obraz',
    '😳' => 'zardeli obraz',
    '🥺' => 'proseč obraz',
    '🥹' => 'obraz, ki zadržuje jok',
    '😦' => 'namrščen obraz z odprtimi usti',
    '😧' => 'trpeč obraz',
    '😨' => 'bojazljiv obraz',
    '😰' => 'obraz z odprtimi usti in mrzlim potom',
    '😥' => 'razočaran, a olajšan obraz',
    '😢' => 'jokajoči obraz',
    '😭' => 'cmerajoči obraz',
    '😱' => 'obraz, ki kriči od strahu',
    '😖' => 'zbegan obraz',
    '😣' => 'vztrajen obraz',
    '😞' => 'razočaran obraz',
    '😓' => 'obraz z mrzlim potom',
    '😩' => 'izčrpan obraz',
    '😫' => 'utrujen obraz',
    '🥱' => 'zehajoč obraz',
    '😤' => 'obraz s paro skozi nos',
    '😡' => 'obraz, ki kuha mulo',
    '😠' => 'jezen obraz',
    '🤬' => 'obraz, ki kolne',
    '😈' => 'smejoči obraz z rogovi',
    '👿' => 'vragec',
    '💀' => 'mrtvaška glava',
    '☠' => 'mrtvaška glava s prekrižanima kostema',
    '💩' => 'kup kakca',
    '🤡' => 'klovnovski obraz',
    '👹' => 'pošastni velikan',
    '👺' => 'škratelj',
    '👻' => 'duh',
    '👽' => 'vesoljec',
    '👾' => 'vesoljska pošast',
    '🤖' => 'robot',
    '😺' => 'režeča se mačka',
    '😸' => 'režeča se mačka s smejočimi očmi',
    '😹' => 'mačka s solzami sreče',
    '😻' => 'smejoča se mačka z očmi v obliki srčka',
    '😼' => 'mačka s prifrknjenim nasmeškom',
    '😽' => 'mačka poljublja z zaprtimi očmi',
    '🙀' => 'presenečena mačka',
    '😿' => 'jokajoča mačka',
    '😾' => 'mačka kuha mulo',
    '🙈' => 'opica, ki ne vidi ničesar',
    '🙉' => 'opica, ki ne sliši ničesar',
    '🙊' => 'opica, ki ničesar ne govori',
    '💌' => 'ljubezensko pismo',
    '💘' => 'srce s puščico',
    '💝' => 'srce s pentljo',
    '💖' => 'lesketajoče srce',
    '💗' => 'rastoče srce',
    '💓' => 'utripajoče srce',
    '💞' => 'vrteči se srci',
    '💕' => 'dve srci',
    '💟' => 'okrasje s srcem',
    '❣' => 'klicaj v obliki srca',
    '💔' => 'strto srce',
    '❤' => 'rdeče srce',
    '🩷' => 'rožnato srce',
    '🧡' => 'oranžno srce',
    '💛' => 'rumeno srce',
    '💚' => 'zeleno srce',
    '💙' => 'modro srce',
    '🩵' => 'svetlomodro srce',
    '💜' => 'vijolično srce',
    '🤎' => 'rjavo srce',
    '🖤' => 'črno srce',
    '🩶' => 'sivo srce',
    '🤍' => 'belo srce',
    '💋' => 'sled šminke po poljubu',
    '💯' => 'sto točk',
    '💢' => 'simbol jeze',
    '🫯' => 'spor v oblaku',
    '💥' => 'trk',
    '💫' => 'omotica',
    '💦' => 'potne kapljice',
    '💨' => 'švigniti',
    '🕳' => 'luknja',
    '💬' => 'oblaček z govorom',
    '🗨' => 'levi oblaček z govorom',
    '🗯' => 'desni oblaček z jeznim govorom',
    '💭' => 'oblaček z mislimi',
    '💤' => 'spanec',
    '👋' => 'mahajoča dlan',
    '🤚' => 'dvignjena hrbtna stran dlani',
    '🖐' => 'dvignjena dlan z razprtimi prsti',
    '✋' => 'dvignjena dlan',
    '🖖' => 'vulkanski pozdrav',
    '🫱' => 'roka, obrnjena v desno',
    '🫲' => 'roka, obrnjena v levo',
    '🫳' => 'roka z dlanjo navzdol',
    '🫴' => 'roka z dlanjo navzgor',
    '🫷' => 'dlan, ki potiska v levo',
    '🫸' => 'dlan, ki potiska v desno',
    '👌' => 'roka, ki kaže V REDU',
    '🤌' => 'prsti, stisnjeni skupaj',
    '🤏' => 'roka s ščepcem',
    '✌' => 'dlan s prstoma, ki kažeta znak za zmago',
    '🤞' => 'prekrižani prsti',
    '🫰' => 'roka s prekrižanima kazalcem in palcem',
    '🤟' => 'znak za ljubim te',
    '🤘' => 'znak za rogove',
    '🤙' => 'pokliči me',
    '👈' => 'hrbtna stran dlani s kazalcem, ki kaže levo',
    '👉' => 'hrbtna stran dlani s kazalcem, ki kaže desno',
    '👆' => 'hrbtna stran dlani s kazalcem, ki kaže gor',
    '🖕' => 'sredinec',
    '👇' => 'hrbtna stran dlani s kazalcem, ki kaže dol',
    '☝' => 'kazalec, ki kaže gor',
    '🫵' => 'kazalec, usmerjen v opazovalca',
    '👍' => 'palec gor',
    '👎' => 'palec dol',
    '✊' => 'dvignjena pest',
    '👊' => 'bližajoča se pest',
    '🤛' => 'pest, obrnjena levo',
    '🤜' => 'pest, obrnjena desno',
    '👏' => 'ploskajoči dlani',
    '🙌' => 'oseba, ki dvigne roke',
    '🫶' => 'roki, sklenjeni v obliki srca',
    '👐' => 'odprti dlani',
    '🤲' => 'dvignjeni odprti dlani',
    '🤝' => 'rokovanje',
    '🙏' => 'staknjeni dlani',
    '✍' => 'pišoča dlan',
    '💅' => 'lak za nohte',
    '🤳' => 'selfi',
    '💪' => 'napeti bicepsi',
    '🦾' => 'mehanska roka',
    '🦿' => 'mehanska noga',
    '🦵' => 'noga',
    '🦶' => 'stopalo',
    '👂' => 'uho',
    '🦻' => 'uho s slušnim aparatom',
    '👃' => 'nos',
    '🧠' => 'možgani',
    '🫀' => 'človeško srce',
    '🫁' => 'pljuča',
    '🦷' => 'zob',
    '🦴' => 'kost',
    '👀' => 'oči',
    '👁' => 'oko',
    '👅' => 'jezik',
    '👄' => 'usta',
    '🫦' => 'grizenje ustnice',
    '👶' => 'dojenček',
    '🧒' => 'otrok',
    '👦' => 'fant',
    '👧' => 'dekle',
    '🧑' => 'odrasla oseba',
    '👱' => 'oseba: svetli lasje',
    '👨' => 'moški',
    '🧔' => 'oseba: brada',
    '👩' => 'ženska',
    '🧓' => 'starejši odrasli',
    '👴' => 'starec',
    '👵' => 'starka',
    '🙍' => 'oseba, ki se mršči',
    '🙎' => 'oseba, ki kuha mulo',
    '🙅' => 'oseba, ki kaže NE',
    '🙆' => 'oseba, ki kaže V REDU',
    '💁' => 'oseba, ki dela na informacijah',
    '🙋' => 'vesela oseba, ki dvigne roko',
    '🧏' => 'gluha oseba',
    '🙇' => 'oseba, ki se prikloni',
    '🤦' => 'oseba z dlanjo na obrazu',
    '🤷' => 'oseba, ki skomiga',
    '👮' => 'policaj',
    '🕵' => 'preiskovalec',
    '💂' => 'stražar',
    '🥷' => 'nindža',
    '👷' => 'gradbenik',
    '🫅' => 'oseba s krono',
    '🤴' => 'princ',
    '👸' => 'princesa',
    '👳' => 'oseba s turbanom',
    '👲' => 'moški s kitajsko čepico',
    '🧕' => 'ženska z naglavno ruto',
    '🤵' => 'oseba v smokingu',
    '👰' => 'oseba s pajčolanom',
    '🤰' => 'nosečnica',
    '🫃' => 'noseč moški',
    '🫄' => 'noseča oseba',
    '🤱' => 'dojenje',
    '👼' => 'angelček',
    '🎅' => 'Božiček',
    '🤶' => 'gospa Božiček',
    '🦸' => 'superjunak/superjunakinja',
    '🦹' => 'super negativec',
    '🧙' => 'oseba, ki čara',
    '🧚' => 'vilinsko bitje',
    '🧛' => 'vampirji',
    '🧜' => 'morska oseba',
    '🧝' => 'škrati',
    '🧞' => 'duh iz steklenice',
    '🧟' => 'zombi',
    '🧌' => 'trol',
    '🫈' => 'kosmato bitje',
    '💆' => 'masaža obraza',
    '💇' => 'striženje',
    '🚶' => 'pešec',
    '🧍' => 'stoječa oseba',
    '🧎' => 'klečeča oseba',
    '🏃' => 'tekač',
    '💃' => 'ženska, ki pleše',
    '🕺' => 'plesalec',
    '🕴' => 'moški v poslovni obleki, ki lebdi',
    '👯' => 'žurerski ženski',
    '🧖' => 'oseba v parni savni',
    '🧗' => 'oseba, ki pleza',
    '🤺' => 'sabljanje',
    '🏇' => 'konjske dirke',
    '⛷' => 'smučar',
    '🏂' => 'deskar na snegu',
    '🏌' => 'igralec golfa',
    '🏄' => 'oseba deska',
    '🚣' => 'oseba v čolnu na vesla',
    '🏊' => 'oseba plava',
    '⛹' => 'oseba z žogo',
    '🏋' => 'oseba dviga uteži',
    '🚴' => 'oseba na kolesu',
    '🚵' => 'gorski kolesar',
    '🤸' => 'oseba izvaja kolo',
    '🤼' => 'rokoborba',
    '🤽' => 'vaterpolo',
    '🤾' => 'rokomet',
    '🤹' => 'žongliranje',
    '🧘' => 'oseba v lotusovem položaju',
    '🛀' => 'oseba, ki se kopa',
    '🛌' => 'oseba v postelji',
    '👭' => 'ženski se držita za roke',
    '👫' => 'moški in ženska se držita za roke',
    '👬' => 'moška se držita za roke',
    '💏' => 'poljub',
    '💑' => 'par s srčkom',
    '🗣' => 'govoreča glava',
    '👤' => 'silhueta doprsja',
    '👥' => 'silhueti doprsja',
    '🫂' => 'osebi, ki se objemata',
    '👪' => 'družina',
    '👣' => 'stopinje',
    '🫆' => 'prstni odtis',
    '🦰' => 'rdeči lasje',
    '🦱' => 'kodrasti lasje',
    '🦳' => 'sivi lasje',
    '🦲' => 'plešast',
    '🐵' => 'opičji obraz',
    '🐒' => 'opica',
    '🦍' => 'gorila',
    '🦧' => 'orangutan',
    '🐶' => 'pasji obraz',
    '🐕' => 'pes',
    '🦮' => 'pes vodnik',
    '🐩' => 'pudelj',
    '🐺' => 'volk',
    '🦊' => 'lisica',
    '🦝' => 'rakun',
    '🐱' => 'mačji obraz',
    '🐈' => 'mačka',
    '🦁' => 'lev',
    '🐯' => 'tigrov obraz',
    '🐅' => 'tiger',
    '🐆' => 'leopard',
    '🐴' => 'konjev obraz',
    '🫎' => 'los',
    '🫏' => 'osel',
    '🐎' => 'konj',
    '🦄' => 'samorog',
    '🦓' => 'zebra',
    '🦌' => 'jelen',
    '🦬' => 'bizon',
    '🐮' => 'kravji obraz',
    '🐂' => 'vol',
    '🐃' => 'vodni bivol',
    '🐄' => 'krava',
    '🐷' => 'prašičji obraz',
    '🐖' => 'prašič',
    '🐗' => 'divja svinja',
    '🐽' => 'prašičji rilec',
    '🐏' => 'oven',
    '🐑' => 'ovca',
    '🐐' => 'koza',
    '🐪' => 'kamela',
    '🐫' => 'dvogrba kamela',
    '🦙' => 'lama',
    '🦒' => 'žirafa',
    '🐘' => 'slon',
    '🦣' => 'mamut',
    '🦏' => 'nosorog',
    '🦛' => 'povodni konj',
    '🐭' => 'mišji obraz',
    '🐁' => 'miš',
    '🐀' => 'podgana',
    '🐹' => 'hrček',
    '🐰' => 'zajčji obraz',
    '🐇' => 'zajec',
    '🐿' => 'progasta veverica',
    '🦫' => 'bober',
    '🦔' => 'jež',
    '🦇' => 'netopir',
    '🐻' => 'medved',
    '🐨' => 'koala',
    '🐼' => 'panda',
    '🦥' => 'lenivec',
    '🦦' => 'vidra',
    '🦨' => 'skunk',
    '🦘' => 'kenguru',
    '🦡' => 'jazbec',
    '🐾' => 'odtisi tac',
    '🦃' => 'puran',
    '🐔' => 'kokoš',
    '🐓' => 'petelin',
    '🐣' => 'pišče, ki se izvali',
    '🐤' => 'piščanček',
    '🐥' => 'piščanček, gleda naprej',
    '🐦' => 'ptica',
    '🐧' => 'pingvin',
    '🕊' => 'golobica',
    '🦅' => 'orel',
    '🦆' => 'raca',
    '🦢' => 'labod',
    '🦉' => 'sova',
    '🦤' => 'dodo',
    '🪶' => 'pero',
    '🦩' => 'flamingo',
    '🦚' => 'pav',
    '🦜' => 'papagaj',
    '🪽' => 'krilo',
    '🪿' => 'gos',
    '🐸' => 'žaba',
    '🐊' => 'krokodil',
    '🐢' => 'želva',
    '🦎' => 'kuščar',
    '🐍' => 'kača',
    '🐲' => 'zmajski obraz',
    '🐉' => 'zmaj',
    '🦕' => 'zavropod',
    '🦖' => 'tiranozaver',
    '🐳' => 'kit, ki brizga vodo',
    '🐋' => 'kit',
    '🐬' => 'delfin',
    '🫍' => 'orka',
    '🦭' => 'tjulenj',
    '🐟' => 'riba',
    '🐠' => 'tropska riba',
    '🐡' => 'riba napihovalka',
    '🦈' => 'morski pes',
    '🐙' => 'hobotnica',
    '🐚' => 'spiralna školjka',
    '🪸' => 'korala',
    '🪼' => 'meduza',
    '🦀' => 'rakovica',
    '🦞' => 'jastog',
    '🦐' => 'morski rakec',
    '🦑' => 'ligenj',
    '🦪' => 'ostriga',
    '🐌' => 'polž',
    '🦋' => 'metulj',
    '🐛' => 'žuželka',
    '🐜' => 'mravlja',
    '🐝' => 'čebela',
    '🪲' => 'hrošč',
    '🐞' => 'pikapolonica',
    '🦗' => 'muren',
    '🪳' => 'ščurek',
    '🕷' => 'pajek',
    '🕸' => 'pajkova mreža',
    '🦂' => 'škorpijon',
    '🦟' => 'komar',
    '🪰' => 'muha',
    '🪱' => 'črv',
    '🦠' => 'mikrob',
    '💐' => 'šopek',
    '🌸' => 'češnjev cvet',
    '💮' => 'bela cvetlica',
    '🪷' => 'lotos',
    '🏵' => 'rozeta',
    '🌹' => 'vrtnica',
    '🥀' => 'ovenela roža',
    '🌺' => 'hibiskus',
    '🌻' => 'sončnica',
    '🌼' => 'cvet',
    '🌷' => 'tulipan',
    '🪻' => 'hijacinta',
    '🌱' => 'sadika',
    '🪴' => 'lončnica',
    '🌲' => 'zimzeleno drevo',
    '🌳' => 'listnato drevo',
    '🌴' => 'palma',
    '🌵' => 'kaktus',
    '🌾' => 'snop riža',
    '🌿' => 'zelišče',
    '☘' => 'deteljica',
    '🍀' => 'štiriperesna deteljica',
    '🍁' => 'javorov list',
    '🍂' => 'padlo listje',
    '🍃' => 'listje, ki se vrtinči v vetru',
    '🪹' => 'prazno gnezdo',
    '🪺' => 'gnezdo z jajcema',
    '🍄' => 'goba',
    '🪾' => 'drevo brez listja',
    '🍇' => 'grozdje',
    '🍈' => 'melona',
    '🍉' => 'lubenica',
    '🍊' => 'mandarina',
    '🍋' => 'limona',
    '🍌' => 'banana',
    '🍍' => 'ananas',
    '🥭' => 'mango',
    '🍎' => 'rdeče jabolko',
    '🍏' => 'zeleno jabolko',
    '🍐' => 'hruška',
    '🍑' => 'breskev',
    '🍒' => 'češnje',
    '🍓' => 'jagoda',
    '🫐' => 'borovnice',
    '🥝' => 'kivi',
    '🍅' => 'paradižnik',
    '🫒' => 'oliva',
    '🥥' => 'kokos',
    '🥑' => 'avokado',
    '🍆' => 'jajčevec',
    '🥔' => 'krompir',
    '🥕' => 'korenje',
    '🌽' => 'koruzni storž',
    '🌶' => 'feferon',
    '🫑' => 'paprika',
    '🥒' => 'kumara',
    '🥬' => 'listnata zelenjava',
    '🥦' => 'brokoli',
    '🧄' => 'česen',
    '🧅' => 'čebula',
    '🥜' => 'arašidi',
    '🫘' => 'fižol',
    '🌰' => 'kostanj',
    '🫚' => 'korenina ingverja',
    '🫛' => 'strok graha',
    '🫜' => 'korenasta zelenjava',
    '🍞' => 'kruh',
    '🥐' => 'rogljič',
    '🥖' => 'bageta',
    '🫓' => 'indijski kruh',
    '🥨' => 'presta',
    '🥯' => 'bagel',
    '🥞' => 'palačinke',
    '🧇' => 'vafelj',
    '🧀' => 'kos sira',
    '🍖' => 'meso s kostjo',
    '🍗' => 'piščančje bedro',
    '🥩' => 'kos mesa',
    '🥓' => 'slanina',
    '🍔' => 'hamburger',
    '🍟' => 'krompirček',
    '🍕' => 'pica',
    '🌭' => 'hot dog',
    '🥪' => 'sendvič',
    '🌮' => 'taco',
    '🌯' => 'burito',
    '🫔' => 'tamale',
    '🥙' => 'kebab',
    '🧆' => 'falafel',
    '🥚' => 'jajce',
    '🍳' => 'kuhanje',
    '🥘' => 'kozica s hrano',
    '🍲' => 'skodelica hrane',
    '🫕' => 'fondi',
    '🥣' => 'skodelica z žlico',
    '🥗' => 'zelena solata',
    '🍿' => 'pokovka',
    '🧈' => 'maslo',
    '🧂' => 'sol',
    '🥫' => 'konzervirana hrana',
    '🍱' => 'posoda za malico bento',
    '🍘' => 'rižev kreker',
    '🍙' => 'riževa kroglica',
    '🍚' => 'kuhan riž',
    '🍛' => 'riž s karijem',
    '🍜' => 'skleda rezancev',
    '🍝' => 'špageti',
    '🍠' => 'pečeni sladki krompir',
    '🍢' => 'oden',
    '🍣' => 'suši',
    '🍤' => 'ocvrt rakec',
    '🍥' => 'ribji kolač s spiralo',
    '🥮' => 'lunina torta',
    '🍡' => 'dango',
    '🥟' => 'cmok',
    '🥠' => 'piškotek sreče',
    '🥡' => 'embalaža za hrano',
    '🍦' => 'rahel sladoled',
    '🍧' => 'nastrgan led',
    '🍨' => 'sladoled',
    '🍩' => 'krof',
    '🍪' => 'piškot',
    '🎂' => 'rojstnodnevna torta',
    '🍰' => 'krhek kolač',
    '🧁' => 'kolaček',
    '🥧' => 'pita',
    '🍫' => 'tablica čokolade',
    '🍬' => 'bonbon',
    '🍭' => 'lizika',
    '🍮' => 'vanilijev puding',
    '🍯' => 'vrč medu',
    '🍼' => 'steklenička',
    '🥛' => 'kozarec mleka',
    '☕' => 'vroč napitek',
    '🫖' => 'čajnik',
    '🍵' => 'čajna skodelica brez ročaja',
    '🍶' => 'sake',
    '🍾' => 'odpiranje penine',
    '🍷' => 'vinski kozarec',
    '🍸' => 'kozarec za koktajl',
    '🍹' => 'tropska pijača',
    '🍺' => 'vrček za pivo',
    '🍻' => 'nazdravljanje z vrčkoma za pivo',
    '🥂' => 'nazdravljanje',
    '🥃' => 'kozarec žgane pijače',
    '🫗' => 'zlivanje tekočine',
    '🥤' => 'kozarec s slamico',
    '🧋' => 'čaj s kroglicami',
    '🧃' => 'tetrapak s pijačo',
    '🧉' => 'mate čaj',
    '🧊' => 'ledena kocka',
    '🥢' => 'jedilne paličice',
    '🍽' => 'vilice, nož in krožnik',
    '🍴' => 'vilice in nož',
    '🥄' => 'žlica',
    '🔪' => 'kuhinjski nož',
    '🫙' => 'kozarec za vlaganje',
    '🏺' => 'amfora',
    '🌍' => 'globus z Evropo in Afriko',
    '🌎' => 'globus s Severno in Južno Ameriko',
    '🌏' => 'globus z Azijo in Avstralijo',
    '🌐' => 'globus s poldnevniki',
    '🗺' => 'zemljevid sveta',
    '🗾' => 'zemljevid Japonske',
    '🧭' => 'kompas',
    '🏔' => 'zasnežena gora',
    '⛰' => 'gora',
    '🛘' => 'zemeljski plaz',
    '🌋' => 'vulkan',
    '🗻' => 'gora Fuji',
    '🏕' => 'kampiranje',
    '🏖' => 'plaža s senčnikom',
    '🏜' => 'puščava',
    '🏝' => 'samotni otok',
    '🏞' => 'nacionalni park',
    '🏟' => 'stadion',
    '🏛' => 'klasična zgradba',
    '🏗' => 'gradbišče',
    '🧱' => 'opeka',
    '🪨' => 'skala',
    '🪵' => 'les',
    '🛖' => 'koča',
    '🏘' => 'hiše',
    '🏚' => 'zapuščena hiša',
    '🏠' => 'hiša',
    '🏡' => 'hiša z vrtom',
    '🏢' => 'poslovna stavba',
    '🏣' => 'japonska pošta',
    '🏤' => 'pošta',
    '🏥' => 'bolnišnica',
    '🏦' => 'banka',
    '🏨' => 'hotel',
    '🏩' => 'ljubezenski hotel',
    '🏪' => 'trgovina z mešanim blagom',
    '🏫' => 'šola',
    '🏬' => 'veleblagovnica',
    '🏭' => 'tovarna',
    '🏯' => 'japonski grad',
    '🏰' => 'grad',
    '💒' => 'poročna kapela',
    '🗼' => 'tokijski stolp',
    '🗽' => 'kip svobode',
    '⛪' => 'cerkev',
    '🕌' => 'mošeja',
    '🛕' => 'hindujski tempelj',
    '🕍' => 'sinagoga',
    '⛩' => 'šintoistično svetišče',
    '🕋' => 'kaaba',
    '⛲' => 'fontana',
    '⛺' => 'šotor',
    '🌁' => 'megleno',
    '🌃' => 'zvezdnata noč',
    '🏙' => 'mestna pokrajina',
    '🌄' => 'sončni vzhod v gorah',
    '🌅' => 'sončni vzhod',
    '🌆' => 'mestna pokrajina v mraku',
    '🌇' => 'sončni zahod',
    '🌉' => 'most ponoči',
    '♨' => 'termalni vrelec',
    '🎠' => 'konj na vrtiljaku',
    '🛝' => 'tobogan',
    '🎡' => 'panoramsko kolo',
    '🎢' => 'vlakec smrti',
    '💈' => 'brivski izvesek',
    '🎪' => 'cirkuški šotor',
    '🚂' => 'lokomotiva',
    '🚃' => 'železniški vagon',
    '🚄' => 'hitri vlak',
    '🚅' => 'hitri vlak s koničastim nosom',
    '🚆' => 'vlak',
    '🚇' => 'podzemni vlak',
    '🚈' => 'ozkotirna železnica',
    '🚉' => 'postaja',
    '🚊' => 'tramvaj',
    '🚝' => 'enotirna železnica',
    '🚞' => 'gorska železnica',
    '🚋' => 'vagon tramvaja',
    '🚌' => 'avtobus',
    '🚍' => 'prihajajoči avtobus',
    '🚎' => 'trolejbus',
    '🚐' => 'minibus',
    '🚑' => 'reševalno vozilo',
    '🚒' => 'gasilsko vozilo',
    '🚓' => 'policijsko vozilo',
    '🚔' => 'prihajajoče policijsko vozilo',
    '🚕' => 'taksi',
    '🚖' => 'prihajajoči taksi',
    '🚗' => 'avtomobil',
    '🚘' => 'prihajajoči avtomobil',
    '🚙' => 'cestni terenec',
    '🛻' => 'poltovornjak',
    '🚚' => 'dostavno vozilo',
    '🚛' => 'tovornjak s prikolico',
    '🚜' => 'traktor',
    '🏎' => 'dirkalnik',
    '🏍' => 'motor',
    '🛵' => 'skuter',
    '🦽' => 'ročni invalidski voziček',
    '🦼' => 'motoriziran invalidski voziček',
    '🛺' => 'rikša z motorjem',
    '🚲' => 'kolo',
    '🛴' => 'skiro',
    '🛹' => 'rolka',
    '🛼' => 'kotalke',
    '🚏' => 'avtobusna postaja',
    '🛣' => 'avtocesta',
    '🛤' => 'železniška proga',
    '🛢' => 'naftni sod',
    '⛽' => 'črpalka za gorivo',
    '🛞' => 'avtomobilsko kolo',
    '🚨' => 'policijska luč',
    '🚥' => 'vodoravni semafor',
    '🚦' => 'navpični semafor',
    '🛑' => 'znak stop',
    '🚧' => 'zaprto zaradi gradnje',
    '⚓' => 'sidro',
    '🛟' => 'rešilni obroč',
    '⛵' => 'jadrnica',
    '🛶' => 'kanu',
    '🚤' => 'gliser',
    '🛳' => 'potniška ladja',
    '⛴' => 'trajekt',
    '🛥' => 'motorni čoln',
    '🚢' => 'ladja',
    '✈' => 'letalo',
    '🛩' => 'manjše letalo',
    '🛫' => 'odhod letala',
    '🛬' => 'prihod letala',
    '🪂' => 'padalo',
    '💺' => 'sedež',
    '🚁' => 'helikopter',
    '🚟' => 'viseča železnica',
    '🚠' => 'nihalka',
    '🚡' => 'žičnica',
    '🛰' => 'satelit',
    '🚀' => 'raketa',
    '🛸' => 'leteči krožnik',
    '🛎' => 'hotelski zvonec',
    '🧳' => 'prtljaga',
    '⌛' => 'peščena ura',
    '⏳' => 'peščena ura, ki teče',
    '⌚' => 'ročna ura',
    '⏰' => 'budilka',
    '⏱' => 'štoparica',
    '⏲' => 'časovnik',
    '🕰' => 'ura za na okvir kamina',
    '🕛' => 'poldne',
    '🕧' => 'pol enih',
    '🕐' => 'ura je ena',
    '🕜' => 'pol dveh',
    '🕑' => 'ura je dve',
    '🕝' => 'pol treh',
    '🕒' => 'ura je tri',
    '🕞' => 'pol štirih',
    '🕓' => 'ura je štiri',
    '🕟' => 'pol petih',
    '🕔' => 'ura je pet',
    '🕠' => 'pol šestih',
    '🕕' => 'ura je šest',
    '🕡' => 'pol sedmih',
    '🕖' => 'ura je sedem',
    '🕢' => 'pol osmih',
    '🕗' => 'ura je osem',
    '🕣' => 'pol devetih',
    '🕘' => 'ura je devet',
    '🕤' => 'pol desetih',
    '🕙' => 'ura je deset',
    '🕥' => 'pol enajstih',
    '🕚' => 'ura je enajst',
    '🕦' => 'pol dvanajstih',
    '🌑' => 'mlada luna',
    '🌒' => 'prvi krajec',
    '🌓' => 'polovično osvetljena luna v prvem krajcu',
    '🌔' => 'več kot polovično osvetljen prvi krajec',
    '🌕' => 'polna luna',
    '🌖' => 'več kot polovično osvetljen zadnji krajec',
    '🌗' => 'polovično osvetljena luna v zadnjem krajcu',
    '🌘' => 'zadnji krajec',
    '🌙' => 'lunin srp',
    '🌚' => 'mlada luna z obrazom',
    '🌛' => 'polovično osvetljena luna v prvem krajcu z obrazom',
    '🌜' => 'polovično osvetljena luna v zadnjem krajcu z obrazom',
    '🌡' => 'termometer',
    '☀' => 'sonce',
    '🌝' => 'polna luna z obrazom',
    '🌞' => 'sonce z obrazom',
    '🪐' => 'planet z obročem',
    '⭐' => 'bela srednja zvezda',
    '🌟' => 'žareča zvezda',
    '🌠' => 'utrinek',
    '🌌' => 'rimska cesta',
    '☁' => 'oblak',
    '⛅' => 'sonce za oblakom',
    '⛈' => 'oblak s strelo in dežjem',
    '🌤' => 'sonce za oblačkom',
    '🌥' => 'sonce za velikim oblakom',
    '🌦' => 'sonce za oblakom z dežjem',
    '🌧' => 'oblak z dežjem',
    '🌨' => 'oblak s snegom',
    '🌩' => 'oblak s strelo',
    '🌪' => 'tornado',
    '🌫' => 'megla',
    '🌬' => 'obraz, ki piha veter',
    '🌀' => 'ciklon',
    '🌈' => 'mavrica',
    '🌂' => 'zaprt dežnik',
    '☂' => 'dežnik',
    '☔' => 'dežnik z dežnimi kapljami',
    '⛱' => 'dežnik na tleh',
    '⚡' => 'visoka napetost',
    '❄' => 'snežinka',
    '☃' => 'snežak',
    '⛄' => 'snežak brez snega',
    '☄' => 'komet',
    '🔥' => 'ogenj',
    '💧' => 'kapljica',
    '🌊' => 'vodni val',
    '🎃' => 'izrezljana buča',
    '🎄' => 'jelka',
    '🎆' => 'ognjemet',
    '🎇' => 'sibirska lučka',
    '🧨' => 'petarda',
    '✨' => 'iskrice',
    '🎈' => 'balon',
    '🎉' => 'pokalica za zabave',
    '🎊' => 'konfeti',
    '🎋' => 'drevo za tanabato',
    '🎍' => 'okrasitev bora',
    '🎎' => 'japonski lutki',
    '🎏' => 'zastavni trak z ribama',
    '🎐' => 'vetrni zvončki',
    '🎑' => 'lunino slavje',
    '🧧' => 'rdeča ovojnica',
    '🎀' => 'pentlja',
    '🎁' => 'zavito darilo',
    '🎗' => 'simbolna pentlja',
    '🎟' => 'vstopnice',
    '🎫' => 'vstopnica',
    '🎖' => 'vojaško odlikovanje',
    '🏆' => 'trofeja',
    '🏅' => 'športna medalja',
    '🥇' => 'zlata medalja',
    '🥈' => 'srebrna medalja',
    '🥉' => 'bronasta medalja',
    '⚽' => 'nogometna žoga',
    '⚾' => 'žogica za baseball',
    '🥎' => 'žoga za softball',
    '🏀' => 'košarkarska žoga',
    '🏐' => 'žoga za odbojko',
    '🏈' => 'žoga za ameriški nogomet',
    '🏉' => 'žoga za ragbi',
    '🎾' => 'tenis',
    '🥏' => 'frizbi',
    '🎳' => 'kegljanje',
    '🏏' => 'kriket',
    '🏑' => 'hokej na travi',
    '🏒' => 'hokejska palica in plošček',
    '🥍' => 'lacrosse',
    '🏓' => 'namizni tenis',
    '🏸' => 'badminton',
    '🥊' => 'boksarska rokavica',
    '🥋' => 'kimono za borilne športe',
    '🥅' => 'gol',
    '⛳' => 'zastavica v luknji',
    '⛸' => 'drsalka',
    '🎣' => 'ribiška palica',
    '🤿' => 'potapljaška maska',
    '🎽' => 'tekaška majica',
    '🎿' => 'smučke',
    '🛷' => 'sanke',
    '🥌' => 'curling',
    '🎯' => 'zadetek v polno',
    '🪀' => 'jojo',
    '🪁' => 'zmaj (igrača)',
    '🔫' => 'vodna pištola',
    '🎱' => 'biljard',
    '🔮' => 'kristalna krogla',
    '🪄' => 'čarobna palica',
    '🎮' => 'videoigra',
    '🕹' => 'igralna palica',
    '🎰' => 'igralni avtomat',
    '🎲' => 'igralna kocka',
    '🧩' => 'delček sestavljanke',
    '🧸' => 'medvedek',
    '🪅' => 'pinjata',
    '🪩' => 'disko krogla',
    '🪆' => 'babuška',
    '♠' => 'pik',
    '♥' => 'srce',
    '♦' => 'karo',
    '♣' => 'križ',
    '♟' => 'kmet pri šahu',
    '🃏' => 'joker',
    '🀄' => 'rdeči zmaj v mahjongu',
    '🎴' => 'rožne igralne karte',
    '🎭' => 'scenske umetnosti',
    '🖼' => 'uokvirjena slika',
    '🎨' => 'slikarska paleta',
    '🧵' => 'sukanec',
    '🪡' => 'šivanka',
    '🧶' => 'preja',
    '🪢' => 'vozel',
    '👓' => 'očala',
    '🕶' => 'sončna očala',
    '🥽' => 'zaščitna očala',
    '🥼' => 'laboratorijska halja',
    '🦺' => 'varnostni jopič',
    '👔' => 'kravata',
    '👕' => 'majica',
    '👖' => 'kavbojke',
    '🧣' => 'šal',
    '🧤' => 'rokavice',
    '🧥' => 'plašč',
    '🧦' => 'nogavice',
    '👗' => 'ženska obleka',
    '👘' => 'kimono',
    '🥻' => 'sari',
    '🩱' => 'enodelne kopalke',
    '🩲' => 'spodnje hlače',
    '🩳' => 'kratke hlače',
    '👙' => 'bikini',
    '👚' => 'ženska oblačila',
    '🪭' => 'pahljača',
    '👛' => 'drobižnica',
    '👜' => 'ročna torbica',
    '👝' => 'večerna torbica',
    '🛍' => 'nakupovalni vrečki',
    '🎒' => 'šolski nahrbtnik',
    '🩴' => 'japonke',
    '👞' => 'moški čevelj',
    '👟' => 'tekaški čevelj',
    '🥾' => 'pohodniški čevelj',
    '🥿' => 'balerinke',
    '👠' => 'čevelj z visoko peto',
    '👡' => 'ženski sandali',
    '🩰' => 'baletni čeveljci',
    '👢' => 'ženski škorenj',
    '🪮' => 'glavnik za v lase',
    '👑' => 'krona',
    '👒' => 'ženski klobuk',
    '🎩' => 'cilinder',
    '🎓' => 'maturantska kapa',
    '🧢' => 'kapa s ščitom',
    '🪖' => 'vojaška čelada',
    '⛑' => 'čelada z belim križem',
    '📿' => 'rožni venec',
    '💄' => 'šminka',
    '💍' => 'prstan',
    '💎' => 'dragulj',
    '🔇' => 'ugasnjen zvočnik',
    '🔈' => 'zvočnik pri nizki glasnosti',
    '🔉' => 'zvočnik pri zmerni glasnosti',
    '🔊' => 'zvočnik na glas',
    '📢' => 'javni zvočnik',
    '📣' => 'megafon',
    '📯' => 'poštni rog',
    '🔔' => 'zvonec',
    '🔕' => 'prečrtan zvonec',
    '🎼' => 'partitura',
    '🎵' => 'glasbena nota',
    '🎶' => 'glasbene note',
    '🎙' => 'studijski mikrofon',
    '🎚' => 'drsnik za raven',
    '🎛' => 'kontrolni gumbi',
    '🎤' => 'mikrofon',
    '🎧' => 'slušalka',
    '📻' => 'radio',
    '🎷' => 'saksofon',
    '🎺' => 'trobenta',
    '🪊' => 'pozavna',
    '🪗' => 'harmonika',
    '🎸' => 'kitara',
    '🎹' => 'klaviatura',
    '🎻' => 'violina',
    '🪕' => 'bendžo',
    '🥁' => 'boben',
    '🪘' => 'visok boben',
    '🪇' => 'marake',
    '🪈' => 'flavta',
    '🪉' => 'harfa',
    '📱' => 'mobilni telefon',
    '📲' => 'mobilni telefon s puščico',
    '☎' => 'telefon',
    '📞' => 'telefonska slušalka',
    '📟' => 'pozivnik',
    '📠' => 'telefaks',
    '🔋' => 'baterija',
    '🪫' => 'skoraj prazna baterija',
    '🔌' => 'električni vtikač',
    '💻' => 'prenosnik',
    '🖥' => 'namizni računalnik',
    '🖨' => 'tiskalnik',
    '⌨' => 'tipkovnica',
    '🖱' => 'računalniška miška',
    '🖲' => 'sledilna kroglica',
    '💽' => 'minidisk',
    '💾' => 'disketa',
    '💿' => 'optični disk',
    '📀' => 'dvd',
    '🧮' => 'abak',
    '🎥' => 'filmska kamera',
    '🎞' => 'filmski trak',
    '📽' => 'filmski projektor',
    '🎬' => 'klapa',
    '📺' => 'televizija',
    '📷' => 'fotoaparat',
    '📸' => 'fotoaparat z bliskavico',
    '📹' => 'videokamera',
    '📼' => 'videokaseta',
    '🔍' => 'povečevalno steklo, usmerjeno v levo',
    '🔎' => 'povečevalno steklo, usmerjeno v desno',
    '🕯' => 'sveča',
    '💡' => 'žarnica',
    '🔦' => 'svetilka',
    '🏮' => 'rdeči lampijon',
    '🪔' => 'oljna lučka',
    '📔' => 'zvezek z okrasno platnico',
    '📕' => 'zaprta knjiga',
    '📖' => 'odprta knjiga',
    '📗' => 'zelena knjiga',
    '📘' => 'modra knjiga',
    '📙' => 'oranžna knjiga',
    '📚' => 'knjige',
    '📓' => 'zvezek',
    '📒' => 'zvezek s spiralno vezavo',
    '📃' => 'zvita stran',
    '📜' => 'zvitek',
    '📄' => 'list s prikazano popisano stranjo',
    '📰' => 'časopis',
    '🗞' => 'zvit časopis',
    '📑' => 'zaznamki na straneh',
    '🔖' => 'zaznamek',
    '🏷' => 'etiketa',
    '🪙' => 'kovanec',
    '💰' => 'vreča denarja',
    '🪎' => 'skrinja z zakladom',
    '💴' => 'bankovec za jen',
    '💵' => 'dolarski bankovec',
    '💶' => 'evrski bankovec',
    '💷' => 'bankovec za funt',
    '💸' => 'denar s krili',
    '💳' => 'kreditna kartica',
    '🧾' => 'račun',
    '💹' => 'grafikon rasti za jen',
    '✉' => 'ovojnica',
    '📧' => 'e-pošta',
    '📨' => 'dohodna ovojnica',
    '📩' => 'ovojnica s puščico',
    '📤' => 'pladenj za odhodno pošto',
    '📥' => 'pladenj za dohodno pošto',
    '📦' => 'paket',
    '📫' => 'zaprt poštni nabiralnik z dvignjeno zastavico',
    '📪' => 'zaprt poštni nabiralnik s spuščeno zastavico',
    '📬' => 'odprt poštni nabiralnik z dvignjeno zastavico',
    '📭' => 'odprt poštni nabiralnik s spuščeno zastavico',
    '📮' => 'poštni nabiralnik',
    '🗳' => 'škatla za glasovanje z glasovnico',
    '✏' => 'svinčnik',
    '✒' => 'črna konica peresa',
    '🖋' => 'nalivnik',
    '🖊' => 'nalivno pero',
    '🖌' => 'čopič',
    '🖍' => 'voščenka',
    '📝' => 'zapisek',
    '💼' => 'kovček',
    '📁' => 'mapa za spise',
    '📂' => 'odprta mapa za spise',
    '🗂' => 'kartoteke z vmesnimi listi',
    '📅' => 'koledar',
    '📆' => 'koledar z odstranljivimi listi',
    '🗒' => 'spiralna beležnica',
    '🗓' => 'spiralni koledar',
    '📇' => 'organizator vizitk',
    '📈' => 'grafikon rasti',
    '📉' => 'grafikon padca',
    '📊' => 'črtni grafikon',
    '📋' => 'podložna mapa',
    '📌' => 'risalni žebljiček',
    '📍' => 'okrogli risalni žebljiček',
    '📎' => 'sponka za papir',
    '🖇' => 'povezani sponki',
    '📏' => 'ravnilo',
    '📐' => 'trikotno ravnilo',
    '✂' => 'škarje',
    '🗃' => 'škatla z dokumenti',
    '🗄' => 'predalnik za dokumente',
    '🗑' => 'koš za smeti',
    '🔒' => 'ključavnica',
    '🔓' => 'odklenjena ključavnica',
    '🔏' => 'ključavnica s peresom',
    '🔐' => 'zaprta ključavnica s ključem',
    '🔑' => 'ključ',
    '🗝' => 'star ključ',
    '🔨' => 'kladivo',
    '🪓' => 'sekira',
    '⛏' => 'kramp',
    '⚒' => 'kladivo in kramp',
    '🛠' => 'kladivo in ključ',
    '🗡' => 'bodalo',
    '⚔' => 'prekrižana meča',
    '💣' => 'bomba',
    '🪃' => 'bumerang',
    '🏹' => 'lok in puščica',
    '🛡' => 'ščit',
    '🪚' => 'lisičja žaga',
    '🔧' => 'ključ za matice',
    '🪛' => 'izvijač',
    '🔩' => 'matica in vijak',
    '⚙' => 'zobnik',
    '🗜' => 'stiskanje',
    '⚖' => 'tehtnica',
    '🦯' => 'palica za slepe',
    '🔗' => 'člen',
    '⛓' => 'verige',
    '🪝' => 'kljuka',
    '🧰' => 'zaboj za orodje',
    '🧲' => 'magnet',
    '🪜' => 'lestev',
    '🪏' => 'lopata',
    '⚗' => 'destilator',
    '🧪' => 'epruveta',
    '🧫' => 'petrijevka',
    '🧬' => 'dnk',
    '🔬' => 'mikroskop',
    '🔭' => 'teleskop',
    '📡' => 'satelitska antena',
    '💉' => 'injekcija',
    '🩸' => 'kaplja krvi',
    '💊' => 'tableta',
    '🩹' => 'samolepilni obliž',
    '🩼' => 'bergla',
    '🩺' => 'stetoskop',
    '🩻' => 'rentgen',
    '🚪' => 'vrata',
    '🛗' => 'dvigalo',
    '🪞' => 'zrcalo',
    '🪟' => 'okno',
    '🛏' => 'postelja',
    '🛋' => 'kavč in svetilka',
    '🪑' => 'stol',
    '🚽' => 'straniščna školjka',
    '🪠' => 'odmaševalnik',
    '🚿' => 'tuš',
    '🛁' => 'kad',
    '🪤' => 'mišolovka',
    '🪒' => 'britev',
    '🧴' => 'steklenička losjona',
    '🧷' => 'varnostna zaponka',
    '🧹' => 'metla',
    '🧺' => 'košara',
    '🧻' => 'zvitek papirja',
    '🪣' => 'vedro',
    '🧼' => 'milo',
    '🫧' => 'mehurčki',
    '🪥' => 'zobna ščetka',
    '🧽' => 'spužva',
    '🧯' => 'gasilni aparat',
    '🛒' => 'nakupovalni voziček',
    '🚬' => 'kajenje',
    '⚰' => 'krsta',
    '🪦' => 'nagrobnik',
    '⚱' => 'žara',
    '🧿' => 'amulet zlobno oko',
    '🪬' => 'roka Fatime',
    '🗿' => 'moai',
    '🪧' => 'plakat',
    '🪪' => 'osebni dokument',
    '🏧' => 'znak za bankomat',
    '🚮' => 'znak za smeti v košu',
    '🚰' => 'pitna voda',
    '♿' => 'invalidski voziček',
    '🚹' => 'stranišče za moške',
    '🚺' => 'stranišče za ženske',
    '🚻' => 'javno stranišče',
    '🚼' => 'simbol dojenčka',
    '🚾' => 'wc',
    '🛂' => 'pregled dokumentov',
    '🛃' => 'carina',
    '🛄' => 'prevzem prtljage',
    '🛅' => 'prtljaga v garderobi',
    '⚠' => 'opozorilo',
    '🚸' => 'otroci prečkajo cesto',
    '⛔' => 'ni prehoda',
    '🚫' => 'prepovedano',
    '🚳' => 'prepovedano za kolesa',
    '🚭' => 'prepovedano kajenje',
    '🚯' => 'prepovedano odlaganje smeti',
    '🚱' => 'voda ni pitna',
    '🚷' => 'prepovedano za pešce',
    '📵' => 'mobilni telefoni prepovedani',
    '🔞' => 'prepovedano za mlajše od 18 let',
    '☢' => 'radioaktivno',
    '☣' => 'bionevarnost',
    '⬆' => 'puščica gor',
    '↗' => 'puščica desno gor',
    '➡' => 'puščica desno',
    '↘' => 'puščica desno dol',
    '⬇' => 'puščica dol',
    '↙' => 'puščica levo dol',
    '⬅' => 'puščica levo',
    '↖' => 'puščica levo gor',
    '↕' => 'puščica gor dol',
    '↔' => 'puščica levo desno',
    '↩' => 'puščica desno, ki se ukrivlja v levo',
    '↪' => 'puščica levo, ki se ukrivlja v desno',
    '⤴' => 'puščica desno, ki se ukrivlja gor',
    '⤵' => 'puščica desno, ki se ukrivlja dol',
    '🔃' => 'navpični puščici, ki se ukrivljata v smeri urnega kazalca',
    '🔄' => 'gumb s puščicama, ki se ukrivljata v obratni smeri urnega kazalca',
    '🔙' => 'puščica nazaj',
    '🔚' => 'puščica konec',
    '🔛' => 'puščica NAPREJ!',
    '🔜' => 'puščica kmalu',
    '🔝' => 'puščica na vrh',
    '🛐' => 'božji hram',
    '⚛' => 'simbol za atom',
    '🕉' => 'om',
    '✡' => 'davidova zvezda',
    '☸' => 'kolo darme',
    '☯' => 'jin jang',
    '✝' => 'krščanski križ',
    '☦' => 'pravoslavni križ',
    '☪' => 'zvezda in polmesec',
    '☮' => 'simbol za mir',
    '🕎' => 'menora',
    '🔯' => 'šestkraka zvezda s piko',
    '🪯' => 'khanda',
    '♈' => 'astrološki znak ovna',
    '♉' => 'bik',
    '♊' => 'dvojčka',
    '♋' => 'rak',
    '♌' => 'zodiak: lev',
    '♍' => 'devica',
    '♎' => 'tehnica',
    '♏' => 'astrološki znak škorpijona',
    '♐' => 'strelec',
    '♑' => 'kozorog',
    '♒' => 'vodnar',
    '♓' => 'ribi',
    '⛎' => 'kačenosec',
    '🔀' => 'gumb za naključno predvajanje',
    '🔁' => 'gumb za vnovično predvajanje',
    '🔂' => 'gumb za vnovično posamezno predvajanje',
    '▶' => 'gumb za predvajanje',
    '⏩' => 'gumb za hitro previjanje naprej',
    '⏭' => 'gumb za naslednjo skladbo',
    '⏯' => 'gumb za predvajanje ali zaustavitev',
    '◀' => 'gumb za predvajanje nazaj',
    '⏪' => 'gumb za hitro previjanje nazaj',
    '⏮' => 'gumb za zadnjo skladbo',
    '🔼' => 'gumb za gor',
    '⏫' => 'gumb za hitro gor',
    '🔽' => 'gumb za dol',
    '⏬' => 'gumb za hitro dol',
    '⏸' => 'gumb za zaustavitev',
    '⏹' => 'gumb za ustavitev',
    '⏺' => 'gumb za snemanje',
    '⏏' => 'gumb za izvrženje',
    '🎦' => 'kino',
    '🔅' => 'gumb za zatemnitev',
    '🔆' => 'gumb za svetlost',
    '📶' => 'signal',
    '🛜' => 'brezžična povezava',
    '📳' => 'način vibriranja',
    '📴' => 'mobilni telefon izklopljen',
    '♀' => 'znak za ženski spol',
    '♂' => 'znak za moški spol',
    '⚧' => 'simbol za transseksualnost',
    '✖' => 'množi',
    '➕' => 'plus',
    '➖' => 'minus',
    '➗' => 'deli',
    '🟰' => 'znak za enakovrednost',
    '♾' => 'neskončnost',
    '‼' => 'dvojni klicaj',
    '⁉' => 'klicaj in vprašaj',
    '❓' => 'rdeč vprašaj',
    '❔' => 'bel vprašaj',
    '❕' => 'bel klicaj',
    '❗' => 'rdeč klicaj',
    '〰' => 'valovit pomišljaj',
    '💱' => 'menjava valut',
    '💲' => 'znak za dolar',
    '⚕' => 'simbol zdravstva',
    '♻' => 'simbol za recikliranje',
    '⚜' => 'lilija',
    '🔱' => 'trizobo znamenje',
    '📛' => 'značka za ime',
    '🔰' => 'japonski simbol za začetnika',
    '⭕' => 'velik prazen krog',
    '✅' => 'gumb s kljukico',
    '☑' => 'potrditveno polje s kljukico',
    '✔' => 'kljukica',
    '❌' => 'križec',
    '❎' => 'gumb s križcem',
    '➰' => 'zavita pentlja',
    '➿' => 'dvojna zavita pentlja',
    '〽' => 'znak za part',
    '✳' => 'zvezdica z osmimi špicami',
    '✴' => 'osemkraka zvezda',
    '❇' => 'iskra',
    '™' => 'blagovna znamka',
    '🫟' => 'packa',
    '🔠' => 'vnos velikih črk latinice',
    '🔡' => 'vnos malih črk latinice',
    '🔢' => 'vnos številk',
    '🔣' => 'vnos simbolov',
    '🔤' => 'vnos črk latinice',
    '🅰' => 'gumb s črko a (krvna skupina)',
    '🆎' => 'gumb AB (krvna skupina)',
    '🅱' => 'gumb s črko b (krvna skupina)',
    '🆑' => 'gumb CL',
    '🆒' => 'gumb COOL',
    '🆓' => 'gumb FREE',
    'ℹ' => 'vir informacij',
    '🆔' => 'gumb ID',
    'Ⓜ' => 'črka m v krogu',
    '🆕' => 'gumb NEW',
    '🆖' => 'gumb NG',
    '🅾' => 'gumb s črko O (krvna skupina)',
    '🆗' => 'gumb OK',
    '🅿' => 'gumb s črko p',
    '🆘' => 'gumb SOS',
    '🆙' => 'gumb z besedo up in klicajem',
    '🆚' => 'gumb VS',
    '🈁' => 'gumb za »tukaj« v japonščini',
    '🈂' => 'japonski znak za "napitnino" v kvadratu',
    '🈷' => 'japonski znak za "mesečni znesek"',
    '🈶' => 'gumb za »ni brezplačno« v japonščini',
    '🈯' => 'gumb za »rezervirano« v japonščini',
    '🉐' => 'gumb za »ugodno« v japonščini',
    '🈹' => 'gumb za »popust« v japonščini',
    '🈚' => 'gumb za »brezplačno« v japonščini',
    '🈲' => 'gumb za »prepovedano« v japonščini',
    '🉑' => 'gumb za »sprejemljivo« v japonščini',
    '🈸' => 'gumb za »aplikacija« v japonščini',
    '🈴' => 'pojmovni znak za skupaj v kvadratu',
    '🈳' => 'gumb za »prosto« v japonščini',
    '㊗' => 'pojmovni znak za čestitanje v krogu',
    '㊙' => 'pojmovni znak za skrivnost v krogu',
    '🈺' => 'gumb za »odprto« v japonščini',
    '🈵' => 'gumb za »ni prosto« v japonščini',
    '🔴' => 'rdeč krog',
    '🟠' => 'oranžen krog',
    '🟡' => 'rumen krog',
    '🟢' => 'zelen krog',
    '🔵' => 'moder krog',
    '🟣' => 'škrlaten krog',
    '🟤' => 'rjav krog',
    '⚫' => 'črn krog',
    '⚪' => 'bel krog',
    '🟥' => 'rdeč kvadrat',
    '🟧' => 'oranžen kvadrat',
    '🟨' => 'rumen kvadrat',
    '🟩' => 'zelen kvadrat',
    '🟦' => 'moder kvadrat',
    '🟪' => 'škrlaten kvadrat',
    '🟫' => 'rjav kvadrat',
    '⬛' => 'velik črn kvadrat',
    '⬜' => 'velik bel kvadrat',
    '◼' => 'srednji črn kvadrat',
    '◻' => 'srednji bel kvadrat',
    '◾' => 'srednje majhen črn kvadrat',
    '◽' => 'srednje mali beli kvadrat',
    '▪' => 'majhen črn kvadrat',
    '▫' => 'majhen bel kvadrat',
    '🔶' => 'velik oranžen karo',
    '🔷' => 'velik moder karo',
    '🔸' => 'majhen oranžen karo',
    '🔹' => 'majhen moder karo',
    '🔺' => 'rdeč trikotnik gor',
    '🔻' => 'rdeč trikotnik dol',
    '💠' => 'karo s piko',
    '🔘' => 'izbirni gumb',
    '🔳' => 'gumb z belim kvadratom',
    '🔲' => 'gumb s črnim kvadratom',
    '🏁' => 'kockasta zastava',
    '🚩' => 'trikotna zastava',
    '🎌' => 'prekrižani zastavi',
    '🏴' => 'črna zastava',
    '🏳' => 'plapolajoča bela zastava',
];
