<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'መሳም: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👨🏻‍❤‍💋‍👨🏻' => 'መሳም: ሰው፣ ሰው እና የቆዳ ዓይነት-1-2',
    '👨🏻‍❤‍💋‍👨🏼' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏻‍❤‍💋‍👨🏽' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👨🏻‍❤‍💋‍👨🏾' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👨🏻‍❤‍💋‍👨🏿' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👨🏼‍❤‍💋‍👨🏻' => 'መሳም: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👨🏼‍❤‍💋‍👨🏼' => 'መሳም: ሰው፣ ሰው እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏼‍❤‍💋‍👨🏽' => 'መሳም: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👨🏼‍❤‍💋‍👨🏾' => 'መሳም: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👨🏼‍❤‍💋‍👨🏿' => 'መሳም: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👨🏽‍❤‍💋‍👨🏻' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👨🏽‍❤‍💋‍👨🏼' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍❤‍💋‍👨🏽' => 'መሳም: ሰው፣ ሰው እና የቆዳ ዓይነት-4',
    '👨🏽‍❤‍💋‍👨🏾' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👨🏽‍❤‍💋‍👨🏿' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👨🏾‍❤‍💋‍👨🏻' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👨🏾‍❤‍💋‍👨🏼' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏾‍❤‍💋‍👨🏽' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👨🏾‍❤‍💋‍👨🏾' => 'መሳም: ሰው፣ ሰው እና የቆዳ ዓይነት-5',
    '👨🏾‍❤‍💋‍👨🏿' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👨🏿‍❤‍💋‍👨🏻' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👨🏿‍❤‍💋‍👨🏼' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏿‍❤‍💋‍👨🏽' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👨🏿‍❤‍💋‍👨🏾' => 'መሳም: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👨🏿‍❤‍💋‍👨🏿' => 'መሳም: ሰው፣ ሰው እና የቆዳ ዓይነት-6',
    '👩🏻‍❤‍💋‍👨🏻' => 'መሳም: ሴት፣ ሰው እና የቆዳ ዓይነት-1-2',
    '👩🏻‍❤‍💋‍👨🏼' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍❤‍💋‍👨🏽' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍❤‍💋‍👨🏾' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍❤‍💋‍👨🏿' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍❤‍💋‍👨🏻' => 'መሳም: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍❤‍💋‍👨🏼' => 'መሳም: ሴት፣ ሰው እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏼‍❤‍💋‍👨🏽' => 'መሳም: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍❤‍💋‍👨🏾' => 'መሳም: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍❤‍💋‍👨🏿' => 'መሳም: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍❤‍💋‍👨🏻' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍❤‍💋‍👨🏼' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍❤‍💋‍👨🏽' => 'መሳም: ሴት፣ ሰው እና የቆዳ ዓይነት-4',
    '👩🏽‍❤‍💋‍👨🏾' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍❤‍💋‍👨🏿' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍❤‍💋‍👨🏻' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍❤‍💋‍👨🏼' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍❤‍💋‍👨🏽' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍❤‍💋‍👨🏾' => 'መሳም: ሴት፣ ሰው እና የቆዳ ዓይነት-5',
    '👩🏾‍❤‍💋‍👨🏿' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍❤‍💋‍👨🏻' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍❤‍💋‍👨🏼' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍❤‍💋‍👨🏽' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍❤‍💋‍👨🏾' => 'መሳም: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩🏿‍❤‍💋‍👨🏿' => 'መሳም: ሴት፣ ሰው እና የቆዳ ዓይነት-6',
    '👩🏻‍❤‍💋‍👩🏻' => 'መሳም: ሴት፣ ሴት እና የቆዳ ዓይነት-1-2',
    '👩🏻‍❤‍💋‍👩🏼' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍❤‍💋‍👩🏽' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍❤‍💋‍👩🏾' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍❤‍💋‍👩🏿' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍❤‍💋‍👩🏻' => 'መሳም: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍❤‍💋‍👩🏼' => 'መሳም: ሴት፣ ሴት እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏼‍❤‍💋‍👩🏽' => 'መሳም: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍❤‍💋‍👩🏾' => 'መሳም: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍❤‍💋‍👩🏿' => 'መሳም: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍❤‍💋‍👩🏻' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍❤‍💋‍👩🏼' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍❤‍💋‍👩🏽' => 'መሳም: ሴት፣ ሴት እና የቆዳ ዓይነት-4',
    '👩🏽‍❤‍💋‍👩🏾' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍❤‍💋‍👩🏿' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍❤‍💋‍👩🏻' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍❤‍💋‍👩🏼' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍❤‍💋‍👩🏽' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍❤‍💋‍👩🏾' => 'መሳም: ሴት፣ ሴት እና የቆዳ ዓይነት-5',
    '👩🏾‍❤‍💋‍👩🏿' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍❤‍💋‍👩🏻' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍❤‍💋‍👩🏼' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍❤‍💋‍👩🏽' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍❤‍💋‍👩🏾' => 'መሳም: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩🏿‍❤‍💋‍👩🏿' => 'መሳም: ሴት፣ ሴት እና የቆዳ ዓይነት-6',
    '🧑🏻‍🫯‍🧑🏼' => 'ነጻ ትግል: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏻‍🫯‍🧑🏽' => 'ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '🧑🏻‍🫯‍🧑🏾' => 'ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '🧑🏻‍🫯‍🧑🏿' => 'ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '🧑🏼‍🫯‍🧑🏻' => 'ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '🧑🏼‍🫯‍🧑🏽' => 'ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '🧑🏼‍🫯‍🧑🏾' => 'ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '🧑🏼‍🫯‍🧑🏿' => 'ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '🧑🏽‍🫯‍🧑🏻' => 'ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '🧑🏽‍🫯‍🧑🏼' => 'ነጻ ትግል: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🫯‍🧑🏾' => 'ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '🧑🏽‍🫯‍🧑🏿' => 'ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '🧑🏾‍🫯‍🧑🏻' => 'ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '🧑🏾‍🫯‍🧑🏼' => 'ነጻ ትግል: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏾‍🫯‍🧑🏽' => 'ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '🧑🏾‍🫯‍🧑🏿' => 'ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '🧑🏿‍🫯‍🧑🏻' => 'ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '🧑🏿‍🫯‍🧑🏼' => 'ነጻ ትግል: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏿‍🫯‍🧑🏽' => 'ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '🧑🏿‍🫯‍🧑🏾' => 'ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '🧑🏻‍❤‍🧑🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏻‍❤‍🧑🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '🧑🏻‍❤‍🧑🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '🧑🏻‍❤‍🧑🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '🧑🏼‍❤‍🧑🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '🧑🏼‍❤‍🧑🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '🧑🏼‍❤‍🧑🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '🧑🏼‍❤‍🧑🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '🧑🏽‍❤‍🧑🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '🧑🏽‍❤‍🧑🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍❤‍🧑🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '🧑🏽‍❤‍🧑🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '🧑🏾‍❤‍🧑🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '🧑🏾‍❤‍🧑🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏾‍❤‍🧑🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '🧑🏾‍❤‍🧑🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '🧑🏿‍❤‍🧑🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '🧑🏿‍❤‍🧑🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏿‍❤‍🧑🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '🧑🏿‍❤‍🧑🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ጎልማሳ፣ ጎልማሳ፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '🧑🏻‍🐰‍🧑🏼' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏻‍🐰‍🧑🏽' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '🧑🏻‍🐰‍🧑🏾' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '🧑🏻‍🐰‍🧑🏿' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '🧑🏼‍🐰‍🧑🏻' => 'ሰዎች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '🧑🏼‍🐰‍🧑🏽' => 'ሰዎች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '🧑🏼‍🐰‍🧑🏾' => 'ሰዎች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '🧑🏼‍🐰‍🧑🏿' => 'ሰዎች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '🧑🏽‍🐰‍🧑🏻' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '🧑🏽‍🐰‍🧑🏼' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🐰‍🧑🏾' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '🧑🏽‍🐰‍🧑🏿' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '🧑🏾‍🐰‍🧑🏻' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '🧑🏾‍🐰‍🧑🏼' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏾‍🐰‍🧑🏽' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '🧑🏾‍🐰‍🧑🏿' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '🧑🏿‍🐰‍🧑🏻' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '🧑🏿‍🐰‍🧑🏼' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏿‍🐰‍🧑🏽' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '🧑🏿‍🐰‍🧑🏾' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👨🏻‍🫯‍👨🏼' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏻‍🫯‍👨🏽' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👨🏻‍🫯‍👨🏾' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👨🏻‍🫯‍👨🏿' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👨🏼‍🫯‍👨🏻' => 'የወንድ ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👨🏼‍🫯‍👨🏽' => 'የወንድ ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👨🏼‍🫯‍👨🏾' => 'የወንድ ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👨🏼‍🫯‍👨🏿' => 'የወንድ ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👨🏽‍🫯‍👨🏻' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👨🏽‍🫯‍👨🏼' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🫯‍👨🏾' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👨🏽‍🫯‍👨🏿' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👨🏾‍🫯‍👨🏻' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👨🏾‍🫯‍👨🏼' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏾‍🫯‍👨🏽' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👨🏾‍🫯‍👨🏿' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👨🏿‍🫯‍👨🏻' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👨🏿‍🫯‍👨🏼' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏿‍🫯‍👨🏽' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👨🏿‍🫯‍👨🏾' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👨🏻‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው እና የቆዳ ዓይነት-1-2',
    '👨🏻‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏻‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👨🏻‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👨🏻‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👨🏼‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👨🏼‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏼‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👨🏼‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👨🏼‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👨🏽‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👨🏽‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው እና የቆዳ ዓይነት-4',
    '👨🏽‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👨🏽‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👨🏾‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👨🏾‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏾‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👨🏾‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው እና የቆዳ ዓይነት-5',
    '👨🏾‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👨🏿‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👨🏿‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏿‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👨🏿‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👨🏿‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው፣ ሰው እና የቆዳ ዓይነት-6',
    '👨🏻‍🐰‍👨🏼' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏻‍🐰‍👨🏽' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👨🏻‍🐰‍👨🏾' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👨🏻‍🐰‍👨🏿' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👨🏼‍🐰‍👨🏻' => 'ወንዶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👨🏼‍🐰‍👨🏽' => 'ወንዶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👨🏼‍🐰‍👨🏾' => 'ወንዶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👨🏼‍🐰‍👨🏿' => 'ወንዶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👨🏽‍🐰‍👨🏻' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👨🏽‍🐰‍👨🏼' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🐰‍👨🏾' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👨🏽‍🐰‍👨🏿' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👨🏾‍🐰‍👨🏻' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👨🏾‍🐰‍👨🏼' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏾‍🐰‍👨🏽' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👨🏾‍🐰‍👨🏿' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👨🏿‍🐰‍👨🏻' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👨🏿‍🐰‍👨🏼' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏿‍🐰‍👨🏽' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👨🏿‍🐰‍👨🏾' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩🏻‍🫯‍👩🏼' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍🫯‍👩🏽' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍🫯‍👩🏾' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍🫯‍👩🏿' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍🫯‍👩🏻' => 'የሴት ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍🫯‍👩🏽' => 'የሴት ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍🫯‍👩🏾' => 'የሴት ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍🫯‍👩🏿' => 'የሴት ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍🫯‍👩🏻' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍🫯‍👩🏼' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🫯‍👩🏾' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍🫯‍👩🏿' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍🫯‍👩🏻' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍🫯‍👩🏼' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍🫯‍👩🏽' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍🫯‍👩🏿' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍🫯‍👩🏻' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍🫯‍👩🏼' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍🫯‍👩🏽' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍🫯‍👩🏾' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩🏻‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው እና የቆዳ ዓይነት-1-2',
    '👩🏻‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏼‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው እና የቆዳ ዓይነት-4',
    '👩🏽‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው እና የቆዳ ዓይነት-5',
    '👩🏾‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍❤‍👨🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍❤‍👨🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍❤‍👨🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍❤‍👨🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩🏿‍❤‍👨🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሰው እና የቆዳ ዓይነት-6',
    '👩🏻‍❤‍👩🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት እና የቆዳ ዓይነት-1-2',
    '👩🏻‍❤‍👩🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍❤‍👩🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍❤‍👩🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍❤‍👩🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍❤‍👩🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍❤‍👩🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏼‍❤‍👩🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍❤‍👩🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍❤‍👩🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍❤‍👩🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍❤‍👩🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍❤‍👩🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት እና የቆዳ ዓይነት-4',
    '👩🏽‍❤‍👩🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍❤‍👩🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍❤‍👩🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍❤‍👩🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍❤‍👩🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍❤‍👩🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት እና የቆዳ ዓይነት-5',
    '👩🏾‍❤‍👩🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍❤‍👩🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍❤‍👩🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍❤‍👩🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍❤‍👩🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት፣ የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩🏿‍❤‍👩🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት፣ ሴት እና የቆዳ ዓይነት-6',
    '👩🏻‍🐰‍👩🏼' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍🐰‍👩🏽' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍🐰‍👩🏾' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍🐰‍👩🏿' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍🐰‍👩🏻' => 'ሴቶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍🐰‍👩🏽' => 'ሴቶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍🐰‍👩🏾' => 'ሴቶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍🐰‍👩🏿' => 'ሴቶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍🐰‍👩🏻' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍🐰‍👩🏼' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🐰‍👩🏾' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍🐰‍👩🏿' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍🐰‍👩🏻' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍🐰‍👩🏼' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍🐰‍👩🏽' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍🐰‍👩🏿' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍🐰‍👩🏻' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍🐰‍👩🏼' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍🐰‍👩🏽' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍🐰‍👩🏾' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '🧑🏻‍🤝‍🧑🏻' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-1-2',
    '🧑🏻‍🤝‍🧑🏼' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏻‍🤝‍🧑🏽' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '🧑🏻‍🤝‍🧑🏾' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '🧑🏻‍🤝‍🧑🏿' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '🧑🏼‍🤝‍🧑🏻' => 'ሰዎች እጅ ለእጅ ተያይዘው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '🧑🏼‍🤝‍🧑🏼' => 'ሰዎች እጅ ለእጅ ተያይዘው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏼‍🤝‍🧑🏽' => 'ሰዎች እጅ ለእጅ ተያይዘው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '🧑🏼‍🤝‍🧑🏾' => 'ሰዎች እጅ ለእጅ ተያይዘው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '🧑🏼‍🤝‍🧑🏿' => 'ሰዎች እጅ ለእጅ ተያይዘው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '🧑🏽‍🤝‍🧑🏻' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '🧑🏽‍🤝‍🧑🏼' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🤝‍🧑🏽' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-4',
    '🧑🏽‍🤝‍🧑🏾' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '🧑🏽‍🤝‍🧑🏿' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '🧑🏾‍🤝‍🧑🏻' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '🧑🏾‍🤝‍🧑🏼' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏾‍🤝‍🧑🏽' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '🧑🏾‍🤝‍🧑🏾' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-5',
    '🧑🏾‍🤝‍🧑🏿' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '🧑🏿‍🤝‍🧑🏻' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '🧑🏿‍🤝‍🧑🏼' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏿‍🤝‍🧑🏽' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '🧑🏿‍🤝‍🧑🏾' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '🧑🏿‍🤝‍🧑🏿' => 'ሰዎች እጅ ለእጅ ተያይዘው: የቆዳ ዓይነት-6',
    '👩🏻‍🤝‍👩🏼' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍🤝‍👩🏽' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍🤝‍👩🏾' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍🤝‍👩🏿' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍🤝‍👩🏻' => 'እጅ ለእጅ የተያያዙ ሴቶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍🤝‍👩🏽' => 'እጅ ለእጅ የተያያዙ ሴቶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍🤝‍👩🏾' => 'እጅ ለእጅ የተያያዙ ሴቶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍🤝‍👩🏿' => 'እጅ ለእጅ የተያያዙ ሴቶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍🤝‍👩🏻' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍🤝‍👩🏼' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🤝‍👩🏾' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍🤝‍👩🏿' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍🤝‍👩🏻' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍🤝‍👩🏼' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍🤝‍👩🏽' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍🤝‍👩🏿' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍🤝‍👩🏻' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍🤝‍👩🏼' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍🤝‍👩🏽' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍🤝‍👩🏾' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩🏻‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏻‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👩🏻‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👩🏻‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👩🏼‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👩🏼‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👩🏼‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👩🏼‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👩🏽‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👩🏽‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👩🏽‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👩🏾‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👩🏾‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏾‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👩🏾‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👩🏿‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👩🏿‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏿‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👩🏿‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👨🏻‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏻‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '👨🏻‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '👨🏻‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '👨🏼‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንዶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '👨🏼‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንዶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '👨🏼‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንዶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '👨🏼‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንዶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '👨🏽‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '👨🏽‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '👨🏽‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '👨🏾‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '👨🏾‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏾‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '👨🏾‍🤝‍👨🏿' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '👨🏿‍🤝‍👨🏻' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '👨🏿‍🤝‍👨🏼' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏿‍🤝‍👨🏽' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '👨🏿‍🤝‍👨🏾' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩‍❤‍💋‍👨' => 'መሳም: ሴት እና ሰው',
    '👨‍❤‍💋‍👨' => 'መሳም: ሰው እና ሰው',
    '👩‍❤‍💋‍👩' => 'መሳም: ሴት እና ሴት',
    '👨‍👩‍👧‍👦' => 'ቤተሰብ: ሰው፣ ሴት፣ ልጃገረድ እና ወንድ ልጅ',
    '👨‍👩‍👦‍👦' => 'ቤተሰብ: ሰው፣ ሴት፣ ወንድ ልጅ እና ወንድ ልጅ',
    '👨‍👩‍👧‍👧' => 'ቤተሰብ: ሰው፣ ሴት፣ ልጃገረድ እና ልጃገረድ',
    '👨‍👨‍👧‍👦' => 'ቤተሰብ: ሰው፣ ሰው፣ ልጃገረድ እና ወንድ ልጅ',
    '👨‍👨‍👦‍👦' => 'ቤተሰብ: ሰው፣ ሰው፣ ወንድ ልጅ እና ወንድ ልጅ',
    '👨‍👨‍👧‍👧' => 'ቤተሰብ: ሰው፣ ሰው፣ ልጃገረድ እና ልጃገረድ',
    '👩‍👩‍👧‍👦' => 'ቤተሰብ: ሴት፣ ሴት፣ ልጃገረድ እና ወንድ ልጅ',
    '👩‍👩‍👦‍👦' => 'ቤተሰብ: ሴት፣ ሴት፣ ወንድ ልጅ እና ወንድ ልጅ',
    '👩‍👩‍👧‍👧' => 'ቤተሰብ: ሴት፣ ሴት፣ ልጃገረድ እና ልጃገረድ',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'ባንዲራ: እንግሊዝ',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'ባንዲራ: ስኮትላንድ',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'ባንዲራ: ዌልስ',
    '🧑‍🧑‍🧒‍🧒' => 'ቤተሰብ፦ አዋቂ፣ አዋቂ፣ ልጅ፣ ልጅ',
    '🫱🏻‍🫲🏼' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-1-2 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫱🏻‍🫲🏽' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-4',
    '🫱🏻‍🫲🏾' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-5',
    '🫱🏻‍🫲🏿' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-1-2 እና የቆዳ ዓይነት-6',
    '🫱🏼‍🫲🏻' => 'እጅ መጨባበጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-1-2',
    '🫱🏼‍🫲🏽' => 'እጅ መጨባበጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-4',
    '🫱🏼‍🫲🏾' => 'እጅ መጨባበጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-5',
    '🫱🏼‍🫲🏿' => 'እጅ መጨባበጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የቆዳ ዓይነት-6',
    '🫱🏽‍🫲🏻' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-1-2',
    '🫱🏽‍🫲🏼' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-4 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫱🏽‍🫲🏾' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-5',
    '🫱🏽‍🫲🏿' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-4 እና የቆዳ ዓይነት-6',
    '🫱🏾‍🫲🏻' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-1-2',
    '🫱🏾‍🫲🏼' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-5 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫱🏾‍🫲🏽' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-4',
    '🫱🏾‍🫲🏿' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-5 እና የቆዳ ዓይነት-6',
    '🫱🏿‍🫲🏻' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-1-2',
    '🫱🏿‍🫲🏼' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-6 እና መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫱🏿‍🫲🏽' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-4',
    '🫱🏿‍🫲🏾' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-6 እና የቆዳ ዓይነት-5',
    '👩‍❤‍👨' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት እና ሰው',
    '👨‍❤‍👨' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሰው እና ሰው',
    '👩‍❤‍👩' => 'ልብ በመሃላቸው ያሉ ጥንዶች: ሴት እና ሴት',
    '👨‍👩‍👦' => 'ቤተሰብ: ሰው፣ ሴት እና ወንድ ልጅ',
    '👨‍👩‍👧' => 'ቤተሰብ: ሰው፣ ሴት እና ልጃገረድ',
    '👨‍👨‍👦' => 'ቤተሰብ: ሰው፣ ሰው እና ወንድ ልጅ',
    '👨‍👨‍👧' => 'ቤተሰብ: ሰው፣ ሰው እና ልጃገረድ',
    '👩‍👩‍👦' => 'ቤተሰብ: ሴት፣ ሴት እና ወንድ ልጅ',
    '👩‍👩‍👧' => 'ቤተሰብ: ሴት፣ ሴት እና ልጃገረድ',
    '👨‍👦‍👦' => 'ቤተሰብ: ሰው፣ ወንድ ልጅ እና ወንድ ልጅ',
    '👨‍👧‍👦' => 'ቤተሰብ: ሰው፣ ልጃገረድ እና ወንድ ልጅ',
    '👨‍👧‍👧' => 'ቤተሰብ: ሰው፣ ልጃገረድ እና ልጃገረድ',
    '👩‍👦‍👦' => 'ቤተሰብ: ሴት፣ ወንድ ልጅ እና ወንድ ልጅ',
    '👩‍👧‍👦' => 'ቤተሰብ: ሴት፣ ልጃገረድ እና ወንድ ልጅ',
    '👩‍👧‍👧' => 'ቤተሰብ: ሴት፣ ልጃገረድ እና ልጃገረድ',
    '🧑‍🤝‍🧑' => 'ሰዎች እጅ ለእጅ ተያይዘው',
    '🧑‍🧑‍🧒' => 'ቤተሰብ፦ አዋቂ፣ አዋቂ፣ ልጅ',
    '🧑‍🧒‍🧒' => 'ቤተሰብ፦ አዋቂ፣ ልጅ፣ ልጅ',
    '🧑🏻‍🦰' => 'ጎልማሳ: የቆዳ ዓይነት-1-2 እና ቀይ ጸጉር',
    '🧑🏼‍🦰' => 'ጎልማሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ቀይ ጸጉር',
    '🧑🏽‍🦰' => 'ጎልማሳ: የቆዳ ዓይነት-4 እና ቀይ ጸጉር',
    '🧑🏾‍🦰' => 'ጎልማሳ: የቆዳ ዓይነት-5 እና ቀይ ጸጉር',
    '🧑🏿‍🦰' => 'ጎልማሳ: የቆዳ ዓይነት-6 እና ቀይ ጸጉር',
    '🧑🏻‍🦱' => 'ጎልማሳ: የቆዳ ዓይነት-1-2 እና የተጠቀለለ ጸጉር',
    '🧑🏼‍🦱' => 'ጎልማሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የተጠቀለለ ጸጉር',
    '🧑🏽‍🦱' => 'ጎልማሳ: የቆዳ ዓይነት-4 እና የተጠቀለለ ጸጉር',
    '🧑🏾‍🦱' => 'ጎልማሳ: የቆዳ ዓይነት-5 እና የተጠቀለለ ጸጉር',
    '🧑🏿‍🦱' => 'ጎልማሳ: የቆዳ ዓይነት-6 እና የተጠቀለለ ጸጉር',
    '🧑🏻‍🦳' => 'ጎልማሳ: የቆዳ ዓይነት-1-2 እና ነጭ ጸጉር',
    '🧑🏼‍🦳' => 'ጎልማሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ነጭ ጸጉር',
    '🧑🏽‍🦳' => 'ጎልማሳ: የቆዳ ዓይነት-4 እና ነጭ ጸጉር',
    '🧑🏾‍🦳' => 'ጎልማሳ: የቆዳ ዓይነት-5 እና ነጭ ጸጉር',
    '🧑🏿‍🦳' => 'ጎልማሳ: የቆዳ ዓይነት-6 እና ነጭ ጸጉር',
    '🧑🏻‍🦲' => 'ጎልማሳ: የቆዳ ዓይነት-1-2 እና መላጣ',
    '🧑🏼‍🦲' => 'ጎልማሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና መላጣ',
    '🧑🏽‍🦲' => 'ጎልማሳ: የቆዳ ዓይነት-4 እና መላጣ',
    '🧑🏾‍🦲' => 'ጎልማሳ: የቆዳ ዓይነት-5 እና መላጣ',
    '🧑🏿‍🦲' => 'ጎልማሳ: የቆዳ ዓይነት-6 እና መላጣ',
    '🧑🏻‍🩰' => 'የባሌት ዳንሰኛ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🩰' => 'የባሌት ዳንሰኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🩰' => 'የባሌት ዳንሰኛ: የቆዳ ዓይነት-4',
    '🧑🏾‍🩰' => 'የባሌት ዳንሰኛ: የቆዳ ዓይነት-5',
    '🧑🏿‍🩰' => 'የባሌት ዳንሰኛ: የቆዳ ዓይነት-6',
    '🧔🏻‍♂' => 'ሰው: የቆዳ ዓይነት-1-2 እና ጺም',
    '🧔🏼‍♂' => 'ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ጺም',
    '🧔🏽‍♂' => 'ሰው: የቆዳ ዓይነት-4 እና ጺም',
    '🧔🏾‍♂' => 'ሰው: የቆዳ ዓይነት-5 እና ጺም',
    '🧔🏿‍♂' => 'ሰው: የቆዳ ዓይነት-6 እና ጺም',
    '👱🏻‍♂' => 'ወርቃማ ጸጉር ያለው ወንድ: የቆዳ ዓይነት-1-2',
    '👱🏼‍♂' => 'ወርቃማ ጸጉር ያለው ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👱🏽‍♂' => 'ወርቃማ ጸጉር ያለው ወንድ: የቆዳ ዓይነት-4',
    '👱🏾‍♂' => 'ወርቃማ ጸጉር ያለው ወንድ: የቆዳ ዓይነት-5',
    '👱🏿‍♂' => 'ወርቃማ ጸጉር ያለው ወንድ: የቆዳ ዓይነት-6',
    '👨🏻‍🦰' => 'ሰው: የቆዳ ዓይነት-1-2 እና ቀይ ጸጉር',
    '👨🏼‍🦰' => 'ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ቀይ ጸጉር',
    '👨🏽‍🦰' => 'ሰው: የቆዳ ዓይነት-4 እና ቀይ ጸጉር',
    '👨🏾‍🦰' => 'ሰው: የቆዳ ዓይነት-5 እና ቀይ ጸጉር',
    '👨🏿‍🦰' => 'ሰው: የቆዳ ዓይነት-6 እና ቀይ ጸጉር',
    '👨🏻‍🦱' => 'ሰው: የቆዳ ዓይነት-1-2 እና የተጠቀለለ ጸጉር',
    '👨🏼‍🦱' => 'ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የተጠቀለለ ጸጉር',
    '👨🏽‍🦱' => 'ሰው: የቆዳ ዓይነት-4 እና የተጠቀለለ ጸጉር',
    '👨🏾‍🦱' => 'ሰው: የቆዳ ዓይነት-5 እና የተጠቀለለ ጸጉር',
    '👨🏿‍🦱' => 'ሰው: የቆዳ ዓይነት-6 እና የተጠቀለለ ጸጉር',
    '👨🏻‍🦳' => 'ሰው: የቆዳ ዓይነት-1-2 እና ነጭ ጸጉር',
    '👨🏼‍🦳' => 'ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ነጭ ጸጉር',
    '👨🏽‍🦳' => 'ሰው: የቆዳ ዓይነት-4 እና ነጭ ጸጉር',
    '👨🏾‍🦳' => 'ሰው: የቆዳ ዓይነት-5 እና ነጭ ጸጉር',
    '👨🏿‍🦳' => 'ሰው: የቆዳ ዓይነት-6 እና ነጭ ጸጉር',
    '👨🏻‍🦲' => 'ሰው: የቆዳ ዓይነት-1-2 እና መላጣ',
    '👨🏼‍🦲' => 'ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና መላጣ',
    '👨🏽‍🦲' => 'ሰው: የቆዳ ዓይነት-4 እና መላጣ',
    '👨🏾‍🦲' => 'ሰው: የቆዳ ዓይነት-5 እና መላጣ',
    '👨🏿‍🦲' => 'ሰው: የቆዳ ዓይነት-6 እና መላጣ',
    '🧔🏻‍♀' => 'ሴት: የቆዳ ዓይነት-1-2 እና ጺም',
    '🧔🏼‍♀' => 'ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ጺም',
    '🧔🏽‍♀' => 'ሴት: የቆዳ ዓይነት-4 እና ጺም',
    '🧔🏾‍♀' => 'ሴት: የቆዳ ዓይነት-5 እና ጺም',
    '🧔🏿‍♀' => 'ሴት: የቆዳ ዓይነት-6 እና ጺም',
    '👱🏻‍♀' => 'ወርቃማ ጸጉር ያላት ሴት: የቆዳ ዓይነት-1-2',
    '👱🏼‍♀' => 'ወርቃማ ጸጉር ያላት ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👱🏽‍♀' => 'ወርቃማ ጸጉር ያላት ሴት: የቆዳ ዓይነት-4',
    '👱🏾‍♀' => 'ወርቃማ ጸጉር ያላት ሴት: የቆዳ ዓይነት-5',
    '👱🏿‍♀' => 'ወርቃማ ጸጉር ያላት ሴት: የቆዳ ዓይነት-6',
    '👩🏻‍🦰' => 'ሴት: የቆዳ ዓይነት-1-2 እና ቀይ ጸጉር',
    '👩🏼‍🦰' => 'ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ቀይ ጸጉር',
    '👩🏽‍🦰' => 'ሴት: የቆዳ ዓይነት-4 እና ቀይ ጸጉር',
    '👩🏾‍🦰' => 'ሴት: የቆዳ ዓይነት-5 እና ቀይ ጸጉር',
    '👩🏿‍🦰' => 'ሴት: የቆዳ ዓይነት-6 እና ቀይ ጸጉር',
    '👩🏻‍🦱' => 'ሴት: የቆዳ ዓይነት-1-2 እና የተጠቀለለ ጸጉር',
    '👩🏼‍🦱' => 'ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና የተጠቀለለ ጸጉር',
    '👩🏽‍🦱' => 'ሴት: የቆዳ ዓይነት-4 እና የተጠቀለለ ጸጉር',
    '👩🏾‍🦱' => 'ሴት: የቆዳ ዓይነት-5 እና የተጠቀለለ ጸጉር',
    '👩🏿‍🦱' => 'ሴት: የቆዳ ዓይነት-6 እና የተጠቀለለ ጸጉር',
    '👩🏻‍🦳' => 'ሴት: የቆዳ ዓይነት-1-2 እና ነጭ ጸጉር',
    '👩🏼‍🦳' => 'ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና ነጭ ጸጉር',
    '👩🏽‍🦳' => 'ሴት: የቆዳ ዓይነት-4 እና ነጭ ጸጉር',
    '👩🏾‍🦳' => 'ሴት: የቆዳ ዓይነት-5 እና ነጭ ጸጉር',
    '👩🏿‍🦳' => 'ሴት: የቆዳ ዓይነት-6 እና ነጭ ጸጉር',
    '👩🏻‍🦲' => 'ሴት: የቆዳ ዓይነት-1-2 እና መላጣ',
    '👩🏼‍🦲' => 'ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም እና መላጣ',
    '👩🏽‍🦲' => 'ሴት: የቆዳ ዓይነት-4 እና መላጣ',
    '👩🏾‍🦲' => 'ሴት: የቆዳ ዓይነት-5 እና መላጣ',
    '👩🏿‍🦲' => 'ሴት: የቆዳ ዓይነት-6 እና መላጣ',
    '🙍🏻‍♂' => 'ወንድ ተኮሳትሮ: የቆዳ ዓይነት-1-2',
    '🙍🏼‍♂' => 'ወንድ ተኮሳትሮ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙍🏽‍♂' => 'ወንድ ተኮሳትሮ: የቆዳ ዓይነት-4',
    '🙍🏾‍♂' => 'ወንድ ተኮሳትሮ: የቆዳ ዓይነት-5',
    '🙍🏿‍♂' => 'ወንድ ተኮሳትሮ: የቆዳ ዓይነት-6',
    '🙍🏻‍♀' => 'ሴት ተኮሳትራ: የቆዳ ዓይነት-1-2',
    '🙍🏼‍♀' => 'ሴት ተኮሳትራ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙍🏽‍♀' => 'ሴት ተኮሳትራ: የቆዳ ዓይነት-4',
    '🙍🏾‍♀' => 'ሴት ተኮሳትራ: የቆዳ ዓይነት-5',
    '🙍🏿‍♀' => 'ሴት ተኮሳትራ: የቆዳ ዓይነት-6',
    '🙎🏻‍♂' => 'ወንድ ማለክለክ: የቆዳ ዓይነት-1-2',
    '🙎🏼‍♂' => 'ወንድ ማለክለክ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙎🏽‍♂' => 'ወንድ ማለክለክ: የቆዳ ዓይነት-4',
    '🙎🏾‍♂' => 'ወንድ ማለክለክ: የቆዳ ዓይነት-5',
    '🙎🏿‍♂' => 'ወንድ ማለክለክ: የቆዳ ዓይነት-6',
    '🙎🏻‍♀' => 'ሴት ማለክለክ: የቆዳ ዓይነት-1-2',
    '🙎🏼‍♀' => 'ሴት ማለክለክ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙎🏽‍♀' => 'ሴት ማለክለክ: የቆዳ ዓይነት-4',
    '🙎🏾‍♀' => 'ሴት ማለክለክ: የቆዳ ዓይነት-5',
    '🙎🏿‍♀' => 'ሴት ማለክለክ: የቆዳ ዓይነት-6',
    '🙅🏻‍♂' => 'የእጅ ምልክት ለአይሆንም ወንድ: የቆዳ ዓይነት-1-2',
    '🙅🏼‍♂' => 'የእጅ ምልክት ለአይሆንም ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙅🏽‍♂' => 'የእጅ ምልክት ለአይሆንም ወንድ: የቆዳ ዓይነት-4',
    '🙅🏾‍♂' => 'የእጅ ምልክት ለአይሆንም ወንድ: የቆዳ ዓይነት-5',
    '🙅🏿‍♂' => 'የእጅ ምልክት ለአይሆንም ወንድ: የቆዳ ዓይነት-6',
    '🙅🏻‍♀' => 'የእጅ ምልክት ለአይሆንም ሴት: የቆዳ ዓይነት-1-2',
    '🙅🏼‍♀' => 'የእጅ ምልክት ለአይሆንም ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙅🏽‍♀' => 'የእጅ ምልክት ለአይሆንም ሴት: የቆዳ ዓይነት-4',
    '🙅🏾‍♀' => 'የእጅ ምልክት ለአይሆንም ሴት: የቆዳ ዓይነት-5',
    '🙅🏿‍♀' => 'የእጅ ምልክት ለአይሆንም ሴት: የቆዳ ዓይነት-6',
    '🙆🏻‍♂' => 'የእጅ ምልክት ለእሺ ወንድ: የቆዳ ዓይነት-1-2',
    '🙆🏼‍♂' => 'የእጅ ምልክት ለእሺ ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙆🏽‍♂' => 'የእጅ ምልክት ለእሺ ወንድ: የቆዳ ዓይነት-4',
    '🙆🏾‍♂' => 'የእጅ ምልክት ለእሺ ወንድ: የቆዳ ዓይነት-5',
    '🙆🏿‍♂' => 'የእጅ ምልክት ለእሺ ወንድ: የቆዳ ዓይነት-6',
    '🙆🏻‍♀' => 'የእጅ ምልክት ለእሺ ሴት: የቆዳ ዓይነት-1-2',
    '🙆🏼‍♀' => 'የእጅ ምልክት ለእሺ ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙆🏽‍♀' => 'የእጅ ምልክት ለእሺ ሴት: የቆዳ ዓይነት-4',
    '🙆🏾‍♀' => 'የእጅ ምልክት ለእሺ ሴት: የቆዳ ዓይነት-5',
    '🙆🏿‍♀' => 'የእጅ ምልክት ለእሺ ሴት: የቆዳ ዓይነት-6',
    '💁🏻‍♂' => 'ወንድ መረጃ ሰጪ: የቆዳ ዓይነት-1-2',
    '💁🏼‍♂' => 'ወንድ መረጃ ሰጪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💁🏽‍♂' => 'ወንድ መረጃ ሰጪ: የቆዳ ዓይነት-4',
    '💁🏾‍♂' => 'ወንድ መረጃ ሰጪ: የቆዳ ዓይነት-5',
    '💁🏿‍♂' => 'ወንድ መረጃ ሰጪ: የቆዳ ዓይነት-6',
    '💁🏻‍♀' => 'ሴት መረጃ ሰጪ: የቆዳ ዓይነት-1-2',
    '💁🏼‍♀' => 'ሴት መረጃ ሰጪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💁🏽‍♀' => 'ሴት መረጃ ሰጪ: የቆዳ ዓይነት-4',
    '💁🏾‍♀' => 'ሴት መረጃ ሰጪ: የቆዳ ዓይነት-5',
    '💁🏿‍♀' => 'ሴት መረጃ ሰጪ: የቆዳ ዓይነት-6',
    '🙋🏻‍♂' => 'የወንድ እጅ ማውጣት: የቆዳ ዓይነት-1-2',
    '🙋🏼‍♂' => 'የወንድ እጅ ማውጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙋🏽‍♂' => 'የወንድ እጅ ማውጣት: የቆዳ ዓይነት-4',
    '🙋🏾‍♂' => 'የወንድ እጅ ማውጣት: የቆዳ ዓይነት-5',
    '🙋🏿‍♂' => 'የወንድ እጅ ማውጣት: የቆዳ ዓይነት-6',
    '🙋🏻‍♀' => 'የሴት እጅ ማውጣት: የቆዳ ዓይነት-1-2',
    '🙋🏼‍♀' => 'የሴት እጅ ማውጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙋🏽‍♀' => 'የሴት እጅ ማውጣት: የቆዳ ዓይነት-4',
    '🙋🏾‍♀' => 'የሴት እጅ ማውጣት: የቆዳ ዓይነት-5',
    '🙋🏿‍♀' => 'የሴት እጅ ማውጣት: የቆዳ ዓይነት-6',
    '🧏🏻‍♂' => 'ደንቆሮ ሰው: የቆዳ ዓይነት-1-2',
    '🧏🏼‍♂' => 'ደንቆሮ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧏🏽‍♂' => 'ደንቆሮ ሰው: የቆዳ ዓይነት-4',
    '🧏🏾‍♂' => 'ደንቆሮ ሰው: የቆዳ ዓይነት-5',
    '🧏🏿‍♂' => 'ደንቆሮ ሰው: የቆዳ ዓይነት-6',
    '🧏🏻‍♀' => 'ደንቆሮ ሴት: የቆዳ ዓይነት-1-2',
    '🧏🏼‍♀' => 'ደንቆሮ ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧏🏽‍♀' => 'ደንቆሮ ሴት: የቆዳ ዓይነት-4',
    '🧏🏾‍♀' => 'ደንቆሮ ሴት: የቆዳ ዓይነት-5',
    '🧏🏿‍♀' => 'ደንቆሮ ሴት: የቆዳ ዓይነት-6',
    '🙇🏻‍♂' => 'ወንድ ማጎንበስ: የቆዳ ዓይነት-1-2',
    '🙇🏼‍♂' => 'ወንድ ማጎንበስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙇🏽‍♂' => 'ወንድ ማጎንበስ: የቆዳ ዓይነት-4',
    '🙇🏾‍♂' => 'ወንድ ማጎንበስ: የቆዳ ዓይነት-5',
    '🙇🏿‍♂' => 'ወንድ ማጎንበስ: የቆዳ ዓይነት-6',
    '🙇🏻‍♀' => 'ሴት ማጎንበስ: የቆዳ ዓይነት-1-2',
    '🙇🏼‍♀' => 'ሴት ማጎንበስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙇🏽‍♀' => 'ሴት ማጎንበስ: የቆዳ ዓይነት-4',
    '🙇🏾‍♀' => 'ሴት ማጎንበስ: የቆዳ ዓይነት-5',
    '🙇🏿‍♀' => 'ሴት ማጎንበስ: የቆዳ ዓይነት-6',
    '🤦🏻‍♂' => 'ወንድ ፊት መቅላት: የቆዳ ዓይነት-1-2',
    '🤦🏼‍♂' => 'ወንድ ፊት መቅላት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤦🏽‍♂' => 'ወንድ ፊት መቅላት: የቆዳ ዓይነት-4',
    '🤦🏾‍♂' => 'ወንድ ፊት መቅላት: የቆዳ ዓይነት-5',
    '🤦🏿‍♂' => 'ወንድ ፊት መቅላት: የቆዳ ዓይነት-6',
    '🤦🏻‍♀' => 'ሴት ፊት መቅላት: የቆዳ ዓይነት-1-2',
    '🤦🏼‍♀' => 'ሴት ፊት መቅላት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤦🏽‍♀' => 'ሴት ፊት መቅላት: የቆዳ ዓይነት-4',
    '🤦🏾‍♀' => 'ሴት ፊት መቅላት: የቆዳ ዓይነት-5',
    '🤦🏿‍♀' => 'ሴት ፊት መቅላት: የቆዳ ዓይነት-6',
    '🤷🏻‍♂' => 'ወንድ አለማወቅ: የቆዳ ዓይነት-1-2',
    '🤷🏼‍♂' => 'ወንድ አለማወቅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤷🏽‍♂' => 'ወንድ አለማወቅ: የቆዳ ዓይነት-4',
    '🤷🏾‍♂' => 'ወንድ አለማወቅ: የቆዳ ዓይነት-5',
    '🤷🏿‍♂' => 'ወንድ አለማወቅ: የቆዳ ዓይነት-6',
    '🤷🏻‍♀' => 'ሴት አለማወቅ: የቆዳ ዓይነት-1-2',
    '🤷🏼‍♀' => 'ሴት አለማወቅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤷🏽‍♀' => 'ሴት አለማወቅ: የቆዳ ዓይነት-4',
    '🤷🏾‍♀' => 'ሴት አለማወቅ: የቆዳ ዓይነት-5',
    '🤷🏿‍♀' => 'ሴት አለማወቅ: የቆዳ ዓይነት-6',
    '🧑🏻‍⚕' => 'የጤና ሰራተኛ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍⚕' => 'የጤና ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍⚕' => 'የጤና ሰራተኛ: የቆዳ ዓይነት-4',
    '🧑🏾‍⚕' => 'የጤና ሰራተኛ: የቆዳ ዓይነት-5',
    '🧑🏿‍⚕' => 'የጤና ሰራተኛ: የቆዳ ዓይነት-6',
    '👨🏻‍⚕' => 'ወንድ ነርስ: የቆዳ ዓይነት-1-2',
    '👨🏼‍⚕' => 'ወንድ ነርስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍⚕' => 'ወንድ ነርስ: የቆዳ ዓይነት-4',
    '👨🏾‍⚕' => 'ወንድ ነርስ: የቆዳ ዓይነት-5',
    '👨🏿‍⚕' => 'ወንድ ነርስ: የቆዳ ዓይነት-6',
    '👩🏻‍⚕' => 'ሴት ነርስ: የቆዳ ዓይነት-1-2',
    '👩🏼‍⚕' => 'ሴት ነርስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍⚕' => 'ሴት ነርስ: የቆዳ ዓይነት-4',
    '👩🏾‍⚕' => 'ሴት ነርስ: የቆዳ ዓይነት-5',
    '👩🏿‍⚕' => 'ሴት ነርስ: የቆዳ ዓይነት-6',
    '🧑🏻‍🎓' => 'ተማሪ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🎓' => 'ተማሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🎓' => 'ተማሪ: የቆዳ ዓይነት-4',
    '🧑🏾‍🎓' => 'ተማሪ: የቆዳ ዓይነት-5',
    '🧑🏿‍🎓' => 'ተማሪ: የቆዳ ዓይነት-6',
    '👨🏻‍🎓' => 'ወንድ ተመራቂ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🎓' => 'ወንድ ተመራቂ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🎓' => 'ወንድ ተመራቂ: የቆዳ ዓይነት-4',
    '👨🏾‍🎓' => 'ወንድ ተመራቂ: የቆዳ ዓይነት-5',
    '👨🏿‍🎓' => 'ወንድ ተመራቂ: የቆዳ ዓይነት-6',
    '👩🏻‍🎓' => 'ሴት ተመራቂ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🎓' => 'ሴት ተመራቂ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🎓' => 'ሴት ተመራቂ: የቆዳ ዓይነት-4',
    '👩🏾‍🎓' => 'ሴት ተመራቂ: የቆዳ ዓይነት-5',
    '👩🏿‍🎓' => 'ሴት ተመራቂ: የቆዳ ዓይነት-6',
    '🧑🏻‍🏫' => 'አስተማሪ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🏫' => 'አስተማሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🏫' => 'አስተማሪ: የቆዳ ዓይነት-4',
    '🧑🏾‍🏫' => 'አስተማሪ: የቆዳ ዓይነት-5',
    '🧑🏿‍🏫' => 'አስተማሪ: የቆዳ ዓይነት-6',
    '👨🏻‍🏫' => 'ወንድ አስተማሪ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🏫' => 'ወንድ አስተማሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🏫' => 'ወንድ አስተማሪ: የቆዳ ዓይነት-4',
    '👨🏾‍🏫' => 'ወንድ አስተማሪ: የቆዳ ዓይነት-5',
    '👨🏿‍🏫' => 'ወንድ አስተማሪ: የቆዳ ዓይነት-6',
    '👩🏻‍🏫' => 'ሴት አስተማሪ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🏫' => 'ሴት አስተማሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🏫' => 'ሴት አስተማሪ: የቆዳ ዓይነት-4',
    '👩🏾‍🏫' => 'ሴት አስተማሪ: የቆዳ ዓይነት-5',
    '👩🏿‍🏫' => 'ሴት አስተማሪ: የቆዳ ዓይነት-6',
    '🧑🏻‍⚖' => 'ዳኛ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍⚖' => 'ዳኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍⚖' => 'ዳኛ: የቆዳ ዓይነት-4',
    '🧑🏾‍⚖' => 'ዳኛ: የቆዳ ዓይነት-5',
    '🧑🏿‍⚖' => 'ዳኛ: የቆዳ ዓይነት-6',
    '👨🏻‍⚖' => 'ወንድ ዳኛ: የቆዳ ዓይነት-1-2',
    '👨🏼‍⚖' => 'ወንድ ዳኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍⚖' => 'ወንድ ዳኛ: የቆዳ ዓይነት-4',
    '👨🏾‍⚖' => 'ወንድ ዳኛ: የቆዳ ዓይነት-5',
    '👨🏿‍⚖' => 'ወንድ ዳኛ: የቆዳ ዓይነት-6',
    '👩🏻‍⚖' => 'ሴት ዳኛ: የቆዳ ዓይነት-1-2',
    '👩🏼‍⚖' => 'ሴት ዳኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍⚖' => 'ሴት ዳኛ: የቆዳ ዓይነት-4',
    '👩🏾‍⚖' => 'ሴት ዳኛ: የቆዳ ዓይነት-5',
    '👩🏿‍⚖' => 'ሴት ዳኛ: የቆዳ ዓይነት-6',
    '🧑🏻‍🌾' => 'ገበሬ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🌾' => 'ገበሬ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🌾' => 'ገበሬ: የቆዳ ዓይነት-4',
    '🧑🏾‍🌾' => 'ገበሬ: የቆዳ ዓይነት-5',
    '🧑🏿‍🌾' => 'ገበሬ: የቆዳ ዓይነት-6',
    '👨🏻‍🌾' => 'ወንድ ገበሬ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🌾' => 'ወንድ ገበሬ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🌾' => 'ወንድ ገበሬ: የቆዳ ዓይነት-4',
    '👨🏾‍🌾' => 'ወንድ ገበሬ: የቆዳ ዓይነት-5',
    '👨🏿‍🌾' => 'ወንድ ገበሬ: የቆዳ ዓይነት-6',
    '👩🏻‍🌾' => 'ሴት ገበሬ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🌾' => 'ሴት ገበሬ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🌾' => 'ሴት ገበሬ: የቆዳ ዓይነት-4',
    '👩🏾‍🌾' => 'ሴት ገበሬ: የቆዳ ዓይነት-5',
    '👩🏿‍🌾' => 'ሴት ገበሬ: የቆዳ ዓይነት-6',
    '🧑🏻‍🍳' => 'ማብሰል: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🍳' => 'ማብሰል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🍳' => 'ማብሰል: የቆዳ ዓይነት-4',
    '🧑🏾‍🍳' => 'ማብሰል: የቆዳ ዓይነት-5',
    '🧑🏿‍🍳' => 'ማብሰል: የቆዳ ዓይነት-6',
    '👨🏻‍🍳' => 'ወንድ አብሳይ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🍳' => 'ወንድ አብሳይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🍳' => 'ወንድ አብሳይ: የቆዳ ዓይነት-4',
    '👨🏾‍🍳' => 'ወንድ አብሳይ: የቆዳ ዓይነት-5',
    '👨🏿‍🍳' => 'ወንድ አብሳይ: የቆዳ ዓይነት-6',
    '👩🏻‍🍳' => 'ሴት አብሳይ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🍳' => 'ሴት አብሳይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🍳' => 'ሴት አብሳይ: የቆዳ ዓይነት-4',
    '👩🏾‍🍳' => 'ሴት አብሳይ: የቆዳ ዓይነት-5',
    '👩🏿‍🍳' => 'ሴት አብሳይ: የቆዳ ዓይነት-6',
    '🧑🏻‍🔧' => 'ሜካኒክ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🔧' => 'ሜካኒክ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🔧' => 'ሜካኒክ: የቆዳ ዓይነት-4',
    '🧑🏾‍🔧' => 'ሜካኒክ: የቆዳ ዓይነት-5',
    '🧑🏿‍🔧' => 'ሜካኒክ: የቆዳ ዓይነት-6',
    '👨🏻‍🔧' => 'ወንድ ሜካኒክ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🔧' => 'ወንድ ሜካኒክ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🔧' => 'ወንድ ሜካኒክ: የቆዳ ዓይነት-4',
    '👨🏾‍🔧' => 'ወንድ ሜካኒክ: የቆዳ ዓይነት-5',
    '👨🏿‍🔧' => 'ወንድ ሜካኒክ: የቆዳ ዓይነት-6',
    '👩🏻‍🔧' => 'ሴት ሜካኒክ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🔧' => 'ሴት ሜካኒክ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🔧' => 'ሴት ሜካኒክ: የቆዳ ዓይነት-4',
    '👩🏾‍🔧' => 'ሴት ሜካኒክ: የቆዳ ዓይነት-5',
    '👩🏿‍🔧' => 'ሴት ሜካኒክ: የቆዳ ዓይነት-6',
    '🧑🏻‍🏭' => 'የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🏭' => 'የፋብሪካ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🏭' => 'የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-4',
    '🧑🏾‍🏭' => 'የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-5',
    '🧑🏿‍🏭' => 'የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-6',
    '👨🏻‍🏭' => 'ወንድ የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🏭' => 'ወንድ የፋብሪካ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🏭' => 'ወንድ የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-4',
    '👨🏾‍🏭' => 'ወንድ የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-5',
    '👨🏿‍🏭' => 'ወንድ የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-6',
    '👩🏻‍🏭' => 'ሴት የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🏭' => 'ሴት የፋብሪካ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🏭' => 'ሴት የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-4',
    '👩🏾‍🏭' => 'ሴት የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-5',
    '👩🏿‍🏭' => 'ሴት የፋብሪካ ሰራተኛ: የቆዳ ዓይነት-6',
    '🧑🏻‍💼' => 'የቤሮ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍💼' => 'የቤሮ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍💼' => 'የቤሮ ሰራተኛ: የቆዳ ዓይነት-4',
    '🧑🏾‍💼' => 'የቤሮ ሰራተኛ: የቆዳ ዓይነት-5',
    '🧑🏿‍💼' => 'የቤሮ ሰራተኛ: የቆዳ ዓይነት-6',
    '👨🏻‍💼' => 'ወንድ የቢሮ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '👨🏼‍💼' => 'ወንድ የቢሮ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍💼' => 'ወንድ የቢሮ ሰራተኛ: የቆዳ ዓይነት-4',
    '👨🏾‍💼' => 'ወንድ የቢሮ ሰራተኛ: የቆዳ ዓይነት-5',
    '👨🏿‍💼' => 'ወንድ የቢሮ ሰራተኛ: የቆዳ ዓይነት-6',
    '👩🏻‍💼' => 'ሴት የቢሮ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '👩🏼‍💼' => 'ሴት የቢሮ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍💼' => 'ሴት የቢሮ ሰራተኛ: የቆዳ ዓይነት-4',
    '👩🏾‍💼' => 'ሴት የቢሮ ሰራተኛ: የቆዳ ዓይነት-5',
    '👩🏿‍💼' => 'ሴት የቢሮ ሰራተኛ: የቆዳ ዓይነት-6',
    '🧑🏻‍🔬' => 'ሳይንቲስት: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🔬' => 'ሳይንቲስት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🔬' => 'ሳይንቲስት: የቆዳ ዓይነት-4',
    '🧑🏾‍🔬' => 'ሳይንቲስት: የቆዳ ዓይነት-5',
    '🧑🏿‍🔬' => 'ሳይንቲስት: የቆዳ ዓይነት-6',
    '👨🏻‍🔬' => 'ወንድ ሳይንቲስት: የቆዳ ዓይነት-1-2',
    '👨🏼‍🔬' => 'ወንድ ሳይንቲስት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🔬' => 'ወንድ ሳይንቲስት: የቆዳ ዓይነት-4',
    '👨🏾‍🔬' => 'ወንድ ሳይንቲስት: የቆዳ ዓይነት-5',
    '👨🏿‍🔬' => 'ወንድ ሳይንቲስት: የቆዳ ዓይነት-6',
    '👩🏻‍🔬' => 'ሴት ሳይንቲስት: የቆዳ ዓይነት-1-2',
    '👩🏼‍🔬' => 'ሴት ሳይንቲስት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🔬' => 'ሴት ሳይንቲስት: የቆዳ ዓይነት-4',
    '👩🏾‍🔬' => 'ሴት ሳይንቲስት: የቆዳ ዓይነት-5',
    '👩🏿‍🔬' => 'ሴት ሳይንቲስት: የቆዳ ዓይነት-6',
    '🧑🏻‍💻' => 'ቴክኖሎጂስት: የቆዳ ዓይነት-1-2',
    '🧑🏼‍💻' => 'ቴክኖሎጂስት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍💻' => 'ቴክኖሎጂስት: የቆዳ ዓይነት-4',
    '🧑🏾‍💻' => 'ቴክኖሎጂስት: የቆዳ ዓይነት-5',
    '🧑🏿‍💻' => 'ቴክኖሎጂስት: የቆዳ ዓይነት-6',
    '👨🏻‍💻' => 'ወንድ ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-1-2',
    '👨🏼‍💻' => 'ወንድ ቴክኖሎጂ አዋቂ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍💻' => 'ወንድ ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-4',
    '👨🏾‍💻' => 'ወንድ ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-5',
    '👨🏿‍💻' => 'ወንድ ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-6',
    '👩🏻‍💻' => 'ሴት ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-1-2',
    '👩🏼‍💻' => 'ሴት ቴክኖሎጂ አዋቂ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍💻' => 'ሴት ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-4',
    '👩🏾‍💻' => 'ሴት ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-5',
    '👩🏿‍💻' => 'ሴት ቴክኖሎጂ አዋቂ: የቆዳ ዓይነት-6',
    '🧑🏻‍🎤' => 'ዘፋኝ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🎤' => 'ዘፋኝ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🎤' => 'ዘፋኝ: የቆዳ ዓይነት-4',
    '🧑🏾‍🎤' => 'ዘፋኝ: የቆዳ ዓይነት-5',
    '🧑🏿‍🎤' => 'ዘፋኝ: የቆዳ ዓይነት-6',
    '👨🏻‍🎤' => 'ወንድ ዘፋኝ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🎤' => 'ወንድ ዘፋኝ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🎤' => 'ወንድ ዘፋኝ: የቆዳ ዓይነት-4',
    '👨🏾‍🎤' => 'ወንድ ዘፋኝ: የቆዳ ዓይነት-5',
    '👨🏿‍🎤' => 'ወንድ ዘፋኝ: የቆዳ ዓይነት-6',
    '👩🏻‍🎤' => 'ሴት ዘፋኝ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🎤' => 'ሴት ዘፋኝ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🎤' => 'ሴት ዘፋኝ: የቆዳ ዓይነት-4',
    '👩🏾‍🎤' => 'ሴት ዘፋኝ: የቆዳ ዓይነት-5',
    '👩🏿‍🎤' => 'ሴት ዘፋኝ: የቆዳ ዓይነት-6',
    '🧑🏻‍🎨' => 'አርቲስት: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🎨' => 'አርቲስት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🎨' => 'አርቲስት: የቆዳ ዓይነት-4',
    '🧑🏾‍🎨' => 'አርቲስት: የቆዳ ዓይነት-5',
    '🧑🏿‍🎨' => 'አርቲስት: የቆዳ ዓይነት-6',
    '👨🏻‍🎨' => 'ወንድ ሠዓሊ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🎨' => 'ወንድ ሠዓሊ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🎨' => 'ወንድ ሠዓሊ: የቆዳ ዓይነት-4',
    '👨🏾‍🎨' => 'ወንድ ሠዓሊ: የቆዳ ዓይነት-5',
    '👨🏿‍🎨' => 'ወንድ ሠዓሊ: የቆዳ ዓይነት-6',
    '👩🏻‍🎨' => 'ሴት ሠዓሊ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🎨' => 'ሴት ሠዓሊ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🎨' => 'ሴት ሠዓሊ: የቆዳ ዓይነት-4',
    '👩🏾‍🎨' => 'ሴት ሠዓሊ: የቆዳ ዓይነት-5',
    '👩🏿‍🎨' => 'ሴት ሠዓሊ: የቆዳ ዓይነት-6',
    '🧑🏻‍✈' => 'ፓይለት: የቆዳ ዓይነት-1-2',
    '🧑🏼‍✈' => 'ፓይለት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍✈' => 'ፓይለት: የቆዳ ዓይነት-4',
    '🧑🏾‍✈' => 'ፓይለት: የቆዳ ዓይነት-5',
    '🧑🏿‍✈' => 'ፓይለት: የቆዳ ዓይነት-6',
    '👨🏻‍✈' => 'ወንድ አውሮፕላን አብራሪ: የቆዳ ዓይነት-1-2',
    '👨🏼‍✈' => 'ወንድ አውሮፕላን አብራሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍✈' => 'ወንድ አውሮፕላን አብራሪ: የቆዳ ዓይነት-4',
    '👨🏾‍✈' => 'ወንድ አውሮፕላን አብራሪ: የቆዳ ዓይነት-5',
    '👨🏿‍✈' => 'ወንድ አውሮፕላን አብራሪ: የቆዳ ዓይነት-6',
    '👩🏻‍✈' => 'ሴት አውሮፕላን አብራሪ: የቆዳ ዓይነት-1-2',
    '👩🏼‍✈' => 'ሴት አውሮፕላን አብራሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍✈' => 'ሴት አውሮፕላን አብራሪ: የቆዳ ዓይነት-4',
    '👩🏾‍✈' => 'ሴት አውሮፕላን አብራሪ: የቆዳ ዓይነት-5',
    '👩🏿‍✈' => 'ሴት አውሮፕላን አብራሪ: የቆዳ ዓይነት-6',
    '🧑🏻‍🚀' => 'የጠፈር ተመራማሪ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🚀' => 'የጠፈር ተመራማሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🚀' => 'የጠፈር ተመራማሪ: የቆዳ ዓይነት-4',
    '🧑🏾‍🚀' => 'የጠፈር ተመራማሪ: የቆዳ ዓይነት-5',
    '🧑🏿‍🚀' => 'የጠፈር ተመራማሪ: የቆዳ ዓይነት-6',
    '👨🏻‍🚀' => 'ወንድ የህዋ ተመራማሪ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🚀' => 'ወንድ የህዋ ተመራማሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🚀' => 'ወንድ የህዋ ተመራማሪ: የቆዳ ዓይነት-4',
    '👨🏾‍🚀' => 'ወንድ የህዋ ተመራማሪ: የቆዳ ዓይነት-5',
    '👨🏿‍🚀' => 'ወንድ የህዋ ተመራማሪ: የቆዳ ዓይነት-6',
    '👩🏻‍🚀' => 'ሴት የህዋ ተመራማሪ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🚀' => 'ሴት የህዋ ተመራማሪ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🚀' => 'ሴት የህዋ ተመራማሪ: የቆዳ ዓይነት-4',
    '👩🏾‍🚀' => 'ሴት የህዋ ተመራማሪ: የቆዳ ዓይነት-5',
    '👩🏿‍🚀' => 'ሴት የህዋ ተመራማሪ: የቆዳ ዓይነት-6',
    '🧑🏻‍🚒' => 'የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🚒' => 'የእሳት አደጋ ተከላካይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🚒' => 'የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-4',
    '🧑🏾‍🚒' => 'የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-5',
    '🧑🏿‍🚒' => 'የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-6',
    '👨🏻‍🚒' => 'ወንድ የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🚒' => 'ወንድ የእሳት አደጋ ተከላካይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🚒' => 'ወንድ የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-4',
    '👨🏾‍🚒' => 'ወንድ የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-5',
    '👨🏿‍🚒' => 'ወንድ የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-6',
    '👩🏻‍🚒' => 'ሴት የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🚒' => 'ሴት የእሳት አደጋ ተከላካይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🚒' => 'ሴት የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-4',
    '👩🏾‍🚒' => 'ሴት የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-5',
    '👩🏿‍🚒' => 'ሴት የእሳት አደጋ ተከላካይ: የቆዳ ዓይነት-6',
    '👮🏻‍♂' => 'ወንድ የፖሊስ ሹም: የቆዳ ዓይነት-1-2',
    '👮🏼‍♂' => 'ወንድ የፖሊስ ሹም: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👮🏽‍♂' => 'ወንድ የፖሊስ ሹም: የቆዳ ዓይነት-4',
    '👮🏾‍♂' => 'ወንድ የፖሊስ ሹም: የቆዳ ዓይነት-5',
    '👮🏿‍♂' => 'ወንድ የፖሊስ ሹም: የቆዳ ዓይነት-6',
    '👮🏻‍♀' => 'ሴት የፖሊስ ሹም: የቆዳ ዓይነት-1-2',
    '👮🏼‍♀' => 'ሴት የፖሊስ ሹም: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👮🏽‍♀' => 'ሴት የፖሊስ ሹም: የቆዳ ዓይነት-4',
    '👮🏾‍♀' => 'ሴት የፖሊስ ሹም: የቆዳ ዓይነት-5',
    '👮🏿‍♀' => 'ሴት የፖሊስ ሹም: የቆዳ ዓይነት-6',
    '🕵🏻‍♂' => 'ወንድ መርማሪ ፖሊስ: የቆዳ ዓይነት-1-2',
    '🕵🏼‍♂' => 'ወንድ መርማሪ ፖሊስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🕵🏽‍♂' => 'ወንድ መርማሪ ፖሊስ: የቆዳ ዓይነት-4',
    '🕵🏾‍♂' => 'ወንድ መርማሪ ፖሊስ: የቆዳ ዓይነት-5',
    '🕵🏿‍♂' => 'ወንድ መርማሪ ፖሊስ: የቆዳ ዓይነት-6',
    '🕵🏻‍♀' => 'ሴት መርማሪ ፖሊስ: የቆዳ ዓይነት-1-2',
    '🕵🏼‍♀' => 'ሴት መርማሪ ፖሊስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🕵🏽‍♀' => 'ሴት መርማሪ ፖሊስ: የቆዳ ዓይነት-4',
    '🕵🏾‍♀' => 'ሴት መርማሪ ፖሊስ: የቆዳ ዓይነት-5',
    '🕵🏿‍♀' => 'ሴት መርማሪ ፖሊስ: የቆዳ ዓይነት-6',
    '💂🏻‍♂' => 'ወንድ ጠባቂ: የቆዳ ዓይነት-1-2',
    '💂🏼‍♂' => 'ወንድ ጠባቂ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💂🏽‍♂' => 'ወንድ ጠባቂ: የቆዳ ዓይነት-4',
    '💂🏾‍♂' => 'ወንድ ጠባቂ: የቆዳ ዓይነት-5',
    '💂🏿‍♂' => 'ወንድ ጠባቂ: የቆዳ ዓይነት-6',
    '💂🏻‍♀' => 'ሴት ጠባቂ: የቆዳ ዓይነት-1-2',
    '💂🏼‍♀' => 'ሴት ጠባቂ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💂🏽‍♀' => 'ሴት ጠባቂ: የቆዳ ዓይነት-4',
    '💂🏾‍♀' => 'ሴት ጠባቂ: የቆዳ ዓይነት-5',
    '💂🏿‍♀' => 'ሴት ጠባቂ: የቆዳ ዓይነት-6',
    '👷🏻‍♂' => 'ወንድ የግንባታ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '👷🏼‍♂' => 'ወንድ የግንባታ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👷🏽‍♂' => 'ወንድ የግንባታ ሰራተኛ: የቆዳ ዓይነት-4',
    '👷🏾‍♂' => 'ወንድ የግንባታ ሰራተኛ: የቆዳ ዓይነት-5',
    '👷🏿‍♂' => 'ወንድ የግንባታ ሰራተኛ: የቆዳ ዓይነት-6',
    '👷🏻‍♀' => 'ሴት የግንባታ ሰራተኛ: የቆዳ ዓይነት-1-2',
    '👷🏼‍♀' => 'ሴት የግንባታ ሰራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👷🏽‍♀' => 'ሴት የግንባታ ሰራተኛ: የቆዳ ዓይነት-4',
    '👷🏾‍♀' => 'ሴት የግንባታ ሰራተኛ: የቆዳ ዓይነት-5',
    '👷🏿‍♀' => 'ሴት የግንባታ ሰራተኛ: የቆዳ ዓይነት-6',
    '👳🏻‍♂' => 'ጥምጣም ያደረገ ወንድ: የቆዳ ዓይነት-1-2',
    '👳🏼‍♂' => 'ጥምጣም ያደረገ ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👳🏽‍♂' => 'ጥምጣም ያደረገ ወንድ: የቆዳ ዓይነት-4',
    '👳🏾‍♂' => 'ጥምጣም ያደረገ ወንድ: የቆዳ ዓይነት-5',
    '👳🏿‍♂' => 'ጥምጣም ያደረገ ወንድ: የቆዳ ዓይነት-6',
    '👳🏻‍♀' => 'ጥምጣም ያደረች ሴት: የቆዳ ዓይነት-1-2',
    '👳🏼‍♀' => 'ጥምጣም ያደረች ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👳🏽‍♀' => 'ጥምጣም ያደረች ሴት: የቆዳ ዓይነት-4',
    '👳🏾‍♀' => 'ጥምጣም ያደረች ሴት: የቆዳ ዓይነት-5',
    '👳🏿‍♀' => 'ጥምጣም ያደረች ሴት: የቆዳ ዓይነት-6',
    '🤵🏻‍♂' => 'ሰው በቶክሲዶ: የቆዳ ዓይነት-1-2',
    '🤵🏼‍♂' => 'ሰው በቶክሲዶ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤵🏽‍♂' => 'ሰው በቶክሲዶ: የቆዳ ዓይነት-4',
    '🤵🏾‍♂' => 'ሰው በቶክሲዶ: የቆዳ ዓይነት-5',
    '🤵🏿‍♂' => 'ሰው በቶክሲዶ: የቆዳ ዓይነት-6',
    '🤵🏻‍♀' => 'ሴት በቶክሲዶ: የቆዳ ዓይነት-1-2',
    '🤵🏼‍♀' => 'ሴት በቶክሲዶ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤵🏽‍♀' => 'ሴት በቶክሲዶ: የቆዳ ዓይነት-4',
    '🤵🏾‍♀' => 'ሴት በቶክሲዶ: የቆዳ ዓይነት-5',
    '🤵🏿‍♀' => 'ሴት በቶክሲዶ: የቆዳ ዓይነት-6',
    '👰🏻‍♂' => 'ወንድ በዓይነ እርግብ: የቆዳ ዓይነት-1-2',
    '👰🏼‍♂' => 'ወንድ በዓይነ እርግብ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👰🏽‍♂' => 'ወንድ በዓይነ እርግብ: የቆዳ ዓይነት-4',
    '👰🏾‍♂' => 'ወንድ በዓይነ እርግብ: የቆዳ ዓይነት-5',
    '👰🏿‍♂' => 'ወንድ በዓይነ እርግብ: የቆዳ ዓይነት-6',
    '👰🏻‍♀' => 'ሴት በዓይነ እርግብ: የቆዳ ዓይነት-1-2',
    '👰🏼‍♀' => 'ሴት በዓይነ እርግብ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👰🏽‍♀' => 'ሴት በዓይነ እርግብ: የቆዳ ዓይነት-4',
    '👰🏾‍♀' => 'ሴት በዓይነ እርግብ: የቆዳ ዓይነት-5',
    '👰🏿‍♀' => 'ሴት በዓይነ እርግብ: የቆዳ ዓይነት-6',
    '👩🏻‍🍼' => 'ሴት ህጻን ስትመግብ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🍼' => 'ሴት ህጻን ስትመግብ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🍼' => 'ሴት ህጻን ስትመግብ: የቆዳ ዓይነት-4',
    '👩🏾‍🍼' => 'ሴት ህጻን ስትመግብ: የቆዳ ዓይነት-5',
    '👩🏿‍🍼' => 'ሴት ህጻን ስትመግብ: የቆዳ ዓይነት-6',
    '👨🏻‍🍼' => 'ወንድ ህጻን ሲመግብ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🍼' => 'ወንድ ህጻን ሲመግብ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🍼' => 'ወንድ ህጻን ሲመግብ: የቆዳ ዓይነት-4',
    '👨🏾‍🍼' => 'ወንድ ህጻን ሲመግብ: የቆዳ ዓይነት-5',
    '👨🏿‍🍼' => 'ወንድ ህጻን ሲመግብ: የቆዳ ዓይነት-6',
    '🧑🏻‍🍼' => 'ሰው ህጻን ሲመግብ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🍼' => 'ሰው ህጻን ሲመግብ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🍼' => 'ሰው ህጻን ሲመግብ: የቆዳ ዓይነት-4',
    '🧑🏾‍🍼' => 'ሰው ህጻን ሲመግብ: የቆዳ ዓይነት-5',
    '🧑🏿‍🍼' => 'ሰው ህጻን ሲመግብ: የቆዳ ዓይነት-6',
    '🧑🏻‍🎄' => 'ሚክስ ክላውስ: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🎄' => 'ሚክስ ክላውስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🎄' => 'ሚክስ ክላውስ: የቆዳ ዓይነት-4',
    '🧑🏾‍🎄' => 'ሚክስ ክላውስ: የቆዳ ዓይነት-5',
    '🧑🏿‍🎄' => 'ሚክስ ክላውስ: የቆዳ ዓይነት-6',
    '🦸🏻‍♂' => 'የወንድ ጀግና: የቆዳ ዓይነት-1-2',
    '🦸🏼‍♂' => 'የወንድ ጀግና: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦸🏽‍♂' => 'የወንድ ጀግና: የቆዳ ዓይነት-4',
    '🦸🏾‍♂' => 'የወንድ ጀግና: የቆዳ ዓይነት-5',
    '🦸🏿‍♂' => 'የወንድ ጀግና: የቆዳ ዓይነት-6',
    '🦸🏻‍♀' => 'የሴት ጀግና: የቆዳ ዓይነት-1-2',
    '🦸🏼‍♀' => 'የሴት ጀግና: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦸🏽‍♀' => 'የሴት ጀግና: የቆዳ ዓይነት-4',
    '🦸🏾‍♀' => 'የሴት ጀግና: የቆዳ ዓይነት-5',
    '🦸🏿‍♀' => 'የሴት ጀግና: የቆዳ ዓይነት-6',
    '🦹🏻‍♂' => 'የወንድ ጀግና ምስል: የቆዳ ዓይነት-1-2',
    '🦹🏼‍♂' => 'የወንድ ጀግና ምስል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦹🏽‍♂' => 'የወንድ ጀግና ምስል: የቆዳ ዓይነት-4',
    '🦹🏾‍♂' => 'የወንድ ጀግና ምስል: የቆዳ ዓይነት-5',
    '🦹🏿‍♂' => 'የወንድ ጀግና ምስል: የቆዳ ዓይነት-6',
    '🦹🏻‍♀' => 'የሴት ጀግና ምስል: የቆዳ ዓይነት-1-2',
    '🦹🏼‍♀' => 'የሴት ጀግና ምስል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦹🏽‍♀' => 'የሴት ጀግና ምስል: የቆዳ ዓይነት-4',
    '🦹🏾‍♀' => 'የሴት ጀግና ምስል: የቆዳ ዓይነት-5',
    '🦹🏿‍♀' => 'የሴት ጀግና ምስል: የቆዳ ዓይነት-6',
    '🧙🏻‍♂' => 'አስማተኛ ወንድ: የቆዳ ዓይነት-1-2',
    '🧙🏼‍♂' => 'አስማተኛ ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧙🏽‍♂' => 'አስማተኛ ወንድ: የቆዳ ዓይነት-4',
    '🧙🏾‍♂' => 'አስማተኛ ወንድ: የቆዳ ዓይነት-5',
    '🧙🏿‍♂' => 'አስማተኛ ወንድ: የቆዳ ዓይነት-6',
    '🧙🏻‍♀' => 'አስማተኛ ሴት: የቆዳ ዓይነት-1-2',
    '🧙🏼‍♀' => 'አስማተኛ ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧙🏽‍♀' => 'አስማተኛ ሴት: የቆዳ ዓይነት-4',
    '🧙🏾‍♀' => 'አስማተኛ ሴት: የቆዳ ዓይነት-5',
    '🧙🏿‍♀' => 'አስማተኛ ሴት: የቆዳ ዓይነት-6',
    '🧚🏻‍♂' => 'ጠንቋይ ወንድ: የቆዳ ዓይነት-1-2',
    '🧚🏼‍♂' => 'ጠንቋይ ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧚🏽‍♂' => 'ጠንቋይ ወንድ: የቆዳ ዓይነት-4',
    '🧚🏾‍♂' => 'ጠንቋይ ወንድ: የቆዳ ዓይነት-5',
    '🧚🏿‍♂' => 'ጠንቋይ ወንድ: የቆዳ ዓይነት-6',
    '🧚🏻‍♀' => 'ጠንቋይ ሴት: የቆዳ ዓይነት-1-2',
    '🧚🏼‍♀' => 'ጠንቋይ ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧚🏽‍♀' => 'ጠንቋይ ሴት: የቆዳ ዓይነት-4',
    '🧚🏾‍♀' => 'ጠንቋይ ሴት: የቆዳ ዓይነት-5',
    '🧚🏿‍♀' => 'ጠንቋይ ሴት: የቆዳ ዓይነት-6',
    '🧛🏻‍♂' => 'ጥርስ ያለው ወንድ ጭራቅ: የቆዳ ዓይነት-1-2',
    '🧛🏼‍♂' => 'ጥርስ ያለው ወንድ ጭራቅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧛🏽‍♂' => 'ጥርስ ያለው ወንድ ጭራቅ: የቆዳ ዓይነት-4',
    '🧛🏾‍♂' => 'ጥርስ ያለው ወንድ ጭራቅ: የቆዳ ዓይነት-5',
    '🧛🏿‍♂' => 'ጥርስ ያለው ወንድ ጭራቅ: የቆዳ ዓይነት-6',
    '🧛🏻‍♀' => 'ጥርስ ያላት ሴት ጭራቅ: የቆዳ ዓይነት-1-2',
    '🧛🏼‍♀' => 'ጥርስ ያላት ሴት ጭራቅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧛🏽‍♀' => 'ጥርስ ያላት ሴት ጭራቅ: የቆዳ ዓይነት-4',
    '🧛🏾‍♀' => 'ጥርስ ያላት ሴት ጭራቅ: የቆዳ ዓይነት-5',
    '🧛🏿‍♀' => 'ጥርስ ያላት ሴት ጭራቅ: የቆዳ ዓይነት-6',
    '🧜🏻‍♂' => 'ወንድ ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-1-2',
    '🧜🏼‍♂' => 'ወንድ ግማሽ ሰው ግማሽ ዓሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧜🏽‍♂' => 'ወንድ ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-4',
    '🧜🏾‍♂' => 'ወንድ ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-5',
    '🧜🏿‍♂' => 'ወንድ ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-6',
    '🧜🏻‍♀' => 'ሴት ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-1-2',
    '🧜🏼‍♀' => 'ሴት ግማሽ ሰው ግማሽ ዓሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧜🏽‍♀' => 'ሴት ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-4',
    '🧜🏾‍♀' => 'ሴት ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-5',
    '🧜🏿‍♀' => 'ሴት ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-6',
    '🧝🏻‍♂' => 'ወንድ ኤልፍ: የቆዳ ዓይነት-1-2',
    '🧝🏼‍♂' => 'ወንድ ኤልፍ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧝🏽‍♂' => 'ወንድ ኤልፍ: የቆዳ ዓይነት-4',
    '🧝🏾‍♂' => 'ወንድ ኤልፍ: የቆዳ ዓይነት-5',
    '🧝🏿‍♂' => 'ወንድ ኤልፍ: የቆዳ ዓይነት-6',
    '🧝🏻‍♀' => 'ሴት ኤልፍ: የቆዳ ዓይነት-1-2',
    '🧝🏼‍♀' => 'ሴት ኤልፍ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧝🏽‍♀' => 'ሴት ኤልፍ: የቆዳ ዓይነት-4',
    '🧝🏾‍♀' => 'ሴት ኤልፍ: የቆዳ ዓይነት-5',
    '🧝🏿‍♀' => 'ሴት ኤልፍ: የቆዳ ዓይነት-6',
    '💆🏻‍♂' => 'ወንድ የፊት ማሳጅ: የቆዳ ዓይነት-1-2',
    '💆🏼‍♂' => 'ወንድ የፊት ማሳጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💆🏽‍♂' => 'ወንድ የፊት ማሳጅ: የቆዳ ዓይነት-4',
    '💆🏾‍♂' => 'ወንድ የፊት ማሳጅ: የቆዳ ዓይነት-5',
    '💆🏿‍♂' => 'ወንድ የፊት ማሳጅ: የቆዳ ዓይነት-6',
    '💆🏻‍♀' => 'ሴት የፊት ማሳጅ: የቆዳ ዓይነት-1-2',
    '💆🏼‍♀' => 'ሴት የፊት ማሳጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💆🏽‍♀' => 'ሴት የፊት ማሳጅ: የቆዳ ዓይነት-4',
    '💆🏾‍♀' => 'ሴት የፊት ማሳጅ: የቆዳ ዓይነት-5',
    '💆🏿‍♀' => 'ሴት የፊት ማሳጅ: የቆዳ ዓይነት-6',
    '💇🏻‍♂' => 'ወንድ ጸጉር መቆረጥ: የቆዳ ዓይነት-1-2',
    '💇🏼‍♂' => 'ወንድ ጸጉር መቆረጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💇🏽‍♂' => 'ወንድ ጸጉር መቆረጥ: የቆዳ ዓይነት-4',
    '💇🏾‍♂' => 'ወንድ ጸጉር መቆረጥ: የቆዳ ዓይነት-5',
    '💇🏿‍♂' => 'ወንድ ጸጉር መቆረጥ: የቆዳ ዓይነት-6',
    '💇🏻‍♀' => 'ሴት ጸጉር መቆረጥ: የቆዳ ዓይነት-1-2',
    '💇🏼‍♀' => 'ሴት ጸጉር መቆረጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💇🏽‍♀' => 'ሴት ጸጉር መቆረጥ: የቆዳ ዓይነት-4',
    '💇🏾‍♀' => 'ሴት ጸጉር መቆረጥ: የቆዳ ዓይነት-5',
    '💇🏿‍♀' => 'ሴት ጸጉር መቆረጥ: የቆዳ ዓይነት-6',
    '🚶🏻‍♂' => 'የወንድ እግር ጉዞ: የቆዳ ዓይነት-1-2',
    '🚶🏼‍♂' => 'የወንድ እግር ጉዞ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚶🏽‍♂' => 'የወንድ እግር ጉዞ: የቆዳ ዓይነት-4',
    '🚶🏾‍♂' => 'የወንድ እግር ጉዞ: የቆዳ ዓይነት-5',
    '🚶🏿‍♂' => 'የወንድ እግር ጉዞ: የቆዳ ዓይነት-6',
    '🚶🏻‍♀' => 'የሴቶች እግር ጉዞ: የቆዳ ዓይነት-1-2',
    '🚶🏼‍♀' => 'የሴቶች እግር ጉዞ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚶🏽‍♀' => 'የሴቶች እግር ጉዞ: የቆዳ ዓይነት-4',
    '🚶🏾‍♀' => 'የሴቶች እግር ጉዞ: የቆዳ ዓይነት-5',
    '🚶🏿‍♀' => 'የሴቶች እግር ጉዞ: የቆዳ ዓይነት-6',
    '🧍🏻‍♂' => 'ወንድ ቆሞ: የቆዳ ዓይነት-1-2',
    '🧍🏼‍♂' => 'ወንድ ቆሞ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧍🏽‍♂' => 'ወንድ ቆሞ: የቆዳ ዓይነት-4',
    '🧍🏾‍♂' => 'ወንድ ቆሞ: የቆዳ ዓይነት-5',
    '🧍🏿‍♂' => 'ወንድ ቆሞ: የቆዳ ዓይነት-6',
    '🧍🏻‍♀' => 'ሴት ቆማ: የቆዳ ዓይነት-1-2',
    '🧍🏼‍♀' => 'ሴት ቆማ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧍🏽‍♀' => 'ሴት ቆማ: የቆዳ ዓይነት-4',
    '🧍🏾‍♀' => 'ሴት ቆማ: የቆዳ ዓይነት-5',
    '🧍🏿‍♀' => 'ሴት ቆማ: የቆዳ ዓይነት-6',
    '🧎🏻‍♂' => 'ወንድ ተንበርክኮ: የቆዳ ዓይነት-1-2',
    '🧎🏼‍♂' => 'ወንድ ተንበርክኮ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧎🏽‍♂' => 'ወንድ ተንበርክኮ: የቆዳ ዓይነት-4',
    '🧎🏾‍♂' => 'ወንድ ተንበርክኮ: የቆዳ ዓይነት-5',
    '🧎🏿‍♂' => 'ወንድ ተንበርክኮ: የቆዳ ዓይነት-6',
    '🧎🏻‍♀' => 'ሴት ተንበርክካ: የቆዳ ዓይነት-1-2',
    '🧎🏼‍♀' => 'ሴት ተንበርክካ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧎🏽‍♀' => 'ሴት ተንበርክካ: የቆዳ ዓይነት-4',
    '🧎🏾‍♀' => 'ሴት ተንበርክካ: የቆዳ ዓይነት-5',
    '🧎🏿‍♀' => 'ሴት ተንበርክካ: የቆዳ ዓይነት-6',
    '🧑🏻‍🦯' => 'ሰው ከመምሪያ አገዳ ጋር: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🦯' => 'ሰው ከመምሪያ አገዳ ጋር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🦯' => 'ሰው ከመምሪያ አገዳ ጋር: የቆዳ ዓይነት-4',
    '🧑🏾‍🦯' => 'ሰው ከመምሪያ አገዳ ጋር: የቆዳ ዓይነት-5',
    '🧑🏿‍🦯' => 'ሰው ከመምሪያ አገዳ ጋር: የቆዳ ዓይነት-6',
    '👨🏻‍🦯' => 'ወንድ በከዘራ: የቆዳ ዓይነት-1-2',
    '👨🏼‍🦯' => 'ወንድ በከዘራ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🦯' => 'ወንድ በከዘራ: የቆዳ ዓይነት-4',
    '👨🏾‍🦯' => 'ወንድ በከዘራ: የቆዳ ዓይነት-5',
    '👨🏿‍🦯' => 'ወንድ በከዘራ: የቆዳ ዓይነት-6',
    '👩🏻‍🦯' => 'ሴት በከዘራ: የቆዳ ዓይነት-1-2',
    '👩🏼‍🦯' => 'ሴት በከዘራ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🦯' => 'ሴት በከዘራ: የቆዳ ዓይነት-4',
    '👩🏾‍🦯' => 'ሴት በከዘራ: የቆዳ ዓይነት-5',
    '👩🏿‍🦯' => 'ሴት በከዘራ: የቆዳ ዓይነት-6',
    '🧑🏻‍🦼' => 'ሰው ከባለሞተር መቀመጫ ጋር: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🦼' => 'ሰው ከባለሞተር መቀመጫ ጋር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🦼' => 'ሰው ከባለሞተር መቀመጫ ጋር: የቆዳ ዓይነት-4',
    '🧑🏾‍🦼' => 'ሰው ከባለሞተር መቀመጫ ጋር: የቆዳ ዓይነት-5',
    '🧑🏿‍🦼' => 'ሰው ከባለሞተር መቀመጫ ጋር: የቆዳ ዓይነት-6',
    '👨🏻‍🦼' => 'ወንድ በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-1-2',
    '👨🏼‍🦼' => 'ወንድ በሞተር ተሽከርካሪ ወንበር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🦼' => 'ወንድ በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-4',
    '👨🏾‍🦼' => 'ወንድ በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-5',
    '👨🏿‍🦼' => 'ወንድ በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-6',
    '👩🏻‍🦼' => 'ሴት በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-1-2',
    '👩🏼‍🦼' => 'ሴት በሞተር ተሽከርካሪ ወንበር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🦼' => 'ሴት በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-4',
    '👩🏾‍🦼' => 'ሴት በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-5',
    '👩🏿‍🦼' => 'ሴት በሞተር ተሽከርካሪ ወንበር: የቆዳ ዓይነት-6',
    '🧑🏻‍🦽' => 'ሰው በባለእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-1-2',
    '🧑🏼‍🦽' => 'ሰው በባለእጅ ተሽከርካሪ ወንበር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽‍🦽' => 'ሰው በባለእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-4',
    '🧑🏾‍🦽' => 'ሰው በባለእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-5',
    '🧑🏿‍🦽' => 'ሰው በባለእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-6',
    '👨🏻‍🦽' => 'ወንድ በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-1-2',
    '👨🏼‍🦽' => 'ወንድ በእጅ ተሽከርካሪ ወንበር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽‍🦽' => 'ወንድ በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-4',
    '👨🏾‍🦽' => 'ወንድ በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-5',
    '👨🏿‍🦽' => 'ወንድ በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-6',
    '👩🏻‍🦽' => 'ሴት በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-1-2',
    '👩🏼‍🦽' => 'ሴት በእጅ ተሽከርካሪ ወንበር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽‍🦽' => 'ሴት በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-4',
    '👩🏾‍🦽' => 'ሴት በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-5',
    '👩🏿‍🦽' => 'ሴት በእጅ ተሽከርካሪ ወንበር: የቆዳ ዓይነት-6',
    '🏃🏻‍♂' => 'የወንድ ሩጫ: የቆዳ ዓይነት-1-2',
    '🏃🏼‍♂' => 'የወንድ ሩጫ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏃🏽‍♂' => 'የወንድ ሩጫ: የቆዳ ዓይነት-4',
    '🏃🏾‍♂' => 'የወንድ ሩጫ: የቆዳ ዓይነት-5',
    '🏃🏿‍♂' => 'የወንድ ሩጫ: የቆዳ ዓይነት-6',
    '🏃🏻‍♀' => 'የሴት ሩጫ: የቆዳ ዓይነት-1-2',
    '🏃🏼‍♀' => 'የሴት ሩጫ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏃🏽‍♀' => 'የሴት ሩጫ: የቆዳ ዓይነት-4',
    '🏃🏾‍♀' => 'የሴት ሩጫ: የቆዳ ዓይነት-5',
    '🏃🏿‍♀' => 'የሴት ሩጫ: የቆዳ ዓይነት-6',
    '👯🏻‍♀' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-1-2',
    '👯🏼‍♀' => 'ሴቶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👯🏽‍♀' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-4',
    '👯🏾‍♀' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-5',
    '👯🏿‍♀' => 'ሴቶች ሲጨፍሩ: የቆዳ ዓይነት-6',
    '👯🏻‍♂' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-1-2',
    '👯🏼‍♂' => 'ወንዶች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👯🏽‍♂' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-4',
    '👯🏾‍♂' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-5',
    '👯🏿‍♂' => 'ወንዶች ሲጨፍሩ: የቆዳ ዓይነት-6',
    '🧖🏻‍♂' => 'ወንድ በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-1-2',
    '🧖🏼‍♂' => 'ወንድ በእንፋሎት ክፍል ውስጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧖🏽‍♂' => 'ወንድ በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-4',
    '🧖🏾‍♂' => 'ወንድ በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-5',
    '🧖🏿‍♂' => 'ወንድ በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-6',
    '🧖🏻‍♀' => 'ሴት በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-1-2',
    '🧖🏼‍♀' => 'ሴት በእንፋሎት ክፍል ውስጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧖🏽‍♀' => 'ሴት በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-4',
    '🧖🏾‍♀' => 'ሴት በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-5',
    '🧖🏿‍♀' => 'ሴት በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-6',
    '🧗🏻‍♂' => 'ወንድ ተራራ ሲወጣ: የቆዳ ዓይነት-1-2',
    '🧗🏼‍♂' => 'ወንድ ተራራ ሲወጣ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧗🏽‍♂' => 'ወንድ ተራራ ሲወጣ: የቆዳ ዓይነት-4',
    '🧗🏾‍♂' => 'ወንድ ተራራ ሲወጣ: የቆዳ ዓይነት-5',
    '🧗🏿‍♂' => 'ወንድ ተራራ ሲወጣ: የቆዳ ዓይነት-6',
    '🧗🏻‍♀' => 'ሴት ተራራ ስትወጣ: የቆዳ ዓይነት-1-2',
    '🧗🏼‍♀' => 'ሴት ተራራ ስትወጣ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧗🏽‍♀' => 'ሴት ተራራ ስትወጣ: የቆዳ ዓይነት-4',
    '🧗🏾‍♀' => 'ሴት ተራራ ስትወጣ: የቆዳ ዓይነት-5',
    '🧗🏿‍♀' => 'ሴት ተራራ ስትወጣ: የቆዳ ዓይነት-6',
    '🏌🏻‍♂' => 'ወንድ ጎልፍ ተጫዋች: የቆዳ ዓይነት-1-2',
    '🏌🏼‍♂' => 'ወንድ ጎልፍ ተጫዋች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏌🏽‍♂' => 'ወንድ ጎልፍ ተጫዋች: የቆዳ ዓይነት-4',
    '🏌🏾‍♂' => 'ወንድ ጎልፍ ተጫዋች: የቆዳ ዓይነት-5',
    '🏌🏿‍♂' => 'ወንድ ጎልፍ ተጫዋች: የቆዳ ዓይነት-6',
    '🏌🏻‍♀' => 'ሴት ጎልፍ ተጫዋች: የቆዳ ዓይነት-1-2',
    '🏌🏼‍♀' => 'ሴት ጎልፍ ተጫዋች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏌🏽‍♀' => 'ሴት ጎልፍ ተጫዋች: የቆዳ ዓይነት-4',
    '🏌🏾‍♀' => 'ሴት ጎልፍ ተጫዋች: የቆዳ ዓይነት-5',
    '🏌🏿‍♀' => 'ሴት ጎልፍ ተጫዋች: የቆዳ ዓይነት-6',
    '🏄🏻‍♂' => 'ወንድ የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-1-2',
    '🏄🏼‍♂' => 'ወንድ የውሃ ሸርተቴ ተጫዋች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏄🏽‍♂' => 'ወንድ የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-4',
    '🏄🏾‍♂' => 'ወንድ የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-5',
    '🏄🏿‍♂' => 'ወንድ የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-6',
    '🏄🏻‍♀' => 'ሴት የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-1-2',
    '🏄🏼‍♀' => 'ሴት የውሃ ሸርተቴ ተጫዋች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏄🏽‍♀' => 'ሴት የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-4',
    '🏄🏾‍♀' => 'ሴት የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-5',
    '🏄🏿‍♀' => 'ሴት የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-6',
    '🚣🏻‍♂' => 'የወንድ ጀልባ ቀዛፊ: የቆዳ ዓይነት-1-2',
    '🚣🏼‍♂' => 'የወንድ ጀልባ ቀዛፊ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚣🏽‍♂' => 'የወንድ ጀልባ ቀዛፊ: የቆዳ ዓይነት-4',
    '🚣🏾‍♂' => 'የወንድ ጀልባ ቀዛፊ: የቆዳ ዓይነት-5',
    '🚣🏿‍♂' => 'የወንድ ጀልባ ቀዛፊ: የቆዳ ዓይነት-6',
    '🚣🏻‍♀' => 'የሴት ጀልባ ቀዛፊ: የቆዳ ዓይነት-1-2',
    '🚣🏼‍♀' => 'የሴት ጀልባ ቀዛፊ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚣🏽‍♀' => 'የሴት ጀልባ ቀዛፊ: የቆዳ ዓይነት-4',
    '🚣🏾‍♀' => 'የሴት ጀልባ ቀዛፊ: የቆዳ ዓይነት-5',
    '🚣🏿‍♀' => 'የሴት ጀልባ ቀዛፊ: የቆዳ ዓይነት-6',
    '🏊🏻‍♂' => 'የወንድ ዋናተኛ: የቆዳ ዓይነት-1-2',
    '🏊🏼‍♂' => 'የወንድ ዋናተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏊🏽‍♂' => 'የወንድ ዋናተኛ: የቆዳ ዓይነት-4',
    '🏊🏾‍♂' => 'የወንድ ዋናተኛ: የቆዳ ዓይነት-5',
    '🏊🏿‍♂' => 'የወንድ ዋናተኛ: የቆዳ ዓይነት-6',
    '🏊🏻‍♀' => 'የሴት ዋናተኛ: የቆዳ ዓይነት-1-2',
    '🏊🏼‍♀' => 'የሴት ዋናተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏊🏽‍♀' => 'የሴት ዋናተኛ: የቆዳ ዓይነት-4',
    '🏊🏾‍♀' => 'የሴት ዋናተኛ: የቆዳ ዓይነት-5',
    '🏊🏿‍♀' => 'የሴት ዋናተኛ: የቆዳ ዓይነት-6',
    '⛹🏻‍♂' => 'ኳስ የያዘ ወንድ: የቆዳ ዓይነት-1-2',
    '⛹🏼‍♂' => 'ኳስ የያዘ ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '⛹🏽‍♂' => 'ኳስ የያዘ ወንድ: የቆዳ ዓይነት-4',
    '⛹🏾‍♂' => 'ኳስ የያዘ ወንድ: የቆዳ ዓይነት-5',
    '⛹🏿‍♂' => 'ኳስ የያዘ ወንድ: የቆዳ ዓይነት-6',
    '⛹🏻‍♀' => 'ኳስ የያዘች ሴት: የቆዳ ዓይነት-1-2',
    '⛹🏼‍♀' => 'ኳስ የያዘች ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '⛹🏽‍♀' => 'ኳስ የያዘች ሴት: የቆዳ ዓይነት-4',
    '⛹🏾‍♀' => 'ኳስ የያዘች ሴት: የቆዳ ዓይነት-5',
    '⛹🏿‍♀' => 'ኳስ የያዘች ሴት: የቆዳ ዓይነት-6',
    '🏋🏻‍♂' => 'ወንድ ክብደት አንሺ: የቆዳ ዓይነት-1-2',
    '🏋🏼‍♂' => 'ወንድ ክብደት አንሺ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏋🏽‍♂' => 'ወንድ ክብደት አንሺ: የቆዳ ዓይነት-4',
    '🏋🏾‍♂' => 'ወንድ ክብደት አንሺ: የቆዳ ዓይነት-5',
    '🏋🏿‍♂' => 'ወንድ ክብደት አንሺ: የቆዳ ዓይነት-6',
    '🏋🏻‍♀' => 'ሴት ክብደት አንሺ: የቆዳ ዓይነት-1-2',
    '🏋🏼‍♀' => 'ሴት ክብደት አንሺ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏋🏽‍♀' => 'ሴት ክብደት አንሺ: የቆዳ ዓይነት-4',
    '🏋🏾‍♀' => 'ሴት ክብደት አንሺ: የቆዳ ዓይነት-5',
    '🏋🏿‍♀' => 'ሴት ክብደት አንሺ: የቆዳ ዓይነት-6',
    '🚴🏻‍♂' => 'ወንድ ብስክሌት ጋላቢ: የቆዳ ዓይነት-1-2',
    '🚴🏼‍♂' => 'ወንድ ብስክሌት ጋላቢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚴🏽‍♂' => 'ወንድ ብስክሌት ጋላቢ: የቆዳ ዓይነት-4',
    '🚴🏾‍♂' => 'ወንድ ብስክሌት ጋላቢ: የቆዳ ዓይነት-5',
    '🚴🏿‍♂' => 'ወንድ ብስክሌት ጋላቢ: የቆዳ ዓይነት-6',
    '🚴🏻‍♀' => 'ሴት ብስክሌት ጋላቢ: የቆዳ ዓይነት-1-2',
    '🚴🏼‍♀' => 'ሴት ብስክሌት ጋላቢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚴🏽‍♀' => 'ሴት ብስክሌት ጋላቢ: የቆዳ ዓይነት-4',
    '🚴🏾‍♀' => 'ሴት ብስክሌት ጋላቢ: የቆዳ ዓይነት-5',
    '🚴🏿‍♀' => 'ሴት ብስክሌት ጋላቢ: የቆዳ ዓይነት-6',
    '🚵🏻‍♂' => 'ወንድ የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-1-2',
    '🚵🏼‍♂' => 'ወንድ የተራራ ብስክሌት ጋላቢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚵🏽‍♂' => 'ወንድ የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-4',
    '🚵🏾‍♂' => 'ወንድ የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-5',
    '🚵🏿‍♂' => 'ወንድ የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-6',
    '🚵🏻‍♀' => 'ሴት የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-1-2',
    '🚵🏼‍♀' => 'ሴት የተራራ ብስክሌት ጋላቢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚵🏽‍♀' => 'ሴት የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-4',
    '🚵🏾‍♀' => 'ሴት የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-5',
    '🚵🏿‍♀' => 'ሴት የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-6',
    '🤸🏻‍♂' => 'የወንድ አክሮባት: የቆዳ ዓይነት-1-2',
    '🤸🏼‍♂' => 'የወንድ አክሮባት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤸🏽‍♂' => 'የወንድ አክሮባት: የቆዳ ዓይነት-4',
    '🤸🏾‍♂' => 'የወንድ አክሮባት: የቆዳ ዓይነት-5',
    '🤸🏿‍♂' => 'የወንድ አክሮባት: የቆዳ ዓይነት-6',
    '🤸🏻‍♀' => 'የሴት አክሮባት: የቆዳ ዓይነት-1-2',
    '🤸🏼‍♀' => 'የሴት አክሮባት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤸🏽‍♀' => 'የሴት አክሮባት: የቆዳ ዓይነት-4',
    '🤸🏾‍♀' => 'የሴት አክሮባት: የቆዳ ዓይነት-5',
    '🤸🏿‍♀' => 'የሴት አክሮባት: የቆዳ ዓይነት-6',
    '🤼🏻‍♀' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-1-2',
    '🤼🏼‍♀' => 'የሴት ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤼🏽‍♀' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-4',
    '🤼🏾‍♀' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-5',
    '🤼🏿‍♀' => 'የሴት ነጻ ትግል: የቆዳ ዓይነት-6',
    '🤼🏻‍♂' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-1-2',
    '🤼🏼‍♂' => 'የወንድ ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤼🏽‍♂' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-4',
    '🤼🏾‍♂' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-5',
    '🤼🏿‍♂' => 'የወንድ ነጻ ትግል: የቆዳ ዓይነት-6',
    '🤽🏻‍♂' => 'የወንድ የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-1-2',
    '🤽🏼‍♂' => 'የወንድ የውሃ ላይ ገና ጨዋታ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤽🏽‍♂' => 'የወንድ የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-4',
    '🤽🏾‍♂' => 'የወንድ የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-5',
    '🤽🏿‍♂' => 'የወንድ የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-6',
    '🤽🏻‍♀' => 'የሴት የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-1-2',
    '🤽🏼‍♀' => 'የሴት የውሃ ላይ ገና ጨዋታ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤽🏽‍♀' => 'የሴት የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-4',
    '🤽🏾‍♀' => 'የሴት የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-5',
    '🤽🏿‍♀' => 'የሴት የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-6',
    '🤾🏻‍♂' => 'የወንድ የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-1-2',
    '🤾🏼‍♂' => 'የወንድ የእጅ ኳስ ጨዋታ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤾🏽‍♂' => 'የወንድ የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-4',
    '🤾🏾‍♂' => 'የወንድ የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-5',
    '🤾🏿‍♂' => 'የወንድ የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-6',
    '🤾🏻‍♀' => 'የሴት የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-1-2',
    '🤾🏼‍♀' => 'የሴት የእጅ ኳስ ጨዋታ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤾🏽‍♀' => 'የሴት የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-4',
    '🤾🏾‍♀' => 'የሴት የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-5',
    '🤾🏿‍♀' => 'የሴት የእጅ ኳስ ጨዋታ: የቆዳ ዓይነት-6',
    '🤹🏻‍♂' => 'የወንድ ቅብብሎሽ: የቆዳ ዓይነት-1-2',
    '🤹🏼‍♂' => 'የወንድ ቅብብሎሽ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤹🏽‍♂' => 'የወንድ ቅብብሎሽ: የቆዳ ዓይነት-4',
    '🤹🏾‍♂' => 'የወንድ ቅብብሎሽ: የቆዳ ዓይነት-5',
    '🤹🏿‍♂' => 'የወንድ ቅብብሎሽ: የቆዳ ዓይነት-6',
    '🤹🏻‍♀' => 'የሴት ቅብብሎሽ: የቆዳ ዓይነት-1-2',
    '🤹🏼‍♀' => 'የሴት ቅብብሎሽ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤹🏽‍♀' => 'የሴት ቅብብሎሽ: የቆዳ ዓይነት-4',
    '🤹🏾‍♀' => 'የሴት ቅብብሎሽ: የቆዳ ዓይነት-5',
    '🤹🏿‍♀' => 'የሴት ቅብብሎሽ: የቆዳ ዓይነት-6',
    '🧘🏻‍♂' => 'በሎታስ ኣቀማመጥ ወንድ: የቆዳ ዓይነት-1-2',
    '🧘🏼‍♂' => 'በሎታስ ኣቀማመጥ ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧘🏽‍♂' => 'በሎታስ ኣቀማመጥ ወንድ: የቆዳ ዓይነት-4',
    '🧘🏾‍♂' => 'በሎታስ ኣቀማመጥ ወንድ: የቆዳ ዓይነት-5',
    '🧘🏿‍♂' => 'በሎታስ ኣቀማመጥ ወንድ: የቆዳ ዓይነት-6',
    '🧘🏻‍♀' => 'በሎታስ ኣቀማመጥ ሴት: የቆዳ ዓይነት-1-2',
    '🧘🏼‍♀' => 'በሎታስ ኣቀማመጥ ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧘🏽‍♀' => 'በሎታስ ኣቀማመጥ ሴት: የቆዳ ዓይነት-4',
    '🧘🏾‍♀' => 'በሎታስ ኣቀማመጥ ሴት: የቆዳ ዓይነት-5',
    '🧘🏿‍♀' => 'በሎታስ ኣቀማመጥ ሴት: የቆዳ ዓይነት-6',
    '🧑‍🦰' => 'ጎልማሳ: ቀይ ጸጉር',
    '🧑‍🦱' => 'ጎልማሳ: የተጠቀለለ ጸጉር',
    '🧑‍🦳' => 'ጎልማሳ: ነጭ ጸጉር',
    '🧑‍🦲' => 'ጎልማሳ: መላጣ',
    '👨‍🦰' => 'ሰው: ቀይ ጸጉር',
    '👨‍🦱' => 'ሰው: የተጠቀለለ ጸጉር',
    '👨‍🦳' => 'ሰው: ነጭ ጸጉር',
    '👨‍🦲' => 'ሰው: መላጣ',
    '👩‍🦰' => 'ሴት: ቀይ ጸጉር',
    '👩‍🦱' => 'ሴት: የተጠቀለለ ጸጉር',
    '👩‍🦳' => 'ሴት: ነጭ ጸጉር',
    '👩‍🦲' => 'ሴት: መላጣ',
    '👨‍👦' => 'ቤተሰብ: ሰው እና ወንድ ልጅ',
    '👨‍👧' => 'ቤተሰብ: ሰው እና ልጃገረድ',
    '👩‍👦' => 'ቤተሰብ: ሴት እና ወንድ ልጅ',
    '👩‍👧' => 'ቤተሰብ: ሴት እና ልጃገረድ',
    '😶‍🌫' => 'ደመናማ ገጽ',
    '😮‍💨' => 'ወደ ውጭ የሚተነፍስ ገጽ',
    '🙂‍↕' => 'ራስ መንቀጥቀጥ አቀባዊ',
    '😵‍💫' => 'የዞሩ ዓይኖች ያለው ፊት',
    '❤‍🔥' => 'በበስጭት ላይ ያለ ልብ',
    '❤‍🩹' => 'የተጠገነ ልብ',
    '👁‍🗨' => 'ዓይን በንግግር አፉፋ',
    '🧔‍♂' => 'ሰው: ጺም',
    '👱‍♂' => 'ወርቃማ ጸጉር ያለው ወንድ',
    '🧔‍♀' => 'ሴት: ጺም',
    '👱‍♀' => 'ወርቃማ ጸጉር ያላት ሴት',
    '🙍‍♂' => 'ወንድ ተኮሳትሮ',
    '🙍‍♀' => 'ሴት ተኮሳትራ',
    '🙎‍♂' => 'ወንድ ማለክለክ',
    '🙎‍♀' => 'ሴት ማለክለክ',
    '🙅‍♂' => 'የእጅ ምልክት ለአይሆንም ወንድ',
    '🙅‍♀' => 'የእጅ ምልክት ለአይሆንም ሴት',
    '🙆‍♂' => 'የእጅ ምልክት ለእሺ ወንድ',
    '🙆‍♀' => 'የእጅ ምልክት ለእሺ ሴት',
    '💁‍♂' => 'ወንድ መረጃ ሰጪ',
    '💁‍♀' => 'ሴት መረጃ ሰጪ',
    '🙋‍♂' => 'የወንድ እጅ ማውጣት',
    '🙋‍♀' => 'የሴት እጅ ማውጣት',
    '🧏‍♂' => 'ደንቆሮ ሰው',
    '🧏‍♀' => 'ደንቆሮ ሴት',
    '🙇‍♂' => 'ወንድ ማጎንበስ',
    '🙇‍♀' => 'ሴት ማጎንበስ',
    '🤦‍♂' => 'ወንድ ፊት መቅላት',
    '🤦‍♀' => 'ሴት ፊት መቅላት',
    '🤷‍♂' => 'ወንድ አለማወቅ',
    '🤷‍♀' => 'ሴት አለማወቅ',
    '🧑‍⚕' => 'የጤና ሰራተኛ',
    '👨‍⚕' => 'ወንድ ነርስ',
    '👩‍⚕' => 'ሴት ነርስ',
    '🧑‍🎓' => 'ተማሪ',
    '👨‍🎓' => 'ወንድ ተመራቂ',
    '👩‍🎓' => 'ሴት ተመራቂ',
    '🧑‍🏫' => 'አስተማሪ',
    '👨‍🏫' => 'ወንድ አስተማሪ',
    '👩‍🏫' => 'ሴት አስተማሪ',
    '🧑‍⚖' => 'ዳኛ',
    '👨‍⚖' => 'ወንድ ዳኛ',
    '👩‍⚖' => 'ሴት ዳኛ',
    '🧑‍🌾' => 'ገበሬ',
    '👨‍🌾' => 'ወንድ ገበሬ',
    '👩‍🌾' => 'ሴት ገበሬ',
    '🧑‍🍳' => 'ማብሰል',
    '👨‍🍳' => 'ወንድ አብሳይ',
    '👩‍🍳' => 'ሴት አብሳይ',
    '🧑‍🔧' => 'ሜካኒክ',
    '👨‍🔧' => 'ወንድ ሜካኒክ',
    '👩‍🔧' => 'ሴት ሜካኒክ',
    '🧑‍🏭' => 'የፋብሪካ ሰራተኛ',
    '👨‍🏭' => 'ወንድ የፋብሪካ ሰራተኛ',
    '👩‍🏭' => 'ሴት የፋብሪካ ሰራተኛ',
    '🧑‍💼' => 'የቤሮ ሰራተኛ',
    '👨‍💼' => 'ወንድ የቢሮ ሰራተኛ',
    '👩‍💼' => 'ሴት የቢሮ ሰራተኛ',
    '🧑‍🔬' => 'ሳይንቲስት',
    '👨‍🔬' => 'ወንድ ሳይንቲስት',
    '👩‍🔬' => 'ሴት ሳይንቲስት',
    '🧑‍💻' => 'ቴክኖሎጂስት',
    '👨‍💻' => 'ወንድ ቴክኖሎጂ አዋቂ',
    '👩‍💻' => 'ሴት ቴክኖሎጂ አዋቂ',
    '🧑‍🎤' => 'ዘፋኝ',
    '👨‍🎤' => 'ወንድ ዘፋኝ',
    '👩‍🎤' => 'ሴት ዘፋኝ',
    '🧑‍🎨' => 'አርቲስት',
    '👨‍🎨' => 'ወንድ ሠዓሊ',
    '👩‍🎨' => 'ሴት ሠዓሊ',
    '🧑‍✈' => 'ፓይለት',
    '👨‍✈' => 'ወንድ አውሮፕላን አብራሪ',
    '👩‍✈' => 'ሴት አውሮፕላን አብራሪ',
    '🧑‍🚀' => 'የጠፈር ተመራማሪ',
    '👨‍🚀' => 'ወንድ የህዋ ተመራማሪ',
    '👩‍🚀' => 'ሴት የህዋ ተመራማሪ',
    '🧑‍🚒' => 'የእሳት አደጋ ተከላካይ',
    '👨‍🚒' => 'ወንድ የእሳት አደጋ ተከላካይ',
    '👩‍🚒' => 'ሴት የእሳት አደጋ ተከላካይ',
    '👮‍♂' => 'ወንድ የፖሊስ ሹም',
    '👮‍♀' => 'ሴት የፖሊስ ሹም',
    '🕵‍♂' => 'ወንድ መርማሪ ፖሊስ',
    '🕵‍♀' => 'ሴት መርማሪ ፖሊስ',
    '💂‍♂' => 'ወንድ ጠባቂ',
    '💂‍♀' => 'ሴት ጠባቂ',
    '👷‍♂' => 'ወንድ የግንባታ ሰራተኛ',
    '👷‍♀' => 'ሴት የግንባታ ሰራተኛ',
    '👳‍♂' => 'ጥምጣም ያደረገ ወንድ',
    '👳‍♀' => 'ጥምጣም ያደረች ሴት',
    '🤵‍♂' => 'ሰው በቶክሲዶ',
    '🤵‍♀' => 'ሴት በቶክሲዶ',
    '👰‍♂' => 'ወንድ በዓይነ እርግብ',
    '👰‍♀' => 'ሴት በዓይነ እርግብ',
    '👩‍🍼' => 'ሴት ህጻን ስትመግብ',
    '👨‍🍼' => 'ወንድ ህጻን ሲመግብ',
    '🧑‍🍼' => 'ሰው ህጻን ሲመግብ',
    '🧑‍🎄' => 'ሚክስ ክላውስ',
    '🦸‍♂' => 'የወንድ ጀግና',
    '🦸‍♀' => 'የሴት ጀግና',
    '🦹‍♂' => 'የወንድ ጀግና ምስል',
    '🦹‍♀' => 'የሴት ጀግና ምስል',
    '🧙‍♂' => 'አስማተኛ ወንድ',
    '🧙‍♀' => 'አስማተኛ ሴት',
    '🧚‍♂' => 'ጠንቋይ ወንድ',
    '🧚‍♀' => 'ጠንቋይ ሴት',
    '🧛‍♂' => 'ጥርስ ያለው ወንድ ጭራቅ',
    '🧛‍♀' => 'ጥርስ ያላት ሴት ጭራቅ',
    '🧜‍♂' => 'ወንድ ግማሽ ሰው ግማሽ ዓሳ',
    '🧜‍♀' => 'ሴት ግማሽ ሰው ግማሽ ዓሳ',
    '🧝‍♂' => 'ወንድ ኤልፍ',
    '🧝‍♀' => 'ሴት ኤልፍ',
    '🧞‍♂' => 'ወንድ ጅኒ',
    '🧞‍♀' => 'ሴት ጅኒ',
    '🧟‍♂' => 'ወንድ ዞምቢ',
    '🧟‍♀' => 'ሴት ዞምቢ',
    '💆‍♂' => 'ወንድ የፊት ማሳጅ',
    '💆‍♀' => 'ሴት የፊት ማሳጅ',
    '💇‍♂' => 'ወንድ ጸጉር መቆረጥ',
    '💇‍♀' => 'ሴት ጸጉር መቆረጥ',
    '🚶‍♂' => 'የወንድ እግር ጉዞ',
    '🚶‍♀' => 'የሴቶች እግር ጉዞ',
    '🧍‍♂' => 'ወንድ ቆሞ',
    '🧍‍♀' => 'ሴት ቆማ',
    '🧎‍♂' => 'ወንድ ተንበርክኮ',
    '🧎‍♀' => 'ሴት ተንበርክካ',
    '🧑‍🦯' => 'ሰው ከመምሪያ አገዳ ጋር',
    '👨‍🦯' => 'ወንድ በከዘራ',
    '👩‍🦯' => 'ሴት በከዘራ',
    '🧑‍🦼' => 'ሰው ከባለሞተር መቀመጫ ጋር',
    '👨‍🦼' => 'ወንድ በሞተር ተሽከርካሪ ወንበር',
    '👩‍🦼' => 'ሴት በሞተር ተሽከርካሪ ወንበር',
    '🧑‍🦽' => 'ሰው በባለእጅ ተሽከርካሪ ወንበር',
    '👨‍🦽' => 'ወንድ በእጅ ተሽከርካሪ ወንበር',
    '👩‍🦽' => 'ሴት በእጅ ተሽከርካሪ ወንበር',
    '🏃‍♂' => 'የወንድ ሩጫ',
    '🏃‍♀' => 'የሴት ሩጫ',
    '🧑‍🩰' => 'የባሌት ዳንሰኛ',
    '👯‍♂' => 'ወንዶች ሲጨፍሩ',
    '👯‍♀' => 'ሴቶች ሲጨፍሩ',
    '🧖‍♂' => 'ወንድ በእንፋሎት ክፍል ውስጥ',
    '🧖‍♀' => 'ሴት በእንፋሎት ክፍል ውስጥ',
    '🧗‍♂' => 'ወንድ ተራራ ሲወጣ',
    '🧗‍♀' => 'ሴት ተራራ ስትወጣ',
    '🏌‍♂' => 'ወንድ ጎልፍ ተጫዋች',
    '🏌‍♀' => 'ሴት ጎልፍ ተጫዋች',
    '🏄‍♂' => 'ወንድ የውሃ ሸርተቴ ተጫዋች',
    '🏄‍♀' => 'ሴት የውሃ ሸርተቴ ተጫዋች',
    '🚣‍♂' => 'የወንድ ጀልባ ቀዛፊ',
    '🚣‍♀' => 'የሴት ጀልባ ቀዛፊ',
    '🏊‍♂' => 'የወንድ ዋናተኛ',
    '🏊‍♀' => 'የሴት ዋናተኛ',
    '⛹‍♂' => 'ኳስ የያዘ ወንድ',
    '⛹‍♀' => 'ኳስ የያዘች ሴት',
    '🏋‍♂' => 'ወንድ ክብደት አንሺ',
    '🏋‍♀' => 'ሴት ክብደት አንሺ',
    '🚴‍♂' => 'ወንድ ብስክሌት ጋላቢ',
    '🚴‍♀' => 'ሴት ብስክሌት ጋላቢ',
    '🚵‍♂' => 'ወንድ የተራራ ብስክሌት ጋላቢ',
    '🚵‍♀' => 'ሴት የተራራ ብስክሌት ጋላቢ',
    '🤸‍♂' => 'የወንድ አክሮባት',
    '🤸‍♀' => 'የሴት አክሮባት',
    '🤼‍♂' => 'የወንድ ነጻ ትግል',
    '🤼‍♀' => 'የሴት ነጻ ትግል',
    '🤽‍♂' => 'የወንድ የውሃ ላይ ገና ጨዋታ',
    '🤽‍♀' => 'የሴት የውሃ ላይ ገና ጨዋታ',
    '🤾‍♂' => 'የወንድ የእጅ ኳስ ጨዋታ',
    '🤾‍♀' => 'የሴት የእጅ ኳስ ጨዋታ',
    '🤹‍♂' => 'የወንድ ቅብብሎሽ',
    '🤹‍♀' => 'የሴት ቅብብሎሽ',
    '🧘‍♂' => 'በሎታስ ኣቀማመጥ ወንድ',
    '🧘‍♀' => 'በሎታስ ኣቀማመጥ ሴት',
    '🧑‍🧒' => 'ቤተሰብ፦ አዋቂ፣ ልጅ',
    '🐕‍🦺' => 'አገልግሎት ሰጪ ውሻ',
    '🐈‍⬛' => 'ጥቁር ድመት',
    '🐻‍❄' => 'የዋልታ ድብ',
    '🐦‍⬛' => 'ጥቁር ወፍ',
    '🐦‍🔥' => 'ፊኒክስ',
    '🍋‍🟩' => 'ኖራ',
    '🍄‍🟫' => 'ቡናማ እንጉዳይ',
    '⛓‍💥' => 'የተሰበረ ሰንሰለት',
    '🏳‍🌈' => 'የቀስተ ደመና ባንዲራ',
    '🏳‍⚧' => 'ሰማያዊ ፤ ሓምራዊ እና ነጭ ሰንደቅ',
    '🏴‍☠' => 'የባሕር ወንበዴ ሰንደቅ',
    '👋🏻' => 'ተወዛዋዥ እጅ: የቆዳ ዓይነት-1-2',
    '👋🏼' => 'ተወዛዋዥ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👋🏽' => 'ተወዛዋዥ እጅ: የቆዳ ዓይነት-4',
    '👋🏾' => 'ተወዛዋዥ እጅ: የቆዳ ዓይነት-5',
    '👋🏿' => 'ተወዛዋዥ እጅ: የቆዳ ዓይነት-6',
    '🤚🏻' => 'ወደ ኋል የተዘረጋ እጅ: የቆዳ ዓይነት-1-2',
    '🤚🏼' => 'ወደ ኋል የተዘረጋ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤚🏽' => 'ወደ ኋል የተዘረጋ እጅ: የቆዳ ዓይነት-4',
    '🤚🏾' => 'ወደ ኋል የተዘረጋ እጅ: የቆዳ ዓይነት-5',
    '🤚🏿' => 'ወደ ኋል የተዘረጋ እጅ: የቆዳ ዓይነት-6',
    '🖐🏻' => 'ለሰላምታ ወደ ላይ የተዘረጉ የእጅ ጣቶች: የቆዳ ዓይነት-1-2',
    '🖐🏼' => 'ለሰላምታ ወደ ላይ የተዘረጉ የእጅ ጣቶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🖐🏽' => 'ለሰላምታ ወደ ላይ የተዘረጉ የእጅ ጣቶች: የቆዳ ዓይነት-4',
    '🖐🏾' => 'ለሰላምታ ወደ ላይ የተዘረጉ የእጅ ጣቶች: የቆዳ ዓይነት-5',
    '🖐🏿' => 'ለሰላምታ ወደ ላይ የተዘረጉ የእጅ ጣቶች: የቆዳ ዓይነት-6',
    '✋🏻' => 'ወደ ላይ የተደረገ እጅ: የቆዳ ዓይነት-1-2',
    '✋🏼' => 'ወደ ላይ የተደረገ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '✋🏽' => 'ወደ ላይ የተደረገ እጅ: የቆዳ ዓይነት-4',
    '✋🏾' => 'ወደ ላይ የተደረገ እጅ: የቆዳ ዓይነት-5',
    '✋🏿' => 'ወደ ላይ የተደረገ እጅ: የቆዳ ዓይነት-6',
    '🖖🏻' => 'ቩልካን ሰላምታ: የቆዳ ዓይነት-1-2',
    '🖖🏼' => 'ቩልካን ሰላምታ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🖖🏽' => 'ቩልካን ሰላምታ: የቆዳ ዓይነት-4',
    '🖖🏾' => 'ቩልካን ሰላምታ: የቆዳ ዓይነት-5',
    '🖖🏿' => 'ቩልካን ሰላምታ: የቆዳ ዓይነት-6',
    '🫱🏻' => 'ወደ ቀኝ የዞረ እጅ: የቆዳ ዓይነት-1-2',
    '🫱🏼' => 'ወደ ቀኝ የዞረ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫱🏽' => 'ወደ ቀኝ የዞረ እጅ: የቆዳ ዓይነት-4',
    '🫱🏾' => 'ወደ ቀኝ የዞረ እጅ: የቆዳ ዓይነት-5',
    '🫱🏿' => 'ወደ ቀኝ የዞረ እጅ: የቆዳ ዓይነት-6',
    '🫲🏻' => 'ወደ ግራ የዞረ እጅ: የቆዳ ዓይነት-1-2',
    '🫲🏼' => 'ወደ ግራ የዞረ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫲🏽' => 'ወደ ግራ የዞረ እጅ: የቆዳ ዓይነት-4',
    '🫲🏾' => 'ወደ ግራ የዞረ እጅ: የቆዳ ዓይነት-5',
    '🫲🏿' => 'ወደ ግራ የዞረ እጅ: የቆዳ ዓይነት-6',
    '🫳🏻' => 'መዳፍ ወደታች እጅ: የቆዳ ዓይነት-1-2',
    '🫳🏼' => 'መዳፍ ወደታች እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫳🏽' => 'መዳፍ ወደታች እጅ: የቆዳ ዓይነት-4',
    '🫳🏾' => 'መዳፍ ወደታች እጅ: የቆዳ ዓይነት-5',
    '🫳🏿' => 'መዳፍ ወደታች እጅ: የቆዳ ዓይነት-6',
    '🫴🏻' => 'መዳፍ ወደላይ እጅ: የቆዳ ዓይነት-1-2',
    '🫴🏼' => 'መዳፍ ወደላይ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫴🏽' => 'መዳፍ ወደላይ እጅ: የቆዳ ዓይነት-4',
    '🫴🏾' => 'መዳፍ ወደላይ እጅ: የቆዳ ዓይነት-5',
    '🫴🏿' => 'መዳፍ ወደላይ እጅ: የቆዳ ዓይነት-6',
    '🫷🏻' => 'ወደግራ የሚገፋ እጅ: የቆዳ ዓይነት-1-2',
    '🫷🏼' => 'ወደግራ የሚገፋ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫷🏽' => 'ወደግራ የሚገፋ እጅ: የቆዳ ዓይነት-4',
    '🫷🏾' => 'ወደግራ የሚገፋ እጅ: የቆዳ ዓይነት-5',
    '🫷🏿' => 'ወደግራ የሚገፋ እጅ: የቆዳ ዓይነት-6',
    '🫸🏻' => 'ወደቀኝ የሚገፋ እጅ: የቆዳ ዓይነት-1-2',
    '🫸🏼' => 'ወደቀኝ የሚገፋ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫸🏽' => 'ወደቀኝ የሚገፋ እጅ: የቆዳ ዓይነት-4',
    '🫸🏾' => 'ወደቀኝ የሚገፋ እጅ: የቆዳ ዓይነት-5',
    '🫸🏿' => 'ወደቀኝ የሚገፋ እጅ: የቆዳ ዓይነት-6',
    '👌🏻' => 'እሺ ምልክት እጅ: የቆዳ ዓይነት-1-2',
    '👌🏼' => 'እሺ ምልክት እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👌🏽' => 'እሺ ምልክት እጅ: የቆዳ ዓይነት-4',
    '👌🏾' => 'እሺ ምልክት እጅ: የቆዳ ዓይነት-5',
    '👌🏿' => 'እሺ ምልክት እጅ: የቆዳ ዓይነት-6',
    '🤌🏻' => 'የቆነጠጡ ጣቶች: የቆዳ ዓይነት-1-2',
    '🤌🏼' => 'የቆነጠጡ ጣቶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤌🏽' => 'የቆነጠጡ ጣቶች: የቆዳ ዓይነት-4',
    '🤌🏾' => 'የቆነጠጡ ጣቶች: የቆዳ ዓይነት-5',
    '🤌🏿' => 'የቆነጠጡ ጣቶች: የቆዳ ዓይነት-6',
    '🤏🏻' => 'እጅ መቆንጠጥ: የቆዳ ዓይነት-1-2',
    '🤏🏼' => 'እጅ መቆንጠጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤏🏽' => 'እጅ መቆንጠጥ: የቆዳ ዓይነት-4',
    '🤏🏾' => 'እጅ መቆንጠጥ: የቆዳ ዓይነት-5',
    '🤏🏿' => 'እጅ መቆንጠጥ: የቆዳ ዓይነት-6',
    '✌🏻' => 'የድል አድራጊነት ምልክት እጅ: የቆዳ ዓይነት-1-2',
    '✌🏼' => 'የድል አድራጊነት ምልክት እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '✌🏽' => 'የድል አድራጊነት ምልክት እጅ: የቆዳ ዓይነት-4',
    '✌🏾' => 'የድል አድራጊነት ምልክት እጅ: የቆዳ ዓይነት-5',
    '✌🏿' => 'የድል አድራጊነት ምልክት እጅ: የቆዳ ዓይነት-6',
    '🤞🏻' => 'ጣት ማጣመር: የቆዳ ዓይነት-1-2',
    '🤞🏼' => 'ጣት ማጣመር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤞🏽' => 'ጣት ማጣመር: የቆዳ ዓይነት-4',
    '🤞🏾' => 'ጣት ማጣመር: የቆዳ ዓይነት-5',
    '🤞🏿' => 'ጣት ማጣመር: የቆዳ ዓይነት-6',
    '🫰🏻' => 'አውራ እና ጠቋሚ ጣት የተጠመሩበት እጅ: የቆዳ ዓይነት-1-2',
    '🫰🏼' => 'አውራ እና ጠቋሚ ጣት የተጠመሩበት እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫰🏽' => 'አውራ እና ጠቋሚ ጣት የተጠመሩበት እጅ: የቆዳ ዓይነት-4',
    '🫰🏾' => 'አውራ እና ጠቋሚ ጣት የተጠመሩበት እጅ: የቆዳ ዓይነት-5',
    '🫰🏿' => 'አውራ እና ጠቋሚ ጣት የተጠመሩበት እጅ: የቆዳ ዓይነት-6',
    '🤟🏻' => 'ምልክትህን ወድጀዋለሁ: የቆዳ ዓይነት-1-2',
    '🤟🏼' => 'ምልክትህን ወድጀዋለሁ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤟🏽' => 'ምልክትህን ወድጀዋለሁ: የቆዳ ዓይነት-4',
    '🤟🏾' => 'ምልክትህን ወድጀዋለሁ: የቆዳ ዓይነት-5',
    '🤟🏿' => 'ምልክትህን ወድጀዋለሁ: የቆዳ ዓይነት-6',
    '🤘🏻' => 'የቀንዶች ምልክት: የቆዳ ዓይነት-1-2',
    '🤘🏼' => 'የቀንዶች ምልክት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤘🏽' => 'የቀንዶች ምልክት: የቆዳ ዓይነት-4',
    '🤘🏾' => 'የቀንዶች ምልክት: የቆዳ ዓይነት-5',
    '🤘🏿' => 'የቀንዶች ምልክት: የቆዳ ዓይነት-6',
    '🤙🏻' => 'ደውልልኝ የእጅ ምልክት: የቆዳ ዓይነት-1-2',
    '🤙🏼' => 'ደውልልኝ የእጅ ምልክት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤙🏽' => 'ደውልልኝ የእጅ ምልክት: የቆዳ ዓይነት-4',
    '🤙🏾' => 'ደውልልኝ የእጅ ምልክት: የቆዳ ዓይነት-5',
    '🤙🏿' => 'ደውልልኝ የእጅ ምልክት: የቆዳ ዓይነት-6',
    '👈🏻' => 'የግራ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-1-2',
    '👈🏼' => 'የግራ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👈🏽' => 'የግራ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-4',
    '👈🏾' => 'የግራ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-5',
    '👈🏿' => 'የግራ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-6',
    '👉🏻' => 'የቀኝ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-1-2',
    '👉🏼' => 'የቀኝ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👉🏽' => 'የቀኝ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-4',
    '👉🏾' => 'የቀኝ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-5',
    '👉🏿' => 'የቀኝ አይበሉባ እጅ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-6',
    '👆🏻' => 'ወደ ላይ የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-1-2',
    '👆🏼' => 'ወደ ላይ የተቀሰረ አይበሉባ በኩል አመልካች ጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👆🏽' => 'ወደ ላይ የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-4',
    '👆🏾' => 'ወደ ላይ የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-5',
    '👆🏿' => 'ወደ ላይ የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-6',
    '🖕🏻' => 'የመሃል ጣት: የቆዳ ዓይነት-1-2',
    '🖕🏼' => 'የመሃል ጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🖕🏽' => 'የመሃል ጣት: የቆዳ ዓይነት-4',
    '🖕🏾' => 'የመሃል ጣት: የቆዳ ዓይነት-5',
    '🖕🏿' => 'የመሃል ጣት: የቆዳ ዓይነት-6',
    '👇🏻' => 'ወደ ታች የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-1-2',
    '👇🏼' => 'ወደ ታች የተቀሰረ አይበሉባ በኩል አመልካች ጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👇🏽' => 'ወደ ታች የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-4',
    '👇🏾' => 'ወደ ታች የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-5',
    '👇🏿' => 'ወደ ታች የተቀሰረ አይበሉባ በኩል አመልካች ጣት: የቆዳ ዓይነት-6',
    '☝🏻' => 'ወደ ላይ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-1-2',
    '☝🏼' => 'ወደ ላይ የተቀሰረ አመልካች ጣት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '☝🏽' => 'ወደ ላይ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-4',
    '☝🏾' => 'ወደ ላይ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-5',
    '☝🏿' => 'ወደ ላይ የተቀሰረ አመልካች ጣት: የቆዳ ዓይነት-6',
    '🫵🏻' => 'ወደ ተመልካቹ በጣት መጠቆም: የቆዳ ዓይነት-1-2',
    '🫵🏼' => 'ወደ ተመልካቹ በጣት መጠቆም: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫵🏽' => 'ወደ ተመልካቹ በጣት መጠቆም: የቆዳ ዓይነት-4',
    '🫵🏾' => 'ወደ ተመልካቹ በጣት መጠቆም: የቆዳ ዓይነት-5',
    '🫵🏿' => 'ወደ ተመልካቹ በጣት መጠቆም: የቆዳ ዓይነት-6',
    '👍🏻' => 'አውራ ጣት ወደ ላይ: የቆዳ ዓይነት-1-2',
    '👍🏼' => 'አውራ ጣት ወደ ላይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👍🏽' => 'አውራ ጣት ወደ ላይ: የቆዳ ዓይነት-4',
    '👍🏾' => 'አውራ ጣት ወደ ላይ: የቆዳ ዓይነት-5',
    '👍🏿' => 'አውራ ጣት ወደ ላይ: የቆዳ ዓይነት-6',
    '👎🏻' => 'አውራ ጣት ወደ ታች: የቆዳ ዓይነት-1-2',
    '👎🏼' => 'አውራ ጣት ወደ ታች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👎🏽' => 'አውራ ጣት ወደ ታች: የቆዳ ዓይነት-4',
    '👎🏾' => 'አውራ ጣት ወደ ታች: የቆዳ ዓይነት-5',
    '👎🏿' => 'አውራ ጣት ወደ ታች: የቆዳ ዓይነት-6',
    '✊🏻' => 'ወደ ላይ የተያዘ ቡጢ: የቆዳ ዓይነት-1-2',
    '✊🏼' => 'ወደ ላይ የተያዘ ቡጢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '✊🏽' => 'ወደ ላይ የተያዘ ቡጢ: የቆዳ ዓይነት-4',
    '✊🏾' => 'ወደ ላይ የተያዘ ቡጢ: የቆዳ ዓይነት-5',
    '✊🏿' => 'ወደ ላይ የተያዘ ቡጢ: የቆዳ ዓይነት-6',
    '👊🏻' => 'የተሰነዘረ ጡጫ: የቆዳ ዓይነት-1-2',
    '👊🏼' => 'የተሰነዘረ ጡጫ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👊🏽' => 'የተሰነዘረ ጡጫ: የቆዳ ዓይነት-4',
    '👊🏾' => 'የተሰነዘረ ጡጫ: የቆዳ ዓይነት-5',
    '👊🏿' => 'የተሰነዘረ ጡጫ: የቆዳ ዓይነት-6',
    '🤛🏻' => 'ወደ ግራ የዞረ ቡጢ: የቆዳ ዓይነት-1-2',
    '🤛🏼' => 'ወደ ግራ የዞረ ቡጢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤛🏽' => 'ወደ ግራ የዞረ ቡጢ: የቆዳ ዓይነት-4',
    '🤛🏾' => 'ወደ ግራ የዞረ ቡጢ: የቆዳ ዓይነት-5',
    '🤛🏿' => 'ወደ ግራ የዞረ ቡጢ: የቆዳ ዓይነት-6',
    '🤜🏻' => 'ወደ ቀኝ የዞረ ቡጢ: የቆዳ ዓይነት-1-2',
    '🤜🏼' => 'ወደ ቀኝ የዞረ ቡጢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤜🏽' => 'ወደ ቀኝ የዞረ ቡጢ: የቆዳ ዓይነት-4',
    '🤜🏾' => 'ወደ ቀኝ የዞረ ቡጢ: የቆዳ ዓይነት-5',
    '🤜🏿' => 'ወደ ቀኝ የዞረ ቡጢ: የቆዳ ዓይነት-6',
    '👏🏻' => 'አጨብጫቢ እጆች: የቆዳ ዓይነት-1-2',
    '👏🏼' => 'አጨብጫቢ እጆች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👏🏽' => 'አጨብጫቢ እጆች: የቆዳ ዓይነት-4',
    '👏🏾' => 'አጨብጫቢ እጆች: የቆዳ ዓይነት-5',
    '👏🏿' => 'አጨብጫቢ እጆች: የቆዳ ዓይነት-6',
    '🙌🏻' => 'እጆቹን በቃ ብሎ ያነሳ ሰው: የቆዳ ዓይነት-1-2',
    '🙌🏼' => 'እጆቹን በቃ ብሎ ያነሳ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙌🏽' => 'እጆቹን በቃ ብሎ ያነሳ ሰው: የቆዳ ዓይነት-4',
    '🙌🏾' => 'እጆቹን በቃ ብሎ ያነሳ ሰው: የቆዳ ዓይነት-5',
    '🙌🏿' => 'እጆቹን በቃ ብሎ ያነሳ ሰው: የቆዳ ዓይነት-6',
    '🫶🏻' => 'የልብ ቅርፅ የሰሩ እጆች: የቆዳ ዓይነት-1-2',
    '🫶🏼' => 'የልብ ቅርፅ የሰሩ እጆች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫶🏽' => 'የልብ ቅርፅ የሰሩ እጆች: የቆዳ ዓይነት-4',
    '🫶🏾' => 'የልብ ቅርፅ የሰሩ እጆች: የቆዳ ዓይነት-5',
    '🫶🏿' => 'የልብ ቅርፅ የሰሩ እጆች: የቆዳ ዓይነት-6',
    '👐🏻' => 'የተከፈቱ ሁለት እጆች: የቆዳ ዓይነት-1-2',
    '👐🏼' => 'የተከፈቱ ሁለት እጆች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👐🏽' => 'የተከፈቱ ሁለት እጆች: የቆዳ ዓይነት-4',
    '👐🏾' => 'የተከፈቱ ሁለት እጆች: የቆዳ ዓይነት-5',
    '👐🏿' => 'የተከፈቱ ሁለት እጆች: የቆዳ ዓይነት-6',
    '🤲🏻' => 'መዳፍ አንድ ላይ ወደላይ: የቆዳ ዓይነት-1-2',
    '🤲🏼' => 'መዳፍ አንድ ላይ ወደላይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤲🏽' => 'መዳፍ አንድ ላይ ወደላይ: የቆዳ ዓይነት-4',
    '🤲🏾' => 'መዳፍ አንድ ላይ ወደላይ: የቆዳ ዓይነት-5',
    '🤲🏿' => 'መዳፍ አንድ ላይ ወደላይ: የቆዳ ዓይነት-6',
    '🤝🏻' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-1-2',
    '🤝🏼' => 'እጅ መጨባበጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤝🏽' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-4',
    '🤝🏾' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-5',
    '🤝🏿' => 'እጅ መጨባበጥ: የቆዳ ዓይነት-6',
    '🙏🏻' => 'የእጅ መዳፎቹን ያጣበቀ ሰው: የቆዳ ዓይነት-1-2',
    '🙏🏼' => 'የእጅ መዳፎቹን ያጣበቀ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙏🏽' => 'የእጅ መዳፎቹን ያጣበቀ ሰው: የቆዳ ዓይነት-4',
    '🙏🏾' => 'የእጅ መዳፎቹን ያጣበቀ ሰው: የቆዳ ዓይነት-5',
    '🙏🏿' => 'የእጅ መዳፎቹን ያጣበቀ ሰው: የቆዳ ዓይነት-6',
    '✍🏻' => 'እየጻፈ ያለ እጅ: የቆዳ ዓይነት-1-2',
    '✍🏼' => 'እየጻፈ ያለ እጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '✍🏽' => 'እየጻፈ ያለ እጅ: የቆዳ ዓይነት-4',
    '✍🏾' => 'እየጻፈ ያለ እጅ: የቆዳ ዓይነት-5',
    '✍🏿' => 'እየጻፈ ያለ እጅ: የቆዳ ዓይነት-6',
    '💅🏻' => 'የጥፍር ቀለም: የቆዳ ዓይነት-1-2',
    '💅🏼' => 'የጥፍር ቀለም: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💅🏽' => 'የጥፍር ቀለም: የቆዳ ዓይነት-4',
    '💅🏾' => 'የጥፍር ቀለም: የቆዳ ዓይነት-5',
    '💅🏿' => 'የጥፍር ቀለም: የቆዳ ዓይነት-6',
    '🤳🏻' => 'እራስ ፎቶ ማንሳት: የቆዳ ዓይነት-1-2',
    '🤳🏼' => 'እራስ ፎቶ ማንሳት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤳🏽' => 'እራስ ፎቶ ማንሳት: የቆዳ ዓይነት-4',
    '🤳🏾' => 'እራስ ፎቶ ማንሳት: የቆዳ ዓይነት-5',
    '🤳🏿' => 'እራስ ፎቶ ማንሳት: የቆዳ ዓይነት-6',
    '💪🏻' => 'ታጥፎ የተወጠረ የእጅ ጡንቻ: የቆዳ ዓይነት-1-2',
    '💪🏼' => 'ታጥፎ የተወጠረ የእጅ ጡንቻ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💪🏽' => 'ታጥፎ የተወጠረ የእጅ ጡንቻ: የቆዳ ዓይነት-4',
    '💪🏾' => 'ታጥፎ የተወጠረ የእጅ ጡንቻ: የቆዳ ዓይነት-5',
    '💪🏿' => 'ታጥፎ የተወጠረ የእጅ ጡንቻ: የቆዳ ዓይነት-6',
    '🦵🏻' => 'እግር: የቆዳ ዓይነት-1-2',
    '🦵🏼' => 'እግር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦵🏽' => 'እግር: የቆዳ ዓይነት-4',
    '🦵🏾' => 'እግር: የቆዳ ዓይነት-5',
    '🦵🏿' => 'እግር: የቆዳ ዓይነት-6',
    '🦶🏻' => 'እግር የታችኛው ክፍል: የቆዳ ዓይነት-1-2',
    '🦶🏼' => 'እግር የታችኛው ክፍል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦶🏽' => 'እግር የታችኛው ክፍል: የቆዳ ዓይነት-4',
    '🦶🏾' => 'እግር የታችኛው ክፍል: የቆዳ ዓይነት-5',
    '🦶🏿' => 'እግር የታችኛው ክፍል: የቆዳ ዓይነት-6',
    '👂🏻' => 'ጆሮ: የቆዳ ዓይነት-1-2',
    '👂🏼' => 'ጆሮ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👂🏽' => 'ጆሮ: የቆዳ ዓይነት-4',
    '👂🏾' => 'ጆሮ: የቆዳ ዓይነት-5',
    '👂🏿' => 'ጆሮ: የቆዳ ዓይነት-6',
    '🦻🏻' => 'ጆሮ መስማት ከሚራዳ መሳሪያ ጋር: የቆዳ ዓይነት-1-2',
    '🦻🏼' => 'ጆሮ መስማት ከሚራዳ መሳሪያ ጋር: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦻🏽' => 'ጆሮ መስማት ከሚራዳ መሳሪያ ጋር: የቆዳ ዓይነት-4',
    '🦻🏾' => 'ጆሮ መስማት ከሚራዳ መሳሪያ ጋር: የቆዳ ዓይነት-5',
    '🦻🏿' => 'ጆሮ መስማት ከሚራዳ መሳሪያ ጋር: የቆዳ ዓይነት-6',
    '👃🏻' => 'አፍንጫ: የቆዳ ዓይነት-1-2',
    '👃🏼' => 'አፍንጫ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👃🏽' => 'አፍንጫ: የቆዳ ዓይነት-4',
    '👃🏾' => 'አፍንጫ: የቆዳ ዓይነት-5',
    '👃🏿' => 'አፍንጫ: የቆዳ ዓይነት-6',
    '👶🏻' => 'ሕጻን: የቆዳ ዓይነት-1-2',
    '👶🏼' => 'ሕጻን: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👶🏽' => 'ሕጻን: የቆዳ ዓይነት-4',
    '👶🏾' => 'ሕጻን: የቆዳ ዓይነት-5',
    '👶🏿' => 'ሕጻን: የቆዳ ዓይነት-6',
    '🧒🏻' => 'ልጅ: የቆዳ ዓይነት-1-2',
    '🧒🏼' => 'ልጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧒🏽' => 'ልጅ: የቆዳ ዓይነት-4',
    '🧒🏾' => 'ልጅ: የቆዳ ዓይነት-5',
    '🧒🏿' => 'ልጅ: የቆዳ ዓይነት-6',
    '👦🏻' => 'ወንድ ልጅ: የቆዳ ዓይነት-1-2',
    '👦🏼' => 'ወንድ ልጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👦🏽' => 'ወንድ ልጅ: የቆዳ ዓይነት-4',
    '👦🏾' => 'ወንድ ልጅ: የቆዳ ዓይነት-5',
    '👦🏿' => 'ወንድ ልጅ: የቆዳ ዓይነት-6',
    '👧🏻' => 'ልጃገረድ: የቆዳ ዓይነት-1-2',
    '👧🏼' => 'ልጃገረድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👧🏽' => 'ልጃገረድ: የቆዳ ዓይነት-4',
    '👧🏾' => 'ልጃገረድ: የቆዳ ዓይነት-5',
    '👧🏿' => 'ልጃገረድ: የቆዳ ዓይነት-6',
    '🧑🏻' => 'ጎልማሳ: የቆዳ ዓይነት-1-2',
    '🧑🏼' => 'ጎልማሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧑🏽' => 'ጎልማሳ: የቆዳ ዓይነት-4',
    '🧑🏾' => 'ጎልማሳ: የቆዳ ዓይነት-5',
    '🧑🏿' => 'ጎልማሳ: የቆዳ ዓይነት-6',
    '👱🏻' => 'ጎልማሳ/ሉጫ ጸጉር ያለው: የቆዳ ዓይነት-1-2',
    '👱🏼' => 'ጎልማሳ/ሉጫ ጸጉር ያለው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👱🏽' => 'ጎልማሳ/ሉጫ ጸጉር ያለው: የቆዳ ዓይነት-4',
    '👱🏾' => 'ጎልማሳ/ሉጫ ጸጉር ያለው: የቆዳ ዓይነት-5',
    '👱🏿' => 'ጎልማሳ/ሉጫ ጸጉር ያለው: የቆዳ ዓይነት-6',
    '👨🏻' => 'ሰው: የቆዳ ዓይነት-1-2',
    '👨🏼' => 'ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👨🏽' => 'ሰው: የቆዳ ዓይነት-4',
    '👨🏾' => 'ሰው: የቆዳ ዓይነት-5',
    '👨🏿' => 'ሰው: የቆዳ ዓይነት-6',
    '🧔🏻' => 'ጺማም ሰው: የቆዳ ዓይነት-1-2',
    '🧔🏼' => 'ጺማም ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧔🏽' => 'ጺማም ሰው: የቆዳ ዓይነት-4',
    '🧔🏾' => 'ጺማም ሰው: የቆዳ ዓይነት-5',
    '🧔🏿' => 'ጺማም ሰው: የቆዳ ዓይነት-6',
    '👩🏻' => 'ሴት: የቆዳ ዓይነት-1-2',
    '👩🏼' => 'ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👩🏽' => 'ሴት: የቆዳ ዓይነት-4',
    '👩🏾' => 'ሴት: የቆዳ ዓይነት-5',
    '👩🏿' => 'ሴት: የቆዳ ዓይነት-6',
    '🧓🏻' => 'ያረጀ ጎልማሳ: የቆዳ ዓይነት-1-2',
    '🧓🏼' => 'ያረጀ ጎልማሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧓🏽' => 'ያረጀ ጎልማሳ: የቆዳ ዓይነት-4',
    '🧓🏾' => 'ያረጀ ጎልማሳ: የቆዳ ዓይነት-5',
    '🧓🏿' => 'ያረጀ ጎልማሳ: የቆዳ ዓይነት-6',
    '👴🏻' => 'ሽማግሌ: የቆዳ ዓይነት-1-2',
    '👴🏼' => 'ሽማግሌ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👴🏽' => 'ሽማግሌ: የቆዳ ዓይነት-4',
    '👴🏾' => 'ሽማግሌ: የቆዳ ዓይነት-5',
    '👴🏿' => 'ሽማግሌ: የቆዳ ዓይነት-6',
    '👵🏻' => 'አሮጊት ሴት: የቆዳ ዓይነት-1-2',
    '👵🏼' => 'አሮጊት ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👵🏽' => 'አሮጊት ሴት: የቆዳ ዓይነት-4',
    '👵🏾' => 'አሮጊት ሴት: የቆዳ ዓይነት-5',
    '👵🏿' => 'አሮጊት ሴት: የቆዳ ዓይነት-6',
    '🙍🏻' => 'የተኮሳተረ ሰው: የቆዳ ዓይነት-1-2',
    '🙍🏼' => 'የተኮሳተረ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙍🏽' => 'የተኮሳተረ ሰው: የቆዳ ዓይነት-4',
    '🙍🏾' => 'የተኮሳተረ ሰው: የቆዳ ዓይነት-5',
    '🙍🏿' => 'የተኮሳተረ ሰው: የቆዳ ዓይነት-6',
    '🙎🏻' => 'ሁኔታው ያልጣመው ሰው: የቆዳ ዓይነት-1-2',
    '🙎🏼' => 'ሁኔታው ያልጣመው ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙎🏽' => 'ሁኔታው ያልጣመው ሰው: የቆዳ ዓይነት-4',
    '🙎🏾' => 'ሁኔታው ያልጣመው ሰው: የቆዳ ዓይነት-5',
    '🙎🏿' => 'ሁኔታው ያልጣመው ሰው: የቆዳ ዓይነት-6',
    '🙅🏻' => 'የእጅ ምልክት ለአይሆንም: የቆዳ ዓይነት-1-2',
    '🙅🏼' => 'የእጅ ምልክት ለአይሆንም: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙅🏽' => 'የእጅ ምልክት ለአይሆንም: የቆዳ ዓይነት-4',
    '🙅🏾' => 'የእጅ ምልክት ለአይሆንም: የቆዳ ዓይነት-5',
    '🙅🏿' => 'የእጅ ምልክት ለአይሆንም: የቆዳ ዓይነት-6',
    '🙆🏻' => 'የእጅ ምልክት ለእሺ: የቆዳ ዓይነት-1-2',
    '🙆🏼' => 'የእጅ ምልክት ለእሺ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙆🏽' => 'የእጅ ምልክት ለእሺ: የቆዳ ዓይነት-4',
    '🙆🏾' => 'የእጅ ምልክት ለእሺ: የቆዳ ዓይነት-5',
    '🙆🏿' => 'የእጅ ምልክት ለእሺ: የቆዳ ዓይነት-6',
    '💁🏻' => 'መረጃ ሰጪ ግለሰብ: የቆዳ ዓይነት-1-2',
    '💁🏼' => 'መረጃ ሰጪ ግለሰብ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💁🏽' => 'መረጃ ሰጪ ግለሰብ: የቆዳ ዓይነት-4',
    '💁🏾' => 'መረጃ ሰጪ ግለሰብ: የቆዳ ዓይነት-5',
    '💁🏿' => 'መረጃ ሰጪ ግለሰብ: የቆዳ ዓይነት-6',
    '🙋🏻' => 'ደስ ያለው እጁን የሚያወጣ ሰው: የቆዳ ዓይነት-1-2',
    '🙋🏼' => 'ደስ ያለው እጁን የሚያወጣ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙋🏽' => 'ደስ ያለው እጁን የሚያወጣ ሰው: የቆዳ ዓይነት-4',
    '🙋🏾' => 'ደስ ያለው እጁን የሚያወጣ ሰው: የቆዳ ዓይነት-5',
    '🙋🏿' => 'ደስ ያለው እጁን የሚያወጣ ሰው: የቆዳ ዓይነት-6',
    '🧏🏻' => 'መስማት የተሳነው ሰው: የቆዳ ዓይነት-1-2',
    '🧏🏼' => 'መስማት የተሳነው ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧏🏽' => 'መስማት የተሳነው ሰው: የቆዳ ዓይነት-4',
    '🧏🏾' => 'መስማት የተሳነው ሰው: የቆዳ ዓይነት-5',
    '🧏🏿' => 'መስማት የተሳነው ሰው: የቆዳ ዓይነት-6',
    '🙇🏻' => 'ከወገቡ ጎንበስ ብሎ ሰላምታ የሚሰጥ ሰው: የቆዳ ዓይነት-1-2',
    '🙇🏼' => 'ከወገቡ ጎንበስ ብሎ ሰላምታ የሚሰጥ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🙇🏽' => 'ከወገቡ ጎንበስ ብሎ ሰላምታ የሚሰጥ ሰው: የቆዳ ዓይነት-4',
    '🙇🏾' => 'ከወገቡ ጎንበስ ብሎ ሰላምታ የሚሰጥ ሰው: የቆዳ ዓይነት-5',
    '🙇🏿' => 'ከወገቡ ጎንበስ ብሎ ሰላምታ የሚሰጥ ሰው: የቆዳ ዓይነት-6',
    '🤦🏻' => 'ማዘን: የቆዳ ዓይነት-1-2',
    '🤦🏼' => 'ማዘን: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤦🏽' => 'ማዘን: የቆዳ ዓይነት-4',
    '🤦🏾' => 'ማዘን: የቆዳ ዓይነት-5',
    '🤦🏿' => 'ማዘን: የቆዳ ዓይነት-6',
    '🤷🏻' => 'ንቆ መተው: የቆዳ ዓይነት-1-2',
    '🤷🏼' => 'ንቆ መተው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤷🏽' => 'ንቆ መተው: የቆዳ ዓይነት-4',
    '🤷🏾' => 'ንቆ መተው: የቆዳ ዓይነት-5',
    '🤷🏿' => 'ንቆ መተው: የቆዳ ዓይነት-6',
    '👮🏻' => 'ፖሊስ: የቆዳ ዓይነት-1-2',
    '👮🏼' => 'ፖሊስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👮🏽' => 'ፖሊስ: የቆዳ ዓይነት-4',
    '👮🏾' => 'ፖሊስ: የቆዳ ዓይነት-5',
    '👮🏿' => 'ፖሊስ: የቆዳ ዓይነት-6',
    '🕵🏻' => 'መርማሪ ፖሊስ: የቆዳ ዓይነት-1-2',
    '🕵🏼' => 'መርማሪ ፖሊስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🕵🏽' => 'መርማሪ ፖሊስ: የቆዳ ዓይነት-4',
    '🕵🏾' => 'መርማሪ ፖሊስ: የቆዳ ዓይነት-5',
    '🕵🏿' => 'መርማሪ ፖሊስ: የቆዳ ዓይነት-6',
    '💂🏻' => 'ጠባቂ: የቆዳ ዓይነት-1-2',
    '💂🏼' => 'ጠባቂ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💂🏽' => 'ጠባቂ: የቆዳ ዓይነት-4',
    '💂🏾' => 'ጠባቂ: የቆዳ ዓይነት-5',
    '💂🏿' => 'ጠባቂ: የቆዳ ዓይነት-6',
    '🥷🏻' => 'ኒንጃ: የቆዳ ዓይነት-1-2',
    '🥷🏼' => 'ኒንጃ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🥷🏽' => 'ኒንጃ: የቆዳ ዓይነት-4',
    '🥷🏾' => 'ኒንጃ: የቆዳ ዓይነት-5',
    '🥷🏿' => 'ኒንጃ: የቆዳ ዓይነት-6',
    '👷🏻' => 'የግንባታ ሥራ ሠራተኛ: የቆዳ ዓይነት-1-2',
    '👷🏼' => 'የግንባታ ሥራ ሠራተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👷🏽' => 'የግንባታ ሥራ ሠራተኛ: የቆዳ ዓይነት-4',
    '👷🏾' => 'የግንባታ ሥራ ሠራተኛ: የቆዳ ዓይነት-5',
    '👷🏿' => 'የግንባታ ሥራ ሠራተኛ: የቆዳ ዓይነት-6',
    '🫅🏻' => 'ዘውድ ያደረገ ሰው: የቆዳ ዓይነት-1-2',
    '🫅🏼' => 'ዘውድ ያደረገ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫅🏽' => 'ዘውድ ያደረገ ሰው: የቆዳ ዓይነት-4',
    '🫅🏾' => 'ዘውድ ያደረገ ሰው: የቆዳ ዓይነት-5',
    '🫅🏿' => 'ዘውድ ያደረገ ሰው: የቆዳ ዓይነት-6',
    '🤴🏻' => 'ልዑል: የቆዳ ዓይነት-1-2',
    '🤴🏼' => 'ልዑል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤴🏽' => 'ልዑል: የቆዳ ዓይነት-4',
    '🤴🏾' => 'ልዑል: የቆዳ ዓይነት-5',
    '🤴🏿' => 'ልዑል: የቆዳ ዓይነት-6',
    '👸🏻' => 'ልዕልት: የቆዳ ዓይነት-1-2',
    '👸🏼' => 'ልዕልት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👸🏽' => 'ልዕልት: የቆዳ ዓይነት-4',
    '👸🏾' => 'ልዕልት: የቆዳ ዓይነት-5',
    '👸🏿' => 'ልዕልት: የቆዳ ዓይነት-6',
    '👳🏻' => 'ጥምጣም ያደረገ ሰው: የቆዳ ዓይነት-1-2',
    '👳🏼' => 'ጥምጣም ያደረገ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👳🏽' => 'ጥምጣም ያደረገ ሰው: የቆዳ ዓይነት-4',
    '👳🏾' => 'ጥምጣም ያደረገ ሰው: የቆዳ ዓይነት-5',
    '👳🏿' => 'ጥምጣም ያደረገ ሰው: የቆዳ ዓይነት-6',
    '👲🏻' => 'የቻይና ኮፍያ ያደረገ ሰው: የቆዳ ዓይነት-1-2',
    '👲🏼' => 'የቻይና ኮፍያ ያደረገ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👲🏽' => 'የቻይና ኮፍያ ያደረገ ሰው: የቆዳ ዓይነት-4',
    '👲🏾' => 'የቻይና ኮፍያ ያደረገ ሰው: የቆዳ ዓይነት-5',
    '👲🏿' => 'የቻይና ኮፍያ ያደረገ ሰው: የቆዳ ዓይነት-6',
    '🧕🏻' => 'ሴት በስካርፍ: የቆዳ ዓይነት-1-2',
    '🧕🏼' => 'ሴት በስካርፍ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧕🏽' => 'ሴት በስካርፍ: የቆዳ ዓይነት-4',
    '🧕🏾' => 'ሴት በስካርፍ: የቆዳ ዓይነት-5',
    '🧕🏿' => 'ሴት በስካርፍ: የቆዳ ዓይነት-6',
    '🤵🏻' => 'ሱፍ ልብስ የለበሰ ሰው: የቆዳ ዓይነት-1-2',
    '🤵🏼' => 'ሱፍ ልብስ የለበሰ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤵🏽' => 'ሱፍ ልብስ የለበሰ ሰው: የቆዳ ዓይነት-4',
    '🤵🏾' => 'ሱፍ ልብስ የለበሰ ሰው: የቆዳ ዓይነት-5',
    '🤵🏿' => 'ሱፍ ልብስ የለበሰ ሰው: የቆዳ ዓይነት-6',
    '👰🏻' => 'ቬሎ ያደረገች ሙሽራ: የቆዳ ዓይነት-1-2',
    '👰🏼' => 'ቬሎ ያደረገች ሙሽራ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👰🏽' => 'ቬሎ ያደረገች ሙሽራ: የቆዳ ዓይነት-4',
    '👰🏾' => 'ቬሎ ያደረገች ሙሽራ: የቆዳ ዓይነት-5',
    '👰🏿' => 'ቬሎ ያደረገች ሙሽራ: የቆዳ ዓይነት-6',
    '🤰🏻' => 'እርጉዝ ሴት: የቆዳ ዓይነት-1-2',
    '🤰🏼' => 'እርጉዝ ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤰🏽' => 'እርጉዝ ሴት: የቆዳ ዓይነት-4',
    '🤰🏾' => 'እርጉዝ ሴት: የቆዳ ዓይነት-5',
    '🤰🏿' => 'እርጉዝ ሴት: የቆዳ ዓይነት-6',
    '🫃🏻' => 'እርጉዝ ወንድ: የቆዳ ዓይነት-1-2',
    '🫃🏼' => 'እርጉዝ ወንድ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫃🏽' => 'እርጉዝ ወንድ: የቆዳ ዓይነት-4',
    '🫃🏾' => 'እርጉዝ ወንድ: የቆዳ ዓይነት-5',
    '🫃🏿' => 'እርጉዝ ወንድ: የቆዳ ዓይነት-6',
    '🫄🏻' => 'እርጉዝ ሰው: የቆዳ ዓይነት-1-2',
    '🫄🏼' => 'እርጉዝ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🫄🏽' => 'እርጉዝ ሰው: የቆዳ ዓይነት-4',
    '🫄🏾' => 'እርጉዝ ሰው: የቆዳ ዓይነት-5',
    '🫄🏿' => 'እርጉዝ ሰው: የቆዳ ዓይነት-6',
    '🤱🏻' => 'ጡት ማጥባት: የቆዳ ዓይነት-1-2',
    '🤱🏼' => 'ጡት ማጥባት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤱🏽' => 'ጡት ማጥባት: የቆዳ ዓይነት-4',
    '🤱🏾' => 'ጡት ማጥባት: የቆዳ ዓይነት-5',
    '🤱🏿' => 'ጡት ማጥባት: የቆዳ ዓይነት-6',
    '👼🏻' => 'ሕፃኑ መልዓክ: የቆዳ ዓይነት-1-2',
    '👼🏼' => 'ሕፃኑ መልዓክ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👼🏽' => 'ሕፃኑ መልዓክ: የቆዳ ዓይነት-4',
    '👼🏾' => 'ሕፃኑ መልዓክ: የቆዳ ዓይነት-5',
    '👼🏿' => 'ሕፃኑ መልዓክ: የቆዳ ዓይነት-6',
    '🎅🏻' => 'አባባ ገና: የቆዳ ዓይነት-1-2',
    '🎅🏼' => 'አባባ ገና: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🎅🏽' => 'አባባ ገና: የቆዳ ዓይነት-4',
    '🎅🏾' => 'አባባ ገና: የቆዳ ዓይነት-5',
    '🎅🏿' => 'አባባ ገና: የቆዳ ዓይነት-6',
    '🤶🏻' => 'የገና እናት: የቆዳ ዓይነት-1-2',
    '🤶🏼' => 'የገና እናት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤶🏽' => 'የገና እናት: የቆዳ ዓይነት-4',
    '🤶🏾' => 'የገና እናት: የቆዳ ዓይነት-5',
    '🤶🏿' => 'የገና እናት: የቆዳ ዓይነት-6',
    '🦸🏻' => 'ጀግና: የቆዳ ዓይነት-1-2',
    '🦸🏼' => 'ጀግና: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦸🏽' => 'ጀግና: የቆዳ ዓይነት-4',
    '🦸🏾' => 'ጀግና: የቆዳ ዓይነት-5',
    '🦸🏿' => 'ጀግና: የቆዳ ዓይነት-6',
    '🦹🏻' => 'የጀግና ምስል: የቆዳ ዓይነት-1-2',
    '🦹🏼' => 'የጀግና ምስል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🦹🏽' => 'የጀግና ምስል: የቆዳ ዓይነት-4',
    '🦹🏾' => 'የጀግና ምስል: የቆዳ ዓይነት-5',
    '🦹🏿' => 'የጀግና ምስል: የቆዳ ዓይነት-6',
    '🧙🏻' => 'አስማት: የቆዳ ዓይነት-1-2',
    '🧙🏼' => 'አስማት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧙🏽' => 'አስማት: የቆዳ ዓይነት-4',
    '🧙🏾' => 'አስማት: የቆዳ ዓይነት-5',
    '🧙🏿' => 'አስማት: የቆዳ ዓይነት-6',
    '🧚🏻' => 'ጠንቋይ: የቆዳ ዓይነት-1-2',
    '🧚🏼' => 'ጠንቋይ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧚🏽' => 'ጠንቋይ: የቆዳ ዓይነት-4',
    '🧚🏾' => 'ጠንቋይ: የቆዳ ዓይነት-5',
    '🧚🏿' => 'ጠንቋይ: የቆዳ ዓይነት-6',
    '🧛🏻' => 'ጥርስ ያለው ጭራቅ: የቆዳ ዓይነት-1-2',
    '🧛🏼' => 'ጥርስ ያለው ጭራቅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧛🏽' => 'ጥርስ ያለው ጭራቅ: የቆዳ ዓይነት-4',
    '🧛🏾' => 'ጥርስ ያለው ጭራቅ: የቆዳ ዓይነት-5',
    '🧛🏿' => 'ጥርስ ያለው ጭራቅ: የቆዳ ዓይነት-6',
    '🧜🏻' => 'ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-1-2',
    '🧜🏼' => 'ግማሽ ሰው ግማሽ ዓሳ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧜🏽' => 'ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-4',
    '🧜🏾' => 'ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-5',
    '🧜🏿' => 'ግማሽ ሰው ግማሽ ዓሳ: የቆዳ ዓይነት-6',
    '🧝🏻' => 'ኤልፍ: የቆዳ ዓይነት-1-2',
    '🧝🏼' => 'ኤልፍ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧝🏽' => 'ኤልፍ: የቆዳ ዓይነት-4',
    '🧝🏾' => 'ኤልፍ: የቆዳ ዓይነት-5',
    '🧝🏿' => 'ኤልፍ: የቆዳ ዓይነት-6',
    '💆🏻' => 'የፊት ማሳጅ: የቆዳ ዓይነት-1-2',
    '💆🏼' => 'የፊት ማሳጅ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💆🏽' => 'የፊት ማሳጅ: የቆዳ ዓይነት-4',
    '💆🏾' => 'የፊት ማሳጅ: የቆዳ ዓይነት-5',
    '💆🏿' => 'የፊት ማሳጅ: የቆዳ ዓይነት-6',
    '💇🏻' => 'ጸጉር መቆረጥ: የቆዳ ዓይነት-1-2',
    '💇🏼' => 'ጸጉር መቆረጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💇🏽' => 'ጸጉር መቆረጥ: የቆዳ ዓይነት-4',
    '💇🏾' => 'ጸጉር መቆረጥ: የቆዳ ዓይነት-5',
    '💇🏿' => 'ጸጉር መቆረጥ: የቆዳ ዓይነት-6',
    '🚶🏻' => 'እግረኛ: የቆዳ ዓይነት-1-2',
    '🚶🏼' => 'እግረኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚶🏽' => 'እግረኛ: የቆዳ ዓይነት-4',
    '🚶🏾' => 'እግረኛ: የቆዳ ዓይነት-5',
    '🚶🏿' => 'እግረኛ: የቆዳ ዓይነት-6',
    '🧍🏻' => 'ሰው ቆሞ: የቆዳ ዓይነት-1-2',
    '🧍🏼' => 'ሰው ቆሞ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧍🏽' => 'ሰው ቆሞ: የቆዳ ዓይነት-4',
    '🧍🏾' => 'ሰው ቆሞ: የቆዳ ዓይነት-5',
    '🧍🏿' => 'ሰው ቆሞ: የቆዳ ዓይነት-6',
    '🧎🏻' => 'ሰው ተንበርክኮ: የቆዳ ዓይነት-1-2',
    '🧎🏼' => 'ሰው ተንበርክኮ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧎🏽' => 'ሰው ተንበርክኮ: የቆዳ ዓይነት-4',
    '🧎🏾' => 'ሰው ተንበርክኮ: የቆዳ ዓይነት-5',
    '🧎🏿' => 'ሰው ተንበርክኮ: የቆዳ ዓይነት-6',
    '🏃🏻' => 'ሯጭ: የቆዳ ዓይነት-1-2',
    '🏃🏼' => 'ሯጭ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏃🏽' => 'ሯጭ: የቆዳ ዓይነት-4',
    '🏃🏾' => 'ሯጭ: የቆዳ ዓይነት-5',
    '🏃🏿' => 'ሯጭ: የቆዳ ዓይነት-6',
    '💃🏻' => 'ሴቶች ሲደንሱ: የቆዳ ዓይነት-1-2',
    '💃🏼' => 'ሴቶች ሲደንሱ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💃🏽' => 'ሴቶች ሲደንሱ: የቆዳ ዓይነት-4',
    '💃🏾' => 'ሴቶች ሲደንሱ: የቆዳ ዓይነት-5',
    '💃🏿' => 'ሴቶች ሲደንሱ: የቆዳ ዓይነት-6',
    '🕺🏻' => 'ሰው ሲደንስ: የቆዳ ዓይነት-1-2',
    '🕺🏼' => 'ሰው ሲደንስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🕺🏽' => 'ሰው ሲደንስ: የቆዳ ዓይነት-4',
    '🕺🏾' => 'ሰው ሲደንስ: የቆዳ ዓይነት-5',
    '🕺🏿' => 'ሰው ሲደንስ: የቆዳ ዓይነት-6',
    '🕴🏻' => 'ሽክ ብሎ የለበሰ የሥራ ሰው: የቆዳ ዓይነት-1-2',
    '🕴🏼' => 'ሽክ ብሎ የለበሰ የሥራ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🕴🏽' => 'ሽክ ብሎ የለበሰ የሥራ ሰው: የቆዳ ዓይነት-4',
    '🕴🏾' => 'ሽክ ብሎ የለበሰ የሥራ ሰው: የቆዳ ዓይነት-5',
    '🕴🏿' => 'ሽክ ብሎ የለበሰ የሥራ ሰው: የቆዳ ዓይነት-6',
    '👯🏻' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-1-2',
    '👯🏼' => 'ሰዎች ሲጨፍሩ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👯🏽' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-4',
    '👯🏾' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-5',
    '👯🏿' => 'ሰዎች ሲጨፍሩ: የቆዳ ዓይነት-6',
    '🧖🏻' => 'ሰው በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-1-2',
    '🧖🏼' => 'ሰው በእንፋሎት ክፍል ውስጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧖🏽' => 'ሰው በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-4',
    '🧖🏾' => 'ሰው በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-5',
    '🧖🏿' => 'ሰው በእንፋሎት ክፍል ውስጥ: የቆዳ ዓይነት-6',
    '🧗🏻' => 'ሰው ተራራ ሲወጣ: የቆዳ ዓይነት-1-2',
    '🧗🏼' => 'ሰው ተራራ ሲወጣ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧗🏽' => 'ሰው ተራራ ሲወጣ: የቆዳ ዓይነት-4',
    '🧗🏾' => 'ሰው ተራራ ሲወጣ: የቆዳ ዓይነት-5',
    '🧗🏿' => 'ሰው ተራራ ሲወጣ: የቆዳ ዓይነት-6',
    '🏇🏻' => 'የፈረስ እሽቅድምድም: የቆዳ ዓይነት-1-2',
    '🏇🏼' => 'የፈረስ እሽቅድምድም: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏇🏽' => 'የፈረስ እሽቅድምድም: የቆዳ ዓይነት-4',
    '🏇🏾' => 'የፈረስ እሽቅድምድም: የቆዳ ዓይነት-5',
    '🏇🏿' => 'የፈረስ እሽቅድምድም: የቆዳ ዓይነት-6',
    '🏂🏻' => 'በረዶ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-1-2',
    '🏂🏼' => 'በረዶ ሸርተቴ ተጫዋች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏂🏽' => 'በረዶ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-4',
    '🏂🏾' => 'በረዶ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-5',
    '🏂🏿' => 'በረዶ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-6',
    '🏌🏻' => 'ጎልፍ ተጫዋች: የቆዳ ዓይነት-1-2',
    '🏌🏼' => 'ጎልፍ ተጫዋች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏌🏽' => 'ጎልፍ ተጫዋች: የቆዳ ዓይነት-4',
    '🏌🏾' => 'ጎልፍ ተጫዋች: የቆዳ ዓይነት-5',
    '🏌🏿' => 'ጎልፍ ተጫዋች: የቆዳ ዓይነት-6',
    '🏄🏻' => 'የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-1-2',
    '🏄🏼' => 'የውሃ ሸርተቴ ተጫዋች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏄🏽' => 'የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-4',
    '🏄🏾' => 'የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-5',
    '🏄🏿' => 'የውሃ ሸርተቴ ተጫዋች: የቆዳ ዓይነት-6',
    '🚣🏻' => 'የታንኳ ቀዘፋ: የቆዳ ዓይነት-1-2',
    '🚣🏼' => 'የታንኳ ቀዘፋ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚣🏽' => 'የታንኳ ቀዘፋ: የቆዳ ዓይነት-4',
    '🚣🏾' => 'የታንኳ ቀዘፋ: የቆዳ ዓይነት-5',
    '🚣🏿' => 'የታንኳ ቀዘፋ: የቆዳ ዓይነት-6',
    '🏊🏻' => 'ዋናተኛ: የቆዳ ዓይነት-1-2',
    '🏊🏼' => 'ዋናተኛ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏊🏽' => 'ዋናተኛ: የቆዳ ዓይነት-4',
    '🏊🏾' => 'ዋናተኛ: የቆዳ ዓይነት-5',
    '🏊🏿' => 'ዋናተኛ: የቆዳ ዓይነት-6',
    '⛹🏻' => 'ኳስ የያዘ ሰው: የቆዳ ዓይነት-1-2',
    '⛹🏼' => 'ኳስ የያዘ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '⛹🏽' => 'ኳስ የያዘ ሰው: የቆዳ ዓይነት-4',
    '⛹🏾' => 'ኳስ የያዘ ሰው: የቆዳ ዓይነት-5',
    '⛹🏿' => 'ኳስ የያዘ ሰው: የቆዳ ዓይነት-6',
    '🏋🏻' => 'ክብደት አንሺ: የቆዳ ዓይነት-1-2',
    '🏋🏼' => 'ክብደት አንሺ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏋🏽' => 'ክብደት አንሺ: የቆዳ ዓይነት-4',
    '🏋🏾' => 'ክብደት አንሺ: የቆዳ ዓይነት-5',
    '🏋🏿' => 'ክብደት አንሺ: የቆዳ ዓይነት-6',
    '🚴🏻' => 'ብስክሌት ጋላቢ: የቆዳ ዓይነት-1-2',
    '🚴🏼' => 'ብስክሌት ጋላቢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚴🏽' => 'ብስክሌት ጋላቢ: የቆዳ ዓይነት-4',
    '🚴🏾' => 'ብስክሌት ጋላቢ: የቆዳ ዓይነት-5',
    '🚴🏿' => 'ብስክሌት ጋላቢ: የቆዳ ዓይነት-6',
    '🚵🏻' => 'የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-1-2',
    '🚵🏼' => 'የተራራ ብስክሌት ጋላቢ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🚵🏽' => 'የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-4',
    '🚵🏾' => 'የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-5',
    '🚵🏿' => 'የተራራ ብስክሌት ጋላቢ: የቆዳ ዓይነት-6',
    '🤸🏻' => 'አክሮባት: የቆዳ ዓይነት-1-2',
    '🤸🏼' => 'አክሮባት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤸🏽' => 'አክሮባት: የቆዳ ዓይነት-4',
    '🤸🏾' => 'አክሮባት: የቆዳ ዓይነት-5',
    '🤸🏿' => 'አክሮባት: የቆዳ ዓይነት-6',
    '🤼🏻' => 'ነጻ ትግል: የቆዳ ዓይነት-1-2',
    '🤼🏼' => 'ነጻ ትግል: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤼🏽' => 'ነጻ ትግል: የቆዳ ዓይነት-4',
    '🤼🏾' => 'ነጻ ትግል: የቆዳ ዓይነት-5',
    '🤼🏿' => 'ነጻ ትግል: የቆዳ ዓይነት-6',
    '🤽🏻' => 'የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-1-2',
    '🤽🏼' => 'የውሃ ላይ ገና ጨዋታ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤽🏽' => 'የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-4',
    '🤽🏾' => 'የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-5',
    '🤽🏿' => 'የውሃ ላይ ገና ጨዋታ: የቆዳ ዓይነት-6',
    '🤾🏻' => 'የእጅ ኳስ: የቆዳ ዓይነት-1-2',
    '🤾🏼' => 'የእጅ ኳስ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤾🏽' => 'የእጅ ኳስ: የቆዳ ዓይነት-4',
    '🤾🏾' => 'የእጅ ኳስ: የቆዳ ዓይነት-5',
    '🤾🏿' => 'የእጅ ኳስ: የቆዳ ዓይነት-6',
    '🤹🏻' => 'ቅልልቦሽ: የቆዳ ዓይነት-1-2',
    '🤹🏼' => 'ቅልልቦሽ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🤹🏽' => 'ቅልልቦሽ: የቆዳ ዓይነት-4',
    '🤹🏾' => 'ቅልልቦሽ: የቆዳ ዓይነት-5',
    '🤹🏿' => 'ቅልልቦሽ: የቆዳ ዓይነት-6',
    '🧘🏻' => 'በሎታስ ኣቀማመጥ: የቆዳ ዓይነት-1-2',
    '🧘🏼' => 'በሎታስ ኣቀማመጥ: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🧘🏽' => 'በሎታስ ኣቀማመጥ: የቆዳ ዓይነት-4',
    '🧘🏾' => 'በሎታስ ኣቀማመጥ: የቆዳ ዓይነት-5',
    '🧘🏿' => 'በሎታስ ኣቀማመጥ: የቆዳ ዓይነት-6',
    '🛀🏻' => 'ገላውን የሚታጠብ ሰው: የቆዳ ዓይነት-1-2',
    '🛀🏼' => 'ገላውን የሚታጠብ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🛀🏽' => 'ገላውን የሚታጠብ ሰው: የቆዳ ዓይነት-4',
    '🛀🏾' => 'ገላውን የሚታጠብ ሰው: የቆዳ ዓይነት-5',
    '🛀🏿' => 'ገላውን የሚታጠብ ሰው: የቆዳ ዓይነት-6',
    '🛌🏻' => 'አልጋ ላይ የተኛ ሰው: የቆዳ ዓይነት-1-2',
    '🛌🏼' => 'አልጋ ላይ የተኛ ሰው: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🛌🏽' => 'አልጋ ላይ የተኛ ሰው: የቆዳ ዓይነት-4',
    '🛌🏾' => 'አልጋ ላይ የተኛ ሰው: የቆዳ ዓይነት-5',
    '🛌🏿' => 'አልጋ ላይ የተኛ ሰው: የቆዳ ዓይነት-6',
    '👭🏻' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-1-2',
    '👭🏼' => 'እጅ ለእጅ የተያያዙ ሴቶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👭🏽' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-4',
    '👭🏾' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-5',
    '👭🏿' => 'እጅ ለእጅ የተያያዙ ሴቶች: የቆዳ ዓይነት-6',
    '👫🏻' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-1-2',
    '👫🏼' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👫🏽' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-4',
    '👫🏾' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-5',
    '👫🏿' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት: የቆዳ ዓይነት-6',
    '👬🏻' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-1-2',
    '👬🏼' => 'እጅ ለእጅ የተያያዙ ወንዶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '👬🏽' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-4',
    '👬🏾' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-5',
    '👬🏿' => 'እጅ ለእጅ የተያያዙ ወንዶች: የቆዳ ዓይነት-6',
    '💏🏻' => 'መሳም: የቆዳ ዓይነት-1-2',
    '💏🏼' => 'መሳም: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💏🏽' => 'መሳም: የቆዳ ዓይነት-4',
    '💏🏾' => 'መሳም: የቆዳ ዓይነት-5',
    '💏🏿' => 'መሳም: የቆዳ ዓይነት-6',
    '💑🏻' => 'ልብ በመሃላቸው ያሉ ጥንዶች: የቆዳ ዓይነት-1-2',
    '💑🏼' => 'ልብ በመሃላቸው ያሉ ጥንዶች: መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '💑🏽' => 'ልብ በመሃላቸው ያሉ ጥንዶች: የቆዳ ዓይነት-4',
    '💑🏾' => 'ልብ በመሃላቸው ያሉ ጥንዶች: የቆዳ ዓይነት-5',
    '💑🏿' => 'ልብ በመሃላቸው ያሉ ጥንዶች: የቆዳ ዓይነት-6',
    '🇦🇨' => 'ባንዲራ: አሴንሽን ደሴት',
    '🇦🇩' => 'ባንዲራ: አንዶራ',
    '🇦🇪' => 'ባንዲራ: የተባበሩት ዓረብ ኤምሬትስ',
    '🇦🇫' => 'ባንዲራ: አፍጋኒስታን',
    '🇦🇬' => 'ባንዲራ: አንቲጓ እና ባርቡዳ',
    '🇦🇮' => 'ባንዲራ: አንጉይላ',
    '🇦🇱' => 'ባንዲራ: አልባኒያ',
    '🇦🇲' => 'ባንዲራ: አርሜኒያ',
    '🇦🇴' => 'ባንዲራ: አንጐላ',
    '🇦🇶' => 'ባንዲራ: አንታርክቲካ',
    '🇦🇷' => 'ባንዲራ: አርጀንቲና',
    '🇦🇸' => 'ባንዲራ: የአሜሪካ ሳሞአ',
    '🇦🇹' => 'ባንዲራ: ኦስትሪያ',
    '🇦🇺' => 'ባንዲራ: አውስትራልያ',
    '🇦🇼' => 'ባንዲራ: አሩባ',
    '🇦🇽' => 'ባንዲራ: የአላንድ ደሴቶች',
    '🇦🇿' => 'ባንዲራ: አዘርባጃን',
    '🇧🇦' => 'ባንዲራ: ቦስኒያ እና ሄርዞጎቪኒያ',
    '🇧🇧' => 'ባንዲራ: ባርቤዶስ',
    '🇧🇩' => 'ባንዲራ: ባንግላዲሽ',
    '🇧🇪' => 'ባንዲራ: ቤልጄም',
    '🇧🇫' => 'ባንዲራ: ቡርኪና ፋሶ',
    '🇧🇬' => 'ባንዲራ: ቡልጋሪያ',
    '🇧🇭' => 'ባንዲራ: ባህሬን',
    '🇧🇮' => 'ባንዲራ: ብሩንዲ',
    '🇧🇯' => 'ባንዲራ: ቤኒን',
    '🇧🇱' => 'ባንዲራ: ሴንት ባርቴሌሚ',
    '🇧🇲' => 'ባንዲራ: ቤርሙዳ',
    '🇧🇳' => 'ባንዲራ: ብሩኒ',
    '🇧🇴' => 'ባንዲራ: ቦሊቪያ',
    '🇧🇶' => 'ባንዲራ: የካሪቢያን ኔዘርላንድስ',
    '🇧🇷' => 'ባንዲራ: ብራዚል',
    '🇧🇸' => 'ባንዲራ: ባሃማስ',
    '🇧🇹' => 'ባንዲራ: ቡህታን',
    '🇧🇻' => 'ባንዲራ: ቡቬት ደሴት',
    '🇧🇼' => 'ባንዲራ: ቦትስዋና',
    '🇧🇾' => 'ባንዲራ: ቤላሩስ',
    '🇧🇿' => 'ባንዲራ: በሊዝ',
    '🇨🇦' => 'ባንዲራ: ካናዳ',
    '🇨🇨' => 'ባንዲራ: ኮኮስ(ኬሊንግ) ደሴቶች',
    '🇨🇩' => 'ባንዲራ: ኮንጎ-ኪንሻሳ',
    '🇨🇫' => 'ባንዲራ: ማዕከላዊ አፍሪካ ሪፑብሊክ',
    '🇨🇬' => 'ባንዲራ: ኮንጎ ብራዛቪል',
    '🇨🇭' => 'ባንዲራ: ስዊዘርላንድ',
    '🇨🇮' => 'ባንዲራ: ኮትዲቯር',
    '🇨🇰' => 'ባንዲራ: ኩክ ደሴቶች',
    '🇨🇱' => 'ባንዲራ: ቺሊ',
    '🇨🇲' => 'ባንዲራ: ካሜሩን',
    '🇨🇳' => 'ባንዲራ: ቻይና',
    '🇨🇴' => 'ባንዲራ: ኮሎምቢያ',
    '🇨🇵' => 'ባንዲራ: ክሊፐርቶን ደሴት',
    '🇨🇶' => 'ባንዲራ: ሳርክ',
    '🇨🇷' => 'ባንዲራ: ኮስታሪካ',
    '🇨🇺' => 'ባንዲራ: ኩባ',
    '🇨🇻' => 'ባንዲራ: ኬፕቨርዴ',
    '🇨🇼' => 'ባንዲራ: ኩራሳዎ',
    '🇨🇽' => 'ባንዲራ: ክሪስማስ ደሴት',
    '🇨🇾' => 'ባንዲራ: ሳይፕረስ',
    '🇨🇿' => 'ባንዲራ: ቼቺያ',
    '🇩🇪' => 'ባንዲራ: ጀርመን',
    '🇩🇬' => 'ባንዲራ: ዲዬጎ ጋርሲያ',
    '🇩🇯' => 'ባንዲራ: ጂቡቲ',
    '🇩🇰' => 'ባንዲራ: ዴንማርክ',
    '🇩🇲' => 'ባንዲራ: ዶሚኒካ',
    '🇩🇴' => 'ባንዲራ: ዶመኒካን ሪፑብሊክ',
    '🇩🇿' => 'ባንዲራ: አልጄሪያ',
    '🇪🇦' => 'ባንዲራ: ሴኡታና ሜሊላ',
    '🇪🇨' => 'ባንዲራ: ኢኳዶር',
    '🇪🇪' => 'ባንዲራ: ኤስቶኒያ',
    '🇪🇬' => 'ባንዲራ: ግብጽ',
    '🇪🇭' => 'ባንዲራ: ምዕራባዊ ሳህራ',
    '🇪🇷' => 'ባንዲራ: ኤርትራ',
    '🇪🇸' => 'ባንዲራ: ስፔን',
    '🇪🇹' => 'ባንዲራ: ኢትዮጵያ',
    '🇪🇺' => 'ባንዲራ: የአውሮፓ ህብረት',
    '🇫🇮' => 'ባንዲራ: ፊንላንድ',
    '🇫🇯' => 'ባንዲራ: ፊጂ',
    '🇫🇰' => 'ባንዲራ: የፎክላንድ ደሴቶች',
    '🇫🇲' => 'ባንዲራ: ማይክሮኔዢያ',
    '🇫🇴' => 'ባንዲራ: የፋሮ ደሴቶች',
    '🇫🇷' => 'ባንዲራ: ፈረንሳይ',
    '🇬🇦' => 'ባንዲራ: ጋቦን',
    '🇬🇧' => 'ባንዲራ: ዩናይትድ ኪንግደም',
    '🇬🇩' => 'ባንዲራ: ግሬናዳ',
    '🇬🇪' => 'ባንዲራ: ጆርጂያ',
    '🇬🇫' => 'ባንዲራ: የፈረንሳይ ጉዊአና',
    '🇬🇬' => 'ባንዲራ: ጉርነሲ',
    '🇬🇭' => 'ባንዲራ: ጋና',
    '🇬🇮' => 'ባንዲራ: ጂብራልተር',
    '🇬🇱' => 'ባንዲራ: ግሪንላንድ',
    '🇬🇲' => 'ባንዲራ: ጋምቢያ',
    '🇬🇳' => 'ባንዲራ: ጊኒ',
    '🇬🇵' => 'ባንዲራ: ጉዋደሉፕ',
    '🇬🇶' => 'ባንዲራ: ኢኳቶሪያል ጊኒ',
    '🇬🇷' => 'ባንዲራ: ግሪክ',
    '🇬🇸' => 'ባንዲራ: ደቡብ ጆርጂያ እና የደቡብ ሳንድዊች ደሴቶች',
    '🇬🇹' => 'ባንዲራ: ጉዋቲማላ',
    '🇬🇺' => 'ባንዲራ: ጉዋም',
    '🇬🇼' => 'ባንዲራ: ጊኒ-ቢሳው',
    '🇬🇾' => 'ባንዲራ: ጉያና',
    '🇭🇰' => 'ባንዲራ: ሆንግ ኮንግ ልዩ የአስተዳደር ክልል ቻይና',
    '🇭🇲' => 'ባንዲራ: ኽርድ ኣና ማክዶናልድ ደሴቶች',
    '🇭🇳' => 'ባንዲራ: ሆንዱራስ',
    '🇭🇷' => 'ባንዲራ: ክሮኤሽያ',
    '🇭🇹' => 'ባንዲራ: ሀይቲ',
    '🇭🇺' => 'ባንዲራ: ሀንጋሪ',
    '🇮🇨' => 'ባንዲራ: የካናሪ ደሴቶች',
    '🇮🇩' => 'ባንዲራ: ኢንዶኔዢያ',
    '🇮🇪' => 'ባንዲራ: አየርላንድ',
    '🇮🇱' => 'ባንዲራ: እስራኤል',
    '🇮🇲' => 'ባንዲራ: አይል ኦፍ ማን',
    '🇮🇳' => 'ባንዲራ: ህንድ',
    '🇮🇴' => 'ባንዲራ: የብሪታኒያ ህንድ ውቂያኖስ ግዛት',
    '🇮🇶' => 'ባንዲራ: ኢራቅ',
    '🇮🇷' => 'ባንዲራ: ኢራን',
    '🇮🇸' => 'ባንዲራ: አይስላንድ',
    '🇮🇹' => 'ባንዲራ: ጣሊያን',
    '🇯🇪' => 'ባንዲራ: ጀርዚ',
    '🇯🇲' => 'ባንዲራ: ጃማይካ',
    '🇯🇴' => 'ባንዲራ: ጆርዳን',
    '🇯🇵' => 'ባንዲራ: ጃፓን',
    '🇰🇪' => 'ባንዲራ: ኬንያ',
    '🇰🇬' => 'ባንዲራ: ኪርጊስታን',
    '🇰🇭' => 'ባንዲራ: ካምቦዲያ',
    '🇰🇮' => 'ባንዲራ: ኪሪባቲ',
    '🇰🇲' => 'ባንዲራ: ኮሞሮስ',
    '🇰🇳' => 'ባንዲራ: ቅዱስ ኪትስ እና ኔቪስ',
    '🇰🇵' => 'ባንዲራ: ሰሜን ኮሪያ',
    '🇰🇷' => 'ባንዲራ: ደቡብ ኮሪያ',
    '🇰🇼' => 'ባንዲራ: ኩዌት',
    '🇰🇾' => 'ባንዲራ: ካይማን ደሴቶች',
    '🇰🇿' => 'ባንዲራ: ካዛኪስታን',
    '🇱🇦' => 'ባንዲራ: ላኦስ',
    '🇱🇧' => 'ባንዲራ: ሊባኖስ',
    '🇱🇨' => 'ባንዲራ: ሴንት ሉቺያ',
    '🇱🇮' => 'ባንዲራ: ሊችተንስታይን',
    '🇱🇰' => 'ባንዲራ: ሲሪላንካ',
    '🇱🇷' => 'ባንዲራ: ላይቤሪያ',
    '🇱🇸' => 'ባንዲራ: ሌሶቶ',
    '🇱🇹' => 'ባንዲራ: ሊቱዌኒያ',
    '🇱🇺' => 'ባንዲራ: ሉክሰምበርግ',
    '🇱🇻' => 'ባንዲራ: ላትቪያ',
    '🇱🇾' => 'ባንዲራ: ሊቢያ',
    '🇲🇦' => 'ባንዲራ: ሞሮኮ',
    '🇲🇨' => 'ባንዲራ: ሞናኮ',
    '🇲🇩' => 'ባንዲራ: ሞልዶቫ',
    '🇲🇪' => 'ባንዲራ: ሞንተኔግሮ',
    '🇲🇫' => 'ባንዲራ: ሴንት ማርቲን',
    '🇲🇬' => 'ባንዲራ: ማዳጋስካር',
    '🇲🇭' => 'ባንዲራ: ማርሻል ደሴቶች',
    '🇲🇰' => 'ባንዲራ: ሰሜን መቄዶንያ',
    '🇲🇱' => 'ባንዲራ: ማሊ',
    '🇲🇲' => 'ባንዲራ: ማይናማር(በርማ)',
    '🇲🇳' => 'ባንዲራ: ሞንጎሊያ',
    '🇲🇴' => 'ባንዲራ: ማካኦ ልዩ የአስተዳደር ክልል ቻይና',
    '🇲🇵' => 'ባንዲራ: የሰሜናዊ ማሪያና ደሴቶች',
    '🇲🇶' => 'ባንዲራ: ማርቲኒክ',
    '🇲🇷' => 'ባንዲራ: ሞሪቴኒያ',
    '🇲🇸' => 'ባንዲራ: ሞንትሴራት',
    '🇲🇹' => 'ባንዲራ: ማልታ',
    '🇲🇺' => 'ባንዲራ: ሞሪሸስ',
    '🇲🇻' => 'ባንዲራ: ማልዲቭስ',
    '🇲🇼' => 'ባንዲራ: ማላዊ',
    '🇲🇽' => 'ባንዲራ: ሜክሲኮ',
    '🇲🇾' => 'ባንዲራ: ማሌዢያ',
    '🇲🇿' => 'ባንዲራ: ሞዛምቢክ',
    '🇳🇦' => 'ባንዲራ: ናሚቢያ',
    '🇳🇨' => 'ባንዲራ: ኒው ካሌዶኒያ',
    '🇳🇪' => 'ባንዲራ: ኒጀር',
    '🇳🇫' => 'ባንዲራ: ኖርፎልክ ደሴት',
    '🇳🇬' => 'ባንዲራ: ናይጄሪያ',
    '🇳🇮' => 'ባንዲራ: ኒካራጓ',
    '🇳🇱' => 'ባንዲራ: ኔዘርላንድ',
    '🇳🇴' => 'ባንዲራ: ኖርዌይ',
    '🇳🇵' => 'ባንዲራ: ኔፓል',
    '🇳🇷' => 'ባንዲራ: ናኡሩ',
    '🇳🇺' => 'ባንዲራ: ኒዌ',
    '🇳🇿' => 'ባንዲራ: ኒው ዚላንድ',
    '🇴🇲' => 'ባንዲራ: ኦማን',
    '🇵🇦' => 'ባንዲራ: ፓናማ',
    '🇵🇪' => 'ባንዲራ: ፔሩ',
    '🇵🇫' => 'ባንዲራ: የፈረንሳይ ፖሊኔዢያ',
    '🇵🇬' => 'ባንዲራ: ፓፑዋ ኒው ጊኒ',
    '🇵🇭' => 'ባንዲራ: ፊሊፒንስ',
    '🇵🇰' => 'ባንዲራ: ፓኪስታን',
    '🇵🇱' => 'ባንዲራ: ፖላንድ',
    '🇵🇲' => 'ባንዲራ: ሴንት ፒዬር እና ሚኩኤሎን',
    '🇵🇳' => 'ባንዲራ: ፒትካኢርን ደሴቶች',
    '🇵🇷' => 'ባንዲራ: ፑዌርቶ ሪኮ',
    '🇵🇸' => 'ባንዲራ: የፍልስጤም ግዛት',
    '🇵🇹' => 'ባንዲራ: ፖርቱጋል',
    '🇵🇼' => 'ባንዲራ: ፓላው',
    '🇵🇾' => 'ባንዲራ: ፓራጓይ',
    '🇶🇦' => 'ባንዲራ: ኳታር',
    '🇷🇪' => 'ባንዲራ: ሪዩኒየን',
    '🇷🇴' => 'ባንዲራ: ሮሜኒያ',
    '🇷🇸' => 'ባንዲራ: ሰርብያ',
    '🇷🇺' => 'ባንዲራ: ሩስያ',
    '🇷🇼' => 'ባንዲራ: ሩዋንዳ',
    '🇸🇦' => 'ባንዲራ: ሳውድአረቢያ',
    '🇸🇧' => 'ባንዲራ: ሰለሞን ደሴቶች',
    '🇸🇨' => 'ባንዲራ: ሲሼልስ',
    '🇸🇩' => 'ባንዲራ: ሱዳን',
    '🇸🇪' => 'ባንዲራ: ስዊድን',
    '🇸🇬' => 'ባንዲራ: ሲንጋፖር',
    '🇸🇭' => 'ባንዲራ: ሴንት ሄለና',
    '🇸🇮' => 'ባንዲራ: ስሎቬኒያ',
    '🇸🇯' => 'ባንዲራ: ስቫልባርድ እና ጃን ማየን',
    '🇸🇰' => 'ባንዲራ: ስሎቫኪያ',
    '🇸🇱' => 'ባንዲራ: ሴራሊዮን',
    '🇸🇲' => 'ባንዲራ: ሳን ማሪኖ',
    '🇸🇳' => 'ባንዲራ: ሴኔጋል',
    '🇸🇴' => 'ባንዲራ: ሶማሊያ',
    '🇸🇷' => 'ባንዲራ: ሱሪናም',
    '🇸🇸' => 'ባንዲራ: ደቡብ ሱዳን',
    '🇸🇹' => 'ባንዲራ: ሳኦ ቶሜ እና ፕሪንሲፔ',
    '🇸🇻' => 'ባንዲራ: ኤል ሳልቫዶር',
    '🇸🇽' => 'ባንዲራ: ሲንት ማርተን',
    '🇸🇾' => 'ባንዲራ: ሶሪያ',
    '🇸🇿' => 'ባንዲራ: ኤስዋቲኒ',
    '🇹🇦' => 'ባንዲራ: ትሪስታን ዳ ኩንሃ',
    '🇹🇨' => 'ባንዲራ: የቱርኮችና የካኢኮስ ደሴቶች',
    '🇹🇩' => 'ባንዲራ: ቻድ',
    '🇹🇫' => 'ባንዲራ: የፈረንሳይ ደቡባዊ ግዛቶች',
    '🇹🇬' => 'ባንዲራ: ቶጐ',
    '🇹🇭' => 'ባንዲራ: ታይላንድ',
    '🇹🇯' => 'ባንዲራ: ታጃኪስታን',
    '🇹🇰' => 'ባንዲራ: ቶክላው',
    '🇹🇱' => 'ባንዲራ: ቲሞር ሌስቴ',
    '🇹🇲' => 'ባንዲራ: ቱርክሜኒስታን',
    '🇹🇳' => 'ባንዲራ: ቱኒዚያ',
    '🇹🇴' => 'ባንዲራ: ቶንጋ',
    '🇹🇷' => 'ባንዲራ: ቱርክ',
    '🇹🇹' => 'ባንዲራ: ትሪናዳድ እና ቶቤጎ',
    '🇹🇻' => 'ባንዲራ: ቱቫሉ',
    '🇹🇼' => 'ባንዲራ: ታይዋን',
    '🇹🇿' => 'ባንዲራ: ታንዛኒያ',
    '🇺🇦' => 'ባንዲራ: ዩክሬን',
    '🇺🇬' => 'ባንዲራ: ዩጋንዳ',
    '🇺🇲' => 'ባንዲራ: የዩ ኤስ ጠረፍ ላይ ያሉ ደሴቶች',
    '🇺🇳' => 'ባንዲራ: የተባበሩት መንግስታት',
    '🇺🇸' => 'ባንዲራ: ዩናይትድ ስቴትስ',
    '🇺🇾' => 'ባንዲራ: ኡራጓይ',
    '🇺🇿' => 'ባንዲራ: ኡዝቤኪስታን',
    '🇻🇦' => 'ባንዲራ: ቫቲካን ከተማ',
    '🇻🇨' => 'ባንዲራ: ሴንት ቪንሴንት እና ግሬናዲንስ',
    '🇻🇪' => 'ባንዲራ: ቬንዙዌላ',
    '🇻🇬' => 'ባንዲራ: የእንግሊዝ ቨርጂን ደሴቶች',
    '🇻🇮' => 'ባንዲራ: የአሜሪካ ቨርጂን ደሴቶች',
    '🇻🇳' => 'ባንዲራ: ቬትናም',
    '🇻🇺' => 'ባንዲራ: ቫኑአቱ',
    '🇼🇫' => 'ባንዲራ: ዋሊስ እና ፉቱና ደሴቶች',
    '🇼🇸' => 'ባንዲራ: ሳሞአ',
    '🇽🇰' => 'ባንዲራ: ኮሶቮ',
    '🇾🇪' => 'ባንዲራ: የመን',
    '🇾🇹' => 'ባንዲራ: ሜይኦቴ',
    '🇿🇦' => 'ባንዲራ: ደቡብ አፍሪካ',
    '🇿🇲' => 'ባንዲራ: ዛምቢያ',
    '🇿🇼' => 'ባንዲራ: ዚምቧቤ',
    '🔟' => 'የአብይ ሆሄ ቁልፍ ማብሪያ: 10',
    '🏻' => 'የቆዳ ዓይነት-1-2',
    '🏼' => 'መሃከለኛ ቀላ ያለ የቆዳ ቀለም',
    '🏽' => 'የቆዳ ዓይነት-4',
    '🏾' => 'የቆዳ ዓይነት-5',
    '🏿' => 'የቆዳ ዓይነት-6',
    '😀' => 'ፈገግ ያለ ፊት',
    '😃' => 'አፉ የተከፈተ ሣቂታ ፊት',
    '😄' => 'ከተከፈተ አፍ ጋር ሣቂታ ፊት እና ሣቂታ ዓይኖች',
    '😁' => 'ሣቂታ ዓይኖች ያሉት ፊት',
    '😆' => 'ከተከፈተ አፍ ጋር ሣቂታ ፊት እና የተጨፈኑ ዓይኖች',
    '😅' => 'ከተከፈተ አፍ ጋር ሣቂታ ፊት እና ቀዝቃዛ ላብ',
    '🤣' => 'በሳቅ መሬት ላይ መሽከርከር',
    '😂' => 'የሐሤት እንባ ያሉት ፊት',
    '🙂' => 'በትንሹ ሣቂታ ፊት',
    '🙃' => 'የተገለበጠ ፊት',
    '🫠' => 'እየቀለጠ ያለ ፊት',
    '😉' => 'የሚጣቀስ ፊት',
    '😊' => 'ሣቂታ ዓይኖች ያሉት ሣቂታ ፊት',
    '😇' => 'ላዩ ላይ ክብ ያለበት ሣቂታ ፊት',
    '🥰' => 'የሚስቅ ፊት ከ 3 ልብ ጋር',
    '😍' => 'የልብ ቅርጽ ያላቸው ዓይኖች ያሉት ሣቂታ ፊት',
    '🤩' => 'የተዋንያን አድናቂ',
    '😘' => 'የሚስም ፊት',
    '😗' => 'እየሳመ ያለ ፊት',
    '☺' => 'ሣቂታ ፊት',
    '😚' => 'የሚስም ፊት ከተዘጉ ዓይኖች ጋር',
    '😙' => 'የሚስም ፊት ከሣቂታ ዓይኖች ጋር',
    '🥲' => 'ከእንባ ጋር የሚስቅ ፊት',
    '😋' => 'በጣፋጭ ምግብ የጎመጀ ፊት',
    '😛' => 'ምላሱን ያወጣ ፊት',
    '😜' => 'ምላሱን ያወጣ እና ዓይኑ የሚጣቀስ ፊት',
    '🤪' => 'እብድ ፊት',
    '😝' => 'ምላሱን ያወጣ እና ዓይኑን የጨፈነ ፊት',
    '🤑' => 'አፉ ላይ ገንዘብ ያለበት ፊት',
    '🤗' => 'እያቀፈ ያለ ፊት',
    '🤭' => 'እጁን ከንፈሩ ላይ ያደረገ ፊት',
    '🫢' => 'ከፈጠጡ ዓይኖች እና አፍ ላይ እጅ የጫነ ፊት',
    '🫣' => 'የሚያጮልቅ አይን ያለበት ፊት',
    '🤫' => 'የጸጥታ ፊት',
    '🤔' => 'ሃሳቢ ፊት',
    '🫡' => 'ሰላምታ የሚሰጥ ፊት',
    '🤐' => 'ባለዚፕ አፍ ፊት',
    '🤨' => 'ቅንድብ የተነሳ ፊት',
    '😐' => 'ገለልተኛ ፊት',
    '😑' => 'ምንም ስሜት የሌለው ፊት',
    '😶' => 'አፍ የሌለው ፊት',
    '🫥' => 'ዙሪያውን ሰረዝ ያለው ፊት',
    '😏' => 'በምጸት ፈገግ የሚል ፊት',
    '😒' => 'ያልተገረመ ፊት',
    '🙄' => 'የሚሽከረከሩ ዓይኖች ያሉት ፊት',
    '😬' => 'ታሞ የተጨማደደ ፊት',
    '🤥' => 'የሚዋሽ ፊት',
    '🫨' => 'የሚንቀጠቀጥ ፊት',
    '😌' => 'እፎይ ያለ ፊት',
    '😔' => 'እያሰበ ያለ ፊት',
    '😪' => 'እንቅልፉ የመጣ ፊት',
    '🤤' => 'የለሀጭ ፊት',
    '😴' => 'የሚያንቀላፋ ፊት',
    '🫩' => 'ከዓይኖች በታች ከረጢቶች ጋር ፊት',
    '😷' => 'የሕክምና ጭንብል ያደረገ ፊት',
    '🤒' => 'ቴርሞሜትር የያዘ ፊት',
    '🤕' => 'እራሱ ላይ ፋሻ የጠቀለለ ፊት',
    '🤢' => 'ያስታወከ ፊት',
    '🤮' => 'የሚያስታውክ ፊት',
    '🤧' => 'የሚያስነጥስ ፊት',
    '🥵' => 'የቀላ ፊት',
    '🥶' => 'የቀዘቀዘ ፊት',
    '🥴' => 'አጥወልዋይ ፊት',
    '😵' => 'የደበተው ፊት',
    '🤯' => 'የፈነዳ ጭንቅላት',
    '🤠' => 'የከብት ጠባቂ ኮፍያ ፊት',
    '🥳' => 'የድግስ ፊት',
    '🥸' => 'አሳሳች ፊት',
    '😎' => 'የፀሐይ መነጽር ያደረገ ሣቂታ ፊት',
    '🤓' => 'ያገጠጠ ባለመነጽር ፊት',
    '🧐' => 'የዓይን ባጅ ያደረገ ፊት',
    '😕' => 'ግራ የተጋባ ፊት',
    '🫤' => 'ሰያፍ አፍ ያለው ፊት',
    '😟' => 'የተጨነቀ ፊት',
    '🙁' => 'በከፊል የተኮሳተረ ፊት',
    '☹' => 'የተኮሳተረ ፊት',
    '😮' => 'አፍ የከፈተ ፊት',
    '😯' => 'አንሾካሿኪ ፊት',
    '😲' => 'የተገረመ ፊት',
    '😳' => 'የደነበረ ፊት',
    '🫪' => 'የተዛባ ፊት',
    '🥺' => 'የሙግት ፊት',
    '🥹' => 'እንባ ያቀረረ ፊት',
    '😦' => 'አፉ የተከፈተ የተኮሳተረ ፊት',
    '😧' => 'ተማሮ የሚያለቅስ ፊት',
    '😨' => 'ፍርሃት የተሞላ ፊት',
    '😰' => 'አፉ የተከፈተ እና ቀዝቃዛ ላብ ያለው ፊት',
    '😥' => 'የተከፋ ግን የተጽናና ፊት',
    '😢' => 'የሚያለቅስ ፊት',
    '😭' => 'በከባድ የሚያለቅስ ፊት',
    '😱' => 'በፍርሃት የሚጮኽ ፊት',
    '😖' => 'እንባው የመጣ ከንፈሩ የሚንቀጠቀጥ ፊት',
    '😣' => 'ጽናትን አንጸባራቂ ፊት',
    '😞' => 'የተከፋ ፊት',
    '😓' => 'ቀዝቃዛ ላብ ያለው ፊት',
    '😩' => 'ምርር ያለው ፊት',
    '😫' => 'የደከመው ፊት',
    '🥱' => 'የሚያዛጋ ፊት',
    '😤' => 'ከአፍንጫ እንፋሎት የሚወጣው ፊት',
    '😡' => 'የበገነ ፊት',
    '😠' => 'የተናደደ ፊት',
    '🤬' => 'የአፍ ምልክት ያለበት ፊት',
    '😈' => 'ቀንድ ያለው ሣቂታ ፊት',
    '👿' => 'ጋኔን',
    '💀' => 'የራስ ቅል ዓፅም',
    '☠' => 'የራስቅል ከተጣመረ ዓፅም ጋር',
    '💩' => 'የካካ ክምር',
    '🤡' => 'የአስቂኝ ተዋናይ ፊት',
    '👹' => 'ጭራቅ',
    '👺' => 'ጆሮ ቆራጭ',
    '👻' => 'የሙት መንፈስ',
    '👽' => 'የሌላ ዓለም ፍጡር',
    '👾' => 'የሌላ ዓለም ጭራቅ ፍጡር',
    '🤖' => 'የሮቦት ፊት',
    '😺' => 'አፏን የከፈተች ሣቂታ ድመት ፊት',
    '😸' => 'ዓይኖቿ ሣቂታ የሆኑ ፈገግ ያለች ድመት ፊት',
    '😹' => 'የሐሤት እንባ ያላት ድመት ፊት',
    '😻' => 'ልብ ቅርጽ ያላቸው ዓይኖች ያላት ሣቂታ ድመት',
    '😼' => 'የብልጥ ሣቅ የምትሥቅ ድመት ፊት',
    '😽' => 'ዓይኖቿን የጨፈነች የምትስም ድመት',
    '🙀' => 'የደከማት ድመት ፊት',
    '😿' => 'የምታለቅስ ድመት ፊት',
    '😾' => 'የምታለከልክ ድመት ፊት',
    '🙈' => 'ምንም ክፉ አላይም',
    '🙉' => 'ምንም ክፉ አልሰማም',
    '🙊' => 'ምንም ክፉ አልናገርም',
    '💌' => 'የፍቅር ደብዳቤ',
    '💘' => 'ቀስት የሰነጠቀው ልብ',
    '💝' => 'በሪባን የታሰረ ልብ',
    '💖' => 'አንጸባራቂ ልብ',
    '💗' => 'እያደገ የሚሄድ ልብ',
    '💓' => 'እየመታ ያለ ልብ',
    '💞' => 'የሚሽከረከሩ ልቦች',
    '💕' => 'ሁለት ልቦች',
    '💟' => 'የልብ ጌጥ',
    '❣' => 'በልብ ቅርፅ የተዘጋጀ የቃለ አጋኖ ምልክት',
    '💔' => 'የተሰበረ ልብ',
    '❤' => 'ቀይ ልብ',
    '🩷' => 'ሮዝ ልብ',
    '🧡' => 'ብርቱካናማ ልብ',
    '💛' => 'ቢጫ ልብ',
    '💚' => 'አረንጓዴ ልብ',
    '💙' => 'ሰማያዊ ልብ',
    '🩵' => 'ፈዛዛ ሰማያዊ ልብ',
    '💜' => 'ወይን ጠጅ ልብ',
    '🤎' => 'ቡናማ ልብ',
    '🖤' => 'ጥቁር ልብ',
    '🩶' => 'ግራጫ ልብ',
    '🤍' => 'ነጭ ልብ',
    '💋' => 'የመሳም ምልክት',
    '💯' => 'መቶ ነጥቦች',
    '💢' => 'የንዴት ምልክት',
    '🫯' => 'ደመናን መዋጋት',
    '💥' => 'ግጭት',
    '💫' => 'ድብት',
    '💦' => 'ጣፋጭ ጠብታዎች',
    '💨' => 'እየቦነነ ያለ',
    '🕳' => 'ቀዳዳ',
    '💬' => 'የንግግር አፉፋ',
    '🗨' => 'የግራ ንግግር አፉፋ',
    '🗯' => 'የቀኝ የንዴት አፉፋ',
    '💭' => 'ሐሳብ ገላጭ አፉፋ',
    '💤' => 'ዝ ዝ ዝ',
    '👋' => 'ተወዛዋዥ እጅ',
    '🤚' => 'ወደ ኋል የተዘረጋ እጅ',
    '🖐' => 'ለሰላምታ ወደ ላይ የተዘረጉ የእጅ ጣቶች',
    '✋' => 'ወደ ላይ የተደረገ እጅ',
    '🖖' => 'ቩልካን ሰላምታ',
    '🫱' => 'ወደ ቀኝ የዞረ እጅ',
    '🫲' => 'ወደ ግራ የዞረ እጅ',
    '🫳' => 'መዳፍ ወደታች እጅ',
    '🫴' => 'መዳፍ ወደላይ እጅ',
    '🫷' => 'ወደግራ የሚገፋ እጅ',
    '🫸' => 'ወደቀኝ የሚገፋ እጅ',
    '👌' => 'እሺ ምልክት እጅ',
    '🤌' => 'የቆነጠጡ ጣቶች',
    '🤏' => 'እጅ መቆንጠጥ',
    '✌' => 'የድል አድራጊነት ምልክት እጅ',
    '🤞' => 'ጣት ማጣመር',
    '🫰' => 'አውራ እና ጠቋሚ ጣት የተጠመሩበት እጅ',
    '🤟' => 'ምልክትህን ወድጀዋለሁ',
    '🤘' => 'የቀንዶች ምልክት',
    '🤙' => 'ደውልልኝ የእጅ ምልክት',
    '👈' => 'የግራ አይበሉባ እጅ የተቀሰረ አመልካች ጣት',
    '👉' => 'የቀኝ አይበሉባ እጅ የተቀሰረ አመልካች ጣት',
    '👆' => 'ወደ ላይ የተቀሰረ አይበሉባ በኩል አመልካች ጣት',
    '🖕' => 'የመሃል ጣት',
    '👇' => 'ወደ ታች የተቀሰረ አይበሉባ በኩል አመልካች ጣት',
    '☝' => 'ወደ ላይ የተቀሰረ አመልካች ጣት',
    '🫵' => 'ወደ ተመልካቹ በጣት መጠቆም',
    '👍' => 'አውራ ጣት ወደ ላይ',
    '👎' => 'አውራ ጣት ወደ ታች',
    '✊' => 'ወደ ላይ የተያዘ ቡጢ',
    '👊' => 'የተሰነዘረ ጡጫ',
    '🤛' => 'ወደ ግራ የዞረ ቡጢ',
    '🤜' => 'ወደ ቀኝ የዞረ ቡጢ',
    '👏' => 'አጨብጫቢ እጆች',
    '🙌' => 'እጆቹን በቃ ብሎ ያነሳ ሰው',
    '🫶' => 'የልብ ቅርፅ የሰሩ እጆች',
    '👐' => 'የተከፈቱ ሁለት እጆች',
    '🤲' => 'መዳፍ አንድ ላይ ወደላይ',
    '🤝' => 'እጅ መጨባበጥ',
    '🙏' => 'የእጅ መዳፎቹን ያጣበቀ ሰው',
    '✍' => 'እየጻፈ ያለ እጅ',
    '💅' => 'የጥፍር ቀለም',
    '🤳' => 'እራስ ፎቶ ማንሳት',
    '💪' => 'ታጥፎ የተወጠረ የእጅ ጡንቻ',
    '🦾' => 'አውቶማቲክ እጅ',
    '🦿' => 'አውቶማቲክ እግር',
    '🦵' => 'እግር',
    '🦶' => 'እግር የታችኛው ክፍል',
    '👂' => 'ጆሮ',
    '🦻' => 'ጆሮ መስማት ከሚራዳ መሳሪያ ጋር',
    '👃' => 'አፍንጫ',
    '🧠' => 'አእምሮ',
    '🫀' => 'የልብ አካል',
    '🫁' => 'ሳንባ',
    '🦷' => 'ጥርስ',
    '🦴' => 'አጥንት',
    '👀' => 'ዓይኖች',
    '👁' => 'ዓይን',
    '👅' => 'ምላስ',
    '👄' => 'አፍ',
    '🫦' => 'ከንፈር መንከስ',
    '👶' => 'ሕጻን',
    '🧒' => 'ልጅ',
    '👦' => 'ወንድ ልጅ',
    '👧' => 'ልጃገረድ',
    '🧑' => 'ጎልማሳ',
    '👱' => 'ጎልማሳ/ሉጫ ጸጉር ያለው',
    '👨' => 'ሰው',
    '🧔' => 'ጺማም ሰው',
    '👩' => 'ሴት',
    '🧓' => 'ያረጀ ጎልማሳ',
    '👴' => 'ሽማግሌ',
    '👵' => 'አሮጊት ሴት',
    '🙍' => 'የተኮሳተረ ሰው',
    '🙎' => 'ሁኔታው ያልጣመው ሰው',
    '🙅' => 'የእጅ ምልክት ለአይሆንም',
    '🙆' => 'የእጅ ምልክት ለእሺ',
    '💁' => 'መረጃ ሰጪ ግለሰብ',
    '🙋' => 'ደስ ያለው እጁን የሚያወጣ ሰው',
    '🧏' => 'መስማት የተሳነው ሰው',
    '🙇' => 'ከወገቡ ጎንበስ ብሎ ሰላምታ የሚሰጥ ሰው',
    '🤦' => 'ማዘን',
    '🤷' => 'ንቆ መተው',
    '👮' => 'ፖሊስ',
    '🕵' => 'መርማሪ ፖሊስ',
    '💂' => 'ጠባቂ',
    '🥷' => 'ኒንጃ',
    '👷' => 'የግንባታ ሥራ ሠራተኛ',
    '🫅' => 'ዘውድ ያደረገ ሰው',
    '🤴' => 'ልዑል',
    '👸' => 'ልዕልት',
    '👳' => 'ጥምጣም ያደረገ ሰው',
    '👲' => 'የቻይና ኮፍያ ያደረገ ሰው',
    '🧕' => 'ሴት በስካርፍ',
    '🤵' => 'ሱፍ ልብስ የለበሰ ሰው',
    '👰' => 'ቬሎ ያደረገች ሙሽራ',
    '🤰' => 'እርጉዝ ሴት',
    '🫃' => 'እርጉዝ ወንድ',
    '🫄' => 'እርጉዝ ሰው',
    '🤱' => 'ጡት ማጥባት',
    '👼' => 'ሕፃኑ መልዓክ',
    '🎅' => 'አባባ ገና',
    '🤶' => 'የገና እናት',
    '🦸' => 'ጀግና',
    '🦹' => 'የጀግና ምስል',
    '🧙' => 'አስማት',
    '🧚' => 'ጠንቋይ',
    '🧛' => 'ጥርስ ያለው ጭራቅ',
    '🧜' => 'ግማሽ ሰው ግማሽ ዓሳ',
    '🧝' => 'ኤልፍ',
    '🧞' => 'ጅኒ',
    '🧟' => 'ዞምቢ',
    '🧌' => 'አስቃያሚ ጭራቅ',
    '🫈' => 'ፀጉራማ ፍጥረት',
    '💆' => 'የፊት ማሳጅ',
    '💇' => 'ጸጉር መቆረጥ',
    '🚶' => 'እግረኛ',
    '🧍' => 'ሰው ቆሞ',
    '🧎' => 'ሰው ተንበርክኮ',
    '🏃' => 'ሯጭ',
    '💃' => 'ሴቶች ሲደንሱ',
    '🕺' => 'ሰው ሲደንስ',
    '🕴' => 'ሽክ ብሎ የለበሰ የሥራ ሰው',
    '👯' => 'ሰዎች ሲጨፍሩ',
    '🧖' => 'ሰው በእንፋሎት ክፍል ውስጥ',
    '🧗' => 'ሰው ተራራ ሲወጣ',
    '🤺' => 'ሻቦላ ተጫዋች',
    '🏇' => 'የፈረስ እሽቅድምድም',
    '⛷' => 'ስኪ ተጫዋች',
    '🏂' => 'በረዶ ሸርተቴ ተጫዋች',
    '🏌' => 'ጎልፍ ተጫዋች',
    '🏄' => 'የውሃ ሸርተቴ ተጫዋች',
    '🚣' => 'የታንኳ ቀዘፋ',
    '🏊' => 'ዋናተኛ',
    '⛹' => 'ኳስ የያዘ ሰው',
    '🏋' => 'ክብደት አንሺ',
    '🚴' => 'ብስክሌት ጋላቢ',
    '🚵' => 'የተራራ ብስክሌት ጋላቢ',
    '🤸' => 'አክሮባት',
    '🤼' => 'ነጻ ትግል',
    '🤽' => 'የውሃ ላይ ገና ጨዋታ',
    '🤾' => 'የእጅ ኳስ',
    '🤹' => 'ቅልልቦሽ',
    '🧘' => 'በሎታስ ኣቀማመጥ',
    '🛀' => 'ገላውን የሚታጠብ ሰው',
    '🛌' => 'አልጋ ላይ የተኛ ሰው',
    '👭' => 'እጅ ለእጅ የተያያዙ ሴቶች',
    '👫' => 'እጅ ለእጅ የተያያዙ ወንድ እና ሴት',
    '👬' => 'እጅ ለእጅ የተያያዙ ወንዶች',
    '💏' => 'መሳም',
    '💑' => 'ልብ በመሃላቸው ያሉ ጥንዶች',
    '🗣' => 'እየተናገረ ያለ ራስ',
    '👤' => 'የሰው ጥላ',
    '👥' => 'የሰው ጥላዎች',
    '🫂' => 'ሰዎች ሲተቃቀፉ',
    '👪' => 'ቤተሰብ',
    '👣' => 'የእግር ዱካ',
    '🫆' => 'የጣት አሻራ',
    '🦰' => 'ቀይ ጸጉር',
    '🦱' => 'የተጠቀለለ ጸጉር',
    '🦳' => 'ነጭ ጸጉር',
    '🦲' => 'መላጣ',
    '🐵' => 'የጦጣ ፊት',
    '🐒' => 'ጦጣ',
    '🦍' => 'ጉሬዛ',
    '🦧' => 'ኦራንጉታን ዝንጀሮ',
    '🐶' => 'የውሻ ፊት',
    '🐕' => 'ውሻ',
    '🦮' => 'የሚመራ ውሻ',
    '🐩' => 'ጸጉራም ፑድል ውሻ',
    '🐺' => 'የተኩላ ፊት',
    '🦊' => 'የቀበሮ ፊት',
    '🦝' => 'ራኩን',
    '🐱' => 'የድመት ፊት',
    '🐈' => 'ድመት',
    '🦁' => 'የአንበሳ ፊት',
    '🐯' => 'የነብር ፊት',
    '🐅' => 'ነብር',
    '🐆' => 'አነር',
    '🐴' => 'የፈረስ ፊት',
    '🫎' => 'ሙስ',
    '🫏' => 'አህያ',
    '🐎' => 'ፈረስ',
    '🦄' => 'ባለ አንድ ቀንድ ፈረስ ፊት',
    '🦓' => 'የሜዳ አህያ',
    '🦌' => 'አጋዘን',
    '🦬' => 'የሰሜን አሜሪካ ጎሽ',
    '🐮' => 'የላም ፊት',
    '🐂' => 'በሬ',
    '🐃' => 'የውሃ አውራሪስ',
    '🐄' => 'ላም',
    '🐷' => 'የዓሣማ ፊት',
    '🐖' => 'ዓሣማ',
    '🐗' => 'የጫካ ዓሣማ',
    '🐽' => 'የዓሣማ አፍንጫ',
    '🐏' => 'ሙክት በግ',
    '🐑' => 'በግ',
    '🐐' => 'ፍየል',
    '🐪' => 'ግመል',
    '🐫' => 'ባለሁለት ሻኛ ግመል',
    '🦙' => 'ላማ',
    '🦒' => 'ቀጭኔ',
    '🐘' => 'ዝሆን',
    '🦣' => 'የዝሆን ዝርያ',
    '🦏' => 'አውራሪስ',
    '🦛' => 'ጉማሬ',
    '🐭' => 'የአይጥ ፊት',
    '🐁' => 'አይጥ',
    '🐀' => 'አይጠሞጎጥ',
    '🐹' => 'ሚጢጢ አይጥ ፊት',
    '🐰' => 'የጥንቸል ፊት',
    '🐇' => 'ጥንቸል',
    '🐿' => 'ቺፕመንክ',
    '🦫' => 'የአይጥ ዝርያ',
    '🦔' => 'አጥቢ እንስሳ',
    '🦇' => 'የሌሊት ወፍ',
    '🐻' => 'የድብ ፊት',
    '🐨' => 'ካኦላ',
    '🐼' => 'የፓንዳ ፊት',
    '🦥' => 'ስንፍና',
    '🦦' => 'ትንሽ አውሬ',
    '🦨' => 'ሸለምጥማጥ',
    '🦘' => 'ካንጋሮ',
    '🦡' => 'ውሻ የሚመስል እንሰሳ',
    '🐾' => 'የእንሳሳ መዳፍ ዱካዎች',
    '🦃' => 'ቆቅ',
    '🐔' => 'ዶሮ',
    '🐓' => 'አውራ ዶሮ',
    '🐣' => 'ከእንቁላል ተፈልፍላ የምትወጣ ጫጩት',
    '🐤' => 'ሚጢጢ ጫጩት',
    '🐥' => 'ፊት ለፊት የምትታይ ሚጢጢ ጫጩት',
    '🐦' => 'ወፍ',
    '🐧' => 'ፔንግዊን',
    '🕊' => 'እርግብ',
    '🦅' => 'ንስር አሞራ',
    '🦆' => 'ዳክዬ',
    '🦢' => 'የውሀ ዶሮ',
    '🦉' => 'ጉጉት',
    '🦤' => 'ትልቅ ወፍ',
    '🪶' => 'ላባ',
    '🦩' => 'ፍላሚንጎ',
    '🦚' => 'ፒኮክ',
    '🦜' => 'በቀቀን',
    '🪽' => 'ክንፍ',
    '🪿' => 'ዝይ',
    '🐸' => 'የእንቁራሪት ፊት',
    '🐊' => 'አዞ',
    '🐢' => 'ኤሊ',
    '🦎' => 'እንሽላሊት',
    '🐍' => 'እባብ',
    '🐲' => 'የድራጎን ፊት',
    '🐉' => 'ድራጎን',
    '🦕' => 'ዳይኖሰር',
    '🦖' => 'ታይኖሳረስ ዳይኖሰር',
    '🐳' => 'ውሃ የሚያንቦጫርቅ ዓሣ ነባሪ',
    '🐋' => 'አሣ ነባሪ',
    '🐬' => 'ዶልፊን',
    '🫍' => 'ኦርካ',
    '🦭' => 'ምልክት',
    '🐟' => 'ዓሣ',
    '🐠' => 'የምድር ወገብ አካባቢ ዓሣ',
    '🐡' => 'ብሎውፊሽ',
    '🦈' => 'ሻርክ',
    '🐙' => 'ኦክቶፐስ',
    '🐚' => 'ሽብልል የቀንድ አውጣ ሼል',
    '🪸' => 'ቋጥኝ',
    '🪼' => 'ጄሊፊሽ',
    '🦀' => 'ሸርጣን',
    '🦞' => 'ሎብስተር',
    '🦐' => 'ትንሽ አሣ',
    '🦑' => 'ስክዊድ',
    '🦪' => 'ሼል',
    '🐌' => 'ቀንድ አውጣ',
    '🦋' => 'ቢራቢሮ',
    '🐛' => 'ትኋን',
    '🐜' => 'ጉንዳን',
    '🐝' => 'ንብ',
    '🪲' => 'ጥንዚዛ',
    '🐞' => 'ሴት ጢንዚዛ',
    '🦗' => 'ክሪኬት',
    '🪳' => 'በረሮ',
    '🕷' => 'እንሸረሪት',
    '🕸' => 'የእንሸረሪት ድር',
    '🦂' => 'ጊንጥ',
    '🦟' => 'ወባ',
    '🪰' => 'ዝንብ',
    '🪱' => 'ትል',
    '🦠' => 'ተህዋሳት',
    '💐' => 'እቅፍ አበባ',
    '🌸' => 'የሚፈካ ቼሪ አበባ',
    '💮' => 'ነጭ አበባ',
    '🪷' => 'ሎተስ',
    '🏵' => 'ሮዜት',
    '🌹' => 'ጽጌሬዳ',
    '🥀' => 'የጠወለገ አበባ',
    '🌺' => 'ሂቢስከስ አበባ',
    '🌻' => 'የሱፍ አበባ',
    '🌼' => 'የአበባ ፍካት',
    '🌷' => 'ቱሊፕ አበባ',
    '🪻' => 'ሀያሲንት',
    '🌱' => 'ችግኝ',
    '🪴' => 'በሸክላ የተተከለ አትክልት',
    '🌲' => 'ሁሌም አረንጓዴ',
    '🌳' => 'ባለጥላ ዛፍ',
    '🌴' => 'ዘንባባ',
    '🌵' => 'ቁልቋል',
    '🌾' => 'የሩዝ ተክል ዞላ',
    '🌿' => 'ቅጠል',
    '☘' => 'ሻምሮክ ቅጠል',
    '🍀' => 'ድንብላል',
    '🍁' => 'ማፕል ቅጠል',
    '🍂' => 'የረገፈ ቅጠል',
    '🍃' => 'በአየር ላይ የሚንሳፈፉ ቅጠሎች',
    '🪹' => 'ባዶ የወፍ ጎጆ',
    '🪺' => 'እንቁላል የያዘ ጎጆ',
    '🍄' => 'እንጉዳይ',
    '🪾' => 'ቅጠል የሌለው ዛፍ',
    '🍇' => 'የወይን ዛላዎች',
    '🍈' => 'ሆምጣጤ',
    '🍉' => 'ሐብሐብ',
    '🍊' => 'መንደሪን',
    '🍋' => 'ሎሚ',
    '🍌' => 'ሙዝ',
    '🍍' => 'አናናስ',
    '🥭' => 'ማንጎ',
    '🍎' => 'ቀይ አፕል',
    '🍏' => 'አረንጓዴ አፕል',
    '🍐' => 'ፖም',
    '🍑' => 'ኮክ',
    '🍒' => 'ቼሪ ፍሬ',
    '🍓' => 'እንጆሪ',
    '🫐' => 'ብሉቤሪስ',
    '🥝' => 'ኪዊ ፍራፍሬ',
    '🍅' => 'ቲማቲም',
    '🫒' => 'የወይራ ዘይት',
    '🥥' => 'ኮኮናት',
    '🥑' => 'አቦካዶ',
    '🍆' => 'ኤግፕላንት ፍሬ',
    '🥔' => 'ድንች',
    '🥕' => 'ካሮት',
    '🌽' => 'የበቆሎ ዛላ',
    '🌶' => 'የሚጥሚጣ ቃሪያ',
    '🫑' => 'የፈረንጅ ቃሪያ',
    '🥒' => 'ዱባ',
    '🥬' => 'ሰላጣ',
    '🥦' => 'ብሮኮሊ',
    '🧄' => 'ነጭ ሽንኩርት',
    '🧅' => 'ሽንኩርት',
    '🥜' => 'ኦቾሎኒ',
    '🫘' => 'ባቄላዎች',
    '🌰' => 'የለውዝ ፍሬ',
    '🫚' => 'የዝንጅብል ሥር',
    '🫛' => 'የአተር ፖድ',
    '🫜' => 'ሥር አትክልቶች',
    '🍞' => 'ዳቦ',
    '🥐' => 'ክሬሰንት',
    '🥖' => 'የፈረንሳይ ዳቦ',
    '🫓' => 'ሽልጦ',
    '🥨' => 'ደረቅ ብስኩት',
    '🥯' => 'ጣፋጭ ዳቦ',
    '🥞' => 'ፓንኬክ',
    '🧇' => 'ኬክ',
    '🧀' => 'የቺዝ ቁራጭ',
    '🍖' => 'ሥጋ ያለው አጥንት',
    '🍗' => 'የዶሮ እግር',
    '🥩' => 'ቁራጭ ስጋ',
    '🥓' => 'ቤከን',
    '🍔' => 'ሃምበርገር',
    '🍟' => 'ፍሬንች ፍራይስ',
    '🍕' => 'ፒዛ',
    '🌭' => 'ሃት ዶግ',
    '🥪' => 'ሳንድዊች',
    '🌮' => 'ታኮ',
    '🌯' => 'ቡሪቶ',
    '🫔' => 'ታማሌ',
    '🥙' => 'በምግብ የተሞላ ዳቦ',
    '🧆' => 'ብስኩት',
    '🥚' => 'እንቁላል',
    '🍳' => 'ምግብ ማብሰል',
    '🥘' => 'ጎድጓዳ ሳህን ለምግብ',
    '🍲' => 'የምግብ ሳሕን',
    '🫕' => 'ፎንዴ',
    '🥣' => 'ሳህን ሙሉ ማንኪያ',
    '🥗' => 'አረንጓዴ ሰላጣ',
    '🍿' => 'ፈንድሻ',
    '🧈' => 'ቅቤ',
    '🧂' => 'ጨው',
    '🥫' => 'የታሸገ ምግብ',
    '🍱' => 'ቤንቶ ቦክስ',
    '🍘' => 'የሩዝ ብስኩት',
    '🍙' => 'የሩዝ ኳስ',
    '🍚' => 'የተቀቀለ ሩዝ',
    '🍛' => 'ከሪ ሩዝ',
    '🍜' => 'ማጥለያ ጎድጓዳ ሳሕን',
    '🍝' => 'ስፓጌቲ',
    '🍠' => 'የተጠበሰ ስኳር ድንች',
    '🍢' => 'ኦዴን',
    '🍣' => 'ሱሺ',
    '🍤' => 'የተጠበሰ ሽሪምፕ ዓሣ',
    '🍥' => 'የዓሣ ኬክ ከተሸረካከፈ ክፈፍ ጋር',
    '🥮' => 'የጨረቃ ኬክ',
    '🍡' => 'ዳንጎ',
    '🥟' => 'ፍራፍሬ',
    '🥠' => 'ፎርቹን ኩኪ',
    '🥡' => 'ምግብ መውሰጃ እቃ',
    '🍦' => 'ቀላል አይስክሬም',
    '🍧' => 'የተሸለተ አይስክሬም',
    '🍨' => 'አይስክሬም',
    '🍩' => 'ዶናት',
    '🍪' => 'ኩኪስ',
    '🎂' => 'የልደት ኬክ',
    '🍰' => 'የልደት ኬክ ቁራሽ',
    '🧁' => 'የኩባያ ኬክ',
    '🥧' => 'ፓይ',
    '🍫' => 'ቸኮላታ',
    '🍬' => 'ከረሜላ',
    '🍭' => 'ሎሊፖፕ ከረሜላ',
    '🍮' => 'ካስታርድ ኬክ',
    '🍯' => 'የማር ማሰሮ',
    '🍼' => 'የህጻን ጡጦ',
    '🥛' => 'የወተት ብርጭቆ',
    '☕' => 'የሚጠጣ ትኩስ ነገር',
    '🫖' => 'የሻይ ስኒ',
    '🍵' => 'ማንጠልጠያ የሌለው የሻይ ስኒ',
    '🍶' => 'ማሰሮ',
    '🍾' => 'ጠርሙስ ከነመክፈቻው',
    '🍷' => 'የወይን ብርጭቆ',
    '🍸' => 'የኮክቴል ብርጭቆ',
    '🍹' => 'ትሮፒካል መጠጥ',
    '🍺' => 'የድራፍት ብርጭቆ',
    '🍻' => 'የሚጋጩ የድራፍት ብርጭቆዎች',
    '🥂' => 'የብርጭቆ ማንቃጨል',
    '🥃' => 'የመጠጥ ብርጭቆ',
    '🫗' => 'ፈሳሽ መድፋት',
    '🥤' => 'ብርጭቆ ከመጠጫ ጋር',
    '🧋' => 'የአረፋ ሻይ',
    '🧃' => 'የመጠጥ ሳጥን',
    '🧉' => 'ግጣም',
    '🧊' => 'በረዶ',
    '🥢' => 'ቾፕስቲክ',
    '🍽' => 'ማንኪያ እና ሹካ ከሳሕን ጋር',
    '🍴' => 'ሹካና ቢለዋ',
    '🥄' => 'ማንኪያ',
    '🔪' => 'የወጥ ቤት ቢላዋ',
    '🫙' => 'ብልቃጥ',
    '🏺' => 'አምፎራ ማሰሮ',
    '🌍' => 'አውሮፓና አፍሪካ የሚያሳይ ሉል',
    '🌎' => 'ሰሜንና ደቡብ አሜሪካ የሚያሳይ ሉል',
    '🌏' => 'እስያና አውስትራሊያ የሚያሳይ ሉል',
    '🌐' => 'ሜሪዲያኖች ያሉበት ሉል',
    '🗺' => 'የዓለም ካርታ',
    '🗾' => 'የጃፓን ካርታ',
    '🧭' => 'ኮምፓስ',
    '🏔' => 'ጫፉ በረዶ ያለበት ተራራ',
    '⛰' => 'ተራራ',
    '🛘' => 'የመሬት መንሸራተት',
    '🌋' => 'እሳተ ጎመራ',
    '🗻' => 'የፉጂ ተራራ',
    '🏕' => 'ሰፈራ',
    '🏖' => 'የባሕር ዳርቻ ከዣንጥላ ጋር',
    '🏜' => 'በርሃ',
    '🏝' => 'የበርሃ ደሴት',
    '🏞' => 'ብሔራዊ ፖርክ',
    '🏟' => 'ስታዲየም',
    '🏛' => 'ጥንታዊ ሕንጻ',
    '🏗' => 'የሕንጻ ግንባታ',
    '🧱' => 'ሸክላ',
    '🪨' => 'አለት',
    '🪵' => 'እንጨት',
    '🛖' => 'ጎጆ',
    '🏘' => 'የመኖሪያ ቤቶች ግንባታ',
    '🏚' => 'የተተወ የቤት ግንባታ',
    '🏠' => 'የቤት ግንባታ',
    '🏡' => 'መናፈሻ ቦታ ያለው ቤት',
    '🏢' => 'የቢሮ ሕንጻ',
    '🏣' => 'የጃፓን ፖስታ ቤት',
    '🏤' => 'ፖስታ ቤት',
    '🏥' => 'ሆስፒታል',
    '🏦' => 'ባንክ',
    '🏨' => 'ሆቴል',
    '🏩' => 'የፍቅር ሆቴል',
    '🏪' => '24 ሰዓት የሚሰራ ገበያ አዳራሽ',
    '🏫' => 'ትምህርት ቤት',
    '🏬' => 'መኖሪያ አፓርታማ',
    '🏭' => 'ፋብሪካ',
    '🏯' => 'የጃፓን ቤተመንግሥት ሕንጻ',
    '🏰' => 'የቤተመንግሥት ሕንጻ',
    '💒' => 'ሰርግ',
    '🗼' => 'የቶክዮ ማማ',
    '🗽' => 'የነጻነት ሐውልት',
    '⛪' => 'ቤተክርስትያን',
    '🕌' => 'መስጂድ',
    '🛕' => 'የሂንዱ ምኩራብ',
    '🕍' => 'ቤተ መቅደስ',
    '⛩' => 'የሺንቶ መካነ መቃብር',
    '🕋' => 'የሙስሊም ካባ',
    '⛲' => 'ፏፏቴ',
    '⛺' => 'ድንኳን',
    '🌁' => 'ጭጋጋማ',
    '🌃' => 'ከዋክብት ያሉበት ለሊት',
    '🏙' => 'የከተማ መልክዓ ምድር',
    '🌄' => 'በተራሮች አናት ላይ የምትወጣ ፀሐይ',
    '🌅' => 'የፀሐይ ውጣት',
    '🌆' => 'የከተማ መልክዓ ምድር በምሽት',
    '🌇' => 'የፀሐይ ግባት',
    '🌉' => 'ድልድይ በለሊት',
    '♨' => 'ትኩስ ፍልውሃ',
    '🎠' => 'ካሮሴል ፈረስ',
    '🛝' => 'መጫወቻ ሸርተቴ',
    '🎡' => 'ፌሪስ ዊል እሽክርክሮሽ',
    '🎢' => 'ሮለር ኮስተር እሽክርክሮሽ',
    '💈' => 'የጸጉር አስተካካይ ስልክ እንጨት',
    '🎪' => 'የሰርከስ ድንኳን',
    '🚂' => 'የባቡር ፉርጎ ጎታች',
    '🚃' => 'ፉርጎ',
    '🚄' => 'ፈጣን ባቡር',
    '🚅' => 'አፍንጫው ጥይት መሳይ ፈጣን ባቡር',
    '🚆' => 'ባቡር',
    '🚇' => 'የመሬት ውስጥ ባቡር',
    '🚈' => 'ቀላል ባቡር',
    '🚉' => 'ጣቢያ',
    '🚊' => 'ባቡር መሰል አውቶቡስ',
    '🚝' => 'የባቡር መሄጃ ድልድይ',
    '🚞' => 'የባቡር ሐዲድ ያለው ተራራ',
    '🚋' => 'የባቡር መሰል አውቶቡስ ፉርጎ',
    '🚌' => 'አውቶቡስ',
    '🚍' => 'መጪ አውቶቡስ',
    '🚎' => 'በኤሌትሪክ የሚጎተት አውቶቡስ',
    '🚐' => 'ሚኒባስ',
    '🚑' => 'አምቡላንስ',
    '🚒' => 'የእሳት አደጋ መኪና',
    '🚓' => 'የፖሊስ መኪና',
    '🚔' => 'መጪ የፖሊስ መኪና',
    '🚕' => 'ታክሲ',
    '🚖' => 'መጪ ታክሲ',
    '🚗' => 'አውቶሞቢል',
    '🚘' => 'መጪ አውቶሞቢል',
    '🚙' => 'የመዝናኛ መኪና',
    '🛻' => 'የመሰብሰቢያ መኪና',
    '🚚' => 'የዕቃ ማድረሻ መኪና',
    '🚛' => 'ሽፍን የጭነት ማመላለሻ መኪና',
    '🚜' => 'ትራክተር',
    '🏎' => 'የውድድር መኪና',
    '🏍' => 'ሞተርሳይክል',
    '🛵' => 'ቀላል ሞተር ሳይክል',
    '🦽' => 'የእጅ ተሽከርካሪ ወንበር',
    '🦼' => 'ባለሞተር ተሽከርካሪ ወንበር',
    '🛺' => 'ራስሰር ሪክሻው',
    '🚲' => 'ብስክሌት',
    '🛴' => 'የግፊ ሳይክል',
    '🛹' => 'ስኬትቦርድ',
    '🛼' => 'ተሽከርካሪ ስኬት',
    '🚏' => 'የአውቶቡስ ማቆሚያ',
    '🛣' => 'የመኪና መንገድ',
    '🛤' => 'የባቡር ሐዲድ',
    '🛢' => 'የዘይት ከበሮ',
    '⛽' => 'የነዳጅ መቅጃ',
    '🛞' => 'ጎማ',
    '🚨' => 'የፖሊስ መኪና መብራት',
    '🚥' => 'አግድማዊ የትራፊክ መብራት',
    '🚦' => 'ተቆልቋይ የትራፊክ መብራት',
    '🛑' => 'የቁም ምልክት',
    '🚧' => 'ግንባታ',
    '⚓' => 'መልህቅ',
    '🛟' => 'አንሳፋፊ ቀለበት',
    '⛵' => 'ጀልባ',
    '🛶' => 'ታንኳ',
    '🚤' => 'ፈጣን ጀልባ',
    '🛳' => 'የመንገደኞች መርከብ',
    '⛴' => 'ከባድ የመንገደኞች መርከብ',
    '🛥' => 'ባለሞተር ጀልባ',
    '🚢' => 'መርከብ',
    '✈' => 'አይሮፕላን',
    '🛩' => 'ትንሽ አይሮፕላን',
    '🛫' => 'የአይሮፕላን አነሳስ',
    '🛬' => 'የአይሮፕላን አስተራረፍ',
    '🪂' => 'ጃንጥላ',
    '💺' => 'ወንበር',
    '🚁' => 'ሄሊኮፕተር',
    '🚟' => 'በአየር ላይ የሚጎተት ባቡር',
    '🚠' => 'የተራራ የባቡር መሳቢያ ገመድ',
    '🚡' => 'በአየር ላይ የሚጎተት ባቡር መሰል አውቶቡስ',
    '🛰' => 'ሳተላይት',
    '🚀' => 'ሮኬት',
    '🛸' => 'በራሪ ዲስክ',
    '🛎' => 'የሆቴል ሻንጣ ተቀባይ ደወል',
    '🧳' => 'ሻንጣ',
    '⌛' => 'የሰዓት ቆጣሪ ብርጭቆ',
    '⏳' => 'የሰዓት ቆጣሪ ብርጭቆ ከፈሳሽ አሸዋ ጋር',
    '⌚' => 'የእጅ ሰዓት',
    '⏰' => 'ማንቂያ ደወል ያለው ሰዓት',
    '⏱' => 'የሩጫ ሰዓት',
    '⏲' => 'ጊዜ ቆጣሪ ሰዓት',
    '🕰' => 'የብፌ ሰዓት',
    '🕛' => 'አስራ ሁለት ሰዓት',
    '🕧' => 'አስራ ሁለት ተኩል',
    '🕐' => 'አንድ ሰዓት',
    '🕜' => 'አንድ ሰዓት ተኩል',
    '🕑' => 'ሁለት ሰዓት',
    '🕝' => 'ሁለት ሰዓት ተኩል',
    '🕒' => 'ሶስት ሰዓት',
    '🕞' => 'ሶስት ሰዓት ተኩል',
    '🕓' => 'አራት ሰዓት',
    '🕟' => 'አራት ሰዓት ተኩል',
    '🕔' => 'አምስት ሰዓት',
    '🕠' => 'አምስት ሰዓት ተኩል',
    '🕕' => 'ስድስት ሰዓት',
    '🕡' => 'ስድስት ሰዓት ተኩል',
    '🕖' => 'ሰባት ሰዓት',
    '🕢' => 'ሰባት ሰዓት ተኩል',
    '🕗' => 'ስምንት ሰዓት',
    '🕣' => 'ስምንት ሰዓት ተኩል',
    '🕘' => 'ዘጠኝ ሰዓት',
    '🕤' => 'ዘጠኝ ሰዓት ተኩል',
    '🕙' => 'አስር ሰዓት',
    '🕥' => 'አስር ሰዓት ተኩል',
    '🕚' => 'አስራ አንድ ሰዓት',
    '🕦' => 'አስራ አንድ ሰዓት ተኩል',
    '🌑' => 'አዲስ ጨረቃ',
    '🌒' => 'ትንሽ የበራች ግማሽ ጨረቃ',
    '🌓' => 'የመጀመሪያ ሩብ ጨረቃ',
    '🌔' => 'ትንሽ የጠቆረች ግማሽ ጨረቃ',
    '🌕' => 'ሙሉ ጨረቃ',
    '🌖' => 'በቀኝ በኩል ትንሽ የጠቆረች ግማሽ ጨረቃ',
    '🌗' => 'የመጨረሻ ሩብ ጨረቃ',
    '🌘' => 'በግራ በኩል ትንሽ የበራች ግማሽ ጨረቃ',
    '🌙' => 'ግማሽ ጨረቃ',
    '🌚' => 'አዲስ ጨረቃ ፊት',
    '🌛' => 'የመጀመሪያ ሩብ ጨረቃ ከፊት ጋር',
    '🌜' => 'የመጨረሻ ሩብ ጨረቃ ከፊት ጋር',
    '🌡' => 'ቴርሞሜትር',
    '☀' => 'ፀሐይ',
    '🌝' => 'ሙሉ ጨረቃ ከፊት ጋር',
    '🌞' => 'ፀሐይ ከፊት ጋር',
    '🪐' => 'ባለቀለበት ፕላኔት',
    '⭐' => 'ነጭ መካከለኛ ኮከብ',
    '🌟' => 'ብልጭ ድርግም የሚል ኮከብ',
    '🌠' => 'ተወርዋሪ ኮከብ',
    '🌌' => 'ፍኖተ ሐሊብ',
    '☁' => 'ደመና',
    '⛅' => 'ከደመና ጀርባ ያለች ፀሐይ',
    '⛈' => 'ከመብረቅ እና ዝናብ ጋር ያለ ደመና',
    '🌤' => 'ከትንሽ ደመና ጀርባ ያለች ፀሐይ',
    '🌥' => 'ከትልቅ ደመና ጀርባ ያለች ፀሐይ',
    '🌦' => 'ዝናብ ካለው ደመና ጀርባ ያለች ፀሐይ',
    '🌧' => 'ደመና ከዝናብ ጋር',
    '🌨' => 'ደመና ከበረዶ ጋር',
    '🌩' => 'ደመና ከመብረቅ ጋር',
    '🌪' => 'ቶርኔዶ',
    '🌫' => 'ጉም',
    '🌬' => 'የነፋስ ፊት',
    '🌀' => 'ሳይክሎን',
    '🌈' => 'ቀስተ ደመና',
    '🌂' => 'የታጠፈ ዣንጥላ',
    '☂' => 'ዣንጥላ',
    '☔' => 'የዝናብ ጠብታዎች ያሉበት ዣንጥላ',
    '⛱' => 'የመሬት ዣንጥላ',
    '⚡' => 'ከፍተኛ ቮልቴጅ',
    '❄' => 'ስኖውፍሌክ',
    '☃' => 'በበረዶ ግግር የተሠራ ሰው',
    '⛄' => 'በበረዶ ግግር የተሠራ ሰው ያለበረዶ',
    '☄' => 'ኮሜት',
    '🔥' => 'እሳት',
    '💧' => 'ጠብታ',
    '🌊' => 'የውሃ ሞገድ',
    '🎃' => 'ጆክ-ኦ-ላንተርን',
    '🎄' => 'የገና ዛፍ',
    '🎆' => 'ርችት',
    '🎇' => 'የእሳት ፍንጣሪ መሥሪያ',
    '🧨' => 'ርችቶች',
    '✨' => 'የእሳት ፍንጣሪዎች',
    '🎈' => 'አፉፋ',
    '🎉' => 'የድግስ ኮፍያ',
    '🎊' => 'የከረሜላ ኳስ',
    '🎋' => 'ታንባታ ዛፍ',
    '🎍' => 'የጽድ ጌጥ',
    '🎎' => 'የጃፓን አሻንጉሊቶች',
    '🎏' => 'ካርፕ ተወርዋሪ ዓሣዎች',
    '🎐' => 'የነፋስ ቺም',
    '🎑' => 'የጨረቃ በዓል',
    '🧧' => 'ቀይ ፖስታ',
    '🎀' => 'ሪባን',
    '🎁' => 'የተጠቀለለ ስጦታ',
    '🎗' => 'የመታሰቢያ ሪባን',
    '🎟' => 'የመግቢያ ትኬቶች',
    '🎫' => 'ትኬት',
    '🎖' => 'ወታደራዊ ሜዳይ',
    '🏆' => 'ዋንጫ',
    '🏅' => 'የስፖርት ሜዳሊያ',
    '🥇' => 'የ1ኛ ስፍራ ሜዳሊያ',
    '🥈' => 'የ2ኛ ስፍራ ሜዳሊያ',
    '🥉' => 'የ3ኛ ስፍራ ሜዳሊያ',
    '⚽' => 'የእግር ኳስ',
    '⚾' => 'ቤዝቦል',
    '🥎' => 'ስስ ኳስ',
    '🏀' => 'የቅርጫት ኳስ',
    '🏐' => 'የመረብ ኳስ',
    '🏈' => 'የአሜሪካ እግር ኳስ',
    '🏉' => 'የራግቢ እግር ኳስ',
    '🎾' => 'ቴኒስ',
    '🥏' => 'የሚበር ዲስክ',
    '🎳' => 'ቦውሊንግ',
    '🏏' => 'የክሪኬት ጨዋታ',
    '🏑' => 'የሜዳ ሆኪ',
    '🏒' => 'የበረዶ ሆኪ ዘንግ እና ፑክ',
    '🥍' => 'ላክሮስ',
    '🏓' => 'ጠረጴዛ ቴኒስ',
    '🏸' => 'ባድሜንተን',
    '🥊' => 'የቦክስ ጓንት',
    '🥋' => 'የማርሻል አርት ልብስ',
    '🥅' => 'የጎል መረብ',
    '⛳' => 'በቀዳዳ የተሰካ ሰንደቅ ዓላማ',
    '⛸' => 'የበረዶ መንሸራተት',
    '🎣' => 'የዓሣ ማጥመጃ ዘንግ',
    '🤿' => 'የመጥለቂያ ጭምብል',
    '🎽' => 'መሮጫ ቀሚስ',
    '🎿' => 'ስኪ',
    '🛷' => 'የበረዶ መንሸራተቻ',
    '🥌' => 'የተጠረበ ድንጋይ',
    '🎯' => 'ቀጥታ የዳርት ውርወራ ውጤት',
    '🪀' => 'በከራ',
    '🪁' => 'ካይት',
    '🔫' => 'ሽጉጥ',
    '🎱' => 'የከረንቦላ ድንጋይ',
    '🔮' => 'ክሪስታል ኳስ',
    '🪄' => 'ምትሃተኛ ዘንግ',
    '🎮' => 'የቪዲዮ ጨዋታ',
    '🕹' => 'ጆይስቲክ',
    '🎰' => 'በሳንቲም የሚሠራ ማሽን',
    '🎲' => 'የጨዋታ ዳይ',
    '🧩' => 'እንቆቅልሽ',
    '🧸' => 'ቴዲ ቢር',
    '🪅' => 'ፒናታ',
    '🪩' => 'የመስታወት ኳስ',
    '🪆' => 'የመረብ አሻንጉሊቶች',
    '♠' => 'የካርታ ጦር',
    '♥' => 'የካርታ ልብ',
    '♦' => 'የካርታ አልማዝ',
    '♣' => 'የካርታ አበባ',
    '♟' => 'የቼዝ መጫወቻ',
    '🃏' => 'የካርታ ጆከር',
    '🀄' => 'ማህጆንግ ቀይ ድራጎን',
    '🎴' => 'ጀርባው አበባ መጫወቻ ካርታ',
    '🎭' => 'ሙዚቃና ትያትር',
    '🖼' => 'ባለክፈፍ ሥዕል',
    '🎨' => 'የሠዓሊ ሳሕን',
    '🧵' => 'ክር',
    '🪡' => 'የመስፊያ መርፌ',
    '🧶' => 'የስፌት ክር',
    '🪢' => 'ቋጠሮ',
    '👓' => 'መነጽር',
    '🕶' => 'የፀሐይ መነጽር',
    '🥽' => 'የመዋኛ መነጽር',
    '🥼' => 'የቤተ ሙከራ ቤት',
    '🦺' => 'የአደጋ መከላከያ ልብስ',
    '👔' => 'ከረባት',
    '👕' => 'ቲሸርት',
    '👖' => 'ጂንስ',
    '🧣' => 'ስካርፍ',
    '🧤' => 'ጓንት',
    '🧥' => 'ኮት',
    '🧦' => 'ካልሲ',
    '👗' => 'ቀሚስ',
    '👘' => 'የጃፓን ቀሚስ',
    '🥻' => 'ሴሪ',
    '🩱' => 'ወጥ የዋና ልብስ',
    '🩲' => 'አጭር መግለጫ',
    '🩳' => 'ቁምጣ',
    '👙' => 'ቢኪኒ',
    '👚' => 'የሴት ልብስ',
    '🪭' => 'ተጣጣፊ የእጅ ማቀዝቀዣ',
    '👛' => 'የሴት የእጅ ቦርሳ',
    '👜' => 'የእጅ ቦርሳ',
    '👝' => 'የሴት ቦርሳ',
    '🛍' => 'የመገበያያ ቦርሳዎች',
    '🎒' => 'የትምህርት ቤት ቦርሳ',
    '🩴' => 'የነጠላ ጫማ ማሰሪያ',
    '👞' => 'የወንድ ጫማ',
    '👟' => 'የሩጫ ጫማ',
    '🥾' => 'የእግር ሽርሽር ጫማ',
    '🥿' => 'ጠፍጣፋ ጫማ',
    '👠' => 'ባለ ተረከዝ የሴት ጫማ',
    '👡' => 'የሴት ሰንደል ጫማ',
    '🩰' => 'የባሌ ዳንስ ጫማ',
    '👢' => 'የሴት ቡትስ ጫማ',
    '🪮' => 'ጸጉር ማንሻ',
    '👑' => 'ዘውድ',
    '👒' => 'የሴት ባርኔጣ',
    '🎩' => 'የወንድ የሙሉ ልብስ ባርኔጣ',
    '🎓' => 'የምርቃት ኮፍያ',
    '🧢' => 'የተከፈለበት ባርኔጣ',
    '🪖' => 'የወታደር ቆብ',
    '⛑' => 'ነጭ መስቀል ያለበት ብረት ኮፍያ',
    '📿' => 'መቁጸሪያ',
    '💄' => 'የከንፈር ቀለም',
    '💍' => 'የጣት ቀለበት',
    '💎' => 'አልማዝ',
    '🔇' => 'ስፒከር ጠፍቷል',
    '🔈' => 'ስፒከር',
    '🔉' => 'ስፒከር በርቷል',
    '🔊' => 'ስፒከር ድምፁ ከፍ ብሏል',
    '📢' => 'ላውድ ስፒከር',
    '📣' => 'ሜጋፎን',
    '📯' => 'የፖስተኛ ጥሩንባ',
    '🔔' => 'ደውል',
    '🔕' => 'የተሰረዘ ደወል',
    '🎼' => 'ሙዚቃዊ ውጤት',
    '🎵' => 'ሙዚቃዊ ኖታ',
    '🎶' => 'ሙዚቃዊ ኖታዎች',
    '🎙' => 'የስቲዲዮ ማይክራፎን',
    '🎚' => 'ማመጣጠኛ አንሸራታች',
    '🎛' => 'መቆጣጠሪያ ቁልፎች',
    '🎤' => 'ማይክራፎን',
    '🎧' => 'የጆሮ ማዳመጫ',
    '📻' => 'ሬዲዮ',
    '🎷' => 'ሳክስፎን',
    '🎺' => 'ትራምፔት',
    '🪊' => 'ትሮምቦን',
    '🪗' => 'አኮርዲዮን',
    '🎸' => 'ጊታር',
    '🎹' => 'የሙዚቃ ቁልፍ ሰሌዳ',
    '🎻' => 'ቫዮሊን',
    '🪕' => 'ባንጆ',
    '🥁' => 'ከበሮ',
    '🪘' => 'ረጅም ከበሮ',
    '🪇' => 'ማራካ',
    '🪈' => 'ዋሽንት',
    '🪉' => 'በገና',
    '📱' => 'ሞባይል ስልክ',
    '📲' => 'ሞባይል ስልክ ከቀስት ጋር',
    '☎' => 'ቴሌፎን',
    '📞' => 'የቴሌፎን መነጋገሪያ',
    '📟' => 'ወረቀት',
    '📠' => 'ፋክስ ማሽን',
    '🔋' => 'ባትሪ',
    '🪫' => 'አነስተኛ ባትሪ',
    '🔌' => 'የኤሌክትሪክ ሶኬት',
    '💻' => 'ላፕቶፕ ኮምፒውተር',
    '🖥' => 'ዴስክቶፕ ኮምፒውተር',
    '🖨' => 'ማተሚያ',
    '⌨' => 'የቁልፍ ሰሌዳ',
    '🖱' => 'የኮምፒውተር መዳፊት',
    '🖲' => 'ትራክቦል',
    '💽' => 'ሚኒዲስክ',
    '💾' => 'ፍሎፒ ዲስክ',
    '💿' => 'ኦፕቲካል ዲስክ',
    '📀' => 'ዲቪዲ',
    '🧮' => 'አባከስ',
    '🎥' => 'የፊልም ካሜራ',
    '🎞' => 'የፊልም ፍሬሞች',
    '📽' => 'የፊልም ፕሮጄክተር',
    '🎬' => 'የፊልም ቀረጻ መቀንጠቢያ ሰሌዳ',
    '📺' => 'ቴሌቪዥን',
    '📷' => 'ካሜራ',
    '📸' => 'ባለፍላሽ ካሜራ',
    '📹' => 'የቪዲዮ ካሜራ',
    '📼' => 'የቪዲዮ ካሴት',
    '🔍' => 'ግራ ጠቋሚ ማጉያ መነጽር',
    '🔎' => 'ቀኝ ጠቋሚ ማጉያ መነጽር',
    '🕯' => 'ሻማ',
    '💡' => 'አምፖል',
    '🔦' => 'የእጅ ባትሪ',
    '🏮' => 'ሬድ ፔፐር መብራት',
    '🪔' => 'ኩራዝ',
    '📔' => 'ባለጌጥ ሽፋን ማስታወሻ ደብተር',
    '📕' => 'የተዘጋ መጽሐፍ',
    '📖' => 'ክፍት መጽሐፍ',
    '📗' => 'አረንጓዴ መጽሐፍ',
    '📘' => 'ሰማያዊ መጽሐፍ',
    '📙' => 'ብርትኳናማ መጽሐፍ',
    '📚' => 'መጽሐፍት',
    '📓' => 'ማስታወሻ ደብተር',
    '📒' => 'መዝገብ',
    '📃' => 'ሸብለል ያለ ወረቀት',
    '📜' => 'ሽብልል',
    '📄' => 'ወደ ላይ የተቀመጠ ወረቀት',
    '📰' => 'ጋዜጣ',
    '🗞' => 'የተጠቀለለ ጋዜጣ',
    '📑' => 'እልባት ማድረጊያ ትሮች',
    '🔖' => 'እልባት',
    '🏷' => 'መለያ',
    '🪙' => 'ሳንቲም',
    '💰' => 'የገንዘብ ከረጢት',
    '🪎' => 'ግምጃ ቤት',
    '💴' => 'የን የወረቀት ገንዘብ',
    '💵' => 'ዶላር የወረቀት ገንዘብ',
    '💶' => 'ዩሮ የወረቀት ገንዘብ',
    '💷' => 'ፓውንድ የወረቀት ገንዘብ',
    '💸' => 'ክንፍ ያለው ገንዘብ',
    '💳' => 'ክሬዲት ካርድ',
    '🧾' => 'ደረሰኝ',
    '💹' => 'ከየን ጋር የሚጨምር ሰንጠረዥ',
    '✉' => 'ኤንቨሎፕ',
    '📧' => 'ኢሜይል',
    '📨' => 'ገቢ ኤንቨሎፕ',
    '📩' => 'ባለ ቀስት ኤንቨሎፕ',
    '📤' => 'የወጪ ኤንቨሎፕ ማስቀመጫ ትሪ',
    '📥' => 'የገቢ ኤንቨሎፕ ማስቀመጫ ትሪ',
    '📦' => 'ጥቅል',
    '📫' => 'ባንዲራ የተሰቀለበት የተዘጋ የፖስታ ሳጥን',
    '📪' => 'ባንዲራ የወረደበት የተዘጋ የፖስታ ሳጥን',
    '📬' => 'ባንዲራ የተሰቀለበት ክፍት የፖስታ ሳጥን',
    '📭' => 'ባንዲራ የወረደበት ክፍት የፖስታ ሳጥን',
    '📮' => 'ፖስታ ሳጥን',
    '🗳' => 'የድምፅ መስጫ ሳጥን ከድምፅ መስጫ ካርድ ጋር',
    '✏' => 'እርሳስ',
    '✒' => 'ጥቁር የብዕር ጫፍ',
    '🖋' => 'ፋውንቴን ፔን',
    '🖊' => 'እስኪብርቶ',
    '🖌' => 'የሠዓሊ ብሩሽ',
    '🖍' => 'የመጻፊያ ከለር',
    '📝' => 'ማስታወሻ',
    '💼' => 'ቦርሳ',
    '📁' => 'የፋይል አቃፊ',
    '📂' => 'ክፍት የፋይል አቃፊ',
    '🗂' => 'ካርድ መጥቁም ከፋፋዮች',
    '📅' => 'ቀን መቁጠሪያ',
    '📆' => 'ተቀዳጅ ቀን መቁጠሪያ',
    '🗒' => 'ባለሽቦ ማስታወሻ ደብተር',
    '🗓' => 'ባለሽቦ ቀን መቁጠሪያ',
    '📇' => 'ካርድ መጥቁም',
    '📈' => 'የሚጨምር ገበታ',
    '📉' => 'የሚቀንስ ገበታ',
    '📊' => 'ባለ አሞሌ ገበታ',
    '📋' => 'ወረቀት ማስደገፊያ',
    '📌' => 'ፑሽ እስፒል',
    '📍' => 'ክብ ፑሽ እስፒል',
    '📎' => 'ወረቀት ማያያዣ',
    '🖇' => 'የተያያዙ የወረቀት ክሊፖች',
    '📏' => 'ቀጥ ያለ ማስመሪያ',
    '📐' => 'ባለ ሦስት ማዕዘን ማስመሪያ',
    '✂' => 'መቀስ',
    '🗃' => 'የካርድ ፋይል ሳጥን',
    '🗄' => 'ፋይል ካቢኔት',
    '🗑' => 'የቆሻሻ መጣያ',
    '🔒' => 'መሸጎጫ',
    '🔓' => 'ክፍት መሸጎጫ',
    '🔏' => 'መሸጎጫ ከብዕር ጋር',
    '🔐' => 'በቁልፍ የተዘጋ መሸጎጫ',
    '🔑' => 'ቁልፍ',
    '🗝' => 'አሮጌ ቁልፍ',
    '🔨' => 'መዶሻ',
    '🪓' => 'መጥረቢያ',
    '⛏' => 'አነስተኛ ዶማ',
    '⚒' => 'መዶሻ እና አነስተኛ ዶማ',
    '🛠' => 'መዶሻ እና መፍቻ',
    '🗡' => 'አነስተኛ ሰይፍ',
    '⚔' => 'የተጠላለፉ ሰይፎች',
    '💣' => 'ቦምብ',
    '🪃' => 'ቡምራንግ',
    '🏹' => 'ደጋን እና ቀስት',
    '🛡' => 'ጋሻ',
    '🪚' => 'የአናጢ መጋዝ',
    '🔧' => 'መፍቻ',
    '🪛' => 'መፍቻ መሳሪያ',
    '🔩' => 'ዳዶ እና ማፈኛ',
    '⚙' => 'ሞተር ጥርስ',
    '🗜' => 'መጨፍለቂያ',
    '⚖' => 'ሚዛን',
    '🦯' => 'ከዘራ',
    '🔗' => 'ማገናኛ',
    '⛓' => 'ሰንሰለቶች',
    '🪝' => 'መንጠቆ',
    '🧰' => 'የመፍቻ ሳጥን',
    '🧲' => 'ማግኔት',
    '🪜' => 'መሰላል',
    '🪏' => 'አካፋ',
    '⚗' => 'አለምቢክ',
    '🧪' => 'መፈተኛ ቱቦ',
    '🧫' => 'የፔትሪ ዲሽ',
    '🧬' => 'ዲኤንኤ',
    '🔬' => 'ማይክሮስኮፕ',
    '🔭' => 'ቴሌስኮፕ',
    '📡' => 'ሳተላይት አንቴና',
    '💉' => 'ስሪንጅ',
    '🩸' => 'የደም ጠብታ',
    '💊' => 'ክኒን',
    '🩹' => 'ፋሻ ማጣበቂያ',
    '🩼' => 'ክራንች',
    '🩺' => 'ማዳመጫ',
    '🩻' => 'ኤክስ-ሬይ',
    '🚪' => 'በር',
    '🛗' => 'አሳንሱር',
    '🪞' => 'መስታወት',
    '🪟' => 'መስኮት',
    '🛏' => 'አልጋ',
    '🛋' => 'ሶፋ እና መብራት',
    '🪑' => 'መቀመጫ',
    '🚽' => 'ሽንት ቤት',
    '🪠' => 'መወርወሪያ',
    '🚿' => 'ሻወር',
    '🛁' => 'የገላ መታጠቢያ ገንዳ',
    '🪤' => 'የአይጥ ማጥመጃ',
    '🪒' => 'ምላጭ',
    '🧴' => 'የቅባት ጠርሙስ',
    '🧷' => 'መርፌ ቁልፍ',
    '🧹' => 'መጥረጊያ',
    '🧺' => 'ቅርጫት',
    '🧻' => 'የወረቀት ጥቅል',
    '🪣' => 'ባልዲ',
    '🧼' => 'ሳሙና',
    '🫧' => 'ሽፋኖች',
    '🪥' => 'የጥርስ ብሩሽ',
    '🧽' => 'ስፖንጅ',
    '🧯' => 'እሳት ማጥፊያ',
    '🛒' => 'የመገብያ ጋሪ',
    '🚬' => 'ሲጋራ',
    '⚰' => 'ሬሳ ሣጥን',
    '🪦' => 'ሃውልት',
    '⚱' => 'የቀብር ማሰሮ',
    '🧿' => 'ናዛር አምዩሌት',
    '🪬' => 'ሐመሳ',
    '🗿' => 'ማኦዬ ፊት',
    '🪧' => 'ማስታወቂያ',
    '🪪' => 'መታወቂያ ካርድ',
    '🏧' => 'የኤቲኤም ምልክት',
    '🚮' => 'በቆሻሻ መጣያ ጣሉ ምልክት',
    '🚰' => 'የሚጠጣ ውሃ',
    '♿' => 'የአካል ጉዳተኛ ዊልቸር',
    '🚹' => 'የወንድ መጸዳጃ',
    '🚺' => 'የሴት መጸዳጃ',
    '🚻' => 'መጸዳጃ ቤት',
    '🚼' => 'የሕፃን ምልክት',
    '🚾' => 'ባኞ ቤት',
    '🛂' => 'የፖስፖርት መቆጣጠሪያ',
    '🛃' => 'ጉምሩክ',
    '🛄' => 'ሻንጣ መረከቢያ',
    '🛅' => 'የተረሳ ሻንጣ',
    '⚠' => 'ማስጠንቀቂያ',
    '🚸' => 'ሕፃናት መንገድ ያቋርጣሉ',
    '⛔' => 'መግባት አይቻልም',
    '🚫' => 'የተከለከለ',
    '🚳' => 'ብስክሌት አይፈቀድም',
    '🚭' => 'ማጨስ ክልክል ነው',
    '🚯' => 'ቆሻሻ መጣል ክልክል ነው',
    '🚱' => 'የማይጠጣ ውሃ',
    '🚷' => 'ለእግረኞች አይፈቀድም',
    '📵' => 'ሞባይል ስልኮች አይፈቀዱም',
    '🔞' => 'ከአስራ ስምንት ዓመት በታ አይፈቀድም',
    '☢' => 'ራዲዮአክቲቭ',
    '☣' => 'የባዮሎጂካል አደገኛ ምልክት',
    '⬆' => 'ወደ ላይ ጠቋሚ ቀስት',
    '↗' => 'ወደ ላይ ቀኝ ጠቋሚ ቀስት',
    '➡' => 'ወደ ቀኝ ጠቋሚ ቀስት',
    '↘' => 'ወደ ታች ቀኝ ጠቋሚ ቀስት',
    '⬇' => 'ወደ ታች ጠቋሚ ቀስት',
    '↙' => 'ወደ ታች ግራ ጠቋሚ ቀስት',
    '⬅' => 'ወደ ግራ ጠቋሚ ቀስት',
    '↖' => 'ወደ ላይ ግራ ጠቋሚ ቀስት',
    '↕' => 'ወደ ላይ ታች ጠቋሚ ቀስት',
    '↔' => 'ግራ-ቀኝ ጠቋሚ ቀስት',
    '↩' => 'ወደ ግራ ታጣፊ ቀኝ ጠቋሚ ቀስት',
    '↪' => 'ወደ ቀኝ ታጣፊ ግራ ጠቋሚ ቀስት',
    '⤴' => 'ወደ ላይ ታጣፊ ቀኝ ጠቋሚ ቀስት',
    '⤵' => 'ወደ ታች ታጣፊ ቀኝ ጠቋሚ ቀስት',
    '🔃' => 'በሰዓት አዟዟር አቅጣጫ ያሉ ቀጥ ያሉ ቀስቶች',
    '🔄' => 'በሰዓት አዟዟር አቅጣጫ ተቃራኒ ያሉ ቀስቶች አዝራር',
    '🔙' => 'ተመለስ ቀስት',
    '🔚' => 'ማብቂያ ቀስት',
    '🔛' => 'በርቷል! ቀስት',
    '🔜' => 'በቅርቡ ይመጣል ቀስት',
    '🔝' => 'ከፍተኛ ጠቋሚ ቀስት',
    '🛐' => 'የአምልኮ ቦታ',
    '⚛' => 'የአቶም ምልክት',
    '🕉' => 'ኦኤም',
    '✡' => 'የዳዊት ኮከብ',
    '☸' => 'የዳርማ መዘውር',
    '☯' => 'ዩን ያንግ',
    '✝' => 'የላቲን መስቀል',
    '☦' => 'የኦርቶዶክስ መስቀል',
    '☪' => 'ኮከብ እና ግማሽ ጨረቃ',
    '☮' => 'የሰላም ምልክት',
    '🕎' => 'ሜኖራሃ',
    '🔯' => 'ባለነጥብ ስድስት አንጓ ኮከብ',
    '🪯' => 'ቻንዳ',
    '♈' => 'ኤሪስ',
    '♉' => 'ታውረስ',
    '♊' => 'ጄሚኒ',
    '♋' => 'ካንሰር',
    '♌' => 'ሊዮ',
    '♍' => 'ቪርጎ',
    '♎' => 'ሊብራ',
    '♏' => 'ስኮርፒዮ',
    '♐' => 'ሳጂታሪየስ',
    '♑' => 'ካፕሪኮርን',
    '♒' => 'አኳይረስ',
    '♓' => 'ፓይሲስ',
    '⛎' => 'ኦፊዩቹስ',
    '🔀' => 'ትራኮችን በውዝ አዝራር',
    '🔁' => 'ድገም አዝራር',
    '🔂' => 'ነጠላ ድገም አዝራር',
    '▶' => 'የአጫውት አዝራር',
    '⏩' => 'ወደፊት አሳልፍ አዝራር',
    '⏭' => 'ቀጣይ ትራክ አዝራር',
    '⏯' => 'አጫውት ወይም ላፍታ ግታ አዝራር',
    '◀' => 'ወደ ኋላ መልስ አዝራር',
    '⏪' => 'ወደኋላ በፍጥነት መልስ አዝራር',
    '⏮' => 'የመጨረሻው ትራክ ላይ ሂድ አዝራር',
    '🔼' => 'የላይ አዝራር',
    '⏫' => 'ወደ ላይ በፍጥነት ሂድ አዝራር',
    '🔽' => 'ወደ ታች አዝራር',
    '⏬' => 'ወደ ታች በፍጥነት ሂድ አዝራር',
    '⏸' => 'ላፍታ ግታ አዝራር',
    '⏹' => 'አቁም አዝራር',
    '⏺' => 'ቅዳ አዝራር',
    '⏏' => 'አስወጣ አዝራር',
    '🎦' => 'ሲኒማ',
    '🔅' => 'አጨልም አዝራር',
    '🔆' => 'ብሩህ አድርግ አዝራር',
    '📶' => 'አንቴና አሞሌዎች',
    '🛜' => 'ገመድ አልባ',
    '📳' => 'የንዝረት ሁነታ',
    '📴' => 'ሞባይል ስልክ ይጥፋ',
    '♀' => 'የሴት ምልክት',
    '♂' => 'የወንድ ምልክት',
    '⚧' => 'የትራንስጀንደር ምልክት',
    '✖' => 'የማባዛት',
    '➕' => 'ወፍራም የመደመር ምልክት',
    '➖' => 'ማይነስ',
    '➗' => 'ማካፈል',
    '🟰' => 'ወፍራም የእኩል ይሆናል ምልክት',
    '♾' => 'ወሰን የሌለው',
    '‼' => 'ድርብ የቃለ አጋኖ ምልክት',
    '⁉' => 'ቃለ አጋኖ ከጥያቄ ምልክት ጋር',
    '❓' => 'የጥያቄ ምልክት',
    '❔' => 'ነጭ የጥያቄ ምልክት',
    '❕' => 'ነጭ የቃለ አጋኖ ምልክት',
    '❗' => 'ቀይ የቃለ አጋኖ ምልክት',
    '〰' => 'ሞገዳማ ሰረዝ',
    '💱' => 'የውጭ ምንዛሬ',
    '💲' => 'ወፍራም የዶላር ምልክት',
    '⚕' => 'የመድሃኒት ምልክት',
    '♻' => 'የመልሶ ጥቅም ላይ ማዋል ምልክት',
    '⚜' => 'የማጌጫ ምልክት',
    '🔱' => 'ትሪደንት አርማ',
    '📛' => 'የስም መለያ',
    '🔰' => 'የጃፓን የጀማሪ ምልክት',
    '⭕' => 'ወፍራም ግዙፍ ክብ',
    '✅' => 'ነጭ ወፍራም ምልክት ማድረጊያ',
    '☑' => 'የድምፅ መስጫ ሳጥን ከምልክት ማድረጊያ ጋር',
    '✔' => 'ወፍራም ምልክት ማድረጊያ',
    '❌' => 'የስረዛ ምልክት',
    '❎' => 'የስረዛ ምልክት አዝራር',
    '➰' => 'ጥቅልል ሸምቀቆ',
    '➿' => 'ድርብ ጥቅልል ሸምቀቆ',
    '〽' => 'የክፍል መቀየሪያ ምልክት',
    '✳' => 'ባለ ስምንት ቀስት አስትሪክስ',
    '✴' => 'ባለስምንት ነጥብ ኮከብ',
    '❇' => 'አንጸባራቂ',
    '™' => 'የንግድ ምልክት',
    '🫟' => 'የውሃ መረጨት',
    '🔠' => 'የላቲን ዓቢይ ፊደል ግቤት',
    '🔡' => 'የላቲን ንዑስ ፊደል ግቤት',
    '🔢' => 'የቁጥር ግቤት',
    '🔣' => 'የግቤት ምልክቶች',
    '🔤' => 'የላቲን ፊደላት ግቤት',
    '🅰' => 'ኤ አዝራር',
    '🆎' => 'ኤቢ አዝራር',
    '🅱' => 'ቢ አዝራር',
    '🆑' => 'መቆጣጠሪያ ካሬ',
    '🆒' => 'ቀዝቃዛ ካሬ',
    '🆓' => 'ነጻ ካሬ',
    'ℹ' => 'የመረጃ ምንጭ',
    '🆔' => 'አይዲ ካሬ',
    'Ⓜ' => 'በክብ ውስጥ ያለ ኤም ፊደል',
    '🆕' => 'አዲስ ካሬ',
    '🆖' => 'ኤንጂ ካሬ',
    '🅾' => 'ኦ አዝራር',
    '🆗' => 'ኦኬ ካሬ',
    '🅿' => 'ፒ አዝራር',
    '🆘' => 'ኤስኦኤስ ካሬ',
    '🆙' => 'ወደላይ! አዝራር',
    '🆚' => 'ቪኤስ ካሬ',
    '🈁' => 'ካታካና ኮኮ ካሬ',
    '🈂' => 'ካታካና ሳ ካሬ',
    '🈷' => 'የጨረቃ ካሬ አይዶግራፍ',
    '🈶' => 'ያኖረ ካሬ አይዶግራፍ',
    '🈯' => 'የጣት ካሬ አይዶግራፍ',
    '🉐' => 'ባለክብ ዕድል አይዶግራፍ',
    '🈹' => 'የተከፋፈለ ካሬ አይዶግራፍ',
    '🈚' => 'የአሉታ ካሬ አይዶግራፍ',
    '🈲' => 'የተከለከለ ካሬ አይዶግራፍ',
    '🉑' => 'ክብ የተቀበል አይዶግራፍ',
    '🈸' => 'የተግብር ካሬ አይዶግራፍ',
    '🈴' => 'አንድነት ካሬ አይዶግራፍ',
    '🈳' => 'ባዶ ካሬ አይዶግራፍ',
    '㊗' => 'ክብ የደስታ መግለጫ አይዶግራፍ',
    '㊙' => 'ክብ የምስጢር አይዶግራፍ',
    '🈺' => 'የስርዓተ ክወና ካሬ አይዶግራፍ',
    '🈵' => 'ሙሉ ካሬ አይዶግራፍ',
    '🔴' => 'ቀይ ክብ',
    '🟠' => 'ብርቱካን ክብ',
    '🟡' => 'ቢጫ ክብ',
    '🟢' => 'አረንጓዴ ክብ',
    '🔵' => 'ሰማያዊ ክብ',
    '🟣' => 'ሐምራዊ ክብ',
    '🟤' => 'ቡናማ ክብ',
    '⚫' => 'ጥቁር ክብ',
    '⚪' => 'ነጭ ክብ',
    '🟥' => 'ቀይ አራት ማእዝን',
    '🟧' => 'ብርቱካናማ አራት ማእዝን',
    '🟨' => 'ቢጫ አራት ማእዝን',
    '🟩' => 'አረንጓዴ አራት ማእዝን',
    '🟦' => 'ሰማያዊ አራት ማእዝን',
    '🟪' => 'ሐምራዊ አራት ማእዝን',
    '🟫' => 'ቡናማ አራት ማእዝን',
    '⬛' => 'ጥቁር ትልቅ ካሬ',
    '⬜' => 'ነጭ ትልቅ ካሬ',
    '◼' => 'ጥቁር መካከለኛ ካሬ',
    '◻' => 'ነጭ መካከለኛ ካሬ',
    '◾' => 'ጥቁር መካከለኛ-ትንሽ ካሬ',
    '◽' => 'ነጭ መካከለኛ- ትንሽ ካሬ',
    '▪' => 'ጥቁር ትንሽ ካሬ',
    '▫' => 'ነጭ ትንሽ ካሬ',
    '🔶' => 'ትልቅ ብርትኳናማ አልማዝ',
    '🔷' => 'ትልቅ ሰማያዊ አልማዝ',
    '🔸' => 'ትንሽ ብርትኳናማ አልማዝ',
    '🔹' => 'ትንሽ ሰማያዊ አልማዝ',
    '🔺' => 'ቀይ ሶስት ማዕዘን ወደ ላይ ጠቋሚ',
    '🔻' => 'ቀይ ሶስት ማዕዘን ወደ ታች ጠቋሚ',
    '💠' => 'ባለ ነቁጥ አልማዝ',
    '🔘' => 'የሬዲዮ አዝራር',
    '🔳' => 'በነጭ የተከበበ ካሬ አዝራር',
    '🔲' => 'በጥቁር የተከበበ ካሬ አዝራር',
    '🏁' => 'የዳማ ሰሌዳ መሰል ባንዴራ',
    '🚩' => 'የፖስታ ምልክት ባንዲራ',
    '🎌' => 'ጥሙር ሰንደቅ ዓላማዎች',
    '🏴' => 'ጥቁር ባንዴራ ማውለብለብ',
    '🏳' => 'ነጭ ባንዴራ ማውለብለብ',
];
