<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'Kis: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'Kis: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'Kis: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'Kis: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Dák Skín Ton',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Laít Skín Ton',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Dák Skín Ton',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'Kis: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'Kis: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Laít Skín Ton',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'Kis: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'Kis: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'Kis: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍❤‍💋‍👨🏻' => 'Kis: Man, Man, Laít Skín Ton',
    '👨🏻‍❤‍💋‍👨🏼' => 'Kis: Man, Man, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏻‍❤‍💋‍👨🏽' => 'Kis: Man, Man, Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏻‍❤‍💋‍👨🏾' => 'Kis: Man, Man, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍❤‍💋‍👨🏿' => 'Kis: Man, Man, Laít Skín Ton, Dák Skín Ton',
    '👨🏼‍❤‍💋‍👨🏻' => 'Kis: Man, Man, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👨🏼‍❤‍💋‍👨🏼' => 'Kis: Man, Man, Mídíọ́m-Laít Skín Ton',
    '👨🏼‍❤‍💋‍👨🏽' => 'Kis: Man, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏼‍❤‍💋‍👨🏾' => 'Kis: Man, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏼‍❤‍💋‍👨🏿' => 'Kis: Man, Man, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👨🏽‍❤‍💋‍👨🏻' => 'Kis: Man, Man, Mídíọ́m Skín Ton, Laít Skín Ton',
    '👨🏽‍❤‍💋‍👨🏼' => 'Kis: Man, Man, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏽‍❤‍💋‍👨🏽' => 'Kis: Man, Man, Mídíọ́m Skín Ton',
    '👨🏽‍❤‍💋‍👨🏾' => 'Kis: Man, Man, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏽‍❤‍💋‍👨🏿' => 'Kis: Man, Man, Mídíọ́m Skín Ton, Dák Skín Ton',
    '👨🏾‍❤‍💋‍👨🏻' => 'Kis: Man, Man, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👨🏾‍❤‍💋‍👨🏼' => 'Kis: Man, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏾‍❤‍💋‍👨🏽' => 'Kis: Man, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏾‍❤‍💋‍👨🏾' => 'Kis: Man, Man, Mídíọ́m Dák Skín Ton',
    '👨🏾‍❤‍💋‍👨🏿' => 'Kis: Man, Man, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👨🏿‍❤‍💋‍👨🏻' => 'Kis: Man, Man, Dák Skín Ton, Laít Skín Ton',
    '👨🏿‍❤‍💋‍👨🏼' => 'Kis: Man, Man, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏿‍❤‍💋‍👨🏽' => 'Kis: Man, Man, Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏿‍❤‍💋‍👨🏾' => 'Kis: Man, Man, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏿‍❤‍💋‍👨🏿' => 'Kis: Man, Man, Dák Skín Ton',
    '👩🏻‍❤‍💋‍👨🏻' => 'Kis: Wúman, Man, Laít Skín Ton',
    '👩🏻‍❤‍💋‍👨🏼' => 'Kis: Wúman, Man, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍❤‍💋‍👨🏽' => 'Kis: Wúman, Man, Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍❤‍💋‍👨🏾' => 'Kis: Wúman, Man, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍❤‍💋‍👨🏿' => 'Kis: Wúman, Man, Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍❤‍💋‍👨🏻' => 'Kis: Wúman, Man, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍❤‍💋‍👨🏼' => 'Kis: Wúman, Man, Mídíọ́m-Laít Skín Ton',
    '👩🏼‍❤‍💋‍👨🏽' => 'Kis: Wúman, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍❤‍💋‍👨🏾' => 'Kis: Wúman, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍❤‍💋‍👨🏿' => 'Kis: Wúman, Man, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍❤‍💋‍👨🏻' => 'Kis: Wúman, Man, Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍❤‍💋‍👨🏼' => 'Kis: Wúman, Man, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍❤‍💋‍👨🏽' => 'Kis: Wúman, Man, Mídíọ́m Skín Ton',
    '👩🏽‍❤‍💋‍👨🏾' => 'Kis: Wúman, Man, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍❤‍💋‍👨🏿' => 'Kis: Wúman, Man, Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍❤‍💋‍👨🏻' => 'Kis: Wúman, Man, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍❤‍💋‍👨🏼' => 'Kis: Wúman, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍❤‍💋‍👨🏽' => 'Kis: Wúman, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍❤‍💋‍👨🏾' => 'Kis: Wúman, Man, Mídíọ́m Dák Skín Ton',
    '👩🏾‍❤‍💋‍👨🏿' => 'Kis: Wúman, Man, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍❤‍💋‍👨🏻' => 'Kis: Wúman, Man, Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍❤‍💋‍👨🏼' => 'Kis: Wúman, Man, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍❤‍💋‍👨🏽' => 'Kis: Wúman, Man, Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍❤‍💋‍👨🏾' => 'Kis: Wúman, Man, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏿‍❤‍💋‍👨🏿' => 'Kis: Wúman, Man, Dák Skín Ton',
    '👩🏻‍❤‍💋‍👩🏻' => 'Kis: Wúman, Wúman, Laít Skín Ton',
    '👩🏻‍❤‍💋‍👩🏼' => 'Kis: Wúman, Wúman, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍❤‍💋‍👩🏽' => 'Kis: Wúman, Wúman, Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍❤‍💋‍👩🏾' => 'Kis: Wúman, Wúman, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍❤‍💋‍👩🏿' => 'Kis: Wúman, Wúman, Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍❤‍💋‍👩🏻' => 'Kis: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍❤‍💋‍👩🏼' => 'Kis: Wúman, Wúman, Mídíọ́m-Laít Skín Ton',
    '👩🏼‍❤‍💋‍👩🏽' => 'Kis: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍❤‍💋‍👩🏾' => 'Kis: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍❤‍💋‍👩🏿' => 'Kis: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍❤‍💋‍👩🏻' => 'Kis: Wúman, Wúman, Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍❤‍💋‍👩🏼' => 'Kis: Wúman, Wúman, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍❤‍💋‍👩🏽' => 'Kis: Wúman, Wúman, Mídíọ́m Skín Ton',
    '👩🏽‍❤‍💋‍👩🏾' => 'Kis: Wúman, Wúman, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍❤‍💋‍👩🏿' => 'Kis: Wúman, Wúman, Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍❤‍💋‍👩🏻' => 'Kis: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍❤‍💋‍👩🏼' => 'Kis: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍❤‍💋‍👩🏽' => 'Kis: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍❤‍💋‍👩🏾' => 'Kis: Wúman, Wúman, Mídíọ́m Dák Skín Ton',
    '👩🏾‍❤‍💋‍👩🏿' => 'Kis: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍❤‍💋‍👩🏻' => 'Kis: Wúman, Wúman, Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍❤‍💋‍👩🏼' => 'Kis: Wúman, Wúman, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍❤‍💋‍👩🏽' => 'Kis: Wúman, Wúman, Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍❤‍💋‍👩🏾' => 'Kis: Wúman, Wúman, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏿‍❤‍💋‍👩🏿' => 'Kis: Wúman, Wúman, Dák Skín Ton',
    '🧑🏻‍🫯‍🧑🏼' => 'Pípul Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏻‍🫯‍🧑🏽' => 'Pípul Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏻‍🫯‍🧑🏾' => 'Pípul Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍🫯‍🧑🏿' => 'Pípul Wé De Rẹ́sul: Laít Skín Ton, Dák Skín Ton',
    '🧑🏼‍🫯‍🧑🏻' => 'Pípul Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '🧑🏼‍🫯‍🧑🏽' => 'Pípul Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏼‍🫯‍🧑🏾' => 'Pípul Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏼‍🫯‍🧑🏿' => 'Pípul Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '🧑🏽‍🫯‍🧑🏻' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Skín Ton, Laít Skín Ton',
    '🧑🏽‍🫯‍🧑🏼' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🫯‍🧑🏾' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏽‍🫯‍🧑🏿' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Skín Ton, Dák Skín Ton',
    '🧑🏾‍🫯‍🧑🏻' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '🧑🏾‍🫯‍🧑🏼' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏾‍🫯‍🧑🏽' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏾‍🫯‍🧑🏿' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '🧑🏿‍🫯‍🧑🏻' => 'Pípul Wé De Rẹ́sul: Dák Skín Ton, Laít Skín Ton',
    '🧑🏿‍🫯‍🧑🏼' => 'Pípul Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏿‍🫯‍🧑🏽' => 'Pípul Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏿‍🫯‍🧑🏾' => 'Pípul Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍❤‍🧑🏼' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏻‍❤‍🧑🏽' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏻‍❤‍🧑🏾' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍❤‍🧑🏿' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Laít Skín Ton, Dák Skín Ton',
    '🧑🏼‍❤‍🧑🏻' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '🧑🏼‍❤‍🧑🏽' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏼‍❤‍🧑🏾' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏼‍❤‍🧑🏿' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '🧑🏽‍❤‍🧑🏻' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Laít Skín Ton',
    '🧑🏽‍❤‍🧑🏼' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍❤‍🧑🏾' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏽‍❤‍🧑🏿' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Skín Ton, Dák Skín Ton',
    '🧑🏾‍❤‍🧑🏻' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '🧑🏾‍❤‍🧑🏼' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏾‍❤‍🧑🏽' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏾‍❤‍🧑🏿' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '🧑🏿‍❤‍🧑🏻' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Laít Skín Ton',
    '🧑🏿‍❤‍🧑🏼' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏿‍❤‍🧑🏽' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏿‍❤‍🧑🏾' => 'Tuú Pípul Wit Hat: Pẹ́sin, Pẹ́sin, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍🐰‍🧑🏼' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏻‍🐰‍🧑🏽' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏻‍🐰‍🧑🏾' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍🐰‍🧑🏿' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Laít Skín Ton, Dák Skín Ton',
    '🧑🏼‍🐰‍🧑🏻' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '🧑🏼‍🐰‍🧑🏽' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏼‍🐰‍🧑🏾' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏼‍🐰‍🧑🏿' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '🧑🏽‍🐰‍🧑🏻' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Skín Ton, Laít Skín Ton',
    '🧑🏽‍🐰‍🧑🏼' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🐰‍🧑🏾' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏽‍🐰‍🧑🏿' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Skín Ton, Dák Skín Ton',
    '🧑🏾‍🐰‍🧑🏻' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '🧑🏾‍🐰‍🧑🏼' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏾‍🐰‍🧑🏽' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏾‍🐰‍🧑🏿' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '🧑🏿‍🐰‍🧑🏻' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Dák Skín Ton, Laít Skín Ton',
    '🧑🏿‍🐰‍🧑🏼' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏿‍🐰‍🧑🏽' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏿‍🐰‍🧑🏾' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍🫯‍👨🏼' => 'Mẹn Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏻‍🫯‍👨🏽' => 'Mẹn Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏻‍🫯‍👨🏾' => 'Mẹn Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍🫯‍👨🏿' => 'Mẹn Wé De Rẹ́sul: Laít Skín Ton, Dák Skín Ton',
    '👨🏼‍🫯‍👨🏻' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👨🏼‍🫯‍👨🏽' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏼‍🫯‍👨🏾' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏼‍🫯‍👨🏿' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👨🏽‍🫯‍👨🏻' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Laít Skín Ton',
    '👨🏽‍🫯‍👨🏼' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🫯‍👨🏾' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏽‍🫯‍👨🏿' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Dák Skín Ton',
    '👨🏾‍🫯‍👨🏻' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👨🏾‍🫯‍👨🏼' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏾‍🫯‍👨🏽' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏾‍🫯‍👨🏿' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👨🏿‍🫯‍👨🏻' => 'Mẹn Wé De Rẹ́sul: Dák Skín Ton, Laít Skín Ton',
    '👨🏿‍🫯‍👨🏼' => 'Mẹn Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏿‍🫯‍👨🏽' => 'Mẹn Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏿‍🫯‍👨🏾' => 'Mẹn Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Man, Man, Laít Skín Ton',
    '👨🏻‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Man, Man, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏻‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Man, Man, Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏻‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Man, Man, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Man, Man, Laít Skín Ton, Dák Skín Ton',
    '👨🏼‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👨🏼‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m-Laít Skín Ton',
    '👨🏼‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏼‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏼‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👨🏽‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Skín Ton, Laít Skín Ton',
    '👨🏽‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏽‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Skín Ton',
    '👨🏽‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏽‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Skín Ton, Dák Skín Ton',
    '👨🏾‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👨🏾‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏾‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏾‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Dák Skín Ton',
    '👨🏾‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Man, Man, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👨🏿‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Man, Man, Dák Skín Ton, Laít Skín Ton',
    '👨🏿‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Man, Man, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏿‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Man, Man, Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏿‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Man, Man, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏿‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Man, Man, Dák Skín Ton',
    '👨🏻‍🐰‍👨🏼' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏻‍🐰‍👨🏽' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏻‍🐰‍👨🏾' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍🐰‍👨🏿' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Dák Skín Ton',
    '👨🏼‍🐰‍👨🏻' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👨🏼‍🐰‍👨🏽' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏼‍🐰‍👨🏾' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏼‍🐰‍👨🏿' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👨🏽‍🐰‍👨🏻' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Laít Skín Ton',
    '👨🏽‍🐰‍👨🏼' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🐰‍👨🏾' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏽‍🐰‍👨🏿' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Dák Skín Ton',
    '👨🏾‍🐰‍👨🏻' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👨🏾‍🐰‍👨🏼' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏾‍🐰‍👨🏽' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏾‍🐰‍👨🏿' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👨🏿‍🐰‍👨🏻' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Laít Skín Ton',
    '👨🏿‍🐰‍👨🏼' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏿‍🐰‍👨🏽' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏿‍🐰‍👨🏾' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍🫯‍👩🏼' => 'Wímẹn Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍🫯‍👩🏽' => 'Wímẹn Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍🫯‍👩🏾' => 'Wímẹn Wé De Rẹ́sul: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍🫯‍👩🏿' => 'Wímẹn Wé De Rẹ́sul: Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍🫯‍👩🏻' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍🫯‍👩🏽' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍🫯‍👩🏾' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍🫯‍👩🏿' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍🫯‍👩🏻' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍🫯‍👩🏼' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🫯‍👩🏾' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍🫯‍👩🏿' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍🫯‍👩🏻' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍🫯‍👩🏼' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍🫯‍👩🏽' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍🫯‍👩🏿' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍🫯‍👩🏻' => 'Wímẹn Wé De Rẹ́sul: Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍🫯‍👩🏼' => 'Wímẹn Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍🫯‍👩🏽' => 'Wímẹn Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍🫯‍👩🏾' => 'Wímẹn Wé De Rẹ́sul: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Wúman, Man, Laít Skín Ton',
    '👩🏻‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Wúman, Man, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Wúman, Man, Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Wúman, Man, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Wúman, Man, Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m-Laít Skín Ton',
    '👩🏼‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Skín Ton',
    '👩🏽‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Dák Skín Ton',
    '👩🏾‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Wúman, Man, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍❤‍👨🏻' => 'Tuú Pípul Wit Hat: Wúman, Man, Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍❤‍👨🏼' => 'Tuú Pípul Wit Hat: Wúman, Man, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍❤‍👨🏽' => 'Tuú Pípul Wit Hat: Wúman, Man, Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍❤‍👨🏾' => 'Tuú Pípul Wit Hat: Wúman, Man, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏿‍❤‍👨🏿' => 'Tuú Pípul Wit Hat: Wúman, Man, Dák Skín Ton',
    '👩🏻‍❤‍👩🏻' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Laít Skín Ton',
    '👩🏻‍❤‍👩🏼' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍❤‍👩🏽' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍❤‍👩🏾' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍❤‍👩🏿' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍❤‍👩🏻' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍❤‍👩🏼' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m-Laít Skín Ton',
    '👩🏼‍❤‍👩🏽' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍❤‍👩🏾' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍❤‍👩🏿' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍❤‍👩🏻' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍❤‍👩🏼' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍❤‍👩🏽' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Skín Ton',
    '👩🏽‍❤‍👩🏾' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍❤‍👩🏿' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍❤‍👩🏻' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍❤‍👩🏼' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍❤‍👩🏽' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍❤‍👩🏾' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Dák Skín Ton',
    '👩🏾‍❤‍👩🏿' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍❤‍👩🏻' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍❤‍👩🏼' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍❤‍👩🏽' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍❤‍👩🏾' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏿‍❤‍👩🏿' => 'Tuú Pípul Wit Hat: Wúman, Wúman, Dák Skín Ton',
    '👩🏻‍🐰‍👩🏼' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍🐰‍👩🏽' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍🐰‍👩🏾' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍🐰‍👩🏿' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍🐰‍👩🏻' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍🐰‍👩🏽' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍🐰‍👩🏾' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍🐰‍👩🏿' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍🐰‍👩🏻' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍🐰‍👩🏼' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🐰‍👩🏾' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍🐰‍👩🏿' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍🐰‍👩🏻' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍🐰‍👩🏼' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍🐰‍👩🏽' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍🐰‍👩🏿' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍🐰‍👩🏻' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍🐰‍👩🏼' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍🐰‍👩🏽' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍🐰‍👩🏾' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍🤝‍🧑🏻' => 'Pípul Wé Hól Hand: Laít Skín Ton',
    '🧑🏻‍🤝‍🧑🏼' => 'Pípul Wé Hól Hand: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏻‍🤝‍🧑🏽' => 'Pípul Wé Hól Hand: Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏻‍🤝‍🧑🏾' => 'Pípul Wé Hól Hand: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏻‍🤝‍🧑🏿' => 'Pípul Wé Hól Hand: Laít Skín Ton, Dák Skín Ton',
    '🧑🏼‍🤝‍🧑🏻' => 'Pípul Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '🧑🏼‍🤝‍🧑🏼' => 'Pípul Wé Hól Hand: Mídíọ́m-Laít Skín Ton',
    '🧑🏼‍🤝‍🧑🏽' => 'Pípul Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏼‍🤝‍🧑🏾' => 'Pípul Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏼‍🤝‍🧑🏿' => 'Pípul Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '🧑🏽‍🤝‍🧑🏻' => 'Pípul Wé Hól Hand: Mídíọ́m Skín Ton, Laít Skín Ton',
    '🧑🏽‍🤝‍🧑🏼' => 'Pípul Wé Hól Hand: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🤝‍🧑🏽' => 'Pípul Wé Hól Hand: Mídíọ́m Skín Ton',
    '🧑🏽‍🤝‍🧑🏾' => 'Pípul Wé Hól Hand: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏽‍🤝‍🧑🏿' => 'Pípul Wé Hól Hand: Mídíọ́m Skín Ton, Dák Skín Ton',
    '🧑🏾‍🤝‍🧑🏻' => 'Pípul Wé Hól Hand: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '🧑🏾‍🤝‍🧑🏼' => 'Pípul Wé Hól Hand: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏾‍🤝‍🧑🏽' => 'Pípul Wé Hól Hand: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏾‍🤝‍🧑🏾' => 'Pípul Wé Hól Hand: Mídíọ́m Dák Skín Ton',
    '🧑🏾‍🤝‍🧑🏿' => 'Pípul Wé Hól Hand: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '🧑🏿‍🤝‍🧑🏻' => 'Pípul Wé Hól Hand: Dák Skín Ton, Laít Skín Ton',
    '🧑🏿‍🤝‍🧑🏼' => 'Pípul Wé Hól Hand: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🧑🏿‍🤝‍🧑🏽' => 'Pípul Wé Hól Hand: Dák Skín Ton, Mídíọ́m Skín Ton',
    '🧑🏿‍🤝‍🧑🏾' => 'Pípul Wé Hól Hand: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🤝‍🧑🏿' => 'Pípul Wé Hól Hand: Dák Skín Ton',
    '👩🏻‍🤝‍👩🏼' => 'Wímẹn Wé Hól Hand: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍🤝‍👩🏽' => 'Wímẹn Wé Hól Hand: Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍🤝‍👩🏾' => 'Wímẹn Wé Hól Hand: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍🤝‍👩🏿' => 'Wímẹn Wé Hól Hand: Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍🤝‍👩🏻' => 'Wímẹn Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍🤝‍👩🏽' => 'Wímẹn Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍🤝‍👩🏾' => 'Wímẹn Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍🤝‍👩🏿' => 'Wímẹn Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍🤝‍👩🏻' => 'Wímẹn Wé Hól Hand: Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍🤝‍👩🏼' => 'Wímẹn Wé Hól Hand: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🤝‍👩🏾' => 'Wímẹn Wé Hól Hand: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍🤝‍👩🏿' => 'Wímẹn Wé Hól Hand: Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍🤝‍👩🏻' => 'Wímẹn Wé Hól Hand: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍🤝‍👩🏼' => 'Wímẹn Wé Hól Hand: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍🤝‍👩🏽' => 'Wímẹn Wé Hól Hand: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍🤝‍👩🏿' => 'Wímẹn Wé Hól Hand: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍🤝‍👩🏻' => 'Wímẹn Wé Hól Hand: Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍🤝‍👩🏼' => 'Wímẹn Wé Hól Hand: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍🤝‍👩🏽' => 'Wímẹn Wé Hól Hand: Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍🤝‍👩🏾' => 'Wímẹn Wé Hól Hand: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍🤝‍👨🏼' => 'Wúman An Man Wé Hól Hand: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏻‍🤝‍👨🏽' => 'Wúman An Man Wé Hól Hand: Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏻‍🤝‍👨🏾' => 'Wúman An Man Wé Hól Hand: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏻‍🤝‍👨🏿' => 'Wúman An Man Wé Hól Hand: Laít Skín Ton, Dák Skín Ton',
    '👩🏼‍🤝‍👨🏻' => 'Wúman An Man Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👩🏼‍🤝‍👨🏽' => 'Wúman An Man Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👩🏼‍🤝‍👨🏾' => 'Wúman An Man Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏼‍🤝‍👨🏿' => 'Wúman An Man Wé Hól Hand: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👩🏽‍🤝‍👨🏻' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Skín Ton, Laít Skín Ton',
    '👩🏽‍🤝‍👨🏼' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🤝‍👨🏾' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩🏽‍🤝‍👨🏿' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Skín Ton, Dák Skín Ton',
    '👩🏾‍🤝‍👨🏻' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👩🏾‍🤝‍👨🏼' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏾‍🤝‍👨🏽' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏾‍🤝‍👨🏿' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👩🏿‍🤝‍👨🏻' => 'Wúman An Man Wé Hól Hand: Dák Skín Ton, Laít Skín Ton',
    '👩🏿‍🤝‍👨🏼' => 'Wúman An Man Wé Hól Hand: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👩🏿‍🤝‍👨🏽' => 'Wúman An Man Wé Hól Hand: Dák Skín Ton, Mídíọ́m Skín Ton',
    '👩🏿‍🤝‍👨🏾' => 'Wúman An Man Wé Hól Hand: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍🤝‍👨🏼' => 'Man-dẹm Wé Hól Han: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏻‍🤝‍👨🏽' => 'Man-dẹm Wé Hól Han: Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏻‍🤝‍👨🏾' => 'Man-dẹm Wé Hól Han: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏻‍🤝‍👨🏿' => 'Man-dẹm Wé Hól Han: Laít Skín Ton, Dák Skín Ton',
    '👨🏼‍🤝‍👨🏻' => 'Man-dẹm Wé Hól Han: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '👨🏼‍🤝‍👨🏽' => 'Man-dẹm Wé Hól Han: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '👨🏼‍🤝‍👨🏾' => 'Man-dẹm Wé Hól Han: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏼‍🤝‍👨🏿' => 'Man-dẹm Wé Hól Han: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '👨🏽‍🤝‍👨🏻' => 'Man-dẹm Wé Hól Han: Mídíọ́m Skín Ton, Laít Skín Ton',
    '👨🏽‍🤝‍👨🏼' => 'Man-dẹm Wé Hól Han: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🤝‍👨🏾' => 'Man-dẹm Wé Hól Han: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '👨🏽‍🤝‍👨🏿' => 'Man-dẹm Wé Hól Han: Mídíọ́m Skín Ton, Dák Skín Ton',
    '👨🏾‍🤝‍👨🏻' => 'Man-dẹm Wé Hól Han: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '👨🏾‍🤝‍👨🏼' => 'Man-dẹm Wé Hól Han: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏾‍🤝‍👨🏽' => 'Man-dẹm Wé Hól Han: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏾‍🤝‍👨🏿' => 'Man-dẹm Wé Hól Han: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '👨🏿‍🤝‍👨🏻' => 'Man-dẹm Wé Hól Han: Dák Skín Ton, Laít Skín Ton',
    '👨🏿‍🤝‍👨🏼' => 'Man-dẹm Wé Hól Han: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '👨🏿‍🤝‍👨🏽' => 'Man-dẹm Wé Hól Han: Dák Skín Ton, Mídíọ́m Skín Ton',
    '👨🏿‍🤝‍👨🏾' => 'Man-dẹm Wé Hól Han: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '👩‍❤‍💋‍👨' => 'Kis: Wúman, Man',
    '👨‍❤‍💋‍👨' => 'Kis: Man, Man',
    '👩‍❤‍💋‍👩' => 'Kis: Wúman, Wúman',
    '👨‍👩‍👧‍👦' => 'Fámíli: Man, Wúman, Gẹl, Smọ́l Bọi',
    '👨‍👩‍👦‍👦' => 'Fámíli: Man, Wúman, Smọ́l Bọi, Smọ́l Bọi',
    '👨‍👩‍👧‍👧' => 'Fámíli: Man, Wúman, Gẹl, Gẹl',
    '👨‍👨‍👧‍👦' => 'Fámíli: Man, Man, Gẹl, Smọ́l Bọi',
    '👨‍👨‍👦‍👦' => 'Fámíli: Man, Man, Smọ́l Bọi, Smọ́l Bọi',
    '👨‍👨‍👧‍👧' => 'Fámíli: Man, Man, Gẹl, Gẹl',
    '👩‍👩‍👧‍👦' => 'Fámíli: Wúman, Wúman, Gẹl, Smọ́l Bọi',
    '👩‍👩‍👦‍👦' => 'Fámíli: Wúman, Wúman, Smọ́l Bọi, Smọ́l Bọi',
    '👩‍👩‍👧‍👧' => 'Fámíli: Wúman, Wúman, Gẹl, Gẹl',
    '🧑‍🧑‍🧒‍🧒' => 'fámíli: machọ́ pẹ́sin, machọ́ pẹ́sin, pikin, pikin',
    '🚶🏻‍♀‍➡' => 'Wúman Wé De Waká: Laít Skín Ton, Fẹsin ráít',
    '🚶🏼‍♀‍➡' => 'Wúman Wé De Waká: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🚶🏽‍♀‍➡' => 'Wúman Wé De Waká: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🚶🏾‍♀‍➡' => 'Wúman Wé De Waká: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🚶🏿‍♀‍➡' => 'Wúman Wé De Waká: Dák Skín Ton, Fẹsin ráít',
    '🚶🏻‍♂‍➡' => 'Man Wé De Waká: Laít Skín Ton, Fẹsin ráít',
    '🚶🏼‍♂‍➡' => 'Man Wé De Waká: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🚶🏽‍♂‍➡' => 'Man Wé De Waká: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🚶🏾‍♂‍➡' => 'Man Wé De Waká: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🚶🏿‍♂‍➡' => 'Man Wé De Waká: Dák Skín Ton, Fẹsin ráít',
    '🧎🏻‍♀‍➡' => 'Wúman Wé Níl Daun: Laít Skín Ton, Fẹsin ráít',
    '🧎🏼‍♀‍➡' => 'Wúman Wé Níl Daun: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🧎🏽‍♀‍➡' => 'Wúman Wé Níl Daun: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🧎🏾‍♀‍➡' => 'Wúman Wé Níl Daun: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🧎🏿‍♀‍➡' => 'Wúman Wé Níl Daun: Dák Skín Ton, Fẹsin ráít',
    '🧎🏻‍♂‍➡' => 'Man Wé Níl Daun: Laít Skín Ton, Fẹsin ráít',
    '🧎🏼‍♂‍➡' => 'Man Wé Níl Daun: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🧎🏽‍♂‍➡' => 'Man Wé Níl Daun: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🧎🏾‍♂‍➡' => 'Man Wé Níl Daun: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🧎🏿‍♂‍➡' => 'Man Wé Níl Daun: Dák Skín Ton, Fẹsin ráít',
    '🧑🏻‍🦯‍➡' => 'Pẹ́sin Wit Waít Wọkín-Stik: Laít Skín Ton, Fẹsin ráít',
    '🧑🏼‍🦯‍➡' => 'Pẹ́sin Wit Waít Wọkín-Stik: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🧑🏽‍🦯‍➡' => 'Pẹ́sin Wit Waít Wọkín-Stik: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🧑🏾‍🦯‍➡' => 'Pẹ́sin Wit Waít Wọkín-Stik: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🧑🏿‍🦯‍➡' => 'Pẹ́sin Wit Waít Wọkín-Stik: Dák Skín Ton, Fẹsin ráít',
    '👨🏻‍🦯‍➡' => 'Man Wit Waít Wọkín-Stik: Laít Skín Ton, Fẹsin ráít',
    '👨🏼‍🦯‍➡' => 'Man Wit Waít Wọkín-Stik: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '👨🏽‍🦯‍➡' => 'Man Wit Waít Wọkín-Stik: Mídíọ́m Skín Ton, Fẹsin ráít',
    '👨🏾‍🦯‍➡' => 'Man Wit Waít Wọkín-Stik: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '👨🏿‍🦯‍➡' => 'Man Wit Waít Wọkín-Stik: Dák Skín Ton, Fẹsin ráít',
    '👩🏻‍🦯‍➡' => 'Wúman Wit Waít Wọkín-Stik: Laít Skín Ton, Fẹsin ráít',
    '👩🏼‍🦯‍➡' => 'Wúman Wit Waít Wọkín-Stik: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '👩🏽‍🦯‍➡' => 'Wúman Wit Waít Wọkín-Stik: Mídíọ́m Skín Ton, Fẹsin ráít',
    '👩🏾‍🦯‍➡' => 'Wúman Wit Waít Wọkín-Stik: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '👩🏿‍🦯‍➡' => 'Wúman Wit Waít Wọkín-Stik: Dák Skín Ton, Fẹsin ráít',
    '🧑🏻‍🦼‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Laít Skín Ton, Fẹsin ráít',
    '🧑🏼‍🦼‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🧑🏽‍🦼‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🧑🏾‍🦼‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🧑🏿‍🦼‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Dák Skín Ton, Fẹsin ráít',
    '👨🏻‍🦼‍➡' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Laít Skín Ton, Fẹsin ráít',
    '👨🏼‍🦼‍➡' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '👨🏽‍🦼‍➡' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton, Fẹsin ráít',
    '👨🏾‍🦼‍➡' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '👨🏿‍🦼‍➡' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Dák Skín Ton, Fẹsin ráít',
    '👩🏻‍🦼‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Laít Skín Ton, Fẹsin ráít',
    '👩🏼‍🦼‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '👩🏽‍🦼‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton, Fẹsin ráít',
    '👩🏾‍🦼‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '👩🏿‍🦼‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Dák Skín Ton, Fẹsin ráít',
    '🧑🏻‍🦽‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Laít Skín Ton, Fẹsin ráít',
    '🧑🏼‍🦽‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🧑🏽‍🦽‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🧑🏾‍🦽‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🧑🏿‍🦽‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Dák Skín Ton, Fẹsin ráít',
    '👨🏻‍🦽‍➡' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Laít Skín Ton, Fẹsin ráít',
    '👨🏼‍🦽‍➡' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '👨🏽‍🦽‍➡' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton, Fẹsin ráít',
    '👨🏾‍🦽‍➡' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '👨🏿‍🦽‍➡' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Dák Skín Ton, Fẹsin ráít',
    '👩🏻‍🦽‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Laít Skín Ton, Fẹsin ráít',
    '👩🏼‍🦽‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '👩🏽‍🦽‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton, Fẹsin ráít',
    '👩🏾‍🦽‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '👩🏿‍🦽‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Dák Skín Ton, Fẹsin ráít',
    '🏃🏻‍♀‍➡' => 'Wúman Wé De Rọn: Laít Skín Ton, Fẹsin ráít',
    '🏃🏼‍♀‍➡' => 'Wúman Wé De Rọn: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🏃🏽‍♀‍➡' => 'Wúman Wé De Rọn: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🏃🏾‍♀‍➡' => 'Wúman Wé De Rọn: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🏃🏿‍♀‍➡' => 'Wúman Wé De Rọn: Dák Skín Ton, Fẹsin ráít',
    '🏃🏻‍♂‍➡' => 'Man Wé De Rọn: Laít Skín Ton, Fẹsin ráít',
    '🏃🏼‍♂‍➡' => 'Man Wé De Rọn: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🏃🏽‍♂‍➡' => 'Man Wé De Rọn: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🏃🏾‍♂‍➡' => 'Man Wé De Rọn: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🏃🏿‍♂‍➡' => 'Man Wé De Rọn: Dák Skín Ton, Fẹsin ráít',
    '🫱🏻‍🫲🏼' => 'Hándshek: Laít Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🫱🏻‍🫲🏽' => 'Hándshek: Laít Skín Ton, Mídíọ́m Skín Ton',
    '🫱🏻‍🫲🏾' => 'Hándshek: Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🫱🏻‍🫲🏿' => 'Hándshek: Laít Skín Ton, Dák Skín Ton',
    '🫱🏼‍🫲🏻' => 'Hándshek: Mídíọ́m-Laít Skín Ton, Laít Skín Ton',
    '🫱🏼‍🫲🏽' => 'Hándshek: Mídíọ́m-Laít Skín Ton, Mídíọ́m Skín Ton',
    '🫱🏼‍🫲🏾' => 'Hándshek: Mídíọ́m-Laít Skín Ton, Mídíọ́m Dák Skín Ton',
    '🫱🏼‍🫲🏿' => 'Hándshek: Mídíọ́m-Laít Skín Ton, Dák Skín Ton',
    '🫱🏽‍🫲🏻' => 'Hándshek: Mídíọ́m Skín Ton, Laít Skín Ton',
    '🫱🏽‍🫲🏼' => 'Hándshek: Mídíọ́m Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🫱🏽‍🫲🏾' => 'Hándshek: Mídíọ́m Skín Ton, Mídíọ́m Dák Skín Ton',
    '🫱🏽‍🫲🏿' => 'Hándshek: Mídíọ́m Skín Ton, Dák Skín Ton',
    '🫱🏾‍🫲🏻' => 'Hándshek: Mídíọ́m Dák Skín Ton, Laít Skín Ton',
    '🫱🏾‍🫲🏼' => 'Hándshek: Mídíọ́m Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🫱🏾‍🫲🏽' => 'Hándshek: Mídíọ́m Dák Skín Ton, Mídíọ́m Skín Ton',
    '🫱🏾‍🫲🏿' => 'Hándshek: Mídíọ́m Dák Skín Ton, Dák Skín Ton',
    '🫱🏿‍🫲🏻' => 'Hándshek: Dák Skín Ton, Laít Skín Ton',
    '🫱🏿‍🫲🏼' => 'Hándshek: Dák Skín Ton, Mídíọ́m-Laít Skín Ton',
    '🫱🏿‍🫲🏽' => 'Hándshek: Dák Skín Ton, Mídíọ́m Skín Ton',
    '🫱🏿‍🫲🏾' => 'Hándshek: Dák Skín Ton, Mídíọ́m Dák Skín Ton',
    '🚶‍♀‍➡' => 'Wúman Wé De Waká: Fẹsin ráít',
    '🚶‍♂‍➡' => 'Man Wé De Waká: Fẹsin ráít',
    '🧎‍♀‍➡' => 'Wúman Wé Níl Daun: Fẹsin ráít',
    '🧎‍♂‍➡' => 'Man Wé Níl Daun: Fẹsin ráít',
    '🧑‍🦯‍➡' => 'Pẹ́sin Wit Waít Wọkín-Stik: Fẹsin ráít',
    '👨‍🦯‍➡' => 'Man Wit Waít Wọkín-Stik: Fẹsin ráít',
    '👩‍🦯‍➡' => 'Wúman Wit Waít Wọkín-Stik: Fẹsin ráít',
    '🧑‍🦼‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Fẹsin ráít',
    '👨‍🦼‍➡' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Fẹsin ráít',
    '👩‍🦼‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Fẹsin ráít',
    '🧑‍🦽‍➡' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Fẹsin ráít',
    '👨‍🦽‍➡' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Fẹsin ráít',
    '👩‍🦽‍➡' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Fẹsin ráít',
    '🏃‍♀‍➡' => 'Wúman Wé De Rọn: Fẹsin ráít',
    '🏃‍♂‍➡' => 'Man Wé De Rọn: Fẹsin ráít',
    '👩‍❤‍👨' => 'Tuú Pípul Wit Hat: Wúman, Man',
    '👨‍❤‍👨' => 'Tuú Pípul Wit Hat: Man, Man',
    '👩‍❤‍👩' => 'Tuú Pípul Wit Hat: Wúman, Wúman',
    '👨‍👩‍👦' => 'Fámíli: Man, Wúman, Smọ́l Bọi',
    '👨‍👩‍👧' => 'Fámíli: Man, Wúman, Gẹl',
    '👨‍👨‍👦' => 'Fámíli: Man, Man, Smọ́l Bọi',
    '👨‍👨‍👧' => 'Fámíli: Man, Man, Gẹl',
    '👩‍👩‍👦' => 'Fámíli: Wúman, Wúman, Smọ́l Bọi',
    '👩‍👩‍👧' => 'Fámíli: Wúman, Wúman, Gẹl',
    '👨‍👦‍👦' => 'Fámíli: Man, Smọ́l Bọi, Smọ́l Bọi',
    '👨‍👧‍👦' => 'Fámíli: Man, Gẹl, Smọ́l Bọi',
    '👨‍👧‍👧' => 'Fámíli: Man, Gẹl, Gẹl',
    '👩‍👦‍👦' => 'Fámíli: Wúman, Smọ́l Bọi, Smọ́l Bọi',
    '👩‍👧‍👦' => 'Fámíli: Wúman, Gẹl, Smọ́l Bọi',
    '👩‍👧‍👧' => 'Fámíli: Wúman, Gẹl, Gẹl',
    '🧑‍🤝‍🧑' => 'Pípul Wé Hól Hand',
    '🧑‍🧑‍🧒' => 'fámíli: machọ́ pẹ́sin, machọ́ pẹ́sin, pikin',
    '🧑‍🧒‍🧒' => 'fámíli: machọ́ pẹ́sin, pikin, pikin',
    '🧑🏻‍🦰' => 'Pẹ́sin: Laít Skín Ton, Rẹ́d Hea',
    '🧑🏼‍🦰' => 'Pẹ́sin: Mídíọ́m-Laít Skín Ton, Rẹ́d Hea',
    '🧑🏽‍🦰' => 'Pẹ́sin: Mídíọ́m Skín Ton, Rẹ́d Hea',
    '🧑🏾‍🦰' => 'Pẹ́sin: Mídíọ́m Dák Skín Ton, Rẹ́d Hea',
    '🧑🏿‍🦰' => 'Pẹ́sin: Dák Skín Ton, Rẹ́d Hea',
    '🧑🏻‍🦱' => 'Pẹ́sin: Laít Skín Ton, Kọ́lí Hea',
    '🧑🏼‍🦱' => 'Pẹ́sin: Mídíọ́m-Laít Skín Ton, Kọ́lí Hea',
    '🧑🏽‍🦱' => 'Pẹ́sin: Mídíọ́m Skín Ton, Kọ́lí Hea',
    '🧑🏾‍🦱' => 'Pẹ́sin: Mídíọ́m Dák Skín Ton, Kọ́lí Hea',
    '🧑🏿‍🦱' => 'Pẹ́sin: Dák Skín Ton, Kọ́lí Hea',
    '🧑🏻‍🦳' => 'Pẹ́sin: Laít Skín Ton, Waít Hea',
    '🧑🏼‍🦳' => 'Pẹ́sin: Mídíọ́m-Laít Skín Ton, Waít Hea',
    '🧑🏽‍🦳' => 'Pẹ́sin: Mídíọ́m Skín Ton, Waít Hea',
    '🧑🏾‍🦳' => 'Pẹ́sin: Mídíọ́m Dák Skín Ton, Waít Hea',
    '🧑🏿‍🦳' => 'Pẹ́sin: Dák Skín Ton, Waít Hea',
    '🧑🏻‍🦲' => 'Pẹ́sin: Laít Skín Ton, Bọld',
    '🧑🏼‍🦲' => 'Pẹ́sin: Mídíọ́m-Laít Skín Ton, Bọld',
    '🧑🏽‍🦲' => 'Pẹ́sin: Mídíọ́m Skín Ton, Bọld',
    '🧑🏾‍🦲' => 'Pẹ́sin: Mídíọ́m Dák Skín Ton, Bọld',
    '🧑🏿‍🦲' => 'Pẹ́sin: Dák Skín Ton, Bọld',
    '🧑🏻‍🩰' => 'bálẹ dánsá: Laít Skín Ton',
    '🧑🏼‍🩰' => 'bálẹ dánsá: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🩰' => 'bálẹ dánsá: Mídíọ́m Skín Ton',
    '🧑🏾‍🩰' => 'bálẹ dánsá: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🩰' => 'bálẹ dánsá: Dák Skín Ton',
    '🧔🏻‍♂' => 'Man Wé Gẹ́t Biabiá: Laít Skín Ton',
    '🧔🏼‍♂' => 'Man Wé Gẹ́t Biabiá: Mídíọ́m-Laít Skín Ton',
    '🧔🏽‍♂' => 'Man Wé Gẹ́t Biabiá: Mídíọ́m Skín Ton',
    '🧔🏾‍♂' => 'Man Wé Gẹ́t Biabiá: Mídíọ́m Dák Skín Ton',
    '🧔🏿‍♂' => 'Man Wé Gẹ́t Biabiá: Dák Skín Ton',
    '👱🏻‍♂' => 'Man Wé Gẹ́t Blọ́nd Hea: Laít Skín Ton',
    '👱🏼‍♂' => 'Man Wé Gẹ́t Blọ́nd Hea: Mídíọ́m-Laít Skín Ton',
    '👱🏽‍♂' => 'Man Wé Gẹ́t Blọ́nd Hea: Mídíọ́m Skín Ton',
    '👱🏾‍♂' => 'Man Wé Gẹ́t Blọ́nd Hea: Mídíọ́m Dák Skín Ton',
    '👱🏿‍♂' => 'Man Wé Gẹ́t Blọ́nd Hea: Dák Skín Ton',
    '👨🏻‍🦰' => 'Man: Laít Skín Ton, Rẹ́d Hea',
    '👨🏼‍🦰' => 'Man: Mídíọ́m-Laít Skín Ton, Rẹ́d Hea',
    '👨🏽‍🦰' => 'Man: Mídíọ́m Skín Ton, Rẹ́d Hea',
    '👨🏾‍🦰' => 'Man: Mídíọ́m Dák Skín Ton, Rẹ́d Hea',
    '👨🏿‍🦰' => 'Man: Dák Skín Ton, Rẹ́d Hea',
    '👨🏻‍🦱' => 'Man: Laít Skín Ton, Kọ́lí Hea',
    '👨🏼‍🦱' => 'Man: Mídíọ́m-Laít Skín Ton, Kọ́lí Hea',
    '👨🏽‍🦱' => 'Man: Mídíọ́m Skín Ton, Kọ́lí Hea',
    '👨🏾‍🦱' => 'Man: Mídíọ́m Dák Skín Ton, Kọ́lí Hea',
    '👨🏿‍🦱' => 'Man: Dák Skín Ton, Kọ́lí Hea',
    '👨🏻‍🦳' => 'Man: Laít Skín Ton, Waít Hea',
    '👨🏼‍🦳' => 'Man: Mídíọ́m-Laít Skín Ton, Waít Hea',
    '👨🏽‍🦳' => 'Man: Mídíọ́m Skín Ton, Waít Hea',
    '👨🏾‍🦳' => 'Man: Mídíọ́m Dák Skín Ton, Waít Hea',
    '👨🏿‍🦳' => 'Man: Dák Skín Ton, Waít Hea',
    '👨🏻‍🦲' => 'Man: Laít Skín Ton, Bọld',
    '👨🏼‍🦲' => 'Man: Mídíọ́m-Laít Skín Ton, Bọld',
    '👨🏽‍🦲' => 'Man: Mídíọ́m Skín Ton, Bọld',
    '👨🏾‍🦲' => 'Man: Mídíọ́m Dák Skín Ton, Bọld',
    '👨🏿‍🦲' => 'Man: Dák Skín Ton, Bọld',
    '🧔🏻‍♀' => 'Wúman Wé Gẹ́t Biabiá: Laít Skín Ton',
    '🧔🏼‍♀' => 'Wúman Wé Gẹ́t Biabiá: Mídíọ́m-Laít Skín Ton',
    '🧔🏽‍♀' => 'Wúman Wé Gẹ́t Biabiá: Mídíọ́m Skín Ton',
    '🧔🏾‍♀' => 'Wúman Wé Gẹ́t Biabiá: Mídíọ́m Dák Skín Ton',
    '🧔🏿‍♀' => 'Wúman Wé Gẹ́t Biabiá: Dák Skín Ton',
    '👱🏻‍♀' => 'Wúman Wé Gẹ́t Blọ́nd Hea: Laít Skín Ton',
    '👱🏼‍♀' => 'Wúman Wé Gẹ́t Blọ́nd Hea: Mídíọ́m-Laít Skín Ton',
    '👱🏽‍♀' => 'Wúman Wé Gẹ́t Blọ́nd Hea: Mídíọ́m Skín Ton',
    '👱🏾‍♀' => 'Wúman Wé Gẹ́t Blọ́nd Hea: Mídíọ́m Dák Skín Ton',
    '👱🏿‍♀' => 'Wúman Wé Gẹ́t Blọ́nd Hea: Dák Skín Ton',
    '👩🏻‍🦰' => 'Wúman: Laít Skín Ton, Rẹ́d Hea',
    '👩🏼‍🦰' => 'Wúman: Mídíọ́m-Laít Skín Ton, Rẹ́d Hea',
    '👩🏽‍🦰' => 'Wúman: Mídíọ́m Skín Ton, Rẹ́d Hea',
    '👩🏾‍🦰' => 'Wúman: Mídíọ́m Dák Skín Ton, Rẹ́d Hea',
    '👩🏿‍🦰' => 'Wúman: Dák Skín Ton, Rẹ́d Hea',
    '👩🏻‍🦱' => 'Wúman: Laít Skín Ton, Kọ́lí Hea',
    '👩🏼‍🦱' => 'Wúman: Mídíọ́m-Laít Skín Ton, Kọ́lí Hea',
    '👩🏽‍🦱' => 'Wúman: Mídíọ́m Skín Ton, Kọ́lí Hea',
    '👩🏾‍🦱' => 'Wúman: Mídíọ́m Dák Skín Ton, Kọ́lí Hea',
    '👩🏿‍🦱' => 'Wúman: Dák Skín Ton, Kọ́lí Hea',
    '👩🏻‍🦳' => 'Wúman: Laít Skín Ton, Waít Hea',
    '👩🏼‍🦳' => 'Wúman: Mídíọ́m-Laít Skín Ton, Waít Hea',
    '👩🏽‍🦳' => 'Wúman: Mídíọ́m Skín Ton, Waít Hea',
    '👩🏾‍🦳' => 'Wúman: Mídíọ́m Dák Skín Ton, Waít Hea',
    '👩🏿‍🦳' => 'Wúman: Dák Skín Ton, Waít Hea',
    '👩🏻‍🦲' => 'Wúman: Laít Skín Ton, Bọld',
    '👩🏼‍🦲' => 'Wúman: Mídíọ́m-Laít Skín Ton, Bọld',
    '👩🏽‍🦲' => 'Wúman: Mídíọ́m Skín Ton, Bọld',
    '👩🏾‍🦲' => 'Wúman: Mídíọ́m Dák Skín Ton, Bọld',
    '👩🏿‍🦲' => 'Wúman: Dák Skín Ton, Bọld',
    '🙍🏻‍♂' => 'Man Wé De Fraún Im Fes: Laít Skín Ton',
    '🙍🏼‍♂' => 'Man Wé De Fraún Im Fes: Mídíọ́m-Laít Skín Ton',
    '🙍🏽‍♂' => 'Man Wé De Fraún Im Fes: Mídíọ́m Skín Ton',
    '🙍🏾‍♂' => 'Man Wé De Fraún Im Fes: Mídíọ́m Dák Skín Ton',
    '🙍🏿‍♂' => 'Man Wé De Fraún Im Fes: Dák Skín Ton',
    '🙍🏻‍♀' => 'Wúman Wé De Fraún Im Fes: Laít Skín Ton',
    '🙍🏼‍♀' => 'Wúman Wé De Fraún Im Fes: Mídíọ́m-Laít Skín Ton',
    '🙍🏽‍♀' => 'Wúman Wé De Fraún Im Fes: Mídíọ́m Skín Ton',
    '🙍🏾‍♀' => 'Wúman Wé De Fraún Im Fes: Mídíọ́m Dák Skín Ton',
    '🙍🏿‍♀' => 'Wúman Wé De Fraún Im Fes: Dák Skín Ton',
    '🙎🏻‍♂' => 'Man Wé De Mék Lọ́ng Fes: Laít Skín Ton',
    '🙎🏼‍♂' => 'Man Wé De Mék Lọ́ng Fes: Mídíọ́m-Laít Skín Ton',
    '🙎🏽‍♂' => 'Man Wé De Mék Lọ́ng Fes: Mídíọ́m Skín Ton',
    '🙎🏾‍♂' => 'Man Wé De Mék Lọ́ng Fes: Mídíọ́m Dák Skín Ton',
    '🙎🏿‍♂' => 'Man Wé De Mék Lọ́ng Fes: Dák Skín Ton',
    '🙎🏻‍♀' => 'Wúman Wé De Mék Lọ́ng Fes: Laít Skín Ton',
    '🙎🏼‍♀' => 'Wúman Wé De Mék Lọ́ng Fes: Mídíọ́m-Laít Skín Ton',
    '🙎🏽‍♀' => 'Wúman Wé De Mék Lọ́ng Fes: Mídíọ́m Skín Ton',
    '🙎🏾‍♀' => 'Wúman Wé De Mék Lọ́ng Fes: Mídíọ́m Dák Skín Ton',
    '🙎🏿‍♀' => 'Wúman Wé De Mék Lọ́ng Fes: Dák Skín Ton',
    '🙅🏻‍♂' => 'Man Wé Ték Hand Shó Sé Na NO I Min: Laít Skín Ton',
    '🙅🏼‍♂' => 'Man Wé Ték Hand Shó Sé Na NO I Min: Mídíọ́m-Laít Skín Ton',
    '🙅🏽‍♂' => 'Man Wé Ték Hand Shó Sé Na NO I Min: Mídíọ́m Skín Ton',
    '🙅🏾‍♂' => 'Man Wé Ték Hand Shó Sé Na NO I Min: Mídíọ́m Dák Skín Ton',
    '🙅🏿‍♂' => 'Man Wé Ték Hand Shó Sé Na NO I Min: Dák Skín Ton',
    '🙅🏻‍♀' => 'Wúman Tẹk Hand De Shó Sé Na NO I Min: Laít Skín Ton',
    '🙅🏼‍♀' => 'Wúman Tẹk Hand De Shó Sé Na NO I Min: Mídíọ́m-Laít Skín Ton',
    '🙅🏽‍♀' => 'Wúman Tẹk Hand De Shó Sé Na NO I Min: Mídíọ́m Skín Ton',
    '🙅🏾‍♀' => 'Wúman Tẹk Hand De Shó Sé Na NO I Min: Mídíọ́m Dák Skín Ton',
    '🙅🏿‍♀' => 'Wúman Tẹk Hand De Shó Sé Na NO I Min: Dák Skín Ton',
    '🙆🏻‍♂' => 'Man Tẹk Hand De Shó Sé Na OK I Min: Laít Skín Ton',
    '🙆🏼‍♂' => 'Man Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m-Laít Skín Ton',
    '🙆🏽‍♂' => 'Man Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m Skín Ton',
    '🙆🏾‍♂' => 'Man Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m Dák Skín Ton',
    '🙆🏿‍♂' => 'Man Tẹk Hand De Shó Sé Na OK I Min: Dák Skín Ton',
    '🙆🏻‍♀' => 'Wúman Tẹk Hand De Shó Sé Na OK I Min: Laít Skín Ton',
    '🙆🏼‍♀' => 'Wúman Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m-Laít Skín Ton',
    '🙆🏽‍♀' => 'Wúman Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m Skín Ton',
    '🙆🏾‍♀' => 'Wúman Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m Dák Skín Ton',
    '🙆🏿‍♀' => 'Wúman Tẹk Hand De Shó Sé Na OK I Min: Dák Skín Ton',
    '💁🏻‍♂' => 'Man De Tró Im Hand: Laít Skín Ton',
    '💁🏼‍♂' => 'Man De Tró Im Hand: Mídíọ́m-Laít Skín Ton',
    '💁🏽‍♂' => 'Man De Tró Im Hand: Mídíọ́m Skín Ton',
    '💁🏾‍♂' => 'Man De Tró Im Hand: Mídíọ́m Dák Skín Ton',
    '💁🏿‍♂' => 'Man De Tró Im Hand: Dák Skín Ton',
    '💁🏻‍♀' => 'Wúman De Tró Im Hand: Laít Skín Ton',
    '💁🏼‍♀' => 'Wúman De Tró Im Hand: Mídíọ́m-Laít Skín Ton',
    '💁🏽‍♀' => 'Wúman De Tró Im Hand: Mídíọ́m Skín Ton',
    '💁🏾‍♀' => 'Wúman De Tró Im Hand: Mídíọ́m Dák Skín Ton',
    '💁🏿‍♀' => 'Wúman De Tró Im Hand: Dák Skín Ton',
    '🙋🏻‍♂' => 'Man Wé De Réz Im Hand Ọp: Laít Skín Ton',
    '🙋🏼‍♂' => 'Man Wé De Réz Im Hand Ọp: Mídíọ́m-Laít Skín Ton',
    '🙋🏽‍♂' => 'Man Wé De Réz Im Hand Ọp: Mídíọ́m Skín Ton',
    '🙋🏾‍♂' => 'Man Wé De Réz Im Hand Ọp: Mídíọ́m Dák Skín Ton',
    '🙋🏿‍♂' => 'Man Wé De Réz Im Hand Ọp: Dák Skín Ton',
    '🙋🏻‍♀' => 'Wúman Wé De Réz Im Hand Ọp: Laít Skín Ton',
    '🙋🏼‍♀' => 'Wúman Wé De Réz Im Hand Ọp: Mídíọ́m-Laít Skín Ton',
    '🙋🏽‍♀' => 'Wúman Wé De Réz Im Hand Ọp: Mídíọ́m Skín Ton',
    '🙋🏾‍♀' => 'Wúman Wé De Réz Im Hand Ọp: Mídíọ́m Dák Skín Ton',
    '🙋🏿‍♀' => 'Wúman Wé De Réz Im Hand Ọp: Dák Skín Ton',
    '🧏🏻‍♂' => 'Man Wé Nọ́ Fít Hia: Laít Skín Ton',
    '🧏🏼‍♂' => 'Man Wé Nọ́ Fít Hia: Mídíọ́m-Laít Skín Ton',
    '🧏🏽‍♂' => 'Man Wé Nọ́ Fít Hia: Mídíọ́m Skín Ton',
    '🧏🏾‍♂' => 'Man Wé Nọ́ Fít Hia: Mídíọ́m Dák Skín Ton',
    '🧏🏿‍♂' => 'Man Wé Nọ́ Fít Hia: Dák Skín Ton',
    '🧏🏻‍♀' => 'Wúman Wé Nọ́ Fít Hia: Laít Skín Ton',
    '🧏🏼‍♀' => 'Wúman Wé Nọ́ Fít Hia: Mídíọ́m-Laít Skín Ton',
    '🧏🏽‍♀' => 'Wúman Wé Nọ́ Fít Hia: Mídíọ́m Skín Ton',
    '🧏🏾‍♀' => 'Wúman Wé Nọ́ Fít Hia: Mídíọ́m Dák Skín Ton',
    '🧏🏿‍♀' => 'Wúman Wé Nọ́ Fít Hia: Dák Skín Ton',
    '🙇🏻‍♂' => 'Man Wé De Baú Daun: Laít Skín Ton',
    '🙇🏼‍♂' => 'Man Wé De Baú Daun: Mídíọ́m-Laít Skín Ton',
    '🙇🏽‍♂' => 'Man Wé De Baú Daun: Mídíọ́m Skín Ton',
    '🙇🏾‍♂' => 'Man Wé De Baú Daun: Mídíọ́m Dák Skín Ton',
    '🙇🏿‍♂' => 'Man Wé De Baú Daun: Dák Skín Ton',
    '🙇🏻‍♀' => 'Wúman Wé De Baú Daun: Laít Skín Ton',
    '🙇🏼‍♀' => 'Wúman Wé De Baú Daun: Mídíọ́m-Laít Skín Ton',
    '🙇🏽‍♀' => 'Wúman Wé De Baú Daun: Mídíọ́m Skín Ton',
    '🙇🏾‍♀' => 'Wúman Wé De Baú Daun: Mídíọ́m Dák Skín Ton',
    '🙇🏿‍♀' => 'Wúman Wé De Baú Daun: Dák Skín Ton',
    '🤦🏻‍♂' => 'Man Wé Ték Wọ́n Hand Kọ́vá Im Fes: Laít Skín Ton',
    '🤦🏼‍♂' => 'Man Wé Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m-Laít Skín Ton',
    '🤦🏽‍♂' => 'Man Wé Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m Skín Ton',
    '🤦🏾‍♂' => 'Man Wé Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m Dák Skín Ton',
    '🤦🏿‍♂' => 'Man Wé Ték Wọ́n Hand Kọ́vá Im Fes: Dák Skín Ton',
    '🤦🏻‍♀' => 'Wúman Ték Wọ́n Hand Kọ́vá Im Fes: Laít Skín Ton',
    '🤦🏼‍♀' => 'Wúman Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m-Laít Skín Ton',
    '🤦🏽‍♀' => 'Wúman Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m Skín Ton',
    '🤦🏾‍♀' => 'Wúman Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m Dák Skín Ton',
    '🤦🏿‍♀' => 'Wúman Ték Wọ́n Hand Kọ́vá Im Fes: Dák Skín Ton',
    '🤷🏻‍♂' => 'Man Wé De Shrọ́g Im Shólda: Laít Skín Ton',
    '🤷🏼‍♂' => 'Man Wé De Shrọ́g Im Shólda: Mídíọ́m-Laít Skín Ton',
    '🤷🏽‍♂' => 'Man Wé De Shrọ́g Im Shólda: Mídíọ́m Skín Ton',
    '🤷🏾‍♂' => 'Man Wé De Shrọ́g Im Shólda: Mídíọ́m Dák Skín Ton',
    '🤷🏿‍♂' => 'Man Wé De Shrọ́g Im Shólda: Dák Skín Ton',
    '🤷🏻‍♀' => 'Wúman Wé De Shrọ́g Im Shólda: Laít Skín Ton',
    '🤷🏼‍♀' => 'Wúman Wé De Shrọ́g Im Shólda: Mídíọ́m-Laít Skín Ton',
    '🤷🏽‍♀' => 'Wúman Wé De Shrọ́g Im Shólda: Mídíọ́m Skín Ton',
    '🤷🏾‍♀' => 'Wúman Wé De Shrọ́g Im Shólda: Mídíọ́m Dák Skín Ton',
    '🤷🏿‍♀' => 'Wúman Wé De Shrọ́g Im Shólda: Dák Skín Ton',
    '🧑🏻‍⚕' => 'Wọ́ka Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Laít Skín Ton',
    '🧑🏼‍⚕' => 'Wọ́ka Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍⚕' => 'Wọ́ka Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Mídíọ́m Skín Ton',
    '🧑🏾‍⚕' => 'Wọ́ka Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍⚕' => 'Wọ́ka Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Dák Skín Ton',
    '👨🏻‍⚕' => 'Wọ́kman Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Laít Skín Ton',
    '👨🏼‍⚕' => 'Wọ́kman Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍⚕' => 'Wọ́kman Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Mídíọ́m Skín Ton',
    '👨🏾‍⚕' => 'Wọ́kman Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Mídíọ́m Dák Skín Ton',
    '👨🏿‍⚕' => 'Wọ́kman Fọ Wiá Dẹm De Lúk Áftá Sík Pípul: Dák Skín Ton',
    '👩🏻‍⚕' => 'Wúman Wé De Lúk Áftá Sík Pípul: Laít Skín Ton',
    '👩🏼‍⚕' => 'Wúman Wé De Lúk Áftá Sík Pípul: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍⚕' => 'Wúman Wé De Lúk Áftá Sík Pípul: Mídíọ́m Skín Ton',
    '👩🏾‍⚕' => 'Wúman Wé De Lúk Áftá Sík Pípul: Mídíọ́m Dák Skín Ton',
    '👩🏿‍⚕' => 'Wúman Wé De Lúk Áftá Sík Pípul: Dák Skín Ton',
    '🧑🏻‍🎓' => 'Stúdẹnt: Laít Skín Ton',
    '🧑🏼‍🎓' => 'Stúdẹnt: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🎓' => 'Stúdẹnt: Mídíọ́m Skín Ton',
    '🧑🏾‍🎓' => 'Stúdẹnt: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🎓' => 'Stúdẹnt: Dák Skín Ton',
    '👨🏻‍🎓' => 'Mán Stúdẹnt: Laít Skín Ton',
    '👨🏼‍🎓' => 'Mán Stúdẹnt: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🎓' => 'Mán Stúdẹnt: Mídíọ́m Skín Ton',
    '👨🏾‍🎓' => 'Mán Stúdẹnt: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🎓' => 'Mán Stúdẹnt: Dák Skín Ton',
    '👩🏻‍🎓' => 'Wúmán Stúdẹnt: Laít Skín Ton',
    '👩🏼‍🎓' => 'Wúmán Stúdẹnt: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🎓' => 'Wúmán Stúdẹnt: Mídíọ́m Skín Ton',
    '👩🏾‍🎓' => 'Wúmán Stúdẹnt: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🎓' => 'Wúmán Stúdẹnt: Dák Skín Ton',
    '🧑🏻‍🏫' => 'Tícha: Laít Skín Ton',
    '🧑🏼‍🏫' => 'Tícha: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🏫' => 'Tícha: Mídíọ́m Skín Ton',
    '🧑🏾‍🏫' => 'Tícha: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🏫' => 'Tícha: Dák Skín Ton',
    '👨🏻‍🏫' => 'Mán Tícha: Laít Skín Ton',
    '👨🏼‍🏫' => 'Mán Tícha: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🏫' => 'Mán Tícha: Mídíọ́m Skín Ton',
    '👨🏾‍🏫' => 'Mán Tícha: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🏫' => 'Mán Tícha: Dák Skín Ton',
    '👩🏻‍🏫' => 'Wúmán Tícha: Laít Skín Ton',
    '👩🏼‍🏫' => 'Wúmán Tícha: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🏫' => 'Wúmán Tícha: Mídíọ́m Skín Ton',
    '👩🏾‍🏫' => 'Wúmán Tícha: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🏫' => 'Wúmán Tícha: Dák Skín Ton',
    '🧑🏻‍⚖' => 'Jọj: Laít Skín Ton',
    '🧑🏼‍⚖' => 'Jọj: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍⚖' => 'Jọj: Mídíọ́m Skín Ton',
    '🧑🏾‍⚖' => 'Jọj: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍⚖' => 'Jọj: Dák Skín Ton',
    '👨🏻‍⚖' => 'Mán Jọj: Laít Skín Ton',
    '👨🏼‍⚖' => 'Mán Jọj: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍⚖' => 'Mán Jọj: Mídíọ́m Skín Ton',
    '👨🏾‍⚖' => 'Mán Jọj: Mídíọ́m Dák Skín Ton',
    '👨🏿‍⚖' => 'Mán Jọj: Dák Skín Ton',
    '👩🏻‍⚖' => 'Wúmán Jọj: Laít Skín Ton',
    '👩🏼‍⚖' => 'Wúmán Jọj: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍⚖' => 'Wúmán Jọj: Mídíọ́m Skín Ton',
    '👩🏾‍⚖' => 'Wúmán Jọj: Mídíọ́m Dák Skín Ton',
    '👩🏿‍⚖' => 'Wúmán Jọj: Dák Skín Ton',
    '🧑🏻‍🌾' => 'Fáma: Laít Skín Ton',
    '🧑🏼‍🌾' => 'Fáma: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🌾' => 'Fáma: Mídíọ́m Skín Ton',
    '🧑🏾‍🌾' => 'Fáma: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🌾' => 'Fáma: Dák Skín Ton',
    '👨🏻‍🌾' => 'Mán Fáma: Laít Skín Ton',
    '👨🏼‍🌾' => 'Mán Fáma: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🌾' => 'Mán Fáma: Mídíọ́m Skín Ton',
    '👨🏾‍🌾' => 'Mán Fáma: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🌾' => 'Mán Fáma: Dák Skín Ton',
    '👩🏻‍🌾' => 'Wúmán Fáma: Laít Skín Ton',
    '👩🏼‍🌾' => 'Wúmán Fáma: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🌾' => 'Wúmán Fáma: Mídíọ́m Skín Ton',
    '👩🏾‍🌾' => 'Wúmán Fáma: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🌾' => 'Wúmán Fáma: Dák Skín Ton',
    '🧑🏻‍🍳' => 'Kuk: Laít Skín Ton',
    '🧑🏼‍🍳' => 'Kuk: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🍳' => 'Kuk: Mídíọ́m Skín Ton',
    '🧑🏾‍🍳' => 'Kuk: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🍳' => 'Kuk: Dák Skín Ton',
    '👨🏻‍🍳' => 'Mán Kuk: Laít Skín Ton',
    '👨🏼‍🍳' => 'Mán Kuk: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🍳' => 'Mán Kuk: Mídíọ́m Skín Ton',
    '👨🏾‍🍳' => 'Mán Kuk: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🍳' => 'Mán Kuk: Dák Skín Ton',
    '👩🏻‍🍳' => 'Wúmán Kuk: Laít Skín Ton',
    '👩🏼‍🍳' => 'Wúmán Kuk: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🍳' => 'Wúmán Kuk: Mídíọ́m Skín Ton',
    '👩🏾‍🍳' => 'Wúmán Kuk: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🍳' => 'Wúmán Kuk: Dák Skín Ton',
    '🧑🏻‍🔧' => 'Mẹkánik: Laít Skín Ton',
    '🧑🏼‍🔧' => 'Mẹkánik: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🔧' => 'Mẹkánik: Mídíọ́m Skín Ton',
    '🧑🏾‍🔧' => 'Mẹkánik: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🔧' => 'Mẹkánik: Dák Skín Ton',
    '👨🏻‍🔧' => 'Mán Mẹkánik: Laít Skín Ton',
    '👨🏼‍🔧' => 'Mán Mẹkánik: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🔧' => 'Mán Mẹkánik: Mídíọ́m Skín Ton',
    '👨🏾‍🔧' => 'Mán Mẹkánik: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🔧' => 'Mán Mẹkánik: Dák Skín Ton',
    '👩🏻‍🔧' => 'Wúmán Mẹkánik: Laít Skín Ton',
    '👩🏼‍🔧' => 'Wúmán Mẹkánik: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🔧' => 'Wúmán Mẹkánik: Mídíọ́m Skín Ton',
    '👩🏾‍🔧' => 'Wúmán Mẹkánik: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🔧' => 'Wúmán Mẹkánik: Dák Skín Ton',
    '🧑🏻‍🏭' => 'Fáktrí Wọ́ka: Laít Skín Ton',
    '🧑🏼‍🏭' => 'Fáktrí Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🏭' => 'Fáktrí Wọ́ka: Mídíọ́m Skín Ton',
    '🧑🏾‍🏭' => 'Fáktrí Wọ́ka: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🏭' => 'Fáktrí Wọ́ka: Dák Skín Ton',
    '👨🏻‍🏭' => 'Mán Fáktrí Wọ́ka: Laít Skín Ton',
    '👨🏼‍🏭' => 'Mán Fáktrí Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🏭' => 'Mán Fáktrí Wọ́ka: Mídíọ́m Skín Ton',
    '👨🏾‍🏭' => 'Mán Fáktrí Wọ́ka: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🏭' => 'Mán Fáktrí Wọ́ka: Dák Skín Ton',
    '👩🏻‍🏭' => 'Wúmán Fáktrí Wọ́ka: Laít Skín Ton',
    '👩🏼‍🏭' => 'Wúmán Fáktrí Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🏭' => 'Wúmán Fáktrí Wọ́ka: Mídíọ́m Skín Ton',
    '👩🏾‍🏭' => 'Wúmán Fáktrí Wọ́ka: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🏭' => 'Wúmán Fáktrí Wọ́ka: Dák Skín Ton',
    '🧑🏻‍💼' => 'Ọ́fís Wọ́ka: Laít Skín Ton',
    '🧑🏼‍💼' => 'Ọ́fís Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍💼' => 'Ọ́fís Wọ́ka: Mídíọ́m Skín Ton',
    '🧑🏾‍💼' => 'Ọ́fís Wọ́ka: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍💼' => 'Ọ́fís Wọ́ka: Dák Skín Ton',
    '👨🏻‍💼' => 'Mán Ọ́fís Wọ́ka: Laít Skín Ton',
    '👨🏼‍💼' => 'Mán Ọ́fís Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍💼' => 'Mán Ọ́fís Wọ́ka: Mídíọ́m Skín Ton',
    '👨🏾‍💼' => 'Mán Ọ́fís Wọ́ka: Mídíọ́m Dák Skín Ton',
    '👨🏿‍💼' => 'Mán Ọ́fís Wọ́ka: Dák Skín Ton',
    '👩🏻‍💼' => 'Wúmán Ọ́fís Wọ́ka: Laít Skín Ton',
    '👩🏼‍💼' => 'Wúmán Ọ́fís Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍💼' => 'Wúmán Ọ́fís Wọ́ka: Mídíọ́m Skín Ton',
    '👩🏾‍💼' => 'Wúmán Ọ́fís Wọ́ka: Mídíọ́m Dák Skín Ton',
    '👩🏿‍💼' => 'Wúmán Ọ́fís Wọ́ka: Dák Skín Ton',
    '🧑🏻‍🔬' => 'Sáyẹ́ns Ẹ́spat: Laít Skín Ton',
    '🧑🏼‍🔬' => 'Sáyẹ́ns Ẹ́spat: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🔬' => 'Sáyẹ́ns Ẹ́spat: Mídíọ́m Skín Ton',
    '🧑🏾‍🔬' => 'Sáyẹ́ns Ẹ́spat: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🔬' => 'Sáyẹ́ns Ẹ́spat: Dák Skín Ton',
    '👨🏻‍🔬' => 'Mán Sáyẹ́ns Ẹ́spat: Laít Skín Ton',
    '👨🏼‍🔬' => 'Mán Sáyẹ́ns Ẹ́spat: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🔬' => 'Mán Sáyẹ́ns Ẹ́spat: Mídíọ́m Skín Ton',
    '👨🏾‍🔬' => 'Mán Sáyẹ́ns Ẹ́spat: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🔬' => 'Mán Sáyẹ́ns Ẹ́spat: Dák Skín Ton',
    '👩🏻‍🔬' => 'Wúmán Sáyẹ́ns Ẹ́spat: Laít Skín Ton',
    '👩🏼‍🔬' => 'Wúmán Sáyẹ́ns Ẹ́spat: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🔬' => 'Wúmán Sáyẹ́ns Ẹ́spat: Mídíọ́m Skín Ton',
    '👩🏾‍🔬' => 'Wúmán Sáyẹ́ns Ẹ́spat: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🔬' => 'Wúmán Sáyẹ́ns Ẹ́spat: Dák Skín Ton',
    '🧑🏻‍💻' => 'Tẹknọ́lọ́jí Ẹ́spat: Laít Skín Ton',
    '🧑🏼‍💻' => 'Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍💻' => 'Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m Skín Ton',
    '🧑🏾‍💻' => 'Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍💻' => 'Tẹknọ́lọ́jí Ẹ́spat: Dák Skín Ton',
    '👨🏻‍💻' => 'Mán Tẹknọ́lọ́jí Ẹ́spat: Laít Skín Ton',
    '👨🏼‍💻' => 'Mán Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍💻' => 'Mán Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m Skín Ton',
    '👨🏾‍💻' => 'Mán Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m Dák Skín Ton',
    '👨🏿‍💻' => 'Mán Tẹknọ́lọ́jí Ẹ́spat: Dák Skín Ton',
    '👩🏻‍💻' => 'Wúman Tẹknọ́lọ́jí Ẹ́spat: Laít Skín Ton',
    '👩🏼‍💻' => 'Wúman Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍💻' => 'Wúman Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m Skín Ton',
    '👩🏾‍💻' => 'Wúman Tẹknọ́lọ́jí Ẹ́spat: Mídíọ́m Dák Skín Ton',
    '👩🏿‍💻' => 'Wúman Tẹknọ́lọ́jí Ẹ́spat: Dák Skín Ton',
    '🧑🏻‍🎤' => 'Sínga: Laít Skín Ton',
    '🧑🏼‍🎤' => 'Sínga: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🎤' => 'Sínga: Mídíọ́m Skín Ton',
    '🧑🏾‍🎤' => 'Sínga: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🎤' => 'Sínga: Dák Skín Ton',
    '👨🏻‍🎤' => 'Mán Sínga: Laít Skín Ton',
    '👨🏼‍🎤' => 'Mán Sínga: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🎤' => 'Mán Sínga: Mídíọ́m Skín Ton',
    '👨🏾‍🎤' => 'Mán Sínga: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🎤' => 'Mán Sínga: Dák Skín Ton',
    '👩🏻‍🎤' => 'Wúmán Sínga: Laít Skín Ton',
    '👩🏼‍🎤' => 'Wúmán Sínga: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🎤' => 'Wúmán Sínga: Mídíọ́m Skín Ton',
    '👩🏾‍🎤' => 'Wúmán Sínga: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🎤' => 'Wúmán Sínga: Dák Skín Ton',
    '🧑🏻‍🎨' => 'Átist: Laít Skín Ton',
    '🧑🏼‍🎨' => 'Átist: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🎨' => 'Átist: Mídíọ́m Skín Ton',
    '🧑🏾‍🎨' => 'Átist: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🎨' => 'Átist: Dák Skín Ton',
    '👨🏻‍🎨' => 'Mán Átist: Laít Skín Ton',
    '👨🏼‍🎨' => 'Mán Átist: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🎨' => 'Mán Átist: Mídíọ́m Skín Ton',
    '👨🏾‍🎨' => 'Mán Átist: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🎨' => 'Mán Átist: Dák Skín Ton',
    '👩🏻‍🎨' => 'Wúmán Átist: Laít Skín Ton',
    '👩🏼‍🎨' => 'Wúmán Átist: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🎨' => 'Wúmán Átist: Mídíọ́m Skín Ton',
    '👩🏾‍🎨' => 'Wúmán Átist: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🎨' => 'Wúmán Átist: Dák Skín Ton',
    '🧑🏻‍✈' => 'Paílọt: Laít Skín Ton',
    '🧑🏼‍✈' => 'Paílọt: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍✈' => 'Paílọt: Mídíọ́m Skín Ton',
    '🧑🏾‍✈' => 'Paílọt: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍✈' => 'Paílọt: Dák Skín Ton',
    '👨🏻‍✈' => 'Mán Paílọt: Laít Skín Ton',
    '👨🏼‍✈' => 'Mán Paílọt: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍✈' => 'Mán Paílọt: Mídíọ́m Skín Ton',
    '👨🏾‍✈' => 'Mán Paílọt: Mídíọ́m Dák Skín Ton',
    '👨🏿‍✈' => 'Mán Paílọt: Dák Skín Ton',
    '👩🏻‍✈' => 'Wúmán Paílọt: Laít Skín Ton',
    '👩🏼‍✈' => 'Wúmán Paílọt: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍✈' => 'Wúmán Paílọt: Mídíọ́m Skín Ton',
    '👩🏾‍✈' => 'Wúmán Paílọt: Mídíọ́m Dák Skín Ton',
    '👩🏿‍✈' => 'Wúmán Paílọt: Dák Skín Ton',
    '🧑🏻‍🚀' => 'Ástrọ́nọt: Laít Skín Ton',
    '🧑🏼‍🚀' => 'Ástrọ́nọt: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🚀' => 'Ástrọ́nọt: Mídíọ́m Skín Ton',
    '🧑🏾‍🚀' => 'Ástrọ́nọt: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🚀' => 'Ástrọ́nọt: Dák Skín Ton',
    '👨🏻‍🚀' => 'Mán Ástrọ́nọt: Laít Skín Ton',
    '👨🏼‍🚀' => 'Mán Ástrọ́nọt: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🚀' => 'Mán Ástrọ́nọt: Mídíọ́m Skín Ton',
    '👨🏾‍🚀' => 'Mán Ástrọ́nọt: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🚀' => 'Mán Ástrọ́nọt: Dák Skín Ton',
    '👩🏻‍🚀' => 'Wúmán Ástrọ́nọt: Laít Skín Ton',
    '👩🏼‍🚀' => 'Wúmán Ástrọ́nọt: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🚀' => 'Wúmán Ástrọ́nọt: Mídíọ́m Skín Ton',
    '👩🏾‍🚀' => 'Wúmán Ástrọ́nọt: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🚀' => 'Wúmán Ástrọ́nọt: Dák Skín Ton',
    '🧑🏻‍🚒' => 'Fáyáfaíta: Laít Skín Ton',
    '🧑🏼‍🚒' => 'Fáyáfaíta: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🚒' => 'Fáyáfaíta: Mídíọ́m Skín Ton',
    '🧑🏾‍🚒' => 'Fáyáfaíta: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🚒' => 'Fáyáfaíta: Dák Skín Ton',
    '👨🏻‍🚒' => 'Mán Fáyáfaíta: Laít Skín Ton',
    '👨🏼‍🚒' => 'Mán Fáyáfaíta: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🚒' => 'Mán Fáyáfaíta: Mídíọ́m Skín Ton',
    '👨🏾‍🚒' => 'Mán Fáyáfaíta: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🚒' => 'Mán Fáyáfaíta: Dák Skín Ton',
    '👩🏻‍🚒' => 'Wúmán Fáyáfaíta: Laít Skín Ton',
    '👩🏼‍🚒' => 'Wúmán Fáyáfaíta: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🚒' => 'Wúmán Fáyáfaíta: Mídíọ́m Skín Ton',
    '👩🏾‍🚒' => 'Wúmán Fáyáfaíta: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🚒' => 'Wúmán Fáyáfaíta: Dák Skín Ton',
    '👮🏻‍♂' => 'Man Polís Ọ́físa: Laít Skín Ton',
    '👮🏼‍♂' => 'Man Polís Ọ́físa: Mídíọ́m-Laít Skín Ton',
    '👮🏽‍♂' => 'Man Polís Ọ́físa: Mídíọ́m Skín Ton',
    '👮🏾‍♂' => 'Man Polís Ọ́físa: Mídíọ́m Dák Skín Ton',
    '👮🏿‍♂' => 'Man Polís Ọ́físa: Dák Skín Ton',
    '👮🏻‍♀' => 'Wúmán Polís Ọ́físa: Laít Skín Ton',
    '👮🏼‍♀' => 'Wúmán Polís Ọ́físa: Mídíọ́m-Laít Skín Ton',
    '👮🏽‍♀' => 'Wúmán Polís Ọ́físa: Mídíọ́m Skín Ton',
    '👮🏾‍♀' => 'Wúmán Polís Ọ́físa: Mídíọ́m Dák Skín Ton',
    '👮🏿‍♀' => 'Wúmán Polís Ọ́físa: Dák Skín Ton',
    '🕵🏻‍♂' => 'Mán Ditẹ́ktiv: Laít Skín Ton',
    '🕵🏼‍♂' => 'Mán Ditẹ́ktiv: Mídíọ́m-Laít Skín Ton',
    '🕵🏽‍♂' => 'Mán Ditẹ́ktiv: Mídíọ́m Skín Ton',
    '🕵🏾‍♂' => 'Mán Ditẹ́ktiv: Mídíọ́m Dák Skín Ton',
    '🕵🏿‍♂' => 'Mán Ditẹ́ktiv: Dák Skín Ton',
    '🕵🏻‍♀' => 'Wúmán Ditẹ́ktiv: Laít Skín Ton',
    '🕵🏼‍♀' => 'Wúmán Ditẹ́ktiv: Mídíọ́m-Laít Skín Ton',
    '🕵🏽‍♀' => 'Wúmán Ditẹ́ktiv: Mídíọ́m Skín Ton',
    '🕵🏾‍♀' => 'Wúmán Ditẹ́ktiv: Mídíọ́m Dák Skín Ton',
    '🕵🏿‍♀' => 'Wúmán Ditẹ́ktiv: Dák Skín Ton',
    '💂🏻‍♂' => 'Mán Gad: Laít Skín Ton',
    '💂🏼‍♂' => 'Mán Gad: Mídíọ́m-Laít Skín Ton',
    '💂🏽‍♂' => 'Mán Gad: Mídíọ́m Skín Ton',
    '💂🏾‍♂' => 'Mán Gad: Mídíọ́m Dák Skín Ton',
    '💂🏿‍♂' => 'Mán Gad: Dák Skín Ton',
    '💂🏻‍♀' => 'Wúmán Gad: Laít Skín Ton',
    '💂🏼‍♀' => 'Wúmán Gad: Mídíọ́m-Laít Skín Ton',
    '💂🏽‍♀' => 'Wúmán Gad: Mídíọ́m Skín Ton',
    '💂🏾‍♀' => 'Wúmán Gad: Mídíọ́m Dák Skín Ton',
    '💂🏿‍♀' => 'Wúmán Gad: Dák Skín Ton',
    '👷🏻‍♂' => 'Mán Kọnstrọ́kshọ́n Wọ́ka: Laít Skín Ton',
    '👷🏼‍♂' => 'Mán Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '👷🏽‍♂' => 'Mán Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m Skín Ton',
    '👷🏾‍♂' => 'Mán Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m Dák Skín Ton',
    '👷🏿‍♂' => 'Mán Kọnstrọ́kshọ́n Wọ́ka: Dák Skín Ton',
    '👷🏻‍♀' => 'Wúmán Kọnstrọ́kshọ́n Wọ́ka: Laít Skín Ton',
    '👷🏼‍♀' => 'Wúmán Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '👷🏽‍♀' => 'Wúmán Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m Skín Ton',
    '👷🏾‍♀' => 'Wúmán Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m Dák Skín Ton',
    '👷🏿‍♀' => 'Wúmán Kọnstrọ́kshọ́n Wọ́ka: Dák Skín Ton',
    '👳🏻‍♂' => 'Man Wé Taí Hẹd-Tai Fọ Hẹd: Laít Skín Ton',
    '👳🏼‍♂' => 'Man Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m-Laít Skín Ton',
    '👳🏽‍♂' => 'Man Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m Skín Ton',
    '👳🏾‍♂' => 'Man Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m Dák Skín Ton',
    '👳🏿‍♂' => 'Man Wé Taí Hẹd-Tai Fọ Hẹd: Dák Skín Ton',
    '👳🏻‍♀' => 'Wúman Wé Taí Hẹd-Tai Fọ Hẹd: Laít Skín Ton',
    '👳🏼‍♀' => 'Wúman Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m-Laít Skín Ton',
    '👳🏽‍♀' => 'Wúman Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m Skín Ton',
    '👳🏾‍♀' => 'Wúman Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m Dák Skín Ton',
    '👳🏿‍♀' => 'Wúman Wé Taí Hẹd-Tai Fọ Hẹd: Dák Skín Ton',
    '🤵🏻‍♂' => 'Man Wé Wiá Tọksído: Laít Skín Ton',
    '🤵🏼‍♂' => 'Man Wé Wiá Tọksído: Mídíọ́m-Laít Skín Ton',
    '🤵🏽‍♂' => 'Man Wé Wiá Tọksído: Mídíọ́m Skín Ton',
    '🤵🏾‍♂' => 'Man Wé Wiá Tọksído: Mídíọ́m Dák Skín Ton',
    '🤵🏿‍♂' => 'Man Wé Wiá Tọksído: Dák Skín Ton',
    '🤵🏻‍♀' => 'Wúman Wé Wiá Tọksído: Laít Skín Ton',
    '🤵🏼‍♀' => 'Wúman Wé Wiá Tọksído: Mídíọ́m-Laít Skín Ton',
    '🤵🏽‍♀' => 'Wúman Wé Wiá Tọksído: Mídíọ́m Skín Ton',
    '🤵🏾‍♀' => 'Wúman Wé Wiá Tọksído: Mídíọ́m Dák Skín Ton',
    '🤵🏿‍♀' => 'Wúman Wé Wiá Tọksído: Dák Skín Ton',
    '👰🏻‍♂' => 'Man Wé Wiá Vẹl: Laít Skín Ton',
    '👰🏼‍♂' => 'Man Wé Wiá Vẹl: Mídíọ́m-Laít Skín Ton',
    '👰🏽‍♂' => 'Man Wé Wiá Vẹl: Mídíọ́m Skín Ton',
    '👰🏾‍♂' => 'Man Wé Wiá Vẹl: Mídíọ́m Dák Skín Ton',
    '👰🏿‍♂' => 'Man Wé Wiá Vẹl: Dák Skín Ton',
    '👰🏻‍♀' => 'Wúman Wé Wiá Vẹl: Laít Skín Ton',
    '👰🏼‍♀' => 'Wúman Wé Wiá Vẹl: Mídíọ́m-Laít Skín Ton',
    '👰🏽‍♀' => 'Wúman Wé Wiá Vẹl: Mídíọ́m Skín Ton',
    '👰🏾‍♀' => 'Wúman Wé Wiá Vẹl: Mídíọ́m Dák Skín Ton',
    '👰🏿‍♀' => 'Wúman Wé Wiá Vẹl: Dák Skín Ton',
    '👩🏻‍🍼' => 'Wúman Wé De Gív Pikin Chọp: Laít Skín Ton',
    '👩🏼‍🍼' => 'Wúman Wé De Gív Pikin Chọp: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🍼' => 'Wúman Wé De Gív Pikin Chọp: Mídíọ́m Skín Ton',
    '👩🏾‍🍼' => 'Wúman Wé De Gív Pikin Chọp: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🍼' => 'Wúman Wé De Gív Pikin Chọp: Dák Skín Ton',
    '👨🏻‍🍼' => 'Man Wé De Gív Pikin Chọp: Laít Skín Ton',
    '👨🏼‍🍼' => 'Man Wé De Gív Pikin Chọp: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🍼' => 'Man Wé De Gív Pikin Chọp: Mídíọ́m Skín Ton',
    '👨🏾‍🍼' => 'Man Wé De Gív Pikin Chọp: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🍼' => 'Man Wé De Gív Pikin Chọp: Dák Skín Ton',
    '🧑🏻‍🍼' => 'Pẹ́sin Wé De Gív Pikin Chọp: Laít Skín Ton',
    '🧑🏼‍🍼' => 'Pẹ́sin Wé De Gív Pikin Chọp: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🍼' => 'Pẹ́sin Wé De Gív Pikin Chọp: Mídíọ́m Skín Ton',
    '🧑🏾‍🍼' => 'Pẹ́sin Wé De Gív Pikin Chọp: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🍼' => 'Pẹ́sin Wé De Gív Pikin Chọp: Dák Skín Ton',
    '🧑🏻‍🎄' => 'Mx Klọz: Laít Skín Ton',
    '🧑🏼‍🎄' => 'Mx Klọz: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🎄' => 'Mx Klọz: Mídíọ́m Skín Ton',
    '🧑🏾‍🎄' => 'Mx Klọz: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🎄' => 'Mx Klọz: Dák Skín Ton',
    '🦸🏻‍♂' => 'Mán Supahíro: Laít Skín Ton',
    '🦸🏼‍♂' => 'Mán Supahíro: Mídíọ́m-Laít Skín Ton',
    '🦸🏽‍♂' => 'Mán Supahíro: Mídíọ́m Skín Ton',
    '🦸🏾‍♂' => 'Mán Supahíro: Mídíọ́m Dák Skín Ton',
    '🦸🏿‍♂' => 'Mán Supahíro: Dák Skín Ton',
    '🦸🏻‍♀' => 'Wúmán Supahíro: Laít Skín Ton',
    '🦸🏼‍♀' => 'Wúmán Supahíro: Mídíọ́m-Laít Skín Ton',
    '🦸🏽‍♀' => 'Wúmán Supahíro: Mídíọ́m Skín Ton',
    '🦸🏾‍♀' => 'Wúmán Supahíro: Mídíọ́m Dák Skín Ton',
    '🦸🏿‍♀' => 'Wúmán Supahíro: Dák Skín Ton',
    '🦹🏻‍♂' => 'Wíkẹ́d Mán Krímínal: Laít Skín Ton',
    '🦹🏼‍♂' => 'Wíkẹ́d Mán Krímínal: Mídíọ́m-Laít Skín Ton',
    '🦹🏽‍♂' => 'Wíkẹ́d Mán Krímínal: Mídíọ́m Skín Ton',
    '🦹🏾‍♂' => 'Wíkẹ́d Mán Krímínal: Mídíọ́m Dák Skín Ton',
    '🦹🏿‍♂' => 'Wíkẹ́d Mán Krímínal: Dák Skín Ton',
    '🦹🏻‍♀' => 'Wíkẹ́d Wúmán Krímínal: Laít Skín Ton',
    '🦹🏼‍♀' => 'Wíkẹ́d Wúmán Krímínal: Mídíọ́m-Laít Skín Ton',
    '🦹🏽‍♀' => 'Wíkẹ́d Wúmán Krímínal: Mídíọ́m Skín Ton',
    '🦹🏾‍♀' => 'Wíkẹ́d Wúmán Krímínal: Mídíọ́m Dák Skín Ton',
    '🦹🏿‍♀' => 'Wíkẹ́d Wúmán Krímínal: Dák Skín Ton',
    '🧙🏻‍♂' => 'Mán Majíshan: Laít Skín Ton',
    '🧙🏼‍♂' => 'Mán Majíshan: Mídíọ́m-Laít Skín Ton',
    '🧙🏽‍♂' => 'Mán Majíshan: Mídíọ́m Skín Ton',
    '🧙🏾‍♂' => 'Mán Majíshan: Mídíọ́m Dák Skín Ton',
    '🧙🏿‍♂' => 'Mán Majíshan: Dák Skín Ton',
    '🧙🏻‍♀' => 'Wúmán Majíshan: Laít Skín Ton',
    '🧙🏼‍♀' => 'Wúmán Majíshan: Mídíọ́m-Laít Skín Ton',
    '🧙🏽‍♀' => 'Wúmán Majíshan: Mídíọ́m Skín Ton',
    '🧙🏾‍♀' => 'Wúmán Majíshan: Mídíọ́m Dák Skín Ton',
    '🧙🏿‍♀' => 'Wúmán Majíshan: Dák Skín Ton',
    '🧚🏻‍♂' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Man: Laít Skín Ton',
    '🧚🏼‍♂' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Man: Mídíọ́m-Laít Skín Ton',
    '🧚🏽‍♂' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Man: Mídíọ́m Skín Ton',
    '🧚🏾‍♂' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Man: Mídíọ́m Dák Skín Ton',
    '🧚🏿‍♂' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Man: Dák Skín Ton',
    '🧚🏻‍♀' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Wúmán: Laít Skín Ton',
    '🧚🏼‍♀' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Wúmán: Mídíọ́m-Laít Skín Ton',
    '🧚🏽‍♀' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Wúmán: Mídíọ́m Skín Ton',
    '🧚🏾‍♀' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Wúmán: Mídíọ́m Dák Skín Ton',
    '🧚🏿‍♀' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Wúmán: Dák Skín Ton',
    '🧛🏻‍♂' => 'Mán Vámpáya: Laít Skín Ton',
    '🧛🏼‍♂' => 'Mán Vámpáya: Mídíọ́m-Laít Skín Ton',
    '🧛🏽‍♂' => 'Mán Vámpáya: Mídíọ́m Skín Ton',
    '🧛🏾‍♂' => 'Mán Vámpáya: Mídíọ́m Dák Skín Ton',
    '🧛🏿‍♂' => 'Mán Vámpáya: Dák Skín Ton',
    '🧛🏻‍♀' => 'Wúmán Vámpáya: Laít Skín Ton',
    '🧛🏼‍♀' => 'Wúmán Vámpáya: Mídíọ́m-Laít Skín Ton',
    '🧛🏽‍♀' => 'Wúmán Vámpáya: Mídíọ́m Skín Ton',
    '🧛🏾‍♀' => 'Wúmán Vámpáya: Mídíọ́m Dák Skín Ton',
    '🧛🏿‍♀' => 'Wúmán Vámpáya: Dák Skín Ton',
    '🧜🏻‍♂' => 'Mán Mamíwọtá: Laít Skín Ton',
    '🧜🏼‍♂' => 'Mán Mamíwọtá: Mídíọ́m-Laít Skín Ton',
    '🧜🏽‍♂' => 'Mán Mamíwọtá: Mídíọ́m Skín Ton',
    '🧜🏾‍♂' => 'Mán Mamíwọtá: Mídíọ́m Dák Skín Ton',
    '🧜🏿‍♂' => 'Mán Mamíwọtá: Dák Skín Ton',
    '🧜🏻‍♀' => 'Wúmán Mamíwọtá: Laít Skín Ton',
    '🧜🏼‍♀' => 'Wúmán Mamíwọtá: Mídíọ́m-Laít Skín Ton',
    '🧜🏽‍♀' => 'Wúmán Mamíwọtá: Mídíọ́m Skín Ton',
    '🧜🏾‍♀' => 'Wúmán Mamíwọtá: Mídíọ́m Dák Skín Ton',
    '🧜🏿‍♀' => 'Wúmán Mamíwọtá: Dák Skín Ton',
    '🧝🏻‍♂' => 'Mán Ẹlf: Laít Skín Ton',
    '🧝🏼‍♂' => 'Mán Ẹlf: Mídíọ́m-Laít Skín Ton',
    '🧝🏽‍♂' => 'Mán Ẹlf: Mídíọ́m Skín Ton',
    '🧝🏾‍♂' => 'Mán Ẹlf: Mídíọ́m Dák Skín Ton',
    '🧝🏿‍♂' => 'Mán Ẹlf: Dák Skín Ton',
    '🧝🏻‍♀' => 'Wúmán Ẹlf: Laít Skín Ton',
    '🧝🏼‍♀' => 'Wúmán Ẹlf: Mídíọ́m-Laít Skín Ton',
    '🧝🏽‍♀' => 'Wúmán Ẹlf: Mídíọ́m Skín Ton',
    '🧝🏾‍♀' => 'Wúmán Ẹlf: Mídíọ́m Dák Skín Ton',
    '🧝🏿‍♀' => 'Wúmán Ẹlf: Dák Skín Ton',
    '💆🏻‍♂' => 'Man Wé Dẹm De Masáj Im Bọdí: Laít Skín Ton',
    '💆🏼‍♂' => 'Man Wé Dẹm De Masáj Im Bọdí: Mídíọ́m-Laít Skín Ton',
    '💆🏽‍♂' => 'Man Wé Dẹm De Masáj Im Bọdí: Mídíọ́m Skín Ton',
    '💆🏾‍♂' => 'Man Wé Dẹm De Masáj Im Bọdí: Mídíọ́m Dák Skín Ton',
    '💆🏿‍♂' => 'Man Wé Dẹm De Masáj Im Bọdí: Dák Skín Ton',
    '💆🏻‍♀' => 'Wúman Wé Dẹm De Masáj Im Bọdí: Laít Skín Ton',
    '💆🏼‍♀' => 'Wúman Wé Dẹm De Masáj Im Bọdí: Mídíọ́m-Laít Skín Ton',
    '💆🏽‍♀' => 'Wúman Wé Dẹm De Masáj Im Bọdí: Mídíọ́m Skín Ton',
    '💆🏾‍♀' => 'Wúman Wé Dẹm De Masáj Im Bọdí: Mídíọ́m Dák Skín Ton',
    '💆🏿‍♀' => 'Wúman Wé Dẹm De Masáj Im Bọdí: Dák Skín Ton',
    '💇🏻‍♂' => 'Man Wé Dẹm De Báb Im Hea: Laít Skín Ton',
    '💇🏼‍♂' => 'Man Wé Dẹm De Báb Im Hea: Mídíọ́m-Laít Skín Ton',
    '💇🏽‍♂' => 'Man Wé Dẹm De Báb Im Hea: Mídíọ́m Skín Ton',
    '💇🏾‍♂' => 'Man Wé Dẹm De Báb Im Hea: Mídíọ́m Dák Skín Ton',
    '💇🏿‍♂' => 'Man Wé Dẹm De Báb Im Hea: Dák Skín Ton',
    '💇🏻‍♀' => 'Wúman Wé Dẹm De Báb Im Hea: Laít Skín Ton',
    '💇🏼‍♀' => 'Wúman Wé Dẹm De Báb Im Hea: Mídíọ́m-Laít Skín Ton',
    '💇🏽‍♀' => 'Wúman Wé Dẹm De Báb Im Hea: Mídíọ́m Skín Ton',
    '💇🏾‍♀' => 'Wúman Wé Dẹm De Báb Im Hea: Mídíọ́m Dák Skín Ton',
    '💇🏿‍♀' => 'Wúman Wé Dẹm De Báb Im Hea: Dák Skín Ton',
    '🚶🏻‍♂' => 'Man Wé De Waká: Laít Skín Ton',
    '🚶🏼‍♂' => 'Man Wé De Waká: Mídíọ́m-Laít Skín Ton',
    '🚶🏽‍♂' => 'Man Wé De Waká: Mídíọ́m Skín Ton',
    '🚶🏾‍♂' => 'Man Wé De Waká: Mídíọ́m Dák Skín Ton',
    '🚶🏿‍♂' => 'Man Wé De Waká: Dák Skín Ton',
    '🚶🏻‍♀' => 'Wúman Wé De Waká: Laít Skín Ton',
    '🚶🏼‍♀' => 'Wúman Wé De Waká: Mídíọ́m-Laít Skín Ton',
    '🚶🏽‍♀' => 'Wúman Wé De Waká: Mídíọ́m Skín Ton',
    '🚶🏾‍♀' => 'Wúman Wé De Waká: Mídíọ́m Dák Skín Ton',
    '🚶🏿‍♀' => 'Wúman Wé De Waká: Dák Skín Ton',
    '🚶🏻‍➡' => 'Pẹ́sin Wé De Waká: Laít Skín Ton, Fẹsin ráít',
    '🚶🏼‍➡' => 'Pẹ́sin Wé De Waká: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🚶🏽‍➡' => 'Pẹ́sin Wé De Waká: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🚶🏾‍➡' => 'Pẹ́sin Wé De Waká: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🚶🏿‍➡' => 'Pẹ́sin Wé De Waká: Dák Skín Ton, Fẹsin ráít',
    '🧍🏻‍♂' => 'Man Wé De Stánd Ọp: Laít Skín Ton',
    '🧍🏼‍♂' => 'Man Wé De Stánd Ọp: Mídíọ́m-Laít Skín Ton',
    '🧍🏽‍♂' => 'Man Wé De Stánd Ọp: Mídíọ́m Skín Ton',
    '🧍🏾‍♂' => 'Man Wé De Stánd Ọp: Mídíọ́m Dák Skín Ton',
    '🧍🏿‍♂' => 'Man Wé De Stánd Ọp: Dák Skín Ton',
    '🧍🏻‍♀' => 'Wúman Wé De Stánd Ọp: Laít Skín Ton',
    '🧍🏼‍♀' => 'Wúman Wé De Stánd Ọp: Mídíọ́m-Laít Skín Ton',
    '🧍🏽‍♀' => 'Wúman Wé De Stánd Ọp: Mídíọ́m Skín Ton',
    '🧍🏾‍♀' => 'Wúman Wé De Stánd Ọp: Mídíọ́m Dák Skín Ton',
    '🧍🏿‍♀' => 'Wúman Wé De Stánd Ọp: Dák Skín Ton',
    '🧎🏻‍♂' => 'Man Wé Níl Daun: Laít Skín Ton',
    '🧎🏼‍♂' => 'Man Wé Níl Daun: Mídíọ́m-Laít Skín Ton',
    '🧎🏽‍♂' => 'Man Wé Níl Daun: Mídíọ́m Skín Ton',
    '🧎🏾‍♂' => 'Man Wé Níl Daun: Mídíọ́m Dák Skín Ton',
    '🧎🏿‍♂' => 'Man Wé Níl Daun: Dák Skín Ton',
    '🧎🏻‍♀' => 'Wúman Wé Níl Daun: Laít Skín Ton',
    '🧎🏼‍♀' => 'Wúman Wé Níl Daun: Mídíọ́m-Laít Skín Ton',
    '🧎🏽‍♀' => 'Wúman Wé Níl Daun: Mídíọ́m Skín Ton',
    '🧎🏾‍♀' => 'Wúman Wé Níl Daun: Mídíọ́m Dák Skín Ton',
    '🧎🏿‍♀' => 'Wúman Wé Níl Daun: Dák Skín Ton',
    '🧎🏻‍➡' => 'Pẹ́sin Wé Níl Daun: Laít Skín Ton, Fẹsin ráít',
    '🧎🏼‍➡' => 'Pẹ́sin Wé Níl Daun: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🧎🏽‍➡' => 'Pẹ́sin Wé Níl Daun: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🧎🏾‍➡' => 'Pẹ́sin Wé Níl Daun: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🧎🏿‍➡' => 'Pẹ́sin Wé Níl Daun: Dák Skín Ton, Fẹsin ráít',
    '🧑🏻‍🦯' => 'Pẹ́sin Wit Waít Wọkín-Stik: Laít Skín Ton',
    '🧑🏼‍🦯' => 'Pẹ́sin Wit Waít Wọkín-Stik: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🦯' => 'Pẹ́sin Wit Waít Wọkín-Stik: Mídíọ́m Skín Ton',
    '🧑🏾‍🦯' => 'Pẹ́sin Wit Waít Wọkín-Stik: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🦯' => 'Pẹ́sin Wit Waít Wọkín-Stik: Dák Skín Ton',
    '👨🏻‍🦯' => 'Man Wit Waít Wọkín-Stik: Laít Skín Ton',
    '👨🏼‍🦯' => 'Man Wit Waít Wọkín-Stik: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🦯' => 'Man Wit Waít Wọkín-Stik: Mídíọ́m Skín Ton',
    '👨🏾‍🦯' => 'Man Wit Waít Wọkín-Stik: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🦯' => 'Man Wit Waít Wọkín-Stik: Dák Skín Ton',
    '👩🏻‍🦯' => 'Wúman Wit Waít Wọkín-Stik: Laít Skín Ton',
    '👩🏼‍🦯' => 'Wúman Wit Waít Wọkín-Stik: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🦯' => 'Wúman Wit Waít Wọkín-Stik: Mídíọ́m Skín Ton',
    '👩🏾‍🦯' => 'Wúman Wit Waít Wọkín-Stik: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🦯' => 'Wúman Wit Waít Wọkín-Stik: Dák Skín Ton',
    '🧑🏻‍🦼' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Laít Skín Ton',
    '🧑🏼‍🦼' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🦼' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton',
    '🧑🏾‍🦼' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🦼' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Dák Skín Ton',
    '👨🏻‍🦼' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Laít Skín Ton',
    '👨🏼‍🦼' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🦼' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton',
    '👨🏾‍🦼' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🦼' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Dák Skín Ton',
    '👩🏻‍🦼' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Laít Skín Ton',
    '👩🏼‍🦼' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🦼' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton',
    '👩🏾‍🦼' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🦼' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin: Dák Skín Ton',
    '🧑🏻‍🦽' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Laít Skín Ton',
    '🧑🏼‍🦽' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton',
    '🧑🏽‍🦽' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton',
    '🧑🏾‍🦽' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton',
    '🧑🏿‍🦽' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Dák Skín Ton',
    '👨🏻‍🦽' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Laít Skín Ton',
    '👨🏼‍🦽' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton',
    '👨🏽‍🦽' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton',
    '👨🏾‍🦽' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton',
    '👨🏿‍🦽' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Dák Skín Ton',
    '👩🏻‍🦽' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Laít Skín Ton',
    '👩🏼‍🦽' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m-Laít Skín Ton',
    '👩🏽‍🦽' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Skín Ton',
    '👩🏾‍🦽' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Mídíọ́m Dák Skín Ton',
    '👩🏿‍🦽' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin: Dák Skín Ton',
    '🏃🏻‍♂' => 'Man Wé De Rọn: Laít Skín Ton',
    '🏃🏼‍♂' => 'Man Wé De Rọn: Mídíọ́m-Laít Skín Ton',
    '🏃🏽‍♂' => 'Man Wé De Rọn: Mídíọ́m Skín Ton',
    '🏃🏾‍♂' => 'Man Wé De Rọn: Mídíọ́m Dák Skín Ton',
    '🏃🏿‍♂' => 'Man Wé De Rọn: Dák Skín Ton',
    '🏃🏻‍♀' => 'Wúman Wé De Rọn: Laít Skín Ton',
    '🏃🏼‍♀' => 'Wúman Wé De Rọn: Mídíọ́m-Laít Skín Ton',
    '🏃🏽‍♀' => 'Wúman Wé De Rọn: Mídíọ́m Skín Ton',
    '🏃🏾‍♀' => 'Wúman Wé De Rọn: Mídíọ́m Dák Skín Ton',
    '🏃🏿‍♀' => 'Wúman Wé De Rọn: Dák Skín Ton',
    '🏃🏻‍➡' => 'Pẹ́sin Wé De Rọn: Laít Skín Ton, Fẹsin ráít',
    '🏃🏼‍➡' => 'Pẹ́sin Wé De Rọn: Mídíọ́m-Laít Skín Ton, Fẹsin ráít',
    '🏃🏽‍➡' => 'Pẹ́sin Wé De Rọn: Mídíọ́m Skín Ton, Fẹsin ráít',
    '🏃🏾‍➡' => 'Pẹ́sin Wé De Rọn: Mídíọ́m Dák Skín Ton, Fẹsin ráít',
    '🏃🏿‍➡' => 'Pẹ́sin Wé De Rọn: Dák Skín Ton, Fẹsin ráít',
    '👯🏻‍♀' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton',
    '👯🏼‍♀' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton',
    '👯🏽‍♀' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton',
    '👯🏾‍♀' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton',
    '👯🏿‍♀' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton',
    '👯🏻‍♂' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Laít Skín Ton',
    '👯🏼‍♂' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m-Laít Skín Ton',
    '👯🏽‍♂' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Skín Ton',
    '👯🏾‍♂' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Mídíọ́m Dák Skín Ton',
    '👯🏿‍♂' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins: Dák Skín Ton',
    '🧖🏻‍♂' => 'Man Wé Dé Ínsaíd Họt Wọtá Smók Rum: Laít Skín Ton',
    '🧖🏼‍♂' => 'Man Wé Dé Ínsaíd Họt Wọtá Smók Rum: Mídíọ́m-Laít Skín Ton',
    '🧖🏽‍♂' => 'Man Wé Dé Ínsaíd Họt Wọtá Smók Rum: Mídíọ́m Skín Ton',
    '🧖🏾‍♂' => 'Man Wé Dé Ínsaíd Họt Wọtá Smók Rum: Mídíọ́m Dák Skín Ton',
    '🧖🏿‍♂' => 'Man Wé Dé Ínsaíd Họt Wọtá Smók Rum: Dák Skín Ton',
    '🧖🏻‍♀' => 'Wúman Wé Dé Ínsaíd Họt Wọtá Smók Rum: Laít Skín Ton',
    '🧖🏼‍♀' => 'Wúman Wé Dé Ínsaíd Họt Wọtá Smók Rum: Mídíọ́m-Laít Skín Ton',
    '🧖🏽‍♀' => 'Wúman Wé Dé Ínsaíd Họt Wọtá Smók Rum: Mídíọ́m Skín Ton',
    '🧖🏾‍♀' => 'Wúman Wé Dé Ínsaíd Họt Wọtá Smók Rum: Mídíọ́m Dák Skín Ton',
    '🧖🏿‍♀' => 'Wúman Wé Dé Ínsaíd Họt Wọtá Smók Rum: Dák Skín Ton',
    '🧗🏻‍♂' => 'Man Wé De Klaimb: Laít Skín Ton',
    '🧗🏼‍♂' => 'Man Wé De Klaimb: Mídíọ́m-Laít Skín Ton',
    '🧗🏽‍♂' => 'Man Wé De Klaimb: Mídíọ́m Skín Ton',
    '🧗🏾‍♂' => 'Man Wé De Klaimb: Mídíọ́m Dák Skín Ton',
    '🧗🏿‍♂' => 'Man Wé De Klaimb: Dák Skín Ton',
    '🧗🏻‍♀' => 'Wúman Wé De Klaimb: Laít Skín Ton',
    '🧗🏼‍♀' => 'Wúman Wé De Klaimb: Mídíọ́m-Laít Skín Ton',
    '🧗🏽‍♀' => 'Wúman Wé De Klaimb: Mídíọ́m Skín Ton',
    '🧗🏾‍♀' => 'Wúman Wé De Klaimb: Mídíọ́m Dák Skín Ton',
    '🧗🏿‍♀' => 'Wúman Wé De Klaimb: Dák Skín Ton',
    '🏌🏻‍♂' => 'Man Wé De Plé Gọlf: Laít Skín Ton',
    '🏌🏼‍♂' => 'Man Wé De Plé Gọlf: Mídíọ́m-Laít Skín Ton',
    '🏌🏽‍♂' => 'Man Wé De Plé Gọlf: Mídíọ́m Skín Ton',
    '🏌🏾‍♂' => 'Man Wé De Plé Gọlf: Mídíọ́m Dák Skín Ton',
    '🏌🏿‍♂' => 'Man Wé De Plé Gọlf: Dák Skín Ton',
    '🏌🏻‍♀' => 'Wúman Wé De Plé Gọlf: Laít Skín Ton',
    '🏌🏼‍♀' => 'Wúman Wé De Plé Gọlf: Mídíọ́m-Laít Skín Ton',
    '🏌🏽‍♀' => 'Wúman Wé De Plé Gọlf: Mídíọ́m Skín Ton',
    '🏌🏾‍♀' => 'Wúman Wé De Plé Gọlf: Mídíọ́m Dák Skín Ton',
    '🏌🏿‍♀' => 'Wúman Wé De Plé Gọlf: Dák Skín Ton',
    '🏄🏻‍♂' => 'Man Wé De Raíd Wọtá Sọ́fín-Bọd: Laít Skín Ton',
    '🏄🏼‍♂' => 'Man Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m-Laít Skín Ton',
    '🏄🏽‍♂' => 'Man Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m Skín Ton',
    '🏄🏾‍♂' => 'Man Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m Dák Skín Ton',
    '🏄🏿‍♂' => 'Man Wé De Raíd Wọtá Sọ́fín-Bọd: Dák Skín Ton',
    '🏄🏻‍♀' => 'Wúman Wé De Raíd Wọtá Sọ́fín-Bọd: Laít Skín Ton',
    '🏄🏼‍♀' => 'Wúman Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m-Laít Skín Ton',
    '🏄🏽‍♀' => 'Wúman Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m Skín Ton',
    '🏄🏾‍♀' => 'Wúman Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m Dák Skín Ton',
    '🏄🏿‍♀' => 'Wúman Wé De Raíd Wọtá Sọ́fín-Bọd: Dák Skín Ton',
    '🚣🏻‍♂' => 'Man Wé De Ró Bot: Laít Skín Ton',
    '🚣🏼‍♂' => 'Man Wé De Ró Bot: Mídíọ́m-Laít Skín Ton',
    '🚣🏽‍♂' => 'Man Wé De Ró Bot: Mídíọ́m Skín Ton',
    '🚣🏾‍♂' => 'Man Wé De Ró Bot: Mídíọ́m Dák Skín Ton',
    '🚣🏿‍♂' => 'Man Wé De Ró Bot: Dák Skín Ton',
    '🚣🏻‍♀' => 'Wúman Wé De Ró Bot: Laít Skín Ton',
    '🚣🏼‍♀' => 'Wúman Wé De Ró Bot: Mídíọ́m-Laít Skín Ton',
    '🚣🏽‍♀' => 'Wúman Wé De Ró Bot: Mídíọ́m Skín Ton',
    '🚣🏾‍♀' => 'Wúman Wé De Ró Bot: Mídíọ́m Dák Skín Ton',
    '🚣🏿‍♀' => 'Wúman Wé De Ró Bot: Dák Skín Ton',
    '🏊🏻‍♂' => 'Man Wé De Swim: Laít Skín Ton',
    '🏊🏼‍♂' => 'Man Wé De Swim: Mídíọ́m-Laít Skín Ton',
    '🏊🏽‍♂' => 'Man Wé De Swim: Mídíọ́m Skín Ton',
    '🏊🏾‍♂' => 'Man Wé De Swim: Mídíọ́m Dák Skín Ton',
    '🏊🏿‍♂' => 'Man Wé De Swim: Dák Skín Ton',
    '🏊🏻‍♀' => 'Wúman Wé De Swim: Laít Skín Ton',
    '🏊🏼‍♀' => 'Wúman Wé De Swim: Mídíọ́m-Laít Skín Ton',
    '🏊🏽‍♀' => 'Wúman Wé De Swim: Mídíọ́m Skín Ton',
    '🏊🏾‍♀' => 'Wúman Wé De Swim: Mídíọ́m Dák Skín Ton',
    '🏊🏿‍♀' => 'Wúman Wé De Swim: Dák Skín Ton',
    '⛹🏻‍♂' => 'Man Wé De Baúns Bọl Fọ Graun: Laít Skín Ton',
    '⛹🏼‍♂' => 'Man Wé De Baúns Bọl Fọ Graun: Mídíọ́m-Laít Skín Ton',
    '⛹🏽‍♂' => 'Man Wé De Baúns Bọl Fọ Graun: Mídíọ́m Skín Ton',
    '⛹🏾‍♂' => 'Man Wé De Baúns Bọl Fọ Graun: Mídíọ́m Dák Skín Ton',
    '⛹🏿‍♂' => 'Man Wé De Baúns Bọl Fọ Graun: Dák Skín Ton',
    '⛹🏻‍♀' => 'Wúman Wé De Baúns Bọl Fọ Graun: Laít Skín Ton',
    '⛹🏼‍♀' => 'Wúman Wé De Baúns Bọl Fọ Graun: Mídíọ́m-Laít Skín Ton',
    '⛹🏽‍♀' => 'Wúman Wé De Baúns Bọl Fọ Graun: Mídíọ́m Skín Ton',
    '⛹🏾‍♀' => 'Wúman Wé De Baúns Bọl Fọ Graun: Mídíọ́m Dák Skín Ton',
    '⛹🏿‍♀' => 'Wúman Wé De Baúns Bọl Fọ Graun: Dák Skín Ton',
    '🏋🏻‍♂' => 'Man Wé De Kárí Ọ́p Hẹ́ví Tins: Laít Skín Ton',
    '🏋🏼‍♂' => 'Man Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m-Laít Skín Ton',
    '🏋🏽‍♂' => 'Man Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m Skín Ton',
    '🏋🏾‍♂' => 'Man Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m Dák Skín Ton',
    '🏋🏿‍♂' => 'Man Wé De Kárí Ọ́p Hẹ́ví Tins: Dák Skín Ton',
    '🏋🏻‍♀' => 'Wúman Wé De Kárí Ọ́p Hẹ́ví Tins: Laít Skín Ton',
    '🏋🏼‍♀' => 'Wúman Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m-Laít Skín Ton',
    '🏋🏽‍♀' => 'Wúman Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m Skín Ton',
    '🏋🏾‍♀' => 'Wúman Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m Dák Skín Ton',
    '🏋🏿‍♀' => 'Wúman Wé De Kárí Ọ́p Hẹ́ví Tins: Dák Skín Ton',
    '🚴🏻‍♂' => 'Man Wé De Raíd Baísíkul: Laít Skín Ton',
    '🚴🏼‍♂' => 'Man Wé De Raíd Baísíkul: Mídíọ́m-Laít Skín Ton',
    '🚴🏽‍♂' => 'Man Wé De Raíd Baísíkul: Mídíọ́m Skín Ton',
    '🚴🏾‍♂' => 'Man Wé De Raíd Baísíkul: Mídíọ́m Dák Skín Ton',
    '🚴🏿‍♂' => 'Man Wé De Raíd Baísíkul: Dák Skín Ton',
    '🚴🏻‍♀' => 'Wúman Wé De Raíd Baísíkul: Laít Skín Ton',
    '🚴🏼‍♀' => 'Wúman Wé De Raíd Baísíkul: Mídíọ́m-Laít Skín Ton',
    '🚴🏽‍♀' => 'Wúman Wé De Raíd Baísíkul: Mídíọ́m Skín Ton',
    '🚴🏾‍♀' => 'Wúman Wé De Raíd Baísíkul: Mídíọ́m Dák Skín Ton',
    '🚴🏿‍♀' => 'Wúman Wé De Raíd Baísíkul: Dák Skín Ton',
    '🚵🏻‍♂' => 'Man Wé De Raíd Baísíkul Fọ Maúntin: Laít Skín Ton',
    '🚵🏼‍♂' => 'Man Wé De Raíd Baísíkul Fọ Maúntin: Mídíọ́m-Laít Skín Ton',
    '🚵🏽‍♂' => 'Man Wé De Raíd Baísíkul Fọ Maúntin: Mídíọ́m Skín Ton',
    '🚵🏾‍♂' => 'Man Wé De Raíd Baísíkul Fọ Maúntin: Mídíọ́m Dák Skín Ton',
    '🚵🏿‍♂' => 'Man Wé De Raíd Baísíkul Fọ Maúntin: Dák Skín Ton',
    '🚵🏻‍♀' => 'Wúman De Raíd Baísíkul Fọ Maúntin: Laít Skín Ton',
    '🚵🏼‍♀' => 'Wúman De Raíd Baísíkul Fọ Maúntin: Mídíọ́m-Laít Skín Ton',
    '🚵🏽‍♀' => 'Wúman De Raíd Baísíkul Fọ Maúntin: Mídíọ́m Skín Ton',
    '🚵🏾‍♀' => 'Wúman De Raíd Baísíkul Fọ Maúntin: Mídíọ́m Dák Skín Ton',
    '🚵🏿‍♀' => 'Wúman De Raíd Baísíkul Fọ Maúntin: Dák Skín Ton',
    '🤸🏻‍♂' => 'Man Wé De Dú Kátwil: Laít Skín Ton',
    '🤸🏼‍♂' => 'Man Wé De Dú Kátwil: Mídíọ́m-Laít Skín Ton',
    '🤸🏽‍♂' => 'Man Wé De Dú Kátwil: Mídíọ́m Skín Ton',
    '🤸🏾‍♂' => 'Man Wé De Dú Kátwil: Mídíọ́m Dák Skín Ton',
    '🤸🏿‍♂' => 'Man Wé De Dú Kátwil: Dák Skín Ton',
    '🤸🏻‍♀' => 'Wúman Wé De Dú Kátwil: Laít Skín Ton',
    '🤸🏼‍♀' => 'Wúman Wé De Dú Kátwil: Mídíọ́m-Laít Skín Ton',
    '🤸🏽‍♀' => 'Wúman Wé De Dú Kátwil: Mídíọ́m Skín Ton',
    '🤸🏾‍♀' => 'Wúman Wé De Dú Kátwil: Mídíọ́m Dák Skín Ton',
    '🤸🏿‍♀' => 'Wúman Wé De Dú Kátwil: Dák Skín Ton',
    '🤼🏻‍♀' => 'Wímẹn Wé De Rẹ́sul: Laít Skín Ton',
    '🤼🏼‍♀' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton',
    '🤼🏽‍♀' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton',
    '🤼🏾‍♀' => 'Wímẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton',
    '🤼🏿‍♀' => 'Wímẹn Wé De Rẹ́sul: Dák Skín Ton',
    '🤼🏻‍♂' => 'Mẹn Wé De Rẹ́sul: Laít Skín Ton',
    '🤼🏼‍♂' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton',
    '🤼🏽‍♂' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Skín Ton',
    '🤼🏾‍♂' => 'Mẹn Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton',
    '🤼🏿‍♂' => 'Mẹn Wé De Rẹ́sul: Dák Skín Ton',
    '🤽🏻‍♂' => 'Man Wé De Plé Wọ́tá Poló: Laít Skín Ton',
    '🤽🏼‍♂' => 'Man Wé De Plé Wọ́tá Poló: Mídíọ́m-Laít Skín Ton',
    '🤽🏽‍♂' => 'Man Wé De Plé Wọ́tá Poló: Mídíọ́m Skín Ton',
    '🤽🏾‍♂' => 'Man Wé De Plé Wọ́tá Poló: Mídíọ́m Dák Skín Ton',
    '🤽🏿‍♂' => 'Man Wé De Plé Wọ́tá Poló: Dák Skín Ton',
    '🤽🏻‍♀' => 'Wúman Wé De Plé Wọ́tá Poló: Laít Skín Ton',
    '🤽🏼‍♀' => 'Wúman Wé De Plé Wọ́tá Poló: Mídíọ́m-Laít Skín Ton',
    '🤽🏽‍♀' => 'Wúman Wé De Plé Wọ́tá Poló: Mídíọ́m Skín Ton',
    '🤽🏾‍♀' => 'Wúman Wé De Plé Wọ́tá Poló: Mídíọ́m Dák Skín Ton',
    '🤽🏿‍♀' => 'Wúman Wé De Plé Wọ́tá Poló: Dák Skín Ton',
    '🤾🏻‍♂' => 'Man Wé De Plé Hándbọl: Laít Skín Ton',
    '🤾🏼‍♂' => 'Man Wé De Plé Hándbọl: Mídíọ́m-Laít Skín Ton',
    '🤾🏽‍♂' => 'Man Wé De Plé Hándbọl: Mídíọ́m Skín Ton',
    '🤾🏾‍♂' => 'Man Wé De Plé Hándbọl: Mídíọ́m Dák Skín Ton',
    '🤾🏿‍♂' => 'Man Wé De Plé Hándbọl: Dák Skín Ton',
    '🤾🏻‍♀' => 'Wúman Wé De Plé Hándbọl: Laít Skín Ton',
    '🤾🏼‍♀' => 'Wúman Wé De Plé Hándbọl: Mídíọ́m-Laít Skín Ton',
    '🤾🏽‍♀' => 'Wúman Wé De Plé Hándbọl: Mídíọ́m Skín Ton',
    '🤾🏾‍♀' => 'Wúman Wé De Plé Hándbọl: Mídíọ́m Dák Skín Ton',
    '🤾🏿‍♀' => 'Wúman Wé De Plé Hándbọl: Dák Skín Ton',
    '🤹🏻‍♂' => 'Man Wé De Jọ́gúl Bọl-Dẹm: Laít Skín Ton',
    '🤹🏼‍♂' => 'Man Wé De Jọ́gúl Bọl-Dẹm: Mídíọ́m-Laít Skín Ton',
    '🤹🏽‍♂' => 'Man Wé De Jọ́gúl Bọl-Dẹm: Mídíọ́m Skín Ton',
    '🤹🏾‍♂' => 'Man Wé De Jọ́gúl Bọl-Dẹm: Mídíọ́m Dák Skín Ton',
    '🤹🏿‍♂' => 'Man Wé De Jọ́gúl Bọl-Dẹm: Dák Skín Ton',
    '🤹🏻‍♀' => 'Wúman Wé De Jọ́gúl Bọl-Dẹm: Laít Skín Ton',
    '🤹🏼‍♀' => 'Wúman Wé De Jọ́gúl Bọl-Dẹm: Mídíọ́m-Laít Skín Ton',
    '🤹🏽‍♀' => 'Wúman Wé De Jọ́gúl Bọl-Dẹm: Mídíọ́m Skín Ton',
    '🤹🏾‍♀' => 'Wúman Wé De Jọ́gúl Bọl-Dẹm: Mídíọ́m Dák Skín Ton',
    '🤹🏿‍♀' => 'Wúman Wé De Jọ́gúl Bọl-Dẹm: Dák Skín Ton',
    '🧘🏻‍♂' => 'Man Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Laít Skín Ton',
    '🧘🏼‍♂' => 'Man Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m-Laít Skín Ton',
    '🧘🏽‍♂' => 'Man Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m Skín Ton',
    '🧘🏾‍♂' => 'Man Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m Dák Skín Ton',
    '🧘🏿‍♂' => 'Man Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Dák Skín Ton',
    '🧘🏻‍♀' => 'Wúman Sidọ́n Fọ Graun Fọ Lótús Wè: Laít Skín Ton',
    '🧘🏼‍♀' => 'Wúman Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m-Laít Skín Ton',
    '🧘🏽‍♀' => 'Wúman Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m Skín Ton',
    '🧘🏾‍♀' => 'Wúman Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m Dák Skín Ton',
    '🧘🏿‍♀' => 'Wúman Sidọ́n Fọ Graun Fọ Lótús Wè: Dák Skín Ton',
    '🧑‍🦰' => 'Pẹ́sin: Rẹ́d Hea',
    '🧑‍🦱' => 'Pẹ́sin: Kọ́lí Hea',
    '🧑‍🦳' => 'Pẹ́sin: Waít Hea',
    '🧑‍🦲' => 'Pẹ́sin: Bọld',
    '👨‍🦰' => 'Man: Rẹ́d Hea',
    '👨‍🦱' => 'Man: Kọ́lí Hea',
    '👨‍🦳' => 'Man: Waít Hea',
    '👨‍🦲' => 'Man: Bọld',
    '👩‍🦰' => 'Wúman: Rẹ́d Hea',
    '👩‍🦱' => 'Wúman: Kọ́lí Hea',
    '👩‍🦳' => 'Wúman: Waít Hea',
    '👩‍🦲' => 'Wúman: Bọld',
    '🚶‍➡' => 'Pẹ́sin Wé De Waká: Fẹsin ráít',
    '🧎‍➡' => 'Pẹ́sin Wé Níl Daun: Fẹsin ráít',
    '🏃‍➡' => 'Pẹ́sin Wé De Rọn: Fẹsin ráít',
    '👨‍👦' => 'Fámíli: Man, Smọ́l Bọi',
    '👨‍👧' => 'Fámíli: Man, Gẹl',
    '👩‍👦' => 'Fámíli: Wúman, Smọ́l Bọi',
    '👩‍👧' => 'Fámíli: Wúman, Gẹl',
    '😶‍🌫' => 'Pẹ́sin De Sí Pípul Fes Fọ Klaud',
    '😮‍💨' => 'Fes Wé De Brít Ea Kọmọt',
    '🙂‍↕' => 'shek shék hẹd vatikali',
    '😵‍💫' => 'Fes Wé Ai De Tọ́n Raúnd An Raund',
    '❤‍🔥' => 'Hat Wé De Bọ́n Wít Fáya',
    '❤‍🩹' => 'To Ripiá Hat Wé Dọn Brẹk',
    '👁‍🗨' => 'Ai Fọ Ínsaid Mẹ́séj Blonbló Píkchọ',
    '🧔‍♂' => 'Man Wé Gẹ́t Biabiá',
    '👱‍♂' => 'Man Wé Gẹ́t Blọ́nd Hea',
    '🧔‍♀' => 'Wúman Wé Gẹ́t Biabiá',
    '👱‍♀' => 'Wúman Wé Gẹ́t Blọ́nd Hea',
    '🙍‍♂' => 'Man Wé De Fraún Im Fes',
    '🙍‍♀' => 'Wúman Wé De Fraún Im Fes',
    '🙎‍♂' => 'Man Wé De Mék Lọ́ng Fes',
    '🙎‍♀' => 'Wúman Wé De Mék Lọ́ng Fes',
    '🙅‍♂' => 'Man Wé Ték Hand Shó Sé Na NO I Min',
    '🙅‍♀' => 'Wúman Tẹk Hand De Shó Sé Na NO I Min',
    '🙆‍♂' => 'Man Tẹk Hand De Shó Sé Na OK I Min',
    '🙆‍♀' => 'Wúman Tẹk Hand De Shó Sé Na OK I Min',
    '💁‍♂' => 'Man De Tró Im Hand',
    '💁‍♀' => 'Wúman De Tró Im Hand',
    '🙋‍♂' => 'Man Wé De Réz Im Hand Ọp',
    '🙋‍♀' => 'Wúman Wé De Réz Im Hand Ọp',
    '🧏‍♂' => 'Man Wé Nọ́ Fít Hia',
    '🧏‍♀' => 'Wúman Wé Nọ́ Fít Hia',
    '🙇‍♂' => 'Man Wé De Baú Daun',
    '🙇‍♀' => 'Wúman Wé De Baú Daun',
    '🤦‍♂' => 'Man Wé Ték Wọ́n Hand Kọ́vá Im Fes',
    '🤦‍♀' => 'Wúman Ték Wọ́n Hand Kọ́vá Im Fes',
    '🤷‍♂' => 'Man Wé De Shrọ́g Im Shólda',
    '🤷‍♀' => 'Wúman Wé De Shrọ́g Im Shólda',
    '🧑‍⚕' => 'Wọ́ka Fọ Wiá Dẹm De Lúk Áftá Sík Pípul',
    '👨‍⚕' => 'Wọ́kman Fọ Wiá Dẹm De Lúk Áftá Sík Pípul',
    '👩‍⚕' => 'Wúman Wé De Lúk Áftá Sík Pípul',
    '🧑‍🎓' => 'Stúdẹnt',
    '👨‍🎓' => 'Mán Stúdẹnt',
    '👩‍🎓' => 'Wúmán Stúdẹnt',
    '🧑‍🏫' => 'Tícha',
    '👨‍🏫' => 'Mán Tícha',
    '👩‍🏫' => 'Wúmán Tícha',
    '🧑‍⚖' => 'Jọj',
    '👨‍⚖' => 'Mán Jọj',
    '👩‍⚖' => 'Wúmán Jọj',
    '🧑‍🌾' => 'Fáma',
    '👨‍🌾' => 'Mán Fáma',
    '👩‍🌾' => 'Wúmán Fáma',
    '🧑‍🍳' => 'Kuk',
    '👨‍🍳' => 'Mán Kuk',
    '👩‍🍳' => 'Wúmán Kuk',
    '🧑‍🔧' => 'Mẹkánik',
    '👨‍🔧' => 'Mán Mẹkánik',
    '👩‍🔧' => 'Wúmán Mẹkánik',
    '🧑‍🏭' => 'Fáktrí Wọ́ka',
    '👨‍🏭' => 'Mán Fáktrí Wọ́ka',
    '👩‍🏭' => 'Wúmán Fáktrí Wọ́ka',
    '🧑‍💼' => 'Ọ́fís Wọ́ka',
    '👨‍💼' => 'Mán Ọ́fís Wọ́ka',
    '👩‍💼' => 'Wúmán Ọ́fís Wọ́ka',
    '🧑‍🔬' => 'Sáyẹ́ns Ẹ́spat',
    '👨‍🔬' => 'Mán Sáyẹ́ns Ẹ́spat',
    '👩‍🔬' => 'Wúmán Sáyẹ́ns Ẹ́spat',
    '🧑‍💻' => 'Tẹknọ́lọ́jí Ẹ́spat',
    '👨‍💻' => 'Mán Tẹknọ́lọ́jí Ẹ́spat',
    '👩‍💻' => 'Wúman Tẹknọ́lọ́jí Ẹ́spat',
    '🧑‍🎤' => 'Sínga',
    '👨‍🎤' => 'Mán Sínga',
    '👩‍🎤' => 'Wúmán Sínga',
    '🧑‍🎨' => 'Átist',
    '👨‍🎨' => 'Mán Átist',
    '👩‍🎨' => 'Wúmán Átist',
    '🧑‍✈' => 'Paílọt',
    '👨‍✈' => 'Mán Paílọt',
    '👩‍✈' => 'Wúmán Paílọt',
    '🧑‍🚀' => 'Ástrọ́nọt',
    '👨‍🚀' => 'Mán Ástrọ́nọt',
    '👩‍🚀' => 'Wúmán Ástrọ́nọt',
    '🧑‍🚒' => 'Fáyáfaíta',
    '👨‍🚒' => 'Mán Fáyáfaíta',
    '👩‍🚒' => 'Wúmán Fáyáfaíta',
    '👮‍♂' => 'Man Polís Ọ́físa',
    '👮‍♀' => 'Wúmán Polís Ọ́físa',
    '🕵‍♂' => 'Mán Ditẹ́ktiv',
    '🕵‍♀' => 'Wúmán Ditẹ́ktiv',
    '💂‍♂' => 'Mán Gad',
    '💂‍♀' => 'Wúmán Gad',
    '👷‍♂' => 'Mán Kọnstrọ́kshọ́n Wọ́ka',
    '👷‍♀' => 'Wúmán Kọnstrọ́kshọ́n Wọ́ka',
    '👳‍♂' => 'Man Wé Taí Hẹd-Tai Fọ Hẹd',
    '👳‍♀' => 'Wúman Wé Taí Hẹd-Tai Fọ Hẹd',
    '🤵‍♂' => 'Man Wé Wiá Tọksído',
    '🤵‍♀' => 'Wúman Wé Wiá Tọksído',
    '👰‍♂' => 'Man Wé Wiá Vẹl',
    '👰‍♀' => 'Wúman Wé Wiá Vẹl',
    '👩‍🍼' => 'Wúman Wé De Gív Pikin Chọp',
    '👨‍🍼' => 'Man Wé De Gív Pikin Chọp',
    '🧑‍🍼' => 'Pẹ́sin Wé De Gív Pikin Chọp',
    '🧑‍🎄' => 'Mx Klọz',
    '🦸‍♂' => 'Mán Supahíro',
    '🦸‍♀' => 'Wúmán Supahíro',
    '🦹‍♂' => 'Wíkẹ́d Mán Krímínal',
    '🦹‍♀' => 'Wíkẹ́d Wúmán Krímínal',
    '🧙‍♂' => 'Mán Majíshan',
    '🧙‍♀' => 'Wúmán Majíshan',
    '🧚‍♂' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Man',
    '🧚‍♀' => 'Bád Spírit Wé Rizẹ́mbúl Smọ́l Wúmán',
    '🧛‍♂' => 'Mán Vámpáya',
    '🧛‍♀' => 'Wúmán Vámpáya',
    '🧜‍♂' => 'Mán Mamíwọtá',
    '🧜‍♀' => 'Wúmán Mamíwọtá',
    '🧝‍♂' => 'Mán Ẹlf',
    '🧝‍♀' => 'Wúmán Ẹlf',
    '🧞‍♂' => 'Mán Jiínii',
    '🧞‍♀' => 'Wúmán Jiínii',
    '🧟‍♂' => 'Mán Zombí',
    '🧟‍♀' => 'Wúmán Zombí',
    '💆‍♂' => 'Man Wé Dẹm De Masáj Im Bọdí',
    '💆‍♀' => 'Wúman Wé Dẹm De Masáj Im Bọdí',
    '💇‍♂' => 'Man Wé Dẹm De Báb Im Hea',
    '💇‍♀' => 'Wúman Wé Dẹm De Báb Im Hea',
    '🚶‍♂' => 'Man Wé De Waká',
    '🚶‍♀' => 'Wúman Wé De Waká',
    '🧍‍♂' => 'Man Wé De Stánd Ọp',
    '🧍‍♀' => 'Wúman Wé De Stánd Ọp',
    '🧎‍♂' => 'Man Wé Níl Daun',
    '🧎‍♀' => 'Wúman Wé Níl Daun',
    '🧑‍🦯' => 'Pẹ́sin Wit Waít Wọkín-Stik',
    '👨‍🦯' => 'Man Wit Waít Wọkín-Stik',
    '👩‍🦯' => 'Wúman Wit Waít Wọkín-Stik',
    '🧑‍🦼' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin',
    '👨‍🦼' => 'Man Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin',
    '👩‍🦼' => 'Wúman Wé Dé Fọ Wílchia Wé Gẹ́t Ẹ́njin',
    '🧑‍🦽' => 'Pẹ́sin Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin',
    '👨‍🦽' => 'Man Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin',
    '👩‍🦽' => 'Wúman Wé Dé Fọ Wílchia Wé Nọ́ Gẹ́t Ẹ́njin',
    '🏃‍♂' => 'Man Wé De Rọn',
    '🏃‍♀' => 'Wúman Wé De Rọn',
    '🧑‍🩰' => 'bálẹ dánsá',
    '👯‍♂' => 'Mẹn Wé Wiá Rábít Iá Kaín Tins',
    '👯‍♀' => 'Wímẹn Wé Wiá Rábít Iá Kaín Tins',
    '🧖‍♂' => 'Man Wé Dé Ínsaíd Họt Wọtá Smók Rum',
    '🧖‍♀' => 'Wúman Wé Dé Ínsaíd Họt Wọtá Smók Rum',
    '🧗‍♂' => 'Man Wé De Klaimb',
    '🧗‍♀' => 'Wúman Wé De Klaimb',
    '🏌‍♂' => 'Man Wé De Plé Gọlf',
    '🏌‍♀' => 'Wúman Wé De Plé Gọlf',
    '🏄‍♂' => 'Man Wé De Raíd Wọtá Sọ́fín-Bọd',
    '🏄‍♀' => 'Wúman Wé De Raíd Wọtá Sọ́fín-Bọd',
    '🚣‍♂' => 'Man Wé De Ró Bot',
    '🚣‍♀' => 'Wúman Wé De Ró Bot',
    '🏊‍♂' => 'Man Wé De Swim',
    '🏊‍♀' => 'Wúman Wé De Swim',
    '⛹‍♂' => 'Man Wé De Baúns Bọl Fọ Graun',
    '⛹‍♀' => 'Wúman Wé De Baúns Bọl Fọ Graun',
    '🏋‍♂' => 'Man Wé De Kárí Ọ́p Hẹ́ví Tins',
    '🏋‍♀' => 'Wúman Wé De Kárí Ọ́p Hẹ́ví Tins',
    '🚴‍♂' => 'Man Wé De Raíd Baísíkul',
    '🚴‍♀' => 'Wúman Wé De Raíd Baísíkul',
    '🚵‍♂' => 'Man Wé De Raíd Baísíkul Fọ Maúntin',
    '🚵‍♀' => 'Wúman De Raíd Baísíkul Fọ Maúntin',
    '🤸‍♂' => 'Man Wé De Dú Kátwil',
    '🤸‍♀' => 'Wúman Wé De Dú Kátwil',
    '🤼‍♂' => 'Mẹn Wé De Rẹ́sul',
    '🤼‍♀' => 'Wímẹn Wé De Rẹ́sul',
    '🤽‍♂' => 'Man Wé De Plé Wọ́tá Poló',
    '🤽‍♀' => 'Wúman Wé De Plé Wọ́tá Poló',
    '🤾‍♂' => 'Man Wé De Plé Hándbọl',
    '🤾‍♀' => 'Wúman Wé De Plé Hándbọl',
    '🤹‍♂' => 'Man Wé De Jọ́gúl Bọl-Dẹm',
    '🤹‍♀' => 'Wúman Wé De Jọ́gúl Bọl-Dẹm',
    '🧘‍♂' => 'Man Wé Sidọ́n Fọ Graun Fọ Lótús Wè',
    '🧘‍♀' => 'Wúman Sidọ́n Fọ Graun Fọ Lótús Wè',
    '🧑‍🧒' => 'fámíli: machọ́ pẹ́sin, pikin',
    '🐕‍🦺' => 'Dọg Wé De Hẹ́lẹ́p Sík Pípul',
    '🐈‍⬛' => 'Blák Kat',
    '🐻‍❄' => 'Pólá Bẹ́a',
    '🐦‍⬛' => 'blák bẹd',
    '🐦‍🔥' => 'fíniks',
    '🍋‍🟩' => 'láim',
    '🍄‍🟫' => 'braun mọ́shrum',
    '⛓‍💥' => 'brókún chen',
    '🏳‍🌈' => 'Rénbo Flag',
    '🏳‍⚧' => 'Transjẹ́nda Flag',
    '🏴‍☠' => 'Paíret Flag',
    '👋🏻' => 'Hand Wé De Wev: Laít Skín Ton',
    '👋🏼' => 'Hand Wé De Wev: Mídíọ́m-Laít Skín Ton',
    '👋🏽' => 'Hand Wé De Wev: Mídíọ́m Skín Ton',
    '👋🏾' => 'Hand Wé De Wev: Mídíọ́m Dák Skín Ton',
    '👋🏿' => 'Hand Wé De Wev: Dák Skín Ton',
    '🤚🏻' => 'Hand Im Bak Wé Dẹm Réz Ọp: Laít Skín Ton',
    '🤚🏼' => 'Hand Im Bak Wé Dẹm Réz Ọp: Mídíọ́m-Laít Skín Ton',
    '🤚🏽' => 'Hand Im Bak Wé Dẹm Réz Ọp: Mídíọ́m Skín Ton',
    '🤚🏾' => 'Hand Im Bak Wé Dẹm Réz Ọp: Mídíọ́m Dák Skín Ton',
    '🤚🏿' => 'Hand Im Bak Wé Dẹm Réz Ọp: Dák Skín Ton',
    '🖐🏻' => 'Hand Wé Dẹm Sprẹ́d Di Fínga-dẹm: Laít Skín Ton',
    '🖐🏼' => 'Hand Wé Dẹm Sprẹ́d Di Fínga-dẹm: Mídíọ́m-Laít Skín Ton',
    '🖐🏽' => 'Hand Wé Dẹm Sprẹ́d Di Fínga-dẹm: Mídíọ́m Skín Ton',
    '🖐🏾' => 'Hand Wé Dẹm Sprẹ́d Di Fínga-dẹm: Mídíọ́m Dák Skín Ton',
    '🖐🏿' => 'Hand Wé Dẹm Sprẹ́d Di Fínga-dẹm: Dák Skín Ton',
    '✋🏻' => 'Hand Wé Dẹm Réz Ọp: Laít Skín Ton',
    '✋🏼' => 'Hand Wé Dẹm Réz Ọp: Mídíọ́m-Laít Skín Ton',
    '✋🏽' => 'Hand Wé Dẹm Réz Ọp: Mídíọ́m Skín Ton',
    '✋🏾' => 'Hand Wé Dẹm Réz Ọp: Mídíọ́m Dák Skín Ton',
    '✋🏿' => 'Hand Wé Dẹm Réz Ọp: Dák Skín Ton',
    '🖖🏻' => 'Vọ́lkán Salut: Laít Skín Ton',
    '🖖🏼' => 'Vọ́lkán Salut: Mídíọ́m-Laít Skín Ton',
    '🖖🏽' => 'Vọ́lkán Salut: Mídíọ́m Skín Ton',
    '🖖🏾' => 'Vọ́lkán Salut: Mídíọ́m Dák Skín Ton',
    '🖖🏿' => 'Vọ́lkán Salut: Dák Skín Ton',
    '🫱🏻' => 'Hand Wé De Pọínt Raít Said: Laít Skín Ton',
    '🫱🏼' => 'Hand Wé De Pọínt Raít Said: Mídíọ́m-Laít Skín Ton',
    '🫱🏽' => 'Hand Wé De Pọínt Raít Said: Mídíọ́m Skín Ton',
    '🫱🏾' => 'Hand Wé De Pọínt Raít Said: Mídíọ́m Dák Skín Ton',
    '🫱🏿' => 'Hand Wé De Pọínt Raít Said: Dák Skín Ton',
    '🫲🏻' => 'Hand Wé De Pọínt Lẹ́ft Said: Laít Skín Ton',
    '🫲🏼' => 'Hand Wé De Pọínt Lẹ́ft Said: Mídíọ́m-Laít Skín Ton',
    '🫲🏽' => 'Hand Wé De Pọínt Lẹ́ft Said: Mídíọ́m Skín Ton',
    '🫲🏾' => 'Hand Wé De Pọínt Lẹ́ft Said: Mídíọ́m Dák Skín Ton',
    '🫲🏿' => 'Hand Wé De Pọínt Lẹ́ft Said: Dák Skín Ton',
    '🫳🏻' => 'Hand Wé De Fés Daun: Laít Skín Ton',
    '🫳🏼' => 'Hand Wé De Fés Daun: Mídíọ́m-Laít Skín Ton',
    '🫳🏽' => 'Hand Wé De Fés Daun: Mídíọ́m Skín Ton',
    '🫳🏾' => 'Hand Wé De Fés Daun: Mídíọ́m Dák Skín Ton',
    '🫳🏿' => 'Hand Wé De Fés Daun: Dák Skín Ton',
    '🫴🏻' => 'Hand Wé De Pọínt Ọp: Laít Skín Ton',
    '🫴🏼' => 'Hand Wé De Pọínt Ọp: Mídíọ́m-Laít Skín Ton',
    '🫴🏽' => 'Hand Wé De Pọínt Ọp: Mídíọ́m Skín Ton',
    '🫴🏾' => 'Hand Wé De Pọínt Ọp: Mídíọ́m Dák Skín Ton',
    '🫴🏿' => 'Hand Wé De Pọínt Ọp: Dák Skín Ton',
    '🫷🏻' => 'Hand Wé De Púsh Gó Lẹ́ft Said: Laít Skín Ton',
    '🫷🏼' => 'Hand Wé De Púsh Gó Lẹ́ft Said: Mídíọ́m-Laít Skín Ton',
    '🫷🏽' => 'Hand Wé De Púsh Gó Lẹ́ft Said: Mídíọ́m Skín Ton',
    '🫷🏾' => 'Hand Wé De Púsh Gó Lẹ́ft Said: Mídíọ́m Dák Skín Ton',
    '🫷🏿' => 'Hand Wé De Púsh Gó Lẹ́ft Said: Dák Skín Ton',
    '🫸🏻' => 'Hand Wé De Púsh Gó Raít Said: Laít Skín Ton',
    '🫸🏼' => 'Hand Wé De Púsh Gó Raít Said: Mídíọ́m-Laít Skín Ton',
    '🫸🏽' => 'Hand Wé De Púsh Gó Raít Said: Mídíọ́m Skín Ton',
    '🫸🏾' => 'Hand Wé De Púsh Gó Raít Said: Mídíọ́m Dák Skín Ton',
    '🫸🏿' => 'Hand Wé De Púsh Gó Raít Said: Dák Skín Ton',
    '👌🏻' => 'OK Hand: Laít Skín Ton',
    '👌🏼' => 'OK Hand: Mídíọ́m-Laít Skín Ton',
    '👌🏽' => 'OK Hand: Mídíọ́m Skín Ton',
    '👌🏾' => 'OK Hand: Mídíọ́m Dák Skín Ton',
    '👌🏿' => 'OK Hand: Dák Skín Ton',
    '🤌🏻' => 'To Pínch Fínga-dẹm Togẹ́da: Laít Skín Ton',
    '🤌🏼' => 'To Pínch Fínga-dẹm Togẹ́da: Mídíọ́m-Laít Skín Ton',
    '🤌🏽' => 'To Pínch Fínga-dẹm Togẹ́da: Mídíọ́m Skín Ton',
    '🤌🏾' => 'To Pínch Fínga-dẹm Togẹ́da: Mídíọ́m Dák Skín Ton',
    '🤌🏿' => 'To Pínch Fínga-dẹm Togẹ́da: Dák Skín Ton',
    '🤏🏻' => 'Hand Wé De Pinch: Laít Skín Ton',
    '🤏🏼' => 'Hand Wé De Pinch: Mídíọ́m-Laít Skín Ton',
    '🤏🏽' => 'Hand Wé De Pinch: Mídíọ́m Skín Ton',
    '🤏🏾' => 'Hand Wé De Pinch: Mídíọ́m Dák Skín Ton',
    '🤏🏿' => 'Hand Wé De Pinch: Dák Skín Ton',
    '✌🏻' => 'Víktri Hand: Laít Skín Ton',
    '✌🏼' => 'Víktri Hand: Mídíọ́m-Laít Skín Ton',
    '✌🏽' => 'Víktri Hand: Mídíọ́m Skín Ton',
    '✌🏾' => 'Víktri Hand: Mídíọ́m Dák Skín Ton',
    '✌🏿' => 'Víktri Hand: Dák Skín Ton',
    '🤞🏻' => 'Krọ́s Fínga-dẹm: Laít Skín Ton',
    '🤞🏼' => 'Krọ́s Fínga-dẹm: Mídíọ́m-Laít Skín Ton',
    '🤞🏽' => 'Krọ́s Fínga-dẹm: Mídíọ́m Skín Ton',
    '🤞🏾' => 'Krọ́s Fínga-dẹm: Mídíọ́m Dák Skín Ton',
    '🤞🏿' => 'Krọ́s Fínga-dẹm: Dák Skín Ton',
    '🫰🏻' => 'Hand Wé Krọ́s Im Sẹ́kọ́n Fínga An Tọmb: Laít Skín Ton',
    '🫰🏼' => 'Hand Wé Krọ́s Im Sẹ́kọ́n Fínga An Tọmb: Mídíọ́m-Laít Skín Ton',
    '🫰🏽' => 'Hand Wé Krọ́s Im Sẹ́kọ́n Fínga An Tọmb: Mídíọ́m Skín Ton',
    '🫰🏾' => 'Hand Wé Krọ́s Im Sẹ́kọ́n Fínga An Tọmb: Mídíọ́m Dák Skín Ton',
    '🫰🏿' => 'Hand Wé Krọ́s Im Sẹ́kọ́n Fínga An Tọmb: Dák Skín Ton',
    '🤟🏻' => 'A Lọ́v Yu Sain: Laít Skín Ton',
    '🤟🏼' => 'A Lọ́v Yu Sain: Mídíọ́m-Laít Skín Ton',
    '🤟🏽' => 'A Lọ́v Yu Sain: Mídíọ́m Skín Ton',
    '🤟🏾' => 'A Lọ́v Yu Sain: Mídíọ́m Dák Skín Ton',
    '🤟🏿' => 'A Lọ́v Yu Sain: Dák Skín Ton',
    '🤘🏻' => 'Họ́n Sain: Laít Skín Ton',
    '🤘🏼' => 'Họ́n Sain: Mídíọ́m-Laít Skín Ton',
    '🤘🏽' => 'Họ́n Sain: Mídíọ́m Skín Ton',
    '🤘🏾' => 'Họ́n Sain: Mídíọ́m Dák Skín Ton',
    '🤘🏿' => 'Họ́n Sain: Dák Skín Ton',
    '🤙🏻' => 'Mék Yu Fón Mí Hand: Laít Skín Ton',
    '🤙🏼' => 'Mék Yu Fón Mí Hand: Mídíọ́m-Laít Skín Ton',
    '🤙🏽' => 'Mék Yu Fón Mí Hand: Mídíọ́m Skín Ton',
    '🤙🏾' => 'Mék Yu Fón Mí Hand: Mídíọ́m Dák Skín Ton',
    '🤙🏿' => 'Mék Yu Fón Mí Hand: Dák Skín Ton',
    '👈🏻' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Lẹft: Laít Skín Ton',
    '👈🏼' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Lẹft: Mídíọ́m-Laít Skín Ton',
    '👈🏽' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Lẹft: Mídíọ́m Skín Ton',
    '👈🏾' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Lẹft: Mídíọ́m Dák Skín Ton',
    '👈🏿' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Lẹft: Dák Skín Ton',
    '👉🏻' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Rait: Laít Skín Ton',
    '👉🏼' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Rait: Mídíọ́m-Laít Skín Ton',
    '👉🏽' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Rait: Mídíọ́m Skín Ton',
    '👉🏾' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Rait: Mídíọ́m Dák Skín Ton',
    '👉🏿' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Rait: Dák Skín Ton',
    '👆🏻' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Ọp: Laít Skín Ton',
    '👆🏼' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Ọp: Mídíọ́m-Laít Skín Ton',
    '👆🏽' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Ọp: Mídíọ́m Skín Ton',
    '👆🏾' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Ọp: Mídíọ́m Dák Skín Ton',
    '👆🏿' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Ọp: Dák Skín Ton',
    '🖕🏻' => 'Mídúl Fínga: Laít Skín Ton',
    '🖕🏼' => 'Mídúl Fínga: Mídíọ́m-Laít Skín Ton',
    '🖕🏽' => 'Mídúl Fínga: Mídíọ́m Skín Ton',
    '🖕🏾' => 'Mídúl Fínga: Mídíọ́m Dák Skín Ton',
    '🖕🏿' => 'Mídúl Fínga: Dák Skín Ton',
    '👇🏻' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Daun: Laít Skín Ton',
    '👇🏼' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Daun: Mídíọ́m-Laít Skín Ton',
    '👇🏽' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Daun: Mídíọ́m Skín Ton',
    '👇🏾' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Daun: Mídíọ́m Dák Skín Ton',
    '👇🏿' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Daun: Dák Skín Ton',
    '☝🏻' => 'Niá-Tọ́mb-Fínga De Pọínt Ọp: Laít Skín Ton',
    '☝🏼' => 'Niá-Tọ́mb-Fínga De Pọínt Ọp: Mídíọ́m-Laít Skín Ton',
    '☝🏽' => 'Niá-Tọ́mb-Fínga De Pọínt Ọp: Mídíọ́m Skín Ton',
    '☝🏾' => 'Niá-Tọ́mb-Fínga De Pọínt Ọp: Mídíọ́m Dák Skín Ton',
    '☝🏿' => 'Niá-Tọ́mb-Fínga De Pọínt Ọp: Dák Skín Ton',
    '🫵🏻' => 'Sẹ́kọ́n Fínga De Pọínt Di Pẹ́sin Wé De Luk: Laít Skín Ton',
    '🫵🏼' => 'Sẹ́kọ́n Fínga De Pọínt Di Pẹ́sin Wé De Luk: Mídíọ́m-Laít Skín Ton',
    '🫵🏽' => 'Sẹ́kọ́n Fínga De Pọínt Di Pẹ́sin Wé De Luk: Mídíọ́m Skín Ton',
    '🫵🏾' => 'Sẹ́kọ́n Fínga De Pọínt Di Pẹ́sin Wé De Luk: Mídíọ́m Dák Skín Ton',
    '🫵🏿' => 'Sẹ́kọ́n Fínga De Pọínt Di Pẹ́sin Wé De Luk: Dák Skín Ton',
    '👍🏻' => 'Tọ́mb Ọp: Laít Skín Ton',
    '👍🏼' => 'Tọ́mb Ọp: Mídíọ́m-Laít Skín Ton',
    '👍🏽' => 'Tọ́mb Ọp: Mídíọ́m Skín Ton',
    '👍🏾' => 'Tọ́mb Ọp: Mídíọ́m Dák Skín Ton',
    '👍🏿' => 'Tọ́mb Ọp: Dák Skín Ton',
    '👎🏻' => 'Tọ́mb Daun: Laít Skín Ton',
    '👎🏼' => 'Tọ́mb Daun: Mídíọ́m-Laít Skín Ton',
    '👎🏽' => 'Tọ́mb Daun: Mídíọ́m Skín Ton',
    '👎🏾' => 'Tọ́mb Daun: Mídíọ́m Dák Skín Ton',
    '👎🏿' => 'Tọ́mb Daun: Dák Skín Ton',
    '✊🏻' => 'Réz Klózd Hand Ọp: Laít Skín Ton',
    '✊🏼' => 'Réz Klózd Hand Ọp: Mídíọ́m-Laít Skín Ton',
    '✊🏽' => 'Réz Klózd Hand Ọp: Mídíọ́m Skín Ton',
    '✊🏾' => 'Réz Klózd Hand Ọp: Mídíọ́m Dák Skín Ton',
    '✊🏿' => 'Réz Klózd Hand Ọp: Dák Skín Ton',
    '👊🏻' => 'Klózd Hand Wán Gív Pẹ́sin Blo: Laít Skín Ton',
    '👊🏼' => 'Klózd Hand Wán Gív Pẹ́sin Blo: Mídíọ́m-Laít Skín Ton',
    '👊🏽' => 'Klózd Hand Wán Gív Pẹ́sin Blo: Mídíọ́m Skín Ton',
    '👊🏾' => 'Klózd Hand Wán Gív Pẹ́sin Blo: Mídíọ́m Dák Skín Ton',
    '👊🏿' => 'Klózd Hand Wán Gív Pẹ́sin Blo: Dák Skín Ton',
    '🤛🏻' => 'Klózd Hand Wé De Fés Lẹft: Laít Skín Ton',
    '🤛🏼' => 'Klózd Hand Wé De Fés Lẹft: Mídíọ́m-Laít Skín Ton',
    '🤛🏽' => 'Klózd Hand Wé De Fés Lẹft: Mídíọ́m Skín Ton',
    '🤛🏾' => 'Klózd Hand Wé De Fés Lẹft: Mídíọ́m Dák Skín Ton',
    '🤛🏿' => 'Klózd Hand Wé De Fés Lẹft: Dák Skín Ton',
    '🤜🏻' => 'Klózd Hand Wé De Fés Rait: Laít Skín Ton',
    '🤜🏼' => 'Klózd Hand Wé De Fés Rait: Mídíọ́m-Laít Skín Ton',
    '🤜🏽' => 'Klózd Hand Wé De Fés Rait: Mídíọ́m Skín Ton',
    '🤜🏾' => 'Klózd Hand Wé De Fés Rait: Mídíọ́m Dák Skín Ton',
    '🤜🏿' => 'Klózd Hand Wé De Fés Rait: Dák Skín Ton',
    '👏🏻' => 'Hand-dẹm Wé De Klap: Laít Skín Ton',
    '👏🏼' => 'Hand-dẹm Wé De Klap: Mídíọ́m-Laít Skín Ton',
    '👏🏽' => 'Hand-dẹm Wé De Klap: Mídíọ́m Skín Ton',
    '👏🏾' => 'Hand-dẹm Wé De Klap: Mídíọ́m Dák Skín Ton',
    '👏🏿' => 'Hand-dẹm Wé De Klap: Dák Skín Ton',
    '🙌🏻' => 'Hand-dẹm Wé Dẹm Réz Ọp: Laít Skín Ton',
    '🙌🏼' => 'Hand-dẹm Wé Dẹm Réz Ọp: Mídíọ́m-Laít Skín Ton',
    '🙌🏽' => 'Hand-dẹm Wé Dẹm Réz Ọp: Mídíọ́m Skín Ton',
    '🙌🏾' => 'Hand-dẹm Wé Dẹm Réz Ọp: Mídíọ́m Dák Skín Ton',
    '🙌🏿' => 'Hand-dẹm Wé Dẹm Réz Ọp: Dák Skín Ton',
    '🫶🏻' => 'Hát hand-dẹm: Laít Skín Ton',
    '🫶🏼' => 'Hát hand-dẹm: Mídíọ́m-Laít Skín Ton',
    '🫶🏽' => 'Hát hand-dẹm: Mídíọ́m Skín Ton',
    '🫶🏾' => 'Hát hand-dẹm: Mídíọ́m Dák Skín Ton',
    '🫶🏿' => 'Hát hand-dẹm: Dák Skín Ton',
    '👐🏻' => 'Ópún Hand-dẹm: Laít Skín Ton',
    '👐🏼' => 'Ópún Hand-dẹm: Mídíọ́m-Laít Skín Ton',
    '👐🏽' => 'Ópún Hand-dẹm: Mídíọ́m Skín Ton',
    '👐🏾' => 'Ópún Hand-dẹm: Mídíọ́m Dák Skín Ton',
    '👐🏿' => 'Ópún Hand-dẹm: Dák Skín Ton',
    '🤲🏻' => 'Ópún Hand-dẹm Wé Dé Togẹ́da: Laít Skín Ton',
    '🤲🏼' => 'Ópún Hand-dẹm Wé Dé Togẹ́da: Mídíọ́m-Laít Skín Ton',
    '🤲🏽' => 'Ópún Hand-dẹm Wé Dé Togẹ́da: Mídíọ́m Skín Ton',
    '🤲🏾' => 'Ópún Hand-dẹm Wé Dé Togẹ́da: Mídíọ́m Dák Skín Ton',
    '🤲🏿' => 'Ópún Hand-dẹm Wé Dé Togẹ́da: Dák Skín Ton',
    '🤝🏻' => 'Hándshek: Laít Skín Ton',
    '🤝🏼' => 'Hándshek: Mídíọ́m-Laít Skín Ton',
    '🤝🏽' => 'Hándshek: Mídíọ́m Skín Ton',
    '🤝🏾' => 'Hándshek: Mídíọ́m Dák Skín Ton',
    '🤝🏿' => 'Hándshek: Dák Skín Ton',
    '🙏🏻' => 'Hand-dẹm Wé Dẹm Fold: Laít Skín Ton',
    '🙏🏼' => 'Hand-dẹm Wé Dẹm Fold: Mídíọ́m-Laít Skín Ton',
    '🙏🏽' => 'Hand-dẹm Wé Dẹm Fold: Mídíọ́m Skín Ton',
    '🙏🏾' => 'Hand-dẹm Wé Dẹm Fold: Mídíọ́m Dák Skín Ton',
    '🙏🏿' => 'Hand-dẹm Wé Dẹm Fold: Dák Skín Ton',
    '✍🏻' => 'Hand Wé De Raít Sọ́mtin: Laít Skín Ton',
    '✍🏼' => 'Hand Wé De Raít Sọ́mtin: Mídíọ́m-Laít Skín Ton',
    '✍🏽' => 'Hand Wé De Raít Sọ́mtin: Mídíọ́m Skín Ton',
    '✍🏾' => 'Hand Wé De Raít Sọ́mtin: Mídíọ́m Dák Skín Ton',
    '✍🏿' => 'Hand Wé De Raít Sọ́mtin: Dák Skín Ton',
    '💅🏻' => 'Nél Pọ́lish: Laít Skín Ton',
    '💅🏼' => 'Nél Pọ́lish: Mídíọ́m-Laít Skín Ton',
    '💅🏽' => 'Nél Pọ́lish: Mídíọ́m Skín Ton',
    '💅🏾' => 'Nél Pọ́lish: Mídíọ́m Dák Skín Ton',
    '💅🏿' => 'Nél Pọ́lish: Dák Skín Ton',
    '🤳🏻' => 'Sẹ́lfi: Laít Skín Ton',
    '🤳🏼' => 'Sẹ́lfi: Mídíọ́m-Laít Skín Ton',
    '🤳🏽' => 'Sẹ́lfi: Mídíọ́m Skín Ton',
    '🤳🏾' => 'Sẹ́lfi: Mídíọ́m Dák Skín Ton',
    '🤳🏿' => 'Sẹ́lfi: Dák Skín Ton',
    '💪🏻' => 'Hánd Mọ́zul Wé Bẹnd: Laít Skín Ton',
    '💪🏼' => 'Hánd Mọ́zul Wé Bẹnd: Mídíọ́m-Laít Skín Ton',
    '💪🏽' => 'Hánd Mọ́zul Wé Bẹnd: Mídíọ́m Skín Ton',
    '💪🏾' => 'Hánd Mọ́zul Wé Bẹnd: Mídíọ́m Dák Skín Ton',
    '💪🏿' => 'Hánd Mọ́zul Wé Bẹnd: Dák Skín Ton',
    '🦵🏻' => 'Lẹg: Laít Skín Ton',
    '🦵🏼' => 'Lẹg: Mídíọ́m-Laít Skín Ton',
    '🦵🏽' => 'Lẹg: Mídíọ́m Skín Ton',
    '🦵🏾' => 'Lẹg: Mídíọ́m Dák Skín Ton',
    '🦵🏿' => 'Lẹg: Dák Skín Ton',
    '🦶🏻' => 'Daunlẹg: Laít Skín Ton',
    '🦶🏼' => 'Daunlẹg: Mídíọ́m-Laít Skín Ton',
    '🦶🏽' => 'Daunlẹg: Mídíọ́m Skín Ton',
    '🦶🏾' => 'Daunlẹg: Mídíọ́m Dák Skín Ton',
    '🦶🏿' => 'Daunlẹg: Dák Skín Ton',
    '👂🏻' => 'Ia: Laít Skín Ton',
    '👂🏼' => 'Ia: Mídíọ́m-Laít Skín Ton',
    '👂🏽' => 'Ia: Mídíọ́m Skín Ton',
    '👂🏾' => 'Ia: Mídíọ́m Dák Skín Ton',
    '👂🏿' => 'Ia: Dák Skín Ton',
    '🦻🏻' => 'Ia Wit Sọ́mtin Wé De Hẹ́lẹ́p Pẹ́sin Mék I Hia: Laít Skín Ton',
    '🦻🏼' => 'Ia Wit Sọ́mtin Wé De Hẹ́lẹ́p Pẹ́sin Mék I Hia: Mídíọ́m-Laít Skín Ton',
    '🦻🏽' => 'Ia Wit Sọ́mtin Wé De Hẹ́lẹ́p Pẹ́sin Mék I Hia: Mídíọ́m Skín Ton',
    '🦻🏾' => 'Ia Wit Sọ́mtin Wé De Hẹ́lẹ́p Pẹ́sin Mék I Hia: Mídíọ́m Dák Skín Ton',
    '🦻🏿' => 'Ia Wit Sọ́mtin Wé De Hẹ́lẹ́p Pẹ́sin Mék I Hia: Dák Skín Ton',
    '👃🏻' => 'Noz: Laít Skín Ton',
    '👃🏼' => 'Noz: Mídíọ́m-Laít Skín Ton',
    '👃🏽' => 'Noz: Mídíọ́m Skín Ton',
    '👃🏾' => 'Noz: Mídíọ́m Dák Skín Ton',
    '👃🏿' => 'Noz: Dák Skín Ton',
    '👶🏻' => 'Taata: Laít Skín Ton',
    '👶🏼' => 'Taata: Mídíọ́m-Laít Skín Ton',
    '👶🏽' => 'Taata: Mídíọ́m Skín Ton',
    '👶🏾' => 'Taata: Mídíọ́m Dák Skín Ton',
    '👶🏿' => 'Taata: Dák Skín Ton',
    '🧒🏻' => 'Smọ́l Pikin: Laít Skín Ton',
    '🧒🏼' => 'Smọ́l Pikin: Mídíọ́m-Laít Skín Ton',
    '🧒🏽' => 'Smọ́l Pikin: Mídíọ́m Skín Ton',
    '🧒🏾' => 'Smọ́l Pikin: Mídíọ́m Dák Skín Ton',
    '🧒🏿' => 'Smọ́l Pikin: Dák Skín Ton',
    '👦🏻' => 'Smọ́l Bọi: Laít Skín Ton',
    '👦🏼' => 'Smọ́l Bọi: Mídíọ́m-Laít Skín Ton',
    '👦🏽' => 'Smọ́l Bọi: Mídíọ́m Skín Ton',
    '👦🏾' => 'Smọ́l Bọi: Mídíọ́m Dák Skín Ton',
    '👦🏿' => 'Smọ́l Bọi: Dák Skín Ton',
    '👧🏻' => 'Gẹl: Laít Skín Ton',
    '👧🏼' => 'Gẹl: Mídíọ́m-Laít Skín Ton',
    '👧🏽' => 'Gẹl: Mídíọ́m Skín Ton',
    '👧🏾' => 'Gẹl: Mídíọ́m Dák Skín Ton',
    '👧🏿' => 'Gẹl: Dák Skín Ton',
    '🧑🏻' => 'Pẹ́sin: Laít Skín Ton',
    '🧑🏼' => 'Pẹ́sin: Mídíọ́m-Laít Skín Ton',
    '🧑🏽' => 'Pẹ́sin: Mídíọ́m Skín Ton',
    '🧑🏾' => 'Pẹ́sin: Mídíọ́m Dák Skín Ton',
    '🧑🏿' => 'Pẹ́sin: Dák Skín Ton',
    '👱🏻' => 'Pẹ́sin Wé Gẹ́t Blọ́nd Hea: Laít Skín Ton',
    '👱🏼' => 'Pẹ́sin Wé Gẹ́t Blọ́nd Hea: Mídíọ́m-Laít Skín Ton',
    '👱🏽' => 'Pẹ́sin Wé Gẹ́t Blọ́nd Hea: Mídíọ́m Skín Ton',
    '👱🏾' => 'Pẹ́sin Wé Gẹ́t Blọ́nd Hea: Mídíọ́m Dák Skín Ton',
    '👱🏿' => 'Pẹ́sin Wé Gẹ́t Blọ́nd Hea: Dák Skín Ton',
    '👨🏻' => 'Man: Laít Skín Ton',
    '👨🏼' => 'Man: Mídíọ́m-Laít Skín Ton',
    '👨🏽' => 'Man: Mídíọ́m Skín Ton',
    '👨🏾' => 'Man: Mídíọ́m Dák Skín Ton',
    '👨🏿' => 'Man: Dák Skín Ton',
    '🧔🏻' => 'Pẹ́sin Wé Gẹ́t Biabiá: Laít Skín Ton',
    '🧔🏼' => 'Pẹ́sin Wé Gẹ́t Biabiá: Mídíọ́m-Laít Skín Ton',
    '🧔🏽' => 'Pẹ́sin Wé Gẹ́t Biabiá: Mídíọ́m Skín Ton',
    '🧔🏾' => 'Pẹ́sin Wé Gẹ́t Biabiá: Mídíọ́m Dák Skín Ton',
    '🧔🏿' => 'Pẹ́sin Wé Gẹ́t Biabiá: Dák Skín Ton',
    '👩🏻' => 'Wúman: Laít Skín Ton',
    '👩🏼' => 'Wúman: Mídíọ́m-Laít Skín Ton',
    '👩🏽' => 'Wúman: Mídíọ́m Skín Ton',
    '👩🏾' => 'Wúman: Mídíọ́m Dák Skín Ton',
    '👩🏿' => 'Wúman: Dák Skín Ton',
    '🧓🏻' => 'Síniọ́ Pẹ́sin: Laít Skín Ton',
    '🧓🏼' => 'Síniọ́ Pẹ́sin: Mídíọ́m-Laít Skín Ton',
    '🧓🏽' => 'Síniọ́ Pẹ́sin: Mídíọ́m Skín Ton',
    '🧓🏾' => 'Síniọ́ Pẹ́sin: Mídíọ́m Dák Skín Ton',
    '🧓🏿' => 'Síniọ́ Pẹ́sin: Dák Skín Ton',
    '👴🏻' => 'Óld Man: Laít Skín Ton',
    '👴🏼' => 'Óld Man: Mídíọ́m-Laít Skín Ton',
    '👴🏽' => 'Óld Man: Mídíọ́m Skín Ton',
    '👴🏾' => 'Óld Man: Mídíọ́m Dák Skín Ton',
    '👴🏿' => 'Óld Man: Dák Skín Ton',
    '👵🏻' => 'Óld Wúman: Laít Skín Ton',
    '👵🏼' => 'Óld Wúman: Mídíọ́m-Laít Skín Ton',
    '👵🏽' => 'Óld Wúman: Mídíọ́m Skín Ton',
    '👵🏾' => 'Óld Wúman: Mídíọ́m Dák Skín Ton',
    '👵🏿' => 'Óld Wúman: Dák Skín Ton',
    '🙍🏻' => 'Pẹ́sin Wé Skwíz Fes: Laít Skín Ton',
    '🙍🏼' => 'Pẹ́sin Wé Skwíz Fes: Mídíọ́m-Laít Skín Ton',
    '🙍🏽' => 'Pẹ́sin Wé Skwíz Fes: Mídíọ́m Skín Ton',
    '🙍🏾' => 'Pẹ́sin Wé Skwíz Fes: Mídíọ́m Dák Skín Ton',
    '🙍🏿' => 'Pẹ́sin Wé Skwíz Fes: Dák Skín Ton',
    '🙎🏻' => 'Pẹ́sin Wé De Mék Lọ́ng Fes: Laít Skín Ton',
    '🙎🏼' => 'Pẹ́sin Wé De Mék Lọ́ng Fes: Mídíọ́m-Laít Skín Ton',
    '🙎🏽' => 'Pẹ́sin Wé De Mék Lọ́ng Fes: Mídíọ́m Skín Ton',
    '🙎🏾' => 'Pẹ́sin Wé De Mék Lọ́ng Fes: Mídíọ́m Dák Skín Ton',
    '🙎🏿' => 'Pẹ́sin Wé De Mék Lọ́ng Fes: Dák Skín Ton',
    '🙅🏻' => 'Pẹ́sin Wé Ték Hand De Shó Sé Na NO I Min: Laít Skín Ton',
    '🙅🏼' => 'Pẹ́sin Wé Ték Hand De Shó Sé Na NO I Min: Mídíọ́m-Laít Skín Ton',
    '🙅🏽' => 'Pẹ́sin Wé Ték Hand De Shó Sé Na NO I Min: Mídíọ́m Skín Ton',
    '🙅🏾' => 'Pẹ́sin Wé Ték Hand De Shó Sé Na NO I Min: Mídíọ́m Dák Skín Ton',
    '🙅🏿' => 'Pẹ́sin Wé Ték Hand De Shó Sé Na NO I Min: Dák Skín Ton',
    '🙆🏻' => 'Pẹ́sin Tẹk Hand De Shó Sé Na OK I Min: Laít Skín Ton',
    '🙆🏼' => 'Pẹ́sin Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m-Laít Skín Ton',
    '🙆🏽' => 'Pẹ́sin Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m Skín Ton',
    '🙆🏾' => 'Pẹ́sin Tẹk Hand De Shó Sé Na OK I Min: Mídíọ́m Dák Skín Ton',
    '🙆🏿' => 'Pẹ́sin Tẹk Hand De Shó Sé Na OK I Min: Dák Skín Ton',
    '💁🏻' => 'Pẹ́sin Wé De Shó Im Hand: Laít Skín Ton',
    '💁🏼' => 'Pẹ́sin Wé De Shó Im Hand: Mídíọ́m-Laít Skín Ton',
    '💁🏽' => 'Pẹ́sin Wé De Shó Im Hand: Mídíọ́m Skín Ton',
    '💁🏾' => 'Pẹ́sin Wé De Shó Im Hand: Mídíọ́m Dák Skín Ton',
    '💁🏿' => 'Pẹ́sin Wé De Shó Im Hand: Dák Skín Ton',
    '🙋🏻' => 'Pẹ́sin Wé De Réz Im Hand Ọp: Laít Skín Ton',
    '🙋🏼' => 'Pẹ́sin Wé De Réz Im Hand Ọp: Mídíọ́m-Laít Skín Ton',
    '🙋🏽' => 'Pẹ́sin Wé De Réz Im Hand Ọp: Mídíọ́m Skín Ton',
    '🙋🏾' => 'Pẹ́sin Wé De Réz Im Hand Ọp: Mídíọ́m Dák Skín Ton',
    '🙋🏿' => 'Pẹ́sin Wé De Réz Im Hand Ọp: Dák Skín Ton',
    '🧏🏻' => 'Pẹ́sin Wé Nọ́ Fít Hia: Laít Skín Ton',
    '🧏🏼' => 'Pẹ́sin Wé Nọ́ Fít Hia: Mídíọ́m-Laít Skín Ton',
    '🧏🏽' => 'Pẹ́sin Wé Nọ́ Fít Hia: Mídíọ́m Skín Ton',
    '🧏🏾' => 'Pẹ́sin Wé Nọ́ Fít Hia: Mídíọ́m Dák Skín Ton',
    '🧏🏿' => 'Pẹ́sin Wé Nọ́ Fít Hia: Dák Skín Ton',
    '🙇🏻' => 'Pẹ́sin Wé De Baú Daun: Laít Skín Ton',
    '🙇🏼' => 'Pẹ́sin Wé De Baú Daun: Mídíọ́m-Laít Skín Ton',
    '🙇🏽' => 'Pẹ́sin Wé De Baú Daun: Mídíọ́m Skín Ton',
    '🙇🏾' => 'Pẹ́sin Wé De Baú Daun: Mídíọ́m Dák Skín Ton',
    '🙇🏿' => 'Pẹ́sin Wé De Baú Daun: Dák Skín Ton',
    '🤦🏻' => 'Pẹ́sin Wé Ték Wọ́n Hand Kọ́vá Im Fes: Laít Skín Ton',
    '🤦🏼' => 'Pẹ́sin Wé Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m-Laít Skín Ton',
    '🤦🏽' => 'Pẹ́sin Wé Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m Skín Ton',
    '🤦🏾' => 'Pẹ́sin Wé Ték Wọ́n Hand Kọ́vá Im Fes: Mídíọ́m Dák Skín Ton',
    '🤦🏿' => 'Pẹ́sin Wé Ték Wọ́n Hand Kọ́vá Im Fes: Dák Skín Ton',
    '🤷🏻' => 'Pẹ́sin Wé De Shrọ́g Im Shólda: Laít Skín Ton',
    '🤷🏼' => 'Pẹ́sin Wé De Shrọ́g Im Shólda: Mídíọ́m-Laít Skín Ton',
    '🤷🏽' => 'Pẹ́sin Wé De Shrọ́g Im Shólda: Mídíọ́m Skín Ton',
    '🤷🏾' => 'Pẹ́sin Wé De Shrọ́g Im Shólda: Mídíọ́m Dák Skín Ton',
    '🤷🏿' => 'Pẹ́sin Wé De Shrọ́g Im Shólda: Dák Skín Ton',
    '👮🏻' => 'Polís Ọ́físa: Laít Skín Ton',
    '👮🏼' => 'Polís Ọ́físa: Mídíọ́m-Laít Skín Ton',
    '👮🏽' => 'Polís Ọ́físa: Mídíọ́m Skín Ton',
    '👮🏾' => 'Polís Ọ́físa: Mídíọ́m Dák Skín Ton',
    '👮🏿' => 'Polís Ọ́físa: Dák Skín Ton',
    '🕵🏻' => 'Ditẹ́ktiv: Laít Skín Ton',
    '🕵🏼' => 'Ditẹ́ktiv: Mídíọ́m-Laít Skín Ton',
    '🕵🏽' => 'Ditẹ́ktiv: Mídíọ́m Skín Ton',
    '🕵🏾' => 'Ditẹ́ktiv: Mídíọ́m Dák Skín Ton',
    '🕵🏿' => 'Ditẹ́ktiv: Dák Skín Ton',
    '💂🏻' => 'Gad: Laít Skín Ton',
    '💂🏼' => 'Gad: Mídíọ́m-Laít Skín Ton',
    '💂🏽' => 'Gad: Mídíọ́m Skín Ton',
    '💂🏾' => 'Gad: Mídíọ́m Dák Skín Ton',
    '💂🏿' => 'Gad: Dák Skín Ton',
    '🥷🏻' => 'Nínja: Laít Skín Ton',
    '🥷🏼' => 'Nínja: Mídíọ́m-Laít Skín Ton',
    '🥷🏽' => 'Nínja: Mídíọ́m Skín Ton',
    '🥷🏾' => 'Nínja: Mídíọ́m Dák Skín Ton',
    '🥷🏿' => 'Nínja: Dák Skín Ton',
    '👷🏻' => 'Kọnstrọ́kshọ́n Wọ́ka: Laít Skín Ton',
    '👷🏼' => 'Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m-Laít Skín Ton',
    '👷🏽' => 'Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m Skín Ton',
    '👷🏾' => 'Kọnstrọ́kshọ́n Wọ́ka: Mídíọ́m Dák Skín Ton',
    '👷🏿' => 'Kọnstrọ́kshọ́n Wọ́ka: Dák Skín Ton',
    '🫅🏻' => 'Pẹ́sin Wé Wiá Kraun: Laít Skín Ton',
    '🫅🏼' => 'Pẹ́sin Wé Wiá Kraun: Mídíọ́m-Laít Skín Ton',
    '🫅🏽' => 'Pẹ́sin Wé Wiá Kraun: Mídíọ́m Skín Ton',
    '🫅🏾' => 'Pẹ́sin Wé Wiá Kraun: Mídíọ́m Dák Skín Ton',
    '🫅🏿' => 'Pẹ́sin Wé Wiá Kraun: Dák Skín Ton',
    '🤴🏻' => 'Prins: Laít Skín Ton',
    '🤴🏼' => 'Prins: Mídíọ́m-Laít Skín Ton',
    '🤴🏽' => 'Prins: Mídíọ́m Skín Ton',
    '🤴🏾' => 'Prins: Mídíọ́m Dák Skín Ton',
    '🤴🏿' => 'Prins: Dák Skín Ton',
    '👸🏻' => 'Prínsẹs: Laít Skín Ton',
    '👸🏼' => 'Prínsẹs: Mídíọ́m-Laít Skín Ton',
    '👸🏽' => 'Prínsẹs: Mídíọ́m Skín Ton',
    '👸🏾' => 'Prínsẹs: Mídíọ́m Dák Skín Ton',
    '👸🏿' => 'Prínsẹs: Dák Skín Ton',
    '👳🏻' => 'Pẹ́sin Wé Taí Hẹd-Tai Fọ Hẹd: Laít Skín Ton',
    '👳🏼' => 'Pẹ́sin Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m-Laít Skín Ton',
    '👳🏽' => 'Pẹ́sin Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m Skín Ton',
    '👳🏾' => 'Pẹ́sin Wé Taí Hẹd-Tai Fọ Hẹd: Mídíọ́m Dák Skín Ton',
    '👳🏿' => 'Pẹ́sin Wé Taí Hẹd-Tai Fọ Hẹd: Dák Skín Ton',
    '👲🏻' => 'Pẹ́sin Wé Wiá Skọ́lkap: Laít Skín Ton',
    '👲🏼' => 'Pẹ́sin Wé Wiá Skọ́lkap: Mídíọ́m-Laít Skín Ton',
    '👲🏽' => 'Pẹ́sin Wé Wiá Skọ́lkap: Mídíọ́m Skín Ton',
    '👲🏾' => 'Pẹ́sin Wé Wiá Skọ́lkap: Mídíọ́m Dák Skín Ton',
    '👲🏿' => 'Pẹ́sin Wé Wiá Skọ́lkap: Dák Skín Ton',
    '🧕🏻' => 'Wúman Wé Taí Hẹdtai: Laít Skín Ton',
    '🧕🏼' => 'Wúman Wé Taí Hẹdtai: Mídíọ́m-Laít Skín Ton',
    '🧕🏽' => 'Wúman Wé Taí Hẹdtai: Mídíọ́m Skín Ton',
    '🧕🏾' => 'Wúman Wé Taí Hẹdtai: Mídíọ́m Dák Skín Ton',
    '🧕🏿' => 'Wúman Wé Taí Hẹdtai: Dák Skín Ton',
    '🤵🏻' => 'Pẹ́sin Wé Wiá Tọksído: Laít Skín Ton',
    '🤵🏼' => 'Pẹ́sin Wé Wiá Tọksído: Mídíọ́m-Laít Skín Ton',
    '🤵🏽' => 'Pẹ́sin Wé Wiá Tọksído: Mídíọ́m Skín Ton',
    '🤵🏾' => 'Pẹ́sin Wé Wiá Tọksído: Mídíọ́m Dák Skín Ton',
    '🤵🏿' => 'Pẹ́sin Wé Wiá Tọksído: Dák Skín Ton',
    '👰🏻' => 'Pẹ́sin Wé Wiá Vẹl: Laít Skín Ton',
    '👰🏼' => 'Pẹ́sin Wé Wiá Vẹl: Mídíọ́m-Laít Skín Ton',
    '👰🏽' => 'Pẹ́sin Wé Wiá Vẹl: Mídíọ́m Skín Ton',
    '👰🏾' => 'Pẹ́sin Wé Wiá Vẹl: Mídíọ́m Dák Skín Ton',
    '👰🏿' => 'Pẹ́sin Wé Wiá Vẹl: Dák Skín Ton',
    '🤰🏻' => 'Wúman Wé Gẹ́t Bẹlẹ́: Laít Skín Ton',
    '🤰🏼' => 'Wúman Wé Gẹ́t Bẹlẹ́: Mídíọ́m-Laít Skín Ton',
    '🤰🏽' => 'Wúman Wé Gẹ́t Bẹlẹ́: Mídíọ́m Skín Ton',
    '🤰🏾' => 'Wúman Wé Gẹ́t Bẹlẹ́: Mídíọ́m Dák Skín Ton',
    '🤰🏿' => 'Wúman Wé Gẹ́t Bẹlẹ́: Dák Skín Ton',
    '🫃🏻' => 'Man Wé Gẹ́t Bẹlẹ́: Laít Skín Ton',
    '🫃🏼' => 'Man Wé Gẹ́t Bẹlẹ́: Mídíọ́m-Laít Skín Ton',
    '🫃🏽' => 'Man Wé Gẹ́t Bẹlẹ́: Mídíọ́m Skín Ton',
    '🫃🏾' => 'Man Wé Gẹ́t Bẹlẹ́: Mídíọ́m Dák Skín Ton',
    '🫃🏿' => 'Man Wé Gẹ́t Bẹlẹ́: Dák Skín Ton',
    '🫄🏻' => 'Pẹ́sin Wé Gẹ́t Bẹlẹ́: Laít Skín Ton',
    '🫄🏼' => 'Pẹ́sin Wé Gẹ́t Bẹlẹ́: Mídíọ́m-Laít Skín Ton',
    '🫄🏽' => 'Pẹ́sin Wé Gẹ́t Bẹlẹ́: Mídíọ́m Skín Ton',
    '🫄🏾' => 'Pẹ́sin Wé Gẹ́t Bẹlẹ́: Mídíọ́m Dák Skín Ton',
    '🫄🏿' => 'Pẹ́sin Wé Gẹ́t Bẹlẹ́: Dák Skín Ton',
    '🤱🏻' => 'Wúman Wé De Gív Pikin Brẹst: Laít Skín Ton',
    '🤱🏼' => 'Wúman Wé De Gív Pikin Brẹst: Mídíọ́m-Laít Skín Ton',
    '🤱🏽' => 'Wúman Wé De Gív Pikin Brẹst: Mídíọ́m Skín Ton',
    '🤱🏾' => 'Wúman Wé De Gív Pikin Brẹst: Mídíọ́m Dák Skín Ton',
    '🤱🏿' => 'Wúman Wé De Gív Pikin Brẹst: Dák Skín Ton',
    '👼🏻' => 'Bẹ́bí Énjẹl: Laít Skín Ton',
    '👼🏼' => 'Bẹ́bí Énjẹl: Mídíọ́m-Laít Skín Ton',
    '👼🏽' => 'Bẹ́bí Énjẹl: Mídíọ́m Skín Ton',
    '👼🏾' => 'Bẹ́bí Énjẹl: Mídíọ́m Dák Skín Ton',
    '👼🏿' => 'Bẹ́bí Énjẹl: Dák Skín Ton',
    '🎅🏻' => 'Sántá Klọz: Laít Skín Ton',
    '🎅🏼' => 'Sántá Klọz: Mídíọ́m-Laít Skín Ton',
    '🎅🏽' => 'Sántá Klọz: Mídíọ́m Skín Ton',
    '🎅🏾' => 'Sántá Klọz: Mídíọ́m Dák Skín Ton',
    '🎅🏿' => 'Sántá Klọz: Dák Skín Ton',
    '🤶🏻' => 'Misis Klọz: Laít Skín Ton',
    '🤶🏼' => 'Misis Klọz: Mídíọ́m-Laít Skín Ton',
    '🤶🏽' => 'Misis Klọz: Mídíọ́m Skín Ton',
    '🤶🏾' => 'Misis Klọz: Mídíọ́m Dák Skín Ton',
    '🤶🏿' => 'Misis Klọz: Dák Skín Ton',
    '🦸🏻' => 'Supahíro: Laít Skín Ton',
    '🦸🏼' => 'Supahíro: Mídíọ́m-Laít Skín Ton',
    '🦸🏽' => 'Supahíro: Mídíọ́m Skín Ton',
    '🦸🏾' => 'Supahíro: Mídíọ́m Dák Skín Ton',
    '🦸🏿' => 'Supahíro: Dák Skín Ton',
    '🦹🏻' => 'Wíkẹ́d Krímínal: Laít Skín Ton',
    '🦹🏼' => 'Wíkẹ́d Krímínal: Mídíọ́m-Laít Skín Ton',
    '🦹🏽' => 'Wíkẹ́d Krímínal: Mídíọ́m Skín Ton',
    '🦹🏾' => 'Wíkẹ́d Krímínal: Mídíọ́m Dák Skín Ton',
    '🦹🏿' => 'Wíkẹ́d Krímínal: Dák Skín Ton',
    '🧙🏻' => 'Majíshan: Laít Skín Ton',
    '🧙🏼' => 'Majíshan: Mídíọ́m-Laít Skín Ton',
    '🧙🏽' => 'Majíshan: Mídíọ́m Skín Ton',
    '🧙🏾' => 'Majíshan: Mídíọ́m Dák Skín Ton',
    '🧙🏿' => 'Majíshan: Dák Skín Ton',
    '🧚🏻' => 'Spírit Wé Rizẹ́mbúl Taíní Húmánbin: Laít Skín Ton',
    '🧚🏼' => 'Spírit Wé Rizẹ́mbúl Taíní Húmánbin: Mídíọ́m-Laít Skín Ton',
    '🧚🏽' => 'Spírit Wé Rizẹ́mbúl Taíní Húmánbin: Mídíọ́m Skín Ton',
    '🧚🏾' => 'Spírit Wé Rizẹ́mbúl Taíní Húmánbin: Mídíọ́m Dák Skín Ton',
    '🧚🏿' => 'Spírit Wé Rizẹ́mbúl Taíní Húmánbin: Dák Skín Ton',
    '🧛🏻' => 'Vámpáya: Laít Skín Ton',
    '🧛🏼' => 'Vámpáya: Mídíọ́m-Laít Skín Ton',
    '🧛🏽' => 'Vámpáya: Mídíọ́m Skín Ton',
    '🧛🏾' => 'Vámpáya: Mídíọ́m Dák Skín Ton',
    '🧛🏿' => 'Vámpáya: Dák Skín Ton',
    '🧜🏻' => 'Pẹ́sin Wé Bi Mamíwọtá: Laít Skín Ton',
    '🧜🏼' => 'Pẹ́sin Wé Bi Mamíwọtá: Mídíọ́m-Laít Skín Ton',
    '🧜🏽' => 'Pẹ́sin Wé Bi Mamíwọtá: Mídíọ́m Skín Ton',
    '🧜🏾' => 'Pẹ́sin Wé Bi Mamíwọtá: Mídíọ́m Dák Skín Ton',
    '🧜🏿' => 'Pẹ́sin Wé Bi Mamíwọtá: Dák Skín Ton',
    '🧝🏻' => 'Ẹlf: Laít Skín Ton',
    '🧝🏼' => 'Ẹlf: Mídíọ́m-Laít Skín Ton',
    '🧝🏽' => 'Ẹlf: Mídíọ́m Skín Ton',
    '🧝🏾' => 'Ẹlf: Mídíọ́m Dák Skín Ton',
    '🧝🏿' => 'Ẹlf: Dák Skín Ton',
    '💆🏻' => 'Pẹ́sin Wé Dẹm De Masáj Im Bọdí: Laít Skín Ton',
    '💆🏼' => 'Pẹ́sin Wé Dẹm De Masáj Im Bọdí: Mídíọ́m-Laít Skín Ton',
    '💆🏽' => 'Pẹ́sin Wé Dẹm De Masáj Im Bọdí: Mídíọ́m Skín Ton',
    '💆🏾' => 'Pẹ́sin Wé Dẹm De Masáj Im Bọdí: Mídíọ́m Dák Skín Ton',
    '💆🏿' => 'Pẹ́sin Wé Dẹm De Masáj Im Bọdí: Dák Skín Ton',
    '💇🏻' => 'Pẹ́sin Wé Dẹm De Báb Im Hea: Laít Skín Ton',
    '💇🏼' => 'Pẹ́sin Wé Dẹm De Báb Im Hea: Mídíọ́m-Laít Skín Ton',
    '💇🏽' => 'Pẹ́sin Wé Dẹm De Báb Im Hea: Mídíọ́m Skín Ton',
    '💇🏾' => 'Pẹ́sin Wé Dẹm De Báb Im Hea: Mídíọ́m Dák Skín Ton',
    '💇🏿' => 'Pẹ́sin Wé Dẹm De Báb Im Hea: Dák Skín Ton',
    '🚶🏻' => 'Pẹ́sin Wé De Waká: Laít Skín Ton',
    '🚶🏼' => 'Pẹ́sin Wé De Waká: Mídíọ́m-Laít Skín Ton',
    '🚶🏽' => 'Pẹ́sin Wé De Waká: Mídíọ́m Skín Ton',
    '🚶🏾' => 'Pẹ́sin Wé De Waká: Mídíọ́m Dák Skín Ton',
    '🚶🏿' => 'Pẹ́sin Wé De Waká: Dák Skín Ton',
    '🧍🏻' => 'Pẹ́sín Wé De Stánd Ọp: Laít Skín Ton',
    '🧍🏼' => 'Pẹ́sín Wé De Stánd Ọp: Mídíọ́m-Laít Skín Ton',
    '🧍🏽' => 'Pẹ́sín Wé De Stánd Ọp: Mídíọ́m Skín Ton',
    '🧍🏾' => 'Pẹ́sín Wé De Stánd Ọp: Mídíọ́m Dák Skín Ton',
    '🧍🏿' => 'Pẹ́sín Wé De Stánd Ọp: Dák Skín Ton',
    '🧎🏻' => 'Pẹ́sin Wé Níl Daun: Laít Skín Ton',
    '🧎🏼' => 'Pẹ́sin Wé Níl Daun: Mídíọ́m-Laít Skín Ton',
    '🧎🏽' => 'Pẹ́sin Wé Níl Daun: Mídíọ́m Skín Ton',
    '🧎🏾' => 'Pẹ́sin Wé Níl Daun: Mídíọ́m Dák Skín Ton',
    '🧎🏿' => 'Pẹ́sin Wé Níl Daun: Dák Skín Ton',
    '🏃🏻' => 'Pẹ́sin Wé De Rọn: Laít Skín Ton',
    '🏃🏼' => 'Pẹ́sin Wé De Rọn: Mídíọ́m-Laít Skín Ton',
    '🏃🏽' => 'Pẹ́sin Wé De Rọn: Mídíọ́m Skín Ton',
    '🏃🏾' => 'Pẹ́sin Wé De Rọn: Mídíọ́m Dák Skín Ton',
    '🏃🏿' => 'Pẹ́sin Wé De Rọn: Dák Skín Ton',
    '💃🏻' => 'Wúman Wé De Dans: Laít Skín Ton',
    '💃🏼' => 'Wúman Wé De Dans: Mídíọ́m-Laít Skín Ton',
    '💃🏽' => 'Wúman Wé De Dans: Mídíọ́m Skín Ton',
    '💃🏾' => 'Wúman Wé De Dans: Mídíọ́m Dák Skín Ton',
    '💃🏿' => 'Wúman Wé De Dans: Dák Skín Ton',
    '🕺🏻' => 'Man Wé De Dans: Laít Skín Ton',
    '🕺🏼' => 'Man Wé De Dans: Mídíọ́m-Laít Skín Ton',
    '🕺🏽' => 'Man Wé De Dans: Mídíọ́m Skín Ton',
    '🕺🏾' => 'Man Wé De Dans: Mídíọ́m Dák Skín Ton',
    '🕺🏿' => 'Man Wé De Dans: Dák Skín Ton',
    '🕴🏻' => 'Pẹ́sin Wé Wiá Sút-An-Sut An De Flót Fọ Ea: Laít Skín Ton',
    '🕴🏼' => 'Pẹ́sin Wé Wiá Sút-An-Sut An De Flót Fọ Ea: Mídíọ́m-Laít Skín Ton',
    '🕴🏽' => 'Pẹ́sin Wé Wiá Sút-An-Sut An De Flót Fọ Ea: Mídíọ́m Skín Ton',
    '🕴🏾' => 'Pẹ́sin Wé Wiá Sút-An-Sut An De Flót Fọ Ea: Mídíọ́m Dák Skín Ton',
    '🕴🏿' => 'Pẹ́sin Wé Wiá Sút-An-Sut An De Flót Fọ Ea: Dák Skín Ton',
    '👯🏻' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Laít Skín Ton',
    '👯🏼' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m-Laít Skín Ton',
    '👯🏽' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Skín Ton',
    '👯🏾' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Mídíọ́m Dák Skín Ton',
    '👯🏿' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia: Dák Skín Ton',
    '🧖🏻' => 'Pẹ́sin Wé Dé Ínsaíd Họ́t-Wọ́ta-Smók Rum: Laít Skín Ton',
    '🧖🏼' => 'Pẹ́sin Wé Dé Ínsaíd Họ́t-Wọ́ta-Smók Rum: Mídíọ́m-Laít Skín Ton',
    '🧖🏽' => 'Pẹ́sin Wé Dé Ínsaíd Họ́t-Wọ́ta-Smók Rum: Mídíọ́m Skín Ton',
    '🧖🏾' => 'Pẹ́sin Wé Dé Ínsaíd Họ́t-Wọ́ta-Smók Rum: Mídíọ́m Dák Skín Ton',
    '🧖🏿' => 'Pẹ́sin Wé Dé Ínsaíd Họ́t-Wọ́ta-Smók Rum: Dák Skín Ton',
    '🧗🏻' => 'Pẹ́sin Wé De Klaimb: Laít Skín Ton',
    '🧗🏼' => 'Pẹ́sin Wé De Klaimb: Mídíọ́m-Laít Skín Ton',
    '🧗🏽' => 'Pẹ́sin Wé De Klaimb: Mídíọ́m Skín Ton',
    '🧗🏾' => 'Pẹ́sin Wé De Klaimb: Mídíọ́m Dák Skín Ton',
    '🧗🏿' => 'Pẹ́sin Wé De Klaimb: Dák Skín Ton',
    '🏇🏻' => 'Pẹ́sin Wé De Rọ́n Wit Họs: Laít Skín Ton',
    '🏇🏼' => 'Pẹ́sin Wé De Rọ́n Wit Họs: Mídíọ́m-Laít Skín Ton',
    '🏇🏽' => 'Pẹ́sin Wé De Rọ́n Wit Họs: Mídíọ́m Skín Ton',
    '🏇🏾' => 'Pẹ́sin Wé De Rọ́n Wit Họs: Mídíọ́m Dák Skín Ton',
    '🏇🏿' => 'Pẹ́sin Wé De Rọ́n Wit Họs: Dák Skín Ton',
    '🏂🏻' => 'Pẹ́sin Wé De Raíd Wúdín-Bọd Ọn Tọ́p Sno: Laít Skín Ton',
    '🏂🏼' => 'Pẹ́sin Wé De Raíd Wúdín-Bọd Ọn Tọ́p Sno: Mídíọ́m-Laít Skín Ton',
    '🏂🏽' => 'Pẹ́sin Wé De Raíd Wúdín-Bọd Ọn Tọ́p Sno: Mídíọ́m Skín Ton',
    '🏂🏾' => 'Pẹ́sin Wé De Raíd Wúdín-Bọd Ọn Tọ́p Sno: Mídíọ́m Dák Skín Ton',
    '🏂🏿' => 'Pẹ́sin Wé De Raíd Wúdín-Bọd Ọn Tọ́p Sno: Dák Skín Ton',
    '🏌🏻' => 'Pẹ́sin Wé De Plé Gọlf: Laít Skín Ton',
    '🏌🏼' => 'Pẹ́sin Wé De Plé Gọlf: Mídíọ́m-Laít Skín Ton',
    '🏌🏽' => 'Pẹ́sin Wé De Plé Gọlf: Mídíọ́m Skín Ton',
    '🏌🏾' => 'Pẹ́sin Wé De Plé Gọlf: Mídíọ́m Dák Skín Ton',
    '🏌🏿' => 'Pẹ́sin Wé De Plé Gọlf: Dák Skín Ton',
    '🏄🏻' => 'Pẹ́sin Wé De Raíd Wọtá Sọ́fín-Bọd: Laít Skín Ton',
    '🏄🏼' => 'Pẹ́sin Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m-Laít Skín Ton',
    '🏄🏽' => 'Pẹ́sin Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m Skín Ton',
    '🏄🏾' => 'Pẹ́sin Wé De Raíd Wọtá Sọ́fín-Bọd: Mídíọ́m Dák Skín Ton',
    '🏄🏿' => 'Pẹ́sin Wé De Raíd Wọtá Sọ́fín-Bọd: Dák Skín Ton',
    '🚣🏻' => 'Pẹ́sin Wé Yúz Pádul De Ró Bot: Laít Skín Ton',
    '🚣🏼' => 'Pẹ́sin Wé Yúz Pádul De Ró Bot: Mídíọ́m-Laít Skín Ton',
    '🚣🏽' => 'Pẹ́sin Wé Yúz Pádul De Ró Bot: Mídíọ́m Skín Ton',
    '🚣🏾' => 'Pẹ́sin Wé Yúz Pádul De Ró Bot: Mídíọ́m Dák Skín Ton',
    '🚣🏿' => 'Pẹ́sin Wé Yúz Pádul De Ró Bot: Dák Skín Ton',
    '🏊🏻' => 'Pẹ́sin Wé De Swim: Laít Skín Ton',
    '🏊🏼' => 'Pẹ́sin Wé De Swim: Mídíọ́m-Laít Skín Ton',
    '🏊🏽' => 'Pẹ́sin Wé De Swim: Mídíọ́m Skín Ton',
    '🏊🏾' => 'Pẹ́sin Wé De Swim: Mídíọ́m Dák Skín Ton',
    '🏊🏿' => 'Pẹ́sin Wé De Swim: Dák Skín Ton',
    '⛹🏻' => 'Pẹ́sin Wé De Baúns Bọl Fọ Graun: Laít Skín Ton',
    '⛹🏼' => 'Pẹ́sin Wé De Baúns Bọl Fọ Graun: Mídíọ́m-Laít Skín Ton',
    '⛹🏽' => 'Pẹ́sin Wé De Baúns Bọl Fọ Graun: Mídíọ́m Skín Ton',
    '⛹🏾' => 'Pẹ́sin Wé De Baúns Bọl Fọ Graun: Mídíọ́m Dák Skín Ton',
    '⛹🏿' => 'Pẹ́sin Wé De Baúns Bọl Fọ Graun: Dák Skín Ton',
    '🏋🏻' => 'Pẹ́sin Wé De Kárí Ọ́p Hẹ́ví Tins: Laít Skín Ton',
    '🏋🏼' => 'Pẹ́sin Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m-Laít Skín Ton',
    '🏋🏽' => 'Pẹ́sin Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m Skín Ton',
    '🏋🏾' => 'Pẹ́sin Wé De Kárí Ọ́p Hẹ́ví Tins: Mídíọ́m Dák Skín Ton',
    '🏋🏿' => 'Pẹ́sin Wé De Kárí Ọ́p Hẹ́ví Tins: Dák Skín Ton',
    '🚴🏻' => 'Pẹ́sin Wé De Dú Raídín Baísíkul Spọt: Laít Skín Ton',
    '🚴🏼' => 'Pẹ́sin Wé De Dú Raídín Baísíkul Spọt: Mídíọ́m-Laít Skín Ton',
    '🚴🏽' => 'Pẹ́sin Wé De Dú Raídín Baísíkul Spọt: Mídíọ́m Skín Ton',
    '🚴🏾' => 'Pẹ́sin Wé De Dú Raídín Baísíkul Spọt: Mídíọ́m Dák Skín Ton',
    '🚴🏿' => 'Pẹ́sin Wé De Dú Raídín Baísíkul Spọt: Dák Skín Ton',
    '🚵🏻' => 'Pẹ́sin Wé De Raíd Baísíkul Fọ Maúntin: Laít Skín Ton',
    '🚵🏼' => 'Pẹ́sin Wé De Raíd Baísíkul Fọ Maúntin: Mídíọ́m-Laít Skín Ton',
    '🚵🏽' => 'Pẹ́sin Wé De Raíd Baísíkul Fọ Maúntin: Mídíọ́m Skín Ton',
    '🚵🏾' => 'Pẹ́sin Wé De Raíd Baísíkul Fọ Maúntin: Mídíọ́m Dák Skín Ton',
    '🚵🏿' => 'Pẹ́sin Wé De Raíd Baísíkul Fọ Maúntin: Dák Skín Ton',
    '🤸🏻' => 'Pẹ́sin Wé De Dú Kátwil: Laít Skín Ton',
    '🤸🏼' => 'Pẹ́sin Wé De Dú Kátwil: Mídíọ́m-Laít Skín Ton',
    '🤸🏽' => 'Pẹ́sin Wé De Dú Kátwil: Mídíọ́m Skín Ton',
    '🤸🏾' => 'Pẹ́sin Wé De Dú Kátwil: Mídíọ́m Dák Skín Ton',
    '🤸🏿' => 'Pẹ́sin Wé De Dú Kátwil: Dák Skín Ton',
    '🤼🏻' => 'Pípul Wé De Rẹ́sul: Laít Skín Ton',
    '🤼🏼' => 'Pípul Wé De Rẹ́sul: Mídíọ́m-Laít Skín Ton',
    '🤼🏽' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Skín Ton',
    '🤼🏾' => 'Pípul Wé De Rẹ́sul: Mídíọ́m Dák Skín Ton',
    '🤼🏿' => 'Pípul Wé De Rẹ́sul: Dák Skín Ton',
    '🤽🏻' => 'Pẹ́sin Wé De Plé Wọ́tá Poló: Laít Skín Ton',
    '🤽🏼' => 'Pẹ́sin Wé De Plé Wọ́tá Poló: Mídíọ́m-Laít Skín Ton',
    '🤽🏽' => 'Pẹ́sin Wé De Plé Wọ́tá Poló: Mídíọ́m Skín Ton',
    '🤽🏾' => 'Pẹ́sin Wé De Plé Wọ́tá Poló: Mídíọ́m Dák Skín Ton',
    '🤽🏿' => 'Pẹ́sin Wé De Plé Wọ́tá Poló: Dák Skín Ton',
    '🤾🏻' => 'Pẹ́sin Wé De Plé Hándbọl: Laít Skín Ton',
    '🤾🏼' => 'Pẹ́sin Wé De Plé Hándbọl: Mídíọ́m-Laít Skín Ton',
    '🤾🏽' => 'Pẹ́sin Wé De Plé Hándbọl: Mídíọ́m Skín Ton',
    '🤾🏾' => 'Pẹ́sin Wé De Plé Hándbọl: Mídíọ́m Dák Skín Ton',
    '🤾🏿' => 'Pẹ́sin Wé De Plé Hándbọl: Dák Skín Ton',
    '🤹🏻' => 'Pẹ́sin Wé De Jọ́gúl Bọl-dẹm: Laít Skín Ton',
    '🤹🏼' => 'Pẹ́sin Wé De Jọ́gúl Bọl-dẹm: Mídíọ́m-Laít Skín Ton',
    '🤹🏽' => 'Pẹ́sin Wé De Jọ́gúl Bọl-dẹm: Mídíọ́m Skín Ton',
    '🤹🏾' => 'Pẹ́sin Wé De Jọ́gúl Bọl-dẹm: Mídíọ́m Dák Skín Ton',
    '🤹🏿' => 'Pẹ́sin Wé De Jọ́gúl Bọl-dẹm: Dák Skín Ton',
    '🧘🏻' => 'Pẹ́sin Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Laít Skín Ton',
    '🧘🏼' => 'Pẹ́sin Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m-Laít Skín Ton',
    '🧘🏽' => 'Pẹ́sin Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m Skín Ton',
    '🧘🏾' => 'Pẹ́sin Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Mídíọ́m Dák Skín Ton',
    '🧘🏿' => 'Pẹ́sin Wé Sidọ́n Fọ Graun Fọ Lótús Wè: Dák Skín Ton',
    '🛀🏻' => 'Pẹ́sin Wé De Baf: Laít Skín Ton',
    '🛀🏼' => 'Pẹ́sin Wé De Baf: Mídíọ́m-Laít Skín Ton',
    '🛀🏽' => 'Pẹ́sin Wé De Baf: Mídíọ́m Skín Ton',
    '🛀🏾' => 'Pẹ́sin Wé De Baf: Mídíọ́m Dák Skín Ton',
    '🛀🏿' => 'Pẹ́sin Wé De Baf: Dák Skín Ton',
    '🛌🏻' => 'Pẹ́sin Wé De Slip: Laít Skín Ton',
    '🛌🏼' => 'Pẹ́sin Wé De Slip: Mídíọ́m-Laít Skín Ton',
    '🛌🏽' => 'Pẹ́sin Wé De Slip: Mídíọ́m Skín Ton',
    '🛌🏾' => 'Pẹ́sin Wé De Slip: Mídíọ́m Dák Skín Ton',
    '🛌🏿' => 'Pẹ́sin Wé De Slip: Dák Skín Ton',
    '👭🏻' => 'Wímẹn Wé Hól Hand: Laít Skín Ton',
    '👭🏼' => 'Wímẹn Wé Hól Hand: Mídíọ́m-Laít Skín Ton',
    '👭🏽' => 'Wímẹn Wé Hól Hand: Mídíọ́m Skín Ton',
    '👭🏾' => 'Wímẹn Wé Hól Hand: Mídíọ́m Dák Skín Ton',
    '👭🏿' => 'Wímẹn Wé Hól Hand: Dák Skín Ton',
    '👫🏻' => 'Wúman An Man Wé Hól Hand: Laít Skín Ton',
    '👫🏼' => 'Wúman An Man Wé Hól Hand: Mídíọ́m-Laít Skín Ton',
    '👫🏽' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Skín Ton',
    '👫🏾' => 'Wúman An Man Wé Hól Hand: Mídíọ́m Dák Skín Ton',
    '👫🏿' => 'Wúman An Man Wé Hól Hand: Dák Skín Ton',
    '👬🏻' => 'Man-dẹm Wé Hól Han: Laít Skín Ton',
    '👬🏼' => 'Man-dẹm Wé Hól Han: Mídíọ́m-Laít Skín Ton',
    '👬🏽' => 'Man-dẹm Wé Hól Han: Mídíọ́m Skín Ton',
    '👬🏾' => 'Man-dẹm Wé Hól Han: Mídíọ́m Dák Skín Ton',
    '👬🏿' => 'Man-dẹm Wé Hól Han: Dák Skín Ton',
    '💏🏻' => 'Kis: Laít Skín Ton',
    '💏🏼' => 'Kis: Mídíọ́m-Laít Skín Ton',
    '💏🏽' => 'Kis: Mídíọ́m Skín Ton',
    '💏🏾' => 'Kis: Mídíọ́m Dák Skín Ton',
    '💏🏿' => 'Kis: Dák Skín Ton',
    '💑🏻' => 'Tuú Pípul Wit Hat: Laít Skín Ton',
    '💑🏼' => 'Tuú Pípul Wit Hat: Mídíọ́m-Laít Skín Ton',
    '💑🏽' => 'Tuú Pípul Wit Hat: Mídíọ́m Skín Ton',
    '💑🏾' => 'Tuú Pípul Wit Hat: Mídíọ́m Dák Skín Ton',
    '💑🏿' => 'Tuú Pípul Wit Hat: Dák Skín Ton',
    '🇦🇨' => 'Flag: Asẹ́nshọ́n Aíland',
    '🇦🇩' => 'Flag: Andọ́ra',
    '🇦🇪' => 'Flag: Yunaítẹ́d Áráb Ẹ́mírets',
    '🇦🇫' => 'Flag: Afgánístan',
    '🇦🇬' => 'Flag: Antígwua & Barbúda',
    '🇦🇮' => 'Flag: Angwíla',
    '🇦🇱' => 'Flag: Albénia',
    '🇦🇲' => 'Flag: Armẹ́niá',
    '🇦🇴' => 'Flag: Angóla',
    '🇦🇶' => 'Flag: Antáktíka',
    '🇦🇷' => 'Flag: Ajẹntína',
    '🇦🇸' => 'Flag: Amẹ́ríká Samoa',
    '🇦🇹' => 'Flag: Ọ́stria',
    '🇦🇺' => 'Flag: Ọstrélia',
    '🇦🇼' => 'Flag: Arúba',
    '🇦🇽' => 'Flag: Ọ́lánd Aílands',
    '🇦🇿' => 'Flag: Azẹrbaijan',
    '🇧🇦' => 'Flag: Bọ́zniá & Hẹzẹgovína',
    '🇧🇧' => 'Flag: Barbédọs',
    '🇧🇩' => 'Flag: Bangladẹsh',
    '🇧🇪' => 'Flag: Bẹ́ljọm',
    '🇧🇫' => 'Flag: Burkína Fáso',
    '🇧🇬' => 'Flag: Bọlgéria',
    '🇧🇭' => 'Flag: Barein',
    '🇧🇮' => 'Flag: Burúndi',
    '🇧🇯' => 'Flag: Binin',
    '🇧🇱' => 'Flag: Sént Batẹlẹ́mi',
    '🇧🇲' => 'Flag: Bẹmiúda',
    '🇧🇳' => 'Flag: Brunẹi',
    '🇧🇴' => 'Flag: Bolívia',
    '🇧🇶' => 'Flag: Karíbián Nẹ́dalands',
    '🇧🇷' => 'Flag: Brázil',
    '🇧🇸' => 'Flag: Bahámas',
    '🇧🇹' => 'Flag: Butan',
    '🇧🇻' => 'Flag: Buvẹ́ Aíland',
    '🇧🇼' => 'Flag: Botswána',
    '🇧🇾' => 'Flag: Bẹ́larus',
    '🇧🇿' => 'Flag: Bẹliz',
    '🇨🇦' => 'Flag: Kánáda',
    '🇨🇨' => 'Flag: Kókós Aílands',
    '🇨🇩' => 'Flag: Kóngó – Kinshása',
    '🇨🇫' => 'Flag: Sẹ́ntrál Áfríkán Ripọ́blik',
    '🇨🇬' => 'Flag: Kóngo – Brázavil',
    '🇨🇭' => 'Flag: Swítsaland',
    '🇨🇮' => 'Flag: Aívri Kost',
    '🇨🇰' => 'Flag: Kúk Aílands',
    '🇨🇱' => 'Flag: Chílẹ',
    '🇨🇲' => 'Flag: Kamẹrun',
    '🇨🇳' => 'Flag: Chaína',
    '🇨🇴' => 'Flag: Kolómbia',
    '🇨🇵' => 'Flag: Klipatọ́n Aíland',
    '🇨🇶' => 'Flag: Sák',
    '🇨🇷' => 'Flag: Kósta Ríka',
    '🇨🇺' => 'Flag: Kiúbá',
    '🇨🇻' => 'Flag: Kép Vẹ́d',
    '🇨🇼' => 'Flag: Kiurásao',
    '🇨🇽' => 'Flag: Krísmás Aíland',
    '🇨🇾' => 'Flag: Saíprọs',
    '🇨🇿' => 'Flag: Chẹ́kia',
    '🇩🇪' => 'Flag: Jámáni',
    '🇩🇬' => 'Flag: Diẹ́gó Garsia',
    '🇩🇯' => 'Flag: Jibúti',
    '🇩🇰' => 'Flag: Dẹ́nmak',
    '🇩🇲' => 'Flag: Dọmíníka',
    '🇩🇴' => 'Flag: Dọmíníka Ripọ́blik',
    '🇩🇿' => 'Flag: Aljíria',
    '🇪🇦' => 'Flag: Sẹúta & Mẹ́líla',
    '🇪🇨' => 'Flag: Ẹ́kwuádọ',
    '🇪🇪' => 'Flag: Ẹstónia',
    '🇪🇬' => 'Flag: Íjipt',
    '🇪🇭' => 'Flag: Wẹ́stán Sahára',
    '🇪🇷' => 'Flag: Ẹritrẹ́a',
    '🇪🇸' => 'Flag: Spen',
    '🇪🇹' => 'Flag: Ẹtiópia',
    '🇪🇺' => 'Flag: Yurópián Yúniọ́n',
    '🇫🇮' => 'Flag: Fínland',
    '🇫🇯' => 'Flag: Fíji',
    '🇫🇰' => 'Flag: Fọ́klánd Aílands',
    '🇫🇲' => 'Flag: Maikroníshia',
    '🇫🇴' => 'Flag: Fáro Aílands',
    '🇫🇷' => 'Flag: Frans',
    '🇬🇦' => 'Flag: Gabọn',
    '🇬🇧' => 'Flag: Yunáítẹ́d Kíndọm',
    '🇬🇩' => 'Flag: Grẹnéda',
    '🇬🇪' => 'Flag: Jọ́jia',
    '🇬🇫' => 'Flag: Frẹ́nch Giána',
    '🇬🇬' => 'Flag: Guẹnzi',
    '🇬🇭' => 'Flag: Gána',
    '🇬🇮' => 'Flag: Jibrọ́lta',
    '🇬🇱' => 'Flag: Grínland',
    '🇬🇲' => 'Flag: Gámbia',
    '🇬🇳' => 'Flag: Gíni',
    '🇬🇵' => 'Flag: Guadalúpẹ',
    '🇬🇶' => 'Flag: Ikwétóriál Gíni',
    '🇬🇷' => 'Flag: Gris',
    '🇬🇸' => 'Flag: Saút Jọ́jia an Saút Sándwích Aílands',
    '🇬🇹' => 'Flag: Guátẹmála',
    '🇬🇺' => 'Flag: Guám',
    '🇬🇼' => 'Flag: Gíní-Bisáu',
    '🇬🇾' => 'Flag: Gayána',
    '🇭🇰' => 'Flag: Họng Kọng SAR',
    '🇭🇲' => 'Flag: Hiád & MakDónáld Aílands',
    '🇭🇳' => 'Flag: Họndúras',
    '🇭🇷' => 'Flag: Kroéshia',
    '🇭🇹' => 'Flag: Haíti',
    '🇭🇺' => 'Flag: Họ́ngári',
    '🇮🇨' => 'Flag: Kenerí Aílands',
    '🇮🇩' => 'Flag: Indoníshia',
    '🇮🇪' => 'Flag: Ayaland',
    '🇮🇱' => 'Flag: Ízrẹl',
    '🇮🇲' => 'Flag: Aíl ọf Man',
    '🇮🇳' => 'Flag: Índia',
    '🇮🇴' => 'Flag: Brítísh Índián Óshen Tẹ́rẹ́tri',
    '🇮🇶' => 'Flag: Irak',
    '🇮🇷' => 'Flag: Irán',
    '🇮🇸' => 'Flag: Aísland',
    '🇮🇹' => 'Flag: Ítáli',
    '🇯🇪' => 'Flag: Jẹ́si',
    '🇯🇲' => 'Flag: Jamaíka',
    '🇯🇴' => 'Flag: Jọ́dan',
    '🇯🇵' => 'Flag: Japán',
    '🇰🇪' => 'Flag: Kẹ́nya',
    '🇰🇬' => 'Flag: Kẹjístan',
    '🇰🇭' => 'Flag: Kambódia',
    '🇰🇮' => 'Flag: Kiribáti',
    '🇰🇲' => 'Flag: Kọ́mọ́ros',
    '🇰🇳' => 'Flag: Sent Kits & Nẹ́vis',
    '🇰🇵' => 'Flag: Nọ́t Koria',
    '🇰🇷' => 'Flag: Saút Koria',
    '🇰🇼' => 'Flag: Kuwét',
    '🇰🇾' => 'Flag: Kéman Aílands',
    '🇰🇿' => 'Flag: Kazakstan',
    '🇱🇦' => 'Flag: Láos',
    '🇱🇧' => 'Flag: Lẹ́bánọn',
    '🇱🇨' => 'Flag: Sent Lúshia',
    '🇱🇮' => 'Flag: Líktẹ́nstain',
    '🇱🇰' => 'Flag: Sri Lánka',
    '🇱🇷' => 'Flag: Laibẹ́ria',
    '🇱🇸' => 'Flag: Lẹsóto',
    '🇱🇹' => 'Flag: Lituénia',
    '🇱🇺' => 'Flag: Lọ́ksẹ́mbọg',
    '🇱🇻' => 'Flag: Látvia',
    '🇱🇾' => 'Flag: Líbia',
    '🇲🇦' => 'Flag: Morọko',
    '🇲🇨' => 'Flag: Mọ́náko',
    '🇲🇩' => 'Flag: Mọldóva',
    '🇲🇪' => 'Flag: Mọntinígro',
    '🇲🇫' => 'Flag: Sent Mátin',
    '🇲🇬' => 'Flag: Madagáska',
    '🇲🇭' => 'Flag: Máshál Aílands',
    '🇲🇰' => 'Flag: Nọ́t Masidónia',
    '🇲🇱' => 'Flag: Máli',
    '🇲🇲' => 'Flag: Miánma (Bọ́ma)',
    '🇲🇳' => 'Flag: Mọngólia',
    '🇲🇴' => 'Flag: Makáo SAR Chaína',
    '🇲🇵' => 'Flag: Nọ́tán Mariána Aílands',
    '🇲🇶' => 'Flag: Matínik',
    '🇲🇷' => 'Flag: Mọriténia',
    '🇲🇸' => 'Flag: Mọntsẹrat',
    '🇲🇹' => 'Flag: Mọ́lta',
    '🇲🇺' => 'Flag: Mọríshọs',
    '🇲🇻' => 'Flag: Mọ́ldivs',
    '🇲🇼' => 'Flag: Maláwi',
    '🇲🇽' => 'Flag: Mẹ́ksíko',
    '🇲🇾' => 'Flag: Maléshia',
    '🇲🇿' => 'Flag: Mozámbik',
    '🇳🇦' => 'Flag: Namíbia',
    '🇳🇨' => 'Flag: Niú Kalẹdónia',
    '🇳🇪' => 'Flag: Nizhẹr',
    '🇳🇫' => 'Flag: Nọ́fọlk Aíland',
    '🇳🇬' => 'Flag: Naijíria',
    '🇳🇮' => 'Flag: Nikarágwua',
    '🇳🇱' => 'Flag: Nẹ́dalands',
    '🇳🇴' => 'Flag: Nọ́we',
    '🇳🇵' => 'Flag: Nẹ́pal',
    '🇳🇷' => 'Flag: Náuru',
    '🇳🇺' => 'Flag: Niúẹ',
    '🇳🇿' => 'Flag: Niú Zíland',
    '🇴🇲' => 'Flag: Omán',
    '🇵🇦' => 'Flag: Pánáma',
    '🇵🇪' => 'Flag: Pẹ́ru',
    '🇵🇫' => 'Flag: Frẹ́nch Poliníshia',
    '🇵🇬' => 'Flag: Pápuá Niú Gíni',
    '🇵🇭' => 'Flag: Fílípins',
    '🇵🇰' => 'Flag: Pakístan',
    '🇵🇱' => 'Flag: Póland',
    '🇵🇲' => 'Flag: Sent Piẹr & Míkẹlọn',
    '🇵🇳' => 'Flag: Pítkén Aílands',
    '🇵🇷' => 'Flag: Puẹ́rto Ríkọ',
    '🇵🇸' => 'Flag: Pálẹ́staín Éria-dẹm',
    '🇵🇹' => 'Flag: Pọ́túgal',
    '🇵🇼' => 'Flag: Paláu',
    '🇵🇾' => 'Flag: Párágwue',
    '🇶🇦' => 'Flag: Kata',
    '🇷🇪' => 'Flag: Réyúniọn',
    '🇷🇴' => 'Flag: Ruménia',
    '🇷🇸' => 'Flag: Sẹ́bia',
    '🇷🇺' => 'Flag: Rọ́shia',
    '🇷🇼' => 'Flag: Ruwánda',
    '🇸🇦' => 'Flag: Saúdí Arébia',
    '🇸🇧' => 'Flag: Sólómọ́n Aílands',
    '🇸🇨' => 'Flag: Sẹ́chẹls',
    '🇸🇩' => 'Flag: Sudán',
    '🇸🇪' => 'Flag: Swídẹn',
    '🇸🇬' => 'Flag: Singapọ',
    '🇸🇭' => 'Flag: Sent Hẹlẹ́na',
    '🇸🇮' => 'Flag: Slovẹ́nia',
    '🇸🇯' => 'Flag: Sválbad & Jén Meyẹn',
    '🇸🇰' => 'Flag: Slovékia',
    '🇸🇱' => 'Flag: Siẹ́ra Líon',
    '🇸🇲' => 'Flag: San Maríno',
    '🇸🇳' => 'Flag: Sẹ́nẹ́gal',
    '🇸🇴' => 'Flag: Sọmália',
    '🇸🇷' => 'Flag: Súrínam',
    '🇸🇸' => 'Flag: Saút Sudan',
    '🇸🇹' => 'Flag: Sao Tómé & Prínsípẹ',
    '🇸🇻' => 'Flag: El Sálvádọ',
    '🇸🇽' => 'Flag: Sint Mátin',
    '🇸🇾' => 'Flag: Síria',
    '🇸🇿' => 'Flag: Ẹswatíni',
    '🇹🇦' => 'Flag: Trístán da Kúna',
    '🇹🇨' => 'Flag: Tọks an Kaíkọ́s Aílands',
    '🇹🇩' => 'Flag: Chád',
    '🇹🇫' => 'Flag: Frẹ́nch Saútán Tẹ́rẹ́tris',
    '🇹🇬' => 'Flag: Tógo',
    '🇹🇭' => 'Flag: Taíland',
    '🇹🇯' => 'Flag: Tajíkstan',
    '🇹🇰' => 'Flag: Tókẹ́lau',
    '🇹🇱' => 'Flag: Íst Tímọ',
    '🇹🇲' => 'Flag: Tọkmẹ́nístan',
    '🇹🇳' => 'Flag: Tuníshia',
    '🇹🇴' => 'Flag: Tónga',
    '🇹🇷' => 'Flag: Tọ́ki',
    '🇹🇹' => 'Flag: Trínídad & Tobágo',
    '🇹🇻' => 'Flag: Tuválu',
    '🇹🇼' => 'Flag: Taíwán',
    '🇹🇿' => 'Flag: Tanzánia',
    '🇺🇦' => 'Flag: Yukrein',
    '🇺🇬' => 'Flag: Yugánda',
    '🇺🇲' => 'Flag: U.S. Faá Faá Aílands',
    '🇺🇳' => 'Flag: Yunaítẹd Néshọns',
    '🇺🇸' => 'Flag: Yunaítẹ́d Stets',
    '🇺🇾' => 'Flag: Yúrugwue',
    '🇺🇿' => 'Flag: Uzbẹ́kistan',
    '🇻🇦' => 'Flag: Vátíkán Síti',
    '🇻🇨' => 'Flag: Sent Vínsẹnt & Grẹ́nádians',
    '🇻🇪' => 'Flag: Vẹnẹzuẹ́la',
    '🇻🇬' => 'Flag: Brítísh Vájín Aílands',
    '🇻🇮' => 'Flag: U.S. Vájín Aílands',
    '🇻🇳' => 'Flag: Viẹ́tnam',
    '🇻🇺' => 'Flag: Vanuátu',
    '🇼🇫' => 'Flag: Wọ́lis & Fiutúna',
    '🇼🇸' => 'Flag: Samóa',
    '🇽🇰' => 'Flag: Kósóvo',
    '🇾🇪' => 'Flag: Yẹ́mẹn',
    '🇾🇹' => 'Flag: Meyọt',
    '🇿🇦' => 'Flag: Saút Áfríka',
    '🇿🇲' => 'Flag: Zámbia',
    '🇿🇼' => 'Flag: Zimbábwẹ',
    '🔟' => 'Kíkap: 10',
    '🏻' => 'Laít Skín Ton',
    '🏼' => 'Mídíọ́m-Laít Skín Ton',
    '🏽' => 'Mídíọ́m Skín Ton',
    '🏾' => 'Mídíọ́m Dák Skín Ton',
    '🏿' => 'Dák Skín Ton',
    '😀' => 'Láfín Fes',
    '😃' => 'Láfín Fes Wé Gẹ́t Bíg Ai-dẹm',
    '😄' => 'Láfín Fés Wit Shaínín Ai',
    '😁' => 'Bíg Laf Wit Wait Tit',
    '😆' => 'Láfín Fes Wit Klóz Ai',
    '😅' => 'Láfín Fes With Smọ́l Swẹt',
    '🤣' => 'Bẹ́nd Hẹd Wọ́n Said De Laf',
    '😂' => 'Fes Wit Laf and Tias Fọ Ai',
    '🙂' => 'Smaílín Fes Wit Jẹ́ntúl Ai',
    '🙃' => 'Ọ́psaíd-daún Fes',
    '🫠' => 'Fes Wé De Mẹ́lt Laik Ọ́yil',
    '😉' => 'Fes Wé Klóz Wọ́n Ai',
    '😊' => 'Smaílín Fes Wit Shaínín Ai',
    '😇' => 'Smaílín Fes With Raúnd Lait Fọ Hẹd',
    '🥰' => 'Smailín Fes Wit Trí Hat',
    '😍' => 'Lafín Fes Wit Hat-ai-dẹm',
    '🤩' => 'Láfín Fes Wit Sta Fọ Ai',
    '😘' => 'Maut Wé De Bló Kis',
    '😗' => 'Kísín Fes',
    '☺' => 'Smaílín Fes',
    '😚' => 'Kísín Fes Wit Klóz Ai',
    '😙' => 'Kísín Fes Wit Smaílín Ai',
    '🥲' => 'Smaílín Fes Wit Tis Fọ Ai',
    '😋' => 'Fes Wé De Ẹnjọí Fud',
    '😛' => 'Fes Wit Tọng',
    '😜' => 'Wínkín Fes Wit Tọng',
    '🤪' => 'Fes De Mék Pẹ́sin Laf',
    '😝' => 'Fes Klóz Ai Smọl De Luk',
    '🤑' => 'Mọní-Maút Fes',
    '🤗' => 'Hand De Họ́l Fes',
    '🤭' => 'Fes Wit Hand Fọ Maut',
    '🫢' => 'Fes Wé Gẹ́t Ópún Ai An Ték Hand Kọ́vá Maut',
    '🫣' => 'Fes Wé De Lúk Wit Wọ́n Ai',
    '🤫' => 'Fes Pút Fínga Fọ Maut De Mék Shhhh',
    '🤔' => 'Fes De Tink',
    '🫡' => 'Fes Wé De Trowé Salut',
    '🤐' => 'Zíp-Maut Fes',
    '🤨' => 'Fes Wé Réz Ọ́p Aíbrau',
    '😐' => 'Plén Fes',
    '😑' => 'Fes Nọ́ Shó Ẹ́nítin Wé De Hápún To Am',
    '😶' => 'Fes Wé Nọ́ Gẹ́t Maut',
    '🫥' => 'Fes Wé Gẹ́t Dọt-dọt Lain',
    '😏' => 'Fes Wit Bẹn Bẹ́n Smail',
    '😒' => 'Fes Wé Nọ́ Dé Hapí',
    '🙄' => 'Fes De Tọ́n Im Ai-dẹm',
    '😬' => 'Ọ́glí Fes Wé Sọ́mtin De Pen',
    '🤥' => 'Lai-laí Fes',
    '🫨' => 'Shek Shék Fes',
    '😌' => 'Fes Wé Riláks Wẹ́l',
    '😔' => 'Fes Wé Dé Tínk Díp Dip',
    '😪' => 'Fes Wé Slip De Kach',
    '🤤' => 'Fes Wé Salaíva De Kọmọ́t Fọ Maut',
    '😴' => 'Fes Wé De Slip',
    '🫩' => 'fes wé gẹ́t bíg bag under ai',
    '😷' => 'Fes Wé Wía Mẹ́díkál Mask',
    '🤒' => 'Fes Wit Tamómẹ́ta',
    '🤕' => 'Fés Wit Bándej Fọ Hẹd',
    '🤢' => 'Fes Wé Wán Vọmit',
    '🤮' => 'Fes Wé De Vọmit',
    '🤧' => 'Fes Wé De Sniz',
    '🥵' => 'Họ́t Fes',
    '🥶' => 'Fes Wé Kold De Kach',
    '🥴' => 'Dízí Fes',
    '😵' => 'Fes Wé Wán Fent',
    '🤯' => 'Hẹd Wé Wán Skáta',
    '🤠' => 'Kaúbọi Hát Fes',
    '🥳' => 'Fes Wé Dé Fọ Patí',
    '🥸' => 'Fes Wé Dẹm De Haid',
    '😎' => 'Smaílín Fes Wé Wiá Sọ́nshed',
    '🤓' => 'Tẹ́kíníkál Ẹ́spat Fes',
    '🧐' => 'Fes Wit Wọ́n Ai Aíglas',
    '😕' => 'Fes Wé Nọ́ Nó Wétín I Go Du',
    '🫤' => 'Fes Wé Gẹ́t Bẹn Bẹ́n Maut',
    '😟' => 'Fes Wé Sọ́mtin De Wọ́ri',
    '🙁' => 'Fes Wé De Fraún Smọ́l Smọl',
    '☹' => 'Fes Wé De Fraun',
    '😮' => 'Fes Wé Gẹt Ópún Maut',
    '😯' => 'Fes Wé Dé Kwáyẹt',
    '😲' => 'Fes Wé Sọpraiz Fúl Am',
    '😳' => 'Rẹ́d Fes',
    '🫪' => 'dístọted fes',
    '🥺' => 'Fes Wé De Bẹ́g Fọ Sọ́mtin',
    '🥹' => 'Fes Wé De Hól Imsẹf Mék I Nọ́ Krai',
    '😦' => 'Fes Wé De Fraún Wit Ópún Maut',
    '😧' => 'Fes Wé Sọ́mtin De Pén Wẹ́l Wẹl',
    '😨' => 'Fes Wé Fia De Kach',
    '😰' => 'Fes Wé De Swẹt An Wé Sọ́mtin De Wọ́ri',
    '😥' => 'Nọ́ Dé Hapí Bọt Dọ́n Dé Hapí',
    '😢' => 'Fes Wé De Krai',
    '😭' => 'Fes Wé De Kraí Láud Laud',
    '😱' => 'Fes Wé Fia Mék Am Shaút Laúd Laud',
    '😖' => 'Fes Wé Nọ́ Sabí Wétín I Go Du',
    '😣' => 'Fes Wé Nọ́ De Táya',
    '😞' => 'Fes Wé Dẹm Dọ́n Disapọint',
    '😓' => 'Fes Wé De Swẹt An Nọ́ Dé Hapí Sám Sám',
    '😩' => 'Fes Wé Dọ́n Táyá Wẹ́l Wẹl',
    '😫' => 'Fes Wé Dọ́n Táya',
    '🥱' => 'Fes Wé De Yọn',
    '😤' => 'Fes Wé Smok De Kọmọ́t Fọ Im Noz',
    '😡' => 'Fes Wé De Vẹ́ks Bád Bad',
    '😠' => 'Fes Wé De Vẹ́ks',
    '🤬' => 'Fes Wit Sain Fọ Maut',
    '😈' => 'Smaílín Fes Wit Tú Họn',
    '👿' => 'Fes Wé De Vẹks An Gẹt Tú Họn',
    '💀' => 'Bon Bón Pát Ọf Hẹd',
    '☠' => 'Bon Bón Pát Ọf Hẹd Wit Tuú Krọ́s Bon-dẹm',
    '💩' => 'Bíg Shit',
    '🤡' => 'Klaun Im Fes',
    '👹' => 'Ọ́glí Wíkẹ́d Fes',
    '👺' => 'Wíkẹ́d Bád Spírit',
    '👻' => 'Gost',
    '👽' => 'Kríchọ Frọm Anọ́dá Wọld',
    '👾' => 'Wíkẹ́d Kríchọ Frọm Anọ́dá Wọld',
    '🤖' => 'Mashin Wé Rizẹ́mbúl Húmánbin',
    '😺' => 'Kat Wé De Laf',
    '😸' => 'Láfín Kat Wit Smaílín Ai',
    '😹' => 'Láfín Kat Wit Tias Fọ Im Ai',
    '😻' => 'Láfín Kat Wit Hát Fọ Im Ai',
    '😼' => 'Kat Wit Krúkẹ́d Smail Fọ Fes',
    '😽' => 'Kísín Kat',
    '🙀' => 'Kat Wé Dọ́n Táya',
    '😿' => 'Kat Wé De Krai',
    '😾' => 'Kat Wé Nọ́ Dé Hapí',
    '🙈' => 'Mọnkí Wé Nọ́ Wán Sí Bád Tin',
    '🙉' => 'Mọnkí Wé Nọ́ Wán Hiá Bád Tin',
    '🙊' => 'Mọnkí Wé Nọ́ Wán Tọ́k Bád Tin',
    '💌' => 'Lọ́v Lẹ́ta',
    '💘' => 'Hat Wé Áro Dọ́n Chuk',
    '💝' => 'Hat Wé Dẹm Taí Ríbọn',
    '💖' => 'Hat Wé De Shaín Brait Wẹ́l Wẹl',
    '💗' => 'Hat Wé De Gro Bíkọs I Dé Hapí',
    '💓' => 'Hat Wé De Bit',
    '💞' => 'Hat-dẹm Wé De Tọ́n Raúnd An Raund',
    '💕' => 'Tú Hat',
    '💟' => 'Hat Wé Dẹm Mék Am Mék I Lúk Fain',
    '❣' => 'Hat Wé Dẹm Ték Mék Ẹsklaméshọ́n Mak',
    '💔' => 'Hat Wé Dọ́n Brẹk',
    '❤' => 'Rẹ́d Hat',
    '🩷' => 'Pínk Kọ́lọ Hat',
    '🧡' => 'Ọ́rénj Kọ́lọ Hat',
    '💛' => 'Yẹ́ló Hat',
    '💚' => 'Grín Hat',
    '💙' => 'Blú Hat',
    '🩵' => 'Hat Wé Dé Laít Blu',
    '💜' => 'Pọ́pul Kọ́lọ Hat',
    '🤎' => 'Braún Hat',
    '🖤' => 'Blák Hat',
    '🩶' => 'Gré Kọ́lọ Hat',
    '🤍' => 'Waít Hat',
    '💋' => 'Kis Mak',
    '💯' => 'Skọ Wé Bi Wọ́n Họ́ndrẹ́d Pọínt',
    '💢' => 'Vẹ́ks Sain',
    '🫯' => 'fait klaọ́d',
    '💥' => 'Jam',
    '💫' => 'Ai De Tọn Tọ́n Yu An Yu Wán Fọl',
    '💦' => 'Sọm Drọ́p Ọf Swẹt',
    '💨' => 'To Ték Spid Rọn',
    '🕳' => 'Hol',
    '💬' => 'Mẹ́séj Blonbló Píkchọ',
    '🗨' => 'Lẹ́ft Hánd Saíd Mẹ́séj Blonbló Píkchọ',
    '🗯' => 'Raít Hánd Saíd Vẹks Blonbló Píkchọ',
    '💭' => 'Blonbló Píkchọ Abaut Tínkin',
    '💤' => 'Zzz',
    '👋' => 'Hand Wé De Wev',
    '🤚' => 'Hand Im Bak Wé Dẹm Réz Ọp',
    '🖐' => 'Hand Wé Dẹm Sprẹ́d Di Fínga-dẹm',
    '✋' => 'Hand Wé Dẹm Réz Ọp',
    '🖖' => 'Vọ́lkán Salut',
    '🫱' => 'Hand Wé De Pọínt Raít Said',
    '🫲' => 'Hand Wé De Pọínt Lẹ́ft Said',
    '🫳' => 'Hand Wé De Fés Daun',
    '🫴' => 'Hand Wé De Pọínt Ọp',
    '🫷' => 'Hand Wé De Púsh Gó Lẹ́ft Said',
    '🫸' => 'Hand Wé De Púsh Gó Raít Said',
    '👌' => 'OK Hand',
    '🤌' => 'To Pínch Fínga-dẹm Togẹ́da',
    '🤏' => 'Hand Wé De Pinch',
    '✌' => 'Víktri Hand',
    '🤞' => 'Krọ́s Fínga-dẹm',
    '🫰' => 'Hand Wé Krọ́s Im Sẹ́kọ́n Fínga An Tọmb',
    '🤟' => 'A Lọ́v Yu Sain',
    '🤘' => 'Họ́n Sain',
    '🤙' => 'Mék Yu Fón Mí Hand',
    '👈' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Lẹft',
    '👉' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Rait',
    '👆' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Ọp',
    '🖕' => 'Mídúl Fínga',
    '👇' => 'Bakhánd Niá-Tọ́mb-Fínga De Pọínt Daun',
    '☝' => 'Niá-Tọ́mb-Fínga De Pọínt Ọp',
    '🫵' => 'Sẹ́kọ́n Fínga De Pọínt Di Pẹ́sin Wé De Luk',
    '👍' => 'Tọ́mb Ọp',
    '👎' => 'Tọ́mb Daun',
    '✊' => 'Réz Klózd Hand Ọp',
    '👊' => 'Klózd Hand Wán Gív Pẹ́sin Blo',
    '🤛' => 'Klózd Hand Wé De Fés Lẹft',
    '🤜' => 'Klózd Hand Wé De Fés Rait',
    '👏' => 'Hand-dẹm Wé De Klap',
    '🙌' => 'Hand-dẹm Wé Dẹm Réz Ọp',
    '🫶' => 'Hát hand-dẹm',
    '👐' => 'Ópún Hand-dẹm',
    '🤲' => 'Ópún Hand-dẹm Wé Dé Togẹ́da',
    '🤝' => 'Hándshek',
    '🙏' => 'Hand-dẹm Wé Dẹm Fold',
    '✍' => 'Hand Wé De Raít Sọ́mtin',
    '💅' => 'Nél Pọ́lish',
    '🤳' => 'Sẹ́lfi',
    '💪' => 'Hánd Mọ́zul Wé Bẹnd',
    '🦾' => 'Atífíshál Hand',
    '🦿' => 'Atífíshál Lẹg',
    '🦵' => 'Lẹg',
    '🦶' => 'Daunlẹg',
    '👂' => 'Ia',
    '🦻' => 'Ia Wit Sọ́mtin Wé De Hẹ́lẹ́p Pẹ́sin Mék I Hia',
    '👃' => 'Noz',
    '🧠' => 'Bren',
    '🫀' => 'As Pẹ́sin Hat Bì',
    '🫁' => 'Lọngs',
    '🦷' => 'Tit',
    '🦴' => 'Bon',
    '👀' => 'Ai-dẹm',
    '👁' => 'Ai',
    '👅' => 'Tọng',
    '👄' => 'Maut',
    '🫦' => 'Pẹ́sin Wé De Baít Im Lip',
    '👶' => 'Taata',
    '🧒' => 'Smọ́l Pikin',
    '👦' => 'Smọ́l Bọi',
    '👧' => 'Gẹl',
    '🧑' => 'Pẹ́sin',
    '👱' => 'Pẹ́sin Wé Gẹ́t Blọ́nd Hea',
    '👨' => 'Man',
    '🧔' => 'Pẹ́sin Wé Gẹ́t Biabiá',
    '👩' => 'Wúman',
    '🧓' => 'Síniọ́ Pẹ́sin',
    '👴' => 'Óld Man',
    '👵' => 'Óld Wúman',
    '🙍' => 'Pẹ́sin Wé Skwíz Fes',
    '🙎' => 'Pẹ́sin Wé De Mék Lọ́ng Fes',
    '🙅' => 'Pẹ́sin Wé Ték Hand De Shó Sé Na NO I Min',
    '🙆' => 'Pẹ́sin Tẹk Hand De Shó Sé Na OK I Min',
    '💁' => 'Pẹ́sin Wé De Shó Im Hand',
    '🙋' => 'Pẹ́sin Wé De Réz Im Hand Ọp',
    '🧏' => 'Pẹ́sin Wé Nọ́ Fít Hia',
    '🙇' => 'Pẹ́sin Wé De Baú Daun',
    '🤦' => 'Pẹ́sin Wé Ték Wọ́n Hand Kọ́vá Im Fes',
    '🤷' => 'Pẹ́sin Wé De Shrọ́g Im Shólda',
    '👮' => 'Polís Ọ́físa',
    '🕵' => 'Ditẹ́ktiv',
    '💂' => 'Gad',
    '🥷' => 'Nínja',
    '👷' => 'Kọnstrọ́kshọ́n Wọ́ka',
    '🫅' => 'Pẹ́sin Wé Wiá Kraun',
    '🤴' => 'Prins',
    '👸' => 'Prínsẹs',
    '👳' => 'Pẹ́sin Wé Taí Hẹd-Tai Fọ Hẹd',
    '👲' => 'Pẹ́sin Wé Wiá Skọ́lkap',
    '🧕' => 'Wúman Wé Taí Hẹdtai',
    '🤵' => 'Pẹ́sin Wé Wiá Tọksído',
    '👰' => 'Pẹ́sin Wé Wiá Vẹl',
    '🤰' => 'Wúman Wé Gẹ́t Bẹlẹ́',
    '🫃' => 'Man Wé Gẹ́t Bẹlẹ́',
    '🫄' => 'Pẹ́sin Wé Gẹ́t Bẹlẹ́',
    '🤱' => 'Wúman Wé De Gív Pikin Brẹst',
    '👼' => 'Bẹ́bí Énjẹl',
    '🎅' => 'Sántá Klọz',
    '🤶' => 'Misis Klọz',
    '🦸' => 'Supahíro',
    '🦹' => 'Wíkẹ́d Krímínal',
    '🧙' => 'Majíshan',
    '🧚' => 'Spírit Wé Rizẹ́mbúl Taíní Húmánbin',
    '🧛' => 'Vámpáya',
    '🧜' => 'Pẹ́sin Wé Bi Mamíwọtá',
    '🧝' => 'Ẹlf',
    '🧞' => 'Jiínii',
    '🧟' => 'Zombí',
    '🧌' => 'Sọ́mtin Wé Rizẹ́mbúl Ọ́glí Húmánbin',
    '🫈' => 'híárí kríẹchọ',
    '💆' => 'Pẹ́sin Wé Dẹm De Masáj Im Bọdí',
    '💇' => 'Pẹ́sin Wé Dẹm De Báb Im Hea',
    '🚶' => 'Pẹ́sin Wé De Waká',
    '🧍' => 'Pẹ́sín Wé De Stánd Ọp',
    '🧎' => 'Pẹ́sin Wé Níl Daun',
    '🏃' => 'Pẹ́sin Wé De Rọn',
    '💃' => 'Wúman Wé De Dans',
    '🕺' => 'Man Wé De Dans',
    '🕴' => 'Pẹ́sin Wé Wiá Sút-An-Sut An De Flót Fọ Ea',
    '👯' => 'Pípul Wé Wiá Tins Wé Rizẹ́mbúl Rábit Ia',
    '🧖' => 'Pẹ́sin Wé Dé Ínsaíd Họ́t-Wọ́ta-Smók Rum',
    '🧗' => 'Pẹ́sin Wé De Klaimb',
    '🤺' => 'Pẹ́sin Wé Hól Sọd De Ple',
    '🏇' => 'Pẹ́sin Wé De Rọ́n Wit Họs',
    '⛷' => 'Pẹ́sin Wé Yúz Ski De Rọ́n Ọn Tọ́p Sno',
    '🏂' => 'Pẹ́sin Wé De Raíd Wúdín-Bọd Ọn Tọ́p Sno',
    '🏌' => 'Pẹ́sin Wé De Plé Gọlf',
    '🏄' => 'Pẹ́sin Wé De Raíd Wọtá Sọ́fín-Bọd',
    '🚣' => 'Pẹ́sin Wé Yúz Pádul De Ró Bot',
    '🏊' => 'Pẹ́sin Wé De Swim',
    '⛹' => 'Pẹ́sin Wé De Baúns Bọl Fọ Graun',
    '🏋' => 'Pẹ́sin Wé De Kárí Ọ́p Hẹ́ví Tins',
    '🚴' => 'Pẹ́sin Wé De Dú Raídín Baísíkul Spọt',
    '🚵' => 'Pẹ́sin Wé De Raíd Baísíkul Fọ Maúntin',
    '🤸' => 'Pẹ́sin Wé De Dú Kátwil',
    '🤼' => 'Pípul Wé De Rẹ́sul',
    '🤽' => 'Pẹ́sin Wé De Plé Wọ́tá Poló',
    '🤾' => 'Pẹ́sin Wé De Plé Hándbọl',
    '🤹' => 'Pẹ́sin Wé De Jọ́gúl Bọl-dẹm',
    '🧘' => 'Pẹ́sin Wé Sidọ́n Fọ Graun Fọ Lótús Wè',
    '🛀' => 'Pẹ́sin Wé De Baf',
    '🛌' => 'Pẹ́sin Wé De Slip',
    '👭' => 'Wímẹn Wé Hól Hand',
    '👫' => 'Wúman An Man Wé Hól Hand',
    '👬' => 'Man-dẹm Wé Hól Han',
    '💏' => 'Kis',
    '💑' => 'Tuú Pípul Wit Hat',
    '🗣' => 'Hẹd Wé De Tọk',
    '👤' => 'Dák Shép Ọf Ọ́p Pát Ọf Pẹ́sin Bọdí',
    '👥' => 'Dák Shép Ọf Ọ́p Pát Ọf Tú Pípul Bọdí',
    '🫂' => 'Pípul Wé Hól Ích Ọ́da Tait',
    '👪' => 'Fámíli',
    '👣' => 'Fútmak-dẹm',
    '🫆' => 'fíngaprint',
    '🦰' => 'Rẹ́d Hea',
    '🦱' => 'Kọ́lí Hea',
    '🦳' => 'Waít Hea',
    '🦲' => 'Bọld',
    '🐵' => 'Monkí Fes',
    '🐒' => 'Mọnkí',
    '🦍' => 'Goríla',
    '🦧' => 'Ọrángutang',
    '🐶' => 'Dọg Fes',
    '🐕' => 'Dọg',
    '🦮' => 'Gaíd Dọg',
    '🐩' => 'Púdul',
    '🐺' => 'Wulf',
    '🦊' => 'Fọks',
    '🦝' => 'Rakun',
    '🐱' => 'Kat Fes',
    '🐈' => 'Kat',
    '🦁' => 'Láyọn',
    '🐯' => 'Taíga Fes',
    '🐅' => 'Taíga',
    '🐆' => 'Liópad',
    '🐴' => 'Họ́s Fes',
    '🫎' => 'muus',
    '🫏' => 'Jakí',
    '🐎' => 'Họs',
    '🦄' => 'Yúnikọn',
    '🦓' => 'Zẹbra',
    '🦌' => 'Dia',
    '🦬' => 'Baísọn',
    '🐮' => 'Kau Fes',
    '🐂' => 'Mán-Kau',
    '🐃' => 'Wọ́ta Búfálo',
    '🐄' => 'Kau',
    '🐷' => 'Pig Fes',
    '🐖' => 'Pig',
    '🐗' => 'Búsh Pig',
    '🐽' => 'Píg Noz',
    '🐏' => 'Ram',
    '🐑' => 'Wuman Ship',
    '🐐' => 'Got',
    '🐪' => 'Kámẹl',
    '🐫' => 'Kámẹl Wé Gẹ́t Tuú Họ́nchbak',
    '🦙' => 'Láma',
    '🦒' => 'Jiraf',
    '🐘' => 'Ẹ́lẹ́fant',
    '🦣' => 'Mámọt',
    '🦏' => 'Raínósẹ́rọs',
    '🦛' => 'Hípopotámọs',
    '🐭' => 'Maus Fes',
    '🐁' => 'Maus',
    '🐀' => 'Rat',
    '🐹' => 'Bíg Rat Wé Im Nem Na Hámsta',
    '🐰' => 'Rábit Fes',
    '🐇' => 'Rábit',
    '🐿' => 'Wọ́n Skwírẹl Wé Im Nem Na Chípmọnk',
    '🦫' => 'Wọ́n Anọmọl Wé Im Nem Na Bíva',
    '🦔' => 'Hẹ́jhọg',
    '🦇' => 'Bat',
    '🐻' => 'Bẹ́a',
    '🐨' => 'Kọála',
    '🐼' => 'Pánda',
    '🦥' => 'Lézí Animọl Wé Im Nem Na Slot',
    '🦦' => 'Wọ́n Anọmọl Wé Im Nem Na Ọ́ta',
    '🦨' => 'Skọnk',
    '🦘' => 'Kángaruu',
    '🦡' => 'Wọ́n Anímọl Wé Im Nem Na Bája',
    '🐾' => 'Mák Ọf Lẹ́g Ọf Ánímọl-dẹm Wé Gẹ́t Nel',
    '🦃' => 'Tọkí',
    '🐔' => 'Chíkin',
    '🐓' => 'Machọ́ Mán Chíkin',
    '🐣' => 'Chíkin Pikin Wé De Kọmọ́t Frọm Ínsaíd Ẹg',
    '🐤' => 'Chíkin Pikin',
    '🐥' => 'Chíkin Pikin Wé De Fés Frọnt',
    '🐦' => 'Bẹd',
    '🐧' => 'Pẹ́ngwin',
    '🕊' => 'Dov',
    '🦅' => 'Ígul',
    '🦆' => 'Dọk',
    '🦢' => 'Bẹd Wé Im Nem Na Swan',
    '🦉' => 'Aul',
    '🦤' => 'Dódo',
    '🪶' => 'Fẹ́da',
    '🦩' => 'Flamíngo',
    '🦚' => 'Píkọk',
    '🦜' => 'Párọt',
    '🪽' => 'wíng',
    '🪿' => 'guus',
    '🐸' => 'Frọg',
    '🐊' => 'Krókodail',
    '🐢' => 'Tọ́tul',
    '🦎' => 'Lízad',
    '🐍' => 'Snek',
    '🐲' => 'Drágọn Fes',
    '🐉' => 'Bíg Wíkẹ́d Anímọl Wé Im Nem Na Drágọn',
    '🦕' => 'Sórópọd',
    '🦖' => 'T-Rẹks',
    '🐳' => 'Weil Wé De Bló Wọtá',
    '🐋' => 'Weil: Dis Na Wọ́n Bíg Ánímal Wé De Lív Fọ Wọtá',
    '🐬' => 'Dọ́lfin',
    '🫍' => 'ọ́ká',
    '🦭' => 'Wọtá Ánímal Wé Im Nem Na Siil',
    '🐟' => 'Fish',
    '🐠' => 'Họ́t Ríjọ́n Fish',
    '🐡' => 'Blófish',
    '🦈' => 'Shak',
    '🐙' => 'Ọ́ktọ́pọs: Dis Na Wọ́n Ánímal Wé De Lív Ínsaíd Bíg-díp-wọtá',
    '🐚' => 'Snel Shẹl',
    '🪸' => 'Wọ́n Tin Wé De Fọ́m Fọ Bọtọ́m Ọf Díp Wọtá An Wé Im Nem Na "Kọ́ral"',
    '🪼' => 'Jẹ́lifish',
    '🦀' => 'Krab',
    '🦞' => 'Lọ́bsta',
    '🦐' => 'Shrimp',
    '🦑' => 'Skwid',
    '🦪' => 'Ọísta',
    '🐌' => 'Snel',
    '🦋' => 'Bọtaflai',
    '🐛' => 'Smọ́l Ínsẹkt',
    '🐜' => 'Ant',
    '🐝' => 'Họ́nibii',
    '🪲' => 'Bítul',
    '🐞' => 'Ledi Bítul',
    '🦗' => 'Kríkẹt',
    '🪳' => 'Kókroch',
    '🕷' => 'Spaída',
    '🕸' => 'Spaídá Wẹb',
    '🦂' => 'Skọ́piọn',
    '🦟' => 'Mọskíto',
    '🪰' => 'Flai',
    '🪱' => 'Wọm',
    '🦠' => 'Maíkrob',
    '💐' => 'Bọ́nch Ọf Fláwa',
    '🌸' => 'Chẹ́rí Fláwa',
    '💮' => 'Waít Fláwa',
    '🪷' => 'Lótús Fláwa',
    '🏵' => 'Rozẹt',
    '🌹' => 'Roz',
    '🥀' => 'Fláwa Wé Dọ́n De Dai',
    '🌺' => 'Haibískọs',
    '🌻' => 'Sọ́nfláwa',
    '🌼' => 'Faín Fláwa',
    '🌷' => 'Túlip',
    '🪻' => 'hayasint',
    '🌱' => 'Yọ́ng Plant',
    '🪴' => 'Plant Ínsaíd Pọt',
    '🌲' => 'Ẹvagrín Trii',
    '🌳' => 'Tri Wé Gẹ́t Plẹ́ntí Líf',
    '🌴' => 'Pám Trii',
    '🌵' => 'Káktọ́s Plant',
    '🌾' => 'Rais Bọ́ndul',
    '🌿' => 'Hab Plant Wé De Mék Fud Swit',
    '☘' => 'Shámrọ́k Plant',
    '🍀' => 'Fọ́ Líf Klóva',
    '🍁' => 'Mápúl Trii Lif',
    '🍂' => 'Lif Wé Fọ́l Fọ Graund',
    '🍃' => 'Lif Wé Wind De Blo',
    '🪹' => 'Ẹ́mtí Nẹst',
    '🪺' => 'Nẹst Wit Ẹg-dẹm',
    '🍄' => 'Mọ́shrum',
    '🪾' => 'trii wey no get lif',
    '🍇' => 'Grep-dẹm',
    '🍈' => 'Mẹ́lọn',
    '🍉' => 'Wọ́tamẹ́lọn',
    '🍊' => 'Tanjẹrin',
    '🍋' => 'Lẹ́mọn',
    '🍌' => 'Banána',
    '🍍' => 'Painápul',
    '🥭' => 'Mángo',
    '🍎' => 'Rẹ́d Ápul',
    '🍏' => 'Grín Ápul',
    '🍐' => 'Pia',
    '🍑' => 'Pich',
    '🍒' => 'Chẹ́ri-dẹm',
    '🍓' => 'Strọ́bẹ́ri',
    '🫐' => 'Blúbẹ́ri-dẹm',
    '🥝' => 'Kíwí Frut',
    '🍅' => 'Tomáto',
    '🫒' => 'Óliv',
    '🥥' => 'Kokonọt',
    '🥑' => 'Ávokádo Pia',
    '🍆' => 'Ẹ́gplant',
    '🥔' => 'Potéto',
    '🥕' => 'Kárọt',
    '🌽' => 'Kọn',
    '🌶' => 'Họ́t Pẹ́pẹ',
    '🫑' => 'Bẹ́l Pẹ́pe',
    '🥒' => 'Kọkúmba',
    '🥬' => 'Grín Lif',
    '🥦' => 'Brọ́kọ́li',
    '🧄' => 'Gálik',
    '🧅' => 'Ọ́niọn',
    '🥜' => 'Granọt',
    '🫘' => 'Bins',
    '🌰' => 'Chẹ́stnọt',
    '🫚' => 'Jínja Rut',
    '🫛' => 'pii pod',
    '🫜' => 'rut vẹ́jítébul',
    '🍞' => 'Brẹd',
    '🥐' => 'Kroásant',
    '🥖' => 'Bagẹt Brẹd',
    '🫓' => 'Flátbrẹd',
    '🥨' => 'Prẹ́tzẹl',
    '🥯' => 'Bégul',
    '🥞' => 'Pankek',
    '🧇' => 'Wáful',
    '🧀' => 'Chíz Wẹj',
    '🍖' => 'Mit Ọn Tọ́p Bon',
    '🍗' => 'Póltrí Lẹg',
    '🥩' => 'Pís Ọf Mit',
    '🥓' => 'Békọn',
    '🍔' => 'Hámbọ́ga',
    '🍟' => 'Potétó Chips',
    '🍕' => 'Píza',
    '🌭' => 'Họ́t Dọg',
    '🥪' => 'Sándwich',
    '🌮' => 'Táko',
    '🌯' => 'Buríto',
    '🫔' => 'Tamálẹ',
    '🥙' => 'Flátbrẹd wit ọ́dá fud ínsaid',
    '🧆' => 'Faláfẹl',
    '🥚' => 'Ẹg',
    '🍳' => 'Kúk',
    '🥘' => 'Pọt Wé Nọ́ Dip',
    '🍲' => 'Pọ́t Ọf Fud',
    '🫕' => 'Fọ́ndu',
    '🥣' => 'Díp-plet Wit Spun',
    '🥗' => 'Grín Salad',
    '🍿' => 'Pọ́pkọn',
    '🧈' => 'Bọ́ta',
    '🧂' => 'Sọlt',
    '🥫' => 'Kán Fud',
    '🍱' => 'Bẹ́nto Bọks',
    '🍘' => 'Raís Krákas',
    '🍙' => 'Raís Bọl',
    '🍚' => 'Rais Wé Dẹm Dọ́n Kuk',
    '🍛' => 'Kọrí Rais',
    '🍜' => 'Họ́t Ból Ọf Fúd',
    '🍝' => 'Spagẹ́ti',
    '🍠' => 'Swít Potéto Wé Dẹm Rost',
    '🍢' => 'Ódẹn',
    '🍣' => 'Súshi',
    '🍤' => 'Fraíd Shrimp',
    '🍥' => 'Físh Kek Wit Sẹ́kul',
    '🥮' => 'Mún Kek',
    '🍡' => 'Dángó',
    '🥟' => 'Dọ́mplin',
    '🥠' => 'Fọtiún Kúki',
    '🥡' => 'Tékaút Bọks',
    '🍦' => 'Sọ́ft Aís Krim',
    '🍧' => 'Ais Wé Dẹm Dọ́n Shev',
    '🍨' => 'Aís Krim',
    '🍩' => 'Dónọt',
    '🍪' => 'Kúki',
    '🎂' => 'Bátdé Kek',
    '🍰' => 'Shọ́tkek',
    '🧁' => 'Kọ́pkek',
    '🥧' => 'Pai',
    '🍫' => 'Ckọ́kọ́lét Stik',
    '🍬' => 'Kándi',
    '🍭' => 'Lọ́lipọp',
    '🍮' => 'Kọ́stád',
    '🍯' => 'Họ́ni Pọt',
    '🍼' => 'Fidín Bọ́tul',
    '🥛' => 'Glás Ọf Mílik',
    '☕' => 'Drink Wé Dé Họt',
    '🫖' => 'Típọt',
    '🍵' => 'Tíkọp Wé Nọ́ Gẹ́t Hand',
    '🍶' => 'Sáki',
    '🍾' => 'Bọtul Wé Gẹ́t Kọk Wé De Mék Pọ́pín Nọiz',
    '🍷' => 'Waín Glas',
    '🍸' => 'Kọ́ktél Glas',
    '🍹' => 'Họ́t Ríjọn Drink',
    '🍺' => 'Biá Kọp',
    '🍻' => 'Klínkín Biá Kọp-dẹm',
    '🥂' => 'Klínkín Glas-dẹm',
    '🥃' => 'Tọ́mbla',
    '🫗' => 'To Pọọ́ Líkwid',
    '🥤' => 'Kọp Wit Strọ',
    '🧋' => 'Bọ́búl Tii',
    '🧃' => 'Drínk Bọks',
    '🧉' => 'Matẹẹ',
    '🧊' => 'Ais',
    '🥢' => 'Chọpstik-dẹm',
    '🍽' => 'Fọk An Naif Wit Plet',
    '🍴' => 'Fọ́k An Naif',
    '🥄' => 'Spun',
    '🔪' => 'Kíchín Naif',
    '🫙' => 'Bọ́tul Wé Im Nem Na "Jaa"',
    '🏺' => 'Ámfora',
    '🌍' => 'Raúnd Wọ́ld Map Wẹ De Shó Yúrop-Áfrika',
    '🌎' => 'Raúnd Wọ́ld Map Wé De Shó Amẹ́ríkas',
    '🌏' => 'Raúnd Wọ́ld Map Wé De Shó Éshia-Ọstrélia',
    '🌐' => 'Raúnd Wọ́ld Map Wit Mẹrídían Lain-dẹm',
    '🗺' => 'Wọ́ld Map',
    '🗾' => 'Japán Map',
    '🧭' => 'Kọ́mpas',
    '🏔' => 'Maúntin Wé Sno Kọ́vá Di Tọp',
    '⛰' => 'Maúntin',
    '🛘' => 'lánd sláid',
    '🌋' => 'Volkáno',
    '🗻' => 'Fújí Maúntin',
    '🏕' => 'Kamp',
    '🏖' => 'Bich Wit Ọ́mbrẹ́la',
    '🏜' => 'Dẹ́zat',
    '🏝' => 'Dẹ́zát Aíland',
    '🏞' => 'Náshọ́nál Pak',
    '🏟' => 'Stádiọm',
    '🏛' => 'Óldín Taím Bíldin',
    '🏗' => 'Bíldín Kọnstrọ́kshọn',
    '🧱' => 'Brík',
    '🪨' => 'Rọk',
    '🪵' => 'Wud',
    '🛖' => 'Họt',
    '🏘' => 'Haus-dẹm',
    '🏚' => 'Haus Wé Dé Fọ Bád Kọndíshọn',
    '🏠' => 'Haus',
    '🏡' => 'Haus Wit Gádin',
    '🏢' => 'Ọ́fís Bíldin',
    '🏣' => 'Japán Póst Ọ́fis',
    '🏤' => 'Póst Ọ́fis',
    '🏥' => 'Họ́spítal',
    '🏦' => 'Bank',
    '🏨' => 'Họtẹl',
    '🏩' => 'Lọ́v Họtẹl',
    '🏪' => 'Smọ́l Stọ Wé De Sẹ́l Dífrẹ́n Dífrẹ́n Tins',
    '🏫' => 'Skul',
    '🏬' => 'Bíg Stọ Wé De Sẹ́l Plẹ́ntí Dífrẹ́n Dífrẹ́n Tins',
    '🏭' => 'Fáktri',
    '🏯' => 'Japán Bíg Strọ́ng Bíldin',
    '🏰' => 'Bíg Strọ́ng Bíldin',
    '💒' => 'Márej',
    '🗼' => 'Tókyo Tọ́wa',
    '🗽' => 'Státiú Ọf Líbáti',
    '⛪' => 'Chọch',
    '🕌' => 'Mọsk',
    '🛕' => 'Híndú Tẹ́mpul',
    '🕍' => 'Juu Pípul Dẹm Preya Haus',
    '⛩' => 'Shínto Jujú Haus',
    '🕋' => 'Kaaba',
    '⛲' => 'Sọ́mtin Wé De Pọ́mp Wọtá Gó Ọp',
    '⛺' => 'Tẹnt',
    '🌁' => 'Tík Klaud Wé Nọ́ De Grí Pẹ́sin Sí Rod Wẹl',
    '🌃' => 'Nait Wit Sta-dẹm',
    '🏙' => 'Taun Píkchọ́',
    '🌄' => 'Sọn As I De Kọmọ́t Ọn Tọ́p Maúntin',
    '🌅' => 'Di Taim Wé Sọn De Kọmọt',
    '🌆' => 'Taun Im Píkchọ Wẹ́n Nait Dọ́n De Niálí Rich',
    '🌇' => 'Sọn Dọ́n De Gò',
    '🌉' => 'Haú Brij Bì Wẹn Yú Lúk Am Fọ Naít Taim',
    '♨' => 'Họt Wọtá Wé De Kọmọ́t Frọm Ínsaíd Graun',
    '🎠' => 'Hápí-Gó-Raúnd Họs',
    '🛝' => 'Slaid Ple Plé Graund',
    '🎡' => 'Fẹ́rís Wiil',
    '🎢' => 'Rólá Kósta',
    '💈' => 'Bába Pol',
    '🎪' => 'Sẹkọ́s Tẹnt',
    '🚂' => 'Lokomótív Tren',
    '🚃' => 'Relwé Móto',
    '🚄' => 'Haí-spíd Tren',
    '🚅' => 'Búlẹ́t Tren',
    '🚆' => 'Tren',
    '🚇' => 'Mẹ́tro',
    '🚈' => 'Laít Rel',
    '🚉' => 'Stéshọn',
    '🚊' => 'Tram',
    '🚝' => 'Mónórel',
    '🚞' => 'Maúntín Relwè',
    '🚋' => 'Trám Kaa',
    '🚌' => 'Bọs',
    '🚍' => 'Bọs Wé De Kọ́m Wiá Yu Dè',
    '🚎' => 'Trọlíbọs',
    '🚐' => 'Míníbọs',
    '🚑' => 'Ámbúlans',
    '🚒' => 'Fáyá Ẹ́njin',
    '🚓' => 'Polís Móto',
    '🚔' => 'Polís Móto Wé De Kọ́m Fọ Yọ Front',
    '🚕' => 'Taksí',
    '🚖' => 'Taksí Wé De Kọ́m Wiá Yu Dè',
    '🚗' => 'Móto',
    '🚘' => 'Móto Wé De Kọ́m Fọ Yọ Frọnt',
    '🚙' => 'Spọ́t Yutílítí Móto',
    '🛻' => 'Píkọ́p',
    '🚚' => 'Lọ́ri Wé De Kárí Tins To Kọ́stọ́ma-dẹm',
    '🚛' => 'Tréla',
    '🚜' => 'Tráktọ',
    '🏎' => 'Móto Wé Bi Fọ Rọ́nín Kọmpitíshọn',
    '🏍' => 'Motosaíkul',
    '🛵' => 'Skúta Motosaíkul',
    '🦽' => 'Wílchia Wé Nọ́ Gẹ́t Ẹ́njin',
    '🦼' => 'Wílchia Wé Gẹ́t Ẹ́njin',
    '🛺' => 'Kẹkẹ́ Napẹp',
    '🚲' => 'Baísíkul',
    '🛴' => 'Kík Skúta',
    '🛹' => 'Skétbọd',
    '🛼' => 'Rólá Sket',
    '🚏' => 'Bọ́s Stọp',
    '🛣' => 'Ẹ́sprẹs',
    '🛤' => 'Relwé Lain',
    '🛢' => 'Ọ́yil Drọm',
    '⛽' => 'Fúẹl Pọmp',
    '🛞' => 'Wiil',
    '🚨' => 'Polís Kaa Lait',
    '🚥' => 'Flát-an-Lẹ́vúl Tráfík Lait',
    '🚦' => 'Tráfík Lait Wé Dé Stret Frọm Ọ́p Rích Daun',
    '🛑' => 'Stọ́p Sain',
    '🚧' => 'Kọnstrọ́kshọn',
    '⚓' => 'Hẹ́ví Áyọn Wé De Hól Ship Fọ Wọ́n Ples',
    '🛟' => 'Ríng Buói',
    '⛵' => 'Sélbot',
    '🛶' => 'Kenú',
    '🚤' => 'Spídbot',
    '🛳' => 'Pásínjá Ship',
    '⛴' => 'Fẹ́ri',
    '🛥' => 'Moto Bot',
    '🚢' => 'Ship',
    '✈' => 'Eáplen',
    '🛩' => 'Smọ́l Eáplen',
    '🛫' => 'Eáplen As I De Lív Eápọt',
    '🛬' => 'Eáplen Wé De Araív Fọ Eápọt',
    '🪂' => 'Páráshut',
    '💺' => 'Sit',
    '🚁' => 'Hẹlikọ́pta',
    '🚟' => 'Hángín Rélwè',
    '🚠' => 'Maúntín Kébulwè',
    '🚡' => 'Eriál Trámwè',
    '🛰' => 'Sátẹ́lait',
    '🚀' => 'Rọ́kẹt',
    '🛸' => 'Fláyín Sọ́sa',
    '🛎' => 'Bẹ́lhọ́p Bẹl',
    '🧳' => 'Lọ́gej',
    '⌛' => 'Áwaglás Sand Dọ́n Fló Fínish',
    '⏳' => 'Áwaglás Sand Nẹ́vá Fló Fínish',
    '⌚' => 'Wọch',
    '⏰' => 'Alám Klọk',
    '⏱' => 'Stọ́pwọch',
    '⏲' => 'Taímá Klọk',
    '🕰' => 'Mántulpís Klọk',
    '🕛' => 'Twẹ́lf O’klọk',
    '🕧' => 'Twẹ́lf-Táti',
    '🕐' => 'Wọ́n O’klọk',
    '🕜' => 'Wọ́n-Táti',
    '🕑' => 'Tuú O’klọk',
    '🕝' => 'Tuú-Táti',
    '🕒' => 'Tírí O’klọk',
    '🕞' => 'Tírí-Táti',
    '🕓' => 'Fọọ́ O’klọk',
    '🕟' => 'Fọọ́-Táti',
    '🕔' => 'Faíf O’klọk',
    '🕠' => 'Faíf-Táti',
    '🕕' => 'Síks O’klọk',
    '🕡' => 'Síks-Táti',
    '🕖' => 'Sẹ́vún O’klọk',
    '🕢' => 'Sẹ́vún-Táti',
    '🕗' => 'Ẹ́t O’klọk',
    '🕣' => 'Ẹ́t-Táti',
    '🕘' => 'Naín O’klọk',
    '🕤' => 'Naín-Táti',
    '🕙' => 'Tẹ́n O’klọk',
    '🕥' => 'Tẹ́n-Táti',
    '🕚' => 'Ilẹ́vún O’klọk',
    '🕦' => 'Ilẹ́vún-Táti',
    '🌑' => 'Níu Mun',
    '🌒' => 'Mun Im Braít Pat Jọ́s Stát To Bíg Mọọ́ Mọọ',
    '🌓' => 'Mún Im Fẹ́st Kwọ́ta',
    '🌔' => 'Mun Braít Pat Bíg Pás Háf Di Mun',
    '🌕' => 'Fúl Mun',
    '🌖' => 'Mun Dọ́n Stát To Go',
    '🌗' => 'Mun Im Lást Kwọ́ta',
    '🌘' => 'Mun Dọ́n De Niálí Gó Fínish',
    '🌙' => 'Krísẹ́nt Mun',
    '🌚' => 'Niú Mún Fes',
    '🌛' => 'Fẹ́st Kwọ́ta Mún Fes',
    '🌜' => 'Lást Kwọ́ta Mún Fes',
    '🌡' => 'Tamómíta',
    '☀' => 'Sọn',
    '🌝' => 'Fúl Mun Fes',
    '🌞' => 'Sọn Wit Fes',
    '🪐' => 'Plánẹt Wé Ring-dẹm Sọraund',
    '⭐' => 'Sta',
    '🌟' => 'Shaínín Sta',
    '🌠' => 'Shútín Sta',
    '🌌' => 'Mílkí Wè',
    '☁' => 'Klaud',
    '⛅' => 'Sọn Bihaínd Klaud',
    '⛈' => 'Klaud Wit Laítnin An Ren',
    '🌤' => 'Sọn Bihaínd Smọ́l Klaud',
    '🌥' => 'Sọn Bihaínd Bíg Klaud',
    '🌦' => 'Sọn Bihaínd Klaud Wé De Bríng Ren',
    '🌧' => 'Klaud Wit Ren',
    '🌨' => 'Klaud Wit Sno',
    '🌩' => 'Klaud Wit Laítnin',
    '🌪' => 'Tọnádo',
    '🌫' => 'Fọg',
    '🌬' => 'Wínd Fes',
    '🌀' => 'Saiklon',
    '🌈' => 'Rénbo',
    '🌂' => 'Klóz Ọ́mbrẹ́la',
    '☂' => 'Ọ́mbrẹ́la',
    '☔' => 'Ọ́mbrẹ́la Wé Rén Drọp-dẹm Dé Ọn Tọp',
    '⛱' => 'Ọ́mbrẹ́la Ọn Tọ́p Graund',
    '⚡' => 'Haí Vóltéj Ẹlẹ́trik',
    '❄' => 'Smọ́l Ais Wé Fọ́l Frọm Skai As Sno',
    '☃' => 'Snóman',
    '⛄' => 'Snóman Wé Sno Nọ́ Dé Ọn Tọ́p Am',
    '☄' => 'Kọ́mẹt',
    '🔥' => 'Fáya',
    '💧' => 'Smọ́l Drọ́p Ọf Líkwid',
    '🌊' => 'Wọtá Wev',
    '🎃' => 'Jak-O-Lántan',
    '🎄' => 'Krísmás Trii',
    '🎆' => 'Fáyáwọks',
    '🎇' => 'Spákla',
    '🧨' => 'Fáyákráka',
    '✨' => 'Spákuls',
    '🎈' => 'Balun',
    '🎉' => 'Pátí Pọ́pa',
    '🎊' => 'Kọnfẹ́ti Bọl',
    '🎋' => 'Tanábátá Tri',
    '🎍' => 'Paín Dẹkọréshọn',
    '🎎' => 'Japán Dọlbebí-dẹm',
    '🎏' => 'Kárp Físh Stríma',
    '🎐' => 'Wínd Chaim',
    '🎑' => 'Fẹ́stival Wé Japán Pípol De Ték Ọ́nọ́ Mun',
    '🧧' => 'Rẹ́d Ẹnvẹ́lop',
    '🎀' => 'Ríbọn',
    '🎁' => 'Gift Wé Dẹm Ráp Wit Sọ́mtin',
    '🎗' => 'Ríbọn To Ték Shó Sọpọt Fọ Sọ́mtin Ọ Grup',
    '🎟' => 'Tíkẹt-dẹm To Gẹ́t Rait To Ẹ́ntá Sọm Ples',
    '🎫' => 'Tíkẹt',
    '🎖' => 'Mílítrí Mẹ́dal',
    '🏆' => 'Kọp',
    '🏅' => 'Spọ́ts Mẹ́dal',
    '🥇' => '1st Plés Mẹ́dal',
    '🥈' => '2nd Plés Mẹ́dal',
    '🥉' => '3rd Plés Mẹ́dal',
    '⚽' => 'Sọ́ka Bọl',
    '⚾' => 'Bésbọl',
    '🥎' => 'Sọ́ftbọl',
    '🏀' => 'Baskẹ́tbọl',
    '🏐' => 'Vọ́libọl',
    '🏈' => 'Amẹ́ríká Fútbọl',
    '🏉' => 'Rọ́gbi Fútbọl',
    '🎾' => 'Tẹnis',
    '🥏' => 'Fláyín Disk',
    '🎳' => 'Bólin',
    '🏏' => 'Kríkẹ́t Gem',
    '🏑' => 'Fíld Họkí',
    '🏒' => 'Aís Họkí',
    '🥍' => 'Lakrọs',
    '🏓' => 'Tebúl Tẹnis',
    '🏸' => 'Bádmíntin',
    '🥊' => 'Bọ́ksín Glov',
    '🥋' => 'Máshál Áts Yúnífọm',
    '🥅' => 'Gól Nẹt',
    '⛳' => 'Flag Wé Dé Ínsaíd Hol',
    '⛸' => 'Aís Sket',
    '🎣' => 'Stik Wé Pípul De Ték Kách Fish',
    '🤿' => 'Daívín Mask',
    '🎽' => 'Rọ́nín Shẹt',
    '🎿' => 'Skii-dẹm',
    '🛷' => 'Slẹd',
    '🥌' => 'Kọ́lín Ston',
    '🎯' => 'Dairẹ́kt Hit',
    '🪀' => 'Yó-Yo',
    '🪁' => 'Kait',
    '🔫' => 'Wọ́tá Pístul',
    '🎱' => 'Pul 8 Bọl',
    '🔮' => 'Krístál Bọl',
    '🪄' => 'Májík Stik',
    '🎮' => 'Vídió Gem',
    '🕹' => 'Jọístik',
    '🎰' => 'Gámblín Máshin',
    '🎲' => 'Gem Dais',
    '🧩' => 'Pọ́zul Sid',
    '🧸' => 'Tẹ́dí Bẹ́a',
    '🪅' => 'Pinyáta',
    '🪩' => 'Mírọ Bọl',
    '🪆' => 'Dọlbebí Ínsaíd Anọ́dá Dọlbebí',
    '♠' => 'Sped Pléyín Kad',
    '♥' => 'Hat Pléyín Kad',
    '♦' => 'Dáyámọnd Pléyín Kad',
    '♣' => 'Klọb Pléyín Kad',
    '♟' => 'Chẹ́s Pọn',
    '🃏' => 'Jóka',
    '🀄' => 'Máhjọng Rẹ́d Drágọn',
    '🎴' => 'Fláwá Pléyín Kad-dẹm',
    '🎭' => 'Átists Dẹm Mask',
    '🖼' => 'Píkchọ Wé Dẹm Pút Fọ Frem',
    '🎨' => 'Plet Wé Átist De Ték Míks Pent',
    '🧵' => 'Trẹd',
    '🪡' => 'Nídul Fọ Sóyín Klọt',
    '🧶' => 'Tred Wé Dẹm De Ték Mék Klọt',
    '🪢' => 'Taí Nọt',
    '👓' => 'Aí Glásis',
    '🕶' => 'Sọ́nglásis',
    '🥽' => 'Gọ́guls',
    '🥼' => 'Láb Kot',
    '🦺' => 'Klọt Fọ Ték Protẹ́t Pẹ́sin',
    '👔' => 'Tai',
    '👕' => 'T-Shẹt',
    '👖' => 'Jins',
    '🧣' => 'Mọ́fla',
    '🧤' => 'Glov-dẹm',
    '🧥' => 'Kot',
    '🧦' => 'Sọks',
    '👗' => 'Gaun',
    '👘' => 'Japán Klọt Wé Im Nem Na Kimóno',
    '🥻' => 'Sári Klọt',
    '🩱' => 'Tuú-In-Wọ́n Swimin Klọt',
    '🩲' => 'Man Swimin Pant',
    '🩳' => 'Shọ́t Níka',
    '👙' => 'Bikíni',
    '👚' => 'Wúmán Klọt',
    '🪭' => 'Fan Wé Dẹm De Ték Hand Fold',
    '👛' => 'Pọs',
    '👜' => 'Hándbag',
    '👝' => 'Bag Wé Wúman De Hóld Taít Tait',
    '🛍' => 'Shọ́pín Bag',
    '🎒' => 'Bákpák Bag',
    '🩴' => 'Slípas',
    '👞' => 'Mán Shu',
    '👟' => 'Shu Fọ Rọ́nin',
    '🥾' => 'But Wé Pípol De Wia Fọ Waká Gó Faa',
    '🥿' => 'Flát Shu',
    '👠' => 'Haí-Híl Shu',
    '👡' => 'Wúmán Sándal',
    '🩰' => 'Shu Fọ Ték Dáns Balẹ́t Dans',
    '👢' => 'Wúmán But',
    '🪮' => 'hia pik',
    '👑' => 'Kraun',
    '👒' => 'Wúmán Hat',
    '🎩' => 'Tọ́l Hat',
    '🎓' => 'Grajuẹt Sẹ́rímọ́ni Kap',
    '🧢' => 'Fes Kap',
    '🪖' => 'Mílítrí Hẹ́lmẹt',
    '⛑' => 'Hẹ́lmẹt Wé Wọ́kas Wé De Sév Pípol De Wia',
    '📿' => 'Préyá Bid',
    '💄' => 'Lípstik',
    '💍' => 'Ring',
    '💎' => 'Kọ́slí Faín Shain-Shaín Ston',
    '🔇' => 'Laúdspíka Wé Nọ́ De Tọk',
    '🔈' => 'Laúdspíka Ló Vọ́lium',
    '🔉' => 'Laúdspíka Mídiúm Vọ́lium',
    '🔊' => 'Laúdspíka Haí Vọ́lium',
    '📢' => 'Laúdspíka',
    '📣' => 'Mẹ́gafon',
    '📯' => 'Póst Ọ́fís Họn',
    '🔔' => 'Bẹl',
    '🔕' => 'Bẹl Wé Gẹ́t Laín Mak',
    '🎼' => 'Miúzíkál Skọ',
    '🎵' => 'Miúzíkál Not',
    '🎶' => 'Miúzíkál Not-dẹm',
    '🎙' => 'Stúdió Maikrofon',
    '🎚' => 'Lẹ́vúl Slaídá Kọntrol',
    '🎛' => 'Kọntról Bọ́tin',
    '🎤' => 'Maikrofon',
    '🎧' => 'Hẹ́dfon',
    '📻' => 'Rédio',
    '🎷' => 'Saksafon',
    '🎺' => 'Trọ́mpẹt',
    '🪊' => 'trọ́mbọ́n',
    '🪗' => 'Akọ́diọn',
    '🎸' => 'Gita',
    '🎹' => 'Miúzik Kíbọd',
    '🎻' => 'Vayọlin',
    '🪕' => 'Banjó',
    '🥁' => 'Drọm',
    '🪘' => 'Lọ́ng Drọm',
    '🪇' => 'Márákas',
    '🪈' => 'fluut',
    '🪉' => 'háp',
    '📱' => 'Mọ́baíl Fon',
    '📲' => 'Mọ́baíl Fon Wit Áro',
    '☎' => 'Tẹlífon',
    '📞' => 'Tẹlífón Risíva',
    '📟' => 'Bípa',
    '📠' => 'Fáks Mashin',
    '🔋' => 'Bátri',
    '🪫' => 'Ló Bátri',
    '🔌' => 'Ẹlẹ́trík Plọg',
    '💻' => 'Láptọp',
    '🖥' => 'Dẹ́sktọ́p Kọmpúta',
    '🖨' => 'Prínta',
    '⌨' => 'Kíbọd',
    '🖱' => 'Kọmpútá Maus',
    '🖲' => 'Trákbọl',
    '💽' => 'Kọmpútá Disk',
    '💾' => 'Flọ́pí Disk',
    '💿' => 'Ọ́ptíkál Disk',
    '📀' => 'DVD',
    '🧮' => 'Ábákọs',
    '🎥' => 'Múví Kámẹ́ra',
    '🎞' => 'Fím Frem-dem',
    '📽' => 'Fím Projẹ́ktọ',
    '🎬' => 'Klápa Bọd',
    '📺' => 'Tẹlívíshọn',
    '📷' => 'Kámẹ́ra',
    '📸' => 'Kámẹ́ra Wé Gẹ́t Flash',
    '📹' => 'Vídió Kámẹ́ra',
    '📼' => 'Vídiókasẹt',
    '🔍' => 'Magnifáyín Glas Wé Bẹ́nd Lẹft',
    '🔎' => 'Magnifáyín Glas Wé Bẹ́nd Rait',
    '🕯' => 'Kándul',
    '💡' => 'Laít Bọlb',
    '🔦' => 'Tọ́chlait',
    '🏮' => 'Rẹ́d Pépá Lamp',
    '🪔' => 'Díya Ọ́yil Lamp',
    '📔' => 'Nótbuk Wé Gẹ́t Faín Kọ́va',
    '📕' => 'Klózd Buk',
    '📖' => 'Ópún Buk',
    '📗' => 'Grín Buk',
    '📘' => 'Blú Buk',
    '📙' => 'Ọ́rénj Kọ́lọ Buk',
    '📚' => 'Buk-dẹm',
    '📓' => 'Nótbuk',
    '📒' => 'Lẹ́ja',
    '📃' => 'Pej Wé Di Maut Bẹnd',
    '📜' => 'Skrol',
    '📄' => 'Pej Wé De Fés Ọp',
    '📰' => 'Niúzpépa',
    '🗞' => 'Niúzpépa Wé Dẹm Ról Togẹ́da',
    '📑' => 'Bukmák Tab-dẹm',
    '🔖' => 'Bukmák',
    '🏷' => 'Lébẹl',
    '🪙' => 'Kọin',
    '💰' => 'Mọní Bag',
    '🪎' => 'Trẹ́zhọ chẹst',
    '💴' => 'Yẹn Pépá Mọní',
    '💵' => 'Dọ́la Pépá Mọní',
    '💶' => 'Yúro Pépá Mọní',
    '💷' => 'Paund Pépá Mọní',
    '💸' => 'Mọní Wé Gẹ́t Wing',
    '💳' => 'Krẹ́dít Kad',
    '🧾' => 'Risit',
    '💹' => 'Graf Wé Shó Sé Yẹn De Gó Ọp',
    '✉' => 'Ẹ́nvẹ́lop',
    '📧' => 'I-Mel',
    '📨' => 'Ínkọ́mín Ẹ́nvẹ́lop',
    '📩' => 'Ẹ́nvẹ́lop Wé Dẹm Drọ́ Áro',
    '📤' => 'Aútbọ́ks Tre',
    '📥' => 'Ínbọ́ks Tre',
    '📦' => 'Pákej',
    '📫' => 'Klozd Mélbọks Wit Flag Wé Réz Ọp',
    '📪' => 'Klozd Mélbọks Wit Flag Wé Dé Daun',
    '📬' => 'Ópún Mélbọks Wit Flag Wé Réz Ọp',
    '📭' => 'Ópún Mélbọks Wit Flag Wé Dé Daun',
    '📮' => 'Póstbọks',
    '🗳' => 'Bálọ́t Bọks Wit Bálọt',
    '✏' => 'Pẹ́nsul',
    '✒' => 'Blák Nib',
    '🖋' => 'Fauntin Pẹn',
    '🖊' => 'Pẹn',
    '🖌' => 'Péntbrọsh',
    '🖍' => 'Kréyọn',
    '📝' => 'Mẹ́mo',
    '💼' => 'Brífkes',
    '📁' => 'Faíl Fólda',
    '📂' => 'Ópún Faíl Fólda',
    '🗂' => 'Kád Índẹ́ks Divaída-dẹm',
    '📅' => 'Kalẹ́nda',
    '📆' => 'Kalẹ́nda Wé Pẹ́sin Fít Tiá Am Kọmọt',
    '🗒' => 'Spaírál Nótpad',
    '🗓' => 'Spaírál Kalẹ́nda',
    '📇' => 'Kád Índẹks',
    '📈' => 'Chat Wit Graf Wé De Gó Ọp',
    '📉' => 'Chat Wit Graf Wé De Gó Daun',
    '📊' => 'Graf Wé Gẹ́t Waíd Lain-dẹm',
    '📋' => 'Klípbọd',
    '📌' => 'Púshpin',
    '📍' => 'Raúnd Púshpin',
    '📎' => 'Pépáklip',
    '🖇' => 'Pépáklip-dẹm wé línk togẹ́da',
    '📏' => 'Strét Rúla',
    '📐' => 'Tráyángúl Rúla',
    '✂' => 'Sízọs',
    '🗃' => 'Bọks Fọ Pút Kád Fail',
    '🗄' => 'Faíl Kábínẹt',
    '🗑' => 'Wést Pépá Báskẹt',
    '🔒' => 'Lọkd',
    '🔓' => 'Ọnlọkd',
    '🔏' => 'Lọ́kd Wit Pẹn',
    '🔐' => 'Lọ́kd Wit Kii',
    '🔑' => 'Kii',
    '🗝' => 'Óld Kii',
    '🔨' => 'Háma',
    '🪓' => 'Aks',
    '⛏' => 'Pik',
    '⚒' => 'Háma An Pik',
    '🛠' => 'Háma An Spána',
    '🗡' => 'Dága',
    '⚔' => 'Sọd-dẹm Wé Dẹm Krọs',
    '💣' => 'Bọmb',
    '🪃' => 'Búmárang',
    '🏹' => 'Bó An Áro',
    '🛡' => 'Shild',
    '🪚' => 'Kápínta Sọ',
    '🔧' => 'Spána',
    '🪛' => 'Skrúdraíva',
    '🔩' => 'Nọ́t An Bolt',
    '⚙' => 'Gia',
    '🗜' => 'Klamp',
    '⚖' => 'Balans Skel',
    '🦯' => 'Waít Stik',
    '🔗' => 'Link',
    '⛓' => 'Chen-dẹm',
    '🪝' => 'Huk',
    '🧰' => 'Túlbọks',
    '🧲' => 'Magnẹ́tọ',
    '🪜' => 'Láda',
    '🪏' => 'shọ́vẹ́l',
    '⚗' => 'Alẹ́mbik',
    '🧪' => 'Tẹ́st Tub',
    '🧫' => 'Pẹ́trí Plet',
    '🧬' => 'DNA',
    '🔬' => 'Maíkroskop',
    '🔭' => 'Tẹlískop',
    '📡' => 'Sátálaít Antẹ́na',
    '💉' => 'Sirinj',
    '🩸' => 'Drọ́p Ọf Blọd',
    '💊' => 'Mẹ́dísin',
    '🩹' => 'Plásta',
    '🩼' => 'Krọ́chis',
    '🩺' => 'Stẹteskop',
    '🩻' => 'Eks-Rè',
    '🚪' => 'Dọọ',
    '🛗' => 'Lìft',
    '🪞' => 'Mírọ',
    '🪟' => 'Wíndo',
    '🛏' => 'Bẹdi',
    '🛋' => 'Sẹtii An Lamp',
    '🪑' => 'Chia',
    '🚽' => 'Wọ́tá Sístẹm',
    '🪠' => 'Plúnja',
    '🚿' => 'Sháwa',
    '🛁' => 'Baf',
    '🪤' => 'Rát Trap',
    '🪒' => 'Rézọ',
    '🧴' => 'Lóshọ́n Bọ́tul',
    '🧷' => 'Séfti Pin',
    '🧹' => 'Brum',
    '🧺' => 'Baskẹt',
    '🧻' => 'Tọílẹ́t Rol',
    '🪣' => 'Bọ́kẹt',
    '🧼' => 'Sop',
    '🫧' => 'Bọ́bul',
    '🪥' => 'Tútbrọsh',
    '🧽' => 'Spọnj',
    '🧯' => 'Fáyá Ẹkstíngwísha',
    '🛒' => 'Kart',
    '🚬' => 'Sigarẹt',
    '⚰' => 'Kọ́fin',
    '🪦' => 'Grévston',
    '⚱' => 'Bẹ́riál Kọnténa Fọ Ásh Ọf Pẹ́sin Wé Dọ́n Dai',
    '🧿' => 'Ívúl-Aí Talisman',
    '🪬' => 'Hámsa',
    '🗿' => 'Moai',
    '🪧' => 'Plákad',
    '🪪' => 'Adẹ́ntíti Kat',
    '🏧' => 'ATM Sain',
    '🚮' => 'Dọtí Ínsaíd Dọstbin',
    '🚰' => 'Klín Wọtá',
    '♿' => 'Wílchiá Símbul',
    '🚹' => 'Man-dẹm Tọílẹ́t Rum',
    '🚺' => 'Wúman-dẹm Tọílẹ́t Rum',
    '🚻' => 'Tọílẹ́t Rum',
    '🚼' => 'Bébí Símbul',
    '🚾' => 'Wọ́tá Kọnténa',
    '🛂' => 'Páspọ́t Kọntrol',
    '🛃' => 'Kọ́stọ́m Pípol',
    '🛄' => 'Wiá To Kọlẹ́kt Bágej',
    '🛅' => 'Lọ́gej Wé Pípol Lív Bihaind',
    '⚠' => 'Wọ́nin',
    '🚸' => 'Smọ́l Pikin-dẹm De Krọs',
    '⛔' => 'Nọ́ Ẹ́nta',
    '🚫' => 'Nọ́ Dú Am',
    '🚳' => 'Nó Baísíkul Hia',
    '🚭' => 'Nó Smókin',
    '🚯' => 'Nó Drọp Drọ́p Dọtí Hia',
    '🚱' => 'Dís Wọtá Nọ́ Gúd Fọ Drink',
    '🚷' => 'Nọ́ Wáka Fọ Lẹg Hia',
    '📵' => 'Nọ́ Yúz Mọ́baíl Fon-dẹm Hia',
    '🔞' => 'Nóbọ́di Wé Dé Ọ́ndá Ẹtín Yiẹs',
    '☢' => 'Rédioáktiv',
    '☣' => 'Báioházad',
    '⬆' => 'Áro De Pọínt Ọp',
    '↗' => 'Áro De Pọínt Ọ́p-Rait',
    '➡' => 'Áro De Pọínt Rait',
    '↘' => 'Áro De Pọínt Daún-Rait',
    '⬇' => 'Áro De Pọínt Daun',
    '↙' => 'Áro De Pọínt Daún-Lẹft',
    '⬅' => 'Áro De Pọínt Lẹft',
    '↖' => 'Áro De Pọínt Ọ́p-Lẹft',
    '↕' => 'Áro De Pọínt Ọ́p An Daun',
    '↔' => 'Áro De Pọínt Lẹ́ft An Rait',
    '↩' => 'Raít Áro Kọ́v Gó Lẹft',
    '↪' => 'Lẹ́ft Áro Kọ́v Gó Raít',
    '⤴' => 'Raít Áro Kọ́v Gó Ọp',
    '⤵' => 'Raít Áro Kọ́v Gó Daun',
    '🔃' => 'Aro-dẹm De Múv Klọ́k Hánd Dairẹ́kshọn',
    '🔄' => 'Aro-dẹm Nọ́ De Múv Klọ́k Hánd Wè',
    '🔙' => 'BACK Áro',
    '🔚' => 'END Áro',
    '🔛' => 'ON! Áro',
    '🔜' => 'SOON Áro',
    '🔝' => 'TOP Áro',
    '🛐' => 'Ples Wiá Pípul De Prè',
    '⚛' => 'Átọ́m Símbul',
    '🕉' => 'Om',
    '✡' => 'Stá Ọf Dévid',
    '☸' => 'Dhárma Wil',
    '☯' => 'Yin Yang',
    '✝' => 'Látín Krọs',
    '☦' => 'Ọ́tọ́dọ́ks Krọs',
    '☪' => 'Sta An Mun Wé Kọv',
    '☮' => 'Piís Símbul',
    '🕎' => 'Mẹnóra',
    '🔯' => 'Síks Pọínt Sta Wit Dọt Fọ Mídul',
    '🪯' => 'Khánda',
    '♈' => 'É́riz',
    '♉' => 'Tọ́rus',
    '♊' => 'Jẹ́míni',
    '♋' => 'Kánsa',
    '♌' => 'Lío',
    '♍' => 'Vẹ́go',
    '♎' => 'Líbra',
    '♏' => 'Skọ́pio',
    '♐' => 'Sajitérius',
    '♑' => 'Kápríkọn',
    '♒' => 'Akwériọs',
    '♓' => 'Paísiz',
    '⛎' => 'Ọfiúkus',
    '🔀' => 'Shọ́fúl Trak-dẹm Bọ́tin',
    '🔁' => 'Ripít Bọ́tin',
    '🔂' => 'Ripít Síngúl Bọ́tin',
    '▶' => 'Plé Bọ́tin',
    '⏩' => 'Fást-Fọ́wọ́d Bọ́tin',
    '⏭' => 'Nẹ́kst Trák Bọ́tin',
    '⏯' => 'Plé ọ Stọ́p-smọl Bọ́tin',
    '◀' => 'Rivás Bọ́tin',
    '⏪' => 'Fást Rivás Bọ́tin',
    '⏮' => 'Lást Trák Bọ́tin',
    '🔼' => 'Ọ́p Bọ́tin',
    '⏫' => 'Fást Ọ́p Bọ́tin',
    '🔽' => 'Daún Bọ́tin',
    '⏬' => 'Fást Daún Bọ́tin',
    '⏸' => 'Stọ́p-smọl Bọ́tin',
    '⏹' => 'Stọ́p Bọ́tin',
    '⏺' => 'Rẹ́kọ́d Bọ́tin',
    '⏏' => 'Ijẹ́kt Bọ́tin',
    '🎦' => 'Sinẹ́ma',
    '🔅' => 'Dim Bọ́tin',
    '🔆' => 'Braít Bọ́tin',
    '📶' => 'Antẹ́ná Baa-dẹm',
    '🛜' => 'Wáyálẹs',
    '📳' => 'Vaibréshọ́n Mod',
    '📴' => 'Mọ́baíl Fon Dé Ọff',
    '♀' => 'Wúman Sain',
    '♂' => 'Man Sain',
    '⚧' => 'Transjẹ́ndá Símbul',
    '✖' => 'Taims Sain',
    '➕' => 'Plọs',
    '➖' => 'Maínọs',
    '➗' => 'Divaid',
    '🟰' => 'Bíg Íkwọ́ls Sain',
    '♾' => 'Infíníti',
    '‼' => 'Dọ́búl Ẹksklaméshọn Mak',
    '⁉' => 'Ẹksklaméshọn Kwẹ́shọn Mak',
    '❓' => 'Kwẹ́shọn Mak',
    '❔' => 'Waít Kwẹ́shọn Mak',
    '❕' => 'Wait Ẹksklaméshọn Mak',
    '❗' => 'Rẹ́d Ẹksklaméshọn Mak',
    '〰' => 'Kọv-kọ́v Dash',
    '💱' => 'Kọ́rẹ́nsí Ẹ́kschenj',
    '💲' => 'Hẹ́ví Dọ́lá Sain',
    '⚕' => 'Mẹ́díkál Sain',
    '♻' => 'Risaíkúl Sain',
    '⚜' => 'Flẹ́-De-Lis',
    '🔱' => 'Traidẹ́nt Sain',
    '📛' => 'Ném Baj',
    '🔰' => 'Japán Sain Fọ Bigína',
    '⭕' => 'Rẹd Sẹ́kul Wé Gẹ́t Hol',
    '✅' => 'Chẹ́k Mák Bọ́tin',
    '☑' => 'Chẹ́k Bọks Wit Chẹk',
    '✔' => 'Chẹ́k Mak',
    '❌' => 'Krọ́s Mak',
    '❎' => 'Krọ́s Mák Bọ́tin',
    '➰' => 'Raúnd Shép Rop',
    '➿' => 'Dọ́búl Raúnd Shép Rop',
    '〽' => 'Pát Ọltanéshọn Mak',
    '✳' => 'Astẹ́risk Wé Gẹ́t Ẹ́t Spok',
    '✴' => 'Sta Wé Gẹ́t Ẹ́t Pọint',
    '❇' => 'Spákul',
    '™' => 'Tréd Mak',
    '🫟' => 'splátá',
    '🔠' => 'Pút Látin Bíg Sain-dẹm',
    '🔡' => 'Pút Látin Smọ́l Lẹ́ta-dẹm',
    '🔢' => 'Pút Nọ́mba-dẹm',
    '🔣' => 'Pút Sain-dẹm',
    '🔤' => 'Pút Látin Lẹ́ta-dẹm',
    '🅰' => 'A Bọ́tin (Blọ́d Taip)',
    '🆎' => 'AB Bọ́tin (Blọ́d Taip)',
    '🅱' => 'B Bọ́tin (Blọ́d Taip)',
    '🆑' => 'CL Bọ́tin',
    '🆒' => 'COOL Bọ́tin',
    '🆓' => 'FREE Bọ́tin',
    'ℹ' => 'Ínfọméshọn',
    '🆔' => 'ID Bọ́tin',
    'Ⓜ' => 'M Wé Dẹm Drọ́ Sẹ́kúl Raúnd Am',
    '🆕' => 'NEW Bọ́tin',
    '🆖' => 'NG Bọ́tin',
    '🅾' => 'O Bọ́tin (Blọ́d Taip)',
    '🆗' => 'OK Bọ́tin',
    '🅿' => 'P Bọ́tin',
    '🆘' => 'SOS Bọ́tin',
    '🆙' => 'UP! Bọ́tin',
    '🆚' => 'VS Bọ́tin',
    '🈁' => 'Japán Bọ́tin Fọ “Hia”',
    '🈂' => 'Japán Bọ́tin Fọ “Sávís Chaj”',
    '🈷' => 'Japán Bọ́tin Fọ “Mọ́nt Bai Mọ́nt Amaunt”',
    '🈶' => 'Japán Bọ́tin Fọ “Nọ́ Bi Frí Ọf Chaj”',
    '🈯' => 'Japán Bọ́tin Fọ “Rizav”',
    '🉐' => 'Japán Bọ́tin Fọ “Gívawé Praiz”',
    '🈹' => 'Japán Bọ́tin Fọ “Diskaunt” Bọ́tin',
    '🈚' => 'Japán Bọ́tin Fọ “Frí Ọf Chaj”',
    '🈲' => 'Japán Bọ́tin Fọ “Fọbid”',
    '🉑' => 'Japán Bọ́tin Fọ “Asẹpt”',
    '🈸' => 'Japán Bọ́tin Fọ “Aplai”',
    '🈴' => 'Japán Bọ́tin Fọ “Mak To Ték Pás Ẹkzam”',
    '🈳' => 'Japán Bọ́tin Fọ “Vákánsi Dè”',
    '㊗' => 'Japán Bọ́tin Fọ “Kọngratuléshọn”',
    '㊙' => 'Japán Bọ́tin Fọ “Síkrit”',
    '🈺' => 'Japán Bọ́tin Fọ “Ópún Fọ Bízẹs”',
    '🈵' => 'Japán Bọ́tin Fọ “Vákánsi Nọ́ Dè”',
    '🔴' => 'Rẹ́d Sẹ́kul',
    '🟠' => 'Ọ́rénj Kọ́lọ Sẹ́kul',
    '🟡' => 'Yẹ́ló Sẹ́kul',
    '🟢' => 'Grín Sẹ́kul',
    '🔵' => 'Blú Sẹ́kul',
    '🟣' => 'Pọ́púl Kọ́lọ Sẹ́kul',
    '🟤' => 'Braún Sẹ́kul',
    '⚫' => 'Blák Sẹ́kul',
    '⚪' => 'Waít Sẹ́kul',
    '🟥' => 'Rẹ́d Skwea',
    '🟧' => 'Ọ́rénj Kọ́lọ Skwea',
    '🟨' => 'Yẹ́ló Skwea',
    '🟩' => 'Grín Skwea',
    '🟦' => 'Blú Skwea',
    '🟪' => 'Pọ́púl Kọ́lọ Skwea',
    '🟫' => 'Braún Skwea',
    '⬛' => 'Blák Bíg Skwea',
    '⬜' => 'Waít Bíg Skwea',
    '◼' => 'Blák Mídiọ́m Saíz Skwea',
    '◻' => 'Waít Mídiọ́m Saíz Skwea',
    '◾' => 'Blák Mídiọ́m-Smọ́l Skwea',
    '◽' => 'Waít Mídiọ́m-Smọ́l Skwea',
    '▪' => 'Blák Smọ́l Skwea',
    '▫' => 'Waít Smọ́l Skwea',
    '🔶' => 'Bíg Ọ́rénj Kọ́lọ́ Dáyámọnd',
    '🔷' => 'Bíg Blú Dáyámọnd',
    '🔸' => 'Smọ́l Ọ́rénj Kọ́lọ́ Dáyámọnd',
    '🔹' => 'Smọ́l Blú Dáyámọnd',
    '🔺' => 'Rẹ́d Tráyángul Wé De Pọínt Ọp',
    '🔻' => 'Rẹ́d Tráyángul Wé De Pọínt Daun',
    '💠' => 'Dáyámọnd Wit Dọt Fọ Mídul',
    '🔘' => 'Rédió Bọ́tin',
    '🔳' => 'Waít Skweá Bọ́tin',
    '🔲' => 'Blák Skweá Bọ́tin',
    '🏁' => 'Flag Wé Dẹm Drọ́ Skwea Skwea Ọn Tọp',
    '🚩' => 'Tráyángul Flag',
    '🎌' => 'Flag-dẹm Wé Krọ́s Ích Ọ́da',
    '🏴' => 'Blák Flag',
    '🏳' => 'Waít Flag',
];
