<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'musi: täiskasvanu, täiskasvanu, hele nahatoon, keskmiselt hele nahatoon',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'musi: täiskasvanu, täiskasvanu, hele nahatoon, keskmine nahatoon',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'musi: täiskasvanu, täiskasvanu, hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'musi: täiskasvanu, täiskasvanu, hele nahatoon, tume nahatoon',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'musi: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, hele nahatoon',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'musi: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, keskmine nahatoon',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'musi: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'musi: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, tume nahatoon',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'musi: täiskasvanu, täiskasvanu, keskmine nahatoon, hele nahatoon',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'musi: täiskasvanu, täiskasvanu, keskmine nahatoon, keskmiselt hele nahatoon',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'musi: täiskasvanu, täiskasvanu, keskmine nahatoon, keskmiselt tume nahatoon',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'musi: täiskasvanu, täiskasvanu, keskmine nahatoon, tume nahatoon',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'musi: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, hele nahatoon',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'musi: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'musi: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, keskmine nahatoon',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'musi: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, tume nahatoon',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'musi: täiskasvanu, täiskasvanu, tume nahatoon, hele nahatoon',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'musi: täiskasvanu, täiskasvanu, tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'musi: täiskasvanu, täiskasvanu, tume nahatoon, keskmine nahatoon',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'musi: täiskasvanu, täiskasvanu, tume nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍❤‍💋‍👨🏻' => 'musi: mees, mees, hele nahatoon',
    '👨🏻‍❤‍💋‍👨🏼' => 'musi: mees, mees, hele nahatoon, keskmiselt hele nahatoon',
    '👨🏻‍❤‍💋‍👨🏽' => 'musi: mees, mees, hele nahatoon, keskmine nahatoon',
    '👨🏻‍❤‍💋‍👨🏾' => 'musi: mees, mees, hele nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍❤‍💋‍👨🏿' => 'musi: mees, mees, hele nahatoon, tume nahatoon',
    '👨🏼‍❤‍💋‍👨🏻' => 'musi: mees, mees, keskmiselt hele nahatoon, hele nahatoon',
    '👨🏼‍❤‍💋‍👨🏼' => 'musi: mees, mees, keskmiselt hele nahatoon',
    '👨🏼‍❤‍💋‍👨🏽' => 'musi: mees, mees, keskmiselt hele nahatoon, keskmine nahatoon',
    '👨🏼‍❤‍💋‍👨🏾' => 'musi: mees, mees, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👨🏼‍❤‍💋‍👨🏿' => 'musi: mees, mees, keskmiselt hele nahatoon, tume nahatoon',
    '👨🏽‍❤‍💋‍👨🏻' => 'musi: mees, mees, keskmine nahatoon, hele nahatoon',
    '👨🏽‍❤‍💋‍👨🏼' => 'musi: mees, mees, keskmine nahatoon, keskmiselt hele nahatoon',
    '👨🏽‍❤‍💋‍👨🏽' => 'musi: mees, mees, keskmine nahatoon',
    '👨🏽‍❤‍💋‍👨🏾' => 'musi: mees, mees, keskmine nahatoon, keskmiselt tume nahatoon',
    '👨🏽‍❤‍💋‍👨🏿' => 'musi: mees, mees, keskmine nahatoon, tume nahatoon',
    '👨🏾‍❤‍💋‍👨🏻' => 'musi: mees, mees, keskmiselt tume nahatoon, hele nahatoon',
    '👨🏾‍❤‍💋‍👨🏼' => 'musi: mees, mees, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👨🏾‍❤‍💋‍👨🏽' => 'musi: mees, mees, keskmiselt tume nahatoon, keskmine nahatoon',
    '👨🏾‍❤‍💋‍👨🏾' => 'musi: mees, mees, keskmiselt tume nahatoon',
    '👨🏾‍❤‍💋‍👨🏿' => 'musi: mees, mees, keskmiselt tume nahatoon, tume nahatoon',
    '👨🏿‍❤‍💋‍👨🏻' => 'musi: mees, mees, tume nahatoon, hele nahatoon',
    '👨🏿‍❤‍💋‍👨🏼' => 'musi: mees, mees, tume nahatoon, keskmiselt hele nahatoon',
    '👨🏿‍❤‍💋‍👨🏽' => 'musi: mees, mees, tume nahatoon, keskmine nahatoon',
    '👨🏿‍❤‍💋‍👨🏾' => 'musi: mees, mees, tume nahatoon, keskmiselt tume nahatoon',
    '👨🏿‍❤‍💋‍👨🏿' => 'musi: mees, mees, tume nahatoon',
    '👩🏻‍❤‍💋‍👨🏻' => 'musi: naine, mees, hele nahatoon',
    '👩🏻‍❤‍💋‍👨🏼' => 'musi: naine, mees, hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍❤‍💋‍👨🏽' => 'musi: naine, mees, hele nahatoon, keskmine nahatoon',
    '👩🏻‍❤‍💋‍👨🏾' => 'musi: naine, mees, hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍❤‍💋‍👨🏿' => 'musi: naine, mees, hele nahatoon, tume nahatoon',
    '👩🏼‍❤‍💋‍👨🏻' => 'musi: naine, mees, keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍❤‍💋‍👨🏼' => 'musi: naine, mees, keskmiselt hele nahatoon',
    '👩🏼‍❤‍💋‍👨🏽' => 'musi: naine, mees, keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍❤‍💋‍👨🏾' => 'musi: naine, mees, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍❤‍💋‍👨🏿' => 'musi: naine, mees, keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍❤‍💋‍👨🏻' => 'musi: naine, mees, keskmine nahatoon, hele nahatoon',
    '👩🏽‍❤‍💋‍👨🏼' => 'musi: naine, mees, keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍❤‍💋‍👨🏽' => 'musi: naine, mees, keskmine nahatoon',
    '👩🏽‍❤‍💋‍👨🏾' => 'musi: naine, mees, keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍❤‍💋‍👨🏿' => 'musi: naine, mees, keskmine nahatoon, tume nahatoon',
    '👩🏾‍❤‍💋‍👨🏻' => 'musi: naine, mees, keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍❤‍💋‍👨🏼' => 'musi: naine, mees, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍❤‍💋‍👨🏽' => 'musi: naine, mees, keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍❤‍💋‍👨🏾' => 'musi: naine, mees, keskmiselt tume nahatoon',
    '👩🏾‍❤‍💋‍👨🏿' => 'musi: naine, mees, keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍❤‍💋‍👨🏻' => 'musi: naine, mees, tume nahatoon, hele nahatoon',
    '👩🏿‍❤‍💋‍👨🏼' => 'musi: naine, mees, tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍❤‍💋‍👨🏽' => 'musi: naine, mees, tume nahatoon, keskmine nahatoon',
    '👩🏿‍❤‍💋‍👨🏾' => 'musi: naine, mees, tume nahatoon, keskmiselt tume nahatoon',
    '👩🏿‍❤‍💋‍👨🏿' => 'musi: naine, mees, tume nahatoon',
    '👩🏻‍❤‍💋‍👩🏻' => 'musi: naine, naine, hele nahatoon',
    '👩🏻‍❤‍💋‍👩🏼' => 'musi: naine, naine, hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍❤‍💋‍👩🏽' => 'musi: naine, naine, hele nahatoon, keskmine nahatoon',
    '👩🏻‍❤‍💋‍👩🏾' => 'musi: naine, naine, hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍❤‍💋‍👩🏿' => 'musi: naine, naine, hele nahatoon, tume nahatoon',
    '👩🏼‍❤‍💋‍👩🏻' => 'musi: naine, naine, keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍❤‍💋‍👩🏼' => 'musi: naine, naine, keskmiselt hele nahatoon',
    '👩🏼‍❤‍💋‍👩🏽' => 'musi: naine, naine, keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍❤‍💋‍👩🏾' => 'musi: naine, naine, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍❤‍💋‍👩🏿' => 'musi: naine, naine, keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍❤‍💋‍👩🏻' => 'musi: naine, naine, keskmine nahatoon, hele nahatoon',
    '👩🏽‍❤‍💋‍👩🏼' => 'musi: naine, naine, keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍❤‍💋‍👩🏽' => 'musi: naine, naine, keskmine nahatoon',
    '👩🏽‍❤‍💋‍👩🏾' => 'musi: naine, naine, keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍❤‍💋‍👩🏿' => 'musi: naine, naine, keskmine nahatoon, tume nahatoon',
    '👩🏾‍❤‍💋‍👩🏻' => 'musi: naine, naine, keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍❤‍💋‍👩🏼' => 'musi: naine, naine, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍❤‍💋‍👩🏽' => 'musi: naine, naine, keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍❤‍💋‍👩🏾' => 'musi: naine, naine, keskmiselt tume nahatoon',
    '👩🏾‍❤‍💋‍👩🏿' => 'musi: naine, naine, keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍❤‍💋‍👩🏻' => 'musi: naine, naine, tume nahatoon, hele nahatoon',
    '👩🏿‍❤‍💋‍👩🏼' => 'musi: naine, naine, tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍❤‍💋‍👩🏽' => 'musi: naine, naine, tume nahatoon, keskmine nahatoon',
    '👩🏿‍❤‍💋‍👩🏾' => 'musi: naine, naine, tume nahatoon, keskmiselt tume nahatoon',
    '👩🏿‍❤‍💋‍👩🏿' => 'musi: naine, naine, tume nahatoon',
    '🧑🏻‍🫯‍🧑🏼' => 'maadlejad: hele nahatoon, keskmiselt hele nahatoon',
    '🧑🏻‍🫯‍🧑🏽' => 'maadlejad: hele nahatoon, keskmine nahatoon',
    '🧑🏻‍🫯‍🧑🏾' => 'maadlejad: hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍🫯‍🧑🏿' => 'maadlejad: hele nahatoon, tume nahatoon',
    '🧑🏼‍🫯‍🧑🏻' => 'maadlejad: keskmiselt hele nahatoon, hele nahatoon',
    '🧑🏼‍🫯‍🧑🏽' => 'maadlejad: keskmiselt hele nahatoon, keskmine nahatoon',
    '🧑🏼‍🫯‍🧑🏾' => 'maadlejad: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏼‍🫯‍🧑🏿' => 'maadlejad: keskmiselt hele nahatoon, tume nahatoon',
    '🧑🏽‍🫯‍🧑🏻' => 'maadlejad: keskmine nahatoon, hele nahatoon',
    '🧑🏽‍🫯‍🧑🏼' => 'maadlejad: keskmine nahatoon, keskmiselt hele nahatoon',
    '🧑🏽‍🫯‍🧑🏾' => 'maadlejad: keskmine nahatoon, keskmiselt tume nahatoon',
    '🧑🏽‍🫯‍🧑🏿' => 'maadlejad: keskmine nahatoon, tume nahatoon',
    '🧑🏾‍🫯‍🧑🏻' => 'maadlejad: keskmiselt tume nahatoon, hele nahatoon',
    '🧑🏾‍🫯‍🧑🏼' => 'maadlejad: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏾‍🫯‍🧑🏽' => 'maadlejad: keskmiselt tume nahatoon, keskmine nahatoon',
    '🧑🏾‍🫯‍🧑🏿' => 'maadlejad: keskmiselt tume nahatoon, tume nahatoon',
    '🧑🏿‍🫯‍🧑🏻' => 'maadlejad: tume nahatoon, hele nahatoon',
    '🧑🏿‍🫯‍🧑🏼' => 'maadlejad: tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏿‍🫯‍🧑🏽' => 'maadlejad: tume nahatoon, keskmine nahatoon',
    '🧑🏿‍🫯‍🧑🏾' => 'maadlejad: tume nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍❤‍🧑🏼' => 'paarike südamega: täiskasvanu, täiskasvanu, hele nahatoon, keskmiselt hele nahatoon',
    '🧑🏻‍❤‍🧑🏽' => 'paarike südamega: täiskasvanu, täiskasvanu, hele nahatoon, keskmine nahatoon',
    '🧑🏻‍❤‍🧑🏾' => 'paarike südamega: täiskasvanu, täiskasvanu, hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍❤‍🧑🏿' => 'paarike südamega: täiskasvanu, täiskasvanu, hele nahatoon, tume nahatoon',
    '🧑🏼‍❤‍🧑🏻' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, hele nahatoon',
    '🧑🏼‍❤‍🧑🏽' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, keskmine nahatoon',
    '🧑🏼‍❤‍🧑🏾' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏼‍❤‍🧑🏿' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt hele nahatoon, tume nahatoon',
    '🧑🏽‍❤‍🧑🏻' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmine nahatoon, hele nahatoon',
    '🧑🏽‍❤‍🧑🏼' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmine nahatoon, keskmiselt hele nahatoon',
    '🧑🏽‍❤‍🧑🏾' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmine nahatoon, keskmiselt tume nahatoon',
    '🧑🏽‍❤‍🧑🏿' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmine nahatoon, tume nahatoon',
    '🧑🏾‍❤‍🧑🏻' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, hele nahatoon',
    '🧑🏾‍❤‍🧑🏼' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏾‍❤‍🧑🏽' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, keskmine nahatoon',
    '🧑🏾‍❤‍🧑🏿' => 'paarike südamega: täiskasvanu, täiskasvanu, keskmiselt tume nahatoon, tume nahatoon',
    '🧑🏿‍❤‍🧑🏻' => 'paarike südamega: täiskasvanu, täiskasvanu, tume nahatoon, hele nahatoon',
    '🧑🏿‍❤‍🧑🏼' => 'paarike südamega: täiskasvanu, täiskasvanu, tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏿‍❤‍🧑🏽' => 'paarike südamega: täiskasvanu, täiskasvanu, tume nahatoon, keskmine nahatoon',
    '🧑🏿‍❤‍🧑🏾' => 'paarike südamega: täiskasvanu, täiskasvanu, tume nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍🐰‍🧑🏼' => 'pidutsevad jänesekõrvadega inimesed: hele nahatoon, keskmiselt hele nahatoon',
    '🧑🏻‍🐰‍🧑🏽' => 'pidutsevad jänesekõrvadega inimesed: hele nahatoon, keskmine nahatoon',
    '🧑🏻‍🐰‍🧑🏾' => 'pidutsevad jänesekõrvadega inimesed: hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍🐰‍🧑🏿' => 'pidutsevad jänesekõrvadega inimesed: hele nahatoon, tume nahatoon',
    '🧑🏼‍🐰‍🧑🏻' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt hele nahatoon, hele nahatoon',
    '🧑🏼‍🐰‍🧑🏽' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt hele nahatoon, keskmine nahatoon',
    '🧑🏼‍🐰‍🧑🏾' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏼‍🐰‍🧑🏿' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt hele nahatoon, tume nahatoon',
    '🧑🏽‍🐰‍🧑🏻' => 'pidutsevad jänesekõrvadega inimesed: keskmine nahatoon, hele nahatoon',
    '🧑🏽‍🐰‍🧑🏼' => 'pidutsevad jänesekõrvadega inimesed: keskmine nahatoon, keskmiselt hele nahatoon',
    '🧑🏽‍🐰‍🧑🏾' => 'pidutsevad jänesekõrvadega inimesed: keskmine nahatoon, keskmiselt tume nahatoon',
    '🧑🏽‍🐰‍🧑🏿' => 'pidutsevad jänesekõrvadega inimesed: keskmine nahatoon, tume nahatoon',
    '🧑🏾‍🐰‍🧑🏻' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt tume nahatoon, hele nahatoon',
    '🧑🏾‍🐰‍🧑🏼' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏾‍🐰‍🧑🏽' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt tume nahatoon, keskmine nahatoon',
    '🧑🏾‍🐰‍🧑🏿' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt tume nahatoon, tume nahatoon',
    '🧑🏿‍🐰‍🧑🏻' => 'pidutsevad jänesekõrvadega inimesed: tume nahatoon, hele nahatoon',
    '🧑🏿‍🐰‍🧑🏼' => 'pidutsevad jänesekõrvadega inimesed: tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏿‍🐰‍🧑🏽' => 'pidutsevad jänesekõrvadega inimesed: tume nahatoon, keskmine nahatoon',
    '🧑🏿‍🐰‍🧑🏾' => 'pidutsevad jänesekõrvadega inimesed: tume nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍🫯‍👨🏼' => 'maadlevad mehed: hele nahatoon, keskmiselt hele nahatoon',
    '👨🏻‍🫯‍👨🏽' => 'maadlevad mehed: hele nahatoon, keskmine nahatoon',
    '👨🏻‍🫯‍👨🏾' => 'maadlevad mehed: hele nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍🫯‍👨🏿' => 'maadlevad mehed: hele nahatoon, tume nahatoon',
    '👨🏼‍🫯‍👨🏻' => 'maadlevad mehed: keskmiselt hele nahatoon, hele nahatoon',
    '👨🏼‍🫯‍👨🏽' => 'maadlevad mehed: keskmiselt hele nahatoon, keskmine nahatoon',
    '👨🏼‍🫯‍👨🏾' => 'maadlevad mehed: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👨🏼‍🫯‍👨🏿' => 'maadlevad mehed: keskmiselt hele nahatoon, tume nahatoon',
    '👨🏽‍🫯‍👨🏻' => 'maadlevad mehed: keskmine nahatoon, hele nahatoon',
    '👨🏽‍🫯‍👨🏼' => 'maadlevad mehed: keskmine nahatoon, keskmiselt hele nahatoon',
    '👨🏽‍🫯‍👨🏾' => 'maadlevad mehed: keskmine nahatoon, keskmiselt tume nahatoon',
    '👨🏽‍🫯‍👨🏿' => 'maadlevad mehed: keskmine nahatoon, tume nahatoon',
    '👨🏾‍🫯‍👨🏻' => 'maadlevad mehed: keskmiselt tume nahatoon, hele nahatoon',
    '👨🏾‍🫯‍👨🏼' => 'maadlevad mehed: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👨🏾‍🫯‍👨🏽' => 'maadlevad mehed: keskmiselt tume nahatoon, keskmine nahatoon',
    '👨🏾‍🫯‍👨🏿' => 'maadlevad mehed: keskmiselt tume nahatoon, tume nahatoon',
    '👨🏿‍🫯‍👨🏻' => 'maadlevad mehed: tume nahatoon, hele nahatoon',
    '👨🏿‍🫯‍👨🏼' => 'maadlevad mehed: tume nahatoon, keskmiselt hele nahatoon',
    '👨🏿‍🫯‍👨🏽' => 'maadlevad mehed: tume nahatoon, keskmine nahatoon',
    '👨🏿‍🫯‍👨🏾' => 'maadlevad mehed: tume nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍❤‍👨🏻' => 'paarike südamega: mees, mees, hele nahatoon',
    '👨🏻‍❤‍👨🏼' => 'paarike südamega: mees, mees, hele nahatoon, keskmiselt hele nahatoon',
    '👨🏻‍❤‍👨🏽' => 'paarike südamega: mees, mees, hele nahatoon, keskmine nahatoon',
    '👨🏻‍❤‍👨🏾' => 'paarike südamega: mees, mees, hele nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍❤‍👨🏿' => 'paarike südamega: mees, mees, hele nahatoon, tume nahatoon',
    '👨🏼‍❤‍👨🏻' => 'paarike südamega: mees, mees, keskmiselt hele nahatoon, hele nahatoon',
    '👨🏼‍❤‍👨🏼' => 'paarike südamega: mees, mees, keskmiselt hele nahatoon',
    '👨🏼‍❤‍👨🏽' => 'paarike südamega: mees, mees, keskmiselt hele nahatoon, keskmine nahatoon',
    '👨🏼‍❤‍👨🏾' => 'paarike südamega: mees, mees, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👨🏼‍❤‍👨🏿' => 'paarike südamega: mees, mees, keskmiselt hele nahatoon, tume nahatoon',
    '👨🏽‍❤‍👨🏻' => 'paarike südamega: mees, mees, keskmine nahatoon, hele nahatoon',
    '👨🏽‍❤‍👨🏼' => 'paarike südamega: mees, mees, keskmine nahatoon, keskmiselt hele nahatoon',
    '👨🏽‍❤‍👨🏽' => 'paarike südamega: mees, mees, keskmine nahatoon',
    '👨🏽‍❤‍👨🏾' => 'paarike südamega: mees, mees, keskmine nahatoon, keskmiselt tume nahatoon',
    '👨🏽‍❤‍👨🏿' => 'paarike südamega: mees, mees, keskmine nahatoon, tume nahatoon',
    '👨🏾‍❤‍👨🏻' => 'paarike südamega: mees, mees, keskmiselt tume nahatoon, hele nahatoon',
    '👨🏾‍❤‍👨🏼' => 'paarike südamega: mees, mees, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👨🏾‍❤‍👨🏽' => 'paarike südamega: mees, mees, keskmiselt tume nahatoon, keskmine nahatoon',
    '👨🏾‍❤‍👨🏾' => 'paarike südamega: mees, mees, keskmiselt tume nahatoon',
    '👨🏾‍❤‍👨🏿' => 'paarike südamega: mees, mees, keskmiselt tume nahatoon, tume nahatoon',
    '👨🏿‍❤‍👨🏻' => 'paarike südamega: mees, mees, tume nahatoon, hele nahatoon',
    '👨🏿‍❤‍👨🏼' => 'paarike südamega: mees, mees, tume nahatoon, keskmiselt hele nahatoon',
    '👨🏿‍❤‍👨🏽' => 'paarike südamega: mees, mees, tume nahatoon, keskmine nahatoon',
    '👨🏿‍❤‍👨🏾' => 'paarike südamega: mees, mees, tume nahatoon, keskmiselt tume nahatoon',
    '👨🏿‍❤‍👨🏿' => 'paarike südamega: mees, mees, tume nahatoon',
    '👨🏻‍🐰‍👨🏼' => 'pidutsevad jänesekõrvadega mehed: hele nahatoon, keskmiselt hele nahatoon',
    '👨🏻‍🐰‍👨🏽' => 'pidutsevad jänesekõrvadega mehed: hele nahatoon, keskmine nahatoon',
    '👨🏻‍🐰‍👨🏾' => 'pidutsevad jänesekõrvadega mehed: hele nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍🐰‍👨🏿' => 'pidutsevad jänesekõrvadega mehed: hele nahatoon, tume nahatoon',
    '👨🏼‍🐰‍👨🏻' => 'pidutsevad jänesekõrvadega mehed: keskmiselt hele nahatoon, hele nahatoon',
    '👨🏼‍🐰‍👨🏽' => 'pidutsevad jänesekõrvadega mehed: keskmiselt hele nahatoon, keskmine nahatoon',
    '👨🏼‍🐰‍👨🏾' => 'pidutsevad jänesekõrvadega mehed: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👨🏼‍🐰‍👨🏿' => 'pidutsevad jänesekõrvadega mehed: keskmiselt hele nahatoon, tume nahatoon',
    '👨🏽‍🐰‍👨🏻' => 'pidutsevad jänesekõrvadega mehed: keskmine nahatoon, hele nahatoon',
    '👨🏽‍🐰‍👨🏼' => 'pidutsevad jänesekõrvadega mehed: keskmine nahatoon, keskmiselt hele nahatoon',
    '👨🏽‍🐰‍👨🏾' => 'pidutsevad jänesekõrvadega mehed: keskmine nahatoon, keskmiselt tume nahatoon',
    '👨🏽‍🐰‍👨🏿' => 'pidutsevad jänesekõrvadega mehed: keskmine nahatoon, tume nahatoon',
    '👨🏾‍🐰‍👨🏻' => 'pidutsevad jänesekõrvadega mehed: keskmiselt tume nahatoon, hele nahatoon',
    '👨🏾‍🐰‍👨🏼' => 'pidutsevad jänesekõrvadega mehed: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👨🏾‍🐰‍👨🏽' => 'pidutsevad jänesekõrvadega mehed: keskmiselt tume nahatoon, keskmine nahatoon',
    '👨🏾‍🐰‍👨🏿' => 'pidutsevad jänesekõrvadega mehed: keskmiselt tume nahatoon, tume nahatoon',
    '👨🏿‍🐰‍👨🏻' => 'pidutsevad jänesekõrvadega mehed: tume nahatoon, hele nahatoon',
    '👨🏿‍🐰‍👨🏼' => 'pidutsevad jänesekõrvadega mehed: tume nahatoon, keskmiselt hele nahatoon',
    '👨🏿‍🐰‍👨🏽' => 'pidutsevad jänesekõrvadega mehed: tume nahatoon, keskmine nahatoon',
    '👨🏿‍🐰‍👨🏾' => 'pidutsevad jänesekõrvadega mehed: tume nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍🫯‍👩🏼' => 'maadlevad naised: hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍🫯‍👩🏽' => 'maadlevad naised: hele nahatoon, keskmine nahatoon',
    '👩🏻‍🫯‍👩🏾' => 'maadlevad naised: hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍🫯‍👩🏿' => 'maadlevad naised: hele nahatoon, tume nahatoon',
    '👩🏼‍🫯‍👩🏻' => 'maadlevad naised: keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍🫯‍👩🏽' => 'maadlevad naised: keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍🫯‍👩🏾' => 'maadlevad naised: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍🫯‍👩🏿' => 'maadlevad naised: keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍🫯‍👩🏻' => 'maadlevad naised: keskmine nahatoon, hele nahatoon',
    '👩🏽‍🫯‍👩🏼' => 'maadlevad naised: keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍🫯‍👩🏾' => 'maadlevad naised: keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍🫯‍👩🏿' => 'maadlevad naised: keskmine nahatoon, tume nahatoon',
    '👩🏾‍🫯‍👩🏻' => 'maadlevad naised: keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍🫯‍👩🏼' => 'maadlevad naised: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍🫯‍👩🏽' => 'maadlevad naised: keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍🫯‍👩🏿' => 'maadlevad naised: keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍🫯‍👩🏻' => 'maadlevad naised: tume nahatoon, hele nahatoon',
    '👩🏿‍🫯‍👩🏼' => 'maadlevad naised: tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍🫯‍👩🏽' => 'maadlevad naised: tume nahatoon, keskmine nahatoon',
    '👩🏿‍🫯‍👩🏾' => 'maadlevad naised: tume nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍❤‍👨🏻' => 'paarike südamega: naine, mees, hele nahatoon',
    '👩🏻‍❤‍👨🏼' => 'paarike südamega: naine, mees, hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍❤‍👨🏽' => 'paarike südamega: naine, mees, hele nahatoon, keskmine nahatoon',
    '👩🏻‍❤‍👨🏾' => 'paarike südamega: naine, mees, hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍❤‍👨🏿' => 'paarike südamega: naine, mees, hele nahatoon, tume nahatoon',
    '👩🏼‍❤‍👨🏻' => 'paarike südamega: naine, mees, keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍❤‍👨🏼' => 'paarike südamega: naine, mees, keskmiselt hele nahatoon',
    '👩🏼‍❤‍👨🏽' => 'paarike südamega: naine, mees, keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍❤‍👨🏾' => 'paarike südamega: naine, mees, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍❤‍👨🏿' => 'paarike südamega: naine, mees, keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍❤‍👨🏻' => 'paarike südamega: naine, mees, keskmine nahatoon, hele nahatoon',
    '👩🏽‍❤‍👨🏼' => 'paarike südamega: naine, mees, keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍❤‍👨🏽' => 'paarike südamega: naine, mees, keskmine nahatoon',
    '👩🏽‍❤‍👨🏾' => 'paarike südamega: naine, mees, keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍❤‍👨🏿' => 'paarike südamega: naine, mees, keskmine nahatoon, tume nahatoon',
    '👩🏾‍❤‍👨🏻' => 'paarike südamega: naine, mees, keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍❤‍👨🏼' => 'paarike südamega: naine, mees, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍❤‍👨🏽' => 'paarike südamega: naine, mees, keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍❤‍👨🏾' => 'paarike südamega: naine, mees, keskmiselt tume nahatoon',
    '👩🏾‍❤‍👨🏿' => 'paarike südamega: naine, mees, keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍❤‍👨🏻' => 'paarike südamega: naine, mees, tume nahatoon, hele nahatoon',
    '👩🏿‍❤‍👨🏼' => 'paarike südamega: naine, mees, tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍❤‍👨🏽' => 'paarike südamega: naine, mees, tume nahatoon, keskmine nahatoon',
    '👩🏿‍❤‍👨🏾' => 'paarike südamega: naine, mees, tume nahatoon, keskmiselt tume nahatoon',
    '👩🏿‍❤‍👨🏿' => 'paarike südamega: naine, mees, tume nahatoon',
    '👩🏻‍❤‍👩🏻' => 'paarike südamega: naine, naine, hele nahatoon',
    '👩🏻‍❤‍👩🏼' => 'paarike südamega: naine, naine, hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍❤‍👩🏽' => 'paarike südamega: naine, naine, hele nahatoon, keskmine nahatoon',
    '👩🏻‍❤‍👩🏾' => 'paarike südamega: naine, naine, hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍❤‍👩🏿' => 'paarike südamega: naine, naine, hele nahatoon, tume nahatoon',
    '👩🏼‍❤‍👩🏻' => 'paarike südamega: naine, naine, keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍❤‍👩🏼' => 'paarike südamega: naine, naine, keskmiselt hele nahatoon',
    '👩🏼‍❤‍👩🏽' => 'paarike südamega: naine, naine, keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍❤‍👩🏾' => 'paarike südamega: naine, naine, keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍❤‍👩🏿' => 'paarike südamega: naine, naine, keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍❤‍👩🏻' => 'paarike südamega: naine, naine, keskmine nahatoon, hele nahatoon',
    '👩🏽‍❤‍👩🏼' => 'paarike südamega: naine, naine, keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍❤‍👩🏽' => 'paarike südamega: naine, naine, keskmine nahatoon',
    '👩🏽‍❤‍👩🏾' => 'paarike südamega: naine, naine, keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍❤‍👩🏿' => 'paarike südamega: naine, naine, keskmine nahatoon, tume nahatoon',
    '👩🏾‍❤‍👩🏻' => 'paarike südamega: naine, naine, keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍❤‍👩🏼' => 'paarike südamega: naine, naine, keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍❤‍👩🏽' => 'paarike südamega: naine, naine, keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍❤‍👩🏾' => 'paarike südamega: naine, naine, keskmiselt tume nahatoon',
    '👩🏾‍❤‍👩🏿' => 'paarike südamega: naine, naine, keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍❤‍👩🏻' => 'paarike südamega: naine, naine, tume nahatoon, hele nahatoon',
    '👩🏿‍❤‍👩🏼' => 'paarike südamega: naine, naine, tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍❤‍👩🏽' => 'paarike südamega: naine, naine, tume nahatoon, keskmine nahatoon',
    '👩🏿‍❤‍👩🏾' => 'paarike südamega: naine, naine, tume nahatoon, keskmiselt tume nahatoon',
    '👩🏿‍❤‍👩🏿' => 'paarike südamega: naine, naine, tume nahatoon',
    '👩🏻‍🐰‍👩🏼' => 'pidutsevad jänesekõrvadega naised: hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍🐰‍👩🏽' => 'pidutsevad jänesekõrvadega naised: hele nahatoon, keskmine nahatoon',
    '👩🏻‍🐰‍👩🏾' => 'pidutsevad jänesekõrvadega naised: hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍🐰‍👩🏿' => 'pidutsevad jänesekõrvadega naised: hele nahatoon, tume nahatoon',
    '👩🏼‍🐰‍👩🏻' => 'pidutsevad jänesekõrvadega naised: keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍🐰‍👩🏽' => 'pidutsevad jänesekõrvadega naised: keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍🐰‍👩🏾' => 'pidutsevad jänesekõrvadega naised: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍🐰‍👩🏿' => 'pidutsevad jänesekõrvadega naised: keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍🐰‍👩🏻' => 'pidutsevad jänesekõrvadega naised: keskmine nahatoon, hele nahatoon',
    '👩🏽‍🐰‍👩🏼' => 'pidutsevad jänesekõrvadega naised: keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍🐰‍👩🏾' => 'pidutsevad jänesekõrvadega naised: keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍🐰‍👩🏿' => 'pidutsevad jänesekõrvadega naised: keskmine nahatoon, tume nahatoon',
    '👩🏾‍🐰‍👩🏻' => 'pidutsevad jänesekõrvadega naised: keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍🐰‍👩🏼' => 'pidutsevad jänesekõrvadega naised: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍🐰‍👩🏽' => 'pidutsevad jänesekõrvadega naised: keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍🐰‍👩🏿' => 'pidutsevad jänesekõrvadega naised: keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍🐰‍👩🏻' => 'pidutsevad jänesekõrvadega naised: tume nahatoon, hele nahatoon',
    '👩🏿‍🐰‍👩🏼' => 'pidutsevad jänesekõrvadega naised: tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍🐰‍👩🏽' => 'pidutsevad jänesekõrvadega naised: tume nahatoon, keskmine nahatoon',
    '👩🏿‍🐰‍👩🏾' => 'pidutsevad jänesekõrvadega naised: tume nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍🤝‍🧑🏻' => 'inimesed hoiavad käest kinni: hele nahatoon',
    '🧑🏻‍🤝‍🧑🏼' => 'inimesed hoiavad käest kinni: hele nahatoon, keskmiselt hele nahatoon',
    '🧑🏻‍🤝‍🧑🏽' => 'inimesed hoiavad käest kinni: hele nahatoon, keskmine nahatoon',
    '🧑🏻‍🤝‍🧑🏾' => 'inimesed hoiavad käest kinni: hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏻‍🤝‍🧑🏿' => 'inimesed hoiavad käest kinni: hele nahatoon, tume nahatoon',
    '🧑🏼‍🤝‍🧑🏻' => 'inimesed hoiavad käest kinni: keskmiselt hele nahatoon, hele nahatoon',
    '🧑🏼‍🤝‍🧑🏼' => 'inimesed hoiavad käest kinni: keskmiselt hele nahatoon',
    '🧑🏼‍🤝‍🧑🏽' => 'inimesed hoiavad käest kinni: keskmiselt hele nahatoon, keskmine nahatoon',
    '🧑🏼‍🤝‍🧑🏾' => 'inimesed hoiavad käest kinni: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '🧑🏼‍🤝‍🧑🏿' => 'inimesed hoiavad käest kinni: keskmiselt hele nahatoon, tume nahatoon',
    '🧑🏽‍🤝‍🧑🏻' => 'inimesed hoiavad käest kinni: keskmine nahatoon, hele nahatoon',
    '🧑🏽‍🤝‍🧑🏼' => 'inimesed hoiavad käest kinni: keskmine nahatoon, keskmiselt hele nahatoon',
    '🧑🏽‍🤝‍🧑🏽' => 'inimesed hoiavad käest kinni: keskmine nahatoon',
    '🧑🏽‍🤝‍🧑🏾' => 'inimesed hoiavad käest kinni: keskmine nahatoon, keskmiselt tume nahatoon',
    '🧑🏽‍🤝‍🧑🏿' => 'inimesed hoiavad käest kinni: keskmine nahatoon, tume nahatoon',
    '🧑🏾‍🤝‍🧑🏻' => 'inimesed hoiavad käest kinni: keskmiselt tume nahatoon, hele nahatoon',
    '🧑🏾‍🤝‍🧑🏼' => 'inimesed hoiavad käest kinni: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏾‍🤝‍🧑🏽' => 'inimesed hoiavad käest kinni: keskmiselt tume nahatoon, keskmine nahatoon',
    '🧑🏾‍🤝‍🧑🏾' => 'inimesed hoiavad käest kinni: keskmiselt tume nahatoon',
    '🧑🏾‍🤝‍🧑🏿' => 'inimesed hoiavad käest kinni: keskmiselt tume nahatoon, tume nahatoon',
    '🧑🏿‍🤝‍🧑🏻' => 'inimesed hoiavad käest kinni: tume nahatoon, hele nahatoon',
    '🧑🏿‍🤝‍🧑🏼' => 'inimesed hoiavad käest kinni: tume nahatoon, keskmiselt hele nahatoon',
    '🧑🏿‍🤝‍🧑🏽' => 'inimesed hoiavad käest kinni: tume nahatoon, keskmine nahatoon',
    '🧑🏿‍🤝‍🧑🏾' => 'inimesed hoiavad käest kinni: tume nahatoon, keskmiselt tume nahatoon',
    '🧑🏿‍🤝‍🧑🏿' => 'inimesed hoiavad käest kinni: tume nahatoon',
    '👩🏻‍🤝‍👩🏼' => 'käest kinni hoidvad naised: hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍🤝‍👩🏽' => 'käest kinni hoidvad naised: hele nahatoon, keskmine nahatoon',
    '👩🏻‍🤝‍👩🏾' => 'käest kinni hoidvad naised: hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍🤝‍👩🏿' => 'käest kinni hoidvad naised: hele nahatoon, tume nahatoon',
    '👩🏼‍🤝‍👩🏻' => 'käest kinni hoidvad naised: keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍🤝‍👩🏽' => 'käest kinni hoidvad naised: keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍🤝‍👩🏾' => 'käest kinni hoidvad naised: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍🤝‍👩🏿' => 'käest kinni hoidvad naised: keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍🤝‍👩🏻' => 'käest kinni hoidvad naised: keskmine nahatoon, hele nahatoon',
    '👩🏽‍🤝‍👩🏼' => 'käest kinni hoidvad naised: keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍🤝‍👩🏾' => 'käest kinni hoidvad naised: keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍🤝‍👩🏿' => 'käest kinni hoidvad naised: keskmine nahatoon, tume nahatoon',
    '👩🏾‍🤝‍👩🏻' => 'käest kinni hoidvad naised: keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍🤝‍👩🏼' => 'käest kinni hoidvad naised: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍🤝‍👩🏽' => 'käest kinni hoidvad naised: keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍🤝‍👩🏿' => 'käest kinni hoidvad naised: keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍🤝‍👩🏻' => 'käest kinni hoidvad naised: tume nahatoon, hele nahatoon',
    '👩🏿‍🤝‍👩🏼' => 'käest kinni hoidvad naised: tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍🤝‍👩🏽' => 'käest kinni hoidvad naised: tume nahatoon, keskmine nahatoon',
    '👩🏿‍🤝‍👩🏾' => 'käest kinni hoidvad naised: tume nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍🤝‍👨🏼' => 'mees ja naine hoiavad käest kinni: hele nahatoon, keskmiselt hele nahatoon',
    '👩🏻‍🤝‍👨🏽' => 'mees ja naine hoiavad käest kinni: hele nahatoon, keskmine nahatoon',
    '👩🏻‍🤝‍👨🏾' => 'mees ja naine hoiavad käest kinni: hele nahatoon, keskmiselt tume nahatoon',
    '👩🏻‍🤝‍👨🏿' => 'mees ja naine hoiavad käest kinni: hele nahatoon, tume nahatoon',
    '👩🏼‍🤝‍👨🏻' => 'mees ja naine hoiavad käest kinni: keskmiselt hele nahatoon, hele nahatoon',
    '👩🏼‍🤝‍👨🏽' => 'mees ja naine hoiavad käest kinni: keskmiselt hele nahatoon, keskmine nahatoon',
    '👩🏼‍🤝‍👨🏾' => 'mees ja naine hoiavad käest kinni: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👩🏼‍🤝‍👨🏿' => 'mees ja naine hoiavad käest kinni: keskmiselt hele nahatoon, tume nahatoon',
    '👩🏽‍🤝‍👨🏻' => 'mees ja naine hoiavad käest kinni: keskmine nahatoon, hele nahatoon',
    '👩🏽‍🤝‍👨🏼' => 'mees ja naine hoiavad käest kinni: keskmine nahatoon, keskmiselt hele nahatoon',
    '👩🏽‍🤝‍👨🏾' => 'mees ja naine hoiavad käest kinni: keskmine nahatoon, keskmiselt tume nahatoon',
    '👩🏽‍🤝‍👨🏿' => 'mees ja naine hoiavad käest kinni: keskmine nahatoon, tume nahatoon',
    '👩🏾‍🤝‍👨🏻' => 'mees ja naine hoiavad käest kinni: keskmiselt tume nahatoon, hele nahatoon',
    '👩🏾‍🤝‍👨🏼' => 'mees ja naine hoiavad käest kinni: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👩🏾‍🤝‍👨🏽' => 'mees ja naine hoiavad käest kinni: keskmiselt tume nahatoon, keskmine nahatoon',
    '👩🏾‍🤝‍👨🏿' => 'mees ja naine hoiavad käest kinni: keskmiselt tume nahatoon, tume nahatoon',
    '👩🏿‍🤝‍👨🏻' => 'mees ja naine hoiavad käest kinni: tume nahatoon, hele nahatoon',
    '👩🏿‍🤝‍👨🏼' => 'mees ja naine hoiavad käest kinni: tume nahatoon, keskmiselt hele nahatoon',
    '👩🏿‍🤝‍👨🏽' => 'mees ja naine hoiavad käest kinni: tume nahatoon, keskmine nahatoon',
    '👩🏿‍🤝‍👨🏾' => 'mees ja naine hoiavad käest kinni: tume nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍🤝‍👨🏼' => 'käest kinni hoidvad mehed: hele nahatoon, keskmiselt hele nahatoon',
    '👨🏻‍🤝‍👨🏽' => 'käest kinni hoidvad mehed: hele nahatoon, keskmine nahatoon',
    '👨🏻‍🤝‍👨🏾' => 'käest kinni hoidvad mehed: hele nahatoon, keskmiselt tume nahatoon',
    '👨🏻‍🤝‍👨🏿' => 'käest kinni hoidvad mehed: hele nahatoon, tume nahatoon',
    '👨🏼‍🤝‍👨🏻' => 'käest kinni hoidvad mehed: keskmiselt hele nahatoon, hele nahatoon',
    '👨🏼‍🤝‍👨🏽' => 'käest kinni hoidvad mehed: keskmiselt hele nahatoon, keskmine nahatoon',
    '👨🏼‍🤝‍👨🏾' => 'käest kinni hoidvad mehed: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '👨🏼‍🤝‍👨🏿' => 'käest kinni hoidvad mehed: keskmiselt hele nahatoon, tume nahatoon',
    '👨🏽‍🤝‍👨🏻' => 'käest kinni hoidvad mehed: keskmine nahatoon, hele nahatoon',
    '👨🏽‍🤝‍👨🏼' => 'käest kinni hoidvad mehed: keskmine nahatoon, keskmiselt hele nahatoon',
    '👨🏽‍🤝‍👨🏾' => 'käest kinni hoidvad mehed: keskmine nahatoon, keskmiselt tume nahatoon',
    '👨🏽‍🤝‍👨🏿' => 'käest kinni hoidvad mehed: keskmine nahatoon, tume nahatoon',
    '👨🏾‍🤝‍👨🏻' => 'käest kinni hoidvad mehed: keskmiselt tume nahatoon, hele nahatoon',
    '👨🏾‍🤝‍👨🏼' => 'käest kinni hoidvad mehed: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '👨🏾‍🤝‍👨🏽' => 'käest kinni hoidvad mehed: keskmiselt tume nahatoon, keskmine nahatoon',
    '👨🏾‍🤝‍👨🏿' => 'käest kinni hoidvad mehed: keskmiselt tume nahatoon, tume nahatoon',
    '👨🏿‍🤝‍👨🏻' => 'käest kinni hoidvad mehed: tume nahatoon, hele nahatoon',
    '👨🏿‍🤝‍👨🏼' => 'käest kinni hoidvad mehed: tume nahatoon, keskmiselt hele nahatoon',
    '👨🏿‍🤝‍👨🏽' => 'käest kinni hoidvad mehed: tume nahatoon, keskmine nahatoon',
    '👨🏿‍🤝‍👨🏾' => 'käest kinni hoidvad mehed: tume nahatoon, keskmiselt tume nahatoon',
    '👩‍❤‍💋‍👨' => 'musi: naine, mees',
    '👨‍❤‍💋‍👨' => 'musi: mees, mees',
    '👩‍❤‍💋‍👩' => 'musi: naine, naine',
    '👨‍👩‍👧‍👦' => 'perekond: mees, naine, tüdruk, poiss',
    '👨‍👩‍👦‍👦' => 'perekond: mees, naine, poiss, poiss',
    '👨‍👩‍👧‍👧' => 'perekond: mees, naine, tüdruk, tüdruk',
    '👨‍👨‍👧‍👦' => 'perekond: mees, mees, tüdruk, poiss',
    '👨‍👨‍👦‍👦' => 'perekond: mees, mees, poiss, poiss',
    '👨‍👨‍👧‍👧' => 'perekond: mees, mees, tüdruk, tüdruk',
    '👩‍👩‍👧‍👦' => 'perekond: naine, naine, tüdruk, poiss',
    '👩‍👩‍👦‍👦' => 'perekond: naine, naine, poiss, poiss',
    '👩‍👩‍👧‍👧' => 'perekond: naine, naine, tüdruk, tüdruk',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'lipp: Inglismaa',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'lipp: Šotimaa',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'lipp: Wales',
    '🧑‍🧑‍🧒‍🧒' => 'perekond: täiskasvanu, täiskasvanu, laps, laps',
    '🚶🏻‍♀‍➡' => 'kõndiv naine: hele nahatoon, suunaga paremale',
    '🚶🏼‍♀‍➡' => 'kõndiv naine: keskmiselt hele nahatoon, suunaga paremale',
    '🚶🏽‍♀‍➡' => 'kõndiv naine: keskmine nahatoon, suunaga paremale',
    '🚶🏾‍♀‍➡' => 'kõndiv naine: keskmiselt tume nahatoon, suunaga paremale',
    '🚶🏿‍♀‍➡' => 'kõndiv naine: tume nahatoon, suunaga paremale',
    '🚶🏻‍♂‍➡' => 'kõndiv mees: hele nahatoon, suunaga paremale',
    '🚶🏼‍♂‍➡' => 'kõndiv mees: keskmiselt hele nahatoon, suunaga paremale',
    '🚶🏽‍♂‍➡' => 'kõndiv mees: keskmine nahatoon, suunaga paremale',
    '🚶🏾‍♂‍➡' => 'kõndiv mees: keskmiselt tume nahatoon, suunaga paremale',
    '🚶🏿‍♂‍➡' => 'kõndiv mees: tume nahatoon, suunaga paremale',
    '🧎🏻‍♀‍➡' => 'põlvitav naine: hele nahatoon, suunaga paremale',
    '🧎🏼‍♀‍➡' => 'põlvitav naine: keskmiselt hele nahatoon, suunaga paremale',
    '🧎🏽‍♀‍➡' => 'põlvitav naine: keskmine nahatoon, suunaga paremale',
    '🧎🏾‍♀‍➡' => 'põlvitav naine: keskmiselt tume nahatoon, suunaga paremale',
    '🧎🏿‍♀‍➡' => 'põlvitav naine: tume nahatoon, suunaga paremale',
    '🧎🏻‍♂‍➡' => 'põlvitav mees: hele nahatoon, suunaga paremale',
    '🧎🏼‍♂‍➡' => 'põlvitav mees: keskmiselt hele nahatoon, suunaga paremale',
    '🧎🏽‍♂‍➡' => 'põlvitav mees: keskmine nahatoon, suunaga paremale',
    '🧎🏾‍♂‍➡' => 'põlvitav mees: keskmiselt tume nahatoon, suunaga paremale',
    '🧎🏿‍♂‍➡' => 'põlvitav mees: tume nahatoon, suunaga paremale',
    '🧑🏻‍🦯‍➡' => 'valge kepiga inimene: hele nahatoon, suunaga paremale',
    '🧑🏼‍🦯‍➡' => 'valge kepiga inimene: keskmiselt hele nahatoon, suunaga paremale',
    '🧑🏽‍🦯‍➡' => 'valge kepiga inimene: keskmine nahatoon, suunaga paremale',
    '🧑🏾‍🦯‍➡' => 'valge kepiga inimene: keskmiselt tume nahatoon, suunaga paremale',
    '🧑🏿‍🦯‍➡' => 'valge kepiga inimene: tume nahatoon, suunaga paremale',
    '👨🏻‍🦯‍➡' => 'valge kepiga mees: hele nahatoon, suunaga paremale',
    '👨🏼‍🦯‍➡' => 'valge kepiga mees: keskmiselt hele nahatoon, suunaga paremale',
    '👨🏽‍🦯‍➡' => 'valge kepiga mees: keskmine nahatoon, suunaga paremale',
    '👨🏾‍🦯‍➡' => 'valge kepiga mees: keskmiselt tume nahatoon, suunaga paremale',
    '👨🏿‍🦯‍➡' => 'valge kepiga mees: tume nahatoon, suunaga paremale',
    '👩🏻‍🦯‍➡' => 'valge kepiga naine: hele nahatoon, suunaga paremale',
    '👩🏼‍🦯‍➡' => 'valge kepiga naine: keskmiselt hele nahatoon, suunaga paremale',
    '👩🏽‍🦯‍➡' => 'valge kepiga naine: keskmine nahatoon, suunaga paremale',
    '👩🏾‍🦯‍➡' => 'valge kepiga naine: keskmiselt tume nahatoon, suunaga paremale',
    '👩🏿‍🦯‍➡' => 'valge kepiga naine: tume nahatoon, suunaga paremale',
    '🧑🏻‍🦼‍➡' => 'elektrilises ratastoolis inimene: hele nahatoon, suunaga paremale',
    '🧑🏼‍🦼‍➡' => 'elektrilises ratastoolis inimene: keskmiselt hele nahatoon, suunaga paremale',
    '🧑🏽‍🦼‍➡' => 'elektrilises ratastoolis inimene: keskmine nahatoon, suunaga paremale',
    '🧑🏾‍🦼‍➡' => 'elektrilises ratastoolis inimene: keskmiselt tume nahatoon, suunaga paremale',
    '🧑🏿‍🦼‍➡' => 'elektrilises ratastoolis inimene: tume nahatoon, suunaga paremale',
    '👨🏻‍🦼‍➡' => 'elektriratastoolis olev mees: hele nahatoon, suunaga paremale',
    '👨🏼‍🦼‍➡' => 'elektriratastoolis olev mees: keskmiselt hele nahatoon, suunaga paremale',
    '👨🏽‍🦼‍➡' => 'elektriratastoolis olev mees: keskmine nahatoon, suunaga paremale',
    '👨🏾‍🦼‍➡' => 'elektriratastoolis olev mees: keskmiselt tume nahatoon, suunaga paremale',
    '👨🏿‍🦼‍➡' => 'elektriratastoolis olev mees: tume nahatoon, suunaga paremale',
    '👩🏻‍🦼‍➡' => 'elektriratastoolis olev naine: hele nahatoon, suunaga paremale',
    '👩🏼‍🦼‍➡' => 'elektriratastoolis olev naine: keskmiselt hele nahatoon, suunaga paremale',
    '👩🏽‍🦼‍➡' => 'elektriratastoolis olev naine: keskmine nahatoon, suunaga paremale',
    '👩🏾‍🦼‍➡' => 'elektriratastoolis olev naine: keskmiselt tume nahatoon, suunaga paremale',
    '👩🏿‍🦼‍➡' => 'elektriratastoolis olev naine: tume nahatoon, suunaga paremale',
    '🧑🏻‍🦽‍➡' => 'manuaalses ratastoolis inimene: hele nahatoon, suunaga paremale',
    '🧑🏼‍🦽‍➡' => 'manuaalses ratastoolis inimene: keskmiselt hele nahatoon, suunaga paremale',
    '🧑🏽‍🦽‍➡' => 'manuaalses ratastoolis inimene: keskmine nahatoon, suunaga paremale',
    '🧑🏾‍🦽‍➡' => 'manuaalses ratastoolis inimene: keskmiselt tume nahatoon, suunaga paremale',
    '🧑🏿‍🦽‍➡' => 'manuaalses ratastoolis inimene: tume nahatoon, suunaga paremale',
    '👨🏻‍🦽‍➡' => 'manuaalratastoolis olev mees: hele nahatoon, suunaga paremale',
    '👨🏼‍🦽‍➡' => 'manuaalratastoolis olev mees: keskmiselt hele nahatoon, suunaga paremale',
    '👨🏽‍🦽‍➡' => 'manuaalratastoolis olev mees: keskmine nahatoon, suunaga paremale',
    '👨🏾‍🦽‍➡' => 'manuaalratastoolis olev mees: keskmiselt tume nahatoon, suunaga paremale',
    '👨🏿‍🦽‍➡' => 'manuaalratastoolis olev mees: tume nahatoon, suunaga paremale',
    '👩🏻‍🦽‍➡' => 'manuaalratastoolis olev naine: hele nahatoon, suunaga paremale',
    '👩🏼‍🦽‍➡' => 'manuaalratastoolis olev naine: keskmiselt hele nahatoon, suunaga paremale',
    '👩🏽‍🦽‍➡' => 'manuaalratastoolis olev naine: keskmine nahatoon, suunaga paremale',
    '👩🏾‍🦽‍➡' => 'manuaalratastoolis olev naine: keskmiselt tume nahatoon, suunaga paremale',
    '👩🏿‍🦽‍➡' => 'manuaalratastoolis olev naine: tume nahatoon, suunaga paremale',
    '🏃🏻‍♀‍➡' => 'jooksev naine: hele nahatoon, suunaga paremale',
    '🏃🏼‍♀‍➡' => 'jooksev naine: keskmiselt hele nahatoon, suunaga paremale',
    '🏃🏽‍♀‍➡' => 'jooksev naine: keskmine nahatoon, suunaga paremale',
    '🏃🏾‍♀‍➡' => 'jooksev naine: keskmiselt tume nahatoon, suunaga paremale',
    '🏃🏿‍♀‍➡' => 'jooksev naine: tume nahatoon, suunaga paremale',
    '🏃🏻‍♂‍➡' => 'jooksev mees: hele nahatoon, suunaga paremale',
    '🏃🏼‍♂‍➡' => 'jooksev mees: keskmiselt hele nahatoon, suunaga paremale',
    '🏃🏽‍♂‍➡' => 'jooksev mees: keskmine nahatoon, suunaga paremale',
    '🏃🏾‍♂‍➡' => 'jooksev mees: keskmiselt tume nahatoon, suunaga paremale',
    '🏃🏿‍♂‍➡' => 'jooksev mees: tume nahatoon, suunaga paremale',
    '🫱🏻‍🫲🏼' => 'käepigistus: hele nahatoon, keskmiselt hele nahatoon',
    '🫱🏻‍🫲🏽' => 'käepigistus: hele nahatoon, keskmine nahatoon',
    '🫱🏻‍🫲🏾' => 'käepigistus: hele nahatoon, keskmiselt tume nahatoon',
    '🫱🏻‍🫲🏿' => 'käepigistus: hele nahatoon, tume nahatoon',
    '🫱🏼‍🫲🏻' => 'käepigistus: keskmiselt hele nahatoon, hele nahatoon',
    '🫱🏼‍🫲🏽' => 'käepigistus: keskmiselt hele nahatoon, keskmine nahatoon',
    '🫱🏼‍🫲🏾' => 'käepigistus: keskmiselt hele nahatoon, keskmiselt tume nahatoon',
    '🫱🏼‍🫲🏿' => 'käepigistus: keskmiselt hele nahatoon, tume nahatoon',
    '🫱🏽‍🫲🏻' => 'käepigistus: keskmine nahatoon, hele nahatoon',
    '🫱🏽‍🫲🏼' => 'käepigistus: keskmine nahatoon, keskmiselt hele nahatoon',
    '🫱🏽‍🫲🏾' => 'käepigistus: keskmine nahatoon, keskmiselt tume nahatoon',
    '🫱🏽‍🫲🏿' => 'käepigistus: keskmine nahatoon, tume nahatoon',
    '🫱🏾‍🫲🏻' => 'käepigistus: keskmiselt tume nahatoon, hele nahatoon',
    '🫱🏾‍🫲🏼' => 'käepigistus: keskmiselt tume nahatoon, keskmiselt hele nahatoon',
    '🫱🏾‍🫲🏽' => 'käepigistus: keskmiselt tume nahatoon, keskmine nahatoon',
    '🫱🏾‍🫲🏿' => 'käepigistus: keskmiselt tume nahatoon, tume nahatoon',
    '🫱🏿‍🫲🏻' => 'käepigistus: tume nahatoon, hele nahatoon',
    '🫱🏿‍🫲🏼' => 'käepigistus: tume nahatoon, keskmiselt hele nahatoon',
    '🫱🏿‍🫲🏽' => 'käepigistus: tume nahatoon, keskmine nahatoon',
    '🫱🏿‍🫲🏾' => 'käepigistus: tume nahatoon, keskmiselt tume nahatoon',
    '🚶‍♀‍➡' => 'kõndiv naine: suunaga paremale',
    '🚶‍♂‍➡' => 'kõndiv mees: suunaga paremale',
    '🧎‍♀‍➡' => 'põlvitav naine: suunaga paremale',
    '🧎‍♂‍➡' => 'põlvitav mees: suunaga paremale',
    '🧑‍🦯‍➡' => 'valge kepiga inimene: suunaga paremale',
    '👨‍🦯‍➡' => 'valge kepiga mees: suunaga paremale',
    '👩‍🦯‍➡' => 'valge kepiga naine: suunaga paremale',
    '🧑‍🦼‍➡' => 'elektrilises ratastoolis inimene: suunaga paremale',
    '👨‍🦼‍➡' => 'elektriratastoolis olev mees: suunaga paremale',
    '👩‍🦼‍➡' => 'elektriratastoolis olev naine: suunaga paremale',
    '🧑‍🦽‍➡' => 'manuaalses ratastoolis inimene: suunaga paremale',
    '👨‍🦽‍➡' => 'manuaalratastoolis olev mees: suunaga paremale',
    '👩‍🦽‍➡' => 'manuaalratastoolis olev naine: suunaga paremale',
    '🏃‍♀‍➡' => 'jooksev naine: suunaga paremale',
    '🏃‍♂‍➡' => 'jooksev mees: suunaga paremale',
    '👩‍❤‍👨' => 'paarike südamega: naine, mees',
    '👨‍❤‍👨' => 'paarike südamega: mees, mees',
    '👩‍❤‍👩' => 'paarike südamega: naine, naine',
    '👨‍👩‍👦' => 'perekond: mees, naine, poiss',
    '👨‍👩‍👧' => 'perekond: mees, naine, tüdruk',
    '👨‍👨‍👦' => 'perekond: mees, mees, poiss',
    '👨‍👨‍👧' => 'perekond: mees, mees, tüdruk',
    '👩‍👩‍👦' => 'perekond: naine, naine, poiss',
    '👩‍👩‍👧' => 'perekond: naine, naine, tüdruk',
    '👨‍👦‍👦' => 'perekond: mees, poiss, poiss',
    '👨‍👧‍👦' => 'perekond: mees, tüdruk, poiss',
    '👨‍👧‍👧' => 'perekond: mees, tüdruk, tüdruk',
    '👩‍👦‍👦' => 'perekond: naine, poiss, poiss',
    '👩‍👧‍👦' => 'perekond: naine, tüdruk, poiss',
    '👩‍👧‍👧' => 'perekond: naine, tüdruk, tüdruk',
    '🧑‍🤝‍🧑' => 'inimesed hoiavad käest kinni',
    '🧑‍🧑‍🧒' => 'perekond: täiskasvanu, täiskasvanu, laps',
    '🧑‍🧒‍🧒' => 'perekond: täiskasvanu, laps, laps',
    '🧑🏻‍🦰' => 'täiskasvanu: hele nahatoon, punased juuksed',
    '🧑🏼‍🦰' => 'täiskasvanu: keskmiselt hele nahatoon, punased juuksed',
    '🧑🏽‍🦰' => 'täiskasvanu: keskmine nahatoon, punased juuksed',
    '🧑🏾‍🦰' => 'täiskasvanu: keskmiselt tume nahatoon, punased juuksed',
    '🧑🏿‍🦰' => 'täiskasvanu: tume nahatoon, punased juuksed',
    '🧑🏻‍🦱' => 'täiskasvanu: hele nahatoon, lokkis juuksed',
    '🧑🏼‍🦱' => 'täiskasvanu: keskmiselt hele nahatoon, lokkis juuksed',
    '🧑🏽‍🦱' => 'täiskasvanu: keskmine nahatoon, lokkis juuksed',
    '🧑🏾‍🦱' => 'täiskasvanu: keskmiselt tume nahatoon, lokkis juuksed',
    '🧑🏿‍🦱' => 'täiskasvanu: tume nahatoon, lokkis juuksed',
    '🧑🏻‍🦳' => 'täiskasvanu: hele nahatoon, heledad juuksed',
    '🧑🏼‍🦳' => 'täiskasvanu: keskmiselt hele nahatoon, heledad juuksed',
    '🧑🏽‍🦳' => 'täiskasvanu: keskmine nahatoon, heledad juuksed',
    '🧑🏾‍🦳' => 'täiskasvanu: keskmiselt tume nahatoon, heledad juuksed',
    '🧑🏿‍🦳' => 'täiskasvanu: tume nahatoon, heledad juuksed',
    '🧑🏻‍🦲' => 'täiskasvanu: hele nahatoon, kiilaspea',
    '🧑🏼‍🦲' => 'täiskasvanu: keskmiselt hele nahatoon, kiilaspea',
    '🧑🏽‍🦲' => 'täiskasvanu: keskmine nahatoon, kiilaspea',
    '🧑🏾‍🦲' => 'täiskasvanu: keskmiselt tume nahatoon, kiilaspea',
    '🧑🏿‍🦲' => 'täiskasvanu: tume nahatoon, kiilaspea',
    '🧑🏻‍🩰' => 'balletitantsija: hele nahatoon',
    '🧑🏼‍🩰' => 'balletitantsija: keskmiselt hele nahatoon',
    '🧑🏽‍🩰' => 'balletitantsija: keskmine nahatoon',
    '🧑🏾‍🩰' => 'balletitantsija: keskmiselt tume nahatoon',
    '🧑🏿‍🩰' => 'balletitantsija: tume nahatoon',
    '🧔🏻‍♂' => 'habemega mees: hele nahatoon',
    '🧔🏼‍♂' => 'habemega mees: keskmiselt hele nahatoon',
    '🧔🏽‍♂' => 'habemega mees: keskmine nahatoon',
    '🧔🏾‍♂' => 'habemega mees: keskmiselt tume nahatoon',
    '🧔🏿‍♂' => 'habemega mees: tume nahatoon',
    '👱🏻‍♂' => 'mees: hele nahatoon, blond',
    '👱🏼‍♂' => 'mees: keskmiselt hele nahatoon, blond',
    '👱🏽‍♂' => 'mees: keskmine nahatoon, blond',
    '👱🏾‍♂' => 'mees: keskmiselt tume nahatoon, blond',
    '👱🏿‍♂' => 'mees: tume nahatoon, blond',
    '👨🏻‍🦰' => 'mees: hele nahatoon, punased juuksed',
    '👨🏼‍🦰' => 'mees: keskmiselt hele nahatoon, punased juuksed',
    '👨🏽‍🦰' => 'mees: keskmine nahatoon, punased juuksed',
    '👨🏾‍🦰' => 'mees: keskmiselt tume nahatoon, punased juuksed',
    '👨🏿‍🦰' => 'mees: tume nahatoon, punased juuksed',
    '👨🏻‍🦱' => 'mees: hele nahatoon, lokkis juuksed',
    '👨🏼‍🦱' => 'mees: keskmiselt hele nahatoon, lokkis juuksed',
    '👨🏽‍🦱' => 'mees: keskmine nahatoon, lokkis juuksed',
    '👨🏾‍🦱' => 'mees: keskmiselt tume nahatoon, lokkis juuksed',
    '👨🏿‍🦱' => 'mees: tume nahatoon, lokkis juuksed',
    '👨🏻‍🦳' => 'mees: hele nahatoon, heledad juuksed',
    '👨🏼‍🦳' => 'mees: keskmiselt hele nahatoon, heledad juuksed',
    '👨🏽‍🦳' => 'mees: keskmine nahatoon, heledad juuksed',
    '👨🏾‍🦳' => 'mees: keskmiselt tume nahatoon, heledad juuksed',
    '👨🏿‍🦳' => 'mees: tume nahatoon, heledad juuksed',
    '👨🏻‍🦲' => 'mees: hele nahatoon, kiilaspea',
    '👨🏼‍🦲' => 'mees: keskmiselt hele nahatoon, kiilaspea',
    '👨🏽‍🦲' => 'mees: keskmine nahatoon, kiilaspea',
    '👨🏾‍🦲' => 'mees: keskmiselt tume nahatoon, kiilaspea',
    '👨🏿‍🦲' => 'mees: tume nahatoon, kiilaspea',
    '🧔🏻‍♀' => 'habemega naine: hele nahatoon',
    '🧔🏼‍♀' => 'habemega naine: keskmiselt hele nahatoon',
    '🧔🏽‍♀' => 'habemega naine: keskmine nahatoon',
    '🧔🏾‍♀' => 'habemega naine: keskmiselt tume nahatoon',
    '🧔🏿‍♀' => 'habemega naine: tume nahatoon',
    '👱🏻‍♀' => 'naine: hele nahatoon, blond',
    '👱🏼‍♀' => 'naine: keskmiselt hele nahatoon, blond',
    '👱🏽‍♀' => 'naine: keskmine nahatoon, blond',
    '👱🏾‍♀' => 'naine: keskmiselt tume nahatoon, blond',
    '👱🏿‍♀' => 'naine: tume nahatoon, blond',
    '👩🏻‍🦰' => 'naine: hele nahatoon, punased juuksed',
    '👩🏼‍🦰' => 'naine: keskmiselt hele nahatoon, punased juuksed',
    '👩🏽‍🦰' => 'naine: keskmine nahatoon, punased juuksed',
    '👩🏾‍🦰' => 'naine: keskmiselt tume nahatoon, punased juuksed',
    '👩🏿‍🦰' => 'naine: tume nahatoon, punased juuksed',
    '👩🏻‍🦱' => 'naine: hele nahatoon, lokkis juuksed',
    '👩🏼‍🦱' => 'naine: keskmiselt hele nahatoon, lokkis juuksed',
    '👩🏽‍🦱' => 'naine: keskmine nahatoon, lokkis juuksed',
    '👩🏾‍🦱' => 'naine: keskmiselt tume nahatoon, lokkis juuksed',
    '👩🏿‍🦱' => 'naine: tume nahatoon, lokkis juuksed',
    '👩🏻‍🦳' => 'naine: hele nahatoon, heledad juuksed',
    '👩🏼‍🦳' => 'naine: keskmiselt hele nahatoon, heledad juuksed',
    '👩🏽‍🦳' => 'naine: keskmine nahatoon, heledad juuksed',
    '👩🏾‍🦳' => 'naine: keskmiselt tume nahatoon, heledad juuksed',
    '👩🏿‍🦳' => 'naine: tume nahatoon, heledad juuksed',
    '👩🏻‍🦲' => 'naine: hele nahatoon, kiilaspea',
    '👩🏼‍🦲' => 'naine: keskmiselt hele nahatoon, kiilaspea',
    '👩🏽‍🦲' => 'naine: keskmine nahatoon, kiilaspea',
    '👩🏾‍🦲' => 'naine: keskmiselt tume nahatoon, kiilaspea',
    '👩🏿‍🦲' => 'naine: tume nahatoon, kiilaspea',
    '🙍🏻‍♂' => 'kulmu kortsutav mees: hele nahatoon',
    '🙍🏼‍♂' => 'kulmu kortsutav mees: keskmiselt hele nahatoon',
    '🙍🏽‍♂' => 'kulmu kortsutav mees: keskmine nahatoon',
    '🙍🏾‍♂' => 'kulmu kortsutav mees: keskmiselt tume nahatoon',
    '🙍🏿‍♂' => 'kulmu kortsutav mees: tume nahatoon',
    '🙍🏻‍♀' => 'kulmu kortsutav naine: hele nahatoon',
    '🙍🏼‍♀' => 'kulmu kortsutav naine: keskmiselt hele nahatoon',
    '🙍🏽‍♀' => 'kulmu kortsutav naine: keskmine nahatoon',
    '🙍🏾‍♀' => 'kulmu kortsutav naine: keskmiselt tume nahatoon',
    '🙍🏿‍♀' => 'kulmu kortsutav naine: tume nahatoon',
    '🙎🏻‍♂' => 'mossitav mees: hele nahatoon',
    '🙎🏼‍♂' => 'mossitav mees: keskmiselt hele nahatoon',
    '🙎🏽‍♂' => 'mossitav mees: keskmine nahatoon',
    '🙎🏾‍♂' => 'mossitav mees: keskmiselt tume nahatoon',
    '🙎🏿‍♂' => 'mossitav mees: tume nahatoon',
    '🙎🏻‍♀' => 'mossitav naine: hele nahatoon',
    '🙎🏼‍♀' => 'mossitav naine: keskmiselt hele nahatoon',
    '🙎🏽‍♀' => 'mossitav naine: keskmine nahatoon',
    '🙎🏾‍♀' => 'mossitav naine: keskmiselt tume nahatoon',
    '🙎🏿‍♀' => 'mossitav naine: tume nahatoon',
    '🙅🏻‍♂' => 'eitavat žesti näitav mees: hele nahatoon',
    '🙅🏼‍♂' => 'eitavat žesti näitav mees: keskmiselt hele nahatoon',
    '🙅🏽‍♂' => 'eitavat žesti näitav mees: keskmine nahatoon',
    '🙅🏾‍♂' => 'eitavat žesti näitav mees: keskmiselt tume nahatoon',
    '🙅🏿‍♂' => 'eitavat žesti näitav mees: tume nahatoon',
    '🙅🏻‍♀' => 'eitavat žesti näitav naine: hele nahatoon',
    '🙅🏼‍♀' => 'eitavat žesti näitav naine: keskmiselt hele nahatoon',
    '🙅🏽‍♀' => 'eitavat žesti näitav naine: keskmine nahatoon',
    '🙅🏾‍♀' => 'eitavat žesti näitav naine: keskmiselt tume nahatoon',
    '🙅🏿‍♀' => 'eitavat žesti näitav naine: tume nahatoon',
    '🙆🏻‍♂' => 'nõustuvat žesti näitav mees: hele nahatoon',
    '🙆🏼‍♂' => 'nõustuvat žesti näitav mees: keskmiselt hele nahatoon',
    '🙆🏽‍♂' => 'nõustuvat žesti näitav mees: keskmine nahatoon',
    '🙆🏾‍♂' => 'nõustuvat žesti näitav mees: keskmiselt tume nahatoon',
    '🙆🏿‍♂' => 'nõustuvat žesti näitav mees: tume nahatoon',
    '🙆🏻‍♀' => 'nõustuvat žesti näitav naine: hele nahatoon',
    '🙆🏼‍♀' => 'nõustuvat žesti näitav naine: keskmiselt hele nahatoon',
    '🙆🏽‍♀' => 'nõustuvat žesti näitav naine: keskmine nahatoon',
    '🙆🏾‍♀' => 'nõustuvat žesti näitav naine: keskmiselt tume nahatoon',
    '🙆🏿‍♀' => 'nõustuvat žesti näitav naine: tume nahatoon',
    '💁🏻‍♂' => 'infoleti meestöötaja: hele nahatoon',
    '💁🏼‍♂' => 'infoleti meestöötaja: keskmiselt hele nahatoon',
    '💁🏽‍♂' => 'infoleti meestöötaja: keskmine nahatoon',
    '💁🏾‍♂' => 'infoleti meestöötaja: keskmiselt tume nahatoon',
    '💁🏿‍♂' => 'infoleti meestöötaja: tume nahatoon',
    '💁🏻‍♀' => 'infoleti naistöötaja: hele nahatoon',
    '💁🏼‍♀' => 'infoleti naistöötaja: keskmiselt hele nahatoon',
    '💁🏽‍♀' => 'infoleti naistöötaja: keskmine nahatoon',
    '💁🏾‍♀' => 'infoleti naistöötaja: keskmiselt tume nahatoon',
    '💁🏿‍♀' => 'infoleti naistöötaja: tume nahatoon',
    '🙋🏻‍♂' => 'kätt tõstev mees: hele nahatoon',
    '🙋🏼‍♂' => 'kätt tõstev mees: keskmiselt hele nahatoon',
    '🙋🏽‍♂' => 'kätt tõstev mees: keskmine nahatoon',
    '🙋🏾‍♂' => 'kätt tõstev mees: keskmiselt tume nahatoon',
    '🙋🏿‍♂' => 'kätt tõstev mees: tume nahatoon',
    '🙋🏻‍♀' => 'kätt tõstev naine: hele nahatoon',
    '🙋🏼‍♀' => 'kätt tõstev naine: keskmiselt hele nahatoon',
    '🙋🏽‍♀' => 'kätt tõstev naine: keskmine nahatoon',
    '🙋🏾‍♀' => 'kätt tõstev naine: keskmiselt tume nahatoon',
    '🙋🏿‍♀' => 'kätt tõstev naine: tume nahatoon',
    '🧏🏻‍♂' => 'kurt mees: hele nahatoon',
    '🧏🏼‍♂' => 'kurt mees: keskmiselt hele nahatoon',
    '🧏🏽‍♂' => 'kurt mees: keskmine nahatoon',
    '🧏🏾‍♂' => 'kurt mees: keskmiselt tume nahatoon',
    '🧏🏿‍♂' => 'kurt mees: tume nahatoon',
    '🧏🏻‍♀' => 'kurt naine: hele nahatoon',
    '🧏🏼‍♀' => 'kurt naine: keskmiselt hele nahatoon',
    '🧏🏽‍♀' => 'kurt naine: keskmine nahatoon',
    '🧏🏾‍♀' => 'kurt naine: keskmiselt tume nahatoon',
    '🧏🏿‍♀' => 'kurt naine: tume nahatoon',
    '🙇🏻‍♂' => 'sügavalt kummarduv mees: hele nahatoon',
    '🙇🏼‍♂' => 'sügavalt kummarduv mees: keskmiselt hele nahatoon',
    '🙇🏽‍♂' => 'sügavalt kummarduv mees: keskmine nahatoon',
    '🙇🏾‍♂' => 'sügavalt kummarduv mees: keskmiselt tume nahatoon',
    '🙇🏿‍♂' => 'sügavalt kummarduv mees: tume nahatoon',
    '🙇🏻‍♀' => 'sügavalt kummarduv naine: hele nahatoon',
    '🙇🏼‍♀' => 'sügavalt kummarduv naine: keskmiselt hele nahatoon',
    '🙇🏽‍♀' => 'sügavalt kummarduv naine: keskmine nahatoon',
    '🙇🏾‍♀' => 'sügavalt kummarduv naine: keskmiselt tume nahatoon',
    '🙇🏿‍♀' => 'sügavalt kummarduv naine: tume nahatoon',
    '🤦🏻‍♂' => 'käega vastu pead lööv mees: hele nahatoon',
    '🤦🏼‍♂' => 'käega vastu pead lööv mees: keskmiselt hele nahatoon',
    '🤦🏽‍♂' => 'käega vastu pead lööv mees: keskmine nahatoon',
    '🤦🏾‍♂' => 'käega vastu pead lööv mees: keskmiselt tume nahatoon',
    '🤦🏿‍♂' => 'käega vastu pead lööv mees: tume nahatoon',
    '🤦🏻‍♀' => 'käega vastu pead lööv naine: hele nahatoon',
    '🤦🏼‍♀' => 'käega vastu pead lööv naine: keskmiselt hele nahatoon',
    '🤦🏽‍♀' => 'käega vastu pead lööv naine: keskmine nahatoon',
    '🤦🏾‍♀' => 'käega vastu pead lööv naine: keskmiselt tume nahatoon',
    '🤦🏿‍♀' => 'käega vastu pead lööv naine: tume nahatoon',
    '🤷🏻‍♂' => 'õlgu kehitav mees: hele nahatoon',
    '🤷🏼‍♂' => 'õlgu kehitav mees: keskmiselt hele nahatoon',
    '🤷🏽‍♂' => 'õlgu kehitav mees: keskmine nahatoon',
    '🤷🏾‍♂' => 'õlgu kehitav mees: keskmiselt tume nahatoon',
    '🤷🏿‍♂' => 'õlgu kehitav mees: tume nahatoon',
    '🤷🏻‍♀' => 'õlgu kehitav naine: hele nahatoon',
    '🤷🏼‍♀' => 'õlgu kehitav naine: keskmiselt hele nahatoon',
    '🤷🏽‍♀' => 'õlgu kehitav naine: keskmine nahatoon',
    '🤷🏾‍♀' => 'õlgu kehitav naine: keskmiselt tume nahatoon',
    '🤷🏿‍♀' => 'õlgu kehitav naine: tume nahatoon',
    '🧑🏻‍⚕' => 'tervishoiutöötaja: hele nahatoon',
    '🧑🏼‍⚕' => 'tervishoiutöötaja: keskmiselt hele nahatoon',
    '🧑🏽‍⚕' => 'tervishoiutöötaja: keskmine nahatoon',
    '🧑🏾‍⚕' => 'tervishoiutöötaja: keskmiselt tume nahatoon',
    '🧑🏿‍⚕' => 'tervishoiutöötaja: tume nahatoon',
    '👨🏻‍⚕' => 'meesarst: hele nahatoon',
    '👨🏼‍⚕' => 'meesarst: keskmiselt hele nahatoon',
    '👨🏽‍⚕' => 'meesarst: keskmine nahatoon',
    '👨🏾‍⚕' => 'meesarst: keskmiselt tume nahatoon',
    '👨🏿‍⚕' => 'meesarst: tume nahatoon',
    '👩🏻‍⚕' => 'naisarst: hele nahatoon',
    '👩🏼‍⚕' => 'naisarst: keskmiselt hele nahatoon',
    '👩🏽‍⚕' => 'naisarst: keskmine nahatoon',
    '👩🏾‍⚕' => 'naisarst: keskmiselt tume nahatoon',
    '👩🏿‍⚕' => 'naisarst: tume nahatoon',
    '🧑🏻‍🎓' => 'õpilane: hele nahatoon',
    '🧑🏼‍🎓' => 'õpilane: keskmiselt hele nahatoon',
    '🧑🏽‍🎓' => 'õpilane: keskmine nahatoon',
    '🧑🏾‍🎓' => 'õpilane: keskmiselt tume nahatoon',
    '🧑🏿‍🎓' => 'õpilane: tume nahatoon',
    '👨🏻‍🎓' => 'meesõpilane: hele nahatoon',
    '👨🏼‍🎓' => 'meesõpilane: keskmiselt hele nahatoon',
    '👨🏽‍🎓' => 'meesõpilane: keskmine nahatoon',
    '👨🏾‍🎓' => 'meesõpilane: keskmiselt tume nahatoon',
    '👨🏿‍🎓' => 'meesõpilane: tume nahatoon',
    '👩🏻‍🎓' => 'naisõpilane: hele nahatoon',
    '👩🏼‍🎓' => 'naisõpilane: keskmiselt hele nahatoon',
    '👩🏽‍🎓' => 'naisõpilane: keskmine nahatoon',
    '👩🏾‍🎓' => 'naisõpilane: keskmiselt tume nahatoon',
    '👩🏿‍🎓' => 'naisõpilane: tume nahatoon',
    '🧑🏻‍🏫' => 'õpetaja: hele nahatoon',
    '🧑🏼‍🏫' => 'õpetaja: keskmiselt hele nahatoon',
    '🧑🏽‍🏫' => 'õpetaja: keskmine nahatoon',
    '🧑🏾‍🏫' => 'õpetaja: keskmiselt tume nahatoon',
    '🧑🏿‍🏫' => 'õpetaja: tume nahatoon',
    '👨🏻‍🏫' => 'meesõpetaja: hele nahatoon',
    '👨🏼‍🏫' => 'meesõpetaja: keskmiselt hele nahatoon',
    '👨🏽‍🏫' => 'meesõpetaja: keskmine nahatoon',
    '👨🏾‍🏫' => 'meesõpetaja: keskmiselt tume nahatoon',
    '👨🏿‍🏫' => 'meesõpetaja: tume nahatoon',
    '👩🏻‍🏫' => 'naisõpetaja: hele nahatoon',
    '👩🏼‍🏫' => 'naisõpetaja: keskmiselt hele nahatoon',
    '👩🏽‍🏫' => 'naisõpetaja: keskmine nahatoon',
    '👩🏾‍🏫' => 'naisõpetaja: keskmiselt tume nahatoon',
    '👩🏿‍🏫' => 'naisõpetaja: tume nahatoon',
    '🧑🏻‍⚖' => 'kohtunik: hele nahatoon',
    '🧑🏼‍⚖' => 'kohtunik: keskmiselt hele nahatoon',
    '🧑🏽‍⚖' => 'kohtunik: keskmine nahatoon',
    '🧑🏾‍⚖' => 'kohtunik: keskmiselt tume nahatoon',
    '🧑🏿‍⚖' => 'kohtunik: tume nahatoon',
    '👨🏻‍⚖' => 'meeskohtunik: hele nahatoon',
    '👨🏼‍⚖' => 'meeskohtunik: keskmiselt hele nahatoon',
    '👨🏽‍⚖' => 'meeskohtunik: keskmine nahatoon',
    '👨🏾‍⚖' => 'meeskohtunik: keskmiselt tume nahatoon',
    '👨🏿‍⚖' => 'meeskohtunik: tume nahatoon',
    '👩🏻‍⚖' => 'naiskohtunik: hele nahatoon',
    '👩🏼‍⚖' => 'naiskohtunik: keskmiselt hele nahatoon',
    '👩🏽‍⚖' => 'naiskohtunik: keskmine nahatoon',
    '👩🏾‍⚖' => 'naiskohtunik: keskmiselt tume nahatoon',
    '👩🏿‍⚖' => 'naiskohtunik: tume nahatoon',
    '🧑🏻‍🌾' => 'talunik: hele nahatoon',
    '🧑🏼‍🌾' => 'talunik: keskmiselt hele nahatoon',
    '🧑🏽‍🌾' => 'talunik: keskmine nahatoon',
    '🧑🏾‍🌾' => 'talunik: keskmiselt tume nahatoon',
    '🧑🏿‍🌾' => 'talunik: tume nahatoon',
    '👨🏻‍🌾' => 'meesfarmer: hele nahatoon',
    '👨🏼‍🌾' => 'meesfarmer: keskmiselt hele nahatoon',
    '👨🏽‍🌾' => 'meesfarmer: keskmine nahatoon',
    '👨🏾‍🌾' => 'meesfarmer: keskmiselt tume nahatoon',
    '👨🏿‍🌾' => 'meesfarmer: tume nahatoon',
    '👩🏻‍🌾' => 'naisfarmer: hele nahatoon',
    '👩🏼‍🌾' => 'naisfarmer: keskmiselt hele nahatoon',
    '👩🏽‍🌾' => 'naisfarmer: keskmine nahatoon',
    '👩🏾‍🌾' => 'naisfarmer: keskmiselt tume nahatoon',
    '👩🏿‍🌾' => 'naisfarmer: tume nahatoon',
    '🧑🏻‍🍳' => 'kokk: hele nahatoon',
    '🧑🏼‍🍳' => 'kokk: keskmiselt hele nahatoon',
    '🧑🏽‍🍳' => 'kokk: keskmine nahatoon',
    '🧑🏾‍🍳' => 'kokk: keskmiselt tume nahatoon',
    '🧑🏿‍🍳' => 'kokk: tume nahatoon',
    '👨🏻‍🍳' => 'meeskokk: hele nahatoon',
    '👨🏼‍🍳' => 'meeskokk: keskmiselt hele nahatoon',
    '👨🏽‍🍳' => 'meeskokk: keskmine nahatoon',
    '👨🏾‍🍳' => 'meeskokk: keskmiselt tume nahatoon',
    '👨🏿‍🍳' => 'meeskokk: tume nahatoon',
    '👩🏻‍🍳' => 'naiskokk: hele nahatoon',
    '👩🏼‍🍳' => 'naiskokk: keskmiselt hele nahatoon',
    '👩🏽‍🍳' => 'naiskokk: keskmine nahatoon',
    '👩🏾‍🍳' => 'naiskokk: keskmiselt tume nahatoon',
    '👩🏿‍🍳' => 'naiskokk: tume nahatoon',
    '🧑🏻‍🔧' => 'mehaanik: hele nahatoon',
    '🧑🏼‍🔧' => 'mehaanik: keskmiselt hele nahatoon',
    '🧑🏽‍🔧' => 'mehaanik: keskmine nahatoon',
    '🧑🏾‍🔧' => 'mehaanik: keskmiselt tume nahatoon',
    '🧑🏿‍🔧' => 'mehaanik: tume nahatoon',
    '👨🏻‍🔧' => 'meesmehaanik: hele nahatoon',
    '👨🏼‍🔧' => 'meesmehaanik: keskmiselt hele nahatoon',
    '👨🏽‍🔧' => 'meesmehaanik: keskmine nahatoon',
    '👨🏾‍🔧' => 'meesmehaanik: keskmiselt tume nahatoon',
    '👨🏿‍🔧' => 'meesmehaanik: tume nahatoon',
    '👩🏻‍🔧' => 'naismehaanik: hele nahatoon',
    '👩🏼‍🔧' => 'naismehaanik: keskmiselt hele nahatoon',
    '👩🏽‍🔧' => 'naismehaanik: keskmine nahatoon',
    '👩🏾‍🔧' => 'naismehaanik: keskmiselt tume nahatoon',
    '👩🏿‍🔧' => 'naismehaanik: tume nahatoon',
    '🧑🏻‍🏭' => 'tehasetööline: hele nahatoon',
    '🧑🏼‍🏭' => 'tehasetööline: keskmiselt hele nahatoon',
    '🧑🏽‍🏭' => 'tehasetööline: keskmine nahatoon',
    '🧑🏾‍🏭' => 'tehasetööline: keskmiselt tume nahatoon',
    '🧑🏿‍🏭' => 'tehasetööline: tume nahatoon',
    '👨🏻‍🏭' => 'meessoost tehasetööline: hele nahatoon',
    '👨🏼‍🏭' => 'meessoost tehasetööline: keskmiselt hele nahatoon',
    '👨🏽‍🏭' => 'meessoost tehasetööline: keskmine nahatoon',
    '👨🏾‍🏭' => 'meessoost tehasetööline: keskmiselt tume nahatoon',
    '👨🏿‍🏭' => 'meessoost tehasetööline: tume nahatoon',
    '👩🏻‍🏭' => 'naissoost tehasetööline: hele nahatoon',
    '👩🏼‍🏭' => 'naissoost tehasetööline: keskmiselt hele nahatoon',
    '👩🏽‍🏭' => 'naissoost tehasetööline: keskmine nahatoon',
    '👩🏾‍🏭' => 'naissoost tehasetööline: keskmiselt tume nahatoon',
    '👩🏿‍🏭' => 'naissoost tehasetööline: tume nahatoon',
    '🧑🏻‍💼' => 'kontoritöötaja: hele nahatoon',
    '🧑🏼‍💼' => 'kontoritöötaja: keskmiselt hele nahatoon',
    '🧑🏽‍💼' => 'kontoritöötaja: keskmine nahatoon',
    '🧑🏾‍💼' => 'kontoritöötaja: keskmiselt tume nahatoon',
    '🧑🏿‍💼' => 'kontoritöötaja: tume nahatoon',
    '👨🏻‍💼' => 'meeskontoritöötaja: hele nahatoon',
    '👨🏼‍💼' => 'meeskontoritöötaja: keskmiselt hele nahatoon',
    '👨🏽‍💼' => 'meeskontoritöötaja: keskmine nahatoon',
    '👨🏾‍💼' => 'meeskontoritöötaja: keskmiselt tume nahatoon',
    '👨🏿‍💼' => 'meeskontoritöötaja: tume nahatoon',
    '👩🏻‍💼' => 'naiskontoritöötaja: hele nahatoon',
    '👩🏼‍💼' => 'naiskontoritöötaja: keskmiselt hele nahatoon',
    '👩🏽‍💼' => 'naiskontoritöötaja: keskmine nahatoon',
    '👩🏾‍💼' => 'naiskontoritöötaja: keskmiselt tume nahatoon',
    '👩🏿‍💼' => 'naiskontoritöötaja: tume nahatoon',
    '🧑🏻‍🔬' => 'teadlane: hele nahatoon',
    '🧑🏼‍🔬' => 'teadlane: keskmiselt hele nahatoon',
    '🧑🏽‍🔬' => 'teadlane: keskmine nahatoon',
    '🧑🏾‍🔬' => 'teadlane: keskmiselt tume nahatoon',
    '🧑🏿‍🔬' => 'teadlane: tume nahatoon',
    '👨🏻‍🔬' => 'meesteadlane: hele nahatoon',
    '👨🏼‍🔬' => 'meesteadlane: keskmiselt hele nahatoon',
    '👨🏽‍🔬' => 'meesteadlane: keskmine nahatoon',
    '👨🏾‍🔬' => 'meesteadlane: keskmiselt tume nahatoon',
    '👨🏿‍🔬' => 'meesteadlane: tume nahatoon',
    '👩🏻‍🔬' => 'naisteadlane: hele nahatoon',
    '👩🏼‍🔬' => 'naisteadlane: keskmiselt hele nahatoon',
    '👩🏽‍🔬' => 'naisteadlane: keskmine nahatoon',
    '👩🏾‍🔬' => 'naisteadlane: keskmiselt tume nahatoon',
    '👩🏿‍🔬' => 'naisteadlane: tume nahatoon',
    '🧑🏻‍💻' => 'programmeerija: hele nahatoon',
    '🧑🏼‍💻' => 'programmeerija: keskmiselt hele nahatoon',
    '🧑🏽‍💻' => 'programmeerija: keskmine nahatoon',
    '🧑🏾‍💻' => 'programmeerija: keskmiselt tume nahatoon',
    '🧑🏿‍💻' => 'programmeerija: tume nahatoon',
    '👨🏻‍💻' => 'meesprogrammeerija: hele nahatoon',
    '👨🏼‍💻' => 'meesprogrammeerija: keskmiselt hele nahatoon',
    '👨🏽‍💻' => 'meesprogrammeerija: keskmine nahatoon',
    '👨🏾‍💻' => 'meesprogrammeerija: keskmiselt tume nahatoon',
    '👨🏿‍💻' => 'meesprogrammeerija: tume nahatoon',
    '👩🏻‍💻' => 'naisprogrammeerija: hele nahatoon',
    '👩🏼‍💻' => 'naisprogrammeerija: keskmiselt hele nahatoon',
    '👩🏽‍💻' => 'naisprogrammeerija: keskmine nahatoon',
    '👩🏾‍💻' => 'naisprogrammeerija: keskmiselt tume nahatoon',
    '👩🏿‍💻' => 'naisprogrammeerija: tume nahatoon',
    '🧑🏻‍🎤' => 'laulja: hele nahatoon',
    '🧑🏼‍🎤' => 'laulja: keskmiselt hele nahatoon',
    '🧑🏽‍🎤' => 'laulja: keskmine nahatoon',
    '🧑🏾‍🎤' => 'laulja: keskmiselt tume nahatoon',
    '🧑🏿‍🎤' => 'laulja: tume nahatoon',
    '👨🏻‍🎤' => 'meeslaulja: hele nahatoon',
    '👨🏼‍🎤' => 'meeslaulja: keskmiselt hele nahatoon',
    '👨🏽‍🎤' => 'meeslaulja: keskmine nahatoon',
    '👨🏾‍🎤' => 'meeslaulja: keskmiselt tume nahatoon',
    '👨🏿‍🎤' => 'meeslaulja: tume nahatoon',
    '👩🏻‍🎤' => 'naislaulja: hele nahatoon',
    '👩🏼‍🎤' => 'naislaulja: keskmiselt hele nahatoon',
    '👩🏽‍🎤' => 'naislaulja: keskmine nahatoon',
    '👩🏾‍🎤' => 'naislaulja: keskmiselt tume nahatoon',
    '👩🏿‍🎤' => 'naislaulja: tume nahatoon',
    '🧑🏻‍🎨' => 'kunstnik: hele nahatoon',
    '🧑🏼‍🎨' => 'kunstnik: keskmiselt hele nahatoon',
    '🧑🏽‍🎨' => 'kunstnik: keskmine nahatoon',
    '🧑🏾‍🎨' => 'kunstnik: keskmiselt tume nahatoon',
    '🧑🏿‍🎨' => 'kunstnik: tume nahatoon',
    '👨🏻‍🎨' => 'meeskunstnik: hele nahatoon',
    '👨🏼‍🎨' => 'meeskunstnik: keskmiselt hele nahatoon',
    '👨🏽‍🎨' => 'meeskunstnik: keskmine nahatoon',
    '👨🏾‍🎨' => 'meeskunstnik: keskmiselt tume nahatoon',
    '👨🏿‍🎨' => 'meeskunstnik: tume nahatoon',
    '👩🏻‍🎨' => 'naiskunstnik: hele nahatoon',
    '👩🏼‍🎨' => 'naiskunstnik: keskmiselt hele nahatoon',
    '👩🏽‍🎨' => 'naiskunstnik: keskmine nahatoon',
    '👩🏾‍🎨' => 'naiskunstnik: keskmiselt tume nahatoon',
    '👩🏿‍🎨' => 'naiskunstnik: tume nahatoon',
    '🧑🏻‍✈' => 'piloot: hele nahatoon',
    '🧑🏼‍✈' => 'piloot: keskmiselt hele nahatoon',
    '🧑🏽‍✈' => 'piloot: keskmine nahatoon',
    '🧑🏾‍✈' => 'piloot: keskmiselt tume nahatoon',
    '🧑🏿‍✈' => 'piloot: tume nahatoon',
    '👨🏻‍✈' => 'meespiloot: hele nahatoon',
    '👨🏼‍✈' => 'meespiloot: keskmiselt hele nahatoon',
    '👨🏽‍✈' => 'meespiloot: keskmine nahatoon',
    '👨🏾‍✈' => 'meespiloot: keskmiselt tume nahatoon',
    '👨🏿‍✈' => 'meespiloot: tume nahatoon',
    '👩🏻‍✈' => 'naispiloot: hele nahatoon',
    '👩🏼‍✈' => 'naispiloot: keskmiselt hele nahatoon',
    '👩🏽‍✈' => 'naispiloot: keskmine nahatoon',
    '👩🏾‍✈' => 'naispiloot: keskmiselt tume nahatoon',
    '👩🏿‍✈' => 'naispiloot: tume nahatoon',
    '🧑🏻‍🚀' => 'astronaut: hele nahatoon',
    '🧑🏼‍🚀' => 'astronaut: keskmiselt hele nahatoon',
    '🧑🏽‍🚀' => 'astronaut: keskmine nahatoon',
    '🧑🏾‍🚀' => 'astronaut: keskmiselt tume nahatoon',
    '🧑🏿‍🚀' => 'astronaut: tume nahatoon',
    '👨🏻‍🚀' => 'meesastronaut: hele nahatoon',
    '👨🏼‍🚀' => 'meesastronaut: keskmiselt hele nahatoon',
    '👨🏽‍🚀' => 'meesastronaut: keskmine nahatoon',
    '👨🏾‍🚀' => 'meesastronaut: keskmiselt tume nahatoon',
    '👨🏿‍🚀' => 'meesastronaut: tume nahatoon',
    '👩🏻‍🚀' => 'naisastronaut: hele nahatoon',
    '👩🏼‍🚀' => 'naisastronaut: keskmiselt hele nahatoon',
    '👩🏽‍🚀' => 'naisastronaut: keskmine nahatoon',
    '👩🏾‍🚀' => 'naisastronaut: keskmiselt tume nahatoon',
    '👩🏿‍🚀' => 'naisastronaut: tume nahatoon',
    '🧑🏻‍🚒' => 'tuletõrjuja: hele nahatoon',
    '🧑🏼‍🚒' => 'tuletõrjuja: keskmiselt hele nahatoon',
    '🧑🏽‍🚒' => 'tuletõrjuja: keskmine nahatoon',
    '🧑🏾‍🚒' => 'tuletõrjuja: keskmiselt tume nahatoon',
    '🧑🏿‍🚒' => 'tuletõrjuja: tume nahatoon',
    '👨🏻‍🚒' => 'meestuletõrjuja: hele nahatoon',
    '👨🏼‍🚒' => 'meestuletõrjuja: keskmiselt hele nahatoon',
    '👨🏽‍🚒' => 'meestuletõrjuja: keskmine nahatoon',
    '👨🏾‍🚒' => 'meestuletõrjuja: keskmiselt tume nahatoon',
    '👨🏿‍🚒' => 'meestuletõrjuja: tume nahatoon',
    '👩🏻‍🚒' => 'naistuletõrjuja: hele nahatoon',
    '👩🏼‍🚒' => 'naistuletõrjuja: keskmiselt hele nahatoon',
    '👩🏽‍🚒' => 'naistuletõrjuja: keskmine nahatoon',
    '👩🏾‍🚒' => 'naistuletõrjuja: keskmiselt tume nahatoon',
    '👩🏿‍🚒' => 'naistuletõrjuja: tume nahatoon',
    '👮🏻‍♂' => 'meespolitseinik: hele nahatoon',
    '👮🏼‍♂' => 'meespolitseinik: keskmiselt hele nahatoon',
    '👮🏽‍♂' => 'meespolitseinik: keskmine nahatoon',
    '👮🏾‍♂' => 'meespolitseinik: keskmiselt tume nahatoon',
    '👮🏿‍♂' => 'meespolitseinik: tume nahatoon',
    '👮🏻‍♀' => 'naispolitseinik: hele nahatoon',
    '👮🏼‍♀' => 'naispolitseinik: keskmiselt hele nahatoon',
    '👮🏽‍♀' => 'naispolitseinik: keskmine nahatoon',
    '👮🏾‍♀' => 'naispolitseinik: keskmiselt tume nahatoon',
    '👮🏿‍♀' => 'naispolitseinik: tume nahatoon',
    '🕵🏻‍♂' => 'meesdetektiiv: hele nahatoon',
    '🕵🏼‍♂' => 'meesdetektiiv: keskmiselt hele nahatoon',
    '🕵🏽‍♂' => 'meesdetektiiv: keskmine nahatoon',
    '🕵🏾‍♂' => 'meesdetektiiv: keskmiselt tume nahatoon',
    '🕵🏿‍♂' => 'meesdetektiiv: tume nahatoon',
    '🕵🏻‍♀' => 'naisdetektiiv: hele nahatoon',
    '🕵🏼‍♀' => 'naisdetektiiv: keskmiselt hele nahatoon',
    '🕵🏽‍♀' => 'naisdetektiiv: keskmine nahatoon',
    '🕵🏾‍♀' => 'naisdetektiiv: keskmiselt tume nahatoon',
    '🕵🏿‍♀' => 'naisdetektiiv: tume nahatoon',
    '💂🏻‍♂' => 'meesvalvur: hele nahatoon',
    '💂🏼‍♂' => 'meesvalvur: keskmiselt hele nahatoon',
    '💂🏽‍♂' => 'meesvalvur: keskmine nahatoon',
    '💂🏾‍♂' => 'meesvalvur: keskmiselt tume nahatoon',
    '💂🏿‍♂' => 'meesvalvur: tume nahatoon',
    '💂🏻‍♀' => 'naisvalvur: hele nahatoon',
    '💂🏼‍♀' => 'naisvalvur: keskmiselt hele nahatoon',
    '💂🏽‍♀' => 'naisvalvur: keskmine nahatoon',
    '💂🏾‍♀' => 'naisvalvur: keskmiselt tume nahatoon',
    '💂🏿‍♀' => 'naisvalvur: tume nahatoon',
    '👷🏻‍♂' => 'meesehitaja: hele nahatoon',
    '👷🏼‍♂' => 'meesehitaja: keskmiselt hele nahatoon',
    '👷🏽‍♂' => 'meesehitaja: keskmine nahatoon',
    '👷🏾‍♂' => 'meesehitaja: keskmiselt tume nahatoon',
    '👷🏿‍♂' => 'meesehitaja: tume nahatoon',
    '👷🏻‍♀' => 'naisehitaja: hele nahatoon',
    '👷🏼‍♀' => 'naisehitaja: keskmiselt hele nahatoon',
    '👷🏽‍♀' => 'naisehitaja: keskmine nahatoon',
    '👷🏾‍♀' => 'naisehitaja: keskmiselt tume nahatoon',
    '👷🏿‍♀' => 'naisehitaja: tume nahatoon',
    '👳🏻‍♂' => 'mees turbaniga: hele nahatoon',
    '👳🏼‍♂' => 'mees turbaniga: keskmiselt hele nahatoon',
    '👳🏽‍♂' => 'mees turbaniga: keskmine nahatoon',
    '👳🏾‍♂' => 'mees turbaniga: keskmiselt tume nahatoon',
    '👳🏿‍♂' => 'mees turbaniga: tume nahatoon',
    '👳🏻‍♀' => 'naine turbaninga: hele nahatoon',
    '👳🏼‍♀' => 'naine turbaninga: keskmiselt hele nahatoon',
    '👳🏽‍♀' => 'naine turbaninga: keskmine nahatoon',
    '👳🏾‍♀' => 'naine turbaninga: keskmiselt tume nahatoon',
    '👳🏿‍♀' => 'naine turbaninga: tume nahatoon',
    '🤵🏻‍♂' => 'mees smokingis: hele nahatoon',
    '🤵🏼‍♂' => 'mees smokingis: keskmiselt hele nahatoon',
    '🤵🏽‍♂' => 'mees smokingis: keskmine nahatoon',
    '🤵🏾‍♂' => 'mees smokingis: keskmiselt tume nahatoon',
    '🤵🏿‍♂' => 'mees smokingis: tume nahatoon',
    '🤵🏻‍♀' => 'naine smokingus: hele nahatoon',
    '🤵🏼‍♀' => 'naine smokingus: keskmiselt hele nahatoon',
    '🤵🏽‍♀' => 'naine smokingus: keskmine nahatoon',
    '🤵🏾‍♀' => 'naine smokingus: keskmiselt tume nahatoon',
    '🤵🏿‍♀' => 'naine smokingus: tume nahatoon',
    '👰🏻‍♂' => 'mees looriga: hele nahatoon',
    '👰🏼‍♂' => 'mees looriga: keskmiselt hele nahatoon',
    '👰🏽‍♂' => 'mees looriga: keskmine nahatoon',
    '👰🏾‍♂' => 'mees looriga: keskmiselt tume nahatoon',
    '👰🏿‍♂' => 'mees looriga: tume nahatoon',
    '👰🏻‍♀' => 'naine looriga: hele nahatoon',
    '👰🏼‍♀' => 'naine looriga: keskmiselt hele nahatoon',
    '👰🏽‍♀' => 'naine looriga: keskmine nahatoon',
    '👰🏾‍♀' => 'naine looriga: keskmiselt tume nahatoon',
    '👰🏿‍♀' => 'naine looriga: tume nahatoon',
    '👩🏻‍🍼' => 'beebit toitev naine: hele nahatoon',
    '👩🏼‍🍼' => 'beebit toitev naine: keskmiselt hele nahatoon',
    '👩🏽‍🍼' => 'beebit toitev naine: keskmine nahatoon',
    '👩🏾‍🍼' => 'beebit toitev naine: keskmiselt tume nahatoon',
    '👩🏿‍🍼' => 'beebit toitev naine: tume nahatoon',
    '👨🏻‍🍼' => 'beebit toitev mees: hele nahatoon',
    '👨🏼‍🍼' => 'beebit toitev mees: keskmiselt hele nahatoon',
    '👨🏽‍🍼' => 'beebit toitev mees: keskmine nahatoon',
    '👨🏾‍🍼' => 'beebit toitev mees: keskmiselt tume nahatoon',
    '👨🏿‍🍼' => 'beebit toitev mees: tume nahatoon',
    '🧑🏻‍🍼' => 'beebit toitev inimene: hele nahatoon',
    '🧑🏼‍🍼' => 'beebit toitev inimene: keskmiselt hele nahatoon',
    '🧑🏽‍🍼' => 'beebit toitev inimene: keskmine nahatoon',
    '🧑🏾‍🍼' => 'beebit toitev inimene: keskmiselt tume nahatoon',
    '🧑🏿‍🍼' => 'beebit toitev inimene: tume nahatoon',
    '🧑🏻‍🎄' => 'päkats: hele nahatoon',
    '🧑🏼‍🎄' => 'päkats: keskmiselt hele nahatoon',
    '🧑🏽‍🎄' => 'päkats: keskmine nahatoon',
    '🧑🏾‍🎄' => 'päkats: keskmiselt tume nahatoon',
    '🧑🏿‍🎄' => 'päkats: tume nahatoon',
    '🦸🏻‍♂' => 'meeskangelane: hele nahatoon',
    '🦸🏼‍♂' => 'meeskangelane: keskmiselt hele nahatoon',
    '🦸🏽‍♂' => 'meeskangelane: keskmine nahatoon',
    '🦸🏾‍♂' => 'meeskangelane: keskmiselt tume nahatoon',
    '🦸🏿‍♂' => 'meeskangelane: tume nahatoon',
    '🦸🏻‍♀' => 'naissuperkangelane: hele nahatoon',
    '🦸🏼‍♀' => 'naissuperkangelane: keskmiselt hele nahatoon',
    '🦸🏽‍♀' => 'naissuperkangelane: keskmine nahatoon',
    '🦸🏾‍♀' => 'naissuperkangelane: keskmiselt tume nahatoon',
    '🦸🏿‍♀' => 'naissuperkangelane: tume nahatoon',
    '🦹🏻‍♂' => 'meessoost superkurikael: hele nahatoon',
    '🦹🏼‍♂' => 'meessoost superkurikael: keskmiselt hele nahatoon',
    '🦹🏽‍♂' => 'meessoost superkurikael: keskmine nahatoon',
    '🦹🏾‍♂' => 'meessoost superkurikael: keskmiselt tume nahatoon',
    '🦹🏿‍♂' => 'meessoost superkurikael: tume nahatoon',
    '🦹🏻‍♀' => 'naissoost superkurikael: hele nahatoon',
    '🦹🏼‍♀' => 'naissoost superkurikael: keskmiselt hele nahatoon',
    '🦹🏽‍♀' => 'naissoost superkurikael: keskmine nahatoon',
    '🦹🏾‍♀' => 'naissoost superkurikael: keskmiselt tume nahatoon',
    '🦹🏿‍♀' => 'naissoost superkurikael: tume nahatoon',
    '🧙🏻‍♂' => 'meesmaag: hele nahatoon',
    '🧙🏼‍♂' => 'meesmaag: keskmiselt hele nahatoon',
    '🧙🏽‍♂' => 'meesmaag: keskmine nahatoon',
    '🧙🏾‍♂' => 'meesmaag: keskmiselt tume nahatoon',
    '🧙🏿‍♂' => 'meesmaag: tume nahatoon',
    '🧙🏻‍♀' => 'naismaag: hele nahatoon',
    '🧙🏼‍♀' => 'naismaag: keskmiselt hele nahatoon',
    '🧙🏽‍♀' => 'naismaag: keskmine nahatoon',
    '🧙🏾‍♀' => 'naismaag: keskmiselt tume nahatoon',
    '🧙🏿‍♀' => 'naismaag: tume nahatoon',
    '🧚🏻‍♂' => 'meeshaldjas: hele nahatoon',
    '🧚🏼‍♂' => 'meeshaldjas: keskmiselt hele nahatoon',
    '🧚🏽‍♂' => 'meeshaldjas: keskmine nahatoon',
    '🧚🏾‍♂' => 'meeshaldjas: keskmiselt tume nahatoon',
    '🧚🏿‍♂' => 'meeshaldjas: tume nahatoon',
    '🧚🏻‍♀' => 'naishaldjas: hele nahatoon',
    '🧚🏼‍♀' => 'naishaldjas: keskmiselt hele nahatoon',
    '🧚🏽‍♀' => 'naishaldjas: keskmine nahatoon',
    '🧚🏾‍♀' => 'naishaldjas: keskmiselt tume nahatoon',
    '🧚🏿‍♀' => 'naishaldjas: tume nahatoon',
    '🧛🏻‍♂' => 'meesvampiir: hele nahatoon',
    '🧛🏼‍♂' => 'meesvampiir: keskmiselt hele nahatoon',
    '🧛🏽‍♂' => 'meesvampiir: keskmine nahatoon',
    '🧛🏾‍♂' => 'meesvampiir: keskmiselt tume nahatoon',
    '🧛🏿‍♂' => 'meesvampiir: tume nahatoon',
    '🧛🏻‍♀' => 'naisvampiir: hele nahatoon',
    '🧛🏼‍♀' => 'naisvampiir: keskmiselt hele nahatoon',
    '🧛🏽‍♀' => 'naisvampiir: keskmine nahatoon',
    '🧛🏾‍♀' => 'naisvampiir: keskmiselt tume nahatoon',
    '🧛🏿‍♀' => 'naisvampiir: tume nahatoon',
    '🧜🏻‍♂' => 'meesnäkk: hele nahatoon',
    '🧜🏼‍♂' => 'meesnäkk: keskmiselt hele nahatoon',
    '🧜🏽‍♂' => 'meesnäkk: keskmine nahatoon',
    '🧜🏾‍♂' => 'meesnäkk: keskmiselt tume nahatoon',
    '🧜🏿‍♂' => 'meesnäkk: tume nahatoon',
    '🧜🏻‍♀' => 'merineitsi: hele nahatoon',
    '🧜🏼‍♀' => 'merineitsi: keskmiselt hele nahatoon',
    '🧜🏽‍♀' => 'merineitsi: keskmine nahatoon',
    '🧜🏾‍♀' => 'merineitsi: keskmiselt tume nahatoon',
    '🧜🏿‍♀' => 'merineitsi: tume nahatoon',
    '🧝🏻‍♂' => 'meespäkapikk: hele nahatoon',
    '🧝🏼‍♂' => 'meespäkapikk: keskmiselt hele nahatoon',
    '🧝🏽‍♂' => 'meespäkapikk: keskmine nahatoon',
    '🧝🏾‍♂' => 'meespäkapikk: keskmiselt tume nahatoon',
    '🧝🏿‍♂' => 'meespäkapikk: tume nahatoon',
    '🧝🏻‍♀' => 'naispäkapikk: hele nahatoon',
    '🧝🏼‍♀' => 'naispäkapikk: keskmiselt hele nahatoon',
    '🧝🏽‍♀' => 'naispäkapikk: keskmine nahatoon',
    '🧝🏾‍♀' => 'naispäkapikk: keskmiselt tume nahatoon',
    '🧝🏿‍♀' => 'naispäkapikk: tume nahatoon',
    '💆🏻‍♂' => 'näomassaaži saav mees: hele nahatoon',
    '💆🏼‍♂' => 'näomassaaži saav mees: keskmiselt hele nahatoon',
    '💆🏽‍♂' => 'näomassaaži saav mees: keskmine nahatoon',
    '💆🏾‍♂' => 'näomassaaži saav mees: keskmiselt tume nahatoon',
    '💆🏿‍♂' => 'näomassaaži saav mees: tume nahatoon',
    '💆🏻‍♀' => 'näomassaaži saav naine: hele nahatoon',
    '💆🏼‍♀' => 'näomassaaži saav naine: keskmiselt hele nahatoon',
    '💆🏽‍♀' => 'näomassaaži saav naine: keskmine nahatoon',
    '💆🏾‍♀' => 'näomassaaži saav naine: keskmiselt tume nahatoon',
    '💆🏿‍♀' => 'näomassaaži saav naine: tume nahatoon',
    '💇🏻‍♂' => 'mees juuksuris: hele nahatoon',
    '💇🏼‍♂' => 'mees juuksuris: keskmiselt hele nahatoon',
    '💇🏽‍♂' => 'mees juuksuris: keskmine nahatoon',
    '💇🏾‍♂' => 'mees juuksuris: keskmiselt tume nahatoon',
    '💇🏿‍♂' => 'mees juuksuris: tume nahatoon',
    '💇🏻‍♀' => 'naine juuksuris: hele nahatoon',
    '💇🏼‍♀' => 'naine juuksuris: keskmiselt hele nahatoon',
    '💇🏽‍♀' => 'naine juuksuris: keskmine nahatoon',
    '💇🏾‍♀' => 'naine juuksuris: keskmiselt tume nahatoon',
    '💇🏿‍♀' => 'naine juuksuris: tume nahatoon',
    '🚶🏻‍♂' => 'kõndiv mees: hele nahatoon',
    '🚶🏼‍♂' => 'kõndiv mees: keskmiselt hele nahatoon',
    '🚶🏽‍♂' => 'kõndiv mees: keskmine nahatoon',
    '🚶🏾‍♂' => 'kõndiv mees: keskmiselt tume nahatoon',
    '🚶🏿‍♂' => 'kõndiv mees: tume nahatoon',
    '🚶🏻‍♀' => 'kõndiv naine: hele nahatoon',
    '🚶🏼‍♀' => 'kõndiv naine: keskmiselt hele nahatoon',
    '🚶🏽‍♀' => 'kõndiv naine: keskmine nahatoon',
    '🚶🏾‍♀' => 'kõndiv naine: keskmiselt tume nahatoon',
    '🚶🏿‍♀' => 'kõndiv naine: tume nahatoon',
    '🚶🏻‍➡' => 'jalakäija: hele nahatoon, suunaga paremale',
    '🚶🏼‍➡' => 'jalakäija: keskmiselt hele nahatoon, suunaga paremale',
    '🚶🏽‍➡' => 'jalakäija: keskmine nahatoon, suunaga paremale',
    '🚶🏾‍➡' => 'jalakäija: keskmiselt tume nahatoon, suunaga paremale',
    '🚶🏿‍➡' => 'jalakäija: tume nahatoon, suunaga paremale',
    '🧍🏻‍♂' => 'seisev mees: hele nahatoon',
    '🧍🏼‍♂' => 'seisev mees: keskmiselt hele nahatoon',
    '🧍🏽‍♂' => 'seisev mees: keskmine nahatoon',
    '🧍🏾‍♂' => 'seisev mees: keskmiselt tume nahatoon',
    '🧍🏿‍♂' => 'seisev mees: tume nahatoon',
    '🧍🏻‍♀' => 'seisev naine: hele nahatoon',
    '🧍🏼‍♀' => 'seisev naine: keskmiselt hele nahatoon',
    '🧍🏽‍♀' => 'seisev naine: keskmine nahatoon',
    '🧍🏾‍♀' => 'seisev naine: keskmiselt tume nahatoon',
    '🧍🏿‍♀' => 'seisev naine: tume nahatoon',
    '🧎🏻‍♂' => 'põlvitav mees: hele nahatoon',
    '🧎🏼‍♂' => 'põlvitav mees: keskmiselt hele nahatoon',
    '🧎🏽‍♂' => 'põlvitav mees: keskmine nahatoon',
    '🧎🏾‍♂' => 'põlvitav mees: keskmiselt tume nahatoon',
    '🧎🏿‍♂' => 'põlvitav mees: tume nahatoon',
    '🧎🏻‍♀' => 'põlvitav naine: hele nahatoon',
    '🧎🏼‍♀' => 'põlvitav naine: keskmiselt hele nahatoon',
    '🧎🏽‍♀' => 'põlvitav naine: keskmine nahatoon',
    '🧎🏾‍♀' => 'põlvitav naine: keskmiselt tume nahatoon',
    '🧎🏿‍♀' => 'põlvitav naine: tume nahatoon',
    '🧎🏻‍➡' => 'põlvitav inimene: hele nahatoon, suunaga paremale',
    '🧎🏼‍➡' => 'põlvitav inimene: keskmiselt hele nahatoon, suunaga paremale',
    '🧎🏽‍➡' => 'põlvitav inimene: keskmine nahatoon, suunaga paremale',
    '🧎🏾‍➡' => 'põlvitav inimene: keskmiselt tume nahatoon, suunaga paremale',
    '🧎🏿‍➡' => 'põlvitav inimene: tume nahatoon, suunaga paremale',
    '🧑🏻‍🦯' => 'valge kepiga inimene: hele nahatoon',
    '🧑🏼‍🦯' => 'valge kepiga inimene: keskmiselt hele nahatoon',
    '🧑🏽‍🦯' => 'valge kepiga inimene: keskmine nahatoon',
    '🧑🏾‍🦯' => 'valge kepiga inimene: keskmiselt tume nahatoon',
    '🧑🏿‍🦯' => 'valge kepiga inimene: tume nahatoon',
    '👨🏻‍🦯' => 'valge kepiga mees: hele nahatoon',
    '👨🏼‍🦯' => 'valge kepiga mees: keskmiselt hele nahatoon',
    '👨🏽‍🦯' => 'valge kepiga mees: keskmine nahatoon',
    '👨🏾‍🦯' => 'valge kepiga mees: keskmiselt tume nahatoon',
    '👨🏿‍🦯' => 'valge kepiga mees: tume nahatoon',
    '👩🏻‍🦯' => 'valge kepiga naine: hele nahatoon',
    '👩🏼‍🦯' => 'valge kepiga naine: keskmiselt hele nahatoon',
    '👩🏽‍🦯' => 'valge kepiga naine: keskmine nahatoon',
    '👩🏾‍🦯' => 'valge kepiga naine: keskmiselt tume nahatoon',
    '👩🏿‍🦯' => 'valge kepiga naine: tume nahatoon',
    '🧑🏻‍🦼' => 'elektrilises ratastoolis inimene: hele nahatoon',
    '🧑🏼‍🦼' => 'elektrilises ratastoolis inimene: keskmiselt hele nahatoon',
    '🧑🏽‍🦼' => 'elektrilises ratastoolis inimene: keskmine nahatoon',
    '🧑🏾‍🦼' => 'elektrilises ratastoolis inimene: keskmiselt tume nahatoon',
    '🧑🏿‍🦼' => 'elektrilises ratastoolis inimene: tume nahatoon',
    '👨🏻‍🦼' => 'elektriratastoolis olev mees: hele nahatoon',
    '👨🏼‍🦼' => 'elektriratastoolis olev mees: keskmiselt hele nahatoon',
    '👨🏽‍🦼' => 'elektriratastoolis olev mees: keskmine nahatoon',
    '👨🏾‍🦼' => 'elektriratastoolis olev mees: keskmiselt tume nahatoon',
    '👨🏿‍🦼' => 'elektriratastoolis olev mees: tume nahatoon',
    '👩🏻‍🦼' => 'elektriratastoolis olev naine: hele nahatoon',
    '👩🏼‍🦼' => 'elektriratastoolis olev naine: keskmiselt hele nahatoon',
    '👩🏽‍🦼' => 'elektriratastoolis olev naine: keskmine nahatoon',
    '👩🏾‍🦼' => 'elektriratastoolis olev naine: keskmiselt tume nahatoon',
    '👩🏿‍🦼' => 'elektriratastoolis olev naine: tume nahatoon',
    '🧑🏻‍🦽' => 'manuaalses ratastoolis inimene: hele nahatoon',
    '🧑🏼‍🦽' => 'manuaalses ratastoolis inimene: keskmiselt hele nahatoon',
    '🧑🏽‍🦽' => 'manuaalses ratastoolis inimene: keskmine nahatoon',
    '🧑🏾‍🦽' => 'manuaalses ratastoolis inimene: keskmiselt tume nahatoon',
    '🧑🏿‍🦽' => 'manuaalses ratastoolis inimene: tume nahatoon',
    '👨🏻‍🦽' => 'manuaalratastoolis olev mees: hele nahatoon',
    '👨🏼‍🦽' => 'manuaalratastoolis olev mees: keskmiselt hele nahatoon',
    '👨🏽‍🦽' => 'manuaalratastoolis olev mees: keskmine nahatoon',
    '👨🏾‍🦽' => 'manuaalratastoolis olev mees: keskmiselt tume nahatoon',
    '👨🏿‍🦽' => 'manuaalratastoolis olev mees: tume nahatoon',
    '👩🏻‍🦽' => 'manuaalratastoolis olev naine: hele nahatoon',
    '👩🏼‍🦽' => 'manuaalratastoolis olev naine: keskmiselt hele nahatoon',
    '👩🏽‍🦽' => 'manuaalratastoolis olev naine: keskmine nahatoon',
    '👩🏾‍🦽' => 'manuaalratastoolis olev naine: keskmiselt tume nahatoon',
    '👩🏿‍🦽' => 'manuaalratastoolis olev naine: tume nahatoon',
    '🏃🏻‍♂' => 'jooksev mees: hele nahatoon',
    '🏃🏼‍♂' => 'jooksev mees: keskmiselt hele nahatoon',
    '🏃🏽‍♂' => 'jooksev mees: keskmine nahatoon',
    '🏃🏾‍♂' => 'jooksev mees: keskmiselt tume nahatoon',
    '🏃🏿‍♂' => 'jooksev mees: tume nahatoon',
    '🏃🏻‍♀' => 'jooksev naine: hele nahatoon',
    '🏃🏼‍♀' => 'jooksev naine: keskmiselt hele nahatoon',
    '🏃🏽‍♀' => 'jooksev naine: keskmine nahatoon',
    '🏃🏾‍♀' => 'jooksev naine: keskmiselt tume nahatoon',
    '🏃🏿‍♀' => 'jooksev naine: tume nahatoon',
    '🏃🏻‍➡' => 'jooksja: hele nahatoon, suunaga paremale',
    '🏃🏼‍➡' => 'jooksja: keskmiselt hele nahatoon, suunaga paremale',
    '🏃🏽‍➡' => 'jooksja: keskmine nahatoon, suunaga paremale',
    '🏃🏾‍➡' => 'jooksja: keskmiselt tume nahatoon, suunaga paremale',
    '🏃🏿‍➡' => 'jooksja: tume nahatoon, suunaga paremale',
    '👯🏻‍♀' => 'pidutsevad jänesekõrvadega naised: hele nahatoon',
    '👯🏼‍♀' => 'pidutsevad jänesekõrvadega naised: keskmiselt hele nahatoon',
    '👯🏽‍♀' => 'pidutsevad jänesekõrvadega naised: keskmine nahatoon',
    '👯🏾‍♀' => 'pidutsevad jänesekõrvadega naised: keskmiselt tume nahatoon',
    '👯🏿‍♀' => 'pidutsevad jänesekõrvadega naised: tume nahatoon',
    '👯🏻‍♂' => 'pidutsevad jänesekõrvadega mehed: hele nahatoon',
    '👯🏼‍♂' => 'pidutsevad jänesekõrvadega mehed: keskmiselt hele nahatoon',
    '👯🏽‍♂' => 'pidutsevad jänesekõrvadega mehed: keskmine nahatoon',
    '👯🏾‍♂' => 'pidutsevad jänesekõrvadega mehed: keskmiselt tume nahatoon',
    '👯🏿‍♂' => 'pidutsevad jänesekõrvadega mehed: tume nahatoon',
    '🧖🏻‍♂' => 'leiliruumis olev mees: hele nahatoon',
    '🧖🏼‍♂' => 'leiliruumis olev mees: keskmiselt hele nahatoon',
    '🧖🏽‍♂' => 'leiliruumis olev mees: keskmine nahatoon',
    '🧖🏾‍♂' => 'leiliruumis olev mees: keskmiselt tume nahatoon',
    '🧖🏿‍♂' => 'leiliruumis olev mees: tume nahatoon',
    '🧖🏻‍♀' => 'naine leiliruumis: hele nahatoon',
    '🧖🏼‍♀' => 'naine leiliruumis: keskmiselt hele nahatoon',
    '🧖🏽‍♀' => 'naine leiliruumis: keskmine nahatoon',
    '🧖🏾‍♀' => 'naine leiliruumis: keskmiselt tume nahatoon',
    '🧖🏿‍♀' => 'naine leiliruumis: tume nahatoon',
    '🧗🏻‍♂' => 'mäkke roniv mees: hele nahatoon',
    '🧗🏼‍♂' => 'mäkke roniv mees: keskmiselt hele nahatoon',
    '🧗🏽‍♂' => 'mäkke roniv mees: keskmine nahatoon',
    '🧗🏾‍♂' => 'mäkke roniv mees: keskmiselt tume nahatoon',
    '🧗🏿‍♂' => 'mäkke roniv mees: tume nahatoon',
    '🧗🏻‍♀' => 'mäkke roniv naine: hele nahatoon',
    '🧗🏼‍♀' => 'mäkke roniv naine: keskmiselt hele nahatoon',
    '🧗🏽‍♀' => 'mäkke roniv naine: keskmine nahatoon',
    '🧗🏾‍♀' => 'mäkke roniv naine: keskmiselt tume nahatoon',
    '🧗🏿‍♀' => 'mäkke roniv naine: tume nahatoon',
    '🏌🏻‍♂' => 'golfi mängiv mees: hele nahatoon',
    '🏌🏼‍♂' => 'golfi mängiv mees: keskmiselt hele nahatoon',
    '🏌🏽‍♂' => 'golfi mängiv mees: keskmine nahatoon',
    '🏌🏾‍♂' => 'golfi mängiv mees: keskmiselt tume nahatoon',
    '🏌🏿‍♂' => 'golfi mängiv mees: tume nahatoon',
    '🏌🏻‍♀' => 'golfi mängiv naine: hele nahatoon',
    '🏌🏼‍♀' => 'golfi mängiv naine: keskmiselt hele nahatoon',
    '🏌🏽‍♀' => 'golfi mängiv naine: keskmine nahatoon',
    '🏌🏾‍♀' => 'golfi mängiv naine: keskmiselt tume nahatoon',
    '🏌🏿‍♀' => 'golfi mängiv naine: tume nahatoon',
    '🏄🏻‍♂' => 'meessurfar: hele nahatoon',
    '🏄🏼‍♂' => 'meessurfar: keskmiselt hele nahatoon',
    '🏄🏽‍♂' => 'meessurfar: keskmine nahatoon',
    '🏄🏾‍♂' => 'meessurfar: keskmiselt tume nahatoon',
    '🏄🏿‍♂' => 'meessurfar: tume nahatoon',
    '🏄🏻‍♀' => 'naissurfar: hele nahatoon',
    '🏄🏼‍♀' => 'naissurfar: keskmiselt hele nahatoon',
    '🏄🏽‍♀' => 'naissurfar: keskmine nahatoon',
    '🏄🏾‍♀' => 'naissurfar: keskmiselt tume nahatoon',
    '🏄🏿‍♀' => 'naissurfar: tume nahatoon',
    '🚣🏻‍♂' => 'aerupaadiga mees: hele nahatoon',
    '🚣🏼‍♂' => 'aerupaadiga mees: keskmiselt hele nahatoon',
    '🚣🏽‍♂' => 'aerupaadiga mees: keskmine nahatoon',
    '🚣🏾‍♂' => 'aerupaadiga mees: keskmiselt tume nahatoon',
    '🚣🏿‍♂' => 'aerupaadiga mees: tume nahatoon',
    '🚣🏻‍♀' => 'aerupaadiga naine: hele nahatoon',
    '🚣🏼‍♀' => 'aerupaadiga naine: keskmiselt hele nahatoon',
    '🚣🏽‍♀' => 'aerupaadiga naine: keskmine nahatoon',
    '🚣🏾‍♀' => 'aerupaadiga naine: keskmiselt tume nahatoon',
    '🚣🏿‍♀' => 'aerupaadiga naine: tume nahatoon',
    '🏊🏻‍♂' => 'meesujuja: hele nahatoon',
    '🏊🏼‍♂' => 'meesujuja: keskmiselt hele nahatoon',
    '🏊🏽‍♂' => 'meesujuja: keskmine nahatoon',
    '🏊🏾‍♂' => 'meesujuja: keskmiselt tume nahatoon',
    '🏊🏿‍♂' => 'meesujuja: tume nahatoon',
    '🏊🏻‍♀' => 'naisujuja: hele nahatoon',
    '🏊🏼‍♀' => 'naisujuja: keskmiselt hele nahatoon',
    '🏊🏽‍♀' => 'naisujuja: keskmine nahatoon',
    '🏊🏾‍♀' => 'naisujuja: keskmiselt tume nahatoon',
    '🏊🏿‍♀' => 'naisujuja: tume nahatoon',
    '⛹🏻‍♂' => 'mees palliga: hele nahatoon',
    '⛹🏼‍♂' => 'mees palliga: keskmiselt hele nahatoon',
    '⛹🏽‍♂' => 'mees palliga: keskmine nahatoon',
    '⛹🏾‍♂' => 'mees palliga: keskmiselt tume nahatoon',
    '⛹🏿‍♂' => 'mees palliga: tume nahatoon',
    '⛹🏻‍♀' => 'naine palliga: hele nahatoon',
    '⛹🏼‍♀' => 'naine palliga: keskmiselt hele nahatoon',
    '⛹🏽‍♀' => 'naine palliga: keskmine nahatoon',
    '⛹🏾‍♀' => 'naine palliga: keskmiselt tume nahatoon',
    '⛹🏿‍♀' => 'naine palliga: tume nahatoon',
    '🏋🏻‍♂' => 'meesjõutõstja: hele nahatoon',
    '🏋🏼‍♂' => 'meesjõutõstja: keskmiselt hele nahatoon',
    '🏋🏽‍♂' => 'meesjõutõstja: keskmine nahatoon',
    '🏋🏾‍♂' => 'meesjõutõstja: keskmiselt tume nahatoon',
    '🏋🏿‍♂' => 'meesjõutõstja: tume nahatoon',
    '🏋🏻‍♀' => 'naisjõutõstja: hele nahatoon',
    '🏋🏼‍♀' => 'naisjõutõstja: keskmiselt hele nahatoon',
    '🏋🏽‍♀' => 'naisjõutõstja: keskmine nahatoon',
    '🏋🏾‍♀' => 'naisjõutõstja: keskmiselt tume nahatoon',
    '🏋🏿‍♀' => 'naisjõutõstja: tume nahatoon',
    '🚴🏻‍♂' => 'meesrattur: hele nahatoon',
    '🚴🏼‍♂' => 'meesrattur: keskmiselt hele nahatoon',
    '🚴🏽‍♂' => 'meesrattur: keskmine nahatoon',
    '🚴🏾‍♂' => 'meesrattur: keskmiselt tume nahatoon',
    '🚴🏿‍♂' => 'meesrattur: tume nahatoon',
    '🚴🏻‍♀' => 'naisrattur: hele nahatoon',
    '🚴🏼‍♀' => 'naisrattur: keskmiselt hele nahatoon',
    '🚴🏽‍♀' => 'naisrattur: keskmine nahatoon',
    '🚴🏾‍♀' => 'naisrattur: keskmiselt tume nahatoon',
    '🚴🏿‍♀' => 'naisrattur: tume nahatoon',
    '🚵🏻‍♂' => 'meesmägirattur: hele nahatoon',
    '🚵🏼‍♂' => 'meesmägirattur: keskmiselt hele nahatoon',
    '🚵🏽‍♂' => 'meesmägirattur: keskmine nahatoon',
    '🚵🏾‍♂' => 'meesmägirattur: keskmiselt tume nahatoon',
    '🚵🏿‍♂' => 'meesmägirattur: tume nahatoon',
    '🚵🏻‍♀' => 'naismägirattur: hele nahatoon',
    '🚵🏼‍♀' => 'naismägirattur: keskmiselt hele nahatoon',
    '🚵🏽‍♀' => 'naismägirattur: keskmine nahatoon',
    '🚵🏾‍♀' => 'naismägirattur: keskmiselt tume nahatoon',
    '🚵🏿‍♀' => 'naismägirattur: tume nahatoon',
    '🤸🏻‍♂' => 'hundiratast viskav mees: hele nahatoon',
    '🤸🏼‍♂' => 'hundiratast viskav mees: keskmiselt hele nahatoon',
    '🤸🏽‍♂' => 'hundiratast viskav mees: keskmine nahatoon',
    '🤸🏾‍♂' => 'hundiratast viskav mees: keskmiselt tume nahatoon',
    '🤸🏿‍♂' => 'hundiratast viskav mees: tume nahatoon',
    '🤸🏻‍♀' => 'hundiratast viskav naine: hele nahatoon',
    '🤸🏼‍♀' => 'hundiratast viskav naine: keskmiselt hele nahatoon',
    '🤸🏽‍♀' => 'hundiratast viskav naine: keskmine nahatoon',
    '🤸🏾‍♀' => 'hundiratast viskav naine: keskmiselt tume nahatoon',
    '🤸🏿‍♀' => 'hundiratast viskav naine: tume nahatoon',
    '🤼🏻‍♀' => 'maadlevad naised: hele nahatoon',
    '🤼🏼‍♀' => 'maadlevad naised: keskmiselt hele nahatoon',
    '🤼🏽‍♀' => 'maadlevad naised: keskmine nahatoon',
    '🤼🏾‍♀' => 'maadlevad naised: keskmiselt tume nahatoon',
    '🤼🏿‍♀' => 'maadlevad naised: tume nahatoon',
    '🤼🏻‍♂' => 'maadlevad mehed: hele nahatoon',
    '🤼🏼‍♂' => 'maadlevad mehed: keskmiselt hele nahatoon',
    '🤼🏽‍♂' => 'maadlevad mehed: keskmine nahatoon',
    '🤼🏾‍♂' => 'maadlevad mehed: keskmiselt tume nahatoon',
    '🤼🏿‍♂' => 'maadlevad mehed: tume nahatoon',
    '🤽🏻‍♂' => 'veepalli mängiv mees: hele nahatoon',
    '🤽🏼‍♂' => 'veepalli mängiv mees: keskmiselt hele nahatoon',
    '🤽🏽‍♂' => 'veepalli mängiv mees: keskmine nahatoon',
    '🤽🏾‍♂' => 'veepalli mängiv mees: keskmiselt tume nahatoon',
    '🤽🏿‍♂' => 'veepalli mängiv mees: tume nahatoon',
    '🤽🏻‍♀' => 'veepalli mängiv naine: hele nahatoon',
    '🤽🏼‍♀' => 'veepalli mängiv naine: keskmiselt hele nahatoon',
    '🤽🏽‍♀' => 'veepalli mängiv naine: keskmine nahatoon',
    '🤽🏾‍♀' => 'veepalli mängiv naine: keskmiselt tume nahatoon',
    '🤽🏿‍♀' => 'veepalli mängiv naine: tume nahatoon',
    '🤾🏻‍♂' => 'käsipalli mängiv mees: hele nahatoon',
    '🤾🏼‍♂' => 'käsipalli mängiv mees: keskmiselt hele nahatoon',
    '🤾🏽‍♂' => 'käsipalli mängiv mees: keskmine nahatoon',
    '🤾🏾‍♂' => 'käsipalli mängiv mees: keskmiselt tume nahatoon',
    '🤾🏿‍♂' => 'käsipalli mängiv mees: tume nahatoon',
    '🤾🏻‍♀' => 'käsipalli mängiv naine: hele nahatoon',
    '🤾🏼‍♀' => 'käsipalli mängiv naine: keskmiselt hele nahatoon',
    '🤾🏽‍♀' => 'käsipalli mängiv naine: keskmine nahatoon',
    '🤾🏾‍♀' => 'käsipalli mängiv naine: keskmiselt tume nahatoon',
    '🤾🏿‍♀' => 'käsipalli mängiv naine: tume nahatoon',
    '🤹🏻‍♂' => 'meesžonglöör: hele nahatoon',
    '🤹🏼‍♂' => 'meesžonglöör: keskmiselt hele nahatoon',
    '🤹🏽‍♂' => 'meesžonglöör: keskmine nahatoon',
    '🤹🏾‍♂' => 'meesžonglöör: keskmiselt tume nahatoon',
    '🤹🏿‍♂' => 'meesžonglöör: tume nahatoon',
    '🤹🏻‍♀' => 'naisžonglöör: hele nahatoon',
    '🤹🏼‍♀' => 'naisžonglöör: keskmiselt hele nahatoon',
    '🤹🏽‍♀' => 'naisžonglöör: keskmine nahatoon',
    '🤹🏾‍♀' => 'naisžonglöör: keskmiselt tume nahatoon',
    '🤹🏿‍♀' => 'naisžonglöör: tume nahatoon',
    '🧘🏻‍♂' => 'lootosasendis mees: hele nahatoon',
    '🧘🏼‍♂' => 'lootosasendis mees: keskmiselt hele nahatoon',
    '🧘🏽‍♂' => 'lootosasendis mees: keskmine nahatoon',
    '🧘🏾‍♂' => 'lootosasendis mees: keskmiselt tume nahatoon',
    '🧘🏿‍♂' => 'lootosasendis mees: tume nahatoon',
    '🧘🏻‍♀' => 'lootosasendis naine: hele nahatoon',
    '🧘🏼‍♀' => 'lootosasendis naine: keskmiselt hele nahatoon',
    '🧘🏽‍♀' => 'lootosasendis naine: keskmine nahatoon',
    '🧘🏾‍♀' => 'lootosasendis naine: keskmiselt tume nahatoon',
    '🧘🏿‍♀' => 'lootosasendis naine: tume nahatoon',
    '🧑‍🦰' => 'täiskasvanu: punased juuksed',
    '🧑‍🦱' => 'täiskasvanu: lokkis juuksed',
    '🧑‍🦳' => 'täiskasvanu: heledad juuksed',
    '🧑‍🦲' => 'täiskasvanu: kiilaspea',
    '👨‍🦰' => 'mees: punased juuksed',
    '👨‍🦱' => 'mees: lokkis juuksed',
    '👨‍🦳' => 'mees: heledad juuksed',
    '👨‍🦲' => 'mees: kiilaspea',
    '👩‍🦰' => 'naine: punased juuksed',
    '👩‍🦱' => 'naine: lokkis juuksed',
    '👩‍🦳' => 'naine: heledad juuksed',
    '👩‍🦲' => 'naine: kiilaspea',
    '🚶‍➡' => 'jalakäija: suunaga paremale',
    '🧎‍➡' => 'põlvitav inimene: suunaga paremale',
    '🏃‍➡' => 'jooksja: suunaga paremale',
    '👨‍👦' => 'perekond: mees, poiss',
    '👨‍👧' => 'perekond: mees, tüdruk',
    '👩‍👦' => 'perekond: naine, poiss',
    '👩‍👧' => 'perekond: naine, tüdruk',
    '😶‍🌫' => 'nägu pilvedes',
    '😮‍💨' => 'väljahingav nägu',
    '🙂‍↕' => 'peanoogutus',
    '😵‍💫' => 'spiraalsilmadega nägu',
    '❤‍🔥' => 'leegitsev süda',
    '❤‍🩹' => 'tervenev süda',
    '👁‍🗨' => 'silm jutumullis',
    '🧔‍♂' => 'habemega mees',
    '👱‍♂' => 'mees: blond',
    '🧔‍♀' => 'habemega naine',
    '👱‍♀' => 'naine: blond',
    '🙍‍♂' => 'kulmu kortsutav mees',
    '🙍‍♀' => 'kulmu kortsutav naine',
    '🙎‍♂' => 'mossitav mees',
    '🙎‍♀' => 'mossitav naine',
    '🙅‍♂' => 'eitavat žesti näitav mees',
    '🙅‍♀' => 'eitavat žesti näitav naine',
    '🙆‍♂' => 'nõustuvat žesti näitav mees',
    '🙆‍♀' => 'nõustuvat žesti näitav naine',
    '💁‍♂' => 'infoleti meestöötaja',
    '💁‍♀' => 'infoleti naistöötaja',
    '🙋‍♂' => 'kätt tõstev mees',
    '🙋‍♀' => 'kätt tõstev naine',
    '🧏‍♂' => 'kurt mees',
    '🧏‍♀' => 'kurt naine',
    '🙇‍♂' => 'sügavalt kummarduv mees',
    '🙇‍♀' => 'sügavalt kummarduv naine',
    '🤦‍♂' => 'käega vastu pead lööv mees',
    '🤦‍♀' => 'käega vastu pead lööv naine',
    '🤷‍♂' => 'õlgu kehitav mees',
    '🤷‍♀' => 'õlgu kehitav naine',
    '🧑‍⚕' => 'tervishoiutöötaja',
    '👨‍⚕' => 'meesarst',
    '👩‍⚕' => 'naisarst',
    '🧑‍🎓' => 'õpilane',
    '👨‍🎓' => 'meesõpilane',
    '👩‍🎓' => 'naisõpilane',
    '🧑‍🏫' => 'õpetaja',
    '👨‍🏫' => 'meesõpetaja',
    '👩‍🏫' => 'naisõpetaja',
    '🧑‍⚖' => 'kohtunik',
    '👨‍⚖' => 'meeskohtunik',
    '👩‍⚖' => 'naiskohtunik',
    '🧑‍🌾' => 'talunik',
    '👨‍🌾' => 'meesfarmer',
    '👩‍🌾' => 'naisfarmer',
    '🧑‍🍳' => 'kokk',
    '👨‍🍳' => 'meeskokk',
    '👩‍🍳' => 'naiskokk',
    '🧑‍🔧' => 'mehaanik',
    '👨‍🔧' => 'meesmehaanik',
    '👩‍🔧' => 'naismehaanik',
    '🧑‍🏭' => 'tehasetööline',
    '👨‍🏭' => 'meessoost tehasetööline',
    '👩‍🏭' => 'naissoost tehasetööline',
    '🧑‍💼' => 'kontoritöötaja',
    '👨‍💼' => 'meeskontoritöötaja',
    '👩‍💼' => 'naiskontoritöötaja',
    '🧑‍🔬' => 'teadlane',
    '👨‍🔬' => 'meesteadlane',
    '👩‍🔬' => 'naisteadlane',
    '🧑‍💻' => 'programmeerija',
    '👨‍💻' => 'meesprogrammeerija',
    '👩‍💻' => 'naisprogrammeerija',
    '🧑‍🎤' => 'laulja',
    '👨‍🎤' => 'meeslaulja',
    '👩‍🎤' => 'naislaulja',
    '🧑‍🎨' => 'kunstnik',
    '👨‍🎨' => 'meeskunstnik',
    '👩‍🎨' => 'naiskunstnik',
    '🧑‍✈' => 'piloot',
    '👨‍✈' => 'meespiloot',
    '👩‍✈' => 'naispiloot',
    '🧑‍🚀' => 'astronaut',
    '👨‍🚀' => 'meesastronaut',
    '👩‍🚀' => 'naisastronaut',
    '🧑‍🚒' => 'tuletõrjuja',
    '👨‍🚒' => 'meestuletõrjuja',
    '👩‍🚒' => 'naistuletõrjuja',
    '👮‍♂' => 'meespolitseinik',
    '👮‍♀' => 'naispolitseinik',
    '🕵‍♂' => 'meesdetektiiv',
    '🕵‍♀' => 'naisdetektiiv',
    '💂‍♂' => 'meesvalvur',
    '💂‍♀' => 'naisvalvur',
    '👷‍♂' => 'meesehitaja',
    '👷‍♀' => 'naisehitaja',
    '👳‍♂' => 'mees turbaniga',
    '👳‍♀' => 'naine turbaninga',
    '🤵‍♂' => 'mees smokingis',
    '🤵‍♀' => 'naine smokingus',
    '👰‍♂' => 'mees looriga',
    '👰‍♀' => 'naine looriga',
    '👩‍🍼' => 'beebit toitev naine',
    '👨‍🍼' => 'beebit toitev mees',
    '🧑‍🍼' => 'beebit toitev inimene',
    '🧑‍🎄' => 'päkats',
    '🦸‍♂' => 'meeskangelane',
    '🦸‍♀' => 'naissuperkangelane',
    '🦹‍♂' => 'meessoost superkurikael',
    '🦹‍♀' => 'naissoost superkurikael',
    '🧙‍♂' => 'meesmaag',
    '🧙‍♀' => 'naismaag',
    '🧚‍♂' => 'meeshaldjas',
    '🧚‍♀' => 'naishaldjas',
    '🧛‍♂' => 'meesvampiir',
    '🧛‍♀' => 'naisvampiir',
    '🧜‍♂' => 'meesnäkk',
    '🧜‍♀' => 'merineitsi',
    '🧝‍♂' => 'meespäkapikk',
    '🧝‍♀' => 'naispäkapikk',
    '🧞‍♂' => 'meesdžinn',
    '🧞‍♀' => 'naisdžinn',
    '🧟‍♂' => 'meeszombi',
    '🧟‍♀' => 'naiszombi',
    '💆‍♂' => 'näomassaaži saav mees',
    '💆‍♀' => 'näomassaaži saav naine',
    '💇‍♂' => 'mees juuksuris',
    '💇‍♀' => 'naine juuksuris',
    '🚶‍♂' => 'kõndiv mees',
    '🚶‍♀' => 'kõndiv naine',
    '🧍‍♂' => 'seisev mees',
    '🧍‍♀' => 'seisev naine',
    '🧎‍♂' => 'põlvitav mees',
    '🧎‍♀' => 'põlvitav naine',
    '🧑‍🦯' => 'valge kepiga inimene',
    '👨‍🦯' => 'valge kepiga mees',
    '👩‍🦯' => 'valge kepiga naine',
    '🧑‍🦼' => 'elektrilises ratastoolis inimene',
    '👨‍🦼' => 'elektriratastoolis olev mees',
    '👩‍🦼' => 'elektriratastoolis olev naine',
    '🧑‍🦽' => 'manuaalses ratastoolis inimene',
    '👨‍🦽' => 'manuaalratastoolis olev mees',
    '👩‍🦽' => 'manuaalratastoolis olev naine',
    '🏃‍♂' => 'jooksev mees',
    '🏃‍♀' => 'jooksev naine',
    '🧑‍🩰' => 'balletitantsija',
    '👯‍♂' => 'pidutsevad jänesekõrvadega mehed',
    '👯‍♀' => 'pidutsevad jänesekõrvadega naised',
    '🧖‍♂' => 'leiliruumis olev mees',
    '🧖‍♀' => 'naine leiliruumis',
    '🧗‍♂' => 'mäkke roniv mees',
    '🧗‍♀' => 'mäkke roniv naine',
    '🏌‍♂' => 'golfi mängiv mees',
    '🏌‍♀' => 'golfi mängiv naine',
    '🏄‍♂' => 'meessurfar',
    '🏄‍♀' => 'naissurfar',
    '🚣‍♂' => 'aerupaadiga mees',
    '🚣‍♀' => 'aerupaadiga naine',
    '🏊‍♂' => 'meesujuja',
    '🏊‍♀' => 'naisujuja',
    '⛹‍♂' => 'mees palliga',
    '⛹‍♀' => 'naine palliga',
    '🏋‍♂' => 'meesjõutõstja',
    '🏋‍♀' => 'naisjõutõstja',
    '🚴‍♂' => 'meesrattur',
    '🚴‍♀' => 'naisrattur',
    '🚵‍♂' => 'meesmägirattur',
    '🚵‍♀' => 'naismägirattur',
    '🤸‍♂' => 'hundiratast viskav mees',
    '🤸‍♀' => 'hundiratast viskav naine',
    '🤼‍♂' => 'maadlevad mehed',
    '🤼‍♀' => 'maadlevad naised',
    '🤽‍♂' => 'veepalli mängiv mees',
    '🤽‍♀' => 'veepalli mängiv naine',
    '🤾‍♂' => 'käsipalli mängiv mees',
    '🤾‍♀' => 'käsipalli mängiv naine',
    '🤹‍♂' => 'meesžonglöör',
    '🤹‍♀' => 'naisžonglöör',
    '🧘‍♂' => 'lootosasendis mees',
    '🧘‍♀' => 'lootosasendis naine',
    '🧑‍🧒' => 'perekond: täiskasvanu, laps',
    '🐕‍🦺' => 'teenistuskoer',
    '🐈‍⬛' => 'must kass',
    '🐻‍❄' => 'jääkaru',
    '🐦‍⬛' => 'must lind',
    '🐦‍🔥' => 'fööniks',
    '🍋‍🟩' => 'laim',
    '🍄‍🟫' => 'šampinjon',
    '⛓‍💥' => 'katkine kett',
    '🏳‍🌈' => 'vikerkaarelipp',
    '🏳‍⚧' => 'transsoolisuse lipp',
    '🏴‍☠' => 'piraadilipp',
    '👋🏻' => 'lehvitav käsi: hele nahatoon',
    '👋🏼' => 'lehvitav käsi: keskmiselt hele nahatoon',
    '👋🏽' => 'lehvitav käsi: keskmine nahatoon',
    '👋🏾' => 'lehvitav käsi: keskmiselt tume nahatoon',
    '👋🏿' => 'lehvitav käsi: tume nahatoon',
    '🤚🏻' => 'tõstetud käsi: hele nahatoon',
    '🤚🏼' => 'tõstetud käsi: keskmiselt hele nahatoon',
    '🤚🏽' => 'tõstetud käsi: keskmine nahatoon',
    '🤚🏾' => 'tõstetud käsi: keskmiselt tume nahatoon',
    '🤚🏿' => 'tõstetud käsi: tume nahatoon',
    '🖐🏻' => 'harali sõrmedega käsi: hele nahatoon',
    '🖐🏼' => 'harali sõrmedega käsi: keskmiselt hele nahatoon',
    '🖐🏽' => 'harali sõrmedega käsi: keskmine nahatoon',
    '🖐🏾' => 'harali sõrmedega käsi: keskmiselt tume nahatoon',
    '🖐🏿' => 'harali sõrmedega käsi: tume nahatoon',
    '✋🏻' => 'ülestõstetud käsi: hele nahatoon',
    '✋🏼' => 'ülestõstetud käsi: keskmiselt hele nahatoon',
    '✋🏽' => 'ülestõstetud käsi: keskmine nahatoon',
    '✋🏾' => 'ülestõstetud käsi: keskmiselt tume nahatoon',
    '✋🏿' => 'ülestõstetud käsi: tume nahatoon',
    '🖖🏻' => 'vulcanite tervitus: hele nahatoon',
    '🖖🏼' => 'vulcanite tervitus: keskmiselt hele nahatoon',
    '🖖🏽' => 'vulcanite tervitus: keskmine nahatoon',
    '🖖🏾' => 'vulcanite tervitus: keskmiselt tume nahatoon',
    '🖖🏿' => 'vulcanite tervitus: tume nahatoon',
    '🫱🏻' => 'käsi paremale: hele nahatoon',
    '🫱🏼' => 'käsi paremale: keskmiselt hele nahatoon',
    '🫱🏽' => 'käsi paremale: keskmine nahatoon',
    '🫱🏾' => 'käsi paremale: keskmiselt tume nahatoon',
    '🫱🏿' => 'käsi paremale: tume nahatoon',
    '🫲🏻' => 'käsi vasakule: hele nahatoon',
    '🫲🏼' => 'käsi vasakule: keskmiselt hele nahatoon',
    '🫲🏽' => 'käsi vasakule: keskmine nahatoon',
    '🫲🏾' => 'käsi vasakule: keskmiselt tume nahatoon',
    '🫲🏿' => 'käsi vasakule: tume nahatoon',
    '🫳🏻' => 'käsi peopesaga allapoole: hele nahatoon',
    '🫳🏼' => 'käsi peopesaga allapoole: keskmiselt hele nahatoon',
    '🫳🏽' => 'käsi peopesaga allapoole: keskmine nahatoon',
    '🫳🏾' => 'käsi peopesaga allapoole: keskmiselt tume nahatoon',
    '🫳🏿' => 'käsi peopesaga allapoole: tume nahatoon',
    '🫴🏻' => 'käsi peopesaga ülespoole: hele nahatoon',
    '🫴🏼' => 'käsi peopesaga ülespoole: keskmiselt hele nahatoon',
    '🫴🏽' => 'käsi peopesaga ülespoole: keskmine nahatoon',
    '🫴🏾' => 'käsi peopesaga ülespoole: keskmiselt tume nahatoon',
    '🫴🏿' => 'käsi peopesaga ülespoole: tume nahatoon',
    '🫷🏻' => 'vasakule lükkav käsi: hele nahatoon',
    '🫷🏼' => 'vasakule lükkav käsi: keskmiselt hele nahatoon',
    '🫷🏽' => 'vasakule lükkav käsi: keskmine nahatoon',
    '🫷🏾' => 'vasakule lükkav käsi: keskmiselt tume nahatoon',
    '🫷🏿' => 'vasakule lükkav käsi: tume nahatoon',
    '🫸🏻' => 'paremale lükkav käsi: hele nahatoon',
    '🫸🏼' => 'paremale lükkav käsi: keskmiselt hele nahatoon',
    '🫸🏽' => 'paremale lükkav käsi: keskmine nahatoon',
    '🫸🏾' => 'paremale lükkav käsi: keskmiselt tume nahatoon',
    '🫸🏿' => 'paremale lükkav käsi: tume nahatoon',
    '👌🏻' => 'OK-käemärk: hele nahatoon',
    '👌🏼' => 'OK-käemärk: keskmiselt hele nahatoon',
    '👌🏽' => 'OK-käemärk: keskmine nahatoon',
    '👌🏾' => 'OK-käemärk: keskmiselt tume nahatoon',
    '👌🏿' => 'OK-käemärk: tume nahatoon',
    '🤌🏻' => 'kokkusurutud sõrmed: hele nahatoon',
    '🤌🏼' => 'kokkusurutud sõrmed: keskmiselt hele nahatoon',
    '🤌🏽' => 'kokkusurutud sõrmed: keskmine nahatoon',
    '🤌🏾' => 'kokkusurutud sõrmed: keskmiselt tume nahatoon',
    '🤌🏿' => 'kokkusurutud sõrmed: tume nahatoon',
    '🤏🏻' => 'näpistav käsi: hele nahatoon',
    '🤏🏼' => 'näpistav käsi: keskmiselt hele nahatoon',
    '🤏🏽' => 'näpistav käsi: keskmine nahatoon',
    '🤏🏾' => 'näpistav käsi: keskmiselt tume nahatoon',
    '🤏🏿' => 'näpistav käsi: tume nahatoon',
    '✌🏻' => 'võit: hele nahatoon',
    '✌🏼' => 'võit: keskmiselt hele nahatoon',
    '✌🏽' => 'võit: keskmine nahatoon',
    '✌🏾' => 'võit: keskmiselt tume nahatoon',
    '✌🏿' => 'võit: tume nahatoon',
    '🤞🏻' => 'sõrmed risti: hele nahatoon',
    '🤞🏼' => 'sõrmed risti: keskmiselt hele nahatoon',
    '🤞🏽' => 'sõrmed risti: keskmine nahatoon',
    '🤞🏾' => 'sõrmed risti: keskmiselt tume nahatoon',
    '🤞🏿' => 'sõrmed risti: tume nahatoon',
    '🫰🏻' => 'pöialt ja nimetissõrme risti hoidev käsi: hele nahatoon',
    '🫰🏼' => 'pöialt ja nimetissõrme risti hoidev käsi: keskmiselt hele nahatoon',
    '🫰🏽' => 'pöialt ja nimetissõrme risti hoidev käsi: keskmine nahatoon',
    '🫰🏾' => 'pöialt ja nimetissõrme risti hoidev käsi: keskmiselt tume nahatoon',
    '🫰🏿' => 'pöialt ja nimetissõrme risti hoidev käsi: tume nahatoon',
    '🤟🏻' => 'armastust väljendav žest: hele nahatoon',
    '🤟🏼' => 'armastust väljendav žest: keskmiselt hele nahatoon',
    '🤟🏽' => 'armastust väljendav žest: keskmine nahatoon',
    '🤟🏾' => 'armastust väljendav žest: keskmiselt tume nahatoon',
    '🤟🏿' => 'armastust väljendav žest: tume nahatoon',
    '🤘🏻' => 'sarvede märk: hele nahatoon',
    '🤘🏼' => 'sarvede märk: keskmiselt hele nahatoon',
    '🤘🏽' => 'sarvede märk: keskmine nahatoon',
    '🤘🏾' => 'sarvede märk: keskmiselt tume nahatoon',
    '🤘🏿' => 'sarvede märk: tume nahatoon',
    '🤙🏻' => 'helista mulle: hele nahatoon',
    '🤙🏼' => 'helista mulle: keskmiselt hele nahatoon',
    '🤙🏽' => 'helista mulle: keskmine nahatoon',
    '🤙🏾' => 'helista mulle: keskmiselt tume nahatoon',
    '🤙🏿' => 'helista mulle: tume nahatoon',
    '👈🏻' => 'vasakule osutav sõrm: hele nahatoon',
    '👈🏼' => 'vasakule osutav sõrm: keskmiselt hele nahatoon',
    '👈🏽' => 'vasakule osutav sõrm: keskmine nahatoon',
    '👈🏾' => 'vasakule osutav sõrm: keskmiselt tume nahatoon',
    '👈🏿' => 'vasakule osutav sõrm: tume nahatoon',
    '👉🏻' => 'paremale osutav sõrm: hele nahatoon',
    '👉🏼' => 'paremale osutav sõrm: keskmiselt hele nahatoon',
    '👉🏽' => 'paremale osutav sõrm: keskmine nahatoon',
    '👉🏾' => 'paremale osutav sõrm: keskmiselt tume nahatoon',
    '👉🏿' => 'paremale osutav sõrm: tume nahatoon',
    '👆🏻' => 'üles osutav sõrm tagantvaates: hele nahatoon',
    '👆🏼' => 'üles osutav sõrm tagantvaates: keskmiselt hele nahatoon',
    '👆🏽' => 'üles osutav sõrm tagantvaates: keskmine nahatoon',
    '👆🏾' => 'üles osutav sõrm tagantvaates: keskmiselt tume nahatoon',
    '👆🏿' => 'üles osutav sõrm tagantvaates: tume nahatoon',
    '🖕🏻' => 'keskmine sõrm: hele nahatoon',
    '🖕🏼' => 'keskmine sõrm: keskmiselt hele nahatoon',
    '🖕🏽' => 'keskmine sõrm: keskmine nahatoon',
    '🖕🏾' => 'keskmine sõrm: keskmiselt tume nahatoon',
    '🖕🏿' => 'keskmine sõrm: tume nahatoon',
    '👇🏻' => 'alla osutav sõrm: hele nahatoon',
    '👇🏼' => 'alla osutav sõrm: keskmiselt hele nahatoon',
    '👇🏽' => 'alla osutav sõrm: keskmine nahatoon',
    '👇🏾' => 'alla osutav sõrm: keskmiselt tume nahatoon',
    '👇🏿' => 'alla osutav sõrm: tume nahatoon',
    '☝🏻' => 'üles osutav sõrm: hele nahatoon',
    '☝🏼' => 'üles osutav sõrm: keskmiselt hele nahatoon',
    '☝🏽' => 'üles osutav sõrm: keskmine nahatoon',
    '☝🏾' => 'üles osutav sõrm: keskmiselt tume nahatoon',
    '☝🏿' => 'üles osutav sõrm: tume nahatoon',
    '🫵🏻' => 'vaataja poole osutav nimetissõrm: hele nahatoon',
    '🫵🏼' => 'vaataja poole osutav nimetissõrm: keskmiselt hele nahatoon',
    '🫵🏽' => 'vaataja poole osutav nimetissõrm: keskmine nahatoon',
    '🫵🏾' => 'vaataja poole osutav nimetissõrm: keskmiselt tume nahatoon',
    '🫵🏿' => 'vaataja poole osutav nimetissõrm: tume nahatoon',
    '👍🏻' => 'meeldib: hele nahatoon',
    '👍🏼' => 'meeldib: keskmiselt hele nahatoon',
    '👍🏽' => 'meeldib: keskmine nahatoon',
    '👍🏾' => 'meeldib: keskmiselt tume nahatoon',
    '👍🏿' => 'meeldib: tume nahatoon',
    '👎🏻' => 'ei meeldi: hele nahatoon',
    '👎🏼' => 'ei meeldi: keskmiselt hele nahatoon',
    '👎🏽' => 'ei meeldi: keskmine nahatoon',
    '👎🏾' => 'ei meeldi: keskmiselt tume nahatoon',
    '👎🏿' => 'ei meeldi: tume nahatoon',
    '✊🏻' => 'ülestõstetud rusikas: hele nahatoon',
    '✊🏼' => 'ülestõstetud rusikas: keskmiselt hele nahatoon',
    '✊🏽' => 'ülestõstetud rusikas: keskmine nahatoon',
    '✊🏾' => 'ülestõstetud rusikas: keskmiselt tume nahatoon',
    '✊🏿' => 'ülestõstetud rusikas: tume nahatoon',
    '👊🏻' => 'rusikalöök: hele nahatoon',
    '👊🏼' => 'rusikalöök: keskmiselt hele nahatoon',
    '👊🏽' => 'rusikalöök: keskmine nahatoon',
    '👊🏾' => 'rusikalöök: keskmiselt tume nahatoon',
    '👊🏿' => 'rusikalöök: tume nahatoon',
    '🤛🏻' => 'vasakule suunatud rusikas: hele nahatoon',
    '🤛🏼' => 'vasakule suunatud rusikas: keskmiselt hele nahatoon',
    '🤛🏽' => 'vasakule suunatud rusikas: keskmine nahatoon',
    '🤛🏾' => 'vasakule suunatud rusikas: keskmiselt tume nahatoon',
    '🤛🏿' => 'vasakule suunatud rusikas: tume nahatoon',
    '🤜🏻' => 'paremale suunatud rusikas: hele nahatoon',
    '🤜🏼' => 'paremale suunatud rusikas: keskmiselt hele nahatoon',
    '🤜🏽' => 'paremale suunatud rusikas: keskmine nahatoon',
    '🤜🏾' => 'paremale suunatud rusikas: keskmiselt tume nahatoon',
    '🤜🏿' => 'paremale suunatud rusikas: tume nahatoon',
    '👏🏻' => 'plaksutavad käed: hele nahatoon',
    '👏🏼' => 'plaksutavad käed: keskmiselt hele nahatoon',
    '👏🏽' => 'plaksutavad käed: keskmine nahatoon',
    '👏🏾' => 'plaksutavad käed: keskmiselt tume nahatoon',
    '👏🏿' => 'plaksutavad käed: tume nahatoon',
    '🙌🏻' => 'ülestõstetud käed: hele nahatoon',
    '🙌🏼' => 'ülestõstetud käed: keskmiselt hele nahatoon',
    '🙌🏽' => 'ülestõstetud käed: keskmine nahatoon',
    '🙌🏾' => 'ülestõstetud käed: keskmiselt tume nahatoon',
    '🙌🏿' => 'ülestõstetud käed: tume nahatoon',
    '🫶🏻' => 'südant moodustavad käed: hele nahatoon',
    '🫶🏼' => 'südant moodustavad käed: keskmiselt hele nahatoon',
    '🫶🏽' => 'südant moodustavad käed: keskmine nahatoon',
    '🫶🏾' => 'südant moodustavad käed: keskmiselt tume nahatoon',
    '🫶🏿' => 'südant moodustavad käed: tume nahatoon',
    '👐🏻' => 'avatud käed: hele nahatoon',
    '👐🏼' => 'avatud käed: keskmiselt hele nahatoon',
    '👐🏽' => 'avatud käed: keskmine nahatoon',
    '👐🏾' => 'avatud käed: keskmiselt tume nahatoon',
    '👐🏿' => 'avatud käed: tume nahatoon',
    '🤲🏻' => 'peopesad üleval koos: hele nahatoon',
    '🤲🏼' => 'peopesad üleval koos: keskmiselt hele nahatoon',
    '🤲🏽' => 'peopesad üleval koos: keskmine nahatoon',
    '🤲🏾' => 'peopesad üleval koos: keskmiselt tume nahatoon',
    '🤲🏿' => 'peopesad üleval koos: tume nahatoon',
    '🤝🏻' => 'käepigistus: hele nahatoon',
    '🤝🏼' => 'käepigistus: keskmiselt hele nahatoon',
    '🤝🏽' => 'käepigistus: keskmine nahatoon',
    '🤝🏾' => 'käepigistus: keskmiselt tume nahatoon',
    '🤝🏿' => 'käepigistus: tume nahatoon',
    '🙏🏻' => 'palveasendis käed: hele nahatoon',
    '🙏🏼' => 'palveasendis käed: keskmiselt hele nahatoon',
    '🙏🏽' => 'palveasendis käed: keskmine nahatoon',
    '🙏🏾' => 'palveasendis käed: keskmiselt tume nahatoon',
    '🙏🏿' => 'palveasendis käed: tume nahatoon',
    '✍🏻' => 'kirjutav käsi: hele nahatoon',
    '✍🏼' => 'kirjutav käsi: keskmiselt hele nahatoon',
    '✍🏽' => 'kirjutav käsi: keskmine nahatoon',
    '✍🏾' => 'kirjutav käsi: keskmiselt tume nahatoon',
    '✍🏿' => 'kirjutav käsi: tume nahatoon',
    '💅🏻' => 'küünelakk: hele nahatoon',
    '💅🏼' => 'küünelakk: keskmiselt hele nahatoon',
    '💅🏽' => 'küünelakk: keskmine nahatoon',
    '💅🏾' => 'küünelakk: keskmiselt tume nahatoon',
    '💅🏿' => 'küünelakk: tume nahatoon',
    '🤳🏻' => 'selfi: hele nahatoon',
    '🤳🏼' => 'selfi: keskmiselt hele nahatoon',
    '🤳🏽' => 'selfi: keskmine nahatoon',
    '🤳🏾' => 'selfi: keskmiselt tume nahatoon',
    '🤳🏿' => 'selfi: tume nahatoon',
    '💪🏻' => 'pingutatud biitseps: hele nahatoon',
    '💪🏼' => 'pingutatud biitseps: keskmiselt hele nahatoon',
    '💪🏽' => 'pingutatud biitseps: keskmine nahatoon',
    '💪🏾' => 'pingutatud biitseps: keskmiselt tume nahatoon',
    '💪🏿' => 'pingutatud biitseps: tume nahatoon',
    '🦵🏻' => 'jalg: hele nahatoon',
    '🦵🏼' => 'jalg: keskmiselt hele nahatoon',
    '🦵🏽' => 'jalg: keskmine nahatoon',
    '🦵🏾' => 'jalg: keskmiselt tume nahatoon',
    '🦵🏿' => 'jalg: tume nahatoon',
    '🦶🏻' => 'jalalaba: hele nahatoon',
    '🦶🏼' => 'jalalaba: keskmiselt hele nahatoon',
    '🦶🏽' => 'jalalaba: keskmine nahatoon',
    '🦶🏾' => 'jalalaba: keskmiselt tume nahatoon',
    '🦶🏿' => 'jalalaba: tume nahatoon',
    '👂🏻' => 'kõrv: hele nahatoon',
    '👂🏼' => 'kõrv: keskmiselt hele nahatoon',
    '👂🏽' => 'kõrv: keskmine nahatoon',
    '👂🏾' => 'kõrv: keskmiselt tume nahatoon',
    '👂🏿' => 'kõrv: tume nahatoon',
    '🦻🏻' => 'kuuldeaparaadiga kõrv: hele nahatoon',
    '🦻🏼' => 'kuuldeaparaadiga kõrv: keskmiselt hele nahatoon',
    '🦻🏽' => 'kuuldeaparaadiga kõrv: keskmine nahatoon',
    '🦻🏾' => 'kuuldeaparaadiga kõrv: keskmiselt tume nahatoon',
    '🦻🏿' => 'kuuldeaparaadiga kõrv: tume nahatoon',
    '👃🏻' => 'nina: hele nahatoon',
    '👃🏼' => 'nina: keskmiselt hele nahatoon',
    '👃🏽' => 'nina: keskmine nahatoon',
    '👃🏾' => 'nina: keskmiselt tume nahatoon',
    '👃🏿' => 'nina: tume nahatoon',
    '👶🏻' => 'beebi: hele nahatoon',
    '👶🏼' => 'beebi: keskmiselt hele nahatoon',
    '👶🏽' => 'beebi: keskmine nahatoon',
    '👶🏾' => 'beebi: keskmiselt tume nahatoon',
    '👶🏿' => 'beebi: tume nahatoon',
    '🧒🏻' => 'laps: hele nahatoon',
    '🧒🏼' => 'laps: keskmiselt hele nahatoon',
    '🧒🏽' => 'laps: keskmine nahatoon',
    '🧒🏾' => 'laps: keskmiselt tume nahatoon',
    '🧒🏿' => 'laps: tume nahatoon',
    '👦🏻' => 'poiss: hele nahatoon',
    '👦🏼' => 'poiss: keskmiselt hele nahatoon',
    '👦🏽' => 'poiss: keskmine nahatoon',
    '👦🏾' => 'poiss: keskmiselt tume nahatoon',
    '👦🏿' => 'poiss: tume nahatoon',
    '👧🏻' => 'tüdruk: hele nahatoon',
    '👧🏼' => 'tüdruk: keskmiselt hele nahatoon',
    '👧🏽' => 'tüdruk: keskmine nahatoon',
    '👧🏾' => 'tüdruk: keskmiselt tume nahatoon',
    '👧🏿' => 'tüdruk: tume nahatoon',
    '🧑🏻' => 'täiskasvanu: hele nahatoon',
    '🧑🏼' => 'täiskasvanu: keskmiselt hele nahatoon',
    '🧑🏽' => 'täiskasvanu: keskmine nahatoon',
    '🧑🏾' => 'täiskasvanu: keskmiselt tume nahatoon',
    '🧑🏿' => 'täiskasvanu: tume nahatoon',
    '👱🏻' => 'inimene: hele nahatoon, blondide juustega',
    '👱🏼' => 'inimene: keskmiselt hele nahatoon, blondide juustega',
    '👱🏽' => 'inimene: keskmine nahatoon, blondide juustega',
    '👱🏾' => 'inimene: keskmiselt tume nahatoon, blondide juustega',
    '👱🏿' => 'inimene: tume nahatoon, blondide juustega',
    '👨🏻' => 'mees: hele nahatoon',
    '👨🏼' => 'mees: keskmiselt hele nahatoon',
    '👨🏽' => 'mees: keskmine nahatoon',
    '👨🏾' => 'mees: keskmiselt tume nahatoon',
    '👨🏿' => 'mees: tume nahatoon',
    '🧔🏻' => 'mees: hele nahatoon, habemega',
    '🧔🏼' => 'mees: keskmiselt hele nahatoon, habemega',
    '🧔🏽' => 'mees: keskmine nahatoon, habemega',
    '🧔🏾' => 'mees: keskmiselt tume nahatoon, habemega',
    '🧔🏿' => 'mees: tume nahatoon, habemega',
    '👩🏻' => 'naine: hele nahatoon',
    '👩🏼' => 'naine: keskmiselt hele nahatoon',
    '👩🏽' => 'naine: keskmine nahatoon',
    '👩🏾' => 'naine: keskmiselt tume nahatoon',
    '👩🏿' => 'naine: tume nahatoon',
    '🧓🏻' => 'eakas: hele nahatoon',
    '🧓🏼' => 'eakas: keskmiselt hele nahatoon',
    '🧓🏽' => 'eakas: keskmine nahatoon',
    '🧓🏾' => 'eakas: keskmiselt tume nahatoon',
    '🧓🏿' => 'eakas: tume nahatoon',
    '👴🏻' => 'eakas mees: hele nahatoon',
    '👴🏼' => 'eakas mees: keskmiselt hele nahatoon',
    '👴🏽' => 'eakas mees: keskmine nahatoon',
    '👴🏾' => 'eakas mees: keskmiselt tume nahatoon',
    '👴🏿' => 'eakas mees: tume nahatoon',
    '👵🏻' => 'eakas naine: hele nahatoon',
    '👵🏼' => 'eakas naine: keskmiselt hele nahatoon',
    '👵🏽' => 'eakas naine: keskmine nahatoon',
    '👵🏾' => 'eakas naine: keskmiselt tume nahatoon',
    '👵🏿' => 'eakas naine: tume nahatoon',
    '🙍🏻' => 'kulmu kortsutav inimene: hele nahatoon',
    '🙍🏼' => 'kulmu kortsutav inimene: keskmiselt hele nahatoon',
    '🙍🏽' => 'kulmu kortsutav inimene: keskmine nahatoon',
    '🙍🏾' => 'kulmu kortsutav inimene: keskmiselt tume nahatoon',
    '🙍🏿' => 'kulmu kortsutav inimene: tume nahatoon',
    '🙎🏻' => 'mossitav inimene: hele nahatoon',
    '🙎🏼' => 'mossitav inimene: keskmiselt hele nahatoon',
    '🙎🏽' => 'mossitav inimene: keskmine nahatoon',
    '🙎🏾' => 'mossitav inimene: keskmiselt tume nahatoon',
    '🙎🏿' => 'mossitav inimene: tume nahatoon',
    '🙅🏻' => 'eitav žest: hele nahatoon',
    '🙅🏼' => 'eitav žest: keskmiselt hele nahatoon',
    '🙅🏽' => 'eitav žest: keskmine nahatoon',
    '🙅🏾' => 'eitav žest: keskmiselt tume nahatoon',
    '🙅🏿' => 'eitav žest: tume nahatoon',
    '🙆🏻' => 'nõustuv žest: hele nahatoon',
    '🙆🏼' => 'nõustuv žest: keskmiselt hele nahatoon',
    '🙆🏽' => 'nõustuv žest: keskmine nahatoon',
    '🙆🏾' => 'nõustuv žest: keskmiselt tume nahatoon',
    '🙆🏿' => 'nõustuv žest: tume nahatoon',
    '💁🏻' => 'infoleti töötaja: hele nahatoon',
    '💁🏼' => 'infoleti töötaja: keskmiselt hele nahatoon',
    '💁🏽' => 'infoleti töötaja: keskmine nahatoon',
    '💁🏾' => 'infoleti töötaja: keskmiselt tume nahatoon',
    '💁🏿' => 'infoleti töötaja: tume nahatoon',
    '🙋🏻' => 'kätt tõstev rõõmus inimene: hele nahatoon',
    '🙋🏼' => 'kätt tõstev rõõmus inimene: keskmiselt hele nahatoon',
    '🙋🏽' => 'kätt tõstev rõõmus inimene: keskmine nahatoon',
    '🙋🏾' => 'kätt tõstev rõõmus inimene: keskmiselt tume nahatoon',
    '🙋🏿' => 'kätt tõstev rõõmus inimene: tume nahatoon',
    '🧏🏻' => 'kurt inimene: hele nahatoon',
    '🧏🏼' => 'kurt inimene: keskmiselt hele nahatoon',
    '🧏🏽' => 'kurt inimene: keskmine nahatoon',
    '🧏🏾' => 'kurt inimene: keskmiselt tume nahatoon',
    '🧏🏿' => 'kurt inimene: tume nahatoon',
    '🙇🏻' => 'kummardav inimene: hele nahatoon',
    '🙇🏼' => 'kummardav inimene: keskmiselt hele nahatoon',
    '🙇🏽' => 'kummardav inimene: keskmine nahatoon',
    '🙇🏾' => 'kummardav inimene: keskmiselt tume nahatoon',
    '🙇🏿' => 'kummardav inimene: tume nahatoon',
    '🤦🏻' => 'meeleheide: hele nahatoon',
    '🤦🏼' => 'meeleheide: keskmiselt hele nahatoon',
    '🤦🏽' => 'meeleheide: keskmine nahatoon',
    '🤦🏾' => 'meeleheide: keskmiselt tume nahatoon',
    '🤦🏿' => 'meeleheide: tume nahatoon',
    '🤷🏻' => 'õlakehitus: hele nahatoon',
    '🤷🏼' => 'õlakehitus: keskmiselt hele nahatoon',
    '🤷🏽' => 'õlakehitus: keskmine nahatoon',
    '🤷🏾' => 'õlakehitus: keskmiselt tume nahatoon',
    '🤷🏿' => 'õlakehitus: tume nahatoon',
    '👮🏻' => 'politseinik: hele nahatoon',
    '👮🏼' => 'politseinik: keskmiselt hele nahatoon',
    '👮🏽' => 'politseinik: keskmine nahatoon',
    '👮🏾' => 'politseinik: keskmiselt tume nahatoon',
    '👮🏿' => 'politseinik: tume nahatoon',
    '🕵🏻' => 'detektiiv: hele nahatoon',
    '🕵🏼' => 'detektiiv: keskmiselt hele nahatoon',
    '🕵🏽' => 'detektiiv: keskmine nahatoon',
    '🕵🏾' => 'detektiiv: keskmiselt tume nahatoon',
    '🕵🏿' => 'detektiiv: tume nahatoon',
    '💂🏻' => 'valvur: hele nahatoon',
    '💂🏼' => 'valvur: keskmiselt hele nahatoon',
    '💂🏽' => 'valvur: keskmine nahatoon',
    '💂🏾' => 'valvur: keskmiselt tume nahatoon',
    '💂🏿' => 'valvur: tume nahatoon',
    '🥷🏻' => 'ninja: hele nahatoon',
    '🥷🏼' => 'ninja: keskmiselt hele nahatoon',
    '🥷🏽' => 'ninja: keskmine nahatoon',
    '🥷🏾' => 'ninja: keskmiselt tume nahatoon',
    '🥷🏿' => 'ninja: tume nahatoon',
    '👷🏻' => 'ehitaja: hele nahatoon',
    '👷🏼' => 'ehitaja: keskmiselt hele nahatoon',
    '👷🏽' => 'ehitaja: keskmine nahatoon',
    '👷🏾' => 'ehitaja: keskmiselt tume nahatoon',
    '👷🏿' => 'ehitaja: tume nahatoon',
    '🫅🏻' => 'krooniga inimene: hele nahatoon',
    '🫅🏼' => 'krooniga inimene: keskmiselt hele nahatoon',
    '🫅🏽' => 'krooniga inimene: keskmine nahatoon',
    '🫅🏾' => 'krooniga inimene: keskmiselt tume nahatoon',
    '🫅🏿' => 'krooniga inimene: tume nahatoon',
    '🤴🏻' => 'prints: hele nahatoon',
    '🤴🏼' => 'prints: keskmiselt hele nahatoon',
    '🤴🏽' => 'prints: keskmine nahatoon',
    '🤴🏾' => 'prints: keskmiselt tume nahatoon',
    '🤴🏿' => 'prints: tume nahatoon',
    '👸🏻' => 'printsess: hele nahatoon',
    '👸🏼' => 'printsess: keskmiselt hele nahatoon',
    '👸🏽' => 'printsess: keskmine nahatoon',
    '👸🏾' => 'printsess: keskmiselt tume nahatoon',
    '👸🏿' => 'printsess: tume nahatoon',
    '👳🏻' => 'turbaniga inimene: hele nahatoon',
    '👳🏼' => 'turbaniga inimene: keskmiselt hele nahatoon',
    '👳🏽' => 'turbaniga inimene: keskmine nahatoon',
    '👳🏾' => 'turbaniga inimene: keskmiselt tume nahatoon',
    '👳🏿' => 'turbaniga inimene: tume nahatoon',
    '👲🏻' => 'Hiina mütsiga mees: hele nahatoon',
    '👲🏼' => 'Hiina mütsiga mees: keskmiselt hele nahatoon',
    '👲🏽' => 'Hiina mütsiga mees: keskmine nahatoon',
    '👲🏾' => 'Hiina mütsiga mees: keskmiselt tume nahatoon',
    '👲🏿' => 'Hiina mütsiga mees: tume nahatoon',
    '🧕🏻' => 'pearätiga naine: hele nahatoon',
    '🧕🏼' => 'pearätiga naine: keskmiselt hele nahatoon',
    '🧕🏽' => 'pearätiga naine: keskmine nahatoon',
    '🧕🏾' => 'pearätiga naine: keskmiselt tume nahatoon',
    '🧕🏿' => 'pearätiga naine: tume nahatoon',
    '🤵🏻' => 'inimene smokingus: hele nahatoon',
    '🤵🏼' => 'inimene smokingus: keskmiselt hele nahatoon',
    '🤵🏽' => 'inimene smokingus: keskmine nahatoon',
    '🤵🏾' => 'inimene smokingus: keskmiselt tume nahatoon',
    '🤵🏿' => 'inimene smokingus: tume nahatoon',
    '👰🏻' => 'loori kandev inimene: hele nahatoon',
    '👰🏼' => 'loori kandev inimene: keskmiselt hele nahatoon',
    '👰🏽' => 'loori kandev inimene: keskmine nahatoon',
    '👰🏾' => 'loori kandev inimene: keskmiselt tume nahatoon',
    '👰🏿' => 'loori kandev inimene: tume nahatoon',
    '🤰🏻' => 'rase naine: hele nahatoon',
    '🤰🏼' => 'rase naine: keskmiselt hele nahatoon',
    '🤰🏽' => 'rase naine: keskmine nahatoon',
    '🤰🏾' => 'rase naine: keskmiselt tume nahatoon',
    '🤰🏿' => 'rase naine: tume nahatoon',
    '🫃🏻' => 'rase mees: hele nahatoon',
    '🫃🏼' => 'rase mees: keskmiselt hele nahatoon',
    '🫃🏽' => 'rase mees: keskmine nahatoon',
    '🫃🏾' => 'rase mees: keskmiselt tume nahatoon',
    '🫃🏿' => 'rase mees: tume nahatoon',
    '🫄🏻' => 'rase inimene: hele nahatoon',
    '🫄🏼' => 'rase inimene: keskmiselt hele nahatoon',
    '🫄🏽' => 'rase inimene: keskmine nahatoon',
    '🫄🏾' => 'rase inimene: keskmiselt tume nahatoon',
    '🫄🏿' => 'rase inimene: tume nahatoon',
    '🤱🏻' => 'rinnaga toitmine: hele nahatoon',
    '🤱🏼' => 'rinnaga toitmine: keskmiselt hele nahatoon',
    '🤱🏽' => 'rinnaga toitmine: keskmine nahatoon',
    '🤱🏾' => 'rinnaga toitmine: keskmiselt tume nahatoon',
    '🤱🏿' => 'rinnaga toitmine: tume nahatoon',
    '👼🏻' => 'inglibeebi: hele nahatoon',
    '👼🏼' => 'inglibeebi: keskmiselt hele nahatoon',
    '👼🏽' => 'inglibeebi: keskmine nahatoon',
    '👼🏾' => 'inglibeebi: keskmiselt tume nahatoon',
    '👼🏿' => 'inglibeebi: tume nahatoon',
    '🎅🏻' => 'jõuluvana: hele nahatoon',
    '🎅🏼' => 'jõuluvana: keskmiselt hele nahatoon',
    '🎅🏽' => 'jõuluvana: keskmine nahatoon',
    '🎅🏾' => 'jõuluvana: keskmiselt tume nahatoon',
    '🎅🏿' => 'jõuluvana: tume nahatoon',
    '🤶🏻' => 'jõuluvana abikaasa: hele nahatoon',
    '🤶🏼' => 'jõuluvana abikaasa: keskmiselt hele nahatoon',
    '🤶🏽' => 'jõuluvana abikaasa: keskmine nahatoon',
    '🤶🏾' => 'jõuluvana abikaasa: keskmiselt tume nahatoon',
    '🤶🏿' => 'jõuluvana abikaasa: tume nahatoon',
    '🦸🏻' => 'superkangelane: hele nahatoon',
    '🦸🏼' => 'superkangelane: keskmiselt hele nahatoon',
    '🦸🏽' => 'superkangelane: keskmine nahatoon',
    '🦸🏾' => 'superkangelane: keskmiselt tume nahatoon',
    '🦸🏿' => 'superkangelane: tume nahatoon',
    '🦹🏻' => 'superkurikael: hele nahatoon',
    '🦹🏼' => 'superkurikael: keskmiselt hele nahatoon',
    '🦹🏽' => 'superkurikael: keskmine nahatoon',
    '🦹🏾' => 'superkurikael: keskmiselt tume nahatoon',
    '🦹🏿' => 'superkurikael: tume nahatoon',
    '🧙🏻' => 'maag: hele nahatoon',
    '🧙🏼' => 'maag: keskmiselt hele nahatoon',
    '🧙🏽' => 'maag: keskmine nahatoon',
    '🧙🏾' => 'maag: keskmiselt tume nahatoon',
    '🧙🏿' => 'maag: tume nahatoon',
    '🧚🏻' => 'haldjas: hele nahatoon',
    '🧚🏼' => 'haldjas: keskmiselt hele nahatoon',
    '🧚🏽' => 'haldjas: keskmine nahatoon',
    '🧚🏾' => 'haldjas: keskmiselt tume nahatoon',
    '🧚🏿' => 'haldjas: tume nahatoon',
    '🧛🏻' => 'vampiir: hele nahatoon',
    '🧛🏼' => 'vampiir: keskmiselt hele nahatoon',
    '🧛🏽' => 'vampiir: keskmine nahatoon',
    '🧛🏾' => 'vampiir: keskmiselt tume nahatoon',
    '🧛🏿' => 'vampiir: tume nahatoon',
    '🧜🏻' => 'näkk: hele nahatoon',
    '🧜🏼' => 'näkk: keskmiselt hele nahatoon',
    '🧜🏽' => 'näkk: keskmine nahatoon',
    '🧜🏾' => 'näkk: keskmiselt tume nahatoon',
    '🧜🏿' => 'näkk: tume nahatoon',
    '🧝🏻' => 'päkapikk: hele nahatoon',
    '🧝🏼' => 'päkapikk: keskmiselt hele nahatoon',
    '🧝🏽' => 'päkapikk: keskmine nahatoon',
    '🧝🏾' => 'päkapikk: keskmiselt tume nahatoon',
    '🧝🏿' => 'päkapikk: tume nahatoon',
    '💆🏻' => 'näomassaaž: hele nahatoon',
    '💆🏼' => 'näomassaaž: keskmiselt hele nahatoon',
    '💆🏽' => 'näomassaaž: keskmine nahatoon',
    '💆🏾' => 'näomassaaž: keskmiselt tume nahatoon',
    '💆🏿' => 'näomassaaž: tume nahatoon',
    '💇🏻' => 'juukselõikus: hele nahatoon',
    '💇🏼' => 'juukselõikus: keskmiselt hele nahatoon',
    '💇🏽' => 'juukselõikus: keskmine nahatoon',
    '💇🏾' => 'juukselõikus: keskmiselt tume nahatoon',
    '💇🏿' => 'juukselõikus: tume nahatoon',
    '🚶🏻' => 'jalakäija: hele nahatoon',
    '🚶🏼' => 'jalakäija: keskmiselt hele nahatoon',
    '🚶🏽' => 'jalakäija: keskmine nahatoon',
    '🚶🏾' => 'jalakäija: keskmiselt tume nahatoon',
    '🚶🏿' => 'jalakäija: tume nahatoon',
    '🧍🏻' => 'seisev inimene: hele nahatoon',
    '🧍🏼' => 'seisev inimene: keskmiselt hele nahatoon',
    '🧍🏽' => 'seisev inimene: keskmine nahatoon',
    '🧍🏾' => 'seisev inimene: keskmiselt tume nahatoon',
    '🧍🏿' => 'seisev inimene: tume nahatoon',
    '🧎🏻' => 'põlvitav inimene: hele nahatoon',
    '🧎🏼' => 'põlvitav inimene: keskmiselt hele nahatoon',
    '🧎🏽' => 'põlvitav inimene: keskmine nahatoon',
    '🧎🏾' => 'põlvitav inimene: keskmiselt tume nahatoon',
    '🧎🏿' => 'põlvitav inimene: tume nahatoon',
    '🏃🏻' => 'jooksja: hele nahatoon',
    '🏃🏼' => 'jooksja: keskmiselt hele nahatoon',
    '🏃🏽' => 'jooksja: keskmine nahatoon',
    '🏃🏾' => 'jooksja: keskmiselt tume nahatoon',
    '🏃🏿' => 'jooksja: tume nahatoon',
    '💃🏻' => 'tantsiv naine: hele nahatoon',
    '💃🏼' => 'tantsiv naine: keskmiselt hele nahatoon',
    '💃🏽' => 'tantsiv naine: keskmine nahatoon',
    '💃🏾' => 'tantsiv naine: keskmiselt tume nahatoon',
    '💃🏿' => 'tantsiv naine: tume nahatoon',
    '🕺🏻' => 'tantsiv mees: hele nahatoon',
    '🕺🏼' => 'tantsiv mees: keskmiselt hele nahatoon',
    '🕺🏽' => 'tantsiv mees: keskmine nahatoon',
    '🕺🏾' => 'tantsiv mees: keskmiselt tume nahatoon',
    '🕺🏿' => 'tantsiv mees: tume nahatoon',
    '🕴🏻' => 'hõljuv ülikonnaga mees: hele nahatoon',
    '🕴🏼' => 'hõljuv ülikonnaga mees: keskmiselt hele nahatoon',
    '🕴🏽' => 'hõljuv ülikonnaga mees: keskmine nahatoon',
    '🕴🏾' => 'hõljuv ülikonnaga mees: keskmiselt tume nahatoon',
    '🕴🏿' => 'hõljuv ülikonnaga mees: tume nahatoon',
    '👯🏻' => 'pidutsevad jänesekõrvadega inimesed: hele nahatoon',
    '👯🏼' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt hele nahatoon',
    '👯🏽' => 'pidutsevad jänesekõrvadega inimesed: keskmine nahatoon',
    '👯🏾' => 'pidutsevad jänesekõrvadega inimesed: keskmiselt tume nahatoon',
    '👯🏿' => 'pidutsevad jänesekõrvadega inimesed: tume nahatoon',
    '🧖🏻' => 'inimene leiliruumis: hele nahatoon',
    '🧖🏼' => 'inimene leiliruumis: keskmiselt hele nahatoon',
    '🧖🏽' => 'inimene leiliruumis: keskmine nahatoon',
    '🧖🏾' => 'inimene leiliruumis: keskmiselt tume nahatoon',
    '🧖🏿' => 'inimene leiliruumis: tume nahatoon',
    '🧗🏻' => 'mäkke roniv inimene: hele nahatoon',
    '🧗🏼' => 'mäkke roniv inimene: keskmiselt hele nahatoon',
    '🧗🏽' => 'mäkke roniv inimene: keskmine nahatoon',
    '🧗🏾' => 'mäkke roniv inimene: keskmiselt tume nahatoon',
    '🧗🏿' => 'mäkke roniv inimene: tume nahatoon',
    '🏇🏻' => 'hobuste võiduajamine: hele nahatoon',
    '🏇🏼' => 'hobuste võiduajamine: keskmiselt hele nahatoon',
    '🏇🏽' => 'hobuste võiduajamine: keskmine nahatoon',
    '🏇🏾' => 'hobuste võiduajamine: keskmiselt tume nahatoon',
    '🏇🏿' => 'hobuste võiduajamine: tume nahatoon',
    '🏂🏻' => 'lumelaudur: hele nahatoon',
    '🏂🏼' => 'lumelaudur: keskmiselt hele nahatoon',
    '🏂🏽' => 'lumelaudur: keskmine nahatoon',
    '🏂🏾' => 'lumelaudur: keskmiselt tume nahatoon',
    '🏂🏿' => 'lumelaudur: tume nahatoon',
    '🏌🏻' => 'golfimängija: hele nahatoon',
    '🏌🏼' => 'golfimängija: keskmiselt hele nahatoon',
    '🏌🏽' => 'golfimängija: keskmine nahatoon',
    '🏌🏾' => 'golfimängija: keskmiselt tume nahatoon',
    '🏌🏿' => 'golfimängija: tume nahatoon',
    '🏄🏻' => 'surfar: hele nahatoon',
    '🏄🏼' => 'surfar: keskmiselt hele nahatoon',
    '🏄🏽' => 'surfar: keskmine nahatoon',
    '🏄🏾' => 'surfar: keskmiselt tume nahatoon',
    '🏄🏿' => 'surfar: tume nahatoon',
    '🚣🏻' => 'aerupaadiga inimene: hele nahatoon',
    '🚣🏼' => 'aerupaadiga inimene: keskmiselt hele nahatoon',
    '🚣🏽' => 'aerupaadiga inimene: keskmine nahatoon',
    '🚣🏾' => 'aerupaadiga inimene: keskmiselt tume nahatoon',
    '🚣🏿' => 'aerupaadiga inimene: tume nahatoon',
    '🏊🏻' => 'ujuja: hele nahatoon',
    '🏊🏼' => 'ujuja: keskmiselt hele nahatoon',
    '🏊🏽' => 'ujuja: keskmine nahatoon',
    '🏊🏾' => 'ujuja: keskmiselt tume nahatoon',
    '🏊🏿' => 'ujuja: tume nahatoon',
    '⛹🏻' => 'inimene palliga: hele nahatoon',
    '⛹🏼' => 'inimene palliga: keskmiselt hele nahatoon',
    '⛹🏽' => 'inimene palliga: keskmine nahatoon',
    '⛹🏾' => 'inimene palliga: keskmiselt tume nahatoon',
    '⛹🏿' => 'inimene palliga: tume nahatoon',
    '🏋🏻' => 'jõutõstja: hele nahatoon',
    '🏋🏼' => 'jõutõstja: keskmiselt hele nahatoon',
    '🏋🏽' => 'jõutõstja: keskmine nahatoon',
    '🏋🏾' => 'jõutõstja: keskmiselt tume nahatoon',
    '🏋🏿' => 'jõutõstja: tume nahatoon',
    '🚴🏻' => 'jalgrattur: hele nahatoon',
    '🚴🏼' => 'jalgrattur: keskmiselt hele nahatoon',
    '🚴🏽' => 'jalgrattur: keskmine nahatoon',
    '🚴🏾' => 'jalgrattur: keskmiselt tume nahatoon',
    '🚴🏿' => 'jalgrattur: tume nahatoon',
    '🚵🏻' => 'mägirattur: hele nahatoon',
    '🚵🏼' => 'mägirattur: keskmiselt hele nahatoon',
    '🚵🏽' => 'mägirattur: keskmine nahatoon',
    '🚵🏾' => 'mägirattur: keskmiselt tume nahatoon',
    '🚵🏿' => 'mägirattur: tume nahatoon',
    '🤸🏻' => 'hundiratast viskav inimene: hele nahatoon',
    '🤸🏼' => 'hundiratast viskav inimene: keskmiselt hele nahatoon',
    '🤸🏽' => 'hundiratast viskav inimene: keskmine nahatoon',
    '🤸🏾' => 'hundiratast viskav inimene: keskmiselt tume nahatoon',
    '🤸🏿' => 'hundiratast viskav inimene: tume nahatoon',
    '🤼🏻' => 'maadlejad: hele nahatoon',
    '🤼🏼' => 'maadlejad: keskmiselt hele nahatoon',
    '🤼🏽' => 'maadlejad: keskmine nahatoon',
    '🤼🏾' => 'maadlejad: keskmiselt tume nahatoon',
    '🤼🏿' => 'maadlejad: tume nahatoon',
    '🤽🏻' => 'veepalli mängiv inimene: hele nahatoon',
    '🤽🏼' => 'veepalli mängiv inimene: keskmiselt hele nahatoon',
    '🤽🏽' => 'veepalli mängiv inimene: keskmine nahatoon',
    '🤽🏾' => 'veepalli mängiv inimene: keskmiselt tume nahatoon',
    '🤽🏿' => 'veepalli mängiv inimene: tume nahatoon',
    '🤾🏻' => 'käsipalli mängiv inimene: hele nahatoon',
    '🤾🏼' => 'käsipalli mängiv inimene: keskmiselt hele nahatoon',
    '🤾🏽' => 'käsipalli mängiv inimene: keskmine nahatoon',
    '🤾🏾' => 'käsipalli mängiv inimene: keskmiselt tume nahatoon',
    '🤾🏿' => 'käsipalli mängiv inimene: tume nahatoon',
    '🤹🏻' => 'žonglöör: hele nahatoon',
    '🤹🏼' => 'žonglöör: keskmiselt hele nahatoon',
    '🤹🏽' => 'žonglöör: keskmine nahatoon',
    '🤹🏾' => 'žonglöör: keskmiselt tume nahatoon',
    '🤹🏿' => 'žonglöör: tume nahatoon',
    '🧘🏻' => 'lootosasendis inimene: hele nahatoon',
    '🧘🏼' => 'lootosasendis inimene: keskmiselt hele nahatoon',
    '🧘🏽' => 'lootosasendis inimene: keskmine nahatoon',
    '🧘🏾' => 'lootosasendis inimene: keskmiselt tume nahatoon',
    '🧘🏿' => 'lootosasendis inimene: tume nahatoon',
    '🛀🏻' => 'inimene vannis: hele nahatoon',
    '🛀🏼' => 'inimene vannis: keskmiselt hele nahatoon',
    '🛀🏽' => 'inimene vannis: keskmine nahatoon',
    '🛀🏾' => 'inimene vannis: keskmiselt tume nahatoon',
    '🛀🏿' => 'inimene vannis: tume nahatoon',
    '🛌🏻' => 'inimene voodis: hele nahatoon',
    '🛌🏼' => 'inimene voodis: keskmiselt hele nahatoon',
    '🛌🏽' => 'inimene voodis: keskmine nahatoon',
    '🛌🏾' => 'inimene voodis: keskmiselt tume nahatoon',
    '🛌🏿' => 'inimene voodis: tume nahatoon',
    '👭🏻' => 'käest kinni hoidvad naised: hele nahatoon',
    '👭🏼' => 'käest kinni hoidvad naised: keskmiselt hele nahatoon',
    '👭🏽' => 'käest kinni hoidvad naised: keskmine nahatoon',
    '👭🏾' => 'käest kinni hoidvad naised: keskmiselt tume nahatoon',
    '👭🏿' => 'käest kinni hoidvad naised: tume nahatoon',
    '👫🏻' => 'mees ja naine hoiavad käest kinni: hele nahatoon',
    '👫🏼' => 'mees ja naine hoiavad käest kinni: keskmiselt hele nahatoon',
    '👫🏽' => 'mees ja naine hoiavad käest kinni: keskmine nahatoon',
    '👫🏾' => 'mees ja naine hoiavad käest kinni: keskmiselt tume nahatoon',
    '👫🏿' => 'mees ja naine hoiavad käest kinni: tume nahatoon',
    '👬🏻' => 'käest kinni hoidvad mehed: hele nahatoon',
    '👬🏼' => 'käest kinni hoidvad mehed: keskmiselt hele nahatoon',
    '👬🏽' => 'käest kinni hoidvad mehed: keskmine nahatoon',
    '👬🏾' => 'käest kinni hoidvad mehed: keskmiselt tume nahatoon',
    '👬🏿' => 'käest kinni hoidvad mehed: tume nahatoon',
    '💏🏻' => 'musi: hele nahatoon',
    '💏🏼' => 'musi: keskmiselt hele nahatoon',
    '💏🏽' => 'musi: keskmine nahatoon',
    '💏🏾' => 'musi: keskmiselt tume nahatoon',
    '💏🏿' => 'musi: tume nahatoon',
    '💑🏻' => 'paarike südamega: hele nahatoon',
    '💑🏼' => 'paarike südamega: keskmiselt hele nahatoon',
    '💑🏽' => 'paarike südamega: keskmine nahatoon',
    '💑🏾' => 'paarike südamega: keskmiselt tume nahatoon',
    '💑🏿' => 'paarike südamega: tume nahatoon',
    '🇦🇨' => 'lipp: Ascensioni saar',
    '🇦🇩' => 'lipp: Andorra',
    '🇦🇪' => 'lipp: Araabia Ühendemiraadid',
    '🇦🇫' => 'lipp: Afganistan',
    '🇦🇬' => 'lipp: Antigua ja Barbuda',
    '🇦🇮' => 'lipp: Anguilla',
    '🇦🇱' => 'lipp: Albaania',
    '🇦🇲' => 'lipp: Armeenia',
    '🇦🇴' => 'lipp: Angola',
    '🇦🇶' => 'lipp: Antarktika',
    '🇦🇷' => 'lipp: Argentina',
    '🇦🇸' => 'lipp: Ameerika Samoa',
    '🇦🇹' => 'lipp: Austria',
    '🇦🇺' => 'lipp: Austraalia',
    '🇦🇼' => 'lipp: Aruba',
    '🇦🇽' => 'lipp: Ahvenamaa',
    '🇦🇿' => 'lipp: Aserbaidžaan',
    '🇧🇦' => 'lipp: Bosnia ja Hertsegoviina',
    '🇧🇧' => 'lipp: Barbados',
    '🇧🇩' => 'lipp: Bangladesh',
    '🇧🇪' => 'lipp: Belgia',
    '🇧🇫' => 'lipp: Burkina Faso',
    '🇧🇬' => 'lipp: Bulgaaria',
    '🇧🇭' => 'lipp: Bahrein',
    '🇧🇮' => 'lipp: Burundi',
    '🇧🇯' => 'lipp: Benin',
    '🇧🇱' => 'lipp: Saint-Barthélemy',
    '🇧🇲' => 'lipp: Bermuda',
    '🇧🇳' => 'lipp: Brunei',
    '🇧🇴' => 'lipp: Boliivia',
    '🇧🇶' => 'lipp: Kariibi Madalmaad',
    '🇧🇷' => 'lipp: Brasiilia',
    '🇧🇸' => 'lipp: Bahama',
    '🇧🇹' => 'lipp: Bhutan',
    '🇧🇻' => 'lipp: Bouvet’ saar',
    '🇧🇼' => 'lipp: Botswana',
    '🇧🇾' => 'lipp: Valgevene',
    '🇧🇿' => 'lipp: Belize',
    '🇨🇦' => 'lipp: Kanada',
    '🇨🇨' => 'lipp: Kookossaared',
    '🇨🇩' => 'lipp: Kongo DV',
    '🇨🇫' => 'lipp: Kesk-Aafrika Vabariik',
    '🇨🇬' => 'lipp: Kongo Vabariik',
    '🇨🇭' => 'lipp: Šveits',
    '🇨🇮' => 'lipp: Elevandiluurannik',
    '🇨🇰' => 'lipp: Cooki saared',
    '🇨🇱' => 'lipp: Tšiili',
    '🇨🇲' => 'lipp: Kamerun',
    '🇨🇳' => 'lipp: Hiina',
    '🇨🇴' => 'lipp: Colombia',
    '🇨🇵' => 'lipp: Clippertoni saar',
    '🇨🇶' => 'lipp: Sark',
    '🇨🇷' => 'lipp: Costa Rica',
    '🇨🇺' => 'lipp: Kuuba',
    '🇨🇻' => 'lipp: Roheneemesaared',
    '🇨🇼' => 'lipp: Curaçao',
    '🇨🇽' => 'lipp: Jõulusaar',
    '🇨🇾' => 'lipp: Küpros',
    '🇨🇿' => 'lipp: Tšehhi',
    '🇩🇪' => 'lipp: Saksamaa',
    '🇩🇬' => 'lipp: Diego Garcia',
    '🇩🇯' => 'lipp: Djibouti',
    '🇩🇰' => 'lipp: Taani',
    '🇩🇲' => 'lipp: Dominica',
    '🇩🇴' => 'lipp: Dominikaani Vabariik',
    '🇩🇿' => 'lipp: Alžeeria',
    '🇪🇦' => 'lipp: Ceuta ja Melilla',
    '🇪🇨' => 'lipp: Ecuador',
    '🇪🇪' => 'lipp: Eesti',
    '🇪🇬' => 'lipp: Egiptus',
    '🇪🇭' => 'lipp: Lääne-Sahara',
    '🇪🇷' => 'lipp: Eritrea',
    '🇪🇸' => 'lipp: Hispaania',
    '🇪🇹' => 'lipp: Etioopia',
    '🇪🇺' => 'lipp: Euroopa Liit',
    '🇫🇮' => 'lipp: Soome',
    '🇫🇯' => 'lipp: Fidži',
    '🇫🇰' => 'lipp: Falklandi saared',
    '🇫🇲' => 'lipp: Mikroneesia',
    '🇫🇴' => 'lipp: Fääri saared',
    '🇫🇷' => 'lipp: Prantsusmaa',
    '🇬🇦' => 'lipp: Gabon',
    '🇬🇧' => 'lipp: Ühendkuningriik',
    '🇬🇩' => 'lipp: Grenada',
    '🇬🇪' => 'lipp: Gruusia',
    '🇬🇫' => 'lipp: Prantsuse Guajaana',
    '🇬🇬' => 'lipp: Guernsey',
    '🇬🇭' => 'lipp: Ghana',
    '🇬🇮' => 'lipp: Gibraltar',
    '🇬🇱' => 'lipp: Gröönimaa',
    '🇬🇲' => 'lipp: Gambia',
    '🇬🇳' => 'lipp: Guinea',
    '🇬🇵' => 'lipp: Guadeloupe',
    '🇬🇶' => 'lipp: Ekvatoriaal-Guinea',
    '🇬🇷' => 'lipp: Kreeka',
    '🇬🇸' => 'lipp: Lõuna-Georgia ja Lõuna-Sandwichi saared',
    '🇬🇹' => 'lipp: Guatemala',
    '🇬🇺' => 'lipp: Guam',
    '🇬🇼' => 'lipp: Guinea-Bissau',
    '🇬🇾' => 'lipp: Guyana',
    '🇭🇰' => 'lipp: Hongkongi erihalduspiirkond',
    '🇭🇲' => 'lipp: Heardi ja McDonaldi saared',
    '🇭🇳' => 'lipp: Honduras',
    '🇭🇷' => 'lipp: Horvaatia',
    '🇭🇹' => 'lipp: Haiti',
    '🇭🇺' => 'lipp: Ungari',
    '🇮🇨' => 'lipp: Kanaari saared',
    '🇮🇩' => 'lipp: Indoneesia',
    '🇮🇪' => 'lipp: Iirimaa',
    '🇮🇱' => 'lipp: Iisrael',
    '🇮🇲' => 'lipp: Mani saar',
    '🇮🇳' => 'lipp: India',
    '🇮🇴' => 'lipp: Briti India ookeani ala',
    '🇮🇶' => 'lipp: Iraak',
    '🇮🇷' => 'lipp: Iraan',
    '🇮🇸' => 'lipp: Island',
    '🇮🇹' => 'lipp: Itaalia',
    '🇯🇪' => 'lipp: Jersey',
    '🇯🇲' => 'lipp: Jamaica',
    '🇯🇴' => 'lipp: Jordaania',
    '🇯🇵' => 'lipp: Jaapan',
    '🇰🇪' => 'lipp: Keenia',
    '🇰🇬' => 'lipp: Kõrgõzstan',
    '🇰🇭' => 'lipp: Kambodža',
    '🇰🇮' => 'lipp: Kiribati',
    '🇰🇲' => 'lipp: Komoorid',
    '🇰🇳' => 'lipp: Saint Kitts ja Nevis',
    '🇰🇵' => 'lipp: Põhja-Korea',
    '🇰🇷' => 'lipp: Lõuna-Korea',
    '🇰🇼' => 'lipp: Kuveit',
    '🇰🇾' => 'lipp: Kaimanisaared',
    '🇰🇿' => 'lipp: Kasahstan',
    '🇱🇦' => 'lipp: Laos',
    '🇱🇧' => 'lipp: Liibanon',
    '🇱🇨' => 'lipp: Saint Lucia',
    '🇱🇮' => 'lipp: Liechtenstein',
    '🇱🇰' => 'lipp: Sri Lanka',
    '🇱🇷' => 'lipp: Libeeria',
    '🇱🇸' => 'lipp: Lesotho',
    '🇱🇹' => 'lipp: Leedu',
    '🇱🇺' => 'lipp: Luksemburg',
    '🇱🇻' => 'lipp: Läti',
    '🇱🇾' => 'lipp: Liibüa',
    '🇲🇦' => 'lipp: Maroko',
    '🇲🇨' => 'lipp: Monaco',
    '🇲🇩' => 'lipp: Moldova',
    '🇲🇪' => 'lipp: Montenegro',
    '🇲🇫' => 'lipp: Saint-Martin',
    '🇲🇬' => 'lipp: Madagaskar',
    '🇲🇭' => 'lipp: Marshalli Saared',
    '🇲🇰' => 'lipp: Põhja-Makedoonia',
    '🇲🇱' => 'lipp: Mali',
    '🇲🇲' => 'lipp: Myanmar (Birma)',
    '🇲🇳' => 'lipp: Mongoolia',
    '🇲🇴' => 'lipp: Macau erihalduspiirkond',
    '🇲🇵' => 'lipp: Põhja-Mariaanid',
    '🇲🇶' => 'lipp: Martinique',
    '🇲🇷' => 'lipp: Mauritaania',
    '🇲🇸' => 'lipp: Montserrat',
    '🇲🇹' => 'lipp: Malta',
    '🇲🇺' => 'lipp: Mauritius',
    '🇲🇻' => 'lipp: Maldiivid',
    '🇲🇼' => 'lipp: Malawi',
    '🇲🇽' => 'lipp: Mehhiko',
    '🇲🇾' => 'lipp: Malaisia',
    '🇲🇿' => 'lipp: Mosambiik',
    '🇳🇦' => 'lipp: Namiibia',
    '🇳🇨' => 'lipp: Uus-Kaledoonia',
    '🇳🇪' => 'lipp: Niger',
    '🇳🇫' => 'lipp: Norfolk',
    '🇳🇬' => 'lipp: Nigeeria',
    '🇳🇮' => 'lipp: Nicaragua',
    '🇳🇱' => 'lipp: Holland',
    '🇳🇴' => 'lipp: Norra',
    '🇳🇵' => 'lipp: Nepal',
    '🇳🇷' => 'lipp: Nauru',
    '🇳🇺' => 'lipp: Niue',
    '🇳🇿' => 'lipp: Uus-Meremaa',
    '🇴🇲' => 'lipp: Omaan',
    '🇵🇦' => 'lipp: Panama',
    '🇵🇪' => 'lipp: Peruu',
    '🇵🇫' => 'lipp: Prantsuse Polüneesia',
    '🇵🇬' => 'lipp: Paapua Uus-Guinea',
    '🇵🇭' => 'lipp: Filipiinid',
    '🇵🇰' => 'lipp: Pakistan',
    '🇵🇱' => 'lipp: Poola',
    '🇵🇲' => 'lipp: Saint-Pierre ja Miquelon',
    '🇵🇳' => 'lipp: Pitcairni saared',
    '🇵🇷' => 'lipp: Puerto Rico',
    '🇵🇸' => 'lipp: Palestiina alad',
    '🇵🇹' => 'lipp: Portugal',
    '🇵🇼' => 'lipp: Belau',
    '🇵🇾' => 'lipp: Paraguay',
    '🇶🇦' => 'lipp: Katar',
    '🇷🇪' => 'lipp: Réunion',
    '🇷🇴' => 'lipp: Rumeenia',
    '🇷🇸' => 'lipp: Serbia',
    '🇷🇺' => 'lipp: Venemaa',
    '🇷🇼' => 'lipp: Rwanda',
    '🇸🇦' => 'lipp: Saudi Araabia',
    '🇸🇧' => 'lipp: Saalomoni Saared',
    '🇸🇨' => 'lipp: Seišellid',
    '🇸🇩' => 'lipp: Sudaan',
    '🇸🇪' => 'lipp: Rootsi',
    '🇸🇬' => 'lipp: Singapur',
    '🇸🇭' => 'lipp: Saint Helena',
    '🇸🇮' => 'lipp: Sloveenia',
    '🇸🇯' => 'lipp: Svalbard ja Jan Mayen',
    '🇸🇰' => 'lipp: Slovakkia',
    '🇸🇱' => 'lipp: Sierra Leone',
    '🇸🇲' => 'lipp: San Marino',
    '🇸🇳' => 'lipp: Senegal',
    '🇸🇴' => 'lipp: Somaalia',
    '🇸🇷' => 'lipp: Suriname',
    '🇸🇸' => 'lipp: Lõuna-Sudaan',
    '🇸🇹' => 'lipp: São Tomé ja Príncipe',
    '🇸🇻' => 'lipp: El Salvador',
    '🇸🇽' => 'lipp: Sint Maarten',
    '🇸🇾' => 'lipp: Süüria',
    '🇸🇿' => 'lipp: Svaasimaa',
    '🇹🇦' => 'lipp: Tristan da Cunha',
    '🇹🇨' => 'lipp: Turks ja Caicos',
    '🇹🇩' => 'lipp: Tšaad',
    '🇹🇫' => 'lipp: Prantsuse Lõunaalad',
    '🇹🇬' => 'lipp: Togo',
    '🇹🇭' => 'lipp: Tai',
    '🇹🇯' => 'lipp: Tadžikistan',
    '🇹🇰' => 'lipp: Tokelau',
    '🇹🇱' => 'lipp: Ida-Timor',
    '🇹🇲' => 'lipp: Türkmenistan',
    '🇹🇳' => 'lipp: Tuneesia',
    '🇹🇴' => 'lipp: Tonga',
    '🇹🇷' => 'lipp: Türgi',
    '🇹🇹' => 'lipp: Trinidad ja Tobago',
    '🇹🇻' => 'lipp: Tuvalu',
    '🇹🇼' => 'lipp: Taiwan',
    '🇹🇿' => 'lipp: Tansaania',
    '🇺🇦' => 'lipp: Ukraina',
    '🇺🇬' => 'lipp: Uganda',
    '🇺🇲' => 'lipp: Ühendriikide hajasaared',
    '🇺🇳' => 'lipp: Ühinenud Rahvaste Organisatsioon',
    '🇺🇸' => 'lipp: Ameerika Ühendriigid',
    '🇺🇾' => 'lipp: Uruguay',
    '🇺🇿' => 'lipp: Usbekistan',
    '🇻🇦' => 'lipp: Vatikan',
    '🇻🇨' => 'lipp: Saint Vincent ja Grenadiinid',
    '🇻🇪' => 'lipp: Venezuela',
    '🇻🇬' => 'lipp: Briti Neitsisaared',
    '🇻🇮' => 'lipp: USA Neitsisaared',
    '🇻🇳' => 'lipp: Vietnam',
    '🇻🇺' => 'lipp: Vanuatu',
    '🇼🇫' => 'lipp: Wallis ja Futuna',
    '🇼🇸' => 'lipp: Samoa',
    '🇽🇰' => 'lipp: Kosovo',
    '🇾🇪' => 'lipp: Jeemen',
    '🇾🇹' => 'lipp: Mayotte',
    '🇿🇦' => 'lipp: Lõuna-Aafrika Vabariik',
    '🇿🇲' => 'lipp: Sambia',
    '🇿🇼' => 'lipp: Zimbabwe',
    '🔟' => 'klahv: 10',
    '🏻' => 'hele nahatoon',
    '🏼' => 'keskmiselt hele nahatoon',
    '🏽' => 'keskmine nahatoon',
    '🏾' => 'keskmiselt tume nahatoon',
    '🏿' => 'tume nahatoon',
    '😀' => 'laia naeratusega nägu',
    '😃' => 'suurte silmadega naerunägu',
    '😄' => 'avatud suu ja naeratavate silmadega naerunägu',
    '😁' => 'naerul silmade ja laia naeratusega nägu',
    '😆' => 'avatud suu ja suletud silmadega naerunägu',
    '😅' => 'higipiisa ja avatud suuga naerunägu',
    '🤣' => 'naerust kõveras',
    '😂' => 'rõõmupisaratega nägu',
    '🙂' => 'naeratusega nägu',
    '🙃' => 'tagurpidi nägu',
    '🫠' => 'sulav nägu',
    '😉' => 'silma pilgutav nägu',
    '😊' => 'naeratavate silmadega naerunägu',
    '😇' => 'aupaistega naerunägu',
    '🥰' => 'naeratav südametega nägu',
    '😍' => 'südamekujuliste silmadega naerunägu',
    '🤩' => 'staariga silmitsi',
    '😘' => 'suudlust saatev nägu',
    '😗' => 'suudlev nägu',
    '☺' => 'naerunägu',
    '😚' => 'suletud silmadega suudlev nägu',
    '😙' => 'naeratavate silmadega suudlev nägu',
    '🥲' => 'naeratav pisaraga nägu',
    '😋' => 'maitsvat toitu nautiv nägu',
    '😛' => 'suust väljas keelega nägu',
    '😜' => 'suust väljas keelega ja silma pilgutav nägu',
    '🤪' => 'pöörase ilmega nägu',
    '😝' => 'keelt näitav suletud silmadega nägu',
    '🤑' => 'nägu, millel on raha suus',
    '🤗' => 'kallistav nägu',
    '🤭' => 'kätt suul hoidev nägu',
    '🫢' => 'kätt suul hoidev pärani silmadega nägu',
    '🫣' => 'käte vahelt piiluva silmaga nägu',
    '🤫' => 'sõrme suul hoidev nägu',
    '🤔' => 'mõtlev nägu',
    '🫡' => 'au andev nägu',
    '🤐' => 'tõmblukusuuga nägu',
    '🤨' => 'kergitatud kulmudega nägu',
    '😐' => 'erapooletu nägu',
    '😑' => 'ilmetu nägu',
    '😶' => 'ilma suuta nägu',
    '🫥' => 'punktiirjoonega nägu',
    '😏' => 'muigav nägu',
    '😒' => 'põlglik nägu',
    '🙄' => 'pööritavate silmadega nägu',
    '😬' => 'grimassitav nägu',
    '🤥' => 'valetav nägu',
    '🫨' => 'värisev nägu',
    '😌' => 'kergendust tundev nägu',
    '😔' => 'mõtlik nägu',
    '😪' => 'unine nägu',
    '🤤' => 'ilastav nägu',
    '😴' => 'magav nägu',
    '🫩' => 'nägu kottidega silmade all',
    '😷' => 'kirurgimaskiga kaetud nägu',
    '🤒' => 'kraadiklaasiga nägu',
    '🤕' => 'kinniseotud peaga nägu',
    '🤢' => 'iiveldust tundev nägu',
    '🤮' => 'oksendav nägu',
    '🤧' => 'aevastav nägu',
    '🥵' => 'higistav nägu',
    '🥶' => 'külmunud nägu',
    '🥴' => 'uimane nägu',
    '😵' => 'oimetu nägu',
    '🤯' => 'plahvatav pea',
    '🤠' => 'kauboimütsiga nägu',
    '🥳' => 'pidutsev nägu',
    '🥸' => 'maskeeritud nägu',
    '😎' => 'päikeseprillidega naerunägu',
    '🤓' => 'nohiku nägu',
    '🧐' => 'monokliga nägu',
    '😕' => 'segaduses nägu',
    '🫤' => 'diagonaalse suuga nägu',
    '😟' => 'murelik nägu',
    '🙁' => 'veidi kulmu kortsutav nägu',
    '☹' => 'kulmu kortsutav nägu',
    '😮' => 'avatud suuga nägu',
    '😯' => 'vaigistatud nägu',
    '😲' => 'hämmastunud nägu',
    '😳' => 'punastav nägu',
    '🫪' => 'moondunud nägu',
    '🥺' => 'anuv nägu',
    '🥹' => 'pisaraid tagasi hoidev nägu',
    '😦' => 'avatud suuga mossitav nägu',
    '😧' => 'ahastuses nägu',
    '😨' => 'hirmunud nägu',
    '😰' => 'külma higiga kaetud ja avatud suuga nägu',
    '😥' => 'pettunud, kuid kergendust tundev nägu',
    '😢' => 'nuttev nägu',
    '😭' => 'valjusti nuttev nägu',
    '😱' => 'hirmunult karjuv nägu',
    '😖' => 'kimbatuses nägu',
    '😣' => 'sihikindel nägu',
    '😞' => 'pettunud nägu',
    '😓' => 'külma higiga kaetud nägu',
    '😩' => 'rammestunud nägu',
    '😫' => 'väsinud nägu',
    '🥱' => 'haigutav nägu',
    '😤' => 'ninast auru ajav nägu',
    '😡' => 'mossitav nägu',
    '😠' => 'tige nägu',
    '🤬' => 'suul olevate sümbolitega nägu',
    '😈' => 'sarviline naerunägu',
    '👿' => 'deemon',
    '💀' => 'kolp',
    '☠' => 'kolp ja sääreluud',
    '💩' => 'kakahunnik',
    '🤡' => 'klouninägu',
    '👹' => 'sookoll',
    '👺' => 'mäekoll',
    '👻' => 'kummitus',
    '👽' => 'tulnukas',
    '👾' => 'tulnukkoletis',
    '🤖' => 'robot',
    '😺' => 'avatud suuga naeratav kass',
    '😸' => 'lõbusate silmadega irvitav kass',
    '😹' => 'rõõmupisarais kass',
    '😻' => 'südamekujuliste silmadega naeratav kass',
    '😼' => 'virilalt naeratav kass',
    '😽' => 'suletud silmadega suudlev kass',
    '🙀' => 'üllatunud kass',
    '😿' => 'nuttev kass',
    '😾' => 'mossitav kass',
    '🙈' => 'kaetud silmadega ahv',
    '🙉' => 'kaetud kõrvadega ahv',
    '🙊' => 'kaetud suuga ahv',
    '💌' => 'armastuskiri',
    '💘' => 'noolega süda',
    '💝' => 'lindiga süda',
    '💖' => 'sädelev süda',
    '💗' => 'kasvav süda',
    '💓' => 'tukslev süda',
    '💞' => 'pöörlevad südamed',
    '💕' => 'kaks südant',
    '💟' => 'südamekujuline kaunistus',
    '❣' => 'südamega hüüumärk',
    '💔' => 'murtud süda',
    '❤' => 'punane süda',
    '🩷' => 'roosa süda',
    '🧡' => 'oranž süda',
    '💛' => 'kollane süda',
    '💚' => 'roheline süda',
    '💙' => 'sinine süda',
    '🩵' => 'helesinine süda',
    '💜' => 'lilla süda',
    '🤎' => 'pruun süda',
    '🖤' => 'must süda',
    '🩶' => 'hall süda',
    '🤍' => 'valge süda',
    '💋' => 'suudluse jälg',
    '💯' => 'sada punkti',
    '💢' => 'viha sümbol',
    '🫯' => 'kakluspilv',
    '💥' => 'piraki',
    '💫' => 'peadpööritav',
    '💦' => 'higipiisad',
    '💨' => 'kiirustamine',
    '🕳' => 'auk',
    '💬' => 'jutumull',
    '🗨' => 'jutumull suunaga vasakule',
    '🗯' => 'vihamull suunaga paremale',
    '💭' => 'mõttemull',
    '💤' => 'zzz',
    '👋' => 'lehvitav käsi',
    '🤚' => 'tõstetud käsi',
    '🖐' => 'harali sõrmedega käsi',
    '✋' => 'ülestõstetud käsi',
    '🖖' => 'vulcanite tervitus',
    '🫱' => 'käsi paremale',
    '🫲' => 'käsi vasakule',
    '🫳' => 'käsi peopesaga allapoole',
    '🫴' => 'käsi peopesaga ülespoole',
    '🫷' => 'vasakule lükkav käsi',
    '🫸' => 'paremale lükkav käsi',
    '👌' => 'OK-käemärk',
    '🤌' => 'kokkusurutud sõrmed',
    '🤏' => 'näpistav käsi',
    '✌' => 'võit',
    '🤞' => 'sõrmed risti',
    '🫰' => 'pöialt ja nimetissõrme risti hoidev käsi',
    '🤟' => 'armastust väljendav žest',
    '🤘' => 'sarvede märk',
    '🤙' => 'helista mulle',
    '👈' => 'vasakule osutav sõrm',
    '👉' => 'paremale osutav sõrm',
    '👆' => 'üles osutav sõrm tagantvaates',
    '🖕' => 'keskmine sõrm',
    '👇' => 'alla osutav sõrm',
    '☝' => 'üles osutav sõrm',
    '🫵' => 'vaataja poole osutav nimetissõrm',
    '👍' => 'meeldib',
    '👎' => 'ei meeldi',
    '✊' => 'ülestõstetud rusikas',
    '👊' => 'rusikalöök',
    '🤛' => 'vasakule suunatud rusikas',
    '🤜' => 'paremale suunatud rusikas',
    '👏' => 'plaksutavad käed',
    '🙌' => 'ülestõstetud käed',
    '🫶' => 'südant moodustavad käed',
    '👐' => 'avatud käed',
    '🤲' => 'peopesad üleval koos',
    '🤝' => 'käepigistus',
    '🙏' => 'palveasendis käed',
    '✍' => 'kirjutav käsi',
    '💅' => 'küünelakk',
    '🤳' => 'selfi',
    '💪' => 'pingutatud biitseps',
    '🦾' => 'mehaaniline käsi',
    '🦿' => 'mehaaniline jalg',
    '🦵' => 'jalg',
    '🦶' => 'jalalaba',
    '👂' => 'kõrv',
    '🦻' => 'kuuldeaparaadiga kõrv',
    '👃' => 'nina',
    '🧠' => 'aju',
    '🫀' => 'süda',
    '🫁' => 'kopsud',
    '🦷' => 'hammas',
    '🦴' => 'kont',
    '👀' => 'silmad',
    '👁' => 'silm',
    '👅' => 'keel',
    '👄' => 'suu',
    '🫦' => 'huulde hammustav',
    '👶' => 'beebi',
    '🧒' => 'laps',
    '👦' => 'poiss',
    '👧' => 'tüdruk',
    '🧑' => 'täiskasvanu',
    '👱' => 'inimene: blondide juustega',
    '👨' => 'mees',
    '🧔' => 'mees: habemega',
    '👩' => 'naine',
    '🧓' => 'eakas',
    '👴' => 'eakas mees',
    '👵' => 'eakas naine',
    '🙍' => 'kulmu kortsutav inimene',
    '🙎' => 'mossitav inimene',
    '🙅' => 'eitav žest',
    '🙆' => 'nõustuv žest',
    '💁' => 'infoleti töötaja',
    '🙋' => 'kätt tõstev rõõmus inimene',
    '🧏' => 'kurt inimene',
    '🙇' => 'kummardav inimene',
    '🤦' => 'meeleheide',
    '🤷' => 'õlakehitus',
    '👮' => 'politseinik',
    '🕵' => 'detektiiv',
    '💂' => 'valvur',
    '🥷' => 'ninja',
    '👷' => 'ehitaja',
    '🫅' => 'krooniga inimene',
    '🤴' => 'prints',
    '👸' => 'printsess',
    '👳' => 'turbaniga inimene',
    '👲' => 'Hiina mütsiga mees',
    '🧕' => 'pearätiga naine',
    '🤵' => 'inimene smokingus',
    '👰' => 'loori kandev inimene',
    '🤰' => 'rase naine',
    '🫃' => 'rase mees',
    '🫄' => 'rase inimene',
    '🤱' => 'rinnaga toitmine',
    '👼' => 'inglibeebi',
    '🎅' => 'jõuluvana',
    '🤶' => 'jõuluvana abikaasa',
    '🦸' => 'superkangelane',
    '🦹' => 'superkurikael',
    '🧙' => 'maag',
    '🧚' => 'haldjas',
    '🧛' => 'vampiir',
    '🧜' => 'näkk',
    '🧝' => 'päkapikk',
    '🧞' => 'džinn',
    '🧟' => 'zombi',
    '🧌' => 'troll',
    '🫈' => 'karvane olend',
    '💆' => 'näomassaaž',
    '💇' => 'juukselõikus',
    '🚶' => 'jalakäija',
    '🧍' => 'seisev inimene',
    '🧎' => 'põlvitav inimene',
    '🏃' => 'jooksja',
    '💃' => 'tantsiv naine',
    '🕺' => 'tantsiv mees',
    '🕴' => 'hõljuv ülikonnaga mees',
    '👯' => 'pidutsevad jänesekõrvadega inimesed',
    '🧖' => 'inimene leiliruumis',
    '🧗' => 'mäkke roniv inimene',
    '🤺' => 'vehkleja',
    '🏇' => 'hobuste võiduajamine',
    '⛷' => 'suusataja',
    '🏂' => 'lumelaudur',
    '🏌' => 'golfimängija',
    '🏄' => 'surfar',
    '🚣' => 'aerupaadiga inimene',
    '🏊' => 'ujuja',
    '⛹' => 'inimene palliga',
    '🏋' => 'jõutõstja',
    '🚴' => 'jalgrattur',
    '🚵' => 'mägirattur',
    '🤸' => 'hundiratast viskav inimene',
    '🤼' => 'maadlejad',
    '🤽' => 'veepalli mängiv inimene',
    '🤾' => 'käsipalli mängiv inimene',
    '🤹' => 'žonglöör',
    '🧘' => 'lootosasendis inimene',
    '🛀' => 'inimene vannis',
    '🛌' => 'inimene voodis',
    '👭' => 'käest kinni hoidvad naised',
    '👫' => 'mees ja naine hoiavad käest kinni',
    '👬' => 'käest kinni hoidvad mehed',
    '💏' => 'musi',
    '💑' => 'paarike südamega',
    '🗣' => 'kõnelev pea',
    '👤' => 'büsti siluett',
    '👥' => 'büstide siluetid',
    '🫂' => 'kallistavad inimesed',
    '👪' => 'perekond',
    '👣' => 'jalajäljed',
    '🫆' => 'sõrmejälg',
    '🦰' => 'punased juuksed',
    '🦱' => 'lokkis juuksed',
    '🦳' => 'heledad juuksed',
    '🦲' => 'kiilaspea',
    '🐵' => 'ahvi nägu',
    '🐒' => 'ahv',
    '🦍' => 'gorilla',
    '🦧' => 'orangutan',
    '🐶' => 'koera nägu',
    '🐕' => 'koer',
    '🦮' => 'juhtkoer',
    '🐩' => 'puudel',
    '🐺' => 'hunt',
    '🦊' => 'rebane',
    '🦝' => 'pesukaru',
    '🐱' => 'kassi nägu',
    '🐈' => 'kass',
    '🦁' => 'lõvi nägu',
    '🐯' => 'tiigri nägu',
    '🐅' => 'tiiger',
    '🐆' => 'leopard',
    '🐴' => 'hobuse nägu',
    '🫎' => 'põder',
    '🫏' => 'eesel',
    '🐎' => 'hobune',
    '🦄' => 'ükssarvik',
    '🦓' => 'sebra',
    '🦌' => 'hirv',
    '🦬' => 'piison',
    '🐮' => 'lehma nägu',
    '🐂' => 'härg',
    '🐃' => 'vesipühvel',
    '🐄' => 'lehm',
    '🐷' => 'sea nägu',
    '🐖' => 'siga',
    '🐗' => 'metssiga',
    '🐽' => 'sea kärss',
    '🐏' => 'oinas',
    '🐑' => 'lammas',
    '🐐' => 'kits',
    '🐪' => 'kaamel',
    '🐫' => 'kahe küüruga kaamel',
    '🦙' => 'laama',
    '🦒' => 'kaelkirjak',
    '🐘' => 'elevant',
    '🦣' => 'mammut',
    '🦏' => 'ninasarvik',
    '🦛' => 'jõehobu',
    '🐭' => 'hiire nägu',
    '🐁' => 'hiir',
    '🐀' => 'rott',
    '🐹' => 'hamster',
    '🐰' => 'jänese nägu',
    '🐇' => 'jänes',
    '🐿' => 'vöötorav',
    '🦫' => 'kobras',
    '🦔' => 'siil',
    '🦇' => 'nahkhiir',
    '🐻' => 'karu',
    '🐨' => 'koaala',
    '🐼' => 'panda',
    '🦥' => 'laiskloom',
    '🦦' => 'saarmas',
    '🦨' => 'skunk',
    '🦘' => 'känguru',
    '🦡' => 'mäger',
    '🐾' => 'käpajäljed',
    '🦃' => 'kalkun',
    '🐔' => 'kana',
    '🐓' => 'kukk',
    '🐣' => 'kooruv tibu',
    '🐤' => 'väike tibu',
    '🐥' => 'väike tibu eestvaates',
    '🐦' => 'lind',
    '🐧' => 'pingviin',
    '🕊' => 'tuvi',
    '🦅' => 'kotkas',
    '🦆' => 'part',
    '🦢' => 'luik',
    '🦉' => 'öökull',
    '🦤' => 'dodo',
    '🪶' => 'sulg',
    '🦩' => 'flamingo',
    '🦚' => 'paabulind',
    '🦜' => 'papagoi',
    '🪽' => 'tiib',
    '🪿' => 'hani',
    '🐸' => 'konn',
    '🐊' => 'krokodill',
    '🐢' => 'kilpkonn',
    '🦎' => 'sisalik',
    '🐍' => 'madu',
    '🐲' => 'draakoni nägu',
    '🐉' => 'draakon',
    '🦕' => 'sauropood',
    '🦖' => 'türannosaurus',
    '🐳' => 'vett pritsiv vaal',
    '🐋' => 'vaal',
    '🐬' => 'delfiin',
    '🫍' => 'mõõkvaal',
    '🦭' => 'hüljes',
    '🐟' => 'kala',
    '🐠' => 'troopiline kala',
    '🐡' => 'siilkala',
    '🦈' => 'hai',
    '🐙' => 'kaheksajalg',
    '🐚' => 'spiraalne teokarp',
    '🪸' => 'korall',
    '🪼' => 'meduus',
    '🦀' => 'krabi',
    '🦞' => 'homaar',
    '🦐' => 'krevett',
    '🦑' => 'kalmaar',
    '🦪' => 'auster',
    '🐌' => 'tigu',
    '🦋' => 'liblikas',
    '🐛' => 'röövik',
    '🐜' => 'sipelgas',
    '🐝' => 'mesilane',
    '🪲' => 'mardikas',
    '🐞' => 'lepatriinu',
    '🦗' => 'kilk',
    '🪳' => 'prussakas',
    '🕷' => 'ämblik',
    '🕸' => 'ämblikuvõrk',
    '🦂' => 'suur skorpion',
    '🦟' => 'sääsk',
    '🪰' => 'kärbes',
    '🪱' => 'uss',
    '🦠' => 'mikroob',
    '💐' => 'kimp',
    '🌸' => 'kirsiõis',
    '💮' => 'valge lill',
    '🪷' => 'lootos',
    '🏵' => 'rosett',
    '🌹' => 'roos',
    '🥀' => 'närbunud lill',
    '🌺' => 'hibisk',
    '🌻' => 'päevalill',
    '🌼' => 'õis',
    '🌷' => 'tulp',
    '🪻' => 'hüatsint',
    '🌱' => 'võrse',
    '🪴' => 'potilill',
    '🌲' => 'igihaljas puu',
    '🌳' => 'lehtpuu',
    '🌴' => 'palmipuu',
    '🌵' => 'kaktus',
    '🌾' => 'riisitaim',
    '🌿' => 'maitsetaim',
    '☘' => 'ristik',
    '🍀' => 'nelja lehega ristik',
    '🍁' => 'vahtraleht',
    '🍂' => 'langenud leht',
    '🍃' => 'tuules lendlev leht',
    '🪹' => 'tühi pesa',
    '🪺' => 'munadega pesa',
    '🍄' => 'seen',
    '🪾' => 'raagus puu',
    '🍇' => 'viinamarjad',
    '🍈' => 'melon',
    '🍉' => 'arbuus',
    '🍊' => 'mandariin',
    '🍋' => 'sidrun',
    '🍌' => 'banaan',
    '🍍' => 'ananass',
    '🥭' => 'mango',
    '🍎' => 'punane õun',
    '🍏' => 'roheline õun',
    '🍐' => 'pirn',
    '🍑' => 'virsik',
    '🍒' => 'kirsid',
    '🍓' => 'maasikas',
    '🫐' => 'mustikad',
    '🥝' => 'kiivi',
    '🍅' => 'tomat',
    '🫒' => 'oliiv',
    '🥥' => 'kookospähkel',
    '🥑' => 'avokaado',
    '🍆' => 'baklažaan',
    '🥔' => 'kartul',
    '🥕' => 'porgand',
    '🌽' => 'maisitõlvik',
    '🌶' => 'tuline pipar',
    '🫑' => 'paprika',
    '🥒' => 'kurk',
    '🥬' => 'lehtköögivili',
    '🥦' => 'brokoli',
    '🧄' => 'küüslauk',
    '🧅' => 'sibul',
    '🥜' => 'maapähklid',
    '🫘' => 'oad',
    '🌰' => 'kastan',
    '🫚' => 'ingver',
    '🫛' => 'hernekaun',
    '🫜' => 'juurvili',
    '🍞' => 'leib',
    '🥐' => 'sarvesai',
    '🥖' => 'baguette',
    '🫓' => 'lameleib',
    '🥨' => 'soolakringel',
    '🥯' => 'rõngassai',
    '🥞' => 'pannkoogid',
    '🧇' => 'vahvel',
    '🧀' => 'juustutükk',
    '🍖' => 'kondiga liha',
    '🍗' => 'kanakoib',
    '🥩' => 'lihalõik',
    '🥓' => 'peekon',
    '🍔' => 'hamburger',
    '🍟' => 'friikartulid',
    '🍕' => 'pitsa',
    '🌭' => 'hot dog',
    '🥪' => 'võileib',
    '🌮' => 'tako',
    '🌯' => 'burrito',
    '🫔' => 'tamale',
    '🥙' => 'täidetud pita',
    '🧆' => 'falafel',
    '🥚' => 'muna',
    '🍳' => 'toiduvalmistamine',
    '🥘' => 'panniroog',
    '🍲' => 'potitäis toitu',
    '🫕' => 'fondüü',
    '🥣' => 'lusikaga kauss',
    '🥗' => 'roheline salat',
    '🍿' => 'popkorn',
    '🧈' => 'või',
    '🧂' => 'sool',
    '🥫' => 'toidukonserv',
    '🍱' => 'bento einekarp',
    '🍘' => 'riisiküpsis',
    '🍙' => 'riisipall',
    '🍚' => 'keedetud riis',
    '🍛' => 'karriga riis',
    '🍜' => 'aurav kauss',
    '🍝' => 'spagetid',
    '🍠' => 'röstitud bataat',
    '🍢' => 'oden',
    '🍣' => 'suši',
    '🍤' => 'praetud krevett',
    '🍥' => 'keerdudega kalakotlet',
    '🥮' => 'kuukook',
    '🍡' => 'dango',
    '🥟' => 'pelmeen',
    '🥠' => 'õnneküpsis',
    '🥡' => 'kaasamüügikarp',
    '🍦' => 'pehme jäätis',
    '🍧' => 'mahlajää',
    '🍨' => 'jäätis',
    '🍩' => 'sõõrik',
    '🍪' => 'küpsis',
    '🎂' => 'sünnipäevatort',
    '🍰' => 'liivataignakook',
    '🧁' => 'muffin',
    '🥧' => 'pirukas',
    '🍫' => 'šokolaaditahvel',
    '🍬' => 'komm',
    '🍭' => 'pulgakomm',
    '🍮' => 'vanillipuding',
    '🍯' => 'meepott',
    '🍼' => 'lutipudel',
    '🥛' => 'klaas piima',
    '☕' => 'kuum jook',
    '🫖' => 'teekann',
    '🍵' => 'kõrvata teetass',
    '🍶' => 'riisiviin',
    '🍾' => 'pudeli avamine',
    '🍷' => 'veiniklaas',
    '🍸' => 'kokteiliklaas',
    '🍹' => 'troopiline jook',
    '🍺' => 'õllekann',
    '🍻' => 'kõlisevad õllekannud',
    '🥂' => 'kõlisevad klaasid',
    '🥃' => 'joogiklaas',
    '🫗' => 'vedeliku kallamine',
    '🥤' => 'kõrrega tops',
    '🧋' => 'pärlitee',
    '🧃' => 'joogipakk',
    '🧉' => 'mate',
    '🧊' => 'jääkuubik',
    '🥢' => 'söögipulgad',
    '🍽' => 'kahvel, nuga ja taldrik',
    '🍴' => 'kahvel ja nuga',
    '🥄' => 'lusikas',
    '🔪' => 'kööginuga',
    '🫙' => 'purk',
    '🏺' => 'amfora',
    '🌍' => 'Euroopa ja Aafrika gloobusel',
    '🌎' => 'Ameerika mandrid gloobusel',
    '🌏' => 'Aasia ja Austraalia gloobusel',
    '🌐' => 'meridiaanidega gloobus',
    '🗺' => 'maailmakaart',
    '🗾' => 'Jaapani kaart',
    '🧭' => 'kompass',
    '🏔' => 'lumine mägi',
    '⛰' => 'mägi',
    '🛘' => 'maalihe',
    '🌋' => 'vulkaan',
    '🗻' => 'Fuji mägi',
    '🏕' => 'telkimine',
    '🏖' => 'päikesevari rannas',
    '🏜' => 'kõrb',
    '🏝' => 'asustamata saar',
    '🏞' => 'rahvuspark',
    '🏟' => 'staadion',
    '🏛' => 'antiikhoone',
    '🏗' => 'ehitus',
    '🧱' => 'telliskivi',
    '🪨' => 'kivi',
    '🪵' => 'puit',
    '🛖' => 'hütt',
    '🏘' => 'majad',
    '🏚' => 'mahajäetud maja',
    '🏠' => 'maja',
    '🏡' => 'aiaga maja',
    '🏢' => 'kontorihoone',
    '🏣' => 'Jaapani postkontor',
    '🏤' => 'postkontor',
    '🏥' => 'haigla',
    '🏦' => 'pank',
    '🏨' => 'hotell',
    '🏩' => 'armuhotell',
    '🏪' => 'kauplus',
    '🏫' => 'kool',
    '🏬' => 'kaubamaja',
    '🏭' => 'tehas',
    '🏯' => 'Jaapani loss',
    '🏰' => 'loss',
    '💒' => 'pulmad',
    '🗼' => 'Tokyo torn',
    '🗽' => 'Vabadussammas',
    '⛪' => 'kirik',
    '🕌' => 'mošee',
    '🛕' => 'hinduistlik tempel',
    '🕍' => 'sünagoog',
    '⛩' => 'shinto tempel',
    '🕋' => 'kaaba',
    '⛲' => 'purskkaev',
    '⛺' => 'telk',
    '🌁' => 'udune',
    '🌃' => 'öine tähistaevas',
    '🏙' => 'linna siluett',
    '🌄' => 'päikesetõus mägedes',
    '🌅' => 'päikesetõus',
    '🌆' => 'linna siluett videvikus',
    '🌇' => 'päikeseloojang',
    '🌉' => 'sild öösel',
    '♨' => 'kuumaveeallikad',
    '🎠' => 'karussellihobune',
    '🛝' => 'liumägi',
    '🎡' => 'vaateratas',
    '🎢' => 'Ameerika mäed',
    '💈' => 'habemeajaja tähis',
    '🎪' => 'tsirkusetelk',
    '🚂' => 'vedur',
    '🚃' => 'rongivagun',
    '🚄' => 'kiirrong',
    '🚅' => 'terava ninaga kiirrong',
    '🚆' => 'rong',
    '🚇' => 'metroo',
    '🚈' => 'kergraudtee',
    '🚉' => 'jaam',
    '🚊' => 'tramm',
    '🚝' => 'üherööpmeline raudtee',
    '🚞' => 'mägiraudtee',
    '🚋' => 'trammivagun',
    '🚌' => 'buss',
    '🚍' => 'buss eestvaates',
    '🚎' => 'trollibuss',
    '🚐' => 'väikebuss',
    '🚑' => 'kiirabiauto',
    '🚒' => 'tuletõrjeauto',
    '🚓' => 'politseiauto',
    '🚔' => 'politseiauto eestvaates',
    '🚕' => 'takso',
    '🚖' => 'takso eestvaates',
    '🚗' => 'auto',
    '🚘' => 'auto eestvaates',
    '🚙' => 'mahtuniversaal',
    '🛻' => 'pikap',
    '🚚' => 'kulleriauto',
    '🚛' => 'veok',
    '🚜' => 'traktor',
    '🏎' => 'võidusõiduauto',
    '🏍' => 'mootorratas',
    '🛵' => 'motoroller',
    '🦽' => 'manuaalratastool',
    '🦼' => 'elektriratastool',
    '🛺' => 'autorikša',
    '🚲' => 'jalgratas',
    '🛴' => 'tõukeratas',
    '🛹' => 'rula',
    '🛼' => 'rulluisk',
    '🚏' => 'bussipeatus',
    '🛣' => 'kiirtee',
    '🛤' => 'raudteerööpad',
    '🛢' => 'õlitünn',
    '⛽' => 'tankur',
    '🛞' => 'ratas',
    '🚨' => 'politseiauto vilkur',
    '🚥' => 'horisontaalne foor',
    '🚦' => 'vertikaalne foor',
    '🛑' => 'stoppmärk',
    '🚧' => 'ehitustööd',
    '⚓' => 'ankur',
    '🛟' => 'päästerõngas',
    '⛵' => 'purjepaat',
    '🛶' => 'kanuu',
    '🚤' => 'kiirkaater',
    '🛳' => 'reisilaev',
    '⛴' => 'praam',
    '🛥' => 'mootorpaat',
    '🚢' => 'laev',
    '✈' => 'lennuk',
    '🛩' => 'väikelennuk',
    '🛫' => 'õhkutõusev lennuk',
    '🛬' => 'maanduv lennuk',
    '🪂' => 'langevarjur',
    '💺' => 'iste',
    '🚁' => 'helikopter',
    '🚟' => 'rippraudtee',
    '🚠' => 'köistee mägedes',
    '🚡' => 'köisraudtee',
    '🛰' => 'satelliit',
    '🚀' => 'rakett',
    '🛸' => 'lendav taldrik',
    '🛎' => 'märgukell',
    '🧳' => 'kohver',
    '⌛' => 'liivakell',
    '⏳' => 'voolava liivaga kell',
    '⌚' => 'käekell',
    '⏰' => 'äratuskell',
    '⏱' => 'stopper',
    '⏲' => 'taimer',
    '🕰' => 'kaminakell',
    '🕛' => 'kell kaksteist',
    '🕧' => 'kell pool üks',
    '🕐' => 'kell üks',
    '🕜' => 'kell pool kaks',
    '🕑' => 'kell kaks',
    '🕝' => 'kell pool kolm',
    '🕒' => 'kell kolm',
    '🕞' => 'kell pool neli',
    '🕓' => 'kell neli',
    '🕟' => 'kell pool viis',
    '🕔' => 'kell viis',
    '🕠' => 'kell pool kuus',
    '🕕' => 'kell kuus',
    '🕡' => 'kell pool seitse',
    '🕖' => 'kell seitse',
    '🕢' => 'kell pool kaheksa',
    '🕗' => 'kell kaheksa',
    '🕣' => 'kell pool üheksa',
    '🕘' => 'kell üheksa',
    '🕤' => 'kell pool kümme',
    '🕙' => 'kell kümme',
    '🕥' => 'kell pool üksteist',
    '🕚' => 'kell üksteist',
    '🕦' => 'kell pool kaksteist',
    '🌑' => 'noorkuu',
    '🌒' => 'kasvav kuusirp',
    '🌓' => 'kasvav poolkuu',
    '🌔' => 'kasvav kuu',
    '🌕' => 'täiskuu',
    '🌖' => 'kahanev kuu',
    '🌗' => 'kahanev poolkuu',
    '🌘' => 'kahanev kuusirp',
    '🌙' => 'kuusirp',
    '🌚' => 'näoga noorkuu',
    '🌛' => 'näoga kasvav kuu',
    '🌜' => 'näoga kahanev kuu',
    '🌡' => 'kraadiklaas',
    '☀' => 'päike',
    '🌝' => 'näoga täiskuu',
    '🌞' => 'näoga päike',
    '🪐' => 'rõngastega planeet',
    '⭐' => 'täht',
    '🌟' => 'helendav täht',
    '🌠' => 'langev täht',
    '🌌' => 'Linnutee',
    '☁' => 'pilv',
    '⛅' => 'päike pilve taga',
    '⛈' => 'äikese- ja vihmapilv',
    '🌤' => 'päike väikese pilve taga',
    '🌥' => 'päike suure pilve taga',
    '🌦' => 'päike vihmapilve taga',
    '🌧' => 'vihmapilv',
    '🌨' => 'lumepilv',
    '🌩' => 'äikesepilv',
    '🌪' => 'keeristorm',
    '🌫' => 'udu',
    '🌬' => 'näoga tuul',
    '🌀' => 'tsüklon',
    '🌈' => 'vikerkaar',
    '🌂' => 'suletud vihmavari',
    '☂' => 'vihmavari',
    '☔' => 'piiskadega vihmavari',
    '⛱' => 'päikesevari maas',
    '⚡' => 'kõrgepinge',
    '❄' => 'lumehelves',
    '☃' => 'lumememm',
    '⛄' => 'lumeta lumememm',
    '☄' => 'komeet',
    '🔥' => 'tuli',
    '💧' => 'piisk',
    '🌊' => 'veelaine',
    '🎃' => 'kõrvitsalatern',
    '🎄' => 'jõulupuu',
    '🎆' => 'ilutulestik',
    '🎇' => 'säraküünal',
    '🧨' => 'ilutulestikurakett',
    '✨' => 'sädemed',
    '🎈' => 'õhupall',
    '🎉' => 'pidupäev',
    '🎊' => 'konfetid',
    '🎋' => 'tanabata soovipuu',
    '🎍' => 'väravamänd',
    '🎎' => 'Jaapani nukud',
    '🎏' => 'kalakujuline vimpel',
    '🎐' => 'tuulekell',
    '🎑' => 'kuutseremoonia',
    '🧧' => 'punane ümbrik',
    '🎀' => 'lint',
    '🎁' => 'kingipakk',
    '🎗' => 'kollane lint',
    '🎟' => 'piletid',
    '🎫' => 'pilet',
    '🎖' => 'sõjamedal',
    '🏆' => 'karikas',
    '🏅' => 'medal',
    '🥇' => 'kuldmedal',
    '🥈' => 'hõbemedal',
    '🥉' => 'pronksmedal',
    '⚽' => 'jalgpall',
    '⚾' => 'pesapall',
    '🥎' => 'pehmepall',
    '🏀' => 'korvpall',
    '🏐' => 'võrkpall',
    '🏈' => 'Ameerika jalgpall',
    '🏉' => 'ragbipall',
    '🎾' => 'tennis',
    '🥏' => 'visketaldrik',
    '🎳' => 'keegel',
    '🏏' => 'kriket',
    '🏑' => 'maahoki',
    '🏒' => 'jäähoki',
    '🥍' => 'kahvpall',
    '🏓' => 'pingpong',
    '🏸' => 'sulgpall',
    '🥊' => 'poksikinnas',
    '🥋' => 'võitluskunstide riietus',
    '🥅' => 'väravavõrk',
    '⛳' => 'golfilipp',
    '⛸' => 'uisk',
    '🎣' => 'õng',
    '🤿' => 'sukeldumismask',
    '🎽' => 'jooksusärk',
    '🎿' => 'suusad',
    '🛷' => 'kelk',
    '🥌' => 'jääkeegli kivi',
    '🎯' => 'täistabamus',
    '🪀' => 'jojo',
    '🪁' => 'tuulelohe',
    '🔫' => 'veepüstol',
    '🎱' => 'piljardikuul',
    '🔮' => 'kristallkuul',
    '🪄' => 'võlukepp',
    '🎮' => 'videomäng',
    '🕹' => 'juhtpult',
    '🎰' => 'mänguautomaat',
    '🎲' => 'täring',
    '🧩' => 'pusletükk',
    '🧸' => 'mängukaru',
    '🪅' => 'piñata',
    '🪩' => 'diskokera',
    '🪆' => 'matrjoška',
    '♠' => 'poti',
    '♥' => 'ärtu',
    '♦' => 'ruutu',
    '♣' => 'risti',
    '♟' => 'ettur',
    '🃏' => 'jokker',
    '🀄' => 'mahjongi punane draakon',
    '🎴' => 'mängukaardid',
    '🎭' => 'lavakunst',
    '🖼' => 'raamitud pilt',
    '🎨' => 'kunstniku palett',
    '🧵' => 'niit',
    '🪡' => 'õmblusnõel',
    '🧶' => 'lõng',
    '🪢' => 'sõlm',
    '👓' => 'prillid',
    '🕶' => 'päikeseprillid',
    '🥽' => 'kaitseprillid',
    '🥼' => 'laborikittel',
    '🦺' => 'ohutusvest',
    '👔' => 'lips',
    '👕' => 'T-särk',
    '👖' => 'teksad',
    '🧣' => 'sall',
    '🧤' => 'kindad',
    '🧥' => 'mantel',
    '🧦' => 'sokid',
    '👗' => 'kleit',
    '👘' => 'kimono',
    '🥻' => 'sari',
    '🩱' => 'ujumistrikoo',
    '🩲' => 'ujumispüksid',
    '🩳' => 'šortsid',
    '👙' => 'bikiinid',
    '👚' => 'naisteriided',
    '🪭' => 'volditav lehvik',
    '👛' => 'ridikül',
    '👜' => 'käekott',
    '👝' => 'kott',
    '🛍' => 'ostukotid',
    '🎒' => 'ranits',
    '🩴' => 'varbavahesandaalid',
    '👞' => 'meesteking',
    '👟' => 'jooksuking',
    '🥾' => 'matkasaabas',
    '🥿' => 'madal king',
    '👠' => 'kõrge kontsaga king',
    '👡' => 'naistesandaal',
    '🩰' => 'balletikingad',
    '👢' => 'naistesaabas',
    '🪮' => 'juuksehark',
    '👑' => 'kroon',
    '👒' => 'naistekübar',
    '🎩' => 'torukübar',
    '🎓' => 'koolilõpumüts',
    '🧢' => 'nokkmüts',
    '🪖' => 'sõdurikiiver',
    '⛑' => 'valge ristiga kiiver',
    '📿' => 'palvehelmed',
    '💄' => 'huulepulk',
    '💍' => 'sõrmus',
    '💎' => 'kalliskivi',
    '🔇' => 'vaigistatud kõlar',
    '🔈' => 'vaikse heliga kõlar',
    '🔉' => 'poolvalju heliga kõlar',
    '🔊' => 'valju heliga kõlar',
    '📢' => 'valjuhääldi',
    '📣' => 'megafon',
    '📯' => 'postisarv',
    '🔔' => 'kelluke',
    '🔕' => 'vaigistatud kelluke',
    '🎼' => 'noodijoonestik',
    '🎵' => 'noot',
    '🎶' => 'noodid',
    '🎙' => 'stuudiomikrofon',
    '🎚' => 'liugur',
    '🎛' => 'juhtnupud',
    '🎤' => 'mikrofon',
    '🎧' => 'kõrvaklapid',
    '📻' => 'raadio',
    '🎷' => 'saksofon',
    '🎺' => 'trompet',
    '🪊' => 'tromboon',
    '🪗' => 'akordion',
    '🎸' => 'kitarr',
    '🎹' => 'klahvpill',
    '🎻' => 'viiul',
    '🪕' => 'bandžo',
    '🥁' => 'trumm',
    '🪘' => 'aafrika trumm',
    '🪇' => 'marakas',
    '🪈' => 'flööt',
    '🪉' => 'harf',
    '📱' => 'mobiiltelefon',
    '📲' => 'noolega mobiiltelefon',
    '☎' => 'telefon',
    '📞' => 'telefonitoru',
    '📟' => 'piipar',
    '📠' => 'faksimasin',
    '🔋' => 'aku',
    '🪫' => 'aku tühjeneb',
    '🔌' => 'elektripistik',
    '💻' => 'sülearvuti',
    '🖥' => 'lauaarvuti',
    '🖨' => 'printer',
    '⌨' => 'klaviatuur',
    '🖱' => 'arvutihiir',
    '🖲' => 'juhtkuul',
    '💽' => 'arvutiketas',
    '💾' => 'diskett',
    '💿' => 'optiline ketas',
    '📀' => 'DVD',
    '🧮' => 'arvelaud',
    '🎥' => 'filmikaamera',
    '🎞' => 'filmikaadrid',
    '📽' => 'filmiprojektor',
    '🎬' => 'klapptahvel',
    '📺' => 'teler',
    '📷' => 'kaamera',
    '📸' => 'välguga kaamera',
    '📹' => 'videokaamera',
    '📼' => 'videokassett',
    '🔍' => 'vasakule kallutatud luup',
    '🔎' => 'paremale kallutatud luup',
    '🕯' => 'küünal',
    '💡' => 'lamp',
    '🔦' => 'taskulamp',
    '🏮' => 'punane paberlatern',
    '🪔' => 'diya-lamp',
    '📔' => 'dekoratiivse kaanega märkmik',
    '📕' => 'suletud raamat',
    '📖' => 'avatud raamat',
    '📗' => 'roheline raamat',
    '📘' => 'sinine raamat',
    '📙' => 'oranž raamat',
    '📚' => 'raamatud',
    '📓' => 'päevik',
    '📒' => 'arveraamat',
    '📃' => 'kaardunud paberileht',
    '📜' => 'käsikiri',
    '📄' => 'paberilehe eestvaade',
    '📰' => 'ajaleht',
    '🗞' => 'rullikeeratud ajaleht',
    '📑' => 'järjehoidjad',
    '🔖' => 'järjehoidja',
    '🏷' => 'silt',
    '🪙' => 'münt',
    '💰' => 'rahapaun',
    '🪎' => 'aardelaegas',
    '💴' => 'jeeni rahatäht',
    '💵' => 'dollari rahatäht',
    '💶' => 'euro rahatäht',
    '💷' => 'naela rahatäht',
    '💸' => 'tiibadega raha',
    '💳' => 'krediitkaart',
    '🧾' => 'kviitung',
    '💹' => 'tõusev graafik jeenisümboliga',
    '✉' => 'ümbrik',
    '📧' => 'meilisõnum',
    '📨' => 'sissetulev ümbrik',
    '📩' => 'noolega ümbrik',
    '📤' => 'väljaminev post',
    '📥' => 'sissetulev post',
    '📦' => 'pakk',
    '📫' => 'suletud postkast tõstetud lipuga',
    '📪' => 'suletud postkast langetatud lipuga',
    '📬' => 'avatud postkast tõstetud lipuga',
    '📭' => 'avatud postkast langetatud lipuga',
    '📮' => 'postkast',
    '🗳' => 'hääletuskast',
    '✏' => 'pliiats',
    '✒' => 'must sulepea',
    '🖋' => 'sulepea',
    '🖊' => 'pastapliiats',
    '🖌' => 'pintsel',
    '🖍' => 'kriit',
    '📝' => 'memo',
    '💼' => 'portfell',
    '📁' => 'kaust',
    '📂' => 'avatud kaust',
    '🗂' => 'register',
    '📅' => 'kalender',
    '📆' => 'rebitav kalender',
    '🗒' => 'spiraalmärkmik',
    '🗓' => 'spiraalkalender',
    '📇' => 'kataloog',
    '📈' => 'tõusev graafik',
    '📉' => 'langev graafik',
    '📊' => 'tulpdiagramm',
    '📋' => 'kirjutamisalus',
    '📌' => 'nööpnõel',
    '📍' => 'ümara peaga nööpnõel',
    '📎' => 'kirjaklamber',
    '🖇' => 'ühendatud kirjaklambrid',
    '📏' => 'sirge joonlaud',
    '📐' => 'kolmnurkne joonlaud',
    '✂' => 'käärid',
    '🗃' => 'dokumendikast',
    '🗄' => 'dokumendikapp',
    '🗑' => 'prügikorv',
    '🔒' => 'lukk',
    '🔓' => 'avatud lukk',
    '🔏' => 'lukk ja pastakas',
    '🔐' => 'suletud lukk võtmega',
    '🔑' => 'võti',
    '🗝' => 'vana võti',
    '🔨' => 'haamer',
    '🪓' => 'kirves',
    '⛏' => 'kirka',
    '⚒' => 'haamer ja kirka',
    '🛠' => 'haamer ja mutrivõti',
    '🗡' => 'pistoda',
    '⚔' => 'ristatud mõõgad',
    '💣' => 'pomm',
    '🪃' => 'bumerang',
    '🏹' => 'vibu ja nool',
    '🛡' => 'kilp',
    '🪚' => 'käsisaag',
    '🔧' => 'mutrivõti',
    '🪛' => 'kruvikeeraja',
    '🔩' => 'polt ja mutter',
    '⚙' => 'hammasratas',
    '🗜' => 'kruustangid',
    '⚖' => 'kaalukausid',
    '🦯' => 'valge kepp',
    '🔗' => 'lüli',
    '⛓' => 'ahelad',
    '🪝' => 'konks',
    '🧰' => 'tööriistakast',
    '🧲' => 'magnet',
    '🪜' => 'redel',
    '🪏' => 'labidas',
    '⚗' => 'destilleerimisnõu',
    '🧪' => 'katseklaas',
    '🧫' => 'Petri tass',
    '🧬' => 'DNA',
    '🔬' => 'mikroskoop',
    '🔭' => 'teleskoop',
    '📡' => 'satelliitantenn',
    '💉' => 'süstal',
    '🩸' => 'veretilk',
    '💊' => 'tablett',
    '🩹' => 'plaaster',
    '🩼' => 'kark',
    '🩺' => 'stetoskoop',
    '🩻' => 'röntgen',
    '🚪' => 'uks',
    '🛗' => 'lift',
    '🪞' => 'peegel',
    '🪟' => 'aken',
    '🛏' => 'voodi',
    '🛋' => 'diivan ja lamp',
    '🪑' => 'tool',
    '🚽' => 'WC-pott',
    '🪠' => 'valamupump',
    '🚿' => 'dušš',
    '🛁' => 'vann',
    '🪤' => 'hiirelõks',
    '🪒' => 'habemenuga',
    '🧴' => 'emulsioonipudel',
    '🧷' => 'haaknõel',
    '🧹' => 'luud',
    '🧺' => 'korv',
    '🧻' => 'paberirull',
    '🪣' => 'ämber',
    '🧼' => 'seep',
    '🫧' => 'mullid',
    '🪥' => 'hambahari',
    '🧽' => 'pesukäsn',
    '🧯' => 'tulekustuti',
    '🛒' => 'ostukäru',
    '🚬' => 'sigaret',
    '⚰' => 'kirst',
    '🪦' => 'hauakivi',
    '⚱' => 'tuhaurn',
    '🧿' => 'Nazari amulett',
    '🪬' => 'hamsa',
    '🗿' => 'moai',
    '🪧' => 'plakat',
    '🪪' => 'ID-kaart',
    '🏧' => 'pangaautomaat',
    '🚮' => 'prügikasti silt',
    '🚰' => 'joogivesi',
    '♿' => 'ratastool',
    '🚹' => 'meeste-WC',
    '🚺' => 'naiste-WC',
    '🚻' => 'tualett',
    '🚼' => 'beebi sümbol',
    '🚾' => 'WC',
    '🛂' => 'passikontroll',
    '🛃' => 'toll',
    '🛄' => 'pagasi kättesaamine',
    '🛅' => 'hoiukapid',
    '⚠' => 'hoiatus',
    '🚸' => 'lapsed teel',
    '⛔' => 'sisenemine keelatud',
    '🚫' => 'keelatud',
    '🚳' => 'jalgrattad keelatud',
    '🚭' => 'suitsetamine keelatud',
    '🚯' => 'risustamine keelatud',
    '🚱' => 'joomiseks kõlbmatu vesi',
    '🚷' => 'jalakäijad keelatud',
    '📵' => 'telefonid on keelatud',
    '🔞' => 'alla 18-aastastele keelatud',
    '☢' => 'radioaktiivne',
    '☣' => 'bioloogiline oht',
    '⬆' => 'nool üles',
    '↗' => 'nool üles paremale',
    '➡' => 'nool paremale',
    '↘' => 'nool alla paremale',
    '⬇' => 'nool alla',
    '↙' => 'nool alla vasakule',
    '⬅' => 'nool vasakule',
    '↖' => 'nool üles vasakule',
    '↕' => 'nool üles-alla',
    '↔' => 'nool paremale-vasakule',
    '↩' => 'kaardus nool vasakule',
    '↪' => 'kaardus nool paremale',
    '⤴' => 'kaardus nool üles',
    '⤵' => 'kaardus nool alla',
    '🔃' => 'vertikaalsed nooled päripäeva',
    '🔄' => 'nooled vastupäeva',
    '🔙' => 'nool BACK',
    '🔚' => 'nool END',
    '🔛' => 'nool ON!',
    '🔜' => 'nool SOON',
    '🔝' => 'nool TOP',
    '🛐' => 'pühakoda',
    '⚛' => 'aatomi sümbol',
    '🕉' => 'hinduism',
    '✡' => 'Taaveti täht',
    '☸' => 'dharma ratas',
    '☯' => 'yin-yang',
    '✝' => 'ladina rist',
    '☦' => 'õigeusu rist',
    '☪' => 'täht ja kuusirp',
    '☮' => 'rahu sümbol',
    '🕎' => 'menoraa',
    '🔯' => 'punktiga kuusnurkne täht',
    '🪯' => 'khanda',
    '♈' => 'Jäär',
    '♉' => 'Sõnn',
    '♊' => 'Kaksikud',
    '♋' => 'Vähk',
    '♌' => 'Lõvi',
    '♍' => 'Neitsi',
    '♎' => 'Kaalud',
    '♏' => 'Skorpion',
    '♐' => 'Ambur',
    '♑' => 'Kaljukits',
    '♒' => 'Veevalaja',
    '♓' => 'Kalad',
    '⛎' => 'Maokandja',
    '🔀' => 'juhusliku järjekorra nupp',
    '🔁' => 'kordamisnupp',
    '🔂' => 'ühe korduse nupp',
    '▶' => 'esitusnupp',
    '⏩' => 'edasikerimisnupp',
    '⏭' => 'järgmise loo nupp',
    '⏯' => 'esitus- ja pausinupp',
    '◀' => 'tagurpidi esitamise nupp',
    '⏪' => 'tagasikerimise nupp',
    '⏮' => 'eelmise loo nupp',
    '🔼' => 'ülesnupp',
    '⏫' => 'kiirkerimine üles',
    '🔽' => 'allanupp',
    '⏬' => 'kiirkerimine alla',
    '⏸' => 'pausinupp',
    '⏹' => 'peatamisnupp',
    '⏺' => 'salvestusnupp',
    '⏏' => 'väljutamisnupp',
    '🎦' => 'kino',
    '🔅' => 'tuhmistamisnupp',
    '🔆' => 'ereduse nupp',
    '📶' => 'levi',
    '🛜' => 'juhtmevaba',
    '📳' => 'värinarežiim',
    '📴' => 'väljalülitatud mobiiltelefon',
    '♀' => 'naisesümbol',
    '♂' => 'mehesümbol',
    '⚧' => 'transsoolisuse sümbol',
    '✖' => 'korrutamine',
    '➕' => 'liitmine',
    '➖' => 'lahutamine',
    '➗' => 'jagamine',
    '🟰' => 'tugev võrdusmärk',
    '♾' => 'lõpmatus',
    '‼' => 'topelthüüumärk',
    '⁉' => 'küsimärgiga hüüumärk',
    '❓' => 'punane küsimärk',
    '❔' => 'valge küsimärk',
    '❕' => 'valge hüüumärk',
    '❗' => 'punane hüüumärk',
    '〰' => 'laineline kriips',
    '💱' => 'valuutavahetus',
    '💲' => 'suur dollarimärk',
    '⚕' => 'meditsiinisümbol',
    '♻' => 'taaskasutuse sümbol',
    '⚜' => 'prantsuse liilia',
    '🔱' => 'kolmhargi sümbol',
    '📛' => 'nimesilt',
    '🔰' => 'algaja sümbol',
    '⭕' => 'seest tühi punane ringjoon',
    '✅' => 'märkenupp',
    '☑' => 'märgitud ruut',
    '✔' => 'märge',
    '❌' => 'rist',
    '❎' => 'korrutusmärgiga nupp',
    '➰' => 'silmus',
    '➿' => 'topeltsilmus',
    '〽' => 'osa vaheldumise märk',
    '✳' => 'kaheksa tipuga tärn',
    '✴' => 'kaheksa tipuga täht',
    '❇' => 'sädelus',
    '™' => 'kaubamärk',
    '🫟' => 'plärakas',
    '🔠' => 'ladina suurtähed',
    '🔡' => 'ladina väiketähed',
    '🔢' => 'numbrid',
    '🔣' => 'sümbolid',
    '🔤' => 'ladina tähed',
    '🅰' => 'nupp A (veregrupp)',
    '🆎' => 'nupp AB (veregrupp)',
    '🅱' => 'nupp B (veregrupp)',
    '🆑' => 'nupp CL',
    '🆒' => 'nupp COOL',
    '🆓' => 'nupp FREE',
    'ℹ' => 'teabeallikas',
    '🆔' => 'nupp ID',
    'Ⓜ' => 'M ringis',
    '🆕' => 'nupp NEW',
    '🆖' => 'nupp NG',
    '🅾' => 'nupp 0 (veregrupp)',
    '🆗' => 'nupp OK',
    '🅿' => 'nupp P',
    '🆘' => 'nupp SOS',
    '🆙' => 'nupp UP!',
    '🆚' => 'nupp VS',
    '🈁' => 'jaapani nupp "siin"',
    '🈂' => 'jaapani nupp "teenustasu"',
    '🈷' => 'jaapani nupp "kuutasu"',
    '🈶' => 'jaapani nupp "pole tasuta"',
    '🈯' => 'jaapani nupp "reserveeritud"',
    '🉐' => 'jaapani nupp "soodne"',
    '🈹' => 'jaapani nupp "allahindlus"',
    '🈚' => 'jaapani nupp "tasuta"',
    '🈲' => 'jaapani nupp "keelatud"',
    '🉑' => 'jaapani nupp "vastuvõetav"',
    '🈸' => 'jaapani nupp "rakendus"',
    '🈴' => 'jaapani nupp "sooritatud"',
    '🈳' => 'jaapani nupp "vaba"',
    '㊗' => 'jaapani nupp "õnnitlused"',
    '㊙' => 'jaapani nupp "saladus"',
    '🈺' => 'jaapani nupp "avatud"',
    '🈵' => 'jaapani nupp "pole vaba"',
    '🔴' => 'punane ring',
    '🟠' => 'oranž ring',
    '🟡' => 'kollane ring',
    '🟢' => 'roheline ring',
    '🔵' => 'sinine ring',
    '🟣' => 'lilla ring',
    '🟤' => 'pruun ring',
    '⚫' => 'must ring',
    '⚪' => 'valge ring',
    '🟥' => 'punane ruut',
    '🟧' => 'oranž ruut',
    '🟨' => 'kollane ruut',
    '🟩' => 'roheline ruut',
    '🟦' => 'sinine ruut',
    '🟪' => 'lilla ruut',
    '🟫' => 'pruun ruut',
    '⬛' => 'suur must ruut',
    '⬜' => 'suur valge ruut',
    '◼' => 'keskmine must ruut',
    '◻' => 'keskmine valge ruut',
    '◾' => 'väiksem must ruut',
    '◽' => 'väiksem valge ruut',
    '▪' => 'väike must ruut',
    '▫' => 'väike valge ruut',
    '🔶' => 'suur oranž romb',
    '🔷' => 'suur sinine romb',
    '🔸' => 'väike oranž romb',
    '🔹' => 'väike sinine romb',
    '🔺' => 'punane kolmnurk',
    '🔻' => 'punane tagurpidi kolmnurk',
    '💠' => 'punktiga romb',
    '🔘' => 'raadionupp',
    '🔳' => 'valge ruudukujuline nupp',
    '🔲' => 'must ruudukujuline nupp',
    '🏁' => 'ruudulipp',
    '🚩' => 'kolmnurkne lipp',
    '🎌' => 'ristatud lipud',
    '🏴' => 'must lipp',
    '🏳' => 'valge lipp',
];
