<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'berciuman: dewasa, dewasa, warna kulit cerah, warna kulit cerah-sedang',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'berciuman: dewasa, dewasa, warna kulit cerah, warna kulit sedang',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'berciuman: dewasa, dewasa, warna kulit cerah, warna kulit gelap-sedang',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'berciuman: dewasa, dewasa, warna kulit cerah, warna kulit gelap',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'berciuman: dewasa, dewasa, warna kulit cerah-sedang, warna kulit cerah',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'berciuman: dewasa, dewasa, warna kulit cerah-sedang, warna kulit sedang',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'berciuman: dewasa, dewasa, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'berciuman: dewasa, dewasa, warna kulit cerah-sedang, warna kulit gelap',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'berciuman: dewasa, dewasa, warna kulit sedang, warna kulit cerah',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'berciuman: dewasa, dewasa, warna kulit sedang, warna kulit cerah-sedang',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'berciuman: dewasa, dewasa, warna kulit sedang, warna kulit gelap-sedang',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'berciuman: dewasa, dewasa, warna kulit sedang, warna kulit gelap',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'berciuman: dewasa, dewasa, warna kulit gelap-sedang, warna kulit cerah',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'berciuman: dewasa, dewasa, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'berciuman: dewasa, dewasa, warna kulit gelap-sedang, warna kulit sedang',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'berciuman: dewasa, dewasa, warna kulit gelap-sedang, warna kulit gelap',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'berciuman: dewasa, dewasa, warna kulit gelap, warna kulit cerah',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'berciuman: dewasa, dewasa, warna kulit gelap, warna kulit cerah-sedang',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'berciuman: dewasa, dewasa, warna kulit gelap, warna kulit sedang',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'berciuman: dewasa, dewasa, warna kulit gelap, warna kulit gelap-sedang',
    '👨🏻‍❤‍💋‍👨🏻' => 'berciuman: pria, pria, warna kulit cerah',
    '👨🏻‍❤‍💋‍👨🏼' => 'berciuman: pria, pria, warna kulit cerah, warna kulit cerah-sedang',
    '👨🏻‍❤‍💋‍👨🏽' => 'berciuman: pria, pria, warna kulit cerah, warna kulit sedang',
    '👨🏻‍❤‍💋‍👨🏾' => 'berciuman: pria, pria, warna kulit cerah, warna kulit gelap-sedang',
    '👨🏻‍❤‍💋‍👨🏿' => 'berciuman: pria, pria, warna kulit cerah, warna kulit gelap',
    '👨🏼‍❤‍💋‍👨🏻' => 'berciuman: pria, pria, warna kulit cerah-sedang, warna kulit cerah',
    '👨🏼‍❤‍💋‍👨🏼' => 'berciuman: pria, pria, warna kulit cerah-sedang',
    '👨🏼‍❤‍💋‍👨🏽' => 'berciuman: pria, pria, warna kulit cerah-sedang, warna kulit sedang',
    '👨🏼‍❤‍💋‍👨🏾' => 'berciuman: pria, pria, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👨🏼‍❤‍💋‍👨🏿' => 'berciuman: pria, pria, warna kulit cerah-sedang, warna kulit gelap',
    '👨🏽‍❤‍💋‍👨🏻' => 'berciuman: pria, pria, warna kulit sedang, warna kulit cerah',
    '👨🏽‍❤‍💋‍👨🏼' => 'berciuman: pria, pria, warna kulit sedang, warna kulit cerah-sedang',
    '👨🏽‍❤‍💋‍👨🏽' => 'berciuman: pria, pria, warna kulit sedang',
    '👨🏽‍❤‍💋‍👨🏾' => 'berciuman: pria, pria, warna kulit sedang, warna kulit gelap-sedang',
    '👨🏽‍❤‍💋‍👨🏿' => 'berciuman: pria, pria, warna kulit sedang, warna kulit gelap',
    '👨🏾‍❤‍💋‍👨🏻' => 'berciuman: pria, pria, warna kulit gelap-sedang, warna kulit cerah',
    '👨🏾‍❤‍💋‍👨🏼' => 'berciuman: pria, pria, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👨🏾‍❤‍💋‍👨🏽' => 'berciuman: pria, pria, warna kulit gelap-sedang, warna kulit sedang',
    '👨🏾‍❤‍💋‍👨🏾' => 'berciuman: pria, pria, warna kulit gelap-sedang',
    '👨🏾‍❤‍💋‍👨🏿' => 'berciuman: pria, pria, warna kulit gelap-sedang, warna kulit gelap',
    '👨🏿‍❤‍💋‍👨🏻' => 'berciuman: pria, pria, warna kulit gelap, warna kulit cerah',
    '👨🏿‍❤‍💋‍👨🏼' => 'berciuman: pria, pria, warna kulit gelap, warna kulit cerah-sedang',
    '👨🏿‍❤‍💋‍👨🏽' => 'berciuman: pria, pria, warna kulit gelap, warna kulit sedang',
    '👨🏿‍❤‍💋‍👨🏾' => 'berciuman: pria, pria, warna kulit gelap, warna kulit gelap-sedang',
    '👨🏿‍❤‍💋‍👨🏿' => 'berciuman: pria, pria, warna kulit gelap',
    '👩🏻‍❤‍💋‍👨🏻' => 'berciuman: wanita, pria, warna kulit cerah',
    '👩🏻‍❤‍💋‍👨🏼' => 'berciuman: wanita, pria, warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍❤‍💋‍👨🏽' => 'berciuman: wanita, pria, warna kulit cerah, warna kulit sedang',
    '👩🏻‍❤‍💋‍👨🏾' => 'berciuman: wanita, pria, warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍❤‍💋‍👨🏿' => 'berciuman: wanita, pria, warna kulit cerah, warna kulit gelap',
    '👩🏼‍❤‍💋‍👨🏻' => 'berciuman: wanita, pria, warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍❤‍💋‍👨🏼' => 'berciuman: wanita, pria, warna kulit cerah-sedang',
    '👩🏼‍❤‍💋‍👨🏽' => 'berciuman: wanita, pria, warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍❤‍💋‍👨🏾' => 'berciuman: wanita, pria, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍❤‍💋‍👨🏿' => 'berciuman: wanita, pria, warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍❤‍💋‍👨🏻' => 'berciuman: wanita, pria, warna kulit sedang, warna kulit cerah',
    '👩🏽‍❤‍💋‍👨🏼' => 'berciuman: wanita, pria, warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍❤‍💋‍👨🏽' => 'berciuman: wanita, pria, warna kulit sedang',
    '👩🏽‍❤‍💋‍👨🏾' => 'berciuman: wanita, pria, warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍❤‍💋‍👨🏿' => 'berciuman: wanita, pria, warna kulit sedang, warna kulit gelap',
    '👩🏾‍❤‍💋‍👨🏻' => 'berciuman: wanita, pria, warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍❤‍💋‍👨🏼' => 'berciuman: wanita, pria, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍❤‍💋‍👨🏽' => 'berciuman: wanita, pria, warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍❤‍💋‍👨🏾' => 'berciuman: wanita, pria, warna kulit gelap-sedang',
    '👩🏾‍❤‍💋‍👨🏿' => 'berciuman: wanita, pria, warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍❤‍💋‍👨🏻' => 'berciuman: wanita, pria, warna kulit gelap, warna kulit cerah',
    '👩🏿‍❤‍💋‍👨🏼' => 'berciuman: wanita, pria, warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍❤‍💋‍👨🏽' => 'berciuman: wanita, pria, warna kulit gelap, warna kulit sedang',
    '👩🏿‍❤‍💋‍👨🏾' => 'berciuman: wanita, pria, warna kulit gelap, warna kulit gelap-sedang',
    '👩🏿‍❤‍💋‍👨🏿' => 'berciuman: wanita, pria, warna kulit gelap',
    '👩🏻‍❤‍💋‍👩🏻' => 'berciuman: wanita, wanita, warna kulit cerah',
    '👩🏻‍❤‍💋‍👩🏼' => 'berciuman: wanita, wanita, warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍❤‍💋‍👩🏽' => 'berciuman: wanita, wanita, warna kulit cerah, warna kulit sedang',
    '👩🏻‍❤‍💋‍👩🏾' => 'berciuman: wanita, wanita, warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍❤‍💋‍👩🏿' => 'berciuman: wanita, wanita, warna kulit cerah, warna kulit gelap',
    '👩🏼‍❤‍💋‍👩🏻' => 'berciuman: wanita, wanita, warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍❤‍💋‍👩🏼' => 'berciuman: wanita, wanita, warna kulit cerah-sedang',
    '👩🏼‍❤‍💋‍👩🏽' => 'berciuman: wanita, wanita, warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍❤‍💋‍👩🏾' => 'berciuman: wanita, wanita, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍❤‍💋‍👩🏿' => 'berciuman: wanita, wanita, warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍❤‍💋‍👩🏻' => 'berciuman: wanita, wanita, warna kulit sedang, warna kulit cerah',
    '👩🏽‍❤‍💋‍👩🏼' => 'berciuman: wanita, wanita, warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍❤‍💋‍👩🏽' => 'berciuman: wanita, wanita, warna kulit sedang',
    '👩🏽‍❤‍💋‍👩🏾' => 'berciuman: wanita, wanita, warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍❤‍💋‍👩🏿' => 'berciuman: wanita, wanita, warna kulit sedang, warna kulit gelap',
    '👩🏾‍❤‍💋‍👩🏻' => 'berciuman: wanita, wanita, warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍❤‍💋‍👩🏼' => 'berciuman: wanita, wanita, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍❤‍💋‍👩🏽' => 'berciuman: wanita, wanita, warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍❤‍💋‍👩🏾' => 'berciuman: wanita, wanita, warna kulit gelap-sedang',
    '👩🏾‍❤‍💋‍👩🏿' => 'berciuman: wanita, wanita, warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍❤‍💋‍👩🏻' => 'berciuman: wanita, wanita, warna kulit gelap, warna kulit cerah',
    '👩🏿‍❤‍💋‍👩🏼' => 'berciuman: wanita, wanita, warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍❤‍💋‍👩🏽' => 'berciuman: wanita, wanita, warna kulit gelap, warna kulit sedang',
    '👩🏿‍❤‍💋‍👩🏾' => 'berciuman: wanita, wanita, warna kulit gelap, warna kulit gelap-sedang',
    '👩🏿‍❤‍💋‍👩🏿' => 'berciuman: wanita, wanita, warna kulit gelap',
    '🧑🏻‍🫯‍🧑🏼' => 'orang bergulat: warna kulit cerah, warna kulit cerah-sedang',
    '🧑🏻‍🫯‍🧑🏽' => 'orang bergulat: warna kulit cerah, warna kulit sedang',
    '🧑🏻‍🫯‍🧑🏾' => 'orang bergulat: warna kulit cerah, warna kulit gelap-sedang',
    '🧑🏻‍🫯‍🧑🏿' => 'orang bergulat: warna kulit cerah, warna kulit gelap',
    '🧑🏼‍🫯‍🧑🏻' => 'orang bergulat: warna kulit cerah-sedang, warna kulit cerah',
    '🧑🏼‍🫯‍🧑🏽' => 'orang bergulat: warna kulit cerah-sedang, warna kulit sedang',
    '🧑🏼‍🫯‍🧑🏾' => 'orang bergulat: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '🧑🏼‍🫯‍🧑🏿' => 'orang bergulat: warna kulit cerah-sedang, warna kulit gelap',
    '🧑🏽‍🫯‍🧑🏻' => 'orang bergulat: warna kulit sedang, warna kulit cerah',
    '🧑🏽‍🫯‍🧑🏼' => 'orang bergulat: warna kulit sedang, warna kulit cerah-sedang',
    '🧑🏽‍🫯‍🧑🏾' => 'orang bergulat: warna kulit sedang, warna kulit gelap-sedang',
    '🧑🏽‍🫯‍🧑🏿' => 'orang bergulat: warna kulit sedang, warna kulit gelap',
    '🧑🏾‍🫯‍🧑🏻' => 'orang bergulat: warna kulit gelap-sedang, warna kulit cerah',
    '🧑🏾‍🫯‍🧑🏼' => 'orang bergulat: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '🧑🏾‍🫯‍🧑🏽' => 'orang bergulat: warna kulit gelap-sedang, warna kulit sedang',
    '🧑🏾‍🫯‍🧑🏿' => 'orang bergulat: warna kulit gelap-sedang, warna kulit gelap',
    '🧑🏿‍🫯‍🧑🏻' => 'orang bergulat: warna kulit gelap, warna kulit cerah',
    '🧑🏿‍🫯‍🧑🏼' => 'orang bergulat: warna kulit gelap, warna kulit cerah-sedang',
    '🧑🏿‍🫯‍🧑🏽' => 'orang bergulat: warna kulit gelap, warna kulit sedang',
    '🧑🏿‍🫯‍🧑🏾' => 'orang bergulat: warna kulit gelap, warna kulit gelap-sedang',
    '🧑🏻‍❤‍🧑🏼' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah, warna kulit cerah-sedang',
    '🧑🏻‍❤‍🧑🏽' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah, warna kulit sedang',
    '🧑🏻‍❤‍🧑🏾' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah, warna kulit gelap-sedang',
    '🧑🏻‍❤‍🧑🏿' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah, warna kulit gelap',
    '🧑🏼‍❤‍🧑🏻' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah-sedang, warna kulit cerah',
    '🧑🏼‍❤‍🧑🏽' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah-sedang, warna kulit sedang',
    '🧑🏼‍❤‍🧑🏾' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '🧑🏼‍❤‍🧑🏿' => 'pasangan dengan hati: dewasa, dewasa, warna kulit cerah-sedang, warna kulit gelap',
    '🧑🏽‍❤‍🧑🏻' => 'pasangan dengan hati: dewasa, dewasa, warna kulit sedang, warna kulit cerah',
    '🧑🏽‍❤‍🧑🏼' => 'pasangan dengan hati: dewasa, dewasa, warna kulit sedang, warna kulit cerah-sedang',
    '🧑🏽‍❤‍🧑🏾' => 'pasangan dengan hati: dewasa, dewasa, warna kulit sedang, warna kulit gelap-sedang',
    '🧑🏽‍❤‍🧑🏿' => 'pasangan dengan hati: dewasa, dewasa, warna kulit sedang, warna kulit gelap',
    '🧑🏾‍❤‍🧑🏻' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap-sedang, warna kulit cerah',
    '🧑🏾‍❤‍🧑🏼' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '🧑🏾‍❤‍🧑🏽' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap-sedang, warna kulit sedang',
    '🧑🏾‍❤‍🧑🏿' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap-sedang, warna kulit gelap',
    '🧑🏿‍❤‍🧑🏻' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap, warna kulit cerah',
    '🧑🏿‍❤‍🧑🏼' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap, warna kulit cerah-sedang',
    '🧑🏿‍❤‍🧑🏽' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap, warna kulit sedang',
    '🧑🏿‍❤‍🧑🏾' => 'pasangan dengan hati: dewasa, dewasa, warna kulit gelap, warna kulit gelap-sedang',
    '🧑🏻‍🐰‍🧑🏼' => 'orang dengan telinga kelinci: warna kulit cerah, warna kulit cerah-sedang',
    '🧑🏻‍🐰‍🧑🏽' => 'orang dengan telinga kelinci: warna kulit cerah, warna kulit sedang',
    '🧑🏻‍🐰‍🧑🏾' => 'orang dengan telinga kelinci: warna kulit cerah, warna kulit gelap-sedang',
    '🧑🏻‍🐰‍🧑🏿' => 'orang dengan telinga kelinci: warna kulit cerah, warna kulit gelap',
    '🧑🏼‍🐰‍🧑🏻' => 'orang dengan telinga kelinci: warna kulit cerah-sedang, warna kulit cerah',
    '🧑🏼‍🐰‍🧑🏽' => 'orang dengan telinga kelinci: warna kulit cerah-sedang, warna kulit sedang',
    '🧑🏼‍🐰‍🧑🏾' => 'orang dengan telinga kelinci: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '🧑🏼‍🐰‍🧑🏿' => 'orang dengan telinga kelinci: warna kulit cerah-sedang, warna kulit gelap',
    '🧑🏽‍🐰‍🧑🏻' => 'orang dengan telinga kelinci: warna kulit sedang, warna kulit cerah',
    '🧑🏽‍🐰‍🧑🏼' => 'orang dengan telinga kelinci: warna kulit sedang, warna kulit cerah-sedang',
    '🧑🏽‍🐰‍🧑🏾' => 'orang dengan telinga kelinci: warna kulit sedang, warna kulit gelap-sedang',
    '🧑🏽‍🐰‍🧑🏿' => 'orang dengan telinga kelinci: warna kulit sedang, warna kulit gelap',
    '🧑🏾‍🐰‍🧑🏻' => 'orang dengan telinga kelinci: warna kulit gelap-sedang, warna kulit cerah',
    '🧑🏾‍🐰‍🧑🏼' => 'orang dengan telinga kelinci: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '🧑🏾‍🐰‍🧑🏽' => 'orang dengan telinga kelinci: warna kulit gelap-sedang, warna kulit sedang',
    '🧑🏾‍🐰‍🧑🏿' => 'orang dengan telinga kelinci: warna kulit gelap-sedang, warna kulit gelap',
    '🧑🏿‍🐰‍🧑🏻' => 'orang dengan telinga kelinci: warna kulit gelap, warna kulit cerah',
    '🧑🏿‍🐰‍🧑🏼' => 'orang dengan telinga kelinci: warna kulit gelap, warna kulit cerah-sedang',
    '🧑🏿‍🐰‍🧑🏽' => 'orang dengan telinga kelinci: warna kulit gelap, warna kulit sedang',
    '🧑🏿‍🐰‍🧑🏾' => 'orang dengan telinga kelinci: warna kulit gelap, warna kulit gelap-sedang',
    '👨🏻‍🫯‍👨🏼' => 'pria bergulat: warna kulit cerah, warna kulit cerah-sedang',
    '👨🏻‍🫯‍👨🏽' => 'pria bergulat: warna kulit cerah, warna kulit sedang',
    '👨🏻‍🫯‍👨🏾' => 'pria bergulat: warna kulit cerah, warna kulit gelap-sedang',
    '👨🏻‍🫯‍👨🏿' => 'pria bergulat: warna kulit cerah, warna kulit gelap',
    '👨🏼‍🫯‍👨🏻' => 'pria bergulat: warna kulit cerah-sedang, warna kulit cerah',
    '👨🏼‍🫯‍👨🏽' => 'pria bergulat: warna kulit cerah-sedang, warna kulit sedang',
    '👨🏼‍🫯‍👨🏾' => 'pria bergulat: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👨🏼‍🫯‍👨🏿' => 'pria bergulat: warna kulit cerah-sedang, warna kulit gelap',
    '👨🏽‍🫯‍👨🏻' => 'pria bergulat: warna kulit sedang, warna kulit cerah',
    '👨🏽‍🫯‍👨🏼' => 'pria bergulat: warna kulit sedang, warna kulit cerah-sedang',
    '👨🏽‍🫯‍👨🏾' => 'pria bergulat: warna kulit sedang, warna kulit gelap-sedang',
    '👨🏽‍🫯‍👨🏿' => 'pria bergulat: warna kulit sedang, warna kulit gelap',
    '👨🏾‍🫯‍👨🏻' => 'pria bergulat: warna kulit gelap-sedang, warna kulit cerah',
    '👨🏾‍🫯‍👨🏼' => 'pria bergulat: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👨🏾‍🫯‍👨🏽' => 'pria bergulat: warna kulit gelap-sedang, warna kulit sedang',
    '👨🏾‍🫯‍👨🏿' => 'pria bergulat: warna kulit gelap-sedang, warna kulit gelap',
    '👨🏿‍🫯‍👨🏻' => 'pria bergulat: warna kulit gelap, warna kulit cerah',
    '👨🏿‍🫯‍👨🏼' => 'pria bergulat: warna kulit gelap, warna kulit cerah-sedang',
    '👨🏿‍🫯‍👨🏽' => 'pria bergulat: warna kulit gelap, warna kulit sedang',
    '👨🏿‍🫯‍👨🏾' => 'pria bergulat: warna kulit gelap, warna kulit gelap-sedang',
    '👨🏻‍❤‍👨🏻' => 'pasangan dengan hati: pria, pria, warna kulit cerah',
    '👨🏻‍❤‍👨🏼' => 'pasangan dengan hati: pria, pria, warna kulit cerah, warna kulit cerah-sedang',
    '👨🏻‍❤‍👨🏽' => 'pasangan dengan hati: pria, pria, warna kulit cerah, warna kulit sedang',
    '👨🏻‍❤‍👨🏾' => 'pasangan dengan hati: pria, pria, warna kulit cerah, warna kulit gelap-sedang',
    '👨🏻‍❤‍👨🏿' => 'pasangan dengan hati: pria, pria, warna kulit cerah, warna kulit gelap',
    '👨🏼‍❤‍👨🏻' => 'pasangan dengan hati: pria, pria, warna kulit cerah-sedang, warna kulit cerah',
    '👨🏼‍❤‍👨🏼' => 'pasangan dengan hati: pria, pria, warna kulit cerah-sedang',
    '👨🏼‍❤‍👨🏽' => 'pasangan dengan hati: pria, pria, warna kulit cerah-sedang, warna kulit sedang',
    '👨🏼‍❤‍👨🏾' => 'pasangan dengan hati: pria, pria, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👨🏼‍❤‍👨🏿' => 'pasangan dengan hati: pria, pria, warna kulit cerah-sedang, warna kulit gelap',
    '👨🏽‍❤‍👨🏻' => 'pasangan dengan hati: pria, pria, warna kulit sedang, warna kulit cerah',
    '👨🏽‍❤‍👨🏼' => 'pasangan dengan hati: pria, pria, warna kulit sedang, warna kulit cerah-sedang',
    '👨🏽‍❤‍👨🏽' => 'pasangan dengan hati: pria, pria, warna kulit sedang',
    '👨🏽‍❤‍👨🏾' => 'pasangan dengan hati: pria, pria, warna kulit sedang, warna kulit gelap-sedang',
    '👨🏽‍❤‍👨🏿' => 'pasangan dengan hati: pria, pria, warna kulit sedang, warna kulit gelap',
    '👨🏾‍❤‍👨🏻' => 'pasangan dengan hati: pria, pria, warna kulit gelap-sedang, warna kulit cerah',
    '👨🏾‍❤‍👨🏼' => 'pasangan dengan hati: pria, pria, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👨🏾‍❤‍👨🏽' => 'pasangan dengan hati: pria, pria, warna kulit gelap-sedang, warna kulit sedang',
    '👨🏾‍❤‍👨🏾' => 'pasangan dengan hati: pria, pria, warna kulit gelap-sedang',
    '👨🏾‍❤‍👨🏿' => 'pasangan dengan hati: pria, pria, warna kulit gelap-sedang, warna kulit gelap',
    '👨🏿‍❤‍👨🏻' => 'pasangan dengan hati: pria, pria, warna kulit gelap, warna kulit cerah',
    '👨🏿‍❤‍👨🏼' => 'pasangan dengan hati: pria, pria, warna kulit gelap, warna kulit cerah-sedang',
    '👨🏿‍❤‍👨🏽' => 'pasangan dengan hati: pria, pria, warna kulit gelap, warna kulit sedang',
    '👨🏿‍❤‍👨🏾' => 'pasangan dengan hati: pria, pria, warna kulit gelap, warna kulit gelap-sedang',
    '👨🏿‍❤‍👨🏿' => 'pasangan dengan hati: pria, pria, warna kulit gelap',
    '👨🏻‍🐰‍👨🏼' => 'pria dengan telinga kelinci: warna kulit cerah, warna kulit cerah-sedang',
    '👨🏻‍🐰‍👨🏽' => 'pria dengan telinga kelinci: warna kulit cerah, warna kulit sedang',
    '👨🏻‍🐰‍👨🏾' => 'pria dengan telinga kelinci: warna kulit cerah, warna kulit gelap-sedang',
    '👨🏻‍🐰‍👨🏿' => 'pria dengan telinga kelinci: warna kulit cerah, warna kulit gelap',
    '👨🏼‍🐰‍👨🏻' => 'pria dengan telinga kelinci: warna kulit cerah-sedang, warna kulit cerah',
    '👨🏼‍🐰‍👨🏽' => 'pria dengan telinga kelinci: warna kulit cerah-sedang, warna kulit sedang',
    '👨🏼‍🐰‍👨🏾' => 'pria dengan telinga kelinci: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👨🏼‍🐰‍👨🏿' => 'pria dengan telinga kelinci: warna kulit cerah-sedang, warna kulit gelap',
    '👨🏽‍🐰‍👨🏻' => 'pria dengan telinga kelinci: warna kulit sedang, warna kulit cerah',
    '👨🏽‍🐰‍👨🏼' => 'pria dengan telinga kelinci: warna kulit sedang, warna kulit cerah-sedang',
    '👨🏽‍🐰‍👨🏾' => 'pria dengan telinga kelinci: warna kulit sedang, warna kulit gelap-sedang',
    '👨🏽‍🐰‍👨🏿' => 'pria dengan telinga kelinci: warna kulit sedang, warna kulit gelap',
    '👨🏾‍🐰‍👨🏻' => 'pria dengan telinga kelinci: warna kulit gelap-sedang, warna kulit cerah',
    '👨🏾‍🐰‍👨🏼' => 'pria dengan telinga kelinci: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👨🏾‍🐰‍👨🏽' => 'pria dengan telinga kelinci: warna kulit gelap-sedang, warna kulit sedang',
    '👨🏾‍🐰‍👨🏿' => 'pria dengan telinga kelinci: warna kulit gelap-sedang, warna kulit gelap',
    '👨🏿‍🐰‍👨🏻' => 'pria dengan telinga kelinci: warna kulit gelap, warna kulit cerah',
    '👨🏿‍🐰‍👨🏼' => 'pria dengan telinga kelinci: warna kulit gelap, warna kulit cerah-sedang',
    '👨🏿‍🐰‍👨🏽' => 'pria dengan telinga kelinci: warna kulit gelap, warna kulit sedang',
    '👨🏿‍🐰‍👨🏾' => 'pria dengan telinga kelinci: warna kulit gelap, warna kulit gelap-sedang',
    '👩🏻‍🫯‍👩🏼' => 'wanita bergulat: warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍🫯‍👩🏽' => 'wanita bergulat: warna kulit cerah, warna kulit sedang',
    '👩🏻‍🫯‍👩🏾' => 'wanita bergulat: warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍🫯‍👩🏿' => 'wanita bergulat: warna kulit cerah, warna kulit gelap',
    '👩🏼‍🫯‍👩🏻' => 'wanita bergulat: warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍🫯‍👩🏽' => 'wanita bergulat: warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍🫯‍👩🏾' => 'wanita bergulat: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍🫯‍👩🏿' => 'wanita bergulat: warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍🫯‍👩🏻' => 'wanita bergulat: warna kulit sedang, warna kulit cerah',
    '👩🏽‍🫯‍👩🏼' => 'wanita bergulat: warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍🫯‍👩🏾' => 'wanita bergulat: warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍🫯‍👩🏿' => 'wanita bergulat: warna kulit sedang, warna kulit gelap',
    '👩🏾‍🫯‍👩🏻' => 'wanita bergulat: warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍🫯‍👩🏼' => 'wanita bergulat: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍🫯‍👩🏽' => 'wanita bergulat: warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍🫯‍👩🏿' => 'wanita bergulat: warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍🫯‍👩🏻' => 'wanita bergulat: warna kulit gelap, warna kulit cerah',
    '👩🏿‍🫯‍👩🏼' => 'wanita bergulat: warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍🫯‍👩🏽' => 'wanita bergulat: warna kulit gelap, warna kulit sedang',
    '👩🏿‍🫯‍👩🏾' => 'wanita bergulat: warna kulit gelap, warna kulit gelap-sedang',
    '👩🏻‍❤‍👨🏻' => 'pasangan dengan hati: wanita, pria, warna kulit cerah',
    '👩🏻‍❤‍👨🏼' => 'pasangan dengan hati: wanita, pria, warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍❤‍👨🏽' => 'pasangan dengan hati: wanita, pria, warna kulit cerah, warna kulit sedang',
    '👩🏻‍❤‍👨🏾' => 'pasangan dengan hati: wanita, pria, warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍❤‍👨🏿' => 'pasangan dengan hati: wanita, pria, warna kulit cerah, warna kulit gelap',
    '👩🏼‍❤‍👨🏻' => 'pasangan dengan hati: wanita, pria, warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍❤‍👨🏼' => 'pasangan dengan hati: wanita, pria, warna kulit cerah-sedang',
    '👩🏼‍❤‍👨🏽' => 'pasangan dengan hati: wanita, pria, warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍❤‍👨🏾' => 'pasangan dengan hati: wanita, pria, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍❤‍👨🏿' => 'pasangan dengan hati: wanita, pria, warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍❤‍👨🏻' => 'pasangan dengan hati: wanita, pria, warna kulit sedang, warna kulit cerah',
    '👩🏽‍❤‍👨🏼' => 'pasangan dengan hati: wanita, pria, warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍❤‍👨🏽' => 'pasangan dengan hati: wanita, pria, warna kulit sedang',
    '👩🏽‍❤‍👨🏾' => 'pasangan dengan hati: wanita, pria, warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍❤‍👨🏿' => 'pasangan dengan hati: wanita, pria, warna kulit sedang, warna kulit gelap',
    '👩🏾‍❤‍👨🏻' => 'pasangan dengan hati: wanita, pria, warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍❤‍👨🏼' => 'pasangan dengan hati: wanita, pria, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍❤‍👨🏽' => 'pasangan dengan hati: wanita, pria, warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍❤‍👨🏾' => 'pasangan dengan hati: wanita, pria, warna kulit gelap-sedang',
    '👩🏾‍❤‍👨🏿' => 'pasangan dengan hati: wanita, pria, warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍❤‍👨🏻' => 'pasangan dengan hati: wanita, pria, warna kulit gelap, warna kulit cerah',
    '👩🏿‍❤‍👨🏼' => 'pasangan dengan hati: wanita, pria, warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍❤‍👨🏽' => 'pasangan dengan hati: wanita, pria, warna kulit gelap, warna kulit sedang',
    '👩🏿‍❤‍👨🏾' => 'pasangan dengan hati: wanita, pria, warna kulit gelap, warna kulit gelap-sedang',
    '👩🏿‍❤‍👨🏿' => 'pasangan dengan hati: wanita, pria, warna kulit gelap',
    '👩🏻‍❤‍👩🏻' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah',
    '👩🏻‍❤‍👩🏼' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍❤‍👩🏽' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah, warna kulit sedang',
    '👩🏻‍❤‍👩🏾' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍❤‍👩🏿' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah, warna kulit gelap',
    '👩🏼‍❤‍👩🏻' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍❤‍👩🏼' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah-sedang',
    '👩🏼‍❤‍👩🏽' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍❤‍👩🏾' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍❤‍👩🏿' => 'pasangan dengan hati: wanita, wanita, warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍❤‍👩🏻' => 'pasangan dengan hati: wanita, wanita, warna kulit sedang, warna kulit cerah',
    '👩🏽‍❤‍👩🏼' => 'pasangan dengan hati: wanita, wanita, warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍❤‍👩🏽' => 'pasangan dengan hati: wanita, wanita, warna kulit sedang',
    '👩🏽‍❤‍👩🏾' => 'pasangan dengan hati: wanita, wanita, warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍❤‍👩🏿' => 'pasangan dengan hati: wanita, wanita, warna kulit sedang, warna kulit gelap',
    '👩🏾‍❤‍👩🏻' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍❤‍👩🏼' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍❤‍👩🏽' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍❤‍👩🏾' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap-sedang',
    '👩🏾‍❤‍👩🏿' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍❤‍👩🏻' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap, warna kulit cerah',
    '👩🏿‍❤‍👩🏼' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍❤‍👩🏽' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap, warna kulit sedang',
    '👩🏿‍❤‍👩🏾' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap, warna kulit gelap-sedang',
    '👩🏿‍❤‍👩🏿' => 'pasangan dengan hati: wanita, wanita, warna kulit gelap',
    '👩🏻‍🐰‍👩🏼' => 'wanita dengan telinga kelinci: warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍🐰‍👩🏽' => 'wanita dengan telinga kelinci: warna kulit cerah, warna kulit sedang',
    '👩🏻‍🐰‍👩🏾' => 'wanita dengan telinga kelinci: warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍🐰‍👩🏿' => 'wanita dengan telinga kelinci: warna kulit cerah, warna kulit gelap',
    '👩🏼‍🐰‍👩🏻' => 'wanita dengan telinga kelinci: warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍🐰‍👩🏽' => 'wanita dengan telinga kelinci: warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍🐰‍👩🏾' => 'wanita dengan telinga kelinci: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍🐰‍👩🏿' => 'wanita dengan telinga kelinci: warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍🐰‍👩🏻' => 'wanita dengan telinga kelinci: warna kulit sedang, warna kulit cerah',
    '👩🏽‍🐰‍👩🏼' => 'wanita dengan telinga kelinci: warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍🐰‍👩🏾' => 'wanita dengan telinga kelinci: warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍🐰‍👩🏿' => 'wanita dengan telinga kelinci: warna kulit sedang, warna kulit gelap',
    '👩🏾‍🐰‍👩🏻' => 'wanita dengan telinga kelinci: warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍🐰‍👩🏼' => 'wanita dengan telinga kelinci: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍🐰‍👩🏽' => 'wanita dengan telinga kelinci: warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍🐰‍👩🏿' => 'wanita dengan telinga kelinci: warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍🐰‍👩🏻' => 'wanita dengan telinga kelinci: warna kulit gelap, warna kulit cerah',
    '👩🏿‍🐰‍👩🏼' => 'wanita dengan telinga kelinci: warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍🐰‍👩🏽' => 'wanita dengan telinga kelinci: warna kulit gelap, warna kulit sedang',
    '👩🏿‍🐰‍👩🏾' => 'wanita dengan telinga kelinci: warna kulit gelap, warna kulit gelap-sedang',
    '🧑🏻‍🤝‍🧑🏻' => 'orang bergandengan: warna kulit cerah',
    '🧑🏻‍🤝‍🧑🏼' => 'orang bergandengan: warna kulit cerah, warna kulit cerah-sedang',
    '🧑🏻‍🤝‍🧑🏽' => 'orang bergandengan: warna kulit cerah, warna kulit sedang',
    '🧑🏻‍🤝‍🧑🏾' => 'orang bergandengan: warna kulit cerah, warna kulit gelap-sedang',
    '🧑🏻‍🤝‍🧑🏿' => 'orang bergandengan: warna kulit cerah, warna kulit gelap',
    '🧑🏼‍🤝‍🧑🏻' => 'orang bergandengan: warna kulit cerah-sedang, warna kulit cerah',
    '🧑🏼‍🤝‍🧑🏼' => 'orang bergandengan: warna kulit cerah-sedang',
    '🧑🏼‍🤝‍🧑🏽' => 'orang bergandengan: warna kulit cerah-sedang, warna kulit sedang',
    '🧑🏼‍🤝‍🧑🏾' => 'orang bergandengan: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '🧑🏼‍🤝‍🧑🏿' => 'orang bergandengan: warna kulit cerah-sedang, warna kulit gelap',
    '🧑🏽‍🤝‍🧑🏻' => 'orang bergandengan: warna kulit sedang, warna kulit cerah',
    '🧑🏽‍🤝‍🧑🏼' => 'orang bergandengan: warna kulit sedang, warna kulit cerah-sedang',
    '🧑🏽‍🤝‍🧑🏽' => 'orang bergandengan: warna kulit sedang',
    '🧑🏽‍🤝‍🧑🏾' => 'orang bergandengan: warna kulit sedang, warna kulit gelap-sedang',
    '🧑🏽‍🤝‍🧑🏿' => 'orang bergandengan: warna kulit sedang, warna kulit gelap',
    '🧑🏾‍🤝‍🧑🏻' => 'orang bergandengan: warna kulit gelap-sedang, warna kulit cerah',
    '🧑🏾‍🤝‍🧑🏼' => 'orang bergandengan: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '🧑🏾‍🤝‍🧑🏽' => 'orang bergandengan: warna kulit gelap-sedang, warna kulit sedang',
    '🧑🏾‍🤝‍🧑🏾' => 'orang bergandengan: warna kulit gelap-sedang',
    '🧑🏾‍🤝‍🧑🏿' => 'orang bergandengan: warna kulit gelap-sedang, warna kulit gelap',
    '🧑🏿‍🤝‍🧑🏻' => 'orang bergandengan: warna kulit gelap, warna kulit cerah',
    '🧑🏿‍🤝‍🧑🏼' => 'orang bergandengan: warna kulit gelap, warna kulit cerah-sedang',
    '🧑🏿‍🤝‍🧑🏽' => 'orang bergandengan: warna kulit gelap, warna kulit sedang',
    '🧑🏿‍🤝‍🧑🏾' => 'orang bergandengan: warna kulit gelap, warna kulit gelap-sedang',
    '🧑🏿‍🤝‍🧑🏿' => 'orang bergandengan: warna kulit gelap',
    '👩🏻‍🤝‍👩🏼' => 'dua wanita bergandengan: warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍🤝‍👩🏽' => 'dua wanita bergandengan: warna kulit cerah, warna kulit sedang',
    '👩🏻‍🤝‍👩🏾' => 'dua wanita bergandengan: warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍🤝‍👩🏿' => 'dua wanita bergandengan: warna kulit cerah, warna kulit gelap',
    '👩🏼‍🤝‍👩🏻' => 'dua wanita bergandengan: warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍🤝‍👩🏽' => 'dua wanita bergandengan: warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍🤝‍👩🏾' => 'dua wanita bergandengan: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍🤝‍👩🏿' => 'dua wanita bergandengan: warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍🤝‍👩🏻' => 'dua wanita bergandengan: warna kulit sedang, warna kulit cerah',
    '👩🏽‍🤝‍👩🏼' => 'dua wanita bergandengan: warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍🤝‍👩🏾' => 'dua wanita bergandengan: warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍🤝‍👩🏿' => 'dua wanita bergandengan: warna kulit sedang, warna kulit gelap',
    '👩🏾‍🤝‍👩🏻' => 'dua wanita bergandengan: warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍🤝‍👩🏼' => 'dua wanita bergandengan: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍🤝‍👩🏽' => 'dua wanita bergandengan: warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍🤝‍👩🏿' => 'dua wanita bergandengan: warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍🤝‍👩🏻' => 'dua wanita bergandengan: warna kulit gelap, warna kulit cerah',
    '👩🏿‍🤝‍👩🏼' => 'dua wanita bergandengan: warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍🤝‍👩🏽' => 'dua wanita bergandengan: warna kulit gelap, warna kulit sedang',
    '👩🏿‍🤝‍👩🏾' => 'dua wanita bergandengan: warna kulit gelap, warna kulit gelap-sedang',
    '👩🏻‍🤝‍👨🏼' => 'pria dan wanita bergandengan: warna kulit cerah, warna kulit cerah-sedang',
    '👩🏻‍🤝‍👨🏽' => 'pria dan wanita bergandengan: warna kulit cerah, warna kulit sedang',
    '👩🏻‍🤝‍👨🏾' => 'pria dan wanita bergandengan: warna kulit cerah, warna kulit gelap-sedang',
    '👩🏻‍🤝‍👨🏿' => 'pria dan wanita bergandengan: warna kulit cerah, warna kulit gelap',
    '👩🏼‍🤝‍👨🏻' => 'pria dan wanita bergandengan: warna kulit cerah-sedang, warna kulit cerah',
    '👩🏼‍🤝‍👨🏽' => 'pria dan wanita bergandengan: warna kulit cerah-sedang, warna kulit sedang',
    '👩🏼‍🤝‍👨🏾' => 'pria dan wanita bergandengan: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👩🏼‍🤝‍👨🏿' => 'pria dan wanita bergandengan: warna kulit cerah-sedang, warna kulit gelap',
    '👩🏽‍🤝‍👨🏻' => 'pria dan wanita bergandengan: warna kulit sedang, warna kulit cerah',
    '👩🏽‍🤝‍👨🏼' => 'pria dan wanita bergandengan: warna kulit sedang, warna kulit cerah-sedang',
    '👩🏽‍🤝‍👨🏾' => 'pria dan wanita bergandengan: warna kulit sedang, warna kulit gelap-sedang',
    '👩🏽‍🤝‍👨🏿' => 'pria dan wanita bergandengan: warna kulit sedang, warna kulit gelap',
    '👩🏾‍🤝‍👨🏻' => 'pria dan wanita bergandengan: warna kulit gelap-sedang, warna kulit cerah',
    '👩🏾‍🤝‍👨🏼' => 'pria dan wanita bergandengan: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👩🏾‍🤝‍👨🏽' => 'pria dan wanita bergandengan: warna kulit gelap-sedang, warna kulit sedang',
    '👩🏾‍🤝‍👨🏿' => 'pria dan wanita bergandengan: warna kulit gelap-sedang, warna kulit gelap',
    '👩🏿‍🤝‍👨🏻' => 'pria dan wanita bergandengan: warna kulit gelap, warna kulit cerah',
    '👩🏿‍🤝‍👨🏼' => 'pria dan wanita bergandengan: warna kulit gelap, warna kulit cerah-sedang',
    '👩🏿‍🤝‍👨🏽' => 'pria dan wanita bergandengan: warna kulit gelap, warna kulit sedang',
    '👩🏿‍🤝‍👨🏾' => 'pria dan wanita bergandengan: warna kulit gelap, warna kulit gelap-sedang',
    '👨🏻‍🤝‍👨🏼' => 'dua pria bergandengan: warna kulit cerah, warna kulit cerah-sedang',
    '👨🏻‍🤝‍👨🏽' => 'dua pria bergandengan: warna kulit cerah, warna kulit sedang',
    '👨🏻‍🤝‍👨🏾' => 'dua pria bergandengan: warna kulit cerah, warna kulit gelap-sedang',
    '👨🏻‍🤝‍👨🏿' => 'dua pria bergandengan: warna kulit cerah, warna kulit gelap',
    '👨🏼‍🤝‍👨🏻' => 'dua pria bergandengan: warna kulit cerah-sedang, warna kulit cerah',
    '👨🏼‍🤝‍👨🏽' => 'dua pria bergandengan: warna kulit cerah-sedang, warna kulit sedang',
    '👨🏼‍🤝‍👨🏾' => 'dua pria bergandengan: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '👨🏼‍🤝‍👨🏿' => 'dua pria bergandengan: warna kulit cerah-sedang, warna kulit gelap',
    '👨🏽‍🤝‍👨🏻' => 'dua pria bergandengan: warna kulit sedang, warna kulit cerah',
    '👨🏽‍🤝‍👨🏼' => 'dua pria bergandengan: warna kulit sedang, warna kulit cerah-sedang',
    '👨🏽‍🤝‍👨🏾' => 'dua pria bergandengan: warna kulit sedang, warna kulit gelap-sedang',
    '👨🏽‍🤝‍👨🏿' => 'dua pria bergandengan: warna kulit sedang, warna kulit gelap',
    '👨🏾‍🤝‍👨🏻' => 'dua pria bergandengan: warna kulit gelap-sedang, warna kulit cerah',
    '👨🏾‍🤝‍👨🏼' => 'dua pria bergandengan: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '👨🏾‍🤝‍👨🏽' => 'dua pria bergandengan: warna kulit gelap-sedang, warna kulit sedang',
    '👨🏾‍🤝‍👨🏿' => 'dua pria bergandengan: warna kulit gelap-sedang, warna kulit gelap',
    '👨🏿‍🤝‍👨🏻' => 'dua pria bergandengan: warna kulit gelap, warna kulit cerah',
    '👨🏿‍🤝‍👨🏼' => 'dua pria bergandengan: warna kulit gelap, warna kulit cerah-sedang',
    '👨🏿‍🤝‍👨🏽' => 'dua pria bergandengan: warna kulit gelap, warna kulit sedang',
    '👨🏿‍🤝‍👨🏾' => 'dua pria bergandengan: warna kulit gelap, warna kulit gelap-sedang',
    '👩‍❤‍💋‍👨' => 'berciuman: wanita, pria',
    '👨‍❤‍💋‍👨' => 'berciuman: pria, pria',
    '👩‍❤‍💋‍👩' => 'berciuman: wanita, wanita',
    '👨‍👩‍👧‍👦' => 'keluarga: pria, wanita, anak perempuan, anak laki-laki',
    '👨‍👩‍👦‍👦' => 'keluarga: pria, wanita, anak laki-laki, anak laki-laki',
    '👨‍👩‍👧‍👧' => 'keluarga: pria, wanita, anak perempuan, anak perempuan',
    '👨‍👨‍👧‍👦' => 'keluarga: pria, pria, anak perempuan, anak laki-laki',
    '👨‍👨‍👦‍👦' => 'keluarga: pria, pria, anak laki-laki, anak laki-laki',
    '👨‍👨‍👧‍👧' => 'keluarga: pria, pria, anak perempuan, anak perempuan',
    '👩‍👩‍👧‍👦' => 'keluarga: wanita, wanita, anak perempuan, anak laki-laki',
    '👩‍👩‍👦‍👦' => 'keluarga: wanita, wanita, anak laki-laki, anak laki-laki',
    '👩‍👩‍👧‍👧' => 'keluarga: wanita, wanita, anak perempuan, anak perempuan',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'bendera: Inggris',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'bendera: Skotlandia',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'bendera: Wales',
    '🧑‍🧑‍🧒‍🧒' => 'keluarga: orang dewasa, orang dewasa, anak, anak',
    '🚶🏻‍♀‍➡' => 'wanita berjalan kaki: warna kulit cerah, menghadap kanan',
    '🚶🏼‍♀‍➡' => 'wanita berjalan kaki: warna kulit cerah-sedang, menghadap kanan',
    '🚶🏽‍♀‍➡' => 'wanita berjalan kaki: warna kulit sedang, menghadap kanan',
    '🚶🏾‍♀‍➡' => 'wanita berjalan kaki: warna kulit gelap-sedang, menghadap kanan',
    '🚶🏿‍♀‍➡' => 'wanita berjalan kaki: warna kulit gelap, menghadap kanan',
    '🚶🏻‍♂‍➡' => 'pria berjalan kaki: warna kulit cerah, menghadap kanan',
    '🚶🏼‍♂‍➡' => 'pria berjalan kaki: warna kulit cerah-sedang, menghadap kanan',
    '🚶🏽‍♂‍➡' => 'pria berjalan kaki: warna kulit sedang, menghadap kanan',
    '🚶🏾‍♂‍➡' => 'pria berjalan kaki: warna kulit gelap-sedang, menghadap kanan',
    '🚶🏿‍♂‍➡' => 'pria berjalan kaki: warna kulit gelap, menghadap kanan',
    '🧎🏻‍♀‍➡' => 'wanita berlutut: warna kulit cerah, menghadap kanan',
    '🧎🏼‍♀‍➡' => 'wanita berlutut: warna kulit cerah-sedang, menghadap kanan',
    '🧎🏽‍♀‍➡' => 'wanita berlutut: warna kulit sedang, menghadap kanan',
    '🧎🏾‍♀‍➡' => 'wanita berlutut: warna kulit gelap-sedang, menghadap kanan',
    '🧎🏿‍♀‍➡' => 'wanita berlutut: warna kulit gelap, menghadap kanan',
    '🧎🏻‍♂‍➡' => 'pria berlutut: warna kulit cerah, menghadap kanan',
    '🧎🏼‍♂‍➡' => 'pria berlutut: warna kulit cerah-sedang, menghadap kanan',
    '🧎🏽‍♂‍➡' => 'pria berlutut: warna kulit sedang, menghadap kanan',
    '🧎🏾‍♂‍➡' => 'pria berlutut: warna kulit gelap-sedang, menghadap kanan',
    '🧎🏿‍♂‍➡' => 'pria berlutut: warna kulit gelap, menghadap kanan',
    '🧑🏻‍🦯‍➡' => 'orang dengan tongkat: warna kulit cerah, menghadap kanan',
    '🧑🏼‍🦯‍➡' => 'orang dengan tongkat: warna kulit cerah-sedang, menghadap kanan',
    '🧑🏽‍🦯‍➡' => 'orang dengan tongkat: warna kulit sedang, menghadap kanan',
    '🧑🏾‍🦯‍➡' => 'orang dengan tongkat: warna kulit gelap-sedang, menghadap kanan',
    '🧑🏿‍🦯‍➡' => 'orang dengan tongkat: warna kulit gelap, menghadap kanan',
    '👨🏻‍🦯‍➡' => 'pria dengan tongkat: warna kulit cerah, menghadap kanan',
    '👨🏼‍🦯‍➡' => 'pria dengan tongkat: warna kulit cerah-sedang, menghadap kanan',
    '👨🏽‍🦯‍➡' => 'pria dengan tongkat: warna kulit sedang, menghadap kanan',
    '👨🏾‍🦯‍➡' => 'pria dengan tongkat: warna kulit gelap-sedang, menghadap kanan',
    '👨🏿‍🦯‍➡' => 'pria dengan tongkat: warna kulit gelap, menghadap kanan',
    '👩🏻‍🦯‍➡' => 'wanita dengan tongkat: warna kulit cerah, menghadap kanan',
    '👩🏼‍🦯‍➡' => 'wanita dengan tongkat: warna kulit cerah-sedang, menghadap kanan',
    '👩🏽‍🦯‍➡' => 'wanita dengan tongkat: warna kulit sedang, menghadap kanan',
    '👩🏾‍🦯‍➡' => 'wanita dengan tongkat: warna kulit gelap-sedang, menghadap kanan',
    '👩🏿‍🦯‍➡' => 'wanita dengan tongkat: warna kulit gelap, menghadap kanan',
    '🧑🏻‍🦼‍➡' => 'orang dengan kursi roda bermotor: warna kulit cerah, menghadap kanan',
    '🧑🏼‍🦼‍➡' => 'orang dengan kursi roda bermotor: warna kulit cerah-sedang, menghadap kanan',
    '🧑🏽‍🦼‍➡' => 'orang dengan kursi roda bermotor: warna kulit sedang, menghadap kanan',
    '🧑🏾‍🦼‍➡' => 'orang dengan kursi roda bermotor: warna kulit gelap-sedang, menghadap kanan',
    '🧑🏿‍🦼‍➡' => 'orang dengan kursi roda bermotor: warna kulit gelap, menghadap kanan',
    '👨🏻‍🦼‍➡' => 'pria dengan kursi roda bermotor: warna kulit cerah, menghadap kanan',
    '👨🏼‍🦼‍➡' => 'pria dengan kursi roda bermotor: warna kulit cerah-sedang, menghadap kanan',
    '👨🏽‍🦼‍➡' => 'pria dengan kursi roda bermotor: warna kulit sedang, menghadap kanan',
    '👨🏾‍🦼‍➡' => 'pria dengan kursi roda bermotor: warna kulit gelap-sedang, menghadap kanan',
    '👨🏿‍🦼‍➡' => 'pria dengan kursi roda bermotor: warna kulit gelap, menghadap kanan',
    '👩🏻‍🦼‍➡' => 'wanita dengan kursi roda bermotor: warna kulit cerah, menghadap kanan',
    '👩🏼‍🦼‍➡' => 'wanita dengan kursi roda bermotor: warna kulit cerah-sedang, menghadap kanan',
    '👩🏽‍🦼‍➡' => 'wanita dengan kursi roda bermotor: warna kulit sedang, menghadap kanan',
    '👩🏾‍🦼‍➡' => 'wanita dengan kursi roda bermotor: warna kulit gelap-sedang, menghadap kanan',
    '👩🏿‍🦼‍➡' => 'wanita dengan kursi roda bermotor: warna kulit gelap, menghadap kanan',
    '🧑🏻‍🦽‍➡' => 'orang dengan kursi roda manual: warna kulit cerah, menghadap kanan',
    '🧑🏼‍🦽‍➡' => 'orang dengan kursi roda manual: warna kulit cerah-sedang, menghadap kanan',
    '🧑🏽‍🦽‍➡' => 'orang dengan kursi roda manual: warna kulit sedang, menghadap kanan',
    '🧑🏾‍🦽‍➡' => 'orang dengan kursi roda manual: warna kulit gelap-sedang, menghadap kanan',
    '🧑🏿‍🦽‍➡' => 'orang dengan kursi roda manual: warna kulit gelap, menghadap kanan',
    '👨🏻‍🦽‍➡' => 'pria dengan kursi roda manual: warna kulit cerah, menghadap kanan',
    '👨🏼‍🦽‍➡' => 'pria dengan kursi roda manual: warna kulit cerah-sedang, menghadap kanan',
    '👨🏽‍🦽‍➡' => 'pria dengan kursi roda manual: warna kulit sedang, menghadap kanan',
    '👨🏾‍🦽‍➡' => 'pria dengan kursi roda manual: warna kulit gelap-sedang, menghadap kanan',
    '👨🏿‍🦽‍➡' => 'pria dengan kursi roda manual: warna kulit gelap, menghadap kanan',
    '👩🏻‍🦽‍➡' => 'wanita dengan kursi roda manual: warna kulit cerah, menghadap kanan',
    '👩🏼‍🦽‍➡' => 'wanita dengan kursi roda manual: warna kulit cerah-sedang, menghadap kanan',
    '👩🏽‍🦽‍➡' => 'wanita dengan kursi roda manual: warna kulit sedang, menghadap kanan',
    '👩🏾‍🦽‍➡' => 'wanita dengan kursi roda manual: warna kulit gelap-sedang, menghadap kanan',
    '👩🏿‍🦽‍➡' => 'wanita dengan kursi roda manual: warna kulit gelap, menghadap kanan',
    '🏃🏻‍♀‍➡' => 'wanita berlari: warna kulit cerah, menghadap kanan',
    '🏃🏼‍♀‍➡' => 'wanita berlari: warna kulit cerah-sedang, menghadap kanan',
    '🏃🏽‍♀‍➡' => 'wanita berlari: warna kulit sedang, menghadap kanan',
    '🏃🏾‍♀‍➡' => 'wanita berlari: warna kulit gelap-sedang, menghadap kanan',
    '🏃🏿‍♀‍➡' => 'wanita berlari: warna kulit gelap, menghadap kanan',
    '🏃🏻‍♂‍➡' => 'pria berlari: warna kulit cerah, menghadap kanan',
    '🏃🏼‍♂‍➡' => 'pria berlari: warna kulit cerah-sedang, menghadap kanan',
    '🏃🏽‍♂‍➡' => 'pria berlari: warna kulit sedang, menghadap kanan',
    '🏃🏾‍♂‍➡' => 'pria berlari: warna kulit gelap-sedang, menghadap kanan',
    '🏃🏿‍♂‍➡' => 'pria berlari: warna kulit gelap, menghadap kanan',
    '🫱🏻‍🫲🏼' => 'jabat tangan: warna kulit cerah, warna kulit cerah-sedang',
    '🫱🏻‍🫲🏽' => 'jabat tangan: warna kulit cerah, warna kulit sedang',
    '🫱🏻‍🫲🏾' => 'jabat tangan: warna kulit cerah, warna kulit gelap-sedang',
    '🫱🏻‍🫲🏿' => 'jabat tangan: warna kulit cerah, warna kulit gelap',
    '🫱🏼‍🫲🏻' => 'jabat tangan: warna kulit cerah-sedang, warna kulit cerah',
    '🫱🏼‍🫲🏽' => 'jabat tangan: warna kulit cerah-sedang, warna kulit sedang',
    '🫱🏼‍🫲🏾' => 'jabat tangan: warna kulit cerah-sedang, warna kulit gelap-sedang',
    '🫱🏼‍🫲🏿' => 'jabat tangan: warna kulit cerah-sedang, warna kulit gelap',
    '🫱🏽‍🫲🏻' => 'jabat tangan: warna kulit sedang, warna kulit cerah',
    '🫱🏽‍🫲🏼' => 'jabat tangan: warna kulit sedang, warna kulit cerah-sedang',
    '🫱🏽‍🫲🏾' => 'jabat tangan: warna kulit sedang, warna kulit gelap-sedang',
    '🫱🏽‍🫲🏿' => 'jabat tangan: warna kulit sedang, warna kulit gelap',
    '🫱🏾‍🫲🏻' => 'jabat tangan: warna kulit gelap-sedang, warna kulit cerah',
    '🫱🏾‍🫲🏼' => 'jabat tangan: warna kulit gelap-sedang, warna kulit cerah-sedang',
    '🫱🏾‍🫲🏽' => 'jabat tangan: warna kulit gelap-sedang, warna kulit sedang',
    '🫱🏾‍🫲🏿' => 'jabat tangan: warna kulit gelap-sedang, warna kulit gelap',
    '🫱🏿‍🫲🏻' => 'jabat tangan: warna kulit gelap, warna kulit cerah',
    '🫱🏿‍🫲🏼' => 'jabat tangan: warna kulit gelap, warna kulit cerah-sedang',
    '🫱🏿‍🫲🏽' => 'jabat tangan: warna kulit gelap, warna kulit sedang',
    '🫱🏿‍🫲🏾' => 'jabat tangan: warna kulit gelap, warna kulit gelap-sedang',
    '🚶‍♀‍➡' => 'wanita berjalan kaki: menghadap kanan',
    '🚶‍♂‍➡' => 'pria berjalan kaki: menghadap kanan',
    '🧎‍♀‍➡' => 'wanita berlutut: menghadap kanan',
    '🧎‍♂‍➡' => 'pria berlutut: menghadap kanan',
    '🧑‍🦯‍➡' => 'orang dengan tongkat: menghadap kanan',
    '👨‍🦯‍➡' => 'pria dengan tongkat: menghadap kanan',
    '👩‍🦯‍➡' => 'wanita dengan tongkat: menghadap kanan',
    '🧑‍🦼‍➡' => 'orang dengan kursi roda bermotor: menghadap kanan',
    '👨‍🦼‍➡' => 'pria dengan kursi roda bermotor: menghadap kanan',
    '👩‍🦼‍➡' => 'wanita dengan kursi roda bermotor: menghadap kanan',
    '🧑‍🦽‍➡' => 'orang dengan kursi roda manual: menghadap kanan',
    '👨‍🦽‍➡' => 'pria dengan kursi roda manual: menghadap kanan',
    '👩‍🦽‍➡' => 'wanita dengan kursi roda manual: menghadap kanan',
    '🏃‍♀‍➡' => 'wanita berlari: menghadap kanan',
    '🏃‍♂‍➡' => 'pria berlari: menghadap kanan',
    '👩‍❤‍👨' => 'pasangan dengan hati: wanita, pria',
    '👨‍❤‍👨' => 'pasangan dengan hati: pria, pria',
    '👩‍❤‍👩' => 'pasangan dengan hati: wanita, wanita',
    '👨‍👩‍👦' => 'keluarga: pria, wanita, anak laki-laki',
    '👨‍👩‍👧' => 'keluarga: pria, wanita, anak perempuan',
    '👨‍👨‍👦' => 'keluarga: pria, pria, anak laki-laki',
    '👨‍👨‍👧' => 'keluarga: pria, pria, anak perempuan',
    '👩‍👩‍👦' => 'keluarga: wanita, wanita, anak laki-laki',
    '👩‍👩‍👧' => 'keluarga: wanita, wanita, anak perempuan',
    '👨‍👦‍👦' => 'keluarga: pria, anak laki-laki, anak laki-laki',
    '👨‍👧‍👦' => 'keluarga: pria, anak perempuan, anak laki-laki',
    '👨‍👧‍👧' => 'keluarga: pria, anak perempuan, anak perempuan',
    '👩‍👦‍👦' => 'keluarga: wanita, anak laki-laki, anak laki-laki',
    '👩‍👧‍👦' => 'keluarga: wanita, anak perempuan, anak laki-laki',
    '👩‍👧‍👧' => 'keluarga: wanita, anak perempuan, anak perempuan',
    '🧑‍🤝‍🧑' => 'orang bergandengan',
    '🧑‍🧑‍🧒' => 'keluarga: orang dewasa, orang dewasa, anak',
    '🧑‍🧒‍🧒' => 'keluarga: orang dewasa, anak, anak',
    '🧑🏻‍🦰' => 'dewasa: warna kulit cerah, rambut merah',
    '🧑🏼‍🦰' => 'dewasa: warna kulit cerah-sedang, rambut merah',
    '🧑🏽‍🦰' => 'dewasa: warna kulit sedang, rambut merah',
    '🧑🏾‍🦰' => 'dewasa: warna kulit gelap-sedang, rambut merah',
    '🧑🏿‍🦰' => 'dewasa: warna kulit gelap, rambut merah',
    '🧑🏻‍🦱' => 'dewasa: warna kulit cerah, rambut keriting',
    '🧑🏼‍🦱' => 'dewasa: warna kulit cerah-sedang, rambut keriting',
    '🧑🏽‍🦱' => 'dewasa: warna kulit sedang, rambut keriting',
    '🧑🏾‍🦱' => 'dewasa: warna kulit gelap-sedang, rambut keriting',
    '🧑🏿‍🦱' => 'dewasa: warna kulit gelap, rambut keriting',
    '🧑🏻‍🦳' => 'dewasa: warna kulit cerah, rambut putih',
    '🧑🏼‍🦳' => 'dewasa: warna kulit cerah-sedang, rambut putih',
    '🧑🏽‍🦳' => 'dewasa: warna kulit sedang, rambut putih',
    '🧑🏾‍🦳' => 'dewasa: warna kulit gelap-sedang, rambut putih',
    '🧑🏿‍🦳' => 'dewasa: warna kulit gelap, rambut putih',
    '🧑🏻‍🦲' => 'dewasa: warna kulit cerah, gundul',
    '🧑🏼‍🦲' => 'dewasa: warna kulit cerah-sedang, gundul',
    '🧑🏽‍🦲' => 'dewasa: warna kulit sedang, gundul',
    '🧑🏾‍🦲' => 'dewasa: warna kulit gelap-sedang, gundul',
    '🧑🏿‍🦲' => 'dewasa: warna kulit gelap, gundul',
    '🧑🏻‍🩰' => 'penari balet: warna kulit cerah',
    '🧑🏼‍🩰' => 'penari balet: warna kulit cerah-sedang',
    '🧑🏽‍🩰' => 'penari balet: warna kulit sedang',
    '🧑🏾‍🩰' => 'penari balet: warna kulit gelap-sedang',
    '🧑🏿‍🩰' => 'penari balet: warna kulit gelap',
    '🧔🏻‍♂' => 'pria: warna kulit cerah, janggut',
    '🧔🏼‍♂' => 'pria: warna kulit cerah-sedang, janggut',
    '🧔🏽‍♂' => 'pria: warna kulit sedang, janggut',
    '🧔🏾‍♂' => 'pria: warna kulit gelap-sedang, janggut',
    '🧔🏿‍♂' => 'pria: warna kulit gelap, janggut',
    '👱🏻‍♂' => 'pria berambut pirang: warna kulit cerah',
    '👱🏼‍♂' => 'pria berambut pirang: warna kulit cerah-sedang',
    '👱🏽‍♂' => 'pria berambut pirang: warna kulit sedang',
    '👱🏾‍♂' => 'pria berambut pirang: warna kulit gelap-sedang',
    '👱🏿‍♂' => 'pria berambut pirang: warna kulit gelap',
    '👨🏻‍🦰' => 'pria: warna kulit cerah, rambut merah',
    '👨🏼‍🦰' => 'pria: warna kulit cerah-sedang, rambut merah',
    '👨🏽‍🦰' => 'pria: warna kulit sedang, rambut merah',
    '👨🏾‍🦰' => 'pria: warna kulit gelap-sedang, rambut merah',
    '👨🏿‍🦰' => 'pria: warna kulit gelap, rambut merah',
    '👨🏻‍🦱' => 'pria: warna kulit cerah, rambut keriting',
    '👨🏼‍🦱' => 'pria: warna kulit cerah-sedang, rambut keriting',
    '👨🏽‍🦱' => 'pria: warna kulit sedang, rambut keriting',
    '👨🏾‍🦱' => 'pria: warna kulit gelap-sedang, rambut keriting',
    '👨🏿‍🦱' => 'pria: warna kulit gelap, rambut keriting',
    '👨🏻‍🦳' => 'pria: warna kulit cerah, rambut putih',
    '👨🏼‍🦳' => 'pria: warna kulit cerah-sedang, rambut putih',
    '👨🏽‍🦳' => 'pria: warna kulit sedang, rambut putih',
    '👨🏾‍🦳' => 'pria: warna kulit gelap-sedang, rambut putih',
    '👨🏿‍🦳' => 'pria: warna kulit gelap, rambut putih',
    '👨🏻‍🦲' => 'pria: warna kulit cerah, gundul',
    '👨🏼‍🦲' => 'pria: warna kulit cerah-sedang, gundul',
    '👨🏽‍🦲' => 'pria: warna kulit sedang, gundul',
    '👨🏾‍🦲' => 'pria: warna kulit gelap-sedang, gundul',
    '👨🏿‍🦲' => 'pria: warna kulit gelap, gundul',
    '🧔🏻‍♀' => 'wanita: warna kulit cerah, janggut',
    '🧔🏼‍♀' => 'wanita: warna kulit cerah-sedang, janggut',
    '🧔🏽‍♀' => 'wanita: warna kulit sedang, janggut',
    '🧔🏾‍♀' => 'wanita: warna kulit gelap-sedang, janggut',
    '🧔🏿‍♀' => 'wanita: warna kulit gelap, janggut',
    '👱🏻‍♀' => 'wanita berambut pirang: warna kulit cerah',
    '👱🏼‍♀' => 'wanita berambut pirang: warna kulit cerah-sedang',
    '👱🏽‍♀' => 'wanita berambut pirang: warna kulit sedang',
    '👱🏾‍♀' => 'wanita berambut pirang: warna kulit gelap-sedang',
    '👱🏿‍♀' => 'wanita berambut pirang: warna kulit gelap',
    '👩🏻‍🦰' => 'wanita: warna kulit cerah, rambut merah',
    '👩🏼‍🦰' => 'wanita: warna kulit cerah-sedang, rambut merah',
    '👩🏽‍🦰' => 'wanita: warna kulit sedang, rambut merah',
    '👩🏾‍🦰' => 'wanita: warna kulit gelap-sedang, rambut merah',
    '👩🏿‍🦰' => 'wanita: warna kulit gelap, rambut merah',
    '👩🏻‍🦱' => 'wanita: warna kulit cerah, rambut keriting',
    '👩🏼‍🦱' => 'wanita: warna kulit cerah-sedang, rambut keriting',
    '👩🏽‍🦱' => 'wanita: warna kulit sedang, rambut keriting',
    '👩🏾‍🦱' => 'wanita: warna kulit gelap-sedang, rambut keriting',
    '👩🏿‍🦱' => 'wanita: warna kulit gelap, rambut keriting',
    '👩🏻‍🦳' => 'wanita: warna kulit cerah, rambut putih',
    '👩🏼‍🦳' => 'wanita: warna kulit cerah-sedang, rambut putih',
    '👩🏽‍🦳' => 'wanita: warna kulit sedang, rambut putih',
    '👩🏾‍🦳' => 'wanita: warna kulit gelap-sedang, rambut putih',
    '👩🏿‍🦳' => 'wanita: warna kulit gelap, rambut putih',
    '👩🏻‍🦲' => 'wanita: warna kulit cerah, gundul',
    '👩🏼‍🦲' => 'wanita: warna kulit cerah-sedang, gundul',
    '👩🏽‍🦲' => 'wanita: warna kulit sedang, gundul',
    '👩🏾‍🦲' => 'wanita: warna kulit gelap-sedang, gundul',
    '👩🏿‍🦲' => 'wanita: warna kulit gelap, gundul',
    '🙍🏻‍♂' => 'pria muka tidak senang: warna kulit cerah',
    '🙍🏼‍♂' => 'pria muka tidak senang: warna kulit cerah-sedang',
    '🙍🏽‍♂' => 'pria muka tidak senang: warna kulit sedang',
    '🙍🏾‍♂' => 'pria muka tidak senang: warna kulit gelap-sedang',
    '🙍🏿‍♂' => 'pria muka tidak senang: warna kulit gelap',
    '🙍🏻‍♀' => 'wanita muka tidak senang: warna kulit cerah',
    '🙍🏼‍♀' => 'wanita muka tidak senang: warna kulit cerah-sedang',
    '🙍🏽‍♀' => 'wanita muka tidak senang: warna kulit sedang',
    '🙍🏾‍♀' => 'wanita muka tidak senang: warna kulit gelap-sedang',
    '🙍🏿‍♀' => 'wanita muka tidak senang: warna kulit gelap',
    '🙎🏻‍♂' => 'pria tidak senang: warna kulit cerah',
    '🙎🏼‍♂' => 'pria tidak senang: warna kulit cerah-sedang',
    '🙎🏽‍♂' => 'pria tidak senang: warna kulit sedang',
    '🙎🏾‍♂' => 'pria tidak senang: warna kulit gelap-sedang',
    '🙎🏿‍♂' => 'pria tidak senang: warna kulit gelap',
    '🙎🏻‍♀' => 'wanita tidak senang: warna kulit cerah',
    '🙎🏼‍♀' => 'wanita tidak senang: warna kulit cerah-sedang',
    '🙎🏽‍♀' => 'wanita tidak senang: warna kulit sedang',
    '🙎🏾‍♀' => 'wanita tidak senang: warna kulit gelap-sedang',
    '🙎🏿‍♀' => 'wanita tidak senang: warna kulit gelap',
    '🙅🏻‍♂' => 'pria mengisyaratkan TIDAK: warna kulit cerah',
    '🙅🏼‍♂' => 'pria mengisyaratkan TIDAK: warna kulit cerah-sedang',
    '🙅🏽‍♂' => 'pria mengisyaratkan TIDAK: warna kulit sedang',
    '🙅🏾‍♂' => 'pria mengisyaratkan TIDAK: warna kulit gelap-sedang',
    '🙅🏿‍♂' => 'pria mengisyaratkan TIDAK: warna kulit gelap',
    '🙅🏻‍♀' => 'wanita mengisyaratkan TIDAK: warna kulit cerah',
    '🙅🏼‍♀' => 'wanita mengisyaratkan TIDAK: warna kulit cerah-sedang',
    '🙅🏽‍♀' => 'wanita mengisyaratkan TIDAK: warna kulit sedang',
    '🙅🏾‍♀' => 'wanita mengisyaratkan TIDAK: warna kulit gelap-sedang',
    '🙅🏿‍♀' => 'wanita mengisyaratkan TIDAK: warna kulit gelap',
    '🙆🏻‍♂' => 'pria mengisyaratkan OKE: warna kulit cerah',
    '🙆🏼‍♂' => 'pria mengisyaratkan OKE: warna kulit cerah-sedang',
    '🙆🏽‍♂' => 'pria mengisyaratkan OKE: warna kulit sedang',
    '🙆🏾‍♂' => 'pria mengisyaratkan OKE: warna kulit gelap-sedang',
    '🙆🏿‍♂' => 'pria mengisyaratkan OKE: warna kulit gelap',
    '🙆🏻‍♀' => 'wanita mengisyaratkan OKE: warna kulit cerah',
    '🙆🏼‍♀' => 'wanita mengisyaratkan OKE: warna kulit cerah-sedang',
    '🙆🏽‍♀' => 'wanita mengisyaratkan OKE: warna kulit sedang',
    '🙆🏾‍♀' => 'wanita mengisyaratkan OKE: warna kulit gelap-sedang',
    '🙆🏿‍♀' => 'wanita mengisyaratkan OKE: warna kulit gelap',
    '💁🏻‍♂' => 'pria menengadahkan tangan: warna kulit cerah',
    '💁🏼‍♂' => 'pria menengadahkan tangan: warna kulit cerah-sedang',
    '💁🏽‍♂' => 'pria menengadahkan tangan: warna kulit sedang',
    '💁🏾‍♂' => 'pria menengadahkan tangan: warna kulit gelap-sedang',
    '💁🏿‍♂' => 'pria menengadahkan tangan: warna kulit gelap',
    '💁🏻‍♀' => 'wanita menengadahkan tangan: warna kulit cerah',
    '💁🏼‍♀' => 'wanita menengadahkan tangan: warna kulit cerah-sedang',
    '💁🏽‍♀' => 'wanita menengadahkan tangan: warna kulit sedang',
    '💁🏾‍♀' => 'wanita menengadahkan tangan: warna kulit gelap-sedang',
    '💁🏿‍♀' => 'wanita menengadahkan tangan: warna kulit gelap',
    '🙋🏻‍♂' => 'pria mengangkat tangan: warna kulit cerah',
    '🙋🏼‍♂' => 'pria mengangkat tangan: warna kulit cerah-sedang',
    '🙋🏽‍♂' => 'pria mengangkat tangan: warna kulit sedang',
    '🙋🏾‍♂' => 'pria mengangkat tangan: warna kulit gelap-sedang',
    '🙋🏿‍♂' => 'pria mengangkat tangan: warna kulit gelap',
    '🙋🏻‍♀' => 'wanita mengangkat tangan: warna kulit cerah',
    '🙋🏼‍♀' => 'wanita mengangkat tangan: warna kulit cerah-sedang',
    '🙋🏽‍♀' => 'wanita mengangkat tangan: warna kulit sedang',
    '🙋🏾‍♀' => 'wanita mengangkat tangan: warna kulit gelap-sedang',
    '🙋🏿‍♀' => 'wanita mengangkat tangan: warna kulit gelap',
    '🧏🏻‍♂' => 'pria tuli: warna kulit cerah',
    '🧏🏼‍♂' => 'pria tuli: warna kulit cerah-sedang',
    '🧏🏽‍♂' => 'pria tuli: warna kulit sedang',
    '🧏🏾‍♂' => 'pria tuli: warna kulit gelap-sedang',
    '🧏🏿‍♂' => 'pria tuli: warna kulit gelap',
    '🧏🏻‍♀' => 'wanita tuli: warna kulit cerah',
    '🧏🏼‍♀' => 'wanita tuli: warna kulit cerah-sedang',
    '🧏🏽‍♀' => 'wanita tuli: warna kulit sedang',
    '🧏🏾‍♀' => 'wanita tuli: warna kulit gelap-sedang',
    '🧏🏿‍♀' => 'wanita tuli: warna kulit gelap',
    '🙇🏻‍♂' => 'pria membungkuk: warna kulit cerah',
    '🙇🏼‍♂' => 'pria membungkuk: warna kulit cerah-sedang',
    '🙇🏽‍♂' => 'pria membungkuk: warna kulit sedang',
    '🙇🏾‍♂' => 'pria membungkuk: warna kulit gelap-sedang',
    '🙇🏿‍♂' => 'pria membungkuk: warna kulit gelap',
    '🙇🏻‍♀' => 'wanita membungkuk: warna kulit cerah',
    '🙇🏼‍♀' => 'wanita membungkuk: warna kulit cerah-sedang',
    '🙇🏽‍♀' => 'wanita membungkuk: warna kulit sedang',
    '🙇🏾‍♀' => 'wanita membungkuk: warna kulit gelap-sedang',
    '🙇🏿‍♀' => 'wanita membungkuk: warna kulit gelap',
    '🤦🏻‍♂' => 'pria tepuk jidat: warna kulit cerah',
    '🤦🏼‍♂' => 'pria tepuk jidat: warna kulit cerah-sedang',
    '🤦🏽‍♂' => 'pria tepuk jidat: warna kulit sedang',
    '🤦🏾‍♂' => 'pria tepuk jidat: warna kulit gelap-sedang',
    '🤦🏿‍♂' => 'pria tepuk jidat: warna kulit gelap',
    '🤦🏻‍♀' => 'wanita tepuk jidat: warna kulit cerah',
    '🤦🏼‍♀' => 'wanita tepuk jidat: warna kulit cerah-sedang',
    '🤦🏽‍♀' => 'wanita tepuk jidat: warna kulit sedang',
    '🤦🏾‍♀' => 'wanita tepuk jidat: warna kulit gelap-sedang',
    '🤦🏿‍♀' => 'wanita tepuk jidat: warna kulit gelap',
    '🤷🏻‍♂' => 'pria mengangkat bahu: warna kulit cerah',
    '🤷🏼‍♂' => 'pria mengangkat bahu: warna kulit cerah-sedang',
    '🤷🏽‍♂' => 'pria mengangkat bahu: warna kulit sedang',
    '🤷🏾‍♂' => 'pria mengangkat bahu: warna kulit gelap-sedang',
    '🤷🏿‍♂' => 'pria mengangkat bahu: warna kulit gelap',
    '🤷🏻‍♀' => 'wanita mengangkat bahu: warna kulit cerah',
    '🤷🏼‍♀' => 'wanita mengangkat bahu: warna kulit cerah-sedang',
    '🤷🏽‍♀' => 'wanita mengangkat bahu: warna kulit sedang',
    '🤷🏾‍♀' => 'wanita mengangkat bahu: warna kulit gelap-sedang',
    '🤷🏿‍♀' => 'wanita mengangkat bahu: warna kulit gelap',
    '🧑🏻‍⚕' => 'petugas kesehatan: warna kulit cerah',
    '🧑🏼‍⚕' => 'petugas kesehatan: warna kulit cerah-sedang',
    '🧑🏽‍⚕' => 'petugas kesehatan: warna kulit sedang',
    '🧑🏾‍⚕' => 'petugas kesehatan: warna kulit gelap-sedang',
    '🧑🏿‍⚕' => 'petugas kesehatan: warna kulit gelap',
    '👨🏻‍⚕' => 'pekerja kesehatan pria: warna kulit cerah',
    '👨🏼‍⚕' => 'pekerja kesehatan pria: warna kulit cerah-sedang',
    '👨🏽‍⚕' => 'pekerja kesehatan pria: warna kulit sedang',
    '👨🏾‍⚕' => 'pekerja kesehatan pria: warna kulit gelap-sedang',
    '👨🏿‍⚕' => 'pekerja kesehatan pria: warna kulit gelap',
    '👩🏻‍⚕' => 'pekerja kesehatan wanita: warna kulit cerah',
    '👩🏼‍⚕' => 'pekerja kesehatan wanita: warna kulit cerah-sedang',
    '👩🏽‍⚕' => 'pekerja kesehatan wanita: warna kulit sedang',
    '👩🏾‍⚕' => 'pekerja kesehatan wanita: warna kulit gelap-sedang',
    '👩🏿‍⚕' => 'pekerja kesehatan wanita: warna kulit gelap',
    '🧑🏻‍🎓' => 'murid: warna kulit cerah',
    '🧑🏼‍🎓' => 'murid: warna kulit cerah-sedang',
    '🧑🏽‍🎓' => 'murid: warna kulit sedang',
    '🧑🏾‍🎓' => 'murid: warna kulit gelap-sedang',
    '🧑🏿‍🎓' => 'murid: warna kulit gelap',
    '👨🏻‍🎓' => 'sarjana pria: warna kulit cerah',
    '👨🏼‍🎓' => 'sarjana pria: warna kulit cerah-sedang',
    '👨🏽‍🎓' => 'sarjana pria: warna kulit sedang',
    '👨🏾‍🎓' => 'sarjana pria: warna kulit gelap-sedang',
    '👨🏿‍🎓' => 'sarjana pria: warna kulit gelap',
    '👩🏻‍🎓' => 'sarjana wanita: warna kulit cerah',
    '👩🏼‍🎓' => 'sarjana wanita: warna kulit cerah-sedang',
    '👩🏽‍🎓' => 'sarjana wanita: warna kulit sedang',
    '👩🏾‍🎓' => 'sarjana wanita: warna kulit gelap-sedang',
    '👩🏿‍🎓' => 'sarjana wanita: warna kulit gelap',
    '🧑🏻‍🏫' => 'guru: warna kulit cerah',
    '🧑🏼‍🏫' => 'guru: warna kulit cerah-sedang',
    '🧑🏽‍🏫' => 'guru: warna kulit sedang',
    '🧑🏾‍🏫' => 'guru: warna kulit gelap-sedang',
    '🧑🏿‍🏫' => 'guru: warna kulit gelap',
    '👨🏻‍🏫' => 'pengajar pria: warna kulit cerah',
    '👨🏼‍🏫' => 'pengajar pria: warna kulit cerah-sedang',
    '👨🏽‍🏫' => 'pengajar pria: warna kulit sedang',
    '👨🏾‍🏫' => 'pengajar pria: warna kulit gelap-sedang',
    '👨🏿‍🏫' => 'pengajar pria: warna kulit gelap',
    '👩🏻‍🏫' => 'pengajar wanita: warna kulit cerah',
    '👩🏼‍🏫' => 'pengajar wanita: warna kulit cerah-sedang',
    '👩🏽‍🏫' => 'pengajar wanita: warna kulit sedang',
    '👩🏾‍🏫' => 'pengajar wanita: warna kulit gelap-sedang',
    '👩🏿‍🏫' => 'pengajar wanita: warna kulit gelap',
    '🧑🏻‍⚖' => 'hakim: warna kulit cerah',
    '🧑🏼‍⚖' => 'hakim: warna kulit cerah-sedang',
    '🧑🏽‍⚖' => 'hakim: warna kulit sedang',
    '🧑🏾‍⚖' => 'hakim: warna kulit gelap-sedang',
    '🧑🏿‍⚖' => 'hakim: warna kulit gelap',
    '👨🏻‍⚖' => 'hakim pria: warna kulit cerah',
    '👨🏼‍⚖' => 'hakim pria: warna kulit cerah-sedang',
    '👨🏽‍⚖' => 'hakim pria: warna kulit sedang',
    '👨🏾‍⚖' => 'hakim pria: warna kulit gelap-sedang',
    '👨🏿‍⚖' => 'hakim pria: warna kulit gelap',
    '👩🏻‍⚖' => 'hakim wanita: warna kulit cerah',
    '👩🏼‍⚖' => 'hakim wanita: warna kulit cerah-sedang',
    '👩🏽‍⚖' => 'hakim wanita: warna kulit sedang',
    '👩🏾‍⚖' => 'hakim wanita: warna kulit gelap-sedang',
    '👩🏿‍⚖' => 'hakim wanita: warna kulit gelap',
    '🧑🏻‍🌾' => 'petani: warna kulit cerah',
    '🧑🏼‍🌾' => 'petani: warna kulit cerah-sedang',
    '🧑🏽‍🌾' => 'petani: warna kulit sedang',
    '🧑🏾‍🌾' => 'petani: warna kulit gelap-sedang',
    '🧑🏿‍🌾' => 'petani: warna kulit gelap',
    '👨🏻‍🌾' => 'petani pria: warna kulit cerah',
    '👨🏼‍🌾' => 'petani pria: warna kulit cerah-sedang',
    '👨🏽‍🌾' => 'petani pria: warna kulit sedang',
    '👨🏾‍🌾' => 'petani pria: warna kulit gelap-sedang',
    '👨🏿‍🌾' => 'petani pria: warna kulit gelap',
    '👩🏻‍🌾' => 'petani wanita: warna kulit cerah',
    '👩🏼‍🌾' => 'petani wanita: warna kulit cerah-sedang',
    '👩🏽‍🌾' => 'petani wanita: warna kulit sedang',
    '👩🏾‍🌾' => 'petani wanita: warna kulit gelap-sedang',
    '👩🏿‍🌾' => 'petani wanita: warna kulit gelap',
    '🧑🏻‍🍳' => 'koki: warna kulit cerah',
    '🧑🏼‍🍳' => 'koki: warna kulit cerah-sedang',
    '🧑🏽‍🍳' => 'koki: warna kulit sedang',
    '🧑🏾‍🍳' => 'koki: warna kulit gelap-sedang',
    '🧑🏿‍🍳' => 'koki: warna kulit gelap',
    '👨🏻‍🍳' => 'koki pria: warna kulit cerah',
    '👨🏼‍🍳' => 'koki pria: warna kulit cerah-sedang',
    '👨🏽‍🍳' => 'koki pria: warna kulit sedang',
    '👨🏾‍🍳' => 'koki pria: warna kulit gelap-sedang',
    '👨🏿‍🍳' => 'koki pria: warna kulit gelap',
    '👩🏻‍🍳' => 'koki wanita: warna kulit cerah',
    '👩🏼‍🍳' => 'koki wanita: warna kulit cerah-sedang',
    '👩🏽‍🍳' => 'koki wanita: warna kulit sedang',
    '👩🏾‍🍳' => 'koki wanita: warna kulit gelap-sedang',
    '👩🏿‍🍳' => 'koki wanita: warna kulit gelap',
    '🧑🏻‍🔧' => 'mekanik: warna kulit cerah',
    '🧑🏼‍🔧' => 'mekanik: warna kulit cerah-sedang',
    '🧑🏽‍🔧' => 'mekanik: warna kulit sedang',
    '🧑🏾‍🔧' => 'mekanik: warna kulit gelap-sedang',
    '🧑🏿‍🔧' => 'mekanik: warna kulit gelap',
    '👨🏻‍🔧' => 'montir pria: warna kulit cerah',
    '👨🏼‍🔧' => 'montir pria: warna kulit cerah-sedang',
    '👨🏽‍🔧' => 'montir pria: warna kulit sedang',
    '👨🏾‍🔧' => 'montir pria: warna kulit gelap-sedang',
    '👨🏿‍🔧' => 'montir pria: warna kulit gelap',
    '👩🏻‍🔧' => 'montir wanita: warna kulit cerah',
    '👩🏼‍🔧' => 'montir wanita: warna kulit cerah-sedang',
    '👩🏽‍🔧' => 'montir wanita: warna kulit sedang',
    '👩🏾‍🔧' => 'montir wanita: warna kulit gelap-sedang',
    '👩🏿‍🔧' => 'montir wanita: warna kulit gelap',
    '🧑🏻‍🏭' => 'pekerja pabrik: warna kulit cerah',
    '🧑🏼‍🏭' => 'pekerja pabrik: warna kulit cerah-sedang',
    '🧑🏽‍🏭' => 'pekerja pabrik: warna kulit sedang',
    '🧑🏾‍🏭' => 'pekerja pabrik: warna kulit gelap-sedang',
    '🧑🏿‍🏭' => 'pekerja pabrik: warna kulit gelap',
    '👨🏻‍🏭' => 'pekerja pabrik pria: warna kulit cerah',
    '👨🏼‍🏭' => 'pekerja pabrik pria: warna kulit cerah-sedang',
    '👨🏽‍🏭' => 'pekerja pabrik pria: warna kulit sedang',
    '👨🏾‍🏭' => 'pekerja pabrik pria: warna kulit gelap-sedang',
    '👨🏿‍🏭' => 'pekerja pabrik pria: warna kulit gelap',
    '👩🏻‍🏭' => 'pekerja pabrik wanita: warna kulit cerah',
    '👩🏼‍🏭' => 'pekerja pabrik wanita: warna kulit cerah-sedang',
    '👩🏽‍🏭' => 'pekerja pabrik wanita: warna kulit sedang',
    '👩🏾‍🏭' => 'pekerja pabrik wanita: warna kulit gelap-sedang',
    '👩🏿‍🏭' => 'pekerja pabrik wanita: warna kulit gelap',
    '🧑🏻‍💼' => 'pekerja kantoran: warna kulit cerah',
    '🧑🏼‍💼' => 'pekerja kantoran: warna kulit cerah-sedang',
    '🧑🏽‍💼' => 'pekerja kantoran: warna kulit sedang',
    '🧑🏾‍💼' => 'pekerja kantoran: warna kulit gelap-sedang',
    '🧑🏿‍💼' => 'pekerja kantoran: warna kulit gelap',
    '👨🏻‍💼' => 'pekerja kantor pria: warna kulit cerah',
    '👨🏼‍💼' => 'pekerja kantor pria: warna kulit cerah-sedang',
    '👨🏽‍💼' => 'pekerja kantor pria: warna kulit sedang',
    '👨🏾‍💼' => 'pekerja kantor pria: warna kulit gelap-sedang',
    '👨🏿‍💼' => 'pekerja kantor pria: warna kulit gelap',
    '👩🏻‍💼' => 'pekerja kantor wanita: warna kulit cerah',
    '👩🏼‍💼' => 'pekerja kantor wanita: warna kulit cerah-sedang',
    '👩🏽‍💼' => 'pekerja kantor wanita: warna kulit sedang',
    '👩🏾‍💼' => 'pekerja kantor wanita: warna kulit gelap-sedang',
    '👩🏿‍💼' => 'pekerja kantor wanita: warna kulit gelap',
    '🧑🏻‍🔬' => 'ilmuwan: warna kulit cerah',
    '🧑🏼‍🔬' => 'ilmuwan: warna kulit cerah-sedang',
    '🧑🏽‍🔬' => 'ilmuwan: warna kulit sedang',
    '🧑🏾‍🔬' => 'ilmuwan: warna kulit gelap-sedang',
    '🧑🏿‍🔬' => 'ilmuwan: warna kulit gelap',
    '👨🏻‍🔬' => 'ilmuwan pria: warna kulit cerah',
    '👨🏼‍🔬' => 'ilmuwan pria: warna kulit cerah-sedang',
    '👨🏽‍🔬' => 'ilmuwan pria: warna kulit sedang',
    '👨🏾‍🔬' => 'ilmuwan pria: warna kulit gelap-sedang',
    '👨🏿‍🔬' => 'ilmuwan pria: warna kulit gelap',
    '👩🏻‍🔬' => 'ilmuwan wanita: warna kulit cerah',
    '👩🏼‍🔬' => 'ilmuwan wanita: warna kulit cerah-sedang',
    '👩🏽‍🔬' => 'ilmuwan wanita: warna kulit sedang',
    '👩🏾‍🔬' => 'ilmuwan wanita: warna kulit gelap-sedang',
    '👩🏿‍🔬' => 'ilmuwan wanita: warna kulit gelap',
    '🧑🏻‍💻' => 'ahli teknologi: warna kulit cerah',
    '🧑🏼‍💻' => 'ahli teknologi: warna kulit cerah-sedang',
    '🧑🏽‍💻' => 'ahli teknologi: warna kulit sedang',
    '🧑🏾‍💻' => 'ahli teknologi: warna kulit gelap-sedang',
    '🧑🏿‍💻' => 'ahli teknologi: warna kulit gelap',
    '👨🏻‍💻' => 'ahli teknologi pria: warna kulit cerah',
    '👨🏼‍💻' => 'ahli teknologi pria: warna kulit cerah-sedang',
    '👨🏽‍💻' => 'ahli teknologi pria: warna kulit sedang',
    '👨🏾‍💻' => 'ahli teknologi pria: warna kulit gelap-sedang',
    '👨🏿‍💻' => 'ahli teknologi pria: warna kulit gelap',
    '👩🏻‍💻' => 'ahli teknologi wanita: warna kulit cerah',
    '👩🏼‍💻' => 'ahli teknologi wanita: warna kulit cerah-sedang',
    '👩🏽‍💻' => 'ahli teknologi wanita: warna kulit sedang',
    '👩🏾‍💻' => 'ahli teknologi wanita: warna kulit gelap-sedang',
    '👩🏿‍💻' => 'ahli teknologi wanita: warna kulit gelap',
    '🧑🏻‍🎤' => 'penyanyi: warna kulit cerah',
    '🧑🏼‍🎤' => 'penyanyi: warna kulit cerah-sedang',
    '🧑🏽‍🎤' => 'penyanyi: warna kulit sedang',
    '🧑🏾‍🎤' => 'penyanyi: warna kulit gelap-sedang',
    '🧑🏿‍🎤' => 'penyanyi: warna kulit gelap',
    '👨🏻‍🎤' => 'penyanyi pria: warna kulit cerah',
    '👨🏼‍🎤' => 'penyanyi pria: warna kulit cerah-sedang',
    '👨🏽‍🎤' => 'penyanyi pria: warna kulit sedang',
    '👨🏾‍🎤' => 'penyanyi pria: warna kulit gelap-sedang',
    '👨🏿‍🎤' => 'penyanyi pria: warna kulit gelap',
    '👩🏻‍🎤' => 'penyanyi wanita: warna kulit cerah',
    '👩🏼‍🎤' => 'penyanyi wanita: warna kulit cerah-sedang',
    '👩🏽‍🎤' => 'penyanyi wanita: warna kulit sedang',
    '👩🏾‍🎤' => 'penyanyi wanita: warna kulit gelap-sedang',
    '👩🏿‍🎤' => 'penyanyi wanita: warna kulit gelap',
    '🧑🏻‍🎨' => 'seniman: warna kulit cerah',
    '🧑🏼‍🎨' => 'seniman: warna kulit cerah-sedang',
    '🧑🏽‍🎨' => 'seniman: warna kulit sedang',
    '🧑🏾‍🎨' => 'seniman: warna kulit gelap-sedang',
    '🧑🏿‍🎨' => 'seniman: warna kulit gelap',
    '👨🏻‍🎨' => 'seniman pria: warna kulit cerah',
    '👨🏼‍🎨' => 'seniman pria: warna kulit cerah-sedang',
    '👨🏽‍🎨' => 'seniman pria: warna kulit sedang',
    '👨🏾‍🎨' => 'seniman pria: warna kulit gelap-sedang',
    '👨🏿‍🎨' => 'seniman pria: warna kulit gelap',
    '👩🏻‍🎨' => 'seniman wanita: warna kulit cerah',
    '👩🏼‍🎨' => 'seniman wanita: warna kulit cerah-sedang',
    '👩🏽‍🎨' => 'seniman wanita: warna kulit sedang',
    '👩🏾‍🎨' => 'seniman wanita: warna kulit gelap-sedang',
    '👩🏿‍🎨' => 'seniman wanita: warna kulit gelap',
    '🧑🏻‍✈' => 'pilot: warna kulit cerah',
    '🧑🏼‍✈' => 'pilot: warna kulit cerah-sedang',
    '🧑🏽‍✈' => 'pilot: warna kulit sedang',
    '🧑🏾‍✈' => 'pilot: warna kulit gelap-sedang',
    '🧑🏿‍✈' => 'pilot: warna kulit gelap',
    '👨🏻‍✈' => 'pilot pria: warna kulit cerah',
    '👨🏼‍✈' => 'pilot pria: warna kulit cerah-sedang',
    '👨🏽‍✈' => 'pilot pria: warna kulit sedang',
    '👨🏾‍✈' => 'pilot pria: warna kulit gelap-sedang',
    '👨🏿‍✈' => 'pilot pria: warna kulit gelap',
    '👩🏻‍✈' => 'pilot wanita: warna kulit cerah',
    '👩🏼‍✈' => 'pilot wanita: warna kulit cerah-sedang',
    '👩🏽‍✈' => 'pilot wanita: warna kulit sedang',
    '👩🏾‍✈' => 'pilot wanita: warna kulit gelap-sedang',
    '👩🏿‍✈' => 'pilot wanita: warna kulit gelap',
    '🧑🏻‍🚀' => 'astronaut: warna kulit cerah',
    '🧑🏼‍🚀' => 'astronaut: warna kulit cerah-sedang',
    '🧑🏽‍🚀' => 'astronaut: warna kulit sedang',
    '🧑🏾‍🚀' => 'astronaut: warna kulit gelap-sedang',
    '🧑🏿‍🚀' => 'astronaut: warna kulit gelap',
    '👨🏻‍🚀' => 'antariksawan pria: warna kulit cerah',
    '👨🏼‍🚀' => 'antariksawan pria: warna kulit cerah-sedang',
    '👨🏽‍🚀' => 'antariksawan pria: warna kulit sedang',
    '👨🏾‍🚀' => 'antariksawan pria: warna kulit gelap-sedang',
    '👨🏿‍🚀' => 'antariksawan pria: warna kulit gelap',
    '👩🏻‍🚀' => 'antariksawan wanita: warna kulit cerah',
    '👩🏼‍🚀' => 'antariksawan wanita: warna kulit cerah-sedang',
    '👩🏽‍🚀' => 'antariksawan wanita: warna kulit sedang',
    '👩🏾‍🚀' => 'antariksawan wanita: warna kulit gelap-sedang',
    '👩🏿‍🚀' => 'antariksawan wanita: warna kulit gelap',
    '🧑🏻‍🚒' => 'pemadam kebakaran: warna kulit cerah',
    '🧑🏼‍🚒' => 'pemadam kebakaran: warna kulit cerah-sedang',
    '🧑🏽‍🚒' => 'pemadam kebakaran: warna kulit sedang',
    '🧑🏾‍🚒' => 'pemadam kebakaran: warna kulit gelap-sedang',
    '🧑🏿‍🚒' => 'pemadam kebakaran: warna kulit gelap',
    '👨🏻‍🚒' => 'pemadam kebakaran pria: warna kulit cerah',
    '👨🏼‍🚒' => 'pemadam kebakaran pria: warna kulit cerah-sedang',
    '👨🏽‍🚒' => 'pemadam kebakaran pria: warna kulit sedang',
    '👨🏾‍🚒' => 'pemadam kebakaran pria: warna kulit gelap-sedang',
    '👨🏿‍🚒' => 'pemadam kebakaran pria: warna kulit gelap',
    '👩🏻‍🚒' => 'pemadam kebakaran wanita: warna kulit cerah',
    '👩🏼‍🚒' => 'pemadam kebakaran wanita: warna kulit cerah-sedang',
    '👩🏽‍🚒' => 'pemadam kebakaran wanita: warna kulit sedang',
    '👩🏾‍🚒' => 'pemadam kebakaran wanita: warna kulit gelap-sedang',
    '👩🏿‍🚒' => 'pemadam kebakaran wanita: warna kulit gelap',
    '👮🏻‍♂' => 'polisi pria: warna kulit cerah',
    '👮🏼‍♂' => 'polisi pria: warna kulit cerah-sedang',
    '👮🏽‍♂' => 'polisi pria: warna kulit sedang',
    '👮🏾‍♂' => 'polisi pria: warna kulit gelap-sedang',
    '👮🏿‍♂' => 'polisi pria: warna kulit gelap',
    '👮🏻‍♀' => 'polisi wanita: warna kulit cerah',
    '👮🏼‍♀' => 'polisi wanita: warna kulit cerah-sedang',
    '👮🏽‍♀' => 'polisi wanita: warna kulit sedang',
    '👮🏾‍♀' => 'polisi wanita: warna kulit gelap-sedang',
    '👮🏿‍♀' => 'polisi wanita: warna kulit gelap',
    '🕵🏻‍♂' => 'detektif pria: warna kulit cerah',
    '🕵🏼‍♂' => 'detektif pria: warna kulit cerah-sedang',
    '🕵🏽‍♂' => 'detektif pria: warna kulit sedang',
    '🕵🏾‍♂' => 'detektif pria: warna kulit gelap-sedang',
    '🕵🏿‍♂' => 'detektif pria: warna kulit gelap',
    '🕵🏻‍♀' => 'detektif wanita: warna kulit cerah',
    '🕵🏼‍♀' => 'detektif wanita: warna kulit cerah-sedang',
    '🕵🏽‍♀' => 'detektif wanita: warna kulit sedang',
    '🕵🏾‍♀' => 'detektif wanita: warna kulit gelap-sedang',
    '🕵🏿‍♀' => 'detektif wanita: warna kulit gelap',
    '💂🏻‍♂' => 'penjaga pria: warna kulit cerah',
    '💂🏼‍♂' => 'penjaga pria: warna kulit cerah-sedang',
    '💂🏽‍♂' => 'penjaga pria: warna kulit sedang',
    '💂🏾‍♂' => 'penjaga pria: warna kulit gelap-sedang',
    '💂🏿‍♂' => 'penjaga pria: warna kulit gelap',
    '💂🏻‍♀' => 'penjaga wanita: warna kulit cerah',
    '💂🏼‍♀' => 'penjaga wanita: warna kulit cerah-sedang',
    '💂🏽‍♀' => 'penjaga wanita: warna kulit sedang',
    '💂🏾‍♀' => 'penjaga wanita: warna kulit gelap-sedang',
    '💂🏿‍♀' => 'penjaga wanita: warna kulit gelap',
    '👷🏻‍♂' => 'pekerja konstruksi pria: warna kulit cerah',
    '👷🏼‍♂' => 'pekerja konstruksi pria: warna kulit cerah-sedang',
    '👷🏽‍♂' => 'pekerja konstruksi pria: warna kulit sedang',
    '👷🏾‍♂' => 'pekerja konstruksi pria: warna kulit gelap-sedang',
    '👷🏿‍♂' => 'pekerja konstruksi pria: warna kulit gelap',
    '👷🏻‍♀' => 'pekerja konstruksi wanita: warna kulit cerah',
    '👷🏼‍♀' => 'pekerja konstruksi wanita: warna kulit cerah-sedang',
    '👷🏽‍♀' => 'pekerja konstruksi wanita: warna kulit sedang',
    '👷🏾‍♀' => 'pekerja konstruksi wanita: warna kulit gelap-sedang',
    '👷🏿‍♀' => 'pekerja konstruksi wanita: warna kulit gelap',
    '👳🏻‍♂' => 'pria mengenakan serban: warna kulit cerah',
    '👳🏼‍♂' => 'pria mengenakan serban: warna kulit cerah-sedang',
    '👳🏽‍♂' => 'pria mengenakan serban: warna kulit sedang',
    '👳🏾‍♂' => 'pria mengenakan serban: warna kulit gelap-sedang',
    '👳🏿‍♂' => 'pria mengenakan serban: warna kulit gelap',
    '👳🏻‍♀' => 'wanita mengenakan serban: warna kulit cerah',
    '👳🏼‍♀' => 'wanita mengenakan serban: warna kulit cerah-sedang',
    '👳🏽‍♀' => 'wanita mengenakan serban: warna kulit sedang',
    '👳🏾‍♀' => 'wanita mengenakan serban: warna kulit gelap-sedang',
    '👳🏿‍♀' => 'wanita mengenakan serban: warna kulit gelap',
    '🤵🏻‍♂' => 'pria dewasa bertuksedo: warna kulit cerah',
    '🤵🏼‍♂' => 'pria dewasa bertuksedo: warna kulit cerah-sedang',
    '🤵🏽‍♂' => 'pria dewasa bertuksedo: warna kulit sedang',
    '🤵🏾‍♂' => 'pria dewasa bertuksedo: warna kulit gelap-sedang',
    '🤵🏿‍♂' => 'pria dewasa bertuksedo: warna kulit gelap',
    '🤵🏻‍♀' => 'wanita bertuksedo: warna kulit cerah',
    '🤵🏼‍♀' => 'wanita bertuksedo: warna kulit cerah-sedang',
    '🤵🏽‍♀' => 'wanita bertuksedo: warna kulit sedang',
    '🤵🏾‍♀' => 'wanita bertuksedo: warna kulit gelap-sedang',
    '🤵🏿‍♀' => 'wanita bertuksedo: warna kulit gelap',
    '👰🏻‍♂' => 'pria berpenutup kepala: warna kulit cerah',
    '👰🏼‍♂' => 'pria berpenutup kepala: warna kulit cerah-sedang',
    '👰🏽‍♂' => 'pria berpenutup kepala: warna kulit sedang',
    '👰🏾‍♂' => 'pria berpenutup kepala: warna kulit gelap-sedang',
    '👰🏿‍♂' => 'pria berpenutup kepala: warna kulit gelap',
    '👰🏻‍♀' => 'wanita berpenutup kepala: warna kulit cerah',
    '👰🏼‍♀' => 'wanita berpenutup kepala: warna kulit cerah-sedang',
    '👰🏽‍♀' => 'wanita berpenutup kepala: warna kulit sedang',
    '👰🏾‍♀' => 'wanita berpenutup kepala: warna kulit gelap-sedang',
    '👰🏿‍♀' => 'wanita berpenutup kepala: warna kulit gelap',
    '👩🏻‍🍼' => 'wanita menyusui bayi: warna kulit cerah',
    '👩🏼‍🍼' => 'wanita menyusui bayi: warna kulit cerah-sedang',
    '👩🏽‍🍼' => 'wanita menyusui bayi: warna kulit sedang',
    '👩🏾‍🍼' => 'wanita menyusui bayi: warna kulit gelap-sedang',
    '👩🏿‍🍼' => 'wanita menyusui bayi: warna kulit gelap',
    '👨🏻‍🍼' => 'pria menyusui bayi: warna kulit cerah',
    '👨🏼‍🍼' => 'pria menyusui bayi: warna kulit cerah-sedang',
    '👨🏽‍🍼' => 'pria menyusui bayi: warna kulit sedang',
    '👨🏾‍🍼' => 'pria menyusui bayi: warna kulit gelap-sedang',
    '👨🏿‍🍼' => 'pria menyusui bayi: warna kulit gelap',
    '🧑🏻‍🍼' => 'orang menyusui bayi: warna kulit cerah',
    '🧑🏼‍🍼' => 'orang menyusui bayi: warna kulit cerah-sedang',
    '🧑🏽‍🍼' => 'orang menyusui bayi: warna kulit sedang',
    '🧑🏾‍🍼' => 'orang menyusui bayi: warna kulit gelap-sedang',
    '🧑🏿‍🍼' => 'orang menyusui bayi: warna kulit gelap',
    '🧑🏻‍🎄' => 'manusia sinterklas: warna kulit cerah',
    '🧑🏼‍🎄' => 'manusia sinterklas: warna kulit cerah-sedang',
    '🧑🏽‍🎄' => 'manusia sinterklas: warna kulit sedang',
    '🧑🏾‍🎄' => 'manusia sinterklas: warna kulit gelap-sedang',
    '🧑🏿‍🎄' => 'manusia sinterklas: warna kulit gelap',
    '🦸🏻‍♂' => 'pahlawan super pria: warna kulit cerah',
    '🦸🏼‍♂' => 'pahlawan super pria: warna kulit cerah-sedang',
    '🦸🏽‍♂' => 'pahlawan super pria: warna kulit sedang',
    '🦸🏾‍♂' => 'pahlawan super pria: warna kulit gelap-sedang',
    '🦸🏿‍♂' => 'pahlawan super pria: warna kulit gelap',
    '🦸🏻‍♀' => 'pahlawan super wanita: warna kulit cerah',
    '🦸🏼‍♀' => 'pahlawan super wanita: warna kulit cerah-sedang',
    '🦸🏽‍♀' => 'pahlawan super wanita: warna kulit sedang',
    '🦸🏾‍♀' => 'pahlawan super wanita: warna kulit gelap-sedang',
    '🦸🏿‍♀' => 'pahlawan super wanita: warna kulit gelap',
    '🦹🏻‍♂' => 'penjahat super pria: warna kulit cerah',
    '🦹🏼‍♂' => 'penjahat super pria: warna kulit cerah-sedang',
    '🦹🏽‍♂' => 'penjahat super pria: warna kulit sedang',
    '🦹🏾‍♂' => 'penjahat super pria: warna kulit gelap-sedang',
    '🦹🏿‍♂' => 'penjahat super pria: warna kulit gelap',
    '🦹🏻‍♀' => 'penjahat super wanita: warna kulit cerah',
    '🦹🏼‍♀' => 'penjahat super wanita: warna kulit cerah-sedang',
    '🦹🏽‍♀' => 'penjahat super wanita: warna kulit sedang',
    '🦹🏾‍♀' => 'penjahat super wanita: warna kulit gelap-sedang',
    '🦹🏿‍♀' => 'penjahat super wanita: warna kulit gelap',
    '🧙🏻‍♂' => 'magi pria: warna kulit cerah',
    '🧙🏼‍♂' => 'magi pria: warna kulit cerah-sedang',
    '🧙🏽‍♂' => 'magi pria: warna kulit sedang',
    '🧙🏾‍♂' => 'magi pria: warna kulit gelap-sedang',
    '🧙🏿‍♂' => 'magi pria: warna kulit gelap',
    '🧙🏻‍♀' => 'magi wanita: warna kulit cerah',
    '🧙🏼‍♀' => 'magi wanita: warna kulit cerah-sedang',
    '🧙🏽‍♀' => 'magi wanita: warna kulit sedang',
    '🧙🏾‍♀' => 'magi wanita: warna kulit gelap-sedang',
    '🧙🏿‍♀' => 'magi wanita: warna kulit gelap',
    '🧚🏻‍♂' => 'peri pria: warna kulit cerah',
    '🧚🏼‍♂' => 'peri pria: warna kulit cerah-sedang',
    '🧚🏽‍♂' => 'peri pria: warna kulit sedang',
    '🧚🏾‍♂' => 'peri pria: warna kulit gelap-sedang',
    '🧚🏿‍♂' => 'peri pria: warna kulit gelap',
    '🧚🏻‍♀' => 'peri wanita: warna kulit cerah',
    '🧚🏼‍♀' => 'peri wanita: warna kulit cerah-sedang',
    '🧚🏽‍♀' => 'peri wanita: warna kulit sedang',
    '🧚🏾‍♀' => 'peri wanita: warna kulit gelap-sedang',
    '🧚🏿‍♀' => 'peri wanita: warna kulit gelap',
    '🧛🏻‍♂' => 'vampir pria: warna kulit cerah',
    '🧛🏼‍♂' => 'vampir pria: warna kulit cerah-sedang',
    '🧛🏽‍♂' => 'vampir pria: warna kulit sedang',
    '🧛🏾‍♂' => 'vampir pria: warna kulit gelap-sedang',
    '🧛🏿‍♂' => 'vampir pria: warna kulit gelap',
    '🧛🏻‍♀' => 'vampir wanita: warna kulit cerah',
    '🧛🏼‍♀' => 'vampir wanita: warna kulit cerah-sedang',
    '🧛🏽‍♀' => 'vampir wanita: warna kulit sedang',
    '🧛🏾‍♀' => 'vampir wanita: warna kulit gelap-sedang',
    '🧛🏿‍♀' => 'vampir wanita: warna kulit gelap',
    '🧜🏻‍♂' => 'pria duyung: warna kulit cerah',
    '🧜🏼‍♂' => 'pria duyung: warna kulit cerah-sedang',
    '🧜🏽‍♂' => 'pria duyung: warna kulit sedang',
    '🧜🏾‍♂' => 'pria duyung: warna kulit gelap-sedang',
    '🧜🏿‍♂' => 'pria duyung: warna kulit gelap',
    '🧜🏻‍♀' => 'putri duyung: warna kulit cerah',
    '🧜🏼‍♀' => 'putri duyung: warna kulit cerah-sedang',
    '🧜🏽‍♀' => 'putri duyung: warna kulit sedang',
    '🧜🏾‍♀' => 'putri duyung: warna kulit gelap-sedang',
    '🧜🏿‍♀' => 'putri duyung: warna kulit gelap',
    '🧝🏻‍♂' => 'kurcaci pria: warna kulit cerah',
    '🧝🏼‍♂' => 'kurcaci pria: warna kulit cerah-sedang',
    '🧝🏽‍♂' => 'kurcaci pria: warna kulit sedang',
    '🧝🏾‍♂' => 'kurcaci pria: warna kulit gelap-sedang',
    '🧝🏿‍♂' => 'kurcaci pria: warna kulit gelap',
    '🧝🏻‍♀' => 'kurcaci wanita: warna kulit cerah',
    '🧝🏼‍♀' => 'kurcaci wanita: warna kulit cerah-sedang',
    '🧝🏽‍♀' => 'kurcaci wanita: warna kulit sedang',
    '🧝🏾‍♀' => 'kurcaci wanita: warna kulit gelap-sedang',
    '🧝🏿‍♀' => 'kurcaci wanita: warna kulit gelap',
    '💆🏻‍♂' => 'pria dipijat kepalanya: warna kulit cerah',
    '💆🏼‍♂' => 'pria dipijat kepalanya: warna kulit cerah-sedang',
    '💆🏽‍♂' => 'pria dipijat kepalanya: warna kulit sedang',
    '💆🏾‍♂' => 'pria dipijat kepalanya: warna kulit gelap-sedang',
    '💆🏿‍♂' => 'pria dipijat kepalanya: warna kulit gelap',
    '💆🏻‍♀' => 'wanita dipijat: warna kulit cerah',
    '💆🏼‍♀' => 'wanita dipijat: warna kulit cerah-sedang',
    '💆🏽‍♀' => 'wanita dipijat: warna kulit sedang',
    '💆🏾‍♀' => 'wanita dipijat: warna kulit gelap-sedang',
    '💆🏿‍♀' => 'wanita dipijat: warna kulit gelap',
    '💇🏻‍♂' => 'pria potong rambut: warna kulit cerah',
    '💇🏼‍♂' => 'pria potong rambut: warna kulit cerah-sedang',
    '💇🏽‍♂' => 'pria potong rambut: warna kulit sedang',
    '💇🏾‍♂' => 'pria potong rambut: warna kulit gelap-sedang',
    '💇🏿‍♂' => 'pria potong rambut: warna kulit gelap',
    '💇🏻‍♀' => 'wanita potong rambut: warna kulit cerah',
    '💇🏼‍♀' => 'wanita potong rambut: warna kulit cerah-sedang',
    '💇🏽‍♀' => 'wanita potong rambut: warna kulit sedang',
    '💇🏾‍♀' => 'wanita potong rambut: warna kulit gelap-sedang',
    '💇🏿‍♀' => 'wanita potong rambut: warna kulit gelap',
    '🚶🏻‍♂' => 'pria berjalan kaki: warna kulit cerah',
    '🚶🏼‍♂' => 'pria berjalan kaki: warna kulit cerah-sedang',
    '🚶🏽‍♂' => 'pria berjalan kaki: warna kulit sedang',
    '🚶🏾‍♂' => 'pria berjalan kaki: warna kulit gelap-sedang',
    '🚶🏿‍♂' => 'pria berjalan kaki: warna kulit gelap',
    '🚶🏻‍♀' => 'wanita berjalan kaki: warna kulit cerah',
    '🚶🏼‍♀' => 'wanita berjalan kaki: warna kulit cerah-sedang',
    '🚶🏽‍♀' => 'wanita berjalan kaki: warna kulit sedang',
    '🚶🏾‍♀' => 'wanita berjalan kaki: warna kulit gelap-sedang',
    '🚶🏿‍♀' => 'wanita berjalan kaki: warna kulit gelap',
    '🚶🏻‍➡' => 'orang berjalan kaki: warna kulit cerah, menghadap kanan',
    '🚶🏼‍➡' => 'orang berjalan kaki: warna kulit cerah-sedang, menghadap kanan',
    '🚶🏽‍➡' => 'orang berjalan kaki: warna kulit sedang, menghadap kanan',
    '🚶🏾‍➡' => 'orang berjalan kaki: warna kulit gelap-sedang, menghadap kanan',
    '🚶🏿‍➡' => 'orang berjalan kaki: warna kulit gelap, menghadap kanan',
    '🧍🏻‍♂' => 'pria berdiri: warna kulit cerah',
    '🧍🏼‍♂' => 'pria berdiri: warna kulit cerah-sedang',
    '🧍🏽‍♂' => 'pria berdiri: warna kulit sedang',
    '🧍🏾‍♂' => 'pria berdiri: warna kulit gelap-sedang',
    '🧍🏿‍♂' => 'pria berdiri: warna kulit gelap',
    '🧍🏻‍♀' => 'wanita berdiri: warna kulit cerah',
    '🧍🏼‍♀' => 'wanita berdiri: warna kulit cerah-sedang',
    '🧍🏽‍♀' => 'wanita berdiri: warna kulit sedang',
    '🧍🏾‍♀' => 'wanita berdiri: warna kulit gelap-sedang',
    '🧍🏿‍♀' => 'wanita berdiri: warna kulit gelap',
    '🧎🏻‍♂' => 'pria berlutut: warna kulit cerah',
    '🧎🏼‍♂' => 'pria berlutut: warna kulit cerah-sedang',
    '🧎🏽‍♂' => 'pria berlutut: warna kulit sedang',
    '🧎🏾‍♂' => 'pria berlutut: warna kulit gelap-sedang',
    '🧎🏿‍♂' => 'pria berlutut: warna kulit gelap',
    '🧎🏻‍♀' => 'wanita berlutut: warna kulit cerah',
    '🧎🏼‍♀' => 'wanita berlutut: warna kulit cerah-sedang',
    '🧎🏽‍♀' => 'wanita berlutut: warna kulit sedang',
    '🧎🏾‍♀' => 'wanita berlutut: warna kulit gelap-sedang',
    '🧎🏿‍♀' => 'wanita berlutut: warna kulit gelap',
    '🧎🏻‍➡' => 'orang berlutut: warna kulit cerah, menghadap kanan',
    '🧎🏼‍➡' => 'orang berlutut: warna kulit cerah-sedang, menghadap kanan',
    '🧎🏽‍➡' => 'orang berlutut: warna kulit sedang, menghadap kanan',
    '🧎🏾‍➡' => 'orang berlutut: warna kulit gelap-sedang, menghadap kanan',
    '🧎🏿‍➡' => 'orang berlutut: warna kulit gelap, menghadap kanan',
    '🧑🏻‍🦯' => 'orang dengan tongkat: warna kulit cerah',
    '🧑🏼‍🦯' => 'orang dengan tongkat: warna kulit cerah-sedang',
    '🧑🏽‍🦯' => 'orang dengan tongkat: warna kulit sedang',
    '🧑🏾‍🦯' => 'orang dengan tongkat: warna kulit gelap-sedang',
    '🧑🏿‍🦯' => 'orang dengan tongkat: warna kulit gelap',
    '👨🏻‍🦯' => 'pria dengan tongkat: warna kulit cerah',
    '👨🏼‍🦯' => 'pria dengan tongkat: warna kulit cerah-sedang',
    '👨🏽‍🦯' => 'pria dengan tongkat: warna kulit sedang',
    '👨🏾‍🦯' => 'pria dengan tongkat: warna kulit gelap-sedang',
    '👨🏿‍🦯' => 'pria dengan tongkat: warna kulit gelap',
    '👩🏻‍🦯' => 'wanita dengan tongkat: warna kulit cerah',
    '👩🏼‍🦯' => 'wanita dengan tongkat: warna kulit cerah-sedang',
    '👩🏽‍🦯' => 'wanita dengan tongkat: warna kulit sedang',
    '👩🏾‍🦯' => 'wanita dengan tongkat: warna kulit gelap-sedang',
    '👩🏿‍🦯' => 'wanita dengan tongkat: warna kulit gelap',
    '🧑🏻‍🦼' => 'orang dengan kursi roda bermotor: warna kulit cerah',
    '🧑🏼‍🦼' => 'orang dengan kursi roda bermotor: warna kulit cerah-sedang',
    '🧑🏽‍🦼' => 'orang dengan kursi roda bermotor: warna kulit sedang',
    '🧑🏾‍🦼' => 'orang dengan kursi roda bermotor: warna kulit gelap-sedang',
    '🧑🏿‍🦼' => 'orang dengan kursi roda bermotor: warna kulit gelap',
    '👨🏻‍🦼' => 'pria dengan kursi roda bermotor: warna kulit cerah',
    '👨🏼‍🦼' => 'pria dengan kursi roda bermotor: warna kulit cerah-sedang',
    '👨🏽‍🦼' => 'pria dengan kursi roda bermotor: warna kulit sedang',
    '👨🏾‍🦼' => 'pria dengan kursi roda bermotor: warna kulit gelap-sedang',
    '👨🏿‍🦼' => 'pria dengan kursi roda bermotor: warna kulit gelap',
    '👩🏻‍🦼' => 'wanita dengan kursi roda bermotor: warna kulit cerah',
    '👩🏼‍🦼' => 'wanita dengan kursi roda bermotor: warna kulit cerah-sedang',
    '👩🏽‍🦼' => 'wanita dengan kursi roda bermotor: warna kulit sedang',
    '👩🏾‍🦼' => 'wanita dengan kursi roda bermotor: warna kulit gelap-sedang',
    '👩🏿‍🦼' => 'wanita dengan kursi roda bermotor: warna kulit gelap',
    '🧑🏻‍🦽' => 'orang dengan kursi roda manual: warna kulit cerah',
    '🧑🏼‍🦽' => 'orang dengan kursi roda manual: warna kulit cerah-sedang',
    '🧑🏽‍🦽' => 'orang dengan kursi roda manual: warna kulit sedang',
    '🧑🏾‍🦽' => 'orang dengan kursi roda manual: warna kulit gelap-sedang',
    '🧑🏿‍🦽' => 'orang dengan kursi roda manual: warna kulit gelap',
    '👨🏻‍🦽' => 'pria dengan kursi roda manual: warna kulit cerah',
    '👨🏼‍🦽' => 'pria dengan kursi roda manual: warna kulit cerah-sedang',
    '👨🏽‍🦽' => 'pria dengan kursi roda manual: warna kulit sedang',
    '👨🏾‍🦽' => 'pria dengan kursi roda manual: warna kulit gelap-sedang',
    '👨🏿‍🦽' => 'pria dengan kursi roda manual: warna kulit gelap',
    '👩🏻‍🦽' => 'wanita dengan kursi roda manual: warna kulit cerah',
    '👩🏼‍🦽' => 'wanita dengan kursi roda manual: warna kulit cerah-sedang',
    '👩🏽‍🦽' => 'wanita dengan kursi roda manual: warna kulit sedang',
    '👩🏾‍🦽' => 'wanita dengan kursi roda manual: warna kulit gelap-sedang',
    '👩🏿‍🦽' => 'wanita dengan kursi roda manual: warna kulit gelap',
    '🏃🏻‍♂' => 'pria berlari: warna kulit cerah',
    '🏃🏼‍♂' => 'pria berlari: warna kulit cerah-sedang',
    '🏃🏽‍♂' => 'pria berlari: warna kulit sedang',
    '🏃🏾‍♂' => 'pria berlari: warna kulit gelap-sedang',
    '🏃🏿‍♂' => 'pria berlari: warna kulit gelap',
    '🏃🏻‍♀' => 'wanita berlari: warna kulit cerah',
    '🏃🏼‍♀' => 'wanita berlari: warna kulit cerah-sedang',
    '🏃🏽‍♀' => 'wanita berlari: warna kulit sedang',
    '🏃🏾‍♀' => 'wanita berlari: warna kulit gelap-sedang',
    '🏃🏿‍♀' => 'wanita berlari: warna kulit gelap',
    '🏃🏻‍➡' => 'orang berlari: warna kulit cerah, menghadap kanan',
    '🏃🏼‍➡' => 'orang berlari: warna kulit cerah-sedang, menghadap kanan',
    '🏃🏽‍➡' => 'orang berlari: warna kulit sedang, menghadap kanan',
    '🏃🏾‍➡' => 'orang berlari: warna kulit gelap-sedang, menghadap kanan',
    '🏃🏿‍➡' => 'orang berlari: warna kulit gelap, menghadap kanan',
    '👯🏻‍♀' => 'wanita dengan telinga kelinci: warna kulit cerah',
    '👯🏼‍♀' => 'wanita dengan telinga kelinci: warna kulit cerah-sedang',
    '👯🏽‍♀' => 'wanita dengan telinga kelinci: warna kulit sedang',
    '👯🏾‍♀' => 'wanita dengan telinga kelinci: warna kulit gelap-sedang',
    '👯🏿‍♀' => 'wanita dengan telinga kelinci: warna kulit gelap',
    '👯🏻‍♂' => 'pria dengan telinga kelinci: warna kulit cerah',
    '👯🏼‍♂' => 'pria dengan telinga kelinci: warna kulit cerah-sedang',
    '👯🏽‍♂' => 'pria dengan telinga kelinci: warna kulit sedang',
    '👯🏾‍♂' => 'pria dengan telinga kelinci: warna kulit gelap-sedang',
    '👯🏿‍♂' => 'pria dengan telinga kelinci: warna kulit gelap',
    '🧖🏻‍♂' => 'pria di ruangan beruap: warna kulit cerah',
    '🧖🏼‍♂' => 'pria di ruangan beruap: warna kulit cerah-sedang',
    '🧖🏽‍♂' => 'pria di ruangan beruap: warna kulit sedang',
    '🧖🏾‍♂' => 'pria di ruangan beruap: warna kulit gelap-sedang',
    '🧖🏿‍♂' => 'pria di ruangan beruap: warna kulit gelap',
    '🧖🏻‍♀' => 'wanita di ruangan beruap: warna kulit cerah',
    '🧖🏼‍♀' => 'wanita di ruangan beruap: warna kulit cerah-sedang',
    '🧖🏽‍♀' => 'wanita di ruangan beruap: warna kulit sedang',
    '🧖🏾‍♀' => 'wanita di ruangan beruap: warna kulit gelap-sedang',
    '🧖🏿‍♀' => 'wanita di ruangan beruap: warna kulit gelap',
    '🧗🏻‍♂' => 'pria memanjat: warna kulit cerah',
    '🧗🏼‍♂' => 'pria memanjat: warna kulit cerah-sedang',
    '🧗🏽‍♂' => 'pria memanjat: warna kulit sedang',
    '🧗🏾‍♂' => 'pria memanjat: warna kulit gelap-sedang',
    '🧗🏿‍♂' => 'pria memanjat: warna kulit gelap',
    '🧗🏻‍♀' => 'wanita memanjat: warna kulit cerah',
    '🧗🏼‍♀' => 'wanita memanjat: warna kulit cerah-sedang',
    '🧗🏽‍♀' => 'wanita memanjat: warna kulit sedang',
    '🧗🏾‍♀' => 'wanita memanjat: warna kulit gelap-sedang',
    '🧗🏿‍♀' => 'wanita memanjat: warna kulit gelap',
    '🏌🏻‍♂' => 'pria bermain golf: warna kulit cerah',
    '🏌🏼‍♂' => 'pria bermain golf: warna kulit cerah-sedang',
    '🏌🏽‍♂' => 'pria bermain golf: warna kulit sedang',
    '🏌🏾‍♂' => 'pria bermain golf: warna kulit gelap-sedang',
    '🏌🏿‍♂' => 'pria bermain golf: warna kulit gelap',
    '🏌🏻‍♀' => 'wanita bermain golf: warna kulit cerah',
    '🏌🏼‍♀' => 'wanita bermain golf: warna kulit cerah-sedang',
    '🏌🏽‍♀' => 'wanita bermain golf: warna kulit sedang',
    '🏌🏾‍♀' => 'wanita bermain golf: warna kulit gelap-sedang',
    '🏌🏿‍♀' => 'wanita bermain golf: warna kulit gelap',
    '🏄🏻‍♂' => 'pria berselancar: warna kulit cerah',
    '🏄🏼‍♂' => 'pria berselancar: warna kulit cerah-sedang',
    '🏄🏽‍♂' => 'pria berselancar: warna kulit sedang',
    '🏄🏾‍♂' => 'pria berselancar: warna kulit gelap-sedang',
    '🏄🏿‍♂' => 'pria berselancar: warna kulit gelap',
    '🏄🏻‍♀' => 'wanita berselancar: warna kulit cerah',
    '🏄🏼‍♀' => 'wanita berselancar: warna kulit cerah-sedang',
    '🏄🏽‍♀' => 'wanita berselancar: warna kulit sedang',
    '🏄🏾‍♀' => 'wanita berselancar: warna kulit gelap-sedang',
    '🏄🏿‍♀' => 'wanita berselancar: warna kulit gelap',
    '🚣🏻‍♂' => 'pria mendayung perahu: warna kulit cerah',
    '🚣🏼‍♂' => 'pria mendayung perahu: warna kulit cerah-sedang',
    '🚣🏽‍♂' => 'pria mendayung perahu: warna kulit sedang',
    '🚣🏾‍♂' => 'pria mendayung perahu: warna kulit gelap-sedang',
    '🚣🏿‍♂' => 'pria mendayung perahu: warna kulit gelap',
    '🚣🏻‍♀' => 'wanita mendayung perahu: warna kulit cerah',
    '🚣🏼‍♀' => 'wanita mendayung perahu: warna kulit cerah-sedang',
    '🚣🏽‍♀' => 'wanita mendayung perahu: warna kulit sedang',
    '🚣🏾‍♀' => 'wanita mendayung perahu: warna kulit gelap-sedang',
    '🚣🏿‍♀' => 'wanita mendayung perahu: warna kulit gelap',
    '🏊🏻‍♂' => 'pria berenang: warna kulit cerah',
    '🏊🏼‍♂' => 'pria berenang: warna kulit cerah-sedang',
    '🏊🏽‍♂' => 'pria berenang: warna kulit sedang',
    '🏊🏾‍♂' => 'pria berenang: warna kulit gelap-sedang',
    '🏊🏿‍♂' => 'pria berenang: warna kulit gelap',
    '🏊🏻‍♀' => 'wanita berenang: warna kulit cerah',
    '🏊🏼‍♀' => 'wanita berenang: warna kulit cerah-sedang',
    '🏊🏽‍♀' => 'wanita berenang: warna kulit sedang',
    '🏊🏾‍♀' => 'wanita berenang: warna kulit gelap-sedang',
    '🏊🏿‍♀' => 'wanita berenang: warna kulit gelap',
    '⛹🏻‍♂' => 'pria memantulkan bola: warna kulit cerah',
    '⛹🏼‍♂' => 'pria memantulkan bola: warna kulit cerah-sedang',
    '⛹🏽‍♂' => 'pria memantulkan bola: warna kulit sedang',
    '⛹🏾‍♂' => 'pria memantulkan bola: warna kulit gelap-sedang',
    '⛹🏿‍♂' => 'pria memantulkan bola: warna kulit gelap',
    '⛹🏻‍♀' => 'wanita memantulkan bola: warna kulit cerah',
    '⛹🏼‍♀' => 'wanita memantulkan bola: warna kulit cerah-sedang',
    '⛹🏽‍♀' => 'wanita memantulkan bola: warna kulit sedang',
    '⛹🏾‍♀' => 'wanita memantulkan bola: warna kulit gelap-sedang',
    '⛹🏿‍♀' => 'wanita memantulkan bola: warna kulit gelap',
    '🏋🏻‍♂' => 'pria mengangkat beban: warna kulit cerah',
    '🏋🏼‍♂' => 'pria mengangkat beban: warna kulit cerah-sedang',
    '🏋🏽‍♂' => 'pria mengangkat beban: warna kulit sedang',
    '🏋🏾‍♂' => 'pria mengangkat beban: warna kulit gelap-sedang',
    '🏋🏿‍♂' => 'pria mengangkat beban: warna kulit gelap',
    '🏋🏻‍♀' => 'wanita mengangkat beban: warna kulit cerah',
    '🏋🏼‍♀' => 'wanita mengangkat beban: warna kulit cerah-sedang',
    '🏋🏽‍♀' => 'wanita mengangkat beban: warna kulit sedang',
    '🏋🏾‍♀' => 'wanita mengangkat beban: warna kulit gelap-sedang',
    '🏋🏿‍♀' => 'wanita mengangkat beban: warna kulit gelap',
    '🚴🏻‍♂' => 'pria bersepeda: warna kulit cerah',
    '🚴🏼‍♂' => 'pria bersepeda: warna kulit cerah-sedang',
    '🚴🏽‍♂' => 'pria bersepeda: warna kulit sedang',
    '🚴🏾‍♂' => 'pria bersepeda: warna kulit gelap-sedang',
    '🚴🏿‍♂' => 'pria bersepeda: warna kulit gelap',
    '🚴🏻‍♀' => 'wanita bersepeda: warna kulit cerah',
    '🚴🏼‍♀' => 'wanita bersepeda: warna kulit cerah-sedang',
    '🚴🏽‍♀' => 'wanita bersepeda: warna kulit sedang',
    '🚴🏾‍♀' => 'wanita bersepeda: warna kulit gelap-sedang',
    '🚴🏿‍♀' => 'wanita bersepeda: warna kulit gelap',
    '🚵🏻‍♂' => 'pria bersepeda gunung: warna kulit cerah',
    '🚵🏼‍♂' => 'pria bersepeda gunung: warna kulit cerah-sedang',
    '🚵🏽‍♂' => 'pria bersepeda gunung: warna kulit sedang',
    '🚵🏾‍♂' => 'pria bersepeda gunung: warna kulit gelap-sedang',
    '🚵🏿‍♂' => 'pria bersepeda gunung: warna kulit gelap',
    '🚵🏻‍♀' => 'wanita bersepeda gunung: warna kulit cerah',
    '🚵🏼‍♀' => 'wanita bersepeda gunung: warna kulit cerah-sedang',
    '🚵🏽‍♀' => 'wanita bersepeda gunung: warna kulit sedang',
    '🚵🏾‍♀' => 'wanita bersepeda gunung: warna kulit gelap-sedang',
    '🚵🏿‍♀' => 'wanita bersepeda gunung: warna kulit gelap',
    '🤸🏻‍♂' => 'pria melakukan gerakan meroda: warna kulit cerah',
    '🤸🏼‍♂' => 'pria melakukan gerakan meroda: warna kulit cerah-sedang',
    '🤸🏽‍♂' => 'pria melakukan gerakan meroda: warna kulit sedang',
    '🤸🏾‍♂' => 'pria melakukan gerakan meroda: warna kulit gelap-sedang',
    '🤸🏿‍♂' => 'pria melakukan gerakan meroda: warna kulit gelap',
    '🤸🏻‍♀' => 'wanita melakukan gerakan meroda: warna kulit cerah',
    '🤸🏼‍♀' => 'wanita melakukan gerakan meroda: warna kulit cerah-sedang',
    '🤸🏽‍♀' => 'wanita melakukan gerakan meroda: warna kulit sedang',
    '🤸🏾‍♀' => 'wanita melakukan gerakan meroda: warna kulit gelap-sedang',
    '🤸🏿‍♀' => 'wanita melakukan gerakan meroda: warna kulit gelap',
    '🤼🏻‍♀' => 'wanita bergulat: warna kulit cerah',
    '🤼🏼‍♀' => 'wanita bergulat: warna kulit cerah-sedang',
    '🤼🏽‍♀' => 'wanita bergulat: warna kulit sedang',
    '🤼🏾‍♀' => 'wanita bergulat: warna kulit gelap-sedang',
    '🤼🏿‍♀' => 'wanita bergulat: warna kulit gelap',
    '🤼🏻‍♂' => 'pria bergulat: warna kulit cerah',
    '🤼🏼‍♂' => 'pria bergulat: warna kulit cerah-sedang',
    '🤼🏽‍♂' => 'pria bergulat: warna kulit sedang',
    '🤼🏾‍♂' => 'pria bergulat: warna kulit gelap-sedang',
    '🤼🏿‍♂' => 'pria bergulat: warna kulit gelap',
    '🤽🏻‍♂' => 'pria bermain polo air: warna kulit cerah',
    '🤽🏼‍♂' => 'pria bermain polo air: warna kulit cerah-sedang',
    '🤽🏽‍♂' => 'pria bermain polo air: warna kulit sedang',
    '🤽🏾‍♂' => 'pria bermain polo air: warna kulit gelap-sedang',
    '🤽🏿‍♂' => 'pria bermain polo air: warna kulit gelap',
    '🤽🏻‍♀' => 'wanita bermain polo air: warna kulit cerah',
    '🤽🏼‍♀' => 'wanita bermain polo air: warna kulit cerah-sedang',
    '🤽🏽‍♀' => 'wanita bermain polo air: warna kulit sedang',
    '🤽🏾‍♀' => 'wanita bermain polo air: warna kulit gelap-sedang',
    '🤽🏿‍♀' => 'wanita bermain polo air: warna kulit gelap',
    '🤾🏻‍♂' => 'pria bermain bola tangan: warna kulit cerah',
    '🤾🏼‍♂' => 'pria bermain bola tangan: warna kulit cerah-sedang',
    '🤾🏽‍♂' => 'pria bermain bola tangan: warna kulit sedang',
    '🤾🏾‍♂' => 'pria bermain bola tangan: warna kulit gelap-sedang',
    '🤾🏿‍♂' => 'pria bermain bola tangan: warna kulit gelap',
    '🤾🏻‍♀' => 'wanita bermain bola tangan: warna kulit cerah',
    '🤾🏼‍♀' => 'wanita bermain bola tangan: warna kulit cerah-sedang',
    '🤾🏽‍♀' => 'wanita bermain bola tangan: warna kulit sedang',
    '🤾🏾‍♀' => 'wanita bermain bola tangan: warna kulit gelap-sedang',
    '🤾🏿‍♀' => 'wanita bermain bola tangan: warna kulit gelap',
    '🤹🏻‍♂' => 'pria melempar: warna kulit cerah',
    '🤹🏼‍♂' => 'pria melempar: warna kulit cerah-sedang',
    '🤹🏽‍♂' => 'pria melempar: warna kulit sedang',
    '🤹🏾‍♂' => 'pria melempar: warna kulit gelap-sedang',
    '🤹🏿‍♂' => 'pria melempar: warna kulit gelap',
    '🤹🏻‍♀' => 'wanita melempar: warna kulit cerah',
    '🤹🏼‍♀' => 'wanita melempar: warna kulit cerah-sedang',
    '🤹🏽‍♀' => 'wanita melempar: warna kulit sedang',
    '🤹🏾‍♀' => 'wanita melempar: warna kulit gelap-sedang',
    '🤹🏿‍♀' => 'wanita melempar: warna kulit gelap',
    '🧘🏻‍♂' => 'pria dalam posisi teratai: warna kulit cerah',
    '🧘🏼‍♂' => 'pria dalam posisi teratai: warna kulit cerah-sedang',
    '🧘🏽‍♂' => 'pria dalam posisi teratai: warna kulit sedang',
    '🧘🏾‍♂' => 'pria dalam posisi teratai: warna kulit gelap-sedang',
    '🧘🏿‍♂' => 'pria dalam posisi teratai: warna kulit gelap',
    '🧘🏻‍♀' => 'wanita dalam posisi teratai: warna kulit cerah',
    '🧘🏼‍♀' => 'wanita dalam posisi teratai: warna kulit cerah-sedang',
    '🧘🏽‍♀' => 'wanita dalam posisi teratai: warna kulit sedang',
    '🧘🏾‍♀' => 'wanita dalam posisi teratai: warna kulit gelap-sedang',
    '🧘🏿‍♀' => 'wanita dalam posisi teratai: warna kulit gelap',
    '🧑‍🦰' => 'dewasa: rambut merah',
    '🧑‍🦱' => 'dewasa: rambut keriting',
    '🧑‍🦳' => 'dewasa: rambut putih',
    '🧑‍🦲' => 'dewasa: gundul',
    '👨‍🦰' => 'pria: rambut merah',
    '👨‍🦱' => 'pria: rambut keriting',
    '👨‍🦳' => 'pria: rambut putih',
    '👨‍🦲' => 'pria: gundul',
    '👩‍🦰' => 'wanita: rambut merah',
    '👩‍🦱' => 'wanita: rambut keriting',
    '👩‍🦳' => 'wanita: rambut putih',
    '👩‍🦲' => 'wanita: gundul',
    '🚶‍➡' => 'orang berjalan kaki: menghadap kanan',
    '🧎‍➡' => 'orang berlutut: menghadap kanan',
    '🏃‍➡' => 'orang berlari: menghadap kanan',
    '👨‍👦' => 'keluarga: pria, anak laki-laki',
    '👨‍👧' => 'keluarga: pria, anak perempuan',
    '👩‍👦' => 'keluarga: wanita, anak laki-laki',
    '👩‍👧' => 'keluarga: wanita, anak perempuan',
    '😶‍🌫' => 'wajah di awan',
    '😮‍💨' => 'wajah mengembuskan napas',
    '🙂‍↕' => 'kepala mengangguk',
    '😵‍💫' => 'wajah dengan mata berputar',
    '❤‍🔥' => 'hati berapi-api',
    '❤‍🩹' => 'mengobati hati terluka',
    '👁‍🗨' => 'mata di dalam gelembung ucapan',
    '🧔‍♂' => 'pria: janggut',
    '👱‍♂' => 'pria berambut pirang',
    '🧔‍♀' => 'wanita: janggut',
    '👱‍♀' => 'wanita berambut pirang',
    '🙍‍♂' => 'pria muka tidak senang',
    '🙍‍♀' => 'wanita muka tidak senang',
    '🙎‍♂' => 'pria tidak senang',
    '🙎‍♀' => 'wanita tidak senang',
    '🙅‍♂' => 'pria mengisyaratkan TIDAK',
    '🙅‍♀' => 'wanita mengisyaratkan TIDAK',
    '🙆‍♂' => 'pria mengisyaratkan OKE',
    '🙆‍♀' => 'wanita mengisyaratkan OKE',
    '💁‍♂' => 'pria menengadahkan tangan',
    '💁‍♀' => 'wanita menengadahkan tangan',
    '🙋‍♂' => 'pria mengangkat tangan',
    '🙋‍♀' => 'wanita mengangkat tangan',
    '🧏‍♂' => 'pria tuli',
    '🧏‍♀' => 'wanita tuli',
    '🙇‍♂' => 'pria membungkuk',
    '🙇‍♀' => 'wanita membungkuk',
    '🤦‍♂' => 'pria tepuk jidat',
    '🤦‍♀' => 'wanita tepuk jidat',
    '🤷‍♂' => 'pria mengangkat bahu',
    '🤷‍♀' => 'wanita mengangkat bahu',
    '🧑‍⚕' => 'petugas kesehatan',
    '👨‍⚕' => 'pekerja kesehatan pria',
    '👩‍⚕' => 'pekerja kesehatan wanita',
    '🧑‍🎓' => 'murid',
    '👨‍🎓' => 'sarjana pria',
    '👩‍🎓' => 'sarjana wanita',
    '🧑‍🏫' => 'guru',
    '👨‍🏫' => 'pengajar pria',
    '👩‍🏫' => 'pengajar wanita',
    '🧑‍⚖' => 'hakim',
    '👨‍⚖' => 'hakim pria',
    '👩‍⚖' => 'hakim wanita',
    '🧑‍🌾' => 'petani',
    '👨‍🌾' => 'petani pria',
    '👩‍🌾' => 'petani wanita',
    '🧑‍🍳' => 'koki',
    '👨‍🍳' => 'koki pria',
    '👩‍🍳' => 'koki wanita',
    '🧑‍🔧' => 'mekanik',
    '👨‍🔧' => 'montir pria',
    '👩‍🔧' => 'montir wanita',
    '🧑‍🏭' => 'pekerja pabrik',
    '👨‍🏭' => 'pekerja pabrik pria',
    '👩‍🏭' => 'pekerja pabrik wanita',
    '🧑‍💼' => 'pekerja kantoran',
    '👨‍💼' => 'pekerja kantor pria',
    '👩‍💼' => 'pekerja kantor wanita',
    '🧑‍🔬' => 'ilmuwan',
    '👨‍🔬' => 'ilmuwan pria',
    '👩‍🔬' => 'ilmuwan wanita',
    '🧑‍💻' => 'ahli teknologi',
    '👨‍💻' => 'ahli teknologi pria',
    '👩‍💻' => 'ahli teknologi wanita',
    '🧑‍🎤' => 'penyanyi',
    '👨‍🎤' => 'penyanyi pria',
    '👩‍🎤' => 'penyanyi wanita',
    '🧑‍🎨' => 'seniman',
    '👨‍🎨' => 'seniman pria',
    '👩‍🎨' => 'seniman wanita',
    '🧑‍✈' => 'pilot',
    '👨‍✈' => 'pilot pria',
    '👩‍✈' => 'pilot wanita',
    '🧑‍🚀' => 'astronaut',
    '👨‍🚀' => 'antariksawan pria',
    '👩‍🚀' => 'antariksawan wanita',
    '🧑‍🚒' => 'pemadam kebakaran',
    '👨‍🚒' => 'pemadam kebakaran pria',
    '👩‍🚒' => 'pemadam kebakaran wanita',
    '👮‍♂' => 'polisi pria',
    '👮‍♀' => 'polisi wanita',
    '🕵‍♂' => 'detektif pria',
    '🕵‍♀' => 'detektif wanita',
    '💂‍♂' => 'penjaga pria',
    '💂‍♀' => 'penjaga wanita',
    '👷‍♂' => 'pekerja konstruksi pria',
    '👷‍♀' => 'pekerja konstruksi wanita',
    '👳‍♂' => 'pria mengenakan serban',
    '👳‍♀' => 'wanita mengenakan serban',
    '🤵‍♂' => 'pria dewasa bertuksedo',
    '🤵‍♀' => 'wanita bertuksedo',
    '👰‍♂' => 'pria berpenutup kepala',
    '👰‍♀' => 'wanita berpenutup kepala',
    '👩‍🍼' => 'wanita menyusui bayi',
    '👨‍🍼' => 'pria menyusui bayi',
    '🧑‍🍼' => 'orang menyusui bayi',
    '🧑‍🎄' => 'manusia sinterklas',
    '🦸‍♂' => 'pahlawan super pria',
    '🦸‍♀' => 'pahlawan super wanita',
    '🦹‍♂' => 'penjahat super pria',
    '🦹‍♀' => 'penjahat super wanita',
    '🧙‍♂' => 'magi pria',
    '🧙‍♀' => 'magi wanita',
    '🧚‍♂' => 'peri pria',
    '🧚‍♀' => 'peri wanita',
    '🧛‍♂' => 'vampir pria',
    '🧛‍♀' => 'vampir wanita',
    '🧜‍♂' => 'pria duyung',
    '🧜‍♀' => 'putri duyung',
    '🧝‍♂' => 'kurcaci pria',
    '🧝‍♀' => 'kurcaci wanita',
    '🧞‍♂' => 'jin pria',
    '🧞‍♀' => 'jin wanita',
    '🧟‍♂' => 'zombi pria',
    '🧟‍♀' => 'zombi wanita',
    '💆‍♂' => 'pria dipijat kepalanya',
    '💆‍♀' => 'wanita dipijat',
    '💇‍♂' => 'pria potong rambut',
    '💇‍♀' => 'wanita potong rambut',
    '🚶‍♂' => 'pria berjalan kaki',
    '🚶‍♀' => 'wanita berjalan kaki',
    '🧍‍♂' => 'pria berdiri',
    '🧍‍♀' => 'wanita berdiri',
    '🧎‍♂' => 'pria berlutut',
    '🧎‍♀' => 'wanita berlutut',
    '🧑‍🦯' => 'orang dengan tongkat',
    '👨‍🦯' => 'pria dengan tongkat',
    '👩‍🦯' => 'wanita dengan tongkat',
    '🧑‍🦼' => 'orang dengan kursi roda bermotor',
    '👨‍🦼' => 'pria dengan kursi roda bermotor',
    '👩‍🦼' => 'wanita dengan kursi roda bermotor',
    '🧑‍🦽' => 'orang dengan kursi roda manual',
    '👨‍🦽' => 'pria dengan kursi roda manual',
    '👩‍🦽' => 'wanita dengan kursi roda manual',
    '🏃‍♂' => 'pria berlari',
    '🏃‍♀' => 'wanita berlari',
    '🧑‍🩰' => 'penari balet',
    '👯‍♂' => 'pria dengan telinga kelinci',
    '👯‍♀' => 'wanita dengan telinga kelinci',
    '🧖‍♂' => 'pria di ruangan beruap',
    '🧖‍♀' => 'wanita di ruangan beruap',
    '🧗‍♂' => 'pria memanjat',
    '🧗‍♀' => 'wanita memanjat',
    '🏌‍♂' => 'pria bermain golf',
    '🏌‍♀' => 'wanita bermain golf',
    '🏄‍♂' => 'pria berselancar',
    '🏄‍♀' => 'wanita berselancar',
    '🚣‍♂' => 'pria mendayung perahu',
    '🚣‍♀' => 'wanita mendayung perahu',
    '🏊‍♂' => 'pria berenang',
    '🏊‍♀' => 'wanita berenang',
    '⛹‍♂' => 'pria memantulkan bola',
    '⛹‍♀' => 'wanita memantulkan bola',
    '🏋‍♂' => 'pria mengangkat beban',
    '🏋‍♀' => 'wanita mengangkat beban',
    '🚴‍♂' => 'pria bersepeda',
    '🚴‍♀' => 'wanita bersepeda',
    '🚵‍♂' => 'pria bersepeda gunung',
    '🚵‍♀' => 'wanita bersepeda gunung',
    '🤸‍♂' => 'pria melakukan gerakan meroda',
    '🤸‍♀' => 'wanita melakukan gerakan meroda',
    '🤼‍♂' => 'pria bergulat',
    '🤼‍♀' => 'wanita bergulat',
    '🤽‍♂' => 'pria bermain polo air',
    '🤽‍♀' => 'wanita bermain polo air',
    '🤾‍♂' => 'pria bermain bola tangan',
    '🤾‍♀' => 'wanita bermain bola tangan',
    '🤹‍♂' => 'pria melempar',
    '🤹‍♀' => 'wanita melempar',
    '🧘‍♂' => 'pria dalam posisi teratai',
    '🧘‍♀' => 'wanita dalam posisi teratai',
    '🧑‍🧒' => 'keluarga: orang dewasa, anak',
    '🐕‍🦺' => 'anjing pembimbing',
    '🐈‍⬛' => 'kucing hitam',
    '🐻‍❄' => 'beruang kutub',
    '🐦‍⬛' => 'burung hitam',
    '🐦‍🔥' => 'phoenix',
    '🍋‍🟩' => 'jeruk limau',
    '🍄‍🟫' => 'jamur cokelat',
    '⛓‍💥' => 'rantai putus',
    '🏳‍🌈' => 'bendera pelangi',
    '🏳‍⚧' => 'bendera transgender',
    '🏴‍☠' => 'bendera bajak laut',
    '👋🏻' => 'tangan melambai: warna kulit cerah',
    '👋🏼' => 'tangan melambai: warna kulit cerah-sedang',
    '👋🏽' => 'tangan melambai: warna kulit sedang',
    '👋🏾' => 'tangan melambai: warna kulit gelap-sedang',
    '👋🏿' => 'tangan melambai: warna kulit gelap',
    '🤚🏻' => 'angkat tangan: warna kulit cerah',
    '🤚🏼' => 'angkat tangan: warna kulit cerah-sedang',
    '🤚🏽' => 'angkat tangan: warna kulit sedang',
    '🤚🏾' => 'angkat tangan: warna kulit gelap-sedang',
    '🤚🏿' => 'angkat tangan: warna kulit gelap',
    '🖐🏻' => 'telapak tangan terbuka: warna kulit cerah',
    '🖐🏼' => 'telapak tangan terbuka: warna kulit cerah-sedang',
    '🖐🏽' => 'telapak tangan terbuka: warna kulit sedang',
    '🖐🏾' => 'telapak tangan terbuka: warna kulit gelap-sedang',
    '🖐🏿' => 'telapak tangan terbuka: warna kulit gelap',
    '✋🏻' => 'telapak tangan terbuka dengan jari rapat: warna kulit cerah',
    '✋🏼' => 'telapak tangan terbuka dengan jari rapat: warna kulit cerah-sedang',
    '✋🏽' => 'telapak tangan terbuka dengan jari rapat: warna kulit sedang',
    '✋🏾' => 'telapak tangan terbuka dengan jari rapat: warna kulit gelap-sedang',
    '✋🏿' => 'telapak tangan terbuka dengan jari rapat: warna kulit gelap',
    '🖖🏻' => 'salam vulcan: warna kulit cerah',
    '🖖🏼' => 'salam vulcan: warna kulit cerah-sedang',
    '🖖🏽' => 'salam vulcan: warna kulit sedang',
    '🖖🏾' => 'salam vulcan: warna kulit gelap-sedang',
    '🖖🏿' => 'salam vulcan: warna kulit gelap',
    '🫱🏻' => 'tangan menghadap kanan: warna kulit cerah',
    '🫱🏼' => 'tangan menghadap kanan: warna kulit cerah-sedang',
    '🫱🏽' => 'tangan menghadap kanan: warna kulit sedang',
    '🫱🏾' => 'tangan menghadap kanan: warna kulit gelap-sedang',
    '🫱🏿' => 'tangan menghadap kanan: warna kulit gelap',
    '🫲🏻' => 'tangan menghadap kiri: warna kulit cerah',
    '🫲🏼' => 'tangan menghadap kiri: warna kulit cerah-sedang',
    '🫲🏽' => 'tangan menghadap kiri: warna kulit sedang',
    '🫲🏾' => 'tangan menghadap kiri: warna kulit gelap-sedang',
    '🫲🏿' => 'tangan menghadap kiri: warna kulit gelap',
    '🫳🏻' => 'telapak tangan menghadap ke bawah: warna kulit cerah',
    '🫳🏼' => 'telapak tangan menghadap ke bawah: warna kulit cerah-sedang',
    '🫳🏽' => 'telapak tangan menghadap ke bawah: warna kulit sedang',
    '🫳🏾' => 'telapak tangan menghadap ke bawah: warna kulit gelap-sedang',
    '🫳🏿' => 'telapak tangan menghadap ke bawah: warna kulit gelap',
    '🫴🏻' => 'telapak tangan menghadap ke atas: warna kulit cerah',
    '🫴🏼' => 'telapak tangan menghadap ke atas: warna kulit cerah-sedang',
    '🫴🏽' => 'telapak tangan menghadap ke atas: warna kulit sedang',
    '🫴🏾' => 'telapak tangan menghadap ke atas: warna kulit gelap-sedang',
    '🫴🏿' => 'telapak tangan menghadap ke atas: warna kulit gelap',
    '🫷🏻' => 'tangan mendorong ke kiri: warna kulit cerah',
    '🫷🏼' => 'tangan mendorong ke kiri: warna kulit cerah-sedang',
    '🫷🏽' => 'tangan mendorong ke kiri: warna kulit sedang',
    '🫷🏾' => 'tangan mendorong ke kiri: warna kulit gelap-sedang',
    '🫷🏿' => 'tangan mendorong ke kiri: warna kulit gelap',
    '🫸🏻' => 'tangan mendorong ke kanan: warna kulit cerah',
    '🫸🏼' => 'tangan mendorong ke kanan: warna kulit cerah-sedang',
    '🫸🏽' => 'tangan mendorong ke kanan: warna kulit sedang',
    '🫸🏾' => 'tangan mendorong ke kanan: warna kulit gelap-sedang',
    '🫸🏿' => 'tangan mendorong ke kanan: warna kulit gelap',
    '👌🏻' => 'oke: warna kulit cerah',
    '👌🏼' => 'oke: warna kulit cerah-sedang',
    '👌🏽' => 'oke: warna kulit sedang',
    '👌🏾' => 'oke: warna kulit gelap-sedang',
    '👌🏿' => 'oke: warna kulit gelap',
    '🤌🏻' => 'jari terkuncup: warna kulit cerah',
    '🤌🏼' => 'jari terkuncup: warna kulit cerah-sedang',
    '🤌🏽' => 'jari terkuncup: warna kulit sedang',
    '🤌🏾' => 'jari terkuncup: warna kulit gelap-sedang',
    '🤌🏿' => 'jari terkuncup: warna kulit gelap',
    '🤏🏻' => 'tangan menjepit: warna kulit cerah',
    '🤏🏼' => 'tangan menjepit: warna kulit cerah-sedang',
    '🤏🏽' => 'tangan menjepit: warna kulit sedang',
    '🤏🏾' => 'tangan menjepit: warna kulit gelap-sedang',
    '🤏🏿' => 'tangan menjepit: warna kulit gelap',
    '✌🏻' => 'isyarat kemenangan: warna kulit cerah',
    '✌🏼' => 'isyarat kemenangan: warna kulit cerah-sedang',
    '✌🏽' => 'isyarat kemenangan: warna kulit sedang',
    '✌🏾' => 'isyarat kemenangan: warna kulit gelap-sedang',
    '✌🏿' => 'isyarat kemenangan: warna kulit gelap',
    '🤞🏻' => 'jari menyilang: warna kulit cerah',
    '🤞🏼' => 'jari menyilang: warna kulit cerah-sedang',
    '🤞🏽' => 'jari menyilang: warna kulit sedang',
    '🤞🏾' => 'jari menyilang: warna kulit gelap-sedang',
    '🤞🏿' => 'jari menyilang: warna kulit gelap',
    '🫰🏻' => 'tangan dengan jari telunjuk dan ibu jari menyilang: warna kulit cerah',
    '🫰🏼' => 'tangan dengan jari telunjuk dan ibu jari menyilang: warna kulit cerah-sedang',
    '🫰🏽' => 'tangan dengan jari telunjuk dan ibu jari menyilang: warna kulit sedang',
    '🫰🏾' => 'tangan dengan jari telunjuk dan ibu jari menyilang: warna kulit gelap-sedang',
    '🫰🏿' => 'tangan dengan jari telunjuk dan ibu jari menyilang: warna kulit gelap',
    '🤟🏻' => 'gerakan sayang kamu: warna kulit cerah',
    '🤟🏼' => 'gerakan sayang kamu: warna kulit cerah-sedang',
    '🤟🏽' => 'gerakan sayang kamu: warna kulit sedang',
    '🤟🏾' => 'gerakan sayang kamu: warna kulit gelap-sedang',
    '🤟🏿' => 'gerakan sayang kamu: warna kulit gelap',
    '🤘🏻' => 'lambang tanduk: warna kulit cerah',
    '🤘🏼' => 'lambang tanduk: warna kulit cerah-sedang',
    '🤘🏽' => 'lambang tanduk: warna kulit sedang',
    '🤘🏾' => 'lambang tanduk: warna kulit gelap-sedang',
    '🤘🏿' => 'lambang tanduk: warna kulit gelap',
    '🤙🏻' => 'panggil saya: warna kulit cerah',
    '🤙🏼' => 'panggil saya: warna kulit cerah-sedang',
    '🤙🏽' => 'panggil saya: warna kulit sedang',
    '🤙🏾' => 'panggil saya: warna kulit gelap-sedang',
    '🤙🏿' => 'panggil saya: warna kulit gelap',
    '👈🏻' => 'menunjuk ke arah kiri: warna kulit cerah',
    '👈🏼' => 'menunjuk ke arah kiri: warna kulit cerah-sedang',
    '👈🏽' => 'menunjuk ke arah kiri: warna kulit sedang',
    '👈🏾' => 'menunjuk ke arah kiri: warna kulit gelap-sedang',
    '👈🏿' => 'menunjuk ke arah kiri: warna kulit gelap',
    '👉🏻' => 'menunjuk ke arah kanan: warna kulit cerah',
    '👉🏼' => 'menunjuk ke arah kanan: warna kulit cerah-sedang',
    '👉🏽' => 'menunjuk ke arah kanan: warna kulit sedang',
    '👉🏾' => 'menunjuk ke arah kanan: warna kulit gelap-sedang',
    '👉🏿' => 'menunjuk ke arah kanan: warna kulit gelap',
    '👆🏻' => 'menunjuk ke atas (dibalik): warna kulit cerah',
    '👆🏼' => 'menunjuk ke atas (dibalik): warna kulit cerah-sedang',
    '👆🏽' => 'menunjuk ke atas (dibalik): warna kulit sedang',
    '👆🏾' => 'menunjuk ke atas (dibalik): warna kulit gelap-sedang',
    '👆🏿' => 'menunjuk ke atas (dibalik): warna kulit gelap',
    '🖕🏻' => 'jari tengah: warna kulit cerah',
    '🖕🏼' => 'jari tengah: warna kulit cerah-sedang',
    '🖕🏽' => 'jari tengah: warna kulit sedang',
    '🖕🏾' => 'jari tengah: warna kulit gelap-sedang',
    '🖕🏿' => 'jari tengah: warna kulit gelap',
    '👇🏻' => 'menunjuk ke bawah: warna kulit cerah',
    '👇🏼' => 'menunjuk ke bawah: warna kulit cerah-sedang',
    '👇🏽' => 'menunjuk ke bawah: warna kulit sedang',
    '👇🏾' => 'menunjuk ke bawah: warna kulit gelap-sedang',
    '👇🏿' => 'menunjuk ke bawah: warna kulit gelap',
    '☝🏻' => 'menunjuk ke atas: warna kulit cerah',
    '☝🏼' => 'menunjuk ke atas: warna kulit cerah-sedang',
    '☝🏽' => 'menunjuk ke atas: warna kulit sedang',
    '☝🏾' => 'menunjuk ke atas: warna kulit gelap-sedang',
    '☝🏿' => 'menunjuk ke atas: warna kulit gelap',
    '🫵🏻' => 'jari telunjuk menunjuk penonton: warna kulit cerah',
    '🫵🏼' => 'jari telunjuk menunjuk penonton: warna kulit cerah-sedang',
    '🫵🏽' => 'jari telunjuk menunjuk penonton: warna kulit sedang',
    '🫵🏾' => 'jari telunjuk menunjuk penonton: warna kulit gelap-sedang',
    '🫵🏿' => 'jari telunjuk menunjuk penonton: warna kulit gelap',
    '👍🏻' => 'jempol ke atas: warna kulit cerah',
    '👍🏼' => 'jempol ke atas: warna kulit cerah-sedang',
    '👍🏽' => 'jempol ke atas: warna kulit sedang',
    '👍🏾' => 'jempol ke atas: warna kulit gelap-sedang',
    '👍🏿' => 'jempol ke atas: warna kulit gelap',
    '👎🏻' => 'jempol ke bawah: warna kulit cerah',
    '👎🏼' => 'jempol ke bawah: warna kulit cerah-sedang',
    '👎🏽' => 'jempol ke bawah: warna kulit sedang',
    '👎🏾' => 'jempol ke bawah: warna kulit gelap-sedang',
    '👎🏿' => 'jempol ke bawah: warna kulit gelap',
    '✊🏻' => 'tangan terkepal ke atas: warna kulit cerah',
    '✊🏼' => 'tangan terkepal ke atas: warna kulit cerah-sedang',
    '✊🏽' => 'tangan terkepal ke atas: warna kulit sedang',
    '✊🏾' => 'tangan terkepal ke atas: warna kulit gelap-sedang',
    '✊🏿' => 'tangan terkepal ke atas: warna kulit gelap',
    '👊🏻' => 'tangan terkepal ke depan: warna kulit cerah',
    '👊🏼' => 'tangan terkepal ke depan: warna kulit cerah-sedang',
    '👊🏽' => 'tangan terkepal ke depan: warna kulit sedang',
    '👊🏾' => 'tangan terkepal ke depan: warna kulit gelap-sedang',
    '👊🏿' => 'tangan terkepal ke depan: warna kulit gelap',
    '🤛🏻' => 'tinju kiri: warna kulit cerah',
    '🤛🏼' => 'tinju kiri: warna kulit cerah-sedang',
    '🤛🏽' => 'tinju kiri: warna kulit sedang',
    '🤛🏾' => 'tinju kiri: warna kulit gelap-sedang',
    '🤛🏿' => 'tinju kiri: warna kulit gelap',
    '🤜🏻' => 'tinju kanan: warna kulit cerah',
    '🤜🏼' => 'tinju kanan: warna kulit cerah-sedang',
    '🤜🏽' => 'tinju kanan: warna kulit sedang',
    '🤜🏾' => 'tinju kanan: warna kulit gelap-sedang',
    '🤜🏿' => 'tinju kanan: warna kulit gelap',
    '👏🏻' => 'tepuk tangan: warna kulit cerah',
    '👏🏼' => 'tepuk tangan: warna kulit cerah-sedang',
    '👏🏽' => 'tepuk tangan: warna kulit sedang',
    '👏🏾' => 'tepuk tangan: warna kulit gelap-sedang',
    '👏🏿' => 'tepuk tangan: warna kulit gelap',
    '🙌🏻' => 'mengangkat tangan: warna kulit cerah',
    '🙌🏼' => 'mengangkat tangan: warna kulit cerah-sedang',
    '🙌🏽' => 'mengangkat tangan: warna kulit sedang',
    '🙌🏾' => 'mengangkat tangan: warna kulit gelap-sedang',
    '🙌🏿' => 'mengangkat tangan: warna kulit gelap',
    '🫶🏻' => 'tangan membentuk hati: warna kulit cerah',
    '🫶🏼' => 'tangan membentuk hati: warna kulit cerah-sedang',
    '🫶🏽' => 'tangan membentuk hati: warna kulit sedang',
    '🫶🏾' => 'tangan membentuk hati: warna kulit gelap-sedang',
    '🫶🏿' => 'tangan membentuk hati: warna kulit gelap',
    '👐🏻' => 'tangan terbuka: warna kulit cerah',
    '👐🏼' => 'tangan terbuka: warna kulit cerah-sedang',
    '👐🏽' => 'tangan terbuka: warna kulit sedang',
    '👐🏾' => 'tangan terbuka: warna kulit gelap-sedang',
    '👐🏿' => 'tangan terbuka: warna kulit gelap',
    '🤲🏻' => 'tangan diangkat dan dirapatkan bersama: warna kulit cerah',
    '🤲🏼' => 'tangan diangkat dan dirapatkan bersama: warna kulit cerah-sedang',
    '🤲🏽' => 'tangan diangkat dan dirapatkan bersama: warna kulit sedang',
    '🤲🏾' => 'tangan diangkat dan dirapatkan bersama: warna kulit gelap-sedang',
    '🤲🏿' => 'tangan diangkat dan dirapatkan bersama: warna kulit gelap',
    '🤝🏻' => 'jabat tangan: warna kulit cerah',
    '🤝🏼' => 'jabat tangan: warna kulit cerah-sedang',
    '🤝🏽' => 'jabat tangan: warna kulit sedang',
    '🤝🏾' => 'jabat tangan: warna kulit gelap-sedang',
    '🤝🏿' => 'jabat tangan: warna kulit gelap',
    '🙏🏻' => 'melipat tangan: warna kulit cerah',
    '🙏🏼' => 'melipat tangan: warna kulit cerah-sedang',
    '🙏🏽' => 'melipat tangan: warna kulit sedang',
    '🙏🏾' => 'melipat tangan: warna kulit gelap-sedang',
    '🙏🏿' => 'melipat tangan: warna kulit gelap',
    '✍🏻' => 'tangan menulis: warna kulit cerah',
    '✍🏼' => 'tangan menulis: warna kulit cerah-sedang',
    '✍🏽' => 'tangan menulis: warna kulit sedang',
    '✍🏾' => 'tangan menulis: warna kulit gelap-sedang',
    '✍🏿' => 'tangan menulis: warna kulit gelap',
    '💅🏻' => 'poles kuku: warna kulit cerah',
    '💅🏼' => 'poles kuku: warna kulit cerah-sedang',
    '💅🏽' => 'poles kuku: warna kulit sedang',
    '💅🏾' => 'poles kuku: warna kulit gelap-sedang',
    '💅🏿' => 'poles kuku: warna kulit gelap',
    '🤳🏻' => 'selfie: warna kulit cerah',
    '🤳🏼' => 'selfie: warna kulit cerah-sedang',
    '🤳🏽' => 'selfie: warna kulit sedang',
    '🤳🏾' => 'selfie: warna kulit gelap-sedang',
    '🤳🏿' => 'selfie: warna kulit gelap',
    '💪🏻' => 'lengan berotot: warna kulit cerah',
    '💪🏼' => 'lengan berotot: warna kulit cerah-sedang',
    '💪🏽' => 'lengan berotot: warna kulit sedang',
    '💪🏾' => 'lengan berotot: warna kulit gelap-sedang',
    '💪🏿' => 'lengan berotot: warna kulit gelap',
    '🦵🏻' => 'tungkai: warna kulit cerah',
    '🦵🏼' => 'tungkai: warna kulit cerah-sedang',
    '🦵🏽' => 'tungkai: warna kulit sedang',
    '🦵🏾' => 'tungkai: warna kulit gelap-sedang',
    '🦵🏿' => 'tungkai: warna kulit gelap',
    '🦶🏻' => 'telapak kaki: warna kulit cerah',
    '🦶🏼' => 'telapak kaki: warna kulit cerah-sedang',
    '🦶🏽' => 'telapak kaki: warna kulit sedang',
    '🦶🏾' => 'telapak kaki: warna kulit gelap-sedang',
    '🦶🏿' => 'telapak kaki: warna kulit gelap',
    '👂🏻' => 'telinga: warna kulit cerah',
    '👂🏼' => 'telinga: warna kulit cerah-sedang',
    '👂🏽' => 'telinga: warna kulit sedang',
    '👂🏾' => 'telinga: warna kulit gelap-sedang',
    '👂🏿' => 'telinga: warna kulit gelap',
    '🦻🏻' => 'telinga dengan alat bantu dengar: warna kulit cerah',
    '🦻🏼' => 'telinga dengan alat bantu dengar: warna kulit cerah-sedang',
    '🦻🏽' => 'telinga dengan alat bantu dengar: warna kulit sedang',
    '🦻🏾' => 'telinga dengan alat bantu dengar: warna kulit gelap-sedang',
    '🦻🏿' => 'telinga dengan alat bantu dengar: warna kulit gelap',
    '👃🏻' => 'hidung: warna kulit cerah',
    '👃🏼' => 'hidung: warna kulit cerah-sedang',
    '👃🏽' => 'hidung: warna kulit sedang',
    '👃🏾' => 'hidung: warna kulit gelap-sedang',
    '👃🏿' => 'hidung: warna kulit gelap',
    '👶🏻' => 'bayi: warna kulit cerah',
    '👶🏼' => 'bayi: warna kulit cerah-sedang',
    '👶🏽' => 'bayi: warna kulit sedang',
    '👶🏾' => 'bayi: warna kulit gelap-sedang',
    '👶🏿' => 'bayi: warna kulit gelap',
    '🧒🏻' => 'anak: warna kulit cerah',
    '🧒🏼' => 'anak: warna kulit cerah-sedang',
    '🧒🏽' => 'anak: warna kulit sedang',
    '🧒🏾' => 'anak: warna kulit gelap-sedang',
    '🧒🏿' => 'anak: warna kulit gelap',
    '👦🏻' => 'anak laki-laki: warna kulit cerah',
    '👦🏼' => 'anak laki-laki: warna kulit cerah-sedang',
    '👦🏽' => 'anak laki-laki: warna kulit sedang',
    '👦🏾' => 'anak laki-laki: warna kulit gelap-sedang',
    '👦🏿' => 'anak laki-laki: warna kulit gelap',
    '👧🏻' => 'anak perempuan: warna kulit cerah',
    '👧🏼' => 'anak perempuan: warna kulit cerah-sedang',
    '👧🏽' => 'anak perempuan: warna kulit sedang',
    '👧🏾' => 'anak perempuan: warna kulit gelap-sedang',
    '👧🏿' => 'anak perempuan: warna kulit gelap',
    '🧑🏻' => 'dewasa: warna kulit cerah',
    '🧑🏼' => 'dewasa: warna kulit cerah-sedang',
    '🧑🏽' => 'dewasa: warna kulit sedang',
    '🧑🏾' => 'dewasa: warna kulit gelap-sedang',
    '🧑🏿' => 'dewasa: warna kulit gelap',
    '👱🏻' => 'orang: warna kulit cerah, rambut pirang',
    '👱🏼' => 'orang: warna kulit cerah-sedang, rambut pirang',
    '👱🏽' => 'orang: warna kulit sedang, rambut pirang',
    '👱🏾' => 'orang: warna kulit gelap-sedang, rambut pirang',
    '👱🏿' => 'orang: warna kulit gelap, rambut pirang',
    '👨🏻' => 'pria: warna kulit cerah',
    '👨🏼' => 'pria: warna kulit cerah-sedang',
    '👨🏽' => 'pria: warna kulit sedang',
    '👨🏾' => 'pria: warna kulit gelap-sedang',
    '👨🏿' => 'pria: warna kulit gelap',
    '🧔🏻' => 'orang: warna kulit cerah, janggut',
    '🧔🏼' => 'orang: warna kulit cerah-sedang, janggut',
    '🧔🏽' => 'orang: warna kulit sedang, janggut',
    '🧔🏾' => 'orang: warna kulit gelap-sedang, janggut',
    '🧔🏿' => 'orang: warna kulit gelap, janggut',
    '👩🏻' => 'wanita: warna kulit cerah',
    '👩🏼' => 'wanita: warna kulit cerah-sedang',
    '👩🏽' => 'wanita: warna kulit sedang',
    '👩🏾' => 'wanita: warna kulit gelap-sedang',
    '👩🏿' => 'wanita: warna kulit gelap',
    '🧓🏻' => 'lanjut usia: warna kulit cerah',
    '🧓🏼' => 'lanjut usia: warna kulit cerah-sedang',
    '🧓🏽' => 'lanjut usia: warna kulit sedang',
    '🧓🏾' => 'lanjut usia: warna kulit gelap-sedang',
    '🧓🏿' => 'lanjut usia: warna kulit gelap',
    '👴🏻' => 'pria tua: warna kulit cerah',
    '👴🏼' => 'pria tua: warna kulit cerah-sedang',
    '👴🏽' => 'pria tua: warna kulit sedang',
    '👴🏾' => 'pria tua: warna kulit gelap-sedang',
    '👴🏿' => 'pria tua: warna kulit gelap',
    '👵🏻' => 'wanita tua: warna kulit cerah',
    '👵🏼' => 'wanita tua: warna kulit cerah-sedang',
    '👵🏽' => 'wanita tua: warna kulit sedang',
    '👵🏾' => 'wanita tua: warna kulit gelap-sedang',
    '👵🏿' => 'wanita tua: warna kulit gelap',
    '🙍🏻' => 'orang marah: warna kulit cerah',
    '🙍🏼' => 'orang marah: warna kulit cerah-sedang',
    '🙍🏽' => 'orang marah: warna kulit sedang',
    '🙍🏾' => 'orang marah: warna kulit gelap-sedang',
    '🙍🏿' => 'orang marah: warna kulit gelap',
    '🙎🏻' => 'orang tidak senang: warna kulit cerah',
    '🙎🏼' => 'orang tidak senang: warna kulit cerah-sedang',
    '🙎🏽' => 'orang tidak senang: warna kulit sedang',
    '🙎🏾' => 'orang tidak senang: warna kulit gelap-sedang',
    '🙎🏿' => 'orang tidak senang: warna kulit gelap',
    '🙅🏻' => 'orang mengisyaratkan TIDAK: warna kulit cerah',
    '🙅🏼' => 'orang mengisyaratkan TIDAK: warna kulit cerah-sedang',
    '🙅🏽' => 'orang mengisyaratkan TIDAK: warna kulit sedang',
    '🙅🏾' => 'orang mengisyaratkan TIDAK: warna kulit gelap-sedang',
    '🙅🏿' => 'orang mengisyaratkan TIDAK: warna kulit gelap',
    '🙆🏻' => 'orang mengisyaratkan OKE: warna kulit cerah',
    '🙆🏼' => 'orang mengisyaratkan OKE: warna kulit cerah-sedang',
    '🙆🏽' => 'orang mengisyaratkan OKE: warna kulit sedang',
    '🙆🏾' => 'orang mengisyaratkan OKE: warna kulit gelap-sedang',
    '🙆🏿' => 'orang mengisyaratkan OKE: warna kulit gelap',
    '💁🏻' => 'orang menengadahkan tangan: warna kulit cerah',
    '💁🏼' => 'orang menengadahkan tangan: warna kulit cerah-sedang',
    '💁🏽' => 'orang menengadahkan tangan: warna kulit sedang',
    '💁🏾' => 'orang menengadahkan tangan: warna kulit gelap-sedang',
    '💁🏿' => 'orang menengadahkan tangan: warna kulit gelap',
    '🙋🏻' => 'orang mengangkat tangan: warna kulit cerah',
    '🙋🏼' => 'orang mengangkat tangan: warna kulit cerah-sedang',
    '🙋🏽' => 'orang mengangkat tangan: warna kulit sedang',
    '🙋🏾' => 'orang mengangkat tangan: warna kulit gelap-sedang',
    '🙋🏿' => 'orang mengangkat tangan: warna kulit gelap',
    '🧏🏻' => 'orang tuli: warna kulit cerah',
    '🧏🏼' => 'orang tuli: warna kulit cerah-sedang',
    '🧏🏽' => 'orang tuli: warna kulit sedang',
    '🧏🏾' => 'orang tuli: warna kulit gelap-sedang',
    '🧏🏿' => 'orang tuli: warna kulit gelap',
    '🙇🏻' => 'orang membungkuk: warna kulit cerah',
    '🙇🏼' => 'orang membungkuk: warna kulit cerah-sedang',
    '🙇🏽' => 'orang membungkuk: warna kulit sedang',
    '🙇🏾' => 'orang membungkuk: warna kulit gelap-sedang',
    '🙇🏿' => 'orang membungkuk: warna kulit gelap',
    '🤦🏻' => 'orang tepuk jidat: warna kulit cerah',
    '🤦🏼' => 'orang tepuk jidat: warna kulit cerah-sedang',
    '🤦🏽' => 'orang tepuk jidat: warna kulit sedang',
    '🤦🏾' => 'orang tepuk jidat: warna kulit gelap-sedang',
    '🤦🏿' => 'orang tepuk jidat: warna kulit gelap',
    '🤷🏻' => 'orang mengangkat bahu: warna kulit cerah',
    '🤷🏼' => 'orang mengangkat bahu: warna kulit cerah-sedang',
    '🤷🏽' => 'orang mengangkat bahu: warna kulit sedang',
    '🤷🏾' => 'orang mengangkat bahu: warna kulit gelap-sedang',
    '🤷🏿' => 'orang mengangkat bahu: warna kulit gelap',
    '👮🏻' => 'polisi: warna kulit cerah',
    '👮🏼' => 'polisi: warna kulit cerah-sedang',
    '👮🏽' => 'polisi: warna kulit sedang',
    '👮🏾' => 'polisi: warna kulit gelap-sedang',
    '👮🏿' => 'polisi: warna kulit gelap',
    '🕵🏻' => 'detektif: warna kulit cerah',
    '🕵🏼' => 'detektif: warna kulit cerah-sedang',
    '🕵🏽' => 'detektif: warna kulit sedang',
    '🕵🏾' => 'detektif: warna kulit gelap-sedang',
    '🕵🏿' => 'detektif: warna kulit gelap',
    '💂🏻' => 'penjaga: warna kulit cerah',
    '💂🏼' => 'penjaga: warna kulit cerah-sedang',
    '💂🏽' => 'penjaga: warna kulit sedang',
    '💂🏾' => 'penjaga: warna kulit gelap-sedang',
    '💂🏿' => 'penjaga: warna kulit gelap',
    '🥷🏻' => 'ninja: warna kulit cerah',
    '🥷🏼' => 'ninja: warna kulit cerah-sedang',
    '🥷🏽' => 'ninja: warna kulit sedang',
    '🥷🏾' => 'ninja: warna kulit gelap-sedang',
    '🥷🏿' => 'ninja: warna kulit gelap',
    '👷🏻' => 'pekerja konstruksi: warna kulit cerah',
    '👷🏼' => 'pekerja konstruksi: warna kulit cerah-sedang',
    '👷🏽' => 'pekerja konstruksi: warna kulit sedang',
    '👷🏾' => 'pekerja konstruksi: warna kulit gelap-sedang',
    '👷🏿' => 'pekerja konstruksi: warna kulit gelap',
    '🫅🏻' => 'orang mengenakan mahkota: warna kulit cerah',
    '🫅🏼' => 'orang mengenakan mahkota: warna kulit cerah-sedang',
    '🫅🏽' => 'orang mengenakan mahkota: warna kulit sedang',
    '🫅🏾' => 'orang mengenakan mahkota: warna kulit gelap-sedang',
    '🫅🏿' => 'orang mengenakan mahkota: warna kulit gelap',
    '🤴🏻' => 'pangeran: warna kulit cerah',
    '🤴🏼' => 'pangeran: warna kulit cerah-sedang',
    '🤴🏽' => 'pangeran: warna kulit sedang',
    '🤴🏾' => 'pangeran: warna kulit gelap-sedang',
    '🤴🏿' => 'pangeran: warna kulit gelap',
    '👸🏻' => 'putri: warna kulit cerah',
    '👸🏼' => 'putri: warna kulit cerah-sedang',
    '👸🏽' => 'putri: warna kulit sedang',
    '👸🏾' => 'putri: warna kulit gelap-sedang',
    '👸🏿' => 'putri: warna kulit gelap',
    '👳🏻' => 'orang mengenakan serban: warna kulit cerah',
    '👳🏼' => 'orang mengenakan serban: warna kulit cerah-sedang',
    '👳🏽' => 'orang mengenakan serban: warna kulit sedang',
    '👳🏾' => 'orang mengenakan serban: warna kulit gelap-sedang',
    '👳🏿' => 'orang mengenakan serban: warna kulit gelap',
    '👲🏻' => 'orang dengan topi Tionghoa: warna kulit cerah',
    '👲🏼' => 'orang dengan topi Tionghoa: warna kulit cerah-sedang',
    '👲🏽' => 'orang dengan topi Tionghoa: warna kulit sedang',
    '👲🏾' => 'orang dengan topi Tionghoa: warna kulit gelap-sedang',
    '👲🏿' => 'orang dengan topi Tionghoa: warna kulit gelap',
    '🧕🏻' => 'wanita dengan kerudung: warna kulit cerah',
    '🧕🏼' => 'wanita dengan kerudung: warna kulit cerah-sedang',
    '🧕🏽' => 'wanita dengan kerudung: warna kulit sedang',
    '🧕🏾' => 'wanita dengan kerudung: warna kulit gelap-sedang',
    '🧕🏿' => 'wanita dengan kerudung: warna kulit gelap',
    '🤵🏻' => 'orang bertuksedo: warna kulit cerah',
    '🤵🏼' => 'orang bertuksedo: warna kulit cerah-sedang',
    '🤵🏽' => 'orang bertuksedo: warna kulit sedang',
    '🤵🏾' => 'orang bertuksedo: warna kulit gelap-sedang',
    '🤵🏿' => 'orang bertuksedo: warna kulit gelap',
    '👰🏻' => 'orang berpenutup kepala: warna kulit cerah',
    '👰🏼' => 'orang berpenutup kepala: warna kulit cerah-sedang',
    '👰🏽' => 'orang berpenutup kepala: warna kulit sedang',
    '👰🏾' => 'orang berpenutup kepala: warna kulit gelap-sedang',
    '👰🏿' => 'orang berpenutup kepala: warna kulit gelap',
    '🤰🏻' => 'wanita hamil: warna kulit cerah',
    '🤰🏼' => 'wanita hamil: warna kulit cerah-sedang',
    '🤰🏽' => 'wanita hamil: warna kulit sedang',
    '🤰🏾' => 'wanita hamil: warna kulit gelap-sedang',
    '🤰🏿' => 'wanita hamil: warna kulit gelap',
    '🫃🏻' => 'pria hamil: warna kulit cerah',
    '🫃🏼' => 'pria hamil: warna kulit cerah-sedang',
    '🫃🏽' => 'pria hamil: warna kulit sedang',
    '🫃🏾' => 'pria hamil: warna kulit gelap-sedang',
    '🫃🏿' => 'pria hamil: warna kulit gelap',
    '🫄🏻' => 'orang hamil: warna kulit cerah',
    '🫄🏼' => 'orang hamil: warna kulit cerah-sedang',
    '🫄🏽' => 'orang hamil: warna kulit sedang',
    '🫄🏾' => 'orang hamil: warna kulit gelap-sedang',
    '🫄🏿' => 'orang hamil: warna kulit gelap',
    '🤱🏻' => 'menyusui: warna kulit cerah',
    '🤱🏼' => 'menyusui: warna kulit cerah-sedang',
    '🤱🏽' => 'menyusui: warna kulit sedang',
    '🤱🏾' => 'menyusui: warna kulit gelap-sedang',
    '🤱🏿' => 'menyusui: warna kulit gelap',
    '👼🏻' => 'malaikat bayi: warna kulit cerah',
    '👼🏼' => 'malaikat bayi: warna kulit cerah-sedang',
    '👼🏽' => 'malaikat bayi: warna kulit sedang',
    '👼🏾' => 'malaikat bayi: warna kulit gelap-sedang',
    '👼🏿' => 'malaikat bayi: warna kulit gelap',
    '🎅🏻' => 'Sinterklas: warna kulit cerah',
    '🎅🏼' => 'Sinterklas: warna kulit cerah-sedang',
    '🎅🏽' => 'Sinterklas: warna kulit sedang',
    '🎅🏾' => 'Sinterklas: warna kulit gelap-sedang',
    '🎅🏿' => 'Sinterklas: warna kulit gelap',
    '🤶🏻' => 'Nyonya Sinterklas: warna kulit cerah',
    '🤶🏼' => 'Nyonya Sinterklas: warna kulit cerah-sedang',
    '🤶🏽' => 'Nyonya Sinterklas: warna kulit sedang',
    '🤶🏾' => 'Nyonya Sinterklas: warna kulit gelap-sedang',
    '🤶🏿' => 'Nyonya Sinterklas: warna kulit gelap',
    '🦸🏻' => 'pahlawan super: warna kulit cerah',
    '🦸🏼' => 'pahlawan super: warna kulit cerah-sedang',
    '🦸🏽' => 'pahlawan super: warna kulit sedang',
    '🦸🏾' => 'pahlawan super: warna kulit gelap-sedang',
    '🦸🏿' => 'pahlawan super: warna kulit gelap',
    '🦹🏻' => 'penjahat super: warna kulit cerah',
    '🦹🏼' => 'penjahat super: warna kulit cerah-sedang',
    '🦹🏽' => 'penjahat super: warna kulit sedang',
    '🦹🏾' => 'penjahat super: warna kulit gelap-sedang',
    '🦹🏿' => 'penjahat super: warna kulit gelap',
    '🧙🏻' => 'magi: warna kulit cerah',
    '🧙🏼' => 'magi: warna kulit cerah-sedang',
    '🧙🏽' => 'magi: warna kulit sedang',
    '🧙🏾' => 'magi: warna kulit gelap-sedang',
    '🧙🏿' => 'magi: warna kulit gelap',
    '🧚🏻' => 'peri: warna kulit cerah',
    '🧚🏼' => 'peri: warna kulit cerah-sedang',
    '🧚🏽' => 'peri: warna kulit sedang',
    '🧚🏾' => 'peri: warna kulit gelap-sedang',
    '🧚🏿' => 'peri: warna kulit gelap',
    '🧛🏻' => 'vampir: warna kulit cerah',
    '🧛🏼' => 'vampir: warna kulit cerah-sedang',
    '🧛🏽' => 'vampir: warna kulit sedang',
    '🧛🏾' => 'vampir: warna kulit gelap-sedang',
    '🧛🏿' => 'vampir: warna kulit gelap',
    '🧜🏻' => 'orang duyung: warna kulit cerah',
    '🧜🏼' => 'orang duyung: warna kulit cerah-sedang',
    '🧜🏽' => 'orang duyung: warna kulit sedang',
    '🧜🏾' => 'orang duyung: warna kulit gelap-sedang',
    '🧜🏿' => 'orang duyung: warna kulit gelap',
    '🧝🏻' => 'kurcaci: warna kulit cerah',
    '🧝🏼' => 'kurcaci: warna kulit cerah-sedang',
    '🧝🏽' => 'kurcaci: warna kulit sedang',
    '🧝🏾' => 'kurcaci: warna kulit gelap-sedang',
    '🧝🏿' => 'kurcaci: warna kulit gelap',
    '💆🏻' => 'pria dipijat: warna kulit cerah',
    '💆🏼' => 'pria dipijat: warna kulit cerah-sedang',
    '💆🏽' => 'pria dipijat: warna kulit sedang',
    '💆🏾' => 'pria dipijat: warna kulit gelap-sedang',
    '💆🏿' => 'pria dipijat: warna kulit gelap',
    '💇🏻' => 'orang potong rambut: warna kulit cerah',
    '💇🏼' => 'orang potong rambut: warna kulit cerah-sedang',
    '💇🏽' => 'orang potong rambut: warna kulit sedang',
    '💇🏾' => 'orang potong rambut: warna kulit gelap-sedang',
    '💇🏿' => 'orang potong rambut: warna kulit gelap',
    '🚶🏻' => 'orang berjalan kaki: warna kulit cerah',
    '🚶🏼' => 'orang berjalan kaki: warna kulit cerah-sedang',
    '🚶🏽' => 'orang berjalan kaki: warna kulit sedang',
    '🚶🏾' => 'orang berjalan kaki: warna kulit gelap-sedang',
    '🚶🏿' => 'orang berjalan kaki: warna kulit gelap',
    '🧍🏻' => 'orang berdiri: warna kulit cerah',
    '🧍🏼' => 'orang berdiri: warna kulit cerah-sedang',
    '🧍🏽' => 'orang berdiri: warna kulit sedang',
    '🧍🏾' => 'orang berdiri: warna kulit gelap-sedang',
    '🧍🏿' => 'orang berdiri: warna kulit gelap',
    '🧎🏻' => 'orang berlutut: warna kulit cerah',
    '🧎🏼' => 'orang berlutut: warna kulit cerah-sedang',
    '🧎🏽' => 'orang berlutut: warna kulit sedang',
    '🧎🏾' => 'orang berlutut: warna kulit gelap-sedang',
    '🧎🏿' => 'orang berlutut: warna kulit gelap',
    '🏃🏻' => 'orang berlari: warna kulit cerah',
    '🏃🏼' => 'orang berlari: warna kulit cerah-sedang',
    '🏃🏽' => 'orang berlari: warna kulit sedang',
    '🏃🏾' => 'orang berlari: warna kulit gelap-sedang',
    '🏃🏿' => 'orang berlari: warna kulit gelap',
    '💃🏻' => 'wanita menari: warna kulit cerah',
    '💃🏼' => 'wanita menari: warna kulit cerah-sedang',
    '💃🏽' => 'wanita menari: warna kulit sedang',
    '💃🏾' => 'wanita menari: warna kulit gelap-sedang',
    '💃🏿' => 'wanita menari: warna kulit gelap',
    '🕺🏻' => 'pria menari: warna kulit cerah',
    '🕺🏼' => 'pria menari: warna kulit cerah-sedang',
    '🕺🏽' => 'pria menari: warna kulit sedang',
    '🕺🏾' => 'pria menari: warna kulit gelap-sedang',
    '🕺🏿' => 'pria menari: warna kulit gelap',
    '🕴🏻' => 'pria melayang mengenakan jas: warna kulit cerah',
    '🕴🏼' => 'pria melayang mengenakan jas: warna kulit cerah-sedang',
    '🕴🏽' => 'pria melayang mengenakan jas: warna kulit sedang',
    '🕴🏾' => 'pria melayang mengenakan jas: warna kulit gelap-sedang',
    '🕴🏿' => 'pria melayang mengenakan jas: warna kulit gelap',
    '👯🏻' => 'orang dengan telinga kelinci: warna kulit cerah',
    '👯🏼' => 'orang dengan telinga kelinci: warna kulit cerah-sedang',
    '👯🏽' => 'orang dengan telinga kelinci: warna kulit sedang',
    '👯🏾' => 'orang dengan telinga kelinci: warna kulit gelap-sedang',
    '👯🏿' => 'orang dengan telinga kelinci: warna kulit gelap',
    '🧖🏻' => 'orang di ruangan beruap: warna kulit cerah',
    '🧖🏼' => 'orang di ruangan beruap: warna kulit cerah-sedang',
    '🧖🏽' => 'orang di ruangan beruap: warna kulit sedang',
    '🧖🏾' => 'orang di ruangan beruap: warna kulit gelap-sedang',
    '🧖🏿' => 'orang di ruangan beruap: warna kulit gelap',
    '🧗🏻' => 'orang memanjat: warna kulit cerah',
    '🧗🏼' => 'orang memanjat: warna kulit cerah-sedang',
    '🧗🏽' => 'orang memanjat: warna kulit sedang',
    '🧗🏾' => 'orang memanjat: warna kulit gelap-sedang',
    '🧗🏿' => 'orang memanjat: warna kulit gelap',
    '🏇🏻' => 'balap kuda: warna kulit cerah',
    '🏇🏼' => 'balap kuda: warna kulit cerah-sedang',
    '🏇🏽' => 'balap kuda: warna kulit sedang',
    '🏇🏾' => 'balap kuda: warna kulit gelap-sedang',
    '🏇🏿' => 'balap kuda: warna kulit gelap',
    '🏂🏻' => 'peselancar salju: warna kulit cerah',
    '🏂🏼' => 'peselancar salju: warna kulit cerah-sedang',
    '🏂🏽' => 'peselancar salju: warna kulit sedang',
    '🏂🏾' => 'peselancar salju: warna kulit gelap-sedang',
    '🏂🏿' => 'peselancar salju: warna kulit gelap',
    '🏌🏻' => 'orang bermain golf: warna kulit cerah',
    '🏌🏼' => 'orang bermain golf: warna kulit cerah-sedang',
    '🏌🏽' => 'orang bermain golf: warna kulit sedang',
    '🏌🏾' => 'orang bermain golf: warna kulit gelap-sedang',
    '🏌🏿' => 'orang bermain golf: warna kulit gelap',
    '🏄🏻' => 'orang berselancar: warna kulit cerah',
    '🏄🏼' => 'orang berselancar: warna kulit cerah-sedang',
    '🏄🏽' => 'orang berselancar: warna kulit sedang',
    '🏄🏾' => 'orang berselancar: warna kulit gelap-sedang',
    '🏄🏿' => 'orang berselancar: warna kulit gelap',
    '🚣🏻' => 'orang mendayung perahu: warna kulit cerah',
    '🚣🏼' => 'orang mendayung perahu: warna kulit cerah-sedang',
    '🚣🏽' => 'orang mendayung perahu: warna kulit sedang',
    '🚣🏾' => 'orang mendayung perahu: warna kulit gelap-sedang',
    '🚣🏿' => 'orang mendayung perahu: warna kulit gelap',
    '🏊🏻' => 'orang berenang: warna kulit cerah',
    '🏊🏼' => 'orang berenang: warna kulit cerah-sedang',
    '🏊🏽' => 'orang berenang: warna kulit sedang',
    '🏊🏾' => 'orang berenang: warna kulit gelap-sedang',
    '🏊🏿' => 'orang berenang: warna kulit gelap',
    '⛹🏻' => 'orang memantulkan bola: warna kulit cerah',
    '⛹🏼' => 'orang memantulkan bola: warna kulit cerah-sedang',
    '⛹🏽' => 'orang memantulkan bola: warna kulit sedang',
    '⛹🏾' => 'orang memantulkan bola: warna kulit gelap-sedang',
    '⛹🏿' => 'orang memantulkan bola: warna kulit gelap',
    '🏋🏻' => 'orang mengangkat beban: warna kulit cerah',
    '🏋🏼' => 'orang mengangkat beban: warna kulit cerah-sedang',
    '🏋🏽' => 'orang mengangkat beban: warna kulit sedang',
    '🏋🏾' => 'orang mengangkat beban: warna kulit gelap-sedang',
    '🏋🏿' => 'orang mengangkat beban: warna kulit gelap',
    '🚴🏻' => 'orang bersepeda: warna kulit cerah',
    '🚴🏼' => 'orang bersepeda: warna kulit cerah-sedang',
    '🚴🏽' => 'orang bersepeda: warna kulit sedang',
    '🚴🏾' => 'orang bersepeda: warna kulit gelap-sedang',
    '🚴🏿' => 'orang bersepeda: warna kulit gelap',
    '🚵🏻' => 'orang bersepeda gunung: warna kulit cerah',
    '🚵🏼' => 'orang bersepeda gunung: warna kulit cerah-sedang',
    '🚵🏽' => 'orang bersepeda gunung: warna kulit sedang',
    '🚵🏾' => 'orang bersepeda gunung: warna kulit gelap-sedang',
    '🚵🏿' => 'orang bersepeda gunung: warna kulit gelap',
    '🤸🏻' => 'orang melakukan gerakan meroda: warna kulit cerah',
    '🤸🏼' => 'orang melakukan gerakan meroda: warna kulit cerah-sedang',
    '🤸🏽' => 'orang melakukan gerakan meroda: warna kulit sedang',
    '🤸🏾' => 'orang melakukan gerakan meroda: warna kulit gelap-sedang',
    '🤸🏿' => 'orang melakukan gerakan meroda: warna kulit gelap',
    '🤼🏻' => 'orang bergulat: warna kulit cerah',
    '🤼🏼' => 'orang bergulat: warna kulit cerah-sedang',
    '🤼🏽' => 'orang bergulat: warna kulit sedang',
    '🤼🏾' => 'orang bergulat: warna kulit gelap-sedang',
    '🤼🏿' => 'orang bergulat: warna kulit gelap',
    '🤽🏻' => 'orang bermain polo air: warna kulit cerah',
    '🤽🏼' => 'orang bermain polo air: warna kulit cerah-sedang',
    '🤽🏽' => 'orang bermain polo air: warna kulit sedang',
    '🤽🏾' => 'orang bermain polo air: warna kulit gelap-sedang',
    '🤽🏿' => 'orang bermain polo air: warna kulit gelap',
    '🤾🏻' => 'orang bermain bola tangan: warna kulit cerah',
    '🤾🏼' => 'orang bermain bola tangan: warna kulit cerah-sedang',
    '🤾🏽' => 'orang bermain bola tangan: warna kulit sedang',
    '🤾🏾' => 'orang bermain bola tangan: warna kulit gelap-sedang',
    '🤾🏿' => 'orang bermain bola tangan: warna kulit gelap',
    '🤹🏻' => 'orang melempar: warna kulit cerah',
    '🤹🏼' => 'orang melempar: warna kulit cerah-sedang',
    '🤹🏽' => 'orang melempar: warna kulit sedang',
    '🤹🏾' => 'orang melempar: warna kulit gelap-sedang',
    '🤹🏿' => 'orang melempar: warna kulit gelap',
    '🧘🏻' => 'orang dalam posisi teratai: warna kulit cerah',
    '🧘🏼' => 'orang dalam posisi teratai: warna kulit cerah-sedang',
    '🧘🏽' => 'orang dalam posisi teratai: warna kulit sedang',
    '🧘🏾' => 'orang dalam posisi teratai: warna kulit gelap-sedang',
    '🧘🏿' => 'orang dalam posisi teratai: warna kulit gelap',
    '🛀🏻' => 'orang di bak mandi: warna kulit cerah',
    '🛀🏼' => 'orang di bak mandi: warna kulit cerah-sedang',
    '🛀🏽' => 'orang di bak mandi: warna kulit sedang',
    '🛀🏾' => 'orang di bak mandi: warna kulit gelap-sedang',
    '🛀🏿' => 'orang di bak mandi: warna kulit gelap',
    '🛌🏻' => 'orang di tempat tidur: warna kulit cerah',
    '🛌🏼' => 'orang di tempat tidur: warna kulit cerah-sedang',
    '🛌🏽' => 'orang di tempat tidur: warna kulit sedang',
    '🛌🏾' => 'orang di tempat tidur: warna kulit gelap-sedang',
    '🛌🏿' => 'orang di tempat tidur: warna kulit gelap',
    '👭🏻' => 'dua wanita bergandengan: warna kulit cerah',
    '👭🏼' => 'dua wanita bergandengan: warna kulit cerah-sedang',
    '👭🏽' => 'dua wanita bergandengan: warna kulit sedang',
    '👭🏾' => 'dua wanita bergandengan: warna kulit gelap-sedang',
    '👭🏿' => 'dua wanita bergandengan: warna kulit gelap',
    '👫🏻' => 'pria dan wanita bergandengan: warna kulit cerah',
    '👫🏼' => 'pria dan wanita bergandengan: warna kulit cerah-sedang',
    '👫🏽' => 'pria dan wanita bergandengan: warna kulit sedang',
    '👫🏾' => 'pria dan wanita bergandengan: warna kulit gelap-sedang',
    '👫🏿' => 'pria dan wanita bergandengan: warna kulit gelap',
    '👬🏻' => 'dua pria bergandengan: warna kulit cerah',
    '👬🏼' => 'dua pria bergandengan: warna kulit cerah-sedang',
    '👬🏽' => 'dua pria bergandengan: warna kulit sedang',
    '👬🏾' => 'dua pria bergandengan: warna kulit gelap-sedang',
    '👬🏿' => 'dua pria bergandengan: warna kulit gelap',
    '💏🏻' => 'berciuman: warna kulit cerah',
    '💏🏼' => 'berciuman: warna kulit cerah-sedang',
    '💏🏽' => 'berciuman: warna kulit sedang',
    '💏🏾' => 'berciuman: warna kulit gelap-sedang',
    '💏🏿' => 'berciuman: warna kulit gelap',
    '💑🏻' => 'pasangan dengan hati: warna kulit cerah',
    '💑🏼' => 'pasangan dengan hati: warna kulit cerah-sedang',
    '💑🏽' => 'pasangan dengan hati: warna kulit sedang',
    '💑🏾' => 'pasangan dengan hati: warna kulit gelap-sedang',
    '💑🏿' => 'pasangan dengan hati: warna kulit gelap',
    '🇦🇨' => 'bendera: Pulau Ascension',
    '🇦🇩' => 'bendera: Andorra',
    '🇦🇪' => 'bendera: Uni Emirat Arab',
    '🇦🇫' => 'bendera: Afganistan',
    '🇦🇬' => 'bendera: Antigua dan Barbuda',
    '🇦🇮' => 'bendera: Anguilla',
    '🇦🇱' => 'bendera: Albania',
    '🇦🇲' => 'bendera: Armenia',
    '🇦🇴' => 'bendera: Angola',
    '🇦🇶' => 'bendera: Antarktika',
    '🇦🇷' => 'bendera: Argentina',
    '🇦🇸' => 'bendera: Samoa Amerika',
    '🇦🇹' => 'bendera: Austria',
    '🇦🇺' => 'bendera: Australia',
    '🇦🇼' => 'bendera: Aruba',
    '🇦🇽' => 'bendera: Kepulauan Aland',
    '🇦🇿' => 'bendera: Azerbaijan',
    '🇧🇦' => 'bendera: Bosnia dan Herzegovina',
    '🇧🇧' => 'bendera: Barbados',
    '🇧🇩' => 'bendera: Bangladesh',
    '🇧🇪' => 'bendera: Belgia',
    '🇧🇫' => 'bendera: Burkina Faso',
    '🇧🇬' => 'bendera: Bulgaria',
    '🇧🇭' => 'bendera: Bahrain',
    '🇧🇮' => 'bendera: Burundi',
    '🇧🇯' => 'bendera: Benin',
    '🇧🇱' => 'bendera: Saint Barthélemy',
    '🇧🇲' => 'bendera: Bermuda',
    '🇧🇳' => 'bendera: Brunei',
    '🇧🇴' => 'bendera: Bolivia',
    '🇧🇶' => 'bendera: Belanda Karibia',
    '🇧🇷' => 'bendera: Brasil',
    '🇧🇸' => 'bendera: Bahama',
    '🇧🇹' => 'bendera: Bhutan',
    '🇧🇻' => 'bendera: Pulau Bouvet',
    '🇧🇼' => 'bendera: Botswana',
    '🇧🇾' => 'bendera: Belarus',
    '🇧🇿' => 'bendera: Belize',
    '🇨🇦' => 'bendera: Kanada',
    '🇨🇨' => 'bendera: Kepulauan Cocos (Keeling)',
    '🇨🇩' => 'bendera: Kongo - Kinshasa',
    '🇨🇫' => 'bendera: Republik Afrika Tengah',
    '🇨🇬' => 'bendera: Kongo - Brazzaville',
    '🇨🇭' => 'bendera: Swiss',
    '🇨🇮' => 'bendera: Côte d’Ivoire',
    '🇨🇰' => 'bendera: Kepulauan Cook',
    '🇨🇱' => 'bendera: Cile',
    '🇨🇲' => 'bendera: Kamerun',
    '🇨🇳' => 'bendera: Tiongkok',
    '🇨🇴' => 'bendera: Kolombia',
    '🇨🇵' => 'bendera: Pulau Clipperton',
    '🇨🇶' => 'bendera: Sark',
    '🇨🇷' => 'bendera: Kosta Rika',
    '🇨🇺' => 'bendera: Kuba',
    '🇨🇻' => 'bendera: Tanjung Verde',
    '🇨🇼' => 'bendera: Curaçao',
    '🇨🇽' => 'bendera: Pulau Natal',
    '🇨🇾' => 'bendera: Siprus',
    '🇨🇿' => 'bendera: Ceko',
    '🇩🇪' => 'bendera: Jerman',
    '🇩🇬' => 'bendera: Diego Garcia',
    '🇩🇯' => 'bendera: Jibuti',
    '🇩🇰' => 'bendera: Denmark',
    '🇩🇲' => 'bendera: Dominika',
    '🇩🇴' => 'bendera: Republik Dominika',
    '🇩🇿' => 'bendera: Aljazair',
    '🇪🇦' => 'bendera: Ceuta dan Melilla',
    '🇪🇨' => 'bendera: Ekuador',
    '🇪🇪' => 'bendera: Estonia',
    '🇪🇬' => 'bendera: Mesir',
    '🇪🇭' => 'bendera: Sahara Barat',
    '🇪🇷' => 'bendera: Eritrea',
    '🇪🇸' => 'bendera: Spanyol',
    '🇪🇹' => 'bendera: Etiopia',
    '🇪🇺' => 'bendera: Uni Eropa',
    '🇫🇮' => 'bendera: Finlandia',
    '🇫🇯' => 'bendera: Fiji',
    '🇫🇰' => 'bendera: Kepulauan Falkland',
    '🇫🇲' => 'bendera: Mikronesia',
    '🇫🇴' => 'bendera: Kepulauan Faroe',
    '🇫🇷' => 'bendera: Prancis',
    '🇬🇦' => 'bendera: Gabon',
    '🇬🇧' => 'bendera: Inggris Raya',
    '🇬🇩' => 'bendera: Grenada',
    '🇬🇪' => 'bendera: Georgia',
    '🇬🇫' => 'bendera: Guyana Prancis',
    '🇬🇬' => 'bendera: Guernsey',
    '🇬🇭' => 'bendera: Ghana',
    '🇬🇮' => 'bendera: Gibraltar',
    '🇬🇱' => 'bendera: Greenland',
    '🇬🇲' => 'bendera: Gambia',
    '🇬🇳' => 'bendera: Guinea',
    '🇬🇵' => 'bendera: Guadeloupe',
    '🇬🇶' => 'bendera: Guinea Ekuatorial',
    '🇬🇷' => 'bendera: Yunani',
    '🇬🇸' => 'bendera: Georgia Selatan & Kep. Sandwich Selatan',
    '🇬🇹' => 'bendera: Guatemala',
    '🇬🇺' => 'bendera: Guam',
    '🇬🇼' => 'bendera: Guinea-Bissau',
    '🇬🇾' => 'bendera: Guyana',
    '🇭🇰' => 'bendera: Hong Kong DAK Tiongkok',
    '🇭🇲' => 'bendera: Pulau Heard dan Kepulauan McDonald',
    '🇭🇳' => 'bendera: Honduras',
    '🇭🇷' => 'bendera: Kroasia',
    '🇭🇹' => 'bendera: Haiti',
    '🇭🇺' => 'bendera: Hungaria',
    '🇮🇨' => 'bendera: Kepulauan Canaria',
    '🇮🇩' => 'bendera: Indonesia',
    '🇮🇪' => 'bendera: Irlandia',
    '🇮🇱' => 'bendera: Israel',
    '🇮🇲' => 'bendera: Pulau Man',
    '🇮🇳' => 'bendera: India',
    '🇮🇴' => 'bendera: Wilayah Inggris di Samudra Hindia',
    '🇮🇶' => 'bendera: Irak',
    '🇮🇷' => 'bendera: Iran',
    '🇮🇸' => 'bendera: Islandia',
    '🇮🇹' => 'bendera: Italia',
    '🇯🇪' => 'bendera: Jersey',
    '🇯🇲' => 'bendera: Jamaika',
    '🇯🇴' => 'bendera: Yordania',
    '🇯🇵' => 'bendera: Jepang',
    '🇰🇪' => 'bendera: Kenya',
    '🇰🇬' => 'bendera: Kirgizstan',
    '🇰🇭' => 'bendera: Kamboja',
    '🇰🇮' => 'bendera: Kiribati',
    '🇰🇲' => 'bendera: Komoro',
    '🇰🇳' => 'bendera: Saint Kitts dan Nevis',
    '🇰🇵' => 'bendera: Korea Utara',
    '🇰🇷' => 'bendera: Korea Selatan',
    '🇰🇼' => 'bendera: Kuwait',
    '🇰🇾' => 'bendera: Kepulauan Cayman',
    '🇰🇿' => 'bendera: Kazakhstan',
    '🇱🇦' => 'bendera: Laos',
    '🇱🇧' => 'bendera: Lebanon',
    '🇱🇨' => 'bendera: Saint Lucia',
    '🇱🇮' => 'bendera: Liechtenstein',
    '🇱🇰' => 'bendera: Sri Lanka',
    '🇱🇷' => 'bendera: Liberia',
    '🇱🇸' => 'bendera: Lesotho',
    '🇱🇹' => 'bendera: Lituania',
    '🇱🇺' => 'bendera: Luksemburg',
    '🇱🇻' => 'bendera: Latvia',
    '🇱🇾' => 'bendera: Libya',
    '🇲🇦' => 'bendera: Maroko',
    '🇲🇨' => 'bendera: Monako',
    '🇲🇩' => 'bendera: Moldova',
    '🇲🇪' => 'bendera: Montenegro',
    '🇲🇫' => 'bendera: Saint Martin',
    '🇲🇬' => 'bendera: Madagaskar',
    '🇲🇭' => 'bendera: Kepulauan Marshall',
    '🇲🇰' => 'bendera: Makedonia Utara',
    '🇲🇱' => 'bendera: Mali',
    '🇲🇲' => 'bendera: Myanmar (Burma)',
    '🇲🇳' => 'bendera: Mongolia',
    '🇲🇴' => 'bendera: Makau DAK Tiongkok',
    '🇲🇵' => 'bendera: Kepulauan Mariana Utara',
    '🇲🇶' => 'bendera: Martinik',
    '🇲🇷' => 'bendera: Mauritania',
    '🇲🇸' => 'bendera: Montserrat',
    '🇲🇹' => 'bendera: Malta',
    '🇲🇺' => 'bendera: Mauritius',
    '🇲🇻' => 'bendera: Maladewa',
    '🇲🇼' => 'bendera: Malawi',
    '🇲🇽' => 'bendera: Meksiko',
    '🇲🇾' => 'bendera: Malaysia',
    '🇲🇿' => 'bendera: Mozambik',
    '🇳🇦' => 'bendera: Namibia',
    '🇳🇨' => 'bendera: Kaledonia Baru',
    '🇳🇪' => 'bendera: Niger',
    '🇳🇫' => 'bendera: Kepulauan Norfolk',
    '🇳🇬' => 'bendera: Nigeria',
    '🇳🇮' => 'bendera: Nikaragua',
    '🇳🇱' => 'bendera: Belanda',
    '🇳🇴' => 'bendera: Norwegia',
    '🇳🇵' => 'bendera: Nepal',
    '🇳🇷' => 'bendera: Nauru',
    '🇳🇺' => 'bendera: Niue',
    '🇳🇿' => 'bendera: Selandia Baru',
    '🇴🇲' => 'bendera: Oman',
    '🇵🇦' => 'bendera: Panama',
    '🇵🇪' => 'bendera: Peru',
    '🇵🇫' => 'bendera: Polinesia Prancis',
    '🇵🇬' => 'bendera: Papua Nugini',
    '🇵🇭' => 'bendera: Filipina',
    '🇵🇰' => 'bendera: Pakistan',
    '🇵🇱' => 'bendera: Polandia',
    '🇵🇲' => 'bendera: Saint Pierre dan Miquelon',
    '🇵🇳' => 'bendera: Kepulauan Pitcairn',
    '🇵🇷' => 'bendera: Puerto Riko',
    '🇵🇸' => 'bendera: Wilayah Palestina',
    '🇵🇹' => 'bendera: Portugal',
    '🇵🇼' => 'bendera: Palau',
    '🇵🇾' => 'bendera: Paraguay',
    '🇶🇦' => 'bendera: Qatar',
    '🇷🇪' => 'bendera: Réunion',
    '🇷🇴' => 'bendera: Rumania',
    '🇷🇸' => 'bendera: Serbia',
    '🇷🇺' => 'bendera: Rusia',
    '🇷🇼' => 'bendera: Rwanda',
    '🇸🇦' => 'bendera: Arab Saudi',
    '🇸🇧' => 'bendera: Kepulauan Solomon',
    '🇸🇨' => 'bendera: Seychelles',
    '🇸🇩' => 'bendera: Sudan',
    '🇸🇪' => 'bendera: Swedia',
    '🇸🇬' => 'bendera: Singapura',
    '🇸🇭' => 'bendera: Saint Helena',
    '🇸🇮' => 'bendera: Slovenia',
    '🇸🇯' => 'bendera: Kepulauan Svalbard dan Jan Mayen',
    '🇸🇰' => 'bendera: Slovakia',
    '🇸🇱' => 'bendera: Sierra Leone',
    '🇸🇲' => 'bendera: San Marino',
    '🇸🇳' => 'bendera: Senegal',
    '🇸🇴' => 'bendera: Somalia',
    '🇸🇷' => 'bendera: Suriname',
    '🇸🇸' => 'bendera: Sudan Selatan',
    '🇸🇹' => 'bendera: Sao Tome dan Principe',
    '🇸🇻' => 'bendera: El Salvador',
    '🇸🇽' => 'bendera: Sint Maarten',
    '🇸🇾' => 'bendera: Suriah',
    '🇸🇿' => 'bendera: eSwatini',
    '🇹🇦' => 'bendera: Tristan da Cunha',
    '🇹🇨' => 'bendera: Kepulauan Turks dan Caicos',
    '🇹🇩' => 'bendera: Chad',
    '🇹🇫' => 'bendera: Wilayah Selatan Prancis',
    '🇹🇬' => 'bendera: Togo',
    '🇹🇭' => 'bendera: Thailand',
    '🇹🇯' => 'bendera: Tajikistan',
    '🇹🇰' => 'bendera: Tokelau',
    '🇹🇱' => 'bendera: Timor Leste',
    '🇹🇲' => 'bendera: Turkmenistan',
    '🇹🇳' => 'bendera: Tunisia',
    '🇹🇴' => 'bendera: Tonga',
    '🇹🇷' => 'bendera: Turki',
    '🇹🇹' => 'bendera: Trinidad dan Tobago',
    '🇹🇻' => 'bendera: Tuvalu',
    '🇹🇼' => 'bendera: Taiwan',
    '🇹🇿' => 'bendera: Tanzania',
    '🇺🇦' => 'bendera: Ukraina',
    '🇺🇬' => 'bendera: Uganda',
    '🇺🇲' => 'bendera: Kepulauan Terluar AS',
    '🇺🇳' => 'bendera: Perserikatan Bangsa-Bangsa',
    '🇺🇸' => 'bendera: Amerika Serikat',
    '🇺🇾' => 'bendera: Uruguay',
    '🇺🇿' => 'bendera: Uzbekistan',
    '🇻🇦' => 'bendera: Vatikan',
    '🇻🇨' => 'bendera: Saint Vincent dan Grenadine',
    '🇻🇪' => 'bendera: Venezuela',
    '🇻🇬' => 'bendera: Kepulauan Virgin Britania Raya',
    '🇻🇮' => 'bendera: Kepulauan Virgin Amerika Serikat',
    '🇻🇳' => 'bendera: Vietnam',
    '🇻🇺' => 'bendera: Vanuatu',
    '🇼🇫' => 'bendera: Kepulauan Wallis dan Futuna',
    '🇼🇸' => 'bendera: Samoa',
    '🇽🇰' => 'bendera: Kosovo',
    '🇾🇪' => 'bendera: Yaman',
    '🇾🇹' => 'bendera: Mayotte',
    '🇿🇦' => 'bendera: Afrika Selatan',
    '🇿🇲' => 'bendera: Zambia',
    '🇿🇼' => 'bendera: Zimbabwe',
    '🏻' => 'warna kulit cerah',
    '🏼' => 'warna kulit cerah-sedang',
    '🏽' => 'warna kulit sedang',
    '🏾' => 'warna kulit gelap-sedang',
    '🏿' => 'warna kulit gelap',
    '😀' => 'wajah gembira',
    '😃' => 'wajah tersenyum lebar',
    '😄' => 'wajah tersenyum lebar dan mata tertutup',
    '😁' => 'wajah gembira dengan mata bahagia',
    '😆' => 'wajah tersenyum lebar dan memejamkan mata',
    '😅' => 'wajah tersenyum lebar dan berkeringat',
    '🤣' => 'tertawa terbahak-bahak',
    '😂' => 'wajah gembira berurai air mata',
    '🙂' => 'wajah agak tersenyum',
    '🙃' => 'wajah terbalik',
    '🫠' => 'wajah meleleh',
    '😉' => 'wajah berkedip',
    '😊' => 'wajah tersenyum dengan mata bahagia',
    '😇' => 'wajah malaikat',
    '🥰' => 'wajah tersenyum dengan 3 hati',
    '😍' => 'wajah tersenyum lebar bermata hati',
    '🤩' => 'wajah dengan mata berbentuk bintang',
    '😘' => 'wajah memberikan ciuman jauh',
    '😗' => 'wajah memberikan ciuman',
    '☺' => 'wajah tersenyum',
    '😚' => 'wajah memberikan ciuman dengan mata tertutup',
    '😙' => 'wajah memberikan ciuman dengan mata bahagia',
    '🥲' => 'wajah tersenyum dengan air mata',
    '😋' => 'wajah menikmati',
    '😛' => 'wajah menjulurkan lidah',
    '😜' => 'wajah menjulurkan lidah dan mengedipkan mata',
    '🤪' => 'wajah konyol',
    '😝' => 'wajah menjulurkan lidah dan memejamkan mata',
    '🤑' => 'wajah materialistis',
    '🤗' => 'wajah memeluk',
    '🤭' => 'wajah dengan tangan menutup mulut',
    '🫢' => 'wajah dengan mata terbuka dan tangan menutupi mulut',
    '🫣' => 'wajah dengan mata mengintip',
    '🤫' => 'wajah menyuruh diam',
    '🤔' => 'wajah berpikir',
    '🫡' => 'wajah memberi hormat',
    '🤐' => 'wajah dengan mulut tertutup rapat',
    '🤨' => 'wajah dengan alis terangkat',
    '😐' => 'wajah netral',
    '😑' => 'wajah tanpa ekspresi',
    '😶' => 'wajah tanpa mulut',
    '🫥' => 'wajah dengan garis titik-titik',
    '😏' => 'wajah tersenyum sombong',
    '😒' => 'wajah bosan',
    '🙄' => 'wajah dengan mata ke atas',
    '😬' => 'wajah menyeringai',
    '🤥' => 'bohong',
    '🫨' => 'wajah gemetar',
    '😌' => 'wajah lega',
    '😔' => 'wajah merenung',
    '😪' => 'wajah mengantuk',
    '🤤' => 'ngiler',
    '😴' => 'wajah tidur',
    '🫩' => 'wajah dengan kantung mata',
    '😷' => 'wajah bermasker',
    '🤒' => 'wajah bertermometer',
    '🤕' => 'wajah diperban di kepala',
    '🤢' => 'mual',
    '🤮' => 'wajah muntah',
    '🤧' => 'bersin',
    '🥵' => 'wajah kepanasan',
    '🥶' => 'wajah kedinginan',
    '🥴' => 'wajah pusing karena mabuk',
    '😵' => 'wajah pusing',
    '🤯' => 'kepala meledak',
    '🤠' => 'wajah koboi',
    '🥳' => 'wajah pesta',
    '🥸' => 'wajah menyamar',
    '😎' => 'wajah berkacamata hitam',
    '🤓' => 'wajah kutu buku',
    '🧐' => 'wajah dengan monokel',
    '😕' => 'wajah bingung',
    '🫤' => 'wajah dengan mulut diagonal',
    '😟' => 'wajah cemas',
    '🙁' => 'wajah agak cemberut',
    '☹' => 'wajah cemberut',
    '😮' => 'wajah dengan mulut terbuka',
    '😯' => 'wajah hening',
    '😲' => 'wajah terpukau',
    '😳' => 'wajah tersipu',
    '🫪' => 'wajah syok',
    '🥺' => 'wajah memelas',
    '🥹' => 'wajah menahan air mata',
    '😦' => 'wajah mengerutkan kening dengan mulut terbuka',
    '😧' => 'wajah menderita',
    '😨' => 'wajah ketakutan',
    '😰' => 'wajah ketakutan berkeringat',
    '😥' => 'wajah kecewa namun lega',
    '😢' => 'wajah menangis',
    '😭' => 'wajah menangis keras',
    '😱' => 'wajah sangat ketakutan',
    '😖' => 'wajah stres',
    '😣' => 'wajah tidak setuju',
    '😞' => 'wajah kecewa',
    '😓' => 'wajah berkeringat',
    '😩' => 'wajah terlalu lelah',
    '😫' => 'wajah kelelahan',
    '🥱' => 'wajah menguap',
    '😤' => 'wajah bersungguh-sungguh',
    '😡' => 'wajah sangat marah',
    '😠' => 'wajah marah',
    '🤬' => 'wajah dengan simbol pada mulut',
    '😈' => 'wajah tersenyum bertanduk',
    '👿' => 'wajah marah dengan tanduk',
    '💀' => 'tengkorak',
    '☠' => 'tengkorak dan tulang',
    '💩' => 'kotoran',
    '🤡' => 'badut',
    '👹' => 'wajah monster',
    '👺' => 'topeng hidung panjang',
    '👻' => 'hantu',
    '👽' => 'alien',
    '👾' => 'monster alien',
    '🤖' => 'robot',
    '😺' => 'wajah kucing tersenyum lebar',
    '😸' => 'wajah kucing gembira dengan mata bahagia',
    '😹' => 'wajah kucing gembira berurai air mata',
    '😻' => 'wajah kucing gembira bermata hati',
    '😼' => 'wajah kucing tersenyum sombong',
    '😽' => 'wajah kucing memberikan ciuman dengan mata tertutup',
    '🙀' => 'wajah kucing ketakutan',
    '😿' => 'wajah kucing menangis',
    '😾' => 'wajah kucing marah',
    '🙈' => 'tidak melihat',
    '🙉' => 'tidak mendengar',
    '🙊' => 'tidak bicara',
    '💌' => 'surat cinta',
    '💘' => 'hati terpanah',
    '💝' => 'hati berpita',
    '💖' => 'hati bersinar',
    '💗' => 'hati membesar',
    '💓' => 'hati berdetak',
    '💞' => 'hati berputar',
    '💕' => 'dua hati',
    '💟' => 'dekorasi hati',
    '❣' => 'tanda seru berbentuk hati',
    '💔' => 'patah hati',
    '❤' => 'hati merah',
    '🩷' => 'hati merah muda',
    '🧡' => 'hati oranye',
    '💛' => 'hati kuning',
    '💚' => 'hati hijau',
    '💙' => 'hati biru',
    '🩵' => 'hati biru muda',
    '💜' => 'hati ungu',
    '🤎' => 'hati cokelat',
    '🖤' => 'hati hitam',
    '🩶' => 'hati abu-abu',
    '🤍' => 'hati putih',
    '💋' => 'tanda ciuman',
    '💯' => 'nilai seratus',
    '💢' => 'marah',
    '🫯' => 'awan duel',
    '💥' => 'tabrakan',
    '💫' => 'pusing',
    '💦' => 'keringat',
    '💨' => 'berlari',
    '🕳' => 'lubang',
    '💬' => 'balon ucapan',
    '🗨' => 'gelembung ucapan kiri',
    '🗯' => 'gelembung marah kanan',
    '💭' => 'gelembung pikiran',
    '💤' => 'zzz',
    '👋' => 'tangan melambai',
    '🤚' => 'angkat tangan',
    '🖐' => 'telapak tangan terbuka',
    '✋' => 'telapak tangan terbuka dengan jari rapat',
    '🖖' => 'salam vulcan',
    '🫱' => 'tangan menghadap kanan',
    '🫲' => 'tangan menghadap kiri',
    '🫳' => 'telapak tangan menghadap ke bawah',
    '🫴' => 'telapak tangan menghadap ke atas',
    '🫷' => 'tangan mendorong ke kiri',
    '🫸' => 'tangan mendorong ke kanan',
    '👌' => 'oke',
    '🤌' => 'jari terkuncup',
    '🤏' => 'tangan menjepit',
    '✌' => 'isyarat kemenangan',
    '🤞' => 'jari menyilang',
    '🫰' => 'tangan dengan jari telunjuk dan ibu jari menyilang',
    '🤟' => 'gerakan sayang kamu',
    '🤘' => 'lambang tanduk',
    '🤙' => 'panggil saya',
    '👈' => 'menunjuk ke arah kiri',
    '👉' => 'menunjuk ke arah kanan',
    '👆' => 'menunjuk ke atas (dibalik)',
    '🖕' => 'jari tengah',
    '👇' => 'menunjuk ke bawah',
    '☝' => 'menunjuk ke atas',
    '🫵' => 'jari telunjuk menunjuk penonton',
    '👍' => 'jempol ke atas',
    '👎' => 'jempol ke bawah',
    '✊' => 'tangan terkepal ke atas',
    '👊' => 'tangan terkepal ke depan',
    '🤛' => 'tinju kiri',
    '🤜' => 'tinju kanan',
    '👏' => 'tepuk tangan',
    '🙌' => 'mengangkat tangan',
    '🫶' => 'tangan membentuk hati',
    '👐' => 'tangan terbuka',
    '🤲' => 'tangan diangkat dan dirapatkan bersama',
    '🤝' => 'jabat tangan',
    '🙏' => 'melipat tangan',
    '✍' => 'tangan menulis',
    '💅' => 'poles kuku',
    '🤳' => 'selfie',
    '💪' => 'lengan berotot',
    '🦾' => 'lengan robot',
    '🦿' => 'tungkai robot',
    '🦵' => 'tungkai',
    '🦶' => 'telapak kaki',
    '👂' => 'telinga',
    '🦻' => 'telinga dengan alat bantu dengar',
    '👃' => 'hidung',
    '🧠' => 'otak',
    '🫀' => 'organ jantung',
    '🫁' => 'paru-paru',
    '🦷' => 'gigi',
    '🦴' => 'tulang',
    '👀' => 'dua mata',
    '👁' => 'satu mata',
    '👅' => 'lidah',
    '👄' => 'mulut',
    '🫦' => 'menggigit bibir',
    '👶' => 'bayi',
    '🧒' => 'anak',
    '👦' => 'anak laki-laki',
    '👧' => 'anak perempuan',
    '🧑' => 'dewasa',
    '👱' => 'orang: rambut pirang',
    '👨' => 'pria',
    '🧔' => 'orang: janggut',
    '👩' => 'wanita',
    '🧓' => 'lanjut usia',
    '👴' => 'pria tua',
    '👵' => 'wanita tua',
    '🙍' => 'orang marah',
    '🙎' => 'orang tidak senang',
    '🙅' => 'orang mengisyaratkan TIDAK',
    '🙆' => 'orang mengisyaratkan OKE',
    '💁' => 'orang menengadahkan tangan',
    '🙋' => 'orang mengangkat tangan',
    '🧏' => 'orang tuli',
    '🙇' => 'orang membungkuk',
    '🤦' => 'orang tepuk jidat',
    '🤷' => 'orang mengangkat bahu',
    '👮' => 'polisi',
    '🕵' => 'detektif',
    '💂' => 'penjaga',
    '🥷' => 'ninja',
    '👷' => 'pekerja konstruksi',
    '🫅' => 'orang mengenakan mahkota',
    '🤴' => 'pangeran',
    '👸' => 'putri',
    '👳' => 'orang mengenakan serban',
    '👲' => 'orang dengan topi Tionghoa',
    '🧕' => 'wanita dengan kerudung',
    '🤵' => 'orang bertuksedo',
    '👰' => 'orang berpenutup kepala',
    '🤰' => 'wanita hamil',
    '🫃' => 'pria hamil',
    '🫄' => 'orang hamil',
    '🤱' => 'menyusui',
    '👼' => 'malaikat bayi',
    '🎅' => 'Sinterklas',
    '🤶' => 'Nyonya Sinterklas',
    '🦸' => 'pahlawan super',
    '🦹' => 'penjahat super',
    '🧙' => 'magi',
    '🧚' => 'peri',
    '🧛' => 'vampir',
    '🧜' => 'orang duyung',
    '🧝' => 'kurcaci',
    '🧞' => 'jin',
    '🧟' => 'zombi',
    '🧌' => 'raksasa',
    '🫈' => 'makhluk berbulu',
    '💆' => 'pria dipijat',
    '💇' => 'orang potong rambut',
    '🚶' => 'orang berjalan kaki',
    '🧍' => 'orang berdiri',
    '🧎' => 'orang berlutut',
    '🏃' => 'orang berlari',
    '💃' => 'wanita menari',
    '🕺' => 'pria menari',
    '🕴' => 'pria melayang mengenakan jas',
    '👯' => 'orang dengan telinga kelinci',
    '🧖' => 'orang di ruangan beruap',
    '🧗' => 'orang memanjat',
    '🤺' => 'orang berolahraga anggar',
    '🏇' => 'balap kuda',
    '⛷' => 'pemain ski',
    '🏂' => 'peselancar salju',
    '🏌' => 'orang bermain golf',
    '🏄' => 'orang berselancar',
    '🚣' => 'orang mendayung perahu',
    '🏊' => 'orang berenang',
    '⛹' => 'orang memantulkan bola',
    '🏋' => 'orang mengangkat beban',
    '🚴' => 'orang bersepeda',
    '🚵' => 'orang bersepeda gunung',
    '🤸' => 'orang melakukan gerakan meroda',
    '🤼' => 'orang bergulat',
    '🤽' => 'orang bermain polo air',
    '🤾' => 'orang bermain bola tangan',
    '🤹' => 'orang melempar',
    '🧘' => 'orang dalam posisi teratai',
    '🛀' => 'orang di bak mandi',
    '🛌' => 'orang di tempat tidur',
    '👭' => 'dua wanita bergandengan',
    '👫' => 'pria dan wanita bergandengan',
    '👬' => 'dua pria bergandengan',
    '💏' => 'berciuman',
    '💑' => 'pasangan dengan hati',
    '🗣' => 'kepala berbicara',
    '👤' => 'siluet foto setengah badan',
    '👥' => 'beberapa siluet foto setengah badan',
    '🫂' => 'orang berpelukan',
    '👪' => 'keluarga',
    '👣' => 'jejak kaki',
    '🫆' => 'sidik jari',
    '🦰' => 'rambut merah',
    '🦱' => 'rambut keriting',
    '🦳' => 'rambut putih',
    '🦲' => 'gundul',
    '🐵' => 'wajah monyet',
    '🐒' => 'monyet',
    '🦍' => 'gorila',
    '🦧' => 'orangutan',
    '🐶' => 'wajah anjing',
    '🐕' => 'anjing',
    '🦮' => 'anjing pemandu',
    '🐩' => 'pudel',
    '🐺' => 'serigala',
    '🦊' => 'rubah',
    '🦝' => 'rakun',
    '🐱' => 'wajah kucing',
    '🐈' => 'kucing',
    '🦁' => 'singa',
    '🐯' => 'wajah harimau',
    '🐅' => 'harimau',
    '🐆' => 'macan tutul',
    '🐴' => 'wajah kuda',
    '🫎' => 'rusa besar',
    '🫏' => 'keledai',
    '🐎' => 'kuda',
    '🦄' => 'unicorn',
    '🦓' => 'zebra',
    '🦌' => 'rusa',
    '🦬' => 'bison',
    '🐮' => 'wajah sapi',
    '🐂' => 'lembu',
    '🐃' => 'kerbau',
    '🐄' => 'sapi',
    '🐷' => 'wajah babi',
    '🐖' => 'babi',
    '🐗' => 'babi hutan',
    '🐽' => 'hidung babi',
    '🐏' => 'domba jantan',
    '🐑' => 'domba betina',
    '🐐' => 'kambing',
    '🐪' => 'unta',
    '🐫' => 'unta berpunuk dua',
    '🦙' => 'llama',
    '🦒' => 'jerapah',
    '🐘' => 'gajah',
    '🦣' => 'mamut',
    '🦏' => 'badak',
    '🦛' => 'kuda nil',
    '🐭' => 'wajah tikus',
    '🐁' => 'tikus putih',
    '🐀' => 'tikus',
    '🐹' => 'hamster',
    '🐰' => 'wajah kelinci',
    '🐇' => 'kelinci',
    '🐿' => 'tupai',
    '🦫' => 'biwara',
    '🦔' => 'landak',
    '🦇' => 'kelelawar',
    '🐻' => 'beruang',
    '🐨' => 'koala',
    '🐼' => 'panda',
    '🦥' => 'kungkang',
    '🦦' => 'berang-berang',
    '🦨' => 'sigung',
    '🦘' => 'kanguru',
    '🦡' => 'musang',
    '🐾' => 'jejak kaki hewan',
    '🦃' => 'kalkun',
    '🐔' => 'ayam',
    '🐓' => 'ayam jago',
    '🐣' => 'itik menetas',
    '🐤' => 'itik bayi',
    '🐥' => 'itik bayi menghadap ke depan',
    '🐦' => 'burung',
    '🐧' => 'pinguin',
    '🕊' => 'merpati',
    '🦅' => 'elang',
    '🦆' => 'bebek',
    '🦢' => 'angsa leher panjang',
    '🦉' => 'burung hantu',
    '🦤' => 'dodo',
    '🪶' => 'bulu',
    '🦩' => 'flamingo',
    '🦚' => 'merak',
    '🦜' => 'nuri',
    '🪽' => 'sayap',
    '🪿' => 'angsa leher pendek',
    '🐸' => 'katak',
    '🐊' => 'buaya',
    '🐢' => 'kura-kura',
    '🦎' => 'kadal',
    '🐍' => 'ular',
    '🐲' => 'wajah naga',
    '🐉' => 'naga',
    '🦕' => 'sauropod',
    '🦖' => 'T-Rex',
    '🐳' => 'ikan paus menyembur',
    '🐋' => 'ikan paus',
    '🐬' => 'lumba-lumba',
    '🫍' => 'orca',
    '🦭' => 'anjing laut',
    '🐟' => 'ikan',
    '🐠' => 'ikan tropis',
    '🐡' => 'ikan gembung',
    '🦈' => 'hiu',
    '🐙' => 'gurita',
    '🐚' => 'cangkang spiral',
    '🪸' => 'terumbu karang',
    '🪼' => 'ubur-ubur',
    '🦀' => 'kepiting',
    '🦞' => 'lobster',
    '🦐' => 'udang',
    '🦑' => 'cumi-cumi',
    '🦪' => 'tiram',
    '🐌' => 'siput',
    '🦋' => 'kupu-kupu',
    '🐛' => 'serangga kecil',
    '🐜' => 'semut',
    '🐝' => 'lebah madu',
    '🪲' => 'kumbang',
    '🐞' => 'kumbang betina',
    '🦗' => 'jangkrik',
    '🪳' => 'kecoak',
    '🕷' => 'laba-laba',
    '🕸' => 'jaring laba-laba',
    '🦂' => 'kalajengking',
    '🦟' => 'nyamuk',
    '🪰' => 'lalat',
    '🪱' => 'cacing',
    '🦠' => 'mikroba',
    '💐' => 'karangan bunga',
    '🌸' => 'sakura',
    '💮' => 'bunga putih',
    '🪷' => 'teratai',
    '🏵' => 'hiasan mawar',
    '🌹' => 'mawar',
    '🥀' => 'bunga layu',
    '🌺' => 'kembang sepatu',
    '🌻' => 'bunga matahari',
    '🌼' => 'mekar',
    '🌷' => 'tulip',
    '🪻' => 'hyacinthus',
    '🌱' => 'semai',
    '🪴' => 'tanaman pot',
    '🌲' => 'pohon hijau abadi',
    '🌳' => 'pohon meranggas',
    '🌴' => 'pohon palem',
    '🌵' => 'kaktus',
    '🌾' => 'seikat padi',
    '🌿' => 'herba',
    '☘' => 'semanggi',
    '🍀' => 'empat daun semanggi',
    '🍁' => 'daun maple',
    '🍂' => 'daun berguguran',
    '🍃' => 'daun tertiup angin',
    '🪹' => 'sarang burung kosong',
    '🪺' => 'sarang dengan telur',
    '🍄' => 'jamur',
    '🪾' => 'pohon tanpa daun',
    '🍇' => 'anggur',
    '🍈' => 'melon',
    '🍉' => 'semangka',
    '🍊' => 'buah jeruk',
    '🍋' => 'lemon',
    '🍌' => 'pisang',
    '🍍' => 'nanas',
    '🥭' => 'mangga',
    '🍎' => 'apel merah',
    '🍏' => 'apel hijau',
    '🍐' => 'pir',
    '🍑' => 'persik',
    '🍒' => 'ceri',
    '🍓' => 'stroberi',
    '🫐' => 'blueberry',
    '🥝' => 'buah kiwi',
    '🍅' => 'tomat',
    '🫒' => 'zaitun',
    '🥥' => 'kelapa',
    '🥑' => 'alpukat',
    '🍆' => 'terung',
    '🥔' => 'kentang',
    '🥕' => 'wortel',
    '🌽' => 'bonggol jagung',
    '🌶' => 'cabai pedas',
    '🫑' => 'paprika',
    '🥒' => 'ketimun',
    '🥬' => 'selada hijau',
    '🥦' => 'brokoli',
    '🧄' => 'bawang putih',
    '🧅' => 'bawang bombay',
    '🥜' => 'kacang tanah',
    '🫘' => 'kacang merah',
    '🌰' => 'kastanye',
    '🫚' => 'akar jahe',
    '🫛' => 'kacang polong',
    '🫜' => 'sayuran umbi',
    '🍞' => 'roti',
    '🥐' => 'croissant',
    '🥖' => 'roti baguette',
    '🫓' => 'roti pipih',
    '🥨' => 'pretzel',
    '🥯' => 'bagel',
    '🥞' => 'panekuk',
    '🧇' => 'wafel',
    '🧀' => 'irisan keju',
    '🍖' => 'daging pada tulang',
    '🍗' => 'kaki ayam',
    '🥩' => 'potongan daging',
    '🥓' => 'babi asap',
    '🍔' => 'hamburger',
    '🍟' => 'kentang goreng',
    '🍕' => 'pizza',
    '🌭' => 'hot dog',
    '🥪' => 'roti lapis',
    '🌮' => 'taco',
    '🌯' => 'burrito',
    '🫔' => 'tamale',
    '🥙' => 'roti tawar isi',
    '🧆' => 'falafel',
    '🥚' => 'telur',
    '🍳' => 'memasak',
    '🥘' => 'wajan dangkal',
    '🍲' => 'panci makanan',
    '🫕' => 'fondue',
    '🥣' => 'mangkuk dengan sendok',
    '🥗' => 'salad hijau',
    '🍿' => 'popcorn',
    '🧈' => 'mentega',
    '🧂' => 'garam',
    '🥫' => 'makanan kaleng',
    '🍱' => 'kotak bento',
    '🍘' => 'kerupuk beras',
    '🍙' => 'nasi kepal',
    '🍚' => 'nasi matang',
    '🍛' => 'nasi kari',
    '🍜' => 'mangkuk mengepul',
    '🍝' => 'spageti',
    '🍠' => 'ubi panggang',
    '🍢' => 'oden',
    '🍣' => 'sushi',
    '🍤' => 'udang goreng',
    '🍥' => 'kue ikan dengan desain pusaran',
    '🥮' => 'kue bulan',
    '🍡' => 'dango',
    '🥟' => 'dumpling',
    '🥠' => 'kue keberuntungan',
    '🥡' => 'kotak makanan',
    '🍦' => 'es krim lembut',
    '🍧' => 'es serut',
    '🍨' => 'es krim',
    '🍩' => 'donat',
    '🍪' => 'biskuit',
    '🎂' => 'kue ulang tahun',
    '🍰' => 'kue',
    '🧁' => 'kue mangkuk',
    '🥧' => 'pai',
    '🍫' => 'sebatang cokelat',
    '🍬' => 'permen',
    '🍭' => 'permen lolipop',
    '🍮' => 'puding susu',
    '🍯' => 'toples madu',
    '🍼' => 'botol bayi',
    '🥛' => 'gelas susu',
    '☕' => 'minuman hangat',
    '🫖' => 'teko',
    '🍵' => 'cangkir teh tanpa gagang',
    '🍶' => 'sake',
    '🍾' => 'botol dengan penyumbat terlepas',
    '🍷' => 'gelas anggur',
    '🍸' => 'gelas cocktail',
    '🍹' => 'minuman tropis',
    '🍺' => 'mug bir',
    '🍻' => 'mug bir berdenting',
    '🥂' => 'mendentingkan gelas',
    '🥃' => 'gelas tumbler',
    '🫗' => 'menuang cairan',
    '🥤' => 'cangkir dengan sedotan',
    '🧋' => 'teh bubble',
    '🧃' => 'kotak minuman',
    '🧉' => 'teh mate',
    '🧊' => 'es batu',
    '🥢' => 'sumpit',
    '🍽' => 'garpu dan pisau dengan piring',
    '🍴' => 'garpu dan pisau',
    '🥄' => 'sendok',
    '🔪' => 'pisau dapur',
    '🫙' => 'toples',
    '🏺' => 'amphora',
    '🌍' => 'bola dunia Eropa-Afrika',
    '🌎' => 'bola dunia Amerika',
    '🌏' => 'bola dunia Asia-Australia',
    '🌐' => 'bola dunia dengan garis meridian',
    '🗺' => 'peta dunia',
    '🗾' => 'peta Jepang',
    '🧭' => 'kompas',
    '🏔' => 'gunung dengan puncak bersalju',
    '⛰' => 'gunung',
    '🛘' => 'longsor',
    '🌋' => 'gunung berapi',
    '🗻' => 'gunung Fuji',
    '🏕' => 'berkemah',
    '🏖' => 'pantai dan payung',
    '🏜' => 'gurun',
    '🏝' => 'pulau terpencil',
    '🏞' => 'taman nasional',
    '🏟' => 'stadion',
    '🏛' => 'bangunan klasik',
    '🏗' => 'konstruksi bangunan',
    '🧱' => 'batu bata',
    '🪨' => 'batu',
    '🪵' => 'kayu',
    '🛖' => 'pondok',
    '🏘' => 'perumahan',
    '🏚' => 'rumah terlantar',
    '🏠' => 'rumah',
    '🏡' => 'rumah dengan taman',
    '🏢' => 'gedung perkantoran',
    '🏣' => 'kantor pos Jepang',
    '🏤' => 'kantor pos',
    '🏥' => 'rumah sakit',
    '🏦' => 'bank',
    '🏨' => 'hotel',
    '🏩' => 'hotel cinta',
    '🏪' => 'minimarket',
    '🏫' => 'sekolah',
    '🏬' => 'toserba',
    '🏭' => 'pabrik',
    '🏯' => 'kastel Jepang',
    '🏰' => 'kastel',
    '💒' => 'pernikahan',
    '🗼' => 'menara Tokyo',
    '🗽' => 'Patung Liberty',
    '⛪' => 'gereja',
    '🕌' => 'masjid',
    '🛕' => 'pura',
    '🕍' => 'sinagog',
    '⛩' => 'kuil Shinto',
    '🕋' => 'ka’bah',
    '⛲' => 'air mancur',
    '⛺' => 'tenda',
    '🌁' => 'berkabut',
    '🌃' => 'malam berbintang',
    '🏙' => 'panorama kota',
    '🌄' => 'matahari terbit di antara gunung',
    '🌅' => 'matahari terbit',
    '🌆' => 'panorama kota di sore hari',
    '🌇' => 'matahari terbenam',
    '🌉' => 'jembatan di malam hari',
    '♨' => 'sumber air panas',
    '🎠' => 'komidi putar',
    '🛝' => 'perosotan',
    '🎡' => 'kincir ria',
    '🎢' => 'roller coaster',
    '💈' => 'tanda salon',
    '🎪' => 'tenda sirkus',
    '🚂' => 'lokomotif',
    '🚃' => 'gerbong',
    '🚄' => 'kereta ekspres',
    '🚅' => 'kereta ekspres peluru',
    '🚆' => 'kereta',
    '🚇' => 'kereta bawah tanah',
    '🚈' => 'light rail',
    '🚉' => 'stasiun',
    '🚊' => 'trem',
    '🚝' => 'monorel',
    '🚞' => 'jalur kereta pegunungan',
    '🚋' => 'gerbong trem',
    '🚌' => 'bus',
    '🚍' => 'bus datang',
    '🚎' => 'bus listrik',
    '🚐' => 'minibus',
    '🚑' => 'ambulans',
    '🚒' => 'mobil pemadam kebakaran',
    '🚓' => 'mobil polisi',
    '🚔' => 'mobil polisi datang',
    '🚕' => 'taksi',
    '🚖' => 'taksi datang',
    '🚗' => 'mobil',
    '🚘' => 'mobil datang',
    '🚙' => 'kendaraan rekreasi',
    '🛻' => 'truk pikap',
    '🚚' => 'truk pengiriman',
    '🚛' => 'lori gandeng',
    '🚜' => 'traktor',
    '🏎' => 'mobil balap',
    '🏍' => 'sepeda motor',
    '🛵' => 'motor skuter',
    '🦽' => 'kursi roda manual',
    '🦼' => 'kursi roda bermotor',
    '🛺' => 'bajaj',
    '🚲' => 'sepeda',
    '🛴' => 'otoped',
    '🛹' => 'papan luncur',
    '🛼' => 'sepatu roda',
    '🚏' => 'halte bus',
    '🛣' => 'jalan raya',
    '🛤' => 'rel kereta',
    '🛢' => 'drum minyak',
    '⛽' => 'pompa bahan bakar',
    '🛞' => 'roda',
    '🚨' => 'lampu mobil polisi',
    '🚥' => 'lampu lalu lintas horizontal',
    '🚦' => 'lampu lalu lintas vertikal',
    '🛑' => 'rambu stop',
    '🚧' => 'konstruksi',
    '⚓' => 'jangkar',
    '🛟' => 'ban pelampung',
    '⛵' => 'perahu layar',
    '🛶' => 'kano',
    '🚤' => 'perahu cepat',
    '🛳' => 'kapal penumpang',
    '⛴' => 'kapal feri',
    '🛥' => 'perahu bermotor',
    '🚢' => 'kapal',
    '✈' => 'pesawat',
    '🛩' => 'pesawat kecil',
    '🛫' => 'pesawat lepas landas',
    '🛬' => 'pesawat mendarat',
    '🪂' => 'parasut',
    '💺' => 'kursi',
    '🚁' => 'helikopter',
    '🚟' => 'kereta suspensi',
    '🚠' => 'kereta gantung',
    '🚡' => 'trem gantung',
    '🛰' => 'satelit',
    '🚀' => 'roket',
    '🛸' => 'piring terbang',
    '🛎' => 'bel meja',
    '🧳' => 'koper',
    '⌛' => 'jam pasir',
    '⏳' => 'jam pasir yang bergerak',
    '⌚' => 'jam tangan',
    '⏰' => 'jam weker',
    '⏱' => 'stopwatch',
    '⏲' => 'penghitung waktu',
    '🕰' => 'jam perapian',
    '🕛' => 'jam dua belas tepat',
    '🕧' => 'jam setengah satu',
    '🕐' => 'jam satu tepat',
    '🕜' => 'jam setengah dua',
    '🕑' => 'jam dua tepat',
    '🕝' => 'jam setengah tiga',
    '🕒' => 'jam tiga tepat',
    '🕞' => 'jam setengah empat',
    '🕓' => 'jam empat tepat',
    '🕟' => 'jam setengah lima',
    '🕔' => 'jam lima tepat',
    '🕠' => 'jam setengah enam',
    '🕕' => 'jam enam tepat',
    '🕡' => 'jam setengah tujuh',
    '🕖' => 'jam tujuh tepat',
    '🕢' => 'jam setengah delapan',
    '🕗' => 'jam delapan tepat',
    '🕣' => 'jam setengah sembilan',
    '🕘' => 'jam sembilan tepat',
    '🕤' => 'jam setengah sepuluh',
    '🕙' => 'jam sepuluh tepat',
    '🕥' => 'jam setengah sebelas',
    '🕚' => 'jam sebelas tepat',
    '🕦' => 'jam setengah dua belas',
    '🌑' => 'bulan baru',
    '🌒' => 'bulan sabit awal',
    '🌓' => 'bulan perbani awal',
    '🌔' => 'bulan cembung awal',
    '🌕' => 'bulan purnama',
    '🌖' => 'bulan cembung akhir',
    '🌗' => 'bulan perbani akhir',
    '🌘' => 'bulan sabit akhir',
    '🌙' => 'bulan sabit',
    '🌚' => 'wajah bulan baru',
    '🌛' => 'wajah bulan perbani awal',
    '🌜' => 'wajah bulan perbani akhir',
    '🌡' => 'termometer',
    '☀' => 'matahari',
    '🌝' => 'wajah bulan purnama',
    '🌞' => 'wajah matahari',
    '🪐' => 'planet bercincin',
    '⭐' => 'bintang medium putih',
    '🌟' => 'bintang bersinar',
    '🌠' => 'bintang jatuh',
    '🌌' => 'bima sakti',
    '☁' => 'awan',
    '⛅' => 'matahari di balik awan',
    '⛈' => 'awan dengan petir dan hujan',
    '🌤' => 'matahari di balik awan kecil',
    '🌥' => 'matahari di balik awan besar',
    '🌦' => 'matahari di balik awan hujan',
    '🌧' => 'awan hujan',
    '🌨' => 'awan salju',
    '🌩' => 'awan petir',
    '🌪' => 'tornado',
    '🌫' => 'kabut',
    '🌬' => 'wajah tertiup angin',
    '🌀' => 'topan',
    '🌈' => 'pelangi',
    '🌂' => 'payung tertutup',
    '☂' => 'payung',
    '☔' => 'payung dengan tetesan hujan',
    '⛱' => 'payung di tanah',
    '⚡' => 'tegangan tinggi',
    '❄' => 'serpihan salju',
    '☃' => 'boneka salju dengan hujan salju',
    '⛄' => 'boneka salju tanpa salju',
    '☄' => 'komet',
    '🔥' => 'api',
    '💧' => 'tetesan air',
    '🌊' => 'ombak',
    '🎃' => 'jack-o-lantern',
    '🎄' => 'pohon Natal',
    '🎆' => 'kembang api',
    '🎇' => 'kembang api batang',
    '🧨' => 'petasan',
    '✨' => 'berkilau',
    '🎈' => 'balon',
    '🎉' => 'party popper',
    '🎊' => 'bola konfeti',
    '🎋' => 'pohon tanabata',
    '🎍' => 'dekorasi pinus',
    '🎎' => 'boneka Jepang',
    '🎏' => 'bendera ikan koi',
    '🎐' => 'lonceng angin',
    '🎑' => 'perayaan bulan',
    '🧧' => 'amplop merah',
    '🎀' => 'pita',
    '🎁' => 'kado',
    '🎗' => 'pita pengingat',
    '🎟' => 'tiket masuk',
    '🎫' => 'tiket',
    '🎖' => 'medali militer',
    '🏆' => 'piala',
    '🏅' => 'medali olahraga',
    '🥇' => 'medali emas',
    '🥈' => 'medali perak',
    '🥉' => 'medali perunggu',
    '⚽' => 'bola sepak',
    '⚾' => 'bisbol',
    '🥎' => 'sofbol',
    '🏀' => 'bola basket',
    '🏐' => 'bola voli',
    '🏈' => 'American football',
    '🏉' => 'sepak bola rugbi',
    '🎾' => 'tenis',
    '🥏' => 'lempar cakram',
    '🎳' => 'bowling',
    '🏏' => 'kriket',
    '🏑' => 'hoki lapangan',
    '🏒' => 'hoki es',
    '🥍' => 'lacrosse',
    '🏓' => 'ping pong',
    '🏸' => 'bulu tangkis',
    '🥊' => 'sarung tinju',
    '🥋' => 'seragam beladiri',
    '🥅' => 'gawang',
    '⛳' => 'bendera di lubang golf',
    '⛸' => 'seluncur es',
    '🎣' => 'tongkat pancing',
    '🤿' => 'kacamata selam',
    '🎽' => 'kaos lari',
    '🎿' => 'ski',
    '🛷' => 'kereta luncur',
    '🥌' => 'batu curling',
    '🎯' => 'dart',
    '🪀' => 'yo-yo',
    '🪁' => 'layang-layang',
    '🔫' => 'pistol air',
    '🎱' => 'biliar 8 bola',
    '🔮' => 'bola kristal',
    '🪄' => 'tongkat ajaib',
    '🎮' => 'video game',
    '🕹' => 'joystick',
    '🎰' => 'mesin slot',
    '🎲' => 'dadu',
    '🧩' => 'keping puzzle',
    '🧸' => 'boneka beruang',
    '🪅' => 'pinata',
    '🪩' => 'bola disko',
    '🪆' => 'boneka bersarang',
    '♠' => 'waru',
    '♥' => 'hati',
    '♦' => 'wajik',
    '♣' => 'keriting',
    '♟' => 'pion catur',
    '🃏' => 'joker',
    '🀄' => 'naga merah mahyong',
    '🎴' => 'kartu bunga',
    '🎭' => 'seni drama',
    '🖼' => 'bingkai bergambar',
    '🎨' => 'palet pelukis',
    '🧵' => 'benang',
    '🪡' => 'jarum jahit',
    '🧶' => 'gulungan benang',
    '🪢' => 'simpul',
    '👓' => 'kacamata',
    '🕶' => 'kacamata hitam',
    '🥽' => 'kacamata renang',
    '🥼' => 'jas laboratorium',
    '🦺' => 'rompi pengaman',
    '👔' => 'dasi',
    '👕' => 'kaos',
    '👖' => 'celana jeans',
    '🧣' => 'syal',
    '🧤' => 'sarung tangan',
    '🧥' => 'mantel',
    '🧦' => 'kaus kaki',
    '👗' => 'gaun',
    '👘' => 'kimono',
    '🥻' => 'sari',
    '🩱' => 'pakaian renang one-piece',
    '🩲' => 'celana renang',
    '🩳' => 'celana pendek',
    '👙' => 'bikini',
    '👚' => 'pakaian wanita',
    '🪭' => 'kipas tangan lipat',
    '👛' => 'dompet',
    '👜' => 'tas jinjing',
    '👝' => 'kantung',
    '🛍' => 'tas belanja',
    '🎒' => 'tas sekolah',
    '🩴' => 'sandal jepit',
    '👞' => 'sepatu pria',
    '👟' => 'sepatu lari',
    '🥾' => 'sepatu boot untuk mendaki',
    '🥿' => 'sepatu flat',
    '👠' => 'sepatu hak tinggi',
    '👡' => 'sandal wanita',
    '🩰' => 'sepatu balet',
    '👢' => 'sepatu bot wanita',
    '🪮' => 'sisir garpu',
    '👑' => 'mahkota',
    '👒' => 'topi wanita',
    '🎩' => 'topi tinggi',
    '🎓' => 'topi wisuda',
    '🧢' => 'topi berlidah',
    '🪖' => 'helm tentara',
    '⛑' => 'helm dengan palang putih',
    '📿' => 'tasbih',
    '💄' => 'lipstik',
    '💍' => 'cincin',
    '💎' => 'batu permata',
    '🔇' => 'speaker mati',
    '🔈' => 'volume speaker rendah',
    '🔉' => 'volume speaker medium',
    '🔊' => 'volume speaker tinggi',
    '📢' => 'pengeras suara',
    '📣' => 'megafon',
    '📯' => 'trompet pos',
    '🔔' => 'lonceng',
    '🔕' => 'lonceng dicoret',
    '🎼' => 'naskah musik',
    '🎵' => 'not musik',
    '🎶' => 'not-not musik',
    '🎙' => 'mikrofon studio',
    '🎚' => 'penggeser tingkat',
    '🎛' => 'kenop kontrol',
    '🎤' => 'mikrofon',
    '🎧' => 'headphone',
    '📻' => 'radio',
    '🎷' => 'saksofon',
    '🎺' => 'trompet',
    '🪊' => 'trombon',
    '🪗' => 'akordeon',
    '🎸' => 'gitar',
    '🎹' => 'keyboard musik',
    '🎻' => 'biola',
    '🪕' => 'banjo',
    '🥁' => 'drum',
    '🪘' => 'gendang',
    '🪇' => 'marakas',
    '🪈' => 'seruling',
    '🪉' => 'harpa',
    '📱' => 'ponsel',
    '📲' => 'telepon masuk',
    '☎' => 'telepon',
    '📞' => 'gagang telepon',
    '📟' => 'pager',
    '📠' => 'mesin faks',
    '🔋' => 'baterai',
    '🪫' => 'baterai lemah',
    '🔌' => 'steker',
    '💻' => 'komputer laptop',
    '🖥' => 'komputer desktop',
    '🖨' => 'printer',
    '⌨' => 'keyboard',
    '🖱' => 'mouse komputer',
    '🖲' => 'trackball',
    '💽' => 'disk komputer',
    '💾' => 'disket',
    '💿' => 'disk optik',
    '📀' => 'dvd',
    '🧮' => 'sempoa',
    '🎥' => 'kamera film',
    '🎞' => 'frame film',
    '📽' => 'proyektor film',
    '🎬' => 'papan sutradara',
    '📺' => 'televisi',
    '📷' => 'kamera',
    '📸' => 'kamera dengan flash',
    '📹' => 'kamera video',
    '📼' => 'kaset video',
    '🔍' => 'kaca pembesar arah kiri',
    '🔎' => 'kaca pembesar arah kanan',
    '🕯' => 'lilin',
    '💡' => 'bohlam',
    '🔦' => 'lampu senter',
    '🏮' => 'lampion merah',
    '🪔' => 'lampu diya',
    '📔' => 'buku catatan bersampul',
    '📕' => 'buku tertutup',
    '📖' => 'buku terbuka',
    '📗' => 'buku hijau',
    '📘' => 'buku biru',
    '📙' => 'buku oranye',
    '📚' => 'buku',
    '📓' => 'buku catatan',
    '📒' => 'buku besar',
    '📃' => 'halaman tergulung',
    '📜' => 'gulungan',
    '📄' => 'halaman tampak depan',
    '📰' => 'koran',
    '🗞' => 'gulungan koran',
    '📑' => 'penanda halaman buku',
    '🔖' => 'pembatas buku',
    '🏷' => 'label',
    '🪙' => 'koin',
    '💰' => 'kantung uang',
    '🪎' => 'peti harta karun',
    '💴' => 'uang kertas yen',
    '💵' => 'uang kertas dolar',
    '💶' => 'uang kertas euro',
    '💷' => 'uang kertas pound',
    '💸' => 'uang bersayap',
    '💳' => 'kartu kredit',
    '🧾' => 'kuitansi',
    '💹' => 'bagan meningkat dengan yen',
    '✉' => 'amplop',
    '📧' => 'email',
    '📨' => 'amplop datang',
    '📩' => 'amplop dengan panah',
    '📤' => 'baki kotak keluar',
    '📥' => 'baki kotak masuk',
    '📦' => 'paket',
    '📫' => 'kotak surat tertutup dengan bendera terangkat',
    '📪' => 'kotak surat tertutup dengan bendera turun',
    '📬' => 'kotak surat terbuka dengan bendera terangkat',
    '📭' => 'kotak surat terbuka dengan bendera turun',
    '📮' => 'kotak pos',
    '🗳' => 'kotak dengan surat suara',
    '✏' => 'pensil',
    '✒' => 'ujung pena',
    '🖋' => 'pena tinta',
    '🖊' => 'pena',
    '🖌' => 'kuas',
    '🖍' => 'krayon',
    '📝' => 'memo',
    '💼' => 'tas kantor',
    '📁' => 'folder file',
    '📂' => 'folder file terbuka',
    '🗂' => 'pembagi indeks kartu',
    '📅' => 'kalender',
    '📆' => 'kalender sobek',
    '🗒' => 'catatan spiral',
    '🗓' => 'kalender spiral',
    '📇' => 'indeks kartu',
    '📈' => 'bagan meningkat',
    '📉' => 'bagan menurun',
    '📊' => 'diagram batang',
    '📋' => 'papan klip',
    '📌' => 'paku penanda',
    '📍' => 'paku penanda bulat',
    '📎' => 'klip kertas',
    '🖇' => 'klip kertas tertaut',
    '📏' => 'penggaris lurus',
    '📐' => 'penggaris segitiga',
    '✂' => 'gunting',
    '🗃' => 'kotak file kartu',
    '🗄' => 'lemari file',
    '🗑' => 'keranjang sampah',
    '🔒' => 'gembok',
    '🔓' => 'gembok terbuka',
    '🔏' => 'gembok terkunci dengan pena',
    '🔐' => 'gembok terkunci dengan kunci',
    '🔑' => 'kunci',
    '🗝' => 'kunci antik',
    '🔨' => 'palu',
    '🪓' => 'kapak',
    '⛏' => 'beliung',
    '⚒' => 'palu dan beliung',
    '🛠' => 'palu dan kunci inggris',
    '🗡' => 'belati',
    '⚔' => 'pedang bersilang',
    '💣' => 'bom',
    '🪃' => 'bumerang',
    '🏹' => 'busur dan panah',
    '🛡' => 'perisai',
    '🪚' => 'gergaji kayu',
    '🔧' => 'kunci inggris',
    '🪛' => 'obeng',
    '🔩' => 'mur dan baut',
    '⚙' => 'roda gigi',
    '🗜' => 'klem',
    '⚖' => 'timbangan gantung',
    '🦯' => 'tongkat tunanetra',
    '🔗' => 'tautan',
    '⛓' => 'rantai',
    '🪝' => 'kait',
    '🧰' => 'kotak perkakas',
    '🧲' => 'magnet',
    '🪜' => 'tangga',
    '🪏' => 'sekop',
    '⚗' => 'alembic',
    '🧪' => 'tabung uji',
    '🧫' => 'cawan petri',
    '🧬' => 'dna',
    '🔬' => 'mikroskop',
    '🔭' => 'teleskop',
    '📡' => 'antena satelit',
    '💉' => 'suntikan',
    '🩸' => 'tetesan darah',
    '💊' => 'pil',
    '🩹' => 'plester luka',
    '🩼' => 'kruk',
    '🩺' => 'stetoskop',
    '🩻' => 'sinar-x',
    '🚪' => 'pintu',
    '🛗' => 'elevator',
    '🪞' => 'cermin',
    '🪟' => 'jendela',
    '🛏' => 'tempat tidur',
    '🛋' => 'sofa dan lampu',
    '🪑' => 'bangku',
    '🚽' => 'toilet',
    '🪠' => 'penyedot wc',
    '🚿' => 'shower',
    '🛁' => 'bathtub',
    '🪤' => 'perangkap tikus',
    '🪒' => 'pisau cukur',
    '🧴' => 'botol losion',
    '🧷' => 'peniti',
    '🧹' => 'sapu',
    '🧺' => 'keranjang',
    '🧻' => 'gulungan kertas',
    '🪣' => 'ember',
    '🧼' => 'sabun',
    '🫧' => 'gelembung',
    '🪥' => 'sikat gigi',
    '🧽' => 'spons',
    '🧯' => 'alat pemadam api ringan',
    '🛒' => 'troli belanja',
    '🚬' => 'rokok',
    '⚰' => 'peti mati',
    '🪦' => 'batu nisan',
    '⚱' => 'guci abu',
    '🧿' => 'jimat nazar',
    '🪬' => 'hamsa',
    '🗿' => 'moai',
    '🪧' => 'plakat',
    '🪪' => 'kartu identitas',
    '🏧' => 'tanda ATM',
    '🚮' => 'tanda membuang sampah',
    '🚰' => 'air dapat diminum',
    '♿' => 'simbol kursi roda',
    '🚹' => 'toilet pria',
    '🚺' => 'toilet wanita',
    '🚻' => 'kamar kecil',
    '🚼' => 'simbol bayi',
    '🚾' => 'kloset air',
    '🛂' => 'kontrol paspor',
    '🛃' => 'bea cukai',
    '🛄' => 'pengambilan bagasi',
    '🛅' => 'koper tertinggal',
    '⚠' => 'peringatan',
    '🚸' => 'anak-anak menyeberang',
    '⛔' => 'dilarang masuk',
    '🚫' => 'dilarang',
    '🚳' => 'dilarang bersepeda',
    '🚭' => 'dilarang merokok',
    '🚯' => 'dilarang membuang sampah sembarangan',
    '🚱' => 'air tidak dapat diminum',
    '🚷' => 'pejalan kaki dilarang masuk',
    '📵' => 'dilarang menggunakan ponsel',
    '🔞' => 'di bawah 18 tahun dilarang',
    '☢' => 'radioaktif',
    '☣' => 'biohazard',
    '⬆' => 'tanda panah atas',
    '↗' => 'tanda panah kanan atas',
    '➡' => 'tanda panah kanan',
    '↘' => 'tanda panah kanan bawah',
    '⬇' => 'tanda panah bawah',
    '↙' => 'tanda panah kiri bawah',
    '⬅' => 'tanda panah kiri',
    '↖' => 'tanda panah kiri atas',
    '↕' => 'tanda panah atas bawah',
    '↔' => 'tanda panah kiri kanan',
    '↩' => 'tanda panah kanan melengkung ke kiri',
    '↪' => 'tanda panah kanan melengkung ke kanan',
    '⤴' => 'tanda panah kanan melengkung ke atas',
    '⤵' => 'tanda panah kanan melengkung ke bawah',
    '🔃' => 'tanda panah searah jarum jam',
    '🔄' => 'tanda panah berlawanan arah jarum jam',
    '🔙' => 'tanda panah BACK',
    '🔚' => 'tanda panah END',
    '🔛' => 'tanda panah ON!',
    '🔜' => 'tanda panah SOON',
    '🔝' => 'tanda panah TOP',
    '🛐' => 'tempat ibadah',
    '⚛' => 'simbol atom',
    '🕉' => 'om',
    '✡' => 'bintang Daud',
    '☸' => 'roda dharma',
    '☯' => 'yin yang',
    '✝' => 'salib latin',
    '☦' => 'salib ortodoks',
    '☪' => 'bintang dan bulan',
    '☮' => 'simbol damai',
    '🕎' => 'menorah',
    '🔯' => 'bintang sudut enam bertitik',
    '🪯' => 'khanda',
    '♈' => 'Aries',
    '♉' => 'Taurus',
    '♊' => 'Gemini',
    '♋' => 'Cancer',
    '♌' => 'Leo',
    '♍' => 'Virgo',
    '♎' => 'Libra',
    '♏' => 'Scorpio',
    '♐' => 'Sagitarius',
    '♑' => 'Capricorn',
    '♒' => 'Aquarius',
    '♓' => 'Pisces',
    '⛎' => 'Ophiuchus',
    '🔀' => 'tombol acak lagu',
    '🔁' => 'tombol ulangi',
    '🔂' => 'tombol ulangi satu lagu',
    '▶' => 'tombol putar',
    '⏩' => 'tombol maju cepat',
    '⏭' => 'tombol lagu berikutnya',
    '⏯' => 'tombol putar atau jeda',
    '◀' => 'tombol mundur',
    '⏪' => 'tombol mundur cepat',
    '⏮' => 'tombol lagu terakhir',
    '🔼' => 'tombol naik',
    '⏫' => 'tombol naik cepat',
    '🔽' => 'tombol turun',
    '⏬' => 'tombol turun cepat',
    '⏸' => 'tombol jeda',
    '⏹' => 'tombol berhenti',
    '⏺' => 'tombol rekam',
    '⏏' => 'tombol keluarkan',
    '🎦' => 'film',
    '🔅' => 'tombol redupkan',
    '🔆' => 'tombol cerahkan',
    '📶' => 'batang antena',
    '🛜' => 'nirkabel',
    '📳' => 'mode getar',
    '📴' => 'ponsel nonaktif',
    '♀' => 'simbol wanita',
    '♂' => 'simbol pria',
    '⚧' => 'simbol transgender',
    '✖' => 'kali',
    '➕' => 'tambah',
    '➖' => 'kurang',
    '➗' => 'bagi',
    '🟰' => 'tanda sama dengan tebal',
    '♾' => 'tak terhingga',
    '‼' => 'tanda seru ganda',
    '⁉' => 'tanda seru dan tanya',
    '❓' => 'tanda tanya merah',
    '❔' => 'tanda tanya putih',
    '❕' => 'tanda seru putih',
    '❗' => 'tanda seru merah',
    '〰' => 'garis bergelombang',
    '💱' => 'kurs',
    '💲' => 'tanda dolar tebal',
    '⚕' => 'simbol kedokteran',
    '♻' => 'simbol daur ulang',
    '⚜' => 'fleur-de-lis',
    '🔱' => 'emblem trisula',
    '📛' => 'lencana nama',
    '🔰' => 'simbol jepang untuk pemula',
    '⭕' => 'lingkaran besar tebal',
    '✅' => 'tanda centang putih tebal',
    '☑' => 'kotak bercentang',
    '✔' => 'tanda centang tebal',
    '❌' => 'tanda silang',
    '❎' => 'tombol tanda silang',
    '➰' => 'untaian bergelombang',
    '➿' => 'untaian bergelombang ganda',
    '〽' => 'tanda alternasi bagian',
    '✳' => 'bintang berujung lancip delapan',
    '✴' => 'bintang berujung delapan',
    '❇' => 'kilauan',
    '™' => 'merek dagang',
    '🫟' => 'percikan',
    '🔠' => 'masukan huruf kapital latin',
    '🔡' => 'masukan huruf kecil latin',
    '🔢' => 'masukan angka',
    '🔣' => 'masukan simbol',
    '🔤' => 'masukan huruf latin',
    '🅰' => 'tombol A (golongan darah)',
    '🆎' => 'tombol AB (golongan darah)',
    '🅱' => 'tombol B (golongan darah)',
    '🆑' => 'tombol CL',
    '🆒' => 'tombol COOL',
    '🆓' => 'tombol FREE',
    'ℹ' => 'informasi',
    '🆔' => 'tombol ID',
    'Ⓜ' => 'huruf M dalam lingkaran',
    '🆕' => 'tombol NEW',
    '🆖' => 'tombol NG',
    '🅾' => 'tombol O (golongan darah)',
    '🆗' => 'tombol OK',
    '🅿' => 'tombol P',
    '🆘' => 'tombol SOS',
    '🆙' => 'tombol UP!',
    '🆚' => 'tombol VS',
    '🈁' => 'tombol Jepang "di sini"',
    '🈂' => 'tombol Jepang "biaya layanan"',
    '🈷' => 'tombol Jepang "jumlah bulanan"',
    '🈶' => 'tombol Jepang "tidak gratis"',
    '🈯' => 'tombol Jepang "sudah dipesan"',
    '🉐' => 'karakter keunggulan dalam lingkaran',
    '🈹' => 'tombol Jepang "diskon"',
    '🈚' => 'tombol Jepang "gratis"',
    '🈲' => 'tombol Jepang "dilarang"',
    '🉑' => 'tombol Jepang "dapat diterima"',
    '🈸' => 'tombol Jepang "lamaran kerja"',
    '🈴' => 'tombol Jepang "nilai kelulusan"',
    '🈳' => 'tombol Jepang "ada lowongan"',
    '㊗' => 'tombol Jepang "selamat"',
    '㊙' => 'tombol Jepang "rahasia"',
    '🈺' => 'tombol Jepang "siap berbisnis"',
    '🈵' => 'tombol Jepang "tidak ada lowongan"',
    '🔴' => 'lingkaran merah',
    '🟠' => 'lingkaran oranye',
    '🟡' => 'lingkaran kuning',
    '🟢' => 'lingkaran hijau',
    '🔵' => 'lingkaran biru',
    '🟣' => 'lingkaran ungu',
    '🟤' => 'lingkaran cokelat',
    '⚫' => 'lingkaran hitam',
    '⚪' => 'lingkaran putih',
    '🟥' => 'persegi merah',
    '🟧' => 'persegi oranye',
    '🟨' => 'persegi kuning',
    '🟩' => 'persegi hijau',
    '🟦' => 'persegi biru',
    '🟪' => 'persegi ungu',
    '🟫' => 'persegi cokelat',
    '⬛' => 'persegi besar hitam',
    '⬜' => 'persegi besar putih',
    '◼' => 'persegi medium hitam',
    '◻' => 'persegi medium putih',
    '◾' => 'persegi kecil medium hitam',
    '◽' => 'persegi kecil medium putih',
    '▪' => 'persegi kecil hitam',
    '▫' => 'persegi kecil putih',
    '🔶' => 'wajik oranye besar',
    '🔷' => 'wajik biru besar',
    '🔸' => 'wajik oranye kecil',
    '🔹' => 'wajik biru kecil',
    '🔺' => 'segitiga merah mengarah ke atas',
    '🔻' => 'segitiga merah mengarah ke bawah',
    '💠' => 'wajik bertitik',
    '🔘' => 'tombol radio',
    '🔳' => 'tombol persegi putih',
    '🔲' => 'tombol persegi hitam',
    '🏁' => 'bendera kotak-kotak',
    '🚩' => 'bendera segitiga',
    '🎌' => 'bendera menyilang',
    '🏴' => 'bendera hitam',
    '🏳' => 'bendera putih',
];
