<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'polibek: dospělý člověk, dospělý člověk, světlý odstín pleti a středně světlý odstín pleti',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'polibek: dospělý člověk, dospělý člověk, světlý odstín pleti a střední odstín pleti',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'polibek: dospělý člověk, dospělý člověk, světlý odstín pleti a středně tmavý odstín pleti',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'polibek: dospělý člověk, dospělý člověk, světlý odstín pleti a tmavý odstín pleti',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'polibek: dospělý člověk, dospělý člověk, středně světlý odstín pleti a světlý odstín pleti',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'polibek: dospělý člověk, dospělý člověk, středně světlý odstín pleti a střední odstín pleti',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'polibek: dospělý člověk, dospělý člověk, středně světlý odstín pleti a středně tmavý odstín pleti',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'polibek: dospělý člověk, dospělý člověk, středně světlý odstín pleti a tmavý odstín pleti',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'polibek: dospělý člověk, dospělý člověk, střední odstín pleti a světlý odstín pleti',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'polibek: dospělý člověk, dospělý člověk, střední odstín pleti a středně světlý odstín pleti',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'polibek: dospělý člověk, dospělý člověk, střední odstín pleti a středně tmavý odstín pleti',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'polibek: dospělý člověk, dospělý člověk, střední odstín pleti a tmavý odstín pleti',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'polibek: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a světlý odstín pleti',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'polibek: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a středně světlý odstín pleti',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'polibek: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a střední odstín pleti',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'polibek: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a tmavý odstín pleti',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'polibek: dospělý člověk, dospělý člověk, tmavý odstín pleti a světlý odstín pleti',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'polibek: dospělý člověk, dospělý člověk, tmavý odstín pleti a středně světlý odstín pleti',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'polibek: dospělý člověk, dospělý člověk, tmavý odstín pleti a střední odstín pleti',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'polibek: dospělý člověk, dospělý člověk, tmavý odstín pleti a středně tmavý odstín pleti',
    '👨🏻‍❤‍💋‍👨🏻' => 'polibek: muž, muž a světlý odstín pleti',
    '👨🏻‍❤‍💋‍👨🏼' => 'polibek: muž, muž, světlý odstín pleti a středně světlý odstín pleti',
    '👨🏻‍❤‍💋‍👨🏽' => 'polibek: muž, muž, světlý odstín pleti a střední odstín pleti',
    '👨🏻‍❤‍💋‍👨🏾' => 'polibek: muž, muž, světlý odstín pleti a středně tmavý odstín pleti',
    '👨🏻‍❤‍💋‍👨🏿' => 'polibek: muž, muž, světlý odstín pleti a tmavý odstín pleti',
    '👨🏼‍❤‍💋‍👨🏻' => 'polibek: muž, muž, středně světlý odstín pleti a světlý odstín pleti',
    '👨🏼‍❤‍💋‍👨🏼' => 'polibek: muž, muž a středně světlý odstín pleti',
    '👨🏼‍❤‍💋‍👨🏽' => 'polibek: muž, muž, středně světlý odstín pleti a střední odstín pleti',
    '👨🏼‍❤‍💋‍👨🏾' => 'polibek: muž, muž, středně světlý odstín pleti a středně tmavý odstín pleti',
    '👨🏼‍❤‍💋‍👨🏿' => 'polibek: muž, muž, středně světlý odstín pleti a tmavý odstín pleti',
    '👨🏽‍❤‍💋‍👨🏻' => 'polibek: muž, muž, střední odstín pleti a světlý odstín pleti',
    '👨🏽‍❤‍💋‍👨🏼' => 'polibek: muž, muž, střední odstín pleti a středně světlý odstín pleti',
    '👨🏽‍❤‍💋‍👨🏽' => 'polibek: muž, muž a střední odstín pleti',
    '👨🏽‍❤‍💋‍👨🏾' => 'polibek: muž, muž, střední odstín pleti a středně tmavý odstín pleti',
    '👨🏽‍❤‍💋‍👨🏿' => 'polibek: muž, muž, střední odstín pleti a tmavý odstín pleti',
    '👨🏾‍❤‍💋‍👨🏻' => 'polibek: muž, muž, středně tmavý odstín pleti a světlý odstín pleti',
    '👨🏾‍❤‍💋‍👨🏼' => 'polibek: muž, muž, středně tmavý odstín pleti a středně světlý odstín pleti',
    '👨🏾‍❤‍💋‍👨🏽' => 'polibek: muž, muž, středně tmavý odstín pleti a střední odstín pleti',
    '👨🏾‍❤‍💋‍👨🏾' => 'polibek: muž, muž a středně tmavý odstín pleti',
    '👨🏾‍❤‍💋‍👨🏿' => 'polibek: muž, muž, středně tmavý odstín pleti a tmavý odstín pleti',
    '👨🏿‍❤‍💋‍👨🏻' => 'polibek: muž, muž, tmavý odstín pleti a světlý odstín pleti',
    '👨🏿‍❤‍💋‍👨🏼' => 'polibek: muž, muž, tmavý odstín pleti a středně světlý odstín pleti',
    '👨🏿‍❤‍💋‍👨🏽' => 'polibek: muž, muž, tmavý odstín pleti a střední odstín pleti',
    '👨🏿‍❤‍💋‍👨🏾' => 'polibek: muž, muž, tmavý odstín pleti a středně tmavý odstín pleti',
    '👨🏿‍❤‍💋‍👨🏿' => 'polibek: muž, muž a tmavý odstín pleti',
    '👩🏻‍❤‍💋‍👨🏻' => 'polibek: žena, muž a světlý odstín pleti',
    '👩🏻‍❤‍💋‍👨🏼' => 'polibek: žena, muž, světlý odstín pleti a středně světlý odstín pleti',
    '👩🏻‍❤‍💋‍👨🏽' => 'polibek: žena, muž, světlý odstín pleti a střední odstín pleti',
    '👩🏻‍❤‍💋‍👨🏾' => 'polibek: žena, muž, světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏻‍❤‍💋‍👨🏿' => 'polibek: žena, muž, světlý odstín pleti a tmavý odstín pleti',
    '👩🏼‍❤‍💋‍👨🏻' => 'polibek: žena, muž, středně světlý odstín pleti a světlý odstín pleti',
    '👩🏼‍❤‍💋‍👨🏼' => 'polibek: žena, muž a středně světlý odstín pleti',
    '👩🏼‍❤‍💋‍👨🏽' => 'polibek: žena, muž, středně světlý odstín pleti a střední odstín pleti',
    '👩🏼‍❤‍💋‍👨🏾' => 'polibek: žena, muž, středně světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏼‍❤‍💋‍👨🏿' => 'polibek: žena, muž, středně světlý odstín pleti a tmavý odstín pleti',
    '👩🏽‍❤‍💋‍👨🏻' => 'polibek: žena, muž, střední odstín pleti a světlý odstín pleti',
    '👩🏽‍❤‍💋‍👨🏼' => 'polibek: žena, muž, střední odstín pleti a středně světlý odstín pleti',
    '👩🏽‍❤‍💋‍👨🏽' => 'polibek: žena, muž a střední odstín pleti',
    '👩🏽‍❤‍💋‍👨🏾' => 'polibek: žena, muž, střední odstín pleti a středně tmavý odstín pleti',
    '👩🏽‍❤‍💋‍👨🏿' => 'polibek: žena, muž, střední odstín pleti a tmavý odstín pleti',
    '👩🏾‍❤‍💋‍👨🏻' => 'polibek: žena, muž, středně tmavý odstín pleti a světlý odstín pleti',
    '👩🏾‍❤‍💋‍👨🏼' => 'polibek: žena, muž, středně tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏾‍❤‍💋‍👨🏽' => 'polibek: žena, muž, středně tmavý odstín pleti a střední odstín pleti',
    '👩🏾‍❤‍💋‍👨🏾' => 'polibek: žena, muž a středně tmavý odstín pleti',
    '👩🏾‍❤‍💋‍👨🏿' => 'polibek: žena, muž, středně tmavý odstín pleti a tmavý odstín pleti',
    '👩🏿‍❤‍💋‍👨🏻' => 'polibek: žena, muž, tmavý odstín pleti a světlý odstín pleti',
    '👩🏿‍❤‍💋‍👨🏼' => 'polibek: žena, muž, tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏿‍❤‍💋‍👨🏽' => 'polibek: žena, muž, tmavý odstín pleti a střední odstín pleti',
    '👩🏿‍❤‍💋‍👨🏾' => 'polibek: žena, muž, tmavý odstín pleti a středně tmavý odstín pleti',
    '👩🏿‍❤‍💋‍👨🏿' => 'polibek: žena, muž a tmavý odstín pleti',
    '👩🏻‍❤‍💋‍👩🏻' => 'polibek: žena, žena a světlý odstín pleti',
    '👩🏻‍❤‍💋‍👩🏼' => 'polibek: žena, žena, světlý odstín pleti a středně světlý odstín pleti',
    '👩🏻‍❤‍💋‍👩🏽' => 'polibek: žena, žena, světlý odstín pleti a střední odstín pleti',
    '👩🏻‍❤‍💋‍👩🏾' => 'polibek: žena, žena, světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏻‍❤‍💋‍👩🏿' => 'polibek: žena, žena, světlý odstín pleti a tmavý odstín pleti',
    '👩🏼‍❤‍💋‍👩🏻' => 'polibek: žena, žena, středně světlý odstín pleti a světlý odstín pleti',
    '👩🏼‍❤‍💋‍👩🏼' => 'polibek: žena, žena a středně světlý odstín pleti',
    '👩🏼‍❤‍💋‍👩🏽' => 'polibek: žena, žena, středně světlý odstín pleti a střední odstín pleti',
    '👩🏼‍❤‍💋‍👩🏾' => 'polibek: žena, žena, středně světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏼‍❤‍💋‍👩🏿' => 'polibek: žena, žena, středně světlý odstín pleti a tmavý odstín pleti',
    '👩🏽‍❤‍💋‍👩🏻' => 'polibek: žena, žena, střední odstín pleti a světlý odstín pleti',
    '👩🏽‍❤‍💋‍👩🏼' => 'polibek: žena, žena, střední odstín pleti a středně světlý odstín pleti',
    '👩🏽‍❤‍💋‍👩🏽' => 'polibek: žena, žena a střední odstín pleti',
    '👩🏽‍❤‍💋‍👩🏾' => 'polibek: žena, žena, střední odstín pleti a středně tmavý odstín pleti',
    '👩🏽‍❤‍💋‍👩🏿' => 'polibek: žena, žena, střední odstín pleti a tmavý odstín pleti',
    '👩🏾‍❤‍💋‍👩🏻' => 'polibek: žena, žena, středně tmavý odstín pleti a světlý odstín pleti',
    '👩🏾‍❤‍💋‍👩🏼' => 'polibek: žena, žena, středně tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏾‍❤‍💋‍👩🏽' => 'polibek: žena, žena, středně tmavý odstín pleti a střední odstín pleti',
    '👩🏾‍❤‍💋‍👩🏾' => 'polibek: žena, žena a středně tmavý odstín pleti',
    '👩🏾‍❤‍💋‍👩🏿' => 'polibek: žena, žena, středně tmavý odstín pleti a tmavý odstín pleti',
    '👩🏿‍❤‍💋‍👩🏻' => 'polibek: žena, žena, tmavý odstín pleti a světlý odstín pleti',
    '👩🏿‍❤‍💋‍👩🏼' => 'polibek: žena, žena, tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏿‍❤‍💋‍👩🏽' => 'polibek: žena, žena, tmavý odstín pleti a střední odstín pleti',
    '👩🏿‍❤‍💋‍👩🏾' => 'polibek: žena, žena, tmavý odstín pleti a středně tmavý odstín pleti',
    '👩🏿‍❤‍💋‍👩🏿' => 'polibek: žena, žena a tmavý odstín pleti',
    '🧑🏻‍🫯‍🧑🏼' => 'zápasící dvojice: světlý odstín pleti, středně světlý odstín pleti',
    '🧑🏻‍🫯‍🧑🏽' => 'zápasící dvojice: světlý odstín pleti, střední odstín pleti',
    '🧑🏻‍🫯‍🧑🏾' => 'zápasící dvojice: světlý odstín pleti, středně tmavý odstín pleti',
    '🧑🏻‍🫯‍🧑🏿' => 'zápasící dvojice: světlý odstín pleti, tmavý odstín pleti',
    '🧑🏼‍🫯‍🧑🏻' => 'zápasící dvojice: středně světlý odstín pleti, světlý odstín pleti',
    '🧑🏼‍🫯‍🧑🏽' => 'zápasící dvojice: středně světlý odstín pleti, střední odstín pleti',
    '🧑🏼‍🫯‍🧑🏾' => 'zápasící dvojice: středně světlý odstín pleti, středně tmavý odstín pleti',
    '🧑🏼‍🫯‍🧑🏿' => 'zápasící dvojice: středně světlý odstín pleti, tmavý odstín pleti',
    '🧑🏽‍🫯‍🧑🏻' => 'zápasící dvojice: střední odstín pleti, světlý odstín pleti',
    '🧑🏽‍🫯‍🧑🏼' => 'zápasící dvojice: střední odstín pleti, středně světlý odstín pleti',
    '🧑🏽‍🫯‍🧑🏾' => 'zápasící dvojice: střední odstín pleti, středně tmavý odstín pleti',
    '🧑🏽‍🫯‍🧑🏿' => 'zápasící dvojice: střední odstín pleti, tmavý odstín pleti',
    '🧑🏾‍🫯‍🧑🏻' => 'zápasící dvojice: středně tmavý odstín pleti, světlý odstín pleti',
    '🧑🏾‍🫯‍🧑🏼' => 'zápasící dvojice: středně tmavý odstín pleti, středně světlý odstín pleti',
    '🧑🏾‍🫯‍🧑🏽' => 'zápasící dvojice: středně tmavý odstín pleti, střední odstín pleti',
    '🧑🏾‍🫯‍🧑🏿' => 'zápasící dvojice: středně tmavý odstín pleti, tmavý odstín pleti',
    '🧑🏿‍🫯‍🧑🏻' => 'zápasící dvojice: tmavý odstín pleti, světlý odstín pleti',
    '🧑🏿‍🫯‍🧑🏼' => 'zápasící dvojice: tmavý odstín pleti, středně světlý odstín pleti',
    '🧑🏿‍🫯‍🧑🏽' => 'zápasící dvojice: tmavý odstín pleti, střední odstín pleti',
    '🧑🏿‍🫯‍🧑🏾' => 'zápasící dvojice: tmavý odstín pleti, středně tmavý odstín pleti',
    '🧑🏻‍❤‍🧑🏼' => 'pár se srdcem: dospělý člověk, dospělý člověk, světlý odstín pleti a středně světlý odstín pleti',
    '🧑🏻‍❤‍🧑🏽' => 'pár se srdcem: dospělý člověk, dospělý člověk, světlý odstín pleti a střední odstín pleti',
    '🧑🏻‍❤‍🧑🏾' => 'pár se srdcem: dospělý člověk, dospělý člověk, světlý odstín pleti a středně tmavý odstín pleti',
    '🧑🏻‍❤‍🧑🏿' => 'pár se srdcem: dospělý člověk, dospělý člověk, světlý odstín pleti a tmavý odstín pleti',
    '🧑🏼‍❤‍🧑🏻' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně světlý odstín pleti a světlý odstín pleti',
    '🧑🏼‍❤‍🧑🏽' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně světlý odstín pleti a střední odstín pleti',
    '🧑🏼‍❤‍🧑🏾' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně světlý odstín pleti a středně tmavý odstín pleti',
    '🧑🏼‍❤‍🧑🏿' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně světlý odstín pleti a tmavý odstín pleti',
    '🧑🏽‍❤‍🧑🏻' => 'pár se srdcem: dospělý člověk, dospělý člověk, střední odstín pleti a světlý odstín pleti',
    '🧑🏽‍❤‍🧑🏼' => 'pár se srdcem: dospělý člověk, dospělý člověk, střední odstín pleti a středně světlý odstín pleti',
    '🧑🏽‍❤‍🧑🏾' => 'pár se srdcem: dospělý člověk, dospělý člověk, střední odstín pleti a středně tmavý odstín pleti',
    '🧑🏽‍❤‍🧑🏿' => 'pár se srdcem: dospělý člověk, dospělý člověk, střední odstín pleti a tmavý odstín pleti',
    '🧑🏾‍❤‍🧑🏻' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a světlý odstín pleti',
    '🧑🏾‍❤‍🧑🏼' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a středně světlý odstín pleti',
    '🧑🏾‍❤‍🧑🏽' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a střední odstín pleti',
    '🧑🏾‍❤‍🧑🏿' => 'pár se srdcem: dospělý člověk, dospělý člověk, středně tmavý odstín pleti a tmavý odstín pleti',
    '🧑🏿‍❤‍🧑🏻' => 'pár se srdcem: dospělý člověk, dospělý člověk, tmavý odstín pleti a světlý odstín pleti',
    '🧑🏿‍❤‍🧑🏼' => 'pár se srdcem: dospělý člověk, dospělý člověk, tmavý odstín pleti a středně světlý odstín pleti',
    '🧑🏿‍❤‍🧑🏽' => 'pár se srdcem: dospělý člověk, dospělý člověk, tmavý odstín pleti a střední odstín pleti',
    '🧑🏿‍❤‍🧑🏾' => 'pár se srdcem: dospělý člověk, dospělý člověk, tmavý odstín pleti a středně tmavý odstín pleti',
    '🧑🏻‍🐰‍🧑🏼' => 'osoby se zaječími oušky: světlý odstín pleti, středně světlý odstín pleti',
    '🧑🏻‍🐰‍🧑🏽' => 'osoby se zaječími oušky: světlý odstín pleti, střední odstín pleti',
    '🧑🏻‍🐰‍🧑🏾' => 'osoby se zaječími oušky: světlý odstín pleti, středně tmavý odstín pleti',
    '🧑🏻‍🐰‍🧑🏿' => 'osoby se zaječími oušky: světlý odstín pleti, tmavý odstín pleti',
    '🧑🏼‍🐰‍🧑🏻' => 'osoby se zaječími oušky: středně světlý odstín pleti, světlý odstín pleti',
    '🧑🏼‍🐰‍🧑🏽' => 'osoby se zaječími oušky: středně světlý odstín pleti, střední odstín pleti',
    '🧑🏼‍🐰‍🧑🏾' => 'osoby se zaječími oušky: středně světlý odstín pleti, středně tmavý odstín pleti',
    '🧑🏼‍🐰‍🧑🏿' => 'osoby se zaječími oušky: středně světlý odstín pleti, tmavý odstín pleti',
    '🧑🏽‍🐰‍🧑🏻' => 'osoby se zaječími oušky: střední odstín pleti, světlý odstín pleti',
    '🧑🏽‍🐰‍🧑🏼' => 'osoby se zaječími oušky: střední odstín pleti, středně světlý odstín pleti',
    '🧑🏽‍🐰‍🧑🏾' => 'osoby se zaječími oušky: střední odstín pleti, středně tmavý odstín pleti',
    '🧑🏽‍🐰‍🧑🏿' => 'osoby se zaječími oušky: střední odstín pleti, tmavý odstín pleti',
    '🧑🏾‍🐰‍🧑🏻' => 'osoby se zaječími oušky: středně tmavý odstín pleti, světlý odstín pleti',
    '🧑🏾‍🐰‍🧑🏼' => 'osoby se zaječími oušky: středně tmavý odstín pleti, středně světlý odstín pleti',
    '🧑🏾‍🐰‍🧑🏽' => 'osoby se zaječími oušky: středně tmavý odstín pleti, střední odstín pleti',
    '🧑🏾‍🐰‍🧑🏿' => 'osoby se zaječími oušky: středně tmavý odstín pleti, tmavý odstín pleti',
    '🧑🏿‍🐰‍🧑🏻' => 'osoby se zaječími oušky: tmavý odstín pleti, světlý odstín pleti',
    '🧑🏿‍🐰‍🧑🏼' => 'osoby se zaječími oušky: tmavý odstín pleti, středně světlý odstín pleti',
    '🧑🏿‍🐰‍🧑🏽' => 'osoby se zaječími oušky: tmavý odstín pleti, střední odstín pleti',
    '🧑🏿‍🐰‍🧑🏾' => 'osoby se zaječími oušky: tmavý odstín pleti, středně tmavý odstín pleti',
    '👨🏻‍🫯‍👨🏼' => 'zápasící muži: světlý odstín pleti, středně světlý odstín pleti',
    '👨🏻‍🫯‍👨🏽' => 'zápasící muži: světlý odstín pleti, střední odstín pleti',
    '👨🏻‍🫯‍👨🏾' => 'zápasící muži: světlý odstín pleti, středně tmavý odstín pleti',
    '👨🏻‍🫯‍👨🏿' => 'zápasící muži: světlý odstín pleti, tmavý odstín pleti',
    '👨🏼‍🫯‍👨🏻' => 'zápasící muži: středně světlý odstín pleti, světlý odstín pleti',
    '👨🏼‍🫯‍👨🏽' => 'zápasící muži: středně světlý odstín pleti, střední odstín pleti',
    '👨🏼‍🫯‍👨🏾' => 'zápasící muži: středně světlý odstín pleti, středně tmavý odstín pleti',
    '👨🏼‍🫯‍👨🏿' => 'zápasící muži: středně světlý odstín pleti, tmavý odstín pleti',
    '👨🏽‍🫯‍👨🏻' => 'zápasící muži: střední odstín pleti, světlý odstín pleti',
    '👨🏽‍🫯‍👨🏼' => 'zápasící muži: střední odstín pleti, středně světlý odstín pleti',
    '👨🏽‍🫯‍👨🏾' => 'zápasící muži: střední odstín pleti, středně tmavý odstín pleti',
    '👨🏽‍🫯‍👨🏿' => 'zápasící muži: střední odstín pleti, tmavý odstín pleti',
    '👨🏾‍🫯‍👨🏻' => 'zápasící muži: středně tmavý odstín pleti, světlý odstín pleti',
    '👨🏾‍🫯‍👨🏼' => 'zápasící muži: středně tmavý odstín pleti, středně světlý odstín pleti',
    '👨🏾‍🫯‍👨🏽' => 'zápasící muži: středně tmavý odstín pleti, střední odstín pleti',
    '👨🏾‍🫯‍👨🏿' => 'zápasící muži: středně tmavý odstín pleti, tmavý odstín pleti',
    '👨🏿‍🫯‍👨🏻' => 'zápasící muži: tmavý odstín pleti, světlý odstín pleti',
    '👨🏿‍🫯‍👨🏼' => 'zápasící muži: tmavý odstín pleti, středně světlý odstín pleti',
    '👨🏿‍🫯‍👨🏽' => 'zápasící muži: tmavý odstín pleti, střední odstín pleti',
    '👨🏿‍🫯‍👨🏾' => 'zápasící muži: tmavý odstín pleti, středně tmavý odstín pleti',
    '👨🏻‍❤‍👨🏻' => 'pár se srdcem: muž, muž a světlý odstín pleti',
    '👨🏻‍❤‍👨🏼' => 'pár se srdcem: muž, muž, světlý odstín pleti a středně světlý odstín pleti',
    '👨🏻‍❤‍👨🏽' => 'pár se srdcem: muž, muž, světlý odstín pleti a střední odstín pleti',
    '👨🏻‍❤‍👨🏾' => 'pár se srdcem: muž, muž, světlý odstín pleti a středně tmavý odstín pleti',
    '👨🏻‍❤‍👨🏿' => 'pár se srdcem: muž, muž, světlý odstín pleti a tmavý odstín pleti',
    '👨🏼‍❤‍👨🏻' => 'pár se srdcem: muž, muž, středně světlý odstín pleti a světlý odstín pleti',
    '👨🏼‍❤‍👨🏼' => 'pár se srdcem: muž, muž a středně světlý odstín pleti',
    '👨🏼‍❤‍👨🏽' => 'pár se srdcem: muž, muž, středně světlý odstín pleti a střední odstín pleti',
    '👨🏼‍❤‍👨🏾' => 'pár se srdcem: muž, muž, středně světlý odstín pleti a středně tmavý odstín pleti',
    '👨🏼‍❤‍👨🏿' => 'pár se srdcem: muž, muž, středně světlý odstín pleti a tmavý odstín pleti',
    '👨🏽‍❤‍👨🏻' => 'pár se srdcem: muž, muž, střední odstín pleti a světlý odstín pleti',
    '👨🏽‍❤‍👨🏼' => 'pár se srdcem: muž, muž, střední odstín pleti a středně světlý odstín pleti',
    '👨🏽‍❤‍👨🏽' => 'pár se srdcem: muž, muž a střední odstín pleti',
    '👨🏽‍❤‍👨🏾' => 'pár se srdcem: muž, muž, střední odstín pleti a středně tmavý odstín pleti',
    '👨🏽‍❤‍👨🏿' => 'pár se srdcem: muž, muž, střední odstín pleti a tmavý odstín pleti',
    '👨🏾‍❤‍👨🏻' => 'pár se srdcem: muž, muž, středně tmavý odstín pleti a světlý odstín pleti',
    '👨🏾‍❤‍👨🏼' => 'pár se srdcem: muž, muž, středně tmavý odstín pleti a středně světlý odstín pleti',
    '👨🏾‍❤‍👨🏽' => 'pár se srdcem: muž, muž, středně tmavý odstín pleti a střední odstín pleti',
    '👨🏾‍❤‍👨🏾' => 'pár se srdcem: muž, muž a středně tmavý odstín pleti',
    '👨🏾‍❤‍👨🏿' => 'pár se srdcem: muž, muž, středně tmavý odstín pleti a tmavý odstín pleti',
    '👨🏿‍❤‍👨🏻' => 'pár se srdcem: muž, muž, tmavý odstín pleti a světlý odstín pleti',
    '👨🏿‍❤‍👨🏼' => 'pár se srdcem: muž, muž, tmavý odstín pleti a středně světlý odstín pleti',
    '👨🏿‍❤‍👨🏽' => 'pár se srdcem: muž, muž, tmavý odstín pleti a střední odstín pleti',
    '👨🏿‍❤‍👨🏾' => 'pár se srdcem: muž, muž, tmavý odstín pleti a středně tmavý odstín pleti',
    '👨🏿‍❤‍👨🏿' => 'pár se srdcem: muž, muž a tmavý odstín pleti',
    '👨🏻‍🐰‍👨🏼' => 'muži se zaječími oušky: světlý odstín pleti, středně světlý odstín pleti',
    '👨🏻‍🐰‍👨🏽' => 'muži se zaječími oušky: světlý odstín pleti, střední odstín pleti',
    '👨🏻‍🐰‍👨🏾' => 'muži se zaječími oušky: světlý odstín pleti, středně tmavý odstín pleti',
    '👨🏻‍🐰‍👨🏿' => 'muži se zaječími oušky: světlý odstín pleti, tmavý odstín pleti',
    '👨🏼‍🐰‍👨🏻' => 'muži se zaječími oušky: středně světlý odstín pleti, světlý odstín pleti',
    '👨🏼‍🐰‍👨🏽' => 'muži se zaječími oušky: středně světlý odstín pleti, střední odstín pleti',
    '👨🏼‍🐰‍👨🏾' => 'muži se zaječími oušky: středně světlý odstín pleti, středně tmavý odstín pleti',
    '👨🏼‍🐰‍👨🏿' => 'muži se zaječími oušky: středně světlý odstín pleti, tmavý odstín pleti',
    '👨🏽‍🐰‍👨🏻' => 'muži se zaječími oušky: střední odstín pleti, světlý odstín pleti',
    '👨🏽‍🐰‍👨🏼' => 'muži se zaječími oušky: střední odstín pleti, středně světlý odstín pleti',
    '👨🏽‍🐰‍👨🏾' => 'muži se zaječími oušky: střední odstín pleti, středně tmavý odstín pleti',
    '👨🏽‍🐰‍👨🏿' => 'muži se zaječími oušky: střední odstín pleti, tmavý odstín pleti',
    '👨🏾‍🐰‍👨🏻' => 'muži se zaječími oušky: středně tmavý odstín pleti, světlý odstín pleti',
    '👨🏾‍🐰‍👨🏼' => 'muži se zaječími oušky: středně tmavý odstín pleti, středně světlý odstín pleti',
    '👨🏾‍🐰‍👨🏽' => 'muži se zaječími oušky: středně tmavý odstín pleti, střední odstín pleti',
    '👨🏾‍🐰‍👨🏿' => 'muži se zaječími oušky: středně tmavý odstín pleti, tmavý odstín pleti',
    '👨🏿‍🐰‍👨🏻' => 'muži se zaječími oušky: tmavý odstín pleti, světlý odstín pleti',
    '👨🏿‍🐰‍👨🏼' => 'muži se zaječími oušky: tmavý odstín pleti, středně světlý odstín pleti',
    '👨🏿‍🐰‍👨🏽' => 'muži se zaječími oušky: tmavý odstín pleti, střední odstín pleti',
    '👨🏿‍🐰‍👨🏾' => 'muži se zaječími oušky: tmavý odstín pleti, středně tmavý odstín pleti',
    '👩🏻‍🫯‍👩🏼' => 'zápasící ženy: světlý odstín pleti, středně světlý odstín pleti',
    '👩🏻‍🫯‍👩🏽' => 'zápasící ženy: světlý odstín pleti, střední odstín pleti',
    '👩🏻‍🫯‍👩🏾' => 'zápasící ženy: světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏻‍🫯‍👩🏿' => 'zápasící ženy: světlý odstín pleti, tmavý odstín pleti',
    '👩🏼‍🫯‍👩🏻' => 'zápasící ženy: středně světlý odstín pleti, světlý odstín pleti',
    '👩🏼‍🫯‍👩🏽' => 'zápasící ženy: středně světlý odstín pleti, střední odstín pleti',
    '👩🏼‍🫯‍👩🏾' => 'zápasící ženy: středně světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏼‍🫯‍👩🏿' => 'zápasící ženy: středně světlý odstín pleti, tmavý odstín pleti',
    '👩🏽‍🫯‍👩🏻' => 'zápasící ženy: střední odstín pleti, světlý odstín pleti',
    '👩🏽‍🫯‍👩🏼' => 'zápasící ženy: střední odstín pleti, středně světlý odstín pleti',
    '👩🏽‍🫯‍👩🏾' => 'zápasící ženy: střední odstín pleti, středně tmavý odstín pleti',
    '👩🏽‍🫯‍👩🏿' => 'zápasící ženy: střední odstín pleti, tmavý odstín pleti',
    '👩🏾‍🫯‍👩🏻' => 'zápasící ženy: středně tmavý odstín pleti, světlý odstín pleti',
    '👩🏾‍🫯‍👩🏼' => 'zápasící ženy: středně tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏾‍🫯‍👩🏽' => 'zápasící ženy: středně tmavý odstín pleti, střední odstín pleti',
    '👩🏾‍🫯‍👩🏿' => 'zápasící ženy: středně tmavý odstín pleti, tmavý odstín pleti',
    '👩🏿‍🫯‍👩🏻' => 'zápasící ženy: tmavý odstín pleti, světlý odstín pleti',
    '👩🏿‍🫯‍👩🏼' => 'zápasící ženy: tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏿‍🫯‍👩🏽' => 'zápasící ženy: tmavý odstín pleti, střední odstín pleti',
    '👩🏿‍🫯‍👩🏾' => 'zápasící ženy: tmavý odstín pleti, středně tmavý odstín pleti',
    '👩🏻‍❤‍👨🏻' => 'pár se srdcem: žena, muž a světlý odstín pleti',
    '👩🏻‍❤‍👨🏼' => 'pár se srdcem: žena, muž, světlý odstín pleti a středně světlý odstín pleti',
    '👩🏻‍❤‍👨🏽' => 'pár se srdcem: žena, muž, světlý odstín pleti a střední odstín pleti',
    '👩🏻‍❤‍👨🏾' => 'pár se srdcem: žena, muž, světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏻‍❤‍👨🏿' => 'pár se srdcem: žena, muž, světlý odstín pleti a tmavý odstín pleti',
    '👩🏼‍❤‍👨🏻' => 'pár se srdcem: žena, muž, středně světlý odstín pleti a světlý odstín pleti',
    '👩🏼‍❤‍👨🏼' => 'pár se srdcem: žena, muž a středně světlý odstín pleti',
    '👩🏼‍❤‍👨🏽' => 'pár se srdcem: žena, muž, středně světlý odstín pleti a střední odstín pleti',
    '👩🏼‍❤‍👨🏾' => 'pár se srdcem: žena, muž, středně světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏼‍❤‍👨🏿' => 'pár se srdcem: žena, muž, středně světlý odstín pleti a tmavý odstín pleti',
    '👩🏽‍❤‍👨🏻' => 'pár se srdcem: žena, muž, střední odstín pleti a světlý odstín pleti',
    '👩🏽‍❤‍👨🏼' => 'pár se srdcem: žena, muž, střední odstín pleti a středně světlý odstín pleti',
    '👩🏽‍❤‍👨🏽' => 'pár se srdcem: žena, muž a střední odstín pleti',
    '👩🏽‍❤‍👨🏾' => 'pár se srdcem: žena, muž, střední odstín pleti a středně tmavý odstín pleti',
    '👩🏽‍❤‍👨🏿' => 'pár se srdcem: žena, muž, střední odstín pleti a tmavý odstín pleti',
    '👩🏾‍❤‍👨🏻' => 'pár se srdcem: žena, muž, středně tmavý odstín pleti a světlý odstín pleti',
    '👩🏾‍❤‍👨🏼' => 'pár se srdcem: žena, muž, středně tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏾‍❤‍👨🏽' => 'pár se srdcem: žena, muž, středně tmavý odstín pleti a střední odstín pleti',
    '👩🏾‍❤‍👨🏾' => 'pár se srdcem: žena, muž a středně tmavý odstín pleti',
    '👩🏾‍❤‍👨🏿' => 'pár se srdcem: žena, muž, středně tmavý odstín pleti a tmavý odstín pleti',
    '👩🏿‍❤‍👨🏻' => 'pár se srdcem: žena, muž, tmavý odstín pleti a světlý odstín pleti',
    '👩🏿‍❤‍👨🏼' => 'pár se srdcem: žena, muž, tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏿‍❤‍👨🏽' => 'pár se srdcem: žena, muž, tmavý odstín pleti a střední odstín pleti',
    '👩🏿‍❤‍👨🏾' => 'pár se srdcem: žena, muž, tmavý odstín pleti a středně tmavý odstín pleti',
    '👩🏿‍❤‍👨🏿' => 'pár se srdcem: žena, muž a tmavý odstín pleti',
    '👩🏻‍❤‍👩🏻' => 'pár se srdcem: žena, žena a světlý odstín pleti',
    '👩🏻‍❤‍👩🏼' => 'pár se srdcem: žena, žena, světlý odstín pleti a středně světlý odstín pleti',
    '👩🏻‍❤‍👩🏽' => 'pár se srdcem: žena, žena, světlý odstín pleti a střední odstín pleti',
    '👩🏻‍❤‍👩🏾' => 'pár se srdcem: žena, žena, světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏻‍❤‍👩🏿' => 'pár se srdcem: žena, žena, světlý odstín pleti a tmavý odstín pleti',
    '👩🏼‍❤‍👩🏻' => 'pár se srdcem: žena, žena, středně světlý odstín pleti a světlý odstín pleti',
    '👩🏼‍❤‍👩🏼' => 'pár se srdcem: žena, žena a středně světlý odstín pleti',
    '👩🏼‍❤‍👩🏽' => 'pár se srdcem: žena, žena, středně světlý odstín pleti a střední odstín pleti',
    '👩🏼‍❤‍👩🏾' => 'pár se srdcem: žena, žena, středně světlý odstín pleti a středně tmavý odstín pleti',
    '👩🏼‍❤‍👩🏿' => 'pár se srdcem: žena, žena, středně světlý odstín pleti a tmavý odstín pleti',
    '👩🏽‍❤‍👩🏻' => 'pár se srdcem: žena, žena, střední odstín pleti a světlý odstín pleti',
    '👩🏽‍❤‍👩🏼' => 'pár se srdcem: žena, žena, střední odstín pleti a středně světlý odstín pleti',
    '👩🏽‍❤‍👩🏽' => 'pár se srdcem: žena, žena a střední odstín pleti',
    '👩🏽‍❤‍👩🏾' => 'pár se srdcem: žena, žena, střední odstín pleti a středně tmavý odstín pleti',
    '👩🏽‍❤‍👩🏿' => 'pár se srdcem: žena, žena, střední odstín pleti a tmavý odstín pleti',
    '👩🏾‍❤‍👩🏻' => 'pár se srdcem: žena, žena, středně tmavý odstín pleti a světlý odstín pleti',
    '👩🏾‍❤‍👩🏼' => 'pár se srdcem: žena, žena, středně tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏾‍❤‍👩🏽' => 'pár se srdcem: žena, žena, středně tmavý odstín pleti a střední odstín pleti',
    '👩🏾‍❤‍👩🏾' => 'pár se srdcem: žena, žena a středně tmavý odstín pleti',
    '👩🏾‍❤‍👩🏿' => 'pár se srdcem: žena, žena, středně tmavý odstín pleti a tmavý odstín pleti',
    '👩🏿‍❤‍👩🏻' => 'pár se srdcem: žena, žena, tmavý odstín pleti a světlý odstín pleti',
    '👩🏿‍❤‍👩🏼' => 'pár se srdcem: žena, žena, tmavý odstín pleti a středně světlý odstín pleti',
    '👩🏿‍❤‍👩🏽' => 'pár se srdcem: žena, žena, tmavý odstín pleti a střední odstín pleti',
    '👩🏿‍❤‍👩🏾' => 'pár se srdcem: žena, žena, tmavý odstín pleti a středně tmavý odstín pleti',
    '👩🏿‍❤‍👩🏿' => 'pár se srdcem: žena, žena a tmavý odstín pleti',
    '👩🏻‍🐰‍👩🏼' => 'ženy se zaječími oušky: světlý odstín pleti, středně světlý odstín pleti',
    '👩🏻‍🐰‍👩🏽' => 'ženy se zaječími oušky: světlý odstín pleti, střední odstín pleti',
    '👩🏻‍🐰‍👩🏾' => 'ženy se zaječími oušky: světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏻‍🐰‍👩🏿' => 'ženy se zaječími oušky: světlý odstín pleti, tmavý odstín pleti',
    '👩🏼‍🐰‍👩🏻' => 'ženy se zaječími oušky: středně světlý odstín pleti, světlý odstín pleti',
    '👩🏼‍🐰‍👩🏽' => 'ženy se zaječími oušky: středně světlý odstín pleti, střední odstín pleti',
    '👩🏼‍🐰‍👩🏾' => 'ženy se zaječími oušky: středně světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏼‍🐰‍👩🏿' => 'ženy se zaječími oušky: středně světlý odstín pleti, tmavý odstín pleti',
    '👩🏽‍🐰‍👩🏻' => 'ženy se zaječími oušky: střední odstín pleti, světlý odstín pleti',
    '👩🏽‍🐰‍👩🏼' => 'ženy se zaječími oušky: střední odstín pleti, středně světlý odstín pleti',
    '👩🏽‍🐰‍👩🏾' => 'ženy se zaječími oušky: střední odstín pleti, středně tmavý odstín pleti',
    '👩🏽‍🐰‍👩🏿' => 'ženy se zaječími oušky: střední odstín pleti, tmavý odstín pleti',
    '👩🏾‍🐰‍👩🏻' => 'ženy se zaječími oušky: středně tmavý odstín pleti, světlý odstín pleti',
    '👩🏾‍🐰‍👩🏼' => 'ženy se zaječími oušky: středně tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏾‍🐰‍👩🏽' => 'ženy se zaječími oušky: středně tmavý odstín pleti, střední odstín pleti',
    '👩🏾‍🐰‍👩🏿' => 'ženy se zaječími oušky: středně tmavý odstín pleti, tmavý odstín pleti',
    '👩🏿‍🐰‍👩🏻' => 'ženy se zaječími oušky: tmavý odstín pleti, světlý odstín pleti',
    '👩🏿‍🐰‍👩🏼' => 'ženy se zaječími oušky: tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏿‍🐰‍👩🏽' => 'ženy se zaječími oušky: tmavý odstín pleti, střední odstín pleti',
    '👩🏿‍🐰‍👩🏾' => 'ženy se zaječími oušky: tmavý odstín pleti, středně tmavý odstín pleti',
    '🧑🏻‍🤝‍🧑🏻' => 'dva lidé držící se za ruce: světlý odstín pleti',
    '🧑🏻‍🤝‍🧑🏼' => 'dva lidé držící se za ruce: světlý odstín pleti, středně světlý odstín pleti',
    '🧑🏻‍🤝‍🧑🏽' => 'dva lidé držící se za ruce: světlý odstín pleti, střední odstín pleti',
    '🧑🏻‍🤝‍🧑🏾' => 'dva lidé držící se za ruce: světlý odstín pleti, středně tmavý odstín pleti',
    '🧑🏻‍🤝‍🧑🏿' => 'dva lidé držící se za ruce: světlý odstín pleti, tmavý odstín pleti',
    '🧑🏼‍🤝‍🧑🏻' => 'dva lidé držící se za ruce: středně světlý odstín pleti, světlý odstín pleti',
    '🧑🏼‍🤝‍🧑🏼' => 'dva lidé držící se za ruce: středně světlý odstín pleti',
    '🧑🏼‍🤝‍🧑🏽' => 'dva lidé držící se za ruce: středně světlý odstín pleti, střední odstín pleti',
    '🧑🏼‍🤝‍🧑🏾' => 'dva lidé držící se za ruce: středně světlý odstín pleti, středně tmavý odstín pleti',
    '🧑🏼‍🤝‍🧑🏿' => 'dva lidé držící se za ruce: středně světlý odstín pleti, tmavý odstín pleti',
    '🧑🏽‍🤝‍🧑🏻' => 'dva lidé držící se za ruce: střední odstín pleti, světlý odstín pleti',
    '🧑🏽‍🤝‍🧑🏼' => 'dva lidé držící se za ruce: střední odstín pleti, středně světlý odstín pleti',
    '🧑🏽‍🤝‍🧑🏽' => 'dva lidé držící se za ruce: střední odstín pleti',
    '🧑🏽‍🤝‍🧑🏾' => 'dva lidé držící se za ruce: střední odstín pleti, středně tmavý odstín pleti',
    '🧑🏽‍🤝‍🧑🏿' => 'dva lidé držící se za ruce: střední odstín pleti, tmavý odstín pleti',
    '🧑🏾‍🤝‍🧑🏻' => 'dva lidé držící se za ruce: středně tmavý odstín pleti, světlý odstín pleti',
    '🧑🏾‍🤝‍🧑🏼' => 'dva lidé držící se za ruce: středně tmavý odstín pleti, středně světlý odstín pleti',
    '🧑🏾‍🤝‍🧑🏽' => 'dva lidé držící se za ruce: středně tmavý odstín pleti, střední odstín pleti',
    '🧑🏾‍🤝‍🧑🏾' => 'dva lidé držící se za ruce: středně tmavý odstín pleti',
    '🧑🏾‍🤝‍🧑🏿' => 'dva lidé držící se za ruce: středně tmavý odstín pleti, tmavý odstín pleti',
    '🧑🏿‍🤝‍🧑🏻' => 'dva lidé držící se za ruce: tmavý odstín pleti, světlý odstín pleti',
    '🧑🏿‍🤝‍🧑🏼' => 'dva lidé držící se za ruce: tmavý odstín pleti, středně světlý odstín pleti',
    '🧑🏿‍🤝‍🧑🏽' => 'dva lidé držící se za ruce: tmavý odstín pleti, střední odstín pleti',
    '🧑🏿‍🤝‍🧑🏾' => 'dva lidé držící se za ruce: tmavý odstín pleti, středně tmavý odstín pleti',
    '🧑🏿‍🤝‍🧑🏿' => 'dva lidé držící se za ruce: tmavý odstín pleti',
    '👩🏻‍🤝‍👩🏼' => 'dvě ženy držící se za ruce: světlý odstín pleti, středně světlý odstín pleti',
    '👩🏻‍🤝‍👩🏽' => 'dvě ženy držící se za ruce: světlý odstín pleti, střední odstín pleti',
    '👩🏻‍🤝‍👩🏾' => 'dvě ženy držící se za ruce: světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏻‍🤝‍👩🏿' => 'dvě ženy držící se za ruce: světlý odstín pleti, tmavý odstín pleti',
    '👩🏼‍🤝‍👩🏻' => 'dvě ženy držící se za ruce: středně světlý odstín pleti, světlý odstín pleti',
    '👩🏼‍🤝‍👩🏽' => 'dvě ženy držící se za ruce: středně světlý odstín pleti, střední odstín pleti',
    '👩🏼‍🤝‍👩🏾' => 'dvě ženy držící se za ruce: středně světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏼‍🤝‍👩🏿' => 'dvě ženy držící se za ruce: středně světlý odstín pleti, tmavý odstín pleti',
    '👩🏽‍🤝‍👩🏻' => 'dvě ženy držící se za ruce: střední odstín pleti, světlý odstín pleti',
    '👩🏽‍🤝‍👩🏼' => 'dvě ženy držící se za ruce: střední odstín pleti, středně světlý odstín pleti',
    '👩🏽‍🤝‍👩🏾' => 'dvě ženy držící se za ruce: střední odstín pleti, středně tmavý odstín pleti',
    '👩🏽‍🤝‍👩🏿' => 'dvě ženy držící se za ruce: střední odstín pleti, tmavý odstín pleti',
    '👩🏾‍🤝‍👩🏻' => 'dvě ženy držící se za ruce: středně tmavý odstín pleti, světlý odstín pleti',
    '👩🏾‍🤝‍👩🏼' => 'dvě ženy držící se za ruce: středně tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏾‍🤝‍👩🏽' => 'dvě ženy držící se za ruce: středně tmavý odstín pleti, střední odstín pleti',
    '👩🏾‍🤝‍👩🏿' => 'dvě ženy držící se za ruce: středně tmavý odstín pleti, tmavý odstín pleti',
    '👩🏿‍🤝‍👩🏻' => 'dvě ženy držící se za ruce: tmavý odstín pleti, světlý odstín pleti',
    '👩🏿‍🤝‍👩🏼' => 'dvě ženy držící se za ruce: tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏿‍🤝‍👩🏽' => 'dvě ženy držící se za ruce: tmavý odstín pleti, střední odstín pleti',
    '👩🏿‍🤝‍👩🏾' => 'dvě ženy držící se za ruce: tmavý odstín pleti, středně tmavý odstín pleti',
    '👩🏻‍🤝‍👨🏼' => 'muž a žena držící se za ruce: světlý odstín pleti, středně světlý odstín pleti',
    '👩🏻‍🤝‍👨🏽' => 'muž a žena držící se za ruce: světlý odstín pleti, střední odstín pleti',
    '👩🏻‍🤝‍👨🏾' => 'muž a žena držící se za ruce: světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏻‍🤝‍👨🏿' => 'muž a žena držící se za ruce: světlý odstín pleti, tmavý odstín pleti',
    '👩🏼‍🤝‍👨🏻' => 'muž a žena držící se za ruce: středně světlý odstín pleti, světlý odstín pleti',
    '👩🏼‍🤝‍👨🏽' => 'muž a žena držící se za ruce: středně světlý odstín pleti, střední odstín pleti',
    '👩🏼‍🤝‍👨🏾' => 'muž a žena držící se za ruce: středně světlý odstín pleti, středně tmavý odstín pleti',
    '👩🏼‍🤝‍👨🏿' => 'muž a žena držící se za ruce: středně světlý odstín pleti, tmavý odstín pleti',
    '👩🏽‍🤝‍👨🏻' => 'muž a žena držící se za ruce: střední odstín pleti, světlý odstín pleti',
    '👩🏽‍🤝‍👨🏼' => 'muž a žena držící se za ruce: střední odstín pleti, středně světlý odstín pleti',
    '👩🏽‍🤝‍👨🏾' => 'muž a žena držící se za ruce: střední odstín pleti, středně tmavý odstín pleti',
    '👩🏽‍🤝‍👨🏿' => 'muž a žena držící se za ruce: střední odstín pleti, tmavý odstín pleti',
    '👩🏾‍🤝‍👨🏻' => 'muž a žena držící se za ruce: středně tmavý odstín pleti, světlý odstín pleti',
    '👩🏾‍🤝‍👨🏼' => 'muž a žena držící se za ruce: středně tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏾‍🤝‍👨🏽' => 'muž a žena držící se za ruce: středně tmavý odstín pleti, střední odstín pleti',
    '👩🏾‍🤝‍👨🏿' => 'muž a žena držící se za ruce: středně tmavý odstín pleti, tmavý odstín pleti',
    '👩🏿‍🤝‍👨🏻' => 'muž a žena držící se za ruce: tmavý odstín pleti, světlý odstín pleti',
    '👩🏿‍🤝‍👨🏼' => 'muž a žena držící se za ruce: tmavý odstín pleti, středně světlý odstín pleti',
    '👩🏿‍🤝‍👨🏽' => 'muž a žena držící se za ruce: tmavý odstín pleti, střední odstín pleti',
    '👩🏿‍🤝‍👨🏾' => 'muž a žena držící se za ruce: tmavý odstín pleti, středně tmavý odstín pleti',
    '👨🏻‍🤝‍👨🏼' => 'dva muži držící se za ruce: světlý odstín pleti, středně světlý odstín pleti',
    '👨🏻‍🤝‍👨🏽' => 'dva muži držící se za ruce: světlý odstín pleti, střední odstín pleti',
    '👨🏻‍🤝‍👨🏾' => 'dva muži držící se za ruce: světlý odstín pleti, středně tmavý odstín pleti',
    '👨🏻‍🤝‍👨🏿' => 'dva muži držící se za ruce: světlý odstín pleti, tmavý odstín pleti',
    '👨🏼‍🤝‍👨🏻' => 'dva muži držící se za ruce: středně světlý odstín pleti, světlý odstín pleti',
    '👨🏼‍🤝‍👨🏽' => 'dva muži držící se za ruce: středně světlý odstín pleti, střední odstín pleti',
    '👨🏼‍🤝‍👨🏾' => 'dva muži držící se za ruce: středně světlý odstín pleti, středně tmavý odstín pleti',
    '👨🏼‍🤝‍👨🏿' => 'dva muži držící se za ruce: středně světlý odstín pleti, tmavý odstín pleti',
    '👨🏽‍🤝‍👨🏻' => 'dva muži držící se za ruce: střední odstín pleti, světlý odstín pleti',
    '👨🏽‍🤝‍👨🏼' => 'dva muži držící se za ruce: střední odstín pleti, středně světlý odstín pleti',
    '👨🏽‍🤝‍👨🏾' => 'dva muži držící se za ruce: střední odstín pleti, středně tmavý odstín pleti',
    '👨🏽‍🤝‍👨🏿' => 'dva muži držící se za ruce: střední odstín pleti, tmavý odstín pleti',
    '👨🏾‍🤝‍👨🏻' => 'dva muži držící se za ruce: středně tmavý odstín pleti, světlý odstín pleti',
    '👨🏾‍🤝‍👨🏼' => 'dva muži držící se za ruce: středně tmavý odstín pleti, středně světlý odstín pleti',
    '👨🏾‍🤝‍👨🏽' => 'dva muži držící se za ruce: středně tmavý odstín pleti, střední odstín pleti',
    '👨🏾‍🤝‍👨🏿' => 'dva muži držící se za ruce: středně tmavý odstín pleti, tmavý odstín pleti',
    '👨🏿‍🤝‍👨🏻' => 'dva muži držící se za ruce: tmavý odstín pleti, světlý odstín pleti',
    '👨🏿‍🤝‍👨🏼' => 'dva muži držící se za ruce: tmavý odstín pleti, středně světlý odstín pleti',
    '👨🏿‍🤝‍👨🏽' => 'dva muži držící se za ruce: tmavý odstín pleti, střední odstín pleti',
    '👨🏿‍🤝‍👨🏾' => 'dva muži držící se za ruce: tmavý odstín pleti, středně tmavý odstín pleti',
    '👩‍❤‍💋‍👨' => 'polibek: žena, muž',
    '👨‍❤‍💋‍👨' => 'polibek: muž, muž',
    '👩‍❤‍💋‍👩' => 'polibek: žena, žena',
    '👨‍👩‍👧‍👦' => 'rodina: muž, žena, dívka a chlapec',
    '👨‍👩‍👦‍👦' => 'rodina: muž, žena, chlapec a chlapec',
    '👨‍👩‍👧‍👧' => 'rodina: muž, žena, dívka a dívka',
    '👨‍👨‍👧‍👦' => 'rodina: muž, muž, dívka a chlapec',
    '👨‍👨‍👦‍👦' => 'rodina: muž, muž, chlapec a chlapec',
    '👨‍👨‍👧‍👧' => 'rodina: muž, muž, dívka a dívka',
    '👩‍👩‍👧‍👦' => 'rodina: žena, žena, dívka a chlapec',
    '👩‍👩‍👦‍👦' => 'rodina: žena, žena, chlapec a chlapec',
    '👩‍👩‍👧‍👧' => 'rodina: žena, žena, dívka a dívka',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'vlajka: Anglie',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'vlajka: Skotsko',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'vlajka: Wales',
    '🧑‍🧑‍🧒‍🧒' => 'rodina: dospělý, dospělý, dítě, dítě',
    '🚶🏻‍♀‍➡' => 'kráčející žena: světlý odstín pleti, – směr doprava',
    '🚶🏼‍♀‍➡' => 'kráčející žena: středně světlý odstín pleti, – směr doprava',
    '🚶🏽‍♀‍➡' => 'kráčející žena: střední odstín pleti, – směr doprava',
    '🚶🏾‍♀‍➡' => 'kráčející žena: středně tmavý odstín pleti, – směr doprava',
    '🚶🏿‍♀‍➡' => 'kráčející žena: tmavý odstín pleti, – směr doprava',
    '🚶🏻‍♂‍➡' => 'kráčející muž: světlý odstín pleti, – směr doprava',
    '🚶🏼‍♂‍➡' => 'kráčející muž: středně světlý odstín pleti, – směr doprava',
    '🚶🏽‍♂‍➡' => 'kráčející muž: střední odstín pleti, – směr doprava',
    '🚶🏾‍♂‍➡' => 'kráčející muž: středně tmavý odstín pleti, – směr doprava',
    '🚶🏿‍♂‍➡' => 'kráčející muž: tmavý odstín pleti, – směr doprava',
    '🧎🏻‍♀‍➡' => 'klečící žena: světlý odstín pleti, – směr doprava',
    '🧎🏼‍♀‍➡' => 'klečící žena: středně světlý odstín pleti, – směr doprava',
    '🧎🏽‍♀‍➡' => 'klečící žena: střední odstín pleti, – směr doprava',
    '🧎🏾‍♀‍➡' => 'klečící žena: středně tmavý odstín pleti, – směr doprava',
    '🧎🏿‍♀‍➡' => 'klečící žena: tmavý odstín pleti, – směr doprava',
    '🧎🏻‍♂‍➡' => 'klečící muž: světlý odstín pleti, – směr doprava',
    '🧎🏼‍♂‍➡' => 'klečící muž: středně světlý odstín pleti, – směr doprava',
    '🧎🏽‍♂‍➡' => 'klečící muž: střední odstín pleti, – směr doprava',
    '🧎🏾‍♂‍➡' => 'klečící muž: středně tmavý odstín pleti, – směr doprava',
    '🧎🏿‍♂‍➡' => 'klečící muž: tmavý odstín pleti, – směr doprava',
    '🧑🏻‍🦯‍➡' => 'člověk se slepeckou holí: světlý odstín pleti, – směr doprava',
    '🧑🏼‍🦯‍➡' => 'člověk se slepeckou holí: středně světlý odstín pleti, – směr doprava',
    '🧑🏽‍🦯‍➡' => 'člověk se slepeckou holí: střední odstín pleti, – směr doprava',
    '🧑🏾‍🦯‍➡' => 'člověk se slepeckou holí: středně tmavý odstín pleti, – směr doprava',
    '🧑🏿‍🦯‍➡' => 'člověk se slepeckou holí: tmavý odstín pleti, – směr doprava',
    '👨🏻‍🦯‍➡' => 'muž se slepeckou holí: světlý odstín pleti, – směr doprava',
    '👨🏼‍🦯‍➡' => 'muž se slepeckou holí: středně světlý odstín pleti, – směr doprava',
    '👨🏽‍🦯‍➡' => 'muž se slepeckou holí: střední odstín pleti, – směr doprava',
    '👨🏾‍🦯‍➡' => 'muž se slepeckou holí: středně tmavý odstín pleti, – směr doprava',
    '👨🏿‍🦯‍➡' => 'muž se slepeckou holí: tmavý odstín pleti, – směr doprava',
    '👩🏻‍🦯‍➡' => 'žena se slepeckou holí: světlý odstín pleti, – směr doprava',
    '👩🏼‍🦯‍➡' => 'žena se slepeckou holí: středně světlý odstín pleti, – směr doprava',
    '👩🏽‍🦯‍➡' => 'žena se slepeckou holí: střední odstín pleti, – směr doprava',
    '👩🏾‍🦯‍➡' => 'žena se slepeckou holí: středně tmavý odstín pleti, – směr doprava',
    '👩🏿‍🦯‍➡' => 'žena se slepeckou holí: tmavý odstín pleti, – směr doprava',
    '🧑🏻‍🦼‍➡' => 'člověk na elektrickém invalidním vozíku: světlý odstín pleti, – směr doprava',
    '🧑🏼‍🦼‍➡' => 'člověk na elektrickém invalidním vozíku: středně světlý odstín pleti, – směr doprava',
    '🧑🏽‍🦼‍➡' => 'člověk na elektrickém invalidním vozíku: střední odstín pleti, – směr doprava',
    '🧑🏾‍🦼‍➡' => 'člověk na elektrickém invalidním vozíku: středně tmavý odstín pleti, – směr doprava',
    '🧑🏿‍🦼‍➡' => 'člověk na elektrickém invalidním vozíku: tmavý odstín pleti, – směr doprava',
    '👨🏻‍🦼‍➡' => 'muž na elektrickém invalidním vozíku: světlý odstín pleti, – směr doprava',
    '👨🏼‍🦼‍➡' => 'muž na elektrickém invalidním vozíku: středně světlý odstín pleti, – směr doprava',
    '👨🏽‍🦼‍➡' => 'muž na elektrickém invalidním vozíku: střední odstín pleti, – směr doprava',
    '👨🏾‍🦼‍➡' => 'muž na elektrickém invalidním vozíku: středně tmavý odstín pleti, – směr doprava',
    '👨🏿‍🦼‍➡' => 'muž na elektrickém invalidním vozíku: tmavý odstín pleti, – směr doprava',
    '👩🏻‍🦼‍➡' => 'žena na elektrickém invalidním vozíku: světlý odstín pleti, – směr doprava',
    '👩🏼‍🦼‍➡' => 'žena na elektrickém invalidním vozíku: středně světlý odstín pleti, – směr doprava',
    '👩🏽‍🦼‍➡' => 'žena na elektrickém invalidním vozíku: střední odstín pleti, – směr doprava',
    '👩🏾‍🦼‍➡' => 'žena na elektrickém invalidním vozíku: středně tmavý odstín pleti, – směr doprava',
    '👩🏿‍🦼‍➡' => 'žena na elektrickém invalidním vozíku: tmavý odstín pleti, – směr doprava',
    '🧑🏻‍🦽‍➡' => 'člověk na mechanickém invalidním vozíku: světlý odstín pleti, – směr doprava',
    '🧑🏼‍🦽‍➡' => 'člověk na mechanickém invalidním vozíku: středně světlý odstín pleti, – směr doprava',
    '🧑🏽‍🦽‍➡' => 'člověk na mechanickém invalidním vozíku: střední odstín pleti, – směr doprava',
    '🧑🏾‍🦽‍➡' => 'člověk na mechanickém invalidním vozíku: středně tmavý odstín pleti, – směr doprava',
    '🧑🏿‍🦽‍➡' => 'člověk na mechanickém invalidním vozíku: tmavý odstín pleti, – směr doprava',
    '👨🏻‍🦽‍➡' => 'muž na mechanickém invalidním vozíku: světlý odstín pleti, – směr doprava',
    '👨🏼‍🦽‍➡' => 'muž na mechanickém invalidním vozíku: středně světlý odstín pleti, – směr doprava',
    '👨🏽‍🦽‍➡' => 'muž na mechanickém invalidním vozíku: střední odstín pleti, – směr doprava',
    '👨🏾‍🦽‍➡' => 'muž na mechanickém invalidním vozíku: středně tmavý odstín pleti, – směr doprava',
    '👨🏿‍🦽‍➡' => 'muž na mechanickém invalidním vozíku: tmavý odstín pleti, – směr doprava',
    '👩🏻‍🦽‍➡' => 'žena na mechanickém invalidním vozíku: světlý odstín pleti, – směr doprava',
    '👩🏼‍🦽‍➡' => 'žena na mechanickém invalidním vozíku: středně světlý odstín pleti, – směr doprava',
    '👩🏽‍🦽‍➡' => 'žena na mechanickém invalidním vozíku: střední odstín pleti, – směr doprava',
    '👩🏾‍🦽‍➡' => 'žena na mechanickém invalidním vozíku: středně tmavý odstín pleti, – směr doprava',
    '👩🏿‍🦽‍➡' => 'žena na mechanickém invalidním vozíku: tmavý odstín pleti, – směr doprava',
    '🏃🏻‍♀‍➡' => 'běžící žena: světlý odstín pleti, – směr doprava',
    '🏃🏼‍♀‍➡' => 'běžící žena: středně světlý odstín pleti, – směr doprava',
    '🏃🏽‍♀‍➡' => 'běžící žena: střední odstín pleti, – směr doprava',
    '🏃🏾‍♀‍➡' => 'běžící žena: středně tmavý odstín pleti, – směr doprava',
    '🏃🏿‍♀‍➡' => 'běžící žena: tmavý odstín pleti, – směr doprava',
    '🏃🏻‍♂‍➡' => 'běžící muž: světlý odstín pleti, – směr doprava',
    '🏃🏼‍♂‍➡' => 'běžící muž: středně světlý odstín pleti, – směr doprava',
    '🏃🏽‍♂‍➡' => 'běžící muž: střední odstín pleti, – směr doprava',
    '🏃🏾‍♂‍➡' => 'běžící muž: středně tmavý odstín pleti, – směr doprava',
    '🏃🏿‍♂‍➡' => 'běžící muž: tmavý odstín pleti, – směr doprava',
    '🫱🏻‍🫲🏼' => 'stisk rukou: světlý odstín pleti, středně světlý odstín pleti',
    '🫱🏻‍🫲🏽' => 'stisk rukou: světlý odstín pleti, střední odstín pleti',
    '🫱🏻‍🫲🏾' => 'stisk rukou: světlý odstín pleti, středně tmavý odstín pleti',
    '🫱🏻‍🫲🏿' => 'stisk rukou: světlý odstín pleti, tmavý odstín pleti',
    '🫱🏼‍🫲🏻' => 'stisk rukou: středně světlý odstín pleti, světlý odstín pleti',
    '🫱🏼‍🫲🏽' => 'stisk rukou: středně světlý odstín pleti, střední odstín pleti',
    '🫱🏼‍🫲🏾' => 'stisk rukou: středně světlý odstín pleti, středně tmavý odstín pleti',
    '🫱🏼‍🫲🏿' => 'stisk rukou: středně světlý odstín pleti, tmavý odstín pleti',
    '🫱🏽‍🫲🏻' => 'stisk rukou: střední odstín pleti, světlý odstín pleti',
    '🫱🏽‍🫲🏼' => 'stisk rukou: střední odstín pleti, středně světlý odstín pleti',
    '🫱🏽‍🫲🏾' => 'stisk rukou: střední odstín pleti, středně tmavý odstín pleti',
    '🫱🏽‍🫲🏿' => 'stisk rukou: střední odstín pleti, tmavý odstín pleti',
    '🫱🏾‍🫲🏻' => 'stisk rukou: středně tmavý odstín pleti, světlý odstín pleti',
    '🫱🏾‍🫲🏼' => 'stisk rukou: středně tmavý odstín pleti, středně světlý odstín pleti',
    '🫱🏾‍🫲🏽' => 'stisk rukou: středně tmavý odstín pleti, střední odstín pleti',
    '🫱🏾‍🫲🏿' => 'stisk rukou: středně tmavý odstín pleti, tmavý odstín pleti',
    '🫱🏿‍🫲🏻' => 'stisk rukou: tmavý odstín pleti, světlý odstín pleti',
    '🫱🏿‍🫲🏼' => 'stisk rukou: tmavý odstín pleti, středně světlý odstín pleti',
    '🫱🏿‍🫲🏽' => 'stisk rukou: tmavý odstín pleti, střední odstín pleti',
    '🫱🏿‍🫲🏾' => 'stisk rukou: tmavý odstín pleti, středně tmavý odstín pleti',
    '🚶‍♀‍➡' => 'kráčející žena: – směr doprava',
    '🚶‍♂‍➡' => 'kráčející muž: – směr doprava',
    '🧎‍♀‍➡' => 'klečící žena: – směr doprava',
    '🧎‍♂‍➡' => 'klečící muž: – směr doprava',
    '🧑‍🦯‍➡' => 'člověk se slepeckou holí: – směr doprava',
    '👨‍🦯‍➡' => 'muž se slepeckou holí: – směr doprava',
    '👩‍🦯‍➡' => 'žena se slepeckou holí: – směr doprava',
    '🧑‍🦼‍➡' => 'člověk na elektrickém invalidním vozíku: – směr doprava',
    '👨‍🦼‍➡' => 'muž na elektrickém invalidním vozíku: – směr doprava',
    '👩‍🦼‍➡' => 'žena na elektrickém invalidním vozíku: – směr doprava',
    '🧑‍🦽‍➡' => 'člověk na mechanickém invalidním vozíku: – směr doprava',
    '👨‍🦽‍➡' => 'muž na mechanickém invalidním vozíku: – směr doprava',
    '👩‍🦽‍➡' => 'žena na mechanickém invalidním vozíku: – směr doprava',
    '🏃‍♀‍➡' => 'běžící žena: – směr doprava',
    '🏃‍♂‍➡' => 'běžící muž: – směr doprava',
    '👩‍❤‍👨' => 'pár se srdcem: žena, muž',
    '👨‍❤‍👨' => 'pár se srdcem: muž, muž',
    '👩‍❤‍👩' => 'pár se srdcem: žena, žena',
    '👨‍👩‍👦' => 'rodina: muž, žena a chlapec',
    '👨‍👩‍👧' => 'rodina: muž, žena a dívka',
    '👨‍👨‍👦' => 'rodina: muž, muž a chlapec',
    '👨‍👨‍👧' => 'rodina: muž, muž a dívka',
    '👩‍👩‍👦' => 'rodina: žena, žena a chlapec',
    '👩‍👩‍👧' => 'rodina: žena, žena a dívka',
    '👨‍👦‍👦' => 'rodina: muž, chlapec a chlapec',
    '👨‍👧‍👦' => 'rodina: muž, dívka a chlapec',
    '👨‍👧‍👧' => 'rodina: muž, dívka a dívka',
    '👩‍👦‍👦' => 'rodina: žena, chlapec a chlapec',
    '👩‍👧‍👦' => 'rodina: žena, dívka a chlapec',
    '👩‍👧‍👧' => 'rodina: žena, dívka a dívka',
    '🧑‍🤝‍🧑' => 'dva lidé držící se za ruce',
    '🧑‍🧑‍🧒' => 'rodina: dospělý, dospělý, dítě',
    '🧑‍🧒‍🧒' => 'rodina: dospělý, dítě, dítě',
    '🧑🏻‍🦰' => 'dospělý člověk: světlý odstín pleti, zrzavé vlasy',
    '🧑🏼‍🦰' => 'dospělý člověk: středně světlý odstín pleti, zrzavé vlasy',
    '🧑🏽‍🦰' => 'dospělý člověk: střední odstín pleti, zrzavé vlasy',
    '🧑🏾‍🦰' => 'dospělý člověk: středně tmavý odstín pleti, zrzavé vlasy',
    '🧑🏿‍🦰' => 'dospělý člověk: tmavý odstín pleti, zrzavé vlasy',
    '🧑🏻‍🦱' => 'dospělý člověk: světlý odstín pleti, kudrnaté vlasy',
    '🧑🏼‍🦱' => 'dospělý člověk: středně světlý odstín pleti, kudrnaté vlasy',
    '🧑🏽‍🦱' => 'dospělý člověk: střední odstín pleti, kudrnaté vlasy',
    '🧑🏾‍🦱' => 'dospělý člověk: středně tmavý odstín pleti, kudrnaté vlasy',
    '🧑🏿‍🦱' => 'dospělý člověk: tmavý odstín pleti, kudrnaté vlasy',
    '🧑🏻‍🦳' => 'dospělý člověk: světlý odstín pleti, bílé vlasy',
    '🧑🏼‍🦳' => 'dospělý člověk: středně světlý odstín pleti, bílé vlasy',
    '🧑🏽‍🦳' => 'dospělý člověk: střední odstín pleti, bílé vlasy',
    '🧑🏾‍🦳' => 'dospělý člověk: středně tmavý odstín pleti, bílé vlasy',
    '🧑🏿‍🦳' => 'dospělý člověk: tmavý odstín pleti, bílé vlasy',
    '🧑🏻‍🦲' => 'dospělý člověk: světlý odstín pleti, bez vlasů',
    '🧑🏼‍🦲' => 'dospělý člověk: středně světlý odstín pleti, bez vlasů',
    '🧑🏽‍🦲' => 'dospělý člověk: střední odstín pleti, bez vlasů',
    '🧑🏾‍🦲' => 'dospělý člověk: středně tmavý odstín pleti, bez vlasů',
    '🧑🏿‍🦲' => 'dospělý člověk: tmavý odstín pleti, bez vlasů',
    '🧑🏻‍🩰' => 'baletní tanečník: světlý odstín pleti',
    '🧑🏼‍🩰' => 'baletní tanečník: středně světlý odstín pleti',
    '🧑🏽‍🩰' => 'baletní tanečník: střední odstín pleti',
    '🧑🏾‍🩰' => 'baletní tanečník: středně tmavý odstín pleti',
    '🧑🏿‍🩰' => 'baletní tanečník: tmavý odstín pleti',
    '🧔🏻‍♂' => 'muž s plnovousem: světlý odstín pleti',
    '🧔🏼‍♂' => 'muž s plnovousem: středně světlý odstín pleti',
    '🧔🏽‍♂' => 'muž s plnovousem: střední odstín pleti',
    '🧔🏾‍♂' => 'muž s plnovousem: středně tmavý odstín pleti',
    '🧔🏿‍♂' => 'muž s plnovousem: tmavý odstín pleti',
    '👱🏻‍♂' => 'blonďák: světlý odstín pleti',
    '👱🏼‍♂' => 'blonďák: středně světlý odstín pleti',
    '👱🏽‍♂' => 'blonďák: střední odstín pleti',
    '👱🏾‍♂' => 'blonďák: středně tmavý odstín pleti',
    '👱🏿‍♂' => 'blonďák: tmavý odstín pleti',
    '👨🏻‍🦰' => 'muž: světlý odstín pleti, zrzavé vlasy',
    '👨🏼‍🦰' => 'muž: středně světlý odstín pleti, zrzavé vlasy',
    '👨🏽‍🦰' => 'muž: střední odstín pleti, zrzavé vlasy',
    '👨🏾‍🦰' => 'muž: středně tmavý odstín pleti, zrzavé vlasy',
    '👨🏿‍🦰' => 'muž: tmavý odstín pleti, zrzavé vlasy',
    '👨🏻‍🦱' => 'muž: světlý odstín pleti, kudrnaté vlasy',
    '👨🏼‍🦱' => 'muž: středně světlý odstín pleti, kudrnaté vlasy',
    '👨🏽‍🦱' => 'muž: střední odstín pleti, kudrnaté vlasy',
    '👨🏾‍🦱' => 'muž: středně tmavý odstín pleti, kudrnaté vlasy',
    '👨🏿‍🦱' => 'muž: tmavý odstín pleti, kudrnaté vlasy',
    '👨🏻‍🦳' => 'muž: světlý odstín pleti, bílé vlasy',
    '👨🏼‍🦳' => 'muž: středně světlý odstín pleti, bílé vlasy',
    '👨🏽‍🦳' => 'muž: střední odstín pleti, bílé vlasy',
    '👨🏾‍🦳' => 'muž: středně tmavý odstín pleti, bílé vlasy',
    '👨🏿‍🦳' => 'muž: tmavý odstín pleti, bílé vlasy',
    '👨🏻‍🦲' => 'muž: světlý odstín pleti, bez vlasů',
    '👨🏼‍🦲' => 'muž: středně světlý odstín pleti, bez vlasů',
    '👨🏽‍🦲' => 'muž: střední odstín pleti, bez vlasů',
    '👨🏾‍🦲' => 'muž: středně tmavý odstín pleti, bez vlasů',
    '👨🏿‍🦲' => 'muž: tmavý odstín pleti, bez vlasů',
    '🧔🏻‍♀' => 'žena s plnovousem: světlý odstín pleti',
    '🧔🏼‍♀' => 'žena s plnovousem: středně světlý odstín pleti',
    '🧔🏽‍♀' => 'žena s plnovousem: střední odstín pleti',
    '🧔🏾‍♀' => 'žena s plnovousem: středně tmavý odstín pleti',
    '🧔🏿‍♀' => 'žena s plnovousem: tmavý odstín pleti',
    '👱🏻‍♀' => 'blondýna: světlý odstín pleti',
    '👱🏼‍♀' => 'blondýna: středně světlý odstín pleti',
    '👱🏽‍♀' => 'blondýna: střední odstín pleti',
    '👱🏾‍♀' => 'blondýna: středně tmavý odstín pleti',
    '👱🏿‍♀' => 'blondýna: tmavý odstín pleti',
    '👩🏻‍🦰' => 'žena: světlý odstín pleti, zrzavé vlasy',
    '👩🏼‍🦰' => 'žena: středně světlý odstín pleti, zrzavé vlasy',
    '👩🏽‍🦰' => 'žena: střední odstín pleti, zrzavé vlasy',
    '👩🏾‍🦰' => 'žena: středně tmavý odstín pleti, zrzavé vlasy',
    '👩🏿‍🦰' => 'žena: tmavý odstín pleti, zrzavé vlasy',
    '👩🏻‍🦱' => 'žena: světlý odstín pleti, kudrnaté vlasy',
    '👩🏼‍🦱' => 'žena: středně světlý odstín pleti, kudrnaté vlasy',
    '👩🏽‍🦱' => 'žena: střední odstín pleti, kudrnaté vlasy',
    '👩🏾‍🦱' => 'žena: středně tmavý odstín pleti, kudrnaté vlasy',
    '👩🏿‍🦱' => 'žena: tmavý odstín pleti, kudrnaté vlasy',
    '👩🏻‍🦳' => 'žena: světlý odstín pleti, bílé vlasy',
    '👩🏼‍🦳' => 'žena: středně světlý odstín pleti, bílé vlasy',
    '👩🏽‍🦳' => 'žena: střední odstín pleti, bílé vlasy',
    '👩🏾‍🦳' => 'žena: středně tmavý odstín pleti, bílé vlasy',
    '👩🏿‍🦳' => 'žena: tmavý odstín pleti, bílé vlasy',
    '👩🏻‍🦲' => 'žena: světlý odstín pleti, bez vlasů',
    '👩🏼‍🦲' => 'žena: středně světlý odstín pleti, bez vlasů',
    '👩🏽‍🦲' => 'žena: střední odstín pleti, bez vlasů',
    '👩🏾‍🦲' => 'žena: středně tmavý odstín pleti, bez vlasů',
    '👩🏿‍🦲' => 'žena: tmavý odstín pleti, bez vlasů',
    '🙍🏻‍♂' => 'zamračený muž: světlý odstín pleti',
    '🙍🏼‍♂' => 'zamračený muž: středně světlý odstín pleti',
    '🙍🏽‍♂' => 'zamračený muž: střední odstín pleti',
    '🙍🏾‍♂' => 'zamračený muž: středně tmavý odstín pleti',
    '🙍🏿‍♂' => 'zamračený muž: tmavý odstín pleti',
    '🙍🏻‍♀' => 'zamračená žena: světlý odstín pleti',
    '🙍🏼‍♀' => 'zamračená žena: středně světlý odstín pleti',
    '🙍🏽‍♀' => 'zamračená žena: střední odstín pleti',
    '🙍🏾‍♀' => 'zamračená žena: středně tmavý odstín pleti',
    '🙍🏿‍♀' => 'zamračená žena: tmavý odstín pleti',
    '🙎🏻‍♂' => 'podrážděný muž: světlý odstín pleti',
    '🙎🏼‍♂' => 'podrážděný muž: středně světlý odstín pleti',
    '🙎🏽‍♂' => 'podrážděný muž: střední odstín pleti',
    '🙎🏾‍♂' => 'podrážděný muž: středně tmavý odstín pleti',
    '🙎🏿‍♂' => 'podrážděný muž: tmavý odstín pleti',
    '🙎🏻‍♀' => 'podrážděná žena: světlý odstín pleti',
    '🙎🏼‍♀' => 'podrážděná žena: středně světlý odstín pleti',
    '🙎🏽‍♀' => 'podrážděná žena: střední odstín pleti',
    '🙎🏾‍♀' => 'podrážděná žena: středně tmavý odstín pleti',
    '🙎🏿‍♀' => 'podrážděná žena: tmavý odstín pleti',
    '🙅🏻‍♂' => 'muž s gestem nesouhlasu: světlý odstín pleti',
    '🙅🏼‍♂' => 'muž s gestem nesouhlasu: středně světlý odstín pleti',
    '🙅🏽‍♂' => 'muž s gestem nesouhlasu: střední odstín pleti',
    '🙅🏾‍♂' => 'muž s gestem nesouhlasu: středně tmavý odstín pleti',
    '🙅🏿‍♂' => 'muž s gestem nesouhlasu: tmavý odstín pleti',
    '🙅🏻‍♀' => 'žena s gestem nesouhlasu: světlý odstín pleti',
    '🙅🏼‍♀' => 'žena s gestem nesouhlasu: středně světlý odstín pleti',
    '🙅🏽‍♀' => 'žena s gestem nesouhlasu: střední odstín pleti',
    '🙅🏾‍♀' => 'žena s gestem nesouhlasu: středně tmavý odstín pleti',
    '🙅🏿‍♀' => 'žena s gestem nesouhlasu: tmavý odstín pleti',
    '🙆🏻‍♂' => 'muž s gestem souhlasu: světlý odstín pleti',
    '🙆🏼‍♂' => 'muž s gestem souhlasu: středně světlý odstín pleti',
    '🙆🏽‍♂' => 'muž s gestem souhlasu: střední odstín pleti',
    '🙆🏾‍♂' => 'muž s gestem souhlasu: středně tmavý odstín pleti',
    '🙆🏿‍♂' => 'muž s gestem souhlasu: tmavý odstín pleti',
    '🙆🏻‍♀' => 'žena s gestem souhlasu: světlý odstín pleti',
    '🙆🏼‍♀' => 'žena s gestem souhlasu: středně světlý odstín pleti',
    '🙆🏽‍♀' => 'žena s gestem souhlasu: střední odstín pleti',
    '🙆🏾‍♀' => 'žena s gestem souhlasu: středně tmavý odstín pleti',
    '🙆🏿‍♀' => 'žena s gestem souhlasu: tmavý odstín pleti',
    '💁🏻‍♂' => 'muž s dlaní vzhůru: světlý odstín pleti',
    '💁🏼‍♂' => 'muž s dlaní vzhůru: středně světlý odstín pleti',
    '💁🏽‍♂' => 'muž s dlaní vzhůru: střední odstín pleti',
    '💁🏾‍♂' => 'muž s dlaní vzhůru: středně tmavý odstín pleti',
    '💁🏿‍♂' => 'muž s dlaní vzhůru: tmavý odstín pleti',
    '💁🏻‍♀' => 'žena s dlaní vzhůru: světlý odstín pleti',
    '💁🏼‍♀' => 'žena s dlaní vzhůru: středně světlý odstín pleti',
    '💁🏽‍♀' => 'žena s dlaní vzhůru: střední odstín pleti',
    '💁🏾‍♀' => 'žena s dlaní vzhůru: středně tmavý odstín pleti',
    '💁🏿‍♀' => 'žena s dlaní vzhůru: tmavý odstín pleti',
    '🙋🏻‍♂' => 'muž se zvednutou rukou: světlý odstín pleti',
    '🙋🏼‍♂' => 'muž se zvednutou rukou: středně světlý odstín pleti',
    '🙋🏽‍♂' => 'muž se zvednutou rukou: střední odstín pleti',
    '🙋🏾‍♂' => 'muž se zvednutou rukou: středně tmavý odstín pleti',
    '🙋🏿‍♂' => 'muž se zvednutou rukou: tmavý odstín pleti',
    '🙋🏻‍♀' => 'žena se zvednutou rukou: světlý odstín pleti',
    '🙋🏼‍♀' => 'žena se zvednutou rukou: středně světlý odstín pleti',
    '🙋🏽‍♀' => 'žena se zvednutou rukou: střední odstín pleti',
    '🙋🏾‍♀' => 'žena se zvednutou rukou: středně tmavý odstín pleti',
    '🙋🏿‍♀' => 'žena se zvednutou rukou: tmavý odstín pleti',
    '🧏🏻‍♂' => 'hluchý muž: světlý odstín pleti',
    '🧏🏼‍♂' => 'hluchý muž: středně světlý odstín pleti',
    '🧏🏽‍♂' => 'hluchý muž: střední odstín pleti',
    '🧏🏾‍♂' => 'hluchý muž: středně tmavý odstín pleti',
    '🧏🏿‍♂' => 'hluchý muž: tmavý odstín pleti',
    '🧏🏻‍♀' => 'hluchá žena: světlý odstín pleti',
    '🧏🏼‍♀' => 'hluchá žena: středně světlý odstín pleti',
    '🧏🏽‍♀' => 'hluchá žena: střední odstín pleti',
    '🧏🏾‍♀' => 'hluchá žena: středně tmavý odstín pleti',
    '🧏🏿‍♀' => 'hluchá žena: tmavý odstín pleti',
    '🙇🏻‍♂' => 'uklánějící se muž: světlý odstín pleti',
    '🙇🏼‍♂' => 'uklánějící se muž: středně světlý odstín pleti',
    '🙇🏽‍♂' => 'uklánějící se muž: střední odstín pleti',
    '🙇🏾‍♂' => 'uklánějící se muž: středně tmavý odstín pleti',
    '🙇🏿‍♂' => 'uklánějící se muž: tmavý odstín pleti',
    '🙇🏻‍♀' => 'uklánějící se žena: světlý odstín pleti',
    '🙇🏼‍♀' => 'uklánějící se žena: středně světlý odstín pleti',
    '🙇🏽‍♀' => 'uklánějící se žena: střední odstín pleti',
    '🙇🏾‍♀' => 'uklánějící se žena: středně tmavý odstín pleti',
    '🙇🏿‍♀' => 'uklánějící se žena: tmavý odstín pleti',
    '🤦🏻‍♂' => 'muž chytající se za hlavu: světlý odstín pleti',
    '🤦🏼‍♂' => 'muž chytající se za hlavu: středně světlý odstín pleti',
    '🤦🏽‍♂' => 'muž chytající se za hlavu: střední odstín pleti',
    '🤦🏾‍♂' => 'muž chytající se za hlavu: středně tmavý odstín pleti',
    '🤦🏿‍♂' => 'muž chytající se za hlavu: tmavý odstín pleti',
    '🤦🏻‍♀' => 'žena chytající se za hlavu: světlý odstín pleti',
    '🤦🏼‍♀' => 'žena chytající se za hlavu: středně světlý odstín pleti',
    '🤦🏽‍♀' => 'žena chytající se za hlavu: střední odstín pleti',
    '🤦🏾‍♀' => 'žena chytající se za hlavu: středně tmavý odstín pleti',
    '🤦🏿‍♀' => 'žena chytající se za hlavu: tmavý odstín pleti',
    '🤷🏻‍♂' => 'muž krčící rameny: světlý odstín pleti',
    '🤷🏼‍♂' => 'muž krčící rameny: středně světlý odstín pleti',
    '🤷🏽‍♂' => 'muž krčící rameny: střední odstín pleti',
    '🤷🏾‍♂' => 'muž krčící rameny: středně tmavý odstín pleti',
    '🤷🏿‍♂' => 'muž krčící rameny: tmavý odstín pleti',
    '🤷🏻‍♀' => 'žena krčící rameny: světlý odstín pleti',
    '🤷🏼‍♀' => 'žena krčící rameny: středně světlý odstín pleti',
    '🤷🏽‍♀' => 'žena krčící rameny: střední odstín pleti',
    '🤷🏾‍♀' => 'žena krčící rameny: středně tmavý odstín pleti',
    '🤷🏿‍♀' => 'žena krčící rameny: tmavý odstín pleti',
    '🧑🏻‍⚕' => 'zdravotník/zdravotnice: světlý odstín pleti',
    '🧑🏼‍⚕' => 'zdravotník/zdravotnice: středně světlý odstín pleti',
    '🧑🏽‍⚕' => 'zdravotník/zdravotnice: střední odstín pleti',
    '🧑🏾‍⚕' => 'zdravotník/zdravotnice: středně tmavý odstín pleti',
    '🧑🏿‍⚕' => 'zdravotník/zdravotnice: tmavý odstín pleti',
    '👨🏻‍⚕' => 'zdravotník: světlý odstín pleti',
    '👨🏼‍⚕' => 'zdravotník: středně světlý odstín pleti',
    '👨🏽‍⚕' => 'zdravotník: střední odstín pleti',
    '👨🏾‍⚕' => 'zdravotník: středně tmavý odstín pleti',
    '👨🏿‍⚕' => 'zdravotník: tmavý odstín pleti',
    '👩🏻‍⚕' => 'zdravotnice: světlý odstín pleti',
    '👩🏼‍⚕' => 'zdravotnice: středně světlý odstín pleti',
    '👩🏽‍⚕' => 'zdravotnice: střední odstín pleti',
    '👩🏾‍⚕' => 'zdravotnice: středně tmavý odstín pleti',
    '👩🏿‍⚕' => 'zdravotnice: tmavý odstín pleti',
    '🧑🏻‍🎓' => 'studující: světlý odstín pleti',
    '🧑🏼‍🎓' => 'studující: středně světlý odstín pleti',
    '🧑🏽‍🎓' => 'studující: střední odstín pleti',
    '🧑🏾‍🎓' => 'studující: středně tmavý odstín pleti',
    '🧑🏿‍🎓' => 'studující: tmavý odstín pleti',
    '👨🏻‍🎓' => 'student: světlý odstín pleti',
    '👨🏼‍🎓' => 'student: středně světlý odstín pleti',
    '👨🏽‍🎓' => 'student: střední odstín pleti',
    '👨🏾‍🎓' => 'student: středně tmavý odstín pleti',
    '👨🏿‍🎓' => 'student: tmavý odstín pleti',
    '👩🏻‍🎓' => 'studentka: světlý odstín pleti',
    '👩🏼‍🎓' => 'studentka: středně světlý odstín pleti',
    '👩🏽‍🎓' => 'studentka: střední odstín pleti',
    '👩🏾‍🎓' => 'studentka: středně tmavý odstín pleti',
    '👩🏿‍🎓' => 'studentka: tmavý odstín pleti',
    '🧑🏻‍🏫' => 'vyučující: světlý odstín pleti',
    '🧑🏼‍🏫' => 'vyučující: středně světlý odstín pleti',
    '🧑🏽‍🏫' => 'vyučující: střední odstín pleti',
    '🧑🏾‍🏫' => 'vyučující: středně tmavý odstín pleti',
    '🧑🏿‍🏫' => 'vyučující: tmavý odstín pleti',
    '👨🏻‍🏫' => 'učitel: světlý odstín pleti',
    '👨🏼‍🏫' => 'učitel: středně světlý odstín pleti',
    '👨🏽‍🏫' => 'učitel: střední odstín pleti',
    '👨🏾‍🏫' => 'učitel: středně tmavý odstín pleti',
    '👨🏿‍🏫' => 'učitel: tmavý odstín pleti',
    '👩🏻‍🏫' => 'učitelka: světlý odstín pleti',
    '👩🏼‍🏫' => 'učitelka: středně světlý odstín pleti',
    '👩🏽‍🏫' => 'učitelka: střední odstín pleti',
    '👩🏾‍🏫' => 'učitelka: středně tmavý odstín pleti',
    '👩🏿‍🏫' => 'učitelka: tmavý odstín pleti',
    '🧑🏻‍⚖' => 'soudce/soudkyně: světlý odstín pleti',
    '🧑🏼‍⚖' => 'soudce/soudkyně: středně světlý odstín pleti',
    '🧑🏽‍⚖' => 'soudce/soudkyně: střední odstín pleti',
    '🧑🏾‍⚖' => 'soudce/soudkyně: středně tmavý odstín pleti',
    '🧑🏿‍⚖' => 'soudce/soudkyně: tmavý odstín pleti',
    '👨🏻‍⚖' => 'soudce: světlý odstín pleti',
    '👨🏼‍⚖' => 'soudce: středně světlý odstín pleti',
    '👨🏽‍⚖' => 'soudce: střední odstín pleti',
    '👨🏾‍⚖' => 'soudce: středně tmavý odstín pleti',
    '👨🏿‍⚖' => 'soudce: tmavý odstín pleti',
    '👩🏻‍⚖' => 'soudkyně: světlý odstín pleti',
    '👩🏼‍⚖' => 'soudkyně: středně světlý odstín pleti',
    '👩🏽‍⚖' => 'soudkyně: střední odstín pleti',
    '👩🏾‍⚖' => 'soudkyně: středně tmavý odstín pleti',
    '👩🏿‍⚖' => 'soudkyně: tmavý odstín pleti',
    '🧑🏻‍🌾' => 'farmář/farmářka: světlý odstín pleti',
    '🧑🏼‍🌾' => 'farmář/farmářka: středně světlý odstín pleti',
    '🧑🏽‍🌾' => 'farmář/farmářka: střední odstín pleti',
    '🧑🏾‍🌾' => 'farmář/farmářka: středně tmavý odstín pleti',
    '🧑🏿‍🌾' => 'farmář/farmářka: tmavý odstín pleti',
    '👨🏻‍🌾' => 'farmář: světlý odstín pleti',
    '👨🏼‍🌾' => 'farmář: středně světlý odstín pleti',
    '👨🏽‍🌾' => 'farmář: střední odstín pleti',
    '👨🏾‍🌾' => 'farmář: středně tmavý odstín pleti',
    '👨🏿‍🌾' => 'farmář: tmavý odstín pleti',
    '👩🏻‍🌾' => 'farmářka: světlý odstín pleti',
    '👩🏼‍🌾' => 'farmářka: středně světlý odstín pleti',
    '👩🏽‍🌾' => 'farmářka: střední odstín pleti',
    '👩🏾‍🌾' => 'farmářka: středně tmavý odstín pleti',
    '👩🏿‍🌾' => 'farmářka: tmavý odstín pleti',
    '🧑🏻‍🍳' => 'kuchař/kuchařka: světlý odstín pleti',
    '🧑🏼‍🍳' => 'kuchař/kuchařka: středně světlý odstín pleti',
    '🧑🏽‍🍳' => 'kuchař/kuchařka: střední odstín pleti',
    '🧑🏾‍🍳' => 'kuchař/kuchařka: středně tmavý odstín pleti',
    '🧑🏿‍🍳' => 'kuchař/kuchařka: tmavý odstín pleti',
    '👨🏻‍🍳' => 'kuchař: světlý odstín pleti',
    '👨🏼‍🍳' => 'kuchař: středně světlý odstín pleti',
    '👨🏽‍🍳' => 'kuchař: střední odstín pleti',
    '👨🏾‍🍳' => 'kuchař: středně tmavý odstín pleti',
    '👨🏿‍🍳' => 'kuchař: tmavý odstín pleti',
    '👩🏻‍🍳' => 'kuchařka: světlý odstín pleti',
    '👩🏼‍🍳' => 'kuchařka: středně světlý odstín pleti',
    '👩🏽‍🍳' => 'kuchařka: střední odstín pleti',
    '👩🏾‍🍳' => 'kuchařka: středně tmavý odstín pleti',
    '👩🏿‍🍳' => 'kuchařka: tmavý odstín pleti',
    '🧑🏻‍🔧' => 'mechanik/mechanička: světlý odstín pleti',
    '🧑🏼‍🔧' => 'mechanik/mechanička: středně světlý odstín pleti',
    '🧑🏽‍🔧' => 'mechanik/mechanička: střední odstín pleti',
    '🧑🏾‍🔧' => 'mechanik/mechanička: středně tmavý odstín pleti',
    '🧑🏿‍🔧' => 'mechanik/mechanička: tmavý odstín pleti',
    '👨🏻‍🔧' => 'mechanik: světlý odstín pleti',
    '👨🏼‍🔧' => 'mechanik: středně světlý odstín pleti',
    '👨🏽‍🔧' => 'mechanik: střední odstín pleti',
    '👨🏾‍🔧' => 'mechanik: středně tmavý odstín pleti',
    '👨🏿‍🔧' => 'mechanik: tmavý odstín pleti',
    '👩🏻‍🔧' => 'mechanička: světlý odstín pleti',
    '👩🏼‍🔧' => 'mechanička: středně světlý odstín pleti',
    '👩🏽‍🔧' => 'mechanička: střední odstín pleti',
    '👩🏾‍🔧' => 'mechanička: středně tmavý odstín pleti',
    '👩🏿‍🔧' => 'mechanička: tmavý odstín pleti',
    '🧑🏻‍🏭' => 'osoba pracující v továrně: světlý odstín pleti',
    '🧑🏼‍🏭' => 'osoba pracující v továrně: středně světlý odstín pleti',
    '🧑🏽‍🏭' => 'osoba pracující v továrně: střední odstín pleti',
    '🧑🏾‍🏭' => 'osoba pracující v továrně: středně tmavý odstín pleti',
    '🧑🏿‍🏭' => 'osoba pracující v továrně: tmavý odstín pleti',
    '👨🏻‍🏭' => 'dělník v továrně: světlý odstín pleti',
    '👨🏼‍🏭' => 'dělník v továrně: středně světlý odstín pleti',
    '👨🏽‍🏭' => 'dělník v továrně: střední odstín pleti',
    '👨🏾‍🏭' => 'dělník v továrně: středně tmavý odstín pleti',
    '👨🏿‍🏭' => 'dělník v továrně: tmavý odstín pleti',
    '👩🏻‍🏭' => 'dělnice v továrně: světlý odstín pleti',
    '👩🏼‍🏭' => 'dělnice v továrně: středně světlý odstín pleti',
    '👩🏽‍🏭' => 'dělnice v továrně: střední odstín pleti',
    '👩🏾‍🏭' => 'dělnice v továrně: středně tmavý odstín pleti',
    '👩🏿‍🏭' => 'dělnice v továrně: tmavý odstín pleti',
    '🧑🏻‍💼' => 'manažer/manažerka: světlý odstín pleti',
    '🧑🏼‍💼' => 'manažer/manažerka: středně světlý odstín pleti',
    '🧑🏽‍💼' => 'manažer/manažerka: střední odstín pleti',
    '🧑🏾‍💼' => 'manažer/manažerka: středně tmavý odstín pleti',
    '🧑🏿‍💼' => 'manažer/manažerka: tmavý odstín pleti',
    '👨🏻‍💼' => 'manažer: světlý odstín pleti',
    '👨🏼‍💼' => 'manažer: středně světlý odstín pleti',
    '👨🏽‍💼' => 'manažer: střední odstín pleti',
    '👨🏾‍💼' => 'manažer: středně tmavý odstín pleti',
    '👨🏿‍💼' => 'manažer: tmavý odstín pleti',
    '👩🏻‍💼' => 'manažerka: světlý odstín pleti',
    '👩🏼‍💼' => 'manažerka: středně světlý odstín pleti',
    '👩🏽‍💼' => 'manažerka: střední odstín pleti',
    '👩🏾‍💼' => 'manažerka: středně tmavý odstín pleti',
    '👩🏿‍💼' => 'manažerka: tmavý odstín pleti',
    '🧑🏻‍🔬' => 'vědec/vědkyně: světlý odstín pleti',
    '🧑🏼‍🔬' => 'vědec/vědkyně: středně světlý odstín pleti',
    '🧑🏽‍🔬' => 'vědec/vědkyně: střední odstín pleti',
    '🧑🏾‍🔬' => 'vědec/vědkyně: středně tmavý odstín pleti',
    '🧑🏿‍🔬' => 'vědec/vědkyně: tmavý odstín pleti',
    '👨🏻‍🔬' => 'vědec: světlý odstín pleti',
    '👨🏼‍🔬' => 'vědec: středně světlý odstín pleti',
    '👨🏽‍🔬' => 'vědec: střední odstín pleti',
    '👨🏾‍🔬' => 'vědec: středně tmavý odstín pleti',
    '👨🏿‍🔬' => 'vědec: tmavý odstín pleti',
    '👩🏻‍🔬' => 'vědkyně: světlý odstín pleti',
    '👩🏼‍🔬' => 'vědkyně: středně světlý odstín pleti',
    '👩🏽‍🔬' => 'vědkyně: střední odstín pleti',
    '👩🏾‍🔬' => 'vědkyně: středně tmavý odstín pleti',
    '👩🏿‍🔬' => 'vědkyně: tmavý odstín pleti',
    '🧑🏻‍💻' => 'programátor/programátorka: světlý odstín pleti',
    '🧑🏼‍💻' => 'programátor/programátorka: středně světlý odstín pleti',
    '🧑🏽‍💻' => 'programátor/programátorka: střední odstín pleti',
    '🧑🏾‍💻' => 'programátor/programátorka: středně tmavý odstín pleti',
    '🧑🏿‍💻' => 'programátor/programátorka: tmavý odstín pleti',
    '👨🏻‍💻' => 'programátor: světlý odstín pleti',
    '👨🏼‍💻' => 'programátor: středně světlý odstín pleti',
    '👨🏽‍💻' => 'programátor: střední odstín pleti',
    '👨🏾‍💻' => 'programátor: středně tmavý odstín pleti',
    '👨🏿‍💻' => 'programátor: tmavý odstín pleti',
    '👩🏻‍💻' => 'programátorka: světlý odstín pleti',
    '👩🏼‍💻' => 'programátorka: středně světlý odstín pleti',
    '👩🏽‍💻' => 'programátorka: střední odstín pleti',
    '👩🏾‍💻' => 'programátorka: středně tmavý odstín pleti',
    '👩🏿‍💻' => 'programátorka: tmavý odstín pleti',
    '🧑🏻‍🎤' => 'zpěvák/zpěvačka: světlý odstín pleti',
    '🧑🏼‍🎤' => 'zpěvák/zpěvačka: středně světlý odstín pleti',
    '🧑🏽‍🎤' => 'zpěvák/zpěvačka: střední odstín pleti',
    '🧑🏾‍🎤' => 'zpěvák/zpěvačka: středně tmavý odstín pleti',
    '🧑🏿‍🎤' => 'zpěvák/zpěvačka: tmavý odstín pleti',
    '👨🏻‍🎤' => 'zpěvák: světlý odstín pleti',
    '👨🏼‍🎤' => 'zpěvák: středně světlý odstín pleti',
    '👨🏽‍🎤' => 'zpěvák: střední odstín pleti',
    '👨🏾‍🎤' => 'zpěvák: středně tmavý odstín pleti',
    '👨🏿‍🎤' => 'zpěvák: tmavý odstín pleti',
    '👩🏻‍🎤' => 'zpěvačka: světlý odstín pleti',
    '👩🏼‍🎤' => 'zpěvačka: středně světlý odstín pleti',
    '👩🏽‍🎤' => 'zpěvačka: střední odstín pleti',
    '👩🏾‍🎤' => 'zpěvačka: středně tmavý odstín pleti',
    '👩🏿‍🎤' => 'zpěvačka: tmavý odstín pleti',
    '🧑🏻‍🎨' => 'malíř/malířka: světlý odstín pleti',
    '🧑🏼‍🎨' => 'malíř/malířka: středně světlý odstín pleti',
    '🧑🏽‍🎨' => 'malíř/malířka: střední odstín pleti',
    '🧑🏾‍🎨' => 'malíř/malířka: středně tmavý odstín pleti',
    '🧑🏿‍🎨' => 'malíř/malířka: tmavý odstín pleti',
    '👨🏻‍🎨' => 'malíř: světlý odstín pleti',
    '👨🏼‍🎨' => 'malíř: středně světlý odstín pleti',
    '👨🏽‍🎨' => 'malíř: střední odstín pleti',
    '👨🏾‍🎨' => 'malíř: středně tmavý odstín pleti',
    '👨🏿‍🎨' => 'malíř: tmavý odstín pleti',
    '👩🏻‍🎨' => 'malířka: světlý odstín pleti',
    '👩🏼‍🎨' => 'malířka: středně světlý odstín pleti',
    '👩🏽‍🎨' => 'malířka: střední odstín pleti',
    '👩🏾‍🎨' => 'malířka: středně tmavý odstín pleti',
    '👩🏿‍🎨' => 'malířka: tmavý odstín pleti',
    '🧑🏻‍✈' => 'pilot/pilotka: světlý odstín pleti',
    '🧑🏼‍✈' => 'pilot/pilotka: středně světlý odstín pleti',
    '🧑🏽‍✈' => 'pilot/pilotka: střední odstín pleti',
    '🧑🏾‍✈' => 'pilot/pilotka: středně tmavý odstín pleti',
    '🧑🏿‍✈' => 'pilot/pilotka: tmavý odstín pleti',
    '👨🏻‍✈' => 'pilot: světlý odstín pleti',
    '👨🏼‍✈' => 'pilot: středně světlý odstín pleti',
    '👨🏽‍✈' => 'pilot: střední odstín pleti',
    '👨🏾‍✈' => 'pilot: středně tmavý odstín pleti',
    '👨🏿‍✈' => 'pilot: tmavý odstín pleti',
    '👩🏻‍✈' => 'pilotka: světlý odstín pleti',
    '👩🏼‍✈' => 'pilotka: středně světlý odstín pleti',
    '👩🏽‍✈' => 'pilotka: střední odstín pleti',
    '👩🏾‍✈' => 'pilotka: středně tmavý odstín pleti',
    '👩🏿‍✈' => 'pilotka: tmavý odstín pleti',
    '🧑🏻‍🚀' => 'kosmonaut/kosmonautka: světlý odstín pleti',
    '🧑🏼‍🚀' => 'kosmonaut/kosmonautka: středně světlý odstín pleti',
    '🧑🏽‍🚀' => 'kosmonaut/kosmonautka: střední odstín pleti',
    '🧑🏾‍🚀' => 'kosmonaut/kosmonautka: středně tmavý odstín pleti',
    '🧑🏿‍🚀' => 'kosmonaut/kosmonautka: tmavý odstín pleti',
    '👨🏻‍🚀' => 'kosmonaut: světlý odstín pleti',
    '👨🏼‍🚀' => 'kosmonaut: středně světlý odstín pleti',
    '👨🏽‍🚀' => 'kosmonaut: střední odstín pleti',
    '👨🏾‍🚀' => 'kosmonaut: středně tmavý odstín pleti',
    '👨🏿‍🚀' => 'kosmonaut: tmavý odstín pleti',
    '👩🏻‍🚀' => 'kosmonautka: světlý odstín pleti',
    '👩🏼‍🚀' => 'kosmonautka: středně světlý odstín pleti',
    '👩🏽‍🚀' => 'kosmonautka: střední odstín pleti',
    '👩🏾‍🚀' => 'kosmonautka: středně tmavý odstín pleti',
    '👩🏿‍🚀' => 'kosmonautka: tmavý odstín pleti',
    '🧑🏻‍🚒' => 'hasič/hasička: světlý odstín pleti',
    '🧑🏼‍🚒' => 'hasič/hasička: středně světlý odstín pleti',
    '🧑🏽‍🚒' => 'hasič/hasička: střední odstín pleti',
    '🧑🏾‍🚒' => 'hasič/hasička: středně tmavý odstín pleti',
    '🧑🏿‍🚒' => 'hasič/hasička: tmavý odstín pleti',
    '👨🏻‍🚒' => 'hasič: světlý odstín pleti',
    '👨🏼‍🚒' => 'hasič: středně světlý odstín pleti',
    '👨🏽‍🚒' => 'hasič: střední odstín pleti',
    '👨🏾‍🚒' => 'hasič: středně tmavý odstín pleti',
    '👨🏿‍🚒' => 'hasič: tmavý odstín pleti',
    '👩🏻‍🚒' => 'hasička: světlý odstín pleti',
    '👩🏼‍🚒' => 'hasička: středně světlý odstín pleti',
    '👩🏽‍🚒' => 'hasička: střední odstín pleti',
    '👩🏾‍🚒' => 'hasička: středně tmavý odstín pleti',
    '👩🏿‍🚒' => 'hasička: tmavý odstín pleti',
    '👮🏻‍♂' => 'policista: světlý odstín pleti',
    '👮🏼‍♂' => 'policista: středně světlý odstín pleti',
    '👮🏽‍♂' => 'policista: střední odstín pleti',
    '👮🏾‍♂' => 'policista: středně tmavý odstín pleti',
    '👮🏿‍♂' => 'policista: tmavý odstín pleti',
    '👮🏻‍♀' => 'policistka: světlý odstín pleti',
    '👮🏼‍♀' => 'policistka: středně světlý odstín pleti',
    '👮🏽‍♀' => 'policistka: střední odstín pleti',
    '👮🏾‍♀' => 'policistka: středně tmavý odstín pleti',
    '👮🏿‍♀' => 'policistka: tmavý odstín pleti',
    '🕵🏻‍♂' => 'kriminální inspektor: světlý odstín pleti',
    '🕵🏼‍♂' => 'kriminální inspektor: středně světlý odstín pleti',
    '🕵🏽‍♂' => 'kriminální inspektor: střední odstín pleti',
    '🕵🏾‍♂' => 'kriminální inspektor: středně tmavý odstín pleti',
    '🕵🏿‍♂' => 'kriminální inspektor: tmavý odstín pleti',
    '🕵🏻‍♀' => 'kriminální inspektorka: světlý odstín pleti',
    '🕵🏼‍♀' => 'kriminální inspektorka: středně světlý odstín pleti',
    '🕵🏽‍♀' => 'kriminální inspektorka: střední odstín pleti',
    '🕵🏾‍♀' => 'kriminální inspektorka: středně tmavý odstín pleti',
    '🕵🏿‍♀' => 'kriminální inspektorka: tmavý odstín pleti',
    '💂🏻‍♂' => 'strážce: světlý odstín pleti',
    '💂🏼‍♂' => 'strážce: středně světlý odstín pleti',
    '💂🏽‍♂' => 'strážce: střední odstín pleti',
    '💂🏾‍♂' => 'strážce: středně tmavý odstín pleti',
    '💂🏿‍♂' => 'strážce: tmavý odstín pleti',
    '💂🏻‍♀' => 'strážkyně: světlý odstín pleti',
    '💂🏼‍♀' => 'strážkyně: středně světlý odstín pleti',
    '💂🏽‍♀' => 'strážkyně: střední odstín pleti',
    '💂🏾‍♀' => 'strážkyně: středně tmavý odstín pleti',
    '💂🏿‍♀' => 'strážkyně: tmavý odstín pleti',
    '👷🏻‍♂' => 'stavební dělník: světlý odstín pleti',
    '👷🏼‍♂' => 'stavební dělník: středně světlý odstín pleti',
    '👷🏽‍♂' => 'stavební dělník: střední odstín pleti',
    '👷🏾‍♂' => 'stavební dělník: středně tmavý odstín pleti',
    '👷🏿‍♂' => 'stavební dělník: tmavý odstín pleti',
    '👷🏻‍♀' => 'stavební dělnice: světlý odstín pleti',
    '👷🏼‍♀' => 'stavební dělnice: středně světlý odstín pleti',
    '👷🏽‍♀' => 'stavební dělnice: střední odstín pleti',
    '👷🏾‍♀' => 'stavební dělnice: středně tmavý odstín pleti',
    '👷🏿‍♀' => 'stavební dělnice: tmavý odstín pleti',
    '👳🏻‍♂' => 'muž s turbanem: světlý odstín pleti',
    '👳🏼‍♂' => 'muž s turbanem: středně světlý odstín pleti',
    '👳🏽‍♂' => 'muž s turbanem: střední odstín pleti',
    '👳🏾‍♂' => 'muž s turbanem: středně tmavý odstín pleti',
    '👳🏿‍♂' => 'muž s turbanem: tmavý odstín pleti',
    '👳🏻‍♀' => 'žena s turbanem: světlý odstín pleti',
    '👳🏼‍♀' => 'žena s turbanem: středně světlý odstín pleti',
    '👳🏽‍♀' => 'žena s turbanem: střední odstín pleti',
    '👳🏾‍♀' => 'žena s turbanem: středně tmavý odstín pleti',
    '👳🏿‍♀' => 'žena s turbanem: tmavý odstín pleti',
    '🤵🏻‍♂' => 'muž ve smokingu: světlý odstín pleti',
    '🤵🏼‍♂' => 'muž ve smokingu: středně světlý odstín pleti',
    '🤵🏽‍♂' => 'muž ve smokingu: střední odstín pleti',
    '🤵🏾‍♂' => 'muž ve smokingu: středně tmavý odstín pleti',
    '🤵🏿‍♂' => 'muž ve smokingu: tmavý odstín pleti',
    '🤵🏻‍♀' => 'žena ve smokingu: světlý odstín pleti',
    '🤵🏼‍♀' => 'žena ve smokingu: středně světlý odstín pleti',
    '🤵🏽‍♀' => 'žena ve smokingu: střední odstín pleti',
    '🤵🏾‍♀' => 'žena ve smokingu: středně tmavý odstín pleti',
    '🤵🏿‍♀' => 'žena ve smokingu: tmavý odstín pleti',
    '👰🏻‍♂' => 'muž se závojem: světlý odstín pleti',
    '👰🏼‍♂' => 'muž se závojem: středně světlý odstín pleti',
    '👰🏽‍♂' => 'muž se závojem: střední odstín pleti',
    '👰🏾‍♂' => 'muž se závojem: středně tmavý odstín pleti',
    '👰🏿‍♂' => 'muž se závojem: tmavý odstín pleti',
    '👰🏻‍♀' => 'žena se závojem: světlý odstín pleti',
    '👰🏼‍♀' => 'žena se závojem: středně světlý odstín pleti',
    '👰🏽‍♀' => 'žena se závojem: střední odstín pleti',
    '👰🏾‍♀' => 'žena se závojem: středně tmavý odstín pleti',
    '👰🏿‍♀' => 'žena se závojem: tmavý odstín pleti',
    '👩🏻‍🍼' => 'žena krmící dítě z lahve: světlý odstín pleti',
    '👩🏼‍🍼' => 'žena krmící dítě z lahve: středně světlý odstín pleti',
    '👩🏽‍🍼' => 'žena krmící dítě z lahve: střední odstín pleti',
    '👩🏾‍🍼' => 'žena krmící dítě z lahve: středně tmavý odstín pleti',
    '👩🏿‍🍼' => 'žena krmící dítě z lahve: tmavý odstín pleti',
    '👨🏻‍🍼' => 'muž krmící dítě z lahve: světlý odstín pleti',
    '👨🏼‍🍼' => 'muž krmící dítě z lahve: středně světlý odstín pleti',
    '👨🏽‍🍼' => 'muž krmící dítě z lahve: střední odstín pleti',
    '👨🏾‍🍼' => 'muž krmící dítě z lahve: středně tmavý odstín pleti',
    '👨🏿‍🍼' => 'muž krmící dítě z lahve: tmavý odstín pleti',
    '🧑🏻‍🍼' => 'osoba krmící dítě z lahve: světlý odstín pleti',
    '🧑🏼‍🍼' => 'osoba krmící dítě z lahve: středně světlý odstín pleti',
    '🧑🏽‍🍼' => 'osoba krmící dítě z lahve: střední odstín pleti',
    '🧑🏾‍🍼' => 'osoba krmící dítě z lahve: středně tmavý odstín pleti',
    '🧑🏿‍🍼' => 'osoba krmící dítě z lahve: tmavý odstín pleti',
    '🧑🏻‍🎄' => 'Santa: světlý odstín pleti',
    '🧑🏼‍🎄' => 'Santa: středně světlý odstín pleti',
    '🧑🏽‍🎄' => 'Santa: střední odstín pleti',
    '🧑🏾‍🎄' => 'Santa: středně tmavý odstín pleti',
    '🧑🏿‍🎄' => 'Santa: tmavý odstín pleti',
    '🦸🏻‍♂' => 'superhrdina: světlý odstín pleti',
    '🦸🏼‍♂' => 'superhrdina: středně světlý odstín pleti',
    '🦸🏽‍♂' => 'superhrdina: střední odstín pleti',
    '🦸🏾‍♂' => 'superhrdina: středně tmavý odstín pleti',
    '🦸🏿‍♂' => 'superhrdina: tmavý odstín pleti',
    '🦸🏻‍♀' => 'superhrdinka: světlý odstín pleti',
    '🦸🏼‍♀' => 'superhrdinka: středně světlý odstín pleti',
    '🦸🏽‍♀' => 'superhrdinka: střední odstín pleti',
    '🦸🏾‍♀' => 'superhrdinka: středně tmavý odstín pleti',
    '🦸🏿‍♀' => 'superhrdinka: tmavý odstín pleti',
    '🦹🏻‍♂' => 'superpadouch: světlý odstín pleti',
    '🦹🏼‍♂' => 'superpadouch: středně světlý odstín pleti',
    '🦹🏽‍♂' => 'superpadouch: střední odstín pleti',
    '🦹🏾‍♂' => 'superpadouch: středně tmavý odstín pleti',
    '🦹🏿‍♂' => 'superpadouch: tmavý odstín pleti',
    '🦹🏻‍♀' => 'superpadouška: světlý odstín pleti',
    '🦹🏼‍♀' => 'superpadouška: středně světlý odstín pleti',
    '🦹🏽‍♀' => 'superpadouška: střední odstín pleti',
    '🦹🏾‍♀' => 'superpadouška: středně tmavý odstín pleti',
    '🦹🏿‍♀' => 'superpadouška: tmavý odstín pleti',
    '🧙🏻‍♂' => 'čaroděj: světlý odstín pleti',
    '🧙🏼‍♂' => 'čaroděj: středně světlý odstín pleti',
    '🧙🏽‍♂' => 'čaroděj: střední odstín pleti',
    '🧙🏾‍♂' => 'čaroděj: středně tmavý odstín pleti',
    '🧙🏿‍♂' => 'čaroděj: tmavý odstín pleti',
    '🧙🏻‍♀' => 'čarodějka: světlý odstín pleti',
    '🧙🏼‍♀' => 'čarodějka: středně světlý odstín pleti',
    '🧙🏽‍♀' => 'čarodějka: střední odstín pleti',
    '🧙🏾‍♀' => 'čarodějka: středně tmavý odstín pleti',
    '🧙🏿‍♀' => 'čarodějka: tmavý odstín pleti',
    '🧚🏻‍♂' => 'okřídlený skřítek: světlý odstín pleti',
    '🧚🏼‍♂' => 'okřídlený skřítek: středně světlý odstín pleti',
    '🧚🏽‍♂' => 'okřídlený skřítek: střední odstín pleti',
    '🧚🏾‍♂' => 'okřídlený skřítek: středně tmavý odstín pleti',
    '🧚🏿‍♂' => 'okřídlený skřítek: tmavý odstín pleti',
    '🧚🏻‍♀' => 'okřídlená víla: světlý odstín pleti',
    '🧚🏼‍♀' => 'okřídlená víla: středně světlý odstín pleti',
    '🧚🏽‍♀' => 'okřídlená víla: střední odstín pleti',
    '🧚🏾‍♀' => 'okřídlená víla: středně tmavý odstín pleti',
    '🧚🏿‍♀' => 'okřídlená víla: tmavý odstín pleti',
    '🧛🏻‍♂' => 'upír: světlý odstín pleti',
    '🧛🏼‍♂' => 'upír: středně světlý odstín pleti',
    '🧛🏽‍♂' => 'upír: střední odstín pleti',
    '🧛🏾‍♂' => 'upír: středně tmavý odstín pleti',
    '🧛🏿‍♂' => 'upír: tmavý odstín pleti',
    '🧛🏻‍♀' => 'upírka: světlý odstín pleti',
    '🧛🏼‍♀' => 'upírka: středně světlý odstín pleti',
    '🧛🏽‍♀' => 'upírka: střední odstín pleti',
    '🧛🏾‍♀' => 'upírka: středně tmavý odstín pleti',
    '🧛🏿‍♀' => 'upírka: tmavý odstín pleti',
    '🧜🏻‍♂' => 'mořský muž: světlý odstín pleti',
    '🧜🏼‍♂' => 'mořský muž: středně světlý odstín pleti',
    '🧜🏽‍♂' => 'mořský muž: střední odstín pleti',
    '🧜🏾‍♂' => 'mořský muž: středně tmavý odstín pleti',
    '🧜🏿‍♂' => 'mořský muž: tmavý odstín pleti',
    '🧜🏻‍♀' => 'mořská panna: světlý odstín pleti',
    '🧜🏼‍♀' => 'mořská panna: středně světlý odstín pleti',
    '🧜🏽‍♀' => 'mořská panna: střední odstín pleti',
    '🧜🏾‍♀' => 'mořská panna: středně tmavý odstín pleti',
    '🧜🏿‍♀' => 'mořská panna: tmavý odstín pleti',
    '🧝🏻‍♂' => 'elf: světlý odstín pleti',
    '🧝🏼‍♂' => 'elf: středně světlý odstín pleti',
    '🧝🏽‍♂' => 'elf: střední odstín pleti',
    '🧝🏾‍♂' => 'elf: středně tmavý odstín pleti',
    '🧝🏿‍♂' => 'elf: tmavý odstín pleti',
    '🧝🏻‍♀' => 'elfka: světlý odstín pleti',
    '🧝🏼‍♀' => 'elfka: středně světlý odstín pleti',
    '🧝🏽‍♀' => 'elfka: střední odstín pleti',
    '🧝🏾‍♀' => 'elfka: středně tmavý odstín pleti',
    '🧝🏿‍♀' => 'elfka: tmavý odstín pleti',
    '💆🏻‍♂' => 'muž na masáži obličeje: světlý odstín pleti',
    '💆🏼‍♂' => 'muž na masáži obličeje: středně světlý odstín pleti',
    '💆🏽‍♂' => 'muž na masáži obličeje: střední odstín pleti',
    '💆🏾‍♂' => 'muž na masáži obličeje: středně tmavý odstín pleti',
    '💆🏿‍♂' => 'muž na masáži obličeje: tmavý odstín pleti',
    '💆🏻‍♀' => 'žena na masáži obličeje: světlý odstín pleti',
    '💆🏼‍♀' => 'žena na masáži obličeje: středně světlý odstín pleti',
    '💆🏽‍♀' => 'žena na masáži obličeje: střední odstín pleti',
    '💆🏾‍♀' => 'žena na masáži obličeje: středně tmavý odstín pleti',
    '💆🏿‍♀' => 'žena na masáži obličeje: tmavý odstín pleti',
    '💇🏻‍♂' => 'muž u kadeřníka: světlý odstín pleti',
    '💇🏼‍♂' => 'muž u kadeřníka: středně světlý odstín pleti',
    '💇🏽‍♂' => 'muž u kadeřníka: střední odstín pleti',
    '💇🏾‍♂' => 'muž u kadeřníka: středně tmavý odstín pleti',
    '💇🏿‍♂' => 'muž u kadeřníka: tmavý odstín pleti',
    '💇🏻‍♀' => 'žena u kadeřníka: světlý odstín pleti',
    '💇🏼‍♀' => 'žena u kadeřníka: středně světlý odstín pleti',
    '💇🏽‍♀' => 'žena u kadeřníka: střední odstín pleti',
    '💇🏾‍♀' => 'žena u kadeřníka: středně tmavý odstín pleti',
    '💇🏿‍♀' => 'žena u kadeřníka: tmavý odstín pleti',
    '🚶🏻‍♂' => 'kráčející muž: světlý odstín pleti',
    '🚶🏼‍♂' => 'kráčející muž: středně světlý odstín pleti',
    '🚶🏽‍♂' => 'kráčející muž: střední odstín pleti',
    '🚶🏾‍♂' => 'kráčející muž: středně tmavý odstín pleti',
    '🚶🏿‍♂' => 'kráčející muž: tmavý odstín pleti',
    '🚶🏻‍♀' => 'kráčející žena: světlý odstín pleti',
    '🚶🏼‍♀' => 'kráčející žena: středně světlý odstín pleti',
    '🚶🏽‍♀' => 'kráčející žena: střední odstín pleti',
    '🚶🏾‍♀' => 'kráčející žena: středně tmavý odstín pleti',
    '🚶🏿‍♀' => 'kráčející žena: tmavý odstín pleti',
    '🚶🏻‍➡' => 'kráčející osoba: světlý odstín pleti, – směr doprava',
    '🚶🏼‍➡' => 'kráčející osoba: středně světlý odstín pleti, – směr doprava',
    '🚶🏽‍➡' => 'kráčející osoba: střední odstín pleti, – směr doprava',
    '🚶🏾‍➡' => 'kráčející osoba: středně tmavý odstín pleti, – směr doprava',
    '🚶🏿‍➡' => 'kráčející osoba: tmavý odstín pleti, – směr doprava',
    '🧍🏻‍♂' => 'stojící muž: světlý odstín pleti',
    '🧍🏼‍♂' => 'stojící muž: středně světlý odstín pleti',
    '🧍🏽‍♂' => 'stojící muž: střední odstín pleti',
    '🧍🏾‍♂' => 'stojící muž: středně tmavý odstín pleti',
    '🧍🏿‍♂' => 'stojící muž: tmavý odstín pleti',
    '🧍🏻‍♀' => 'stojící žena: světlý odstín pleti',
    '🧍🏼‍♀' => 'stojící žena: středně světlý odstín pleti',
    '🧍🏽‍♀' => 'stojící žena: střední odstín pleti',
    '🧍🏾‍♀' => 'stojící žena: středně tmavý odstín pleti',
    '🧍🏿‍♀' => 'stojící žena: tmavý odstín pleti',
    '🧎🏻‍♂' => 'klečící muž: světlý odstín pleti',
    '🧎🏼‍♂' => 'klečící muž: středně světlý odstín pleti',
    '🧎🏽‍♂' => 'klečící muž: střední odstín pleti',
    '🧎🏾‍♂' => 'klečící muž: středně tmavý odstín pleti',
    '🧎🏿‍♂' => 'klečící muž: tmavý odstín pleti',
    '🧎🏻‍♀' => 'klečící žena: světlý odstín pleti',
    '🧎🏼‍♀' => 'klečící žena: středně světlý odstín pleti',
    '🧎🏽‍♀' => 'klečící žena: střední odstín pleti',
    '🧎🏾‍♀' => 'klečící žena: středně tmavý odstín pleti',
    '🧎🏿‍♀' => 'klečící žena: tmavý odstín pleti',
    '🧎🏻‍➡' => 'klečící osoba: světlý odstín pleti, – směr doprava',
    '🧎🏼‍➡' => 'klečící osoba: středně světlý odstín pleti, – směr doprava',
    '🧎🏽‍➡' => 'klečící osoba: střední odstín pleti, – směr doprava',
    '🧎🏾‍➡' => 'klečící osoba: středně tmavý odstín pleti, – směr doprava',
    '🧎🏿‍➡' => 'klečící osoba: tmavý odstín pleti, – směr doprava',
    '🧑🏻‍🦯' => 'člověk se slepeckou holí: světlý odstín pleti',
    '🧑🏼‍🦯' => 'člověk se slepeckou holí: středně světlý odstín pleti',
    '🧑🏽‍🦯' => 'člověk se slepeckou holí: střední odstín pleti',
    '🧑🏾‍🦯' => 'člověk se slepeckou holí: středně tmavý odstín pleti',
    '🧑🏿‍🦯' => 'člověk se slepeckou holí: tmavý odstín pleti',
    '👨🏻‍🦯' => 'muž se slepeckou holí: světlý odstín pleti',
    '👨🏼‍🦯' => 'muž se slepeckou holí: středně světlý odstín pleti',
    '👨🏽‍🦯' => 'muž se slepeckou holí: střední odstín pleti',
    '👨🏾‍🦯' => 'muž se slepeckou holí: středně tmavý odstín pleti',
    '👨🏿‍🦯' => 'muž se slepeckou holí: tmavý odstín pleti',
    '👩🏻‍🦯' => 'žena se slepeckou holí: světlý odstín pleti',
    '👩🏼‍🦯' => 'žena se slepeckou holí: středně světlý odstín pleti',
    '👩🏽‍🦯' => 'žena se slepeckou holí: střední odstín pleti',
    '👩🏾‍🦯' => 'žena se slepeckou holí: středně tmavý odstín pleti',
    '👩🏿‍🦯' => 'žena se slepeckou holí: tmavý odstín pleti',
    '🧑🏻‍🦼' => 'člověk na elektrickém invalidním vozíku: světlý odstín pleti',
    '🧑🏼‍🦼' => 'člověk na elektrickém invalidním vozíku: středně světlý odstín pleti',
    '🧑🏽‍🦼' => 'člověk na elektrickém invalidním vozíku: střední odstín pleti',
    '🧑🏾‍🦼' => 'člověk na elektrickém invalidním vozíku: středně tmavý odstín pleti',
    '🧑🏿‍🦼' => 'člověk na elektrickém invalidním vozíku: tmavý odstín pleti',
    '👨🏻‍🦼' => 'muž na elektrickém invalidním vozíku: světlý odstín pleti',
    '👨🏼‍🦼' => 'muž na elektrickém invalidním vozíku: středně světlý odstín pleti',
    '👨🏽‍🦼' => 'muž na elektrickém invalidním vozíku: střední odstín pleti',
    '👨🏾‍🦼' => 'muž na elektrickém invalidním vozíku: středně tmavý odstín pleti',
    '👨🏿‍🦼' => 'muž na elektrickém invalidním vozíku: tmavý odstín pleti',
    '👩🏻‍🦼' => 'žena na elektrickém invalidním vozíku: světlý odstín pleti',
    '👩🏼‍🦼' => 'žena na elektrickém invalidním vozíku: středně světlý odstín pleti',
    '👩🏽‍🦼' => 'žena na elektrickém invalidním vozíku: střední odstín pleti',
    '👩🏾‍🦼' => 'žena na elektrickém invalidním vozíku: středně tmavý odstín pleti',
    '👩🏿‍🦼' => 'žena na elektrickém invalidním vozíku: tmavý odstín pleti',
    '🧑🏻‍🦽' => 'člověk na mechanickém invalidním vozíku: světlý odstín pleti',
    '🧑🏼‍🦽' => 'člověk na mechanickém invalidním vozíku: středně světlý odstín pleti',
    '🧑🏽‍🦽' => 'člověk na mechanickém invalidním vozíku: střední odstín pleti',
    '🧑🏾‍🦽' => 'člověk na mechanickém invalidním vozíku: středně tmavý odstín pleti',
    '🧑🏿‍🦽' => 'člověk na mechanickém invalidním vozíku: tmavý odstín pleti',
    '👨🏻‍🦽' => 'muž na mechanickém invalidním vozíku: světlý odstín pleti',
    '👨🏼‍🦽' => 'muž na mechanickém invalidním vozíku: středně světlý odstín pleti',
    '👨🏽‍🦽' => 'muž na mechanickém invalidním vozíku: střední odstín pleti',
    '👨🏾‍🦽' => 'muž na mechanickém invalidním vozíku: středně tmavý odstín pleti',
    '👨🏿‍🦽' => 'muž na mechanickém invalidním vozíku: tmavý odstín pleti',
    '👩🏻‍🦽' => 'žena na mechanickém invalidním vozíku: světlý odstín pleti',
    '👩🏼‍🦽' => 'žena na mechanickém invalidním vozíku: středně světlý odstín pleti',
    '👩🏽‍🦽' => 'žena na mechanickém invalidním vozíku: střední odstín pleti',
    '👩🏾‍🦽' => 'žena na mechanickém invalidním vozíku: středně tmavý odstín pleti',
    '👩🏿‍🦽' => 'žena na mechanickém invalidním vozíku: tmavý odstín pleti',
    '🏃🏻‍♂' => 'běžící muž: světlý odstín pleti',
    '🏃🏼‍♂' => 'běžící muž: středně světlý odstín pleti',
    '🏃🏽‍♂' => 'běžící muž: střední odstín pleti',
    '🏃🏾‍♂' => 'běžící muž: středně tmavý odstín pleti',
    '🏃🏿‍♂' => 'běžící muž: tmavý odstín pleti',
    '🏃🏻‍♀' => 'běžící žena: světlý odstín pleti',
    '🏃🏼‍♀' => 'běžící žena: středně světlý odstín pleti',
    '🏃🏽‍♀' => 'běžící žena: střední odstín pleti',
    '🏃🏾‍♀' => 'běžící žena: středně tmavý odstín pleti',
    '🏃🏿‍♀' => 'běžící žena: tmavý odstín pleti',
    '🏃🏻‍➡' => 'běžící osoba: světlý odstín pleti, – směr doprava',
    '🏃🏼‍➡' => 'běžící osoba: středně světlý odstín pleti, – směr doprava',
    '🏃🏽‍➡' => 'běžící osoba: střední odstín pleti, – směr doprava',
    '🏃🏾‍➡' => 'běžící osoba: středně tmavý odstín pleti, – směr doprava',
    '🏃🏿‍➡' => 'běžící osoba: tmavý odstín pleti, – směr doprava',
    '👯🏻‍♀' => 'ženy se zaječími oušky: světlý odstín pleti',
    '👯🏼‍♀' => 'ženy se zaječími oušky: středně světlý odstín pleti',
    '👯🏽‍♀' => 'ženy se zaječími oušky: střední odstín pleti',
    '👯🏾‍♀' => 'ženy se zaječími oušky: středně tmavý odstín pleti',
    '👯🏿‍♀' => 'ženy se zaječími oušky: tmavý odstín pleti',
    '👯🏻‍♂' => 'muži se zaječími oušky: světlý odstín pleti',
    '👯🏼‍♂' => 'muži se zaječími oušky: středně světlý odstín pleti',
    '👯🏽‍♂' => 'muži se zaječími oušky: střední odstín pleti',
    '👯🏾‍♂' => 'muži se zaječími oušky: středně tmavý odstín pleti',
    '👯🏿‍♂' => 'muži se zaječími oušky: tmavý odstín pleti',
    '🧖🏻‍♂' => 'muž v sauně: světlý odstín pleti',
    '🧖🏼‍♂' => 'muž v sauně: středně světlý odstín pleti',
    '🧖🏽‍♂' => 'muž v sauně: střední odstín pleti',
    '🧖🏾‍♂' => 'muž v sauně: středně tmavý odstín pleti',
    '🧖🏿‍♂' => 'muž v sauně: tmavý odstín pleti',
    '🧖🏻‍♀' => 'žena v sauně: světlý odstín pleti',
    '🧖🏼‍♀' => 'žena v sauně: středně světlý odstín pleti',
    '🧖🏽‍♀' => 'žena v sauně: střední odstín pleti',
    '🧖🏾‍♀' => 'žena v sauně: středně tmavý odstín pleti',
    '🧖🏿‍♀' => 'žena v sauně: tmavý odstín pleti',
    '🧗🏻‍♂' => 'horolezec: světlý odstín pleti',
    '🧗🏼‍♂' => 'horolezec: středně světlý odstín pleti',
    '🧗🏽‍♂' => 'horolezec: střední odstín pleti',
    '🧗🏾‍♂' => 'horolezec: středně tmavý odstín pleti',
    '🧗🏿‍♂' => 'horolezec: tmavý odstín pleti',
    '🧗🏻‍♀' => 'horolezkyně: světlý odstín pleti',
    '🧗🏼‍♀' => 'horolezkyně: středně světlý odstín pleti',
    '🧗🏽‍♀' => 'horolezkyně: střední odstín pleti',
    '🧗🏾‍♀' => 'horolezkyně: středně tmavý odstín pleti',
    '🧗🏿‍♀' => 'horolezkyně: tmavý odstín pleti',
    '🏌🏻‍♂' => 'golfista: světlý odstín pleti',
    '🏌🏼‍♂' => 'golfista: středně světlý odstín pleti',
    '🏌🏽‍♂' => 'golfista: střední odstín pleti',
    '🏌🏾‍♂' => 'golfista: středně tmavý odstín pleti',
    '🏌🏿‍♂' => 'golfista: tmavý odstín pleti',
    '🏌🏻‍♀' => 'golfistka: světlý odstín pleti',
    '🏌🏼‍♀' => 'golfistka: středně světlý odstín pleti',
    '🏌🏽‍♀' => 'golfistka: střední odstín pleti',
    '🏌🏾‍♀' => 'golfistka: středně tmavý odstín pleti',
    '🏌🏿‍♀' => 'golfistka: tmavý odstín pleti',
    '🏄🏻‍♂' => 'surfař: světlý odstín pleti',
    '🏄🏼‍♂' => 'surfař: středně světlý odstín pleti',
    '🏄🏽‍♂' => 'surfař: střední odstín pleti',
    '🏄🏾‍♂' => 'surfař: středně tmavý odstín pleti',
    '🏄🏿‍♂' => 'surfař: tmavý odstín pleti',
    '🏄🏻‍♀' => 'surfařka: světlý odstín pleti',
    '🏄🏼‍♀' => 'surfařka: středně světlý odstín pleti',
    '🏄🏽‍♀' => 'surfařka: střední odstín pleti',
    '🏄🏾‍♀' => 'surfařka: středně tmavý odstín pleti',
    '🏄🏿‍♀' => 'surfařka: tmavý odstín pleti',
    '🚣🏻‍♂' => 'veslař: světlý odstín pleti',
    '🚣🏼‍♂' => 'veslař: středně světlý odstín pleti',
    '🚣🏽‍♂' => 'veslař: střední odstín pleti',
    '🚣🏾‍♂' => 'veslař: středně tmavý odstín pleti',
    '🚣🏿‍♂' => 'veslař: tmavý odstín pleti',
    '🚣🏻‍♀' => 'veslařka: světlý odstín pleti',
    '🚣🏼‍♀' => 'veslařka: středně světlý odstín pleti',
    '🚣🏽‍♀' => 'veslařka: střední odstín pleti',
    '🚣🏾‍♀' => 'veslařka: středně tmavý odstín pleti',
    '🚣🏿‍♀' => 'veslařka: tmavý odstín pleti',
    '🏊🏻‍♂' => 'plavec: světlý odstín pleti',
    '🏊🏼‍♂' => 'plavec: středně světlý odstín pleti',
    '🏊🏽‍♂' => 'plavec: střední odstín pleti',
    '🏊🏾‍♂' => 'plavec: středně tmavý odstín pleti',
    '🏊🏿‍♂' => 'plavec: tmavý odstín pleti',
    '🏊🏻‍♀' => 'plavkyně: světlý odstín pleti',
    '🏊🏼‍♀' => 'plavkyně: středně světlý odstín pleti',
    '🏊🏽‍♀' => 'plavkyně: střední odstín pleti',
    '🏊🏾‍♀' => 'plavkyně: středně tmavý odstín pleti',
    '🏊🏿‍♀' => 'plavkyně: tmavý odstín pleti',
    '⛹🏻‍♂' => 'muž driblující s míčem: světlý odstín pleti',
    '⛹🏼‍♂' => 'muž driblující s míčem: středně světlý odstín pleti',
    '⛹🏽‍♂' => 'muž driblující s míčem: střední odstín pleti',
    '⛹🏾‍♂' => 'muž driblující s míčem: středně tmavý odstín pleti',
    '⛹🏿‍♂' => 'muž driblující s míčem: tmavý odstín pleti',
    '⛹🏻‍♀' => 'žena driblující s míčem: světlý odstín pleti',
    '⛹🏼‍♀' => 'žena driblující s míčem: středně světlý odstín pleti',
    '⛹🏽‍♀' => 'žena driblující s míčem: střední odstín pleti',
    '⛹🏾‍♀' => 'žena driblující s míčem: středně tmavý odstín pleti',
    '⛹🏿‍♀' => 'žena driblující s míčem: tmavý odstín pleti',
    '🏋🏻‍♂' => 'vzpěrač: světlý odstín pleti',
    '🏋🏼‍♂' => 'vzpěrač: středně světlý odstín pleti',
    '🏋🏽‍♂' => 'vzpěrač: střední odstín pleti',
    '🏋🏾‍♂' => 'vzpěrač: středně tmavý odstín pleti',
    '🏋🏿‍♂' => 'vzpěrač: tmavý odstín pleti',
    '🏋🏻‍♀' => 'vzpěračka: světlý odstín pleti',
    '🏋🏼‍♀' => 'vzpěračka: středně světlý odstín pleti',
    '🏋🏽‍♀' => 'vzpěračka: střední odstín pleti',
    '🏋🏾‍♀' => 'vzpěračka: středně tmavý odstín pleti',
    '🏋🏿‍♀' => 'vzpěračka: tmavý odstín pleti',
    '🚴🏻‍♂' => 'cyklista: světlý odstín pleti',
    '🚴🏼‍♂' => 'cyklista: středně světlý odstín pleti',
    '🚴🏽‍♂' => 'cyklista: střední odstín pleti',
    '🚴🏾‍♂' => 'cyklista: středně tmavý odstín pleti',
    '🚴🏿‍♂' => 'cyklista: tmavý odstín pleti',
    '🚴🏻‍♀' => 'cyklistka: světlý odstín pleti',
    '🚴🏼‍♀' => 'cyklistka: středně světlý odstín pleti',
    '🚴🏽‍♀' => 'cyklistka: střední odstín pleti',
    '🚴🏾‍♀' => 'cyklistka: středně tmavý odstín pleti',
    '🚴🏿‍♀' => 'cyklistka: tmavý odstín pleti',
    '🚵🏻‍♂' => 'horský cyklista: světlý odstín pleti',
    '🚵🏼‍♂' => 'horský cyklista: středně světlý odstín pleti',
    '🚵🏽‍♂' => 'horský cyklista: střední odstín pleti',
    '🚵🏾‍♂' => 'horský cyklista: středně tmavý odstín pleti',
    '🚵🏿‍♂' => 'horský cyklista: tmavý odstín pleti',
    '🚵🏻‍♀' => 'horská cyklistka: světlý odstín pleti',
    '🚵🏼‍♀' => 'horská cyklistka: středně světlý odstín pleti',
    '🚵🏽‍♀' => 'horská cyklistka: střední odstín pleti',
    '🚵🏾‍♀' => 'horská cyklistka: středně tmavý odstín pleti',
    '🚵🏿‍♀' => 'horská cyklistka: tmavý odstín pleti',
    '🤸🏻‍♂' => 'muž dělající přemet stranou: světlý odstín pleti',
    '🤸🏼‍♂' => 'muž dělající přemet stranou: středně světlý odstín pleti',
    '🤸🏽‍♂' => 'muž dělající přemet stranou: střední odstín pleti',
    '🤸🏾‍♂' => 'muž dělající přemet stranou: středně tmavý odstín pleti',
    '🤸🏿‍♂' => 'muž dělající přemet stranou: tmavý odstín pleti',
    '🤸🏻‍♀' => 'žena dělající přemet stranou: světlý odstín pleti',
    '🤸🏼‍♀' => 'žena dělající přemet stranou: středně světlý odstín pleti',
    '🤸🏽‍♀' => 'žena dělající přemet stranou: střední odstín pleti',
    '🤸🏾‍♀' => 'žena dělající přemet stranou: středně tmavý odstín pleti',
    '🤸🏿‍♀' => 'žena dělající přemet stranou: tmavý odstín pleti',
    '🤼🏻‍♀' => 'zápasící ženy: světlý odstín pleti',
    '🤼🏼‍♀' => 'zápasící ženy: středně světlý odstín pleti',
    '🤼🏽‍♀' => 'zápasící ženy: střední odstín pleti',
    '🤼🏾‍♀' => 'zápasící ženy: středně tmavý odstín pleti',
    '🤼🏿‍♀' => 'zápasící ženy: tmavý odstín pleti',
    '🤼🏻‍♂' => 'zápasící muži: světlý odstín pleti',
    '🤼🏼‍♂' => 'zápasící muži: středně světlý odstín pleti',
    '🤼🏽‍♂' => 'zápasící muži: střední odstín pleti',
    '🤼🏾‍♂' => 'zápasící muži: středně tmavý odstín pleti',
    '🤼🏿‍♂' => 'zápasící muži: tmavý odstín pleti',
    '🤽🏻‍♂' => 'hráč vodního póla: světlý odstín pleti',
    '🤽🏼‍♂' => 'hráč vodního póla: středně světlý odstín pleti',
    '🤽🏽‍♂' => 'hráč vodního póla: střední odstín pleti',
    '🤽🏾‍♂' => 'hráč vodního póla: středně tmavý odstín pleti',
    '🤽🏿‍♂' => 'hráč vodního póla: tmavý odstín pleti',
    '🤽🏻‍♀' => 'hráčka vodního póla: světlý odstín pleti',
    '🤽🏼‍♀' => 'hráčka vodního póla: středně světlý odstín pleti',
    '🤽🏽‍♀' => 'hráčka vodního póla: střední odstín pleti',
    '🤽🏾‍♀' => 'hráčka vodního póla: středně tmavý odstín pleti',
    '🤽🏿‍♀' => 'hráčka vodního póla: tmavý odstín pleti',
    '🤾🏻‍♂' => 'házenkář: světlý odstín pleti',
    '🤾🏼‍♂' => 'házenkář: středně světlý odstín pleti',
    '🤾🏽‍♂' => 'házenkář: střední odstín pleti',
    '🤾🏾‍♂' => 'házenkář: středně tmavý odstín pleti',
    '🤾🏿‍♂' => 'házenkář: tmavý odstín pleti',
    '🤾🏻‍♀' => 'házenkářka: světlý odstín pleti',
    '🤾🏼‍♀' => 'házenkářka: středně světlý odstín pleti',
    '🤾🏽‍♀' => 'házenkářka: střední odstín pleti',
    '🤾🏾‍♀' => 'házenkářka: středně tmavý odstín pleti',
    '🤾🏿‍♀' => 'házenkářka: tmavý odstín pleti',
    '🤹🏻‍♂' => 'žonglér: světlý odstín pleti',
    '🤹🏼‍♂' => 'žonglér: středně světlý odstín pleti',
    '🤹🏽‍♂' => 'žonglér: střední odstín pleti',
    '🤹🏾‍♂' => 'žonglér: středně tmavý odstín pleti',
    '🤹🏿‍♂' => 'žonglér: tmavý odstín pleti',
    '🤹🏻‍♀' => 'žonglérka: světlý odstín pleti',
    '🤹🏼‍♀' => 'žonglérka: středně světlý odstín pleti',
    '🤹🏽‍♀' => 'žonglérka: střední odstín pleti',
    '🤹🏾‍♀' => 'žonglérka: středně tmavý odstín pleti',
    '🤹🏿‍♀' => 'žonglérka: tmavý odstín pleti',
    '🧘🏻‍♂' => 'muž v lotosovém sedu: světlý odstín pleti',
    '🧘🏼‍♂' => 'muž v lotosovém sedu: středně světlý odstín pleti',
    '🧘🏽‍♂' => 'muž v lotosovém sedu: střední odstín pleti',
    '🧘🏾‍♂' => 'muž v lotosovém sedu: středně tmavý odstín pleti',
    '🧘🏿‍♂' => 'muž v lotosovém sedu: tmavý odstín pleti',
    '🧘🏻‍♀' => 'žena v lotosovém sedu: světlý odstín pleti',
    '🧘🏼‍♀' => 'žena v lotosovém sedu: středně světlý odstín pleti',
    '🧘🏽‍♀' => 'žena v lotosovém sedu: střední odstín pleti',
    '🧘🏾‍♀' => 'žena v lotosovém sedu: středně tmavý odstín pleti',
    '🧘🏿‍♀' => 'žena v lotosovém sedu: tmavý odstín pleti',
    '🧑‍🦰' => 'dospělý člověk: zrzavé vlasy',
    '🧑‍🦱' => 'dospělý člověk: kudrnaté vlasy',
    '🧑‍🦳' => 'dospělý člověk: bílé vlasy',
    '🧑‍🦲' => 'dospělý člověk: bez vlasů',
    '👨‍🦰' => 'muž: zrzavé vlasy',
    '👨‍🦱' => 'muž: kudrnaté vlasy',
    '👨‍🦳' => 'muž: bílé vlasy',
    '👨‍🦲' => 'muž: bez vlasů',
    '👩‍🦰' => 'žena: zrzavé vlasy',
    '👩‍🦱' => 'žena: kudrnaté vlasy',
    '👩‍🦳' => 'žena: bílé vlasy',
    '👩‍🦲' => 'žena: bez vlasů',
    '🚶‍➡' => 'kráčející osoba: – směr doprava',
    '🧎‍➡' => 'klečící osoba: – směr doprava',
    '🏃‍➡' => 'běžící osoba: – směr doprava',
    '👨‍👦' => 'rodina: muž, chlapec',
    '👨‍👧' => 'rodina: muž, dívka',
    '👩‍👦' => 'rodina: žena, chlapec',
    '👩‍👧' => 'rodina: žena, dívka',
    '😶‍🌫' => 'obličej v oblacích',
    '😮‍💨' => 'vydechující obličej',
    '🙂‍↕' => 'kývání hlavou nahoru a dolů',
    '😵‍💫' => 'obličej se spirálami místo očí',
    '❤‍🔥' => 'hořící srdce',
    '❤‍🩹' => 'záplatované srdce',
    '👁‍🗨' => 'oko v řečové bublině',
    '🧔‍♂' => 'muž s plnovousem',
    '👱‍♂' => 'blonďák',
    '🧔‍♀' => 'žena s plnovousem',
    '👱‍♀' => 'blondýna',
    '🙍‍♂' => 'zamračený muž',
    '🙍‍♀' => 'zamračená žena',
    '🙎‍♂' => 'podrážděný muž',
    '🙎‍♀' => 'podrážděná žena',
    '🙅‍♂' => 'muž s gestem nesouhlasu',
    '🙅‍♀' => 'žena s gestem nesouhlasu',
    '🙆‍♂' => 'muž s gestem souhlasu',
    '🙆‍♀' => 'žena s gestem souhlasu',
    '💁‍♂' => 'muž s dlaní vzhůru',
    '💁‍♀' => 'žena s dlaní vzhůru',
    '🙋‍♂' => 'muž se zvednutou rukou',
    '🙋‍♀' => 'žena se zvednutou rukou',
    '🧏‍♂' => 'hluchý muž',
    '🧏‍♀' => 'hluchá žena',
    '🙇‍♂' => 'uklánějící se muž',
    '🙇‍♀' => 'uklánějící se žena',
    '🤦‍♂' => 'muž chytající se za hlavu',
    '🤦‍♀' => 'žena chytající se za hlavu',
    '🤷‍♂' => 'muž krčící rameny',
    '🤷‍♀' => 'žena krčící rameny',
    '🧑‍⚕' => 'zdravotník/zdravotnice',
    '👨‍⚕' => 'zdravotník',
    '👩‍⚕' => 'zdravotnice',
    '🧑‍🎓' => 'studující',
    '👨‍🎓' => 'student',
    '👩‍🎓' => 'studentka',
    '🧑‍🏫' => 'vyučující',
    '👨‍🏫' => 'učitel',
    '👩‍🏫' => 'učitelka',
    '🧑‍⚖' => 'soudce/soudkyně',
    '👨‍⚖' => 'soudce',
    '👩‍⚖' => 'soudkyně',
    '🧑‍🌾' => 'farmář/farmářka',
    '👨‍🌾' => 'farmář',
    '👩‍🌾' => 'farmářka',
    '🧑‍🍳' => 'kuchař/kuchařka',
    '👨‍🍳' => 'kuchař',
    '👩‍🍳' => 'kuchařka',
    '🧑‍🔧' => 'mechanik/mechanička',
    '👨‍🔧' => 'mechanik',
    '👩‍🔧' => 'mechanička',
    '🧑‍🏭' => 'osoba pracující v továrně',
    '👨‍🏭' => 'dělník v továrně',
    '👩‍🏭' => 'dělnice v továrně',
    '🧑‍💼' => 'manažer/manažerka',
    '👨‍💼' => 'manažer',
    '👩‍💼' => 'manažerka',
    '🧑‍🔬' => 'vědec/vědkyně',
    '👨‍🔬' => 'vědec',
    '👩‍🔬' => 'vědkyně',
    '🧑‍💻' => 'programátor/programátorka',
    '👨‍💻' => 'programátor',
    '👩‍💻' => 'programátorka',
    '🧑‍🎤' => 'zpěvák/zpěvačka',
    '👨‍🎤' => 'zpěvák',
    '👩‍🎤' => 'zpěvačka',
    '🧑‍🎨' => 'malíř/malířka',
    '👨‍🎨' => 'malíř',
    '👩‍🎨' => 'malířka',
    '🧑‍✈' => 'pilot/pilotka',
    '👨‍✈' => 'pilot',
    '👩‍✈' => 'pilotka',
    '🧑‍🚀' => 'kosmonaut/kosmonautka',
    '👨‍🚀' => 'kosmonaut',
    '👩‍🚀' => 'kosmonautka',
    '🧑‍🚒' => 'hasič/hasička',
    '👨‍🚒' => 'hasič',
    '👩‍🚒' => 'hasička',
    '👮‍♂' => 'policista',
    '👮‍♀' => 'policistka',
    '🕵‍♂' => 'kriminální inspektor',
    '🕵‍♀' => 'kriminální inspektorka',
    '💂‍♂' => 'strážce',
    '💂‍♀' => 'strážkyně',
    '👷‍♂' => 'stavební dělník',
    '👷‍♀' => 'stavební dělnice',
    '👳‍♂' => 'muž s turbanem',
    '👳‍♀' => 'žena s turbanem',
    '🤵‍♂' => 'muž ve smokingu',
    '🤵‍♀' => 'žena ve smokingu',
    '👰‍♂' => 'muž se závojem',
    '👰‍♀' => 'žena se závojem',
    '👩‍🍼' => 'žena krmící dítě z lahve',
    '👨‍🍼' => 'muž krmící dítě z lahve',
    '🧑‍🍼' => 'osoba krmící dítě z lahve',
    '🧑‍🎄' => 'Santa',
    '🦸‍♂' => 'superhrdina',
    '🦸‍♀' => 'superhrdinka',
    '🦹‍♂' => 'superpadouch',
    '🦹‍♀' => 'superpadouška',
    '🧙‍♂' => 'čaroděj',
    '🧙‍♀' => 'čarodějka',
    '🧚‍♂' => 'okřídlený skřítek',
    '🧚‍♀' => 'okřídlená víla',
    '🧛‍♂' => 'upír',
    '🧛‍♀' => 'upírka',
    '🧜‍♂' => 'mořský muž',
    '🧜‍♀' => 'mořská panna',
    '🧝‍♂' => 'elf',
    '🧝‍♀' => 'elfka',
    '🧞‍♂' => 'džin',
    '🧞‍♀' => 'džinka',
    '🧟‍♂' => 'nemrtvý muž',
    '🧟‍♀' => 'nemrtvá žena',
    '💆‍♂' => 'muž na masáži obličeje',
    '💆‍♀' => 'žena na masáži obličeje',
    '💇‍♂' => 'muž u kadeřníka',
    '💇‍♀' => 'žena u kadeřníka',
    '🚶‍♂' => 'kráčející muž',
    '🚶‍♀' => 'kráčející žena',
    '🧍‍♂' => 'stojící muž',
    '🧍‍♀' => 'stojící žena',
    '🧎‍♂' => 'klečící muž',
    '🧎‍♀' => 'klečící žena',
    '🧑‍🦯' => 'člověk se slepeckou holí',
    '👨‍🦯' => 'muž se slepeckou holí',
    '👩‍🦯' => 'žena se slepeckou holí',
    '🧑‍🦼' => 'člověk na elektrickém invalidním vozíku',
    '👨‍🦼' => 'muž na elektrickém invalidním vozíku',
    '👩‍🦼' => 'žena na elektrickém invalidním vozíku',
    '🧑‍🦽' => 'člověk na mechanickém invalidním vozíku',
    '👨‍🦽' => 'muž na mechanickém invalidním vozíku',
    '👩‍🦽' => 'žena na mechanickém invalidním vozíku',
    '🏃‍♂' => 'běžící muž',
    '🏃‍♀' => 'běžící žena',
    '🧑‍🩰' => 'baletní tanečník',
    '👯‍♂' => 'muži se zaječími oušky',
    '👯‍♀' => 'ženy se zaječími oušky',
    '🧖‍♂' => 'muž v sauně',
    '🧖‍♀' => 'žena v sauně',
    '🧗‍♂' => 'horolezec',
    '🧗‍♀' => 'horolezkyně',
    '🏌‍♂' => 'golfista',
    '🏌‍♀' => 'golfistka',
    '🏄‍♂' => 'surfař',
    '🏄‍♀' => 'surfařka',
    '🚣‍♂' => 'veslař',
    '🚣‍♀' => 'veslařka',
    '🏊‍♂' => 'plavec',
    '🏊‍♀' => 'plavkyně',
    '⛹‍♂' => 'muž driblující s míčem',
    '⛹‍♀' => 'žena driblující s míčem',
    '🏋‍♂' => 'vzpěrač',
    '🏋‍♀' => 'vzpěračka',
    '🚴‍♂' => 'cyklista',
    '🚴‍♀' => 'cyklistka',
    '🚵‍♂' => 'horský cyklista',
    '🚵‍♀' => 'horská cyklistka',
    '🤸‍♂' => 'muž dělající přemet stranou',
    '🤸‍♀' => 'žena dělající přemet stranou',
    '🤼‍♂' => 'zápasící muži',
    '🤼‍♀' => 'zápasící ženy',
    '🤽‍♂' => 'hráč vodního póla',
    '🤽‍♀' => 'hráčka vodního póla',
    '🤾‍♂' => 'házenkář',
    '🤾‍♀' => 'házenkářka',
    '🤹‍♂' => 'žonglér',
    '🤹‍♀' => 'žonglérka',
    '🧘‍♂' => 'muž v lotosovém sedu',
    '🧘‍♀' => 'žena v lotosovém sedu',
    '🧑‍🧒' => 'rodina: dospělý, dítě',
    '🐕‍🦺' => 'služební pes',
    '🐈‍⬛' => 'černá kočka',
    '🐻‍❄' => 'hlava ledního medvěda',
    '🐦‍⬛' => 'černý pták',
    '🐦‍🔥' => 'fénix',
    '🍋‍🟩' => 'limeta',
    '🍄‍🟫' => 'hnědá houba',
    '⛓‍💥' => 'přetržený řetěz',
    '🏳‍🌈' => 'duhová vlajka',
    '🏳‍⚧' => 'vlajka trans hrdosti',
    '🏴‍☠' => 'pirátská vlajka',
    '👋🏻' => 'mávající ruka: světlý odstín pleti',
    '👋🏼' => 'mávající ruka: středně světlý odstín pleti',
    '👋🏽' => 'mávající ruka: střední odstín pleti',
    '👋🏾' => 'mávající ruka: středně tmavý odstín pleti',
    '👋🏿' => 'mávající ruka: tmavý odstín pleti',
    '🤚🏻' => 'hřbet zvednuté ruky: světlý odstín pleti',
    '🤚🏼' => 'hřbet zvednuté ruky: středně světlý odstín pleti',
    '🤚🏽' => 'hřbet zvednuté ruky: střední odstín pleti',
    '🤚🏾' => 'hřbet zvednuté ruky: středně tmavý odstín pleti',
    '🤚🏿' => 'hřbet zvednuté ruky: tmavý odstín pleti',
    '🖐🏻' => 'ruka s roztaženými prsty: světlý odstín pleti',
    '🖐🏼' => 'ruka s roztaženými prsty: středně světlý odstín pleti',
    '🖐🏽' => 'ruka s roztaženými prsty: střední odstín pleti',
    '🖐🏾' => 'ruka s roztaženými prsty: středně tmavý odstín pleti',
    '🖐🏿' => 'ruka s roztaženými prsty: tmavý odstín pleti',
    '✋🏻' => 'zvednutá ruka: světlý odstín pleti',
    '✋🏼' => 'zvednutá ruka: středně světlý odstín pleti',
    '✋🏽' => 'zvednutá ruka: střední odstín pleti',
    '✋🏾' => 'zvednutá ruka: středně tmavý odstín pleti',
    '✋🏿' => 'zvednutá ruka: tmavý odstín pleti',
    '🖖🏻' => 'vulkánský pozdrav: světlý odstín pleti',
    '🖖🏼' => 'vulkánský pozdrav: středně světlý odstín pleti',
    '🖖🏽' => 'vulkánský pozdrav: střední odstín pleti',
    '🖖🏾' => 'vulkánský pozdrav: středně tmavý odstín pleti',
    '🖖🏿' => 'vulkánský pozdrav: tmavý odstín pleti',
    '🫱🏻' => 'ruka otočená doprava: světlý odstín pleti',
    '🫱🏼' => 'ruka otočená doprava: středně světlý odstín pleti',
    '🫱🏽' => 'ruka otočená doprava: střední odstín pleti',
    '🫱🏾' => 'ruka otočená doprava: středně tmavý odstín pleti',
    '🫱🏿' => 'ruka otočená doprava: tmavý odstín pleti',
    '🫲🏻' => 'ruka otočená doleva: světlý odstín pleti',
    '🫲🏼' => 'ruka otočená doleva: středně světlý odstín pleti',
    '🫲🏽' => 'ruka otočená doleva: střední odstín pleti',
    '🫲🏾' => 'ruka otočená doleva: středně tmavý odstín pleti',
    '🫲🏿' => 'ruka otočená doleva: tmavý odstín pleti',
    '🫳🏻' => 'ruka otočená dlaní dolů: světlý odstín pleti',
    '🫳🏼' => 'ruka otočená dlaní dolů: středně světlý odstín pleti',
    '🫳🏽' => 'ruka otočená dlaní dolů: střední odstín pleti',
    '🫳🏾' => 'ruka otočená dlaní dolů: středně tmavý odstín pleti',
    '🫳🏿' => 'ruka otočená dlaní dolů: tmavý odstín pleti',
    '🫴🏻' => 'ruka otočená dlaní vzhůru: světlý odstín pleti',
    '🫴🏼' => 'ruka otočená dlaní vzhůru: středně světlý odstín pleti',
    '🫴🏽' => 'ruka otočená dlaní vzhůru: střední odstín pleti',
    '🫴🏾' => 'ruka otočená dlaní vzhůru: středně tmavý odstín pleti',
    '🫴🏿' => 'ruka otočená dlaní vzhůru: tmavý odstín pleti',
    '🫷🏻' => 'ruka tlačící vlevo: světlý odstín pleti',
    '🫷🏼' => 'ruka tlačící vlevo: středně světlý odstín pleti',
    '🫷🏽' => 'ruka tlačící vlevo: střední odstín pleti',
    '🫷🏾' => 'ruka tlačící vlevo: středně tmavý odstín pleti',
    '🫷🏿' => 'ruka tlačící vlevo: tmavý odstín pleti',
    '🫸🏻' => 'ruka tlačící vpravo: světlý odstín pleti',
    '🫸🏼' => 'ruka tlačící vpravo: středně světlý odstín pleti',
    '🫸🏽' => 'ruka tlačící vpravo: střední odstín pleti',
    '🫸🏾' => 'ruka tlačící vpravo: středně tmavý odstín pleti',
    '🫸🏿' => 'ruka tlačící vpravo: tmavý odstín pleti',
    '👌🏻' => 'gesto OK: světlý odstín pleti',
    '👌🏼' => 'gesto OK: středně světlý odstín pleti',
    '👌🏽' => 'gesto OK: střední odstín pleti',
    '👌🏾' => 'gesto OK: středně tmavý odstín pleti',
    '👌🏿' => 'gesto OK: tmavý odstín pleti',
    '🤌🏻' => 'špetka z pěti prstů: světlý odstín pleti',
    '🤌🏼' => 'špetka z pěti prstů: středně světlý odstín pleti',
    '🤌🏽' => 'špetka z pěti prstů: střední odstín pleti',
    '🤌🏾' => 'špetka z pěti prstů: středně tmavý odstín pleti',
    '🤌🏿' => 'špetka z pěti prstů: tmavý odstín pleti',
    '🤏🏻' => 'gesto „málo“: světlý odstín pleti',
    '🤏🏼' => 'gesto „málo“: středně světlý odstín pleti',
    '🤏🏽' => 'gesto „málo“: střední odstín pleti',
    '🤏🏾' => 'gesto „málo“: středně tmavý odstín pleti',
    '🤏🏿' => 'gesto „málo“: tmavý odstín pleti',
    '✌🏻' => 'gesto vítězství: světlý odstín pleti',
    '✌🏼' => 'gesto vítězství: středně světlý odstín pleti',
    '✌🏽' => 'gesto vítězství: střední odstín pleti',
    '✌🏾' => 'gesto vítězství: středně tmavý odstín pleti',
    '✌🏿' => 'gesto vítězství: tmavý odstín pleti',
    '🤞🏻' => 'zkřížené prsty: světlý odstín pleti',
    '🤞🏼' => 'zkřížené prsty: středně světlý odstín pleti',
    '🤞🏽' => 'zkřížené prsty: střední odstín pleti',
    '🤞🏾' => 'zkřížené prsty: středně tmavý odstín pleti',
    '🤞🏿' => 'zkřížené prsty: tmavý odstín pleti',
    '🫰🏻' => 'ruka se zkříženým palcem a ukazováčkem: světlý odstín pleti',
    '🫰🏼' => 'ruka se zkříženým palcem a ukazováčkem: středně světlý odstín pleti',
    '🫰🏽' => 'ruka se zkříženým palcem a ukazováčkem: střední odstín pleti',
    '🫰🏾' => 'ruka se zkříženým palcem a ukazováčkem: středně tmavý odstín pleti',
    '🫰🏿' => 'ruka se zkříženým palcem a ukazováčkem: tmavý odstín pleti',
    '🤟🏻' => 'gesto „miluji tě“: světlý odstín pleti',
    '🤟🏼' => 'gesto „miluji tě“: středně světlý odstín pleti',
    '🤟🏽' => 'gesto „miluji tě“: střední odstín pleti',
    '🤟🏾' => 'gesto „miluji tě“: středně tmavý odstín pleti',
    '🤟🏿' => 'gesto „miluji tě“: tmavý odstín pleti',
    '🤘🏻' => 'gesto parohů: světlý odstín pleti',
    '🤘🏼' => 'gesto parohů: středně světlý odstín pleti',
    '🤘🏽' => 'gesto parohů: střední odstín pleti',
    '🤘🏾' => 'gesto parohů: středně tmavý odstín pleti',
    '🤘🏿' => 'gesto parohů: tmavý odstín pleti',
    '🤙🏻' => 'gesto „zavolej mi“: světlý odstín pleti',
    '🤙🏼' => 'gesto „zavolej mi“: středně světlý odstín pleti',
    '🤙🏽' => 'gesto „zavolej mi“: střední odstín pleti',
    '🤙🏾' => 'gesto „zavolej mi“: středně tmavý odstín pleti',
    '🤙🏿' => 'gesto „zavolej mi“: tmavý odstín pleti',
    '👈🏻' => 'ruka s prstem ukazujícím vlevo: světlý odstín pleti',
    '👈🏼' => 'ruka s prstem ukazujícím vlevo: středně světlý odstín pleti',
    '👈🏽' => 'ruka s prstem ukazujícím vlevo: střední odstín pleti',
    '👈🏾' => 'ruka s prstem ukazujícím vlevo: středně tmavý odstín pleti',
    '👈🏿' => 'ruka s prstem ukazujícím vlevo: tmavý odstín pleti',
    '👉🏻' => 'ruka s prstem ukazujícím vpravo: světlý odstín pleti',
    '👉🏼' => 'ruka s prstem ukazujícím vpravo: středně světlý odstín pleti',
    '👉🏽' => 'ruka s prstem ukazujícím vpravo: střední odstín pleti',
    '👉🏾' => 'ruka s prstem ukazujícím vpravo: středně tmavý odstín pleti',
    '👉🏿' => 'ruka s prstem ukazujícím vpravo: tmavý odstín pleti',
    '👆🏻' => 'ruka s prstem ukazujícím nahoru: světlý odstín pleti',
    '👆🏼' => 'ruka s prstem ukazujícím nahoru: středně světlý odstín pleti',
    '👆🏽' => 'ruka s prstem ukazujícím nahoru: střední odstín pleti',
    '👆🏾' => 'ruka s prstem ukazujícím nahoru: středně tmavý odstín pleti',
    '👆🏿' => 'ruka s prstem ukazujícím nahoru: tmavý odstín pleti',
    '🖕🏻' => 'prostředníček: světlý odstín pleti',
    '🖕🏼' => 'prostředníček: středně světlý odstín pleti',
    '🖕🏽' => 'prostředníček: střední odstín pleti',
    '🖕🏾' => 'prostředníček: středně tmavý odstín pleti',
    '🖕🏿' => 'prostředníček: tmavý odstín pleti',
    '👇🏻' => 'ruka s prstem ukazujícím dolů: světlý odstín pleti',
    '👇🏼' => 'ruka s prstem ukazujícím dolů: středně světlý odstín pleti',
    '👇🏽' => 'ruka s prstem ukazujícím dolů: střední odstín pleti',
    '👇🏾' => 'ruka s prstem ukazujícím dolů: středně tmavý odstín pleti',
    '👇🏿' => 'ruka s prstem ukazujícím dolů: tmavý odstín pleti',
    '☝🏻' => 'ruka ukazující nahoru: světlý odstín pleti',
    '☝🏼' => 'ruka ukazující nahoru: středně světlý odstín pleti',
    '☝🏽' => 'ruka ukazující nahoru: střední odstín pleti',
    '☝🏾' => 'ruka ukazující nahoru: středně tmavý odstín pleti',
    '☝🏿' => 'ruka ukazující nahoru: tmavý odstín pleti',
    '🫵🏻' => 'ukazováček namířený na diváka: světlý odstín pleti',
    '🫵🏼' => 'ukazováček namířený na diváka: středně světlý odstín pleti',
    '🫵🏽' => 'ukazováček namířený na diváka: střední odstín pleti',
    '🫵🏾' => 'ukazováček namířený na diváka: středně tmavý odstín pleti',
    '🫵🏿' => 'ukazováček namířený na diváka: tmavý odstín pleti',
    '👍🏻' => 'palec nahoru: světlý odstín pleti',
    '👍🏼' => 'palec nahoru: středně světlý odstín pleti',
    '👍🏽' => 'palec nahoru: střední odstín pleti',
    '👍🏾' => 'palec nahoru: středně tmavý odstín pleti',
    '👍🏿' => 'palec nahoru: tmavý odstín pleti',
    '👎🏻' => 'palec dolů: světlý odstín pleti',
    '👎🏼' => 'palec dolů: středně světlý odstín pleti',
    '👎🏽' => 'palec dolů: střední odstín pleti',
    '👎🏾' => 'palec dolů: středně tmavý odstín pleti',
    '👎🏿' => 'palec dolů: tmavý odstín pleti',
    '✊🏻' => 'zvednutá pěst: světlý odstín pleti',
    '✊🏼' => 'zvednutá pěst: středně světlý odstín pleti',
    '✊🏽' => 'zvednutá pěst: střední odstín pleti',
    '✊🏾' => 'zvednutá pěst: středně tmavý odstín pleti',
    '✊🏿' => 'zvednutá pěst: tmavý odstín pleti',
    '👊🏻' => 'namířená pěst: světlý odstín pleti',
    '👊🏼' => 'namířená pěst: středně světlý odstín pleti',
    '👊🏽' => 'namířená pěst: střední odstín pleti',
    '👊🏾' => 'namířená pěst: středně tmavý odstín pleti',
    '👊🏿' => 'namířená pěst: tmavý odstín pleti',
    '🤛🏻' => 'pěst mířící doleva: světlý odstín pleti',
    '🤛🏼' => 'pěst mířící doleva: středně světlý odstín pleti',
    '🤛🏽' => 'pěst mířící doleva: střední odstín pleti',
    '🤛🏾' => 'pěst mířící doleva: středně tmavý odstín pleti',
    '🤛🏿' => 'pěst mířící doleva: tmavý odstín pleti',
    '🤜🏻' => 'pěst mířící doprava: světlý odstín pleti',
    '🤜🏼' => 'pěst mířící doprava: středně světlý odstín pleti',
    '🤜🏽' => 'pěst mířící doprava: střední odstín pleti',
    '🤜🏾' => 'pěst mířící doprava: středně tmavý odstín pleti',
    '🤜🏿' => 'pěst mířící doprava: tmavý odstín pleti',
    '👏🏻' => 'tleskající ruce: světlý odstín pleti',
    '👏🏼' => 'tleskající ruce: středně světlý odstín pleti',
    '👏🏽' => 'tleskající ruce: střední odstín pleti',
    '👏🏾' => 'tleskající ruce: středně tmavý odstín pleti',
    '👏🏿' => 'tleskající ruce: tmavý odstín pleti',
    '🙌🏻' => 'zvednuté ruce: světlý odstín pleti',
    '🙌🏼' => 'zvednuté ruce: středně světlý odstín pleti',
    '🙌🏽' => 'zvednuté ruce: střední odstín pleti',
    '🙌🏾' => 'zvednuté ruce: středně tmavý odstín pleti',
    '🙌🏿' => 'zvednuté ruce: tmavý odstín pleti',
    '🫶🏻' => 'ruce spojené do tvaru srdce: světlý odstín pleti',
    '🫶🏼' => 'ruce spojené do tvaru srdce: středně světlý odstín pleti',
    '🫶🏽' => 'ruce spojené do tvaru srdce: střední odstín pleti',
    '🫶🏾' => 'ruce spojené do tvaru srdce: středně tmavý odstín pleti',
    '🫶🏿' => 'ruce spojené do tvaru srdce: tmavý odstín pleti',
    '👐🏻' => 'rozevřené dlaně: světlý odstín pleti',
    '👐🏼' => 'rozevřené dlaně: středně světlý odstín pleti',
    '👐🏽' => 'rozevřené dlaně: střední odstín pleti',
    '👐🏾' => 'rozevřené dlaně: středně tmavý odstín pleti',
    '👐🏿' => 'rozevřené dlaně: tmavý odstín pleti',
    '🤲🏻' => 'zdvižené dlaně vedle sebe: světlý odstín pleti',
    '🤲🏼' => 'zdvižené dlaně vedle sebe: středně světlý odstín pleti',
    '🤲🏽' => 'zdvižené dlaně vedle sebe: střední odstín pleti',
    '🤲🏾' => 'zdvižené dlaně vedle sebe: středně tmavý odstín pleti',
    '🤲🏿' => 'zdvižené dlaně vedle sebe: tmavý odstín pleti',
    '🤝🏻' => 'stisk rukou: světlý odstín pleti',
    '🤝🏼' => 'stisk rukou: středně světlý odstín pleti',
    '🤝🏽' => 'stisk rukou: střední odstín pleti',
    '🤝🏾' => 'stisk rukou: středně tmavý odstín pleti',
    '🤝🏿' => 'stisk rukou: tmavý odstín pleti',
    '🙏🏻' => 'sepjaté ruce: světlý odstín pleti',
    '🙏🏼' => 'sepjaté ruce: středně světlý odstín pleti',
    '🙏🏽' => 'sepjaté ruce: střední odstín pleti',
    '🙏🏾' => 'sepjaté ruce: středně tmavý odstín pleti',
    '🙏🏿' => 'sepjaté ruce: tmavý odstín pleti',
    '✍🏻' => 'píšící ruka: světlý odstín pleti',
    '✍🏼' => 'píšící ruka: středně světlý odstín pleti',
    '✍🏽' => 'píšící ruka: střední odstín pleti',
    '✍🏾' => 'píšící ruka: středně tmavý odstín pleti',
    '✍🏿' => 'píšící ruka: tmavý odstín pleti',
    '💅🏻' => 'lak na nehty: světlý odstín pleti',
    '💅🏼' => 'lak na nehty: středně světlý odstín pleti',
    '💅🏽' => 'lak na nehty: střední odstín pleti',
    '💅🏾' => 'lak na nehty: středně tmavý odstín pleti',
    '💅🏿' => 'lak na nehty: tmavý odstín pleti',
    '🤳🏻' => 'selfie: světlý odstín pleti',
    '🤳🏼' => 'selfie: středně světlý odstín pleti',
    '🤳🏽' => 'selfie: střední odstín pleti',
    '🤳🏾' => 'selfie: středně tmavý odstín pleti',
    '🤳🏿' => 'selfie: tmavý odstín pleti',
    '💪🏻' => 'zatnutý biceps: světlý odstín pleti',
    '💪🏼' => 'zatnutý biceps: středně světlý odstín pleti',
    '💪🏽' => 'zatnutý biceps: střední odstín pleti',
    '💪🏾' => 'zatnutý biceps: středně tmavý odstín pleti',
    '💪🏿' => 'zatnutý biceps: tmavý odstín pleti',
    '🦵🏻' => 'noha: světlý odstín pleti',
    '🦵🏼' => 'noha: středně světlý odstín pleti',
    '🦵🏽' => 'noha: střední odstín pleti',
    '🦵🏾' => 'noha: středně tmavý odstín pleti',
    '🦵🏿' => 'noha: tmavý odstín pleti',
    '🦶🏻' => 'chodidlo: světlý odstín pleti',
    '🦶🏼' => 'chodidlo: středně světlý odstín pleti',
    '🦶🏽' => 'chodidlo: střední odstín pleti',
    '🦶🏾' => 'chodidlo: středně tmavý odstín pleti',
    '🦶🏿' => 'chodidlo: tmavý odstín pleti',
    '👂🏻' => 'ucho: světlý odstín pleti',
    '👂🏼' => 'ucho: středně světlý odstín pleti',
    '👂🏽' => 'ucho: střední odstín pleti',
    '👂🏾' => 'ucho: středně tmavý odstín pleti',
    '👂🏿' => 'ucho: tmavý odstín pleti',
    '🦻🏻' => 'ucho s naslouchátkem: světlý odstín pleti',
    '🦻🏼' => 'ucho s naslouchátkem: středně světlý odstín pleti',
    '🦻🏽' => 'ucho s naslouchátkem: střední odstín pleti',
    '🦻🏾' => 'ucho s naslouchátkem: středně tmavý odstín pleti',
    '🦻🏿' => 'ucho s naslouchátkem: tmavý odstín pleti',
    '👃🏻' => 'nos: světlý odstín pleti',
    '👃🏼' => 'nos: středně světlý odstín pleti',
    '👃🏽' => 'nos: střední odstín pleti',
    '👃🏾' => 'nos: středně tmavý odstín pleti',
    '👃🏿' => 'nos: tmavý odstín pleti',
    '👶🏻' => 'nemluvně: světlý odstín pleti',
    '👶🏼' => 'nemluvně: středně světlý odstín pleti',
    '👶🏽' => 'nemluvně: střední odstín pleti',
    '👶🏾' => 'nemluvně: středně tmavý odstín pleti',
    '👶🏿' => 'nemluvně: tmavý odstín pleti',
    '🧒🏻' => 'dítě: světlý odstín pleti',
    '🧒🏼' => 'dítě: středně světlý odstín pleti',
    '🧒🏽' => 'dítě: střední odstín pleti',
    '🧒🏾' => 'dítě: středně tmavý odstín pleti',
    '🧒🏿' => 'dítě: tmavý odstín pleti',
    '👦🏻' => 'chlapec: světlý odstín pleti',
    '👦🏼' => 'chlapec: středně světlý odstín pleti',
    '👦🏽' => 'chlapec: střední odstín pleti',
    '👦🏾' => 'chlapec: středně tmavý odstín pleti',
    '👦🏿' => 'chlapec: tmavý odstín pleti',
    '👧🏻' => 'dívka: světlý odstín pleti',
    '👧🏼' => 'dívka: středně světlý odstín pleti',
    '👧🏽' => 'dívka: střední odstín pleti',
    '👧🏾' => 'dívka: středně tmavý odstín pleti',
    '👧🏿' => 'dívka: tmavý odstín pleti',
    '🧑🏻' => 'dospělý člověk: světlý odstín pleti',
    '🧑🏼' => 'dospělý člověk: středně světlý odstín pleti',
    '🧑🏽' => 'dospělý člověk: střední odstín pleti',
    '🧑🏾' => 'dospělý člověk: středně tmavý odstín pleti',
    '🧑🏿' => 'dospělý člověk: tmavý odstín pleti',
    '👱🏻' => 'člověk s blond vlasy: světlý odstín pleti',
    '👱🏼' => 'člověk s blond vlasy: středně světlý odstín pleti',
    '👱🏽' => 'člověk s blond vlasy: střední odstín pleti',
    '👱🏾' => 'člověk s blond vlasy: středně tmavý odstín pleti',
    '👱🏿' => 'člověk s blond vlasy: tmavý odstín pleti',
    '👨🏻' => 'muž: světlý odstín pleti',
    '👨🏼' => 'muž: středně světlý odstín pleti',
    '👨🏽' => 'muž: střední odstín pleti',
    '👨🏾' => 'muž: středně tmavý odstín pleti',
    '👨🏿' => 'muž: tmavý odstín pleti',
    '🧔🏻' => 'osoba s plnovousem: světlý odstín pleti',
    '🧔🏼' => 'osoba s plnovousem: středně světlý odstín pleti',
    '🧔🏽' => 'osoba s plnovousem: střední odstín pleti',
    '🧔🏾' => 'osoba s plnovousem: středně tmavý odstín pleti',
    '🧔🏿' => 'osoba s plnovousem: tmavý odstín pleti',
    '👩🏻' => 'žena: světlý odstín pleti',
    '👩🏼' => 'žena: středně světlý odstín pleti',
    '👩🏽' => 'žena: střední odstín pleti',
    '👩🏾' => 'žena: středně tmavý odstín pleti',
    '👩🏿' => 'žena: tmavý odstín pleti',
    '🧓🏻' => 'starší dospělý člověk: světlý odstín pleti',
    '🧓🏼' => 'starší dospělý člověk: středně světlý odstín pleti',
    '🧓🏽' => 'starší dospělý člověk: střední odstín pleti',
    '🧓🏾' => 'starší dospělý člověk: středně tmavý odstín pleti',
    '🧓🏿' => 'starší dospělý člověk: tmavý odstín pleti',
    '👴🏻' => 'starý muž: světlý odstín pleti',
    '👴🏼' => 'starý muž: středně světlý odstín pleti',
    '👴🏽' => 'starý muž: střední odstín pleti',
    '👴🏾' => 'starý muž: středně tmavý odstín pleti',
    '👴🏿' => 'starý muž: tmavý odstín pleti',
    '👵🏻' => 'stará žena: světlý odstín pleti',
    '👵🏼' => 'stará žena: středně světlý odstín pleti',
    '👵🏽' => 'stará žena: střední odstín pleti',
    '👵🏾' => 'stará žena: středně tmavý odstín pleti',
    '👵🏿' => 'stará žena: tmavý odstín pleti',
    '🙍🏻' => 'zamračená osoba: světlý odstín pleti',
    '🙍🏼' => 'zamračená osoba: středně světlý odstín pleti',
    '🙍🏽' => 'zamračená osoba: střední odstín pleti',
    '🙍🏾' => 'zamračená osoba: středně tmavý odstín pleti',
    '🙍🏿' => 'zamračená osoba: tmavý odstín pleti',
    '🙎🏻' => 'podrážděná osoba: světlý odstín pleti',
    '🙎🏼' => 'podrážděná osoba: středně světlý odstín pleti',
    '🙎🏽' => 'podrážděná osoba: střední odstín pleti',
    '🙎🏾' => 'podrážděná osoba: středně tmavý odstín pleti',
    '🙎🏿' => 'podrážděná osoba: tmavý odstín pleti',
    '🙅🏻' => 'gesto nesouhlasu: světlý odstín pleti',
    '🙅🏼' => 'gesto nesouhlasu: středně světlý odstín pleti',
    '🙅🏽' => 'gesto nesouhlasu: střední odstín pleti',
    '🙅🏾' => 'gesto nesouhlasu: středně tmavý odstín pleti',
    '🙅🏿' => 'gesto nesouhlasu: tmavý odstín pleti',
    '🙆🏻' => 'gesto souhlasu: světlý odstín pleti',
    '🙆🏼' => 'gesto souhlasu: středně světlý odstín pleti',
    '🙆🏽' => 'gesto souhlasu: střední odstín pleti',
    '🙆🏾' => 'gesto souhlasu: středně tmavý odstín pleti',
    '🙆🏿' => 'gesto souhlasu: tmavý odstín pleti',
    '💁🏻' => 'osoba s dlaní vzhůru: světlý odstín pleti',
    '💁🏼' => 'osoba s dlaní vzhůru: středně světlý odstín pleti',
    '💁🏽' => 'osoba s dlaní vzhůru: střední odstín pleti',
    '💁🏾' => 'osoba s dlaní vzhůru: středně tmavý odstín pleti',
    '💁🏿' => 'osoba s dlaní vzhůru: tmavý odstín pleti',
    '🙋🏻' => 'osoba se zvednutou rukou: světlý odstín pleti',
    '🙋🏼' => 'osoba se zvednutou rukou: středně světlý odstín pleti',
    '🙋🏽' => 'osoba se zvednutou rukou: střední odstín pleti',
    '🙋🏾' => 'osoba se zvednutou rukou: středně tmavý odstín pleti',
    '🙋🏿' => 'osoba se zvednutou rukou: tmavý odstín pleti',
    '🧏🏻' => 'hluchá osoba: světlý odstín pleti',
    '🧏🏼' => 'hluchá osoba: středně světlý odstín pleti',
    '🧏🏽' => 'hluchá osoba: střední odstín pleti',
    '🧏🏾' => 'hluchá osoba: středně tmavý odstín pleti',
    '🧏🏿' => 'hluchá osoba: tmavý odstín pleti',
    '🙇🏻' => 'uklánějící se osoba: světlý odstín pleti',
    '🙇🏼' => 'uklánějící se osoba: středně světlý odstín pleti',
    '🙇🏽' => 'uklánějící se osoba: střední odstín pleti',
    '🙇🏾' => 'uklánějící se osoba: středně tmavý odstín pleti',
    '🙇🏿' => 'uklánějící se osoba: tmavý odstín pleti',
    '🤦🏻' => 'osoba chytající se za hlavu: světlý odstín pleti',
    '🤦🏼' => 'osoba chytající se za hlavu: středně světlý odstín pleti',
    '🤦🏽' => 'osoba chytající se za hlavu: střední odstín pleti',
    '🤦🏾' => 'osoba chytající se za hlavu: středně tmavý odstín pleti',
    '🤦🏿' => 'osoba chytající se za hlavu: tmavý odstín pleti',
    '🤷🏻' => 'osoba krčící rameny: světlý odstín pleti',
    '🤷🏼' => 'osoba krčící rameny: středně světlý odstín pleti',
    '🤷🏽' => 'osoba krčící rameny: střední odstín pleti',
    '🤷🏾' => 'osoba krčící rameny: středně tmavý odstín pleti',
    '🤷🏿' => 'osoba krčící rameny: tmavý odstín pleti',
    '👮🏻' => 'strážník: světlý odstín pleti',
    '👮🏼' => 'strážník: středně světlý odstín pleti',
    '👮🏽' => 'strážník: střední odstín pleti',
    '👮🏾' => 'strážník: středně tmavý odstín pleti',
    '👮🏿' => 'strážník: tmavý odstín pleti',
    '🕵🏻' => 'detektiv: světlý odstín pleti',
    '🕵🏼' => 'detektiv: středně světlý odstín pleti',
    '🕵🏽' => 'detektiv: střední odstín pleti',
    '🕵🏾' => 'detektiv: středně tmavý odstín pleti',
    '🕵🏿' => 'detektiv: tmavý odstín pleti',
    '💂🏻' => 'gardista: světlý odstín pleti',
    '💂🏼' => 'gardista: středně světlý odstín pleti',
    '💂🏽' => 'gardista: střední odstín pleti',
    '💂🏾' => 'gardista: středně tmavý odstín pleti',
    '💂🏿' => 'gardista: tmavý odstín pleti',
    '🥷🏻' => 'nindža: světlý odstín pleti',
    '🥷🏼' => 'nindža: středně světlý odstín pleti',
    '🥷🏽' => 'nindža: střední odstín pleti',
    '🥷🏾' => 'nindža: středně tmavý odstín pleti',
    '🥷🏿' => 'nindža: tmavý odstín pleti',
    '👷🏻' => 'dělník na stavbě: světlý odstín pleti',
    '👷🏼' => 'dělník na stavbě: středně světlý odstín pleti',
    '👷🏽' => 'dělník na stavbě: střední odstín pleti',
    '👷🏾' => 'dělník na stavbě: středně tmavý odstín pleti',
    '👷🏿' => 'dělník na stavbě: tmavý odstín pleti',
    '🫅🏻' => 'člověk s korunou na hlavě: světlý odstín pleti',
    '🫅🏼' => 'člověk s korunou na hlavě: středně světlý odstín pleti',
    '🫅🏽' => 'člověk s korunou na hlavě: střední odstín pleti',
    '🫅🏾' => 'člověk s korunou na hlavě: středně tmavý odstín pleti',
    '🫅🏿' => 'člověk s korunou na hlavě: tmavý odstín pleti',
    '🤴🏻' => 'princ: světlý odstín pleti',
    '🤴🏼' => 'princ: středně světlý odstín pleti',
    '🤴🏽' => 'princ: střední odstín pleti',
    '🤴🏾' => 'princ: středně tmavý odstín pleti',
    '🤴🏿' => 'princ: tmavý odstín pleti',
    '👸🏻' => 'princezna: světlý odstín pleti',
    '👸🏼' => 'princezna: středně světlý odstín pleti',
    '👸🏽' => 'princezna: střední odstín pleti',
    '👸🏾' => 'princezna: středně tmavý odstín pleti',
    '👸🏿' => 'princezna: tmavý odstín pleti',
    '👳🏻' => 'člověk s turbanem: světlý odstín pleti',
    '👳🏼' => 'člověk s turbanem: středně světlý odstín pleti',
    '👳🏽' => 'člověk s turbanem: střední odstín pleti',
    '👳🏾' => 'člověk s turbanem: středně tmavý odstín pleti',
    '👳🏿' => 'člověk s turbanem: tmavý odstín pleti',
    '👲🏻' => 'muž s čínskou čepicí: světlý odstín pleti',
    '👲🏼' => 'muž s čínskou čepicí: středně světlý odstín pleti',
    '👲🏽' => 'muž s čínskou čepicí: střední odstín pleti',
    '👲🏾' => 'muž s čínskou čepicí: středně tmavý odstín pleti',
    '👲🏿' => 'muž s čínskou čepicí: tmavý odstín pleti',
    '🧕🏻' => 'žena v šátku: světlý odstín pleti',
    '🧕🏼' => 'žena v šátku: středně světlý odstín pleti',
    '🧕🏽' => 'žena v šátku: střední odstín pleti',
    '🧕🏾' => 'žena v šátku: středně tmavý odstín pleti',
    '🧕🏿' => 'žena v šátku: tmavý odstín pleti',
    '🤵🏻' => 'osoba ve smokingu: světlý odstín pleti',
    '🤵🏼' => 'osoba ve smokingu: středně světlý odstín pleti',
    '🤵🏽' => 'osoba ve smokingu: střední odstín pleti',
    '🤵🏾' => 'osoba ve smokingu: středně tmavý odstín pleti',
    '🤵🏿' => 'osoba ve smokingu: tmavý odstín pleti',
    '👰🏻' => 'osoba se závojem: světlý odstín pleti',
    '👰🏼' => 'osoba se závojem: středně světlý odstín pleti',
    '👰🏽' => 'osoba se závojem: střední odstín pleti',
    '👰🏾' => 'osoba se závojem: středně tmavý odstín pleti',
    '👰🏿' => 'osoba se závojem: tmavý odstín pleti',
    '🤰🏻' => 'těhotná žena: světlý odstín pleti',
    '🤰🏼' => 'těhotná žena: středně světlý odstín pleti',
    '🤰🏽' => 'těhotná žena: střední odstín pleti',
    '🤰🏾' => 'těhotná žena: středně tmavý odstín pleti',
    '🤰🏿' => 'těhotná žena: tmavý odstín pleti',
    '🫃🏻' => 'těhotný muž: světlý odstín pleti',
    '🫃🏼' => 'těhotný muž: středně světlý odstín pleti',
    '🫃🏽' => 'těhotný muž: střední odstín pleti',
    '🫃🏾' => 'těhotný muž: středně tmavý odstín pleti',
    '🫃🏿' => 'těhotný muž: tmavý odstín pleti',
    '🫄🏻' => 'těhotná osoba: světlý odstín pleti',
    '🫄🏼' => 'těhotná osoba: středně světlý odstín pleti',
    '🫄🏽' => 'těhotná osoba: střední odstín pleti',
    '🫄🏾' => 'těhotná osoba: středně tmavý odstín pleti',
    '🫄🏿' => 'těhotná osoba: tmavý odstín pleti',
    '🤱🏻' => 'kojení: světlý odstín pleti',
    '🤱🏼' => 'kojení: středně světlý odstín pleti',
    '🤱🏽' => 'kojení: střední odstín pleti',
    '🤱🏾' => 'kojení: středně tmavý odstín pleti',
    '🤱🏿' => 'kojení: tmavý odstín pleti',
    '👼🏻' => 'andílek: světlý odstín pleti',
    '👼🏼' => 'andílek: středně světlý odstín pleti',
    '👼🏽' => 'andílek: střední odstín pleti',
    '👼🏾' => 'andílek: středně tmavý odstín pleti',
    '👼🏿' => 'andílek: tmavý odstín pleti',
    '🎅🏻' => 'Santa Claus: světlý odstín pleti',
    '🎅🏼' => 'Santa Claus: středně světlý odstín pleti',
    '🎅🏽' => 'Santa Claus: střední odstín pleti',
    '🎅🏾' => 'Santa Claus: středně tmavý odstín pleti',
    '🎅🏿' => 'Santa Claus: tmavý odstín pleti',
    '🤶🏻' => 'Santa Clausová: světlý odstín pleti',
    '🤶🏼' => 'Santa Clausová: středně světlý odstín pleti',
    '🤶🏽' => 'Santa Clausová: střední odstín pleti',
    '🤶🏾' => 'Santa Clausová: středně tmavý odstín pleti',
    '🤶🏿' => 'Santa Clausová: tmavý odstín pleti',
    '🦸🏻' => 'superhrdinská postava: světlý odstín pleti',
    '🦸🏼' => 'superhrdinská postava: středně světlý odstín pleti',
    '🦸🏽' => 'superhrdinská postava: střední odstín pleti',
    '🦸🏾' => 'superhrdinská postava: středně tmavý odstín pleti',
    '🦸🏿' => 'superhrdinská postava: tmavý odstín pleti',
    '🦹🏻' => 'superzločinec: světlý odstín pleti',
    '🦹🏼' => 'superzločinec: středně světlý odstín pleti',
    '🦹🏽' => 'superzločinec: střední odstín pleti',
    '🦹🏾' => 'superzločinec: středně tmavý odstín pleti',
    '🦹🏿' => 'superzločinec: tmavý odstín pleti',
    '🧙🏻' => 'mág: světlý odstín pleti',
    '🧙🏼' => 'mág: středně světlý odstín pleti',
    '🧙🏽' => 'mág: střední odstín pleti',
    '🧙🏾' => 'mág: středně tmavý odstín pleti',
    '🧙🏿' => 'mág: tmavý odstín pleti',
    '🧚🏻' => 'víla: světlý odstín pleti',
    '🧚🏼' => 'víla: středně světlý odstín pleti',
    '🧚🏽' => 'víla: střední odstín pleti',
    '🧚🏾' => 'víla: středně tmavý odstín pleti',
    '🧚🏿' => 'víla: tmavý odstín pleti',
    '🧛🏻' => 'obličej upíra: světlý odstín pleti',
    '🧛🏼' => 'obličej upíra: středně světlý odstín pleti',
    '🧛🏽' => 'obličej upíra: střední odstín pleti',
    '🧛🏾' => 'obličej upíra: středně tmavý odstín pleti',
    '🧛🏿' => 'obličej upíra: tmavý odstín pleti',
    '🧜🏻' => 'mořská bytost: světlý odstín pleti',
    '🧜🏼' => 'mořská bytost: středně světlý odstín pleti',
    '🧜🏽' => 'mořská bytost: střední odstín pleti',
    '🧜🏾' => 'mořská bytost: středně tmavý odstín pleti',
    '🧜🏿' => 'mořská bytost: tmavý odstín pleti',
    '🧝🏻' => 'obličej elfa: světlý odstín pleti',
    '🧝🏼' => 'obličej elfa: středně světlý odstín pleti',
    '🧝🏽' => 'obličej elfa: střední odstín pleti',
    '🧝🏾' => 'obličej elfa: středně tmavý odstín pleti',
    '🧝🏿' => 'obličej elfa: tmavý odstín pleti',
    '💆🏻' => 'osoba na masáži obličeje: světlý odstín pleti',
    '💆🏼' => 'osoba na masáži obličeje: středně světlý odstín pleti',
    '💆🏽' => 'osoba na masáži obličeje: střední odstín pleti',
    '💆🏾' => 'osoba na masáži obličeje: středně tmavý odstín pleti',
    '💆🏿' => 'osoba na masáži obličeje: tmavý odstín pleti',
    '💇🏻' => 'osoba u kadeřníka: světlý odstín pleti',
    '💇🏼' => 'osoba u kadeřníka: středně světlý odstín pleti',
    '💇🏽' => 'osoba u kadeřníka: střední odstín pleti',
    '💇🏾' => 'osoba u kadeřníka: středně tmavý odstín pleti',
    '💇🏿' => 'osoba u kadeřníka: tmavý odstín pleti',
    '🚶🏻' => 'kráčející osoba: světlý odstín pleti',
    '🚶🏼' => 'kráčející osoba: středně světlý odstín pleti',
    '🚶🏽' => 'kráčející osoba: střední odstín pleti',
    '🚶🏾' => 'kráčející osoba: středně tmavý odstín pleti',
    '🚶🏿' => 'kráčející osoba: tmavý odstín pleti',
    '🧍🏻' => 'stojící osoba: světlý odstín pleti',
    '🧍🏼' => 'stojící osoba: středně světlý odstín pleti',
    '🧍🏽' => 'stojící osoba: střední odstín pleti',
    '🧍🏾' => 'stojící osoba: středně tmavý odstín pleti',
    '🧍🏿' => 'stojící osoba: tmavý odstín pleti',
    '🧎🏻' => 'klečící osoba: světlý odstín pleti',
    '🧎🏼' => 'klečící osoba: středně světlý odstín pleti',
    '🧎🏽' => 'klečící osoba: střední odstín pleti',
    '🧎🏾' => 'klečící osoba: středně tmavý odstín pleti',
    '🧎🏿' => 'klečící osoba: tmavý odstín pleti',
    '🏃🏻' => 'běžící osoba: světlý odstín pleti',
    '🏃🏼' => 'běžící osoba: středně světlý odstín pleti',
    '🏃🏽' => 'běžící osoba: střední odstín pleti',
    '🏃🏾' => 'běžící osoba: středně tmavý odstín pleti',
    '🏃🏿' => 'běžící osoba: tmavý odstín pleti',
    '💃🏻' => 'tanečnice: světlý odstín pleti',
    '💃🏼' => 'tanečnice: středně světlý odstín pleti',
    '💃🏽' => 'tanečnice: střední odstín pleti',
    '💃🏾' => 'tanečnice: středně tmavý odstín pleti',
    '💃🏿' => 'tanečnice: tmavý odstín pleti',
    '🕺🏻' => 'tanečník: světlý odstín pleti',
    '🕺🏼' => 'tanečník: středně světlý odstín pleti',
    '🕺🏽' => 'tanečník: střední odstín pleti',
    '🕺🏾' => 'tanečník: středně tmavý odstín pleti',
    '🕺🏿' => 'tanečník: tmavý odstín pleti',
    '🕴🏻' => 'levitující byznysmen: světlý odstín pleti',
    '🕴🏼' => 'levitující byznysmen: středně světlý odstín pleti',
    '🕴🏽' => 'levitující byznysmen: střední odstín pleti',
    '🕴🏾' => 'levitující byznysmen: středně tmavý odstín pleti',
    '🕴🏿' => 'levitující byznysmen: tmavý odstín pleti',
    '👯🏻' => 'osoby se zaječími oušky: světlý odstín pleti',
    '👯🏼' => 'osoby se zaječími oušky: středně světlý odstín pleti',
    '👯🏽' => 'osoby se zaječími oušky: střední odstín pleti',
    '👯🏾' => 'osoby se zaječími oušky: středně tmavý odstín pleti',
    '👯🏿' => 'osoby se zaječími oušky: tmavý odstín pleti',
    '🧖🏻' => 'osoba v sauně: světlý odstín pleti',
    '🧖🏼' => 'osoba v sauně: středně světlý odstín pleti',
    '🧖🏽' => 'osoba v sauně: střední odstín pleti',
    '🧖🏾' => 'osoba v sauně: středně tmavý odstín pleti',
    '🧖🏿' => 'osoba v sauně: tmavý odstín pleti',
    '🧗🏻' => 'lezec: světlý odstín pleti',
    '🧗🏼' => 'lezec: středně světlý odstín pleti',
    '🧗🏽' => 'lezec: střední odstín pleti',
    '🧗🏾' => 'lezec: středně tmavý odstín pleti',
    '🧗🏿' => 'lezec: tmavý odstín pleti',
    '🏇🏻' => 'žokej na koni: světlý odstín pleti',
    '🏇🏼' => 'žokej na koni: středně světlý odstín pleti',
    '🏇🏽' => 'žokej na koni: střední odstín pleti',
    '🏇🏾' => 'žokej na koni: středně tmavý odstín pleti',
    '🏇🏿' => 'žokej na koni: tmavý odstín pleti',
    '🏂🏻' => 'snowboardista: světlý odstín pleti',
    '🏂🏼' => 'snowboardista: středně světlý odstín pleti',
    '🏂🏽' => 'snowboardista: střední odstín pleti',
    '🏂🏾' => 'snowboardista: středně tmavý odstín pleti',
    '🏂🏿' => 'snowboardista: tmavý odstín pleti',
    '🏌🏻' => 'osoba hrající golf: světlý odstín pleti',
    '🏌🏼' => 'osoba hrající golf: středně světlý odstín pleti',
    '🏌🏽' => 'osoba hrající golf: střední odstín pleti',
    '🏌🏾' => 'osoba hrající golf: středně tmavý odstín pleti',
    '🏌🏿' => 'osoba hrající golf: tmavý odstín pleti',
    '🏄🏻' => 'osoba na surfu: světlý odstín pleti',
    '🏄🏼' => 'osoba na surfu: středně světlý odstín pleti',
    '🏄🏽' => 'osoba na surfu: střední odstín pleti',
    '🏄🏾' => 'osoba na surfu: středně tmavý odstín pleti',
    '🏄🏿' => 'osoba na surfu: tmavý odstín pleti',
    '🚣🏻' => 'veslující osoba: světlý odstín pleti',
    '🚣🏼' => 'veslující osoba: středně světlý odstín pleti',
    '🚣🏽' => 'veslující osoba: střední odstín pleti',
    '🚣🏾' => 'veslující osoba: středně tmavý odstín pleti',
    '🚣🏿' => 'veslující osoba: tmavý odstín pleti',
    '🏊🏻' => 'plavající osoba: světlý odstín pleti',
    '🏊🏼' => 'plavající osoba: středně světlý odstín pleti',
    '🏊🏽' => 'plavající osoba: střední odstín pleti',
    '🏊🏾' => 'plavající osoba: středně tmavý odstín pleti',
    '🏊🏿' => 'plavající osoba: tmavý odstín pleti',
    '⛹🏻' => 'osoba driblující s míčem: světlý odstín pleti',
    '⛹🏼' => 'osoba driblující s míčem: středně světlý odstín pleti',
    '⛹🏽' => 'osoba driblující s míčem: střední odstín pleti',
    '⛹🏾' => 'osoba driblující s míčem: středně tmavý odstín pleti',
    '⛹🏿' => 'osoba driblující s míčem: tmavý odstín pleti',
    '🏋🏻' => 'osoba vzpírající činku: světlý odstín pleti',
    '🏋🏼' => 'osoba vzpírající činku: středně světlý odstín pleti',
    '🏋🏽' => 'osoba vzpírající činku: střední odstín pleti',
    '🏋🏾' => 'osoba vzpírající činku: středně tmavý odstín pleti',
    '🏋🏿' => 'osoba vzpírající činku: tmavý odstín pleti',
    '🚴🏻' => 'osoba na kole: světlý odstín pleti',
    '🚴🏼' => 'osoba na kole: středně světlý odstín pleti',
    '🚴🏽' => 'osoba na kole: střední odstín pleti',
    '🚴🏾' => 'osoba na kole: středně tmavý odstín pleti',
    '🚴🏿' => 'osoba na kole: tmavý odstín pleti',
    '🚵🏻' => 'osoba na horském kole: světlý odstín pleti',
    '🚵🏼' => 'osoba na horském kole: středně světlý odstín pleti',
    '🚵🏽' => 'osoba na horském kole: střední odstín pleti',
    '🚵🏾' => 'osoba na horském kole: středně tmavý odstín pleti',
    '🚵🏿' => 'osoba na horském kole: tmavý odstín pleti',
    '🤸🏻' => 'osoba dělající přemet stranou: světlý odstín pleti',
    '🤸🏼' => 'osoba dělající přemet stranou: středně světlý odstín pleti',
    '🤸🏽' => 'osoba dělající přemet stranou: střední odstín pleti',
    '🤸🏾' => 'osoba dělající přemet stranou: středně tmavý odstín pleti',
    '🤸🏿' => 'osoba dělající přemet stranou: tmavý odstín pleti',
    '🤼🏻' => 'zápasící dvojice: světlý odstín pleti',
    '🤼🏼' => 'zápasící dvojice: středně světlý odstín pleti',
    '🤼🏽' => 'zápasící dvojice: střední odstín pleti',
    '🤼🏾' => 'zápasící dvojice: středně tmavý odstín pleti',
    '🤼🏿' => 'zápasící dvojice: tmavý odstín pleti',
    '🤽🏻' => 'osoba hrající vodní pólo: světlý odstín pleti',
    '🤽🏼' => 'osoba hrající vodní pólo: středně světlý odstín pleti',
    '🤽🏽' => 'osoba hrající vodní pólo: střední odstín pleti',
    '🤽🏾' => 'osoba hrající vodní pólo: středně tmavý odstín pleti',
    '🤽🏿' => 'osoba hrající vodní pólo: tmavý odstín pleti',
    '🤾🏻' => 'osoba hrající házenou: světlý odstín pleti',
    '🤾🏼' => 'osoba hrající házenou: středně světlý odstín pleti',
    '🤾🏽' => 'osoba hrající házenou: střední odstín pleti',
    '🤾🏾' => 'osoba hrající házenou: středně tmavý odstín pleti',
    '🤾🏿' => 'osoba hrající házenou: tmavý odstín pleti',
    '🤹🏻' => 'žonglující osoba: světlý odstín pleti',
    '🤹🏼' => 'žonglující osoba: středně světlý odstín pleti',
    '🤹🏽' => 'žonglující osoba: střední odstín pleti',
    '🤹🏾' => 'žonglující osoba: středně tmavý odstín pleti',
    '🤹🏿' => 'žonglující osoba: tmavý odstín pleti',
    '🧘🏻' => 'osoba v lotosovém sedu: světlý odstín pleti',
    '🧘🏼' => 'osoba v lotosovém sedu: středně světlý odstín pleti',
    '🧘🏽' => 'osoba v lotosovém sedu: střední odstín pleti',
    '🧘🏾' => 'osoba v lotosovém sedu: středně tmavý odstín pleti',
    '🧘🏿' => 'osoba v lotosovém sedu: tmavý odstín pleti',
    '🛀🏻' => 'osoba ve vaně: světlý odstín pleti',
    '🛀🏼' => 'osoba ve vaně: středně světlý odstín pleti',
    '🛀🏽' => 'osoba ve vaně: střední odstín pleti',
    '🛀🏾' => 'osoba ve vaně: středně tmavý odstín pleti',
    '🛀🏿' => 'osoba ve vaně: tmavý odstín pleti',
    '🛌🏻' => 'osoba v posteli: světlý odstín pleti',
    '🛌🏼' => 'osoba v posteli: středně světlý odstín pleti',
    '🛌🏽' => 'osoba v posteli: střední odstín pleti',
    '🛌🏾' => 'osoba v posteli: středně tmavý odstín pleti',
    '🛌🏿' => 'osoba v posteli: tmavý odstín pleti',
    '👭🏻' => 'dvě ženy držící se za ruce: světlý odstín pleti',
    '👭🏼' => 'dvě ženy držící se za ruce: středně světlý odstín pleti',
    '👭🏽' => 'dvě ženy držící se za ruce: střední odstín pleti',
    '👭🏾' => 'dvě ženy držící se za ruce: středně tmavý odstín pleti',
    '👭🏿' => 'dvě ženy držící se za ruce: tmavý odstín pleti',
    '👫🏻' => 'muž a žena držící se za ruce: světlý odstín pleti',
    '👫🏼' => 'muž a žena držící se za ruce: středně světlý odstín pleti',
    '👫🏽' => 'muž a žena držící se za ruce: střední odstín pleti',
    '👫🏾' => 'muž a žena držící se za ruce: středně tmavý odstín pleti',
    '👫🏿' => 'muž a žena držící se za ruce: tmavý odstín pleti',
    '👬🏻' => 'dva muži držící se za ruce: světlý odstín pleti',
    '👬🏼' => 'dva muži držící se za ruce: středně světlý odstín pleti',
    '👬🏽' => 'dva muži držící se za ruce: střední odstín pleti',
    '👬🏾' => 'dva muži držící se za ruce: středně tmavý odstín pleti',
    '👬🏿' => 'dva muži držící se za ruce: tmavý odstín pleti',
    '💏🏻' => 'polibek: světlý odstín pleti',
    '💏🏼' => 'polibek: středně světlý odstín pleti',
    '💏🏽' => 'polibek: střední odstín pleti',
    '💏🏾' => 'polibek: středně tmavý odstín pleti',
    '💏🏿' => 'polibek: tmavý odstín pleti',
    '💑🏻' => 'pár se srdcem: světlý odstín pleti',
    '💑🏼' => 'pár se srdcem: středně světlý odstín pleti',
    '💑🏽' => 'pár se srdcem: střední odstín pleti',
    '💑🏾' => 'pár se srdcem: středně tmavý odstín pleti',
    '💑🏿' => 'pár se srdcem: tmavý odstín pleti',
    '🇦🇨' => 'vlajka: Ascension',
    '🇦🇩' => 'vlajka: Andorra',
    '🇦🇪' => 'vlajka: Spojené arabské emiráty',
    '🇦🇫' => 'vlajka: Afghánistán',
    '🇦🇬' => 'vlajka: Antigua a Barbuda',
    '🇦🇮' => 'vlajka: Anguilla',
    '🇦🇱' => 'vlajka: Albánie',
    '🇦🇲' => 'vlajka: Arménie',
    '🇦🇴' => 'vlajka: Angola',
    '🇦🇶' => 'vlajka: Antarktida',
    '🇦🇷' => 'vlajka: Argentina',
    '🇦🇸' => 'vlajka: Americká Samoa',
    '🇦🇹' => 'vlajka: Rakousko',
    '🇦🇺' => 'vlajka: Austrálie',
    '🇦🇼' => 'vlajka: Aruba',
    '🇦🇽' => 'vlajka: Ålandy',
    '🇦🇿' => 'vlajka: Ázerbájdžán',
    '🇧🇦' => 'vlajka: Bosna a Hercegovina',
    '🇧🇧' => 'vlajka: Barbados',
    '🇧🇩' => 'vlajka: Bangladéš',
    '🇧🇪' => 'vlajka: Belgie',
    '🇧🇫' => 'vlajka: Burkina Faso',
    '🇧🇬' => 'vlajka: Bulharsko',
    '🇧🇭' => 'vlajka: Bahrajn',
    '🇧🇮' => 'vlajka: Burundi',
    '🇧🇯' => 'vlajka: Benin',
    '🇧🇱' => 'vlajka: Svatý Bartoloměj',
    '🇧🇲' => 'vlajka: Bermudy',
    '🇧🇳' => 'vlajka: Brunej',
    '🇧🇴' => 'vlajka: Bolívie',
    '🇧🇶' => 'vlajka: Karibské Nizozemsko',
    '🇧🇷' => 'vlajka: Brazílie',
    '🇧🇸' => 'vlajka: Bahamy',
    '🇧🇹' => 'vlajka: Bhútán',
    '🇧🇻' => 'vlajka: Bouvetův ostrov',
    '🇧🇼' => 'vlajka: Botswana',
    '🇧🇾' => 'vlajka: Bělorusko',
    '🇧🇿' => 'vlajka: Belize',
    '🇨🇦' => 'vlajka: Kanada',
    '🇨🇨' => 'vlajka: Kokosové ostrovy',
    '🇨🇩' => 'vlajka: Kongo – Kinshasa',
    '🇨🇫' => 'vlajka: Středoafrická republika',
    '🇨🇬' => 'vlajka: Kongo – Brazzaville',
    '🇨🇭' => 'vlajka: Švýcarsko',
    '🇨🇮' => 'vlajka: Pobřeží slonoviny',
    '🇨🇰' => 'vlajka: Cookovy ostrovy',
    '🇨🇱' => 'vlajka: Chile',
    '🇨🇲' => 'vlajka: Kamerun',
    '🇨🇳' => 'vlajka: Čína',
    '🇨🇴' => 'vlajka: Kolumbie',
    '🇨🇵' => 'vlajka: Clippertonův ostrov',
    '🇨🇶' => 'vlajka: Sark',
    '🇨🇷' => 'vlajka: Kostarika',
    '🇨🇺' => 'vlajka: Kuba',
    '🇨🇻' => 'vlajka: Kapverdy',
    '🇨🇼' => 'vlajka: Curaçao',
    '🇨🇽' => 'vlajka: Vánoční ostrov',
    '🇨🇾' => 'vlajka: Kypr',
    '🇨🇿' => 'vlajka: Česko',
    '🇩🇪' => 'vlajka: Německo',
    '🇩🇬' => 'vlajka: Diego García',
    '🇩🇯' => 'vlajka: Džibutsko',
    '🇩🇰' => 'vlajka: Dánsko',
    '🇩🇲' => 'vlajka: Dominika',
    '🇩🇴' => 'vlajka: Dominikánská republika',
    '🇩🇿' => 'vlajka: Alžírsko',
    '🇪🇦' => 'vlajka: Ceuta a Melilla',
    '🇪🇨' => 'vlajka: Ekvádor',
    '🇪🇪' => 'vlajka: Estonsko',
    '🇪🇬' => 'vlajka: Egypt',
    '🇪🇭' => 'vlajka: Západní Sahara',
    '🇪🇷' => 'vlajka: Eritrea',
    '🇪🇸' => 'vlajka: Španělsko',
    '🇪🇹' => 'vlajka: Etiopie',
    '🇪🇺' => 'vlajka: Evropská unie',
    '🇫🇮' => 'vlajka: Finsko',
    '🇫🇯' => 'vlajka: Fidži',
    '🇫🇰' => 'vlajka: Falklandské ostrovy',
    '🇫🇲' => 'vlajka: Mikronésie',
    '🇫🇴' => 'vlajka: Faerské ostrovy',
    '🇫🇷' => 'vlajka: Francie',
    '🇬🇦' => 'vlajka: Gabon',
    '🇬🇧' => 'vlajka: Spojené království',
    '🇬🇩' => 'vlajka: Grenada',
    '🇬🇪' => 'vlajka: Gruzie',
    '🇬🇫' => 'vlajka: Francouzská Guyana',
    '🇬🇬' => 'vlajka: Guernsey',
    '🇬🇭' => 'vlajka: Ghana',
    '🇬🇮' => 'vlajka: Gibraltar',
    '🇬🇱' => 'vlajka: Grónsko',
    '🇬🇲' => 'vlajka: Gambie',
    '🇬🇳' => 'vlajka: Guinea',
    '🇬🇵' => 'vlajka: Guadeloupe',
    '🇬🇶' => 'vlajka: Rovníková Guinea',
    '🇬🇷' => 'vlajka: Řecko',
    '🇬🇸' => 'vlajka: Jižní Georgie a Jižní Sandwichovy ostrovy',
    '🇬🇹' => 'vlajka: Guatemala',
    '🇬🇺' => 'vlajka: Guam',
    '🇬🇼' => 'vlajka: Guinea-Bissau',
    '🇬🇾' => 'vlajka: Guyana',
    '🇭🇰' => 'vlajka: Hongkong – ZAO Číny',
    '🇭🇲' => 'vlajka: Heardův ostrov a McDonaldovy ostrovy',
    '🇭🇳' => 'vlajka: Honduras',
    '🇭🇷' => 'vlajka: Chorvatsko',
    '🇭🇹' => 'vlajka: Haiti',
    '🇭🇺' => 'vlajka: Maďarsko',
    '🇮🇨' => 'vlajka: Kanárské ostrovy',
    '🇮🇩' => 'vlajka: Indonésie',
    '🇮🇪' => 'vlajka: Irsko',
    '🇮🇱' => 'vlajka: Izrael',
    '🇮🇲' => 'vlajka: Ostrov Man',
    '🇮🇳' => 'vlajka: Indie',
    '🇮🇴' => 'vlajka: Britské indickooceánské území',
    '🇮🇶' => 'vlajka: Irák',
    '🇮🇷' => 'vlajka: Írán',
    '🇮🇸' => 'vlajka: Island',
    '🇮🇹' => 'vlajka: Itálie',
    '🇯🇪' => 'vlajka: Jersey',
    '🇯🇲' => 'vlajka: Jamajka',
    '🇯🇴' => 'vlajka: Jordánsko',
    '🇯🇵' => 'vlajka: Japonsko',
    '🇰🇪' => 'vlajka: Keňa',
    '🇰🇬' => 'vlajka: Kyrgyzstán',
    '🇰🇭' => 'vlajka: Kambodža',
    '🇰🇮' => 'vlajka: Kiribati',
    '🇰🇲' => 'vlajka: Komory',
    '🇰🇳' => 'vlajka: Svatý Kryštof a Nevis',
    '🇰🇵' => 'vlajka: Severní Korea',
    '🇰🇷' => 'vlajka: Jižní Korea',
    '🇰🇼' => 'vlajka: Kuvajt',
    '🇰🇾' => 'vlajka: Kajmanské ostrovy',
    '🇰🇿' => 'vlajka: Kazachstán',
    '🇱🇦' => 'vlajka: Laos',
    '🇱🇧' => 'vlajka: Libanon',
    '🇱🇨' => 'vlajka: Svatá Lucie',
    '🇱🇮' => 'vlajka: Lichtenštejnsko',
    '🇱🇰' => 'vlajka: Srí Lanka',
    '🇱🇷' => 'vlajka: Libérie',
    '🇱🇸' => 'vlajka: Lesotho',
    '🇱🇹' => 'vlajka: Litva',
    '🇱🇺' => 'vlajka: Lucembursko',
    '🇱🇻' => 'vlajka: Lotyšsko',
    '🇱🇾' => 'vlajka: Libye',
    '🇲🇦' => 'vlajka: Maroko',
    '🇲🇨' => 'vlajka: Monako',
    '🇲🇩' => 'vlajka: Moldavsko',
    '🇲🇪' => 'vlajka: Černá Hora',
    '🇲🇫' => 'vlajka: Svatý Martin (Francie)',
    '🇲🇬' => 'vlajka: Madagaskar',
    '🇲🇭' => 'vlajka: Marshallovy ostrovy',
    '🇲🇰' => 'vlajka: Severní Makedonie',
    '🇲🇱' => 'vlajka: Mali',
    '🇲🇲' => 'vlajka: Myanmar (Barma)',
    '🇲🇳' => 'vlajka: Mongolsko',
    '🇲🇴' => 'vlajka: Macao – ZAO Číny',
    '🇲🇵' => 'vlajka: Severní Mariany',
    '🇲🇶' => 'vlajka: Martinik',
    '🇲🇷' => 'vlajka: Mauritánie',
    '🇲🇸' => 'vlajka: Montserrat',
    '🇲🇹' => 'vlajka: Malta',
    '🇲🇺' => 'vlajka: Mauricius',
    '🇲🇻' => 'vlajka: Maledivy',
    '🇲🇼' => 'vlajka: Malawi',
    '🇲🇽' => 'vlajka: Mexiko',
    '🇲🇾' => 'vlajka: Malajsie',
    '🇲🇿' => 'vlajka: Mosambik',
    '🇳🇦' => 'vlajka: Namibie',
    '🇳🇨' => 'vlajka: Nová Kaledonie',
    '🇳🇪' => 'vlajka: Niger',
    '🇳🇫' => 'vlajka: Norfolk',
    '🇳🇬' => 'vlajka: Nigérie',
    '🇳🇮' => 'vlajka: Nikaragua',
    '🇳🇱' => 'vlajka: Nizozemsko',
    '🇳🇴' => 'vlajka: Norsko',
    '🇳🇵' => 'vlajka: Nepál',
    '🇳🇷' => 'vlajka: Nauru',
    '🇳🇺' => 'vlajka: Niue',
    '🇳🇿' => 'vlajka: Nový Zéland',
    '🇴🇲' => 'vlajka: Omán',
    '🇵🇦' => 'vlajka: Panama',
    '🇵🇪' => 'vlajka: Peru',
    '🇵🇫' => 'vlajka: Francouzská Polynésie',
    '🇵🇬' => 'vlajka: Papua-Nová Guinea',
    '🇵🇭' => 'vlajka: Filipíny',
    '🇵🇰' => 'vlajka: Pákistán',
    '🇵🇱' => 'vlajka: Polsko',
    '🇵🇲' => 'vlajka: Saint-Pierre a Miquelon',
    '🇵🇳' => 'vlajka: Pitcairnovy ostrovy',
    '🇵🇷' => 'vlajka: Portoriko',
    '🇵🇸' => 'vlajka: Palestinská území',
    '🇵🇹' => 'vlajka: Portugalsko',
    '🇵🇼' => 'vlajka: Palau',
    '🇵🇾' => 'vlajka: Paraguay',
    '🇶🇦' => 'vlajka: Katar',
    '🇷🇪' => 'vlajka: Réunion',
    '🇷🇴' => 'vlajka: Rumunsko',
    '🇷🇸' => 'vlajka: Srbsko',
    '🇷🇺' => 'vlajka: Rusko',
    '🇷🇼' => 'vlajka: Rwanda',
    '🇸🇦' => 'vlajka: Saúdská Arábie',
    '🇸🇧' => 'vlajka: Šalamounovy ostrovy',
    '🇸🇨' => 'vlajka: Seychely',
    '🇸🇩' => 'vlajka: Súdán',
    '🇸🇪' => 'vlajka: Švédsko',
    '🇸🇬' => 'vlajka: Singapur',
    '🇸🇭' => 'vlajka: Svatá Helena',
    '🇸🇮' => 'vlajka: Slovinsko',
    '🇸🇯' => 'vlajka: Špicberky a Jan Mayen',
    '🇸🇰' => 'vlajka: Slovensko',
    '🇸🇱' => 'vlajka: Sierra Leone',
    '🇸🇲' => 'vlajka: San Marino',
    '🇸🇳' => 'vlajka: Senegal',
    '🇸🇴' => 'vlajka: Somálsko',
    '🇸🇷' => 'vlajka: Surinam',
    '🇸🇸' => 'vlajka: Jižní Súdán',
    '🇸🇹' => 'vlajka: Svatý Tomáš a Princův ostrov',
    '🇸🇻' => 'vlajka: Salvador',
    '🇸🇽' => 'vlajka: Svatý Martin (Nizozemsko)',
    '🇸🇾' => 'vlajka: Sýrie',
    '🇸🇿' => 'vlajka: Eswatini',
    '🇹🇦' => 'vlajka: Tristan da Cunha',
    '🇹🇨' => 'vlajka: Turks a Caicos',
    '🇹🇩' => 'vlajka: Čad',
    '🇹🇫' => 'vlajka: Francouzská jižní území',
    '🇹🇬' => 'vlajka: Togo',
    '🇹🇭' => 'vlajka: Thajsko',
    '🇹🇯' => 'vlajka: Tádžikistán',
    '🇹🇰' => 'vlajka: Tokelau',
    '🇹🇱' => 'vlajka: Východní Timor',
    '🇹🇲' => 'vlajka: Turkmenistán',
    '🇹🇳' => 'vlajka: Tunisko',
    '🇹🇴' => 'vlajka: Tonga',
    '🇹🇷' => 'vlajka: Turecko',
    '🇹🇹' => 'vlajka: Trinidad a Tobago',
    '🇹🇻' => 'vlajka: Tuvalu',
    '🇹🇼' => 'vlajka: Tchaj-wan',
    '🇹🇿' => 'vlajka: Tanzanie',
    '🇺🇦' => 'vlajka: Ukrajina',
    '🇺🇬' => 'vlajka: Uganda',
    '🇺🇲' => 'vlajka: Menší odlehlé ostrovy USA',
    '🇺🇳' => 'vlajka: Organizace spojených národů',
    '🇺🇸' => 'vlajka: Spojené státy',
    '🇺🇾' => 'vlajka: Uruguay',
    '🇺🇿' => 'vlajka: Uzbekistán',
    '🇻🇦' => 'vlajka: Vatikán',
    '🇻🇨' => 'vlajka: Svatý Vincenc a Grenadiny',
    '🇻🇪' => 'vlajka: Venezuela',
    '🇻🇬' => 'vlajka: Britské Panenské ostrovy',
    '🇻🇮' => 'vlajka: Americké Panenské ostrovy',
    '🇻🇳' => 'vlajka: Vietnam',
    '🇻🇺' => 'vlajka: Vanuatu',
    '🇼🇫' => 'vlajka: Wallis a Futuna',
    '🇼🇸' => 'vlajka: Samoa',
    '🇽🇰' => 'vlajka: Kosovo',
    '🇾🇪' => 'vlajka: Jemen',
    '🇾🇹' => 'vlajka: Mayotte',
    '🇿🇦' => 'vlajka: Jihoafrická republika',
    '🇿🇲' => 'vlajka: Zambie',
    '🇿🇼' => 'vlajka: Zimbabwe',
    '🔟' => 'klávesa: 10',
    '🏻' => 'světlý odstín pleti',
    '🏼' => 'středně světlý odstín pleti',
    '🏽' => 'střední odstín pleti',
    '🏾' => 'středně tmavý odstín pleti',
    '🏿' => 'tmavý odstín pleti',
    '😀' => 'zubící se obličej',
    '😃' => 'zubící se obličej s velkýma očima',
    '😄' => 'zubící se obličej s usměvavýma očima',
    '😁' => 'rozzářený obličej s usměvavýma očima',
    '😆' => 'zubící se obličej s mžourajícíma očima',
    '😅' => 'zubící se obličej s kapkou potu',
    '🤣' => 'obličej válející se smíchy',
    '😂' => 'obličej slzící smíchy',
    '🙂' => 'obličej s mírným úsměvem',
    '🙃' => 'převrácený obličej',
    '🫠' => 'tající obličej',
    '😉' => 'mrkající obličej',
    '😊' => 'smějící se obličej s usměvavýma očima',
    '😇' => 'usmívající se obličej se svatozáří',
    '🥰' => 'usmívající se obličej se třemi srdíčky',
    '😍' => 'usmívající se obličej se srdíčky místo očí',
    '🤩' => 'obličej s hvězdami v očích',
    '😘' => 'obličej posílající polibek',
    '😗' => 'líbající obličej',
    '☺' => 'usmívající se obličej',
    '😚' => 'líbající obličej se zavřenýma očima',
    '😙' => 'líbající obličej s usměvavýma očima',
    '🥲' => 'usmívající se obličej se slzou',
    '😋' => 'olizující se obličej',
    '😛' => 'obličej s vyplazeným jazykem',
    '😜' => 'mrkající obličej s vypláznutým jazykem',
    '🤪' => 'šílený obličej',
    '😝' => 'mžourající obličej s vypláznutým jazykem',
    '🤑' => 'obličej s penězi na jazyku',
    '🤗' => 'obličej s otevřenou náručí',
    '🤭' => 'obličej s dlaní před pusou',
    '🫢' => 'obličej s otevřenýma očima a dlaní před pusou',
    '🫣' => 'obličej vykukující jedním okem',
    '🤫' => 'obličej s prstem na puse',
    '🤔' => 'zamyšlený obličej',
    '🫡' => 'salutující obličej',
    '🤐' => 'obličej s pusou na zip',
    '🤨' => 'obličej se zvednutým obočím',
    '😐' => 'obličej s neutrálním výrazem',
    '😑' => 'obličej bez výrazu',
    '😶' => 'obličej bez úst',
    '🫥' => 'obličej s tečkovaným obrysem',
    '😏' => 'ušklíbající se obličej',
    '😒' => 'obličej se znuděným výrazem',
    '🙄' => 'obličej s očima v sloup',
    '😬' => 'grimasa s vyceněnými zuby',
    '🤥' => 'obličej lháře',
    '🫨' => 'třesoucí se obličej',
    '😌' => 'obličej s výrazem úlevy',
    '😔' => 'obličej se skleslým výrazem',
    '😪' => 'ospalý obličej',
    '🤤' => 'slintající obličej',
    '😴' => 'spící obličej',
    '🫩' => 'obličej s kruhy pod očima',
    '😷' => 'obličej s lékařskou rouškou',
    '🤒' => 'obličej s teploměrem',
    '🤕' => 'obličej s ovázanou hlavou',
    '🤢' => 'obličej se znechuceným výrazem',
    '🤮' => 'zvracející obličej',
    '🤧' => 'kýchající obličej',
    '🥵' => 'rozpálený obličej',
    '🥶' => 'zmrzlý obličej',
    '🥴' => 'zpitomělý obličej',
    '😵' => 'obličej s výrazem závrati',
    '🤯' => 'explodující hlava',
    '🤠' => 'obličej s kovbojským kloboukem',
    '🥳' => 'oslavující obličej',
    '🥸' => 'obličej v přestrojení',
    '😎' => 'usmívající se obličej s brýlemi',
    '🤓' => 'obličej šprta',
    '🧐' => 'obličej s brýlovým monoklem',
    '😕' => 'zmatený obličej',
    '🫤' => 'obličej s pusou našikmo',
    '😟' => 'ustaraný obličej',
    '🙁' => 'mírně zamračený obličej',
    '☹' => 'zamračený obličej',
    '😮' => 'obličej s otevřenou pusou',
    '😯' => 'obličej se zaraženým výrazem',
    '😲' => 'užaslý obličej',
    '😳' => 'červenající se obličej',
    '🫪' => 'pokřivený obličej',
    '🥺' => 'prosebný obličej',
    '🥹' => 'obličej se slzami na krajíčku',
    '😦' => 'zamračený obličej s otevřenou pusou',
    '😧' => 'obličej s úzkostným výrazem',
    '😨' => 'ustrašený obličej',
    '😰' => 'úzkostný obličej se studeným potem',
    '😥' => 'smutný obličej s výrazem úlevy',
    '😢' => 'plačící obličej',
    '😭' => 'zoufale plačící obličej',
    '😱' => 'obličej křičící hrůzou',
    '😖' => 'obličej s pláčem na krajíčku',
    '😣' => 'obličej s umíněným výrazem',
    '😞' => 'zklamaný obličej',
    '😓' => 'sklíčený obličej se studeným potem',
    '😩' => 'vyčerpaný obličej',
    '😫' => 'unavený obličej',
    '🥱' => 'zívající obličej',
    '😤' => 'obličej pouštějící páru nosem',
    '😡' => 'obličej s podrážděným výrazem',
    '😠' => 'rozzlobený obličej',
    '🤬' => 'obličej se symboly na puse',
    '😈' => 'usmívající se obličej s rohy',
    '👿' => 'rozzlobený obličej s rohy',
    '💀' => 'lebka',
    '☠' => 'lebka a zkřížené hnáty',
    '💩' => 'hovínko',
    '🤡' => 'klaunský obličej',
    '👹' => 'zlobr',
    '👺' => 'šotek',
    '👻' => 'duch',
    '👽' => 'mimozemšťan',
    '👾' => 'mimozemská příšera',
    '🤖' => 'obličej robota',
    '😺' => 'šklebící se kočičí obličej',
    '😸' => 'šklebící se kočka s usměvavýma očima',
    '😹' => 'kočičí obličej slzící smíchy',
    '😻' => 'usmívající se kočka se srdíčky místo očí',
    '😼' => 'kočičí obličej s jízlivým úsměvem',
    '😽' => 'líbající kočičí obličej',
    '🙀' => 'vyčerpaný kočičí obličej',
    '😿' => 'plačící kočičí obličej',
    '😾' => 'kočičí obličej s podrážděným výrazem',
    '🙈' => 'opička „nic nevidím“',
    '🙉' => 'opička „nic neslyším“',
    '🙊' => 'opička „nic nepovím“',
    '💌' => 'milostný dopis',
    '💘' => 'srdce se šípem',
    '💝' => 'srdce s mašlí',
    '💖' => 'jiskřící srdce',
    '💗' => 'rostoucí srdce',
    '💓' => 'bušící srdce',
    '💞' => 'srdce vířící v kole',
    '💕' => 'dvě srdce',
    '💟' => 'srdce jako ozdoba',
    '❣' => 'srdce jako vykřičník',
    '💔' => 'zlomené srdce',
    '❤' => 'rudé srdce',
    '🩷' => 'růžové srdce',
    '🧡' => 'oranžové srdce',
    '💛' => 'žluté srdce',
    '💚' => 'zelené srdce',
    '💙' => 'modré srdce',
    '🩵' => 'světlemodré srdce',
    '💜' => 'fialové srdce',
    '🤎' => 'hnědé srdce',
    '🖤' => 'černé srdce',
    '🩶' => 'šedé srdce',
    '🤍' => 'bílé srdce',
    '💋' => 'otisk rtů',
    '💯' => 'sto bodů',
    '💢' => 'japonský symbol hněvu',
    '🫯' => 'oblak sršící hádkou',
    '💥' => 'srážka',
    '💫' => 'závrať',
    '💦' => 'kapky potu',
    '💨' => 'spěch',
    '🕳' => 'díra',
    '💬' => 'řečová bublina',
    '🗨' => 'levá řečová bublina',
    '🗯' => 'pravá rozzlobená bublina',
    '💭' => 'myšlenková bublina',
    '💤' => 'zzz',
    '👋' => 'mávající ruka',
    '🤚' => 'hřbet zvednuté ruky',
    '🖐' => 'ruka s roztaženými prsty',
    '✋' => 'zvednutá ruka',
    '🖖' => 'vulkánský pozdrav',
    '🫱' => 'ruka otočená doprava',
    '🫲' => 'ruka otočená doleva',
    '🫳' => 'ruka otočená dlaní dolů',
    '🫴' => 'ruka otočená dlaní vzhůru',
    '🫷' => 'ruka tlačící vlevo',
    '🫸' => 'ruka tlačící vpravo',
    '👌' => 'gesto OK',
    '🤌' => 'špetka z pěti prstů',
    '🤏' => 'gesto „málo“',
    '✌' => 'gesto vítězství',
    '🤞' => 'zkřížené prsty',
    '🫰' => 'ruka se zkříženým palcem a ukazováčkem',
    '🤟' => 'gesto „miluji tě“',
    '🤘' => 'gesto parohů',
    '🤙' => 'gesto „zavolej mi“',
    '👈' => 'ruka s prstem ukazujícím vlevo',
    '👉' => 'ruka s prstem ukazujícím vpravo',
    '👆' => 'ruka s prstem ukazujícím nahoru',
    '🖕' => 'prostředníček',
    '👇' => 'ruka s prstem ukazujícím dolů',
    '☝' => 'ruka ukazující nahoru',
    '🫵' => 'ukazováček namířený na diváka',
    '👍' => 'palec nahoru',
    '👎' => 'palec dolů',
    '✊' => 'zvednutá pěst',
    '👊' => 'namířená pěst',
    '🤛' => 'pěst mířící doleva',
    '🤜' => 'pěst mířící doprava',
    '👏' => 'tleskající ruce',
    '🙌' => 'zvednuté ruce',
    '🫶' => 'ruce spojené do tvaru srdce',
    '👐' => 'rozevřené dlaně',
    '🤲' => 'zdvižené dlaně vedle sebe',
    '🤝' => 'stisk rukou',
    '🙏' => 'sepjaté ruce',
    '✍' => 'píšící ruka',
    '💅' => 'lak na nehty',
    '🤳' => 'selfie',
    '💪' => 'zatnutý biceps',
    '🦾' => 'mechanická paže',
    '🦿' => 'mechanická noha',
    '🦵' => 'noha',
    '🦶' => 'chodidlo',
    '👂' => 'ucho',
    '🦻' => 'ucho s naslouchátkem',
    '👃' => 'nos',
    '🧠' => 'mozek',
    '🫀' => 'srdeční orgán',
    '🫁' => 'plíce',
    '🦷' => 'zub',
    '🦴' => 'kost',
    '👀' => 'oči',
    '👁' => 'oko',
    '👅' => 'jazyk',
    '👄' => 'ústa',
    '🫦' => 'skousnutý ret',
    '👶' => 'nemluvně',
    '🧒' => 'dítě',
    '👦' => 'chlapec',
    '👧' => 'dívka',
    '🧑' => 'dospělý člověk',
    '👱' => 'člověk s blond vlasy',
    '👨' => 'muž',
    '🧔' => 'osoba s plnovousem',
    '👩' => 'žena',
    '🧓' => 'starší dospělý člověk',
    '👴' => 'starý muž',
    '👵' => 'stará žena',
    '🙍' => 'zamračená osoba',
    '🙎' => 'podrážděná osoba',
    '🙅' => 'gesto nesouhlasu',
    '🙆' => 'gesto souhlasu',
    '💁' => 'osoba s dlaní vzhůru',
    '🙋' => 'osoba se zvednutou rukou',
    '🧏' => 'hluchá osoba',
    '🙇' => 'uklánějící se osoba',
    '🤦' => 'osoba chytající se za hlavu',
    '🤷' => 'osoba krčící rameny',
    '👮' => 'strážník',
    '🕵' => 'detektiv',
    '💂' => 'gardista',
    '🥷' => 'nindža',
    '👷' => 'dělník na stavbě',
    '🫅' => 'člověk s korunou na hlavě',
    '🤴' => 'princ',
    '👸' => 'princezna',
    '👳' => 'člověk s turbanem',
    '👲' => 'muž s čínskou čepicí',
    '🧕' => 'žena v šátku',
    '🤵' => 'osoba ve smokingu',
    '👰' => 'osoba se závojem',
    '🤰' => 'těhotná žena',
    '🫃' => 'těhotný muž',
    '🫄' => 'těhotná osoba',
    '🤱' => 'kojení',
    '👼' => 'andílek',
    '🎅' => 'Santa Claus',
    '🤶' => 'Santa Clausová',
    '🦸' => 'superhrdinská postava',
    '🦹' => 'superzločinec',
    '🧙' => 'mág',
    '🧚' => 'víla',
    '🧛' => 'obličej upíra',
    '🧜' => 'mořská bytost',
    '🧝' => 'obličej elfa',
    '🧞' => 'duch z lahve',
    '🧟' => 'zombie',
    '🧌' => 'troll',
    '🫈' => 'chlupatý tvor',
    '💆' => 'osoba na masáži obličeje',
    '💇' => 'osoba u kadeřníka',
    '🚶' => 'kráčející osoba',
    '🧍' => 'stojící osoba',
    '🧎' => 'klečící osoba',
    '🏃' => 'běžící osoba',
    '💃' => 'tanečnice',
    '🕺' => 'tanečník',
    '🕴' => 'levitující byznysmen',
    '👯' => 'osoby se zaječími oušky',
    '🧖' => 'osoba v sauně',
    '🧗' => 'lezec',
    '🤺' => 'šermíř',
    '🏇' => 'žokej na koni',
    '⛷' => 'lyžař',
    '🏂' => 'snowboardista',
    '🏌' => 'osoba hrající golf',
    '🏄' => 'osoba na surfu',
    '🚣' => 'veslující osoba',
    '🏊' => 'plavající osoba',
    '⛹' => 'osoba driblující s míčem',
    '🏋' => 'osoba vzpírající činku',
    '🚴' => 'osoba na kole',
    '🚵' => 'osoba na horském kole',
    '🤸' => 'osoba dělající přemet stranou',
    '🤼' => 'zápasící dvojice',
    '🤽' => 'osoba hrající vodní pólo',
    '🤾' => 'osoba hrající házenou',
    '🤹' => 'žonglující osoba',
    '🧘' => 'osoba v lotosovém sedu',
    '🛀' => 'osoba ve vaně',
    '🛌' => 'osoba v posteli',
    '👭' => 'dvě ženy držící se za ruce',
    '👫' => 'muž a žena držící se za ruce',
    '👬' => 'dva muži držící se za ruce',
    '💏' => 'polibek',
    '💑' => 'pár se srdcem',
    '🗣' => 'mluvící hlava',
    '👤' => 'silueta busty',
    '👥' => 'silueta dvojice bust',
    '🫂' => 'objímající se lidé',
    '👪' => 'rodina',
    '👣' => 'stopy',
    '🫆' => 'otisk prstu',
    '🦰' => 'zrzavé vlasy',
    '🦱' => 'kudrnaté vlasy',
    '🦳' => 'bílé vlasy',
    '🦲' => 'bez vlasů',
    '🐵' => 'hlava opice',
    '🐒' => 'opice',
    '🦍' => 'gorila',
    '🦧' => 'orangutan',
    '🐶' => 'hlava psa',
    '🐕' => 'pes',
    '🦮' => 'vodicí pes',
    '🐩' => 'pudl',
    '🐺' => 'hlava vlka',
    '🦊' => 'hlava lišky',
    '🦝' => 'mýval',
    '🐱' => 'hlava kočky',
    '🐈' => 'kočka',
    '🦁' => 'hlava lva',
    '🐯' => 'hlava tygra',
    '🐅' => 'tygr',
    '🐆' => 'leopard',
    '🐴' => 'hlava koně',
    '🫎' => 'los',
    '🫏' => 'osel',
    '🐎' => 'kůň',
    '🦄' => 'hlava jednorožce',
    '🦓' => 'zebra',
    '🦌' => 'jelen',
    '🦬' => 'bizon',
    '🐮' => 'hlava krávy',
    '🐂' => 'vůl',
    '🐃' => 'buvol domácí',
    '🐄' => 'kráva',
    '🐷' => 'hlava prasete',
    '🐖' => 'prase',
    '🐗' => 'kanec',
    '🐽' => 'prasečí rypák',
    '🐏' => 'beran',
    '🐑' => 'ovce',
    '🐐' => 'koza',
    '🐪' => 'velbloud jednohrbý',
    '🐫' => 'velbloud dvouhrbý',
    '🦙' => 'lama',
    '🦒' => 'žirafa',
    '🐘' => 'slon',
    '🦣' => 'mamut',
    '🦏' => 'nosorožec',
    '🦛' => 'hroch',
    '🐭' => 'hlava myši',
    '🐁' => 'myš',
    '🐀' => 'krysa',
    '🐹' => 'hlava křečka',
    '🐰' => 'hlava králíka',
    '🐇' => 'králík',
    '🐿' => 'veverka',
    '🦫' => 'bobr',
    '🦔' => 'ježek',
    '🦇' => 'netopýr',
    '🐻' => 'hlava medvěda',
    '🐨' => 'koala',
    '🐼' => 'hlava pandy',
    '🦥' => 'lenochod',
    '🦦' => 'vydra',
    '🦨' => 'skunk',
    '🦘' => 'klokan',
    '🦡' => 'jezevec',
    '🐾' => 'otisky tlap',
    '🦃' => 'krocan',
    '🐔' => 'slepice',
    '🐓' => 'kohout',
    '🐣' => 'líhnoucí se kuře',
    '🐤' => 'kuřátko',
    '🐥' => 'kuřátko zepředu',
    '🐦' => 'pták',
    '🐧' => 'tučňák',
    '🕊' => 'holubice',
    '🦅' => 'orel',
    '🦆' => 'kachna',
    '🦢' => 'labuť',
    '🦉' => 'sova',
    '🦤' => 'dodo',
    '🪶' => 'pírko',
    '🦩' => 'plameňák',
    '🦚' => 'páv',
    '🦜' => 'papoušek',
    '🪽' => 'křídlo',
    '🪿' => 'husa',
    '🐸' => 'hlava žáby',
    '🐊' => 'krokodýl',
    '🐢' => 'želva',
    '🦎' => 'ještěrka',
    '🐍' => 'had',
    '🐲' => 'hlava draka',
    '🐉' => 'drak',
    '🦕' => 'brontosaurus',
    '🦖' => 'tyranosaurus rex',
    '🐳' => 'velryba stříkající vodu',
    '🐋' => 'velryba',
    '🐬' => 'delfín',
    '🫍' => 'kosatka',
    '🦭' => 'tuleň',
    '🐟' => 'ryba',
    '🐠' => 'tropická ryba',
    '🐡' => 'ježík',
    '🦈' => 'žralok',
    '🐙' => 'chobotnice',
    '🐚' => 'ulita',
    '🪸' => 'korál',
    '🪼' => 'medúza',
    '🦀' => 'krab',
    '🦞' => 'humr',
    '🦐' => 'kreveta',
    '🦑' => 'oliheň',
    '🦪' => 'ústřice',
    '🐌' => 'hlemýžď',
    '🦋' => 'motýl',
    '🐛' => 'housenka',
    '🐜' => 'mravenec',
    '🐝' => 'včela',
    '🪲' => 'brouk',
    '🐞' => 'beruška',
    '🦗' => 'cvrček',
    '🪳' => 'šváb',
    '🕷' => 'pavouk',
    '🕸' => 'pavučina',
    '🦂' => 'škorpion',
    '🦟' => 'komár',
    '🪰' => 'moucha',
    '🪱' => 'červ',
    '🦠' => 'mikrob',
    '💐' => 'kytice',
    '🌸' => 'třešňový květ',
    '💮' => 'bílá květina',
    '🪷' => 'lotos',
    '🏵' => 'rozeta',
    '🌹' => 'růže',
    '🥀' => 'zvadlá květina',
    '🌺' => 'ibišek',
    '🌻' => 'slunečnice',
    '🌼' => 'květ',
    '🌷' => 'tulipán',
    '🪻' => 'hyacint',
    '🌱' => 'výhonek',
    '🪴' => 'květina v květináči',
    '🌲' => 'jehličnatý strom',
    '🌳' => 'listnatý strom',
    '🌴' => 'palma',
    '🌵' => 'kaktus',
    '🌾' => 'rýžové klasy',
    '🌿' => 'bylina',
    '☘' => 'trojlístek',
    '🍀' => 'čtyřlístek',
    '🍁' => 'javorový list',
    '🍂' => 'spadlý list',
    '🍃' => 'padající list ve větru',
    '🪹' => 'prázdné hnízdo',
    '🪺' => 'hnízdo s vejci',
    '🍄' => 'houba',
    '🪾' => 'strom bez listí',
    '🍇' => 'hrozny',
    '🍈' => 'cukrový meloun',
    '🍉' => 'meloun',
    '🍊' => 'mandarinka',
    '🍋' => 'citron',
    '🍌' => 'banán',
    '🍍' => 'ananas',
    '🥭' => 'mango',
    '🍎' => 'červené jablko',
    '🍏' => 'zelené jablko',
    '🍐' => 'hruška',
    '🍑' => 'broskev',
    '🍒' => 'třešně',
    '🍓' => 'jahoda',
    '🫐' => 'borůvky',
    '🥝' => 'kiwi',
    '🍅' => 'rajče',
    '🫒' => 'oliva',
    '🥥' => 'kokos',
    '🥑' => 'avokádo',
    '🍆' => 'lilek',
    '🥔' => 'brambor',
    '🥕' => 'mrkev',
    '🌽' => 'kukuřičný klas',
    '🌶' => 'feferonka',
    '🫑' => 'paprika',
    '🥒' => 'okurka',
    '🥬' => 'salátové listy',
    '🥦' => 'brokolice',
    '🧄' => 'česnek',
    '🧅' => 'cibule',
    '🥜' => 'arašídy',
    '🫘' => 'fazole',
    '🌰' => 'kaštan',
    '🫚' => 'zázvor',
    '🫛' => 'hrachový lusk',
    '🫜' => 'kořenová zelenina',
    '🍞' => 'chleba',
    '🥐' => 'croissant',
    '🥖' => 'bageta',
    '🫓' => 'chlebová placka',
    '🥨' => 'preclík',
    '🥯' => 'bagel',
    '🥞' => 'palačinky',
    '🧇' => 'vafle',
    '🧀' => 'klínek sýra',
    '🍖' => 'maso na kosti',
    '🍗' => 'kuřecí stehýnko',
    '🥩' => 'plátek masa',
    '🥓' => 'slanina',
    '🍔' => 'hamburger',
    '🍟' => 'hranolky',
    '🍕' => 'pizza',
    '🌭' => 'párek v rohlíku',
    '🥪' => 'sendvič',
    '🌮' => 'taco',
    '🌯' => 'burrito',
    '🫔' => 'tamale',
    '🥙' => 'plněná pita',
    '🧆' => 'falafel',
    '🥚' => 'vejce',
    '🍳' => 'vejce na pánvi',
    '🥘' => 'mělký rendlík s jídlem',
    '🍲' => 'hrnec jídla',
    '🫕' => 'fondue',
    '🥣' => 'miska se lžící',
    '🥗' => 'salát',
    '🍿' => 'popcorn',
    '🧈' => 'máslo',
    '🧂' => 'sůl',
    '🥫' => 'jídlo v plechovce',
    '🍱' => 'krabička bentó',
    '🍘' => 'rýžová sušenka',
    '🍙' => 'rýžová koule',
    '🍚' => 'vařená rýže',
    '🍛' => 'kari rýže',
    '🍜' => 'miska nudlí',
    '🍝' => 'špagety',
    '🍠' => 'opékaný batát',
    '🍢' => 'oden',
    '🍣' => 'suši',
    '🍤' => 'smažená kreveta',
    '🍥' => 'rybí koláček se spirálkou',
    '🥮' => 'měsíční koláč',
    '🍡' => 'dango',
    '🥟' => 'piroh',
    '🥠' => 'čínský koláček štěstí',
    '🥡' => 'krabice na jídlo',
    '🍦' => 'točená zmrzlina',
    '🍧' => 'ledová tříšť',
    '🍨' => 'kopečková zmrzlina',
    '🍩' => 'donut',
    '🍪' => 'koláček',
    '🎂' => 'narozeninový dort',
    '🍰' => 'dortový řez',
    '🧁' => 'košíček',
    '🥧' => 'koláč',
    '🍫' => 'tabulka čokolády',
    '🍬' => 'bonbon',
    '🍭' => 'lízátko',
    '🍮' => 'pudink',
    '🍯' => 'sklenice medu',
    '🍼' => 'kojenecká láhev',
    '🥛' => 'sklenice mléka',
    '☕' => 'teplý nápoj',
    '🫖' => 'čajová konvice',
    '🍵' => 'šálek bez ouška',
    '🍶' => 'saké',
    '🍾' => 'láhev sektu',
    '🍷' => 'sklenička vína',
    '🍸' => 'koktejlová sklenička',
    '🍹' => 'tropický koktejl',
    '🍺' => 'půllitr piva',
    '🍻' => 'cinkající půllitry piva',
    '🥂' => 'cinkající sklenice',
    '🥃' => 'sklenice whisky',
    '🫗' => 'vylévání tekutiny',
    '🥤' => 'pohárek s brčkem',
    '🧋' => 'bublinkový čaj',
    '🧃' => 'nápoj v krabičce',
    '🧉' => 'maté',
    '🧊' => 'kostka ledu',
    '🥢' => 'jídelní hůlky',
    '🍽' => 'talíř s vidličkou a nožem',
    '🍴' => 'vidlička a nůž',
    '🥄' => 'lžíce',
    '🔪' => 'kuchyňský nůž',
    '🫙' => 'dóza',
    '🏺' => 'amfora',
    '🌍' => 'glóbus s Evropou a Afrikou',
    '🌎' => 'glóbus se Severní a Jižní Amerikou',
    '🌏' => 'glóbus s Asií a Austrálií',
    '🌐' => 'glóbus s poledníky a rovnoběžkami',
    '🗺' => 'mapa světa',
    '🗾' => 'mapa Japonska',
    '🧭' => 'kompas',
    '🏔' => 'zasněžená hora',
    '⛰' => 'hora',
    '🛘' => 'sesuv půdy',
    '🌋' => 'sopka',
    '🗻' => 'hora Fudži',
    '🏕' => 'tábor',
    '🏖' => 'pláž se slunečníkem',
    '🏜' => 'poušť',
    '🏝' => 'pustý ostrov',
    '🏞' => 'národní park',
    '🏟' => 'stadion',
    '🏛' => 'antická budova',
    '🏗' => 'staveniště',
    '🧱' => 'cihly',
    '🪨' => 'kámen',
    '🪵' => 'dřevo',
    '🛖' => 'chýše',
    '🏘' => 'domky',
    '🏚' => 'opuštěný dům',
    '🏠' => 'domek',
    '🏡' => 'domek se zahradou',
    '🏢' => 'kancelářská budova',
    '🏣' => 'japonská pošta',
    '🏤' => 'pošta',
    '🏥' => 'nemocnice',
    '🏦' => 'banka',
    '🏨' => 'hotel',
    '🏩' => 'hodinový hotel',
    '🏪' => 'obchod',
    '🏫' => 'škola',
    '🏬' => 'obchodní dům',
    '🏭' => 'továrna',
    '🏯' => 'japonský hrad',
    '🏰' => 'hrad',
    '💒' => 'svatba',
    '🗼' => 'Tokijská věž',
    '🗽' => 'socha Svobody',
    '⛪' => 'kostel',
    '🕌' => 'mešita',
    '🛕' => 'hinduistický chrám',
    '🕍' => 'synagoga',
    '⛩' => 'šintoistická svatyně',
    '🕋' => 'Kába',
    '⛲' => 'fontána',
    '⛺' => 'stan',
    '🌁' => 'mlha',
    '🌃' => 'hvězdná noc',
    '🏙' => 'městská zástavba',
    '🌄' => 'svítání nad horami',
    '🌅' => 'východ slunce',
    '🌆' => 'město za soumraku',
    '🌇' => 'západ slunce',
    '🌉' => 'most v noci',
    '♨' => 'horké prameny',
    '🎠' => 'kolotočový kůň',
    '🛝' => 'dětská skluzavka',
    '🎡' => 'ruské kolo',
    '🎢' => 'horská dráha',
    '💈' => 'značka holičství',
    '🎪' => 'cirkusový stan',
    '🚂' => 'lokomotiva',
    '🚃' => 'vagón',
    '🚄' => 'pendolino',
    '🚅' => 'japonský rychlovlak',
    '🚆' => 'vlak',
    '🚇' => 'metro',
    '🚈' => 'rychlodrážní tramvaj',
    '🚉' => 'stanice',
    '🚊' => 'tramvaj',
    '🚝' => 'jednokolejka',
    '🚞' => 'horská železnice',
    '🚋' => 'vagón tramvaje',
    '🚌' => 'autobus',
    '🚍' => 'přijíždějící autobus',
    '🚎' => 'trolejbus',
    '🚐' => 'minibus',
    '🚑' => 'sanitka',
    '🚒' => 'hasičské auto',
    '🚓' => 'policejní auto',
    '🚔' => 'přijíždějící policejní auto',
    '🚕' => 'taxík',
    '🚖' => 'přijíždějící taxík',
    '🚗' => 'auto',
    '🚘' => 'přijíždějící auto',
    '🚙' => 'SUV',
    '🛻' => 'pickup',
    '🚚' => 'dodávka',
    '🚛' => 'kamion',
    '🚜' => 'traktor',
    '🏎' => 'formule',
    '🏍' => 'motorka',
    '🛵' => 'skútr',
    '🦽' => 'mechanický invalidní vozík',
    '🦼' => 'elektrický invalidní vozík',
    '🛺' => 'autorikša',
    '🚲' => 'jízdní kolo',
    '🛴' => 'koloběžka',
    '🛹' => 'skateboard',
    '🛼' => 'kolečková brusle',
    '🚏' => 'zastávka autobusu',
    '🛣' => 'dálnice',
    '🛤' => 'železniční trať',
    '🛢' => 'barel',
    '⛽' => 'benzínová pumpa',
    '🛞' => 'kolo',
    '🚨' => 'policejní maják',
    '🚥' => 'vodorovný semafor',
    '🚦' => 'svislý semafor',
    '🛑' => 'značka stop',
    '🚧' => 'stavební zábrana',
    '⚓' => 'kotva',
    '🛟' => 'záchranný kruh',
    '⛵' => 'plachetnice',
    '🛶' => 'kánoe',
    '🚤' => 'motorový člun',
    '🛳' => 'zaoceánská loď',
    '⛴' => 'trajekt',
    '🛥' => 'motorová jachta',
    '🚢' => 'loď',
    '✈' => 'letadlo',
    '🛩' => 'malé letadlo',
    '🛫' => 'vzlétající letadlo',
    '🛬' => 'přistávající letadlo',
    '🪂' => 'padák',
    '💺' => 'sedadlo',
    '🚁' => 'vrtulník',
    '🚟' => 'lanovka',
    '🚠' => 'horská lanovka',
    '🚡' => 'kabinová lanovka',
    '🛰' => 'satelit',
    '🚀' => 'raketa',
    '🛸' => 'létající talíř',
    '🛎' => 'portýrský zvonek',
    '🧳' => 'cestovní kufr',
    '⌛' => 'přesýpací hodiny',
    '⏳' => 'běžící přesýpací hodiny',
    '⌚' => 'hodinky',
    '⏰' => 'budík',
    '⏱' => 'stopky',
    '⏲' => 'minutka',
    '🕰' => 'krbové hodiny',
    '🕛' => 'dvanáct hodin',
    '🕧' => 'půl jedné',
    '🕐' => 'jedna hodina',
    '🕜' => 'půl druhé',
    '🕑' => 'dvě hodiny',
    '🕝' => 'půl třetí',
    '🕒' => 'tři hodiny',
    '🕞' => 'půl čtvrté',
    '🕓' => 'čtyři hodiny',
    '🕟' => 'půl páté',
    '🕔' => 'pět hodin',
    '🕠' => 'půl šesté',
    '🕕' => 'šest hodin',
    '🕡' => 'půl sedmé',
    '🕖' => 'sedm hodin',
    '🕢' => 'půl osmé',
    '🕗' => 'osm hodin',
    '🕣' => 'půl deváté',
    '🕘' => 'devět hodin',
    '🕤' => 'půl desáté',
    '🕙' => 'deset hodin',
    '🕥' => 'půl jedenácté',
    '🕚' => 'jedenáct hodin',
    '🕦' => 'půl dvanácté',
    '🌑' => 'měsíc v novu',
    '🌒' => 'dorůstající měsíc po novoluní',
    '🌓' => 'měsíc v první čtvrti',
    '🌔' => 'dorůstající měsíc před úplňkem',
    '🌕' => 'měsíc v úplňku',
    '🌖' => 'ubývající měsíc po úplňku',
    '🌗' => 'měsíc v poslední čtvrti',
    '🌘' => 'ubývající měsíc před novoluním',
    '🌙' => 'srpek měsíce',
    '🌚' => 'měsíc v novu s obličejem',
    '🌛' => 'měsíc v první čtvrti s obličejem',
    '🌜' => 'měsíc v poslední čtvrti s obličejem',
    '🌡' => 'teploměr',
    '☀' => 'slunce',
    '🌝' => 'měsíc v úplňku s obličejem',
    '🌞' => 'slunce s obličejem',
    '🪐' => 'planeta s prstencem',
    '⭐' => 'bílá hvězda střední velikosti',
    '🌟' => 'zářící hvězda',
    '🌠' => 'padající hvězda',
    '🌌' => 'mléčná dráha',
    '☁' => 'mrak',
    '⛅' => 'slunce za mrakem',
    '⛈' => 'mrak s bleskem a deštěm',
    '🌤' => 'slunce za malým mrakem',
    '🌥' => 'slunce za velkým mrakem',
    '🌦' => 'slunce za dešťovým mrakem',
    '🌧' => 'mrak s deštěm',
    '🌨' => 'mrak se sněhem',
    '🌩' => 'mrak s bleskem',
    '🌪' => 'tornádo',
    '🌫' => 'mlhavé počasí',
    '🌬' => 'větrný obličej',
    '🌀' => 'cyklóna',
    '🌈' => 'duha',
    '🌂' => 'zavřený deštník',
    '☂' => 'deštník',
    '☔' => 'deštník s kapkami deště',
    '⛱' => 'slunečník',
    '⚡' => 'vysoké napětí',
    '❄' => 'sněhová vločka',
    '☃' => 'sněhulák',
    '⛄' => 'sněhulák bez sněhu',
    '☄' => 'kometa',
    '🔥' => 'oheň',
    '💧' => 'kapka',
    '🌊' => 'vodní vlna',
    '🎃' => 'dýňová lucerna',
    '🎄' => 'vánoční stromeček',
    '🎆' => 'ohňostroj',
    '🎇' => 'prskavka',
    '🧨' => 'petarda',
    '✨' => 'jiskry',
    '🎈' => 'balónek',
    '🎉' => 'vystřelovací konfety',
    '🎊' => 'konfetová koule',
    '🎋' => 'strom přání',
    '🎍' => 'bambusové dekorace',
    '🎎' => 'japonské panenky',
    '🎏' => 'koinobori',
    '🎐' => 'větrný zvonek',
    '🎑' => 'slavnosti měsíce',
    '🧧' => 'červená obálka',
    '🎀' => 'mašle',
    '🎁' => 'zabalený dárek',
    '🎗' => 'symbolická stužka',
    '🎟' => 'vstupenky',
    '🎫' => 'lístek',
    '🎖' => 'vojenská medaile',
    '🏆' => 'trofejní pohár',
    '🏅' => 'medaile',
    '🥇' => 'medaile za 1. místo',
    '🥈' => 'medaile za 2. místo',
    '🥉' => 'medaile za 3. místo',
    '⚽' => 'fotbalový míč',
    '⚾' => 'baseballový míček',
    '🥎' => 'softballový míček',
    '🏀' => 'basketbalový míč',
    '🏐' => 'volejbalový míč',
    '🏈' => 'míč na americký fotbal',
    '🏉' => 'míč na rugby',
    '🎾' => 'tenis',
    '🥏' => 'frisbee',
    '🎳' => 'bowling',
    '🏏' => 'kriket',
    '🏑' => 'pozemní hokej',
    '🏒' => 'lední hokej',
    '🥍' => 'lakros',
    '🏓' => 'stolní tenis',
    '🏸' => 'badminton',
    '🥊' => 'boxerská rukavice',
    '🥋' => 'sportovní kimono',
    '🥅' => 'sportovní branka',
    '⛳' => 'jamka s vlajkou',
    '⛸' => 'brusle',
    '🎣' => 'rybářský prut',
    '🤿' => 'potápěčská maska',
    '🎽' => 'běžecké tričko',
    '🎿' => 'lyže',
    '🛷' => 'sáňky',
    '🥌' => 'curlingový kámen',
    '🎯' => 'přesný zásah do terče',
    '🪀' => 'jojo',
    '🪁' => 'papírový drak',
    '🔫' => 'vodní pistole',
    '🎱' => 'kulečníková koule 8',
    '🔮' => 'křišťálová koule',
    '🪄' => 'kouzelná hůlka',
    '🎮' => 'videohra',
    '🕹' => 'joystick',
    '🎰' => 'výherní automat',
    '🎲' => 'hrací kostka',
    '🧩' => 'puzzle',
    '🧸' => 'medvídek',
    '🪅' => 'piñata',
    '🪩' => 'zrcadlová koule',
    '🪆' => 'matrjoška',
    '♠' => 'piky',
    '♥' => 'srdce',
    '♦' => 'káry',
    '♣' => 'kříže',
    '♟' => 'pěšec',
    '🃏' => 'žolík',
    '🀄' => 'červený drak v mahjongu',
    '🎴' => 'hrací karty s květinami',
    '🎭' => 'múzická umění',
    '🖼' => 'obraz v rámu',
    '🎨' => 'malířská paleta',
    '🧵' => 'cívka nití',
    '🪡' => 'šicí jehla',
    '🧶' => 'klubíčko',
    '🪢' => 'uzel',
    '👓' => 'brýle',
    '🕶' => 'sluneční brýle',
    '🥽' => 'ochranné brýle',
    '🥼' => 'laboratorní plášť',
    '🦺' => 'bezpečnostní vesta',
    '👔' => 'kravata',
    '👕' => 'tričko',
    '👖' => 'džíny',
    '🧣' => 'šála',
    '🧤' => 'rukavice',
    '🧥' => 'kabát',
    '🧦' => 'ponožky',
    '👗' => 'šaty',
    '👘' => 'kimono',
    '🥻' => 'sárí',
    '🩱' => 'jednodílné plavky',
    '🩲' => 'pánské plavky',
    '🩳' => 'šortky',
    '👙' => 'bikiny',
    '👚' => 'dámská blůza',
    '🪭' => 'vějíř',
    '👛' => 'peněženka',
    '👜' => 'kabelka',
    '👝' => 'kosmetická taška',
    '🛍' => 'nákupní tašky',
    '🎒' => 'školní batoh',
    '🩴' => 'žabka',
    '👞' => 'pánská bota',
    '👟' => 'běžecká bota',
    '🥾' => 'pohorka',
    '🥿' => 'střevíček bez podpatku',
    '👠' => 'bota na podpatku',
    '👡' => 'dámský sandál',
    '🩰' => 'baletní piškoty',
    '👢' => 'dámská kozačka',
    '🪮' => 'hřeben na kudrnaté vlasy',
    '👑' => 'koruna',
    '👒' => 'dámský klobouk',
    '🎩' => 'cylindr',
    '🎓' => 'promoční čepice',
    '🧢' => 'kšiltovka',
    '🪖' => 'vojenská přilba',
    '⛑' => 'záchranářská helma',
    '📿' => 'růženec',
    '💄' => 'rtěnka',
    '💍' => 'prsten',
    '💎' => 'drahokam',
    '🔇' => 'vypnutý reproduktor',
    '🔈' => 'tiše hrající reproduktor',
    '🔉' => 'středně hlasitý reproduktor',
    '🔊' => 'hlasitý reproduktor',
    '📢' => 'tlampač',
    '📣' => 'megafon',
    '📯' => 'poštovní roh',
    '🔔' => 'zvonek',
    '🔕' => 'přeškrtnutý zvonek',
    '🎼' => 'notová osnova',
    '🎵' => 'nota',
    '🎶' => 'noty',
    '🎙' => 'studiový mikrofon',
    '🎚' => 'posuvný ovladač hlasitosti',
    '🎛' => 'ovládací knoflíky',
    '🎤' => 'mikrofon',
    '🎧' => 'sluchátka',
    '📻' => 'rádio',
    '🎷' => 'saxofon',
    '🎺' => 'trubka',
    '🪊' => 'pozoun',
    '🪗' => 'akordeon',
    '🎸' => 'kytara',
    '🎹' => 'klávesy',
    '🎻' => 'housle',
    '🪕' => 'banjo',
    '🥁' => 'buben',
    '🪘' => 'protáhlý buben',
    '🪇' => 'rumbakoule',
    '🪈' => 'flétna',
    '🪉' => 'harfa',
    '📱' => 'mobil',
    '📲' => 'mobil se šipkou',
    '☎' => 'telefon',
    '📞' => 'telefonní sluchátko',
    '📟' => 'pager',
    '📠' => 'fax',
    '🔋' => 'baterie',
    '🪫' => 'slabá baterie',
    '🔌' => 'zástrčka',
    '💻' => 'notebook',
    '🖥' => 'počítač',
    '🖨' => 'tiskárna',
    '⌨' => 'klávesnice',
    '🖱' => 'počítačová myš',
    '🖲' => 'trackball',
    '💽' => 'minidisk',
    '💾' => 'disketa',
    '💿' => 'optický disk',
    '📀' => 'dvd',
    '🧮' => 'počitadlo',
    '🎥' => 'filmová kamera',
    '🎞' => 'filmová okénka',
    '📽' => 'filmový projektor',
    '🎬' => 'filmová klapka',
    '📺' => 'televizor',
    '📷' => 'fotoaparát',
    '📸' => 'fotoaparát s bleskem',
    '📹' => 'videokamera',
    '📼' => 'videokazeta',
    '🔍' => 'lupa nakloněná doleva',
    '🔎' => 'lupa nakloněná doprava',
    '🕯' => 'svíčka',
    '💡' => 'žárovka',
    '🔦' => 'baterka',
    '🏮' => 'červený lampion',
    '🪔' => 'indická olejová lampa',
    '📔' => 'zápisník v ozdobných deskách',
    '📕' => 'zavřená kniha',
    '📖' => 'otevřená kniha',
    '📗' => 'zelená kniha',
    '📘' => 'modrá kniha',
    '📙' => 'oranžová kniha',
    '📚' => 'knihy',
    '📓' => 'sešit',
    '📒' => 'účetní kniha',
    '📃' => 'zkroucená stránka',
    '📜' => 'svitek',
    '📄' => 'stránka lícem nahoru',
    '📰' => 'noviny',
    '🗞' => 'srolované noviny',
    '📑' => 'stránky se záložkami',
    '🔖' => 'záložka',
    '🏷' => 'visačka',
    '🪙' => 'mince',
    '💰' => 'pytel peněz',
    '🪎' => 'truhla s pokladem',
    '💴' => 'jenová bankovka',
    '💵' => 'dolarová bankovka',
    '💶' => 'eurobankovka',
    '💷' => 'librová bankovka',
    '💸' => 'okřídlené peníze',
    '💳' => 'platební karta',
    '🧾' => 'účtenka',
    '💹' => 'rostoucí graf se znakem jenu',
    '✉' => 'obálka',
    '📧' => 'e-mail',
    '📨' => 'příchozí obálka',
    '📩' => 'obálka se šipkou',
    '📤' => 'přihrádka na odchozí poštu',
    '📥' => 'přihrádka na příchozí poštu',
    '📦' => 'balíček',
    '📫' => 'zavřená schránka s praporkem nahoře',
    '📪' => 'zavřená schránka s praporkem dole',
    '📬' => 'otevřená schránka s praporkem nahoře',
    '📭' => 'otevřená schránka s praporkem dole',
    '📮' => 'poštovní schránka',
    '🗳' => 'urna s volebním lístkem',
    '✏' => 'tužka',
    '✒' => 'hrot černého pera',
    '🖋' => 'plnicí pero',
    '🖊' => 'propiska',
    '🖌' => 'štětec',
    '🖍' => 'pastelka',
    '📝' => 'lístek se vzkazem',
    '💼' => 'kufřík',
    '📁' => 'složka',
    '📂' => 'otevřená složka',
    '🗂' => 'kartotéka',
    '📅' => 'kalendář',
    '📆' => 'trhací kalendář',
    '🗒' => 'kroužkový zápisník',
    '🗓' => 'kroužkový kalendář',
    '📇' => 'vizitkovník',
    '📈' => 'rostoucí graf',
    '📉' => 'klesající graf',
    '📊' => 'sloupcový graf',
    '📋' => 'psací podložka',
    '📌' => 'připínáček',
    '📍' => 'kulatý připínáček',
    '📎' => 'kancelářská sponka',
    '🖇' => 'spojené kancelářské sponky',
    '📏' => 'rovné pravítko',
    '📐' => 'trojúhelníkové pravítko',
    '✂' => 'nůžky',
    '🗃' => 'krabice s kartotékou',
    '🗄' => 'skříň na spisy',
    '🗑' => 'odpadkový koš',
    '🔒' => 'zamknutý zámek',
    '🔓' => 'odemknutý zámek',
    '🔏' => 'zamknutý zámek s plnicím perem',
    '🔐' => 'zamknutý zámek s klíčem',
    '🔑' => 'klíč',
    '🗝' => 'starý klíč',
    '🔨' => 'kladivo',
    '🪓' => 'sekera',
    '⛏' => 'krumpáč',
    '⚒' => 'kladivo a špičák',
    '🛠' => 'kladivo a maticový klíč',
    '🗡' => 'dýka',
    '⚔' => 'zkřížené meče',
    '💣' => 'bomba',
    '🪃' => 'bumerang',
    '🏹' => 'luk a šíp',
    '🛡' => 'štít',
    '🪚' => 'ruční pila',
    '🔧' => 'maticový klíč',
    '🪛' => 'šroubovák',
    '🔩' => 'šroub a matice',
    '⚙' => 'ozubené kolo',
    '🗜' => 'svěrka',
    '⚖' => 'miskové váhy',
    '🦯' => 'slepecká hůl',
    '🔗' => 'články řetězu',
    '⛓' => 'řetězy',
    '🪝' => 'hák',
    '🧰' => 'box na nářadí',
    '🧲' => 'magnet',
    '🪜' => 'žebřík',
    '🪏' => 'lopata',
    '⚗' => 'křivule',
    '🧪' => 'zkumavka',
    '🧫' => 'Petriho miska',
    '🧬' => 'DNA',
    '🔬' => 'mikroskop',
    '🔭' => 'teleskop',
    '📡' => 'satelitní anténa',
    '💉' => 'injekční stříkačka',
    '🩸' => 'kapka krve',
    '💊' => 'pilulka',
    '🩹' => 'náplast',
    '🩼' => 'berle',
    '🩺' => 'stetoskop',
    '🩻' => 'rentgen',
    '🚪' => 'dveře',
    '🛗' => 'výtah',
    '🪞' => 'zrcadlo',
    '🪟' => 'okno',
    '🛏' => 'postel',
    '🛋' => 'gauč a lampa',
    '🪑' => 'židle',
    '🚽' => 'toaletní mísa',
    '🪠' => 'zvon na WC',
    '🚿' => 'sprcha',
    '🛁' => 'vana',
    '🪤' => 'pastička na myši',
    '🪒' => 'břitva',
    '🧴' => 'láhev na kosmetiku',
    '🧷' => 'spínací špendlík',
    '🧹' => 'koště',
    '🧺' => 'koš',
    '🧻' => 'role papíru',
    '🪣' => 'kbelík',
    '🧼' => 'mýdlo',
    '🫧' => 'bubliny',
    '🪥' => 'zubní kartáček',
    '🧽' => 'mycí houba',
    '🧯' => 'hasicí přístroj',
    '🛒' => 'nákupní vozík',
    '🚬' => 'cigareta',
    '⚰' => 'rakev',
    '🪦' => 'náhrobek',
    '⚱' => 'pohřební urna',
    '🧿' => 'amulet nazar',
    '🪬' => 'hamsa',
    '🗿' => 'socha moai',
    '🪧' => 'cedule',
    '🪪' => 'průkaz',
    '🏧' => 'značka bankomatu',
    '🚮' => 'značka odpadkového koše',
    '🚰' => 'pitná voda',
    '♿' => 'symbol invalidního vozíku',
    '🚹' => 'pánské toalety',
    '🚺' => 'dámské toalety',
    '🚻' => 'toalety',
    '🚼' => 'symbol dítěte',
    '🚾' => 'záchod',
    '🛂' => 'pasová kontrola',
    '🛃' => 'celnice',
    '🛄' => 'výdej zavazadel',
    '🛅' => 'úschovna zavazadel',
    '⚠' => 'varování',
    '🚸' => 'pozor, děti',
    '⛔' => 'zákaz vjezdu v jednom směru',
    '🚫' => 'zákaz',
    '🚳' => 'zákaz vjezdu cyklistů',
    '🚭' => 'zákaz kouření',
    '🚯' => 'zákaz odhazování odpadků',
    '🚱' => 'nepitná voda',
    '🚷' => 'zákaz vstupu chodců',
    '📵' => 'zákaz používání mobilních telefonů',
    '🔞' => 'zákaz vstupu osobám mladším 18 let',
    '☢' => 'radioaktivita',
    '☣' => 'biohazard',
    '⬆' => 'šipka nahoru',
    '↗' => 'šipka doprava nahoru',
    '➡' => 'šipka doprava',
    '↘' => 'šipka doprava dolů',
    '⬇' => 'šipka dolů',
    '↙' => 'šipka doleva dolů',
    '⬅' => 'šipka doleva',
    '↖' => 'šipka doleva nahoru',
    '↕' => 'šipka nahoru a dolů',
    '↔' => 'šipka doleva a doprava',
    '↩' => 'šipka doprava stáčející se doleva',
    '↪' => 'šipka doleva stáčející se doprava',
    '⤴' => 'šipka doprava stáčející se nahoru',
    '⤵' => 'šipka doprava stáčející se dolů',
    '🔃' => 'svislé šipky ve směru hodinových ručiček',
    '🔄' => 'šipky proti směru hodinových ručiček',
    '🔙' => 'šipka zpět',
    '🔚' => 'šipka s nápisem END',
    '🔛' => 'šipky s nápisem ON!',
    '🔜' => 'šipka s nápisem SOON',
    '🔝' => 'šipka s nápisem TOP',
    '🛐' => 'modlitebna',
    '⚛' => 'symbol atomu',
    '🕉' => 'óm',
    '✡' => 'Davidova hvězda',
    '☸' => 'kolo dharmy',
    '☯' => 'jin a jang',
    '✝' => 'latinský kříž',
    '☦' => 'pravoslavný kříž',
    '☪' => 'hvězda a půlměsíc',
    '☮' => 'symbol míru',
    '🕎' => 'sedmiramenný svícen',
    '🔯' => 'šesticípá hvězda s tečkou uprostřed',
    '🪯' => 'khanda',
    '♈' => 'skopec',
    '♉' => 'býk',
    '♊' => 'blíženci',
    '♋' => 'rak',
    '♌' => 'lev',
    '♍' => 'panna',
    '♎' => 'váhy',
    '♏' => 'štír',
    '♐' => 'střelec',
    '♑' => 'kozoroh',
    '♒' => 'vodnář',
    '♓' => 'ryby',
    '⛎' => 'hadonoš',
    '🔀' => 'tlačítko náhodného přehrávání',
    '🔁' => 'tlačítko opakování',
    '🔂' => 'tlačítko opakování jedné stopy',
    '▶' => 'tlačítko přehrávání',
    '⏩' => 'tlačítko převíjení vpřed',
    '⏭' => 'tlačítko další stopy',
    '⏯' => 'tlačítko přehrávání nebo pozastavení',
    '◀' => 'tlačítko přehrávání pozpátku',
    '⏪' => 'tlačítko převíjení vzad',
    '⏮' => 'tlačítko předchozí stopy',
    '🔼' => 'tlačítko nahoru',
    '⏫' => 'tlačítko rychle nahoru',
    '🔽' => 'tlačítko dolů',
    '⏬' => 'tlačítko rychle dolů',
    '⏸' => 'tlačítko pozastavení',
    '⏹' => 'tlačítko zastavení',
    '⏺' => 'tlačítko nahrávání',
    '⏏' => 'tlačítko vysunutí',
    '🎦' => 'kino',
    '🔅' => 'tlačítko ztlumení jasu',
    '🔆' => 'tlačítko zvýšení jasu',
    '📶' => 'indikátor síly signálu',
    '🛜' => 'bezdrátové připojení',
    '📳' => 'vibrační režim',
    '📴' => 'vypnutý mobilní telefon',
    '♀' => 'symbol ženského pohlaví',
    '♂' => 'symbol mužského pohlaví',
    '⚧' => 'symbol pro transgender osoby',
    '✖' => 'znak násobení',
    '➕' => 'znak plus',
    '➖' => 'znak minus',
    '➗' => 'znak dělení',
    '🟰' => 'tučný znak rovnosti',
    '♾' => 'nekonečno',
    '‼' => 'dvojitý vykřičník',
    '⁉' => 'vykřičník a otazník',
    '❓' => 'červený otazník',
    '❔' => 'bílý otazník',
    '❕' => 'bílý vykřičník',
    '❗' => 'červený vykřičník',
    '〰' => 'dlouhá vlnovka',
    '💱' => 'směna peněz',
    '💲' => 'znak dolaru',
    '⚕' => 'znak medicíny',
    '♻' => 'symbol recyklace',
    '⚜' => 'heraldická lilie',
    '🔱' => 'emblém trojzubce',
    '📛' => 'jmenovka',
    '🔰' => 'japonský symbol pro začátečníka',
    '⭕' => 'červený kruh bez výplně',
    '✅' => 'tlačítko se značkou zaškrtnutí',
    '☑' => 'zaškrtnuté políčko',
    '✔' => 'značka zaškrtnutí',
    '❌' => 'křížek',
    '❎' => 'tlačítko s křížkem',
    '➰' => 'smyčka',
    '➿' => 'dvojitá smyčka',
    '〽' => 'značka přechodu mezi částmi skladby',
    '✳' => 'osmicípá hvězdička',
    '✴' => 'osmicípá hvězda',
    '❇' => 'jiskra',
    '™' => 'znak ochranné známky',
    '🫟' => 'skvrna',
    '🔠' => 'zadávání v latince, velká písmena',
    '🔡' => 'zadávání v latince, malá písmena',
    '🔢' => 'zadávání čísel',
    '🔣' => 'zadávání symbolů',
    '🔤' => 'zadávání písmen v latince',
    '🅰' => 'štítek A (krevní skupina)',
    '🆎' => 'štítek AB (krevní skupina)',
    '🅱' => 'štítek B (krevní skupina)',
    '🆑' => 'štítek CL',
    '🆒' => 'štítek COOL',
    '🆓' => 'štítek FREE',
    'ℹ' => 'informace',
    '🆔' => 'štítek ID',
    'Ⓜ' => 'M v kroužku',
    '🆕' => 'štítek NEW',
    '🆖' => 'štítek NG',
    '🅾' => 'štítek 0 (krevní skupina)',
    '🆗' => 'štítek OK',
    '🅿' => 'štítek P',
    '🆘' => 'štítek SOS',
    '🆙' => 'štítek UP!',
    '🆚' => 'štítek VS',
    '🈁' => 'štítek s japonským znakem „zde“',
    '🈂' => 'štítek – japonský znak „pozornost podniku“',
    '🈷' => 'štítek s japonským znakem „měsíc“',
    '🈶' => 'štítek s japonským znakem „není zdarma“',
    '🈯' => 'štítek s japonským znakem „rezervováno“',
    '🉐' => 'štítek – japonský znak „výhodná koupě“',
    '🈹' => 'štítek s japonským znakem „sleva“',
    '🈚' => 'štítek s japonským znakem „zdarma“',
    '🈲' => 'štítek s japonským znakem „zákaz“',
    '🉑' => 'štítek s japonským znakem „přijatelné“',
    '🈸' => 'štítek s japonským znakem „žádost”',
    '🈴' => 'štítek s japonským znakem „prospěl“',
    '🈳' => 'štítek s japonským znakem „volno“',
    '㊗' => 'štítek s japonským znakem „blahopřání“',
    '㊙' => 'štítek s japonským znakem „tajné“',
    '🈺' => 'štítek s japonským znakem „otevřeno“',
    '🈵' => 'štítek s japonským znakem „obsazeno“',
    '🔴' => 'červený kroužek',
    '🟠' => 'oranžový kroužek',
    '🟡' => 'žlutý kroužek',
    '🟢' => 'zelený kroužek',
    '🔵' => 'modrý kroužek',
    '🟣' => 'fialový kroužek',
    '🟤' => 'hnědý kroužek',
    '⚫' => 'černý kroužek',
    '⚪' => 'bílý kroužek',
    '🟥' => 'červený čtvereček',
    '🟧' => 'oranžový čtvereček',
    '🟨' => 'žlutý čtvereček',
    '🟩' => 'zelený čtvereček',
    '🟦' => 'modrý čtvereček',
    '🟪' => 'fialový čtvereček',
    '🟫' => 'hnědý čtvereček',
    '⬛' => 'velký černý čtverec',
    '⬜' => 'velký bílý čtverec',
    '◼' => 'střední černý čtverec',
    '◻' => 'střední bílý čtverec',
    '◾' => 'větší černý čtvereček',
    '◽' => 'větší bílý čtvereček',
    '▪' => 'černý čtvereček',
    '▫' => 'bílý čtvereček',
    '🔶' => 'velký oranžový kosočtverec',
    '🔷' => 'velký modrý kosočtverec',
    '🔸' => 'malý oranžový kosočtverec',
    '🔹' => 'malý modrý kosočtverec',
    '🔺' => 'červený trojúhelník mířící nahoru',
    '🔻' => 'červený trojúhelník mířící dolů',
    '💠' => 'kosočtverec s tečkou',
    '🔘' => 'přepínač',
    '🔳' => 'bílé čtvercové tlačítko',
    '🔲' => 'černé čtvercové tlačítko',
    '🏁' => 'šachovnicová vlajka',
    '🚩' => 'trojúhelníková vlajka',
    '🎌' => 'zkřížené vlajky',
    '🏴' => 'černá vlajka',
    '🏳' => 'bílá vlajka',
];
