<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'үнсэлт: том хүн том хүн цайвар бага зэргийн цайвар',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'үнсэлт: том хүн том хүн цайвар дунд зэргийн',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'үнсэлт: том хүн том хүн цайвар бордуу',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'үнсэлт: том хүн том хүн цайвар бор',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'үнсэлт: том хүн том хүн бага зэргийн цайвар цайвар',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'үнсэлт: том хүн том хүн бага зэргийн цайвар дунд зэргийн',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'үнсэлт: том хүн том хүн бага зэргийн цайвар бордуу',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'үнсэлт: том хүн том хүн бага зэргийн цайвар бор',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'үнсэлт: том хүн том хүн дунд зэргийн цайвар',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'үнсэлт: том хүн том хүн дунд зэргийн бага зэргийн цайвар',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'үнсэлт: том хүн том хүн дунд зэргийн бордуу',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'үнсэлт: том хүн том хүн дунд зэргийн бор',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'үнсэлт: том хүн том хүн бордуу цайвар',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'үнсэлт: том хүн том хүн бордуу бага зэргийн цайвар',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'үнсэлт: том хүн том хүн бордуу дунд зэргийн',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'үнсэлт: том хүн том хүн бордуу бор',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'үнсэлт: том хүн том хүн бор цайвар',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'үнсэлт: том хүн том хүн бор бага зэргийн цайвар',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'үнсэлт: том хүн том хүн бор дунд зэргийн',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'үнсэлт: том хүн том хүн бор бордуу',
    '👨🏻‍❤‍💋‍👨🏻' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн цайвар',
    '👨🏻‍❤‍💋‍👨🏼' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн цайвар бага зэргийн цайвар',
    '👨🏻‍❤‍💋‍👨🏽' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн цайвар дунд зэргийн',
    '👨🏻‍❤‍💋‍👨🏾' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн цайвар бордуу',
    '👨🏻‍❤‍💋‍👨🏿' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн цайвар бор',
    '👨🏼‍❤‍💋‍👨🏻' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар цайвар',
    '👨🏼‍❤‍💋‍👨🏼' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар',
    '👨🏼‍❤‍💋‍👨🏽' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар дунд зэргийн',
    '👨🏼‍❤‍💋‍👨🏾' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бордуу',
    '👨🏼‍❤‍💋‍👨🏿' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бор',
    '👨🏽‍❤‍💋‍👨🏻' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн дунд зэргийн цайвар',
    '👨🏽‍❤‍💋‍👨🏼' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн дунд зэргийн бага зэргийн цайвар',
    '👨🏽‍❤‍💋‍👨🏽' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн дунд зэргийн',
    '👨🏽‍❤‍💋‍👨🏾' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн дунд зэргийн бордуу',
    '👨🏽‍❤‍💋‍👨🏿' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн дунд зэргийн бор',
    '👨🏾‍❤‍💋‍👨🏻' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бордуу цайвар',
    '👨🏾‍❤‍💋‍👨🏼' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бордуу бага зэргийн цайвар',
    '👨🏾‍❤‍💋‍👨🏽' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бордуу дунд зэргийн',
    '👨🏾‍❤‍💋‍👨🏾' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бордуу',
    '👨🏾‍❤‍💋‍👨🏿' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бордуу бор',
    '👨🏿‍❤‍💋‍👨🏻' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бор цайвар',
    '👨🏿‍❤‍💋‍👨🏼' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бор бага зэргийн цайвар',
    '👨🏿‍❤‍💋‍👨🏽' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бор дунд зэргийн',
    '👨🏿‍❤‍💋‍👨🏾' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бор бордуу',
    '👨🏿‍❤‍💋‍👨🏿' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн бор',
    '👩🏻‍❤‍💋‍👨🏻' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн цайвар',
    '👩🏻‍❤‍💋‍👨🏼' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн цайвар бага зэргийн цайвар',
    '👩🏻‍❤‍💋‍👨🏽' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн цайвар дунд зэргийн',
    '👩🏻‍❤‍💋‍👨🏾' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн цайвар бордуу',
    '👩🏻‍❤‍💋‍👨🏿' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн цайвар бор',
    '👩🏼‍❤‍💋‍👨🏻' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар цайвар',
    '👩🏼‍❤‍💋‍👨🏼' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар',
    '👩🏼‍❤‍💋‍👨🏽' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍❤‍💋‍👨🏾' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бордуу',
    '👩🏼‍❤‍💋‍👨🏿' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бор',
    '👩🏽‍❤‍💋‍👨🏻' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн дунд зэргийн цайвар',
    '👩🏽‍❤‍💋‍👨🏼' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍❤‍💋‍👨🏽' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн дунд зэргийн',
    '👩🏽‍❤‍💋‍👨🏾' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн дунд зэргийн бордуу',
    '👩🏽‍❤‍💋‍👨🏿' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн дунд зэргийн бор',
    '👩🏾‍❤‍💋‍👨🏻' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бордуу цайвар',
    '👩🏾‍❤‍💋‍👨🏼' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бордуу бага зэргийн цайвар',
    '👩🏾‍❤‍💋‍👨🏽' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бордуу дунд зэргийн',
    '👩🏾‍❤‍💋‍👨🏾' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бордуу',
    '👩🏾‍❤‍💋‍👨🏿' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бордуу бор',
    '👩🏿‍❤‍💋‍👨🏻' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бор цайвар',
    '👩🏿‍❤‍💋‍👨🏼' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бор бага зэргийн цайвар',
    '👩🏿‍❤‍💋‍👨🏽' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бор дунд зэргийн',
    '👩🏿‍❤‍💋‍👨🏾' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бор бордуу',
    '👩🏿‍❤‍💋‍👨🏿' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн бор',
    '👩🏻‍❤‍💋‍👩🏻' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн цайвар',
    '👩🏻‍❤‍💋‍👩🏼' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн цайвар бага зэргийн цайвар',
    '👩🏻‍❤‍💋‍👩🏽' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн цайвар дунд зэргийн',
    '👩🏻‍❤‍💋‍👩🏾' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн цайвар бордуу',
    '👩🏻‍❤‍💋‍👩🏿' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн цайвар бор',
    '👩🏼‍❤‍💋‍👩🏻' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар цайвар',
    '👩🏼‍❤‍💋‍👩🏼' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар',
    '👩🏼‍❤‍💋‍👩🏽' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍❤‍💋‍👩🏾' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар бордуу',
    '👩🏼‍❤‍💋‍👩🏿' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар бор',
    '👩🏽‍❤‍💋‍👩🏻' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн дунд зэргийн цайвар',
    '👩🏽‍❤‍💋‍👩🏼' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍❤‍💋‍👩🏽' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн дунд зэргийн',
    '👩🏽‍❤‍💋‍👩🏾' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн дунд зэргийн бордуу',
    '👩🏽‍❤‍💋‍👩🏿' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн дунд зэргийн бор',
    '👩🏾‍❤‍💋‍👩🏻' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бордуу цайвар',
    '👩🏾‍❤‍💋‍👩🏼' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бордуу бага зэргийн цайвар',
    '👩🏾‍❤‍💋‍👩🏽' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бордуу дунд зэргийн',
    '👩🏾‍❤‍💋‍👩🏾' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бордуу',
    '👩🏾‍❤‍💋‍👩🏿' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бордуу бор',
    '👩🏿‍❤‍💋‍👩🏻' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бор цайвар',
    '👩🏿‍❤‍💋‍👩🏼' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бор бага зэргийн цайвар',
    '👩🏿‍❤‍💋‍👩🏽' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бор дунд зэргийн',
    '👩🏿‍❤‍💋‍👩🏾' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бор бордуу',
    '👩🏿‍❤‍💋‍👩🏿' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн бор',
    '🧑🏻‍🫯‍🧑🏼' => 'бөх: цайвар бага зэргийн цайвар',
    '🧑🏻‍🫯‍🧑🏽' => 'бөх: цайвар дунд зэргийн',
    '🧑🏻‍🫯‍🧑🏾' => 'бөх: цайвар бордуу',
    '🧑🏻‍🫯‍🧑🏿' => 'бөх: цайвар бор',
    '🧑🏼‍🫯‍🧑🏻' => 'бөх: бага зэргийн цайвар цайвар',
    '🧑🏼‍🫯‍🧑🏽' => 'бөх: бага зэргийн цайвар дунд зэргийн',
    '🧑🏼‍🫯‍🧑🏾' => 'бөх: бага зэргийн цайвар бордуу',
    '🧑🏼‍🫯‍🧑🏿' => 'бөх: бага зэргийн цайвар бор',
    '🧑🏽‍🫯‍🧑🏻' => 'бөх: дунд зэргийн цайвар',
    '🧑🏽‍🫯‍🧑🏼' => 'бөх: дунд зэргийн бага зэргийн цайвар',
    '🧑🏽‍🫯‍🧑🏾' => 'бөх: дунд зэргийн бордуу',
    '🧑🏽‍🫯‍🧑🏿' => 'бөх: дунд зэргийн бор',
    '🧑🏾‍🫯‍🧑🏻' => 'бөх: бордуу цайвар',
    '🧑🏾‍🫯‍🧑🏼' => 'бөх: бордуу бага зэргийн цайвар',
    '🧑🏾‍🫯‍🧑🏽' => 'бөх: бордуу дунд зэргийн',
    '🧑🏾‍🫯‍🧑🏿' => 'бөх: бордуу бор',
    '🧑🏿‍🫯‍🧑🏻' => 'бөх: бор цайвар',
    '🧑🏿‍🫯‍🧑🏼' => 'бөх: бор бага зэргийн цайвар',
    '🧑🏿‍🫯‍🧑🏽' => 'бөх: бор дунд зэргийн',
    '🧑🏿‍🫯‍🧑🏾' => 'бөх: бор бордуу',
    '🧑🏻‍❤‍🧑🏼' => 'дундаа зүрхтэй хосууд: том хүн том хүн цайвар бага зэргийн цайвар',
    '🧑🏻‍❤‍🧑🏽' => 'дундаа зүрхтэй хосууд: том хүн том хүн цайвар дунд зэргийн',
    '🧑🏻‍❤‍🧑🏾' => 'дундаа зүрхтэй хосууд: том хүн том хүн цайвар бордуу',
    '🧑🏻‍❤‍🧑🏿' => 'дундаа зүрхтэй хосууд: том хүн том хүн цайвар бор',
    '🧑🏼‍❤‍🧑🏻' => 'дундаа зүрхтэй хосууд: том хүн том хүн бага зэргийн цайвар цайвар',
    '🧑🏼‍❤‍🧑🏽' => 'дундаа зүрхтэй хосууд: том хүн том хүн бага зэргийн цайвар дунд зэргийн',
    '🧑🏼‍❤‍🧑🏾' => 'дундаа зүрхтэй хосууд: том хүн том хүн бага зэргийн цайвар бордуу',
    '🧑🏼‍❤‍🧑🏿' => 'дундаа зүрхтэй хосууд: том хүн том хүн бага зэргийн цайвар бор',
    '🧑🏽‍❤‍🧑🏻' => 'дундаа зүрхтэй хосууд: том хүн том хүн дунд зэргийн цайвар',
    '🧑🏽‍❤‍🧑🏼' => 'дундаа зүрхтэй хосууд: том хүн том хүн дунд зэргийн бага зэргийн цайвар',
    '🧑🏽‍❤‍🧑🏾' => 'дундаа зүрхтэй хосууд: том хүн том хүн дунд зэргийн бордуу',
    '🧑🏽‍❤‍🧑🏿' => 'дундаа зүрхтэй хосууд: том хүн том хүн дунд зэргийн бор',
    '🧑🏾‍❤‍🧑🏻' => 'дундаа зүрхтэй хосууд: том хүн том хүн бордуу цайвар',
    '🧑🏾‍❤‍🧑🏼' => 'дундаа зүрхтэй хосууд: том хүн том хүн бордуу бага зэргийн цайвар',
    '🧑🏾‍❤‍🧑🏽' => 'дундаа зүрхтэй хосууд: том хүн том хүн бордуу дунд зэргийн',
    '🧑🏾‍❤‍🧑🏿' => 'дундаа зүрхтэй хосууд: том хүн том хүн бордуу бор',
    '🧑🏿‍❤‍🧑🏻' => 'дундаа зүрхтэй хосууд: том хүн том хүн бор цайвар',
    '🧑🏿‍❤‍🧑🏼' => 'дундаа зүрхтэй хосууд: том хүн том хүн бор бага зэргийн цайвар',
    '🧑🏿‍❤‍🧑🏽' => 'дундаа зүрхтэй хосууд: том хүн том хүн бор дунд зэргийн',
    '🧑🏿‍❤‍🧑🏾' => 'дундаа зүрхтэй хосууд: том хүн том хүн бор бордуу',
    '🧑🏻‍🐰‍🧑🏼' => 'үдэшлэгт байгаа бүсгүй: цайвар бага зэргийн цайвар',
    '🧑🏻‍🐰‍🧑🏽' => 'үдэшлэгт байгаа бүсгүй: цайвар дунд зэргийн',
    '🧑🏻‍🐰‍🧑🏾' => 'үдэшлэгт байгаа бүсгүй: цайвар бордуу',
    '🧑🏻‍🐰‍🧑🏿' => 'үдэшлэгт байгаа бүсгүй: цайвар бор',
    '🧑🏼‍🐰‍🧑🏻' => 'үдэшлэгт байгаа бүсгүй: бага зэргийн цайвар цайвар',
    '🧑🏼‍🐰‍🧑🏽' => 'үдэшлэгт байгаа бүсгүй: бага зэргийн цайвар дунд зэргийн',
    '🧑🏼‍🐰‍🧑🏾' => 'үдэшлэгт байгаа бүсгүй: бага зэргийн цайвар бордуу',
    '🧑🏼‍🐰‍🧑🏿' => 'үдэшлэгт байгаа бүсгүй: бага зэргийн цайвар бор',
    '🧑🏽‍🐰‍🧑🏻' => 'үдэшлэгт байгаа бүсгүй: дунд зэргийн цайвар',
    '🧑🏽‍🐰‍🧑🏼' => 'үдэшлэгт байгаа бүсгүй: дунд зэргийн бага зэргийн цайвар',
    '🧑🏽‍🐰‍🧑🏾' => 'үдэшлэгт байгаа бүсгүй: дунд зэргийн бордуу',
    '🧑🏽‍🐰‍🧑🏿' => 'үдэшлэгт байгаа бүсгүй: дунд зэргийн бор',
    '🧑🏾‍🐰‍🧑🏻' => 'үдэшлэгт байгаа бүсгүй: бордуу цайвар',
    '🧑🏾‍🐰‍🧑🏼' => 'үдэшлэгт байгаа бүсгүй: бордуу бага зэргийн цайвар',
    '🧑🏾‍🐰‍🧑🏽' => 'үдэшлэгт байгаа бүсгүй: бордуу дунд зэргийн',
    '🧑🏾‍🐰‍🧑🏿' => 'үдэшлэгт байгаа бүсгүй: бордуу бор',
    '🧑🏿‍🐰‍🧑🏻' => 'үдэшлэгт байгаа бүсгүй: бор цайвар',
    '🧑🏿‍🐰‍🧑🏼' => 'үдэшлэгт байгаа бүсгүй: бор бага зэргийн цайвар',
    '🧑🏿‍🐰‍🧑🏽' => 'үдэшлэгт байгаа бүсгүй: бор дунд зэргийн',
    '🧑🏿‍🐰‍🧑🏾' => 'үдэшлэгт байгаа бүсгүй: бор бордуу',
    '👨🏻‍🫯‍👨🏼' => 'барилдаж буй эрэгтэйчүүд: цайвар бага зэргийн цайвар',
    '👨🏻‍🫯‍👨🏽' => 'барилдаж буй эрэгтэйчүүд: цайвар дунд зэргийн',
    '👨🏻‍🫯‍👨🏾' => 'барилдаж буй эрэгтэйчүүд: цайвар бордуу',
    '👨🏻‍🫯‍👨🏿' => 'барилдаж буй эрэгтэйчүүд: цайвар бор',
    '👨🏼‍🫯‍👨🏻' => 'барилдаж буй эрэгтэйчүүд: бага зэргийн цайвар цайвар',
    '👨🏼‍🫯‍👨🏽' => 'барилдаж буй эрэгтэйчүүд: бага зэргийн цайвар дунд зэргийн',
    '👨🏼‍🫯‍👨🏾' => 'барилдаж буй эрэгтэйчүүд: бага зэргийн цайвар бордуу',
    '👨🏼‍🫯‍👨🏿' => 'барилдаж буй эрэгтэйчүүд: бага зэргийн цайвар бор',
    '👨🏽‍🫯‍👨🏻' => 'барилдаж буй эрэгтэйчүүд: дунд зэргийн цайвар',
    '👨🏽‍🫯‍👨🏼' => 'барилдаж буй эрэгтэйчүүд: дунд зэргийн бага зэргийн цайвар',
    '👨🏽‍🫯‍👨🏾' => 'барилдаж буй эрэгтэйчүүд: дунд зэргийн бордуу',
    '👨🏽‍🫯‍👨🏿' => 'барилдаж буй эрэгтэйчүүд: дунд зэргийн бор',
    '👨🏾‍🫯‍👨🏻' => 'барилдаж буй эрэгтэйчүүд: бордуу цайвар',
    '👨🏾‍🫯‍👨🏼' => 'барилдаж буй эрэгтэйчүүд: бордуу бага зэргийн цайвар',
    '👨🏾‍🫯‍👨🏽' => 'барилдаж буй эрэгтэйчүүд: бордуу дунд зэргийн',
    '👨🏾‍🫯‍👨🏿' => 'барилдаж буй эрэгтэйчүүд: бордуу бор',
    '👨🏿‍🫯‍👨🏻' => 'барилдаж буй эрэгтэйчүүд: бор цайвар',
    '👨🏿‍🫯‍👨🏼' => 'барилдаж буй эрэгтэйчүүд: бор бага зэргийн цайвар',
    '👨🏿‍🫯‍👨🏽' => 'барилдаж буй эрэгтэйчүүд: бор дунд зэргийн',
    '👨🏿‍🫯‍👨🏾' => 'барилдаж буй эрэгтэйчүүд: бор бордуу',
    '👨🏻‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн цайвар',
    '👨🏻‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн цайвар бага зэргийн цайвар',
    '👨🏻‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн цайвар дунд зэргийн',
    '👨🏻‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн цайвар бордуу',
    '👨🏻‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн цайвар бор',
    '👨🏼‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар цайвар',
    '👨🏼‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар',
    '👨🏼‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар дунд зэргийн',
    '👨🏼‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бордуу',
    '👨🏼‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бор',
    '👨🏽‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн дунд зэргийн цайвар',
    '👨🏽‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн дунд зэргийн бага зэргийн цайвар',
    '👨🏽‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн дунд зэргийн',
    '👨🏽‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн дунд зэргийн бордуу',
    '👨🏽‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн дунд зэргийн бор',
    '👨🏾‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бордуу цайвар',
    '👨🏾‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бордуу бага зэргийн цайвар',
    '👨🏾‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бордуу дунд зэргийн',
    '👨🏾‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бордуу',
    '👨🏾‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бордуу бор',
    '👨🏿‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бор цайвар',
    '👨🏿‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бор бага зэргийн цайвар',
    '👨🏿‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бор дунд зэргийн',
    '👨🏿‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бор бордуу',
    '👨🏿‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн бор',
    '👨🏻‍🐰‍👨🏼' => 'цэнгэж буй эрэгтэйчүүд: цайвар бага зэргийн цайвар',
    '👨🏻‍🐰‍👨🏽' => 'цэнгэж буй эрэгтэйчүүд: цайвар дунд зэргийн',
    '👨🏻‍🐰‍👨🏾' => 'цэнгэж буй эрэгтэйчүүд: цайвар бордуу',
    '👨🏻‍🐰‍👨🏿' => 'цэнгэж буй эрэгтэйчүүд: цайвар бор',
    '👨🏼‍🐰‍👨🏻' => 'цэнгэж буй эрэгтэйчүүд: бага зэргийн цайвар цайвар',
    '👨🏼‍🐰‍👨🏽' => 'цэнгэж буй эрэгтэйчүүд: бага зэргийн цайвар дунд зэргийн',
    '👨🏼‍🐰‍👨🏾' => 'цэнгэж буй эрэгтэйчүүд: бага зэргийн цайвар бордуу',
    '👨🏼‍🐰‍👨🏿' => 'цэнгэж буй эрэгтэйчүүд: бага зэргийн цайвар бор',
    '👨🏽‍🐰‍👨🏻' => 'цэнгэж буй эрэгтэйчүүд: дунд зэргийн цайвар',
    '👨🏽‍🐰‍👨🏼' => 'цэнгэж буй эрэгтэйчүүд: дунд зэргийн бага зэргийн цайвар',
    '👨🏽‍🐰‍👨🏾' => 'цэнгэж буй эрэгтэйчүүд: дунд зэргийн бордуу',
    '👨🏽‍🐰‍👨🏿' => 'цэнгэж буй эрэгтэйчүүд: дунд зэргийн бор',
    '👨🏾‍🐰‍👨🏻' => 'цэнгэж буй эрэгтэйчүүд: бордуу цайвар',
    '👨🏾‍🐰‍👨🏼' => 'цэнгэж буй эрэгтэйчүүд: бордуу бага зэргийн цайвар',
    '👨🏾‍🐰‍👨🏽' => 'цэнгэж буй эрэгтэйчүүд: бордуу дунд зэргийн',
    '👨🏾‍🐰‍👨🏿' => 'цэнгэж буй эрэгтэйчүүд: бордуу бор',
    '👨🏿‍🐰‍👨🏻' => 'цэнгэж буй эрэгтэйчүүд: бор цайвар',
    '👨🏿‍🐰‍👨🏼' => 'цэнгэж буй эрэгтэйчүүд: бор бага зэргийн цайвар',
    '👨🏿‍🐰‍👨🏽' => 'цэнгэж буй эрэгтэйчүүд: бор дунд зэргийн',
    '👨🏿‍🐰‍👨🏾' => 'цэнгэж буй эрэгтэйчүүд: бор бордуу',
    '👩🏻‍🫯‍👩🏼' => 'барилдаж буй эмэгтэйчүүд: цайвар бага зэргийн цайвар',
    '👩🏻‍🫯‍👩🏽' => 'барилдаж буй эмэгтэйчүүд: цайвар дунд зэргийн',
    '👩🏻‍🫯‍👩🏾' => 'барилдаж буй эмэгтэйчүүд: цайвар бордуу',
    '👩🏻‍🫯‍👩🏿' => 'барилдаж буй эмэгтэйчүүд: цайвар бор',
    '👩🏼‍🫯‍👩🏻' => 'барилдаж буй эмэгтэйчүүд: бага зэргийн цайвар цайвар',
    '👩🏼‍🫯‍👩🏽' => 'барилдаж буй эмэгтэйчүүд: бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍🫯‍👩🏾' => 'барилдаж буй эмэгтэйчүүд: бага зэргийн цайвар бордуу',
    '👩🏼‍🫯‍👩🏿' => 'барилдаж буй эмэгтэйчүүд: бага зэргийн цайвар бор',
    '👩🏽‍🫯‍👩🏻' => 'барилдаж буй эмэгтэйчүүд: дунд зэргийн цайвар',
    '👩🏽‍🫯‍👩🏼' => 'барилдаж буй эмэгтэйчүүд: дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍🫯‍👩🏾' => 'барилдаж буй эмэгтэйчүүд: дунд зэргийн бордуу',
    '👩🏽‍🫯‍👩🏿' => 'барилдаж буй эмэгтэйчүүд: дунд зэргийн бор',
    '👩🏾‍🫯‍👩🏻' => 'барилдаж буй эмэгтэйчүүд: бордуу цайвар',
    '👩🏾‍🫯‍👩🏼' => 'барилдаж буй эмэгтэйчүүд: бордуу бага зэргийн цайвар',
    '👩🏾‍🫯‍👩🏽' => 'барилдаж буй эмэгтэйчүүд: бордуу дунд зэргийн',
    '👩🏾‍🫯‍👩🏿' => 'барилдаж буй эмэгтэйчүүд: бордуу бор',
    '👩🏿‍🫯‍👩🏻' => 'барилдаж буй эмэгтэйчүүд: бор цайвар',
    '👩🏿‍🫯‍👩🏼' => 'барилдаж буй эмэгтэйчүүд: бор бага зэргийн цайвар',
    '👩🏿‍🫯‍👩🏽' => 'барилдаж буй эмэгтэйчүүд: бор дунд зэргийн',
    '👩🏿‍🫯‍👩🏾' => 'барилдаж буй эмэгтэйчүүд: бор бордуу',
    '👩🏻‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн цайвар',
    '👩🏻‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн цайвар бага зэргийн цайвар',
    '👩🏻‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн цайвар дунд зэргийн',
    '👩🏻‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн цайвар бордуу',
    '👩🏻‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн цайвар бор',
    '👩🏼‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар цайвар',
    '👩🏼‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар',
    '👩🏼‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бордуу',
    '👩🏼‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бага зэргийн цайвар бор',
    '👩🏽‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн дунд зэргийн цайвар',
    '👩🏽‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн дунд зэргийн',
    '👩🏽‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн дунд зэргийн бордуу',
    '👩🏽‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн дунд зэргийн бор',
    '👩🏾‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бордуу цайвар',
    '👩🏾‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бордуу бага зэргийн цайвар',
    '👩🏾‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бордуу дунд зэргийн',
    '👩🏾‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бордуу',
    '👩🏾‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бордуу бор',
    '👩🏿‍❤‍👨🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бор цайвар',
    '👩🏿‍❤‍👨🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бор бага зэргийн цайвар',
    '👩🏿‍❤‍👨🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бор дунд зэргийн',
    '👩🏿‍❤‍👨🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бор бордуу',
    '👩🏿‍❤‍👨🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн бор',
    '👩🏻‍❤‍👩🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн цайвар',
    '👩🏻‍❤‍👩🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн цайвар бага зэргийн цайвар',
    '👩🏻‍❤‍👩🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн цайвар дунд зэргийн',
    '👩🏻‍❤‍👩🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн цайвар бордуу',
    '👩🏻‍❤‍👩🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн цайвар бор',
    '👩🏼‍❤‍👩🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар цайвар',
    '👩🏼‍❤‍👩🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар',
    '👩🏼‍❤‍👩🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍❤‍👩🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар бордуу',
    '👩🏼‍❤‍👩🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бага зэргийн цайвар бор',
    '👩🏽‍❤‍👩🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн дунд зэргийн цайвар',
    '👩🏽‍❤‍👩🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍❤‍👩🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн дунд зэргийн',
    '👩🏽‍❤‍👩🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн дунд зэргийн бордуу',
    '👩🏽‍❤‍👩🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн дунд зэргийн бор',
    '👩🏾‍❤‍👩🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бордуу цайвар',
    '👩🏾‍❤‍👩🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бордуу бага зэргийн цайвар',
    '👩🏾‍❤‍👩🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бордуу дунд зэргийн',
    '👩🏾‍❤‍👩🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бордуу',
    '👩🏾‍❤‍👩🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бордуу бор',
    '👩🏿‍❤‍👩🏻' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бор цайвар',
    '👩🏿‍❤‍👩🏼' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бор бага зэргийн цайвар',
    '👩🏿‍❤‍👩🏽' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бор дунд зэргийн',
    '👩🏿‍❤‍👩🏾' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бор бордуу',
    '👩🏿‍❤‍👩🏿' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн бор',
    '👩🏻‍🐰‍👩🏼' => 'цэнгэж буй эмэгтэйчүүд: цайвар бага зэргийн цайвар',
    '👩🏻‍🐰‍👩🏽' => 'цэнгэж буй эмэгтэйчүүд: цайвар дунд зэргийн',
    '👩🏻‍🐰‍👩🏾' => 'цэнгэж буй эмэгтэйчүүд: цайвар бордуу',
    '👩🏻‍🐰‍👩🏿' => 'цэнгэж буй эмэгтэйчүүд: цайвар бор',
    '👩🏼‍🐰‍👩🏻' => 'цэнгэж буй эмэгтэйчүүд: бага зэргийн цайвар цайвар',
    '👩🏼‍🐰‍👩🏽' => 'цэнгэж буй эмэгтэйчүүд: бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍🐰‍👩🏾' => 'цэнгэж буй эмэгтэйчүүд: бага зэргийн цайвар бордуу',
    '👩🏼‍🐰‍👩🏿' => 'цэнгэж буй эмэгтэйчүүд: бага зэргийн цайвар бор',
    '👩🏽‍🐰‍👩🏻' => 'цэнгэж буй эмэгтэйчүүд: дунд зэргийн цайвар',
    '👩🏽‍🐰‍👩🏼' => 'цэнгэж буй эмэгтэйчүүд: дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍🐰‍👩🏾' => 'цэнгэж буй эмэгтэйчүүд: дунд зэргийн бордуу',
    '👩🏽‍🐰‍👩🏿' => 'цэнгэж буй эмэгтэйчүүд: дунд зэргийн бор',
    '👩🏾‍🐰‍👩🏻' => 'цэнгэж буй эмэгтэйчүүд: бордуу цайвар',
    '👩🏾‍🐰‍👩🏼' => 'цэнгэж буй эмэгтэйчүүд: бордуу бага зэргийн цайвар',
    '👩🏾‍🐰‍👩🏽' => 'цэнгэж буй эмэгтэйчүүд: бордуу дунд зэргийн',
    '👩🏾‍🐰‍👩🏿' => 'цэнгэж буй эмэгтэйчүүд: бордуу бор',
    '👩🏿‍🐰‍👩🏻' => 'цэнгэж буй эмэгтэйчүүд: бор цайвар',
    '👩🏿‍🐰‍👩🏼' => 'цэнгэж буй эмэгтэйчүүд: бор бага зэргийн цайвар',
    '👩🏿‍🐰‍👩🏽' => 'цэнгэж буй эмэгтэйчүүд: бор дунд зэргийн',
    '👩🏿‍🐰‍👩🏾' => 'цэнгэж буй эмэгтэйчүүд: бор бордуу',
    '🧑🏻‍🤝‍🧑🏻' => 'гар барилцаж буй хүмүүс: цайвар',
    '🧑🏻‍🤝‍🧑🏼' => 'гар барилцаж буй хүмүүс: цайвар бага зэргийн цайвар',
    '🧑🏻‍🤝‍🧑🏽' => 'гар барилцаж буй хүмүүс: цайвар дунд зэргийн',
    '🧑🏻‍🤝‍🧑🏾' => 'гар барилцаж буй хүмүүс: цайвар бордуу',
    '🧑🏻‍🤝‍🧑🏿' => 'гар барилцаж буй хүмүүс: цайвар бор',
    '🧑🏼‍🤝‍🧑🏻' => 'гар барилцаж буй хүмүүс: бага зэргийн цайвар цайвар',
    '🧑🏼‍🤝‍🧑🏼' => 'гар барилцаж буй хүмүүс: бага зэргийн цайвар',
    '🧑🏼‍🤝‍🧑🏽' => 'гар барилцаж буй хүмүүс: бага зэргийн цайвар дунд зэргийн',
    '🧑🏼‍🤝‍🧑🏾' => 'гар барилцаж буй хүмүүс: бага зэргийн цайвар бордуу',
    '🧑🏼‍🤝‍🧑🏿' => 'гар барилцаж буй хүмүүс: бага зэргийн цайвар бор',
    '🧑🏽‍🤝‍🧑🏻' => 'гар барилцаж буй хүмүүс: дунд зэргийн цайвар',
    '🧑🏽‍🤝‍🧑🏼' => 'гар барилцаж буй хүмүүс: дунд зэргийн бага зэргийн цайвар',
    '🧑🏽‍🤝‍🧑🏽' => 'гар барилцаж буй хүмүүс: дунд зэргийн',
    '🧑🏽‍🤝‍🧑🏾' => 'гар барилцаж буй хүмүүс: дунд зэргийн бордуу',
    '🧑🏽‍🤝‍🧑🏿' => 'гар барилцаж буй хүмүүс: дунд зэргийн бор',
    '🧑🏾‍🤝‍🧑🏻' => 'гар барилцаж буй хүмүүс: бордуу цайвар',
    '🧑🏾‍🤝‍🧑🏼' => 'гар барилцаж буй хүмүүс: бордуу бага зэргийн цайвар',
    '🧑🏾‍🤝‍🧑🏽' => 'гар барилцаж буй хүмүүс: бордуу дунд зэргийн',
    '🧑🏾‍🤝‍🧑🏾' => 'гар барилцаж буй хүмүүс: бордуу',
    '🧑🏾‍🤝‍🧑🏿' => 'гар барилцаж буй хүмүүс: бордуу бор',
    '🧑🏿‍🤝‍🧑🏻' => 'гар барилцаж буй хүмүүс: бор цайвар',
    '🧑🏿‍🤝‍🧑🏼' => 'гар барилцаж буй хүмүүс: бор бага зэргийн цайвар',
    '🧑🏿‍🤝‍🧑🏽' => 'гар барилцаж буй хүмүүс: бор дунд зэргийн',
    '🧑🏿‍🤝‍🧑🏾' => 'гар барилцаж буй хүмүүс: бор бордуу',
    '🧑🏿‍🤝‍🧑🏿' => 'гар барилцаж буй хүмүүс: бор',
    '👩🏻‍🤝‍👩🏼' => 'хөтлөлцсөн хоёр эмэгтэй: цайвар бага зэргийн цайвар',
    '👩🏻‍🤝‍👩🏽' => 'хөтлөлцсөн хоёр эмэгтэй: цайвар дунд зэргийн',
    '👩🏻‍🤝‍👩🏾' => 'хөтлөлцсөн хоёр эмэгтэй: цайвар бордуу',
    '👩🏻‍🤝‍👩🏿' => 'хөтлөлцсөн хоёр эмэгтэй: цайвар бор',
    '👩🏼‍🤝‍👩🏻' => 'хөтлөлцсөн хоёр эмэгтэй: бага зэргийн цайвар цайвар',
    '👩🏼‍🤝‍👩🏽' => 'хөтлөлцсөн хоёр эмэгтэй: бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍🤝‍👩🏾' => 'хөтлөлцсөн хоёр эмэгтэй: бага зэргийн цайвар бордуу',
    '👩🏼‍🤝‍👩🏿' => 'хөтлөлцсөн хоёр эмэгтэй: бага зэргийн цайвар бор',
    '👩🏽‍🤝‍👩🏻' => 'хөтлөлцсөн хоёр эмэгтэй: дунд зэргийн цайвар',
    '👩🏽‍🤝‍👩🏼' => 'хөтлөлцсөн хоёр эмэгтэй: дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍🤝‍👩🏾' => 'хөтлөлцсөн хоёр эмэгтэй: дунд зэргийн бордуу',
    '👩🏽‍🤝‍👩🏿' => 'хөтлөлцсөн хоёр эмэгтэй: дунд зэргийн бор',
    '👩🏾‍🤝‍👩🏻' => 'хөтлөлцсөн хоёр эмэгтэй: бордуу цайвар',
    '👩🏾‍🤝‍👩🏼' => 'хөтлөлцсөн хоёр эмэгтэй: бордуу бага зэргийн цайвар',
    '👩🏾‍🤝‍👩🏽' => 'хөтлөлцсөн хоёр эмэгтэй: бордуу дунд зэргийн',
    '👩🏾‍🤝‍👩🏿' => 'хөтлөлцсөн хоёр эмэгтэй: бордуу бор',
    '👩🏿‍🤝‍👩🏻' => 'хөтлөлцсөн хоёр эмэгтэй: бор цайвар',
    '👩🏿‍🤝‍👩🏼' => 'хөтлөлцсөн хоёр эмэгтэй: бор бага зэргийн цайвар',
    '👩🏿‍🤝‍👩🏽' => 'хөтлөлцсөн хоёр эмэгтэй: бор дунд зэргийн',
    '👩🏿‍🤝‍👩🏾' => 'хөтлөлцсөн хоёр эмэгтэй: бор бордуу',
    '👩🏻‍🤝‍👨🏼' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: цайвар бага зэргийн цайвар',
    '👩🏻‍🤝‍👨🏽' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: цайвар дунд зэргийн',
    '👩🏻‍🤝‍👨🏾' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: цайвар бордуу',
    '👩🏻‍🤝‍👨🏿' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: цайвар бор',
    '👩🏼‍🤝‍👨🏻' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бага зэргийн цайвар цайвар',
    '👩🏼‍🤝‍👨🏽' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бага зэргийн цайвар дунд зэргийн',
    '👩🏼‍🤝‍👨🏾' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бага зэргийн цайвар бордуу',
    '👩🏼‍🤝‍👨🏿' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бага зэргийн цайвар бор',
    '👩🏽‍🤝‍👨🏻' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: дунд зэргийн цайвар',
    '👩🏽‍🤝‍👨🏼' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: дунд зэргийн бага зэргийн цайвар',
    '👩🏽‍🤝‍👨🏾' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: дунд зэргийн бордуу',
    '👩🏽‍🤝‍👨🏿' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: дунд зэргийн бор',
    '👩🏾‍🤝‍👨🏻' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бордуу цайвар',
    '👩🏾‍🤝‍👨🏼' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бордуу бага зэргийн цайвар',
    '👩🏾‍🤝‍👨🏽' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бордуу дунд зэргийн',
    '👩🏾‍🤝‍👨🏿' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бордуу бор',
    '👩🏿‍🤝‍👨🏻' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бор цайвар',
    '👩🏿‍🤝‍👨🏼' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бор бага зэргийн цайвар',
    '👩🏿‍🤝‍👨🏽' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бор дунд зэргийн',
    '👩🏿‍🤝‍👨🏾' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бор бордуу',
    '👨🏻‍🤝‍👨🏼' => 'хөтлөлцсөн хоёр эрэгтэй: цайвар бага зэргийн цайвар',
    '👨🏻‍🤝‍👨🏽' => 'хөтлөлцсөн хоёр эрэгтэй: цайвар дунд зэргийн',
    '👨🏻‍🤝‍👨🏾' => 'хөтлөлцсөн хоёр эрэгтэй: цайвар бордуу',
    '👨🏻‍🤝‍👨🏿' => 'хөтлөлцсөн хоёр эрэгтэй: цайвар бор',
    '👨🏼‍🤝‍👨🏻' => 'хөтлөлцсөн хоёр эрэгтэй: бага зэргийн цайвар цайвар',
    '👨🏼‍🤝‍👨🏽' => 'хөтлөлцсөн хоёр эрэгтэй: бага зэргийн цайвар дунд зэргийн',
    '👨🏼‍🤝‍👨🏾' => 'хөтлөлцсөн хоёр эрэгтэй: бага зэргийн цайвар бордуу',
    '👨🏼‍🤝‍👨🏿' => 'хөтлөлцсөн хоёр эрэгтэй: бага зэргийн цайвар бор',
    '👨🏽‍🤝‍👨🏻' => 'хөтлөлцсөн хоёр эрэгтэй: дунд зэргийн цайвар',
    '👨🏽‍🤝‍👨🏼' => 'хөтлөлцсөн хоёр эрэгтэй: дунд зэргийн бага зэргийн цайвар',
    '👨🏽‍🤝‍👨🏾' => 'хөтлөлцсөн хоёр эрэгтэй: дунд зэргийн бордуу',
    '👨🏽‍🤝‍👨🏿' => 'хөтлөлцсөн хоёр эрэгтэй: дунд зэргийн бор',
    '👨🏾‍🤝‍👨🏻' => 'хөтлөлцсөн хоёр эрэгтэй: бордуу цайвар',
    '👨🏾‍🤝‍👨🏼' => 'хөтлөлцсөн хоёр эрэгтэй: бордуу бага зэргийн цайвар',
    '👨🏾‍🤝‍👨🏽' => 'хөтлөлцсөн хоёр эрэгтэй: бордуу дунд зэргийн',
    '👨🏾‍🤝‍👨🏿' => 'хөтлөлцсөн хоёр эрэгтэй: бордуу бор',
    '👨🏿‍🤝‍👨🏻' => 'хөтлөлцсөн хоёр эрэгтэй: бор цайвар',
    '👨🏿‍🤝‍👨🏼' => 'хөтлөлцсөн хоёр эрэгтэй: бор бага зэргийн цайвар',
    '👨🏿‍🤝‍👨🏽' => 'хөтлөлцсөн хоёр эрэгтэй: бор дунд зэргийн',
    '👨🏿‍🤝‍👨🏾' => 'хөтлөлцсөн хоёр эрэгтэй: бор бордуу',
    '👩‍❤‍💋‍👨' => 'үнсэлт: эмэгтэй хүн эрэгтэй хүн',
    '👨‍❤‍💋‍👨' => 'үнсэлт: эрэгтэй хүн эрэгтэй хүн',
    '👩‍❤‍💋‍👩' => 'үнсэлт: эмэгтэй хүн эмэгтэй хүн',
    '👨‍👩‍👧‍👦' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүн эмэгтэй хүүхэд эрэгтэй хүүхэд',
    '👨‍👩‍👦‍👦' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүн эрэгтэй хүүхэд эрэгтэй хүүхэд',
    '👨‍👩‍👧‍👧' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүн эмэгтэй хүүхэд эмэгтэй хүүхэд',
    '👨‍👨‍👧‍👦' => 'гэр бүл: эрэгтэй хүн эрэгтэй хүн эмэгтэй хүүхэд эрэгтэй хүүхэд',
    '👨‍👨‍👦‍👦' => 'гэр бүл: эрэгтэй хүн эрэгтэй хүн эрэгтэй хүүхэд эрэгтэй хүүхэд',
    '👨‍👨‍👧‍👧' => 'гэр бүл: эрэгтэй хүн эрэгтэй хүн эмэгтэй хүүхэд эмэгтэй хүүхэд',
    '👩‍👩‍👧‍👦' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүн эмэгтэй хүүхэд эрэгтэй хүүхэд',
    '👩‍👩‍👦‍👦' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүн эрэгтэй хүүхэд эрэгтэй хүүхэд',
    '👩‍👩‍👧‍👧' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүн эмэгтэй хүүхэд эмэгтэй хүүхэд',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'туг: Англи',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'туг: Шотланд',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'туг: Уэльс',
    '🧑‍🧑‍🧒‍🧒' => 'гэр бүл: насанд хүрсэн хүн, насанд хүрсэн хүн, хүүхэд, хүүхэд',
    '🚶🏻‍♀‍➡' => 'алхаж буй эмэгтэй: цайвар баруун тийш харсан',
    '🚶🏼‍♀‍➡' => 'алхаж буй эмэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '🚶🏽‍♀‍➡' => 'алхаж буй эмэгтэй: дунд зэргийн баруун тийш харсан',
    '🚶🏾‍♀‍➡' => 'алхаж буй эмэгтэй: бордуу баруун тийш харсан',
    '🚶🏿‍♀‍➡' => 'алхаж буй эмэгтэй: бор баруун тийш харсан',
    '🚶🏻‍♂‍➡' => 'алхаж буй эрэгтэй: цайвар баруун тийш харсан',
    '🚶🏼‍♂‍➡' => 'алхаж буй эрэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '🚶🏽‍♂‍➡' => 'алхаж буй эрэгтэй: дунд зэргийн баруун тийш харсан',
    '🚶🏾‍♂‍➡' => 'алхаж буй эрэгтэй: бордуу баруун тийш харсан',
    '🚶🏿‍♂‍➡' => 'алхаж буй эрэгтэй: бор баруун тийш харсан',
    '🧎🏻‍♀‍➡' => 'сөхөрч буй эмэгтэй: цайвар баруун тийш харсан',
    '🧎🏼‍♀‍➡' => 'сөхөрч буй эмэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '🧎🏽‍♀‍➡' => 'сөхөрч буй эмэгтэй: дунд зэргийн баруун тийш харсан',
    '🧎🏾‍♀‍➡' => 'сөхөрч буй эмэгтэй: бордуу баруун тийш харсан',
    '🧎🏿‍♀‍➡' => 'сөхөрч буй эмэгтэй: бор баруун тийш харсан',
    '🧎🏻‍♂‍➡' => 'сөхөрч буй эрэгтэй: цайвар баруун тийш харсан',
    '🧎🏼‍♂‍➡' => 'сөхөрч буй эрэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '🧎🏽‍♂‍➡' => 'сөхөрч буй эрэгтэй: дунд зэргийн баруун тийш харсан',
    '🧎🏾‍♂‍➡' => 'сөхөрч буй эрэгтэй: бордуу баруун тийш харсан',
    '🧎🏿‍♂‍➡' => 'сөхөрч буй эрэгтэй: бор баруун тийш харсан',
    '🧑🏻‍🦯‍➡' => 'метал илрүүлэгчтэй хүн: цайвар баруун тийш харсан',
    '🧑🏼‍🦯‍➡' => 'метал илрүүлэгчтэй хүн: бага зэргийн цайвар баруун тийш харсан',
    '🧑🏽‍🦯‍➡' => 'метал илрүүлэгчтэй хүн: дунд зэргийн баруун тийш харсан',
    '🧑🏾‍🦯‍➡' => 'метал илрүүлэгчтэй хүн: бордуу баруун тийш харсан',
    '🧑🏿‍🦯‍➡' => 'метал илрүүлэгчтэй хүн: бор баруун тийш харсан',
    '👨🏻‍🦯‍➡' => 'таягтай хараагүй эрэгтэй: цайвар баруун тийш харсан',
    '👨🏼‍🦯‍➡' => 'таягтай хараагүй эрэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '👨🏽‍🦯‍➡' => 'таягтай хараагүй эрэгтэй: дунд зэргийн баруун тийш харсан',
    '👨🏾‍🦯‍➡' => 'таягтай хараагүй эрэгтэй: бордуу баруун тийш харсан',
    '👨🏿‍🦯‍➡' => 'таягтай хараагүй эрэгтэй: бор баруун тийш харсан',
    '👩🏻‍🦯‍➡' => 'таягтай хараагүй эмэгтэй: цайвар баруун тийш харсан',
    '👩🏼‍🦯‍➡' => 'таягтай хараагүй эмэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '👩🏽‍🦯‍➡' => 'таягтай хараагүй эмэгтэй: дунд зэргийн баруун тийш харсан',
    '👩🏾‍🦯‍➡' => 'таягтай хараагүй эмэгтэй: бордуу баруун тийш харсан',
    '👩🏿‍🦯‍➡' => 'таягтай хараагүй эмэгтэй: бор баруун тийш харсан',
    '🧑🏻‍🦼‍➡' => 'автомат удирдлагатай тэргэнцэртэй хүн: цайвар баруун тийш харсан',
    '🧑🏼‍🦼‍➡' => 'автомат удирдлагатай тэргэнцэртэй хүн: бага зэргийн цайвар баруун тийш харсан',
    '🧑🏽‍🦼‍➡' => 'автомат удирдлагатай тэргэнцэртэй хүн: дунд зэргийн баруун тийш харсан',
    '🧑🏾‍🦼‍➡' => 'автомат удирдлагатай тэргэнцэртэй хүн: бордуу баруун тийш харсан',
    '🧑🏿‍🦼‍➡' => 'автомат удирдлагатай тэргэнцэртэй хүн: бор баруун тийш харсан',
    '👨🏻‍🦼‍➡' => 'моторт тэргэнцэртэй эрэгтэй: цайвар баруун тийш харсан',
    '👨🏼‍🦼‍➡' => 'моторт тэргэнцэртэй эрэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '👨🏽‍🦼‍➡' => 'моторт тэргэнцэртэй эрэгтэй: дунд зэргийн баруун тийш харсан',
    '👨🏾‍🦼‍➡' => 'моторт тэргэнцэртэй эрэгтэй: бордуу баруун тийш харсан',
    '👨🏿‍🦼‍➡' => 'моторт тэргэнцэртэй эрэгтэй: бор баруун тийш харсан',
    '👩🏻‍🦼‍➡' => 'моторт тэргэнцэртэй эмэгтэй: цайвар баруун тийш харсан',
    '👩🏼‍🦼‍➡' => 'моторт тэргэнцэртэй эмэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '👩🏽‍🦼‍➡' => 'моторт тэргэнцэртэй эмэгтэй: дунд зэргийн баруун тийш харсан',
    '👩🏾‍🦼‍➡' => 'моторт тэргэнцэртэй эмэгтэй: бордуу баруун тийш харсан',
    '👩🏿‍🦼‍➡' => 'моторт тэргэнцэртэй эмэгтэй: бор баруун тийш харсан',
    '🧑🏻‍🦽‍➡' => 'гар ажиллагаатай тэргэнцэртэй хүн: цайвар баруун тийш харсан',
    '🧑🏼‍🦽‍➡' => 'гар ажиллагаатай тэргэнцэртэй хүн: бага зэргийн цайвар баруун тийш харсан',
    '🧑🏽‍🦽‍➡' => 'гар ажиллагаатай тэргэнцэртэй хүн: дунд зэргийн баруун тийш харсан',
    '🧑🏾‍🦽‍➡' => 'гар ажиллагаатай тэргэнцэртэй хүн: бордуу баруун тийш харсан',
    '🧑🏿‍🦽‍➡' => 'гар ажиллагаатай тэргэнцэртэй хүн: бор баруун тийш харсан',
    '👨🏻‍🦽‍➡' => 'тэргэнцэртэй эрэгтэй: цайвар баруун тийш харсан',
    '👨🏼‍🦽‍➡' => 'тэргэнцэртэй эрэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '👨🏽‍🦽‍➡' => 'тэргэнцэртэй эрэгтэй: дунд зэргийн баруун тийш харсан',
    '👨🏾‍🦽‍➡' => 'тэргэнцэртэй эрэгтэй: бордуу баруун тийш харсан',
    '👨🏿‍🦽‍➡' => 'тэргэнцэртэй эрэгтэй: бор баруун тийш харсан',
    '👩🏻‍🦽‍➡' => 'тэргэнцэртэй эмэгтэй: цайвар баруун тийш харсан',
    '👩🏼‍🦽‍➡' => 'тэргэнцэртэй эмэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '👩🏽‍🦽‍➡' => 'тэргэнцэртэй эмэгтэй: дунд зэргийн баруун тийш харсан',
    '👩🏾‍🦽‍➡' => 'тэргэнцэртэй эмэгтэй: бордуу баруун тийш харсан',
    '👩🏿‍🦽‍➡' => 'тэргэнцэртэй эмэгтэй: бор баруун тийш харсан',
    '🏃🏻‍♀‍➡' => 'гүйж буй эмэгтэй: цайвар баруун тийш харсан',
    '🏃🏼‍♀‍➡' => 'гүйж буй эмэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '🏃🏽‍♀‍➡' => 'гүйж буй эмэгтэй: дунд зэргийн баруун тийш харсан',
    '🏃🏾‍♀‍➡' => 'гүйж буй эмэгтэй: бордуу баруун тийш харсан',
    '🏃🏿‍♀‍➡' => 'гүйж буй эмэгтэй: бор баруун тийш харсан',
    '🏃🏻‍♂‍➡' => 'гүйж буй эрэгтэй: цайвар баруун тийш харсан',
    '🏃🏼‍♂‍➡' => 'гүйж буй эрэгтэй: бага зэргийн цайвар баруун тийш харсан',
    '🏃🏽‍♂‍➡' => 'гүйж буй эрэгтэй: дунд зэргийн баруун тийш харсан',
    '🏃🏾‍♂‍➡' => 'гүйж буй эрэгтэй: бордуу баруун тийш харсан',
    '🏃🏿‍♂‍➡' => 'гүйж буй эрэгтэй: бор баруун тийш харсан',
    '🫱🏻‍🫲🏼' => 'гар барилт: цайвар бага зэргийн цайвар',
    '🫱🏻‍🫲🏽' => 'гар барилт: цайвар дунд зэргийн',
    '🫱🏻‍🫲🏾' => 'гар барилт: цайвар бордуу',
    '🫱🏻‍🫲🏿' => 'гар барилт: цайвар бор',
    '🫱🏼‍🫲🏻' => 'гар барилт: бага зэргийн цайвар цайвар',
    '🫱🏼‍🫲🏽' => 'гар барилт: бага зэргийн цайвар дунд зэргийн',
    '🫱🏼‍🫲🏾' => 'гар барилт: бага зэргийн цайвар бордуу',
    '🫱🏼‍🫲🏿' => 'гар барилт: бага зэргийн цайвар бор',
    '🫱🏽‍🫲🏻' => 'гар барилт: дунд зэргийн цайвар',
    '🫱🏽‍🫲🏼' => 'гар барилт: дунд зэргийн бага зэргийн цайвар',
    '🫱🏽‍🫲🏾' => 'гар барилт: дунд зэргийн бордуу',
    '🫱🏽‍🫲🏿' => 'гар барилт: дунд зэргийн бор',
    '🫱🏾‍🫲🏻' => 'гар барилт: бордуу цайвар',
    '🫱🏾‍🫲🏼' => 'гар барилт: бордуу бага зэргийн цайвар',
    '🫱🏾‍🫲🏽' => 'гар барилт: бордуу дунд зэргийн',
    '🫱🏾‍🫲🏿' => 'гар барилт: бордуу бор',
    '🫱🏿‍🫲🏻' => 'гар барилт: бор цайвар',
    '🫱🏿‍🫲🏼' => 'гар барилт: бор бага зэргийн цайвар',
    '🫱🏿‍🫲🏽' => 'гар барилт: бор дунд зэргийн',
    '🫱🏿‍🫲🏾' => 'гар барилт: бор бордуу',
    '🚶‍♀‍➡' => 'алхаж буй эмэгтэй: баруун тийш харсан',
    '🚶‍♂‍➡' => 'алхаж буй эрэгтэй: баруун тийш харсан',
    '🧎‍♀‍➡' => 'сөхөрч буй эмэгтэй: баруун тийш харсан',
    '🧎‍♂‍➡' => 'сөхөрч буй эрэгтэй: баруун тийш харсан',
    '🧑‍🦯‍➡' => 'метал илрүүлэгчтэй хүн: баруун тийш харсан',
    '👨‍🦯‍➡' => 'таягтай хараагүй эрэгтэй: баруун тийш харсан',
    '👩‍🦯‍➡' => 'таягтай хараагүй эмэгтэй: баруун тийш харсан',
    '🧑‍🦼‍➡' => 'автомат удирдлагатай тэргэнцэртэй хүн: баруун тийш харсан',
    '👨‍🦼‍➡' => 'моторт тэргэнцэртэй эрэгтэй: баруун тийш харсан',
    '👩‍🦼‍➡' => 'моторт тэргэнцэртэй эмэгтэй: баруун тийш харсан',
    '🧑‍🦽‍➡' => 'гар ажиллагаатай тэргэнцэртэй хүн: баруун тийш харсан',
    '👨‍🦽‍➡' => 'тэргэнцэртэй эрэгтэй: баруун тийш харсан',
    '👩‍🦽‍➡' => 'тэргэнцэртэй эмэгтэй: баруун тийш харсан',
    '🏃‍♀‍➡' => 'гүйж буй эмэгтэй: баруун тийш харсан',
    '🏃‍♂‍➡' => 'гүйж буй эрэгтэй: баруун тийш харсан',
    '👩‍❤‍👨' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эрэгтэй хүн',
    '👨‍❤‍👨' => 'дундаа зүрхтэй хосууд: эрэгтэй хүн эрэгтэй хүн',
    '👩‍❤‍👩' => 'дундаа зүрхтэй хосууд: эмэгтэй хүн эмэгтэй хүн',
    '👨‍👩‍👦' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүн эрэгтэй хүүхэд',
    '👨‍👩‍👧' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүн эмэгтэй хүүхэд',
    '👨‍👨‍👦' => 'гэр бүл: эрэгтэй хүн эрэгтэй хүн эрэгтэй хүүхэд',
    '👨‍👨‍👧' => 'гэр бүл: эрэгтэй хүн эрэгтэй хүн эмэгтэй хүүхэд',
    '👩‍👩‍👦' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүн эрэгтэй хүүхэд',
    '👩‍👩‍👧' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүн эмэгтэй хүүхэд',
    '👨‍👦‍👦' => 'гэр бүл: эрэгтэй хүн эрэгтэй хүүхэд эрэгтэй хүүхэд',
    '👨‍👧‍👦' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүүхэд эрэгтэй хүүхэд',
    '👨‍👧‍👧' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүүхэд эмэгтэй хүүхэд',
    '👩‍👦‍👦' => 'гэр бүл: эмэгтэй хүн эрэгтэй хүүхэд эрэгтэй хүүхэд',
    '👩‍👧‍👦' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүүхэд эрэгтэй хүүхэд',
    '👩‍👧‍👧' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүүхэд эмэгтэй хүүхэд',
    '🧑‍🤝‍🧑' => 'гар барилцаж буй хүмүүс',
    '🧑‍🧑‍🧒' => 'гэр бүл: насанд хүрсэн хүн, насанд хүрсэн хүн, хүүхэд',
    '🧑‍🧒‍🧒' => 'гэр бүл: насанд хүрсэн хүн, хүүхэд, хүүхэд',
    '🧑🏻‍🦰' => 'том хүн: цайвар улаан үс',
    '🧑🏼‍🦰' => 'том хүн: бага зэргийн цайвар улаан үс',
    '🧑🏽‍🦰' => 'том хүн: дунд зэргийн улаан үс',
    '🧑🏾‍🦰' => 'том хүн: бордуу улаан үс',
    '🧑🏿‍🦰' => 'том хүн: бор улаан үс',
    '🧑🏻‍🦱' => 'том хүн: цайвар буржгар үс',
    '🧑🏼‍🦱' => 'том хүн: бага зэргийн цайвар буржгар үс',
    '🧑🏽‍🦱' => 'том хүн: дунд зэргийн буржгар үс',
    '🧑🏾‍🦱' => 'том хүн: бордуу буржгар үс',
    '🧑🏿‍🦱' => 'том хүн: бор буржгар үс',
    '🧑🏻‍🦳' => 'том хүн: цайвар цагаан үс',
    '🧑🏼‍🦳' => 'том хүн: бага зэргийн цайвар цагаан үс',
    '🧑🏽‍🦳' => 'том хүн: дунд зэргийн цагаан үс',
    '🧑🏾‍🦳' => 'том хүн: бордуу цагаан үс',
    '🧑🏿‍🦳' => 'том хүн: бор цагаан үс',
    '🧑🏻‍🦲' => 'том хүн: цайвар халзан',
    '🧑🏼‍🦲' => 'том хүн: бага зэргийн цайвар халзан',
    '🧑🏽‍🦲' => 'том хүн: дунд зэргийн халзан',
    '🧑🏾‍🦲' => 'том хүн: бордуу халзан',
    '🧑🏿‍🦲' => 'том хүн: бор халзан',
    '🧑🏻‍🩰' => 'баллетчин: цайвар',
    '🧑🏼‍🩰' => 'баллетчин: бага зэргийн цайвар',
    '🧑🏽‍🩰' => 'баллетчин: дунд зэргийн',
    '🧑🏾‍🩰' => 'баллетчин: бордуу',
    '🧑🏿‍🩰' => 'баллетчин: бор',
    '🧔🏻‍♂' => 'эрэгтэй: цайвар сахал',
    '🧔🏼‍♂' => 'эрэгтэй: бага зэргийн цайвар сахал',
    '🧔🏽‍♂' => 'эрэгтэй: дунд зэргийн сахал',
    '🧔🏾‍♂' => 'эрэгтэй: бордуу сахал',
    '🧔🏿‍♂' => 'эрэгтэй: бор сахал',
    '👱🏻‍♂' => 'шар үстэй эрэгтэй: цайвар',
    '👱🏼‍♂' => 'шар үстэй эрэгтэй: бага зэргийн цайвар',
    '👱🏽‍♂' => 'шар үстэй эрэгтэй: дунд зэргийн',
    '👱🏾‍♂' => 'шар үстэй эрэгтэй: бордуу',
    '👱🏿‍♂' => 'шар үстэй эрэгтэй: бор',
    '👨🏻‍🦰' => 'эрэгтэй хүн: цайвар улаан үс',
    '👨🏼‍🦰' => 'эрэгтэй хүн: бага зэргийн цайвар улаан үс',
    '👨🏽‍🦰' => 'эрэгтэй хүн: дунд зэргийн улаан үс',
    '👨🏾‍🦰' => 'эрэгтэй хүн: бордуу улаан үс',
    '👨🏿‍🦰' => 'эрэгтэй хүн: бор улаан үс',
    '👨🏻‍🦱' => 'эрэгтэй хүн: цайвар буржгар үс',
    '👨🏼‍🦱' => 'эрэгтэй хүн: бага зэргийн цайвар буржгар үс',
    '👨🏽‍🦱' => 'эрэгтэй хүн: дунд зэргийн буржгар үс',
    '👨🏾‍🦱' => 'эрэгтэй хүн: бордуу буржгар үс',
    '👨🏿‍🦱' => 'эрэгтэй хүн: бор буржгар үс',
    '👨🏻‍🦳' => 'эрэгтэй хүн: цайвар цагаан үс',
    '👨🏼‍🦳' => 'эрэгтэй хүн: бага зэргийн цайвар цагаан үс',
    '👨🏽‍🦳' => 'эрэгтэй хүн: дунд зэргийн цагаан үс',
    '👨🏾‍🦳' => 'эрэгтэй хүн: бордуу цагаан үс',
    '👨🏿‍🦳' => 'эрэгтэй хүн: бор цагаан үс',
    '👨🏻‍🦲' => 'эрэгтэй хүн: цайвар халзан',
    '👨🏼‍🦲' => 'эрэгтэй хүн: бага зэргийн цайвар халзан',
    '👨🏽‍🦲' => 'эрэгтэй хүн: дунд зэргийн халзан',
    '👨🏾‍🦲' => 'эрэгтэй хүн: бордуу халзан',
    '👨🏿‍🦲' => 'эрэгтэй хүн: бор халзан',
    '🧔🏻‍♀' => 'эмэгтэй: цайвар сахал',
    '🧔🏼‍♀' => 'эмэгтэй: бага зэргийн цайвар сахал',
    '🧔🏽‍♀' => 'эмэгтэй: дунд зэргийн сахал',
    '🧔🏾‍♀' => 'эмэгтэй: бордуу сахал',
    '🧔🏿‍♀' => 'эмэгтэй: бор сахал',
    '👱🏻‍♀' => 'шар үстэй эмэгтэй: цайвар',
    '👱🏼‍♀' => 'шар үстэй эмэгтэй: бага зэргийн цайвар',
    '👱🏽‍♀' => 'шар үстэй эмэгтэй: дунд зэргийн',
    '👱🏾‍♀' => 'шар үстэй эмэгтэй: бордуу',
    '👱🏿‍♀' => 'шар үстэй эмэгтэй: бор',
    '👩🏻‍🦰' => 'эмэгтэй хүн: цайвар улаан үс',
    '👩🏼‍🦰' => 'эмэгтэй хүн: бага зэргийн цайвар улаан үс',
    '👩🏽‍🦰' => 'эмэгтэй хүн: дунд зэргийн улаан үс',
    '👩🏾‍🦰' => 'эмэгтэй хүн: бордуу улаан үс',
    '👩🏿‍🦰' => 'эмэгтэй хүн: бор улаан үс',
    '👩🏻‍🦱' => 'эмэгтэй хүн: цайвар буржгар үс',
    '👩🏼‍🦱' => 'эмэгтэй хүн: бага зэргийн цайвар буржгар үс',
    '👩🏽‍🦱' => 'эмэгтэй хүн: дунд зэргийн буржгар үс',
    '👩🏾‍🦱' => 'эмэгтэй хүн: бордуу буржгар үс',
    '👩🏿‍🦱' => 'эмэгтэй хүн: бор буржгар үс',
    '👩🏻‍🦳' => 'эмэгтэй хүн: цайвар цагаан үс',
    '👩🏼‍🦳' => 'эмэгтэй хүн: бага зэргийн цайвар цагаан үс',
    '👩🏽‍🦳' => 'эмэгтэй хүн: дунд зэргийн цагаан үс',
    '👩🏾‍🦳' => 'эмэгтэй хүн: бордуу цагаан үс',
    '👩🏿‍🦳' => 'эмэгтэй хүн: бор цагаан үс',
    '👩🏻‍🦲' => 'эмэгтэй хүн: цайвар халзан',
    '👩🏼‍🦲' => 'эмэгтэй хүн: бага зэргийн цайвар халзан',
    '👩🏽‍🦲' => 'эмэгтэй хүн: дунд зэргийн халзан',
    '👩🏾‍🦲' => 'эмэгтэй хүн: бордуу халзан',
    '👩🏿‍🦲' => 'эмэгтэй хүн: бор халзан',
    '🙍🏻‍♂' => 'дурамжхан эрэгтэй: цайвар',
    '🙍🏼‍♂' => 'дурамжхан эрэгтэй: бага зэргийн цайвар',
    '🙍🏽‍♂' => 'дурамжхан эрэгтэй: дунд зэргийн',
    '🙍🏾‍♂' => 'дурамжхан эрэгтэй: бордуу',
    '🙍🏿‍♂' => 'дурамжхан эрэгтэй: бор',
    '🙍🏻‍♀' => 'дурамжхан эмэгтэй: цайвар',
    '🙍🏼‍♀' => 'дурамжхан эмэгтэй: бага зэргийн цайвар',
    '🙍🏽‍♀' => 'дурамжхан эмэгтэй: дунд зэргийн',
    '🙍🏾‍♀' => 'дурамжхан эмэгтэй: бордуу',
    '🙍🏿‍♀' => 'дурамжхан эмэгтэй: бор',
    '🙎🏻‍♂' => 'урвагар эрэгтэй: цайвар',
    '🙎🏼‍♂' => 'урвагар эрэгтэй: бага зэргийн цайвар',
    '🙎🏽‍♂' => 'урвагар эрэгтэй: дунд зэргийн',
    '🙎🏾‍♂' => 'урвагар эрэгтэй: бордуу',
    '🙎🏿‍♂' => 'урвагар эрэгтэй: бор',
    '🙎🏻‍♀' => 'урвагар эмэгтэй: цайвар',
    '🙎🏼‍♀' => 'урвагар эмэгтэй: бага зэргийн цайвар',
    '🙎🏽‍♀' => 'урвагар эмэгтэй: дунд зэргийн',
    '🙎🏾‍♀' => 'урвагар эмэгтэй: бордуу',
    '🙎🏿‍♀' => 'урвагар эмэгтэй: бор',
    '🙅🏻‍♂' => 'зүгээр биш гэж харуулж буй эрэгтэй: цайвар',
    '🙅🏼‍♂' => 'зүгээр биш гэж харуулж буй эрэгтэй: бага зэргийн цайвар',
    '🙅🏽‍♂' => 'зүгээр биш гэж харуулж буй эрэгтэй: дунд зэргийн',
    '🙅🏾‍♂' => 'зүгээр биш гэж харуулж буй эрэгтэй: бордуу',
    '🙅🏿‍♂' => 'зүгээр биш гэж харуулж буй эрэгтэй: бор',
    '🙅🏻‍♀' => 'зүгээр биш гэж харуулж буй эмэгтэй: цайвар',
    '🙅🏼‍♀' => 'зүгээр биш гэж харуулж буй эмэгтэй: бага зэргийн цайвар',
    '🙅🏽‍♀' => 'зүгээр биш гэж харуулж буй эмэгтэй: дунд зэргийн',
    '🙅🏾‍♀' => 'зүгээр биш гэж харуулж буй эмэгтэй: бордуу',
    '🙅🏿‍♀' => 'зүгээр биш гэж харуулж буй эмэгтэй: бор',
    '🙆🏻‍♂' => 'зүгээр гэж харуулж буй эрэгтэй: цайвар',
    '🙆🏼‍♂' => 'зүгээр гэж харуулж буй эрэгтэй: бага зэргийн цайвар',
    '🙆🏽‍♂' => 'зүгээр гэж харуулж буй эрэгтэй: дунд зэргийн',
    '🙆🏾‍♂' => 'зүгээр гэж харуулж буй эрэгтэй: бордуу',
    '🙆🏿‍♂' => 'зүгээр гэж харуулж буй эрэгтэй: бор',
    '🙆🏻‍♀' => 'зүгээр гэж харуулж буй эмэгтэй: цайвар',
    '🙆🏼‍♀' => 'зүгээр гэж харуулж буй эмэгтэй: бага зэргийн цайвар',
    '🙆🏽‍♀' => 'зүгээр гэж харуулж буй эмэгтэй: дунд зэргийн',
    '🙆🏾‍♀' => 'зүгээр гэж харуулж буй эмэгтэй: бордуу',
    '🙆🏿‍♀' => 'зүгээр гэж харуулж буй эмэгтэй: бор',
    '💁🏻‍♂' => 'гараа дэлгэж буй эрэгтэй: цайвар',
    '💁🏼‍♂' => 'гараа дэлгэж буй эрэгтэй: бага зэргийн цайвар',
    '💁🏽‍♂' => 'гараа дэлгэж буй эрэгтэй: дунд зэргийн',
    '💁🏾‍♂' => 'гараа дэлгэж буй эрэгтэй: бордуу',
    '💁🏿‍♂' => 'гараа дэлгэж буй эрэгтэй: бор',
    '💁🏻‍♀' => 'гараа дэлгэж буй эмэгтэй: цайвар',
    '💁🏼‍♀' => 'гараа дэлгэж буй эмэгтэй: бага зэргийн цайвар',
    '💁🏽‍♀' => 'гараа дэлгэж буй эмэгтэй: дунд зэргийн',
    '💁🏾‍♀' => 'гараа дэлгэж буй эмэгтэй: бордуу',
    '💁🏿‍♀' => 'гараа дэлгэж буй эмэгтэй: бор',
    '🙋🏻‍♂' => 'гараа өргөж буй эрэгтэй: цайвар',
    '🙋🏼‍♂' => 'гараа өргөж буй эрэгтэй: бага зэргийн цайвар',
    '🙋🏽‍♂' => 'гараа өргөж буй эрэгтэй: дунд зэргийн',
    '🙋🏾‍♂' => 'гараа өргөж буй эрэгтэй: бордуу',
    '🙋🏿‍♂' => 'гараа өргөж буй эрэгтэй: бор',
    '🙋🏻‍♀' => 'гараа өргөж буй эмэгтэй: цайвар',
    '🙋🏼‍♀' => 'гараа өргөж буй эмэгтэй: бага зэргийн цайвар',
    '🙋🏽‍♀' => 'гараа өргөж буй эмэгтэй: дунд зэргийн',
    '🙋🏾‍♀' => 'гараа өргөж буй эмэгтэй: бордуу',
    '🙋🏿‍♀' => 'гараа өргөж буй эмэгтэй: бор',
    '🧏🏻‍♂' => 'дүлий эрэгтэй: цайвар',
    '🧏🏼‍♂' => 'дүлий эрэгтэй: бага зэргийн цайвар',
    '🧏🏽‍♂' => 'дүлий эрэгтэй: дунд зэргийн',
    '🧏🏾‍♂' => 'дүлий эрэгтэй: бордуу',
    '🧏🏿‍♂' => 'дүлий эрэгтэй: бор',
    '🧏🏻‍♀' => 'дүлий эмэгтэй: цайвар',
    '🧏🏼‍♀' => 'дүлий эмэгтэй: бага зэргийн цайвар',
    '🧏🏽‍♀' => 'дүлий эмэгтэй: дунд зэргийн',
    '🧏🏾‍♀' => 'дүлий эмэгтэй: бордуу',
    '🧏🏿‍♀' => 'дүлий эмэгтэй: бор',
    '🙇🏻‍♂' => 'бөхийж буй эрэгтэй: цайвар',
    '🙇🏼‍♂' => 'бөхийж буй эрэгтэй: бага зэргийн цайвар',
    '🙇🏽‍♂' => 'бөхийж буй эрэгтэй: дунд зэргийн',
    '🙇🏾‍♂' => 'бөхийж буй эрэгтэй: бордуу',
    '🙇🏿‍♂' => 'бөхийж буй эрэгтэй: бор',
    '🙇🏻‍♀' => 'бөхийж буй эмэгтэй: цайвар',
    '🙇🏼‍♀' => 'бөхийж буй эмэгтэй: бага зэргийн цайвар',
    '🙇🏽‍♀' => 'бөхийж буй эмэгтэй: дунд зэргийн',
    '🙇🏾‍♀' => 'бөхийж буй эмэгтэй: бордуу',
    '🙇🏿‍♀' => 'бөхийж буй эмэгтэй: бор',
    '🤦🏻‍♂' => 'урам хугарч буй эрэгтэй: цайвар',
    '🤦🏼‍♂' => 'урам хугарч буй эрэгтэй: бага зэргийн цайвар',
    '🤦🏽‍♂' => 'урам хугарч буй эрэгтэй: дунд зэргийн',
    '🤦🏾‍♂' => 'урам хугарч буй эрэгтэй: бордуу',
    '🤦🏿‍♂' => 'урам хугарч буй эрэгтэй: бор',
    '🤦🏻‍♀' => 'урам хугарч буй эмэгтэй: цайвар',
    '🤦🏼‍♀' => 'урам хугарч буй эмэгтэй: бага зэргийн цайвар',
    '🤦🏽‍♀' => 'урам хугарч буй эмэгтэй: дунд зэргийн',
    '🤦🏾‍♀' => 'урам хугарч буй эмэгтэй: бордуу',
    '🤦🏿‍♀' => 'урам хугарч буй эмэгтэй: бор',
    '🤷🏻‍♂' => 'мөрөө хавчиж буй эрэгтэй: цайвар',
    '🤷🏼‍♂' => 'мөрөө хавчиж буй эрэгтэй: бага зэргийн цайвар',
    '🤷🏽‍♂' => 'мөрөө хавчиж буй эрэгтэй: дунд зэргийн',
    '🤷🏾‍♂' => 'мөрөө хавчиж буй эрэгтэй: бордуу',
    '🤷🏿‍♂' => 'мөрөө хавчиж буй эрэгтэй: бор',
    '🤷🏻‍♀' => 'мөрөө хавчиж буй эмэгтэй: цайвар',
    '🤷🏼‍♀' => 'мөрөө хавчиж буй эмэгтэй: бага зэргийн цайвар',
    '🤷🏽‍♀' => 'мөрөө хавчиж буй эмэгтэй: дунд зэргийн',
    '🤷🏾‍♀' => 'мөрөө хавчиж буй эмэгтэй: бордуу',
    '🤷🏿‍♀' => 'мөрөө хавчиж буй эмэгтэй: бор',
    '🧑🏻‍⚕' => 'эрүүл мэндийн ажилтан: цайвар',
    '🧑🏼‍⚕' => 'эрүүл мэндийн ажилтан: бага зэргийн цайвар',
    '🧑🏽‍⚕' => 'эрүүл мэндийн ажилтан: дунд зэргийн',
    '🧑🏾‍⚕' => 'эрүүл мэндийн ажилтан: бордуу',
    '🧑🏿‍⚕' => 'эрүүл мэндийн ажилтан: бор',
    '👨🏻‍⚕' => 'эрэгтэй эрүүл мэндийн ажилтан: цайвар',
    '👨🏼‍⚕' => 'эрэгтэй эрүүл мэндийн ажилтан: бага зэргийн цайвар',
    '👨🏽‍⚕' => 'эрэгтэй эрүүл мэндийн ажилтан: дунд зэргийн',
    '👨🏾‍⚕' => 'эрэгтэй эрүүл мэндийн ажилтан: бордуу',
    '👨🏿‍⚕' => 'эрэгтэй эрүүл мэндийн ажилтан: бор',
    '👩🏻‍⚕' => 'эмэгтэй эрүүл мэндийн ажилтан: цайвар',
    '👩🏼‍⚕' => 'эмэгтэй эрүүл мэндийн ажилтан: бага зэргийн цайвар',
    '👩🏽‍⚕' => 'эмэгтэй эрүүл мэндийн ажилтан: дунд зэргийн',
    '👩🏾‍⚕' => 'эмэгтэй эрүүл мэндийн ажилтан: бордуу',
    '👩🏿‍⚕' => 'эмэгтэй эрүүл мэндийн ажилтан: бор',
    '🧑🏻‍🎓' => 'оюутан: цайвар',
    '🧑🏼‍🎓' => 'оюутан: бага зэргийн цайвар',
    '🧑🏽‍🎓' => 'оюутан: дунд зэргийн',
    '🧑🏾‍🎓' => 'оюутан: бордуу',
    '🧑🏿‍🎓' => 'оюутан: бор',
    '👨🏻‍🎓' => 'эрэгтэй оюутан: цайвар',
    '👨🏼‍🎓' => 'эрэгтэй оюутан: бага зэргийн цайвар',
    '👨🏽‍🎓' => 'эрэгтэй оюутан: дунд зэргийн',
    '👨🏾‍🎓' => 'эрэгтэй оюутан: бордуу',
    '👨🏿‍🎓' => 'эрэгтэй оюутан: бор',
    '👩🏻‍🎓' => 'эмэгтэй оюутан: цайвар',
    '👩🏼‍🎓' => 'эмэгтэй оюутан: бага зэргийн цайвар',
    '👩🏽‍🎓' => 'эмэгтэй оюутан: дунд зэргийн',
    '👩🏾‍🎓' => 'эмэгтэй оюутан: бордуу',
    '👩🏿‍🎓' => 'эмэгтэй оюутан: бор',
    '🧑🏻‍🏫' => 'багш: цайвар',
    '🧑🏼‍🏫' => 'багш: бага зэргийн цайвар',
    '🧑🏽‍🏫' => 'багш: дунд зэргийн',
    '🧑🏾‍🏫' => 'багш: бордуу',
    '🧑🏿‍🏫' => 'багш: бор',
    '👨🏻‍🏫' => 'эрэгтэй багш: цайвар',
    '👨🏼‍🏫' => 'эрэгтэй багш: бага зэргийн цайвар',
    '👨🏽‍🏫' => 'эрэгтэй багш: дунд зэргийн',
    '👨🏾‍🏫' => 'эрэгтэй багш: бордуу',
    '👨🏿‍🏫' => 'эрэгтэй багш: бор',
    '👩🏻‍🏫' => 'эмэгтэй багш: цайвар',
    '👩🏼‍🏫' => 'эмэгтэй багш: бага зэргийн цайвар',
    '👩🏽‍🏫' => 'эмэгтэй багш: дунд зэргийн',
    '👩🏾‍🏫' => 'эмэгтэй багш: бордуу',
    '👩🏿‍🏫' => 'эмэгтэй багш: бор',
    '🧑🏻‍⚖' => 'шүүгч: цайвар',
    '🧑🏼‍⚖' => 'шүүгч: бага зэргийн цайвар',
    '🧑🏽‍⚖' => 'шүүгч: дунд зэргийн',
    '🧑🏾‍⚖' => 'шүүгч: бордуу',
    '🧑🏿‍⚖' => 'шүүгч: бор',
    '👨🏻‍⚖' => 'эрэгтэй шүүгч: цайвар',
    '👨🏼‍⚖' => 'эрэгтэй шүүгч: бага зэргийн цайвар',
    '👨🏽‍⚖' => 'эрэгтэй шүүгч: дунд зэргийн',
    '👨🏾‍⚖' => 'эрэгтэй шүүгч: бордуу',
    '👨🏿‍⚖' => 'эрэгтэй шүүгч: бор',
    '👩🏻‍⚖' => 'эмэгтэй шүүгч: цайвар',
    '👩🏼‍⚖' => 'эмэгтэй шүүгч: бага зэргийн цайвар',
    '👩🏽‍⚖' => 'эмэгтэй шүүгч: дунд зэргийн',
    '👩🏾‍⚖' => 'эмэгтэй шүүгч: бордуу',
    '👩🏿‍⚖' => 'эмэгтэй шүүгч: бор',
    '🧑🏻‍🌾' => 'фермер: цайвар',
    '🧑🏼‍🌾' => 'фермер: бага зэргийн цайвар',
    '🧑🏽‍🌾' => 'фермер: дунд зэргийн',
    '🧑🏾‍🌾' => 'фермер: бордуу',
    '🧑🏿‍🌾' => 'фермер: бор',
    '👨🏻‍🌾' => 'эрэгтэй тариачин: цайвар',
    '👨🏼‍🌾' => 'эрэгтэй тариачин: бага зэргийн цайвар',
    '👨🏽‍🌾' => 'эрэгтэй тариачин: дунд зэргийн',
    '👨🏾‍🌾' => 'эрэгтэй тариачин: бордуу',
    '👨🏿‍🌾' => 'эрэгтэй тариачин: бор',
    '👩🏻‍🌾' => 'эмэгтэй тариачин: цайвар',
    '👩🏼‍🌾' => 'эмэгтэй тариачин: бага зэргийн цайвар',
    '👩🏽‍🌾' => 'эмэгтэй тариачин: дунд зэргийн',
    '👩🏾‍🌾' => 'эмэгтэй тариачин: бордуу',
    '👩🏿‍🌾' => 'эмэгтэй тариачин: бор',
    '🧑🏻‍🍳' => 'хоол хийх: цайвар',
    '🧑🏼‍🍳' => 'хоол хийх: бага зэргийн цайвар',
    '🧑🏽‍🍳' => 'хоол хийх: дунд зэргийн',
    '🧑🏾‍🍳' => 'хоол хийх: бордуу',
    '🧑🏿‍🍳' => 'хоол хийх: бор',
    '👨🏻‍🍳' => 'эрэгтэй тогооч: цайвар',
    '👨🏼‍🍳' => 'эрэгтэй тогооч: бага зэргийн цайвар',
    '👨🏽‍🍳' => 'эрэгтэй тогооч: дунд зэргийн',
    '👨🏾‍🍳' => 'эрэгтэй тогооч: бордуу',
    '👨🏿‍🍳' => 'эрэгтэй тогооч: бор',
    '👩🏻‍🍳' => 'эмэгтэй тогооч: цайвар',
    '👩🏼‍🍳' => 'эмэгтэй тогооч: бага зэргийн цайвар',
    '👩🏽‍🍳' => 'эмэгтэй тогооч: дунд зэргийн',
    '👩🏾‍🍳' => 'эмэгтэй тогооч: бордуу',
    '👩🏿‍🍳' => 'эмэгтэй тогооч: бор',
    '🧑🏻‍🔧' => 'механик: цайвар',
    '🧑🏼‍🔧' => 'механик: бага зэргийн цайвар',
    '🧑🏽‍🔧' => 'механик: дунд зэргийн',
    '🧑🏾‍🔧' => 'механик: бордуу',
    '🧑🏿‍🔧' => 'механик: бор',
    '👨🏻‍🔧' => 'эрэгтэй механикч: цайвар',
    '👨🏼‍🔧' => 'эрэгтэй механикч: бага зэргийн цайвар',
    '👨🏽‍🔧' => 'эрэгтэй механикч: дунд зэргийн',
    '👨🏾‍🔧' => 'эрэгтэй механикч: бордуу',
    '👨🏿‍🔧' => 'эрэгтэй механикч: бор',
    '👩🏻‍🔧' => 'эмэгтэй механикч: цайвар',
    '👩🏼‍🔧' => 'эмэгтэй механикч: бага зэргийн цайвар',
    '👩🏽‍🔧' => 'эмэгтэй механикч: дунд зэргийн',
    '👩🏾‍🔧' => 'эмэгтэй механикч: бордуу',
    '👩🏿‍🔧' => 'эмэгтэй механикч: бор',
    '🧑🏻‍🏭' => 'үйлдвэрийн ажилтан: цайвар',
    '🧑🏼‍🏭' => 'үйлдвэрийн ажилтан: бага зэргийн цайвар',
    '🧑🏽‍🏭' => 'үйлдвэрийн ажилтан: дунд зэргийн',
    '🧑🏾‍🏭' => 'үйлдвэрийн ажилтан: бордуу',
    '🧑🏿‍🏭' => 'үйлдвэрийн ажилтан: бор',
    '👨🏻‍🏭' => 'эрэгтэй үйлдвэрийн ажилтан: цайвар',
    '👨🏼‍🏭' => 'эрэгтэй үйлдвэрийн ажилтан: бага зэргийн цайвар',
    '👨🏽‍🏭' => 'эрэгтэй үйлдвэрийн ажилтан: дунд зэргийн',
    '👨🏾‍🏭' => 'эрэгтэй үйлдвэрийн ажилтан: бордуу',
    '👨🏿‍🏭' => 'эрэгтэй үйлдвэрийн ажилтан: бор',
    '👩🏻‍🏭' => 'эмэгтэй үйлдвэрийн ажилтан: цайвар',
    '👩🏼‍🏭' => 'эмэгтэй үйлдвэрийн ажилтан: бага зэргийн цайвар',
    '👩🏽‍🏭' => 'эмэгтэй үйлдвэрийн ажилтан: дунд зэргийн',
    '👩🏾‍🏭' => 'эмэгтэй үйлдвэрийн ажилтан: бордуу',
    '👩🏿‍🏭' => 'эмэгтэй үйлдвэрийн ажилтан: бор',
    '🧑🏻‍💼' => 'оффисын ажилтан: цайвар',
    '🧑🏼‍💼' => 'оффисын ажилтан: бага зэргийн цайвар',
    '🧑🏽‍💼' => 'оффисын ажилтан: дунд зэргийн',
    '🧑🏾‍💼' => 'оффисын ажилтан: бордуу',
    '🧑🏿‍💼' => 'оффисын ажилтан: бор',
    '👨🏻‍💼' => 'эрэгтэй оффисын ажилтан: цайвар',
    '👨🏼‍💼' => 'эрэгтэй оффисын ажилтан: бага зэргийн цайвар',
    '👨🏽‍💼' => 'эрэгтэй оффисын ажилтан: дунд зэргийн',
    '👨🏾‍💼' => 'эрэгтэй оффисын ажилтан: бордуу',
    '👨🏿‍💼' => 'эрэгтэй оффисын ажилтан: бор',
    '👩🏻‍💼' => 'эмэгтэй оффисын ажилтан: цайвар',
    '👩🏼‍💼' => 'эмэгтэй оффисын ажилтан: бага зэргийн цайвар',
    '👩🏽‍💼' => 'эмэгтэй оффисын ажилтан: дунд зэргийн',
    '👩🏾‍💼' => 'эмэгтэй оффисын ажилтан: бордуу',
    '👩🏿‍💼' => 'эмэгтэй оффисын ажилтан: бор',
    '🧑🏻‍🔬' => 'шинжлэх ухаанч: цайвар',
    '🧑🏼‍🔬' => 'шинжлэх ухаанч: бага зэргийн цайвар',
    '🧑🏽‍🔬' => 'шинжлэх ухаанч: дунд зэргийн',
    '🧑🏾‍🔬' => 'шинжлэх ухаанч: бордуу',
    '🧑🏿‍🔬' => 'шинжлэх ухаанч: бор',
    '👨🏻‍🔬' => 'эрэгтэй эрдэмтэн: цайвар',
    '👨🏼‍🔬' => 'эрэгтэй эрдэмтэн: бага зэргийн цайвар',
    '👨🏽‍🔬' => 'эрэгтэй эрдэмтэн: дунд зэргийн',
    '👨🏾‍🔬' => 'эрэгтэй эрдэмтэн: бордуу',
    '👨🏿‍🔬' => 'эрэгтэй эрдэмтэн: бор',
    '👩🏻‍🔬' => 'эмэгтэй эрдэмтэн: цайвар',
    '👩🏼‍🔬' => 'эмэгтэй эрдэмтэн: бага зэргийн цайвар',
    '👩🏽‍🔬' => 'эмэгтэй эрдэмтэн: дунд зэргийн',
    '👩🏾‍🔬' => 'эмэгтэй эрдэмтэн: бордуу',
    '👩🏿‍🔬' => 'эмэгтэй эрдэмтэн: бор',
    '🧑🏻‍💻' => 'технологич: цайвар',
    '🧑🏼‍💻' => 'технологич: бага зэргийн цайвар',
    '🧑🏽‍💻' => 'технологич: дунд зэргийн',
    '🧑🏾‍💻' => 'технологич: бордуу',
    '🧑🏿‍💻' => 'технологич: бор',
    '👨🏻‍💻' => 'эрэгтэй технологич: цайвар',
    '👨🏼‍💻' => 'эрэгтэй технологич: бага зэргийн цайвар',
    '👨🏽‍💻' => 'эрэгтэй технологич: дунд зэргийн',
    '👨🏾‍💻' => 'эрэгтэй технологич: бордуу',
    '👨🏿‍💻' => 'эрэгтэй технологич: бор',
    '👩🏻‍💻' => 'эмэгтэй технологич: цайвар',
    '👩🏼‍💻' => 'эмэгтэй технологич: бага зэргийн цайвар',
    '👩🏽‍💻' => 'эмэгтэй технологич: дунд зэргийн',
    '👩🏾‍💻' => 'эмэгтэй технологич: бордуу',
    '👩🏿‍💻' => 'эмэгтэй технологич: бор',
    '🧑🏻‍🎤' => 'дуучин: цайвар',
    '🧑🏼‍🎤' => 'дуучин: бага зэргийн цайвар',
    '🧑🏽‍🎤' => 'дуучин: дунд зэргийн',
    '🧑🏾‍🎤' => 'дуучин: бордуу',
    '🧑🏿‍🎤' => 'дуучин: бор',
    '👨🏻‍🎤' => 'эрэгтэй дуучин: цайвар',
    '👨🏼‍🎤' => 'эрэгтэй дуучин: бага зэргийн цайвар',
    '👨🏽‍🎤' => 'эрэгтэй дуучин: дунд зэргийн',
    '👨🏾‍🎤' => 'эрэгтэй дуучин: бордуу',
    '👨🏿‍🎤' => 'эрэгтэй дуучин: бор',
    '👩🏻‍🎤' => 'эмэгтэй дуучин: цайвар',
    '👩🏼‍🎤' => 'эмэгтэй дуучин: бага зэргийн цайвар',
    '👩🏽‍🎤' => 'эмэгтэй дуучин: дунд зэргийн',
    '👩🏾‍🎤' => 'эмэгтэй дуучин: бордуу',
    '👩🏿‍🎤' => 'эмэгтэй дуучин: бор',
    '🧑🏻‍🎨' => 'уран бүтээлч: цайвар',
    '🧑🏼‍🎨' => 'уран бүтээлч: бага зэргийн цайвар',
    '🧑🏽‍🎨' => 'уран бүтээлч: дунд зэргийн',
    '🧑🏾‍🎨' => 'уран бүтээлч: бордуу',
    '🧑🏿‍🎨' => 'уран бүтээлч: бор',
    '👨🏻‍🎨' => 'эрэгтэй зураач: цайвар',
    '👨🏼‍🎨' => 'эрэгтэй зураач: бага зэргийн цайвар',
    '👨🏽‍🎨' => 'эрэгтэй зураач: дунд зэргийн',
    '👨🏾‍🎨' => 'эрэгтэй зураач: бордуу',
    '👨🏿‍🎨' => 'эрэгтэй зураач: бор',
    '👩🏻‍🎨' => 'эмэгтэй зураач: цайвар',
    '👩🏼‍🎨' => 'эмэгтэй зураач: бага зэргийн цайвар',
    '👩🏽‍🎨' => 'эмэгтэй зураач: дунд зэргийн',
    '👩🏾‍🎨' => 'эмэгтэй зураач: бордуу',
    '👩🏿‍🎨' => 'эмэгтэй зураач: бор',
    '🧑🏻‍✈' => 'нисгэгч: цайвар',
    '🧑🏼‍✈' => 'нисгэгч: бага зэргийн цайвар',
    '🧑🏽‍✈' => 'нисгэгч: дунд зэргийн',
    '🧑🏾‍✈' => 'нисгэгч: бордуу',
    '🧑🏿‍✈' => 'нисгэгч: бор',
    '👨🏻‍✈' => 'эрэгтэй нисгэгч: цайвар',
    '👨🏼‍✈' => 'эрэгтэй нисгэгч: бага зэргийн цайвар',
    '👨🏽‍✈' => 'эрэгтэй нисгэгч: дунд зэргийн',
    '👨🏾‍✈' => 'эрэгтэй нисгэгч: бордуу',
    '👨🏿‍✈' => 'эрэгтэй нисгэгч: бор',
    '👩🏻‍✈' => 'эмэгтэй нисгэгч: цайвар',
    '👩🏼‍✈' => 'эмэгтэй нисгэгч: бага зэргийн цайвар',
    '👩🏽‍✈' => 'эмэгтэй нисгэгч: дунд зэргийн',
    '👩🏾‍✈' => 'эмэгтэй нисгэгч: бордуу',
    '👩🏿‍✈' => 'эмэгтэй нисгэгч: бор',
    '🧑🏻‍🚀' => 'сансрын нисэгч: цайвар',
    '🧑🏼‍🚀' => 'сансрын нисэгч: бага зэргийн цайвар',
    '🧑🏽‍🚀' => 'сансрын нисэгч: дунд зэргийн',
    '🧑🏾‍🚀' => 'сансрын нисэгч: бордуу',
    '🧑🏿‍🚀' => 'сансрын нисэгч: бор',
    '👨🏻‍🚀' => 'эрэгтэй сансрын нисгэгч: цайвар',
    '👨🏼‍🚀' => 'эрэгтэй сансрын нисгэгч: бага зэргийн цайвар',
    '👨🏽‍🚀' => 'эрэгтэй сансрын нисгэгч: дунд зэргийн',
    '👨🏾‍🚀' => 'эрэгтэй сансрын нисгэгч: бордуу',
    '👨🏿‍🚀' => 'эрэгтэй сансрын нисгэгч: бор',
    '👩🏻‍🚀' => 'эмэгтэй сансрын нисгэгч: цайвар',
    '👩🏼‍🚀' => 'эмэгтэй сансрын нисгэгч: бага зэргийн цайвар',
    '👩🏽‍🚀' => 'эмэгтэй сансрын нисгэгч: дунд зэргийн',
    '👩🏾‍🚀' => 'эмэгтэй сансрын нисгэгч: бордуу',
    '👩🏿‍🚀' => 'эмэгтэй сансрын нисгэгч: бор',
    '🧑🏻‍🚒' => 'гал сөнөөгч: цайвар',
    '🧑🏼‍🚒' => 'гал сөнөөгч: бага зэргийн цайвар',
    '🧑🏽‍🚒' => 'гал сөнөөгч: дунд зэргийн',
    '🧑🏾‍🚒' => 'гал сөнөөгч: бордуу',
    '🧑🏿‍🚒' => 'гал сөнөөгч: бор',
    '👨🏻‍🚒' => 'эрэгтэй гал сөнөөгч: цайвар',
    '👨🏼‍🚒' => 'эрэгтэй гал сөнөөгч: бага зэргийн цайвар',
    '👨🏽‍🚒' => 'эрэгтэй гал сөнөөгч: дунд зэргийн',
    '👨🏾‍🚒' => 'эрэгтэй гал сөнөөгч: бордуу',
    '👨🏿‍🚒' => 'эрэгтэй гал сөнөөгч: бор',
    '👩🏻‍🚒' => 'эмэгтэй гал сөнөөгч: цайвар',
    '👩🏼‍🚒' => 'эмэгтэй гал сөнөөгч: бага зэргийн цайвар',
    '👩🏽‍🚒' => 'эмэгтэй гал сөнөөгч: дунд зэргийн',
    '👩🏾‍🚒' => 'эмэгтэй гал сөнөөгч: бордуу',
    '👩🏿‍🚒' => 'эмэгтэй гал сөнөөгч: бор',
    '👮🏻‍♂' => 'эрэгтэй цагдаа: цайвар',
    '👮🏼‍♂' => 'эрэгтэй цагдаа: бага зэргийн цайвар',
    '👮🏽‍♂' => 'эрэгтэй цагдаа: дунд зэргийн',
    '👮🏾‍♂' => 'эрэгтэй цагдаа: бордуу',
    '👮🏿‍♂' => 'эрэгтэй цагдаа: бор',
    '👮🏻‍♀' => 'эмэгтэй цагдаа: цайвар',
    '👮🏼‍♀' => 'эмэгтэй цагдаа: бага зэргийн цайвар',
    '👮🏽‍♀' => 'эмэгтэй цагдаа: дунд зэргийн',
    '👮🏾‍♀' => 'эмэгтэй цагдаа: бордуу',
    '👮🏿‍♀' => 'эмэгтэй цагдаа: бор',
    '🕵🏻‍♂' => 'эрэгтэй мөрдөгч: цайвар',
    '🕵🏼‍♂' => 'эрэгтэй мөрдөгч: бага зэргийн цайвар',
    '🕵🏽‍♂' => 'эрэгтэй мөрдөгч: дунд зэргийн',
    '🕵🏾‍♂' => 'эрэгтэй мөрдөгч: бордуу',
    '🕵🏿‍♂' => 'эрэгтэй мөрдөгч: бор',
    '🕵🏻‍♀' => 'эмэгтэй мөрдөгч: цайвар',
    '🕵🏼‍♀' => 'эмэгтэй мөрдөгч: бага зэргийн цайвар',
    '🕵🏽‍♀' => 'эмэгтэй мөрдөгч: дунд зэргийн',
    '🕵🏾‍♀' => 'эмэгтэй мөрдөгч: бордуу',
    '🕵🏿‍♀' => 'эмэгтэй мөрдөгч: бор',
    '💂🏻‍♂' => 'эрэгтэй харуул: цайвар',
    '💂🏼‍♂' => 'эрэгтэй харуул: бага зэргийн цайвар',
    '💂🏽‍♂' => 'эрэгтэй харуул: дунд зэргийн',
    '💂🏾‍♂' => 'эрэгтэй харуул: бордуу',
    '💂🏿‍♂' => 'эрэгтэй харуул: бор',
    '💂🏻‍♀' => 'эмэгтэй харуул: цайвар',
    '💂🏼‍♀' => 'эмэгтэй харуул: бага зэргийн цайвар',
    '💂🏽‍♀' => 'эмэгтэй харуул: дунд зэргийн',
    '💂🏾‍♀' => 'эмэгтэй харуул: бордуу',
    '💂🏿‍♀' => 'эмэгтэй харуул: бор',
    '👷🏻‍♂' => 'эрэгтэй барилгын ажилтан: цайвар',
    '👷🏼‍♂' => 'эрэгтэй барилгын ажилтан: бага зэргийн цайвар',
    '👷🏽‍♂' => 'эрэгтэй барилгын ажилтан: дунд зэргийн',
    '👷🏾‍♂' => 'эрэгтэй барилгын ажилтан: бордуу',
    '👷🏿‍♂' => 'эрэгтэй барилгын ажилтан: бор',
    '👷🏻‍♀' => 'эмэгтэй барилгын ажилтан: цайвар',
    '👷🏼‍♀' => 'эмэгтэй барилгын ажилтан: бага зэргийн цайвар',
    '👷🏽‍♀' => 'эмэгтэй барилгын ажилтан: дунд зэргийн',
    '👷🏾‍♀' => 'эмэгтэй барилгын ажилтан: бордуу',
    '👷🏿‍♀' => 'эмэгтэй барилгын ажилтан: бор',
    '👳🏻‍♂' => 'толгойн ороолттой эрэгтэй хүн: цайвар',
    '👳🏼‍♂' => 'толгойн ороолттой эрэгтэй хүн: бага зэргийн цайвар',
    '👳🏽‍♂' => 'толгойн ороолттой эрэгтэй хүн: дунд зэргийн',
    '👳🏾‍♂' => 'толгойн ороолттой эрэгтэй хүн: бордуу',
    '👳🏿‍♂' => 'толгойн ороолттой эрэгтэй хүн: бор',
    '👳🏻‍♀' => 'толгойн ороолттой эмэгтэй хүн: цайвар',
    '👳🏼‍♀' => 'толгойн ороолттой эмэгтэй хүн: бага зэргийн цайвар',
    '👳🏽‍♀' => 'толгойн ороолттой эмэгтэй хүн: дунд зэргийн',
    '👳🏾‍♀' => 'толгойн ороолттой эмэгтэй хүн: бордуу',
    '👳🏿‍♀' => 'толгойн ороолттой эмэгтэй хүн: бор',
    '🤵🏻‍♂' => 'хослолтой эрэгтэй: цайвар',
    '🤵🏼‍♂' => 'хослолтой эрэгтэй: бага зэргийн цайвар',
    '🤵🏽‍♂' => 'хослолтой эрэгтэй: дунд зэргийн',
    '🤵🏾‍♂' => 'хослолтой эрэгтэй: бордуу',
    '🤵🏿‍♂' => 'хослолтой эрэгтэй: бор',
    '🤵🏻‍♀' => 'хослолтой эмэгтэй: цайвар',
    '🤵🏼‍♀' => 'хослолтой эмэгтэй: бага зэргийн цайвар',
    '🤵🏽‍♀' => 'хослолтой эмэгтэй: дунд зэргийн',
    '🤵🏾‍♀' => 'хослолтой эмэгтэй: бордуу',
    '🤵🏿‍♀' => 'хослолтой эмэгтэй: бор',
    '👰🏻‍♂' => 'гивлүүртэй эрэгтэй: цайвар',
    '👰🏼‍♂' => 'гивлүүртэй эрэгтэй: бага зэргийн цайвар',
    '👰🏽‍♂' => 'гивлүүртэй эрэгтэй: дунд зэргийн',
    '👰🏾‍♂' => 'гивлүүртэй эрэгтэй: бордуу',
    '👰🏿‍♂' => 'гивлүүртэй эрэгтэй: бор',
    '👰🏻‍♀' => 'гивлүүртэй эмэгтэй: цайвар',
    '👰🏼‍♀' => 'гивлүүртэй эмэгтэй: бага зэргийн цайвар',
    '👰🏽‍♀' => 'гивлүүртэй эмэгтэй: дунд зэргийн',
    '👰🏾‍♀' => 'гивлүүртэй эмэгтэй: бордуу',
    '👰🏿‍♀' => 'гивлүүртэй эмэгтэй: бор',
    '👩🏻‍🍼' => 'нялх хүүхэд хооллож байгаа эмэгтэй: цайвар',
    '👩🏼‍🍼' => 'нялх хүүхэд хооллож байгаа эмэгтэй: бага зэргийн цайвар',
    '👩🏽‍🍼' => 'нялх хүүхэд хооллож байгаа эмэгтэй: дунд зэргийн',
    '👩🏾‍🍼' => 'нялх хүүхэд хооллож байгаа эмэгтэй: бордуу',
    '👩🏿‍🍼' => 'нялх хүүхэд хооллож байгаа эмэгтэй: бор',
    '👨🏻‍🍼' => 'нялх хүүхэд хооллож байгаа эрэгтэй: цайвар',
    '👨🏼‍🍼' => 'нялх хүүхэд хооллож байгаа эрэгтэй: бага зэргийн цайвар',
    '👨🏽‍🍼' => 'нялх хүүхэд хооллож байгаа эрэгтэй: дунд зэргийн',
    '👨🏾‍🍼' => 'нялх хүүхэд хооллож байгаа эрэгтэй: бордуу',
    '👨🏿‍🍼' => 'нялх хүүхэд хооллож байгаа эрэгтэй: бор',
    '🧑🏻‍🍼' => 'нялх хүүхэд хооллож байгаа хүн: цайвар',
    '🧑🏼‍🍼' => 'нялх хүүхэд хооллож байгаа хүн: бага зэргийн цайвар',
    '🧑🏽‍🍼' => 'нялх хүүхэд хооллож байгаа хүн: дунд зэргийн',
    '🧑🏾‍🍼' => 'нялх хүүхэд хооллож байгаа хүн: бордуу',
    '🧑🏿‍🍼' => 'нялх хүүхэд хооллож байгаа хүн: бор',
    '🧑🏻‍🎄' => 'хатагтай клаус: цайвар',
    '🧑🏼‍🎄' => 'хатагтай клаус: бага зэргийн цайвар',
    '🧑🏽‍🎄' => 'хатагтай клаус: дунд зэргийн',
    '🧑🏾‍🎄' => 'хатагтай клаус: бордуу',
    '🧑🏿‍🎄' => 'хатагтай клаус: бор',
    '🦸🏻‍♂' => 'эрэгтэй супер баатар: цайвар',
    '🦸🏼‍♂' => 'эрэгтэй супер баатар: бага зэргийн цайвар',
    '🦸🏽‍♂' => 'эрэгтэй супер баатар: дунд зэргийн',
    '🦸🏾‍♂' => 'эрэгтэй супер баатар: бордуу',
    '🦸🏿‍♂' => 'эрэгтэй супер баатар: бор',
    '🦸🏻‍♀' => 'эмэгтэй супер баатар: цайвар',
    '🦸🏼‍♀' => 'эмэгтэй супер баатар: бага зэргийн цайвар',
    '🦸🏽‍♀' => 'эмэгтэй супер баатар: дунд зэргийн',
    '🦸🏾‍♀' => 'эмэгтэй супер баатар: бордуу',
    '🦸🏿‍♀' => 'эмэгтэй супер баатар: бор',
    '🦹🏻‍♂' => 'ид шидийн хүчтэй эрэгтэй эсрэг дүр: цайвар',
    '🦹🏼‍♂' => 'ид шидийн хүчтэй эрэгтэй эсрэг дүр: бага зэргийн цайвар',
    '🦹🏽‍♂' => 'ид шидийн хүчтэй эрэгтэй эсрэг дүр: дунд зэргийн',
    '🦹🏾‍♂' => 'ид шидийн хүчтэй эрэгтэй эсрэг дүр: бордуу',
    '🦹🏿‍♂' => 'ид шидийн хүчтэй эрэгтэй эсрэг дүр: бор',
    '🦹🏻‍♀' => 'ид шидийн хүчтэй эмэгтэй эсрэг дүр: цайвар',
    '🦹🏼‍♀' => 'ид шидийн хүчтэй эмэгтэй эсрэг дүр: бага зэргийн цайвар',
    '🦹🏽‍♀' => 'ид шидийн хүчтэй эмэгтэй эсрэг дүр: дунд зэргийн',
    '🦹🏾‍♀' => 'ид шидийн хүчтэй эмэгтэй эсрэг дүр: бордуу',
    '🦹🏿‍♀' => 'ид шидийн хүчтэй эмэгтэй эсрэг дүр: бор',
    '🧙🏻‍♂' => 'эрэгтэй шидтэн: цайвар',
    '🧙🏼‍♂' => 'эрэгтэй шидтэн: бага зэргийн цайвар',
    '🧙🏽‍♂' => 'эрэгтэй шидтэн: дунд зэргийн',
    '🧙🏾‍♂' => 'эрэгтэй шидтэн: бордуу',
    '🧙🏿‍♂' => 'эрэгтэй шидтэн: бор',
    '🧙🏻‍♀' => 'эмэгтэй шидтэн: цайвар',
    '🧙🏼‍♀' => 'эмэгтэй шидтэн: бага зэргийн цайвар',
    '🧙🏽‍♀' => 'эмэгтэй шидтэн: дунд зэргийн',
    '🧙🏾‍♀' => 'эмэгтэй шидтэн: бордуу',
    '🧙🏿‍♀' => 'эмэгтэй шидтэн: бор',
    '🧚🏻‍♂' => 'эрэгтэй илбэчин: цайвар',
    '🧚🏼‍♂' => 'эрэгтэй илбэчин: бага зэргийн цайвар',
    '🧚🏽‍♂' => 'эрэгтэй илбэчин: дунд зэргийн',
    '🧚🏾‍♂' => 'эрэгтэй илбэчин: бордуу',
    '🧚🏿‍♂' => 'эрэгтэй илбэчин: бор',
    '🧚🏻‍♀' => 'эмэгтэй илбэчин: цайвар',
    '🧚🏼‍♀' => 'эмэгтэй илбэчин: бага зэргийн цайвар',
    '🧚🏽‍♀' => 'эмэгтэй илбэчин: дунд зэргийн',
    '🧚🏾‍♀' => 'эмэгтэй илбэчин: бордуу',
    '🧚🏿‍♀' => 'эмэгтэй илбэчин: бор',
    '🧛🏻‍♂' => 'эрэгтэй цус сорогч: цайвар',
    '🧛🏼‍♂' => 'эрэгтэй цус сорогч: бага зэргийн цайвар',
    '🧛🏽‍♂' => 'эрэгтэй цус сорогч: дунд зэргийн',
    '🧛🏾‍♂' => 'эрэгтэй цус сорогч: бордуу',
    '🧛🏿‍♂' => 'эрэгтэй цус сорогч: бор',
    '🧛🏻‍♀' => 'эмэгтэй цус сорогч: цайвар',
    '🧛🏼‍♀' => 'эмэгтэй цус сорогч: бага зэргийн цайвар',
    '🧛🏽‍♀' => 'эмэгтэй цус сорогч: дунд зэргийн',
    '🧛🏾‍♀' => 'эмэгтэй цус сорогч: бордуу',
    '🧛🏿‍♀' => 'эмэгтэй цус сорогч: бор',
    '🧜🏻‍♂' => 'лусын хаан: цайвар',
    '🧜🏼‍♂' => 'лусын хаан: бага зэргийн цайвар',
    '🧜🏽‍♂' => 'лусын хаан: дунд зэргийн',
    '🧜🏾‍♂' => 'лусын хаан: бордуу',
    '🧜🏿‍♂' => 'лусын хаан: бор',
    '🧜🏻‍♀' => 'лусын дагина: цайвар',
    '🧜🏼‍♀' => 'лусын дагина: бага зэргийн цайвар',
    '🧜🏽‍♀' => 'лусын дагина: дунд зэргийн',
    '🧜🏾‍♀' => 'лусын дагина: бордуу',
    '🧜🏿‍♀' => 'лусын дагина: бор',
    '🧝🏻‍♂' => 'эрэгтэй одой: цайвар',
    '🧝🏼‍♂' => 'эрэгтэй одой: бага зэргийн цайвар',
    '🧝🏽‍♂' => 'эрэгтэй одой: дунд зэргийн',
    '🧝🏾‍♂' => 'эрэгтэй одой: бордуу',
    '🧝🏿‍♂' => 'эрэгтэй одой: бор',
    '🧝🏻‍♀' => 'эмэгтэй одой: цайвар',
    '🧝🏼‍♀' => 'эмэгтэй одой: бага зэргийн цайвар',
    '🧝🏽‍♀' => 'эмэгтэй одой: дунд зэргийн',
    '🧝🏾‍♀' => 'эмэгтэй одой: бордуу',
    '🧝🏿‍♀' => 'эмэгтэй одой: бор',
    '💆🏻‍♂' => 'нүүрэндээ массаж хийлгэж буй эрэгтэй: цайвар',
    '💆🏼‍♂' => 'нүүрэндээ массаж хийлгэж буй эрэгтэй: бага зэргийн цайвар',
    '💆🏽‍♂' => 'нүүрэндээ массаж хийлгэж буй эрэгтэй: дунд зэргийн',
    '💆🏾‍♂' => 'нүүрэндээ массаж хийлгэж буй эрэгтэй: бордуу',
    '💆🏿‍♂' => 'нүүрэндээ массаж хийлгэж буй эрэгтэй: бор',
    '💆🏻‍♀' => 'нүүрэндээ массаж хийлгэж буй эмэгтэй: цайвар',
    '💆🏼‍♀' => 'нүүрэндээ массаж хийлгэж буй эмэгтэй: бага зэргийн цайвар',
    '💆🏽‍♀' => 'нүүрэндээ массаж хийлгэж буй эмэгтэй: дунд зэргийн',
    '💆🏾‍♀' => 'нүүрэндээ массаж хийлгэж буй эмэгтэй: бордуу',
    '💆🏿‍♀' => 'нүүрэндээ массаж хийлгэж буй эмэгтэй: бор',
    '💇🏻‍♂' => 'үсээ тайруулж буй эрэгтэй: цайвар',
    '💇🏼‍♂' => 'үсээ тайруулж буй эрэгтэй: бага зэргийн цайвар',
    '💇🏽‍♂' => 'үсээ тайруулж буй эрэгтэй: дунд зэргийн',
    '💇🏾‍♂' => 'үсээ тайруулж буй эрэгтэй: бордуу',
    '💇🏿‍♂' => 'үсээ тайруулж буй эрэгтэй: бор',
    '💇🏻‍♀' => 'үсээ тайруулж буй эмэгтэй: цайвар',
    '💇🏼‍♀' => 'үсээ тайруулж буй эмэгтэй: бага зэргийн цайвар',
    '💇🏽‍♀' => 'үсээ тайруулж буй эмэгтэй: дунд зэргийн',
    '💇🏾‍♀' => 'үсээ тайруулж буй эмэгтэй: бордуу',
    '💇🏿‍♀' => 'үсээ тайруулж буй эмэгтэй: бор',
    '🚶🏻‍♂' => 'алхаж буй эрэгтэй: цайвар',
    '🚶🏼‍♂' => 'алхаж буй эрэгтэй: бага зэргийн цайвар',
    '🚶🏽‍♂' => 'алхаж буй эрэгтэй: дунд зэргийн',
    '🚶🏾‍♂' => 'алхаж буй эрэгтэй: бордуу',
    '🚶🏿‍♂' => 'алхаж буй эрэгтэй: бор',
    '🚶🏻‍♀' => 'алхаж буй эмэгтэй: цайвар',
    '🚶🏼‍♀' => 'алхаж буй эмэгтэй: бага зэргийн цайвар',
    '🚶🏽‍♀' => 'алхаж буй эмэгтэй: дунд зэргийн',
    '🚶🏾‍♀' => 'алхаж буй эмэгтэй: бордуу',
    '🚶🏿‍♀' => 'алхаж буй эмэгтэй: бор',
    '🚶🏻‍➡' => 'алхаж буй хүн: цайвар баруун тийш харсан',
    '🚶🏼‍➡' => 'алхаж буй хүн: бага зэргийн цайвар баруун тийш харсан',
    '🚶🏽‍➡' => 'алхаж буй хүн: дунд зэргийн баруун тийш харсан',
    '🚶🏾‍➡' => 'алхаж буй хүн: бордуу баруун тийш харсан',
    '🚶🏿‍➡' => 'алхаж буй хүн: бор баруун тийш харсан',
    '🧍🏻‍♂' => 'зогсож буй эрэгтэй: цайвар',
    '🧍🏼‍♂' => 'зогсож буй эрэгтэй: бага зэргийн цайвар',
    '🧍🏽‍♂' => 'зогсож буй эрэгтэй: дунд зэргийн',
    '🧍🏾‍♂' => 'зогсож буй эрэгтэй: бордуу',
    '🧍🏿‍♂' => 'зогсож буй эрэгтэй: бор',
    '🧍🏻‍♀' => 'зогсож буй эмэгтэй: цайвар',
    '🧍🏼‍♀' => 'зогсож буй эмэгтэй: бага зэргийн цайвар',
    '🧍🏽‍♀' => 'зогсож буй эмэгтэй: дунд зэргийн',
    '🧍🏾‍♀' => 'зогсож буй эмэгтэй: бордуу',
    '🧍🏿‍♀' => 'зогсож буй эмэгтэй: бор',
    '🧎🏻‍♂' => 'сөхөрч буй эрэгтэй: цайвар',
    '🧎🏼‍♂' => 'сөхөрч буй эрэгтэй: бага зэргийн цайвар',
    '🧎🏽‍♂' => 'сөхөрч буй эрэгтэй: дунд зэргийн',
    '🧎🏾‍♂' => 'сөхөрч буй эрэгтэй: бордуу',
    '🧎🏿‍♂' => 'сөхөрч буй эрэгтэй: бор',
    '🧎🏻‍♀' => 'сөхөрч буй эмэгтэй: цайвар',
    '🧎🏼‍♀' => 'сөхөрч буй эмэгтэй: бага зэргийн цайвар',
    '🧎🏽‍♀' => 'сөхөрч буй эмэгтэй: дунд зэргийн',
    '🧎🏾‍♀' => 'сөхөрч буй эмэгтэй: бордуу',
    '🧎🏿‍♀' => 'сөхөрч буй эмэгтэй: бор',
    '🧎🏻‍➡' => 'сөхөрч суусан хүн: цайвар баруун тийш харсан',
    '🧎🏼‍➡' => 'сөхөрч суусан хүн: бага зэргийн цайвар баруун тийш харсан',
    '🧎🏽‍➡' => 'сөхөрч суусан хүн: дунд зэргийн баруун тийш харсан',
    '🧎🏾‍➡' => 'сөхөрч суусан хүн: бордуу баруун тийш харсан',
    '🧎🏿‍➡' => 'сөхөрч суусан хүн: бор баруун тийш харсан',
    '🧑🏻‍🦯' => 'метал илрүүлэгчтэй хүн: цайвар',
    '🧑🏼‍🦯' => 'метал илрүүлэгчтэй хүн: бага зэргийн цайвар',
    '🧑🏽‍🦯' => 'метал илрүүлэгчтэй хүн: дунд зэргийн',
    '🧑🏾‍🦯' => 'метал илрүүлэгчтэй хүн: бордуу',
    '🧑🏿‍🦯' => 'метал илрүүлэгчтэй хүн: бор',
    '👨🏻‍🦯' => 'таягтай хараагүй эрэгтэй: цайвар',
    '👨🏼‍🦯' => 'таягтай хараагүй эрэгтэй: бага зэргийн цайвар',
    '👨🏽‍🦯' => 'таягтай хараагүй эрэгтэй: дунд зэргийн',
    '👨🏾‍🦯' => 'таягтай хараагүй эрэгтэй: бордуу',
    '👨🏿‍🦯' => 'таягтай хараагүй эрэгтэй: бор',
    '👩🏻‍🦯' => 'таягтай хараагүй эмэгтэй: цайвар',
    '👩🏼‍🦯' => 'таягтай хараагүй эмэгтэй: бага зэргийн цайвар',
    '👩🏽‍🦯' => 'таягтай хараагүй эмэгтэй: дунд зэргийн',
    '👩🏾‍🦯' => 'таягтай хараагүй эмэгтэй: бордуу',
    '👩🏿‍🦯' => 'таягтай хараагүй эмэгтэй: бор',
    '🧑🏻‍🦼' => 'автомат удирдлагатай тэргэнцэртэй хүн: цайвар',
    '🧑🏼‍🦼' => 'автомат удирдлагатай тэргэнцэртэй хүн: бага зэргийн цайвар',
    '🧑🏽‍🦼' => 'автомат удирдлагатай тэргэнцэртэй хүн: дунд зэргийн',
    '🧑🏾‍🦼' => 'автомат удирдлагатай тэргэнцэртэй хүн: бордуу',
    '🧑🏿‍🦼' => 'автомат удирдлагатай тэргэнцэртэй хүн: бор',
    '👨🏻‍🦼' => 'моторт тэргэнцэртэй эрэгтэй: цайвар',
    '👨🏼‍🦼' => 'моторт тэргэнцэртэй эрэгтэй: бага зэргийн цайвар',
    '👨🏽‍🦼' => 'моторт тэргэнцэртэй эрэгтэй: дунд зэргийн',
    '👨🏾‍🦼' => 'моторт тэргэнцэртэй эрэгтэй: бордуу',
    '👨🏿‍🦼' => 'моторт тэргэнцэртэй эрэгтэй: бор',
    '👩🏻‍🦼' => 'моторт тэргэнцэртэй эмэгтэй: цайвар',
    '👩🏼‍🦼' => 'моторт тэргэнцэртэй эмэгтэй: бага зэргийн цайвар',
    '👩🏽‍🦼' => 'моторт тэргэнцэртэй эмэгтэй: дунд зэргийн',
    '👩🏾‍🦼' => 'моторт тэргэнцэртэй эмэгтэй: бордуу',
    '👩🏿‍🦼' => 'моторт тэргэнцэртэй эмэгтэй: бор',
    '🧑🏻‍🦽' => 'гар ажиллагаатай тэргэнцэртэй хүн: цайвар',
    '🧑🏼‍🦽' => 'гар ажиллагаатай тэргэнцэртэй хүн: бага зэргийн цайвар',
    '🧑🏽‍🦽' => 'гар ажиллагаатай тэргэнцэртэй хүн: дунд зэргийн',
    '🧑🏾‍🦽' => 'гар ажиллагаатай тэргэнцэртэй хүн: бордуу',
    '🧑🏿‍🦽' => 'гар ажиллагаатай тэргэнцэртэй хүн: бор',
    '👨🏻‍🦽' => 'тэргэнцэртэй эрэгтэй: цайвар',
    '👨🏼‍🦽' => 'тэргэнцэртэй эрэгтэй: бага зэргийн цайвар',
    '👨🏽‍🦽' => 'тэргэнцэртэй эрэгтэй: дунд зэргийн',
    '👨🏾‍🦽' => 'тэргэнцэртэй эрэгтэй: бордуу',
    '👨🏿‍🦽' => 'тэргэнцэртэй эрэгтэй: бор',
    '👩🏻‍🦽' => 'тэргэнцэртэй эмэгтэй: цайвар',
    '👩🏼‍🦽' => 'тэргэнцэртэй эмэгтэй: бага зэргийн цайвар',
    '👩🏽‍🦽' => 'тэргэнцэртэй эмэгтэй: дунд зэргийн',
    '👩🏾‍🦽' => 'тэргэнцэртэй эмэгтэй: бордуу',
    '👩🏿‍🦽' => 'тэргэнцэртэй эмэгтэй: бор',
    '🏃🏻‍♂' => 'гүйж буй эрэгтэй: цайвар',
    '🏃🏼‍♂' => 'гүйж буй эрэгтэй: бага зэргийн цайвар',
    '🏃🏽‍♂' => 'гүйж буй эрэгтэй: дунд зэргийн',
    '🏃🏾‍♂' => 'гүйж буй эрэгтэй: бордуу',
    '🏃🏿‍♂' => 'гүйж буй эрэгтэй: бор',
    '🏃🏻‍♀' => 'гүйж буй эмэгтэй: цайвар',
    '🏃🏼‍♀' => 'гүйж буй эмэгтэй: бага зэргийн цайвар',
    '🏃🏽‍♀' => 'гүйж буй эмэгтэй: дунд зэргийн',
    '🏃🏾‍♀' => 'гүйж буй эмэгтэй: бордуу',
    '🏃🏿‍♀' => 'гүйж буй эмэгтэй: бор',
    '🏃🏻‍➡' => 'гүйж байгаа хүн: цайвар баруун тийш харсан',
    '🏃🏼‍➡' => 'гүйж байгаа хүн: бага зэргийн цайвар баруун тийш харсан',
    '🏃🏽‍➡' => 'гүйж байгаа хүн: дунд зэргийн баруун тийш харсан',
    '🏃🏾‍➡' => 'гүйж байгаа хүн: бордуу баруун тийш харсан',
    '🏃🏿‍➡' => 'гүйж байгаа хүн: бор баруун тийш харсан',
    '👯🏻‍♀' => 'цэнгэж буй эмэгтэйчүүд: цайвар',
    '👯🏼‍♀' => 'цэнгэж буй эмэгтэйчүүд: бага зэргийн цайвар',
    '👯🏽‍♀' => 'цэнгэж буй эмэгтэйчүүд: дунд зэргийн',
    '👯🏾‍♀' => 'цэнгэж буй эмэгтэйчүүд: бордуу',
    '👯🏿‍♀' => 'цэнгэж буй эмэгтэйчүүд: бор',
    '👯🏻‍♂' => 'цэнгэж буй эрэгтэйчүүд: цайвар',
    '👯🏼‍♂' => 'цэнгэж буй эрэгтэйчүүд: бага зэргийн цайвар',
    '👯🏽‍♂' => 'цэнгэж буй эрэгтэйчүүд: дунд зэргийн',
    '👯🏾‍♂' => 'цэнгэж буй эрэгтэйчүүд: бордуу',
    '👯🏿‍♂' => 'цэнгэж буй эрэгтэйчүүд: бор',
    '🧖🏻‍♂' => 'уур савссан өрөөнд байгаа эрэгтэй: цайвар',
    '🧖🏼‍♂' => 'уур савссан өрөөнд байгаа эрэгтэй: бага зэргийн цайвар',
    '🧖🏽‍♂' => 'уур савссан өрөөнд байгаа эрэгтэй: дунд зэргийн',
    '🧖🏾‍♂' => 'уур савссан өрөөнд байгаа эрэгтэй: бордуу',
    '🧖🏿‍♂' => 'уур савссан өрөөнд байгаа эрэгтэй: бор',
    '🧖🏻‍♀' => 'уур савссан өрөөнд байгаа эмэгтэй: цайвар',
    '🧖🏼‍♀' => 'уур савссан өрөөнд байгаа эмэгтэй: бага зэргийн цайвар',
    '🧖🏽‍♀' => 'уур савссан өрөөнд байгаа эмэгтэй: дунд зэргийн',
    '🧖🏾‍♀' => 'уур савссан өрөөнд байгаа эмэгтэй: бордуу',
    '🧖🏿‍♀' => 'уур савссан өрөөнд байгаа эмэгтэй: бор',
    '🧗🏻‍♂' => 'авирагч эрэгтэй: цайвар',
    '🧗🏼‍♂' => 'авирагч эрэгтэй: бага зэргийн цайвар',
    '🧗🏽‍♂' => 'авирагч эрэгтэй: дунд зэргийн',
    '🧗🏾‍♂' => 'авирагч эрэгтэй: бордуу',
    '🧗🏿‍♂' => 'авирагч эрэгтэй: бор',
    '🧗🏻‍♀' => 'эмэгтэй авирагч: цайвар',
    '🧗🏼‍♀' => 'эмэгтэй авирагч: бага зэргийн цайвар',
    '🧗🏽‍♀' => 'эмэгтэй авирагч: дунд зэргийн',
    '🧗🏾‍♀' => 'эмэгтэй авирагч: бордуу',
    '🧗🏿‍♀' => 'эмэгтэй авирагч: бор',
    '🏌🏻‍♂' => 'гольф тоглож буй эрэгтэй: цайвар',
    '🏌🏼‍♂' => 'гольф тоглож буй эрэгтэй: бага зэргийн цайвар',
    '🏌🏽‍♂' => 'гольф тоглож буй эрэгтэй: дунд зэргийн',
    '🏌🏾‍♂' => 'гольф тоглож буй эрэгтэй: бордуу',
    '🏌🏿‍♂' => 'гольф тоглож буй эрэгтэй: бор',
    '🏌🏻‍♀' => 'гольф тоглож буй эмэгтэй: цайвар',
    '🏌🏼‍♀' => 'гольф тоглож буй эмэгтэй: бага зэргийн цайвар',
    '🏌🏽‍♀' => 'гольф тоглож буй эмэгтэй: дунд зэргийн',
    '🏌🏾‍♀' => 'гольф тоглож буй эмэгтэй: бордуу',
    '🏌🏿‍♀' => 'гольф тоглож буй эмэгтэй: бор',
    '🏄🏻‍♂' => 'серфингээр гулгаж буй эрэгтэй: цайвар',
    '🏄🏼‍♂' => 'серфингээр гулгаж буй эрэгтэй: бага зэргийн цайвар',
    '🏄🏽‍♂' => 'серфингээр гулгаж буй эрэгтэй: дунд зэргийн',
    '🏄🏾‍♂' => 'серфингээр гулгаж буй эрэгтэй: бордуу',
    '🏄🏿‍♂' => 'серфингээр гулгаж буй эрэгтэй: бор',
    '🏄🏻‍♀' => 'серфингээр гулгаж буй эмэгтэй: цайвар',
    '🏄🏼‍♀' => 'серфингээр гулгаж буй эмэгтэй: бага зэргийн цайвар',
    '🏄🏽‍♀' => 'серфингээр гулгаж буй эмэгтэй: дунд зэргийн',
    '🏄🏾‍♀' => 'серфингээр гулгаж буй эмэгтэй: бордуу',
    '🏄🏿‍♀' => 'серфингээр гулгаж буй эмэгтэй: бор',
    '🚣🏻‍♂' => 'завь сэлүүрдэж буй эрэгтэй: цайвар',
    '🚣🏼‍♂' => 'завь сэлүүрдэж буй эрэгтэй: бага зэргийн цайвар',
    '🚣🏽‍♂' => 'завь сэлүүрдэж буй эрэгтэй: дунд зэргийн',
    '🚣🏾‍♂' => 'завь сэлүүрдэж буй эрэгтэй: бордуу',
    '🚣🏿‍♂' => 'завь сэлүүрдэж буй эрэгтэй: бор',
    '🚣🏻‍♀' => 'завь сэлүүрдэж буй эмэгтэй: цайвар',
    '🚣🏼‍♀' => 'завь сэлүүрдэж буй эмэгтэй: бага зэргийн цайвар',
    '🚣🏽‍♀' => 'завь сэлүүрдэж буй эмэгтэй: дунд зэргийн',
    '🚣🏾‍♀' => 'завь сэлүүрдэж буй эмэгтэй: бордуу',
    '🚣🏿‍♀' => 'завь сэлүүрдэж буй эмэгтэй: бор',
    '🏊🏻‍♂' => 'сэлж буй эрэгтэй: цайвар',
    '🏊🏼‍♂' => 'сэлж буй эрэгтэй: бага зэргийн цайвар',
    '🏊🏽‍♂' => 'сэлж буй эрэгтэй: дунд зэргийн',
    '🏊🏾‍♂' => 'сэлж буй эрэгтэй: бордуу',
    '🏊🏿‍♂' => 'сэлж буй эрэгтэй: бор',
    '🏊🏻‍♀' => 'сэлж буй эмэгтэй: цайвар',
    '🏊🏼‍♀' => 'сэлж буй эмэгтэй: бага зэргийн цайвар',
    '🏊🏽‍♀' => 'сэлж буй эмэгтэй: дунд зэргийн',
    '🏊🏾‍♀' => 'сэлж буй эмэгтэй: бордуу',
    '🏊🏿‍♀' => 'сэлж буй эмэгтэй: бор',
    '⛹🏻‍♂' => 'бөмбөгтэй эрэгтэй хүн: цайвар',
    '⛹🏼‍♂' => 'бөмбөгтэй эрэгтэй хүн: бага зэргийн цайвар',
    '⛹🏽‍♂' => 'бөмбөгтэй эрэгтэй хүн: дунд зэргийн',
    '⛹🏾‍♂' => 'бөмбөгтэй эрэгтэй хүн: бордуу',
    '⛹🏿‍♂' => 'бөмбөгтэй эрэгтэй хүн: бор',
    '⛹🏻‍♀' => 'бөмбөгтэй эмэгтэй хүн: цайвар',
    '⛹🏼‍♀' => 'бөмбөгтэй эмэгтэй хүн: бага зэргийн цайвар',
    '⛹🏽‍♀' => 'бөмбөгтэй эмэгтэй хүн: дунд зэргийн',
    '⛹🏾‍♀' => 'бөмбөгтэй эмэгтэй хүн: бордуу',
    '⛹🏿‍♀' => 'бөмбөгтэй эмэгтэй хүн: бор',
    '🏋🏻‍♂' => 'хүндийн өргөгч эрэгтэй: цайвар',
    '🏋🏼‍♂' => 'хүндийн өргөгч эрэгтэй: бага зэргийн цайвар',
    '🏋🏽‍♂' => 'хүндийн өргөгч эрэгтэй: дунд зэргийн',
    '🏋🏾‍♂' => 'хүндийн өргөгч эрэгтэй: бордуу',
    '🏋🏿‍♂' => 'хүндийн өргөгч эрэгтэй: бор',
    '🏋🏻‍♀' => 'хүндийн өргөгч эмэгтэй: цайвар',
    '🏋🏼‍♀' => 'хүндийн өргөгч эмэгтэй: бага зэргийн цайвар',
    '🏋🏽‍♀' => 'хүндийн өргөгч эмэгтэй: дунд зэргийн',
    '🏋🏾‍♀' => 'хүндийн өргөгч эмэгтэй: бордуу',
    '🏋🏿‍♀' => 'хүндийн өргөгч эмэгтэй: бор',
    '🚴🏻‍♂' => 'дугуй унаж буй эрэгтэй: цайвар',
    '🚴🏼‍♂' => 'дугуй унаж буй эрэгтэй: бага зэргийн цайвар',
    '🚴🏽‍♂' => 'дугуй унаж буй эрэгтэй: дунд зэргийн',
    '🚴🏾‍♂' => 'дугуй унаж буй эрэгтэй: бордуу',
    '🚴🏿‍♂' => 'дугуй унаж буй эрэгтэй: бор',
    '🚴🏻‍♀' => 'дугуй унаж буй эмэгтэй: цайвар',
    '🚴🏼‍♀' => 'дугуй унаж буй эмэгтэй: бага зэргийн цайвар',
    '🚴🏽‍♀' => 'дугуй унаж буй эмэгтэй: дунд зэргийн',
    '🚴🏾‍♀' => 'дугуй унаж буй эмэгтэй: бордуу',
    '🚴🏿‍♀' => 'дугуй унаж буй эмэгтэй: бор',
    '🚵🏻‍♂' => 'уулын дугуй унаж буй эрэгтэй: цайвар',
    '🚵🏼‍♂' => 'уулын дугуй унаж буй эрэгтэй: бага зэргийн цайвар',
    '🚵🏽‍♂' => 'уулын дугуй унаж буй эрэгтэй: дунд зэргийн',
    '🚵🏾‍♂' => 'уулын дугуй унаж буй эрэгтэй: бордуу',
    '🚵🏿‍♂' => 'уулын дугуй унаж буй эрэгтэй: бор',
    '🚵🏻‍♀' => 'уулын дугуй унаж буй эмэгтэй: цайвар',
    '🚵🏼‍♀' => 'уулын дугуй унаж буй эмэгтэй: бага зэргийн цайвар',
    '🚵🏽‍♀' => 'уулын дугуй унаж буй эмэгтэй: дунд зэргийн',
    '🚵🏾‍♀' => 'уулын дугуй унаж буй эмэгтэй: бордуу',
    '🚵🏿‍♀' => 'уулын дугуй унаж буй эмэгтэй: бор',
    '🤸🏻‍♂' => 'сальто эргэх эрэгтэй: цайвар',
    '🤸🏼‍♂' => 'сальто эргэх эрэгтэй: бага зэргийн цайвар',
    '🤸🏽‍♂' => 'сальто эргэх эрэгтэй: дунд зэргийн',
    '🤸🏾‍♂' => 'сальто эргэх эрэгтэй: бордуу',
    '🤸🏿‍♂' => 'сальто эргэх эрэгтэй: бор',
    '🤸🏻‍♀' => 'сальто эргэх эмэгтэй: цайвар',
    '🤸🏼‍♀' => 'сальто эргэх эмэгтэй: бага зэргийн цайвар',
    '🤸🏽‍♀' => 'сальто эргэх эмэгтэй: дунд зэргийн',
    '🤸🏾‍♀' => 'сальто эргэх эмэгтэй: бордуу',
    '🤸🏿‍♀' => 'сальто эргэх эмэгтэй: бор',
    '🤼🏻‍♀' => 'барилдаж буй эмэгтэйчүүд: цайвар',
    '🤼🏼‍♀' => 'барилдаж буй эмэгтэйчүүд: бага зэргийн цайвар',
    '🤼🏽‍♀' => 'барилдаж буй эмэгтэйчүүд: дунд зэргийн',
    '🤼🏾‍♀' => 'барилдаж буй эмэгтэйчүүд: бордуу',
    '🤼🏿‍♀' => 'барилдаж буй эмэгтэйчүүд: бор',
    '🤼🏻‍♂' => 'барилдаж буй эрэгтэйчүүд: цайвар',
    '🤼🏼‍♂' => 'барилдаж буй эрэгтэйчүүд: бага зэргийн цайвар',
    '🤼🏽‍♂' => 'барилдаж буй эрэгтэйчүүд: дунд зэргийн',
    '🤼🏾‍♂' => 'барилдаж буй эрэгтэйчүүд: бордуу',
    '🤼🏿‍♂' => 'барилдаж буй эрэгтэйчүүд: бор',
    '🤽🏻‍♂' => 'усан поло тоглох эрэгтэй: цайвар',
    '🤽🏼‍♂' => 'усан поло тоглох эрэгтэй: бага зэргийн цайвар',
    '🤽🏽‍♂' => 'усан поло тоглох эрэгтэй: дунд зэргийн',
    '🤽🏾‍♂' => 'усан поло тоглох эрэгтэй: бордуу',
    '🤽🏿‍♂' => 'усан поло тоглох эрэгтэй: бор',
    '🤽🏻‍♀' => 'усан поло тоглох эмэгтэй: цайвар',
    '🤽🏼‍♀' => 'усан поло тоглох эмэгтэй: бага зэргийн цайвар',
    '🤽🏽‍♀' => 'усан поло тоглох эмэгтэй: дунд зэргийн',
    '🤽🏾‍♀' => 'усан поло тоглох эмэгтэй: бордуу',
    '🤽🏿‍♀' => 'усан поло тоглох эмэгтэй: бор',
    '🤾🏻‍♂' => 'гар бөмбөг тоглох эрэгтэй: цайвар',
    '🤾🏼‍♂' => 'гар бөмбөг тоглох эрэгтэй: бага зэргийн цайвар',
    '🤾🏽‍♂' => 'гар бөмбөг тоглох эрэгтэй: дунд зэргийн',
    '🤾🏾‍♂' => 'гар бөмбөг тоглох эрэгтэй: бордуу',
    '🤾🏿‍♂' => 'гар бөмбөг тоглох эрэгтэй: бор',
    '🤾🏻‍♀' => 'гар бөмбөг тоглох эмэгтэй: цайвар',
    '🤾🏼‍♀' => 'гар бөмбөг тоглох эмэгтэй: бага зэргийн цайвар',
    '🤾🏽‍♀' => 'гар бөмбөг тоглох эмэгтэй: дунд зэргийн',
    '🤾🏾‍♀' => 'гар бөмбөг тоглох эмэгтэй: бордуу',
    '🤾🏿‍♀' => 'гар бөмбөг тоглох эмэгтэй: бор',
    '🤹🏻‍♂' => 'жонглёрдох эрэгтэй: цайвар',
    '🤹🏼‍♂' => 'жонглёрдох эрэгтэй: бага зэргийн цайвар',
    '🤹🏽‍♂' => 'жонглёрдох эрэгтэй: дунд зэргийн',
    '🤹🏾‍♂' => 'жонглёрдох эрэгтэй: бордуу',
    '🤹🏿‍♂' => 'жонглёрдох эрэгтэй: бор',
    '🤹🏻‍♀' => 'жонглёрдох эмэгтэй: цайвар',
    '🤹🏼‍♀' => 'жонглёрдох эмэгтэй: бага зэргийн цайвар',
    '🤹🏽‍♀' => 'жонглёрдох эмэгтэй: дунд зэргийн',
    '🤹🏾‍♀' => 'жонглёрдох эмэгтэй: бордуу',
    '🤹🏿‍♀' => 'жонглёрдох эмэгтэй: бор',
    '🧘🏻‍♂' => 'лотус байрлалтай эрэгтэй: цайвар',
    '🧘🏼‍♂' => 'лотус байрлалтай эрэгтэй: бага зэргийн цайвар',
    '🧘🏽‍♂' => 'лотус байрлалтай эрэгтэй: дунд зэргийн',
    '🧘🏾‍♂' => 'лотус байрлалтай эрэгтэй: бордуу',
    '🧘🏿‍♂' => 'лотус байрлалтай эрэгтэй: бор',
    '🧘🏻‍♀' => 'лотус байрлалтай эмэгтэй: цайвар',
    '🧘🏼‍♀' => 'лотус байрлалтай эмэгтэй: бага зэргийн цайвар',
    '🧘🏽‍♀' => 'лотус байрлалтай эмэгтэй: дунд зэргийн',
    '🧘🏾‍♀' => 'лотус байрлалтай эмэгтэй: бордуу',
    '🧘🏿‍♀' => 'лотус байрлалтай эмэгтэй: бор',
    '🧑‍🦰' => 'том хүн: улаан үс',
    '🧑‍🦱' => 'том хүн: буржгар үс',
    '🧑‍🦳' => 'том хүн: цагаан үс',
    '🧑‍🦲' => 'том хүн: халзан',
    '👨‍🦰' => 'эрэгтэй хүн: улаан үс',
    '👨‍🦱' => 'эрэгтэй хүн: буржгар үс',
    '👨‍🦳' => 'эрэгтэй хүн: цагаан үс',
    '👨‍🦲' => 'эрэгтэй хүн: халзан',
    '👩‍🦰' => 'эмэгтэй хүн: улаан үс',
    '👩‍🦱' => 'эмэгтэй хүн: буржгар үс',
    '👩‍🦳' => 'эмэгтэй хүн: цагаан үс',
    '👩‍🦲' => 'эмэгтэй хүн: халзан',
    '🚶‍➡' => 'алхаж буй хүн: баруун тийш харсан',
    '🧎‍➡' => 'сөхөрч суусан хүн: баруун тийш харсан',
    '🏃‍➡' => 'гүйж байгаа хүн: баруун тийш харсан',
    '👨‍👦' => 'гэр бүл: эрэгтэй хүн эрэгтэй хүүхэд',
    '👨‍👧' => 'гэр бүл: эрэгтэй хүн эмэгтэй хүүхэд',
    '👩‍👦' => 'гэр бүл: эмэгтэй хүн эрэгтэй хүүхэд',
    '👩‍👧' => 'гэр бүл: эмэгтэй хүн эмэгтэй хүүхэд',
    '😶‍🌫' => 'алмайрсан царай',
    '😮‍💨' => 'амьсгалж буй царай',
    '🙂‍↕' => 'толгойгоо хоёр тийш дохих',
    '😵‍💫' => 'нүд нь эрчлээтэй царай',
    '❤‍🔥' => 'шатаж буй зүрх',
    '❤‍🩹' => 'засарч буй',
    '👁‍🗨' => 'ярианы хөөсөн доторх нүд',
    '🧔‍♂' => 'эрэгтэй: сахал',
    '👱‍♂' => 'шар үстэй эрэгтэй',
    '🧔‍♀' => 'эмэгтэй: сахал',
    '👱‍♀' => 'шар үстэй эмэгтэй',
    '🙍‍♂' => 'дурамжхан эрэгтэй',
    '🙍‍♀' => 'дурамжхан эмэгтэй',
    '🙎‍♂' => 'урвагар эрэгтэй',
    '🙎‍♀' => 'урвагар эмэгтэй',
    '🙅‍♂' => 'зүгээр биш гэж харуулж буй эрэгтэй',
    '🙅‍♀' => 'зүгээр биш гэж харуулж буй эмэгтэй',
    '🙆‍♂' => 'зүгээр гэж харуулж буй эрэгтэй',
    '🙆‍♀' => 'зүгээр гэж харуулж буй эмэгтэй',
    '💁‍♂' => 'гараа дэлгэж буй эрэгтэй',
    '💁‍♀' => 'гараа дэлгэж буй эмэгтэй',
    '🙋‍♂' => 'гараа өргөж буй эрэгтэй',
    '🙋‍♀' => 'гараа өргөж буй эмэгтэй',
    '🧏‍♂' => 'дүлий эрэгтэй',
    '🧏‍♀' => 'дүлий эмэгтэй',
    '🙇‍♂' => 'бөхийж буй эрэгтэй',
    '🙇‍♀' => 'бөхийж буй эмэгтэй',
    '🤦‍♂' => 'урам хугарч буй эрэгтэй',
    '🤦‍♀' => 'урам хугарч буй эмэгтэй',
    '🤷‍♂' => 'мөрөө хавчиж буй эрэгтэй',
    '🤷‍♀' => 'мөрөө хавчиж буй эмэгтэй',
    '🧑‍⚕' => 'эрүүл мэндийн ажилтан',
    '👨‍⚕' => 'эрэгтэй эрүүл мэндийн ажилтан',
    '👩‍⚕' => 'эмэгтэй эрүүл мэндийн ажилтан',
    '🧑‍🎓' => 'оюутан',
    '👨‍🎓' => 'эрэгтэй оюутан',
    '👩‍🎓' => 'эмэгтэй оюутан',
    '🧑‍🏫' => 'багш',
    '👨‍🏫' => 'эрэгтэй багш',
    '👩‍🏫' => 'эмэгтэй багш',
    '🧑‍⚖' => 'шүүгч',
    '👨‍⚖' => 'эрэгтэй шүүгч',
    '👩‍⚖' => 'эмэгтэй шүүгч',
    '🧑‍🌾' => 'фермер',
    '👨‍🌾' => 'эрэгтэй тариачин',
    '👩‍🌾' => 'эмэгтэй тариачин',
    '🧑‍🍳' => 'хоол хийх',
    '👨‍🍳' => 'эрэгтэй тогооч',
    '👩‍🍳' => 'эмэгтэй тогооч',
    '🧑‍🔧' => 'механик',
    '👨‍🔧' => 'эрэгтэй механикч',
    '👩‍🔧' => 'эмэгтэй механикч',
    '🧑‍🏭' => 'үйлдвэрийн ажилтан',
    '👨‍🏭' => 'эрэгтэй үйлдвэрийн ажилтан',
    '👩‍🏭' => 'эмэгтэй үйлдвэрийн ажилтан',
    '🧑‍💼' => 'оффисын ажилтан',
    '👨‍💼' => 'эрэгтэй оффисын ажилтан',
    '👩‍💼' => 'эмэгтэй оффисын ажилтан',
    '🧑‍🔬' => 'шинжлэх ухаанч',
    '👨‍🔬' => 'эрэгтэй эрдэмтэн',
    '👩‍🔬' => 'эмэгтэй эрдэмтэн',
    '🧑‍💻' => 'технологич',
    '👨‍💻' => 'эрэгтэй технологич',
    '👩‍💻' => 'эмэгтэй технологич',
    '🧑‍🎤' => 'дуучин',
    '👨‍🎤' => 'эрэгтэй дуучин',
    '👩‍🎤' => 'эмэгтэй дуучин',
    '🧑‍🎨' => 'уран бүтээлч',
    '👨‍🎨' => 'эрэгтэй зураач',
    '👩‍🎨' => 'эмэгтэй зураач',
    '🧑‍✈' => 'нисгэгч',
    '👨‍✈' => 'эрэгтэй нисгэгч',
    '👩‍✈' => 'эмэгтэй нисгэгч',
    '🧑‍🚀' => 'сансрын нисэгч',
    '👨‍🚀' => 'эрэгтэй сансрын нисгэгч',
    '👩‍🚀' => 'эмэгтэй сансрын нисгэгч',
    '🧑‍🚒' => 'гал сөнөөгч',
    '👨‍🚒' => 'эрэгтэй гал сөнөөгч',
    '👩‍🚒' => 'эмэгтэй гал сөнөөгч',
    '👮‍♂' => 'эрэгтэй цагдаа',
    '👮‍♀' => 'эмэгтэй цагдаа',
    '🕵‍♂' => 'эрэгтэй мөрдөгч',
    '🕵‍♀' => 'эмэгтэй мөрдөгч',
    '💂‍♂' => 'эрэгтэй харуул',
    '💂‍♀' => 'эмэгтэй харуул',
    '👷‍♂' => 'эрэгтэй барилгын ажилтан',
    '👷‍♀' => 'эмэгтэй барилгын ажилтан',
    '👳‍♂' => 'толгойн ороолттой эрэгтэй хүн',
    '👳‍♀' => 'толгойн ороолттой эмэгтэй хүн',
    '🤵‍♂' => 'хослолтой эрэгтэй',
    '🤵‍♀' => 'хослолтой эмэгтэй',
    '👰‍♂' => 'гивлүүртэй эрэгтэй',
    '👰‍♀' => 'гивлүүртэй эмэгтэй',
    '👩‍🍼' => 'нялх хүүхэд хооллож байгаа эмэгтэй',
    '👨‍🍼' => 'нялх хүүхэд хооллож байгаа эрэгтэй',
    '🧑‍🍼' => 'нялх хүүхэд хооллож байгаа хүн',
    '🧑‍🎄' => 'хатагтай клаус',
    '🦸‍♂' => 'эрэгтэй супер баатар',
    '🦸‍♀' => 'эмэгтэй супер баатар',
    '🦹‍♂' => 'ид шидийн хүчтэй эрэгтэй эсрэг дүр',
    '🦹‍♀' => 'ид шидийн хүчтэй эмэгтэй эсрэг дүр',
    '🧙‍♂' => 'эрэгтэй шидтэн',
    '🧙‍♀' => 'эмэгтэй шидтэн',
    '🧚‍♂' => 'эрэгтэй илбэчин',
    '🧚‍♀' => 'эмэгтэй илбэчин',
    '🧛‍♂' => 'эрэгтэй цус сорогч',
    '🧛‍♀' => 'эмэгтэй цус сорогч',
    '🧜‍♂' => 'лусын хаан',
    '🧜‍♀' => 'лусын дагина',
    '🧝‍♂' => 'эрэгтэй одой',
    '🧝‍♀' => 'эмэгтэй одой',
    '🧞‍♂' => 'эрэгтэй жини',
    '🧞‍♀' => 'Эмэгтэй жини',
    '🧟‍♂' => 'эрэгтэй зомби',
    '🧟‍♀' => 'эмэгтэй зомби',
    '💆‍♂' => 'нүүрэндээ массаж хийлгэж буй эрэгтэй',
    '💆‍♀' => 'нүүрэндээ массаж хийлгэж буй эмэгтэй',
    '💇‍♂' => 'үсээ тайруулж буй эрэгтэй',
    '💇‍♀' => 'үсээ тайруулж буй эмэгтэй',
    '🚶‍♂' => 'алхаж буй эрэгтэй',
    '🚶‍♀' => 'алхаж буй эмэгтэй',
    '🧍‍♂' => 'зогсож буй эрэгтэй',
    '🧍‍♀' => 'зогсож буй эмэгтэй',
    '🧎‍♂' => 'сөхөрч буй эрэгтэй',
    '🧎‍♀' => 'сөхөрч буй эмэгтэй',
    '🧑‍🦯' => 'метал илрүүлэгчтэй хүн',
    '👨‍🦯' => 'таягтай хараагүй эрэгтэй',
    '👩‍🦯' => 'таягтай хараагүй эмэгтэй',
    '🧑‍🦼' => 'автомат удирдлагатай тэргэнцэртэй хүн',
    '👨‍🦼' => 'моторт тэргэнцэртэй эрэгтэй',
    '👩‍🦼' => 'моторт тэргэнцэртэй эмэгтэй',
    '🧑‍🦽' => 'гар ажиллагаатай тэргэнцэртэй хүн',
    '👨‍🦽' => 'тэргэнцэртэй эрэгтэй',
    '👩‍🦽' => 'тэргэнцэртэй эмэгтэй',
    '🏃‍♂' => 'гүйж буй эрэгтэй',
    '🏃‍♀' => 'гүйж буй эмэгтэй',
    '🧑‍🩰' => 'баллетчин',
    '👯‍♂' => 'цэнгэж буй эрэгтэйчүүд',
    '👯‍♀' => 'цэнгэж буй эмэгтэйчүүд',
    '🧖‍♂' => 'уур савссан өрөөнд байгаа эрэгтэй',
    '🧖‍♀' => 'уур савссан өрөөнд байгаа эмэгтэй',
    '🧗‍♂' => 'авирагч эрэгтэй',
    '🧗‍♀' => 'эмэгтэй авирагч',
    '🏌‍♂' => 'гольф тоглож буй эрэгтэй',
    '🏌‍♀' => 'гольф тоглож буй эмэгтэй',
    '🏄‍♂' => 'серфингээр гулгаж буй эрэгтэй',
    '🏄‍♀' => 'серфингээр гулгаж буй эмэгтэй',
    '🚣‍♂' => 'завь сэлүүрдэж буй эрэгтэй',
    '🚣‍♀' => 'завь сэлүүрдэж буй эмэгтэй',
    '🏊‍♂' => 'сэлж буй эрэгтэй',
    '🏊‍♀' => 'сэлж буй эмэгтэй',
    '⛹‍♂' => 'бөмбөгтэй эрэгтэй хүн',
    '⛹‍♀' => 'бөмбөгтэй эмэгтэй хүн',
    '🏋‍♂' => 'хүндийн өргөгч эрэгтэй',
    '🏋‍♀' => 'хүндийн өргөгч эмэгтэй',
    '🚴‍♂' => 'дугуй унаж буй эрэгтэй',
    '🚴‍♀' => 'дугуй унаж буй эмэгтэй',
    '🚵‍♂' => 'уулын дугуй унаж буй эрэгтэй',
    '🚵‍♀' => 'уулын дугуй унаж буй эмэгтэй',
    '🤸‍♂' => 'сальто эргэх эрэгтэй',
    '🤸‍♀' => 'сальто эргэх эмэгтэй',
    '🤼‍♂' => 'барилдаж буй эрэгтэйчүүд',
    '🤼‍♀' => 'барилдаж буй эмэгтэйчүүд',
    '🤽‍♂' => 'усан поло тоглох эрэгтэй',
    '🤽‍♀' => 'усан поло тоглох эмэгтэй',
    '🤾‍♂' => 'гар бөмбөг тоглох эрэгтэй',
    '🤾‍♀' => 'гар бөмбөг тоглох эмэгтэй',
    '🤹‍♂' => 'жонглёрдох эрэгтэй',
    '🤹‍♀' => 'жонглёрдох эмэгтэй',
    '🧘‍♂' => 'лотус байрлалтай эрэгтэй',
    '🧘‍♀' => 'лотус байрлалтай эмэгтэй',
    '🧑‍🧒' => 'гэр бүл: насанд хүрсэн хүн, хүүхэд',
    '🐕‍🦺' => 'үйлчлэгч нохой',
    '🐈‍⬛' => 'хар муур',
    '🐻‍❄' => 'цагаан баавгай',
    '🐦‍⬛' => 'хар шувуу',
    '🐦‍🔥' => 'галт шувуу',
    '🍋‍🟩' => 'нимбэг',
    '🍄‍🟫' => 'бор мөөг',
    '⛓‍💥' => 'эвдэрсэн гинж',
    '🏳‍🌈' => 'солонгон туг',
    '🏳‍⚧' => 'трансжендер туг',
    '🏴‍☠' => 'далайн дээрэмчний туг',
    '👋🏻' => 'мэндчилж байгаа гар: цайвар',
    '👋🏼' => 'мэндчилж байгаа гар: бага зэргийн цайвар',
    '👋🏽' => 'мэндчилж байгаа гар: дунд зэргийн',
    '👋🏾' => 'мэндчилж байгаа гар: бордуу',
    '👋🏿' => 'мэндчилж байгаа гар: бор',
    '🤚🏻' => 'өргөсөн гарын ар: цайвар',
    '🤚🏼' => 'өргөсөн гарын ар: бага зэргийн цайвар',
    '🤚🏽' => 'өргөсөн гарын ар: дунд зэргийн',
    '🤚🏾' => 'өргөсөн гарын ар: бордуу',
    '🤚🏿' => 'өргөсөн гарын ар: бор',
    '🖐🏻' => 'хуруунуудаа салгасан гарын алга: цайвар',
    '🖐🏼' => 'хуруунуудаа салгасан гарын алга: бага зэргийн цайвар',
    '🖐🏽' => 'хуруунуудаа салгасан гарын алга: дунд зэргийн',
    '🖐🏾' => 'хуруунуудаа салгасан гарын алга: бордуу',
    '🖐🏿' => 'хуруунуудаа салгасан гарын алга: бор',
    '✋🏻' => 'өргөсөн гар: цайвар',
    '✋🏼' => 'өргөсөн гар: бага зэргийн цайвар',
    '✋🏽' => 'өргөсөн гар: дунд зэргийн',
    '✋🏾' => 'өргөсөн гар: бордуу',
    '✋🏿' => 'өргөсөн гар: бор',
    '🖖🏻' => 'вулкан мэндчилгээ: цайвар',
    '🖖🏼' => 'вулкан мэндчилгээ: бага зэргийн цайвар',
    '🖖🏽' => 'вулкан мэндчилгээ: дунд зэргийн',
    '🖖🏾' => 'вулкан мэндчилгээ: бордуу',
    '🖖🏿' => 'вулкан мэндчилгээ: бор',
    '🫱🏻' => 'баруун тийш чиглэсэн гар: цайвар',
    '🫱🏼' => 'баруун тийш чиглэсэн гар: бага зэргийн цайвар',
    '🫱🏽' => 'баруун тийш чиглэсэн гар: дунд зэргийн',
    '🫱🏾' => 'баруун тийш чиглэсэн гар: бордуу',
    '🫱🏿' => 'баруун тийш чиглэсэн гар: бор',
    '🫲🏻' => 'зүүн тийш чиглэсэн гар: цайвар',
    '🫲🏼' => 'зүүн тийш чиглэсэн гар: бага зэргийн цайвар',
    '🫲🏽' => 'зүүн тийш чиглэсэн гар: дунд зэргийн',
    '🫲🏾' => 'зүүн тийш чиглэсэн гар: бордуу',
    '🫲🏿' => 'зүүн тийш чиглэсэн гар: бор',
    '🫳🏻' => 'алга нь доошоо харсан гар: цайвар',
    '🫳🏼' => 'алга нь доошоо харсан гар: бага зэргийн цайвар',
    '🫳🏽' => 'алга нь доошоо харсан гар: дунд зэргийн',
    '🫳🏾' => 'алга нь доошоо харсан гар: бордуу',
    '🫳🏿' => 'алга нь доошоо харсан гар: бор',
    '🫴🏻' => 'алга нь дээшээ харсан гар: цайвар',
    '🫴🏼' => 'алга нь дээшээ харсан гар: бага зэргийн цайвар',
    '🫴🏽' => 'алга нь дээшээ харсан гар: дунд зэргийн',
    '🫴🏾' => 'алга нь дээшээ харсан гар: бордуу',
    '🫴🏿' => 'алга нь дээшээ харсан гар: бор',
    '🫷🏻' => 'зүүн тийш түлхсэн гар: цайвар',
    '🫷🏼' => 'зүүн тийш түлхсэн гар: бага зэргийн цайвар',
    '🫷🏽' => 'зүүн тийш түлхсэн гар: дунд зэргийн',
    '🫷🏾' => 'зүүн тийш түлхсэн гар: бордуу',
    '🫷🏿' => 'зүүн тийш түлхсэн гар: бор',
    '🫸🏻' => 'баруун тийш түлхсэн гар: цайвар',
    '🫸🏼' => 'баруун тийш түлхсэн гар: бага зэргийн цайвар',
    '🫸🏽' => 'баруун тийш түлхсэн гар: дунд зэргийн',
    '🫸🏾' => 'баруун тийш түлхсэн гар: бордуу',
    '🫸🏿' => 'баруун тийш түлхсэн гар: бор',
    '👌🏻' => 'ок: цайвар',
    '👌🏼' => 'ок: бага зэргийн цайвар',
    '👌🏽' => 'ок: дунд зэргийн',
    '👌🏾' => 'ок: бордуу',
    '👌🏿' => 'ок: бор',
    '🤌🏻' => 'гарын хуруунуудаа нийлүүлж чимхэх: цайвар',
    '🤌🏼' => 'гарын хуруунуудаа нийлүүлж чимхэх: бага зэргийн цайвар',
    '🤌🏽' => 'гарын хуруунуудаа нийлүүлж чимхэх: дунд зэргийн',
    '🤌🏾' => 'гарын хуруунуудаа нийлүүлж чимхэх: бордуу',
    '🤌🏿' => 'гарын хуруунуудаа нийлүүлж чимхэх: бор',
    '🤏🏻' => 'чимхэж буй гар: цайвар',
    '🤏🏼' => 'чимхэж буй гар: бага зэргийн цайвар',
    '🤏🏽' => 'чимхэж буй гар: дунд зэргийн',
    '🤏🏾' => 'чимхэж буй гар: бордуу',
    '🤏🏿' => 'чимхэж буй гар: бор',
    '✌🏻' => 'амжилт хүссэн гар: цайвар',
    '✌🏼' => 'амжилт хүссэн гар: бага зэргийн цайвар',
    '✌🏽' => 'амжилт хүссэн гар: дунд зэргийн',
    '✌🏾' => 'амжилт хүссэн гар: бордуу',
    '✌🏿' => 'амжилт хүссэн гар: бор',
    '🤞🏻' => 'солбисон хуруу: цайвар',
    '🤞🏼' => 'солбисон хуруу: бага зэргийн цайвар',
    '🤞🏽' => 'солбисон хуруу: дунд зэргийн',
    '🤞🏾' => 'солбисон хуруу: бордуу',
    '🤞🏿' => 'солбисон хуруу: бор',
    '🫰🏻' => 'долоовор хуруу, эрхий хуруугаа зөрүүлж барьсан гар: цайвар',
    '🫰🏼' => 'долоовор хуруу, эрхий хуруугаа зөрүүлж барьсан гар: бага зэргийн цайвар',
    '🫰🏽' => 'долоовор хуруу, эрхий хуруугаа зөрүүлж барьсан гар: дунд зэргийн',
    '🫰🏾' => 'долоовор хуруу, эрхий хуруугаа зөрүүлж барьсан гар: бордуу',
    '🫰🏿' => 'долоовор хуруу, эрхий хуруугаа зөрүүлж барьсан гар: бор',
    '🤟🏻' => 'хайртай гэсэн дохио: цайвар',
    '🤟🏼' => 'хайртай гэсэн дохио: бага зэргийн цайвар',
    '🤟🏽' => 'хайртай гэсэн дохио: дунд зэргийн',
    '🤟🏾' => 'хайртай гэсэн дохио: бордуу',
    '🤟🏿' => 'хайртай гэсэн дохио: бор',
    '🤘🏻' => 'эвэрний тэмдэг: цайвар',
    '🤘🏼' => 'эвэрний тэмдэг: бага зэргийн цайвар',
    '🤘🏽' => 'эвэрний тэмдэг: дунд зэргийн',
    '🤘🏾' => 'эвэрний тэмдэг: бордуу',
    '🤘🏿' => 'эвэрний тэмдэг: бор',
    '🤙🏻' => 'залгаарай дохио: цайвар',
    '🤙🏼' => 'залгаарай дохио: бага зэргийн цайвар',
    '🤙🏽' => 'залгаарай дохио: дунд зэргийн',
    '🤙🏾' => 'залгаарай дохио: бордуу',
    '🤙🏿' => 'залгаарай дохио: бор',
    '👈🏻' => 'зүүн зүг зааж байгаа: цайвар',
    '👈🏼' => 'зүүн зүг зааж байгаа: бага зэргийн цайвар',
    '👈🏽' => 'зүүн зүг зааж байгаа: дунд зэргийн',
    '👈🏾' => 'зүүн зүг зааж байгаа: бордуу',
    '👈🏿' => 'зүүн зүг зааж байгаа: бор',
    '👉🏻' => 'баруун зүг зааж байгаа: цайвар',
    '👉🏼' => 'баруун зүг зааж байгаа: бага зэргийн цайвар',
    '👉🏽' => 'баруун зүг зааж байгаа: дунд зэргийн',
    '👉🏾' => 'баруун зүг зааж байгаа: бордуу',
    '👉🏿' => 'баруун зүг зааж байгаа: бор',
    '👆🏻' => 'дээшээ зааж буй алганы ар тал: цайвар',
    '👆🏼' => 'дээшээ зааж буй алганы ар тал: бага зэргийн цайвар',
    '👆🏽' => 'дээшээ зааж буй алганы ар тал: дунд зэргийн',
    '👆🏾' => 'дээшээ зааж буй алганы ар тал: бордуу',
    '👆🏿' => 'дээшээ зааж буй алганы ар тал: бор',
    '🖕🏻' => 'дунд хуруу: цайвар',
    '🖕🏼' => 'дунд хуруу: бага зэргийн цайвар',
    '🖕🏽' => 'дунд хуруу: дунд зэргийн',
    '🖕🏾' => 'дунд хуруу: бордуу',
    '🖕🏿' => 'дунд хуруу: бор',
    '👇🏻' => 'доошоо зааж буй алганы ард тал: цайвар',
    '👇🏼' => 'доошоо зааж буй алганы ард тал: бага зэргийн цайвар',
    '👇🏽' => 'доошоо зааж буй алганы ард тал: дунд зэргийн',
    '👇🏾' => 'доошоо зааж буй алганы ард тал: бордуу',
    '👇🏿' => 'доошоо зааж буй алганы ард тал: бор',
    '☝🏻' => 'дээшээг зааж байгаа: цайвар',
    '☝🏼' => 'дээшээг зааж байгаа: бага зэргийн цайвар',
    '☝🏽' => 'дээшээг зааж байгаа: дунд зэргийн',
    '☝🏾' => 'дээшээг зааж байгаа: бордуу',
    '☝🏿' => 'дээшээг зааж байгаа: бор',
    '🫵🏻' => 'үзэгч рүү долоовор хуруугаар заах: цайвар',
    '🫵🏼' => 'үзэгч рүү долоовор хуруугаар заах: бага зэргийн цайвар',
    '🫵🏽' => 'үзэгч рүү долоовор хуруугаар заах: дунд зэргийн',
    '🫵🏾' => 'үзэгч рүү долоовор хуруугаар заах: бордуу',
    '🫵🏿' => 'үзэгч рүү долоовор хуруугаар заах: бор',
    '👍🏻' => 'сайн: цайвар',
    '👍🏼' => 'сайн: бага зэргийн цайвар',
    '👍🏽' => 'сайн: дунд зэргийн',
    '👍🏾' => 'сайн: бордуу',
    '👍🏿' => 'сайн: бор',
    '👎🏻' => 'муу: цайвар',
    '👎🏼' => 'муу: бага зэргийн цайвар',
    '👎🏽' => 'муу: дунд зэргийн',
    '👎🏾' => 'муу: бордуу',
    '👎🏿' => 'муу: бор',
    '✊🏻' => 'зангидсан гар: цайвар',
    '✊🏼' => 'зангидсан гар: бага зэргийн цайвар',
    '✊🏽' => 'зангидсан гар: дунд зэргийн',
    '✊🏾' => 'зангидсан гар: бордуу',
    '✊🏿' => 'зангидсан гар: бор',
    '👊🏻' => 'цохилт: цайвар',
    '👊🏼' => 'цохилт: бага зэргийн цайвар',
    '👊🏽' => 'цохилт: дунд зэргийн',
    '👊🏾' => 'цохилт: бордуу',
    '👊🏿' => 'цохилт: бор',
    '🤛🏻' => 'зүүн харсан зангидсан гар: цайвар',
    '🤛🏼' => 'зүүн харсан зангидсан гар: бага зэргийн цайвар',
    '🤛🏽' => 'зүүн харсан зангидсан гар: дунд зэргийн',
    '🤛🏾' => 'зүүн харсан зангидсан гар: бордуу',
    '🤛🏿' => 'зүүн харсан зангидсан гар: бор',
    '🤜🏻' => 'баруун харсан зангидсан гар: цайвар',
    '🤜🏼' => 'баруун харсан зангидсан гар: бага зэргийн цайвар',
    '🤜🏽' => 'баруун харсан зангидсан гар: дунд зэргийн',
    '🤜🏾' => 'баруун харсан зангидсан гар: бордуу',
    '🤜🏿' => 'баруун харсан зангидсан гар: бор',
    '👏🏻' => 'алга таших: цайвар',
    '👏🏼' => 'алга таших: бага зэргийн цайвар',
    '👏🏽' => 'алга таших: дунд зэргийн',
    '👏🏾' => 'алга таших: бордуу',
    '👏🏿' => 'алга таших: бор',
    '🙌🏻' => 'гараа өргөж байгаа хүн: цайвар',
    '🙌🏼' => 'гараа өргөж байгаа хүн: бага зэргийн цайвар',
    '🙌🏽' => 'гараа өргөж байгаа хүн: дунд зэргийн',
    '🙌🏾' => 'гараа өргөж байгаа хүн: бордуу',
    '🙌🏿' => 'гараа өргөж байгаа хүн: бор',
    '🫶🏻' => 'зүрхэн гар: цайвар',
    '🫶🏼' => 'зүрхэн гар: бага зэргийн цайвар',
    '🫶🏽' => 'зүрхэн гар: дунд зэргийн',
    '🫶🏾' => 'зүрхэн гар: бордуу',
    '🫶🏿' => 'зүрхэн гар: бор',
    '👐🏻' => 'дэлгэсэн алга: цайвар',
    '👐🏼' => 'дэлгэсэн алга: бага зэргийн цайвар',
    '👐🏽' => 'дэлгэсэн алга: дунд зэргийн',
    '👐🏾' => 'дэлгэсэн алга: бордуу',
    '👐🏿' => 'дэлгэсэн алга: бор',
    '🤲🏻' => 'алгаа хавсрах: цайвар',
    '🤲🏼' => 'алгаа хавсрах: бага зэргийн цайвар',
    '🤲🏽' => 'алгаа хавсрах: дунд зэргийн',
    '🤲🏾' => 'алгаа хавсрах: бордуу',
    '🤲🏿' => 'алгаа хавсрах: бор',
    '🤝🏻' => 'гар барилт: цайвар',
    '🤝🏼' => 'гар барилт: бага зэргийн цайвар',
    '🤝🏽' => 'гар барилт: дунд зэргийн',
    '🤝🏾' => 'гар барилт: бордуу',
    '🤝🏿' => 'гар барилт: бор',
    '🙏🏻' => 'хавсарсан гарын алга: цайвар',
    '🙏🏼' => 'хавсарсан гарын алга: бага зэргийн цайвар',
    '🙏🏽' => 'хавсарсан гарын алга: дунд зэргийн',
    '🙏🏾' => 'хавсарсан гарын алга: бордуу',
    '🙏🏿' => 'хавсарсан гарын алга: бор',
    '✍🏻' => 'бичиж байгаа гар: цайвар',
    '✍🏼' => 'бичиж байгаа гар: бага зэргийн цайвар',
    '✍🏽' => 'бичиж байгаа гар: дунд зэргийн',
    '✍🏾' => 'бичиж байгаа гар: бордуу',
    '✍🏿' => 'бичиж байгаа гар: бор',
    '💅🏻' => 'хумсаа будах: цайвар',
    '💅🏼' => 'хумсаа будах: бага зэргийн цайвар',
    '💅🏽' => 'хумсаа будах: дунд зэргийн',
    '💅🏾' => 'хумсаа будах: бордуу',
    '💅🏿' => 'хумсаа будах: бор',
    '🤳🏻' => 'селфи: цайвар',
    '🤳🏼' => 'селфи: бага зэргийн цайвар',
    '🤳🏽' => 'селфи: дунд зэргийн',
    '🤳🏾' => 'селфи: бордуу',
    '🤳🏿' => 'селфи: бор',
    '💪🏻' => 'гарын булчин: цайвар',
    '💪🏼' => 'гарын булчин: бага зэргийн цайвар',
    '💪🏽' => 'гарын булчин: дунд зэргийн',
    '💪🏾' => 'гарын булчин: бордуу',
    '💪🏿' => 'гарын булчин: бор',
    '🦵🏻' => 'хөл: цайвар',
    '🦵🏼' => 'хөл: бага зэргийн цайвар',
    '🦵🏽' => 'хөл: дунд зэргийн',
    '🦵🏾' => 'хөл: бордуу',
    '🦵🏿' => 'хөл: бор',
    '🦶🏻' => 'тавхай: цайвар',
    '🦶🏼' => 'тавхай: бага зэргийн цайвар',
    '🦶🏽' => 'тавхай: дунд зэргийн',
    '🦶🏾' => 'тавхай: бордуу',
    '🦶🏿' => 'тавхай: бор',
    '👂🏻' => 'чих: цайвар',
    '👂🏼' => 'чих: бага зэргийн цайвар',
    '👂🏽' => 'чих: дунд зэргийн',
    '👂🏾' => 'чих: бордуу',
    '👂🏿' => 'чих: бор',
    '🦻🏻' => 'сонсголын аппараттай чих: цайвар',
    '🦻🏼' => 'сонсголын аппараттай чих: бага зэргийн цайвар',
    '🦻🏽' => 'сонсголын аппараттай чих: дунд зэргийн',
    '🦻🏾' => 'сонсголын аппараттай чих: бордуу',
    '🦻🏿' => 'сонсголын аппараттай чих: бор',
    '👃🏻' => 'хамар: цайвар',
    '👃🏼' => 'хамар: бага зэргийн цайвар',
    '👃🏽' => 'хамар: дунд зэргийн',
    '👃🏾' => 'хамар: бордуу',
    '👃🏿' => 'хамар: бор',
    '👶🏻' => 'нялх хүүхэд: цайвар',
    '👶🏼' => 'нялх хүүхэд: бага зэргийн цайвар',
    '👶🏽' => 'нялх хүүхэд: дунд зэргийн',
    '👶🏾' => 'нялх хүүхэд: бордуу',
    '👶🏿' => 'нялх хүүхэд: бор',
    '🧒🏻' => 'хүүхэд: цайвар',
    '🧒🏼' => 'хүүхэд: бага зэргийн цайвар',
    '🧒🏽' => 'хүүхэд: дунд зэргийн',
    '🧒🏾' => 'хүүхэд: бордуу',
    '🧒🏿' => 'хүүхэд: бор',
    '👦🏻' => 'эрэгтэй хүүхэд: цайвар',
    '👦🏼' => 'эрэгтэй хүүхэд: бага зэргийн цайвар',
    '👦🏽' => 'эрэгтэй хүүхэд: дунд зэргийн',
    '👦🏾' => 'эрэгтэй хүүхэд: бордуу',
    '👦🏿' => 'эрэгтэй хүүхэд: бор',
    '👧🏻' => 'эмэгтэй хүүхэд: цайвар',
    '👧🏼' => 'эмэгтэй хүүхэд: бага зэргийн цайвар',
    '👧🏽' => 'эмэгтэй хүүхэд: дунд зэргийн',
    '👧🏾' => 'эмэгтэй хүүхэд: бордуу',
    '👧🏿' => 'эмэгтэй хүүхэд: бор',
    '🧑🏻' => 'том хүн: цайвар',
    '🧑🏼' => 'том хүн: бага зэргийн цайвар',
    '🧑🏽' => 'том хүн: дунд зэргийн',
    '🧑🏾' => 'том хүн: бордуу',
    '🧑🏿' => 'том хүн: бор',
    '👱🏻' => 'шаргал үстэй хүн: цайвар',
    '👱🏼' => 'шаргал үстэй хүн: бага зэргийн цайвар',
    '👱🏽' => 'шаргал үстэй хүн: дунд зэргийн',
    '👱🏾' => 'шаргал үстэй хүн: бордуу',
    '👱🏿' => 'шаргал үстэй хүн: бор',
    '👨🏻' => 'эрэгтэй хүн: цайвар',
    '👨🏼' => 'эрэгтэй хүн: бага зэргийн цайвар',
    '👨🏽' => 'эрэгтэй хүн: дунд зэргийн',
    '👨🏾' => 'эрэгтэй хүн: бордуу',
    '👨🏿' => 'эрэгтэй хүн: бор',
    '🧔🏻' => 'сахалтай хүн: цайвар',
    '🧔🏼' => 'сахалтай хүн: бага зэргийн цайвар',
    '🧔🏽' => 'сахалтай хүн: дунд зэргийн',
    '🧔🏾' => 'сахалтай хүн: бордуу',
    '🧔🏿' => 'сахалтай хүн: бор',
    '👩🏻' => 'эмэгтэй хүн: цайвар',
    '👩🏼' => 'эмэгтэй хүн: бага зэргийн цайвар',
    '👩🏽' => 'эмэгтэй хүн: дунд зэргийн',
    '👩🏾' => 'эмэгтэй хүн: бордуу',
    '👩🏿' => 'эмэгтэй хүн: бор',
    '🧓🏻' => 'настай хүн: цайвар',
    '🧓🏼' => 'настай хүн: бага зэргийн цайвар',
    '🧓🏽' => 'настай хүн: дунд зэргийн',
    '🧓🏾' => 'настай хүн: бордуу',
    '🧓🏿' => 'настай хүн: бор',
    '👴🏻' => 'ахимаг насны эрэгтэй хүн: цайвар',
    '👴🏼' => 'ахимаг насны эрэгтэй хүн: бага зэргийн цайвар',
    '👴🏽' => 'ахимаг насны эрэгтэй хүн: дунд зэргийн',
    '👴🏾' => 'ахимаг насны эрэгтэй хүн: бордуу',
    '👴🏿' => 'ахимаг насны эрэгтэй хүн: бор',
    '👵🏻' => 'ахимаг насны эмэгтэй хүн: цайвар',
    '👵🏼' => 'ахимаг насны эмэгтэй хүн: бага зэргийн цайвар',
    '👵🏽' => 'ахимаг насны эмэгтэй хүн: дунд зэргийн',
    '👵🏾' => 'ахимаг насны эмэгтэй хүн: бордуу',
    '👵🏿' => 'ахимаг насны эмэгтэй хүн: бор',
    '🙍🏻' => 'урвайсан хүн: цайвар',
    '🙍🏼' => 'урвайсан хүн: бага зэргийн цайвар',
    '🙍🏽' => 'урвайсан хүн: дунд зэргийн',
    '🙍🏾' => 'урвайсан хүн: бордуу',
    '🙍🏿' => 'урвайсан хүн: бор',
    '🙎🏻' => 'гайхсан хүн: цайвар',
    '🙎🏼' => 'гайхсан хүн: бага зэргийн цайвар',
    '🙎🏽' => 'гайхсан хүн: дунд зэргийн',
    '🙎🏾' => 'гайхсан хүн: бордуу',
    '🙎🏿' => 'гайхсан хүн: бор',
    '🙅🏻' => 'үгүйсгэсэн дохио: цайвар',
    '🙅🏼' => 'үгүйсгэсэн дохио: бага зэргийн цайвар',
    '🙅🏽' => 'үгүйсгэсэн дохио: дунд зэргийн',
    '🙅🏾' => 'үгүйсгэсэн дохио: бордуу',
    '🙅🏿' => 'үгүйсгэсэн дохио: бор',
    '🙆🏻' => 'зөвшөөрч байгаа дохио: цайвар',
    '🙆🏼' => 'зөвшөөрч байгаа дохио: бага зэргийн цайвар',
    '🙆🏽' => 'зөвшөөрч байгаа дохио: дунд зэргийн',
    '🙆🏾' => 'зөвшөөрч байгаа дохио: бордуу',
    '🙆🏿' => 'зөвшөөрч байгаа дохио: бор',
    '💁🏻' => 'мэдээллийн ажилтан: цайвар',
    '💁🏼' => 'мэдээллийн ажилтан: бага зэргийн цайвар',
    '💁🏽' => 'мэдээллийн ажилтан: дунд зэргийн',
    '💁🏾' => 'мэдээллийн ажилтан: бордуу',
    '💁🏿' => 'мэдээллийн ажилтан: бор',
    '🙋🏻' => 'баярлаж, гараа өргөсөн хүн: цайвар',
    '🙋🏼' => 'баярлаж, гараа өргөсөн хүн: бага зэргийн цайвар',
    '🙋🏽' => 'баярлаж, гараа өргөсөн хүн: дунд зэргийн',
    '🙋🏾' => 'баярлаж, гараа өргөсөн хүн: бордуу',
    '🙋🏿' => 'баярлаж, гараа өргөсөн хүн: бор',
    '🧏🏻' => 'дүлий хүн: цайвар',
    '🧏🏼' => 'дүлий хүн: бага зэргийн цайвар',
    '🧏🏽' => 'дүлий хүн: дунд зэргийн',
    '🧏🏾' => 'дүлий хүн: бордуу',
    '🧏🏿' => 'дүлий хүн: бор',
    '🙇🏻' => 'бөхийж буй хүн: цайвар',
    '🙇🏼' => 'бөхийж буй хүн: бага зэргийн цайвар',
    '🙇🏽' => 'бөхийж буй хүн: дунд зэргийн',
    '🙇🏾' => 'бөхийж буй хүн: бордуу',
    '🙇🏿' => 'бөхийж буй хүн: бор',
    '🤦🏻' => 'нүүр лүү алгадах: цайвар',
    '🤦🏼' => 'нүүр лүү алгадах: бага зэргийн цайвар',
    '🤦🏽' => 'нүүр лүү алгадах: дунд зэргийн',
    '🤦🏾' => 'нүүр лүү алгадах: бордуу',
    '🤦🏿' => 'нүүр лүү алгадах: бор',
    '🤷🏻' => 'мөрөө хавчих: цайвар',
    '🤷🏼' => 'мөрөө хавчих: бага зэргийн цайвар',
    '🤷🏽' => 'мөрөө хавчих: дунд зэргийн',
    '🤷🏾' => 'мөрөө хавчих: бордуу',
    '🤷🏿' => 'мөрөө хавчих: бор',
    '👮🏻' => 'цагдаа: цайвар',
    '👮🏼' => 'цагдаа: бага зэргийн цайвар',
    '👮🏽' => 'цагдаа: дунд зэргийн',
    '👮🏾' => 'цагдаа: бордуу',
    '👮🏿' => 'цагдаа: бор',
    '🕵🏻' => 'мөрдөгч: цайвар',
    '🕵🏼' => 'мөрдөгч: бага зэргийн цайвар',
    '🕵🏽' => 'мөрдөгч: дунд зэргийн',
    '🕵🏾' => 'мөрдөгч: бордуу',
    '🕵🏿' => 'мөрдөгч: бор',
    '💂🏻' => 'харуул: цайвар',
    '💂🏼' => 'харуул: бага зэргийн цайвар',
    '💂🏽' => 'харуул: дунд зэргийн',
    '💂🏾' => 'харуул: бордуу',
    '💂🏿' => 'харуул: бор',
    '🥷🏻' => 'нинжа: цайвар',
    '🥷🏼' => 'нинжа: бага зэргийн цайвар',
    '🥷🏽' => 'нинжа: дунд зэргийн',
    '🥷🏾' => 'нинжа: бордуу',
    '🥷🏿' => 'нинжа: бор',
    '👷🏻' => 'барилгын ажилтан: цайвар',
    '👷🏼' => 'барилгын ажилтан: бага зэргийн цайвар',
    '👷🏽' => 'барилгын ажилтан: дунд зэргийн',
    '👷🏾' => 'барилгын ажилтан: бордуу',
    '👷🏿' => 'барилгын ажилтан: бор',
    '🫅🏻' => 'титэмтэй хүн: цайвар',
    '🫅🏼' => 'титэмтэй хүн: бага зэргийн цайвар',
    '🫅🏽' => 'титэмтэй хүн: дунд зэргийн',
    '🫅🏾' => 'титэмтэй хүн: бордуу',
    '🫅🏿' => 'титэмтэй хүн: бор',
    '🤴🏻' => 'ханхүү: цайвар',
    '🤴🏼' => 'ханхүү: бага зэргийн цайвар',
    '🤴🏽' => 'ханхүү: дунд зэргийн',
    '🤴🏾' => 'ханхүү: бордуу',
    '🤴🏿' => 'ханхүү: бор',
    '👸🏻' => 'гүнж: цайвар',
    '👸🏼' => 'гүнж: бага зэргийн цайвар',
    '👸🏽' => 'гүнж: дунд зэргийн',
    '👸🏾' => 'гүнж: бордуу',
    '👸🏿' => 'гүнж: бор',
    '👳🏻' => 'тюрбан малгайтай эрэгтэй: цайвар',
    '👳🏼' => 'тюрбан малгайтай эрэгтэй: бага зэргийн цайвар',
    '👳🏽' => 'тюрбан малгайтай эрэгтэй: дунд зэргийн',
    '👳🏾' => 'тюрбан малгайтай эрэгтэй: бордуу',
    '👳🏿' => 'тюрбан малгайтай эрэгтэй: бор',
    '👲🏻' => 'хятад малгай өмссөн эрэгтэй: цайвар',
    '👲🏼' => 'хятад малгай өмссөн эрэгтэй: бага зэргийн цайвар',
    '👲🏽' => 'хятад малгай өмссөн эрэгтэй: дунд зэргийн',
    '👲🏾' => 'хятад малгай өмссөн эрэгтэй: бордуу',
    '👲🏿' => 'хятад малгай өмссөн эрэгтэй: бор',
    '🧕🏻' => 'толгойндоо алчууртай эмэгтэй: цайвар',
    '🧕🏼' => 'толгойндоо алчууртай эмэгтэй: бага зэргийн цайвар',
    '🧕🏽' => 'толгойндоо алчууртай эмэгтэй: дунд зэргийн',
    '🧕🏾' => 'толгойндоо алчууртай эмэгтэй: бордуу',
    '🧕🏿' => 'толгойндоо алчууртай эмэгтэй: бор',
    '🤵🏻' => 'хослолтой хүн: цайвар',
    '🤵🏼' => 'хослолтой хүн: бага зэргийн цайвар',
    '🤵🏽' => 'хослолтой хүн: дунд зэргийн',
    '🤵🏾' => 'хослолтой хүн: бордуу',
    '🤵🏿' => 'хослолтой хүн: бор',
    '👰🏻' => 'гивлүүртэй хүн: цайвар',
    '👰🏼' => 'гивлүүртэй хүн: бага зэргийн цайвар',
    '👰🏽' => 'гивлүүртэй хүн: дунд зэргийн',
    '👰🏾' => 'гивлүүртэй хүн: бордуу',
    '👰🏿' => 'гивлүүртэй хүн: бор',
    '🤰🏻' => 'жирэмсэн эмэгтэй: цайвар',
    '🤰🏼' => 'жирэмсэн эмэгтэй: бага зэргийн цайвар',
    '🤰🏽' => 'жирэмсэн эмэгтэй: дунд зэргийн',
    '🤰🏾' => 'жирэмсэн эмэгтэй: бордуу',
    '🤰🏿' => 'жирэмсэн эмэгтэй: бор',
    '🫃🏻' => 'жирэмсэн эрэгтэй: цайвар',
    '🫃🏼' => 'жирэмсэн эрэгтэй: бага зэргийн цайвар',
    '🫃🏽' => 'жирэмсэн эрэгтэй: дунд зэргийн',
    '🫃🏾' => 'жирэмсэн эрэгтэй: бордуу',
    '🫃🏿' => 'жирэмсэн эрэгтэй: бор',
    '🫄🏻' => 'жирэмсэн хүн: цайвар',
    '🫄🏼' => 'жирэмсэн хүн: бага зэргийн цайвар',
    '🫄🏽' => 'жирэмсэн хүн: дунд зэргийн',
    '🫄🏾' => 'жирэмсэн хүн: бордуу',
    '🫄🏿' => 'жирэмсэн хүн: бор',
    '🤱🏻' => 'хөхөөр хооллох: цайвар',
    '🤱🏼' => 'хөхөөр хооллох: бага зэргийн цайвар',
    '🤱🏽' => 'хөхөөр хооллох: дунд зэргийн',
    '🤱🏾' => 'хөхөөр хооллох: бордуу',
    '🤱🏿' => 'хөхөөр хооллох: бор',
    '👼🏻' => 'нялх сахиусан тэнгэр: цайвар',
    '👼🏼' => 'нялх сахиусан тэнгэр: бага зэргийн цайвар',
    '👼🏽' => 'нялх сахиусан тэнгэр: дунд зэргийн',
    '👼🏾' => 'нялх сахиусан тэнгэр: бордуу',
    '👼🏿' => 'нялх сахиусан тэнгэр: бор',
    '🎅🏻' => 'Санта Клаус: цайвар',
    '🎅🏼' => 'Санта Клаус: бага зэргийн цайвар',
    '🎅🏽' => 'Санта Клаус: дунд зэргийн',
    '🎅🏾' => 'Санта Клаус: бордуу',
    '🎅🏿' => 'Санта Клаус: бор',
    '🤶🏻' => 'Зулсарын хатагтай: цайвар',
    '🤶🏼' => 'Зулсарын хатагтай: бага зэргийн цайвар',
    '🤶🏽' => 'Зулсарын хатагтай: дунд зэргийн',
    '🤶🏾' => 'Зулсарын хатагтай: бордуу',
    '🤶🏿' => 'Зулсарын хатагтай: бор',
    '🦸🏻' => 'супер баатар: цайвар',
    '🦸🏼' => 'супер баатар: бага зэргийн цайвар',
    '🦸🏽' => 'супер баатар: дунд зэргийн',
    '🦸🏾' => 'супер баатар: бордуу',
    '🦸🏿' => 'супер баатар: бор',
    '🦹🏻' => 'ид шидийн хүчтэй эсрэг дүр: цайвар',
    '🦹🏼' => 'ид шидийн хүчтэй эсрэг дүр: бага зэргийн цайвар',
    '🦹🏽' => 'ид шидийн хүчтэй эсрэг дүр: дунд зэргийн',
    '🦹🏾' => 'ид шидийн хүчтэй эсрэг дүр: бордуу',
    '🦹🏿' => 'ид шидийн хүчтэй эсрэг дүр: бор',
    '🧙🏻' => 'шидтэн: цайвар',
    '🧙🏼' => 'шидтэн: бага зэргийн цайвар',
    '🧙🏽' => 'шидтэн: дунд зэргийн',
    '🧙🏾' => 'шидтэн: бордуу',
    '🧙🏿' => 'шидтэн: бор',
    '🧚🏻' => 'дагина: цайвар',
    '🧚🏼' => 'дагина: бага зэргийн цайвар',
    '🧚🏽' => 'дагина: дунд зэргийн',
    '🧚🏾' => 'дагина: бордуу',
    '🧚🏿' => 'дагина: бор',
    '🧛🏻' => 'цус сорогч: цайвар',
    '🧛🏼' => 'цус сорогч: бага зэргийн цайвар',
    '🧛🏽' => 'цус сорогч: дунд зэргийн',
    '🧛🏾' => 'цус сорогч: бордуу',
    '🧛🏿' => 'цус сорогч: бор',
    '🧜🏻' => 'лусын хүн: цайвар',
    '🧜🏼' => 'лусын хүн: бага зэргийн цайвар',
    '🧜🏽' => 'лусын хүн: дунд зэргийн',
    '🧜🏾' => 'лусын хүн: бордуу',
    '🧜🏿' => 'лусын хүн: бор',
    '🧝🏻' => 'одой хүн: цайвар',
    '🧝🏼' => 'одой хүн: бага зэргийн цайвар',
    '🧝🏽' => 'одой хүн: дунд зэргийн',
    '🧝🏾' => 'одой хүн: бордуу',
    '🧝🏿' => 'одой хүн: бор',
    '💆🏻' => 'нүүрний массаж: цайвар',
    '💆🏼' => 'нүүрний массаж: бага зэргийн цайвар',
    '💆🏽' => 'нүүрний массаж: дунд зэргийн',
    '💆🏾' => 'нүүрний массаж: бордуу',
    '💆🏿' => 'нүүрний массаж: бор',
    '💇🏻' => 'үс засалт: цайвар',
    '💇🏼' => 'үс засалт: бага зэргийн цайвар',
    '💇🏽' => 'үс засалт: дунд зэргийн',
    '💇🏾' => 'үс засалт: бордуу',
    '💇🏿' => 'үс засалт: бор',
    '🚶🏻' => 'алхаж буй хүн: цайвар',
    '🚶🏼' => 'алхаж буй хүн: бага зэргийн цайвар',
    '🚶🏽' => 'алхаж буй хүн: дунд зэргийн',
    '🚶🏾' => 'алхаж буй хүн: бордуу',
    '🚶🏿' => 'алхаж буй хүн: бор',
    '🧍🏻' => 'зогсож буй хүн: цайвар',
    '🧍🏼' => 'зогсож буй хүн: бага зэргийн цайвар',
    '🧍🏽' => 'зогсож буй хүн: дунд зэргийн',
    '🧍🏾' => 'зогсож буй хүн: бордуу',
    '🧍🏿' => 'зогсож буй хүн: бор',
    '🧎🏻' => 'сөхөрч суусан хүн: цайвар',
    '🧎🏼' => 'сөхөрч суусан хүн: бага зэргийн цайвар',
    '🧎🏽' => 'сөхөрч суусан хүн: дунд зэргийн',
    '🧎🏾' => 'сөхөрч суусан хүн: бордуу',
    '🧎🏿' => 'сөхөрч суусан хүн: бор',
    '🏃🏻' => 'гүйж байгаа хүн: цайвар',
    '🏃🏼' => 'гүйж байгаа хүн: бага зэргийн цайвар',
    '🏃🏽' => 'гүйж байгаа хүн: дунд зэргийн',
    '🏃🏾' => 'гүйж байгаа хүн: бордуу',
    '🏃🏿' => 'гүйж байгаа хүн: бор',
    '💃🏻' => 'бүжигчин: цайвар',
    '💃🏼' => 'бүжигчин: бага зэргийн цайвар',
    '💃🏽' => 'бүжигчин: дунд зэргийн',
    '💃🏾' => 'бүжигчин: бордуу',
    '💃🏿' => 'бүжигчин: бор',
    '🕺🏻' => 'бүжиглэх залуу: цайвар',
    '🕺🏼' => 'бүжиглэх залуу: бага зэргийн цайвар',
    '🕺🏽' => 'бүжиглэх залуу: дунд зэргийн',
    '🕺🏾' => 'бүжиглэх залуу: бордуу',
    '🕺🏿' => 'бүжиглэх залуу: бор',
    '🕴🏻' => 'хослол өмссөн эрэгтэй: цайвар',
    '🕴🏼' => 'хослол өмссөн эрэгтэй: бага зэргийн цайвар',
    '🕴🏽' => 'хослол өмссөн эрэгтэй: дунд зэргийн',
    '🕴🏾' => 'хослол өмссөн эрэгтэй: бордуу',
    '🕴🏿' => 'хослол өмссөн эрэгтэй: бор',
    '👯🏻' => 'үдэшлэгт байгаа бүсгүй: цайвар',
    '👯🏼' => 'үдэшлэгт байгаа бүсгүй: бага зэргийн цайвар',
    '👯🏽' => 'үдэшлэгт байгаа бүсгүй: дунд зэргийн',
    '👯🏾' => 'үдэшлэгт байгаа бүсгүй: бордуу',
    '👯🏿' => 'үдэшлэгт байгаа бүсгүй: бор',
    '🧖🏻' => 'уур савссан өрөөнд байгаа хүн: цайвар',
    '🧖🏼' => 'уур савссан өрөөнд байгаа хүн: бага зэргийн цайвар',
    '🧖🏽' => 'уур савссан өрөөнд байгаа хүн: дунд зэргийн',
    '🧖🏾' => 'уур савссан өрөөнд байгаа хүн: бордуу',
    '🧖🏿' => 'уур савссан өрөөнд байгаа хүн: бор',
    '🧗🏻' => 'авирагч: цайвар',
    '🧗🏼' => 'авирагч: бага зэргийн цайвар',
    '🧗🏽' => 'авирагч: дунд зэргийн',
    '🧗🏾' => 'авирагч: бордуу',
    '🧗🏿' => 'авирагч: бор',
    '🏇🏻' => 'морин уралдаан: цайвар',
    '🏇🏼' => 'морин уралдаан: бага зэргийн цайвар',
    '🏇🏽' => 'морин уралдаан: дунд зэргийн',
    '🏇🏾' => 'морин уралдаан: бордуу',
    '🏇🏿' => 'морин уралдаан: бор',
    '🏂🏻' => 'цасан дээр гулгагч: цайвар',
    '🏂🏼' => 'цасан дээр гулгагч: бага зэргийн цайвар',
    '🏂🏽' => 'цасан дээр гулгагч: дунд зэргийн',
    '🏂🏾' => 'цасан дээр гулгагч: бордуу',
    '🏂🏿' => 'цасан дээр гулгагч: бор',
    '🏌🏻' => 'гольф: цайвар',
    '🏌🏼' => 'гольф: бага зэргийн цайвар',
    '🏌🏽' => 'гольф: дунд зэргийн',
    '🏌🏾' => 'гольф: бордуу',
    '🏌🏿' => 'гольф: бор',
    '🏄🏻' => 'серфингээр гулгаж байгаа хүн: цайвар',
    '🏄🏼' => 'серфингээр гулгаж байгаа хүн: бага зэргийн цайвар',
    '🏄🏽' => 'серфингээр гулгаж байгаа хүн: дунд зэргийн',
    '🏄🏾' => 'серфингээр гулгаж байгаа хүн: бордуу',
    '🏄🏿' => 'серфингээр гулгаж байгаа хүн: бор',
    '🚣🏻' => 'завь сэлүүрдэж буй хүн: цайвар',
    '🚣🏼' => 'завь сэлүүрдэж буй хүн: бага зэргийн цайвар',
    '🚣🏽' => 'завь сэлүүрдэж буй хүн: дунд зэргийн',
    '🚣🏾' => 'завь сэлүүрдэж буй хүн: бордуу',
    '🚣🏿' => 'завь сэлүүрдэж буй хүн: бор',
    '🏊🏻' => 'усанд сэлэгч: цайвар',
    '🏊🏼' => 'усанд сэлэгч: бага зэргийн цайвар',
    '🏊🏽' => 'усанд сэлэгч: дунд зэргийн',
    '🏊🏾' => 'усанд сэлэгч: бордуу',
    '🏊🏿' => 'усанд сэлэгч: бор',
    '⛹🏻' => 'бөмбөгтэй хүн: цайвар',
    '⛹🏼' => 'бөмбөгтэй хүн: бага зэргийн цайвар',
    '⛹🏽' => 'бөмбөгтэй хүн: дунд зэргийн',
    '⛹🏾' => 'бөмбөгтэй хүн: бордуу',
    '⛹🏿' => 'бөмбөгтэй хүн: бор',
    '🏋🏻' => 'хүндийн жингийн тамирчин: цайвар',
    '🏋🏼' => 'хүндийн жингийн тамирчин: бага зэргийн цайвар',
    '🏋🏽' => 'хүндийн жингийн тамирчин: дунд зэргийн',
    '🏋🏾' => 'хүндийн жингийн тамирчин: бордуу',
    '🏋🏿' => 'хүндийн жингийн тамирчин: бор',
    '🚴🏻' => 'дугуйчин: цайвар',
    '🚴🏼' => 'дугуйчин: бага зэргийн цайвар',
    '🚴🏽' => 'дугуйчин: дунд зэргийн',
    '🚴🏾' => 'дугуйчин: бордуу',
    '🚴🏿' => 'дугуйчин: бор',
    '🚵🏻' => 'уулын дугуйчин: цайвар',
    '🚵🏼' => 'уулын дугуйчин: бага зэргийн цайвар',
    '🚵🏽' => 'уулын дугуйчин: дунд зэргийн',
    '🚵🏾' => 'уулын дугуйчин: бордуу',
    '🚵🏿' => 'уулын дугуйчин: бор',
    '🤸🏻' => 'сальто эргэх: цайвар',
    '🤸🏼' => 'сальто эргэх: бага зэргийн цайвар',
    '🤸🏽' => 'сальто эргэх: дунд зэргийн',
    '🤸🏾' => 'сальто эргэх: бордуу',
    '🤸🏿' => 'сальто эргэх: бор',
    '🤼🏻' => 'бөх: цайвар',
    '🤼🏼' => 'бөх: бага зэргийн цайвар',
    '🤼🏽' => 'бөх: дунд зэргийн',
    '🤼🏾' => 'бөх: бордуу',
    '🤼🏿' => 'бөх: бор',
    '🤽🏻' => 'усны поло: цайвар',
    '🤽🏼' => 'усны поло: бага зэргийн цайвар',
    '🤽🏽' => 'усны поло: дунд зэргийн',
    '🤽🏾' => 'усны поло: бордуу',
    '🤽🏿' => 'усны поло: бор',
    '🤾🏻' => 'гар бөмбөг тоглож буй хүн: цайвар',
    '🤾🏼' => 'гар бөмбөг тоглож буй хүн: бага зэргийн цайвар',
    '🤾🏽' => 'гар бөмбөг тоглож буй хүн: дунд зэргийн',
    '🤾🏾' => 'гар бөмбөг тоглож буй хүн: бордуу',
    '🤾🏿' => 'гар бөмбөг тоглож буй хүн: бор',
    '🤹🏻' => 'жонглёрдох: цайвар',
    '🤹🏼' => 'жонглёрдох: бага зэргийн цайвар',
    '🤹🏽' => 'жонглёрдох: дунд зэргийн',
    '🤹🏾' => 'жонглёрдох: бордуу',
    '🤹🏿' => 'жонглёрдох: бор',
    '🧘🏻' => 'лотус байрлалтай хүн: цайвар',
    '🧘🏼' => 'лотус байрлалтай хүн: бага зэргийн цайвар',
    '🧘🏽' => 'лотус байрлалтай хүн: дунд зэргийн',
    '🧘🏾' => 'лотус байрлалтай хүн: бордуу',
    '🧘🏿' => 'лотус байрлалтай хүн: бор',
    '🛀🏻' => 'усанд орж байгаа хүн: цайвар',
    '🛀🏼' => 'усанд орж байгаа хүн: бага зэргийн цайвар',
    '🛀🏽' => 'усанд орж байгаа хүн: дунд зэргийн',
    '🛀🏾' => 'усанд орж байгаа хүн: бордуу',
    '🛀🏿' => 'усанд орж байгаа хүн: бор',
    '🛌🏻' => 'орондоо байгаа хүн: цайвар',
    '🛌🏼' => 'орондоо байгаа хүн: бага зэргийн цайвар',
    '🛌🏽' => 'орондоо байгаа хүн: дунд зэргийн',
    '🛌🏾' => 'орондоо байгаа хүн: бордуу',
    '🛌🏿' => 'орондоо байгаа хүн: бор',
    '👭🏻' => 'хөтлөлцсөн хоёр эмэгтэй: цайвар',
    '👭🏼' => 'хөтлөлцсөн хоёр эмэгтэй: бага зэргийн цайвар',
    '👭🏽' => 'хөтлөлцсөн хоёр эмэгтэй: дунд зэргийн',
    '👭🏾' => 'хөтлөлцсөн хоёр эмэгтэй: бордуу',
    '👭🏿' => 'хөтлөлцсөн хоёр эмэгтэй: бор',
    '👫🏻' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: цайвар',
    '👫🏼' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бага зэргийн цайвар',
    '👫🏽' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: дунд зэргийн',
    '👫🏾' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бордуу',
    '👫🏿' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос: бор',
    '👬🏻' => 'хөтлөлцсөн хоёр эрэгтэй: цайвар',
    '👬🏼' => 'хөтлөлцсөн хоёр эрэгтэй: бага зэргийн цайвар',
    '👬🏽' => 'хөтлөлцсөн хоёр эрэгтэй: дунд зэргийн',
    '👬🏾' => 'хөтлөлцсөн хоёр эрэгтэй: бордуу',
    '👬🏿' => 'хөтлөлцсөн хоёр эрэгтэй: бор',
    '💏🏻' => 'үнсэлт: цайвар',
    '💏🏼' => 'үнсэлт: бага зэргийн цайвар',
    '💏🏽' => 'үнсэлт: дунд зэргийн',
    '💏🏾' => 'үнсэлт: бордуу',
    '💏🏿' => 'үнсэлт: бор',
    '💑🏻' => 'дундаа зүрхтэй хосууд: цайвар',
    '💑🏼' => 'дундаа зүрхтэй хосууд: бага зэргийн цайвар',
    '💑🏽' => 'дундаа зүрхтэй хосууд: дунд зэргийн',
    '💑🏾' => 'дундаа зүрхтэй хосууд: бордуу',
    '💑🏿' => 'дундаа зүрхтэй хосууд: бор',
    '🇦🇨' => 'туг: Асенсион арал',
    '🇦🇩' => 'туг: Андорра',
    '🇦🇪' => 'туг: Арабын Нэгдсэн Эмират Улс',
    '🇦🇫' => 'туг: Афганистан',
    '🇦🇬' => 'туг: Антигуа ба Барбуда',
    '🇦🇮' => 'туг: Ангилья',
    '🇦🇱' => 'туг: Албани',
    '🇦🇲' => 'туг: Армени',
    '🇦🇴' => 'туг: Ангол',
    '🇦🇶' => 'туг: Антарктид',
    '🇦🇷' => 'туг: Аргентин',
    '🇦🇸' => 'туг: Америкийн Самоа',
    '🇦🇹' => 'туг: Австри',
    '🇦🇺' => 'туг: Австрали',
    '🇦🇼' => 'туг: Аруба',
    '🇦🇽' => 'туг: Аландын арлууд',
    '🇦🇿' => 'туг: Азербайжан',
    '🇧🇦' => 'туг: Босни-Херцеговин',
    '🇧🇧' => 'туг: Барбадос',
    '🇧🇩' => 'туг: Бангладеш',
    '🇧🇪' => 'туг: Бельги',
    '🇧🇫' => 'туг: Буркина Фасо',
    '🇧🇬' => 'туг: Болгар',
    '🇧🇭' => 'туг: Бахрейн',
    '🇧🇮' => 'туг: Бурунди',
    '🇧🇯' => 'туг: Бенин',
    '🇧🇱' => 'туг: Сент-Бартельми',
    '🇧🇲' => 'туг: Бермуда',
    '🇧🇳' => 'туг: Бруней',
    '🇧🇴' => 'туг: Боливи',
    '🇧🇶' => 'туг: Карибын Нидерланд',
    '🇧🇷' => 'туг: Бразил',
    '🇧🇸' => 'туг: Багамын арлууд',
    '🇧🇹' => 'туг: Бутан',
    '🇧🇻' => 'туг: Буве арал',
    '🇧🇼' => 'туг: Ботсван',
    '🇧🇾' => 'туг: Беларусь',
    '🇧🇿' => 'туг: Белизе',
    '🇨🇦' => 'туг: Канад',
    '🇨🇨' => 'туг: Кокос (Кийлинг) арлууд',
    '🇨🇩' => 'туг: Конго-Киншаса',
    '🇨🇫' => 'туг: Төв Африкийн Бүгд Найрамдах Улс',
    '🇨🇬' => 'туг: Конго-Браззавиль',
    '🇨🇭' => 'туг: Швейцар',
    '🇨🇮' => 'туг: Кот-д’Ивуар',
    '🇨🇰' => 'туг: Күүкийн арлууд',
    '🇨🇱' => 'туг: Чили',
    '🇨🇲' => 'туг: Камерун',
    '🇨🇳' => 'туг: Хятад',
    '🇨🇴' => 'туг: Колумби',
    '🇨🇵' => 'туг: Клиппертон арал',
    '🇨🇶' => 'туг: Сарк',
    '🇨🇷' => 'туг: Коста-Рика',
    '🇨🇺' => 'туг: Куба',
    '🇨🇻' => 'туг: Кабо-Верде',
    '🇨🇼' => 'туг: Кюрасао',
    '🇨🇽' => 'туг: Зул сарын арал',
    '🇨🇾' => 'туг: Кипр',
    '🇨🇿' => 'туг: Чех',
    '🇩🇪' => 'туг: Герман',
    '🇩🇬' => 'туг: Диего Гарсиа',
    '🇩🇯' => 'туг: Джибути',
    '🇩🇰' => 'туг: Дани',
    '🇩🇲' => 'туг: Доминика',
    '🇩🇴' => 'туг: Бүгд Найрамдах Доминикан Улс',
    '🇩🇿' => 'туг: Алжир',
    '🇪🇦' => 'туг: Сеута ба Мелилья',
    '🇪🇨' => 'туг: Эквадор',
    '🇪🇪' => 'туг: Эстони',
    '🇪🇬' => 'туг: Египет',
    '🇪🇭' => 'туг: Баруун Сахар',
    '🇪🇷' => 'туг: Эритрей',
    '🇪🇸' => 'туг: Испани',
    '🇪🇹' => 'туг: Этиоп',
    '🇪🇺' => 'туг: Европын Холбоо',
    '🇫🇮' => 'туг: Финланд',
    '🇫🇯' => 'туг: Фижи',
    '🇫🇰' => 'туг: Фолклендийн арлууд',
    '🇫🇲' => 'туг: Микронези',
    '🇫🇴' => 'туг: Фарерын арлууд',
    '🇫🇷' => 'туг: Франц',
    '🇬🇦' => 'туг: Габон',
    '🇬🇧' => 'туг: Их Британи',
    '🇬🇩' => 'туг: Гренада',
    '🇬🇪' => 'туг: Гүрж',
    '🇬🇫' => 'туг: Францын Гвиана',
    '🇬🇬' => 'туг: Гернси',
    '🇬🇭' => 'туг: Гана',
    '🇬🇮' => 'туг: Гибралтар',
    '🇬🇱' => 'туг: Гренланд',
    '🇬🇲' => 'туг: Гамби',
    '🇬🇳' => 'туг: Гвиней',
    '🇬🇵' => 'туг: Гваделуп',
    '🇬🇶' => 'туг: Экваторын Гвиней',
    '🇬🇷' => 'туг: Грек',
    '🇬🇸' => 'туг: Өмнөд Жоржиа ба Өмнөд Сэндвичийн арлууд',
    '🇬🇹' => 'туг: Гватемал',
    '🇬🇺' => 'туг: Гуам',
    '🇬🇼' => 'туг: Гвиней-Бисау',
    '🇬🇾' => 'туг: Гайана',
    '🇭🇰' => 'туг: БНХАУ-ын Тусгай захиргааны бүс Хонг-Конг',
    '🇭🇲' => 'туг: Херд ба Макдональдийн арлууд',
    '🇭🇳' => 'туг: Гондурас',
    '🇭🇷' => 'туг: Хорват',
    '🇭🇹' => 'туг: Гаити',
    '🇭🇺' => 'туг: Унгар',
    '🇮🇨' => 'туг: Канарын арлууд',
    '🇮🇩' => 'туг: Индонез',
    '🇮🇪' => 'туг: Ирланд',
    '🇮🇱' => 'туг: Израил',
    '🇮🇲' => 'туг: Мэн Арал',
    '🇮🇳' => 'туг: Энэтхэг',
    '🇮🇴' => 'туг: Британийн харьяа Энэтхэгийн далай дахь нутаг дэвсгэр',
    '🇮🇶' => 'туг: Ирак',
    '🇮🇷' => 'туг: Иран',
    '🇮🇸' => 'туг: Исланд',
    '🇮🇹' => 'туг: Итали',
    '🇯🇪' => 'туг: Жерси',
    '🇯🇲' => 'туг: Ямайка',
    '🇯🇴' => 'туг: Йордан',
    '🇯🇵' => 'туг: Япон',
    '🇰🇪' => 'туг: Кени',
    '🇰🇬' => 'туг: Киргиз',
    '🇰🇭' => 'туг: Камбож',
    '🇰🇮' => 'туг: Кирибати',
    '🇰🇲' => 'туг: Коморын арлууд',
    '🇰🇳' => 'туг: Сент-Киттс ба Невис',
    '🇰🇵' => 'туг: Хойд Солонгос',
    '🇰🇷' => 'туг: Өмнөд Солонгос',
    '🇰🇼' => 'туг: Кувейт',
    '🇰🇾' => 'туг: Кайманы арлууд',
    '🇰🇿' => 'туг: Казахстан',
    '🇱🇦' => 'туг: Лаос',
    '🇱🇧' => 'туг: Ливан',
    '🇱🇨' => 'туг: Сент Люсиа',
    '🇱🇮' => 'туг: Лихтенштайн',
    '🇱🇰' => 'туг: Шри-Ланка',
    '🇱🇷' => 'туг: Либери',
    '🇱🇸' => 'туг: Лесото',
    '🇱🇹' => 'туг: Литва',
    '🇱🇺' => 'туг: Люксембург',
    '🇱🇻' => 'туг: Латви',
    '🇱🇾' => 'туг: Ливи',
    '🇲🇦' => 'туг: Морокко',
    '🇲🇨' => 'туг: Монако',
    '🇲🇩' => 'туг: Молдова',
    '🇲🇪' => 'туг: Монтенегро',
    '🇲🇫' => 'туг: Сент-Мартин',
    '🇲🇬' => 'туг: Мадагаскар',
    '🇲🇭' => 'туг: Маршаллын арлууд',
    '🇲🇰' => 'туг: Хойд Македон',
    '🇲🇱' => 'туг: Мали',
    '🇲🇲' => 'туг: Мьянмар',
    '🇲🇳' => 'туг: Монгол',
    '🇲🇴' => 'туг: БНХАУ-ын Тусгай захиргааны бүс Макао',
    '🇲🇵' => 'туг: Хойд Марианы арлууд',
    '🇲🇶' => 'туг: Мартиник',
    '🇲🇷' => 'туг: Мавритани',
    '🇲🇸' => 'туг: Монтсеррат',
    '🇲🇹' => 'туг: Мальта',
    '🇲🇺' => 'туг: Маврикий',
    '🇲🇻' => 'туг: Мальдив',
    '🇲🇼' => 'туг: Малави',
    '🇲🇽' => 'туг: Мексик',
    '🇲🇾' => 'туг: Малайз',
    '🇲🇿' => 'туг: Мозамбик',
    '🇳🇦' => 'туг: Намиби',
    '🇳🇨' => 'туг: Шинэ Каледони',
    '🇳🇪' => 'туг: Нигер',
    '🇳🇫' => 'туг: Норфолк арал',
    '🇳🇬' => 'туг: Нигери',
    '🇳🇮' => 'туг: Никарагуа',
    '🇳🇱' => 'туг: Нидерланд',
    '🇳🇴' => 'туг: Норвег',
    '🇳🇵' => 'туг: Балба',
    '🇳🇷' => 'туг: Науру',
    '🇳🇺' => 'туг: Ниуэ',
    '🇳🇿' => 'туг: Шинэ Зеланд',
    '🇴🇲' => 'туг: Оман',
    '🇵🇦' => 'туг: Панам',
    '🇵🇪' => 'туг: Перу',
    '🇵🇫' => 'туг: Францын Полинез',
    '🇵🇬' => 'туг: Папуа Шинэ Гвиней',
    '🇵🇭' => 'туг: Филиппин',
    '🇵🇰' => 'туг: Пакистан',
    '🇵🇱' => 'туг: Польш',
    '🇵🇲' => 'туг: Сент-Пьер ба Микело',
    '🇵🇳' => 'туг: Питкэрн арлууд',
    '🇵🇷' => 'туг: Пуэрто-Рико',
    '🇵🇸' => 'туг: Палестины нутаг дэвсгэр',
    '🇵🇹' => 'туг: Португал',
    '🇵🇼' => 'туг: Палау',
    '🇵🇾' => 'туг: Парагвай',
    '🇶🇦' => 'туг: Катар',
    '🇷🇪' => 'туг: Реюнион',
    '🇷🇴' => 'туг: Румын',
    '🇷🇸' => 'туг: Серби',
    '🇷🇺' => 'туг: Орос',
    '🇷🇼' => 'туг: Руанда',
    '🇸🇦' => 'туг: Саудын Араб',
    '🇸🇧' => 'туг: Соломоны арлууд',
    '🇸🇨' => 'туг: Сейшелийн арлууд',
    '🇸🇩' => 'туг: Судан',
    '🇸🇪' => 'туг: Швед',
    '🇸🇬' => 'туг: Сингапур',
    '🇸🇭' => 'туг: Сент Хелена',
    '🇸🇮' => 'туг: Словени',
    '🇸🇯' => 'туг: Свалбард ба Ян Майен',
    '🇸🇰' => 'туг: Словак',
    '🇸🇱' => 'туг: Сьерра-Леоне',
    '🇸🇲' => 'туг: Сан-Марино',
    '🇸🇳' => 'туг: Сенегал',
    '🇸🇴' => 'туг: Сомали',
    '🇸🇷' => 'туг: Суринам',
    '🇸🇸' => 'туг: Өмнөд Судан',
    '🇸🇹' => 'туг: Сан-Томе ба Принсипи',
    '🇸🇻' => 'туг: Эль Сальвадор',
    '🇸🇽' => 'туг: Синт Мартен',
    '🇸🇾' => 'туг: Сири',
    '🇸🇿' => 'туг: Эсватини',
    '🇹🇦' => 'туг: Тристан да Кунъя',
    '🇹🇨' => 'туг: Турк ба Кайкосын Арлууд',
    '🇹🇩' => 'туг: Чад',
    '🇹🇫' => 'туг: Францын өмнөд газар нутаг',
    '🇹🇬' => 'туг: Того',
    '🇹🇭' => 'туг: Тайланд',
    '🇹🇯' => 'туг: Тажикистан',
    '🇹🇰' => 'туг: Токелау',
    '🇹🇱' => 'туг: Тимор-Лесте',
    '🇹🇲' => 'туг: Туркменистан',
    '🇹🇳' => 'туг: Тунис',
    '🇹🇴' => 'туг: Тонга',
    '🇹🇷' => 'туг: Турк',
    '🇹🇹' => 'туг: Тринидад ба Тобаго',
    '🇹🇻' => 'туг: Тувалу',
    '🇹🇼' => 'туг: Тайвань',
    '🇹🇿' => 'туг: Танзани',
    '🇺🇦' => 'туг: Украин',
    '🇺🇬' => 'туг: Уганда',
    '🇺🇲' => 'туг: Америкийн Нэгдсэн Улсын бага арлууд',
    '🇺🇳' => 'туг: Нэгдсэн Үндэстний Байгууллага',
    '🇺🇸' => 'туг: Америкийн Нэгдсэн Улс',
    '🇺🇾' => 'туг: Уругвай',
    '🇺🇿' => 'туг: Узбекистан',
    '🇻🇦' => 'туг: Ватикан хот улс',
    '🇻🇨' => 'туг: Сент-Винсент ба Гренадин',
    '🇻🇪' => 'туг: Венесуэл',
    '🇻🇬' => 'туг: Британийн Виржиний арлууд',
    '🇻🇮' => 'туг: АНУ-ын Виржиний арлууд',
    '🇻🇳' => 'туг: Вьетнам',
    '🇻🇺' => 'туг: Вануату',
    '🇼🇫' => 'туг: Уоллис ба Футуна',
    '🇼🇸' => 'туг: Самоа',
    '🇽🇰' => 'туг: Косово',
    '🇾🇪' => 'туг: Йемен',
    '🇾🇹' => 'туг: Майотта',
    '🇿🇦' => 'туг: Өмнөд Африк',
    '🇿🇲' => 'туг: Замби',
    '🇿🇼' => 'туг: Зимбабве',
    '🔟' => 'түлхүүрийн гэр: 10',
    '🏻' => 'цайвар',
    '🏼' => 'бага зэргийн цайвар',
    '🏽' => 'дунд зэргийн',
    '🏾' => 'бордуу',
    '🏿' => 'бор',
    '😀' => 'инээсэн царай',
    '😃' => 'амаа ангайлган инээж буй царай',
    '😄' => 'нүдээ онийлгон маасайтал инээсэн царай',
    '😁' => 'нүдээ онийлгон инээсэн царай',
    '😆' => 'нүдээ тас аниж маасайтал инээсэн царай',
    '😅' => 'хөлсөө гартал маасайтал инээсэн царай',
    '🤣' => 'инээж, газар өнхрөх царай',
    '😂' => 'нулимсаа гартал инээж буй царай',
    '🙂' => 'бага зэрэг инээмсэглэсэн царай',
    '🙃' => 'доошоо харсан царай',
    '🫠' => 'хайлж буй царай',
    '😉' => 'ирмэж байгаа царай',
    '😊' => 'онийж инээмсэглэсэн царай',
    '😇' => 'инээмсэглэж буй сахиусан тэнгэрийн царай',
    '🥰' => 'инээж буй зүрхтэй царай',
    '😍' => 'бундан нүдтэй инээмсэглэж байгаа царай',
    '🤩' => 'одон нүдтэй инээж буй царай',
    '😘' => 'үнсэлт илгээж байгаа царай',
    '😗' => 'үнсэж байгаа царай',
    '☺' => 'ичингүйрсэн инээмсэглэл',
    '😚' => 'ичингүйрэн үнсэж байгаа царай',
    '😙' => 'нүдээ аниад үнсэж байгаа царай',
    '🥲' => 'нулимстай инээсэн царай',
    '😋' => 'амттай хоол илэрхийлсэн царай',
    '😛' => 'хэлээ гаргаж буй царай',
    '😜' => 'хэлээ гаргаад, нүдээ ирмэж буй царай',
    '🤪' => 'галзуу царай',
    '😝' => 'хэлээ гаргаад, нүдээ тас аньсан царай',
    '🤑' => 'амандаа мөнгөтэй царай',
    '🤗' => 'тэвэрч байгаа царай',
    '🤭' => 'амаа гараараа тагласан царай',
    '🫢' => 'нүдээ томруулж, гараараа амаа дарсан байдалтай царай',
    '🫣' => 'нүдээрээ сэм харсан царай',
    '🤫' => 'чишш гэсэн царай',
    '🤔' => 'бодож байгаа царай',
    '🫡' => 'ёслон мэндчилж буй царай',
    '🤐' => 'амаа үдсэн царай',
    '🤨' => 'хөмсгөө өргөсөн царай',
    '😐' => 'төвийг сахисан царай',
    '😑' => 'сэтгэл хөдлөлийн илрэлгүй царай',
    '😶' => 'амгүй царай',
    '🫥' => 'цэгэн хүрээтэй царай',
    '😏' => 'ёжтой царай',
    '😒' => 'үл тоосон царай',
    '🙄' => 'нүдээ эргэлдүүлэх царай',
    '😬' => 'ярвалзсан царай',
    '🤥' => 'худалч царай',
    '🫨' => 'чичирч буй царай',
    '😌' => 'тайвширсан царай',
    '😔' => 'гомдсон царай',
    '😪' => 'нойрмоглож буй царай',
    '🤤' => 'унжгар царай',
    '😴' => 'унтаж буй царай',
    '🫩' => 'доогуураа хөхөрсөн нүдтэй царай',
    '😷' => 'маск зүүсэн царай',
    '🤒' => 'халуунтай байгаа царай',
    '🤕' => 'толгойндоо шархны боолт зүүсэн царай',
    '🤢' => 'дурамжхан царай',
    '🤮' => 'бөөлжсөн царай',
    '🤧' => 'найтаах царай',
    '🥵' => 'халууцсан царай',
    '🥶' => 'даарсан царай',
    '🥴' => 'толгой эргэсэн царай',
    '😵' => 'толгой нь эргэсэн царай',
    '🤯' => 'толгой дэлбэрэх',
    '🤠' => 'ковбой малгайтай царай',
    '🥳' => 'үдэшлэг дээр баярлаж буй царай',
    '🥸' => 'өнгөлөн далдалсан царай',
    '😎' => 'нарны шил зүүж инээмсэглэсэн царай',
    '🤓' => 'ухаантай царай',
    '🧐' => 'нэг нүдний нүдний шил зүүсэн царай',
    '😕' => 'эргэлзсэн царай',
    '🫤' => 'муруй амтай царай',
    '😟' => 'санаа зовсон царай',
    '🙁' => 'бага зэрэг барайсан царай',
    '☹' => 'баргар царай',
    '😮' => 'цочирдсон царай',
    '😯' => 'гайхширсан царай',
    '😲' => 'мэл гайхсан царай',
    '😳' => 'ичсэн царай',
    '🫪' => 'гайхсан царай',
    '🥺' => 'ямар нэг юм хүсч буй царай',
    '🥹' => 'нулимсаа барьж буй царай',
    '😦' => 'гайхсан дурамжхан царай',
    '😧' => 'шаналсан царай',
    '😨' => 'айсан царай',
    '😰' => 'амаа ангайж, хүйтэн хөлс чахарсан царай',
    '😥' => 'урам хугарсан ч тайвширсан царай',
    '😢' => 'уйлж байгаа царай',
    '😭' => 'цурхиран уйлж байгаа царай',
    '😱' => 'айгаад орилж байгаа царай',
    '😖' => 'будилсан царай',
    '😣' => 'цөхрөлтгүй царай',
    '😞' => 'урам хугарсан царай',
    '😓' => 'хүйтэн хөлс даасан царай',
    '😩' => 'залхсан царай',
    '😫' => 'ядарсан царай',
    '🥱' => 'эвшээж буй царай',
    '😤' => 'их уурласан царай',
    '😡' => 'маш их уурласан царай',
    '😠' => 'ууртай царай',
    '🤬' => 'тэмдэгтүүд амандаа зуусан царай',
    '😈' => 'хорноор инээж байгаа царай',
    '👿' => 'чөтгөр',
    '💀' => 'гавал',
    '☠' => 'гавлын яс',
    '💩' => 'овоолсон баас',
    '🤡' => 'алиалагчийн царай',
    '👹' => 'аварга махчин',
    '👺' => 'шүтээн, онгод',
    '👻' => 'сүнс',
    '👽' => 'харь гаригийн амьтан',
    '👾' => 'харь гаригийн мангас',
    '🤖' => 'робот царай',
    '😺' => 'амаа ангайгаад инээж байгаа муур',
    '😸' => 'нүдээрээ инээж байгаа муур',
    '😹' => 'нулимсаа гартал инээж байгаа муур',
    '😻' => 'зүрхэн нүдтэй инээж байгаа муур',
    '😼' => 'жоготой инээмсэглэж байгаа муур',
    '😽' => 'нүдээ аниад үнсэж байгаа муур',
    '🙀' => 'гайхшралд орсон муур',
    '😿' => 'уйлж байгаа муур',
    '😾' => 'урвагар царайлсан муур',
    '🙈' => 'муу үйлийг харахгүй',
    '🙉' => 'муу үйлийг сонсохгүй',
    '🙊' => 'муу үйлийг хэлэхгүй',
    '💌' => 'хайрын захиа',
    '💘' => 'сумаар харвасан зүрх',
    '💝' => 'туузтай зүрх',
    '💖' => 'гялалзаж байгаа зүрх',
    '💗' => 'лавширч буй зүрх',
    '💓' => 'цохилж буй зүрх',
    '💞' => 'эргэлдэж буй зүрх',
    '💕' => 'хоёр зүрх',
    '💟' => 'зүрхэн чимэглэл',
    '❣' => 'зүрхэн хэлбэртэй анхааруулгын тэмдэг',
    '💔' => 'шархалсан зүрх',
    '❤' => 'улаан зүрх',
    '🩷' => 'ягаан зүрх',
    '🧡' => 'улбар шар зүрх',
    '💛' => 'шар зүрх',
    '💚' => 'ногоон зүрх',
    '💙' => 'цэнхэр зүрх',
    '🩵' => 'цайвар цэнхэр зүрх',
    '💜' => 'нил ягаан зүрх',
    '🤎' => 'хүрэн зүрх',
    '🖤' => 'хар зүрх',
    '🩶' => 'саарал зүрх',
    '🤍' => 'цагаан зүрх',
    '💋' => 'үнсэлтийн мөр',
    '💯' => 'зуун оноо',
    '💢' => 'уурлаж байгаа дүрслэл',
    '🫯' => 'бөөгнөрсөн зодолдоон',
    '💥' => 'мөргөлдөөн',
    '💫' => 'төөрөгдүүлсэн',
    '💦' => 'хөлсний дусал',
    '💨' => 'эрч хүчтэй',
    '🕳' => 'нүх',
    '💬' => 'ярианы хэсэг',
    '🗨' => 'зүүн талын ярианы хэсэг',
    '🗯' => 'баруун талын ууртай ярианы хэсэг',
    '💭' => 'бодож байгаа бөмбөлөг',
    '💤' => 'унтаж байгаа',
    '👋' => 'мэндчилж байгаа гар',
    '🤚' => 'өргөсөн гарын ар',
    '🖐' => 'хуруунуудаа салгасан гарын алга',
    '✋' => 'өргөсөн гар',
    '🖖' => 'вулкан мэндчилгээ',
    '🫱' => 'баруун тийш чиглэсэн гар',
    '🫲' => 'зүүн тийш чиглэсэн гар',
    '🫳' => 'алга нь доошоо харсан гар',
    '🫴' => 'алга нь дээшээ харсан гар',
    '🫷' => 'зүүн тийш түлхсэн гар',
    '🫸' => 'баруун тийш түлхсэн гар',
    '👌' => 'ок',
    '🤌' => 'гарын хуруунуудаа нийлүүлж чимхэх',
    '🤏' => 'чимхэж буй гар',
    '✌' => 'амжилт хүссэн гар',
    '🤞' => 'солбисон хуруу',
    '🫰' => 'долоовор хуруу, эрхий хуруугаа зөрүүлж барьсан гар',
    '🤟' => 'хайртай гэсэн дохио',
    '🤘' => 'эвэрний тэмдэг',
    '🤙' => 'залгаарай дохио',
    '👈' => 'зүүн зүг зааж байгаа',
    '👉' => 'баруун зүг зааж байгаа',
    '👆' => 'дээшээ зааж буй алганы ар тал',
    '🖕' => 'дунд хуруу',
    '👇' => 'доошоо зааж буй алганы ард тал',
    '☝' => 'дээшээг зааж байгаа',
    '🫵' => 'үзэгч рүү долоовор хуруугаар заах',
    '👍' => 'сайн',
    '👎' => 'муу',
    '✊' => 'зангидсан гар',
    '👊' => 'цохилт',
    '🤛' => 'зүүн харсан зангидсан гар',
    '🤜' => 'баруун харсан зангидсан гар',
    '👏' => 'алга таших',
    '🙌' => 'гараа өргөж байгаа хүн',
    '🫶' => 'зүрхэн гар',
    '👐' => 'дэлгэсэн алга',
    '🤲' => 'алгаа хавсрах',
    '🤝' => 'гар барилт',
    '🙏' => 'хавсарсан гарын алга',
    '✍' => 'бичиж байгаа гар',
    '💅' => 'хумсаа будах',
    '🤳' => 'селфи',
    '💪' => 'гарын булчин',
    '🦾' => 'хиймэл механик гар',
    '🦿' => 'хиймэл механик хөл',
    '🦵' => 'хөл',
    '🦶' => 'тавхай',
    '👂' => 'чих',
    '🦻' => 'сонсголын аппараттай чих',
    '👃' => 'хамар',
    '🧠' => 'тархи',
    '🫀' => 'зүрх',
    '🫁' => 'уушиг',
    '🦷' => 'шүд',
    '🦴' => 'яс',
    '👀' => 'хоёр нүд',
    '👁' => 'нүд',
    '👅' => 'хэл',
    '👄' => 'ам',
    '🫦' => 'хазсан уруул',
    '👶' => 'нялх хүүхэд',
    '🧒' => 'хүүхэд',
    '👦' => 'эрэгтэй хүүхэд',
    '👧' => 'эмэгтэй хүүхэд',
    '🧑' => 'том хүн',
    '👱' => 'шаргал үстэй хүн',
    '👨' => 'эрэгтэй хүн',
    '🧔' => 'сахалтай хүн',
    '👩' => 'эмэгтэй хүн',
    '🧓' => 'настай хүн',
    '👴' => 'ахимаг насны эрэгтэй хүн',
    '👵' => 'ахимаг насны эмэгтэй хүн',
    '🙍' => 'урвайсан хүн',
    '🙎' => 'гайхсан хүн',
    '🙅' => 'үгүйсгэсэн дохио',
    '🙆' => 'зөвшөөрч байгаа дохио',
    '💁' => 'мэдээллийн ажилтан',
    '🙋' => 'баярлаж, гараа өргөсөн хүн',
    '🧏' => 'дүлий хүн',
    '🙇' => 'бөхийж буй хүн',
    '🤦' => 'нүүр лүү алгадах',
    '🤷' => 'мөрөө хавчих',
    '👮' => 'цагдаа',
    '🕵' => 'мөрдөгч',
    '💂' => 'харуул',
    '🥷' => 'нинжа',
    '👷' => 'барилгын ажилтан',
    '🫅' => 'титэмтэй хүн',
    '🤴' => 'ханхүү',
    '👸' => 'гүнж',
    '👳' => 'тюрбан малгайтай эрэгтэй',
    '👲' => 'хятад малгай өмссөн эрэгтэй',
    '🧕' => 'толгойндоо алчууртай эмэгтэй',
    '🤵' => 'хослолтой хүн',
    '👰' => 'гивлүүртэй хүн',
    '🤰' => 'жирэмсэн эмэгтэй',
    '🫃' => 'жирэмсэн эрэгтэй',
    '🫄' => 'жирэмсэн хүн',
    '🤱' => 'хөхөөр хооллох',
    '👼' => 'нялх сахиусан тэнгэр',
    '🎅' => 'Санта Клаус',
    '🤶' => 'Зулсарын хатагтай',
    '🦸' => 'супер баатар',
    '🦹' => 'ид шидийн хүчтэй эсрэг дүр',
    '🧙' => 'шидтэн',
    '🧚' => 'дагина',
    '🧛' => 'цус сорогч',
    '🧜' => 'лусын хүн',
    '🧝' => 'одой хүн',
    '🧞' => 'жини',
    '🧟' => 'зомби',
    '🧌' => 'тролл',
    '🫈' => 'үсэрхэг амьтан',
    '💆' => 'нүүрний массаж',
    '💇' => 'үс засалт',
    '🚶' => 'алхаж буй хүн',
    '🧍' => 'зогсож буй хүн',
    '🧎' => 'сөхөрч суусан хүн',
    '🏃' => 'гүйж байгаа хүн',
    '💃' => 'бүжигчин',
    '🕺' => 'бүжиглэх залуу',
    '🕴' => 'хослол өмссөн эрэгтэй',
    '👯' => 'үдэшлэгт байгаа бүсгүй',
    '🧖' => 'уур савссан өрөөнд байгаа хүн',
    '🧗' => 'авирагч',
    '🤺' => 'туялзуур жадаар тоглогч',
    '🏇' => 'морин уралдаан',
    '⛷' => 'цанаар гулгагч',
    '🏂' => 'цасан дээр гулгагч',
    '🏌' => 'гольф',
    '🏄' => 'серфингээр гулгаж байгаа хүн',
    '🚣' => 'завь сэлүүрдэж буй хүн',
    '🏊' => 'усанд сэлэгч',
    '⛹' => 'бөмбөгтэй хүн',
    '🏋' => 'хүндийн жингийн тамирчин',
    '🚴' => 'дугуйчин',
    '🚵' => 'уулын дугуйчин',
    '🤸' => 'сальто эргэх',
    '🤼' => 'бөх',
    '🤽' => 'усны поло',
    '🤾' => 'гар бөмбөг тоглож буй хүн',
    '🤹' => 'жонглёрдох',
    '🧘' => 'лотус байрлалтай хүн',
    '🛀' => 'усанд орж байгаа хүн',
    '🛌' => 'орондоо байгаа хүн',
    '👭' => 'хөтлөлцсөн хоёр эмэгтэй',
    '👫' => 'хөтлөлцсөн эрэгтэй эмэгтэй хос',
    '👬' => 'хөтлөлцсөн хоёр эрэгтэй',
    '💏' => 'үнсэлт',
    '💑' => 'дундаа зүрхтэй хосууд',
    '🗣' => 'ярьж буй толгой',
    '👤' => 'хүний сүүдэр дүрс',
    '👥' => '2 сүүдэр',
    '🫂' => 'тэврэлдэж буй хүмүүс',
    '👪' => 'гэр бүл',
    '👣' => 'хөлийн мөр',
    '🫆' => 'хурууны хээ',
    '🦰' => 'улаан үс',
    '🦱' => 'буржгар үс',
    '🦳' => 'цагаан үс',
    '🦲' => 'халзан',
    '🐵' => 'сармагчны толгой',
    '🐒' => 'сармагчин',
    '🦍' => 'горилла',
    '🦧' => 'орангутан',
    '🐶' => 'нохойны толгой',
    '🐕' => 'нохой',
    '🦮' => 'хөтөч нохой',
    '🐩' => 'пүүдл үүлдрийн нохой',
    '🐺' => 'чонын толгой',
    '🦊' => 'үнэг',
    '🦝' => 'элбэнх',
    '🐱' => 'муурын толгой',
    '🐈' => 'муур',
    '🦁' => 'арслангийн толгой',
    '🐯' => 'барын толгой',
    '🐅' => 'бар',
    '🐆' => 'ирвэс',
    '🐴' => 'морины толгой',
    '🫎' => 'хандгай',
    '🫏' => 'илжиг',
    '🐎' => 'морь',
    '🦄' => 'нэг эвэртийн толгой',
    '🦓' => 'тахь',
    '🦌' => 'буга',
    '🦬' => 'бизон',
    '🐮' => 'үнээний толгой',
    '🐂' => 'шар үхэр',
    '🐃' => 'бух',
    '🐄' => 'үнээ',
    '🐷' => 'гахайны толгой',
    '🐖' => 'гахай',
    '🐗' => 'бодон гахай',
    '🐽' => 'гахайны хоншоор',
    '🐏' => 'хуц',
    '🐑' => 'хонь',
    '🐐' => 'ямаа',
    '🐪' => 'тэмээ',
    '🐫' => '2 бөхт тэмээ',
    '🦙' => 'тэмээн гөрөөс',
    '🦒' => 'анааш',
    '🐘' => 'заан',
    '🦣' => 'арслан заан',
    '🦏' => 'хирс',
    '🦛' => 'усны үхэр',
    '🐭' => 'хулганы толгой',
    '🐁' => 'хулгана',
    '🐀' => 'харх',
    '🐹' => 'үхэр зусгийн толгой',
    '🐰' => 'туулайны толгой',
    '🐇' => 'туулай',
    '🐿' => 'жирх',
    '🦫' => 'минж',
    '🦔' => 'зараа',
    '🦇' => 'сарьсан багваахай',
    '🐻' => 'баавгайны толгой',
    '🐨' => 'коала',
    '🐼' => 'хулсны баавгайны толгой',
    '🦥' => 'залхууч',
    '🦦' => 'халиу',
    '🦨' => 'өмхий хүрнэ',
    '🦘' => 'имж',
    '🦡' => 'дорго',
    '🐾' => 'саварны мөр',
    '🦃' => 'цацагт хяруул',
    '🐔' => 'тахиа',
    '🐓' => 'азарган тахиа',
    '🐣' => 'дэгдээхэй',
    '🐤' => 'ангаахай',
    '🐥' => 'дэгдээхэй урд талаасаа',
    '🐦' => 'шувуу',
    '🐧' => 'оцон шувуу',
    '🕊' => 'тагтаа',
    '🦅' => 'бүргэд',
    '🦆' => 'нугас',
    '🦢' => 'хун',
    '🦉' => 'шар шувуу',
    '🦤' => 'додо шувуу',
    '🪶' => 'өд',
    '🦩' => 'ягаан наль',
    '🦚' => 'тогос',
    '🦜' => 'тоть',
    '🪽' => 'далавч',
    '🪿' => 'галуу',
    '🐸' => 'мэлхий',
    '🐊' => 'матар',
    '🐢' => 'яст мэлхий',
    '🦎' => 'гүрвэл',
    '🐍' => 'могой',
    '🐲' => 'лууны толгой',
    '🐉' => 'луу',
    '🦕' => 'сауропод',
    '🦖' => 'Т-рекс',
    '🐳' => 'ус оргилуулж байгаа халим',
    '🐋' => 'халим',
    '🐬' => 'далайн гахай',
    '🫍' => 'хар далайн гахай',
    '🦭' => 'далайн хав',
    '🐟' => 'загас',
    '🐠' => 'халуун орны загас',
    '🐡' => 'нохой загас',
    '🦈' => 'акул',
    '🐙' => 'наймаалж',
    '🐚' => 'дунгийн хясаа',
    '🪸' => 'шүр',
    '🪼' => 'далайн мөгөөрс',
    '🦀' => 'хавч',
    '🦞' => 'голын хавч',
    '🦐' => 'сам хорхой',
    '🦑' => 'кальмар',
    '🦪' => 'хясаа',
    '🐌' => 'эмгэн хумс',
    '🦋' => 'эрвээхэй',
    '🐛' => 'хорхой',
    '🐜' => 'шоргоолж',
    '🐝' => 'зөгий',
    '🪲' => 'хорхой шавж',
    '🐞' => 'цох хорхой',
    '🦗' => 'хүрэлзгэнэ',
    '🪳' => 'жоом',
    '🕷' => 'аалз',
    '🕸' => 'аалзны тор',
    '🦂' => 'хилэнц',
    '🦟' => 'шумуул',
    '🪰' => 'ялаа',
    '🪱' => 'өт хорхой',
    '🦠' => 'микроб',
    '💐' => 'цэцгийн баглаа',
    '🌸' => 'интоорын цэцэг',
    '💮' => 'цагаан цэцэг',
    '🪷' => 'бадамлянхуа',
    '🏵' => 'сарнай хэлбэртэй чимэглэл',
    '🌹' => 'сарнай',
    '🥀' => 'хатсан цэцэг',
    '🌺' => 'гибискус сарнай',
    '🌻' => 'наран цэцэг',
    '🌼' => 'цэцэг',
    '🌷' => 'алтан зул цэцэг',
    '🪻' => 'хэлгий цэцэг',
    '🌱' => 'үрсэлгээ',
    '🪴' => 'ваартай ургамал',
    '🌲' => 'гацуур',
    '🌳' => 'навчит мод',
    '🌴' => 'дал мод',
    '🌵' => 'кактус',
    '🌾' => 'будааны тариан түрүү',
    '🌿' => 'ургамал',
    '☘' => 'айргана',
    '🍀' => '4 навчтай хошоонгор',
    '🍁' => 'агч модны навч',
    '🍂' => 'унаж байгаа навч',
    '🍃' => 'салхинд хийсч буй навч',
    '🪹' => 'хоосон үүр',
    '🪺' => 'өндөгтэй үүр',
    '🍄' => 'мөөг',
    '🪾' => 'навчгүй мод',
    '🍇' => 'усан үзэм',
    '🍈' => 'гуа',
    '🍉' => 'тарвас',
    '🍊' => 'мандарин',
    '🍋' => 'лимон',
    '🍌' => 'гадил',
    '🍍' => 'хан боргоцой',
    '🥭' => 'манго',
    '🍎' => 'улаан алим',
    '🍏' => 'ногоон алим',
    '🍐' => 'лийр',
    '🍑' => 'тоор',
    '🍒' => 'интоор',
    '🍓' => 'гүзээлзгэнэ',
    '🫐' => 'нэрс',
    '🥝' => 'киви жимс',
    '🍅' => 'улаан лооль',
    '🫒' => 'олив',
    '🥥' => 'наргил',
    '🥑' => 'авокадо',
    '🍆' => 'хаш',
    '🥔' => 'төмс',
    '🥕' => 'лууван',
    '🌽' => 'шошны навч',
    '🌶' => 'халуун чинжүү',
    '🫑' => 'амтат чинжүү',
    '🥒' => 'өргөст хэмх',
    '🥬' => 'навчит ногоо',
    '🥦' => 'цэцэгт байцаа',
    '🧄' => 'саримс',
    '🧅' => 'сонгино',
    '🥜' => 'газрын самар',
    '🫘' => 'шош',
    '🌰' => 'каштан',
    '🫚' => 'цагаан гааны үндэс',
    '🫛' => 'буурцаг',
    '🫜' => 'үндэстэй ногоо',
    '🍞' => 'талх',
    '🥐' => 'ороомог талх',
    '🥖' => 'багет франц талх',
    '🫓' => 'гамбир',
    '🥨' => 'претцел',
    '🥯' => 'цагирган талх',
    '🥞' => 'бин',
    '🧇' => 'вафль',
    '🧀' => 'зүсэм бяслаг',
    '🍖' => 'ястай мах',
    '🍗' => 'тахиан шарсан мөч',
    '🥩' => 'зүсэм мах',
    '🥓' => 'гахайн мах',
    '🍔' => 'гамбургер',
    '🍟' => 'шарсан төмс',
    '🍕' => 'пицца',
    '🌭' => 'хот-дог',
    '🥪' => 'сэндвич',
    '🌮' => 'тако',
    '🌯' => 'буррито',
    '🫔' => 'тамале',
    '🥙' => 'хавчуургатай ороомог',
    '🧆' => 'фалафель',
    '🥚' => 'өндөг',
    '🍳' => 'өндөг шарах',
    '🥘' => 'тогоотой хоол',
    '🍲' => 'хоолтой тогоо',
    '🫕' => 'фондю',
    '🥣' => 'халбагатай сав',
    '🥗' => 'ногооны салат',
    '🍿' => 'попкорн',
    '🧈' => 'цөцгийн тос',
    '🧂' => 'давс',
    '🥫' => 'лаазтай хоол',
    '🍱' => 'өдрийн хоол',
    '🍘' => 'будааны жигнэмэг',
    '🍙' => 'будааны ороомог',
    '🍚' => 'агшаасан будаа',
    '🍛' => 'карритай будаа',
    '🍜' => 'чанасан гоймон',
    '🍝' => 'шпагетти',
    '🍠' => 'шарсан чихэрлэг төмс',
    '🍢' => 'удэн',
    '🍣' => 'суши',
    '🍤' => 'шарсан сам хорхой',
    '🍥' => 'загасны бялуу',
    '🥮' => 'саран боов',
    '🍡' => 'данго',
    '🥟' => 'бууз',
    '🥠' => 'азын жигнэмэг',
    '🥡' => 'авч явах',
    '🍦' => 'шинэхэн мөхөөлдөс',
    '🍧' => 'хөлдөөсөн мөс',
    '🍨' => 'мөхөөлдөс',
    '🍩' => 'хайрсан нарийн боов',
    '🍪' => 'жигнэмэг',
    '🎂' => 'төрсөн өдрийн бялуу',
    '🍰' => 'бялууны зүсэм',
    '🧁' => 'аяган кекс',
    '🥧' => 'зүсэм бялуу',
    '🍫' => 'шоколад',
    '🍬' => 'чихэр',
    '🍭' => 'иштэй чихэр',
    '🍮' => 'кастард',
    '🍯' => 'зөгийн балны сав',
    '🍼' => 'хүүхдийн сүүний сав',
    '🥛' => 'аяга сүү',
    '☕' => 'халуун уух зүйл',
    '🫖' => 'цайны гүц',
    '🍵' => 'бариулгүй цайны аяга',
    '🍶' => 'сакэ',
    '🍾' => 'оргилуун дарс',
    '🍷' => 'виноны хундага',
    '🍸' => 'коктэйл',
    '🍹' => 'жимсний ундаа',
    '🍺' => 'шар айраг',
    '🍻' => 'тулгаж буй шар айрагны хундага',
    '🥂' => 'хундага тулгах',
    '🥃' => 'архины жүнз',
    '🫗' => 'шингэн гоожуулах',
    '🥤' => 'соруултай аяга',
    '🧋' => 'бөмбөлөгтэй цай',
    '🧃' => 'ундааны хайрцаг',
    '🧉' => 'матэ',
    '🧊' => 'мөс',
    '🥢' => 'савх',
    '🍽' => 'таваг, хутга сэрээ',
    '🍴' => 'хутга сэрээ',
    '🥄' => 'халбага',
    '🔪' => 'гал тогооны хутга',
    '🫙' => 'шилэн сав',
    '🏺' => 'бумба',
    '🌍' => 'Европ-Африкийг харуулсан бөмбөрцөг',
    '🌎' => 'Америкийг харуулсан бөмбөрцөг',
    '🌏' => 'Ази-Австралийг харуулсан бөмбөрцөг',
    '🌐' => 'уртраг, өргөргийн зураастай бөмбөрцөг',
    '🗺' => 'дэлхийн газрын зураг',
    '🗾' => 'Япон улсын газрын зураг',
    '🧭' => 'луужин',
    '🏔' => 'цаст уул',
    '⛰' => 'уул',
    '🛘' => 'нуранги',
    '🌋' => 'галт уул',
    '🗻' => 'фүжи уул',
    '🏕' => 'аялал',
    '🏖' => 'далайн эрэг',
    '🏜' => 'цөл',
    '🏝' => 'арал',
    '🏞' => 'үндэсний цэцэрлэгт хүрээлэн',
    '🏟' => 'цэнгэлдэх хүрээлэн',
    '🏛' => 'сонгодог барилга',
    '🏗' => 'барилга угсралт',
    '🧱' => 'тоосго',
    '🪨' => 'хад',
    '🪵' => 'мод',
    '🛖' => 'урц',
    '🏘' => 'байшингууд',
    '🏚' => 'балгас',
    '🏠' => 'байшин',
    '🏡' => 'цэцэрлэгтэй байшин',
    '🏢' => 'оффисын барилга',
    '🏣' => 'японы шуудан',
    '🏤' => 'шуудан',
    '🏥' => 'эмнэлэг',
    '🏦' => 'банк',
    '🏨' => 'зочид буудал',
    '🏩' => 'хайрын зочид буудал',
    '🏪' => '24 цагийн дэлгүүр',
    '🏫' => 'сургууль',
    '🏬' => 'их дэлгүүр',
    '🏭' => 'үйлдвэр',
    '🏯' => 'японы ордон',
    '🏰' => 'ордон',
    '💒' => 'хурим',
    '🗼' => 'токио цамхаг',
    '🗽' => 'эрх чөлөөний хөшөө',
    '⛪' => 'сүм',
    '🕌' => 'лалын сүм',
    '🛕' => 'хиндү сүм',
    '🕍' => 'жүүдийн сүм',
    '⛩' => 'бурхны газар',
    '🕋' => 'кааба',
    '⛲' => 'усан оргилуур',
    '⛺' => 'майхан',
    '🌁' => 'манантай',
    '🌃' => 'одтой шөнө',
    '🏙' => 'хот',
    '🌄' => 'уулын толгод дээгүүр нар мандах',
    '🌅' => 'нар мандах',
    '🌆' => 'үдшийн бүрийд хотын харагдац',
    '🌇' => 'нар жаргах',
    '🌉' => 'шөнийн гүүр',
    '♨' => 'халуун рашаан',
    '🎠' => 'морин тойруулга',
    '🛝' => 'тоглоомын талбайн гулсуур',
    '🎡' => 'алсыг харагч',
    '🎢' => 'галзуу хулгана',
    '💈' => 'үсчин салон',
    '🎪' => 'циркийн майхан',
    '🚂' => 'зүтгүүр',
    '🚃' => 'төмөр замын машин',
    '🚄' => 'хурдны галт тэрэг',
    '🚅' => 'хурдны сумт галт тэрэг',
    '🚆' => 'галт тэрэг',
    '🚇' => 'метро',
    '🚈' => 'хөнгөн төмөр зам',
    '🚉' => 'галт тэрэгний буудал',
    '🚊' => 'трамвай',
    '🚝' => 'дүүжин төмөр зам',
    '🚞' => 'уулын төмөр зам',
    '🚋' => 'трамвай машин',
    '🚌' => 'автобус',
    '🚍' => 'ирж буй автобус',
    '🚎' => 'троллейбус',
    '🚐' => 'жижиг оврын автобус',
    '🚑' => 'эмнэлгийн машин',
    '🚒' => 'галын машин',
    '🚓' => 'цагдаагийн машин',
    '🚔' => 'ирж буй цагдаагийн машин',
    '🚕' => 'такси',
    '🚖' => 'ирж буй такси',
    '🚗' => 'автомашин',
    '🚘' => 'ирж байгаа машин',
    '🚙' => 'спорт машин',
    '🛻' => 'пикап хагас чиргүүлтэй машин',
    '🚚' => 'хүргэлтийн ачааны тэрэг',
    '🚛' => 'бүхээгтэй ачааны тэрэг',
    '🚜' => 'трактор',
    '🏎' => 'уралдааны машин',
    '🏍' => 'мотоцикл',
    '🛵' => 'моторт дугуй',
    '🦽' => 'гар тэргэнцэр',
    '🦼' => 'моторт тэргэнцэр',
    '🛺' => 'рикша',
    '🚲' => 'унадаг дугуй',
    '🛴' => 'самокат',
    '🛹' => 'скейтборд',
    '🛼' => 'дугуйтай тэшүүр',
    '🚏' => 'автобусны зогсоол',
    '🛣' => 'зам',
    '🛤' => 'галт тэрэгний зам',
    '🛢' => 'газрын тосны хүрд',
    '⛽' => 'шатахуун түгээгүүр',
    '🛞' => 'дугуй',
    '🚨' => 'цагдаагийн машины гэрэл',
    '🚥' => 'замын хөндлөн гэрэл',
    '🚦' => 'замын босоо гэрэл',
    '🛑' => 'зогсох тэмдэг',
    '🚧' => 'бүтээн байгуулалт',
    '⚓' => 'зангуу',
    '🛟' => 'цагираг хөвүүр тэмдэг',
    '⛵' => 'завь',
    '🛶' => 'сэлүүрт завь',
    '🚤' => 'хурдан завь',
    '🛳' => 'зорчих хөлөг онгоц',
    '⛴' => 'гатлага хөлөг онгоц',
    '🛥' => 'моторт онгоц',
    '🚢' => 'хөлөг онгоц',
    '✈' => 'онгоц',
    '🛩' => 'жижиг онгоц',
    '🛫' => 'хөөрч буй онгоц',
    '🛬' => 'ирж буй онгоц',
    '🪂' => 'шүхэр',
    '💺' => 'суудал',
    '🚁' => 'нисдэг тэрэг',
    '🚟' => 'дүүжин тээвэр',
    '🚠' => 'уулын кабель зам',
    '🚡' => 'агаарын трамвай',
    '🛰' => 'хиймэл дагуул',
    '🚀' => 'пуужин',
    '🛸' => 'нисдэг таваг',
    '🛎' => 'хонх',
    '🧳' => 'ачаа тээш',
    '⌛' => 'элсэн цаг',
    '⏳' => 'элс нь гойжиж буй элсэн цаг',
    '⌚' => 'бугуйн цаг',
    '⏰' => 'сэрүүлэгтэй цаг',
    '⏱' => 'секунд хэмжигч',
    '⏲' => 'цаг хэмжигч',
    '🕰' => 'ил зуухны цаг',
    '🕛' => 'арван хоёр цаг',
    '🕧' => 'арван хоёр цаг гучин минут',
    '🕐' => 'нэг цаг',
    '🕜' => 'нэг цаг гучин минут',
    '🕑' => 'хоёр цаг',
    '🕝' => 'хоёр цаг гучин минут',
    '🕒' => 'гурван цаг',
    '🕞' => 'гурван цаг гучин минут',
    '🕓' => 'дөрвөн цаг',
    '🕟' => 'дөрвөн цаг гучин минут',
    '🕔' => 'таван цаг',
    '🕠' => 'таван цаг гучин минут',
    '🕕' => 'зургаан цаг',
    '🕡' => 'зургаан цаг гучин минут',
    '🕖' => 'долоон цаг',
    '🕢' => 'долоон цаг гучин минут',
    '🕗' => 'найман цаг',
    '🕣' => 'найман цаг гучин минут',
    '🕘' => 'есөн цаг',
    '🕤' => 'есөн цаг гучин минут',
    '🕙' => 'арван цаг',
    '🕥' => 'арван цаг гучин минут',
    '🕚' => 'арван нэгэн цаг',
    '🕦' => 'арван нэгэн цаг гучин минут',
    '🌑' => 'шинэ сар',
    '🌒' => 'хавирган сар (баруун)',
    '🌓' => 'эхний улирлын сар',
    '🌔' => 'бүтэн сарын өмнөх сар',
    '🌕' => 'бүтэн сар',
    '🌖' => 'бүтэн сарын дараах сар',
    '🌗' => 'сүүлийн улирлын сар',
    '🌘' => 'хавирган сар (зүүн)',
    '🌙' => 'хавирган сар',
    '🌚' => 'нүүртэй шинэ сар',
    '🌛' => 'нүүртэй эхний хагас сар',
    '🌜' => 'нүүртэй сүүлийн хагас сар',
    '🌡' => 'термометр',
    '☀' => 'нар',
    '🌝' => 'нүүртэй бүтэн сар',
    '🌞' => 'нүүртэй нар',
    '🪐' => 'тойрог замтай гараг',
    '⭐' => 'од',
    '🌟' => 'гялалзаж буй од',
    '🌠' => 'од харвах',
    '🌌' => 'сүүн зам',
    '☁' => 'үүл',
    '⛅' => 'үүлний цаагуурх нар',
    '⛈' => 'цахилгаантай, нартай бороо',
    '🌤' => 'жижиг үүлний цаагуурх нар',
    '🌥' => 'том үүлний цаагуурх нар',
    '🌦' => 'бороотой үүлний цаагуурх нар',
    '🌧' => 'бороотой үүл',
    '🌨' => 'цастай үүл',
    '🌩' => 'цахилгаантай үүл',
    '🌪' => 'хар салхи',
    '🌫' => 'манан',
    '🌬' => 'салхиар үлээж буй нүүр',
    '🌀' => 'циклон',
    '🌈' => 'солонго',
    '🌂' => 'нээгээгүй шүхэр',
    '☂' => 'нээсэн шүхэр',
    '☔' => 'борооны дусалтай шүхэр',
    '⛱' => 'газар байгаа шүхэр',
    '⚡' => 'өндөр хүчдэл',
    '❄' => 'цасан ширхэг',
    '☃' => 'цасан хүн дээр цас орж байна',
    '⛄' => 'цасан хүн',
    '☄' => 'сүүлт од',
    '🔥' => 'гал',
    '💧' => 'усны дусал',
    '🌊' => 'усны давлагаа',
    '🎃' => 'хулуугаар хийсэн дэнлүү',
    '🎄' => 'зул сарын гацуур',
    '🎆' => 'галын наадам',
    '🎇' => 'бенгалын гал',
    '🧨' => 'салют',
    '✨' => 'гэрэлтэх',
    '🎈' => 'шаар',
    '🎉' => 'үдэшлэгийн чимэглэл',
    '🎊' => 'жингэнэгч хонх',
    '🎋' => 'танабата мод',
    '🎍' => 'нарсан чимэглэл',
    '🎎' => 'япон хүүхэлдэй',
    '🎏' => 'мөрөг загасан дарцаг',
    '🎐' => 'салхин хонх',
    '🎑' => 'сар харах баяр',
    '🧧' => 'бэлэгний улаан дугтуй',
    '🎀' => 'тууз',
    '🎁' => 'боолттой бэлэг',
    '🎗' => 'дурсгалын тууз',
    '🎟' => 'орох тасалбар',
    '🎫' => 'тасалбар',
    '🎖' => 'цэргийн медаль',
    '🏆' => 'цом',
    '🏅' => 'спортын медаль',
    '🥇' => '1-р байрны медаль',
    '🥈' => '2-р байрны медаль',
    '🥉' => '3-р байрны медаль',
    '⚽' => 'хөлбөмбөгийн бөмбөг',
    '⚾' => 'бейсбол',
    '🥎' => 'софтбол',
    '🏀' => 'сагсан бөмбөг',
    '🏐' => 'гар бөмбөг',
    '🏈' => 'америк хөл бөмбөг',
    '🏉' => 'регбигийн бөмбөг',
    '🎾' => 'теннис',
    '🥏' => 'нисдэг тоглоом',
    '🎳' => 'боулинг',
    '🏏' => 'крикет',
    '🏑' => 'хоккей',
    '🏒' => 'мөсөн хоккейны хэрэгсэл',
    '🥍' => 'лакрос',
    '🏓' => 'ширээний теннис',
    '🏸' => 'агаарын теннис',
    '🥊' => 'боксын бээлий',
    '🥋' => 'зодооны өмсгөл',
    '🥅' => 'гоолын хаалга',
    '⛳' => 'гольфын нүх',
    '⛸' => 'мөсөн тэшүүр',
    '🎣' => 'загасны уурга',
    '🤿' => 'сноркель',
    '🎽' => 'гүйлтийн майк',
    '🎿' => 'цана',
    '🛷' => 'чарга',
    '🥌' => 'кёрлинг',
    '🎯' => 'дартс',
    '🪀' => 'ёо-ёо',
    '🪁' => 'цаасан шувуу',
    '🔫' => 'гар буу',
    '🎱' => 'биллиард',
    '🔮' => 'кристал бөмбөлөг',
    '🪄' => 'шидэт дохиур',
    '🎮' => 'видео тоглоом',
    '🕹' => 'удирдлага',
    '🎰' => 'хонжворт тоглоомын машин',
    '🎲' => 'тоглоомон шоо',
    '🧩' => 'эвлүүлдэг зураг',
    '🧸' => 'тэдди бамбарууш',
    '🪅' => 'пината',
    '🪩' => 'толин бөмбөг',
    '🪆' => 'угсардаг хүүхэлдэй',
    '♠' => 'хөзрийн гил',
    '♥' => 'хөзрийн бунд',
    '♦' => 'хөзрийн дөрвөлжин',
    '♣' => 'хөзрийн цэцэг',
    '♟' => 'шатрын хүү',
    '🃏' => 'хөзрийн хүн',
    '🀄' => 'мажонг улаан луу',
    '🎴' => 'хөзөр',
    '🎭' => 'жүжиг',
    '🖼' => 'жаазтай зураг',
    '🎨' => 'уран зургийн паллет',
    '🧵' => 'утас',
    '🪡' => 'оёдлын зүү',
    '🧶' => 'нэхмэлийн утас',
    '🪢' => 'зангилаа',
    '👓' => 'харааны шил',
    '🕶' => 'нарны шил',
    '🥽' => 'нүдний шил',
    '🥼' => 'лабораторын өмсгөл',
    '🦺' => 'аврах хантааз',
    '👔' => 'зангиа',
    '👕' => 'подволк',
    '👖' => 'жинс',
    '🧣' => 'алчуур',
    '🧤' => 'бээлий',
    '🧥' => 'цув',
    '🧦' => 'оймс',
    '👗' => 'даашинз',
    '👘' => 'кимоно',
    '🥻' => 'саарий',
    '🩱' => 'усны үргэлж хувцас',
    '🩲' => 'дотуур өмд',
    '🩳' => 'богино өмд',
    '👙' => 'усны хувцас',
    '👚' => 'эмэгтэй хувцас',
    '🪭' => 'эвхдэг гар дэвүүр',
    '👛' => 'түрийвч',
    '👜' => 'гар цүнх',
    '👝' => 'цүнх',
    '🛍' => 'дэлгүүр хэсэг зориулалттай цүнх',
    '🎒' => 'үүргэвч',
    '🩴' => 'углааш',
    '👞' => 'эрэгтэй гутал',
    '👟' => 'пүүз',
    '🥾' => 'уулны гутал',
    '🥿' => 'намхан ултай гутал',
    '👠' => 'өндөр өсгийтэй гутал',
    '👡' => 'эмэгтэй углааш',
    '🩰' => 'балетын шаахай',
    '👢' => 'эмэгтэй гутал',
    '🪮' => 'үсний сам',
    '👑' => 'титэм',
    '👒' => 'эмэгтэй малгай',
    '🎩' => 'өндөр малгай',
    '🎓' => 'төгсөлтийн малгай',
    '🧢' => 'саравчтай малгай',
    '🪖' => 'цэргийн дуулга',
    '⛑' => 'аврагчийн хамгаалалтын малгай',
    '📿' => 'эрх',
    '💄' => 'уруулын будаг',
    '💍' => 'бөгж',
    '💎' => 'эрдэнийн чулуу',
    '🔇' => 'чанга яригч хаалттай',
    '🔈' => 'чанга яригчийн дууны түвшин нам',
    '🔉' => 'чанга яригчийн дууны түвшин дунд зэрэг',
    '🔊' => 'чанга яригчийн дууны түвшин чанга',
    '📢' => 'чанга яригч',
    '📣' => 'цагаан хоолойн чанга яригч',
    '📯' => 'шуудангийн бүрээ',
    '🔔' => 'нээлттэй хонх',
    '🔕' => 'хориглосон хонх',
    '🎼' => 'нотны шугам',
    '🎵' => 'хөгжмийн нот',
    '🎶' => 'хөгжмийн нотнууд',
    '🎙' => 'студийн микрофон',
    '🎚' => 'дууны түвшин тохируулагч',
    '🎛' => 'тохируулагч',
    '🎤' => 'микрофон',
    '🎧' => 'чихэвч',
    '📻' => 'радио',
    '🎷' => 'саксафон',
    '🎺' => 'эвэр бүрээ',
    '🪊' => 'тромбон',
    '🪗' => 'аккордеон',
    '🎸' => 'гитар',
    '🎹' => 'төгөлдөр хуурын товчлуур',
    '🎻' => 'хийл',
    '🪕' => 'банжо',
    '🥁' => 'бөмбөр',
    '🪘' => 'урт бөмбөр',
    '🪇' => 'маракус',
    '🪈' => 'лимбэ',
    '🪉' => 'ятга',
    '📱' => 'гар утас',
    '📲' => 'гар утсыг заасан нум',
    '☎' => 'суурин утас',
    '📞' => 'утасны харилцуур',
    '📟' => 'пейжер',
    '📠' => 'факсын машин',
    '🔋' => 'батарей',
    '🪫' => 'батарей бага байна',
    '🔌' => 'залгуур',
    '💻' => 'зөөврийн компьютер',
    '🖥' => 'ширээний компьютер',
    '🖨' => 'хэвлэгч',
    '⌨' => 'компьютерийн гар',
    '🖱' => 'компьютерийн хулгана',
    '🖲' => 'удирдлагын бөмбөг',
    '💽' => 'компьютерын диск',
    '💾' => 'уян диск',
    '💿' => 'оптик диск',
    '📀' => 'dvd',
    '🧮' => 'сампин',
    '🎥' => 'киноны камер',
    '🎞' => 'кино хальсны кадр',
    '📽' => 'кино проектор',
    '🎬' => 'кадр',
    '📺' => 'зурагт',
    '📷' => 'камер',
    '📸' => 'гэрэл асдаг камер',
    '📹' => 'видео камер',
    '📼' => 'видеоны хуурцаг',
    '🔍' => 'зүүн тийш хазайлгасан томруулагч шил',
    '🔎' => 'баруун тийш хазайлгасан томруулагч шил',
    '🕯' => 'лаа',
    '💡' => 'гэрлийн чийдэн',
    '🔦' => 'гар чийдэн',
    '🏮' => 'улаан цаасан дэнлүү',
    '🪔' => 'тосон дэн',
    '📔' => 'чимэглэсэн хавтастай тэмдэглэлийн дэвтэр',
    '📕' => 'хаалттай ном',
    '📖' => 'нээлттэй ном',
    '📗' => 'ногоон ном',
    '📘' => 'цэнхэр ном',
    '📙' => 'улбар шар ном',
    '📚' => 'ном',
    '📓' => 'дэвтэр',
    '📒' => 'өврийн дэвтэр',
    '📃' => 'хуудас',
    '📜' => 'хуйлмал',
    '📄' => 'цаас',
    '📰' => 'сонин',
    '🗞' => 'хуйлсан сонин',
    '📑' => 'номын хавчуургын табууд',
    '🔖' => 'номны хавчуурга',
    '🏷' => 'шошго',
    '🪙' => 'зоос',
    '💰' => 'мөнгөтэй цүнх',
    '🪎' => 'эрдэнэсийн авдар',
    '💴' => 'иений мөнгөн тэмдэгт',
    '💵' => 'долларын мөнгөн тэмдэгт',
    '💶' => 'евроны мөнгөн тэмдэгт',
    '💷' => 'фунтын мөнгөн тэмдэгт',
    '💸' => 'далавчтай мөнгө',
    '💳' => 'кредит карт',
    '🧾' => 'төлбөрийн баримт',
    '💹' => 'иенийн өсч буй диаграм',
    '✉' => 'дугтуй',
    '📧' => 'имэйл',
    '📨' => 'ирж буй дугтуй',
    '📩' => 'сумтай дугтуй',
    '📤' => 'явуулсан хайрцаг',
    '📥' => 'ирсэн хайрцаг',
    '📦' => 'илгээмж',
    '📫' => 'өргөсөн тугтай хаалттай шуудангийн хайрцаг',
    '📪' => 'туг нь буусан хаалттай шуудангийн хайрцаг',
    '📬' => 'Тугтай онгорхой шуудангийн хайрцаг',
    '📭' => 'тугийг нь буулгасан онгорхой шуудангийн хайрцаг',
    '📮' => 'шуудангийн хайрцаг',
    '🗳' => 'саналын хайрцаг',
    '✏' => 'харандаа',
    '✒' => 'хар үзгийн хошуу',
    '🖋' => 'бэхтэй үзэг',
    '🖊' => 'бал',
    '🖌' => 'зургийн бийр',
    '🖍' => 'тосон харандаа',
    '📝' => 'тэмдэглэл',
    '💼' => 'ажил хэргийн цүнх',
    '📁' => 'файлын хавтас',
    '📂' => 'онгорхой файлын хавтас',
    '🗂' => 'карт хуваарилагч',
    '📅' => 'календарь',
    '📆' => 'урдаг календарь',
    '🗒' => 'эргүүлдэг тэмдэглэлийн хуудас',
    '🗓' => 'эргүүлдэг календарь',
    '📇' => 'картын индекс',
    '📈' => 'өсч буй диаграмм',
    '📉' => 'буурч буй диаграмм',
    '📊' => 'график хүснэгт',
    '📋' => 'самбар',
    '📌' => 'хатгуур',
    '📍' => 'бөөрөнхий тогтоогч',
    '📎' => 'цаасны хавчаар',
    '🖇' => 'холбоотой цаасны хавчаарууд',
    '📏' => 'шугам',
    '📐' => 'гурвалжин шугам',
    '✂' => 'хайч',
    '🗃' => 'картын хайрцаг',
    '🗄' => 'файлын шүүгээ',
    '🗑' => 'хогийн сав',
    '🔒' => 'цоожтой',
    '🔓' => 'онгорхой цоож',
    '🔏' => 'цоож, бал',
    '🔐' => 'цоож, түлхүүр',
    '🔑' => 'түлхүүр',
    '🗝' => 'хуучин түлхүүр',
    '🔨' => 'алх',
    '🪓' => 'сүх',
    '⛏' => 'зээтүү',
    '⚒' => 'алх, зээтүү',
    '🛠' => 'алх, түлхүүр',
    '🗡' => 'чинжаал хутга',
    '⚔' => 'солбиж тавьсан илбэ',
    '💣' => 'тэсрэх бөмбөг',
    '🪃' => 'boomerang',
    '🏹' => 'нум сум',
    '🛡' => 'бамбай',
    '🪚' => 'мужааны хөрөө',
    '🔧' => 'эрэг чангалах түлхүүр',
    '🪛' => 'халив',
    '🔩' => 'эрэг боолт',
    '⚙' => 'хөдөлгүүр',
    '🗜' => 'шахагч',
    '⚖' => 'жин',
    '🦯' => 'хараагүй хүний таяг',
    '🔗' => 'гогцоо',
    '⛓' => 'гинж',
    '🪝' => 'дэгээ',
    '🧰' => 'багажны хайрцаг',
    '🧲' => 'соронз',
    '🪜' => 'шат',
    '🪏' => 'хүрз',
    '⚗' => 'нэрэх тогоо',
    '🧪' => 'хуруу шил',
    '🧫' => 'петрийн аяга',
    '🧬' => 'ДНА',
    '🔬' => 'микроскоп',
    '🔭' => 'телескоп',
    '📡' => 'хиймэл дагуулын антен',
    '💉' => 'тариа',
    '🩸' => 'дусал цус',
    '💊' => 'эм',
    '🩹' => 'хурууны наалт',
    '🩼' => 'суга таяг',
    '🩺' => 'чагнуур',
    '🩻' => 'рентген зураг',
    '🚪' => 'хаалга',
    '🛗' => 'лифт',
    '🪞' => 'толь',
    '🪟' => 'цонх',
    '🛏' => 'ор',
    '🛋' => 'буйдан, ширээний гэрэл',
    '🪑' => 'сандал',
    '🚽' => 'жорлон',
    '🪠' => 'жорлонгийн бөглөө гаргагч',
    '🚿' => 'усанд орох',
    '🛁' => 'ванн',
    '🪤' => 'хулганы хавх',
    '🪒' => 'сахлын хутга',
    '🧴' => 'тосны сав',
    '🧷' => 'сүлбээр зүү',
    '🧹' => 'хогийн шүүр',
    '🧺' => 'сагс',
    '🧻' => 'хуйлсан цаас',
    '🪣' => 'хувин',
    '🧼' => 'саван',
    '🫧' => 'бөмбөлгүүд',
    '🪥' => 'шүдний сойз',
    '🧽' => 'хөвсгөр ус шингээгч резин',
    '🧯' => 'гал унтраагч',
    '🛒' => 'дэлгүүрийн тэрэг',
    '🚬' => 'тамхи',
    '⚰' => 'авс',
    '🪦' => 'хөшөө чулуу',
    '⚱' => 'чандар хадгалах сав',
    '🧿' => 'цэнхэр нүд',
    '🪬' => 'хамса',
    '🗿' => 'хүн чулуу',
    '🪧' => 'зурагт хуудас',
    '🪪' => 'биеийн байцаалт',
    '🏧' => 'ATM-н тэмдэг',
    '🚮' => 'хогоо хогийн саванд хийх тэмдэг',
    '🚰' => 'ундны ус',
    '♿' => 'тэргэнцэрийн тэмдэг',
    '🚹' => 'эрэгтэйчүүдийн өрөө',
    '🚺' => 'эмэгтэйчүүдийн өрөө',
    '🚻' => 'ариун цэврийн өрөө',
    '🚼' => 'нялх хүүхдийн дүрс тэмдэг',
    '🚾' => 'жорлонгийн суултуур',
    '🛂' => 'паспортын шалгалт',
    '🛃' => 'гааль',
    '🛄' => 'ачаа тээшээ авах',
    '🛅' => 'орхисон тээш',
    '⚠' => 'анхааруулга',
    '🚸' => 'хүүхэд зам гарах',
    '⛔' => 'нэвтрэх хориотой',
    '🚫' => 'хориглосон',
    '🚳' => 'дугуй унахыг хориглоно',
    '🚭' => 'тамхи татахыг хориглоно',
    '🚯' => 'хог хаяхыг хориглоно',
    '🚱' => 'ундны зориулалтын бус ус',
    '🚷' => 'явган хүн явах хориотой',
    '📵' => 'утас ашиглах хориотой',
    '🔞' => '18-с доош насныханд хориглоно',
    '☢' => 'цацраг идэвхт',
    '☣' => 'биологийн аюултай',
    '⬆' => 'дээшээ сум',
    '↗' => 'баруун дээшээ сум',
    '➡' => 'баруун сум',
    '↘' => 'баруун доошоо сум',
    '⬇' => 'доошоо сум',
    '↙' => 'зүүн доошоо сум',
    '⬅' => 'зүүн сум',
    '↖' => 'зүүн дээшээ сум',
    '↕' => 'дээш доошоо сум',
    '↔' => 'зүүн-баруун сум',
    '↩' => 'зүүн эргэсэн баруун сум',
    '↪' => 'баруун эргэсэн зүүн сум',
    '⤴' => 'дээшээ заасан баруун сум',
    '⤵' => 'доошоо заасан баруун сум',
    '🔃' => 'цагийн зүүний босоо сум',
    '🔄' => 'цагийн зүүний эсрэг товчлуур',
    '🔙' => 'БУЦАХ сум',
    '🔚' => 'төгсөх сум',
    '🔛' => 'дээр гэсэн сум',
    '🔜' => 'удахгүй гэсэн сум',
    '🔝' => 'дээшээ гэсэн сум',
    '🛐' => 'мөргөлийн газар',
    '⚛' => 'атомын тэмдэг',
    '🕉' => 'маань',
    '✡' => 'давидийн од',
    '☸' => 'хүрд',
    '☯' => 'арга билиг',
    '✝' => 'латин загалмай',
    '☦' => 'ортодокс загалмай',
    '☪' => 'од ба хавирган сар',
    '☮' => 'энх тайвны бэлгэ тэмдэг',
    '🕎' => 'лааны суурь',
    '🔯' => '6 хошуутай од',
    '🪯' => 'ханда тэмдэг',
    '♈' => 'хонины орд',
    '♉' => 'үхрийн орд',
    '♊' => 'ихрийн орд',
    '♋' => 'мэлхийн орд',
    '♌' => 'Арслангийн орд',
    '♍' => 'Охины орд',
    '♎' => 'Жинлүүрийн орд',
    '♏' => 'Хилэнцийн орд',
    '♐' => 'нумын орд',
    '♑' => 'матрын орд',
    '♒' => 'бумбын орд',
    '♓' => 'загасны орд',
    '⛎' => 'одны орд',
    '🔀' => 'дуунуудыг холих товчлуур',
    '🔁' => 'давтах товчлуур',
    '🔂' => 'дахин 1 удаа давтах товчлуур',
    '▶' => 'тоглуулах товчлуур',
    '⏩' => 'урагшлуулах',
    '⏭' => 'дараагийн дууны товчлуур',
    '⏯' => 'тоглуулах, түр зогсоох товчлуур',
    '◀' => 'буцаах товчлуур',
    '⏪' => 'ухраах',
    '⏮' => 'сүүлийн дууны товчлуур',
    '🔼' => 'дээшээ заасан товчлуур',
    '⏫' => 'хурдан нэмэх товчлуур',
    '🔽' => 'доошоо заасан товчлуур',
    '⏬' => 'хурдан буулгах товчлуур',
    '⏸' => 'түр зогсоох товчлуур',
    '⏹' => 'зогсоох товчлур',
    '⏺' => 'бичих товчлуур',
    '⏏' => 'цуцлах товчлуур',
    '🎦' => 'кино театр',
    '🔅' => 'бүдгэрүүлэх товчлуур',
    '🔆' => 'цайруулах товчлуур',
    '📶' => 'сүлжээний багана',
    '🛜' => 'утасгүй',
    '📳' => 'чичрэх горим',
    '📴' => 'гар утас унтраалттай',
    '♀' => 'эмэгтэй тэмдэг',
    '♂' => 'эрэгтэй тэмдэг',
    '⚧' => 'трансжендер тэмдэг',
    '✖' => 'үржүүлэх',
    '➕' => 'нэмэх',
    '➖' => 'хасах',
    '➗' => 'хуваах',
    '🟰' => 'тод тэнцүүгийн тэмдэг',
    '♾' => 'хязгааргүй',
    '‼' => 'хоёр анхааруулгын тэмдэг',
    '⁉' => 'анхааруулгын, асуултын тэмдэг',
    '❓' => 'улаан асуултын тэмдэг',
    '❔' => 'цагаан асуултын тэмдэг',
    '❕' => 'цагаан анхааруулгын тэмдэг',
    '❗' => 'улаан анхаарлын тэмдэг',
    '〰' => 'долгионтой зураас',
    '💱' => 'валют арилжаа',
    '💲' => 'долларын тэмдэг',
    '⚕' => 'эмнэлгийн тэмдэг',
    '♻' => 'дахин боловсруулах тэмдэг',
    '⚜' => 'сараана цэцэг',
    '🔱' => 'лусын хааны сэрээ',
    '📛' => 'нэрийн тэмдэг',
    '🔰' => 'Японы анхлан суралцагчийн тэмдэг',
    '⭕' => 'улаан цагираг',
    '✅' => 'цагаан зөвлөх тэмдэг',
    '☑' => 'дөрвөлжинд байгаа зөвлөх тэмдэг',
    '✔' => 'зөвлөх тэмдэг',
    '❌' => 'хэрээсэн тэмдэг',
    '❎' => 'хэрээсэн товч',
    '➰' => 'буржгар хүрд',
    '➿' => 'хоёр буржгартай хүрд',
    '〽' => 'хэсэг нь ээлжилсэн тэмдэг',
    '✳' => 'найман зовхист од',
    '✴' => 'найман хошуутай од',
    '❇' => 'оч',
    '™' => 'барааны тэмдэг',
    '🫟' => 'цацах',
    '🔠' => 'латин том үсгийн оролт',
    '🔡' => 'латин жижиг үсгийн оролт',
    '🔢' => 'тоон оролт',
    '🔣' => 'дүрс тэмдгийн оролт',
    '🔤' => 'латин үсгийн оролт',
    '🅰' => 'А товчлуур (цусны бүлэг)',
    '🆎' => 'AB товчлуур (цусны бүлэг)',
    '🅱' => 'B товчлуур (цусны бүлэг)',
    '🆑' => 'CL товчлуур',
    '🆒' => 'COOL товчлуур',
    '🆓' => 'FREE товчлуур',
    'ℹ' => 'мэдээлэл',
    '🆔' => 'ID товчлуур',
    'Ⓜ' => 'бөөрөнхий доторх m',
    '🆕' => 'NEW товчлуур',
    '🆖' => 'NG товчлуур',
    '🅾' => 'О товчлуур (цусны бүлэг)',
    '🆗' => 'OK товчлуур',
    '🅿' => 'p товчлуур',
    '🆘' => 'SOS товчлуур',
    '🆙' => 'дээшээ товчлуур',
    '🆚' => 'дөрвөлжин эсрэг тэмдэг',
    '🈁' => 'Японоор "энд" гэсэн товчлуур',
    '🈂' => 'Японоор "үйлчилгээний төлбөр" гэсэн товчлуур',
    '🈷' => 'Японоор "сарын дүн" гэсэн товчлуур',
    '🈶' => 'Японоор "үнэгүй биш" гэсэн товчлуур',
    '🈯' => 'Японоор "захиалгатай" гэсэн товчлуур',
    '🉐' => 'Японоор "арилжаа" гэсэн товчлуур',
    '🈹' => 'Японоор "хямдрал" гэсэн товчлуур',
    '🈚' => 'Японоор "үнэгүй" гэсэн товчлуур',
    '🈲' => 'Японоор "хориотой" гэсэн товчлуур',
    '🉑' => 'Японоор "зөвшөөрөх" гэсэн товчлуур',
    '🈸' => 'Японоор "хүсэлт гаргах" гэсэн товчлуур',
    '🈴' => 'Японоор "босго оноо" гэсэн товчлуур',
    '🈳' => 'Японоор "сул" гэсэн товчлуур',
    '㊗' => 'Японоор "баяр хүргэе" гэсэн товчлуур',
    '㊙' => 'Японоор "нууц" гэсэн товчлуур',
    '🈺' => 'Японоор "бизнесүүдэд нээлттэй" гэсэн товчлуур',
    '🈵' => 'Японоор "сул зай байхгүй" гэсэн товчлуур',
    '🔴' => 'улаан тойрог',
    '🟠' => 'улбар шар тойрог',
    '🟡' => 'шар тойрог',
    '🟢' => 'ногоон тойрог',
    '🔵' => 'цэнхэр тойрог',
    '🟣' => 'нил ягаан тойрог',
    '🟤' => 'бор тойрог',
    '⚫' => 'хар тойрог',
    '⚪' => 'цагаан тойрог',
    '🟥' => 'улаан дөрвөлжин',
    '🟧' => 'улбар шар дөрвөлжин',
    '🟨' => 'шар дөрвөлжин',
    '🟩' => 'ногоон дөрвөлжин',
    '🟦' => 'хөх дөрвөлжин',
    '🟪' => 'нил ягаан дөрвөлжин',
    '🟫' => 'бор дөрвөлжин',
    '⬛' => 'том хар дөрвөлжин',
    '⬜' => 'том цагаан дөрвөлжин',
    '◼' => 'хар дундаж дөрвөлжин',
    '◻' => 'цагаан дундаж дөрвөлжин',
    '◾' => 'хар жижгэвтэр дөрвөлжин',
    '◽' => 'цагаан жижгэвтэр дөрвөлжин',
    '▪' => 'хар жижиг дөрвөлжин',
    '▫' => 'цагаан жижиг дөрвөлжин',
    '🔶' => 'том улбар шар алмааз',
    '🔷' => 'том цэнхэр алмааз',
    '🔸' => 'жижиг улбар шар алмааз',
    '🔹' => 'жижиг цэнхэр алмааз',
    '🔺' => 'дээшээ харсан гурвалжин',
    '🔻' => 'доошоо харсан гурвалжин',
    '💠' => 'дундаа цэгтэй алмааз',
    '🔘' => 'бөөрөнхий товчлуур',
    '🔳' => 'цагаан дөрвөлжин товчлуур',
    '🔲' => 'хар дөрвөлжин товчлуур',
    '🏁' => 'алаг туг',
    '🚩' => 'гурвалжин туг',
    '🎌' => 'солбисон туг',
    '🏴' => 'хар туг',
    '🏳' => 'цагаан туг',
];
