<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'ءوبىس: ادام، ادام، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'ءوبىس: ادام، ادام، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'ءوبىس: ادام، ادام، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'ءوبىس: ادام، ادام، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'ءوبىس: ادام، ادام، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'ءوبىس: ادام، ادام، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'ءوبىس: ادام، ادام، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'ءوبىس: ادام، ادام، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'ءوبىس: ادام، ادام، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'ءوبىس: ادام، ادام، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'ءوبىس: ادام، ادام، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'ءوبىس: ادام، ادام، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'ءوبىس: ادام، ادام، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'ءوبىس: ادام، ادام، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'ءوبىس: ادام، ادام، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'ءوبىس: ادام، ادام، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'ءوبىس: ادام، ادام، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'ءوبىس: ادام، ادام، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'ءوبىس: ادام، ادام، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'ءوبىس: ادام، ادام، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍❤‍💋‍👨🏻' => 'ءوبىس: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى',
    '👨🏻‍❤‍💋‍👨🏼' => 'ءوبىس: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏻‍❤‍💋‍👨🏽' => 'ءوبىس: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏻‍❤‍💋‍👨🏾' => 'ءوبىس: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍❤‍💋‍👨🏿' => 'ءوبىس: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏼‍❤‍💋‍👨🏻' => 'ءوبىس: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👨🏼‍❤‍💋‍👨🏼' => 'ءوبىس: ەر ادام، ەر ادام، 3-تەرى ءتۇرى',
    '👨🏼‍❤‍💋‍👨🏽' => 'ءوبىس: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👨🏼‍❤‍💋‍👨🏾' => 'ءوبىس: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👨🏼‍❤‍💋‍👨🏿' => 'ءوبىس: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👨🏽‍❤‍💋‍👨🏻' => 'ءوبىس: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏽‍❤‍💋‍👨🏼' => 'ءوبىس: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏽‍❤‍💋‍👨🏽' => 'ءوبىس: ەر ادام، ەر ادام، 4-تەرى ءتۇسى',
    '👨🏽‍❤‍💋‍👨🏾' => 'ءوبىس: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏽‍❤‍💋‍👨🏿' => 'ءوبىس: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏾‍❤‍💋‍👨🏻' => 'ءوبىس: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏾‍❤‍💋‍👨🏼' => 'ءوبىس: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏾‍❤‍💋‍👨🏽' => 'ءوبىس: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏾‍❤‍💋‍👨🏾' => 'ءوبىس: ەر ادام، ەر ادام، 5-تەرى ءتۇسى',
    '👨🏾‍❤‍💋‍👨🏿' => 'ءوبىس: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏿‍❤‍💋‍👨🏻' => 'ءوبىس: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏿‍❤‍💋‍👨🏼' => 'ءوبىس: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏿‍❤‍💋‍👨🏽' => 'ءوبىس: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏿‍❤‍💋‍👨🏾' => 'ءوبىس: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏿‍❤‍💋‍👨🏿' => 'ءوبىس: ەر ادام، ەر ادام، 6-تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👨🏻' => 'ءوبىس: ايەل، ەر ادام، 1–2 تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👨🏼' => 'ءوبىس: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍❤‍💋‍👨🏽' => 'ءوبىس: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👨🏾' => 'ءوبىس: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👨🏿' => 'ءوبىس: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👨🏻' => 'ءوبىس: ايەل، ەر ادام، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👨🏼' => 'ءوبىس: ايەل، ەر ادام، 3-تەرى ءتۇرى',
    '👩🏼‍❤‍💋‍👨🏽' => 'ءوبىس: ايەل، ەر ادام، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👨🏾' => 'ءوبىس: ايەل، ەر ادام، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👨🏿' => 'ءوبىس: ايەل، ەر ادام، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👨🏻' => 'ءوبىس: ايەل، ەر ادام، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👨🏼' => 'ءوبىس: ايەل، ەر ادام، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍❤‍💋‍👨🏽' => 'ءوبىس: ايەل، ەر ادام، 4-تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👨🏾' => 'ءوبىس: ايەل، ەر ادام، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👨🏿' => 'ءوبىس: ايەل، ەر ادام، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👨🏻' => 'ءوبىس: ايەل، ەر ادام، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👨🏼' => 'ءوبىس: ايەل، ەر ادام، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍❤‍💋‍👨🏽' => 'ءوبىس: ايەل، ەر ادام، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👨🏾' => 'ءوبىس: ايەل، ەر ادام، 5-تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👨🏿' => 'ءوبىس: ايەل، ەر ادام، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👨🏻' => 'ءوبىس: ايەل، ەر ادام، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👨🏼' => 'ءوبىس: ايەل، ەر ادام، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍❤‍💋‍👨🏽' => 'ءوبىس: ايەل، ەر ادام، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👨🏾' => 'ءوبىس: ايەل، ەر ادام، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👨🏿' => 'ءوبىس: ايەل، ەر ادام، 6-تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👩🏻' => 'ءوبىس: ايەل، ايەل، 1–2 تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👩🏼' => 'ءوبىس: ايەل، ايەل، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍❤‍💋‍👩🏽' => 'ءوبىس: ايەل، ايەل، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👩🏾' => 'ءوبىس: ايەل، ايەل، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍❤‍💋‍👩🏿' => 'ءوبىس: ايەل، ايەل، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👩🏻' => 'ءوبىس: ايەل، ايەل، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👩🏼' => 'ءوبىس: ايەل، ايەل، 3-تەرى ءتۇرى',
    '👩🏼‍❤‍💋‍👩🏽' => 'ءوبىس: ايەل، ايەل، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👩🏾' => 'ءوبىس: ايەل، ايەل، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍❤‍💋‍👩🏿' => 'ءوبىس: ايەل، ايەل، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👩🏻' => 'ءوبىس: ايەل، ايەل، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👩🏼' => 'ءوبىس: ايەل، ايەل، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍❤‍💋‍👩🏽' => 'ءوبىس: ايەل، ايەل، 4-تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👩🏾' => 'ءوبىس: ايەل، ايەل، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍❤‍💋‍👩🏿' => 'ءوبىس: ايەل، ايەل، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👩🏻' => 'ءوبىس: ايەل، ايەل، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👩🏼' => 'ءوبىس: ايەل، ايەل، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍❤‍💋‍👩🏽' => 'ءوبىس: ايەل، ايەل، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👩🏾' => 'ءوبىس: ايەل، ايەل، 5-تەرى ءتۇسى',
    '👩🏾‍❤‍💋‍👩🏿' => 'ءوبىس: ايەل، ايەل، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👩🏻' => 'ءوبىس: ايەل، ايەل، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👩🏼' => 'ءوبىس: ايەل، ايەل، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍❤‍💋‍👩🏽' => 'ءوبىس: ايەل، ايەل، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👩🏾' => 'ءوبىس: ايەل، ايەل، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏿‍❤‍💋‍👩🏿' => 'ءوبىس: ايەل، ايەل، 6-تەرى ءتۇسى',
    '🧑🏻‍🫯‍🧑🏼' => 'كۇرەسشىلەر: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏻‍🫯‍🧑🏽' => 'كۇرەسشىلەر: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏻‍🫯‍🧑🏾' => 'كۇرەسشىلەر: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍🫯‍🧑🏿' => 'كۇرەسشىلەر: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏼‍🫯‍🧑🏻' => 'كۇرەسشىلەر: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '🧑🏼‍🫯‍🧑🏽' => 'كۇرەسشىلەر: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '🧑🏼‍🫯‍🧑🏾' => 'كۇرەسشىلەر: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '🧑🏼‍🫯‍🧑🏿' => 'كۇرەسشىلەر: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '🧑🏽‍🫯‍🧑🏻' => 'كۇرەسشىلەر: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏽‍🫯‍🧑🏼' => 'كۇرەسشىلەر: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏽‍🫯‍🧑🏾' => 'كۇرەسشىلەر: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏽‍🫯‍🧑🏿' => 'كۇرەسشىلەر: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏾‍🫯‍🧑🏻' => 'كۇرەسشىلەر: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏾‍🫯‍🧑🏼' => 'كۇرەسشىلەر: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏾‍🫯‍🧑🏽' => 'كۇرەسشىلەر: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏾‍🫯‍🧑🏿' => 'كۇرەسشىلەر: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏿‍🫯‍🧑🏻' => 'كۇرەسشىلەر: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏿‍🫯‍🧑🏼' => 'كۇرەسشىلەر: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏿‍🫯‍🧑🏽' => 'كۇرەسشىلەر: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏿‍🫯‍🧑🏾' => 'كۇرەسشىلەر: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍❤‍🧑🏼' => 'ماحاببات: ادام، ادام، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏻‍❤‍🧑🏽' => 'ماحاببات: ادام، ادام، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏻‍❤‍🧑🏾' => 'ماحاببات: ادام، ادام، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍❤‍🧑🏿' => 'ماحاببات: ادام، ادام، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏼‍❤‍🧑🏻' => 'ماحاببات: ادام، ادام، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '🧑🏼‍❤‍🧑🏽' => 'ماحاببات: ادام، ادام، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '🧑🏼‍❤‍🧑🏾' => 'ماحاببات: ادام، ادام، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '🧑🏼‍❤‍🧑🏿' => 'ماحاببات: ادام، ادام، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '🧑🏽‍❤‍🧑🏻' => 'ماحاببات: ادام، ادام، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏽‍❤‍🧑🏼' => 'ماحاببات: ادام، ادام، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏽‍❤‍🧑🏾' => 'ماحاببات: ادام، ادام، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏽‍❤‍🧑🏿' => 'ماحاببات: ادام، ادام، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏾‍❤‍🧑🏻' => 'ماحاببات: ادام، ادام، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏾‍❤‍🧑🏼' => 'ماحاببات: ادام، ادام، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏾‍❤‍🧑🏽' => 'ماحاببات: ادام، ادام، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏾‍❤‍🧑🏿' => 'ماحاببات: ادام، ادام، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏿‍❤‍🧑🏻' => 'ماحاببات: ادام، ادام، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏿‍❤‍🧑🏼' => 'ماحاببات: ادام، ادام، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏿‍❤‍🧑🏽' => 'ماحاببات: ادام، ادام، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏿‍❤‍🧑🏾' => 'ماحاببات: ادام، ادام، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍🐰‍🧑🏼' => 'بيلەپ جاتقان ايەلدەر: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏻‍🐰‍🧑🏽' => 'بيلەپ جاتقان ايەلدەر: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏻‍🐰‍🧑🏾' => 'بيلەپ جاتقان ايەلدەر: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍🐰‍🧑🏿' => 'بيلەپ جاتقان ايەلدەر: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏼‍🐰‍🧑🏻' => 'بيلەپ جاتقان ايەلدەر: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '🧑🏼‍🐰‍🧑🏽' => 'بيلەپ جاتقان ايەلدەر: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '🧑🏼‍🐰‍🧑🏾' => 'بيلەپ جاتقان ايەلدەر: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '🧑🏼‍🐰‍🧑🏿' => 'بيلەپ جاتقان ايەلدەر: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '🧑🏽‍🐰‍🧑🏻' => 'بيلەپ جاتقان ايەلدەر: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏽‍🐰‍🧑🏼' => 'بيلەپ جاتقان ايەلدەر: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏽‍🐰‍🧑🏾' => 'بيلەپ جاتقان ايەلدەر: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏽‍🐰‍🧑🏿' => 'بيلەپ جاتقان ايەلدەر: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏾‍🐰‍🧑🏻' => 'بيلەپ جاتقان ايەلدەر: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏾‍🐰‍🧑🏼' => 'بيلەپ جاتقان ايەلدەر: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏾‍🐰‍🧑🏽' => 'بيلەپ جاتقان ايەلدەر: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏾‍🐰‍🧑🏿' => 'بيلەپ جاتقان ايەلدەر: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏿‍🐰‍🧑🏻' => 'بيلەپ جاتقان ايەلدەر: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏿‍🐰‍🧑🏼' => 'بيلەپ جاتقان ايەلدەر: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏿‍🐰‍🧑🏽' => 'بيلەپ جاتقان ايەلدەر: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏿‍🐰‍🧑🏾' => 'بيلەپ جاتقان ايەلدەر: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍🫯‍👨🏼' => 'كۇرەسشى ەر: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏻‍🫯‍👨🏽' => 'كۇرەسشى ەر: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏻‍🫯‍👨🏾' => 'كۇرەسشى ەر: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍🫯‍👨🏿' => 'كۇرەسشى ەر: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏼‍🫯‍👨🏻' => 'كۇرەسشى ەر: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👨🏼‍🫯‍👨🏽' => 'كۇرەسشى ەر: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👨🏼‍🫯‍👨🏾' => 'كۇرەسشى ەر: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👨🏼‍🫯‍👨🏿' => 'كۇرەسشى ەر: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👨🏽‍🫯‍👨🏻' => 'كۇرەسشى ەر: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏽‍🫯‍👨🏼' => 'كۇرەسشى ەر: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏽‍🫯‍👨🏾' => 'كۇرەسشى ەر: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏽‍🫯‍👨🏿' => 'كۇرەسشى ەر: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏾‍🫯‍👨🏻' => 'كۇرەسشى ەر: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏾‍🫯‍👨🏼' => 'كۇرەسشى ەر: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏾‍🫯‍👨🏽' => 'كۇرەسشى ەر: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏾‍🫯‍👨🏿' => 'كۇرەسشى ەر: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏿‍🫯‍👨🏻' => 'كۇرەسشى ەر: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏿‍🫯‍👨🏼' => 'كۇرەسشى ەر: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏿‍🫯‍👨🏽' => 'كۇرەسشى ەر: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏿‍🫯‍👨🏾' => 'كۇرەسشى ەر: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍❤‍👨🏻' => 'ماحاببات: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى',
    '👨🏻‍❤‍👨🏼' => 'ماحاببات: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏻‍❤‍👨🏽' => 'ماحاببات: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏻‍❤‍👨🏾' => 'ماحاببات: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍❤‍👨🏿' => 'ماحاببات: ەر ادام، ەر ادام، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏼‍❤‍👨🏻' => 'ماحاببات: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👨🏼‍❤‍👨🏼' => 'ماحاببات: ەر ادام، ەر ادام، 3-تەرى ءتۇرى',
    '👨🏼‍❤‍👨🏽' => 'ماحاببات: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👨🏼‍❤‍👨🏾' => 'ماحاببات: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👨🏼‍❤‍👨🏿' => 'ماحاببات: ەر ادام، ەر ادام، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👨🏽‍❤‍👨🏻' => 'ماحاببات: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏽‍❤‍👨🏼' => 'ماحاببات: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏽‍❤‍👨🏽' => 'ماحاببات: ەر ادام، ەر ادام، 4-تەرى ءتۇسى',
    '👨🏽‍❤‍👨🏾' => 'ماحاببات: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏽‍❤‍👨🏿' => 'ماحاببات: ەر ادام، ەر ادام، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏾‍❤‍👨🏻' => 'ماحاببات: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏾‍❤‍👨🏼' => 'ماحاببات: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏾‍❤‍👨🏽' => 'ماحاببات: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏾‍❤‍👨🏾' => 'ماحاببات: ەر ادام، ەر ادام، 5-تەرى ءتۇسى',
    '👨🏾‍❤‍👨🏿' => 'ماحاببات: ەر ادام، ەر ادام، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏿‍❤‍👨🏻' => 'ماحاببات: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏿‍❤‍👨🏼' => 'ماحاببات: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏿‍❤‍👨🏽' => 'ماحاببات: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏿‍❤‍👨🏾' => 'ماحاببات: ەر ادام، ەر ادام، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏿‍❤‍👨🏿' => 'ماحاببات: ەر ادام، ەر ادام، 6-تەرى ءتۇسى',
    '👨🏻‍🐰‍👨🏼' => 'تويلاتىپ جۇرگەن ەرلەر: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏻‍🐰‍👨🏽' => 'تويلاتىپ جۇرگەن ەرلەر: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏻‍🐰‍👨🏾' => 'تويلاتىپ جۇرگەن ەرلەر: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍🐰‍👨🏿' => 'تويلاتىپ جۇرگەن ەرلەر: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏼‍🐰‍👨🏻' => 'تويلاتىپ جۇرگەن ەرلەر: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👨🏼‍🐰‍👨🏽' => 'تويلاتىپ جۇرگەن ەرلەر: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👨🏼‍🐰‍👨🏾' => 'تويلاتىپ جۇرگەن ەرلەر: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👨🏼‍🐰‍👨🏿' => 'تويلاتىپ جۇرگەن ەرلەر: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👨🏽‍🐰‍👨🏻' => 'تويلاتىپ جۇرگەن ەرلەر: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏽‍🐰‍👨🏼' => 'تويلاتىپ جۇرگەن ەرلەر: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏽‍🐰‍👨🏾' => 'تويلاتىپ جۇرگەن ەرلەر: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏽‍🐰‍👨🏿' => 'تويلاتىپ جۇرگەن ەرلەر: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏾‍🐰‍👨🏻' => 'تويلاتىپ جۇرگەن ەرلەر: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏾‍🐰‍👨🏼' => 'تويلاتىپ جۇرگەن ەرلەر: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏾‍🐰‍👨🏽' => 'تويلاتىپ جۇرگەن ەرلەر: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏾‍🐰‍👨🏿' => 'تويلاتىپ جۇرگەن ەرلەر: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏿‍🐰‍👨🏻' => 'تويلاتىپ جۇرگەن ەرلەر: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏿‍🐰‍👨🏼' => 'تويلاتىپ جۇرگەن ەرلەر: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏿‍🐰‍👨🏽' => 'تويلاتىپ جۇرگەن ەرلەر: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏿‍🐰‍👨🏾' => 'تويلاتىپ جۇرگەن ەرلەر: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍🫯‍👩🏼' => 'كۇرەسشى ايەل: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍🫯‍👩🏽' => 'كۇرەسشى ايەل: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍🫯‍👩🏾' => 'كۇرەسشى ايەل: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍🫯‍👩🏿' => 'كۇرەسشى ايەل: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍🫯‍👩🏻' => 'كۇرەسشى ايەل: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍🫯‍👩🏽' => 'كۇرەسشى ايەل: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍🫯‍👩🏾' => 'كۇرەسشى ايەل: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍🫯‍👩🏿' => 'كۇرەسشى ايەل: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍🫯‍👩🏻' => 'كۇرەسشى ايەل: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍🫯‍👩🏼' => 'كۇرەسشى ايەل: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍🫯‍👩🏾' => 'كۇرەسشى ايەل: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍🫯‍👩🏿' => 'كۇرەسشى ايەل: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍🫯‍👩🏻' => 'كۇرەسشى ايەل: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍🫯‍👩🏼' => 'كۇرەسشى ايەل: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍🫯‍👩🏽' => 'كۇرەسشى ايەل: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍🫯‍👩🏿' => 'كۇرەسشى ايەل: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍🫯‍👩🏻' => 'كۇرەسشى ايەل: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍🫯‍👩🏼' => 'كۇرەسشى ايەل: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍🫯‍👩🏽' => 'كۇرەسشى ايەل: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍🫯‍👩🏾' => 'كۇرەسشى ايەل: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍❤‍👨🏻' => 'ماحاببات: ايەل، ەر ادام، 1–2 تەرى ءتۇسى',
    '👩🏻‍❤‍👨🏼' => 'ماحاببات: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍❤‍👨🏽' => 'ماحاببات: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍❤‍👨🏾' => 'ماحاببات: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍❤‍👨🏿' => 'ماحاببات: ايەل، ەر ادام، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍❤‍👨🏻' => 'ماحاببات: ايەل، ەر ادام، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍❤‍👨🏼' => 'ماحاببات: ايەل، ەر ادام، 3-تەرى ءتۇرى',
    '👩🏼‍❤‍👨🏽' => 'ماحاببات: ايەل، ەر ادام، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍❤‍👨🏾' => 'ماحاببات: ايەل، ەر ادام، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍❤‍👨🏿' => 'ماحاببات: ايەل، ەر ادام، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍❤‍👨🏻' => 'ماحاببات: ايەل، ەر ادام، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍❤‍👨🏼' => 'ماحاببات: ايەل، ەر ادام، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍❤‍👨🏽' => 'ماحاببات: ايەل، ەر ادام، 4-تەرى ءتۇسى',
    '👩🏽‍❤‍👨🏾' => 'ماحاببات: ايەل، ەر ادام، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍❤‍👨🏿' => 'ماحاببات: ايەل، ەر ادام، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍❤‍👨🏻' => 'ماحاببات: ايەل، ەر ادام، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍❤‍👨🏼' => 'ماحاببات: ايەل، ەر ادام، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍❤‍👨🏽' => 'ماحاببات: ايەل، ەر ادام، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍❤‍👨🏾' => 'ماحاببات: ايەل، ەر ادام، 5-تەرى ءتۇسى',
    '👩🏾‍❤‍👨🏿' => 'ماحاببات: ايەل، ەر ادام، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍❤‍👨🏻' => 'ماحاببات: ايەل، ەر ادام، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍❤‍👨🏼' => 'ماحاببات: ايەل، ەر ادام، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍❤‍👨🏽' => 'ماحاببات: ايەل، ەر ادام، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍❤‍👨🏾' => 'ماحاببات: ايەل، ەر ادام، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏿‍❤‍👨🏿' => 'ماحاببات: ايەل، ەر ادام، 6-تەرى ءتۇسى',
    '👩🏻‍❤‍👩🏻' => 'ماحاببات: ايەل، ايەل، 1–2 تەرى ءتۇسى',
    '👩🏻‍❤‍👩🏼' => 'ماحاببات: ايەل، ايەل، 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍❤‍👩🏽' => 'ماحاببات: ايەل، ايەل، 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍❤‍👩🏾' => 'ماحاببات: ايەل، ايەل، 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍❤‍👩🏿' => 'ماحاببات: ايەل، ايەل، 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍❤‍👩🏻' => 'ماحاببات: ايەل، ايەل، 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍❤‍👩🏼' => 'ماحاببات: ايەل، ايەل، 3-تەرى ءتۇرى',
    '👩🏼‍❤‍👩🏽' => 'ماحاببات: ايەل، ايەل، 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍❤‍👩🏾' => 'ماحاببات: ايەل، ايەل، 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍❤‍👩🏿' => 'ماحاببات: ايەل، ايەل، 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍❤‍👩🏻' => 'ماحاببات: ايەل، ايەل، 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍❤‍👩🏼' => 'ماحاببات: ايەل، ايەل، 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍❤‍👩🏽' => 'ماحاببات: ايەل، ايەل، 4-تەرى ءتۇسى',
    '👩🏽‍❤‍👩🏾' => 'ماحاببات: ايەل، ايەل، 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍❤‍👩🏿' => 'ماحاببات: ايەل، ايەل، 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍❤‍👩🏻' => 'ماحاببات: ايەل، ايەل، 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍❤‍👩🏼' => 'ماحاببات: ايەل، ايەل، 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍❤‍👩🏽' => 'ماحاببات: ايەل، ايەل، 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍❤‍👩🏾' => 'ماحاببات: ايەل، ايەل، 5-تەرى ءتۇسى',
    '👩🏾‍❤‍👩🏿' => 'ماحاببات: ايەل، ايەل، 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍❤‍👩🏻' => 'ماحاببات: ايەل، ايەل، 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍❤‍👩🏼' => 'ماحاببات: ايەل، ايەل، 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍❤‍👩🏽' => 'ماحاببات: ايەل، ايەل، 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍❤‍👩🏾' => 'ماحاببات: ايەل، ايەل، 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏿‍❤‍👩🏿' => 'ماحاببات: ايەل، ايەل، 6-تەرى ءتۇسى',
    '👩🏻‍🐰‍👩🏼' => 'تويلاتىپ جۇرگەن ايەلدەر: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍🐰‍👩🏽' => 'تويلاتىپ جۇرگەن ايەلدەر: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍🐰‍👩🏾' => 'تويلاتىپ جۇرگەن ايەلدەر: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍🐰‍👩🏿' => 'تويلاتىپ جۇرگەن ايەلدەر: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍🐰‍👩🏻' => 'تويلاتىپ جۇرگەن ايەلدەر: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍🐰‍👩🏽' => 'تويلاتىپ جۇرگەن ايەلدەر: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍🐰‍👩🏾' => 'تويلاتىپ جۇرگەن ايەلدەر: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍🐰‍👩🏿' => 'تويلاتىپ جۇرگەن ايەلدەر: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍🐰‍👩🏻' => 'تويلاتىپ جۇرگەن ايەلدەر: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍🐰‍👩🏼' => 'تويلاتىپ جۇرگەن ايەلدەر: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍🐰‍👩🏾' => 'تويلاتىپ جۇرگەن ايەلدەر: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍🐰‍👩🏿' => 'تويلاتىپ جۇرگەن ايەلدەر: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍🐰‍👩🏻' => 'تويلاتىپ جۇرگەن ايەلدەر: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍🐰‍👩🏼' => 'تويلاتىپ جۇرگەن ايەلدەر: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍🐰‍👩🏽' => 'تويلاتىپ جۇرگەن ايەلدەر: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍🐰‍👩🏿' => 'تويلاتىپ جۇرگەن ايەلدەر: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍🐰‍👩🏻' => 'تويلاتىپ جۇرگەن ايەلدەر: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍🐰‍👩🏼' => 'تويلاتىپ جۇرگەن ايەلدەر: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍🐰‍👩🏽' => 'تويلاتىپ جۇرگەن ايەلدەر: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍🐰‍👩🏾' => 'تويلاتىپ جۇرگەن ايەلدەر: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍🤝‍🧑🏻' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 1–2 تەرى ءتۇسى',
    '🧑🏻‍🤝‍🧑🏼' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏻‍🤝‍🧑🏽' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏻‍🤝‍🧑🏾' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏻‍🤝‍🧑🏿' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏼‍🤝‍🧑🏻' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '🧑🏼‍🤝‍🧑🏼' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 3-تەرى ءتۇرى',
    '🧑🏼‍🤝‍🧑🏽' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '🧑🏼‍🤝‍🧑🏾' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '🧑🏼‍🤝‍🧑🏿' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '🧑🏽‍🤝‍🧑🏻' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏽‍🤝‍🧑🏼' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏽‍🤝‍🧑🏽' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 4-تەرى ءتۇسى',
    '🧑🏽‍🤝‍🧑🏾' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏽‍🤝‍🧑🏿' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏾‍🤝‍🧑🏻' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏾‍🤝‍🧑🏼' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏾‍🤝‍🧑🏽' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏾‍🤝‍🧑🏾' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 5-تەرى ءتۇسى',
    '🧑🏾‍🤝‍🧑🏿' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🧑🏿‍🤝‍🧑🏻' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🧑🏿‍🤝‍🧑🏼' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🧑🏿‍🤝‍🧑🏽' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🧑🏿‍🤝‍🧑🏾' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🧑🏿‍🤝‍🧑🏿' => 'قولدارىن كوتەرىپ تۇرعان ادامدار: 6-تەرى ءتۇسى',
    '👩🏻‍🤝‍👩🏼' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍🤝‍👩🏽' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍🤝‍👩🏾' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍🤝‍👩🏿' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍🤝‍👩🏻' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍🤝‍👩🏽' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍🤝‍👩🏾' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍🤝‍👩🏿' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍🤝‍👩🏻' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍🤝‍👩🏼' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍🤝‍👩🏾' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍🤝‍👩🏿' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍🤝‍👩🏻' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍🤝‍👩🏼' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍🤝‍👩🏽' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍🤝‍👩🏿' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍🤝‍👩🏻' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍🤝‍👩🏼' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍🤝‍👩🏽' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍🤝‍👩🏾' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏻‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏻‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏻‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏼‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👩🏼‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👩🏼‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👩🏼‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👩🏽‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏽‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏽‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩🏽‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏾‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏾‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏾‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏾‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👩🏿‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👩🏿‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👩🏿‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👩🏿‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏻‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏻‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏻‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏼‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '👨🏼‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '👨🏼‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '👨🏼‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '👨🏽‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏽‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏽‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👨🏽‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏾‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏾‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏾‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏾‍🤝‍👨🏿' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '👨🏿‍🤝‍👨🏻' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '👨🏿‍🤝‍👨🏼' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '👨🏿‍🤝‍👨🏽' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '👨🏿‍🤝‍👨🏾' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '👩‍❤‍💋‍👨' => 'ءوبىس: ايەل، ەر ادام',
    '👨‍❤‍💋‍👨' => 'ءوبىس: ەر ادام، ەر ادام',
    '👩‍❤‍💋‍👩' => 'ءوبىس: ايەل، ايەل',
    '👨‍👩‍👧‍👦' => 'وتباسى: ەر ادام، ايەل، قىز، ۇل',
    '👨‍👩‍👦‍👦' => 'وتباسى: ەر ادام، ايەل، ۇل، ۇل',
    '👨‍👩‍👧‍👧' => 'وتباسى: ەر ادام، ايەل، قىز، قىز',
    '👨‍👨‍👧‍👦' => 'وتباسى: ەر ادام، ەر ادام، قىز، ۇل',
    '👨‍👨‍👦‍👦' => 'وتباسى: ەر ادام، ەر ادام، ۇل، ۇل',
    '👨‍👨‍👧‍👧' => 'وتباسى: ەر ادام، ەر ادام، قىز، قىز',
    '👩‍👩‍👧‍👦' => 'وتباسى: ايەل، ايەل، قىز، ۇل',
    '👩‍👩‍👦‍👦' => 'وتباسى: ايەل، ايەل، ۇل، ۇل',
    '👩‍👩‍👧‍👧' => 'وتباسى: ايەل، ايەل، قىز، قىز',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'ту: Англия',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'ту: Шотландия',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'ту: Уэльс',
    '🧑‍🧑‍🧒‍🧒' => 'وتباسى: ەرەسەك، ەرەسەك، بالا، بالا',
    '🚶🏻‍♀‍➡' => 'جاياۋ ايەل: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏼‍♀‍➡' => 'جاياۋ ايەل: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🚶🏽‍♀‍➡' => 'جاياۋ ايەل: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏾‍♀‍➡' => 'جاياۋ ايەل: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏿‍♀‍➡' => 'جاياۋ ايەل: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏻‍♂‍➡' => 'جاياۋ ەر: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏼‍♂‍➡' => 'جاياۋ ەر: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🚶🏽‍♂‍➡' => 'جاياۋ ەر: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏾‍♂‍➡' => 'جاياۋ ەر: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏿‍♂‍➡' => 'جاياۋ ەر: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏻‍♀‍➡' => 'تىزەرلەپ وتىرعان ايەل: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏼‍♀‍➡' => 'تىزەرلەپ وتىرعان ايەل: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🧎🏽‍♀‍➡' => 'تىزەرلەپ وتىرعان ايەل: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏾‍♀‍➡' => 'تىزەرلەپ وتىرعان ايەل: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏿‍♀‍➡' => 'تىزەرلەپ وتىرعان ايەل: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏻‍♂‍➡' => 'تىزەرلەپ وتىرعان ەركەك: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏼‍♂‍➡' => 'تىزەرلەپ وتىرعان ەركەك: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🧎🏽‍♂‍➡' => 'تىزەرلەپ وتىرعان ەركەك: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏾‍♂‍➡' => 'تىزەرلەپ وتىرعان ەركەك: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏿‍♂‍➡' => 'تىزەرلەپ وتىرعان ەركەك: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏻‍🦯‍➡' => 'تاياعى بار ادام: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏼‍🦯‍➡' => 'تاياعى بار ادام: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🧑🏽‍🦯‍➡' => 'تاياعى بار ادام: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏾‍🦯‍➡' => 'تاياعى بار ادام: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏿‍🦯‍➡' => 'تاياعى بار ادام: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏻‍🦯‍➡' => 'تاياعى بار ەركەك: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏼‍🦯‍➡' => 'تاياعى بار ەركەك: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '👨🏽‍🦯‍➡' => 'تاياعى بار ەركەك: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏾‍🦯‍➡' => 'تاياعى بار ەركەك: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏿‍🦯‍➡' => 'تاياعى بار ەركەك: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏻‍🦯‍➡' => 'تاياعى بار ايەل: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏼‍🦯‍➡' => 'تاياعى بار ايەل: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '👩🏽‍🦯‍➡' => 'تاياعى بار ايەل: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏾‍🦯‍➡' => 'تاياعى بار ايەل: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏿‍🦯‍➡' => 'تاياعى بار ايەل: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏻‍🦼‍➡' => 'موتورلى ارباداعى ادام: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏼‍🦼‍➡' => 'موتورلى ارباداعى ادام: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🧑🏽‍🦼‍➡' => 'موتورلى ارباداعى ادام: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏾‍🦼‍➡' => 'موتورلى ارباداعى ادام: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏿‍🦼‍➡' => 'موتورلى ارباداعى ادام: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏻‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏼‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '👨🏽‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏾‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏿‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏻‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏼‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '👩🏽‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏾‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏿‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏻‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ادام: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏼‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ادام: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🧑🏽‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ادام: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏾‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ادام: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏿‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ادام: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏻‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ەركەك: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏼‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ەركەك: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '👨🏽‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ەركەك: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏾‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ەركەك: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👨🏿‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ەركەك: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏻‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ايەل: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏼‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ايەل: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '👩🏽‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ايەل: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏾‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ايەل: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👩🏿‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ايەل: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏻‍♀‍➡' => 'جۇگىرگەن ايەل: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏼‍♀‍➡' => 'جۇگىرگەن ايەل: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🏃🏽‍♀‍➡' => 'جۇگىرگەن ايەل: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏾‍♀‍➡' => 'جۇگىرگەن ايەل: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏿‍♀‍➡' => 'جۇگىرگەن ايەل: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏻‍♂‍➡' => 'جۇگىرگەن ەر: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏼‍♂‍➡' => 'جۇگىرگەن ەر: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🏃🏽‍♂‍➡' => 'جۇگىرگەن ەر: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏾‍♂‍➡' => 'جۇگىرگەن ەر: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏿‍♂‍➡' => 'جۇگىرگەن ەر: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🫱🏻‍🫲🏼' => 'قول الىسۋ: 1–2 تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🫱🏻‍🫲🏽' => 'قول الىسۋ: 1–2 تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🫱🏻‍🫲🏾' => 'قول الىسۋ: 1–2 تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🫱🏻‍🫲🏿' => 'قول الىسۋ: 1–2 تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🫱🏼‍🫲🏻' => 'قول الىسۋ: 3-تەرى ءتۇرى، 1–2 تەرى ءتۇسى',
    '🫱🏼‍🫲🏽' => 'قول الىسۋ: 3-تەرى ءتۇرى، 4-تەرى ءتۇسى',
    '🫱🏼‍🫲🏾' => 'قول الىسۋ: 3-تەرى ءتۇرى، 5-تەرى ءتۇسى',
    '🫱🏼‍🫲🏿' => 'قول الىسۋ: 3-تەرى ءتۇرى، 6-تەرى ءتۇسى',
    '🫱🏽‍🫲🏻' => 'قول الىسۋ: 4-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🫱🏽‍🫲🏼' => 'قول الىسۋ: 4-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🫱🏽‍🫲🏾' => 'قول الىسۋ: 4-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🫱🏽‍🫲🏿' => 'قول الىسۋ: 4-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🫱🏾‍🫲🏻' => 'قول الىسۋ: 5-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🫱🏾‍🫲🏼' => 'قول الىسۋ: 5-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🫱🏾‍🫲🏽' => 'قول الىسۋ: 5-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🫱🏾‍🫲🏿' => 'قول الىسۋ: 5-تەرى ءتۇسى، 6-تەرى ءتۇسى',
    '🫱🏿‍🫲🏻' => 'قول الىسۋ: 6-تەرى ءتۇسى، 1–2 تەرى ءتۇسى',
    '🫱🏿‍🫲🏼' => 'قول الىسۋ: 6-تەرى ءتۇسى، 3-تەرى ءتۇرى',
    '🫱🏿‍🫲🏽' => 'قول الىسۋ: 6-تەرى ءتۇسى، 4-تەرى ءتۇسى',
    '🫱🏿‍🫲🏾' => 'قول الىسۋ: 6-تەرى ءتۇسى، 5-تەرى ءتۇسى',
    '🚶‍♀‍➡' => 'جاياۋ ايەل: وڭعا قاراتىلعان',
    '🚶‍♂‍➡' => 'جاياۋ ەر: وڭعا قاراتىلعان',
    '🧎‍♀‍➡' => 'تىزەرلەپ وتىرعان ايەل: وڭعا قاراتىلعان',
    '🧎‍♂‍➡' => 'تىزەرلەپ وتىرعان ەركەك: وڭعا قاراتىلعان',
    '🧑‍🦯‍➡' => 'تاياعى بار ادام: وڭعا قاراتىلعان',
    '👨‍🦯‍➡' => 'تاياعى بار ەركەك: وڭعا قاراتىلعان',
    '👩‍🦯‍➡' => 'تاياعى بار ايەل: وڭعا قاراتىلعان',
    '🧑‍🦼‍➡' => 'موتورلى ارباداعى ادام: وڭعا قاراتىلعان',
    '👨‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: وڭعا قاراتىلعان',
    '👩‍🦼‍➡' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: وڭعا قاراتىلعان',
    '🧑‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ادام: وڭعا قاراتىلعان',
    '👨‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ەركەك: وڭعا قاراتىلعان',
    '👩‍🦽‍➡' => 'قول جەتەگى بار ارباشاداعى ايەل: وڭعا قاراتىلعان',
    '🏃‍♀‍➡' => 'جۇگىرگەن ايەل: وڭعا قاراتىلعان',
    '🏃‍♂‍➡' => 'جۇگىرگەن ەر: وڭعا قاراتىلعان',
    '👩‍❤‍👨' => 'ماحاببات: ايەل، ەر ادام',
    '👨‍❤‍👨' => 'ماحاببات: ەر ادام، ەر ادام',
    '👩‍❤‍👩' => 'ماحاببات: ايەل، ايەل',
    '👨‍👩‍👦' => 'وتباسى: ەر ادام، ايەل، ۇل',
    '👨‍👩‍👧' => 'وتباسى: ەر ادام، ايەل، قىز',
    '👨‍👨‍👦' => 'وتباسى: ەر ادام، ەر ادام، ۇل',
    '👨‍👨‍👧' => 'وتباسى: ەر ادام، ەر ادام، قىز',
    '👩‍👩‍👦' => 'وتباسى: ايەل، ايەل، ۇل',
    '👩‍👩‍👧' => 'وتباسى: ايەل، ايەل، قىز',
    '👨‍👦‍👦' => 'وتباسى: ەر ادام، ۇل، ۇل',
    '👨‍👧‍👦' => 'وتباسى: ەر ادام، قىز، ۇل',
    '👨‍👧‍👧' => 'وتباسى: ەر ادام، قىز، قىز',
    '👩‍👦‍👦' => 'وتباسى: ايەل، ۇل، ۇل',
    '👩‍👧‍👦' => 'وتباسى: ايەل، قىز، ۇل',
    '👩‍👧‍👧' => 'وتباسى: ايەل، قىز، قىز',
    '🧑‍🤝‍🧑' => 'قولدارىن كوتەرىپ تۇرعان ادامدار',
    '🧑‍🧑‍🧒' => 'وتباسى: ەرەسەك، ەرەسەك، بالا',
    '🧑‍🧒‍🧒' => 'وتباسى: ەرەسەك، بالا، بالا',
    '🧑🏻‍🦰' => 'ادام: 1–2 تەرى ءتۇسى، جيرەن شاش',
    '🧑🏼‍🦰' => 'ادام: 3-تەرى ءتۇرى، جيرەن شاش',
    '🧑🏽‍🦰' => 'ادام: 4-تەرى ءتۇسى، جيرەن شاش',
    '🧑🏾‍🦰' => 'ادام: 5-تەرى ءتۇسى، جيرەن شاش',
    '🧑🏿‍🦰' => 'ادام: 6-تەرى ءتۇسى، جيرەن شاش',
    '🧑🏻‍🦱' => 'ادام: 1–2 تەرى ءتۇسى، بۇيرا شاش',
    '🧑🏼‍🦱' => 'ادام: 3-تەرى ءتۇرى، بۇيرا شاش',
    '🧑🏽‍🦱' => 'ادام: 4-تەرى ءتۇسى، بۇيرا شاش',
    '🧑🏾‍🦱' => 'ادام: 5-تەرى ءتۇسى، بۇيرا شاش',
    '🧑🏿‍🦱' => 'ادام: 6-تەرى ءتۇسى، بۇيرا شاش',
    '🧑🏻‍🦳' => 'ادام: 1–2 تەرى ءتۇسى، اق شاش',
    '🧑🏼‍🦳' => 'ادام: 3-تەرى ءتۇرى، اق شاش',
    '🧑🏽‍🦳' => 'ادام: 4-تەرى ءتۇسى، اق شاش',
    '🧑🏾‍🦳' => 'ادام: 5-تەرى ءتۇسى، اق شاش',
    '🧑🏿‍🦳' => 'ادام: 6-تەرى ءتۇسى، اق شاش',
    '🧑🏻‍🦲' => 'ادام: 1–2 تەرى ءتۇسى، تاقىرباس',
    '🧑🏼‍🦲' => 'ادام: 3-تەرى ءتۇرى، تاقىرباس',
    '🧑🏽‍🦲' => 'ادام: 4-تەرى ءتۇسى، تاقىرباس',
    '🧑🏾‍🦲' => 'ادام: 5-تەرى ءتۇسى، تاقىرباس',
    '🧑🏿‍🦲' => 'ادام: 6-تەرى ءتۇسى، تاقىرباس',
    '🧑🏻‍🩰' => 'بالەرينا: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🩰' => 'بالەرينا: 3-تەرى ءتۇرى',
    '🧑🏽‍🩰' => 'بالەرينا: 4-تەرى ءتۇسى',
    '🧑🏾‍🩰' => 'بالەرينا: 5-تەرى ءتۇسى',
    '🧑🏿‍🩰' => 'بالەرينا: 6-تەرى ءتۇسى',
    '🧔🏻‍♂' => 'ساقالدى ەر ادام: 1–2 تەرى ءتۇسى',
    '🧔🏼‍♂' => 'ساقالدى ەر ادام: 3-تەرى ءتۇرى',
    '🧔🏽‍♂' => 'ساقالدى ەر ادام: 4-تەرى ءتۇسى',
    '🧔🏾‍♂' => 'ساقالدى ەر ادام: 5-تەرى ءتۇسى',
    '🧔🏿‍♂' => 'ساقالدى ەر ادام: 6-تەرى ءتۇسى',
    '👱🏻‍♂' => 'اققۇبا ەر ادام: 1–2 تەرى ءتۇسى',
    '👱🏼‍♂' => 'اققۇبا ەر ادام: 3-تەرى ءتۇرى',
    '👱🏽‍♂' => 'اققۇبا ەر ادام: 4-تەرى ءتۇسى',
    '👱🏾‍♂' => 'اققۇبا ەر ادام: 5-تەرى ءتۇسى',
    '👱🏿‍♂' => 'اققۇبا ەر ادام: 6-تەرى ءتۇسى',
    '👨🏻‍🦰' => 'ەر ادام: 1–2 تەرى ءتۇسى، جيرەن شاش',
    '👨🏼‍🦰' => 'ەر ادام: 3-تەرى ءتۇرى، جيرەن شاش',
    '👨🏽‍🦰' => 'ەر ادام: 4-تەرى ءتۇسى، جيرەن شاش',
    '👨🏾‍🦰' => 'ەر ادام: 5-تەرى ءتۇسى، جيرەن شاش',
    '👨🏿‍🦰' => 'ەر ادام: 6-تەرى ءتۇسى، جيرەن شاش',
    '👨🏻‍🦱' => 'ەر ادام: 1–2 تەرى ءتۇسى، بۇيرا شاش',
    '👨🏼‍🦱' => 'ەر ادام: 3-تەرى ءتۇرى، بۇيرا شاش',
    '👨🏽‍🦱' => 'ەر ادام: 4-تەرى ءتۇسى، بۇيرا شاش',
    '👨🏾‍🦱' => 'ەر ادام: 5-تەرى ءتۇسى، بۇيرا شاش',
    '👨🏿‍🦱' => 'ەر ادام: 6-تەرى ءتۇسى، بۇيرا شاش',
    '👨🏻‍🦳' => 'ەر ادام: 1–2 تەرى ءتۇسى، اق شاش',
    '👨🏼‍🦳' => 'ەر ادام: 3-تەرى ءتۇرى، اق شاش',
    '👨🏽‍🦳' => 'ەر ادام: 4-تەرى ءتۇسى، اق شاش',
    '👨🏾‍🦳' => 'ەر ادام: 5-تەرى ءتۇسى، اق شاش',
    '👨🏿‍🦳' => 'ەر ادام: 6-تەرى ءتۇسى، اق شاش',
    '👨🏻‍🦲' => 'ەر ادام: 1–2 تەرى ءتۇسى، تاقىرباس',
    '👨🏼‍🦲' => 'ەر ادام: 3-تەرى ءتۇرى، تاقىرباس',
    '👨🏽‍🦲' => 'ەر ادام: 4-تەرى ءتۇسى، تاقىرباس',
    '👨🏾‍🦲' => 'ەر ادام: 5-تەرى ءتۇسى، تاقىرباس',
    '👨🏿‍🦲' => 'ەر ادام: 6-تەرى ءتۇسى، تاقىرباس',
    '🧔🏻‍♀' => 'ساقالدى ايەل ادام: 1–2 تەرى ءتۇسى',
    '🧔🏼‍♀' => 'ساقالدى ايەل ادام: 3-تەرى ءتۇرى',
    '🧔🏽‍♀' => 'ساقالدى ايەل ادام: 4-تەرى ءتۇسى',
    '🧔🏾‍♀' => 'ساقالدى ايەل ادام: 5-تەرى ءتۇسى',
    '🧔🏿‍♀' => 'ساقالدى ايەل ادام: 6-تەرى ءتۇسى',
    '👱🏻‍♀' => 'اققۇبا ايەل: 1–2 تەرى ءتۇسى',
    '👱🏼‍♀' => 'اققۇبا ايەل: 3-تەرى ءتۇرى',
    '👱🏽‍♀' => 'اققۇبا ايەل: 4-تەرى ءتۇسى',
    '👱🏾‍♀' => 'اققۇبا ايەل: 5-تەرى ءتۇسى',
    '👱🏿‍♀' => 'اققۇبا ايەل: 6-تەرى ءتۇسى',
    '👩🏻‍🦰' => 'ايەل: 1–2 تەرى ءتۇسى، جيرەن شاش',
    '👩🏼‍🦰' => 'ايەل: 3-تەرى ءتۇرى، جيرەن شاش',
    '👩🏽‍🦰' => 'ايەل: 4-تەرى ءتۇسى، جيرەن شاش',
    '👩🏾‍🦰' => 'ايەل: 5-تەرى ءتۇسى، جيرەن شاش',
    '👩🏿‍🦰' => 'ايەل: 6-تەرى ءتۇسى، جيرەن شاش',
    '👩🏻‍🦱' => 'ايەل: 1–2 تەرى ءتۇسى، بۇيرا شاش',
    '👩🏼‍🦱' => 'ايەل: 3-تەرى ءتۇرى، بۇيرا شاش',
    '👩🏽‍🦱' => 'ايەل: 4-تەرى ءتۇسى، بۇيرا شاش',
    '👩🏾‍🦱' => 'ايەل: 5-تەرى ءتۇسى، بۇيرا شاش',
    '👩🏿‍🦱' => 'ايەل: 6-تەرى ءتۇسى، بۇيرا شاش',
    '👩🏻‍🦳' => 'ايەل: 1–2 تەرى ءتۇسى، اق شاش',
    '👩🏼‍🦳' => 'ايەل: 3-تەرى ءتۇرى، اق شاش',
    '👩🏽‍🦳' => 'ايەل: 4-تەرى ءتۇسى، اق شاش',
    '👩🏾‍🦳' => 'ايەل: 5-تەرى ءتۇسى، اق شاش',
    '👩🏿‍🦳' => 'ايەل: 6-تەرى ءتۇسى، اق شاش',
    '👩🏻‍🦲' => 'ايەل: 1–2 تەرى ءتۇسى، تاقىرباس',
    '👩🏼‍🦲' => 'ايەل: 3-تەرى ءتۇرى، تاقىرباس',
    '👩🏽‍🦲' => 'ايەل: 4-تەرى ءتۇسى، تاقىرباس',
    '👩🏾‍🦲' => 'ايەل: 5-تەرى ءتۇسى، تاقىرباس',
    '👩🏿‍🦲' => 'ايەل: 6-تەرى ءتۇسى، تاقىرباس',
    '🙍🏻‍♂' => 'اشۋلى ەر: 1–2 تەرى ءتۇسى',
    '🙍🏼‍♂' => 'اشۋلى ەر: 3-تەرى ءتۇرى',
    '🙍🏽‍♂' => 'اشۋلى ەر: 4-تەرى ءتۇسى',
    '🙍🏾‍♂' => 'اشۋلى ەر: 5-تەرى ءتۇسى',
    '🙍🏿‍♂' => 'اشۋلى ەر: 6-تەرى ءتۇسى',
    '🙍🏻‍♀' => 'اشۋلى ايەل: 1–2 تەرى ءتۇسى',
    '🙍🏼‍♀' => 'اشۋلى ايەل: 3-تەرى ءتۇرى',
    '🙍🏽‍♀' => 'اشۋلى ايەل: 4-تەرى ءتۇسى',
    '🙍🏾‍♀' => 'اشۋلى ايەل: 5-تەرى ءتۇسى',
    '🙍🏿‍♀' => 'اشۋلى ايەل: 6-تەرى ءتۇسى',
    '🙎🏻‍♂' => 'رەنجۋلى ەر: 1–2 تەرى ءتۇسى',
    '🙎🏼‍♂' => 'رەنجۋلى ەر: 3-تەرى ءتۇرى',
    '🙎🏽‍♂' => 'رەنجۋلى ەر: 4-تەرى ءتۇسى',
    '🙎🏾‍♂' => 'رەنجۋلى ەر: 5-تەرى ءتۇسى',
    '🙎🏿‍♂' => 'رەنجۋلى ەر: 6-تەرى ءتۇسى',
    '🙎🏻‍♀' => 'رەنجۋلى ايەل: 1–2 تەرى ءتۇسى',
    '🙎🏼‍♀' => 'رەنجۋلى ايەل: 3-تەرى ءتۇرى',
    '🙎🏽‍♀' => 'رەنجۋلى ايەل: 4-تەرى ءتۇسى',
    '🙎🏾‍♀' => 'رەنجۋلى ايەل: 5-تەرى ءتۇسى',
    '🙎🏿‍♀' => 'رەنجۋلى ايەل: 6-تەرى ءتۇسى',
    '🙅🏻‍♂' => 'قارسىمىن دەپ تۇرعان ەر: 1–2 تەرى ءتۇسى',
    '🙅🏼‍♂' => 'قارسىمىن دەپ تۇرعان ەر: 3-تەرى ءتۇرى',
    '🙅🏽‍♂' => 'قارسىمىن دەپ تۇرعان ەر: 4-تەرى ءتۇسى',
    '🙅🏾‍♂' => 'قارسىمىن دەپ تۇرعان ەر: 5-تەرى ءتۇسى',
    '🙅🏿‍♂' => 'قارسىمىن دەپ تۇرعان ەر: 6-تەرى ءتۇسى',
    '🙅🏻‍♀' => 'قارسىمىن دەپ تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '🙅🏼‍♀' => 'قارسىمىن دەپ تۇرعان ايەل: 3-تەرى ءتۇرى',
    '🙅🏽‍♀' => 'قارسىمىن دەپ تۇرعان ايەل: 4-تەرى ءتۇسى',
    '🙅🏾‍♀' => 'قارسىمىن دەپ تۇرعان ايەل: 5-تەرى ءتۇسى',
    '🙅🏿‍♀' => 'قارسىمىن دەپ تۇرعان ايەل: 6-تەرى ءتۇسى',
    '🙆🏻‍♂' => 'قۇپتاپ تۇرعان ەر: 1–2 تەرى ءتۇسى',
    '🙆🏼‍♂' => 'قۇپتاپ تۇرعان ەر: 3-تەرى ءتۇرى',
    '🙆🏽‍♂' => 'قۇپتاپ تۇرعان ەر: 4-تەرى ءتۇسى',
    '🙆🏾‍♂' => 'قۇپتاپ تۇرعان ەر: 5-تەرى ءتۇسى',
    '🙆🏿‍♂' => 'قۇپتاپ تۇرعان ەر: 6-تەرى ءتۇسى',
    '🙆🏻‍♀' => 'قۇپتاپ تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '🙆🏼‍♀' => 'قۇپتاپ تۇرعان ايەل: 3-تەرى ءتۇرى',
    '🙆🏽‍♀' => 'قۇپتاپ تۇرعان ايەل: 4-تەرى ءتۇسى',
    '🙆🏾‍♀' => 'قۇپتاپ تۇرعان ايەل: 5-تەرى ءتۇسى',
    '🙆🏿‍♀' => 'قۇپتاپ تۇرعان ايەل: 6-تەرى ءتۇسى',
    '💁🏻‍♂' => 'قولىن سوزىپ تۇرعان ەر: 1–2 تەرى ءتۇسى',
    '💁🏼‍♂' => 'قولىن سوزىپ تۇرعان ەر: 3-تەرى ءتۇرى',
    '💁🏽‍♂' => 'قولىن سوزىپ تۇرعان ەر: 4-تەرى ءتۇسى',
    '💁🏾‍♂' => 'قولىن سوزىپ تۇرعان ەر: 5-تەرى ءتۇسى',
    '💁🏿‍♂' => 'قولىن سوزىپ تۇرعان ەر: 6-تەرى ءتۇسى',
    '💁🏻‍♀' => 'قولىن سوزىپ تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '💁🏼‍♀' => 'قولىن سوزىپ تۇرعان ايەل: 3-تەرى ءتۇرى',
    '💁🏽‍♀' => 'قولىن سوزىپ تۇرعان ايەل: 4-تەرى ءتۇسى',
    '💁🏾‍♀' => 'قولىن سوزىپ تۇرعان ايەل: 5-تەرى ءتۇسى',
    '💁🏿‍♀' => 'قولىن سوزىپ تۇرعان ايەل: 6-تەرى ءتۇسى',
    '🙋🏻‍♂' => 'قولىن كوتەرىپ تۇرعان ەر: 1–2 تەرى ءتۇسى',
    '🙋🏼‍♂' => 'قولىن كوتەرىپ تۇرعان ەر: 3-تەرى ءتۇرى',
    '🙋🏽‍♂' => 'قولىن كوتەرىپ تۇرعان ەر: 4-تەرى ءتۇسى',
    '🙋🏾‍♂' => 'قولىن كوتەرىپ تۇرعان ەر: 5-تەرى ءتۇسى',
    '🙋🏿‍♂' => 'قولىن كوتەرىپ تۇرعان ەر: 6-تەرى ءتۇسى',
    '🙋🏻‍♀' => 'قولىن كوتەرىپ تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '🙋🏼‍♀' => 'قولىن كوتەرىپ تۇرعان ايەل: 3-تەرى ءتۇرى',
    '🙋🏽‍♀' => 'قولىن كوتەرىپ تۇرعان ايەل: 4-تەرى ءتۇسى',
    '🙋🏾‍♀' => 'قولىن كوتەرىپ تۇرعان ايەل: 5-تەرى ءتۇسى',
    '🙋🏿‍♀' => 'قولىن كوتەرىپ تۇرعان ايەل: 6-تەرى ءتۇسى',
    '🧏🏻‍♂' => 'ساڭىراۋ ەركەك: 1–2 تەرى ءتۇسى',
    '🧏🏼‍♂' => 'ساڭىراۋ ەركەك: 3-تەرى ءتۇرى',
    '🧏🏽‍♂' => 'ساڭىراۋ ەركەك: 4-تەرى ءتۇسى',
    '🧏🏾‍♂' => 'ساڭىراۋ ەركەك: 5-تەرى ءتۇسى',
    '🧏🏿‍♂' => 'ساڭىراۋ ەركەك: 6-تەرى ءتۇسى',
    '🧏🏻‍♀' => 'ساڭىراۋ ايەل: 1–2 تەرى ءتۇسى',
    '🧏🏼‍♀' => 'ساڭىراۋ ايەل: 3-تەرى ءتۇرى',
    '🧏🏽‍♀' => 'ساڭىراۋ ايەل: 4-تەرى ءتۇسى',
    '🧏🏾‍♀' => 'ساڭىراۋ ايەل: 5-تەرى ءتۇسى',
    '🧏🏿‍♀' => 'ساڭىراۋ ايەل: 6-تەرى ءتۇسى',
    '🙇🏻‍♂' => 'باسىن ءيىپ تۇرعان ەر: 1–2 تەرى ءتۇسى',
    '🙇🏼‍♂' => 'باسىن ءيىپ تۇرعان ەر: 3-تەرى ءتۇرى',
    '🙇🏽‍♂' => 'باسىن ءيىپ تۇرعان ەر: 4-تەرى ءتۇسى',
    '🙇🏾‍♂' => 'باسىن ءيىپ تۇرعان ەر: 5-تەرى ءتۇسى',
    '🙇🏿‍♂' => 'باسىن ءيىپ تۇرعان ەر: 6-تەرى ءتۇسى',
    '🙇🏻‍♀' => 'باسىن ءيىپ تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '🙇🏼‍♀' => 'باسىن ءيىپ تۇرعان ايەل: 3-تەرى ءتۇرى',
    '🙇🏽‍♀' => 'باسىن ءيىپ تۇرعان ايەل: 4-تەرى ءتۇسى',
    '🙇🏾‍♀' => 'باسىن ءيىپ تۇرعان ايەل: 5-تەرى ءتۇسى',
    '🙇🏿‍♀' => 'باسىن ءيىپ تۇرعان ايەل: 6-تەرى ءتۇسى',
    '🤦🏻‍♂' => 'ايتۋعا ءسوز تاپپاي تۇرعان ەر: 1–2 تەرى ءتۇسى',
    '🤦🏼‍♂' => 'ايتۋعا ءسوز تاپپاي تۇرعان ەر: 3-تەرى ءتۇرى',
    '🤦🏽‍♂' => 'ايتۋعا ءسوز تاپپاي تۇرعان ەر: 4-تەرى ءتۇسى',
    '🤦🏾‍♂' => 'ايتۋعا ءسوز تاپپاي تۇرعان ەر: 5-تەرى ءتۇسى',
    '🤦🏿‍♂' => 'ايتۋعا ءسوز تاپپاي تۇرعان ەر: 6-تەرى ءتۇسى',
    '🤦🏻‍♀' => 'ايتۋعا ءسوز تاپپاي تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '🤦🏼‍♀' => 'ايتۋعا ءسوز تاپپاي تۇرعان ايەل: 3-تەرى ءتۇرى',
    '🤦🏽‍♀' => 'ايتۋعا ءسوز تاپپاي تۇرعان ايەل: 4-تەرى ءتۇسى',
    '🤦🏾‍♀' => 'ايتۋعا ءسوز تاپپاي تۇرعان ايەل: 5-تەرى ءتۇسى',
    '🤦🏿‍♀' => 'ايتۋعا ءسوز تاپپاي تۇرعان ايەل: 6-تەرى ءتۇسى',
    '🤷🏻‍♂' => 'يىعىن كوتەرىپ تۇرعان ەر: 1–2 تەرى ءتۇسى',
    '🤷🏼‍♂' => 'يىعىن كوتەرىپ تۇرعان ەر: 3-تەرى ءتۇرى',
    '🤷🏽‍♂' => 'يىعىن كوتەرىپ تۇرعان ەر: 4-تەرى ءتۇسى',
    '🤷🏾‍♂' => 'يىعىن كوتەرىپ تۇرعان ەر: 5-تەرى ءتۇسى',
    '🤷🏿‍♂' => 'يىعىن كوتەرىپ تۇرعان ەر: 6-تەرى ءتۇسى',
    '🤷🏻‍♀' => 'يىعىن كوتەرىپ تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '🤷🏼‍♀' => 'يىعىن كوتەرىپ تۇرعان ايەل: 3-تەرى ءتۇرى',
    '🤷🏽‍♀' => 'يىعىن كوتەرىپ تۇرعان ايەل: 4-تەرى ءتۇسى',
    '🤷🏾‍♀' => 'يىعىن كوتەرىپ تۇرعان ايەل: 5-تەرى ءتۇسى',
    '🤷🏿‍♀' => 'يىعىن كوتەرىپ تۇرعان ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍⚕' => 'مەديتسينالىق قىزمەتكەر: 1–2 تەرى ءتۇسى',
    '🧑🏼‍⚕' => 'مەديتسينالىق قىزمەتكەر: 3-تەرى ءتۇرى',
    '🧑🏽‍⚕' => 'مەديتسينالىق قىزمەتكەر: 4-تەرى ءتۇسى',
    '🧑🏾‍⚕' => 'مەديتسينالىق قىزمەتكەر: 5-تەرى ءتۇسى',
    '🧑🏿‍⚕' => 'مەديتسينالىق قىزمەتكەر: 6-تەرى ءتۇسى',
    '👨🏻‍⚕' => 'دارىگەر: 1–2 تەرى ءتۇسى',
    '👨🏼‍⚕' => 'دارىگەر: 3-تەرى ءتۇرى',
    '👨🏽‍⚕' => 'دارىگەر: 4-تەرى ءتۇسى',
    '👨🏾‍⚕' => 'دارىگەر: 5-تەرى ءتۇسى',
    '👨🏿‍⚕' => 'دارىگەر: 6-تەرى ءتۇسى',
    '👩🏻‍⚕' => 'دارىگەر ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍⚕' => 'دارىگەر ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍⚕' => 'دارىگەر ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍⚕' => 'دارىگەر ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍⚕' => 'دارىگەر ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🎓' => 'ستۋدەنت: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🎓' => 'ستۋدەنت: 3-تەرى ءتۇرى',
    '🧑🏽‍🎓' => 'ستۋدەنت: 4-تەرى ءتۇسى',
    '🧑🏾‍🎓' => 'ستۋدەنت: 5-تەرى ءتۇسى',
    '🧑🏿‍🎓' => 'ستۋدەنت: 6-تەرى ءتۇسى',
    '👨🏻‍🎓' => 'ستۋدەنت بالا: 1–2 تەرى ءتۇسى',
    '👨🏼‍🎓' => 'ستۋدەنت بالا: 3-تەرى ءتۇرى',
    '👨🏽‍🎓' => 'ستۋدەنت بالا: 4-تەرى ءتۇسى',
    '👨🏾‍🎓' => 'ستۋدەنت بالا: 5-تەرى ءتۇسى',
    '👨🏿‍🎓' => 'ستۋدەنت بالا: 6-تەرى ءتۇسى',
    '👩🏻‍🎓' => 'ستۋدەنت قىز: 1–2 تەرى ءتۇسى',
    '👩🏼‍🎓' => 'ستۋدەنت قىز: 3-تەرى ءتۇرى',
    '👩🏽‍🎓' => 'ستۋدەنت قىز: 4-تەرى ءتۇسى',
    '👩🏾‍🎓' => 'ستۋدەنت قىز: 5-تەرى ءتۇسى',
    '👩🏿‍🎓' => 'ستۋدەنت قىز: 6-تەرى ءتۇسى',
    '🧑🏻‍🏫' => 'وقىتۋشى: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🏫' => 'وقىتۋشى: 3-تەرى ءتۇرى',
    '🧑🏽‍🏫' => 'وقىتۋشى: 4-تەرى ءتۇسى',
    '🧑🏾‍🏫' => 'وقىتۋشى: 5-تەرى ءتۇسى',
    '🧑🏿‍🏫' => 'وقىتۋشى: 6-تەرى ءتۇسى',
    '👨🏻‍🏫' => 'مۇعالىم: 1–2 تەرى ءتۇسى',
    '👨🏼‍🏫' => 'مۇعالىم: 3-تەرى ءتۇرى',
    '👨🏽‍🏫' => 'مۇعالىم: 4-تەرى ءتۇسى',
    '👨🏾‍🏫' => 'مۇعالىم: 5-تەرى ءتۇسى',
    '👨🏿‍🏫' => 'مۇعالىم: 6-تەرى ءتۇسى',
    '👩🏻‍🏫' => 'مۇعالىم ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🏫' => 'مۇعالىم ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🏫' => 'مۇعالىم ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🏫' => 'مۇعالىم ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🏫' => 'مۇعالىم ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍⚖' => 'سوت: 1–2 تەرى ءتۇسى',
    '🧑🏼‍⚖' => 'سوت: 3-تەرى ءتۇرى',
    '🧑🏽‍⚖' => 'سوت: 4-تەرى ءتۇسى',
    '🧑🏾‍⚖' => 'سوت: 5-تەرى ءتۇسى',
    '🧑🏿‍⚖' => 'سوت: 6-تەرى ءتۇسى',
    '👨🏻‍⚖' => 'سوت ەر: 1–2 تەرى ءتۇسى',
    '👨🏼‍⚖' => 'سوت ەر: 3-تەرى ءتۇرى',
    '👨🏽‍⚖' => 'سوت ەر: 4-تەرى ءتۇسى',
    '👨🏾‍⚖' => 'سوت ەر: 5-تەرى ءتۇسى',
    '👨🏿‍⚖' => 'سوت ەر: 6-تەرى ءتۇسى',
    '👩🏻‍⚖' => 'سوت ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍⚖' => 'سوت ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍⚖' => 'سوت ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍⚖' => 'سوت ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍⚖' => 'سوت ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🌾' => 'ەگىنشى: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🌾' => 'ەگىنشى: 3-تەرى ءتۇرى',
    '🧑🏽‍🌾' => 'ەگىنشى: 4-تەرى ءتۇسى',
    '🧑🏾‍🌾' => 'ەگىنشى: 5-تەرى ءتۇسى',
    '🧑🏿‍🌾' => 'ەگىنشى: 6-تەرى ءتۇسى',
    '👨🏻‍🌾' => 'ەر ەگىنشى: 1–2 تەرى ءتۇسى',
    '👨🏼‍🌾' => 'ەر ەگىنشى: 3-تەرى ءتۇرى',
    '👨🏽‍🌾' => 'ەر ەگىنشى: 4-تەرى ءتۇسى',
    '👨🏾‍🌾' => 'ەر ەگىنشى: 5-تەرى ءتۇسى',
    '👨🏿‍🌾' => 'ەر ەگىنشى: 6-تەرى ءتۇسى',
    '👩🏻‍🌾' => 'ايەل ەگىنشى: 1–2 تەرى ءتۇسى',
    '👩🏼‍🌾' => 'ايەل ەگىنشى: 3-تەرى ءتۇرى',
    '👩🏽‍🌾' => 'ايەل ەگىنشى: 4-تەرى ءتۇسى',
    '👩🏾‍🌾' => 'ايەل ەگىنشى: 5-تەرى ءتۇسى',
    '👩🏿‍🌾' => 'ايەل ەگىنشى: 6-تەرى ءتۇسى',
    '🧑🏻‍🍳' => 'اسپاز: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🍳' => 'اسپاز: 3-تەرى ءتۇرى',
    '🧑🏽‍🍳' => 'اسپاز: 4-تەرى ءتۇسى',
    '🧑🏾‍🍳' => 'اسپاز: 5-تەرى ءتۇسى',
    '🧑🏿‍🍳' => 'اسپاز: 6-تەرى ءتۇسى',
    '👨🏻‍🍳' => 'ەر اسپاز: 1–2 تەرى ءتۇسى',
    '👨🏼‍🍳' => 'ەر اسپاز: 3-تەرى ءتۇرى',
    '👨🏽‍🍳' => 'ەر اسپاز: 4-تەرى ءتۇسى',
    '👨🏾‍🍳' => 'ەر اسپاز: 5-تەرى ءتۇسى',
    '👨🏿‍🍳' => 'ەر اسپاز: 6-تەرى ءتۇسى',
    '👩🏻‍🍳' => 'ايەل اسپاز: 1–2 تەرى ءتۇسى',
    '👩🏼‍🍳' => 'ايەل اسپاز: 3-تەرى ءتۇرى',
    '👩🏽‍🍳' => 'ايەل اسپاز: 4-تەرى ءتۇسى',
    '👩🏾‍🍳' => 'ايەل اسپاز: 5-تەرى ءتۇسى',
    '👩🏿‍🍳' => 'ايەل اسپاز: 6-تەرى ءتۇسى',
    '🧑🏻‍🔧' => 'مەحانيك: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🔧' => 'مەحانيك: 3-تەرى ءتۇرى',
    '🧑🏽‍🔧' => 'مەحانيك: 4-تەرى ءتۇسى',
    '🧑🏾‍🔧' => 'مەحانيك: 5-تەرى ءتۇسى',
    '🧑🏿‍🔧' => 'مەحانيك: 6-تەرى ءتۇسى',
    '👨🏻‍🔧' => 'ەر مەحانيك: 1–2 تەرى ءتۇسى',
    '👨🏼‍🔧' => 'ەر مەحانيك: 3-تەرى ءتۇرى',
    '👨🏽‍🔧' => 'ەر مەحانيك: 4-تەرى ءتۇسى',
    '👨🏾‍🔧' => 'ەر مەحانيك: 5-تەرى ءتۇسى',
    '👨🏿‍🔧' => 'ەر مەحانيك: 6-تەرى ءتۇسى',
    '👩🏻‍🔧' => 'مەحانيك ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🔧' => 'مەحانيك ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🔧' => 'مەحانيك ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🔧' => 'مەحانيك ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🔧' => 'مەحانيك ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🏭' => 'زاۋىت جۇمىسشىسى: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🏭' => 'زاۋىت جۇمىسشىسى: 3-تەرى ءتۇرى',
    '🧑🏽‍🏭' => 'زاۋىت جۇمىسشىسى: 4-تەرى ءتۇسى',
    '🧑🏾‍🏭' => 'زاۋىت جۇمىسشىسى: 5-تەرى ءتۇسى',
    '🧑🏿‍🏭' => 'زاۋىت جۇمىسشىسى: 6-تەرى ءتۇسى',
    '👨🏻‍🏭' => 'جۇمىسشى: 1–2 تەرى ءتۇسى',
    '👨🏼‍🏭' => 'جۇمىسشى: 3-تەرى ءتۇرى',
    '👨🏽‍🏭' => 'جۇمىسشى: 4-تەرى ءتۇسى',
    '👨🏾‍🏭' => 'جۇمىسشى: 5-تەرى ءتۇسى',
    '👨🏿‍🏭' => 'جۇمىسشى: 6-تەرى ءتۇسى',
    '👩🏻‍🏭' => 'جۇمىسشى ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🏭' => 'جۇمىسشى ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🏭' => 'جۇمىسشى ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🏭' => 'جۇمىسشى ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🏭' => 'جۇمىسشى ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍💼' => 'كەڭسە قىزمەتكەرى: 1–2 تەرى ءتۇسى',
    '🧑🏼‍💼' => 'كەڭسە قىزمەتكەرى: 3-تەرى ءتۇرى',
    '🧑🏽‍💼' => 'كەڭسە قىزمەتكەرى: 4-تەرى ءتۇسى',
    '🧑🏾‍💼' => 'كەڭسە قىزمەتكەرى: 5-تەرى ءتۇسى',
    '🧑🏿‍💼' => 'كەڭسە قىزمەتكەرى: 6-تەرى ءتۇسى',
    '👨🏻‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر: 1–2 تەرى ءتۇسى',
    '👨🏼‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر: 3-تەرى ءتۇرى',
    '👨🏽‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر: 4-تەرى ءتۇسى',
    '👨🏾‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر: 5-تەرى ءتۇسى',
    '👨🏿‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر: 6-تەرى ءتۇسى',
    '👩🏻‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🔬' => 'عالىم: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🔬' => 'عالىم: 3-تەرى ءتۇرى',
    '🧑🏽‍🔬' => 'عالىم: 4-تەرى ءتۇسى',
    '🧑🏾‍🔬' => 'عالىم: 5-تەرى ءتۇسى',
    '🧑🏿‍🔬' => 'عالىم: 6-تەرى ءتۇسى',
    '👨🏻‍🔬' => 'ەر عالىم: 1–2 تەرى ءتۇسى',
    '👨🏼‍🔬' => 'ەر عالىم: 3-تەرى ءتۇرى',
    '👨🏽‍🔬' => 'ەر عالىم: 4-تەرى ءتۇسى',
    '👨🏾‍🔬' => 'ەر عالىم: 5-تەرى ءتۇسى',
    '👨🏿‍🔬' => 'ەر عالىم: 6-تەرى ءتۇسى',
    '👩🏻‍🔬' => 'عالىم ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🔬' => 'عالىم ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🔬' => 'عالىم ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🔬' => 'عالىم ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🔬' => 'عالىم ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍💻' => 'تەحنولوگ: 1–2 تەرى ءتۇسى',
    '🧑🏼‍💻' => 'تەحنولوگ: 3-تەرى ءتۇرى',
    '🧑🏽‍💻' => 'تەحنولوگ: 4-تەرى ءتۇسى',
    '🧑🏾‍💻' => 'تەحنولوگ: 5-تەرى ءتۇسى',
    '🧑🏿‍💻' => 'تەحنولوگ: 6-تەرى ءتۇسى',
    '👨🏻‍💻' => 'ەر تەحنولوگ: 1–2 تەرى ءتۇسى',
    '👨🏼‍💻' => 'ەر تەحنولوگ: 3-تەرى ءتۇرى',
    '👨🏽‍💻' => 'ەر تەحنولوگ: 4-تەرى ءتۇسى',
    '👨🏾‍💻' => 'ەر تەحنولوگ: 5-تەرى ءتۇسى',
    '👨🏿‍💻' => 'ەر تەحنولوگ: 6-تەرى ءتۇسى',
    '👩🏻‍💻' => 'تەحنولوگ ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍💻' => 'تەحنولوگ ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍💻' => 'تەحنولوگ ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍💻' => 'تەحنولوگ ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍💻' => 'تەحنولوگ ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🎤' => 'ءانشى: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🎤' => 'ءانشى: 3-تەرى ءتۇرى',
    '🧑🏽‍🎤' => 'ءانشى: 4-تەرى ءتۇسى',
    '🧑🏾‍🎤' => 'ءانشى: 5-تەرى ءتۇسى',
    '🧑🏿‍🎤' => 'ءانشى: 6-تەرى ءتۇسى',
    '👨🏻‍🎤' => 'ەر ءانشى: 1–2 تەرى ءتۇسى',
    '👨🏼‍🎤' => 'ەر ءانشى: 3-تەرى ءتۇرى',
    '👨🏽‍🎤' => 'ەر ءانشى: 4-تەرى ءتۇسى',
    '👨🏾‍🎤' => 'ەر ءانشى: 5-تەرى ءتۇسى',
    '👨🏿‍🎤' => 'ەر ءانشى: 6-تەرى ءتۇسى',
    '👩🏻‍🎤' => 'ءانشى ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🎤' => 'ءانشى ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🎤' => 'ءانشى ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🎤' => 'ءانشى ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🎤' => 'ءانشى ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🎨' => 'سۋرەتشى: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🎨' => 'سۋرەتشى: 3-تەرى ءتۇرى',
    '🧑🏽‍🎨' => 'سۋرەتشى: 4-تەرى ءتۇسى',
    '🧑🏾‍🎨' => 'سۋرەتشى: 5-تەرى ءتۇسى',
    '🧑🏿‍🎨' => 'سۋرەتشى: 6-تەرى ءتۇسى',
    '👨🏻‍🎨' => 'ەر سۋرەتشى: 1–2 تەرى ءتۇسى',
    '👨🏼‍🎨' => 'ەر سۋرەتشى: 3-تەرى ءتۇرى',
    '👨🏽‍🎨' => 'ەر سۋرەتشى: 4-تەرى ءتۇسى',
    '👨🏾‍🎨' => 'ەر سۋرەتشى: 5-تەرى ءتۇسى',
    '👨🏿‍🎨' => 'ەر سۋرەتشى: 6-تەرى ءتۇسى',
    '👩🏻‍🎨' => 'سۋرەتشى ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🎨' => 'سۋرەتشى ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🎨' => 'سۋرەتشى ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🎨' => 'سۋرەتشى ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🎨' => 'سۋرەتشى ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍✈' => 'ۇشاق: 1–2 تەرى ءتۇسى',
    '🧑🏼‍✈' => 'ۇشاق: 3-تەرى ءتۇرى',
    '🧑🏽‍✈' => 'ۇشاق: 4-تەرى ءتۇسى',
    '🧑🏾‍✈' => 'ۇشاق: 5-تەرى ءتۇسى',
    '🧑🏿‍✈' => 'ۇشاق: 6-تەرى ءتۇسى',
    '👨🏻‍✈' => 'پيلوت: 1–2 تەرى ءتۇسى',
    '👨🏼‍✈' => 'پيلوت: 3-تەرى ءتۇرى',
    '👨🏽‍✈' => 'پيلوت: 4-تەرى ءتۇسى',
    '👨🏾‍✈' => 'پيلوت: 5-تەرى ءتۇسى',
    '👨🏿‍✈' => 'پيلوت: 6-تەرى ءتۇسى',
    '👩🏻‍✈' => 'ۇشقىش ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍✈' => 'ۇشقىش ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍✈' => 'ۇشقىش ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍✈' => 'ۇشقىش ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍✈' => 'ۇشقىش ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🚀' => 'استروناۆت: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🚀' => 'استروناۆت: 3-تەرى ءتۇرى',
    '🧑🏽‍🚀' => 'استروناۆت: 4-تەرى ءتۇسى',
    '🧑🏾‍🚀' => 'استروناۆت: 5-تەرى ءتۇسى',
    '🧑🏿‍🚀' => 'استروناۆت: 6-تەرى ءتۇسى',
    '👨🏻‍🚀' => 'عارىشكەر: 1–2 تەرى ءتۇسى',
    '👨🏼‍🚀' => 'عارىشكەر: 3-تەرى ءتۇرى',
    '👨🏽‍🚀' => 'عارىشكەر: 4-تەرى ءتۇسى',
    '👨🏾‍🚀' => 'عارىشكەر: 5-تەرى ءتۇسى',
    '👨🏿‍🚀' => 'عارىشكەر: 6-تەرى ءتۇسى',
    '👩🏻‍🚀' => 'عارىشكەر ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🚀' => 'عارىشكەر ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🚀' => 'عارىشكەر ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🚀' => 'عارىشكەر ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🚀' => 'عارىشكەر ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🚒' => 'ءورت ءسوندىرۋشى: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🚒' => 'ءورت ءسوندىرۋشى: 3-تەرى ءتۇرى',
    '🧑🏽‍🚒' => 'ءورت ءسوندىرۋشى: 4-تەرى ءتۇسى',
    '🧑🏾‍🚒' => 'ءورت ءسوندىرۋشى: 5-تەرى ءتۇسى',
    '🧑🏿‍🚒' => 'ءورت ءسوندىرۋشى: 6-تەرى ءتۇسى',
    '👨🏻‍🚒' => 'ەر ءورت ءسوندىرۋشى: 1–2 تەرى ءتۇسى',
    '👨🏼‍🚒' => 'ەر ءورت ءسوندىرۋشى: 3-تەرى ءتۇرى',
    '👨🏽‍🚒' => 'ەر ءورت ءسوندىرۋشى: 4-تەرى ءتۇسى',
    '👨🏾‍🚒' => 'ەر ءورت ءسوندىرۋشى: 5-تەرى ءتۇسى',
    '👨🏿‍🚒' => 'ەر ءورت ءسوندىرۋشى: 6-تەرى ءتۇسى',
    '👩🏻‍🚒' => 'ءورت ءسوندىرۋشى ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🚒' => 'ءورت ءسوندىرۋشى ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🚒' => 'ءورت ءسوندىرۋشى ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🚒' => 'ءورت ءسوندىرۋشى ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🚒' => 'ءورت ءسوندىرۋشى ايەل: 6-تەرى ءتۇسى',
    '👮🏻‍♂' => 'ەر پوليتسيا قىزمەتكەرى: 1–2 تەرى ءتۇسى',
    '👮🏼‍♂' => 'ەر پوليتسيا قىزمەتكەرى: 3-تەرى ءتۇرى',
    '👮🏽‍♂' => 'ەر پوليتسيا قىزمەتكەرى: 4-تەرى ءتۇسى',
    '👮🏾‍♂' => 'ەر پوليتسيا قىزمەتكەرى: 5-تەرى ءتۇسى',
    '👮🏿‍♂' => 'ەر پوليتسيا قىزمەتكەرى: 6-تەرى ءتۇسى',
    '👮🏻‍♀' => 'ايەل پوليتسيا قىزمەتكەرى: 1–2 تەرى ءتۇسى',
    '👮🏼‍♀' => 'ايەل پوليتسيا قىزمەتكەرى: 3-تەرى ءتۇرى',
    '👮🏽‍♀' => 'ايەل پوليتسيا قىزمەتكەرى: 4-تەرى ءتۇسى',
    '👮🏾‍♀' => 'ايەل پوليتسيا قىزمەتكەرى: 5-تەرى ءتۇسى',
    '👮🏿‍♀' => 'ايەل پوليتسيا قىزمەتكەرى: 6-تەرى ءتۇسى',
    '🕵🏻‍♂' => 'بارلاۋشى ەر ادام: 1–2 تەرى ءتۇسى',
    '🕵🏼‍♂' => 'بارلاۋشى ەر ادام: 3-تەرى ءتۇرى',
    '🕵🏽‍♂' => 'بارلاۋشى ەر ادام: 4-تەرى ءتۇسى',
    '🕵🏾‍♂' => 'بارلاۋشى ەر ادام: 5-تەرى ءتۇسى',
    '🕵🏿‍♂' => 'بارلاۋشى ەر ادام: 6-تەرى ءتۇسى',
    '🕵🏻‍♀' => 'بارلاۋشى ايەل ادام: 1–2 تەرى ءتۇسى',
    '🕵🏼‍♀' => 'بارلاۋشى ايەل ادام: 3-تەرى ءتۇرى',
    '🕵🏽‍♀' => 'بارلاۋشى ايەل ادام: 4-تەرى ءتۇسى',
    '🕵🏾‍♀' => 'بارلاۋشى ايەل ادام: 5-تەرى ءتۇسى',
    '🕵🏿‍♀' => 'بارلاۋشى ايەل ادام: 6-تەرى ءتۇسى',
    '💂🏻‍♂' => 'كۇزەتشى ەر ادام: 1–2 تەرى ءتۇسى',
    '💂🏼‍♂' => 'كۇزەتشى ەر ادام: 3-تەرى ءتۇرى',
    '💂🏽‍♂' => 'كۇزەتشى ەر ادام: 4-تەرى ءتۇسى',
    '💂🏾‍♂' => 'كۇزەتشى ەر ادام: 5-تەرى ءتۇسى',
    '💂🏿‍♂' => 'كۇزەتشى ەر ادام: 6-تەرى ءتۇسى',
    '💂🏻‍♀' => 'كۇزەتشى ايەل: 1–2 تەرى ءتۇسى',
    '💂🏼‍♀' => 'كۇزەتشى ايەل: 3-تەرى ءتۇرى',
    '💂🏽‍♀' => 'كۇزەتشى ايەل: 4-تەرى ءتۇسى',
    '💂🏾‍♀' => 'كۇزەتشى ايەل: 5-تەرى ءتۇسى',
    '💂🏿‍♀' => 'كۇزەتشى ايەل: 6-تەرى ءتۇسى',
    '👷🏻‍♂' => 'قۇرىلىسشى ەر ادام: 1–2 تەرى ءتۇسى',
    '👷🏼‍♂' => 'قۇرىلىسشى ەر ادام: 3-تەرى ءتۇرى',
    '👷🏽‍♂' => 'قۇرىلىسشى ەر ادام: 4-تەرى ءتۇسى',
    '👷🏾‍♂' => 'قۇرىلىسشى ەر ادام: 5-تەرى ءتۇسى',
    '👷🏿‍♂' => 'قۇرىلىسشى ەر ادام: 6-تەرى ءتۇسى',
    '👷🏻‍♀' => 'قۇرىلىسشى ايەل: 1–2 تەرى ءتۇسى',
    '👷🏼‍♀' => 'قۇرىلىسشى ايەل: 3-تەرى ءتۇرى',
    '👷🏽‍♀' => 'قۇرىلىسشى ايەل: 4-تەرى ءتۇسى',
    '👷🏾‍♀' => 'قۇرىلىسشى ايەل: 5-تەرى ءتۇسى',
    '👷🏿‍♀' => 'قۇرىلىسشى ايەل: 6-تەرى ءتۇسى',
    '👳🏻‍♂' => 'سالدەلى ەر ادام: 1–2 تەرى ءتۇسى',
    '👳🏼‍♂' => 'سالدەلى ەر ادام: 3-تەرى ءتۇرى',
    '👳🏽‍♂' => 'سالدەلى ەر ادام: 4-تەرى ءتۇسى',
    '👳🏾‍♂' => 'سالدەلى ەر ادام: 5-تەرى ءتۇسى',
    '👳🏿‍♂' => 'سالدەلى ەر ادام: 6-تەرى ءتۇسى',
    '👳🏻‍♀' => 'سالدەلى ايەل: 1–2 تەرى ءتۇسى',
    '👳🏼‍♀' => 'سالدەلى ايەل: 3-تەرى ءتۇرى',
    '👳🏽‍♀' => 'سالدەلى ايەل: 4-تەرى ءتۇسى',
    '👳🏾‍♀' => 'سالدەلى ايەل: 5-تەرى ءتۇسى',
    '👳🏿‍♀' => 'سالدەلى ايەل: 6-تەرى ءتۇسى',
    '🤵🏻‍♂' => 'سموكيڭ كيگەن ەر ادام: 1–2 تەرى ءتۇسى',
    '🤵🏼‍♂' => 'سموكيڭ كيگەن ەر ادام: 3-تەرى ءتۇرى',
    '🤵🏽‍♂' => 'سموكيڭ كيگەن ەر ادام: 4-تەرى ءتۇسى',
    '🤵🏾‍♂' => 'سموكيڭ كيگەن ەر ادام: 5-تەرى ءتۇسى',
    '🤵🏿‍♂' => 'سموكيڭ كيگەن ەر ادام: 6-تەرى ءتۇسى',
    '🤵🏻‍♀' => 'سموكينگ كيگەن ايەل ادام: 1–2 تەرى ءتۇسى',
    '🤵🏼‍♀' => 'سموكينگ كيگەن ايەل ادام: 3-تەرى ءتۇرى',
    '🤵🏽‍♀' => 'سموكينگ كيگەن ايەل ادام: 4-تەرى ءتۇسى',
    '🤵🏾‍♀' => 'سموكينگ كيگەن ايەل ادام: 5-تەرى ءتۇسى',
    '🤵🏿‍♀' => 'سموكينگ كيگەن ايەل ادام: 6-تەرى ءتۇسى',
    '👰🏻‍♂' => 'جاۋلىق تاققان ەر ادام: 1–2 تەرى ءتۇسى',
    '👰🏼‍♂' => 'جاۋلىق تاققان ەر ادام: 3-تەرى ءتۇرى',
    '👰🏽‍♂' => 'جاۋلىق تاققان ەر ادام: 4-تەرى ءتۇسى',
    '👰🏾‍♂' => 'جاۋلىق تاققان ەر ادام: 5-تەرى ءتۇسى',
    '👰🏿‍♂' => 'جاۋلىق تاققان ەر ادام: 6-تەرى ءتۇسى',
    '👰🏻‍♀' => 'جاۋلىق تاققان ايەل: 1–2 تەرى ءتۇسى',
    '👰🏼‍♀' => 'جاۋلىق تاققان ايەل: 3-تەرى ءتۇرى',
    '👰🏽‍♀' => 'جاۋلىق تاققان ايەل: 4-تەرى ءتۇسى',
    '👰🏾‍♀' => 'جاۋلىق تاققان ايەل: 5-تەرى ءتۇسى',
    '👰🏿‍♀' => 'جاۋلىق تاققان ايەل: 6-تەرى ءتۇسى',
    '👩🏻‍🍼' => 'بالا ەمىزىپ وتىرعان ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🍼' => 'بالا ەمىزىپ وتىرعان ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🍼' => 'بالا ەمىزىپ وتىرعان ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🍼' => 'بالا ەمىزىپ وتىرعان ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🍼' => 'بالا ەمىزىپ وتىرعان ايەل: 6-تەرى ءتۇسى',
    '👨🏻‍🍼' => 'بالا ەمىزىپ وتىرعان ەركەك: 1–2 تەرى ءتۇسى',
    '👨🏼‍🍼' => 'بالا ەمىزىپ وتىرعان ەركەك: 3-تەرى ءتۇرى',
    '👨🏽‍🍼' => 'بالا ەمىزىپ وتىرعان ەركەك: 4-تەرى ءتۇسى',
    '👨🏾‍🍼' => 'بالا ەمىزىپ وتىرعان ەركەك: 5-تەرى ءتۇسى',
    '👨🏿‍🍼' => 'بالا ەمىزىپ وتىرعان ەركەك: 6-تەرى ءتۇسى',
    '🧑🏻‍🍼' => 'بالا ەمىزىپ وتىرعان ادام: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🍼' => 'بالا ەمىزىپ وتىرعان ادام: 3-تەرى ءتۇرى',
    '🧑🏽‍🍼' => 'بالا ەمىزىپ وتىرعان ادام: 4-تەرى ءتۇسى',
    '🧑🏾‍🍼' => 'بالا ەمىزىپ وتىرعان ادام: 5-تەرى ءتۇسى',
    '🧑🏿‍🍼' => 'بالا ەمىزىپ وتىرعان ادام: 6-تەرى ءتۇسى',
    '🧑🏻‍🎄' => 'كلاۋس مىرزا: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🎄' => 'كلاۋس مىرزا: 3-تەرى ءتۇرى',
    '🧑🏽‍🎄' => 'كلاۋس مىرزا: 4-تەرى ءتۇسى',
    '🧑🏾‍🎄' => 'كلاۋس مىرزا: 5-تەرى ءتۇسى',
    '🧑🏿‍🎄' => 'كلاۋس مىرزا: 6-تەرى ءتۇسى',
    '🦸🏻‍♂' => 'ەركەك باتىر: 1–2 تەرى ءتۇسى',
    '🦸🏼‍♂' => 'ەركەك باتىر: 3-تەرى ءتۇرى',
    '🦸🏽‍♂' => 'ەركەك باتىر: 4-تەرى ءتۇسى',
    '🦸🏾‍♂' => 'ەركەك باتىر: 5-تەرى ءتۇسى',
    '🦸🏿‍♂' => 'ەركەك باتىر: 6-تەرى ءتۇسى',
    '🦸🏻‍♀' => 'باتىر ايەل: 1–2 تەرى ءتۇسى',
    '🦸🏼‍♀' => 'باتىر ايەل: 3-تەرى ءتۇرى',
    '🦸🏽‍♀' => 'باتىر ايەل: 4-تەرى ءتۇسى',
    '🦸🏾‍♀' => 'باتىر ايەل: 5-تەرى ءتۇسى',
    '🦸🏿‍♀' => 'باتىر ايەل: 6-تەرى ءتۇسى',
    '🦹🏻‍♂' => 'ەر زالىم: 1–2 تەرى ءتۇسى',
    '🦹🏼‍♂' => 'ەر زالىم: 3-تەرى ءتۇرى',
    '🦹🏽‍♂' => 'ەر زالىم: 4-تەرى ءتۇسى',
    '🦹🏾‍♂' => 'ەر زالىم: 5-تەرى ءتۇسى',
    '🦹🏿‍♂' => 'ەر زالىم: 6-تەرى ءتۇسى',
    '🦹🏻‍♀' => 'ايەل زالىم: 1–2 تەرى ءتۇسى',
    '🦹🏼‍♀' => 'ايەل زالىم: 3-تەرى ءتۇرى',
    '🦹🏽‍♀' => 'ايەل زالىم: 4-تەرى ءتۇسى',
    '🦹🏾‍♀' => 'ايەل زالىم: 5-تەرى ءتۇسى',
    '🦹🏿‍♀' => 'ايەل زالىم: 6-تەرى ءتۇسى',
    '🧙🏻‍♂' => 'ەر سيقىرشى: 1–2 تەرى ءتۇسى',
    '🧙🏼‍♂' => 'ەر سيقىرشى: 3-تەرى ءتۇرى',
    '🧙🏽‍♂' => 'ەر سيقىرشى: 4-تەرى ءتۇسى',
    '🧙🏾‍♂' => 'ەر سيقىرشى: 5-تەرى ءتۇسى',
    '🧙🏿‍♂' => 'ەر سيقىرشى: 6-تەرى ءتۇسى',
    '🧙🏻‍♀' => 'ايەل سيقىرشى: 1–2 تەرى ءتۇسى',
    '🧙🏼‍♀' => 'ايەل سيقىرشى: 3-تەرى ءتۇرى',
    '🧙🏽‍♀' => 'ايەل سيقىرشى: 4-تەرى ءتۇسى',
    '🧙🏾‍♀' => 'ايەل سيقىرشى: 5-تەرى ءتۇسى',
    '🧙🏿‍♀' => 'ايەل سيقىرشى: 6-تەرى ءتۇسى',
    '🧚🏻‍♂' => 'ەر پەرى: 1–2 تەرى ءتۇسى',
    '🧚🏼‍♂' => 'ەر پەرى: 3-تەرى ءتۇرى',
    '🧚🏽‍♂' => 'ەر پەرى: 4-تەرى ءتۇسى',
    '🧚🏾‍♂' => 'ەر پەرى: 5-تەرى ءتۇسى',
    '🧚🏿‍♂' => 'ەر پەرى: 6-تەرى ءتۇسى',
    '🧚🏻‍♀' => 'ايەل پەرى: 1–2 تەرى ءتۇسى',
    '🧚🏼‍♀' => 'ايەل پەرى: 3-تەرى ءتۇرى',
    '🧚🏽‍♀' => 'ايەل پەرى: 4-تەرى ءتۇسى',
    '🧚🏾‍♀' => 'ايەل پەرى: 5-تەرى ءتۇسى',
    '🧚🏿‍♀' => 'ايەل پەرى: 6-تەرى ءتۇسى',
    '🧛🏻‍♂' => 'ەر قانىشەر: 1–2 تەرى ءتۇسى',
    '🧛🏼‍♂' => 'ەر قانىشەر: 3-تەرى ءتۇرى',
    '🧛🏽‍♂' => 'ەر قانىشەر: 4-تەرى ءتۇسى',
    '🧛🏾‍♂' => 'ەر قانىشەر: 5-تەرى ءتۇسى',
    '🧛🏿‍♂' => 'ەر قانىشەر: 6-تەرى ءتۇسى',
    '🧛🏻‍♀' => 'ايەل قانىشەر: 1–2 تەرى ءتۇسى',
    '🧛🏼‍♀' => 'ايەل قانىشەر: 3-تەرى ءتۇرى',
    '🧛🏽‍♀' => 'ايەل قانىشەر: 4-تەرى ءتۇسى',
    '🧛🏾‍♀' => 'ايەل قانىشەر: 5-تەرى ءتۇسى',
    '🧛🏿‍♀' => 'ايەل قانىشەر: 6-تەرى ءتۇسى',
    '🧜🏻‍♂' => 'قۇيرىقتى: 1–2 تەرى ءتۇسى',
    '🧜🏼‍♂' => 'قۇيرىقتى: 3-تەرى ءتۇرى',
    '🧜🏽‍♂' => 'قۇيرىقتى: 4-تەرى ءتۇسى',
    '🧜🏾‍♂' => 'قۇيرىقتى: 5-تەرى ءتۇسى',
    '🧜🏿‍♂' => 'قۇيرىقتى: 6-تەرى ءتۇسى',
    '🧜🏻‍♀' => 'سۋ پەرىسى: 1–2 تەرى ءتۇسى',
    '🧜🏼‍♀' => 'سۋ پەرىسى: 3-تەرى ءتۇرى',
    '🧜🏽‍♀' => 'سۋ پەرىسى: 4-تەرى ءتۇسى',
    '🧜🏾‍♀' => 'سۋ پەرىسى: 5-تەرى ءتۇسى',
    '🧜🏿‍♀' => 'سۋ پەرىسى: 6-تەرى ءتۇسى',
    '🧝🏻‍♂' => 'ەر ەرگەجەيلى: 1–2 تەرى ءتۇسى',
    '🧝🏼‍♂' => 'ەر ەرگەجەيلى: 3-تەرى ءتۇرى',
    '🧝🏽‍♂' => 'ەر ەرگەجەيلى: 4-تەرى ءتۇسى',
    '🧝🏾‍♂' => 'ەر ەرگەجەيلى: 5-تەرى ءتۇسى',
    '🧝🏿‍♂' => 'ەر ەرگەجەيلى: 6-تەرى ءتۇسى',
    '🧝🏻‍♀' => 'ايەل ەرگەجەيلى: 1–2 تەرى ءتۇسى',
    '🧝🏼‍♀' => 'ايەل ەرگەجەيلى: 3-تەرى ءتۇرى',
    '🧝🏽‍♀' => 'ايەل ەرگەجەيلى: 4-تەرى ءتۇسى',
    '🧝🏾‍♀' => 'ايەل ەرگەجەيلى: 5-تەرى ءتۇسى',
    '🧝🏿‍♀' => 'ايەل ەرگەجەيلى: 6-تەرى ءتۇسى',
    '💆🏻‍♂' => 'بەتىن ۋاقالاتىپ وتىرعان ەر: 1–2 تەرى ءتۇسى',
    '💆🏼‍♂' => 'بەتىن ۋاقالاتىپ وتىرعان ەر: 3-تەرى ءتۇرى',
    '💆🏽‍♂' => 'بەتىن ۋاقالاتىپ وتىرعان ەر: 4-تەرى ءتۇسى',
    '💆🏾‍♂' => 'بەتىن ۋاقالاتىپ وتىرعان ەر: 5-تەرى ءتۇسى',
    '💆🏿‍♂' => 'بەتىن ۋاقالاتىپ وتىرعان ەر: 6-تەرى ءتۇسى',
    '💆🏻‍♀' => 'بەتىن ۋاقالاتىپ وتىرعان ايەل: 1–2 تەرى ءتۇسى',
    '💆🏼‍♀' => 'بەتىن ۋاقالاتىپ وتىرعان ايەل: 3-تەرى ءتۇرى',
    '💆🏽‍♀' => 'بەتىن ۋاقالاتىپ وتىرعان ايەل: 4-تەرى ءتۇسى',
    '💆🏾‍♀' => 'بەتىن ۋاقالاتىپ وتىرعان ايەل: 5-تەرى ءتۇسى',
    '💆🏿‍♀' => 'بەتىن ۋاقالاتىپ وتىرعان ايەل: 6-تەرى ءتۇسى',
    '💇🏻‍♂' => 'شاشىن قيدىرىپ وتىرعان ەر: 1–2 تەرى ءتۇسى',
    '💇🏼‍♂' => 'شاشىن قيدىرىپ وتىرعان ەر: 3-تەرى ءتۇرى',
    '💇🏽‍♂' => 'شاشىن قيدىرىپ وتىرعان ەر: 4-تەرى ءتۇسى',
    '💇🏾‍♂' => 'شاشىن قيدىرىپ وتىرعان ەر: 5-تەرى ءتۇسى',
    '💇🏿‍♂' => 'شاشىن قيدىرىپ وتىرعان ەر: 6-تەرى ءتۇسى',
    '💇🏻‍♀' => 'شاشىن قيدىرىپ وتىرعان ايەل: 1–2 تەرى ءتۇسى',
    '💇🏼‍♀' => 'شاشىن قيدىرىپ وتىرعان ايەل: 3-تەرى ءتۇرى',
    '💇🏽‍♀' => 'شاشىن قيدىرىپ وتىرعان ايەل: 4-تەرى ءتۇسى',
    '💇🏾‍♀' => 'شاشىن قيدىرىپ وتىرعان ايەل: 5-تەرى ءتۇسى',
    '💇🏿‍♀' => 'شاشىن قيدىرىپ وتىرعان ايەل: 6-تەرى ءتۇسى',
    '🚶🏻‍♂' => 'جاياۋ ەر: 1–2 تەرى ءتۇسى',
    '🚶🏼‍♂' => 'جاياۋ ەر: 3-تەرى ءتۇرى',
    '🚶🏽‍♂' => 'جاياۋ ەر: 4-تەرى ءتۇسى',
    '🚶🏾‍♂' => 'جاياۋ ەر: 5-تەرى ءتۇسى',
    '🚶🏿‍♂' => 'جاياۋ ەر: 6-تەرى ءتۇسى',
    '🚶🏻‍♀' => 'جاياۋ ايەل: 1–2 تەرى ءتۇسى',
    '🚶🏼‍♀' => 'جاياۋ ايەل: 3-تەرى ءتۇرى',
    '🚶🏽‍♀' => 'جاياۋ ايەل: 4-تەرى ءتۇسى',
    '🚶🏾‍♀' => 'جاياۋ ايەل: 5-تەرى ءتۇسى',
    '🚶🏿‍♀' => 'جاياۋ ايەل: 6-تەرى ءتۇسى',
    '🚶🏻‍➡' => 'جاياۋ: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏼‍➡' => 'جاياۋ: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🚶🏽‍➡' => 'جاياۋ: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏾‍➡' => 'جاياۋ: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🚶🏿‍➡' => 'جاياۋ: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧍🏻‍♂' => 'تۇرىپ تۇرعان ەركەك: 1–2 تەرى ءتۇسى',
    '🧍🏼‍♂' => 'تۇرىپ تۇرعان ەركەك: 3-تەرى ءتۇرى',
    '🧍🏽‍♂' => 'تۇرىپ تۇرعان ەركەك: 4-تەرى ءتۇسى',
    '🧍🏾‍♂' => 'تۇرىپ تۇرعان ەركەك: 5-تەرى ءتۇسى',
    '🧍🏿‍♂' => 'تۇرىپ تۇرعان ەركەك: 6-تەرى ءتۇسى',
    '🧍🏻‍♀' => 'تۇرىپ تۇرعان ايەل: 1–2 تەرى ءتۇسى',
    '🧍🏼‍♀' => 'تۇرىپ تۇرعان ايەل: 3-تەرى ءتۇرى',
    '🧍🏽‍♀' => 'تۇرىپ تۇرعان ايەل: 4-تەرى ءتۇسى',
    '🧍🏾‍♀' => 'تۇرىپ تۇرعان ايەل: 5-تەرى ءتۇسى',
    '🧍🏿‍♀' => 'تۇرىپ تۇرعان ايەل: 6-تەرى ءتۇسى',
    '🧎🏻‍♂' => 'تىزەرلەپ وتىرعان ەركەك: 1–2 تەرى ءتۇسى',
    '🧎🏼‍♂' => 'تىزەرلەپ وتىرعان ەركەك: 3-تەرى ءتۇرى',
    '🧎🏽‍♂' => 'تىزەرلەپ وتىرعان ەركەك: 4-تەرى ءتۇسى',
    '🧎🏾‍♂' => 'تىزەرلەپ وتىرعان ەركەك: 5-تەرى ءتۇسى',
    '🧎🏿‍♂' => 'تىزەرلەپ وتىرعان ەركەك: 6-تەرى ءتۇسى',
    '🧎🏻‍♀' => 'تىزەرلەپ وتىرعان ايەل: 1–2 تەرى ءتۇسى',
    '🧎🏼‍♀' => 'تىزەرلەپ وتىرعان ايەل: 3-تەرى ءتۇرى',
    '🧎🏽‍♀' => 'تىزەرلەپ وتىرعان ايەل: 4-تەرى ءتۇسى',
    '🧎🏾‍♀' => 'تىزەرلەپ وتىرعان ايەل: 5-تەرى ءتۇسى',
    '🧎🏿‍♀' => 'تىزەرلەپ وتىرعان ايەل: 6-تەرى ءتۇسى',
    '🧎🏻‍➡' => 'تىزەرلەپ وتىرعان ادام: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏼‍➡' => 'تىزەرلەپ وتىرعان ادام: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🧎🏽‍➡' => 'تىزەرلەپ وتىرعان ادام: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏾‍➡' => 'تىزەرلەپ وتىرعان ادام: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧎🏿‍➡' => 'تىزەرلەپ وتىرعان ادام: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🧑🏻‍🦯' => 'تاياعى بار ادام: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🦯' => 'تاياعى بار ادام: 3-تەرى ءتۇرى',
    '🧑🏽‍🦯' => 'تاياعى بار ادام: 4-تەرى ءتۇسى',
    '🧑🏾‍🦯' => 'تاياعى بار ادام: 5-تەرى ءتۇسى',
    '🧑🏿‍🦯' => 'تاياعى بار ادام: 6-تەرى ءتۇسى',
    '👨🏻‍🦯' => 'تاياعى بار ەركەك: 1–2 تەرى ءتۇسى',
    '👨🏼‍🦯' => 'تاياعى بار ەركەك: 3-تەرى ءتۇرى',
    '👨🏽‍🦯' => 'تاياعى بار ەركەك: 4-تەرى ءتۇسى',
    '👨🏾‍🦯' => 'تاياعى بار ەركەك: 5-تەرى ءتۇسى',
    '👨🏿‍🦯' => 'تاياعى بار ەركەك: 6-تەرى ءتۇسى',
    '👩🏻‍🦯' => 'تاياعى بار ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🦯' => 'تاياعى بار ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🦯' => 'تاياعى بار ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🦯' => 'تاياعى بار ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🦯' => 'تاياعى بار ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🦼' => 'موتورلى ارباداعى ادام: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🦼' => 'موتورلى ارباداعى ادام: 3-تەرى ءتۇرى',
    '🧑🏽‍🦼' => 'موتورلى ارباداعى ادام: 4-تەرى ءتۇسى',
    '🧑🏾‍🦼' => 'موتورلى ارباداعى ادام: 5-تەرى ءتۇسى',
    '🧑🏿‍🦼' => 'موتورلى ارباداعى ادام: 6-تەرى ءتۇسى',
    '👨🏻‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 1–2 تەرى ءتۇسى',
    '👨🏼‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 3-تەرى ءتۇرى',
    '👨🏽‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 4-تەرى ءتۇسى',
    '👨🏾‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 5-تەرى ءتۇسى',
    '👨🏿‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك: 6-تەرى ءتۇسى',
    '👩🏻‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل: 6-تەرى ءتۇسى',
    '🧑🏻‍🦽' => 'قول جەتەگى بار ارباشاداعى ادام: 1–2 تەرى ءتۇسى',
    '🧑🏼‍🦽' => 'قول جەتەگى بار ارباشاداعى ادام: 3-تەرى ءتۇرى',
    '🧑🏽‍🦽' => 'قول جەتەگى بار ارباشاداعى ادام: 4-تەرى ءتۇسى',
    '🧑🏾‍🦽' => 'قول جەتەگى بار ارباشاداعى ادام: 5-تەرى ءتۇسى',
    '🧑🏿‍🦽' => 'قول جەتەگى بار ارباشاداعى ادام: 6-تەرى ءتۇسى',
    '👨🏻‍🦽' => 'قول جەتەگى بار ارباشاداعى ەركەك: 1–2 تەرى ءتۇسى',
    '👨🏼‍🦽' => 'قول جەتەگى بار ارباشاداعى ەركەك: 3-تەرى ءتۇرى',
    '👨🏽‍🦽' => 'قول جەتەگى بار ارباشاداعى ەركەك: 4-تەرى ءتۇسى',
    '👨🏾‍🦽' => 'قول جەتەگى بار ارباشاداعى ەركەك: 5-تەرى ءتۇسى',
    '👨🏿‍🦽' => 'قول جەتەگى بار ارباشاداعى ەركەك: 6-تەرى ءتۇسى',
    '👩🏻‍🦽' => 'قول جەتەگى بار ارباشاداعى ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼‍🦽' => 'قول جەتەگى بار ارباشاداعى ايەل: 3-تەرى ءتۇرى',
    '👩🏽‍🦽' => 'قول جەتەگى بار ارباشاداعى ايەل: 4-تەرى ءتۇسى',
    '👩🏾‍🦽' => 'قول جەتەگى بار ارباشاداعى ايەل: 5-تەرى ءتۇسى',
    '👩🏿‍🦽' => 'قول جەتەگى بار ارباشاداعى ايەل: 6-تەرى ءتۇسى',
    '🏃🏻‍♂' => 'جۇگىرگەن ەر: 1–2 تەرى ءتۇسى',
    '🏃🏼‍♂' => 'جۇگىرگەن ەر: 3-تەرى ءتۇرى',
    '🏃🏽‍♂' => 'جۇگىرگەن ەر: 4-تەرى ءتۇسى',
    '🏃🏾‍♂' => 'جۇگىرگەن ەر: 5-تەرى ءتۇسى',
    '🏃🏿‍♂' => 'جۇگىرگەن ەر: 6-تەرى ءتۇسى',
    '🏃🏻‍♀' => 'جۇگىرگەن ايەل: 1–2 تەرى ءتۇسى',
    '🏃🏼‍♀' => 'جۇگىرگەن ايەل: 3-تەرى ءتۇرى',
    '🏃🏽‍♀' => 'جۇگىرگەن ايەل: 4-تەرى ءتۇسى',
    '🏃🏾‍♀' => 'جۇگىرگەن ايەل: 5-تەرى ءتۇسى',
    '🏃🏿‍♀' => 'جۇگىرگەن ايەل: 6-تەرى ءتۇسى',
    '🏃🏻‍➡' => 'جۇگىرۋ: 1–2 تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏼‍➡' => 'جۇگىرۋ: 3-تەرى ءتۇرى، وڭعا قاراتىلعان',
    '🏃🏽‍➡' => 'جۇگىرۋ: 4-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏾‍➡' => 'جۇگىرۋ: 5-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '🏃🏿‍➡' => 'جۇگىرۋ: 6-تەرى ءتۇسى، وڭعا قاراتىلعان',
    '👯🏻‍♀' => 'تويلاتىپ جۇرگەن ايەلدەر: 1–2 تەرى ءتۇسى',
    '👯🏼‍♀' => 'تويلاتىپ جۇرگەن ايەلدەر: 3-تەرى ءتۇرى',
    '👯🏽‍♀' => 'تويلاتىپ جۇرگەن ايەلدەر: 4-تەرى ءتۇسى',
    '👯🏾‍♀' => 'تويلاتىپ جۇرگەن ايەلدەر: 5-تەرى ءتۇسى',
    '👯🏿‍♀' => 'تويلاتىپ جۇرگەن ايەلدەر: 6-تەرى ءتۇسى',
    '👯🏻‍♂' => 'تويلاتىپ جۇرگەن ەرلەر: 1–2 تەرى ءتۇسى',
    '👯🏼‍♂' => 'تويلاتىپ جۇرگەن ەرلەر: 3-تەرى ءتۇرى',
    '👯🏽‍♂' => 'تويلاتىپ جۇرگەن ەرلەر: 4-تەرى ءتۇسى',
    '👯🏾‍♂' => 'تويلاتىپ جۇرگەن ەرلەر: 5-تەرى ءتۇسى',
    '👯🏿‍♂' => 'تويلاتىپ جۇرگەن ەرلەر: 6-تەرى ءتۇسى',
    '🧖🏻‍♂' => 'بۋ بولمەدەگى ەر: 1–2 تەرى ءتۇسى',
    '🧖🏼‍♂' => 'بۋ بولمەدەگى ەر: 3-تەرى ءتۇرى',
    '🧖🏽‍♂' => 'بۋ بولمەدەگى ەر: 4-تەرى ءتۇسى',
    '🧖🏾‍♂' => 'بۋ بولمەدەگى ەر: 5-تەرى ءتۇسى',
    '🧖🏿‍♂' => 'بۋ بولمەدەگى ەر: 6-تەرى ءتۇسى',
    '🧖🏻‍♀' => 'بۋ بولمەدەگى ايەل: 1–2 تەرى ءتۇسى',
    '🧖🏼‍♀' => 'بۋ بولمەدەگى ايەل: 3-تەرى ءتۇرى',
    '🧖🏽‍♀' => 'بۋ بولمەدەگى ايەل: 4-تەرى ءتۇسى',
    '🧖🏾‍♀' => 'بۋ بولمەدەگى ايەل: 5-تەرى ءتۇسى',
    '🧖🏿‍♀' => 'بۋ بولمەدەگى ايەل: 6-تەرى ءتۇسى',
    '🧗🏻‍♂' => 'كوتەرىلىپ بارا جاتقان ەر ادام: 1–2 تەرى ءتۇسى',
    '🧗🏼‍♂' => 'كوتەرىلىپ بارا جاتقان ەر ادام: 3-تەرى ءتۇرى',
    '🧗🏽‍♂' => 'كوتەرىلىپ بارا جاتقان ەر ادام: 4-تەرى ءتۇسى',
    '🧗🏾‍♂' => 'كوتەرىلىپ بارا جاتقان ەر ادام: 5-تەرى ءتۇسى',
    '🧗🏿‍♂' => 'كوتەرىلىپ بارا جاتقان ەر ادام: 6-تەرى ءتۇسى',
    '🧗🏻‍♀' => 'كوتەرىلىپ بارا جاتقان ايەل: 1–2 تەرى ءتۇسى',
    '🧗🏼‍♀' => 'كوتەرىلىپ بارا جاتقان ايەل: 3-تەرى ءتۇرى',
    '🧗🏽‍♀' => 'كوتەرىلىپ بارا جاتقان ايەل: 4-تەرى ءتۇسى',
    '🧗🏾‍♀' => 'كوتەرىلىپ بارا جاتقان ايەل: 5-تەرى ءتۇسى',
    '🧗🏿‍♀' => 'كوتەرىلىپ بارا جاتقان ايەل: 6-تەرى ءتۇسى',
    '🏌🏻‍♂' => 'گولىف ويناعان ەر: 1–2 تەرى ءتۇسى',
    '🏌🏼‍♂' => 'گولىف ويناعان ەر: 3-تەرى ءتۇرى',
    '🏌🏽‍♂' => 'گولىف ويناعان ەر: 4-تەرى ءتۇسى',
    '🏌🏾‍♂' => 'گولىف ويناعان ەر: 5-تەرى ءتۇسى',
    '🏌🏿‍♂' => 'گولىف ويناعان ەر: 6-تەرى ءتۇسى',
    '🏌🏻‍♀' => 'گولىف ويناعان ايەل: 1–2 تەرى ءتۇسى',
    '🏌🏼‍♀' => 'گولىف ويناعان ايەل: 3-تەرى ءتۇرى',
    '🏌🏽‍♀' => 'گولىف ويناعان ايەل: 4-تەرى ءتۇسى',
    '🏌🏾‍♀' => 'گولىف ويناعان ايەل: 5-تەرى ءتۇسى',
    '🏌🏿‍♀' => 'گولىف ويناعان ايەل: 6-تەرى ءتۇسى',
    '🏄🏻‍♂' => 'سەرفيڭشى ەر: 1–2 تەرى ءتۇسى',
    '🏄🏼‍♂' => 'سەرفيڭشى ەر: 3-تەرى ءتۇرى',
    '🏄🏽‍♂' => 'سەرفيڭشى ەر: 4-تەرى ءتۇسى',
    '🏄🏾‍♂' => 'سەرفيڭشى ەر: 5-تەرى ءتۇسى',
    '🏄🏿‍♂' => 'سەرفيڭشى ەر: 6-تەرى ءتۇسى',
    '🏄🏻‍♀' => 'سەرفيڭشى ايەل: 1–2 تەرى ءتۇسى',
    '🏄🏼‍♀' => 'سەرفيڭشى ايەل: 3-تەرى ءتۇرى',
    '🏄🏽‍♀' => 'سەرفيڭشى ايەل: 4-تەرى ءتۇسى',
    '🏄🏾‍♀' => 'سەرفيڭشى ايەل: 5-تەرى ءتۇسى',
    '🏄🏿‍♀' => 'سەرفيڭشى ايەل: 6-تەرى ءتۇسى',
    '🚣🏻‍♂' => 'قايىق ەسۋشى ەر: 1–2 تەرى ءتۇسى',
    '🚣🏼‍♂' => 'قايىق ەسۋشى ەر: 3-تەرى ءتۇرى',
    '🚣🏽‍♂' => 'قايىق ەسۋشى ەر: 4-تەرى ءتۇسى',
    '🚣🏾‍♂' => 'قايىق ەسۋشى ەر: 5-تەرى ءتۇسى',
    '🚣🏿‍♂' => 'قايىق ەسۋشى ەر: 6-تەرى ءتۇسى',
    '🚣🏻‍♀' => 'قايىق ەسۋشى ايەل: 1–2 تەرى ءتۇسى',
    '🚣🏼‍♀' => 'قايىق ەسۋشى ايەل: 3-تەرى ءتۇرى',
    '🚣🏽‍♀' => 'قايىق ەسۋشى ايەل: 4-تەرى ءتۇسى',
    '🚣🏾‍♀' => 'قايىق ەسۋشى ايەل: 5-تەرى ءتۇسى',
    '🚣🏿‍♀' => 'قايىق ەسۋشى ايەل: 6-تەرى ءتۇسى',
    '🏊🏻‍♂' => 'ءجۇزۋشى ەر: 1–2 تەرى ءتۇسى',
    '🏊🏼‍♂' => 'ءجۇزۋشى ەر: 3-تەرى ءتۇرى',
    '🏊🏽‍♂' => 'ءجۇزۋشى ەر: 4-تەرى ءتۇسى',
    '🏊🏾‍♂' => 'ءجۇزۋشى ەر: 5-تەرى ءتۇسى',
    '🏊🏿‍♂' => 'ءجۇزۋشى ەر: 6-تەرى ءتۇسى',
    '🏊🏻‍♀' => 'ءجۇزۋشى ايەل: 1–2 تەرى ءتۇسى',
    '🏊🏼‍♀' => 'ءجۇزۋشى ايەل: 3-تەرى ءتۇرى',
    '🏊🏽‍♀' => 'ءجۇزۋشى ايەل: 4-تەرى ءتۇسى',
    '🏊🏾‍♀' => 'ءجۇزۋشى ايەل: 5-تەرى ءتۇسى',
    '🏊🏿‍♀' => 'ءجۇزۋشى ايەل: 6-تەرى ءتۇسى',
    '⛹🏻‍♂' => 'دوپ ويناعان ەر: 1–2 تەرى ءتۇسى',
    '⛹🏼‍♂' => 'دوپ ويناعان ەر: 3-تەرى ءتۇرى',
    '⛹🏽‍♂' => 'دوپ ويناعان ەر: 4-تەرى ءتۇسى',
    '⛹🏾‍♂' => 'دوپ ويناعان ەر: 5-تەرى ءتۇسى',
    '⛹🏿‍♂' => 'دوپ ويناعان ەر: 6-تەرى ءتۇسى',
    '⛹🏻‍♀' => 'دوپ ويناعان ايەل: 1–2 تەرى ءتۇسى',
    '⛹🏼‍♀' => 'دوپ ويناعان ايەل: 3-تەرى ءتۇرى',
    '⛹🏽‍♀' => 'دوپ ويناعان ايەل: 4-تەرى ءتۇسى',
    '⛹🏾‍♀' => 'دوپ ويناعان ايەل: 5-تەرى ءتۇسى',
    '⛹🏿‍♀' => 'دوپ ويناعان ايەل: 6-تەرى ءتۇسى',
    '🏋🏻‍♂' => 'اۋىر اتلەت ەر: 1–2 تەرى ءتۇسى',
    '🏋🏼‍♂' => 'اۋىر اتلەت ەر: 3-تەرى ءتۇرى',
    '🏋🏽‍♂' => 'اۋىر اتلەت ەر: 4-تەرى ءتۇسى',
    '🏋🏾‍♂' => 'اۋىر اتلەت ەر: 5-تەرى ءتۇسى',
    '🏋🏿‍♂' => 'اۋىر اتلەت ەر: 6-تەرى ءتۇسى',
    '🏋🏻‍♀' => 'اۋىر اتلەت ايەل: 1–2 تەرى ءتۇسى',
    '🏋🏼‍♀' => 'اۋىر اتلەت ايەل: 3-تەرى ءتۇرى',
    '🏋🏽‍♀' => 'اۋىر اتلەت ايەل: 4-تەرى ءتۇسى',
    '🏋🏾‍♀' => 'اۋىر اتلەت ايەل: 5-تەرى ءتۇسى',
    '🏋🏿‍♀' => 'اۋىر اتلەت ايەل: 6-تەرى ءتۇسى',
    '🚴🏻‍♂' => 'ۆەلوسيپەدشى ەر: 1–2 تەرى ءتۇسى',
    '🚴🏼‍♂' => 'ۆەلوسيپەدشى ەر: 3-تەرى ءتۇرى',
    '🚴🏽‍♂' => 'ۆەلوسيپەدشى ەر: 4-تەرى ءتۇسى',
    '🚴🏾‍♂' => 'ۆەلوسيپەدشى ەر: 5-تەرى ءتۇسى',
    '🚴🏿‍♂' => 'ۆەلوسيپەدشى ەر: 6-تەرى ءتۇسى',
    '🚴🏻‍♀' => 'ۆەلوسيپەدشى ايەل: 1–2 تەرى ءتۇسى',
    '🚴🏼‍♀' => 'ۆەلوسيپەدشى ايەل: 3-تەرى ءتۇرى',
    '🚴🏽‍♀' => 'ۆەلوسيپەدشى ايەل: 4-تەرى ءتۇسى',
    '🚴🏾‍♀' => 'ۆەلوسيپەدشى ايەل: 5-تەرى ءتۇسى',
    '🚴🏿‍♀' => 'ۆەلوسيپەدشى ايەل: 6-تەرى ءتۇسى',
    '🚵🏻‍♂' => 'تاۋ ۆەلوسيپەدىن مىنگەن ەر: 1–2 تەرى ءتۇسى',
    '🚵🏼‍♂' => 'تاۋ ۆەلوسيپەدىن مىنگەن ەر: 3-تەرى ءتۇرى',
    '🚵🏽‍♂' => 'تاۋ ۆەلوسيپەدىن مىنگەن ەر: 4-تەرى ءتۇسى',
    '🚵🏾‍♂' => 'تاۋ ۆەلوسيپەدىن مىنگەن ەر: 5-تەرى ءتۇسى',
    '🚵🏿‍♂' => 'تاۋ ۆەلوسيپەدىن مىنگەن ەر: 6-تەرى ءتۇسى',
    '🚵🏻‍♀' => 'تاۋ ۆەلوسيپەدىن مىنگەن ايەل: 1–2 تەرى ءتۇسى',
    '🚵🏼‍♀' => 'تاۋ ۆەلوسيپەدىن مىنگەن ايەل: 3-تەرى ءتۇرى',
    '🚵🏽‍♀' => 'تاۋ ۆەلوسيپەدىن مىنگەن ايەل: 4-تەرى ءتۇسى',
    '🚵🏾‍♀' => 'تاۋ ۆەلوسيپەدىن مىنگەن ايەل: 5-تەرى ءتۇسى',
    '🚵🏿‍♀' => 'تاۋ ۆەلوسيپەدىن مىنگەن ايەل: 6-تەرى ءتۇسى',
    '🤸🏻‍♂' => 'اكروبات ەر: 1–2 تەرى ءتۇسى',
    '🤸🏼‍♂' => 'اكروبات ەر: 3-تەرى ءتۇرى',
    '🤸🏽‍♂' => 'اكروبات ەر: 4-تەرى ءتۇسى',
    '🤸🏾‍♂' => 'اكروبات ەر: 5-تەرى ءتۇسى',
    '🤸🏿‍♂' => 'اكروبات ەر: 6-تەرى ءتۇسى',
    '🤸🏻‍♀' => 'اكروبات ايەل: 1–2 تەرى ءتۇسى',
    '🤸🏼‍♀' => 'اكروبات ايەل: 3-تەرى ءتۇرى',
    '🤸🏽‍♀' => 'اكروبات ايەل: 4-تەرى ءتۇسى',
    '🤸🏾‍♀' => 'اكروبات ايەل: 5-تەرى ءتۇسى',
    '🤸🏿‍♀' => 'اكروبات ايەل: 6-تەرى ءتۇسى',
    '🤼🏻‍♀' => 'كۇرەسشى ايەل: 1–2 تەرى ءتۇسى',
    '🤼🏼‍♀' => 'كۇرەسشى ايەل: 3-تەرى ءتۇرى',
    '🤼🏽‍♀' => 'كۇرەسشى ايەل: 4-تەرى ءتۇسى',
    '🤼🏾‍♀' => 'كۇرەسشى ايەل: 5-تەرى ءتۇسى',
    '🤼🏿‍♀' => 'كۇرەسشى ايەل: 6-تەرى ءتۇسى',
    '🤼🏻‍♂' => 'كۇرەسشى ەر: 1–2 تەرى ءتۇسى',
    '🤼🏼‍♂' => 'كۇرەسشى ەر: 3-تەرى ءتۇرى',
    '🤼🏽‍♂' => 'كۇرەسشى ەر: 4-تەرى ءتۇسى',
    '🤼🏾‍♂' => 'كۇرەسشى ەر: 5-تەرى ءتۇسى',
    '🤼🏿‍♂' => 'كۇرەسشى ەر: 6-تەرى ءتۇسى',
    '🤽🏻‍♂' => 'سۋ پولوسىن ويناعان ەر: 1–2 تەرى ءتۇسى',
    '🤽🏼‍♂' => 'سۋ پولوسىن ويناعان ەر: 3-تەرى ءتۇرى',
    '🤽🏽‍♂' => 'سۋ پولوسىن ويناعان ەر: 4-تەرى ءتۇسى',
    '🤽🏾‍♂' => 'سۋ پولوسىن ويناعان ەر: 5-تەرى ءتۇسى',
    '🤽🏿‍♂' => 'سۋ پولوسىن ويناعان ەر: 6-تەرى ءتۇسى',
    '🤽🏻‍♀' => 'سۋ پولوسىن ويناعان ايەل: 1–2 تەرى ءتۇسى',
    '🤽🏼‍♀' => 'سۋ پولوسىن ويناعان ايەل: 3-تەرى ءتۇرى',
    '🤽🏽‍♀' => 'سۋ پولوسىن ويناعان ايەل: 4-تەرى ءتۇسى',
    '🤽🏾‍♀' => 'سۋ پولوسىن ويناعان ايەل: 5-تەرى ءتۇسى',
    '🤽🏿‍♀' => 'سۋ پولوسىن ويناعان ايەل: 6-تەرى ءتۇسى',
    '🤾🏻‍♂' => 'گاندبول ويناعان ەر: 1–2 تەرى ءتۇسى',
    '🤾🏼‍♂' => 'گاندبول ويناعان ەر: 3-تەرى ءتۇرى',
    '🤾🏽‍♂' => 'گاندبول ويناعان ەر: 4-تەرى ءتۇسى',
    '🤾🏾‍♂' => 'گاندبول ويناعان ەر: 5-تەرى ءتۇسى',
    '🤾🏿‍♂' => 'گاندبول ويناعان ەر: 6-تەرى ءتۇسى',
    '🤾🏻‍♀' => 'گاندبول ويناعان ايەل: 1–2 تەرى ءتۇسى',
    '🤾🏼‍♀' => 'گاندبول ويناعان ايەل: 3-تەرى ءتۇرى',
    '🤾🏽‍♀' => 'گاندبول ويناعان ايەل: 4-تەرى ءتۇسى',
    '🤾🏾‍♀' => 'گاندبول ويناعان ايەل: 5-تەرى ءتۇسى',
    '🤾🏿‍♀' => 'گاندبول ويناعان ايەل: 6-تەرى ءتۇسى',
    '🤹🏻‍♂' => 'جونگلەر ەر: 1–2 تەرى ءتۇسى',
    '🤹🏼‍♂' => 'جونگلەر ەر: 3-تەرى ءتۇرى',
    '🤹🏽‍♂' => 'جونگلەر ەر: 4-تەرى ءتۇسى',
    '🤹🏾‍♂' => 'جونگلەر ەر: 5-تەرى ءتۇسى',
    '🤹🏿‍♂' => 'جونگلەر ەر: 6-تەرى ءتۇسى',
    '🤹🏻‍♀' => 'جونگلەر ايەل: 1–2 تەرى ءتۇسى',
    '🤹🏼‍♀' => 'جونگلەر ايەل: 3-تەرى ءتۇرى',
    '🤹🏽‍♀' => 'جونگلەر ايەل: 4-تەرى ءتۇسى',
    '🤹🏾‍♀' => 'جونگلەر ايەل: 5-تەرى ءتۇسى',
    '🤹🏿‍♀' => 'جونگلەر ايەل: 6-تەرى ءتۇسى',
    '🧘🏻‍♂' => 'مالداس قۇرىپ وتىرعان ەر ادام: 1–2 تەرى ءتۇسى',
    '🧘🏼‍♂' => 'مالداس قۇرىپ وتىرعان ەر ادام: 3-تەرى ءتۇرى',
    '🧘🏽‍♂' => 'مالداس قۇرىپ وتىرعان ەر ادام: 4-تەرى ءتۇسى',
    '🧘🏾‍♂' => 'مالداس قۇرىپ وتىرعان ەر ادام: 5-تەرى ءتۇسى',
    '🧘🏿‍♂' => 'مالداس قۇرىپ وتىرعان ەر ادام: 6-تەرى ءتۇسى',
    '🧘🏻‍♀' => 'مالداس قۇرىپ وتىرعان ايەل: 1–2 تەرى ءتۇسى',
    '🧘🏼‍♀' => 'مالداس قۇرىپ وتىرعان ايەل: 3-تەرى ءتۇرى',
    '🧘🏽‍♀' => 'مالداس قۇرىپ وتىرعان ايەل: 4-تەرى ءتۇسى',
    '🧘🏾‍♀' => 'مالداس قۇرىپ وتىرعان ايەل: 5-تەرى ءتۇسى',
    '🧘🏿‍♀' => 'مالداس قۇرىپ وتىرعان ايەل: 6-تەرى ءتۇسى',
    '🧑‍🦰' => 'ادام: جيرەن شاش',
    '🧑‍🦱' => 'ادام: بۇيرا شاش',
    '🧑‍🦳' => 'ادام: اق شاش',
    '🧑‍🦲' => 'ادام: تاقىرباس',
    '👨‍🦰' => 'ەر ادام: جيرەن شاش',
    '👨‍🦱' => 'ەر ادام: بۇيرا شاش',
    '👨‍🦳' => 'ەر ادام: اق شاش',
    '👨‍🦲' => 'ەر ادام: تاقىرباس',
    '👩‍🦰' => 'ايەل: جيرەن شاش',
    '👩‍🦱' => 'ايەل: بۇيرا شاش',
    '👩‍🦳' => 'ايەل: اق شاش',
    '👩‍🦲' => 'ايەل: تاقىرباس',
    '🚶‍➡' => 'جاياۋ: وڭعا قاراتىلعان',
    '🧎‍➡' => 'تىزەرلەپ وتىرعان ادام: وڭعا قاراتىلعان',
    '🏃‍➡' => 'جۇگىرۋ: وڭعا قاراتىلعان',
    '👨‍👦' => 'وتباسى: ەر ادام، ۇل',
    '👨‍👧' => 'وتباسى: ەر ادام، قىز',
    '👩‍👦' => 'وتباسى: ايەل، ۇل',
    '👩‍👧' => 'وتباسى: ايەل، قىز',
    '😶‍🌫' => 'بۇلت باسقان بەت',
    '😮‍💨' => 'اۋانى شىعارىپ تۇرعان بەت',
    '🙂‍↕' => 'تىگىنەن شايقالعان بەت',
    '😵‍💫' => 'كوزى شيىرشىق بەت',
    '❤‍🔥' => 'لاۋلاپ تۇرعان جۇرەك',
    '❤‍🩹' => 'ەمدىك جۇرەك',
    '👁‍🗨' => 'كۋاگەر بەلگىشەسى',
    '🧔‍♂' => 'ساقالدى ەر ادام',
    '👱‍♂' => 'اققۇبا ەر ادام',
    '🧔‍♀' => 'ساقالدى ايەل ادام',
    '👱‍♀' => 'اققۇبا ايەل',
    '🙍‍♂' => 'اشۋلى ەر',
    '🙍‍♀' => 'اشۋلى ايەل',
    '🙎‍♂' => 'رەنجۋلى ەر',
    '🙎‍♀' => 'رەنجۋلى ايەل',
    '🙅‍♂' => 'قارسىمىن دەپ تۇرعان ەر',
    '🙅‍♀' => 'قارسىمىن دەپ تۇرعان ايەل',
    '🙆‍♂' => 'قۇپتاپ تۇرعان ەر',
    '🙆‍♀' => 'قۇپتاپ تۇرعان ايەل',
    '💁‍♂' => 'قولىن سوزىپ تۇرعان ەر',
    '💁‍♀' => 'قولىن سوزىپ تۇرعان ايەل',
    '🙋‍♂' => 'قولىن كوتەرىپ تۇرعان ەر',
    '🙋‍♀' => 'قولىن كوتەرىپ تۇرعان ايەل',
    '🧏‍♂' => 'ساڭىراۋ ەركەك',
    '🧏‍♀' => 'ساڭىراۋ ايەل',
    '🙇‍♂' => 'باسىن ءيىپ تۇرعان ەر',
    '🙇‍♀' => 'باسىن ءيىپ تۇرعان ايەل',
    '🤦‍♂' => 'ايتۋعا ءسوز تاپپاي تۇرعان ەر',
    '🤦‍♀' => 'ايتۋعا ءسوز تاپپاي تۇرعان ايەل',
    '🤷‍♂' => 'يىعىن كوتەرىپ تۇرعان ەر',
    '🤷‍♀' => 'يىعىن كوتەرىپ تۇرعان ايەل',
    '🧑‍⚕' => 'مەديتسينالىق قىزمەتكەر',
    '👨‍⚕' => 'دارىگەر',
    '👩‍⚕' => 'دارىگەر ايەل',
    '🧑‍🎓' => 'ستۋدەنت',
    '👨‍🎓' => 'ستۋدەنت بالا',
    '👩‍🎓' => 'ستۋدەنت قىز',
    '🧑‍🏫' => 'وقىتۋشى',
    '👨‍🏫' => 'مۇعالىم',
    '👩‍🏫' => 'مۇعالىم ايەل',
    '🧑‍⚖' => 'سوت',
    '👨‍⚖' => 'سوت ەر',
    '👩‍⚖' => 'سوت ايەل',
    '🧑‍🌾' => 'ەگىنشى',
    '👨‍🌾' => 'ەر ەگىنشى',
    '👩‍🌾' => 'ايەل ەگىنشى',
    '🧑‍🍳' => 'اسپاز',
    '👨‍🍳' => 'ەر اسپاز',
    '👩‍🍳' => 'ايەل اسپاز',
    '🧑‍🔧' => 'مەحانيك',
    '👨‍🔧' => 'ەر مەحانيك',
    '👩‍🔧' => 'مەحانيك ايەل',
    '🧑‍🏭' => 'زاۋىت جۇمىسشىسى',
    '👨‍🏭' => 'جۇمىسشى',
    '👩‍🏭' => 'جۇمىسشى ايەل',
    '🧑‍💼' => 'كەڭسە قىزمەتكەرى',
    '👨‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر',
    '👩‍💼' => 'كەڭسەدە جۇمىس ىستەيتىن قىزمەتكەر ايەل',
    '🧑‍🔬' => 'عالىم',
    '👨‍🔬' => 'ەر عالىم',
    '👩‍🔬' => 'عالىم ايەل',
    '🧑‍💻' => 'تەحنولوگ',
    '👨‍💻' => 'ەر تەحنولوگ',
    '👩‍💻' => 'تەحنولوگ ايەل',
    '🧑‍🎤' => 'ءانشى',
    '👨‍🎤' => 'ەر ءانشى',
    '👩‍🎤' => 'ءانشى ايەل',
    '🧑‍🎨' => 'سۋرەتشى',
    '👨‍🎨' => 'ەر سۋرەتشى',
    '👩‍🎨' => 'سۋرەتشى ايەل',
    '🧑‍✈' => 'ۇشاق',
    '👨‍✈' => 'پيلوت',
    '👩‍✈' => 'ۇشقىش ايەل',
    '🧑‍🚀' => 'استروناۆت',
    '👨‍🚀' => 'عارىشكەر',
    '👩‍🚀' => 'عارىشكەر ايەل',
    '🧑‍🚒' => 'ءورت ءسوندىرۋشى',
    '👨‍🚒' => 'ەر ءورت ءسوندىرۋشى',
    '👩‍🚒' => 'ءورت ءسوندىرۋشى ايەل',
    '👮‍♂' => 'ەر پوليتسيا قىزمەتكەرى',
    '👮‍♀' => 'ايەل پوليتسيا قىزمەتكەرى',
    '🕵‍♂' => 'بارلاۋشى ەر ادام',
    '🕵‍♀' => 'بارلاۋشى ايەل ادام',
    '💂‍♂' => 'كۇزەتشى ەر ادام',
    '💂‍♀' => 'كۇزەتشى ايەل',
    '👷‍♂' => 'قۇرىلىسشى ەر ادام',
    '👷‍♀' => 'قۇرىلىسشى ايەل',
    '👳‍♂' => 'سالدەلى ەر ادام',
    '👳‍♀' => 'سالدەلى ايەل',
    '🤵‍♂' => 'سموكيڭ كيگەن ەر ادام',
    '🤵‍♀' => 'سموكينگ كيگەن ايەل ادام',
    '👰‍♂' => 'جاۋلىق تاققان ەر ادام',
    '👰‍♀' => 'جاۋلىق تاققان ايەل',
    '👩‍🍼' => 'بالا ەمىزىپ وتىرعان ايەل',
    '👨‍🍼' => 'بالا ەمىزىپ وتىرعان ەركەك',
    '🧑‍🍼' => 'بالا ەمىزىپ وتىرعان ادام',
    '🧑‍🎄' => 'كلاۋس مىرزا',
    '🦸‍♂' => 'ەركەك باتىر',
    '🦸‍♀' => 'باتىر ايەل',
    '🦹‍♂' => 'ەر زالىم',
    '🦹‍♀' => 'ايەل زالىم',
    '🧙‍♂' => 'ەر سيقىرشى',
    '🧙‍♀' => 'ايەل سيقىرشى',
    '🧚‍♂' => 'ەر پەرى',
    '🧚‍♀' => 'ايەل پەرى',
    '🧛‍♂' => 'ەر قانىشەر',
    '🧛‍♀' => 'ايەل قانىشەر',
    '🧜‍♂' => 'قۇيرىقتى',
    '🧜‍♀' => 'سۋ پەرىسى',
    '🧝‍♂' => 'ەر ەرگەجەيلى',
    '🧝‍♀' => 'ايەل ەرگەجەيلى',
    '🧞‍♂' => 'ەر جىن',
    '🧞‍♀' => 'ايەل جىن',
    '🧟‍♂' => 'ەر زومبي',
    '🧟‍♀' => 'ايەل زومبي',
    '💆‍♂' => 'بەتىن ۋاقالاتىپ وتىرعان ەر',
    '💆‍♀' => 'بەتىن ۋاقالاتىپ وتىرعان ايەل',
    '💇‍♂' => 'شاشىن قيدىرىپ وتىرعان ەر',
    '💇‍♀' => 'شاشىن قيدىرىپ وتىرعان ايەل',
    '🚶‍♂' => 'جاياۋ ەر',
    '🚶‍♀' => 'جاياۋ ايەل',
    '🧍‍♂' => 'تۇرىپ تۇرعان ەركەك',
    '🧍‍♀' => 'تۇرىپ تۇرعان ايەل',
    '🧎‍♂' => 'تىزەرلەپ وتىرعان ەركەك',
    '🧎‍♀' => 'تىزەرلەپ وتىرعان ايەل',
    '🧑‍🦯' => 'تاياعى بار ادام',
    '👨‍🦯' => 'تاياعى بار ەركەك',
    '👩‍🦯' => 'تاياعى بار ايەل',
    '🧑‍🦼' => 'موتورلى ارباداعى ادام',
    '👨‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ەركەك',
    '👩‍🦼' => 'ەلەكتىر جەتەگى بار ارباشاداعى ايەل',
    '🧑‍🦽' => 'قول جەتەگى بار ارباشاداعى ادام',
    '👨‍🦽' => 'قول جەتەگى بار ارباشاداعى ەركەك',
    '👩‍🦽' => 'قول جەتەگى بار ارباشاداعى ايەل',
    '🏃‍♂' => 'جۇگىرگەن ەر',
    '🏃‍♀' => 'جۇگىرگەن ايەل',
    '🧑‍🩰' => 'بالەرينا',
    '👯‍♂' => 'تويلاتىپ جۇرگەن ەرلەر',
    '👯‍♀' => 'تويلاتىپ جۇرگەن ايەلدەر',
    '🧖‍♂' => 'بۋ بولمەدەگى ەر',
    '🧖‍♀' => 'بۋ بولمەدەگى ايەل',
    '🧗‍♂' => 'كوتەرىلىپ بارا جاتقان ەر ادام',
    '🧗‍♀' => 'كوتەرىلىپ بارا جاتقان ايەل',
    '🏌‍♂' => 'گولىف ويناعان ەر',
    '🏌‍♀' => 'گولىف ويناعان ايەل',
    '🏄‍♂' => 'سەرفيڭشى ەر',
    '🏄‍♀' => 'سەرفيڭشى ايەل',
    '🚣‍♂' => 'قايىق ەسۋشى ەر',
    '🚣‍♀' => 'قايىق ەسۋشى ايەل',
    '🏊‍♂' => 'ءجۇزۋشى ەر',
    '🏊‍♀' => 'ءجۇزۋشى ايەل',
    '⛹‍♂' => 'دوپ ويناعان ەر',
    '⛹‍♀' => 'دوپ ويناعان ايەل',
    '🏋‍♂' => 'اۋىر اتلەت ەر',
    '🏋‍♀' => 'اۋىر اتلەت ايەل',
    '🚴‍♂' => 'ۆەلوسيپەدشى ەر',
    '🚴‍♀' => 'ۆەلوسيپەدشى ايەل',
    '🚵‍♂' => 'تاۋ ۆەلوسيپەدىن مىنگەن ەر',
    '🚵‍♀' => 'تاۋ ۆەلوسيپەدىن مىنگەن ايەل',
    '🤸‍♂' => 'اكروبات ەر',
    '🤸‍♀' => 'اكروبات ايەل',
    '🤼‍♂' => 'كۇرەسشى ەر',
    '🤼‍♀' => 'كۇرەسشى ايەل',
    '🤽‍♂' => 'سۋ پولوسىن ويناعان ەر',
    '🤽‍♀' => 'سۋ پولوسىن ويناعان ايەل',
    '🤾‍♂' => 'گاندبول ويناعان ەر',
    '🤾‍♀' => 'گاندبول ويناعان ايەل',
    '🤹‍♂' => 'جونگلەر ەر',
    '🤹‍♀' => 'جونگلەر ايەل',
    '🧘‍♂' => 'مالداس قۇرىپ وتىرعان ەر ادام',
    '🧘‍♀' => 'مالداس قۇرىپ وتىرعان ايەل',
    '🧑‍🧒' => 'وتباسى: ەرەسەك، بالا',
    '🐕‍🦺' => 'كومەكشى يت',
    '🐈‍⬛' => 'قارا مىسىق',
    '🐻‍❄' => 'پوليارلىق ايۋ',
    '🐦‍⬛' => 'ۇزاق',
    '🐦‍🔥' => 'فەنيكس',
    '🍋‍🟩' => 'لايم',
    '🍄‍🟫' => 'قوڭىر ساڭىراۋقۇلاق',
    '⛓‍💥' => 'ۇزىلگەن شىنجىر',
    '🏳‍🌈' => 'كەمپىرقوساق ءتۇستى تۋ',
    '🏳‍⚧' => 'كوك، قىزعىلت جانە اق جالاۋ',
    '🏴‍☠' => 'قاراقشى جالاۋى',
    '👋🏻' => 'قول بۇلعاۋ: 1–2 تەرى ءتۇسى',
    '👋🏼' => 'قول بۇلعاۋ: 3-تەرى ءتۇرى',
    '👋🏽' => 'قول بۇلعاۋ: 4-تەرى ءتۇسى',
    '👋🏾' => 'قول بۇلعاۋ: 5-تەرى ءتۇسى',
    '👋🏿' => 'قول بۇلعاۋ: 6-تەرى ءتۇسى',
    '🤚🏻' => 'الاقان سىرتىن كورسەتۋ: 1–2 تەرى ءتۇسى',
    '🤚🏼' => 'الاقان سىرتىن كورسەتۋ: 3-تەرى ءتۇرى',
    '🤚🏽' => 'الاقان سىرتىن كورسەتۋ: 4-تەرى ءتۇسى',
    '🤚🏾' => 'الاقان سىرتىن كورسەتۋ: 5-تەرى ءتۇسى',
    '🤚🏿' => 'الاقان سىرتىن كورسەتۋ: 6-تەرى ءتۇسى',
    '🖐🏻' => 'ساۋساقتارىنىڭ اراسىن اشىپ، قولدى كوتەرۋ: 1–2 تەرى ءتۇسى',
    '🖐🏼' => 'ساۋساقتارىنىڭ اراسىن اشىپ، قولدى كوتەرۋ: 3-تەرى ءتۇرى',
    '🖐🏽' => 'ساۋساقتارىنىڭ اراسىن اشىپ، قولدى كوتەرۋ: 4-تەرى ءتۇسى',
    '🖐🏾' => 'ساۋساقتارىنىڭ اراسىن اشىپ، قولدى كوتەرۋ: 5-تەرى ءتۇسى',
    '🖐🏿' => 'ساۋساقتارىنىڭ اراسىن اشىپ، قولدى كوتەرۋ: 6-تەرى ءتۇسى',
    '✋🏻' => 'قول كوتەرۋ: 1–2 تەرى ءتۇسى',
    '✋🏼' => 'قول كوتەرۋ: 3-تەرى ءتۇرى',
    '✋🏽' => 'قول كوتەرۋ: 4-تەرى ءتۇسى',
    '✋🏾' => 'قول كوتەرۋ: 5-تەرى ءتۇسى',
    '✋🏿' => 'قول كوتەرۋ: 6-تەرى ءتۇسى',
    '🖖🏻' => 'جانارتاۋشا سالەمدەسۋ: 1–2 تەرى ءتۇسى',
    '🖖🏼' => 'جانارتاۋشا سالەمدەسۋ: 3-تەرى ءتۇرى',
    '🖖🏽' => 'جانارتاۋشا سالەمدەسۋ: 4-تەرى ءتۇسى',
    '🖖🏾' => 'جانارتاۋشا سالەمدەسۋ: 5-تەرى ءتۇسى',
    '🖖🏿' => 'جانارتاۋشا سالەمدەسۋ: 6-تەرى ءتۇسى',
    '🫱🏻' => 'وڭعا قاراتىلعان قول: 1–2 تەرى ءتۇسى',
    '🫱🏼' => 'وڭعا قاراتىلعان قول: 3-تەرى ءتۇرى',
    '🫱🏽' => 'وڭعا قاراتىلعان قول: 4-تەرى ءتۇسى',
    '🫱🏾' => 'وڭعا قاراتىلعان قول: 5-تەرى ءتۇسى',
    '🫱🏿' => 'وڭعا قاراتىلعان قول: 6-تەرى ءتۇسى',
    '🫲🏻' => 'سولعا قاراتىلعان قول: 1–2 تەرى ءتۇسى',
    '🫲🏼' => 'سولعا قاراتىلعان قول: 3-تەرى ءتۇرى',
    '🫲🏽' => 'سولعا قاراتىلعان قول: 4-تەرى ءتۇسى',
    '🫲🏾' => 'سولعا قاراتىلعان قول: 5-تەرى ءتۇسى',
    '🫲🏿' => 'سولعا قاراتىلعان قول: 6-تەرى ءتۇسى',
    '🫳🏻' => 'الاقانىمەن تومەن قاراتىلعان قول: 1–2 تەرى ءتۇسى',
    '🫳🏼' => 'الاقانىمەن تومەن قاراتىلعان قول: 3-تەرى ءتۇرى',
    '🫳🏽' => 'الاقانىمەن تومەن قاراتىلعان قول: 4-تەرى ءتۇسى',
    '🫳🏾' => 'الاقانىمەن تومەن قاراتىلعان قول: 5-تەرى ءتۇسى',
    '🫳🏿' => 'الاقانىمەن تومەن قاراتىلعان قول: 6-تەرى ءتۇسى',
    '🫴🏻' => 'الاقانىمەن جوعارى قاراتىلعان قول: 1–2 تەرى ءتۇسى',
    '🫴🏼' => 'الاقانىمەن جوعارى قاراتىلعان قول: 3-تەرى ءتۇرى',
    '🫴🏽' => 'الاقانىمەن جوعارى قاراتىلعان قول: 4-تەرى ءتۇسى',
    '🫴🏾' => 'الاقانىمەن جوعارى قاراتىلعان قول: 5-تەرى ءتۇسى',
    '🫴🏿' => 'الاقانىمەن جوعارى قاراتىلعان قول: 6-تەرى ءتۇسى',
    '🫷🏻' => 'سولعا يتەرگەن قول: 1–2 تەرى ءتۇسى',
    '🫷🏼' => 'سولعا يتەرگەن قول: 3-تەرى ءتۇرى',
    '🫷🏽' => 'سولعا يتەرگەن قول: 4-تەرى ءتۇسى',
    '🫷🏾' => 'سولعا يتەرگەن قول: 5-تەرى ءتۇسى',
    '🫷🏿' => 'سولعا يتەرگەن قول: 6-تەرى ءتۇسى',
    '🫸🏻' => 'وڭعا يتەرگەن قول: 1–2 تەرى ءتۇسى',
    '🫸🏼' => 'وڭعا يتەرگەن قول: 3-تەرى ءتۇرى',
    '🫸🏽' => 'وڭعا يتەرگەن قول: 4-تەرى ءتۇسى',
    '🫸🏾' => 'وڭعا يتەرگەن قول: 5-تەرى ءتۇسى',
    '🫸🏿' => 'وڭعا يتەرگەن قول: 6-تەرى ءتۇسى',
    '👌🏻' => 'جاقسى: 1–2 تەرى ءتۇسى',
    '👌🏼' => 'جاقسى: 3-تەرى ءتۇرى',
    '👌🏽' => 'جاقسى: 4-تەرى ءتۇسى',
    '👌🏾' => 'جاقسى: 5-تەرى ءتۇسى',
    '👌🏿' => 'جاقسى: 6-تەرى ءتۇسى',
    '🤌🏻' => 'تۇيىستىرىلگەن بەس ساۋساق: 1–2 تەرى ءتۇسى',
    '🤌🏼' => 'تۇيىستىرىلگەن بەس ساۋساق: 3-تەرى ءتۇرى',
    '🤌🏽' => 'تۇيىستىرىلگەن بەس ساۋساق: 4-تەرى ءتۇسى',
    '🤌🏾' => 'تۇيىستىرىلگەن بەس ساۋساق: 5-تەرى ءتۇسى',
    '🤌🏿' => 'تۇيىستىرىلگەن بەس ساۋساق: 6-تەرى ءتۇسى',
    '🤏🏻' => 'ساۋساقپەن قىسىپ الۋ: 1–2 تەرى ءتۇسى',
    '🤏🏼' => 'ساۋساقپەن قىسىپ الۋ: 3-تەرى ءتۇرى',
    '🤏🏽' => 'ساۋساقپەن قىسىپ الۋ: 4-تەرى ءتۇسى',
    '🤏🏾' => 'ساۋساقپەن قىسىپ الۋ: 5-تەرى ءتۇسى',
    '🤏🏿' => 'ساۋساقپەن قىسىپ الۋ: 6-تەرى ءتۇسى',
    '✌🏻' => 'ساۋساقتارمەن «جەڭىس» كورسەتۋ: 1–2 تەرى ءتۇسى',
    '✌🏼' => 'ساۋساقتارمەن «جەڭىس» كورسەتۋ: 3-تەرى ءتۇرى',
    '✌🏽' => 'ساۋساقتارمەن «جەڭىس» كورسەتۋ: 4-تەرى ءتۇسى',
    '✌🏾' => 'ساۋساقتارمەن «جەڭىس» كورسەتۋ: 5-تەرى ءتۇسى',
    '✌🏿' => 'ساۋساقتارمەن «جەڭىس» كورسەتۋ: 6-تەرى ءتۇسى',
    '🤞🏻' => 'ساۋساقتارىن ايقاستىرعان: 1–2 تەرى ءتۇسى',
    '🤞🏼' => 'ساۋساقتارىن ايقاستىرعان: 3-تەرى ءتۇرى',
    '🤞🏽' => 'ساۋساقتارىن ايقاستىرعان: 4-تەرى ءتۇسى',
    '🤞🏾' => 'ساۋساقتارىن ايقاستىرعان: 5-تەرى ءتۇسى',
    '🤞🏿' => 'ساۋساقتارىن ايقاستىرعان: 6-تەرى ءتۇسى',
    '🫰🏻' => 'باس بارماق پەن سۇق ساۋساق ايقاستىرىلعان قول: 1–2 تەرى ءتۇسى',
    '🫰🏼' => 'باس بارماق پەن سۇق ساۋساق ايقاستىرىلعان قول: 3-تەرى ءتۇرى',
    '🫰🏽' => 'باس بارماق پەن سۇق ساۋساق ايقاستىرىلعان قول: 4-تەرى ءتۇسى',
    '🫰🏾' => 'باس بارماق پەن سۇق ساۋساق ايقاستىرىلعان قول: 5-تەرى ءتۇسى',
    '🫰🏿' => 'باس بارماق پەن سۇق ساۋساق ايقاستىرىلعان قول: 6-تەرى ءتۇسى',
    '🤟🏻' => 'سەنى سۇيەمىن بەلگىسى: 1–2 تەرى ءتۇسى',
    '🤟🏼' => 'سەنى سۇيەمىن بەلگىسى: 3-تەرى ءتۇرى',
    '🤟🏽' => 'سەنى سۇيەمىن بەلگىسى: 4-تەرى ءتۇسى',
    '🤟🏾' => 'سەنى سۇيەمىن بەلگىسى: 5-تەرى ءتۇسى',
    '🤟🏿' => 'سەنى سۇيەمىن بەلگىسى: 6-تەرى ءتۇسى',
    '🤘🏻' => 'ءمۇيىز بەلگىسى: 1–2 تەرى ءتۇسى',
    '🤘🏼' => 'ءمۇيىز بەلگىسى: 3-تەرى ءتۇرى',
    '🤘🏽' => 'ءمۇيىز بەلگىسى: 4-تەرى ءتۇسى',
    '🤘🏾' => 'ءمۇيىز بەلگىسى: 5-تەرى ءتۇسى',
    '🤘🏿' => 'ءمۇيىز بەلگىسى: 6-تەرى ءتۇسى',
    '🤙🏻' => 'حابارلاس: 1–2 تەرى ءتۇسى',
    '🤙🏼' => 'حابارلاس: 3-تەرى ءتۇرى',
    '🤙🏽' => 'حابارلاس: 4-تەرى ءتۇسى',
    '🤙🏾' => 'حابارلاس: 5-تەرى ءتۇسى',
    '🤙🏿' => 'حابارلاس: 6-تەرى ءتۇسى',
    '👈🏻' => 'سول جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 1–2 تەرى ءتۇسى',
    '👈🏼' => 'سول جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 3-تەرى ءتۇرى',
    '👈🏽' => 'سول جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 4-تەرى ءتۇسى',
    '👈🏾' => 'سول جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 5-تەرى ءتۇسى',
    '👈🏿' => 'سول جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 6-تەرى ءتۇسى',
    '👉🏻' => 'وڭ جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 1–2 تەرى ءتۇسى',
    '👉🏼' => 'وڭ جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 3-تەرى ءتۇرى',
    '👉🏽' => 'وڭ جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 4-تەرى ءتۇسى',
    '👉🏾' => 'وڭ جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 5-تەرى ءتۇسى',
    '👉🏿' => 'وڭ جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 6-تەرى ءتۇسى',
    '👆🏻' => 'جوعارى ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 1–2 تەرى ءتۇسى',
    '👆🏼' => 'جوعارى ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 3-تەرى ءتۇرى',
    '👆🏽' => 'جوعارى ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 4-تەرى ءتۇسى',
    '👆🏾' => 'جوعارى ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 5-تەرى ءتۇسى',
    '👆🏿' => 'جوعارى ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 6-تەرى ءتۇسى',
    '🖕🏻' => 'ورتاڭعى ساۋساق: 1–2 تەرى ءتۇسى',
    '🖕🏼' => 'ورتاڭعى ساۋساق: 3-تەرى ءتۇرى',
    '🖕🏽' => 'ورتاڭعى ساۋساق: 4-تەرى ءتۇسى',
    '🖕🏾' => 'ورتاڭعى ساۋساق: 5-تەرى ءتۇسى',
    '🖕🏿' => 'ورتاڭعى ساۋساق: 6-تەرى ءتۇسى',
    '👇🏻' => 'تومەن ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 1–2 تەرى ءتۇسى',
    '👇🏼' => 'تومەن ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 3-تەرى ءتۇرى',
    '👇🏽' => 'تومەن ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 4-تەرى ءتۇسى',
    '👇🏾' => 'تومەن ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 5-تەرى ءتۇسى',
    '👇🏿' => 'تومەن ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ: 6-تەرى ءتۇسى',
    '☝🏻' => 'جوعارىعا ساۋساقپەن كورسەتۋ: 1–2 تەرى ءتۇسى',
    '☝🏼' => 'جوعارىعا ساۋساقپەن كورسەتۋ: 3-تەرى ءتۇرى',
    '☝🏽' => 'جوعارىعا ساۋساقپەن كورسەتۋ: 4-تەرى ءتۇسى',
    '☝🏾' => 'جوعارىعا ساۋساقپەن كورسەتۋ: 5-تەرى ءتۇسى',
    '☝🏿' => 'جوعارىعا ساۋساقپەن كورسەتۋ: 6-تەرى ءتۇسى',
    '🫵🏻' => 'تىكە قاراتىلعان قول: 1–2 تەرى ءتۇسى',
    '🫵🏼' => 'تىكە قاراتىلعان قول: 3-تەرى ءتۇرى',
    '🫵🏽' => 'تىكە قاراتىلعان قول: 4-تەرى ءتۇسى',
    '🫵🏾' => 'تىكە قاراتىلعان قول: 5-تەرى ءتۇسى',
    '🫵🏿' => 'تىكە قاراتىلعان قول: 6-تەرى ءتۇسى',
    '👍🏻' => 'تاماشا: 1–2 تەرى ءتۇسى',
    '👍🏼' => 'تاماشا: 3-تەرى ءتۇرى',
    '👍🏽' => 'تاماشا: 4-تەرى ءتۇسى',
    '👍🏾' => 'تاماشا: 5-تەرى ءتۇسى',
    '👍🏿' => 'تاماشا: 6-تەرى ءتۇسى',
    '👎🏻' => 'ناشار: 1–2 تەرى ءتۇسى',
    '👎🏼' => 'ناشار: 3-تەرى ءتۇرى',
    '👎🏽' => 'ناشار: 4-تەرى ءتۇسى',
    '👎🏾' => 'ناشار: 5-تەرى ءتۇسى',
    '👎🏿' => 'ناشار: 6-تەرى ءتۇسى',
    '✊🏻' => 'جۇدىرىق كوتەرۋ: 1–2 تەرى ءتۇسى',
    '✊🏼' => 'جۇدىرىق كوتەرۋ: 3-تەرى ءتۇرى',
    '✊🏽' => 'جۇدىرىق كوتەرۋ: 4-تەرى ءتۇسى',
    '✊🏾' => 'جۇدىرىق كوتەرۋ: 5-تەرى ءتۇسى',
    '✊🏿' => 'جۇدىرىق كوتەرۋ: 6-تەرى ءتۇسى',
    '👊🏻' => 'جۇدىرىق: 1–2 تەرى ءتۇسى',
    '👊🏼' => 'جۇدىرىق: 3-تەرى ءتۇرى',
    '👊🏽' => 'جۇدىرىق: 4-تەرى ءتۇسى',
    '👊🏾' => 'جۇدىرىق: 5-تەرى ءتۇسى',
    '👊🏿' => 'جۇدىرىق: 6-تەرى ءتۇسى',
    '🤛🏻' => 'سولعا قاراتىلعان جۇدىرىق: 1–2 تەرى ءتۇسى',
    '🤛🏼' => 'سولعا قاراتىلعان جۇدىرىق: 3-تەرى ءتۇرى',
    '🤛🏽' => 'سولعا قاراتىلعان جۇدىرىق: 4-تەرى ءتۇسى',
    '🤛🏾' => 'سولعا قاراتىلعان جۇدىرىق: 5-تەرى ءتۇسى',
    '🤛🏿' => 'سولعا قاراتىلعان جۇدىرىق: 6-تەرى ءتۇسى',
    '🤜🏻' => 'وڭعا قاراتىلعان جۇدىرىق: 1–2 تەرى ءتۇسى',
    '🤜🏼' => 'وڭعا قاراتىلعان جۇدىرىق: 3-تەرى ءتۇرى',
    '🤜🏽' => 'وڭعا قاراتىلعان جۇدىرىق: 4-تەرى ءتۇسى',
    '🤜🏾' => 'وڭعا قاراتىلعان جۇدىرىق: 5-تەرى ءتۇسى',
    '🤜🏿' => 'وڭعا قاراتىلعان جۇدىرىق: 6-تەرى ءتۇسى',
    '👏🏻' => 'شاپالاقتاۋ: 1–2 تەرى ءتۇسى',
    '👏🏼' => 'شاپالاقتاۋ: 3-تەرى ءتۇرى',
    '👏🏽' => 'شاپالاقتاۋ: 4-تەرى ءتۇسى',
    '👏🏾' => 'شاپالاقتاۋ: 5-تەرى ءتۇسى',
    '👏🏿' => 'شاپالاقتاۋ: 6-تەرى ءتۇسى',
    '🙌🏻' => 'قولداردى كوتەرۋ: 1–2 تەرى ءتۇسى',
    '🙌🏼' => 'قولداردى كوتەرۋ: 3-تەرى ءتۇرى',
    '🙌🏽' => 'قولداردى كوتەرۋ: 4-تەرى ءتۇسى',
    '🙌🏾' => 'قولداردى كوتەرۋ: 5-تەرى ءتۇسى',
    '🙌🏿' => 'قولداردى كوتەرۋ: 6-تەرى ءتۇسى',
    '🫶🏻' => 'جۇرەكتى كورسەتىپ تۇرعان قولدار: 1–2 تەرى ءتۇسى',
    '🫶🏼' => 'جۇرەكتى كورسەتىپ تۇرعان قولدار: 3-تەرى ءتۇرى',
    '🫶🏽' => 'جۇرەكتى كورسەتىپ تۇرعان قولدار: 4-تەرى ءتۇسى',
    '🫶🏾' => 'جۇرەكتى كورسەتىپ تۇرعان قولدار: 5-تەرى ءتۇسى',
    '🫶🏿' => 'جۇرەكتى كورسەتىپ تۇرعان قولدار: 6-تەرى ءتۇسى',
    '👐🏻' => 'الاقاندى كورسەتۋ: 1–2 تەرى ءتۇسى',
    '👐🏼' => 'الاقاندى كورسەتۋ: 3-تەرى ءتۇرى',
    '👐🏽' => 'الاقاندى كورسەتۋ: 4-تەرى ءتۇسى',
    '👐🏾' => 'الاقاندى كورسەتۋ: 5-تەرى ءتۇسى',
    '👐🏿' => 'الاقاندى كورسەتۋ: 6-تەرى ءتۇسى',
    '🤲🏻' => 'الاقاندارىن كوتەرۋ: 1–2 تەرى ءتۇسى',
    '🤲🏼' => 'الاقاندارىن كوتەرۋ: 3-تەرى ءتۇرى',
    '🤲🏽' => 'الاقاندارىن كوتەرۋ: 4-تەرى ءتۇسى',
    '🤲🏾' => 'الاقاندارىن كوتەرۋ: 5-تەرى ءتۇسى',
    '🤲🏿' => 'الاقاندارىن كوتەرۋ: 6-تەرى ءتۇسى',
    '🤝🏻' => 'قول الىسۋ: 1–2 تەرى ءتۇسى',
    '🤝🏼' => 'قول الىسۋ: 3-تەرى ءتۇرى',
    '🤝🏽' => 'قول الىسۋ: 4-تەرى ءتۇسى',
    '🤝🏾' => 'قول الىسۋ: 5-تەرى ءتۇسى',
    '🤝🏿' => 'قول الىسۋ: 6-تەرى ءتۇسى',
    '🙏🏻' => 'قولدى ءتۇيىستىرۋ: 1–2 تەرى ءتۇسى',
    '🙏🏼' => 'قولدى ءتۇيىستىرۋ: 3-تەرى ءتۇرى',
    '🙏🏽' => 'قولدى ءتۇيىستىرۋ: 4-تەرى ءتۇسى',
    '🙏🏾' => 'قولدى ءتۇيىستىرۋ: 5-تەرى ءتۇسى',
    '🙏🏿' => 'قولدى ءتۇيىستىرۋ: 6-تەرى ءتۇسى',
    '✍🏻' => 'قولمەن جازۋ: 1–2 تەرى ءتۇسى',
    '✍🏼' => 'قولمەن جازۋ: 3-تەرى ءتۇرى',
    '✍🏽' => 'قولمەن جازۋ: 4-تەرى ءتۇسى',
    '✍🏾' => 'قولمەن جازۋ: 5-تەرى ءتۇسى',
    '✍🏿' => 'قولمەن جازۋ: 6-تەرى ءتۇسى',
    '💅🏻' => 'مانيكيۋر: 1–2 تەرى ءتۇسى',
    '💅🏼' => 'مانيكيۋر: 3-تەرى ءتۇرى',
    '💅🏽' => 'مانيكيۋر: 4-تەرى ءتۇسى',
    '💅🏾' => 'مانيكيۋر: 5-تەرى ءتۇسى',
    '💅🏿' => 'مانيكيۋر: 6-تەرى ءتۇسى',
    '🤳🏻' => 'سەلفي: 1–2 تەرى ءتۇسى',
    '🤳🏼' => 'سەلفي: 3-تەرى ءتۇرى',
    '🤳🏽' => 'سەلفي: 4-تەرى ءتۇسى',
    '🤳🏾' => 'سەلفي: 5-تەرى ءتۇسى',
    '🤳🏿' => 'سەلفي: 6-تەرى ءتۇسى',
    '💪🏻' => 'قول بۇلشىق ەتى: 1–2 تەرى ءتۇسى',
    '💪🏼' => 'قول بۇلشىق ەتى: 3-تەرى ءتۇرى',
    '💪🏽' => 'قول بۇلشىق ەتى: 4-تەرى ءتۇسى',
    '💪🏾' => 'قول بۇلشىق ەتى: 5-تەرى ءتۇسى',
    '💪🏿' => 'قول بۇلشىق ەتى: 6-تەرى ءتۇسى',
    '🦵🏻' => 'اياق: 1–2 تەرى ءتۇسى',
    '🦵🏼' => 'اياق: 3-تەرى ءتۇرى',
    '🦵🏽' => 'اياق: 4-تەرى ءتۇسى',
    '🦵🏾' => 'اياق: 5-تەرى ءتۇسى',
    '🦵🏿' => 'اياق: 6-تەرى ءتۇسى',
    '🦶🏻' => 'تابان: 1–2 تەرى ءتۇسى',
    '🦶🏼' => 'تابان: 3-تەرى ءتۇرى',
    '🦶🏽' => 'تابان: 4-تەرى ءتۇسى',
    '🦶🏾' => 'تابان: 5-تەرى ءتۇسى',
    '🦶🏿' => 'تابان: 6-تەرى ءتۇسى',
    '👂🏻' => 'قۇلاق: 1–2 تەرى ءتۇسى',
    '👂🏼' => 'قۇلاق: 3-تەرى ءتۇرى',
    '👂🏽' => 'قۇلاق: 4-تەرى ءتۇسى',
    '👂🏾' => 'قۇلاق: 5-تەرى ءتۇسى',
    '👂🏿' => 'قۇلاق: 6-تەرى ءتۇسى',
    '🦻🏻' => 'ەستۋ قۇرالى بار قۇلاق: 1–2 تەرى ءتۇسى',
    '🦻🏼' => 'ەستۋ قۇرالى بار قۇلاق: 3-تەرى ءتۇرى',
    '🦻🏽' => 'ەستۋ قۇرالى بار قۇلاق: 4-تەرى ءتۇسى',
    '🦻🏾' => 'ەستۋ قۇرالى بار قۇلاق: 5-تەرى ءتۇسى',
    '🦻🏿' => 'ەستۋ قۇرالى بار قۇلاق: 6-تەرى ءتۇسى',
    '👃🏻' => 'مۇرىن: 1–2 تەرى ءتۇسى',
    '👃🏼' => 'مۇرىن: 3-تەرى ءتۇرى',
    '👃🏽' => 'مۇرىن: 4-تەرى ءتۇسى',
    '👃🏾' => 'مۇرىن: 5-تەرى ءتۇسى',
    '👃🏿' => 'مۇرىن: 6-تەرى ءتۇسى',
    '👶🏻' => 'ءسابي: 1–2 تەرى ءتۇسى',
    '👶🏼' => 'ءسابي: 3-تەرى ءتۇرى',
    '👶🏽' => 'ءسابي: 4-تەرى ءتۇسى',
    '👶🏾' => 'ءسابي: 5-تەرى ءتۇسى',
    '👶🏿' => 'ءسابي: 6-تەرى ءتۇسى',
    '🧒🏻' => 'بالا: 1–2 تەرى ءتۇسى',
    '🧒🏼' => 'بالا: 3-تەرى ءتۇرى',
    '🧒🏽' => 'بالا: 4-تەرى ءتۇسى',
    '🧒🏾' => 'بالا: 5-تەرى ءتۇسى',
    '🧒🏿' => 'بالا: 6-تەرى ءتۇسى',
    '👦🏻' => 'ۇل: 1–2 تەرى ءتۇسى',
    '👦🏼' => 'ۇل: 3-تەرى ءتۇرى',
    '👦🏽' => 'ۇل: 4-تەرى ءتۇسى',
    '👦🏾' => 'ۇل: 5-تەرى ءتۇسى',
    '👦🏿' => 'ۇل: 6-تەرى ءتۇسى',
    '👧🏻' => 'قىز: 1–2 تەرى ءتۇسى',
    '👧🏼' => 'قىز: 3-تەرى ءتۇرى',
    '👧🏽' => 'قىز: 4-تەرى ءتۇسى',
    '👧🏾' => 'قىز: 5-تەرى ءتۇسى',
    '👧🏿' => 'قىز: 6-تەرى ءتۇسى',
    '🧑🏻' => 'ادام: 1–2 تەرى ءتۇسى',
    '🧑🏼' => 'ادام: 3-تەرى ءتۇرى',
    '🧑🏽' => 'ادام: 4-تەرى ءتۇسى',
    '🧑🏾' => 'ادام: 5-تەرى ءتۇسى',
    '🧑🏿' => 'ادام: 6-تەرى ءتۇسى',
    '👱🏻' => 'اققۇبا: 1–2 تەرى ءتۇسى',
    '👱🏼' => 'اققۇبا: 3-تەرى ءتۇرى',
    '👱🏽' => 'اققۇبا: 4-تەرى ءتۇسى',
    '👱🏾' => 'اققۇبا: 5-تەرى ءتۇسى',
    '👱🏿' => 'اققۇبا: 6-تەرى ءتۇسى',
    '👨🏻' => 'ەر ادام: 1–2 تەرى ءتۇسى',
    '👨🏼' => 'ەر ادام: 3-تەرى ءتۇرى',
    '👨🏽' => 'ەر ادام: 4-تەرى ءتۇسى',
    '👨🏾' => 'ەر ادام: 5-تەرى ءتۇسى',
    '👨🏿' => 'ەر ادام: 6-تەرى ءتۇسى',
    '🧔🏻' => 'ساقالدى ادام: 1–2 تەرى ءتۇسى',
    '🧔🏼' => 'ساقالدى ادام: 3-تەرى ءتۇرى',
    '🧔🏽' => 'ساقالدى ادام: 4-تەرى ءتۇسى',
    '🧔🏾' => 'ساقالدى ادام: 5-تەرى ءتۇسى',
    '🧔🏿' => 'ساقالدى ادام: 6-تەرى ءتۇسى',
    '👩🏻' => 'ايەل: 1–2 تەرى ءتۇسى',
    '👩🏼' => 'ايەل: 3-تەرى ءتۇرى',
    '👩🏽' => 'ايەل: 4-تەرى ءتۇسى',
    '👩🏾' => 'ايەل: 5-تەرى ءتۇسى',
    '👩🏿' => 'ايەل: 6-تەرى ءتۇسى',
    '🧓🏻' => 'قارت ادام: 1–2 تەرى ءتۇسى',
    '🧓🏼' => 'قارت ادام: 3-تەرى ءتۇرى',
    '🧓🏽' => 'قارت ادام: 4-تەرى ءتۇسى',
    '🧓🏾' => 'قارت ادام: 5-تەرى ءتۇسى',
    '🧓🏿' => 'قارت ادام: 6-تەرى ءتۇسى',
    '👴🏻' => 'قارت: 1–2 تەرى ءتۇسى',
    '👴🏼' => 'قارت: 3-تەرى ءتۇرى',
    '👴🏽' => 'قارت: 4-تەرى ءتۇسى',
    '👴🏾' => 'قارت: 5-تەرى ءتۇسى',
    '👴🏿' => 'قارت: 6-تەرى ءتۇسى',
    '👵🏻' => 'قارت ايەل: 1–2 تەرى ءتۇسى',
    '👵🏼' => 'قارت ايەل: 3-تەرى ءتۇرى',
    '👵🏽' => 'قارت ايەل: 4-تەرى ءتۇسى',
    '👵🏾' => 'قارت ايەل: 5-تەرى ءتۇسى',
    '👵🏿' => 'قارت ايەل: 6-تەرى ءتۇسى',
    '🙍🏻' => 'قاباعىن ءتۇيۋ: 1–2 تەرى ءتۇسى',
    '🙍🏼' => 'قاباعىن ءتۇيۋ: 3-تەرى ءتۇرى',
    '🙍🏽' => 'قاباعىن ءتۇيۋ: 4-تەرى ءتۇسى',
    '🙍🏾' => 'قاباعىن ءتۇيۋ: 5-تەرى ءتۇسى',
    '🙍🏿' => 'قاباعىن ءتۇيۋ: 6-تەرى ءتۇسى',
    '🙎🏻' => 'بەتى بۇرتيۋ: 1–2 تەرى ءتۇسى',
    '🙎🏼' => 'بەتى بۇرتيۋ: 3-تەرى ءتۇرى',
    '🙎🏽' => 'بەتى بۇرتيۋ: 4-تەرى ءتۇسى',
    '🙎🏾' => 'بەتى بۇرتيۋ: 5-تەرى ءتۇسى',
    '🙎🏿' => 'بەتى بۇرتيۋ: 6-تەرى ءتۇسى',
    '🙅🏻' => 'جوق: 1–2 تەرى ءتۇسى',
    '🙅🏼' => 'جوق: 3-تەرى ءتۇرى',
    '🙅🏽' => 'جوق: 4-تەرى ءتۇسى',
    '🙅🏾' => 'جوق: 5-تەرى ءتۇسى',
    '🙅🏿' => 'جوق: 6-تەرى ءتۇسى',
    '🙆🏻' => 'جارايدى: 1–2 تەرى ءتۇسى',
    '🙆🏼' => 'جارايدى: 3-تەرى ءتۇرى',
    '🙆🏽' => 'جارايدى: 4-تەرى ءتۇسى',
    '🙆🏾' => 'جارايدى: 5-تەرى ءتۇسى',
    '🙆🏿' => 'جارايدى: 6-تەرى ءتۇسى',
    '💁🏻' => 'اقپارات ورتالىعى: 1–2 تەرى ءتۇسى',
    '💁🏼' => 'اقپارات ورتالىعى: 3-تەرى ءتۇرى',
    '💁🏽' => 'اقپارات ورتالىعى: 4-تەرى ءتۇسى',
    '💁🏾' => 'اقپارات ورتالىعى: 5-تەرى ءتۇسى',
    '💁🏿' => 'اقپارات ورتالىعى: 6-تەرى ءتۇسى',
    '🙋🏻' => 'قولدى كوتەرۋ: 1–2 تەرى ءتۇسى',
    '🙋🏼' => 'قولدى كوتەرۋ: 3-تەرى ءتۇرى',
    '🙋🏽' => 'قولدى كوتەرۋ: 4-تەرى ءتۇسى',
    '🙋🏾' => 'قولدى كوتەرۋ: 5-تەرى ءتۇسى',
    '🙋🏿' => 'قولدى كوتەرۋ: 6-تەرى ءتۇسى',
    '🧏🏻' => 'ساڭىراۋ ادام: 1–2 تەرى ءتۇسى',
    '🧏🏼' => 'ساڭىراۋ ادام: 3-تەرى ءتۇرى',
    '🧏🏽' => 'ساڭىراۋ ادام: 4-تەرى ءتۇسى',
    '🧏🏾' => 'ساڭىراۋ ادام: 5-تەرى ءتۇسى',
    '🧏🏿' => 'ساڭىراۋ ادام: 6-تەرى ءتۇسى',
    '🙇🏻' => 'يىلگەن ادام: 1–2 تەرى ءتۇسى',
    '🙇🏼' => 'يىلگەن ادام: 3-تەرى ءتۇرى',
    '🙇🏽' => 'يىلگەن ادام: 4-تەرى ءتۇسى',
    '🙇🏾' => 'يىلگەن ادام: 5-تەرى ءتۇسى',
    '🙇🏿' => 'يىلگەن ادام: 6-تەرى ءتۇسى',
    '🤦🏻' => 'بەتىن قولمەن جاپقان ادام: 1–2 تەرى ءتۇسى',
    '🤦🏼' => 'بەتىن قولمەن جاپقان ادام: 3-تەرى ءتۇرى',
    '🤦🏽' => 'بەتىن قولمەن جاپقان ادام: 4-تەرى ءتۇسى',
    '🤦🏾' => 'بەتىن قولمەن جاپقان ادام: 5-تەرى ءتۇسى',
    '🤦🏿' => 'بەتىن قولمەن جاپقان ادام: 6-تەرى ءتۇسى',
    '🤷🏻' => 'بىلمەگەندىك بەلگىسى: 1–2 تەرى ءتۇسى',
    '🤷🏼' => 'بىلمەگەندىك بەلگىسى: 3-تەرى ءتۇرى',
    '🤷🏽' => 'بىلمەگەندىك بەلگىسى: 4-تەرى ءتۇسى',
    '🤷🏾' => 'بىلمەگەندىك بەلگىسى: 5-تەرى ءتۇسى',
    '🤷🏿' => 'بىلمەگەندىك بەلگىسى: 6-تەرى ءتۇسى',
    '👮🏻' => 'پوليتسيا قىزمەتكەرى: 1–2 تەرى ءتۇسى',
    '👮🏼' => 'پوليتسيا قىزمەتكەرى: 3-تەرى ءتۇرى',
    '👮🏽' => 'پوليتسيا قىزمەتكەرى: 4-تەرى ءتۇسى',
    '👮🏾' => 'پوليتسيا قىزمەتكەرى: 5-تەرى ءتۇسى',
    '👮🏿' => 'پوليتسيا قىزمەتكەرى: 6-تەرى ءتۇسى',
    '🕵🏻' => 'بارلاۋشى: 1–2 تەرى ءتۇسى',
    '🕵🏼' => 'بارلاۋشى: 3-تەرى ءتۇرى',
    '🕵🏽' => 'بارلاۋشى: 4-تەرى ءتۇسى',
    '🕵🏾' => 'بارلاۋشى: 5-تەرى ءتۇسى',
    '🕵🏿' => 'بارلاۋشى: 6-تەرى ءتۇسى',
    '💂🏻' => 'كۇزەتشى: 1–2 تەرى ءتۇسى',
    '💂🏼' => 'كۇزەتشى: 3-تەرى ءتۇرى',
    '💂🏽' => 'كۇزەتشى: 4-تەرى ءتۇسى',
    '💂🏾' => 'كۇزەتشى: 5-تەرى ءتۇسى',
    '💂🏿' => 'كۇزەتشى: 6-تەرى ءتۇسى',
    '🥷🏻' => 'جەنكەشتى: 1–2 تەرى ءتۇسى',
    '🥷🏼' => 'جەنكەشتى: 3-تەرى ءتۇرى',
    '🥷🏽' => 'جەنكەشتى: 4-تەرى ءتۇسى',
    '🥷🏾' => 'جەنكەشتى: 5-تەرى ءتۇسى',
    '🥷🏿' => 'جەنكەشتى: 6-تەرى ءتۇسى',
    '👷🏻' => 'قۇرىلىسشى: 1–2 تەرى ءتۇسى',
    '👷🏼' => 'قۇرىلىسشى: 3-تەرى ءتۇرى',
    '👷🏽' => 'قۇرىلىسشى: 4-تەرى ءتۇسى',
    '👷🏾' => 'قۇرىلىسشى: 5-تەرى ءتۇسى',
    '👷🏿' => 'قۇرىلىسشى: 6-تەرى ءتۇسى',
    '🫅🏻' => 'ءتاج تاققان ادام: 1–2 تەرى ءتۇسى',
    '🫅🏼' => 'ءتاج تاققان ادام: 3-تەرى ءتۇرى',
    '🫅🏽' => 'ءتاج تاققان ادام: 4-تەرى ءتۇسى',
    '🫅🏾' => 'ءتاج تاققان ادام: 5-تەرى ءتۇسى',
    '🫅🏿' => 'ءتاج تاققان ادام: 6-تەرى ءتۇسى',
    '🤴🏻' => 'حانزادا: 1–2 تەرى ءتۇسى',
    '🤴🏼' => 'حانزادا: 3-تەرى ءتۇرى',
    '🤴🏽' => 'حانزادا: 4-تەرى ءتۇسى',
    '🤴🏾' => 'حانزادا: 5-تەرى ءتۇسى',
    '🤴🏿' => 'حانزادا: 6-تەرى ءتۇسى',
    '👸🏻' => 'حانشايىم: 1–2 تەرى ءتۇسى',
    '👸🏼' => 'حانشايىم: 3-تەرى ءتۇرى',
    '👸🏽' => 'حانشايىم: 4-تەرى ءتۇسى',
    '👸🏾' => 'حانشايىم: 5-تەرى ءتۇسى',
    '👸🏿' => 'حانشايىم: 6-تەرى ءتۇسى',
    '👳🏻' => 'سالدەلى ادام: 1–2 تەرى ءتۇسى',
    '👳🏼' => 'سالدەلى ادام: 3-تەرى ءتۇرى',
    '👳🏽' => 'سالدەلى ادام: 4-تەرى ءتۇسى',
    '👳🏾' => 'سالدەلى ادام: 5-تەرى ءتۇسى',
    '👳🏿' => 'سالدەلى ادام: 6-تەرى ءتۇسى',
    '👲🏻' => 'قىتاي قالپاعىن كيگەن ادام: 1–2 تەرى ءتۇسى',
    '👲🏼' => 'قىتاي قالپاعىن كيگەن ادام: 3-تەرى ءتۇرى',
    '👲🏽' => 'قىتاي قالپاعىن كيگەن ادام: 4-تەرى ءتۇسى',
    '👲🏾' => 'قىتاي قالپاعىن كيگەن ادام: 5-تەرى ءتۇسى',
    '👲🏿' => 'قىتاي قالپاعىن كيگەن ادام: 6-تەرى ءتۇسى',
    '🧕🏻' => 'ورامال تاققان ايەل: 1–2 تەرى ءتۇسى',
    '🧕🏼' => 'ورامال تاققان ايەل: 3-تەرى ءتۇرى',
    '🧕🏽' => 'ورامال تاققان ايەل: 4-تەرى ءتۇسى',
    '🧕🏾' => 'ورامال تاققان ايەل: 5-تەرى ءتۇسى',
    '🧕🏿' => 'ورامال تاققان ايەل: 6-تەرى ءتۇسى',
    '🤵🏻' => 'سموكيڭ كيگەن ادام: 1–2 تەرى ءتۇسى',
    '🤵🏼' => 'سموكيڭ كيگەن ادام: 3-تەرى ءتۇرى',
    '🤵🏽' => 'سموكيڭ كيگەن ادام: 4-تەرى ءتۇسى',
    '🤵🏾' => 'سموكيڭ كيگەن ادام: 5-تەرى ءتۇسى',
    '🤵🏿' => 'سموكيڭ كيگەن ادام: 6-تەرى ءتۇسى',
    '👰🏻' => 'قالىڭدىق: 1–2 تەرى ءتۇسى',
    '👰🏼' => 'قالىڭدىق: 3-تەرى ءتۇرى',
    '👰🏽' => 'قالىڭدىق: 4-تەرى ءتۇسى',
    '👰🏾' => 'قالىڭدىق: 5-تەرى ءتۇسى',
    '👰🏿' => 'قالىڭدىق: 6-تەرى ءتۇسى',
    '🤰🏻' => 'ەكىقابات ايەل: 1–2 تەرى ءتۇسى',
    '🤰🏼' => 'ەكىقابات ايەل: 3-تەرى ءتۇرى',
    '🤰🏽' => 'ەكىقابات ايەل: 4-تەرى ءتۇسى',
    '🤰🏾' => 'ەكىقابات ايەل: 5-تەرى ءتۇسى',
    '🤰🏿' => 'ەكىقابات ايەل: 6-تەرى ءتۇسى',
    '🫃🏻' => 'جۇكتى ەر ادام: 1–2 تەرى ءتۇسى',
    '🫃🏼' => 'جۇكتى ەر ادام: 3-تەرى ءتۇرى',
    '🫃🏽' => 'جۇكتى ەر ادام: 4-تەرى ءتۇسى',
    '🫃🏾' => 'جۇكتى ەر ادام: 5-تەرى ءتۇسى',
    '🫃🏿' => 'جۇكتى ەر ادام: 6-تەرى ءتۇسى',
    '🫄🏻' => 'جۇكتى ادام: 1–2 تەرى ءتۇسى',
    '🫄🏼' => 'جۇكتى ادام: 3-تەرى ءتۇرى',
    '🫄🏽' => 'جۇكتى ادام: 4-تەرى ءتۇسى',
    '🫄🏾' => 'جۇكتى ادام: 5-تەرى ءتۇسى',
    '🫄🏿' => 'جۇكتى ادام: 6-تەرى ءتۇسى',
    '🤱🏻' => 'بالا ەمىزۋ: 1–2 تەرى ءتۇسى',
    '🤱🏼' => 'بالا ەمىزۋ: 3-تەرى ءتۇرى',
    '🤱🏽' => 'بالا ەمىزۋ: 4-تەرى ءتۇسى',
    '🤱🏾' => 'بالا ەمىزۋ: 5-تەرى ءتۇسى',
    '🤱🏿' => 'بالا ەمىزۋ: 6-تەرى ءتۇسى',
    '👼🏻' => 'كىشكەنتاي پەرىشتە: 1–2 تەرى ءتۇسى',
    '👼🏼' => 'كىشكەنتاي پەرىشتە: 3-تەرى ءتۇرى',
    '👼🏽' => 'كىشكەنتاي پەرىشتە: 4-تەرى ءتۇسى',
    '👼🏾' => 'كىشكەنتاي پەرىشتە: 5-تەرى ءتۇسى',
    '👼🏿' => 'كىشكەنتاي پەرىشتە: 6-تەرى ءتۇسى',
    '🎅🏻' => 'اياز اتا: 1–2 تەرى ءتۇسى',
    '🎅🏼' => 'اياز اتا: 3-تەرى ءتۇرى',
    '🎅🏽' => 'اياز اتا: 4-تەرى ءتۇسى',
    '🎅🏾' => 'اياز اتا: 5-تەرى ءتۇسى',
    '🎅🏿' => 'اياز اتا: 6-تەرى ءتۇسى',
    '🤶🏻' => 'اقشاقار: 1–2 تەرى ءتۇسى',
    '🤶🏼' => 'اقشاقار: 3-تەرى ءتۇرى',
    '🤶🏽' => 'اقشاقار: 4-تەرى ءتۇسى',
    '🤶🏾' => 'اقشاقار: 5-تەرى ءتۇسى',
    '🤶🏿' => 'اقشاقار: 6-تەرى ءتۇسى',
    '🦸🏻' => 'سۋپەر باتىر: 1–2 تەرى ءتۇسى',
    '🦸🏼' => 'سۋپەر باتىر: 3-تەرى ءتۇرى',
    '🦸🏽' => 'سۋپەر باتىر: 4-تەرى ءتۇسى',
    '🦸🏾' => 'سۋپەر باتىر: 5-تەرى ءتۇسى',
    '🦸🏿' => 'سۋپەر باتىر: 6-تەرى ءتۇسى',
    '🦹🏻' => 'سۋپەر زۇلىم: 1–2 تەرى ءتۇسى',
    '🦹🏼' => 'سۋپەر زۇلىم: 3-تەرى ءتۇرى',
    '🦹🏽' => 'سۋپەر زۇلىم: 4-تەرى ءتۇسى',
    '🦹🏾' => 'سۋپەر زۇلىم: 5-تەرى ءتۇسى',
    '🦹🏿' => 'سۋپەر زۇلىم: 6-تەرى ءتۇسى',
    '🧙🏻' => 'سيقىرشى: 1–2 تەرى ءتۇسى',
    '🧙🏼' => 'سيقىرشى: 3-تەرى ءتۇرى',
    '🧙🏽' => 'سيقىرشى: 4-تەرى ءتۇسى',
    '🧙🏾' => 'سيقىرشى: 5-تەرى ءتۇسى',
    '🧙🏿' => 'سيقىرشى: 6-تەرى ءتۇسى',
    '🧚🏻' => 'پەرى: 1–2 تەرى ءتۇسى',
    '🧚🏼' => 'پەرى: 3-تەرى ءتۇرى',
    '🧚🏽' => 'پەرى: 4-تەرى ءتۇسى',
    '🧚🏾' => 'پەرى: 5-تەرى ءتۇسى',
    '🧚🏿' => 'پەرى: 6-تەرى ءتۇسى',
    '🧛🏻' => 'قانىشەر: 1–2 تەرى ءتۇسى',
    '🧛🏼' => 'قانىشەر: 3-تەرى ءتۇرى',
    '🧛🏽' => 'قانىشەر: 4-تەرى ءتۇسى',
    '🧛🏾' => 'قانىشەر: 5-تەرى ءتۇسى',
    '🧛🏿' => 'قانىشەر: 6-تەرى ءتۇسى',
    '🧜🏻' => 'تەڭىز تۇرعىنى: 1–2 تەرى ءتۇسى',
    '🧜🏼' => 'تەڭىز تۇرعىنى: 3-تەرى ءتۇرى',
    '🧜🏽' => 'تەڭىز تۇرعىنى: 4-تەرى ءتۇسى',
    '🧜🏾' => 'تەڭىز تۇرعىنى: 5-تەرى ءتۇسى',
    '🧜🏿' => 'تەڭىز تۇرعىنى: 6-تەرى ءتۇسى',
    '🧝🏻' => 'ەلىف: 1–2 تەرى ءتۇسى',
    '🧝🏼' => 'ەلىف: 3-تەرى ءتۇرى',
    '🧝🏽' => 'ەلىف: 4-تەرى ءتۇسى',
    '🧝🏾' => 'ەلىف: 5-تەرى ءتۇسى',
    '🧝🏿' => 'ەلىف: 6-تەرى ءتۇسى',
    '💆🏻' => 'بەتكە ماسساج جاساۋ: 1–2 تەرى ءتۇسى',
    '💆🏼' => 'بەتكە ماسساج جاساۋ: 3-تەرى ءتۇرى',
    '💆🏽' => 'بەتكە ماسساج جاساۋ: 4-تەرى ءتۇسى',
    '💆🏾' => 'بەتكە ماسساج جاساۋ: 5-تەرى ءتۇسى',
    '💆🏿' => 'بەتكە ماسساج جاساۋ: 6-تەرى ءتۇسى',
    '💇🏻' => 'شاش قيدىرۋ: 1–2 تەرى ءتۇسى',
    '💇🏼' => 'شاش قيدىرۋ: 3-تەرى ءتۇرى',
    '💇🏽' => 'شاش قيدىرۋ: 4-تەرى ءتۇسى',
    '💇🏾' => 'شاش قيدىرۋ: 5-تەرى ءتۇسى',
    '💇🏿' => 'شاش قيدىرۋ: 6-تەرى ءتۇسى',
    '🚶🏻' => 'جاياۋ: 1–2 تەرى ءتۇسى',
    '🚶🏼' => 'جاياۋ: 3-تەرى ءتۇرى',
    '🚶🏽' => 'جاياۋ: 4-تەرى ءتۇسى',
    '🚶🏾' => 'جاياۋ: 5-تەرى ءتۇسى',
    '🚶🏿' => 'جاياۋ: 6-تەرى ءتۇسى',
    '🧍🏻' => 'تۇرىپ تۇرعان ادام: 1–2 تەرى ءتۇسى',
    '🧍🏼' => 'تۇرىپ تۇرعان ادام: 3-تەرى ءتۇرى',
    '🧍🏽' => 'تۇرىپ تۇرعان ادام: 4-تەرى ءتۇسى',
    '🧍🏾' => 'تۇرىپ تۇرعان ادام: 5-تەرى ءتۇسى',
    '🧍🏿' => 'تۇرىپ تۇرعان ادام: 6-تەرى ءتۇسى',
    '🧎🏻' => 'تىزەرلەپ وتىرعان ادام: 1–2 تەرى ءتۇسى',
    '🧎🏼' => 'تىزەرلەپ وتىرعان ادام: 3-تەرى ءتۇرى',
    '🧎🏽' => 'تىزەرلەپ وتىرعان ادام: 4-تەرى ءتۇسى',
    '🧎🏾' => 'تىزەرلەپ وتىرعان ادام: 5-تەرى ءتۇسى',
    '🧎🏿' => 'تىزەرلەپ وتىرعان ادام: 6-تەرى ءتۇسى',
    '🏃🏻' => 'جۇگىرۋ: 1–2 تەرى ءتۇسى',
    '🏃🏼' => 'جۇگىرۋ: 3-تەرى ءتۇرى',
    '🏃🏽' => 'جۇگىرۋ: 4-تەرى ءتۇسى',
    '🏃🏾' => 'جۇگىرۋ: 5-تەرى ءتۇسى',
    '🏃🏿' => 'جۇگىرۋ: 6-تەرى ءتۇسى',
    '💃🏻' => 'بيلەگەن ايەل: 1–2 تەرى ءتۇسى',
    '💃🏼' => 'بيلەگەن ايەل: 3-تەرى ءتۇرى',
    '💃🏽' => 'بيلەگەن ايەل: 4-تەرى ءتۇسى',
    '💃🏾' => 'بيلەگەن ايەل: 5-تەرى ءتۇسى',
    '💃🏿' => 'بيلەگەن ايەل: 6-تەرى ءتۇسى',
    '🕺🏻' => 'بيلەپ تۇرعان ەر ادام: 1–2 تەرى ءتۇسى',
    '🕺🏼' => 'بيلەپ تۇرعان ەر ادام: 3-تەرى ءتۇرى',
    '🕺🏽' => 'بيلەپ تۇرعان ەر ادام: 4-تەرى ءتۇسى',
    '🕺🏾' => 'بيلەپ تۇرعان ەر ادام: 5-تەرى ءتۇسى',
    '🕺🏿' => 'بيلەپ تۇرعان ەر ادام: 6-تەرى ءتۇسى',
    '🕴🏻' => 'اۋادا قالقىپ تۇرعان ىسكەر: 1–2 تەرى ءتۇسى',
    '🕴🏼' => 'اۋادا قالقىپ تۇرعان ىسكەر: 3-تەرى ءتۇرى',
    '🕴🏽' => 'اۋادا قالقىپ تۇرعان ىسكەر: 4-تەرى ءتۇسى',
    '🕴🏾' => 'اۋادا قالقىپ تۇرعان ىسكەر: 5-تەرى ءتۇسى',
    '🕴🏿' => 'اۋادا قالقىپ تۇرعان ىسكەر: 6-تەرى ءتۇسى',
    '👯🏻' => 'بيلەپ جاتقان ايەلدەر: 1–2 تەرى ءتۇسى',
    '👯🏼' => 'بيلەپ جاتقان ايەلدەر: 3-تەرى ءتۇرى',
    '👯🏽' => 'بيلەپ جاتقان ايەلدەر: 4-تەرى ءتۇسى',
    '👯🏾' => 'بيلەپ جاتقان ايەلدەر: 5-تەرى ءتۇسى',
    '👯🏿' => 'بيلەپ جاتقان ايەلدەر: 6-تەرى ءتۇسى',
    '🧖🏻' => 'بۋ بولمەدەگى ادام: 1–2 تەرى ءتۇسى',
    '🧖🏼' => 'بۋ بولمەدەگى ادام: 3-تەرى ءتۇرى',
    '🧖🏽' => 'بۋ بولمەدەگى ادام: 4-تەرى ءتۇسى',
    '🧖🏾' => 'بۋ بولمەدەگى ادام: 5-تەرى ءتۇسى',
    '🧖🏿' => 'بۋ بولمەدەگى ادام: 6-تەرى ءتۇسى',
    '🧗🏻' => 'ورمەلەپ بارا جاتقان ادام: 1–2 تەرى ءتۇسى',
    '🧗🏼' => 'ورمەلەپ بارا جاتقان ادام: 3-تەرى ءتۇرى',
    '🧗🏽' => 'ورمەلەپ بارا جاتقان ادام: 4-تەرى ءتۇسى',
    '🧗🏾' => 'ورمەلەپ بارا جاتقان ادام: 5-تەرى ءتۇسى',
    '🧗🏿' => 'ورمەلەپ بارا جاتقان ادام: 6-تەرى ءتۇسى',
    '🏇🏻' => 'اتشابار: 1–2 تەرى ءتۇسى',
    '🏇🏼' => 'اتشابار: 3-تەرى ءتۇرى',
    '🏇🏽' => 'اتشابار: 4-تەرى ءتۇسى',
    '🏇🏾' => 'اتشابار: 5-تەرى ءتۇسى',
    '🏇🏿' => 'اتشابار: 6-تەرى ءتۇسى',
    '🏂🏻' => 'سنوۋبوردشى: 1–2 تەرى ءتۇسى',
    '🏂🏼' => 'سنوۋبوردشى: 3-تەرى ءتۇرى',
    '🏂🏽' => 'سنوۋبوردشى: 4-تەرى ءتۇسى',
    '🏂🏾' => 'سنوۋبوردشى: 5-تەرى ءتۇسى',
    '🏂🏿' => 'سنوۋبوردشى: 6-تەرى ءتۇسى',
    '🏌🏻' => 'گولىف ويىنشىسى: 1–2 تەرى ءتۇسى',
    '🏌🏼' => 'گولىف ويىنشىسى: 3-تەرى ءتۇرى',
    '🏌🏽' => 'گولىف ويىنشىسى: 4-تەرى ءتۇسى',
    '🏌🏾' => 'گولىف ويىنشىسى: 5-تەرى ءتۇسى',
    '🏌🏿' => 'گولىف ويىنشىسى: 6-تەرى ءتۇسى',
    '🏄🏻' => 'سەرفەر: 1–2 تەرى ءتۇسى',
    '🏄🏼' => 'سەرفەر: 3-تەرى ءتۇرى',
    '🏄🏽' => 'سەرفەر: 4-تەرى ءتۇسى',
    '🏄🏾' => 'سەرفەر: 5-تەرى ءتۇسى',
    '🏄🏿' => 'سەرفەر: 6-تەرى ءتۇسى',
    '🚣🏻' => 'قايىق ەسىپ جاتقان ادام: 1–2 تەرى ءتۇسى',
    '🚣🏼' => 'قايىق ەسىپ جاتقان ادام: 3-تەرى ءتۇرى',
    '🚣🏽' => 'قايىق ەسىپ جاتقان ادام: 4-تەرى ءتۇسى',
    '🚣🏾' => 'قايىق ەسىپ جاتقان ادام: 5-تەرى ءتۇسى',
    '🚣🏿' => 'قايىق ەسىپ جاتقان ادام: 6-تەرى ءتۇسى',
    '🏊🏻' => 'ءجۇزۋشى: 1–2 تەرى ءتۇسى',
    '🏊🏼' => 'ءجۇزۋشى: 3-تەرى ءتۇرى',
    '🏊🏽' => 'ءجۇزۋشى: 4-تەرى ءتۇسى',
    '🏊🏾' => 'ءجۇزۋشى: 5-تەرى ءتۇسى',
    '🏊🏿' => 'ءجۇزۋشى: 6-تەرى ءتۇسى',
    '⛹🏻' => 'دوپ ويناعان ادام: 1–2 تەرى ءتۇسى',
    '⛹🏼' => 'دوپ ويناعان ادام: 3-تەرى ءتۇرى',
    '⛹🏽' => 'دوپ ويناعان ادام: 4-تەرى ءتۇسى',
    '⛹🏾' => 'دوپ ويناعان ادام: 5-تەرى ءتۇسى',
    '⛹🏿' => 'دوپ ويناعان ادام: 6-تەرى ءتۇسى',
    '🏋🏻' => 'زىلتەمىرشى: 1–2 تەرى ءتۇسى',
    '🏋🏼' => 'زىلتەمىرشى: 3-تەرى ءتۇرى',
    '🏋🏽' => 'زىلتەمىرشى: 4-تەرى ءتۇسى',
    '🏋🏾' => 'زىلتەمىرشى: 5-تەرى ءتۇسى',
    '🏋🏿' => 'زىلتەمىرشى: 6-تەرى ءتۇسى',
    '🚴🏻' => 'ۆەلوسيپەدشى: 1–2 تەرى ءتۇسى',
    '🚴🏼' => 'ۆەلوسيپەدشى: 3-تەرى ءتۇرى',
    '🚴🏽' => 'ۆەلوسيپەدشى: 4-تەرى ءتۇسى',
    '🚴🏾' => 'ۆەلوسيپەدشى: 5-تەرى ءتۇسى',
    '🚴🏿' => 'ۆەلوسيپەدشى: 6-تەرى ءتۇسى',
    '🚵🏻' => 'تاۋ ۆەلوسيپەدشىسى: 1–2 تەرى ءتۇسى',
    '🚵🏼' => 'تاۋ ۆەلوسيپەدشىسى: 3-تەرى ءتۇرى',
    '🚵🏽' => 'تاۋ ۆەلوسيپەدشىسى: 4-تەرى ءتۇسى',
    '🚵🏾' => 'تاۋ ۆەلوسيپەدشىسى: 5-تەرى ءتۇسى',
    '🚵🏿' => 'تاۋ ۆەلوسيپەدشىسى: 6-تەرى ءتۇسى',
    '🤸🏻' => 'اكروبات تريۋگى: 1–2 تەرى ءتۇسى',
    '🤸🏼' => 'اكروبات تريۋگى: 3-تەرى ءتۇرى',
    '🤸🏽' => 'اكروبات تريۋگى: 4-تەرى ءتۇسى',
    '🤸🏾' => 'اكروبات تريۋگى: 5-تەرى ءتۇسى',
    '🤸🏿' => 'اكروبات تريۋگى: 6-تەرى ءتۇسى',
    '🤼🏻' => 'كۇرەسشىلەر: 1–2 تەرى ءتۇسى',
    '🤼🏼' => 'كۇرەسشىلەر: 3-تەرى ءتۇرى',
    '🤼🏽' => 'كۇرەسشىلەر: 4-تەرى ءتۇسى',
    '🤼🏾' => 'كۇرەسشىلەر: 5-تەرى ءتۇسى',
    '🤼🏿' => 'كۇرەسشىلەر: 6-تەرى ءتۇسى',
    '🤽🏻' => 'سۋ پولوسى: 1–2 تەرى ءتۇسى',
    '🤽🏼' => 'سۋ پولوسى: 3-تەرى ءتۇرى',
    '🤽🏽' => 'سۋ پولوسى: 4-تەرى ءتۇسى',
    '🤽🏾' => 'سۋ پولوسى: 5-تەرى ءتۇسى',
    '🤽🏿' => 'سۋ پولوسى: 6-تەرى ءتۇسى',
    '🤾🏻' => 'گاندبول: 1–2 تەرى ءتۇسى',
    '🤾🏼' => 'گاندبول: 3-تەرى ءتۇرى',
    '🤾🏽' => 'گاندبول: 4-تەرى ءتۇسى',
    '🤾🏾' => 'گاندبول: 5-تەرى ءتۇسى',
    '🤾🏿' => 'گاندبول: 6-تەرى ءتۇسى',
    '🤹🏻' => 'جونگلەر: 1–2 تەرى ءتۇسى',
    '🤹🏼' => 'جونگلەر: 3-تەرى ءتۇرى',
    '🤹🏽' => 'جونگلەر: 4-تەرى ءتۇسى',
    '🤹🏾' => 'جونگلەر: 5-تەرى ءتۇسى',
    '🤹🏿' => 'جونگلەر: 6-تەرى ءتۇسى',
    '🧘🏻' => 'مالداس قۇرىپ وتىرعان ادام: 1–2 تەرى ءتۇسى',
    '🧘🏼' => 'مالداس قۇرىپ وتىرعان ادام: 3-تەرى ءتۇرى',
    '🧘🏽' => 'مالداس قۇرىپ وتىرعان ادام: 4-تەرى ءتۇسى',
    '🧘🏾' => 'مالداس قۇرىپ وتىرعان ادام: 5-تەرى ءتۇسى',
    '🧘🏿' => 'مالداس قۇرىپ وتىرعان ادام: 6-تەرى ءتۇسى',
    '🛀🏻' => 'ۆاننادا جۋىنۋ: 1–2 تەرى ءتۇسى',
    '🛀🏼' => 'ۆاننادا جۋىنۋ: 3-تەرى ءتۇرى',
    '🛀🏽' => 'ۆاننادا جۋىنۋ: 4-تەرى ءتۇسى',
    '🛀🏾' => 'ۆاننادا جۋىنۋ: 5-تەرى ءتۇسى',
    '🛀🏿' => 'ۆاننادا جۋىنۋ: 6-تەرى ءتۇسى',
    '🛌🏻' => 'ۇيىقتاپ جاتقان ادام: 1–2 تەرى ءتۇسى',
    '🛌🏼' => 'ۇيىقتاپ جاتقان ادام: 3-تەرى ءتۇرى',
    '🛌🏽' => 'ۇيىقتاپ جاتقان ادام: 4-تەرى ءتۇسى',
    '🛌🏾' => 'ۇيىقتاپ جاتقان ادام: 5-تەرى ءتۇسى',
    '🛌🏿' => 'ۇيىقتاپ جاتقان ادام: 6-تەرى ءتۇسى',
    '👭🏻' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 1–2 تەرى ءتۇسى',
    '👭🏼' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 3-تەرى ءتۇرى',
    '👭🏽' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 4-تەرى ءتۇسى',
    '👭🏾' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 5-تەرى ءتۇسى',
    '👭🏿' => 'قول ۇستاسىپ تۇرعان ەكى ايەل: 6-تەرى ءتۇسى',
    '👫🏻' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 1–2 تەرى ءتۇسى',
    '👫🏼' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 3-تەرى ءتۇرى',
    '👫🏽' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 4-تەرى ءتۇسى',
    '👫🏾' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 5-تەرى ءتۇسى',
    '👫🏿' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل: 6-تەرى ءتۇسى',
    '👬🏻' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 1–2 تەرى ءتۇسى',
    '👬🏼' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 3-تەرى ءتۇرى',
    '👬🏽' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 4-تەرى ءتۇسى',
    '👬🏾' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 5-تەرى ءتۇسى',
    '👬🏿' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك: 6-تەرى ءتۇسى',
    '💏🏻' => 'ءوبىس: 1–2 تەرى ءتۇسى',
    '💏🏼' => 'ءوبىس: 3-تەرى ءتۇرى',
    '💏🏽' => 'ءوبىس: 4-تەرى ءتۇسى',
    '💏🏾' => 'ءوبىس: 5-تەرى ءتۇسى',
    '💏🏿' => 'ءوبىس: 6-تەرى ءتۇسى',
    '💑🏻' => 'ماحاببات: 1–2 تەرى ءتۇسى',
    '💑🏼' => 'ماحاببات: 3-تەرى ءتۇرى',
    '💑🏽' => 'ماحاببات: 4-تەرى ءتۇسى',
    '💑🏾' => 'ماحاببات: 5-تەرى ءتۇسى',
    '💑🏿' => 'ماحاببات: 6-تەرى ءتۇسى',
    '🇦🇨' => 'تۋ: اسكەنجىن ارالى',
    '🇦🇩' => 'تۋ: اندوررا',
    '🇦🇪' => 'تۋ: بىرىككەن اراب امىرلىكتەرى',
    '🇦🇫' => 'تۋ: اۋعانستان',
    '🇦🇬' => 'تۋ: انتيگۋا جانە باربۋدا',
    '🇦🇮' => 'تۋ: انگيليا',
    '🇦🇱' => 'تۋ: البانيا',
    '🇦🇲' => 'تۋ: ارمەنيا',
    '🇦🇴' => 'تۋ: انگولا',
    '🇦🇶' => 'تۋ: انتاركتيدا',
    '🇦🇷' => 'تۋ: ارگەنتينا',
    '🇦🇸' => 'تۋ: امەريكالىق ساموا',
    '🇦🇹' => 'تۋ: اۋستريا',
    '🇦🇺' => 'تۋ: اۋستراليا',
    '🇦🇼' => 'تۋ: ارۋبا',
    '🇦🇽' => 'تۋ: الاند ارالدارى',
    '🇦🇿' => 'تۋ: ءازىربايجان',
    '🇧🇦' => 'تۋ: بوسنيا جانە گەرتسەگوۆينا',
    '🇧🇧' => 'تۋ: باربادوس',
    '🇧🇩' => 'تۋ: بانگلادەش',
    '🇧🇪' => 'تۋ: بەلگيا',
    '🇧🇫' => 'تۋ: بۋركينا-فاسو',
    '🇧🇬' => 'تۋ: بولگاريا',
    '🇧🇭' => 'تۋ: باحرەين',
    '🇧🇮' => 'تۋ: بۋرۋندي',
    '🇧🇯' => 'تۋ: بەنين',
    '🇧🇱' => 'تۋ: سان-بارتەمەلي',
    '🇧🇲' => 'تۋ: بەرمۋد ارالدارى',
    '🇧🇳' => 'تۋ: برۋنەي',
    '🇧🇴' => 'تۋ: بوليۆيا',
    '🇧🇶' => 'تۋ: بونەير، سينت-ەستاتيۋس جانە سابا',
    '🇧🇷' => 'تۋ: برازيليا',
    '🇧🇸' => 'تۋ: باگام ارالدارى',
    '🇧🇹' => 'تۋ: بۋتان',
    '🇧🇻' => 'تۋ: بۋۆە ارالى',
    '🇧🇼' => 'تۋ: بوتسۆانا',
    '🇧🇾' => 'تۋ: بەلارۋس',
    '🇧🇿' => 'تۋ: بەليز',
    '🇨🇦' => 'تۋ: كانادا',
    '🇨🇨' => 'تۋ: كوكوس (كيليڭ) ارالدارى',
    '🇨🇩' => 'تۋ: كونگو',
    '🇨🇫' => 'تۋ: ورتالىق افريكا رەسپۋبليكاسى',
    '🇨🇬' => 'تۋ: كونگو-براززاۆيل رەسپۋبليكاسى',
    '🇨🇭' => 'تۋ: شۆەيساريا',
    '🇨🇮' => 'تۋ: كوت-ديۆۋار',
    '🇨🇰' => 'تۋ: كۋك ارالدارى',
    '🇨🇱' => 'تۋ: چيلي',
    '🇨🇲' => 'تۋ: كامەرۋن',
    '🇨🇳' => 'تۋ: قىتاي',
    '🇨🇴' => 'تۋ: كولۋمبيا',
    '🇨🇵' => 'تۋ: كليپپەرتون ارالى',
    '🇨🇶' => 'تۋ: سارك',
    '🇨🇷' => 'تۋ: كوستا-ريكا',
    '🇨🇺' => 'تۋ: كۋبا',
    '🇨🇻' => 'تۋ: كابو-ۆەردە',
    '🇨🇼' => 'تۋ: كيۋراساو',
    '🇨🇽' => 'تۋ: حريستماس ارالى',
    '🇨🇾' => 'تۋ: كيپر',
    '🇨🇿' => 'تۋ: چەحيا',
    '🇩🇪' => 'تۋ: گەرمانيا',
    '🇩🇬' => 'تۋ: ديەگو-گارسيا',
    '🇩🇯' => 'تۋ: دجيبۋتي',
    '🇩🇰' => 'تۋ: دانيا',
    '🇩🇲' => 'تۋ: دومينيكا',
    '🇩🇴' => 'تۋ: دومينيكان رەسپۋبليكاسى',
    '🇩🇿' => 'تۋ: الجىر',
    '🇪🇦' => 'تۋ: سەۋتا جانە مەليليا',
    '🇪🇨' => 'تۋ: ەكۆادور',
    '🇪🇪' => 'تۋ: ەستونيا',
    '🇪🇬' => 'تۋ: مىسىر',
    '🇪🇭' => 'تۋ: باتىس ساحارا',
    '🇪🇷' => 'تۋ: ەريترەيا',
    '🇪🇸' => 'تۋ: يسپانيا',
    '🇪🇹' => 'تۋ: ەفيوپيا',
    '🇪🇺' => 'تۋ: ەۋروپالىق وداق',
    '🇫🇮' => 'تۋ: فينليانديا',
    '🇫🇯' => 'تۋ: فيجي',
    '🇫🇰' => 'تۋ: فولكلەند ارالدارى',
    '🇫🇲' => 'تۋ: ميكرونەزيا',
    '🇫🇴' => 'تۋ: فارەر ارالدارى',
    '🇫🇷' => 'تۋ: فرانسيا',
    '🇬🇦' => 'تۋ: گابون',
    '🇬🇧' => 'تۋ: ۇلىبريتانيا',
    '🇬🇩' => 'تۋ: گرەنادا',
    '🇬🇪' => 'تۋ: گرۋزيا',
    '🇬🇫' => 'تۋ: فرانسۋز گۆياناسى',
    '🇬🇬' => 'تۋ: گەرنسي',
    '🇬🇭' => 'تۋ: گانا',
    '🇬🇮' => 'تۋ: گيبرالتار',
    '🇬🇱' => 'تۋ: گرەنلانديا',
    '🇬🇲' => 'تۋ: گامبيا',
    '🇬🇳' => 'تۋ: گۆەنيا',
    '🇬🇵' => 'تۋ: گۆادەلۋپا',
    '🇬🇶' => 'تۋ: ەكۆاتورلىق گۆينەيا',
    '🇬🇷' => 'تۋ: گرەكيا',
    '🇬🇸' => 'تۋ: وڭتۇستىك گەورگيا جانە وڭتۇستىك ساندۆيچ ارالدارى',
    '🇬🇹' => 'تۋ: گۆاتەمالا',
    '🇬🇺' => 'تۋ: گۋان',
    '🇬🇼' => 'تۋ: گۆەنيا-بيساۋ',
    '🇬🇾' => 'تۋ: گايانا',
    '🇭🇰' => 'تۋ: حوڭكوڭ',
    '🇭🇲' => 'تۋ: حەرد ارالى جانە ماكدونالد ارالدارى',
    '🇭🇳' => 'تۋ: گوندۋراس',
    '🇭🇷' => 'تۋ: حورۆاتيا',
    '🇭🇹' => 'تۋ: گايتي',
    '🇭🇺' => 'تۋ: ماجارستان',
    '🇮🇨' => 'تۋ: كانار ارالدارى',
    '🇮🇩' => 'تۋ: يندونەزيا',
    '🇮🇪' => 'تۋ: يرلانديا',
    '🇮🇱' => 'تۋ: يزرايل',
    '🇮🇲' => 'تۋ: مەن ارالى',
    '🇮🇳' => 'تۋ: ءۇندىستان',
    '🇮🇴' => 'تۋ: ءۇندى مۇحيتىنداعى بريتان ايماعى',
    '🇮🇶' => 'تۋ: يراك',
    '🇮🇷' => 'تۋ: يران',
    '🇮🇸' => 'تۋ: يسلانديا',
    '🇮🇹' => 'تۋ: يتاليا',
    '🇯🇪' => 'تۋ: جەرسي',
    '🇯🇲' => 'تۋ: يامايكا',
    '🇯🇴' => 'تۋ: يوردانيا',
    '🇯🇵' => 'تۋ: جاپونيا',
    '🇰🇪' => 'تۋ: كەنيا',
    '🇰🇬' => 'تۋ: قىرعىزستان',
    '🇰🇭' => 'تۋ: كامباجا',
    '🇰🇮' => 'تۋ: كيريباتي',
    '🇰🇲' => 'تۋ: كومور ارالدارى',
    '🇰🇳' => 'تۋ: سەنت-كيتس جانە نەۆيس',
    '🇰🇵' => 'تۋ: سولتۇستىك كورەيا',
    '🇰🇷' => 'تۋ: وڭتۇستىك كورەيا',
    '🇰🇼' => 'تۋ: كۋۆەيت',
    '🇰🇾' => 'تۋ: كايمان ارالدارى',
    '🇰🇿' => 'تۋ: قازاق ەلى',
    '🇱🇦' => 'تۋ: لاوس',
    '🇱🇧' => 'تۋ: ليۆان',
    '🇱🇨' => 'تۋ: سەنت-ليۋسيا',
    '🇱🇮' => 'تۋ: ليحتەنشتەين',
    '🇱🇰' => 'تۋ: شري-لانكا',
    '🇱🇷' => 'تۋ: ليبەريا',
    '🇱🇸' => 'تۋ: لەسوتو',
    '🇱🇹' => 'تۋ: ليتۆا',
    '🇱🇺' => 'تۋ: ليۋكسەمبۋرگ',
    '🇱🇻' => 'تۋ: لاتۆيا',
    '🇱🇾' => 'تۋ: ليۆيا',
    '🇲🇦' => 'تۋ: ماروككو',
    '🇲🇨' => 'تۋ: موناكو',
    '🇲🇩' => 'تۋ: مولدوۆا',
    '🇲🇪' => 'تۋ: چەرنوگوريا',
    '🇲🇫' => 'تۋ: سەن-مارتەن',
    '🇲🇬' => 'تۋ: ماداگاسكار',
    '🇲🇭' => 'تۋ: مارشال ارالدارى',
    '🇲🇰' => 'تۋ: سولتۇستىك ماكەدونيا',
    '🇲🇱' => 'تۋ: مالي',
    '🇲🇲' => 'تۋ: ميانما (بيرما)',
    '🇲🇳' => 'تۋ: موڭعوليا',
    '🇲🇴' => 'تۋ: ماكاو',
    '🇲🇵' => 'تۋ: سولتۇستىك ماريانا ارالدارى',
    '🇲🇶' => 'تۋ: مارتينيكا',
    '🇲🇷' => 'تۋ: ماۆريتانيا',
    '🇲🇸' => 'تۋ: مونتسەررات',
    '🇲🇹' => 'تۋ: مالتا',
    '🇲🇺' => 'تۋ: ماۆريكي',
    '🇲🇻' => 'تۋ: مالديۆ ارالدارى',
    '🇲🇼' => 'تۋ: مالاۆي',
    '🇲🇽' => 'تۋ: مەكسيكا',
    '🇲🇾' => 'تۋ: مالايزيا',
    '🇲🇿' => 'تۋ: موزامبيك',
    '🇳🇦' => 'تۋ: ناميبيا',
    '🇳🇨' => 'تۋ: جاڭا كالەدونيا',
    '🇳🇪' => 'تۋ: نيگەر',
    '🇳🇫' => 'تۋ: نورفولك ارالى',
    '🇳🇬' => 'تۋ: نيگەريا',
    '🇳🇮' => 'تۋ: نيكاراگۋا',
    '🇳🇱' => 'تۋ: نيدەرلاند',
    '🇳🇴' => 'تۋ: نورۆەگيا',
    '🇳🇵' => 'تۋ: نەپال',
    '🇳🇷' => 'تۋ: ناۋرۋ',
    '🇳🇺' => 'تۋ: نيۋە',
    '🇳🇿' => 'تۋ: جاڭا زەلانديا',
    '🇴🇲' => 'تۋ: ومان',
    '🇵🇦' => 'تۋ: پاناما',
    '🇵🇪' => 'تۋ: پەرۋ',
    '🇵🇫' => 'تۋ: فرانتسۋز پولينەزياسى',
    '🇵🇬' => 'تۋ: پاپۋا — جاڭا گۆينەيا',
    '🇵🇭' => 'تۋ: فيليپين ارالدارى',
    '🇵🇰' => 'تۋ: پاكىستان',
    '🇵🇱' => 'تۋ: پولشا',
    '🇵🇲' => 'تۋ: سەن-پەر جانە ميكەلون',
    '🇵🇳' => 'تۋ: پيتكەرن ارالدارى',
    '🇵🇷' => 'تۋ: پۋەرتو-ريكو',
    '🇵🇸' => 'تۋ: پالەستينا ايماقتارى',
    '🇵🇹' => 'تۋ: پورتۋگاليا',
    '🇵🇼' => 'تۋ: پالاۋ',
    '🇵🇾' => 'تۋ: پاراگۆاي',
    '🇶🇦' => 'تۋ: كاتار',
    '🇷🇪' => 'تۋ: رەيۋنيون',
    '🇷🇴' => 'تۋ: رۋمىنيا',
    '🇷🇸' => 'تۋ: سەربيا',
    '🇷🇺' => 'تۋ: حەسەي',
    '🇷🇼' => 'تۋ: رۋاندا',
    '🇸🇦' => 'تۋ: ساۋد ارابياسى',
    '🇸🇧' => 'تۋ: سولومون ارالدارى',
    '🇸🇨' => 'تۋ: سەيشەل ارالدارى',
    '🇸🇩' => 'تۋ: سۋدان',
    '🇸🇪' => 'تۋ: شۆەسيا',
    '🇸🇬' => 'تۋ: سينگاپۋر',
    '🇸🇭' => 'تۋ: اۋليە ەلەنا ارالى',
    '🇸🇮' => 'تۋ: سلوۆەنيا',
    '🇸🇯' => 'تۋ: شپيتسبەرگەن جانە يان-مايەن',
    '🇸🇰' => 'تۋ: سلوۆاكيا',
    '🇸🇱' => 'تۋ: سەررا-لەونە',
    '🇸🇲' => 'تۋ: سان-مارينو',
    '🇸🇳' => 'تۋ: سەنەگال',
    '🇸🇴' => 'تۋ: سومالي',
    '🇸🇷' => 'تۋ: سۋرينام',
    '🇸🇸' => 'تۋ: وڭتۇستىك سۋدان',
    '🇸🇹' => 'تۋ: سان-تومە جانە پرينسيپي',
    '🇸🇻' => 'تۋ: سالۆادور',
    '🇸🇽' => 'تۋ: سينت-مارتەن',
    '🇸🇾' => 'تۋ: سيريا',
    '🇸🇿' => 'تۋ: ەسۆاتيني',
    '🇹🇦' => 'تۋ: تريستان-دا-كۋنيا',
    '🇹🇨' => 'تۋ: تەركس جانە كايكوس ارالدارى',
    '🇹🇩' => 'تۋ: چاد',
    '🇹🇫' => 'تۋ: فرانتسيانىڭ وڭتۇستىك ايماقتارى',
    '🇹🇬' => 'تۋ: توگو',
    '🇹🇭' => 'تۋ: تايلاند',
    '🇹🇯' => 'تۋ: تاجىكستان',
    '🇹🇰' => 'تۋ: توكەلاۋ',
    '🇹🇱' => 'تۋ: تيمور-لەستە',
    '🇹🇲' => 'تۋ: تۇرىكمەنستان',
    '🇹🇳' => 'تۋ: تۋنيس',
    '🇹🇴' => 'تۋ: تونگا',
    '🇹🇷' => 'تۋ: تۇركيا',
    '🇹🇹' => 'تۋ: ترينيداد جانە توباگو',
    '🇹🇻' => 'تۋ: تۋۆالۋ',
    '🇹🇼' => 'تۋ: تايۋان',
    '🇹🇿' => 'تۋ: تانزانيا',
    '🇺🇦' => 'تۋ: ۋكراينا',
    '🇺🇬' => 'تۋ: ۋگاندا',
    '🇺🇲' => 'تۋ: ا ق ش-تىڭ سىرتقى كىشى ارالدارى',
    '🇺🇳' => 'تۋ: بىرىككەن ۇلتتار ۇيىمى',
    '🇺🇸' => 'تۋ: امەريكا قۇراما شتاتتارى',
    '🇺🇾' => 'تۋ: ۋرۋگۆاي',
    '🇺🇿' => 'تۋ: وزبەكستان',
    '🇻🇦' => 'تۋ: ۆاتيكان',
    '🇻🇨' => 'تۋ: سەنت-ۆينسەنت جانە گرەنادين ارالدارى',
    '🇻🇪' => 'تۋ: ۆەنەسۋەلا',
    '🇻🇬' => 'تۋ: بريتاندىق ۆيرگين ارالدارى',
    '🇻🇮' => 'تۋ: ا ق ش-تىڭ ۆيرگين ارالدارى',
    '🇻🇳' => 'تۋ: ۆيەتنام',
    '🇻🇺' => 'تۋ: ۆانۋاتۋ',
    '🇼🇫' => 'تۋ: ۋولليس جانە فۋتۋنا',
    '🇼🇸' => 'تۋ: ساموا',
    '🇽🇰' => 'تۋ: كوسوۆو',
    '🇾🇪' => 'تۋ: يەمەن',
    '🇾🇹' => 'تۋ: مايوتتا',
    '🇿🇦' => 'تۋ: وڭتۇستىك افريكا',
    '🇿🇲' => 'تۋ: زامبيا',
    '🇿🇼' => 'تۋ: زيمبابۆە',
    '🔟' => 'پەرنە: 10',
    '🏻' => '1–2 تەرى ءتۇسى',
    '🏼' => '3-تەرى ءتۇرى',
    '🏽' => '4-تەرى ءتۇسى',
    '🏾' => '5-تەرى ءتۇسى',
    '🏿' => '6-تەرى ءتۇسى',
    '😀' => 'ەزۋ تارتىپ كۇلۋ',
    '😃' => 'اۋزىن اشىپ كۇلۋ',
    '😄' => 'اۋزىن اشىپ، ىرجيىپ تۇرىپ كۇلۋ',
    '😁' => 'بال-بۇل جايناپ كۇلۋ',
    '😆' => 'اۋزىن اشىپ، كوزىن جۇمىپ كۇلۋ',
    '😅' => 'ساسقالاقتاپ كۇلۋ',
    '🤣' => 'ىشەك-سىلەسى قاتىپ كۇلۋ',
    '😂' => 'كوزدەن جاس شىققانشا كۇلۋ',
    '🙂' => 'جىميۋ',
    '🙃' => 'مىسقىل',
    '🫠' => 'ەرىگەن بەت',
    '😉' => 'كوز قىسۋ',
    '😊' => 'ريزا بولىپ كۇلۋ',
    '😇' => 'بەيۋاز',
    '🥰' => 'جۇرەكشەلەرى بار بەت',
    '😍' => 'ءسۇيسىنىپ قاراۋ',
    '🤩' => 'جەلىگۋ',
    '😘' => 'كوزدى قىسىپ قويىپ ءسۇيۋ',
    '😗' => 'ءسۇيۋ',
    '☺' => 'كۇلۋ',
    '😚' => 'كوزدى جۇمىپ تۇرىپ ءسۇيۋ',
    '😙' => 'كۇلىمدەگەن كوزبەن ءسۇيۋ',
    '🥲' => 'جىلاپ تۇرىپ كۇل',
    '😋' => 'ءتىلىن شىعارعان',
    '😛' => 'ءتىلىن شىعارۋ',
    '😜' => 'كوزىن قىسىپ، ءتىلىن شىعارۋ',
    '🤪' => 'جىندى',
    '😝' => 'كوزىن جۇمىپ، ءتىلىن شىعارۋ',
    '🤑' => 'اقشادان كوز ايىرماۋ',
    '🤗' => 'قۇشاقتاۋ',
    '🤭' => 'اۋزىن قولمەن جاۋىپ كۇلۋ',
    '🫢' => 'كوزى اشىق، ال اۋزىن قولىمەن جاپقان',
    '🫣' => 'استىرتىن قاراۋ',
    '🤫' => 'ۇندەمە',
    '🤔' => 'ويلانۋ',
    '🫡' => 'ساليۋت بەرۋ',
    '🤐' => 'سويلەمەۋ',
    '🤨' => 'سەنبەۋ',
    '😐' => 'ءبارىبىر',
    '😑' => 'سەزىمسىز',
    '😶' => 'اۋزى جوق بەت',
    '🫥' => 'پۋنكتيرمەن سالىنعان بەت',
    '😏' => 'مىرس-مىرس كۇلۋ',
    '😒' => 'قىزىق ەمەس',
    '🙄' => 'كوزى الارۋ',
    '😬' => 'بەت-اۋزىن قيسايتۋ',
    '🤥' => 'وتىرىكشى',
    '🫨' => 'شايقالعان بەت',
    '😌' => 'جەڭىلدەپ قالۋ',
    '😔' => 'مۇڭدى',
    '😪' => 'ۇيقى باسۋ',
    '🤤' => 'سىلەكەيى اعىپ تۇر',
    '😴' => 'ۇيىقتاۋ',
    '🫩' => 'كوز ىسىگى',
    '😷' => 'بەتپەردە تاققان بەت',
    '🤒' => 'تەرمومەترى بار بەت',
    '🤕' => 'باسى بينتتەلگەن',
    '🤢' => 'جۇرەگى اينىپ تۇر',
    '🤮' => 'قۇسۋ',
    '🤧' => 'تۇشكىرۋ',
    '🥵' => 'ىستىقتاعان بەت',
    '🥶' => 'تونعان بەت',
    '🥴' => 'السىرەگەن ءجۇز',
    '😵' => 'كوزى كىرەسپەن سىزىلعان',
    '🤯' => 'باس جارىلعالى تۇر',
    '🤠' => 'سىيىرشى قالپاعى',
    '🥳' => 'تويلاۋ',
    '🥸' => 'بەتىن بۇركەپ الۋ',
    '😎' => 'كوزىلدىرىك تاعىپ جىميۋ',
    '🤓' => 'كوزىلدىرىك تاققان',
    '🧐' => 'مونوكىل تاعۋ',
    '😕' => 'ابدىراۋ',
    '🫤' => 'اۋزىن قيسايتۋ',
    '😟' => 'ۋايىمداۋ',
    '🙁' => 'ازداپ تۇنجىراۋ',
    '☹' => 'تۇنجىراۋ',
    '😮' => 'تاڭدانۋ',
    '😯' => 'تاڭعالۋ',
    '😲' => 'اڭ-تاڭ بولۋ',
    '😳' => 'بەتى قىزارۋ',
    '🫪' => 'بۇرمالانعان بەت',
    '🥺' => 'جالىنۋ',
    '🥹' => 'كوز جاسىن ارەڭ ۇستاۋ',
    '😦' => 'اۋزىن اشىپ، بۇرتيۋ',
    '😧' => 'قينالۋ',
    '😨' => 'شوشىپ كەتۋ',
    '😰' => 'قاتتى قوبالجۋ',
    '😥' => 'جەڭىلدەنۋ',
    '😢' => 'جىلاۋ',
    '😭' => 'ايقايلاپ جىلاۋ',
    '😱' => 'زارەسى ۇشۋ',
    '😖' => 'ەسى شىعۋ',
    '😣' => 'شىدامدىلىق',
    '😞' => 'ءتۇڭىلۋ',
    '😓' => 'قاتتى شارشاپ، تەرلەۋ',
    '😩' => 'قالجىراۋ',
    '😫' => 'قاتتى شارشاۋ',
    '🥱' => 'ەسىنەۋ',
    '😤' => 'مۇرنىنان بۋ شىققان',
    '😡' => 'اشۋلانۋ',
    '😠' => 'اشۋلى',
    '🤬' => 'دورەكى',
    '😈' => 'كۇلىمدەگەن ءىبىلىس',
    '👿' => 'اشۋلانعان ءىبىلىس',
    '💀' => 'باس سۇيەك',
    '☠' => 'باس سۇيەك پەن سۇيەكتەر',
    '💩' => 'بوق',
    '🤡' => 'سايقىمازاق',
    '👹' => 'قۇبىجىق',
    '👺' => 'گوبلين',
    '👻' => 'ەلەس',
    '👽' => 'وزگە عالامشارلىق',
    '👾' => 'وزگە عالامشاردان كەلگەن قۇبىجىق',
    '🤖' => 'قۇلتەمىر',
    '😺' => 'كۇلىمدەگەن مىسىق',
    '😸' => 'كۇلىپ تۇرعان مىسىق',
    '😹' => 'كوزىنەن جاس شىققانشا كۇلگەن مىسىق',
    '😻' => 'ءسۇيسىنىپ قاراپ تۇرعان مىسىق',
    '😼' => 'مىرس-مىرس كۇلىپ تۇرعان مىسىق',
    '😽' => 'ءسۇيىپ تۇرعان مىسىق',
    '🙀' => 'زارەسى ۇشقان مىسىق',
    '😿' => 'جىلاپ تۇرعان مىسىق',
    '😾' => 'اشۋلانعان مىسىق',
    '🙈' => 'كوزىن جاۋىپ تۇرعان مايمىل',
    '🙉' => 'قۇلاعىن جاۋىپ تۇرعان مايمىل',
    '🙊' => 'اۋزىن جاۋىپ تۇرعان مايمىل',
    '💌' => 'ماحاببات حاتى',
    '💘' => 'جەبە تيگەن جۇرەك',
    '💝' => 'تاسپالى جۇرەك',
    '💖' => 'جىلتىراعان جۇرەك',
    '💗' => 'ۇلكەيەتىن جۇرەك',
    '💓' => 'سوعىپ تۇرعان جۇرەك',
    '💞' => 'اينالىپ تۇرعان جۇرەكتەر',
    '💕' => 'قوس جۇرەك',
    '💟' => 'دەكوراتيۆتىك جۇرەك',
    '❣' => 'لەپ بەلگىسى ءتارىزدى جۇرەك',
    '💔' => 'جارالى جۇرەك',
    '❤' => 'قىزىل جۇرەك',
    '🩷' => 'قىزعىلت جۇرەك',
    '🧡' => 'قىزعىلت-سارى جۇرەك',
    '💛' => 'سارى جۇرەك',
    '💚' => 'جاسىل جۇرەك',
    '💙' => 'كوك جۇرەك',
    '🩵' => 'اقشىل كوك جۇرەك',
    '💜' => 'كۇلگىن جۇرەك',
    '🤎' => 'قوڭىر جۇرەك',
    '🖤' => 'قارا جۇرەك',
    '🩶' => 'سۇر جۇرەك',
    '🤍' => 'اق جۇرەك',
    '💋' => 'ەرىن',
    '💯' => 'ءجۇز ۇپاي',
    '💢' => 'اشۋ',
    '🫯' => 'بۇلتپەن كۇرەسۋ',
    '💥' => 'قاقتىعىس',
    '💫' => 'باسى اينالعان',
    '💦' => 'تەر تامشىلارى',
    '💨' => 'شاپشاڭ',
    '🕳' => 'تەسىك',
    '💬' => 'اڭگىمە بۇلتى',
    '🗨' => 'سول جاق اڭگىمە بۇلتى',
    '🗯' => 'وڭ جاق اشۋلى سۇحباتتاسۋ كوپىرشىگى',
    '💭' => 'وي بۇلتى',
    '💤' => 'ۇيقى',
    '👋' => 'قول بۇلعاۋ',
    '🤚' => 'الاقان سىرتىن كورسەتۋ',
    '🖐' => 'ساۋساقتارىنىڭ اراسىن اشىپ، قولدى كوتەرۋ',
    '✋' => 'قول كوتەرۋ',
    '🖖' => 'جانارتاۋشا سالەمدەسۋ',
    '🫱' => 'وڭعا قاراتىلعان قول',
    '🫲' => 'سولعا قاراتىلعان قول',
    '🫳' => 'الاقانىمەن تومەن قاراتىلعان قول',
    '🫴' => 'الاقانىمەن جوعارى قاراتىلعان قول',
    '🫷' => 'سولعا يتەرگەن قول',
    '🫸' => 'وڭعا يتەرگەن قول',
    '👌' => 'جاقسى',
    '🤌' => 'تۇيىستىرىلگەن بەس ساۋساق',
    '🤏' => 'ساۋساقپەن قىسىپ الۋ',
    '✌' => 'ساۋساقتارمەن «جەڭىس» كورسەتۋ',
    '🤞' => 'ساۋساقتارىن ايقاستىرعان',
    '🫰' => 'باس بارماق پەن سۇق ساۋساق ايقاستىرىلعان قول',
    '🤟' => 'سەنى سۇيەمىن بەلگىسى',
    '🤘' => 'ءمۇيىز بەلگىسى',
    '🤙' => 'حابارلاس',
    '👈' => 'سول جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ',
    '👉' => 'وڭ جاققا ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ',
    '👆' => 'جوعارى ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ',
    '🖕' => 'ورتاڭعى ساۋساق',
    '👇' => 'تومەن ساۋساقپەن قولدى سىرتقا قاراتىپ كورسەتۋ',
    '☝' => 'جوعارىعا ساۋساقپەن كورسەتۋ',
    '🫵' => 'تىكە قاراتىلعان قول',
    '👍' => 'تاماشا',
    '👎' => 'ناشار',
    '✊' => 'جۇدىرىق كوتەرۋ',
    '👊' => 'جۇدىرىق',
    '🤛' => 'سولعا قاراتىلعان جۇدىرىق',
    '🤜' => 'وڭعا قاراتىلعان جۇدىرىق',
    '👏' => 'شاپالاقتاۋ',
    '🙌' => 'قولداردى كوتەرۋ',
    '🫶' => 'جۇرەكتى كورسەتىپ تۇرعان قولدار',
    '👐' => 'الاقاندى كورسەتۋ',
    '🤲' => 'الاقاندارىن كوتەرۋ',
    '🤝' => 'قول الىسۋ',
    '🙏' => 'قولدى ءتۇيىستىرۋ',
    '✍' => 'قولمەن جازۋ',
    '💅' => 'مانيكيۋر',
    '🤳' => 'سەلفي',
    '💪' => 'قول بۇلشىق ەتى',
    '🦾' => 'مەحانيكالىق قول',
    '🦿' => 'مەحانيكالىق اياق',
    '🦵' => 'اياق',
    '🦶' => 'تابان',
    '👂' => 'قۇلاق',
    '🦻' => 'ەستۋ قۇرالى بار قۇلاق',
    '👃' => 'مۇرىن',
    '🧠' => 'مي',
    '🫀' => 'جۇرەك ورگانى',
    '🫁' => 'وكپە',
    '🦷' => 'ءتىس',
    '🦴' => 'سۇيەك',
    '👀' => 'كوزدەر',
    '👁' => 'كوز',
    '👅' => 'ءتىل',
    '👄' => 'اۋىز',
    '🫦' => 'ەرنىن تىستەۋ',
    '👶' => 'ءسابي',
    '🧒' => 'بالا',
    '👦' => 'ۇل',
    '👧' => 'قىز',
    '🧑' => 'ادام',
    '👱' => 'اققۇبا',
    '👨' => 'ەر ادام',
    '🧔' => 'ساقالدى ادام',
    '👩' => 'ايەل',
    '🧓' => 'قارت ادام',
    '👴' => 'قارت',
    '👵' => 'قارت ايەل',
    '🙍' => 'قاباعىن ءتۇيۋ',
    '🙎' => 'بەتى بۇرتيۋ',
    '🙅' => 'جوق',
    '🙆' => 'جارايدى',
    '💁' => 'اقپارات ورتالىعى',
    '🙋' => 'قولدى كوتەرۋ',
    '🧏' => 'ساڭىراۋ ادام',
    '🙇' => 'يىلگەن ادام',
    '🤦' => 'بەتىن قولمەن جاپقان ادام',
    '🤷' => 'بىلمەگەندىك بەلگىسى',
    '👮' => 'پوليتسيا قىزمەتكەرى',
    '🕵' => 'بارلاۋشى',
    '💂' => 'كۇزەتشى',
    '🥷' => 'جەنكەشتى',
    '👷' => 'قۇرىلىسشى',
    '🫅' => 'ءتاج تاققان ادام',
    '🤴' => 'حانزادا',
    '👸' => 'حانشايىم',
    '👳' => 'سالدەلى ادام',
    '👲' => 'قىتاي قالپاعىن كيگەن ادام',
    '🧕' => 'ورامال تاققان ايەل',
    '🤵' => 'سموكيڭ كيگەن ادام',
    '👰' => 'قالىڭدىق',
    '🤰' => 'ەكىقابات ايەل',
    '🫃' => 'جۇكتى ەر ادام',
    '🫄' => 'جۇكتى ادام',
    '🤱' => 'بالا ەمىزۋ',
    '👼' => 'كىشكەنتاي پەرىشتە',
    '🎅' => 'اياز اتا',
    '🤶' => 'اقشاقار',
    '🦸' => 'سۋپەر باتىر',
    '🦹' => 'سۋپەر زۇلىم',
    '🧙' => 'سيقىرشى',
    '🧚' => 'پەرى',
    '🧛' => 'قانىشەر',
    '🧜' => 'تەڭىز تۇرعىنى',
    '🧝' => 'ەلىف',
    '🧞' => 'جىن',
    '🧟' => 'زومبي',
    '🧌' => 'تىرول',
    '🫈' => 'كىسىكيىك',
    '💆' => 'بەتكە ماسساج جاساۋ',
    '💇' => 'شاش قيدىرۋ',
    '🚶' => 'جاياۋ',
    '🧍' => 'تۇرىپ تۇرعان ادام',
    '🧎' => 'تىزەرلەپ وتىرعان ادام',
    '🏃' => 'جۇگىرۋ',
    '💃' => 'بيلەگەن ايەل',
    '🕺' => 'بيلەپ تۇرعان ەر ادام',
    '🕴' => 'اۋادا قالقىپ تۇرعان ىسكەر',
    '👯' => 'بيلەپ جاتقان ايەلدەر',
    '🧖' => 'بۋ بولمەدەگى ادام',
    '🧗' => 'ورمەلەپ بارا جاتقان ادام',
    '🤺' => 'سەمسەرلەسىپ جاتقان ادام',
    '🏇' => 'اتشابار',
    '⛷' => 'شاڭعىشى',
    '🏂' => 'سنوۋبوردشى',
    '🏌' => 'گولىف ويىنشىسى',
    '🏄' => 'سەرفەر',
    '🚣' => 'قايىق ەسىپ جاتقان ادام',
    '🏊' => 'ءجۇزۋشى',
    '⛹' => 'دوپ ويناعان ادام',
    '🏋' => 'زىلتەمىرشى',
    '🚴' => 'ۆەلوسيپەدشى',
    '🚵' => 'تاۋ ۆەلوسيپەدشىسى',
    '🤸' => 'اكروبات تريۋگى',
    '🤼' => 'كۇرەسشىلەر',
    '🤽' => 'سۋ پولوسى',
    '🤾' => 'گاندبول',
    '🤹' => 'جونگلەر',
    '🧘' => 'مالداس قۇرىپ وتىرعان ادام',
    '🛀' => 'ۆاننادا جۋىنۋ',
    '🛌' => 'ۇيىقتاپ جاتقان ادام',
    '👭' => 'قول ۇستاسىپ تۇرعان ەكى ايەل',
    '👫' => 'قول ۇستاسىپ تۇرعان ەركەك پەن ايەل',
    '👬' => 'قول ۇستاسىپ تۇرعان ەكى ەركەك',
    '💏' => 'ءوبىس',
    '💑' => 'ماحاببات',
    '🗣' => 'سويلەۋ',
    '👤' => 'ادام سۇلباسى',
    '👥' => 'ادامدار سۇلباسى',
    '🫂' => 'قۇشاقتاسقان ادامدار',
    '👪' => 'وتباسى',
    '👣' => 'اياق ىزدەرى',
    '🫆' => 'ساۋساق ءىزى',
    '🦰' => 'جيرەن شاش',
    '🦱' => 'بۇيرا شاش',
    '🦳' => 'اق شاش',
    '🦲' => 'تاقىرباس',
    '🐵' => 'مايمىلدىڭ بەتى',
    '🐒' => 'مايمىل',
    '🦍' => 'گوريللا',
    '🦧' => 'ورانگۋتانگ',
    '🐶' => 'ءيتتىڭ بەتى',
    '🐕' => 'يت',
    '🦮' => 'جەتەكتەۋشى يت',
    '🐩' => 'پۋدەل',
    '🐺' => 'قاسقىردىڭ بەتى',
    '🦊' => 'تۇلكىنىڭ بەتى',
    '🦝' => 'جانات',
    '🐱' => 'مىسىقتىڭ بەتى',
    '🐈' => 'مىسىق',
    '🦁' => 'ارىستاننىڭ بەتى',
    '🐯' => 'جولبارىستىڭ بەتى',
    '🐅' => 'جولبارىس',
    '🐆' => 'قابىلان',
    '🐴' => 'جىلقىنىڭ بەتى',
    '🫎' => 'بۇلان',
    '🫏' => 'ەسەك',
    '🐎' => 'جىلقى',
    '🦄' => 'جالعىز ءمۇيىزدىڭ بەتى',
    '🦓' => 'الا قۇلان',
    '🦌' => 'بۇعى',
    '🦬' => 'بيزون',
    '🐮' => 'سيىردىڭ بەتى',
    '🐂' => 'بۇقا',
    '🐃' => 'ەنەكە',
    '🐄' => 'سيىر',
    '🐷' => 'شوشقانىڭ بەتى',
    '🐖' => 'شوشقا',
    '🐗' => 'قابان',
    '🐽' => 'شوشقانىڭ مۇرنى',
    '🐏' => 'قوشقار',
    '🐑' => 'قوي',
    '🐐' => 'ەشكى',
    '🐪' => 'تۇيە',
    '🐫' => 'قوس وركەشتى تۇيە',
    '🦙' => 'لاما',
    '🦒' => 'كەرىك',
    '🐘' => 'ءپىل',
    '🦣' => 'ءزىل',
    '🦏' => 'ءمۇيىز تۇمسىق',
    '🦛' => 'بەگەموت',
    '🐭' => 'تىشقاننىڭ بەتى',
    '🐁' => 'تىشقان',
    '🐀' => 'ەگەۋقۇيرىق',
    '🐹' => 'الاقورجىننىڭ بەتى',
    '🐰' => 'قوياننىڭ بەتى',
    '🐇' => 'قويان',
    '🐿' => 'بورشاتىشقان',
    '🦫' => 'قۇندىز',
    '🦔' => 'كىرپى',
    '🦇' => 'جارقانات',
    '🐻' => 'ايۋدىڭ بەتى',
    '🐨' => 'كوالا',
    '🐼' => 'پاندانىڭ بەتى',
    '🦥' => 'جالقاۋ اڭ',
    '🦦' => 'كامشات',
    '🦨' => 'ساسىقيىستى',
    '🦘' => 'كەنگۋرۋ',
    '🦡' => 'بورسىق',
    '🐾' => 'تابان ىزدەرى',
    '🦃' => 'كۇركەتاۋىق',
    '🐔' => 'تاۋىق',
    '🐓' => 'اتەش',
    '🐣' => 'جۇمىرتقانى جارىپ شىققان بالاپان',
    '🐤' => 'بالاپان',
    '🐥' => 'العا قاراپ تۇرعان بالاپان',
    '🐦' => 'قۇس',
    '🐧' => 'پينگۆين',
    '🕊' => 'كوگەرشىن',
    '🦅' => 'بۇركىت',
    '🦆' => 'ۇيرەك',
    '🦢' => 'اققۋ',
    '🦉' => 'جاپالاق',
    '🦤' => 'درونت',
    '🪶' => 'قاۋىرسىن',
    '🦩' => 'فلامينگو',
    '🦚' => 'تاۋىس',
    '🦜' => 'توتىقۇس',
    '🪽' => 'قانات',
    '🪿' => 'قاز',
    '🐸' => 'باقانىڭ بەتى',
    '🐊' => 'قولتىراۋىن',
    '🐢' => 'تاسباقا',
    '🦎' => 'كەسىرتكە',
    '🐍' => 'جىلان',
    '🐲' => 'ايداھاردىڭ بەتى',
    '🐉' => 'ايداھار',
    '🦕' => 'زاۋروپود',
    '🦖' => 'تي-رەكس',
    '🐳' => 'سۋ بۇركىپ تۇرعان كيت',
    '🐋' => 'كيت',
    '🐬' => 'دەلفين',
    '🫍' => 'كوساتكا',
    '🦭' => 'يتبالىق',
    '🐟' => 'بالىق',
    '🐠' => 'تروپيكالىق بالىق',
    '🐡' => 'كىرپىبالىق',
    '🦈' => 'اكۋلا',
    '🐙' => 'سەگىزاياق',
    '🐚' => 'شيىرشىقتى باقالشىق',
    '🪸' => 'مارجان',
    '🪼' => 'مەدۋزا',
    '🦀' => 'تەڭىز شايانى',
    '🦞' => 'لوبستەر',
    '🦐' => 'اسشايان',
    '🦑' => 'كالمار',
    '🦪' => 'ۋستريسا',
    '🐌' => 'ۇلۋ',
    '🦋' => 'كوبەلەك',
    '🐛' => 'جاندىك',
    '🐜' => 'قۇمىرسقا',
    '🐝' => 'بال ارا',
    '🪲' => 'قوڭىز',
    '🐞' => 'حانقىزى',
    '🦗' => 'شەگىرتكە',
    '🪳' => 'تاراقان',
    '🕷' => 'ورمەكشى',
    '🕸' => 'ورمەكشى تورى',
    '🦂' => 'سارىشايان',
    '🦟' => 'ماسا',
    '🪰' => 'شىبىن',
    '🪱' => 'قۇرت',
    '🦠' => 'ميكروب',
    '💐' => 'گۇل شوعى',
    '🌸' => 'شيە گۇلى',
    '💮' => 'اق گۇل',
    '🪷' => 'لوتوس',
    '🏵' => 'روزەتتا',
    '🌹' => 'راۋشان',
    '🥀' => 'سولعان گۇل',
    '🌺' => 'بورىتاراق',
    '🌻' => 'كۇنباعىس',
    '🌼' => 'گۇلدەۋ',
    '🌷' => 'قىزعالداق',
    '🪻' => 'ءسۇمبىل',
    '🌱' => 'كوشەت',
    '🪴' => 'بولمە وسىمدىگى',
    '🌲' => 'شىرشا',
    '🌳' => 'جاپىراقتى اعاش',
    '🌴' => 'پالما',
    '🌵' => 'كاكتۋس',
    '🌾' => 'ءبىر بايلام كۇرىش',
    '🌿' => 'ءشوپ',
    '☘' => 'قويان ساۋمالدىق',
    '🍀' => 'ءتورت جاپىراقتى جوڭىشقا',
    '🍁' => 'ۇيەڭكى جاپىراعى',
    '🍂' => 'سارعايىپ تۇسكەن جاپىراقتار',
    '🍃' => 'جەلمەن ۇشقان جاپىراقتار',
    '🪹' => 'بوس ۇيا',
    '🪺' => 'ىشىندە جۇمىرتقالارى بار ۇيا',
    '🍄' => 'ساڭىراۋقۇلاق',
    '🪾' => 'جاپىراقسىز اعاش',
    '🍇' => 'ءجۇزىم',
    '🍈' => 'قاۋىن',
    '🍉' => 'قاربىز',
    '🍊' => 'ماندارين',
    '🍋' => 'ليمون',
    '🍌' => 'بانان',
    '🍍' => 'اناناس',
    '🥭' => 'مانگو',
    '🍎' => 'قىزىل الما',
    '🍏' => 'كوك الما',
    '🍐' => 'المۇرت',
    '🍑' => 'شابدالى',
    '🍒' => 'شيە',
    '🍓' => 'قۇلپىناي',
    '🫐' => 'قاراجيدەك',
    '🥝' => 'كيۆي',
    '🍅' => 'قىزاناق',
    '🫒' => 'ءزايتۇن',
    '🥥' => 'كوكوس',
    '🥑' => 'اۆوكادو',
    '🍆' => 'بايالدى',
    '🥔' => 'كارتوپ',
    '🥕' => 'ءسابىز',
    '🌽' => 'جۇگەرى سوبىعى',
    '🌶' => 'اشى بۇرىش',
    '🫑' => 'بولگار بۇرىشى',
    '🥒' => 'قيار',
    '🥬' => 'جاسىل جاپىراق',
    '🥦' => 'بروككولي',
    '🧄' => 'سارىمساق',
    '🧅' => 'پياز',
    '🥜' => 'جەرجاڭعاق',
    '🫘' => 'بۇرشاق',
    '🌰' => 'كاشتان',
    '🫚' => 'ءزىمبىر تامىرى',
    '🫛' => 'بۇرشاققىندى اسبۇرشاق',
    '🫜' => 'تامىر جەمىس',
    '🍞' => 'نان',
    '🥐' => 'كرۋاسسان',
    '🥖' => 'باگەت',
    '🫓' => 'شەلپەك نان',
    '🥨' => 'تۇزدى توقاش',
    '🥯' => 'بەيگل',
    '🥞' => 'قۇيماق',
    '🧇' => 'ۆافليا',
    '🧀' => 'ىرىمشىك بولىگى',
    '🍖' => 'سۇيەگى بار ەت',
    '🍗' => 'تاۋىقتىڭ اياعى',
    '🥩' => 'ەت كەسىمى',
    '🥓' => 'سۇرسۇبە',
    '🍔' => 'گامبۋرگەر',
    '🍟' => 'فري',
    '🍕' => 'پيتستسا',
    '🌭' => 'حوت-دوگ',
    '🥪' => 'سەندۆيچ',
    '🌮' => 'تاكو',
    '🌯' => 'بۋريتو',
    '🫔' => 'تامال',
    '🥙' => 'شاۋرما',
    '🧆' => 'فالافەل',
    '🥚' => 'جۇمىرتقا',
    '🍳' => 'ءپىسىرۋ',
    '🥘' => 'تاباداعى راگۋ',
    '🍲' => 'تاماق تولى كاسترول',
    '🫕' => 'فونديۋ',
    '🥣' => 'قاسىعى بار ىدىس',
    '🥗' => 'كوكونىس سالاتى',
    '🍿' => 'پوپكورن',
    '🧈' => 'سارى ماي',
    '🧂' => 'تۇز',
    '🥫' => 'كونسەرۆىلەر',
    '🍱' => 'بەنتو',
    '🍘' => 'كۇرىشتەن جاسالعان كرەكەر',
    '🍙' => 'كۇرىش شارى',
    '🍚' => 'پىسىرىلگەن كۇرىش',
    '🍛' => 'كارري قوسىلعان كۇرىش',
    '🍜' => 'بۋى شىعىپ تۇرعان ىدىس',
    '🍝' => 'سپاگەتتي',
    '🍠' => 'قۋىرىلعان باتات',
    '🍢' => 'ودەن',
    '🍣' => 'سۋشي',
    '🍤' => 'قۋىرىلعان اسشايان',
    '🍥' => 'ءيىرىم ءتارىزدى بالىقتان جاسالعان كوتلەت',
    '🥮' => 'قىتاي پيروگى',
    '🍡' => 'دانگو',
    '🥟' => 'ۇزبە كەسپە',
    '🥠' => 'بولجاۋ پىسپەنانى',
    '🥡' => 'جول تاعامى',
    '🍦' => 'جۇمساق بالمۇزداق',
    '🍧' => 'مۇزدى بالمۇزداق',
    '🍨' => 'بالمۇزداق',
    '🍩' => 'بۇيرەكشە ءبالىش',
    '🍪' => 'پىسپەنان',
    '🎂' => 'تۋعان كۇنگە ارنالعان تورت',
    '🍰' => 'قاتپارلى تورت',
    '🧁' => 'كەكس',
    '🥧' => 'ءبالىش',
    '🍫' => 'شوكولاد تاقتاسى',
    '🍬' => 'مۇز كامپيت',
    '🍭' => 'تاياقشالى مۇز كامپيت',
    '🍮' => 'كاستارد',
    '🍯' => 'بال قۇتىسى',
    '🍼' => 'ءسابي بوتەلكەسى',
    '🥛' => 'ستاقان ءسۇت',
    '☕' => 'ىستىق سۋسىن',
    '🫖' => 'شاينەك',
    '🍵' => 'تۇتقاسى جوق شىناياق',
    '🍶' => 'ساكە',
    '🍾' => 'تىعىنى اتىلعان بوتەلكە',
    '🍷' => 'شاراپ بوكالى',
    '🍸' => 'كوكتەيل بوكالى',
    '🍹' => 'تروپيكالىق سۋسىن',
    '🍺' => 'سىرا ساپتىاياعى',
    '🍻' => 'قاعىستىرىلعان سىرا ساپتىاياقتارى',
    '🥂' => 'ستاقان سوعىستىرۋ',
    '🥃' => 'ستاقان',
    '🫗' => 'سۇيىقتىقتى قۇيۋ',
    '🥤' => 'تۇتىكشەلى ىدىس',
    '🧋' => 'كوبىكتى شاي',
    '🧃' => 'سۋسىن پاكەتى',
    '🧉' => 'ماتە',
    '🧊' => 'مۇز تەكشەسى',
    '🥢' => 'تاماق تاياقشالارى',
    '🍽' => 'تارەلكە، شانىشقى جانە پىشاق',
    '🍴' => 'شانىشقى مەن پىشاق',
    '🥄' => 'قاسىق',
    '🔪' => 'اسحانا پىشاعى',
    '🫙' => 'قۇتى',
    '🏺' => 'امفورا',
    '🌍' => 'ەۋروپا مەن افريكا كورسەتىلگەن گلوبۋس',
    '🌎' => 'ەكى امەريكا قۇرلىعى كورسەتىلگەن گلوبۋس',
    '🌏' => 'ازيا مەن اۋستراليا كورسەتىلگەن گلوبۋس',
    '🌐' => 'مەريدياندارى كورسەتىلگەن گلوبۋس',
    '🗺' => 'كارتا',
    '🗾' => 'جاپونيا كارتاسى',
    '🧭' => 'تۇسباعدار',
    '🏔' => 'توبەسىن قار باسقان تاۋ',
    '⛰' => 'تاۋ',
    '🛘' => 'كوشكىن',
    '🌋' => 'جانارتاۋ',
    '🗻' => 'فۋجياما',
    '🏕' => 'كەمپيڭ',
    '🏖' => 'جاعاجايدا شاتىرمەن',
    '🏜' => 'ءشول',
    '🏝' => 'ءشولدى ارال',
    '🏞' => 'ۇلتتىق پارك',
    '🏟' => 'ستاديون',
    '🏛' => 'كلاسسيكالىق عيمارات',
    '🏗' => 'عيمارات سالۋ',
    '🧱' => 'كىرپىش',
    '🪨' => 'تاس',
    '🪵' => 'اعاش',
    '🛖' => 'لاشىق',
    '🏘' => 'ۇيلەر',
    '🏚' => 'قاراۋسىز قالعان ءۇي',
    '🏠' => 'ءۇي سالۋ',
    '🏡' => 'باقشاسى بار ءۇي',
    '🏢' => 'كەڭسەلىك عيمارات',
    '🏣' => 'جاپوندىق پوشتا بولىمشەسى',
    '🏤' => 'پوشتا بولىمشەسى',
    '🏥' => 'اۋرۋحانا',
    '🏦' => 'بانك',
    '🏨' => 'قوناق ءۇي',
    '🏩' => 'ماحاببات ءۇيى',
    '🏪' => 'شاعىن ماركەت',
    '🏫' => 'مەكتەپ',
    '🏬' => 'امبەباپ دۇكەن',
    '🏭' => 'زاۋىت',
    '🏯' => 'جاپوندىق ساراي',
    '🏰' => 'ساراي',
    '💒' => 'توي',
    '🗼' => 'توكيو مۇناراسى',
    '🗽' => 'ازاتتىق ءمۇسىنى',
    '⛪' => 'شىركەۋ',
    '🕌' => 'مەشىت',
    '🛕' => 'ءۇندى عيباداتحاناسى',
    '🕍' => 'سيناگوگا',
    '⛩' => 'سينتويستىك حرام',
    '🕋' => 'قاعبا',
    '⛲' => 'بۇرقاق',
    '⛺' => 'شاتىر',
    '🌁' => 'تۇماندى',
    '🌃' => 'جۇلدىزدى ءتۇن',
    '🏙' => 'قالا كورىنىسى',
    '🌄' => 'تاۋلاردىڭ ارتىنان شىققان كۇن',
    '🌅' => 'كۇننىڭ شىعۋى',
    '🌆' => 'ىمىرتتاعى قالا كورىنىسى',
    '🌇' => 'كۇننىڭ باتۋى',
    '🌉' => 'كوپىردىڭ تۇنگى كورىنىسى',
    '♨' => 'گەيزەر',
    '🎠' => 'اتكەنشەكتەگى ات',
    '🛝' => 'توبەشىك',
    '🎡' => 'شولۋ دوڭگەلەگى',
    '🎢' => 'ويىن-ساۋىق پاركى',
    '💈' => 'شاشتاراز باعانى',
    '🎪' => 'تسيرك شاتىرى',
    '🚂' => 'لوكوموتيۆ',
    '🚃' => 'تەمىر جول ۆاگونى',
    '🚄' => 'ەكسپرەسس پويىزى',
    '🚅' => 'تۇمسىعى وق ءتارىزدى ەكسپرەسس پويىزى',
    '🚆' => 'پويىز',
    '🚇' => 'مەترو',
    '🚈' => 'تار تاباندى تەمىر جول',
    '🚉' => 'ستانتسيا',
    '🚊' => 'ترامۆاي',
    '🚝' => 'دارا رەلستى تەمىر جول',
    '🚞' => 'تاۋلى ايماقتاعى تەمىر جول',
    '🚋' => 'ترامۆاي ۆاگونى',
    '🚌' => 'اۆتوبۋس',
    '🚍' => 'كەلە جاتقان اۆتوبۋس',
    '🚎' => 'تروللەيبۋس',
    '🚐' => 'شاعىن اۆتوبۋس',
    '🚑' => 'جەدەل جاردەم',
    '🚒' => 'ءورت ءسوندىرۋ اۆتوكولىگى',
    '🚓' => 'ساقشى كولىگى',
    '🚔' => 'كەلە جاتقان ساقشى كولىگى',
    '🚕' => 'تاكسي',
    '🚖' => 'كەلە جاتقان تاكسي',
    '🚗' => 'اۆتوكولىك',
    '🚘' => 'كەلە جاتقان اۆتوكولىك',
    '🚙' => 'كروسسوۆەر',
    '🛻' => 'پيكاپ',
    '🚚' => 'جۇك كولىگى',
    '🚛' => 'اۆتوپويىز',
    '🚜' => 'تراكتور',
    '🏎' => 'جارىس كولىگى',
    '🏍' => 'موتوسيكل',
    '🛵' => 'سكۋتەر',
    '🦽' => 'قول جەتەگى بار ارباشا',
    '🦼' => 'ەلەكتر جەتەگى بار ارباشا',
    '🛺' => 'موتوريكشا',
    '🚲' => 'بەلسەبەت',
    '🛴' => 'ساموكات',
    '🛹' => 'سكەيتبورد',
    '🛼' => 'روليك',
    '🚏' => 'ايالداما',
    '🛣' => 'اۆتوجول',
    '🛤' => 'تەمىر جول رەلسى',
    '🛢' => 'مۇناي ونىمدەرىنە ارنالعان بوشكە',
    '⛽' => 'جانارماي سورعىسى',
    '🛞' => 'دوڭگەلەك',
    '🚨' => 'دابىل شامى',
    '🚥' => 'كولدەنەڭ باعدارشام',
    '🚦' => 'تىك باعدارشام',
    '🛑' => 'توقتا بەلگىسى',
    '🚧' => 'قۇرىلىس',
    '⚓' => 'زاكىر',
    '🛟' => 'قۇتقارۋ شەڭبەرى',
    '⛵' => 'جەلكەندى قايىق',
    '🛶' => 'كانوە',
    '🚤' => 'جىلدام جۇرەتىن قايىق',
    '🛳' => 'جولاۋشىلار كەمەسى',
    '⛴' => 'پاروم',
    '🛥' => 'موتورلى قايىق',
    '🚢' => 'كەمە',
    '✈' => 'ۇشۋ',
    '🛩' => 'شاعىن ۇشاق',
    '🛫' => 'ۇشاقتىڭ ۇشىپ كەتۋى',
    '🛬' => 'ۇشاقتىڭ قونۋى',
    '🪂' => 'پاراشۋت',
    '💺' => 'ورىندىق',
    '🚁' => 'تىك ۇشاق',
    '🚟' => 'اسپالى تەمىر جول',
    '🚠' => 'تاۋداعى ارقان جولى',
    '🚡' => 'اسپالى ارقان جولى',
    '🛰' => 'جەرسەرىك',
    '🚀' => 'زىمىران',
    '🛸' => 'ۇشاتىن تارەلكا',
    '🛎' => 'شابارمان قوڭىراۋى',
    '🧳' => 'جۇك',
    '⌛' => 'قۇمساعات',
    '⏳' => 'قۇمساعاتتان قۇم اعۋدا',
    '⌚' => 'قول ساعات',
    '⏰' => 'وياتقىش',
    '⏱' => 'سەكۋند ولشەۋىش',
    '⏲' => 'تايمەر',
    '🕰' => 'سورەگە قويىلاتىن ساعات',
    '🕛' => 'ساعات ون ەكى',
    '🕧' => 'ون ەكى جارىم',
    '🕐' => 'ساعات ءبىر',
    '🕜' => 'ءبىر جارىم',
    '🕑' => 'ساعات ەكى',
    '🕝' => 'ەكى جارىم',
    '🕒' => 'ساعات ءۇش',
    '🕞' => 'ءۇش جارىم',
    '🕓' => 'ساعات ءتورت',
    '🕟' => 'ءتورت جارىم',
    '🕔' => 'ساعات بەس',
    '🕠' => 'بەس جارىم',
    '🕕' => 'ساعات التى',
    '🕡' => 'التى جارىم',
    '🕖' => 'ساعات جەتى',
    '🕢' => 'جەتى جارىم',
    '🕗' => 'ساعات سەگىز',
    '🕣' => 'سەگىز جارىم',
    '🕘' => 'ساعات توعىز',
    '🕤' => 'توعىز جارىم',
    '🕙' => 'ساعات ون',
    '🕥' => 'ون جارىم',
    '🕚' => 'ساعات ون ءبىر',
    '🕦' => 'ون ءبىر جارىم',
    '🌑' => 'جاڭا اي',
    '🌒' => 'تولىپ جاتقان جارتى اي',
    '🌓' => 'ايدىڭ ءبىرىنشى شيرەگى',
    '🌔' => 'تولىپ جاتقان دوڭەس اي',
    '🌕' => 'تولعان اي',
    '🌖' => 'كىشىرەيىپ جاتقان دوڭەس اي',
    '🌗' => 'ايدىڭ سوڭعى شيرەگى',
    '🌘' => 'كىشىرەيىپ جاتقان جارتى اي',
    '🌙' => 'جارتى اي',
    '🌚' => 'جاڭا اي بەتى',
    '🌛' => 'بەتى بار ءبىرىنشى شيرەك ايى',
    '🌜' => 'بەتى بار سوڭعى شيرەك ايى',
    '🌡' => 'تەرمومەتر',
    '☀' => 'كۇن',
    '🌝' => 'تولعان اي بەتى',
    '🌞' => 'بەتى بەينەلەنگەن كۇن',
    '🪐' => 'ساقيناسى بار عالامشار',
    '⭐' => 'جۇلدىز',
    '🌟' => 'جارقىراپ تۇرعان جۇلدىز',
    '🌠' => 'ۇشقان جۇلدىز',
    '🌌' => 'قۇس جولى',
    '☁' => 'بۇلت',
    '⛅' => 'بۇلتتىڭ ارتىنداعى كۇن',
    '⛈' => 'نايزاعاي مەن جاڭبىر',
    '🌤' => 'كىشى بۇلتتىڭ ارتىنداعى كۇن',
    '🌥' => 'ۇلكەن بۇلتتىڭ ارتىنداعى كۇن',
    '🌦' => 'جاڭبىر جاۋىپ تۇرعان بۇلتتىڭ ارتىنداعى كۇن',
    '🌧' => 'جاڭبىر جاۋىپ تۇرعان بۇلت',
    '🌨' => 'قار جاۋىپ تۇرعان بۇلت',
    '🌩' => 'نايزاعاي سوعىپ تۇرعان بۇلت',
    '🌪' => 'تورنادو',
    '🌫' => 'تۇمان',
    '🌬' => 'بەت كورىنىسىندەگى جەل',
    '🌀' => 'تسيكلون',
    '🌈' => 'كەمپىرقوساق',
    '🌂' => 'جابىلعان قولشاتىر',
    '☂' => 'قولشاتىر',
    '☔' => 'جاڭبىر تامشىلارى بار قولشاتىر',
    '⛱' => 'قولشاتىر جەردە تۇر',
    '⚡' => 'جوعارى كەرنەۋ',
    '❄' => 'قار بۇرشىگى',
    '☃' => 'اققالا',
    '⛄' => 'قارسىز اققالا',
    '☄' => 'كومەتا',
    '🔥' => 'ءورت',
    '💧' => 'تامشى',
    '🌊' => 'تولقىن',
    '🎃' => 'اسقاباقتان جاسالعان شام',
    '🎄' => 'جاڭا جىل شىرشاسى',
    '🎆' => 'فەيەرۆەرك',
    '🎇' => 'بەنگال وتى',
    '🧨' => 'وتشاشۋ',
    '✨' => 'ۇشقىندار',
    '🎈' => 'شار',
    '🎉' => 'شارتىلداق',
    '🎊' => 'كونفەتتي شارى',
    '🎋' => 'تاناباتا اعاشى',
    '🎍' => 'قاراعاي دەكوراسياسى',
    '🎎' => 'جاپون قۋىرشاقتارى',
    '🎏' => 'تۇقى تاسپا',
    '🎐' => 'قىتاي قوڭىراۋى',
    '🎑' => 'سۋكۋمي',
    '🧧' => 'قىزىل كونۆەرت',
    '🎀' => 'تاسپا',
    '🎁' => 'ورالعان سىيلىق',
    '🎗' => 'ەسكە سالعىش تاسپاسى',
    '🎟' => 'قابىلداۋ بيلەتتەرى',
    '🎫' => 'بيلەت',
    '🎖' => 'اسكەري مەدال',
    '🏆' => 'ولجا',
    '🏅' => 'سپورتتىق مەدال',
    '🥇' => 'التىن مەدال',
    '🥈' => 'كۇمىس مەدال',
    '🥉' => 'قولا مەدال',
    '⚽' => 'فۋتبول دوبى',
    '⚾' => 'بەيسبول',
    '🥎' => 'سوفتبول',
    '🏀' => 'باسكەتبول',
    '🏐' => 'ۆولەيبول',
    '🏈' => 'امەريكالىق فۋتبول',
    '🏉' => 'رەگبي',
    '🎾' => 'تەننيس',
    '🥏' => 'ۇشاتىن ديسك',
    '🎳' => 'بوۋليڭ',
    '🏏' => 'كريكەت',
    '🏑' => 'ءشوپ ۇستىندەگى حوككەي',
    '🏒' => 'حوككەي',
    '🥍' => 'لاكروس',
    '🏓' => 'پيڭ-پوڭ',
    '🏸' => 'بادمينتون',
    '🥊' => 'بوكس قولعابى',
    '🥋' => 'جاۋىنگەرلىك ونەر فورماسى',
    '🥅' => 'قاقپا',
    '⛳' => 'گولىف',
    '⛸' => 'مۇز شاڭعى',
    '🎣' => 'قارماق',
    '🤿' => 'دايۆيڭگە ارنالعان ماسكا',
    '🎽' => 'جۇگىرۋگە ارنالعان جەيدەشە',
    '🎿' => 'شاڭعى',
    '🛷' => 'شانا',
    '🥌' => 'كەرليڭ تاسى',
    '🎯' => 'نىسانا ورتاسى',
    '🪀' => 'يو-يو',
    '🪁' => 'قاعاز باتپىراۋىق',
    '🔫' => 'تاپانشا',
    '🎱' => 'پۋل-8',
    '🔮' => 'سىيقىرلى شار',
    '🪄' => 'سيقىرلى تاياقشا',
    '🎮' => 'بەينە ويىن',
    '🕹' => 'جوستىك',
    '🎰' => 'ويىن اۆتوماتى',
    '🎲' => 'ويىن تەكشەسى',
    '🧩' => 'موزايكا',
    '🧸' => 'قىجىم قونجىق',
    '🪅' => 'پينياتا',
    '🪩' => 'اينالى شار',
    '🪆' => 'ماترەشكا',
    '♠' => 'قارعا',
    '♥' => 'جۇرەك',
    '♦' => 'قيىق',
    '♣' => 'شىبىندار',
    '♟' => 'پەشكا',
    '🃏' => 'جوكەر',
    '🀄' => 'ماجوڭ قىزىل ايداحار',
    '🎴' => 'گۇلدى ويىن كارتالارى',
    '🎭' => 'تەاتر ونەرى',
    '🖼' => 'جاقتاۋلى سۋرەت',
    '🎨' => 'پاليترا',
    '🧵' => 'ءجىپ',
    '🪡' => 'تىگىن ينە',
    '🧶' => 'ءيىرىمجىپ',
    '🪢' => 'ءتۇيىن',
    '👓' => 'كوزىلدىرىك',
    '🕶' => 'كۇننەن قورعايتىن كوزىلدىرىك',
    '🥽' => 'جازدىق كوزىلدىرىك',
    '🥼' => 'زەرتحانالىق كيىم',
    '🦺' => 'سيگنالدىق جيلەت',
    '👔' => 'گالستۋك',
    '👕' => 'جەيدەشە',
    '👖' => 'جينسا',
    '🧣' => 'شارىف',
    '🧤' => 'قولعاپ',
    '🧥' => 'پالتو',
    '🧦' => 'شۇلىق',
    '👗' => 'كويلەك',
    '👘' => 'كيمونو',
    '🥻' => 'ساري',
    '🩱' => 'جابىق شومىلۋ كيىمى',
    '🩲' => 'پلاۆكي',
    '🩳' => 'شولاق شالبار',
    '👙' => 'بيكيني',
    '👚' => 'ايەلدەر كيىمى',
    '🪭' => 'جەلپۋىش',
    '👛' => 'ايەلدەر ءاميانى',
    '👜' => 'قولدوربا',
    '👝' => 'سومكە',
    '🛍' => 'ساۋدا قالتالارى',
    '🎒' => 'پاپكا',
    '🩴' => 'ۆيەتنامدىق ساندال',
    '👞' => 'تۋفلي',
    '👟' => 'كروسوۆكى',
    '🥾' => 'تۋريستىك اياقكيىم',
    '🥿' => 'جايپاق تۋفلي',
    '👠' => 'بيىك وكشەلى تۋفلي',
    '👡' => 'ايەلدەر ساندالى',
    '🩰' => 'بالەتشىنىڭ اياق كيىمى',
    '👢' => 'ايەلدەر ەتىگى',
    '🪮' => 'تاراق',
    '👑' => 'ءتاج',
    '👒' => 'ايەلدەر قالپاعى',
    '🎩' => 'قوقىرايما',
    '🎓' => 'ءبىتىرۋ قالپاعى',
    '🧢' => 'كۇنقاعار كەپكا',
    '🪖' => 'اسكەري كاسكا',
    '⛑' => 'اق كرەست بەينەلەنگەن دۋلىعا',
    '📿' => 'تاسپيق',
    '💄' => 'دالاپ',
    '💍' => 'جۇزىك',
    '💎' => 'اسىل تاس',
    '🔇' => 'ديناميكتى ءوشىرۋ',
    '🔈' => 'ديناميك',
    '🔉' => 'ديناميكتى قوسۋ',
    '🔊' => 'ديناميكتىڭ دىبىسىن كوبەيتۋ',
    '📢' => 'داۋىس زورايتقىش',
    '📣' => 'مەگافون',
    '📯' => 'كەرنەي',
    '🔔' => 'قوڭىراۋ',
    '🔕' => 'قوڭىراۋ دىبىسى وشىرىلگەن',
    '🎼' => 'پارتيتۋرا',
    '🎵' => 'نوتا',
    '🎶' => 'نوتالار',
    '🎙' => 'ستۋديو ميكروفونى',
    '🎚' => 'دەڭگەي سىرعىتپاسى',
    '🎛' => 'باسقارۋ تۇيمەلەرى',
    '🎤' => 'ميكروفون',
    '🎧' => 'قۇلاققاپ',
    '📻' => 'راديو',
    '🎷' => 'ساكسوفون',
    '🎺' => 'ترۋبا',
    '🪊' => 'ترومبون',
    '🪗' => 'اككوردەون',
    '🎸' => 'گيتارا',
    '🎹' => 'مۋزىكالىق پەرنەتاقتا',
    '🎻' => 'سكريپكا',
    '🪕' => 'بانجو',
    '🥁' => 'بارابان',
    '🪘' => 'بارابان اسپابى',
    '🪇' => 'ماراكاس',
    '🪈' => 'سىبىزعى',
    '🪉' => 'ارفا',
    '📱' => 'ۇيالى تەلەفون',
    '📲' => 'ۇيالى تەلەفونعا قابىلداۋ',
    '☎' => 'تەلەفون',
    '📞' => 'رەسيۆەر',
    '📟' => 'پەيجەر',
    '📠' => 'فاكس ماشينا',
    '🔋' => 'باتارەيا',
    '🪫' => 'باتارەيا قۋاتى تومەن',
    '🔌' => 'تىعىن',
    '💻' => 'نوۋتبۋك',
    '🖥' => 'دەربەس كومپيۋتەر',
    '🖨' => 'پرينتەر',
    '⌨' => 'پەرنەتاقتا',
    '🖱' => 'ءتىنتۋىر',
    '🖲' => 'ترەكبول',
    '💽' => 'شاعىن ديسك',
    '💾' => 'المالى ديسك',
    '💿' => 'وپتيكالىق ديسك',
    '📀' => 'ديسك',
    '🧮' => 'اباكۋس',
    '🎥' => 'بەينەفيلم تۇسىرەتىن كامەرا',
    '🎞' => 'فيلم كادرلارى',
    '📽' => 'كينوپروەكتور',
    '🎬' => 'شارتىلداق نومىرلەۋىش',
    '📺' => 'تەلەديدار',
    '📷' => 'كامەرا',
    '📸' => 'جارقىلى بار كامەرا',
    '📹' => 'بەينەكامەرا',
    '📼' => 'بەينە كاسسەتا',
    '🔍' => 'سول جاقتى كورسەتەتىن ۇلكەيتكىش اينەك',
    '🔎' => 'وڭ جاقتى كورسەتەتىن ۇلكەيتكىش اينەك',
    '🕯' => 'مايشام',
    '💡' => 'لامپا',
    '🔦' => 'جارقىل جارىعى',
    '🏮' => 'قاعازدان جاسالعان قىزىل شام',
    '🪔' => 'ديا',
    '📔' => 'دەكوراتيۆتىك مۇقاباسى بار داپتەر',
    '📕' => 'جابىق كىتاپ',
    '📖' => 'اشىق كىتاپ',
    '📗' => 'جاسىل كىتاپ',
    '📘' => 'كوك كىتاپ',
    '📙' => 'سارعىش كىتاپ',
    '📚' => 'كىتاپتار',
    '📓' => 'داپتەر',
    '📒' => 'باستى كىتاپ',
    '📃' => 'ءبىر شەتى ورالعان بەت',
    '📜' => 'بۇراما قاعاز',
    '📄' => 'جوعارى قاراپ تۇرعان بەت',
    '📰' => 'گازەت',
    '🗞' => 'بۇكتەلگەن گازەت',
    '📑' => 'بەتبەلگىلەر قويىندىسى',
    '🔖' => 'بەتبەلگى',
    '🏷' => 'زاتبەلگى',
    '🪙' => 'تيىن',
    '💰' => 'اقشا سالىنعان قالتا',
    '🪎' => 'قازىنا ساندىعى',
    '💴' => 'يەنا بانكنوتاسى',
    '💵' => 'دوللار بانكنوتاسى',
    '💶' => 'ەۋرو بانكنوتاسى',
    '💷' => 'فۋنت بانكنوتاسى',
    '💸' => 'قاناتى بار اقشا',
    '💳' => 'نەسيە كارتاسى',
    '🧾' => 'تۇبىرتەك',
    '💹' => 'يەنا باعامى كوتەرىلگەنىن كورسەتەتىن دياگرامما',
    '✉' => 'حات قاپ',
    '📧' => 'ەلەكتروندىق پوشتا',
    '📨' => 'كەلگەن حات قاپ',
    '📩' => 'كورسەتكىشى بار حات',
    '📤' => 'شىعىس جاشىگى',
    '📥' => 'كىرىس جاشىگى',
    '📦' => 'قوراپ',
    '📫' => 'جالاۋشاسى كوتەرىلگەن جابىق پوشتا جاشىگى',
    '📪' => 'جالاۋشاسى تۇسىرىلگەن جابىق پوشتا جاشىگى',
    '📬' => 'جالاۋشاسى كوتەرىلگەن اشىق پوشتا جاشىگى',
    '📭' => 'جالاۋشاسى تۇسىرىلگەن اشىق پوشتا جاشىگى',
    '📮' => 'پوشتا جاشىگى',
    '🗳' => 'سايلاۋ بيۋللەتەنىن تاستايتىن جاشىك',
    '✏' => 'قارىنداش',
    '✒' => 'قارا قاۋىرسىن',
    '🖋' => 'اۆتوقالام',
    '🖊' => 'قالام',
    '🖌' => 'بوياۋ شەتكاسى',
    '🖍' => 'پاستەل',
    '📝' => 'جادناما',
    '💼' => 'پورتفەل',
    '📁' => 'فايل قالتاسى',
    '📂' => 'فايل قالتاسىن اشۋ',
    '🗂' => 'كارتا يندەكسىن بولگىشتەر',
    '📅' => 'كۇنتىزبە',
    '📆' => 'جىرتپالى كۇنتىزبە',
    '🗒' => 'سپيرالدى داپتەر',
    '🗓' => 'سپيرالدى كۇنتىزبە',
    '📇' => 'كارتا يندەكسى',
    '📈' => 'ءوسىم تۋرالى دياگرامما',
    '📉' => 'كەمۋ تۋرالى دياگرامما',
    '📊' => 'گيستوگرامما',
    '📋' => 'تاقتا',
    '📌' => 'كەڭسەلىك باتىرما',
    '📍' => 'باسى دوڭگەلەك باتىرما',
    '📎' => 'قاعاز تۇيرەۋىش',
    '🖇' => 'بايلانىسقان قاعاز قىسقىشتارى',
    '📏' => 'ءتۇزۋ سىزعىش',
    '📐' => 'ءۇشبۇرىشتى سىزعىش',
    '✂' => 'قايشى',
    '🗃' => 'كارتالىق فايلدارعا ارنالعان جاشىك',
    '🗄' => 'كارتوتەكالىق سورە',
    '🗑' => 'قوقىس سەبەتى',
    '🔒' => 'قۇلىپ',
    '🔓' => 'اشىق قۇلىپ',
    '🔏' => 'قالام مەن قۇلىپ',
    '🔐' => 'كىلتى بار جابىق قۇلىپ',
    '🔑' => 'كىلت',
    '🗝' => 'ەسكى كىلت',
    '🔨' => 'بالعا',
    '🪓' => 'بالتا',
    '⛏' => 'قايلا',
    '⚒' => 'بالعا مەن قايلا',
    '🛠' => 'بالعا مەن سومىن كىلت',
    '🗡' => 'قانجار',
    '⚔' => 'ايقاس قىلىشتار',
    '💣' => 'بومبا',
    '🪃' => 'بۋمەرانگ',
    '🏹' => 'جەبە مەن ساداق',
    '🛡' => 'قالقان',
    '🪚' => 'ارا',
    '🔧' => 'سومىن كىلت',
    '🪛' => 'بۇراعىش',
    '🔩' => 'سومىن مەن بۇراندا',
    '⚙' => 'تەگەرشىك',
    '🗜' => 'قىسۋ',
    '⚖' => 'يىنتىرەكتى تارازى',
    '🦯' => 'تاياق',
    '🔗' => 'شىنجىر',
    '⛓' => 'تىزبەك',
    '🪝' => 'ىلگەك',
    '🧰' => 'قۇرالدار جاشىگى',
    '🧲' => 'ماگنيت',
    '🪜' => 'ساتى',
    '🪏' => 'كۇرەك',
    '⚗' => 'ديستيللياتور',
    '🧪' => 'شىنى تۇتىك',
    '🧫' => 'پەتري ىدىسى',
    '🧬' => 'د ن ق',
    '🔬' => 'ميكروسكوپ',
    '🔭' => 'تەلەسكوپ',
    '📡' => 'سەرىكتىك انتەننا',
    '💉' => 'شپريس',
    '🩸' => 'قان تامشىسى',
    '💊' => 'ءدارى',
    '🩹' => 'ەمدەۋ',
    '🩼' => 'بالداق',
    '🩺' => 'ستەتوسكوپ',
    '🩻' => 'رەنگەن ساۋلەسى',
    '🚪' => 'ەسىك',
    '🛗' => 'ليفت',
    '🪞' => 'اينا',
    '🪟' => 'تەرەزە',
    '🛏' => 'كەرەۋەت',
    '🛋' => 'ديۆان مەن شام',
    '🪑' => 'ارقالىعى بار ورىندىق',
    '🚽' => 'ۋنيتاز',
    '🪠' => 'ۆانتۋز',
    '🚿' => 'سەبەزگى',
    '🛁' => 'ۆاننا',
    '🪤' => 'قاقپان',
    '🪒' => 'ۇستارا',
    '🧴' => 'سۋسابىن',
    '🧷' => 'تۇيرەۋىش',
    '🧹' => 'سىپىرعى',
    '🧺' => 'سەبەت',
    '🧻' => 'قاعاز وراماسى',
    '🪣' => 'شەلەك',
    '🧼' => 'سابىن',
    '🫧' => 'كوپىرشىكتەر',
    '🪥' => 'ءتىس شەتكاسى',
    '🧽' => 'گۋبكا',
    '🧯' => 'ءورت سوندىرگىش',
    '🛒' => 'ساۋدا ارباسى',
    '🚬' => 'تەمەكى',
    '⚰' => 'تابىت',
    '🪦' => 'ساعانا',
    '⚱' => 'ولىكتىڭ كۇلىن سالاتىن ساۋىت',
    '🧿' => 'بويتۇمار',
    '🪬' => 'حامسا',
    '🗿' => 'مواي',
    '🪧' => 'پلاكات',
    '🪪' => 'سايكەستەندىرۋ كارتاسى',
    '🏧' => 'بانكومات بەلگىسى',
    '🚮' => 'قوقىستى سەبەتكە تاستاۋ بەلگىسى',
    '🚰' => 'ىشەتىن سۋ',
    '♿' => 'مۇگەدەكتەر ارباشاسى',
    '🚹' => 'ەرلەر بولمەسى',
    '🚺' => 'ايەلدەر بولمەسى',
    '🚻' => 'اجەتحانا',
    '🚼' => 'سابيلەر',
    '🚾' => 'ۆاتەركلوزەت',
    '🛂' => 'قۇجاتتاردى تەكسەرۋ',
    '🛃' => 'كەدەن تەكسەرىسى',
    '🛄' => 'جولجۇكتى بەرۋ',
    '🛅' => 'ساقتاۋ كامەراسى',
    '⚠' => 'ەسكەرتۋ',
    '🚸' => 'بالالار كەسىپ وتەدى',
    '⛔' => 'جول جوق',
    '🚫' => 'تىيىم سالىنعان',
    '🚳' => 'بەلسەبەت تەبۋگە بولمايدى',
    '🚭' => 'شىلىم شەگۋگە بولمايدى',
    '🚯' => 'قوقىس تاستاۋعا تىيىم سالىنعان',
    '🚱' => 'ىشەتىن سۋ ەمەس',
    '🚷' => 'جاياۋ جۇرگىنشىلەرگە تىيىم سالىنعان',
    '📵' => 'ۇيالى تەلەفونداردى پايدالانۋعا بولمايدى',
    '🔞' => '18 جاستان كىشىلەرگە تىيىم سالىنعان',
    '☢' => 'راديواكتيۆتى ايماق',
    '☣' => 'بيولوگيالىق قاۋىپ',
    '⬆' => 'جوعارى قاراعان كورسەتكى',
    '↗' => 'جوعارعى وڭ جاق كورسەتكىشى',
    '➡' => 'وڭ جاق كورسەتكىشى',
    '↘' => 'تومەنگى وڭ جاق كورسەتكىشى',
    '⬇' => 'تومەن قاراعان كورسەتكى',
    '↙' => 'تومەنگى سول جاق كورسەتكىشى',
    '⬅' => 'سول جاق باعىتتى كورسەتكى',
    '↖' => 'جوعارعى سول جاق كورسەتكىشى',
    '↕' => 'جوعارى-تومەن كورسەتكىسى',
    '↔' => 'سول-وڭ كورسەتكىسى',
    '↩' => 'وڭ جاقتان شىعىپ، سول جاققا بۇرىلاتىن كورسەتكى',
    '↪' => 'سول جاقتان شىعىپ، وڭ جاققا بۇرىلاتىن كورسەتكى',
    '⤴' => 'وڭ جاقتان شىعىپ، جوعارىعا بۇرىلاتىن كورسەتكى',
    '⤵' => 'وڭ جاقتان شىعىپ، تومەنگە بۇرىلاتىن كورسەتكى',
    '🔃' => 'ساعات ءتىلى بويىنشا تىك كورسەتكىشتەر',
    '🔄' => 'ساعات تىلىنە قاراما-قارسى كورسەتكىلەر تۇيمەسى',
    '🔙' => '«ارتقا» كورسەتكىسى',
    '🔚' => '«سوڭى» كورسەتكىسى',
    '🔛' => '«قوسۋلى» كورسەتكىسى',
    '🔜' => '«جاقىندا» كورسەتكىسى',
    '🔝' => '«جوعارى» كورسەتكىشى',
    '🛐' => 'تابىنۋ ورنى',
    '⚛' => 'اتوم بەلگىسى',
    '🕉' => 'وم بەلگىسى',
    '✡' => 'ءداۋىد جۇلدىزى',
    '☸' => 'دحارما دوڭگەلەگى',
    '☯' => 'ين-يان بەلگىسى',
    '✝' => 'لاتىنشا كىرەس',
    '☦' => 'ورتودوكسالدىق كىرەس',
    '☪' => 'جۇلدىز بەن جارىم اي',
    '☮' => 'تىنىشتىق بەلگىسى',
    '🕎' => 'مەنورا',
    '🔯' => 'نۇكتەلى التى بۇرىشتى جۇلدىز',
    '🪯' => 'كحاندا',
    '♈' => 'توقتى بەلگىسى',
    '♉' => 'تورپاق بەلگىسى',
    '♊' => 'ەگىزدەر بەلگىسى',
    '♋' => 'شايان بەلگىسى',
    '♌' => 'ارىستان بەلگىسى',
    '♍' => 'بيكەش بەلگىسى',
    '♎' => 'تارازى بەلگىسى',
    '♏' => 'سارىشايان بەلگىسى',
    '♐' => 'مەرگەن بەلگىسى',
    '♑' => 'تاۋەشكى بەلگىسى',
    '♒' => 'سۋقۇيعىش بەلگىسى',
    '♓' => 'بالىقتار بەلگىسى',
    '⛎' => 'جىلان كوتەرۋشى بەلگىسى',
    '🔀' => 'كەزدەيسوق ترەكتەر تۇيمەسى',
    '🔁' => 'قايتالاۋ تۇيمەسى',
    '🔂' => 'ءبىر ترەكتى قايتالاۋ تۇيمەسى',
    '▶' => 'ويناتۋ تۇيمەسى',
    '⏩' => 'العا جىلدام وتكىزۋ تۇيمەسى',
    '⏭' => 'كەلەسى ترەك تۇيمەسى',
    '⏯' => 'ويناتۋ نە كىدىرتۋ تۇيمەسى',
    '◀' => 'كەرى قايتارۋ تۇيمەسى',
    '⏪' => 'ارتقا جىلدام قايتارۋ تۇيمەسى',
    '⏮' => 'سوڭعى ترەك تۇيمەسى',
    '🔼' => 'جوعارى تۇيمەسى',
    '⏫' => 'جوعارىعا جىلدام وتكىزۋ تۇيمەسى',
    '🔽' => 'تومەن تۇيمەسى',
    '⏬' => 'تومەنگە جىلدام وتكىزۋ تۇيمەسى',
    '⏸' => 'كىدىرتۋ تۇيمەسى',
    '⏹' => 'توقتاتۋ تۇيمەسى',
    '⏺' => 'جازىپ الۋ تۇيمەسۋ',
    '⏏' => 'شىعارىپ الۋ تۇيمەسى',
    '🎦' => 'كينو',
    '🔅' => 'كۇڭگىرتتەۋ تۇيمەسى',
    '🔆' => 'جارىقتاندىرۋ تۇيمەسى',
    '📶' => 'انتەننا تاياقشالارى',
    '🛜' => 'سىمسىز جەلى',
    '📳' => 'دىرىلدەۋ رەجيمى',
    '📴' => 'ۇيالى تەلەفوندى ءوشىرۋ',
    '♀' => 'ايەل تاڭباسى',
    '♂' => 'ەر تاڭباسى',
    '⚧' => 'ترانسگەندەرلىك تاڭبا',
    '✖' => 'كوبەيتۋ',
    '➕' => 'پليۋس',
    '➖' => 'مينۋس',
    '➗' => 'ءبولۋ',
    '🟰' => 'تەڭدىك بەلگىسى',
    '♾' => 'شەكسىزدىك',
    '‼' => 'قوس لەپ بەلگىسى',
    '⁉' => 'لەپ جانە سۇراق بەلگىسى',
    '❓' => 'قىزىل سۇراق بەلگىسى',
    '❔' => 'اق سۇراق بەلگىسى',
    '❕' => 'اق لەپ بەلگىسى',
    '❗' => 'لەپ بەلگىسى',
    '〰' => 'يرەك سىزىقشا',
    '💱' => 'ۆاليۋتا ايىرباستاۋ',
    '💲' => 'قالىڭ دوللار بەلگىسى',
    '⚕' => 'مەديسينا تاڭباسى',
    '♻' => 'قايتا وڭدەۋ بەلگىسى',
    '⚜' => 'فرانتسۋز ليلياسى',
    '🔱' => 'ءۇش اشالى ايىر بەلگىسى',
    '📛' => 'ات بەيجيگى',
    '🔰' => 'باستاۋشىلارعا ارنالعان جاپون بەلگىسى',
    '⭕' => 'قالىڭ، ۇلكەن شەڭبەر',
    '✅' => 'اق، قالىڭ قۇسبەلگى',
    '☑' => 'قۇسبەلگىسى بار داۋىس بەرۋ شارشىسى',
    '✔' => 'قالىڭ قۇسبەلگى',
    '❌' => 'قيىلىسقان سىزىقتار بەلگىسى',
    '❎' => 'قيىلىسقان سىزىقتار تۇيمەسى',
    '➰' => 'ىلمەك',
    '➿' => 'قوس ىلمەك',
    '〽' => 'بولىكتىڭ وزگەرۋ بەلگىسى',
    '✳' => 'سەگىز بۇرىشتى جۇلدىز',
    '✴' => 'سەگىز قىرلى جۇلدىز',
    '❇' => 'ۇشقىن',
    '™' => 'ساۋدا بەلگىسى',
    '🫟' => 'شاشىراۋ',
    '🔠' => 'ۇلكەن لاتىن ارىپتەرىن ەنگىزۋ',
    '🔡' => 'كىشى لاتىن ارىپتەرىن ەنگىزۋ',
    '🔢' => 'سانداردى ەنگىزۋ',
    '🔣' => 'بەلگىلەردى ەنگىزۋ',
    '🔤' => 'لاتىن ارىپتەرىن ەنگىزۋ',
    '🅰' => 'a تۇيمەسى (قىزىل ءتۇستى)',
    '🆎' => 'ab تۇيمەسى (قىزىل ءتۇستى)',
    '🅱' => 'b تۇيمەسى (قىزىل ءتۇستى)',
    '🆑' => 'cl تۇيمەسى',
    '🆒' => 'cool تۇيمەسى',
    '🆓' => 'free تۇيمەسى',
    'ℹ' => 'اقپارات كوزى',
    '🆔' => 'id تۇيمەسى',
    'Ⓜ' => 'شەڭبەردەگى m',
    '🆕' => 'new تۇيمەسى',
    '🆖' => 'ng تۇيمەسى',
    '🅾' => 'o تۇيمەسى (قىزىل ءتۇستى)',
    '🆗' => 'جاقسى تۇيمەسى',
    '🅿' => 'p تۇيمەسى',
    '🆘' => 'كومەك تۇيمەسى',
    '🆙' => 'UP! түймесі',
    '🆚' => 'قارسى تۇيمەسى',
    '🈁' => 'شارشىداعى «كاتاكانا كوكو»',
    '🈂' => 'شارشىداعى «كاتاكانا سا»',
    '🈷' => 'شارشىداعى «اي» تاڭباسى',
    '🈶' => 'شارشىداعى «تەگىن ەمەس» يەروگليفى',
    '🈯' => 'شارشىداعى «ساۋساق» يەروگليفى',
    '🉐' => 'دوڭعالاقتاعى «ارتىقشىلىق» يەروگليفى',
    '🈹' => 'شارشىداعى «ءبولۋ» يەروگليفى',
    '🈚' => 'شارشىداعى «باس تارتۋ» يەروگليفى',
    '🈲' => 'شارشىداعى «تىيىم سالۋ» يەروگليفى',
    '🉑' => 'دوڭعالاقتاعى «قابىلداۋ» يەروگليفى',
    '🈸' => 'شارشىداعى «قولدانۋ» يەروگليفى',
    '🈴' => 'شارشىداعى «بىرگە» يەروگليفى',
    '🈳' => 'شارشىداعى «بوس» يەروگليفى',
    '㊗' => 'دوڭعالاقتاعى «قۇتتىقتاۋ» يەروگليفى',
    '㊙' => 'دوڭعالاقتاعى «قۇپيا» يەروگليفى',
    '🈺' => 'شارشىداعى «جۇمىس» يەروگليفى',
    '🈵' => 'شارشىداعى «تولىقتىق» يەروگليفى',
    '🔴' => 'قىزىل دوڭگەلەك',
    '🟠' => 'قىزعىلت سارى دوڭگەلەك',
    '🟡' => 'سارى دوڭگەلەك',
    '🟢' => 'جاسىل دوڭگەلەك',
    '🔵' => 'كوك دوڭگەلەك',
    '🟣' => 'كۇلگىن دوڭگەلەك',
    '🟤' => 'قوڭىر دوڭگەلەك',
    '⚫' => 'قارا دوڭگەلەك',
    '⚪' => 'اق دوڭگەلەك',
    '🟥' => 'قىزىل شارشى',
    '🟧' => 'قىزعىلت سارى شارشى',
    '🟨' => 'سارى شارشى',
    '🟩' => 'جاسىل شارشى',
    '🟦' => 'كوك شارشى',
    '🟪' => 'كۇلگىن شارشى',
    '🟫' => 'قوڭىر شارشى',
    '⬛' => 'ۇلكەن قارا شارشى',
    '⬜' => 'ۇلكەن اق شارشى',
    '◼' => 'ورتاشا قارا شارشى',
    '◻' => 'ورتاشا اق شارشى',
    '◾' => 'ورتاشا-كىشى قارا شارشى',
    '◽' => 'ورتاشا-كىشى اق شارشى',
    '▪' => 'كىشى قارا شارشى',
    '▫' => 'كىشى اق شارشى',
    '🔶' => 'ۇلكەن سارعىش رومب',
    '🔷' => 'ۇلكەن كوك رومب',
    '🔸' => 'كىشى سارعىش رومب',
    '🔹' => 'كىشى كوك رومب',
    '🔺' => 'جوعارى قاراتىلعان كىشى ءۇشبۇرىش',
    '🔻' => 'تومەن قاراتىلعان كىشى ءۇشبۇرىش',
    '💠' => 'نۇكتەلى رومب',
    '🔘' => 'راديو تۇيمەسى',
    '🔳' => 'اق شارشى تۇيمەسى',
    '🔲' => 'قارا شارشى تۇيمەسى',
    '🏁' => 'تورلى جالاۋ',
    '🚩' => 'ءۇشبۇرىشتى جالاۋ',
    '🎌' => 'ايقاستىرىلعان جالاۋلار',
    '🏴' => 'قارا جالاۋ',
    '🏳' => 'اق جالاۋ',
];
