<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure e errët',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure e çelët',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'puthje: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👨🏻‍❤‍💋‍👨🏻' => 'puthje: burrë, burrë e nuancë lëkure e çelët',
    '👨🏻‍❤‍💋‍👨🏼' => 'puthje: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👨🏻‍❤‍💋‍👨🏽' => 'puthje: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👨🏻‍❤‍💋‍👨🏾' => 'puthje: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👨🏻‍❤‍💋‍👨🏿' => 'puthje: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure e errët',
    '👨🏼‍❤‍💋‍👨🏻' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👨🏼‍❤‍💋‍👨🏼' => 'puthje: burrë, burrë e nuancë lëkure gjysmë e çelët',
    '👨🏼‍❤‍💋‍👨🏽' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👨🏼‍❤‍💋‍👨🏾' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👨🏼‍❤‍💋‍👨🏿' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👨🏽‍❤‍💋‍👨🏻' => 'puthje: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👨🏽‍❤‍💋‍👨🏼' => 'puthje: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👨🏽‍❤‍💋‍👨🏽' => 'puthje: burrë, burrë e nuancë lëkure zeshkane',
    '👨🏽‍❤‍💋‍👨🏾' => 'puthje: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👨🏽‍❤‍💋‍👨🏿' => 'puthje: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👨🏾‍❤‍💋‍👨🏻' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👨🏾‍❤‍💋‍👨🏼' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👨🏾‍❤‍💋‍👨🏽' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👨🏾‍❤‍💋‍👨🏾' => 'puthje: burrë, burrë e nuancë lëkure gjysmë e errët',
    '👨🏾‍❤‍💋‍👨🏿' => 'puthje: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👨🏿‍❤‍💋‍👨🏻' => 'puthje: burrë, burrë, nuancë lëkure e errët e nuancë lëkure e çelët',
    '👨🏿‍❤‍💋‍👨🏼' => 'puthje: burrë, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👨🏿‍❤‍💋‍👨🏽' => 'puthje: burrë, burrë, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👨🏿‍❤‍💋‍👨🏾' => 'puthje: burrë, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👨🏿‍❤‍💋‍👨🏿' => 'puthje: burrë, burrë e nuancë lëkure e errët',
    '👩🏻‍❤‍💋‍👨🏻' => 'puthje: grua, burrë e nuancë lëkure e çelët',
    '👩🏻‍❤‍💋‍👨🏼' => 'puthje: grua, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍❤‍💋‍👨🏽' => 'puthje: grua, burrë, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍❤‍💋‍👨🏾' => 'puthje: grua, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍❤‍💋‍👨🏿' => 'puthje: grua, burrë, nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍❤‍💋‍👨🏻' => 'puthje: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍❤‍💋‍👨🏼' => 'puthje: grua, burrë e nuancë lëkure gjysmë e çelët',
    '👩🏼‍❤‍💋‍👨🏽' => 'puthje: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍❤‍💋‍👨🏾' => 'puthje: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍❤‍💋‍👨🏿' => 'puthje: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍❤‍💋‍👨🏻' => 'puthje: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍❤‍💋‍👨🏼' => 'puthje: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍❤‍💋‍👨🏽' => 'puthje: grua, burrë e nuancë lëkure zeshkane',
    '👩🏽‍❤‍💋‍👨🏾' => 'puthje: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍❤‍💋‍👨🏿' => 'puthje: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍❤‍💋‍👨🏻' => 'puthje: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍❤‍💋‍👨🏼' => 'puthje: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍❤‍💋‍👨🏽' => 'puthje: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍❤‍💋‍👨🏾' => 'puthje: grua, burrë e nuancë lëkure gjysmë e errët',
    '👩🏾‍❤‍💋‍👨🏿' => 'puthje: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍❤‍💋‍👨🏻' => 'puthje: grua, burrë, nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍❤‍💋‍👨🏼' => 'puthje: grua, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍❤‍💋‍👨🏽' => 'puthje: grua, burrë, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍❤‍💋‍👨🏾' => 'puthje: grua, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩🏿‍❤‍💋‍👨🏿' => 'puthje: grua, burrë e nuancë lëkure e errët',
    '👩🏻‍❤‍💋‍👩🏻' => 'puthje: grua, grua e nuancë lëkure e çelët',
    '👩🏻‍❤‍💋‍👩🏼' => 'puthje: grua, grua, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍❤‍💋‍👩🏽' => 'puthje: grua, grua, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍❤‍💋‍👩🏾' => 'puthje: grua, grua, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍❤‍💋‍👩🏿' => 'puthje: grua, grua, nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍❤‍💋‍👩🏻' => 'puthje: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍❤‍💋‍👩🏼' => 'puthje: grua, grua e nuancë lëkure gjysmë e çelët',
    '👩🏼‍❤‍💋‍👩🏽' => 'puthje: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍❤‍💋‍👩🏾' => 'puthje: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍❤‍💋‍👩🏿' => 'puthje: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍❤‍💋‍👩🏻' => 'puthje: grua, grua, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍❤‍💋‍👩🏼' => 'puthje: grua, grua, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍❤‍💋‍👩🏽' => 'puthje: grua, grua e nuancë lëkure zeshkane',
    '👩🏽‍❤‍💋‍👩🏾' => 'puthje: grua, grua, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍❤‍💋‍👩🏿' => 'puthje: grua, grua, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍❤‍💋‍👩🏻' => 'puthje: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍❤‍💋‍👩🏼' => 'puthje: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍❤‍💋‍👩🏽' => 'puthje: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍❤‍💋‍👩🏾' => 'puthje: grua, grua e nuancë lëkure gjysmë e errët',
    '👩🏾‍❤‍💋‍👩🏿' => 'puthje: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍❤‍💋‍👩🏻' => 'puthje: grua, grua, nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍❤‍💋‍👩🏼' => 'puthje: grua, grua, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍❤‍💋‍👩🏽' => 'puthje: grua, grua, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍❤‍💋‍👩🏾' => 'puthje: grua, grua, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩🏿‍❤‍💋‍👩🏿' => 'puthje: grua, grua e nuancë lëkure e errët',
    '🧑🏻‍🫯‍🧑🏼' => 'mundës: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '🧑🏻‍🫯‍🧑🏽' => 'mundës: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '🧑🏻‍🫯‍🧑🏾' => 'mundës: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍🫯‍🧑🏿' => 'mundës: nuancë lëkure e çelët e nuancë lëkure e errët',
    '🧑🏼‍🫯‍🧑🏻' => 'mundës: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '🧑🏼‍🫯‍🧑🏽' => 'mundës: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '🧑🏼‍🫯‍🧑🏾' => 'mundës: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏼‍🫯‍🧑🏿' => 'mundës: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '🧑🏽‍🫯‍🧑🏻' => 'mundës: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '🧑🏽‍🫯‍🧑🏼' => 'mundës: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🫯‍🧑🏾' => 'mundës: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '🧑🏽‍🫯‍🧑🏿' => 'mundës: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '🧑🏾‍🫯‍🧑🏻' => 'mundës: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '🧑🏾‍🫯‍🧑🏼' => 'mundës: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏾‍🫯‍🧑🏽' => 'mundës: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '🧑🏾‍🫯‍🧑🏿' => 'mundës: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '🧑🏿‍🫯‍🧑🏻' => 'mundës: nuancë lëkure e errët e nuancë lëkure e çelët',
    '🧑🏿‍🫯‍🧑🏼' => 'mundës: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏿‍🫯‍🧑🏽' => 'mundës: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '🧑🏿‍🫯‍🧑🏾' => 'mundës: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍❤‍🧑🏼' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '🧑🏻‍❤‍🧑🏽' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '🧑🏻‍❤‍🧑🏾' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍❤‍🧑🏿' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e çelët e nuancë lëkure e errët',
    '🧑🏼‍❤‍🧑🏻' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '🧑🏼‍❤‍🧑🏽' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '🧑🏼‍❤‍🧑🏾' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏼‍❤‍🧑🏿' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '🧑🏽‍❤‍🧑🏻' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '🧑🏽‍❤‍🧑🏼' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '🧑🏽‍❤‍🧑🏾' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '🧑🏽‍❤‍🧑🏿' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '🧑🏾‍❤‍🧑🏻' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '🧑🏾‍❤‍🧑🏼' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏾‍❤‍🧑🏽' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '🧑🏾‍❤‍🧑🏿' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '🧑🏿‍❤‍🧑🏻' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure e çelët',
    '🧑🏿‍❤‍🧑🏼' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏿‍❤‍🧑🏽' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '🧑🏿‍❤‍🧑🏾' => 'çift me zemër: fëmijë i rritur, fëmijë i rritur, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍🐰‍🧑🏼' => 'gra duke festuar: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '🧑🏻‍🐰‍🧑🏽' => 'gra duke festuar: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '🧑🏻‍🐰‍🧑🏾' => 'gra duke festuar: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍🐰‍🧑🏿' => 'gra duke festuar: nuancë lëkure e çelët e nuancë lëkure e errët',
    '🧑🏼‍🐰‍🧑🏻' => 'gra duke festuar: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '🧑🏼‍🐰‍🧑🏽' => 'gra duke festuar: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '🧑🏼‍🐰‍🧑🏾' => 'gra duke festuar: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏼‍🐰‍🧑🏿' => 'gra duke festuar: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '🧑🏽‍🐰‍🧑🏻' => 'gra duke festuar: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '🧑🏽‍🐰‍🧑🏼' => 'gra duke festuar: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🐰‍🧑🏾' => 'gra duke festuar: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '🧑🏽‍🐰‍🧑🏿' => 'gra duke festuar: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '🧑🏾‍🐰‍🧑🏻' => 'gra duke festuar: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '🧑🏾‍🐰‍🧑🏼' => 'gra duke festuar: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏾‍🐰‍🧑🏽' => 'gra duke festuar: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '🧑🏾‍🐰‍🧑🏿' => 'gra duke festuar: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '🧑🏿‍🐰‍🧑🏻' => 'gra duke festuar: nuancë lëkure e errët e nuancë lëkure e çelët',
    '🧑🏿‍🐰‍🧑🏼' => 'gra duke festuar: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏿‍🐰‍🧑🏽' => 'gra duke festuar: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '🧑🏿‍🐰‍🧑🏾' => 'gra duke festuar: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👨🏻‍🫯‍👨🏼' => 'burra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👨🏻‍🫯‍👨🏽' => 'burra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👨🏻‍🫯‍👨🏾' => 'burra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👨🏻‍🫯‍👨🏿' => 'burra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure e errët',
    '👨🏼‍🫯‍👨🏻' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👨🏼‍🫯‍👨🏽' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👨🏼‍🫯‍👨🏾' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👨🏼‍🫯‍👨🏿' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👨🏽‍🫯‍👨🏻' => 'burra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👨🏽‍🫯‍👨🏼' => 'burra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👨🏽‍🫯‍👨🏾' => 'burra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👨🏽‍🫯‍👨🏿' => 'burra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👨🏾‍🫯‍👨🏻' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👨🏾‍🫯‍👨🏼' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👨🏾‍🫯‍👨🏽' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👨🏾‍🫯‍👨🏿' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👨🏿‍🫯‍👨🏻' => 'burra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure e çelët',
    '👨🏿‍🫯‍👨🏼' => 'burra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👨🏿‍🫯‍👨🏽' => 'burra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👨🏿‍🫯‍👨🏾' => 'burra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👨🏻‍❤‍👨🏻' => 'çift me zemër: burrë, burrë e nuancë lëkure e çelët',
    '👨🏻‍❤‍👨🏼' => 'çift me zemër: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👨🏻‍❤‍👨🏽' => 'çift me zemër: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👨🏻‍❤‍👨🏾' => 'çift me zemër: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👨🏻‍❤‍👨🏿' => 'çift me zemër: burrë, burrë, nuancë lëkure e çelët e nuancë lëkure e errët',
    '👨🏼‍❤‍👨🏻' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👨🏼‍❤‍👨🏼' => 'çift me zemër: burrë, burrë e nuancë lëkure gjysmë e çelët',
    '👨🏼‍❤‍👨🏽' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👨🏼‍❤‍👨🏾' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👨🏼‍❤‍👨🏿' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👨🏽‍❤‍👨🏻' => 'çift me zemër: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👨🏽‍❤‍👨🏼' => 'çift me zemër: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👨🏽‍❤‍👨🏽' => 'çift me zemër: burrë, burrë e nuancë lëkure zeshkane',
    '👨🏽‍❤‍👨🏾' => 'çift me zemër: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👨🏽‍❤‍👨🏿' => 'çift me zemër: burrë, burrë, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👨🏾‍❤‍👨🏻' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👨🏾‍❤‍👨🏼' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👨🏾‍❤‍👨🏽' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👨🏾‍❤‍👨🏾' => 'çift me zemër: burrë, burrë e nuancë lëkure gjysmë e errët',
    '👨🏾‍❤‍👨🏿' => 'çift me zemër: burrë, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👨🏿‍❤‍👨🏻' => 'çift me zemër: burrë, burrë, nuancë lëkure e errët e nuancë lëkure e çelët',
    '👨🏿‍❤‍👨🏼' => 'çift me zemër: burrë, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👨🏿‍❤‍👨🏽' => 'çift me zemër: burrë, burrë, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👨🏿‍❤‍👨🏾' => 'çift me zemër: burrë, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👨🏿‍❤‍👨🏿' => 'çift me zemër: burrë, burrë e nuancë lëkure e errët',
    '👨🏻‍🐰‍👨🏼' => 'burra që festojnë: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👨🏻‍🐰‍👨🏽' => 'burra që festojnë: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👨🏻‍🐰‍👨🏾' => 'burra që festojnë: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👨🏻‍🐰‍👨🏿' => 'burra që festojnë: nuancë lëkure e çelët e nuancë lëkure e errët',
    '👨🏼‍🐰‍👨🏻' => 'burra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👨🏼‍🐰‍👨🏽' => 'burra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👨🏼‍🐰‍👨🏾' => 'burra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👨🏼‍🐰‍👨🏿' => 'burra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👨🏽‍🐰‍👨🏻' => 'burra që festojnë: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👨🏽‍🐰‍👨🏼' => 'burra që festojnë: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👨🏽‍🐰‍👨🏾' => 'burra që festojnë: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👨🏽‍🐰‍👨🏿' => 'burra që festojnë: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👨🏾‍🐰‍👨🏻' => 'burra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👨🏾‍🐰‍👨🏼' => 'burra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👨🏾‍🐰‍👨🏽' => 'burra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👨🏾‍🐰‍👨🏿' => 'burra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👨🏿‍🐰‍👨🏻' => 'burra që festojnë: nuancë lëkure e errët e nuancë lëkure e çelët',
    '👨🏿‍🐰‍👨🏼' => 'burra që festojnë: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👨🏿‍🐰‍👨🏽' => 'burra që festojnë: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👨🏿‍🐰‍👨🏾' => 'burra që festojnë: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩🏻‍🫯‍👩🏼' => 'gra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍🫯‍👩🏽' => 'gra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍🫯‍👩🏾' => 'gra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍🫯‍👩🏿' => 'gra që bëjnë mundje: nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍🫯‍👩🏻' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍🫯‍👩🏽' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍🫯‍👩🏾' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍🫯‍👩🏿' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍🫯‍👩🏻' => 'gra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍🫯‍👩🏼' => 'gra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍🫯‍👩🏾' => 'gra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍🫯‍👩🏿' => 'gra që bëjnë mundje: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍🫯‍👩🏻' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍🫯‍👩🏼' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍🫯‍👩🏽' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍🫯‍👩🏿' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍🫯‍👩🏻' => 'gra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍🫯‍👩🏼' => 'gra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍🫯‍👩🏽' => 'gra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍🫯‍👩🏾' => 'gra që bëjnë mundje: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩🏻‍❤‍👨🏻' => 'çift me zemër: grua, burrë e nuancë lëkure e çelët',
    '👩🏻‍❤‍👨🏼' => 'çift me zemër: grua, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍❤‍👨🏽' => 'çift me zemër: grua, burrë, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍❤‍👨🏾' => 'çift me zemër: grua, burrë, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍❤‍👨🏿' => 'çift me zemër: grua, burrë, nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍❤‍👨🏻' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍❤‍👨🏼' => 'çift me zemër: grua, burrë e nuancë lëkure gjysmë e çelët',
    '👩🏼‍❤‍👨🏽' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍❤‍👨🏾' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍❤‍👨🏿' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍❤‍👨🏻' => 'çift me zemër: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍❤‍👨🏼' => 'çift me zemër: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍❤‍👨🏽' => 'çift me zemër: grua, burrë e nuancë lëkure zeshkane',
    '👩🏽‍❤‍👨🏾' => 'çift me zemër: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍❤‍👨🏿' => 'çift me zemër: grua, burrë, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍❤‍👨🏻' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍❤‍👨🏼' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍❤‍👨🏽' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍❤‍👨🏾' => 'çift me zemër: grua, burrë e nuancë lëkure gjysmë e errët',
    '👩🏾‍❤‍👨🏿' => 'çift me zemër: grua, burrë, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍❤‍👨🏻' => 'çift me zemër: grua, burrë, nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍❤‍👨🏼' => 'çift me zemër: grua, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍❤‍👨🏽' => 'çift me zemër: grua, burrë, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍❤‍👨🏾' => 'çift me zemër: grua, burrë, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩🏿‍❤‍👨🏿' => 'çift me zemër: grua, burrë e nuancë lëkure e errët',
    '👩🏻‍❤‍👩🏻' => 'çift me zemër: grua, grua e nuancë lëkure e çelët',
    '👩🏻‍❤‍👩🏼' => 'çift me zemër: grua, grua, nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍❤‍👩🏽' => 'çift me zemër: grua, grua, nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍❤‍👩🏾' => 'çift me zemër: grua, grua, nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍❤‍👩🏿' => 'çift me zemër: grua, grua, nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍❤‍👩🏻' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍❤‍👩🏼' => 'çift me zemër: grua, grua e nuancë lëkure gjysmë e çelët',
    '👩🏼‍❤‍👩🏽' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍❤‍👩🏾' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍❤‍👩🏿' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍❤‍👩🏻' => 'çift me zemër: grua, grua, nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍❤‍👩🏼' => 'çift me zemër: grua, grua, nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍❤‍👩🏽' => 'çift me zemër: grua, grua e nuancë lëkure zeshkane',
    '👩🏽‍❤‍👩🏾' => 'çift me zemër: grua, grua, nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍❤‍👩🏿' => 'çift me zemër: grua, grua, nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍❤‍👩🏻' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍❤‍👩🏼' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍❤‍👩🏽' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍❤‍👩🏾' => 'çift me zemër: grua, grua e nuancë lëkure gjysmë e errët',
    '👩🏾‍❤‍👩🏿' => 'çift me zemër: grua, grua, nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍❤‍👩🏻' => 'çift me zemër: grua, grua, nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍❤‍👩🏼' => 'çift me zemër: grua, grua, nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍❤‍👩🏽' => 'çift me zemër: grua, grua, nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍❤‍👩🏾' => 'çift me zemër: grua, grua, nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩🏿‍❤‍👩🏿' => 'çift me zemër: grua, grua e nuancë lëkure e errët',
    '👩🏻‍🐰‍👩🏼' => 'gra që festojnë: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍🐰‍👩🏽' => 'gra që festojnë: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍🐰‍👩🏾' => 'gra që festojnë: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍🐰‍👩🏿' => 'gra që festojnë: nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍🐰‍👩🏻' => 'gra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍🐰‍👩🏽' => 'gra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍🐰‍👩🏾' => 'gra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍🐰‍👩🏿' => 'gra që festojnë: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍🐰‍👩🏻' => 'gra që festojnë: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍🐰‍👩🏼' => 'gra që festojnë: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍🐰‍👩🏾' => 'gra që festojnë: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍🐰‍👩🏿' => 'gra që festojnë: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍🐰‍👩🏻' => 'gra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍🐰‍👩🏼' => 'gra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍🐰‍👩🏽' => 'gra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍🐰‍👩🏿' => 'gra që festojnë: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍🐰‍👩🏻' => 'gra që festojnë: nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍🐰‍👩🏼' => 'gra që festojnë: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍🐰‍👩🏽' => 'gra që festojnë: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍🐰‍👩🏾' => 'gra që festojnë: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍🤝‍🧑🏻' => 'njerëz të kapur për duarsh: nuancë lëkure e çelët',
    '🧑🏻‍🤝‍🧑🏼' => 'njerëz të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '🧑🏻‍🤝‍🧑🏽' => 'njerëz të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '🧑🏻‍🤝‍🧑🏾' => 'njerëz të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏻‍🤝‍🧑🏿' => 'njerëz të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure e errët',
    '🧑🏼‍🤝‍🧑🏻' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '🧑🏼‍🤝‍🧑🏼' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e çelët',
    '🧑🏼‍🤝‍🧑🏽' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '🧑🏼‍🤝‍🧑🏾' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '🧑🏼‍🤝‍🧑🏿' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '🧑🏽‍🤝‍🧑🏻' => 'njerëz të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '🧑🏽‍🤝‍🧑🏼' => 'njerëz të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🤝‍🧑🏽' => 'njerëz të kapur për duarsh: nuancë lëkure zeshkane',
    '🧑🏽‍🤝‍🧑🏾' => 'njerëz të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '🧑🏽‍🤝‍🧑🏿' => 'njerëz të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '🧑🏾‍🤝‍🧑🏻' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '🧑🏾‍🤝‍🧑🏼' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏾‍🤝‍🧑🏽' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '🧑🏾‍🤝‍🧑🏾' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e errët',
    '🧑🏾‍🤝‍🧑🏿' => 'njerëz të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '🧑🏿‍🤝‍🧑🏻' => 'njerëz të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure e çelët',
    '🧑🏿‍🤝‍🧑🏼' => 'njerëz të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '🧑🏿‍🤝‍🧑🏽' => 'njerëz të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '🧑🏿‍🤝‍🧑🏾' => 'njerëz të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '🧑🏿‍🤝‍🧑🏿' => 'njerëz të kapur për duarsh: nuancë lëkure e errët',
    '👩🏻‍🤝‍👩🏼' => 'dy gra të kapura për duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍🤝‍👩🏽' => 'dy gra të kapura për duarsh: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍🤝‍👩🏾' => 'dy gra të kapura për duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍🤝‍👩🏿' => 'dy gra të kapura për duarsh: nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍🤝‍👩🏻' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍🤝‍👩🏽' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍🤝‍👩🏾' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍🤝‍👩🏿' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍🤝‍👩🏻' => 'dy gra të kapura për duarsh: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍🤝‍👩🏼' => 'dy gra të kapura për duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍🤝‍👩🏾' => 'dy gra të kapura për duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍🤝‍👩🏿' => 'dy gra të kapura për duarsh: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍🤝‍👩🏻' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍🤝‍👩🏼' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍🤝‍👩🏽' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍🤝‍👩🏿' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍🤝‍👩🏻' => 'dy gra të kapura për duarsh: nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍🤝‍👩🏼' => 'dy gra të kapura për duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍🤝‍👩🏽' => 'dy gra të kapura për duarsh: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍🤝‍👩🏾' => 'dy gra të kapura për duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩🏻‍🤝‍👨🏼' => 'burrë dhe grua që kapin duart: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👩🏻‍🤝‍👨🏽' => 'burrë dhe grua që kapin duart: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👩🏻‍🤝‍👨🏾' => 'burrë dhe grua që kapin duart: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👩🏻‍🤝‍👨🏿' => 'burrë dhe grua që kapin duart: nuancë lëkure e çelët e nuancë lëkure e errët',
    '👩🏼‍🤝‍👨🏻' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👩🏼‍🤝‍👨🏽' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👩🏼‍🤝‍👨🏾' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👩🏼‍🤝‍👨🏿' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👩🏽‍🤝‍👨🏻' => 'burrë dhe grua që kapin duart: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👩🏽‍🤝‍👨🏼' => 'burrë dhe grua që kapin duart: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👩🏽‍🤝‍👨🏾' => 'burrë dhe grua që kapin duart: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👩🏽‍🤝‍👨🏿' => 'burrë dhe grua që kapin duart: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👩🏾‍🤝‍👨🏻' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👩🏾‍🤝‍👨🏼' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👩🏾‍🤝‍👨🏽' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👩🏾‍🤝‍👨🏿' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👩🏿‍🤝‍👨🏻' => 'burrë dhe grua që kapin duart: nuancë lëkure e errët e nuancë lëkure e çelët',
    '👩🏿‍🤝‍👨🏼' => 'burrë dhe grua që kapin duart: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👩🏿‍🤝‍👨🏽' => 'burrë dhe grua që kapin duart: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👩🏿‍🤝‍👨🏾' => 'burrë dhe grua që kapin duart: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👨🏻‍🤝‍👨🏼' => 'dy burra të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '👨🏻‍🤝‍👨🏽' => 'dy burra të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '👨🏻‍🤝‍👨🏾' => 'dy burra të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '👨🏻‍🤝‍👨🏿' => 'dy burra të kapur për duarsh: nuancë lëkure e çelët e nuancë lëkure e errët',
    '👨🏼‍🤝‍👨🏻' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '👨🏼‍🤝‍👨🏽' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '👨🏼‍🤝‍👨🏾' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '👨🏼‍🤝‍👨🏿' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '👨🏽‍🤝‍👨🏻' => 'dy burra të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '👨🏽‍🤝‍👨🏼' => 'dy burra të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '👨🏽‍🤝‍👨🏾' => 'dy burra të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '👨🏽‍🤝‍👨🏿' => 'dy burra të kapur për duarsh: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '👨🏾‍🤝‍👨🏻' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '👨🏾‍🤝‍👨🏼' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '👨🏾‍🤝‍👨🏽' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '👨🏾‍🤝‍👨🏿' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '👨🏿‍🤝‍👨🏻' => 'dy burra të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure e çelët',
    '👨🏿‍🤝‍👨🏼' => 'dy burra të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '👨🏿‍🤝‍👨🏽' => 'dy burra të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '👨🏿‍🤝‍👨🏾' => 'dy burra të kapur për duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '👩‍❤‍💋‍👨' => 'puthje: grua e burrë',
    '👨‍❤‍💋‍👨' => 'puthje: burrë e burrë',
    '👩‍❤‍💋‍👩' => 'puthje: grua e grua',
    '👨‍👩‍👧‍👦' => 'familje: burrë, grua, vajzë e djalë',
    '👨‍👩‍👦‍👦' => 'familje: burrë, grua, djalë e djalë',
    '👨‍👩‍👧‍👧' => 'familje: burrë, grua, vajzë e vajzë',
    '👨‍👨‍👧‍👦' => 'familje: burrë, burrë, vajzë e djalë',
    '👨‍👨‍👦‍👦' => 'familje: burrë, burrë, djalë e djalë',
    '👨‍👨‍👧‍👧' => 'familje: burrë, burrë, vajzë e vajzë',
    '👩‍👩‍👧‍👦' => 'familje: grua, grua, vajzë e djalë',
    '👩‍👩‍👦‍👦' => 'familje: grua, grua, djalë e djalë',
    '👩‍👩‍👧‍👧' => 'familje: grua, grua, vajzë e vajzë',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'flamur: Angli',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'flamur: Skoci',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'flamur: Uells',
    '🧑‍🧑‍🧒‍🧒' => 'familje: i rritur, i rritur, fëmijë, fëmijë',
    '🚶🏻‍♀‍➡' => 'grua që ecën: nuancë lëkure e çelët e me drejtim djathtas',
    '🚶🏼‍♀‍➡' => 'grua që ecën: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🚶🏽‍♀‍➡' => 'grua që ecën: nuancë lëkure zeshkane e me drejtim djathtas',
    '🚶🏾‍♀‍➡' => 'grua që ecën: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🚶🏿‍♀‍➡' => 'grua që ecën: nuancë lëkure e errët e me drejtim djathtas',
    '🚶🏻‍♂‍➡' => 'burrë që ecën: nuancë lëkure e çelët e me drejtim djathtas',
    '🚶🏼‍♂‍➡' => 'burrë që ecën: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🚶🏽‍♂‍➡' => 'burrë që ecën: nuancë lëkure zeshkane e me drejtim djathtas',
    '🚶🏾‍♂‍➡' => 'burrë që ecën: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🚶🏿‍♂‍➡' => 'burrë që ecën: nuancë lëkure e errët e me drejtim djathtas',
    '🧎🏻‍♀‍➡' => 'grua në gjunjë: nuancë lëkure e çelët e me drejtim djathtas',
    '🧎🏼‍♀‍➡' => 'grua në gjunjë: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🧎🏽‍♀‍➡' => 'grua në gjunjë: nuancë lëkure zeshkane e me drejtim djathtas',
    '🧎🏾‍♀‍➡' => 'grua në gjunjë: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🧎🏿‍♀‍➡' => 'grua në gjunjë: nuancë lëkure e errët e me drejtim djathtas',
    '🧎🏻‍♂‍➡' => 'burrë në gjunjë: nuancë lëkure e çelët e me drejtim djathtas',
    '🧎🏼‍♂‍➡' => 'burrë në gjunjë: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🧎🏽‍♂‍➡' => 'burrë në gjunjë: nuancë lëkure zeshkane e me drejtim djathtas',
    '🧎🏾‍♂‍➡' => 'burrë në gjunjë: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🧎🏿‍♂‍➡' => 'burrë në gjunjë: nuancë lëkure e errët e me drejtim djathtas',
    '🧑🏻‍🦯‍➡' => 'person me shkop gjurmimi: nuancë lëkure e çelët e me drejtim djathtas',
    '🧑🏼‍🦯‍➡' => 'person me shkop gjurmimi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🧑🏽‍🦯‍➡' => 'person me shkop gjurmimi: nuancë lëkure zeshkane e me drejtim djathtas',
    '🧑🏾‍🦯‍➡' => 'person me shkop gjurmimi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🧑🏿‍🦯‍➡' => 'person me shkop gjurmimi: nuancë lëkure e errët e me drejtim djathtas',
    '👨🏻‍🦯‍➡' => 'burrë me shkop gjurmimi: nuancë lëkure e çelët e me drejtim djathtas',
    '👨🏼‍🦯‍➡' => 'burrë me shkop gjurmimi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '👨🏽‍🦯‍➡' => 'burrë me shkop gjurmimi: nuancë lëkure zeshkane e me drejtim djathtas',
    '👨🏾‍🦯‍➡' => 'burrë me shkop gjurmimi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '👨🏿‍🦯‍➡' => 'burrë me shkop gjurmimi: nuancë lëkure e errët e me drejtim djathtas',
    '👩🏻‍🦯‍➡' => 'grua me shkop gjurmimi: nuancë lëkure e çelët e me drejtim djathtas',
    '👩🏼‍🦯‍➡' => 'grua me shkop gjurmimi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '👩🏽‍🦯‍➡' => 'grua me shkop gjurmimi: nuancë lëkure zeshkane e me drejtim djathtas',
    '👩🏾‍🦯‍➡' => 'grua me shkop gjurmimi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '👩🏿‍🦯‍➡' => 'grua me shkop gjurmimi: nuancë lëkure e errët e me drejtim djathtas',
    '🧑🏻‍🦼‍➡' => 'person në karrocë të motorizuar invalidi: nuancë lëkure e çelët e me drejtim djathtas',
    '🧑🏼‍🦼‍➡' => 'person në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🧑🏽‍🦼‍➡' => 'person në karrocë të motorizuar invalidi: nuancë lëkure zeshkane e me drejtim djathtas',
    '🧑🏾‍🦼‍➡' => 'person në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🧑🏿‍🦼‍➡' => 'person në karrocë të motorizuar invalidi: nuancë lëkure e errët e me drejtim djathtas',
    '👨🏻‍🦼‍➡' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure e çelët e me drejtim djathtas',
    '👨🏼‍🦼‍➡' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '👨🏽‍🦼‍➡' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure zeshkane e me drejtim djathtas',
    '👨🏾‍🦼‍➡' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '👨🏿‍🦼‍➡' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure e errët e me drejtim djathtas',
    '👩🏻‍🦼‍➡' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure e çelët e me drejtim djathtas',
    '👩🏼‍🦼‍➡' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '👩🏽‍🦼‍➡' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure zeshkane e me drejtim djathtas',
    '👩🏾‍🦼‍➡' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '👩🏿‍🦼‍➡' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure e errët e me drejtim djathtas',
    '🧑🏻‍🦽‍➡' => 'person në karrocë invalidi manuale: nuancë lëkure e çelët e me drejtim djathtas',
    '🧑🏼‍🦽‍➡' => 'person në karrocë invalidi manuale: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🧑🏽‍🦽‍➡' => 'person në karrocë invalidi manuale: nuancë lëkure zeshkane e me drejtim djathtas',
    '🧑🏾‍🦽‍➡' => 'person në karrocë invalidi manuale: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🧑🏿‍🦽‍➡' => 'person në karrocë invalidi manuale: nuancë lëkure e errët e me drejtim djathtas',
    '👨🏻‍🦽‍➡' => 'burrë në karrocë manuale invalidi: nuancë lëkure e çelët e me drejtim djathtas',
    '👨🏼‍🦽‍➡' => 'burrë në karrocë manuale invalidi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '👨🏽‍🦽‍➡' => 'burrë në karrocë manuale invalidi: nuancë lëkure zeshkane e me drejtim djathtas',
    '👨🏾‍🦽‍➡' => 'burrë në karrocë manuale invalidi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '👨🏿‍🦽‍➡' => 'burrë në karrocë manuale invalidi: nuancë lëkure e errët e me drejtim djathtas',
    '👩🏻‍🦽‍➡' => 'grua në karrocë manuale invalidi: nuancë lëkure e çelët e me drejtim djathtas',
    '👩🏼‍🦽‍➡' => 'grua në karrocë manuale invalidi: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '👩🏽‍🦽‍➡' => 'grua në karrocë manuale invalidi: nuancë lëkure zeshkane e me drejtim djathtas',
    '👩🏾‍🦽‍➡' => 'grua në karrocë manuale invalidi: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '👩🏿‍🦽‍➡' => 'grua në karrocë manuale invalidi: nuancë lëkure e errët e me drejtim djathtas',
    '🏃🏻‍♀‍➡' => 'grua që vrapon: nuancë lëkure e çelët e me drejtim djathtas',
    '🏃🏼‍♀‍➡' => 'grua që vrapon: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🏃🏽‍♀‍➡' => 'grua që vrapon: nuancë lëkure zeshkane e me drejtim djathtas',
    '🏃🏾‍♀‍➡' => 'grua që vrapon: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🏃🏿‍♀‍➡' => 'grua që vrapon: nuancë lëkure e errët e me drejtim djathtas',
    '🏃🏻‍♂‍➡' => 'burrë që vrapon: nuancë lëkure e çelët e me drejtim djathtas',
    '🏃🏼‍♂‍➡' => 'burrë që vrapon: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🏃🏽‍♂‍➡' => 'burrë që vrapon: nuancë lëkure zeshkane e me drejtim djathtas',
    '🏃🏾‍♂‍➡' => 'burrë që vrapon: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🏃🏿‍♂‍➡' => 'burrë që vrapon: nuancë lëkure e errët e me drejtim djathtas',
    '🫱🏻‍🫲🏼' => 'shtrëngim duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e çelët',
    '🫱🏻‍🫲🏽' => 'shtrëngim duarsh: nuancë lëkure e çelët e nuancë lëkure zeshkane',
    '🫱🏻‍🫲🏾' => 'shtrëngim duarsh: nuancë lëkure e çelët e nuancë lëkure gjysmë e errët',
    '🫱🏻‍🫲🏿' => 'shtrëngim duarsh: nuancë lëkure e çelët e nuancë lëkure e errët',
    '🫱🏼‍🫲🏻' => 'shtrëngim duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e çelët',
    '🫱🏼‍🫲🏽' => 'shtrëngim duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure zeshkane',
    '🫱🏼‍🫲🏾' => 'shtrëngim duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure gjysmë e errët',
    '🫱🏼‍🫲🏿' => 'shtrëngim duarsh: nuancë lëkure gjysmë e çelët e nuancë lëkure e errët',
    '🫱🏽‍🫲🏻' => 'shtrëngim duarsh: nuancë lëkure zeshkane e nuancë lëkure e çelët',
    '🫱🏽‍🫲🏼' => 'shtrëngim duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e çelët',
    '🫱🏽‍🫲🏾' => 'shtrëngim duarsh: nuancë lëkure zeshkane e nuancë lëkure gjysmë e errët',
    '🫱🏽‍🫲🏿' => 'shtrëngim duarsh: nuancë lëkure zeshkane e nuancë lëkure e errët',
    '🫱🏾‍🫲🏻' => 'shtrëngim duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e çelët',
    '🫱🏾‍🫲🏼' => 'shtrëngim duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure gjysmë e çelët',
    '🫱🏾‍🫲🏽' => 'shtrëngim duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure zeshkane',
    '🫱🏾‍🫲🏿' => 'shtrëngim duarsh: nuancë lëkure gjysmë e errët e nuancë lëkure e errët',
    '🫱🏿‍🫲🏻' => 'shtrëngim duarsh: nuancë lëkure e errët e nuancë lëkure e çelët',
    '🫱🏿‍🫲🏼' => 'shtrëngim duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e çelët',
    '🫱🏿‍🫲🏽' => 'shtrëngim duarsh: nuancë lëkure e errët e nuancë lëkure zeshkane',
    '🫱🏿‍🫲🏾' => 'shtrëngim duarsh: nuancë lëkure e errët e nuancë lëkure gjysmë e errët',
    '🚶‍♀‍➡' => 'grua që ecën: me drejtim djathtas',
    '🚶‍♂‍➡' => 'burrë që ecën: me drejtim djathtas',
    '🧎‍♀‍➡' => 'grua në gjunjë: me drejtim djathtas',
    '🧎‍♂‍➡' => 'burrë në gjunjë: me drejtim djathtas',
    '🧑‍🦯‍➡' => 'person me shkop gjurmimi: me drejtim djathtas',
    '👨‍🦯‍➡' => 'burrë me shkop gjurmimi: me drejtim djathtas',
    '👩‍🦯‍➡' => 'grua me shkop gjurmimi: me drejtim djathtas',
    '🧑‍🦼‍➡' => 'person në karrocë të motorizuar invalidi: me drejtim djathtas',
    '👨‍🦼‍➡' => 'burrë në karrocë të motorizuar invalidi: me drejtim djathtas',
    '👩‍🦼‍➡' => 'grua në karrocë të motorizuar invalidi: me drejtim djathtas',
    '🧑‍🦽‍➡' => 'person në karrocë invalidi manuale: me drejtim djathtas',
    '👨‍🦽‍➡' => 'burrë në karrocë manuale invalidi: me drejtim djathtas',
    '👩‍🦽‍➡' => 'grua në karrocë manuale invalidi: me drejtim djathtas',
    '🏃‍♀‍➡' => 'grua që vrapon: me drejtim djathtas',
    '🏃‍♂‍➡' => 'burrë që vrapon: me drejtim djathtas',
    '👩‍❤‍👨' => 'çift me zemër: grua e burrë',
    '👨‍❤‍👨' => 'çift me zemër: burrë e burrë',
    '👩‍❤‍👩' => 'çift me zemër: grua e grua',
    '👨‍👩‍👦' => 'familje: burrë, grua e djalë',
    '👨‍👩‍👧' => 'familje: burrë, grua e vajzë',
    '👨‍👨‍👦' => 'familje: burrë, burrë e djalë',
    '👨‍👨‍👧' => 'familje: burrë, burrë e vajzë',
    '👩‍👩‍👦' => 'familje: grua, grua e djalë',
    '👩‍👩‍👧' => 'familje: grua, grua e vajzë',
    '👨‍👦‍👦' => 'familje: burrë, djalë e djalë',
    '👨‍👧‍👦' => 'familje: burrë, vajzë e djalë',
    '👨‍👧‍👧' => 'familje: burrë, vajzë e vajzë',
    '👩‍👦‍👦' => 'familje: grua, djalë e djalë',
    '👩‍👧‍👦' => 'familje: grua, vajzë e djalë',
    '👩‍👧‍👧' => 'familje: grua, vajzë e vajzë',
    '🧑‍🤝‍🧑' => 'njerëz të kapur për duarsh',
    '🧑‍🧑‍🧒' => 'familje: i rritur, i rritur, fëmijë',
    '🧑‍🧒‍🧒' => 'familje: i rritur, fëmijë, fëmijë',
    '🧑🏻‍🦰' => 'fëmijë i rritur: nuancë lëkure e çelët e flokë të kuqe',
    '🧑🏼‍🦰' => 'fëmijë i rritur: nuancë lëkure gjysmë e çelët e flokë të kuqe',
    '🧑🏽‍🦰' => 'fëmijë i rritur: nuancë lëkure zeshkane e flokë të kuqe',
    '🧑🏾‍🦰' => 'fëmijë i rritur: nuancë lëkure gjysmë e errët e flokë të kuqe',
    '🧑🏿‍🦰' => 'fëmijë i rritur: nuancë lëkure e errët e flokë të kuqe',
    '🧑🏻‍🦱' => 'fëmijë i rritur: nuancë lëkure e çelët e flokë kaçurrele',
    '🧑🏼‍🦱' => 'fëmijë i rritur: nuancë lëkure gjysmë e çelët e flokë kaçurrele',
    '🧑🏽‍🦱' => 'fëmijë i rritur: nuancë lëkure zeshkane e flokë kaçurrele',
    '🧑🏾‍🦱' => 'fëmijë i rritur: nuancë lëkure gjysmë e errët e flokë kaçurrele',
    '🧑🏿‍🦱' => 'fëmijë i rritur: nuancë lëkure e errët e flokë kaçurrele',
    '🧑🏻‍🦳' => 'fëmijë i rritur: nuancë lëkure e çelët e flokë të bardha',
    '🧑🏼‍🦳' => 'fëmijë i rritur: nuancë lëkure gjysmë e çelët e flokë të bardha',
    '🧑🏽‍🦳' => 'fëmijë i rritur: nuancë lëkure zeshkane e flokë të bardha',
    '🧑🏾‍🦳' => 'fëmijë i rritur: nuancë lëkure gjysmë e errët e flokë të bardha',
    '🧑🏿‍🦳' => 'fëmijë i rritur: nuancë lëkure e errët e flokë të bardha',
    '🧑🏻‍🦲' => 'fëmijë i rritur: nuancë lëkure e çelët e qeros',
    '🧑🏼‍🦲' => 'fëmijë i rritur: nuancë lëkure gjysmë e çelët e qeros',
    '🧑🏽‍🦲' => 'fëmijë i rritur: nuancë lëkure zeshkane e qeros',
    '🧑🏾‍🦲' => 'fëmijë i rritur: nuancë lëkure gjysmë e errët e qeros',
    '🧑🏿‍🦲' => 'fëmijë i rritur: nuancë lëkure e errët e qeros',
    '🧑🏻‍🩰' => 'balerin: nuancë lëkure e çelët',
    '🧑🏼‍🩰' => 'balerin: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🩰' => 'balerin: nuancë lëkure zeshkane',
    '🧑🏾‍🩰' => 'balerin: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🩰' => 'balerin: nuancë lëkure e errët',
    '🧔🏻‍♂' => 'burrë: nuancë lëkure e çelët e mjekër',
    '🧔🏼‍♂' => 'burrë: nuancë lëkure gjysmë e çelët e mjekër',
    '🧔🏽‍♂' => 'burrë: nuancë lëkure zeshkane e mjekër',
    '🧔🏾‍♂' => 'burrë: nuancë lëkure gjysmë e errët e mjekër',
    '🧔🏿‍♂' => 'burrë: nuancë lëkure e errët e mjekër',
    '👱🏻‍♂' => 'burrë biond: nuancë lëkure e çelët',
    '👱🏼‍♂' => 'burrë biond: nuancë lëkure gjysmë e çelët',
    '👱🏽‍♂' => 'burrë biond: nuancë lëkure zeshkane',
    '👱🏾‍♂' => 'burrë biond: nuancë lëkure gjysmë e errët',
    '👱🏿‍♂' => 'burrë biond: nuancë lëkure e errët',
    '👨🏻‍🦰' => 'burrë: nuancë lëkure e çelët e flokë të kuqe',
    '👨🏼‍🦰' => 'burrë: nuancë lëkure gjysmë e çelët e flokë të kuqe',
    '👨🏽‍🦰' => 'burrë: nuancë lëkure zeshkane e flokë të kuqe',
    '👨🏾‍🦰' => 'burrë: nuancë lëkure gjysmë e errët e flokë të kuqe',
    '👨🏿‍🦰' => 'burrë: nuancë lëkure e errët e flokë të kuqe',
    '👨🏻‍🦱' => 'burrë: nuancë lëkure e çelët e flokë kaçurrele',
    '👨🏼‍🦱' => 'burrë: nuancë lëkure gjysmë e çelët e flokë kaçurrele',
    '👨🏽‍🦱' => 'burrë: nuancë lëkure zeshkane e flokë kaçurrele',
    '👨🏾‍🦱' => 'burrë: nuancë lëkure gjysmë e errët e flokë kaçurrele',
    '👨🏿‍🦱' => 'burrë: nuancë lëkure e errët e flokë kaçurrele',
    '👨🏻‍🦳' => 'burrë: nuancë lëkure e çelët e flokë të bardha',
    '👨🏼‍🦳' => 'burrë: nuancë lëkure gjysmë e çelët e flokë të bardha',
    '👨🏽‍🦳' => 'burrë: nuancë lëkure zeshkane e flokë të bardha',
    '👨🏾‍🦳' => 'burrë: nuancë lëkure gjysmë e errët e flokë të bardha',
    '👨🏿‍🦳' => 'burrë: nuancë lëkure e errët e flokë të bardha',
    '👨🏻‍🦲' => 'burrë: nuancë lëkure e çelët e qeros',
    '👨🏼‍🦲' => 'burrë: nuancë lëkure gjysmë e çelët e qeros',
    '👨🏽‍🦲' => 'burrë: nuancë lëkure zeshkane e qeros',
    '👨🏾‍🦲' => 'burrë: nuancë lëkure gjysmë e errët e qeros',
    '👨🏿‍🦲' => 'burrë: nuancë lëkure e errët e qeros',
    '🧔🏻‍♀' => 'grua: nuancë lëkure e çelët e mjekër',
    '🧔🏼‍♀' => 'grua: nuancë lëkure gjysmë e çelët e mjekër',
    '🧔🏽‍♀' => 'grua: nuancë lëkure zeshkane e mjekër',
    '🧔🏾‍♀' => 'grua: nuancë lëkure gjysmë e errët e mjekër',
    '🧔🏿‍♀' => 'grua: nuancë lëkure e errët e mjekër',
    '👱🏻‍♀' => 'grua bionde: nuancë lëkure e çelët',
    '👱🏼‍♀' => 'grua bionde: nuancë lëkure gjysmë e çelët',
    '👱🏽‍♀' => 'grua bionde: nuancë lëkure zeshkane',
    '👱🏾‍♀' => 'grua bionde: nuancë lëkure gjysmë e errët',
    '👱🏿‍♀' => 'grua bionde: nuancë lëkure e errët',
    '👩🏻‍🦰' => 'grua: nuancë lëkure e çelët e flokë të kuqe',
    '👩🏼‍🦰' => 'grua: nuancë lëkure gjysmë e çelët e flokë të kuqe',
    '👩🏽‍🦰' => 'grua: nuancë lëkure zeshkane e flokë të kuqe',
    '👩🏾‍🦰' => 'grua: nuancë lëkure gjysmë e errët e flokë të kuqe',
    '👩🏿‍🦰' => 'grua: nuancë lëkure e errët e flokë të kuqe',
    '👩🏻‍🦱' => 'grua: nuancë lëkure e çelët e flokë kaçurrele',
    '👩🏼‍🦱' => 'grua: nuancë lëkure gjysmë e çelët e flokë kaçurrele',
    '👩🏽‍🦱' => 'grua: nuancë lëkure zeshkane e flokë kaçurrele',
    '👩🏾‍🦱' => 'grua: nuancë lëkure gjysmë e errët e flokë kaçurrele',
    '👩🏿‍🦱' => 'grua: nuancë lëkure e errët e flokë kaçurrele',
    '👩🏻‍🦳' => 'grua: nuancë lëkure e çelët e flokë të bardha',
    '👩🏼‍🦳' => 'grua: nuancë lëkure gjysmë e çelët e flokë të bardha',
    '👩🏽‍🦳' => 'grua: nuancë lëkure zeshkane e flokë të bardha',
    '👩🏾‍🦳' => 'grua: nuancë lëkure gjysmë e errët e flokë të bardha',
    '👩🏿‍🦳' => 'grua: nuancë lëkure e errët e flokë të bardha',
    '👩🏻‍🦲' => 'grua: nuancë lëkure e çelët e qeros',
    '👩🏼‍🦲' => 'grua: nuancë lëkure gjysmë e çelët e qeros',
    '👩🏽‍🦲' => 'grua: nuancë lëkure zeshkane e qeros',
    '👩🏾‍🦲' => 'grua: nuancë lëkure gjysmë e errët e qeros',
    '👩🏿‍🦲' => 'grua: nuancë lëkure e errët e qeros',
    '🙍🏻‍♂' => 'burrë i vrenjtur: nuancë lëkure e çelët',
    '🙍🏼‍♂' => 'burrë i vrenjtur: nuancë lëkure gjysmë e çelët',
    '🙍🏽‍♂' => 'burrë i vrenjtur: nuancë lëkure zeshkane',
    '🙍🏾‍♂' => 'burrë i vrenjtur: nuancë lëkure gjysmë e errët',
    '🙍🏿‍♂' => 'burrë i vrenjtur: nuancë lëkure e errët',
    '🙍🏻‍♀' => 'grua e vrenjtur: nuancë lëkure e çelët',
    '🙍🏼‍♀' => 'grua e vrenjtur: nuancë lëkure gjysmë e çelët',
    '🙍🏽‍♀' => 'grua e vrenjtur: nuancë lëkure zeshkane',
    '🙍🏾‍♀' => 'grua e vrenjtur: nuancë lëkure gjysmë e errët',
    '🙍🏿‍♀' => 'grua e vrenjtur: nuancë lëkure e errët',
    '🙎🏻‍♂' => 'burrë i pakënaqur: nuancë lëkure e çelët',
    '🙎🏼‍♂' => 'burrë i pakënaqur: nuancë lëkure gjysmë e çelët',
    '🙎🏽‍♂' => 'burrë i pakënaqur: nuancë lëkure zeshkane',
    '🙎🏾‍♂' => 'burrë i pakënaqur: nuancë lëkure gjysmë e errët',
    '🙎🏿‍♂' => 'burrë i pakënaqur: nuancë lëkure e errët',
    '🙎🏻‍♀' => 'grua e pakënaqur: nuancë lëkure e çelët',
    '🙎🏼‍♀' => 'grua e pakënaqur: nuancë lëkure gjysmë e çelët',
    '🙎🏽‍♀' => 'grua e pakënaqur: nuancë lëkure zeshkane',
    '🙎🏾‍♀' => 'grua e pakënaqur: nuancë lëkure gjysmë e errët',
    '🙎🏿‍♀' => 'grua e pakënaqur: nuancë lëkure e errët',
    '🙅🏻‍♂' => 'burrë që bën gjestin e ndaluar: nuancë lëkure e çelët',
    '🙅🏼‍♂' => 'burrë që bën gjestin e ndaluar: nuancë lëkure gjysmë e çelët',
    '🙅🏽‍♂' => 'burrë që bën gjestin e ndaluar: nuancë lëkure zeshkane',
    '🙅🏾‍♂' => 'burrë që bën gjestin e ndaluar: nuancë lëkure gjysmë e errët',
    '🙅🏿‍♂' => 'burrë që bën gjestin e ndaluar: nuancë lëkure e errët',
    '🙅🏻‍♀' => 'grua që bën gjestin e ndaluar: nuancë lëkure e çelët',
    '🙅🏼‍♀' => 'grua që bën gjestin e ndaluar: nuancë lëkure gjysmë e çelët',
    '🙅🏽‍♀' => 'grua që bën gjestin e ndaluar: nuancë lëkure zeshkane',
    '🙅🏾‍♀' => 'grua që bën gjestin e ndaluar: nuancë lëkure gjysmë e errët',
    '🙅🏿‍♀' => 'grua që bën gjestin e ndaluar: nuancë lëkure e errët',
    '🙆🏻‍♂' => 'burrë që bën gjestin në rregull: nuancë lëkure e çelët',
    '🙆🏼‍♂' => 'burrë që bën gjestin në rregull: nuancë lëkure gjysmë e çelët',
    '🙆🏽‍♂' => 'burrë që bën gjestin në rregull: nuancë lëkure zeshkane',
    '🙆🏾‍♂' => 'burrë që bën gjestin në rregull: nuancë lëkure gjysmë e errët',
    '🙆🏿‍♂' => 'burrë që bën gjestin në rregull: nuancë lëkure e errët',
    '🙆🏻‍♀' => 'grua që bën gjestin në rregull: nuancë lëkure e çelët',
    '🙆🏼‍♀' => 'grua që bën gjestin në rregull: nuancë lëkure gjysmë e çelët',
    '🙆🏽‍♀' => 'grua që bën gjestin në rregull: nuancë lëkure zeshkane',
    '🙆🏾‍♀' => 'grua që bën gjestin në rregull: nuancë lëkure gjysmë e errët',
    '🙆🏿‍♀' => 'grua që bën gjestin në rregull: nuancë lëkure e errët',
    '💁🏻‍♂' => 'burrë që tregon me dorë: nuancë lëkure e çelët',
    '💁🏼‍♂' => 'burrë që tregon me dorë: nuancë lëkure gjysmë e çelët',
    '💁🏽‍♂' => 'burrë që tregon me dorë: nuancë lëkure zeshkane',
    '💁🏾‍♂' => 'burrë që tregon me dorë: nuancë lëkure gjysmë e errët',
    '💁🏿‍♂' => 'burrë që tregon me dorë: nuancë lëkure e errët',
    '💁🏻‍♀' => 'grua që tregon me dorë: nuancë lëkure e çelët',
    '💁🏼‍♀' => 'grua që tregon me dorë: nuancë lëkure gjysmë e çelët',
    '💁🏽‍♀' => 'grua që tregon me dorë: nuancë lëkure zeshkane',
    '💁🏾‍♀' => 'grua që tregon me dorë: nuancë lëkure gjysmë e errët',
    '💁🏿‍♀' => 'grua që tregon me dorë: nuancë lëkure e errët',
    '🙋🏻‍♂' => 'burrë që ngre dorën: nuancë lëkure e çelët',
    '🙋🏼‍♂' => 'burrë që ngre dorën: nuancë lëkure gjysmë e çelët',
    '🙋🏽‍♂' => 'burrë që ngre dorën: nuancë lëkure zeshkane',
    '🙋🏾‍♂' => 'burrë që ngre dorën: nuancë lëkure gjysmë e errët',
    '🙋🏿‍♂' => 'burrë që ngre dorën: nuancë lëkure e errët',
    '🙋🏻‍♀' => 'grua që ngre dorën: nuancë lëkure e çelët',
    '🙋🏼‍♀' => 'grua që ngre dorën: nuancë lëkure gjysmë e çelët',
    '🙋🏽‍♀' => 'grua që ngre dorën: nuancë lëkure zeshkane',
    '🙋🏾‍♀' => 'grua që ngre dorën: nuancë lëkure gjysmë e errët',
    '🙋🏿‍♀' => 'grua që ngre dorën: nuancë lëkure e errët',
    '🧏🏻‍♂' => 'burrë i shurdhër: nuancë lëkure e çelët',
    '🧏🏼‍♂' => 'burrë i shurdhër: nuancë lëkure gjysmë e çelët',
    '🧏🏽‍♂' => 'burrë i shurdhër: nuancë lëkure zeshkane',
    '🧏🏾‍♂' => 'burrë i shurdhër: nuancë lëkure gjysmë e errët',
    '🧏🏿‍♂' => 'burrë i shurdhër: nuancë lëkure e errët',
    '🧏🏻‍♀' => 'grua e shurdhër: nuancë lëkure e çelët',
    '🧏🏼‍♀' => 'grua e shurdhër: nuancë lëkure gjysmë e çelët',
    '🧏🏽‍♀' => 'grua e shurdhër: nuancë lëkure zeshkane',
    '🧏🏾‍♀' => 'grua e shurdhër: nuancë lëkure gjysmë e errët',
    '🧏🏿‍♀' => 'grua e shurdhër: nuancë lëkure e errët',
    '🙇🏻‍♂' => 'burrë që përkulet për falje: nuancë lëkure e çelët',
    '🙇🏼‍♂' => 'burrë që përkulet për falje: nuancë lëkure gjysmë e çelët',
    '🙇🏽‍♂' => 'burrë që përkulet për falje: nuancë lëkure zeshkane',
    '🙇🏾‍♂' => 'burrë që përkulet për falje: nuancë lëkure gjysmë e errët',
    '🙇🏿‍♂' => 'burrë që përkulet për falje: nuancë lëkure e errët',
    '🙇🏻‍♀' => 'grua që përkulet për falje: nuancë lëkure e çelët',
    '🙇🏼‍♀' => 'grua që përkulet për falje: nuancë lëkure gjysmë e çelët',
    '🙇🏽‍♀' => 'grua që përkulet për falje: nuancë lëkure zeshkane',
    '🙇🏾‍♀' => 'grua që përkulet për falje: nuancë lëkure gjysmë e errët',
    '🙇🏿‍♀' => 'grua që përkulet për falje: nuancë lëkure e errët',
    '🤦🏻‍♂' => 'burrë që vë dorën në kokë: nuancë lëkure e çelët',
    '🤦🏼‍♂' => 'burrë që vë dorën në kokë: nuancë lëkure gjysmë e çelët',
    '🤦🏽‍♂' => 'burrë që vë dorën në kokë: nuancë lëkure zeshkane',
    '🤦🏾‍♂' => 'burrë që vë dorën në kokë: nuancë lëkure gjysmë e errët',
    '🤦🏿‍♂' => 'burrë që vë dorën në kokë: nuancë lëkure e errët',
    '🤦🏻‍♀' => 'grua që vë dorën në kokë: nuancë lëkure e çelët',
    '🤦🏼‍♀' => 'grua që vë dorën në kokë: nuancë lëkure gjysmë e çelët',
    '🤦🏽‍♀' => 'grua që vë dorën në kokë: nuancë lëkure zeshkane',
    '🤦🏾‍♀' => 'grua që vë dorën në kokë: nuancë lëkure gjysmë e errët',
    '🤦🏿‍♀' => 'grua që vë dorën në kokë: nuancë lëkure e errët',
    '🤷🏻‍♂' => 'burrë që ngre supet: nuancë lëkure e çelët',
    '🤷🏼‍♂' => 'burrë që ngre supet: nuancë lëkure gjysmë e çelët',
    '🤷🏽‍♂' => 'burrë që ngre supet: nuancë lëkure zeshkane',
    '🤷🏾‍♂' => 'burrë që ngre supet: nuancë lëkure gjysmë e errët',
    '🤷🏿‍♂' => 'burrë që ngre supet: nuancë lëkure e errët',
    '🤷🏻‍♀' => 'grua që ngre supet: nuancë lëkure e çelët',
    '🤷🏼‍♀' => 'grua që ngre supet: nuancë lëkure gjysmë e çelët',
    '🤷🏽‍♀' => 'grua që ngre supet: nuancë lëkure zeshkane',
    '🤷🏾‍♀' => 'grua që ngre supet: nuancë lëkure gjysmë e errët',
    '🤷🏿‍♀' => 'grua që ngre supet: nuancë lëkure e errët',
    '🧑🏻‍⚕' => 'person punonjës shëndetësor: nuancë lëkure e çelët',
    '🧑🏼‍⚕' => 'person punonjës shëndetësor: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍⚕' => 'person punonjës shëndetësor: nuancë lëkure zeshkane',
    '🧑🏾‍⚕' => 'person punonjës shëndetësor: nuancë lëkure gjysmë e errët',
    '🧑🏿‍⚕' => 'person punonjës shëndetësor: nuancë lëkure e errët',
    '👨🏻‍⚕' => 'punonjës shëndetësor: nuancë lëkure e çelët',
    '👨🏼‍⚕' => 'punonjës shëndetësor: nuancë lëkure gjysmë e çelët',
    '👨🏽‍⚕' => 'punonjës shëndetësor: nuancë lëkure zeshkane',
    '👨🏾‍⚕' => 'punonjës shëndetësor: nuancë lëkure gjysmë e errët',
    '👨🏿‍⚕' => 'punonjës shëndetësor: nuancë lëkure e errët',
    '👩🏻‍⚕' => 'punonjëse shëndetësor: nuancë lëkure e çelët',
    '👩🏼‍⚕' => 'punonjëse shëndetësor: nuancë lëkure gjysmë e çelët',
    '👩🏽‍⚕' => 'punonjëse shëndetësor: nuancë lëkure zeshkane',
    '👩🏾‍⚕' => 'punonjëse shëndetësor: nuancë lëkure gjysmë e errët',
    '👩🏿‍⚕' => 'punonjëse shëndetësor: nuancë lëkure e errët',
    '🧑🏻‍🎓' => 'person student: nuancë lëkure e çelët',
    '🧑🏼‍🎓' => 'person student: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🎓' => 'person student: nuancë lëkure zeshkane',
    '🧑🏾‍🎓' => 'person student: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🎓' => 'person student: nuancë lëkure e errët',
    '👨🏻‍🎓' => 'student: nuancë lëkure e çelët',
    '👨🏼‍🎓' => 'student: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🎓' => 'student: nuancë lëkure zeshkane',
    '👨🏾‍🎓' => 'student: nuancë lëkure gjysmë e errët',
    '👨🏿‍🎓' => 'student: nuancë lëkure e errët',
    '👩🏻‍🎓' => 'studente: nuancë lëkure e çelët',
    '👩🏼‍🎓' => 'studente: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🎓' => 'studente: nuancë lëkure zeshkane',
    '👩🏾‍🎓' => 'studente: nuancë lëkure gjysmë e errët',
    '👩🏿‍🎓' => 'studente: nuancë lëkure e errët',
    '🧑🏻‍🏫' => 'person mësues: nuancë lëkure e çelët',
    '🧑🏼‍🏫' => 'person mësues: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🏫' => 'person mësues: nuancë lëkure zeshkane',
    '🧑🏾‍🏫' => 'person mësues: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🏫' => 'person mësues: nuancë lëkure e errët',
    '👨🏻‍🏫' => 'mësues: nuancë lëkure e çelët',
    '👨🏼‍🏫' => 'mësues: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🏫' => 'mësues: nuancë lëkure zeshkane',
    '👨🏾‍🏫' => 'mësues: nuancë lëkure gjysmë e errët',
    '👨🏿‍🏫' => 'mësues: nuancë lëkure e errët',
    '👩🏻‍🏫' => 'mësuese: nuancë lëkure e çelët',
    '👩🏼‍🏫' => 'mësuese: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🏫' => 'mësuese: nuancë lëkure zeshkane',
    '👩🏾‍🏫' => 'mësuese: nuancë lëkure gjysmë e errët',
    '👩🏿‍🏫' => 'mësuese: nuancë lëkure e errët',
    '🧑🏻‍⚖' => 'person gjykatës: nuancë lëkure e çelët',
    '🧑🏼‍⚖' => 'person gjykatës: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍⚖' => 'person gjykatës: nuancë lëkure zeshkane',
    '🧑🏾‍⚖' => 'person gjykatës: nuancë lëkure gjysmë e errët',
    '🧑🏿‍⚖' => 'person gjykatës: nuancë lëkure e errët',
    '👨🏻‍⚖' => 'gjykatës: nuancë lëkure e çelët',
    '👨🏼‍⚖' => 'gjykatës: nuancë lëkure gjysmë e çelët',
    '👨🏽‍⚖' => 'gjykatës: nuancë lëkure zeshkane',
    '👨🏾‍⚖' => 'gjykatës: nuancë lëkure gjysmë e errët',
    '👨🏿‍⚖' => 'gjykatës: nuancë lëkure e errët',
    '👩🏻‍⚖' => 'gjykatëse: nuancë lëkure e çelët',
    '👩🏼‍⚖' => 'gjykatëse: nuancë lëkure gjysmë e çelët',
    '👩🏽‍⚖' => 'gjykatëse: nuancë lëkure zeshkane',
    '👩🏾‍⚖' => 'gjykatëse: nuancë lëkure gjysmë e errët',
    '👩🏿‍⚖' => 'gjykatëse: nuancë lëkure e errët',
    '🧑🏻‍🌾' => 'person bujk: nuancë lëkure e çelët',
    '🧑🏼‍🌾' => 'person bujk: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🌾' => 'person bujk: nuancë lëkure zeshkane',
    '🧑🏾‍🌾' => 'person bujk: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🌾' => 'person bujk: nuancë lëkure e errët',
    '👨🏻‍🌾' => 'bujk: nuancë lëkure e çelët',
    '👨🏼‍🌾' => 'bujk: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🌾' => 'bujk: nuancë lëkure zeshkane',
    '👨🏾‍🌾' => 'bujk: nuancë lëkure gjysmë e errët',
    '👨🏿‍🌾' => 'bujk: nuancë lëkure e errët',
    '👩🏻‍🌾' => 'bujkeshë: nuancë lëkure e çelët',
    '👩🏼‍🌾' => 'bujkeshë: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🌾' => 'bujkeshë: nuancë lëkure zeshkane',
    '👩🏾‍🌾' => 'bujkeshë: nuancë lëkure gjysmë e errët',
    '👩🏿‍🌾' => 'bujkeshë: nuancë lëkure e errët',
    '🧑🏻‍🍳' => 'person kuzhinier: nuancë lëkure e çelët',
    '🧑🏼‍🍳' => 'person kuzhinier: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🍳' => 'person kuzhinier: nuancë lëkure zeshkane',
    '🧑🏾‍🍳' => 'person kuzhinier: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🍳' => 'person kuzhinier: nuancë lëkure e errët',
    '👨🏻‍🍳' => 'kuzhinier: nuancë lëkure e çelët',
    '👨🏼‍🍳' => 'kuzhinier: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🍳' => 'kuzhinier: nuancë lëkure zeshkane',
    '👨🏾‍🍳' => 'kuzhinier: nuancë lëkure gjysmë e errët',
    '👨🏿‍🍳' => 'kuzhinier: nuancë lëkure e errët',
    '👩🏻‍🍳' => 'kuzhiniere: nuancë lëkure e çelët',
    '👩🏼‍🍳' => 'kuzhiniere: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🍳' => 'kuzhiniere: nuancë lëkure zeshkane',
    '👩🏾‍🍳' => 'kuzhiniere: nuancë lëkure gjysmë e errët',
    '👩🏿‍🍳' => 'kuzhiniere: nuancë lëkure e errët',
    '🧑🏻‍🔧' => 'person mekanik: nuancë lëkure e çelët',
    '🧑🏼‍🔧' => 'person mekanik: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🔧' => 'person mekanik: nuancë lëkure zeshkane',
    '🧑🏾‍🔧' => 'person mekanik: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🔧' => 'person mekanik: nuancë lëkure e errët',
    '👨🏻‍🔧' => 'mekanik: nuancë lëkure e çelët',
    '👨🏼‍🔧' => 'mekanik: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🔧' => 'mekanik: nuancë lëkure zeshkane',
    '👨🏾‍🔧' => 'mekanik: nuancë lëkure gjysmë e errët',
    '👨🏿‍🔧' => 'mekanik: nuancë lëkure e errët',
    '👩🏻‍🔧' => 'mekanike: nuancë lëkure e çelët',
    '👩🏼‍🔧' => 'mekanike: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🔧' => 'mekanike: nuancë lëkure zeshkane',
    '👩🏾‍🔧' => 'mekanike: nuancë lëkure gjysmë e errët',
    '👩🏿‍🔧' => 'mekanike: nuancë lëkure e errët',
    '🧑🏻‍🏭' => 'person punëtor fabrike: nuancë lëkure e çelët',
    '🧑🏼‍🏭' => 'person punëtor fabrike: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🏭' => 'person punëtor fabrike: nuancë lëkure zeshkane',
    '🧑🏾‍🏭' => 'person punëtor fabrike: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🏭' => 'person punëtor fabrike: nuancë lëkure e errët',
    '👨🏻‍🏭' => 'punëtor fabrike: nuancë lëkure e çelët',
    '👨🏼‍🏭' => 'punëtor fabrike: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🏭' => 'punëtor fabrike: nuancë lëkure zeshkane',
    '👨🏾‍🏭' => 'punëtor fabrike: nuancë lëkure gjysmë e errët',
    '👨🏿‍🏭' => 'punëtor fabrike: nuancë lëkure e errët',
    '👩🏻‍🏭' => 'punëtore fabrike: nuancë lëkure e çelët',
    '👩🏼‍🏭' => 'punëtore fabrike: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🏭' => 'punëtore fabrike: nuancë lëkure zeshkane',
    '👩🏾‍🏭' => 'punëtore fabrike: nuancë lëkure gjysmë e errët',
    '👩🏿‍🏭' => 'punëtore fabrike: nuancë lëkure e errët',
    '🧑🏻‍💼' => 'person punonjës zyre: nuancë lëkure e çelët',
    '🧑🏼‍💼' => 'person punonjës zyre: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍💼' => 'person punonjës zyre: nuancë lëkure zeshkane',
    '🧑🏾‍💼' => 'person punonjës zyre: nuancë lëkure gjysmë e errët',
    '🧑🏿‍💼' => 'person punonjës zyre: nuancë lëkure e errët',
    '👨🏻‍💼' => 'punonjës zyre: nuancë lëkure e çelët',
    '👨🏼‍💼' => 'punonjës zyre: nuancë lëkure gjysmë e çelët',
    '👨🏽‍💼' => 'punonjës zyre: nuancë lëkure zeshkane',
    '👨🏾‍💼' => 'punonjës zyre: nuancë lëkure gjysmë e errët',
    '👨🏿‍💼' => 'punonjës zyre: nuancë lëkure e errët',
    '👩🏻‍💼' => 'punonjëse zyre: nuancë lëkure e çelët',
    '👩🏼‍💼' => 'punonjëse zyre: nuancë lëkure gjysmë e çelët',
    '👩🏽‍💼' => 'punonjëse zyre: nuancë lëkure zeshkane',
    '👩🏾‍💼' => 'punonjëse zyre: nuancë lëkure gjysmë e errët',
    '👩🏿‍💼' => 'punonjëse zyre: nuancë lëkure e errët',
    '🧑🏻‍🔬' => 'person shkencëtar: nuancë lëkure e çelët',
    '🧑🏼‍🔬' => 'person shkencëtar: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🔬' => 'person shkencëtar: nuancë lëkure zeshkane',
    '🧑🏾‍🔬' => 'person shkencëtar: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🔬' => 'person shkencëtar: nuancë lëkure e errët',
    '👨🏻‍🔬' => 'shkencëtar: nuancë lëkure e çelët',
    '👨🏼‍🔬' => 'shkencëtar: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🔬' => 'shkencëtar: nuancë lëkure zeshkane',
    '👨🏾‍🔬' => 'shkencëtar: nuancë lëkure gjysmë e errët',
    '👨🏿‍🔬' => 'shkencëtar: nuancë lëkure e errët',
    '👩🏻‍🔬' => 'shkencëtare: nuancë lëkure e çelët',
    '👩🏼‍🔬' => 'shkencëtare: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🔬' => 'shkencëtare: nuancë lëkure zeshkane',
    '👩🏾‍🔬' => 'shkencëtare: nuancë lëkure gjysmë e errët',
    '👩🏿‍🔬' => 'shkencëtare: nuancë lëkure e errët',
    '🧑🏻‍💻' => 'person teknolog: nuancë lëkure e çelët',
    '🧑🏼‍💻' => 'person teknolog: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍💻' => 'person teknolog: nuancë lëkure zeshkane',
    '🧑🏾‍💻' => 'person teknolog: nuancë lëkure gjysmë e errët',
    '🧑🏿‍💻' => 'person teknolog: nuancë lëkure e errët',
    '👨🏻‍💻' => 'teknolog: nuancë lëkure e çelët',
    '👨🏼‍💻' => 'teknolog: nuancë lëkure gjysmë e çelët',
    '👨🏽‍💻' => 'teknolog: nuancë lëkure zeshkane',
    '👨🏾‍💻' => 'teknolog: nuancë lëkure gjysmë e errët',
    '👨🏿‍💻' => 'teknolog: nuancë lëkure e errët',
    '👩🏻‍💻' => 'teknologe: nuancë lëkure e çelët',
    '👩🏼‍💻' => 'teknologe: nuancë lëkure gjysmë e çelët',
    '👩🏽‍💻' => 'teknologe: nuancë lëkure zeshkane',
    '👩🏾‍💻' => 'teknologe: nuancë lëkure gjysmë e errët',
    '👩🏿‍💻' => 'teknologe: nuancë lëkure e errët',
    '🧑🏻‍🎤' => 'person këngëtar: nuancë lëkure e çelët',
    '🧑🏼‍🎤' => 'person këngëtar: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🎤' => 'person këngëtar: nuancë lëkure zeshkane',
    '🧑🏾‍🎤' => 'person këngëtar: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🎤' => 'person këngëtar: nuancë lëkure e errët',
    '👨🏻‍🎤' => 'këngëtar: nuancë lëkure e çelët',
    '👨🏼‍🎤' => 'këngëtar: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🎤' => 'këngëtar: nuancë lëkure zeshkane',
    '👨🏾‍🎤' => 'këngëtar: nuancë lëkure gjysmë e errët',
    '👨🏿‍🎤' => 'këngëtar: nuancë lëkure e errët',
    '👩🏻‍🎤' => 'këngëtare: nuancë lëkure e çelët',
    '👩🏼‍🎤' => 'këngëtare: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🎤' => 'këngëtare: nuancë lëkure zeshkane',
    '👩🏾‍🎤' => 'këngëtare: nuancë lëkure gjysmë e errët',
    '👩🏿‍🎤' => 'këngëtare: nuancë lëkure e errët',
    '🧑🏻‍🎨' => 'person artist: nuancë lëkure e çelët',
    '🧑🏼‍🎨' => 'person artist: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🎨' => 'person artist: nuancë lëkure zeshkane',
    '🧑🏾‍🎨' => 'person artist: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🎨' => 'person artist: nuancë lëkure e errët',
    '👨🏻‍🎨' => 'artist: nuancë lëkure e çelët',
    '👨🏼‍🎨' => 'artist: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🎨' => 'artist: nuancë lëkure zeshkane',
    '👨🏾‍🎨' => 'artist: nuancë lëkure gjysmë e errët',
    '👨🏿‍🎨' => 'artist: nuancë lëkure e errët',
    '👩🏻‍🎨' => 'artiste: nuancë lëkure e çelët',
    '👩🏼‍🎨' => 'artiste: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🎨' => 'artiste: nuancë lëkure zeshkane',
    '👩🏾‍🎨' => 'artiste: nuancë lëkure gjysmë e errët',
    '👩🏿‍🎨' => 'artiste: nuancë lëkure e errët',
    '🧑🏻‍✈' => 'person pilot: nuancë lëkure e çelët',
    '🧑🏼‍✈' => 'person pilot: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍✈' => 'person pilot: nuancë lëkure zeshkane',
    '🧑🏾‍✈' => 'person pilot: nuancë lëkure gjysmë e errët',
    '🧑🏿‍✈' => 'person pilot: nuancë lëkure e errët',
    '👨🏻‍✈' => 'pilot: nuancë lëkure e çelët',
    '👨🏼‍✈' => 'pilot: nuancë lëkure gjysmë e çelët',
    '👨🏽‍✈' => 'pilot: nuancë lëkure zeshkane',
    '👨🏾‍✈' => 'pilot: nuancë lëkure gjysmë e errët',
    '👨🏿‍✈' => 'pilot: nuancë lëkure e errët',
    '👩🏻‍✈' => 'pilote: nuancë lëkure e çelët',
    '👩🏼‍✈' => 'pilote: nuancë lëkure gjysmë e çelët',
    '👩🏽‍✈' => 'pilote: nuancë lëkure zeshkane',
    '👩🏾‍✈' => 'pilote: nuancë lëkure gjysmë e errët',
    '👩🏿‍✈' => 'pilote: nuancë lëkure e errët',
    '🧑🏻‍🚀' => 'person astronaut: nuancë lëkure e çelët',
    '🧑🏼‍🚀' => 'person astronaut: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🚀' => 'person astronaut: nuancë lëkure zeshkane',
    '🧑🏾‍🚀' => 'person astronaut: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🚀' => 'person astronaut: nuancë lëkure e errët',
    '👨🏻‍🚀' => 'astronaut: nuancë lëkure e çelët',
    '👨🏼‍🚀' => 'astronaut: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🚀' => 'astronaut: nuancë lëkure zeshkane',
    '👨🏾‍🚀' => 'astronaut: nuancë lëkure gjysmë e errët',
    '👨🏿‍🚀' => 'astronaut: nuancë lëkure e errët',
    '👩🏻‍🚀' => 'astronaute: nuancë lëkure e çelët',
    '👩🏼‍🚀' => 'astronaute: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🚀' => 'astronaute: nuancë lëkure zeshkane',
    '👩🏾‍🚀' => 'astronaute: nuancë lëkure gjysmë e errët',
    '👩🏿‍🚀' => 'astronaute: nuancë lëkure e errët',
    '🧑🏻‍🚒' => 'person zjarrfikës: nuancë lëkure e çelët',
    '🧑🏼‍🚒' => 'person zjarrfikës: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🚒' => 'person zjarrfikës: nuancë lëkure zeshkane',
    '🧑🏾‍🚒' => 'person zjarrfikës: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🚒' => 'person zjarrfikës: nuancë lëkure e errët',
    '👨🏻‍🚒' => 'zjarrfikës: nuancë lëkure e çelët',
    '👨🏼‍🚒' => 'zjarrfikës: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🚒' => 'zjarrfikës: nuancë lëkure zeshkane',
    '👨🏾‍🚒' => 'zjarrfikës: nuancë lëkure gjysmë e errët',
    '👨🏿‍🚒' => 'zjarrfikës: nuancë lëkure e errët',
    '👩🏻‍🚒' => 'grua zjarrëfikëse: nuancë lëkure e çelët',
    '👩🏼‍🚒' => 'grua zjarrëfikëse: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🚒' => 'grua zjarrëfikëse: nuancë lëkure zeshkane',
    '👩🏾‍🚒' => 'grua zjarrëfikëse: nuancë lëkure gjysmë e errët',
    '👩🏿‍🚒' => 'grua zjarrëfikëse: nuancë lëkure e errët',
    '👮🏻‍♂' => 'oficer policie: nuancë lëkure e çelët',
    '👮🏼‍♂' => 'oficer policie: nuancë lëkure gjysmë e çelët',
    '👮🏽‍♂' => 'oficer policie: nuancë lëkure zeshkane',
    '👮🏾‍♂' => 'oficer policie: nuancë lëkure gjysmë e errët',
    '👮🏿‍♂' => 'oficer policie: nuancë lëkure e errët',
    '👮🏻‍♀' => 'oficere policie: nuancë lëkure e çelët',
    '👮🏼‍♀' => 'oficere policie: nuancë lëkure gjysmë e çelët',
    '👮🏽‍♀' => 'oficere policie: nuancë lëkure zeshkane',
    '👮🏾‍♀' => 'oficere policie: nuancë lëkure gjysmë e errët',
    '👮🏿‍♀' => 'oficere policie: nuancë lëkure e errët',
    '🕵🏻‍♂' => 'hetues: nuancë lëkure e çelët',
    '🕵🏼‍♂' => 'hetues: nuancë lëkure gjysmë e çelët',
    '🕵🏽‍♂' => 'hetues: nuancë lëkure zeshkane',
    '🕵🏾‍♂' => 'hetues: nuancë lëkure gjysmë e errët',
    '🕵🏿‍♂' => 'hetues: nuancë lëkure e errët',
    '🕵🏻‍♀' => 'hetuese: nuancë lëkure e çelët',
    '🕵🏼‍♀' => 'hetuese: nuancë lëkure gjysmë e çelët',
    '🕵🏽‍♀' => 'hetuese: nuancë lëkure zeshkane',
    '🕵🏾‍♀' => 'hetuese: nuancë lëkure gjysmë e errët',
    '🕵🏿‍♀' => 'hetuese: nuancë lëkure e errët',
    '💂🏻‍♂' => 'gardist: nuancë lëkure e çelët',
    '💂🏼‍♂' => 'gardist: nuancë lëkure gjysmë e çelët',
    '💂🏽‍♂' => 'gardist: nuancë lëkure zeshkane',
    '💂🏾‍♂' => 'gardist: nuancë lëkure gjysmë e errët',
    '💂🏿‍♂' => 'gardist: nuancë lëkure e errët',
    '💂🏻‍♀' => 'ushtare garde: nuancë lëkure e çelët',
    '💂🏼‍♀' => 'ushtare garde: nuancë lëkure gjysmë e çelët',
    '💂🏽‍♀' => 'ushtare garde: nuancë lëkure zeshkane',
    '💂🏾‍♀' => 'ushtare garde: nuancë lëkure gjysmë e errët',
    '💂🏿‍♀' => 'ushtare garde: nuancë lëkure e errët',
    '👷🏻‍♂' => 'murator: nuancë lëkure e çelët',
    '👷🏼‍♂' => 'murator: nuancë lëkure gjysmë e çelët',
    '👷🏽‍♂' => 'murator: nuancë lëkure zeshkane',
    '👷🏾‍♂' => 'murator: nuancë lëkure gjysmë e errët',
    '👷🏿‍♂' => 'murator: nuancë lëkure e errët',
    '👷🏻‍♀' => 'muratore: nuancë lëkure e çelët',
    '👷🏼‍♀' => 'muratore: nuancë lëkure gjysmë e çelët',
    '👷🏽‍♀' => 'muratore: nuancë lëkure zeshkane',
    '👷🏾‍♀' => 'muratore: nuancë lëkure gjysmë e errët',
    '👷🏿‍♀' => 'muratore: nuancë lëkure e errët',
    '👳🏻‍♂' => 'djalë me çallmë: nuancë lëkure e çelët',
    '👳🏼‍♂' => 'djalë me çallmë: nuancë lëkure gjysmë e çelët',
    '👳🏽‍♂' => 'djalë me çallmë: nuancë lëkure zeshkane',
    '👳🏾‍♂' => 'djalë me çallmë: nuancë lëkure gjysmë e errët',
    '👳🏿‍♂' => 'djalë me çallmë: nuancë lëkure e errët',
    '👳🏻‍♀' => 'vajzë me çallmë: nuancë lëkure e çelët',
    '👳🏼‍♀' => 'vajzë me çallmë: nuancë lëkure gjysmë e çelët',
    '👳🏽‍♀' => 'vajzë me çallmë: nuancë lëkure zeshkane',
    '👳🏾‍♀' => 'vajzë me çallmë: nuancë lëkure gjysmë e errët',
    '👳🏿‍♀' => 'vajzë me çallmë: nuancë lëkure e errët',
    '🤵🏻‍♂' => 'burrë me kostum: nuancë lëkure e çelët',
    '🤵🏼‍♂' => 'burrë me kostum: nuancë lëkure gjysmë e çelët',
    '🤵🏽‍♂' => 'burrë me kostum: nuancë lëkure zeshkane',
    '🤵🏾‍♂' => 'burrë me kostum: nuancë lëkure gjysmë e errët',
    '🤵🏿‍♂' => 'burrë me kostum: nuancë lëkure e errët',
    '🤵🏻‍♀' => 'grua me kostum: nuancë lëkure e çelët',
    '🤵🏼‍♀' => 'grua me kostum: nuancë lëkure gjysmë e çelët',
    '🤵🏽‍♀' => 'grua me kostum: nuancë lëkure zeshkane',
    '🤵🏾‍♀' => 'grua me kostum: nuancë lëkure gjysmë e errët',
    '🤵🏿‍♀' => 'grua me kostum: nuancë lëkure e errët',
    '👰🏻‍♂' => 'burrë me vello: nuancë lëkure e çelët',
    '👰🏼‍♂' => 'burrë me vello: nuancë lëkure gjysmë e çelët',
    '👰🏽‍♂' => 'burrë me vello: nuancë lëkure zeshkane',
    '👰🏾‍♂' => 'burrë me vello: nuancë lëkure gjysmë e errët',
    '👰🏿‍♂' => 'burrë me vello: nuancë lëkure e errët',
    '👰🏻‍♀' => 'grua me vello: nuancë lëkure e çelët',
    '👰🏼‍♀' => 'grua me vello: nuancë lëkure gjysmë e çelët',
    '👰🏽‍♀' => 'grua me vello: nuancë lëkure zeshkane',
    '👰🏾‍♀' => 'grua me vello: nuancë lëkure gjysmë e errët',
    '👰🏿‍♀' => 'grua me vello: nuancë lëkure e errët',
    '👩🏻‍🍼' => 'grua që ushqen foshnjë: nuancë lëkure e çelët',
    '👩🏼‍🍼' => 'grua që ushqen foshnjë: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🍼' => 'grua që ushqen foshnjë: nuancë lëkure zeshkane',
    '👩🏾‍🍼' => 'grua që ushqen foshnjë: nuancë lëkure gjysmë e errët',
    '👩🏿‍🍼' => 'grua që ushqen foshnjë: nuancë lëkure e errët',
    '👨🏻‍🍼' => 'burrë që ushqen foshnjë: nuancë lëkure e çelët',
    '👨🏼‍🍼' => 'burrë që ushqen foshnjë: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🍼' => 'burrë që ushqen foshnjë: nuancë lëkure zeshkane',
    '👨🏾‍🍼' => 'burrë që ushqen foshnjë: nuancë lëkure gjysmë e errët',
    '👨🏿‍🍼' => 'burrë që ushqen foshnjë: nuancë lëkure e errët',
    '🧑🏻‍🍼' => 'person që ushqen foshnjë: nuancë lëkure e çelët',
    '🧑🏼‍🍼' => 'person që ushqen foshnjë: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🍼' => 'person që ushqen foshnjë: nuancë lëkure zeshkane',
    '🧑🏾‍🍼' => 'person që ushqen foshnjë: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🍼' => 'person që ushqen foshnjë: nuancë lëkure e errët',
    '🧑🏻‍🎄' => 'zx klaus: nuancë lëkure e çelët',
    '🧑🏼‍🎄' => 'zx klaus: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🎄' => 'zx klaus: nuancë lëkure zeshkane',
    '🧑🏾‍🎄' => 'zx klaus: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🎄' => 'zx klaus: nuancë lëkure e errët',
    '🦸🏻‍♂' => 'burrë superhero: nuancë lëkure e çelët',
    '🦸🏼‍♂' => 'burrë superhero: nuancë lëkure gjysmë e çelët',
    '🦸🏽‍♂' => 'burrë superhero: nuancë lëkure zeshkane',
    '🦸🏾‍♂' => 'burrë superhero: nuancë lëkure gjysmë e errët',
    '🦸🏿‍♂' => 'burrë superhero: nuancë lëkure e errët',
    '🦸🏻‍♀' => 'superheroinë: nuancë lëkure e çelët',
    '🦸🏼‍♀' => 'superheroinë: nuancë lëkure gjysmë e çelët',
    '🦸🏽‍♀' => 'superheroinë: nuancë lëkure zeshkane',
    '🦸🏾‍♀' => 'superheroinë: nuancë lëkure gjysmë e errët',
    '🦸🏿‍♀' => 'superheroinë: nuancë lëkure e errët',
    '🦹🏻‍♂' => 'burrë superzuzar: nuancë lëkure e çelët',
    '🦹🏼‍♂' => 'burrë superzuzar: nuancë lëkure gjysmë e çelët',
    '🦹🏽‍♂' => 'burrë superzuzar: nuancë lëkure zeshkane',
    '🦹🏾‍♂' => 'burrë superzuzar: nuancë lëkure gjysmë e errët',
    '🦹🏿‍♂' => 'burrë superzuzar: nuancë lëkure e errët',
    '🦹🏻‍♀' => 'superzuzare: nuancë lëkure e çelët',
    '🦹🏼‍♀' => 'superzuzare: nuancë lëkure gjysmë e çelët',
    '🦹🏽‍♀' => 'superzuzare: nuancë lëkure zeshkane',
    '🦹🏾‍♀' => 'superzuzare: nuancë lëkure gjysmë e errët',
    '🦹🏿‍♀' => 'superzuzare: nuancë lëkure e errët',
    '🧙🏻‍♂' => 'burrë magjistar: nuancë lëkure e çelët',
    '🧙🏼‍♂' => 'burrë magjistar: nuancë lëkure gjysmë e çelët',
    '🧙🏽‍♂' => 'burrë magjistar: nuancë lëkure zeshkane',
    '🧙🏾‍♂' => 'burrë magjistar: nuancë lëkure gjysmë e errët',
    '🧙🏿‍♂' => 'burrë magjistar: nuancë lëkure e errët',
    '🧙🏻‍♀' => 'magjistare: nuancë lëkure e çelët',
    '🧙🏼‍♀' => 'magjistare: nuancë lëkure gjysmë e çelët',
    '🧙🏽‍♀' => 'magjistare: nuancë lëkure zeshkane',
    '🧙🏾‍♀' => 'magjistare: nuancë lëkure gjysmë e errët',
    '🧙🏿‍♀' => 'magjistare: nuancë lëkure e errët',
    '🧚🏻‍♂' => 'burrë zanë: nuancë lëkure e çelët',
    '🧚🏼‍♂' => 'burrë zanë: nuancë lëkure gjysmë e çelët',
    '🧚🏽‍♂' => 'burrë zanë: nuancë lëkure zeshkane',
    '🧚🏾‍♂' => 'burrë zanë: nuancë lëkure gjysmë e errët',
    '🧚🏿‍♂' => 'burrë zanë: nuancë lëkure e errët',
    '🧚🏻‍♀' => 'grua zanë: nuancë lëkure e çelët',
    '🧚🏼‍♀' => 'grua zanë: nuancë lëkure gjysmë e çelët',
    '🧚🏽‍♀' => 'grua zanë: nuancë lëkure zeshkane',
    '🧚🏾‍♀' => 'grua zanë: nuancë lëkure gjysmë e errët',
    '🧚🏿‍♀' => 'grua zanë: nuancë lëkure e errët',
    '🧛🏻‍♂' => 'burrë vampir: nuancë lëkure e çelët',
    '🧛🏼‍♂' => 'burrë vampir: nuancë lëkure gjysmë e çelët',
    '🧛🏽‍♂' => 'burrë vampir: nuancë lëkure zeshkane',
    '🧛🏾‍♂' => 'burrë vampir: nuancë lëkure gjysmë e errët',
    '🧛🏿‍♂' => 'burrë vampir: nuancë lëkure e errët',
    '🧛🏻‍♀' => 'grua vampir: nuancë lëkure e çelët',
    '🧛🏼‍♀' => 'grua vampir: nuancë lëkure gjysmë e çelët',
    '🧛🏽‍♀' => 'grua vampir: nuancë lëkure zeshkane',
    '🧛🏾‍♀' => 'grua vampir: nuancë lëkure gjysmë e errët',
    '🧛🏿‍♀' => 'grua vampir: nuancë lëkure e errët',
    '🧜🏻‍♂' => 'burrë sirenë: nuancë lëkure e çelët',
    '🧜🏼‍♂' => 'burrë sirenë: nuancë lëkure gjysmë e çelët',
    '🧜🏽‍♂' => 'burrë sirenë: nuancë lëkure zeshkane',
    '🧜🏾‍♂' => 'burrë sirenë: nuancë lëkure gjysmë e errët',
    '🧜🏿‍♂' => 'burrë sirenë: nuancë lëkure e errët',
    '🧜🏻‍♀' => 'grua sirenë: nuancë lëkure e çelët',
    '🧜🏼‍♀' => 'grua sirenë: nuancë lëkure gjysmë e çelët',
    '🧜🏽‍♀' => 'grua sirenë: nuancë lëkure zeshkane',
    '🧜🏾‍♀' => 'grua sirenë: nuancë lëkure gjysmë e errët',
    '🧜🏿‍♀' => 'grua sirenë: nuancë lëkure e errët',
    '🧝🏻‍♂' => 'burrë kukudh: nuancë lëkure e çelët',
    '🧝🏼‍♂' => 'burrë kukudh: nuancë lëkure gjysmë e çelët',
    '🧝🏽‍♂' => 'burrë kukudh: nuancë lëkure zeshkane',
    '🧝🏾‍♂' => 'burrë kukudh: nuancë lëkure gjysmë e errët',
    '🧝🏿‍♂' => 'burrë kukudh: nuancë lëkure e errët',
    '🧝🏻‍♀' => 'grua kukudh: nuancë lëkure e çelët',
    '🧝🏼‍♀' => 'grua kukudh: nuancë lëkure gjysmë e çelët',
    '🧝🏽‍♀' => 'grua kukudh: nuancë lëkure zeshkane',
    '🧝🏾‍♀' => 'grua kukudh: nuancë lëkure gjysmë e errët',
    '🧝🏿‍♀' => 'grua kukudh: nuancë lëkure e errët',
    '💆🏻‍♂' => 'burrë që i bëhet masazh fytyre: nuancë lëkure e çelët',
    '💆🏼‍♂' => 'burrë që i bëhet masazh fytyre: nuancë lëkure gjysmë e çelët',
    '💆🏽‍♂' => 'burrë që i bëhet masazh fytyre: nuancë lëkure zeshkane',
    '💆🏾‍♂' => 'burrë që i bëhet masazh fytyre: nuancë lëkure gjysmë e errët',
    '💆🏿‍♂' => 'burrë që i bëhet masazh fytyre: nuancë lëkure e errët',
    '💆🏻‍♀' => 'grua që i bëhet masazh fytyre: nuancë lëkure e çelët',
    '💆🏼‍♀' => 'grua që i bëhet masazh fytyre: nuancë lëkure gjysmë e çelët',
    '💆🏽‍♀' => 'grua që i bëhet masazh fytyre: nuancë lëkure zeshkane',
    '💆🏾‍♀' => 'grua që i bëhet masazh fytyre: nuancë lëkure gjysmë e errët',
    '💆🏿‍♀' => 'grua që i bëhet masazh fytyre: nuancë lëkure e errët',
    '💇🏻‍♂' => 'burrë që i priten flokët: nuancë lëkure e çelët',
    '💇🏼‍♂' => 'burrë që i priten flokët: nuancë lëkure gjysmë e çelët',
    '💇🏽‍♂' => 'burrë që i priten flokët: nuancë lëkure zeshkane',
    '💇🏾‍♂' => 'burrë që i priten flokët: nuancë lëkure gjysmë e errët',
    '💇🏿‍♂' => 'burrë që i priten flokët: nuancë lëkure e errët',
    '💇🏻‍♀' => 'grua që i priten flokët: nuancë lëkure e çelët',
    '💇🏼‍♀' => 'grua që i priten flokët: nuancë lëkure gjysmë e çelët',
    '💇🏽‍♀' => 'grua që i priten flokët: nuancë lëkure zeshkane',
    '💇🏾‍♀' => 'grua që i priten flokët: nuancë lëkure gjysmë e errët',
    '💇🏿‍♀' => 'grua që i priten flokët: nuancë lëkure e errët',
    '🚶🏻‍♂' => 'burrë që ecën: nuancë lëkure e çelët',
    '🚶🏼‍♂' => 'burrë që ecën: nuancë lëkure gjysmë e çelët',
    '🚶🏽‍♂' => 'burrë që ecën: nuancë lëkure zeshkane',
    '🚶🏾‍♂' => 'burrë që ecën: nuancë lëkure gjysmë e errët',
    '🚶🏿‍♂' => 'burrë që ecën: nuancë lëkure e errët',
    '🚶🏻‍♀' => 'grua që ecën: nuancë lëkure e çelët',
    '🚶🏼‍♀' => 'grua që ecën: nuancë lëkure gjysmë e çelët',
    '🚶🏽‍♀' => 'grua që ecën: nuancë lëkure zeshkane',
    '🚶🏾‍♀' => 'grua që ecën: nuancë lëkure gjysmë e errët',
    '🚶🏿‍♀' => 'grua që ecën: nuancë lëkure e errët',
    '🚶🏻‍➡' => 'këmbësor: nuancë lëkure e çelët e me drejtim djathtas',
    '🚶🏼‍➡' => 'këmbësor: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🚶🏽‍➡' => 'këmbësor: nuancë lëkure zeshkane e me drejtim djathtas',
    '🚶🏾‍➡' => 'këmbësor: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🚶🏿‍➡' => 'këmbësor: nuancë lëkure e errët e me drejtim djathtas',
    '🧍🏻‍♂' => 'burrë në këmbë: nuancë lëkure e çelët',
    '🧍🏼‍♂' => 'burrë në këmbë: nuancë lëkure gjysmë e çelët',
    '🧍🏽‍♂' => 'burrë në këmbë: nuancë lëkure zeshkane',
    '🧍🏾‍♂' => 'burrë në këmbë: nuancë lëkure gjysmë e errët',
    '🧍🏿‍♂' => 'burrë në këmbë: nuancë lëkure e errët',
    '🧍🏻‍♀' => 'grua në këmbë: nuancë lëkure e çelët',
    '🧍🏼‍♀' => 'grua në këmbë: nuancë lëkure gjysmë e çelët',
    '🧍🏽‍♀' => 'grua në këmbë: nuancë lëkure zeshkane',
    '🧍🏾‍♀' => 'grua në këmbë: nuancë lëkure gjysmë e errët',
    '🧍🏿‍♀' => 'grua në këmbë: nuancë lëkure e errët',
    '🧎🏻‍♂' => 'burrë në gjunjë: nuancë lëkure e çelët',
    '🧎🏼‍♂' => 'burrë në gjunjë: nuancë lëkure gjysmë e çelët',
    '🧎🏽‍♂' => 'burrë në gjunjë: nuancë lëkure zeshkane',
    '🧎🏾‍♂' => 'burrë në gjunjë: nuancë lëkure gjysmë e errët',
    '🧎🏿‍♂' => 'burrë në gjunjë: nuancë lëkure e errët',
    '🧎🏻‍♀' => 'grua në gjunjë: nuancë lëkure e çelët',
    '🧎🏼‍♀' => 'grua në gjunjë: nuancë lëkure gjysmë e çelët',
    '🧎🏽‍♀' => 'grua në gjunjë: nuancë lëkure zeshkane',
    '🧎🏾‍♀' => 'grua në gjunjë: nuancë lëkure gjysmë e errët',
    '🧎🏿‍♀' => 'grua në gjunjë: nuancë lëkure e errët',
    '🧎🏻‍➡' => 'person në gjunjë: nuancë lëkure e çelët e me drejtim djathtas',
    '🧎🏼‍➡' => 'person në gjunjë: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🧎🏽‍➡' => 'person në gjunjë: nuancë lëkure zeshkane e me drejtim djathtas',
    '🧎🏾‍➡' => 'person në gjunjë: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🧎🏿‍➡' => 'person në gjunjë: nuancë lëkure e errët e me drejtim djathtas',
    '🧑🏻‍🦯' => 'person me shkop gjurmimi: nuancë lëkure e çelët',
    '🧑🏼‍🦯' => 'person me shkop gjurmimi: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🦯' => 'person me shkop gjurmimi: nuancë lëkure zeshkane',
    '🧑🏾‍🦯' => 'person me shkop gjurmimi: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🦯' => 'person me shkop gjurmimi: nuancë lëkure e errët',
    '👨🏻‍🦯' => 'burrë me shkop gjurmimi: nuancë lëkure e çelët',
    '👨🏼‍🦯' => 'burrë me shkop gjurmimi: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🦯' => 'burrë me shkop gjurmimi: nuancë lëkure zeshkane',
    '👨🏾‍🦯' => 'burrë me shkop gjurmimi: nuancë lëkure gjysmë e errët',
    '👨🏿‍🦯' => 'burrë me shkop gjurmimi: nuancë lëkure e errët',
    '👩🏻‍🦯' => 'grua me shkop gjurmimi: nuancë lëkure e çelët',
    '👩🏼‍🦯' => 'grua me shkop gjurmimi: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🦯' => 'grua me shkop gjurmimi: nuancë lëkure zeshkane',
    '👩🏾‍🦯' => 'grua me shkop gjurmimi: nuancë lëkure gjysmë e errët',
    '👩🏿‍🦯' => 'grua me shkop gjurmimi: nuancë lëkure e errët',
    '🧑🏻‍🦼' => 'person në karrocë të motorizuar invalidi: nuancë lëkure e çelët',
    '🧑🏼‍🦼' => 'person në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🦼' => 'person në karrocë të motorizuar invalidi: nuancë lëkure zeshkane',
    '🧑🏾‍🦼' => 'person në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🦼' => 'person në karrocë të motorizuar invalidi: nuancë lëkure e errët',
    '👨🏻‍🦼' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure e çelët',
    '👨🏼‍🦼' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🦼' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure zeshkane',
    '👨🏾‍🦼' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e errët',
    '👨🏿‍🦼' => 'burrë në karrocë të motorizuar invalidi: nuancë lëkure e errët',
    '👩🏻‍🦼' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure e çelët',
    '👩🏼‍🦼' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🦼' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure zeshkane',
    '👩🏾‍🦼' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure gjysmë e errët',
    '👩🏿‍🦼' => 'grua në karrocë të motorizuar invalidi: nuancë lëkure e errët',
    '🧑🏻‍🦽' => 'person në karrocë invalidi manuale: nuancë lëkure e çelët',
    '🧑🏼‍🦽' => 'person në karrocë invalidi manuale: nuancë lëkure gjysmë e çelët',
    '🧑🏽‍🦽' => 'person në karrocë invalidi manuale: nuancë lëkure zeshkane',
    '🧑🏾‍🦽' => 'person në karrocë invalidi manuale: nuancë lëkure gjysmë e errët',
    '🧑🏿‍🦽' => 'person në karrocë invalidi manuale: nuancë lëkure e errët',
    '👨🏻‍🦽' => 'burrë në karrocë manuale invalidi: nuancë lëkure e çelët',
    '👨🏼‍🦽' => 'burrë në karrocë manuale invalidi: nuancë lëkure gjysmë e çelët',
    '👨🏽‍🦽' => 'burrë në karrocë manuale invalidi: nuancë lëkure zeshkane',
    '👨🏾‍🦽' => 'burrë në karrocë manuale invalidi: nuancë lëkure gjysmë e errët',
    '👨🏿‍🦽' => 'burrë në karrocë manuale invalidi: nuancë lëkure e errët',
    '👩🏻‍🦽' => 'grua në karrocë manuale invalidi: nuancë lëkure e çelët',
    '👩🏼‍🦽' => 'grua në karrocë manuale invalidi: nuancë lëkure gjysmë e çelët',
    '👩🏽‍🦽' => 'grua në karrocë manuale invalidi: nuancë lëkure zeshkane',
    '👩🏾‍🦽' => 'grua në karrocë manuale invalidi: nuancë lëkure gjysmë e errët',
    '👩🏿‍🦽' => 'grua në karrocë manuale invalidi: nuancë lëkure e errët',
    '🏃🏻‍♂' => 'burrë që vrapon: nuancë lëkure e çelët',
    '🏃🏼‍♂' => 'burrë që vrapon: nuancë lëkure gjysmë e çelët',
    '🏃🏽‍♂' => 'burrë që vrapon: nuancë lëkure zeshkane',
    '🏃🏾‍♂' => 'burrë që vrapon: nuancë lëkure gjysmë e errët',
    '🏃🏿‍♂' => 'burrë që vrapon: nuancë lëkure e errët',
    '🏃🏻‍♀' => 'grua që vrapon: nuancë lëkure e çelët',
    '🏃🏼‍♀' => 'grua që vrapon: nuancë lëkure gjysmë e çelët',
    '🏃🏽‍♀' => 'grua që vrapon: nuancë lëkure zeshkane',
    '🏃🏾‍♀' => 'grua që vrapon: nuancë lëkure gjysmë e errët',
    '🏃🏿‍♀' => 'grua që vrapon: nuancë lëkure e errët',
    '🏃🏻‍➡' => 'vrapues: nuancë lëkure e çelët e me drejtim djathtas',
    '🏃🏼‍➡' => 'vrapues: nuancë lëkure gjysmë e çelët e me drejtim djathtas',
    '🏃🏽‍➡' => 'vrapues: nuancë lëkure zeshkane e me drejtim djathtas',
    '🏃🏾‍➡' => 'vrapues: nuancë lëkure gjysmë e errët e me drejtim djathtas',
    '🏃🏿‍➡' => 'vrapues: nuancë lëkure e errët e me drejtim djathtas',
    '👯🏻‍♀' => 'gra që festojnë: nuancë lëkure e çelët',
    '👯🏼‍♀' => 'gra që festojnë: nuancë lëkure gjysmë e çelët',
    '👯🏽‍♀' => 'gra që festojnë: nuancë lëkure zeshkane',
    '👯🏾‍♀' => 'gra që festojnë: nuancë lëkure gjysmë e errët',
    '👯🏿‍♀' => 'gra që festojnë: nuancë lëkure e errët',
    '👯🏻‍♂' => 'burra që festojnë: nuancë lëkure e çelët',
    '👯🏼‍♂' => 'burra që festojnë: nuancë lëkure gjysmë e çelët',
    '👯🏽‍♂' => 'burra që festojnë: nuancë lëkure zeshkane',
    '👯🏾‍♂' => 'burra që festojnë: nuancë lëkure gjysmë e errët',
    '👯🏿‍♂' => 'burra që festojnë: nuancë lëkure e errët',
    '🧖🏻‍♂' => 'burrë në dhomë me avull: nuancë lëkure e çelët',
    '🧖🏼‍♂' => 'burrë në dhomë me avull: nuancë lëkure gjysmë e çelët',
    '🧖🏽‍♂' => 'burrë në dhomë me avull: nuancë lëkure zeshkane',
    '🧖🏾‍♂' => 'burrë në dhomë me avull: nuancë lëkure gjysmë e errët',
    '🧖🏿‍♂' => 'burrë në dhomë me avull: nuancë lëkure e errët',
    '🧖🏻‍♀' => 'grua në dhomë me avull: nuancë lëkure e çelët',
    '🧖🏼‍♀' => 'grua në dhomë me avull: nuancë lëkure gjysmë e çelët',
    '🧖🏽‍♀' => 'grua në dhomë me avull: nuancë lëkure zeshkane',
    '🧖🏾‍♀' => 'grua në dhomë me avull: nuancë lëkure gjysmë e errët',
    '🧖🏿‍♀' => 'grua në dhomë me avull: nuancë lëkure e errët',
    '🧗🏻‍♂' => 'burrë që ngjitet: nuancë lëkure e çelët',
    '🧗🏼‍♂' => 'burrë që ngjitet: nuancë lëkure gjysmë e çelët',
    '🧗🏽‍♂' => 'burrë që ngjitet: nuancë lëkure zeshkane',
    '🧗🏾‍♂' => 'burrë që ngjitet: nuancë lëkure gjysmë e errët',
    '🧗🏿‍♂' => 'burrë që ngjitet: nuancë lëkure e errët',
    '🧗🏻‍♀' => 'grua që ngjitet: nuancë lëkure e çelët',
    '🧗🏼‍♀' => 'grua që ngjitet: nuancë lëkure gjysmë e çelët',
    '🧗🏽‍♀' => 'grua që ngjitet: nuancë lëkure zeshkane',
    '🧗🏾‍♀' => 'grua që ngjitet: nuancë lëkure gjysmë e errët',
    '🧗🏿‍♀' => 'grua që ngjitet: nuancë lëkure e errët',
    '🏌🏻‍♂' => 'burrë që luan golf: nuancë lëkure e çelët',
    '🏌🏼‍♂' => 'burrë që luan golf: nuancë lëkure gjysmë e çelët',
    '🏌🏽‍♂' => 'burrë që luan golf: nuancë lëkure zeshkane',
    '🏌🏾‍♂' => 'burrë që luan golf: nuancë lëkure gjysmë e errët',
    '🏌🏿‍♂' => 'burrë që luan golf: nuancë lëkure e errët',
    '🏌🏻‍♀' => 'grua që luan golf: nuancë lëkure e çelët',
    '🏌🏼‍♀' => 'grua që luan golf: nuancë lëkure gjysmë e çelët',
    '🏌🏽‍♀' => 'grua që luan golf: nuancë lëkure zeshkane',
    '🏌🏾‍♀' => 'grua që luan golf: nuancë lëkure gjysmë e errët',
    '🏌🏿‍♀' => 'grua që luan golf: nuancë lëkure e errët',
    '🏄🏻‍♂' => 'burrë që bën sërf: nuancë lëkure e çelët',
    '🏄🏼‍♂' => 'burrë që bën sërf: nuancë lëkure gjysmë e çelët',
    '🏄🏽‍♂' => 'burrë që bën sërf: nuancë lëkure zeshkane',
    '🏄🏾‍♂' => 'burrë që bën sërf: nuancë lëkure gjysmë e errët',
    '🏄🏿‍♂' => 'burrë që bën sërf: nuancë lëkure e errët',
    '🏄🏻‍♀' => 'grua që bën sërf: nuancë lëkure e çelët',
    '🏄🏼‍♀' => 'grua që bën sërf: nuancë lëkure gjysmë e çelët',
    '🏄🏽‍♀' => 'grua që bën sërf: nuancë lëkure zeshkane',
    '🏄🏾‍♀' => 'grua që bën sërf: nuancë lëkure gjysmë e errët',
    '🏄🏿‍♀' => 'grua që bën sërf: nuancë lëkure e errët',
    '🚣🏻‍♂' => 'burrë që vozit barkë: nuancë lëkure e çelët',
    '🚣🏼‍♂' => 'burrë që vozit barkë: nuancë lëkure gjysmë e çelët',
    '🚣🏽‍♂' => 'burrë që vozit barkë: nuancë lëkure zeshkane',
    '🚣🏾‍♂' => 'burrë që vozit barkë: nuancë lëkure gjysmë e errët',
    '🚣🏿‍♂' => 'burrë që vozit barkë: nuancë lëkure e errët',
    '🚣🏻‍♀' => 'grua që vozit barkë: nuancë lëkure e çelët',
    '🚣🏼‍♀' => 'grua që vozit barkë: nuancë lëkure gjysmë e çelët',
    '🚣🏽‍♀' => 'grua që vozit barkë: nuancë lëkure zeshkane',
    '🚣🏾‍♀' => 'grua që vozit barkë: nuancë lëkure gjysmë e errët',
    '🚣🏿‍♀' => 'grua që vozit barkë: nuancë lëkure e errët',
    '🏊🏻‍♂' => 'burrë që noton: nuancë lëkure e çelët',
    '🏊🏼‍♂' => 'burrë që noton: nuancë lëkure gjysmë e çelët',
    '🏊🏽‍♂' => 'burrë që noton: nuancë lëkure zeshkane',
    '🏊🏾‍♂' => 'burrë që noton: nuancë lëkure gjysmë e errët',
    '🏊🏿‍♂' => 'burrë që noton: nuancë lëkure e errët',
    '🏊🏻‍♀' => 'grua që noton: nuancë lëkure e çelët',
    '🏊🏼‍♀' => 'grua që noton: nuancë lëkure gjysmë e çelët',
    '🏊🏽‍♀' => 'grua që noton: nuancë lëkure zeshkane',
    '🏊🏾‍♀' => 'grua që noton: nuancë lëkure gjysmë e errët',
    '🏊🏿‍♀' => 'grua që noton: nuancë lëkure e errët',
    '⛹🏻‍♂' => 'burrë që luan me top: nuancë lëkure e çelët',
    '⛹🏼‍♂' => 'burrë që luan me top: nuancë lëkure gjysmë e çelët',
    '⛹🏽‍♂' => 'burrë që luan me top: nuancë lëkure zeshkane',
    '⛹🏾‍♂' => 'burrë që luan me top: nuancë lëkure gjysmë e errët',
    '⛹🏿‍♂' => 'burrë që luan me top: nuancë lëkure e errët',
    '⛹🏻‍♀' => 'grua që luan me top: nuancë lëkure e çelët',
    '⛹🏼‍♀' => 'grua që luan me top: nuancë lëkure gjysmë e çelët',
    '⛹🏽‍♀' => 'grua që luan me top: nuancë lëkure zeshkane',
    '⛹🏾‍♀' => 'grua që luan me top: nuancë lëkure gjysmë e errët',
    '⛹🏿‍♀' => 'grua që luan me top: nuancë lëkure e errët',
    '🏋🏻‍♂' => 'burrë që bën peshëngritje: nuancë lëkure e çelët',
    '🏋🏼‍♂' => 'burrë që bën peshëngritje: nuancë lëkure gjysmë e çelët',
    '🏋🏽‍♂' => 'burrë që bën peshëngritje: nuancë lëkure zeshkane',
    '🏋🏾‍♂' => 'burrë që bën peshëngritje: nuancë lëkure gjysmë e errët',
    '🏋🏿‍♂' => 'burrë që bën peshëngritje: nuancë lëkure e errët',
    '🏋🏻‍♀' => 'grua që bën peshëngritje: nuancë lëkure e çelët',
    '🏋🏼‍♀' => 'grua që bën peshëngritje: nuancë lëkure gjysmë e çelët',
    '🏋🏽‍♀' => 'grua që bën peshëngritje: nuancë lëkure zeshkane',
    '🏋🏾‍♀' => 'grua që bën peshëngritje: nuancë lëkure gjysmë e errët',
    '🏋🏿‍♀' => 'grua që bën peshëngritje: nuancë lëkure e errët',
    '🚴🏻‍♂' => 'burrë që nget biçikletë: nuancë lëkure e çelët',
    '🚴🏼‍♂' => 'burrë që nget biçikletë: nuancë lëkure gjysmë e çelët',
    '🚴🏽‍♂' => 'burrë që nget biçikletë: nuancë lëkure zeshkane',
    '🚴🏾‍♂' => 'burrë që nget biçikletë: nuancë lëkure gjysmë e errët',
    '🚴🏿‍♂' => 'burrë që nget biçikletë: nuancë lëkure e errët',
    '🚴🏻‍♀' => 'grua që nget biçikletë: nuancë lëkure e çelët',
    '🚴🏼‍♀' => 'grua që nget biçikletë: nuancë lëkure gjysmë e çelët',
    '🚴🏽‍♀' => 'grua që nget biçikletë: nuancë lëkure zeshkane',
    '🚴🏾‍♀' => 'grua që nget biçikletë: nuancë lëkure gjysmë e errët',
    '🚴🏿‍♀' => 'grua që nget biçikletë: nuancë lëkure e errët',
    '🚵🏻‍♂' => 'burrë që bën çiklizëm malor: nuancë lëkure e çelët',
    '🚵🏼‍♂' => 'burrë që bën çiklizëm malor: nuancë lëkure gjysmë e çelët',
    '🚵🏽‍♂' => 'burrë që bën çiklizëm malor: nuancë lëkure zeshkane',
    '🚵🏾‍♂' => 'burrë që bën çiklizëm malor: nuancë lëkure gjysmë e errët',
    '🚵🏿‍♂' => 'burrë që bën çiklizëm malor: nuancë lëkure e errët',
    '🚵🏻‍♀' => 'grua që bën çiklizëm malor: nuancë lëkure e çelët',
    '🚵🏼‍♀' => 'grua që bën çiklizëm malor: nuancë lëkure gjysmë e çelët',
    '🚵🏽‍♀' => 'grua që bën çiklizëm malor: nuancë lëkure zeshkane',
    '🚵🏾‍♀' => 'grua që bën çiklizëm malor: nuancë lëkure gjysmë e errët',
    '🚵🏿‍♀' => 'grua që bën çiklizëm malor: nuancë lëkure e errët',
    '🤸🏻‍♂' => 'burrë që bën kapërdimje: nuancë lëkure e çelët',
    '🤸🏼‍♂' => 'burrë që bën kapërdimje: nuancë lëkure gjysmë e çelët',
    '🤸🏽‍♂' => 'burrë që bën kapërdimje: nuancë lëkure zeshkane',
    '🤸🏾‍♂' => 'burrë që bën kapërdimje: nuancë lëkure gjysmë e errët',
    '🤸🏿‍♂' => 'burrë që bën kapërdimje: nuancë lëkure e errët',
    '🤸🏻‍♀' => 'grua që bën kapërdimje: nuancë lëkure e çelët',
    '🤸🏼‍♀' => 'grua që bën kapërdimje: nuancë lëkure gjysmë e çelët',
    '🤸🏽‍♀' => 'grua që bën kapërdimje: nuancë lëkure zeshkane',
    '🤸🏾‍♀' => 'grua që bën kapërdimje: nuancë lëkure gjysmë e errët',
    '🤸🏿‍♀' => 'grua që bën kapërdimje: nuancë lëkure e errët',
    '🤼🏻‍♀' => 'gra që bëjnë mundje: nuancë lëkure e çelët',
    '🤼🏼‍♀' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e çelët',
    '🤼🏽‍♀' => 'gra që bëjnë mundje: nuancë lëkure zeshkane',
    '🤼🏾‍♀' => 'gra që bëjnë mundje: nuancë lëkure gjysmë e errët',
    '🤼🏿‍♀' => 'gra që bëjnë mundje: nuancë lëkure e errët',
    '🤼🏻‍♂' => 'burra që bëjnë mundje: nuancë lëkure e çelët',
    '🤼🏼‍♂' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e çelët',
    '🤼🏽‍♂' => 'burra që bëjnë mundje: nuancë lëkure zeshkane',
    '🤼🏾‍♂' => 'burra që bëjnë mundje: nuancë lëkure gjysmë e errët',
    '🤼🏿‍♂' => 'burra që bëjnë mundje: nuancë lëkure e errët',
    '🤽🏻‍♂' => 'burra që luajnë vaterpolo: nuancë lëkure e çelët',
    '🤽🏼‍♂' => 'burra që luajnë vaterpolo: nuancë lëkure gjysmë e çelët',
    '🤽🏽‍♂' => 'burra që luajnë vaterpolo: nuancë lëkure zeshkane',
    '🤽🏾‍♂' => 'burra që luajnë vaterpolo: nuancë lëkure gjysmë e errët',
    '🤽🏿‍♂' => 'burra që luajnë vaterpolo: nuancë lëkure e errët',
    '🤽🏻‍♀' => 'gra që luajnë vaterpolo: nuancë lëkure e çelët',
    '🤽🏼‍♀' => 'gra që luajnë vaterpolo: nuancë lëkure gjysmë e çelët',
    '🤽🏽‍♀' => 'gra që luajnë vaterpolo: nuancë lëkure zeshkane',
    '🤽🏾‍♀' => 'gra që luajnë vaterpolo: nuancë lëkure gjysmë e errët',
    '🤽🏿‍♀' => 'gra që luajnë vaterpolo: nuancë lëkure e errët',
    '🤾🏻‍♂' => 'burra që luajnë hendboll: nuancë lëkure e çelët',
    '🤾🏼‍♂' => 'burra që luajnë hendboll: nuancë lëkure gjysmë e çelët',
    '🤾🏽‍♂' => 'burra që luajnë hendboll: nuancë lëkure zeshkane',
    '🤾🏾‍♂' => 'burra që luajnë hendboll: nuancë lëkure gjysmë e errët',
    '🤾🏿‍♂' => 'burra që luajnë hendboll: nuancë lëkure e errët',
    '🤾🏻‍♀' => 'gra që luajnë hendboll: nuancë lëkure e çelët',
    '🤾🏼‍♀' => 'gra që luajnë hendboll: nuancë lëkure gjysmë e çelët',
    '🤾🏽‍♀' => 'gra që luajnë hendboll: nuancë lëkure zeshkane',
    '🤾🏾‍♀' => 'gra që luajnë hendboll: nuancë lëkure gjysmë e errët',
    '🤾🏿‍♀' => 'gra që luajnë hendboll: nuancë lëkure e errët',
    '🤹🏻‍♂' => 'burrë që zhonglon: nuancë lëkure e çelët',
    '🤹🏼‍♂' => 'burrë që zhonglon: nuancë lëkure gjysmë e çelët',
    '🤹🏽‍♂' => 'burrë që zhonglon: nuancë lëkure zeshkane',
    '🤹🏾‍♂' => 'burrë që zhonglon: nuancë lëkure gjysmë e errët',
    '🤹🏿‍♂' => 'burrë që zhonglon: nuancë lëkure e errët',
    '🤹🏻‍♀' => 'grua që zhonglon: nuancë lëkure e çelët',
    '🤹🏼‍♀' => 'grua që zhonglon: nuancë lëkure gjysmë e çelët',
    '🤹🏽‍♀' => 'grua që zhonglon: nuancë lëkure zeshkane',
    '🤹🏾‍♀' => 'grua që zhonglon: nuancë lëkure gjysmë e errët',
    '🤹🏿‍♀' => 'grua që zhonglon: nuancë lëkure e errët',
    '🧘🏻‍♂' => 'burrë në pozicion ëndërrues: nuancë lëkure e çelët',
    '🧘🏼‍♂' => 'burrë në pozicion ëndërrues: nuancë lëkure gjysmë e çelët',
    '🧘🏽‍♂' => 'burrë në pozicion ëndërrues: nuancë lëkure zeshkane',
    '🧘🏾‍♂' => 'burrë në pozicion ëndërrues: nuancë lëkure gjysmë e errët',
    '🧘🏿‍♂' => 'burrë në pozicion ëndërrues: nuancë lëkure e errët',
    '🧘🏻‍♀' => 'grua në pozicion ëndërrues: nuancë lëkure e çelët',
    '🧘🏼‍♀' => 'grua në pozicion ëndërrues: nuancë lëkure gjysmë e çelët',
    '🧘🏽‍♀' => 'grua në pozicion ëndërrues: nuancë lëkure zeshkane',
    '🧘🏾‍♀' => 'grua në pozicion ëndërrues: nuancë lëkure gjysmë e errët',
    '🧘🏿‍♀' => 'grua në pozicion ëndërrues: nuancë lëkure e errët',
    '🧑‍🦰' => 'fëmijë i rritur: flokë të kuqe',
    '🧑‍🦱' => 'fëmijë i rritur: flokë kaçurrele',
    '🧑‍🦳' => 'fëmijë i rritur: flokë të bardha',
    '🧑‍🦲' => 'fëmijë i rritur: qeros',
    '👨‍🦰' => 'burrë: flokë të kuqe',
    '👨‍🦱' => 'burrë: flokë kaçurrele',
    '👨‍🦳' => 'burrë: flokë të bardha',
    '👨‍🦲' => 'burrë: qeros',
    '👩‍🦰' => 'grua: flokë të kuqe',
    '👩‍🦱' => 'grua: flokë kaçurrele',
    '👩‍🦳' => 'grua: flokë të bardha',
    '👩‍🦲' => 'grua: qeros',
    '🚶‍➡' => 'këmbësor: me drejtim djathtas',
    '🧎‍➡' => 'person në gjunjë: me drejtim djathtas',
    '🏃‍➡' => 'vrapues: me drejtim djathtas',
    '👨‍👦' => 'familje: burrë e djalë',
    '👨‍👧' => 'familje: burrë e vajzë',
    '👩‍👦' => 'familje: grua e djalë',
    '👩‍👧' => 'familje: grua e vajzë',
    '😶‍🌫' => 'fytyrë në re',
    '😮‍💨' => 'fytyrë me shfryrje',
    '🙂‍↕' => 'kokë që tundet vertikalisht',
    '😵‍💫' => 'fytyrë me sy spiralë',
    '❤‍🔥' => 'zemër në flakë',
    '❤‍🩹' => 'zemër në shërim',
    '👁‍🗨' => 'sy në flluskën e të folurit',
    '🧔‍♂' => 'burrë: mjekër',
    '👱‍♂' => 'burrë biond',
    '🧔‍♀' => 'grua: mjekër',
    '👱‍♀' => 'grua bionde',
    '🙍‍♂' => 'burrë i vrenjtur',
    '🙍‍♀' => 'grua e vrenjtur',
    '🙎‍♂' => 'burrë i pakënaqur',
    '🙎‍♀' => 'grua e pakënaqur',
    '🙅‍♂' => 'burrë që bën gjestin e ndaluar',
    '🙅‍♀' => 'grua që bën gjestin e ndaluar',
    '🙆‍♂' => 'burrë që bën gjestin në rregull',
    '🙆‍♀' => 'grua që bën gjestin në rregull',
    '💁‍♂' => 'burrë që tregon me dorë',
    '💁‍♀' => 'grua që tregon me dorë',
    '🙋‍♂' => 'burrë që ngre dorën',
    '🙋‍♀' => 'grua që ngre dorën',
    '🧏‍♂' => 'burrë i shurdhër',
    '🧏‍♀' => 'grua e shurdhër',
    '🙇‍♂' => 'burrë që përkulet për falje',
    '🙇‍♀' => 'grua që përkulet për falje',
    '🤦‍♂' => 'burrë që vë dorën në kokë',
    '🤦‍♀' => 'grua që vë dorën në kokë',
    '🤷‍♂' => 'burrë që ngre supet',
    '🤷‍♀' => 'grua që ngre supet',
    '🧑‍⚕' => 'person punonjës shëndetësor',
    '👨‍⚕' => 'punonjës shëndetësor',
    '👩‍⚕' => 'punonjëse shëndetësor',
    '🧑‍🎓' => 'person student',
    '👨‍🎓' => 'student',
    '👩‍🎓' => 'studente',
    '🧑‍🏫' => 'person mësues',
    '👨‍🏫' => 'mësues',
    '👩‍🏫' => 'mësuese',
    '🧑‍⚖' => 'person gjykatës',
    '👨‍⚖' => 'gjykatës',
    '👩‍⚖' => 'gjykatëse',
    '🧑‍🌾' => 'person bujk',
    '👨‍🌾' => 'bujk',
    '👩‍🌾' => 'bujkeshë',
    '🧑‍🍳' => 'person kuzhinier',
    '👨‍🍳' => 'kuzhinier',
    '👩‍🍳' => 'kuzhiniere',
    '🧑‍🔧' => 'person mekanik',
    '👨‍🔧' => 'mekanik',
    '👩‍🔧' => 'mekanike',
    '🧑‍🏭' => 'person punëtor fabrike',
    '👨‍🏭' => 'punëtor fabrike',
    '👩‍🏭' => 'punëtore fabrike',
    '🧑‍💼' => 'person punonjës zyre',
    '👨‍💼' => 'punonjës zyre',
    '👩‍💼' => 'punonjëse zyre',
    '🧑‍🔬' => 'person shkencëtar',
    '👨‍🔬' => 'shkencëtar',
    '👩‍🔬' => 'shkencëtare',
    '🧑‍💻' => 'person teknolog',
    '👨‍💻' => 'teknolog',
    '👩‍💻' => 'teknologe',
    '🧑‍🎤' => 'person këngëtar',
    '👨‍🎤' => 'këngëtar',
    '👩‍🎤' => 'këngëtare',
    '🧑‍🎨' => 'person artist',
    '👨‍🎨' => 'artist',
    '👩‍🎨' => 'artiste',
    '🧑‍✈' => 'person pilot',
    '👨‍✈' => 'pilot',
    '👩‍✈' => 'pilote',
    '🧑‍🚀' => 'person astronaut',
    '👨‍🚀' => 'astronaut',
    '👩‍🚀' => 'astronaute',
    '🧑‍🚒' => 'person zjarrfikës',
    '👨‍🚒' => 'zjarrfikës',
    '👩‍🚒' => 'grua zjarrëfikëse',
    '👮‍♂' => 'oficer policie',
    '👮‍♀' => 'oficere policie',
    '🕵‍♂' => 'hetues',
    '🕵‍♀' => 'hetuese',
    '💂‍♂' => 'gardist',
    '💂‍♀' => 'ushtare garde',
    '👷‍♂' => 'murator',
    '👷‍♀' => 'muratore',
    '👳‍♂' => 'djalë me çallmë',
    '👳‍♀' => 'vajzë me çallmë',
    '🤵‍♂' => 'burrë me kostum',
    '🤵‍♀' => 'grua me kostum',
    '👰‍♂' => 'burrë me vello',
    '👰‍♀' => 'grua me vello',
    '👩‍🍼' => 'grua që ushqen foshnjë',
    '👨‍🍼' => 'burrë që ushqen foshnjë',
    '🧑‍🍼' => 'person që ushqen foshnjë',
    '🧑‍🎄' => 'zx klaus',
    '🦸‍♂' => 'burrë superhero',
    '🦸‍♀' => 'superheroinë',
    '🦹‍♂' => 'burrë superzuzar',
    '🦹‍♀' => 'superzuzare',
    '🧙‍♂' => 'burrë magjistar',
    '🧙‍♀' => 'magjistare',
    '🧚‍♂' => 'burrë zanë',
    '🧚‍♀' => 'grua zanë',
    '🧛‍♂' => 'burrë vampir',
    '🧛‍♀' => 'grua vampir',
    '🧜‍♂' => 'burrë sirenë',
    '🧜‍♀' => 'grua sirenë',
    '🧝‍♂' => 'burrë kukudh',
    '🧝‍♀' => 'grua kukudh',
    '🧞‍♂' => 'burrë xhind',
    '🧞‍♀' => 'grua xhind',
    '🧟‍♂' => 'burrë zombi',
    '🧟‍♀' => 'grua zombi',
    '💆‍♂' => 'burrë që i bëhet masazh fytyre',
    '💆‍♀' => 'grua që i bëhet masazh fytyre',
    '💇‍♂' => 'burrë që i priten flokët',
    '💇‍♀' => 'grua që i priten flokët',
    '🚶‍♂' => 'burrë që ecën',
    '🚶‍♀' => 'grua që ecën',
    '🧍‍♂' => 'burrë në këmbë',
    '🧍‍♀' => 'grua në këmbë',
    '🧎‍♂' => 'burrë në gjunjë',
    '🧎‍♀' => 'grua në gjunjë',
    '🧑‍🦯' => 'person me shkop gjurmimi',
    '👨‍🦯' => 'burrë me shkop gjurmimi',
    '👩‍🦯' => 'grua me shkop gjurmimi',
    '🧑‍🦼' => 'person në karrocë të motorizuar invalidi',
    '👨‍🦼' => 'burrë në karrocë të motorizuar invalidi',
    '👩‍🦼' => 'grua në karrocë të motorizuar invalidi',
    '🧑‍🦽' => 'person në karrocë invalidi manuale',
    '👨‍🦽' => 'burrë në karrocë manuale invalidi',
    '👩‍🦽' => 'grua në karrocë manuale invalidi',
    '🏃‍♂' => 'burrë që vrapon',
    '🏃‍♀' => 'grua që vrapon',
    '🧑‍🩰' => 'balerin',
    '👯‍♂' => 'burra që festojnë',
    '👯‍♀' => 'gra që festojnë',
    '🧖‍♂' => 'burrë në dhomë me avull',
    '🧖‍♀' => 'grua në dhomë me avull',
    '🧗‍♂' => 'burrë që ngjitet',
    '🧗‍♀' => 'grua që ngjitet',
    '🏌‍♂' => 'burrë që luan golf',
    '🏌‍♀' => 'grua që luan golf',
    '🏄‍♂' => 'burrë që bën sërf',
    '🏄‍♀' => 'grua që bën sërf',
    '🚣‍♂' => 'burrë që vozit barkë',
    '🚣‍♀' => 'grua që vozit barkë',
    '🏊‍♂' => 'burrë që noton',
    '🏊‍♀' => 'grua që noton',
    '⛹‍♂' => 'burrë që luan me top',
    '⛹‍♀' => 'grua që luan me top',
    '🏋‍♂' => 'burrë që bën peshëngritje',
    '🏋‍♀' => 'grua që bën peshëngritje',
    '🚴‍♂' => 'burrë që nget biçikletë',
    '🚴‍♀' => 'grua që nget biçikletë',
    '🚵‍♂' => 'burrë që bën çiklizëm malor',
    '🚵‍♀' => 'grua që bën çiklizëm malor',
    '🤸‍♂' => 'burrë që bën kapërdimje',
    '🤸‍♀' => 'grua që bën kapërdimje',
    '🤼‍♂' => 'burra që bëjnë mundje',
    '🤼‍♀' => 'gra që bëjnë mundje',
    '🤽‍♂' => 'burra që luajnë vaterpolo',
    '🤽‍♀' => 'gra që luajnë vaterpolo',
    '🤾‍♂' => 'burra që luajnë hendboll',
    '🤾‍♀' => 'gra që luajnë hendboll',
    '🤹‍♂' => 'burrë që zhonglon',
    '🤹‍♀' => 'grua që zhonglon',
    '🧘‍♂' => 'burrë në pozicion ëndërrues',
    '🧘‍♀' => 'grua në pozicion ëndërrues',
    '🧑‍🧒' => 'familje: i rritur, fëmijë',
    '🐕‍🦺' => 'qen shërbimi',
    '🐈‍⬛' => 'mace e zezë',
    '🐻‍❄' => 'ari polar',
    '🐦‍⬛' => 'zog i zi',
    '🐦‍🔥' => 'feniks',
    '🍋‍🟩' => 'limon jeshil',
    '🍄‍🟫' => 'kërpudhë kafe',
    '⛓‍💥' => 'zinxhir i këputur',
    '🏳‍🌈' => 'flamur i ylbertë',
    '🏳‍⚧' => 'flamur blu, rozë dhe i bardhë',
    '🏴‍☠' => 'flamur piratësh',
    '👋🏻' => 'dorë që tundet: nuancë lëkure e çelët',
    '👋🏼' => 'dorë që tundet: nuancë lëkure gjysmë e çelët',
    '👋🏽' => 'dorë që tundet: nuancë lëkure zeshkane',
    '👋🏾' => 'dorë që tundet: nuancë lëkure gjysmë e errët',
    '👋🏿' => 'dorë që tundet: nuancë lëkure e errët',
    '🤚🏻' => 'dorë e ngritur nga ana e kurrizit: nuancë lëkure e çelët',
    '🤚🏼' => 'dorë e ngritur nga ana e kurrizit: nuancë lëkure gjysmë e çelët',
    '🤚🏽' => 'dorë e ngritur nga ana e kurrizit: nuancë lëkure zeshkane',
    '🤚🏾' => 'dorë e ngritur nga ana e kurrizit: nuancë lëkure gjysmë e errët',
    '🤚🏿' => 'dorë e ngritur nga ana e kurrizit: nuancë lëkure e errët',
    '🖐🏻' => 'dorë e ngritur me gishtat të hapur: nuancë lëkure e çelët',
    '🖐🏼' => 'dorë e ngritur me gishtat të hapur: nuancë lëkure gjysmë e çelët',
    '🖐🏽' => 'dorë e ngritur me gishtat të hapur: nuancë lëkure zeshkane',
    '🖐🏾' => 'dorë e ngritur me gishtat të hapur: nuancë lëkure gjysmë e errët',
    '🖐🏿' => 'dorë e ngritur me gishtat të hapur: nuancë lëkure e errët',
    '✋🏻' => 'dorë e ngritur: nuancë lëkure e çelët',
    '✋🏼' => 'dorë e ngritur: nuancë lëkure gjysmë e çelët',
    '✋🏽' => 'dorë e ngritur: nuancë lëkure zeshkane',
    '✋🏾' => 'dorë e ngritur: nuancë lëkure gjysmë e errët',
    '✋🏿' => 'dorë e ngritur: nuancë lëkure e errët',
    '🖖🏻' => 'përshëndetja e vulkanit: nuancë lëkure e çelët',
    '🖖🏼' => 'përshëndetja e vulkanit: nuancë lëkure gjysmë e çelët',
    '🖖🏽' => 'përshëndetja e vulkanit: nuancë lëkure zeshkane',
    '🖖🏾' => 'përshëndetja e vulkanit: nuancë lëkure gjysmë e errët',
    '🖖🏿' => 'përshëndetja e vulkanit: nuancë lëkure e errët',
    '🫱🏻' => 'dorë djathtas: nuancë lëkure e çelët',
    '🫱🏼' => 'dorë djathtas: nuancë lëkure gjysmë e çelët',
    '🫱🏽' => 'dorë djathtas: nuancë lëkure zeshkane',
    '🫱🏾' => 'dorë djathtas: nuancë lëkure gjysmë e errët',
    '🫱🏿' => 'dorë djathtas: nuancë lëkure e errët',
    '🫲🏻' => 'dorë majtas: nuancë lëkure e çelët',
    '🫲🏼' => 'dorë majtas: nuancë lëkure gjysmë e çelët',
    '🫲🏽' => 'dorë majtas: nuancë lëkure zeshkane',
    '🫲🏾' => 'dorë majtas: nuancë lëkure gjysmë e errët',
    '🫲🏿' => 'dorë majtas: nuancë lëkure e errët',
    '🫳🏻' => 'dorë me pëllëmbë poshtë: nuancë lëkure e çelët',
    '🫳🏼' => 'dorë me pëllëmbë poshtë: nuancë lëkure gjysmë e çelët',
    '🫳🏽' => 'dorë me pëllëmbë poshtë: nuancë lëkure zeshkane',
    '🫳🏾' => 'dorë me pëllëmbë poshtë: nuancë lëkure gjysmë e errët',
    '🫳🏿' => 'dorë me pëllëmbë poshtë: nuancë lëkure e errët',
    '🫴🏻' => 'dorë me pëllëmbë lart: nuancë lëkure e çelët',
    '🫴🏼' => 'dorë me pëllëmbë lart: nuancë lëkure gjysmë e çelët',
    '🫴🏽' => 'dorë me pëllëmbë lart: nuancë lëkure zeshkane',
    '🫴🏾' => 'dorë me pëllëmbë lart: nuancë lëkure gjysmë e errët',
    '🫴🏿' => 'dorë me pëllëmbë lart: nuancë lëkure e errët',
    '🫷🏻' => 'dorë që shtyn majtas: nuancë lëkure e çelët',
    '🫷🏼' => 'dorë që shtyn majtas: nuancë lëkure gjysmë e çelët',
    '🫷🏽' => 'dorë që shtyn majtas: nuancë lëkure zeshkane',
    '🫷🏾' => 'dorë që shtyn majtas: nuancë lëkure gjysmë e errët',
    '🫷🏿' => 'dorë që shtyn majtas: nuancë lëkure e errët',
    '🫸🏻' => 'dorë që shtyn djathtas: nuancë lëkure e çelët',
    '🫸🏼' => 'dorë që shtyn djathtas: nuancë lëkure gjysmë e çelët',
    '🫸🏽' => 'dorë që shtyn djathtas: nuancë lëkure zeshkane',
    '🫸🏾' => 'dorë që shtyn djathtas: nuancë lëkure gjysmë e errët',
    '🫸🏿' => 'dorë që shtyn djathtas: nuancë lëkure e errët',
    '👌🏻' => '«ok» me dorë: nuancë lëkure e çelët',
    '👌🏼' => '«ok» me dorë: nuancë lëkure gjysmë e çelët',
    '👌🏽' => '«ok» me dorë: nuancë lëkure zeshkane',
    '👌🏾' => '«ok» me dorë: nuancë lëkure gjysmë e errët',
    '👌🏿' => '«ok» me dorë: nuancë lëkure e errët',
    '🤌🏻' => 'gishta të mbledhur: nuancë lëkure e çelët',
    '🤌🏼' => 'gishta të mbledhur: nuancë lëkure gjysmë e çelët',
    '🤌🏽' => 'gishta të mbledhur: nuancë lëkure zeshkane',
    '🤌🏾' => 'gishta të mbledhur: nuancë lëkure gjysmë e errët',
    '🤌🏿' => 'gishta të mbledhur: nuancë lëkure e errët',
    '🤏🏻' => 'dorështrënguar: nuancë lëkure e çelët',
    '🤏🏼' => 'dorështrënguar: nuancë lëkure gjysmë e çelët',
    '🤏🏽' => 'dorështrënguar: nuancë lëkure zeshkane',
    '🤏🏾' => 'dorështrënguar: nuancë lëkure gjysmë e errët',
    '🤏🏿' => 'dorështrënguar: nuancë lëkure e errët',
    '✌🏻' => 'dora e fitores: nuancë lëkure e çelët',
    '✌🏼' => 'dora e fitores: nuancë lëkure gjysmë e çelët',
    '✌🏽' => 'dora e fitores: nuancë lëkure zeshkane',
    '✌🏾' => 'dora e fitores: nuancë lëkure gjysmë e errët',
    '✌🏿' => 'dora e fitores: nuancë lëkure e errët',
    '🤞🏻' => 'gishta të kryqëzuar: nuancë lëkure e çelët',
    '🤞🏼' => 'gishta të kryqëzuar: nuancë lëkure gjysmë e çelët',
    '🤞🏽' => 'gishta të kryqëzuar: nuancë lëkure zeshkane',
    '🤞🏾' => 'gishta të kryqëzuar: nuancë lëkure gjysmë e errët',
    '🤞🏿' => 'gishta të kryqëzuar: nuancë lëkure e errët',
    '🫰🏻' => 'dorë me gishtin tregues dhe të madhin të kapur: nuancë lëkure e çelët',
    '🫰🏼' => 'dorë me gishtin tregues dhe të madhin të kapur: nuancë lëkure gjysmë e çelët',
    '🫰🏽' => 'dorë me gishtin tregues dhe të madhin të kapur: nuancë lëkure zeshkane',
    '🫰🏾' => 'dorë me gishtin tregues dhe të madhin të kapur: nuancë lëkure gjysmë e errët',
    '🫰🏿' => 'dorë me gishtin tregues dhe të madhin të kapur: nuancë lëkure e errët',
    '🤟🏻' => 'gjesti "të dua": nuancë lëkure e çelët',
    '🤟🏼' => 'gjesti "të dua": nuancë lëkure gjysmë e çelët',
    '🤟🏽' => 'gjesti "të dua": nuancë lëkure zeshkane',
    '🤟🏾' => 'gjesti "të dua": nuancë lëkure gjysmë e errët',
    '🤟🏿' => 'gjesti "të dua": nuancë lëkure e errët',
    '🤘🏻' => 'shenja e brirëve: nuancë lëkure e çelët',
    '🤘🏼' => 'shenja e brirëve: nuancë lëkure gjysmë e çelët',
    '🤘🏽' => 'shenja e brirëve: nuancë lëkure zeshkane',
    '🤘🏾' => 'shenja e brirëve: nuancë lëkure gjysmë e errët',
    '🤘🏿' => 'shenja e brirëve: nuancë lëkure e errët',
    '🤙🏻' => 'dorë me shenjën “më telefono”: nuancë lëkure e çelët',
    '🤙🏼' => 'dorë me shenjën “më telefono”: nuancë lëkure gjysmë e çelët',
    '🤙🏽' => 'dorë me shenjën “më telefono”: nuancë lëkure zeshkane',
    '🤙🏾' => 'dorë me shenjën “më telefono”: nuancë lëkure gjysmë e errët',
    '🤙🏿' => 'dorë me shenjën “më telefono”: nuancë lëkure e errët',
    '👈🏻' => 'gishti tregues që tregon majtas: nuancë lëkure e çelët',
    '👈🏼' => 'gishti tregues që tregon majtas: nuancë lëkure gjysmë e çelët',
    '👈🏽' => 'gishti tregues që tregon majtas: nuancë lëkure zeshkane',
    '👈🏾' => 'gishti tregues që tregon majtas: nuancë lëkure gjysmë e errët',
    '👈🏿' => 'gishti tregues që tregon majtas: nuancë lëkure e errët',
    '👉🏻' => 'gisht tregues i drejtuar djathtas: nuancë lëkure e çelët',
    '👉🏼' => 'gisht tregues i drejtuar djathtas: nuancë lëkure gjysmë e çelët',
    '👉🏽' => 'gisht tregues i drejtuar djathtas: nuancë lëkure zeshkane',
    '👉🏾' => 'gisht tregues i drejtuar djathtas: nuancë lëkure gjysmë e errët',
    '👉🏿' => 'gisht tregues i drejtuar djathtas: nuancë lëkure e errët',
    '👆🏻' => 'gisht tregues i drejtuar lart: nuancë lëkure e çelët',
    '👆🏼' => 'gisht tregues i drejtuar lart: nuancë lëkure gjysmë e çelët',
    '👆🏽' => 'gisht tregues i drejtuar lart: nuancë lëkure zeshkane',
    '👆🏾' => 'gisht tregues i drejtuar lart: nuancë lëkure gjysmë e errët',
    '👆🏿' => 'gisht tregues i drejtuar lart: nuancë lëkure e errët',
    '🖕🏻' => 'gishti i mesit: nuancë lëkure e çelët',
    '🖕🏼' => 'gishti i mesit: nuancë lëkure gjysmë e çelët',
    '🖕🏽' => 'gishti i mesit: nuancë lëkure zeshkane',
    '🖕🏾' => 'gishti i mesit: nuancë lëkure gjysmë e errët',
    '🖕🏿' => 'gishti i mesit: nuancë lëkure e errët',
    '👇🏻' => 'gisht tregues i drejtuar poshtë: nuancë lëkure e çelët',
    '👇🏼' => 'gisht tregues i drejtuar poshtë: nuancë lëkure gjysmë e çelët',
    '👇🏽' => 'gisht tregues i drejtuar poshtë: nuancë lëkure zeshkane',
    '👇🏾' => 'gisht tregues i drejtuar poshtë: nuancë lëkure gjysmë e errët',
    '👇🏿' => 'gisht tregues i drejtuar poshtë: nuancë lëkure e errët',
    '☝🏻' => 'gisht tregues që tregon lart: nuancë lëkure e çelët',
    '☝🏼' => 'gisht tregues që tregon lart: nuancë lëkure gjysmë e çelët',
    '☝🏽' => 'gisht tregues që tregon lart: nuancë lëkure zeshkane',
    '☝🏾' => 'gisht tregues që tregon lart: nuancë lëkure gjysmë e errët',
    '☝🏿' => 'gisht tregues që tregon lart: nuancë lëkure e errët',
    '🫵🏻' => 'gishti tregues nga shikuesi: nuancë lëkure e çelët',
    '🫵🏼' => 'gishti tregues nga shikuesi: nuancë lëkure gjysmë e çelët',
    '🫵🏽' => 'gishti tregues nga shikuesi: nuancë lëkure zeshkane',
    '🫵🏾' => 'gishti tregues nga shikuesi: nuancë lëkure gjysmë e errët',
    '🫵🏿' => 'gishti tregues nga shikuesi: nuancë lëkure e errët',
    '👍🏻' => 'gishti i madh lart: nuancë lëkure e çelët',
    '👍🏼' => 'gishti i madh lart: nuancë lëkure gjysmë e çelët',
    '👍🏽' => 'gishti i madh lart: nuancë lëkure zeshkane',
    '👍🏾' => 'gishti i madh lart: nuancë lëkure gjysmë e errët',
    '👍🏿' => 'gishti i madh lart: nuancë lëkure e errët',
    '👎🏻' => 'gishti i madh poshtë: nuancë lëkure e çelët',
    '👎🏼' => 'gishti i madh poshtë: nuancë lëkure gjysmë e çelët',
    '👎🏽' => 'gishti i madh poshtë: nuancë lëkure zeshkane',
    '👎🏾' => 'gishti i madh poshtë: nuancë lëkure gjysmë e errët',
    '👎🏿' => 'gishti i madh poshtë: nuancë lëkure e errët',
    '✊🏻' => 'grusht i ngritur: nuancë lëkure e çelët',
    '✊🏼' => 'grusht i ngritur: nuancë lëkure gjysmë e çelët',
    '✊🏽' => 'grusht i ngritur: nuancë lëkure zeshkane',
    '✊🏾' => 'grusht i ngritur: nuancë lëkure gjysmë e errët',
    '✊🏿' => 'grusht i ngritur: nuancë lëkure e errët',
    '👊🏻' => 'grusht që vjen: nuancë lëkure e çelët',
    '👊🏼' => 'grusht që vjen: nuancë lëkure gjysmë e çelët',
    '👊🏽' => 'grusht që vjen: nuancë lëkure zeshkane',
    '👊🏾' => 'grusht që vjen: nuancë lëkure gjysmë e errët',
    '👊🏿' => 'grusht që vjen: nuancë lëkure e errët',
    '🤛🏻' => 'grusht i majtë në anën e përparme: nuancë lëkure e çelët',
    '🤛🏼' => 'grusht i majtë në anën e përparme: nuancë lëkure gjysmë e çelët',
    '🤛🏽' => 'grusht i majtë në anën e përparme: nuancë lëkure zeshkane',
    '🤛🏾' => 'grusht i majtë në anën e përparme: nuancë lëkure gjysmë e errët',
    '🤛🏿' => 'grusht i majtë në anën e përparme: nuancë lëkure e errët',
    '🤜🏻' => 'grusht i djathtë në anën e përparme: nuancë lëkure e çelët',
    '🤜🏼' => 'grusht i djathtë në anën e përparme: nuancë lëkure gjysmë e çelët',
    '🤜🏽' => 'grusht i djathtë në anën e përparme: nuancë lëkure zeshkane',
    '🤜🏾' => 'grusht i djathtë në anën e përparme: nuancë lëkure gjysmë e errët',
    '🤜🏿' => 'grusht i djathtë në anën e përparme: nuancë lëkure e errët',
    '👏🏻' => 'duar që duartrokasin: nuancë lëkure e çelët',
    '👏🏼' => 'duar që duartrokasin: nuancë lëkure gjysmë e çelët',
    '👏🏽' => 'duar që duartrokasin: nuancë lëkure zeshkane',
    '👏🏾' => 'duar që duartrokasin: nuancë lëkure gjysmë e errët',
    '👏🏿' => 'duar që duartrokasin: nuancë lëkure e errët',
    '🙌🏻' => 'njeri që ngre duart: nuancë lëkure e çelët',
    '🙌🏼' => 'njeri që ngre duart: nuancë lëkure gjysmë e çelët',
    '🙌🏽' => 'njeri që ngre duart: nuancë lëkure zeshkane',
    '🙌🏾' => 'njeri që ngre duart: nuancë lëkure gjysmë e errët',
    '🙌🏿' => 'njeri që ngre duart: nuancë lëkure e errët',
    '🫶🏻' => 'duar si zemër: nuancë lëkure e çelët',
    '🫶🏼' => 'duar si zemër: nuancë lëkure gjysmë e çelët',
    '🫶🏽' => 'duar si zemër: nuancë lëkure zeshkane',
    '🫶🏾' => 'duar si zemër: nuancë lëkure gjysmë e errët',
    '🫶🏿' => 'duar si zemër: nuancë lëkure e errët',
    '👐🏻' => 'duar të hapura: nuancë lëkure e çelët',
    '👐🏼' => 'duar të hapura: nuancë lëkure gjysmë e çelët',
    '👐🏽' => 'duar të hapura: nuancë lëkure zeshkane',
    '👐🏾' => 'duar të hapura: nuancë lëkure gjysmë e errët',
    '👐🏿' => 'duar të hapura: nuancë lëkure e errët',
    '🤲🏻' => 'pëllëmbë të bashkuara: nuancë lëkure e çelët',
    '🤲🏼' => 'pëllëmbë të bashkuara: nuancë lëkure gjysmë e çelët',
    '🤲🏽' => 'pëllëmbë të bashkuara: nuancë lëkure zeshkane',
    '🤲🏾' => 'pëllëmbë të bashkuara: nuancë lëkure gjysmë e errët',
    '🤲🏿' => 'pëllëmbë të bashkuara: nuancë lëkure e errët',
    '🤝🏻' => 'shtrëngim duarsh: nuancë lëkure e çelët',
    '🤝🏼' => 'shtrëngim duarsh: nuancë lëkure gjysmë e çelët',
    '🤝🏽' => 'shtrëngim duarsh: nuancë lëkure zeshkane',
    '🤝🏾' => 'shtrëngim duarsh: nuancë lëkure gjysmë e errët',
    '🤝🏿' => 'shtrëngim duarsh: nuancë lëkure e errët',
    '🙏🏻' => 'duar të mbledhura: nuancë lëkure e çelët',
    '🙏🏼' => 'duar të mbledhura: nuancë lëkure gjysmë e çelët',
    '🙏🏽' => 'duar të mbledhura: nuancë lëkure zeshkane',
    '🙏🏾' => 'duar të mbledhura: nuancë lëkure gjysmë e errët',
    '🙏🏿' => 'duar të mbledhura: nuancë lëkure e errët',
    '✍🏻' => 'dorë që shkruan: nuancë lëkure e çelët',
    '✍🏼' => 'dorë që shkruan: nuancë lëkure gjysmë e çelët',
    '✍🏽' => 'dorë që shkruan: nuancë lëkure zeshkane',
    '✍🏾' => 'dorë që shkruan: nuancë lëkure gjysmë e errët',
    '✍🏿' => 'dorë që shkruan: nuancë lëkure e errët',
    '💅🏻' => 'manikyr: nuancë lëkure e çelët',
    '💅🏼' => 'manikyr: nuancë lëkure gjysmë e çelët',
    '💅🏽' => 'manikyr: nuancë lëkure zeshkane',
    '💅🏾' => 'manikyr: nuancë lëkure gjysmë e errët',
    '💅🏿' => 'manikyr: nuancë lëkure e errët',
    '🤳🏻' => 'foto vetjake: nuancë lëkure e çelët',
    '🤳🏼' => 'foto vetjake: nuancë lëkure gjysmë e çelët',
    '🤳🏽' => 'foto vetjake: nuancë lëkure zeshkane',
    '🤳🏾' => 'foto vetjake: nuancë lëkure gjysmë e errët',
    '🤳🏿' => 'foto vetjake: nuancë lëkure e errët',
    '💪🏻' => 'muskuli dykrerësh i mbledhur: nuancë lëkure e çelët',
    '💪🏼' => 'muskuli dykrerësh i mbledhur: nuancë lëkure gjysmë e çelët',
    '💪🏽' => 'muskuli dykrerësh i mbledhur: nuancë lëkure zeshkane',
    '💪🏾' => 'muskuli dykrerësh i mbledhur: nuancë lëkure gjysmë e errët',
    '💪🏿' => 'muskuli dykrerësh i mbledhur: nuancë lëkure e errët',
    '🦵🏻' => 'këmbë: nuancë lëkure e çelët',
    '🦵🏼' => 'këmbë: nuancë lëkure gjysmë e çelët',
    '🦵🏽' => 'këmbë: nuancë lëkure zeshkane',
    '🦵🏾' => 'këmbë: nuancë lëkure gjysmë e errët',
    '🦵🏿' => 'këmbë: nuancë lëkure e errët',
    '🦶🏻' => 'shputë: nuancë lëkure e çelët',
    '🦶🏼' => 'shputë: nuancë lëkure gjysmë e çelët',
    '🦶🏽' => 'shputë: nuancë lëkure zeshkane',
    '🦶🏾' => 'shputë: nuancë lëkure gjysmë e errët',
    '🦶🏿' => 'shputë: nuancë lëkure e errët',
    '👂🏻' => 'vesh: nuancë lëkure e çelët',
    '👂🏼' => 'vesh: nuancë lëkure gjysmë e çelët',
    '👂🏽' => 'vesh: nuancë lëkure zeshkane',
    '👂🏾' => 'vesh: nuancë lëkure gjysmë e errët',
    '👂🏿' => 'vesh: nuancë lëkure e errët',
    '🦻🏻' => 'vesh me aparat dëgjimi: nuancë lëkure e çelët',
    '🦻🏼' => 'vesh me aparat dëgjimi: nuancë lëkure gjysmë e çelët',
    '🦻🏽' => 'vesh me aparat dëgjimi: nuancë lëkure zeshkane',
    '🦻🏾' => 'vesh me aparat dëgjimi: nuancë lëkure gjysmë e errët',
    '🦻🏿' => 'vesh me aparat dëgjimi: nuancë lëkure e errët',
    '👃🏻' => 'hundë: nuancë lëkure e çelët',
    '👃🏼' => 'hundë: nuancë lëkure gjysmë e çelët',
    '👃🏽' => 'hundë: nuancë lëkure zeshkane',
    '👃🏾' => 'hundë: nuancë lëkure gjysmë e errët',
    '👃🏿' => 'hundë: nuancë lëkure e errët',
    '👶🏻' => 'bebe: nuancë lëkure e çelët',
    '👶🏼' => 'bebe: nuancë lëkure gjysmë e çelët',
    '👶🏽' => 'bebe: nuancë lëkure zeshkane',
    '👶🏾' => 'bebe: nuancë lëkure gjysmë e errët',
    '👶🏿' => 'bebe: nuancë lëkure e errët',
    '🧒🏻' => 'fëmijë: nuancë lëkure e çelët',
    '🧒🏼' => 'fëmijë: nuancë lëkure gjysmë e çelët',
    '🧒🏽' => 'fëmijë: nuancë lëkure zeshkane',
    '🧒🏾' => 'fëmijë: nuancë lëkure gjysmë e errët',
    '🧒🏿' => 'fëmijë: nuancë lëkure e errët',
    '👦🏻' => 'djalë: nuancë lëkure e çelët',
    '👦🏼' => 'djalë: nuancë lëkure gjysmë e çelët',
    '👦🏽' => 'djalë: nuancë lëkure zeshkane',
    '👦🏾' => 'djalë: nuancë lëkure gjysmë e errët',
    '👦🏿' => 'djalë: nuancë lëkure e errët',
    '👧🏻' => 'vajzë: nuancë lëkure e çelët',
    '👧🏼' => 'vajzë: nuancë lëkure gjysmë e çelët',
    '👧🏽' => 'vajzë: nuancë lëkure zeshkane',
    '👧🏾' => 'vajzë: nuancë lëkure gjysmë e errët',
    '👧🏿' => 'vajzë: nuancë lëkure e errët',
    '🧑🏻' => 'fëmijë i rritur: nuancë lëkure e çelët',
    '🧑🏼' => 'fëmijë i rritur: nuancë lëkure gjysmë e çelët',
    '🧑🏽' => 'fëmijë i rritur: nuancë lëkure zeshkane',
    '🧑🏾' => 'fëmijë i rritur: nuancë lëkure gjysmë e errët',
    '🧑🏿' => 'fëmijë i rritur: nuancë lëkure e errët',
    '👱🏻' => 'njeri biond: nuancë lëkure e çelët',
    '👱🏼' => 'njeri biond: nuancë lëkure gjysmë e çelët',
    '👱🏽' => 'njeri biond: nuancë lëkure zeshkane',
    '👱🏾' => 'njeri biond: nuancë lëkure gjysmë e errët',
    '👱🏿' => 'njeri biond: nuancë lëkure e errët',
    '👨🏻' => 'burrë: nuancë lëkure e çelët',
    '👨🏼' => 'burrë: nuancë lëkure gjysmë e çelët',
    '👨🏽' => 'burrë: nuancë lëkure zeshkane',
    '👨🏾' => 'burrë: nuancë lëkure gjysmë e errët',
    '👨🏿' => 'burrë: nuancë lëkure e errët',
    '🧔🏻' => 'njëri me mjekër: nuancë lëkure e çelët',
    '🧔🏼' => 'njëri me mjekër: nuancë lëkure gjysmë e çelët',
    '🧔🏽' => 'njëri me mjekër: nuancë lëkure zeshkane',
    '🧔🏾' => 'njëri me mjekër: nuancë lëkure gjysmë e errët',
    '🧔🏿' => 'njëri me mjekër: nuancë lëkure e errët',
    '👩🏻' => 'grua: nuancë lëkure e çelët',
    '👩🏼' => 'grua: nuancë lëkure gjysmë e çelët',
    '👩🏽' => 'grua: nuancë lëkure zeshkane',
    '👩🏾' => 'grua: nuancë lëkure gjysmë e errët',
    '👩🏿' => 'grua: nuancë lëkure e errët',
    '🧓🏻' => 'i ri: nuancë lëkure e çelët',
    '🧓🏼' => 'i ri: nuancë lëkure gjysmë e çelët',
    '🧓🏽' => 'i ri: nuancë lëkure zeshkane',
    '🧓🏾' => 'i ri: nuancë lëkure gjysmë e errët',
    '🧓🏿' => 'i ri: nuancë lëkure e errët',
    '👴🏻' => 'burrë i moshuar: nuancë lëkure e çelët',
    '👴🏼' => 'burrë i moshuar: nuancë lëkure gjysmë e çelët',
    '👴🏽' => 'burrë i moshuar: nuancë lëkure zeshkane',
    '👴🏾' => 'burrë i moshuar: nuancë lëkure gjysmë e errët',
    '👴🏿' => 'burrë i moshuar: nuancë lëkure e errët',
    '👵🏻' => 'grua e moshuar: nuancë lëkure e çelët',
    '👵🏼' => 'grua e moshuar: nuancë lëkure gjysmë e çelët',
    '👵🏽' => 'grua e moshuar: nuancë lëkure zeshkane',
    '👵🏾' => 'grua e moshuar: nuancë lëkure gjysmë e errët',
    '👵🏿' => 'grua e moshuar: nuancë lëkure e errët',
    '🙍🏻' => 'njeri i vrenjtur: nuancë lëkure e çelët',
    '🙍🏼' => 'njeri i vrenjtur: nuancë lëkure gjysmë e çelët',
    '🙍🏽' => 'njeri i vrenjtur: nuancë lëkure zeshkane',
    '🙍🏾' => 'njeri i vrenjtur: nuancë lëkure gjysmë e errët',
    '🙍🏿' => 'njeri i vrenjtur: nuancë lëkure e errët',
    '🙎🏻' => 'njeri i pakënaqur: nuancë lëkure e çelët',
    '🙎🏼' => 'njeri i pakënaqur: nuancë lëkure gjysmë e çelët',
    '🙎🏽' => 'njeri i pakënaqur: nuancë lëkure zeshkane',
    '🙎🏾' => 'njeri i pakënaqur: nuancë lëkure gjysmë e errët',
    '🙎🏿' => 'njeri i pakënaqur: nuancë lëkure e errët',
    '🙅🏻' => '«jo» me gjeste: nuancë lëkure e çelët',
    '🙅🏼' => '«jo» me gjeste: nuancë lëkure gjysmë e çelët',
    '🙅🏽' => '«jo» me gjeste: nuancë lëkure zeshkane',
    '🙅🏾' => '«jo» me gjeste: nuancë lëkure gjysmë e errët',
    '🙅🏿' => '«jo» me gjeste: nuancë lëkure e errët',
    '🙆🏻' => '«në rregull» me gjeste: nuancë lëkure e çelët',
    '🙆🏼' => '«në rregull» me gjeste: nuancë lëkure gjysmë e çelët',
    '🙆🏽' => '«në rregull» me gjeste: nuancë lëkure zeshkane',
    '🙆🏾' => '«në rregull» me gjeste: nuancë lëkure gjysmë e errët',
    '🙆🏿' => '«në rregull» me gjeste: nuancë lëkure e errët',
    '💁🏻' => 'personi i sportelit të informacionit: nuancë lëkure e çelët',
    '💁🏼' => 'personi i sportelit të informacionit: nuancë lëkure gjysmë e çelët',
    '💁🏽' => 'personi i sportelit të informacionit: nuancë lëkure zeshkane',
    '💁🏾' => 'personi i sportelit të informacionit: nuancë lëkure gjysmë e errët',
    '💁🏿' => 'personi i sportelit të informacionit: nuancë lëkure e errët',
    '🙋🏻' => 'njeri i gëzuar që ngre dorën: nuancë lëkure e çelët',
    '🙋🏼' => 'njeri i gëzuar që ngre dorën: nuancë lëkure gjysmë e çelët',
    '🙋🏽' => 'njeri i gëzuar që ngre dorën: nuancë lëkure zeshkane',
    '🙋🏾' => 'njeri i gëzuar që ngre dorën: nuancë lëkure gjysmë e errët',
    '🙋🏿' => 'njeri i gëzuar që ngre dorën: nuancë lëkure e errët',
    '🧏🏻' => 'person i shurdhër: nuancë lëkure e çelët',
    '🧏🏼' => 'person i shurdhër: nuancë lëkure gjysmë e çelët',
    '🧏🏽' => 'person i shurdhër: nuancë lëkure zeshkane',
    '🧏🏾' => 'person i shurdhër: nuancë lëkure gjysmë e errët',
    '🧏🏿' => 'person i shurdhër: nuancë lëkure e errët',
    '🙇🏻' => 'njeri i përkulur: nuancë lëkure e çelët',
    '🙇🏼' => 'njeri i përkulur: nuancë lëkure gjysmë e çelët',
    '🙇🏽' => 'njeri i përkulur: nuancë lëkure zeshkane',
    '🙇🏾' => 'njeri i përkulur: nuancë lëkure gjysmë e errët',
    '🙇🏿' => 'njeri i përkulur: nuancë lëkure e errët',
    '🤦🏻' => 'fytyrë me pëllëmbë në ballë: nuancë lëkure e çelët',
    '🤦🏼' => 'fytyrë me pëllëmbë në ballë: nuancë lëkure gjysmë e çelët',
    '🤦🏽' => 'fytyrë me pëllëmbë në ballë: nuancë lëkure zeshkane',
    '🤦🏾' => 'fytyrë me pëllëmbë në ballë: nuancë lëkure gjysmë e errët',
    '🤦🏿' => 'fytyrë me pëllëmbë në ballë: nuancë lëkure e errët',
    '🤷🏻' => 'ngritje supesh: nuancë lëkure e çelët',
    '🤷🏼' => 'ngritje supesh: nuancë lëkure gjysmë e çelët',
    '🤷🏽' => 'ngritje supesh: nuancë lëkure zeshkane',
    '🤷🏾' => 'ngritje supesh: nuancë lëkure gjysmë e errët',
    '🤷🏿' => 'ngritje supesh: nuancë lëkure e errët',
    '👮🏻' => 'polic: nuancë lëkure e çelët',
    '👮🏼' => 'polic: nuancë lëkure gjysmë e çelët',
    '👮🏽' => 'polic: nuancë lëkure zeshkane',
    '👮🏾' => 'polic: nuancë lëkure gjysmë e errët',
    '👮🏿' => 'polic: nuancë lëkure e errët',
    '🕵🏻' => 'detektiv: nuancë lëkure e çelët',
    '🕵🏼' => 'detektiv: nuancë lëkure gjysmë e çelët',
    '🕵🏽' => 'detektiv: nuancë lëkure zeshkane',
    '🕵🏾' => 'detektiv: nuancë lëkure gjysmë e errët',
    '🕵🏿' => 'detektiv: nuancë lëkure e errët',
    '💂🏻' => 'ushtar garde: nuancë lëkure e çelët',
    '💂🏼' => 'ushtar garde: nuancë lëkure gjysmë e çelët',
    '💂🏽' => 'ushtar garde: nuancë lëkure zeshkane',
    '💂🏾' => 'ushtar garde: nuancë lëkure gjysmë e errët',
    '💂🏿' => 'ushtar garde: nuancë lëkure e errët',
    '🥷🏻' => 'ninxhë: nuancë lëkure e çelët',
    '🥷🏼' => 'ninxhë: nuancë lëkure gjysmë e çelët',
    '🥷🏽' => 'ninxhë: nuancë lëkure zeshkane',
    '🥷🏾' => 'ninxhë: nuancë lëkure gjysmë e errët',
    '🥷🏿' => 'ninxhë: nuancë lëkure e errët',
    '👷🏻' => 'punëtor ndërtimi: nuancë lëkure e çelët',
    '👷🏼' => 'punëtor ndërtimi: nuancë lëkure gjysmë e çelët',
    '👷🏽' => 'punëtor ndërtimi: nuancë lëkure zeshkane',
    '👷🏾' => 'punëtor ndërtimi: nuancë lëkure gjysmë e errët',
    '👷🏿' => 'punëtor ndërtimi: nuancë lëkure e errët',
    '🫅🏻' => 'person me kurorë: nuancë lëkure e çelët',
    '🫅🏼' => 'person me kurorë: nuancë lëkure gjysmë e çelët',
    '🫅🏽' => 'person me kurorë: nuancë lëkure zeshkane',
    '🫅🏾' => 'person me kurorë: nuancë lëkure gjysmë e errët',
    '🫅🏿' => 'person me kurorë: nuancë lëkure e errët',
    '🤴🏻' => 'princ: nuancë lëkure e çelët',
    '🤴🏼' => 'princ: nuancë lëkure gjysmë e çelët',
    '🤴🏽' => 'princ: nuancë lëkure zeshkane',
    '🤴🏾' => 'princ: nuancë lëkure gjysmë e errët',
    '🤴🏿' => 'princ: nuancë lëkure e errët',
    '👸🏻' => 'princeshë: nuancë lëkure e çelët',
    '👸🏼' => 'princeshë: nuancë lëkure gjysmë e çelët',
    '👸🏽' => 'princeshë: nuancë lëkure zeshkane',
    '👸🏾' => 'princeshë: nuancë lëkure gjysmë e errët',
    '👸🏿' => 'princeshë: nuancë lëkure e errët',
    '👳🏻' => 'burrë me çallmë: nuancë lëkure e çelët',
    '👳🏼' => 'burrë me çallmë: nuancë lëkure gjysmë e çelët',
    '👳🏽' => 'burrë me çallmë: nuancë lëkure zeshkane',
    '👳🏾' => 'burrë me çallmë: nuancë lëkure gjysmë e errët',
    '👳🏿' => 'burrë me çallmë: nuancë lëkure e errët',
    '👲🏻' => 'burrë me kapelë kineze: nuancë lëkure e çelët',
    '👲🏼' => 'burrë me kapelë kineze: nuancë lëkure gjysmë e çelët',
    '👲🏽' => 'burrë me kapelë kineze: nuancë lëkure zeshkane',
    '👲🏾' => 'burrë me kapelë kineze: nuancë lëkure gjysmë e errët',
    '👲🏿' => 'burrë me kapelë kineze: nuancë lëkure e errët',
    '🧕🏻' => 'grua me shami: nuancë lëkure e çelët',
    '🧕🏼' => 'grua me shami: nuancë lëkure gjysmë e çelët',
    '🧕🏽' => 'grua me shami: nuancë lëkure zeshkane',
    '🧕🏾' => 'grua me shami: nuancë lëkure gjysmë e errët',
    '🧕🏿' => 'grua me shami: nuancë lëkure e errët',
    '🤵🏻' => 'person me kostum: nuancë lëkure e çelët',
    '🤵🏼' => 'person me kostum: nuancë lëkure gjysmë e çelët',
    '🤵🏽' => 'person me kostum: nuancë lëkure zeshkane',
    '🤵🏾' => 'person me kostum: nuancë lëkure gjysmë e errët',
    '🤵🏿' => 'person me kostum: nuancë lëkure e errët',
    '👰🏻' => 'nuse me vello: nuancë lëkure e çelët',
    '👰🏼' => 'nuse me vello: nuancë lëkure gjysmë e çelët',
    '👰🏽' => 'nuse me vello: nuancë lëkure zeshkane',
    '👰🏾' => 'nuse me vello: nuancë lëkure gjysmë e errët',
    '👰🏿' => 'nuse me vello: nuancë lëkure e errët',
    '🤰🏻' => 'grua shtatzënë: nuancë lëkure e çelët',
    '🤰🏼' => 'grua shtatzënë: nuancë lëkure gjysmë e çelët',
    '🤰🏽' => 'grua shtatzënë: nuancë lëkure zeshkane',
    '🤰🏾' => 'grua shtatzënë: nuancë lëkure gjysmë e errët',
    '🤰🏿' => 'grua shtatzënë: nuancë lëkure e errët',
    '🫃🏻' => 'burrë shtatzënë: nuancë lëkure e çelët',
    '🫃🏼' => 'burrë shtatzënë: nuancë lëkure gjysmë e çelët',
    '🫃🏽' => 'burrë shtatzënë: nuancë lëkure zeshkane',
    '🫃🏾' => 'burrë shtatzënë: nuancë lëkure gjysmë e errët',
    '🫃🏿' => 'burrë shtatzënë: nuancë lëkure e errët',
    '🫄🏻' => 'person shtatzënë: nuancë lëkure e çelët',
    '🫄🏼' => 'person shtatzënë: nuancë lëkure gjysmë e çelët',
    '🫄🏽' => 'person shtatzënë: nuancë lëkure zeshkane',
    '🫄🏾' => 'person shtatzënë: nuancë lëkure gjysmë e errët',
    '🫄🏿' => 'person shtatzënë: nuancë lëkure e errët',
    '🤱🏻' => 'gji: nuancë lëkure e çelët',
    '🤱🏼' => 'gji: nuancë lëkure gjysmë e çelët',
    '🤱🏽' => 'gji: nuancë lëkure zeshkane',
    '🤱🏾' => 'gji: nuancë lëkure gjysmë e errët',
    '🤱🏿' => 'gji: nuancë lëkure e errët',
    '👼🏻' => 'engjëll bebe: nuancë lëkure e çelët',
    '👼🏼' => 'engjëll bebe: nuancë lëkure gjysmë e çelët',
    '👼🏽' => 'engjëll bebe: nuancë lëkure zeshkane',
    '👼🏾' => 'engjëll bebe: nuancë lëkure gjysmë e errët',
    '👼🏿' => 'engjëll bebe: nuancë lëkure e errët',
    '🎅🏻' => 'babagjyshi i krishtlindjes: nuancë lëkure e çelët',
    '🎅🏼' => 'babagjyshi i krishtlindjes: nuancë lëkure gjysmë e çelët',
    '🎅🏽' => 'babagjyshi i krishtlindjes: nuancë lëkure zeshkane',
    '🎅🏾' => 'babagjyshi i krishtlindjes: nuancë lëkure gjysmë e errët',
    '🎅🏿' => 'babagjyshi i krishtlindjes: nuancë lëkure e errët',
    '🤶🏻' => 'gjyshe krishtlindjesh: nuancë lëkure e çelët',
    '🤶🏼' => 'gjyshe krishtlindjesh: nuancë lëkure gjysmë e çelët',
    '🤶🏽' => 'gjyshe krishtlindjesh: nuancë lëkure zeshkane',
    '🤶🏾' => 'gjyshe krishtlindjesh: nuancë lëkure gjysmë e errët',
    '🤶🏿' => 'gjyshe krishtlindjesh: nuancë lëkure e errët',
    '🦸🏻' => 'superhero: nuancë lëkure e çelët',
    '🦸🏼' => 'superhero: nuancë lëkure gjysmë e çelët',
    '🦸🏽' => 'superhero: nuancë lëkure zeshkane',
    '🦸🏾' => 'superhero: nuancë lëkure gjysmë e errët',
    '🦸🏿' => 'superhero: nuancë lëkure e errët',
    '🦹🏻' => 'superzuzar: nuancë lëkure e çelët',
    '🦹🏼' => 'superzuzar: nuancë lëkure gjysmë e çelët',
    '🦹🏽' => 'superzuzar: nuancë lëkure zeshkane',
    '🦹🏾' => 'superzuzar: nuancë lëkure gjysmë e errët',
    '🦹🏿' => 'superzuzar: nuancë lëkure e errët',
    '🧙🏻' => 'magjistar: nuancë lëkure e çelët',
    '🧙🏼' => 'magjistar: nuancë lëkure gjysmë e çelët',
    '🧙🏽' => 'magjistar: nuancë lëkure zeshkane',
    '🧙🏾' => 'magjistar: nuancë lëkure gjysmë e errët',
    '🧙🏿' => 'magjistar: nuancë lëkure e errët',
    '🧚🏻' => 'zanë: nuancë lëkure e çelët',
    '🧚🏼' => 'zanë: nuancë lëkure gjysmë e çelët',
    '🧚🏽' => 'zanë: nuancë lëkure zeshkane',
    '🧚🏾' => 'zanë: nuancë lëkure gjysmë e errët',
    '🧚🏿' => 'zanë: nuancë lëkure e errët',
    '🧛🏻' => 'vampir: nuancë lëkure e çelët',
    '🧛🏼' => 'vampir: nuancë lëkure gjysmë e çelët',
    '🧛🏽' => 'vampir: nuancë lëkure zeshkane',
    '🧛🏾' => 'vampir: nuancë lëkure gjysmë e errët',
    '🧛🏿' => 'vampir: nuancë lëkure e errët',
    '🧜🏻' => 'sirenë deti: nuancë lëkure e çelët',
    '🧜🏼' => 'sirenë deti: nuancë lëkure gjysmë e çelët',
    '🧜🏽' => 'sirenë deti: nuancë lëkure zeshkane',
    '🧜🏾' => 'sirenë deti: nuancë lëkure gjysmë e errët',
    '🧜🏿' => 'sirenë deti: nuancë lëkure e errët',
    '🧝🏻' => 'kukudh: nuancë lëkure e çelët',
    '🧝🏼' => 'kukudh: nuancë lëkure gjysmë e çelët',
    '🧝🏽' => 'kukudh: nuancë lëkure zeshkane',
    '🧝🏾' => 'kukudh: nuancë lëkure gjysmë e errët',
    '🧝🏿' => 'kukudh: nuancë lëkure e errët',
    '💆🏻' => 'masazh fytyre: nuancë lëkure e çelët',
    '💆🏼' => 'masazh fytyre: nuancë lëkure gjysmë e çelët',
    '💆🏽' => 'masazh fytyre: nuancë lëkure zeshkane',
    '💆🏾' => 'masazh fytyre: nuancë lëkure gjysmë e errët',
    '💆🏿' => 'masazh fytyre: nuancë lëkure e errët',
    '💇🏻' => 'prerje flokësh: nuancë lëkure e çelët',
    '💇🏼' => 'prerje flokësh: nuancë lëkure gjysmë e çelët',
    '💇🏽' => 'prerje flokësh: nuancë lëkure zeshkane',
    '💇🏾' => 'prerje flokësh: nuancë lëkure gjysmë e errët',
    '💇🏿' => 'prerje flokësh: nuancë lëkure e errët',
    '🚶🏻' => 'këmbësor: nuancë lëkure e çelët',
    '🚶🏼' => 'këmbësor: nuancë lëkure gjysmë e çelët',
    '🚶🏽' => 'këmbësor: nuancë lëkure zeshkane',
    '🚶🏾' => 'këmbësor: nuancë lëkure gjysmë e errët',
    '🚶🏿' => 'këmbësor: nuancë lëkure e errët',
    '🧍🏻' => 'person në këmbë: nuancë lëkure e çelët',
    '🧍🏼' => 'person në këmbë: nuancë lëkure gjysmë e çelët',
    '🧍🏽' => 'person në këmbë: nuancë lëkure zeshkane',
    '🧍🏾' => 'person në këmbë: nuancë lëkure gjysmë e errët',
    '🧍🏿' => 'person në këmbë: nuancë lëkure e errët',
    '🧎🏻' => 'person në gjunjë: nuancë lëkure e çelët',
    '🧎🏼' => 'person në gjunjë: nuancë lëkure gjysmë e çelët',
    '🧎🏽' => 'person në gjunjë: nuancë lëkure zeshkane',
    '🧎🏾' => 'person në gjunjë: nuancë lëkure gjysmë e errët',
    '🧎🏿' => 'person në gjunjë: nuancë lëkure e errët',
    '🏃🏻' => 'vrapues: nuancë lëkure e çelët',
    '🏃🏼' => 'vrapues: nuancë lëkure gjysmë e çelët',
    '🏃🏽' => 'vrapues: nuancë lëkure zeshkane',
    '🏃🏾' => 'vrapues: nuancë lëkure gjysmë e errët',
    '🏃🏿' => 'vrapues: nuancë lëkure e errët',
    '💃🏻' => 'grua që kërcen: nuancë lëkure e çelët',
    '💃🏼' => 'grua që kërcen: nuancë lëkure gjysmë e çelët',
    '💃🏽' => 'grua që kërcen: nuancë lëkure zeshkane',
    '💃🏾' => 'grua që kërcen: nuancë lëkure gjysmë e errët',
    '💃🏿' => 'grua që kërcen: nuancë lëkure e errët',
    '🕺🏻' => 'burrë që kërcen: nuancë lëkure e çelët',
    '🕺🏼' => 'burrë që kërcen: nuancë lëkure gjysmë e çelët',
    '🕺🏽' => 'burrë që kërcen: nuancë lëkure zeshkane',
    '🕺🏾' => 'burrë që kërcen: nuancë lëkure gjysmë e errët',
    '🕺🏿' => 'burrë që kërcen: nuancë lëkure e errët',
    '🕴🏻' => 'burrë me kostum që ngrihet: nuancë lëkure e çelët',
    '🕴🏼' => 'burrë me kostum që ngrihet: nuancë lëkure gjysmë e çelët',
    '🕴🏽' => 'burrë me kostum që ngrihet: nuancë lëkure zeshkane',
    '🕴🏾' => 'burrë me kostum që ngrihet: nuancë lëkure gjysmë e errët',
    '🕴🏿' => 'burrë me kostum që ngrihet: nuancë lëkure e errët',
    '👯🏻' => 'gra duke festuar: nuancë lëkure e çelët',
    '👯🏼' => 'gra duke festuar: nuancë lëkure gjysmë e çelët',
    '👯🏽' => 'gra duke festuar: nuancë lëkure zeshkane',
    '👯🏾' => 'gra duke festuar: nuancë lëkure gjysmë e errët',
    '👯🏿' => 'gra duke festuar: nuancë lëkure e errët',
    '🧖🏻' => 'person në dhomë me avull: nuancë lëkure e çelët',
    '🧖🏼' => 'person në dhomë me avull: nuancë lëkure gjysmë e çelët',
    '🧖🏽' => 'person në dhomë me avull: nuancë lëkure zeshkane',
    '🧖🏾' => 'person në dhomë me avull: nuancë lëkure gjysmë e errët',
    '🧖🏿' => 'person në dhomë me avull: nuancë lëkure e errët',
    '🧗🏻' => 'person që ngjitet: nuancë lëkure e çelët',
    '🧗🏼' => 'person që ngjitet: nuancë lëkure gjysmë e çelët',
    '🧗🏽' => 'person që ngjitet: nuancë lëkure zeshkane',
    '🧗🏾' => 'person që ngjitet: nuancë lëkure gjysmë e errët',
    '🧗🏿' => 'person që ngjitet: nuancë lëkure e errët',
    '🏇🏻' => 'gara me kuaj: nuancë lëkure e çelët',
    '🏇🏼' => 'gara me kuaj: nuancë lëkure gjysmë e çelët',
    '🏇🏽' => 'gara me kuaj: nuancë lëkure zeshkane',
    '🏇🏾' => 'gara me kuaj: nuancë lëkure gjysmë e errët',
    '🏇🏿' => 'gara me kuaj: nuancë lëkure e errët',
    '🏂🏻' => 'sërfist në borë: nuancë lëkure e çelët',
    '🏂🏼' => 'sërfist në borë: nuancë lëkure gjysmë e çelët',
    '🏂🏽' => 'sërfist në borë: nuancë lëkure zeshkane',
    '🏂🏾' => 'sërfist në borë: nuancë lëkure gjysmë e errët',
    '🏂🏿' => 'sërfist në borë: nuancë lëkure e errët',
    '🏌🏻' => 'lojtar golfi: nuancë lëkure e çelët',
    '🏌🏼' => 'lojtar golfi: nuancë lëkure gjysmë e çelët',
    '🏌🏽' => 'lojtar golfi: nuancë lëkure zeshkane',
    '🏌🏾' => 'lojtar golfi: nuancë lëkure gjysmë e errët',
    '🏌🏿' => 'lojtar golfi: nuancë lëkure e errët',
    '🏄🏻' => 'sërfist: nuancë lëkure e çelët',
    '🏄🏼' => 'sërfist: nuancë lëkure gjysmë e çelët',
    '🏄🏽' => 'sërfist: nuancë lëkure zeshkane',
    '🏄🏾' => 'sërfist: nuancë lëkure gjysmë e errët',
    '🏄🏿' => 'sërfist: nuancë lëkure e errët',
    '🚣🏻' => 'varkë me rrema: nuancë lëkure e çelët',
    '🚣🏼' => 'varkë me rrema: nuancë lëkure gjysmë e çelët',
    '🚣🏽' => 'varkë me rrema: nuancë lëkure zeshkane',
    '🚣🏾' => 'varkë me rrema: nuancë lëkure gjysmë e errët',
    '🚣🏿' => 'varkë me rrema: nuancë lëkure e errët',
    '🏊🏻' => 'notar: nuancë lëkure e çelët',
    '🏊🏼' => 'notar: nuancë lëkure gjysmë e çelët',
    '🏊🏽' => 'notar: nuancë lëkure zeshkane',
    '🏊🏾' => 'notar: nuancë lëkure gjysmë e errët',
    '🏊🏿' => 'notar: nuancë lëkure e errët',
    '⛹🏻' => 'njeri me top: nuancë lëkure e çelët',
    '⛹🏼' => 'njeri me top: nuancë lëkure gjysmë e çelët',
    '⛹🏽' => 'njeri me top: nuancë lëkure zeshkane',
    '⛹🏾' => 'njeri me top: nuancë lëkure gjysmë e errët',
    '⛹🏿' => 'njeri me top: nuancë lëkure e errët',
    '🏋🏻' => 'peshëngritës: nuancë lëkure e çelët',
    '🏋🏼' => 'peshëngritës: nuancë lëkure gjysmë e çelët',
    '🏋🏽' => 'peshëngritës: nuancë lëkure zeshkane',
    '🏋🏾' => 'peshëngritës: nuancë lëkure gjysmë e errët',
    '🏋🏿' => 'peshëngritës: nuancë lëkure e errët',
    '🚴🏻' => 'çiklist: nuancë lëkure e çelët',
    '🚴🏼' => 'çiklist: nuancë lëkure gjysmë e çelët',
    '🚴🏽' => 'çiklist: nuancë lëkure zeshkane',
    '🚴🏾' => 'çiklist: nuancë lëkure gjysmë e errët',
    '🚴🏿' => 'çiklist: nuancë lëkure e errët',
    '🚵🏻' => 'çiklist në mal: nuancë lëkure e çelët',
    '🚵🏼' => 'çiklist në mal: nuancë lëkure gjysmë e çelët',
    '🚵🏽' => 'çiklist në mal: nuancë lëkure zeshkane',
    '🚵🏾' => 'çiklist në mal: nuancë lëkure gjysmë e errët',
    '🚵🏿' => 'çiklist në mal: nuancë lëkure e errët',
    '🤸🏻' => 'kapërdimje: nuancë lëkure e çelët',
    '🤸🏼' => 'kapërdimje: nuancë lëkure gjysmë e çelët',
    '🤸🏽' => 'kapërdimje: nuancë lëkure zeshkane',
    '🤸🏾' => 'kapërdimje: nuancë lëkure gjysmë e errët',
    '🤸🏿' => 'kapërdimje: nuancë lëkure e errët',
    '🤼🏻' => 'mundës: nuancë lëkure e çelët',
    '🤼🏼' => 'mundës: nuancë lëkure gjysmë e çelët',
    '🤼🏽' => 'mundës: nuancë lëkure zeshkane',
    '🤼🏾' => 'mundës: nuancë lëkure gjysmë e errët',
    '🤼🏿' => 'mundës: nuancë lëkure e errët',
    '🤽🏻' => 'vaterpolo: nuancë lëkure e çelët',
    '🤽🏼' => 'vaterpolo: nuancë lëkure gjysmë e çelët',
    '🤽🏽' => 'vaterpolo: nuancë lëkure zeshkane',
    '🤽🏾' => 'vaterpolo: nuancë lëkure gjysmë e errët',
    '🤽🏿' => 'vaterpolo: nuancë lëkure e errët',
    '🤾🏻' => 'hendboll: nuancë lëkure e çelët',
    '🤾🏼' => 'hendboll: nuancë lëkure gjysmë e çelët',
    '🤾🏽' => 'hendboll: nuancë lëkure zeshkane',
    '🤾🏾' => 'hendboll: nuancë lëkure gjysmë e errët',
    '🤾🏿' => 'hendboll: nuancë lëkure e errët',
    '🤹🏻' => 'zhongler: nuancë lëkure e çelët',
    '🤹🏼' => 'zhongler: nuancë lëkure gjysmë e çelët',
    '🤹🏽' => 'zhongler: nuancë lëkure zeshkane',
    '🤹🏾' => 'zhongler: nuancë lëkure gjysmë e errët',
    '🤹🏿' => 'zhongler: nuancë lëkure e errët',
    '🧘🏻' => 'person në pozicion ëndërrues: nuancë lëkure e çelët',
    '🧘🏼' => 'person në pozicion ëndërrues: nuancë lëkure gjysmë e çelët',
    '🧘🏽' => 'person në pozicion ëndërrues: nuancë lëkure zeshkane',
    '🧘🏾' => 'person në pozicion ëndërrues: nuancë lëkure gjysmë e errët',
    '🧘🏿' => 'person në pozicion ëndërrues: nuancë lëkure e errët',
    '🛀🏻' => 'njeri që lahet: nuancë lëkure e çelët',
    '🛀🏼' => 'njeri që lahet: nuancë lëkure gjysmë e çelët',
    '🛀🏽' => 'njeri që lahet: nuancë lëkure zeshkane',
    '🛀🏾' => 'njeri që lahet: nuancë lëkure gjysmë e errët',
    '🛀🏿' => 'njeri që lahet: nuancë lëkure e errët',
    '🛌🏻' => 'njeri në krevat: nuancë lëkure e çelët',
    '🛌🏼' => 'njeri në krevat: nuancë lëkure gjysmë e çelët',
    '🛌🏽' => 'njeri në krevat: nuancë lëkure zeshkane',
    '🛌🏾' => 'njeri në krevat: nuancë lëkure gjysmë e errët',
    '🛌🏿' => 'njeri në krevat: nuancë lëkure e errët',
    '👭🏻' => 'dy gra të kapura për duarsh: nuancë lëkure e çelët',
    '👭🏼' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e çelët',
    '👭🏽' => 'dy gra të kapura për duarsh: nuancë lëkure zeshkane',
    '👭🏾' => 'dy gra të kapura për duarsh: nuancë lëkure gjysmë e errët',
    '👭🏿' => 'dy gra të kapura për duarsh: nuancë lëkure e errët',
    '👫🏻' => 'burrë dhe grua që kapin duart: nuancë lëkure e çelët',
    '👫🏼' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e çelët',
    '👫🏽' => 'burrë dhe grua që kapin duart: nuancë lëkure zeshkane',
    '👫🏾' => 'burrë dhe grua që kapin duart: nuancë lëkure gjysmë e errët',
    '👫🏿' => 'burrë dhe grua që kapin duart: nuancë lëkure e errët',
    '👬🏻' => 'dy burra të kapur për duarsh: nuancë lëkure e çelët',
    '👬🏼' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e çelët',
    '👬🏽' => 'dy burra të kapur për duarsh: nuancë lëkure zeshkane',
    '👬🏾' => 'dy burra të kapur për duarsh: nuancë lëkure gjysmë e errët',
    '👬🏿' => 'dy burra të kapur për duarsh: nuancë lëkure e errët',
    '💏🏻' => 'puthje: nuancë lëkure e çelët',
    '💏🏼' => 'puthje: nuancë lëkure gjysmë e çelët',
    '💏🏽' => 'puthje: nuancë lëkure zeshkane',
    '💏🏾' => 'puthje: nuancë lëkure gjysmë e errët',
    '💏🏿' => 'puthje: nuancë lëkure e errët',
    '💑🏻' => 'çift me zemër: nuancë lëkure e çelët',
    '💑🏼' => 'çift me zemër: nuancë lëkure gjysmë e çelët',
    '💑🏽' => 'çift me zemër: nuancë lëkure zeshkane',
    '💑🏾' => 'çift me zemër: nuancë lëkure gjysmë e errët',
    '💑🏿' => 'çift me zemër: nuancë lëkure e errët',
    '🇦🇨' => 'flamur: Ishulli Asenshion',
    '🇦🇩' => 'flamur: Andorrë',
    '🇦🇪' => 'flamur: Emiratet e Bashkuara Arabe',
    '🇦🇫' => 'flamur: Afganistan',
    '🇦🇬' => 'flamur: Antigua e Barbuda',
    '🇦🇮' => 'flamur: Anguilë',
    '🇦🇱' => 'flamur: Shqipëri',
    '🇦🇲' => 'flamur: Armeni',
    '🇦🇴' => 'flamur: Angolë',
    '🇦🇶' => 'flamur: Antarktikë',
    '🇦🇷' => 'flamur: Argjentinë',
    '🇦🇸' => 'flamur: Samoa Amerikane',
    '🇦🇹' => 'flamur: Austri',
    '🇦🇺' => 'flamur: Australi',
    '🇦🇼' => 'flamur: Arubë',
    '🇦🇽' => 'flamur: Ishujt Alandë',
    '🇦🇿' => 'flamur: Azerbajxhan',
    '🇧🇦' => 'flamur: Bosnjë-Hercegovinë',
    '🇧🇧' => 'flamur: Barbados',
    '🇧🇩' => 'flamur: Bangladesh',
    '🇧🇪' => 'flamur: Belgjikë',
    '🇧🇫' => 'flamur: Burkina-Faso',
    '🇧🇬' => 'flamur: Bullgari',
    '🇧🇭' => 'flamur: Bahrejn',
    '🇧🇮' => 'flamur: Burundi',
    '🇧🇯' => 'flamur: Benin',
    '🇧🇱' => 'flamur: Sen-Bartelemi',
    '🇧🇲' => 'flamur: Bermude',
    '🇧🇳' => 'flamur: Brunei',
    '🇧🇴' => 'flamur: Bolivi',
    '🇧🇶' => 'flamur: Karaibet holandeze',
    '🇧🇷' => 'flamur: Brazil',
    '🇧🇸' => 'flamur: Bahama',
    '🇧🇹' => 'flamur: Butan',
    '🇧🇻' => 'flamur: Ishulli Bove',
    '🇧🇼' => 'flamur: Botsvanë',
    '🇧🇾' => 'flamur: Bjellorusi',
    '🇧🇿' => 'flamur: Belizë',
    '🇨🇦' => 'flamur: Kanada',
    '🇨🇨' => 'flamur: Ishujt Kokos',
    '🇨🇩' => 'flamur: Kongo-Kinshasa',
    '🇨🇫' => 'flamur: Republika e Afrikës Qendrore',
    '🇨🇬' => 'flamur: Kongo-Brazavilë',
    '🇨🇭' => 'flamur: Zvicër',
    '🇨🇮' => 'flamur: Côte d’Ivoire',
    '🇨🇰' => 'flamur: Ishujt Kuk',
    '🇨🇱' => 'flamur: Kili',
    '🇨🇲' => 'flamur: Kamerun',
    '🇨🇳' => 'flamur: Kinë',
    '🇨🇴' => 'flamur: Kolumbi',
    '🇨🇵' => 'flamur: Ishulli Klipërton',
    '🇨🇶' => 'flamur: Sark',
    '🇨🇷' => 'flamur: Kosta-Rikë',
    '🇨🇺' => 'flamur: Kubë',
    '🇨🇻' => 'flamur: Kepi i Gjelbër',
    '🇨🇼' => 'flamur: Kurasao',
    '🇨🇽' => 'flamur: Ishulli i Krishtlindjes',
    '🇨🇾' => 'flamur: Qipro',
    '🇨🇿' => 'flamur: Çeki',
    '🇩🇪' => 'flamur: Gjermani',
    '🇩🇬' => 'flamur: Diego-Garsia',
    '🇩🇯' => 'flamur: Xhibuti',
    '🇩🇰' => 'flamur: Danimarkë',
    '🇩🇲' => 'flamur: Dominikë',
    '🇩🇴' => 'flamur: Republika Dominikane',
    '🇩🇿' => 'flamur: Algjeri',
    '🇪🇦' => 'flamur: Theuta e Melila',
    '🇪🇨' => 'flamur: Ekuador',
    '🇪🇪' => 'flamur: Estoni',
    '🇪🇬' => 'flamur: Egjipt',
    '🇪🇭' => 'flamur: Saharaja Perëndimore',
    '🇪🇷' => 'flamur: Eritre',
    '🇪🇸' => 'flamur: Spanjë',
    '🇪🇹' => 'flamur: Etiopi',
    '🇪🇺' => 'flamur: Bashkimi Evropian',
    '🇫🇮' => 'flamur: Finlandë',
    '🇫🇯' => 'flamur: Fixhi',
    '🇫🇰' => 'flamur: Ishujt Falkland',
    '🇫🇲' => 'flamur: Mikronezi',
    '🇫🇴' => 'flamur: Ishujt Faroe',
    '🇫🇷' => 'flamur: Francë',
    '🇬🇦' => 'flamur: Gabon',
    '🇬🇧' => 'flamur: Mbretëria e Bashkuar',
    '🇬🇩' => 'flamur: Granadë',
    '🇬🇪' => 'flamur: Gjeorgji',
    '🇬🇫' => 'flamur: Guajana Franceze',
    '🇬🇬' => 'flamur: Gernsej',
    '🇬🇭' => 'flamur: Ganë',
    '🇬🇮' => 'flamur: Gjibraltar',
    '🇬🇱' => 'flamur: Grënlandë',
    '🇬🇲' => 'flamur: Gambi',
    '🇬🇳' => 'flamur: Guine',
    '🇬🇵' => 'flamur: Guadelupë',
    '🇬🇶' => 'flamur: Guineja Ekuatoriale',
    '🇬🇷' => 'flamur: Greqi',
    '🇬🇸' => 'flamur: Xhorxha Jugore dhe Ishujt Senduiçë të Jugut',
    '🇬🇹' => 'flamur: Guatemalë',
    '🇬🇺' => 'flamur: Guam',
    '🇬🇼' => 'flamur: Guine-Bisau',
    '🇬🇾' => 'flamur: Guajanë',
    '🇭🇰' => 'flamur: RPA i Hong-Kongut',
    '🇭🇲' => 'flamur: Ishujt Hërd e Mekdonald',
    '🇭🇳' => 'flamur: Honduras',
    '🇭🇷' => 'flamur: Kroaci',
    '🇭🇹' => 'flamur: Haiti',
    '🇭🇺' => 'flamur: Hungari',
    '🇮🇨' => 'flamur: Ishujt Kanarie',
    '🇮🇩' => 'flamur: Indonezi',
    '🇮🇪' => 'flamur: Irlandë',
    '🇮🇱' => 'flamur: Izrael',
    '🇮🇲' => 'flamur: Ishulli i Manit',
    '🇮🇳' => 'flamur: Indi',
    '🇮🇴' => 'flamur: Territori Britanik i Oqeanit Indian',
    '🇮🇶' => 'flamur: Irak',
    '🇮🇷' => 'flamur: Iran',
    '🇮🇸' => 'flamur: Islandë',
    '🇮🇹' => 'flamur: Itali',
    '🇯🇪' => 'flamur: Xhersej',
    '🇯🇲' => 'flamur: Xhamajkë',
    '🇯🇴' => 'flamur: Jordani',
    '🇯🇵' => 'flamur: Japoni',
    '🇰🇪' => 'flamur: Kenia',
    '🇰🇬' => 'flamur: Kirgizi',
    '🇰🇭' => 'flamur: Kamboxhia',
    '🇰🇮' => 'flamur: Kiribati',
    '🇰🇲' => 'flamur: Komore',
    '🇰🇳' => 'flamur: Shën-Kits dhe Nevis',
    '🇰🇵' => 'flamur: Kore e Veriut',
    '🇰🇷' => 'flamur: Kore e Jugut',
    '🇰🇼' => 'flamur: Kuvajt',
    '🇰🇾' => 'flamur: Ishujt Kajman',
    '🇰🇿' => 'flamur: Kazakistan',
    '🇱🇦' => 'flamur: Laos',
    '🇱🇧' => 'flamur: Liban',
    '🇱🇨' => 'flamur: Shën-Luçia',
    '🇱🇮' => 'flamur: Lihtenshtajn',
    '🇱🇰' => 'flamur: Sri-Lankë',
    '🇱🇷' => 'flamur: Liberi',
    '🇱🇸' => 'flamur: Lesoto',
    '🇱🇹' => 'flamur: Lituani',
    '🇱🇺' => 'flamur: Luksemburg',
    '🇱🇻' => 'flamur: Letoni',
    '🇱🇾' => 'flamur: Libi',
    '🇲🇦' => 'flamur: Marok',
    '🇲🇨' => 'flamur: Monako',
    '🇲🇩' => 'flamur: Moldavi',
    '🇲🇪' => 'flamur: Mal i Zi',
    '🇲🇫' => 'flamur: Sen-Marten',
    '🇲🇬' => 'flamur: Madagaskar',
    '🇲🇭' => 'flamur: Ishujt Marshall',
    '🇲🇰' => 'flamur: Maqedonia e Veriut',
    '🇲🇱' => 'flamur: Mali',
    '🇲🇲' => 'flamur: Mianmar (Burmë)',
    '🇲🇳' => 'flamur: Mongoli',
    '🇲🇴' => 'flamur: RPA i Makaos',
    '🇲🇵' => 'flamur: Ishujt e Marianës Veriore',
    '🇲🇶' => 'flamur: Martinikë',
    '🇲🇷' => 'flamur: Mauritani',
    '🇲🇸' => 'flamur: Montserat',
    '🇲🇹' => 'flamur: Maltë',
    '🇲🇺' => 'flamur: Mauritius',
    '🇲🇻' => 'flamur: Maldive',
    '🇲🇼' => 'flamur: Malavi',
    '🇲🇽' => 'flamur: Meksikë',
    '🇲🇾' => 'flamur: Malajzi',
    '🇲🇿' => 'flamur: Mozambik',
    '🇳🇦' => 'flamur: Namibi',
    '🇳🇨' => 'flamur: Kaledoni e Re',
    '🇳🇪' => 'flamur: Niger',
    '🇳🇫' => 'flamur: Ishulli Norfolk',
    '🇳🇬' => 'flamur: Nigeri',
    '🇳🇮' => 'flamur: Nikaragua',
    '🇳🇱' => 'flamur: Holandë',
    '🇳🇴' => 'flamur: Norvegji',
    '🇳🇵' => 'flamur: Nepal',
    '🇳🇷' => 'flamur: Nauru',
    '🇳🇺' => 'flamur: Niue',
    '🇳🇿' => 'flamur: Zelandë e Re',
    '🇴🇲' => 'flamur: Oman',
    '🇵🇦' => 'flamur: Panama',
    '🇵🇪' => 'flamur: Peru',
    '🇵🇫' => 'flamur: Polinezia Franceze',
    '🇵🇬' => 'flamur: Guineja e Re-Papua',
    '🇵🇭' => 'flamur: Filipine',
    '🇵🇰' => 'flamur: Pakistan',
    '🇵🇱' => 'flamur: Poloni',
    '🇵🇲' => 'flamur: Shën-Pier dhe Mikelon',
    '🇵🇳' => 'flamur: Ishujt Pitkern',
    '🇵🇷' => 'flamur: Porto-Riko',
    '🇵🇸' => 'flamur: Territoret Palestineze',
    '🇵🇹' => 'flamur: Portugali',
    '🇵🇼' => 'flamur: Palau',
    '🇵🇾' => 'flamur: Paraguai',
    '🇶🇦' => 'flamur: Katar',
    '🇷🇪' => 'flamur: Reunion',
    '🇷🇴' => 'flamur: Rumani',
    '🇷🇸' => 'flamur: Serbi',
    '🇷🇺' => 'flamur: Rusi',
    '🇷🇼' => 'flamur: Ruandë',
    '🇸🇦' => 'flamur: Arabi Saudite',
    '🇸🇧' => 'flamur: Ishujt Solomon',
    '🇸🇨' => 'flamur: Sejshelle',
    '🇸🇩' => 'flamur: Sudan',
    '🇸🇪' => 'flamur: Suedi',
    '🇸🇬' => 'flamur: Singapor',
    '🇸🇭' => 'flamur: Shën-Elenë',
    '🇸🇮' => 'flamur: Slloveni',
    '🇸🇯' => 'flamur: Svalbard e Jan-Majen',
    '🇸🇰' => 'flamur: Sllovaki',
    '🇸🇱' => 'flamur: Sierra-Leone',
    '🇸🇲' => 'flamur: San-Marino',
    '🇸🇳' => 'flamur: Senegal',
    '🇸🇴' => 'flamur: Somali',
    '🇸🇷' => 'flamur: Surinami',
    '🇸🇸' => 'flamur: Sudani i Jugut',
    '🇸🇹' => 'flamur: Sao-Tome e Principe',
    '🇸🇻' => 'flamur: Salvador',
    '🇸🇽' => 'flamur: Sint-Marten',
    '🇸🇾' => 'flamur: Siri',
    '🇸🇿' => 'flamur: Esvatini',
    '🇹🇦' => 'flamur: Tristan-da-Kuna',
    '🇹🇨' => 'flamur: Ishujt Turks dhe Kaikos',
    '🇹🇩' => 'flamur: Çad',
    '🇹🇫' => 'flamur: Territoret Jugore Franceze',
    '🇹🇬' => 'flamur: Togo',
    '🇹🇭' => 'flamur: Tajlandë',
    '🇹🇯' => 'flamur: Taxhikistan',
    '🇹🇰' => 'flamur: Tokelau',
    '🇹🇱' => 'flamur: Timor-Leste',
    '🇹🇲' => 'flamur: Turkmenistan',
    '🇹🇳' => 'flamur: Tunizi',
    '🇹🇴' => 'flamur: Tonga',
    '🇹🇷' => 'flamur: Turqi',
    '🇹🇹' => 'flamur: Trinidad e Tobago',
    '🇹🇻' => 'flamur: Tuvalu',
    '🇹🇼' => 'flamur: Tajvan',
    '🇹🇿' => 'flamur: Tanzani',
    '🇺🇦' => 'flamur: Ukrainë',
    '🇺🇬' => 'flamur: Ugandë',
    '🇺🇲' => 'flamur: Ishujt Periferikë të SHBA-së',
    '🇺🇳' => 'flamur: Organizata e Kombeve të Bashkuara',
    '🇺🇸' => 'flamur: SHBA',
    '🇺🇾' => 'flamur: Uruguai',
    '🇺🇿' => 'flamur: Uzbekistan',
    '🇻🇦' => 'flamur: Vatikan',
    '🇻🇨' => 'flamur: Shën-Vincent dhe Grenadine',
    '🇻🇪' => 'flamur: Venezuelë',
    '🇻🇬' => 'flamur: Ishujt e Virgjër Britanikë',
    '🇻🇮' => 'flamur: Ishujt e Virgjër të SHBA-së',
    '🇻🇳' => 'flamur: Vietnam',
    '🇻🇺' => 'flamur: Vanuatu',
    '🇼🇫' => 'flamur: Uollis e Futuna',
    '🇼🇸' => 'flamur: Samoa',
    '🇽🇰' => 'flamur: Kosovë',
    '🇾🇪' => 'flamur: Jemen',
    '🇾🇹' => 'flamur: Majotë',
    '🇿🇦' => 'flamur: Afrika e Jugut',
    '🇿🇲' => 'flamur: Zambi',
    '🇿🇼' => 'flamur: Zimbabve',
    '🔟' => 'tast: 10',
    '🏻' => 'nuancë lëkure e çelët',
    '🏼' => 'nuancë lëkure gjysmë e çelët',
    '🏽' => 'nuancë lëkure zeshkane',
    '🏾' => 'nuancë lëkure gjysmë e errët',
    '🏿' => 'nuancë lëkure e errët',
    '😀' => 'fytyrë e zgërdheshur',
    '😃' => 'fytyrë e qeshur me gojën e hapur',
    '😄' => 'fytyrë me gojë të hapur dhe sy të qeshur',
    '😁' => 'fytyrë e zgërdheshur me sy të qeshur',
    '😆' => 'fytyrë e qeshur me sy të mbyllur fort',
    '😅' => 'fytyrë e qeshur me djersë të ftohta',
    '🤣' => 'rrokullisje në dysheme nga e qeshura',
    '😂' => 'fytyrë me lot gëzimi',
    '🙂' => 'fytyrë pak e qeshur',
    '🙃' => 'fytyrë përmbys',
    '🫠' => 'fytyrë që shkrin',
    '😉' => 'fytyrë që shkel syrin',
    '😊' => 'fytyrë e qeshur me sy të qeshur',
    '😇' => 'fytyrë e qeshur me aureolë',
    '🥰' => 'fytyrë e qeshur me 3 zemra',
    '😍' => 'fytyrë e qeshur me sytë në formë zemre',
    '🤩' => 'fytyrë e magjepsur',
    '😘' => 'fytyrë që jep një puthje',
    '😗' => 'fytyrë që puth',
    '☺' => 'fytyrë e qeshur',
    '😚' => 'fytyrë që puth me sytë e mbyllur',
    '😙' => 'fytyrë që puth me sy të qeshur',
    '🥲' => 'fytyrë e qeshur me lot',
    '😋' => 'fytyrë duke shijuar ushqime të shijshme',
    '😛' => 'fytyrë me gjuhën jashtë',
    '😜' => 'fytyrë me gjuhën jashtë dhe syrin e shkelur',
    '🤪' => 'fytyrë e "çmendur"',
    '😝' => 'fytyrë me gjuhën jashtë dhe sytë të mbyllur',
    '🤑' => 'fytyrë me gjuhë si para',
    '🤗' => 'fytyrë që përqafon',
    '🤭' => 'fytyrë me dorën mbi gojë',
    '🫢' => 'fytyrë me sy të hapur dhe dorën mbi gojë',
    '🫣' => 'fytyrë me sy përgjues',
    '🤫' => 'fytyrë që bën "shtttt!"',
    '🤔' => 'fytyrë që mendohet',
    '🫡' => 'fytyrë përshëndetëse',
    '🤐' => 'fytyrë me gojën e kyçur',
    '🤨' => 'fytyrë me vetull të ngritur',
    '😐' => 'fytyrë neutrale',
    '😑' => 'fytyrë pa shprehje',
    '😶' => 'fytyrë pa gojë',
    '🫥' => 'fytyrë me vijë të pikëzuar',
    '😏' => 'fytyrë e vetëkënaqur',
    '😒' => 'fytyrë e pakënaqur',
    '🙄' => 'fytyrë me sytë që rrotullohen',
    '😬' => 'fytyrë e ngërdheshur',
    '🤥' => 'fytyrë mashtrimi',
    '🫨' => 'fytyrë që tundet',
    '😌' => 'fytyrë e lehtësuar',
    '😔' => 'fytyrë e menduar',
    '😪' => 'fytyrë e përgjumur',
    '🤤' => 'fytyrë jargëse',
    '😴' => 'fytyrë në gjumë',
    '🫩' => 'fytyrë me qeska poshtë syve',
    '😷' => 'fytyrë me maskë mjekësore',
    '🤒' => 'fytyrë me termometër',
    '🤕' => 'fytyrë me fashë në kokë',
    '🤢' => 'fytyrë neverie',
    '🤮' => 'fytyrë që vjell',
    '🤧' => 'fytyrë me teshtitje',
    '🥵' => 'fytyrëkuq',
    '🥶' => 'fytyrë e ngrirë',
    '🥴' => 'fytyrë e mjegulluar',
    '😵' => 'fytyrë e trullosur',
    '🤯' => 'kokë që shpërthen',
    '🤠' => 'fytyrë me kapele lopari',
    '🥳' => 'fytyrë festive',
    '🥸' => 'fytyrë e maskuar',
    '😎' => 'fytyrë e qeshur me syze dielli',
    '🤓' => 'fytyrë peshkopi',
    '🧐' => 'fytyrë me monokël',
    '😕' => 'fytyrë e hutuar',
    '🫤' => 'fytyrë me gojë diagonale',
    '😟' => 'fytyrë e shqetësuar',
    '🙁' => 'fytyrë pak e vrenjtur',
    '☹' => 'fytyrë e vrenjtur',
    '😮' => 'fytyrë me gojën e hapur',
    '😯' => 'fytyrë e heshtur',
    '😲' => 'fytyrë e çuditur',
    '😳' => 'fytyrë e skuqur',
    '🫪' => 'fytyrë e shtrembëruar',
    '🥺' => 'fytyrë qaramane',
    '🥹' => 'fytyrë që mezi mban lotët',
    '😦' => 'fytyrë e vrenjtur me gojë të hapur',
    '😧' => 'fytyrë në ankth',
    '😨' => 'fytyrë e frikësuar',
    '😰' => 'fytyrë me gojën e hapur dhe djersë të ftohta',
    '😥' => 'fytyrë e zhgënjyer por e lehtësuar',
    '😢' => 'fytyrë që qan',
    '😭' => 'fytyrë që qan me zë',
    '😱' => 'fytyrë duke ulëritur nga frika',
    '😖' => 'fytyrë e turbulluar',
    '😣' => 'fytyrë këmbëngulëse',
    '😞' => 'fytyrë e zhgënjyer',
    '😓' => 'fytyrë me djersë të ftohta',
    '😩' => 'fytyrë e rraskapitur',
    '😫' => 'fytyrë e lodhur',
    '🥱' => 'fytyrë e mërzitur',
    '😤' => 'fytyrë me tym nga hunda',
    '😡' => 'fytyrë me buzë të varura',
    '😠' => 'fytyrë e zemëruar',
    '🤬' => 'fytyrë me simbole në gojë',
    '😈' => 'fytyrë e qeshur me brirë',
    '👿' => 'djall',
    '💀' => 'kafkë',
    '☠' => 'kafkë me kocka të kryqëzuara',
    '💩' => 'kakë',
    '🤡' => 'fytyrë gaztori',
    '👹' => 'ogër',
    '👺' => 'goblin',
    '👻' => 'fantazmë',
    '👽' => 'alien',
    '👾' => 'përbindësh alien',
    '🤖' => 'fytyrë roboti',
    '😺' => 'fytyrë maceje e qeshur me gojën e hapur',
    '😸' => 'fytyrë maceje me sy të qeshur',
    '😹' => 'fytyrë maceje me lot gëzimi',
    '😻' => 'fytyrë maceje me sy në formë zemre',
    '😼' => 'fytyrë maceje me buzëqeshje të sforcuar',
    '😽' => 'fytyrë maceje që puth me sy të mbyllur',
    '🙀' => 'fytyrë maceje e rraskapitur',
    '😿' => 'fytyrë maceje që qan',
    '😾' => 'fytyrë maceje me buzë të varura',
    '🙈' => 'mos shiko gjëra të këqija',
    '🙉' => 'mos dëgjo fjalët e këqija',
    '🙊' => 'mos fol gjëra të këqija',
    '💌' => 'letër dashurie',
    '💘' => 'zemër me shigjetë',
    '💝' => 'zemër me fjongo',
    '💖' => 'zemër që shkëlqen',
    '💗' => 'zemër që rritet',
    '💓' => 'zemër që rreh',
    '💞' => 'zemra që rrotullohen',
    '💕' => 'dy zemra',
    '💟' => 'dekorim me zemër',
    '❣' => 'zbukurim i trashë me pikëçuditje me zemër',
    '💔' => 'zemër e thyer',
    '❤' => 'zemër e kuqe',
    '🩷' => 'zemër ngjyrë rozë',
    '🧡' => 'zemër ngjyrë portokalli',
    '💛' => 'zemër e verdhë',
    '💚' => 'zemër jeshile',
    '💙' => 'zemër blu',
    '🩵' => 'zemër ngjyrë blu e çelët',
    '💜' => 'zemër ngjyrë vjollcë',
    '🤎' => 'zemër në ngjyrë kafe',
    '🖤' => 'zemër e zezë',
    '🩶' => 'zemër ngjyrë gri',
    '🤍' => 'zemër e bardhë',
    '💋' => 'shenja e puthjes',
    '💯' => 'njëqind pikë',
    '💢' => 'simboli i zemërimit',
    '🫯' => 're grindjesh',
    '💥' => 'përplasje',
    '💫' => 'i trullosur',
    '💦' => 'pika djerse',
    '💨' => 'i vrullshëm',
    '🕳' => 'vrimë',
    '💬' => 'flluskë e të folurit',
    '🗨' => 'flluskë e të folurit majtas',
    '🗯' => 'flluskë zemërimi djathtas',
    '💭' => 'flluskë mendimi',
    '💤' => 'zzz',
    '👋' => 'dorë që tundet',
    '🤚' => 'dorë e ngritur nga ana e kurrizit',
    '🖐' => 'dorë e ngritur me gishtat të hapur',
    '✋' => 'dorë e ngritur',
    '🖖' => 'përshëndetja e vulkanit',
    '🫱' => 'dorë djathtas',
    '🫲' => 'dorë majtas',
    '🫳' => 'dorë me pëllëmbë poshtë',
    '🫴' => 'dorë me pëllëmbë lart',
    '🫷' => 'dorë që shtyn majtas',
    '🫸' => 'dorë që shtyn djathtas',
    '👌' => '«ok» me dorë',
    '🤌' => 'gishta të mbledhur',
    '🤏' => 'dorështrënguar',
    '✌' => 'dora e fitores',
    '🤞' => 'gishta të kryqëzuar',
    '🫰' => 'dorë me gishtin tregues dhe të madhin të kapur',
    '🤟' => 'gjesti "të dua"',
    '🤘' => 'shenja e brirëve',
    '🤙' => 'dorë me shenjën “më telefono”',
    '👈' => 'gishti tregues që tregon majtas',
    '👉' => 'gisht tregues i drejtuar djathtas',
    '👆' => 'gisht tregues i drejtuar lart',
    '🖕' => 'gishti i mesit',
    '👇' => 'gisht tregues i drejtuar poshtë',
    '☝' => 'gisht tregues që tregon lart',
    '🫵' => 'gishti tregues nga shikuesi',
    '👍' => 'gishti i madh lart',
    '👎' => 'gishti i madh poshtë',
    '✊' => 'grusht i ngritur',
    '👊' => 'grusht që vjen',
    '🤛' => 'grusht i majtë në anën e përparme',
    '🤜' => 'grusht i djathtë në anën e përparme',
    '👏' => 'duar që duartrokasin',
    '🙌' => 'njeri që ngre duart',
    '🫶' => 'duar si zemër',
    '👐' => 'duar të hapura',
    '🤲' => 'pëllëmbë të bashkuara',
    '🤝' => 'shtrëngim duarsh',
    '🙏' => 'duar të mbledhura',
    '✍' => 'dorë që shkruan',
    '💅' => 'manikyr',
    '🤳' => 'foto vetjake',
    '💪' => 'muskuli dykrerësh i mbledhur',
    '🦾' => 'krah mekanik',
    '🦿' => 'këmbë mekanike',
    '🦵' => 'këmbë',
    '🦶' => 'shputë',
    '👂' => 'vesh',
    '🦻' => 'vesh me aparat dëgjimi',
    '👃' => 'hundë',
    '🧠' => 'tru',
    '🫀' => 'zemër',
    '🫁' => 'mushkëri',
    '🦷' => 'dhëmb',
    '🦴' => 'eshtër',
    '👀' => 'sytë',
    '👁' => 'sy',
    '👅' => 'gjuhë',
    '👄' => 'gojë',
    '🫦' => 'kafshim buze',
    '👶' => 'bebe',
    '🧒' => 'fëmijë',
    '👦' => 'djalë',
    '👧' => 'vajzë',
    '🧑' => 'fëmijë i rritur',
    '👱' => 'njeri biond',
    '👨' => 'burrë',
    '🧔' => 'njëri me mjekër',
    '👩' => 'grua',
    '🧓' => 'i ri',
    '👴' => 'burrë i moshuar',
    '👵' => 'grua e moshuar',
    '🙍' => 'njeri i vrenjtur',
    '🙎' => 'njeri i pakënaqur',
    '🙅' => '«jo» me gjeste',
    '🙆' => '«në rregull» me gjeste',
    '💁' => 'personi i sportelit të informacionit',
    '🙋' => 'njeri i gëzuar që ngre dorën',
    '🧏' => 'person i shurdhër',
    '🙇' => 'njeri i përkulur',
    '🤦' => 'fytyrë me pëllëmbë në ballë',
    '🤷' => 'ngritje supesh',
    '👮' => 'polic',
    '🕵' => 'detektiv',
    '💂' => 'ushtar garde',
    '🥷' => 'ninxhë',
    '👷' => 'punëtor ndërtimi',
    '🫅' => 'person me kurorë',
    '🤴' => 'princ',
    '👸' => 'princeshë',
    '👳' => 'burrë me çallmë',
    '👲' => 'burrë me kapelë kineze',
    '🧕' => 'grua me shami',
    '🤵' => 'person me kostum',
    '👰' => 'nuse me vello',
    '🤰' => 'grua shtatzënë',
    '🫃' => 'burrë shtatzënë',
    '🫄' => 'person shtatzënë',
    '🤱' => 'gji',
    '👼' => 'engjëll bebe',
    '🎅' => 'babagjyshi i krishtlindjes',
    '🤶' => 'gjyshe krishtlindjesh',
    '🦸' => 'superhero',
    '🦹' => 'superzuzar',
    '🧙' => 'magjistar',
    '🧚' => 'zanë',
    '🧛' => 'vampir',
    '🧜' => 'sirenë deti',
    '🧝' => 'kukudh',
    '🧞' => 'xhind',
    '🧟' => 'zombi',
    '🧌' => 'xhuxh',
    '🫈' => 'krijesë leshatore',
    '💆' => 'masazh fytyre',
    '💇' => 'prerje flokësh',
    '🚶' => 'këmbësor',
    '🧍' => 'person në këmbë',
    '🧎' => 'person në gjunjë',
    '🏃' => 'vrapues',
    '💃' => 'grua që kërcen',
    '🕺' => 'burrë që kërcen',
    '🕴' => 'burrë me kostum që ngrihet',
    '👯' => 'gra duke festuar',
    '🧖' => 'person në dhomë me avull',
    '🧗' => 'person që ngjitet',
    '🤺' => 'skermist',
    '🏇' => 'gara me kuaj',
    '⛷' => 'skiator',
    '🏂' => 'sërfist në borë',
    '🏌' => 'lojtar golfi',
    '🏄' => 'sërfist',
    '🚣' => 'varkë me rrema',
    '🏊' => 'notar',
    '⛹' => 'njeri me top',
    '🏋' => 'peshëngritës',
    '🚴' => 'çiklist',
    '🚵' => 'çiklist në mal',
    '🤸' => 'kapërdimje',
    '🤼' => 'mundës',
    '🤽' => 'vaterpolo',
    '🤾' => 'hendboll',
    '🤹' => 'zhongler',
    '🧘' => 'person në pozicion ëndërrues',
    '🛀' => 'njeri që lahet',
    '🛌' => 'njeri në krevat',
    '👭' => 'dy gra të kapura për duarsh',
    '👫' => 'burrë dhe grua që kapin duart',
    '👬' => 'dy burra të kapur për duarsh',
    '💏' => 'puthje',
    '💑' => 'çift me zemër',
    '🗣' => 'kokë që flet',
    '👤' => 'bust siluetë',
    '👥' => 'buste siluetë',
    '🫂' => 'njerëz që përqafohen',
    '👪' => 'familje',
    '👣' => 'gjurmë këmbësh',
    '🫆' => 'gjurmë gishti',
    '🦰' => 'flokë të kuqe',
    '🦱' => 'flokë kaçurrele',
    '🦳' => 'flokë të bardha',
    '🦲' => 'qeros',
    '🐵' => 'fytyrë majmuni',
    '🐒' => 'majmun',
    '🦍' => 'gorillë',
    '🦧' => 'orangutang',
    '🐶' => 'fytyrë qeni',
    '🐕' => 'qen',
    '🦮' => 'qen udhëzues',
    '🐩' => 'pudël',
    '🐺' => 'fytyrë ujku',
    '🦊' => 'fytyrë dhelpre',
    '🦝' => 'rakun',
    '🐱' => 'fytyrë maceje',
    '🐈' => 'mace',
    '🦁' => 'fytyrë luani',
    '🐯' => 'fytyrë tigri',
    '🐅' => 'tigër',
    '🐆' => 'leopard',
    '🐴' => 'fytyrë kali',
    '🫎' => 'dre brilopatë',
    '🫏' => 'gomar',
    '🐎' => 'kalë',
    '🦄' => 'fytyrë njëbrirëshi',
    '🦓' => 'zebër',
    '🦌' => 'dre',
    '🦬' => 'bizon',
    '🐮' => 'fytyrë lope',
    '🐂' => 'dem',
    '🐃' => 'buall indian',
    '🐄' => 'lopë',
    '🐷' => 'fytyrë derri',
    '🐖' => 'derr',
    '🐗' => 'derr i egër',
    '🐽' => 'hundë derri',
    '🐏' => 'dash',
    '🐑' => 'dele',
    '🐐' => 'dhi',
    '🐪' => 'gamile',
    '🐫' => 'gamile me dy gunga',
    '🦙' => 'lamë',
    '🦒' => 'gjirafë',
    '🐘' => 'elefant',
    '🦣' => 'mamuth',
    '🦏' => 'rinoqeront',
    '🦛' => 'hipopotam',
    '🐭' => 'fytyrë miu',
    '🐁' => 'mi shtëpiak',
    '🐀' => 'mi',
    '🐹' => 'fytyrë brejtësi',
    '🐰' => 'fytyrë lepuri',
    '🐇' => 'lepur',
    '🐿' => 'ketër',
    '🦫' => 'kastor',
    '🦔' => 'iriq',
    '🦇' => 'lakuriq nate',
    '🐻' => 'fytyrë ariu',
    '🐨' => 'koalë',
    '🐼' => 'fytyrë pande',
    '🦥' => 'majmun i Bengalit',
    '🦦' => 'vidër',
    '🦨' => 'qelbës',
    '🦘' => 'kangur',
    '🦡' => 'vjedull',
    '🐾' => 'gjurmë putrash',
    '🦃' => 'gjel deti',
    '🐔' => 'pulë',
    '🐓' => 'gjel',
    '🐣' => 'zog në çelje',
    '🐤' => 'zog pule',
    '🐥' => 'zog pule përballë',
    '🐦' => 'zog',
    '🐧' => 'pinguin',
    '🕊' => 'pëllumb',
    '🦅' => 'shqiponjë',
    '🦆' => 'rosak',
    '🦢' => 'mjellmë',
    '🦉' => 'buf',
    '🦤' => 'zog i rëndë',
    '🪶' => 'pendë',
    '🦩' => 'flamingo',
    '🦚' => 'pallua',
    '🦜' => 'papagall',
    '🪽' => 'krah',
    '🪿' => 'patë',
    '🐸' => 'fytyrë bretkose',
    '🐊' => 'krokodil',
    '🐢' => 'breshkë',
    '🦎' => 'hardhucë',
    '🐍' => 'gjarpër',
    '🐲' => 'fytyrë dragoi',
    '🐉' => 'dragua',
    '🦕' => 'sauropod',
    '🦖' => 'tiranozaur-reks',
    '🐳' => 'balenë që hedh ujë',
    '🐋' => 'balenë',
    '🐬' => 'delfin',
    '🫍' => 'orkë',
    '🦭' => 'fokë',
    '🐟' => 'peshk',
    '🐠' => 'peshk tropikal',
    '🐡' => 'peshk tullumbace',
    '🦈' => 'peshkaqen',
    '🐙' => 'oktapod',
    '🐚' => 'guaskë spirale',
    '🪸' => 'koral',
    '🪼' => 'kandil deti',
    '🦀' => 'gaforre',
    '🦞' => 'karavidhe',
    '🦐' => 'karkalec deti',
    '🦑' => 'kallamar',
    '🦪' => 'gocë deti',
    '🐌' => 'kërmill',
    '🦋' => 'flutur',
    '🐛' => 'shumëkëmbësh',
    '🐜' => 'milingonë',
    '🐝' => 'bletë',
    '🪲' => 'brumbull',
    '🐞' => 'mollëkuqe',
    '🦗' => 'karkalec',
    '🪳' => 'kacabu',
    '🕷' => 'merimangë',
    '🕸' => 'rrjetë merimange',
    '🦂' => 'akrep',
    '🦟' => 'mushkonjë',
    '🪰' => 'mizë',
    '🪱' => 'krimb',
    '🦠' => 'mikrob',
    '💐' => 'buqetë',
    '🌸' => 'lule qershie',
    '💮' => 'lule e bardhë',
    '🪷' => 'zambak uji',
    '🏵' => 'rozetë',
    '🌹' => 'trëndafil',
    '🥀' => 'lule e vyshkur',
    '🌺' => 'hibisk',
    '🌻' => 'luledielli',
    '🌼' => 'lule',
    '🌷' => 'tulipan',
    '🪻' => 'zymbyl',
    '🌱' => 'filiz',
    '🪴' => 'bimë në vazo',
    '🌲' => 'gjelbërim i përjetshëm',
    '🌳' => 'pemë gjetherënëse',
    '🌴' => 'palmë',
    '🌵' => 'kaktus',
    '🌾' => 'kapule orizi',
    '🌿' => 'erëza',
    '☘' => 'tërfil',
    '🍀' => 'tërfil me katër fletë',
    '🍁' => 'gjethe panje',
    '🍂' => 'gjethe e rënë',
    '🍃' => 'gjethe që fluturon në erë',
    '🪹' => 'fole bosh',
    '🪺' => 'fole me vezë',
    '🍄' => 'kërpudhë',
    '🪾' => 'pemë e zhveshur',
    '🍇' => 'rrush',
    '🍈' => 'pjepër',
    '🍉' => 'shalqi',
    '🍊' => 'mandarinë',
    '🍋' => 'limon',
    '🍌' => 'banane',
    '🍍' => 'ananas',
    '🥭' => 'mango',
    '🍎' => 'mollë e kuqe',
    '🍏' => 'mollë jeshile',
    '🍐' => 'dardhë',
    '🍑' => 'pjeshkë',
    '🍒' => 'qershi',
    '🍓' => 'luleshtrydhe',
    '🫐' => 'boronica',
    '🥝' => 'kivi',
    '🍅' => 'domate',
    '🫒' => 'ulli',
    '🥥' => 'arrë kokosi',
    '🥑' => 'avokado',
    '🍆' => 'patëllxhan',
    '🥔' => 'patate',
    '🥕' => 'karotë',
    '🌽' => 'kalli misri',
    '🌶' => 'spec djegës',
    '🫑' => 'patëllxhanë për mbushje',
    '🥒' => 'kastravec',
    '🥬' => 'sallatë',
    '🥦' => 'brokoli',
    '🧄' => 'hudhër',
    '🧅' => 'qepë',
    '🥜' => 'kikirik',
    '🫘' => 'fasule',
    '🌰' => 'lajthi',
    '🫚' => 'rrënjë xhenxhefili',
    '🫛' => 'bizele bishtajë',
    '🫜' => 'perime me rrënjë',
    '🍞' => 'bukë',
    '🥐' => 'kroasant',
    '🥖' => 'bagetë',
    '🫓' => 'bukë e sheshtë',
    '🥨' => 'gjevrek i përdredhur',
    '🥯' => 'gjevrek',
    '🥞' => 'petulla',
    '🧇' => 'vafer',
    '🧀' => 'copë djathi',
    '🍖' => 'mish me kockë',
    '🍗' => 'kofshë pule',
    '🥩' => 'mish tul',
    '🥓' => 'pastërma',
    '🍔' => 'hamburger',
    '🍟' => 'patate të skuqura',
    '🍕' => 'pica',
    '🌭' => 'hot-dog',
    '🥪' => 'sandviç',
    '🌮' => 'tako',
    '🌯' => 'burrito',
    '🫔' => 'tamale',
    '🥙' => 'bukë e sheshtë me mbushje',
    '🧆' => 'qofte falafel',
    '🥚' => 'vezë',
    '🍳' => 'gatim',
    '🥘' => 'tigan me ushqim',
    '🍲' => 'kupë me ushqime',
    '🫕' => 'fondue',
    '🥣' => 'tas me lugë',
    '🥗' => 'sallatë e gjelbër',
    '🍿' => 'kokoshka',
    '🧈' => 'gjalpë',
    '🧂' => 'kripë',
    '🥫' => 'ushqim i konservuar',
    '🍱' => 'kuti bento',
    '🍘' => 'biskotë me oriz',
    '🍙' => 'topth orizi',
    '🍚' => 'oriz i gatuar',
    '🍛' => 'oriz me mish me erëza',
    '🍜' => 'kupë me avull',
    '🍝' => 'spageti',
    '🍠' => 'patate e ëmbël e pjekur',
    '🍢' => 'oden',
    '🍣' => 'sushi',
    '🍤' => 'karkalec i skuqur',
    '🍥' => 'kroketë me peshk me spirale',
    '🥮' => 'kek-hënë',
    '🍡' => 'dango',
    '🥟' => 'top brumi i mbushur',
    '🥠' => 'biskotë fati',
    '🥡' => 'kuti për ushqim me vete',
    '🍦' => 'akullore e butë',
    '🍧' => 'akull i grirë',
    '🍨' => 'akullore',
    '🍩' => 'petull e ëmbël',
    '🍪' => 'biskotë',
    '🎂' => 'tortë ditëlindjeje',
    '🍰' => 'kek i shkrifët',
    '🧁' => 'kek i vogël',
    '🥧' => 'byrek',
    '🍫' => 'çokollatë',
    '🍬' => 'ëmbëlsirë',
    '🍭' => 'lëpirëse',
    '🍮' => 'krem-karamel',
    '🍯' => 'vazo me mjaltë',
    '🍼' => 'shishe për bebe',
    '🥛' => 'gotë me qumësht',
    '☕' => 'pije e nxehtë',
    '🫖' => 'çajnik',
    '🍵' => 'filxhan çaji pa bisht',
    '🍶' => 'sake',
    '🍾' => 'shishe me tapë që kërcet',
    '🍷' => 'gotë vere',
    '🍸' => 'gotë kokteji',
    '🍹' => 'pije tropikale',
    '🍺' => 'krikëll birre',
    '🍻' => 'kriklla birre që trokiten',
    '🥂' => 'trokitje gotash',
    '🥃' => 'gotë',
    '🫗' => 'hedhje lëngu',
    '🥤' => 'gotë me pipë',
    '🧋' => 'çaj me flluska',
    '🧃' => 'kuti pijeje',
    '🧉' => 'kimarrao',
    '🧊' => 'kub akulli',
    '🥢' => 'shkopinj (për të ngrënë)',
    '🍽' => 'pirun dhe thikë me pjatë',
    '🍴' => 'pirun dhe thikë',
    '🥄' => 'lugë',
    '🔪' => 'thikë kuzhine',
    '🫙' => 'kanë',
    '🏺' => 'amforë',
    '🌍' => 'globi që tregon Evropën dhe Afrikën',
    '🌎' => 'globi që tregon Amerikat',
    '🌏' => 'globi që tregon Azinë dhe Australinë',
    '🌐' => 'globi me meridianë',
    '🗺' => 'harta e botës',
    '🗾' => 'harta e Japonisë',
    '🧭' => 'busull',
    '🏔' => 'mal i mbuluar me borë në majë',
    '⛰' => 'mal',
    '🛘' => 'rrëshqitje dheu',
    '🌋' => 'vullkan',
    '🗻' => 'mali i Fuxhit',
    '🏕' => 'kamp',
    '🏖' => 'plazh me çadër',
    '🏜' => 'shkretëtirë',
    '🏝' => 'ishull shkretëtirë',
    '🏞' => 'park kombëtar',
    '🏟' => 'stadium',
    '🏛' => 'ndërtesë klasike',
    '🏗' => 'ndërtim ndërtese',
    '🧱' => 'tullë',
    '🪨' => 'shkëmb',
    '🪵' => 'dru',
    '🛖' => 'kasolle',
    '🏘' => 'shtëpitë',
    '🏚' => 'shtëpi e braktisur',
    '🏠' => 'shtëpi',
    '🏡' => 'shtëpi me kopsht',
    '🏢' => 'ndërtesë zyrash',
    '🏣' => 'zyrë postare japoneze',
    '🏤' => 'zyrë postare',
    '🏥' => 'spital',
    '🏦' => 'bankë',
    '🏨' => 'hotel',
    '🏩' => 'hotel për dashuri',
    '🏪' => 'minimarket',
    '🏫' => 'shkollë',
    '🏬' => 'dyqan me shumë njësi',
    '🏭' => 'fabrikë',
    '🏯' => 'kështjellë japoneze',
    '🏰' => 'kështjellë',
    '💒' => 'dasmë',
    '🗼' => 'kullë në Tokio',
    '🗽' => 'Statuja e Lirisë',
    '⛪' => 'kishë',
    '🕌' => 'xhami',
    '🛕' => 'tempull indian',
    '🕍' => 'sinagogë',
    '⛩' => 'faltore shinto',
    '🕋' => 'qabeja',
    '⛲' => 'shatërvan',
    '⛺' => 'tendë',
    '🌁' => 'me mjegull',
    '🌃' => 'natë me yje',
    '🏙' => 'panoramë qyteti',
    '🌄' => 'lindja e diellit mbi male',
    '🌅' => 'lindja e diellit',
    '🌆' => 'panoramë qyteti në muzg',
    '🌇' => 'perëndim dielli',
    '🌉' => 'urë natën',
    '♨' => 'llixhë',
    '🎠' => 'kalë në karusel',
    '🛝' => 'rrëshqitëse këndi lojërash',
    '🎡' => 'rrotullame',
    '🎢' => 'tren lojërash me valëzim',
    '💈' => 'shenja e berberit',
    '🎪' => 'tendë cirku',
    '🚂' => 'lokomotivë',
    '🚃' => 'vagon',
    '🚄' => 'tren me shpejtësi të madhe',
    '🚅' => 'tren-plumb',
    '🚆' => 'tren',
    '🚇' => 'metro',
    '🚈' => 'metro e lehtë',
    '🚉' => 'stacion',
    '🚊' => 'tramvaj',
    '🚝' => 'hekurudhë me një shinë',
    '🚞' => 'hekurudhë në mal',
    '🚋' => 'vagon tramvaji',
    '🚌' => 'autobus',
    '🚍' => 'autobus që vjen',
    '🚎' => 'filobus',
    '🚐' => 'autobus i vogël',
    '🚑' => 'ambulancë',
    '🚒' => 'zjarrfikëse',
    '🚓' => 'makinë policie',
    '🚔' => 'makinë policie që vjen',
    '🚕' => 'taksi',
    '🚖' => 'taksi që vjen',
    '🚗' => 'automobil',
    '🚘' => 'automobil që vjen',
    '🚙' => 'makinë sportive për kamp',
    '🛻' => 'karroatrec',
    '🚚' => 'kamion transporti',
    '🚛' => 'kamion me rimorkio',
    '🚜' => 'traktor',
    '🏎' => 'makinë garash',
    '🏍' => 'motoçikletë',
    '🛵' => 'skuter',
    '🦽' => 'karrocë invalidi manuale',
    '🦼' => 'karrocë invalidi me motor',
    '🛺' => 'autorikshë',
    '🚲' => 'biçikletë',
    '🛴' => 'ballëz dyrrotëshe',
    '🛹' => 'dërrasë rrëshqitëse',
    '🛼' => 'rollerskejt',
    '🚏' => 'stacion autobusi',
    '🛣' => 'autostradë',
    '🛤' => 'shina',
    '🛢' => 'fuçi karburanti',
    '⛽' => 'pompë karburanti',
    '🛞' => 'rrotë',
    '🚨' => 'dritat sinjalizuese të policisë',
    '🚥' => 'semafor horizontal',
    '🚦' => 'semafor vertikal',
    '🛑' => 'shenja “ndalo”',
    '🚧' => 'ndërtim',
    '⚓' => 'spirancë',
    '🛟' => 'bovë rrethore',
    '⛵' => 'varkë me vela',
    '🛶' => 'kaike',
    '🚤' => 'motobarkë e shpejtë',
    '🛳' => 'anije udhëtarësh',
    '⛴' => 'traget',
    '🛥' => 'motobarkë',
    '🚢' => 'anije',
    '✈' => 'aeroplan',
    '🛩' => 'aeroplan i vogël',
    '🛫' => 'nisje aeroplani',
    '🛬' => 'mbërritje aeroplani',
    '🪂' => 'parashutë',
    '💺' => 'ndenjëse',
    '🚁' => 'helikopter',
    '🚟' => 'hekurudhë me varje',
    '🚠' => 'teleferik në mal',
    '🚡' => 'teleferik',
    '🛰' => 'satelit',
    '🚀' => 'raketë',
    '🛸' => 'disk fluturues',
    '🛎' => 'zile shërbimi',
    '🧳' => 'bagazh',
    '⌛' => 'orë me rërë',
    '⏳' => 'orë me rërë që rrjedh',
    '⌚' => 'orë',
    '⏰' => 'orë me zile',
    '⏱' => 'kronometër',
    '⏲' => 'kohëmatës',
    '🕰' => 'orë oxhaku',
    '🕛' => 'ora dymbëdhjetë',
    '🕧' => 'dymbëdhjetë e gjysmë',
    '🕐' => 'ora një',
    '🕜' => 'një e gjysmë',
    '🕑' => 'ora dy',
    '🕝' => 'dy e gjysmë',
    '🕒' => 'ora tre',
    '🕞' => 'tre e gjysmë',
    '🕓' => 'ora katër',
    '🕟' => 'katër e gjysmë',
    '🕔' => 'ora pesë',
    '🕠' => 'pesë e gjysmë',
    '🕕' => 'ora gjashtë',
    '🕡' => 'gjashtë e gjysmë',
    '🕖' => 'ora shtatë',
    '🕢' => 'shtatë e gjysmë',
    '🕗' => 'ora tetë',
    '🕣' => 'tetë e gjysmë',
    '🕘' => 'ora nëntë',
    '🕤' => 'nëntë e gjysmë',
    '🕙' => 'ora dhjetë',
    '🕥' => 'dhjetë e gjysmë',
    '🕚' => 'ora njëmbëdhjetë',
    '🕦' => 'njëmbëdhjetë e gjysmë',
    '🌑' => 'hëna e re',
    '🌒' => 'hënë e re më pak se gjysma',
    '🌓' => 'hëna e çerekut të parë',
    '🌔' => 'hënë e re më shumë se gjysma',
    '🌕' => 'hënë e plotë',
    '🌖' => 'hënë e ngrënë më shumë se gjysma',
    '🌗' => 'hëna e çerekut të fundit',
    '🌘' => 'hënë e ngrënë më pak se gjysma',
    '🌙' => 'hënë e ngrënë',
    '🌚' => 'fytyra e hënës së re',
    '🌛' => 'hëna e çerekut të parë me fytyrë',
    '🌜' => 'hëna e çerekut të fundit me fytyrë',
    '🌡' => 'termometër',
    '☀' => 'diell',
    '🌝' => 'hënë e plotë me fytyrë',
    '🌞' => 'diell me fytyrë',
    '🪐' => 'planet me unazë',
    '⭐' => 'yll mesatar i bardhë',
    '🌟' => 'yll që shkëlqen',
    '🌠' => 'yll që bie',
    '🌌' => 'rruga e qumështit',
    '☁' => 're',
    '⛅' => 'diell pas reve',
    '⛈' => 're me vetëtimë dhe shi',
    '🌤' => 'diell pas reve të vogla',
    '🌥' => 'diell pas reve të mëdha',
    '🌦' => 'diell pas reve me shi',
    '🌧' => 're me shi',
    '🌨' => 're me borë',
    '🌩' => 're me vetëtimë',
    '🌪' => 'tornado',
    '🌫' => 'mjegull',
    '🌬' => 'fytyrë në erë',
    '🌀' => 'ciklon',
    '🌈' => 'ylber',
    '🌂' => 'çadër e mbyllur',
    '☂' => 'çadër',
    '☔' => 'çadër me pika shiu',
    '⛱' => 'çadër në tokë',
    '⚡' => 'tension i lartë',
    '❄' => 'flokë bore',
    '☃' => 'njeri prej bore',
    '⛄' => 'njeri prej bore pa borë',
    '☄' => 'kometë',
    '🔥' => 'zjarr',
    '💧' => 'pikëz',
    '🌊' => 'dallgë uji',
    '🎃' => 'kungulli i Halloween',
    '🎄' => 'pemë krishtlindjeje',
    '🎆' => 'fishekzjarrë',
    '🎇' => 'xixëllimë',
    '🧨' => 'fishekzjarr',
    '✨' => 'xixëllima',
    '🎈' => 'tullumbace',
    '🎉' => 'fishek feste',
    '🎊' => 'top me konfetë',
    '🎋' => 'pemë tanabata',
    '🎍' => 'dekorim pishe',
    '🎎' => 'kukulla japoneze',
    '🎏' => 'koinobori',
    '🎐' => 'zile ere',
    '🎑' => 'ceremoni për hënën',
    '🧧' => 'zarf i kuq',
    '🎀' => 'fjongo',
    '🎁' => 'dhuratë e mbështjellë',
    '🎗' => 'fjongo kujtese',
    '🎟' => 'biletat e hyrjes',
    '🎫' => 'biletë',
    '🎖' => 'medalje ushtarake',
    '🏆' => 'trofe',
    '🏅' => 'medalje sportive',
    '🥇' => 'medalje e vendit të parë',
    '🥈' => 'medalje e vendit të dytë',
    '🥉' => 'medalje e vendit të tretë',
    '⚽' => 'top futbolli',
    '⚾' => 'bejsboll',
    '🥎' => 'top softbolli',
    '🏀' => 'basketboll',
    '🏐' => 'volejboll',
    '🏈' => 'futboll amerikan',
    '🏉' => 'regbi',
    '🎾' => 'tenis',
    '🥏' => 'hedhje disku',
    '🎳' => 'bouling',
    '🏏' => 'kriket',
    '🏑' => 'hokej në fushë',
    '🏒' => 'shkopi dhe disku i hokejit në akull',
    '🥍' => 'lakros',
    '🏓' => 'pingpong',
    '🏸' => 'badminton',
    '🥊' => 'dorezë boksi',
    '🥋' => 'uniformë artesh ushtarake',
    '🥅' => 'rrjetë golash',
    '⛳' => 'flamur në vrimë',
    '⛸' => 'patina akulli',
    '🎣' => 'grep',
    '🤿' => 'maskë zhytjeje',
    '🎽' => 'bluzë vrapimi',
    '🎿' => 'ski',
    '🛷' => 'sajë',
    '🥌' => 'gur kurlingu',
    '🎯' => 'goditje e drejtpërdrejtë',
    '🪀' => 'jo-jo',
    '🪁' => 'balonë',
    '🔫' => 'pistoletë',
    '🎱' => 'bilardo',
    '🔮' => 'top kristali',
    '🪄' => 'shkop magjik',
    '🎮' => 'videolojë',
    '🕹' => 'levë',
    '🎰' => 'lojë me monedha',
    '🎲' => 'zar loje',
    '🧩' => 'lojë me bashkim pjesësh figure',
    '🧸' => 'arush pelushi',
    '🪅' => 'pinjata',
    '🪩' => 'sferë diskoje',
    '🪆' => 'matrioshka',
    '♠' => 'figurë maç',
    '♥' => 'figurë kupë',
    '♦' => 'figurë karo',
    '♣' => 'figura spathi',
    '♟' => 'gur shahu',
    '🃏' => 'xholi',
    '🀄' => 'dragua i kuq mahjong',
    '🎴' => 'letra me lule',
    '🎭' => 'arte interpretimi',
    '🖼' => 'kornizë me pikturë',
    '🎨' => 'paleta e artistit',
    '🧵' => 'fije',
    '🪡' => 'gjilpërë qepjeje',
    '🧶' => 'fill',
    '🪢' => 'nyjë',
    '👓' => 'syze',
    '🕶' => 'syze dielli',
    '🥽' => 'syze mbrojtëse',
    '🥼' => 'pardesy laboratori',
    '🦺' => 'jelek sigurimi',
    '👔' => 'kravatë',
    '👕' => 'bluzë',
    '👖' => 'xhinse',
    '🧣' => 'shall',
    '🧤' => 'dorashka',
    '🧥' => 'pallto',
    '🧦' => 'çorape',
    '👗' => 'fustan',
    '👘' => 'kimono',
    '🥻' => 'sari',
    '🩱' => 'kostum plazhi',
    '🩲' => 'mbathje plazhi',
    '🩳' => 'pantallona të shkurtra',
    '👙' => 'bikini',
    '👚' => 'rroba femrash',
    '🪭' => 'freskore',
    '👛' => 'kuletë',
    '👜' => 'çantë dore',
    '👝' => 'çantë',
    '🛍' => 'qese dyqani',
    '🎒' => 'çantë shkolle',
    '🩴' => 'sandale me rrip përpara',
    '👞' => 'këpucë burrash',
    '👟' => 'atlete',
    '🥾' => 'këpucë ekskursioni',
    '🥿' => 'këpucë e sheshtë',
    '👠' => 'këpucë me takë të lartë',
    '👡' => 'sandale femrash',
    '🩰' => 'këpucë baleti',
    '👢' => 'çizme femrash',
    '🪮' => 'krehër për të mbledhur flokët',
    '👑' => 'kurorë',
    '👒' => 'kapelë femrash',
    '🎩' => 'kapelë cilindrike',
    '🎓' => 'kapelë diplomimi',
    '🧢' => 'kapelë "sqep"',
    '🪖' => 'kokore ushtrarake',
    '⛑' => 'helmetë me kryq të bardhë',
    '📿' => 'rruzare',
    '💄' => 'buzëkuq',
    '💍' => 'unazë',
    '💎' => 'gur i çmuar',
    '🔇' => 'altoparlant joaktiv',
    '🔈' => 'altoparlant me zë të ulët',
    '🔉' => 'altoparlant me zë mesatar',
    '🔊' => 'altoparlant me zë të lartë',
    '📢' => 'altoparlant me zë',
    '📣' => 'megafon',
    '📯' => 'briri i postës',
    '🔔' => 'zile',
    '🔕' => 'zile me vizë të pjerrët',
    '🎼' => 'pentagram',
    '🎵' => 'notë muzikore',
    '🎶' => 'nota muzikore',
    '🎙' => 'mikrofon studioje',
    '🎚' => 'rrëshqitësi i nivelit',
    '🎛' => 'çelësat e kontrollit',
    '🎤' => 'mikrofon',
    '🎧' => 'kufje',
    '📻' => 'radio',
    '🎷' => 'saksofon',
    '🎺' => 'trombë',
    '🪊' => 'trombon',
    '🪗' => 'fizarmonikë',
    '🎸' => 'kitarë',
    '🎹' => 'tastierë muzikore',
    '🎻' => 'violinë',
    '🪕' => 'banxho',
    '🥁' => 'daulle',
    '🪘' => 'daulle e gjatë',
    '🪇' => 'maraka',
    '🪈' => 'flaut',
    '🪉' => 'harpë',
    '📱' => 'celular',
    '📲' => 'celular me shigjetë',
    '☎' => 'telefon',
    '📞' => 'receptor telefoni',
    '📟' => 'biper',
    '📠' => 'faks',
    '🔋' => 'bateri',
    '🪫' => 'bateri e ulët',
    '🔌' => 'spinë elektrike',
    '💻' => 'laptop',
    '🖥' => 'kompjuter desktop',
    '🖨' => 'printer',
    '⌨' => 'tastierë',
    '🖱' => 'miu i kompjuterit',
    '🖲' => 'sfera rrotulluese',
    '💽' => 'disk i vogël',
    '💾' => 'disketë',
    '💿' => 'disk optik',
    '📀' => 'dvd',
    '🧮' => 'abak',
    '🎥' => 'kamerë filmi',
    '🎞' => 'kuadro filmi',
    '📽' => 'projektor filmi',
    '🎬' => 'tabelë dublimi e skenës',
    '📺' => 'televizor',
    '📷' => 'kamerë',
    '📸' => 'kamerë me blic',
    '📹' => 'videokamerë',
    '📼' => 'videokasetë',
    '🔍' => 'lupë zmadhuese që tregon majtas',
    '🔎' => 'lupë zmadhuese e drejtuar djathtas',
    '🕯' => 'qiri',
    '💡' => 'llambë',
    '🔦' => 'elektrik dore',
    '🏮' => 'fener me letër të kuqe',
    '🪔' => 'llambë me vaj',
    '📔' => 'bllok shënimesh me kapak dekorativ',
    '📕' => 'libër i mbyllur',
    '📖' => 'libër i hapur',
    '📗' => 'libër jeshil',
    '📘' => 'libër blu',
    '📙' => 'libër portokalli',
    '📚' => 'libra',
    '📓' => 'bllok shënimesh',
    '📒' => 'libër llogarish',
    '📃' => 'faqe e përthyer',
    '📜' => 'dorëshkrim i mbledhur rrotull',
    '📄' => 'faqe e drejtuar lart',
    '📰' => 'gazetë',
    '🗞' => 'gazetë e mbledhur rrotull',
    '📑' => 'skedat e faqeshënuesve',
    '🔖' => 'faqeshënues',
    '🏷' => 'etiketë',
    '🪙' => 'monedhë',
    '💰' => 'qese parash',
    '🪎' => 'arkë thesari',
    '💴' => 'kartëmonedhë jeni',
    '💵' => 'kartëmonedhë dollari',
    '💶' => 'kartëmonedhë euroje',
    '💷' => 'kartëmonedhë paundi',
    '💸' => 'para me krahë',
    '💳' => 'kartë krediti',
    '🧾' => 'faturë',
    '💹' => 'grafik rritës me jenë',
    '✉' => 'zarf',
    '📧' => 'email',
    '📨' => 'zarf në ardhje',
    '📩' => 'zarf me shigjetë',
    '📤' => 'kutia postare në dalje',
    '📥' => 'sirtari i kutisë postare',
    '📦' => 'pako',
    '📫' => 'kuti postare e mbyllur me flamurin lart',
    '📪' => 'kuti postare e mbyllur me flamurin poshtë',
    '📬' => 'kuti postare e hapur me flamurin lart',
    '📭' => 'kuti postare e hapur me flamurin poshtë',
    '📮' => 'kuti postare',
    '🗳' => 'kuti votimi me votë',
    '✏' => 'laps',
    '✒' => 'penë e zezë',
    '🖋' => 'stilograf',
    '🖊' => 'stilolaps',
    '🖌' => 'furçë piktori',
    '🖍' => 'shkumës me ngjyrë',
    '📝' => 'shënime',
    '💼' => 'valixhe',
    '📁' => 'dosje dokumentesh',
    '📂' => 'dosje dokumentesh e hapur',
    '🗂' => 'ndarëse skedash',
    '📅' => 'kalendar',
    '📆' => 'kalendar që griset',
    '🗒' => 'bllok shënimesh me spirale',
    '🗓' => 'kalendar me spirale',
    '📇' => 'katalog skedash',
    '📈' => 'grafik rritës',
    '📉' => 'grafik zbritës',
    '📊' => 'grafik me shirita',
    '📋' => 'mbajtëse letrash',
    '📌' => 'gjilpërë kapëse',
    '📍' => 'gjilpërë kapëse e rrumbullakët',
    '📎' => 'kapëse letrash',
    '🖇' => 'kapëse teli të lidhura',
    '📏' => 'vizore e drejtë',
    '📐' => 'vizore trekëndëshe',
    '✂' => 'gërshërë',
    '🗃' => 'kuti skedarësh',
    '🗄' => 'sirtar dokumentesh',
    '🗑' => 'kosh basketbolli',
    '🔒' => 'kyç',
    '🔓' => 'kyç i hapur',
    '🔏' => 'kyç me penë',
    '🔐' => 'kyç i mbyllur me çelës',
    '🔑' => 'çelës',
    '🗝' => 'çelës i vjetër',
    '🔨' => 'çekiç',
    '🪓' => 'sëpatë',
    '⛏' => 'kazmë',
    '⚒' => 'çekiç dhe kazmë',
    '🛠' => 'çekiç dhe çelës',
    '🗡' => 'kamë',
    '⚔' => 'shpata të kryqëzuara',
    '💣' => 'bombë',
    '🪃' => 'bumerang',
    '🏹' => 'hark dhe shigjetë',
    '🛡' => 'mburojë',
    '🪚' => 'sharrë marangozi',
    '🔧' => 'çelës anglez',
    '🪛' => 'kaçavidë',
    '🔩' => 'bulon dhe dado',
    '⚙' => 'ingranazh',
    '🗜' => 'ngjeshje',
    '⚖' => 'peshore',
    '🦯' => 'shkop gjurmimi',
    '🔗' => 'lidh',
    '⛓' => 'zinxhirë',
    '🪝' => 'kanxhë',
    '🧰' => 'kuti veglash',
    '🧲' => 'magnet',
    '🪜' => 'shkallë',
    '🪏' => 'lopatë',
    '⚗' => 'lambik',
    '🧪' => 'epruvetë',
    '🧫' => 'lamelë',
    '🧬' => 'ADN',
    '🔬' => 'mikroskop',
    '🔭' => 'teleskop',
    '📡' => 'antenë satelitore',
    '💉' => 'shiringë',
    '🩸' => 'pikë gjaku',
    '💊' => 'pilulë',
    '🩹' => 'fashë ngjitëse',
    '🩼' => 'patericë',
    '🩺' => 'stetoskop',
    '🩻' => 'grafi',
    '🚪' => 'derë',
    '🛗' => 'ashensor',
    '🪞' => 'pasqyrë',
    '🪟' => 'dritare',
    '🛏' => 'krevat',
    '🛋' => 'divan dhe llambë',
    '🪑' => 'karrige',
    '🚽' => 'tualet',
    '🪠' => 'pompë',
    '🚿' => 'dush',
    '🛁' => 'vaskë',
    '🪤' => 'çark',
    '🪒' => 'brisk rroje',
    '🧴' => 'shishe solucioni',
    '🧷' => 'paramane',
    '🧹' => 'fshesë',
    '🧺' => 'shportë',
    '🧻' => 'rrotë letre',
    '🪣' => 'kovë',
    '🧼' => 'sapun',
    '🫧' => 'flluska',
    '🪥' => 'furçë dhëmbësh',
    '🧽' => 'sfungjer',
    '🧯' => 'fikës zjarri',
    '🛒' => 'karrocë pazari',
    '🚬' => 'cigare',
    '⚰' => 'arkivol',
    '🪦' => 'gur varri',
    '⚱' => 'urnë funerali',
    '🧿' => 'hajmali kundër syrit të keq',
    '🪬' => 'hajmali',
    '🗿' => 'moai',
    '🪧' => 'pankartë',
    '🪪' => 'letërnjoftim',
    '🏧' => 'shenja e bankomatit',
    '🚮' => 'shenja e koshit të mbeturinave',
    '🚰' => 'ujë i pijshëm',
    '♿' => 'karrocë me rrota',
    '🚹' => 'banja e meshkujve',
    '🚺' => 'banja e femrave',
    '🚻' => 'banja',
    '🚼' => 'simboli i bebes',
    '🚾' => 'banjë',
    '🛂' => 'kontrolli i pasaportave',
    '🛃' => 'doganë',
    '🛄' => 'marrje bagazhesh',
    '🛅' => 'bagazh i lënë',
    '⚠' => 'kujdes',
    '🚸' => 'kalim fëmijësh',
    '⛔' => 'ndalohet hyrja',
    '🚫' => 'ndalohet',
    '🚳' => 'ndalohen biçikletat',
    '🚭' => 'ndalohet duhani',
    '🚯' => 'ndalohet hedhja e mbeturinave',
    '🚱' => 'ujë jo i pijshëm',
    '🚷' => 'ndalohen kalimtarët',
    '📵' => 'ndalohen celularët',
    '🔞' => 'ndalohet nën moshën tetëmbëdhjetë',
    '☢' => 'rrezik radioaktiv',
    '☣' => 'rrezik biologjik',
    '⬆' => 'shigjeta lart',
    '↗' => 'shigjeta lart-djathtas',
    '➡' => 'shigjetë djathtas',
    '↘' => 'shigjetë poshtë djathtas',
    '⬇' => 'shigjetë poshtë',
    '↙' => 'shigjetë poshtë majtas',
    '⬅' => 'shigjeta majtas',
    '↖' => 'shigjeta lart-majtas',
    '↕' => 'shigjeta lart-poshtë',
    '↔' => 'shigjetë majtas-djathtas',
    '↩' => 'shigjetë djathtas e përkulur majtas',
    '↪' => 'shigjeta majtas me hark djathtas',
    '⤴' => 'shigjetë djathtas e përkulur lart',
    '⤵' => 'shigjetë djathtas e përkulur poshtë',
    '🔃' => 'shigjeta vertikale orare',
    '🔄' => 'butoni me shigjetat antiorare',
    '🔙' => 'shigjetë prapa',
    '🔚' => 'shigjeta e fundit',
    '🔛' => 'shigjeta aktive!',
    '🔜' => 'shigjeta shpejt',
    '🔝' => 'shigjetë lart',
    '🛐' => 'vend kulti',
    '⚛' => 'simboli i atomit',
    '🕉' => 'om',
    '✡' => 'ylli i davidit',
    '☸' => 'rrota e darmës',
    '☯' => 'jin jang',
    '✝' => 'kryqi latin',
    '☦' => 'kryq ortodoks',
    '☪' => 'yll dhe gjysmëhënë',
    '☮' => 'simboli i paqes',
    '🕎' => 'hanukia',
    '🔯' => 'yll me gjashtë cepa me pikë',
    '🪯' => 'kandë',
    '♈' => 'dashi',
    '♉' => 'demi',
    '♊' => 'binjakët',
    '♋' => 'gaforrja',
    '♌' => 'luani',
    '♍' => 'virgjëresha',
    '♎' => 'peshorja',
    '♏' => 'akrepi',
    '♐' => 'shigjetari',
    '♑' => 'bricjapi',
    '♒' => 'ujori',
    '♓' => 'peshqit',
    '⛎' => 'ofiuku',
    '🔀' => 'butoni i përzierjes së këngëve',
    '🔁' => 'butoni i përsëritjes',
    '🔂' => 'butoni i përsëritjes një herë',
    '▶' => 'butoni i luajtjes',
    '⏩' => 'butoni përpara shpejt',
    '⏭' => 'butoni i këngës tjetër',
    '⏯' => 'butoni i luajtjes ose pauzës',
    '◀' => 'butoni i kthimit prapa',
    '⏪' => 'butoni i kthimit shpejt',
    '⏮' => 'butoni i këngës së fundit',
    '🔼' => 'butoni lart',
    '⏫' => 'butoni lart shpejt',
    '🔽' => 'butoni poshtë',
    '⏬' => 'butoni poshtë shpejt',
    '⏸' => 'butoni i pauzës',
    '⏹' => 'butoni i ndalimit',
    '⏺' => 'butoni i regjistrimit',
    '⏏' => 'butoni i nxjerrjes',
    '🎦' => 'kinema',
    '🔅' => 'butoni i zbehjes',
    '🔆' => 'butoni i ndriçimit',
    '📶' => 'shiritat e antenës',
    '🛜' => 'lidhje me valë',
    '📳' => 'modaliteti i dridhjeve',
    '📴' => 'fik celularin',
    '♀' => 'shenja femër',
    '♂' => 'shenja mashkull',
    '⚧' => 'simboli për transgjinorë',
    '✖' => 'shenja x e shumëzimit e trashë',
    '➕' => 'shenja plus e trashë',
    '➖' => 'shenja minus e trashë',
    '➗' => 'shenja e pjesëtimit e trashë',
    '🟰' => 'shenjë e fortë barazie',
    '♾' => 'pafundësi',
    '‼' => 'dy pikëçuditje',
    '⁉' => 'shenja e pikëpyetjes dhe pikëçuditjes',
    '❓' => 'shenja e pikëpyetjes e kuqe',
    '❔' => 'pikëpyetje e bardhë',
    '❕' => 'shenjë pikëçuditje e bardhë',
    '❗' => 'shenja e pikëçuditjes',
    '〰' => 'vijë e valëzuar',
    '💱' => 'këmbimi i valutës',
    '💲' => 'shenjë dollari e trashur',
    '⚕' => 'simboli mjekësor',
    '♻' => 'simboli i riciklimit',
    '⚜' => 'shpatore',
    '🔱' => 'emblemë e fuzhnjës tredhëmbëshe',
    '📛' => 'etiketë emri',
    '🔰' => 'simboli japonez për fillestar',
    '⭕' => 'rreth i madh i trashë',
    '✅' => 'shenja e zgjedhjes e bardhë e trashë',
    '☑' => 'kutia e votimit me shenjën e zgjedhjes',
    '✔' => 'shenja e zgjedhjes e trashë',
    '❌' => 'shenja e vendosjes së kryqit',
    '❎' => 'butoni me shenjën e shumëzimit',
    '➰' => 'lak i përdredhur',
    '➿' => 'lak i përdredhur dy herë',
    '〽' => 'shenja e ndryshimit të pjesës',
    '✳' => 'yllth me tetë cepa me majë',
    '✴' => 'yll me tetë cepa',
    '❇' => 'xixëllim',
    '™' => 'markë tregtare',
    '🫟' => 'spërkë',
    '🔠' => 'fut shkronja latine kapitale',
    '🔡' => 'fut shkronja latine jokapitale',
    '🔢' => 'fut numra',
    '🔣' => 'fut simbole',
    '🔤' => 'fut shkronja latine',
    '🅰' => 'butoni «A»',
    '🆎' => 'butoni «AB»',
    '🅱' => 'butoni «B»',
    '🆑' => 'butoni «CL»',
    '🆒' => 'butoni «COOL»',
    '🆓' => 'butoni «FREE»',
    'ℹ' => 'informacion',
    '🆔' => 'butoni «ID»',
    'Ⓜ' => '«M» e rrethuar',
    '🆕' => 'butoni «NEW»',
    '🆖' => 'butoni «NG»',
    '🅾' => 'butoni «O»',
    '🆗' => 'butoni «OK»',
    '🅿' => 'butoni «P»',
    '🆘' => 'butoni «SOS»',
    '🆙' => 'butoni «UP!»',
    '🆚' => 'butoni «VS»',
    '🈁' => 'butoni «këtu» në japonisht',
    '🈂' => 'butoni «tarifa e shërbimit» në japonisht',
    '🈷' => 'butoni «shuma mujore» në japonisht',
    '🈶' => 'butoni «jo falas» në japonisht',
    '🈯' => 'butoni «rezervuar» në japonisht',
    '🉐' => 'butoni «me leverdi» në japonisht',
    '🈹' => 'butoni «ulje çmimi» në japonisht',
    '🈚' => 'butoni «falas» në japonisht',
    '🈲' => 'butoni «ndaluar» në japonisht',
    '🉑' => 'butoni «miratuar» në japonisht',
    '🈸' => 'butoni «zbatim» në japonisht',
    '🈴' => 'butoni «notë kaluese» në japonisht',
    '🈳' => 'butoni «të lira» në japonisht',
    '㊗' => 'butoni «urime» në japonisht',
    '㊙' => 'butoni «sekret» në japonisht',
    '🈺' => 'butoni «hapur për punë» në japonisht',
    '🈵' => 'butoni «jo të lira» në japonisht',
    '🔴' => 'rreth i kuq',
    '🟠' => 'rreth i portokalltë',
    '🟡' => 'rreth i verdhë',
    '🟢' => 'rreth i gjelbër',
    '🔵' => 'rreth blu',
    '🟣' => 'rreth në ngjyrë vjollce',
    '🟤' => 'rreth në ngjyrë kafe',
    '⚫' => 'rreth i zi',
    '⚪' => 'rreth i bardhë',
    '🟥' => 'katror i kuq',
    '🟧' => 'katror i portokalltë',
    '🟨' => 'katror i verdhë',
    '🟩' => 'katror i gjelbër',
    '🟦' => 'katror i kaltër',
    '🟪' => 'katror në ngjyrë vjollce',
    '🟫' => 'katror në ngjyrë kafe',
    '⬛' => 'katror i madh i zi',
    '⬜' => 'katror i madh i bardhë',
    '◼' => 'katror mesatar i zi',
    '◻' => 'katror mesatar i bardhë',
    '◾' => 'katror mesatarisht i vogël i zi',
    '◽' => 'katror mesatarisht i vogël i bardhë',
    '▪' => 'katror i vogël i zi',
    '▫' => 'katror i vogël i bardhë',
    '🔶' => 'diamant i madh portokalli',
    '🔷' => 'diamant i madh blu',
    '🔸' => 'diamant i vogël portokalli',
    '🔹' => 'diamant i vogël blu',
    '🔺' => 'trekëndësh i kuq i drejtuar lart',
    '🔻' => 'trekëndësh i kuq i drejtuar poshtë',
    '💠' => 'diamant me një pikë',
    '🔘' => 'buton radioje',
    '🔳' => 'buton katror i bardhë',
    '🔲' => 'buton katror i zi',
    '🏁' => 'flamur me katrorë',
    '🚩' => 'flamur trekëndësh',
    '🎌' => 'flamuj të kryqëzuar',
    '🏴' => 'flamur i zi',
    '🏳' => 'flamur i bardhë',
];
