<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'öpücük: yetişkin yetişkin açık cilt tonu orta açık cilt tonu',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'öpücük: yetişkin yetişkin açık cilt tonu orta cilt tonu',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'öpücük: yetişkin yetişkin açık cilt tonu orta koyu cilt tonu',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'öpücük: yetişkin yetişkin açık cilt tonu koyu cilt tonu',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'öpücük: yetişkin yetişkin orta açık cilt tonu açık cilt tonu',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'öpücük: yetişkin yetişkin orta açık cilt tonu orta cilt tonu',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'öpücük: yetişkin yetişkin orta açık cilt tonu orta koyu cilt tonu',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'öpücük: yetişkin yetişkin orta açık cilt tonu koyu cilt tonu',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'öpücük: yetişkin yetişkin orta cilt tonu açık cilt tonu',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'öpücük: yetişkin yetişkin orta cilt tonu orta açık cilt tonu',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'öpücük: yetişkin yetişkin orta cilt tonu orta koyu cilt tonu',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'öpücük: yetişkin yetişkin orta cilt tonu koyu cilt tonu',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'öpücük: yetişkin yetişkin orta koyu cilt tonu açık cilt tonu',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'öpücük: yetişkin yetişkin orta koyu cilt tonu orta açık cilt tonu',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'öpücük: yetişkin yetişkin orta koyu cilt tonu orta cilt tonu',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'öpücük: yetişkin yetişkin orta koyu cilt tonu koyu cilt tonu',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'öpücük: yetişkin yetişkin koyu cilt tonu açık cilt tonu',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'öpücük: yetişkin yetişkin koyu cilt tonu orta açık cilt tonu',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'öpücük: yetişkin yetişkin koyu cilt tonu orta cilt tonu',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'öpücük: yetişkin yetişkin koyu cilt tonu orta koyu cilt tonu',
    '👨🏻‍❤‍💋‍👨🏻' => 'öpücük: erkek erkek açık cilt tonu',
    '👨🏻‍❤‍💋‍👨🏼' => 'öpücük: erkek erkek açık cilt tonu orta açık cilt tonu',
    '👨🏻‍❤‍💋‍👨🏽' => 'öpücük: erkek erkek açık cilt tonu orta cilt tonu',
    '👨🏻‍❤‍💋‍👨🏾' => 'öpücük: erkek erkek açık cilt tonu orta koyu cilt tonu',
    '👨🏻‍❤‍💋‍👨🏿' => 'öpücük: erkek erkek açık cilt tonu koyu cilt tonu',
    '👨🏼‍❤‍💋‍👨🏻' => 'öpücük: erkek erkek orta açık cilt tonu açık cilt tonu',
    '👨🏼‍❤‍💋‍👨🏼' => 'öpücük: erkek erkek orta açık cilt tonu',
    '👨🏼‍❤‍💋‍👨🏽' => 'öpücük: erkek erkek orta açık cilt tonu orta cilt tonu',
    '👨🏼‍❤‍💋‍👨🏾' => 'öpücük: erkek erkek orta açık cilt tonu orta koyu cilt tonu',
    '👨🏼‍❤‍💋‍👨🏿' => 'öpücük: erkek erkek orta açık cilt tonu koyu cilt tonu',
    '👨🏽‍❤‍💋‍👨🏻' => 'öpücük: erkek erkek orta cilt tonu açık cilt tonu',
    '👨🏽‍❤‍💋‍👨🏼' => 'öpücük: erkek erkek orta cilt tonu orta açık cilt tonu',
    '👨🏽‍❤‍💋‍👨🏽' => 'öpücük: erkek erkek orta cilt tonu',
    '👨🏽‍❤‍💋‍👨🏾' => 'öpücük: erkek erkek orta cilt tonu orta koyu cilt tonu',
    '👨🏽‍❤‍💋‍👨🏿' => 'öpücük: erkek erkek orta cilt tonu koyu cilt tonu',
    '👨🏾‍❤‍💋‍👨🏻' => 'öpücük: erkek erkek orta koyu cilt tonu açık cilt tonu',
    '👨🏾‍❤‍💋‍👨🏼' => 'öpücük: erkek erkek orta koyu cilt tonu orta açık cilt tonu',
    '👨🏾‍❤‍💋‍👨🏽' => 'öpücük: erkek erkek orta koyu cilt tonu orta cilt tonu',
    '👨🏾‍❤‍💋‍👨🏾' => 'öpücük: erkek erkek orta koyu cilt tonu',
    '👨🏾‍❤‍💋‍👨🏿' => 'öpücük: erkek erkek orta koyu cilt tonu koyu cilt tonu',
    '👨🏿‍❤‍💋‍👨🏻' => 'öpücük: erkek erkek koyu cilt tonu açık cilt tonu',
    '👨🏿‍❤‍💋‍👨🏼' => 'öpücük: erkek erkek koyu cilt tonu orta açık cilt tonu',
    '👨🏿‍❤‍💋‍👨🏽' => 'öpücük: erkek erkek koyu cilt tonu orta cilt tonu',
    '👨🏿‍❤‍💋‍👨🏾' => 'öpücük: erkek erkek koyu cilt tonu orta koyu cilt tonu',
    '👨🏿‍❤‍💋‍👨🏿' => 'öpücük: erkek erkek koyu cilt tonu',
    '👩🏻‍❤‍💋‍👨🏻' => 'öpücük: kadın erkek açık cilt tonu',
    '👩🏻‍❤‍💋‍👨🏼' => 'öpücük: kadın erkek açık cilt tonu orta açık cilt tonu',
    '👩🏻‍❤‍💋‍👨🏽' => 'öpücük: kadın erkek açık cilt tonu orta cilt tonu',
    '👩🏻‍❤‍💋‍👨🏾' => 'öpücük: kadın erkek açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍❤‍💋‍👨🏿' => 'öpücük: kadın erkek açık cilt tonu koyu cilt tonu',
    '👩🏼‍❤‍💋‍👨🏻' => 'öpücük: kadın erkek orta açık cilt tonu açık cilt tonu',
    '👩🏼‍❤‍💋‍👨🏼' => 'öpücük: kadın erkek orta açık cilt tonu',
    '👩🏼‍❤‍💋‍👨🏽' => 'öpücük: kadın erkek orta açık cilt tonu orta cilt tonu',
    '👩🏼‍❤‍💋‍👨🏾' => 'öpücük: kadın erkek orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍❤‍💋‍👨🏿' => 'öpücük: kadın erkek orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍❤‍💋‍👨🏻' => 'öpücük: kadın erkek orta cilt tonu açık cilt tonu',
    '👩🏽‍❤‍💋‍👨🏼' => 'öpücük: kadın erkek orta cilt tonu orta açık cilt tonu',
    '👩🏽‍❤‍💋‍👨🏽' => 'öpücük: kadın erkek orta cilt tonu',
    '👩🏽‍❤‍💋‍👨🏾' => 'öpücük: kadın erkek orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍❤‍💋‍👨🏿' => 'öpücük: kadın erkek orta cilt tonu koyu cilt tonu',
    '👩🏾‍❤‍💋‍👨🏻' => 'öpücük: kadın erkek orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍❤‍💋‍👨🏼' => 'öpücük: kadın erkek orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍❤‍💋‍👨🏽' => 'öpücük: kadın erkek orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍❤‍💋‍👨🏾' => 'öpücük: kadın erkek orta koyu cilt tonu',
    '👩🏾‍❤‍💋‍👨🏿' => 'öpücük: kadın erkek orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍❤‍💋‍👨🏻' => 'öpücük: kadın erkek koyu cilt tonu açık cilt tonu',
    '👩🏿‍❤‍💋‍👨🏼' => 'öpücük: kadın erkek koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍❤‍💋‍👨🏽' => 'öpücük: kadın erkek koyu cilt tonu orta cilt tonu',
    '👩🏿‍❤‍💋‍👨🏾' => 'öpücük: kadın erkek koyu cilt tonu orta koyu cilt tonu',
    '👩🏿‍❤‍💋‍👨🏿' => 'öpücük: kadın erkek koyu cilt tonu',
    '👩🏻‍❤‍💋‍👩🏻' => 'öpücük: kadın kadın açık cilt tonu',
    '👩🏻‍❤‍💋‍👩🏼' => 'öpücük: kadın kadın açık cilt tonu orta açık cilt tonu',
    '👩🏻‍❤‍💋‍👩🏽' => 'öpücük: kadın kadın açık cilt tonu orta cilt tonu',
    '👩🏻‍❤‍💋‍👩🏾' => 'öpücük: kadın kadın açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍❤‍💋‍👩🏿' => 'öpücük: kadın kadın açık cilt tonu koyu cilt tonu',
    '👩🏼‍❤‍💋‍👩🏻' => 'öpücük: kadın kadın orta açık cilt tonu açık cilt tonu',
    '👩🏼‍❤‍💋‍👩🏼' => 'öpücük: kadın kadın orta açık cilt tonu',
    '👩🏼‍❤‍💋‍👩🏽' => 'öpücük: kadın kadın orta açık cilt tonu orta cilt tonu',
    '👩🏼‍❤‍💋‍👩🏾' => 'öpücük: kadın kadın orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍❤‍💋‍👩🏿' => 'öpücük: kadın kadın orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍❤‍💋‍👩🏻' => 'öpücük: kadın kadın orta cilt tonu açık cilt tonu',
    '👩🏽‍❤‍💋‍👩🏼' => 'öpücük: kadın kadın orta cilt tonu orta açık cilt tonu',
    '👩🏽‍❤‍💋‍👩🏽' => 'öpücük: kadın kadın orta cilt tonu',
    '👩🏽‍❤‍💋‍👩🏾' => 'öpücük: kadın kadın orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍❤‍💋‍👩🏿' => 'öpücük: kadın kadın orta cilt tonu koyu cilt tonu',
    '👩🏾‍❤‍💋‍👩🏻' => 'öpücük: kadın kadın orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍❤‍💋‍👩🏼' => 'öpücük: kadın kadın orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍❤‍💋‍👩🏽' => 'öpücük: kadın kadın orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍❤‍💋‍👩🏾' => 'öpücük: kadın kadın orta koyu cilt tonu',
    '👩🏾‍❤‍💋‍👩🏿' => 'öpücük: kadın kadın orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍❤‍💋‍👩🏻' => 'öpücük: kadın kadın koyu cilt tonu açık cilt tonu',
    '👩🏿‍❤‍💋‍👩🏼' => 'öpücük: kadın kadın koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍❤‍💋‍👩🏽' => 'öpücük: kadın kadın koyu cilt tonu orta cilt tonu',
    '👩🏿‍❤‍💋‍👩🏾' => 'öpücük: kadın kadın koyu cilt tonu orta koyu cilt tonu',
    '👩🏿‍❤‍💋‍👩🏿' => 'öpücük: kadın kadın koyu cilt tonu',
    '🧑🏻‍🫯‍🧑🏼' => 'güreşen kişiler: açık cilt tonu orta açık cilt tonu',
    '🧑🏻‍🫯‍🧑🏽' => 'güreşen kişiler: açık cilt tonu orta cilt tonu',
    '🧑🏻‍🫯‍🧑🏾' => 'güreşen kişiler: açık cilt tonu orta koyu cilt tonu',
    '🧑🏻‍🫯‍🧑🏿' => 'güreşen kişiler: açık cilt tonu koyu cilt tonu',
    '🧑🏼‍🫯‍🧑🏻' => 'güreşen kişiler: orta açık cilt tonu açık cilt tonu',
    '🧑🏼‍🫯‍🧑🏽' => 'güreşen kişiler: orta açık cilt tonu orta cilt tonu',
    '🧑🏼‍🫯‍🧑🏾' => 'güreşen kişiler: orta açık cilt tonu orta koyu cilt tonu',
    '🧑🏼‍🫯‍🧑🏿' => 'güreşen kişiler: orta açık cilt tonu koyu cilt tonu',
    '🧑🏽‍🫯‍🧑🏻' => 'güreşen kişiler: orta cilt tonu açık cilt tonu',
    '🧑🏽‍🫯‍🧑🏼' => 'güreşen kişiler: orta cilt tonu orta açık cilt tonu',
    '🧑🏽‍🫯‍🧑🏾' => 'güreşen kişiler: orta cilt tonu orta koyu cilt tonu',
    '🧑🏽‍🫯‍🧑🏿' => 'güreşen kişiler: orta cilt tonu koyu cilt tonu',
    '🧑🏾‍🫯‍🧑🏻' => 'güreşen kişiler: orta koyu cilt tonu açık cilt tonu',
    '🧑🏾‍🫯‍🧑🏼' => 'güreşen kişiler: orta koyu cilt tonu orta açık cilt tonu',
    '🧑🏾‍🫯‍🧑🏽' => 'güreşen kişiler: orta koyu cilt tonu orta cilt tonu',
    '🧑🏾‍🫯‍🧑🏿' => 'güreşen kişiler: orta koyu cilt tonu koyu cilt tonu',
    '🧑🏿‍🫯‍🧑🏻' => 'güreşen kişiler: koyu cilt tonu açık cilt tonu',
    '🧑🏿‍🫯‍🧑🏼' => 'güreşen kişiler: koyu cilt tonu orta açık cilt tonu',
    '🧑🏿‍🫯‍🧑🏽' => 'güreşen kişiler: koyu cilt tonu orta cilt tonu',
    '🧑🏿‍🫯‍🧑🏾' => 'güreşen kişiler: koyu cilt tonu orta koyu cilt tonu',
    '🧑🏻‍❤‍🧑🏼' => 'kalpli çift: yetişkin yetişkin açık cilt tonu orta açık cilt tonu',
    '🧑🏻‍❤‍🧑🏽' => 'kalpli çift: yetişkin yetişkin açık cilt tonu orta cilt tonu',
    '🧑🏻‍❤‍🧑🏾' => 'kalpli çift: yetişkin yetişkin açık cilt tonu orta koyu cilt tonu',
    '🧑🏻‍❤‍🧑🏿' => 'kalpli çift: yetişkin yetişkin açık cilt tonu koyu cilt tonu',
    '🧑🏼‍❤‍🧑🏻' => 'kalpli çift: yetişkin yetişkin orta açık cilt tonu açık cilt tonu',
    '🧑🏼‍❤‍🧑🏽' => 'kalpli çift: yetişkin yetişkin orta açık cilt tonu orta cilt tonu',
    '🧑🏼‍❤‍🧑🏾' => 'kalpli çift: yetişkin yetişkin orta açık cilt tonu orta koyu cilt tonu',
    '🧑🏼‍❤‍🧑🏿' => 'kalpli çift: yetişkin yetişkin orta açık cilt tonu koyu cilt tonu',
    '🧑🏽‍❤‍🧑🏻' => 'kalpli çift: yetişkin yetişkin orta cilt tonu açık cilt tonu',
    '🧑🏽‍❤‍🧑🏼' => 'kalpli çift: yetişkin yetişkin orta cilt tonu orta açık cilt tonu',
    '🧑🏽‍❤‍🧑🏾' => 'kalpli çift: yetişkin yetişkin orta cilt tonu orta koyu cilt tonu',
    '🧑🏽‍❤‍🧑🏿' => 'kalpli çift: yetişkin yetişkin orta cilt tonu koyu cilt tonu',
    '🧑🏾‍❤‍🧑🏻' => 'kalpli çift: yetişkin yetişkin orta koyu cilt tonu açık cilt tonu',
    '🧑🏾‍❤‍🧑🏼' => 'kalpli çift: yetişkin yetişkin orta koyu cilt tonu orta açık cilt tonu',
    '🧑🏾‍❤‍🧑🏽' => 'kalpli çift: yetişkin yetişkin orta koyu cilt tonu orta cilt tonu',
    '🧑🏾‍❤‍🧑🏿' => 'kalpli çift: yetişkin yetişkin orta koyu cilt tonu koyu cilt tonu',
    '🧑🏿‍❤‍🧑🏻' => 'kalpli çift: yetişkin yetişkin koyu cilt tonu açık cilt tonu',
    '🧑🏿‍❤‍🧑🏼' => 'kalpli çift: yetişkin yetişkin koyu cilt tonu orta açık cilt tonu',
    '🧑🏿‍❤‍🧑🏽' => 'kalpli çift: yetişkin yetişkin koyu cilt tonu orta cilt tonu',
    '🧑🏿‍❤‍🧑🏾' => 'kalpli çift: yetişkin yetişkin koyu cilt tonu orta koyu cilt tonu',
    '🧑🏻‍🐰‍🧑🏼' => 'tavşan kulaklı kişiler: açık cilt tonu orta açık cilt tonu',
    '🧑🏻‍🐰‍🧑🏽' => 'tavşan kulaklı kişiler: açık cilt tonu orta cilt tonu',
    '🧑🏻‍🐰‍🧑🏾' => 'tavşan kulaklı kişiler: açık cilt tonu orta koyu cilt tonu',
    '🧑🏻‍🐰‍🧑🏿' => 'tavşan kulaklı kişiler: açık cilt tonu koyu cilt tonu',
    '🧑🏼‍🐰‍🧑🏻' => 'tavşan kulaklı kişiler: orta açık cilt tonu açık cilt tonu',
    '🧑🏼‍🐰‍🧑🏽' => 'tavşan kulaklı kişiler: orta açık cilt tonu orta cilt tonu',
    '🧑🏼‍🐰‍🧑🏾' => 'tavşan kulaklı kişiler: orta açık cilt tonu orta koyu cilt tonu',
    '🧑🏼‍🐰‍🧑🏿' => 'tavşan kulaklı kişiler: orta açık cilt tonu koyu cilt tonu',
    '🧑🏽‍🐰‍🧑🏻' => 'tavşan kulaklı kişiler: orta cilt tonu açık cilt tonu',
    '🧑🏽‍🐰‍🧑🏼' => 'tavşan kulaklı kişiler: orta cilt tonu orta açık cilt tonu',
    '🧑🏽‍🐰‍🧑🏾' => 'tavşan kulaklı kişiler: orta cilt tonu orta koyu cilt tonu',
    '🧑🏽‍🐰‍🧑🏿' => 'tavşan kulaklı kişiler: orta cilt tonu koyu cilt tonu',
    '🧑🏾‍🐰‍🧑🏻' => 'tavşan kulaklı kişiler: orta koyu cilt tonu açık cilt tonu',
    '🧑🏾‍🐰‍🧑🏼' => 'tavşan kulaklı kişiler: orta koyu cilt tonu orta açık cilt tonu',
    '🧑🏾‍🐰‍🧑🏽' => 'tavşan kulaklı kişiler: orta koyu cilt tonu orta cilt tonu',
    '🧑🏾‍🐰‍🧑🏿' => 'tavşan kulaklı kişiler: orta koyu cilt tonu koyu cilt tonu',
    '🧑🏿‍🐰‍🧑🏻' => 'tavşan kulaklı kişiler: koyu cilt tonu açık cilt tonu',
    '🧑🏿‍🐰‍🧑🏼' => 'tavşan kulaklı kişiler: koyu cilt tonu orta açık cilt tonu',
    '🧑🏿‍🐰‍🧑🏽' => 'tavşan kulaklı kişiler: koyu cilt tonu orta cilt tonu',
    '🧑🏿‍🐰‍🧑🏾' => 'tavşan kulaklı kişiler: koyu cilt tonu orta koyu cilt tonu',
    '👨🏻‍🫯‍👨🏼' => 'güreş yapan erkek: açık cilt tonu orta açık cilt tonu',
    '👨🏻‍🫯‍👨🏽' => 'güreş yapan erkek: açık cilt tonu orta cilt tonu',
    '👨🏻‍🫯‍👨🏾' => 'güreş yapan erkek: açık cilt tonu orta koyu cilt tonu',
    '👨🏻‍🫯‍👨🏿' => 'güreş yapan erkek: açık cilt tonu koyu cilt tonu',
    '👨🏼‍🫯‍👨🏻' => 'güreş yapan erkek: orta açık cilt tonu açık cilt tonu',
    '👨🏼‍🫯‍👨🏽' => 'güreş yapan erkek: orta açık cilt tonu orta cilt tonu',
    '👨🏼‍🫯‍👨🏾' => 'güreş yapan erkek: orta açık cilt tonu orta koyu cilt tonu',
    '👨🏼‍🫯‍👨🏿' => 'güreş yapan erkek: orta açık cilt tonu koyu cilt tonu',
    '👨🏽‍🫯‍👨🏻' => 'güreş yapan erkek: orta cilt tonu açık cilt tonu',
    '👨🏽‍🫯‍👨🏼' => 'güreş yapan erkek: orta cilt tonu orta açık cilt tonu',
    '👨🏽‍🫯‍👨🏾' => 'güreş yapan erkek: orta cilt tonu orta koyu cilt tonu',
    '👨🏽‍🫯‍👨🏿' => 'güreş yapan erkek: orta cilt tonu koyu cilt tonu',
    '👨🏾‍🫯‍👨🏻' => 'güreş yapan erkek: orta koyu cilt tonu açık cilt tonu',
    '👨🏾‍🫯‍👨🏼' => 'güreş yapan erkek: orta koyu cilt tonu orta açık cilt tonu',
    '👨🏾‍🫯‍👨🏽' => 'güreş yapan erkek: orta koyu cilt tonu orta cilt tonu',
    '👨🏾‍🫯‍👨🏿' => 'güreş yapan erkek: orta koyu cilt tonu koyu cilt tonu',
    '👨🏿‍🫯‍👨🏻' => 'güreş yapan erkek: koyu cilt tonu açık cilt tonu',
    '👨🏿‍🫯‍👨🏼' => 'güreş yapan erkek: koyu cilt tonu orta açık cilt tonu',
    '👨🏿‍🫯‍👨🏽' => 'güreş yapan erkek: koyu cilt tonu orta cilt tonu',
    '👨🏿‍🫯‍👨🏾' => 'güreş yapan erkek: koyu cilt tonu orta koyu cilt tonu',
    '👨🏻‍❤‍👨🏻' => 'kalpli çift: erkek erkek açık cilt tonu',
    '👨🏻‍❤‍👨🏼' => 'kalpli çift: erkek erkek açık cilt tonu orta açık cilt tonu',
    '👨🏻‍❤‍👨🏽' => 'kalpli çift: erkek erkek açık cilt tonu orta cilt tonu',
    '👨🏻‍❤‍👨🏾' => 'kalpli çift: erkek erkek açık cilt tonu orta koyu cilt tonu',
    '👨🏻‍❤‍👨🏿' => 'kalpli çift: erkek erkek açık cilt tonu koyu cilt tonu',
    '👨🏼‍❤‍👨🏻' => 'kalpli çift: erkek erkek orta açık cilt tonu açık cilt tonu',
    '👨🏼‍❤‍👨🏼' => 'kalpli çift: erkek erkek orta açık cilt tonu',
    '👨🏼‍❤‍👨🏽' => 'kalpli çift: erkek erkek orta açık cilt tonu orta cilt tonu',
    '👨🏼‍❤‍👨🏾' => 'kalpli çift: erkek erkek orta açık cilt tonu orta koyu cilt tonu',
    '👨🏼‍❤‍👨🏿' => 'kalpli çift: erkek erkek orta açık cilt tonu koyu cilt tonu',
    '👨🏽‍❤‍👨🏻' => 'kalpli çift: erkek erkek orta cilt tonu açık cilt tonu',
    '👨🏽‍❤‍👨🏼' => 'kalpli çift: erkek erkek orta cilt tonu orta açık cilt tonu',
    '👨🏽‍❤‍👨🏽' => 'kalpli çift: erkek erkek orta cilt tonu',
    '👨🏽‍❤‍👨🏾' => 'kalpli çift: erkek erkek orta cilt tonu orta koyu cilt tonu',
    '👨🏽‍❤‍👨🏿' => 'kalpli çift: erkek erkek orta cilt tonu koyu cilt tonu',
    '👨🏾‍❤‍👨🏻' => 'kalpli çift: erkek erkek orta koyu cilt tonu açık cilt tonu',
    '👨🏾‍❤‍👨🏼' => 'kalpli çift: erkek erkek orta koyu cilt tonu orta açık cilt tonu',
    '👨🏾‍❤‍👨🏽' => 'kalpli çift: erkek erkek orta koyu cilt tonu orta cilt tonu',
    '👨🏾‍❤‍👨🏾' => 'kalpli çift: erkek erkek orta koyu cilt tonu',
    '👨🏾‍❤‍👨🏿' => 'kalpli çift: erkek erkek orta koyu cilt tonu koyu cilt tonu',
    '👨🏿‍❤‍👨🏻' => 'kalpli çift: erkek erkek koyu cilt tonu açık cilt tonu',
    '👨🏿‍❤‍👨🏼' => 'kalpli çift: erkek erkek koyu cilt tonu orta açık cilt tonu',
    '👨🏿‍❤‍👨🏽' => 'kalpli çift: erkek erkek koyu cilt tonu orta cilt tonu',
    '👨🏿‍❤‍👨🏾' => 'kalpli çift: erkek erkek koyu cilt tonu orta koyu cilt tonu',
    '👨🏿‍❤‍👨🏿' => 'kalpli çift: erkek erkek koyu cilt tonu',
    '👨🏻‍🐰‍👨🏼' => 'tavşan kulaklı erkekler: açık cilt tonu orta açık cilt tonu',
    '👨🏻‍🐰‍👨🏽' => 'tavşan kulaklı erkekler: açık cilt tonu orta cilt tonu',
    '👨🏻‍🐰‍👨🏾' => 'tavşan kulaklı erkekler: açık cilt tonu orta koyu cilt tonu',
    '👨🏻‍🐰‍👨🏿' => 'tavşan kulaklı erkekler: açık cilt tonu koyu cilt tonu',
    '👨🏼‍🐰‍👨🏻' => 'tavşan kulaklı erkekler: orta açık cilt tonu açık cilt tonu',
    '👨🏼‍🐰‍👨🏽' => 'tavşan kulaklı erkekler: orta açık cilt tonu orta cilt tonu',
    '👨🏼‍🐰‍👨🏾' => 'tavşan kulaklı erkekler: orta açık cilt tonu orta koyu cilt tonu',
    '👨🏼‍🐰‍👨🏿' => 'tavşan kulaklı erkekler: orta açık cilt tonu koyu cilt tonu',
    '👨🏽‍🐰‍👨🏻' => 'tavşan kulaklı erkekler: orta cilt tonu açık cilt tonu',
    '👨🏽‍🐰‍👨🏼' => 'tavşan kulaklı erkekler: orta cilt tonu orta açık cilt tonu',
    '👨🏽‍🐰‍👨🏾' => 'tavşan kulaklı erkekler: orta cilt tonu orta koyu cilt tonu',
    '👨🏽‍🐰‍👨🏿' => 'tavşan kulaklı erkekler: orta cilt tonu koyu cilt tonu',
    '👨🏾‍🐰‍👨🏻' => 'tavşan kulaklı erkekler: orta koyu cilt tonu açık cilt tonu',
    '👨🏾‍🐰‍👨🏼' => 'tavşan kulaklı erkekler: orta koyu cilt tonu orta açık cilt tonu',
    '👨🏾‍🐰‍👨🏽' => 'tavşan kulaklı erkekler: orta koyu cilt tonu orta cilt tonu',
    '👨🏾‍🐰‍👨🏿' => 'tavşan kulaklı erkekler: orta koyu cilt tonu koyu cilt tonu',
    '👨🏿‍🐰‍👨🏻' => 'tavşan kulaklı erkekler: koyu cilt tonu açık cilt tonu',
    '👨🏿‍🐰‍👨🏼' => 'tavşan kulaklı erkekler: koyu cilt tonu orta açık cilt tonu',
    '👨🏿‍🐰‍👨🏽' => 'tavşan kulaklı erkekler: koyu cilt tonu orta cilt tonu',
    '👨🏿‍🐰‍👨🏾' => 'tavşan kulaklı erkekler: koyu cilt tonu orta koyu cilt tonu',
    '👩🏻‍🫯‍👩🏼' => 'güreş yapan kadın: açık cilt tonu orta açık cilt tonu',
    '👩🏻‍🫯‍👩🏽' => 'güreş yapan kadın: açık cilt tonu orta cilt tonu',
    '👩🏻‍🫯‍👩🏾' => 'güreş yapan kadın: açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍🫯‍👩🏿' => 'güreş yapan kadın: açık cilt tonu koyu cilt tonu',
    '👩🏼‍🫯‍👩🏻' => 'güreş yapan kadın: orta açık cilt tonu açık cilt tonu',
    '👩🏼‍🫯‍👩🏽' => 'güreş yapan kadın: orta açık cilt tonu orta cilt tonu',
    '👩🏼‍🫯‍👩🏾' => 'güreş yapan kadın: orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍🫯‍👩🏿' => 'güreş yapan kadın: orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍🫯‍👩🏻' => 'güreş yapan kadın: orta cilt tonu açık cilt tonu',
    '👩🏽‍🫯‍👩🏼' => 'güreş yapan kadın: orta cilt tonu orta açık cilt tonu',
    '👩🏽‍🫯‍👩🏾' => 'güreş yapan kadın: orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍🫯‍👩🏿' => 'güreş yapan kadın: orta cilt tonu koyu cilt tonu',
    '👩🏾‍🫯‍👩🏻' => 'güreş yapan kadın: orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍🫯‍👩🏼' => 'güreş yapan kadın: orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍🫯‍👩🏽' => 'güreş yapan kadın: orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍🫯‍👩🏿' => 'güreş yapan kadın: orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍🫯‍👩🏻' => 'güreş yapan kadın: koyu cilt tonu açık cilt tonu',
    '👩🏿‍🫯‍👩🏼' => 'güreş yapan kadın: koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍🫯‍👩🏽' => 'güreş yapan kadın: koyu cilt tonu orta cilt tonu',
    '👩🏿‍🫯‍👩🏾' => 'güreş yapan kadın: koyu cilt tonu orta koyu cilt tonu',
    '👩🏻‍❤‍👨🏻' => 'kalpli çift: kadın erkek açık cilt tonu',
    '👩🏻‍❤‍👨🏼' => 'kalpli çift: kadın erkek açık cilt tonu orta açık cilt tonu',
    '👩🏻‍❤‍👨🏽' => 'kalpli çift: kadın erkek açık cilt tonu orta cilt tonu',
    '👩🏻‍❤‍👨🏾' => 'kalpli çift: kadın erkek açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍❤‍👨🏿' => 'kalpli çift: kadın erkek açık cilt tonu koyu cilt tonu',
    '👩🏼‍❤‍👨🏻' => 'kalpli çift: kadın erkek orta açık cilt tonu açık cilt tonu',
    '👩🏼‍❤‍👨🏼' => 'kalpli çift: kadın erkek orta açık cilt tonu',
    '👩🏼‍❤‍👨🏽' => 'kalpli çift: kadın erkek orta açık cilt tonu orta cilt tonu',
    '👩🏼‍❤‍👨🏾' => 'kalpli çift: kadın erkek orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍❤‍👨🏿' => 'kalpli çift: kadın erkek orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍❤‍👨🏻' => 'kalpli çift: kadın erkek orta cilt tonu açık cilt tonu',
    '👩🏽‍❤‍👨🏼' => 'kalpli çift: kadın erkek orta cilt tonu orta açık cilt tonu',
    '👩🏽‍❤‍👨🏽' => 'kalpli çift: kadın erkek orta cilt tonu',
    '👩🏽‍❤‍👨🏾' => 'kalpli çift: kadın erkek orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍❤‍👨🏿' => 'kalpli çift: kadın erkek orta cilt tonu koyu cilt tonu',
    '👩🏾‍❤‍👨🏻' => 'kalpli çift: kadın erkek orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍❤‍👨🏼' => 'kalpli çift: kadın erkek orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍❤‍👨🏽' => 'kalpli çift: kadın erkek orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍❤‍👨🏾' => 'kalpli çift: kadın erkek orta koyu cilt tonu',
    '👩🏾‍❤‍👨🏿' => 'kalpli çift: kadın erkek orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍❤‍👨🏻' => 'kalpli çift: kadın erkek koyu cilt tonu açık cilt tonu',
    '👩🏿‍❤‍👨🏼' => 'kalpli çift: kadın erkek koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍❤‍👨🏽' => 'kalpli çift: kadın erkek koyu cilt tonu orta cilt tonu',
    '👩🏿‍❤‍👨🏾' => 'kalpli çift: kadın erkek koyu cilt tonu orta koyu cilt tonu',
    '👩🏿‍❤‍👨🏿' => 'kalpli çift: kadın erkek koyu cilt tonu',
    '👩🏻‍❤‍👩🏻' => 'kalpli çift: kadın kadın açık cilt tonu',
    '👩🏻‍❤‍👩🏼' => 'kalpli çift: kadın kadın açık cilt tonu orta açık cilt tonu',
    '👩🏻‍❤‍👩🏽' => 'kalpli çift: kadın kadın açık cilt tonu orta cilt tonu',
    '👩🏻‍❤‍👩🏾' => 'kalpli çift: kadın kadın açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍❤‍👩🏿' => 'kalpli çift: kadın kadın açık cilt tonu koyu cilt tonu',
    '👩🏼‍❤‍👩🏻' => 'kalpli çift: kadın kadın orta açık cilt tonu açık cilt tonu',
    '👩🏼‍❤‍👩🏼' => 'kalpli çift: kadın kadın orta açık cilt tonu',
    '👩🏼‍❤‍👩🏽' => 'kalpli çift: kadın kadın orta açık cilt tonu orta cilt tonu',
    '👩🏼‍❤‍👩🏾' => 'kalpli çift: kadın kadın orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍❤‍👩🏿' => 'kalpli çift: kadın kadın orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍❤‍👩🏻' => 'kalpli çift: kadın kadın orta cilt tonu açık cilt tonu',
    '👩🏽‍❤‍👩🏼' => 'kalpli çift: kadın kadın orta cilt tonu orta açık cilt tonu',
    '👩🏽‍❤‍👩🏽' => 'kalpli çift: kadın kadın orta cilt tonu',
    '👩🏽‍❤‍👩🏾' => 'kalpli çift: kadın kadın orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍❤‍👩🏿' => 'kalpli çift: kadın kadın orta cilt tonu koyu cilt tonu',
    '👩🏾‍❤‍👩🏻' => 'kalpli çift: kadın kadın orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍❤‍👩🏼' => 'kalpli çift: kadın kadın orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍❤‍👩🏽' => 'kalpli çift: kadın kadın orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍❤‍👩🏾' => 'kalpli çift: kadın kadın orta koyu cilt tonu',
    '👩🏾‍❤‍👩🏿' => 'kalpli çift: kadın kadın orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍❤‍👩🏻' => 'kalpli çift: kadın kadın koyu cilt tonu açık cilt tonu',
    '👩🏿‍❤‍👩🏼' => 'kalpli çift: kadın kadın koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍❤‍👩🏽' => 'kalpli çift: kadın kadın koyu cilt tonu orta cilt tonu',
    '👩🏿‍❤‍👩🏾' => 'kalpli çift: kadın kadın koyu cilt tonu orta koyu cilt tonu',
    '👩🏿‍❤‍👩🏿' => 'kalpli çift: kadın kadın koyu cilt tonu',
    '👩🏻‍🐰‍👩🏼' => 'tavşan kulaklı kadınlar: açık cilt tonu orta açık cilt tonu',
    '👩🏻‍🐰‍👩🏽' => 'tavşan kulaklı kadınlar: açık cilt tonu orta cilt tonu',
    '👩🏻‍🐰‍👩🏾' => 'tavşan kulaklı kadınlar: açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍🐰‍👩🏿' => 'tavşan kulaklı kadınlar: açık cilt tonu koyu cilt tonu',
    '👩🏼‍🐰‍👩🏻' => 'tavşan kulaklı kadınlar: orta açık cilt tonu açık cilt tonu',
    '👩🏼‍🐰‍👩🏽' => 'tavşan kulaklı kadınlar: orta açık cilt tonu orta cilt tonu',
    '👩🏼‍🐰‍👩🏾' => 'tavşan kulaklı kadınlar: orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍🐰‍👩🏿' => 'tavşan kulaklı kadınlar: orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍🐰‍👩🏻' => 'tavşan kulaklı kadınlar: orta cilt tonu açık cilt tonu',
    '👩🏽‍🐰‍👩🏼' => 'tavşan kulaklı kadınlar: orta cilt tonu orta açık cilt tonu',
    '👩🏽‍🐰‍👩🏾' => 'tavşan kulaklı kadınlar: orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍🐰‍👩🏿' => 'tavşan kulaklı kadınlar: orta cilt tonu koyu cilt tonu',
    '👩🏾‍🐰‍👩🏻' => 'tavşan kulaklı kadınlar: orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍🐰‍👩🏼' => 'tavşan kulaklı kadınlar: orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍🐰‍👩🏽' => 'tavşan kulaklı kadınlar: orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍🐰‍👩🏿' => 'tavşan kulaklı kadınlar: orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍🐰‍👩🏻' => 'tavşan kulaklı kadınlar: koyu cilt tonu açık cilt tonu',
    '👩🏿‍🐰‍👩🏼' => 'tavşan kulaklı kadınlar: koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍🐰‍👩🏽' => 'tavşan kulaklı kadınlar: koyu cilt tonu orta cilt tonu',
    '👩🏿‍🐰‍👩🏾' => 'tavşan kulaklı kadınlar: koyu cilt tonu orta koyu cilt tonu',
    '🧑🏻‍🤝‍🧑🏻' => 'el ele tutuşan insanlar: açık cilt tonu',
    '🧑🏻‍🤝‍🧑🏼' => 'el ele tutuşan insanlar: açık cilt tonu orta açık cilt tonu',
    '🧑🏻‍🤝‍🧑🏽' => 'el ele tutuşan insanlar: açık cilt tonu orta cilt tonu',
    '🧑🏻‍🤝‍🧑🏾' => 'el ele tutuşan insanlar: açık cilt tonu orta koyu cilt tonu',
    '🧑🏻‍🤝‍🧑🏿' => 'el ele tutuşan insanlar: açık cilt tonu koyu cilt tonu',
    '🧑🏼‍🤝‍🧑🏻' => 'el ele tutuşan insanlar: orta açık cilt tonu açık cilt tonu',
    '🧑🏼‍🤝‍🧑🏼' => 'el ele tutuşan insanlar: orta açık cilt tonu',
    '🧑🏼‍🤝‍🧑🏽' => 'el ele tutuşan insanlar: orta açık cilt tonu orta cilt tonu',
    '🧑🏼‍🤝‍🧑🏾' => 'el ele tutuşan insanlar: orta açık cilt tonu orta koyu cilt tonu',
    '🧑🏼‍🤝‍🧑🏿' => 'el ele tutuşan insanlar: orta açık cilt tonu koyu cilt tonu',
    '🧑🏽‍🤝‍🧑🏻' => 'el ele tutuşan insanlar: orta cilt tonu açık cilt tonu',
    '🧑🏽‍🤝‍🧑🏼' => 'el ele tutuşan insanlar: orta cilt tonu orta açık cilt tonu',
    '🧑🏽‍🤝‍🧑🏽' => 'el ele tutuşan insanlar: orta cilt tonu',
    '🧑🏽‍🤝‍🧑🏾' => 'el ele tutuşan insanlar: orta cilt tonu orta koyu cilt tonu',
    '🧑🏽‍🤝‍🧑🏿' => 'el ele tutuşan insanlar: orta cilt tonu koyu cilt tonu',
    '🧑🏾‍🤝‍🧑🏻' => 'el ele tutuşan insanlar: orta koyu cilt tonu açık cilt tonu',
    '🧑🏾‍🤝‍🧑🏼' => 'el ele tutuşan insanlar: orta koyu cilt tonu orta açık cilt tonu',
    '🧑🏾‍🤝‍🧑🏽' => 'el ele tutuşan insanlar: orta koyu cilt tonu orta cilt tonu',
    '🧑🏾‍🤝‍🧑🏾' => 'el ele tutuşan insanlar: orta koyu cilt tonu',
    '🧑🏾‍🤝‍🧑🏿' => 'el ele tutuşan insanlar: orta koyu cilt tonu koyu cilt tonu',
    '🧑🏿‍🤝‍🧑🏻' => 'el ele tutuşan insanlar: koyu cilt tonu açık cilt tonu',
    '🧑🏿‍🤝‍🧑🏼' => 'el ele tutuşan insanlar: koyu cilt tonu orta açık cilt tonu',
    '🧑🏿‍🤝‍🧑🏽' => 'el ele tutuşan insanlar: koyu cilt tonu orta cilt tonu',
    '🧑🏿‍🤝‍🧑🏾' => 'el ele tutuşan insanlar: koyu cilt tonu orta koyu cilt tonu',
    '🧑🏿‍🤝‍🧑🏿' => 'el ele tutuşan insanlar: koyu cilt tonu',
    '👩🏻‍🤝‍👩🏼' => 'el ele tutuşan kadınlar: açık cilt tonu orta açık cilt tonu',
    '👩🏻‍🤝‍👩🏽' => 'el ele tutuşan kadınlar: açık cilt tonu orta cilt tonu',
    '👩🏻‍🤝‍👩🏾' => 'el ele tutuşan kadınlar: açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍🤝‍👩🏿' => 'el ele tutuşan kadınlar: açık cilt tonu koyu cilt tonu',
    '👩🏼‍🤝‍👩🏻' => 'el ele tutuşan kadınlar: orta açık cilt tonu açık cilt tonu',
    '👩🏼‍🤝‍👩🏽' => 'el ele tutuşan kadınlar: orta açık cilt tonu orta cilt tonu',
    '👩🏼‍🤝‍👩🏾' => 'el ele tutuşan kadınlar: orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍🤝‍👩🏿' => 'el ele tutuşan kadınlar: orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍🤝‍👩🏻' => 'el ele tutuşan kadınlar: orta cilt tonu açık cilt tonu',
    '👩🏽‍🤝‍👩🏼' => 'el ele tutuşan kadınlar: orta cilt tonu orta açık cilt tonu',
    '👩🏽‍🤝‍👩🏾' => 'el ele tutuşan kadınlar: orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍🤝‍👩🏿' => 'el ele tutuşan kadınlar: orta cilt tonu koyu cilt tonu',
    '👩🏾‍🤝‍👩🏻' => 'el ele tutuşan kadınlar: orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍🤝‍👩🏼' => 'el ele tutuşan kadınlar: orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍🤝‍👩🏽' => 'el ele tutuşan kadınlar: orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍🤝‍👩🏿' => 'el ele tutuşan kadınlar: orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍🤝‍👩🏻' => 'el ele tutuşan kadınlar: koyu cilt tonu açık cilt tonu',
    '👩🏿‍🤝‍👩🏼' => 'el ele tutuşan kadınlar: koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍🤝‍👩🏽' => 'el ele tutuşan kadınlar: koyu cilt tonu orta cilt tonu',
    '👩🏿‍🤝‍👩🏾' => 'el ele tutuşan kadınlar: koyu cilt tonu orta koyu cilt tonu',
    '👩🏻‍🤝‍👨🏼' => 'el ele tutuşan erkek ve kadın: açık cilt tonu orta açık cilt tonu',
    '👩🏻‍🤝‍👨🏽' => 'el ele tutuşan erkek ve kadın: açık cilt tonu orta cilt tonu',
    '👩🏻‍🤝‍👨🏾' => 'el ele tutuşan erkek ve kadın: açık cilt tonu orta koyu cilt tonu',
    '👩🏻‍🤝‍👨🏿' => 'el ele tutuşan erkek ve kadın: açık cilt tonu koyu cilt tonu',
    '👩🏼‍🤝‍👨🏻' => 'el ele tutuşan erkek ve kadın: orta açık cilt tonu açık cilt tonu',
    '👩🏼‍🤝‍👨🏽' => 'el ele tutuşan erkek ve kadın: orta açık cilt tonu orta cilt tonu',
    '👩🏼‍🤝‍👨🏾' => 'el ele tutuşan erkek ve kadın: orta açık cilt tonu orta koyu cilt tonu',
    '👩🏼‍🤝‍👨🏿' => 'el ele tutuşan erkek ve kadın: orta açık cilt tonu koyu cilt tonu',
    '👩🏽‍🤝‍👨🏻' => 'el ele tutuşan erkek ve kadın: orta cilt tonu açık cilt tonu',
    '👩🏽‍🤝‍👨🏼' => 'el ele tutuşan erkek ve kadın: orta cilt tonu orta açık cilt tonu',
    '👩🏽‍🤝‍👨🏾' => 'el ele tutuşan erkek ve kadın: orta cilt tonu orta koyu cilt tonu',
    '👩🏽‍🤝‍👨🏿' => 'el ele tutuşan erkek ve kadın: orta cilt tonu koyu cilt tonu',
    '👩🏾‍🤝‍👨🏻' => 'el ele tutuşan erkek ve kadın: orta koyu cilt tonu açık cilt tonu',
    '👩🏾‍🤝‍👨🏼' => 'el ele tutuşan erkek ve kadın: orta koyu cilt tonu orta açık cilt tonu',
    '👩🏾‍🤝‍👨🏽' => 'el ele tutuşan erkek ve kadın: orta koyu cilt tonu orta cilt tonu',
    '👩🏾‍🤝‍👨🏿' => 'el ele tutuşan erkek ve kadın: orta koyu cilt tonu koyu cilt tonu',
    '👩🏿‍🤝‍👨🏻' => 'el ele tutuşan erkek ve kadın: koyu cilt tonu açık cilt tonu',
    '👩🏿‍🤝‍👨🏼' => 'el ele tutuşan erkek ve kadın: koyu cilt tonu orta açık cilt tonu',
    '👩🏿‍🤝‍👨🏽' => 'el ele tutuşan erkek ve kadın: koyu cilt tonu orta cilt tonu',
    '👩🏿‍🤝‍👨🏾' => 'el ele tutuşan erkek ve kadın: koyu cilt tonu orta koyu cilt tonu',
    '👨🏻‍🤝‍👨🏼' => 'el ele tutuşan erkekler: açık cilt tonu orta açık cilt tonu',
    '👨🏻‍🤝‍👨🏽' => 'el ele tutuşan erkekler: açık cilt tonu orta cilt tonu',
    '👨🏻‍🤝‍👨🏾' => 'el ele tutuşan erkekler: açık cilt tonu orta koyu cilt tonu',
    '👨🏻‍🤝‍👨🏿' => 'el ele tutuşan erkekler: açık cilt tonu koyu cilt tonu',
    '👨🏼‍🤝‍👨🏻' => 'el ele tutuşan erkekler: orta açık cilt tonu açık cilt tonu',
    '👨🏼‍🤝‍👨🏽' => 'el ele tutuşan erkekler: orta açık cilt tonu orta cilt tonu',
    '👨🏼‍🤝‍👨🏾' => 'el ele tutuşan erkekler: orta açık cilt tonu orta koyu cilt tonu',
    '👨🏼‍🤝‍👨🏿' => 'el ele tutuşan erkekler: orta açık cilt tonu koyu cilt tonu',
    '👨🏽‍🤝‍👨🏻' => 'el ele tutuşan erkekler: orta cilt tonu açık cilt tonu',
    '👨🏽‍🤝‍👨🏼' => 'el ele tutuşan erkekler: orta cilt tonu orta açık cilt tonu',
    '👨🏽‍🤝‍👨🏾' => 'el ele tutuşan erkekler: orta cilt tonu orta koyu cilt tonu',
    '👨🏽‍🤝‍👨🏿' => 'el ele tutuşan erkekler: orta cilt tonu koyu cilt tonu',
    '👨🏾‍🤝‍👨🏻' => 'el ele tutuşan erkekler: orta koyu cilt tonu açık cilt tonu',
    '👨🏾‍🤝‍👨🏼' => 'el ele tutuşan erkekler: orta koyu cilt tonu orta açık cilt tonu',
    '👨🏾‍🤝‍👨🏽' => 'el ele tutuşan erkekler: orta koyu cilt tonu orta cilt tonu',
    '👨🏾‍🤝‍👨🏿' => 'el ele tutuşan erkekler: orta koyu cilt tonu koyu cilt tonu',
    '👨🏿‍🤝‍👨🏻' => 'el ele tutuşan erkekler: koyu cilt tonu açık cilt tonu',
    '👨🏿‍🤝‍👨🏼' => 'el ele tutuşan erkekler: koyu cilt tonu orta açık cilt tonu',
    '👨🏿‍🤝‍👨🏽' => 'el ele tutuşan erkekler: koyu cilt tonu orta cilt tonu',
    '👨🏿‍🤝‍👨🏾' => 'el ele tutuşan erkekler: koyu cilt tonu orta koyu cilt tonu',
    '👩‍❤‍💋‍👨' => 'öpücük: kadın erkek',
    '👨‍❤‍💋‍👨' => 'öpücük: erkek erkek',
    '👩‍❤‍💋‍👩' => 'öpücük: kadın kadın',
    '👨‍👩‍👧‍👦' => 'aile: erkek kadın kız çocuk erkek çocuk',
    '👨‍👩‍👦‍👦' => 'aile: erkek kadın erkek çocuk erkek çocuk',
    '👨‍👩‍👧‍👧' => 'aile: erkek kadın kız çocuk kız çocuk',
    '👨‍👨‍👧‍👦' => 'aile: erkek erkek kız çocuk erkek çocuk',
    '👨‍👨‍👦‍👦' => 'aile: erkek erkek erkek çocuk erkek çocuk',
    '👨‍👨‍👧‍👧' => 'aile: erkek erkek kız çocuk kız çocuk',
    '👩‍👩‍👧‍👦' => 'aile: kadın kadın kız çocuk erkek çocuk',
    '👩‍👩‍👦‍👦' => 'aile: kadın kadın erkek çocuk erkek çocuk',
    '👩‍👩‍👧‍👧' => 'aile: kadın kadın kız çocuk kız çocuk',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'bayrak: İngiltere',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'bayrak: İskoçya',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'bayrak: Galler',
    '🧑‍🧑‍🧒‍🧒' => 'aile: yetişkin, yetişkin, çocuk, çocuk',
    '🚶🏻‍♀‍➡' => 'yürüyen kadın: açık cilt tonu sağa dönük',
    '🚶🏼‍♀‍➡' => 'yürüyen kadın: orta açık cilt tonu sağa dönük',
    '🚶🏽‍♀‍➡' => 'yürüyen kadın: orta cilt tonu sağa dönük',
    '🚶🏾‍♀‍➡' => 'yürüyen kadın: orta koyu cilt tonu sağa dönük',
    '🚶🏿‍♀‍➡' => 'yürüyen kadın: koyu cilt tonu sağa dönük',
    '🚶🏻‍♂‍➡' => 'yürüyen erkek: açık cilt tonu sağa dönük',
    '🚶🏼‍♂‍➡' => 'yürüyen erkek: orta açık cilt tonu sağa dönük',
    '🚶🏽‍♂‍➡' => 'yürüyen erkek: orta cilt tonu sağa dönük',
    '🚶🏾‍♂‍➡' => 'yürüyen erkek: orta koyu cilt tonu sağa dönük',
    '🚶🏿‍♂‍➡' => 'yürüyen erkek: koyu cilt tonu sağa dönük',
    '🧎🏻‍♀‍➡' => 'diz çöken kadın: açık cilt tonu sağa dönük',
    '🧎🏼‍♀‍➡' => 'diz çöken kadın: orta açık cilt tonu sağa dönük',
    '🧎🏽‍♀‍➡' => 'diz çöken kadın: orta cilt tonu sağa dönük',
    '🧎🏾‍♀‍➡' => 'diz çöken kadın: orta koyu cilt tonu sağa dönük',
    '🧎🏿‍♀‍➡' => 'diz çöken kadın: koyu cilt tonu sağa dönük',
    '🧎🏻‍♂‍➡' => 'diz çöken erkek: açık cilt tonu sağa dönük',
    '🧎🏼‍♂‍➡' => 'diz çöken erkek: orta açık cilt tonu sağa dönük',
    '🧎🏽‍♂‍➡' => 'diz çöken erkek: orta cilt tonu sağa dönük',
    '🧎🏾‍♂‍➡' => 'diz çöken erkek: orta koyu cilt tonu sağa dönük',
    '🧎🏿‍♂‍➡' => 'diz çöken erkek: koyu cilt tonu sağa dönük',
    '🧑🏻‍🦯‍➡' => 'görme engelli bastonlu kişi: açık cilt tonu sağa dönük',
    '🧑🏼‍🦯‍➡' => 'görme engelli bastonlu kişi: orta açık cilt tonu sağa dönük',
    '🧑🏽‍🦯‍➡' => 'görme engelli bastonlu kişi: orta cilt tonu sağa dönük',
    '🧑🏾‍🦯‍➡' => 'görme engelli bastonlu kişi: orta koyu cilt tonu sağa dönük',
    '🧑🏿‍🦯‍➡' => 'görme engelli bastonlu kişi: koyu cilt tonu sağa dönük',
    '👨🏻‍🦯‍➡' => 'görme engelli bastonlu erkek: açık cilt tonu sağa dönük',
    '👨🏼‍🦯‍➡' => 'görme engelli bastonlu erkek: orta açık cilt tonu sağa dönük',
    '👨🏽‍🦯‍➡' => 'görme engelli bastonlu erkek: orta cilt tonu sağa dönük',
    '👨🏾‍🦯‍➡' => 'görme engelli bastonlu erkek: orta koyu cilt tonu sağa dönük',
    '👨🏿‍🦯‍➡' => 'görme engelli bastonlu erkek: koyu cilt tonu sağa dönük',
    '👩🏻‍🦯‍➡' => 'görme engelli bastonlu kadın: açık cilt tonu sağa dönük',
    '👩🏼‍🦯‍➡' => 'görme engelli bastonlu kadın: orta açık cilt tonu sağa dönük',
    '👩🏽‍🦯‍➡' => 'görme engelli bastonlu kadın: orta cilt tonu sağa dönük',
    '👩🏾‍🦯‍➡' => 'görme engelli bastonlu kadın: orta koyu cilt tonu sağa dönük',
    '👩🏿‍🦯‍➡' => 'görme engelli bastonlu kadın: koyu cilt tonu sağa dönük',
    '🧑🏻‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kişi: açık cilt tonu sağa dönük',
    '🧑🏼‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kişi: orta açık cilt tonu sağa dönük',
    '🧑🏽‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kişi: orta cilt tonu sağa dönük',
    '🧑🏾‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kişi: orta koyu cilt tonu sağa dönük',
    '🧑🏿‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kişi: koyu cilt tonu sağa dönük',
    '👨🏻‍🦼‍➡' => 'elektrikli tekerlekli sandalyede erkek: açık cilt tonu sağa dönük',
    '👨🏼‍🦼‍➡' => 'elektrikli tekerlekli sandalyede erkek: orta açık cilt tonu sağa dönük',
    '👨🏽‍🦼‍➡' => 'elektrikli tekerlekli sandalyede erkek: orta cilt tonu sağa dönük',
    '👨🏾‍🦼‍➡' => 'elektrikli tekerlekli sandalyede erkek: orta koyu cilt tonu sağa dönük',
    '👨🏿‍🦼‍➡' => 'elektrikli tekerlekli sandalyede erkek: koyu cilt tonu sağa dönük',
    '👩🏻‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kadın: açık cilt tonu sağa dönük',
    '👩🏼‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kadın: orta açık cilt tonu sağa dönük',
    '👩🏽‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kadın: orta cilt tonu sağa dönük',
    '👩🏾‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kadın: orta koyu cilt tonu sağa dönük',
    '👩🏿‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kadın: koyu cilt tonu sağa dönük',
    '🧑🏻‍🦽‍➡' => 'tekerlekli sandalyede kişi: açık cilt tonu sağa dönük',
    '🧑🏼‍🦽‍➡' => 'tekerlekli sandalyede kişi: orta açık cilt tonu sağa dönük',
    '🧑🏽‍🦽‍➡' => 'tekerlekli sandalyede kişi: orta cilt tonu sağa dönük',
    '🧑🏾‍🦽‍➡' => 'tekerlekli sandalyede kişi: orta koyu cilt tonu sağa dönük',
    '🧑🏿‍🦽‍➡' => 'tekerlekli sandalyede kişi: koyu cilt tonu sağa dönük',
    '👨🏻‍🦽‍➡' => 'tekerlekli sandalyede erkek: açık cilt tonu sağa dönük',
    '👨🏼‍🦽‍➡' => 'tekerlekli sandalyede erkek: orta açık cilt tonu sağa dönük',
    '👨🏽‍🦽‍➡' => 'tekerlekli sandalyede erkek: orta cilt tonu sağa dönük',
    '👨🏾‍🦽‍➡' => 'tekerlekli sandalyede erkek: orta koyu cilt tonu sağa dönük',
    '👨🏿‍🦽‍➡' => 'tekerlekli sandalyede erkek: koyu cilt tonu sağa dönük',
    '👩🏻‍🦽‍➡' => 'tekerlekli sandalyede kadın: açık cilt tonu sağa dönük',
    '👩🏼‍🦽‍➡' => 'tekerlekli sandalyede kadın: orta açık cilt tonu sağa dönük',
    '👩🏽‍🦽‍➡' => 'tekerlekli sandalyede kadın: orta cilt tonu sağa dönük',
    '👩🏾‍🦽‍➡' => 'tekerlekli sandalyede kadın: orta koyu cilt tonu sağa dönük',
    '👩🏿‍🦽‍➡' => 'tekerlekli sandalyede kadın: koyu cilt tonu sağa dönük',
    '🏃🏻‍♀‍➡' => 'koşan kadın: açık cilt tonu sağa dönük',
    '🏃🏼‍♀‍➡' => 'koşan kadın: orta açık cilt tonu sağa dönük',
    '🏃🏽‍♀‍➡' => 'koşan kadın: orta cilt tonu sağa dönük',
    '🏃🏾‍♀‍➡' => 'koşan kadın: orta koyu cilt tonu sağa dönük',
    '🏃🏿‍♀‍➡' => 'koşan kadın: koyu cilt tonu sağa dönük',
    '🏃🏻‍♂‍➡' => 'koşan erkek: açık cilt tonu sağa dönük',
    '🏃🏼‍♂‍➡' => 'koşan erkek: orta açık cilt tonu sağa dönük',
    '🏃🏽‍♂‍➡' => 'koşan erkek: orta cilt tonu sağa dönük',
    '🏃🏾‍♂‍➡' => 'koşan erkek: orta koyu cilt tonu sağa dönük',
    '🏃🏿‍♂‍➡' => 'koşan erkek: koyu cilt tonu sağa dönük',
    '🫱🏻‍🫲🏼' => 'el sıkışma: açık cilt tonu orta açık cilt tonu',
    '🫱🏻‍🫲🏽' => 'el sıkışma: açık cilt tonu orta cilt tonu',
    '🫱🏻‍🫲🏾' => 'el sıkışma: açık cilt tonu orta koyu cilt tonu',
    '🫱🏻‍🫲🏿' => 'el sıkışma: açık cilt tonu koyu cilt tonu',
    '🫱🏼‍🫲🏻' => 'el sıkışma: orta açık cilt tonu açık cilt tonu',
    '🫱🏼‍🫲🏽' => 'el sıkışma: orta açık cilt tonu orta cilt tonu',
    '🫱🏼‍🫲🏾' => 'el sıkışma: orta açık cilt tonu orta koyu cilt tonu',
    '🫱🏼‍🫲🏿' => 'el sıkışma: orta açık cilt tonu koyu cilt tonu',
    '🫱🏽‍🫲🏻' => 'el sıkışma: orta cilt tonu açık cilt tonu',
    '🫱🏽‍🫲🏼' => 'el sıkışma: orta cilt tonu orta açık cilt tonu',
    '🫱🏽‍🫲🏾' => 'el sıkışma: orta cilt tonu orta koyu cilt tonu',
    '🫱🏽‍🫲🏿' => 'el sıkışma: orta cilt tonu koyu cilt tonu',
    '🫱🏾‍🫲🏻' => 'el sıkışma: orta koyu cilt tonu açık cilt tonu',
    '🫱🏾‍🫲🏼' => 'el sıkışma: orta koyu cilt tonu orta açık cilt tonu',
    '🫱🏾‍🫲🏽' => 'el sıkışma: orta koyu cilt tonu orta cilt tonu',
    '🫱🏾‍🫲🏿' => 'el sıkışma: orta koyu cilt tonu koyu cilt tonu',
    '🫱🏿‍🫲🏻' => 'el sıkışma: koyu cilt tonu açık cilt tonu',
    '🫱🏿‍🫲🏼' => 'el sıkışma: koyu cilt tonu orta açık cilt tonu',
    '🫱🏿‍🫲🏽' => 'el sıkışma: koyu cilt tonu orta cilt tonu',
    '🫱🏿‍🫲🏾' => 'el sıkışma: koyu cilt tonu orta koyu cilt tonu',
    '🚶‍♀‍➡' => 'yürüyen kadın: sağa dönük',
    '🚶‍♂‍➡' => 'yürüyen erkek: sağa dönük',
    '🧎‍♀‍➡' => 'diz çöken kadın: sağa dönük',
    '🧎‍♂‍➡' => 'diz çöken erkek: sağa dönük',
    '🧑‍🦯‍➡' => 'görme engelli bastonlu kişi: sağa dönük',
    '👨‍🦯‍➡' => 'görme engelli bastonlu erkek: sağa dönük',
    '👩‍🦯‍➡' => 'görme engelli bastonlu kadın: sağa dönük',
    '🧑‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kişi: sağa dönük',
    '👨‍🦼‍➡' => 'elektrikli tekerlekli sandalyede erkek: sağa dönük',
    '👩‍🦼‍➡' => 'elektrikli tekerlekli sandalyede kadın: sağa dönük',
    '🧑‍🦽‍➡' => 'tekerlekli sandalyede kişi: sağa dönük',
    '👨‍🦽‍➡' => 'tekerlekli sandalyede erkek: sağa dönük',
    '👩‍🦽‍➡' => 'tekerlekli sandalyede kadın: sağa dönük',
    '🏃‍♀‍➡' => 'koşan kadın: sağa dönük',
    '🏃‍♂‍➡' => 'koşan erkek: sağa dönük',
    '👩‍❤‍👨' => 'kalpli çift: kadın erkek',
    '👨‍❤‍👨' => 'kalpli çift: erkek erkek',
    '👩‍❤‍👩' => 'kalpli çift: kadın kadın',
    '👨‍👩‍👦' => 'aile: erkek kadın erkek çocuk',
    '👨‍👩‍👧' => 'aile: erkek kadın kız çocuk',
    '👨‍👨‍👦' => 'aile: erkek erkek erkek çocuk',
    '👨‍👨‍👧' => 'aile: erkek erkek kız çocuk',
    '👩‍👩‍👦' => 'aile: kadın kadın erkek çocuk',
    '👩‍👩‍👧' => 'aile: kadın kadın kız çocuk',
    '👨‍👦‍👦' => 'aile: erkek erkek çocuk erkek çocuk',
    '👨‍👧‍👦' => 'aile: erkek kız çocuk erkek çocuk',
    '👨‍👧‍👧' => 'aile: erkek kız çocuk kız çocuk',
    '👩‍👦‍👦' => 'aile: kadın erkek çocuk erkek çocuk',
    '👩‍👧‍👦' => 'aile: kadın kız çocuk erkek çocuk',
    '👩‍👧‍👧' => 'aile: kadın kız çocuk kız çocuk',
    '🧑‍🤝‍🧑' => 'el ele tutuşan insanlar',
    '🧑‍🧑‍🧒' => 'aile: yetişkin, yetişkin, çocuk',
    '🧑‍🧒‍🧒' => 'aile: yetişkin, çocuk, çocuk',
    '🧑🏻‍🦰' => 'yetişkin: açık cilt tonu kızıl saç',
    '🧑🏼‍🦰' => 'yetişkin: orta açık cilt tonu kızıl saç',
    '🧑🏽‍🦰' => 'yetişkin: orta cilt tonu kızıl saç',
    '🧑🏾‍🦰' => 'yetişkin: orta koyu cilt tonu kızıl saç',
    '🧑🏿‍🦰' => 'yetişkin: koyu cilt tonu kızıl saç',
    '🧑🏻‍🦱' => 'yetişkin: açık cilt tonu kıvırcık saç',
    '🧑🏼‍🦱' => 'yetişkin: orta açık cilt tonu kıvırcık saç',
    '🧑🏽‍🦱' => 'yetişkin: orta cilt tonu kıvırcık saç',
    '🧑🏾‍🦱' => 'yetişkin: orta koyu cilt tonu kıvırcık saç',
    '🧑🏿‍🦱' => 'yetişkin: koyu cilt tonu kıvırcık saç',
    '🧑🏻‍🦳' => 'yetişkin: açık cilt tonu beyaz saç',
    '🧑🏼‍🦳' => 'yetişkin: orta açık cilt tonu beyaz saç',
    '🧑🏽‍🦳' => 'yetişkin: orta cilt tonu beyaz saç',
    '🧑🏾‍🦳' => 'yetişkin: orta koyu cilt tonu beyaz saç',
    '🧑🏿‍🦳' => 'yetişkin: koyu cilt tonu beyaz saç',
    '🧑🏻‍🦲' => 'yetişkin: açık cilt tonu kel',
    '🧑🏼‍🦲' => 'yetişkin: orta açık cilt tonu kel',
    '🧑🏽‍🦲' => 'yetişkin: orta cilt tonu kel',
    '🧑🏾‍🦲' => 'yetişkin: orta koyu cilt tonu kel',
    '🧑🏿‍🦲' => 'yetişkin: koyu cilt tonu kel',
    '🧑🏻‍🩰' => 'bale dansçısı: açık cilt tonu',
    '🧑🏼‍🩰' => 'bale dansçısı: orta açık cilt tonu',
    '🧑🏽‍🩰' => 'bale dansçısı: orta cilt tonu',
    '🧑🏾‍🩰' => 'bale dansçısı: orta koyu cilt tonu',
    '🧑🏿‍🩰' => 'bale dansçısı: koyu cilt tonu',
    '🧔🏻‍♂' => 'erkek: açık cilt tonu sakal',
    '🧔🏼‍♂' => 'erkek: orta açık cilt tonu sakal',
    '🧔🏽‍♂' => 'erkek: orta cilt tonu sakal',
    '🧔🏾‍♂' => 'erkek: orta koyu cilt tonu sakal',
    '🧔🏿‍♂' => 'erkek: koyu cilt tonu sakal',
    '👱🏻‍♂' => 'sarışın erkek: açık cilt tonu',
    '👱🏼‍♂' => 'sarışın erkek: orta açık cilt tonu',
    '👱🏽‍♂' => 'sarışın erkek: orta cilt tonu',
    '👱🏾‍♂' => 'sarışın erkek: orta koyu cilt tonu',
    '👱🏿‍♂' => 'sarışın erkek: koyu cilt tonu',
    '👨🏻‍🦰' => 'erkek: açık cilt tonu kızıl saç',
    '👨🏼‍🦰' => 'erkek: orta açık cilt tonu kızıl saç',
    '👨🏽‍🦰' => 'erkek: orta cilt tonu kızıl saç',
    '👨🏾‍🦰' => 'erkek: orta koyu cilt tonu kızıl saç',
    '👨🏿‍🦰' => 'erkek: koyu cilt tonu kızıl saç',
    '👨🏻‍🦱' => 'erkek: açık cilt tonu kıvırcık saç',
    '👨🏼‍🦱' => 'erkek: orta açık cilt tonu kıvırcık saç',
    '👨🏽‍🦱' => 'erkek: orta cilt tonu kıvırcık saç',
    '👨🏾‍🦱' => 'erkek: orta koyu cilt tonu kıvırcık saç',
    '👨🏿‍🦱' => 'erkek: koyu cilt tonu kıvırcık saç',
    '👨🏻‍🦳' => 'erkek: açık cilt tonu beyaz saç',
    '👨🏼‍🦳' => 'erkek: orta açık cilt tonu beyaz saç',
    '👨🏽‍🦳' => 'erkek: orta cilt tonu beyaz saç',
    '👨🏾‍🦳' => 'erkek: orta koyu cilt tonu beyaz saç',
    '👨🏿‍🦳' => 'erkek: koyu cilt tonu beyaz saç',
    '👨🏻‍🦲' => 'erkek: açık cilt tonu kel',
    '👨🏼‍🦲' => 'erkek: orta açık cilt tonu kel',
    '👨🏽‍🦲' => 'erkek: orta cilt tonu kel',
    '👨🏾‍🦲' => 'erkek: orta koyu cilt tonu kel',
    '👨🏿‍🦲' => 'erkek: koyu cilt tonu kel',
    '🧔🏻‍♀' => 'kadın: açık cilt tonu sakal',
    '🧔🏼‍♀' => 'kadın: orta açık cilt tonu sakal',
    '🧔🏽‍♀' => 'kadın: orta cilt tonu sakal',
    '🧔🏾‍♀' => 'kadın: orta koyu cilt tonu sakal',
    '🧔🏿‍♀' => 'kadın: koyu cilt tonu sakal',
    '👱🏻‍♀' => 'sarışın kadın: açık cilt tonu',
    '👱🏼‍♀' => 'sarışın kadın: orta açık cilt tonu',
    '👱🏽‍♀' => 'sarışın kadın: orta cilt tonu',
    '👱🏾‍♀' => 'sarışın kadın: orta koyu cilt tonu',
    '👱🏿‍♀' => 'sarışın kadın: koyu cilt tonu',
    '👩🏻‍🦰' => 'kadın: açık cilt tonu kızıl saç',
    '👩🏼‍🦰' => 'kadın: orta açık cilt tonu kızıl saç',
    '👩🏽‍🦰' => 'kadın: orta cilt tonu kızıl saç',
    '👩🏾‍🦰' => 'kadın: orta koyu cilt tonu kızıl saç',
    '👩🏿‍🦰' => 'kadın: koyu cilt tonu kızıl saç',
    '👩🏻‍🦱' => 'kadın: açık cilt tonu kıvırcık saç',
    '👩🏼‍🦱' => 'kadın: orta açık cilt tonu kıvırcık saç',
    '👩🏽‍🦱' => 'kadın: orta cilt tonu kıvırcık saç',
    '👩🏾‍🦱' => 'kadın: orta koyu cilt tonu kıvırcık saç',
    '👩🏿‍🦱' => 'kadın: koyu cilt tonu kıvırcık saç',
    '👩🏻‍🦳' => 'kadın: açık cilt tonu beyaz saç',
    '👩🏼‍🦳' => 'kadın: orta açık cilt tonu beyaz saç',
    '👩🏽‍🦳' => 'kadın: orta cilt tonu beyaz saç',
    '👩🏾‍🦳' => 'kadın: orta koyu cilt tonu beyaz saç',
    '👩🏿‍🦳' => 'kadın: koyu cilt tonu beyaz saç',
    '👩🏻‍🦲' => 'kadın: açık cilt tonu kel',
    '👩🏼‍🦲' => 'kadın: orta açık cilt tonu kel',
    '👩🏽‍🦲' => 'kadın: orta cilt tonu kel',
    '👩🏾‍🦲' => 'kadın: orta koyu cilt tonu kel',
    '👩🏿‍🦲' => 'kadın: koyu cilt tonu kel',
    '🙍🏻‍♂' => 'somurtan erkek: açık cilt tonu',
    '🙍🏼‍♂' => 'somurtan erkek: orta açık cilt tonu',
    '🙍🏽‍♂' => 'somurtan erkek: orta cilt tonu',
    '🙍🏾‍♂' => 'somurtan erkek: orta koyu cilt tonu',
    '🙍🏿‍♂' => 'somurtan erkek: koyu cilt tonu',
    '🙍🏻‍♀' => 'somurtan kadın: açık cilt tonu',
    '🙍🏼‍♀' => 'somurtan kadın: orta açık cilt tonu',
    '🙍🏽‍♀' => 'somurtan kadın: orta cilt tonu',
    '🙍🏾‍♀' => 'somurtan kadın: orta koyu cilt tonu',
    '🙍🏿‍♀' => 'somurtan kadın: koyu cilt tonu',
    '🙎🏻‍♂' => 'darılan erkek: açık cilt tonu',
    '🙎🏼‍♂' => 'darılan erkek: orta açık cilt tonu',
    '🙎🏽‍♂' => 'darılan erkek: orta cilt tonu',
    '🙎🏾‍♂' => 'darılan erkek: orta koyu cilt tonu',
    '🙎🏿‍♂' => 'darılan erkek: koyu cilt tonu',
    '🙎🏻‍♀' => 'darılan kadın: açık cilt tonu',
    '🙎🏼‍♀' => 'darılan kadın: orta açık cilt tonu',
    '🙎🏽‍♀' => 'darılan kadın: orta cilt tonu',
    '🙎🏾‍♀' => 'darılan kadın: orta koyu cilt tonu',
    '🙎🏿‍♀' => 'darılan kadın: koyu cilt tonu',
    '🙅🏻‍♂' => 'hayır hareketi yapan erkek: açık cilt tonu',
    '🙅🏼‍♂' => 'hayır hareketi yapan erkek: orta açık cilt tonu',
    '🙅🏽‍♂' => 'hayır hareketi yapan erkek: orta cilt tonu',
    '🙅🏾‍♂' => 'hayır hareketi yapan erkek: orta koyu cilt tonu',
    '🙅🏿‍♂' => 'hayır hareketi yapan erkek: koyu cilt tonu',
    '🙅🏻‍♀' => 'hayır hareketi yapan kadın: açık cilt tonu',
    '🙅🏼‍♀' => 'hayır hareketi yapan kadın: orta açık cilt tonu',
    '🙅🏽‍♀' => 'hayır hareketi yapan kadın: orta cilt tonu',
    '🙅🏾‍♀' => 'hayır hareketi yapan kadın: orta koyu cilt tonu',
    '🙅🏿‍♀' => 'hayır hareketi yapan kadın: koyu cilt tonu',
    '🙆🏻‍♂' => 'tamam hareketi yapan erkek: açık cilt tonu',
    '🙆🏼‍♂' => 'tamam hareketi yapan erkek: orta açık cilt tonu',
    '🙆🏽‍♂' => 'tamam hareketi yapan erkek: orta cilt tonu',
    '🙆🏾‍♂' => 'tamam hareketi yapan erkek: orta koyu cilt tonu',
    '🙆🏿‍♂' => 'tamam hareketi yapan erkek: koyu cilt tonu',
    '🙆🏻‍♀' => 'tamam hareketi yapan kadın: açık cilt tonu',
    '🙆🏼‍♀' => 'tamam hareketi yapan kadın: orta açık cilt tonu',
    '🙆🏽‍♀' => 'tamam hareketi yapan kadın: orta cilt tonu',
    '🙆🏾‍♀' => 'tamam hareketi yapan kadın: orta koyu cilt tonu',
    '🙆🏿‍♀' => 'tamam hareketi yapan kadın: koyu cilt tonu',
    '💁🏻‍♂' => 'elini yana yatıran erkek: açık cilt tonu',
    '💁🏼‍♂' => 'elini yana yatıran erkek: orta açık cilt tonu',
    '💁🏽‍♂' => 'elini yana yatıran erkek: orta cilt tonu',
    '💁🏾‍♂' => 'elini yana yatıran erkek: orta koyu cilt tonu',
    '💁🏿‍♂' => 'elini yana yatıran erkek: koyu cilt tonu',
    '💁🏻‍♀' => 'elini yana yatıran kadın: açık cilt tonu',
    '💁🏼‍♀' => 'elini yana yatıran kadın: orta açık cilt tonu',
    '💁🏽‍♀' => 'elini yana yatıran kadın: orta cilt tonu',
    '💁🏾‍♀' => 'elini yana yatıran kadın: orta koyu cilt tonu',
    '💁🏿‍♀' => 'elini yana yatıran kadın: koyu cilt tonu',
    '🙋🏻‍♂' => 'elini kaldıran erkek: açık cilt tonu',
    '🙋🏼‍♂' => 'elini kaldıran erkek: orta açık cilt tonu',
    '🙋🏽‍♂' => 'elini kaldıran erkek: orta cilt tonu',
    '🙋🏾‍♂' => 'elini kaldıran erkek: orta koyu cilt tonu',
    '🙋🏿‍♂' => 'elini kaldıran erkek: koyu cilt tonu',
    '🙋🏻‍♀' => 'elini kaldıran kadın: açık cilt tonu',
    '🙋🏼‍♀' => 'elini kaldıran kadın: orta açık cilt tonu',
    '🙋🏽‍♀' => 'elini kaldıran kadın: orta cilt tonu',
    '🙋🏾‍♀' => 'elini kaldıran kadın: orta koyu cilt tonu',
    '🙋🏿‍♀' => 'elini kaldıran kadın: koyu cilt tonu',
    '🧏🏻‍♂' => 'işitme engelli erkek: açık cilt tonu',
    '🧏🏼‍♂' => 'işitme engelli erkek: orta açık cilt tonu',
    '🧏🏽‍♂' => 'işitme engelli erkek: orta cilt tonu',
    '🧏🏾‍♂' => 'işitme engelli erkek: orta koyu cilt tonu',
    '🧏🏿‍♂' => 'işitme engelli erkek: koyu cilt tonu',
    '🧏🏻‍♀' => 'işitme engelli kadın: açık cilt tonu',
    '🧏🏼‍♀' => 'işitme engelli kadın: orta açık cilt tonu',
    '🧏🏽‍♀' => 'işitme engelli kadın: orta cilt tonu',
    '🧏🏾‍♀' => 'işitme engelli kadın: orta koyu cilt tonu',
    '🧏🏿‍♀' => 'işitme engelli kadın: koyu cilt tonu',
    '🙇🏻‍♂' => 'başını öne eğen erkek: açık cilt tonu',
    '🙇🏼‍♂' => 'başını öne eğen erkek: orta açık cilt tonu',
    '🙇🏽‍♂' => 'başını öne eğen erkek: orta cilt tonu',
    '🙇🏾‍♂' => 'başını öne eğen erkek: orta koyu cilt tonu',
    '🙇🏿‍♂' => 'başını öne eğen erkek: koyu cilt tonu',
    '🙇🏻‍♀' => 'başını öne eğen kadın: açık cilt tonu',
    '🙇🏼‍♀' => 'başını öne eğen kadın: orta açık cilt tonu',
    '🙇🏽‍♀' => 'başını öne eğen kadın: orta cilt tonu',
    '🙇🏾‍♀' => 'başını öne eğen kadın: orta koyu cilt tonu',
    '🙇🏿‍♀' => 'başını öne eğen kadın: koyu cilt tonu',
    '🤦🏻‍♂' => 'eliyle yüzünü kapatan erkek: açık cilt tonu',
    '🤦🏼‍♂' => 'eliyle yüzünü kapatan erkek: orta açık cilt tonu',
    '🤦🏽‍♂' => 'eliyle yüzünü kapatan erkek: orta cilt tonu',
    '🤦🏾‍♂' => 'eliyle yüzünü kapatan erkek: orta koyu cilt tonu',
    '🤦🏿‍♂' => 'eliyle yüzünü kapatan erkek: koyu cilt tonu',
    '🤦🏻‍♀' => 'eliyle yüzünü kapatan kadın: açık cilt tonu',
    '🤦🏼‍♀' => 'eliyle yüzünü kapatan kadın: orta açık cilt tonu',
    '🤦🏽‍♀' => 'eliyle yüzünü kapatan kadın: orta cilt tonu',
    '🤦🏾‍♀' => 'eliyle yüzünü kapatan kadın: orta koyu cilt tonu',
    '🤦🏿‍♀' => 'eliyle yüzünü kapatan kadın: koyu cilt tonu',
    '🤷🏻‍♂' => 'omuz silken erkek: açık cilt tonu',
    '🤷🏼‍♂' => 'omuz silken erkek: orta açık cilt tonu',
    '🤷🏽‍♂' => 'omuz silken erkek: orta cilt tonu',
    '🤷🏾‍♂' => 'omuz silken erkek: orta koyu cilt tonu',
    '🤷🏿‍♂' => 'omuz silken erkek: koyu cilt tonu',
    '🤷🏻‍♀' => 'omuz silken kadın: açık cilt tonu',
    '🤷🏼‍♀' => 'omuz silken kadın: orta açık cilt tonu',
    '🤷🏽‍♀' => 'omuz silken kadın: orta cilt tonu',
    '🤷🏾‍♀' => 'omuz silken kadın: orta koyu cilt tonu',
    '🤷🏿‍♀' => 'omuz silken kadın: koyu cilt tonu',
    '🧑🏻‍⚕' => 'sağlık çalışanı: açık cilt tonu',
    '🧑🏼‍⚕' => 'sağlık çalışanı: orta açık cilt tonu',
    '🧑🏽‍⚕' => 'sağlık çalışanı: orta cilt tonu',
    '🧑🏾‍⚕' => 'sağlık çalışanı: orta koyu cilt tonu',
    '🧑🏿‍⚕' => 'sağlık çalışanı: koyu cilt tonu',
    '👨🏻‍⚕' => 'erkek sağlık çalışanı: açık cilt tonu',
    '👨🏼‍⚕' => 'erkek sağlık çalışanı: orta açık cilt tonu',
    '👨🏽‍⚕' => 'erkek sağlık çalışanı: orta cilt tonu',
    '👨🏾‍⚕' => 'erkek sağlık çalışanı: orta koyu cilt tonu',
    '👨🏿‍⚕' => 'erkek sağlık çalışanı: koyu cilt tonu',
    '👩🏻‍⚕' => 'kadın sağlık çalışanı: açık cilt tonu',
    '👩🏼‍⚕' => 'kadın sağlık çalışanı: orta açık cilt tonu',
    '👩🏽‍⚕' => 'kadın sağlık çalışanı: orta cilt tonu',
    '👩🏾‍⚕' => 'kadın sağlık çalışanı: orta koyu cilt tonu',
    '👩🏿‍⚕' => 'kadın sağlık çalışanı: koyu cilt tonu',
    '🧑🏻‍🎓' => 'öğrenci: açık cilt tonu',
    '🧑🏼‍🎓' => 'öğrenci: orta açık cilt tonu',
    '🧑🏽‍🎓' => 'öğrenci: orta cilt tonu',
    '🧑🏾‍🎓' => 'öğrenci: orta koyu cilt tonu',
    '🧑🏿‍🎓' => 'öğrenci: koyu cilt tonu',
    '👨🏻‍🎓' => 'erkek öğrenci: açık cilt tonu',
    '👨🏼‍🎓' => 'erkek öğrenci: orta açık cilt tonu',
    '👨🏽‍🎓' => 'erkek öğrenci: orta cilt tonu',
    '👨🏾‍🎓' => 'erkek öğrenci: orta koyu cilt tonu',
    '👨🏿‍🎓' => 'erkek öğrenci: koyu cilt tonu',
    '👩🏻‍🎓' => 'kız öğrenci: açık cilt tonu',
    '👩🏼‍🎓' => 'kız öğrenci: orta açık cilt tonu',
    '👩🏽‍🎓' => 'kız öğrenci: orta cilt tonu',
    '👩🏾‍🎓' => 'kız öğrenci: orta koyu cilt tonu',
    '👩🏿‍🎓' => 'kız öğrenci: koyu cilt tonu',
    '🧑🏻‍🏫' => 'öğretmen: açık cilt tonu',
    '🧑🏼‍🏫' => 'öğretmen: orta açık cilt tonu',
    '🧑🏽‍🏫' => 'öğretmen: orta cilt tonu',
    '🧑🏾‍🏫' => 'öğretmen: orta koyu cilt tonu',
    '🧑🏿‍🏫' => 'öğretmen: koyu cilt tonu',
    '👨🏻‍🏫' => 'erkek öğretmen: açık cilt tonu',
    '👨🏼‍🏫' => 'erkek öğretmen: orta açık cilt tonu',
    '👨🏽‍🏫' => 'erkek öğretmen: orta cilt tonu',
    '👨🏾‍🏫' => 'erkek öğretmen: orta koyu cilt tonu',
    '👨🏿‍🏫' => 'erkek öğretmen: koyu cilt tonu',
    '👩🏻‍🏫' => 'kadın öğretmen: açık cilt tonu',
    '👩🏼‍🏫' => 'kadın öğretmen: orta açık cilt tonu',
    '👩🏽‍🏫' => 'kadın öğretmen: orta cilt tonu',
    '👩🏾‍🏫' => 'kadın öğretmen: orta koyu cilt tonu',
    '👩🏿‍🏫' => 'kadın öğretmen: koyu cilt tonu',
    '🧑🏻‍⚖' => 'hakim: açık cilt tonu',
    '🧑🏼‍⚖' => 'hakim: orta açık cilt tonu',
    '🧑🏽‍⚖' => 'hakim: orta cilt tonu',
    '🧑🏾‍⚖' => 'hakim: orta koyu cilt tonu',
    '🧑🏿‍⚖' => 'hakim: koyu cilt tonu',
    '👨🏻‍⚖' => 'erkek hakim: açık cilt tonu',
    '👨🏼‍⚖' => 'erkek hakim: orta açık cilt tonu',
    '👨🏽‍⚖' => 'erkek hakim: orta cilt tonu',
    '👨🏾‍⚖' => 'erkek hakim: orta koyu cilt tonu',
    '👨🏿‍⚖' => 'erkek hakim: koyu cilt tonu',
    '👩🏻‍⚖' => 'kadın hakim: açık cilt tonu',
    '👩🏼‍⚖' => 'kadın hakim: orta açık cilt tonu',
    '👩🏽‍⚖' => 'kadın hakim: orta cilt tonu',
    '👩🏾‍⚖' => 'kadın hakim: orta koyu cilt tonu',
    '👩🏿‍⚖' => 'kadın hakim: koyu cilt tonu',
    '🧑🏻‍🌾' => 'çiftçi: açık cilt tonu',
    '🧑🏼‍🌾' => 'çiftçi: orta açık cilt tonu',
    '🧑🏽‍🌾' => 'çiftçi: orta cilt tonu',
    '🧑🏾‍🌾' => 'çiftçi: orta koyu cilt tonu',
    '🧑🏿‍🌾' => 'çiftçi: koyu cilt tonu',
    '👨🏻‍🌾' => 'erkek çiftçi: açık cilt tonu',
    '👨🏼‍🌾' => 'erkek çiftçi: orta açık cilt tonu',
    '👨🏽‍🌾' => 'erkek çiftçi: orta cilt tonu',
    '👨🏾‍🌾' => 'erkek çiftçi: orta koyu cilt tonu',
    '👨🏿‍🌾' => 'erkek çiftçi: koyu cilt tonu',
    '👩🏻‍🌾' => 'kadın çiftçi: açık cilt tonu',
    '👩🏼‍🌾' => 'kadın çiftçi: orta açık cilt tonu',
    '👩🏽‍🌾' => 'kadın çiftçi: orta cilt tonu',
    '👩🏾‍🌾' => 'kadın çiftçi: orta koyu cilt tonu',
    '👩🏿‍🌾' => 'kadın çiftçi: koyu cilt tonu',
    '🧑🏻‍🍳' => 'aşçı: açık cilt tonu',
    '🧑🏼‍🍳' => 'aşçı: orta açık cilt tonu',
    '🧑🏽‍🍳' => 'aşçı: orta cilt tonu',
    '🧑🏾‍🍳' => 'aşçı: orta koyu cilt tonu',
    '🧑🏿‍🍳' => 'aşçı: koyu cilt tonu',
    '👨🏻‍🍳' => 'erkek aşçı: açık cilt tonu',
    '👨🏼‍🍳' => 'erkek aşçı: orta açık cilt tonu',
    '👨🏽‍🍳' => 'erkek aşçı: orta cilt tonu',
    '👨🏾‍🍳' => 'erkek aşçı: orta koyu cilt tonu',
    '👨🏿‍🍳' => 'erkek aşçı: koyu cilt tonu',
    '👩🏻‍🍳' => 'kadın aşçı: açık cilt tonu',
    '👩🏼‍🍳' => 'kadın aşçı: orta açık cilt tonu',
    '👩🏽‍🍳' => 'kadın aşçı: orta cilt tonu',
    '👩🏾‍🍳' => 'kadın aşçı: orta koyu cilt tonu',
    '👩🏿‍🍳' => 'kadın aşçı: koyu cilt tonu',
    '🧑🏻‍🔧' => 'tamirci: açık cilt tonu',
    '🧑🏼‍🔧' => 'tamirci: orta açık cilt tonu',
    '🧑🏽‍🔧' => 'tamirci: orta cilt tonu',
    '🧑🏾‍🔧' => 'tamirci: orta koyu cilt tonu',
    '🧑🏿‍🔧' => 'tamirci: koyu cilt tonu',
    '👨🏻‍🔧' => 'erkek tamirci: açık cilt tonu',
    '👨🏼‍🔧' => 'erkek tamirci: orta açık cilt tonu',
    '👨🏽‍🔧' => 'erkek tamirci: orta cilt tonu',
    '👨🏾‍🔧' => 'erkek tamirci: orta koyu cilt tonu',
    '👨🏿‍🔧' => 'erkek tamirci: koyu cilt tonu',
    '👩🏻‍🔧' => 'kadın tamirci: açık cilt tonu',
    '👩🏼‍🔧' => 'kadın tamirci: orta açık cilt tonu',
    '👩🏽‍🔧' => 'kadın tamirci: orta cilt tonu',
    '👩🏾‍🔧' => 'kadın tamirci: orta koyu cilt tonu',
    '👩🏿‍🔧' => 'kadın tamirci: koyu cilt tonu',
    '🧑🏻‍🏭' => 'fabrika işçisi: açık cilt tonu',
    '🧑🏼‍🏭' => 'fabrika işçisi: orta açık cilt tonu',
    '🧑🏽‍🏭' => 'fabrika işçisi: orta cilt tonu',
    '🧑🏾‍🏭' => 'fabrika işçisi: orta koyu cilt tonu',
    '🧑🏿‍🏭' => 'fabrika işçisi: koyu cilt tonu',
    '👨🏻‍🏭' => 'erkek fabrika işçisi: açık cilt tonu',
    '👨🏼‍🏭' => 'erkek fabrika işçisi: orta açık cilt tonu',
    '👨🏽‍🏭' => 'erkek fabrika işçisi: orta cilt tonu',
    '👨🏾‍🏭' => 'erkek fabrika işçisi: orta koyu cilt tonu',
    '👨🏿‍🏭' => 'erkek fabrika işçisi: koyu cilt tonu',
    '👩🏻‍🏭' => 'kadın fabrika işçisi: açık cilt tonu',
    '👩🏼‍🏭' => 'kadın fabrika işçisi: orta açık cilt tonu',
    '👩🏽‍🏭' => 'kadın fabrika işçisi: orta cilt tonu',
    '👩🏾‍🏭' => 'kadın fabrika işçisi: orta koyu cilt tonu',
    '👩🏿‍🏭' => 'kadın fabrika işçisi: koyu cilt tonu',
    '🧑🏻‍💼' => 'ofis çalışanı: açık cilt tonu',
    '🧑🏼‍💼' => 'ofis çalışanı: orta açık cilt tonu',
    '🧑🏽‍💼' => 'ofis çalışanı: orta cilt tonu',
    '🧑🏾‍💼' => 'ofis çalışanı: orta koyu cilt tonu',
    '🧑🏿‍💼' => 'ofis çalışanı: koyu cilt tonu',
    '👨🏻‍💼' => 'erkek ofis çalışanı: açık cilt tonu',
    '👨🏼‍💼' => 'erkek ofis çalışanı: orta açık cilt tonu',
    '👨🏽‍💼' => 'erkek ofis çalışanı: orta cilt tonu',
    '👨🏾‍💼' => 'erkek ofis çalışanı: orta koyu cilt tonu',
    '👨🏿‍💼' => 'erkek ofis çalışanı: koyu cilt tonu',
    '👩🏻‍💼' => 'kadın ofis çalışanı: açık cilt tonu',
    '👩🏼‍💼' => 'kadın ofis çalışanı: orta açık cilt tonu',
    '👩🏽‍💼' => 'kadın ofis çalışanı: orta cilt tonu',
    '👩🏾‍💼' => 'kadın ofis çalışanı: orta koyu cilt tonu',
    '👩🏿‍💼' => 'kadın ofis çalışanı: koyu cilt tonu',
    '🧑🏻‍🔬' => 'bilim insanı: açık cilt tonu',
    '🧑🏼‍🔬' => 'bilim insanı: orta açık cilt tonu',
    '🧑🏽‍🔬' => 'bilim insanı: orta cilt tonu',
    '🧑🏾‍🔬' => 'bilim insanı: orta koyu cilt tonu',
    '🧑🏿‍🔬' => 'bilim insanı: koyu cilt tonu',
    '👨🏻‍🔬' => 'erkek bilim insanı: açık cilt tonu',
    '👨🏼‍🔬' => 'erkek bilim insanı: orta açık cilt tonu',
    '👨🏽‍🔬' => 'erkek bilim insanı: orta cilt tonu',
    '👨🏾‍🔬' => 'erkek bilim insanı: orta koyu cilt tonu',
    '👨🏿‍🔬' => 'erkek bilim insanı: koyu cilt tonu',
    '👩🏻‍🔬' => 'kadın bilim insanı: açık cilt tonu',
    '👩🏼‍🔬' => 'kadın bilim insanı: orta açık cilt tonu',
    '👩🏽‍🔬' => 'kadın bilim insanı: orta cilt tonu',
    '👩🏾‍🔬' => 'kadın bilim insanı: orta koyu cilt tonu',
    '👩🏿‍🔬' => 'kadın bilim insanı: koyu cilt tonu',
    '🧑🏻‍💻' => 'teknoloji uzmanı: açık cilt tonu',
    '🧑🏼‍💻' => 'teknoloji uzmanı: orta açık cilt tonu',
    '🧑🏽‍💻' => 'teknoloji uzmanı: orta cilt tonu',
    '🧑🏾‍💻' => 'teknoloji uzmanı: orta koyu cilt tonu',
    '🧑🏿‍💻' => 'teknoloji uzmanı: koyu cilt tonu',
    '👨🏻‍💻' => 'erkek teknoloji uzmanı: açık cilt tonu',
    '👨🏼‍💻' => 'erkek teknoloji uzmanı: orta açık cilt tonu',
    '👨🏽‍💻' => 'erkek teknoloji uzmanı: orta cilt tonu',
    '👨🏾‍💻' => 'erkek teknoloji uzmanı: orta koyu cilt tonu',
    '👨🏿‍💻' => 'erkek teknoloji uzmanı: koyu cilt tonu',
    '👩🏻‍💻' => 'kadın teknoloji uzmanı: açık cilt tonu',
    '👩🏼‍💻' => 'kadın teknoloji uzmanı: orta açık cilt tonu',
    '👩🏽‍💻' => 'kadın teknoloji uzmanı: orta cilt tonu',
    '👩🏾‍💻' => 'kadın teknoloji uzmanı: orta koyu cilt tonu',
    '👩🏿‍💻' => 'kadın teknoloji uzmanı: koyu cilt tonu',
    '🧑🏻‍🎤' => 'şarkıcı: açık cilt tonu',
    '🧑🏼‍🎤' => 'şarkıcı: orta açık cilt tonu',
    '🧑🏽‍🎤' => 'şarkıcı: orta cilt tonu',
    '🧑🏾‍🎤' => 'şarkıcı: orta koyu cilt tonu',
    '🧑🏿‍🎤' => 'şarkıcı: koyu cilt tonu',
    '👨🏻‍🎤' => 'erkek şarkıcı: açık cilt tonu',
    '👨🏼‍🎤' => 'erkek şarkıcı: orta açık cilt tonu',
    '👨🏽‍🎤' => 'erkek şarkıcı: orta cilt tonu',
    '👨🏾‍🎤' => 'erkek şarkıcı: orta koyu cilt tonu',
    '👨🏿‍🎤' => 'erkek şarkıcı: koyu cilt tonu',
    '👩🏻‍🎤' => 'kadın şarkıcı: açık cilt tonu',
    '👩🏼‍🎤' => 'kadın şarkıcı: orta açık cilt tonu',
    '👩🏽‍🎤' => 'kadın şarkıcı: orta cilt tonu',
    '👩🏾‍🎤' => 'kadın şarkıcı: orta koyu cilt tonu',
    '👩🏿‍🎤' => 'kadın şarkıcı: koyu cilt tonu',
    '🧑🏻‍🎨' => 'ressam: açık cilt tonu',
    '🧑🏼‍🎨' => 'ressam: orta açık cilt tonu',
    '🧑🏽‍🎨' => 'ressam: orta cilt tonu',
    '🧑🏾‍🎨' => 'ressam: orta koyu cilt tonu',
    '🧑🏿‍🎨' => 'ressam: koyu cilt tonu',
    '👨🏻‍🎨' => 'erkek ressam: açık cilt tonu',
    '👨🏼‍🎨' => 'erkek ressam: orta açık cilt tonu',
    '👨🏽‍🎨' => 'erkek ressam: orta cilt tonu',
    '👨🏾‍🎨' => 'erkek ressam: orta koyu cilt tonu',
    '👨🏿‍🎨' => 'erkek ressam: koyu cilt tonu',
    '👩🏻‍🎨' => 'kadın ressam: açık cilt tonu',
    '👩🏼‍🎨' => 'kadın ressam: orta açık cilt tonu',
    '👩🏽‍🎨' => 'kadın ressam: orta cilt tonu',
    '👩🏾‍🎨' => 'kadın ressam: orta koyu cilt tonu',
    '👩🏿‍🎨' => 'kadın ressam: koyu cilt tonu',
    '🧑🏻‍✈' => 'pilot: açık cilt tonu',
    '🧑🏼‍✈' => 'pilot: orta açık cilt tonu',
    '🧑🏽‍✈' => 'pilot: orta cilt tonu',
    '🧑🏾‍✈' => 'pilot: orta koyu cilt tonu',
    '🧑🏿‍✈' => 'pilot: koyu cilt tonu',
    '👨🏻‍✈' => 'erkek pilot: açık cilt tonu',
    '👨🏼‍✈' => 'erkek pilot: orta açık cilt tonu',
    '👨🏽‍✈' => 'erkek pilot: orta cilt tonu',
    '👨🏾‍✈' => 'erkek pilot: orta koyu cilt tonu',
    '👨🏿‍✈' => 'erkek pilot: koyu cilt tonu',
    '👩🏻‍✈' => 'kadın pilot: açık cilt tonu',
    '👩🏼‍✈' => 'kadın pilot: orta açık cilt tonu',
    '👩🏽‍✈' => 'kadın pilot: orta cilt tonu',
    '👩🏾‍✈' => 'kadın pilot: orta koyu cilt tonu',
    '👩🏿‍✈' => 'kadın pilot: koyu cilt tonu',
    '🧑🏻‍🚀' => 'astronot: açık cilt tonu',
    '🧑🏼‍🚀' => 'astronot: orta açık cilt tonu',
    '🧑🏽‍🚀' => 'astronot: orta cilt tonu',
    '🧑🏾‍🚀' => 'astronot: orta koyu cilt tonu',
    '🧑🏿‍🚀' => 'astronot: koyu cilt tonu',
    '👨🏻‍🚀' => 'erkek astronot: açık cilt tonu',
    '👨🏼‍🚀' => 'erkek astronot: orta açık cilt tonu',
    '👨🏽‍🚀' => 'erkek astronot: orta cilt tonu',
    '👨🏾‍🚀' => 'erkek astronot: orta koyu cilt tonu',
    '👨🏿‍🚀' => 'erkek astronot: koyu cilt tonu',
    '👩🏻‍🚀' => 'kadın astronot: açık cilt tonu',
    '👩🏼‍🚀' => 'kadın astronot: orta açık cilt tonu',
    '👩🏽‍🚀' => 'kadın astronot: orta cilt tonu',
    '👩🏾‍🚀' => 'kadın astronot: orta koyu cilt tonu',
    '👩🏿‍🚀' => 'kadın astronot: koyu cilt tonu',
    '🧑🏻‍🚒' => 'itfaiyeci: açık cilt tonu',
    '🧑🏼‍🚒' => 'itfaiyeci: orta açık cilt tonu',
    '🧑🏽‍🚒' => 'itfaiyeci: orta cilt tonu',
    '🧑🏾‍🚒' => 'itfaiyeci: orta koyu cilt tonu',
    '🧑🏿‍🚒' => 'itfaiyeci: koyu cilt tonu',
    '👨🏻‍🚒' => 'erkek itfaiyeci: açık cilt tonu',
    '👨🏼‍🚒' => 'erkek itfaiyeci: orta açık cilt tonu',
    '👨🏽‍🚒' => 'erkek itfaiyeci: orta cilt tonu',
    '👨🏾‍🚒' => 'erkek itfaiyeci: orta koyu cilt tonu',
    '👨🏿‍🚒' => 'erkek itfaiyeci: koyu cilt tonu',
    '👩🏻‍🚒' => 'kadın itfaiyeci: açık cilt tonu',
    '👩🏼‍🚒' => 'kadın itfaiyeci: orta açık cilt tonu',
    '👩🏽‍🚒' => 'kadın itfaiyeci: orta cilt tonu',
    '👩🏾‍🚒' => 'kadın itfaiyeci: orta koyu cilt tonu',
    '👩🏿‍🚒' => 'kadın itfaiyeci: koyu cilt tonu',
    '👮🏻‍♂' => 'erkek polis memuru: açık cilt tonu',
    '👮🏼‍♂' => 'erkek polis memuru: orta açık cilt tonu',
    '👮🏽‍♂' => 'erkek polis memuru: orta cilt tonu',
    '👮🏾‍♂' => 'erkek polis memuru: orta koyu cilt tonu',
    '👮🏿‍♂' => 'erkek polis memuru: koyu cilt tonu',
    '👮🏻‍♀' => 'kadın polis memuru: açık cilt tonu',
    '👮🏼‍♀' => 'kadın polis memuru: orta açık cilt tonu',
    '👮🏽‍♀' => 'kadın polis memuru: orta cilt tonu',
    '👮🏾‍♀' => 'kadın polis memuru: orta koyu cilt tonu',
    '👮🏿‍♀' => 'kadın polis memuru: koyu cilt tonu',
    '🕵🏻‍♂' => 'erkek dedektif: açık cilt tonu',
    '🕵🏼‍♂' => 'erkek dedektif: orta açık cilt tonu',
    '🕵🏽‍♂' => 'erkek dedektif: orta cilt tonu',
    '🕵🏾‍♂' => 'erkek dedektif: orta koyu cilt tonu',
    '🕵🏿‍♂' => 'erkek dedektif: koyu cilt tonu',
    '🕵🏻‍♀' => 'kadın dedektif: açık cilt tonu',
    '🕵🏼‍♀' => 'kadın dedektif: orta açık cilt tonu',
    '🕵🏽‍♀' => 'kadın dedektif: orta cilt tonu',
    '🕵🏾‍♀' => 'kadın dedektif: orta koyu cilt tonu',
    '🕵🏿‍♀' => 'kadın dedektif: koyu cilt tonu',
    '💂🏻‍♂' => 'erkek muhafız: açık cilt tonu',
    '💂🏼‍♂' => 'erkek muhafız: orta açık cilt tonu',
    '💂🏽‍♂' => 'erkek muhafız: orta cilt tonu',
    '💂🏾‍♂' => 'erkek muhafız: orta koyu cilt tonu',
    '💂🏿‍♂' => 'erkek muhafız: koyu cilt tonu',
    '💂🏻‍♀' => 'kadın muhafız: açık cilt tonu',
    '💂🏼‍♀' => 'kadın muhafız: orta açık cilt tonu',
    '💂🏽‍♀' => 'kadın muhafız: orta cilt tonu',
    '💂🏾‍♀' => 'kadın muhafız: orta koyu cilt tonu',
    '💂🏿‍♀' => 'kadın muhafız: koyu cilt tonu',
    '👷🏻‍♂' => 'erkek inşaat işçisi: açık cilt tonu',
    '👷🏼‍♂' => 'erkek inşaat işçisi: orta açık cilt tonu',
    '👷🏽‍♂' => 'erkek inşaat işçisi: orta cilt tonu',
    '👷🏾‍♂' => 'erkek inşaat işçisi: orta koyu cilt tonu',
    '👷🏿‍♂' => 'erkek inşaat işçisi: koyu cilt tonu',
    '👷🏻‍♀' => 'kadın inşaat işçisi: açık cilt tonu',
    '👷🏼‍♀' => 'kadın inşaat işçisi: orta açık cilt tonu',
    '👷🏽‍♀' => 'kadın inşaat işçisi: orta cilt tonu',
    '👷🏾‍♀' => 'kadın inşaat işçisi: orta koyu cilt tonu',
    '👷🏿‍♀' => 'kadın inşaat işçisi: koyu cilt tonu',
    '👳🏻‍♂' => 'sarıklı erkek: açık cilt tonu',
    '👳🏼‍♂' => 'sarıklı erkek: orta açık cilt tonu',
    '👳🏽‍♂' => 'sarıklı erkek: orta cilt tonu',
    '👳🏾‍♂' => 'sarıklı erkek: orta koyu cilt tonu',
    '👳🏿‍♂' => 'sarıklı erkek: koyu cilt tonu',
    '👳🏻‍♀' => 'sarıklı kadın: açık cilt tonu',
    '👳🏼‍♀' => 'sarıklı kadın: orta açık cilt tonu',
    '👳🏽‍♀' => 'sarıklı kadın: orta cilt tonu',
    '👳🏾‍♀' => 'sarıklı kadın: orta koyu cilt tonu',
    '👳🏿‍♀' => 'sarıklı kadın: koyu cilt tonu',
    '🤵🏻‍♂' => 'smokinli erkek: açık cilt tonu',
    '🤵🏼‍♂' => 'smokinli erkek: orta açık cilt tonu',
    '🤵🏽‍♂' => 'smokinli erkek: orta cilt tonu',
    '🤵🏾‍♂' => 'smokinli erkek: orta koyu cilt tonu',
    '🤵🏿‍♂' => 'smokinli erkek: koyu cilt tonu',
    '🤵🏻‍♀' => 'smokinli kadın: açık cilt tonu',
    '🤵🏼‍♀' => 'smokinli kadın: orta açık cilt tonu',
    '🤵🏽‍♀' => 'smokinli kadın: orta cilt tonu',
    '🤵🏾‍♀' => 'smokinli kadın: orta koyu cilt tonu',
    '🤵🏿‍♀' => 'smokinli kadın: koyu cilt tonu',
    '👰🏻‍♂' => 'duvaklı erkek: açık cilt tonu',
    '👰🏼‍♂' => 'duvaklı erkek: orta açık cilt tonu',
    '👰🏽‍♂' => 'duvaklı erkek: orta cilt tonu',
    '👰🏾‍♂' => 'duvaklı erkek: orta koyu cilt tonu',
    '👰🏿‍♂' => 'duvaklı erkek: koyu cilt tonu',
    '👰🏻‍♀' => 'duvaklı kadın: açık cilt tonu',
    '👰🏼‍♀' => 'duvaklı kadın: orta açık cilt tonu',
    '👰🏽‍♀' => 'duvaklı kadın: orta cilt tonu',
    '👰🏾‍♀' => 'duvaklı kadın: orta koyu cilt tonu',
    '👰🏿‍♀' => 'duvaklı kadın: koyu cilt tonu',
    '👩🏻‍🍼' => 'bebek emziren kadın: açık cilt tonu',
    '👩🏼‍🍼' => 'bebek emziren kadın: orta açık cilt tonu',
    '👩🏽‍🍼' => 'bebek emziren kadın: orta cilt tonu',
    '👩🏾‍🍼' => 'bebek emziren kadın: orta koyu cilt tonu',
    '👩🏿‍🍼' => 'bebek emziren kadın: koyu cilt tonu',
    '👨🏻‍🍼' => 'bebek emziren erkek: açık cilt tonu',
    '👨🏼‍🍼' => 'bebek emziren erkek: orta açık cilt tonu',
    '👨🏽‍🍼' => 'bebek emziren erkek: orta cilt tonu',
    '👨🏾‍🍼' => 'bebek emziren erkek: orta koyu cilt tonu',
    '👨🏿‍🍼' => 'bebek emziren erkek: koyu cilt tonu',
    '🧑🏻‍🍼' => 'bebek emziren kişi: açık cilt tonu',
    '🧑🏼‍🍼' => 'bebek emziren kişi: orta açık cilt tonu',
    '🧑🏽‍🍼' => 'bebek emziren kişi: orta cilt tonu',
    '🧑🏾‍🍼' => 'bebek emziren kişi: orta koyu cilt tonu',
    '🧑🏿‍🍼' => 'bebek emziren kişi: koyu cilt tonu',
    '🧑🏻‍🎄' => 'cinsiyet belirtmeyen noel karakteri: açık cilt tonu',
    '🧑🏼‍🎄' => 'cinsiyet belirtmeyen noel karakteri: orta açık cilt tonu',
    '🧑🏽‍🎄' => 'cinsiyet belirtmeyen noel karakteri: orta cilt tonu',
    '🧑🏾‍🎄' => 'cinsiyet belirtmeyen noel karakteri: orta koyu cilt tonu',
    '🧑🏿‍🎄' => 'cinsiyet belirtmeyen noel karakteri: koyu cilt tonu',
    '🦸🏻‍♂' => 'erkek süper kahraman: açık cilt tonu',
    '🦸🏼‍♂' => 'erkek süper kahraman: orta açık cilt tonu',
    '🦸🏽‍♂' => 'erkek süper kahraman: orta cilt tonu',
    '🦸🏾‍♂' => 'erkek süper kahraman: orta koyu cilt tonu',
    '🦸🏿‍♂' => 'erkek süper kahraman: koyu cilt tonu',
    '🦸🏻‍♀' => 'kadın süper kahraman: açık cilt tonu',
    '🦸🏼‍♀' => 'kadın süper kahraman: orta açık cilt tonu',
    '🦸🏽‍♀' => 'kadın süper kahraman: orta cilt tonu',
    '🦸🏾‍♀' => 'kadın süper kahraman: orta koyu cilt tonu',
    '🦸🏿‍♀' => 'kadın süper kahraman: koyu cilt tonu',
    '🦹🏻‍♂' => 'erkek süper kötü: açık cilt tonu',
    '🦹🏼‍♂' => 'erkek süper kötü: orta açık cilt tonu',
    '🦹🏽‍♂' => 'erkek süper kötü: orta cilt tonu',
    '🦹🏾‍♂' => 'erkek süper kötü: orta koyu cilt tonu',
    '🦹🏿‍♂' => 'erkek süper kötü: koyu cilt tonu',
    '🦹🏻‍♀' => 'kadın süper kötü: açık cilt tonu',
    '🦹🏼‍♀' => 'kadın süper kötü: orta açık cilt tonu',
    '🦹🏽‍♀' => 'kadın süper kötü: orta cilt tonu',
    '🦹🏾‍♀' => 'kadın süper kötü: orta koyu cilt tonu',
    '🦹🏿‍♀' => 'kadın süper kötü: koyu cilt tonu',
    '🧙🏻‍♂' => 'erkek büyücü: açık cilt tonu',
    '🧙🏼‍♂' => 'erkek büyücü: orta açık cilt tonu',
    '🧙🏽‍♂' => 'erkek büyücü: orta cilt tonu',
    '🧙🏾‍♂' => 'erkek büyücü: orta koyu cilt tonu',
    '🧙🏿‍♂' => 'erkek büyücü: koyu cilt tonu',
    '🧙🏻‍♀' => 'kadın büyücü: açık cilt tonu',
    '🧙🏼‍♀' => 'kadın büyücü: orta açık cilt tonu',
    '🧙🏽‍♀' => 'kadın büyücü: orta cilt tonu',
    '🧙🏾‍♀' => 'kadın büyücü: orta koyu cilt tonu',
    '🧙🏿‍♀' => 'kadın büyücü: koyu cilt tonu',
    '🧚🏻‍♂' => 'erkek peri: açık cilt tonu',
    '🧚🏼‍♂' => 'erkek peri: orta açık cilt tonu',
    '🧚🏽‍♂' => 'erkek peri: orta cilt tonu',
    '🧚🏾‍♂' => 'erkek peri: orta koyu cilt tonu',
    '🧚🏿‍♂' => 'erkek peri: koyu cilt tonu',
    '🧚🏻‍♀' => 'kadın peri: açık cilt tonu',
    '🧚🏼‍♀' => 'kadın peri: orta açık cilt tonu',
    '🧚🏽‍♀' => 'kadın peri: orta cilt tonu',
    '🧚🏾‍♀' => 'kadın peri: orta koyu cilt tonu',
    '🧚🏿‍♀' => 'kadın peri: koyu cilt tonu',
    '🧛🏻‍♂' => 'erkek vampir: açık cilt tonu',
    '🧛🏼‍♂' => 'erkek vampir: orta açık cilt tonu',
    '🧛🏽‍♂' => 'erkek vampir: orta cilt tonu',
    '🧛🏾‍♂' => 'erkek vampir: orta koyu cilt tonu',
    '🧛🏿‍♂' => 'erkek vampir: koyu cilt tonu',
    '🧛🏻‍♀' => 'kadın vampir: açık cilt tonu',
    '🧛🏼‍♀' => 'kadın vampir: orta açık cilt tonu',
    '🧛🏽‍♀' => 'kadın vampir: orta cilt tonu',
    '🧛🏾‍♀' => 'kadın vampir: orta koyu cilt tonu',
    '🧛🏿‍♀' => 'kadın vampir: koyu cilt tonu',
    '🧜🏻‍♂' => 'balık adam: açık cilt tonu',
    '🧜🏼‍♂' => 'balık adam: orta açık cilt tonu',
    '🧜🏽‍♂' => 'balık adam: orta cilt tonu',
    '🧜🏾‍♂' => 'balık adam: orta koyu cilt tonu',
    '🧜🏿‍♂' => 'balık adam: koyu cilt tonu',
    '🧜🏻‍♀' => 'denizkızı: açık cilt tonu',
    '🧜🏼‍♀' => 'denizkızı: orta açık cilt tonu',
    '🧜🏽‍♀' => 'denizkızı: orta cilt tonu',
    '🧜🏾‍♀' => 'denizkızı: orta koyu cilt tonu',
    '🧜🏿‍♀' => 'denizkızı: koyu cilt tonu',
    '🧝🏻‍♂' => 'erkek elf: açık cilt tonu',
    '🧝🏼‍♂' => 'erkek elf: orta açık cilt tonu',
    '🧝🏽‍♂' => 'erkek elf: orta cilt tonu',
    '🧝🏾‍♂' => 'erkek elf: orta koyu cilt tonu',
    '🧝🏿‍♂' => 'erkek elf: koyu cilt tonu',
    '🧝🏻‍♀' => 'kadın elf: açık cilt tonu',
    '🧝🏼‍♀' => 'kadın elf: orta açık cilt tonu',
    '🧝🏽‍♀' => 'kadın elf: orta cilt tonu',
    '🧝🏾‍♀' => 'kadın elf: orta koyu cilt tonu',
    '🧝🏿‍♀' => 'kadın elf: koyu cilt tonu',
    '💆🏻‍♂' => 'yüz masajı yaptıran erkek: açık cilt tonu',
    '💆🏼‍♂' => 'yüz masajı yaptıran erkek: orta açık cilt tonu',
    '💆🏽‍♂' => 'yüz masajı yaptıran erkek: orta cilt tonu',
    '💆🏾‍♂' => 'yüz masajı yaptıran erkek: orta koyu cilt tonu',
    '💆🏿‍♂' => 'yüz masajı yaptıran erkek: koyu cilt tonu',
    '💆🏻‍♀' => 'yüz masajı yaptıran kadın: açık cilt tonu',
    '💆🏼‍♀' => 'yüz masajı yaptıran kadın: orta açık cilt tonu',
    '💆🏽‍♀' => 'yüz masajı yaptıran kadın: orta cilt tonu',
    '💆🏾‍♀' => 'yüz masajı yaptıran kadın: orta koyu cilt tonu',
    '💆🏿‍♀' => 'yüz masajı yaptıran kadın: koyu cilt tonu',
    '💇🏻‍♂' => 'saç kestiren erkek: açık cilt tonu',
    '💇🏼‍♂' => 'saç kestiren erkek: orta açık cilt tonu',
    '💇🏽‍♂' => 'saç kestiren erkek: orta cilt tonu',
    '💇🏾‍♂' => 'saç kestiren erkek: orta koyu cilt tonu',
    '💇🏿‍♂' => 'saç kestiren erkek: koyu cilt tonu',
    '💇🏻‍♀' => 'saç kestiren kadın: açık cilt tonu',
    '💇🏼‍♀' => 'saç kestiren kadın: orta açık cilt tonu',
    '💇🏽‍♀' => 'saç kestiren kadın: orta cilt tonu',
    '💇🏾‍♀' => 'saç kestiren kadın: orta koyu cilt tonu',
    '💇🏿‍♀' => 'saç kestiren kadın: koyu cilt tonu',
    '🚶🏻‍♂' => 'yürüyen erkek: açık cilt tonu',
    '🚶🏼‍♂' => 'yürüyen erkek: orta açık cilt tonu',
    '🚶🏽‍♂' => 'yürüyen erkek: orta cilt tonu',
    '🚶🏾‍♂' => 'yürüyen erkek: orta koyu cilt tonu',
    '🚶🏿‍♂' => 'yürüyen erkek: koyu cilt tonu',
    '🚶🏻‍♀' => 'yürüyen kadın: açık cilt tonu',
    '🚶🏼‍♀' => 'yürüyen kadın: orta açık cilt tonu',
    '🚶🏽‍♀' => 'yürüyen kadın: orta cilt tonu',
    '🚶🏾‍♀' => 'yürüyen kadın: orta koyu cilt tonu',
    '🚶🏿‍♀' => 'yürüyen kadın: koyu cilt tonu',
    '🚶🏻‍➡' => 'yürüyen kişi: açık cilt tonu sağa dönük',
    '🚶🏼‍➡' => 'yürüyen kişi: orta açık cilt tonu sağa dönük',
    '🚶🏽‍➡' => 'yürüyen kişi: orta cilt tonu sağa dönük',
    '🚶🏾‍➡' => 'yürüyen kişi: orta koyu cilt tonu sağa dönük',
    '🚶🏿‍➡' => 'yürüyen kişi: koyu cilt tonu sağa dönük',
    '🧍🏻‍♂' => 'ayakta duran erkek: açık cilt tonu',
    '🧍🏼‍♂' => 'ayakta duran erkek: orta açık cilt tonu',
    '🧍🏽‍♂' => 'ayakta duran erkek: orta cilt tonu',
    '🧍🏾‍♂' => 'ayakta duran erkek: orta koyu cilt tonu',
    '🧍🏿‍♂' => 'ayakta duran erkek: koyu cilt tonu',
    '🧍🏻‍♀' => 'ayakta duran kadın: açık cilt tonu',
    '🧍🏼‍♀' => 'ayakta duran kadın: orta açık cilt tonu',
    '🧍🏽‍♀' => 'ayakta duran kadın: orta cilt tonu',
    '🧍🏾‍♀' => 'ayakta duran kadın: orta koyu cilt tonu',
    '🧍🏿‍♀' => 'ayakta duran kadın: koyu cilt tonu',
    '🧎🏻‍♂' => 'diz çöken erkek: açık cilt tonu',
    '🧎🏼‍♂' => 'diz çöken erkek: orta açık cilt tonu',
    '🧎🏽‍♂' => 'diz çöken erkek: orta cilt tonu',
    '🧎🏾‍♂' => 'diz çöken erkek: orta koyu cilt tonu',
    '🧎🏿‍♂' => 'diz çöken erkek: koyu cilt tonu',
    '🧎🏻‍♀' => 'diz çöken kadın: açık cilt tonu',
    '🧎🏼‍♀' => 'diz çöken kadın: orta açık cilt tonu',
    '🧎🏽‍♀' => 'diz çöken kadın: orta cilt tonu',
    '🧎🏾‍♀' => 'diz çöken kadın: orta koyu cilt tonu',
    '🧎🏿‍♀' => 'diz çöken kadın: koyu cilt tonu',
    '🧎🏻‍➡' => 'diz çöken kişi: açık cilt tonu sağa dönük',
    '🧎🏼‍➡' => 'diz çöken kişi: orta açık cilt tonu sağa dönük',
    '🧎🏽‍➡' => 'diz çöken kişi: orta cilt tonu sağa dönük',
    '🧎🏾‍➡' => 'diz çöken kişi: orta koyu cilt tonu sağa dönük',
    '🧎🏿‍➡' => 'diz çöken kişi: koyu cilt tonu sağa dönük',
    '🧑🏻‍🦯' => 'görme engelli bastonlu kişi: açık cilt tonu',
    '🧑🏼‍🦯' => 'görme engelli bastonlu kişi: orta açık cilt tonu',
    '🧑🏽‍🦯' => 'görme engelli bastonlu kişi: orta cilt tonu',
    '🧑🏾‍🦯' => 'görme engelli bastonlu kişi: orta koyu cilt tonu',
    '🧑🏿‍🦯' => 'görme engelli bastonlu kişi: koyu cilt tonu',
    '👨🏻‍🦯' => 'görme engelli bastonlu erkek: açık cilt tonu',
    '👨🏼‍🦯' => 'görme engelli bastonlu erkek: orta açık cilt tonu',
    '👨🏽‍🦯' => 'görme engelli bastonlu erkek: orta cilt tonu',
    '👨🏾‍🦯' => 'görme engelli bastonlu erkek: orta koyu cilt tonu',
    '👨🏿‍🦯' => 'görme engelli bastonlu erkek: koyu cilt tonu',
    '👩🏻‍🦯' => 'görme engelli bastonlu kadın: açık cilt tonu',
    '👩🏼‍🦯' => 'görme engelli bastonlu kadın: orta açık cilt tonu',
    '👩🏽‍🦯' => 'görme engelli bastonlu kadın: orta cilt tonu',
    '👩🏾‍🦯' => 'görme engelli bastonlu kadın: orta koyu cilt tonu',
    '👩🏿‍🦯' => 'görme engelli bastonlu kadın: koyu cilt tonu',
    '🧑🏻‍🦼' => 'elektrikli tekerlekli sandalyede kişi: açık cilt tonu',
    '🧑🏼‍🦼' => 'elektrikli tekerlekli sandalyede kişi: orta açık cilt tonu',
    '🧑🏽‍🦼' => 'elektrikli tekerlekli sandalyede kişi: orta cilt tonu',
    '🧑🏾‍🦼' => 'elektrikli tekerlekli sandalyede kişi: orta koyu cilt tonu',
    '🧑🏿‍🦼' => 'elektrikli tekerlekli sandalyede kişi: koyu cilt tonu',
    '👨🏻‍🦼' => 'elektrikli tekerlekli sandalyede erkek: açık cilt tonu',
    '👨🏼‍🦼' => 'elektrikli tekerlekli sandalyede erkek: orta açık cilt tonu',
    '👨🏽‍🦼' => 'elektrikli tekerlekli sandalyede erkek: orta cilt tonu',
    '👨🏾‍🦼' => 'elektrikli tekerlekli sandalyede erkek: orta koyu cilt tonu',
    '👨🏿‍🦼' => 'elektrikli tekerlekli sandalyede erkek: koyu cilt tonu',
    '👩🏻‍🦼' => 'elektrikli tekerlekli sandalyede kadın: açık cilt tonu',
    '👩🏼‍🦼' => 'elektrikli tekerlekli sandalyede kadın: orta açık cilt tonu',
    '👩🏽‍🦼' => 'elektrikli tekerlekli sandalyede kadın: orta cilt tonu',
    '👩🏾‍🦼' => 'elektrikli tekerlekli sandalyede kadın: orta koyu cilt tonu',
    '👩🏿‍🦼' => 'elektrikli tekerlekli sandalyede kadın: koyu cilt tonu',
    '🧑🏻‍🦽' => 'tekerlekli sandalyede kişi: açık cilt tonu',
    '🧑🏼‍🦽' => 'tekerlekli sandalyede kişi: orta açık cilt tonu',
    '🧑🏽‍🦽' => 'tekerlekli sandalyede kişi: orta cilt tonu',
    '🧑🏾‍🦽' => 'tekerlekli sandalyede kişi: orta koyu cilt tonu',
    '🧑🏿‍🦽' => 'tekerlekli sandalyede kişi: koyu cilt tonu',
    '👨🏻‍🦽' => 'tekerlekli sandalyede erkek: açık cilt tonu',
    '👨🏼‍🦽' => 'tekerlekli sandalyede erkek: orta açık cilt tonu',
    '👨🏽‍🦽' => 'tekerlekli sandalyede erkek: orta cilt tonu',
    '👨🏾‍🦽' => 'tekerlekli sandalyede erkek: orta koyu cilt tonu',
    '👨🏿‍🦽' => 'tekerlekli sandalyede erkek: koyu cilt tonu',
    '👩🏻‍🦽' => 'tekerlekli sandalyede kadın: açık cilt tonu',
    '👩🏼‍🦽' => 'tekerlekli sandalyede kadın: orta açık cilt tonu',
    '👩🏽‍🦽' => 'tekerlekli sandalyede kadın: orta cilt tonu',
    '👩🏾‍🦽' => 'tekerlekli sandalyede kadın: orta koyu cilt tonu',
    '👩🏿‍🦽' => 'tekerlekli sandalyede kadın: koyu cilt tonu',
    '🏃🏻‍♂' => 'koşan erkek: açık cilt tonu',
    '🏃🏼‍♂' => 'koşan erkek: orta açık cilt tonu',
    '🏃🏽‍♂' => 'koşan erkek: orta cilt tonu',
    '🏃🏾‍♂' => 'koşan erkek: orta koyu cilt tonu',
    '🏃🏿‍♂' => 'koşan erkek: koyu cilt tonu',
    '🏃🏻‍♀' => 'koşan kadın: açık cilt tonu',
    '🏃🏼‍♀' => 'koşan kadın: orta açık cilt tonu',
    '🏃🏽‍♀' => 'koşan kadın: orta cilt tonu',
    '🏃🏾‍♀' => 'koşan kadın: orta koyu cilt tonu',
    '🏃🏿‍♀' => 'koşan kadın: koyu cilt tonu',
    '🏃🏻‍➡' => 'koşan kişi: açık cilt tonu sağa dönük',
    '🏃🏼‍➡' => 'koşan kişi: orta açık cilt tonu sağa dönük',
    '🏃🏽‍➡' => 'koşan kişi: orta cilt tonu sağa dönük',
    '🏃🏾‍➡' => 'koşan kişi: orta koyu cilt tonu sağa dönük',
    '🏃🏿‍➡' => 'koşan kişi: koyu cilt tonu sağa dönük',
    '👯🏻‍♀' => 'tavşan kulaklı kadınlar: açık cilt tonu',
    '👯🏼‍♀' => 'tavşan kulaklı kadınlar: orta açık cilt tonu',
    '👯🏽‍♀' => 'tavşan kulaklı kadınlar: orta cilt tonu',
    '👯🏾‍♀' => 'tavşan kulaklı kadınlar: orta koyu cilt tonu',
    '👯🏿‍♀' => 'tavşan kulaklı kadınlar: koyu cilt tonu',
    '👯🏻‍♂' => 'tavşan kulaklı erkekler: açık cilt tonu',
    '👯🏼‍♂' => 'tavşan kulaklı erkekler: orta açık cilt tonu',
    '👯🏽‍♂' => 'tavşan kulaklı erkekler: orta cilt tonu',
    '👯🏾‍♂' => 'tavşan kulaklı erkekler: orta koyu cilt tonu',
    '👯🏿‍♂' => 'tavşan kulaklı erkekler: koyu cilt tonu',
    '🧖🏻‍♂' => 'buhar odasındaki erkek: açık cilt tonu',
    '🧖🏼‍♂' => 'buhar odasındaki erkek: orta açık cilt tonu',
    '🧖🏽‍♂' => 'buhar odasındaki erkek: orta cilt tonu',
    '🧖🏾‍♂' => 'buhar odasındaki erkek: orta koyu cilt tonu',
    '🧖🏿‍♂' => 'buhar odasındaki erkek: koyu cilt tonu',
    '🧖🏻‍♀' => 'buhar odasındaki kadın: açık cilt tonu',
    '🧖🏼‍♀' => 'buhar odasındaki kadın: orta açık cilt tonu',
    '🧖🏽‍♀' => 'buhar odasındaki kadın: orta cilt tonu',
    '🧖🏾‍♀' => 'buhar odasındaki kadın: orta koyu cilt tonu',
    '🧖🏿‍♀' => 'buhar odasındaki kadın: koyu cilt tonu',
    '🧗🏻‍♂' => 'dağcı erkek: açık cilt tonu',
    '🧗🏼‍♂' => 'dağcı erkek: orta açık cilt tonu',
    '🧗🏽‍♂' => 'dağcı erkek: orta cilt tonu',
    '🧗🏾‍♂' => 'dağcı erkek: orta koyu cilt tonu',
    '🧗🏿‍♂' => 'dağcı erkek: koyu cilt tonu',
    '🧗🏻‍♀' => 'dağcı kadın: açık cilt tonu',
    '🧗🏼‍♀' => 'dağcı kadın: orta açık cilt tonu',
    '🧗🏽‍♀' => 'dağcı kadın: orta cilt tonu',
    '🧗🏾‍♀' => 'dağcı kadın: orta koyu cilt tonu',
    '🧗🏿‍♀' => 'dağcı kadın: koyu cilt tonu',
    '🏌🏻‍♂' => 'golf oynayan erkek: açık cilt tonu',
    '🏌🏼‍♂' => 'golf oynayan erkek: orta açık cilt tonu',
    '🏌🏽‍♂' => 'golf oynayan erkek: orta cilt tonu',
    '🏌🏾‍♂' => 'golf oynayan erkek: orta koyu cilt tonu',
    '🏌🏿‍♂' => 'golf oynayan erkek: koyu cilt tonu',
    '🏌🏻‍♀' => 'golf oynayan kadın: açık cilt tonu',
    '🏌🏼‍♀' => 'golf oynayan kadın: orta açık cilt tonu',
    '🏌🏽‍♀' => 'golf oynayan kadın: orta cilt tonu',
    '🏌🏾‍♀' => 'golf oynayan kadın: orta koyu cilt tonu',
    '🏌🏿‍♀' => 'golf oynayan kadın: koyu cilt tonu',
    '🏄🏻‍♂' => 'sörf yapan erkek: açık cilt tonu',
    '🏄🏼‍♂' => 'sörf yapan erkek: orta açık cilt tonu',
    '🏄🏽‍♂' => 'sörf yapan erkek: orta cilt tonu',
    '🏄🏾‍♂' => 'sörf yapan erkek: orta koyu cilt tonu',
    '🏄🏿‍♂' => 'sörf yapan erkek: koyu cilt tonu',
    '🏄🏻‍♀' => 'sörf yapan kadın: açık cilt tonu',
    '🏄🏼‍♀' => 'sörf yapan kadın: orta açık cilt tonu',
    '🏄🏽‍♀' => 'sörf yapan kadın: orta cilt tonu',
    '🏄🏾‍♀' => 'sörf yapan kadın: orta koyu cilt tonu',
    '🏄🏿‍♀' => 'sörf yapan kadın: koyu cilt tonu',
    '🚣🏻‍♂' => 'sandalda kürek çeken erkek: açık cilt tonu',
    '🚣🏼‍♂' => 'sandalda kürek çeken erkek: orta açık cilt tonu',
    '🚣🏽‍♂' => 'sandalda kürek çeken erkek: orta cilt tonu',
    '🚣🏾‍♂' => 'sandalda kürek çeken erkek: orta koyu cilt tonu',
    '🚣🏿‍♂' => 'sandalda kürek çeken erkek: koyu cilt tonu',
    '🚣🏻‍♀' => 'sandalda kürek çeken kadın: açık cilt tonu',
    '🚣🏼‍♀' => 'sandalda kürek çeken kadın: orta açık cilt tonu',
    '🚣🏽‍♀' => 'sandalda kürek çeken kadın: orta cilt tonu',
    '🚣🏾‍♀' => 'sandalda kürek çeken kadın: orta koyu cilt tonu',
    '🚣🏿‍♀' => 'sandalda kürek çeken kadın: koyu cilt tonu',
    '🏊🏻‍♂' => 'yüzen erkek: açık cilt tonu',
    '🏊🏼‍♂' => 'yüzen erkek: orta açık cilt tonu',
    '🏊🏽‍♂' => 'yüzen erkek: orta cilt tonu',
    '🏊🏾‍♂' => 'yüzen erkek: orta koyu cilt tonu',
    '🏊🏿‍♂' => 'yüzen erkek: koyu cilt tonu',
    '🏊🏻‍♀' => 'yüzen kadın: açık cilt tonu',
    '🏊🏼‍♀' => 'yüzen kadın: orta açık cilt tonu',
    '🏊🏽‍♀' => 'yüzen kadın: orta cilt tonu',
    '🏊🏾‍♀' => 'yüzen kadın: orta koyu cilt tonu',
    '🏊🏿‍♀' => 'yüzen kadın: koyu cilt tonu',
    '⛹🏻‍♂' => 'top sektiren erkek: açık cilt tonu',
    '⛹🏼‍♂' => 'top sektiren erkek: orta açık cilt tonu',
    '⛹🏽‍♂' => 'top sektiren erkek: orta cilt tonu',
    '⛹🏾‍♂' => 'top sektiren erkek: orta koyu cilt tonu',
    '⛹🏿‍♂' => 'top sektiren erkek: koyu cilt tonu',
    '⛹🏻‍♀' => 'top sektiren kadın: açık cilt tonu',
    '⛹🏼‍♀' => 'top sektiren kadın: orta açık cilt tonu',
    '⛹🏽‍♀' => 'top sektiren kadın: orta cilt tonu',
    '⛹🏾‍♀' => 'top sektiren kadın: orta koyu cilt tonu',
    '⛹🏿‍♀' => 'top sektiren kadın: koyu cilt tonu',
    '🏋🏻‍♂' => 'ağırlık kaldıran erkek: açık cilt tonu',
    '🏋🏼‍♂' => 'ağırlık kaldıran erkek: orta açık cilt tonu',
    '🏋🏽‍♂' => 'ağırlık kaldıran erkek: orta cilt tonu',
    '🏋🏾‍♂' => 'ağırlık kaldıran erkek: orta koyu cilt tonu',
    '🏋🏿‍♂' => 'ağırlık kaldıran erkek: koyu cilt tonu',
    '🏋🏻‍♀' => 'ağırlık kaldıran kadın: açık cilt tonu',
    '🏋🏼‍♀' => 'ağırlık kaldıran kadın: orta açık cilt tonu',
    '🏋🏽‍♀' => 'ağırlık kaldıran kadın: orta cilt tonu',
    '🏋🏾‍♀' => 'ağırlık kaldıran kadın: orta koyu cilt tonu',
    '🏋🏿‍♀' => 'ağırlık kaldıran kadın: koyu cilt tonu',
    '🚴🏻‍♂' => 'bisiklet süren erkek: açık cilt tonu',
    '🚴🏼‍♂' => 'bisiklet süren erkek: orta açık cilt tonu',
    '🚴🏽‍♂' => 'bisiklet süren erkek: orta cilt tonu',
    '🚴🏾‍♂' => 'bisiklet süren erkek: orta koyu cilt tonu',
    '🚴🏿‍♂' => 'bisiklet süren erkek: koyu cilt tonu',
    '🚴🏻‍♀' => 'bisiklet süren kadın: açık cilt tonu',
    '🚴🏼‍♀' => 'bisiklet süren kadın: orta açık cilt tonu',
    '🚴🏽‍♀' => 'bisiklet süren kadın: orta cilt tonu',
    '🚴🏾‍♀' => 'bisiklet süren kadın: orta koyu cilt tonu',
    '🚴🏿‍♀' => 'bisiklet süren kadın: koyu cilt tonu',
    '🚵🏻‍♂' => 'dağ bisikleti süren erkek: açık cilt tonu',
    '🚵🏼‍♂' => 'dağ bisikleti süren erkek: orta açık cilt tonu',
    '🚵🏽‍♂' => 'dağ bisikleti süren erkek: orta cilt tonu',
    '🚵🏾‍♂' => 'dağ bisikleti süren erkek: orta koyu cilt tonu',
    '🚵🏿‍♂' => 'dağ bisikleti süren erkek: koyu cilt tonu',
    '🚵🏻‍♀' => 'dağ bisikleti süren kadın: açık cilt tonu',
    '🚵🏼‍♀' => 'dağ bisikleti süren kadın: orta açık cilt tonu',
    '🚵🏽‍♀' => 'dağ bisikleti süren kadın: orta cilt tonu',
    '🚵🏾‍♀' => 'dağ bisikleti süren kadın: orta koyu cilt tonu',
    '🚵🏿‍♀' => 'dağ bisikleti süren kadın: koyu cilt tonu',
    '🤸🏻‍♂' => 'perende atan erkek: açık cilt tonu',
    '🤸🏼‍♂' => 'perende atan erkek: orta açık cilt tonu',
    '🤸🏽‍♂' => 'perende atan erkek: orta cilt tonu',
    '🤸🏾‍♂' => 'perende atan erkek: orta koyu cilt tonu',
    '🤸🏿‍♂' => 'perende atan erkek: koyu cilt tonu',
    '🤸🏻‍♀' => 'perende atan kadın: açık cilt tonu',
    '🤸🏼‍♀' => 'perende atan kadın: orta açık cilt tonu',
    '🤸🏽‍♀' => 'perende atan kadın: orta cilt tonu',
    '🤸🏾‍♀' => 'perende atan kadın: orta koyu cilt tonu',
    '🤸🏿‍♀' => 'perende atan kadın: koyu cilt tonu',
    '🤼🏻‍♀' => 'güreş yapan kadın: açık cilt tonu',
    '🤼🏼‍♀' => 'güreş yapan kadın: orta açık cilt tonu',
    '🤼🏽‍♀' => 'güreş yapan kadın: orta cilt tonu',
    '🤼🏾‍♀' => 'güreş yapan kadın: orta koyu cilt tonu',
    '🤼🏿‍♀' => 'güreş yapan kadın: koyu cilt tonu',
    '🤼🏻‍♂' => 'güreş yapan erkek: açık cilt tonu',
    '🤼🏼‍♂' => 'güreş yapan erkek: orta açık cilt tonu',
    '🤼🏽‍♂' => 'güreş yapan erkek: orta cilt tonu',
    '🤼🏾‍♂' => 'güreş yapan erkek: orta koyu cilt tonu',
    '🤼🏿‍♂' => 'güreş yapan erkek: koyu cilt tonu',
    '🤽🏻‍♂' => 'su topu oynayan erkek: açık cilt tonu',
    '🤽🏼‍♂' => 'su topu oynayan erkek: orta açık cilt tonu',
    '🤽🏽‍♂' => 'su topu oynayan erkek: orta cilt tonu',
    '🤽🏾‍♂' => 'su topu oynayan erkek: orta koyu cilt tonu',
    '🤽🏿‍♂' => 'su topu oynayan erkek: koyu cilt tonu',
    '🤽🏻‍♀' => 'su topu oynayan kadın: açık cilt tonu',
    '🤽🏼‍♀' => 'su topu oynayan kadın: orta açık cilt tonu',
    '🤽🏽‍♀' => 'su topu oynayan kadın: orta cilt tonu',
    '🤽🏾‍♀' => 'su topu oynayan kadın: orta koyu cilt tonu',
    '🤽🏿‍♀' => 'su topu oynayan kadın: koyu cilt tonu',
    '🤾🏻‍♂' => 'hentbol oynayan erkek: açık cilt tonu',
    '🤾🏼‍♂' => 'hentbol oynayan erkek: orta açık cilt tonu',
    '🤾🏽‍♂' => 'hentbol oynayan erkek: orta cilt tonu',
    '🤾🏾‍♂' => 'hentbol oynayan erkek: orta koyu cilt tonu',
    '🤾🏿‍♂' => 'hentbol oynayan erkek: koyu cilt tonu',
    '🤾🏻‍♀' => 'hentbol oynayan kadın: açık cilt tonu',
    '🤾🏼‍♀' => 'hentbol oynayan kadın: orta açık cilt tonu',
    '🤾🏽‍♀' => 'hentbol oynayan kadın: orta cilt tonu',
    '🤾🏾‍♀' => 'hentbol oynayan kadın: orta koyu cilt tonu',
    '🤾🏿‍♀' => 'hentbol oynayan kadın: koyu cilt tonu',
    '🤹🏻‍♂' => 'jonglör erkek: açık cilt tonu',
    '🤹🏼‍♂' => 'jonglör erkek: orta açık cilt tonu',
    '🤹🏽‍♂' => 'jonglör erkek: orta cilt tonu',
    '🤹🏾‍♂' => 'jonglör erkek: orta koyu cilt tonu',
    '🤹🏿‍♂' => 'jonglör erkek: koyu cilt tonu',
    '🤹🏻‍♀' => 'jonglör kadın: açık cilt tonu',
    '🤹🏼‍♀' => 'jonglör kadın: orta açık cilt tonu',
    '🤹🏽‍♀' => 'jonglör kadın: orta cilt tonu',
    '🤹🏾‍♀' => 'jonglör kadın: orta koyu cilt tonu',
    '🤹🏿‍♀' => 'jonglör kadın: koyu cilt tonu',
    '🧘🏻‍♂' => 'lotus pozisyonundaki erkek: açık cilt tonu',
    '🧘🏼‍♂' => 'lotus pozisyonundaki erkek: orta açık cilt tonu',
    '🧘🏽‍♂' => 'lotus pozisyonundaki erkek: orta cilt tonu',
    '🧘🏾‍♂' => 'lotus pozisyonundaki erkek: orta koyu cilt tonu',
    '🧘🏿‍♂' => 'lotus pozisyonundaki erkek: koyu cilt tonu',
    '🧘🏻‍♀' => 'lotus pozisyonundaki kadın: açık cilt tonu',
    '🧘🏼‍♀' => 'lotus pozisyonundaki kadın: orta açık cilt tonu',
    '🧘🏽‍♀' => 'lotus pozisyonundaki kadın: orta cilt tonu',
    '🧘🏾‍♀' => 'lotus pozisyonundaki kadın: orta koyu cilt tonu',
    '🧘🏿‍♀' => 'lotus pozisyonundaki kadın: koyu cilt tonu',
    '🧑‍🦰' => 'yetişkin: kızıl saç',
    '🧑‍🦱' => 'yetişkin: kıvırcık saç',
    '🧑‍🦳' => 'yetişkin: beyaz saç',
    '🧑‍🦲' => 'yetişkin: kel',
    '👨‍🦰' => 'erkek: kızıl saç',
    '👨‍🦱' => 'erkek: kıvırcık saç',
    '👨‍🦳' => 'erkek: beyaz saç',
    '👨‍🦲' => 'erkek: kel',
    '👩‍🦰' => 'kadın: kızıl saç',
    '👩‍🦱' => 'kadın: kıvırcık saç',
    '👩‍🦳' => 'kadın: beyaz saç',
    '👩‍🦲' => 'kadın: kel',
    '🚶‍➡' => 'yürüyen kişi: sağa dönük',
    '🧎‍➡' => 'diz çöken kişi: sağa dönük',
    '🏃‍➡' => 'koşan kişi: sağa dönük',
    '👨‍👦' => 'aile: erkek erkek çocuk',
    '👨‍👧' => 'aile: erkek kız çocuk',
    '👩‍👦' => 'aile: kadın erkek çocuk',
    '👩‍👧' => 'aile: kadın kız çocuk',
    '😶‍🌫' => 'bulutlardaki yüz',
    '😮‍💨' => 'nefes veren yüz',
    '🙂‍↕' => 'dikey olarak sallanan yüz',
    '😵‍💫' => 'sarmal gözlü yüz',
    '❤‍🔥' => 'ateşli kalp',
    '❤‍🩹' => 'iyileşen kalp',
    '👁‍🗨' => 'konuşma balonunda göz',
    '🧔‍♂' => 'erkek: sakal',
    '👱‍♂' => 'sarışın erkek',
    '🧔‍♀' => 'kadın: sakal',
    '👱‍♀' => 'sarışın kadın',
    '🙍‍♂' => 'somurtan erkek',
    '🙍‍♀' => 'somurtan kadın',
    '🙎‍♂' => 'darılan erkek',
    '🙎‍♀' => 'darılan kadın',
    '🙅‍♂' => 'hayır hareketi yapan erkek',
    '🙅‍♀' => 'hayır hareketi yapan kadın',
    '🙆‍♂' => 'tamam hareketi yapan erkek',
    '🙆‍♀' => 'tamam hareketi yapan kadın',
    '💁‍♂' => 'elini yana yatıran erkek',
    '💁‍♀' => 'elini yana yatıran kadın',
    '🙋‍♂' => 'elini kaldıran erkek',
    '🙋‍♀' => 'elini kaldıran kadın',
    '🧏‍♂' => 'işitme engelli erkek',
    '🧏‍♀' => 'işitme engelli kadın',
    '🙇‍♂' => 'başını öne eğen erkek',
    '🙇‍♀' => 'başını öne eğen kadın',
    '🤦‍♂' => 'eliyle yüzünü kapatan erkek',
    '🤦‍♀' => 'eliyle yüzünü kapatan kadın',
    '🤷‍♂' => 'omuz silken erkek',
    '🤷‍♀' => 'omuz silken kadın',
    '🧑‍⚕' => 'sağlık çalışanı',
    '👨‍⚕' => 'erkek sağlık çalışanı',
    '👩‍⚕' => 'kadın sağlık çalışanı',
    '🧑‍🎓' => 'öğrenci',
    '👨‍🎓' => 'erkek öğrenci',
    '👩‍🎓' => 'kız öğrenci',
    '🧑‍🏫' => 'öğretmen',
    '👨‍🏫' => 'erkek öğretmen',
    '👩‍🏫' => 'kadın öğretmen',
    '🧑‍⚖' => 'hakim',
    '👨‍⚖' => 'erkek hakim',
    '👩‍⚖' => 'kadın hakim',
    '🧑‍🌾' => 'çiftçi',
    '👨‍🌾' => 'erkek çiftçi',
    '👩‍🌾' => 'kadın çiftçi',
    '🧑‍🍳' => 'aşçı',
    '👨‍🍳' => 'erkek aşçı',
    '👩‍🍳' => 'kadın aşçı',
    '🧑‍🔧' => 'tamirci',
    '👨‍🔧' => 'erkek tamirci',
    '👩‍🔧' => 'kadın tamirci',
    '🧑‍🏭' => 'fabrika işçisi',
    '👨‍🏭' => 'erkek fabrika işçisi',
    '👩‍🏭' => 'kadın fabrika işçisi',
    '🧑‍💼' => 'ofis çalışanı',
    '👨‍💼' => 'erkek ofis çalışanı',
    '👩‍💼' => 'kadın ofis çalışanı',
    '🧑‍🔬' => 'bilim insanı',
    '👨‍🔬' => 'erkek bilim insanı',
    '👩‍🔬' => 'kadın bilim insanı',
    '🧑‍💻' => 'teknoloji uzmanı',
    '👨‍💻' => 'erkek teknoloji uzmanı',
    '👩‍💻' => 'kadın teknoloji uzmanı',
    '🧑‍🎤' => 'şarkıcı',
    '👨‍🎤' => 'erkek şarkıcı',
    '👩‍🎤' => 'kadın şarkıcı',
    '🧑‍🎨' => 'ressam',
    '👨‍🎨' => 'erkek ressam',
    '👩‍🎨' => 'kadın ressam',
    '🧑‍✈' => 'pilot',
    '👨‍✈' => 'erkek pilot',
    '👩‍✈' => 'kadın pilot',
    '🧑‍🚀' => 'astronot',
    '👨‍🚀' => 'erkek astronot',
    '👩‍🚀' => 'kadın astronot',
    '🧑‍🚒' => 'itfaiyeci',
    '👨‍🚒' => 'erkek itfaiyeci',
    '👩‍🚒' => 'kadın itfaiyeci',
    '👮‍♂' => 'erkek polis memuru',
    '👮‍♀' => 'kadın polis memuru',
    '🕵‍♂' => 'erkek dedektif',
    '🕵‍♀' => 'kadın dedektif',
    '💂‍♂' => 'erkek muhafız',
    '💂‍♀' => 'kadın muhafız',
    '👷‍♂' => 'erkek inşaat işçisi',
    '👷‍♀' => 'kadın inşaat işçisi',
    '👳‍♂' => 'sarıklı erkek',
    '👳‍♀' => 'sarıklı kadın',
    '🤵‍♂' => 'smokinli erkek',
    '🤵‍♀' => 'smokinli kadın',
    '👰‍♂' => 'duvaklı erkek',
    '👰‍♀' => 'duvaklı kadın',
    '👩‍🍼' => 'bebek emziren kadın',
    '👨‍🍼' => 'bebek emziren erkek',
    '🧑‍🍼' => 'bebek emziren kişi',
    '🧑‍🎄' => 'cinsiyet belirtmeyen noel karakteri',
    '🦸‍♂' => 'erkek süper kahraman',
    '🦸‍♀' => 'kadın süper kahraman',
    '🦹‍♂' => 'erkek süper kötü',
    '🦹‍♀' => 'kadın süper kötü',
    '🧙‍♂' => 'erkek büyücü',
    '🧙‍♀' => 'kadın büyücü',
    '🧚‍♂' => 'erkek peri',
    '🧚‍♀' => 'kadın peri',
    '🧛‍♂' => 'erkek vampir',
    '🧛‍♀' => 'kadın vampir',
    '🧜‍♂' => 'balık adam',
    '🧜‍♀' => 'denizkızı',
    '🧝‍♂' => 'erkek elf',
    '🧝‍♀' => 'kadın elf',
    '🧞‍♂' => 'erkek cin',
    '🧞‍♀' => 'kadın cin',
    '🧟‍♂' => 'erkek zombi',
    '🧟‍♀' => 'kadın zombi',
    '💆‍♂' => 'yüz masajı yaptıran erkek',
    '💆‍♀' => 'yüz masajı yaptıran kadın',
    '💇‍♂' => 'saç kestiren erkek',
    '💇‍♀' => 'saç kestiren kadın',
    '🚶‍♂' => 'yürüyen erkek',
    '🚶‍♀' => 'yürüyen kadın',
    '🧍‍♂' => 'ayakta duran erkek',
    '🧍‍♀' => 'ayakta duran kadın',
    '🧎‍♂' => 'diz çöken erkek',
    '🧎‍♀' => 'diz çöken kadın',
    '🧑‍🦯' => 'görme engelli bastonlu kişi',
    '👨‍🦯' => 'görme engelli bastonlu erkek',
    '👩‍🦯' => 'görme engelli bastonlu kadın',
    '🧑‍🦼' => 'elektrikli tekerlekli sandalyede kişi',
    '👨‍🦼' => 'elektrikli tekerlekli sandalyede erkek',
    '👩‍🦼' => 'elektrikli tekerlekli sandalyede kadın',
    '🧑‍🦽' => 'tekerlekli sandalyede kişi',
    '👨‍🦽' => 'tekerlekli sandalyede erkek',
    '👩‍🦽' => 'tekerlekli sandalyede kadın',
    '🏃‍♂' => 'koşan erkek',
    '🏃‍♀' => 'koşan kadın',
    '🧑‍🩰' => 'bale dansçısı',
    '👯‍♂' => 'tavşan kulaklı erkekler',
    '👯‍♀' => 'tavşan kulaklı kadınlar',
    '🧖‍♂' => 'buhar odasındaki erkek',
    '🧖‍♀' => 'buhar odasındaki kadın',
    '🧗‍♂' => 'dağcı erkek',
    '🧗‍♀' => 'dağcı kadın',
    '🏌‍♂' => 'golf oynayan erkek',
    '🏌‍♀' => 'golf oynayan kadın',
    '🏄‍♂' => 'sörf yapan erkek',
    '🏄‍♀' => 'sörf yapan kadın',
    '🚣‍♂' => 'sandalda kürek çeken erkek',
    '🚣‍♀' => 'sandalda kürek çeken kadın',
    '🏊‍♂' => 'yüzen erkek',
    '🏊‍♀' => 'yüzen kadın',
    '⛹‍♂' => 'top sektiren erkek',
    '⛹‍♀' => 'top sektiren kadın',
    '🏋‍♂' => 'ağırlık kaldıran erkek',
    '🏋‍♀' => 'ağırlık kaldıran kadın',
    '🚴‍♂' => 'bisiklet süren erkek',
    '🚴‍♀' => 'bisiklet süren kadın',
    '🚵‍♂' => 'dağ bisikleti süren erkek',
    '🚵‍♀' => 'dağ bisikleti süren kadın',
    '🤸‍♂' => 'perende atan erkek',
    '🤸‍♀' => 'perende atan kadın',
    '🤼‍♂' => 'güreş yapan erkek',
    '🤼‍♀' => 'güreş yapan kadın',
    '🤽‍♂' => 'su topu oynayan erkek',
    '🤽‍♀' => 'su topu oynayan kadın',
    '🤾‍♂' => 'hentbol oynayan erkek',
    '🤾‍♀' => 'hentbol oynayan kadın',
    '🤹‍♂' => 'jonglör erkek',
    '🤹‍♀' => 'jonglör kadın',
    '🧘‍♂' => 'lotus pozisyonundaki erkek',
    '🧘‍♀' => 'lotus pozisyonundaki kadın',
    '🧑‍🧒' => 'aile: yetişkin, çocuk',
    '🐕‍🦺' => 'yardımcı köpek',
    '🐈‍⬛' => 'kara kedi',
    '🐻‍❄' => 'kutup ayısı',
    '🐦‍⬛' => 'siyah kuş',
    '🐦‍🔥' => 'Anka',
    '🍋‍🟩' => 'misket limonu',
    '🍄‍🟫' => 'kahverengi mantar',
    '⛓‍💥' => 'zincirleri kırmak',
    '🏳‍🌈' => 'gökkuşağı bayrağı',
    '🏳‍⚧' => 'transgender bayrağı',
    '🏴‍☠' => 'korsan bayrağı',
    '👋🏻' => 'el sallama: açık cilt tonu',
    '👋🏼' => 'el sallama: orta açık cilt tonu',
    '👋🏽' => 'el sallama: orta cilt tonu',
    '👋🏾' => 'el sallama: orta koyu cilt tonu',
    '👋🏿' => 'el sallama: koyu cilt tonu',
    '🤚🏻' => 'elinin tersini kaldırma: açık cilt tonu',
    '🤚🏼' => 'elinin tersini kaldırma: orta açık cilt tonu',
    '🤚🏽' => 'elinin tersini kaldırma: orta cilt tonu',
    '🤚🏾' => 'elinin tersini kaldırma: orta koyu cilt tonu',
    '🤚🏿' => 'elinin tersini kaldırma: koyu cilt tonu',
    '🖐🏻' => 'parmaklar açık el kaldırma: açık cilt tonu',
    '🖐🏼' => 'parmaklar açık el kaldırma: orta açık cilt tonu',
    '🖐🏽' => 'parmaklar açık el kaldırma: orta cilt tonu',
    '🖐🏾' => 'parmaklar açık el kaldırma: orta koyu cilt tonu',
    '🖐🏿' => 'parmaklar açık el kaldırma: koyu cilt tonu',
    '✋🏻' => 'el kaldırma: açık cilt tonu',
    '✋🏼' => 'el kaldırma: orta açık cilt tonu',
    '✋🏽' => 'el kaldırma: orta cilt tonu',
    '✋🏾' => 'el kaldırma: orta koyu cilt tonu',
    '✋🏿' => 'el kaldırma: koyu cilt tonu',
    '🖖🏻' => 'Vulkan selamı: açık cilt tonu',
    '🖖🏼' => 'Vulkan selamı: orta açık cilt tonu',
    '🖖🏽' => 'Vulkan selamı: orta cilt tonu',
    '🖖🏾' => 'Vulkan selamı: orta koyu cilt tonu',
    '🖖🏿' => 'Vulkan selamı: koyu cilt tonu',
    '🫱🏻' => 'sağa bakan el: açık cilt tonu',
    '🫱🏼' => 'sağa bakan el: orta açık cilt tonu',
    '🫱🏽' => 'sağa bakan el: orta cilt tonu',
    '🫱🏾' => 'sağa bakan el: orta koyu cilt tonu',
    '🫱🏿' => 'sağa bakan el: koyu cilt tonu',
    '🫲🏻' => 'sola bakan el: açık cilt tonu',
    '🫲🏼' => 'sola bakan el: orta açık cilt tonu',
    '🫲🏽' => 'sola bakan el: orta cilt tonu',
    '🫲🏾' => 'sola bakan el: orta koyu cilt tonu',
    '🫲🏿' => 'sola bakan el: koyu cilt tonu',
    '🫳🏻' => 'avuç içi aşağı bakan el: açık cilt tonu',
    '🫳🏼' => 'avuç içi aşağı bakan el: orta açık cilt tonu',
    '🫳🏽' => 'avuç içi aşağı bakan el: orta cilt tonu',
    '🫳🏾' => 'avuç içi aşağı bakan el: orta koyu cilt tonu',
    '🫳🏿' => 'avuç içi aşağı bakan el: koyu cilt tonu',
    '🫴🏻' => 'avuç içi yukarı bakan el: açık cilt tonu',
    '🫴🏼' => 'avuç içi yukarı bakan el: orta açık cilt tonu',
    '🫴🏽' => 'avuç içi yukarı bakan el: orta cilt tonu',
    '🫴🏾' => 'avuç içi yukarı bakan el: orta koyu cilt tonu',
    '🫴🏿' => 'avuç içi yukarı bakan el: koyu cilt tonu',
    '🫷🏻' => 'sola iten el: açık cilt tonu',
    '🫷🏼' => 'sola iten el: orta açık cilt tonu',
    '🫷🏽' => 'sola iten el: orta cilt tonu',
    '🫷🏾' => 'sola iten el: orta koyu cilt tonu',
    '🫷🏿' => 'sola iten el: koyu cilt tonu',
    '🫸🏻' => 'sağa iten el: açık cilt tonu',
    '🫸🏼' => 'sağa iten el: orta açık cilt tonu',
    '🫸🏽' => 'sağa iten el: orta cilt tonu',
    '🫸🏾' => 'sağa iten el: orta koyu cilt tonu',
    '🫸🏿' => 'sağa iten el: koyu cilt tonu',
    '👌🏻' => 'tamam işareti: açık cilt tonu',
    '👌🏼' => 'tamam işareti: orta açık cilt tonu',
    '👌🏽' => 'tamam işareti: orta cilt tonu',
    '👌🏾' => 'tamam işareti: orta koyu cilt tonu',
    '👌🏿' => 'tamam işareti: koyu cilt tonu',
    '🤌🏻' => 'kıstırma hareketi yapan parmaklar: açık cilt tonu',
    '🤌🏼' => 'kıstırma hareketi yapan parmaklar: orta açık cilt tonu',
    '🤌🏽' => 'kıstırma hareketi yapan parmaklar: orta cilt tonu',
    '🤌🏾' => 'kıstırma hareketi yapan parmaklar: orta koyu cilt tonu',
    '🤌🏿' => 'kıstırma hareketi yapan parmaklar: koyu cilt tonu',
    '🤏🏻' => 'kıstırma hareketi yapan el: açık cilt tonu',
    '🤏🏼' => 'kıstırma hareketi yapan el: orta açık cilt tonu',
    '🤏🏽' => 'kıstırma hareketi yapan el: orta cilt tonu',
    '🤏🏾' => 'kıstırma hareketi yapan el: orta koyu cilt tonu',
    '🤏🏿' => 'kıstırma hareketi yapan el: koyu cilt tonu',
    '✌🏻' => 'zafer işareti: açık cilt tonu',
    '✌🏼' => 'zafer işareti: orta açık cilt tonu',
    '✌🏽' => 'zafer işareti: orta cilt tonu',
    '✌🏾' => 'zafer işareti: orta koyu cilt tonu',
    '✌🏿' => 'zafer işareti: koyu cilt tonu',
    '🤞🏻' => 'şans dileme: açık cilt tonu',
    '🤞🏼' => 'şans dileme: orta açık cilt tonu',
    '🤞🏽' => 'şans dileme: orta cilt tonu',
    '🤞🏾' => 'şans dileme: orta koyu cilt tonu',
    '🤞🏿' => 'şans dileme: koyu cilt tonu',
    '🫰🏻' => 'işaret parmağı ve baş parmak birbirine geçmiş el: açık cilt tonu',
    '🫰🏼' => 'işaret parmağı ve baş parmak birbirine geçmiş el: orta açık cilt tonu',
    '🫰🏽' => 'işaret parmağı ve baş parmak birbirine geçmiş el: orta cilt tonu',
    '🫰🏾' => 'işaret parmağı ve baş parmak birbirine geçmiş el: orta koyu cilt tonu',
    '🫰🏿' => 'işaret parmağı ve baş parmak birbirine geçmiş el: koyu cilt tonu',
    '🤟🏻' => 'seni seviyorum işareti: açık cilt tonu',
    '🤟🏼' => 'seni seviyorum işareti: orta açık cilt tonu',
    '🤟🏽' => 'seni seviyorum işareti: orta cilt tonu',
    '🤟🏾' => 'seni seviyorum işareti: orta koyu cilt tonu',
    '🤟🏿' => 'seni seviyorum işareti: koyu cilt tonu',
    '🤘🏻' => 'boynuz işareti: açık cilt tonu',
    '🤘🏼' => 'boynuz işareti: orta açık cilt tonu',
    '🤘🏽' => 'boynuz işareti: orta cilt tonu',
    '🤘🏾' => 'boynuz işareti: orta koyu cilt tonu',
    '🤘🏿' => 'boynuz işareti: koyu cilt tonu',
    '🤙🏻' => 'beni ara işareti: açık cilt tonu',
    '🤙🏼' => 'beni ara işareti: orta açık cilt tonu',
    '🤙🏽' => 'beni ara işareti: orta cilt tonu',
    '🤙🏾' => 'beni ara işareti: orta koyu cilt tonu',
    '🤙🏿' => 'beni ara işareti: koyu cilt tonu',
    '👈🏻' => 'elinin tersiyle solu gösteren işaret parmağı: açık cilt tonu',
    '👈🏼' => 'elinin tersiyle solu gösteren işaret parmağı: orta açık cilt tonu',
    '👈🏽' => 'elinin tersiyle solu gösteren işaret parmağı: orta cilt tonu',
    '👈🏾' => 'elinin tersiyle solu gösteren işaret parmağı: orta koyu cilt tonu',
    '👈🏿' => 'elinin tersiyle solu gösteren işaret parmağı: koyu cilt tonu',
    '👉🏻' => 'elinin tersiyle sağı gösteren işaret parmağı: açık cilt tonu',
    '👉🏼' => 'elinin tersiyle sağı gösteren işaret parmağı: orta açık cilt tonu',
    '👉🏽' => 'elinin tersiyle sağı gösteren işaret parmağı: orta cilt tonu',
    '👉🏾' => 'elinin tersiyle sağı gösteren işaret parmağı: orta koyu cilt tonu',
    '👉🏿' => 'elinin tersiyle sağı gösteren işaret parmağı: koyu cilt tonu',
    '👆🏻' => 'elinin tersiyle yukarıyı gösteren işaret parmağı: açık cilt tonu',
    '👆🏼' => 'elinin tersiyle yukarıyı gösteren işaret parmağı: orta açık cilt tonu',
    '👆🏽' => 'elinin tersiyle yukarıyı gösteren işaret parmağı: orta cilt tonu',
    '👆🏾' => 'elinin tersiyle yukarıyı gösteren işaret parmağı: orta koyu cilt tonu',
    '👆🏿' => 'elinin tersiyle yukarıyı gösteren işaret parmağı: koyu cilt tonu',
    '🖕🏻' => 'orta parmak: açık cilt tonu',
    '🖕🏼' => 'orta parmak: orta açık cilt tonu',
    '🖕🏽' => 'orta parmak: orta cilt tonu',
    '🖕🏾' => 'orta parmak: orta koyu cilt tonu',
    '🖕🏿' => 'orta parmak: koyu cilt tonu',
    '👇🏻' => 'elinin tersiyle aşağıyı gösteren işaret parmağı: açık cilt tonu',
    '👇🏼' => 'elinin tersiyle aşağıyı gösteren işaret parmağı: orta açık cilt tonu',
    '👇🏽' => 'elinin tersiyle aşağıyı gösteren işaret parmağı: orta cilt tonu',
    '👇🏾' => 'elinin tersiyle aşağıyı gösteren işaret parmağı: orta koyu cilt tonu',
    '👇🏿' => 'elinin tersiyle aşağıyı gösteren işaret parmağı: koyu cilt tonu',
    '☝🏻' => 'yukarıyı gösteren işaret parmağı: açık cilt tonu',
    '☝🏼' => 'yukarıyı gösteren işaret parmağı: orta açık cilt tonu',
    '☝🏽' => 'yukarıyı gösteren işaret parmağı: orta cilt tonu',
    '☝🏾' => 'yukarıyı gösteren işaret parmağı: orta koyu cilt tonu',
    '☝🏿' => 'yukarıyı gösteren işaret parmağı: koyu cilt tonu',
    '🫵🏻' => 'karşıyı işaret eden işaret parmağı: açık cilt tonu',
    '🫵🏼' => 'karşıyı işaret eden işaret parmağı: orta açık cilt tonu',
    '🫵🏽' => 'karşıyı işaret eden işaret parmağı: orta cilt tonu',
    '🫵🏾' => 'karşıyı işaret eden işaret parmağı: orta koyu cilt tonu',
    '🫵🏿' => 'karşıyı işaret eden işaret parmağı: koyu cilt tonu',
    '👍🏻' => 'baş parmak yukarıda: açık cilt tonu',
    '👍🏼' => 'baş parmak yukarıda: orta açık cilt tonu',
    '👍🏽' => 'baş parmak yukarıda: orta cilt tonu',
    '👍🏾' => 'baş parmak yukarıda: orta koyu cilt tonu',
    '👍🏿' => 'baş parmak yukarıda: koyu cilt tonu',
    '👎🏻' => 'baş parmak aşağıda: açık cilt tonu',
    '👎🏼' => 'baş parmak aşağıda: orta açık cilt tonu',
    '👎🏽' => 'baş parmak aşağıda: orta cilt tonu',
    '👎🏾' => 'baş parmak aşağıda: orta koyu cilt tonu',
    '👎🏿' => 'baş parmak aşağıda: koyu cilt tonu',
    '✊🏻' => 'yumruk kaldırma: açık cilt tonu',
    '✊🏼' => 'yumruk kaldırma: orta açık cilt tonu',
    '✊🏽' => 'yumruk kaldırma: orta cilt tonu',
    '✊🏾' => 'yumruk kaldırma: orta koyu cilt tonu',
    '✊🏿' => 'yumruk kaldırma: koyu cilt tonu',
    '👊🏻' => 'yaklaşan yumruk: açık cilt tonu',
    '👊🏼' => 'yaklaşan yumruk: orta açık cilt tonu',
    '👊🏽' => 'yaklaşan yumruk: orta cilt tonu',
    '👊🏾' => 'yaklaşan yumruk: orta koyu cilt tonu',
    '👊🏿' => 'yaklaşan yumruk: koyu cilt tonu',
    '🤛🏻' => 'sola bakan yumruk: açık cilt tonu',
    '🤛🏼' => 'sola bakan yumruk: orta açık cilt tonu',
    '🤛🏽' => 'sola bakan yumruk: orta cilt tonu',
    '🤛🏾' => 'sola bakan yumruk: orta koyu cilt tonu',
    '🤛🏿' => 'sola bakan yumruk: koyu cilt tonu',
    '🤜🏻' => 'sağa bakan yumruk: açık cilt tonu',
    '🤜🏼' => 'sağa bakan yumruk: orta açık cilt tonu',
    '🤜🏽' => 'sağa bakan yumruk: orta cilt tonu',
    '🤜🏾' => 'sağa bakan yumruk: orta koyu cilt tonu',
    '🤜🏿' => 'sağa bakan yumruk: koyu cilt tonu',
    '👏🏻' => 'el çırpma: açık cilt tonu',
    '👏🏼' => 'el çırpma: orta açık cilt tonu',
    '👏🏽' => 'el çırpma: orta cilt tonu',
    '👏🏾' => 'el çırpma: orta koyu cilt tonu',
    '👏🏿' => 'el çırpma: koyu cilt tonu',
    '🙌🏻' => 'eller havaya: açık cilt tonu',
    '🙌🏼' => 'eller havaya: orta açık cilt tonu',
    '🙌🏽' => 'eller havaya: orta cilt tonu',
    '🙌🏾' => 'eller havaya: orta koyu cilt tonu',
    '🙌🏿' => 'eller havaya: koyu cilt tonu',
    '🫶🏻' => 'kalp oluşturan eller: açık cilt tonu',
    '🫶🏼' => 'kalp oluşturan eller: orta açık cilt tonu',
    '🫶🏽' => 'kalp oluşturan eller: orta cilt tonu',
    '🫶🏾' => 'kalp oluşturan eller: orta koyu cilt tonu',
    '🫶🏿' => 'kalp oluşturan eller: koyu cilt tonu',
    '👐🏻' => 'eller açık: açık cilt tonu',
    '👐🏼' => 'eller açık: orta açık cilt tonu',
    '👐🏽' => 'eller açık: orta cilt tonu',
    '👐🏾' => 'eller açık: orta koyu cilt tonu',
    '👐🏿' => 'eller açık: koyu cilt tonu',
    '🤲🏻' => 'iki avuç açık: açık cilt tonu',
    '🤲🏼' => 'iki avuç açık: orta açık cilt tonu',
    '🤲🏽' => 'iki avuç açık: orta cilt tonu',
    '🤲🏾' => 'iki avuç açık: orta koyu cilt tonu',
    '🤲🏿' => 'iki avuç açık: koyu cilt tonu',
    '🤝🏻' => 'el sıkışma: açık cilt tonu',
    '🤝🏼' => 'el sıkışma: orta açık cilt tonu',
    '🤝🏽' => 'el sıkışma: orta cilt tonu',
    '🤝🏾' => 'el sıkışma: orta koyu cilt tonu',
    '🤝🏿' => 'el sıkışma: koyu cilt tonu',
    '🙏🏻' => 'birleşen eller: açık cilt tonu',
    '🙏🏼' => 'birleşen eller: orta açık cilt tonu',
    '🙏🏽' => 'birleşen eller: orta cilt tonu',
    '🙏🏾' => 'birleşen eller: orta koyu cilt tonu',
    '🙏🏿' => 'birleşen eller: koyu cilt tonu',
    '✍🏻' => 'yazı yazan el: açık cilt tonu',
    '✍🏼' => 'yazı yazan el: orta açık cilt tonu',
    '✍🏽' => 'yazı yazan el: orta cilt tonu',
    '✍🏾' => 'yazı yazan el: orta koyu cilt tonu',
    '✍🏿' => 'yazı yazan el: koyu cilt tonu',
    '💅🏻' => 'oje sürme: açık cilt tonu',
    '💅🏼' => 'oje sürme: orta açık cilt tonu',
    '💅🏽' => 'oje sürme: orta cilt tonu',
    '💅🏾' => 'oje sürme: orta koyu cilt tonu',
    '💅🏿' => 'oje sürme: koyu cilt tonu',
    '🤳🏻' => 'selfie: açık cilt tonu',
    '🤳🏼' => 'selfie: orta açık cilt tonu',
    '🤳🏽' => 'selfie: orta cilt tonu',
    '🤳🏾' => 'selfie: orta koyu cilt tonu',
    '🤳🏿' => 'selfie: koyu cilt tonu',
    '💪🏻' => 'pazı: açık cilt tonu',
    '💪🏼' => 'pazı: orta açık cilt tonu',
    '💪🏽' => 'pazı: orta cilt tonu',
    '💪🏾' => 'pazı: orta koyu cilt tonu',
    '💪🏿' => 'pazı: koyu cilt tonu',
    '🦵🏻' => 'bacak: açık cilt tonu',
    '🦵🏼' => 'bacak: orta açık cilt tonu',
    '🦵🏽' => 'bacak: orta cilt tonu',
    '🦵🏾' => 'bacak: orta koyu cilt tonu',
    '🦵🏿' => 'bacak: koyu cilt tonu',
    '🦶🏻' => 'ayak: açık cilt tonu',
    '🦶🏼' => 'ayak: orta açık cilt tonu',
    '🦶🏽' => 'ayak: orta cilt tonu',
    '🦶🏾' => 'ayak: orta koyu cilt tonu',
    '🦶🏿' => 'ayak: koyu cilt tonu',
    '👂🏻' => 'kulak: açık cilt tonu',
    '👂🏼' => 'kulak: orta açık cilt tonu',
    '👂🏽' => 'kulak: orta cilt tonu',
    '👂🏾' => 'kulak: orta koyu cilt tonu',
    '👂🏿' => 'kulak: koyu cilt tonu',
    '🦻🏻' => 'işitme cihazı olan kulak: açık cilt tonu',
    '🦻🏼' => 'işitme cihazı olan kulak: orta açık cilt tonu',
    '🦻🏽' => 'işitme cihazı olan kulak: orta cilt tonu',
    '🦻🏾' => 'işitme cihazı olan kulak: orta koyu cilt tonu',
    '🦻🏿' => 'işitme cihazı olan kulak: koyu cilt tonu',
    '👃🏻' => 'burun: açık cilt tonu',
    '👃🏼' => 'burun: orta açık cilt tonu',
    '👃🏽' => 'burun: orta cilt tonu',
    '👃🏾' => 'burun: orta koyu cilt tonu',
    '👃🏿' => 'burun: koyu cilt tonu',
    '👶🏻' => 'bebek: açık cilt tonu',
    '👶🏼' => 'bebek: orta açık cilt tonu',
    '👶🏽' => 'bebek: orta cilt tonu',
    '👶🏾' => 'bebek: orta koyu cilt tonu',
    '👶🏿' => 'bebek: koyu cilt tonu',
    '🧒🏻' => 'çocuk: açık cilt tonu',
    '🧒🏼' => 'çocuk: orta açık cilt tonu',
    '🧒🏽' => 'çocuk: orta cilt tonu',
    '🧒🏾' => 'çocuk: orta koyu cilt tonu',
    '🧒🏿' => 'çocuk: koyu cilt tonu',
    '👦🏻' => 'erkek çocuk: açık cilt tonu',
    '👦🏼' => 'erkek çocuk: orta açık cilt tonu',
    '👦🏽' => 'erkek çocuk: orta cilt tonu',
    '👦🏾' => 'erkek çocuk: orta koyu cilt tonu',
    '👦🏿' => 'erkek çocuk: koyu cilt tonu',
    '👧🏻' => 'kız çocuk: açık cilt tonu',
    '👧🏼' => 'kız çocuk: orta açık cilt tonu',
    '👧🏽' => 'kız çocuk: orta cilt tonu',
    '👧🏾' => 'kız çocuk: orta koyu cilt tonu',
    '👧🏿' => 'kız çocuk: koyu cilt tonu',
    '🧑🏻' => 'yetişkin: açık cilt tonu',
    '🧑🏼' => 'yetişkin: orta açık cilt tonu',
    '🧑🏽' => 'yetişkin: orta cilt tonu',
    '🧑🏾' => 'yetişkin: orta koyu cilt tonu',
    '🧑🏿' => 'yetişkin: koyu cilt tonu',
    '👱🏻' => 'sarışın kişi: açık cilt tonu',
    '👱🏼' => 'sarışın kişi: orta açık cilt tonu',
    '👱🏽' => 'sarışın kişi: orta cilt tonu',
    '👱🏾' => 'sarışın kişi: orta koyu cilt tonu',
    '👱🏿' => 'sarışın kişi: koyu cilt tonu',
    '👨🏻' => 'erkek: açık cilt tonu',
    '👨🏼' => 'erkek: orta açık cilt tonu',
    '👨🏽' => 'erkek: orta cilt tonu',
    '👨🏾' => 'erkek: orta koyu cilt tonu',
    '👨🏿' => 'erkek: koyu cilt tonu',
    '🧔🏻' => 'sakallı kişi: açık cilt tonu',
    '🧔🏼' => 'sakallı kişi: orta açık cilt tonu',
    '🧔🏽' => 'sakallı kişi: orta cilt tonu',
    '🧔🏾' => 'sakallı kişi: orta koyu cilt tonu',
    '🧔🏿' => 'sakallı kişi: koyu cilt tonu',
    '👩🏻' => 'kadın: açık cilt tonu',
    '👩🏼' => 'kadın: orta açık cilt tonu',
    '👩🏽' => 'kadın: orta cilt tonu',
    '👩🏾' => 'kadın: orta koyu cilt tonu',
    '👩🏿' => 'kadın: koyu cilt tonu',
    '🧓🏻' => 'yaşlı yetişkin: açık cilt tonu',
    '🧓🏼' => 'yaşlı yetişkin: orta açık cilt tonu',
    '🧓🏽' => 'yaşlı yetişkin: orta cilt tonu',
    '🧓🏾' => 'yaşlı yetişkin: orta koyu cilt tonu',
    '🧓🏿' => 'yaşlı yetişkin: koyu cilt tonu',
    '👴🏻' => 'yaşlı erkek: açık cilt tonu',
    '👴🏼' => 'yaşlı erkek: orta açık cilt tonu',
    '👴🏽' => 'yaşlı erkek: orta cilt tonu',
    '👴🏾' => 'yaşlı erkek: orta koyu cilt tonu',
    '👴🏿' => 'yaşlı erkek: koyu cilt tonu',
    '👵🏻' => 'yaşlı kadın: açık cilt tonu',
    '👵🏼' => 'yaşlı kadın: orta açık cilt tonu',
    '👵🏽' => 'yaşlı kadın: orta cilt tonu',
    '👵🏾' => 'yaşlı kadın: orta koyu cilt tonu',
    '👵🏿' => 'yaşlı kadın: koyu cilt tonu',
    '🙍🏻' => 'somurtma: açık cilt tonu',
    '🙍🏼' => 'somurtma: orta açık cilt tonu',
    '🙍🏽' => 'somurtma: orta cilt tonu',
    '🙍🏾' => 'somurtma: orta koyu cilt tonu',
    '🙍🏿' => 'somurtma: koyu cilt tonu',
    '🙎🏻' => 'darılma: açık cilt tonu',
    '🙎🏼' => 'darılma: orta açık cilt tonu',
    '🙎🏽' => 'darılma: orta cilt tonu',
    '🙎🏾' => 'darılma: orta koyu cilt tonu',
    '🙎🏿' => 'darılma: koyu cilt tonu',
    '🙅🏻' => 'hayır hareketi: açık cilt tonu',
    '🙅🏼' => 'hayır hareketi: orta açık cilt tonu',
    '🙅🏽' => 'hayır hareketi: orta cilt tonu',
    '🙅🏾' => 'hayır hareketi: orta koyu cilt tonu',
    '🙅🏿' => 'hayır hareketi: koyu cilt tonu',
    '🙆🏻' => 'tamam hareketi: açık cilt tonu',
    '🙆🏼' => 'tamam hareketi: orta açık cilt tonu',
    '🙆🏽' => 'tamam hareketi: orta cilt tonu',
    '🙆🏾' => 'tamam hareketi: orta koyu cilt tonu',
    '🙆🏿' => 'tamam hareketi: koyu cilt tonu',
    '💁🏻' => 'elini yana yatırma: açık cilt tonu',
    '💁🏼' => 'elini yana yatırma: orta açık cilt tonu',
    '💁🏽' => 'elini yana yatırma: orta cilt tonu',
    '💁🏾' => 'elini yana yatırma: orta koyu cilt tonu',
    '💁🏿' => 'elini yana yatırma: koyu cilt tonu',
    '🙋🏻' => 'elini kaldırma: açık cilt tonu',
    '🙋🏼' => 'elini kaldırma: orta açık cilt tonu',
    '🙋🏽' => 'elini kaldırma: orta cilt tonu',
    '🙋🏾' => 'elini kaldırma: orta koyu cilt tonu',
    '🙋🏿' => 'elini kaldırma: koyu cilt tonu',
    '🧏🏻' => 'işitme engelli kişi: açık cilt tonu',
    '🧏🏼' => 'işitme engelli kişi: orta açık cilt tonu',
    '🧏🏽' => 'işitme engelli kişi: orta cilt tonu',
    '🧏🏾' => 'işitme engelli kişi: orta koyu cilt tonu',
    '🧏🏿' => 'işitme engelli kişi: koyu cilt tonu',
    '🙇🏻' => 'başını öne eğme: açık cilt tonu',
    '🙇🏼' => 'başını öne eğme: orta açık cilt tonu',
    '🙇🏽' => 'başını öne eğme: orta cilt tonu',
    '🙇🏾' => 'başını öne eğme: orta koyu cilt tonu',
    '🙇🏿' => 'başını öne eğme: koyu cilt tonu',
    '🤦🏻' => 'yüzünü eliyle kapama: açık cilt tonu',
    '🤦🏼' => 'yüzünü eliyle kapama: orta açık cilt tonu',
    '🤦🏽' => 'yüzünü eliyle kapama: orta cilt tonu',
    '🤦🏾' => 'yüzünü eliyle kapama: orta koyu cilt tonu',
    '🤦🏿' => 'yüzünü eliyle kapama: koyu cilt tonu',
    '🤷🏻' => 'omuz silkme: açık cilt tonu',
    '🤷🏼' => 'omuz silkme: orta açık cilt tonu',
    '🤷🏽' => 'omuz silkme: orta cilt tonu',
    '🤷🏾' => 'omuz silkme: orta koyu cilt tonu',
    '🤷🏿' => 'omuz silkme: koyu cilt tonu',
    '👮🏻' => 'polis memuru: açık cilt tonu',
    '👮🏼' => 'polis memuru: orta açık cilt tonu',
    '👮🏽' => 'polis memuru: orta cilt tonu',
    '👮🏾' => 'polis memuru: orta koyu cilt tonu',
    '👮🏿' => 'polis memuru: koyu cilt tonu',
    '🕵🏻' => 'dedektif: açık cilt tonu',
    '🕵🏼' => 'dedektif: orta açık cilt tonu',
    '🕵🏽' => 'dedektif: orta cilt tonu',
    '🕵🏾' => 'dedektif: orta koyu cilt tonu',
    '🕵🏿' => 'dedektif: koyu cilt tonu',
    '💂🏻' => 'muhafız: açık cilt tonu',
    '💂🏼' => 'muhafız: orta açık cilt tonu',
    '💂🏽' => 'muhafız: orta cilt tonu',
    '💂🏾' => 'muhafız: orta koyu cilt tonu',
    '💂🏿' => 'muhafız: koyu cilt tonu',
    '🥷🏻' => 'ninja: açık cilt tonu',
    '🥷🏼' => 'ninja: orta açık cilt tonu',
    '🥷🏽' => 'ninja: orta cilt tonu',
    '🥷🏾' => 'ninja: orta koyu cilt tonu',
    '🥷🏿' => 'ninja: koyu cilt tonu',
    '👷🏻' => 'inşaat işçisi: açık cilt tonu',
    '👷🏼' => 'inşaat işçisi: orta açık cilt tonu',
    '👷🏽' => 'inşaat işçisi: orta cilt tonu',
    '👷🏾' => 'inşaat işçisi: orta koyu cilt tonu',
    '👷🏿' => 'inşaat işçisi: koyu cilt tonu',
    '🫅🏻' => 'taç takmış insan: açık cilt tonu',
    '🫅🏼' => 'taç takmış insan: orta açık cilt tonu',
    '🫅🏽' => 'taç takmış insan: orta cilt tonu',
    '🫅🏾' => 'taç takmış insan: orta koyu cilt tonu',
    '🫅🏿' => 'taç takmış insan: koyu cilt tonu',
    '🤴🏻' => 'prens: açık cilt tonu',
    '🤴🏼' => 'prens: orta açık cilt tonu',
    '🤴🏽' => 'prens: orta cilt tonu',
    '🤴🏾' => 'prens: orta koyu cilt tonu',
    '🤴🏿' => 'prens: koyu cilt tonu',
    '👸🏻' => 'prenses: açık cilt tonu',
    '👸🏼' => 'prenses: orta açık cilt tonu',
    '👸🏽' => 'prenses: orta cilt tonu',
    '👸🏾' => 'prenses: orta koyu cilt tonu',
    '👸🏿' => 'prenses: koyu cilt tonu',
    '👳🏻' => 'sarıklı kişi: açık cilt tonu',
    '👳🏼' => 'sarıklı kişi: orta açık cilt tonu',
    '👳🏽' => 'sarıklı kişi: orta cilt tonu',
    '👳🏾' => 'sarıklı kişi: orta koyu cilt tonu',
    '👳🏿' => 'sarıklı kişi: koyu cilt tonu',
    '👲🏻' => 'çin şapkalı erkek: açık cilt tonu',
    '👲🏼' => 'çin şapkalı erkek: orta açık cilt tonu',
    '👲🏽' => 'çin şapkalı erkek: orta cilt tonu',
    '👲🏾' => 'çin şapkalı erkek: orta koyu cilt tonu',
    '👲🏿' => 'çin şapkalı erkek: koyu cilt tonu',
    '🧕🏻' => 'başörtülü kadın: açık cilt tonu',
    '🧕🏼' => 'başörtülü kadın: orta açık cilt tonu',
    '🧕🏽' => 'başörtülü kadın: orta cilt tonu',
    '🧕🏾' => 'başörtülü kadın: orta koyu cilt tonu',
    '🧕🏿' => 'başörtülü kadın: koyu cilt tonu',
    '🤵🏻' => 'smokinli kişi: açık cilt tonu',
    '🤵🏼' => 'smokinli kişi: orta açık cilt tonu',
    '🤵🏽' => 'smokinli kişi: orta cilt tonu',
    '🤵🏾' => 'smokinli kişi: orta koyu cilt tonu',
    '🤵🏿' => 'smokinli kişi: koyu cilt tonu',
    '👰🏻' => 'duvaklı gelin: açık cilt tonu',
    '👰🏼' => 'duvaklı gelin: orta açık cilt tonu',
    '👰🏽' => 'duvaklı gelin: orta cilt tonu',
    '👰🏾' => 'duvaklı gelin: orta koyu cilt tonu',
    '👰🏿' => 'duvaklı gelin: koyu cilt tonu',
    '🤰🏻' => 'hamile kadın: açık cilt tonu',
    '🤰🏼' => 'hamile kadın: orta açık cilt tonu',
    '🤰🏽' => 'hamile kadın: orta cilt tonu',
    '🤰🏾' => 'hamile kadın: orta koyu cilt tonu',
    '🤰🏿' => 'hamile kadın: koyu cilt tonu',
    '🫃🏻' => 'hamile adam: açık cilt tonu',
    '🫃🏼' => 'hamile adam: orta açık cilt tonu',
    '🫃🏽' => 'hamile adam: orta cilt tonu',
    '🫃🏾' => 'hamile adam: orta koyu cilt tonu',
    '🫃🏿' => 'hamile adam: koyu cilt tonu',
    '🫄🏻' => 'hamile kişi: açık cilt tonu',
    '🫄🏼' => 'hamile kişi: orta açık cilt tonu',
    '🫄🏽' => 'hamile kişi: orta cilt tonu',
    '🫄🏾' => 'hamile kişi: orta koyu cilt tonu',
    '🫄🏿' => 'hamile kişi: koyu cilt tonu',
    '🤱🏻' => 'emziren kadın: açık cilt tonu',
    '🤱🏼' => 'emziren kadın: orta açık cilt tonu',
    '🤱🏽' => 'emziren kadın: orta cilt tonu',
    '🤱🏾' => 'emziren kadın: orta koyu cilt tonu',
    '🤱🏿' => 'emziren kadın: koyu cilt tonu',
    '👼🏻' => 'bebek melek: açık cilt tonu',
    '👼🏼' => 'bebek melek: orta açık cilt tonu',
    '👼🏽' => 'bebek melek: orta cilt tonu',
    '👼🏾' => 'bebek melek: orta koyu cilt tonu',
    '👼🏿' => 'bebek melek: koyu cilt tonu',
    '🎅🏻' => 'Noel Baba: açık cilt tonu',
    '🎅🏼' => 'Noel Baba: orta açık cilt tonu',
    '🎅🏽' => 'Noel Baba: orta cilt tonu',
    '🎅🏾' => 'Noel Baba: orta koyu cilt tonu',
    '🎅🏿' => 'Noel Baba: koyu cilt tonu',
    '🤶🏻' => 'Noel Anne: açık cilt tonu',
    '🤶🏼' => 'Noel Anne: orta açık cilt tonu',
    '🤶🏽' => 'Noel Anne: orta cilt tonu',
    '🤶🏾' => 'Noel Anne: orta koyu cilt tonu',
    '🤶🏿' => 'Noel Anne: koyu cilt tonu',
    '🦸🏻' => 'süper kahraman: açık cilt tonu',
    '🦸🏼' => 'süper kahraman: orta açık cilt tonu',
    '🦸🏽' => 'süper kahraman: orta cilt tonu',
    '🦸🏾' => 'süper kahraman: orta koyu cilt tonu',
    '🦸🏿' => 'süper kahraman: koyu cilt tonu',
    '🦹🏻' => 'süper kötü: açık cilt tonu',
    '🦹🏼' => 'süper kötü: orta açık cilt tonu',
    '🦹🏽' => 'süper kötü: orta cilt tonu',
    '🦹🏾' => 'süper kötü: orta koyu cilt tonu',
    '🦹🏿' => 'süper kötü: koyu cilt tonu',
    '🧙🏻' => 'büyücü: açık cilt tonu',
    '🧙🏼' => 'büyücü: orta açık cilt tonu',
    '🧙🏽' => 'büyücü: orta cilt tonu',
    '🧙🏾' => 'büyücü: orta koyu cilt tonu',
    '🧙🏿' => 'büyücü: koyu cilt tonu',
    '🧚🏻' => 'peri: açık cilt tonu',
    '🧚🏼' => 'peri: orta açık cilt tonu',
    '🧚🏽' => 'peri: orta cilt tonu',
    '🧚🏾' => 'peri: orta koyu cilt tonu',
    '🧚🏿' => 'peri: koyu cilt tonu',
    '🧛🏻' => 'vampir: açık cilt tonu',
    '🧛🏼' => 'vampir: orta açık cilt tonu',
    '🧛🏽' => 'vampir: orta cilt tonu',
    '🧛🏾' => 'vampir: orta koyu cilt tonu',
    '🧛🏿' => 'vampir: koyu cilt tonu',
    '🧜🏻' => 'balık insan: açık cilt tonu',
    '🧜🏼' => 'balık insan: orta açık cilt tonu',
    '🧜🏽' => 'balık insan: orta cilt tonu',
    '🧜🏾' => 'balık insan: orta koyu cilt tonu',
    '🧜🏿' => 'balık insan: koyu cilt tonu',
    '🧝🏻' => 'elf: açık cilt tonu',
    '🧝🏼' => 'elf: orta açık cilt tonu',
    '🧝🏽' => 'elf: orta cilt tonu',
    '🧝🏾' => 'elf: orta koyu cilt tonu',
    '🧝🏿' => 'elf: koyu cilt tonu',
    '💆🏻' => 'yüz masajı: açık cilt tonu',
    '💆🏼' => 'yüz masajı: orta açık cilt tonu',
    '💆🏽' => 'yüz masajı: orta cilt tonu',
    '💆🏾' => 'yüz masajı: orta koyu cilt tonu',
    '💆🏿' => 'yüz masajı: koyu cilt tonu',
    '💇🏻' => 'saç kesimi: açık cilt tonu',
    '💇🏼' => 'saç kesimi: orta açık cilt tonu',
    '💇🏽' => 'saç kesimi: orta cilt tonu',
    '💇🏾' => 'saç kesimi: orta koyu cilt tonu',
    '💇🏿' => 'saç kesimi: koyu cilt tonu',
    '🚶🏻' => 'yürüyen kişi: açık cilt tonu',
    '🚶🏼' => 'yürüyen kişi: orta açık cilt tonu',
    '🚶🏽' => 'yürüyen kişi: orta cilt tonu',
    '🚶🏾' => 'yürüyen kişi: orta koyu cilt tonu',
    '🚶🏿' => 'yürüyen kişi: koyu cilt tonu',
    '🧍🏻' => 'ayakta duran kişi: açık cilt tonu',
    '🧍🏼' => 'ayakta duran kişi: orta açık cilt tonu',
    '🧍🏽' => 'ayakta duran kişi: orta cilt tonu',
    '🧍🏾' => 'ayakta duran kişi: orta koyu cilt tonu',
    '🧍🏿' => 'ayakta duran kişi: koyu cilt tonu',
    '🧎🏻' => 'diz çöken kişi: açık cilt tonu',
    '🧎🏼' => 'diz çöken kişi: orta açık cilt tonu',
    '🧎🏽' => 'diz çöken kişi: orta cilt tonu',
    '🧎🏾' => 'diz çöken kişi: orta koyu cilt tonu',
    '🧎🏿' => 'diz çöken kişi: koyu cilt tonu',
    '🏃🏻' => 'koşan kişi: açık cilt tonu',
    '🏃🏼' => 'koşan kişi: orta açık cilt tonu',
    '🏃🏽' => 'koşan kişi: orta cilt tonu',
    '🏃🏾' => 'koşan kişi: orta koyu cilt tonu',
    '🏃🏿' => 'koşan kişi: koyu cilt tonu',
    '💃🏻' => 'dans eden kadın: açık cilt tonu',
    '💃🏼' => 'dans eden kadın: orta açık cilt tonu',
    '💃🏽' => 'dans eden kadın: orta cilt tonu',
    '💃🏾' => 'dans eden kadın: orta koyu cilt tonu',
    '💃🏿' => 'dans eden kadın: koyu cilt tonu',
    '🕺🏻' => 'dans eden adam: açık cilt tonu',
    '🕺🏼' => 'dans eden adam: orta açık cilt tonu',
    '🕺🏽' => 'dans eden adam: orta cilt tonu',
    '🕺🏾' => 'dans eden adam: orta koyu cilt tonu',
    '🕺🏿' => 'dans eden adam: koyu cilt tonu',
    '🕴🏻' => 'havalanan takım elbiseli adam: açık cilt tonu',
    '🕴🏼' => 'havalanan takım elbiseli adam: orta açık cilt tonu',
    '🕴🏽' => 'havalanan takım elbiseli adam: orta cilt tonu',
    '🕴🏾' => 'havalanan takım elbiseli adam: orta koyu cilt tonu',
    '🕴🏿' => 'havalanan takım elbiseli adam: koyu cilt tonu',
    '👯🏻' => 'tavşan kulaklı kişiler: açık cilt tonu',
    '👯🏼' => 'tavşan kulaklı kişiler: orta açık cilt tonu',
    '👯🏽' => 'tavşan kulaklı kişiler: orta cilt tonu',
    '👯🏾' => 'tavşan kulaklı kişiler: orta koyu cilt tonu',
    '👯🏿' => 'tavşan kulaklı kişiler: koyu cilt tonu',
    '🧖🏻' => 'buhar odasındaki kişi: açık cilt tonu',
    '🧖🏼' => 'buhar odasındaki kişi: orta açık cilt tonu',
    '🧖🏽' => 'buhar odasındaki kişi: orta cilt tonu',
    '🧖🏾' => 'buhar odasındaki kişi: orta koyu cilt tonu',
    '🧖🏿' => 'buhar odasındaki kişi: koyu cilt tonu',
    '🧗🏻' => 'dağcı kişi: açık cilt tonu',
    '🧗🏼' => 'dağcı kişi: orta açık cilt tonu',
    '🧗🏽' => 'dağcı kişi: orta cilt tonu',
    '🧗🏾' => 'dağcı kişi: orta koyu cilt tonu',
    '🧗🏿' => 'dağcı kişi: koyu cilt tonu',
    '🏇🏻' => 'at yarışı: açık cilt tonu',
    '🏇🏼' => 'at yarışı: orta açık cilt tonu',
    '🏇🏽' => 'at yarışı: orta cilt tonu',
    '🏇🏾' => 'at yarışı: orta koyu cilt tonu',
    '🏇🏿' => 'at yarışı: koyu cilt tonu',
    '🏂🏻' => 'snowbordçu: açık cilt tonu',
    '🏂🏼' => 'snowbordçu: orta açık cilt tonu',
    '🏂🏽' => 'snowbordçu: orta cilt tonu',
    '🏂🏾' => 'snowbordçu: orta koyu cilt tonu',
    '🏂🏿' => 'snowbordçu: koyu cilt tonu',
    '🏌🏻' => 'golf oynayan kişi: açık cilt tonu',
    '🏌🏼' => 'golf oynayan kişi: orta açık cilt tonu',
    '🏌🏽' => 'golf oynayan kişi: orta cilt tonu',
    '🏌🏾' => 'golf oynayan kişi: orta koyu cilt tonu',
    '🏌🏿' => 'golf oynayan kişi: koyu cilt tonu',
    '🏄🏻' => 'sörf yapan kişi: açık cilt tonu',
    '🏄🏼' => 'sörf yapan kişi: orta açık cilt tonu',
    '🏄🏽' => 'sörf yapan kişi: orta cilt tonu',
    '🏄🏾' => 'sörf yapan kişi: orta koyu cilt tonu',
    '🏄🏿' => 'sörf yapan kişi: koyu cilt tonu',
    '🚣🏻' => 'sandalda kürek çeken kişi: açık cilt tonu',
    '🚣🏼' => 'sandalda kürek çeken kişi: orta açık cilt tonu',
    '🚣🏽' => 'sandalda kürek çeken kişi: orta cilt tonu',
    '🚣🏾' => 'sandalda kürek çeken kişi: orta koyu cilt tonu',
    '🚣🏿' => 'sandalda kürek çeken kişi: koyu cilt tonu',
    '🏊🏻' => 'yüzen kişi: açık cilt tonu',
    '🏊🏼' => 'yüzen kişi: orta açık cilt tonu',
    '🏊🏽' => 'yüzen kişi: orta cilt tonu',
    '🏊🏾' => 'yüzen kişi: orta koyu cilt tonu',
    '🏊🏿' => 'yüzen kişi: koyu cilt tonu',
    '⛹🏻' => 'top sektiren kişi: açık cilt tonu',
    '⛹🏼' => 'top sektiren kişi: orta açık cilt tonu',
    '⛹🏽' => 'top sektiren kişi: orta cilt tonu',
    '⛹🏾' => 'top sektiren kişi: orta koyu cilt tonu',
    '⛹🏿' => 'top sektiren kişi: koyu cilt tonu',
    '🏋🏻' => 'ağırlık kaldıran kişi: açık cilt tonu',
    '🏋🏼' => 'ağırlık kaldıran kişi: orta açık cilt tonu',
    '🏋🏽' => 'ağırlık kaldıran kişi: orta cilt tonu',
    '🏋🏾' => 'ağırlık kaldıran kişi: orta koyu cilt tonu',
    '🏋🏿' => 'ağırlık kaldıran kişi: koyu cilt tonu',
    '🚴🏻' => 'bisiklet süren kişi: açık cilt tonu',
    '🚴🏼' => 'bisiklet süren kişi: orta açık cilt tonu',
    '🚴🏽' => 'bisiklet süren kişi: orta cilt tonu',
    '🚴🏾' => 'bisiklet süren kişi: orta koyu cilt tonu',
    '🚴🏿' => 'bisiklet süren kişi: koyu cilt tonu',
    '🚵🏻' => 'dağ bisikletçisi: açık cilt tonu',
    '🚵🏼' => 'dağ bisikletçisi: orta açık cilt tonu',
    '🚵🏽' => 'dağ bisikletçisi: orta cilt tonu',
    '🚵🏾' => 'dağ bisikletçisi: orta koyu cilt tonu',
    '🚵🏿' => 'dağ bisikletçisi: koyu cilt tonu',
    '🤸🏻' => 'perende atan kişi: açık cilt tonu',
    '🤸🏼' => 'perende atan kişi: orta açık cilt tonu',
    '🤸🏽' => 'perende atan kişi: orta cilt tonu',
    '🤸🏾' => 'perende atan kişi: orta koyu cilt tonu',
    '🤸🏿' => 'perende atan kişi: koyu cilt tonu',
    '🤼🏻' => 'güreşen kişiler: açık cilt tonu',
    '🤼🏼' => 'güreşen kişiler: orta açık cilt tonu',
    '🤼🏽' => 'güreşen kişiler: orta cilt tonu',
    '🤼🏾' => 'güreşen kişiler: orta koyu cilt tonu',
    '🤼🏿' => 'güreşen kişiler: koyu cilt tonu',
    '🤽🏻' => 'su topu: açık cilt tonu',
    '🤽🏼' => 'su topu: orta açık cilt tonu',
    '🤽🏽' => 'su topu: orta cilt tonu',
    '🤽🏾' => 'su topu: orta koyu cilt tonu',
    '🤽🏿' => 'su topu: koyu cilt tonu',
    '🤾🏻' => 'hentbol: açık cilt tonu',
    '🤾🏼' => 'hentbol: orta açık cilt tonu',
    '🤾🏽' => 'hentbol: orta cilt tonu',
    '🤾🏾' => 'hentbol: orta koyu cilt tonu',
    '🤾🏿' => 'hentbol: koyu cilt tonu',
    '🤹🏻' => 'jonglör: açık cilt tonu',
    '🤹🏼' => 'jonglör: orta açık cilt tonu',
    '🤹🏽' => 'jonglör: orta cilt tonu',
    '🤹🏾' => 'jonglör: orta koyu cilt tonu',
    '🤹🏿' => 'jonglör: koyu cilt tonu',
    '🧘🏻' => 'lotus pozisyonundaki kişi: açık cilt tonu',
    '🧘🏼' => 'lotus pozisyonundaki kişi: orta açık cilt tonu',
    '🧘🏽' => 'lotus pozisyonundaki kişi: orta cilt tonu',
    '🧘🏾' => 'lotus pozisyonundaki kişi: orta koyu cilt tonu',
    '🧘🏿' => 'lotus pozisyonundaki kişi: koyu cilt tonu',
    '🛀🏻' => 'banyo yapan kişi: açık cilt tonu',
    '🛀🏼' => 'banyo yapan kişi: orta açık cilt tonu',
    '🛀🏽' => 'banyo yapan kişi: orta cilt tonu',
    '🛀🏾' => 'banyo yapan kişi: orta koyu cilt tonu',
    '🛀🏿' => 'banyo yapan kişi: koyu cilt tonu',
    '🛌🏻' => 'yatakta yatan kişi: açık cilt tonu',
    '🛌🏼' => 'yatakta yatan kişi: orta açık cilt tonu',
    '🛌🏽' => 'yatakta yatan kişi: orta cilt tonu',
    '🛌🏾' => 'yatakta yatan kişi: orta koyu cilt tonu',
    '🛌🏿' => 'yatakta yatan kişi: koyu cilt tonu',
    '👭🏻' => 'el ele tutuşan kadınlar: açık cilt tonu',
    '👭🏼' => 'el ele tutuşan kadınlar: orta açık cilt tonu',
    '👭🏽' => 'el ele tutuşan kadınlar: orta cilt tonu',
    '👭🏾' => 'el ele tutuşan kadınlar: orta koyu cilt tonu',
    '👭🏿' => 'el ele tutuşan kadınlar: koyu cilt tonu',
    '👫🏻' => 'el ele tutuşan erkek ve kadın: açık cilt tonu',
    '👫🏼' => 'el ele tutuşan erkek ve kadın: orta açık cilt tonu',
    '👫🏽' => 'el ele tutuşan erkek ve kadın: orta cilt tonu',
    '👫🏾' => 'el ele tutuşan erkek ve kadın: orta koyu cilt tonu',
    '👫🏿' => 'el ele tutuşan erkek ve kadın: koyu cilt tonu',
    '👬🏻' => 'el ele tutuşan erkekler: açık cilt tonu',
    '👬🏼' => 'el ele tutuşan erkekler: orta açık cilt tonu',
    '👬🏽' => 'el ele tutuşan erkekler: orta cilt tonu',
    '👬🏾' => 'el ele tutuşan erkekler: orta koyu cilt tonu',
    '👬🏿' => 'el ele tutuşan erkekler: koyu cilt tonu',
    '💏🏻' => 'öpücük: açık cilt tonu',
    '💏🏼' => 'öpücük: orta açık cilt tonu',
    '💏🏽' => 'öpücük: orta cilt tonu',
    '💏🏾' => 'öpücük: orta koyu cilt tonu',
    '💏🏿' => 'öpücük: koyu cilt tonu',
    '💑🏻' => 'kalpli çift: açık cilt tonu',
    '💑🏼' => 'kalpli çift: orta açık cilt tonu',
    '💑🏽' => 'kalpli çift: orta cilt tonu',
    '💑🏾' => 'kalpli çift: orta koyu cilt tonu',
    '💑🏿' => 'kalpli çift: koyu cilt tonu',
    '🇦🇨' => 'bayrak: Ascension Adası',
    '🇦🇩' => 'bayrak: Andorra',
    '🇦🇪' => 'bayrak: Birleşik Arap Emirlikleri',
    '🇦🇫' => 'bayrak: Afganistan',
    '🇦🇬' => 'bayrak: Antigua ve Barbuda',
    '🇦🇮' => 'bayrak: Anguilla',
    '🇦🇱' => 'bayrak: Arnavutluk',
    '🇦🇲' => 'bayrak: Ermenistan',
    '🇦🇴' => 'bayrak: Angola',
    '🇦🇶' => 'bayrak: Antarktika',
    '🇦🇷' => 'bayrak: Arjantin',
    '🇦🇸' => 'bayrak: Amerikan Samoası',
    '🇦🇹' => 'bayrak: Avusturya',
    '🇦🇺' => 'bayrak: Avustralya',
    '🇦🇼' => 'bayrak: Aruba',
    '🇦🇽' => 'bayrak: Åland Adaları',
    '🇦🇿' => 'bayrak: Azerbaycan',
    '🇧🇦' => 'bayrak: Bosna-Hersek',
    '🇧🇧' => 'bayrak: Barbados',
    '🇧🇩' => 'bayrak: Bangladeş',
    '🇧🇪' => 'bayrak: Belçika',
    '🇧🇫' => 'bayrak: Burkina Faso',
    '🇧🇬' => 'bayrak: Bulgaristan',
    '🇧🇭' => 'bayrak: Bahreyn',
    '🇧🇮' => 'bayrak: Burundi',
    '🇧🇯' => 'bayrak: Benin',
    '🇧🇱' => 'bayrak: Saint Barthelemy',
    '🇧🇲' => 'bayrak: Bermuda',
    '🇧🇳' => 'bayrak: Brunei',
    '🇧🇴' => 'bayrak: Bolivya',
    '🇧🇶' => 'bayrak: Karayip Hollandası',
    '🇧🇷' => 'bayrak: Brezilya',
    '🇧🇸' => 'bayrak: Bahamalar',
    '🇧🇹' => 'bayrak: Butan',
    '🇧🇻' => 'bayrak: Bouvet Adası',
    '🇧🇼' => 'bayrak: Botsvana',
    '🇧🇾' => 'bayrak: Belarus',
    '🇧🇿' => 'bayrak: Belize',
    '🇨🇦' => 'bayrak: Kanada',
    '🇨🇨' => 'bayrak: Cocos (Keeling) Adaları',
    '🇨🇩' => 'bayrak: Kongo - Kinşasa',
    '🇨🇫' => 'bayrak: Orta Afrika Cumhuriyeti',
    '🇨🇬' => 'bayrak: Kongo - Brazavil',
    '🇨🇭' => 'bayrak: İsviçre',
    '🇨🇮' => 'bayrak: Côte d’Ivoire',
    '🇨🇰' => 'bayrak: Cook Adaları',
    '🇨🇱' => 'bayrak: Şili',
    '🇨🇲' => 'bayrak: Kamerun',
    '🇨🇳' => 'bayrak: Çin',
    '🇨🇴' => 'bayrak: Kolombiya',
    '🇨🇵' => 'bayrak: Clipperton Adası',
    '🇨🇶' => 'bayrak: Sark',
    '🇨🇷' => 'bayrak: Kosta Rika',
    '🇨🇺' => 'bayrak: Küba',
    '🇨🇻' => 'bayrak: Cabo Verde',
    '🇨🇼' => 'bayrak: Curaçao',
    '🇨🇽' => 'bayrak: Christmas Adası',
    '🇨🇾' => 'bayrak: Kıbrıs',
    '🇨🇿' => 'bayrak: Çekya',
    '🇩🇪' => 'bayrak: Almanya',
    '🇩🇬' => 'bayrak: Diego Garcia',
    '🇩🇯' => 'bayrak: Cibuti',
    '🇩🇰' => 'bayrak: Danimarka',
    '🇩🇲' => 'bayrak: Dominika',
    '🇩🇴' => 'bayrak: Dominik Cumhuriyeti',
    '🇩🇿' => 'bayrak: Cezayir',
    '🇪🇦' => 'bayrak: Ceuta ve Melilla',
    '🇪🇨' => 'bayrak: Ekvador',
    '🇪🇪' => 'bayrak: Estonya',
    '🇪🇬' => 'bayrak: Mısır',
    '🇪🇭' => 'bayrak: Batı Sahra',
    '🇪🇷' => 'bayrak: Eritre',
    '🇪🇸' => 'bayrak: İspanya',
    '🇪🇹' => 'bayrak: Etiyopya',
    '🇪🇺' => 'bayrak: Avrupa Birliği',
    '🇫🇮' => 'bayrak: Finlandiya',
    '🇫🇯' => 'bayrak: Fiji',
    '🇫🇰' => 'bayrak: Falkland Adaları',
    '🇫🇲' => 'bayrak: Mikronezya',
    '🇫🇴' => 'bayrak: Faroe Adaları',
    '🇫🇷' => 'bayrak: Fransa',
    '🇬🇦' => 'bayrak: Gabon',
    '🇬🇧' => 'bayrak: Birleşik Krallık',
    '🇬🇩' => 'bayrak: Grenada',
    '🇬🇪' => 'bayrak: Gürcistan',
    '🇬🇫' => 'bayrak: Fransız Guyanası',
    '🇬🇬' => 'bayrak: Guernsey',
    '🇬🇭' => 'bayrak: Gana',
    '🇬🇮' => 'bayrak: Cebelitarık',
    '🇬🇱' => 'bayrak: Grönland',
    '🇬🇲' => 'bayrak: Gambiya',
    '🇬🇳' => 'bayrak: Gine',
    '🇬🇵' => 'bayrak: Guadeloupe',
    '🇬🇶' => 'bayrak: Ekvator Ginesi',
    '🇬🇷' => 'bayrak: Yunanistan',
    '🇬🇸' => 'bayrak: Güney Georgia ve Güney Sandwich Adaları',
    '🇬🇹' => 'bayrak: Guatemala',
    '🇬🇺' => 'bayrak: Guam',
    '🇬🇼' => 'bayrak: Gine-Bissau',
    '🇬🇾' => 'bayrak: Guyana',
    '🇭🇰' => 'bayrak: Çin Hong Kong ÖİB',
    '🇭🇲' => 'bayrak: Heard Adası ve McDonald Adaları',
    '🇭🇳' => 'bayrak: Honduras',
    '🇭🇷' => 'bayrak: Hırvatistan',
    '🇭🇹' => 'bayrak: Haiti',
    '🇭🇺' => 'bayrak: Macaristan',
    '🇮🇨' => 'bayrak: Kanarya Adaları',
    '🇮🇩' => 'bayrak: Endonezya',
    '🇮🇪' => 'bayrak: İrlanda',
    '🇮🇱' => 'bayrak: İsrail',
    '🇮🇲' => 'bayrak: Man Adası',
    '🇮🇳' => 'bayrak: Hindistan',
    '🇮🇴' => 'bayrak: Britanya Hint Okyanusu Toprakları',
    '🇮🇶' => 'bayrak: Irak',
    '🇮🇷' => 'bayrak: İran',
    '🇮🇸' => 'bayrak: İzlanda',
    '🇮🇹' => 'bayrak: İtalya',
    '🇯🇪' => 'bayrak: Jersey',
    '🇯🇲' => 'bayrak: Jamaika',
    '🇯🇴' => 'bayrak: Ürdün',
    '🇯🇵' => 'bayrak: Japonya',
    '🇰🇪' => 'bayrak: Kenya',
    '🇰🇬' => 'bayrak: Kırgızistan',
    '🇰🇭' => 'bayrak: Kamboçya',
    '🇰🇮' => 'bayrak: Kiribati',
    '🇰🇲' => 'bayrak: Komorlar',
    '🇰🇳' => 'bayrak: Saint Kitts ve Nevis',
    '🇰🇵' => 'bayrak: Kuzey Kore',
    '🇰🇷' => 'bayrak: Güney Kore',
    '🇰🇼' => 'bayrak: Kuveyt',
    '🇰🇾' => 'bayrak: Cayman Adaları',
    '🇰🇿' => 'bayrak: Kazakistan',
    '🇱🇦' => 'bayrak: Laos',
    '🇱🇧' => 'bayrak: Lübnan',
    '🇱🇨' => 'bayrak: Saint Lucia',
    '🇱🇮' => 'bayrak: Liechtenstein',
    '🇱🇰' => 'bayrak: Sri Lanka',
    '🇱🇷' => 'bayrak: Liberya',
    '🇱🇸' => 'bayrak: Lesotho',
    '🇱🇹' => 'bayrak: Litvanya',
    '🇱🇺' => 'bayrak: Lüksemburg',
    '🇱🇻' => 'bayrak: Letonya',
    '🇱🇾' => 'bayrak: Libya',
    '🇲🇦' => 'bayrak: Fas',
    '🇲🇨' => 'bayrak: Monako',
    '🇲🇩' => 'bayrak: Moldova',
    '🇲🇪' => 'bayrak: Karadağ',
    '🇲🇫' => 'bayrak: Saint Martin',
    '🇲🇬' => 'bayrak: Madagaskar',
    '🇲🇭' => 'bayrak: Marshall Adaları',
    '🇲🇰' => 'bayrak: Kuzey Makedonya',
    '🇲🇱' => 'bayrak: Mali',
    '🇲🇲' => 'bayrak: Myanmar (Burma)',
    '🇲🇳' => 'bayrak: Moğolistan',
    '🇲🇴' => 'bayrak: Çin Makao ÖİB',
    '🇲🇵' => 'bayrak: Kuzey Mariana Adaları',
    '🇲🇶' => 'bayrak: Martinik',
    '🇲🇷' => 'bayrak: Moritanya',
    '🇲🇸' => 'bayrak: Montserrat',
    '🇲🇹' => 'bayrak: Malta',
    '🇲🇺' => 'bayrak: Mauritius',
    '🇲🇻' => 'bayrak: Maldivler',
    '🇲🇼' => 'bayrak: Malavi',
    '🇲🇽' => 'bayrak: Meksika',
    '🇲🇾' => 'bayrak: Malezya',
    '🇲🇿' => 'bayrak: Mozambik',
    '🇳🇦' => 'bayrak: Namibya',
    '🇳🇨' => 'bayrak: Yeni Kaledonya',
    '🇳🇪' => 'bayrak: Nijer',
    '🇳🇫' => 'bayrak: Norfolk Adası',
    '🇳🇬' => 'bayrak: Nijerya',
    '🇳🇮' => 'bayrak: Nikaragua',
    '🇳🇱' => 'bayrak: Hollanda',
    '🇳🇴' => 'bayrak: Norveç',
    '🇳🇵' => 'bayrak: Nepal',
    '🇳🇷' => 'bayrak: Nauru',
    '🇳🇺' => 'bayrak: Niue',
    '🇳🇿' => 'bayrak: Yeni Zelanda',
    '🇴🇲' => 'bayrak: Umman',
    '🇵🇦' => 'bayrak: Panama',
    '🇵🇪' => 'bayrak: Peru',
    '🇵🇫' => 'bayrak: Fransız Polinezyası',
    '🇵🇬' => 'bayrak: Papua Yeni Gine',
    '🇵🇭' => 'bayrak: Filipinler',
    '🇵🇰' => 'bayrak: Pakistan',
    '🇵🇱' => 'bayrak: Polonya',
    '🇵🇲' => 'bayrak: Saint Pierre ve Miquelon',
    '🇵🇳' => 'bayrak: Pitcairn Adaları',
    '🇵🇷' => 'bayrak: Porto Riko',
    '🇵🇸' => 'bayrak: Filistin Bölgeleri',
    '🇵🇹' => 'bayrak: Portekiz',
    '🇵🇼' => 'bayrak: Palau',
    '🇵🇾' => 'bayrak: Paraguay',
    '🇶🇦' => 'bayrak: Katar',
    '🇷🇪' => 'bayrak: Reunion',
    '🇷🇴' => 'bayrak: Romanya',
    '🇷🇸' => 'bayrak: Sırbistan',
    '🇷🇺' => 'bayrak: Rusya',
    '🇷🇼' => 'bayrak: Ruanda',
    '🇸🇦' => 'bayrak: Suudi Arabistan',
    '🇸🇧' => 'bayrak: Solomon Adaları',
    '🇸🇨' => 'bayrak: Seyşeller',
    '🇸🇩' => 'bayrak: Sudan',
    '🇸🇪' => 'bayrak: İsveç',
    '🇸🇬' => 'bayrak: Singapur',
    '🇸🇭' => 'bayrak: Saint Helena',
    '🇸🇮' => 'bayrak: Slovenya',
    '🇸🇯' => 'bayrak: Svalbard ve Jan Mayen',
    '🇸🇰' => 'bayrak: Slovakya',
    '🇸🇱' => 'bayrak: Sierra Leone',
    '🇸🇲' => 'bayrak: San Marino',
    '🇸🇳' => 'bayrak: Senegal',
    '🇸🇴' => 'bayrak: Somali',
    '🇸🇷' => 'bayrak: Surinam',
    '🇸🇸' => 'bayrak: Güney Sudan',
    '🇸🇹' => 'bayrak: Sao Tome ve Principe',
    '🇸🇻' => 'bayrak: El Salvador',
    '🇸🇽' => 'bayrak: Sint Maarten',
    '🇸🇾' => 'bayrak: Suriye',
    '🇸🇿' => 'bayrak: Esvatini',
    '🇹🇦' => 'bayrak: Tristan da Cunha',
    '🇹🇨' => 'bayrak: Turks ve Caicos Adaları',
    '🇹🇩' => 'bayrak: Çad',
    '🇹🇫' => 'bayrak: Fransız Güney Toprakları',
    '🇹🇬' => 'bayrak: Togo',
    '🇹🇭' => 'bayrak: Tayland',
    '🇹🇯' => 'bayrak: Tacikistan',
    '🇹🇰' => 'bayrak: Tokelau',
    '🇹🇱' => 'bayrak: Timor-Leste',
    '🇹🇲' => 'bayrak: Türkmenistan',
    '🇹🇳' => 'bayrak: Tunus',
    '🇹🇴' => 'bayrak: Tonga',
    '🇹🇷' => 'bayrak: Türkiye',
    '🇹🇹' => 'bayrak: Trinidad ve Tobago',
    '🇹🇻' => 'bayrak: Tuvalu',
    '🇹🇼' => 'bayrak: Tayvan',
    '🇹🇿' => 'bayrak: Tanzanya',
    '🇺🇦' => 'bayrak: Ukrayna',
    '🇺🇬' => 'bayrak: Uganda',
    '🇺🇲' => 'bayrak: ABD Küçük Harici Adaları',
    '🇺🇳' => 'bayrak: Birleşmiş Milletler',
    '🇺🇸' => 'bayrak: Amerika Birleşik Devletleri',
    '🇺🇾' => 'bayrak: Uruguay',
    '🇺🇿' => 'bayrak: Özbekistan',
    '🇻🇦' => 'bayrak: Vatikan',
    '🇻🇨' => 'bayrak: Saint Vincent ve Grenadinler',
    '🇻🇪' => 'bayrak: Venezuela',
    '🇻🇬' => 'bayrak: Britanya Virjin Adaları',
    '🇻🇮' => 'bayrak: ABD Virjin Adaları',
    '🇻🇳' => 'bayrak: Vietnam',
    '🇻🇺' => 'bayrak: Vanuatu',
    '🇼🇫' => 'bayrak: Wallis ve Futuna',
    '🇼🇸' => 'bayrak: Samoa',
    '🇽🇰' => 'bayrak: Kosova',
    '🇾🇪' => 'bayrak: Yemen',
    '🇾🇹' => 'bayrak: Mayotte',
    '🇿🇦' => 'bayrak: Güney Afrika',
    '🇿🇲' => 'bayrak: Zambiya',
    '🇿🇼' => 'bayrak: Zimbabve',
    '🔟' => 'tuş: 10',
    '🏻' => 'açık cilt tonu',
    '🏼' => 'orta açık cilt tonu',
    '🏽' => 'orta cilt tonu',
    '🏾' => 'orta koyu cilt tonu',
    '🏿' => 'koyu cilt tonu',
    '😀' => 'sırıtan yüz',
    '😃' => 'ağzı açık sırıtma',
    '😄' => 'gülen gözlerle ağzı açık sırıtma',
    '😁' => 'gülen gözlerle sırıtma',
    '😆' => 'kapalı gözlerle ağzı açık sırıtma',
    '😅' => 'soğuk terli ağzı açık gülümseme',
    '🤣' => 'gülmekten yerlere yatma',
    '😂' => 'sevinç gözyaşları',
    '🙂' => 'hafifçe gülümseyen yüz',
    '🙃' => 'ters yüz',
    '🫠' => 'eriyen yüz',
    '😉' => 'göz kırpan yüz',
    '😊' => 'ağzı açık gülme',
    '😇' => 'ışık halkalı gülümseme',
    '🥰' => '3 kalpli gülümseyen yüz',
    '😍' => 'kalp gözlü gülümseme',
    '🤩' => 'yıldız çarpmış',
    '😘' => 'öpücük gönderen yüz',
    '😗' => 'öpen yüz',
    '☺' => 'gülen yüz',
    '😚' => 'kapalı gözlerle öpen yüz',
    '😙' => 'gülen gözlerle öpen yüz',
    '🥲' => 'gözü yaşlı gülümseyen yüz',
    '😋' => 'lezzetli yiyecek beğenen yüz',
    '😛' => 'dil dışarıda',
    '😜' => 'dili dışarda göz kırpma',
    '🤪' => 'çılgın surat',
    '😝' => 'dil dışarıda ve gözler kapalı',
    '🤑' => 'para ağızlı yüz',
    '🤗' => 'kucaklayarak gülümseme',
    '🤭' => 'eliyle ağzını kapatan yüz',
    '🫢' => 'gözleri açık ve eliyle ağzını kapatan yüz',
    '🫣' => 'gizlice bakan yüz',
    '🤫' => 'sus işareti yapan yüz',
    '🤔' => 'düşünen yüz',
    '🫡' => 'selamlayan yüz',
    '🤐' => 'ağzı fermuarlı yüz',
    '🤨' => 'tek kaşı çatık yüz',
    '😐' => 'duygusuz yüz',
    '😑' => 'ifadesiz yüz',
    '😶' => 'ağzı olmayan yüz',
    '🫥' => 'noktalı çizgili yüz',
    '😏' => 'imalı gülümseme',
    '😒' => 'keyifsiz yüz',
    '🙄' => 'gözlerini deviren yüz',
    '😬' => 'dişlerini gösteren yüz',
    '🤥' => 'yalan söyleyen yüz',
    '🫨' => 'sallanan yüz',
    '😌' => 'rahatlamış yüz',
    '😔' => 'düşünceli yüz',
    '😪' => 'uykulu yüz',
    '🤤' => 'salya akıtan yüz',
    '😴' => 'uyuyan yüz',
    '🫩' => 'gözlerinin altı torbalı yüz',
    '😷' => 'maskeli yüz',
    '🤒' => 'ağzında termometre olan yüz',
    '🤕' => 'kafası sarılı yüz',
    '🤢' => 'kusmak üzere olan yüz',
    '🤮' => 'kusan yüz',
    '🤧' => 'hapşıran yüz',
    '🥵' => 'kızarmış yüz',
    '🥶' => 'üşümüş yüz',
    '🥴' => 'sarhoş yüz',
    '😵' => 'sersemlemiş yüz',
    '🤯' => 'patlayan kafa',
    '🤠' => 'kovboy şapkalı yüz',
    '🥳' => 'partide eğlenen yüz',
    '🥸' => 'gizlenmiş yüz',
    '😎' => 'güneş gözlüklü gülümseme',
    '🤓' => 'ukala yüz',
    '🧐' => 'tek cam gözlüklü yüz',
    '😕' => 'kafası karışık yüz',
    '🫤' => 'yamuk ağızlı yüz',
    '😟' => 'endişeli yüz',
    '🙁' => 'biraz asık surat',
    '☹' => 'asık surat',
    '😮' => 'ağzı açık yüz',
    '😯' => 'şaşkın yüz',
    '😲' => 'afallamış yüz',
    '😳' => 'kızarmış ve şaşkın yüz',
    '🫪' => 'biçimsiz yüz',
    '🥺' => 'yalvaran yüz',
    '🥹' => 'gözleri dolmuş yüz',
    '😦' => 'hoşnutsuz yüz',
    '😧' => 'kederli yüz',
    '😨' => 'korkulu yüz',
    '😰' => 'terli ve endişeli yüz',
    '😥' => 'üzgün ama rahatlamış yüz',
    '😢' => 'ağlayan yüz',
    '😭' => 'avaz avaz ağlayan yüz',
    '😱' => 'korkuyla çığlık atan yüz',
    '😖' => 'kafası çok karışık yüz',
    '😣' => 'kararlı yüz',
    '😞' => 'memnuniyetsiz yüz',
    '😓' => 'soğuk ter',
    '😩' => 'çok yorgun',
    '😫' => 'yorgun yüz',
    '🥱' => 'esneyen yüz',
    '😤' => 'burnundan soluyan yüz',
    '😡' => 'somurtkan yüz',
    '😠' => 'öfkeli yüz',
    '🤬' => 'ağzında semboller olan yüz',
    '😈' => 'boynuzlu gülen yüz',
    '👿' => 'boynuzlu kızgın yüz',
    '💀' => 'kafatası',
    '☠' => 'kuru kafa',
    '💩' => 'kaka',
    '🤡' => 'palyaço yüzü',
    '👹' => 'dev',
    '👺' => 'goblin',
    '👻' => 'hayalet',
    '👽' => 'uzaylı',
    '👾' => 'uzaylı canavar',
    '🤖' => 'robot',
    '😺' => 'sırıtan kedi',
    '😸' => 'gülen gözlerle sırıtan kedi',
    '😹' => 'sevinç gözyaşları döken kedi',
    '😻' => 'kalp gözlü gülümseyen kedi',
    '😼' => 'iğneleyici kedi yüzü',
    '😽' => 'öpen kedi',
    '🙀' => 'korkuyla bağıran kedi',
    '😿' => 'ağlayan kedi',
    '😾' => 'somurtan kedi',
    '🙈' => 'görmedim',
    '🙉' => 'duymadım',
    '🙊' => 'bilmiyorum',
    '💌' => 'aşk mektubu',
    '💘' => 'ok saplanmış kalp',
    '💝' => 'kurdeleli kalp',
    '💖' => 'pırıldayan kalp',
    '💗' => 'büyüyen kalp',
    '💓' => 'çarpan kalp',
    '💞' => 'dönen kalpler',
    '💕' => 'iki kalp',
    '💟' => 'kalpli dekorasyon',
    '❣' => 'kalp şeklinde ünlem işareti',
    '💔' => 'kırık kalp',
    '❤' => 'kırmızı kalp',
    '🩷' => 'pembe kalp',
    '🧡' => 'turuncu kalp',
    '💛' => 'sarı kalp',
    '💚' => 'yeşil kalp',
    '💙' => 'mavi kalp',
    '🩵' => 'açık mavi kalp',
    '💜' => 'mor kalp',
    '🤎' => 'kahverengi kalp',
    '🖤' => 'siyah kalp',
    '🩶' => 'gri kalp',
    '🤍' => 'beyaz kalp',
    '💋' => 'öpücük izi',
    '💯' => 'yüz puan',
    '💢' => 'öfke simgesi',
    '🫯' => 'dövüş bulutu',
    '💥' => 'çarpışma',
    '💫' => 'baş dönmesi',
    '💦' => 'ter damlası',
    '💨' => 'hızla fırlama',
    '🕳' => 'delik',
    '💬' => 'konuşma balonu',
    '🗨' => 'sol konuşma balonu',
    '🗯' => 'sağ öfke balonu',
    '💭' => 'düşünce balonu',
    '💤' => 'zzz',
    '👋' => 'el sallama',
    '🤚' => 'elinin tersini kaldırma',
    '🖐' => 'parmaklar açık el kaldırma',
    '✋' => 'el kaldırma',
    '🖖' => 'Vulkan selamı',
    '🫱' => 'sağa bakan el',
    '🫲' => 'sola bakan el',
    '🫳' => 'avuç içi aşağı bakan el',
    '🫴' => 'avuç içi yukarı bakan el',
    '🫷' => 'sola iten el',
    '🫸' => 'sağa iten el',
    '👌' => 'tamam işareti',
    '🤌' => 'kıstırma hareketi yapan parmaklar',
    '🤏' => 'kıstırma hareketi yapan el',
    '✌' => 'zafer işareti',
    '🤞' => 'şans dileme',
    '🫰' => 'işaret parmağı ve baş parmak birbirine geçmiş el',
    '🤟' => 'seni seviyorum işareti',
    '🤘' => 'boynuz işareti',
    '🤙' => 'beni ara işareti',
    '👈' => 'elinin tersiyle solu gösteren işaret parmağı',
    '👉' => 'elinin tersiyle sağı gösteren işaret parmağı',
    '👆' => 'elinin tersiyle yukarıyı gösteren işaret parmağı',
    '🖕' => 'orta parmak',
    '👇' => 'elinin tersiyle aşağıyı gösteren işaret parmağı',
    '☝' => 'yukarıyı gösteren işaret parmağı',
    '🫵' => 'karşıyı işaret eden işaret parmağı',
    '👍' => 'baş parmak yukarıda',
    '👎' => 'baş parmak aşağıda',
    '✊' => 'yumruk kaldırma',
    '👊' => 'yaklaşan yumruk',
    '🤛' => 'sola bakan yumruk',
    '🤜' => 'sağa bakan yumruk',
    '👏' => 'el çırpma',
    '🙌' => 'eller havaya',
    '🫶' => 'kalp oluşturan eller',
    '👐' => 'eller açık',
    '🤲' => 'iki avuç açık',
    '🤝' => 'el sıkışma',
    '🙏' => 'birleşen eller',
    '✍' => 'yazı yazan el',
    '💅' => 'oje sürme',
    '🤳' => 'selfie',
    '💪' => 'pazı',
    '🦾' => 'mekanik kol',
    '🦿' => 'mekanik bacak',
    '🦵' => 'bacak',
    '🦶' => 'ayak',
    '👂' => 'kulak',
    '🦻' => 'işitme cihazı olan kulak',
    '👃' => 'burun',
    '🧠' => 'beyin',
    '🫀' => 'kalp',
    '🫁' => 'akciğer',
    '🦷' => 'diş',
    '🦴' => 'kemik',
    '👀' => 'gözler',
    '👁' => 'göz',
    '👅' => 'dil',
    '👄' => 'ağız',
    '🫦' => 'ısırılan dudak',
    '👶' => 'bebek',
    '🧒' => 'çocuk',
    '👦' => 'erkek çocuk',
    '👧' => 'kız çocuk',
    '🧑' => 'yetişkin',
    '👱' => 'sarışın kişi',
    '👨' => 'erkek',
    '🧔' => 'sakallı kişi',
    '👩' => 'kadın',
    '🧓' => 'yaşlı yetişkin',
    '👴' => 'yaşlı erkek',
    '👵' => 'yaşlı kadın',
    '🙍' => 'somurtma',
    '🙎' => 'darılma',
    '🙅' => 'hayır hareketi',
    '🙆' => 'tamam hareketi',
    '💁' => 'elini yana yatırma',
    '🙋' => 'elini kaldırma',
    '🧏' => 'işitme engelli kişi',
    '🙇' => 'başını öne eğme',
    '🤦' => 'yüzünü eliyle kapama',
    '🤷' => 'omuz silkme',
    '👮' => 'polis memuru',
    '🕵' => 'dedektif',
    '💂' => 'muhafız',
    '🥷' => 'ninja',
    '👷' => 'inşaat işçisi',
    '🫅' => 'taç takmış insan',
    '🤴' => 'prens',
    '👸' => 'prenses',
    '👳' => 'sarıklı kişi',
    '👲' => 'çin şapkalı erkek',
    '🧕' => 'başörtülü kadın',
    '🤵' => 'smokinli kişi',
    '👰' => 'duvaklı gelin',
    '🤰' => 'hamile kadın',
    '🫃' => 'hamile adam',
    '🫄' => 'hamile kişi',
    '🤱' => 'emziren kadın',
    '👼' => 'bebek melek',
    '🎅' => 'Noel Baba',
    '🤶' => 'Noel Anne',
    '🦸' => 'süper kahraman',
    '🦹' => 'süper kötü',
    '🧙' => 'büyücü',
    '🧚' => 'peri',
    '🧛' => 'vampir',
    '🧜' => 'balık insan',
    '🧝' => 'elf',
    '🧞' => 'cin',
    '🧟' => 'zombi',
    '🧌' => 'trol',
    '🫈' => 'tüylü yaratık',
    '💆' => 'yüz masajı',
    '💇' => 'saç kesimi',
    '🚶' => 'yürüyen kişi',
    '🧍' => 'ayakta duran kişi',
    '🧎' => 'diz çöken kişi',
    '🏃' => 'koşan kişi',
    '💃' => 'dans eden kadın',
    '🕺' => 'dans eden adam',
    '🕴' => 'havalanan takım elbiseli adam',
    '👯' => 'tavşan kulaklı kişiler',
    '🧖' => 'buhar odasındaki kişi',
    '🧗' => 'dağcı kişi',
    '🤺' => 'eskrimci',
    '🏇' => 'at yarışı',
    '⛷' => 'kayakçı',
    '🏂' => 'snowbordçu',
    '🏌' => 'golf oynayan kişi',
    '🏄' => 'sörf yapan kişi',
    '🚣' => 'sandalda kürek çeken kişi',
    '🏊' => 'yüzen kişi',
    '⛹' => 'top sektiren kişi',
    '🏋' => 'ağırlık kaldıran kişi',
    '🚴' => 'bisiklet süren kişi',
    '🚵' => 'dağ bisikletçisi',
    '🤸' => 'perende atan kişi',
    '🤼' => 'güreşen kişiler',
    '🤽' => 'su topu',
    '🤾' => 'hentbol',
    '🤹' => 'jonglör',
    '🧘' => 'lotus pozisyonundaki kişi',
    '🛀' => 'banyo yapan kişi',
    '🛌' => 'yatakta yatan kişi',
    '👭' => 'el ele tutuşan kadınlar',
    '👫' => 'el ele tutuşan erkek ve kadın',
    '👬' => 'el ele tutuşan erkekler',
    '💏' => 'öpücük',
    '💑' => 'kalpli çift',
    '🗣' => 'konuşan kişi',
    '👤' => 'siluet büst',
    '👥' => 'siluet büstler',
    '🫂' => 'kucaklaşan insanlar',
    '👪' => 'aile',
    '👣' => 'ayak izi',
    '🫆' => 'parmak izi',
    '🦰' => 'kızıl saç',
    '🦱' => 'kıvırcık saç',
    '🦳' => 'beyaz saç',
    '🦲' => 'kel',
    '🐵' => 'maymun yüzü',
    '🐒' => 'maymun',
    '🦍' => 'goril',
    '🦧' => 'orangutan',
    '🐶' => 'köpek yüzü',
    '🐕' => 'köpek',
    '🦮' => 'rehber köpek',
    '🐩' => 'kaniş',
    '🐺' => 'kurt',
    '🦊' => 'tilki',
    '🦝' => 'rakun',
    '🐱' => 'kedi yüzü',
    '🐈' => 'kedi',
    '🦁' => 'aslan',
    '🐯' => 'kaplan yüzü',
    '🐅' => 'kaplan',
    '🐆' => 'leopar',
    '🐴' => 'at yüzü',
    '🫎' => 'sığın',
    '🫏' => 'eşek',
    '🐎' => 'at',
    '🦄' => 'tek boynuzlu at',
    '🦓' => 'zebra',
    '🦌' => 'geyik',
    '🦬' => 'bizon',
    '🐮' => 'inek yüzü',
    '🐂' => 'öküz',
    '🐃' => 'manda',
    '🐄' => 'inek',
    '🐷' => 'domuz yüzü',
    '🐖' => 'domuz',
    '🐗' => 'yaban domuzu',
    '🐽' => 'domuz burnu',
    '🐏' => 'koç',
    '🐑' => 'koyun',
    '🐐' => 'keçi',
    '🐪' => 'deve',
    '🐫' => 'iki hörgüçlü deve',
    '🦙' => 'lama',
    '🦒' => 'zürafa',
    '🐘' => 'fil',
    '🦣' => 'mamut',
    '🦏' => 'gergedan',
    '🦛' => 'hipopotam',
    '🐭' => 'fare yüzü',
    '🐁' => 'fare',
    '🐀' => 'sıçan',
    '🐹' => 'hamster',
    '🐰' => 'tavşan yüzü',
    '🐇' => 'tavşan',
    '🐿' => 'sincap',
    '🦫' => 'kunduz',
    '🦔' => 'kirpi',
    '🦇' => 'yarasa',
    '🐻' => 'ayı',
    '🐨' => 'koala',
    '🐼' => 'panda',
    '🦥' => 'tembel hayvan',
    '🦦' => 'su samuru',
    '🦨' => 'kokarca',
    '🦘' => 'kanguru',
    '🦡' => 'porsuk',
    '🐾' => 'pençe izi',
    '🦃' => 'hindi',
    '🐔' => 'tavuk',
    '🐓' => 'horoz',
    '🐣' => 'yumurtadan çıkan civciv',
    '🐤' => 'civciv',
    '🐥' => 'önden civciv',
    '🐦' => 'kuş',
    '🐧' => 'penguen',
    '🕊' => 'güvercin',
    '🦅' => 'kartal',
    '🦆' => 'ördek',
    '🦢' => 'kuğu',
    '🦉' => 'baykuş',
    '🦤' => 'dodo kuşu',
    '🪶' => 'kuş tüyü',
    '🦩' => 'flamingo',
    '🦚' => 'tavus kuşu',
    '🦜' => 'papağan',
    '🪽' => 'kanat',
    '🪿' => 'kaz',
    '🐸' => 'kurbağa',
    '🐊' => 'timsah',
    '🐢' => 'kaplumbağa',
    '🦎' => 'kertenkele',
    '🐍' => 'yılan',
    '🐲' => 'ejderha yüzü',
    '🐉' => 'ejderha',
    '🦕' => 'soropod',
    '🦖' => 'T-Rex',
    '🐳' => 'su püskürten balina',
    '🐋' => 'balina',
    '🐬' => 'yunus',
    '🫍' => 'orka',
    '🦭' => 'fok',
    '🐟' => 'balık',
    '🐠' => 'tropikal balık',
    '🐡' => 'balon balığı',
    '🦈' => 'köpek balığı',
    '🐙' => 'ahtapot',
    '🐚' => 'deniz kabuğu',
    '🪸' => 'mercan',
    '🪼' => 'denizanası',
    '🦀' => 'yengeç',
    '🦞' => 'ıstakoz',
    '🦐' => 'karides',
    '🦑' => 'mürekkep balığı',
    '🦪' => 'istiridye',
    '🐌' => 'salyangoz',
    '🦋' => 'kelebek',
    '🐛' => 'tırtıl',
    '🐜' => 'karınca',
    '🐝' => 'bal arısı',
    '🪲' => 'böcek',
    '🐞' => 'uğur böceği',
    '🦗' => 'cırcır böceği',
    '🪳' => 'hamam böceği',
    '🕷' => 'örümcek',
    '🕸' => 'örümcek ağı',
    '🦂' => 'akrep',
    '🦟' => 'sivrisinek',
    '🪰' => 'karasinek',
    '🪱' => 'solucan',
    '🦠' => 'mikrop',
    '💐' => 'buket',
    '🌸' => 'kiraz çiçeği',
    '💮' => 'beyaz çiçek',
    '🪷' => 'nilüfer',
    '🏵' => 'çiçek rozet',
    '🌹' => 'gül',
    '🥀' => 'solmuş çiçek',
    '🌺' => 'çingülü',
    '🌻' => 'ayçiçeği',
    '🌼' => 'çiçek açma',
    '🌷' => 'lale',
    '🪻' => 'sümbül',
    '🌱' => 'filizlenme',
    '🪴' => 'saksı bitkisi',
    '🌲' => 'yaprak dökmeyen ağaç',
    '🌳' => 'yaprak döken ağaç',
    '🌴' => 'palmiye ağacı',
    '🌵' => 'kaktüs',
    '🌾' => 'pirinç bitkisi',
    '🌿' => 'ot',
    '☘' => 'yonca',
    '🍀' => 'dört yapraklı yonca',
    '🍁' => 'akçaağaç yaprağı',
    '🍂' => 'düşen yapraklar',
    '🍃' => 'rüzgardaki yaprak',
    '🪹' => 'boş yuva',
    '🪺' => 'yumurta bulunan yuva',
    '🍄' => 'mantar',
    '🪾' => 'yapraksız ağaç',
    '🍇' => 'üzüm',
    '🍈' => 'kavun',
    '🍉' => 'karpuz',
    '🍊' => 'mandalina',
    '🍋' => 'limon',
    '🍌' => 'muz',
    '🍍' => 'ananas',
    '🥭' => 'mango',
    '🍎' => 'kırmızı elma',
    '🍏' => 'yeşil elma',
    '🍐' => 'armut',
    '🍑' => 'şeftali',
    '🍒' => 'kiraz',
    '🍓' => 'çilek',
    '🫐' => 'yaban mersini',
    '🥝' => 'kivi',
    '🍅' => 'domates',
    '🫒' => 'zeytin',
    '🥥' => 'Hindistan cevizi',
    '🥑' => 'avokado',
    '🍆' => 'patlıcan',
    '🥔' => 'patates',
    '🥕' => 'havuç',
    '🌽' => 'mısır koçanı',
    '🌶' => 'acı biber',
    '🫑' => 'dolma biber',
    '🥒' => 'salatalık',
    '🥬' => 'yeşil yapraklı sebze',
    '🥦' => 'brokoli',
    '🧄' => 'sarımsak',
    '🧅' => 'soğan',
    '🥜' => 'yer fıstığı',
    '🫘' => 'fasulye',
    '🌰' => 'kestane',
    '🫚' => 'kök zencefil',
    '🫛' => 'kabuklu bezelye',
    '🫜' => 'kök sebze',
    '🍞' => 'ekmek',
    '🥐' => 'kruvasan',
    '🥖' => 'baget ekmek',
    '🫓' => 'yassı ekmek',
    '🥨' => 'pretzel',
    '🥯' => 'simit',
    '🥞' => 'krep',
    '🧇' => 'waffle',
    '🧀' => 'peynir dilimi',
    '🍖' => 'kemikli et',
    '🍗' => 'tavuk butu',
    '🥩' => 'parça et',
    '🥓' => 'domuz pastırması',
    '🍔' => 'hamburger',
    '🍟' => 'patates kızartması',
    '🍕' => 'pizza dilimi',
    '🌭' => 'sosisli sandviç',
    '🥪' => 'sandviç',
    '🌮' => 'tako',
    '🌯' => 'burito',
    '🫔' => 'tamale',
    '🥙' => 'dürüm',
    '🧆' => 'falafel',
    '🥚' => 'yumurta',
    '🍳' => 'yemek pişirme',
    '🥘' => 'sahanda yemek',
    '🍲' => 'yemek tenceresi',
    '🫕' => 'fondü',
    '🥣' => 'kaşıklı kase',
    '🥗' => 'yeşil salata',
    '🍿' => 'patlamış mısır',
    '🧈' => 'tereyağı',
    '🧂' => 'tuz',
    '🥫' => 'konserve gıda',
    '🍱' => 'bento',
    '🍘' => 'pirinç gevreği',
    '🍙' => 'pirinç topu',
    '🍚' => 'pilav',
    '🍛' => 'körili pilav',
    '🍜' => 'erişte',
    '🍝' => 'spagetti',
    '🍠' => 'fırınlanmış patates',
    '🍢' => 'oden',
    '🍣' => 'suşi',
    '🍤' => 'kızarmış karides',
    '🍥' => 'balık köftesi',
    '🥮' => 'ay pastası',
    '🍡' => 'pirinç tatlısı',
    '🥟' => 'mantı',
    '🥠' => 'fal kurabiyesi',
    '🥡' => 'yiyecek paketi',
    '🍦' => 'yumuşak dondurma',
    '🍧' => 'meyveli buz',
    '🍨' => 'dondurma',
    '🍩' => 'donut',
    '🍪' => 'kurabiye',
    '🎂' => 'doğum günü pastası',
    '🍰' => 'pasta dilimi',
    '🧁' => 'cupcake',
    '🥧' => 'tart',
    '🍫' => 'çikolata',
    '🍬' => 'şekerleme',
    '🍭' => 'lolipop',
    '🍮' => 'krem karamel',
    '🍯' => 'bal kabı',
    '🍼' => 'biberon',
    '🥛' => 'süt bardağı',
    '☕' => 'sıcak içecek',
    '🫖' => 'demlik',
    '🍵' => 'çay fincanı',
    '🍶' => 'sake',
    '🍾' => 'patlayan mantarlı şişe',
    '🍷' => 'şarap kadehi',
    '🍸' => 'kokteyl kadehi',
    '🍹' => 'tropik içecek',
    '🍺' => 'bira kupası',
    '🍻' => 'bira kupaları tokuşturma',
    '🥂' => 'kadeh tokuşturma',
    '🥃' => 'kısa ve geniş bardak',
    '🫗' => 'dökülen sıvı',
    '🥤' => 'pipetli bardak',
    '🧋' => 'bubble tea',
    '🧃' => 'içecek kutusu',
    '🧉' => 'mate çayı',
    '🧊' => 'buz küpü',
    '🥢' => 'yemek çubukları',
    '🍽' => 'tabakla birlikte çatal ve bıçak',
    '🍴' => 'çatal ve bıçak',
    '🥄' => 'kaşık',
    '🔪' => 'mutfak bıçağı',
    '🫙' => 'kavanoz',
    '🏺' => 'amfora',
    '🌍' => 'yerkürede Avrupa-Afrika',
    '🌎' => 'yerkürede Amerika',
    '🌏' => 'yerkürede Asya-Avustralya',
    '🌐' => 'meridyenli yerküre',
    '🗺' => 'dünya haritası',
    '🗾' => 'Japonya haritası',
    '🧭' => 'pusula',
    '🏔' => 'karla kaplı dağ',
    '⛰' => 'dağ',
    '🛘' => 'heyelan',
    '🌋' => 'yanardağ',
    '🗻' => 'Fuji Dağı',
    '🏕' => 'kamp',
    '🏖' => 'plaj ve şemsiye',
    '🏜' => 'çöl',
    '🏝' => 'ıssız ada',
    '🏞' => 'milli park',
    '🏟' => 'stadyum',
    '🏛' => 'tarihi yapı',
    '🏗' => 'bina inşaatı',
    '🧱' => 'tuğla',
    '🪨' => 'kaya',
    '🪵' => 'kütük',
    '🛖' => 'kulübe',
    '🏘' => 'evler',
    '🏚' => 'terkedilmiş ev',
    '🏠' => 'ev',
    '🏡' => 'bahçeli ev',
    '🏢' => 'ofis binası',
    '🏣' => 'Japon postanesi',
    '🏤' => 'postane',
    '🏥' => 'hastane',
    '🏦' => 'banka',
    '🏨' => 'otel',
    '🏩' => 'aşk oteli',
    '🏪' => 'sürekli açık market',
    '🏫' => 'okul',
    '🏬' => 'çok katlı mağaza',
    '🏭' => 'fabrika',
    '🏯' => 'Japon şatosu',
    '🏰' => 'şato',
    '💒' => 'düğün',
    '🗼' => 'Tokyo kulesi',
    '🗽' => 'Özgürlük Heykeli',
    '⛪' => 'kilise',
    '🕌' => 'cami',
    '🛕' => 'Hindu tapınağı',
    '🕍' => 'sinagog',
    '⛩' => 'Şinto tapınağı',
    '🕋' => 'Kâbe',
    '⛲' => 'fıskiye',
    '⛺' => 'çadır',
    '🌁' => 'sisli',
    '🌃' => 'yıldızlı gece',
    '🏙' => 'şehir manzarası',
    '🌄' => 'dağların arasından güneşin doğuşu',
    '🌅' => 'güneşin doğuşu',
    '🌆' => 'şehirde alacakaranlık',
    '🌇' => 'gün batımı',
    '🌉' => 'gece vakti köprü',
    '♨' => 'kaplıca',
    '🎠' => 'atlıkarınca',
    '🛝' => 'kaydırak',
    '🎡' => 'dönme dolap',
    '🎢' => 'hız treni',
    '💈' => 'berber dükkanı',
    '🎪' => 'sirk çadırı',
    '🚂' => 'buharlı lokomotif',
    '🚃' => 'vagon',
    '🚄' => 'yüksek hızlı tren',
    '🚅' => 'yuvarlak uçlu yüksek hızlı tren',
    '🚆' => 'tren',
    '🚇' => 'metro',
    '🚈' => 'hafif raylı',
    '🚉' => 'tren istasyonu',
    '🚊' => 'tramvay',
    '🚝' => 'monoray',
    '🚞' => 'dağ demiryolu',
    '🚋' => 'tramvay vagonu',
    '🚌' => 'otobüs',
    '🚍' => 'yaklaşan otobüs',
    '🚎' => 'troleybüs',
    '🚐' => 'minibüs',
    '🚑' => 'ambulans',
    '🚒' => 'itfaiye arabası',
    '🚓' => 'polis arabası',
    '🚔' => 'yaklaşan polis arabası',
    '🚕' => 'taksi',
    '🚖' => 'yaklaşan taksi',
    '🚗' => 'araba',
    '🚘' => 'yaklaşan araba',
    '🚙' => 'cip',
    '🛻' => 'pikap',
    '🚚' => 'nakliye kamyonu',
    '🚛' => 'tır',
    '🚜' => 'traktör',
    '🏎' => 'yarış arabası',
    '🏍' => 'motosiklet',
    '🛵' => 'motorlu skuter',
    '🦽' => 'tekerlekli sandalye',
    '🦼' => 'elektrikli tekerlekli sandalye',
    '🛺' => 'elektrikli çekçek',
    '🚲' => 'bisiklet',
    '🛴' => 'ayakla sürülen skuter',
    '🛹' => 'kaykay',
    '🛼' => 'paten',
    '🚏' => 'otobüs durağı',
    '🛣' => 'otoban',
    '🛤' => 'tren yolu',
    '🛢' => 'petrol varili',
    '⛽' => 'akaryakıt istasyonu',
    '🛞' => 'tekerlek',
    '🚨' => 'polis lambası',
    '🚥' => 'yatay trafik ışıkları',
    '🚦' => 'dikey trafik ışıkları',
    '🛑' => 'dur işareti',
    '🚧' => 'inşaat',
    '⚓' => 'çıpa',
    '🛟' => 'cankurtaran simidi',
    '⛵' => 'yelkenli tekne',
    '🛶' => 'kano',
    '🚤' => 'sürat teknesi',
    '🛳' => 'yolcu gemisi',
    '⛴' => 'feribot',
    '🛥' => 'motorlu tekne',
    '🚢' => 'gemi',
    '✈' => 'uçak',
    '🛩' => 'küçük uçak',
    '🛫' => 'kalkan uçak',
    '🛬' => 'inen uçak',
    '🪂' => 'paraşüt',
    '💺' => 'koltuk',
    '🚁' => 'helikopter',
    '🚟' => 'asma demiryolu',
    '🚠' => 'dağ teleferik hattı',
    '🚡' => 'teleferik',
    '🛰' => 'uydu',
    '🚀' => 'roket',
    '🛸' => 'uçan daire',
    '🛎' => 'resepsiyon zili',
    '🧳' => 'bavul',
    '⌛' => 'kum saati',
    '⏳' => 'akan kum saati',
    '⌚' => 'kol saati',
    '⏰' => 'çalar saat',
    '⏱' => 'kronometre',
    '⏲' => 'alarm',
    '🕰' => 'raf saati',
    '🕛' => 'saat 12',
    '🕧' => 'on iki buçuk',
    '🕐' => 'saat 1',
    '🕜' => 'bir buçuk',
    '🕑' => 'saat 2',
    '🕝' => 'iki buçuk',
    '🕒' => 'saat 3',
    '🕞' => 'üç buçuk',
    '🕓' => 'saat 4',
    '🕟' => 'dört buçuk',
    '🕔' => 'saat 5',
    '🕠' => 'beş buçuk',
    '🕕' => 'saat 6',
    '🕡' => 'altı buçuk',
    '🕖' => 'saat 7',
    '🕢' => 'yedi buçuk',
    '🕗' => 'saat 8',
    '🕣' => 'sekiz buçuk',
    '🕘' => 'saat 9',
    '🕤' => 'dokuz buçuk',
    '🕙' => 'saat 10',
    '🕥' => 'on buçuk',
    '🕚' => 'saat 11',
    '🕦' => 'on bir buçuk',
    '🌑' => 'yeni ay',
    '🌒' => 'büyüyen ay',
    '🌓' => 'ilk dördün',
    '🌔' => 'geniş ay',
    '🌕' => 'dolunay',
    '🌖' => 'küçülen ay',
    '🌗' => 'son dördün',
    '🌘' => 'ince ay',
    '🌙' => 'hilal',
    '🌚' => 'yeni ay yüzü',
    '🌛' => 'ilk dördün ay yüzü',
    '🌜' => 'son dördün ay yüzü',
    '🌡' => 'termometre',
    '☀' => 'güneş',
    '🌝' => 'dolunay yüzü',
    '🌞' => 'güneş yüz',
    '🪐' => 'halkalı gezegen',
    '⭐' => 'yıldız',
    '🌟' => 'parıldayan yıldız',
    '🌠' => 'yıldız kayması',
    '🌌' => 'samanyolu',
    '☁' => 'bulut',
    '⛅' => 'bulutlar ardındaki güneş',
    '⛈' => 'şimşek ve yağmurlu bulut',
    '🌤' => 'bulutun arkasındaki güneş',
    '🌥' => 'bulutta kaybolan güneş',
    '🌦' => 'yağmur bulutunun arkasındaki güneş',
    '🌧' => 'yağmur bulutu',
    '🌨' => 'kar bulutu',
    '🌩' => 'şimşek bulutu',
    '🌪' => 'hortum',
    '🌫' => 'sis',
    '🌬' => 'rüzgar',
    '🌀' => 'siklon',
    '🌈' => 'gökkuşağı',
    '🌂' => 'kapalı şemsiye',
    '☂' => 'şemsiye',
    '☔' => 'yağmur damlalı şemsiye',
    '⛱' => 'sabit şemsiye',
    '⚡' => 'yüksek voltaj',
    '❄' => 'kar tanesi',
    '☃' => 'kardan adam',
    '⛄' => 'kar yağmadan kardan adam',
    '☄' => 'kuyruklu yıldız',
    '🔥' => 'ateş',
    '💧' => 'damlacık',
    '🌊' => 'dalga',
    '🎃' => 'kabak fener',
    '🎄' => 'Noel ağacı',
    '🎆' => 'havai fişek',
    '🎇' => 'fişek',
    '🧨' => 'maytap',
    '✨' => 'parıltılar',
    '🎈' => 'balon',
    '🎉' => 'parti konfetisi',
    '🎊' => 'konfeti',
    '🎋' => 'tanabata ağacı',
    '🎍' => 'çam süsleme',
    '🎎' => 'Japon oyuncak bebekleri',
    '🎏' => 'sazan şekilli flama',
    '🎐' => 'rüzgar çanı',
    '🎑' => 'ay izleme',
    '🧧' => 'kırmızı zarf',
    '🎀' => 'kurdele',
    '🎁' => 'paketlenmiş hediye',
    '🎗' => 'anma kurdelesi',
    '🎟' => 'giriş bileti',
    '🎫' => 'bilet',
    '🎖' => 'askeri madalya',
    '🏆' => 'zafer kupası',
    '🏅' => 'spor madalyası',
    '🥇' => 'birincilik madalyası',
    '🥈' => 'ikincilik madalyası',
    '🥉' => 'üçüncülük madalyası',
    '⚽' => 'futbol topu',
    '⚾' => 'beyzbol',
    '🥎' => 'softball',
    '🏀' => 'basketbol',
    '🏐' => 'voleybol',
    '🏈' => 'amerikan futbolu',
    '🏉' => 'ragbi',
    '🎾' => 'tenis',
    '🥏' => 'frizbi',
    '🎳' => 'bovling',
    '🏏' => 'kriket',
    '🏑' => 'çim hokeyi',
    '🏒' => 'buz hokeyi',
    '🥍' => 'lakros',
    '🏓' => 'masa tenisi',
    '🏸' => 'badminton',
    '🥊' => 'boks eldiveni',
    '🥋' => 'dövüş sporu forması',
    '🥅' => 'kale ağı',
    '⛳' => 'delikteki bayrak',
    '⛸' => 'buz pateni',
    '🎣' => 'olta',
    '🤿' => 'dalgıç maskesi',
    '🎽' => 'koşu tişörtü',
    '🎿' => 'kayak',
    '🛷' => 'kızak',
    '🥌' => 'curling taşı',
    '🎯' => 'tam isabet',
    '🪀' => 'yoyo',
    '🪁' => 'uçurtma',
    '🔫' => 'su tabancası',
    '🎱' => 'bilardo',
    '🔮' => 'kristal küre',
    '🪄' => 'sihirli asa',
    '🎮' => 'video oyunu',
    '🕹' => 'oyun çubuğu',
    '🎰' => 'kumar makinesi',
    '🎲' => 'oyun zarı',
    '🧩' => 'yapboz parçası',
    '🧸' => 'oyuncak ayı',
    '🪅' => 'pinyata',
    '🪩' => 'aynalı küre',
    '🪆' => 'matruşka',
    '♠' => 'maça',
    '♥' => 'kupa',
    '♦' => 'karo',
    '♣' => 'sinek',
    '♟' => 'piyon',
    '🃏' => 'joker',
    '🀄' => 'kızıl ejder mahjong taşı',
    '🎴' => 'çiçekli oyun kağıtları',
    '🎭' => 'sahne sanatları',
    '🖼' => 'resimli çerçeve',
    '🎨' => 'boya paleti',
    '🧵' => 'iplik',
    '🪡' => 'dikiş iğnesi',
    '🧶' => 'iplik yumağı',
    '🪢' => 'denizci düğümü',
    '👓' => 'gözlük',
    '🕶' => 'güneş gözlüğü',
    '🥽' => 'koruyucu gözlük',
    '🥼' => 'laboratuvar önlüğü',
    '🦺' => 'güvenlik yeleği',
    '👔' => 'kravat',
    '👕' => 'tişört',
    '👖' => 'kot',
    '🧣' => 'atkı',
    '🧤' => 'eldiven',
    '🧥' => 'kaban',
    '🧦' => 'çorap',
    '👗' => 'elbise',
    '👘' => 'kimono',
    '🥻' => 'sari',
    '🩱' => 'tek parça mayo',
    '🩲' => 'slip mayo',
    '🩳' => 'şort',
    '👙' => 'bikini',
    '👚' => 'kadın giysisi',
    '🪭' => 'yelpaze',
    '👛' => 'cüzdan',
    '👜' => 'el çantası',
    '👝' => 'para kesesi',
    '🛍' => 'alışveriş çantası',
    '🎒' => 'okul çantası',
    '🩴' => 'parmak arası terlik',
    '👞' => 'erkek ayakkabısı',
    '👟' => 'koşu ayakkabısı',
    '🥾' => 'yürüyüş botu',
    '🥿' => 'düz ayakkabı',
    '👠' => 'yüksek topuklu ayakkabı',
    '👡' => 'kadın sandaleti',
    '🩰' => 'bale ayakkabıları',
    '👢' => 'kadın botu',
    '🪮' => 'perma tarağı',
    '👑' => 'taç',
    '👒' => 'kadın şapkası',
    '🎩' => 'silindir şapka',
    '🎓' => 'mezuniyet kepi',
    '🧢' => 'siperli şapka',
    '🪖' => 'asker kaskı',
    '⛑' => 'yardım ekibi kaskı',
    '📿' => 'tespih',
    '💄' => 'ruj',
    '💍' => 'yüzük',
    '💎' => 'mücevher',
    '🔇' => 'ses kapalı',
    '🔈' => 'kısık ses',
    '🔉' => 'ses açık',
    '🔊' => 'yüksek ses',
    '📢' => 'hoparlör',
    '📣' => 'megafon',
    '📯' => 'posta klaksonu',
    '🔔' => 'zil',
    '🔕' => 'üzeri çizili zil',
    '🎼' => 'sol anahtarı',
    '🎵' => 'müzik notası',
    '🎶' => 'müzik notaları',
    '🎙' => 'stüdyo mikrofonu',
    '🎚' => 'ayarlama çubuğu',
    '🎛' => 'kontrol düğmeleri',
    '🎤' => 'mikrofon',
    '🎧' => 'kulaklık',
    '📻' => 'radyo',
    '🎷' => 'saksafon',
    '🎺' => 'trompet',
    '🪊' => 'trombon',
    '🪗' => 'akordiyon',
    '🎸' => 'gitar',
    '🎹' => 'müzik klavyesi',
    '🎻' => 'keman',
    '🪕' => 'banjo',
    '🥁' => 'davul',
    '🪘' => 'uzun davul',
    '🪇' => 'marakas',
    '🪈' => 'flüt',
    '🪉' => 'arp',
    '📱' => 'cep telefonu',
    '📲' => 'ok işaretli cep telefonu',
    '☎' => 'telefon',
    '📞' => 'ahize',
    '📟' => 'çağrı cihazı',
    '📠' => 'faks makinesi',
    '🔋' => 'pil',
    '🪫' => 'düşük pil',
    '🔌' => 'elektrik fişi',
    '💻' => 'dizüstü bilgisayar',
    '🖥' => 'masaüstü bilgisayar',
    '🖨' => 'yazıcı',
    '⌨' => 'klavye',
    '🖱' => 'bilgisayar faresi',
    '🖲' => 'iz topu',
    '💽' => 'mini disk',
    '💾' => 'disket',
    '💿' => 'optik disk',
    '📀' => 'dvd',
    '🧮' => 'abaküs',
    '🎥' => 'film kamerası',
    '🎞' => 'film kareleri',
    '📽' => 'film projektörü',
    '🎬' => 'film tahtası',
    '📺' => 'televizyon',
    '📷' => 'fotoğraf makinesi',
    '📸' => 'flaşlı fotoğraf makinesi',
    '📹' => 'video kamera',
    '📼' => 'videokaset',
    '🔍' => 'sola dönük büyüteç',
    '🔎' => 'sağa dönük büyüteç',
    '🕯' => 'mum',
    '💡' => 'ampul',
    '🔦' => 'el feneri',
    '🏮' => 'kırmızı kağıt fener',
    '🪔' => 'diya lamba',
    '📔' => 'süslü not defteri',
    '📕' => 'kapalı kitap',
    '📖' => 'açık kitap',
    '📗' => 'yeşil kitap',
    '📘' => 'mavi kitap',
    '📙' => 'turuncu kitap',
    '📚' => 'kitaplar',
    '📓' => 'not defteri',
    '📒' => 'resmi defter',
    '📃' => 'kıvrık sayfa',
    '📜' => 'parşömen',
    '📄' => 'yüzü yukarıda sayfa',
    '📰' => 'gazete',
    '🗞' => 'rulo gazete',
    '📑' => 'sayfa ayraçları',
    '🔖' => 'yer işareti',
    '🏷' => 'etiket',
    '🪙' => 'bozuk para',
    '💰' => 'para torbası',
    '🪎' => 'hazine sandığı',
    '💴' => 'yen banknot',
    '💵' => 'dolar banknot',
    '💶' => 'euro banknot',
    '💷' => 'sterlin banknot',
    '💸' => 'kanatlı para',
    '💳' => 'kredi kartı',
    '🧾' => 'makbuz',
    '💹' => 'yen simgeli artış grafiği',
    '✉' => 'zarf',
    '📧' => 'e-posta',
    '📨' => 'gelen zarf',
    '📩' => 'oklu zarf',
    '📤' => 'giden kutusu',
    '📥' => 'gelen kutusu',
    '📦' => 'paket',
    '📫' => 'işareti yukarıda kapalı posta kutusu',
    '📪' => 'işareti aşağıda kapalı posta kutusu',
    '📬' => 'işareti yukarıda açık posta kutusu',
    '📭' => 'işareti aşağıda açık posta kutusu',
    '📮' => 'posta kutusu',
    '🗳' => 'oy sandığı',
    '✏' => 'kurşun kalem',
    '✒' => 'siyah uç',
    '🖋' => 'dolma kalem',
    '🖊' => 'kalem',
    '🖌' => 'boya fırçası',
    '🖍' => 'pastel boya',
    '📝' => 'not',
    '💼' => 'evrak çantası',
    '📁' => 'dosya klasörü',
    '📂' => 'açık dosya klasörü',
    '🗂' => 'dosya ayırıcı',
    '📅' => 'takvim',
    '📆' => 'koparmalı takvim',
    '🗒' => 'spiral not defteri',
    '🗓' => 'spiral takvim',
    '📇' => 'kart dizini',
    '📈' => 'artış grafiği',
    '📉' => 'iniş grafiği',
    '📊' => 'çubuk grafiği',
    '📋' => 'pano',
    '📌' => 'raptiye',
    '📍' => 'yuvarlak raptiye',
    '📎' => 'ataş',
    '🖇' => 'iç içe geçmiş ataşlar',
    '📏' => 'düz cetvel',
    '📐' => 'gönye',
    '✂' => 'makas',
    '🗃' => 'dosya kutusu',
    '🗄' => 'dosya dolabı',
    '🗑' => 'çöp sepeti',
    '🔒' => 'kilit',
    '🔓' => 'açık kilit',
    '🔏' => 'kilit ve kalem',
    '🔐' => 'anahtarlı kapalı kilit',
    '🔑' => 'anahtar',
    '🗝' => 'eski anahtar',
    '🔨' => 'çekiç',
    '🪓' => 'balta',
    '⛏' => 'kazma',
    '⚒' => 'çekiç ve kazma',
    '🛠' => 'çekiç ve somun anahtarı',
    '🗡' => 'hançer',
    '⚔' => 'çapraz kılıçlar',
    '💣' => 'bomba',
    '🪃' => 'bumerang',
    '🏹' => 'ok ve yay',
    '🛡' => 'kalkan',
    '🪚' => 'testere',
    '🔧' => 'somun anahtarı',
    '🪛' => 'tornavida',
    '🔩' => 'somun ve vida',
    '⚙' => 'dişli',
    '🗜' => 'mengene',
    '⚖' => 'terazi',
    '🦯' => 'görme engelli bastonu',
    '🔗' => 'bağlantı işareti',
    '⛓' => 'zincir',
    '🪝' => 'çengel',
    '🧰' => 'alet çantası',
    '🧲' => 'mıknatıs',
    '🪜' => 'merdiven',
    '🪏' => 'kürek',
    '⚗' => 'imbik',
    '🧪' => 'deney tüpü',
    '🧫' => 'petri kabı',
    '🧬' => 'dna',
    '🔬' => 'mikroskop',
    '🔭' => 'teleskop',
    '📡' => 'uydu anteni',
    '💉' => 'şırınga',
    '🩸' => 'kan damlası',
    '💊' => 'hap',
    '🩹' => 'yara bandı',
    '🩼' => 'koltuk değneği',
    '🩺' => 'stetoskop',
    '🩻' => 'x ışını',
    '🚪' => 'kapı',
    '🛗' => 'asansör',
    '🪞' => 'ayna',
    '🪟' => 'pencere',
    '🛏' => 'yatak',
    '🛋' => 'kanepe ve lamba',
    '🪑' => 'sandalye',
    '🚽' => 'klozet',
    '🪠' => 'lavabo pompası',
    '🚿' => 'duş',
    '🛁' => 'banyo küveti',
    '🪤' => 'fare kapanı',
    '🪒' => 'ustura',
    '🧴' => 'losyon şişesi',
    '🧷' => 'çengelli iğne',
    '🧹' => 'süpürge',
    '🧺' => 'sepet',
    '🧻' => 'kağıt rulosu',
    '🪣' => 'kova',
    '🧼' => 'sabun',
    '🫧' => 'baloncuk',
    '🪥' => 'diş fırçası',
    '🧽' => 'sünger',
    '🧯' => 'yangın söndürücü',
    '🛒' => 'alışveriş arabası',
    '🚬' => 'sigara',
    '⚰' => 'tabut',
    '🪦' => 'mezar taşı',
    '⚱' => 'kül kavanozu',
    '🧿' => 'nazarlık',
    '🪬' => 'Fatıma’nın Eli',
    '🗿' => 'moyai heykeli',
    '🪧' => 'tabela',
    '🪪' => 'kimlik kartı',
    '🏧' => 'ATM işareti',
    '🚮' => 'çöp kutusu işareti',
    '🚰' => 'içilebilir su',
    '♿' => 'tekerlekli sandalye sembolü',
    '🚹' => 'erkekler tuvaleti',
    '🚺' => 'kadınlar tuvaleti',
    '🚻' => 'tuvalet',
    '🚼' => 'bebek işareti',
    '🚾' => 'wc',
    '🛂' => 'pasaport kontrolü',
    '🛃' => 'gümrük',
    '🛄' => 'bagaj teslim',
    '🛅' => 'eşya emanet',
    '⚠' => 'uyarı işareti',
    '🚸' => 'karşıya geçen çocuklar',
    '⛔' => 'girilmez',
    '🚫' => 'girmek yasak',
    '🚳' => 'bisiklet yasak',
    '🚭' => 'sigara içilmez',
    '🚯' => 'çöp atılmaz',
    '🚱' => 'içilmeyen su',
    '🚷' => 'yayalara yasak',
    '📵' => 'cep telefonu yasak',
    '🔞' => '18 yaş altı yasak',
    '☢' => 'radyoaktif',
    '☣' => 'biyolojik tehlike',
    '⬆' => 'yukarı ok',
    '↗' => 'sağ yukarı ok',
    '➡' => 'sağ ok',
    '↘' => 'sağ aşağı ok',
    '⬇' => 'aşağı ok',
    '↙' => 'sol aşağı ok',
    '⬅' => 'sol ok',
    '↖' => 'sol yukarı ok',
    '↕' => 'yukarı ve aşağı ok',
    '↔' => 'sol ve sağ ok',
    '↩' => 'sola kıvrımlı sağ ok',
    '↪' => 'sağa kıvrımlı sol ok',
    '⤴' => 'yukarı kıvrımlı sağ ok',
    '⤵' => 'aşağı kıvrımlı sağ ok',
    '🔃' => 'saat yönünde dönen oklar',
    '🔄' => 'saat yönünün tersinde dönen oklar',
    '🔙' => 'geri oku',
    '🔚' => 'bitiş oku',
    '🔛' => 'açık oku',
    '🔜' => 'yakında işareti',
    '🔝' => 'yukarı oku',
    '🛐' => 'ibadethane',
    '⚛' => 'atom sembolü',
    '🕉' => 'om',
    '✡' => 'Davut yıldızı',
    '☸' => 'darma çemberi',
    '☯' => 'yin yang',
    '✝' => 'Latin haçı',
    '☦' => 'Ortodoks haçı',
    '☪' => 'ay yıldız',
    '☮' => 'barış sembolü',
    '🕎' => 'Yahudi şamdanı',
    '🔯' => 'altı köşeli yıldız',
    '🪯' => 'khanda',
    '♈' => 'koç burcu',
    '♉' => 'boğa burcu',
    '♊' => 'ikizler burcu',
    '♋' => 'yengeç burcu',
    '♌' => 'aslan burcu',
    '♍' => 'başak burcu',
    '♎' => 'terazi burcu',
    '♏' => 'akrep burcu',
    '♐' => 'yay burcu',
    '♑' => 'oğlak burcu',
    '♒' => 'kova burcu',
    '♓' => 'balık burcu',
    '⛎' => 'Yılancı',
    '🔀' => 'parça karıştırma düğmesi',
    '🔁' => 'tekrarlama düğmesi',
    '🔂' => 'tek parça tekrarlama düğmesi',
    '▶' => 'oynatma düğmesi',
    '⏩' => 'ileri sarma düğmesi',
    '⏭' => 'sonraki parça düğmesi',
    '⏯' => 'oynatma veya duraklatma düğmesi',
    '◀' => 'geri düğmesi',
    '⏪' => 'hızlı geri düğmesi',
    '⏮' => 'son parça düğmesi',
    '🔼' => 'yukarı düğmesi',
    '⏫' => 'hızlı yukarı düğmesi',
    '🔽' => 'aşağı düğmesi',
    '⏬' => 'hızlı aşağı düğmesi',
    '⏸' => 'duraklatma düğmesi',
    '⏹' => 'durdurma düğmesi',
    '⏺' => 'kayıt düğmesi',
    '⏏' => 'çıkarma düğmesi',
    '🎦' => 'sinema',
    '🔅' => 'düşük parlaklık',
    '🔆' => 'yüksek parlaklık',
    '📶' => 'sinyal gücü',
    '🛜' => 'kablosuz',
    '📳' => 'titreşim modu',
    '📴' => 'cep telefonu kapalı',
    '♀' => 'kadın sembolü',
    '♂' => 'erkek sembolü',
    '⚧' => 'transgender sembolü',
    '✖' => 'çarpım işareti',
    '➕' => 'artı',
    '➖' => 'eksi',
    '➗' => 'bölü işareti',
    '🟰' => 'kalın eşittir işareti',
    '♾' => 'sonsuz',
    '‼' => 'çift ünlem',
    '⁉' => 'ünlemli soru işareti',
    '❓' => 'kırmızı soru işareti',
    '❔' => 'beyaz soru işareti',
    '❕' => 'beyaz ünlem işareti',
    '❗' => 'kırmızı ünlem işareti',
    '〰' => 'dalgalı çizgi',
    '💱' => 'kambiyo',
    '💲' => 'dolar işareti',
    '⚕' => 'tıp sembolü',
    '♻' => 'geri dönüşüm simgesi',
    '⚜' => 'zambak',
    '🔱' => 'üçlü çatal işareti',
    '📛' => 'ad rozeti',
    '🔰' => 'Japonca yeni başlayan sembolü',
    '⭕' => 'kalın kırmızı çember',
    '✅' => 'onay işareti düğmesi',
    '☑' => 'seçilmiş onay kutusu',
    '✔' => 'onay işareti',
    '❌' => 'çarpı işareti',
    '❎' => 'çarpı işareti düğmesi',
    '➰' => 'kıvrık döngü',
    '➿' => 'çift kıvrık döngü',
    '〽' => 'parça değiştirme işareti',
    '✳' => 'sekiz uçlu yıldız',
    '✴' => 'sekiz noktalı yıldız',
    '❇' => 'parlama',
    '™' => 'ticari marka işareti',
    '🫟' => 'sıçratma',
    '🔠' => 'büyük harfler',
    '🔡' => 'küçük harfler',
    '🔢' => 'sayılar',
    '🔣' => 'işaretler',
    '🔤' => 'Latin alfabesi',
    '🅰' => 'A düğmesi (kan grubu)',
    '🆎' => 'AB düğmesi (kan grubu)',
    '🅱' => 'B grubu (kan grubu)',
    '🆑' => 'CL düğmesi',
    '🆒' => 'COOL düğmesi',
    '🆓' => 'FREE düğmesi',
    'ℹ' => 'bilgi',
    '🆔' => 'ID düğmesi',
    'Ⓜ' => 'daire içinde M harfi',
    '🆕' => 'NEW düğmesi',
    '🆖' => 'NG düğmesi',
    '🅾' => '0 düğmesi (kan grubu)',
    '🆗' => 'OK düğmesi',
    '🅿' => 'P düğmesi',
    '🆘' => 'SOS düğmesi',
    '🆙' => 'UP! düğmesi',
    '🆚' => 'VS düğmesi',
    '🈁' => 'Japonca "burada" düğmesi',
    '🈂' => 'Japonca "servis ücreti" düğmesi',
    '🈷' => 'Japonca "aylık ücret" düğmesi',
    '🈶' => 'Japonca "ücretsiz değil" düğmesi',
    '🈯' => 'Japonca "rezerve" düğmesi',
    '🉐' => 'Japonca "avantaj" düğmesi',
    '🈹' => 'Japonca "indirim" düğmesi',
    '🈚' => 'Japonca "ücretsiz" düğmesi',
    '🈲' => 'Japonca "yasak" düğmesi',
    '🉑' => 'Japonca "kabul edilebilir" düğmesi',
    '🈸' => 'Japonca "uygulama" düğmesi',
    '🈴' => 'Japonca "geçer not" düğmesi',
    '🈳' => 'Japonca "yer var" düğmesi',
    '㊗' => 'Japonca "tebrikler" düğmesi',
    '㊙' => 'Japonca "sır" düğmesi',
    '🈺' => 'Japonca "hizmete hazır" düğmesi',
    '🈵' => 'Japonca "yer yok" düğmesi',
    '🔴' => 'kırmızı daire',
    '🟠' => 'turuncu daire',
    '🟡' => 'sarı daire',
    '🟢' => 'yeşil daire',
    '🔵' => 'mavi daire',
    '🟣' => 'mor daire',
    '🟤' => 'kahverengi daire',
    '⚫' => 'siyah daire',
    '⚪' => 'beyaz daire',
    '🟥' => 'kırmızı kare',
    '🟧' => 'turuncu kare',
    '🟨' => 'sarı kare',
    '🟩' => 'yeşil kare',
    '🟦' => 'mavi kare',
    '🟪' => 'mor kare',
    '🟫' => 'kahverengi kare',
    '⬛' => 'büyük siyah kare',
    '⬜' => 'büyük beyaz kare',
    '◼' => 'orta boy siyah kare',
    '◻' => 'orta boy beyaz kare',
    '◾' => 'küçük-orta siyah kare',
    '◽' => 'küçük-orta beyaz kare',
    '▪' => 'küçük siyah kare',
    '▫' => 'küçük beyaz kare',
    '🔶' => 'büyük turuncu karo',
    '🔷' => 'büyük mavi karo',
    '🔸' => 'küçük turuncu karo',
    '🔹' => 'küçük mavi karo',
    '🔺' => 'yukarı bakan kırmızı üçgen',
    '🔻' => 'aşağı bakan kırmızı üçgen',
    '💠' => 'içinde noktalı karo',
    '🔘' => 'radyo düğmesi',
    '🔳' => 'beyaz kare düğme',
    '🔲' => 'siyah kare düğme',
    '🏁' => 'damalı bayrak',
    '🚩' => 'üçgen bayrak',
    '🎌' => 'çapraz bayraklar',
    '🏴' => 'siyah bayrak',
    '🏳' => 'beyaz bayrak',
];
