<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'csók: felnőtt, felnőtt, világos bőrtónus és közepesen világos bőrtónus',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'csók: felnőtt, felnőtt, világos bőrtónus és közepes bőrtónus',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'csók: felnőtt, felnőtt, világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'csók: felnőtt, felnőtt, világos bőrtónus és sötét bőrtónus',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'csók: felnőtt, felnőtt, közepesen világos bőrtónus és világos bőrtónus',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'csók: felnőtt, felnőtt, közepesen világos bőrtónus és közepes bőrtónus',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'csók: felnőtt, felnőtt, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'csók: felnőtt, felnőtt, közepesen világos bőrtónus és sötét bőrtónus',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'csók: felnőtt, felnőtt, közepes bőrtónus és világos bőrtónus',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'csók: felnőtt, felnőtt, közepes bőrtónus és közepesen világos bőrtónus',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'csók: felnőtt, felnőtt, közepes bőrtónus és közepesen sötét bőrtónus',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'csók: felnőtt, felnőtt, közepes bőrtónus és sötét bőrtónus',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'csók: felnőtt, felnőtt, közepesen sötét bőrtónus és világos bőrtónus',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'csók: felnőtt, felnőtt, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'csók: felnőtt, felnőtt, közepesen sötét bőrtónus és közepes bőrtónus',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'csók: felnőtt, felnőtt, közepesen sötét bőrtónus és sötét bőrtónus',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'csók: felnőtt, felnőtt, sötét bőrtónus és világos bőrtónus',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'csók: felnőtt, felnőtt, sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'csók: felnőtt, felnőtt, sötét bőrtónus és közepes bőrtónus',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'csók: felnőtt, felnőtt, sötét bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍❤‍💋‍👨🏻' => 'csók: férfi, férfi és világos bőrtónus',
    '👨🏻‍❤‍💋‍👨🏼' => 'csók: férfi, férfi, világos bőrtónus és közepesen világos bőrtónus',
    '👨🏻‍❤‍💋‍👨🏽' => 'csók: férfi, férfi, világos bőrtónus és közepes bőrtónus',
    '👨🏻‍❤‍💋‍👨🏾' => 'csók: férfi, férfi, világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍❤‍💋‍👨🏿' => 'csók: férfi, férfi, világos bőrtónus és sötét bőrtónus',
    '👨🏼‍❤‍💋‍👨🏻' => 'csók: férfi, férfi, közepesen világos bőrtónus és világos bőrtónus',
    '👨🏼‍❤‍💋‍👨🏼' => 'csók: férfi, férfi és közepesen világos bőrtónus',
    '👨🏼‍❤‍💋‍👨🏽' => 'csók: férfi, férfi, közepesen világos bőrtónus és közepes bőrtónus',
    '👨🏼‍❤‍💋‍👨🏾' => 'csók: férfi, férfi, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏼‍❤‍💋‍👨🏿' => 'csók: férfi, férfi, közepesen világos bőrtónus és sötét bőrtónus',
    '👨🏽‍❤‍💋‍👨🏻' => 'csók: férfi, férfi, közepes bőrtónus és világos bőrtónus',
    '👨🏽‍❤‍💋‍👨🏼' => 'csók: férfi, férfi, közepes bőrtónus és közepesen világos bőrtónus',
    '👨🏽‍❤‍💋‍👨🏽' => 'csók: férfi, férfi és közepes bőrtónus',
    '👨🏽‍❤‍💋‍👨🏾' => 'csók: férfi, férfi, közepes bőrtónus és közepesen sötét bőrtónus',
    '👨🏽‍❤‍💋‍👨🏿' => 'csók: férfi, férfi, közepes bőrtónus és sötét bőrtónus',
    '👨🏾‍❤‍💋‍👨🏻' => 'csók: férfi, férfi, közepesen sötét bőrtónus és világos bőrtónus',
    '👨🏾‍❤‍💋‍👨🏼' => 'csók: férfi, férfi, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏾‍❤‍💋‍👨🏽' => 'csók: férfi, férfi, közepesen sötét bőrtónus és közepes bőrtónus',
    '👨🏾‍❤‍💋‍👨🏾' => 'csók: férfi, férfi és közepesen sötét bőrtónus',
    '👨🏾‍❤‍💋‍👨🏿' => 'csók: férfi, férfi, közepesen sötét bőrtónus és sötét bőrtónus',
    '👨🏿‍❤‍💋‍👨🏻' => 'csók: férfi, férfi, sötét bőrtónus és világos bőrtónus',
    '👨🏿‍❤‍💋‍👨🏼' => 'csók: férfi, férfi, sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏿‍❤‍💋‍👨🏽' => 'csók: férfi, férfi, sötét bőrtónus és közepes bőrtónus',
    '👨🏿‍❤‍💋‍👨🏾' => 'csók: férfi, férfi, sötét bőrtónus és közepesen sötét bőrtónus',
    '👨🏿‍❤‍💋‍👨🏿' => 'csók: férfi, férfi és sötét bőrtónus',
    '👩🏻‍❤‍💋‍👨🏻' => 'csók: nő, férfi és világos bőrtónus',
    '👩🏻‍❤‍💋‍👨🏼' => 'csók: nő, férfi, világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍❤‍💋‍👨🏽' => 'csók: nő, férfi, világos bőrtónus és közepes bőrtónus',
    '👩🏻‍❤‍💋‍👨🏾' => 'csók: nő, férfi, világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍❤‍💋‍👨🏿' => 'csók: nő, férfi, világos bőrtónus és sötét bőrtónus',
    '👩🏼‍❤‍💋‍👨🏻' => 'csók: nő, férfi, közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍❤‍💋‍👨🏼' => 'csók: nő, férfi és közepesen világos bőrtónus',
    '👩🏼‍❤‍💋‍👨🏽' => 'csók: nő, férfi, közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍❤‍💋‍👨🏾' => 'csók: nő, férfi, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍❤‍💋‍👨🏿' => 'csók: nő, férfi, közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍❤‍💋‍👨🏻' => 'csók: nő, férfi, közepes bőrtónus és világos bőrtónus',
    '👩🏽‍❤‍💋‍👨🏼' => 'csók: nő, férfi, közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍❤‍💋‍👨🏽' => 'csók: nő, férfi és közepes bőrtónus',
    '👩🏽‍❤‍💋‍👨🏾' => 'csók: nő, férfi, közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍❤‍💋‍👨🏿' => 'csók: nő, férfi, közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍❤‍💋‍👨🏻' => 'csók: nő, férfi, közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍❤‍💋‍👨🏼' => 'csók: nő, férfi, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍❤‍💋‍👨🏽' => 'csók: nő, férfi, közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍❤‍💋‍👨🏾' => 'csók: nő, férfi és közepesen sötét bőrtónus',
    '👩🏾‍❤‍💋‍👨🏿' => 'csók: nő, férfi, közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍❤‍💋‍👨🏻' => 'csók: nő, férfi, sötét bőrtónus és világos bőrtónus',
    '👩🏿‍❤‍💋‍👨🏼' => 'csók: nő, férfi, sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍❤‍💋‍👨🏽' => 'csók: nő, férfi, sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍❤‍💋‍👨🏾' => 'csók: nő, férfi, sötét bőrtónus és közepesen sötét bőrtónus',
    '👩🏿‍❤‍💋‍👨🏿' => 'csók: nő, férfi és sötét bőrtónus',
    '👩🏻‍❤‍💋‍👩🏻' => 'csók: nő, nő és világos bőrtónus',
    '👩🏻‍❤‍💋‍👩🏼' => 'csók: nő, nő, világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍❤‍💋‍👩🏽' => 'csók: nő, nő, világos bőrtónus és közepes bőrtónus',
    '👩🏻‍❤‍💋‍👩🏾' => 'csók: nő, nő, világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍❤‍💋‍👩🏿' => 'csók: nő, nő, világos bőrtónus és sötét bőrtónus',
    '👩🏼‍❤‍💋‍👩🏻' => 'csók: nő, nő, közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍❤‍💋‍👩🏼' => 'csók: nő, nő és közepesen világos bőrtónus',
    '👩🏼‍❤‍💋‍👩🏽' => 'csók: nő, nő, közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍❤‍💋‍👩🏾' => 'csók: nő, nő, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍❤‍💋‍👩🏿' => 'csók: nő, nő, közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍❤‍💋‍👩🏻' => 'csók: nő, nő, közepes bőrtónus és világos bőrtónus',
    '👩🏽‍❤‍💋‍👩🏼' => 'csók: nő, nő, közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍❤‍💋‍👩🏽' => 'csók: nő, nő és közepes bőrtónus',
    '👩🏽‍❤‍💋‍👩🏾' => 'csók: nő, nő, közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍❤‍💋‍👩🏿' => 'csók: nő, nő, közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍❤‍💋‍👩🏻' => 'csók: nő, nő, közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍❤‍💋‍👩🏼' => 'csók: nő, nő, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍❤‍💋‍👩🏽' => 'csók: nő, nő, közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍❤‍💋‍👩🏾' => 'csók: nő, nő és közepesen sötét bőrtónus',
    '👩🏾‍❤‍💋‍👩🏿' => 'csók: nő, nő, közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍❤‍💋‍👩🏻' => 'csók: nő, nő, sötét bőrtónus és világos bőrtónus',
    '👩🏿‍❤‍💋‍👩🏼' => 'csók: nő, nő, sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍❤‍💋‍👩🏽' => 'csók: nő, nő, sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍❤‍💋‍👩🏾' => 'csók: nő, nő, sötét bőrtónus és közepesen sötét bőrtónus',
    '👩🏿‍❤‍💋‍👩🏿' => 'csók: nő, nő és sötét bőrtónus',
    '🧑🏻‍🫯‍🧑🏼' => 'birkózók: világos bőrtónus és közepesen világos bőrtónus',
    '🧑🏻‍🫯‍🧑🏽' => 'birkózók: világos bőrtónus és közepes bőrtónus',
    '🧑🏻‍🫯‍🧑🏾' => 'birkózók: világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍🫯‍🧑🏿' => 'birkózók: világos bőrtónus és sötét bőrtónus',
    '🧑🏼‍🫯‍🧑🏻' => 'birkózók: közepesen világos bőrtónus és világos bőrtónus',
    '🧑🏼‍🫯‍🧑🏽' => 'birkózók: közepesen világos bőrtónus és közepes bőrtónus',
    '🧑🏼‍🫯‍🧑🏾' => 'birkózók: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏼‍🫯‍🧑🏿' => 'birkózók: közepesen világos bőrtónus és sötét bőrtónus',
    '🧑🏽‍🫯‍🧑🏻' => 'birkózók: közepes bőrtónus és világos bőrtónus',
    '🧑🏽‍🫯‍🧑🏼' => 'birkózók: közepes bőrtónus és közepesen világos bőrtónus',
    '🧑🏽‍🫯‍🧑🏾' => 'birkózók: közepes bőrtónus és közepesen sötét bőrtónus',
    '🧑🏽‍🫯‍🧑🏿' => 'birkózók: közepes bőrtónus és sötét bőrtónus',
    '🧑🏾‍🫯‍🧑🏻' => 'birkózók: közepesen sötét bőrtónus és világos bőrtónus',
    '🧑🏾‍🫯‍🧑🏼' => 'birkózók: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏾‍🫯‍🧑🏽' => 'birkózók: közepesen sötét bőrtónus és közepes bőrtónus',
    '🧑🏾‍🫯‍🧑🏿' => 'birkózók: közepesen sötét bőrtónus és sötét bőrtónus',
    '🧑🏿‍🫯‍🧑🏻' => 'birkózók: sötét bőrtónus és világos bőrtónus',
    '🧑🏿‍🫯‍🧑🏼' => 'birkózók: sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏿‍🫯‍🧑🏽' => 'birkózók: sötét bőrtónus és közepes bőrtónus',
    '🧑🏿‍🫯‍🧑🏾' => 'birkózók: sötét bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍❤‍🧑🏼' => 'pár szívvel: felnőtt, felnőtt, világos bőrtónus és közepesen világos bőrtónus',
    '🧑🏻‍❤‍🧑🏽' => 'pár szívvel: felnőtt, felnőtt, világos bőrtónus és közepes bőrtónus',
    '🧑🏻‍❤‍🧑🏾' => 'pár szívvel: felnőtt, felnőtt, világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍❤‍🧑🏿' => 'pár szívvel: felnőtt, felnőtt, világos bőrtónus és sötét bőrtónus',
    '🧑🏼‍❤‍🧑🏻' => 'pár szívvel: felnőtt, felnőtt, közepesen világos bőrtónus és világos bőrtónus',
    '🧑🏼‍❤‍🧑🏽' => 'pár szívvel: felnőtt, felnőtt, közepesen világos bőrtónus és közepes bőrtónus',
    '🧑🏼‍❤‍🧑🏾' => 'pár szívvel: felnőtt, felnőtt, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏼‍❤‍🧑🏿' => 'pár szívvel: felnőtt, felnőtt, közepesen világos bőrtónus és sötét bőrtónus',
    '🧑🏽‍❤‍🧑🏻' => 'pár szívvel: felnőtt, felnőtt, közepes bőrtónus és világos bőrtónus',
    '🧑🏽‍❤‍🧑🏼' => 'pár szívvel: felnőtt, felnőtt, közepes bőrtónus és közepesen világos bőrtónus',
    '🧑🏽‍❤‍🧑🏾' => 'pár szívvel: felnőtt, felnőtt, közepes bőrtónus és közepesen sötét bőrtónus',
    '🧑🏽‍❤‍🧑🏿' => 'pár szívvel: felnőtt, felnőtt, közepes bőrtónus és sötét bőrtónus',
    '🧑🏾‍❤‍🧑🏻' => 'pár szívvel: felnőtt, felnőtt, közepesen sötét bőrtónus és világos bőrtónus',
    '🧑🏾‍❤‍🧑🏼' => 'pár szívvel: felnőtt, felnőtt, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏾‍❤‍🧑🏽' => 'pár szívvel: felnőtt, felnőtt, közepesen sötét bőrtónus és közepes bőrtónus',
    '🧑🏾‍❤‍🧑🏿' => 'pár szívvel: felnőtt, felnőtt, közepesen sötét bőrtónus és sötét bőrtónus',
    '🧑🏿‍❤‍🧑🏻' => 'pár szívvel: felnőtt, felnőtt, sötét bőrtónus és világos bőrtónus',
    '🧑🏿‍❤‍🧑🏼' => 'pár szívvel: felnőtt, felnőtt, sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏿‍❤‍🧑🏽' => 'pár szívvel: felnőtt, felnőtt, sötét bőrtónus és közepes bőrtónus',
    '🧑🏿‍❤‍🧑🏾' => 'pár szívvel: felnőtt, felnőtt, sötét bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍🐰‍🧑🏼' => 'bulizó emberek: világos bőrtónus és közepesen világos bőrtónus',
    '🧑🏻‍🐰‍🧑🏽' => 'bulizó emberek: világos bőrtónus és közepes bőrtónus',
    '🧑🏻‍🐰‍🧑🏾' => 'bulizó emberek: világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍🐰‍🧑🏿' => 'bulizó emberek: világos bőrtónus és sötét bőrtónus',
    '🧑🏼‍🐰‍🧑🏻' => 'bulizó emberek: közepesen világos bőrtónus és világos bőrtónus',
    '🧑🏼‍🐰‍🧑🏽' => 'bulizó emberek: közepesen világos bőrtónus és közepes bőrtónus',
    '🧑🏼‍🐰‍🧑🏾' => 'bulizó emberek: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏼‍🐰‍🧑🏿' => 'bulizó emberek: közepesen világos bőrtónus és sötét bőrtónus',
    '🧑🏽‍🐰‍🧑🏻' => 'bulizó emberek: közepes bőrtónus és világos bőrtónus',
    '🧑🏽‍🐰‍🧑🏼' => 'bulizó emberek: közepes bőrtónus és közepesen világos bőrtónus',
    '🧑🏽‍🐰‍🧑🏾' => 'bulizó emberek: közepes bőrtónus és közepesen sötét bőrtónus',
    '🧑🏽‍🐰‍🧑🏿' => 'bulizó emberek: közepes bőrtónus és sötét bőrtónus',
    '🧑🏾‍🐰‍🧑🏻' => 'bulizó emberek: közepesen sötét bőrtónus és világos bőrtónus',
    '🧑🏾‍🐰‍🧑🏼' => 'bulizó emberek: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏾‍🐰‍🧑🏽' => 'bulizó emberek: közepesen sötét bőrtónus és közepes bőrtónus',
    '🧑🏾‍🐰‍🧑🏿' => 'bulizó emberek: közepesen sötét bőrtónus és sötét bőrtónus',
    '🧑🏿‍🐰‍🧑🏻' => 'bulizó emberek: sötét bőrtónus és világos bőrtónus',
    '🧑🏿‍🐰‍🧑🏼' => 'bulizó emberek: sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏿‍🐰‍🧑🏽' => 'bulizó emberek: sötét bőrtónus és közepes bőrtónus',
    '🧑🏿‍🐰‍🧑🏾' => 'bulizó emberek: sötét bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍🫯‍👨🏼' => 'birkózó férfiak: világos bőrtónus és közepesen világos bőrtónus',
    '👨🏻‍🫯‍👨🏽' => 'birkózó férfiak: világos bőrtónus és közepes bőrtónus',
    '👨🏻‍🫯‍👨🏾' => 'birkózó férfiak: világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍🫯‍👨🏿' => 'birkózó férfiak: világos bőrtónus és sötét bőrtónus',
    '👨🏼‍🫯‍👨🏻' => 'birkózó férfiak: közepesen világos bőrtónus és világos bőrtónus',
    '👨🏼‍🫯‍👨🏽' => 'birkózó férfiak: közepesen világos bőrtónus és közepes bőrtónus',
    '👨🏼‍🫯‍👨🏾' => 'birkózó férfiak: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏼‍🫯‍👨🏿' => 'birkózó férfiak: közepesen világos bőrtónus és sötét bőrtónus',
    '👨🏽‍🫯‍👨🏻' => 'birkózó férfiak: közepes bőrtónus és világos bőrtónus',
    '👨🏽‍🫯‍👨🏼' => 'birkózó férfiak: közepes bőrtónus és közepesen világos bőrtónus',
    '👨🏽‍🫯‍👨🏾' => 'birkózó férfiak: közepes bőrtónus és közepesen sötét bőrtónus',
    '👨🏽‍🫯‍👨🏿' => 'birkózó férfiak: közepes bőrtónus és sötét bőrtónus',
    '👨🏾‍🫯‍👨🏻' => 'birkózó férfiak: közepesen sötét bőrtónus és világos bőrtónus',
    '👨🏾‍🫯‍👨🏼' => 'birkózó férfiak: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏾‍🫯‍👨🏽' => 'birkózó férfiak: közepesen sötét bőrtónus és közepes bőrtónus',
    '👨🏾‍🫯‍👨🏿' => 'birkózó férfiak: közepesen sötét bőrtónus és sötét bőrtónus',
    '👨🏿‍🫯‍👨🏻' => 'birkózó férfiak: sötét bőrtónus és világos bőrtónus',
    '👨🏿‍🫯‍👨🏼' => 'birkózó férfiak: sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏿‍🫯‍👨🏽' => 'birkózó férfiak: sötét bőrtónus és közepes bőrtónus',
    '👨🏿‍🫯‍👨🏾' => 'birkózó férfiak: sötét bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍❤‍👨🏻' => 'pár szívvel: férfi, férfi és világos bőrtónus',
    '👨🏻‍❤‍👨🏼' => 'pár szívvel: férfi, férfi, világos bőrtónus és közepesen világos bőrtónus',
    '👨🏻‍❤‍👨🏽' => 'pár szívvel: férfi, férfi, világos bőrtónus és közepes bőrtónus',
    '👨🏻‍❤‍👨🏾' => 'pár szívvel: férfi, férfi, világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍❤‍👨🏿' => 'pár szívvel: férfi, férfi, világos bőrtónus és sötét bőrtónus',
    '👨🏼‍❤‍👨🏻' => 'pár szívvel: férfi, férfi, közepesen világos bőrtónus és világos bőrtónus',
    '👨🏼‍❤‍👨🏼' => 'pár szívvel: férfi, férfi és közepesen világos bőrtónus',
    '👨🏼‍❤‍👨🏽' => 'pár szívvel: férfi, férfi, közepesen világos bőrtónus és közepes bőrtónus',
    '👨🏼‍❤‍👨🏾' => 'pár szívvel: férfi, férfi, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏼‍❤‍👨🏿' => 'pár szívvel: férfi, férfi, közepesen világos bőrtónus és sötét bőrtónus',
    '👨🏽‍❤‍👨🏻' => 'pár szívvel: férfi, férfi, közepes bőrtónus és világos bőrtónus',
    '👨🏽‍❤‍👨🏼' => 'pár szívvel: férfi, férfi, közepes bőrtónus és közepesen világos bőrtónus',
    '👨🏽‍❤‍👨🏽' => 'pár szívvel: férfi, férfi és közepes bőrtónus',
    '👨🏽‍❤‍👨🏾' => 'pár szívvel: férfi, férfi, közepes bőrtónus és közepesen sötét bőrtónus',
    '👨🏽‍❤‍👨🏿' => 'pár szívvel: férfi, férfi, közepes bőrtónus és sötét bőrtónus',
    '👨🏾‍❤‍👨🏻' => 'pár szívvel: férfi, férfi, közepesen sötét bőrtónus és világos bőrtónus',
    '👨🏾‍❤‍👨🏼' => 'pár szívvel: férfi, férfi, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏾‍❤‍👨🏽' => 'pár szívvel: férfi, férfi, közepesen sötét bőrtónus és közepes bőrtónus',
    '👨🏾‍❤‍👨🏾' => 'pár szívvel: férfi, férfi és közepesen sötét bőrtónus',
    '👨🏾‍❤‍👨🏿' => 'pár szívvel: férfi, férfi, közepesen sötét bőrtónus és sötét bőrtónus',
    '👨🏿‍❤‍👨🏻' => 'pár szívvel: férfi, férfi, sötét bőrtónus és világos bőrtónus',
    '👨🏿‍❤‍👨🏼' => 'pár szívvel: férfi, férfi, sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏿‍❤‍👨🏽' => 'pár szívvel: férfi, férfi, sötét bőrtónus és közepes bőrtónus',
    '👨🏿‍❤‍👨🏾' => 'pár szívvel: férfi, férfi, sötét bőrtónus és közepesen sötét bőrtónus',
    '👨🏿‍❤‍👨🏿' => 'pár szívvel: férfi, férfi és sötét bőrtónus',
    '👨🏻‍🐰‍👨🏼' => 'bulizó férfiak: világos bőrtónus és közepesen világos bőrtónus',
    '👨🏻‍🐰‍👨🏽' => 'bulizó férfiak: világos bőrtónus és közepes bőrtónus',
    '👨🏻‍🐰‍👨🏾' => 'bulizó férfiak: világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍🐰‍👨🏿' => 'bulizó férfiak: világos bőrtónus és sötét bőrtónus',
    '👨🏼‍🐰‍👨🏻' => 'bulizó férfiak: közepesen világos bőrtónus és világos bőrtónus',
    '👨🏼‍🐰‍👨🏽' => 'bulizó férfiak: közepesen világos bőrtónus és közepes bőrtónus',
    '👨🏼‍🐰‍👨🏾' => 'bulizó férfiak: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏼‍🐰‍👨🏿' => 'bulizó férfiak: közepesen világos bőrtónus és sötét bőrtónus',
    '👨🏽‍🐰‍👨🏻' => 'bulizó férfiak: közepes bőrtónus és világos bőrtónus',
    '👨🏽‍🐰‍👨🏼' => 'bulizó férfiak: közepes bőrtónus és közepesen világos bőrtónus',
    '👨🏽‍🐰‍👨🏾' => 'bulizó férfiak: közepes bőrtónus és közepesen sötét bőrtónus',
    '👨🏽‍🐰‍👨🏿' => 'bulizó férfiak: közepes bőrtónus és sötét bőrtónus',
    '👨🏾‍🐰‍👨🏻' => 'bulizó férfiak: közepesen sötét bőrtónus és világos bőrtónus',
    '👨🏾‍🐰‍👨🏼' => 'bulizó férfiak: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏾‍🐰‍👨🏽' => 'bulizó férfiak: közepesen sötét bőrtónus és közepes bőrtónus',
    '👨🏾‍🐰‍👨🏿' => 'bulizó férfiak: közepesen sötét bőrtónus és sötét bőrtónus',
    '👨🏿‍🐰‍👨🏻' => 'bulizó férfiak: sötét bőrtónus és világos bőrtónus',
    '👨🏿‍🐰‍👨🏼' => 'bulizó férfiak: sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏿‍🐰‍👨🏽' => 'bulizó férfiak: sötét bőrtónus és közepes bőrtónus',
    '👨🏿‍🐰‍👨🏾' => 'bulizó férfiak: sötét bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍🫯‍👩🏼' => 'birkózó nők: világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍🫯‍👩🏽' => 'birkózó nők: világos bőrtónus és közepes bőrtónus',
    '👩🏻‍🫯‍👩🏾' => 'birkózó nők: világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍🫯‍👩🏿' => 'birkózó nők: világos bőrtónus és sötét bőrtónus',
    '👩🏼‍🫯‍👩🏻' => 'birkózó nők: közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍🫯‍👩🏽' => 'birkózó nők: közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍🫯‍👩🏾' => 'birkózó nők: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍🫯‍👩🏿' => 'birkózó nők: közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍🫯‍👩🏻' => 'birkózó nők: közepes bőrtónus és világos bőrtónus',
    '👩🏽‍🫯‍👩🏼' => 'birkózó nők: közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍🫯‍👩🏾' => 'birkózó nők: közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍🫯‍👩🏿' => 'birkózó nők: közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍🫯‍👩🏻' => 'birkózó nők: közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍🫯‍👩🏼' => 'birkózó nők: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍🫯‍👩🏽' => 'birkózó nők: közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍🫯‍👩🏿' => 'birkózó nők: közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍🫯‍👩🏻' => 'birkózó nők: sötét bőrtónus és világos bőrtónus',
    '👩🏿‍🫯‍👩🏼' => 'birkózó nők: sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍🫯‍👩🏽' => 'birkózó nők: sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍🫯‍👩🏾' => 'birkózó nők: sötét bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍❤‍👨🏻' => 'pár szívvel: nő, férfi és világos bőrtónus',
    '👩🏻‍❤‍👨🏼' => 'pár szívvel: nő, férfi, világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍❤‍👨🏽' => 'pár szívvel: nő, férfi, világos bőrtónus és közepes bőrtónus',
    '👩🏻‍❤‍👨🏾' => 'pár szívvel: nő, férfi, világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍❤‍👨🏿' => 'pár szívvel: nő, férfi, világos bőrtónus és sötét bőrtónus',
    '👩🏼‍❤‍👨🏻' => 'pár szívvel: nő, férfi, közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍❤‍👨🏼' => 'pár szívvel: nő, férfi és közepesen világos bőrtónus',
    '👩🏼‍❤‍👨🏽' => 'pár szívvel: nő, férfi, közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍❤‍👨🏾' => 'pár szívvel: nő, férfi, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍❤‍👨🏿' => 'pár szívvel: nő, férfi, közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍❤‍👨🏻' => 'pár szívvel: nő, férfi, közepes bőrtónus és világos bőrtónus',
    '👩🏽‍❤‍👨🏼' => 'pár szívvel: nő, férfi, közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍❤‍👨🏽' => 'pár szívvel: nő, férfi és közepes bőrtónus',
    '👩🏽‍❤‍👨🏾' => 'pár szívvel: nő, férfi, közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍❤‍👨🏿' => 'pár szívvel: nő, férfi, közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍❤‍👨🏻' => 'pár szívvel: nő, férfi, közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍❤‍👨🏼' => 'pár szívvel: nő, férfi, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍❤‍👨🏽' => 'pár szívvel: nő, férfi, közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍❤‍👨🏾' => 'pár szívvel: nő, férfi és közepesen sötét bőrtónus',
    '👩🏾‍❤‍👨🏿' => 'pár szívvel: nő, férfi, közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍❤‍👨🏻' => 'pár szívvel: nő, férfi, sötét bőrtónus és világos bőrtónus',
    '👩🏿‍❤‍👨🏼' => 'pár szívvel: nő, férfi, sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍❤‍👨🏽' => 'pár szívvel: nő, férfi, sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍❤‍👨🏾' => 'pár szívvel: nő, férfi, sötét bőrtónus és közepesen sötét bőrtónus',
    '👩🏿‍❤‍👨🏿' => 'pár szívvel: nő, férfi és sötét bőrtónus',
    '👩🏻‍❤‍👩🏻' => 'pár szívvel: nő, nő és világos bőrtónus',
    '👩🏻‍❤‍👩🏼' => 'pár szívvel: nő, nő, világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍❤‍👩🏽' => 'pár szívvel: nő, nő, világos bőrtónus és közepes bőrtónus',
    '👩🏻‍❤‍👩🏾' => 'pár szívvel: nő, nő, világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍❤‍👩🏿' => 'pár szívvel: nő, nő, világos bőrtónus és sötét bőrtónus',
    '👩🏼‍❤‍👩🏻' => 'pár szívvel: nő, nő, közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍❤‍👩🏼' => 'pár szívvel: nő, nő és közepesen világos bőrtónus',
    '👩🏼‍❤‍👩🏽' => 'pár szívvel: nő, nő, közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍❤‍👩🏾' => 'pár szívvel: nő, nő, közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍❤‍👩🏿' => 'pár szívvel: nő, nő, közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍❤‍👩🏻' => 'pár szívvel: nő, nő, közepes bőrtónus és világos bőrtónus',
    '👩🏽‍❤‍👩🏼' => 'pár szívvel: nő, nő, közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍❤‍👩🏽' => 'pár szívvel: nő, nő és közepes bőrtónus',
    '👩🏽‍❤‍👩🏾' => 'pár szívvel: nő, nő, közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍❤‍👩🏿' => 'pár szívvel: nő, nő, közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍❤‍👩🏻' => 'pár szívvel: nő, nő, közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍❤‍👩🏼' => 'pár szívvel: nő, nő, közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍❤‍👩🏽' => 'pár szívvel: nő, nő, közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍❤‍👩🏾' => 'pár szívvel: nő, nő és közepesen sötét bőrtónus',
    '👩🏾‍❤‍👩🏿' => 'pár szívvel: nő, nő, közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍❤‍👩🏻' => 'pár szívvel: nő, nő, sötét bőrtónus és világos bőrtónus',
    '👩🏿‍❤‍👩🏼' => 'pár szívvel: nő, nő, sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍❤‍👩🏽' => 'pár szívvel: nő, nő, sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍❤‍👩🏾' => 'pár szívvel: nő, nő, sötét bőrtónus és közepesen sötét bőrtónus',
    '👩🏿‍❤‍👩🏿' => 'pár szívvel: nő, nő és sötét bőrtónus',
    '👩🏻‍🐰‍👩🏼' => 'bulizó nők: világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍🐰‍👩🏽' => 'bulizó nők: világos bőrtónus és közepes bőrtónus',
    '👩🏻‍🐰‍👩🏾' => 'bulizó nők: világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍🐰‍👩🏿' => 'bulizó nők: világos bőrtónus és sötét bőrtónus',
    '👩🏼‍🐰‍👩🏻' => 'bulizó nők: közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍🐰‍👩🏽' => 'bulizó nők: közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍🐰‍👩🏾' => 'bulizó nők: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍🐰‍👩🏿' => 'bulizó nők: közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍🐰‍👩🏻' => 'bulizó nők: közepes bőrtónus és világos bőrtónus',
    '👩🏽‍🐰‍👩🏼' => 'bulizó nők: közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍🐰‍👩🏾' => 'bulizó nők: közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍🐰‍👩🏿' => 'bulizó nők: közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍🐰‍👩🏻' => 'bulizó nők: közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍🐰‍👩🏼' => 'bulizó nők: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍🐰‍👩🏽' => 'bulizó nők: közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍🐰‍👩🏿' => 'bulizó nők: közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍🐰‍👩🏻' => 'bulizó nők: sötét bőrtónus és világos bőrtónus',
    '👩🏿‍🐰‍👩🏼' => 'bulizó nők: sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍🐰‍👩🏽' => 'bulizó nők: sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍🐰‍👩🏾' => 'bulizó nők: sötét bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍🤝‍🧑🏻' => 'egymás kezét fogó emberek: világos bőrtónus',
    '🧑🏻‍🤝‍🧑🏼' => 'egymás kezét fogó emberek: világos bőrtónus és közepesen világos bőrtónus',
    '🧑🏻‍🤝‍🧑🏽' => 'egymás kezét fogó emberek: világos bőrtónus és közepes bőrtónus',
    '🧑🏻‍🤝‍🧑🏾' => 'egymás kezét fogó emberek: világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏻‍🤝‍🧑🏿' => 'egymás kezét fogó emberek: világos bőrtónus és sötét bőrtónus',
    '🧑🏼‍🤝‍🧑🏻' => 'egymás kezét fogó emberek: közepesen világos bőrtónus és világos bőrtónus',
    '🧑🏼‍🤝‍🧑🏼' => 'egymás kezét fogó emberek: közepesen világos bőrtónus',
    '🧑🏼‍🤝‍🧑🏽' => 'egymás kezét fogó emberek: közepesen világos bőrtónus és közepes bőrtónus',
    '🧑🏼‍🤝‍🧑🏾' => 'egymás kezét fogó emberek: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '🧑🏼‍🤝‍🧑🏿' => 'egymás kezét fogó emberek: közepesen világos bőrtónus és sötét bőrtónus',
    '🧑🏽‍🤝‍🧑🏻' => 'egymás kezét fogó emberek: közepes bőrtónus és világos bőrtónus',
    '🧑🏽‍🤝‍🧑🏼' => 'egymás kezét fogó emberek: közepes bőrtónus és közepesen világos bőrtónus',
    '🧑🏽‍🤝‍🧑🏽' => 'egymás kezét fogó emberek: közepes bőrtónus',
    '🧑🏽‍🤝‍🧑🏾' => 'egymás kezét fogó emberek: közepes bőrtónus és közepesen sötét bőrtónus',
    '🧑🏽‍🤝‍🧑🏿' => 'egymás kezét fogó emberek: közepes bőrtónus és sötét bőrtónus',
    '🧑🏾‍🤝‍🧑🏻' => 'egymás kezét fogó emberek: közepesen sötét bőrtónus és világos bőrtónus',
    '🧑🏾‍🤝‍🧑🏼' => 'egymás kezét fogó emberek: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏾‍🤝‍🧑🏽' => 'egymás kezét fogó emberek: közepesen sötét bőrtónus és közepes bőrtónus',
    '🧑🏾‍🤝‍🧑🏾' => 'egymás kezét fogó emberek: közepesen sötét bőrtónus',
    '🧑🏾‍🤝‍🧑🏿' => 'egymás kezét fogó emberek: közepesen sötét bőrtónus és sötét bőrtónus',
    '🧑🏿‍🤝‍🧑🏻' => 'egymás kezét fogó emberek: sötét bőrtónus és világos bőrtónus',
    '🧑🏿‍🤝‍🧑🏼' => 'egymás kezét fogó emberek: sötét bőrtónus és közepesen világos bőrtónus',
    '🧑🏿‍🤝‍🧑🏽' => 'egymás kezét fogó emberek: sötét bőrtónus és közepes bőrtónus',
    '🧑🏿‍🤝‍🧑🏾' => 'egymás kezét fogó emberek: sötét bőrtónus és közepesen sötét bőrtónus',
    '🧑🏿‍🤝‍🧑🏿' => 'egymás kezét fogó emberek: sötét bőrtónus',
    '👩🏻‍🤝‍👩🏼' => 'kézen fogva álló nők: világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍🤝‍👩🏽' => 'kézen fogva álló nők: világos bőrtónus és közepes bőrtónus',
    '👩🏻‍🤝‍👩🏾' => 'kézen fogva álló nők: világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍🤝‍👩🏿' => 'kézen fogva álló nők: világos bőrtónus és sötét bőrtónus',
    '👩🏼‍🤝‍👩🏻' => 'kézen fogva álló nők: közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍🤝‍👩🏽' => 'kézen fogva álló nők: közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍🤝‍👩🏾' => 'kézen fogva álló nők: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍🤝‍👩🏿' => 'kézen fogva álló nők: közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍🤝‍👩🏻' => 'kézen fogva álló nők: közepes bőrtónus és világos bőrtónus',
    '👩🏽‍🤝‍👩🏼' => 'kézen fogva álló nők: közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍🤝‍👩🏾' => 'kézen fogva álló nők: közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍🤝‍👩🏿' => 'kézen fogva álló nők: közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍🤝‍👩🏻' => 'kézen fogva álló nők: közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍🤝‍👩🏼' => 'kézen fogva álló nők: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍🤝‍👩🏽' => 'kézen fogva álló nők: közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍🤝‍👩🏿' => 'kézen fogva álló nők: közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍🤝‍👩🏻' => 'kézen fogva álló nők: sötét bőrtónus és világos bőrtónus',
    '👩🏿‍🤝‍👩🏼' => 'kézen fogva álló nők: sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍🤝‍👩🏽' => 'kézen fogva álló nők: sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍🤝‍👩🏾' => 'kézen fogva álló nők: sötét bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍🤝‍👨🏼' => 'kézen fogva álló férfi és nő: világos bőrtónus és közepesen világos bőrtónus',
    '👩🏻‍🤝‍👨🏽' => 'kézen fogva álló férfi és nő: világos bőrtónus és közepes bőrtónus',
    '👩🏻‍🤝‍👨🏾' => 'kézen fogva álló férfi és nő: világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏻‍🤝‍👨🏿' => 'kézen fogva álló férfi és nő: világos bőrtónus és sötét bőrtónus',
    '👩🏼‍🤝‍👨🏻' => 'kézen fogva álló férfi és nő: közepesen világos bőrtónus és világos bőrtónus',
    '👩🏼‍🤝‍👨🏽' => 'kézen fogva álló férfi és nő: közepesen világos bőrtónus és közepes bőrtónus',
    '👩🏼‍🤝‍👨🏾' => 'kézen fogva álló férfi és nő: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👩🏼‍🤝‍👨🏿' => 'kézen fogva álló férfi és nő: közepesen világos bőrtónus és sötét bőrtónus',
    '👩🏽‍🤝‍👨🏻' => 'kézen fogva álló férfi és nő: közepes bőrtónus és világos bőrtónus',
    '👩🏽‍🤝‍👨🏼' => 'kézen fogva álló férfi és nő: közepes bőrtónus és közepesen világos bőrtónus',
    '👩🏽‍🤝‍👨🏾' => 'kézen fogva álló férfi és nő: közepes bőrtónus és közepesen sötét bőrtónus',
    '👩🏽‍🤝‍👨🏿' => 'kézen fogva álló férfi és nő: közepes bőrtónus és sötét bőrtónus',
    '👩🏾‍🤝‍👨🏻' => 'kézen fogva álló férfi és nő: közepesen sötét bőrtónus és világos bőrtónus',
    '👩🏾‍🤝‍👨🏼' => 'kézen fogva álló férfi és nő: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏾‍🤝‍👨🏽' => 'kézen fogva álló férfi és nő: közepesen sötét bőrtónus és közepes bőrtónus',
    '👩🏾‍🤝‍👨🏿' => 'kézen fogva álló férfi és nő: közepesen sötét bőrtónus és sötét bőrtónus',
    '👩🏿‍🤝‍👨🏻' => 'kézen fogva álló férfi és nő: sötét bőrtónus és világos bőrtónus',
    '👩🏿‍🤝‍👨🏼' => 'kézen fogva álló férfi és nő: sötét bőrtónus és közepesen világos bőrtónus',
    '👩🏿‍🤝‍👨🏽' => 'kézen fogva álló férfi és nő: sötét bőrtónus és közepes bőrtónus',
    '👩🏿‍🤝‍👨🏾' => 'kézen fogva álló férfi és nő: sötét bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍🤝‍👨🏼' => 'kézen fogva álló férfiak: világos bőrtónus és közepesen világos bőrtónus',
    '👨🏻‍🤝‍👨🏽' => 'kézen fogva álló férfiak: világos bőrtónus és közepes bőrtónus',
    '👨🏻‍🤝‍👨🏾' => 'kézen fogva álló férfiak: világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏻‍🤝‍👨🏿' => 'kézen fogva álló férfiak: világos bőrtónus és sötét bőrtónus',
    '👨🏼‍🤝‍👨🏻' => 'kézen fogva álló férfiak: közepesen világos bőrtónus és világos bőrtónus',
    '👨🏼‍🤝‍👨🏽' => 'kézen fogva álló férfiak: közepesen világos bőrtónus és közepes bőrtónus',
    '👨🏼‍🤝‍👨🏾' => 'kézen fogva álló férfiak: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '👨🏼‍🤝‍👨🏿' => 'kézen fogva álló férfiak: közepesen világos bőrtónus és sötét bőrtónus',
    '👨🏽‍🤝‍👨🏻' => 'kézen fogva álló férfiak: közepes bőrtónus és világos bőrtónus',
    '👨🏽‍🤝‍👨🏼' => 'kézen fogva álló férfiak: közepes bőrtónus és közepesen világos bőrtónus',
    '👨🏽‍🤝‍👨🏾' => 'kézen fogva álló férfiak: közepes bőrtónus és közepesen sötét bőrtónus',
    '👨🏽‍🤝‍👨🏿' => 'kézen fogva álló férfiak: közepes bőrtónus és sötét bőrtónus',
    '👨🏾‍🤝‍👨🏻' => 'kézen fogva álló férfiak: közepesen sötét bőrtónus és világos bőrtónus',
    '👨🏾‍🤝‍👨🏼' => 'kézen fogva álló férfiak: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏾‍🤝‍👨🏽' => 'kézen fogva álló férfiak: közepesen sötét bőrtónus és közepes bőrtónus',
    '👨🏾‍🤝‍👨🏿' => 'kézen fogva álló férfiak: közepesen sötét bőrtónus és sötét bőrtónus',
    '👨🏿‍🤝‍👨🏻' => 'kézen fogva álló férfiak: sötét bőrtónus és világos bőrtónus',
    '👨🏿‍🤝‍👨🏼' => 'kézen fogva álló férfiak: sötét bőrtónus és közepesen világos bőrtónus',
    '👨🏿‍🤝‍👨🏽' => 'kézen fogva álló férfiak: sötét bőrtónus és közepes bőrtónus',
    '👨🏿‍🤝‍👨🏾' => 'kézen fogva álló férfiak: sötét bőrtónus és közepesen sötét bőrtónus',
    '👩‍❤‍💋‍👨' => 'csók: nő és férfi',
    '👨‍❤‍💋‍👨' => 'csók: férfi és férfi',
    '👩‍❤‍💋‍👩' => 'csók: nő és nő',
    '👨‍👩‍👧‍👦' => 'család: férfi, nő, lány és fiú',
    '👨‍👩‍👦‍👦' => 'család: férfi, nő, fiú és fiú',
    '👨‍👩‍👧‍👧' => 'család: férfi, nő, lány és lány',
    '👨‍👨‍👧‍👦' => 'család: férfi, férfi, lány és fiú',
    '👨‍👨‍👦‍👦' => 'család: férfi, férfi, fiú és fiú',
    '👨‍👨‍👧‍👧' => 'család: férfi, férfi, lány és lány',
    '👩‍👩‍👧‍👦' => 'család: nő, nő, lány és fiú',
    '👩‍👩‍👦‍👦' => 'család: nő, nő, fiú és fiú',
    '👩‍👩‍👧‍👧' => 'család: nő, nő, lány és lány',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'zászló: Anglia',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'zászló: Skócia',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'zászló: Wales',
    '🧑‍🧑‍🧒‍🧒' => 'család: felnőtt, felnőtt, gyerek, gyerek',
    '🚶🏻‍♀‍➡' => 'sétáló nő: világos bőrtónus és jobbra néző',
    '🚶🏼‍♀‍➡' => 'sétáló nő: közepesen világos bőrtónus és jobbra néző',
    '🚶🏽‍♀‍➡' => 'sétáló nő: közepes bőrtónus és jobbra néző',
    '🚶🏾‍♀‍➡' => 'sétáló nő: közepesen sötét bőrtónus és jobbra néző',
    '🚶🏿‍♀‍➡' => 'sétáló nő: sötét bőrtónus és jobbra néző',
    '🚶🏻‍♂‍➡' => 'sétáló férfi: világos bőrtónus és jobbra néző',
    '🚶🏼‍♂‍➡' => 'sétáló férfi: közepesen világos bőrtónus és jobbra néző',
    '🚶🏽‍♂‍➡' => 'sétáló férfi: közepes bőrtónus és jobbra néző',
    '🚶🏾‍♂‍➡' => 'sétáló férfi: közepesen sötét bőrtónus és jobbra néző',
    '🚶🏿‍♂‍➡' => 'sétáló férfi: sötét bőrtónus és jobbra néző',
    '🧎🏻‍♀‍➡' => 'térdelő nő: világos bőrtónus és jobbra néző',
    '🧎🏼‍♀‍➡' => 'térdelő nő: közepesen világos bőrtónus és jobbra néző',
    '🧎🏽‍♀‍➡' => 'térdelő nő: közepes bőrtónus és jobbra néző',
    '🧎🏾‍♀‍➡' => 'térdelő nő: közepesen sötét bőrtónus és jobbra néző',
    '🧎🏿‍♀‍➡' => 'térdelő nő: sötét bőrtónus és jobbra néző',
    '🧎🏻‍♂‍➡' => 'térdelő férfi: világos bőrtónus és jobbra néző',
    '🧎🏼‍♂‍➡' => 'térdelő férfi: közepesen világos bőrtónus és jobbra néző',
    '🧎🏽‍♂‍➡' => 'térdelő férfi: közepes bőrtónus és jobbra néző',
    '🧎🏾‍♂‍➡' => 'térdelő férfi: közepesen sötét bőrtónus és jobbra néző',
    '🧎🏿‍♂‍➡' => 'térdelő férfi: sötét bőrtónus és jobbra néző',
    '🧑🏻‍🦯‍➡' => 'ember fehér bottal: világos bőrtónus és jobbra néző',
    '🧑🏼‍🦯‍➡' => 'ember fehér bottal: közepesen világos bőrtónus és jobbra néző',
    '🧑🏽‍🦯‍➡' => 'ember fehér bottal: közepes bőrtónus és jobbra néző',
    '🧑🏾‍🦯‍➡' => 'ember fehér bottal: közepesen sötét bőrtónus és jobbra néző',
    '🧑🏿‍🦯‍➡' => 'ember fehér bottal: sötét bőrtónus és jobbra néző',
    '👨🏻‍🦯‍➡' => 'férfi fehér bottal: világos bőrtónus és jobbra néző',
    '👨🏼‍🦯‍➡' => 'férfi fehér bottal: közepesen világos bőrtónus és jobbra néző',
    '👨🏽‍🦯‍➡' => 'férfi fehér bottal: közepes bőrtónus és jobbra néző',
    '👨🏾‍🦯‍➡' => 'férfi fehér bottal: közepesen sötét bőrtónus és jobbra néző',
    '👨🏿‍🦯‍➡' => 'férfi fehér bottal: sötét bőrtónus és jobbra néző',
    '👩🏻‍🦯‍➡' => 'nő fehér bottal: világos bőrtónus és jobbra néző',
    '👩🏼‍🦯‍➡' => 'nő fehér bottal: közepesen világos bőrtónus és jobbra néző',
    '👩🏽‍🦯‍➡' => 'nő fehér bottal: közepes bőrtónus és jobbra néző',
    '👩🏾‍🦯‍➡' => 'nő fehér bottal: közepesen sötét bőrtónus és jobbra néző',
    '👩🏿‍🦯‍➡' => 'nő fehér bottal: sötét bőrtónus és jobbra néző',
    '🧑🏻‍🦼‍➡' => 'ember motoros kerekesszékben: világos bőrtónus és jobbra néző',
    '🧑🏼‍🦼‍➡' => 'ember motoros kerekesszékben: közepesen világos bőrtónus és jobbra néző',
    '🧑🏽‍🦼‍➡' => 'ember motoros kerekesszékben: közepes bőrtónus és jobbra néző',
    '🧑🏾‍🦼‍➡' => 'ember motoros kerekesszékben: közepesen sötét bőrtónus és jobbra néző',
    '🧑🏿‍🦼‍➡' => 'ember motoros kerekesszékben: sötét bőrtónus és jobbra néző',
    '👨🏻‍🦼‍➡' => 'férfi motoros kerekesszékben: világos bőrtónus és jobbra néző',
    '👨🏼‍🦼‍➡' => 'férfi motoros kerekesszékben: közepesen világos bőrtónus és jobbra néző',
    '👨🏽‍🦼‍➡' => 'férfi motoros kerekesszékben: közepes bőrtónus és jobbra néző',
    '👨🏾‍🦼‍➡' => 'férfi motoros kerekesszékben: közepesen sötét bőrtónus és jobbra néző',
    '👨🏿‍🦼‍➡' => 'férfi motoros kerekesszékben: sötét bőrtónus és jobbra néző',
    '👩🏻‍🦼‍➡' => 'nő motoros kerekesszékben: világos bőrtónus és jobbra néző',
    '👩🏼‍🦼‍➡' => 'nő motoros kerekesszékben: közepesen világos bőrtónus és jobbra néző',
    '👩🏽‍🦼‍➡' => 'nő motoros kerekesszékben: közepes bőrtónus és jobbra néző',
    '👩🏾‍🦼‍➡' => 'nő motoros kerekesszékben: közepesen sötét bőrtónus és jobbra néző',
    '👩🏿‍🦼‍➡' => 'nő motoros kerekesszékben: sötét bőrtónus és jobbra néző',
    '🧑🏻‍🦽‍➡' => 'kerekesszékes ember: világos bőrtónus és jobbra néző',
    '🧑🏼‍🦽‍➡' => 'kerekesszékes ember: közepesen világos bőrtónus és jobbra néző',
    '🧑🏽‍🦽‍➡' => 'kerekesszékes ember: közepes bőrtónus és jobbra néző',
    '🧑🏾‍🦽‍➡' => 'kerekesszékes ember: közepesen sötét bőrtónus és jobbra néző',
    '🧑🏿‍🦽‍➡' => 'kerekesszékes ember: sötét bőrtónus és jobbra néző',
    '👨🏻‍🦽‍➡' => 'férfi kézi hajtású kerekesszékben: világos bőrtónus és jobbra néző',
    '👨🏼‍🦽‍➡' => 'férfi kézi hajtású kerekesszékben: közepesen világos bőrtónus és jobbra néző',
    '👨🏽‍🦽‍➡' => 'férfi kézi hajtású kerekesszékben: közepes bőrtónus és jobbra néző',
    '👨🏾‍🦽‍➡' => 'férfi kézi hajtású kerekesszékben: közepesen sötét bőrtónus és jobbra néző',
    '👨🏿‍🦽‍➡' => 'férfi kézi hajtású kerekesszékben: sötét bőrtónus és jobbra néző',
    '👩🏻‍🦽‍➡' => 'nő kézi hajtású kerekesszékben: világos bőrtónus és jobbra néző',
    '👩🏼‍🦽‍➡' => 'nő kézi hajtású kerekesszékben: közepesen világos bőrtónus és jobbra néző',
    '👩🏽‍🦽‍➡' => 'nő kézi hajtású kerekesszékben: közepes bőrtónus és jobbra néző',
    '👩🏾‍🦽‍➡' => 'nő kézi hajtású kerekesszékben: közepesen sötét bőrtónus és jobbra néző',
    '👩🏿‍🦽‍➡' => 'nő kézi hajtású kerekesszékben: sötét bőrtónus és jobbra néző',
    '🏃🏻‍♀‍➡' => 'futó nő: világos bőrtónus és jobbra néző',
    '🏃🏼‍♀‍➡' => 'futó nő: közepesen világos bőrtónus és jobbra néző',
    '🏃🏽‍♀‍➡' => 'futó nő: közepes bőrtónus és jobbra néző',
    '🏃🏾‍♀‍➡' => 'futó nő: közepesen sötét bőrtónus és jobbra néző',
    '🏃🏿‍♀‍➡' => 'futó nő: sötét bőrtónus és jobbra néző',
    '🏃🏻‍♂‍➡' => 'futó férfi: világos bőrtónus és jobbra néző',
    '🏃🏼‍♂‍➡' => 'futó férfi: közepesen világos bőrtónus és jobbra néző',
    '🏃🏽‍♂‍➡' => 'futó férfi: közepes bőrtónus és jobbra néző',
    '🏃🏾‍♂‍➡' => 'futó férfi: közepesen sötét bőrtónus és jobbra néző',
    '🏃🏿‍♂‍➡' => 'futó férfi: sötét bőrtónus és jobbra néző',
    '🫱🏻‍🫲🏼' => 'kézfogás: világos bőrtónus és közepesen világos bőrtónus',
    '🫱🏻‍🫲🏽' => 'kézfogás: világos bőrtónus és közepes bőrtónus',
    '🫱🏻‍🫲🏾' => 'kézfogás: világos bőrtónus és közepesen sötét bőrtónus',
    '🫱🏻‍🫲🏿' => 'kézfogás: világos bőrtónus és sötét bőrtónus',
    '🫱🏼‍🫲🏻' => 'kézfogás: közepesen világos bőrtónus és világos bőrtónus',
    '🫱🏼‍🫲🏽' => 'kézfogás: közepesen világos bőrtónus és közepes bőrtónus',
    '🫱🏼‍🫲🏾' => 'kézfogás: közepesen világos bőrtónus és közepesen sötét bőrtónus',
    '🫱🏼‍🫲🏿' => 'kézfogás: közepesen világos bőrtónus és sötét bőrtónus',
    '🫱🏽‍🫲🏻' => 'kézfogás: közepes bőrtónus és világos bőrtónus',
    '🫱🏽‍🫲🏼' => 'kézfogás: közepes bőrtónus és közepesen világos bőrtónus',
    '🫱🏽‍🫲🏾' => 'kézfogás: közepes bőrtónus és közepesen sötét bőrtónus',
    '🫱🏽‍🫲🏿' => 'kézfogás: közepes bőrtónus és sötét bőrtónus',
    '🫱🏾‍🫲🏻' => 'kézfogás: közepesen sötét bőrtónus és világos bőrtónus',
    '🫱🏾‍🫲🏼' => 'kézfogás: közepesen sötét bőrtónus és közepesen világos bőrtónus',
    '🫱🏾‍🫲🏽' => 'kézfogás: közepesen sötét bőrtónus és közepes bőrtónus',
    '🫱🏾‍🫲🏿' => 'kézfogás: közepesen sötét bőrtónus és sötét bőrtónus',
    '🫱🏿‍🫲🏻' => 'kézfogás: sötét bőrtónus és világos bőrtónus',
    '🫱🏿‍🫲🏼' => 'kézfogás: sötét bőrtónus és közepesen világos bőrtónus',
    '🫱🏿‍🫲🏽' => 'kézfogás: sötét bőrtónus és közepes bőrtónus',
    '🫱🏿‍🫲🏾' => 'kézfogás: sötét bőrtónus és közepesen sötét bőrtónus',
    '🚶‍♀‍➡' => 'sétáló nő: jobbra néző',
    '🚶‍♂‍➡' => 'sétáló férfi: jobbra néző',
    '🧎‍♀‍➡' => 'térdelő nő: jobbra néző',
    '🧎‍♂‍➡' => 'térdelő férfi: jobbra néző',
    '🧑‍🦯‍➡' => 'ember fehér bottal: jobbra néző',
    '👨‍🦯‍➡' => 'férfi fehér bottal: jobbra néző',
    '👩‍🦯‍➡' => 'nő fehér bottal: jobbra néző',
    '🧑‍🦼‍➡' => 'ember motoros kerekesszékben: jobbra néző',
    '👨‍🦼‍➡' => 'férfi motoros kerekesszékben: jobbra néző',
    '👩‍🦼‍➡' => 'nő motoros kerekesszékben: jobbra néző',
    '🧑‍🦽‍➡' => 'kerekesszékes ember: jobbra néző',
    '👨‍🦽‍➡' => 'férfi kézi hajtású kerekesszékben: jobbra néző',
    '👩‍🦽‍➡' => 'nő kézi hajtású kerekesszékben: jobbra néző',
    '🏃‍♀‍➡' => 'futó nő: jobbra néző',
    '🏃‍♂‍➡' => 'futó férfi: jobbra néző',
    '👩‍❤‍👨' => 'pár szívvel: nő és férfi',
    '👨‍❤‍👨' => 'pár szívvel: férfi és férfi',
    '👩‍❤‍👩' => 'pár szívvel: nő és nő',
    '👨‍👩‍👦' => 'család: férfi, nő és fiú',
    '👨‍👩‍👧' => 'család: férfi, nő és lány',
    '👨‍👨‍👦' => 'család: férfi, férfi és fiú',
    '👨‍👨‍👧' => 'család: férfi, férfi és lány',
    '👩‍👩‍👦' => 'család: nő, nő és fiú',
    '👩‍👩‍👧' => 'család: nő, nő és lány',
    '👨‍👦‍👦' => 'család: férfi, fiú és fiú',
    '👨‍👧‍👦' => 'család: férfi, lány és fiú',
    '👨‍👧‍👧' => 'család: férfi, lány és lány',
    '👩‍👦‍👦' => 'család: nő, fiú és fiú',
    '👩‍👧‍👦' => 'család: nő, lány és fiú',
    '👩‍👧‍👧' => 'család: nő, lány és lány',
    '🧑‍🤝‍🧑' => 'egymás kezét fogó emberek',
    '🧑‍🧑‍🧒' => 'család: felnőtt, felnőtt, gyerek',
    '🧑‍🧒‍🧒' => 'család: felnőtt, gyerek, gyerek',
    '🧑🏻‍🦰' => 'felnőtt: világos bőrtónus és vörös haj',
    '🧑🏼‍🦰' => 'felnőtt: közepesen világos bőrtónus és vörös haj',
    '🧑🏽‍🦰' => 'felnőtt: közepes bőrtónus és vörös haj',
    '🧑🏾‍🦰' => 'felnőtt: közepesen sötét bőrtónus és vörös haj',
    '🧑🏿‍🦰' => 'felnőtt: sötét bőrtónus és vörös haj',
    '🧑🏻‍🦱' => 'felnőtt: világos bőrtónus és göndör haj',
    '🧑🏼‍🦱' => 'felnőtt: közepesen világos bőrtónus és göndör haj',
    '🧑🏽‍🦱' => 'felnőtt: közepes bőrtónus és göndör haj',
    '🧑🏾‍🦱' => 'felnőtt: közepesen sötét bőrtónus és göndör haj',
    '🧑🏿‍🦱' => 'felnőtt: sötét bőrtónus és göndör haj',
    '🧑🏻‍🦳' => 'felnőtt: világos bőrtónus és ősz haj',
    '🧑🏼‍🦳' => 'felnőtt: közepesen világos bőrtónus és ősz haj',
    '🧑🏽‍🦳' => 'felnőtt: közepes bőrtónus és ősz haj',
    '🧑🏾‍🦳' => 'felnőtt: közepesen sötét bőrtónus és ősz haj',
    '🧑🏿‍🦳' => 'felnőtt: sötét bőrtónus és ősz haj',
    '🧑🏻‍🦲' => 'felnőtt: világos bőrtónus és kopasz',
    '🧑🏼‍🦲' => 'felnőtt: közepesen világos bőrtónus és kopasz',
    '🧑🏽‍🦲' => 'felnőtt: közepes bőrtónus és kopasz',
    '🧑🏾‍🦲' => 'felnőtt: közepesen sötét bőrtónus és kopasz',
    '🧑🏿‍🦲' => 'felnőtt: sötét bőrtónus és kopasz',
    '🧑🏻‍🩰' => 'balett-táncos: világos bőrtónus',
    '🧑🏼‍🩰' => 'balett-táncos: közepesen világos bőrtónus',
    '🧑🏽‍🩰' => 'balett-táncos: közepes bőrtónus',
    '🧑🏾‍🩰' => 'balett-táncos: közepesen sötét bőrtónus',
    '🧑🏿‍🩰' => 'balett-táncos: sötét bőrtónus',
    '🧔🏻‍♂' => 'férfi: világos bőrtónus és szakáll',
    '🧔🏼‍♂' => 'férfi: közepesen világos bőrtónus és szakáll',
    '🧔🏽‍♂' => 'férfi: közepes bőrtónus és szakáll',
    '🧔🏾‍♂' => 'férfi: közepesen sötét bőrtónus és szakáll',
    '🧔🏿‍♂' => 'férfi: sötét bőrtónus és szakáll',
    '👱🏻‍♂' => 'szőke férfi: világos bőrtónus',
    '👱🏼‍♂' => 'szőke férfi: közepesen világos bőrtónus',
    '👱🏽‍♂' => 'szőke férfi: közepes bőrtónus',
    '👱🏾‍♂' => 'szőke férfi: közepesen sötét bőrtónus',
    '👱🏿‍♂' => 'szőke férfi: sötét bőrtónus',
    '👨🏻‍🦰' => 'férfi: világos bőrtónus és vörös haj',
    '👨🏼‍🦰' => 'férfi: közepesen világos bőrtónus és vörös haj',
    '👨🏽‍🦰' => 'férfi: közepes bőrtónus és vörös haj',
    '👨🏾‍🦰' => 'férfi: közepesen sötét bőrtónus és vörös haj',
    '👨🏿‍🦰' => 'férfi: sötét bőrtónus és vörös haj',
    '👨🏻‍🦱' => 'férfi: világos bőrtónus és göndör haj',
    '👨🏼‍🦱' => 'férfi: közepesen világos bőrtónus és göndör haj',
    '👨🏽‍🦱' => 'férfi: közepes bőrtónus és göndör haj',
    '👨🏾‍🦱' => 'férfi: közepesen sötét bőrtónus és göndör haj',
    '👨🏿‍🦱' => 'férfi: sötét bőrtónus és göndör haj',
    '👨🏻‍🦳' => 'férfi: világos bőrtónus és ősz haj',
    '👨🏼‍🦳' => 'férfi: közepesen világos bőrtónus és ősz haj',
    '👨🏽‍🦳' => 'férfi: közepes bőrtónus és ősz haj',
    '👨🏾‍🦳' => 'férfi: közepesen sötét bőrtónus és ősz haj',
    '👨🏿‍🦳' => 'férfi: sötét bőrtónus és ősz haj',
    '👨🏻‍🦲' => 'férfi: világos bőrtónus és kopasz',
    '👨🏼‍🦲' => 'férfi: közepesen világos bőrtónus és kopasz',
    '👨🏽‍🦲' => 'férfi: közepes bőrtónus és kopasz',
    '👨🏾‍🦲' => 'férfi: közepesen sötét bőrtónus és kopasz',
    '👨🏿‍🦲' => 'férfi: sötét bőrtónus és kopasz',
    '🧔🏻‍♀' => 'nő: világos bőrtónus és szakáll',
    '🧔🏼‍♀' => 'nő: közepesen világos bőrtónus és szakáll',
    '🧔🏽‍♀' => 'nő: közepes bőrtónus és szakáll',
    '🧔🏾‍♀' => 'nő: közepesen sötét bőrtónus és szakáll',
    '🧔🏿‍♀' => 'nő: sötét bőrtónus és szakáll',
    '👱🏻‍♀' => 'szőke nő: világos bőrtónus',
    '👱🏼‍♀' => 'szőke nő: közepesen világos bőrtónus',
    '👱🏽‍♀' => 'szőke nő: közepes bőrtónus',
    '👱🏾‍♀' => 'szőke nő: közepesen sötét bőrtónus',
    '👱🏿‍♀' => 'szőke nő: sötét bőrtónus',
    '👩🏻‍🦰' => 'nő: világos bőrtónus és vörös haj',
    '👩🏼‍🦰' => 'nő: közepesen világos bőrtónus és vörös haj',
    '👩🏽‍🦰' => 'nő: közepes bőrtónus és vörös haj',
    '👩🏾‍🦰' => 'nő: közepesen sötét bőrtónus és vörös haj',
    '👩🏿‍🦰' => 'nő: sötét bőrtónus és vörös haj',
    '👩🏻‍🦱' => 'nő: világos bőrtónus és göndör haj',
    '👩🏼‍🦱' => 'nő: közepesen világos bőrtónus és göndör haj',
    '👩🏽‍🦱' => 'nő: közepes bőrtónus és göndör haj',
    '👩🏾‍🦱' => 'nő: közepesen sötét bőrtónus és göndör haj',
    '👩🏿‍🦱' => 'nő: sötét bőrtónus és göndör haj',
    '👩🏻‍🦳' => 'nő: világos bőrtónus és ősz haj',
    '👩🏼‍🦳' => 'nő: közepesen világos bőrtónus és ősz haj',
    '👩🏽‍🦳' => 'nő: közepes bőrtónus és ősz haj',
    '👩🏾‍🦳' => 'nő: közepesen sötét bőrtónus és ősz haj',
    '👩🏿‍🦳' => 'nő: sötét bőrtónus és ősz haj',
    '👩🏻‍🦲' => 'nő: világos bőrtónus és kopasz',
    '👩🏼‍🦲' => 'nő: közepesen világos bőrtónus és kopasz',
    '👩🏽‍🦲' => 'nő: közepes bőrtónus és kopasz',
    '👩🏾‍🦲' => 'nő: közepesen sötét bőrtónus és kopasz',
    '👩🏿‍🦲' => 'nő: sötét bőrtónus és kopasz',
    '🙍🏻‍♂' => 'rosszalló férfi: világos bőrtónus',
    '🙍🏼‍♂' => 'rosszalló férfi: közepesen világos bőrtónus',
    '🙍🏽‍♂' => 'rosszalló férfi: közepes bőrtónus',
    '🙍🏾‍♂' => 'rosszalló férfi: közepesen sötét bőrtónus',
    '🙍🏿‍♂' => 'rosszalló férfi: sötét bőrtónus',
    '🙍🏻‍♀' => 'rosszalló nő: világos bőrtónus',
    '🙍🏼‍♀' => 'rosszalló nő: közepesen világos bőrtónus',
    '🙍🏽‍♀' => 'rosszalló nő: közepes bőrtónus',
    '🙍🏾‍♀' => 'rosszalló nő: közepesen sötét bőrtónus',
    '🙍🏿‍♀' => 'rosszalló nő: sötét bőrtónus',
    '🙎🏻‍♂' => 'duzzogó férfi: világos bőrtónus',
    '🙎🏼‍♂' => 'duzzogó férfi: közepesen világos bőrtónus',
    '🙎🏽‍♂' => 'duzzogó férfi: közepes bőrtónus',
    '🙎🏾‍♂' => 'duzzogó férfi: közepesen sötét bőrtónus',
    '🙎🏿‍♂' => 'duzzogó férfi: sötét bőrtónus',
    '🙎🏻‍♀' => 'duzzogó nő: világos bőrtónus',
    '🙎🏼‍♀' => 'duzzogó nő: közepesen világos bőrtónus',
    '🙎🏽‍♀' => 'duzzogó nő: közepes bőrtónus',
    '🙎🏾‍♀' => 'duzzogó nő: közepesen sötét bőrtónus',
    '🙎🏿‍♀' => 'duzzogó nő: sötét bőrtónus',
    '🙅🏻‍♂' => 'tiltást mutató férfi: világos bőrtónus',
    '🙅🏼‍♂' => 'tiltást mutató férfi: közepesen világos bőrtónus',
    '🙅🏽‍♂' => 'tiltást mutató férfi: közepes bőrtónus',
    '🙅🏾‍♂' => 'tiltást mutató férfi: közepesen sötét bőrtónus',
    '🙅🏿‍♂' => 'tiltást mutató férfi: sötét bőrtónus',
    '🙅🏻‍♀' => 'tiltást mutató nő: világos bőrtónus',
    '🙅🏼‍♀' => 'tiltást mutató nő: közepesen világos bőrtónus',
    '🙅🏽‍♀' => 'tiltást mutató nő: közepes bőrtónus',
    '🙅🏾‍♀' => 'tiltást mutató nő: közepesen sötét bőrtónus',
    '🙅🏿‍♀' => 'tiltást mutató nő: sötét bőrtónus',
    '🙆🏻‍♂' => 'beleegyezést mutató férfi: világos bőrtónus',
    '🙆🏼‍♂' => 'beleegyezést mutató férfi: közepesen világos bőrtónus',
    '🙆🏽‍♂' => 'beleegyezést mutató férfi: közepes bőrtónus',
    '🙆🏾‍♂' => 'beleegyezést mutató férfi: közepesen sötét bőrtónus',
    '🙆🏿‍♂' => 'beleegyezést mutató férfi: sötét bőrtónus',
    '🙆🏻‍♀' => 'beleegyezést mutató nő: világos bőrtónus',
    '🙆🏼‍♀' => 'beleegyezést mutató nő: közepesen világos bőrtónus',
    '🙆🏽‍♀' => 'beleegyezést mutató nő: közepes bőrtónus',
    '🙆🏾‍♀' => 'beleegyezést mutató nő: közepesen sötét bőrtónus',
    '🙆🏿‍♀' => 'beleegyezést mutató nő: sötét bőrtónus',
    '💁🏻‍♂' => 'nyitott tenyerű férfi: világos bőrtónus',
    '💁🏼‍♂' => 'nyitott tenyerű férfi: közepesen világos bőrtónus',
    '💁🏽‍♂' => 'nyitott tenyerű férfi: közepes bőrtónus',
    '💁🏾‍♂' => 'nyitott tenyerű férfi: közepesen sötét bőrtónus',
    '💁🏿‍♂' => 'nyitott tenyerű férfi: sötét bőrtónus',
    '💁🏻‍♀' => 'nyitott tenyerű nő: világos bőrtónus',
    '💁🏼‍♀' => 'nyitott tenyerű nő: közepesen világos bőrtónus',
    '💁🏽‍♀' => 'nyitott tenyerű nő: közepes bőrtónus',
    '💁🏾‍♀' => 'nyitott tenyerű nő: közepesen sötét bőrtónus',
    '💁🏿‍♀' => 'nyitott tenyerű nő: sötét bőrtónus',
    '🙋🏻‍♂' => 'vidáman intő férfi: világos bőrtónus',
    '🙋🏼‍♂' => 'vidáman intő férfi: közepesen világos bőrtónus',
    '🙋🏽‍♂' => 'vidáman intő férfi: közepes bőrtónus',
    '🙋🏾‍♂' => 'vidáman intő férfi: közepesen sötét bőrtónus',
    '🙋🏿‍♂' => 'vidáman intő férfi: sötét bőrtónus',
    '🙋🏻‍♀' => 'vidáman intő nő: világos bőrtónus',
    '🙋🏼‍♀' => 'vidáman intő nő: közepesen világos bőrtónus',
    '🙋🏽‍♀' => 'vidáman intő nő: közepes bőrtónus',
    '🙋🏾‍♀' => 'vidáman intő nő: közepesen sötét bőrtónus',
    '🙋🏿‍♀' => 'vidáman intő nő: sötét bőrtónus',
    '🧏🏻‍♂' => 'siket férfi: világos bőrtónus',
    '🧏🏼‍♂' => 'siket férfi: közepesen világos bőrtónus',
    '🧏🏽‍♂' => 'siket férfi: közepes bőrtónus',
    '🧏🏾‍♂' => 'siket férfi: közepesen sötét bőrtónus',
    '🧏🏿‍♂' => 'siket férfi: sötét bőrtónus',
    '🧏🏻‍♀' => 'siket nő: világos bőrtónus',
    '🧏🏼‍♀' => 'siket nő: közepesen világos bőrtónus',
    '🧏🏽‍♀' => 'siket nő: közepes bőrtónus',
    '🧏🏾‍♀' => 'siket nő: közepesen sötét bőrtónus',
    '🧏🏿‍♀' => 'siket nő: sötét bőrtónus',
    '🙇🏻‍♂' => 'meghajló férfi: világos bőrtónus',
    '🙇🏼‍♂' => 'meghajló férfi: közepesen világos bőrtónus',
    '🙇🏽‍♂' => 'meghajló férfi: közepes bőrtónus',
    '🙇🏾‍♂' => 'meghajló férfi: közepesen sötét bőrtónus',
    '🙇🏿‍♂' => 'meghajló férfi: sötét bőrtónus',
    '🙇🏻‍♀' => 'meghajló nő: világos bőrtónus',
    '🙇🏼‍♀' => 'meghajló nő: közepesen világos bőrtónus',
    '🙇🏽‍♀' => 'meghajló nő: közepes bőrtónus',
    '🙇🏾‍♀' => 'meghajló nő: közepesen sötét bőrtónus',
    '🙇🏿‍♀' => 'meghajló nő: sötét bőrtónus',
    '🤦🏻‍♂' => 'arcát a kezébe temető férfi: világos bőrtónus',
    '🤦🏼‍♂' => 'arcát a kezébe temető férfi: közepesen világos bőrtónus',
    '🤦🏽‍♂' => 'arcát a kezébe temető férfi: közepes bőrtónus',
    '🤦🏾‍♂' => 'arcát a kezébe temető férfi: közepesen sötét bőrtónus',
    '🤦🏿‍♂' => 'arcát a kezébe temető férfi: sötét bőrtónus',
    '🤦🏻‍♀' => 'arcát a kezébe temető nő: világos bőrtónus',
    '🤦🏼‍♀' => 'arcát a kezébe temető nő: közepesen világos bőrtónus',
    '🤦🏽‍♀' => 'arcát a kezébe temető nő: közepes bőrtónus',
    '🤦🏾‍♀' => 'arcát a kezébe temető nő: közepesen sötét bőrtónus',
    '🤦🏿‍♀' => 'arcát a kezébe temető nő: sötét bőrtónus',
    '🤷🏻‍♂' => 'vállrándító férfi: világos bőrtónus',
    '🤷🏼‍♂' => 'vállrándító férfi: közepesen világos bőrtónus',
    '🤷🏽‍♂' => 'vállrándító férfi: közepes bőrtónus',
    '🤷🏾‍♂' => 'vállrándító férfi: közepesen sötét bőrtónus',
    '🤷🏿‍♂' => 'vállrándító férfi: sötét bőrtónus',
    '🤷🏻‍♀' => 'vállrándító nő: világos bőrtónus',
    '🤷🏼‍♀' => 'vállrándító nő: közepesen világos bőrtónus',
    '🤷🏽‍♀' => 'vállrándító nő: közepes bőrtónus',
    '🤷🏾‍♀' => 'vállrándító nő: közepesen sötét bőrtónus',
    '🤷🏿‍♀' => 'vállrándító nő: sötét bőrtónus',
    '🧑🏻‍⚕' => 'egészségügyi dolgozó: világos bőrtónus',
    '🧑🏼‍⚕' => 'egészségügyi dolgozó: közepesen világos bőrtónus',
    '🧑🏽‍⚕' => 'egészségügyi dolgozó: közepes bőrtónus',
    '🧑🏾‍⚕' => 'egészségügyi dolgozó: közepesen sötét bőrtónus',
    '🧑🏿‍⚕' => 'egészségügyi dolgozó: sötét bőrtónus',
    '👨🏻‍⚕' => 'doktor: világos bőrtónus',
    '👨🏼‍⚕' => 'doktor: közepesen világos bőrtónus',
    '👨🏽‍⚕' => 'doktor: közepes bőrtónus',
    '👨🏾‍⚕' => 'doktor: közepesen sötét bőrtónus',
    '👨🏿‍⚕' => 'doktor: sötét bőrtónus',
    '👩🏻‍⚕' => 'doktornő: világos bőrtónus',
    '👩🏼‍⚕' => 'doktornő: közepesen világos bőrtónus',
    '👩🏽‍⚕' => 'doktornő: közepes bőrtónus',
    '👩🏾‍⚕' => 'doktornő: közepesen sötét bőrtónus',
    '👩🏿‍⚕' => 'doktornő: sötét bőrtónus',
    '🧑🏻‍🎓' => 'diák: világos bőrtónus',
    '🧑🏼‍🎓' => 'diák: közepesen világos bőrtónus',
    '🧑🏽‍🎓' => 'diák: közepes bőrtónus',
    '🧑🏾‍🎓' => 'diák: közepesen sötét bőrtónus',
    '🧑🏿‍🎓' => 'diák: sötét bőrtónus',
    '👨🏻‍🎓' => 'férfi tanuló: világos bőrtónus',
    '👨🏼‍🎓' => 'férfi tanuló: közepesen világos bőrtónus',
    '👨🏽‍🎓' => 'férfi tanuló: közepes bőrtónus',
    '👨🏾‍🎓' => 'férfi tanuló: közepesen sötét bőrtónus',
    '👨🏿‍🎓' => 'férfi tanuló: sötét bőrtónus',
    '👩🏻‍🎓' => 'női tanuló: világos bőrtónus',
    '👩🏼‍🎓' => 'női tanuló: közepesen világos bőrtónus',
    '👩🏽‍🎓' => 'női tanuló: közepes bőrtónus',
    '👩🏾‍🎓' => 'női tanuló: közepesen sötét bőrtónus',
    '👩🏿‍🎓' => 'női tanuló: sötét bőrtónus',
    '🧑🏻‍🏫' => 'tanár / tanárnő: világos bőrtónus',
    '🧑🏼‍🏫' => 'tanár / tanárnő: közepesen világos bőrtónus',
    '🧑🏽‍🏫' => 'tanár / tanárnő: közepes bőrtónus',
    '🧑🏾‍🏫' => 'tanár / tanárnő: közepesen sötét bőrtónus',
    '🧑🏿‍🏫' => 'tanár / tanárnő: sötét bőrtónus',
    '👨🏻‍🏫' => 'tanár: világos bőrtónus',
    '👨🏼‍🏫' => 'tanár: közepesen világos bőrtónus',
    '👨🏽‍🏫' => 'tanár: közepes bőrtónus',
    '👨🏾‍🏫' => 'tanár: közepesen sötét bőrtónus',
    '👨🏿‍🏫' => 'tanár: sötét bőrtónus',
    '👩🏻‍🏫' => 'tanárnő: világos bőrtónus',
    '👩🏼‍🏫' => 'tanárnő: közepesen világos bőrtónus',
    '👩🏽‍🏫' => 'tanárnő: közepes bőrtónus',
    '👩🏾‍🏫' => 'tanárnő: közepesen sötét bőrtónus',
    '👩🏿‍🏫' => 'tanárnő: sötét bőrtónus',
    '🧑🏻‍⚖' => 'bíró / bírónő: világos bőrtónus',
    '🧑🏼‍⚖' => 'bíró / bírónő: közepesen világos bőrtónus',
    '🧑🏽‍⚖' => 'bíró / bírónő: közepes bőrtónus',
    '🧑🏾‍⚖' => 'bíró / bírónő: közepesen sötét bőrtónus',
    '🧑🏿‍⚖' => 'bíró / bírónő: sötét bőrtónus',
    '👨🏻‍⚖' => 'bíró: világos bőrtónus',
    '👨🏼‍⚖' => 'bíró: közepesen világos bőrtónus',
    '👨🏽‍⚖' => 'bíró: közepes bőrtónus',
    '👨🏾‍⚖' => 'bíró: közepesen sötét bőrtónus',
    '👨🏿‍⚖' => 'bíró: sötét bőrtónus',
    '👩🏻‍⚖' => 'bírónő: világos bőrtónus',
    '👩🏼‍⚖' => 'bírónő: közepesen világos bőrtónus',
    '👩🏽‍⚖' => 'bírónő: közepes bőrtónus',
    '👩🏾‍⚖' => 'bírónő: közepesen sötét bőrtónus',
    '👩🏿‍⚖' => 'bírónő: sötét bőrtónus',
    '🧑🏻‍🌾' => 'gazda / női gazda: világos bőrtónus',
    '🧑🏼‍🌾' => 'gazda / női gazda: közepesen világos bőrtónus',
    '🧑🏽‍🌾' => 'gazda / női gazda: közepes bőrtónus',
    '🧑🏾‍🌾' => 'gazda / női gazda: közepesen sötét bőrtónus',
    '🧑🏿‍🌾' => 'gazda / női gazda: sötét bőrtónus',
    '👨🏻‍🌾' => 'gazda: világos bőrtónus',
    '👨🏼‍🌾' => 'gazda: közepesen világos bőrtónus',
    '👨🏽‍🌾' => 'gazda: közepes bőrtónus',
    '👨🏾‍🌾' => 'gazda: közepesen sötét bőrtónus',
    '👨🏿‍🌾' => 'gazda: sötét bőrtónus',
    '👩🏻‍🌾' => 'női gazda: világos bőrtónus',
    '👩🏼‍🌾' => 'női gazda: közepesen világos bőrtónus',
    '👩🏽‍🌾' => 'női gazda: közepes bőrtónus',
    '👩🏾‍🌾' => 'női gazda: közepesen sötét bőrtónus',
    '👩🏿‍🌾' => 'női gazda: sötét bőrtónus',
    '🧑🏻‍🍳' => 'szakács / szakácsnő: világos bőrtónus',
    '🧑🏼‍🍳' => 'szakács / szakácsnő: közepesen világos bőrtónus',
    '🧑🏽‍🍳' => 'szakács / szakácsnő: közepes bőrtónus',
    '🧑🏾‍🍳' => 'szakács / szakácsnő: közepesen sötét bőrtónus',
    '🧑🏿‍🍳' => 'szakács / szakácsnő: sötét bőrtónus',
    '👨🏻‍🍳' => 'szakács: világos bőrtónus',
    '👨🏼‍🍳' => 'szakács: közepesen világos bőrtónus',
    '👨🏽‍🍳' => 'szakács: közepes bőrtónus',
    '👨🏾‍🍳' => 'szakács: közepesen sötét bőrtónus',
    '👨🏿‍🍳' => 'szakács: sötét bőrtónus',
    '👩🏻‍🍳' => 'szakácsnő: világos bőrtónus',
    '👩🏼‍🍳' => 'szakácsnő: közepesen világos bőrtónus',
    '👩🏽‍🍳' => 'szakácsnő: közepes bőrtónus',
    '👩🏾‍🍳' => 'szakácsnő: közepesen sötét bőrtónus',
    '👩🏿‍🍳' => 'szakácsnő: sötét bőrtónus',
    '🧑🏻‍🔧' => 'szerelő / szerelőnő: világos bőrtónus',
    '🧑🏼‍🔧' => 'szerelő / szerelőnő: közepesen világos bőrtónus',
    '🧑🏽‍🔧' => 'szerelő / szerelőnő: közepes bőrtónus',
    '🧑🏾‍🔧' => 'szerelő / szerelőnő: közepesen sötét bőrtónus',
    '🧑🏿‍🔧' => 'szerelő / szerelőnő: sötét bőrtónus',
    '👨🏻‍🔧' => 'szerelő: világos bőrtónus',
    '👨🏼‍🔧' => 'szerelő: közepesen világos bőrtónus',
    '👨🏽‍🔧' => 'szerelő: közepes bőrtónus',
    '👨🏾‍🔧' => 'szerelő: közepesen sötét bőrtónus',
    '👨🏿‍🔧' => 'szerelő: sötét bőrtónus',
    '👩🏻‍🔧' => 'szerelőnő: világos bőrtónus',
    '👩🏼‍🔧' => 'szerelőnő: közepesen világos bőrtónus',
    '👩🏽‍🔧' => 'szerelőnő: közepes bőrtónus',
    '👩🏾‍🔧' => 'szerelőnő: közepesen sötét bőrtónus',
    '👩🏿‍🔧' => 'szerelőnő: sötét bőrtónus',
    '🧑🏻‍🏭' => 'munkás: világos bőrtónus',
    '🧑🏼‍🏭' => 'munkás: közepesen világos bőrtónus',
    '🧑🏽‍🏭' => 'munkás: közepes bőrtónus',
    '🧑🏾‍🏭' => 'munkás: közepesen sötét bőrtónus',
    '🧑🏿‍🏭' => 'munkás: sötét bőrtónus',
    '👨🏻‍🏭' => 'férfi munkás: világos bőrtónus',
    '👨🏼‍🏭' => 'férfi munkás: közepesen világos bőrtónus',
    '👨🏽‍🏭' => 'férfi munkás: közepes bőrtónus',
    '👨🏾‍🏭' => 'férfi munkás: közepesen sötét bőrtónus',
    '👨🏿‍🏭' => 'férfi munkás: sötét bőrtónus',
    '👩🏻‍🏭' => 'női munkás: világos bőrtónus',
    '👩🏼‍🏭' => 'női munkás: közepesen világos bőrtónus',
    '👩🏽‍🏭' => 'női munkás: közepes bőrtónus',
    '👩🏾‍🏭' => 'női munkás: közepesen sötét bőrtónus',
    '👩🏿‍🏭' => 'női munkás: sötét bőrtónus',
    '🧑🏻‍💼' => 'irodai dolgozó: világos bőrtónus',
    '🧑🏼‍💼' => 'irodai dolgozó: közepesen világos bőrtónus',
    '🧑🏽‍💼' => 'irodai dolgozó: közepes bőrtónus',
    '🧑🏾‍💼' => 'irodai dolgozó: közepesen sötét bőrtónus',
    '🧑🏿‍💼' => 'irodai dolgozó: sötét bőrtónus',
    '👨🏻‍💼' => 'férfi irodista: világos bőrtónus',
    '👨🏼‍💼' => 'férfi irodista: közepesen világos bőrtónus',
    '👨🏽‍💼' => 'férfi irodista: közepes bőrtónus',
    '👨🏾‍💼' => 'férfi irodista: közepesen sötét bőrtónus',
    '👨🏿‍💼' => 'férfi irodista: sötét bőrtónus',
    '👩🏻‍💼' => 'női irodista: világos bőrtónus',
    '👩🏼‍💼' => 'női irodista: közepesen világos bőrtónus',
    '👩🏽‍💼' => 'női irodista: közepes bőrtónus',
    '👩🏾‍💼' => 'női irodista: közepesen sötét bőrtónus',
    '👩🏿‍💼' => 'női irodista: sötét bőrtónus',
    '🧑🏻‍🔬' => 'tudás: világos bőrtónus',
    '🧑🏼‍🔬' => 'tudás: közepesen világos bőrtónus',
    '🧑🏽‍🔬' => 'tudás: közepes bőrtónus',
    '🧑🏾‍🔬' => 'tudás: közepesen sötét bőrtónus',
    '🧑🏿‍🔬' => 'tudás: sötét bőrtónus',
    '👨🏻‍🔬' => 'férfi tudós: világos bőrtónus',
    '👨🏼‍🔬' => 'férfi tudós: közepesen világos bőrtónus',
    '👨🏽‍🔬' => 'férfi tudós: közepes bőrtónus',
    '👨🏾‍🔬' => 'férfi tudós: közepesen sötét bőrtónus',
    '👨🏿‍🔬' => 'férfi tudós: sötét bőrtónus',
    '👩🏻‍🔬' => 'női tudós: világos bőrtónus',
    '👩🏼‍🔬' => 'női tudós: közepesen világos bőrtónus',
    '👩🏽‍🔬' => 'női tudós: közepes bőrtónus',
    '👩🏾‍🔬' => 'női tudós: közepesen sötét bőrtónus',
    '👩🏿‍🔬' => 'női tudós: sötét bőrtónus',
    '🧑🏻‍💻' => 'informatikus: világos bőrtónus',
    '🧑🏼‍💻' => 'informatikus: közepesen világos bőrtónus',
    '🧑🏽‍💻' => 'informatikus: közepes bőrtónus',
    '🧑🏾‍💻' => 'informatikus: közepesen sötét bőrtónus',
    '🧑🏿‍💻' => 'informatikus: sötét bőrtónus',
    '👨🏻‍💻' => 'programozó férfi: világos bőrtónus',
    '👨🏼‍💻' => 'programozó férfi: közepesen világos bőrtónus',
    '👨🏽‍💻' => 'programozó férfi: közepes bőrtónus',
    '👨🏾‍💻' => 'programozó férfi: közepesen sötét bőrtónus',
    '👨🏿‍💻' => 'programozó férfi: sötét bőrtónus',
    '👩🏻‍💻' => 'programozó nő: világos bőrtónus',
    '👩🏼‍💻' => 'programozó nő: közepesen világos bőrtónus',
    '👩🏽‍💻' => 'programozó nő: közepes bőrtónus',
    '👩🏾‍💻' => 'programozó nő: közepesen sötét bőrtónus',
    '👩🏿‍💻' => 'programozó nő: sötét bőrtónus',
    '🧑🏻‍🎤' => 'énekes: világos bőrtónus',
    '🧑🏼‍🎤' => 'énekes: közepesen világos bőrtónus',
    '🧑🏽‍🎤' => 'énekes: közepes bőrtónus',
    '🧑🏾‍🎤' => 'énekes: közepesen sötét bőrtónus',
    '🧑🏿‍🎤' => 'énekes: sötét bőrtónus',
    '👨🏻‍🎤' => 'férfi énekes: világos bőrtónus',
    '👨🏼‍🎤' => 'férfi énekes: közepesen világos bőrtónus',
    '👨🏽‍🎤' => 'férfi énekes: közepes bőrtónus',
    '👨🏾‍🎤' => 'férfi énekes: közepesen sötét bőrtónus',
    '👨🏿‍🎤' => 'férfi énekes: sötét bőrtónus',
    '👩🏻‍🎤' => 'női énekes: világos bőrtónus',
    '👩🏼‍🎤' => 'női énekes: közepesen világos bőrtónus',
    '👩🏽‍🎤' => 'női énekes: közepes bőrtónus',
    '👩🏾‍🎤' => 'női énekes: közepesen sötét bőrtónus',
    '👩🏿‍🎤' => 'női énekes: sötét bőrtónus',
    '🧑🏻‍🎨' => 'művész: világos bőrtónus',
    '🧑🏼‍🎨' => 'művész: közepesen világos bőrtónus',
    '🧑🏽‍🎨' => 'művész: közepes bőrtónus',
    '🧑🏾‍🎨' => 'művész: közepesen sötét bőrtónus',
    '🧑🏿‍🎨' => 'művész: sötét bőrtónus',
    '👨🏻‍🎨' => 'férfi művész: világos bőrtónus',
    '👨🏼‍🎨' => 'férfi művész: közepesen világos bőrtónus',
    '👨🏽‍🎨' => 'férfi művész: közepes bőrtónus',
    '👨🏾‍🎨' => 'férfi művész: közepesen sötét bőrtónus',
    '👨🏿‍🎨' => 'férfi művész: sötét bőrtónus',
    '👩🏻‍🎨' => 'női művész: világos bőrtónus',
    '👩🏼‍🎨' => 'női művész: közepesen világos bőrtónus',
    '👩🏽‍🎨' => 'női művész: közepes bőrtónus',
    '👩🏾‍🎨' => 'női művész: közepesen sötét bőrtónus',
    '👩🏿‍🎨' => 'női művész: sötét bőrtónus',
    '🧑🏻‍✈' => 'pilóta / pilótanő: világos bőrtónus',
    '🧑🏼‍✈' => 'pilóta / pilótanő: közepesen világos bőrtónus',
    '🧑🏽‍✈' => 'pilóta / pilótanő: közepes bőrtónus',
    '🧑🏾‍✈' => 'pilóta / pilótanő: közepesen sötét bőrtónus',
    '🧑🏿‍✈' => 'pilóta / pilótanő: sötét bőrtónus',
    '👨🏻‍✈' => 'pilóta: világos bőrtónus',
    '👨🏼‍✈' => 'pilóta: közepesen világos bőrtónus',
    '👨🏽‍✈' => 'pilóta: közepes bőrtónus',
    '👨🏾‍✈' => 'pilóta: közepesen sötét bőrtónus',
    '👨🏿‍✈' => 'pilóta: sötét bőrtónus',
    '👩🏻‍✈' => 'pilótanő: világos bőrtónus',
    '👩🏼‍✈' => 'pilótanő: közepesen világos bőrtónus',
    '👩🏽‍✈' => 'pilótanő: közepes bőrtónus',
    '👩🏾‍✈' => 'pilótanő: közepesen sötét bőrtónus',
    '👩🏿‍✈' => 'pilótanő: sötét bőrtónus',
    '🧑🏻‍🚀' => 'űrhajós / űrhajósnő: világos bőrtónus',
    '🧑🏼‍🚀' => 'űrhajós / űrhajósnő: közepesen világos bőrtónus',
    '🧑🏽‍🚀' => 'űrhajós / űrhajósnő: közepes bőrtónus',
    '🧑🏾‍🚀' => 'űrhajós / űrhajósnő: közepesen sötét bőrtónus',
    '🧑🏿‍🚀' => 'űrhajós / űrhajósnő: sötét bőrtónus',
    '👨🏻‍🚀' => 'űrhajós: világos bőrtónus',
    '👨🏼‍🚀' => 'űrhajós: közepesen világos bőrtónus',
    '👨🏽‍🚀' => 'űrhajós: közepes bőrtónus',
    '👨🏾‍🚀' => 'űrhajós: közepesen sötét bőrtónus',
    '👨🏿‍🚀' => 'űrhajós: sötét bőrtónus',
    '👩🏻‍🚀' => 'űrhajósnő: világos bőrtónus',
    '👩🏼‍🚀' => 'űrhajósnő: közepesen világos bőrtónus',
    '👩🏽‍🚀' => 'űrhajósnő: közepes bőrtónus',
    '👩🏾‍🚀' => 'űrhajósnő: közepesen sötét bőrtónus',
    '👩🏿‍🚀' => 'űrhajósnő: sötét bőrtónus',
    '🧑🏻‍🚒' => 'tűzoltó / tűzoltónő: világos bőrtónus',
    '🧑🏼‍🚒' => 'tűzoltó / tűzoltónő: közepesen világos bőrtónus',
    '🧑🏽‍🚒' => 'tűzoltó / tűzoltónő: közepes bőrtónus',
    '🧑🏾‍🚒' => 'tűzoltó / tűzoltónő: közepesen sötét bőrtónus',
    '🧑🏿‍🚒' => 'tűzoltó / tűzoltónő: sötét bőrtónus',
    '👨🏻‍🚒' => 'tűzoltó: világos bőrtónus',
    '👨🏼‍🚒' => 'tűzoltó: közepesen világos bőrtónus',
    '👨🏽‍🚒' => 'tűzoltó: közepes bőrtónus',
    '👨🏾‍🚒' => 'tűzoltó: közepesen sötét bőrtónus',
    '👨🏿‍🚒' => 'tűzoltó: sötét bőrtónus',
    '👩🏻‍🚒' => 'tűzoltónő: világos bőrtónus',
    '👩🏼‍🚒' => 'tűzoltónő: közepesen világos bőrtónus',
    '👩🏽‍🚒' => 'tűzoltónő: közepes bőrtónus',
    '👩🏾‍🚒' => 'tűzoltónő: közepesen sötét bőrtónus',
    '👩🏿‍🚒' => 'tűzoltónő: sötét bőrtónus',
    '👮🏻‍♂' => 'rendőr: világos bőrtónus',
    '👮🏼‍♂' => 'rendőr: közepesen világos bőrtónus',
    '👮🏽‍♂' => 'rendőr: közepes bőrtónus',
    '👮🏾‍♂' => 'rendőr: közepesen sötét bőrtónus',
    '👮🏿‍♂' => 'rendőr: sötét bőrtónus',
    '👮🏻‍♀' => 'rendőrnő: világos bőrtónus',
    '👮🏼‍♀' => 'rendőrnő: közepesen világos bőrtónus',
    '👮🏽‍♀' => 'rendőrnő: közepes bőrtónus',
    '👮🏾‍♀' => 'rendőrnő: közepesen sötét bőrtónus',
    '👮🏿‍♀' => 'rendőrnő: sötét bőrtónus',
    '🕵🏻‍♂' => 'nyomozó: világos bőrtónus',
    '🕵🏼‍♂' => 'nyomozó: közepesen világos bőrtónus',
    '🕵🏽‍♂' => 'nyomozó: közepes bőrtónus',
    '🕵🏾‍♂' => 'nyomozó: közepesen sötét bőrtónus',
    '🕵🏿‍♂' => 'nyomozó: sötét bőrtónus',
    '🕵🏻‍♀' => 'nyomozónő: világos bőrtónus',
    '🕵🏼‍♀' => 'nyomozónő: közepesen világos bőrtónus',
    '🕵🏽‍♀' => 'nyomozónő: közepes bőrtónus',
    '🕵🏾‍♀' => 'nyomozónő: közepesen sötét bőrtónus',
    '🕵🏿‍♀' => 'nyomozónő: sötét bőrtónus',
    '💂🏻‍♂' => 'őr: világos bőrtónus',
    '💂🏼‍♂' => 'őr: közepesen világos bőrtónus',
    '💂🏽‍♂' => 'őr: közepes bőrtónus',
    '💂🏾‍♂' => 'őr: közepesen sötét bőrtónus',
    '💂🏿‍♂' => 'őr: sötét bőrtónus',
    '💂🏻‍♀' => 'női őr: világos bőrtónus',
    '💂🏼‍♀' => 'női őr: közepesen világos bőrtónus',
    '💂🏽‍♀' => 'női őr: közepes bőrtónus',
    '💂🏾‍♀' => 'női őr: közepesen sötét bőrtónus',
    '💂🏿‍♀' => 'női őr: sötét bőrtónus',
    '👷🏻‍♂' => 'építőmunkás férfi: világos bőrtónus',
    '👷🏼‍♂' => 'építőmunkás férfi: közepesen világos bőrtónus',
    '👷🏽‍♂' => 'építőmunkás férfi: közepes bőrtónus',
    '👷🏾‍♂' => 'építőmunkás férfi: közepesen sötét bőrtónus',
    '👷🏿‍♂' => 'építőmunkás férfi: sötét bőrtónus',
    '👷🏻‍♀' => 'építőmunkás nő: világos bőrtónus',
    '👷🏼‍♀' => 'építőmunkás nő: közepesen világos bőrtónus',
    '👷🏽‍♀' => 'építőmunkás nő: közepes bőrtónus',
    '👷🏾‍♀' => 'építőmunkás nő: közepesen sötét bőrtónus',
    '👷🏿‍♀' => 'építőmunkás nő: sötét bőrtónus',
    '👳🏻‍♂' => 'turbános férfi: világos bőrtónus',
    '👳🏼‍♂' => 'turbános férfi: közepesen világos bőrtónus',
    '👳🏽‍♂' => 'turbános férfi: közepes bőrtónus',
    '👳🏾‍♂' => 'turbános férfi: közepesen sötét bőrtónus',
    '👳🏿‍♂' => 'turbános férfi: sötét bőrtónus',
    '👳🏻‍♀' => 'turbános nő: világos bőrtónus',
    '👳🏼‍♀' => 'turbános nő: közepesen világos bőrtónus',
    '👳🏽‍♀' => 'turbános nő: közepes bőrtónus',
    '👳🏾‍♀' => 'turbános nő: közepesen sötét bőrtónus',
    '👳🏿‍♀' => 'turbános nő: sötét bőrtónus',
    '🤵🏻‍♂' => 'szmokingos férfi: világos bőrtónus',
    '🤵🏼‍♂' => 'szmokingos férfi: közepesen világos bőrtónus',
    '🤵🏽‍♂' => 'szmokingos férfi: közepes bőrtónus',
    '🤵🏾‍♂' => 'szmokingos férfi: közepesen sötét bőrtónus',
    '🤵🏿‍♂' => 'szmokingos férfi: sötét bőrtónus',
    '🤵🏻‍♀' => 'szmokingos nő: világos bőrtónus',
    '🤵🏼‍♀' => 'szmokingos nő: közepesen világos bőrtónus',
    '🤵🏽‍♀' => 'szmokingos nő: közepes bőrtónus',
    '🤵🏾‍♀' => 'szmokingos nő: közepesen sötét bőrtónus',
    '🤵🏿‍♀' => 'szmokingos nő: sötét bőrtónus',
    '👰🏻‍♂' => 'férfi fátyollal: világos bőrtónus',
    '👰🏼‍♂' => 'férfi fátyollal: közepesen világos bőrtónus',
    '👰🏽‍♂' => 'férfi fátyollal: közepes bőrtónus',
    '👰🏾‍♂' => 'férfi fátyollal: közepesen sötét bőrtónus',
    '👰🏿‍♂' => 'férfi fátyollal: sötét bőrtónus',
    '👰🏻‍♀' => 'nő fátyollal: világos bőrtónus',
    '👰🏼‍♀' => 'nő fátyollal: közepesen világos bőrtónus',
    '👰🏽‍♀' => 'nő fátyollal: közepes bőrtónus',
    '👰🏾‍♀' => 'nő fátyollal: közepesen sötét bőrtónus',
    '👰🏿‍♀' => 'nő fátyollal: sötét bőrtónus',
    '👩🏻‍🍼' => 'kisbabát etető nő: világos bőrtónus',
    '👩🏼‍🍼' => 'kisbabát etető nő: közepesen világos bőrtónus',
    '👩🏽‍🍼' => 'kisbabát etető nő: közepes bőrtónus',
    '👩🏾‍🍼' => 'kisbabát etető nő: közepesen sötét bőrtónus',
    '👩🏿‍🍼' => 'kisbabát etető nő: sötét bőrtónus',
    '👨🏻‍🍼' => 'kisbabát etető férfi: világos bőrtónus',
    '👨🏼‍🍼' => 'kisbabát etető férfi: közepesen világos bőrtónus',
    '👨🏽‍🍼' => 'kisbabát etető férfi: közepes bőrtónus',
    '👨🏾‍🍼' => 'kisbabát etető férfi: közepesen sötét bőrtónus',
    '👨🏿‍🍼' => 'kisbabát etető férfi: sötét bőrtónus',
    '🧑🏻‍🍼' => 'kisbabát etető személy: világos bőrtónus',
    '🧑🏼‍🍼' => 'kisbabát etető személy: közepesen világos bőrtónus',
    '🧑🏽‍🍼' => 'kisbabát etető személy: közepes bőrtónus',
    '🧑🏾‍🍼' => 'kisbabát etető személy: közepesen sötét bőrtónus',
    '🧑🏿‍🍼' => 'kisbabát etető személy: sötét bőrtónus',
    '🧑🏻‍🎄' => 'télapó: világos bőrtónus',
    '🧑🏼‍🎄' => 'télapó: közepesen világos bőrtónus',
    '🧑🏽‍🎄' => 'télapó: közepes bőrtónus',
    '🧑🏾‍🎄' => 'télapó: közepesen sötét bőrtónus',
    '🧑🏿‍🎄' => 'télapó: sötét bőrtónus',
    '🦸🏻‍♂' => 'férfi szuperhős: világos bőrtónus',
    '🦸🏼‍♂' => 'férfi szuperhős: közepesen világos bőrtónus',
    '🦸🏽‍♂' => 'férfi szuperhős: közepes bőrtónus',
    '🦸🏾‍♂' => 'férfi szuperhős: közepesen sötét bőrtónus',
    '🦸🏿‍♂' => 'férfi szuperhős: sötét bőrtónus',
    '🦸🏻‍♀' => 'női szuperhős: világos bőrtónus',
    '🦸🏼‍♀' => 'női szuperhős: közepesen világos bőrtónus',
    '🦸🏽‍♀' => 'női szuperhős: közepes bőrtónus',
    '🦸🏾‍♀' => 'női szuperhős: közepesen sötét bőrtónus',
    '🦸🏿‍♀' => 'női szuperhős: sötét bőrtónus',
    '🦹🏻‍♂' => 'férfi főgonosz: világos bőrtónus',
    '🦹🏼‍♂' => 'férfi főgonosz: közepesen világos bőrtónus',
    '🦹🏽‍♂' => 'férfi főgonosz: közepes bőrtónus',
    '🦹🏾‍♂' => 'férfi főgonosz: közepesen sötét bőrtónus',
    '🦹🏿‍♂' => 'férfi főgonosz: sötét bőrtónus',
    '🦹🏻‍♀' => 'női főgonosz: világos bőrtónus',
    '🦹🏼‍♀' => 'női főgonosz: közepesen világos bőrtónus',
    '🦹🏽‍♀' => 'női főgonosz: közepes bőrtónus',
    '🦹🏾‍♀' => 'női főgonosz: közepesen sötét bőrtónus',
    '🦹🏿‍♀' => 'női főgonosz: sötét bőrtónus',
    '🧙🏻‍♂' => 'férfi varázsló: világos bőrtónus',
    '🧙🏼‍♂' => 'férfi varázsló: közepesen világos bőrtónus',
    '🧙🏽‍♂' => 'férfi varázsló: közepes bőrtónus',
    '🧙🏾‍♂' => 'férfi varázsló: közepesen sötét bőrtónus',
    '🧙🏿‍♂' => 'férfi varázsló: sötét bőrtónus',
    '🧙🏻‍♀' => 'varázslónő: világos bőrtónus',
    '🧙🏼‍♀' => 'varázslónő: közepesen világos bőrtónus',
    '🧙🏽‍♀' => 'varázslónő: közepes bőrtónus',
    '🧙🏾‍♀' => 'varázslónő: közepesen sötét bőrtónus',
    '🧙🏿‍♀' => 'varázslónő: sötét bőrtónus',
    '🧚🏻‍♂' => 'férfi tündér: világos bőrtónus',
    '🧚🏼‍♂' => 'férfi tündér: közepesen világos bőrtónus',
    '🧚🏽‍♂' => 'férfi tündér: közepes bőrtónus',
    '🧚🏾‍♂' => 'férfi tündér: közepesen sötét bőrtónus',
    '🧚🏿‍♂' => 'férfi tündér: sötét bőrtónus',
    '🧚🏻‍♀' => 'női tündér: világos bőrtónus',
    '🧚🏼‍♀' => 'női tündér: közepesen világos bőrtónus',
    '🧚🏽‍♀' => 'női tündér: közepes bőrtónus',
    '🧚🏾‍♀' => 'női tündér: közepesen sötét bőrtónus',
    '🧚🏿‍♀' => 'női tündér: sötét bőrtónus',
    '🧛🏻‍♂' => 'férfi vámpír: világos bőrtónus',
    '🧛🏼‍♂' => 'férfi vámpír: közepesen világos bőrtónus',
    '🧛🏽‍♂' => 'férfi vámpír: közepes bőrtónus',
    '🧛🏾‍♂' => 'férfi vámpír: közepesen sötét bőrtónus',
    '🧛🏿‍♂' => 'férfi vámpír: sötét bőrtónus',
    '🧛🏻‍♀' => 'női vámpír: világos bőrtónus',
    '🧛🏼‍♀' => 'női vámpír: közepesen világos bőrtónus',
    '🧛🏽‍♀' => 'női vámpír: közepes bőrtónus',
    '🧛🏾‍♀' => 'női vámpír: közepesen sötét bőrtónus',
    '🧛🏿‍♀' => 'női vámpír: sötét bőrtónus',
    '🧜🏻‍♂' => 'férfi sellő: világos bőrtónus',
    '🧜🏼‍♂' => 'férfi sellő: közepesen világos bőrtónus',
    '🧜🏽‍♂' => 'férfi sellő: közepes bőrtónus',
    '🧜🏾‍♂' => 'férfi sellő: közepesen sötét bőrtónus',
    '🧜🏿‍♂' => 'férfi sellő: sötét bőrtónus',
    '🧜🏻‍♀' => 'sellőlány: világos bőrtónus',
    '🧜🏼‍♀' => 'sellőlány: közepesen világos bőrtónus',
    '🧜🏽‍♀' => 'sellőlány: közepes bőrtónus',
    '🧜🏾‍♀' => 'sellőlány: közepesen sötét bőrtónus',
    '🧜🏿‍♀' => 'sellőlány: sötét bőrtónus',
    '🧝🏻‍♂' => 'férfi tünde: világos bőrtónus',
    '🧝🏼‍♂' => 'férfi tünde: közepesen világos bőrtónus',
    '🧝🏽‍♂' => 'férfi tünde: közepes bőrtónus',
    '🧝🏾‍♂' => 'férfi tünde: közepesen sötét bőrtónus',
    '🧝🏿‍♂' => 'férfi tünde: sötét bőrtónus',
    '🧝🏻‍♀' => 'női tünde: világos bőrtónus',
    '🧝🏼‍♀' => 'női tünde: közepesen világos bőrtónus',
    '🧝🏽‍♀' => 'női tünde: közepes bőrtónus',
    '🧝🏾‍♀' => 'női tünde: közepesen sötét bőrtónus',
    '🧝🏿‍♀' => 'női tünde: sötét bőrtónus',
    '💆🏻‍♂' => 'férfi arcmasszázson: világos bőrtónus',
    '💆🏼‍♂' => 'férfi arcmasszázson: közepesen világos bőrtónus',
    '💆🏽‍♂' => 'férfi arcmasszázson: közepes bőrtónus',
    '💆🏾‍♂' => 'férfi arcmasszázson: közepesen sötét bőrtónus',
    '💆🏿‍♂' => 'férfi arcmasszázson: sötét bőrtónus',
    '💆🏻‍♀' => 'nő arcmasszázson: világos bőrtónus',
    '💆🏼‍♀' => 'nő arcmasszázson: közepesen világos bőrtónus',
    '💆🏽‍♀' => 'nő arcmasszázson: közepes bőrtónus',
    '💆🏾‍♀' => 'nő arcmasszázson: közepesen sötét bőrtónus',
    '💆🏿‍♀' => 'nő arcmasszázson: sötét bőrtónus',
    '💇🏻‍♂' => 'férfi fodrásznál: világos bőrtónus',
    '💇🏼‍♂' => 'férfi fodrásznál: közepesen világos bőrtónus',
    '💇🏽‍♂' => 'férfi fodrásznál: közepes bőrtónus',
    '💇🏾‍♂' => 'férfi fodrásznál: közepesen sötét bőrtónus',
    '💇🏿‍♂' => 'férfi fodrásznál: sötét bőrtónus',
    '💇🏻‍♀' => 'nő fodrásznál: világos bőrtónus',
    '💇🏼‍♀' => 'nő fodrásznál: közepesen világos bőrtónus',
    '💇🏽‍♀' => 'nő fodrásznál: közepes bőrtónus',
    '💇🏾‍♀' => 'nő fodrásznál: közepesen sötét bőrtónus',
    '💇🏿‍♀' => 'nő fodrásznál: sötét bőrtónus',
    '🚶🏻‍♂' => 'sétáló férfi: világos bőrtónus',
    '🚶🏼‍♂' => 'sétáló férfi: közepesen világos bőrtónus',
    '🚶🏽‍♂' => 'sétáló férfi: közepes bőrtónus',
    '🚶🏾‍♂' => 'sétáló férfi: közepesen sötét bőrtónus',
    '🚶🏿‍♂' => 'sétáló férfi: sötét bőrtónus',
    '🚶🏻‍♀' => 'sétáló nő: világos bőrtónus',
    '🚶🏼‍♀' => 'sétáló nő: közepesen világos bőrtónus',
    '🚶🏽‍♀' => 'sétáló nő: közepes bőrtónus',
    '🚶🏾‍♀' => 'sétáló nő: közepesen sötét bőrtónus',
    '🚶🏿‍♀' => 'sétáló nő: sötét bőrtónus',
    '🚶🏻‍➡' => 'gyalogos: világos bőrtónus és jobbra néző',
    '🚶🏼‍➡' => 'gyalogos: közepesen világos bőrtónus és jobbra néző',
    '🚶🏽‍➡' => 'gyalogos: közepes bőrtónus és jobbra néző',
    '🚶🏾‍➡' => 'gyalogos: közepesen sötét bőrtónus és jobbra néző',
    '🚶🏿‍➡' => 'gyalogos: sötét bőrtónus és jobbra néző',
    '🧍🏻‍♂' => 'álló férfi: világos bőrtónus',
    '🧍🏼‍♂' => 'álló férfi: közepesen világos bőrtónus',
    '🧍🏽‍♂' => 'álló férfi: közepes bőrtónus',
    '🧍🏾‍♂' => 'álló férfi: közepesen sötét bőrtónus',
    '🧍🏿‍♂' => 'álló férfi: sötét bőrtónus',
    '🧍🏻‍♀' => 'álló nő: világos bőrtónus',
    '🧍🏼‍♀' => 'álló nő: közepesen világos bőrtónus',
    '🧍🏽‍♀' => 'álló nő: közepes bőrtónus',
    '🧍🏾‍♀' => 'álló nő: közepesen sötét bőrtónus',
    '🧍🏿‍♀' => 'álló nő: sötét bőrtónus',
    '🧎🏻‍♂' => 'térdelő férfi: világos bőrtónus',
    '🧎🏼‍♂' => 'térdelő férfi: közepesen világos bőrtónus',
    '🧎🏽‍♂' => 'térdelő férfi: közepes bőrtónus',
    '🧎🏾‍♂' => 'térdelő férfi: közepesen sötét bőrtónus',
    '🧎🏿‍♂' => 'térdelő férfi: sötét bőrtónus',
    '🧎🏻‍♀' => 'térdelő nő: világos bőrtónus',
    '🧎🏼‍♀' => 'térdelő nő: közepesen világos bőrtónus',
    '🧎🏽‍♀' => 'térdelő nő: közepes bőrtónus',
    '🧎🏾‍♀' => 'térdelő nő: közepesen sötét bőrtónus',
    '🧎🏿‍♀' => 'térdelő nő: sötét bőrtónus',
    '🧎🏻‍➡' => 'térdelő ember: világos bőrtónus és jobbra néző',
    '🧎🏼‍➡' => 'térdelő ember: közepesen világos bőrtónus és jobbra néző',
    '🧎🏽‍➡' => 'térdelő ember: közepes bőrtónus és jobbra néző',
    '🧎🏾‍➡' => 'térdelő ember: közepesen sötét bőrtónus és jobbra néző',
    '🧎🏿‍➡' => 'térdelő ember: sötét bőrtónus és jobbra néző',
    '🧑🏻‍🦯' => 'ember fehér bottal: világos bőrtónus',
    '🧑🏼‍🦯' => 'ember fehér bottal: közepesen világos bőrtónus',
    '🧑🏽‍🦯' => 'ember fehér bottal: közepes bőrtónus',
    '🧑🏾‍🦯' => 'ember fehér bottal: közepesen sötét bőrtónus',
    '🧑🏿‍🦯' => 'ember fehér bottal: sötét bőrtónus',
    '👨🏻‍🦯' => 'férfi fehér bottal: világos bőrtónus',
    '👨🏼‍🦯' => 'férfi fehér bottal: közepesen világos bőrtónus',
    '👨🏽‍🦯' => 'férfi fehér bottal: közepes bőrtónus',
    '👨🏾‍🦯' => 'férfi fehér bottal: közepesen sötét bőrtónus',
    '👨🏿‍🦯' => 'férfi fehér bottal: sötét bőrtónus',
    '👩🏻‍🦯' => 'nő fehér bottal: világos bőrtónus',
    '👩🏼‍🦯' => 'nő fehér bottal: közepesen világos bőrtónus',
    '👩🏽‍🦯' => 'nő fehér bottal: közepes bőrtónus',
    '👩🏾‍🦯' => 'nő fehér bottal: közepesen sötét bőrtónus',
    '👩🏿‍🦯' => 'nő fehér bottal: sötét bőrtónus',
    '🧑🏻‍🦼' => 'ember motoros kerekesszékben: világos bőrtónus',
    '🧑🏼‍🦼' => 'ember motoros kerekesszékben: közepesen világos bőrtónus',
    '🧑🏽‍🦼' => 'ember motoros kerekesszékben: közepes bőrtónus',
    '🧑🏾‍🦼' => 'ember motoros kerekesszékben: közepesen sötét bőrtónus',
    '🧑🏿‍🦼' => 'ember motoros kerekesszékben: sötét bőrtónus',
    '👨🏻‍🦼' => 'férfi motoros kerekesszékben: világos bőrtónus',
    '👨🏼‍🦼' => 'férfi motoros kerekesszékben: közepesen világos bőrtónus',
    '👨🏽‍🦼' => 'férfi motoros kerekesszékben: közepes bőrtónus',
    '👨🏾‍🦼' => 'férfi motoros kerekesszékben: közepesen sötét bőrtónus',
    '👨🏿‍🦼' => 'férfi motoros kerekesszékben: sötét bőrtónus',
    '👩🏻‍🦼' => 'nő motoros kerekesszékben: világos bőrtónus',
    '👩🏼‍🦼' => 'nő motoros kerekesszékben: közepesen világos bőrtónus',
    '👩🏽‍🦼' => 'nő motoros kerekesszékben: közepes bőrtónus',
    '👩🏾‍🦼' => 'nő motoros kerekesszékben: közepesen sötét bőrtónus',
    '👩🏿‍🦼' => 'nő motoros kerekesszékben: sötét bőrtónus',
    '🧑🏻‍🦽' => 'kerekesszékes ember: világos bőrtónus',
    '🧑🏼‍🦽' => 'kerekesszékes ember: közepesen világos bőrtónus',
    '🧑🏽‍🦽' => 'kerekesszékes ember: közepes bőrtónus',
    '🧑🏾‍🦽' => 'kerekesszékes ember: közepesen sötét bőrtónus',
    '🧑🏿‍🦽' => 'kerekesszékes ember: sötét bőrtónus',
    '👨🏻‍🦽' => 'férfi kézi hajtású kerekesszékben: világos bőrtónus',
    '👨🏼‍🦽' => 'férfi kézi hajtású kerekesszékben: közepesen világos bőrtónus',
    '👨🏽‍🦽' => 'férfi kézi hajtású kerekesszékben: közepes bőrtónus',
    '👨🏾‍🦽' => 'férfi kézi hajtású kerekesszékben: közepesen sötét bőrtónus',
    '👨🏿‍🦽' => 'férfi kézi hajtású kerekesszékben: sötét bőrtónus',
    '👩🏻‍🦽' => 'nő kézi hajtású kerekesszékben: világos bőrtónus',
    '👩🏼‍🦽' => 'nő kézi hajtású kerekesszékben: közepesen világos bőrtónus',
    '👩🏽‍🦽' => 'nő kézi hajtású kerekesszékben: közepes bőrtónus',
    '👩🏾‍🦽' => 'nő kézi hajtású kerekesszékben: közepesen sötét bőrtónus',
    '👩🏿‍🦽' => 'nő kézi hajtású kerekesszékben: sötét bőrtónus',
    '🏃🏻‍♂' => 'futó férfi: világos bőrtónus',
    '🏃🏼‍♂' => 'futó férfi: közepesen világos bőrtónus',
    '🏃🏽‍♂' => 'futó férfi: közepes bőrtónus',
    '🏃🏾‍♂' => 'futó férfi: közepesen sötét bőrtónus',
    '🏃🏿‍♂' => 'futó férfi: sötét bőrtónus',
    '🏃🏻‍♀' => 'futó nő: világos bőrtónus',
    '🏃🏼‍♀' => 'futó nő: közepesen világos bőrtónus',
    '🏃🏽‍♀' => 'futó nő: közepes bőrtónus',
    '🏃🏾‍♀' => 'futó nő: közepesen sötét bőrtónus',
    '🏃🏿‍♀' => 'futó nő: sötét bőrtónus',
    '🏃🏻‍➡' => 'futó: világos bőrtónus és jobbra néző',
    '🏃🏼‍➡' => 'futó: közepesen világos bőrtónus és jobbra néző',
    '🏃🏽‍➡' => 'futó: közepes bőrtónus és jobbra néző',
    '🏃🏾‍➡' => 'futó: közepesen sötét bőrtónus és jobbra néző',
    '🏃🏿‍➡' => 'futó: sötét bőrtónus és jobbra néző',
    '👯🏻‍♀' => 'bulizó nők: világos bőrtónus',
    '👯🏼‍♀' => 'bulizó nők: közepesen világos bőrtónus',
    '👯🏽‍♀' => 'bulizó nők: közepes bőrtónus',
    '👯🏾‍♀' => 'bulizó nők: közepesen sötét bőrtónus',
    '👯🏿‍♀' => 'bulizó nők: sötét bőrtónus',
    '👯🏻‍♂' => 'bulizó férfiak: világos bőrtónus',
    '👯🏼‍♂' => 'bulizó férfiak: közepesen világos bőrtónus',
    '👯🏽‍♂' => 'bulizó férfiak: közepes bőrtónus',
    '👯🏾‍♂' => 'bulizó férfiak: közepesen sötét bőrtónus',
    '👯🏿‍♂' => 'bulizó férfiak: sötét bőrtónus',
    '🧖🏻‍♂' => 'férfi a gőzfürdőben: világos bőrtónus',
    '🧖🏼‍♂' => 'férfi a gőzfürdőben: közepesen világos bőrtónus',
    '🧖🏽‍♂' => 'férfi a gőzfürdőben: közepes bőrtónus',
    '🧖🏾‍♂' => 'férfi a gőzfürdőben: közepesen sötét bőrtónus',
    '🧖🏿‍♂' => 'férfi a gőzfürdőben: sötét bőrtónus',
    '🧖🏻‍♀' => 'nő a gőzfürdőben: világos bőrtónus',
    '🧖🏼‍♀' => 'nő a gőzfürdőben: közepesen világos bőrtónus',
    '🧖🏽‍♀' => 'nő a gőzfürdőben: közepes bőrtónus',
    '🧖🏾‍♀' => 'nő a gőzfürdőben: közepesen sötét bőrtónus',
    '🧖🏿‍♀' => 'nő a gőzfürdőben: sötét bőrtónus',
    '🧗🏻‍♂' => 'mászó férfi: világos bőrtónus',
    '🧗🏼‍♂' => 'mászó férfi: közepesen világos bőrtónus',
    '🧗🏽‍♂' => 'mászó férfi: közepes bőrtónus',
    '🧗🏾‍♂' => 'mászó férfi: közepesen sötét bőrtónus',
    '🧗🏿‍♂' => 'mászó férfi: sötét bőrtónus',
    '🧗🏻‍♀' => 'mászó nő: világos bőrtónus',
    '🧗🏼‍♀' => 'mászó nő: közepesen világos bőrtónus',
    '🧗🏽‍♀' => 'mászó nő: közepes bőrtónus',
    '🧗🏾‍♀' => 'mászó nő: közepesen sötét bőrtónus',
    '🧗🏿‍♀' => 'mászó nő: sötét bőrtónus',
    '🏌🏻‍♂' => 'golfozó férfi: világos bőrtónus',
    '🏌🏼‍♂' => 'golfozó férfi: közepesen világos bőrtónus',
    '🏌🏽‍♂' => 'golfozó férfi: közepes bőrtónus',
    '🏌🏾‍♂' => 'golfozó férfi: közepesen sötét bőrtónus',
    '🏌🏿‍♂' => 'golfozó férfi: sötét bőrtónus',
    '🏌🏻‍♀' => 'golfozó nő: világos bőrtónus',
    '🏌🏼‍♀' => 'golfozó nő: közepesen világos bőrtónus',
    '🏌🏽‍♀' => 'golfozó nő: közepes bőrtónus',
    '🏌🏾‍♀' => 'golfozó nő: közepesen sötét bőrtónus',
    '🏌🏿‍♀' => 'golfozó nő: sötét bőrtónus',
    '🏄🏻‍♂' => 'szörföző férfi: világos bőrtónus',
    '🏄🏼‍♂' => 'szörföző férfi: közepesen világos bőrtónus',
    '🏄🏽‍♂' => 'szörföző férfi: közepes bőrtónus',
    '🏄🏾‍♂' => 'szörföző férfi: közepesen sötét bőrtónus',
    '🏄🏿‍♂' => 'szörföző férfi: sötét bőrtónus',
    '🏄🏻‍♀' => 'szörföző nő: világos bőrtónus',
    '🏄🏼‍♀' => 'szörföző nő: közepesen világos bőrtónus',
    '🏄🏽‍♀' => 'szörföző nő: közepes bőrtónus',
    '🏄🏾‍♀' => 'szörföző nő: közepesen sötét bőrtónus',
    '🏄🏿‍♀' => 'szörföző nő: sötét bőrtónus',
    '🚣🏻‍♂' => 'evező férfi: világos bőrtónus',
    '🚣🏼‍♂' => 'evező férfi: közepesen világos bőrtónus',
    '🚣🏽‍♂' => 'evező férfi: közepes bőrtónus',
    '🚣🏾‍♂' => 'evező férfi: közepesen sötét bőrtónus',
    '🚣🏿‍♂' => 'evező férfi: sötét bőrtónus',
    '🚣🏻‍♀' => 'evező nő: világos bőrtónus',
    '🚣🏼‍♀' => 'evező nő: közepesen világos bőrtónus',
    '🚣🏽‍♀' => 'evező nő: közepes bőrtónus',
    '🚣🏾‍♀' => 'evező nő: közepesen sötét bőrtónus',
    '🚣🏿‍♀' => 'evező nő: sötét bőrtónus',
    '🏊🏻‍♂' => 'úszó férfi: világos bőrtónus',
    '🏊🏼‍♂' => 'úszó férfi: közepesen világos bőrtónus',
    '🏊🏽‍♂' => 'úszó férfi: közepes bőrtónus',
    '🏊🏾‍♂' => 'úszó férfi: közepesen sötét bőrtónus',
    '🏊🏿‍♂' => 'úszó férfi: sötét bőrtónus',
    '🏊🏻‍♀' => 'úszó nő: világos bőrtónus',
    '🏊🏼‍♀' => 'úszó nő: közepesen világos bőrtónus',
    '🏊🏽‍♀' => 'úszó nő: közepes bőrtónus',
    '🏊🏾‍♀' => 'úszó nő: közepesen sötét bőrtónus',
    '🏊🏿‍♀' => 'úszó nő: sötét bőrtónus',
    '⛹🏻‍♂' => 'férfi labdával: világos bőrtónus',
    '⛹🏼‍♂' => 'férfi labdával: közepesen világos bőrtónus',
    '⛹🏽‍♂' => 'férfi labdával: közepes bőrtónus',
    '⛹🏾‍♂' => 'férfi labdával: közepesen sötét bőrtónus',
    '⛹🏿‍♂' => 'férfi labdával: sötét bőrtónus',
    '⛹🏻‍♀' => 'nő labdával: világos bőrtónus',
    '⛹🏼‍♀' => 'nő labdával: közepesen világos bőrtónus',
    '⛹🏽‍♀' => 'nő labdával: közepes bőrtónus',
    '⛹🏾‍♀' => 'nő labdával: közepesen sötét bőrtónus',
    '⛹🏿‍♀' => 'nő labdával: sötét bőrtónus',
    '🏋🏻‍♂' => 'súlyemelő férfi: világos bőrtónus',
    '🏋🏼‍♂' => 'súlyemelő férfi: közepesen világos bőrtónus',
    '🏋🏽‍♂' => 'súlyemelő férfi: közepes bőrtónus',
    '🏋🏾‍♂' => 'súlyemelő férfi: közepesen sötét bőrtónus',
    '🏋🏿‍♂' => 'súlyemelő férfi: sötét bőrtónus',
    '🏋🏻‍♀' => 'súlyemelő nő: világos bőrtónus',
    '🏋🏼‍♀' => 'súlyemelő nő: közepesen világos bőrtónus',
    '🏋🏽‍♀' => 'súlyemelő nő: közepes bőrtónus',
    '🏋🏾‍♀' => 'súlyemelő nő: közepesen sötét bőrtónus',
    '🏋🏿‍♀' => 'súlyemelő nő: sötét bőrtónus',
    '🚴🏻‍♂' => 'kerékpáros férfi: világos bőrtónus',
    '🚴🏼‍♂' => 'kerékpáros férfi: közepesen világos bőrtónus',
    '🚴🏽‍♂' => 'kerékpáros férfi: közepes bőrtónus',
    '🚴🏾‍♂' => 'kerékpáros férfi: közepesen sötét bőrtónus',
    '🚴🏿‍♂' => 'kerékpáros férfi: sötét bőrtónus',
    '🚴🏻‍♀' => 'kerékpáros nő: világos bőrtónus',
    '🚴🏼‍♀' => 'kerékpáros nő: közepesen világos bőrtónus',
    '🚴🏽‍♀' => 'kerékpáros nő: közepes bőrtónus',
    '🚴🏾‍♀' => 'kerékpáros nő: közepesen sötét bőrtónus',
    '🚴🏿‍♀' => 'kerékpáros nő: sötét bőrtónus',
    '🚵🏻‍♂' => 'hegyikerékpáros férfi: világos bőrtónus',
    '🚵🏼‍♂' => 'hegyikerékpáros férfi: közepesen világos bőrtónus',
    '🚵🏽‍♂' => 'hegyikerékpáros férfi: közepes bőrtónus',
    '🚵🏾‍♂' => 'hegyikerékpáros férfi: közepesen sötét bőrtónus',
    '🚵🏿‍♂' => 'hegyikerékpáros férfi: sötét bőrtónus',
    '🚵🏻‍♀' => 'hegyikerékpáros nő: világos bőrtónus',
    '🚵🏼‍♀' => 'hegyikerékpáros nő: közepesen világos bőrtónus',
    '🚵🏽‍♀' => 'hegyikerékpáros nő: közepes bőrtónus',
    '🚵🏾‍♀' => 'hegyikerékpáros nő: közepesen sötét bőrtónus',
    '🚵🏿‍♀' => 'hegyikerékpáros nő: sötét bőrtónus',
    '🤸🏻‍♂' => 'cigánykerekező férfi: világos bőrtónus',
    '🤸🏼‍♂' => 'cigánykerekező férfi: közepesen világos bőrtónus',
    '🤸🏽‍♂' => 'cigánykerekező férfi: közepes bőrtónus',
    '🤸🏾‍♂' => 'cigánykerekező férfi: közepesen sötét bőrtónus',
    '🤸🏿‍♂' => 'cigánykerekező férfi: sötét bőrtónus',
    '🤸🏻‍♀' => 'cigánykerekező nő: világos bőrtónus',
    '🤸🏼‍♀' => 'cigánykerekező nő: közepesen világos bőrtónus',
    '🤸🏽‍♀' => 'cigánykerekező nő: közepes bőrtónus',
    '🤸🏾‍♀' => 'cigánykerekező nő: közepesen sötét bőrtónus',
    '🤸🏿‍♀' => 'cigánykerekező nő: sötét bőrtónus',
    '🤼🏻‍♀' => 'birkózó nők: világos bőrtónus',
    '🤼🏼‍♀' => 'birkózó nők: közepesen világos bőrtónus',
    '🤼🏽‍♀' => 'birkózó nők: közepes bőrtónus',
    '🤼🏾‍♀' => 'birkózó nők: közepesen sötét bőrtónus',
    '🤼🏿‍♀' => 'birkózó nők: sötét bőrtónus',
    '🤼🏻‍♂' => 'birkózó férfiak: világos bőrtónus',
    '🤼🏼‍♂' => 'birkózó férfiak: közepesen világos bőrtónus',
    '🤼🏽‍♂' => 'birkózó férfiak: közepes bőrtónus',
    '🤼🏾‍♂' => 'birkózó férfiak: közepesen sötét bőrtónus',
    '🤼🏿‍♂' => 'birkózó férfiak: sötét bőrtónus',
    '🤽🏻‍♂' => 'vízilabdázó férfi: világos bőrtónus',
    '🤽🏼‍♂' => 'vízilabdázó férfi: közepesen világos bőrtónus',
    '🤽🏽‍♂' => 'vízilabdázó férfi: közepes bőrtónus',
    '🤽🏾‍♂' => 'vízilabdázó férfi: közepesen sötét bőrtónus',
    '🤽🏿‍♂' => 'vízilabdázó férfi: sötét bőrtónus',
    '🤽🏻‍♀' => 'vízilabdázó nő: világos bőrtónus',
    '🤽🏼‍♀' => 'vízilabdázó nő: közepesen világos bőrtónus',
    '🤽🏽‍♀' => 'vízilabdázó nő: közepes bőrtónus',
    '🤽🏾‍♀' => 'vízilabdázó nő: közepesen sötét bőrtónus',
    '🤽🏿‍♀' => 'vízilabdázó nő: sötét bőrtónus',
    '🤾🏻‍♂' => 'kézilabdázó férfi: világos bőrtónus',
    '🤾🏼‍♂' => 'kézilabdázó férfi: közepesen világos bőrtónus',
    '🤾🏽‍♂' => 'kézilabdázó férfi: közepes bőrtónus',
    '🤾🏾‍♂' => 'kézilabdázó férfi: közepesen sötét bőrtónus',
    '🤾🏿‍♂' => 'kézilabdázó férfi: sötét bőrtónus',
    '🤾🏻‍♀' => 'kézilabdázó nő: világos bőrtónus',
    '🤾🏼‍♀' => 'kézilabdázó nő: közepesen világos bőrtónus',
    '🤾🏽‍♀' => 'kézilabdázó nő: közepes bőrtónus',
    '🤾🏾‍♀' => 'kézilabdázó nő: közepesen sötét bőrtónus',
    '🤾🏿‍♀' => 'kézilabdázó nő: sötét bőrtónus',
    '🤹🏻‍♂' => 'zsonglőr férfi: világos bőrtónus',
    '🤹🏼‍♂' => 'zsonglőr férfi: közepesen világos bőrtónus',
    '🤹🏽‍♂' => 'zsonglőr férfi: közepes bőrtónus',
    '🤹🏾‍♂' => 'zsonglőr férfi: közepesen sötét bőrtónus',
    '🤹🏿‍♂' => 'zsonglőr férfi: sötét bőrtónus',
    '🤹🏻‍♀' => 'zsonglőr nő: világos bőrtónus',
    '🤹🏼‍♀' => 'zsonglőr nő: közepesen világos bőrtónus',
    '🤹🏽‍♀' => 'zsonglőr nő: közepes bőrtónus',
    '🤹🏾‍♀' => 'zsonglőr nő: közepesen sötét bőrtónus',
    '🤹🏿‍♀' => 'zsonglőr nő: sötét bőrtónus',
    '🧘🏻‍♂' => 'férfi lótuszülésben: világos bőrtónus',
    '🧘🏼‍♂' => 'férfi lótuszülésben: közepesen világos bőrtónus',
    '🧘🏽‍♂' => 'férfi lótuszülésben: közepes bőrtónus',
    '🧘🏾‍♂' => 'férfi lótuszülésben: közepesen sötét bőrtónus',
    '🧘🏿‍♂' => 'férfi lótuszülésben: sötét bőrtónus',
    '🧘🏻‍♀' => 'nő lótuszülésben: világos bőrtónus',
    '🧘🏼‍♀' => 'nő lótuszülésben: közepesen világos bőrtónus',
    '🧘🏽‍♀' => 'nő lótuszülésben: közepes bőrtónus',
    '🧘🏾‍♀' => 'nő lótuszülésben: közepesen sötét bőrtónus',
    '🧘🏿‍♀' => 'nő lótuszülésben: sötét bőrtónus',
    '🧑‍🦰' => 'felnőtt: vörös haj',
    '🧑‍🦱' => 'felnőtt: göndör haj',
    '🧑‍🦳' => 'felnőtt: ősz haj',
    '🧑‍🦲' => 'felnőtt: kopasz',
    '👨‍🦰' => 'férfi: vörös haj',
    '👨‍🦱' => 'férfi: göndör haj',
    '👨‍🦳' => 'férfi: ősz haj',
    '👨‍🦲' => 'férfi: kopasz',
    '👩‍🦰' => 'nő: vörös haj',
    '👩‍🦱' => 'nő: göndör haj',
    '👩‍🦳' => 'nő: ősz haj',
    '👩‍🦲' => 'nő: kopasz',
    '🚶‍➡' => 'gyalogos: jobbra néző',
    '🧎‍➡' => 'térdelő ember: jobbra néző',
    '🏃‍➡' => 'futó: jobbra néző',
    '👨‍👦' => 'család: férfi és fiú',
    '👨‍👧' => 'család: férfi és lány',
    '👩‍👦' => 'család: nő és fiú',
    '👩‍👧' => 'család: nő és lány',
    '😶‍🌫' => 'arc a felhőben',
    '😮‍💨' => 'levegőt kifújó arc',
    '🙂‍↕' => 'függőleges fejrázás',
    '😵‍💫' => 'spirálszemű arc',
    '❤‍🔥' => 'lángoló szív',
    '❤‍🩹' => 'gyógyuló szív',
    '👁‍🗨' => 'szem szövegbuborékban',
    '🧔‍♂' => 'férfi: szakáll',
    '👱‍♂' => 'szőke férfi',
    '🧔‍♀' => 'nő: szakáll',
    '👱‍♀' => 'szőke nő',
    '🙍‍♂' => 'rosszalló férfi',
    '🙍‍♀' => 'rosszalló nő',
    '🙎‍♂' => 'duzzogó férfi',
    '🙎‍♀' => 'duzzogó nő',
    '🙅‍♂' => 'tiltást mutató férfi',
    '🙅‍♀' => 'tiltást mutató nő',
    '🙆‍♂' => 'beleegyezést mutató férfi',
    '🙆‍♀' => 'beleegyezést mutató nő',
    '💁‍♂' => 'nyitott tenyerű férfi',
    '💁‍♀' => 'nyitott tenyerű nő',
    '🙋‍♂' => 'vidáman intő férfi',
    '🙋‍♀' => 'vidáman intő nő',
    '🧏‍♂' => 'siket férfi',
    '🧏‍♀' => 'siket nő',
    '🙇‍♂' => 'meghajló férfi',
    '🙇‍♀' => 'meghajló nő',
    '🤦‍♂' => 'arcát a kezébe temető férfi',
    '🤦‍♀' => 'arcát a kezébe temető nő',
    '🤷‍♂' => 'vállrándító férfi',
    '🤷‍♀' => 'vállrándító nő',
    '🧑‍⚕' => 'egészségügyi dolgozó',
    '👨‍⚕' => 'doktor',
    '👩‍⚕' => 'doktornő',
    '🧑‍🎓' => 'diák',
    '👨‍🎓' => 'férfi tanuló',
    '👩‍🎓' => 'női tanuló',
    '🧑‍🏫' => 'tanár / tanárnő',
    '👨‍🏫' => 'tanár',
    '👩‍🏫' => 'tanárnő',
    '🧑‍⚖' => 'bíró / bírónő',
    '👨‍⚖' => 'bíró',
    '👩‍⚖' => 'bírónő',
    '🧑‍🌾' => 'gazda / női gazda',
    '👨‍🌾' => 'gazda',
    '👩‍🌾' => 'női gazda',
    '🧑‍🍳' => 'szakács / szakácsnő',
    '👨‍🍳' => 'szakács',
    '👩‍🍳' => 'szakácsnő',
    '🧑‍🔧' => 'szerelő / szerelőnő',
    '👨‍🔧' => 'szerelő',
    '👩‍🔧' => 'szerelőnő',
    '🧑‍🏭' => 'munkás',
    '👨‍🏭' => 'férfi munkás',
    '👩‍🏭' => 'női munkás',
    '🧑‍💼' => 'irodai dolgozó',
    '👨‍💼' => 'férfi irodista',
    '👩‍💼' => 'női irodista',
    '🧑‍🔬' => 'tudás',
    '👨‍🔬' => 'férfi tudós',
    '👩‍🔬' => 'női tudós',
    '🧑‍💻' => 'informatikus',
    '👨‍💻' => 'programozó férfi',
    '👩‍💻' => 'programozó nő',
    '🧑‍🎤' => 'énekes',
    '👨‍🎤' => 'férfi énekes',
    '👩‍🎤' => 'női énekes',
    '🧑‍🎨' => 'művész',
    '👨‍🎨' => 'férfi művész',
    '👩‍🎨' => 'női művész',
    '🧑‍✈' => 'pilóta / pilótanő',
    '👨‍✈' => 'pilóta',
    '👩‍✈' => 'pilótanő',
    '🧑‍🚀' => 'űrhajós / űrhajósnő',
    '👨‍🚀' => 'űrhajós',
    '👩‍🚀' => 'űrhajósnő',
    '🧑‍🚒' => 'tűzoltó / tűzoltónő',
    '👨‍🚒' => 'tűzoltó',
    '👩‍🚒' => 'tűzoltónő',
    '👮‍♂' => 'rendőr',
    '👮‍♀' => 'rendőrnő',
    '🕵‍♂' => 'nyomozó',
    '🕵‍♀' => 'nyomozónő',
    '💂‍♂' => 'őr',
    '💂‍♀' => 'női őr',
    '👷‍♂' => 'építőmunkás férfi',
    '👷‍♀' => 'építőmunkás nő',
    '👳‍♂' => 'turbános férfi',
    '👳‍♀' => 'turbános nő',
    '🤵‍♂' => 'szmokingos férfi',
    '🤵‍♀' => 'szmokingos nő',
    '👰‍♂' => 'férfi fátyollal',
    '👰‍♀' => 'nő fátyollal',
    '👩‍🍼' => 'kisbabát etető nő',
    '👨‍🍼' => 'kisbabát etető férfi',
    '🧑‍🍼' => 'kisbabát etető személy',
    '🧑‍🎄' => 'télapó',
    '🦸‍♂' => 'férfi szuperhős',
    '🦸‍♀' => 'női szuperhős',
    '🦹‍♂' => 'férfi főgonosz',
    '🦹‍♀' => 'női főgonosz',
    '🧙‍♂' => 'férfi varázsló',
    '🧙‍♀' => 'varázslónő',
    '🧚‍♂' => 'férfi tündér',
    '🧚‍♀' => 'női tündér',
    '🧛‍♂' => 'férfi vámpír',
    '🧛‍♀' => 'női vámpír',
    '🧜‍♂' => 'férfi sellő',
    '🧜‍♀' => 'sellőlány',
    '🧝‍♂' => 'férfi tünde',
    '🧝‍♀' => 'női tünde',
    '🧞‍♂' => 'férfi dzsinn',
    '🧞‍♀' => 'női dzsinn',
    '🧟‍♂' => 'férfi zombi',
    '🧟‍♀' => 'női zombi',
    '💆‍♂' => 'férfi arcmasszázson',
    '💆‍♀' => 'nő arcmasszázson',
    '💇‍♂' => 'férfi fodrásznál',
    '💇‍♀' => 'nő fodrásznál',
    '🚶‍♂' => 'sétáló férfi',
    '🚶‍♀' => 'sétáló nő',
    '🧍‍♂' => 'álló férfi',
    '🧍‍♀' => 'álló nő',
    '🧎‍♂' => 'térdelő férfi',
    '🧎‍♀' => 'térdelő nő',
    '🧑‍🦯' => 'ember fehér bottal',
    '👨‍🦯' => 'férfi fehér bottal',
    '👩‍🦯' => 'nő fehér bottal',
    '🧑‍🦼' => 'ember motoros kerekesszékben',
    '👨‍🦼' => 'férfi motoros kerekesszékben',
    '👩‍🦼' => 'nő motoros kerekesszékben',
    '🧑‍🦽' => 'kerekesszékes ember',
    '👨‍🦽' => 'férfi kézi hajtású kerekesszékben',
    '👩‍🦽' => 'nő kézi hajtású kerekesszékben',
    '🏃‍♂' => 'futó férfi',
    '🏃‍♀' => 'futó nő',
    '🧑‍🩰' => 'balett-táncos',
    '👯‍♂' => 'bulizó férfiak',
    '👯‍♀' => 'bulizó nők',
    '🧖‍♂' => 'férfi a gőzfürdőben',
    '🧖‍♀' => 'nő a gőzfürdőben',
    '🧗‍♂' => 'mászó férfi',
    '🧗‍♀' => 'mászó nő',
    '🏌‍♂' => 'golfozó férfi',
    '🏌‍♀' => 'golfozó nő',
    '🏄‍♂' => 'szörföző férfi',
    '🏄‍♀' => 'szörföző nő',
    '🚣‍♂' => 'evező férfi',
    '🚣‍♀' => 'evező nő',
    '🏊‍♂' => 'úszó férfi',
    '🏊‍♀' => 'úszó nő',
    '⛹‍♂' => 'férfi labdával',
    '⛹‍♀' => 'nő labdával',
    '🏋‍♂' => 'súlyemelő férfi',
    '🏋‍♀' => 'súlyemelő nő',
    '🚴‍♂' => 'kerékpáros férfi',
    '🚴‍♀' => 'kerékpáros nő',
    '🚵‍♂' => 'hegyikerékpáros férfi',
    '🚵‍♀' => 'hegyikerékpáros nő',
    '🤸‍♂' => 'cigánykerekező férfi',
    '🤸‍♀' => 'cigánykerekező nő',
    '🤼‍♂' => 'birkózó férfiak',
    '🤼‍♀' => 'birkózó nők',
    '🤽‍♂' => 'vízilabdázó férfi',
    '🤽‍♀' => 'vízilabdázó nő',
    '🤾‍♂' => 'kézilabdázó férfi',
    '🤾‍♀' => 'kézilabdázó nő',
    '🤹‍♂' => 'zsonglőr férfi',
    '🤹‍♀' => 'zsonglőr nő',
    '🧘‍♂' => 'férfi lótuszülésben',
    '🧘‍♀' => 'nő lótuszülésben',
    '🧑‍🧒' => 'család: felnőtt, gyerek',
    '🐕‍🦺' => 'segítőkutya',
    '🐈‍⬛' => 'fekete macska',
    '🐻‍❄' => 'jegesmedve',
    '🐦‍⬛' => 'fekete madár',
    '🐦‍🔥' => 'főnix',
    '🍋‍🟩' => 'lime',
    '🍄‍🟫' => 'barna gomba',
    '⛓‍💥' => 'elszakadt lánc',
    '🏳‍🌈' => 'szivárványszínű zászló',
    '🏳‍⚧' => 'transznemű zászló',
    '🏴‍☠' => 'kalózzászló',
    '👋🏻' => 'integető kéz: világos bőrtónus',
    '👋🏼' => 'integető kéz: közepesen világos bőrtónus',
    '👋🏽' => 'integető kéz: közepes bőrtónus',
    '👋🏾' => 'integető kéz: közepesen sötét bőrtónus',
    '👋🏿' => 'integető kéz: sötét bőrtónus',
    '🤚🏻' => 'kézfej: világos bőrtónus',
    '🤚🏼' => 'kézfej: közepesen világos bőrtónus',
    '🤚🏽' => 'kézfej: közepes bőrtónus',
    '🤚🏾' => 'kézfej: közepesen sötét bőrtónus',
    '🤚🏿' => 'kézfej: sötét bőrtónus',
    '🖐🏻' => 'felemelt kéz szétálló ujjakkal: világos bőrtónus',
    '🖐🏼' => 'felemelt kéz szétálló ujjakkal: közepesen világos bőrtónus',
    '🖐🏽' => 'felemelt kéz szétálló ujjakkal: közepes bőrtónus',
    '🖐🏾' => 'felemelt kéz szétálló ujjakkal: közepesen sötét bőrtónus',
    '🖐🏿' => 'felemelt kéz szétálló ujjakkal: sötét bőrtónus',
    '✋🏻' => 'felemelt kéz: világos bőrtónus',
    '✋🏼' => 'felemelt kéz: közepesen világos bőrtónus',
    '✋🏽' => 'felemelt kéz: közepes bőrtónus',
    '✋🏾' => 'felemelt kéz: közepesen sötét bőrtónus',
    '✋🏿' => 'felemelt kéz: sötét bőrtónus',
    '🖖🏻' => 'star trek-üdvözlet: világos bőrtónus',
    '🖖🏼' => 'star trek-üdvözlet: közepesen világos bőrtónus',
    '🖖🏽' => 'star trek-üdvözlet: közepes bőrtónus',
    '🖖🏾' => 'star trek-üdvözlet: közepesen sötét bőrtónus',
    '🖖🏿' => 'star trek-üdvözlet: sötét bőrtónus',
    '🫱🏻' => 'jobbra néző kéz: világos bőrtónus',
    '🫱🏼' => 'jobbra néző kéz: közepesen világos bőrtónus',
    '🫱🏽' => 'jobbra néző kéz: közepes bőrtónus',
    '🫱🏾' => 'jobbra néző kéz: közepesen sötét bőrtónus',
    '🫱🏿' => 'jobbra néző kéz: sötét bőrtónus',
    '🫲🏻' => 'balra néző kéz: világos bőrtónus',
    '🫲🏼' => 'balra néző kéz: közepesen világos bőrtónus',
    '🫲🏽' => 'balra néző kéz: közepes bőrtónus',
    '🫲🏾' => 'balra néző kéz: közepesen sötét bőrtónus',
    '🫲🏿' => 'balra néző kéz: sötét bőrtónus',
    '🫳🏻' => 'lefelé néző kéz: világos bőrtónus',
    '🫳🏼' => 'lefelé néző kéz: közepesen világos bőrtónus',
    '🫳🏽' => 'lefelé néző kéz: közepes bőrtónus',
    '🫳🏾' => 'lefelé néző kéz: közepesen sötét bőrtónus',
    '🫳🏿' => 'lefelé néző kéz: sötét bőrtónus',
    '🫴🏻' => 'felfelé néző kéz: világos bőrtónus',
    '🫴🏼' => 'felfelé néző kéz: közepesen világos bőrtónus',
    '🫴🏽' => 'felfelé néző kéz: közepes bőrtónus',
    '🫴🏾' => 'felfelé néző kéz: közepesen sötét bőrtónus',
    '🫴🏿' => 'felfelé néző kéz: sötét bőrtónus',
    '🫷🏻' => 'balra toló kéz: világos bőrtónus',
    '🫷🏼' => 'balra toló kéz: közepesen világos bőrtónus',
    '🫷🏽' => 'balra toló kéz: közepes bőrtónus',
    '🫷🏾' => 'balra toló kéz: közepesen sötét bőrtónus',
    '🫷🏿' => 'balra toló kéz: sötét bőrtónus',
    '🫸🏻' => 'jobbra toló kéz: világos bőrtónus',
    '🫸🏼' => 'jobbra toló kéz: közepesen világos bőrtónus',
    '🫸🏽' => 'jobbra toló kéz: közepes bőrtónus',
    '🫸🏾' => 'jobbra toló kéz: közepesen sötét bőrtónus',
    '🫸🏿' => 'jobbra toló kéz: sötét bőrtónus',
    '👌🏻' => 'ok jel: világos bőrtónus',
    '👌🏼' => 'ok jel: közepesen világos bőrtónus',
    '👌🏽' => 'ok jel: közepes bőrtónus',
    '👌🏾' => 'ok jel: közepesen sötét bőrtónus',
    '👌🏿' => 'ok jel: sötét bőrtónus',
    '🤌🏻' => 'összecsippentett ujjak: világos bőrtónus',
    '🤌🏼' => 'összecsippentett ujjak: közepesen világos bőrtónus',
    '🤌🏽' => 'összecsippentett ujjak: közepes bőrtónus',
    '🤌🏾' => 'összecsippentett ujjak: közepesen sötét bőrtónus',
    '🤌🏿' => 'összecsippentett ujjak: sötét bőrtónus',
    '🤏🏻' => 'csippentő kéz: világos bőrtónus',
    '🤏🏼' => 'csippentő kéz: közepesen világos bőrtónus',
    '🤏🏽' => 'csippentő kéz: közepes bőrtónus',
    '🤏🏾' => 'csippentő kéz: közepesen sötét bőrtónus',
    '🤏🏿' => 'csippentő kéz: sötét bőrtónus',
    '✌🏻' => 'győzelmi jelet mutató kéz: világos bőrtónus',
    '✌🏼' => 'győzelmi jelet mutató kéz: közepesen világos bőrtónus',
    '✌🏽' => 'győzelmi jelet mutató kéz: közepes bőrtónus',
    '✌🏾' => 'győzelmi jelet mutató kéz: közepesen sötét bőrtónus',
    '✌🏿' => 'győzelmi jelet mutató kéz: sötét bőrtónus',
    '🤞🏻' => 'keresztezett ujjak: világos bőrtónus',
    '🤞🏼' => 'keresztezett ujjak: közepesen világos bőrtónus',
    '🤞🏽' => 'keresztezett ujjak: közepes bőrtónus',
    '🤞🏾' => 'keresztezett ujjak: közepesen sötét bőrtónus',
    '🤞🏿' => 'keresztezett ujjak: sötét bőrtónus',
    '🫰🏻' => 'keresztezett mutató- és hüvelykujjú kéz: világos bőrtónus',
    '🫰🏼' => 'keresztezett mutató- és hüvelykujjú kéz: közepesen világos bőrtónus',
    '🫰🏽' => 'keresztezett mutató- és hüvelykujjú kéz: közepes bőrtónus',
    '🫰🏾' => 'keresztezett mutató- és hüvelykujjú kéz: közepesen sötét bőrtónus',
    '🫰🏿' => 'keresztezett mutató- és hüvelykujjú kéz: sötét bőrtónus',
    '🤟🏻' => 'szeretlek kézmozdulat: világos bőrtónus',
    '🤟🏼' => 'szeretlek kézmozdulat: közepesen világos bőrtónus',
    '🤟🏽' => 'szeretlek kézmozdulat: közepes bőrtónus',
    '🤟🏾' => 'szeretlek kézmozdulat: közepesen sötét bőrtónus',
    '🤟🏿' => 'szeretlek kézmozdulat: sötét bőrtónus',
    '🤘🏻' => 'metálvilla: világos bőrtónus',
    '🤘🏼' => 'metálvilla: közepesen világos bőrtónus',
    '🤘🏽' => 'metálvilla: közepes bőrtónus',
    '🤘🏾' => 'metálvilla: közepesen sötét bőrtónus',
    '🤘🏿' => 'metálvilla: sötét bőrtónus',
    '🤙🏻' => '„hívj fel” kézjel: világos bőrtónus',
    '🤙🏼' => '„hívj fel” kézjel: közepesen világos bőrtónus',
    '🤙🏽' => '„hívj fel” kézjel: közepes bőrtónus',
    '🤙🏾' => '„hívj fel” kézjel: közepesen sötét bőrtónus',
    '🤙🏿' => '„hívj fel” kézjel: sötét bőrtónus',
    '👈🏻' => 'kéz balra mutató ujjal: világos bőrtónus',
    '👈🏼' => 'kéz balra mutató ujjal: közepesen világos bőrtónus',
    '👈🏽' => 'kéz balra mutató ujjal: közepes bőrtónus',
    '👈🏾' => 'kéz balra mutató ujjal: közepesen sötét bőrtónus',
    '👈🏿' => 'kéz balra mutató ujjal: sötét bőrtónus',
    '👉🏻' => 'kéz jobbra mutató ujjal: világos bőrtónus',
    '👉🏼' => 'kéz jobbra mutató ujjal: közepesen világos bőrtónus',
    '👉🏽' => 'kéz jobbra mutató ujjal: közepes bőrtónus',
    '👉🏾' => 'kéz jobbra mutató ujjal: közepesen sötét bőrtónus',
    '👉🏿' => 'kéz jobbra mutató ujjal: sötét bőrtónus',
    '👆🏻' => 'tenyér felfelé mutató ujjal: világos bőrtónus',
    '👆🏼' => 'tenyér felfelé mutató ujjal: közepesen világos bőrtónus',
    '👆🏽' => 'tenyér felfelé mutató ujjal: közepes bőrtónus',
    '👆🏾' => 'tenyér felfelé mutató ujjal: közepesen sötét bőrtónus',
    '👆🏿' => 'tenyér felfelé mutató ujjal: sötét bőrtónus',
    '🖕🏻' => 'középső ujj: világos bőrtónus',
    '🖕🏼' => 'középső ujj: közepesen világos bőrtónus',
    '🖕🏽' => 'középső ujj: közepes bőrtónus',
    '🖕🏾' => 'középső ujj: közepesen sötét bőrtónus',
    '🖕🏿' => 'középső ujj: sötét bőrtónus',
    '👇🏻' => 'tenyér lefelé mutató ujjal: világos bőrtónus',
    '👇🏼' => 'tenyér lefelé mutató ujjal: közepesen világos bőrtónus',
    '👇🏽' => 'tenyér lefelé mutató ujjal: közepes bőrtónus',
    '👇🏾' => 'tenyér lefelé mutató ujjal: közepesen sötét bőrtónus',
    '👇🏿' => 'tenyér lefelé mutató ujjal: sötét bőrtónus',
    '☝🏻' => 'kéz felfelé mutató ujjal: világos bőrtónus',
    '☝🏼' => 'kéz felfelé mutató ujjal: közepesen világos bőrtónus',
    '☝🏽' => 'kéz felfelé mutató ujjal: közepes bőrtónus',
    '☝🏾' => 'kéz felfelé mutató ujjal: közepesen sötét bőrtónus',
    '☝🏿' => 'kéz felfelé mutató ujjal: sötét bőrtónus',
    '🫵🏻' => 'felhasználóra mutató ujj: világos bőrtónus',
    '🫵🏼' => 'felhasználóra mutató ujj: közepesen világos bőrtónus',
    '🫵🏽' => 'felhasználóra mutató ujj: közepes bőrtónus',
    '🫵🏾' => 'felhasználóra mutató ujj: közepesen sötét bőrtónus',
    '🫵🏿' => 'felhasználóra mutató ujj: sötét bőrtónus',
    '👍🏻' => 'felfelé mutató hüvelykujj: világos bőrtónus',
    '👍🏼' => 'felfelé mutató hüvelykujj: közepesen világos bőrtónus',
    '👍🏽' => 'felfelé mutató hüvelykujj: közepes bőrtónus',
    '👍🏾' => 'felfelé mutató hüvelykujj: közepesen sötét bőrtónus',
    '👍🏿' => 'felfelé mutató hüvelykujj: sötét bőrtónus',
    '👎🏻' => 'lefelé mutató hüvelykujj: világos bőrtónus',
    '👎🏼' => 'lefelé mutató hüvelykujj: közepesen világos bőrtónus',
    '👎🏽' => 'lefelé mutató hüvelykujj: közepes bőrtónus',
    '👎🏾' => 'lefelé mutató hüvelykujj: közepesen sötét bőrtónus',
    '👎🏿' => 'lefelé mutató hüvelykujj: sötét bőrtónus',
    '✊🏻' => 'felemelt ököl: világos bőrtónus',
    '✊🏼' => 'felemelt ököl: közepesen világos bőrtónus',
    '✊🏽' => 'felemelt ököl: közepes bőrtónus',
    '✊🏾' => 'felemelt ököl: közepesen sötét bőrtónus',
    '✊🏿' => 'felemelt ököl: sötét bőrtónus',
    '👊🏻' => 'ütésre emelt ököl: világos bőrtónus',
    '👊🏼' => 'ütésre emelt ököl: közepesen világos bőrtónus',
    '👊🏽' => 'ütésre emelt ököl: közepes bőrtónus',
    '👊🏾' => 'ütésre emelt ököl: közepesen sötét bőrtónus',
    '👊🏿' => 'ütésre emelt ököl: sötét bőrtónus',
    '🤛🏻' => 'balra néző ököl: világos bőrtónus',
    '🤛🏼' => 'balra néző ököl: közepesen világos bőrtónus',
    '🤛🏽' => 'balra néző ököl: közepes bőrtónus',
    '🤛🏾' => 'balra néző ököl: közepesen sötét bőrtónus',
    '🤛🏿' => 'balra néző ököl: sötét bőrtónus',
    '🤜🏻' => 'jobbra néző ököl: világos bőrtónus',
    '🤜🏼' => 'jobbra néző ököl: közepesen világos bőrtónus',
    '🤜🏽' => 'jobbra néző ököl: közepes bőrtónus',
    '🤜🏾' => 'jobbra néző ököl: közepesen sötét bőrtónus',
    '🤜🏿' => 'jobbra néző ököl: sötét bőrtónus',
    '👏🏻' => 'tapsoló kéz: világos bőrtónus',
    '👏🏼' => 'tapsoló kéz: közepesen világos bőrtónus',
    '👏🏽' => 'tapsoló kéz: közepes bőrtónus',
    '👏🏾' => 'tapsoló kéz: közepesen sötét bőrtónus',
    '👏🏿' => 'tapsoló kéz: sötét bőrtónus',
    '🙌🏻' => 'kezét felemelő ember: világos bőrtónus',
    '🙌🏼' => 'kezét felemelő ember: közepesen világos bőrtónus',
    '🙌🏽' => 'kezét felemelő ember: közepes bőrtónus',
    '🙌🏾' => 'kezét felemelő ember: közepesen sötét bőrtónus',
    '🙌🏿' => 'kezét felemelő ember: sötét bőrtónus',
    '🫶🏻' => 'szívet formázó kezek: világos bőrtónus',
    '🫶🏼' => 'szívet formázó kezek: közepesen világos bőrtónus',
    '🫶🏽' => 'szívet formázó kezek: közepes bőrtónus',
    '🫶🏾' => 'szívet formázó kezek: közepesen sötét bőrtónus',
    '🫶🏿' => 'szívet formázó kezek: sötét bőrtónus',
    '👐🏻' => 'széttárt kéz: világos bőrtónus',
    '👐🏼' => 'széttárt kéz: közepesen világos bőrtónus',
    '👐🏽' => 'széttárt kéz: közepes bőrtónus',
    '👐🏾' => 'széttárt kéz: közepesen sötét bőrtónus',
    '👐🏿' => 'széttárt kéz: sötét bőrtónus',
    '🤲🏻' => 'felfelé fordított tenyerek: világos bőrtónus',
    '🤲🏼' => 'felfelé fordított tenyerek: közepesen világos bőrtónus',
    '🤲🏽' => 'felfelé fordított tenyerek: közepes bőrtónus',
    '🤲🏾' => 'felfelé fordított tenyerek: közepesen sötét bőrtónus',
    '🤲🏿' => 'felfelé fordított tenyerek: sötét bőrtónus',
    '🤝🏻' => 'kézfogás: világos bőrtónus',
    '🤝🏼' => 'kézfogás: közepesen világos bőrtónus',
    '🤝🏽' => 'kézfogás: közepes bőrtónus',
    '🤝🏾' => 'kézfogás: közepesen sötét bőrtónus',
    '🤝🏿' => 'kézfogás: sötét bőrtónus',
    '🙏🏻' => 'összetett kéz: világos bőrtónus',
    '🙏🏼' => 'összetett kéz: közepesen világos bőrtónus',
    '🙏🏽' => 'összetett kéz: közepes bőrtónus',
    '🙏🏾' => 'összetett kéz: közepesen sötét bőrtónus',
    '🙏🏿' => 'összetett kéz: sötét bőrtónus',
    '✍🏻' => 'író kéz: világos bőrtónus',
    '✍🏼' => 'író kéz: közepesen világos bőrtónus',
    '✍🏽' => 'író kéz: közepes bőrtónus',
    '✍🏾' => 'író kéz: közepesen sötét bőrtónus',
    '✍🏿' => 'író kéz: sötét bőrtónus',
    '💅🏻' => 'körömlakk: világos bőrtónus',
    '💅🏼' => 'körömlakk: közepesen világos bőrtónus',
    '💅🏽' => 'körömlakk: közepes bőrtónus',
    '💅🏾' => 'körömlakk: közepesen sötét bőrtónus',
    '💅🏿' => 'körömlakk: sötét bőrtónus',
    '🤳🏻' => 'szelfi: világos bőrtónus',
    '🤳🏼' => 'szelfi: közepesen világos bőrtónus',
    '🤳🏽' => 'szelfi: közepes bőrtónus',
    '🤳🏾' => 'szelfi: közepesen sötét bőrtónus',
    '🤳🏿' => 'szelfi: sötét bőrtónus',
    '💪🏻' => 'befeszített bicepsz: világos bőrtónus',
    '💪🏼' => 'befeszített bicepsz: közepesen világos bőrtónus',
    '💪🏽' => 'befeszített bicepsz: közepes bőrtónus',
    '💪🏾' => 'befeszített bicepsz: közepesen sötét bőrtónus',
    '💪🏿' => 'befeszített bicepsz: sötét bőrtónus',
    '🦵🏻' => 'lábszár: világos bőrtónus',
    '🦵🏼' => 'lábszár: közepesen világos bőrtónus',
    '🦵🏽' => 'lábszár: közepes bőrtónus',
    '🦵🏾' => 'lábszár: közepesen sötét bőrtónus',
    '🦵🏿' => 'lábszár: sötét bőrtónus',
    '🦶🏻' => 'lábfej: világos bőrtónus',
    '🦶🏼' => 'lábfej: közepesen világos bőrtónus',
    '🦶🏽' => 'lábfej: közepes bőrtónus',
    '🦶🏾' => 'lábfej: közepesen sötét bőrtónus',
    '🦶🏿' => 'lábfej: sötét bőrtónus',
    '👂🏻' => 'fül: világos bőrtónus',
    '👂🏼' => 'fül: közepesen világos bőrtónus',
    '👂🏽' => 'fül: közepes bőrtónus',
    '👂🏾' => 'fül: közepesen sötét bőrtónus',
    '👂🏿' => 'fül: sötét bőrtónus',
    '🦻🏻' => 'fül hallókészülékkel: világos bőrtónus',
    '🦻🏼' => 'fül hallókészülékkel: közepesen világos bőrtónus',
    '🦻🏽' => 'fül hallókészülékkel: közepes bőrtónus',
    '🦻🏾' => 'fül hallókészülékkel: közepesen sötét bőrtónus',
    '🦻🏿' => 'fül hallókészülékkel: sötét bőrtónus',
    '👃🏻' => 'orr: világos bőrtónus',
    '👃🏼' => 'orr: közepesen világos bőrtónus',
    '👃🏽' => 'orr: közepes bőrtónus',
    '👃🏾' => 'orr: közepesen sötét bőrtónus',
    '👃🏿' => 'orr: sötét bőrtónus',
    '👶🏻' => 'baba: világos bőrtónus',
    '👶🏼' => 'baba: közepesen világos bőrtónus',
    '👶🏽' => 'baba: közepes bőrtónus',
    '👶🏾' => 'baba: közepesen sötét bőrtónus',
    '👶🏿' => 'baba: sötét bőrtónus',
    '🧒🏻' => 'gyerek: világos bőrtónus',
    '🧒🏼' => 'gyerek: közepesen világos bőrtónus',
    '🧒🏽' => 'gyerek: közepes bőrtónus',
    '🧒🏾' => 'gyerek: közepesen sötét bőrtónus',
    '🧒🏿' => 'gyerek: sötét bőrtónus',
    '👦🏻' => 'fiú: világos bőrtónus',
    '👦🏼' => 'fiú: közepesen világos bőrtónus',
    '👦🏽' => 'fiú: közepes bőrtónus',
    '👦🏾' => 'fiú: közepesen sötét bőrtónus',
    '👦🏿' => 'fiú: sötét bőrtónus',
    '👧🏻' => 'lány: világos bőrtónus',
    '👧🏼' => 'lány: közepesen világos bőrtónus',
    '👧🏽' => 'lány: közepes bőrtónus',
    '👧🏾' => 'lány: közepesen sötét bőrtónus',
    '👧🏿' => 'lány: sötét bőrtónus',
    '🧑🏻' => 'felnőtt: világos bőrtónus',
    '🧑🏼' => 'felnőtt: közepesen világos bőrtónus',
    '🧑🏽' => 'felnőtt: közepes bőrtónus',
    '🧑🏾' => 'felnőtt: közepesen sötét bőrtónus',
    '🧑🏿' => 'felnőtt: sötét bőrtónus',
    '👱🏻' => 'szőke hajú ember: világos bőrtónus',
    '👱🏼' => 'szőke hajú ember: közepesen világos bőrtónus',
    '👱🏽' => 'szőke hajú ember: közepes bőrtónus',
    '👱🏾' => 'szőke hajú ember: közepesen sötét bőrtónus',
    '👱🏿' => 'szőke hajú ember: sötét bőrtónus',
    '👨🏻' => 'férfi: világos bőrtónus',
    '👨🏼' => 'férfi: közepesen világos bőrtónus',
    '👨🏽' => 'férfi: közepes bőrtónus',
    '👨🏾' => 'férfi: közepesen sötét bőrtónus',
    '👨🏿' => 'férfi: sötét bőrtónus',
    '🧔🏻' => 'szakállas ember: világos bőrtónus',
    '🧔🏼' => 'szakállas ember: közepesen világos bőrtónus',
    '🧔🏽' => 'szakállas ember: közepes bőrtónus',
    '🧔🏾' => 'szakállas ember: közepesen sötét bőrtónus',
    '🧔🏿' => 'szakállas ember: sötét bőrtónus',
    '👩🏻' => 'nő: világos bőrtónus',
    '👩🏼' => 'nő: közepesen világos bőrtónus',
    '👩🏽' => 'nő: közepes bőrtónus',
    '👩🏾' => 'nő: közepesen sötét bőrtónus',
    '👩🏿' => 'nő: sötét bőrtónus',
    '🧓🏻' => 'idősebb felnőtt: világos bőrtónus',
    '🧓🏼' => 'idősebb felnőtt: közepesen világos bőrtónus',
    '🧓🏽' => 'idősebb felnőtt: közepes bőrtónus',
    '🧓🏾' => 'idősebb felnőtt: közepesen sötét bőrtónus',
    '🧓🏿' => 'idősebb felnőtt: sötét bőrtónus',
    '👴🏻' => 'idős férfi: világos bőrtónus',
    '👴🏼' => 'idős férfi: közepesen világos bőrtónus',
    '👴🏽' => 'idős férfi: közepes bőrtónus',
    '👴🏾' => 'idős férfi: közepesen sötét bőrtónus',
    '👴🏿' => 'idős férfi: sötét bőrtónus',
    '👵🏻' => 'idős nő: világos bőrtónus',
    '👵🏼' => 'idős nő: közepesen világos bőrtónus',
    '👵🏽' => 'idős nő: közepes bőrtónus',
    '👵🏾' => 'idős nő: közepesen sötét bőrtónus',
    '👵🏿' => 'idős nő: sötét bőrtónus',
    '🙍🏻' => 'rosszalló ember: világos bőrtónus',
    '🙍🏼' => 'rosszalló ember: közepesen világos bőrtónus',
    '🙍🏽' => 'rosszalló ember: közepes bőrtónus',
    '🙍🏾' => 'rosszalló ember: közepesen sötét bőrtónus',
    '🙍🏿' => 'rosszalló ember: sötét bőrtónus',
    '🙎🏻' => 'duzzogó ember: világos bőrtónus',
    '🙎🏼' => 'duzzogó ember: közepesen világos bőrtónus',
    '🙎🏽' => 'duzzogó ember: közepes bőrtónus',
    '🙎🏾' => 'duzzogó ember: közepesen sötét bőrtónus',
    '🙎🏿' => 'duzzogó ember: sötét bőrtónus',
    '🙅🏻' => 'tiltás mutatása: világos bőrtónus',
    '🙅🏼' => 'tiltás mutatása: közepesen világos bőrtónus',
    '🙅🏽' => 'tiltás mutatása: közepes bőrtónus',
    '🙅🏾' => 'tiltás mutatása: közepesen sötét bőrtónus',
    '🙅🏿' => 'tiltás mutatása: sötét bőrtónus',
    '🙆🏻' => 'beleegyezés mutatása: világos bőrtónus',
    '🙆🏼' => 'beleegyezés mutatása: közepesen világos bőrtónus',
    '🙆🏽' => 'beleegyezés mutatása: közepes bőrtónus',
    '🙆🏾' => 'beleegyezés mutatása: közepesen sötét bőrtónus',
    '🙆🏿' => 'beleegyezés mutatása: sötét bőrtónus',
    '💁🏻' => 'ember az információs pultnál: világos bőrtónus',
    '💁🏼' => 'ember az információs pultnál: közepesen világos bőrtónus',
    '💁🏽' => 'ember az információs pultnál: közepes bőrtónus',
    '💁🏾' => 'ember az információs pultnál: közepesen sötét bőrtónus',
    '💁🏿' => 'ember az információs pultnál: sötét bőrtónus',
    '🙋🏻' => 'vidáman intő ember: világos bőrtónus',
    '🙋🏼' => 'vidáman intő ember: közepesen világos bőrtónus',
    '🙋🏽' => 'vidáman intő ember: közepes bőrtónus',
    '🙋🏾' => 'vidáman intő ember: közepesen sötét bőrtónus',
    '🙋🏿' => 'vidáman intő ember: sötét bőrtónus',
    '🧏🏻' => 'siket ember: világos bőrtónus',
    '🧏🏼' => 'siket ember: közepesen világos bőrtónus',
    '🧏🏽' => 'siket ember: közepes bőrtónus',
    '🧏🏾' => 'siket ember: közepesen sötét bőrtónus',
    '🧏🏿' => 'siket ember: sötét bőrtónus',
    '🙇🏻' => 'meghajló ember: világos bőrtónus',
    '🙇🏼' => 'meghajló ember: közepesen világos bőrtónus',
    '🙇🏽' => 'meghajló ember: közepes bőrtónus',
    '🙇🏾' => 'meghajló ember: közepesen sötét bőrtónus',
    '🙇🏿' => 'meghajló ember: sötét bőrtónus',
    '🤦🏻' => 'kézbe temetett arc: világos bőrtónus',
    '🤦🏼' => 'kézbe temetett arc: közepesen világos bőrtónus',
    '🤦🏽' => 'kézbe temetett arc: közepes bőrtónus',
    '🤦🏾' => 'kézbe temetett arc: közepesen sötét bőrtónus',
    '🤦🏿' => 'kézbe temetett arc: sötét bőrtónus',
    '🤷🏻' => 'vállrándítás: világos bőrtónus',
    '🤷🏼' => 'vállrándítás: közepesen világos bőrtónus',
    '🤷🏽' => 'vállrándítás: közepes bőrtónus',
    '🤷🏾' => 'vállrándítás: közepesen sötét bőrtónus',
    '🤷🏿' => 'vállrándítás: sötét bőrtónus',
    '👮🏻' => 'zsaru: világos bőrtónus',
    '👮🏼' => 'zsaru: közepesen világos bőrtónus',
    '👮🏽' => 'zsaru: közepes bőrtónus',
    '👮🏾' => 'zsaru: közepesen sötét bőrtónus',
    '👮🏿' => 'zsaru: sötét bőrtónus',
    '🕵🏻' => 'detektív: világos bőrtónus',
    '🕵🏼' => 'detektív: közepesen világos bőrtónus',
    '🕵🏽' => 'detektív: közepes bőrtónus',
    '🕵🏾' => 'detektív: közepesen sötét bőrtónus',
    '🕵🏿' => 'detektív: sötét bőrtónus',
    '💂🏻' => 'gárdista: világos bőrtónus',
    '💂🏼' => 'gárdista: közepesen világos bőrtónus',
    '💂🏽' => 'gárdista: közepes bőrtónus',
    '💂🏾' => 'gárdista: közepesen sötét bőrtónus',
    '💂🏿' => 'gárdista: sötét bőrtónus',
    '🥷🏻' => 'nindzsa: világos bőrtónus',
    '🥷🏼' => 'nindzsa: közepesen világos bőrtónus',
    '🥷🏽' => 'nindzsa: közepes bőrtónus',
    '🥷🏾' => 'nindzsa: közepesen sötét bőrtónus',
    '🥷🏿' => 'nindzsa: sötét bőrtónus',
    '👷🏻' => 'építőmunkás: világos bőrtónus',
    '👷🏼' => 'építőmunkás: közepesen világos bőrtónus',
    '👷🏽' => 'építőmunkás: közepes bőrtónus',
    '👷🏾' => 'építőmunkás: közepesen sötét bőrtónus',
    '👷🏿' => 'építőmunkás: sötét bőrtónus',
    '🫅🏻' => 'koronás személy: világos bőrtónus',
    '🫅🏼' => 'koronás személy: közepesen világos bőrtónus',
    '🫅🏽' => 'koronás személy: közepes bőrtónus',
    '🫅🏾' => 'koronás személy: közepesen sötét bőrtónus',
    '🫅🏿' => 'koronás személy: sötét bőrtónus',
    '🤴🏻' => 'herceg: világos bőrtónus',
    '🤴🏼' => 'herceg: közepesen világos bőrtónus',
    '🤴🏽' => 'herceg: közepes bőrtónus',
    '🤴🏾' => 'herceg: közepesen sötét bőrtónus',
    '🤴🏿' => 'herceg: sötét bőrtónus',
    '👸🏻' => 'hercegnő: világos bőrtónus',
    '👸🏼' => 'hercegnő: közepesen világos bőrtónus',
    '👸🏽' => 'hercegnő: közepes bőrtónus',
    '👸🏾' => 'hercegnő: közepesen sötét bőrtónus',
    '👸🏿' => 'hercegnő: sötét bőrtónus',
    '👳🏻' => 'turbános ember: világos bőrtónus',
    '👳🏼' => 'turbános ember: közepesen világos bőrtónus',
    '👳🏽' => 'turbános ember: közepes bőrtónus',
    '👳🏾' => 'turbános ember: közepesen sötét bőrtónus',
    '👳🏿' => 'turbános ember: sötét bőrtónus',
    '👲🏻' => 'kínai sapkás ember: világos bőrtónus',
    '👲🏼' => 'kínai sapkás ember: közepesen világos bőrtónus',
    '👲🏽' => 'kínai sapkás ember: közepes bőrtónus',
    '👲🏾' => 'kínai sapkás ember: közepesen sötét bőrtónus',
    '👲🏿' => 'kínai sapkás ember: sötét bőrtónus',
    '🧕🏻' => 'nő fejkendővel: világos bőrtónus',
    '🧕🏼' => 'nő fejkendővel: közepesen világos bőrtónus',
    '🧕🏽' => 'nő fejkendővel: közepes bőrtónus',
    '🧕🏾' => 'nő fejkendővel: közepesen sötét bőrtónus',
    '🧕🏿' => 'nő fejkendővel: sötét bőrtónus',
    '🤵🏻' => 'szmokingos ember: világos bőrtónus',
    '🤵🏼' => 'szmokingos ember: közepesen világos bőrtónus',
    '🤵🏽' => 'szmokingos ember: közepes bőrtónus',
    '🤵🏾' => 'szmokingos ember: közepesen sötét bőrtónus',
    '🤵🏿' => 'szmokingos ember: sötét bőrtónus',
    '👰🏻' => 'ember fátyollal: világos bőrtónus',
    '👰🏼' => 'ember fátyollal: közepesen világos bőrtónus',
    '👰🏽' => 'ember fátyollal: közepes bőrtónus',
    '👰🏾' => 'ember fátyollal: közepesen sötét bőrtónus',
    '👰🏿' => 'ember fátyollal: sötét bőrtónus',
    '🤰🏻' => 'terhes nő: világos bőrtónus',
    '🤰🏼' => 'terhes nő: közepesen világos bőrtónus',
    '🤰🏽' => 'terhes nő: közepes bőrtónus',
    '🤰🏾' => 'terhes nő: közepesen sötét bőrtónus',
    '🤰🏿' => 'terhes nő: sötét bőrtónus',
    '🫃🏻' => 'terhes férfi: világos bőrtónus',
    '🫃🏼' => 'terhes férfi: közepesen világos bőrtónus',
    '🫃🏽' => 'terhes férfi: közepes bőrtónus',
    '🫃🏾' => 'terhes férfi: közepesen sötét bőrtónus',
    '🫃🏿' => 'terhes férfi: sötét bőrtónus',
    '🫄🏻' => 'terhes személy: világos bőrtónus',
    '🫄🏼' => 'terhes személy: közepesen világos bőrtónus',
    '🫄🏽' => 'terhes személy: közepes bőrtónus',
    '🫄🏾' => 'terhes személy: közepesen sötét bőrtónus',
    '🫄🏿' => 'terhes személy: sötét bőrtónus',
    '🤱🏻' => 'szoptatás: világos bőrtónus',
    '🤱🏼' => 'szoptatás: közepesen világos bőrtónus',
    '🤱🏽' => 'szoptatás: közepes bőrtónus',
    '🤱🏾' => 'szoptatás: közepesen sötét bőrtónus',
    '🤱🏿' => 'szoptatás: sötét bőrtónus',
    '👼🏻' => 'babaangyal: világos bőrtónus',
    '👼🏼' => 'babaangyal: közepesen világos bőrtónus',
    '👼🏽' => 'babaangyal: közepes bőrtónus',
    '👼🏾' => 'babaangyal: közepesen sötét bőrtónus',
    '👼🏿' => 'babaangyal: sötét bőrtónus',
    '🎅🏻' => 'mikulás: világos bőrtónus',
    '🎅🏼' => 'mikulás: közepesen világos bőrtónus',
    '🎅🏽' => 'mikulás: közepes bőrtónus',
    '🎅🏾' => 'mikulás: közepesen sötét bőrtónus',
    '🎅🏿' => 'mikulás: sötét bőrtónus',
    '🤶🏻' => 'mikulás ruhás anyó: világos bőrtónus',
    '🤶🏼' => 'mikulás ruhás anyó: közepesen világos bőrtónus',
    '🤶🏽' => 'mikulás ruhás anyó: közepes bőrtónus',
    '🤶🏾' => 'mikulás ruhás anyó: közepesen sötét bőrtónus',
    '🤶🏿' => 'mikulás ruhás anyó: sötét bőrtónus',
    '🦸🏻' => 'szuperhős: világos bőrtónus',
    '🦸🏼' => 'szuperhős: közepesen világos bőrtónus',
    '🦸🏽' => 'szuperhős: közepes bőrtónus',
    '🦸🏾' => 'szuperhős: közepesen sötét bőrtónus',
    '🦸🏿' => 'szuperhős: sötét bőrtónus',
    '🦹🏻' => 'főgonosz: világos bőrtónus',
    '🦹🏼' => 'főgonosz: közepesen világos bőrtónus',
    '🦹🏽' => 'főgonosz: közepes bőrtónus',
    '🦹🏾' => 'főgonosz: közepesen sötét bőrtónus',
    '🦹🏿' => 'főgonosz: sötét bőrtónus',
    '🧙🏻' => 'varázsló: világos bőrtónus',
    '🧙🏼' => 'varázsló: közepesen világos bőrtónus',
    '🧙🏽' => 'varázsló: közepes bőrtónus',
    '🧙🏾' => 'varázsló: közepesen sötét bőrtónus',
    '🧙🏿' => 'varázsló: sötét bőrtónus',
    '🧚🏻' => 'tündér: világos bőrtónus',
    '🧚🏼' => 'tündér: közepesen világos bőrtónus',
    '🧚🏽' => 'tündér: közepes bőrtónus',
    '🧚🏾' => 'tündér: közepesen sötét bőrtónus',
    '🧚🏿' => 'tündér: sötét bőrtónus',
    '🧛🏻' => 'vámpír: világos bőrtónus',
    '🧛🏼' => 'vámpír: közepesen világos bőrtónus',
    '🧛🏽' => 'vámpír: közepes bőrtónus',
    '🧛🏾' => 'vámpír: közepesen sötét bőrtónus',
    '🧛🏿' => 'vámpír: sötét bőrtónus',
    '🧜🏻' => 'sellő: világos bőrtónus',
    '🧜🏼' => 'sellő: közepesen világos bőrtónus',
    '🧜🏽' => 'sellő: közepes bőrtónus',
    '🧜🏾' => 'sellő: közepesen sötét bőrtónus',
    '🧜🏿' => 'sellő: sötét bőrtónus',
    '🧝🏻' => 'tünde: világos bőrtónus',
    '🧝🏼' => 'tünde: közepesen világos bőrtónus',
    '🧝🏽' => 'tünde: közepes bőrtónus',
    '🧝🏾' => 'tünde: közepesen sötét bőrtónus',
    '🧝🏿' => 'tünde: sötét bőrtónus',
    '💆🏻' => 'arcmasszázs: világos bőrtónus',
    '💆🏼' => 'arcmasszázs: közepesen világos bőrtónus',
    '💆🏽' => 'arcmasszázs: közepes bőrtónus',
    '💆🏾' => 'arcmasszázs: közepesen sötét bőrtónus',
    '💆🏿' => 'arcmasszázs: sötét bőrtónus',
    '💇🏻' => 'hajvágás: világos bőrtónus',
    '💇🏼' => 'hajvágás: közepesen világos bőrtónus',
    '💇🏽' => 'hajvágás: közepes bőrtónus',
    '💇🏾' => 'hajvágás: közepesen sötét bőrtónus',
    '💇🏿' => 'hajvágás: sötét bőrtónus',
    '🚶🏻' => 'gyalogos: világos bőrtónus',
    '🚶🏼' => 'gyalogos: közepesen világos bőrtónus',
    '🚶🏽' => 'gyalogos: közepes bőrtónus',
    '🚶🏾' => 'gyalogos: közepesen sötét bőrtónus',
    '🚶🏿' => 'gyalogos: sötét bőrtónus',
    '🧍🏻' => 'álló ember: világos bőrtónus',
    '🧍🏼' => 'álló ember: közepesen világos bőrtónus',
    '🧍🏽' => 'álló ember: közepes bőrtónus',
    '🧍🏾' => 'álló ember: közepesen sötét bőrtónus',
    '🧍🏿' => 'álló ember: sötét bőrtónus',
    '🧎🏻' => 'térdelő ember: világos bőrtónus',
    '🧎🏼' => 'térdelő ember: közepesen világos bőrtónus',
    '🧎🏽' => 'térdelő ember: közepes bőrtónus',
    '🧎🏾' => 'térdelő ember: közepesen sötét bőrtónus',
    '🧎🏿' => 'térdelő ember: sötét bőrtónus',
    '🏃🏻' => 'futó: világos bőrtónus',
    '🏃🏼' => 'futó: közepesen világos bőrtónus',
    '🏃🏽' => 'futó: közepes bőrtónus',
    '🏃🏾' => 'futó: közepesen sötét bőrtónus',
    '🏃🏿' => 'futó: sötét bőrtónus',
    '💃🏻' => 'táncoló nő: világos bőrtónus',
    '💃🏼' => 'táncoló nő: közepesen világos bőrtónus',
    '💃🏽' => 'táncoló nő: közepes bőrtónus',
    '💃🏾' => 'táncoló nő: közepesen sötét bőrtónus',
    '💃🏿' => 'táncoló nő: sötét bőrtónus',
    '🕺🏻' => 'táncoló férfi: világos bőrtónus',
    '🕺🏼' => 'táncoló férfi: közepesen világos bőrtónus',
    '🕺🏽' => 'táncoló férfi: közepes bőrtónus',
    '🕺🏾' => 'táncoló férfi: közepesen sötét bőrtónus',
    '🕺🏿' => 'táncoló férfi: sötét bőrtónus',
    '🕴🏻' => 'öltönyös, lebegő ember: világos bőrtónus',
    '🕴🏼' => 'öltönyös, lebegő ember: közepesen világos bőrtónus',
    '🕴🏽' => 'öltönyös, lebegő ember: közepes bőrtónus',
    '🕴🏾' => 'öltönyös, lebegő ember: közepesen sötét bőrtónus',
    '🕴🏿' => 'öltönyös, lebegő ember: sötét bőrtónus',
    '👯🏻' => 'bulizó emberek: világos bőrtónus',
    '👯🏼' => 'bulizó emberek: közepesen világos bőrtónus',
    '👯🏽' => 'bulizó emberek: közepes bőrtónus',
    '👯🏾' => 'bulizó emberek: közepesen sötét bőrtónus',
    '👯🏿' => 'bulizó emberek: sötét bőrtónus',
    '🧖🏻' => 'ember a gőzfürdőben: világos bőrtónus',
    '🧖🏼' => 'ember a gőzfürdőben: közepesen világos bőrtónus',
    '🧖🏽' => 'ember a gőzfürdőben: közepes bőrtónus',
    '🧖🏾' => 'ember a gőzfürdőben: közepesen sötét bőrtónus',
    '🧖🏿' => 'ember a gőzfürdőben: sötét bőrtónus',
    '🧗🏻' => 'mászó ember: világos bőrtónus',
    '🧗🏼' => 'mászó ember: közepesen világos bőrtónus',
    '🧗🏽' => 'mászó ember: közepes bőrtónus',
    '🧗🏾' => 'mászó ember: közepesen sötét bőrtónus',
    '🧗🏿' => 'mászó ember: sötét bőrtónus',
    '🏇🏻' => 'lóverseny: világos bőrtónus',
    '🏇🏼' => 'lóverseny: közepesen világos bőrtónus',
    '🏇🏽' => 'lóverseny: közepes bőrtónus',
    '🏇🏾' => 'lóverseny: közepesen sötét bőrtónus',
    '🏇🏿' => 'lóverseny: sötét bőrtónus',
    '🏂🏻' => 'snowboardozó: világos bőrtónus',
    '🏂🏼' => 'snowboardozó: közepesen világos bőrtónus',
    '🏂🏽' => 'snowboardozó: közepes bőrtónus',
    '🏂🏾' => 'snowboardozó: közepesen sötét bőrtónus',
    '🏂🏿' => 'snowboardozó: sötét bőrtónus',
    '🏌🏻' => 'golfozó: világos bőrtónus',
    '🏌🏼' => 'golfozó: közepesen világos bőrtónus',
    '🏌🏽' => 'golfozó: közepes bőrtónus',
    '🏌🏾' => 'golfozó: közepesen sötét bőrtónus',
    '🏌🏿' => 'golfozó: sötét bőrtónus',
    '🏄🏻' => 'szörföző: világos bőrtónus',
    '🏄🏼' => 'szörföző: közepesen világos bőrtónus',
    '🏄🏽' => 'szörföző: közepes bőrtónus',
    '🏄🏾' => 'szörföző: közepesen sötét bőrtónus',
    '🏄🏿' => 'szörföző: sötét bőrtónus',
    '🚣🏻' => 'evező ember: világos bőrtónus',
    '🚣🏼' => 'evező ember: közepesen világos bőrtónus',
    '🚣🏽' => 'evező ember: közepes bőrtónus',
    '🚣🏾' => 'evező ember: közepesen sötét bőrtónus',
    '🚣🏿' => 'evező ember: sötét bőrtónus',
    '🏊🏻' => 'úszó: világos bőrtónus',
    '🏊🏼' => 'úszó: közepesen világos bőrtónus',
    '🏊🏽' => 'úszó: közepes bőrtónus',
    '🏊🏾' => 'úszó: közepesen sötét bőrtónus',
    '🏊🏿' => 'úszó: sötét bőrtónus',
    '⛹🏻' => 'ember labdával: világos bőrtónus',
    '⛹🏼' => 'ember labdával: közepesen világos bőrtónus',
    '⛹🏽' => 'ember labdával: közepes bőrtónus',
    '⛹🏾' => 'ember labdával: közepesen sötét bőrtónus',
    '⛹🏿' => 'ember labdával: sötét bőrtónus',
    '🏋🏻' => 'súlyemelő: világos bőrtónus',
    '🏋🏼' => 'súlyemelő: közepesen világos bőrtónus',
    '🏋🏽' => 'súlyemelő: közepes bőrtónus',
    '🏋🏾' => 'súlyemelő: közepesen sötét bőrtónus',
    '🏋🏿' => 'súlyemelő: sötét bőrtónus',
    '🚴🏻' => 'kerékpáros: világos bőrtónus',
    '🚴🏼' => 'kerékpáros: közepesen világos bőrtónus',
    '🚴🏽' => 'kerékpáros: közepes bőrtónus',
    '🚴🏾' => 'kerékpáros: közepesen sötét bőrtónus',
    '🚴🏿' => 'kerékpáros: sötét bőrtónus',
    '🚵🏻' => 'hegyikerékpáros: világos bőrtónus',
    '🚵🏼' => 'hegyikerékpáros: közepesen világos bőrtónus',
    '🚵🏽' => 'hegyikerékpáros: közepes bőrtónus',
    '🚵🏾' => 'hegyikerékpáros: közepesen sötét bőrtónus',
    '🚵🏿' => 'hegyikerékpáros: sötét bőrtónus',
    '🤸🏻' => 'cigánykerék: világos bőrtónus',
    '🤸🏼' => 'cigánykerék: közepesen világos bőrtónus',
    '🤸🏽' => 'cigánykerék: közepes bőrtónus',
    '🤸🏾' => 'cigánykerék: közepesen sötét bőrtónus',
    '🤸🏿' => 'cigánykerék: sötét bőrtónus',
    '🤼🏻' => 'birkózók: világos bőrtónus',
    '🤼🏼' => 'birkózók: közepesen világos bőrtónus',
    '🤼🏽' => 'birkózók: közepes bőrtónus',
    '🤼🏾' => 'birkózók: közepesen sötét bőrtónus',
    '🤼🏿' => 'birkózók: sötét bőrtónus',
    '🤽🏻' => 'vízilabda: világos bőrtónus',
    '🤽🏼' => 'vízilabda: közepesen világos bőrtónus',
    '🤽🏽' => 'vízilabda: közepes bőrtónus',
    '🤽🏾' => 'vízilabda: közepesen sötét bőrtónus',
    '🤽🏿' => 'vízilabda: sötét bőrtónus',
    '🤾🏻' => 'kézilabda: világos bőrtónus',
    '🤾🏼' => 'kézilabda: közepesen világos bőrtónus',
    '🤾🏽' => 'kézilabda: közepes bőrtónus',
    '🤾🏾' => 'kézilabda: közepesen sötét bőrtónus',
    '🤾🏿' => 'kézilabda: sötét bőrtónus',
    '🤹🏻' => 'zsonglőr: világos bőrtónus',
    '🤹🏼' => 'zsonglőr: közepesen világos bőrtónus',
    '🤹🏽' => 'zsonglőr: közepes bőrtónus',
    '🤹🏾' => 'zsonglőr: közepesen sötét bőrtónus',
    '🤹🏿' => 'zsonglőr: sötét bőrtónus',
    '🧘🏻' => 'ember lótuszülésben: világos bőrtónus',
    '🧘🏼' => 'ember lótuszülésben: közepesen világos bőrtónus',
    '🧘🏽' => 'ember lótuszülésben: közepes bőrtónus',
    '🧘🏾' => 'ember lótuszülésben: közepesen sötét bőrtónus',
    '🧘🏿' => 'ember lótuszülésben: sötét bőrtónus',
    '🛀🏻' => 'fürdő ember: világos bőrtónus',
    '🛀🏼' => 'fürdő ember: közepesen világos bőrtónus',
    '🛀🏽' => 'fürdő ember: közepes bőrtónus',
    '🛀🏾' => 'fürdő ember: közepesen sötét bőrtónus',
    '🛀🏿' => 'fürdő ember: sötét bőrtónus',
    '🛌🏻' => 'ember az ágyban: világos bőrtónus',
    '🛌🏼' => 'ember az ágyban: közepesen világos bőrtónus',
    '🛌🏽' => 'ember az ágyban: közepes bőrtónus',
    '🛌🏾' => 'ember az ágyban: közepesen sötét bőrtónus',
    '🛌🏿' => 'ember az ágyban: sötét bőrtónus',
    '👭🏻' => 'kézen fogva álló nők: világos bőrtónus',
    '👭🏼' => 'kézen fogva álló nők: közepesen világos bőrtónus',
    '👭🏽' => 'kézen fogva álló nők: közepes bőrtónus',
    '👭🏾' => 'kézen fogva álló nők: közepesen sötét bőrtónus',
    '👭🏿' => 'kézen fogva álló nők: sötét bőrtónus',
    '👫🏻' => 'kézen fogva álló férfi és nő: világos bőrtónus',
    '👫🏼' => 'kézen fogva álló férfi és nő: közepesen világos bőrtónus',
    '👫🏽' => 'kézen fogva álló férfi és nő: közepes bőrtónus',
    '👫🏾' => 'kézen fogva álló férfi és nő: közepesen sötét bőrtónus',
    '👫🏿' => 'kézen fogva álló férfi és nő: sötét bőrtónus',
    '👬🏻' => 'kézen fogva álló férfiak: világos bőrtónus',
    '👬🏼' => 'kézen fogva álló férfiak: közepesen világos bőrtónus',
    '👬🏽' => 'kézen fogva álló férfiak: közepes bőrtónus',
    '👬🏾' => 'kézen fogva álló férfiak: közepesen sötét bőrtónus',
    '👬🏿' => 'kézen fogva álló férfiak: sötét bőrtónus',
    '💏🏻' => 'csók: világos bőrtónus',
    '💏🏼' => 'csók: közepesen világos bőrtónus',
    '💏🏽' => 'csók: közepes bőrtónus',
    '💏🏾' => 'csók: közepesen sötét bőrtónus',
    '💏🏿' => 'csók: sötét bőrtónus',
    '💑🏻' => 'pár szívvel: világos bőrtónus',
    '💑🏼' => 'pár szívvel: közepesen világos bőrtónus',
    '💑🏽' => 'pár szívvel: közepes bőrtónus',
    '💑🏾' => 'pár szívvel: közepesen sötét bőrtónus',
    '💑🏿' => 'pár szívvel: sötét bőrtónus',
    '🇦🇨' => 'zászló: Ascension-sziget',
    '🇦🇩' => 'zászló: Andorra',
    '🇦🇪' => 'zászló: Egyesült Arab Emírségek',
    '🇦🇫' => 'zászló: Afganisztán',
    '🇦🇬' => 'zászló: Antigua és Barbuda',
    '🇦🇮' => 'zászló: Anguilla',
    '🇦🇱' => 'zászló: Albánia',
    '🇦🇲' => 'zászló: Örményország',
    '🇦🇴' => 'zászló: Angola',
    '🇦🇶' => 'zászló: Antarktisz',
    '🇦🇷' => 'zászló: Argentína',
    '🇦🇸' => 'zászló: Amerikai Szamoa',
    '🇦🇹' => 'zászló: Ausztria',
    '🇦🇺' => 'zászló: Ausztrália',
    '🇦🇼' => 'zászló: Aruba',
    '🇦🇽' => 'zászló: Åland-szigetek',
    '🇦🇿' => 'zászló: Azerbajdzsán',
    '🇧🇦' => 'zászló: Bosznia-Hercegovina',
    '🇧🇧' => 'zászló: Barbados',
    '🇧🇩' => 'zászló: Banglades',
    '🇧🇪' => 'zászló: Belgium',
    '🇧🇫' => 'zászló: Burkina Faso',
    '🇧🇬' => 'zászló: Bulgária',
    '🇧🇭' => 'zászló: Bahrein',
    '🇧🇮' => 'zászló: Burundi',
    '🇧🇯' => 'zászló: Benin',
    '🇧🇱' => 'zászló: Saint-Barthélemy',
    '🇧🇲' => 'zászló: Bermuda',
    '🇧🇳' => 'zászló: Brunei',
    '🇧🇴' => 'zászló: Bolívia',
    '🇧🇶' => 'zászló: Holland Karib-térség',
    '🇧🇷' => 'zászló: Brazília',
    '🇧🇸' => 'zászló: Bahama-szigetek',
    '🇧🇹' => 'zászló: Bhután',
    '🇧🇻' => 'zászló: Bouvet-sziget',
    '🇧🇼' => 'zászló: Botswana',
    '🇧🇾' => 'zászló: Belarusz',
    '🇧🇿' => 'zászló: Belize',
    '🇨🇦' => 'zászló: Kanada',
    '🇨🇨' => 'zászló: Kókusz (Keeling)-szigetek',
    '🇨🇩' => 'zászló: Kongó – Kinshasa',
    '🇨🇫' => 'zászló: Közép-afrikai Köztársaság',
    '🇨🇬' => 'zászló: Kongó – Brazzaville',
    '🇨🇭' => 'zászló: Svájc',
    '🇨🇮' => 'zászló: Elefántcsontpart',
    '🇨🇰' => 'zászló: Cook-szigetek',
    '🇨🇱' => 'zászló: Chile',
    '🇨🇲' => 'zászló: Kamerun',
    '🇨🇳' => 'zászló: Kína',
    '🇨🇴' => 'zászló: Kolumbia',
    '🇨🇵' => 'zászló: Clipperton-sziget',
    '🇨🇶' => 'zászló: Sark',
    '🇨🇷' => 'zászló: Costa Rica',
    '🇨🇺' => 'zászló: Kuba',
    '🇨🇻' => 'zászló: Zöld-foki Köztársaság',
    '🇨🇼' => 'zászló: Curaçao',
    '🇨🇽' => 'zászló: Karácsony-sziget',
    '🇨🇾' => 'zászló: Ciprus',
    '🇨🇿' => 'zászló: Csehország',
    '🇩🇪' => 'zászló: Németország',
    '🇩🇬' => 'zászló: Diego Garcia',
    '🇩🇯' => 'zászló: Dzsibuti',
    '🇩🇰' => 'zászló: Dánia',
    '🇩🇲' => 'zászló: Dominika',
    '🇩🇴' => 'zászló: Dominikai Köztársaság',
    '🇩🇿' => 'zászló: Algéria',
    '🇪🇦' => 'zászló: Ceuta és Melilla',
    '🇪🇨' => 'zászló: Ecuador',
    '🇪🇪' => 'zászló: Észtország',
    '🇪🇬' => 'zászló: Egyiptom',
    '🇪🇭' => 'zászló: Nyugat-Szahara',
    '🇪🇷' => 'zászló: Eritrea',
    '🇪🇸' => 'zászló: Spanyolország',
    '🇪🇹' => 'zászló: Etiópia',
    '🇪🇺' => 'zászló: Európai Unió',
    '🇫🇮' => 'zászló: Finnország',
    '🇫🇯' => 'zászló: Fidzsi',
    '🇫🇰' => 'zászló: Falkland-szigetek',
    '🇫🇲' => 'zászló: Mikronézia',
    '🇫🇴' => 'zászló: Feröer szigetek',
    '🇫🇷' => 'zászló: Franciaország',
    '🇬🇦' => 'zászló: Gabon',
    '🇬🇧' => 'zászló: Egyesült Királyság',
    '🇬🇩' => 'zászló: Grenada',
    '🇬🇪' => 'zászló: Grúzia',
    '🇬🇫' => 'zászló: Francia Guyana',
    '🇬🇬' => 'zászló: Guernsey',
    '🇬🇭' => 'zászló: Ghána',
    '🇬🇮' => 'zászló: Gibraltár',
    '🇬🇱' => 'zászló: Grönland',
    '🇬🇲' => 'zászló: Gambia',
    '🇬🇳' => 'zászló: Guinea',
    '🇬🇵' => 'zászló: Guadeloupe',
    '🇬🇶' => 'zászló: Egyenlítői-Guinea',
    '🇬🇷' => 'zászló: Görögország',
    '🇬🇸' => 'zászló: Déli-Georgia és Déli-Sandwich-szigetek',
    '🇬🇹' => 'zászló: Guatemala',
    '🇬🇺' => 'zászló: Guam',
    '🇬🇼' => 'zászló: Bissau-Guinea',
    '🇬🇾' => 'zászló: Guyana',
    '🇭🇰' => 'zászló: Hongkong KKT',
    '🇭🇲' => 'zászló: Heard-sziget és McDonald-szigetek',
    '🇭🇳' => 'zászló: Honduras',
    '🇭🇷' => 'zászló: Horvátország',
    '🇭🇹' => 'zászló: Haiti',
    '🇭🇺' => 'zászló: Magyarország',
    '🇮🇨' => 'zászló: Kanári-szigetek',
    '🇮🇩' => 'zászló: Indonézia',
    '🇮🇪' => 'zászló: Írország',
    '🇮🇱' => 'zászló: Izrael',
    '🇮🇲' => 'zászló: Man-sziget',
    '🇮🇳' => 'zászló: India',
    '🇮🇴' => 'zászló: Brit Indiai-óceáni Terület',
    '🇮🇶' => 'zászló: Irak',
    '🇮🇷' => 'zászló: Irán',
    '🇮🇸' => 'zászló: Izland',
    '🇮🇹' => 'zászló: Olaszország',
    '🇯🇪' => 'zászló: Jersey',
    '🇯🇲' => 'zászló: Jamaica',
    '🇯🇴' => 'zászló: Jordánia',
    '🇯🇵' => 'zászló: Japán',
    '🇰🇪' => 'zászló: Kenya',
    '🇰🇬' => 'zászló: Kirgizisztán',
    '🇰🇭' => 'zászló: Kambodzsa',
    '🇰🇮' => 'zászló: Kiribati',
    '🇰🇲' => 'zászló: Comore-szigetek',
    '🇰🇳' => 'zászló: Saint Kitts és Nevis',
    '🇰🇵' => 'zászló: Észak-Korea',
    '🇰🇷' => 'zászló: Dél-Korea',
    '🇰🇼' => 'zászló: Kuvait',
    '🇰🇾' => 'zászló: Kajmán-szigetek',
    '🇰🇿' => 'zászló: Kazahsztán',
    '🇱🇦' => 'zászló: Laosz',
    '🇱🇧' => 'zászló: Libanon',
    '🇱🇨' => 'zászló: Saint Lucia',
    '🇱🇮' => 'zászló: Liechtenstein',
    '🇱🇰' => 'zászló: Srí Lanka',
    '🇱🇷' => 'zászló: Libéria',
    '🇱🇸' => 'zászló: Lesotho',
    '🇱🇹' => 'zászló: Litvánia',
    '🇱🇺' => 'zászló: Luxemburg',
    '🇱🇻' => 'zászló: Lettország',
    '🇱🇾' => 'zászló: Líbia',
    '🇲🇦' => 'zászló: Marokkó',
    '🇲🇨' => 'zászló: Monaco',
    '🇲🇩' => 'zászló: Moldova',
    '🇲🇪' => 'zászló: Montenegró',
    '🇲🇫' => 'zászló: Saint Martin',
    '🇲🇬' => 'zászló: Madagaszkár',
    '🇲🇭' => 'zászló: Marshall-szigetek',
    '🇲🇰' => 'zászló: Észak-Macedónia',
    '🇲🇱' => 'zászló: Mali',
    '🇲🇲' => 'zászló: Mianmar',
    '🇲🇳' => 'zászló: Mongólia',
    '🇲🇴' => 'zászló: Makaó KKT',
    '🇲🇵' => 'zászló: Északi Mariana-szigetek',
    '🇲🇶' => 'zászló: Martinique',
    '🇲🇷' => 'zászló: Mauritánia',
    '🇲🇸' => 'zászló: Montserrat',
    '🇲🇹' => 'zászló: Málta',
    '🇲🇺' => 'zászló: Mauritius',
    '🇲🇻' => 'zászló: Maldív-szigetek',
    '🇲🇼' => 'zászló: Malawi',
    '🇲🇽' => 'zászló: Mexikó',
    '🇲🇾' => 'zászló: Malajzia',
    '🇲🇿' => 'zászló: Mozambik',
    '🇳🇦' => 'zászló: Namíbia',
    '🇳🇨' => 'zászló: Új-Kaledónia',
    '🇳🇪' => 'zászló: Niger',
    '🇳🇫' => 'zászló: Norfolk-sziget',
    '🇳🇬' => 'zászló: Nigéria',
    '🇳🇮' => 'zászló: Nicaragua',
    '🇳🇱' => 'zászló: Hollandia',
    '🇳🇴' => 'zászló: Norvégia',
    '🇳🇵' => 'zászló: Nepál',
    '🇳🇷' => 'zászló: Nauru',
    '🇳🇺' => 'zászló: Niue',
    '🇳🇿' => 'zászló: Új-Zéland',
    '🇴🇲' => 'zászló: Omán',
    '🇵🇦' => 'zászló: Panama',
    '🇵🇪' => 'zászló: Peru',
    '🇵🇫' => 'zászló: Francia Polinézia',
    '🇵🇬' => 'zászló: Pápua Új-Guinea',
    '🇵🇭' => 'zászló: Fülöp-szigetek',
    '🇵🇰' => 'zászló: Pakisztán',
    '🇵🇱' => 'zászló: Lengyelország',
    '🇵🇲' => 'zászló: Saint-Pierre és Miquelon',
    '🇵🇳' => 'zászló: Pitcairn-szigetek',
    '🇵🇷' => 'zászló: Puerto Rico',
    '🇵🇸' => 'zászló: Palesztin Autonómia',
    '🇵🇹' => 'zászló: Portugália',
    '🇵🇼' => 'zászló: Palau',
    '🇵🇾' => 'zászló: Paraguay',
    '🇶🇦' => 'zászló: Katar',
    '🇷🇪' => 'zászló: Réunion',
    '🇷🇴' => 'zászló: Románia',
    '🇷🇸' => 'zászló: Szerbia',
    '🇷🇺' => 'zászló: Oroszország',
    '🇷🇼' => 'zászló: Ruanda',
    '🇸🇦' => 'zászló: Szaúd-Arábia',
    '🇸🇧' => 'zászló: Salamon-szigetek',
    '🇸🇨' => 'zászló: Seychelle-szigetek',
    '🇸🇩' => 'zászló: Szudán',
    '🇸🇪' => 'zászló: Svédország',
    '🇸🇬' => 'zászló: Szingapúr',
    '🇸🇭' => 'zászló: Szent Ilona',
    '🇸🇮' => 'zászló: Szlovénia',
    '🇸🇯' => 'zászló: Svalbard és Jan Mayen',
    '🇸🇰' => 'zászló: Szlovákia',
    '🇸🇱' => 'zászló: Sierra Leone',
    '🇸🇲' => 'zászló: San Marino',
    '🇸🇳' => 'zászló: Szenegál',
    '🇸🇴' => 'zászló: Szomália',
    '🇸🇷' => 'zászló: Suriname',
    '🇸🇸' => 'zászló: Dél-Szudán',
    '🇸🇹' => 'zászló: São Tomé és Príncipe',
    '🇸🇻' => 'zászló: Salvador',
    '🇸🇽' => 'zászló: Sint Maarten',
    '🇸🇾' => 'zászló: Szíria',
    '🇸🇿' => 'zászló: Szváziföld',
    '🇹🇦' => 'zászló: Tristan da Cunha',
    '🇹🇨' => 'zászló: Turks- és Caicos-szigetek',
    '🇹🇩' => 'zászló: Csád',
    '🇹🇫' => 'zászló: Francia Déli Területek',
    '🇹🇬' => 'zászló: Togo',
    '🇹🇭' => 'zászló: Thaiföld',
    '🇹🇯' => 'zászló: Tádzsikisztán',
    '🇹🇰' => 'zászló: Tokelau',
    '🇹🇱' => 'zászló: Kelet-Timor',
    '🇹🇲' => 'zászló: Türkmenisztán',
    '🇹🇳' => 'zászló: Tunézia',
    '🇹🇴' => 'zászló: Tonga',
    '🇹🇷' => 'zászló: Törökország',
    '🇹🇹' => 'zászló: Trinidad és Tobago',
    '🇹🇻' => 'zászló: Tuvalu',
    '🇹🇼' => 'zászló: Tajvan',
    '🇹🇿' => 'zászló: Tanzánia',
    '🇺🇦' => 'zászló: Ukrajna',
    '🇺🇬' => 'zászló: Uganda',
    '🇺🇲' => 'zászló: Az USA lakatlan külbirtokai',
    '🇺🇳' => 'zászló: Egyesült Nemzetek Szervezete',
    '🇺🇸' => 'zászló: Egyesült Államok',
    '🇺🇾' => 'zászló: Uruguay',
    '🇺🇿' => 'zászló: Üzbegisztán',
    '🇻🇦' => 'zászló: Vatikán',
    '🇻🇨' => 'zászló: Saint Vincent és a Grenadine-szigetek',
    '🇻🇪' => 'zászló: Venezuela',
    '🇻🇬' => 'zászló: Brit Virgin-szigetek',
    '🇻🇮' => 'zászló: Amerikai Virgin-szigetek',
    '🇻🇳' => 'zászló: Vietnám',
    '🇻🇺' => 'zászló: Vanuatu',
    '🇼🇫' => 'zászló: Wallis és Futuna',
    '🇼🇸' => 'zászló: Szamoa',
    '🇽🇰' => 'zászló: Koszovó',
    '🇾🇪' => 'zászló: Jemen',
    '🇾🇹' => 'zászló: Mayotte',
    '🇿🇦' => 'zászló: Dél-afrikai Köztársaság',
    '🇿🇲' => 'zászló: Zambia',
    '🇿🇼' => 'zászló: Zimbabwe',
    '🔟' => 'gombfej: 10',
    '🏻' => 'világos bőrtónus',
    '🏼' => 'közepesen világos bőrtónus',
    '🏽' => 'közepes bőrtónus',
    '🏾' => 'közepesen sötét bőrtónus',
    '🏿' => 'sötét bőrtónus',
    '😀' => 'nevető arc',
    '😃' => 'nagyra nyílt szemmel mosolygó arc',
    '😄' => 'nevető arc mosolygó szemmel',
    '😁' => 'mosolygó szemmel nevető arc',
    '😆' => 'nyitott szájjal és lehunyt szemmel mosolygó arc',
    '😅' => 'nevető arc verejtékkel',
    '🤣' => 'gurul a nevetéstől',
    '😂' => 'arc örömkönnyekkel',
    '🙂' => 'finoman mosolygó arc',
    '🙃' => 'arc fejjel lefelé',
    '🫠' => 'elolvadó arc',
    '😉' => 'kacsintó arc',
    '😊' => 'mosolygó arc mosolygó szemmel',
    '😇' => 'mosolygó arc glóriával',
    '🥰' => 'mosolygó arc szívekkel',
    '😍' => 'mosolygó arc szív alakú szemmel',
    '🤩' => 'csillagszemű',
    '😘' => 'csókot dobó arc',
    '😗' => 'csókoló arc',
    '☺' => 'mosolygó arc',
    '😚' => 'csókoló arc lehunyt szemmel',
    '😙' => 'csókoló arc mosolygó szemmel',
    '🥲' => 'mosolygó arc könnycseppel',
    '😋' => 'finom ételt élvező arc',
    '😛' => 'arc kinyújtott nyelvvel',
    '😜' => 'arc kinyújtott nyelvvel és kacsintó szemmel',
    '🤪' => 'őrült fej',
    '😝' => 'arc kilógó nyelvvel és erősen behunyt szemmel',
    '🤑' => 'arc dollárjelekkel',
    '🤗' => 'ölelés',
    '🤭' => 'arc szájra tett kézzel',
    '🫢' => 'nyitott szemű, száját takaró arc',
    '🫣' => 'kémlelő szemű arc',
    '🤫' => 'csitító arc',
    '🤔' => 'gondolkodó arc',
    '🫡' => 'tisztelgő arc',
    '🤐' => 'arc összecipzározott szájjal',
    '🤨' => 'felhúzott szemöldökű arc',
    '😐' => 'semleges arc',
    '😑' => 'kifejezéstelen arc',
    '😶' => 'száj nélküli arc',
    '🫥' => 'pontozott vonalas arc',
    '😏' => 'önelégült vigyor',
    '😒' => 'unott arc',
    '🙄' => 'szemét forgató arc',
    '😬' => 'grimaszoló arc',
    '🤥' => 'hazudó arc',
    '🫨' => 'rázkódó arc',
    '😌' => 'megkönnyebbült arc',
    '😔' => 'elgondolkodó arc',
    '😪' => 'álmos arc',
    '🤤' => 'nyáladzó fej',
    '😴' => 'alvó arc',
    '🫩' => 'arc táskás szemekkel',
    '😷' => 'arc egészségügyi maszkkal',
    '🤒' => 'arc lázmérővel',
    '🤕' => 'arc bekötött fejjel',
    '🤢' => 'émelygő arc',
    '🤮' => 'hányó arc',
    '🤧' => 'tüsszentő arc',
    '🥵' => 'forró arc',
    '🥶' => 'fagyos arc',
    '🥴' => 'részeges arc',
    '😵' => 'szédülő arc',
    '🤯' => 'felrobbanó fej',
    '🤠' => 'cowboykalapos fej',
    '🥳' => 'bulizó arc',
    '🥸' => 'álcázott arc',
    '😎' => 'mosolygó arc napszemüveggel',
    '🤓' => 'kockaarc',
    '🧐' => 'arc monoklival',
    '😕' => 'zavart arc',
    '🫤' => 'átlós szájú arc',
    '😟' => 'aggódó arc',
    '🙁' => 'kissé komor arc',
    '☹' => 'rosszalló arc',
    '😮' => 'arc nyitott szájjal',
    '😯' => 'meglepődött arc',
    '😲' => 'lenyűgözött arc',
    '😳' => 'elpirult arc',
    '🫪' => 'eltorzult arc',
    '🥺' => 'esedező arc',
    '🥹' => 'könnyeivel küszködő arc',
    '😦' => 'rosszalló arc nyitott szájjal',
    '😧' => 'elkínzott arc',
    '😨' => 'félelemmel teli arc',
    '😰' => 'aggódó arc nyitott szájjal és verejtékkel',
    '😥' => 'csalódott, de megkönnyebbült arc',
    '😢' => 'síró arc',
    '😭' => 'zokogó arc',
    '😱' => 'félelemtől sikoltó arc',
    '😖' => 'összezavarodott arc',
    '😣' => 'állhatatos arc',
    '😞' => 'csalódott arc',
    '😓' => 'lehangolt arc verejtékkel',
    '😩' => 'kimerült arc',
    '😫' => 'fáradt arc',
    '🥱' => 'ásító arc',
    '😤' => 'orrából gőzt fújtató arc',
    '😡' => 'duzzogó arc',
    '😠' => 'mérges arc',
    '🤬' => 'arc jelekkel a száján',
    '😈' => 'mosolygó arc szarvakkal',
    '👿' => 'mérges arc szarvakkal',
    '💀' => 'koponya',
    '☠' => 'koponya csontokkal',
    '💩' => 'kakihalom',
    '🤡' => 'bohócfej',
    '👹' => 'ogre',
    '👺' => 'kobold',
    '👻' => 'szellem',
    '👽' => 'földönkívüli',
    '👾' => 'földönkívüli szörny',
    '🤖' => 'robot',
    '😺' => 'nevető macska',
    '😸' => 'nevető macska mosolygó szemmel',
    '😹' => 'macska örömkönnyekkel',
    '😻' => 'mosolygó macska szív alakú szemmel',
    '😼' => 'macska fanyar mosollyal',
    '😽' => 'csókoló macska',
    '🙀' => 'kimerült macska',
    '😿' => 'síró macska',
    '😾' => 'duzzogó macska',
    '🙈' => 'nem láttam semmit',
    '🙉' => 'nem hallottam semmit',
    '🙊' => 'nem szólok semmit',
    '💌' => 'szerelmes levél',
    '💘' => 'nyíllal átlőtt szív',
    '💝' => 'szív szalaggal',
    '💖' => 'ragyogó szív',
    '💗' => 'növekvő szív',
    '💓' => 'dobogó szív',
    '💞' => 'forgó szívek',
    '💕' => 'két szív',
    '💟' => 'szív formájú dekoráció',
    '❣' => 'nagy szív alakú felkiáltójel',
    '💔' => 'összetört szív',
    '❤' => 'vörös szív',
    '🩷' => 'rózsaszín szív',
    '🧡' => 'narancssárga szív',
    '💛' => 'sárga szív',
    '💚' => 'zöld szív',
    '💙' => 'kék szív',
    '🩵' => 'világoskék szív',
    '💜' => 'lila szív',
    '🤎' => 'barna szív',
    '🖤' => 'fekete szív',
    '🩶' => 'szürke szív',
    '🤍' => 'fehér szív',
    '💋' => 'csóknyom',
    '💯' => 'száz pont',
    '💢' => 'a düh szimbóluma',
    '🫯' => 'verekedésfelhő',
    '💥' => 'ütközés',
    '💫' => 'szédülés',
    '💦' => 'izzadságcseppek',
    '💨' => 'rohanás',
    '🕳' => 'lyuk',
    '💬' => 'szövegbuborék',
    '🗨' => 'balra nyíló szövegbuborék',
    '🗯' => 'jobbra nyíló dühös szövegbuborék',
    '💭' => 'gondolatbuborék',
    '💤' => 'alvás',
    '👋' => 'integető kéz',
    '🤚' => 'kézfej',
    '🖐' => 'felemelt kéz szétálló ujjakkal',
    '✋' => 'felemelt kéz',
    '🖖' => 'star trek-üdvözlet',
    '🫱' => 'jobbra néző kéz',
    '🫲' => 'balra néző kéz',
    '🫳' => 'lefelé néző kéz',
    '🫴' => 'felfelé néző kéz',
    '🫷' => 'balra toló kéz',
    '🫸' => 'jobbra toló kéz',
    '👌' => 'ok jel',
    '🤌' => 'összecsippentett ujjak',
    '🤏' => 'csippentő kéz',
    '✌' => 'győzelmi jelet mutató kéz',
    '🤞' => 'keresztezett ujjak',
    '🫰' => 'keresztezett mutató- és hüvelykujjú kéz',
    '🤟' => 'szeretlek kézmozdulat',
    '🤘' => 'metálvilla',
    '🤙' => '„hívj fel” kézjel',
    '👈' => 'kéz balra mutató ujjal',
    '👉' => 'kéz jobbra mutató ujjal',
    '👆' => 'tenyér felfelé mutató ujjal',
    '🖕' => 'középső ujj',
    '👇' => 'tenyér lefelé mutató ujjal',
    '☝' => 'kéz felfelé mutató ujjal',
    '🫵' => 'felhasználóra mutató ujj',
    '👍' => 'felfelé mutató hüvelykujj',
    '👎' => 'lefelé mutató hüvelykujj',
    '✊' => 'felemelt ököl',
    '👊' => 'ütésre emelt ököl',
    '🤛' => 'balra néző ököl',
    '🤜' => 'jobbra néző ököl',
    '👏' => 'tapsoló kéz',
    '🙌' => 'kezét felemelő ember',
    '🫶' => 'szívet formázó kezek',
    '👐' => 'széttárt kéz',
    '🤲' => 'felfelé fordított tenyerek',
    '🤝' => 'kézfogás',
    '🙏' => 'összetett kéz',
    '✍' => 'író kéz',
    '💅' => 'körömlakk',
    '🤳' => 'szelfi',
    '💪' => 'befeszített bicepsz',
    '🦾' => 'mechanikus kar',
    '🦿' => 'mechanikus láb',
    '🦵' => 'lábszár',
    '🦶' => 'lábfej',
    '👂' => 'fül',
    '🦻' => 'fül hallókészülékkel',
    '👃' => 'orr',
    '🧠' => 'agy',
    '🫀' => 'szív',
    '🫁' => 'tüdő',
    '🦷' => 'fog',
    '🦴' => 'csont',
    '👀' => 'szemek',
    '👁' => 'szem',
    '👅' => 'nyelv',
    '👄' => 'száj',
    '🫦' => 'ajkába harapó száj',
    '👶' => 'baba',
    '🧒' => 'gyerek',
    '👦' => 'fiú',
    '👧' => 'lány',
    '🧑' => 'felnőtt',
    '👱' => 'szőke hajú ember',
    '👨' => 'férfi',
    '🧔' => 'szakállas ember',
    '👩' => 'nő',
    '🧓' => 'idősebb felnőtt',
    '👴' => 'idős férfi',
    '👵' => 'idős nő',
    '🙍' => 'rosszalló ember',
    '🙎' => 'duzzogó ember',
    '🙅' => 'tiltás mutatása',
    '🙆' => 'beleegyezés mutatása',
    '💁' => 'ember az információs pultnál',
    '🙋' => 'vidáman intő ember',
    '🧏' => 'siket ember',
    '🙇' => 'meghajló ember',
    '🤦' => 'kézbe temetett arc',
    '🤷' => 'vállrándítás',
    '👮' => 'zsaru',
    '🕵' => 'detektív',
    '💂' => 'gárdista',
    '🥷' => 'nindzsa',
    '👷' => 'építőmunkás',
    '🫅' => 'koronás személy',
    '🤴' => 'herceg',
    '👸' => 'hercegnő',
    '👳' => 'turbános ember',
    '👲' => 'kínai sapkás ember',
    '🧕' => 'nő fejkendővel',
    '🤵' => 'szmokingos ember',
    '👰' => 'ember fátyollal',
    '🤰' => 'terhes nő',
    '🫃' => 'terhes férfi',
    '🫄' => 'terhes személy',
    '🤱' => 'szoptatás',
    '👼' => 'babaangyal',
    '🎅' => 'mikulás',
    '🤶' => 'mikulás ruhás anyó',
    '🦸' => 'szuperhős',
    '🦹' => 'főgonosz',
    '🧙' => 'varázsló',
    '🧚' => 'tündér',
    '🧛' => 'vámpír',
    '🧜' => 'sellő',
    '🧝' => 'tünde',
    '🧞' => 'dzsinn',
    '🧟' => 'zombi',
    '🧌' => 'troll',
    '🫈' => 'szőrös lény',
    '💆' => 'arcmasszázs',
    '💇' => 'hajvágás',
    '🚶' => 'gyalogos',
    '🧍' => 'álló ember',
    '🧎' => 'térdelő ember',
    '🏃' => 'futó',
    '💃' => 'táncoló nő',
    '🕺' => 'táncoló férfi',
    '🕴' => 'öltönyös, lebegő ember',
    '👯' => 'bulizó emberek',
    '🧖' => 'ember a gőzfürdőben',
    '🧗' => 'mászó ember',
    '🤺' => 'vívó',
    '🏇' => 'lóverseny',
    '⛷' => 'síelő',
    '🏂' => 'snowboardozó',
    '🏌' => 'golfozó',
    '🏄' => 'szörföző',
    '🚣' => 'evező ember',
    '🏊' => 'úszó',
    '⛹' => 'ember labdával',
    '🏋' => 'súlyemelő',
    '🚴' => 'kerékpáros',
    '🚵' => 'hegyikerékpáros',
    '🤸' => 'cigánykerék',
    '🤼' => 'birkózók',
    '🤽' => 'vízilabda',
    '🤾' => 'kézilabda',
    '🤹' => 'zsonglőr',
    '🧘' => 'ember lótuszülésben',
    '🛀' => 'fürdő ember',
    '🛌' => 'ember az ágyban',
    '👭' => 'kézen fogva álló nők',
    '👫' => 'kézen fogva álló férfi és nő',
    '👬' => 'kézen fogva álló férfiak',
    '💏' => 'csók',
    '💑' => 'pár szívvel',
    '🗣' => 'beszélő fej',
    '👤' => 'ember sziluettje melltől felfelé',
    '👥' => 'két ember sziluettje melltől felfelé',
    '🫂' => 'egymást ölelő emberek',
    '👪' => 'család',
    '👣' => 'lábnyomok',
    '🫆' => 'ujjlenyomat',
    '🦰' => 'vörös haj',
    '🦱' => 'göndör haj',
    '🦳' => 'ősz haj',
    '🦲' => 'kopasz',
    '🐵' => 'majomfej',
    '🐒' => 'majom',
    '🦍' => 'gorilla',
    '🦧' => 'orangután',
    '🐶' => 'kutyafej',
    '🐕' => 'kutya',
    '🦮' => 'vakvezető kutya',
    '🐩' => 'pudli',
    '🐺' => 'farkasfej',
    '🦊' => 'rókafej',
    '🦝' => 'mosómedve',
    '🐱' => 'macskafej',
    '🐈' => 'macska',
    '🦁' => 'oroszlánfej',
    '🐯' => 'tigrisfej',
    '🐅' => 'tigris',
    '🐆' => 'leopárd',
    '🐴' => 'lófej',
    '🫎' => 'jávorszarvas',
    '🫏' => 'szamár',
    '🐎' => 'ló',
    '🦄' => 'unikornisfej',
    '🦓' => 'zebra',
    '🦌' => 'szarvas',
    '🦬' => 'bölény',
    '🐮' => 'tehénfej',
    '🐂' => 'ökör',
    '🐃' => 'vízibölény',
    '🐄' => 'tehén',
    '🐷' => 'malacfej',
    '🐖' => 'disznó',
    '🐗' => 'vaddisznó',
    '🐽' => 'malacorr',
    '🐏' => 'kos',
    '🐑' => 'birka',
    '🐐' => 'kecske',
    '🐪' => 'teve',
    '🐫' => 'kétpúpú teve',
    '🦙' => 'láma',
    '🦒' => 'zsiráf',
    '🐘' => 'elefánt',
    '🦣' => 'mamut',
    '🦏' => 'orrszarvú',
    '🦛' => 'víziló',
    '🐭' => 'egérfej',
    '🐁' => 'egér',
    '🐀' => 'patkány',
    '🐹' => 'hörcsögfej',
    '🐰' => 'nyúlfej',
    '🐇' => 'nyúl',
    '🐿' => 'mókus',
    '🦫' => 'hód',
    '🦔' => 'sün',
    '🦇' => 'denevér',
    '🐻' => 'medvefej',
    '🐨' => 'koala',
    '🐼' => 'pandafej',
    '🦥' => 'lajhár',
    '🦦' => 'vidra',
    '🦨' => 'bűzösborz',
    '🦘' => 'kenguru',
    '🦡' => 'borz',
    '🐾' => 'mancsnyomok',
    '🦃' => 'pulyka',
    '🐔' => 'tyúk',
    '🐓' => 'kakas',
    '🐣' => 'kikelő kiscsibe',
    '🐤' => 'naposcsibe',
    '🐥' => 'előre néző kiscsibe',
    '🐦' => 'madár',
    '🐧' => 'pingvin',
    '🕊' => 'galamb',
    '🦅' => 'sas',
    '🦆' => 'kacsa',
    '🦢' => 'hattyú',
    '🦉' => 'bagoly',
    '🦤' => 'dodó',
    '🪶' => 'madártoll',
    '🦩' => 'flamingó',
    '🦚' => 'páva',
    '🦜' => 'papagáj',
    '🪽' => 'szárny',
    '🪿' => 'liba',
    '🐸' => 'békafej',
    '🐊' => 'krokodil',
    '🐢' => 'teknős',
    '🦎' => 'gyík',
    '🐍' => 'kígyó',
    '🐲' => 'sárkányfej',
    '🐉' => 'sárkány',
    '🦕' => 'sauropoda',
    '🦖' => 'T. rex',
    '🐳' => 'fröcskölő bálna',
    '🐋' => 'bálna',
    '🐬' => 'delfin',
    '🫍' => 'kardszárnyú delfin',
    '🦭' => 'fóka',
    '🐟' => 'hal',
    '🐠' => 'trópusi hal',
    '🐡' => 'gömbhal',
    '🦈' => 'cápa',
    '🐙' => 'polip',
    '🐚' => 'tengeri csiga',
    '🪸' => 'korall',
    '🪼' => 'medúza',
    '🦀' => 'rák',
    '🦞' => 'homár',
    '🦐' => 'garnélarák',
    '🦑' => 'tintahal',
    '🦪' => 'osztriga',
    '🐌' => 'csiga',
    '🦋' => 'pillangó',
    '🐛' => 'rovar',
    '🐜' => 'hangya',
    '🐝' => 'méh',
    '🪲' => 'bogár',
    '🐞' => 'katica',
    '🦗' => 'tücsök',
    '🪳' => 'csótány',
    '🕷' => 'pók',
    '🕸' => 'pókháló',
    '🦂' => 'skorpió',
    '🦟' => 'szúnyog',
    '🪰' => 'légy',
    '🪱' => 'kukac',
    '🦠' => 'mikróba',
    '💐' => 'csokor',
    '🌸' => 'cseresznyevirág',
    '💮' => 'fehér virág',
    '🪷' => 'lótusz',
    '🏵' => 'rozetta',
    '🌹' => 'rózsa',
    '🥀' => 'hervadt virág',
    '🌺' => 'hibiszkusz',
    '🌻' => 'napraforgó',
    '🌼' => 'virágzás',
    '🌷' => 'tulipán',
    '🪻' => 'jácint',
    '🌱' => 'palánta',
    '🪴' => 'cserepes növény',
    '🌲' => 'örökzöld',
    '🌳' => 'lombhullató fa',
    '🌴' => 'pálmafa',
    '🌵' => 'kaktusz',
    '🌾' => 'rizs',
    '🌿' => 'gyógynövény',
    '☘' => 'lóhere',
    '🍀' => 'négylevelű lóhere',
    '🍁' => 'juharlevél',
    '🍂' => 'lehullott levél',
    '🍃' => 'szélfútta levél',
    '🪹' => 'üres fészek',
    '🪺' => 'fészek tojásokkal',
    '🍄' => 'gomba',
    '🪾' => 'levél nélküli fa',
    '🍇' => 'szőlő',
    '🍈' => 'sárgadinnye',
    '🍉' => 'görögdinnye',
    '🍊' => 'mandarin',
    '🍋' => 'citrom',
    '🍌' => 'banán',
    '🍍' => 'ananász',
    '🥭' => 'mangó',
    '🍎' => 'piros alma',
    '🍏' => 'zöld alma',
    '🍐' => 'körte',
    '🍑' => 'őszibarack',
    '🍒' => 'cseresznye',
    '🍓' => 'eper',
    '🫐' => 'áfonya',
    '🥝' => 'kivi',
    '🍅' => 'paradicsom',
    '🫒' => 'olajbogyó',
    '🥥' => 'kókuszdió',
    '🥑' => 'avokádó',
    '🍆' => 'padlizsán',
    '🥔' => 'krumpli',
    '🥕' => 'sárgarépa',
    '🌽' => 'kukoricacső',
    '🌶' => 'erőspaprika',
    '🫑' => 'kaliforniai paprika',
    '🥒' => 'uborka',
    '🥬' => 'leveles zöldség',
    '🥦' => 'brokkoli',
    '🧄' => 'fokhagyma',
    '🧅' => 'hagyma',
    '🥜' => 'földimogyoró',
    '🫘' => 'bab',
    '🌰' => 'gesztenye',
    '🫚' => 'gyömbérgyökér',
    '🫛' => 'borsóhüvely',
    '🫜' => 'gyökérzöldség',
    '🍞' => 'kenyér',
    '🥐' => 'croissant',
    '🥖' => 'bagett',
    '🫓' => 'laposkenyér',
    '🥨' => 'perec',
    '🥯' => 'bagel',
    '🥞' => 'palacsinta',
    '🧇' => 'gofri',
    '🧀' => 'sajtdarab',
    '🍖' => 'csontos hús',
    '🍗' => 'csirkecomb',
    '🥩' => 'húsdarab',
    '🥓' => 'szalonna',
    '🍔' => 'hamburger',
    '🍟' => 'sült krumpli',
    '🍕' => 'pizza',
    '🌭' => 'hot dog',
    '🥪' => 'szendvics',
    '🌮' => 'taco',
    '🌯' => 'burrito',
    '🫔' => 'tamale',
    '🥙' => 'pita',
    '🧆' => 'falafel',
    '🥚' => 'tojás',
    '🍳' => 'sütés-főzés',
    '🥘' => 'serpenyős étel',
    '🍲' => 'egy tál étel',
    '🫕' => 'fondü',
    '🥣' => 'tál kanállal',
    '🥗' => 'zöld saláta',
    '🍿' => 'pattogatott kukorica',
    '🧈' => 'vaj',
    '🧂' => 'só',
    '🥫' => 'konzerv',
    '🍱' => 'japán ételdoboz',
    '🍘' => 'rizskeksz',
    '🍙' => 'rizsgolyó',
    '🍚' => 'főtt rizs',
    '🍛' => 'currys rizs',
    '🍜' => 'gőzölgő tál',
    '🍝' => 'spagetti',
    '🍠' => 'sült édesburgonya',
    '🍢' => 'oden',
    '🍣' => 'szusi',
    '🍤' => 'sült garnélarák',
    '🍥' => 'csavart haltorta',
    '🥮' => 'holdsütemény',
    '🍡' => 'dango',
    '🥟' => 'gombóc',
    '🥠' => 'szerencsesüti',
    '🥡' => 'elviteles doboz',
    '🍦' => 'lágyfagylalt',
    '🍧' => 'fagylalt öntettel',
    '🍨' => 'fagylalt',
    '🍩' => 'fánk',
    '🍪' => 'sütemény',
    '🎂' => 'születésnapi torta',
    '🍰' => 'gyümölcstorta',
    '🧁' => 'cupcake',
    '🥧' => 'pite',
    '🍫' => 'csokoládé',
    '🍬' => 'cukorka',
    '🍭' => 'nyalóka',
    '🍮' => 'puding',
    '🍯' => 'mézes csupor',
    '🍼' => 'cumisüveg',
    '🥛' => 'egy pohár tej',
    '☕' => 'forró ital',
    '🫖' => 'teáskanna',
    '🍵' => 'fül nélküli teáscsésze',
    '🍶' => 'szaké',
    '🍾' => 'üveg kiugró dugóval',
    '🍷' => 'borospohár',
    '🍸' => 'koktélos pohár',
    '🍹' => 'trópusi ital',
    '🍺' => 'söröskorsó',
    '🍻' => 'összekoccanó söröskorsók',
    '🥂' => 'koccintó poharak',
    '🥃' => 'feles pohár',
    '🫗' => 'ömlő folyadék',
    '🥤' => 'pohár szívószállal',
    '🧋' => 'buborékos tea',
    '🧃' => 'italos doboz',
    '🧉' => 'maté',
    '🧊' => 'jégkocka',
    '🥢' => 'evőpálcikák',
    '🍽' => 'kés és villa tányérral',
    '🍴' => 'kés és villa',
    '🥄' => 'kanál',
    '🔪' => 'konyhakés',
    '🫙' => 'befőttesüveg',
    '🏺' => 'amfora',
    '🌍' => 'Európát és Afrikát mutató földgömb',
    '🌎' => 'Amerikát mutató földgömb',
    '🌏' => 'Ázsiát és Ausztráliát mutató földgömb',
    '🌐' => 'földgömb délkörökkel',
    '🗺' => 'világtérkép',
    '🗾' => 'Japán térképe',
    '🧭' => 'iránytű',
    '🏔' => 'hegy hótakaróval',
    '⛰' => 'hegy',
    '🛘' => 'földcsuszamlás',
    '🌋' => 'vulkán',
    '🗻' => 'Fudzsi-hegy',
    '🏕' => 'sátorozás',
    '🏖' => 'strand napernyővel',
    '🏜' => 'sivatag',
    '🏝' => 'lakatlan sziget',
    '🏞' => 'nemzeti park',
    '🏟' => 'stadion',
    '🏛' => 'klasszikus épület',
    '🏗' => 'építkezés',
    '🧱' => 'tégla',
    '🪨' => 'szikla',
    '🪵' => 'fa',
    '🛖' => 'kunyhó',
    '🏘' => 'lakóházak',
    '🏚' => 'elhagyatott ház',
    '🏠' => 'lakóház',
    '🏡' => 'ház kerttel',
    '🏢' => 'irodaépület',
    '🏣' => 'japán postahivatal',
    '🏤' => 'postahivatal',
    '🏥' => 'kórház',
    '🏦' => 'bank',
    '🏨' => 'hotel',
    '🏩' => 'szálló szerelmeseknek',
    '🏪' => 'kisbolt',
    '🏫' => 'iskola',
    '🏬' => 'áruház',
    '🏭' => 'gyár',
    '🏯' => 'japán kastély',
    '🏰' => 'vár',
    '💒' => 'esküvő',
    '🗼' => 'Tokiói torony',
    '🗽' => 'Szabadság-szobor',
    '⛪' => 'templom',
    '🕌' => 'mecset',
    '🛕' => 'hindu templom',
    '🕍' => 'zsinagóga',
    '⛩' => 'sintó szentély',
    '🕋' => 'kába',
    '⛲' => 'szökőkút',
    '⛺' => 'sátor',
    '🌁' => 'ködös',
    '🌃' => 'éjszaka csillagokkal',
    '🏙' => 'városkép',
    '🌄' => 'hegyek mögött felkelő nap',
    '🌅' => 'napfelkelte',
    '🌆' => 'városkép alkonyatkor',
    '🌇' => 'naplemente',
    '🌉' => 'híd éjszaka',
    '♨' => 'hőforrás',
    '🎠' => 'ló a körhintáról',
    '🛝' => 'játszótéri csúszda',
    '🎡' => 'óriáskerék',
    '🎢' => 'hullámvasút',
    '💈' => 'fodrászcégér',
    '🎪' => 'cirkuszsátor',
    '🚂' => 'mozdony',
    '🚃' => 'vasúti kocsi',
    '🚄' => 'gyorsvasút',
    '🚅' => 'golyóorrú gyorsvasút',
    '🚆' => 'vonat',
    '🚇' => 'metró',
    '🚈' => 'hév',
    '🚉' => 'állomás',
    '🚊' => 'villamos',
    '🚝' => 'egysínű vasút',
    '🚞' => 'hegyi vasút',
    '🚋' => 'villamoskocsi',
    '🚌' => 'busz',
    '🚍' => 'érkező busz',
    '🚎' => 'trolibusz',
    '🚐' => 'mikrobusz',
    '🚑' => 'mentő',
    '🚒' => 'tűzoltóautó',
    '🚓' => 'rendőrautó',
    '🚔' => 'érkező rendőrautó',
    '🚕' => 'taxi',
    '🚖' => 'érkező taxi',
    '🚗' => 'autó',
    '🚘' => 'érkező autó',
    '🚙' => 'szabadidőautó',
    '🛻' => 'kisteherautó',
    '🚚' => 'teherautó',
    '🚛' => 'billenős teherautó',
    '🚜' => 'traktor',
    '🏎' => 'versenyautó',
    '🏍' => 'motorkerékpár',
    '🛵' => 'robogó',
    '🦽' => 'kézi hajtású kerekesszék',
    '🦼' => 'motoros kerekesszék',
    '🛺' => 'motoros riksa',
    '🚲' => 'kerékpár',
    '🛴' => 'roller',
    '🛹' => 'gördeszka',
    '🛼' => 'görkorcsolya',
    '🚏' => 'buszmegálló',
    '🛣' => 'autópálya',
    '🛤' => 'vasúti sín',
    '🛢' => 'olajoshordó',
    '⛽' => 'benzinkút',
    '🛞' => 'kerék',
    '🚨' => 'rendőrautó villogója',
    '🚥' => 'vízszintes közlekedési lámpa',
    '🚦' => 'függőleges közlekedési lámpa',
    '🛑' => 'stoptábla',
    '🚧' => 'útépítés',
    '⚓' => 'vasmacska',
    '🛟' => 'mentőöv',
    '⛵' => 'vitorlás hajó',
    '🛶' => 'kenu',
    '🚤' => 'motoros siklócsónak',
    '🛳' => 'utasszállító hajó',
    '⛴' => 'komp',
    '🛥' => 'motorcsónak',
    '🚢' => 'hajó',
    '✈' => 'repülőgép',
    '🛩' => 'kisrepülő',
    '🛫' => 'repülőgép indulása',
    '🛬' => 'repülőgép érkezése',
    '🪂' => 'ejtőernyő',
    '💺' => 'ülés',
    '🚁' => 'helikopter',
    '🚟' => 'függővasút',
    '🚠' => 'hegyi felvonó',
    '🚡' => 'kötélpályás felvonó',
    '🛰' => 'műhold',
    '🚀' => 'rakéta',
    '🛸' => 'repülő csészealj',
    '🛎' => 'londinerhívó csengő',
    '🧳' => 'bőrönd',
    '⌛' => 'lejárt homokóra',
    '⏳' => 'nem lejárt homokóra',
    '⌚' => 'óra',
    '⏰' => 'ébresztőóra',
    '⏱' => 'stopperóra',
    '⏲' => 'visszaszámláló óra',
    '🕰' => 'régies óra',
    '🕛' => 'tizenkét óra',
    '🕧' => 'fél egy',
    '🕐' => 'egy óra',
    '🕜' => 'fél kettő',
    '🕑' => 'két óra',
    '🕝' => 'fél három',
    '🕒' => 'három óra',
    '🕞' => 'fél négy',
    '🕓' => 'négy óra',
    '🕟' => 'fél öt',
    '🕔' => 'öt óra',
    '🕠' => 'fél hat',
    '🕕' => 'hat óra',
    '🕡' => 'fél hét',
    '🕖' => 'hét óra',
    '🕢' => 'fél nyolc',
    '🕗' => 'nyolc óra',
    '🕣' => 'fél kilenc',
    '🕘' => 'kilenc óra',
    '🕤' => 'fél tíz',
    '🕙' => 'tíz óra',
    '🕥' => 'fél tizenegy',
    '🕚' => 'tizenegy óra',
    '🕦' => 'fél tizenkettő',
    '🌑' => 'újhold',
    '🌒' => 'növő, sarló alakú hold',
    '🌓' => 'első negyedben lévő hold',
    '🌔' => 'növő domború hold',
    '🌕' => 'telihold',
    '🌖' => 'fogyó domború hold',
    '🌗' => 'utolsó negyedben lévő hold',
    '🌘' => 'fogyó, sarló alakú hold',
    '🌙' => 'növekvő hold',
    '🌚' => 'újhold arccal',
    '🌛' => 'első negyedben lévő hold arccal',
    '🌜' => 'utolsó negyedben lévő hold arccal',
    '🌡' => 'hőmérő',
    '☀' => 'nap',
    '🌝' => 'telihold arccal',
    '🌞' => 'nap arccal',
    '🪐' => 'gyűrűs bolygó',
    '⭐' => 'csillag',
    '🌟' => 'fénylő csillag',
    '🌠' => 'hullócsillag',
    '🌌' => 'tejút',
    '☁' => 'felhő',
    '⛅' => 'nap egy felhő mögött',
    '⛈' => 'felhő villámmal és esővel',
    '🌤' => 'nap egy kis felhő mögött',
    '🌥' => 'nap egy nagy felhő mögött',
    '🌦' => 'nap a felhő mögött esővel',
    '🌧' => 'felhő esővel',
    '🌨' => 'felhő hóval',
    '🌩' => 'felhő villámmal',
    '🌪' => 'tornádó',
    '🌫' => 'köd',
    '🌬' => 'szélarc',
    '🌀' => 'ciklon',
    '🌈' => 'szivárvány',
    '🌂' => 'becsukott esernyő',
    '☂' => 'esernyő',
    '☔' => 'esernyő esőcseppekkel',
    '⛱' => 'ernyő a földön',
    '⚡' => 'magasfeszültség',
    '❄' => 'hópehely',
    '☃' => 'hóember',
    '⛄' => 'hóember hó nélkül',
    '☄' => 'üstökös',
    '🔥' => 'tűz',
    '💧' => 'csepp',
    '🌊' => 'vízhullám',
    '🎃' => 'töklámpás',
    '🎄' => 'karácsonyfa',
    '🎆' => 'tűzijáték',
    '🎇' => 'csillagszóró',
    '🧨' => 'petárda',
    '✨' => 'szikrák',
    '🎈' => 'léggömb',
    '🎉' => 'partikellékek',
    '🎊' => 'konfetti',
    '🎋' => 'tanabata ünnepi fa',
    '🎍' => 'fenyődekoráció',
    '🎎' => 'japán babák',
    '🎏' => 'rizspapír pontyok',
    '🎐' => 'szélharang',
    '🎑' => 'holdünnep',
    '🧧' => 'piros boríték',
    '🎀' => 'szalag',
    '🎁' => 'becsomagolt ajándék',
    '🎗' => 'emlékező szalag',
    '🎟' => 'belépőjegyek',
    '🎫' => 'jegy',
    '🎖' => 'katonai kitüntetés',
    '🏆' => 'kupa',
    '🏅' => 'sportérem',
    '🥇' => 'aranyérem',
    '🥈' => 'ezüstérem',
    '🥉' => 'bronzérem',
    '⚽' => 'foci',
    '⚾' => 'baseball',
    '🥎' => 'softball',
    '🏀' => 'kosárlabda',
    '🏐' => 'röplabda',
    '🏈' => 'amerikai foci',
    '🏉' => 'rögbi',
    '🎾' => 'tenisz',
    '🥏' => 'frizbi',
    '🎳' => 'bowling',
    '🏏' => 'krikett',
    '🏑' => 'gyeplabda',
    '🏒' => 'hokiütő és korong',
    '🥍' => 'lacrosse',
    '🏓' => 'pingpong',
    '🏸' => 'tollaslabda',
    '🥊' => 'bokszkesztyű',
    '🥋' => 'karateruha',
    '🥅' => 'kapu',
    '⛳' => 'zászló a lyuknál',
    '⛸' => 'jégkorcsolya',
    '🎣' => 'horgászbot',
    '🤿' => 'búvármaszk',
    '🎽' => 'futómez',
    '🎿' => 'sílécek',
    '🛷' => 'szán',
    '🥌' => 'curlingkő',
    '🎯' => 'telitalálat',
    '🪀' => 'jojó',
    '🪁' => 'papírsárkány',
    '🔫' => 'vízipisztoly',
    '🎱' => 'biliárd',
    '🔮' => 'kristálygömb',
    '🪄' => 'varázspálca',
    '🎮' => 'videojáték',
    '🕹' => 'joystick',
    '🎰' => 'nyerőgép',
    '🎲' => 'dobókocka',
    '🧩' => 'kirakó',
    '🧸' => 'plüssmaci',
    '🪅' => 'pinyáta',
    '🪩' => 'tükörgömb',
    '🪆' => 'matrjoska',
    '♠' => 'pikk szín',
    '♥' => 'kőr szín',
    '♦' => 'káró szín',
    '♣' => 'treff szín',
    '♟' => 'gyalog',
    '🃏' => 'dzsóker',
    '🀄' => 'madzsong vörös sárkány',
    '🎴' => 'virágos kártyajáték',
    '🎭' => 'előadóművészetek',
    '🖼' => 'kép kerettel',
    '🎨' => 'festőpaletta',
    '🧵' => 'cérna',
    '🪡' => 'varrótű',
    '🧶' => 'fonál',
    '🪢' => 'csomó',
    '👓' => 'szemüveg',
    '🕶' => 'napszemüveg',
    '🥽' => 'védőszemüveg',
    '🥼' => 'laborköpeny',
    '🦺' => 'mentőmellény',
    '👔' => 'nyakkendő',
    '👕' => 'póló',
    '👖' => 'farmer',
    '🧣' => 'sál',
    '🧤' => 'kesztyű',
    '🧥' => 'kabát',
    '🧦' => 'zokni',
    '👗' => 'ruha',
    '👘' => 'kimonó',
    '🥻' => 'szári',
    '🩱' => 'egyrészes fürdőruha',
    '🩲' => 'fürdőnadrág',
    '🩳' => 'rövidnadrág',
    '👙' => 'bikini',
    '👚' => 'női ruha',
    '🪭' => 'összecsukható kézi legyező',
    '👛' => 'pénztárca',
    '👜' => 'női táska',
    '👝' => 'táska',
    '🛍' => 'bevásárlótáskák',
    '🎒' => 'iskolatáska',
    '🩴' => 'tangapapucs',
    '👞' => 'férfi cipő',
    '👟' => 'futócipő',
    '🥾' => 'túrabakancs',
    '🥿' => 'lapos sarkú cipő',
    '👠' => 'magas sarkú cipő',
    '👡' => 'női szandál',
    '🩰' => 'balettcipő',
    '👢' => 'női csizma',
    '🪮' => 'hosszú fogú fésű',
    '👑' => 'korona',
    '👒' => 'női kalap',
    '🎩' => 'cilinder',
    '🎓' => 'diplomaosztón használt kalap',
    '🧢' => 'sildes sapka',
    '🪖' => 'katonai sisak',
    '⛑' => 'sisak vörös kereszttel',
    '📿' => 'imafüzér',
    '💄' => 'ajakrúzs',
    '💍' => 'gyűrű',
    '💎' => 'ékkő',
    '🔇' => 'elnémított hangszóró',
    '🔈' => 'hangszóró',
    '🔉' => 'bekapcsolt hangszóró',
    '🔊' => 'hangos hangszóró',
    '📢' => 'hangosbeszélő',
    '📣' => 'megafon',
    '📯' => 'postakürt',
    '🔔' => 'harang',
    '🔕' => 'áthúzott harang',
    '🎼' => 'zenei kotta',
    '🎵' => 'hangjegy',
    '🎶' => 'hangjegyek',
    '🎙' => 'stúdiómikrofon',
    '🎚' => 'csúszka',
    '🎛' => 'vezérlőgombok',
    '🎤' => 'mikrofon',
    '🎧' => 'fülhallgató',
    '📻' => 'rádió',
    '🎷' => 'szaxofon',
    '🎺' => 'trombita',
    '🪊' => 'harsona',
    '🪗' => 'harmonika',
    '🎸' => 'gitár',
    '🎹' => 'zenei billentyűzet',
    '🎻' => 'hegedű',
    '🪕' => 'bendzsó',
    '🥁' => 'dob',
    '🪘' => 'hosszú dob',
    '🪇' => 'rumbatök',
    '🪈' => 'fuvola',
    '🪉' => 'hárfa',
    '📱' => 'mobiltelefon',
    '📲' => 'mobiltelefon nyíllal',
    '☎' => 'telefon',
    '📞' => 'telefonkagyló',
    '📟' => 'személyhívó',
    '📠' => 'faxkészülék',
    '🔋' => 'elem',
    '🪫' => 'alacsony töltöttség',
    '🔌' => 'elektromos dugó',
    '💻' => 'laptop',
    '🖥' => 'asztali számítógép',
    '🖨' => 'nyomtató',
    '⌨' => 'billentyűzet',
    '🖱' => 'számítógépes egér',
    '🖲' => 'trackball',
    '💽' => 'minidisk',
    '💾' => 'floppylemez',
    '💿' => 'optikai lemez',
    '📀' => 'dvd',
    '🧮' => 'abakusz',
    '🎥' => 'kamera',
    '🎞' => 'filmkockák',
    '📽' => 'filmvetítő',
    '🎬' => 'csapó',
    '📺' => 'televízió',
    '📷' => 'fényképezőgép',
    '📸' => 'fényképező vakuval',
    '📹' => 'videokamera',
    '📼' => 'videokazetta',
    '🔍' => 'balra mutató nagyító',
    '🔎' => 'jobbra mutató nagyító',
    '🕯' => 'gyertya',
    '💡' => 'villanykörte',
    '🔦' => 'zseblámpa',
    '🏮' => 'piros papírlámpa',
    '🪔' => 'olajlámpa',
    '📔' => 'jegyzetfüzet díszkötésben',
    '📕' => 'becsukott könyv',
    '📖' => 'nyitott könyv',
    '📗' => 'zöld könyv',
    '📘' => 'kék könyv',
    '📙' => 'narancssárga könyv',
    '📚' => 'könyvek',
    '📓' => 'jegyzetfüzet',
    '📒' => 'főkönyv',
    '📃' => 'kunkorodó lap',
    '📜' => 'tekercs',
    '📄' => 'felfelé néző oldal',
    '📰' => 'újság',
    '🗞' => 'feltekert újság',
    '📑' => 'könyvjelzők',
    '🔖' => 'könyvjelző',
    '🏷' => 'címke',
    '🪙' => 'érme',
    '💰' => 'pénzeszsák',
    '🪎' => 'kincsesláda',
    '💴' => 'jen bankjegy',
    '💵' => 'dollár bankjegy',
    '💶' => 'euró bankjegy',
    '💷' => 'font bankjegy',
    '💸' => 'pénz szárnyakkal',
    '💳' => 'hitelkártya',
    '🧾' => 'nyugta',
    '💹' => 'növekvő jendiagram',
    '✉' => 'boríték',
    '📧' => 'e-mail',
    '📨' => 'beérkező boríték',
    '📩' => 'boríték nyíllal',
    '📤' => 'kimenő tálca',
    '📥' => 'beérkező tálca',
    '📦' => 'csomag',
    '📫' => 'csukott levelesláda felemelt zászlóval',
    '📪' => 'csukott levelesláda leengedett zászlóval',
    '📬' => 'nyitott levelesláda felemelt zászlóval',
    '📭' => 'nyitott levelesláda leengedett zászlóval',
    '📮' => 'postaláda',
    '🗳' => 'urna szavazólappal',
    '✏' => 'ceruza',
    '✒' => 'fekete tollhegy',
    '🖋' => 'töltőtoll',
    '🖊' => 'toll',
    '🖌' => 'ecset',
    '🖍' => 'zsírkréta',
    '📝' => 'feljegyzés',
    '💼' => 'aktatáska',
    '📁' => 'fájlmappa',
    '📂' => 'nyitott fájlmappa',
    '🗂' => 'kartotékelválasztók',
    '📅' => 'naptár',
    '📆' => 'fali naptár',
    '🗒' => 'spirálfüzet',
    '🗓' => 'spirálos naptár',
    '📇' => 'forgó névjegykártyatartó',
    '📈' => 'növekvő diagram',
    '📉' => 'csökkenő diagram',
    '📊' => 'oszlopdiagram',
    '📋' => 'írótábla',
    '📌' => 'rajzszög',
    '📍' => 'kerek rajzszög',
    '📎' => 'gemkapocs',
    '🖇' => 'összekapcsolt gemkapcsok',
    '📏' => 'egyenes vonalzó',
    '📐' => 'háromszögvonalzó',
    '✂' => 'olló',
    '🗃' => 'kartotékdoboz',
    '🗄' => 'iratszekrény',
    '🗑' => 'papírkosár',
    '🔒' => 'lakat',
    '🔓' => 'nyitott lakat',
    '🔏' => 'lakat tollal',
    '🔐' => 'zárt lakat kulccsal',
    '🔑' => 'kulcs',
    '🗝' => 'régi kulcs',
    '🔨' => 'kalapács',
    '🪓' => 'balta',
    '⛏' => 'csákány',
    '⚒' => 'kalapács és csákány',
    '🛠' => 'kalapács és csavarkulcs',
    '🗡' => 'tőr',
    '⚔' => 'keresztezett kardok',
    '💣' => 'bomba',
    '🪃' => 'bumeráng',
    '🏹' => 'íj és nyíl',
    '🛡' => 'pajzs',
    '🪚' => 'rókafarkú fűrész',
    '🔧' => 'csavarkulcs',
    '🪛' => 'csavarhúzó',
    '🔩' => 'csavar és anya',
    '⚙' => 'fogaskerék',
    '🗜' => 'satu',
    '⚖' => 'kétkarú mérleg',
    '🦯' => 'fehér bot',
    '🔗' => 'kapocs',
    '⛓' => 'lánc',
    '🪝' => 'kampó',
    '🧰' => 'szerszámosláda',
    '🧲' => 'mágnes',
    '🪜' => 'létra',
    '🪏' => 'ásó',
    '⚗' => 'lombik',
    '🧪' => 'kémcső',
    '🧫' => 'Petri-csésze',
    '🧬' => 'DNS',
    '🔬' => 'mikroszkóp',
    '🔭' => 'távcső',
    '📡' => 'parabolaantenna',
    '💉' => 'fecskendő',
    '🩸' => 'vércsepp',
    '💊' => 'tabletta',
    '🩹' => 'ragtapasz',
    '🩼' => 'mankó',
    '🩺' => 'sztetoszkóp',
    '🩻' => 'röntgen',
    '🚪' => 'ajtó',
    '🛗' => 'lift',
    '🪞' => 'tükör',
    '🪟' => 'ablak',
    '🛏' => 'ágy',
    '🛋' => 'kanapé és lámpa',
    '🪑' => 'szék',
    '🚽' => 'vécé',
    '🪠' => 'vécépumpa',
    '🚿' => 'zuhanyzó',
    '🛁' => 'fürdőkád',
    '🪤' => 'egércsapda',
    '🪒' => 'borotva',
    '🧴' => 'krémes flakon',
    '🧷' => 'biztosítótű',
    '🧹' => 'seprű',
    '🧺' => 'kosár',
    '🧻' => 'papírtekercs',
    '🪣' => 'vödör',
    '🧼' => 'szappan',
    '🫧' => 'buborékok',
    '🪥' => 'fogkefe',
    '🧽' => 'szivacs',
    '🧯' => 'tűzoltó készülék',
    '🛒' => 'bevásárlókocsi',
    '🚬' => 'dohányzás',
    '⚰' => 'koporsó',
    '🪦' => 'sírkő',
    '⚱' => 'halotti urna',
    '🧿' => 'nazar amulett',
    '🪬' => 'hamsza',
    '🗿' => 'moai',
    '🪧' => 'transzparens',
    '🪪' => 'személyi igazolvány',
    '🏧' => 'ATM-jel',
    '🚮' => 'szemét a kukában szimbólum',
    '🚰' => 'ivóvíz',
    '♿' => 'kerekesszék',
    '🚹' => 'férfimosdó',
    '🚺' => 'női mosdó',
    '🚻' => 'mosdó',
    '🚼' => 'baba szimbólum',
    '🚾' => 'toalett',
    '🛂' => 'útlevél-ellenőrzés',
    '🛃' => 'vám',
    '🛄' => 'poggyászfelvétel',
    '🛅' => 'csomagmegőrző',
    '⚠' => 'figyelmeztetés',
    '🚸' => 'átkelő gyermekek',
    '⛔' => 'behajtani tilos',
    '🚫' => 'tilos',
    '🚳' => 'kerékpározni tilos',
    '🚭' => 'dohányozni tilos',
    '🚯' => 'szemetelni tilos',
    '🚱' => 'nem ivóvíz',
    '🚷' => 'nem gyalogos zóna',
    '📵' => 'mobiltelefon használata tilos',
    '🔞' => '18 év alatt tilos',
    '☢' => 'radioaktív',
    '☣' => 'biológiai veszély',
    '⬆' => 'felfelé mutató nyíl',
    '↗' => 'jobbra felfelé mutató nyíl',
    '➡' => 'jobbra mutató nyíl',
    '↘' => 'jobbra lefelé mutató nyíl',
    '⬇' => 'lefelé mutató nyíl',
    '↙' => 'balra lefelé mutató nyíl',
    '⬅' => 'balra mutató nyíl',
    '↖' => 'balra felfelé mutató nyíl',
    '↕' => 'fel-le mutató nyíl',
    '↔' => 'balra-jobbra mutató nyíl',
    '↩' => 'balra görbülő jobb nyíl',
    '↪' => 'jobbra görbülő bal nyíl',
    '⤴' => 'felfelé görbülő jobb nyíl',
    '⤵' => 'lefelé görbülő jobb nyíl',
    '🔃' => 'óramutató járásával megegyező irányú függőleges nyilak',
    '🔄' => 'óramutató járásával ellentétes irányú nyilak gombja',
    '🔙' => 'vissza nyíl',
    '🔚' => 'vége nyíl',
    '🔛' => 'bekapcsolva nyíl',
    '🔜' => 'hamarosan nyíl',
    '🔝' => 'csúcsra nyíl',
    '🛐' => 'imahely',
    '⚛' => 'atom szimbólum',
    '🕉' => 'om',
    '✡' => 'Dávid-csillag',
    '☸' => 'dharma-kerék',
    '☯' => 'jin-jang',
    '✝' => 'latin kereszt',
    '☦' => 'ortodox kereszt',
    '☪' => 'csillag és félhold',
    '☮' => 'békejel',
    '🕎' => 'menóra',
    '🔯' => 'hatágú csillag ponttal',
    '🪯' => 'khanda',
    '♈' => 'kos csillagjegy',
    '♉' => 'bika csillagjegy',
    '♊' => 'ikrek csillagjegy',
    '♋' => 'rák csillagjegy',
    '♌' => 'oroszlán csillagjegy',
    '♍' => 'szűz csillagjegy',
    '♎' => 'mérleg csillagjegy',
    '♏' => 'skorpió csillagjegy',
    '♐' => 'nyilas csillagjegy',
    '♑' => 'bak csillagjegy',
    '♒' => 'vízöntő csillagjegy',
    '♓' => 'halak csillagjegy',
    '⛎' => 'kígyótartó',
    '🔀' => 'véletlenszerű lejátszás gomb',
    '🔁' => 'ismétlés gomb',
    '🔂' => 'egy szám ismétlése gomb',
    '▶' => 'lejátszás gomb',
    '⏩' => 'előretekerés gomb',
    '⏭' => 'következő szám gomb',
    '⏯' => 'lejátszás vagy szünet gomb',
    '◀' => 'visszajátszás gomb',
    '⏪' => 'visszatekerés gomb',
    '⏮' => 'legutóbbi szám gomb',
    '🔼' => 'felfelé gomb',
    '⏫' => 'gyors felfelé gomb',
    '🔽' => 'lefelé gomb',
    '⏬' => 'gyors lefelé gomb',
    '⏸' => 'szüneteltetés gomb',
    '⏹' => 'leállítás gomb',
    '⏺' => 'felvétel gomb',
    '⏏' => 'lemez kiadása gomb',
    '🎦' => 'mozi',
    '🔅' => 'sötétítés gomb',
    '🔆' => 'világosítás gomb',
    '📶' => 'antennasávok',
    '🛜' => 'vezeték nélküli',
    '📳' => 'rezgő mód',
    '📴' => 'mobiltelefon kikapcsolva',
    '♀' => 'női nem jelképe',
    '♂' => 'férfi nem jelképe',
    '⚧' => 'transznemű szimbólum',
    '✖' => 'szorzás',
    '➕' => 'plusz',
    '➖' => 'mínusz',
    '➗' => 'osztás',
    '🟰' => 'vastag egyenlőségjel',
    '♾' => 'végtelen',
    '‼' => 'dupla felkiáltójel',
    '⁉' => 'felkiáltójel kérdőjellel',
    '❓' => 'piros kérdőjel',
    '❔' => 'fehér kérdőjel',
    '❕' => 'fehér felkiáltójel',
    '❗' => 'piros felkiáltójel',
    '〰' => 'hullámos gondolatjel',
    '💱' => 'pénzváltás',
    '💲' => 'vastag dollárjel',
    '⚕' => 'orvosi szimbólum',
    '♻' => 'újrahasznosítás szimbólum',
    '⚜' => 'Anjou-liliom',
    '🔱' => 'háromágú szigony',
    '📛' => 'kitűző',
    '🔰' => 'a kezdő japán szimbóluma',
    '⭕' => 'nagy, vastag kör',
    '✅' => 'vastag fehér pipa',
    '☑' => 'jelölőnégyzet pipával',
    '✔' => 'vastag pipa',
    '❌' => 'áthúzásjel',
    '❎' => 'áthúzásjel gomb',
    '➰' => 'hurok',
    '➿' => 'dupla hurok',
    '〽' => 'részelválasztó',
    '✳' => 'nyolcágú csillag',
    '✴' => 'nyolcas csillag',
    '❇' => 'szikra',
    '™' => 'védjegy',
    '🫟' => 'fröccsenés',
    '🔠' => 'latin nagybetűs bevitel',
    '🔡' => 'latin kisbetűs bevitel',
    '🔢' => 'számbevitel',
    '🔣' => 'szimbólumok bevitele',
    '🔤' => 'latin betűk bevitele',
    '🅰' => 'A gomb (vércsoport)',
    '🆎' => 'AB gomb (vércsoport)',
    '🅱' => 'B gomb (vércsoport)',
    '🆑' => 'CL gomb',
    '🆒' => 'COOL gomb',
    '🆓' => 'FREE gomb',
    'ℹ' => 'információforrás',
    '🆔' => 'ID gomb',
    'Ⓜ' => 'bekarikázott M betű',
    '🆕' => 'NEW gomb',
    '🆖' => 'NG gomb',
    '🅾' => '0 gomb (vércsoport)',
    '🆗' => 'OK gomb',
    '🅿' => 'P gomb',
    '🆘' => 'SOS gomb',
    '🆙' => 'UP! gomb',
    '🆚' => 'VS gomb',
    '🈁' => 'japán „itt” gomb',
    '🈂' => 'japán „szervizdíj” gomb',
    '🈷' => 'japán „havi összeg” gomb',
    '🈶' => 'japán „nem ingyenes” gomb',
    '🈯' => 'japán „foglalt” gomb',
    '🉐' => 'japán „alkalmi vétel” gomb',
    '🈹' => 'japán „engedmény” gomb',
    '🈚' => 'japán „ingyenes” gomb',
    '🈲' => 'japán „tiltott” gomb',
    '🉑' => 'japán „elfogadás” gomb',
    '🈸' => 'japán „alkalmazás” gomb',
    '🈴' => 'japán „minimális pontszám” gomb',
    '🈳' => 'japán „megüresedés” gomb',
    '㊗' => 'japán „gratuláció” gomb',
    '㊙' => 'japán „titok” gomb',
    '🈺' => 'japán „működik” gomb',
    '🈵' => 'japán „megtelt” gomb',
    '🔴' => 'piros kör',
    '🟠' => 'narancssárga kör',
    '🟡' => 'sárga kör',
    '🟢' => 'zöld kör',
    '🔵' => 'kék kör',
    '🟣' => 'lila kör',
    '🟤' => 'barna kör',
    '⚫' => 'fekete kör',
    '⚪' => 'fehér kör',
    '🟥' => 'vörös négyzet',
    '🟧' => 'narancssárga négyzet',
    '🟨' => 'sárga négyzet',
    '🟩' => 'zöld négyzet',
    '🟦' => 'kék négyzet',
    '🟪' => 'lila négyzet',
    '🟫' => 'barna négyzet',
    '⬛' => 'nagy fekete négyzet',
    '⬜' => 'nagy fehér négyzet',
    '◼' => 'közepes fekete négyzet',
    '◻' => 'közepes fehér négyzet',
    '◾' => 'kis-közepes fekete négyzet',
    '◽' => 'kis-közepes fehér négyzet',
    '▪' => 'kis fekete négyzet',
    '▫' => 'kis fehér négyzet',
    '🔶' => 'nagy narancssárga rombusz',
    '🔷' => 'nagy kék rombusz',
    '🔸' => 'kis narancssárga rombusz',
    '🔹' => 'kis kék rombusz',
    '🔺' => 'fölfelé mutató piros háromszög',
    '🔻' => 'lefelé mutató piros háromszög',
    '💠' => 'rombusz ponttal',
    '🔘' => 'választógomb',
    '🔳' => 'fehér, négyzet alakú gomb',
    '🔲' => 'fekete, négyzet alakú gomb',
    '🏁' => 'kockás zászló',
    '🚩' => 'háromszög alakú zászló',
    '🎌' => 'keresztbe tett zászlók',
    '🏴' => 'lengő fekete zászló',
    '🏳' => 'lengő fehér zászló',
];
