<?php

return [
    '🧑🏻‍❤‍💋‍🧑🏼' => 'φιλί: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏻‍❤‍💋‍🧑🏽' => 'φιλί: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏻‍❤‍💋‍🧑🏾' => 'φιλί: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍❤‍💋‍🧑🏿' => 'φιλί: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏼‍❤‍💋‍🧑🏻' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍❤‍💋‍🧑🏽' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏼‍❤‍💋‍🧑🏾' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏼‍❤‍💋‍🧑🏿' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏽‍❤‍💋‍🧑🏻' => 'φιλί: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍❤‍💋‍🧑🏼' => 'φιλί: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍❤‍💋‍🧑🏾' => 'φιλί: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏽‍❤‍💋‍🧑🏿' => 'φιλί: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏾‍❤‍💋‍🧑🏻' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍❤‍💋‍🧑🏼' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍❤‍💋‍🧑🏽' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏾‍❤‍💋‍🧑🏿' => 'φιλί: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏿‍❤‍💋‍🧑🏻' => 'φιλί: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍❤‍💋‍🧑🏼' => 'φιλί: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍❤‍💋‍🧑🏽' => 'φιλί: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏿‍❤‍💋‍🧑🏾' => 'φιλί: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍❤‍💋‍👨🏻' => 'φιλί: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏻‍❤‍💋‍👨🏼' => 'φιλί: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏻‍❤‍💋‍👨🏽' => 'φιλί: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏻‍❤‍💋‍👨🏾' => 'φιλί: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍❤‍💋‍👨🏿' => 'φιλί: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏼‍❤‍💋‍👨🏻' => 'φιλί: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍❤‍💋‍👨🏼' => 'φιλί: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍❤‍💋‍👨🏽' => 'φιλί: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏼‍❤‍💋‍👨🏾' => 'φιλί: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏼‍❤‍💋‍👨🏿' => 'φιλί: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏽‍❤‍💋‍👨🏻' => 'φιλί: άντρας, άντρας, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍❤‍💋‍👨🏼' => 'φιλί: άντρας, άντρας, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍❤‍💋‍👨🏽' => 'φιλί: άντρας, άντρας, μεσαίος τόνος δέρματος',
    '👨🏽‍❤‍💋‍👨🏾' => 'φιλί: άντρας, άντρας, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏽‍❤‍💋‍👨🏿' => 'φιλί: άντρας, άντρας, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏾‍❤‍💋‍👨🏻' => 'φιλί: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍❤‍💋‍👨🏼' => 'φιλί: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍❤‍💋‍👨🏽' => 'φιλί: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏾‍❤‍💋‍👨🏾' => 'φιλί: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏾‍❤‍💋‍👨🏿' => 'φιλί: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏿‍❤‍💋‍👨🏻' => 'φιλί: άντρας, άντρας, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍❤‍💋‍👨🏼' => 'φιλί: άντρας, άντρας, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍❤‍💋‍👨🏽' => 'φιλί: άντρας, άντρας, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏿‍❤‍💋‍👨🏾' => 'φιλί: άντρας, άντρας, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍❤‍💋‍👨🏿' => 'φιλί: άντρας, άντρας, σκούρος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👨🏻' => 'φιλί: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👨🏼' => 'φιλί: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👨🏽' => 'φιλί: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👨🏾' => 'φιλί: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👨🏿' => 'φιλί: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👨🏻' => 'φιλί: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👨🏼' => 'φιλί: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👨🏽' => 'φιλί: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👨🏾' => 'φιλί: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👨🏿' => 'φιλί: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👨🏻' => 'φιλί: γυναίκα, άντρας, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👨🏼' => 'φιλί: γυναίκα, άντρας, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👨🏽' => 'φιλί: γυναίκα, άντρας, μεσαίος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👨🏾' => 'φιλί: γυναίκα, άντρας, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👨🏿' => 'φιλί: γυναίκα, άντρας, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👨🏻' => 'φιλί: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👨🏼' => 'φιλί: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👨🏽' => 'φιλί: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👨🏾' => 'φιλί: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👨🏿' => 'φιλί: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👨🏻' => 'φιλί: γυναίκα, άντρας, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👨🏼' => 'φιλί: γυναίκα, άντρας, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👨🏽' => 'φιλί: γυναίκα, άντρας, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👨🏾' => 'φιλί: γυναίκα, άντρας, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👨🏿' => 'φιλί: γυναίκα, άντρας, σκούρος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👩🏻' => 'φιλί: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👩🏼' => 'φιλί: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👩🏽' => 'φιλί: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👩🏾' => 'φιλί: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍❤‍💋‍👩🏿' => 'φιλί: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👩🏻' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👩🏼' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👩🏽' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👩🏾' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍❤‍💋‍👩🏿' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👩🏻' => 'φιλί: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👩🏼' => 'φιλί: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👩🏽' => 'φιλί: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👩🏾' => 'φιλί: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍❤‍💋‍👩🏿' => 'φιλί: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👩🏻' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👩🏼' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👩🏽' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👩🏾' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏾‍❤‍💋‍👩🏿' => 'φιλί: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👩🏻' => 'φιλί: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👩🏼' => 'φιλί: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👩🏽' => 'φιλί: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👩🏾' => 'φιλί: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍❤‍💋‍👩🏿' => 'φιλί: γυναίκα, γυναίκα, σκούρος τόνος δέρματος',
    '🧑🏻‍🫯‍🧑🏼' => 'πάλη: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏻‍🫯‍🧑🏽' => 'πάλη: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏻‍🫯‍🧑🏾' => 'πάλη: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍🫯‍🧑🏿' => 'πάλη: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏼‍🫯‍🧑🏻' => 'πάλη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🫯‍🧑🏽' => 'πάλη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏼‍🫯‍🧑🏾' => 'πάλη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏼‍🫯‍🧑🏿' => 'πάλη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏽‍🫯‍🧑🏻' => 'πάλη: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🫯‍🧑🏼' => 'πάλη: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🫯‍🧑🏾' => 'πάλη: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏽‍🫯‍🧑🏿' => 'πάλη: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏾‍🫯‍🧑🏻' => 'πάλη: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍🫯‍🧑🏼' => 'πάλη: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍🫯‍🧑🏽' => 'πάλη: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏾‍🫯‍🧑🏿' => 'πάλη: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏿‍🫯‍🧑🏻' => 'πάλη: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍🫯‍🧑🏼' => 'πάλη: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍🫯‍🧑🏽' => 'πάλη: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏿‍🫯‍🧑🏾' => 'πάλη: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍❤‍🧑🏼' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏻‍❤‍🧑🏽' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏻‍❤‍🧑🏾' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍❤‍🧑🏿' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏼‍❤‍🧑🏻' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍❤‍🧑🏽' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏼‍❤‍🧑🏾' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏼‍❤‍🧑🏿' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏽‍❤‍🧑🏻' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍❤‍🧑🏼' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍❤‍🧑🏾' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏽‍❤‍🧑🏿' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏾‍❤‍🧑🏻' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍❤‍🧑🏼' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍❤‍🧑🏽' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏾‍❤‍🧑🏿' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏿‍❤‍🧑🏻' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍❤‍🧑🏼' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍❤‍🧑🏽' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏿‍❤‍🧑🏾' => 'ζευγάρι με καρδιά: ενήλικας, ενήλικας, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍🐰‍🧑🏼' => 'άτομα με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏻‍🐰‍🧑🏽' => 'άτομα με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏻‍🐰‍🧑🏾' => 'άτομα με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍🐰‍🧑🏿' => 'άτομα με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏼‍🐰‍🧑🏻' => 'άτομα με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🐰‍🧑🏽' => 'άτομα με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏼‍🐰‍🧑🏾' => 'άτομα με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏼‍🐰‍🧑🏿' => 'άτομα με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏽‍🐰‍🧑🏻' => 'άτομα με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🐰‍🧑🏼' => 'άτομα με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🐰‍🧑🏾' => 'άτομα με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏽‍🐰‍🧑🏿' => 'άτομα με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏾‍🐰‍🧑🏻' => 'άτομα με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍🐰‍🧑🏼' => 'άτομα με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍🐰‍🧑🏽' => 'άτομα με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏾‍🐰‍🧑🏿' => 'άτομα με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏿‍🐰‍🧑🏻' => 'άτομα με λαγουδένια αυτιά: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍🐰‍🧑🏼' => 'άτομα με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍🐰‍🧑🏽' => 'άτομα με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏿‍🐰‍🧑🏾' => 'άτομα με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍🫯‍👨🏼' => 'παλαιστές: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏻‍🫯‍👨🏽' => 'παλαιστές: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏻‍🫯‍👨🏾' => 'παλαιστές: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍🫯‍👨🏿' => 'παλαιστές: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏼‍🫯‍👨🏻' => 'παλαιστές: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🫯‍👨🏽' => 'παλαιστές: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏼‍🫯‍👨🏾' => 'παλαιστές: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏼‍🫯‍👨🏿' => 'παλαιστές: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏽‍🫯‍👨🏻' => 'παλαιστές: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🫯‍👨🏼' => 'παλαιστές: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🫯‍👨🏾' => 'παλαιστές: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏽‍🫯‍👨🏿' => 'παλαιστές: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏾‍🫯‍👨🏻' => 'παλαιστές: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍🫯‍👨🏼' => 'παλαιστές: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍🫯‍👨🏽' => 'παλαιστές: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏾‍🫯‍👨🏿' => 'παλαιστές: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏿‍🫯‍👨🏻' => 'παλαιστές: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍🫯‍👨🏼' => 'παλαιστές: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍🫯‍👨🏽' => 'παλαιστές: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏿‍🫯‍👨🏾' => 'παλαιστές: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍❤‍👨🏻' => 'ζευγάρι με καρδιά: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏻‍❤‍👨🏼' => 'ζευγάρι με καρδιά: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏻‍❤‍👨🏽' => 'ζευγάρι με καρδιά: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏻‍❤‍👨🏾' => 'ζευγάρι με καρδιά: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍❤‍👨🏿' => 'ζευγάρι με καρδιά: άντρας, άντρας, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏼‍❤‍👨🏻' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍❤‍👨🏼' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍❤‍👨🏽' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏼‍❤‍👨🏾' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏼‍❤‍👨🏿' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏽‍❤‍👨🏻' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍❤‍👨🏼' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍❤‍👨🏽' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος τόνος δέρματος',
    '👨🏽‍❤‍👨🏾' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏽‍❤‍👨🏿' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏾‍❤‍👨🏻' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍❤‍👨🏼' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍❤‍👨🏽' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏾‍❤‍👨🏾' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏾‍❤‍👨🏿' => 'ζευγάρι με καρδιά: άντρας, άντρας, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏿‍❤‍👨🏻' => 'ζευγάρι με καρδιά: άντρας, άντρας, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍❤‍👨🏼' => 'ζευγάρι με καρδιά: άντρας, άντρας, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍❤‍👨🏽' => 'ζευγάρι με καρδιά: άντρας, άντρας, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏿‍❤‍👨🏾' => 'ζευγάρι με καρδιά: άντρας, άντρας, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍❤‍👨🏿' => 'ζευγάρι με καρδιά: άντρας, άντρας, σκούρος τόνος δέρματος',
    '👨🏻‍🐰‍👨🏼' => 'άντρες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏻‍🐰‍👨🏽' => 'άντρες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏻‍🐰‍👨🏾' => 'άντρες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍🐰‍👨🏿' => 'άντρες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏼‍🐰‍👨🏻' => 'άντρες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🐰‍👨🏽' => 'άντρες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏼‍🐰‍👨🏾' => 'άντρες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏼‍🐰‍👨🏿' => 'άντρες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏽‍🐰‍👨🏻' => 'άντρες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🐰‍👨🏼' => 'άντρες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🐰‍👨🏾' => 'άντρες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏽‍🐰‍👨🏿' => 'άντρες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏾‍🐰‍👨🏻' => 'άντρες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍🐰‍👨🏼' => 'άντρες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍🐰‍👨🏽' => 'άντρες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏾‍🐰‍👨🏿' => 'άντρες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏿‍🐰‍👨🏻' => 'άντρες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍🐰‍👨🏼' => 'άντρες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍🐰‍👨🏽' => 'άντρες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏿‍🐰‍👨🏾' => 'άντρες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍🫯‍👩🏼' => 'παλαίστριες: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍🫯‍👩🏽' => 'παλαίστριες: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍🫯‍👩🏾' => 'παλαίστριες: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍🫯‍👩🏿' => 'παλαίστριες: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍🫯‍👩🏻' => 'παλαίστριες: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🫯‍👩🏽' => 'παλαίστριες: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍🫯‍👩🏾' => 'παλαίστριες: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍🫯‍👩🏿' => 'παλαίστριες: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍🫯‍👩🏻' => 'παλαίστριες: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🫯‍👩🏼' => 'παλαίστριες: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🫯‍👩🏾' => 'παλαίστριες: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍🫯‍👩🏿' => 'παλαίστριες: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍🫯‍👩🏻' => 'παλαίστριες: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🫯‍👩🏼' => 'παλαίστριες: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🫯‍👩🏽' => 'παλαίστριες: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍🫯‍👩🏿' => 'παλαίστριες: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍🫯‍👩🏻' => 'παλαίστριες: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🫯‍👩🏼' => 'παλαίστριες: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🫯‍👩🏽' => 'παλαίστριες: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍🫯‍👩🏾' => 'παλαίστριες: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍❤‍👨🏻' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍👨🏼' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍👨🏽' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍❤‍👨🏾' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍❤‍👨🏿' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍❤‍👨🏻' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍👨🏼' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍👨🏽' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍❤‍👨🏾' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍❤‍👨🏿' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍❤‍👨🏻' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍👨🏼' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍👨🏽' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος τόνος δέρματος',
    '👩🏽‍❤‍👨🏾' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍❤‍👨🏿' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍❤‍👨🏻' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍👨🏼' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍👨🏽' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍❤‍👨🏾' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏾‍❤‍👨🏿' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍❤‍👨🏻' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍👨🏼' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍👨🏽' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍❤‍👨🏾' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍❤‍👨🏿' => 'ζευγάρι με καρδιά: γυναίκα, άντρας, σκούρος τόνος δέρματος',
    '👩🏻‍❤‍👩🏻' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍👩🏼' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍❤‍👩🏽' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍❤‍👩🏾' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍❤‍👩🏿' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍❤‍👩🏻' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍👩🏼' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍❤‍👩🏽' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍❤‍👩🏾' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍❤‍👩🏿' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍❤‍👩🏻' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍👩🏼' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍❤‍👩🏽' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος',
    '👩🏽‍❤‍👩🏾' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍❤‍👩🏿' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍❤‍👩🏻' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍👩🏼' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍❤‍👩🏽' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍❤‍👩🏾' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏾‍❤‍👩🏿' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍❤‍👩🏻' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍👩🏼' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍❤‍👩🏽' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍❤‍👩🏾' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍❤‍👩🏿' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα, σκούρος τόνος δέρματος',
    '👩🏻‍🐰‍👩🏼' => 'γυναίκες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍🐰‍👩🏽' => 'γυναίκες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍🐰‍👩🏾' => 'γυναίκες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍🐰‍👩🏿' => 'γυναίκες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍🐰‍👩🏻' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🐰‍👩🏽' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍🐰‍👩🏾' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍🐰‍👩🏿' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍🐰‍👩🏻' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🐰‍👩🏼' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🐰‍👩🏾' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍🐰‍👩🏿' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍🐰‍👩🏻' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🐰‍👩🏼' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🐰‍👩🏽' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍🐰‍👩🏿' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍🐰‍👩🏻' => 'γυναίκες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🐰‍👩🏼' => 'γυναίκες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🐰‍👩🏽' => 'γυναίκες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍🐰‍👩🏾' => 'γυναίκες με λαγουδένια αυτιά: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍🤝‍🧑🏻' => 'άτομα πιασμένα χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏻‍🤝‍🧑🏼' => 'άτομα πιασμένα χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏻‍🤝‍🧑🏽' => 'άτομα πιασμένα χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏻‍🤝‍🧑🏾' => 'άτομα πιασμένα χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏻‍🤝‍🧑🏿' => 'άτομα πιασμένα χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏼‍🤝‍🧑🏻' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🤝‍🧑🏼' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🤝‍🧑🏽' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏼‍🤝‍🧑🏾' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏼‍🤝‍🧑🏿' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏽‍🤝‍🧑🏻' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🤝‍🧑🏼' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🤝‍🧑🏽' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος τόνος δέρματος',
    '🧑🏽‍🤝‍🧑🏾' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏽‍🤝‍🧑🏿' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏾‍🤝‍🧑🏻' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍🤝‍🧑🏼' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏾‍🤝‍🧑🏽' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏾‍🤝‍🧑🏾' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏾‍🤝‍🧑🏿' => 'άτομα πιασμένα χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🧑🏿‍🤝‍🧑🏻' => 'άτομα πιασμένα χέρι-χέρι: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍🤝‍🧑🏼' => 'άτομα πιασμένα χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏿‍🤝‍🧑🏽' => 'άτομα πιασμένα χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🧑🏿‍🤝‍🧑🏾' => 'άτομα πιασμένα χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🤝‍🧑🏿' => 'άτομα πιασμένα χέρι-χέρι: σκούρος τόνος δέρματος',
    '👩🏻‍🤝‍👩🏼' => 'γυναίκες πιασμένες χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍🤝‍👩🏽' => 'γυναίκες πιασμένες χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍🤝‍👩🏾' => 'γυναίκες πιασμένες χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍🤝‍👩🏿' => 'γυναίκες πιασμένες χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍🤝‍👩🏻' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🤝‍👩🏽' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍🤝‍👩🏾' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍🤝‍👩🏿' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍🤝‍👩🏻' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🤝‍👩🏼' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🤝‍👩🏾' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍🤝‍👩🏿' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍🤝‍👩🏻' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🤝‍👩🏼' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🤝‍👩🏽' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍🤝‍👩🏿' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍🤝‍👩🏻' => 'γυναίκες πιασμένες χέρι-χέρι: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🤝‍👩🏼' => 'γυναίκες πιασμένες χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🤝‍👩🏽' => 'γυναίκες πιασμένες χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍🤝‍👩🏾' => 'γυναίκες πιασμένες χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍🤝‍👨🏼' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏻‍🤝‍👨🏽' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏻‍🤝‍👨🏾' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏻‍🤝‍👨🏿' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏼‍🤝‍👨🏻' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🤝‍👨🏽' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏼‍🤝‍👨🏾' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏼‍🤝‍👨🏿' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏽‍🤝‍👨🏻' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🤝‍👨🏼' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🤝‍👨🏾' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩🏽‍🤝‍👨🏿' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏾‍🤝‍👨🏻' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🤝‍👨🏼' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏾‍🤝‍👨🏽' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏾‍🤝‍👨🏿' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👩🏿‍🤝‍👨🏻' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🤝‍👨🏼' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏿‍🤝‍👨🏽' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👩🏿‍🤝‍👨🏾' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍🤝‍👨🏼' => 'άντρες πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏻‍🤝‍👨🏽' => 'άντρες πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏻‍🤝‍👨🏾' => 'άντρες πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏻‍🤝‍👨🏿' => 'άντρες πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏼‍🤝‍👨🏻' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🤝‍👨🏽' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏼‍🤝‍👨🏾' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏼‍🤝‍👨🏿' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏽‍🤝‍👨🏻' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🤝‍👨🏼' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🤝‍👨🏾' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👨🏽‍🤝‍👨🏿' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏾‍🤝‍👨🏻' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍🤝‍👨🏼' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏾‍🤝‍👨🏽' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏾‍🤝‍👨🏿' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '👨🏿‍🤝‍👨🏻' => 'άντρες πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍🤝‍👨🏼' => 'άντρες πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏿‍🤝‍👨🏽' => 'άντρες πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '👨🏿‍🤝‍👨🏾' => 'άντρες πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '👩‍❤‍💋‍👨' => 'φιλί: γυναίκα, άντρας',
    '👨‍❤‍💋‍👨' => 'φιλί: άντρας, άντρας',
    '👩‍❤‍💋‍👩' => 'φιλί: γυναίκα, γυναίκα',
    '👨‍👩‍👧‍👦' => 'οικογένεια: άντρας, γυναίκα, κορίτσι, αγόρι',
    '👨‍👩‍👦‍👦' => 'οικογένεια: άντρας, γυναίκα, αγόρι, αγόρι',
    '👨‍👩‍👧‍👧' => 'οικογένεια: άντρας, γυναίκα, κορίτσι, κορίτσι',
    '👨‍👨‍👧‍👦' => 'οικογένεια: άντρας, άντρας, κορίτσι, αγόρι',
    '👨‍👨‍👦‍👦' => 'οικογένεια: άντρας, άντρας, αγόρι, αγόρι',
    '👨‍👨‍👧‍👧' => 'οικογένεια: άντρας, άντρας, κορίτσι, κορίτσι',
    '👩‍👩‍👧‍👦' => 'οικογένεια: γυναίκα, γυναίκα, κορίτσι, αγόρι',
    '👩‍👩‍👦‍👦' => 'οικογένεια: γυναίκα, γυναίκα, αγόρι, αγόρι',
    '👩‍👩‍👧‍👧' => 'οικογένεια: γυναίκα, γυναίκα, κορίτσι, κορίτσι',
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿' => 'σημαία: Αγγλία',
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿' => 'σημαία: Σκοτία',
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿' => 'σημαία: Ουαλία',
    '🧑‍🧑‍🧒‍🧒' => 'οικογένεια: ενήλικας, ενήλικας, παιδί, παιδί',
    '🚶🏻‍♀‍➡' => 'γυναίκα περπατά: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏼‍♀‍➡' => 'γυναίκα περπατά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏽‍♀‍➡' => 'γυναίκα περπατά: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏾‍♀‍➡' => 'γυναίκα περπατά: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏿‍♀‍➡' => 'γυναίκα περπατά: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏻‍♂‍➡' => 'άντρας περπατά: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏼‍♂‍➡' => 'άντρας περπατά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏽‍♂‍➡' => 'άντρας περπατά: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏾‍♂‍➡' => 'άντρας περπατά: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏿‍♂‍➡' => 'άντρας περπατά: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏻‍♀‍➡' => 'γυναίκα που γονατίζει: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏼‍♀‍➡' => 'γυναίκα που γονατίζει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏽‍♀‍➡' => 'γυναίκα που γονατίζει: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏾‍♀‍➡' => 'γυναίκα που γονατίζει: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏿‍♀‍➡' => 'γυναίκα που γονατίζει: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏻‍♂‍➡' => 'άντρας που γονατίζει: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏼‍♂‍➡' => 'άντρας που γονατίζει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏽‍♂‍➡' => 'άντρας που γονατίζει: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏾‍♂‍➡' => 'άντρας που γονατίζει: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏿‍♂‍➡' => 'άντρας που γονατίζει: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏻‍🦯‍➡' => 'άτομο με μπαστούνι για τυφλούς: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏼‍🦯‍➡' => 'άτομο με μπαστούνι για τυφλούς: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏽‍🦯‍➡' => 'άτομο με μπαστούνι για τυφλούς: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏾‍🦯‍➡' => 'άτομο με μπαστούνι για τυφλούς: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏿‍🦯‍➡' => 'άτομο με μπαστούνι για τυφλούς: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏻‍🦯‍➡' => 'άντρας με μπαστούνι για τυφλούς: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏼‍🦯‍➡' => 'άντρας με μπαστούνι για τυφλούς: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏽‍🦯‍➡' => 'άντρας με μπαστούνι για τυφλούς: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏾‍🦯‍➡' => 'άντρας με μπαστούνι για τυφλούς: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏿‍🦯‍➡' => 'άντρας με μπαστούνι για τυφλούς: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏻‍🦯‍➡' => 'γυναίκα με μπαστούνι για τυφλούς: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏼‍🦯‍➡' => 'γυναίκα με μπαστούνι για τυφλούς: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏽‍🦯‍➡' => 'γυναίκα με μπαστούνι για τυφλούς: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏾‍🦯‍➡' => 'γυναίκα με μπαστούνι για τυφλούς: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏿‍🦯‍➡' => 'γυναίκα με μπαστούνι για τυφλούς: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏻‍🦼‍➡' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏼‍🦼‍➡' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏽‍🦼‍➡' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏾‍🦼‍➡' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏿‍🦼‍➡' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏻‍🦼‍➡' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏼‍🦼‍➡' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏽‍🦼‍➡' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏾‍🦼‍➡' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏿‍🦼‍➡' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏻‍🦼‍➡' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏼‍🦼‍➡' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏽‍🦼‍➡' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏾‍🦼‍➡' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏿‍🦼‍➡' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏻‍🦽‍➡' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏼‍🦽‍➡' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏽‍🦽‍➡' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏾‍🦽‍➡' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏿‍🦽‍➡' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏻‍🦽‍➡' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏼‍🦽‍➡' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏽‍🦽‍➡' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏾‍🦽‍➡' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👨🏿‍🦽‍➡' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏻‍🦽‍➡' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏼‍🦽‍➡' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏽‍🦽‍➡' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏾‍🦽‍➡' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👩🏿‍🦽‍➡' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏻‍♀‍➡' => 'γυναίκα τρέχει: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏼‍♀‍➡' => 'γυναίκα τρέχει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏽‍♀‍➡' => 'γυναίκα τρέχει: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏾‍♀‍➡' => 'γυναίκα τρέχει: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏿‍♀‍➡' => 'γυναίκα τρέχει: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏻‍♂‍➡' => 'άντρας τρέχει: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏼‍♂‍➡' => 'άντρας τρέχει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏽‍♂‍➡' => 'άντρας τρέχει: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏾‍♂‍➡' => 'άντρας τρέχει: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏿‍♂‍➡' => 'άντρας τρέχει: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🫱🏻‍🫲🏼' => 'χειραψία: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏻‍🫲🏽' => 'χειραψία: ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🫱🏻‍🫲🏾' => 'χειραψία: ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🫱🏻‍🫲🏿' => 'χειραψία: ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🫱🏼‍🫲🏻' => 'χειραψία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏼‍🫲🏽' => 'χειραψία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🫱🏼‍🫲🏾' => 'χειραψία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🫱🏼‍🫲🏿' => 'χειραψία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🫱🏽‍🫲🏻' => 'χειραψία: μεσαίος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏽‍🫲🏼' => 'χειραψία: μεσαίος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏽‍🫲🏾' => 'χειραψία: μεσαίος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🫱🏽‍🫲🏿' => 'χειραψία: μεσαίος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🫱🏾‍🫲🏻' => 'χειραψία: μεσαίος-σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏾‍🫲🏼' => 'χειραψία: μεσαίος-σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏾‍🫲🏽' => 'χειραψία: μεσαίος-σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🫱🏾‍🫲🏿' => 'χειραψία: μεσαίος-σκούρος τόνος δέρματος, σκούρος τόνος δέρματος',
    '🫱🏿‍🫲🏻' => 'χειραψία: σκούρος τόνος δέρματος, ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏿‍🫲🏼' => 'χειραψία: σκούρος τόνος δέρματος, μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏿‍🫲🏽' => 'χειραψία: σκούρος τόνος δέρματος, μεσαίος τόνος δέρματος',
    '🫱🏿‍🫲🏾' => 'χειραψία: σκούρος τόνος δέρματος, μεσαίος-σκούρος τόνος δέρματος',
    '🚶‍♀‍➡' => 'γυναίκα περπατά: με κατεύθυνση προς τα δεξιά',
    '🚶‍♂‍➡' => 'άντρας περπατά: με κατεύθυνση προς τα δεξιά',
    '🧎‍♀‍➡' => 'γυναίκα που γονατίζει: με κατεύθυνση προς τα δεξιά',
    '🧎‍♂‍➡' => 'άντρας που γονατίζει: με κατεύθυνση προς τα δεξιά',
    '🧑‍🦯‍➡' => 'άτομο με μπαστούνι για τυφλούς: με κατεύθυνση προς τα δεξιά',
    '👨‍🦯‍➡' => 'άντρας με μπαστούνι για τυφλούς: με κατεύθυνση προς τα δεξιά',
    '👩‍🦯‍➡' => 'γυναίκα με μπαστούνι για τυφλούς: με κατεύθυνση προς τα δεξιά',
    '🧑‍🦼‍➡' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: με κατεύθυνση προς τα δεξιά',
    '👨‍🦼‍➡' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: με κατεύθυνση προς τα δεξιά',
    '👩‍🦼‍➡' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: με κατεύθυνση προς τα δεξιά',
    '🧑‍🦽‍➡' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: με κατεύθυνση προς τα δεξιά',
    '👨‍🦽‍➡' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: με κατεύθυνση προς τα δεξιά',
    '👩‍🦽‍➡' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: με κατεύθυνση προς τα δεξιά',
    '🏃‍♀‍➡' => 'γυναίκα τρέχει: με κατεύθυνση προς τα δεξιά',
    '🏃‍♂‍➡' => 'άντρας τρέχει: με κατεύθυνση προς τα δεξιά',
    '👩‍❤‍👨' => 'ζευγάρι με καρδιά: γυναίκα, άντρας',
    '👨‍❤‍👨' => 'ζευγάρι με καρδιά: άντρας, άντρας',
    '👩‍❤‍👩' => 'ζευγάρι με καρδιά: γυναίκα, γυναίκα',
    '👨‍👩‍👦' => 'οικογένεια: άντρας, γυναίκα, αγόρι',
    '👨‍👩‍👧' => 'οικογένεια: άντρας, γυναίκα, κορίτσι',
    '👨‍👨‍👦' => 'οικογένεια: άντρας, άντρας, αγόρι',
    '👨‍👨‍👧' => 'οικογένεια: άντρας, άντρας, κορίτσι',
    '👩‍👩‍👦' => 'οικογένεια: γυναίκα, γυναίκα, αγόρι',
    '👩‍👩‍👧' => 'οικογένεια: γυναίκα, γυναίκα, κορίτσι',
    '👨‍👦‍👦' => 'οικογένεια: άντρας, αγόρι, αγόρι',
    '👨‍👧‍👦' => 'οικογένεια: άντρας, κορίτσι, αγόρι',
    '👨‍👧‍👧' => 'οικογένεια: άντρας, κορίτσι, κορίτσι',
    '👩‍👦‍👦' => 'οικογένεια: γυναίκα, αγόρι, αγόρι',
    '👩‍👧‍👦' => 'οικογένεια: γυναίκα, κορίτσι, αγόρι',
    '👩‍👧‍👧' => 'οικογένεια: γυναίκα, κορίτσι, κορίτσι',
    '🧑‍🤝‍🧑' => 'άτομα πιασμένα χέρι-χέρι',
    '🧑‍🧑‍🧒' => 'οικογένεια: ενήλικας, ενήλικας, παιδί',
    '🧑‍🧒‍🧒' => 'οικογένεια: ενήλικας, παιδί, παιδί',
    '🧑🏻‍🦰' => 'ενήλικας: ανοιχτόχρωμος τόνος δέρματος, κόκκινα μαλλιά',
    '🧑🏼‍🦰' => 'ενήλικας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, κόκκινα μαλλιά',
    '🧑🏽‍🦰' => 'ενήλικας: μεσαίος τόνος δέρματος, κόκκινα μαλλιά',
    '🧑🏾‍🦰' => 'ενήλικας: μεσαίος-σκούρος τόνος δέρματος, κόκκινα μαλλιά',
    '🧑🏿‍🦰' => 'ενήλικας: σκούρος τόνος δέρματος, κόκκινα μαλλιά',
    '🧑🏻‍🦱' => 'ενήλικας: ανοιχτόχρωμος τόνος δέρματος, σγουρά μαλλιά',
    '🧑🏼‍🦱' => 'ενήλικας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σγουρά μαλλιά',
    '🧑🏽‍🦱' => 'ενήλικας: μεσαίος τόνος δέρματος, σγουρά μαλλιά',
    '🧑🏾‍🦱' => 'ενήλικας: μεσαίος-σκούρος τόνος δέρματος, σγουρά μαλλιά',
    '🧑🏿‍🦱' => 'ενήλικας: σκούρος τόνος δέρματος, σγουρά μαλλιά',
    '🧑🏻‍🦳' => 'ενήλικας: ανοιχτόχρωμος τόνος δέρματος, άσπρα μαλλιά',
    '🧑🏼‍🦳' => 'ενήλικας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, άσπρα μαλλιά',
    '🧑🏽‍🦳' => 'ενήλικας: μεσαίος τόνος δέρματος, άσπρα μαλλιά',
    '🧑🏾‍🦳' => 'ενήλικας: μεσαίος-σκούρος τόνος δέρματος, άσπρα μαλλιά',
    '🧑🏿‍🦳' => 'ενήλικας: σκούρος τόνος δέρματος, άσπρα μαλλιά',
    '🧑🏻‍🦲' => 'ενήλικας: ανοιχτόχρωμος τόνος δέρματος, καραφλός',
    '🧑🏼‍🦲' => 'ενήλικας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, καραφλός',
    '🧑🏽‍🦲' => 'ενήλικας: μεσαίος τόνος δέρματος, καραφλός',
    '🧑🏾‍🦲' => 'ενήλικας: μεσαίος-σκούρος τόνος δέρματος, καραφλός',
    '🧑🏿‍🦲' => 'ενήλικας: σκούρος τόνος δέρματος, καραφλός',
    '🧑🏻‍🩰' => 'χορευτής μπαλέτου: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🩰' => 'χορευτής μπαλέτου: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🩰' => 'χορευτής μπαλέτου: μεσαίος τόνος δέρματος',
    '🧑🏾‍🩰' => 'χορευτής μπαλέτου: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🩰' => 'χορευτής μπαλέτου: σκούρος τόνος δέρματος',
    '🧔🏻‍♂' => 'άντρας: ανοιχτόχρωμος τόνος δέρματος, μούσι',
    '🧔🏼‍♂' => 'άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μούσι',
    '🧔🏽‍♂' => 'άντρας: μεσαίος τόνος δέρματος, μούσι',
    '🧔🏾‍♂' => 'άντρας: μεσαίος-σκούρος τόνος δέρματος, μούσι',
    '🧔🏿‍♂' => 'άντρας: σκούρος τόνος δέρματος, μούσι',
    '👱🏻‍♂' => 'άντρας: ανοιχτόχρωμος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏼‍♂' => 'άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏽‍♂' => 'άντρας: μεσαίος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏾‍♂' => 'άντρας: μεσαίος-σκούρος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏿‍♂' => 'άντρας: σκούρος τόνος δέρματος, ξανθά μαλλιά',
    '👨🏻‍🦰' => 'άντρας: ανοιχτόχρωμος τόνος δέρματος, κόκκινα μαλλιά',
    '👨🏼‍🦰' => 'άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, κόκκινα μαλλιά',
    '👨🏽‍🦰' => 'άντρας: μεσαίος τόνος δέρματος, κόκκινα μαλλιά',
    '👨🏾‍🦰' => 'άντρας: μεσαίος-σκούρος τόνος δέρματος, κόκκινα μαλλιά',
    '👨🏿‍🦰' => 'άντρας: σκούρος τόνος δέρματος, κόκκινα μαλλιά',
    '👨🏻‍🦱' => 'άντρας: ανοιχτόχρωμος τόνος δέρματος, σγουρά μαλλιά',
    '👨🏼‍🦱' => 'άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σγουρά μαλλιά',
    '👨🏽‍🦱' => 'άντρας: μεσαίος τόνος δέρματος, σγουρά μαλλιά',
    '👨🏾‍🦱' => 'άντρας: μεσαίος-σκούρος τόνος δέρματος, σγουρά μαλλιά',
    '👨🏿‍🦱' => 'άντρας: σκούρος τόνος δέρματος, σγουρά μαλλιά',
    '👨🏻‍🦳' => 'άντρας: ανοιχτόχρωμος τόνος δέρματος, άσπρα μαλλιά',
    '👨🏼‍🦳' => 'άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, άσπρα μαλλιά',
    '👨🏽‍🦳' => 'άντρας: μεσαίος τόνος δέρματος, άσπρα μαλλιά',
    '👨🏾‍🦳' => 'άντρας: μεσαίος-σκούρος τόνος δέρματος, άσπρα μαλλιά',
    '👨🏿‍🦳' => 'άντρας: σκούρος τόνος δέρματος, άσπρα μαλλιά',
    '👨🏻‍🦲' => 'άντρας: ανοιχτόχρωμος τόνος δέρματος, καραφλός',
    '👨🏼‍🦲' => 'άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, καραφλός',
    '👨🏽‍🦲' => 'άντρας: μεσαίος τόνος δέρματος, καραφλός',
    '👨🏾‍🦲' => 'άντρας: μεσαίος-σκούρος τόνος δέρματος, καραφλός',
    '👨🏿‍🦲' => 'άντρας: σκούρος τόνος δέρματος, καραφλός',
    '🧔🏻‍♀' => 'γυναίκα: ανοιχτόχρωμος τόνος δέρματος, μούσι',
    '🧔🏼‍♀' => 'γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μούσι',
    '🧔🏽‍♀' => 'γυναίκα: μεσαίος τόνος δέρματος, μούσι',
    '🧔🏾‍♀' => 'γυναίκα: μεσαίος-σκούρος τόνος δέρματος, μούσι',
    '🧔🏿‍♀' => 'γυναίκα: σκούρος τόνος δέρματος, μούσι',
    '👱🏻‍♀' => 'γυναίκα: ανοιχτόχρωμος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏼‍♀' => 'γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏽‍♀' => 'γυναίκα: μεσαίος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏾‍♀' => 'γυναίκα: μεσαίος-σκούρος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏿‍♀' => 'γυναίκα: σκούρος τόνος δέρματος, ξανθά μαλλιά',
    '👩🏻‍🦰' => 'γυναίκα: ανοιχτόχρωμος τόνος δέρματος, κόκκινα μαλλιά',
    '👩🏼‍🦰' => 'γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, κόκκινα μαλλιά',
    '👩🏽‍🦰' => 'γυναίκα: μεσαίος τόνος δέρματος, κόκκινα μαλλιά',
    '👩🏾‍🦰' => 'γυναίκα: μεσαίος-σκούρος τόνος δέρματος, κόκκινα μαλλιά',
    '👩🏿‍🦰' => 'γυναίκα: σκούρος τόνος δέρματος, κόκκινα μαλλιά',
    '👩🏻‍🦱' => 'γυναίκα: ανοιχτόχρωμος τόνος δέρματος, σγουρά μαλλιά',
    '👩🏼‍🦱' => 'γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, σγουρά μαλλιά',
    '👩🏽‍🦱' => 'γυναίκα: μεσαίος τόνος δέρματος, σγουρά μαλλιά',
    '👩🏾‍🦱' => 'γυναίκα: μεσαίος-σκούρος τόνος δέρματος, σγουρά μαλλιά',
    '👩🏿‍🦱' => 'γυναίκα: σκούρος τόνος δέρματος, σγουρά μαλλιά',
    '👩🏻‍🦳' => 'γυναίκα: ανοιχτόχρωμος τόνος δέρματος, άσπρα μαλλιά',
    '👩🏼‍🦳' => 'γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, άσπρα μαλλιά',
    '👩🏽‍🦳' => 'γυναίκα: μεσαίος τόνος δέρματος, άσπρα μαλλιά',
    '👩🏾‍🦳' => 'γυναίκα: μεσαίος-σκούρος τόνος δέρματος, άσπρα μαλλιά',
    '👩🏿‍🦳' => 'γυναίκα: σκούρος τόνος δέρματος, άσπρα μαλλιά',
    '👩🏻‍🦲' => 'γυναίκα: ανοιχτόχρωμος τόνος δέρματος, καραφλός',
    '👩🏼‍🦲' => 'γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, καραφλός',
    '👩🏽‍🦲' => 'γυναίκα: μεσαίος τόνος δέρματος, καραφλός',
    '👩🏾‍🦲' => 'γυναίκα: μεσαίος-σκούρος τόνος δέρματος, καραφλός',
    '👩🏿‍🦲' => 'γυναίκα: σκούρος τόνος δέρματος, καραφλός',
    '🙍🏻‍♂' => 'συνοφρυωμένος: ανοιχτόχρωμος τόνος δέρματος',
    '🙍🏼‍♂' => 'συνοφρυωμένος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙍🏽‍♂' => 'συνοφρυωμένος: μεσαίος τόνος δέρματος',
    '🙍🏾‍♂' => 'συνοφρυωμένος: μεσαίος-σκούρος τόνος δέρματος',
    '🙍🏿‍♂' => 'συνοφρυωμένος: σκούρος τόνος δέρματος',
    '🙍🏻‍♀' => 'συνοφρυωμένη: ανοιχτόχρωμος τόνος δέρματος',
    '🙍🏼‍♀' => 'συνοφρυωμένη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙍🏽‍♀' => 'συνοφρυωμένη: μεσαίος τόνος δέρματος',
    '🙍🏾‍♀' => 'συνοφρυωμένη: μεσαίος-σκούρος τόνος δέρματος',
    '🙍🏿‍♀' => 'συνοφρυωμένη: σκούρος τόνος δέρματος',
    '🙎🏻‍♂' => 'κατσουφιασμένος άντρας: ανοιχτόχρωμος τόνος δέρματος',
    '🙎🏼‍♂' => 'κατσουφιασμένος άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙎🏽‍♂' => 'κατσουφιασμένος άντρας: μεσαίος τόνος δέρματος',
    '🙎🏾‍♂' => 'κατσουφιασμένος άντρας: μεσαίος-σκούρος τόνος δέρματος',
    '🙎🏿‍♂' => 'κατσουφιασμένος άντρας: σκούρος τόνος δέρματος',
    '🙎🏻‍♀' => 'κατσουφιασμένη γυναίκα: ανοιχτόχρωμος τόνος δέρματος',
    '🙎🏼‍♀' => 'κατσουφιασμένη γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙎🏽‍♀' => 'κατσουφιασμένη γυναίκα: μεσαίος τόνος δέρματος',
    '🙎🏾‍♀' => 'κατσουφιασμένη γυναίκα: μεσαίος-σκούρος τόνος δέρματος',
    '🙎🏿‍♀' => 'κατσουφιασμένη γυναίκα: σκούρος τόνος δέρματος',
    '🙅🏻‍♂' => 'άντρας διαφωνεί: ανοιχτόχρωμος τόνος δέρματος',
    '🙅🏼‍♂' => 'άντρας διαφωνεί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙅🏽‍♂' => 'άντρας διαφωνεί: μεσαίος τόνος δέρματος',
    '🙅🏾‍♂' => 'άντρας διαφωνεί: μεσαίος-σκούρος τόνος δέρματος',
    '🙅🏿‍♂' => 'άντρας διαφωνεί: σκούρος τόνος δέρματος',
    '🙅🏻‍♀' => 'γυναίκα διαφωνεί: ανοιχτόχρωμος τόνος δέρματος',
    '🙅🏼‍♀' => 'γυναίκα διαφωνεί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙅🏽‍♀' => 'γυναίκα διαφωνεί: μεσαίος τόνος δέρματος',
    '🙅🏾‍♀' => 'γυναίκα διαφωνεί: μεσαίος-σκούρος τόνος δέρματος',
    '🙅🏿‍♀' => 'γυναίκα διαφωνεί: σκούρος τόνος δέρματος',
    '🙆🏻‍♂' => 'άντρας συμφωνεί: ανοιχτόχρωμος τόνος δέρματος',
    '🙆🏼‍♂' => 'άντρας συμφωνεί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙆🏽‍♂' => 'άντρας συμφωνεί: μεσαίος τόνος δέρματος',
    '🙆🏾‍♂' => 'άντρας συμφωνεί: μεσαίος-σκούρος τόνος δέρματος',
    '🙆🏿‍♂' => 'άντρας συμφωνεί: σκούρος τόνος δέρματος',
    '🙆🏻‍♀' => 'γυναίκα συμφωνεί: ανοιχτόχρωμος τόνος δέρματος',
    '🙆🏼‍♀' => 'γυναίκα συμφωνεί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙆🏽‍♀' => 'γυναίκα συμφωνεί: μεσαίος τόνος δέρματος',
    '🙆🏾‍♀' => 'γυναίκα συμφωνεί: μεσαίος-σκούρος τόνος δέρματος',
    '🙆🏿‍♀' => 'γυναίκα συμφωνεί: σκούρος τόνος δέρματος',
    '💁🏻‍♂' => 'άντρας προσφέρει: ανοιχτόχρωμος τόνος δέρματος',
    '💁🏼‍♂' => 'άντρας προσφέρει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💁🏽‍♂' => 'άντρας προσφέρει: μεσαίος τόνος δέρματος',
    '💁🏾‍♂' => 'άντρας προσφέρει: μεσαίος-σκούρος τόνος δέρματος',
    '💁🏿‍♂' => 'άντρας προσφέρει: σκούρος τόνος δέρματος',
    '💁🏻‍♀' => 'γυναίκα προσφέρει: ανοιχτόχρωμος τόνος δέρματος',
    '💁🏼‍♀' => 'γυναίκα προσφέρει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💁🏽‍♀' => 'γυναίκα προσφέρει: μεσαίος τόνος δέρματος',
    '💁🏾‍♀' => 'γυναίκα προσφέρει: μεσαίος-σκούρος τόνος δέρματος',
    '💁🏿‍♀' => 'γυναίκα προσφέρει: σκούρος τόνος δέρματος',
    '🙋🏻‍♂' => 'άντρας με σηκωμένο χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '🙋🏼‍♂' => 'άντρας με σηκωμένο χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙋🏽‍♂' => 'άντρας με σηκωμένο χέρι: μεσαίος τόνος δέρματος',
    '🙋🏾‍♂' => 'άντρας με σηκωμένο χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '🙋🏿‍♂' => 'άντρας με σηκωμένο χέρι: σκούρος τόνος δέρματος',
    '🙋🏻‍♀' => 'γυναίκα με σηκωμένο χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '🙋🏼‍♀' => 'γυναίκα με σηκωμένο χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙋🏽‍♀' => 'γυναίκα με σηκωμένο χέρι: μεσαίος τόνος δέρματος',
    '🙋🏾‍♀' => 'γυναίκα με σηκωμένο χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '🙋🏿‍♀' => 'γυναίκα με σηκωμένο χέρι: σκούρος τόνος δέρματος',
    '🧏🏻‍♂' => 'κωφός άντρας: ανοιχτόχρωμος τόνος δέρματος',
    '🧏🏼‍♂' => 'κωφός άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧏🏽‍♂' => 'κωφός άντρας: μεσαίος τόνος δέρματος',
    '🧏🏾‍♂' => 'κωφός άντρας: μεσαίος-σκούρος τόνος δέρματος',
    '🧏🏿‍♂' => 'κωφός άντρας: σκούρος τόνος δέρματος',
    '🧏🏻‍♀' => 'κωφή γυναίκα: ανοιχτόχρωμος τόνος δέρματος',
    '🧏🏼‍♀' => 'κωφή γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧏🏽‍♀' => 'κωφή γυναίκα: μεσαίος τόνος δέρματος',
    '🧏🏾‍♀' => 'κωφή γυναίκα: μεσαίος-σκούρος τόνος δέρματος',
    '🧏🏿‍♀' => 'κωφή γυναίκα: σκούρος τόνος δέρματος',
    '🙇🏻‍♂' => 'άντρας υποκλίνεται: ανοιχτόχρωμος τόνος δέρματος',
    '🙇🏼‍♂' => 'άντρας υποκλίνεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙇🏽‍♂' => 'άντρας υποκλίνεται: μεσαίος τόνος δέρματος',
    '🙇🏾‍♂' => 'άντρας υποκλίνεται: μεσαίος-σκούρος τόνος δέρματος',
    '🙇🏿‍♂' => 'άντρας υποκλίνεται: σκούρος τόνος δέρματος',
    '🙇🏻‍♀' => 'γυναίκα υποκλίνεται: ανοιχτόχρωμος τόνος δέρματος',
    '🙇🏼‍♀' => 'γυναίκα υποκλίνεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙇🏽‍♀' => 'γυναίκα υποκλίνεται: μεσαίος τόνος δέρματος',
    '🙇🏾‍♀' => 'γυναίκα υποκλίνεται: μεσαίος-σκούρος τόνος δέρματος',
    '🙇🏿‍♀' => 'γυναίκα υποκλίνεται: σκούρος τόνος δέρματος',
    '🤦🏻‍♂' => 'άντρας αγανακτά: ανοιχτόχρωμος τόνος δέρματος',
    '🤦🏼‍♂' => 'άντρας αγανακτά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤦🏽‍♂' => 'άντρας αγανακτά: μεσαίος τόνος δέρματος',
    '🤦🏾‍♂' => 'άντρας αγανακτά: μεσαίος-σκούρος τόνος δέρματος',
    '🤦🏿‍♂' => 'άντρας αγανακτά: σκούρος τόνος δέρματος',
    '🤦🏻‍♀' => 'γυναίκα αγανακτά: ανοιχτόχρωμος τόνος δέρματος',
    '🤦🏼‍♀' => 'γυναίκα αγανακτά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤦🏽‍♀' => 'γυναίκα αγανακτά: μεσαίος τόνος δέρματος',
    '🤦🏾‍♀' => 'γυναίκα αγανακτά: μεσαίος-σκούρος τόνος δέρματος',
    '🤦🏿‍♀' => 'γυναίκα αγανακτά: σκούρος τόνος δέρματος',
    '🤷🏻‍♂' => 'άντρας σηκώνει τους ώμους: ανοιχτόχρωμος τόνος δέρματος',
    '🤷🏼‍♂' => 'άντρας σηκώνει τους ώμους: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤷🏽‍♂' => 'άντρας σηκώνει τους ώμους: μεσαίος τόνος δέρματος',
    '🤷🏾‍♂' => 'άντρας σηκώνει τους ώμους: μεσαίος-σκούρος τόνος δέρματος',
    '🤷🏿‍♂' => 'άντρας σηκώνει τους ώμους: σκούρος τόνος δέρματος',
    '🤷🏻‍♀' => 'γυναίκα σηκώνει τους ώμους: ανοιχτόχρωμος τόνος δέρματος',
    '🤷🏼‍♀' => 'γυναίκα σηκώνει τους ώμους: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤷🏽‍♀' => 'γυναίκα σηκώνει τους ώμους: μεσαίος τόνος δέρματος',
    '🤷🏾‍♀' => 'γυναίκα σηκώνει τους ώμους: μεσαίος-σκούρος τόνος δέρματος',
    '🤷🏿‍♀' => 'γυναίκα σηκώνει τους ώμους: σκούρος τόνος δέρματος',
    '🧑🏻‍⚕' => 'εργαζόμενος του τομέα υγείας: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍⚕' => 'εργαζόμενος του τομέα υγείας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍⚕' => 'εργαζόμενος του τομέα υγείας: μεσαίος τόνος δέρματος',
    '🧑🏾‍⚕' => 'εργαζόμενος του τομέα υγείας: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍⚕' => 'εργαζόμενος του τομέα υγείας: σκούρος τόνος δέρματος',
    '👨🏻‍⚕' => 'εργαζόμενος στην υγεία: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍⚕' => 'εργαζόμενος στην υγεία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍⚕' => 'εργαζόμενος στην υγεία: μεσαίος τόνος δέρματος',
    '👨🏾‍⚕' => 'εργαζόμενος στην υγεία: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍⚕' => 'εργαζόμενος στην υγεία: σκούρος τόνος δέρματος',
    '👩🏻‍⚕' => 'εργαζόμενη στην υγεία: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍⚕' => 'εργαζόμενη στην υγεία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍⚕' => 'εργαζόμενη στην υγεία: μεσαίος τόνος δέρματος',
    '👩🏾‍⚕' => 'εργαζόμενη στην υγεία: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍⚕' => 'εργαζόμενη στην υγεία: σκούρος τόνος δέρματος',
    '🧑🏻‍🎓' => 'σπουδαστής: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🎓' => 'σπουδαστής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🎓' => 'σπουδαστής: μεσαίος τόνος δέρματος',
    '🧑🏾‍🎓' => 'σπουδαστής: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🎓' => 'σπουδαστής: σκούρος τόνος δέρματος',
    '👨🏻‍🎓' => 'φοιτητής: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🎓' => 'φοιτητής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🎓' => 'φοιτητής: μεσαίος τόνος δέρματος',
    '👨🏾‍🎓' => 'φοιτητής: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🎓' => 'φοιτητής: σκούρος τόνος δέρματος',
    '👩🏻‍🎓' => 'φοιτήτρια: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🎓' => 'φοιτήτρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🎓' => 'φοιτήτρια: μεσαίος τόνος δέρματος',
    '👩🏾‍🎓' => 'φοιτήτρια: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🎓' => 'φοιτήτρια: σκούρος τόνος δέρματος',
    '🧑🏻‍🏫' => 'καθηγητής: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🏫' => 'καθηγητής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🏫' => 'καθηγητής: μεσαίος τόνος δέρματος',
    '🧑🏾‍🏫' => 'καθηγητής: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🏫' => 'καθηγητής: σκούρος τόνος δέρματος',
    '👨🏻‍🏫' => 'δάσκαλος: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🏫' => 'δάσκαλος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🏫' => 'δάσκαλος: μεσαίος τόνος δέρματος',
    '👨🏾‍🏫' => 'δάσκαλος: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🏫' => 'δάσκαλος: σκούρος τόνος δέρματος',
    '👩🏻‍🏫' => 'δασκάλα: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🏫' => 'δασκάλα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🏫' => 'δασκάλα: μεσαίος τόνος δέρματος',
    '👩🏾‍🏫' => 'δασκάλα: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🏫' => 'δασκάλα: σκούρος τόνος δέρματος',
    '🧑🏻‍⚖' => 'δικαστής: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍⚖' => 'δικαστής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍⚖' => 'δικαστής: μεσαίος τόνος δέρματος',
    '🧑🏾‍⚖' => 'δικαστής: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍⚖' => 'δικαστής: σκούρος τόνος δέρματος',
    '👨🏻‍⚖' => 'άντρας δικαστής: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍⚖' => 'άντρας δικαστής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍⚖' => 'άντρας δικαστής: μεσαίος τόνος δέρματος',
    '👨🏾‍⚖' => 'άντρας δικαστής: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍⚖' => 'άντρας δικαστής: σκούρος τόνος δέρματος',
    '👩🏻‍⚖' => 'γυναίκα δικαστής: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍⚖' => 'γυναίκα δικαστής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍⚖' => 'γυναίκα δικαστής: μεσαίος τόνος δέρματος',
    '👩🏾‍⚖' => 'γυναίκα δικαστής: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍⚖' => 'γυναίκα δικαστής: σκούρος τόνος δέρματος',
    '🧑🏻‍🌾' => 'καλλιεργητής: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🌾' => 'καλλιεργητής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🌾' => 'καλλιεργητής: μεσαίος τόνος δέρματος',
    '🧑🏾‍🌾' => 'καλλιεργητής: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🌾' => 'καλλιεργητής: σκούρος τόνος δέρματος',
    '👨🏻‍🌾' => 'αγρότης: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🌾' => 'αγρότης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🌾' => 'αγρότης: μεσαίος τόνος δέρματος',
    '👨🏾‍🌾' => 'αγρότης: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🌾' => 'αγρότης: σκούρος τόνος δέρματος',
    '👩🏻‍🌾' => 'αγρότισσα: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🌾' => 'αγρότισσα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🌾' => 'αγρότισσα: μεσαίος τόνος δέρματος',
    '👩🏾‍🌾' => 'αγρότισσα: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🌾' => 'αγρότισσα: σκούρος τόνος δέρματος',
    '🧑🏻‍🍳' => 'σεφ: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🍳' => 'σεφ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🍳' => 'σεφ: μεσαίος τόνος δέρματος',
    '🧑🏾‍🍳' => 'σεφ: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🍳' => 'σεφ: σκούρος τόνος δέρματος',
    '👨🏻‍🍳' => 'μάγειρας: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🍳' => 'μάγειρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🍳' => 'μάγειρας: μεσαίος τόνος δέρματος',
    '👨🏾‍🍳' => 'μάγειρας: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🍳' => 'μάγειρας: σκούρος τόνος δέρματος',
    '👩🏻‍🍳' => 'μαγείρισσα: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🍳' => 'μαγείρισσα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🍳' => 'μαγείρισσα: μεσαίος τόνος δέρματος',
    '👩🏾‍🍳' => 'μαγείρισσα: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🍳' => 'μαγείρισσα: σκούρος τόνος δέρματος',
    '🧑🏻‍🔧' => 'μηχανικός: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🔧' => 'μηχανικός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🔧' => 'μηχανικός: μεσαίος τόνος δέρματος',
    '🧑🏾‍🔧' => 'μηχανικός: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🔧' => 'μηχανικός: σκούρος τόνος δέρματος',
    '👨🏻‍🔧' => 'άντρας μηχανικός: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🔧' => 'άντρας μηχανικός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🔧' => 'άντρας μηχανικός: μεσαίος τόνος δέρματος',
    '👨🏾‍🔧' => 'άντρας μηχανικός: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🔧' => 'άντρας μηχανικός: σκούρος τόνος δέρματος',
    '👩🏻‍🔧' => 'γυναίκα μηχανικός: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🔧' => 'γυναίκα μηχανικός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🔧' => 'γυναίκα μηχανικός: μεσαίος τόνος δέρματος',
    '👩🏾‍🔧' => 'γυναίκα μηχανικός: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🔧' => 'γυναίκα μηχανικός: σκούρος τόνος δέρματος',
    '🧑🏻‍🏭' => 'εργάτης εργοστασίου: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🏭' => 'εργάτης εργοστασίου: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🏭' => 'εργάτης εργοστασίου: μεσαίος τόνος δέρματος',
    '🧑🏾‍🏭' => 'εργάτης εργοστασίου: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🏭' => 'εργάτης εργοστασίου: σκούρος τόνος δέρματος',
    '👨🏻‍🏭' => 'εργάτης: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🏭' => 'εργάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🏭' => 'εργάτης: μεσαίος τόνος δέρματος',
    '👨🏾‍🏭' => 'εργάτης: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🏭' => 'εργάτης: σκούρος τόνος δέρματος',
    '👩🏻‍🏭' => 'εργάτρια: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🏭' => 'εργάτρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🏭' => 'εργάτρια: μεσαίος τόνος δέρματος',
    '👩🏾‍🏭' => 'εργάτρια: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🏭' => 'εργάτρια: σκούρος τόνος δέρματος',
    '🧑🏻‍💼' => 'υπάλληλος γραφείου: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍💼' => 'υπάλληλος γραφείου: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍💼' => 'υπάλληλος γραφείου: μεσαίος τόνος δέρματος',
    '🧑🏾‍💼' => 'υπάλληλος γραφείου: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍💼' => 'υπάλληλος γραφείου: σκούρος τόνος δέρματος',
    '👨🏻‍💼' => 'άντρας υπάλληλος γραφείου: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍💼' => 'άντρας υπάλληλος γραφείου: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍💼' => 'άντρας υπάλληλος γραφείου: μεσαίος τόνος δέρματος',
    '👨🏾‍💼' => 'άντρας υπάλληλος γραφείου: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍💼' => 'άντρας υπάλληλος γραφείου: σκούρος τόνος δέρματος',
    '👩🏻‍💼' => 'γυναίκα υπάλληλος γραφείου: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍💼' => 'γυναίκα υπάλληλος γραφείου: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍💼' => 'γυναίκα υπάλληλος γραφείου: μεσαίος τόνος δέρματος',
    '👩🏾‍💼' => 'γυναίκα υπάλληλος γραφείου: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍💼' => 'γυναίκα υπάλληλος γραφείου: σκούρος τόνος δέρματος',
    '🧑🏻‍🔬' => 'επιστήμονας: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🔬' => 'επιστήμονας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🔬' => 'επιστήμονας: μεσαίος τόνος δέρματος',
    '🧑🏾‍🔬' => 'επιστήμονας: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🔬' => 'επιστήμονας: σκούρος τόνος δέρματος',
    '👨🏻‍🔬' => 'άντρας επιστήμονας: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🔬' => 'άντρας επιστήμονας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🔬' => 'άντρας επιστήμονας: μεσαίος τόνος δέρματος',
    '👨🏾‍🔬' => 'άντρας επιστήμονας: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🔬' => 'άντρας επιστήμονας: σκούρος τόνος δέρματος',
    '👩🏻‍🔬' => 'γυναίκα επιστήμονας: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🔬' => 'γυναίκα επιστήμονας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🔬' => 'γυναίκα επιστήμονας: μεσαίος τόνος δέρματος',
    '👩🏾‍🔬' => 'γυναίκα επιστήμονας: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🔬' => 'γυναίκα επιστήμονας: σκούρος τόνος δέρματος',
    '🧑🏻‍💻' => 'τεχνολόγος: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍💻' => 'τεχνολόγος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍💻' => 'τεχνολόγος: μεσαίος τόνος δέρματος',
    '🧑🏾‍💻' => 'τεχνολόγος: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍💻' => 'τεχνολόγος: σκούρος τόνος δέρματος',
    '👨🏻‍💻' => 'άντρας τεχνολόγος: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍💻' => 'άντρας τεχνολόγος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍💻' => 'άντρας τεχνολόγος: μεσαίος τόνος δέρματος',
    '👨🏾‍💻' => 'άντρας τεχνολόγος: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍💻' => 'άντρας τεχνολόγος: σκούρος τόνος δέρματος',
    '👩🏻‍💻' => 'γυναίκα τεχνολόγος: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍💻' => 'γυναίκα τεχνολόγος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍💻' => 'γυναίκα τεχνολόγος: μεσαίος τόνος δέρματος',
    '👩🏾‍💻' => 'γυναίκα τεχνολόγος: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍💻' => 'γυναίκα τεχνολόγος: σκούρος τόνος δέρματος',
    '🧑🏻‍🎤' => 'ερμηνευτής: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🎤' => 'ερμηνευτής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🎤' => 'ερμηνευτής: μεσαίος τόνος δέρματος',
    '🧑🏾‍🎤' => 'ερμηνευτής: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🎤' => 'ερμηνευτής: σκούρος τόνος δέρματος',
    '👨🏻‍🎤' => 'τραγουδιστής: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🎤' => 'τραγουδιστής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🎤' => 'τραγουδιστής: μεσαίος τόνος δέρματος',
    '👨🏾‍🎤' => 'τραγουδιστής: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🎤' => 'τραγουδιστής: σκούρος τόνος δέρματος',
    '👩🏻‍🎤' => 'τραγουδίστρια: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🎤' => 'τραγουδίστρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🎤' => 'τραγουδίστρια: μεσαίος τόνος δέρματος',
    '👩🏾‍🎤' => 'τραγουδίστρια: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🎤' => 'τραγουδίστρια: σκούρος τόνος δέρματος',
    '🧑🏻‍🎨' => 'καλλιτέχνης: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🎨' => 'καλλιτέχνης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🎨' => 'καλλιτέχνης: μεσαίος τόνος δέρματος',
    '🧑🏾‍🎨' => 'καλλιτέχνης: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🎨' => 'καλλιτέχνης: σκούρος τόνος δέρματος',
    '👨🏻‍🎨' => 'άντρας καλλιτέχνης: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🎨' => 'άντρας καλλιτέχνης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🎨' => 'άντρας καλλιτέχνης: μεσαίος τόνος δέρματος',
    '👨🏾‍🎨' => 'άντρας καλλιτέχνης: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🎨' => 'άντρας καλλιτέχνης: σκούρος τόνος δέρματος',
    '👩🏻‍🎨' => 'γυναίκα καλλιτέχνης: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🎨' => 'γυναίκα καλλιτέχνης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🎨' => 'γυναίκα καλλιτέχνης: μεσαίος τόνος δέρματος',
    '👩🏾‍🎨' => 'γυναίκα καλλιτέχνης: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🎨' => 'γυναίκα καλλιτέχνης: σκούρος τόνος δέρματος',
    '🧑🏻‍✈' => 'πιλότος: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍✈' => 'πιλότος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍✈' => 'πιλότος: μεσαίος τόνος δέρματος',
    '🧑🏾‍✈' => 'πιλότος: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍✈' => 'πιλότος: σκούρος τόνος δέρματος',
    '👨🏻‍✈' => 'άντρας πιλότος: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍✈' => 'άντρας πιλότος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍✈' => 'άντρας πιλότος: μεσαίος τόνος δέρματος',
    '👨🏾‍✈' => 'άντρας πιλότος: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍✈' => 'άντρας πιλότος: σκούρος τόνος δέρματος',
    '👩🏻‍✈' => 'γυναίκα πιλότος: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍✈' => 'γυναίκα πιλότος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍✈' => 'γυναίκα πιλότος: μεσαίος τόνος δέρματος',
    '👩🏾‍✈' => 'γυναίκα πιλότος: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍✈' => 'γυναίκα πιλότος: σκούρος τόνος δέρματος',
    '🧑🏻‍🚀' => 'αστροναύτης: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🚀' => 'αστροναύτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🚀' => 'αστροναύτης: μεσαίος τόνος δέρματος',
    '🧑🏾‍🚀' => 'αστροναύτης: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🚀' => 'αστροναύτης: σκούρος τόνος δέρματος',
    '👨🏻‍🚀' => 'άντρας αστροναύτης: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🚀' => 'άντρας αστροναύτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🚀' => 'άντρας αστροναύτης: μεσαίος τόνος δέρματος',
    '👨🏾‍🚀' => 'άντρας αστροναύτης: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🚀' => 'άντρας αστροναύτης: σκούρος τόνος δέρματος',
    '👩🏻‍🚀' => 'γυναίκα αστροναύτης: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🚀' => 'γυναίκα αστροναύτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🚀' => 'γυναίκα αστροναύτης: μεσαίος τόνος δέρματος',
    '👩🏾‍🚀' => 'γυναίκα αστροναύτης: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🚀' => 'γυναίκα αστροναύτης: σκούρος τόνος δέρματος',
    '🧑🏻‍🚒' => 'πυροσβέστης: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🚒' => 'πυροσβέστης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🚒' => 'πυροσβέστης: μεσαίος τόνος δέρματος',
    '🧑🏾‍🚒' => 'πυροσβέστης: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🚒' => 'πυροσβέστης: σκούρος τόνος δέρματος',
    '👨🏻‍🚒' => 'άντρας πυροσβέστης: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🚒' => 'άντρας πυροσβέστης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🚒' => 'άντρας πυροσβέστης: μεσαίος τόνος δέρματος',
    '👨🏾‍🚒' => 'άντρας πυροσβέστης: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🚒' => 'άντρας πυροσβέστης: σκούρος τόνος δέρματος',
    '👩🏻‍🚒' => 'γυναίκα πυροσβέστης: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🚒' => 'γυναίκα πυροσβέστης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🚒' => 'γυναίκα πυροσβέστης: μεσαίος τόνος δέρματος',
    '👩🏾‍🚒' => 'γυναίκα πυροσβέστης: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🚒' => 'γυναίκα πυροσβέστης: σκούρος τόνος δέρματος',
    '👮🏻‍♂' => 'άντρας αστυνομικός: ανοιχτόχρωμος τόνος δέρματος',
    '👮🏼‍♂' => 'άντρας αστυνομικός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👮🏽‍♂' => 'άντρας αστυνομικός: μεσαίος τόνος δέρματος',
    '👮🏾‍♂' => 'άντρας αστυνομικός: μεσαίος-σκούρος τόνος δέρματος',
    '👮🏿‍♂' => 'άντρας αστυνομικός: σκούρος τόνος δέρματος',
    '👮🏻‍♀' => 'γυναίκα αστυνομικός: ανοιχτόχρωμος τόνος δέρματος',
    '👮🏼‍♀' => 'γυναίκα αστυνομικός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👮🏽‍♀' => 'γυναίκα αστυνομικός: μεσαίος τόνος δέρματος',
    '👮🏾‍♀' => 'γυναίκα αστυνομικός: μεσαίος-σκούρος τόνος δέρματος',
    '👮🏿‍♀' => 'γυναίκα αστυνομικός: σκούρος τόνος δέρματος',
    '🕵🏻‍♂' => 'άντρας ντετέκτιβ: ανοιχτόχρωμος τόνος δέρματος',
    '🕵🏼‍♂' => 'άντρας ντετέκτιβ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🕵🏽‍♂' => 'άντρας ντετέκτιβ: μεσαίος τόνος δέρματος',
    '🕵🏾‍♂' => 'άντρας ντετέκτιβ: μεσαίος-σκούρος τόνος δέρματος',
    '🕵🏿‍♂' => 'άντρας ντετέκτιβ: σκούρος τόνος δέρματος',
    '🕵🏻‍♀' => 'γυναίκα ντετέκτιβ: ανοιχτόχρωμος τόνος δέρματος',
    '🕵🏼‍♀' => 'γυναίκα ντετέκτιβ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🕵🏽‍♀' => 'γυναίκα ντετέκτιβ: μεσαίος τόνος δέρματος',
    '🕵🏾‍♀' => 'γυναίκα ντετέκτιβ: μεσαίος-σκούρος τόνος δέρματος',
    '🕵🏿‍♀' => 'γυναίκα ντετέκτιβ: σκούρος τόνος δέρματος',
    '💂🏻‍♂' => 'άντρας φρουρός: ανοιχτόχρωμος τόνος δέρματος',
    '💂🏼‍♂' => 'άντρας φρουρός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💂🏽‍♂' => 'άντρας φρουρός: μεσαίος τόνος δέρματος',
    '💂🏾‍♂' => 'άντρας φρουρός: μεσαίος-σκούρος τόνος δέρματος',
    '💂🏿‍♂' => 'άντρας φρουρός: σκούρος τόνος δέρματος',
    '💂🏻‍♀' => 'γυναίκα φρουρός: ανοιχτόχρωμος τόνος δέρματος',
    '💂🏼‍♀' => 'γυναίκα φρουρός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💂🏽‍♀' => 'γυναίκα φρουρός: μεσαίος τόνος δέρματος',
    '💂🏾‍♀' => 'γυναίκα φρουρός: μεσαίος-σκούρος τόνος δέρματος',
    '💂🏿‍♀' => 'γυναίκα φρουρός: σκούρος τόνος δέρματος',
    '👷🏻‍♂' => 'εργάτης οικοδομής: ανοιχτόχρωμος τόνος δέρματος',
    '👷🏼‍♂' => 'εργάτης οικοδομής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👷🏽‍♂' => 'εργάτης οικοδομής: μεσαίος τόνος δέρματος',
    '👷🏾‍♂' => 'εργάτης οικοδομής: μεσαίος-σκούρος τόνος δέρματος',
    '👷🏿‍♂' => 'εργάτης οικοδομής: σκούρος τόνος δέρματος',
    '👷🏻‍♀' => 'εργάτρια οικοδομής: ανοιχτόχρωμος τόνος δέρματος',
    '👷🏼‍♀' => 'εργάτρια οικοδομής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👷🏽‍♀' => 'εργάτρια οικοδομής: μεσαίος τόνος δέρματος',
    '👷🏾‍♀' => 'εργάτρια οικοδομής: μεσαίος-σκούρος τόνος δέρματος',
    '👷🏿‍♀' => 'εργάτρια οικοδομής: σκούρος τόνος δέρματος',
    '👳🏻‍♂' => 'άντρας με τουρμπάνι: ανοιχτόχρωμος τόνος δέρματος',
    '👳🏼‍♂' => 'άντρας με τουρμπάνι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👳🏽‍♂' => 'άντρας με τουρμπάνι: μεσαίος τόνος δέρματος',
    '👳🏾‍♂' => 'άντρας με τουρμπάνι: μεσαίος-σκούρος τόνος δέρματος',
    '👳🏿‍♂' => 'άντρας με τουρμπάνι: σκούρος τόνος δέρματος',
    '👳🏻‍♀' => 'γυναίκα με τουρμπάνι: ανοιχτόχρωμος τόνος δέρματος',
    '👳🏼‍♀' => 'γυναίκα με τουρμπάνι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👳🏽‍♀' => 'γυναίκα με τουρμπάνι: μεσαίος τόνος δέρματος',
    '👳🏾‍♀' => 'γυναίκα με τουρμπάνι: μεσαίος-σκούρος τόνος δέρματος',
    '👳🏿‍♀' => 'γυναίκα με τουρμπάνι: σκούρος τόνος δέρματος',
    '🤵🏻‍♂' => 'άντρας με σμόκιν: ανοιχτόχρωμος τόνος δέρματος',
    '🤵🏼‍♂' => 'άντρας με σμόκιν: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤵🏽‍♂' => 'άντρας με σμόκιν: μεσαίος τόνος δέρματος',
    '🤵🏾‍♂' => 'άντρας με σμόκιν: μεσαίος-σκούρος τόνος δέρματος',
    '🤵🏿‍♂' => 'άντρας με σμόκιν: σκούρος τόνος δέρματος',
    '🤵🏻‍♀' => 'γυναίκα με σμόκιν: ανοιχτόχρωμος τόνος δέρματος',
    '🤵🏼‍♀' => 'γυναίκα με σμόκιν: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤵🏽‍♀' => 'γυναίκα με σμόκιν: μεσαίος τόνος δέρματος',
    '🤵🏾‍♀' => 'γυναίκα με σμόκιν: μεσαίος-σκούρος τόνος δέρματος',
    '🤵🏿‍♀' => 'γυναίκα με σμόκιν: σκούρος τόνος δέρματος',
    '👰🏻‍♂' => 'άντρας με πέπλο: ανοιχτόχρωμος τόνος δέρματος',
    '👰🏼‍♂' => 'άντρας με πέπλο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👰🏽‍♂' => 'άντρας με πέπλο: μεσαίος τόνος δέρματος',
    '👰🏾‍♂' => 'άντρας με πέπλο: μεσαίος-σκούρος τόνος δέρματος',
    '👰🏿‍♂' => 'άντρας με πέπλο: σκούρος τόνος δέρματος',
    '👰🏻‍♀' => 'γυναίκα με πέπλο: ανοιχτόχρωμος τόνος δέρματος',
    '👰🏼‍♀' => 'γυναίκα με πέπλο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👰🏽‍♀' => 'γυναίκα με πέπλο: μεσαίος τόνος δέρματος',
    '👰🏾‍♀' => 'γυναίκα με πέπλο: μεσαίος-σκούρος τόνος δέρματος',
    '👰🏿‍♀' => 'γυναίκα με πέπλο: σκούρος τόνος δέρματος',
    '👩🏻‍🍼' => 'γυναίκα που ταΐζει μωρό: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🍼' => 'γυναίκα που ταΐζει μωρό: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🍼' => 'γυναίκα που ταΐζει μωρό: μεσαίος τόνος δέρματος',
    '👩🏾‍🍼' => 'γυναίκα που ταΐζει μωρό: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🍼' => 'γυναίκα που ταΐζει μωρό: σκούρος τόνος δέρματος',
    '👨🏻‍🍼' => 'άντρας που ταΐζει μωρό: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🍼' => 'άντρας που ταΐζει μωρό: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🍼' => 'άντρας που ταΐζει μωρό: μεσαίος τόνος δέρματος',
    '👨🏾‍🍼' => 'άντρας που ταΐζει μωρό: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🍼' => 'άντρας που ταΐζει μωρό: σκούρος τόνος δέρματος',
    '🧑🏻‍🍼' => 'άτομο που ταΐζει μωρό: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🍼' => 'άτομο που ταΐζει μωρό: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🍼' => 'άτομο που ταΐζει μωρό: μεσαίος τόνος δέρματος',
    '🧑🏾‍🍼' => 'άτομο που ταΐζει μωρό: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🍼' => 'άτομο που ταΐζει μωρό: σκούρος τόνος δέρματος',
    '🧑🏻‍🎄' => 'Άγιος Βασίλης: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🎄' => 'Άγιος Βασίλης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🎄' => 'Άγιος Βασίλης: μεσαίος τόνος δέρματος',
    '🧑🏾‍🎄' => 'Άγιος Βασίλης: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🎄' => 'Άγιος Βασίλης: σκούρος τόνος δέρματος',
    '🦸🏻‍♂' => 'άντρας υπερήρωας: ανοιχτόχρωμος τόνος δέρματος',
    '🦸🏼‍♂' => 'άντρας υπερήρωας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦸🏽‍♂' => 'άντρας υπερήρωας: μεσαίος τόνος δέρματος',
    '🦸🏾‍♂' => 'άντρας υπερήρωας: μεσαίος-σκούρος τόνος δέρματος',
    '🦸🏿‍♂' => 'άντρας υπερήρωας: σκούρος τόνος δέρματος',
    '🦸🏻‍♀' => 'γυναίκα υπερήρωας: ανοιχτόχρωμος τόνος δέρματος',
    '🦸🏼‍♀' => 'γυναίκα υπερήρωας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦸🏽‍♀' => 'γυναίκα υπερήρωας: μεσαίος τόνος δέρματος',
    '🦸🏾‍♀' => 'γυναίκα υπερήρωας: μεσαίος-σκούρος τόνος δέρματος',
    '🦸🏿‍♀' => 'γυναίκα υπερήρωας: σκούρος τόνος δέρματος',
    '🦹🏻‍♂' => 'άντρας αρχικακός: ανοιχτόχρωμος τόνος δέρματος',
    '🦹🏼‍♂' => 'άντρας αρχικακός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦹🏽‍♂' => 'άντρας αρχικακός: μεσαίος τόνος δέρματος',
    '🦹🏾‍♂' => 'άντρας αρχικακός: μεσαίος-σκούρος τόνος δέρματος',
    '🦹🏿‍♂' => 'άντρας αρχικακός: σκούρος τόνος δέρματος',
    '🦹🏻‍♀' => 'γυναίκα αρχικακός: ανοιχτόχρωμος τόνος δέρματος',
    '🦹🏼‍♀' => 'γυναίκα αρχικακός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦹🏽‍♀' => 'γυναίκα αρχικακός: μεσαίος τόνος δέρματος',
    '🦹🏾‍♀' => 'γυναίκα αρχικακός: μεσαίος-σκούρος τόνος δέρματος',
    '🦹🏿‍♀' => 'γυναίκα αρχικακός: σκούρος τόνος δέρματος',
    '🧙🏻‍♂' => 'άντρας μάγος: ανοιχτόχρωμος τόνος δέρματος',
    '🧙🏼‍♂' => 'άντρας μάγος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧙🏽‍♂' => 'άντρας μάγος: μεσαίος τόνος δέρματος',
    '🧙🏾‍♂' => 'άντρας μάγος: μεσαίος-σκούρος τόνος δέρματος',
    '🧙🏿‍♂' => 'άντρας μάγος: σκούρος τόνος δέρματος',
    '🧙🏻‍♀' => 'μάγισσα: ανοιχτόχρωμος τόνος δέρματος',
    '🧙🏼‍♀' => 'μάγισσα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧙🏽‍♀' => 'μάγισσα: μεσαίος τόνος δέρματος',
    '🧙🏾‍♀' => 'μάγισσα: μεσαίος-σκούρος τόνος δέρματος',
    '🧙🏿‍♀' => 'μάγισσα: σκούρος τόνος δέρματος',
    '🧚🏻‍♂' => 'νεράιδος: ανοιχτόχρωμος τόνος δέρματος',
    '🧚🏼‍♂' => 'νεράιδος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧚🏽‍♂' => 'νεράιδος: μεσαίος τόνος δέρματος',
    '🧚🏾‍♂' => 'νεράιδος: μεσαίος-σκούρος τόνος δέρματος',
    '🧚🏿‍♂' => 'νεράιδος: σκούρος τόνος δέρματος',
    '🧚🏻‍♀' => 'γυναίκα νεράιδα: ανοιχτόχρωμος τόνος δέρματος',
    '🧚🏼‍♀' => 'γυναίκα νεράιδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧚🏽‍♀' => 'γυναίκα νεράιδα: μεσαίος τόνος δέρματος',
    '🧚🏾‍♀' => 'γυναίκα νεράιδα: μεσαίος-σκούρος τόνος δέρματος',
    '🧚🏿‍♀' => 'γυναίκα νεράιδα: σκούρος τόνος δέρματος',
    '🧛🏻‍♂' => 'άντρας βρυκόλακας: ανοιχτόχρωμος τόνος δέρματος',
    '🧛🏼‍♂' => 'άντρας βρυκόλακας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧛🏽‍♂' => 'άντρας βρυκόλακας: μεσαίος τόνος δέρματος',
    '🧛🏾‍♂' => 'άντρας βρυκόλακας: μεσαίος-σκούρος τόνος δέρματος',
    '🧛🏿‍♂' => 'άντρας βρυκόλακας: σκούρος τόνος δέρματος',
    '🧛🏻‍♀' => 'γυναίκα βρυκόλακας: ανοιχτόχρωμος τόνος δέρματος',
    '🧛🏼‍♀' => 'γυναίκα βρυκόλακας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧛🏽‍♀' => 'γυναίκα βρυκόλακας: μεσαίος τόνος δέρματος',
    '🧛🏾‍♀' => 'γυναίκα βρυκόλακας: μεσαίος-σκούρος τόνος δέρματος',
    '🧛🏿‍♀' => 'γυναίκα βρυκόλακας: σκούρος τόνος δέρματος',
    '🧜🏻‍♂' => 'γοργόνος: ανοιχτόχρωμος τόνος δέρματος',
    '🧜🏼‍♂' => 'γοργόνος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧜🏽‍♂' => 'γοργόνος: μεσαίος τόνος δέρματος',
    '🧜🏾‍♂' => 'γοργόνος: μεσαίος-σκούρος τόνος δέρματος',
    '🧜🏿‍♂' => 'γοργόνος: σκούρος τόνος δέρματος',
    '🧜🏻‍♀' => 'γυναίκα γοργόνα: ανοιχτόχρωμος τόνος δέρματος',
    '🧜🏼‍♀' => 'γυναίκα γοργόνα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧜🏽‍♀' => 'γυναίκα γοργόνα: μεσαίος τόνος δέρματος',
    '🧜🏾‍♀' => 'γυναίκα γοργόνα: μεσαίος-σκούρος τόνος δέρματος',
    '🧜🏿‍♀' => 'γυναίκα γοργόνα: σκούρος τόνος δέρματος',
    '🧝🏻‍♂' => 'άντρας ξωτικό: ανοιχτόχρωμος τόνος δέρματος',
    '🧝🏼‍♂' => 'άντρας ξωτικό: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧝🏽‍♂' => 'άντρας ξωτικό: μεσαίος τόνος δέρματος',
    '🧝🏾‍♂' => 'άντρας ξωτικό: μεσαίος-σκούρος τόνος δέρματος',
    '🧝🏿‍♂' => 'άντρας ξωτικό: σκούρος τόνος δέρματος',
    '🧝🏻‍♀' => 'γυναίκα ξωτικό: ανοιχτόχρωμος τόνος δέρματος',
    '🧝🏼‍♀' => 'γυναίκα ξωτικό: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧝🏽‍♀' => 'γυναίκα ξωτικό: μεσαίος τόνος δέρματος',
    '🧝🏾‍♀' => 'γυναίκα ξωτικό: μεσαίος-σκούρος τόνος δέρματος',
    '🧝🏿‍♀' => 'γυναίκα ξωτικό: σκούρος τόνος δέρματος',
    '💆🏻‍♂' => 'μασάζ στο κεφάλι άντρα: ανοιχτόχρωμος τόνος δέρματος',
    '💆🏼‍♂' => 'μασάζ στο κεφάλι άντρα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💆🏽‍♂' => 'μασάζ στο κεφάλι άντρα: μεσαίος τόνος δέρματος',
    '💆🏾‍♂' => 'μασάζ στο κεφάλι άντρα: μεσαίος-σκούρος τόνος δέρματος',
    '💆🏿‍♂' => 'μασάζ στο κεφάλι άντρα: σκούρος τόνος δέρματος',
    '💆🏻‍♀' => 'μασάζ στο κεφάλι γυναίκας: ανοιχτόχρωμος τόνος δέρματος',
    '💆🏼‍♀' => 'μασάζ στο κεφάλι γυναίκας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💆🏽‍♀' => 'μασάζ στο κεφάλι γυναίκας: μεσαίος τόνος δέρματος',
    '💆🏾‍♀' => 'μασάζ στο κεφάλι γυναίκας: μεσαίος-σκούρος τόνος δέρματος',
    '💆🏿‍♀' => 'μασάζ στο κεφάλι γυναίκας: σκούρος τόνος δέρματος',
    '💇🏻‍♂' => 'άντρας κουρεύεται: ανοιχτόχρωμος τόνος δέρματος',
    '💇🏼‍♂' => 'άντρας κουρεύεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💇🏽‍♂' => 'άντρας κουρεύεται: μεσαίος τόνος δέρματος',
    '💇🏾‍♂' => 'άντρας κουρεύεται: μεσαίος-σκούρος τόνος δέρματος',
    '💇🏿‍♂' => 'άντρας κουρεύεται: σκούρος τόνος δέρματος',
    '💇🏻‍♀' => 'γυναίκα κουρεύεται: ανοιχτόχρωμος τόνος δέρματος',
    '💇🏼‍♀' => 'γυναίκα κουρεύεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💇🏽‍♀' => 'γυναίκα κουρεύεται: μεσαίος τόνος δέρματος',
    '💇🏾‍♀' => 'γυναίκα κουρεύεται: μεσαίος-σκούρος τόνος δέρματος',
    '💇🏿‍♀' => 'γυναίκα κουρεύεται: σκούρος τόνος δέρματος',
    '🚶🏻‍♂' => 'άντρας περπατά: ανοιχτόχρωμος τόνος δέρματος',
    '🚶🏼‍♂' => 'άντρας περπατά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚶🏽‍♂' => 'άντρας περπατά: μεσαίος τόνος δέρματος',
    '🚶🏾‍♂' => 'άντρας περπατά: μεσαίος-σκούρος τόνος δέρματος',
    '🚶🏿‍♂' => 'άντρας περπατά: σκούρος τόνος δέρματος',
    '🚶🏻‍♀' => 'γυναίκα περπατά: ανοιχτόχρωμος τόνος δέρματος',
    '🚶🏼‍♀' => 'γυναίκα περπατά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚶🏽‍♀' => 'γυναίκα περπατά: μεσαίος τόνος δέρματος',
    '🚶🏾‍♀' => 'γυναίκα περπατά: μεσαίος-σκούρος τόνος δέρματος',
    '🚶🏿‍♀' => 'γυναίκα περπατά: σκούρος τόνος δέρματος',
    '🚶🏻‍➡' => 'πεζός: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏼‍➡' => 'πεζός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏽‍➡' => 'πεζός: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏾‍➡' => 'πεζός: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🚶🏿‍➡' => 'πεζός: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧍🏻‍♂' => 'άντρας που στέκεται: ανοιχτόχρωμος τόνος δέρματος',
    '🧍🏼‍♂' => 'άντρας που στέκεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧍🏽‍♂' => 'άντρας που στέκεται: μεσαίος τόνος δέρματος',
    '🧍🏾‍♂' => 'άντρας που στέκεται: μεσαίος-σκούρος τόνος δέρματος',
    '🧍🏿‍♂' => 'άντρας που στέκεται: σκούρος τόνος δέρματος',
    '🧍🏻‍♀' => 'γυναίκα που στέκεται: ανοιχτόχρωμος τόνος δέρματος',
    '🧍🏼‍♀' => 'γυναίκα που στέκεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧍🏽‍♀' => 'γυναίκα που στέκεται: μεσαίος τόνος δέρματος',
    '🧍🏾‍♀' => 'γυναίκα που στέκεται: μεσαίος-σκούρος τόνος δέρματος',
    '🧍🏿‍♀' => 'γυναίκα που στέκεται: σκούρος τόνος δέρματος',
    '🧎🏻‍♂' => 'άντρας που γονατίζει: ανοιχτόχρωμος τόνος δέρματος',
    '🧎🏼‍♂' => 'άντρας που γονατίζει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧎🏽‍♂' => 'άντρας που γονατίζει: μεσαίος τόνος δέρματος',
    '🧎🏾‍♂' => 'άντρας που γονατίζει: μεσαίος-σκούρος τόνος δέρματος',
    '🧎🏿‍♂' => 'άντρας που γονατίζει: σκούρος τόνος δέρματος',
    '🧎🏻‍♀' => 'γυναίκα που γονατίζει: ανοιχτόχρωμος τόνος δέρματος',
    '🧎🏼‍♀' => 'γυναίκα που γονατίζει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧎🏽‍♀' => 'γυναίκα που γονατίζει: μεσαίος τόνος δέρματος',
    '🧎🏾‍♀' => 'γυναίκα που γονατίζει: μεσαίος-σκούρος τόνος δέρματος',
    '🧎🏿‍♀' => 'γυναίκα που γονατίζει: σκούρος τόνος δέρματος',
    '🧎🏻‍➡' => 'άτομο που γονατίζει: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏼‍➡' => 'άτομο που γονατίζει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏽‍➡' => 'άτομο που γονατίζει: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏾‍➡' => 'άτομο που γονατίζει: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧎🏿‍➡' => 'άτομο που γονατίζει: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🧑🏻‍🦯' => 'άτομο με μπαστούνι για τυφλούς: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🦯' => 'άτομο με μπαστούνι για τυφλούς: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🦯' => 'άτομο με μπαστούνι για τυφλούς: μεσαίος τόνος δέρματος',
    '🧑🏾‍🦯' => 'άτομο με μπαστούνι για τυφλούς: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🦯' => 'άτομο με μπαστούνι για τυφλούς: σκούρος τόνος δέρματος',
    '👨🏻‍🦯' => 'άντρας με μπαστούνι για τυφλούς: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🦯' => 'άντρας με μπαστούνι για τυφλούς: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🦯' => 'άντρας με μπαστούνι για τυφλούς: μεσαίος τόνος δέρματος',
    '👨🏾‍🦯' => 'άντρας με μπαστούνι για τυφλούς: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🦯' => 'άντρας με μπαστούνι για τυφλούς: σκούρος τόνος δέρματος',
    '👩🏻‍🦯' => 'γυναίκα με μπαστούνι για τυφλούς: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🦯' => 'γυναίκα με μπαστούνι για τυφλούς: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🦯' => 'γυναίκα με μπαστούνι για τυφλούς: μεσαίος τόνος δέρματος',
    '👩🏾‍🦯' => 'γυναίκα με μπαστούνι για τυφλούς: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🦯' => 'γυναίκα με μπαστούνι για τυφλούς: σκούρος τόνος δέρματος',
    '🧑🏻‍🦼' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🦼' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🦼' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος',
    '🧑🏾‍🦼' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🦼' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος',
    '👨🏻‍🦼' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🦼' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🦼' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος',
    '👨🏾‍🦼' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🦼' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος',
    '👩🏻‍🦼' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🦼' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🦼' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος',
    '👩🏾‍🦼' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🦼' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος',
    '🧑🏻‍🦽' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼‍🦽' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽‍🦽' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος',
    '🧑🏾‍🦽' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿‍🦽' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος',
    '👨🏻‍🦽' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼‍🦽' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽‍🦽' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος',
    '👨🏾‍🦽' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿‍🦽' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος',
    '👩🏻‍🦽' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼‍🦽' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽‍🦽' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος τόνος δέρματος',
    '👩🏾‍🦽' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿‍🦽' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο: σκούρος τόνος δέρματος',
    '🏃🏻‍♂' => 'άντρας τρέχει: ανοιχτόχρωμος τόνος δέρματος',
    '🏃🏼‍♂' => 'άντρας τρέχει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏃🏽‍♂' => 'άντρας τρέχει: μεσαίος τόνος δέρματος',
    '🏃🏾‍♂' => 'άντρας τρέχει: μεσαίος-σκούρος τόνος δέρματος',
    '🏃🏿‍♂' => 'άντρας τρέχει: σκούρος τόνος δέρματος',
    '🏃🏻‍♀' => 'γυναίκα τρέχει: ανοιχτόχρωμος τόνος δέρματος',
    '🏃🏼‍♀' => 'γυναίκα τρέχει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏃🏽‍♀' => 'γυναίκα τρέχει: μεσαίος τόνος δέρματος',
    '🏃🏾‍♀' => 'γυναίκα τρέχει: μεσαίος-σκούρος τόνος δέρματος',
    '🏃🏿‍♀' => 'γυναίκα τρέχει: σκούρος τόνος δέρματος',
    '🏃🏻‍➡' => 'άτομο τρέχει: ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏼‍➡' => 'άτομο τρέχει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏽‍➡' => 'άτομο τρέχει: μεσαίος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏾‍➡' => 'άτομο τρέχει: μεσαίος-σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '🏃🏿‍➡' => 'άτομο τρέχει: σκούρος τόνος δέρματος, με κατεύθυνση προς τα δεξιά',
    '👯🏻‍♀' => 'γυναίκες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος',
    '👯🏼‍♀' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👯🏽‍♀' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος',
    '👯🏾‍♀' => 'γυναίκες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος',
    '👯🏿‍♀' => 'γυναίκες με λαγουδένια αυτιά: σκούρος τόνος δέρματος',
    '👯🏻‍♂' => 'άντρες με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος',
    '👯🏼‍♂' => 'άντρες με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👯🏽‍♂' => 'άντρες με λαγουδένια αυτιά: μεσαίος τόνος δέρματος',
    '👯🏾‍♂' => 'άντρες με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος',
    '👯🏿‍♂' => 'άντρες με λαγουδένια αυτιά: σκούρος τόνος δέρματος',
    '🧖🏻‍♂' => 'άντρας σε σάουνα: ανοιχτόχρωμος τόνος δέρματος',
    '🧖🏼‍♂' => 'άντρας σε σάουνα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧖🏽‍♂' => 'άντρας σε σάουνα: μεσαίος τόνος δέρματος',
    '🧖🏾‍♂' => 'άντρας σε σάουνα: μεσαίος-σκούρος τόνος δέρματος',
    '🧖🏿‍♂' => 'άντρας σε σάουνα: σκούρος τόνος δέρματος',
    '🧖🏻‍♀' => 'γυναίκα σε σάουνα: ανοιχτόχρωμος τόνος δέρματος',
    '🧖🏼‍♀' => 'γυναίκα σε σάουνα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧖🏽‍♀' => 'γυναίκα σε σάουνα: μεσαίος τόνος δέρματος',
    '🧖🏾‍♀' => 'γυναίκα σε σάουνα: μεσαίος-σκούρος τόνος δέρματος',
    '🧖🏿‍♀' => 'γυναίκα σε σάουνα: σκούρος τόνος δέρματος',
    '🧗🏻‍♂' => 'άντρας ορειβάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🧗🏼‍♂' => 'άντρας ορειβάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧗🏽‍♂' => 'άντρας ορειβάτης: μεσαίος τόνος δέρματος',
    '🧗🏾‍♂' => 'άντρας ορειβάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🧗🏿‍♂' => 'άντρας ορειβάτης: σκούρος τόνος δέρματος',
    '🧗🏻‍♀' => 'γυναίκα ορειβάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🧗🏼‍♀' => 'γυναίκα ορειβάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧗🏽‍♀' => 'γυναίκα ορειβάτης: μεσαίος τόνος δέρματος',
    '🧗🏾‍♀' => 'γυναίκα ορειβάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🧗🏿‍♀' => 'γυναίκα ορειβάτης: σκούρος τόνος δέρματος',
    '🏌🏻‍♂' => 'άντρας παίζει γκολφ: ανοιχτόχρωμος τόνος δέρματος',
    '🏌🏼‍♂' => 'άντρας παίζει γκολφ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏌🏽‍♂' => 'άντρας παίζει γκολφ: μεσαίος τόνος δέρματος',
    '🏌🏾‍♂' => 'άντρας παίζει γκολφ: μεσαίος-σκούρος τόνος δέρματος',
    '🏌🏿‍♂' => 'άντρας παίζει γκολφ: σκούρος τόνος δέρματος',
    '🏌🏻‍♀' => 'γυναίκα παίζει γκολφ: ανοιχτόχρωμος τόνος δέρματος',
    '🏌🏼‍♀' => 'γυναίκα παίζει γκολφ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏌🏽‍♀' => 'γυναίκα παίζει γκολφ: μεσαίος τόνος δέρματος',
    '🏌🏾‍♀' => 'γυναίκα παίζει γκολφ: μεσαίος-σκούρος τόνος δέρματος',
    '🏌🏿‍♀' => 'γυναίκα παίζει γκολφ: σκούρος τόνος δέρματος',
    '🏄🏻‍♂' => 'άντρας σε σανίδα σερφ: ανοιχτόχρωμος τόνος δέρματος',
    '🏄🏼‍♂' => 'άντρας σε σανίδα σερφ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏄🏽‍♂' => 'άντρας σε σανίδα σερφ: μεσαίος τόνος δέρματος',
    '🏄🏾‍♂' => 'άντρας σε σανίδα σερφ: μεσαίος-σκούρος τόνος δέρματος',
    '🏄🏿‍♂' => 'άντρας σε σανίδα σερφ: σκούρος τόνος δέρματος',
    '🏄🏻‍♀' => 'γυναίκα σε σανίδα σερφ: ανοιχτόχρωμος τόνος δέρματος',
    '🏄🏼‍♀' => 'γυναίκα σε σανίδα σερφ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏄🏽‍♀' => 'γυναίκα σε σανίδα σερφ: μεσαίος τόνος δέρματος',
    '🏄🏾‍♀' => 'γυναίκα σε σανίδα σερφ: μεσαίος-σκούρος τόνος δέρματος',
    '🏄🏿‍♀' => 'γυναίκα σε σανίδα σερφ: σκούρος τόνος δέρματος',
    '🚣🏻‍♂' => 'άντρας κωπηλάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🚣🏼‍♂' => 'άντρας κωπηλάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚣🏽‍♂' => 'άντρας κωπηλάτης: μεσαίος τόνος δέρματος',
    '🚣🏾‍♂' => 'άντρας κωπηλάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🚣🏿‍♂' => 'άντρας κωπηλάτης: σκούρος τόνος δέρματος',
    '🚣🏻‍♀' => 'γυναίκα κωπηλάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🚣🏼‍♀' => 'γυναίκα κωπηλάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚣🏽‍♀' => 'γυναίκα κωπηλάτης: μεσαίος τόνος δέρματος',
    '🚣🏾‍♀' => 'γυναίκα κωπηλάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🚣🏿‍♀' => 'γυναίκα κωπηλάτης: σκούρος τόνος δέρματος',
    '🏊🏻‍♂' => 'άντρας κολυμβητής: ανοιχτόχρωμος τόνος δέρματος',
    '🏊🏼‍♂' => 'άντρας κολυμβητής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏊🏽‍♂' => 'άντρας κολυμβητής: μεσαίος τόνος δέρματος',
    '🏊🏾‍♂' => 'άντρας κολυμβητής: μεσαίος-σκούρος τόνος δέρματος',
    '🏊🏿‍♂' => 'άντρας κολυμβητής: σκούρος τόνος δέρματος',
    '🏊🏻‍♀' => 'κολυμβήτρια: ανοιχτόχρωμος τόνος δέρματος',
    '🏊🏼‍♀' => 'κολυμβήτρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏊🏽‍♀' => 'κολυμβήτρια: μεσαίος τόνος δέρματος',
    '🏊🏾‍♀' => 'κολυμβήτρια: μεσαίος-σκούρος τόνος δέρματος',
    '🏊🏿‍♀' => 'κολυμβήτρια: σκούρος τόνος δέρματος',
    '⛹🏻‍♂' => 'άντρας παίζει μπάσκετ: ανοιχτόχρωμος τόνος δέρματος',
    '⛹🏼‍♂' => 'άντρας παίζει μπάσκετ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '⛹🏽‍♂' => 'άντρας παίζει μπάσκετ: μεσαίος τόνος δέρματος',
    '⛹🏾‍♂' => 'άντρας παίζει μπάσκετ: μεσαίος-σκούρος τόνος δέρματος',
    '⛹🏿‍♂' => 'άντρας παίζει μπάσκετ: σκούρος τόνος δέρματος',
    '⛹🏻‍♀' => 'γυναίκα παίζει μπάσκετ: ανοιχτόχρωμος τόνος δέρματος',
    '⛹🏼‍♀' => 'γυναίκα παίζει μπάσκετ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '⛹🏽‍♀' => 'γυναίκα παίζει μπάσκετ: μεσαίος τόνος δέρματος',
    '⛹🏾‍♀' => 'γυναίκα παίζει μπάσκετ: μεσαίος-σκούρος τόνος δέρματος',
    '⛹🏿‍♀' => 'γυναίκα παίζει μπάσκετ: σκούρος τόνος δέρματος',
    '🏋🏻‍♂' => 'άντρας αρσιβαρίστας: ανοιχτόχρωμος τόνος δέρματος',
    '🏋🏼‍♂' => 'άντρας αρσιβαρίστας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏋🏽‍♂' => 'άντρας αρσιβαρίστας: μεσαίος τόνος δέρματος',
    '🏋🏾‍♂' => 'άντρας αρσιβαρίστας: μεσαίος-σκούρος τόνος δέρματος',
    '🏋🏿‍♂' => 'άντρας αρσιβαρίστας: σκούρος τόνος δέρματος',
    '🏋🏻‍♀' => 'αρσιβαρίστρια: ανοιχτόχρωμος τόνος δέρματος',
    '🏋🏼‍♀' => 'αρσιβαρίστρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏋🏽‍♀' => 'αρσιβαρίστρια: μεσαίος τόνος δέρματος',
    '🏋🏾‍♀' => 'αρσιβαρίστρια: μεσαίος-σκούρος τόνος δέρματος',
    '🏋🏿‍♀' => 'αρσιβαρίστρια: σκούρος τόνος δέρματος',
    '🚴🏻‍♂' => 'άντρας ποδηλάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🚴🏼‍♂' => 'άντρας ποδηλάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚴🏽‍♂' => 'άντρας ποδηλάτης: μεσαίος τόνος δέρματος',
    '🚴🏾‍♂' => 'άντρας ποδηλάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🚴🏿‍♂' => 'άντρας ποδηλάτης: σκούρος τόνος δέρματος',
    '🚴🏻‍♀' => 'ποδηλάτισσα: ανοιχτόχρωμος τόνος δέρματος',
    '🚴🏼‍♀' => 'ποδηλάτισσα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚴🏽‍♀' => 'ποδηλάτισσα: μεσαίος τόνος δέρματος',
    '🚴🏾‍♀' => 'ποδηλάτισσα: μεσαίος-σκούρος τόνος δέρματος',
    '🚴🏿‍♀' => 'ποδηλάτισσα: σκούρος τόνος δέρματος',
    '🚵🏻‍♂' => 'ορεινός ποδηλάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🚵🏼‍♂' => 'ορεινός ποδηλάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚵🏽‍♂' => 'ορεινός ποδηλάτης: μεσαίος τόνος δέρματος',
    '🚵🏾‍♂' => 'ορεινός ποδηλάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🚵🏿‍♂' => 'ορεινός ποδηλάτης: σκούρος τόνος δέρματος',
    '🚵🏻‍♀' => 'ορεινή ποδηλάτισσα: ανοιχτόχρωμος τόνος δέρματος',
    '🚵🏼‍♀' => 'ορεινή ποδηλάτισσα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚵🏽‍♀' => 'ορεινή ποδηλάτισσα: μεσαίος τόνος δέρματος',
    '🚵🏾‍♀' => 'ορεινή ποδηλάτισσα: μεσαίος-σκούρος τόνος δέρματος',
    '🚵🏿‍♀' => 'ορεινή ποδηλάτισσα: σκούρος τόνος δέρματος',
    '🤸🏻‍♂' => 'άντρας κάνει ρόδα: ανοιχτόχρωμος τόνος δέρματος',
    '🤸🏼‍♂' => 'άντρας κάνει ρόδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤸🏽‍♂' => 'άντρας κάνει ρόδα: μεσαίος τόνος δέρματος',
    '🤸🏾‍♂' => 'άντρας κάνει ρόδα: μεσαίος-σκούρος τόνος δέρματος',
    '🤸🏿‍♂' => 'άντρας κάνει ρόδα: σκούρος τόνος δέρματος',
    '🤸🏻‍♀' => 'γυναίκα κάνει ρόδα: ανοιχτόχρωμος τόνος δέρματος',
    '🤸🏼‍♀' => 'γυναίκα κάνει ρόδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤸🏽‍♀' => 'γυναίκα κάνει ρόδα: μεσαίος τόνος δέρματος',
    '🤸🏾‍♀' => 'γυναίκα κάνει ρόδα: μεσαίος-σκούρος τόνος δέρματος',
    '🤸🏿‍♀' => 'γυναίκα κάνει ρόδα: σκούρος τόνος δέρματος',
    '🤼🏻‍♀' => 'παλαίστριες: ανοιχτόχρωμος τόνος δέρματος',
    '🤼🏼‍♀' => 'παλαίστριες: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤼🏽‍♀' => 'παλαίστριες: μεσαίος τόνος δέρματος',
    '🤼🏾‍♀' => 'παλαίστριες: μεσαίος-σκούρος τόνος δέρματος',
    '🤼🏿‍♀' => 'παλαίστριες: σκούρος τόνος δέρματος',
    '🤼🏻‍♂' => 'παλαιστές: ανοιχτόχρωμος τόνος δέρματος',
    '🤼🏼‍♂' => 'παλαιστές: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤼🏽‍♂' => 'παλαιστές: μεσαίος τόνος δέρματος',
    '🤼🏾‍♂' => 'παλαιστές: μεσαίος-σκούρος τόνος δέρματος',
    '🤼🏿‍♂' => 'παλαιστές: σκούρος τόνος δέρματος',
    '🤽🏻‍♂' => 'πολίστας: ανοιχτόχρωμος τόνος δέρματος',
    '🤽🏼‍♂' => 'πολίστας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤽🏽‍♂' => 'πολίστας: μεσαίος τόνος δέρματος',
    '🤽🏾‍♂' => 'πολίστας: μεσαίος-σκούρος τόνος δέρματος',
    '🤽🏿‍♂' => 'πολίστας: σκούρος τόνος δέρματος',
    '🤽🏻‍♀' => 'πολίστρια: ανοιχτόχρωμος τόνος δέρματος',
    '🤽🏼‍♀' => 'πολίστρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤽🏽‍♀' => 'πολίστρια: μεσαίος τόνος δέρματος',
    '🤽🏾‍♀' => 'πολίστρια: μεσαίος-σκούρος τόνος δέρματος',
    '🤽🏿‍♀' => 'πολίστρια: σκούρος τόνος δέρματος',
    '🤾🏻‍♂' => 'άντρας παίζει χάντμπολ: ανοιχτόχρωμος τόνος δέρματος',
    '🤾🏼‍♂' => 'άντρας παίζει χάντμπολ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤾🏽‍♂' => 'άντρας παίζει χάντμπολ: μεσαίος τόνος δέρματος',
    '🤾🏾‍♂' => 'άντρας παίζει χάντμπολ: μεσαίος-σκούρος τόνος δέρματος',
    '🤾🏿‍♂' => 'άντρας παίζει χάντμπολ: σκούρος τόνος δέρματος',
    '🤾🏻‍♀' => 'γυναίκα παίζει χάντμπολ: ανοιχτόχρωμος τόνος δέρματος',
    '🤾🏼‍♀' => 'γυναίκα παίζει χάντμπολ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤾🏽‍♀' => 'γυναίκα παίζει χάντμπολ: μεσαίος τόνος δέρματος',
    '🤾🏾‍♀' => 'γυναίκα παίζει χάντμπολ: μεσαίος-σκούρος τόνος δέρματος',
    '🤾🏿‍♀' => 'γυναίκα παίζει χάντμπολ: σκούρος τόνος δέρματος',
    '🤹🏻‍♂' => 'άντρας ζογκλέρ: ανοιχτόχρωμος τόνος δέρματος',
    '🤹🏼‍♂' => 'άντρας ζογκλέρ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤹🏽‍♂' => 'άντρας ζογκλέρ: μεσαίος τόνος δέρματος',
    '🤹🏾‍♂' => 'άντρας ζογκλέρ: μεσαίος-σκούρος τόνος δέρματος',
    '🤹🏿‍♂' => 'άντρας ζογκλέρ: σκούρος τόνος δέρματος',
    '🤹🏻‍♀' => 'γυναίκα ζογκλέρ: ανοιχτόχρωμος τόνος δέρματος',
    '🤹🏼‍♀' => 'γυναίκα ζογκλέρ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤹🏽‍♀' => 'γυναίκα ζογκλέρ: μεσαίος τόνος δέρματος',
    '🤹🏾‍♀' => 'γυναίκα ζογκλέρ: μεσαίος-σκούρος τόνος δέρματος',
    '🤹🏿‍♀' => 'γυναίκα ζογκλέρ: σκούρος τόνος δέρματος',
    '🧘🏻‍♂' => 'άντρας σε θέση λωτού: ανοιχτόχρωμος τόνος δέρματος',
    '🧘🏼‍♂' => 'άντρας σε θέση λωτού: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧘🏽‍♂' => 'άντρας σε θέση λωτού: μεσαίος τόνος δέρματος',
    '🧘🏾‍♂' => 'άντρας σε θέση λωτού: μεσαίος-σκούρος τόνος δέρματος',
    '🧘🏿‍♂' => 'άντρας σε θέση λωτού: σκούρος τόνος δέρματος',
    '🧘🏻‍♀' => 'γυναίκα σε θέση λωτού: ανοιχτόχρωμος τόνος δέρματος',
    '🧘🏼‍♀' => 'γυναίκα σε θέση λωτού: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧘🏽‍♀' => 'γυναίκα σε θέση λωτού: μεσαίος τόνος δέρματος',
    '🧘🏾‍♀' => 'γυναίκα σε θέση λωτού: μεσαίος-σκούρος τόνος δέρματος',
    '🧘🏿‍♀' => 'γυναίκα σε θέση λωτού: σκούρος τόνος δέρματος',
    '🧑‍🦰' => 'ενήλικας: κόκκινα μαλλιά',
    '🧑‍🦱' => 'ενήλικας: σγουρά μαλλιά',
    '🧑‍🦳' => 'ενήλικας: άσπρα μαλλιά',
    '🧑‍🦲' => 'ενήλικας: καραφλός',
    '👨‍🦰' => 'άντρας: κόκκινα μαλλιά',
    '👨‍🦱' => 'άντρας: σγουρά μαλλιά',
    '👨‍🦳' => 'άντρας: άσπρα μαλλιά',
    '👨‍🦲' => 'άντρας: καραφλός',
    '👩‍🦰' => 'γυναίκα: κόκκινα μαλλιά',
    '👩‍🦱' => 'γυναίκα: σγουρά μαλλιά',
    '👩‍🦳' => 'γυναίκα: άσπρα μαλλιά',
    '👩‍🦲' => 'γυναίκα: καραφλός',
    '🚶‍➡' => 'πεζός: με κατεύθυνση προς τα δεξιά',
    '🧎‍➡' => 'άτομο που γονατίζει: με κατεύθυνση προς τα δεξιά',
    '🏃‍➡' => 'άτομο τρέχει: με κατεύθυνση προς τα δεξιά',
    '👨‍👦' => 'οικογένεια: άντρας, αγόρι',
    '👨‍👧' => 'οικογένεια: άντρας, κορίτσι',
    '👩‍👦' => 'οικογένεια: γυναίκα, αγόρι',
    '👩‍👧' => 'οικογένεια: γυναίκα, κορίτσι',
    '😶‍🌫' => 'πρόσωπο στα σύννεφα',
    '😮‍💨' => 'πρόσωπο που ξεφυσάει',
    '🙂‍↕' => 'κεφάλι που κουνιέται κατακόρυφα',
    '😵‍💫' => 'πρόσωπο με μάτια σπιράλ',
    '❤‍🔥' => 'καρδιά που καίγεται',
    '❤‍🩹' => 'καρδιά που αναρρώνει',
    '👁‍🗨' => 'μάτι σε συννεφάκι ομιλίας',
    '🧔‍♂' => 'άντρας: μούσι',
    '👱‍♂' => 'άντρας: ξανθά μαλλιά',
    '🧔‍♀' => 'γυναίκα: μούσι',
    '👱‍♀' => 'γυναίκα: ξανθά μαλλιά',
    '🙍‍♂' => 'συνοφρυωμένος',
    '🙍‍♀' => 'συνοφρυωμένη',
    '🙎‍♂' => 'κατσουφιασμένος άντρας',
    '🙎‍♀' => 'κατσουφιασμένη γυναίκα',
    '🙅‍♂' => 'άντρας διαφωνεί',
    '🙅‍♀' => 'γυναίκα διαφωνεί',
    '🙆‍♂' => 'άντρας συμφωνεί',
    '🙆‍♀' => 'γυναίκα συμφωνεί',
    '💁‍♂' => 'άντρας προσφέρει',
    '💁‍♀' => 'γυναίκα προσφέρει',
    '🙋‍♂' => 'άντρας με σηκωμένο χέρι',
    '🙋‍♀' => 'γυναίκα με σηκωμένο χέρι',
    '🧏‍♂' => 'κωφός άντρας',
    '🧏‍♀' => 'κωφή γυναίκα',
    '🙇‍♂' => 'άντρας υποκλίνεται',
    '🙇‍♀' => 'γυναίκα υποκλίνεται',
    '🤦‍♂' => 'άντρας αγανακτά',
    '🤦‍♀' => 'γυναίκα αγανακτά',
    '🤷‍♂' => 'άντρας σηκώνει τους ώμους',
    '🤷‍♀' => 'γυναίκα σηκώνει τους ώμους',
    '🧑‍⚕' => 'εργαζόμενος του τομέα υγείας',
    '👨‍⚕' => 'εργαζόμενος στην υγεία',
    '👩‍⚕' => 'εργαζόμενη στην υγεία',
    '🧑‍🎓' => 'σπουδαστής',
    '👨‍🎓' => 'φοιτητής',
    '👩‍🎓' => 'φοιτήτρια',
    '🧑‍🏫' => 'καθηγητής',
    '👨‍🏫' => 'δάσκαλος',
    '👩‍🏫' => 'δασκάλα',
    '🧑‍⚖' => 'δικαστής',
    '👨‍⚖' => 'άντρας δικαστής',
    '👩‍⚖' => 'γυναίκα δικαστής',
    '🧑‍🌾' => 'καλλιεργητής',
    '👨‍🌾' => 'αγρότης',
    '👩‍🌾' => 'αγρότισσα',
    '🧑‍🍳' => 'σεφ',
    '👨‍🍳' => 'μάγειρας',
    '👩‍🍳' => 'μαγείρισσα',
    '🧑‍🔧' => 'μηχανικός',
    '👨‍🔧' => 'άντρας μηχανικός',
    '👩‍🔧' => 'γυναίκα μηχανικός',
    '🧑‍🏭' => 'εργάτης εργοστασίου',
    '👨‍🏭' => 'εργάτης',
    '👩‍🏭' => 'εργάτρια',
    '🧑‍💼' => 'υπάλληλος γραφείου',
    '👨‍💼' => 'άντρας υπάλληλος γραφείου',
    '👩‍💼' => 'γυναίκα υπάλληλος γραφείου',
    '🧑‍🔬' => 'επιστήμονας',
    '👨‍🔬' => 'άντρας επιστήμονας',
    '👩‍🔬' => 'γυναίκα επιστήμονας',
    '🧑‍💻' => 'τεχνολόγος',
    '👨‍💻' => 'άντρας τεχνολόγος',
    '👩‍💻' => 'γυναίκα τεχνολόγος',
    '🧑‍🎤' => 'ερμηνευτής',
    '👨‍🎤' => 'τραγουδιστής',
    '👩‍🎤' => 'τραγουδίστρια',
    '🧑‍🎨' => 'καλλιτέχνης',
    '👨‍🎨' => 'άντρας καλλιτέχνης',
    '👩‍🎨' => 'γυναίκα καλλιτέχνης',
    '🧑‍✈' => 'πιλότος',
    '👨‍✈' => 'άντρας πιλότος',
    '👩‍✈' => 'γυναίκα πιλότος',
    '🧑‍🚀' => 'αστροναύτης',
    '👨‍🚀' => 'άντρας αστροναύτης',
    '👩‍🚀' => 'γυναίκα αστροναύτης',
    '🧑‍🚒' => 'πυροσβέστης',
    '👨‍🚒' => 'άντρας πυροσβέστης',
    '👩‍🚒' => 'γυναίκα πυροσβέστης',
    '👮‍♂' => 'άντρας αστυνομικός',
    '👮‍♀' => 'γυναίκα αστυνομικός',
    '🕵‍♂' => 'άντρας ντετέκτιβ',
    '🕵‍♀' => 'γυναίκα ντετέκτιβ',
    '💂‍♂' => 'άντρας φρουρός',
    '💂‍♀' => 'γυναίκα φρουρός',
    '👷‍♂' => 'εργάτης οικοδομής',
    '👷‍♀' => 'εργάτρια οικοδομής',
    '👳‍♂' => 'άντρας με τουρμπάνι',
    '👳‍♀' => 'γυναίκα με τουρμπάνι',
    '🤵‍♂' => 'άντρας με σμόκιν',
    '🤵‍♀' => 'γυναίκα με σμόκιν',
    '👰‍♂' => 'άντρας με πέπλο',
    '👰‍♀' => 'γυναίκα με πέπλο',
    '👩‍🍼' => 'γυναίκα που ταΐζει μωρό',
    '👨‍🍼' => 'άντρας που ταΐζει μωρό',
    '🧑‍🍼' => 'άτομο που ταΐζει μωρό',
    '🧑‍🎄' => 'Άγιος Βασίλης',
    '🦸‍♂' => 'άντρας υπερήρωας',
    '🦸‍♀' => 'γυναίκα υπερήρωας',
    '🦹‍♂' => 'άντρας αρχικακός',
    '🦹‍♀' => 'γυναίκα αρχικακός',
    '🧙‍♂' => 'άντρας μάγος',
    '🧙‍♀' => 'μάγισσα',
    '🧚‍♂' => 'νεράιδος',
    '🧚‍♀' => 'γυναίκα νεράιδα',
    '🧛‍♂' => 'άντρας βρυκόλακας',
    '🧛‍♀' => 'γυναίκα βρυκόλακας',
    '🧜‍♂' => 'γοργόνος',
    '🧜‍♀' => 'γυναίκα γοργόνα',
    '🧝‍♂' => 'άντρας ξωτικό',
    '🧝‍♀' => 'γυναίκα ξωτικό',
    '🧞‍♂' => 'άντρας τζίνι',
    '🧞‍♀' => 'γυναίκα τζίνι',
    '🧟‍♂' => 'άντρας ζόμπι',
    '🧟‍♀' => 'γυναίκα ζόμπι',
    '💆‍♂' => 'μασάζ στο κεφάλι άντρα',
    '💆‍♀' => 'μασάζ στο κεφάλι γυναίκας',
    '💇‍♂' => 'άντρας κουρεύεται',
    '💇‍♀' => 'γυναίκα κουρεύεται',
    '🚶‍♂' => 'άντρας περπατά',
    '🚶‍♀' => 'γυναίκα περπατά',
    '🧍‍♂' => 'άντρας που στέκεται',
    '🧍‍♀' => 'γυναίκα που στέκεται',
    '🧎‍♂' => 'άντρας που γονατίζει',
    '🧎‍♀' => 'γυναίκα που γονατίζει',
    '🧑‍🦯' => 'άτομο με μπαστούνι για τυφλούς',
    '👨‍🦯' => 'άντρας με μπαστούνι για τυφλούς',
    '👩‍🦯' => 'γυναίκα με μπαστούνι για τυφλούς',
    '🧑‍🦼' => 'άτομο σε ηλεκτρικό αναπηρικό αμαξίδιο',
    '👨‍🦼' => 'άντρας σε ηλεκτροκίνητο αναπηρικό αμαξίδιο',
    '👩‍🦼' => 'γυναίκα σε ηλεκτροκίνητο αναπηρικό αμαξίδιο',
    '🧑‍🦽' => 'άτομο σε χειροκίνητο αναπηρικό αμαξίδιο',
    '👨‍🦽' => 'άντρας σε χειροκίνητο αναπηρικό αμαξίδιο',
    '👩‍🦽' => 'γυναίκα σε χειροκίνητο αναπηρικό αμαξίδιο',
    '🏃‍♂' => 'άντρας τρέχει',
    '🏃‍♀' => 'γυναίκα τρέχει',
    '🧑‍🩰' => 'χορευτής μπαλέτου',
    '👯‍♂' => 'άντρες με λαγουδένια αυτιά',
    '👯‍♀' => 'γυναίκες με λαγουδένια αυτιά',
    '🧖‍♂' => 'άντρας σε σάουνα',
    '🧖‍♀' => 'γυναίκα σε σάουνα',
    '🧗‍♂' => 'άντρας ορειβάτης',
    '🧗‍♀' => 'γυναίκα ορειβάτης',
    '🏌‍♂' => 'άντρας παίζει γκολφ',
    '🏌‍♀' => 'γυναίκα παίζει γκολφ',
    '🏄‍♂' => 'άντρας σε σανίδα σερφ',
    '🏄‍♀' => 'γυναίκα σε σανίδα σερφ',
    '🚣‍♂' => 'άντρας κωπηλάτης',
    '🚣‍♀' => 'γυναίκα κωπηλάτης',
    '🏊‍♂' => 'άντρας κολυμβητής',
    '🏊‍♀' => 'κολυμβήτρια',
    '⛹‍♂' => 'άντρας παίζει μπάσκετ',
    '⛹‍♀' => 'γυναίκα παίζει μπάσκετ',
    '🏋‍♂' => 'άντρας αρσιβαρίστας',
    '🏋‍♀' => 'αρσιβαρίστρια',
    '🚴‍♂' => 'άντρας ποδηλάτης',
    '🚴‍♀' => 'ποδηλάτισσα',
    '🚵‍♂' => 'ορεινός ποδηλάτης',
    '🚵‍♀' => 'ορεινή ποδηλάτισσα',
    '🤸‍♂' => 'άντρας κάνει ρόδα',
    '🤸‍♀' => 'γυναίκα κάνει ρόδα',
    '🤼‍♂' => 'παλαιστές',
    '🤼‍♀' => 'παλαίστριες',
    '🤽‍♂' => 'πολίστας',
    '🤽‍♀' => 'πολίστρια',
    '🤾‍♂' => 'άντρας παίζει χάντμπολ',
    '🤾‍♀' => 'γυναίκα παίζει χάντμπολ',
    '🤹‍♂' => 'άντρας ζογκλέρ',
    '🤹‍♀' => 'γυναίκα ζογκλέρ',
    '🧘‍♂' => 'άντρας σε θέση λωτού',
    '🧘‍♀' => 'γυναίκα σε θέση λωτού',
    '🧑‍🧒' => 'οικογένεια: ενήλικας, παιδί',
    '🐕‍🦺' => 'σκύλος βοηθός',
    '🐈‍⬛' => 'μαύρη γάτα',
    '🐻‍❄' => 'πολική αρκούδα',
    '🐦‍⬛' => 'μαύρο πουλί',
    '🐦‍🔥' => 'φοίνικας',
    '🍋‍🟩' => 'λάιμ',
    '🍄‍🟫' => 'καφέ μανιτάρι',
    '⛓‍💥' => 'σπασμένη αλυσίδα',
    '🏳‍🌈' => 'σημαία ουράνιου τόξου',
    '🏳‍⚧' => 'σημαία των διεμφυλικών',
    '🏴‍☠' => 'πειρατική σημαία',
    '👋🏻' => 'χέρι που χαιρετά: ανοιχτόχρωμος τόνος δέρματος',
    '👋🏼' => 'χέρι που χαιρετά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👋🏽' => 'χέρι που χαιρετά: μεσαίος τόνος δέρματος',
    '👋🏾' => 'χέρι που χαιρετά: μεσαίος-σκούρος τόνος δέρματος',
    '👋🏿' => 'χέρι που χαιρετά: σκούρος τόνος δέρματος',
    '🤚🏻' => 'σηκωμένο χέρι ανάποδα: ανοιχτόχρωμος τόνος δέρματος',
    '🤚🏼' => 'σηκωμένο χέρι ανάποδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤚🏽' => 'σηκωμένο χέρι ανάποδα: μεσαίος τόνος δέρματος',
    '🤚🏾' => 'σηκωμένο χέρι ανάποδα: μεσαίος-σκούρος τόνος δέρματος',
    '🤚🏿' => 'σηκωμένο χέρι ανάποδα: σκούρος τόνος δέρματος',
    '🖐🏻' => 'μούντζα: ανοιχτόχρωμος τόνος δέρματος',
    '🖐🏼' => 'μούντζα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🖐🏽' => 'μούντζα: μεσαίος τόνος δέρματος',
    '🖐🏾' => 'μούντζα: μεσαίος-σκούρος τόνος δέρματος',
    '🖐🏿' => 'μούντζα: σκούρος τόνος δέρματος',
    '✋🏻' => 'σηκωμένο χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '✋🏼' => 'σηκωμένο χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '✋🏽' => 'σηκωμένο χέρι: μεσαίος τόνος δέρματος',
    '✋🏾' => 'σηκωμένο χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '✋🏿' => 'σηκωμένο χέρι: σκούρος τόνος δέρματος',
    '🖖🏻' => 'χαιρετισμός σποκ: ανοιχτόχρωμος τόνος δέρματος',
    '🖖🏼' => 'χαιρετισμός σποκ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🖖🏽' => 'χαιρετισμός σποκ: μεσαίος τόνος δέρματος',
    '🖖🏾' => 'χαιρετισμός σποκ: μεσαίος-σκούρος τόνος δέρματος',
    '🖖🏿' => 'χαιρετισμός σποκ: σκούρος τόνος δέρματος',
    '🫱🏻' => 'χέρι προς τα δεξιά: ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏼' => 'χέρι προς τα δεξιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫱🏽' => 'χέρι προς τα δεξιά: μεσαίος τόνος δέρματος',
    '🫱🏾' => 'χέρι προς τα δεξιά: μεσαίος-σκούρος τόνος δέρματος',
    '🫱🏿' => 'χέρι προς τα δεξιά: σκούρος τόνος δέρματος',
    '🫲🏻' => 'χέρι προς τα αριστερά: ανοιχτόχρωμος τόνος δέρματος',
    '🫲🏼' => 'χέρι προς τα αριστερά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫲🏽' => 'χέρι προς τα αριστερά: μεσαίος τόνος δέρματος',
    '🫲🏾' => 'χέρι προς τα αριστερά: μεσαίος-σκούρος τόνος δέρματος',
    '🫲🏿' => 'χέρι προς τα αριστερά: σκούρος τόνος δέρματος',
    '🫳🏻' => 'παλάμη προς τα κάτω: ανοιχτόχρωμος τόνος δέρματος',
    '🫳🏼' => 'παλάμη προς τα κάτω: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫳🏽' => 'παλάμη προς τα κάτω: μεσαίος τόνος δέρματος',
    '🫳🏾' => 'παλάμη προς τα κάτω: μεσαίος-σκούρος τόνος δέρματος',
    '🫳🏿' => 'παλάμη προς τα κάτω: σκούρος τόνος δέρματος',
    '🫴🏻' => 'παλάμη προς τα πάνω: ανοιχτόχρωμος τόνος δέρματος',
    '🫴🏼' => 'παλάμη προς τα πάνω: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫴🏽' => 'παλάμη προς τα πάνω: μεσαίος τόνος δέρματος',
    '🫴🏾' => 'παλάμη προς τα πάνω: μεσαίος-σκούρος τόνος δέρματος',
    '🫴🏿' => 'παλάμη προς τα πάνω: σκούρος τόνος δέρματος',
    '🫷🏻' => 'χέρι που σπρώχνει προς τα αριστερά: ανοιχτόχρωμος τόνος δέρματος',
    '🫷🏼' => 'χέρι που σπρώχνει προς τα αριστερά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫷🏽' => 'χέρι που σπρώχνει προς τα αριστερά: μεσαίος τόνος δέρματος',
    '🫷🏾' => 'χέρι που σπρώχνει προς τα αριστερά: μεσαίος-σκούρος τόνος δέρματος',
    '🫷🏿' => 'χέρι που σπρώχνει προς τα αριστερά: σκούρος τόνος δέρματος',
    '🫸🏻' => 'χέρι που σπρώχνει προς τα δεξιά: ανοιχτόχρωμος τόνος δέρματος',
    '🫸🏼' => 'χέρι που σπρώχνει προς τα δεξιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫸🏽' => 'χέρι που σπρώχνει προς τα δεξιά: μεσαίος τόνος δέρματος',
    '🫸🏾' => 'χέρι που σπρώχνει προς τα δεξιά: μεσαίος-σκούρος τόνος δέρματος',
    '🫸🏿' => 'χέρι που σπρώχνει προς τα δεξιά: σκούρος τόνος δέρματος',
    '👌🏻' => 'χειρονομία οκ: ανοιχτόχρωμος τόνος δέρματος',
    '👌🏼' => 'χειρονομία οκ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👌🏽' => 'χειρονομία οκ: μεσαίος τόνος δέρματος',
    '👌🏾' => 'χειρονομία οκ: μεσαίος-σκούρος τόνος δέρματος',
    '👌🏿' => 'χειρονομία οκ: σκούρος τόνος δέρματος',
    '🤌🏻' => 'δάχτυλα που κλείνουν: ανοιχτόχρωμος τόνος δέρματος',
    '🤌🏼' => 'δάχτυλα που κλείνουν: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤌🏽' => 'δάχτυλα που κλείνουν: μεσαίος τόνος δέρματος',
    '🤌🏾' => 'δάχτυλα που κλείνουν: μεσαίος-σκούρος τόνος δέρματος',
    '🤌🏿' => 'δάχτυλα που κλείνουν: σκούρος τόνος δέρματος',
    '🤏🏻' => 'χέρι που τσιμπάει: ανοιχτόχρωμος τόνος δέρματος',
    '🤏🏼' => 'χέρι που τσιμπάει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤏🏽' => 'χέρι που τσιμπάει: μεσαίος τόνος δέρματος',
    '🤏🏾' => 'χέρι που τσιμπάει: μεσαίος-σκούρος τόνος δέρματος',
    '🤏🏿' => 'χέρι που τσιμπάει: σκούρος τόνος δέρματος',
    '✌🏻' => 'σήμα της νίκης: ανοιχτόχρωμος τόνος δέρματος',
    '✌🏼' => 'σήμα της νίκης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '✌🏽' => 'σήμα της νίκης: μεσαίος τόνος δέρματος',
    '✌🏾' => 'σήμα της νίκης: μεσαίος-σκούρος τόνος δέρματος',
    '✌🏿' => 'σήμα της νίκης: σκούρος τόνος δέρματος',
    '🤞🏻' => 'καλή τύχη: ανοιχτόχρωμος τόνος δέρματος',
    '🤞🏼' => 'καλή τύχη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤞🏽' => 'καλή τύχη: μεσαίος τόνος δέρματος',
    '🤞🏾' => 'καλή τύχη: μεσαίος-σκούρος τόνος δέρματος',
    '🤞🏿' => 'καλή τύχη: σκούρος τόνος δέρματος',
    '🫰🏻' => 'δείκτης και αντίχειρας σταυρωμένοι: ανοιχτόχρωμος τόνος δέρματος',
    '🫰🏼' => 'δείκτης και αντίχειρας σταυρωμένοι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫰🏽' => 'δείκτης και αντίχειρας σταυρωμένοι: μεσαίος τόνος δέρματος',
    '🫰🏾' => 'δείκτης και αντίχειρας σταυρωμένοι: μεσαίος-σκούρος τόνος δέρματος',
    '🫰🏿' => 'δείκτης και αντίχειρας σταυρωμένοι: σκούρος τόνος δέρματος',
    '🤟🏻' => 'κίνηση σε αγαπώ: ανοιχτόχρωμος τόνος δέρματος',
    '🤟🏼' => 'κίνηση σε αγαπώ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤟🏽' => 'κίνηση σε αγαπώ: μεσαίος τόνος δέρματος',
    '🤟🏾' => 'κίνηση σε αγαπώ: μεσαίος-σκούρος τόνος δέρματος',
    '🤟🏿' => 'κίνηση σε αγαπώ: σκούρος τόνος δέρματος',
    '🤘🏻' => 'χειρονομία ροκ: ανοιχτόχρωμος τόνος δέρματος',
    '🤘🏼' => 'χειρονομία ροκ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤘🏽' => 'χειρονομία ροκ: μεσαίος τόνος δέρματος',
    '🤘🏾' => 'χειρονομία ροκ: μεσαίος-σκούρος τόνος δέρματος',
    '🤘🏿' => 'χειρονομία ροκ: σκούρος τόνος δέρματος',
    '🤙🏻' => 'τηλεφώνησέ μου: ανοιχτόχρωμος τόνος δέρματος',
    '🤙🏼' => 'τηλεφώνησέ μου: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤙🏽' => 'τηλεφώνησέ μου: μεσαίος τόνος δέρματος',
    '🤙🏾' => 'τηλεφώνησέ μου: μεσαίος-σκούρος τόνος δέρματος',
    '🤙🏿' => 'τηλεφώνησέ μου: σκούρος τόνος δέρματος',
    '👈🏻' => 'δείκτης χεριού προς τα αριστερά: ανοιχτόχρωμος τόνος δέρματος',
    '👈🏼' => 'δείκτης χεριού προς τα αριστερά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👈🏽' => 'δείκτης χεριού προς τα αριστερά: μεσαίος τόνος δέρματος',
    '👈🏾' => 'δείκτης χεριού προς τα αριστερά: μεσαίος-σκούρος τόνος δέρματος',
    '👈🏿' => 'δείκτης χεριού προς τα αριστερά: σκούρος τόνος δέρματος',
    '👉🏻' => 'δείκτης χεριού προς τα δεξιά: ανοιχτόχρωμος τόνος δέρματος',
    '👉🏼' => 'δείκτης χεριού προς τα δεξιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👉🏽' => 'δείκτης χεριού προς τα δεξιά: μεσαίος τόνος δέρματος',
    '👉🏾' => 'δείκτης χεριού προς τα δεξιά: μεσαίος-σκούρος τόνος δέρματος',
    '👉🏿' => 'δείκτης χεριού προς τα δεξιά: σκούρος τόνος δέρματος',
    '👆🏻' => 'δείκτης χεριού προς τα επάνω ανάποδα: ανοιχτόχρωμος τόνος δέρματος',
    '👆🏼' => 'δείκτης χεριού προς τα επάνω ανάποδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👆🏽' => 'δείκτης χεριού προς τα επάνω ανάποδα: μεσαίος τόνος δέρματος',
    '👆🏾' => 'δείκτης χεριού προς τα επάνω ανάποδα: μεσαίος-σκούρος τόνος δέρματος',
    '👆🏿' => 'δείκτης χεριού προς τα επάνω ανάποδα: σκούρος τόνος δέρματος',
    '🖕🏻' => 'μεσαίο δάχτυλο: ανοιχτόχρωμος τόνος δέρματος',
    '🖕🏼' => 'μεσαίο δάχτυλο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🖕🏽' => 'μεσαίο δάχτυλο: μεσαίος τόνος δέρματος',
    '🖕🏾' => 'μεσαίο δάχτυλο: μεσαίος-σκούρος τόνος δέρματος',
    '🖕🏿' => 'μεσαίο δάχτυλο: σκούρος τόνος δέρματος',
    '👇🏻' => 'δείκτης χεριού προς τα κάτω ανάποδα: ανοιχτόχρωμος τόνος δέρματος',
    '👇🏼' => 'δείκτης χεριού προς τα κάτω ανάποδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👇🏽' => 'δείκτης χεριού προς τα κάτω ανάποδα: μεσαίος τόνος δέρματος',
    '👇🏾' => 'δείκτης χεριού προς τα κάτω ανάποδα: μεσαίος-σκούρος τόνος δέρματος',
    '👇🏿' => 'δείκτης χεριού προς τα κάτω ανάποδα: σκούρος τόνος δέρματος',
    '☝🏻' => 'δείκτης χεριού προς τα επάνω: ανοιχτόχρωμος τόνος δέρματος',
    '☝🏼' => 'δείκτης χεριού προς τα επάνω: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '☝🏽' => 'δείκτης χεριού προς τα επάνω: μεσαίος τόνος δέρματος',
    '☝🏾' => 'δείκτης χεριού προς τα επάνω: μεσαίος-σκούρος τόνος δέρματος',
    '☝🏿' => 'δείκτης χεριού προς τα επάνω: σκούρος τόνος δέρματος',
    '🫵🏻' => 'δείκτης που δείχνει προς τον θεατή: ανοιχτόχρωμος τόνος δέρματος',
    '🫵🏼' => 'δείκτης που δείχνει προς τον θεατή: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫵🏽' => 'δείκτης που δείχνει προς τον θεατή: μεσαίος τόνος δέρματος',
    '🫵🏾' => 'δείκτης που δείχνει προς τον θεατή: μεσαίος-σκούρος τόνος δέρματος',
    '🫵🏿' => 'δείκτης που δείχνει προς τον θεατή: σκούρος τόνος δέρματος',
    '👍🏻' => 'σηκωμένος αντίχειρας: ανοιχτόχρωμος τόνος δέρματος',
    '👍🏼' => 'σηκωμένος αντίχειρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👍🏽' => 'σηκωμένος αντίχειρας: μεσαίος τόνος δέρματος',
    '👍🏾' => 'σηκωμένος αντίχειρας: μεσαίος-σκούρος τόνος δέρματος',
    '👍🏿' => 'σηκωμένος αντίχειρας: σκούρος τόνος δέρματος',
    '👎🏻' => 'κατεβασμένος αντίχειρας: ανοιχτόχρωμος τόνος δέρματος',
    '👎🏼' => 'κατεβασμένος αντίχειρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👎🏽' => 'κατεβασμένος αντίχειρας: μεσαίος τόνος δέρματος',
    '👎🏾' => 'κατεβασμένος αντίχειρας: μεσαίος-σκούρος τόνος δέρματος',
    '👎🏿' => 'κατεβασμένος αντίχειρας: σκούρος τόνος δέρματος',
    '✊🏻' => 'σηκωμένη γροθιά: ανοιχτόχρωμος τόνος δέρματος',
    '✊🏼' => 'σηκωμένη γροθιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '✊🏽' => 'σηκωμένη γροθιά: μεσαίος τόνος δέρματος',
    '✊🏾' => 'σηκωμένη γροθιά: μεσαίος-σκούρος τόνος δέρματος',
    '✊🏿' => 'σηκωμένη γροθιά: σκούρος τόνος δέρματος',
    '👊🏻' => 'γροθιά: ανοιχτόχρωμος τόνος δέρματος',
    '👊🏼' => 'γροθιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👊🏽' => 'γροθιά: μεσαίος τόνος δέρματος',
    '👊🏾' => 'γροθιά: μεσαίος-σκούρος τόνος δέρματος',
    '👊🏿' => 'γροθιά: σκούρος τόνος δέρματος',
    '🤛🏻' => 'γροθιά προς αριστερά: ανοιχτόχρωμος τόνος δέρματος',
    '🤛🏼' => 'γροθιά προς αριστερά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤛🏽' => 'γροθιά προς αριστερά: μεσαίος τόνος δέρματος',
    '🤛🏾' => 'γροθιά προς αριστερά: μεσαίος-σκούρος τόνος δέρματος',
    '🤛🏿' => 'γροθιά προς αριστερά: σκούρος τόνος δέρματος',
    '🤜🏻' => 'γροθιά προς δεξιά: ανοιχτόχρωμος τόνος δέρματος',
    '🤜🏼' => 'γροθιά προς δεξιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤜🏽' => 'γροθιά προς δεξιά: μεσαίος τόνος δέρματος',
    '🤜🏾' => 'γροθιά προς δεξιά: μεσαίος-σκούρος τόνος δέρματος',
    '🤜🏿' => 'γροθιά προς δεξιά: σκούρος τόνος δέρματος',
    '👏🏻' => 'παλαμάκια: ανοιχτόχρωμος τόνος δέρματος',
    '👏🏼' => 'παλαμάκια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👏🏽' => 'παλαμάκια: μεσαίος τόνος δέρματος',
    '👏🏾' => 'παλαμάκια: μεσαίος-σκούρος τόνος δέρματος',
    '👏🏿' => 'παλαμάκια: σκούρος τόνος δέρματος',
    '🙌🏻' => 'χέρια ψηλά: ανοιχτόχρωμος τόνος δέρματος',
    '🙌🏼' => 'χέρια ψηλά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙌🏽' => 'χέρια ψηλά: μεσαίος τόνος δέρματος',
    '🙌🏾' => 'χέρια ψηλά: μεσαίος-σκούρος τόνος δέρματος',
    '🙌🏿' => 'χέρια ψηλά: σκούρος τόνος δέρματος',
    '🫶🏻' => 'καρδούλα με τα χέρια: ανοιχτόχρωμος τόνος δέρματος',
    '🫶🏼' => 'καρδούλα με τα χέρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫶🏽' => 'καρδούλα με τα χέρια: μεσαίος τόνος δέρματος',
    '🫶🏾' => 'καρδούλα με τα χέρια: μεσαίος-σκούρος τόνος δέρματος',
    '🫶🏿' => 'καρδούλα με τα χέρια: σκούρος τόνος δέρματος',
    '👐🏻' => 'ανοικτά χέρια: ανοιχτόχρωμος τόνος δέρματος',
    '👐🏼' => 'ανοικτά χέρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👐🏽' => 'ανοικτά χέρια: μεσαίος τόνος δέρματος',
    '👐🏾' => 'ανοικτά χέρια: μεσαίος-σκούρος τόνος δέρματος',
    '👐🏿' => 'ανοικτά χέρια: σκούρος τόνος δέρματος',
    '🤲🏻' => 'παλάμες μαζί: ανοιχτόχρωμος τόνος δέρματος',
    '🤲🏼' => 'παλάμες μαζί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤲🏽' => 'παλάμες μαζί: μεσαίος τόνος δέρματος',
    '🤲🏾' => 'παλάμες μαζί: μεσαίος-σκούρος τόνος δέρματος',
    '🤲🏿' => 'παλάμες μαζί: σκούρος τόνος δέρματος',
    '🤝🏻' => 'χειραψία: ανοιχτόχρωμος τόνος δέρματος',
    '🤝🏼' => 'χειραψία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤝🏽' => 'χειραψία: μεσαίος τόνος δέρματος',
    '🤝🏾' => 'χειραψία: μεσαίος-σκούρος τόνος δέρματος',
    '🤝🏿' => 'χειραψία: σκούρος τόνος δέρματος',
    '🙏🏻' => 'χέρια ενωμένα: ανοιχτόχρωμος τόνος δέρματος',
    '🙏🏼' => 'χέρια ενωμένα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙏🏽' => 'χέρια ενωμένα: μεσαίος τόνος δέρματος',
    '🙏🏾' => 'χέρια ενωμένα: μεσαίος-σκούρος τόνος δέρματος',
    '🙏🏿' => 'χέρια ενωμένα: σκούρος τόνος δέρματος',
    '✍🏻' => 'χέρι που γράφει: ανοιχτόχρωμος τόνος δέρματος',
    '✍🏼' => 'χέρι που γράφει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '✍🏽' => 'χέρι που γράφει: μεσαίος τόνος δέρματος',
    '✍🏾' => 'χέρι που γράφει: μεσαίος-σκούρος τόνος δέρματος',
    '✍🏿' => 'χέρι που γράφει: σκούρος τόνος δέρματος',
    '💅🏻' => 'βάψιμο νυχιών: ανοιχτόχρωμος τόνος δέρματος',
    '💅🏼' => 'βάψιμο νυχιών: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💅🏽' => 'βάψιμο νυχιών: μεσαίος τόνος δέρματος',
    '💅🏾' => 'βάψιμο νυχιών: μεσαίος-σκούρος τόνος δέρματος',
    '💅🏿' => 'βάψιμο νυχιών: σκούρος τόνος δέρματος',
    '🤳🏻' => 'selfie: ανοιχτόχρωμος τόνος δέρματος',
    '🤳🏼' => 'selfie: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤳🏽' => 'selfie: μεσαίος τόνος δέρματος',
    '🤳🏾' => 'selfie: μεσαίος-σκούρος τόνος δέρματος',
    '🤳🏿' => 'selfie: σκούρος τόνος δέρματος',
    '💪🏻' => 'σφιχτός δικέφαλος: ανοιχτόχρωμος τόνος δέρματος',
    '💪🏼' => 'σφιχτός δικέφαλος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💪🏽' => 'σφιχτός δικέφαλος: μεσαίος τόνος δέρματος',
    '💪🏾' => 'σφιχτός δικέφαλος: μεσαίος-σκούρος τόνος δέρματος',
    '💪🏿' => 'σφιχτός δικέφαλος: σκούρος τόνος δέρματος',
    '🦵🏻' => 'πόδι: ανοιχτόχρωμος τόνος δέρματος',
    '🦵🏼' => 'πόδι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦵🏽' => 'πόδι: μεσαίος τόνος δέρματος',
    '🦵🏾' => 'πόδι: μεσαίος-σκούρος τόνος δέρματος',
    '🦵🏿' => 'πόδι: σκούρος τόνος δέρματος',
    '🦶🏻' => 'κάτω άκρο: ανοιχτόχρωμος τόνος δέρματος',
    '🦶🏼' => 'κάτω άκρο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦶🏽' => 'κάτω άκρο: μεσαίος τόνος δέρματος',
    '🦶🏾' => 'κάτω άκρο: μεσαίος-σκούρος τόνος δέρματος',
    '🦶🏿' => 'κάτω άκρο: σκούρος τόνος δέρματος',
    '👂🏻' => 'αυτί: ανοιχτόχρωμος τόνος δέρματος',
    '👂🏼' => 'αυτί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👂🏽' => 'αυτί: μεσαίος τόνος δέρματος',
    '👂🏾' => 'αυτί: μεσαίος-σκούρος τόνος δέρματος',
    '👂🏿' => 'αυτί: σκούρος τόνος δέρματος',
    '🦻🏻' => 'αυτί με ακουστικό βαρηκοΐας: ανοιχτόχρωμος τόνος δέρματος',
    '🦻🏼' => 'αυτί με ακουστικό βαρηκοΐας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦻🏽' => 'αυτί με ακουστικό βαρηκοΐας: μεσαίος τόνος δέρματος',
    '🦻🏾' => 'αυτί με ακουστικό βαρηκοΐας: μεσαίος-σκούρος τόνος δέρματος',
    '🦻🏿' => 'αυτί με ακουστικό βαρηκοΐας: σκούρος τόνος δέρματος',
    '👃🏻' => 'μύτη: ανοιχτόχρωμος τόνος δέρματος',
    '👃🏼' => 'μύτη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👃🏽' => 'μύτη: μεσαίος τόνος δέρματος',
    '👃🏾' => 'μύτη: μεσαίος-σκούρος τόνος δέρματος',
    '👃🏿' => 'μύτη: σκούρος τόνος δέρματος',
    '👶🏻' => 'μωρό: ανοιχτόχρωμος τόνος δέρματος',
    '👶🏼' => 'μωρό: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👶🏽' => 'μωρό: μεσαίος τόνος δέρματος',
    '👶🏾' => 'μωρό: μεσαίος-σκούρος τόνος δέρματος',
    '👶🏿' => 'μωρό: σκούρος τόνος δέρματος',
    '🧒🏻' => 'παιδί: ανοιχτόχρωμος τόνος δέρματος',
    '🧒🏼' => 'παιδί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧒🏽' => 'παιδί: μεσαίος τόνος δέρματος',
    '🧒🏾' => 'παιδί: μεσαίος-σκούρος τόνος δέρματος',
    '🧒🏿' => 'παιδί: σκούρος τόνος δέρματος',
    '👦🏻' => 'αγόρι: ανοιχτόχρωμος τόνος δέρματος',
    '👦🏼' => 'αγόρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👦🏽' => 'αγόρι: μεσαίος τόνος δέρματος',
    '👦🏾' => 'αγόρι: μεσαίος-σκούρος τόνος δέρματος',
    '👦🏿' => 'αγόρι: σκούρος τόνος δέρματος',
    '👧🏻' => 'κορίτσι: ανοιχτόχρωμος τόνος δέρματος',
    '👧🏼' => 'κορίτσι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👧🏽' => 'κορίτσι: μεσαίος τόνος δέρματος',
    '👧🏾' => 'κορίτσι: μεσαίος-σκούρος τόνος δέρματος',
    '👧🏿' => 'κορίτσι: σκούρος τόνος δέρματος',
    '🧑🏻' => 'ενήλικας: ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏼' => 'ενήλικας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧑🏽' => 'ενήλικας: μεσαίος τόνος δέρματος',
    '🧑🏾' => 'ενήλικας: μεσαίος-σκούρος τόνος δέρματος',
    '🧑🏿' => 'ενήλικας: σκούρος τόνος δέρματος',
    '👱🏻' => 'άτομο: ανοιχτόχρωμος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏼' => 'άτομο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏽' => 'άτομο: μεσαίος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏾' => 'άτομο: μεσαίος-σκούρος τόνος δέρματος, ξανθά μαλλιά',
    '👱🏿' => 'άτομο: σκούρος τόνος δέρματος, ξανθά μαλλιά',
    '👨🏻' => 'άντρας: ανοιχτόχρωμος τόνος δέρματος',
    '👨🏼' => 'άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👨🏽' => 'άντρας: μεσαίος τόνος δέρματος',
    '👨🏾' => 'άντρας: μεσαίος-σκούρος τόνος δέρματος',
    '👨🏿' => 'άντρας: σκούρος τόνος δέρματος',
    '🧔🏻' => 'άτομο: ανοιχτόχρωμος τόνος δέρματος, μούσι',
    '🧔🏼' => 'άτομο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος, μούσι',
    '🧔🏽' => 'άτομο: μεσαίος τόνος δέρματος, μούσι',
    '🧔🏾' => 'άτομο: μεσαίος-σκούρος τόνος δέρματος, μούσι',
    '🧔🏿' => 'άτομο: σκούρος τόνος δέρματος, μούσι',
    '👩🏻' => 'γυναίκα: ανοιχτόχρωμος τόνος δέρματος',
    '👩🏼' => 'γυναίκα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👩🏽' => 'γυναίκα: μεσαίος τόνος δέρματος',
    '👩🏾' => 'γυναίκα: μεσαίος-σκούρος τόνος δέρματος',
    '👩🏿' => 'γυναίκα: σκούρος τόνος δέρματος',
    '🧓🏻' => 'μεγαλύτερος ενήλικας: ανοιχτόχρωμος τόνος δέρματος',
    '🧓🏼' => 'μεγαλύτερος ενήλικας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧓🏽' => 'μεγαλύτερος ενήλικας: μεσαίος τόνος δέρματος',
    '🧓🏾' => 'μεγαλύτερος ενήλικας: μεσαίος-σκούρος τόνος δέρματος',
    '🧓🏿' => 'μεγαλύτερος ενήλικας: σκούρος τόνος δέρματος',
    '👴🏻' => 'ηλικιωμένος: ανοιχτόχρωμος τόνος δέρματος',
    '👴🏼' => 'ηλικιωμένος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👴🏽' => 'ηλικιωμένος: μεσαίος τόνος δέρματος',
    '👴🏾' => 'ηλικιωμένος: μεσαίος-σκούρος τόνος δέρματος',
    '👴🏿' => 'ηλικιωμένος: σκούρος τόνος δέρματος',
    '👵🏻' => 'ηλικιωμένη: ανοιχτόχρωμος τόνος δέρματος',
    '👵🏼' => 'ηλικιωμένη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👵🏽' => 'ηλικιωμένη: μεσαίος τόνος δέρματος',
    '👵🏾' => 'ηλικιωμένη: μεσαίος-σκούρος τόνος δέρματος',
    '👵🏿' => 'ηλικιωμένη: σκούρος τόνος δέρματος',
    '🙍🏻' => 'συνοφρυωμένο άτομο: ανοιχτόχρωμος τόνος δέρματος',
    '🙍🏼' => 'συνοφρυωμένο άτομο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙍🏽' => 'συνοφρυωμένο άτομο: μεσαίος τόνος δέρματος',
    '🙍🏾' => 'συνοφρυωμένο άτομο: μεσαίος-σκούρος τόνος δέρματος',
    '🙍🏿' => 'συνοφρυωμένο άτομο: σκούρος τόνος δέρματος',
    '🙎🏻' => 'κατσουφιασμένο άτομο: ανοιχτόχρωμος τόνος δέρματος',
    '🙎🏼' => 'κατσουφιασμένο άτομο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙎🏽' => 'κατσουφιασμένο άτομο: μεσαίος τόνος δέρματος',
    '🙎🏾' => 'κατσουφιασμένο άτομο: μεσαίος-σκούρος τόνος δέρματος',
    '🙎🏿' => 'κατσουφιασμένο άτομο: σκούρος τόνος δέρματος',
    '🙅🏻' => 'άτομο διαφωνεί: ανοιχτόχρωμος τόνος δέρματος',
    '🙅🏼' => 'άτομο διαφωνεί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙅🏽' => 'άτομο διαφωνεί: μεσαίος τόνος δέρματος',
    '🙅🏾' => 'άτομο διαφωνεί: μεσαίος-σκούρος τόνος δέρματος',
    '🙅🏿' => 'άτομο διαφωνεί: σκούρος τόνος δέρματος',
    '🙆🏻' => 'άτομο συμφωνεί: ανοιχτόχρωμος τόνος δέρματος',
    '🙆🏼' => 'άτομο συμφωνεί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙆🏽' => 'άτομο συμφωνεί: μεσαίος τόνος δέρματος',
    '🙆🏾' => 'άτομο συμφωνεί: μεσαίος-σκούρος τόνος δέρματος',
    '🙆🏿' => 'άτομο συμφωνεί: σκούρος τόνος δέρματος',
    '💁🏻' => 'άτομο προσφέρει: ανοιχτόχρωμος τόνος δέρματος',
    '💁🏼' => 'άτομο προσφέρει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💁🏽' => 'άτομο προσφέρει: μεσαίος τόνος δέρματος',
    '💁🏾' => 'άτομο προσφέρει: μεσαίος-σκούρος τόνος δέρματος',
    '💁🏿' => 'άτομο προσφέρει: σκούρος τόνος δέρματος',
    '🙋🏻' => 'άτομο με σηκωμένο χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '🙋🏼' => 'άτομο με σηκωμένο χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙋🏽' => 'άτομο με σηκωμένο χέρι: μεσαίος τόνος δέρματος',
    '🙋🏾' => 'άτομο με σηκωμένο χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '🙋🏿' => 'άτομο με σηκωμένο χέρι: σκούρος τόνος δέρματος',
    '🧏🏻' => 'κωφό άτομο: ανοιχτόχρωμος τόνος δέρματος',
    '🧏🏼' => 'κωφό άτομο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧏🏽' => 'κωφό άτομο: μεσαίος τόνος δέρματος',
    '🧏🏾' => 'κωφό άτομο: μεσαίος-σκούρος τόνος δέρματος',
    '🧏🏿' => 'κωφό άτομο: σκούρος τόνος δέρματος',
    '🙇🏻' => 'άτομο υποκλίνεται: ανοιχτόχρωμος τόνος δέρματος',
    '🙇🏼' => 'άτομο υποκλίνεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🙇🏽' => 'άτομο υποκλίνεται: μεσαίος τόνος δέρματος',
    '🙇🏾' => 'άτομο υποκλίνεται: μεσαίος-σκούρος τόνος δέρματος',
    '🙇🏿' => 'άτομο υποκλίνεται: σκούρος τόνος δέρματος',
    '🤦🏻' => 'άτομο αγανακτά: ανοιχτόχρωμος τόνος δέρματος',
    '🤦🏼' => 'άτομο αγανακτά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤦🏽' => 'άτομο αγανακτά: μεσαίος τόνος δέρματος',
    '🤦🏾' => 'άτομο αγανακτά: μεσαίος-σκούρος τόνος δέρματος',
    '🤦🏿' => 'άτομο αγανακτά: σκούρος τόνος δέρματος',
    '🤷🏻' => 'άτομο σηκώνει τους ώμους: ανοιχτόχρωμος τόνος δέρματος',
    '🤷🏼' => 'άτομο σηκώνει τους ώμους: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤷🏽' => 'άτομο σηκώνει τους ώμους: μεσαίος τόνος δέρματος',
    '🤷🏾' => 'άτομο σηκώνει τους ώμους: μεσαίος-σκούρος τόνος δέρματος',
    '🤷🏿' => 'άτομο σηκώνει τους ώμους: σκούρος τόνος δέρματος',
    '👮🏻' => 'αστυνομικός: ανοιχτόχρωμος τόνος δέρματος',
    '👮🏼' => 'αστυνομικός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👮🏽' => 'αστυνομικός: μεσαίος τόνος δέρματος',
    '👮🏾' => 'αστυνομικός: μεσαίος-σκούρος τόνος δέρματος',
    '👮🏿' => 'αστυνομικός: σκούρος τόνος δέρματος',
    '🕵🏻' => 'ντετέκτιβ: ανοιχτόχρωμος τόνος δέρματος',
    '🕵🏼' => 'ντετέκτιβ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🕵🏽' => 'ντετέκτιβ: μεσαίος τόνος δέρματος',
    '🕵🏾' => 'ντετέκτιβ: μεσαίος-σκούρος τόνος δέρματος',
    '🕵🏿' => 'ντετέκτιβ: σκούρος τόνος δέρματος',
    '💂🏻' => 'υπεύθυνος φρουράς: ανοιχτόχρωμος τόνος δέρματος',
    '💂🏼' => 'υπεύθυνος φρουράς: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💂🏽' => 'υπεύθυνος φρουράς: μεσαίος τόνος δέρματος',
    '💂🏾' => 'υπεύθυνος φρουράς: μεσαίος-σκούρος τόνος δέρματος',
    '💂🏿' => 'υπεύθυνος φρουράς: σκούρος τόνος δέρματος',
    '🥷🏻' => 'νίντζα: ανοιχτόχρωμος τόνος δέρματος',
    '🥷🏼' => 'νίντζα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🥷🏽' => 'νίντζα: μεσαίος τόνος δέρματος',
    '🥷🏾' => 'νίντζα: μεσαίος-σκούρος τόνος δέρματος',
    '🥷🏿' => 'νίντζα: σκούρος τόνος δέρματος',
    '👷🏻' => 'οικοδόμος: ανοιχτόχρωμος τόνος δέρματος',
    '👷🏼' => 'οικοδόμος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👷🏽' => 'οικοδόμος: μεσαίος τόνος δέρματος',
    '👷🏾' => 'οικοδόμος: μεσαίος-σκούρος τόνος δέρματος',
    '👷🏿' => 'οικοδόμος: σκούρος τόνος δέρματος',
    '🫅🏻' => 'άτομο με στέμμα: ανοιχτόχρωμος τόνος δέρματος',
    '🫅🏼' => 'άτομο με στέμμα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫅🏽' => 'άτομο με στέμμα: μεσαίος τόνος δέρματος',
    '🫅🏾' => 'άτομο με στέμμα: μεσαίος-σκούρος τόνος δέρματος',
    '🫅🏿' => 'άτομο με στέμμα: σκούρος τόνος δέρματος',
    '🤴🏻' => 'πρίγκιπας: ανοιχτόχρωμος τόνος δέρματος',
    '🤴🏼' => 'πρίγκιπας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤴🏽' => 'πρίγκιπας: μεσαίος τόνος δέρματος',
    '🤴🏾' => 'πρίγκιπας: μεσαίος-σκούρος τόνος δέρματος',
    '🤴🏿' => 'πρίγκιπας: σκούρος τόνος δέρματος',
    '👸🏻' => 'πριγκίπισσα: ανοιχτόχρωμος τόνος δέρματος',
    '👸🏼' => 'πριγκίπισσα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👸🏽' => 'πριγκίπισσα: μεσαίος τόνος δέρματος',
    '👸🏾' => 'πριγκίπισσα: μεσαίος-σκούρος τόνος δέρματος',
    '👸🏿' => 'πριγκίπισσα: σκούρος τόνος δέρματος',
    '👳🏻' => 'άτομο με τουρμπάνι: ανοιχτόχρωμος τόνος δέρματος',
    '👳🏼' => 'άτομο με τουρμπάνι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👳🏽' => 'άτομο με τουρμπάνι: μεσαίος τόνος δέρματος',
    '👳🏾' => 'άτομο με τουρμπάνι: μεσαίος-σκούρος τόνος δέρματος',
    '👳🏿' => 'άτομο με τουρμπάνι: σκούρος τόνος δέρματος',
    '👲🏻' => 'άντρας με κινέζικο καπέλο: ανοιχτόχρωμος τόνος δέρματος',
    '👲🏼' => 'άντρας με κινέζικο καπέλο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👲🏽' => 'άντρας με κινέζικο καπέλο: μεσαίος τόνος δέρματος',
    '👲🏾' => 'άντρας με κινέζικο καπέλο: μεσαίος-σκούρος τόνος δέρματος',
    '👲🏿' => 'άντρας με κινέζικο καπέλο: σκούρος τόνος δέρματος',
    '🧕🏻' => 'γυναίκα με μαντίλα: ανοιχτόχρωμος τόνος δέρματος',
    '🧕🏼' => 'γυναίκα με μαντίλα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧕🏽' => 'γυναίκα με μαντίλα: μεσαίος τόνος δέρματος',
    '🧕🏾' => 'γυναίκα με μαντίλα: μεσαίος-σκούρος τόνος δέρματος',
    '🧕🏿' => 'γυναίκα με μαντίλα: σκούρος τόνος δέρματος',
    '🤵🏻' => 'άτομο με σμόκιν: ανοιχτόχρωμος τόνος δέρματος',
    '🤵🏼' => 'άτομο με σμόκιν: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤵🏽' => 'άτομο με σμόκιν: μεσαίος τόνος δέρματος',
    '🤵🏾' => 'άτομο με σμόκιν: μεσαίος-σκούρος τόνος δέρματος',
    '🤵🏿' => 'άτομο με σμόκιν: σκούρος τόνος δέρματος',
    '👰🏻' => 'άτομο με πέπλο: ανοιχτόχρωμος τόνος δέρματος',
    '👰🏼' => 'άτομο με πέπλο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👰🏽' => 'άτομο με πέπλο: μεσαίος τόνος δέρματος',
    '👰🏾' => 'άτομο με πέπλο: μεσαίος-σκούρος τόνος δέρματος',
    '👰🏿' => 'άτομο με πέπλο: σκούρος τόνος δέρματος',
    '🤰🏻' => 'έγκυος: ανοιχτόχρωμος τόνος δέρματος',
    '🤰🏼' => 'έγκυος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤰🏽' => 'έγκυος: μεσαίος τόνος δέρματος',
    '🤰🏾' => 'έγκυος: μεσαίος-σκούρος τόνος δέρματος',
    '🤰🏿' => 'έγκυος: σκούρος τόνος δέρματος',
    '🫃🏻' => 'έγκυος άντρας: ανοιχτόχρωμος τόνος δέρματος',
    '🫃🏼' => 'έγκυος άντρας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫃🏽' => 'έγκυος άντρας: μεσαίος τόνος δέρματος',
    '🫃🏾' => 'έγκυος άντρας: μεσαίος-σκούρος τόνος δέρματος',
    '🫃🏿' => 'έγκυος άντρας: σκούρος τόνος δέρματος',
    '🫄🏻' => 'έγκυο άτομο: ανοιχτόχρωμος τόνος δέρματος',
    '🫄🏼' => 'έγκυο άτομο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🫄🏽' => 'έγκυο άτομο: μεσαίος τόνος δέρματος',
    '🫄🏾' => 'έγκυο άτομο: μεσαίος-σκούρος τόνος δέρματος',
    '🫄🏿' => 'έγκυο άτομο: σκούρος τόνος δέρματος',
    '🤱🏻' => 'θηλασμός: ανοιχτόχρωμος τόνος δέρματος',
    '🤱🏼' => 'θηλασμός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤱🏽' => 'θηλασμός: μεσαίος τόνος δέρματος',
    '🤱🏾' => 'θηλασμός: μεσαίος-σκούρος τόνος δέρματος',
    '🤱🏿' => 'θηλασμός: σκούρος τόνος δέρματος',
    '👼🏻' => 'μωρό άγγελος: ανοιχτόχρωμος τόνος δέρματος',
    '👼🏼' => 'μωρό άγγελος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👼🏽' => 'μωρό άγγελος: μεσαίος τόνος δέρματος',
    '👼🏾' => 'μωρό άγγελος: μεσαίος-σκούρος τόνος δέρματος',
    '👼🏿' => 'μωρό άγγελος: σκούρος τόνος δέρματος',
    '🎅🏻' => 'Άι Βασίλης: ανοιχτόχρωμος τόνος δέρματος',
    '🎅🏼' => 'Άι Βασίλης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🎅🏽' => 'Άι Βασίλης: μεσαίος τόνος δέρματος',
    '🎅🏾' => 'Άι Βασίλης: μεσαίος-σκούρος τόνος δέρματος',
    '🎅🏿' => 'Άι Βασίλης: σκούρος τόνος δέρματος',
    '🤶🏻' => 'Κυρία Άι Βασίλη: ανοιχτόχρωμος τόνος δέρματος',
    '🤶🏼' => 'Κυρία Άι Βασίλη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤶🏽' => 'Κυρία Άι Βασίλη: μεσαίος τόνος δέρματος',
    '🤶🏾' => 'Κυρία Άι Βασίλη: μεσαίος-σκούρος τόνος δέρματος',
    '🤶🏿' => 'Κυρία Άι Βασίλη: σκούρος τόνος δέρματος',
    '🦸🏻' => 'υπερήρωας: ανοιχτόχρωμος τόνος δέρματος',
    '🦸🏼' => 'υπερήρωας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦸🏽' => 'υπερήρωας: μεσαίος τόνος δέρματος',
    '🦸🏾' => 'υπερήρωας: μεσαίος-σκούρος τόνος δέρματος',
    '🦸🏿' => 'υπερήρωας: σκούρος τόνος δέρματος',
    '🦹🏻' => 'αρχικακός: ανοιχτόχρωμος τόνος δέρματος',
    '🦹🏼' => 'αρχικακός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🦹🏽' => 'αρχικακός: μεσαίος τόνος δέρματος',
    '🦹🏾' => 'αρχικακός: μεσαίος-σκούρος τόνος δέρματος',
    '🦹🏿' => 'αρχικακός: σκούρος τόνος δέρματος',
    '🧙🏻' => 'μάγος: ανοιχτόχρωμος τόνος δέρματος',
    '🧙🏼' => 'μάγος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧙🏽' => 'μάγος: μεσαίος τόνος δέρματος',
    '🧙🏾' => 'μάγος: μεσαίος-σκούρος τόνος δέρματος',
    '🧙🏿' => 'μάγος: σκούρος τόνος δέρματος',
    '🧚🏻' => 'νεράιδα: ανοιχτόχρωμος τόνος δέρματος',
    '🧚🏼' => 'νεράιδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧚🏽' => 'νεράιδα: μεσαίος τόνος δέρματος',
    '🧚🏾' => 'νεράιδα: μεσαίος-σκούρος τόνος δέρματος',
    '🧚🏿' => 'νεράιδα: σκούρος τόνος δέρματος',
    '🧛🏻' => 'βρυκόλακας: ανοιχτόχρωμος τόνος δέρματος',
    '🧛🏼' => 'βρυκόλακας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧛🏽' => 'βρυκόλακας: μεσαίος τόνος δέρματος',
    '🧛🏾' => 'βρυκόλακας: μεσαίος-σκούρος τόνος δέρματος',
    '🧛🏿' => 'βρυκόλακας: σκούρος τόνος δέρματος',
    '🧜🏻' => 'γοργόνα: ανοιχτόχρωμος τόνος δέρματος',
    '🧜🏼' => 'γοργόνα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧜🏽' => 'γοργόνα: μεσαίος τόνος δέρματος',
    '🧜🏾' => 'γοργόνα: μεσαίος-σκούρος τόνος δέρματος',
    '🧜🏿' => 'γοργόνα: σκούρος τόνος δέρματος',
    '🧝🏻' => 'ξωτικό: ανοιχτόχρωμος τόνος δέρματος',
    '🧝🏼' => 'ξωτικό: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧝🏽' => 'ξωτικό: μεσαίος τόνος δέρματος',
    '🧝🏾' => 'ξωτικό: μεσαίος-σκούρος τόνος δέρματος',
    '🧝🏿' => 'ξωτικό: σκούρος τόνος δέρματος',
    '💆🏻' => 'μασάζ στο κεφάλι: ανοιχτόχρωμος τόνος δέρματος',
    '💆🏼' => 'μασάζ στο κεφάλι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💆🏽' => 'μασάζ στο κεφάλι: μεσαίος τόνος δέρματος',
    '💆🏾' => 'μασάζ στο κεφάλι: μεσαίος-σκούρος τόνος δέρματος',
    '💆🏿' => 'μασάζ στο κεφάλι: σκούρος τόνος δέρματος',
    '💇🏻' => 'κούρεμα: ανοιχτόχρωμος τόνος δέρματος',
    '💇🏼' => 'κούρεμα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💇🏽' => 'κούρεμα: μεσαίος τόνος δέρματος',
    '💇🏾' => 'κούρεμα: μεσαίος-σκούρος τόνος δέρματος',
    '💇🏿' => 'κούρεμα: σκούρος τόνος δέρματος',
    '🚶🏻' => 'πεζός: ανοιχτόχρωμος τόνος δέρματος',
    '🚶🏼' => 'πεζός: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚶🏽' => 'πεζός: μεσαίος τόνος δέρματος',
    '🚶🏾' => 'πεζός: μεσαίος-σκούρος τόνος δέρματος',
    '🚶🏿' => 'πεζός: σκούρος τόνος δέρματος',
    '🧍🏻' => 'άτομο που στέκεται: ανοιχτόχρωμος τόνος δέρματος',
    '🧍🏼' => 'άτομο που στέκεται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧍🏽' => 'άτομο που στέκεται: μεσαίος τόνος δέρματος',
    '🧍🏾' => 'άτομο που στέκεται: μεσαίος-σκούρος τόνος δέρματος',
    '🧍🏿' => 'άτομο που στέκεται: σκούρος τόνος δέρματος',
    '🧎🏻' => 'άτομο που γονατίζει: ανοιχτόχρωμος τόνος δέρματος',
    '🧎🏼' => 'άτομο που γονατίζει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧎🏽' => 'άτομο που γονατίζει: μεσαίος τόνος δέρματος',
    '🧎🏾' => 'άτομο που γονατίζει: μεσαίος-σκούρος τόνος δέρματος',
    '🧎🏿' => 'άτομο που γονατίζει: σκούρος τόνος δέρματος',
    '🏃🏻' => 'άτομο τρέχει: ανοιχτόχρωμος τόνος δέρματος',
    '🏃🏼' => 'άτομο τρέχει: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏃🏽' => 'άτομο τρέχει: μεσαίος τόνος δέρματος',
    '🏃🏾' => 'άτομο τρέχει: μεσαίος-σκούρος τόνος δέρματος',
    '🏃🏿' => 'άτομο τρέχει: σκούρος τόνος δέρματος',
    '💃🏻' => 'χορεύτρια: ανοιχτόχρωμος τόνος δέρματος',
    '💃🏼' => 'χορεύτρια: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💃🏽' => 'χορεύτρια: μεσαίος τόνος δέρματος',
    '💃🏾' => 'χορεύτρια: μεσαίος-σκούρος τόνος δέρματος',
    '💃🏿' => 'χορεύτρια: σκούρος τόνος δέρματος',
    '🕺🏻' => 'χορευτής: ανοιχτόχρωμος τόνος δέρματος',
    '🕺🏼' => 'χορευτής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🕺🏽' => 'χορευτής: μεσαίος τόνος δέρματος',
    '🕺🏾' => 'χορευτής: μεσαίος-σκούρος τόνος δέρματος',
    '🕺🏿' => 'χορευτής: σκούρος τόνος δέρματος',
    '🕴🏻' => 'κοστουμαρισμένος άντρας που αιωρείται: ανοιχτόχρωμος τόνος δέρματος',
    '🕴🏼' => 'κοστουμαρισμένος άντρας που αιωρείται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🕴🏽' => 'κοστουμαρισμένος άντρας που αιωρείται: μεσαίος τόνος δέρματος',
    '🕴🏾' => 'κοστουμαρισμένος άντρας που αιωρείται: μεσαίος-σκούρος τόνος δέρματος',
    '🕴🏿' => 'κοστουμαρισμένος άντρας που αιωρείται: σκούρος τόνος δέρματος',
    '👯🏻' => 'άτομα με λαγουδένια αυτιά: ανοιχτόχρωμος τόνος δέρματος',
    '👯🏼' => 'άτομα με λαγουδένια αυτιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👯🏽' => 'άτομα με λαγουδένια αυτιά: μεσαίος τόνος δέρματος',
    '👯🏾' => 'άτομα με λαγουδένια αυτιά: μεσαίος-σκούρος τόνος δέρματος',
    '👯🏿' => 'άτομα με λαγουδένια αυτιά: σκούρος τόνος δέρματος',
    '🧖🏻' => 'άτομο σε σάουνα: ανοιχτόχρωμος τόνος δέρματος',
    '🧖🏼' => 'άτομο σε σάουνα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧖🏽' => 'άτομο σε σάουνα: μεσαίος τόνος δέρματος',
    '🧖🏾' => 'άτομο σε σάουνα: μεσαίος-σκούρος τόνος δέρματος',
    '🧖🏿' => 'άτομο σε σάουνα: σκούρος τόνος δέρματος',
    '🧗🏻' => 'άτομο που αναρριχείται: ανοιχτόχρωμος τόνος δέρματος',
    '🧗🏼' => 'άτομο που αναρριχείται: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧗🏽' => 'άτομο που αναρριχείται: μεσαίος τόνος δέρματος',
    '🧗🏾' => 'άτομο που αναρριχείται: μεσαίος-σκούρος τόνος δέρματος',
    '🧗🏿' => 'άτομο που αναρριχείται: σκούρος τόνος δέρματος',
    '🏇🏻' => 'ιππόδρομος: ανοιχτόχρωμος τόνος δέρματος',
    '🏇🏼' => 'ιππόδρομος: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏇🏽' => 'ιππόδρομος: μεσαίος τόνος δέρματος',
    '🏇🏾' => 'ιππόδρομος: μεσαίος-σκούρος τόνος δέρματος',
    '🏇🏿' => 'ιππόδρομος: σκούρος τόνος δέρματος',
    '🏂🏻' => 'σνόουμπορντ: ανοιχτόχρωμος τόνος δέρματος',
    '🏂🏼' => 'σνόουμπορντ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏂🏽' => 'σνόουμπορντ: μεσαίος τόνος δέρματος',
    '🏂🏾' => 'σνόουμπορντ: μεσαίος-σκούρος τόνος δέρματος',
    '🏂🏿' => 'σνόουμπορντ: σκούρος τόνος δέρματος',
    '🏌🏻' => 'γκόλφερ: ανοιχτόχρωμος τόνος δέρματος',
    '🏌🏼' => 'γκόλφερ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏌🏽' => 'γκόλφερ: μεσαίος τόνος δέρματος',
    '🏌🏾' => 'γκόλφερ: μεσαίος-σκούρος τόνος δέρματος',
    '🏌🏿' => 'γκόλφερ: σκούρος τόνος δέρματος',
    '🏄🏻' => 'σέρφερ: ανοιχτόχρωμος τόνος δέρματος',
    '🏄🏼' => 'σέρφερ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏄🏽' => 'σέρφερ: μεσαίος τόνος δέρματος',
    '🏄🏾' => 'σέρφερ: μεσαίος-σκούρος τόνος δέρματος',
    '🏄🏿' => 'σέρφερ: σκούρος τόνος δέρματος',
    '🚣🏻' => 'κωπηλάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🚣🏼' => 'κωπηλάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚣🏽' => 'κωπηλάτης: μεσαίος τόνος δέρματος',
    '🚣🏾' => 'κωπηλάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🚣🏿' => 'κωπηλάτης: σκούρος τόνος δέρματος',
    '🏊🏻' => 'κολυμβητής: ανοιχτόχρωμος τόνος δέρματος',
    '🏊🏼' => 'κολυμβητής: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏊🏽' => 'κολυμβητής: μεσαίος τόνος δέρματος',
    '🏊🏾' => 'κολυμβητής: μεσαίος-σκούρος τόνος δέρματος',
    '🏊🏿' => 'κολυμβητής: σκούρος τόνος δέρματος',
    '⛹🏻' => 'άτομο παίζει μπάσκετ: ανοιχτόχρωμος τόνος δέρματος',
    '⛹🏼' => 'άτομο παίζει μπάσκετ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '⛹🏽' => 'άτομο παίζει μπάσκετ: μεσαίος τόνος δέρματος',
    '⛹🏾' => 'άτομο παίζει μπάσκετ: μεσαίος-σκούρος τόνος δέρματος',
    '⛹🏿' => 'άτομο παίζει μπάσκετ: σκούρος τόνος δέρματος',
    '🏋🏻' => 'αρσιβαρίστας: ανοιχτόχρωμος τόνος δέρματος',
    '🏋🏼' => 'αρσιβαρίστας: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏋🏽' => 'αρσιβαρίστας: μεσαίος τόνος δέρματος',
    '🏋🏾' => 'αρσιβαρίστας: μεσαίος-σκούρος τόνος δέρματος',
    '🏋🏿' => 'αρσιβαρίστας: σκούρος τόνος δέρματος',
    '🚴🏻' => 'ποδηλάτης: ανοιχτόχρωμος τόνος δέρματος',
    '🚴🏼' => 'ποδηλάτης: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚴🏽' => 'ποδηλάτης: μεσαίος τόνος δέρματος',
    '🚴🏾' => 'ποδηλάτης: μεσαίος-σκούρος τόνος δέρματος',
    '🚴🏿' => 'ποδηλάτης: σκούρος τόνος δέρματος',
    '🚵🏻' => 'ορεινή ποδηλασία: ανοιχτόχρωμος τόνος δέρματος',
    '🚵🏼' => 'ορεινή ποδηλασία: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🚵🏽' => 'ορεινή ποδηλασία: μεσαίος τόνος δέρματος',
    '🚵🏾' => 'ορεινή ποδηλασία: μεσαίος-σκούρος τόνος δέρματος',
    '🚵🏿' => 'ορεινή ποδηλασία: σκούρος τόνος δέρματος',
    '🤸🏻' => 'άτομο κάνει ρόδα: ανοιχτόχρωμος τόνος δέρματος',
    '🤸🏼' => 'άτομο κάνει ρόδα: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤸🏽' => 'άτομο κάνει ρόδα: μεσαίος τόνος δέρματος',
    '🤸🏾' => 'άτομο κάνει ρόδα: μεσαίος-σκούρος τόνος δέρματος',
    '🤸🏿' => 'άτομο κάνει ρόδα: σκούρος τόνος δέρματος',
    '🤼🏻' => 'πάλη: ανοιχτόχρωμος τόνος δέρματος',
    '🤼🏼' => 'πάλη: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤼🏽' => 'πάλη: μεσαίος τόνος δέρματος',
    '🤼🏾' => 'πάλη: μεσαίος-σκούρος τόνος δέρματος',
    '🤼🏿' => 'πάλη: σκούρος τόνος δέρματος',
    '🤽🏻' => 'πόλο: ανοιχτόχρωμος τόνος δέρματος',
    '🤽🏼' => 'πόλο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤽🏽' => 'πόλο: μεσαίος τόνος δέρματος',
    '🤽🏾' => 'πόλο: μεσαίος-σκούρος τόνος δέρματος',
    '🤽🏿' => 'πόλο: σκούρος τόνος δέρματος',
    '🤾🏻' => 'χάντμπολ: ανοιχτόχρωμος τόνος δέρματος',
    '🤾🏼' => 'χάντμπολ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤾🏽' => 'χάντμπολ: μεσαίος τόνος δέρματος',
    '🤾🏾' => 'χάντμπολ: μεσαίος-σκούρος τόνος δέρματος',
    '🤾🏿' => 'χάντμπολ: σκούρος τόνος δέρματος',
    '🤹🏻' => 'ζογκλέρ: ανοιχτόχρωμος τόνος δέρματος',
    '🤹🏼' => 'ζογκλέρ: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🤹🏽' => 'ζογκλέρ: μεσαίος τόνος δέρματος',
    '🤹🏾' => 'ζογκλέρ: μεσαίος-σκούρος τόνος δέρματος',
    '🤹🏿' => 'ζογκλέρ: σκούρος τόνος δέρματος',
    '🧘🏻' => 'άτομο σε θέση λωτού: ανοιχτόχρωμος τόνος δέρματος',
    '🧘🏼' => 'άτομο σε θέση λωτού: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🧘🏽' => 'άτομο σε θέση λωτού: μεσαίος τόνος δέρματος',
    '🧘🏾' => 'άτομο σε θέση λωτού: μεσαίος-σκούρος τόνος δέρματος',
    '🧘🏿' => 'άτομο σε θέση λωτού: σκούρος τόνος δέρματος',
    '🛀🏻' => 'άνθρωπος που κάνει μπάνιο: ανοιχτόχρωμος τόνος δέρματος',
    '🛀🏼' => 'άνθρωπος που κάνει μπάνιο: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🛀🏽' => 'άνθρωπος που κάνει μπάνιο: μεσαίος τόνος δέρματος',
    '🛀🏾' => 'άνθρωπος που κάνει μπάνιο: μεσαίος-σκούρος τόνος δέρματος',
    '🛀🏿' => 'άνθρωπος που κάνει μπάνιο: σκούρος τόνος δέρματος',
    '🛌🏻' => 'άνθρωπος σε κρεβάτι: ανοιχτόχρωμος τόνος δέρματος',
    '🛌🏼' => 'άνθρωπος σε κρεβάτι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🛌🏽' => 'άνθρωπος σε κρεβάτι: μεσαίος τόνος δέρματος',
    '🛌🏾' => 'άνθρωπος σε κρεβάτι: μεσαίος-σκούρος τόνος δέρματος',
    '🛌🏿' => 'άνθρωπος σε κρεβάτι: σκούρος τόνος δέρματος',
    '👭🏻' => 'γυναίκες πιασμένες χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '👭🏼' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👭🏽' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος τόνος δέρματος',
    '👭🏾' => 'γυναίκες πιασμένες χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '👭🏿' => 'γυναίκες πιασμένες χέρι-χέρι: σκούρος τόνος δέρματος',
    '👫🏻' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '👫🏼' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👫🏽' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος',
    '👫🏾' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '👫🏿' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος',
    '👬🏻' => 'άντρες πιασμένοι χέρι-χέρι: ανοιχτόχρωμος τόνος δέρματος',
    '👬🏼' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '👬🏽' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος τόνος δέρματος',
    '👬🏾' => 'άντρες πιασμένοι χέρι-χέρι: μεσαίος-σκούρος τόνος δέρματος',
    '👬🏿' => 'άντρες πιασμένοι χέρι-χέρι: σκούρος τόνος δέρματος',
    '💏🏻' => 'φιλί: ανοιχτόχρωμος τόνος δέρματος',
    '💏🏼' => 'φιλί: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💏🏽' => 'φιλί: μεσαίος τόνος δέρματος',
    '💏🏾' => 'φιλί: μεσαίος-σκούρος τόνος δέρματος',
    '💏🏿' => 'φιλί: σκούρος τόνος δέρματος',
    '💑🏻' => 'ζευγάρι με καρδιά: ανοιχτόχρωμος τόνος δέρματος',
    '💑🏼' => 'ζευγάρι με καρδιά: μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '💑🏽' => 'ζευγάρι με καρδιά: μεσαίος τόνος δέρματος',
    '💑🏾' => 'ζευγάρι με καρδιά: μεσαίος-σκούρος τόνος δέρματος',
    '💑🏿' => 'ζευγάρι με καρδιά: σκούρος τόνος δέρματος',
    '🇦🇨' => 'σημαία: Νήσος Ασενσιόν',
    '🇦🇩' => 'σημαία: Ανδόρα',
    '🇦🇪' => 'σημαία: Ηνωμένα Αραβικά Εμιράτα',
    '🇦🇫' => 'σημαία: Αφγανιστάν',
    '🇦🇬' => 'σημαία: Αντίγκουα και Μπαρμπούντα',
    '🇦🇮' => 'σημαία: Ανγκουίλα',
    '🇦🇱' => 'σημαία: Αλβανία',
    '🇦🇲' => 'σημαία: Αρμενία',
    '🇦🇴' => 'σημαία: Αγκόλα',
    '🇦🇶' => 'σημαία: Ανταρκτική',
    '🇦🇷' => 'σημαία: Αργεντινή',
    '🇦🇸' => 'σημαία: Αμερικανική Σαμόα',
    '🇦🇹' => 'σημαία: Αυστρία',
    '🇦🇺' => 'σημαία: Αυστραλία',
    '🇦🇼' => 'σημαία: Αρούμπα',
    '🇦🇽' => 'σημαία: Νήσοι Όλαντ',
    '🇦🇿' => 'σημαία: Αζερμπαϊτζάν',
    '🇧🇦' => 'σημαία: Βοσνία - Ερζεγοβίνη',
    '🇧🇧' => 'σημαία: Μπαρμπέιντος',
    '🇧🇩' => 'σημαία: Μπανγκλαντές',
    '🇧🇪' => 'σημαία: Βέλγιο',
    '🇧🇫' => 'σημαία: Μπουρκίνα Φάσο',
    '🇧🇬' => 'σημαία: Βουλγαρία',
    '🇧🇭' => 'σημαία: Μπαχρέιν',
    '🇧🇮' => 'σημαία: Μπουρούντι',
    '🇧🇯' => 'σημαία: Μπενίν',
    '🇧🇱' => 'σημαία: Άγιος Βαρθολομαίος',
    '🇧🇲' => 'σημαία: Βερμούδες',
    '🇧🇳' => 'σημαία: Μπρουνέι',
    '🇧🇴' => 'σημαία: Βολιβία',
    '🇧🇶' => 'σημαία: Ολλανδία Καραϊβικής',
    '🇧🇷' => 'σημαία: Βραζιλία',
    '🇧🇸' => 'σημαία: Μπαχάμες',
    '🇧🇹' => 'σημαία: Μπουτάν',
    '🇧🇻' => 'σημαία: Νήσος Μπουβέ',
    '🇧🇼' => 'σημαία: Μποτσουάνα',
    '🇧🇾' => 'σημαία: Λευκορωσία',
    '🇧🇿' => 'σημαία: Μπελίζ',
    '🇨🇦' => 'σημαία: Καναδάς',
    '🇨🇨' => 'σημαία: Νήσοι Κόκος (Κίλινγκ)',
    '🇨🇩' => 'σημαία: Κονγκό - Κινσάσα',
    '🇨🇫' => 'σημαία: Κεντροαφρικανική Δημοκρατία',
    '🇨🇬' => 'σημαία: Κονγκό - Μπραζαβίλ',
    '🇨🇭' => 'σημαία: Ελβετία',
    '🇨🇮' => 'σημαία: Ακτή Ελεφαντοστού',
    '🇨🇰' => 'σημαία: Νήσοι Κουκ',
    '🇨🇱' => 'σημαία: Χιλή',
    '🇨🇲' => 'σημαία: Καμερούν',
    '🇨🇳' => 'σημαία: Κίνα',
    '🇨🇴' => 'σημαία: Κολομβία',
    '🇨🇵' => 'σημαία: Νήσος Κλίπερτον',
    '🇨🇶' => 'σημαία: Σαρκ',
    '🇨🇷' => 'σημαία: Κόστα Ρίκα',
    '🇨🇺' => 'σημαία: Κούβα',
    '🇨🇻' => 'σημαία: Πράσινο Ακρωτήριο',
    '🇨🇼' => 'σημαία: Κουρασάο',
    '🇨🇽' => 'σημαία: Νήσος των Χριστουγέννων',
    '🇨🇾' => 'σημαία: Κύπρος',
    '🇨🇿' => 'σημαία: Τσεχία',
    '🇩🇪' => 'σημαία: Γερμανία',
    '🇩🇬' => 'σημαία: Ντιέγκο Γκαρσία',
    '🇩🇯' => 'σημαία: Τζιμπουτί',
    '🇩🇰' => 'σημαία: Δανία',
    '🇩🇲' => 'σημαία: Ντομίνικα',
    '🇩🇴' => 'σημαία: Δομινικανή Δημοκρατία',
    '🇩🇿' => 'σημαία: Αλγερία',
    '🇪🇦' => 'σημαία: Θέουτα και Μελίγια',
    '🇪🇨' => 'σημαία: Ισημερινός',
    '🇪🇪' => 'σημαία: Εσθονία',
    '🇪🇬' => 'σημαία: Αίγυπτος',
    '🇪🇭' => 'σημαία: Δυτική Σαχάρα',
    '🇪🇷' => 'σημαία: Ερυθραία',
    '🇪🇸' => 'σημαία: Ισπανία',
    '🇪🇹' => 'σημαία: Αιθιοπία',
    '🇪🇺' => 'σημαία: Ευρωπαϊκή Ένωση',
    '🇫🇮' => 'σημαία: Φινλανδία',
    '🇫🇯' => 'σημαία: Φίτζι',
    '🇫🇰' => 'σημαία: Νήσοι Φόκλαντ',
    '🇫🇲' => 'σημαία: Μικρονησία',
    '🇫🇴' => 'σημαία: Νήσοι Φερόες',
    '🇫🇷' => 'σημαία: Γαλλία',
    '🇬🇦' => 'σημαία: Γκαμπόν',
    '🇬🇧' => 'σημαία: Ηνωμένο Βασίλειο',
    '🇬🇩' => 'σημαία: Γρενάδα',
    '🇬🇪' => 'σημαία: Γεωργία',
    '🇬🇫' => 'σημαία: Γαλλική Γουιάνα',
    '🇬🇬' => 'σημαία: Γκέρνζι',
    '🇬🇭' => 'σημαία: Γκάνα',
    '🇬🇮' => 'σημαία: Γιβραλτάρ',
    '🇬🇱' => 'σημαία: Γροιλανδία',
    '🇬🇲' => 'σημαία: Γκάμπια',
    '🇬🇳' => 'σημαία: Γουινέα',
    '🇬🇵' => 'σημαία: Γουαδελούπη',
    '🇬🇶' => 'σημαία: Ισημερινή Γουινέα',
    '🇬🇷' => 'σημαία: Ελλάδα',
    '🇬🇸' => 'σημαία: Νήσοι Νότια Γεωργία και Νότιες Σάντουιτς',
    '🇬🇹' => 'σημαία: Γουατεμάλα',
    '🇬🇺' => 'σημαία: Γκουάμ',
    '🇬🇼' => 'σημαία: Γουινέα Μπισάου',
    '🇬🇾' => 'σημαία: Γουιάνα',
    '🇭🇰' => 'σημαία: Χονγκ Κονγκ ΕΔΠ Κίνας',
    '🇭🇲' => 'σημαία: Νήσοι Χερντ και Μακντόναλντ',
    '🇭🇳' => 'σημαία: Ονδούρα',
    '🇭🇷' => 'σημαία: Κροατία',
    '🇭🇹' => 'σημαία: Αϊτή',
    '🇭🇺' => 'σημαία: Ουγγαρία',
    '🇮🇨' => 'σημαία: Κανάριοι Νήσοι',
    '🇮🇩' => 'σημαία: Ινδονησία',
    '🇮🇪' => 'σημαία: Ιρλανδία',
    '🇮🇱' => 'σημαία: Ισραήλ',
    '🇮🇲' => 'σημαία: Νήσος του Μαν',
    '🇮🇳' => 'σημαία: Ινδία',
    '🇮🇴' => 'σημαία: Βρετανικά Εδάφη Ινδικού Ωκεανού',
    '🇮🇶' => 'σημαία: Ιράκ',
    '🇮🇷' => 'σημαία: Ιράν',
    '🇮🇸' => 'σημαία: Ισλανδία',
    '🇮🇹' => 'σημαία: Ιταλία',
    '🇯🇪' => 'σημαία: Τζέρζι',
    '🇯🇲' => 'σημαία: Τζαμάικα',
    '🇯🇴' => 'σημαία: Ιορδανία',
    '🇯🇵' => 'σημαία: Ιαπωνία',
    '🇰🇪' => 'σημαία: Κένυα',
    '🇰🇬' => 'σημαία: Κιργιστάν',
    '🇰🇭' => 'σημαία: Καμπότζη',
    '🇰🇮' => 'σημαία: Κιριμπάτι',
    '🇰🇲' => 'σημαία: Κομόρες',
    '🇰🇳' => 'σημαία: Σεν Κιτς και Νέβις',
    '🇰🇵' => 'σημαία: Βόρεια Κορέα',
    '🇰🇷' => 'σημαία: Νότια Κορέα',
    '🇰🇼' => 'σημαία: Κουβέιτ',
    '🇰🇾' => 'σημαία: Νήσοι Κέιμαν',
    '🇰🇿' => 'σημαία: Καζακστάν',
    '🇱🇦' => 'σημαία: Λάος',
    '🇱🇧' => 'σημαία: Λίβανος',
    '🇱🇨' => 'σημαία: Αγία Λουκία',
    '🇱🇮' => 'σημαία: Λιχτενστάιν',
    '🇱🇰' => 'σημαία: Σρι Λάνκα',
    '🇱🇷' => 'σημαία: Λιβερία',
    '🇱🇸' => 'σημαία: Λεσότο',
    '🇱🇹' => 'σημαία: Λιθουανία',
    '🇱🇺' => 'σημαία: Λουξεμβούργο',
    '🇱🇻' => 'σημαία: Λετονία',
    '🇱🇾' => 'σημαία: Λιβύη',
    '🇲🇦' => 'σημαία: Μαρόκο',
    '🇲🇨' => 'σημαία: Μονακό',
    '🇲🇩' => 'σημαία: Μολδαβία',
    '🇲🇪' => 'σημαία: Μαυροβούνιο',
    '🇲🇫' => 'σημαία: Άγιος Μαρτίνος (Γαλλικό τμήμα)',
    '🇲🇬' => 'σημαία: Μαδαγασκάρη',
    '🇲🇭' => 'σημαία: Νήσοι Μάρσαλ',
    '🇲🇰' => 'σημαία: Βόρεια Μακεδονία',
    '🇲🇱' => 'σημαία: Μάλι',
    '🇲🇲' => 'σημαία: Μιανμάρ (Βιρμανία)',
    '🇲🇳' => 'σημαία: Μογγολία',
    '🇲🇴' => 'σημαία: Μακάο ΕΔΠ Κίνας',
    '🇲🇵' => 'σημαία: Νήσοι Βόρειες Μαριάνες',
    '🇲🇶' => 'σημαία: Μαρτινίκα',
    '🇲🇷' => 'σημαία: Μαυριτανία',
    '🇲🇸' => 'σημαία: Μονσεράτ',
    '🇲🇹' => 'σημαία: Μάλτα',
    '🇲🇺' => 'σημαία: Μαυρίκιος',
    '🇲🇻' => 'σημαία: Μαλδίβες',
    '🇲🇼' => 'σημαία: Μαλάουι',
    '🇲🇽' => 'σημαία: Μεξικό',
    '🇲🇾' => 'σημαία: Μαλαισία',
    '🇲🇿' => 'σημαία: Μοζαμβίκη',
    '🇳🇦' => 'σημαία: Ναμίμπια',
    '🇳🇨' => 'σημαία: Νέα Καληδονία',
    '🇳🇪' => 'σημαία: Νίγηρας',
    '🇳🇫' => 'σημαία: Νήσος Νόρφολκ',
    '🇳🇬' => 'σημαία: Νιγηρία',
    '🇳🇮' => 'σημαία: Νικαράγουα',
    '🇳🇱' => 'σημαία: Κάτω Χώρες',
    '🇳🇴' => 'σημαία: Νορβηγία',
    '🇳🇵' => 'σημαία: Νεπάλ',
    '🇳🇷' => 'σημαία: Ναουρού',
    '🇳🇺' => 'σημαία: Νιούε',
    '🇳🇿' => 'σημαία: Νέα Ζηλανδία',
    '🇴🇲' => 'σημαία: Ομάν',
    '🇵🇦' => 'σημαία: Παναμάς',
    '🇵🇪' => 'σημαία: Περού',
    '🇵🇫' => 'σημαία: Γαλλική Πολυνησία',
    '🇵🇬' => 'σημαία: Παπούα Νέα Γουινέα',
    '🇵🇭' => 'σημαία: Φιλιππίνες',
    '🇵🇰' => 'σημαία: Πακιστάν',
    '🇵🇱' => 'σημαία: Πολωνία',
    '🇵🇲' => 'σημαία: Σεν Πιερ και Μικελόν',
    '🇵🇳' => 'σημαία: Νήσοι Πίτκερν',
    '🇵🇷' => 'σημαία: Πουέρτο Ρίκο',
    '🇵🇸' => 'σημαία: Παλαιστινιακά Εδάφη',
    '🇵🇹' => 'σημαία: Πορτογαλία',
    '🇵🇼' => 'σημαία: Παλάου',
    '🇵🇾' => 'σημαία: Παραγουάη',
    '🇶🇦' => 'σημαία: Κατάρ',
    '🇷🇪' => 'σημαία: Ρεϊνιόν',
    '🇷🇴' => 'σημαία: Ρουμανία',
    '🇷🇸' => 'σημαία: Σερβία',
    '🇷🇺' => 'σημαία: Ρωσία',
    '🇷🇼' => 'σημαία: Ρουάντα',
    '🇸🇦' => 'σημαία: Σαουδική Αραβία',
    '🇸🇧' => 'σημαία: Νήσοι Σολομώντος',
    '🇸🇨' => 'σημαία: Σεϋχέλλες',
    '🇸🇩' => 'σημαία: Σουδάν',
    '🇸🇪' => 'σημαία: Σουηδία',
    '🇸🇬' => 'σημαία: Σιγκαπούρη',
    '🇸🇭' => 'σημαία: Αγία Ελένη',
    '🇸🇮' => 'σημαία: Σλοβενία',
    '🇸🇯' => 'σημαία: Σβάλμπαρντ και Γιαν Μαγιέν',
    '🇸🇰' => 'σημαία: Σλοβακία',
    '🇸🇱' => 'σημαία: Σιέρα Λεόνε',
    '🇸🇲' => 'σημαία: Άγιος Μαρίνος',
    '🇸🇳' => 'σημαία: Σενεγάλη',
    '🇸🇴' => 'σημαία: Σομαλία',
    '🇸🇷' => 'σημαία: Σουρινάμ',
    '🇸🇸' => 'σημαία: Νότιο Σουδάν',
    '🇸🇹' => 'σημαία: Σάο Τομέ και Πρίνσιπε',
    '🇸🇻' => 'σημαία: Ελ Σαλβαδόρ',
    '🇸🇽' => 'σημαία: Άγιος Μαρτίνος (Ολλανδικό τμήμα)',
    '🇸🇾' => 'σημαία: Συρία',
    '🇸🇿' => 'σημαία: Εσουατίνι',
    '🇹🇦' => 'σημαία: Τριστάν ντα Κούνια',
    '🇹🇨' => 'σημαία: Νήσοι Τερκς και Κάικος',
    '🇹🇩' => 'σημαία: Τσαντ',
    '🇹🇫' => 'σημαία: Γαλλικά Νότια Εδάφη',
    '🇹🇬' => 'σημαία: Τόγκο',
    '🇹🇭' => 'σημαία: Ταϊλάνδη',
    '🇹🇯' => 'σημαία: Τατζικιστάν',
    '🇹🇰' => 'σημαία: Τοκελάου',
    '🇹🇱' => 'σημαία: Τιμόρ-Λέστε',
    '🇹🇲' => 'σημαία: Τουρκμενιστάν',
    '🇹🇳' => 'σημαία: Τυνησία',
    '🇹🇴' => 'σημαία: Τόνγκα',
    '🇹🇷' => 'σημαία: Τουρκία',
    '🇹🇹' => 'σημαία: Τρινιντάντ και Τομπάγκο',
    '🇹🇻' => 'σημαία: Τουβαλού',
    '🇹🇼' => 'σημαία: Ταϊβάν',
    '🇹🇿' => 'σημαία: Τανζανία',
    '🇺🇦' => 'σημαία: Ουκρανία',
    '🇺🇬' => 'σημαία: Ουγκάντα',
    '🇺🇲' => 'σημαία: Απομακρυσμένες Νησίδες ΗΠΑ',
    '🇺🇳' => 'σημαία: Ηνωμένα Έθνη',
    '🇺🇸' => 'σημαία: Ηνωμένες Πολιτείες',
    '🇺🇾' => 'σημαία: Ουρουγουάη',
    '🇺🇿' => 'σημαία: Ουζμπεκιστάν',
    '🇻🇦' => 'σημαία: Βατικανό',
    '🇻🇨' => 'σημαία: Άγιος Βικέντιος και Γρεναδίνες',
    '🇻🇪' => 'σημαία: Βενεζουέλα',
    '🇻🇬' => 'σημαία: Βρετανικές Παρθένες Νήσοι',
    '🇻🇮' => 'σημαία: Αμερικανικές Παρθένες Νήσοι',
    '🇻🇳' => 'σημαία: Βιετνάμ',
    '🇻🇺' => 'σημαία: Βανουάτου',
    '🇼🇫' => 'σημαία: Γουάλις και Φουτούνα',
    '🇼🇸' => 'σημαία: Σαμόα',
    '🇽🇰' => 'σημαία: Κοσσυφοπέδιο',
    '🇾🇪' => 'σημαία: Υεμένη',
    '🇾🇹' => 'σημαία: Μαγιότ',
    '🇿🇦' => 'σημαία: Νότια Αφρική',
    '🇿🇲' => 'σημαία: Ζάμπια',
    '🇿🇼' => 'σημαία: Ζιμπάμπουε',
    '🔟' => 'πλήκτρο: 10',
    '🏻' => 'ανοιχτόχρωμος τόνος δέρματος',
    '🏼' => 'μεσαίος-ανοιχτόχρωμος τόνος δέρματος',
    '🏽' => 'μεσαίος τόνος δέρματος',
    '🏾' => 'μεσαίος-σκούρος τόνος δέρματος',
    '🏿' => 'σκούρος τόνος δέρματος',
    '😀' => 'πλατύ χαμόγελο',
    '😃' => 'πλατύ χαμόγελο με μεγάλα μάτια',
    '😄' => 'πλατύ χαμόγελο με γελαστά μάτια',
    '😁' => 'περιχαρές πρόσωπο με γελαστά μάτια',
    '😆' => 'πλατύ χαμόγελο με κλειστά μάτια',
    '😅' => 'πλατύ χαμόγελο και κρύος ιδρώτας',
    '🤣' => 'ξεκαρδίζομαι',
    '😂' => 'κλαίω από χαρά',
    '🙂' => 'ελαφρώς χαμογελαστό πρόσωπο',
    '🙃' => 'αναποδογυρισμένο πρόσωπο',
    '🫠' => 'πρόσωπο που λιώνει',
    '😉' => 'κλείσιμο ματιού',
    '😊' => 'χαμογελαστό πρόσωπο με γελαστά μάτια',
    '😇' => 'χαμογελαστό πρόσωπο με φωτοστέφανο',
    '🥰' => 'χαμογελαστό προσωπάκι με καρδούλες',
    '😍' => 'χαμογελαστό πρόσωπο με μάτια καρδούλες',
    '🤩' => 'βλέπω αστεράκια',
    '😘' => 'στέλνω φιλί',
    '😗' => 'φιλώ',
    '☺' => 'χαμογελαστό πρόσωπο',
    '😚' => 'φιλώ με κλειστά μάτια',
    '😙' => 'φιλώ με γελαστά μάτια',
    '🥲' => 'χαμογελαστό πρόσωπο με δάκρυ',
    '😋' => 'απολαμβάνω πεντανόστιμο φαγητό',
    '😛' => 'γλώσσα έξω',
    '😜' => 'γλώσσα έξω με κλειστό μάτι',
    '🤪' => 'τρελή φάτσα',
    '😝' => 'γλώσσα έξω με κλειστά μάτια',
    '🤑' => 'γλώσσα με λεφτά',
    '🤗' => 'αγκαλιά',
    '🤭' => 'πρόσωπο με χέρι στο στόμα',
    '🫢' => 'πρόσωπο με ανοιχτά μάτια και το χέρι πάνω στο στόμα',
    '🫣' => 'κρυφοκοίταγμα',
    '🤫' => 'πρόσωπο που κάνει σσσς',
    '🤔' => 'σκέφτομαι',
    '🫡' => 'πρόσωπο που χαιρετά στρατιωτικά',
    '🤐' => 'ερμητικά κλειστό στόμα',
    '🤨' => 'πρόσωπο με σηκωμένο φρύδι',
    '😐' => 'ουδέτερο πρόσωπο',
    '😑' => 'ανέκφραστο πρόσωπο',
    '😶' => 'πρόσωπο χωρίς στόμα',
    '🫥' => 'πρόσωπο σε διάστικτο κύκλο',
    '😏' => 'μειδίαμα',
    '😒' => 'δυσαρέσκεια',
    '🙄' => 'βλέμμα αποδοκιμασίας',
    '😬' => 'γκριμάτσα',
    '🤥' => 'φάτσα ψεύτη',
    '🫨' => 'πρόσωπο που κουνιέται',
    '😌' => 'ανακούφιση',
    '😔' => 'περισυλλογή',
    '😪' => 'νυσταγμένο πρόσωπο',
    '🤤' => 'σαλιαρίζω',
    '😴' => 'ύπνος',
    '🫩' => 'πρόσωπο με σακούλες κάτω από τα μάτια',
    '😷' => 'ιατρική μάσκα',
    '🤒' => 'πρόσωπο με θερμόμετρο',
    '🤕' => 'επίδεσμος στο κεφάλι',
    '🤢' => 'ανακατεύομαι',
    '🤮' => 'πρόσωπο που κάνει εμετό',
    '🤧' => 'φταρνίζομαι',
    '🥵' => 'ζεστό προσωπάκι',
    '🥶' => 'κρυώνω',
    '🥴' => 'ζαλισμένο προσωπάκι',
    '😵' => 'ζαλάδα',
    '🤯' => 'κεφάλι που εκρήγνυται',
    '🤠' => 'καουμπόης με καπέλο',
    '🥳' => 'προσωπάκι πάρτι',
    '🥸' => 'μεταμφιεσμένο πρόσωπο',
    '😎' => 'χαμογελαστό πρόσωπο με γυαλιά ηλίου',
    '🤓' => 'σπασίκλας',
    '🧐' => 'πρόσωπο με μονόκλ',
    '😕' => 'σύγχυση',
    '🫤' => 'πρόσωπο με διαγώνιο στόμα',
    '😟' => 'ανησυχία',
    '🙁' => 'ελαφρύ κατσούφιασμα',
    '☹' => 'κατσούφιασμα',
    '😮' => 'ανοικτό στόμα',
    '😯' => 'έκπληκτο βλέμμα',
    '😲' => 'ξάφνιασμα',
    '😳' => 'κοκκινισμένο πρόσωπο',
    '🫪' => 'παραμορφωμένο πρόσωπο',
    '🥺' => 'προσωπάκι παρακαλετό',
    '🥹' => 'πρόσωπο έτοιμο να κλάψει',
    '😦' => 'κατσούφιασμα με ανοικτό στόμα',
    '😧' => 'αγωνία',
    '😨' => 'φόβος',
    '😰' => 'ανοικτό στόμα και κρύος ιδρώτας',
    '😥' => 'λυπημένο αλλά ανακουφισμένο πρόσωπο',
    '😢' => 'κλάμα',
    '😭' => 'γοερό κλάμα',
    '😱' => 'ουρλιαχτό φόβου',
    '😖' => 'σάστισμα',
    '😣' => 'βλέμμα απόγνωσης',
    '😞' => 'απογοήτευση',
    '😓' => 'κρύος ιδρώτας',
    '😩' => 'εξάντληση',
    '😫' => 'κουρασμένο πρόσωπο',
    '🥱' => 'πρόσωπο που χασμουριέται',
    '😤' => 'καπνοί από τη μύτη',
    '😡' => 'οργή',
    '😠' => 'θυμός',
    '🤬' => 'πρόσωπο με σύμβολα στο στόμα',
    '😈' => 'χαμόγελο με κέρατα',
    '👿' => 'δαίμονας',
    '💀' => 'κρανίο',
    '☠' => 'κρανίο με κόκκαλα',
    '💩' => 'περιττώματα',
    '🤡' => 'αστεία φάτσα',
    '👹' => 'τέρας',
    '👺' => 'καλικάντζαρος',
    '👻' => 'φάντασμα',
    '👽' => 'εξωγήινος',
    '👾' => 'εξωγήινο τέρας',
    '🤖' => 'ρομπότ',
    '😺' => 'γάτα με πλατύ χαμόγελο',
    '😸' => 'γάτα με πλατύ χαμόγελο και γελαστά μάτια',
    '😹' => 'γάτα που κλαίει από χαρά',
    '😻' => 'γάτα που χαμογελάει με μάτια καρδούλες',
    '😼' => 'γάτα που χαμογελάει ειρωνικά',
    '😽' => 'γάτα που φιλάει με κλειστά μάτια',
    '🙀' => 'έκπληκτη γάτα',
    '😿' => 'γάτα που κλαίει',
    '😾' => 'κατσουφιασμένη γάτα',
    '🙈' => 'κάνω πως δεν βλέπω',
    '🙉' => 'κάνω πως δεν ακούω',
    '🙊' => 'κάνω πως δεν μιλώ',
    '💌' => 'ερωτικό γράμμα',
    '💘' => 'καρδιά με βέλος',
    '💝' => 'καρδιά με κορδέλα',
    '💖' => 'καρδιά που αστράφτει',
    '💗' => 'καρδιά που μεγαλώνει',
    '💓' => 'καρδιά που χτυπά',
    '💞' => 'καρδιές που γυρίζουν',
    '💕' => 'δύο καρδιές',
    '💟' => 'καρδιά σε πλαίσιο',
    '❣' => 'καρδιά θαυμαστικό',
    '💔' => 'ραγισμένη καρδιά',
    '❤' => 'κόκκινη καρδιά',
    '🩷' => 'ροζ καρδιά',
    '🧡' => 'πορτοκαλί καρδιά',
    '💛' => 'κίτρινη καρδιά',
    '💚' => 'πράσινη καρδιά',
    '💙' => 'μπλε καρδιά',
    '🩵' => 'γαλάζια καρδιά',
    '💜' => 'μοβ καρδιά',
    '🤎' => 'καφέ καρδιά',
    '🖤' => 'μαύρη καρδιά',
    '🩶' => 'γκρι καρδιά',
    '🤍' => 'λευκή καρδιά',
    '💋' => 'σημάδι από φιλί',
    '💯' => 'εκατό πόντοι',
    '💢' => 'σύμβολο θυμού',
    '🫯' => 'σύννεφο καβγά',
    '💥' => 'σύγκρουση',
    '💫' => 'αστεράκια',
    '💦' => 'σταγόνες ιδρώτα',
    '💨' => 'τρέξιμο',
    '🕳' => 'τρύπα',
    '💬' => 'συννεφάκι ομιλίας',
    '🗨' => 'αριστερό συννεφάκι ομιλίας',
    '🗯' => 'δεξί συννεφάκι θυμού',
    '💭' => 'συννεφάκι σκέψης',
    '💤' => 'ζζζ',
    '👋' => 'χέρι που χαιρετά',
    '🤚' => 'σηκωμένο χέρι ανάποδα',
    '🖐' => 'μούντζα',
    '✋' => 'σηκωμένο χέρι',
    '🖖' => 'χαιρετισμός σποκ',
    '🫱' => 'χέρι προς τα δεξιά',
    '🫲' => 'χέρι προς τα αριστερά',
    '🫳' => 'παλάμη προς τα κάτω',
    '🫴' => 'παλάμη προς τα πάνω',
    '🫷' => 'χέρι που σπρώχνει προς τα αριστερά',
    '🫸' => 'χέρι που σπρώχνει προς τα δεξιά',
    '👌' => 'χειρονομία οκ',
    '🤌' => 'δάχτυλα που κλείνουν',
    '🤏' => 'χέρι που τσιμπάει',
    '✌' => 'σήμα της νίκης',
    '🤞' => 'καλή τύχη',
    '🫰' => 'δείκτης και αντίχειρας σταυρωμένοι',
    '🤟' => 'κίνηση σε αγαπώ',
    '🤘' => 'χειρονομία ροκ',
    '🤙' => 'τηλεφώνησέ μου',
    '👈' => 'δείκτης χεριού προς τα αριστερά',
    '👉' => 'δείκτης χεριού προς τα δεξιά',
    '👆' => 'δείκτης χεριού προς τα επάνω ανάποδα',
    '🖕' => 'μεσαίο δάχτυλο',
    '👇' => 'δείκτης χεριού προς τα κάτω ανάποδα',
    '☝' => 'δείκτης χεριού προς τα επάνω',
    '🫵' => 'δείκτης που δείχνει προς τον θεατή',
    '👍' => 'σηκωμένος αντίχειρας',
    '👎' => 'κατεβασμένος αντίχειρας',
    '✊' => 'σηκωμένη γροθιά',
    '👊' => 'γροθιά',
    '🤛' => 'γροθιά προς αριστερά',
    '🤜' => 'γροθιά προς δεξιά',
    '👏' => 'παλαμάκια',
    '🙌' => 'χέρια ψηλά',
    '🫶' => 'καρδούλα με τα χέρια',
    '👐' => 'ανοικτά χέρια',
    '🤲' => 'παλάμες μαζί',
    '🤝' => 'χειραψία',
    '🙏' => 'χέρια ενωμένα',
    '✍' => 'χέρι που γράφει',
    '💅' => 'βάψιμο νυχιών',
    '🤳' => 'selfie',
    '💪' => 'σφιχτός δικέφαλος',
    '🦾' => 'μηχανικό χέρι',
    '🦿' => 'μηχανικό πόδι',
    '🦵' => 'πόδι',
    '🦶' => 'κάτω άκρο',
    '👂' => 'αυτί',
    '🦻' => 'αυτί με ακουστικό βαρηκοΐας',
    '👃' => 'μύτη',
    '🧠' => 'μυαλό',
    '🫀' => 'καρδιά',
    '🫁' => 'πνεύμονες',
    '🦷' => 'δόντι',
    '🦴' => 'κόκαλο',
    '👀' => 'μάτια',
    '👁' => 'μάτι',
    '👅' => 'γλώσσα',
    '👄' => 'στόμα',
    '🫦' => 'δάγκωμα χειλιών',
    '👶' => 'μωρό',
    '🧒' => 'παιδί',
    '👦' => 'αγόρι',
    '👧' => 'κορίτσι',
    '🧑' => 'ενήλικας',
    '👱' => 'άτομο: ξανθά μαλλιά',
    '👨' => 'άντρας',
    '🧔' => 'άτομο: μούσι',
    '👩' => 'γυναίκα',
    '🧓' => 'μεγαλύτερος ενήλικας',
    '👴' => 'ηλικιωμένος',
    '👵' => 'ηλικιωμένη',
    '🙍' => 'συνοφρυωμένο άτομο',
    '🙎' => 'κατσουφιασμένο άτομο',
    '🙅' => 'άτομο διαφωνεί',
    '🙆' => 'άτομο συμφωνεί',
    '💁' => 'άτομο προσφέρει',
    '🙋' => 'άτομο με σηκωμένο χέρι',
    '🧏' => 'κωφό άτομο',
    '🙇' => 'άτομο υποκλίνεται',
    '🤦' => 'άτομο αγανακτά',
    '🤷' => 'άτομο σηκώνει τους ώμους',
    '👮' => 'αστυνομικός',
    '🕵' => 'ντετέκτιβ',
    '💂' => 'υπεύθυνος φρουράς',
    '🥷' => 'νίντζα',
    '👷' => 'οικοδόμος',
    '🫅' => 'άτομο με στέμμα',
    '🤴' => 'πρίγκιπας',
    '👸' => 'πριγκίπισσα',
    '👳' => 'άτομο με τουρμπάνι',
    '👲' => 'άντρας με κινέζικο καπέλο',
    '🧕' => 'γυναίκα με μαντίλα',
    '🤵' => 'άτομο με σμόκιν',
    '👰' => 'άτομο με πέπλο',
    '🤰' => 'έγκυος',
    '🫃' => 'έγκυος άντρας',
    '🫄' => 'έγκυο άτομο',
    '🤱' => 'θηλασμός',
    '👼' => 'μωρό άγγελος',
    '🎅' => 'Άι Βασίλης',
    '🤶' => 'Κυρία Άι Βασίλη',
    '🦸' => 'υπερήρωας',
    '🦹' => 'αρχικακός',
    '🧙' => 'μάγος',
    '🧚' => 'νεράιδα',
    '🧛' => 'βρυκόλακας',
    '🧜' => 'γοργόνα',
    '🧝' => 'ξωτικό',
    '🧞' => 'τζίνι',
    '🧟' => 'ζόμπι',
    '🧌' => 'τρολ',
    '🫈' => 'τριχωτό πλάσμα',
    '💆' => 'μασάζ στο κεφάλι',
    '💇' => 'κούρεμα',
    '🚶' => 'πεζός',
    '🧍' => 'άτομο που στέκεται',
    '🧎' => 'άτομο που γονατίζει',
    '🏃' => 'άτομο τρέχει',
    '💃' => 'χορεύτρια',
    '🕺' => 'χορευτής',
    '🕴' => 'κοστουμαρισμένος άντρας που αιωρείται',
    '👯' => 'άτομα με λαγουδένια αυτιά',
    '🧖' => 'άτομο σε σάουνα',
    '🧗' => 'άτομο που αναρριχείται',
    '🤺' => 'ξιφομάχος',
    '🏇' => 'ιππόδρομος',
    '⛷' => 'σκιέρ',
    '🏂' => 'σνόουμπορντ',
    '🏌' => 'γκόλφερ',
    '🏄' => 'σέρφερ',
    '🚣' => 'κωπηλάτης',
    '🏊' => 'κολυμβητής',
    '⛹' => 'άτομο παίζει μπάσκετ',
    '🏋' => 'αρσιβαρίστας',
    '🚴' => 'ποδηλάτης',
    '🚵' => 'ορεινή ποδηλασία',
    '🤸' => 'άτομο κάνει ρόδα',
    '🤼' => 'πάλη',
    '🤽' => 'πόλο',
    '🤾' => 'χάντμπολ',
    '🤹' => 'ζογκλέρ',
    '🧘' => 'άτομο σε θέση λωτού',
    '🛀' => 'άνθρωπος που κάνει μπάνιο',
    '🛌' => 'άνθρωπος σε κρεβάτι',
    '👭' => 'γυναίκες πιασμένες χέρι-χέρι',
    '👫' => 'γυναίκα και άντρας πιασμένοι χέρι-χέρι',
    '👬' => 'άντρες πιασμένοι χέρι-χέρι',
    '💏' => 'φιλί',
    '💑' => 'ζευγάρι με καρδιά',
    '🗣' => 'κεφάλι που μιλάει',
    '👤' => 'περίγραμμα προτομής',
    '👥' => 'περίγραμμα προτομών',
    '🫂' => 'άτομα αγκαλιάζονται',
    '👪' => 'οικογένεια',
    '👣' => 'πατημασιές',
    '🫆' => 'δακτυλικό αποτύπωμα',
    '🦰' => 'κόκκινα μαλλιά',
    '🦱' => 'σγουρά μαλλιά',
    '🦳' => 'άσπρα μαλλιά',
    '🦲' => 'καραφλός',
    '🐵' => 'πρόσωπο μαϊμούς',
    '🐒' => 'μαϊμού',
    '🦍' => 'γορίλας',
    '🦧' => 'ουρακοτάγκος',
    '🐶' => 'πρόσωπο σκύλου',
    '🐕' => 'σκύλος',
    '🦮' => 'σκύλος οδηγός',
    '🐩' => 'κανίς',
    '🐺' => 'λύκος',
    '🦊' => 'αλεπού',
    '🦝' => 'ρακούν',
    '🐱' => 'πρόσωπο γάτας',
    '🐈' => 'γάτα',
    '🦁' => 'λιοντάρι',
    '🐯' => 'πρόσωπο τίγρης',
    '🐅' => 'τίγρη',
    '🐆' => 'λεοπάρδαλη',
    '🐴' => 'πρόσωπο αλόγου',
    '🫎' => 'αμερικάνικη έλαφος',
    '🫏' => 'γάιδαρος',
    '🐎' => 'άλογο',
    '🦄' => 'μονόκερος',
    '🦓' => 'ζέβρα',
    '🦌' => 'ελάφι',
    '🦬' => 'βίσωνας',
    '🐮' => 'πρόσωπο αγελάδας',
    '🐂' => 'βόδι',
    '🐃' => 'νεροβούβαλος',
    '🐄' => 'αγελάδα',
    '🐷' => 'πρόσωπο γουρουνιού',
    '🐖' => 'γουρούνι',
    '🐗' => 'αγριογούρουνο',
    '🐽' => 'μύτη γουρουνιού',
    '🐏' => 'κριάρι',
    '🐑' => 'πρόβατο',
    '🐐' => 'κατσίκα',
    '🐪' => 'καμήλα',
    '🐫' => 'βακτριανή καμήλα',
    '🦙' => 'λάμα',
    '🦒' => 'καμηλοπάρδαλη',
    '🐘' => 'ελέφαντας',
    '🦣' => 'μαμούθ',
    '🦏' => 'ρινόκερος',
    '🦛' => 'ιπποπόταμος',
    '🐭' => 'πρόσωπο ποντικιού',
    '🐁' => 'ποντίκι',
    '🐀' => 'αρουραίος',
    '🐹' => 'χάμστερ',
    '🐰' => 'πρόσωπο κουνελιού',
    '🐇' => 'κουνέλι',
    '🐿' => 'σκιουράκι',
    '🦫' => 'κάστορας',
    '🦔' => 'σκαντζόχοιρος',
    '🦇' => 'νυχτερίδα',
    '🐻' => 'αρκούδα',
    '🐨' => 'κοάλα',
    '🐼' => 'πάντα',
    '🦥' => 'βραδύπους',
    '🦦' => 'βίδρα',
    '🦨' => 'μεφίτης',
    '🦘' => 'καγκουρό',
    '🦡' => 'ασβός',
    '🐾' => 'ίχνη ζώου',
    '🦃' => 'γαλοπούλα',
    '🐔' => 'κότα',
    '🐓' => 'κόκορας',
    '🐣' => 'εκκολαπτόμενο κοτοπουλάκι',
    '🐤' => 'κοτοπουλάκι',
    '🐥' => 'κοτοπουλάκι που κοιτά μπροστά',
    '🐦' => 'πουλί',
    '🐧' => 'πιγκουίνος',
    '🕊' => 'περιστέρι',
    '🦅' => 'αετός',
    '🦆' => 'πάπια',
    '🦢' => 'κύκνος',
    '🦉' => 'κουκουβάγια',
    '🦤' => 'ντόντο',
    '🪶' => 'πούπουλο',
    '🦩' => 'φλαμίνγκο',
    '🦚' => 'παγώνι',
    '🦜' => 'παπαγάλος',
    '🪽' => 'φτερό',
    '🪿' => 'χήνα',
    '🐸' => 'βάτραχος',
    '🐊' => 'κροκόδειλος',
    '🐢' => 'χελώνα',
    '🦎' => 'σαύρα',
    '🐍' => 'φίδι',
    '🐲' => 'πρόσωπο δράκου',
    '🐉' => 'δράκος',
    '🦕' => 'σαυρόποδο',
    '🦖' => 'τυραννόσαυρος',
    '🐳' => 'φάλαινα που πετά νερό',
    '🐋' => 'φάλαινα',
    '🐬' => 'δελφίνι',
    '🫍' => 'όρκα',
    '🦭' => 'φώκια',
    '🐟' => 'ψάρι',
    '🐠' => 'τροπικό ψάρι',
    '🐡' => 'γουρουνόψαρο',
    '🦈' => 'καρχαρίας',
    '🐙' => 'χταπόδι',
    '🐚' => 'σπειροειδές κοχύλι',
    '🪸' => 'κοράλλι',
    '🪼' => 'μέδουσα',
    '🦀' => 'κάβουρας',
    '🦞' => 'αστακός',
    '🦐' => 'γαρίδα',
    '🦑' => 'καλαμάρι',
    '🦪' => 'στρείδι',
    '🐌' => 'σαλιγκάρι',
    '🦋' => 'πεταλούδα',
    '🐛' => 'κάμπια',
    '🐜' => 'μυρμήγκι',
    '🐝' => 'μέλισσα',
    '🪲' => 'σκαθάρι',
    '🐞' => 'πασχαλίτσα',
    '🦗' => 'τριζόνι',
    '🪳' => 'κατσαρίδα',
    '🕷' => 'αράχνη',
    '🕸' => 'ιστός αράχνης',
    '🦂' => 'σκορπιός',
    '🦟' => 'κουνούπι',
    '🪰' => 'μύγα',
    '🪱' => 'σκουλήκι',
    '🦠' => 'μικρόβιο',
    '💐' => 'μπουκέτο λουλούδια',
    '🌸' => 'άνθος κερασιάς',
    '💮' => 'λευκό λουλούδι',
    '🪷' => 'λωτός',
    '🏵' => 'ροζέτα',
    '🌹' => 'τριαντάφυλλο',
    '🥀' => 'μαραμένο λουλούδι',
    '🌺' => 'ιβίσκος',
    '🌻' => 'ηλιοτρόπιο',
    '🌼' => 'άνθος',
    '🌷' => 'τουλίπα',
    '🪻' => 'υάκινθος',
    '🌱' => 'φιντάνι',
    '🪴' => 'φυτό σε γλάστρα',
    '🌲' => 'αειθαλές δέντρο',
    '🌳' => 'φυλλοβόλο δέντρο',
    '🌴' => 'φοινικόδεντρο',
    '🌵' => 'κάκτος',
    '🌾' => 'όρυζα',
    '🌿' => 'βότανο',
    '☘' => 'τριφύλλι',
    '🍀' => 'τετράφυλλο τριφύλλι',
    '🍁' => 'φύλλο σφενδάμου',
    '🍂' => 'πεσμένα φύλλα',
    '🍃' => 'φύλλα στον άνεμο',
    '🪹' => 'άδεια φωλιά',
    '🪺' => 'φωλιά με αυγά',
    '🍄' => 'μανιτάρι',
    '🪾' => 'γυμνό δέντρο',
    '🍇' => 'σταφύλι',
    '🍈' => 'πεπόνι',
    '🍉' => 'καρπούζι',
    '🍊' => 'μανταρίνι',
    '🍋' => 'λεμόνι',
    '🍌' => 'μπανάνα',
    '🍍' => 'ανανάς',
    '🥭' => 'μάνγκο',
    '🍎' => 'κόκκινο μήλο',
    '🍏' => 'πράσινο μήλο',
    '🍐' => 'αχλάδι',
    '🍑' => 'ροδάκινο',
    '🍒' => 'κεράσια',
    '🍓' => 'φράουλα',
    '🫐' => 'μύρτιλο',
    '🥝' => 'ακτινίδιο',
    '🍅' => 'ντομάτα',
    '🫒' => 'ελιά',
    '🥥' => 'καρύδα',
    '🥑' => 'αβοκάντο',
    '🍆' => 'μελιτζάνα',
    '🥔' => 'πατάτα',
    '🥕' => 'καρότο',
    '🌽' => 'καλαμπόκι',
    '🌶' => 'καυτερή πιπεριά',
    '🫑' => 'πιπεριά',
    '🥒' => 'αγγούρι',
    '🥬' => 'πράσινο φυλλώδες',
    '🥦' => 'μπρόκολο',
    '🧄' => 'σκόρδο',
    '🧅' => 'κρεμμύδι',
    '🥜' => 'φιστίκια',
    '🫘' => 'φασόλια',
    '🌰' => 'κάστανο',
    '🫚' => 'πιπερόριζα',
    '🫛' => 'κέλυφος μπιζελιού',
    '🫜' => 'ριζώδες λαχανικό',
    '🍞' => 'ψωμί',
    '🥐' => 'κρουασάν',
    '🥖' => 'μπαγκέτα',
    '🫓' => 'πίτα ψωμιού',
    '🥨' => 'πρέτσελ',
    '🥯' => 'μπέιγκελ',
    '🥞' => 'τηγανίτες',
    '🧇' => 'βάφλα',
    '🧀' => 'κομμάτι τυρί',
    '🍖' => 'κότσι',
    '🍗' => 'πόδι πουλερικού',
    '🥩' => 'κομμάτι κρέας',
    '🥓' => 'μπέικον',
    '🍔' => 'χάμπουργκερ',
    '🍟' => 'τηγανητές πατάτες',
    '🍕' => 'πίτσα',
    '🌭' => 'χοτ ντογκ',
    '🥪' => 'σάντουιτς',
    '🌮' => 'τάκο',
    '🌯' => 'μπουρίτο',
    '🫔' => 'ταμάλες',
    '🥙' => 'γεμιστή λαγάνα',
    '🧆' => 'φαλάφελ',
    '🥚' => 'αυγό',
    '🍳' => 'μαγείρεμα',
    '🥘' => 'τηγάνι',
    '🍲' => 'κατσαρόλα με φαγητό',
    '🫕' => 'φοντί',
    '🥣' => 'μπολ με κουτάλι',
    '🥗' => 'πράσινη σαλάτα',
    '🍿' => 'ποπ-κορν',
    '🧈' => 'βούτυρο',
    '🧂' => 'αλάτι',
    '🥫' => 'κονσέρβα φαγητού',
    '🍱' => 'κουτί φαγητού',
    '🍘' => 'κράκερ ρυζιού',
    '🍙' => 'μπάλα ρυζιού',
    '🍚' => 'μαγειρεμένο ρύζι',
    '🍛' => 'ρύζι με κάρυ',
    '🍜' => 'μπολ με αχνιστό φαγητό',
    '🍝' => 'σπαγκέτι',
    '🍠' => 'ψητή γλυκοπατάτα',
    '🍢' => 'όντεν',
    '🍣' => 'σούσι',
    '🍤' => 'τηγανητή γαρίδα',
    '🍥' => 'καμαμπόκο',
    '🥮' => 'κέικ φεγγάρι',
    '🍡' => 'ντάνγκο',
    '🥟' => 'ντάμπλινγκ',
    '🥠' => 'τυχερό μπισκότο',
    '🥡' => 'συσκευασία φαγητού σε πακέτο',
    '🍦' => 'παγωτό χωνάκι',
    '🍧' => 'γρανίτα',
    '🍨' => 'παγωτό',
    '🍩' => 'ντόνατ',
    '🍪' => 'μπισκότο',
    '🎂' => 'τούρτα γενεθλίων',
    '🍰' => 'κομμάτι τούρτας',
    '🧁' => 'κεκάκι',
    '🥧' => 'πίτα',
    '🍫' => 'σοκολάτα',
    '🍬' => 'καραμέλα',
    '🍭' => 'γλειφιτζούρι',
    '🍮' => 'κρέμα καραμελέ',
    '🍯' => 'βάζο με μέλι',
    '🍼' => 'μπιμπερό',
    '🥛' => 'ποτήρι γάλα',
    '☕' => 'ζεστό ρόφημα',
    '🫖' => 'τσαγιέρα',
    '🍵' => 'κούπα τσαγιού χωρίς χερούλι',
    '🍶' => 'σάκε',
    '🍾' => 'μπουκάλι με φελλό που πετάγεται',
    '🍷' => 'ποτήρι κρασί',
    '🍸' => 'κοκτέιλ',
    '🍹' => 'τροπικό ποτό',
    '🍺' => 'ποτήρι μπίρας',
    '🍻' => 'ποτήρια μπίρας που τσουγκρίζουν',
    '🥂' => 'ποτήρια που τσουγκρίζουν',
    '🥃' => 'ποτήρι ουίσκι',
    '🫗' => 'περίχυση υγρού',
    '🥤' => 'ποτήρι με καλαμάκι',
    '🧋' => 'τσάι μπόμπα',
    '🧃' => 'κουτί ροφήματος',
    '🧉' => 'μάτε',
    '🧊' => 'παγάκι',
    '🥢' => 'ξυλάκια',
    '🍽' => 'μαχαιροπίρουνα με πιάτο',
    '🍴' => 'μαχαιροπίρουνα',
    '🥄' => 'κουτάλι',
    '🔪' => 'κουζινομάχαιρο',
    '🫙' => 'βαζάκι',
    '🏺' => 'αμφορέας',
    '🌍' => 'υδρόγειος με Ευρώπη και Αφρική',
    '🌎' => 'υδρόγειος με Βόρεια και Νότια Αμερική',
    '🌏' => 'υδρόγειος με Ασία και Αυστραλία',
    '🌐' => 'υδρόγειος με μεσημβρινούς',
    '🗺' => 'παγκόσμιος χάρτης',
    '🗾' => 'χάρτης της Ιαπωνίας',
    '🧭' => 'πυξίδα',
    '🏔' => 'χιονισμένο βουνό',
    '⛰' => 'βουνό',
    '🛘' => 'κατολίσθηση',
    '🌋' => 'ηφαίστειο',
    '🗻' => 'ηφαίστειο Φούτζι',
    '🏕' => 'κατασκήνωση',
    '🏖' => 'παραλία με ομπρέλα',
    '🏜' => 'έρημος',
    '🏝' => 'ερημονήσι',
    '🏞' => 'εθνικό πάρκο',
    '🏟' => 'στάδιο',
    '🏛' => 'κλασικό κτίριο',
    '🏗' => 'κατασκευή κτιρίου',
    '🧱' => 'τούβλο',
    '🪨' => 'πέτρα',
    '🪵' => 'ξύλο',
    '🛖' => 'καλύβα',
    '🏘' => 'σπίτια',
    '🏚' => 'εγκαταλελειμμένο σπίτι',
    '🏠' => 'σπίτι',
    '🏡' => 'σπίτι με κήπο',
    '🏢' => 'κτίριο γραφείων',
    '🏣' => 'ιαπωνικό ταχυδρομείο',
    '🏤' => 'ταχυδρομείο',
    '🏥' => 'νοσοκομείο',
    '🏦' => 'τράπεζα',
    '🏨' => 'ξενοδοχείο',
    '🏩' => 'ξενοδοχείο ημιδιαμονής',
    '🏪' => 'παντοπωλείο',
    '🏫' => 'σχολείο',
    '🏬' => 'πολυκατάστημα',
    '🏭' => 'εργοστάσιο',
    '🏯' => 'ιαπωνικό κάστρο',
    '🏰' => 'κάστρο',
    '💒' => 'γάμος',
    '🗼' => 'Πύργος του Τόκιο',
    '🗽' => 'Άγαλμα της Ελευθερίας',
    '⛪' => 'εκκλησία',
    '🕌' => 'τέμενος',
    '🛕' => 'ινδουιστικός ναός',
    '🕍' => 'συναγωγή',
    '⛩' => 'σιντοϊστικός ναός',
    '🕋' => 'καάμπα',
    '⛲' => 'σιντριβάνι',
    '⛺' => 'σκηνή',
    '🌁' => 'ομιχλώδες τοπίο',
    '🌃' => 'έναστρη νύχτα',
    '🏙' => 'αστικό τοπίο',
    '🌄' => 'αυγή πίσω από βουνά',
    '🌅' => 'αυγή',
    '🌆' => 'αστικό τοπίο το σούρουπο',
    '🌇' => 'ηλιοβασίλεμα',
    '🌉' => 'γέφυρα τη νύχτα',
    '♨' => 'θερμές πηγές',
    '🎠' => 'αλογάκι καρουσέλ',
    '🛝' => 'τσουλήθρα',
    '🎡' => 'ρόδα λούνα-παρκ',
    '🎢' => 'τρενάκι',
    '💈' => 'σύμβολο κουρείου',
    '🎪' => 'σκηνή τσίρκου',
    '🚂' => 'ατμομηχανή',
    '🚃' => 'βαγόνι τρένου',
    '🚄' => 'ταχεία',
    '🚅' => 'υπερταχεία',
    '🚆' => 'τρένο',
    '🚇' => 'μετρό',
    '🚈' => 'ελαφρύς σιδηρόδρομος',
    '🚉' => 'σταθμός',
    '🚊' => 'τραμ',
    '🚝' => 'τρένο μονής τροχιάς',
    '🚞' => 'ορεινός σιδηρόδρομος',
    '🚋' => 'βαγόνι τραμ',
    '🚌' => 'λεωφορείο',
    '🚍' => 'διερχόμενο λεωφορείο',
    '🚎' => 'τρόλεϊ',
    '🚐' => 'μίνιμπας',
    '🚑' => 'ασθενοφόρο',
    '🚒' => 'πυροσβεστικό',
    '🚓' => 'περιπολικό',
    '🚔' => 'διερχόμενο περιπολικό',
    '🚕' => 'ταξί',
    '🚖' => 'διερχόμενο ταξί',
    '🚗' => 'αυτοκίνητο',
    '🚘' => 'διερχόμενο αυτοκίνητο',
    '🚙' => 'όχημα αναψυχής',
    '🛻' => 'αγροτικό',
    '🚚' => 'φορτηγό',
    '🚛' => 'αρθρωτό όχημα',
    '🚜' => 'τρακτέρ',
    '🏎' => 'αγωνιστικό όχημα',
    '🏍' => 'μοτοσικλέτα',
    '🛵' => 'σκουτεράκι',
    '🦽' => 'αναπηρικό αμαξίδιο',
    '🦼' => 'ηλεκτροκίνητο αναπηρικό αμαξίδιο',
    '🛺' => 'ποδήλατο-ταξί',
    '🚲' => 'ποδήλατο',
    '🛴' => 'πατίνι',
    '🛹' => 'σκέιτμπορντ',
    '🛼' => 'πατίνια',
    '🚏' => 'στάση λεωφορείου',
    '🛣' => 'αυτοκινητόδρομος',
    '🛤' => 'σιδηρόδρομος',
    '🛢' => 'βαρέλι πετρελαίου',
    '⛽' => 'αντλία καυσίμου',
    '🛞' => 'τροχός',
    '🚨' => 'φάρος περιπολικού',
    '🚥' => 'οριζόντιο φανάρι',
    '🚦' => 'κάθετο φανάρι',
    '🛑' => 'σύμβολο στοπ',
    '🚧' => 'κατασκευές',
    '⚓' => 'άγκυρα',
    '🛟' => 'στρογγυλό σωσίβιο',
    '⛵' => 'ιστιοφόρο',
    '🛶' => 'κανό',
    '🚤' => 'ταχύπλοο',
    '🛳' => 'επιβατηγό πλοίο',
    '⛴' => 'φεριμπότ',
    '🛥' => 'μηχανοκίνητο σκάφος',
    '🚢' => 'πλοίο',
    '✈' => 'αεροπλάνο',
    '🛩' => 'μικρό αεροπλάνο',
    '🛫' => 'αναχώρηση αεροπλάνου',
    '🛬' => 'άφιξη αεροπλάνου',
    '🪂' => 'αλεξίπτωτο',
    '💺' => 'κάθισμα',
    '🚁' => 'ελικόπτερο',
    '🚟' => 'κρεμαστός σιδηρόδρομος',
    '🚠' => 'τελεφερίκ',
    '🚡' => 'εναέριος καλωδιοκίνητος σιδηρόδρομος',
    '🛰' => 'δορυφόρος',
    '🚀' => 'πύραυλος',
    '🛸' => 'ιπτάμενος δίσκος',
    '🛎' => 'κουδούνι αχθοφόρου',
    '🧳' => 'αποσκευή',
    '⌛' => 'κλεψύδρα',
    '⏳' => 'κλεψύδρα με άμμο που ρέει',
    '⌚' => 'ρολόι',
    '⏰' => 'ξυπνητήρι',
    '⏱' => 'χρονόμετρο',
    '⏲' => 'χρονομετρητής',
    '🕰' => 'επιτραπέζιο ρολόι',
    '🕛' => 'δώδεκα η ώρα',
    '🕧' => 'δώδεκα και μισή',
    '🕐' => 'μία η ώρα',
    '🕜' => 'μία και μισή',
    '🕑' => 'δύο η ώρα',
    '🕝' => 'δύο και μισή',
    '🕒' => 'τρεις η ώρα',
    '🕞' => 'τρεις και μισή',
    '🕓' => 'τέσσερις η ώρα',
    '🕟' => 'τέσσερις και μισή',
    '🕔' => 'πέντε η ώρα',
    '🕠' => 'πέντε και μισή',
    '🕕' => 'έξι η ώρα',
    '🕡' => 'έξι και μισή',
    '🕖' => 'επτά η ώρα',
    '🕢' => 'επτά και μισή',
    '🕗' => 'οκτώ η ώρα',
    '🕣' => 'οκτώ και μισή',
    '🕘' => 'εννέα η ώρα',
    '🕤' => 'εννέα και μισή',
    '🕙' => 'δέκα η ώρα',
    '🕥' => 'δέκα και μισή',
    '🕚' => 'έντεκα η ώρα',
    '🕦' => 'έντεκα και μισή',
    '🌑' => 'νέα σελήνη',
    '🌒' => 'αύξων μηνίσκος',
    '🌓' => 'σελήνη πρώτου τετάρτου',
    '🌔' => 'αύξων αμφίκυρτος',
    '🌕' => 'πανσέληνος',
    '🌖' => 'φθίνων αμφίκυρτος',
    '🌗' => 'σελήνη τελευταίου τετάρτου',
    '🌘' => 'φθίνων μηνίσκος',
    '🌙' => 'ημισέληνος',
    '🌚' => 'νέα σελήνη με πρόσωπο',
    '🌛' => 'σελήνη πρώτου τετάρτου με πρόσωπο',
    '🌜' => 'σελήνη τελευταίου τετάρτου με πρόσωπο',
    '🌡' => 'θερμόμετρο',
    '☀' => 'ήλιος',
    '🌝' => 'πανσέληνος με πρόσωπο',
    '🌞' => 'ήλιος με πρόσωπο',
    '🪐' => 'πλανήτης με δακτύλιο',
    '⭐' => 'αστέρι',
    '🌟' => 'λαμπερό αστέρι',
    '🌠' => 'αστέρι που πέφτει',
    '🌌' => 'γαλαξίας',
    '☁' => 'σύννεφο',
    '⛅' => 'ήλιος πίσω από σύννεφο',
    '⛈' => 'σύννεφο με αστραπή και βροχή',
    '🌤' => 'ήλιος πίσω από μικρό σύννεφο',
    '🌥' => 'ήλιος πίσω από μεγάλο σύννεφο',
    '🌦' => 'ήλιος πίσω από σύννεφο με βροχή',
    '🌧' => 'σύννεφο με βροχή',
    '🌨' => 'σύννεφο με χιόνι',
    '🌩' => 'σύννεφο με αστραπή',
    '🌪' => 'ανεμοστρόβιλος',
    '🌫' => 'ομίχλη',
    '🌬' => 'πρόσωπο ανέμου',
    '🌀' => 'κυκλώνας',
    '🌈' => 'ουράνιο τόξο',
    '🌂' => 'κλειστή ομπρέλα',
    '☂' => 'ομπρέλα',
    '☔' => 'ομπρέλα με σταγόνες βροχής',
    '⛱' => 'ομπρέλα θαλάσσης',
    '⚡' => 'υψηλή τάση',
    '❄' => 'χιονονιφάδα',
    '☃' => 'χιονάνθρωπος',
    '⛄' => 'χιονάνθρωπος χωρίς χιόνι',
    '☄' => 'κομήτης',
    '🔥' => 'φωτιά',
    '💧' => 'σταγόνα',
    '🌊' => 'θαλάσσιο κύμα',
    '🎃' => 'φανάρι από κολοκύθα',
    '🎄' => 'χριστουγεννιάτικο δέντρο',
    '🎆' => 'βεγγαλικά',
    '🎇' => 'αστράκι',
    '🧨' => 'κροτίδα',
    '✨' => 'αστράκια',
    '🎈' => 'μπαλόνι',
    '🎉' => 'χαρτοπόλεμος',
    '🎊' => 'κομφετί',
    '🎋' => 'δέντρο ταναμπάτα',
    '🎍' => 'διακοσμητικό πεύκο',
    '🎎' => 'ιαπωνικές κούκλες',
    '🎏' => 'διακοσμητικοί κυπρίνοι',
    '🎐' => 'μελωδός',
    '🎑' => 'τελετή θέασης της σελήνης',
    '🧧' => 'κόκκινος φάκελος',
    '🎀' => 'κορδέλα',
    '🎁' => 'τυλιγμένο δώρο',
    '🎗' => 'κορδέλα τιμής',
    '🎟' => 'κάρτες εισόδου',
    '🎫' => 'εισιτήριο',
    '🎖' => 'στρατιωτικό παράσημο',
    '🏆' => 'τρόπαιο',
    '🏅' => 'μετάλλιο',
    '🥇' => 'μετάλλιο 1ης θέσης',
    '🥈' => 'μετάλλιο 2ης θέσης',
    '🥉' => 'μετάλλιο 3ης θέσης',
    '⚽' => 'μπάλα ποδοσφαίρου',
    '⚾' => 'μπάλα μπέιζμπολ',
    '🥎' => 'σόφτμπολ',
    '🏀' => 'μπάλα μπάσκετ',
    '🏐' => 'μπάλα βόλεϊ',
    '🏈' => 'μπάλα αμερικάνικου ποδοσφαίρου',
    '🏉' => 'μπάλα ράγκμπι',
    '🎾' => 'μπάλα τένις',
    '🥏' => 'φρίσμπι',
    '🎳' => 'μπόουλινγκ',
    '🏏' => 'κρίκετ',
    '🏑' => 'χόκεϊ επί χόρτου',
    '🏒' => 'χόκεϊ επί πάγου',
    '🥍' => 'λακρός',
    '🏓' => 'πινγκ πονγκ',
    '🏸' => 'μπάντμιντον',
    '🥊' => 'γάντι του μποξ',
    '🥋' => 'στολή πολεμικών τεχνών',
    '🥅' => 'τέρμα',
    '⛳' => 'σημαία σε οπή',
    '⛸' => 'πατινάζ στον πάγο',
    '🎣' => 'καλάμι ψαρέματος',
    '🤿' => 'μάσκα κατάδυσης',
    '🎽' => 'μπλουζάκι δρομέα',
    '🎿' => 'σκι',
    '🛷' => 'έλκηθρο',
    '🥌' => 'πέτρα κέρλινγκ',
    '🎯' => 'βελάκια',
    '🪀' => 'γιογιό',
    '🪁' => 'χαρταετός',
    '🔫' => 'νεροπίστολο',
    '🎱' => 'μπάλα 8 μπιλιάρδου',
    '🔮' => 'κρυστάλλινη σφαίρα',
    '🪄' => 'μαγικό ραβδί',
    '🎮' => 'βιντεοπαιχνίδι',
    '🕹' => 'joystick',
    '🎰' => 'κουλοχέρης',
    '🎲' => 'ζάρι',
    '🧩' => 'κομμάτι παζλ',
    '🧸' => 'αρκουδάκι',
    '🪅' => 'πινιάτα',
    '🪩' => 'μπάλα με καθρέφτες',
    '🪆' => 'κούκλες μπάμπουσκα',
    '♠' => 'μπαστούνι',
    '♥' => 'κούπα',
    '♦' => 'καρό',
    '♣' => 'σπαθί',
    '♟' => 'πιόνι σκακιού',
    '🃏' => 'τζόκερ',
    '🀄' => 'πλακίδιο mahjong κόκκινος δράκος',
    '🎴' => 'τραπουλόχαρτα χαναφούντα',
    '🎭' => 'θέατρο',
    '🖼' => 'εικόνα σε κορνίζα',
    '🎨' => 'παλέτα ζωγράφου',
    '🧵' => 'κλωστή',
    '🪡' => 'βελόνα ραψίματος',
    '🧶' => 'κουβάρι',
    '🪢' => 'κόμπος',
    '👓' => 'γυαλιά',
    '🕶' => 'γυαλιά ηλίου',
    '🥽' => 'προστατευτικά γυαλιά',
    '🥼' => 'ρόμπα εργαστηρίου',
    '🦺' => 'γιλέκο ασφαλείας',
    '👔' => 'γραβάτα',
    '👕' => 'μπλουζάκι',
    '👖' => 'τζιν παντελόνι',
    '🧣' => 'κασκόλ',
    '🧤' => 'γάντια',
    '🧥' => 'παλτό',
    '🧦' => 'κάλτσες',
    '👗' => 'φόρεμα',
    '👘' => 'κιμονό',
    '🥻' => 'σάρι',
    '🩱' => 'ολόσωμο μαγιό',
    '🩲' => 'σλιπ',
    '🩳' => 'σορτσάκι',
    '👙' => 'μπικίνι',
    '👚' => 'γυναικεία ρούχα',
    '🪭' => 'αναδιπλούμενη βεντάλια',
    '👛' => 'πορτοφόλι',
    '👜' => 'τσάντα χειρός',
    '👝' => 'τσαντάκι',
    '🛍' => 'τσάντες για ψώνια',
    '🎒' => 'σχολική σάκα',
    '🩴' => 'σαγιονάρα',
    '👞' => 'ανδρικό παπούτσι',
    '👟' => 'αθλητικό παπούτσι',
    '🥾' => 'μπότα ορειβασίας',
    '🥿' => 'ίσιο παπούτσι',
    '👠' => 'ψηλοτάκουνο παπούτσι',
    '👡' => 'γυναικείο σανδάλι',
    '🩰' => 'παπούτσια μπαλέτου',
    '👢' => 'γυναικεία μπότα',
    '🪮' => 'πηρούνα μαλλιών',
    '👑' => 'στέμμα',
    '👒' => 'γυναικείο καπέλο',
    '🎩' => 'ημίψηλο καπέλο',
    '🎓' => 'καπέλο αποφοίτησης',
    '🧢' => 'καπελάκι',
    '🪖' => 'στρατιωτικό κράνος',
    '⛑' => 'κράνος με λευκό σταυρό',
    '📿' => 'κομπολόι',
    '💄' => 'κραγιόν',
    '💍' => 'δαχτυλίδι',
    '💎' => 'διαμάντι',
    '🔇' => 'ηχείο σε σίγαση',
    '🔈' => 'ηχείο σε χαμηλή ένταση',
    '🔉' => 'ηχείο σε μέτρια ένταση',
    '🔊' => 'ηχείο σε υψηλή ένταση',
    '📢' => 'ντουντούκα',
    '📣' => 'μεγάφωνο',
    '📯' => 'ταχυδρομική κόρνα',
    '🔔' => 'κουδούνι',
    '🔕' => 'κουδούνι με κόκκινη γραμμή',
    '🎼' => 'παρτιτούρα',
    '🎵' => 'νότα μουσικής',
    '🎶' => 'νότες μουσικής',
    '🎙' => 'μικρόφωνο στούντιο',
    '🎚' => 'ρυθμιστικό έντασης',
    '🎛' => 'κουμπιά ελέγχου',
    '🎤' => 'μικρόφωνο',
    '🎧' => 'ακουστικά',
    '📻' => 'ραδιόφωνο',
    '🎷' => 'σαξόφωνο',
    '🎺' => 'τρομπέτα',
    '🪊' => 'τρομπόνι',
    '🪗' => 'ακορντεόν',
    '🎸' => 'κιθάρα',
    '🎹' => 'μουσικό πληκτρολόγιο',
    '🎻' => 'βιολί',
    '🪕' => 'μπάντζο',
    '🥁' => 'τύμπανο',
    '🪘' => 'κόνγκα',
    '🪇' => 'μαράκες',
    '🪈' => 'φλάουτο',
    '🪉' => 'άρπα',
    '📱' => 'κινητό τηλέφωνο',
    '📲' => 'κινητό τηλέφωνο με βέλος',
    '☎' => 'τηλέφωνο',
    '📞' => 'ακουστικό τηλεφώνου',
    '📟' => 'βομβητής',
    '📠' => 'συσκευή φαξ',
    '🔋' => 'μπαταρία',
    '🪫' => 'χαμηλή μπαταρία',
    '🔌' => 'βύσμα ρεύματος',
    '💻' => 'φορητός υπολογιστής',
    '🖥' => 'επιτραπέζιος υπολογιστής',
    '🖨' => 'εκτυπωτής',
    '⌨' => 'πληκτρολόγιο',
    '🖱' => 'ποντίκι υπολογιστή',
    '🖲' => 'ιχνόσφαιρα',
    '💽' => 'δίσκος υπολογιστή',
    '💾' => 'δισκέτα',
    '💿' => 'οπτικός δίσκος',
    '📀' => 'dvd',
    '🧮' => 'άβακας',
    '🎥' => 'κινηματογραφική κάμερα',
    '🎞' => 'καρέ φιλμ',
    '📽' => 'προβολέας ταινιών',
    '🎬' => 'κλακέτα φιλμ',
    '📺' => 'τηλεόραση',
    '📷' => 'φωτογραφική μηχανή',
    '📸' => 'φωτογραφική μηχανή με φλας',
    '📹' => 'βιντεοκάμερα',
    '📼' => 'βιντεοκασέτα',
    '🔍' => 'αριστερός μεγεθυντικός φακός',
    '🔎' => 'δεξιός μεγεθυντικός φακός',
    '🕯' => 'κερί',
    '💡' => 'λαμπτήρας',
    '🔦' => 'φακός',
    '🏮' => 'κόκκινο χάρτινο φανάρι',
    '🪔' => 'λάμπα diya',
    '📔' => 'σημειωματάριο με διακοσμητικό εξώφυλλο',
    '📕' => 'κλειστό βιβλίο',
    '📖' => 'ανοικτό βιβλίο',
    '📗' => 'πράσινο βιβλίο',
    '📘' => 'μπλε βιβλίο',
    '📙' => 'πορτοκαλί βιβλίο',
    '📚' => 'βιβλία',
    '📓' => 'σημειωματάριο',
    '📒' => 'λογιστικό βιβλίο',
    '📃' => 'κυρτωμένη σελίδα',
    '📜' => 'περγαμηνή',
    '📄' => 'όρθια σελίδα',
    '📰' => 'εφημερίδα',
    '🗞' => 'τυλιγμένη εφημερίδα',
    '📑' => 'καρτέλες σελιδοδεικτών',
    '🔖' => 'σελιδοδείκτης',
    '🏷' => 'ετικέτα',
    '🪙' => 'κέρμα',
    '💰' => 'σάκος με λεφτά',
    '🪎' => 'σεντούκι θησαυρού',
    '💴' => 'χαρτονόμισμα γιεν',
    '💵' => 'χαρτονόμισμα δολαρίου',
    '💶' => 'χαρτονόμισμα ευρώ',
    '💷' => 'χαρτονόμισμα λίρας',
    '💸' => 'λεφτά με φτερά',
    '💳' => 'πιστωτική κάρτα',
    '🧾' => 'απόδειξη',
    '💹' => 'ανοδικό γράφημα με γιεν',
    '✉' => 'φάκελος',
    '📧' => 'email',
    '📨' => 'εισερχόμενος φάκελος',
    '📩' => 'φάκελος με βέλος',
    '📤' => 'δίσκος εξερχομένων',
    '📥' => 'δίσκος εισερχομένων',
    '📦' => 'πακέτο',
    '📫' => 'κλειστό γραμματοκιβώτιο με σηκωμένη σημαία',
    '📪' => 'κλειστό γραμματοκιβώτιο με χαμηλωμένη σημαία',
    '📬' => 'ανοικτό γραμματοκιβώτιο με σηκωμένη σημαία',
    '📭' => 'ανοικτό γραμματοκιβώτιο με χαμηλωμένη σημαία',
    '📮' => 'γραμματοκιβώτιο ταχυδρομείου',
    '🗳' => 'κάλπη με ψηφοδέλτιο',
    '✏' => 'μολύβι',
    '✒' => 'μαύρη πένα',
    '🖋' => 'μελανοφόρος',
    '🖊' => 'στιλό',
    '🖌' => 'πινέλο',
    '🖍' => 'κηρομπογιά',
    '📝' => 'σημείωμα',
    '💼' => 'χαρτοφύλακας',
    '📁' => 'φάκελος αρχείων',
    '📂' => 'ανοικτός φάκελος αρχείων',
    '🗂' => 'διαχωριστικά ευρετηρίου καρτών',
    '📅' => 'ημερολόγιο',
    '📆' => 'ημερολόγιο με αποσπώμενες σελίδες',
    '🗒' => 'σπιράλ σημειωματάριο',
    '🗓' => 'σπιράλ ημερολόγιο',
    '📇' => 'ευρετήριο καρτών',
    '📈' => 'ανοδικό γράφημα',
    '📉' => 'καθοδικό γράφημα',
    '📊' => 'γράφημα ράβδων',
    '📋' => 'πρόχειρο',
    '📌' => 'πινέζα',
    '📍' => 'στρογγυλοκέφαλη πινέζα',
    '📎' => 'συνδετήρας',
    '🖇' => 'ενωμένοι συνδετήρες',
    '📏' => 'ίσιος χάρακας',
    '📐' => 'τριγωνικός χάρακας',
    '✂' => 'ψαλίδι',
    '🗃' => 'καρτελοθήκη',
    '🗄' => 'αρχειοθήκη',
    '🗑' => 'καλάθι αχρήστων',
    '🔒' => 'κλειδωμένο',
    '🔓' => 'ξεκλείδωτο',
    '🔏' => 'κλειδαριά με πένα',
    '🔐' => 'κλειδωμένο με κλειδί',
    '🔑' => 'κλειδί',
    '🗝' => 'παλιό κλειδί',
    '🔨' => 'σφυρί',
    '🪓' => 'τσεκούρι',
    '⛏' => 'αξίνα',
    '⚒' => 'σφυρί και αξίνα',
    '🛠' => 'σφυρί και γαλλικό κλειδί',
    '🗡' => 'στιλέτο',
    '⚔' => 'σταυρωτά σπαθιά',
    '💣' => 'βόμβα',
    '🪃' => 'μπούμερανγκ',
    '🏹' => 'τόξο και βέλος',
    '🛡' => 'ασπίδα',
    '🪚' => 'πριόνι ξυλουργού',
    '🔧' => 'γαλλικό κλειδί',
    '🪛' => 'κατσαβίδι',
    '🔩' => 'βίδα και παξιμάδι',
    '⚙' => 'γρανάζι',
    '🗜' => 'σφιγκτήρας',
    '⚖' => 'ζυγαριά',
    '🦯' => 'μπαστούνι τυφλών',
    '🔗' => 'κρίκος',
    '⛓' => 'αλυσίδες',
    '🪝' => 'γάντζος',
    '🧰' => 'εργαλειοθήκη',
    '🧲' => 'μαγνήτης',
    '🪜' => 'σκάλα',
    '🪏' => 'φτυάρι',
    '⚗' => 'αποστακτήριο',
    '🧪' => 'δοκιμαστικός σωλήνας',
    '🧫' => 'τρυβλίο καλλιέργειας',
    '🧬' => 'dna',
    '🔬' => 'μικροσκόπιο',
    '🔭' => 'τηλεσκόπιο',
    '📡' => 'δορυφορική κεραία',
    '💉' => 'σύριγγα',
    '🩸' => 'σταγόνα αίματος',
    '💊' => 'χάπι',
    '🩹' => 'αυτοκόλλητο χανζαπλάστ',
    '🩼' => 'πατερίτσα',
    '🩺' => 'στηθοσκόπιο',
    '🩻' => 'ακτινογραφία',
    '🚪' => 'πόρτα',
    '🛗' => 'ασανσέρ',
    '🪞' => 'καθρέφτης',
    '🪟' => 'παράθυρο',
    '🛏' => 'κρεβάτι',
    '🛋' => 'καναπές και φωτιστικό',
    '🪑' => 'καρέκλα',
    '🚽' => 'τουαλέτα',
    '🪠' => 'βεντούζα απόφραξης',
    '🚿' => 'ντους',
    '🛁' => 'μπανιέρα',
    '🪤' => 'ποντικοπαγίδα',
    '🪒' => 'ξυράφι',
    '🧴' => 'μπουκάλι λοσιόν',
    '🧷' => 'παραμάνα',
    '🧹' => 'σκούπα',
    '🧺' => 'καλάθι',
    '🧻' => 'ρολό χαρτί',
    '🪣' => 'κουβάς',
    '🧼' => 'σαπούνι',
    '🫧' => 'φυσαλίδες',
    '🪥' => 'οδοντόβουρτσα',
    '🧽' => 'σφουγγάρι',
    '🧯' => 'πυροσβεστήρας',
    '🛒' => 'καρότσι αγορών',
    '🚬' => 'τσιγάρο',
    '⚰' => 'φέρετρο',
    '🪦' => 'ταφόπλακα',
    '⚱' => 'τεφροδόχος',
    '🧿' => 'ματάκι',
    '🪬' => 'χάμσα',
    '🗿' => 'μοάι',
    '🪧' => 'πινακίδα',
    '🪪' => 'ταυτότητα',
    '🏧' => 'σήμα ATM',
    '🚮' => 'σήμα ρίψης σκουπιδιών σε κάδο',
    '🚰' => 'πόσιμο νερό',
    '♿' => 'αναπηρικό καροτσάκι',
    '🚹' => 'τουαλέτα ανδρών',
    '🚺' => 'τουαλέτα γυναικών',
    '🚻' => 'δημόσια τουαλέτα',
    '🚼' => 'σύμβολο μωρού',
    '🚾' => 'WC',
    '🛂' => 'έλεγχος διαβατηρίων',
    '🛃' => 'τελωνείο',
    '🛄' => 'παραλαβή αποσκευών',
    '🛅' => 'φύλαξη αποσκευών',
    '⚠' => 'προειδοποίηση',
    '🚸' => 'διάβαση παιδιών',
    '⛔' => 'απαγορεύεται η είσοδος',
    '🚫' => 'απαγορεύεται',
    '🚳' => 'απαγορεύονται τα ποδήλατα',
    '🚭' => 'απαγορεύεται το κάπνισμα',
    '🚯' => 'απαγορεύεται η ρίψη σκουπιδιών',
    '🚱' => 'μη πόσιμο νερό',
    '🚷' => 'απαγορεύεται η διάβαση πεζών',
    '📵' => 'απαγορεύονται τα κινητά',
    '🔞' => 'μόνο άνω των 18',
    '☢' => 'ραδιενέργεια',
    '☣' => 'βιολογικός κίνδυνος',
    '⬆' => 'πάνω βέλος',
    '↗' => 'πάνω δεξιό βέλος',
    '➡' => 'δεξιό βέλος',
    '↘' => 'κάτω δεξιό βέλος',
    '⬇' => 'κάτω βέλος',
    '↙' => 'κάτω αριστερό βέλος',
    '⬅' => 'αριστερό βέλος',
    '↖' => 'πάνω αριστερό βέλος',
    '↕' => 'πάνω κάτω βέλος',
    '↔' => 'αριστερό δεξιό βέλος',
    '↩' => 'δεξιό βέλος που στρίβει αριστερά',
    '↪' => 'αριστερό βέλος που στρίβει δεξιά',
    '⤴' => 'δεξιό βέλος που στρίβει πάνω',
    '⤵' => 'δεξιό βέλος που στρίβει κάτω',
    '🔃' => 'δεξιόστροφα κάθετα βέλη',
    '🔄' => 'κουμπί αριστερόστροφων βελών',
    '🔙' => 'βέλος BACK',
    '🔚' => 'βέλος END',
    '🔛' => 'βέλος ON!',
    '🔜' => 'βέλος SOON',
    '🔝' => 'βέλος TOP',
    '🛐' => 'τόπος λατρείας',
    '⚛' => 'σύμβολο ατόμου',
    '🕉' => 'σύμβολο ομ',
    '✡' => 'άστρο του Δαβίδ',
    '☸' => 'τροχός ντάρμα',
    '☯' => 'γιν γιανγκ',
    '✝' => 'λατινικός σταυρός',
    '☦' => 'ορθόδοξος σταυρός',
    '☪' => 'αστέρι και ημισέληνος',
    '☮' => 'σύμβολο ειρήνης',
    '🕎' => 'επτάφωτη λυχνία',
    '🔯' => 'εξάλφα με σημάδι',
    '🪯' => 'Χάντα',
    '♈' => 'Κριός',
    '♉' => 'Ταύρος',
    '♊' => 'Δίδυμοι',
    '♋' => 'Καρκίνος',
    '♌' => 'Λέων',
    '♍' => 'Παρθένος',
    '♎' => 'Ζυγός',
    '♏' => 'Σκορπιός (ζώδιο)',
    '♐' => 'Τοξότης',
    '♑' => 'Αιγόκερως',
    '♒' => 'Υδροχόος',
    '♓' => 'Ιχθύς',
    '⛎' => 'Οφιούχος',
    '🔀' => 'κουμπί τυχαίας αναπαραγωγής',
    '🔁' => 'κουμπί επανάληψης',
    '🔂' => 'κουμπί επανάληψης ενός',
    '▶' => 'κουμπί αναπαραγωγής',
    '⏩' => 'κουμπί γρήγορης προώθησης',
    '⏭' => 'κουμπί επόμενου κομματιού',
    '⏯' => 'κουμπί αναπαραγωγής ή παύσης',
    '◀' => 'κουμπί επαναφοράς',
    '⏪' => 'κουμπί γρήγορης επαναφοράς',
    '⏮' => 'κουμπί τελευταίου κομματιού',
    '🔼' => 'κουμπί πάνω',
    '⏫' => 'κουμπί πάνω γρήγορα',
    '🔽' => 'κουμπί κάτω',
    '⏬' => 'κουμπί κάτω γρήγορα',
    '⏸' => 'κουμπί παύσης',
    '⏹' => 'κουμπί διακοπής',
    '⏺' => 'κουμπί εγγραφής',
    '⏏' => 'κουμπί εξαγωγής',
    '🎦' => 'κινηματογράφος',
    '🔅' => 'κουμπί μείωσης φωτισμού',
    '🔆' => 'κουμπί αύξησης φωτισμού',
    '📶' => 'γραμμές ισχύος σήματος',
    '🛜' => 'ασύρματο',
    '📳' => 'λειτουργία δόνησης',
    '📴' => 'κινητό απενεργοποιημένο',
    '♀' => 'θηλυκό',
    '♂' => 'αρσενικό',
    '⚧' => 'σύμβολο της διεμφυλικότητας',
    '✖' => 'πολλαπλασιασμός',
    '➕' => 'συν',
    '➖' => 'πλην',
    '➗' => 'διαίρεση',
    '🟰' => 'σύμβολο ισότητας έντονο',
    '♾' => 'άπειρο',
    '‼' => 'διπλό θαυμαστικό',
    '⁉' => 'θαυμαστικό με ερωτηματικό',
    '❓' => 'κόκκινο λατινικό ερωτηματικό',
    '❔' => 'λευκό λατινικό ερωτηματικό',
    '❕' => 'λευκό θαυμαστικό',
    '❗' => 'κόκκινο θαυμαστικό',
    '〰' => 'κυματιστή παύλα',
    '💱' => 'ανταλλαγή νομισμάτων',
    '💲' => 'έντονο σύμβολο δολαρίου',
    '⚕' => 'ιατρικό σύμβολο',
    '♻' => 'σύμβολο ανακύκλωσης',
    '⚜' => 'άνθος του κρίνου',
    '🔱' => 'έμβλημα τρίαινας',
    '📛' => 'ταμπέλα ονόματος',
    '🔰' => 'ιαπωνικό σύμβολο για αρχάριους',
    '⭕' => 'κενός κόκκινος κύκλος',
    '✅' => 'κουμπί σημαδιού επιλογής',
    '☑' => 'πλαίσιο επιλογής με σημάδι επιλογής',
    '✔' => 'σημάδι επιλογής',
    '❌' => 'σημάδι x',
    '❎' => 'κουμπί σημαδιού x',
    '➰' => 'βρόχος',
    '➿' => 'διπλός βρόχος',
    '〽' => 'σύμβολο εναλλαγής μέρους',
    '✳' => 'οκτάκτινος αστερίσκος',
    '✴' => 'οκτάκτινο αστέρι',
    '❇' => 'σπινθήρας',
    '™' => 'εμπορικό σήμα',
    '🫟' => 'πιτσιλιά',
    '🔠' => 'κεφαλαία λατινικά',
    '🔡' => 'πεζά λατινικά',
    '🔢' => 'αριθμοί',
    '🔣' => 'σύμβολα',
    '🔤' => 'λατινικά γράμματα',
    '🅰' => 'κουμπί Α (ομάδα αίματος)',
    '🆎' => 'κουμπί ΑΒ (ομάδα αίματος)',
    '🅱' => 'κουμπί Β (ομάδα αίματος)',
    '🆑' => 'κουμπί CL',
    '🆒' => 'κουμπί COOL',
    '🆓' => 'κουμπί FREE',
    'ℹ' => 'πληροφορίες',
    '🆔' => 'κουμπί ID',
    'Ⓜ' => 'M σε κύκλο',
    '🆕' => 'κουμπί NEW',
    '🆖' => 'κουμπί NG',
    '🅾' => 'κουμπί O (ομάδα αίματος)',
    '🆗' => 'κουμπί OK',
    '🅿' => 'κουμπί P',
    '🆘' => 'κουμπί SOS',
    '🆙' => 'κουμπί UP!',
    '🆚' => 'κουμπί VS',
    '🈁' => 'ιδεόγραμμα εδώ σε τετράγωνο',
    '🈂' => 'ιδεόγραμμα χρέωσης υπηρεσίας σε τετράγωνο',
    '🈷' => 'ιδεόγραμμα μηνιαίου ποσού σε τετράγωνο',
    '🈶' => 'ιδεόγραμμα επί πληρωμή σε τετράγωνο',
    '🈯' => 'ιδεόγραμμα κράτησης σε τετράγωνο',
    '🉐' => 'ιδεόγραμμα προσφοράς σε τετράγωνο',
    '🈹' => 'ιδεόγραμμα έκπτωσης σε τετράγωνο',
    '🈚' => 'ιδεόγραμμα δωρεάν σε τετράγωνο',
    '🈲' => 'ιδεόγραμμα απαγόρευσης σε τετράγωνο',
    '🉑' => 'ιδεόγραμμα αποδοχής σε κύκλο',
    '🈸' => 'ιδεόγραμμα εφαρμογής σε τετράγωνο',
    '🈴' => 'ιδεόγραμμα βαθμού βάσης σε τετράγωνο',
    '🈳' => 'ιδεόγραμμα διαθεσιμότητας δωματίων σε τετράγωνο',
    '㊗' => 'ιδεόγραμμα συγχαρητηρίων σε κύκλο',
    '㊙' => 'ιδεόγραμμα μυστικού σε κύκλο',
    '🈺' => 'ιδεόγραμμα λειτουργίας σε τετράγωνο',
    '🈵' => 'ιδεόγραμμα πληρότητας σε τετράγωνο',
    '🔴' => 'κόκκινος κύκλος',
    '🟠' => 'πορτοκαλί κύκλος',
    '🟡' => 'κίτρινος κύκλος',
    '🟢' => 'πράσινος κύκλος',
    '🔵' => 'μπλε κύκλος',
    '🟣' => 'μοβ κύκλος',
    '🟤' => 'καφέ κύκλος',
    '⚫' => 'μαύρος κύκλος',
    '⚪' => 'λευκός κύκλος',
    '🟥' => 'κόκκινο τετράγωνο',
    '🟧' => 'πορτοκαλί τετράγωνο',
    '🟨' => 'κίτρινο τετράγωνο',
    '🟩' => 'πράσινο τετράγωνο',
    '🟦' => 'μπλε τετράγωνο',
    '🟪' => 'μοβ τετράγωνο',
    '🟫' => 'καφέ τετράγωνο',
    '⬛' => 'μεγάλο μαύρο τετράγωνο',
    '⬜' => 'μεγάλο λευκό τετράγωνο',
    '◼' => 'μεσαίο μαύρο τετράγωνο',
    '◻' => 'μεσαίο λευκό τετράγωνο',
    '◾' => 'μικρομεσαίο μαύρο τετράγωνο',
    '◽' => 'μικρομεσαίο λευκό τετράγωνο',
    '▪' => 'μικρό μαύρο τετράγωνο',
    '▫' => 'μικρό λευκό τετράγωνο',
    '🔶' => 'μεγάλος πορτοκαλί ρόμβος',
    '🔷' => 'μεγάλος μπλε ρόμβος',
    '🔸' => 'μικρός πορτοκαλί ρόμβος',
    '🔹' => 'μικρός μπλε ρόμβος',
    '🔺' => 'κόκκινο τρίγωνο πάνω',
    '🔻' => 'κόκκινο τρίγωνο κάτω',
    '💠' => 'ρόμβος με τελεία',
    '🔘' => 'στρογγυλό κουμπί',
    '🔳' => 'λευκό τετράγωνο κουμπί',
    '🔲' => 'μαύρο τετράγωνο κουμπί',
    '🏁' => 'καρό σημαία',
    '🚩' => 'τριγωνική σημαία',
    '🎌' => 'σταυρωτές σημαίες',
    '🏴' => 'μαύρη σημαία',
    '🏳' => 'λευκή σημαία',
];
